#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsCdt.sh,v 4.7 2005/06/16 14:47:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  01/09/95  created
# alongino  23/02/96  modified to consider INTROOT and VLTROOT
# alongino  20/05/96  cmdManager JUN96 does not require links any more
# alongino  10/06/96  -parse option replaced with -load
#                     _$CCDNAME removed from CDT name (change in JUN96)

#************************************************************************
#   NAME
#   ccdDcsCdt.sh - Load CDTs for CCD DCS processes for a CCD camera
# 
#   SYNOPSIS
#   ccdDcsCdt.sh [<camera>]
# 
#   DESCRIPTION
#   This shell script loads the CDTs for CCS Message System
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdDcsCdt.sh ccdFors
#       CDTs of all processes involved in the control of the CCD camere 
#       ccdFors are loaded in CMD shared memeory
#
#   SEE ALSO
#   ccdDcsStart.sh
#
#   BUGS
#------------------------------------------------------------------------
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# 1 - Load CDTs
#
echo ""
echo "Loading CDTs for CCD processes for camera $CCDNAME ... \c"

# WS processes
cmdSetup -load ccdconCI  >/dev/null
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

cmdSetup -load ccditWs  >/dev/null
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

# LCU processes
cmdSetup -load ccdcon >/dev/null
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

cmdSetup -load ccdit >/dev/null 
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

cmdSetup -load ccdrdt >/dev/null 
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

cmdSetup -load ccdsht >/dev/null 
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

echo "done"
echo ""

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___





