#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsDbInit.sh,v 4.7 2005/06/16 14:47:54 vltsccm Exp $"
#
# who          when      what
# --------     --------  ----------------------------------------------
# A.Longinotti 26/11/94  created
# A.Longinotti 07/08/95  modified in order to use dbRestore
# A.Longinotti 20/02/96  modified in order to use new co option of dbRestore
#

#************************************************************************
#   NAME
#   ccdDcsDbInit.sh - Initialise database values for the CCD DCS part
# 
#   SYNOPSIS
#   ccdDcsDbInit.sh [<camera>] [<env>] [<backup_dir>]
# 
#   DESCRIPTION
#   This shell script initialises some values in the database needed at 
#   startup by the CCD software.
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <env>       environment where to write (default env. var. RTAPENV)
#   <bckup_dir> directory where backup files are looked for 
#               (default $INS_ROOT/SYSTEM/COMMON/CONFIGFILES)
# 
#   FILES
#   ccdDcsFixed.dbcfg  contains fixed values for all cameras
#   <camera>.dbcfg     contains values for <camera>
#
#   ENVIRONMENT
#   CCDNAME   default for camera name (e.g. ccdFors)
#   RTAPENV   default environment (e.g.wte16)
#   INS_ROOT  default for instrument root directory
#   INS_USER  default to SYSTEM if not defined
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdDcsDbInit.sh ccdFors wte16 $INS_ROOT/$INS_USER/COMMON/CONFIGFILES
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
#
# WS part
#
#
# set environment variables (if not yet done)
#
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    environ=$2
else
    environ=$RTAPENV
fi
if [ "$INS_USER" != "" ]
then
    export INS_USER=SYSTEM
fi
if [ $# -ge 3 ]
then
    bckdir=$3
elif [ "$INS_ROOT" != "" ]
then
    bckdir=$INS_ROOT/$INS_USER/COMMON/CONFIGFILES
else
    echo $INS_ROOT
    exit 1
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

echo ""
echo "Load DB configuration in environment $environ for camera $CCDNAME ... \c"

#
# Restore CCD DCS fixed configuration values from the backup file
#
bckfile=$bckdir/ccdDcsFixed.dbcfg
dbRestore -f $bckfile -co "@$environ<alias>$CCDNAME"
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi

#
# Restore CCD DCS system-dependant  configuration values from the backup file
#
bckfile=$bckdir/$CCDNAME.dbcfg
dbRestore -f $bckfile -co "@$environ<alias>$CCDNAME"
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi

#
# Ensure that the directories in which the SETUP and DET files will
# be are setup with this INS_ROOT
#
dbWrite "<alias>$CCDNAME:observations.setupDirectory" "$INS_ROOT/$INS_USER/COMMON/SETUPFILES/DET"
dbWrite "<alias>$CCDNAME:images.imageDirectory" "$INS_ROOT/$INS_USER/DETDATA"

echo "done"
echo ""

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___
