#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsDbSave.sh,v 4.7 2005/06/16 14:47:54 vltsccm Exp $"
#
# who          when      what
# --------     --------  ----------------------------------------------
# A.Longinotti 23/04/96  created
# A.Longinotti 22/10/96  added k option for RCS keyword to dbBackup call
#

#************************************************************************
#   NAME
#   ccdDcsDbSave.sh - Save CCD database values into backup file
# 
#   SYNOPSIS
#   ccdDcsDbSave.sh [<camera>] [<env>] [<file>]
# 
#   DESCRIPTION
#   This shell script reads some values from the CCD WS database and
#   stores them into <file> using the utility dbBackup
#
#   <camera> camera name, also root point in DB (default env. var. CCDNAME)
#   <env>    environment where to read from (default env. var. RTAPENV)
#   <file>   name of backup file
#            (default INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<camera>.dbcfg)
# 
#   FILES
#   ccdConfig.inp     input file for utility dbBackup
#
#   ENVIRONMENT
#   CCDNAME   default for camera name (e.g. ccdFors)
#   RTAPENV   default environment (e.g.wte16)
#   INS_ROOT  default for instrument root directory
#   INS_USER  default to SYSTEM if not defined
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdDcsDbSave.sh ccdFors wte16 $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/ccdFors.dbcfg
#
#   SEE ALSO
#   dbBackup
#
#   BUGS
#------------------------------------------------------------------------
#
# set environment variables (if not yet done)
#
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    environ=$2
else
    environ=$RTAPENV
fi
if [ "$INS_USER" != "" ]
then
    export INS_USER=SYSTEM
fi
if [ $# -ge 3 ]
then
    bckfile=$3
elif [ "$INS_ROOT" != "" ]
then
    bckfile=$INS_ROOT/$INS_USER/COMMON/CONFIGFILES/$CCDNAME.dbcfg
else
    echo $INS_ROOT
    exit 1
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

echo ""
echo "Save DB configuration for camera $CCDNAME in $bckfile ... \c"
#
# Save DB configuration with dbBackup
#
file=ccdConfig.inp
if [ "$INTROOT" != "" ]
then
    inpFile=$INTROOT/config/$file
    if [ -f $inpFile ]
    then
	inpFile=$INTROOT/config/$file
    else
	inpFile=$VLTROOT/config/$file
    fi
else
    inpFile=$VLTROOT/config/$file
fi

dbBackup -k -c "@$environ<alias>$CCDNAME" -f $inpFile -o $bckfile
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi

echo "done"
echo ""

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___
