#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsLcuReboot.sh,v 4.7 2005/06/16 14:47:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  17/01/95  created
# alongino  09/07/96  ported to VLT sw JUN96 and lccBoot
# alongino  21/10/96  consider if NO_ERR_DISPLAY defined when started
#

#************************************************************************
#   NAME
#   ccdDcsLcuReboot.sh - Reboots CCD DCS LCU
# 
#   SYNOPSIS
#   ccdDcsLcuReboot.sh [<camera>] [<WS env.>] [<LCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script reboots the CCD LCU, independently from what is going
#   on there.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV).
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, no action on LCU part of CCD sw is taken
#   <option>   s    restart also the scan system
# 
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   This script should be used with care. If anything is going on in the
#   LCU it will get lost !!!!
#
#   EXAMPLES
#   > ccdDcsLcuReboot.sh ccdFors 0 lte25
#         Reboot LCU lte25, used for camera "ccdFors"
#
#   > ccdDcsLcuReboot.sh ccdFors wte13 lte25 s
#         Reboot LCU lte25, used for camera "ccdFors" and restart the scan
#         system
#
#   SEE ALSO
#   ccdDcsScan.sh
#
#   BUGS
#------------------------------------------------------------------------
module=ccdws
option=0
scan=0
message=""
if [ $# -ge 1 ]
then
    camera=$argv[1]
else
    camera=$CCDNAME
fi

if [ $# -ge 2 ]
then
    locEnv=$2
else
    locEnv=$RTAPENV
fi
if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi
if [ $# -ge 4 ]
then
    option=$4
    if [ "$option" = "s" ]
    then
	scan=1
    fi
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# in CCD LCU DB camera opState to OFF
#
echo ""
echo "Setting CCD $camera in LCU DB $remEnv to OFF state ... \c"
dbWrite "@$remEnv<alias>$camera.opStateLcu" 1
if [ $? = 0 ]
then
    echo "done"
else
    echo "FAILURE"
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

#
# execute utility lccBoot
#
message="Rebooting LCU $remEnv"
echo ""
echo "$message ... \c"
# log information that LCU is going to be booted
logUserData $module 0 "$message"

# 3 min timeout
timeout=180
lccBoot $remEnv $timeout
if [ $? = 0 ]
then 
    echo "done"
else
    echo "FAILURE"
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi


#
# Restart the scan system if requested
#
if [ $scan = 1 ]
then
    ccdDcsScan.sh $camera $locEnv $remEnv r
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
        exit 1
    fi
fi 

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___
