#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsLcuSim.sh,v 4.7 2005/06/16 14:47:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  06/09/95  created
# alongino  22/10/96  added appropriate setting of CCSSTATE variable
#

#************************************************************************
#   NAME
#   ccdDcsLcuSim.sh - Set CCD LCU sw simulation state
# 
#   SYNOPSIS
#   ccdDcsLcuSim.sh [<camera>] [<WS env.>] [<LCU env.>]
# 
#   DESCRIPTION
#   This shell script do all actions needed to set the simulation mode
#   of the CCD LCU sw using CCS facilities.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, LCU sw is simulated, otherwise not
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdDcsLcuSim.sh ccdFors wte13 lte25
#         The cmdManager running on WS environment wte13 is informed that
#         the CCD LCU sw for camera "ccdFors" is running on lte25 and must
#         not be simulated.
#
#   > ccdDcsLcuSim.sh ccdFors wte13 0
#         The cmdManager running on WS environment wte13 is informed that
#         the CCD LCU sw for camera "ccdFors" is running must be simulated.
#
#   SEE ALSO
#   ccdDcsStart.sh
#
#   BUGS
#------------------------------------------------------------------------
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    export RTAPENV=$2
fi
if [ $# -ge 3 ]
then
    export CCDLENV=$3
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# Inform cmdManager if LCU processes are simulated or not
#
if [ "$CCDLENV" != "" ]
then
    msg="Setting LCU processes for camera $CCDNAME NOT in simulation ... "
    command="SIMUSET"
    export CCSSTATE=NORMAL
else
    msg="Setting LCU processes for camera $CCDNAME in simulation ... "
    command="SIMSET"
    export CCSSTATE=SIMULATION
fi
echo ""
echo "$msg \c"
msgSend -n $RTAPENV cmdManager $command "ccdit_$CCDNAME" >/dev/null
if [ $? != 0 ] 
then
    echo "FAILED"
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

msgSend -n $RTAPENV cmdManager $command "ccdcon_$CCDNAME" >/dev/null
if [ $? != 0 ] 
then
    echo "FAILED"
	export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi
echo "done"
echo ""

#
# CCD LCU sw state to invalid high value, in order to have the global
# state, returned for example by command STATUS, to the CCD WS opState
#
if [ "$CCDLENV" = "" ]
then
    echo ""
    echo "Setting CCD LCU operational state to simulation ... \c"
    dbWrite "<alias>$CCDNAME.opStateLcu" "6"
    if [ $? != 0 ]
    then
	echo "FAILED"
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    echo "done"
    echo ""
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___


