#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsScan.sh,v 4.7 2005/06/16 14:47:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  18/10/94  created
# alongino  21/10/94  env. var. names modified
# alongino  12/09/95  modified according to new database and JUL95 scan system
# alongino  08/02/96  scan mechanism (poll/srbx) changed to accomodate TCCDS
# alongino  29/02/96  Bug fixed: removed items for transfer and display
#                     shutter status moved to polling (must be as exp. status)
# alongino  29/05/96  failureLcu added
# alongino  05/06/96  some attributes moved to ccdosScan.sh for stand-alone
# alongino  02/07/96  FAILURE removed if scan entry already there
# alongino  02/07/96  scanDevOn called before scanConfig
# alongino  18/10/96  added RtapForceSnap
# alongino  21/10/96  consider if NO_ERR_DISPLAY defined when started
# pduhoux   10/04/97  use CCS scanLinks utility (MAY97)
#                     removed option w
# pduhoux   15/04/97  modified functionality:
#                     a - scanDevOn + scanLinks -E
#                     e - scanDevOn
#                     d - nothing
#                     c - scanLinks -d
# pduhoux   2000-07-11 Replaced RtapForceSnap by dbForceSnap
#

#************************************************************************
#   NAME 
#   ccdDcsScan.sh - configure the DB values to be scanned from LCU for CCD DCS
# 
#   SYNOPSIS
#   ccdDcsScan.sh [<camera>] [<WS env.>] [<LCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script configures the Scan System to retrieve DB values
#   for the CCD LCU to WS.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#   <option>   a    add entries to scan list
#              c    clean entries from scan list
#              d    disable scanning from the LCU
#              e    enable scanning from the LCU
#              default: a 
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   1 - It is assumed here that CCD branch in the LCU database is attached
#       directly to the root point (not a sub-point of something else).
#   2 - If this script is run twice or more, the system may not behave properly
#       any more. In this case better to shutdown the whole system and restart 
#       it again.
#
#   EXAMPLES
#   ccdDcsScan.sh ccdFors wte13 lte25
#        Add entries for LCU lte25 and WS wte13 camera ccdFors
#   ccdDcsScan.sh ccdFors wte13 lte25 c
#        Clean entries for LCU lte25 and WS wte13 camera ccdFors
#   ccdDcsScan.sh 0 wte13 lte25 d
#        Disable scanning from LCU lte25 to WS wte13
#   ccdDcsScan.sh 0 wte13 lte25 e
#        Enable scanning from LCU lte25 to WS wte13
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
add=0
delete=0
disable=0
enable=0
option=0
message=""
module=ccdws

if [ $# -ge 1 ]
then
    camera=$1
else
    camera=$CCDNAME
fi

if [ $# -ge 2 ]
then
    locEnv=$2
else
    locEnv=$RTAPENV
fi

if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi

if [ $# -ge 4 ]
then
    option=$4
    if [ "$option" = "a" ]
    then
	add=1
    fi
    if [ "$option" = "c" ]
    then
	delete=1
    fi
    if [ "$option" = "d" ]
    then
	disable=1
    fi
    if [ "$option" = "e" ]
    then
	enable=1
    fi
else
    add=1
fi

export RTAPENV=$locEnv

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""  

option="-l $locEnv -e $remEnv -s <alias>$camera -r <alias>$camera"

file=ccdDcsScan.links
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ] 
    then
	scanFile=$INTROOT/config/$file
    else
	scanFile=$VLTROOT/config/$file
    fi
else
    scanFile=$VLTROOT/config/$file
fi

#
# Configure scan links from specified scan device
#
if [ $add = 1 ]
then
    message="Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
    echo ""
    echo "$message ... \c"
    scanDevOn $remEnv >/dev/null
    if [ $? = 0 ] 
    then
	echo "done"
	logUserData $module 0 "$message"
    else
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    fi

    message="Checking & Enabling scan links from $remEnv to $locEnv for CCD $camera"
    echo ""
    echo "$message ... \c"
    scanLinks -c -E $option -f $scanFile >/dev/null
    if [ $? = 0 ] 
    then
	echo "done"
	logUserData $module 0 "$message"
    else
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    fi
fi

#
# Clean scan links from specified scan device
#
if [ $delete = 1 ]
then
    message="Cleaning scan links from $remEnv to $locEnv for CCD $camera"
    echo ""
    echo "$message ... \c"
    scanLinks -c -d $option -f $scanFile -o >/dev/null
    if [ $? = 0 ] 
    then
	echo "done"
	logUserData $module 0 "$message"
    else
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    fi
fi

#
# Disable scanning from specified scan device
#
#if [ $disable = 1 ]
#then
#    message="Disabling scanning of CCD $camera DB data from $remEnv to $locEnv"
#    echo ""
#    echo "$message ... \c"
#    scanLinksDisable -l $locEnv -r <alias>$camera $scanFile >/dev/null
#    if [ $? = 0 ] 
#    then
#	echo "done"
#	logUserData $module 0 "$message"
#    else
#	export NO_ERR_DISPLAY=$noErrDisplay
#	echo "FAILURE"
#	exit 1
#    fi
#fi

#
# Enable scanning from specified scan device
#
if [ $enable = 1 ]
then
    message="Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
    echo ""
    echo "$message ... \c"
    scanDevOn $remEnv >/dev/null
    if [ $? = 0 ] 
    then
	echo "done"
	logUserData $module 0 "$message"
    else
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    fi

#    message="Enabling scanning of CCD $camera DB data from $remEnv to $locEnv stand-alone part"
#    scanLinksEnable -l $locEnv -r <alias>$camera $scanFile >/dev/null
#    if [ $? = 0 ] 
#    then
#	echo "done"
#	logUserData $module 0 "$message"
#    else
#	export NO_ERR_DISPLAY=$noErrDisplay
#	echo "FAILURE"
#	exit 1
#    fi
fi

dbForceSnap -e $locEnv
if [ $? != 0 ] 
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILURE"
    exit 1
fi

export NO_ERR_DISPLAY=$noErrDisplay

#
# add a 2s delay for other scan scripts to work fine (SPR20000480)
#
sleep 2

exit 0
#
# ___oOo___
#

