#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsSetLogMask.sh,v 4.7 2005/06/16 14:47:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   16/09/97  created
# pduhoux   2000-07-11 Replaced RtapForceSnap by dbForceSnap
#

#************************************************************************
#   NAME
#   ccdDcsSetLogMask.sh - Set FITS logging mask in OLDB for CCD camera
# 
#   SYNOPSIS
#   ccdDcsSetLogMask.sh <camera> <WS env.> <logMask>
# 
#   DESCRIPTION
#   This shell script writes in the WS CCD OLDB the value of the attribute
#   related to the FITS logging of main info.
#   An update of the $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<camera>.dbcfg file 
#   is performed. As well a snap of the WS CCD OLDB is made.
#   It is meant as a utility to applications having to configure the
#   logMask parameter for the camera used.
#
#   <camera>     camera name, also root point in DB
#   <WS env.>    name of workstation environment
#   <logMask>    Mask for FITS logging           
# 
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   EXAMPLES
#   > ccdDcsSetLogMask.sh ccdAGA wut1tcs UT1AGA
#         FITS Logs [UT1AGA] 
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
if [ $# -lt 3 ]
then
    echo "Usage: ccdDcsSetLogMask.sh <camera> <WS env.> <logMask>"
    exit 1
fi
camera=$1
locEnv=$2
logMask=$3

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# Write values in the DB and do a snapshot
#
dbWrite "<alias>$camera.logMask" $logMask
ccdDcsDbSave.sh $camera $locEnv
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

dbForceSnap -e $locEnv
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___
