#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsWcs.sh,v 4.7 2005/06/16 14:47:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/02/97  created
# pduhoux   11/02/97  removed ra & dec attributes
#                     added   xrefpix & yrefpix attributes
# pduhoux   24/11/99  optional arg <flip>
# pduhoux   2000-07-11 Replaced RtapForceSnap by dbForceSnap
#

#************************************************************************
#   NAME
#   ccdDcsWcs.sh - Set World Coordinate values in OLDB for CCD camera
# 
#   SYNOPSIS
#   ccdDcsWcs.sh <camera> <WS env.> <xrefpix> <yrefpix>
#                <secpix> <rotate> <equinox> <epoch> <proj> [<flip>]
# 
#   DESCRIPTION
#   This shell script writes in the WS CCD OLDB the values of attributes
#   related to the display of World Coordinates with rtd.
#   It is meant as a utility to applications having to configure the
#   WCS parameters for the camera used.
#
#   <camera>     camera name, also root point in DB
#   <WS env.>    name of workstation environment
#   <xrefpix>    X Coordinate of Reference Pixel
#   <yrefpix>    Y Coordinate of Reference Pixel
#   <secpix>     Number of arcseconds per pixel
#   <rotate>	 Rotation angle (clockwise positive) in degrees
#   <equinox>	 Equinox of coordinates, 1950 and 2000 supported
#   <epoch>	 Epoch of coordinates, used for FK4/FK5 conversion
#   <proj>	 Projection
#  [<flip>]      Display Flip flags "NORMAL","FLIP_RA","FLIP_DEC" or "FLIP_RADEC"
# 
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   EXAMPLES
#   > ccdDcsWcs.sh ccdTEC wtccds 200.0 145.0 0.3 0.0 1950 0.0 PIXEL FLIP_RA
#         Reference pixel [200.0;145.0]
#         Each pixel covers 0.3 arcseconds
#         Rotation angle is 0.0 degrees
#         Equinox of ccordinates set to 1950
#         Epoch of ccordinates set to 0.0
#         Projection set to PIXEL
#         RA coordinate is flipped
#
#   SEE ALSO
#   rtdImageEvent.h for more information about parameters meaning and
#                   possible values
#
#   BUGS
#------------------------------------------------------------------------
if [ $# -lt 9 ] 
then
    echo "Usage: ccdDcsWcs.sh <camera> <WS env.> <xrefpix> <yrefpix> <secpix> <rotate> <equinox> <epoch> <proj> [<flip>]"
    exit 1
fi
camera=$1
locEnv=$2
xRefPix=$3
yRefPix=$4
secPix=$5
rotate=$6
equinox=$7
epoch=$8
proj=$9

if [ $# = 10 ] 
then 
    flip=$10
    if [ "$flip" = "FLIP_RA" ] 
    then
        flags=1
    elif [ "$flip" = "FLIP_DEC" ]
    then
	flags=2
    elif [ "$flip" = "FLIP_RADEC" ] 
    then
	flags=3
    else 
	flags=0
    fi
fi

#
# Write values in the DB and do a snapshot
#
dbWrite "<alias>$camera:wcs.xRefPix" $xRefPix
dbWrite "<alias>$camera:wcs.yRefPix" $yRefPix
dbWrite "<alias>$camera:wcs.secPix"  $secPix
dbWrite "<alias>$camera:wcs.rotate"  $rotate
dbWrite "<alias>$camera:wcs.equinox" $equinox
dbWrite "<alias>$camera:wcs.epoch"   $epoch
dbWrite "<alias>$camera:wcs.proj"    $proj
if [ $# = 10 ]
then 
    dbWrite "<alias>$camera:wcs.flags" $flags
fi
dbForceSnap -e $locEnv

exit 0

# ___oOo___
