#! /bin/ksh
# option -f to avoid running .cshrc and override PATH set with setmod
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDemo.sh,v 1.79 2005/05/09 22:34:44 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  08/09/95  created
#

#************************************************************************
#   NAME
#   ccdDemo.sh - start the CCD stand-alone sw
# 
#   SYNOPSIS
#   ccdDemo.sh [<camera>] [<WS env.>] [<LCU env.>] [<INS root>]
# 
#   DESCRIPTION
#   This shell script pops up the panels used to control a CCD camera in
#   stand-alone. Actions are simulated within the panels. No CCD program need
#   to run.
#
#   <camera>     camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>    name of workstation environment (default env. var. RTAPENV)
#   <LCU env.>   name of LCU environment (default env. var. CCDLENV)
#   <INS root>   INS_ROOT environment variable
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#   INS_ROOT default root directory for instrument data 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdDemo.sh ccdFors wte13 lte25 $INS_ROOT
#   Start CCD sw for camera "ccdFors" on environments wte13 (WS) and 
#   lte25 (LCU). Panels run in simulation
#
#   SEE ALSO
#   ccdStart.sh
#
#   BUGS
#------------------------------------------------------------------------
#
# set environment variables (if not yet done)
#
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    export RTAPENV=$2
fi
if [ $# -ge 3 ]
then
    export CCDLENV=$3
fi
if [ $# -ge 4 ]
then
    export INS_ROOT=$4
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

ccdStart.sh $CCDNAME $RTAPENV $CCDLENV $INS_ROOT 1
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___




