#! /bin/ksh
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdMidasBgStart.sh,v 4.11 2005/06/22 10:58:13 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  07/11/96  created
#

#************************************************************************
#   NAME
#   ccdMidasBgStart.sh - start a Midas background session and pco
# 
#   SYNOPSIS
#   ccdMidasBgStart.sh [<unit ID>]
# 
#   DESCRIPTION
#   This shell script starts a Midas session in background mode
#   and the VLT pco program to communicate with it.
#   <unit ID> is the Midas unit (00 to 99) to be started. Default : 50
# 
#   FILES
#
#   ENVIRONMENT
#   RTAPENV current local environment used
#   HOST    name of host computer
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdMidasBgStart.sh
#         Start pco and Background Midas session with default unit 50.
#
#   > ccdMidasBgStart.sh 51
#         Start pco and Background Midas session with unit 51.
#
#   SEE ALSO
#   ccdMidasBgStop.sh
#
#   BUGS
#------------------------------------------------------------------------
midasId=50
if [ $# -ge 1 ]
then
    midasId=$1
fi
# If < 0 we do not know how many trailing 0s. Therefore, remove all and add one
if [ $midasId -lt 10 ]
then
    @ midasId+= 0
    midasId=0$midasId
fi
#
# Verify that pco is running.
#
noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

notRegistered=1
timeout=5
time=1
msgSend -n "" pco "PING" "" >/dev/null
# If not running, schedule it and wait until registered in CCS
if [ $? != 0 ]
then
    ccsRunCmd -e $RTAPENV "pco &"
    echo ""
    echo "Scheduling process pco in local environment ... \c"
    while [ $notRegistered = 1 ]
    do
	msgSend -n $RTAPENV pco "PING" "" >/dev/null
	if [ $? = 0 ]
	then
	    notRegistered=0
	else
	    sleep 1
	    time=`expr $time + 1`
	fi
	if [ $time -gt $timeout ]
	then
	    export NO_ERR_DISPLAY=$noErrDisplay
	    echo "TIMEOUT"
	    exit 1
	fi
    done
    echo "done"
    echo ""
fi

#
# Verify that background MIDAS is running.
#
notRegistered=1
timeout=10
time=1
midasMsg="echo/off all"
msgSend $RTAPENV pco FORWMSG "-protocol MIDAS -procID $midasId -hostName $HOST -message $midasMsg" >/dev/null
if [ $? != 0 ]
then
    # If not running, start it in a separate xterm window
    xterm -T "Background Midas Unit $midasId" -e inmidas $midasId -p -j "set/back socket,remote off" &
    echo ""
    echo "Scheduling Background Midas unit $midasId on host $HOST ... \c"
    while [ $notRegistered = 1 ]
    do
	msgSend $RTAPENV pco FORWMSG "-protocol MIDAS -procID $midasId -hostName $HOST -message $midasMsg" >/dev/null
	if [ $? = 0 ]
	then
	    notRegistered=0
	else
	    sleep 1
	    time=`expr $time + 1`
	fi
	if [ $time -gt $timeout ]
	then
	    export NO_ERR_DISPLAY=$noErrDisplay
	    echo "TIMEOUT"
	    exit 1
	fi
    done
    echo "done"
    echo ""
else
    echo "Midas unit $midasId is running on host $HOST"
    echo ""
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0
#
# ___oOo___
#


