#! /bin/ksh
# option -f to avoid running .cshrc and override PATH set with setmod
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdStart.sh,v 1.79 2005/05/09 22:34:44 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  18/10/94  created
# alongino  21/10/95  if panels simulated, disable scan system
# alongino  27/10/95  database must be filled in before (problems if empty)
# alongino  29/01/96  name of panels changed (moved to ccdms)
# alongino  18/02/96  ccdosDbInit.sh removed. Obsolete
#                     auxiliary panels moved into ccdCtrl
#                     In simulation, exposure status forced to Idle
# alongino  17/05/96  Added startup of oslxServer
# alongino  18/10/96  Added definition of INS_USER
# alongino  21/10/96  consider if NO_ERR_DISPLAY defined when started
# alongino  05/05/97  oslxServer not needed any more
#

#************************************************************************
#   NAME
#   ccdStart.sh - start the CCD stand-alone sw
# 
#   SYNOPSIS
#   ccdStart.sh [<camera>] [<WS env.>] [<LCU env.>] [<INS root>] [<simulation>]
# 
#   DESCRIPTION
#   This shell script pops up the panels from which the operations with
#   the CCD stand-alone can be executed.
#   It also loads the on-line database values for the selected camera.
#
#   <camera>     camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>    name of workstation environment (default env. var. RTAPENV)
#   <LCU env.>   name of LCU environment (default env. var. CCDLENV)
#   <INS root>   INS_ROOT environment variable
#   <simulation> 1 = panels are entered in simulation mode: no action done.
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#   INS_ROOT default root directory for instrument data 
#   INS_USER default SYSTEM
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdStart.sh ccdFors wte13 lte25 $INS_ROOT 1
#   Start CCD sw for camera "ccdFors" on environments wte13 (WS) and 
#   lte25 (LCU). Panels run in simulation
#
#   SEE ALSO
#   ccdDcsStart.sh ccdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
#
# WS part
#
#
# set environment variables (if not yet done)
#
sim=0
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    export RTAPENV=$2
fi
if [ $# -ge 3 ]
then
    export CCDLENV=$3
fi
if [ $# -ge 4 ]
then
    export INS_ROOT=$4
fi
if [ $# -ge 5 ]
then
    sim=$5
fi
if [ "$CCDDID" = "" ]
then
    export CCDDID="CCDDCS"
fi
if [ "$INS_USER" = ""  ]
then
    export INS_USER="SYSTEM"
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# 1 - Load database configuration values on WS
# AL951027 This step must be done before starting panels, otherwise some
#          panels might read wrong values from the database when they are
#          pop-ed up, specially when database has just been new created.
#          Better also to load DCS database values
#
if [ "$RTAPENV" != "" ]
then
    ccdDcsDbInit.sh $CCDNAME $RTAPENV $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
fi

#
# 2 - Pop-up the main panel for CCD stand-alone
#     If panels run in simulation, make sure that scan system is disabled
#     and intialise operational states
#
if [ $sim = 1 ]
then
    echo "The whole CCD sw is simulated within panels"

    # Disable scan links
    ccdDcsScan.sh 0 $RTAPENV $CCDLENV d
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi

    # Operational mode forced to WS simulated
    dbWrite "<alias>$CCDNAME.opMode" 1
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi

    # Operational state forced to OFF
    dbWrite "<alias>$CCDNAME.opStateLcu" 6
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    dbWrite "<alias>$CCDNAME.opStateWs"  1
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    # Exposure status forced to IDLE
    dbWrite "<alias>$CCDNAME:exposures:exposure_1.expStatus"  1
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
fi

nohup ccdCtrl "cwp=<alias>$CCDNAME" "simul=$sim"  > /dev/null 2>&1 &
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

echo " "
echo "CCD stand-alone control panel for camera $CCDNAME started"
echo " "

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

#___oOo___
