#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#
# E.S.O. - VLT project - 1993
#
# "@(#) $Id: errFilter.awk,v 2.36 2005/07/27 12:48:50 vltsccm Exp $" 
#
# errFilter.awk
#
# HISTORY
#
# who          when            what
# ---------    ------------    --------------------------------------------
# M.Comin      17/11/94        First version
# T.Ebert      18/07/95        Replaced awk by gawk
#
############################################################################
#
#   NAME
#
#   errFilter.awk -  This awk script filters error logs
#
#   SYNOPSIS
#
#   errFilter.awk  <list-of-parameters>  inputFile > outputFile
# 
#   DESCRIPTION
#
#   Scans the file identified by the name 'inputFile' and filters the
#   error logs according to the following parameters :
#
#
#    Parameters                |  Keyword
#   ____________________________________________
#
#   - date                     | date
#   - start/stop   timeStamp   | start/stop
#   - stackId                  | stackId
#   - error severity           | sev
#   - process                  | proc
#   - module                   | mod
#   - environment              | env
#
#   It the outputFile is not specified, the output is displayed on the screen.
#
#   Parameters are specified  with the syntax : <parameter-name>=<value>
#   
# 
#   EXAMPLES
#
#   Collect all logs from 11:34:89 and severity=W
#
#   errFilter.awk start=11:34:89 sev=W /vltdev/etc/tmp/logFile
#
#   Collect all logs from 11:34:89 to 12:11 
#
#   errFilter.awk start=11:34:89 stop=12:11 /vltdev/etc/tmp/logFile
#
#   Collect all RTAP errors generated by process errTest
#
#   errFilter.awk mod=RTAP proc=errTest /vltdev/etc/tmp/logFile
#
#   Collect errors belonging to stackId=3241
#
#   errFilter.awk stackId=3241 /vltdev/etc/tmp/logFile > myStack
#
#
#   CAUTION
#
#   The filter can give unpredictable results if the format of the logs
#   is wrong or some parameters (such as  module) are missing.
#
############################################################################

gawk '

BEGIN { 

  stackId = severity = process = module = time1 = time2 = day = rtapEnv = ""
  
  for (i=1; i<ARGC ; i++ )  {

      if ( ARGV[i] ~ /day=/ )     { day       = substr(ARGV[i],5) }  
      if ( ARGV[i] ~ /start=/ )   { time1     = substr(ARGV[i],7) }
      if ( ARGV[i] ~ /stop=/ )    { time2     = substr(ARGV[i],6) }  
      if ( ARGV[i] ~ /stackId=/ ) { stackId   = substr(ARGV[i],9) }
      if ( ARGV[i] ~ /sev=/ )     { severity  = substr(ARGV[i],5) }
      if ( ARGV[i] ~ /proc=/ )    { process   = substr(ARGV[i],6) }
      if ( ARGV[i] ~ /mod=/ )     { module    = substr(ARGV[i],5) }           
      if ( ARGV[i] ~ /env=/ )     { rtapEnv   = substr(ARGV[i],5) }           
  }
}

{

  timeOnRange = ( $7 < time2  || time2 == "" ) && $7 > time1 
  noRange     = ( time1 == "" && time2 == "" ) 
  dayFlag     = ( day == "" || $6 == day)

# Print for debug
#printf ("time = %s  OnRange = %d  dayFlag = %d \n",$7,timeOnRange,dayFlag) 
  
    
  if ( ( noRange || timeOnRange ) && dayFlag && $12 == "62" ) {  

     buffer = $0 
             
     if (stackId  == "" ) { $15 = "" }  
     if (severity == "" ) { $16 = "" }
     if (module   == "" ) { $9  = "" }
     if (process  == "" ) { $10 = "" }
     if (rtapEnv  == "" ) { $8  = "" }

     if ( $9 == module && $10 == process && $15 == stackId && $16 == severity && $8 == rtapEnv) {
        print buffer
     }
  } 
}

' $*
