#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: errchCheck.awk,v 1.5 1998/03/02 17:48:32 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mgajardo  08/04/97  created
# mgajardo  19/19/97  added ccs parameter 
#

#************************************************************************
#   NAME     
#   errchCheck.awk
# 
#   SYNOPSIS   
#   awk -f errchCheck.awk  ccs=[CCS|NOCCS] [verbose=on]
#                  <module>/ERRORS/<module>_ERRORS 
#                    <module>/object/*.cpp
# 
#   DESCRIPTION  
#   Check correspondance between number of parameters in errAdd 
#   calls and the error definition files.
#   *.cpp files comes from run: 
#                   make cppFiles
#
#   FILES   errchCheck.awk
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

gawk '
/* get parameter "ccs=[CCS|NOCCS]" */
BEGIN {
 ccs = ""
 verbose=0
 for (i=1; i<ARGC ; i++ ) {
       if ( ARGV[i] ~ /ccs=/ )     { ccs       = substr(ARGV[i],5) }
       if ( ARGV[i] ~ /verbose=/ )     { verbose=1 }
 }
 ncpp=0

}


# Parse of Error definition files
/Error Definition File/{
     if ( ccs == "CCS")
     {
	      while($0 !~ /Offset/)
		{getline}
		modName=$2
		mod=$3
     }
     else
     {
        n=split(FILENAME,a,"/")
        m=split(a[n],b,"_")
        mod=b[1]
     }
 }

     
	     
/^[0-9]+/{
    if((ccs == "NOCCS") ||  (match($0,modName) || match($0,/ {}/)))
      {
       x=$1
       getline	
       gsub(/%%/,"",$0)
       nf=split($0,a,"%")
       array[x,mod]=nf-1
       errorName[x,mod]=$1
      }
}

#end error definition file

# Line counter
{ if (inline) 
	nl++
}

/^# /{  
      if ( match($0,getFileName(FILENAME)))
	{ 
	  if ($0 ~/\.C/)  EXT="C"
	  else EXT="c"
	 
#	 print "EXT["EXT"]"
	  nl=$2-1
	  inline=1
	}
     else inline = 0

}

# counter of cpp file line number
# match first line of file at every change of input file
FILENAME != fileName {
  fileName = FILENAME 
  ncpp =0
}
# increment  file line number in every line
{ncpp ++}


# Parse of source files

# get one or more lines  where errAdd is called  
/errAdd/ { 

# do not match function declaration
if ( $0 ~ /ccsCOMPL_STAT/ )
   next

# initialize errchCall to NULL
errchCall = ""

# line number in the source file where the call was found
   nline=nl
# line number in the cpp  file where the call was found
   ncppline=ncpp
# concatenate one or more lines where the call is made
   errchCall=errchCall$0
   while ($0 !~ /;[ \t]*$/)
          {
   	   getline
	   errchCall=errchCall$0
	   nl++
	   ncpp++
           }

# split into the different instructions that can be present in the errchCall string
split(errchCall,errchCallArray,";")

# isolate the errAddCall 
for ( i in errchCallArray )
{
     if(( errchCallArray[i] ~ /errAdd/) && (errchCallArray[i] !~ /ccsCOMPL_STAT/))
          errchCallString= errchCallArray[i]
     else 
	next

#debug
#print "#errchCall:"nline":"ncpp":"errchCallString


# split line in parameters and count them
	nParameter=split(errchCallString,arrayParameter,",")

# match  situations like "%s,%s"  avoiding colons between quotes that can be counted
# as an extra parameter 

        if ( match (errchCallString,/^([^"]*"[^,"]*")*([^"]*"[^,"]*,[^,"]*")+([^"]*"[^,"]*")*/ ))
	{
	   ncolons=0
	   s1=errchCallString
           while(  match(s1,/"[^"]*"/) )
           {
             s2=substr(s1,RSTART,RLENGTH)
             ncolons= ncolons+ split(s2,a,",")-1
             s2=substr(s1,RSTART+RLENGTH)
             s1=s2
           }
           nParameter=nParameter - ncolons
        }



#debug
# print every parameter in the call
#for ( i in arrayParameter)
#   print "#array["i"]="arrayParameter[i]
#print "#nParameter="nParameter
#end debug

# to support eccs calls to errAdd("eccs" ,( 0  + 30 ), ... 
# set position of parameter that contains the  error number
	if(match(errchCallString,/errAdd\("(eccs|tif|tcs|fcd|evh|lcuwd)"/))
	 param=2 
	else param=3


# set field to the parameter with error number.
      field= arrayParameter[param]

# get module name from field like  .., "seq" ,...
           if ( ccs == "NOCCS") {
              split(arrayParameter[param-1],b,"\"")
              xmod=b[2]
              }


# to support line number on eccs like errAdd calls.
# ex : errAdd("eccs",( 100  + 17 ) ,"eccsTestTools.C" ":" "186" ... 
	     if( match(arrayParameter[param+1],/"[a-zA-z]*.[cC]" ":" "[0-9]*" */))
	     { 
	     split(a[param+1],c,"\"")
	     nline=c[6]
	     }

# match format like ex.: [( 0 + 9)] ==>  (offset + n) or  just [n]
	if( field ~ /\( [0-9]+ +\+ +[0-9]+ \)/){
	   split(field,a," ") 
	   x=a[4]
	   if (ccs == "CCS"){ xmod=a[2] }
	}
        else if (field ~ /[0-9]+/  )  {
	   gsub(/ /,"",field)
	     x= field
	     if (ccs == "CCS"){ xmod = 0 }
        }
	else{
		print printCpp(ncppline)getFileName(FILENAME)"."EXT":"nline":Warning error mnemonic:["field"] not found"
		warningStatus=1
		next
	}
	if(param == 2)
	  n_param = nParameter - 3
	else n_param = nParameter - 5

        if( (x,xmod) in array)
           {if( array[x,xmod]!= n_param)
                   {
                    print printCpp(ncppline)getFileName(FILENAME)"."EXT":"nline": ERROR Definition of "errorName[x,xmod]" have "array[x,xmod]" parameters while call provides "n_param
                    errStatus=1
                   }
           }
        else{
                print printCpp(ncppline)getFileName(FILENAME)"."EXT":"nline":Warning Error definition for mod:"xmod"  Error nr.:"x" was not found"
                warningStatus=1
           } 




#debug
#print "#verbose:"verbose
if (verbose)
{ print "#  "getFileName(FILENAME)"."EXT":"nline" x["x"]  mod["xmod"] array[x,xmod]["array[x,xmod]"]  nro_errors["n_param"]"}

}


}
END {
#for ( i in array)
#{
#	n=split(i,y,SUBSEP)
#
#}
if (errStatus != 0)
     exit 2
else if (warningStatus != 0)
     exit 1
}


# getFileName extract de name of the .c file from the path
function getFileName(s)
{ n=split(s,a,"/")
  name=a[n]
  gsub(".cpp","",a[n])
  return a[n]
}

# function to print line in cppFile if verbose
function printCpp ( l )
{
 if (verbose ) { printf ("%s  : ",l) }
}

' $*

