#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDbFromTemplate.sh,v 2.27 2005/05/17 17:13:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  05/06/97  created (ported from CCD software)
# ccumani   19/05/98  Header for manpage modified
# abalestr  04/03/99  INS_USER now defaults to SYSTEM if not defined
# abalestr  12/09/03  Ported to Bourne shell
# abalestr  30/08/04  added 3rd parameter to script in order to work also if
#                     env not active. Use it carefully!!!
#

#************************************************************************
#   NAME
#   fcdDbFromTemplate.sh - Store FIERA configuration from template in DB
# 
#   SYNOPSIS
#   fcdDbFromTemplate.sh <DbFile> [<ins_root_dir>]
# 
#   DESCRIPTION
#   This shell script stores in the on-line database the values contained 
#   in a template configuration file.
#
#   <DbFile>       Name of the .dbcfg template file.
#   <ins_root_dir> root directory for the instrument the CCD belongs to
#                  Default: $INS_ROOT (env. variable)
# 
#   FILES
#   $VLTROOT/config/fcd*.dbcfg 
#   Files containing on-line database initialisation values for FIERA systems
#   known.
#   For technical CCDs there is one file for each system available. They
#   normally do not need to be changed by the user. The name is 
#   fcdTec<nn>.dbcfg whereby <nn> is the number of camera head. 
#   Example: head number 21 ---> fcdTec21.dbcfg
#   For scientific CCDs there is one file for each system to be delivered.
#   They normally have to be checked and possibly modified by experts of
#   the specific CCD system. The name is fcdSci<id>.dbcfg whereby <id> is
#   a symbolic name of the system.
#   Example: "myccd" CCD ---> fcdSciMyccd.dbcfg
#   
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   CCDNAME  CCD camera name
#   RTAPENV  FIERA Rtap environment name
#   INS_ROOT default instrument root directory
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdDbFromTemplate.sh fcdSciMyccd.dbcfg $INS_ROOT
#   Store DB configuration for "myccd" scientific CCD
#   > fcdDbFromTemplate.sh fcdSciMyccd.dbcfg
#   Store DB configuration for "myccd" scientific CCD
#
#   SEE ALSO
#   fcdInstall.sh
#
#------------------------------------------------------------------------
if [ "$#" -lt "1" ]
then
   echo "Usage: fcdDbFromTemplate.sh <DbFile> [<ins_root_dir>]"
   exit 1
fi
template=$1
NODB=0
if [ "$#" -ge "2" ]
then
  INS_ROOT=$2
fi
if [ $# = 3 ]
then
	NODB=$3
fi

# If INS_USER not defined default to SYSTEM
INS_USER=${INS_USER:="SYSTEM"}

#
# If database configuration file does not exist, create it from template
#
cd $INS_ROOT/$INS_USER/COMMON/CONFIGFILES
file=$CCDNAME.dbcfg
if [ -f $file ]
then
  mv $file $file.$$
fi
# remove comment lines, starting with ##
grep -v "##" $template > $file.tmp
# replace CWP line with the one needed for the camera used
# replace image and setup files directory according to INS_ROOT path
sed \
-e "s/@.*<alias>.*/@$RTAPENV<alias>$CCDNAME/g" \
$file.tmp > $file
rm $file.tmp


#
# Copy contents of database backup file into database
#
if [ $NODB = "0" ]
then
	fcdDcsDbInit.sh $CCDNAME
fi
