#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsDbSave.sh,v 2.130 2005/09/08 12:50:18 vltsccm Exp $"
#
# who          when      what
# --------     --------  ----------------------------------------------
# A.Balestra   04/06/97  created (ported from CCD software)
# ccumani      14/05/98  header for manpage modified
# rdonalds     26/08/98  Added file of points (fcdConfig.inp) for db save
# abalestr     04/03/99  INS_USER now defaults to SYSTEM if not defined
#

#************************************************************************
#   NAME
#   fcdDcsDbSave.sh - Save FIERA database values into backup file
# 
#   SYNOPSIS
#   fcdDcsDbSave.sh [<camera>] [<env>] [<file>]
# 
#   DESCRIPTION
#   This shell script reads some values from the FIERA WS database and
#   stores them into <file> using the utility dbBackup
#
#   <camera> camera name, also root point in DB (default env. var. CCDNAME)
#   <env>    environment where to read from (default env. var. RTAPENV)
#   <file>   name of backup file
#            (default $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<camera>.dbcfg)
# 
#   FILES
#   fcdConfig.inp     input file for utility dbBackup
#
#   ENVIRONMENT
#   CCDNAME   default for camera name (e.g. myccd)
#   RTAPENV   default environment (e.g. myws)
#   INS_ROOT  default for instrument root directory
#   INS_USER  default to SYSTEM if not defined
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   fcdDcsDbSave.sh myccd myws $INS_ROOT/$INS_USER/COMMON/CONFIGFILES/myccd.dbcfg
#
#   SEE ALSO
#   dbBackup
#
#   BUGS
#
#------------------------------------------------------------------------
#

# If INS_USER not defined default to SYSTEM
if (!($?INS_USER)) then
   setenv INS_USER SYSTEM
endif

#
# set environment variables (if not yet done)
#
if ($#argv >= 1) then
  setenv CCDNAME $argv[1]
endif
if ($#argv >= 2) then
  set environ = $argv[2]
else
  set environ = $RTAPENV
endif
if ($#argv >= 3) then
  set bckfile = $argv[3]
else if ($?INS_ROOT) then
  set bckfile = $INS_ROOT/$INS_USER/COMMON/CONFIGFILES/$CCDNAME.dbcfg
else
  echo $INS_ROOT
  exit 1
endif

echo ""
echo -n "Save DB configuration for camera $CCDNAME in $bckfile..."

#
# Save DB configuration with dbBackup
#
set file = fcdConfig.inp
if ($?INTROOT) then
  set inpFile = $INTROOT/config/$file
  if (!(-f $inpFile)) then
     set inpFile = $VLTROOT/config/$file
  else
     set inpFile = $INTROOT/config/$file
  endif
else
  set inpFile = $VLTROOT/config/$file
endif
 
dbBackup -k -c "@$environ<alias>$CCDNAME" -f $inpFile -o $bckfile
if ($status != 0) then
  echo "FAILED"
  exit 1
endif

echo "done"
echo ""
