#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsKill.sh,v 2.130 2005/09/08 12:50:18 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# abalestr  16/02/99  added fcdsimWs to the list of process to kill
# abalestr  15/10/99  fixed bug: reference to env var CCDNAME instead of 
#                     parameter camera
# abalestr  19/06/02  Modified definition of rsh command: on HP-UX you must use
#                     'remsh', on Linux 'rsh' (On Solaris both are allowed).
# ccumani   04/10/04  explicit "kill -9" of WS processes
# abalestr  29/07/05  removed reference to fcdrun and changed with FIERA_USER
# abalestr  11/08/05  fixed killing -9 as it was not actually killing anyone...
#

#************************************************************************
#   NAME
#   fcdDcsKill.sh - kills all programs belonging to FIERA DCS
# 
#   SYNOPSIS
#   fcdDcsKill.sh [<camera>] [<WS env.>] [<SLCU env.>] 
# 
#   DESCRIPTION
#   This shell script kills all programs involved in FIERA DCS module,
#   independently in which state they are or what they are doing.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV).
#              If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#              If value is 0, no action on SLCU part of FIERA sw is taken
# 
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   This script should be used with care. By killing processes
#   'blindly', the system could remain in a dangerous state. To be used
#   only to recover when the system gets stuck.
#
#   EXAMPLES
#   > fcdDcsKill.sh fcdFors wte13 lte25
#         Kill all FIERA DCS processes both at WS and SLCU level
#         for camera "fcdFors", WS environment wte13, SLCU environment lte25
#
#   > fcdDcsKill.sh fcdFors wte13 0
#         Kill all FIERA DCS processes at WS level only
#         for camera "fcdFors", WS environment wte13
#
#   > fcdDcsKill.sh fcdFors 0 lte25
#         Kill all FIERA DCS processes at SLCU level
#         for camera "fcdFors", SLCU environment lte25
#
#   SEE ALSO
#   fcdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
set null = 0
if ($#argv >= 1) then
  set camera = $argv[1]
else
  set camera = $CCDNAME
endif
if ($#argv >= 2) then
  set locEnv = $argv[2]
else
  set locEnv = $RTAPENV
endif
if ($#argv >= 3) then
  set remEnv = $argv[3]
else
  set remEnv = $CCDLENV
endif

set noErrDisplay = $?NO_ERR_DISPLAY
setenv NO_ERR_DISPLAY ""

#
# Set the INS_HOST to a sensible value (if not set)
if (!($?INS_HOST)) then
  setenv INS_HOST $HOST
endif

# Set the FIERA_USER to a sensible value (if not set)
if (!($?FIERA_USER)) then
  setenv FIERA_USER fcdrun
endif

#
# Check the Operating System
#
set OS=`uname`

# Set OS dependant commands
if ($OS == "HP-UX") then
        set RSH = "/usr/bin/remsh"
else
        set RSH = "/usr/bin/rsh"
endif

# log information that programs are going to be killed
set module = fcdws
if ($locEnv != 0) then
  set platform = "WS"
  if ($remEnv != 0) then
    set platform = "WS and SLCU"
  endif
else if ($remEnv != 0) then
  set platform = "SLCU"
else  
  set platform = ""
endif
logUserData $module 0 "Killing all $platform programs for camera $camera"

#
# 1) kill all WS programs by sending a KILL command
#
if ($locEnv != 0) then
  set process = fcdconCI_$camera
  echo -n "Kill FIERA WS command interface process for camera $camera ..."
  msgSend -n $locEnv $process "KILL" "" >/dev/null  
  if ($status != 0) then
    echo "FAILED"
  else
    echo "done"
  endif
  set PROCESSID=`ps -eaf | grep $process | grep -v grep | awk '{print $2}'`
  if ($?PROCESSID) then
      kill -9 $PROCESSID > /dev/null
  endif

  set process = fcditWs_$camera
  echo -n "Kill FIERA WS image transfer process for camera $camera ..."
  msgSend -n $locEnv $process "KILL" "" >/dev/null  
  if ($status != 0) then
    echo "FAILED"
  else
    echo "done"
  endif
# be rude now...
  set process = fcditWs
  set PROCESSID=`ps -eaf | grep $process | grep $camera | grep -v grep | awk '{print $2}'`
  if ($?PROCESSID) then
      kill -9 $PROCESSID > /dev/null
  endif


# Kill also simulation process (just in case...)
  set process = fcdsimWs_$camera
  echo -n "Kill FIERA WS simulation process for camera $camera ..."
  msgSend -n $locEnv $process "KILL" "" >/dev/null  
  if ($status != 0) then
    echo "FAILED"
  else
    echo "done"
# be rude now...
  set process = fcdsimWs
  set PROCESSID=`ps -eaf | grep $process | grep $camera | grep -v grep | awk '{print $2}'`
 endif
  set PROCESSID=`ps -eaf | grep $process | grep -v grep | awk '{print $2}'`
  if ($?PROCESSID) then
      kill -9 $PROCESSID > /dev/null
  endif

endif

#
# 2) kill all SLCU programs by executing the appropriate script
#
if ($remEnv != 0) then
    # Work out the host name
    set remHost = `fcdEnvToHost $remEnv`

    echo "Killing SLCU S/W on host $remHost"

    set run = "fcdSlcuCleanCamera $camera $INS_HOST $INS_ROOT $INS_USER" 
    echo ""
    echo -n "Killing SLCU processes for camera $camera ..."
    if ($remHost == $INS_HOST) then
	$run
    else
	$RSH $remHost -l $FIERA_USER $run
    endif
    if ($status != 0) then
	echo "FAILED"
	exit 1
    endif
endif
if ($noErrDisplay == 0) then
  unsetenv NO_ERR_DISPLAY
endif








