#! /bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsResetDsp.sh,v 2.130 2005/09/08 12:50:23 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  08/06/05  created
#
#************************************************************************
#   NAME
#   fcdDcsResetDsp.sh - Reset DSP on the SLCU
# 
#   SYNOPSIS
#   fcdDcsResetDsp.sh remote_environment
# 
#   DESCRIPTION
#   Shell script to remotely reset DSP on the SLCU. 
# 
#   FILES
#
#   ENVIRONMENT
#   FIERA_USER (optional)
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------

# Get remote environment parameter
if [ $# != "1" ]
then
	echo "Usage: fcdDcsResetDsp remoteEnvironment"
	exit 1
fi
remoteEnv=$1

# Set the FIERA_USER to a sensible value (if not set)
remoteUser=${FIERA_USER:-fcdrun} 

#
# Check the Operating System
#
OS=`uname`

# Set OS dependant commands
if [ ${OS} = "HP-UX" ]
then
        RSH="/usr/bin/remsh"
else
        RSH="/usr/bin/rsh"
fi

# Work out the host name of the SPARC
remHost=`fcdEnvToHost $remoteEnv`

# Give an error if remote host unknown
if [ ${remHost} = "Unknown" -o $? = 1 ]
then
	echo "Remote Environment $remoteEnv Unknown"
	exit 1 
fi

# Command to be executed remotely
remoteCommand="$RSH $remHost -l $remoteUser"

# Actually execute
a=`$remoteCommand fcdDSPReset`

# Check for errors
b=`echo $a | grep Error`
if [ "${b}" != "" ]
then
	echo "Remote Error: $b"
	exit 1
fi

# last check on completion of rsh command
if [ $? != "0" ]
then
	echo "Error running command: \"$remoteCommand fcdDSPReset\""
	exit 1
fi

# Done
exit 0

