#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsSetLogMask.sh,v 2.130 2005/09/08 12:50:22 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   16/09/97  created
# ccumani   14/05/98  header for manpage modified
# abalestr  05/12/00  force snap using dbForceSnap that works both on CCS 
#                     and CCS_LITE
#

#************************************************************************
#   NAME
#   fcdDcsSetLogMask.sh - Set FITS logging mask in OLDB for CCD camera
# 
#   SYNOPSIS
#   fcdDcsSetLogMask.sh <camera> <WS env.> <logMask>
# 
#   DESCRIPTION
#   This shell script writes in the WS CCD OLDB the value of the attribute
#   related to the FITS logging of main info.
#   An update of the $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<camera>.dbcfg file 
#   is performed. As well a snap of the WS CCD OLDB is made.
#   It is meant as a utility to applications having to configure the
#   logMask parameter for the camera used.
#
#   <camera>     camera name, also root point in DB
#   <WS env.>    name of workstation environment
#   <logMask>    Mask for FITS logging           
# 
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   EXAMPLES
#   > fcdDcsSetLogMask.sh myccd myws UT1AGA
#         FITS Logs [UT1AGA] 
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
if ($#argv < 3) then
  echo "Usage: fcdDcsSetLogMask.sh <camera> <WS env.> <logMask>"
  exit 1
endif
set camera  = $argv[1]
set locEnv  = $argv[2]
set logMask = $argv[3]

#
# Write values in the DB and do a snapshot
#
dbWrite "<alias>${camera}.logMask" $logMask
fcdDcsDbSave.sh $camera $locEnv
dbForceSnap -e $locEnv
