#! /bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsSlcuHalt.sh,v 2.130 2005/09/08 12:50:22 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# ccumani   21/04/98  created
# ccumani   14/05/98  header for manpage modified
# abalestr  19/06/02  added possibility of Linux os
#

#************************************************************************
#   NAME
#   fcdDcsSlcuHalt.sh - halt FIERA SLCU
# 
#   SYNOPSIS
#   fcdDcsSlcuHalt.sh [<SLCU env.>]
# 
#   DESCRIPTION
#   This shell script performs a halting of FIERA SLCU.
#
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#               If value is 0, no action on FIERA SLCU is taken
# 
#   FILES
#
#   ENVIRONMENT
#   CCDLENV  default for SLCU environment (e.g myfiera) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdDcsSlcuHalt.sh myfiera
#         Halt the FIERA SLCU defined by the "myfiera" environment
#
#   > fcdDcsSlcuHalt.sh
#         Halt the FIERA SLCU defined by the CCDLENV environment
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------

SUCCESS=0
FAILURE=1

echo ""

#
# Set the Remote SLCU Environment
#
if [ $# -gt 0 ]
then
    REMENV=$1
else
    REMENV=${CCDLENV="0"}
fi


#
# Set the Remote SLCU Host
#
if [ $REMENV = "0" ]
then
    echo "*** ERROR: remote environment not set"
    echo "*** ACTION: define CCDLENV value or run script with parameter"
    echo ""; exit $FAILURE
else
    SLCUHOST=`fcdEnvToHost $REMENV`
fi

if [ $SLCUHOST = "Unknown" ]
then
    echo "*** ERROR: environment $RTAPENV is not recognized"
    echo "*** ACTION: check CCDLENV value or script parameter"
    echo ""; exit $FAILURE
fi


#
# Check the Operating System
#
OS=`uname`
case $OS in
        "HP-UX") FCDOS=hp;;
        "SunOS") FCDOS=sun;;
        "Linux") FCDOS=linux;;
        *) echo "*** ERROR: unknown operating system"; echo ""; exit $FAILURE;;
esac


#
# Ping the Remote SLCU
#
echo "==> Ping SLCU $SLCUHOST ..."
if [ $FCDOS = "hp" ]
then
        a=`/etc/ping $SLCUHOST -n 3`
else
        a=`/usr/sbin/ping -s $SLCUHOST 56 3`
fi

if [ "`echo $a | grep \"unknown host\"`" != "" ] 
then 
        echo "*** ERROR: $SLCUHOST is an unknown host"
        echo "*** ACTION: check SLCUHOST value"
        echo "*** ACTION: check hosts database (/etc/hosts, DNS, NIS...)"
        echo ""; exit $FAILURE
elif [ "`echo $a | grep \"100%\"`" ]
then
        echo "*** ERROR: $SLCUHOST is not reachable"
        echo "*** ACTION: check remote host"
        echo "*** ACTION: check network"
        echo ""; exit $FAILURE
fi
echo "==> SLCU $SLCUHOST is alive"


#
# Halt the Remote SLCU
#
echo "==> Halting SLCU $SLCUHOST"
rlogin $SLCUHOST -l halt
echo "==> Waiting (be patient !) ... "
sleep 60
echo "==> $SLCUHOST ready to be powered off "


#
# Return
#
echo ""; exit $SUCCESS


#__oOo__








