#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsSlcuReboot.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# ccumani   14/05/98  header for manpage modified
# abalestr  28/06/99  changed behaviour when pinging cmdManager: now it pings 
#                     once every 12 secs for a maximum of 40 times
# abalestr  19/06/02  Modified definition of rsh command: on HP-UX you must use
#                     'remsh', on Linux 'rsh' (On Solaris both are allowed).
#

#************************************************************************
#   NAME
#   fcdDcsSlcuReboot.sh - Reboots FIERA DCS SLCU
# 
#   SYNOPSIS
#   fcdDcsSlcuReboot.sh [<camera>] [<WS env.>] [<SLCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script reboots the FIERA SLCU, independently from what is going
#   on there.
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>   name of workstation environment (default env. var. RTAPENV).
#               If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#               If value is 0, no action on SLCU part of FIERA sw is taken
#   <option>    s - restart also the scan system
# 
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   This script should be used with care. If anything is going on in the
#   SLCU it will get lost !!!!
#
#   EXAMPLES
#   > fcdDcsSlcuReboot.sh myccd 0 myfiera
#         Reboot SLCU defined by the "myfiera" environment, used for
#         camera "myccd"
#
#   > fcdDcsSlcuReboot.sh myccd myws myfiera s
#         Reboot SLCU defined by the "myfiera" environment, used for
#         camera "myccd" and restart the scan system
#
#   SEE ALSO
#   fcdDcsScan.sh
#
#   BUGS
#------------------------------------------------------------------------
set module = fcdws
set option = 0
set scan = 0
set message = ""
if ($#argv >= 1) then
  set camera = $argv[1]
else
  set camera = $CCDNAME
endif
if ($#argv >= 2) then
  set locEnv = $argv[2]
else
  set locEnv = $RTAPENV
endif
if ($#argv >= 3) then
  set remEnv = $argv[3]
else
  set remEnv = $CCDLENV
endif
if ($#argv >= 4) then
  set option = $argv[4]
  if ($option == "s") then
    set scan = 1
  endif
endif
set noErrDisplay = $?NO_ERR_DISPLAY

#
# Check the Operating System
#
set OS=`uname`

# Set OS dependant commands
if ($OS == "HP-UX") then
        set RSH = "/usr/bin/remsh"
else
        set RSH = "/usr/bin/rsh"
endif

#
# Set in FIERA SLCU DB camera status to OFF
#
  echo ""
  echo -n "Setting to OFF status of camera $camera in SLCU DB $remEnv..."
  # This is done in two stages otherwise the evaluation of the ":" characters
  # gets in the way
  set var="@${remEnv}:<alias>${camera}.opStateSlcu"
  dbWrite "$var" 1
  if ($status == 0) then
    echo "done"
  else
    echo "FAILURE"
  endif

#
# execute reboot by remotely logging onto the machine as the reboot
# user
#
set remHost = `fcdEnvToHost $remEnv`
set message = "Rebooting SLCU $remEnv on host $remHost"
echo ""
echo -n "$message..."
# log information that SLCU is going to be booted
logUserData $module 0 "$message"
setenv NO_ERR_DISPLAY ""
# 
# Perform the reboot by using the reboot user account
$RSH $remHost -l reboot

#
# Now we try and wait for the environment on the remote machine to startup
# 
# First sleep for a while. It will take this long at least
set waitIndex = 40
set waitTime = 12
#echo "Sleeping for $waitIndex periods of $waitTime seconds for SLCU to boot"
#while ($waitIndex)
#    echo -n "$waitIndex, "
#    sleep $waitTime
#    @ waitIndex--
#end
#
# Try and contact the cmdManager in the remote environment
# 
set notRegistered = 1
set time = 1
set timeout = 40
set msgSendTimeout = 5000

echo "Trying to contact cmdManager in environment $remEnv on host $remHost"
echo "We will sleep for $waitTime seconds between each try for a maximum of $timeout attempts"

while ($notRegistered)
  echo "Sending PING to ENV $remEnv cmdManager"
  msgSend -n $remEnv cmdManager "PING" "" $msgSendTimeout >/dev/null
  if (($status == 0) || ($time == $timeout)) then
    set notRegistered = 0
  else
    echo "$time - Sleeping $waitTime seconds..."
    sleep $waitTime
    @ time++
  endif
end
# Terminate the output above
echo ""
if ($time == $timeout) then
    echo "Waiting for remote environment to be active failed"
    exit 1
endif

# If we have arrived here then the process must
# have been successfull
if ($noErrDisplay == 0) then
  unsetenv NO_ERR_DISPLAY
endif
#
# Restart the scan system if requested
#
if ($scan == 1) then
  fcdDcsScan.sh ${camera} ${locEnv} ${remEnv} c
  fcdDcsScan.sh ${camera} ${locEnv} ${remEnv} a
  if ($status != 0) exit $status
endif  
