#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsSlcuSim.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
#

#************************************************************************
#   NAME
#   fcdDcsSlcuSim.sh - Set FIERA SLCU sw simulation state
# 
#   SYNOPSIS
#   fcdDcsSlcuSim.sh [<camera>] [<WS env.>] [<SLCU env.>]
# 
#   DESCRIPTION
#   This shell script do all actions needed to set the simulation mode
#   of the FIERA SLCU sw using CCS facilities.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#              If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#              If value is 0, SLCU sw is simulated, otherwise not
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. fcdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for SLCU environment (e.g lte25) 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdDcsSlcuSim.sh fcdFors wte13 lte25
#         The cmdManager running on WS environment wte13 is informed that
#         the FIERA SLCU sw for camera "fcdFors" is running on lte25 and must
#         not be simulated.
#
#   > fcdDcsSlcuSim.sh fcdFors wte13 0
#         The cmdManager running on WS environment wte13 is informed that
#         the FIERA SLCU sw for camera "fcdFors" must be simulated.
#
#   SEE ALSO
#   fcdDcsStart.sh
#
#   BUGS
#------------------------------------------------------------------------
if ($#argv >= 1) then
  setenv CCDNAME $argv[1]
endif
if ($#argv >= 2) then
  setenv RTAPENV $argv[2]
endif
if ($#argv >= 3) then
  setenv CCDLENV $argv[3]
endif
#
# Inform cmdManager if SLCU processes are simulated or not
#
if ($CCDLENV != 0) then
  set msg = "Setting SLCU processes for camera $CCDNAME NOT in simulation ..."
  set command = "SIMUSET"
  setenv CCSSTATE NORMAL
else
  set msg = "Setting SLCU processes for camera $CCDNAME in simulation ..."
  set command = "SIMSET"
  setenv CCSSTATE SIMULATION
endif
echo ""
echo -n $msg

echo "Should be setting fcdit_$CCDNAME into simulation mode now"
msgSend -n $RTAPENV cmdManager SIMSET "fcdit_$CCDNAME" >/dev/null
#msgSend -n $RTAPENV cmdManager $command "fcdit_$CCDNAME" >/dev/null
#if ($status != 0) then
#  echo "FAILED"
#  exit 1
#endif
msgSend -n $RTAPENV cmdManager $command "fcdcon_$CCDNAME" >/dev/null
if ($status != 0) then
  echo "FAILED"
  exit 1
endif
echo "done"
echo ""
#
# Set FIERA SLCU sw state to invalid high value, in order to have the global
# state, returned for example by command STATUS, set to the FIERA WS opState
#
if ($CCDLENV == 0) then
  echo ""
  echo -n "Setting FIERA SLCU operational state to simulation ..."
  dbWrite "<alias>$CCDNAME.opStateSlcu" "6"
  if ($status != 0) then
    echo "FAILED"
    exit 1
  endif
  echo "done"
  echo ""
endif








