#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsWarmStart.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# rdonalds  16/01/98  Now looks for the failure indication in DB during start
# rdonalds  29/01/98  Added some logUserData logs
# rdonalds  04/02/98  Removed the setting of SLCU_INS_ROOT. This is now defined
#		      in the fcdrun users .cshrc.local file on the SPARC
# ccumani   14/05/98  header for manpage modified
# abalestr  27/08/98  now checks database to start it process for image 
#                     extensions or old-style
# abalestr  28/08/98  PING command substituted with VERSION command for ws 
#                     processes as evh processes with sockets (i.e. 
#		      fcditWsImExt) crash with the PING command.
# abalestr  04/09/98  added start of process for ws simulation
# rdonalds  07/09/98  Was checking the point failureSlcu possibly before
#                     the Scan system was enabled. Now rejects the point
#                     if wuality is bad
# abalestr  12/11/98  fixed spr VLTSW980566 and VLTSW980580
# abalestr  16/02/99  fixed spr VLTSW980694
# abalestr  03/03/99  re-fixed spr VLTSW980566 as previous fix was not working 
#                     if CCDLENV=0
# abalestr  23/02/00  FCD_IT_HOST set to default of remote SLCU hostname.
#                     No need for such a variable because only one connection
#                     is allowed.
# ccumani   14/03/01  manpage corrected (reference to INTROOT deleted)
# abalestr  19/06/02  Modified definition of rsh command: on HP-UX you must use
#                     'remsh', on Linux 'rsh' (On Solaris both are allowed).
# ccumani   19/11/02  check of SW versions on WS and SLCU returns only
#                     a WARNING, if they are different
# ccumani   15/01/03  usage of FIERA_USER introduced (SPR VLTSW20020695)
# abalestr  20/02/03  All processes are now started in background through nohup
# abalestr  07/04/03  Modified script in order to run on Linux and in order to
#                     have both ws and slcu on the same machine.
# abalestr  01/03/04  Added setting in the oldb of the lcu cpu architecture
# abalestr  13/08/04  Now SLCU sw is started with same INS_ROOT as ws
# abalestr  16/08/04  RSH is now done only if necessary, i.e. if the SLCU host 
#                     is different from the ws host
# ccumani   22/11/04  starting of fcdSlcuStartupCamera modified
# ccumani   01/12/04  remoteCommand defined also in the simulation case
# abortolu  13/01/05  quick change in WS simulation management (TO BE IMPROVED)
# abalestr  14/06/05  added parameter for external dsp reset and point in db
#
#************************************************************************
#   NAME
#   fcdDcsWarmStart.sh - warm startup of FIERA DCS
# 
#   SYNOPSIS
#   fcdDcsWarmStart.sh [<camera>] [<WS env.>] [<SLCU env.>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a warm startup of FIERA DCS.
#   No FIERA stand-alone module is started.
#   To be run once at startup only!
#   It performs the following steps:
#     1 - Terminate all running processes
#     2 - Set SLCU sw in appropriate simulation mode
#     3 - Schedule processes
#     4 - Enable scanning of data from SLCU to WS
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>   name of workstation environment (default env. var. RTAPENV)
#               If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#               If value is 0, no action on SLCU part of FIERA sw is taken
#   kill        kill all already running processes before starting          
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. myccd)
#   RTAPENV  default for WS local environment (e.g. myws)
#   CCDLENV  default for SLCU environment (e.g myfiera)
#   INS_USER default SYSTEM
#   INS_HOST Host where the INS_ROOT directory is mounted. Default HOST
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdDcsWarmStart.sh myccd myws myfiera
#         Start the FIERA DCS sw both at WS and SLCU level
#         for camera "myccd", WS environment "myws", SLCU environment myfiera"
#
#   > fcdDcsWarmStart.sh myccd myws 0
#         Start the FIERA DCS sw at WS level only
#         for camera "myccd", WS environment "myws"
#
#   > fcdDcsWarmStart.sh myccd myws 0 kill
#         Kill and restart the FIERA DCS sw at WS level only
#         for camera "myccd", WS environment "myws"
#
#   > fcdDcsWarmStart.sh myccd 0 myfiera
#         Start the FIERA DCS sw at SLCU level only
#         for camera "myccd", SLCU environment "myfiera"
#
#   SEE ALSO
#   fcdDcsStart.sh fcdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
set killCcd = ""
set dspExternalReset = ""
if ($#argv >= 1) then
  setenv CCDNAME $argv[1]
endif
if ($#argv >= 2) then
  setenv CCDWENV $argv[2]
else
  setenv CCDWENV $RTAPENV
endif
if ($#argv >= 3) then
  setenv CCDLENV $argv[3]
endif
if ($#argv >= 4) then
  set killCcd = $argv[4]
endif
if ($#argv >= 5) then
# we want that only a value of 1 set sync on
  if ($argv[5] == 1) then
    set dspExternalReset = $argv[5]
  else
    set dspExternalReset = ""
  endif
endif
# If INS_USER is NOT defined, fcdconCI would crash on slx call
if (!($?INS_USER)) then
  setenv INS_USER SYSTEM
endif
# Set the INS_HOST to a sensible value (if not set)
if (!($?INS_HOST)) then
  setenv INS_HOST $HOST
endif
# Set the SLCU_DISPLAY to a sensible value (if not set)
if (!($?SLCU_DISPLAY)) then
  setenv SLCU_DISPLAY ""
endif
# Set the SLCU_DEBUG_LEVEL to a sensible value (if not set)
if (!($?SLCU_DEBUG_LEVEL)) then
  setenv SLCU_DEBUG_LEVEL "0"
endif

# Set the FIERA_USER to a sensible value (if not set)
if (!($?FIERA_USER)) then
  setenv FIERA_USER fcdrun
endif

#
# Check the Operating System
#
set OS=`uname`

# Set OS dependant commands
if ($OS == "HP-UX") then
        set RSH = "/usr/bin/remsh"
else
        set RSH = "/usr/bin/rsh"
endif

set noErrDisplay = $?NO_ERR_DISPLAY

set tmpMode = `dbRead "<alias>$CCDNAME.opMode"`
set opMode = $tmpMode[4]

if (($CCDLENV == 0) || ($opMode == 3)) then
  # $CCDLENV = $RTAPENV
  setenv CCDLENV $RTAPENV
endif


# Work out the host name of the SPARC
setenv remHost `fcdEnvToHost $CCDLENV`

# Work out also our host name
setenv localHost `fcdEnvToHost $RTAPENV`

# If they are the same, we are in simulation on the iws, so "sh -c" is needed
# otherwise
#    nohup $remoteCommand "$run >/dev/null 2>/dev/null </dev/null"
# below will not work, since it will appear as
#    nohup  "$run >/dev/null 2>/dev/null </dev/null"
# with the result "Command not found"
if ($remHost == $localHost) then
	set remoteCommand = "sh -c"
else
	set remoteCommand = "$RSH $remHost -l $FIERA_USER"
endif


echo "fcdDcsWarmStart CCDNAME $CCDNAME CCDWENV $CCDWENV CCDLENV $CCDLENV remHost $remHost"

# Inform of progress
logUserData "fcd" 0 "fcdDcsWarmStart starting for $CCDNAME"

#
# Set in the db my endianness
#
set wsLittleEndian=`fcdCheckEndianness`
dbWrite "<alias>${CCDNAME}.wsLittleEndian" "$wsLittleEndian"
#
# Set in the db the endianness of the lcu
#
if ($CCDLENV != 0 && $CCDLENV != $RTAPENV && $remHost != $localHost) then
    set lcuLittleEndian=`$remoteCommand fcdCheckEndianness`
    dbWrite "<alias>${CCDNAME}.lcuLittleEndian" "$lcuLittleEndian"
else
    dbWrite "<alias>${CCDNAME}.lcuLittleEndian" "$wsLittleEndian"
endif

#
# 1 - Terminate all running processes. If requested, kill them
# 
if (($CCDWENV != 0) || ($CCDLENV != 0)) then
# Since this is a clean-up procedure, possible failure on not running processes
# must be ignored. The env. var. NO_ERR_DISPLAY must be set to avoid that
# errDisplay blocks execution with dialog box
  setenv NO_ERR_DISPLAY ""
  fcdDcsStop.sh $CCDNAME $CCDWENV $CCDLENV $killCcd
  if ($noErrDisplay == 0) then
    unsetenv NO_ERR_DISPLAY
  endif
endif

#
# 2 - If WS simulation, disable scan system (if active) and set exposure status
#     to "idle"
#     
if ($CCDLENV == $RTAPENV) then
  # If scan system active, disable it
  set st = `dbRead "<alias>ccs_config:scan config:LAN:${CCDLENV}.scan input status"`
  set ct = `dbRead "<alias>ccs_config:scan config:LAN:${CCDLENV}.scan input control"`
  if (($ct[$#ct] == 1) && ($st[$#st] == 1)) then
    fcdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV c
    fcdosScan.sh $CCDNAME $RTAPENV $CCDLENV c
    scanDevOff $CCDLENV
  endif
endif

#
# Check the environment for running. 
#    Ensure that the data is in INS_ROOT
#    Ensure the local and remote rtap databases are running
#
fcdDcsCheckEnv.sh
if ($status != 0) then
   echo "FATAL ERROR Environment is wrong"
   exit 1
endif

#
# Check if it is required that the DSP gets the reset by high level software.
# If yes, set point dspExternalReset to TRUE. Set to false in all other cases.
# The external reset will guarantee full synchronization e.g. in omegacam.
#
if ($dspExternalReset == 1) then
  dbWrite "<alias>${CCDNAME}.dspExternalReset" 1
else
  dbWrite "<alias>${CCDNAME}.dspExternalReset" 0
endif

#
# 3 - Set SLCU sw in appropriate simulation mode (at CCS Message System level)
#
fcdDcsSlcuSim.sh $CCDNAME $RTAPENV $CCDLENV

#
# 4 - Schedule processes
#
if ($CCDLENV != 0) then
#
# If SLCU environemnt specified, schedule program fcdconStartupCamera on
# SLCU
  set run = "fcdSlcuStartupCamera $CCDNAME $opMode $CCDLENV $INS_HOST $INS_USER $RTAPENV $INS_ROOT CCDDCS FCDDCS $dspExternalReset"
  echo ""
  echo "Scheduling FIERA SLCU processes for camera $CCDNAME ..."
  #
  # We have to run this in the background because the RSH command
  # hangs until all the processes spawned on the remote machine are
  # finished
  #
  # This of course means that we need to wait until the script has
  # realy started on the remote machine. Luckily we will start pinging
  # the remote processes in the stage below

  # Inform of progress
  logUserData "fcd" 0 "fcdDcsWarmStart starting processes on $remHost"
  echo "Executing Fiera SPARC CCD S/W on machine $remHost"
  echo "Executing nohup $remoteCommand $run"
  nohup $remoteCommand "$run >/dev/null 2>/dev/null </dev/null"  >/dev/null &

  # sleep while the remote environment is cleaned up and checked
  sleep 20
  if ($status != 0) then
	echo "FAILED"
	exit 1
  endif

  setenv NO_ERR_DISPLAY ""
  set notRegistered = 1
# Normally it takes a few seconds for the processes to startup. However if
# we have started up remotely (using the &) above. Then we will have to
# wait for the whole script to execute before the processes start
  set timeout = 30
  set time = 1
  set process = fcdcon_$CCDNAME
# Sometime the SLCU process terminates abnormally and cannot reply. It is
# therefore necessary to add a timeout to msgSend, otherwise this script,
# and the panel calling it, if any, would hang forever
  sleep 5
  set msgSendTimeout = 10000
  while ($notRegistered)
    echo "Sending PING to ENV $CCDLENV process $process"
    msgSend -n $CCDLENV $process "PING" "" $msgSendTimeout >/dev/null
    if (($status == 0) || ($time == $timeout)) then
      set notRegistered = 0
    else
      sleep 1
      @ time++
    endif

    # See if the SLCU has reported a failure via the db
    set tmpFail = `dbRead "<alias>$CCDNAME.failureSlcu"`
    if ($#tmpFail >= 4) then
      set fail = $tmpFail[4]
  
      if ($fail == 1) then
         set notRegistered = 0  
         set time = $timeout  
      endif
    endif

  end
  if ($noErrDisplay == 0) then
    unsetenv NO_ERR_DISPLAY
  endif
  if ($time == $timeout) then
    echo "FAILED"
    fcdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV e
    exit 1
  else
    echo "done"
    echo ""
  endif
else
  setenv CCSSTATE SIMULATION
endif

if ($CCDWENV != 0) then
#
# schedule program fcdconCI (high-level coordination) on WS
#
  nohup fcdconCI $CCDNAME > /dev/null &
#
# schedule program fcditWs (image transfer) on WS
#
  set imExt = `dbRead "<alias>${CCDNAME}:images:transfer.imageExtensions"`
  if ($imExt[$#imExt] == 0) then
        logUserData "fcd" 0 "Image extensions will NOT be used..."
  	echo "Image extensions will NOT be used..."
	set FCDITWSBIN = "fcditWs"
  else
        logUserData "fcd" 0 "Image extensions WILL be used..."
  	echo "Image extensions WILL be used..."
	set FCDITWSBIN = "fcditWsImExt"
	if ($CCDLENV == 0) then
                echo "Starting process for ws simulation..."
                setenv FCD_IT_HOST $HOST
                fcdsimWs $CCDNAME &
	else
                setenv FCD_IT_HOST `fcdEnvToHost $CCDLENV`
	endif
  endif
  nohup $FCDITWSBIN $CCDNAME >/dev/null &
  echo ""
  echo -n "Scheduling FIERA WS processes for camera $CCDNAME ..."
#
# Check that processes registered to CCS (5 secs timeout)
#
  setenv NO_ERR_DISPLAY ""
  set notRegistered = 1
  set timeout = 5
  set time = 1
  set process = fcdconCI_$CCDNAME
  set msgSendTimeout = 5000
  while ($notRegistered)
# We use VERSION command instead of PING because, due to a bug in eccs, the PING
# command makes eccs processes crash on hp ws...
    msgSend -n $CCDWENV $process "VERSION" "" $msgSendTimeout >/dev/null
    if (($status == 0) || ($time == $timeout)) then
      set notRegistered = 0
    else
      sleep 1
      @ time++
    endif
    if (($notRegistered == 0) && ($process == "fcdconCI_$CCDNAME")) then
      set process = fcditWs_$CCDNAME
      set notRegistered = 1      
    endif
  end
  if ($noErrDisplay == 0) then
    unsetenv NO_ERR_DISPLAY
  endif
  if ($time == $timeout) then
    echo "FAILED"
    exit 1
  else
    echo "done"
    echo ""
  endif
  
# Check versions
  if ($CCDLENV != 0) then
    set wsAnswer = `msgSend -n $CCDWENV fcdconCI_$CCDNAME "VERSION" "" $msgSendTimeout`
    set slcuAnswer = `msgSend -n $CCDLENV fcdcon_$CCDNAME "VERSION" "" $msgSendTimeout`
    if ("$wsAnswer" != "$slcuAnswer") then
      logUserData "fcd" 0 "Software version running on ws differs from slcu"
      echo "WARNING:"
      echo "Software version running on ws differs from slcu version"
      echo "ws answer to version command ==> $wsAnswer"
      echo "slcu answer to version command ==> $slcuAnswer"
      echo "This should not be, but if you are happy about that ... "
      echo " ... I go on with the startup process"
    endif
  endif

endif
#
# 5 - Enable scanning of data from SLCU to WS
#
if (($CCDWENV != 0) && ($CCDLENV != 0)) then 
  fcdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV e
  if ($status != 0) exit $status
endif

