#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsWcs.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# ccumani   14/05/98  header for manpage modified
# abalestr  05/12/00  force snap using dbForceSnap that works both on CCS 
#                     and CCS_LITE
# ccumani   17/01/02  exit added (SRP VLTSW20020030)
#

#************************************************************************
#   NAME
#   fcdDcsWcs.sh - Set World Coordinate values in OLDB for CCD camera
# 
#   SYNOPSIS
#   fcdDcsWcs.sh <camera> <WS env.> <xrefpix> <yrefpix>
#                <secpix> <rotate> <equinox> <epoch> <proj>
# 
#   DESCRIPTION
#   This shell script writes in the WS FIERA OLDB the values of attributes
#   related to the display of World Coordinates with rtd.
#   It is meant as a utility to applications having to configure the
#   WCS parameters for the camera used.
#
#   <camera>     camera name, also root point in DB
#   <WS env.>    name of workstation environment
#   <xrefpix>    X Coordinate of Reference Pixel
#   <yrefpix>    Y Coordinate of Reference Pixel
#   <secpix>     Number of arcseconds per pixel
#   <rotate>	 Rotation angle (clockwise positive) in degrees
#   <equinox>	 Equinox of coordinates, 1950 and 2000 supported
#   <epoch>	 Epoch of coordinates, used for FK4/FK5 conversion
#   <proj>	 Projection
# 
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   EXAMPLES
#   > fcdDcsWcs.sh myccd myws 200.0 145.0 0.3 0.0 1950 0.0 PIXEL
#         Reference pixel [200.0;145.0]
#         Each pixel covers 0.3 arcseconds
#         Rotation angle is 0.0 degrees
#         Equinox of ccordinates set to 1950
#         Epoch of ccordinates set to 0.0
#         Projection set to PIXEL
#
#   SEE ALSO
#   rtdImageEvent.h for more information about parameters meaning and
#                   possible values
#
#   BUGS
#------------------------------------------------------------------------
if ($#argv < 9) then
  echo "Usage: fcdDcsWcs.sh <camera> <WS env.> <xrefpix> <yrefpix> <secpix> <rotate> <equinox> <epoch> <proj>"
  exit 1
endif
set camera  = $argv[1]
set locEnv  = $argv[2]
set xRefPix = $argv[3]
set yRefPix = $argv[4]
set secPix  = $argv[5]
set rotate  = $argv[6]
set equinox = $argv[7]
set epoch   = $argv[8]
set proj    = $argv[9]

#
# Write values in the DB and do a snapshot
#
dbWrite "<alias>${camera}:wcs.xRefPix" $xRefPix
dbWrite "<alias>${camera}:wcs.yRefPix" $yRefPix
dbWrite "<alias>${camera}:wcs.secPix" $secPix
dbWrite "<alias>${camera}:wcs.rotate" $rotate
dbWrite "<alias>${camera}:wcs.equinox" $equinox
dbWrite "<alias>${camera}:wcs.epoch" $epoch
dbWrite "<alias>${camera}:wcs.proj" $proj
dbForceSnap -e $locEnv

exit 0
