#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdStart.sh,v 2.27 2005/05/17 17:13:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  05/06/97  created (ported from CCD software)
# ccumani   19/05/98  Header for manpage modified
# ccumani   07/05/01  check of existence of RTAPENV, CCDNAME, CCDLENV
#                     and INS_ROOT added
# abalestr  18/12/01  Check of existence of $CCDNAME branch taken out (see
#   		      SPR no. VLTSW20010758
# abalestr  12/06/03  Ported to Bourne shell
# ccumani   19/01/04  Checking environment modified for Linux
#

#************************************************************************
#   NAME
#   fcdStart.sh - start the FIERA stand-alone sw
# 
#   SYNOPSIS
#   fcdStart.sh [<camera>] [<WS env.>] [<SLCU env.>] [<INS root>] [<simulation>]
# 
#   DESCRIPTION
#   This shell script pops up the panels from which the operations with
#   the FIERA stand-alone can be executed.
#   It also loads the on-line database values for the selected camera.
#
#   <camera>     camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>    name of workstation environment (default env. var. RTAPENV)
#   <SLCU env.>  name of SLCU environment (default env. var. CCDLENV)
#   <INS root>   INS_ROOT environment variable
#   <simulation> 1 = panels are entered in simulation mode: no action done.
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. myccd)
#   RTAPENV  default for WS local environment (e.g. myws)
#   CCDLENV  default for SLCU environment (e.g myfiera) 
#   INS_ROOT default root directory for instrument data 
#   INS_USER default SYSTEM
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   fcdStart.sh myccd myws myfiera $INS_ROOT 1
#   Start FIERA sw for camera "myccd", WS environment "myws",
#   SLCU environment "myfiera". Panels run in simulation.
#
#   SEE ALSO
#   fcdDcsStart.sh fcdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
#
# WS part
#
#
# set environment variables (if not yet done)
#
sim=0
if [ $# -ge 1 ] 
then
  CCDNAME=$1
fi
if [ $# -ge 2 ] 
then
  RTAPENV=$2
fi
if [ $# -ge 3 ] 
then
  CCDLENV=$3
fi
if [ $# -ge 4 ] 
then
  INS_ROOT=$4
fi
if [ $# -ge 5 ] 
then
  sim=$5
fi

# Set default values for unset environment variables
CCDDID=${CCDDID:="CCDDCS"}
export CCDDID
FCDDID=${FCDDID:="FCDDCS"}
export FCDDID
INS_USER=${INS_USER:="SYSTEM"}
export INS_USER

# Check existence of environment variables
CCDNAME=${CCDNAME:?"not defined"}
export CCDNAME
CCDLENV=${CCDLENV:?"not defined"}
export CCDLENV
INS_ROOT=${INS_ROOT:?"not defined"}
export INS_ROOT
RTAPENV=${RTAPENV:?"not defined"}
export RTAPENV

# Check environment
if [ ! -d $VLTDATA/ENVIRONMENTS/$RTAPENV ]
then
    echo "RTAPENV defined as $RTAPENV, but not existing"
    exit 1
fi

if [ "`uname`" = "Linux" ]
then

    # If .$RTAPENV.lock file does not exist, $RTAPENV is not running
    PIDFILE="$VLTDATA/ENVIRONMENTS/$RTAPENV/.$RTAPENV.lock"
    if [ ! -f $PIDFILE ]
    then
        echo "It seems that the $RTAPENV environment is not running"
        exit 1

    # If .$RTAPENV.lock exists, it contains the uid of the process running $RTAPENV
    else
        PID="`cat \"$PIDFILE\"`"
        if [ "$PID" = "" ]
        then
            echo "pid file empty"
            exit 1
        fi

        # Check that one of the $RTAPENVs is run by the process whose uid is in .$RTAPENV.lock
        answer="`ps -eaf | grep -v grep | grep Scheduler | awk '{ print $2 }' | fgrep $PID`"
    fi
else
    answer=`ps -eaf | grep $RTAPENV | grep Scheduler | grep -v grep`
fi

if [ "$answer" = "" ]
then
    echo "It seems that the $RTAPENV environment is not running"
    exit 1
fi


noErrDisplay=${NO_ERR_DISPLAY:="0"}

#
# 1 - Load database configuration values on WS
#
fcdDcsDbInit.sh $CCDNAME $RTAPENV $INS_ROOT/$INS_USER/COMMON/CONFIGFILES
if [ $? != 0 ] 
then
	exit $?
fi

#
# 2 - Pop-up the main panel for FIERA stand-alone
#     If panels run in simulation, make sure that scan system is disabled
#     and intialise operational states
#
if [ "$sim" = "1" ]
then
	echo "The whole FIERA sw is simulated within panels"
	fcdDcsScan.sh 0 $RTAPENV $CCDLENV d
# Operational mode forced to WS simulated
	dbWrite "<alias>$CCDNAME.opMode" 1
# Operational state forced to OFF
	dbWrite "<alias>$CCDNAME.opStateSlcu" 6
	dbWrite "<alias>$CCDNAME.opStateWs"  1
# Exposure status forced to IDLE
	dbWrite "<alias>$CCDNAME:exposures:exposure_1.expStatus"  1
fi
fcdCtrl "cwp=<alias>$CCDNAME" "simul=$sim" &
echo " "
echo "FIERA stand-alone control panel for camera $CCDNAME started"
echo " "

#__oOo__
