#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#
# This scripts takes the logFile and creates a new file according to given
# filtering criteria :
#
# - Formats the FITS logs according to the rules used by logMonitor.
# - Selects only error logs
# - Select logs filtered by process name, env name 
#
# Examples :
#
#  Select all the CCS errors in the logFile
#       logFilterFile.awk errFlag=1 <logFile>
#
#
#
#


gawk '

BEGIN {
   env = process = module = day  =time1 = time2 = procID = ""

   errorsOnly = 0
   fitsOnly   = 0
   noFits     = 0

   # Execute shell command and assign value to a variable 
   "uname -s" | getline osType
   "uname -r" | getline osRelease
   "uname -n" | getline nodeName

   for (i=1; i<ARGC ; i++ )  {
      if ( ARGV[i] ~ /day=/ )         { day       = substr(ARGV[i],5) }
      if ( ARGV[i] ~ /start=/ )       { time1     = substr(ARGV[i],7) }
      if ( ARGV[i] ~ /stop=/ )        { time2     = substr(ARGV[i],6) }
      if ( ARGV[i] ~ /proc=/ )        { process   = substr(ARGV[i],6) }
      if ( ARGV[i] ~ /mod=/ )         { module    = substr(ARGV[i],5) }
      if ( ARGV[i] ~ /env=/ )         { env       = substr(ARGV[i],5) }
      if ( ARGV[i] ~ /pid=/ )         { procID    = substr(ARGV[i],5) }
      if ( ARGV[i] ~ /errorOnly=/ )   { errorsOnly = 1 }
      if ( ARGV[i] ~ /fitsOnly=/ )    { fitsOnly   = 1 }
      if ( ARGV[i] ~ /noFits=/ )      { noFits     = 1 }
   }

#   print "errorsOnly = " errorsOnly "fitsOnly = " fitsOnly
#    print "noFits = " noFits "fitsOnly = " fitsOnly
#   print "module = " module " env = " env

}

# Start of the main awk body

{
   # Delete all fields automatically appended by syslog   
   $1 = $2 = $3 = $4 = $5 = ""

   if (osType == "SunOS" && osRelease >= 5.8) {
          # Delete all fields added by Sun OS syslog
          $6 = $7 = $8 = ""

          fitsEnvName = $9

          fitsTime    = $10
          oldFitsTime = $9

          idxDate     = 9

          errNumber   = $17 
          errStack    = $18
          errSeverity = $19
          pname       = $13
          myPID       = $14
          modName     = $12
          envName     = $11
   } else {

          fitsEnvName = $6

          fitsTime    = $7
          oldFitsTime = $6

          idxDate      = 6

          errNumber   = $14 
          errStack    = $15
          errSeverity = $16

          pname       = $10
          myPID       = $11
          modName     = $9
          envName     = $8
   }

   # Check if log meets filtering criteria

   if ( process == "" || process == pname) {
      procFlag = 1
   } else {
      procFlag = 0
   }

   if ( env == "" || env == envName) {
      envFlag = 1
   } else {
      envFlag = 0
   }

   if ( procID == "" || procID == myPID) {
      pidFlag = 1
   } else {
      pidFlag = 0
   }

   if ( module == "" || module == modName) {
      modFlag = 1
   } else {
      modFlag = 0
   }


   printFlag = ( procFlag && envFlag && modFlag && pidFlag)

#   print "printFlag = " printFlag

   # Check if the log is in FITS format

   if (oldFitsTime ~ /([0-9]+):([0-9]+):([0-9]+)>/ ) {

      # This part is left for backwards compatibility
      # On older releases the FITS logs do not have
      # the name of the environment.

      if (length(oldFitsTime) > 9) {
         tmp = " FITS_LOG " substr(oldFitsTime,0,8) " " nodeName " - - 0 0 " substr(oldFitsTime,10)
      } else {
         tmp = " FITS_LOG " substr(oldFitsTime,0,8) " " nodeName " - - 0 0 "
      }

      for (i=idxDate+1 ; i<=NF ; i++)
          {
          tmp = tmp " " $i
          }

      if ( ! errorsOnly && printFlag && ! noFits) {
             print tmp
      } 


   } else if (fitsTime ~ /([0-9]+):([0-9]+):([0-9]+)>/ ) {

      if (length(fitsTime) > 9) {
         tmp = " FITS_LOG " substr(fitsTime,0,8) " " fitsEnvName " - - 0 0 " substr(fitsTime,10)
      } else {
         tmp = " FITS_LOG " substr(fitsTime,0,8) " " fitsEnvName " - - 0 0 "
      }

      for (i=(idxDate+2) ; i<=NF ; i++)
          {
          tmp = tmp " " $i
          }

      if ( ! errorsOnly && ! noFits) {

          # Check again filter on env. name because its position is different
          # with respect to the normal logs

#          print "procFlag =" procFlag "  modFlag =" modFlag " pidFlag =" pidFlag " envName = " $6

          printFlag = ( procFlag && modFlag && pidFlag && (env == "" || $6 == env))

#          print "printFlag =" printFlag

          if ( printFlag )
             {
             print tmp
             }
      } 

   } else {

      if ( errorsOnly ) {

         # Check if the log is a VLT error

         if (errNumber ~ /([0-9]+)/ && errStack ~ /([0-9]+)/ && errSeverity ~ /([0-9]+)/ ) {

            if (printFlag && ! fitsOnly) { 
               print $0
            }
         }
      } else {

         if ( printFlag && ! fitsOnly) { 
            print $0
         }
      }
   }

}
' $*

