#! /bin/csh -f
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tcssimInstall.sh,v 1.94 2005/08/08 10:24:17 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rkarban 2000-10-16 remove files before copying
# rkarban 2000-04-11 copy tcssimTrkDut.dat from INTROOT to VLTDATA
# rkarban 2000-02-21 retrieve DIC,ALIAS files from config directory
# rkarban  10/03/99  SPR990019, insert blank between if an condition
# gchiozzi  16/09/97  Improved search of files in INTROOT and VLTROOT
# gchiozzi  16/09/97  Removed installation of setupHdDisplay
# gchiozzi  18/08/97  Fixed error while copying TCS.alias
# gchiozzi  15/01/97  Modified to use new structure for setup file dictionary and alias
# nfiebig   09/01/96  corrected typing errors
# gchiozzi  11/12/95  created from alongino's ccsInstall.sh
# alongino  25/09/95  created

#************************************************************************
#   NAME
#   tcssimInstall.sh - Install TCS simulation files in INS_ROOT
# 
#   SYNOPSIS
#   tcssimInstall.sh [<ins_root_dir>]
# 
#   DESCRIPTION
#   This shell script installs all files needed to run a TCS simulation
#   system.
# 
#   FILES
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   RTAPENV  CCD Rtap environment name
#   INS_ROOT default instruments root directory
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   > tcssimInstall.sh $TCS_ROOT
#
#   SEE ALSO
#   INS common sw specification
#
#------------------------------------------------------------------------
echo "tcssimInstall: creating/checking INS_ROOT for TCS"

if ($#argv >= 1) then
  setenv INS_ROOT $argv[1]
endif
echo "INS_ROOT is: " $INS_ROOT

set curdir = $cwd

#
# Create INS_ROOT if not existing
#
set dir = $INS_ROOT
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

#
# Create INS_ROOT sub-directories needed by the TCS software
#
cd $INS_ROOT
set dir = SYSTEM
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/Dictionary
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/Alias
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/DETDATA
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif
set dir = SYSTEM/COMMON
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/COMMON/CONFIGFILES
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/COMMON/SETUPFILES
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/COMMON/SETUPFILES/DET
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/COMMON/SETUPFILES/REF
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

set dir = SYSTEM/COMMON/SETUPFILES/TARG
if (!(-d $dir)) then
  mkdir $dir
  if ( $status ) then
	echo $dir - cannot create!
  else
	echo $dir - created!
  endif
else
	echo $dir - OK!
endif

#
# Select where to copy files from
#
if ($?INTROOT) then
  set INTDIR = $INTROOT
else
  set INTDIR = $VLTROOT
endif
echo "Configuration files are copied from: $INTDIR"
echo "                            or from: $VLTROOT"

cd $curdir 
cd $INS_ROOT/SYSTEM/COMMON/SETUPFILES/TARG
#
# Copy default setup file
# 
if ( -f $INTDIR/include/prsDefaultSetup.targ ) then
    set FILEDIR = $INTDIR
else
    set FILEDIR = $VLTROOT
endif

rm -f prsDefaultSetup.targ
cp $FILEDIR/include/prsDefaultSetup.targ .
if ( $status ) then
  echo prsDefaultSetup.targ - cannot install!
else
  echo prsDefaultSetup.targ - installed from $FILEDIR/include/prsDefaultSetup.targ
endif

cd $curdir 
cd $INS_ROOT/SYSTEM/Dictionary
#
# Copy TCS dictionary
# 
if ( -f $INTDIR/config/ESO-VLT-DIC.TCS ) then
    set FILEDIR = $INTDIR
else
    set FILEDIR = $VLTROOT
endif

rm -f ESO-VLT-DIC.TCS
cp $FILEDIR/config/ESO-VLT-DIC.TCS ESO-VLT-DIC.TCS
if ( $status ) then
  echo ESO-VLT-DIC.TCS - cannot install!
else
  echo ESO-VLT-DIC.TCS - installed from $FILEDIR/config/ESO-VLT-DIC.TCS
endif

cd $curdir 
cd $INS_ROOT/SYSTEM/Alias

#
# Copy TCS alias tables
# 
if ( -f $INTDIR/config/TCS.alias ) then
    set FILEDIR = $INTDIR
else
    set FILEDIR = $VLTROOT
endif

rm -f TCS.alias
cp $FILEDIR/config/TCS.alias TCS.alias
if ( $status ) then
  echo TCS.alias - cannot install!
else
  echo TCS.alias - installed from $FILEDIR/config/TCS.alias
endif

cd $VLTDATA/config

#
# Copy tcssim trk config files
# 
set TCSSIMTRKDUT=${USER}_tcssimTrkDut.dat
if ( -f $INTDIR/config/tcssimTrkDut.dat ) then
    set FILEDIR = $INTDIR
else
    set FILEDIR = $VLTDATA
endif

rm -f ${TCSSIMTRKDUT}
cp $FILEDIR/config/tcssimTrkDut.dat ${TCSSIMTRKDUT}
if ( $status ) then
  echo ${TCSSIMTRKDUT} - cannot install!
else
  echo ${TCSSIMTRKDUT} - installed from $FILEDIR/config/tcssimTrkDut.dat
endif

#
# Should we need also database init and configuration files
# they should be added here
#


#
# Go back to starting directory
#
cd $curdir 

