!******************************************************************************
! E.S.O. - VLT project
!
! "@(#) $Id: ccdMidasObj.prg,v 4.11 2005/06/22 10:58:21 vltsccm Exp $"
!
! who       when      what
! --------  --------  ----------------------------------------------
! alongino  07/11/96  created
!

!************************************************************************
!   NAME
!   ccdMidasObj.prg - Midas procedure used to find stars in an image
! 
!   SYNOPSIS
!   ccdMidasObj.prg [<fits_file>] [<threshold>]
! 
!   DESCRIPTION
!   This Midas procedure determines the position of stars in an image.
!   The <threshold> value should be passed as expected by the Midas command
!   SEARCH/ROMAFIT, namely:
!   C,val: the threshold will be constant with value val;
!   R,val: the threshold will be set at val times the sky standard deviation.
!   Default: R,3
! 
!   FILES
!   midasObj0001.bdf    image in bdf format for Midas
!   midasObj0001SKY.tbl table containing intermediate sky parameters
!   midasObj0001CAT.tbl table containing final object parameters
!
!   ENVIRONMENT
!
!   RETURN VALUES
!   objN   number of objects found
!   objX   array with X-coordinate of objects
!   objY   array with Y-coordinate of objects
!   objI   array with intensity of objects
!
!   CAUTIONS
!
!   EXAMPLES
!   > @@ ccdMidasObj myImage.fits R,3
!         Analyse image FITS file myImage.fits in the local directory 
!         with threshold 3 times sky standard deviation.
!
!   SEE ALSO
!   Midas ROMAFIT package (Midas Manual Volume B)
!
!   BUGS
!------------------------------------------------------------------------
!
! Find objects in the image
!
! set context to romafot
set/context romafot >Null
! convert FITS file in Midas bdf file
inta/fits 1 midasObj {p1}
! do computation on sky level
sky/roma midasObj0001
! search for objects in the image using sky information
search/roma midasObj0001 ? ? ? ? {p2} 4000
! re-format useful columns only
name/col midasObj0001CAT.tbl :x ? ? ? f8.2
name/col midasObj0001CAT.tbl :y ? ? ? f8.2
name/col midasObj0001CAT.tbl :int ? ? ? f8.2
read/tab midasObj0001CAT.tbl :x,:y,:int
! now useful columns should be copied into keywords for pco
show/tab midasObj0001CAT.tbl >Null
write/key objN/I/1/1 {outputi(2)}
if objN .gt. 0 then
! trick needed because for global keywords one cannot increase the size
! first one does a dummy assignment to be sure that the keyword exists,
! then deletes it and re-creates with the right size
  write/key objX/R/1/1 0 
  write/key objY/R/1/1 0
  write/key objI/R/1/1 0
  dele/key objX
  dele/key objY
  dele/key objI
  write/key objX/R/1/{objN} 0 all
  write/key objY/R/1/{objN} 0 all
  write/key objI/R/1/{objN} 0 all
  define/local N/I/1/1 0
  do N = 1 objN
    objX({N}) = m$value(midasObj0001CAT.tbl,:x,@{N})
    objY({N}) = m$value(midasObj0001CAT.tbl,:y,@{N})
    objI({N}) = m$value(midasObj0001CAT.tbl,:int,@{N})
  enddo
endif
!
! The End
!