#ifndef agwsCMD_HANDLER_H
#define agwsCMD_HANDLER_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCMD_HANDLER.h,v 1.198 2005/08/26 15:24:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* rkarban  01/07/98  add attribute replyInternally
* rkarban  01/07/98  add to SendInternal loopSafe parameter
* rkarban  01/07/98  remove method Revert2ExternalCall
* rkarban  24/06/98  add method+attribute OriginIsExtern
* rkarban  24/06/98  add member orgExtMsg+access methods+method Revert2ExternalCall
* gchiozzi  14/11/97  Added CmdMsg() method.
* gchiozzi  13/10/97  Renamed PUSH_MODE as CMD_MODE
* gchiozzi  03/10/97  Implemented agwsCMD_DICT and static SendInternal()
* gchiozzi  01/10/97  made RegResetHandler a stati method.
* gchiozzi  05/08/97  Restructured
* nfiebig   28/06/96  created
*/

/******************************************************************************
 * Declaration of the agwsCMD_HANDLER class.
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * module includes
 */
#include "agws.h"
#include "agwsSTATE.h"

/*
 * evh toolkit includes
 */
#include "evhTASK.h"
#include "evhSTD_COMMANDS.h"

enum agwsCMD_MODE
{
    agwsCMD_NONE,            // command not active
    agwsCMD_ACCEPTED,        // command accepted, possibly in parallel with others
    agwsCMD_REJECTED,        // command rejected
    agwsCMD_PREV_ABORTED,    // command accepted aborting a previous one
    agwsCMD_PREV_OVERRIDDEN  // command accepted overriding a previous one
};

class agwsCMD_HANDLER;

class agwsCMD_DICT: public fndDICTIONARY, virtual public eccsERROR_CLASS
{
  public:

    agwsCMD_DICT(const unsigned sz = fndDEFAULT_DICT_SIZE ):
	fndDICTIONARY(sz) { DelItemsOnDelete(); }

    virtual ccsCOMPL_STAT    AddCmdHandler(const char *cmd,
					 agwsCMD_HANDLER &handler);
    virtual ccsCOMPL_STAT    DelCmdHandler(const char *cmd);
    virtual agwsCMD_HANDLER &Lookup(const char *cmd);

    fndStdObjectDef(agwsCMD_DICT, fndDICTIONARY);

  protected:

}; /* end class evhCALLBACK_DICT */


typedef evhCB_COMPL_STAT 
              (fndOBJECT::*agwsCMD_METHOD)(msgMESSAGE &, agwsCMD_HANDLER &);

class agwsCMD_HANDLER : public evhSIMPLE_TASK
{
  public:
    agwsCMD_HANDLER(const msgCMD command, 
		    const evhCALLBACK &cb);
    agwsCMD_HANDLER(const msgCMD         command,
                    fndOBJECT           *obj,
		    const agwsCMD_METHOD procedure);

    virtual ~agwsCMD_HANDLER();

    vltLOGICAL       CmdActive()    const;
    char *           CmdBuffer()    const;
    agwsCMD_MODE     CmdMode()      const;
    msgMESSAGE      &CmdMsg();             // originator command message
    msgMESSAGE      &CmdExtMsg();
    vltLOGICAL       OriginIsExtern(void);

    virtual evhCB_COMPL_STAT CommandCB(msgMESSAGE &msg, void *);

    vltLOGICAL               InternalCall() const;

    virtual ccsCOMPL_STAT    SendReply(const char *buffer, 
				       msgLENGTH size = 0,
				       vltLOGICAL last = ccsTRUE);
    virtual ccsCOMPL_STAT    SendReply(const ccsERROR &errMsg);

    
    static ccsCOMPL_STAT    RegResetHandler(const evhCALLBACK &cb);

    static ccsCOMPL_STAT    SendInternal(msgMESSAGE &msg,
					 const evhCALLBACK &reply, 
					 const evhCALLBACK &errReply,
					 vltLOGICAL loopSafe = ccsFALSE);
    ccsCOMPL_STAT           Reset();

  protected:
    virtual ccsCOMPL_STAT    AcceptCommand(msgMESSAGE &msg);
    virtual ccsCOMPL_STAT    CheckState(const char *cmd);

    virtual ccsCOMPL_STAT    IntCommand(msgMESSAGE &msg,
					const evhCALLBACK &reply, 
					const evhCALLBACK &errReply,
					vltLOGICAL loopSafe);

    ccsCOMPL_STAT RegCommandCB(const evhCALLBACK &cb);
    ccsCOMPL_STAT RegReplyHandlers(const evhCALLBACK &reply, 
				   const evhCALLBACK &errReply);

  private:
    msgCMD        command;         // name of command handled by this instance
    msgMESSAGE    orgMsg;          // originator command message
    msgMESSAGE    orgExtMsg;       // originator external command message
    evhCALLBACK  *applCmdCB;       // application command call-back
    evhCALLBACK  *applReplyCB;     // reply handler for internal calls
    evhCALLBACK  *applErrorCB;     // error reply handler for internal calls
    vltLOGICAL    internalCall;    // flag if called from internally
    vltLOGICAL    replyInternally; // flag if we have to reply internally
    vltLOGICAL    cmdActive;       // flag if a command is active
    agwsSTATE     state;           // state handling object
    agwsCMD_MODE  cmdMode;         // mode of successive command handling
    vltLOGICAL    originIsExtern;

    // the following static members are defined in agwsCMD_HANDLER.C
    static vltLOGICAL       parActive;   // flag if a parallel command is active
    static agwsCMD_HANDLER *parCmd;      // currently active parallel command

    static vltLOGICAL       exclActive;  // flag if an exclusive command is active
    static agwsCMD_HANDLER *exclCmd;     // currently active exclusive command

    static evhCALLBACK     *resetHandler; // general reset handler

    static agwsCMD_DICT     intCommands;    // Dictionary of registered commands
};

#endif /*!agwsCMD_HANDLER_H*/
