#ifndef agwsCMD_PARALLEL_H
#define agwsCMD_PARALLEL_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCMD_PARALLEL.h,v 1.198 2005/08/26 15:24:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* nfiebig   24/06/96  created
*/

/******************************************************************************
 * Declaration of the agwsCMD_PARALLEL class. This class serves to send a list
 * of commands in parallel (various commands to various targets) and collect
 * all the replies.
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * eccs and evh includes
 */
#include <eccsErr.h>
#include <evhOBJ_CALLBACK.h>
#include <evhCOMMAND.h>
#include <evhTASK.h>

/*
 * module includes
 */
#include "agws.h"

class agwsCMD_PARALLEL : public evhSIMPLE_TASK
{
  public:
    agwsCMD_PARALLEL();
    ~agwsCMD_PARALLEL();

    ccsCOMPL_STAT    Reply(evhCALLBACK &cb);
    evhCALLBACK     *Reply();

    ccsCOMPL_STAT    ErrReply(evhCALLBACK &cb);
    evhCALLBACK     *ErrReply();

    ccsCOMPL_STAT    Timeout(evhCALLBACK &cb, ccsTIMEVAL &timeout);
    ccsCOMPL_STAT    Timeout(ccsTIMEVAL &timeout);
    evhCALLBACK     *Timeout();

    ccsCOMPL_STAT    Send(msgMESSAGE &orgMsg, vltINT32 numMsgs, msgMESSAGE msg[]);

    evhCB_COMPL_STAT SendReply(const char *buffer);
    evhCB_COMPL_STAT SendErrorReply(const ccsERROR &errMsg);
	
    virtual ccsCOMPL_STAT Reset();
    virtual vltLOGICAL    Status();

  protected:
    virtual evhCB_COMPL_STAT ReplyCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT ErrReplyCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT TimeoutCB(msgMESSAGE &msg, void *);

    vltINT32     numCmds;        // number of commands handled in parallel
    evhCOMMAND  *cmdList;        // list of commands handled in parallel

  private:
    vltINT32     numWait;        // number of replies still pending
    ccsTIMEVAL   timeout;        // overall timeout on waiting for replies
    msgMESSAGE   userMsg;        // user message to be used for replies
    evhCALLBACK *userReplyCB;    // user defined reply handler
    evhCALLBACK *userErrReplyCB; // user defined error reply handler
    evhCALLBACK *userTimeoutCB;  // user defined timeout handler
};

#endif // !agwsCMD_PARALLEL_H
