#ifndef agwsCON_ADAPTER_H
#define agwsCON_ADAPTER_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCON_ADAPTER.h,v 1.198 2005/08/26 15:24:43 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* gchiozzi 2001-03-10 Some fixex to documentation
* tcsmgr 2001-03-08 Added ProbeAbsRstCB.
* gchiozzi 2000-08-30 Added to LoadSearchCrit() min and max search radius
* gchiozzi 2000-07-27 Added agwsCON_SERVER to constructors and as member.
* mkiekebu  01/07/99  command handling for command OFFSGP added
* rkarban  31/10/98  add methods for handling adrot events
* rkarban  31/10/98  add insdataMonitor callback and object
* rkarban  25/07/98  add timer for polling probe position
* gchiozzi  20/07/98  Added OFFSADG command handling
* rkarban  05/05/98  add attribute adOffset to store msg of OFFSAD
* gchiozzi  28/11/97  Replaced pointer to gsCatalogue with reference.
* gchiozzi  28/10/97  Removed NTT/VLT ifdefs.
* gchiozzi  28/10/97  Added CheckProbePos() method.
* gchiozzi  19/10/97  Removed private member adapterCmd. Not used.
* gchiozzi  19/10/97  Removed private members agwsConCheck and agwsConServer.
* gchiozzi  19/10/97  Removed ProbeAbs() method. Merged with ProbeAbsCB().
* gchiozzi  19/10/97  Added new ProbeCmdSend protected method.
* gchiozzi  09/10/97  Changed all callbacks to get the agwsCMD_HANDLER in the udata
* gchiozzi  02/10/97  Now subclass of evhDB_TASK and not any more of evhSIMPLE_TASK
* gchiozzi  01/08/97  Changed prototype for ProbeGsCB
* tphan     17/10/96  add NTT probe (thanks to rabuter)
* nfiebig   26/07/96  added STOP command handler and call-back
* nfiebig   05/06/96  implemented
* gchiozzi  29/02/96  created
*/

/******************************************************************************
 * Declaration of the agwsCON_ADAPTER class. This is the generic class for
 * interfacing to the adapter/guide-probe sub-system. There exist two 
 * polymorphic sub-classes ..._VLT and ..._NTT to overload the specific 
 * features of the VLT or NTT, resp.
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "math.h"
extern "C" {
#include "tcsVcc.h"
#include "slalib.h"
#include "slamac.h"
}
#include "agws.h"
#include "agwsCON_CAT.h"
#include "agwsCON_CHECK.h"
#include "agwsCON_SERVER.h"
#include "agwsCON_SETUP.h"
#include "agwsCMD_HANDLER.h"
#include "agwsTRANSFER.h"

#include "evhDB_TASK.h"
#include "evhCOMMAND.h"
#include "evhSTD_COMMANDS.h"
#include "mswMOD_LIST.h"
#include "mswINS_DATA_MONITOR.h"

class agwsCON_ADAPTER : public evhDB_TASK
{
  public:
    agwsCON_ADAPTER(const dbSYMADDRESS  dbPoint, agwsCON_CAT &gsCat, agwsCON_SERVER &agServer);
    agwsCON_ADAPTER(const dbSYMADDRESS  dbPoint, agwsCON_CAT &gsCat, agwsCON_SERVER &agServer,
		    agwsCON_SETUP *gsSetup);
    virtual ~agwsCON_ADAPTER();
    virtual ccsCOMPL_STAT Init();
    virtual ccsCOMPL_STAT Reset();

    virtual evhCB_COMPL_STAT ProbeAbsCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeAbsRstCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeCheckCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeCenterCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeDObjCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeGsCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeParkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffsetAdCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffsetGpCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);   
    virtual evhCB_COMPL_STAT OffsetXyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffsetAdgCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StartManCorrCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopManCorrCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ProbeStopCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT GeneralReplyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT GeneralErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT OffsetAdGpOkCB(msgMESSAGE &msg, void *udata);

    virtual evhCB_COMPL_STAT OffsetGpReplyCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT OffsetGpErrorCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT OffsetGpStepCB(evtEVENT_MSG &msg,  void *udata);


    virtual evhCB_COMPL_STAT OffsetAdgStopOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffsetAdgReplyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT GpInPosOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT GpInPosErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual vltLOGICAL       CheckProbePos(vltDOUBLE  alpha,
					   vltDOUBLE  delta,
					   vltLOGICAL usedMargin,
					   vltDOUBLE  rotAngle   );

    virtual ccsCOMPL_STAT    StopOffsAdg();

    virtual ccsCOMPL_STAT    GetProbeMagicScaleDB(vltDOUBLE *scaleX,
						  vltDOUBLE *scaleY);
    virtual ccsCOMPL_STAT EnableAdRotEvents();
    virtual ccsCOMPL_STAT DisableAdRotEvents();

    virtual ccsCOMPL_STAT UpdateVignettingLimits();

 protected:
    virtual ccsCOMPL_STAT ProbeCmdSend (const msgCMD     cmd,
					const char      *buffer,
					ccsTIMEVAL       timeout,
					agwsCMD_HANDLER &handler,
					vltINT32 startState = agwsSTATE_OPERATING,
					vltINT32 endState = agwsSTATE_IDLE);
    virtual ccsCOMPL_STAT LoadSearchCrit(vltDOUBLE *minMag, vltDOUBLE *maxMag,
				 vltDOUBLE *minRad, vltDOUBLE *maxRad,
                                 char *vignettingLimit);

    virtual evhCB_COMPL_STAT InsDataChangedCB(msgMESSAGE &msg, void *);
    virtual ccsCOMPL_STAT EvtRegister(evtEVENT *evt, const char *dbAttr, 
			     evhCB_METHOD DbEventCB, evtFILTER filter = evtANY_WRITE);
    virtual ccsCOMPL_STAT EvtDetach(evtEVENT *evt, evhCB_METHOD DbEventCB);

    agwsCMD_HANDLER  prAbsCmdHandler;   // PRABS command handler
    agwsCMD_HANDLER  prChkCmdHandler;   // PRCHK command handler
    agwsCMD_HANDLER  prCntCmdHandler;   // PRCNT command handler
    agwsCMD_HANDLER  prDObjCmdHandler;  // PRDOBJ command handler
    agwsCMD_HANDLER  prGsCmdHandler;    // PRGS command handler
    agwsCMD_HANDLER  prParkCmdHandler;  // PRPARK command handler
    agwsCMD_HANDLER  offsAdCmdHandler;  // OFFSAD command handler
    agwsCMD_HANDLER  offsGpCmdHandler;  // OFFSGP command handler
    agwsCMD_HANDLER  offsAdgCmdHandler; // OFFSADG command handler
    agwsCMD_HANDLER  offsXyCmdHandler;  // OFFSXY command handler
    agwsCMD_HANDLER  startMcCmdHandler; // STARTMC command handler
    agwsCMD_HANDLER  stopMcCmdHandler;  // STOPMC command handler
    agwsCMD_HANDLER  prStopCmdHandler;  // PRSTOP command handler

    agwsTRANSFER     probeCmd;          // transfer command to move probe
    agwsTRANSFER     checkCmd;          // transfer command to check probe
    agwsTRANSFER     trkoffCmd;         // transfer command to trkws for offset
    agwsTRANSFER     gpInPosCmd;         // transfer command to ad for probe in position

    agwsCON_CAT     &gsCatalogue;       // internal guide-star catalogue
    agwsCON_SERVER  &agServer;          // reference to AG server
    agwsCON_SETUP   *gsSetupObj;        // internal handling setup values

    ccsTIMEVAL       prAbsTimeout;      // PRABS transfer command timeout
    ccsTIMEVAL       prChkTimeout;      // PRCHK transfer command timeout
    ccsTIMEVAL       prCntTimeout;      // PRCNT transfer command timeout
    ccsTIMEVAL       prParkTimeout;     // PRDOBJ transfer command timeout
    ccsTIMEVAL       offsAdTimeout;     // OFFSAD transfer command timeout
    ccsTIMEVAL       offsAdgTimeout;    // OFFSADG transfer command timeout
    ccsTIMEVAL       offsXyTimeout;     // OFFSXY transfer command timeout
    ccsTIMEVAL       startMcTimeout;    // STARTMC transfer command timeout
    ccsTIMEVAL       stopMcTimeout;     // STOPMC transfer command timeout
    ccsTIMEVAL       gpInPosTimeout;    // GPINPOS transfer command timeout

    vltLOGICAL       alreadyGuiding;
    msgMESSAGE       adOffset;
    evhCOMMAND       offsadCmd;
    evhCOMMAND       offsadGpCmd;
    mswINS_DATA_MONITOR insConfigData;       
    evtEVENT         evtOffsAlpha;

private:    
};

#endif // !agwsCON_ADAPTER_H

// __oOo__
