#ifndef agwsCON_CAT_H
#define agwsCON_CAT_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCON_CAT.h,v 1.198 2005/08/26 15:24:43 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* gchiozzi 2001-03-01 Added probePos parm to AddCandGs
* gchiozzi 2000-08-30 Removed minRadius, maxRadius from members. Uses DB and setup files now
* mkiekebu  11/08/99  method ResetInUse added
* tphan     17/01/98  added TcsCatalog
* gchiozzi  27/11/97  Modified SearchGs() prototype. Added agwsSEARCH_MODE enum. 
* gchiozzi  12/11/97  Added selected data member
* gchiozzi  12/11/97  Added index parameter to AddCandGs().
* gchiozzi  12/11/97  Added inUse member.
* gchiozzi  28/10/97  Added handling of probe check object.
* gchiozzi  17/10/97  Removed GetRotAngleDb() method. Using tif.
* gchiozzi  08/08/97  Added Reset() method
* gchiozzi  22/07/97  Added default values to SerchGs() method
* gchiozzi  22/07/97  
* nfiebig   23/08/96  added interface for external catalog query
* gchiozzi  29/02/96  created
*/

/******************************************************************************
 * Declaration of class agwsCON_CAT
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * module includes
 */
extern "C" {
#include "tcsVcc.h"
}
#include "agws.h"
#include "agwsGUIDE_STAR.h"
#include "eccsDB_TABLE.h"

/*
 * evh toolkit includes
 */
#include <evhDB_TASK.h>
#include <evhCOMMAND.h>

enum agwsSEARCH_MODE { agwsTARGET_POS, agwsACTUAL_POS, agwsAUTO_POS };

class agwsCON_ADAPTER;
/*
 * definition of catalogue handler class
 */
class agwsCON_CAT : public evhDB_TASK
{
  public:
    agwsCON_CAT(const dbSYMADDRESS dbPoint, agwsCON_ADAPTER *checker = NULL);
    virtual ~agwsCON_CAT();

    virtual evhCB_COMPL_STAT AddCandGsCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT GetGsCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT GetGssCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT SelectGsCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT SearchGsCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT PrintCatCB(msgMESSAGE &msg, void *);

    virtual ccsCOMPL_STAT Init();
    virtual ccsCOMPL_STAT Reset();
    virtual ccsCOMPL_STAT AddCandGs(vltDOUBLE        alpha, 
				    vltDOUBLE        delta, 
				    vltDOUBLE        mag,
				    agwsGS_PROBE_POS probePos,
				    vltDOUBLE        wavelen = 650.0,
				    vltINT32  index = 0);

    virtual ccsCOMPL_STAT SearchGs(agwsSEARCH_MODE searchMode = agwsAUTO_POS,
				   vltLOGICAL      noDefCriteria = ccsFALSE,
				   vltDOUBLE       minMag = 0.0, vltDOUBLE maxMag = 0.0,
				   vltBYTES256     vigLimit = (unsigned char*)"", 
				   vltDOUBLE       rotAngle = 0.0);

    virtual ccsCOMPL_STAT GetNewCandidate(agwsGUIDE_STAR &candidate);
    virtual vltINT32      UntriedEntries();

    virtual ccsCOMPL_STAT SetProbeChecker(agwsCON_ADAPTER *checkObj);

    virtual ccsCOMPL_STAT ResetInUse();
    virtual ccsCOMPL_STAT SetCurrentGs(double alpha, double delta,
				       double magnitude = 0.0, double wavelength = 0.0);
    
    const agwsGUIDE_STAR &GetCatalogEntry(const vltINT32 index) const;

  protected:
    virtual int           ComparePriority(const agwsGUIDE_STAR &star1, 
					  const agwsGUIDE_STAR &star2) const;
    virtual ccsCOMPL_STAT PopCandidate(vltINT32 index);
    virtual ccsCOMPL_STAT StoreCurrentGs();
    virtual ccsCOMPL_STAT StoreCurrentGsTable();
    virtual ccsCOMPL_STAT QueryExtCat(vltDOUBLE alpha, vltDOUBLE delta,
                              vltDOUBLE minRad, vltDOUBLE maxRad,
                              vltDOUBLE minMag, vltDOUBLE maxMag,
                              int & numFound, TcsQueryResult & result);
    virtual ccsCOMPL_STAT FilterResult(int & numFound, TcsQueryResult & result);
    virtual ccsCOMPL_STAT UpdateInUse();

  private:
    TcsCatalog*      catalog;		     // ext. guide-star catalogue pointer
    vltBYTES32       catalogName;            // ext. guide-star catalogue name
    agwsGUIDE_STAR   gsCat[agwsGS_CAT_SIZE]; // guide-star candidate list

    vltINT32         inUse;                  // Index of the entry in use
    vltINT32         selected;               // Index of the entry selected for next
                                             // GetNewCandidate()
    agwsCON_ADAPTER *checker;                // object used to check probe pos.
};

#endif /*!agwsCON_CAT_H*/
