#ifndef agwsCON_MAIN_TASK_H
#define agwsCON_MAIN_TASK_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCON_MAIN_TASK.h,v 1.198 2005/08/26 15:24:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban 2001-12-03 add methods for checking state at start/stop guiding
* gchiozzi 2001-03-06 Added OFFSAG command (spr.20000550)
* rkarban 2000-10-31 move inclusion of CON_ADAPTER class to CON_MAIN_TASK.C
* rkarban 2000-10-31 add ATS_TCS target
* gchiozzi  15/11/99  Removed FINDGS command
* mkiekebu  10/09/99  DetGpInPosOkCB and DetGpInPosErrorCB added
* gchiozzi  22/07/98  Added SetupSaveAndOkCB and SetupReloadAndErrorCB methods
* rkarban  18/07/98  replace cntgs with pr2star,star2pr
* tphan     25/05/98  Added cmd CNTGS and CntGsCB (Move probe to center gs)
* rkarban  24/06/98  add callback DetGsStopTccdOkCB
* gchiozzi  05/02/98  Added stop of TCCD to BOX2GS command.
* gchiozzi  08/01/98  Added StopAgStrtTccdCB() method
* gchiozzi  28/11/97  Support objects are created outside and bassed by reference.
* gchiozzi  17/11/97  Added counter for max loop in FINDGS command.
* gchiozzi  11/11/97  Added STRTCCD and STOPCCD commands
* gchiozzi  17/10/97  Removed xOffset and yOffset members. Not necessary any more.
* gchiozzi  17/10/97  Removed step reading ROT ON SKY for center obj. Done via tif.
* gchiozzi  09/10/97  Removed oslxServer
* gchiozzi  09/10/97  Removed SETIT command
* gchiozzi  09/10/97  Changed all callbacks to get the agwsCMD_HANDLER in the udata
* gchiozzi  01/10/97  Removed StartAgExposureOkCB. No used.
* tphan     06/06/97  added  RemoteLoadOkCB to load rtd setup file for oslxServer
* tphan     24/05/97  added agwsTccdSetupFile for ccd 
* tphan     16/05/97  added agwsTccdUserFunction for use of ccd user function
* nttmgr    19/03/97  remove attributes startAgCmdHandler&stopAgCmdHandler
* gchiozzi  29/02/96  created
*/

/************************************************************************
 * Definition of class agwsCON_MAIN_TASK
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "agws.h"
#include "agwsCMD_HANDLER.h"
#include "agwsCON_CAT.h"
#include "agwsCON_CHECK.h"
#include "agwsCON_SETUP.h"
#include "agwsCON_SERVER.h"
#include "agwsCON_ADAPTER.h"

class agwsCON_MAIN_TASK : public evhDB_TASK
{
  public:
    agwsCON_MAIN_TASK(const dbSYMADDRESS  dbPoint,
		      agwsCON_ADAPTER     &conAdapter,
		      agwsCON_CHECK       &conCheck,
		      agwsCON_CAT         &conCat,
		      agwsCON_SERVER      &conServer,
		      agwsCON_SETUP       &conSetup);

    virtual ~agwsCON_MAIN_TASK();
    virtual ccsCOMPL_STAT    Init();
    virtual ccsCOMPL_STAT    Reset(msgMESSAGE &msg, void *);
    virtual ccsCOMPL_STAT    Recover();
    virtual vltLOGICAL CheckStateStopGuiding (msgMESSAGE &msg);
    virtual vltLOGICAL CheckStateStartGuiding (msgMESSAGE &msg, const char *modestr);
    virtual ccsCOMPL_STAT StartAgFitsLog();

    // Overloading of inherited commands
    virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT ExitOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StandbyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StandbyOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT OffOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopReplyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    // Callbacks for module specific commands
    virtual evhCB_COMPL_STAT StartAgCB(msgMESSAGE &msg, agwsCMD_HANDLER &);
    virtual evhCB_COMPL_STAT StartAgStopTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &);
    virtual evhCB_COMPL_STAT StartAgSetupOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StartAgOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopAgCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopAgStrtTccdCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopAg(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopAgReplyCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT PrToGsCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    //
    // DETGS command handler methods
    //
    virtual evhCB_COMPL_STAT DetGsCB(msgMESSAGE &msg, 
				     agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT DetGsSetupOkCB(msgMESSAGE &msg, 
					    agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT DetGsTccdOkCB(msgMESSAGE &msg, 
					   agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT DetGsRestoreSetupCB(msgMESSAGE &msg, 
						 agwsCMD_HANDLER &cmdHandler);
    virtual evhCB_COMPL_STAT DetgsErrorCB(msgMESSAGE &msg, 
					  agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT Pr2StarCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT Star2PrCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT Tel2ObjCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT XY2ADOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT XY2ADAgOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT XY2ADAgDetGsOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT CntObjCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT CntObjProbeOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT CntObjSetupOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT CntObjTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT CntObjOffsetOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT RelaxCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT RelaxStopTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &);
    virtual evhCB_COMPL_STAT RelaxSetupOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT RelaxTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT DetGsStopTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT DetGsStartTccdOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);   
    virtual  evhCB_COMPL_STAT DetGpInPosOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
   
    virtual evhCB_COMPL_STAT OffsAgCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT StrtTccdCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StrtTccdSetupOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);
    virtual evhCB_COMPL_STAT StopTccdCB(msgMESSAGE &msg, agwsCMD_HANDLER &handler);

    virtual evhCB_COMPL_STAT GeneralOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);
    virtual evhCB_COMPL_STAT GeneralErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);

    virtual evhCB_COMPL_STAT SetupSaveAndOkCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);
    virtual evhCB_COMPL_STAT SetupReloadAndErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);

    virtual evhCB_COMPL_STAT GpInPosErrorCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);
    virtual evhCB_COMPL_STAT GpInPosTimeoutCB(msgMESSAGE &msg, agwsCMD_HANDLER &cmdHandler);

    // sub-system interface objects
    agwsCON_ADAPTER &agwsConAdapter;
    agwsCON_CHECK   &agwsConCheck;
    agwsCON_CAT     &agwsConCat; 
    agwsCON_SERVER  &agwsConServer;
    agwsCON_SETUP   &agwsConSetup;

    vltDOUBLE        relaxMaxError;
    vltINT32         relaxLoopCount, relaxLoopMax;
  private:

    ccsCOMPL_STAT SendSetup(evhCB_METHOD mtd, 
			    agwsCMD_HANDLER &handler);


    // high level command handlers

    agwsCMD_HANDLER  startAgCmdHandler; // STARTAG command handler
    agwsCMD_HANDLER  stopAgCmdHandler;  // STOPAG command handler
    agwsCMD_HANDLER  prToGsCmdHandler;  // PRTOGS command handler
    agwsCMD_HANDLER  detGsCmdHandler;   // DETGS command handler
    agwsCMD_HANDLER  pr2StarCmdHandler; // PR2STAR command handler
    agwsCMD_HANDLER  star2PrCmdHandler; // STAR2PR command handler
    agwsCMD_HANDLER  tel2objCmdHandler; // TEL2OBJ command handler
    agwsCMD_HANDLER  cntObjCmdHandler;  // CNTOBJ command handler
    agwsCMD_HANDLER  relaxCmdHandler;   // BOX2GS command handler
    agwsCMD_HANDLER  offsagCmdHandler;  // OFFSAG command handler

    agwsCMD_HANDLER  initCmdHandler;    // INIT command handler
    agwsCMD_HANDLER  exitCmdHandler;    // EXIT command handler
    agwsCMD_HANDLER  stopCmdHandler;    // STOP command handler
    agwsCMD_HANDLER  onlineCmdHandler;  // ONLINE command handler
    agwsCMD_HANDLER  standbyCmdHandler; // STANDBY command handler
    agwsCMD_HANDLER  offCmdHandler;     // OFF command handler

    agwsCMD_HANDLER  strtccdCmdHandler; // STRTCCD command handler
    agwsCMD_HANDLER  stopccdCmdHandler; // STOPCCD command handler

    vltINT32         stopReplyCount;    // counter for pending STOP replies


    vltLOGICAL       restartDetgsTccd;


    vltINT32        detgsWindowSize ;
    vltINT32	    detgsOrigWindowSize;
    vltDOUBLE       detgsIntTime;
    vltDOUBLE       detgsOrigIntTime;
    char            checkGp[4];

    agwsTRANSFER     gpInPosCmd;
    agwsTRANSFER     setlimCmd;
};


#endif /*!agwsCON_MAIN_TASK_H*/
