#ifndef agwsCON_SERVER_H
#define agwsCON_SERVER_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsCON_SERVER.h,v 1.198 2005/08/26 15:24:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* gchiozzi 2000-07-27 Moved handling of pix size here from fswsServer.
* rkarban   18/07/98  add method ConvertXY2AD
* rkarban   18/07/98  add methods and attributes to send OFFSAD to trk
* rkarban   24/06/98  add two access methods for the guideMode
* gchiozzi  27/02/98  Data members are now protected. Methods are virtual.
* gchiozzi  27/02/98  Packed callback setting and command sending methods in a single one
* gchiozzi  08/01/98  agwsCON_SERVER_STARTAG: Implem. of ErrReplyCB() and TimeoutCB()
* gchiozzi  09/12/97  Removed orgMsg data member. Not necessary and wrongly used.
* gchiozzi  11/11/97  Added methods to handle Start and Stop of just TCCD
* gchiozzi  16/10/97  Removed get rotator angle command. Uses tif.
* gchiozzi  01/10/97  Renamed DetGs() to ErrVector()
* gchiozzi  29/02/96  created
*/

/******************************************************************************
 * Definition of the agwsCON_SERVER class.
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "agws.h"
#include "agwsCON_CAT.h"
#include "agwsGUIDE_STAR.h"
#include "agwsCMD_PARALLEL.h"
#include "evhDB_TASK.h"
#include "evhDB_LIST_COMMAND.h"

#include "agwsCON_SETUP.h"

class agwsCON_SERVER_STARTAG : public agwsCMD_PARALLEL
{
  public:
    agwsCON_SERVER_STARTAG();

    virtual ccsCOMPL_STAT    Send(msgMESSAGE &orgMsg, vltINT32 numMsgs, msgMESSAGE msg[]);

    virtual evhCB_COMPL_STAT ReplyCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT ErrReplyCB(msgMESSAGE &msg, void *);
    virtual evhCB_COMPL_STAT TimeoutCB(msgMESSAGE &msg, void *);

    virtual evhCB_COMPL_STAT ErrCorrCB(msgMESSAGE &msg, void *udata);

  private:
    vltLOGICAL      lookForFirstReply; // flag if first of periodic replies pending
    msgPROCESSID    agServerProcId;    // agServer CCS process ID
};

class agwsCON_SERVER_OBJCLOS : public agwsCMD_PARALLEL
{
  public:
    agwsCON_SERVER_OBJCLOS();

    ccsCOMPL_STAT    Init();

    evhCB_COMPL_STAT ReplyCB(msgMESSAGE &msg, void *udata);

    evhCB_COMPL_STAT WaitReplyCB(msgMESSAGE &msg, void *);
    ccsCOMPL_STAT    Reset();

  private:
    evhCOMMAND waitCmd;      // command to TCCD to wait for exposure completion
    ccsTIMEVAL waitTimeout;  // WAIT command timeout
};

class agwsCON_SERVER : public evhDB_TASK
{
  public:
    agwsCON_SERVER(const dbSYMADDRESS  dbPoint,
		   agwsCON_SETUP      &conSetup);
    virtual ~agwsCON_SERVER();

    virtual ccsCOMPL_STAT Init();
    virtual ccsCOMPL_STAT Reset();

    virtual ccsCOMPL_STAT StartAg(msgMESSAGE   &msg,
				  fndOBJECT    *obj,
				  evhCB_METHOD  reply,
				  evhCB_METHOD  errReply, 
				  evhCB_METHOD  timeout = NULL,
				  void         *udata   = NULL);

    virtual ccsCOMPL_STAT StopAg(msgMESSAGE   &msg,
				 fndOBJECT    *obj,
				 evhCB_METHOD  reply,
				 evhCB_METHOD  errReply, 
				 evhCB_METHOD  timeout = NULL,
				 void         *udata   = NULL);

    virtual ccsCOMPL_STAT OffsAg(msgMESSAGE   &msg,
				 fndOBJECT    *obj,
				 evhCB_METHOD  reply,
				 evhCB_METHOD  errReply, 
				 evhCB_METHOD  timeout = NULL,
				 void         *udata   = NULL);

    virtual ccsCOMPL_STAT OffsTrkAd(msgMESSAGE   &msg,
				    fndOBJECT    *obj,
				    evhCB_METHOD  reply,
				    evhCB_METHOD  errReply, 
				    evhCB_METHOD  timeout = NULL,
				    void         *udata   = NULL);

    virtual ccsCOMPL_STAT ErrVector(msgMESSAGE   &msg,
				    fndOBJECT    *obj,
				    evhCB_METHOD  reply,
				    evhCB_METHOD  errReply, 
				    evhCB_METHOD  timeout = NULL,
				    void         *udata   = NULL);

    virtual ccsCOMPL_STAT Setup(fndOBJECT    *obj,
				evhCB_METHOD  reply,
				evhCB_METHOD  errReply, 
				evhCB_METHOD  timeout = NULL,
				void         *udata   = NULL);

    virtual ccsCOMPL_STAT Stop(msgMESSAGE    &msg,
			       fndOBJECT    *obj,
			       evhCB_METHOD  reply,
			       evhCB_METHOD  errReply, 
			       evhCB_METHOD  timeout = NULL,
			       void         *udata   = NULL);

    virtual ccsCOMPL_STAT StrtTccd(msgMESSAGE   &msg,
				   fndOBJECT    *obj,
				   evhCB_METHOD  reply,
				   evhCB_METHOD  errReply, 
				   evhCB_METHOD  timeout = NULL,
				   void         *udata   = NULL);

    virtual ccsCOMPL_STAT StopTccd(msgMESSAGE   &msg,
				   fndOBJECT    *obj,
				   evhCB_METHOD  reply,
				   evhCB_METHOD  errReply, 
				   evhCB_METHOD  timeout = NULL,
				   void         *udata   = NULL);

    virtual ccsCOMPL_STAT XY2AD(vltDOUBLE x, vltDOUBLE y,
				msgMESSAGE   &msg,
				fndOBJECT    *obj,
				evhCB_METHOD  reply,
				evhCB_METHOD  errReply, 
				evhCB_METHOD  timeout = NULL,
				void         *udata   = NULL);

    virtual const char *GuideMode(void);

    virtual void GuideMode(const char *guideMode);

    virtual vltDOUBLE CcdPixSize();

  protected:
    virtual ccsCOMPL_STAT SetCallbacks(agwsCMD_PARALLEL *cmd,
			       fndOBJECT        *obj, 
			       evhCB_METHOD      reply,
			       evhCB_METHOD      errReply, 
			       evhCB_METHOD      timeout,
			       eccsTIMEVAL       time,
			       void             *udata      = NULL);

    /*
     * Internal callbacks for PIXSIZE command
     */
    virtual evhCB_COMPL_STAT PixSizeCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT PixSizeErrorCB(msgMESSAGE &msg, void *udata);

    // Protected Data
    agwsCON_SERVER_STARTAG  startAgCmd;     // STARTAG transfer command object
    agwsCMD_PARALLEL        stopAgCmd;      // STOPAG  transfer command object
    agwsCMD_PARALLEL        offsAgCmd;      // OFFSAG  transfer command object
    agwsCMD_PARALLEL        offsTrkAdCmd;   // trk OFFSAD  transfer command object
    evhCOMMAND              xy2adCmd;       // XY2AD  transfer command object
    agwsCON_SERVER_OBJCLOS  objClosCmd;     // OBJCLOS transfer command object
    agwsCMD_PARALLEL        setupCmd;       // SETUP   transfer command object
    agwsCMD_PARALLEL        startCmd;       // START   transfer command object
    agwsCMD_PARALLEL        stopCmd;        // STOP    transfer command object

    agwsCMD_PARALLEL        offsProbeAdCmd; // admain OFFSAD command TEST!!!!!!!

    agwsCON_CAT     	   *gsCatalogue;   // internal guide-star catalogue
    agwsCON_SETUP          &agwsConSetup;  // Reference to setup file handling obj

    ccsTIMEVAL              startagTimeout; // STARTAG transfer command timeout
    ccsTIMEVAL              stopagTimeout;  // STOPAG transfer command timeout
    ccsTIMEVAL              offsagTimeout;  // OFFSAG transfer command timeout
    ccsTIMEVAL              offsTrkAdTimeout;  // trk OFFSAD transfer command timeout
    ccsTIMEVAL              xy2adTimeout;  // XY2AD transfer command timeout
    ccsTIMEVAL              setupTimeout;   // SETUP transfer command timeout
    ccsTIMEVAL              objclosTimeout; // OBJCLOS transfer command timeout

    char                    guideMode[agwsMODE_STR_MAX_LEN + 1];

private:

    /*
     * Pixel size is initialised at init time sending 
     * the PIXSIZE command to the proper guide LCU
     */
    vltDOUBLE    ccdPixSize;
    evhCOMMAND   ccdPixSizeCommand;


};

#endif // !agwsCON_SERVER_H
