#ifndef agwsGUIDE_STAR_H
#define agwsGUIDE_STAR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: agwsGUIDE_STAR.h,v 1.198 2005/08/26 15:24:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2001-03-01 Added agwsGS_PROBE_POS enum to define positive/negative probe position
* gchiozzi  12/11/97  Replaced agwsGS_TRY_STATE enum with typedef and defines.
* nfiebig   27/09/96  derive class from fndOBJECT
* nfiebig   14/06/96  inline code move to file agwsGUIDE_STAR.icc
* nfiebig   14/06/96  class renamed to agwsGUIDE_STAR
* nfiebig   02/05/96  created 
*/


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "agws.h"
#include "fndOBJECT.h"

/*
 * literals for state when trying all guide-stars for useability
 */
typedef vltINT32 agwsGS_TRY_STATE;

/*
 * class to model a guide star
 */
enum agwsGS_PROBE_POS {
  POS = 0,
  NEG,
  NONE
};

class agwsGUIDE_STAR : public fndOBJECT
{
  public:
    agwsGUIDE_STAR();
    agwsGUIDE_STAR(vltDOUBLE alpha, vltDOUBLE delta, 
		   vltDOUBLE magnitude, 
		   agwsGS_PROBE_POS probePos = NONE,
		   vltDOUBLE wavelen = 650.0);

    void Reset();

    void Set(vltDOUBLE alpha, vltDOUBLE delta, 
	     vltDOUBLE magnitude, 
	     agwsGS_PROBE_POS probePos = NONE,
	     vltDOUBLE wavelen = 650.0,
	     agwsGS_PROBE_POS triedPos = NONE);
    void SetProbePos(agwsGS_PROBE_POS probePos);
    void SetTriedPos(agwsGS_PROBE_POS triedPos);

    vltDOUBLE        Alpha() const;
    vltDOUBLE        Delta() const;
    vltDOUBLE        Magnitude() const;
    agwsGS_PROBE_POS ProbePos() const;
    agwsGS_PROBE_POS TriedPos() const;
    vltDOUBLE        Wavelen() const;

    void             TryState(agwsGS_TRY_STATE state);
    agwsGS_TRY_STATE TryState() const;

  private:
    vltDOUBLE        alpha, delta;      // position
    vltDOUBLE        magnitude;         // magnitude
    agwsGS_PROBE_POS probePos;          // POS/NEG/NONE
    agwsGS_PROBE_POS triedPos;          // POS/NEG/NONE last prcheck probe pos
    agwsGS_TRY_STATE tryState;          // try-state
    vltDOUBLE        wavelen;           // wavelength in nm
};

/*
 * inline definition of guide star class
 */
#include "agwsGUIDE_STAR.icc"

#endif /*!agwsGUIDE_STAR_H*/
