#ifndef bossEXPOSURE_H
#define bossEXPOSURE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: bossEXPOSURE.h,v 1.153 2005/08/23 16:46:37 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* zins      08/04/99   created
* epozna    27/03/00   adopted for BOSS
* epozna    07/07/00   added function Init()     
* epozna    19/07/00   eliminate expomode argument, use expotable, restructure
*                       new func.: CleanTable(),AddExposure(),DeleteExposure(),
*                       ModifyExposure(),IsDefined(),GetFirstEmptyLine(),
*                       GetLine(), getSubsystems(),IsSubystOnList(),...    
*                      some function and priv value moved to bossINTERFACE_DCS
* epozna     25/01/01  added GetInsMode()
* epozna     26/05/01  added ResolveComplexStatus()
* epozna     28/05/01  boss observation status modified to handle CCD running status  
* - - - - - - - - - MAR2002 - - - - - - - - - - - - - - - - - - - - 
* epozna     15/05/02 VLTSW20010669 - header handling
* - - - - - - - - - APR2003 - - - - - - - - - - - - - - - - - - - -  
* epozna    04/08/03  VLTSW20020505 standard naming
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif
/*
 * VLT Software
 */
#include "slx.h"
#include "eccsErr.h"
#include "evhHANDLER.h"
#include "oslx.h"
#include "inscExpStates.h" 
#include "bossExpoStatus.h"
#include "bossEXPOTBL_HANDLE.h"
#include "bossFITS_HEADER.h"
#include "bossINTERFACE_LIST.h"
#include "bossEXPOSTATE_HANDLE.h"

// max number of exposures boss can deal with at any time
#define   MAX_EXPOSURES 10

//exposure table are refreshed after 20 exposures
//leaving the last MAX_EXPOSURES in the table always
#define   REFRESH_RATE  20

enum bossEvtType  { bossEvtSTATE=1, bossEvtSTATUS, bossEvtNEWDATA } ;
		    
class bossEXPOSURE : public bossEXPOTBL_HANDLE
{
public:
    bossEXPOSURE(const dbSYMADDRESS dbPoint);
    // base class eccsERROR class non virtual destructor
    virtual ~bossEXPOSURE();
    ccsCOMPL_STAT Init(bossINTERFACE_LIST *subSystemList,
		       oslxDICTIONARY     *dictPtr,
		       ixacKEY_DB_MAP     *keyDbMPtr);
    
    // Exposure type
    virtual ccsCOMPL_STAT SetExpoType(char *expoType);
    virtual char          *GetExpoType();
    
    // Exposure category
    virtual ccsCOMPL_STAT SetExpoCategory(char *expoCategory);
    virtual char          *GetExpoCategory();
    
    // Exposure technique
    virtual ccsCOMPL_STAT SetExpoTechnique(char *expoTechnique);
    virtual char          *GetExpoTechnique();
    
    
    // Exposure Id
    virtual ccsCOMPL_STAT GetLastExpoId(vltINT32 *expoId);
    virtual void          SetLastExpoId(vltINT32 expoId);
    virtual char          *GetExpoIdStr(vltINT32 expoId);  
   
    virtual ccsCOMPL_STAT NewExpoId(vltINT32 expoId=0);   
    virtual ccsCOMPL_STAT GetIdOfTheLastExposureInTbl(vltINT32 *lastExpoId);
    virtual ccsCOMPL_STAT UpdateExpoInfo(vltLOGICAL isStart=ccsFALSE);

    // Merging Status
    virtual ccsCOMPL_STAT  GetMergingStatus(vltLOGICAL *ismergin);
    virtual ccsCOMPL_STAT  SetMergingStatus(vltLOGICAL  ismergin); 
    virtual ccsCOMPL_STAT  SetArchQueue(vltINT32 num);

    // Exposure status
    static   char*         ExpoStatus2Str(vltUINT32 expoStatus);       
    static   vltUINT32     ExpoStatusStr2Int(char *expoStatus);
    static   vltUINT32     ResolveComplexStatus(vltUINT32 status);
    static   ccsCOMPL_STAT CompareLists(char *s1,char* s2,vltLOGICAL *issame);
    


    // Observation status
    virtual ccsCOMPL_STAT SetObsStatus(vltINT32 status);  
    virtual ccsCOMPL_STAT UpdateObsStatus(vltINT32 newExpoStatus=0);
    ccsCOMPL_STAT CommonStatus(vltINT32      *obsStatus,
			       bossINTERFACE *subSystemPtr);
    
    vltUINT32 GetGlobalExpoStatus();


    // functions for handling/checking the exposures through  database table
    // some overloaded for header handling 
    virtual ccsCOMPL_STAT CleanTable();
    virtual ccsCOMPL_STAT RefreshTable();
    virtual ccsCOMPL_STAT AddNewExposure (int expoId,       char *mode,
					  char *subsystems, char *inspath, 
					  char *detector,   char *filename,  
					  char *state,      char *expstrt);
    virtual ccsCOMPL_STAT ModifyExposure (int expoId,char * parameter, 
					  char * value);
    virtual ccsCOMPL_STAT  DeleteExposure (char * expoIdString);
    virtual  ccsCOMPL_STAT DeleteExposure (int expoId);

    virtual vltINT32      NoExposures();
    
    //identify subsystem from its data basepoint 
    ccsCOMPL_STAT        GetDbAddrSubsyst( bossEvtType     type, 
					   bossINTERFACE * subsPtr, 
					   char          * dbAddr);
    ccsCOMPL_STAT        DbAddressConv( const char *dbAddr,char* dbAddrConv);
    ccsCOMPL_STAT        GetSubsystOfDbAttr(bossEvtType     evtType, 
					    const char    * dbAttr,
					    char          * subsName);
    // -- kept for back compatibility
    virtual char *        GetDetectorOfDbAttr( const char *dbAttr); 
    virtual ccsCOMPL_STAT GetDetectorOfDbAttr( const char *dbAttr,char * detector);

    //Support functions
    virtual ccsCOMPL_STAT GetExposureStatus2(vltINT32 expoId,vltINT32 *status );
    virtual vltLOGICAL    IsSubystOnList( vltINT32 expoId, char *detId);
    virtual vltLOGICAL    IsDetectorOnDetectorList( vltINT32 expoId,
						    char *detId);
    virtual char *        DeclareDetectors(char * subSystems );
    virtual ccsCOMPL_STAT DeclareDetectors(char *subSystems, char * detectors);
    virtual ccsCOMPL_STAT CheckSubsystems(char *subSystems );
    virtual vltLOGICAL    IsTcsIcsGivenBySubsyst(vltINT32 expoId);

    // Copy exposure for repeated exposure
    virtual ccsCOMPL_STAT CopyLastExposure(vltINT32 expoId);
    virtual ccsCOMPL_STAT RmSeqNumFromName(char * filename);

    //cancel all expousres (to be called when all DCS interfaces are reset)
    ccsCOMPL_STAT CancelInactivExposures();
    ccsCOMPL_STAT InvalidExposure(vltINT32 expoId);


    //VLTSW20010669 - header handling functions
    bossFITS_HEADER * GetHeader(vltINT32 expoId);
    ccsCOMPL_STAT     GetNoOfHeaders(int &nohdr);

    bossEXPOSTATE_HANDLE    stateHandle;

    


  protected:
    //VLTSW20010669 - header handling functions
    virtual ccsCOMPL_STAT FindDeleteHdrFromTable(vltINT32 expoId);
    virtual ccsCOMPL_STAT FindDeleteHdrFromTable_noChck(vltINT32 expoId);
    virtual ccsCOMPL_STAT DeleteHdrFromTable_ChckSt(vltINT32 expoId);
    virtual ccsCOMPL_STAT CancelUnusedHeaders();
    virtual ccsCOMPL_STAT AddHeader(vltINT32 expoId);


  private:
    // support functions for CompareLists
    static ccsCOMPL_STAT ArrMng             (vltLOGICAL isa, char* arr[],int size );
    static ccsCOMPL_STAT TransferList2Array (char*      lst, char* arr[20] , int *size);
    static int           CompareEl          (const void *s1, const void *s2);
    static ccsCOMPL_STAT SortList           (char*      lst, char* newlist );

    // Declaration of copy constructor and assignment operator as private
    // methods, in order to hide them from the users.
    bossEXPOSURE& operator=(const bossEXPOSURE&);
    bossEXPOSURE (const bossEXPOSURE&);
    
    dbSYMADDRESS expoGlobStatDbAddress_;  
    
   
    // Exposure type
    vltBYTES256         expoType_;
    
    // Exposure category
    vltBYTES256         expoCategory_;
    
    // Exposure technique
    vltBYTES256         expoTechnique_;
    
    // Last expoId
    vltINT32            lastExpoId_;
    
    //Base db point of the OS
    dbSYMADDRESS        dbBaseRef_;
    bossINTERFACE_LIST *subSystemListPtr;

    //VLTSW20010669
    oslxDICTIONARY     *dictionaryPtr; 
    ixacKEY_DB_MAP     *keyDbMapPtr;
    ixacHASH            headerTable_;
    
    //VLTSW20030312
    vltLOGICAL          initialised_;

    //archiver status
    vltLOGICAL          mergingStatus_;
    vltLOGICAL          archQueue_;
};


#endif /*!bossEXPOSURE_H*/
