#ifndef bossINS_MODES_H
#define bossINS_MODES_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: bossINS_MODES.h,v 1.153 2005/08/23 16:46:38 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* lefort  10/07/98  created
* epozna  04/19/00  Modified search path for (mode) setup files. Type of variable 
*                   'insModeSetup' changed from ixacKEYW_CMD_PARAM to 
*                   bossKEYW_CMD_PARAM, Files are now searched in dir 
*                   "$INS_ROOT/$INS_USER/COMMON/SETUP/REF/"
*                   insted of envvar $INS_USER_PATH.
* epozna   10/07/00 new mode ..
*                   added functions    : ModeSetup() ,ModeSubsystems(), ModePath()
*                                        GetInsPath(), GetSubsystems()
*                   function 
*                       AddMode(): new argument 
*                       HandleInsMode() new ou argument (sets the mode name)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * VLT Software
 */
#include "fndOBJECT.h"
#include "eccsErr.h"
#include "eccsDB_TABLE.h"
#include "oslxSETUP.h"
#include "oslxSHORT_FITS.h"

/*
 * Local Headers
 */
#include "bossMODES_TABLE_ENTRY.h"
#include "bossKEYW_CMD_PARAM.h"

//#include "ixacKEYW_CMD_PARAM.h"

class bossINS_MODES : public fndOBJECT, public eccsERROR_CLASS
{
public:
    // Constructor & Destructor
    bossINS_MODES(const dbSYMADDRESS dbPoint, 
                  oslxDICTIONARY     *dictionary = NULL,
                  oslxALIAS          *alias      = NULL);
    virtual  ~bossINS_MODES();
    
    // Adds each pair of keyword.NAME and keyword.SETUP
    virtual ccsCOMPL_STAT Configure(oslxSHORT_FITS &appConfig,
                                    vltLOGICAL update = ccsTRUE);
    virtual  ccsCOMPL_STAT ModeSetup(oslxSHORT_FITS &appConfig,  
				     char **subsystems);
  
    virtual  ccsCOMPL_STAT ModeSubsystems(oslxSHORT_FITS &appConfig,  
					  char **setupParams);

    virtual  ccsCOMPL_STAT ModePath(oslxSHORT_FITS &appConfig,  
				    char **path);

   // Sets the SHORT-FITS keyword which defines the instrument mode in the
    // command SETUP. 
    virtual ccsCOMPL_STAT SetInsModeKeyword(const slxKEYW_LINE keyword);

    // Adds setup parameters corresponding to selected instrument mode.
    virtual ccsCOMPL_STAT HandleInsMode (oslxSETUP &setup,char *insModeName);
    
    // Check if one of the setup keywords influence the current instrument mode,
    virtual ccsCOMPL_STAT CheckInsMode (oslxSETUP &setup,vltLOGICAL &altered);
    
    // Reset current instrument mode, i.e set to "Undefined"
    virtual ccsCOMPL_STAT ResetCurrInsMode();

    // get the  parameters of a given mode
    virtual ccsCOMPL_STAT GetSubsystems(char *modeName, char *subsystems);
    virtual ccsCOMPL_STAT GetInsPath   (char *modeName, char *inspath);
   

    // Get number of modes, and default mode (when there is only one mode)
    virtual vltINT32  GetNoInsMode();
    virtual char     *GetDefaultMode();
    virtual char     *GetCurrentMode();

    // SOS setup prefix
    virtual ccsCOMPL_STAT AddSOSPrefix (char *prefix); 
    virtual vltLOGICAL    IsModeSet();
protected:  

    virtual ccsCOMPL_STAT AddMode (char *modeName, char *setupParams,
				   char *subsystems,char *path,
                                   vltLOGICAL update );

    virtual ccsCOMPL_STAT CleanTable();
    virtual vltLOGICAL    IsDefined(char *modeName);
    virtual vltINT32      NoInsModes();

     // Retrieve setup parameters list
    virtual ccsCOMPL_STAT GetSetupParams (char *modeName, char *setupParams);
    virtual ccsCOMPL_STAT NewMode (char *modeName, oslxSETUP &setup );

    // Update the current instrument mode in database
    virtual ccsCOMPL_STAT DbWriteInsModeName(char *insModeName);
    
    dbSYMADDRESS          dbRef;
    
    // Pointers to the dictionary and alias table
    oslxDICTIONARY        *dictionary; 
    oslxALIAS             *alias;
        
    // Database table
    eccsDB_TABLE<bossMODES_TABLE_ENTRY> *insModesTbl;

    // Filter to extract Instrument Mode Keyword from setup keywords.
    oslxKEYW_FILTER       insModeKeywFilter;

    // Command used to parse setup parameters of instrument mode
    //ixacKEYW_CMD_PARAM    insModeSetup;
    bossKEYW_CMD_PARAM   insModeSetup;

    // number of insmode defined in the cfg file
    vltINT32     noInsModeDefined_;
    vltLOGICAL   isModeSet_;

    // default mode when there is only one mode
    vltBYTES64 defaultMode_;
    vltBYTES64 currentMode_;

private:
    // Declaration of copy constructor and assignment operator as private 
    // methods, in order to hide them to users.
    bossINS_MODES& operator=(const bossINS_MODES&);
    bossINS_MODES  (const bossINS_MODES&);
};

#endif /*!bossINS_MODES_H*/
