#ifndef bossINTERFACE_DCS_H
#define bossINTERFACE_DCS_H

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: bossINTERFACE_DCS.h,v 1.153 2005/08/23 16:46:38 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gzins     04/12/99  created
* epozna              BOSS
* epozna    07/06/00  added functions (from xxo) :
*                     SetImageFileName,SetImageFileSize,GetImageFileSize
* epozna    10/07/00  o added protected variables:dbAddressStatus_, dbAddressFileName_
*                     o added functions to set get these variables  
* epozna    24/04/01  added functions Cancel
* ------------------------MAR2002----------------------------------------------
* epozna    21/05/02  VLTSW20010669 added nested class to improve sinchronisation
* epozna    14/02/03  VLTSW20020360 unexpected event
*-------------------------APR2003---------------------------------------------- 
* epozna    04/08/03  VLTSW20020505 - standard naming Set/GetImgExt;GetBaseName
*-------------------------APR2004---------------------------------------------- 
* epozna    14/12/04  - Updated Wait error reply
*                     - VLTSW20040321 Added detectortype WITH or WITHOUT_SHUTTER
*                       IsWithShutter,SetType,GetType
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "bossINTERFACE.h"
#include "inscExpStates.h"

#include "slx.h"
#include "eccsErr.h"
#include "evhHANDLER.h"
#include "oslx.h"
#include "inscExpStates.h"
#include "bossEXPOSURE_TABLE_ENTRY.h"
#include "bossINTERFACE_LIST.h"
//#include "bossFITS_HEADER.h"
#include "bossWaitCmdHandler.h"


//For synchronisation flags
enum ExpAction      { START_ACTION=1, EXPEND_ACTION, 
		      SUCCESS_ACTION, NEWDATA_ACTION, MERGE_ACTION} ;
enum ExpActionState { IDLE=1,PASSED,RUNNING,DONE,FAILED }  ;
enum DcsType        { WITH_SHUTTER=1, WITHOUT_SHUTTER }  ;

//PPRS-00010177 - For enable/disbale exposure flags
enum  enbld    { enbldFalse=0, enbldTrue, enbldUnknown } ;

class bossINTERFACE_DCS : public bossINTERFACE
{
public:
  
    // >>> construcror
    bossINTERFACE_DCS(const dbSYMADDRESS dbPoint, vltINT32 srvId); // obsolate
    bossINTERFACE_DCS(const dbSYMADDRESS dbPoint, char * srvId);
    

    // >>> desructor
    virtual ~bossINTERFACE_DCS();
    
    
    // >>> nested class to hold synchronisation data 
    class Synchronise
    {
    public:
      Synchronise(ccsERROR *errorstack);
      virtual ~Synchronise();
      ccsCOMPL_STAT  SetAction(ExpAction action,ExpActionState state,char*data=NULL);
      ExpActionState GetAction(ExpAction action);
      virtual ccsCOMPL_STAT  IsActionAllowed( ExpAction action ,vltLOGICAL & isallowed);
      ccsCOMPL_STAT  Reset();

    private:
      ExpActionState expstart_;
      ExpActionState expend_;
      ExpActionState newdata_;
      ExpActionState success_;

      ccsERROR *errStackPtr;
    };
  
    // >>> overloaded to reset the synchr object
    ccsCOMPL_STAT SetExpoId(vltINT32 expoId);

    // >>> functions for synchronisation
    ccsCOMPL_STAT  SynchroniseExpActions (ExpAction action,ExpActionState state,
					  char*data=NULL);
    ExpActionState ActionState (ExpAction action);
    ccsCOMPL_STAT  IsActionAllowed( ExpAction action ,vltLOGICAL & isallowed);
    vltLOGICAL     IsCurrExpOutOfScope();

    // >>> configure
    virtual ccsCOMPL_STAT ConfigureIfSpecKeywords(char* param,char* value);
    virtual ccsCOMPL_STAT ConfigureIfSpecPostProc(ctooCONFIG *appConfig);

    // >>> init
    virtual ccsCOMPL_STAT  Init(oslxDICTIONARY     *dictionaryPtr,
				bossWaitCmdHandler *waitCmdHandlPtr,
				ixacKEY_DB_MAP     *keyDbMapPtr);
    // >>> abstract function to set the filename of the DCS automatically
    virtual ccsCOMPL_STAT SetImageFileName(const char *filename,
					   vltINT32 expoId)=0;

    virtual ccsCOMPL_STAT SetImageFileSize(vltINT32 noOfBlocks);
    virtual vltINT32      GetImageFileSize();
    
    virtual ccsCOMPL_STAT SetDbAddressStatus(char *); 
    virtual ccsCOMPL_STAT SetDbAddressStatus(); // set the deafult values
    virtual char *        GetDbAddressStatus();
    
    virtual ccsCOMPL_STAT SetDbAddressFileName(char *);
    virtual ccsCOMPL_STAT SetDbAddressFileName();
    virtual char *        GetDbAddressFilename();
    virtual ccsCOMPL_STAT SetDbAddressErrDesc();

    // >>> Get exposure status from the databasepoint
    virtual vltUINT32 GetExpoStatus();
  
    virtual ccsCOMPL_STAT SetExpoStatusEvtCB(evhHANDLER *evhHandler,
                                             evhOBJ_CALLBACK &callback);

    virtual ccsCOMPL_STAT DisableExpoStatusEvt();
    virtual ccsCOMPL_STAT EnableExpoStatusEvt();

    // >>> Handle new data file name status events
    virtual ccsCOMPL_STAT SetNewDataFileNameEvtCB(evhHANDLER *evhHandler,
                                                  evhOBJ_CALLBACK &callback);
    virtual ccsCOMPL_STAT DisableNewDataFileNameEvt(); 
    virtual ccsCOMPL_STAT EnableNewDataFileNameEvt();

    // >>> Exposure control
    ccsCOMPL_STAT SetStatusEvt(evtEVENT* evtptr);
    ccsCOMPL_STAT SetNewDataEvt(evtEVENT* evtptr);
    
    virtual ccsCOMPL_STAT EnableExposure();
    virtual ccsCOMPL_STAT DisableExposure();
    virtual vltLOGICAL IsActive(); 
    // Error during Failure event  (VLTSW20030437)       
    virtual ccsCOMPL_STAT AddFailureError(); 

    // >>>  Handle new data file name
    virtual ccsCOMPL_STAT StoreNewDataFileName(const char *fileName);
    virtual vltINT32      NoOfNewDataFileName();
    virtual char          *GetNewDataFileName(vltINT32 fileNo);  
    virtual vltLOGICAL    IsOnList(char * filename);

    virtual ccsCOMPL_STAT Clear(); 
    virtual ccsCOMPL_STAT ResetData(); 
    virtual void          Reset(vltINT32 expoId); 
    virtual ccsCOMPL_STAT Cancel(vltINT32 expoId); 

    // >>>  file
    vltLOGICAL    ImageFileExits(char *fileName);
    ccsCOMPL_STAT CheckForPartialFiles(char *filename,vltLOGICAL*exists);

    char *  GetFullFileName(); 
    char *  GetSimpleFileName(const char* completeName);
    ccsCOMPL_STAT  GetSimpleFileName(const char* completeName,char* fname);
    char *  GetFileName();
    virtual ccsCOMPL_STAT GetFileName(vltINT32 expoId, char *imageFileName);
    virtual ccsCOMPL_STAT SetNextFileName( vltLOGICAL checkFile=ccsFALSE);

    virtual ccsCOMPL_STAT ClearFileName();

    virtual vltINT32      GetNamingType();
    virtual ccsCOMPL_STAT SetNamingType(vltINT32 namingType,vltLOGICAL setname=ccsTRUE);
    virtual ccsCOMPL_STAT SetReqFileName(char *newFileName);
    virtual ccsCOMPL_STAT SetSeqBaseName(const char *baseName);
    virtual ccsCOMPL_STAT SetSeqIndex(vltUINT32 seqIndex);
    virtual vltUINT32     GetSeqIndex();
    virtual ccsCOMPL_STAT SetNextSeqName();
    virtual ccsCOMPL_STAT SetFileName(char *newFileName); 
    // VLTSW20020505 
    virtual ccsCOMPL_STAT SetImgExt(char* imgext); 
    char *  GetImgExt();                             
    char *  GetBaseName();

    // >>> Inform VOLAC
    virtual void          Archive(vltLOGICAL isArchive);
    virtual vltLOGICAL    isArchive();
    ccsCOMPL_STAT         DisplayLifeCycle(vltINT32 expoId, vltUINT32 newExpoStatus);
    ccsCOMPL_STAT         DbWriteLifeCycle(char*dbAddress , vltLOGICAL value);

    // >>> fitsheader
    bossFITS_HEADER    *fitsStatusPtr;

    virtual ccsCOMPL_STAT    SendWaitCmd(msgMESSAGE &msg);
    virtual void             WaitEvhCmdReset();	
    virtual vltLOGICAL       IsWait(); 	     
    virtual evhCB_COMPL_STAT WaitReplyCB    (msgMESSAGE &msg, void *  );
    virtual evhCB_COMPL_STAT WaitErrReplyCB (msgMESSAGE &msg, void *  );
    virtual evhCB_COMPL_STAT WaitTimeOutCB  (msgMESSAGE &msg, void *  );
    virtual ccsCOMPL_STAT    subsMsgModify (const msgCMD command,
					    msgMESSAGE &msg);

    // >>> add check for stop (VLTSW20020280)
    virtual ccsCOMPL_STAT Forward(msgMESSAGE &msg,
                                  vltLOGICAL timeoutFlag=ccsTRUE,
				  vltLOGICAL checkstop =ccsFALSE);			  
 
    virtual ccsCOMPL_STAT Forward(const msgCMD command,
                                  const char *parameters,
                                  msgLENGTH parametersLen=0,
                                  vltLOGICAL timeoutFlag=ccsTRUE);
    
    // >>> Retrieve configuration data for  wipe/stop or multifiles
    vltLOGICAL IsWipe(); 
    vltLOGICAL IsStop();
    vltLOGICAL IsArcModeMergeAll();

    // >>> Handle errors o during events or err reply to WAIT
    ccsCOMPL_STAT ClearWaitErr();
    ccsERROR *    GetWaitErr();

    // flag denoting if unexpected event has been arrived from the subsystem
    // i.e. subsystem is handled bypassing the OS (VLTSW20020360)
    void Standalone(vltLOGICAL value);
    vltLOGICAL IsStandalone();
   
    //Type of detector (VLTSW20040321)
    DcsType       GetType();
    ccsCOMPL_STAT SetType(DcsType type); 
    vltLOGICAL    IsWithShutter();

    // Set handling of multifiles
    ccsCOMPL_STAT      SetArcMode(char* value);

protected:
    // executed after forwarded command is completed
    virtual evhCB_COMPL_STAT CompletedCB(msgMESSAGE &msg, void *userData);

    // Set configuration data for  wipe/stop 
    void               SetWipe(char* value);
    void               SetStop(char* value);

    // db address of status and filename
    dbSYMADDRESS        dbAddressStatus_          ;  
    dbSYMADDRESS        dbAddressFileName_        ;
    dbSYMADDRESS        dbAddressStatusDefault_   ;  
    dbSYMADDRESS        dbAddressFileNameDefault_ ;
    // db address of error description of failure evt (VLTSW20030437)
    dbSYMADDRESS        dbAddrErrDesc_            ;  
    dbSYMADDRESS        dbAddrErrDescDef_         ;

    // Set the image data file 
    
    slxFILENAME         imageFileName_;  
    slxFILENAME         fullFileName_; 
    slxFILENAME         imgext_; //VLTSW20020505

    // The size (in Kb) of the the image data file produced at the end of
    // exposure.
    vltINT32            imageFileSize_;

    //vltINT32          expoId_;  // declared in  bossINTERFACE.h
    evtEVENT            *evtExpoStatus_;   
    evtEVENT            *evtNewDataFileName_;
    vltLOGICAL          expoActive_; 
    vltINT32            noOfDataFileName_;
    oslxDYN_BUF         dataFileName_;  
  
    vltINT32            namingType_;       
    // Base filename when sequence naming
    slxFILENAME         reqFileName_;      
    slxFILENAME         seqBaseName_;      
    
    // Index when sequence naming
    vltINT32            seqIndex_;
    vltINT32            currentDayOfYear_;

    vltLOGICAL          isArchive_;
    evhCOMMAND          waitEvhCmd_;
    bossWaitCmdHandler   *waitCmdHandlerPtr;

    // Store configuration data for  wipe/stop isfiles (VLTSW20020280/1,VLTSW20040364) 
    vltLOGICAL wipe_;
    vltLOGICAL iswipe_;
    vltLOGICAL stop_; 
    vltLOGICAL isArcModeMergeAll_; 

    //Type of detector (VLTSW20040321)
    DcsType detectorType_;
    // this flag is set to be true when subsystem is accessed directly
    // bypassing the OS 
    vltLOGICAL standalone_;

private:

    // Called during the constructor
    ccsCOMPL_STAT  InitVarsIfDcs();

    // data of previous exposure used for reseting seqindex when changes
    vltINT32    prevExpDayOfYear_  ;
    vltINT32    prevExpNamingType_ ;
    vltBYTES256 prevExpImgExt_     ;
    vltBYTES256 prevExpSeqBaseName_;

    // flags for status
    enbld isEnabledStatus_  ;
    enbld isEnblNewData_    ;

    //VLTSW20010669: nested class to hold data for the sincronistaion    
    Synchronise sincronise_;

    //Collect the errors e.g. returned during the internal wait cmds or events
    ccsERROR   waitErrorStack_;
    // Declaration of copy constructor and assignment operator as private
    // methods, in order to hide them from the users.
    bossINTERFACE_DCS& operator=(const bossINTERFACE_DCS&);
    bossINTERFACE_DCS (const bossINTERFACE_DCS&); 
  
};



#endif /*!bossINTERFACE_DCS_H*/
