#ifndef bossINTERFACE_LIST_H
#define bossINTERFACE_LIST_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: bossINTERFACE_LIST.h,v 1.153 2005/08/23 16:46:39 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* zins     05/06/98  created
* epozna   27/03/00  updated to bossDic added GetSubSystem(char *)
* epozna   10/10/00  added function: GetSubSystemByServId(char*)
* epozna   06/02/02  update to ctooCONFIG 
* - - - - - - - - - - - - APR2003 - - - - - - - - -
* epozna   28/07/03  VLTSW20030101 GetOverallState updated with isTcs argument
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/* 
 * System Headers 
 */
#include <list>
//#include "std::list"
#include <algorithm>

#include "errCCS.h"
#include "evhDB_SYNC.h"
#include "bossINTERFACE.h"
#include "ixacKEYW_CMD_PARAM.h" 
#include "ixacKEY_DB_MAP.h"
#include "ctooCONFIG.h" 


using namespace std;

#define bossMAX_SUBSYST   15
#define bossMAX_DBEVENTS  5*bossMAX_SUBSYST

typedef unsigned char      bossSUBSYSTEM_CATG[16]; 

class bossINTERFACE_LIST : public evhDB_SYNC 
{
public:
    bossINTERFACE_LIST();
    virtual ~bossINTERFACE_LIST();

    // Set database root point
    virtual ccsCOMPL_STAT SetDbPoint(const dbSYMADDRESS dbPoint);
 
    // Add sub-system to the list
    virtual ccsCOMPL_STAT AddAtTail(bossINTERFACE *subSystemPtr);

    // Return number of sub-system
    virtual vltINT32 NoOfSubSystem();

    // Return pointer to the requested sub-system
    virtual bossINTERFACE *GetSubSystem(char *subSystemName);
    virtual bossINTERFACE *GetSubSystem(vltINT32 index);
    virtual bossINTERFACE *GetSubSystemByServId(char * servId);
    virtual bossINTERFACE *GetSubSystemByProcName(char * process);

    // Generic INTERFACE configuration
    virtual ccsCOMPL_STAT Configure(ctooCONFIG *appConfig,
				    oslxDICTIONARY *DictionaryPtr);

    //initialisation of the dcs subsystems
    virtual ccsCOMPL_STAT Init(oslxDICTIONARY *dictionaryPtr,
			       ixacKEY_DB_MAP *keyDbMapPtr);

    // Set access mode for sub-systems
    virtual ccsCOMPL_STAT SetAccess(char *accessMode);
    virtual vltLOGICAL    IsSubSystemAvailable();

    // Load sub-system dictionaries
    virtual ccsCOMPL_STAT LoadDic(oslxDICTIONARY &dictionary);

    // Handle sub-system state events
    virtual ccsCOMPL_STAT SetStateEvtCB(evhHANDLER *evhHandler,
                                        evhOBJ_CALLBACK &callback);
    virtual ccsCOMPL_STAT SetExpoStatusEvtCB(evhHANDLER *evhHandler,
                                        evhOBJ_CALLBACK &callback);
    virtual ccsCOMPL_STAT SetNewDataFileEvtCB(evhHANDLER *evhHandler,
                                        evhOBJ_CALLBACK &callback);
    virtual ccsCOMPL_STAT SetOsNewDataEvtCB(evhHANDLER *evhHandler,
					    evhOBJ_CALLBACK &callback);

    virtual ccsCOMPL_STAT DisableStateEvt(); 
    virtual ccsCOMPL_STAT EnableStateEvt(); 
    virtual vltLOGICAL    IsSubSystemStateAvailable();

    // Get 'global' sub-systems state
    virtual vltINT32 GetOverallState(vltLOGICAL isTcs=ccsTRUE);

    // Send message to sub-systems
    virtual ccsCOMPL_STAT Send(msgMESSAGE      &msg, 
                               const vltUINT32 category=bossALL_SUBSYS,
                               vltLOGICAL      availableOnly=ccsFALSE,
                               vltLOGICAL      timeoutFlag=ccsTRUE);
    
    // Send command that contains keywords to sub-systems
    virtual ccsCOMPL_STAT Send(ixacKEYW_CMD_PARAM &cmd, 
                               vltLOGICAL delKeywords);


    // Forward message to sub-systems
    virtual ccsCOMPL_STAT Forward(msgMESSAGE      &msg, 
                                  const vltUINT32 category=bossALL_SUBSYS,
                                  vltLOGICAL      availableOnly=ccsFALSE,
                                  vltLOGICAL      timeoutFlag=ccsTRUE);
    // Forward command that contains keywords to sub-systems
    virtual ccsCOMPL_STAT Forward(ixacKEYW_CMD_PARAM &cmd, 
                                  vltLOGICAL delKeywords);
    // Forward SETUP command to sub-systems
    virtual ccsCOMPL_STAT Forward(oslxSETUP &setup, 
                                  vltLOGICAL delKeywords,char * subsystems);

    // Set and get calling process
    virtual ccsCOMPL_STAT SetRequestor(msgMESSAGE &msg);
    virtual msgMESSAGE &GetRequestor();

    // Check if a reply from one or more sub-system is waited
    virtual vltLOGICAL IsWaitingReply();
    virtual vltLOGICAL IsIcsWaitingReply();
    
    // Synchronize replies from sub-systems
    virtual ccsCOMPL_STAT SyncReply();
    virtual ccsCOMPL_STAT SyncReply(evhOBJ_CALLBACK callBack);

    // Default callback to be executed when all replies are received
    virtual evhCB_COMPL_STAT SyncReplyCB(msgMESSAGE &msg, void* userData);

    // Attach callback to command completion
    virtual ccsCOMPL_STAT NotifyCmdCompletion(evhOBJ_CALLBACK callBack);

    // Reset sub-systems synchronization
    virtual ccsCOMPL_STAT SyncReset();

    // Return the error status for the last execution of the command
    virtual ccsCOMPL_STAT GetErrStatus();
    
    // Return the error message for the last execution
    virtual ccsERROR *GetErrMsg();

    // Clear the error message
    virtual ccsCOMPL_STAT ClearErr();
    
    // Message reply buffer
    virtual oslxDYN_BUF   *Buffer();

    //useful function
    static vltLOGICAL IsElement(char *elem,char *list);
  

  protected:
    // Real callbacks. Call hooks and do internal administration
    virtual evhCB_COMPL_STAT EventSyncReplyCB(msgMESSAGE &msg, void *udata);

   // Notify that command execution is finished
    virtual ccsCOMPL_STAT Notify(); 
   
  private:
    // Copy constructor
    bossINTERFACE_LIST& operator=(const bossINTERFACE_LIST&);
    bossINTERFACE_LIST(const bossINTERFACE_LIST&);
    
    list<bossINTERFACE *> subSystemList;
    list<bossINTERFACE *>::const_iterator subSystemPtr;
    
    list<evtEVENT *> evhList;
    list<evtEVENT *>::const_iterator evhPtr;

    // Number of sub-systems in the list
    vltBYTES256 subswCurr;
    vltINT32 noOfSubsystem;
    
    // Root database point
    dbSYMADDRESS dbRef;

    // Communication handling parameters 
    ccsERROR     errMsg;       // Error message (when error occured).
    msgMESSAGE   requestor;    // Request message  
    oslxDYN_BUF  buffer;       // Internal message buffer to store reply.

    // Callback to be called when replies received 
    evhCALLBACK  *syncReplyCB; // User's callback

    // Database events handling
    evtEVENT     cmdCompletionEvt;// Command completion event

    evtEVENT *dbevents[bossMAX_DBEVENTS];
    vltINT32 evtCount;
   
};

#endif /*!bossINTERFACE_LIST_H*/
