#ifndef bossKEYW_CMD_PARAM_H
#define bossKEYW_CMD_PARAM_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: bossKEYW_CMD_PARAM.h,v 1.153 2005/08/23 16:46:41 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* zins      02/12/98  created
* zins      19/05/99  added CmdParamDefault() method.
* martiny   20/05/99  declared ATTOL class as friend to be able to access
*                     private and protected members for unit test purposes
* epozna    19/04/00  updated search for mode setup file in HandleCmd()
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * VLT Software
 */
#include "oslxSHORT_FITS.h"
#include "ixacCMD_PARAM.h"

class bossKEYW_CMD_PARAM : public oslxSHORT_FITS
{
#ifdef TEST_ATTOL
  friend class  ATTOL;
#endif
public:
    bossKEYW_CMD_PARAM(const char           *cmdName,
                       const char           *paramPtr,
                       const char           *fileParamName  = "file",
                       const char           *keywParamName  = "function",
                       const vltLOGICAL     valuedKeywFlag  = ccsFALSE,
                       const oslxDICTIONARY *dictionaryPtr = NULL, 
                       const oslxALIAS      *aliasTablePtr = NULL);

    virtual ~bossKEYW_CMD_PARAM();

    virtual ccsCOMPL_STAT LoadKeywords(const vltLOGICAL  updateOnly = ccsFALSE);
    virtual ccsCOMPL_STAT HandleCmd(const vltLOGICAL  updateOnly = ccsFALSE);

    // Methods to generate a buffer which is suitable for sending via CCS.
    // The format of the generated buffer will be the position dependent
    // format as specified by the CDT for the command.
    // The oslxKEYW_FILTER class can be used to extract special keywords.
    virtual ccsCOMPL_STAT MakeMsgBuf(oslxKEYW_FILTER *filter = NULL);
    virtual ccsCOMPL_STAT MakeMsgBufWithoutKeyword();
    virtual const char *GetMsgBuf();

    // The following methods are defined as inline functions to access
    // ixacCMD_PARAM methods as if ixacCMD_PARAM was a parent class.
    virtual ccsCOMPL_STAT CmdParamParse(char *paramPtr = NULL );
    virtual ccsCOMPL_STAT CmdParamDefault(const ccsPROCNAME processName);

    virtual const char* GetCommand();
    virtual const char* GetParam();
    virtual const vltLOGICAL IsValuedKeyw();
    
    virtual ccsCOMPL_STAT CmdParamGetLogical
        (const vltINT32 index, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdParamGetInt
        (const vltINT32 index, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetDouble
        (const vltINT32 index, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetString
        (const vltINT32 index, vltINT32 *paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetLogical
        (const char *paramName, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdParamGetInt
        (const char *paramName, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetDouble
        (const char *paramName, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetString
        (const char *paramName, vltINT32 *paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetLogical
        (const vltINT32 index, vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamSetInt
        (const vltINT32 index, vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetDouble
        (const vltINT32 index, vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetString
        (const vltINT32 index, vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetSingleString
        (const vltINT32 index, char *value);

    virtual ccsCOMPL_STAT CmdParamSetLogical
        (char *paramName, vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamSetInt
        (char *paramName, vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetDouble
        (char *paramName, vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetString
        (char *paramName, vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetSingleString
        (char *paramName, char *value);

    virtual ccsCOMPL_STAT CmdParamAddLogical(vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamAddInt
        (vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamAddDouble
        (vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamAddString
        (vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamAddSingleString(char *value);

    virtual ccsCOMPL_STAT CmdParamDelete(const vltINT32 index);

    virtual ccsCOMPL_STAT CmdParamDelete(const char *paramName);

    // virtual ccsCOMPL_STAT CmdParamToMsg(char*param, msgLENGTH *paramLen);

protected:

private:
    bossKEYW_CMD_PARAM& operator=(const bossKEYW_CMD_PARAM&);

    bossKEYW_CMD_PARAM (const bossKEYW_CMD_PARAM&);

    ixacCMD_PARAM cmdParam;
    vltBYTES32    fileParam;
    vltBYTES32    keywParam;
    slxDYN_BUF    msgBuffer;
    vltLOGICAL    valuedKeyw;
};


/* Include the code for inline methods */
#include "bossKEYW_CMD_PARAM.icc"

/*  Inline methods */

#endif /*!bossKEYW_CMD_PARAM_H*/
