/******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: cai.h,v 1.41 2005/07/27 11:57:34 vltsccm Exp $" 
*
* cai.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* P.Duhoux  19/09/94  Created  for Release 1.1
* P.Duhoux  26/10/94  Modified for Release 1.2
*                     - New interface to va functions
* P.Duhoux  14/11/94  Modified for Release 1.3
*                     - add functions caiRWTable() and caiRWVector()
*                     - removed entry number from caiGetXxx()
* P.Duhoux  26/04/95  Modified for Release 1.4
*                     - extend addressing capability (SPR 950173)
* P.Duhoux  16/05/95  Modified for Release 1.5
*                     - adapted for WS & LCU
* P.Duhoux  19/07/95  Function macros replaced by prototypes
* P.Duhoux  22/08/95  Added function caiGetAttrInfo()
* P.Duhoux  03/04/96  RW Function prototypes replaced by macros
*/
/******************************************************************************
* cai.h - common access interface functions interface file
*
* This header file contains data type definitions and function prototypes for 
* the resolution and loading of SDL function entry and attribute direct
* address tables.
*
*------------------------------------------------------------------------------
*/

#ifndef CAI_H
#define CAI_H

/******************************************************************************
 * Header Files
 *****************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

#include "CCS.h"                               /* CCS/LCC common definitions */
#include "caiErrors.h"                         /* CAI error codes            */

/* literal for CAI library version */
#define caiVERSION      "$Revision: 1.41 $"
#define caiVERSION_DATE "MAR2005"

/* CAI module name */
#define caiMODULE       "cai"
#define caiMODULE_NAME  "Common Access Interface"
 
/*
 * Macros for RW functions
 */
#define caiReadAttribute(t,a,b,e)      caiRWAttribute(t,FALSE,a,b,e)
#define caiWriteAttribute(t,a,b,e)     caiRWAttribute(t,TRUE,a,b,e)
#define caiReadTable(t,a,b,F,L,f,l,e)  caiRWTable(t,FALSE,a,b,F,L,f,l,e)
#define caiWriteTable(t,a,b,F,L,f,l,e) caiRWTable(t,TRUE,a,b,F,L,f,l,e)
#define caiReadVector(t,a,b,fe,le,e)   caiRWVector(t,FALSE,a,b,fe,le,e)
#define caiWriteVector(t,a,b,fe,le,e)  caiRWVector(t,TRUE,a,b,fe,le,e)

/*
 * Function prototypes
 */
#ifdef MAKE_VXWORKS
/*
 * Generic SDL Function prototype
 */
typedef ccsCOMPL_STAT (* caiVFUNCTION)( void *desc, ccsERROR *error, 
				        va_list parList );

ccsCOMPL_STAT caiGetFunctions  ( char **table, const vltBYTES8 prefix, 
				 char *funcNames[], ccsERROR *error);
ccsCOMPL_STAT caiCallFunction  ( const char *table, int funcNum, 
				 ccsERROR *error, void *desc, ... );
void          caiFreeFuncTable ( char *table );
#endif

ccsCOMPL_STAT caiGetAttributes ( char **table, const dbSYMADDRESS point, 
				 char *attrNames[], ccsERROR *error);
ccsCOMPL_STAT caiGetAttrInfo   ( const char *table, int attrNum, 
				 dbATTRTYPE *attrTyp, vltUINT8 *fldCnt, 
                                 vltUINT16 *recCnt, vltUINT32 *recSize, 
				 vltUINT16 *recsUsed, dbTYPE *dtaTyp, 
				 ccsERROR *error);
ccsCOMPL_STAT caiRWAttribute   ( const char *table, vltLOGICAL writeFlag, 
				 int attrNum, void *attrBuffer,
				 ccsERROR *error);
ccsCOMPL_STAT caiRWTable       ( const char *table, vltLOGICAL writeFlag, 
				 int attrNum, void *attrBuffer, 
				 int firstRec, int lastRec, 
				 int firstFld, int lastFld, 
				 ccsERROR *error);
ccsCOMPL_STAT caiRWVector      ( const char *table, vltLOGICAL writeFlag, 
				 int attrNum, void *attrBuffer, 
				 int firstElt, int lastElt, 
				 ccsERROR *error);
void          caiFreeAttrTable ( char *table );

void          caiVersion       ( vltBYTES80 versionStr );

#ifdef __cplusplus
}
#endif

#endif /* if !CAI_H */

/*************************************************************************/

/*___oOo___*/
