/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccd.h,v 4.11 2005/06/22 10:58:08 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* pduhoux  08/07/99   SPR990046 : Added macro ccdFITSDATASUFF = "fits" 
*                     for FITS file extensions
* alongino  24/07/95  created
* alongino  17/08/95  ccdCAMERANAME_LEN moved from ccdInternal.h
* alongino  18/08/95  SETUP parameters moved from ccdInternal.h and modified
* alongino  23/08/95  added ccdSETUPRES and ccdCheckSetup
* alongino  24/08/95  added ccdWINDOW and ccdSETUP
*                     modified macro for commands (added CMD in front)
* alongino  31/08/95  added ccdFITSHIERSUFF
* alongino  04/09/95  ccdGetIndexFromId moved from ccdInternal.h
* alongino  29/09/95  ccdDatabase.h replaced with ccdDbPublic.h
* alongino  30/01/96  added definitions for command parameters
* pduhoux   07/03/96  removed 'const' from prototypes
* pduhoux   24/05/96  added WAIT modes               
* alongino  10/06/96  moved from module ccdcon to ccd             
* alongino  09/07/96  added ccdIP_NO_USER_FUN and ccdIP_NO_USER_BUF           
* alongino  12/07/96  added ccdCMD_WAIT          
* pduhoux   22/07/96  added doTransfer/doProcess flags in ccdSETUPRES
* alongino  09/10/96  DET.FRAM.TYPE DET.FRAM.BIAS DET.FRAM.FLATF implemented
* pduhoux   11/11/96  added setup parameters to ccdIPIMAGE data structure
* alongino  15/11/96  added ccdObj.h
* alongino  13/12/96  added ccdCMD_STAND_ALONE
* alongino  16/12/96  added parameters for ccdCMD_STAND_ALONE
* pduhoux   17/01/97  added macro ccdMAXLENFILE = 31 (SPR960683)
* pduhoux   12/02/97  added setup macros ccdTIME_START/REP/DEF
* pduhoux   04/03/97  added ccdPROCESS data structure
*                     replaced IP setup by array for multiple window IP
* pduhoux   12/03/97  VERSION_DATE is MAY97
* pduhoux   12/03/97  added cmd STARTTL/STOPTL and STARTTM/STOPTM
*                     moved cmd OFF/ONLINE from ccdInternal.h
* pduhoux   13/03/97  removed setup keywords for former one window processing
* pduhoux   14/03/97  added ccdKEY_TMPSTAT & ccdKEY_TELSTAT for STATUS cmd
* pduhoux   16/04/97  added <gainIndex> in ccdCONFCLOCK
* pduhoux   02/06/97  new data structure ccdIPIMAGE
*                     enhanced ccdSETUP/ccdPROCESS data structures
* pduhoux   30/06/97  added <timeDefIdem>, <linesToShiftType> and
*                     <linesToShiftMulti[]> in ccdSETUP data structure
*                     added setup macros ccdEXP_STEP_DEF, ccdTIME_DEF_IDEM,
*                     ccdTIME_DEF1 to ccdTIME_DEF10, ccdREAD_SHIFT_TYPE,
*                     ccdREAD_SHIFT1 to ccdREAD_SHIFT10
* pduhoux   30/06/97  added <elAdu[]> in ccdCONFCLOCK
* pduhoux   26/07/97  added ccdTHRMIN_5SIGMA_WINDOW_SELF/PREV
*                     added setup macro ccdREAD_IMAGE_SIM
*                     added field <fileNameSimu> in ccdSETUP
* pduhoux   04/08/97  generalized support to ccdTHRMIN & ccdBACKGND
* pduhoux   16/09/97  removed timeStart from ccdSETUP (unused)
* pduhoux   10/11/97  moved ccdVERSION & ccdVERSION_DATE to 
*                     ccdint/include/ccdintDefines.h
* alongino  09/05/98  ccdIPEXPOSURE enhanced with more time info
* alongino  12/05/98  added ccdDO_WIPE and ccdXFER_SYNC
*                     wipeTime and itSync in ccdSETUP
* pduhoux   25/05/98  added some fields in ccdIPEXPOSURE
* pduhoux   12/08/98  added command ccdCMD_STOP_WAIT (STPWAIT)
*                     renamed DET.WINi.ASUIT into DET.WINi.ASUIT1
*/
/*************************************************************************
*  This file contains all definitions used by external modules using the 
*  CCD sw
*************************************************************************/

#ifndef CCD_H
#define CCD_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCS & LCC header files
 */
#include "CCS.h"
#include "cai.h"

/*
 * CCD header files
 */
#include "ccdDbPublic.h"
#include "ccdErrors.h"
#include "ccdObj.h"

/* CCD module name */
#define ccdMODULE       "ccd"

/*
 * Constants
 */
#define ccdENV_LENV       "CCDLENV"   /* name of env. var. for LCU env.      */
#define ccdENV_DID        "CCDDID"    /* name of env. var. for Dictionary    */
#define ccdDID            "CCDDCS"    /* name of default CCD Data Dictionary */
#define ccdCAT            slxDETECTOR /* name of FITS category               */
#define ccdEXP_NEXT       -1          /* ID for next exposure to start       */
#define ccdEXP_LAST       0           /* ID for last started exposure        */
#define ccdCAMERANAME_LEN 8           /* maximum length of camera name       */
#define ccdMAXLENFILE     63          /* maximum length of FITS file names   */
#define ccdMAXBINX        8           /* maximum binning allowed along x     */
#define ccdMAXBINY        8           /* maximum binning allowed along y     */
#define ccdFITSHIERSUFF   "det"       /* suffix for FITS HIER DET file       */
#define ccdFITSDATASUFF   "fits"      /* suffix for FITS data file           */
#define ccdDCSHEADLINES   21      /* lines written by CCD DCS in FITS header */
#define ccdDCSXTNDLINES   22      /* lines written by CCD DCS in FITS header */
                                  /* XTENSION part                           */

/* commands implemented */
#define ccdCMD_ABORT       "ABORT"
#define ccdCMD_BIAS        "BIAS"
#define ccdCMD_CONTINUE    "CONT"
#define ccdCMD_DISPLAY     "DISPLAY"
#define ccdCMD_DUMP        "DUMP"
#define ccdCMD_END         "END"
#define ccdCMD_EXIT        "EXIT"
#define ccdCMD_FLAT        "FLAT"
#define ccdCMD_GRAB        "GRAB"
#define ccdCMD_INIT        "INIT"
#define ccdCMD_OFF         "OFF"
#define ccdCMD_ONLINE      "ONLINE"
#define ccdCMD_OPERATE     "OPERATE"
#define ccdCMD_PAUSE       "PAUSE"
#define ccdCMD_PDOWN       "PDOWN"
#define ccdCMD_SETUP       "SETUP"
#define ccdCMD_STAND_ALONE "STANDAL"
#define ccdCMD_STANDBY     "STANDBY"
#define ccdCMD_START       "START"
#define ccdCMD_STARTAG     "STARTAG"
#define ccdCMD_STARTLP     "STARTLP"
#define ccdCMD_START_WIPE  "STARTWP"
#define ccdCMD_STARTTL     "STARTTL"
#define ccdCMD_STARTTM     "STARTTM"
#define ccdCMD_STATUS      "STATUS"
#define ccdCMD_STOP        "STOP"
#define ccdCMD_STOP_WAIT   "STPWAIT"
#define ccdCMD_STOP_WIPE   "STOPWP"
#define ccdCMD_STOPTL      "STOPTL"
#define ccdCMD_STOPTM      "STOPTM"
#define ccdCMD_VERS        "VERSION"
#define ccdCMD_WAIT        "WAIT"

/* commands implemented in WS simulation only (not supported by LCU yet ) */
#define ccdCMD_DISABLE     "DISABLE"
#define ccdCMD_ENABLE      "ENABLE"
#define ccdCMD_ENHANCE     "ENHANCE"
#define ccdCMD_PROCESS     "PROCESS"
#define ccdCMD_PXQUERY     "PXQUERY"
#define ccdCMD_SELFTST     "SELFTST"
#define ccdCMD_VERBOSE     "VERBOSE"

/* commands not yet implemented */
#define ccdCMD_CONFIG      "CONFIG"
#define ccdCMD_SIM         "SIM"
#define ccdCMD_STOPSIM     "STOPSIM"

/* parameters */
/* command ccdCMD_STATUS */
#define ccdPAR_STATUS_EXPOID   "expoId"          /* exposure ID parameter name  */
#define ccdPAR_STATUS_FUNCTION "function"        /* function parameter name     */

/* paremeters currently used for ccdPAR_STATUS_FUNCTION */
#define ccdALL                "all"
#define ccdGLOBAL             "global"
#define ccdKEY_OPMODE         "DET.MODE"         /* see also ccdOPMODE          */
#define ccdKEY_OPSTATE        "DET.STATE"        /* see also ccdSTATE           */
#define ccdKEY_SHTSTAT        "DET.SHUT"         /* see also ccdSHT_STATUS      */
#define ccdKEY_TMPSTAT        "DET.TEMP"         /* Temperature values          */
#define ccdKEY_TELSTAT        "DET.TELE"         /* Telemetry values            */

/* reply parameters for ccdCMD_STATUS */
#define ccdPAR_STATUS_EXPSTATUS  "expStatus"     /* exposure status par. name   */
				                 /* see also ccdEXP_xxx         */
#define ccdPAR_STATUS_FUNCSTATUS "funcStatus"    /* function status par. name   */

/* command ccdCMD_SETUP */
#define ccdPAR_SETUP_EXPOID   "expoId"           /* exposure ID parameter name  */
#define ccdPAR_SETUP_FILE     "file"             /* setup file parameter name   */
#define ccdPAR_SETUP_FUNCTION "function"         /* setup function name         */
#define ccdPAR_SETUP_CHECK    "check"            /* setup check required        */

/* command ccdCMD_START */
#define ccdPAR_START_AT       "at"               /* start at UTC                */

/* paremeters currently used for ccdPAR_SETUP_FUNCTION */
#define ccdEXP_TYPE           "DET.EXP.TYPE"     /* exposure type               */
				                 /* see also ccdEXPTYPE         */
#define ccdEXP_STEP_DEF       "DET.WIN1.NDIT"    /* Number of steps for MULTI   */

#define ccdTIME_DEF_IDEM      "DET.WIN1.ASUIT1" /* integration time as UIT1    */
#define ccdTIME_DEF_BASE      "DET.WIN1.UIT"
#define ccdTIME_DEFi          "DET.WIN1.UIT%i"   /* integration time            */
#define ccdTIME_DEF           "DET.WIN1.UIT1"    /* integration time single     */

#define ccdREPEAT_DEF         "DET.EXP.NREP"     /* exp. repetition count       */
				                 /* see also ccdREPEAT_FOREVER  */
#define ccdDO_WIPE            "DET.EXP.WIPETIM"  /* wipe at the begin of exp.   */
#define ccdTIME_REP           "DET.EXP.TIMEREP"  /* Interval between 2 exposures*/
#define ccdFILE_UNC           "DET.FRAM.FITSUNC" /* file for uncompr. image data*/
#define ccdDISK_SAVE          "DET.FRAM.FITSMTD" /* disk save method            */
#define ccdXFER_SYNC          "DET.FRAM.XFERSYN" /* image transfer synchronous  */
				                 /* see also ccdDISKSAVE        */
#define ccdSAMPLE             "DET.FRAM.SAMPLE"  /* transfer sampling rate      */
#define ccdDISPLAY            "DET.DISPLAY"      /* real-time display frame Id  */
				                 /* see also ccdNO_DISPLAY      */
#define ccdREAD_CLOCK         "DET.READ.CLKIND"  /* clock index                 */
#define ccdREAD_GAIN          "DET.OUT1.GAININD" /* readout gain                */
#define ccdREAD_FRAME_TYPE    "DET.FRAM.TYPE"    /* frame type                  */
				                 /* see also ccdFRAME_xxx       */
#define ccdREAD_SHIFT_TYPE    "DET.READ.SHIFTYP" /* line shift type for MULTI  */
#define ccdREAD_SHIFT_BASE    "DET.READ.SHIFT"   /* line shift                  */
#define ccdREAD_SHIFTi        "DET.READ.SHIFT%i" /* line shift step #i          */

#define ccdREAD_IMAGE_SIM     "DET.READ.SIMIMG"  /* image to load for ACE sim   */

/*===========*/
/* WINDOW #1 */
/*===========*/
#define ccdREAD_WIN1_BIN_X    "DET.WIN1.BINX"    /* readout x binning           */
#define ccdREAD_WIN1_BIN_Y    "DET.WIN1.BINY"    /* readout y binning           */
#define ccdREAD_WIN1_ENAB     "DET.WIN1.ST"      /* enabled flag                */
#define ccdREAD_WIN1_FIR_X    "DET.WIN1.STRX"    /* first x pixel               */
#define ccdREAD_WIN1_DIM_X    "DET.WIN1.NX"      /* x dimensions                */
#define ccdREAD_WIN1_FIR_Y    "DET.WIN1.STRY"    /* first y pixel               */
#define ccdREAD_WIN1_DIM_Y    "DET.WIN1.NY"      /* y dimensions                */
#define ccdPROC_WIN1_MINMAX   "DET.WIN1.MINMAX"  /* search image extrema        */
#define ccdPROC_WIN1_BIAS     "DET.WIN1.BIAS"    /* bias subtraction wanted     */
#define ccdPROC_WIN1_FLAT     "DET.WIN1.FLATF"   /* flat field division wanted  */
#define ccdPROC_WIN1_CENTROID "DET.WIN1.CENTROID"/* type of centroiding         */
				                 /* see also ccdCENTROID_xxx    */
#define ccdPROC_WIN1_REF_X    "DET.WIN1.REFX"    /* X ref. star for centroiding */
#define ccdPROC_WIN1_REF_Y    "DET.WIN1.REFY"    /* Y ref. star for centroiding */
#define ccdPROC_WIN1_USERFUNC "DET.WIN1.IPFUNC"  /* User Function name          */
#define ccdPROC_WIN1_USERBUFF "DET.WIN1.IPBUFF"  /* User Buffer name            */
#define ccdPROC_WIN1_THRMIN   "DET.WIN1.THRMIN"  /* threshold for min intensity */
#define ccdPROC_WIN1_THRMAX   "DET.WIN1.THRMAX"  /* threshold for max intensity */
#define ccdPROC_WIN1_BACKGND  "DET.WIN1.BACKGND" /* sky background              */
#define ccdPROC_WIN1_IPLLX    "DET.WIN1.IPLLX"   /* proc subWin offsets         */
#define ccdPROC_WIN1_IPLLY    "DET.WIN1.IPLLY"   /* proc subWin offsets         */
#define ccdPROC_WIN1_IPURX    "DET.WIN1.IPURX"   /* proc subWin offsets         */
#define ccdPROC_WIN1_IPURY    "DET.WIN1.IPURY"   /* proc subWin offsets         */

/*===========*/
/* WINDOW #2 */
/*===========*/
#define ccdREAD_WIN2_BIN_X    "DET.WIN2.BINX"    /* readout x binning           */
#define ccdREAD_WIN2_BIN_Y    "DET.WIN2.BINY"    /* readout y binning           */
#define ccdREAD_WIN2_ENAB     "DET.WIN2.ST"      /* enabled flag                */
#define ccdREAD_WIN2_FIR_X    "DET.WIN2.STRX"    /* first x pixel               */
#define ccdREAD_WIN2_DIM_X    "DET.WIN2.NX"      /* x dimensions                */
#define ccdREAD_WIN2_FIR_Y    "DET.WIN2.STRY"    /* first y pixel               */
#define ccdREAD_WIN2_DIM_Y    "DET.WIN2.NY"      /* y dimensions                */
#define ccdPROC_WIN2_MINMAX   "DET.WIN2.MINMAX"  /* search image extrema        */
#define ccdPROC_WIN2_BIAS     "DET.WIN2.BIAS"    /* bias subtraction wanted     */
#define ccdPROC_WIN2_FLAT     "DET.WIN2.FLATF"   /* flat field division wanted  */
#define ccdPROC_WIN2_CENTROID "DET.WIN2.CENTROID"/* type of centroiding         */
				                 /* see also ccdCENTROID_xxx    */
#define ccdPROC_WIN2_REF_X    "DET.WIN2.REFX"    /* X ref. star for centroiding */
#define ccdPROC_WIN2_REF_Y    "DET.WIN2.REFY"    /* Y ref. star for centroiding */
#define ccdPROC_WIN2_USERFUNC "DET.WIN2.IPFUNC"  /* User Function name          */
#define ccdPROC_WIN2_USERBUFF "DET.WIN2.IPBUFF"  /* User Buffer name            */
#define ccdPROC_WIN2_THRMIN   "DET.WIN2.THRMIN"  /* threshold for min intensity */
#define ccdPROC_WIN2_THRMAX   "DET.WIN2.THRMAX"  /* threshold for max intensity */
#define ccdPROC_WIN2_BACKGND  "DET.WIN2.BACKGND" /* sky background              */
#define ccdPROC_WIN2_IPLLX    "DET.WIN2.IPLLX"   /* proc subWin offsets         */
#define ccdPROC_WIN2_IPLLY    "DET.WIN2.IPLLY"   /* proc subWin offsets         */
#define ccdPROC_WIN2_IPURX    "DET.WIN2.IPURX"   /* proc subWin offsets         */
#define ccdPROC_WIN2_IPURY    "DET.WIN2.IPURY"   /* proc subWin offsets         */

/* paremeters foreseen but not used (the list is not finalized) */
#define ccdREADPIX            "DET.READ.NO"      /* pixel readout times (skip)  */
#define ccdMPPMODE            "DET.READ.MODE"    /* mpp mode                    */
#define ccdFILE_COM           "DET.FRAM.FITSCMP" /* file for compr. image data  */

/* command ccdCMD_START */
#define ccdPAR_START_EXPOID   "expoId"           /* exposure ID parameter name  */

/* command ccdCMD_VERS */
#define ccdPAR_VERS_CCDVERS   "ccdVersion"       /* version of CCD software     */

/* parameters for ccdCMD_STAND_ALONE */
#define ccdPAR_STANDAL_ON     "on"              /* stand-alone flag          */
#define ccdPAR_STANDAL_ARC    "archive"         /* archive images            */

/* 
 * exposure status (ccdPAR_STATUS_EXPSTATUS). Bit-field
 */
#define ccdEXP_NONE             0
#define ccdEXP_INACTIVE         1
#define ccdEXP_PENDING          2
#define ccdEXP_INTEGRATING      4
#define ccdEXP_PAUSED           8
#define ccdEXP_READING         16
#define ccdEXP_PROCESSING      32
#define ccdEXP_TRANSFERING     64
#define ccdEXP_COMPLETED      128
#define ccdEXP_FAILED         256
#define ccdEXP_ABORTED        512
#define ccdEXP_LOOP_FINITE   1024
#define ccdEXP_LOOP_INFINITE 2048

#define ccdEXP_DONE \
       (ccdEXP_COMPLETED | ccdEXP_FAILED | ccdEXP_ABORTED)
#define ccdEXP_RUNNING \
       (ccdEXP_PENDING | ccdEXP_INTEGRATING | ccdEXP_PAUSED | \
	ccdEXP_READING | ccdEXP_PROCESSING | ccdEXP_TRANSFERING )
#define ccdEXP_LOOP \
       (ccdEXP_LOOP_FINITE | ccdEXP_LOOP_INFINITE)

/*
 * Infinite loop (ccdREPEAT_DEF)
 */
#define ccdREPEAT_FOREVER 0

/*
 * Display types
 */
#define ccdNO_DISPLAY	-1

/*
 * Wait modes
 */
#define ccdWAIT_SINGLE 0
#define ccdWAIT_GLOBAL 1

#define ccdWAIT_SINGLE_STR "Single"
#define ccdWAIT_GLOBAL_STR "Global" 

/* ccdEXP_TYPE */
typedef enum
    {
    ccdEXP_NORMAL = 1,
    ccdEXP_DARK,
    ccdEXP_MULTI
    } ccdEXPTYPE;
#define ccdEXP_NORMAL_STR "Normal"
#define ccdEXP_DARK_STR   "Dark"
#define ccdEXP_MULTI_STR  "Multiple"   /* not implemented yet */

typedef char ccdCAMERANAME[ccdCAMERANAME_LEN + 1];   /* name of CCD camera  */

/* operational modes */
typedef enum                       /* possible operational modes           */
    {
    ccdNOT_AVAILABLE = 1,          /* CCD sw is not available              */
    ccdNORMAL,                     /* CCD sw and hw available              */
    ccdSIM_WS,                     /* LCU CCD sw is simulated at WS level  */
    ccdSIM_LCU,                    /* CCD hw is simulated at LCU level     */
    ccdSIM_ACE                     /* CCD hw is simulated at ACE level     */
    } ccdOPMODE;

/*
 * operational states
 */
typedef enum {
    ccdUNKNOWN   = ccsSTATE_UNK,     /* unknown                              */
    ccdOFF       = ccsSTATE_OFF,     /* terminating                          */
    ccdLOADED    = ccsSTATE_LOADED,  /* need to be initialised               */
    ccdSTANDBY   = ccsSTATE_STANDBY, /* stand-by                             */
    ccdONLINE    = ccsSTATE_ONLINE,  /* ready for exposures                  */
} ccdSTATE;

#define ccdOPERATING ccsSTATE_ONLINE

/*
 * shutter status definition
 */
typedef enum
    {
    ccdSHT_ERROR = -1,
    ccdSHT_CLOSED,
    ccdSHT_OPENED,
    ccdSHT_CLOSING,
    ccdSHT_OPENING
    } ccdSHT_STATUS;

/* Various ways of saving an image in disk file(s) */
typedef enum
    {
    ccdDISK_NONE = 0,
    ccdDISK_COMPRESS,               /* not implemented yet */
    ccdDISK_UNCOMPRESS,
    ccdDISK_BOTH                    /* not implemented yet */     
    } ccdDISKSAVE;

#define ccdDISK_NONE_STR       "None"
#define ccdDISK_COMPRESS_STR   "Compress"
#define ccdDISK_UNCOMPRESS_STR "Uncompress"
#define ccdDISK_BOTH_STR       "Both"

/*
 * Frame type (for storage in LCU memory, see setup param. ccdREAD_FRAME_TYPE)
 */
#define ccdFRAME_NORMAL "Normal"          /* normal frame: temporary storage */
#define ccdFRAME_BIAS   "Bias"    /* reference bias frame: permanent storage */
#define ccdFRAME_DARK   "Dark"              /* dark frame: permanent storage */
#define ccdFRAME_FLAT   "FF" /* reference flatfield frame: permanent storage */

typedef enum
    {
    ccdFRM_NORMAL = 0,
    ccdFRM_BIAS,
    ccdFRM_DARK,
    ccdFRM_FLAT
    } ccdFRMTYPE;

/*
 * Multi-Step Integration time
 */
#define ccdEXPTIME_IDEM ccsTRUE  /* use DET.WIN1.UIT1 for all steps */
#define ccdEXPTIME_LIST ccsFALSE /* use DET.WIN1.UITi as defined    */

/*
 * Line Shift type
 */
    /* Setup values for Line Shift type */
typedef enum 
    {
    ccdLINE_SHIFT_ALT = -1,      /* use DET.READ.SHIFT1 alt. pos & neg*/
                                 /* Alternate exposure mode           */
    ccdLINE_SHIFT_IDEM = 0,      /* use DET.READ.SHIFT1 for all steps */
                                 /* Focus & Sky Coherence modes       */
    ccdLINE_SHIFT_LIST = 1       /* use DET.READ.SHIFTi as defined    */
    } ccdLINESHIFT_TYPE;

#define ccdLINE_SHIFT_ALT_STR  "alternate"
#define ccdLINE_SHIFT_IDEM_STR "idem"
#define ccdLINE_SHIFT_LIST_STR "list"

/*
 * Centroiding type (see setup param. ccdIP_CENTROID)
 */
typedef enum
    {
    ccdCENTROID_NONE = 0,
    ccdCENTROID_STANDARD,
    ccdCENTROID_THRESHOLD
    } ccdCENTYPE;

#define ccdCEN_NONE_STR      "none"       /* no centroiding calculation      */
#define ccdCEN_STANDARD_STR  "standard"   /* standard centroiding calc.      */
#define ccdCEN_THRESHOLD_STR "threshold"  /* standard centroiding calc.      */

    /* 
     * Maximum Threshold :
     * ===================
     * - Ignored in the current implementation
     */

    /* 
     * Minimum Threshold :
     * ===================
     * => value >= 0 : use this value
     * => value within the integer range [-1;-9] : 
     *    the threshold min is preset to N StdDev of Intensity distribution 
     *    over this window
     * => value within the integer range [-11;-19] : 
     *    the threshold min is preset to N StdDev of Intensity distribution 
     *    over the first window
     *    This setting is only valid for the second window
     */
    /* The following macros are standard settings */
#define ccdTHRMIN_3SIGMA_WINDOW_SELF  -3
#define ccdTHRMIN_3SIGMA_WINDOW_PREV -13
#define ccdTHRMIN_5SIGMA_WINDOW_SELF  -5
#define ccdTHRMIN_5SIGMA_WINDOW_PREV -15

    /* 
     * BackGround :
     * ============
     * => value within the >= 0 : use this value
     * => value set to -1 : 
     *    the BackGround is preset to the Average Intensity 
     *    over this window
     * => value set to -11 : 
     *    the BackGround is preset to the Average Intensity 
     *    over the first window
     *   This setting is only valid for the second window
     */
    /* The following macros are standard settings */
#define ccdBCKGND_FLUX_WINDOW_SELF    -1
#define ccdBCKGND_FLUX_WINDOW_PREV   -11

/* real-time image processing user function definitions */
#define ccdIP_NO_USER_FUN   "None"        /* no user function to be called   */
#define ccdIP_NO_USER_BUF   "None"        /* user function has no buffer     */

/*
 * IMAGE PROCESSING DATA STRUCTURES
 * --------------------------------
 * The coordinates are, unless specified, absolute to the lower left corner
 * of the detector. This pixel has the coordinates (1,1).
 */
typedef struct 
    {            
    /*
     * The exposure structure provides general information on the
     * exposure.
     */
    ccdEXPTYPE   expType;
    vltINT32     stepsDef;            /* number of steps (MULTI)             */
    ccsTIMEVAL   wipeStart;           /* actual wipe start time (UTC)        */
    vltDOUBLE    wipeTime;            /* actual wipe duration                */
    ccsTIMEVAL   expStart;            /* actual integration start time (UTC) */
    vltDOUBLE    expTime;             /* Effective exposure time             */
    vltDOUBLE    expTimeMulti[ccdMAXINTEGR];
    vltINT32     shiftLine[ccdMAXINTEGR];
    ccsTIMEVAL   shiftStart[ccdMAXINTEGR]; /* Line shift start time (UTC)    */
    vltDOUBLE    shiftTime;           /* Effective line shift time           */
    ccsTIMEVAL   readStart;           /* actual readout start time (UTC)     */
    vltDOUBLE    readTime;            /* actual readout duration             */
    ccsTIMEVAL   procStart;           /* actual image proc. start time (UTC) */
    } ccdIPEXPOSURE;

/* data types */
typedef enum
    {
    ccdIMAGE_CHAR = 0,               /* 8 bit unsigned                     */  
    ccdIMAGE_INT,                    /* 16 bit signed                      */
    ccdIMAGE_FLOAT                   /* 32 bit floating point              */
    } ccdIMAGE_REP;

typedef struct 
    {            
    /*
     * The window structure describes the current window passed to the
     * IP function. Besides the information on the window position on
     * the detector and its dimensions, available are also the window
     * index (starting from 0 as first window defined), the pixel 
     * representation (integer 8 or 16 bits, or 32-bit floating point)
     * which is required for proper casting of the data pointer.
     */
    vltINT32     winNum;              /* Window index                        */
    ccdIMAGE_REP pixelRep;            /* Pixel value representation          */
    vltINT32     xPos,yPos;           /* Location of lower left corner       */
    vltINT32     xDim,yDim;           /* Window dimension                    */
    void        *data;                /* Pointer to image data               */
    } ccdIPWINDOW;

typedef struct 
    {            
    /*
     * The setup structure contains the setup parameters used by the 
     * image processing functions.
     */
    vltINT32     llx,lly;             /* Offset to Lower Left  corner        */
    vltINT32     urx,ury;             /* Offset to Upper Right corner        */
    vltDOUBLE    xRef,yRef;           /* Reference point                     */
    vltDOUBLE    minThr,maxThr;       /* Thresholds                          */
    vltDOUBLE    bckGnd;              /* Estimation of Sky Back Ground       */
    } ccdIPSETUP;

typedef struct 
    {  
    /*
     * The result structure is filled in according to the setup
     * The first part (STATISTICS) is automatically updated 
     * whenever the centroid calculation or a user-function is
     * invoked. 
     * Since the two advanced image processing functions 
     * ccdipAG(3) and ccdipIQE(3) have the same interface (ccdIPUSERFUNC),
     * it is possible to design a more complex function complying the
     * interface and invoking these functions in the processing sequence.
     * However, the designer has to considere the computation times which
     * are involved (for 20x20 window) :
     *       ccdipAG  : <20ms
     *       ccdipIQE : 800ms
     */
	/* STATISTICS : ccdipStatistic() */
    vltDOUBLE    max;                 /* First Maximum Intensity             */
    vltDOUBLE    xMax,yMax;           /* Location                            */
    vltDOUBLE    min;                 /* First Minimum Intensity             */
    vltDOUBLE    xMin,yMin;           /* Location                            */
    vltDOUBLE    flux;                /* Raw Average ADC counts / pixel      */
    vltDOUBLE    sigma;               /* Standard-Deviation on the flux      */
    vltINT32     numSpot;             /* Number of spots in image            */
    vltINT32     numPix;              /* Number of valid pixels in image     */
	/* CENTROID : ccdipCentroid() or ccdipAG() or ccdipWCE() */
    vltDOUBLE    cen;                 /* Centroid Intensity (closest pixel)  */
    vltDOUBLE    xCen,yCen;           /* Location                            */
    vltDOUBLE    xErr,yErr;           /* Error vector (relative to Ref Point)*/
    vltDOUBLE    SNR;                 /* Signal-to-Noise Ratio               */
    vltDOUBLE    xFWHM,yFWHM;         /* FWHM estimation                     */
	/* ANALYSIS : ccdipIQE() */
    vltDOUBLE    xFWHM_SD,yFWHM_SD;   /* FWHM Std Dev                        */
    vltDOUBLE    ObjAngle,ObjAngle_SD;/* Object position angle  + Std Dev    */
	/* BACKGROUND : ccdipBackground() */
    vltDOUBLE    BckGnd,BckGnd_SD;    /* Back Ground estimation + Std Dev    */
    } ccdIPRESULT;

typedef struct 
    {    
    ccdIPEXPOSURE exposure;
    ccdIPWINDOW   window;
    ccdIPSETUP    setup[ccdMAXWINDOW];
    ccdIPRESULT   result[ccdMAXWINDOW];
    } ccdIPIMAGE;

/* Function prototype for LCU real-time image processing user function */
typedef ccsCOMPL_STAT (* ccdIPUSERFUNC)(ccdIPIMAGE *,void *,ccsERROR *);

/* info about chip configuration */
typedef struct 
    {                     
    vltLOGICAL available;            /* available                          */
    vltINT32   xLocation;            /* x location in mosaic               */
    vltINT32   yLocation;            /* y location in mosaic               */
    } ccdCONFCHIP;

/* info about chip configuration */
typedef struct 
    {
    vltBYTES16 description;          /* description of output              */
    vltLOGICAL available;            /* available                          */
    vltINT32   chipIndex;            /* index of chip it belongs to        */
    vltINT32   xLocation;            /* x location in mosaic               */
    vltINT32   yLocation;            /* y location in mosaic               */
    vltINT32   leftToRight;          /* horizontal readout direction       */
    vltINT32   downToUp;             /* vertical readout direction         */
    vltINT32   xPrescan;             /* prescan pixels along x             */
    vltINT32   yPrescan;             /* prescan pixels along y             */
    vltINT32   xPix;                 /* active pixels along x              */
    vltINT32   yPix;                 /* active pixels along y              */
    vltINT32   xOverscan;            /* overscan pixels along x            */
    vltINT32   yOverscan;            /* overscan pixels along y            */
    } ccdCONFOUTPUT;

/* info about clock pattern */
typedef struct 
    {                     
    vltBYTES32 description;          /* description of clock pattern used  */
    vltINT32   firstOutput;          /* output generating first pixel value*/
    vltINT32   gainIndex;            /* default gain index                 */
    vltINT32   nWindows;             /* maximum number of windows supported*/
    vltINT32   adcTime;              /* usec needed to digitize one pixel  */
    vltINT32   lineShiftMode;        /* supported line shift mode          */
    vltLOGICAL outEnab[ccdMAXOUTPUT];/* outputs enabled                    */
    vltDOUBLE  elAdu[ccdMAXOUTPUT];
    } ccdCONFCLOCK;

/* info about camera shutter          */
typedef struct 
    {
    vltLOGICAL available;            /* available                          */
    } ccdCONFSHUTTER;

/* info about camera configuration    */
typedef struct 
    {                     
    ccdCAMERANAME name;              /* camera name                        */
    ccsENVNAME    envName;           /* environment name                   */
    } ccdCAMERA;

/* info about camera configuration  */
typedef struct 
    {                       
    vltLOGICAL     frameTransfer;       /* frame transfer CCD               */
    vltLOGICAL     skipper;             /* skipper CCD                      */
    vltLOGICAL     mpp;                 /* mpp mode supported               */
    vltLOGICAL     dual;                /* dual chip                        */
    vltINT32       xPixels;             /* Number of columns                */
    vltINT32       yPixels;             /* Number of rows                   */
    vltINT32       bitsPixel;           /* ADC resolution                   */
    vltINT32       numOutputs;          /* Number of outputs                */
    ccdCONFCHIP    chip[ccdMAXCHIPS];   /* single chip description          */
    ccdCONFOUTPUT  output[ccdMAXOUTPUT];/* single output description        */
    ccdCONFCLOCK   clock[ccdMAXCLOCKS]; /* single clock pattern information */
    ccdCONFSHUTTER shutter;             /* shutter information              */
    } ccdCONFIG;

/* window readout definition */
typedef struct
    { 
    vltLOGICAL enabled;             /* this window is enabled for readout  */
    vltLOGICAL idem;                /* same binning as Window #1           */
    vltINT32   xBinning;            /* horizontal binning factor           */
    vltINT32   yBinning;            /* vertical binning factor             */
    vltINT32   xFirst;              /* first column                        */
    vltINT32   xDim;                /* number of columns                   */
    vltINT32   yFirst;              /* first row                           */
    vltINT32   yDim;                /* number of rows                      */
    } ccdWINDOW;

/* window process definition */
typedef struct
    { 
    vltLOGICAL   minMax;            /* do MinMax                         */
    vltLOGICAL   bias;              /* do Bias correction                */
    vltLOGICAL   dark;              /* do Dark correction                */
    vltLOGICAL   flat;              /* do Flat Field correction          */
    vltINT32     averageN;          /* do average over N frames          */
    vltINT32     centroiding;       /* do Centroid calculation (mode)    */
    vltDOUBLE    centrRefX;         /* Reference Point X                 */
    vltDOUBLE    centrRefY;         /* Reference Point Y                 */
    vltDOUBLE    centrThreshMin;    /* Centroid Threshold Min            */
    vltDOUBLE    centrThreshMax;    /* Centroid Threshold Max            */
    vltDOUBLE    backGround;        /* Sky BackGround                    */
    vltBYTES32   userFuncName;      /* User-Defined function name        */
    vltBYTES32   userBuffName;      /* User-Defined buffer address       */
    vltINT32     procLLX,procLLY;   /* Lower Left & Upper Right corners  */
    vltINT32     procURX,procURY;   /* of Processing window (offset from */
	                            /* physical window)                  */
    } ccdPROCESS;

/* exposure setup parameters */
typedef struct 
    {                 
    ccdEXPTYPE   expType;        /* exposure type                           */
    vltINT32     stepsDef;       /* number of steps (MULTI)                 */
    vltINT32     repeatDef;      /* repetition number (0 is forever)        */
    vltINT32     wipeTime;       /* wipe wanted (>= 0) or not (<0)          */
    ccdFRMTYPE   frameType;      /* frame type                              */
    vltLOGICAL   timeDefIdem;    /* exposure time UITi as UIT1 ?            */
    vltDOUBLE    timeDef;        /* exposure time (step 1)                  */
    vltDOUBLE    timeDefMulti[ccdMAXINTEGR];
                                 /* exposure time (steps 2+ for MULTI)      */
    vltDOUBLE    timeRepeat;     /* sec between repeated exposures          */
    vltLOGICAL   fitsInfoCollect;/* Collect info for FITS header            */
    vltLOGICAL   tempExp;        /* Store tmp. periodically during exposure */
    vltINT32     tempSecs;       /* sec between temperature checks          */
    vltLOGICAL   teleExp;        /* Store tel. periodically during exposure */
    vltINT32     teleSecs;       /* sec between telemetry checks            */
	/* IT  : */
    ccdDISKSAVE  diskSave;             /* disk save method                  */
    vltBYTES64   fileNameUnComp;       /* uncompressed FITS file name       */
    vltBYTES64   fileNameComp;         /* compressed FITS file name         */
    vltINT32     pixRepr;              /* # bit for representation          */
    vltINT32     bitShift;             /* # bit shift for RTD               */
    vltINT32     compression;          /* compression type                  */
    vltINT32     sampling;             /* 1 image transferred out of N read */
    vltLOGICAL   itSync;               /* image transfer synchronous        */
	/* RTD : */
    vltINT32     frameId;              /* real-time display frame Id        */
	/* IP  : */
    ccdPROCESS   process[ccdMAXWINDOW];/* single window setup               */ 
	/* RDT : */ 
    vltINT32     linesToShiftType;     /* how lines shall be shifted        */
    vltINT32     linesToShift;         /* # lines to be shifted step 1      */ 
    vltINT32     linesToShiftMulti[ccdMAXINTEGR];
	                               /* # lines to be shifted steps 2+    */ 
    vltINT32     readPixelN;           /* # times the same pixel is read    */ 
    vltLOGICAL   mpp;                  /* MPP mode flag                     */
    vltINT32     clockIndex;           /* readout clock to use              */
    vltINT32     gainIndex;            /* readout gain index                */
    vltBYTES64   fileNameSimu;         /* simulated image (ccdSIM_LCU)      */
    ccdWINDOW    window[ccdMAXWINDOW]; /*  single window setup              */
    } ccdSETUP;

/* results of setup check */
typedef struct 
    {                     
    vltLOGICAL changed;              /* some setup values have been changed*/
    vltLOGICAL doTransfer;           /* Image transfer required            */
    vltLOGICAL doProcess;            /* Image processing required          */
    vltDOUBLE  readTime;             /* estimated readout time (sec)       */
    vltDOUBLE  wipeTime;             /* estimated wiping  time (sec)       */
    } ccdSETUPRES;

/* 
 * Macros 
 */

/*
 * function prototypes
 */
/* Get information about a CCD camera configuration */
ccsCOMPL_STAT ccdGetConf    ( ccdCAMERA *camera, ccdCONFIG *config,
			      ccsERROR  *error);

/* Check a complete exposure setup */
ccsCOMPL_STAT ccdCheckSetup ( ccdCONFIG *config, ccdSETUP *setup,
                              ccdSETUPRES *results, ccsERROR *error);

/* Check the setup for a windowed readout */
ccsCOMPL_STAT ccdCheckSetupWindow ( ccdCONFIG *config, ccdSETUP *setup,
                                    ccdSETUPRES *results, ccsERROR *error); 

/* Get information about a CCD camera configuration */
void ccdGetCIName           ( ccdCAMERANAME camera, ccsPROCNAME procName );

/* Extract index of DB point describing an exposure from the exposure ID     */
vltINT32 ccdGetIndexFromId  ( vltINT32 expId );

#ifdef __cplusplus
}
#endif

#endif /* ! CCD_H */
