/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccdDbPublic.h,v 4.11 2005/06/22 10:58:08 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  31/08/94  created
* alongino  19/05/95  modified while rearranging database for dbLoader
* alongino  18/07/95  ccdMAXEXP added
* alongino  20/07/95  ccdMAXOUTCHIP added
* alongino  29/09/95  changed name from ccdDbDimensions.h to ccdDbPublic.h
*                     added macros for public part of the database
* pduhoux   30/01/96  added macro ccdDB_EXP_FILECOM
* alongino  21/02/96  added macros define ccdDB_IP_XCEN ccdDB_IP_YCEN 
*                     ccdDB_IP_CENVAL
* pduhoux   28/05/96  added macros ccdDB_STA_FAILURE_LCU/WS
* alongino  18/07/96  added macro ccdDB_STA_EXPID
* P.Duhoux  21/01/97  moved ccdDB_WCS_RA & DEC for WorldCoord attributes
*                     from ccdDatabase.h
* P.Duhoux  27/02/97  Added macros for images:process_2 attributes
*                     ccdDB_IP2_
* P.Duhoux  04/03/97  Renamed point <process> into <process:window_1>
* alongino  26/05/97  ccdDB_IP_USERBUFFER removed (SPR 970177)
*                     same for ccdDB_IP_OBJNUMBER and ccdDB_IP_OBJECTS
* P.Duhoux  02/06/97  added ccdDB_IP_FLUX & ccdDB_IP_NUMPIX 
* P.Duhoux  21/07/97  added ccdDB_IP_SNR
* P.Duhoux  12/08/97  added ccdDB_STA_INITDONE
* P.Duhoux  09/10/97  added ccdDB_STA_XXX1 for full path to exposure_1 attr.
*                     added ccdDB_IPxxx for extrema
*                     added ccdDB_STA_TELMESSAGE & ccdDB_STA_TMPMESSAGE
*/

/*************************************************************************
*  This file contains all definitions for CCD database public attributes
*************************************************************************/

#ifndef CCDDBPUBLIC_H
#define CCDDBPUBLIC_H

/* 
 * Definition of database vector and table dimensions. Keep consistant !
 * NOTE: This file is used by dbLoader as well. Nothing else than define
 *       instructions or comment lines are possible here.
 */
/* These constants are actually not used directly in the database definition
 * files, but the database itself supports configuration for these values
 * only. To increase capabilities, modifications in the class definition
 * files are needed as well, in particular ccdDETECTOR and ccdEXPOSURES
 */
#define ccdMAXCHIPS   1       /* maximum number of chips in one mosaic      */
#define ccdMAXWINDOW  2       /* maximum number of windows for one readout  */
#define ccdMAXCLOCKS  10      /* maximum number of clock patterns           */
#define ccdMAXEXP     2       /* maximum number of expositions              */

/* These constants are used directly in the database definition files */
#define ccdMAXLINKS   15      /* maximum number of transputer links to ACE  */
#define ccdMAXLENMSG  32      /* maximum length of message to/from ACE      */
#define ccdMAXGAIN    8       /* maximum number of possible gains           */
#define ccdMAXOUTCHIP 4       /* maximum number of outputs in one chip      */
#define ccdMAXOUTPUT  4       /* maximum number of outputs in mosaic        */
#define ccdMAXOBJECTS 100     /* maximum number of objects in image proc.   */
#define ccdMAXINTEGR  10      /* maximum number of integrations in one exp. */
#define ccdMAXIMAGES  100     /* maximum number of images in memory         */
#define ccdMAXTELSRC  100     /* maximum number of telemetry sources        */
#define ccdMAXTMPSRC  10      /* maximum number of temperature sensors      */

/* Public part of the CCD database */

/* operational mode */
#define ccdDB_CON_OPMODE      ".opMode"

/* system state */
#define ccdDB_STA_SYSTEM      ".opState"

/* system errors */
#define ccdDB_STA_FAILURE_WS  ".failureWs"
#define ccdDB_STA_FAILURE_LCU ".failureLcu"

/* default image file directory  */
#define ccdDB_CON_IMGPATH     "images.imageDirectory"

/* running exposures prefix  */
#define ccdDB_EXP_POINT       "exposures:exposure"
#define ccdDB_EXP_POINT1      "exposures:exposure_1"

/* exposure status - under ccdDB_EXP_POINT_<expIndex> */
#define ccdDB_STA_EXPOSURE    "expStatus"
#define ccdDB_STA_EXPOSURE1   "exposures:exposure_1.expStatus"

/* exposure ID - under ccdDB_EXP_POINT_<expIndex> */
#define ccdDB_STA_EXPID       "expId"
#define ccdDB_STA_EXPID1      "exposures:exposure_1.expId"

/* name of uncompressed FITS file */
#define ccdDB_EXP_FILEUNC     "transfer.fileNameUnComp"
#define ccdDB_EXP_FILEUNC1    "exposures:exposure_1:transfer.fileNameUnComp"

/* Shared Memory Id associated to the Frame (with rtdServer) */
#define ccdDB_EXP_SHMID       "display.shmId"
#define ccdDB_EXP_SHMID1      "exposures:exposure_1:display.shmId"

/* Extrema */
#define ccdDB_IP_XMIN         "images:process:window_1.ipXMin"
#define ccdDB_IP1_XMIN        ccdDB_IP_XMIN
#define ccdDB_IP2_XMIN        "images:process:window_2.ipXMin"

#define ccdDB_IP_YMIN         "images:process:window_1.ipYMin"
#define ccdDB_IP1_YMIN        ccdDB_IP_YMIN
#define ccdDB_IP2_YMIN        "images:process:window_2.ipYMin"

#define ccdDB_IP_MINVAL       "images:process:window_1.ipMinVal"
#define ccdDB_IP1_MINVAL      ccdDB_IP_MINVAL
#define ccdDB_IP2_MINVAL      "images:process:window_2.ipMinVal"

#define ccdDB_IP_XMAX         "images:process:window_1.ipXMax"
#define ccdDB_IP1_XMAX        ccdDB_IP_XMAX
#define ccdDB_IP2_XMAX        "images:process:window_2.ipXMax"

#define ccdDB_IP_YMAX         "images:process:window_1.ipYMax"
#define ccdDB_IP1_YMAX        ccdDB_IP_YMAX
#define ccdDB_IP2_YMAX        "images:process:window_2.ipYMax"

#define ccdDB_IP_MAXVAL       "images:process:window_1.ipMaxVal"
#define ccdDB_IP1_MAXVAL      ccdDB_IP_MAXVAL
#define ccdDB_IP2_MAXVAL      "images:process:window_2.ipMaxVal"

/* Centroiding Error vector */
#define ccdDB_IP_XCEN         "images:process:window_1.ipXCen"
#define ccdDB_IP1_XCEN        ccdDB_IP_XCEN
#define ccdDB_IP2_XCEN        "images:process:window_2.ipXCen"

#define ccdDB_IP_YCEN         "images:process:window_1.ipYCen"
#define ccdDB_IP1_YCEN        ccdDB_IP_YCEN
#define ccdDB_IP2_YCEN        "images:process:window_2.ipYCen"

/* Intensity of image center, as computed from centroiding algorythm */
#define ccdDB_IP_CENVAL       "images:process:window_1.ipCenVal"
#define ccdDB_IP1_CENVAL      ccdDB_IP_CENVAL 
#define ccdDB_IP2_CENVAL      "images:process:window_2.ipCenVal"

/* Average flux / pixel */
#define ccdDB_IP_FLUX         "images:process:window_1.ipFlux"
#define ccdDB_IP1_FLUX        ccdDB_IP_FLUX 
#define ccdDB_IP2_FLUX        "images:process:window_2.ipFlux"

/* Standard Deviation of Pixel Intensity */
#define ccdDB_IP_STDDEV       "images:process:window_1.ipStdDev"
#define ccdDB_IP1_STDDEV      ccdDB_IP_STDDEV 
#define ccdDB_IP2_STDDEV      "images:process:window_2.ipStdDev"

/* Signal-to-Noise Ratio */
#define ccdDB_IP_SNR          "images:process:window_1.ipSNR"
#define ccdDB_IP1_SNR         ccdDB_IP_SNR    
#define ccdDB_IP2_SNR         "images:process:window_2.ipSNR"

/* number of pixels between the thresholds */
#define ccdDB_IP_NUMPIX       "images:process:window_1.ipNumPix"
#define ccdDB_IP1_NUMPIX      ccdDB_IP_NUMPIX 
#define ccdDB_IP2_NUMPIX      "images:process:window_2.ipNumPix"

/* Background estimation */
#define ccdDB_IP_BGND         "images:process:window_1.ipBGnd"
#define ccdDB_IP1_BGND        ccdDB_IP_BGND 
#define ccdDB_IP2_BGND        "images:process:window_2.ipBGnd"

/* StdDev on Background */
#define ccdDB_IP_BGND_SD      "images:process:window_1.ipBGnd_SD"
#define ccdDB_IP1_BGND_SD     ccdDB_IP_BGND_SD 
#define ccdDB_IP2_BGND_SD     "images:process:window_2.ipBGnd_SD"

/* Object FWHM X estimation */
#define ccdDB_IP_FWHM_X       "images:process:window_1.ipFWHM_X"
#define ccdDB_IP1_FWHM_X      ccdDB_IP_FWHM_X 
#define ccdDB_IP2_FWHM_X      "images:process:window_2.ipFWHM_X"

/* Object FWHM Y estimation */
#define ccdDB_IP_FWHM_Y       "images:process:window_1.ipFWHM_Y"
#define ccdDB_IP1_FWHM_Y      ccdDB_IP_FWHM_Y 
#define ccdDB_IP2_FWHM_Y      "images:process:window_2.ipFWHM_Y"

/* attributes for WCS handling */
#define ccdDB_WCS_RA          "wcs.ra"
#define ccdDB_WCS_DEC         "wcs.dec"

/* 
 * Attributes supposed to be used ONLY within GUI panels
 */

/* remaining exposure time for each step - under ccdDB_EXP_POINT_<expIndex> */
#define ccdDB_STA_TIMEREM     "timeRem"  
#define ccdDB_STA_TIMEREM1    "exposures:exposure_1.timeRem"

/* % of image transf. */
#define ccdDB_IT_STA_PERC     "images:transfer.percent"

/* last line transf. */
#define ccdDB_IT_STA_LINE     "images:transfer.last"

/* shutter status */
#define ccdDB_STA_SHTSTATUS   "shutter.status"

/* telemetry status message */
#define ccdDB_STA_TELMESSAGE  "telemetry.message"

/* temperature status message */
#define ccdDB_STA_TMPMESSAGE  "temperature.message"

#endif /* ! CCDDBPUBLIC_H */
