/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccdObj.h,v 4.11 2005/06/22 10:58:08 vltsccm Exp $"
*
* who          when      what
* -----------  --------  ----------------------------------------------
* A.Longinotti 12/11/96  created
* A.Longinotti 28/04/97  removed pco.h. Problems on Solaris
*/

/*************************************************************************
*  This file contains all definitions used by the library ccdObj
*************************************************************************/

#ifndef CCDOBJ_H
#define CCDOBJ_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCS & LCC header files
 */
#include "CCS.h"

/*
 * CCD header files
 */

/*
 * Constants
 */

/*
 * Types definitions
 */
typedef struct                 /* parameters needed to find objects        */
    {                     
    vltUINT8 procId;           /* Midas unit ID (between 0 and 99)         */
    vltLOGICAL threshAbs;      /* meaning of threshVal                     */
    vltINT32 threshVal;        /* intensity threshold                      */
    } ccdOBJMETHOD;

typedef struct                 /* object information                       */
    {                     
    vltFLOAT xPos;             /* position along x axis (pixel coord.)     */
    vltFLOAT yPos;             /* position along y axis (pixel coord.)     */
    vltFLOAT intensity;        /* intensity                                */
    } ccdOBJECT;

/*
 * function prototypes
 */
/* Get all objects detected in an image */
ccsCOMPL_STAT ccdObjAll(
    char *file,               /* name of FITS file containing image    */
    const ccdOBJMETHOD *method, /* parameters needed for the computation */
    vltINT32  *objectsN,      /* number of objects found               */
    ccdOBJECT **objects,      /* info about objects found              */
    ccsERROR *error);         /* error structure                       */

/* Get the brightest object in an image */
ccsCOMPL_STAT ccdObjBright(
    char *file,               /* name of FITS file containing image    */
    const ccdOBJMETHOD *method, /* parameters needed for the computation */
    ccdOBJECT *object,        /* info about object found               */
    ccsERROR *error);         /* error structure                       */

/* Get the closest object to a reference point in an image */
ccsCOMPL_STAT ccdObjClose(
    char *file,               /* name of FITS file containing image    */
    const ccdOBJMETHOD *method, /* parameters needed for the computation */
    ccdOBJECT *reference,     /* reference position                    */
    ccdOBJECT *object,        /* info about object found               */
    ccsERROR *error);         /* error structure                       */

/* Display the image analysed with all objects detected */
ccsCOMPL_STAT ccdObjDisplay(
    char *file,               /* name of FITS file containing image    */
    vltUINT8 procId,          /* Midas unit ID                         */
    ccsERROR *error);         /* error structure                       */

#ifdef __cplusplus
}
#endif

#endif /* ! CCDOBJ_H */
