/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccdWsInternal.h,v 4.11 2005/06/22 10:58:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  12/08/94  created
* alongino  24/07/95  commands moved to ccd.h
*                     removed IMREADY. Obsolete
* alongino  14/08/95  added definition of ccdPARAM
* alongino  17/08/95  changed structure of ccdENVVAR to include ccdCAMERA
* alongino  29/01/96  ccdPARAM replaced with cmdPARAM_LIST
* alongino  15/11/96  added ccdMidasCmd and ccdObjGetMidasProc prototypes
* pduhoux   31/01/97  added param <revision> to ccdLog functions
* pduhoux   31/01/97  fixed DB/dataType mismatch in ccdFITSHIERARCH_WS
* pduhoux   01/07/97  <id> set to vltBYTES32 in ccdFITSHIERARCH_WS to match DIC
*/

/*************************************************************************
*  This file contains the definitions for the WS part of the CCD sw.
*  Common file Used by all WS CCD modules
*************************************************************************/

#ifndef CCDWSINTERNAL_H
#define CCDWSINTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * System header files
 */
#include <stdlib.h>
#include <unistd.h>

/*
 * CCD include files
 */
#include "ccdInternal.h"               /* common definitions for WS and LCU */

 /*
 * Constants
 */
#define ccdTIMEOUT_DEFAULT 25000      /* default timeout for replies 25 sec  */
                                      /* AL941214 10 sec not enough for wipe */
#define ccdSETUP_VERS_PAR "VERSION"   /* keyword in setup file for version   */
#define ccdMAX_LINE        80         /* maximum size of a setup line        */
#define ccdMAX_PARAM       10         /* maximum number of command parameters*/

#define ccdMAX_UTC_DELTA   5.0        /* Max UTC difference WS vs LCU        */

/* commands */
/* command ccdCMD_VERS */
#define ccdPAR_START_MODE   "imageSave"    /* image save parameter name     */

/*
 * database paths for common attributes 
 */

/*
 * New types definitions
 */
/*
 * operational states of CCD WS part
 */
typedef enum {                     /* possible states of CCD DCS WS part   */
    ccdWsOFF         = ccdOFF,         /* terminating                      */
    ccdWsLOADED      = ccdLOADED,      /* need to be initialised           */
    ccdWsSTANDBY     = ccdSTANDBY,     /* stand-by                         */
    ccdWsONLINE      = ccdONLINE       /* ready for commands               */
} ccdWsSTATE;

/*
 * FITS for CCD WS part
 */
typedef struct {                     /* basic FITS info                    */
    ccdFITSBASIC    lcu;             /* basic info from LCU                */
    vltBYTES16      dateFile;        /* date the file has been written     */
} ccdFITSBASIC_WS;

typedef struct {                     /* cryostat FITS info                 */
    vltBYTES16      description;     /* type                               */
    vltUINT8        esoNumber;       /* eso sequencial number              */
} ccdFITSCRYO_WS;

typedef struct {                     /* FITS info on camera shutter        */
    vltBYTES16      type;            /* shutter type                       */
    vltBYTES16      description;     /* unique identifier                  */
} ccdFITSSHUT_WS;

typedef struct {                     /* FITS info on tempepature sources   */
    vltBYTES16      id;              /* id as from manufacturer            */
    vltBYTES16      description;     /* unique identifier                  */
} ccdFITSTEMP_WS;

typedef struct {                     /* FITS info on telemetry sensors     */
    vltBYTES16      id;              /* id as from manufacturer            */
    vltBYTES16      description;     /* unique identifier                  */
} ccdFITSTELE_WS;

typedef struct {                     /* FITS info on chips in the mosaic   */
    vltBYTES16      id;              /* chip description                   */
    vltBYTES16      description;     /* chip identifier                    */
    vltBYTES16      date;            /* chip installation date             */
    vltDOUBLE       xGap;            /* right gap to next chip [um]        */
    vltDOUBLE       yGap;            /* top   gap to next chip [um]        */
} ccdFITSCHIP_WS;

typedef struct {                     /* FITS info on outputs               */
    vltBYTES16      id;              /* id as from manufacturer            */
    vltBYTES16      description;     /* name of the output                 */
} ccdFITSOUTP_WS;

typedef struct {                     /* FITS info on chip mosaic           */
    vltBYTES16      description;     /* chip type                          */
} ccdFITSMOSAIC_WS;

typedef struct {                     /* hierarchical FITS info             */
    ccdFITSHIERARCH    lcu;                 /* info from LCU               */
    vltBYTES32         id;                  /* software version identifier */
    vltBYTES32         name;                /* name of detector system     */
    vltBYTES16         date;                /* detector commissioning date */
    vltBYTES32         installed;           /* software installation date  */
    vltBYTES32         did;                 /* dictionary identifier       */
    vltDOUBLE          tranTime;            /* transfer time [s]           */
    vltDOUBLE          xPixSize;            /* pixel size in X [um]        */
    vltDOUBLE          yPixSize;            /* pixel size in Y [um]        */
    vltINT32           dumdit;              /* # dummy rdt / exposure      */
    ccdFITSCRYO_WS     cryo;                /* cryostat info               */
    ccdFITSSHUT_WS     shut;                /* shutter  info               */
    ccdFITSTEMP_WS     temp[ccdMAXTMPSRC];  /* temp. sensor info           */
    ccdFITSTELE_WS     tele[ccdMAXTELSRC];  /* telemetry info              */
    ccdFITSCHIP_WS     chip[ccdMAXCHIPS];   /* chip info                   */
    ccdFITSOUTP_WS     outp[ccdMAXOUTPUT];  /* output info                 */
    ccdFITSMOSAIC_WS   mosaic;              /* mosaic info                 */
} ccdFITSHIERARCH_WS;

typedef struct {                     /* complete DCS FITS info             */
    ccdFITSBASIC_WS    basic;        /* basic info                         */
    ccdFITSHIERARCH_WS hierarch;     /* hierarchical info                  */
} ccdFITS_WS;

typedef struct {                              /* Environment information   */
    ccdCAMERA         camera;                 /* camera info               */  
    ccsPROCNUM        myProcId;               /* my process ID             */
    ccsPROCNAME       myProcName;             /* my process name           */
    ccsPROCNAME       destProcName;           /* destination process name  */
    ccsENVNAME        destEnv;                /* destination CCS enviroment*/
    dbSYMADDRESS      myRootPoint;            /* name of my DB root point  */
    dbSYMADDRESS      destRootPoint;          /* name of dest. DB root point */
    ccdCONFIG         config;                 /* configuration information */
    slxDICTIONARY     dictionary;             /* dictionary name           */
    logMASK           logMask;                /* FITS log mask             */
} ccdENVVAR;

typedef struct {                  /* information associated to any message */
    vltUINT8          type;                   /* type of message           */
    msgCMD            name;                   /* name                      */
    msgCMDID          id;                     /* ID                        */
    msgPROCESSID      orgId;                  /* originator of message     */
    ccsPROCNAME       proc;                   /* destination process name  */
    ccsENVNAME        env;                    /* destination CCS enviroment*/
    char              body[msgMAXLEN];        /* body of message           */
    msgLENGTH         length;                 /* length of message body    */
    cmdPARAM_LIST     param;                  /* parameters structure      */
    msgTIMEOUT        timeout;                /* timeout to wait for reply */
    msgCHECKFLAG      check;                  /* check flag                */
    vltLOGICAL        sendReply;              /* reply must be sent        */
    vltLOGICAL        lastReply;              /* flag for last reply       */
    ccsERROR          error;                  /* error returned in reply   */
} ccdMSG;

typedef struct ccdMsgLst *ccdMSGLSTPTR;       /* pointer to ccdMSGLST      */
/* 
   Note: better to define variables as (ccdMSGLST *) rather than ccdMSGLSTPTR
   because some debugger, e.g. gdb, shows the contents of the structure only
   in the former case 
*/

typedef struct ccdMsgLst{ /* list of msg whose execution must be completed */
    ccdMSG            msg;                    /* message info              */
    ccdMSGLSTPTR      orgCmd;                 /* entry of original command */
    ccdMSGLSTPTR      prev;                   /* previous entry in the list*/
    ccdMSGLSTPTR      next;                   /* next entry in the list    */
} ccdMSGLST;

/* 
 * Macros 
 */

/*
 * function prototypes
 */

/* log that a CCD program started */
void ccdLogStart( 
    IN  ccsPROCNAME  procName,              /* process name               */
    IN  const char  *revision);             /* RCS code revision string   */

/* log that a CCD program terminated */
void ccdLogTerminate( 
    IN  ccsPROCNAME  procName,              /* process name               */
    IN  const char  *revision);             /* RCS code revision string   */

/* log that a CCD program runs in a certain operational mode */
void ccdLogOpMode( 
    IN  const char  *opModeDescr);        /* operational mode description */

/* add error to stack, print error info on stdout and close stack */
void ccdPrintErr( 
    IN    ccsPROCNAME   procName,            /* process name              */
    IN    char          *reason,             /* reason for error          */
    IN    ccsERROR      *error);             /* error structure           */

/* do startup actions for any WS CCD program */
ccsCOMPL_STAT ccdProcStart( 
    IN  int argc,                         /* number of runtime parameters */
    IN  char *argv[],                     /* runtime parameters           */
    IN  const ccsPROCNAME destProc,       /* name of commands destination */
    IN  const char        *revision,      /* RCS code revision string     */
    OUT ccdENVVAR *envInfo);              /* pointer to env. var. values  */

/* terminate CCD process */
void ccdProcTerminate( 
    IN    ccsPROCNAME   procName,            /* process name              */
    IN    const char   *revision,            /* RCS code revision string   */
    IN    vltLOGICAL    ccsRegistered,       /* registered already to CCS */
    IN    const char   *reason,              /* reason for error          */
    IN    ccsERROR     *error);              /* error structure           */

/* check operational mode */
ccsCOMPL_STAT ccdCheckOpMode( 
    IN  ccdOPMODE opMode,                    /* operational mode          */
    OUT ccsERROR  *error);                   /* error structure           */

/* receive and parse next message */
ccsCOMPL_STAT ccdRecvMsg( 
    INOUT ccdMSGLST         **msg,           /* pointer to message entry  */
    OUT   vltLOGICAL        *logErr,         /* log the error             */
    INOUT ccsERROR          *error);         /* error structure           */

/* Send command to other process and add expected reply to message list */
ccsCOMPL_STAT ccdSendCmd(
    INOUT ccdMSGLST    **msg,             /* entry to list                */
    IN    ccdMSG       *cmd,              /* command info                 */
    IN    vltLOGICAL   link,              /* link to associated command   */
    OUT   ccsERROR     *error);           /* error structure              */

/* send all pending replies in the pending messages list */
ccsCOMPL_STAT ccdSendReply( 
    INOUT ccdMSGLST    **msg,             /* pointer to message entry     */
    OUT   vltLOGICAL   *exit,             /* last reply to EXIT sent      */
    INOUT ccsERROR     *error);           /* error structure              */

/* add suffux to file name */
void ccdFileNameAddSuffix( 
    IN     char       *header,         /* header of original pathname */
    IN     char       *suffix,         /* suffix of original pathname */
    IN     int        index,           /* index to be added as suffix */
    OUT    char       *path);          /* new pathname                */

/* separate suffix from header in a file name */
void ccdFileNameSepSuffix( 
    IN     char       *path,           /* original pathname           */
    OUT    char       *header,         /* file name header            */
    OUT    char       *suffix);        /* file name suffix            */

/* get env. information needed by any CCD WS program */
ccsCOMPL_STAT ccdGetEnv( 
    INOUT ccdENVVAR    *envInfo,           /* pointer to env. var. values */
    OUT   ccdOPMODE    *opMode,            /* operational mode            */
    OUT   ccsERROR     *error);            /* error structure             */

/* get CCD sw operational mode */
ccsCOMPL_STAT ccdGetOpMode (
    OUT ccdOPMODE     *opMode,             /* operational mode            */
    OUT ccsERROR      *error);             /* error structure             */

/* convert operational mode from code to string */
char *ccdOpModeEnumToStr(
    IN ccdOPMODE      opMode);             /* operational mode            */

void ccdHandleBreak(             
    IN  int           sigNum);             /* signal Number               */

/* add new entry in the list of pending messages for any CCD WS program */
ccsCOMPL_STAT ccdMsgLstAdd(
    INOUT ccdMSGLST    **msg,              /* pointer to new entry in list */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a command entry in the list of pending messages */
ccsCOMPL_STAT ccdMsgLstCmdAdd(
    INOUT ccdMSGLST    **msg,              /* pointer to entry in list     */
    IN    msgCMD       *cmdName,           /* command name                 */
    IN    msgCMDID     cmdId,              /* command ID                   */
    IN    msgPROCESSID *orgId,             /* originator ID                */
    IN    const char   *body,              /* command body                 */
    IN    msgLENGTH    length,             /* command body length          */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a command entry in the list of pending messages */
ccsCOMPL_STAT ccdMsgLstRplyAdd(
    INOUT ccdMSGLST    **msg,              /* entry to list                */
    IN    vltLOGICAL   link,               /* link to associated command   */
    IN    ccdMSG       *cmd,               /* command info                 */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a reply entry in the list of pending messages */
ccsCOMPL_STAT ccdMsgLstRplyFill(
    IN    ccdMSGLST    *rply,              /* reply entry in list          */
    IN    vltUINT8     type,               /* reply type                   */
    IN    const char   *command,           /* associated command           */
    IN    msgPROCESSID *orgId,             /* reply originator ID          */
    IN    const char   *body,              /* pointer to reply body        */
    IN    msgLENGTH    length,             /* command length               */
    IN    vltLOGICAL   last,               /* last reply                   */
    IN    ccsERROR     *error);            /* reply error                  */

/* delete one entry in the list of pending messages for CCD WS program*/
ccsCOMPL_STAT ccdMsgLstDel(
    INOUT ccdMSGLST    **msg,              /* pointer to next entry in list*/
    OUT   ccsERROR     *error);            /* error structure              */

/* query list of pending messages for CCD WS program */
ccsCOMPL_STAT ccdMsgLstSearch(
    INOUT ccdMSGLST    **msg,              /* pointer to new entry in list */
    IN    vltUINT8     type,               /* message type                 */
    IN    msgCMD       *cmdName,           /* command name                 */
    IN    msgCMDID     cmdId,              /* command ID                   */
    IN    msgPROCESSID *orgId,             /* originator ID                */
    OUT   vltLOGICAL   *logErr,            /* if TRUE, log error           */
    OUT   ccsERROR     *error);            /* error structure              */

/* add reply information for pending command */
void ccdMsgLstRplyBuild(
    IN    ccdMSGLST    *cmd,               /* entry to command associated  */ 
    IN    const char    *reply,             /* reply body                   */
    IN    msgLENGTH    length,             /* reply length                 */
    IN    vltLOGICAL   last,               /* last reply                   */
    IN    ccsERROR     *error);            /* error reply                  */

/* Delete all links to a command in the pending messages list */
void ccdMsgLstCmdLinkDel(
    IN    ccdMSGLST    *cmd);              /* entry to command associated  */ 

/* Check if there are links to a command in the pending messages list */
vltLOGICAL ccdMsgLstCmdLinkCheck(
    IN    ccdMSGLST    *cmd);              /* entry to command associated  */ 

/* Check if there are pending commands ready for reply */
vltLOGICAL ccdMsgLstCmdReady(
    INOUT    ccdMSGLST   **cmd);           /* pointer to entry to command  */

/* Return timeout value for next message receive and associated message */
void ccdMsgLstTimeout(
    OUT      msgTIMEOUT   *timeout,        /* timeout value               */
    INOUT    ccdMSGLST    **msg);          /* pointer to message entry    */

/* Initialise message structure */
void ccdMsgInit(
    INOUT ccdMSG       *msg);              /* message structure to init   */ 

/* Add one value to the message body */
void ccdMsgAddValue(
    INOUT ccdMSG       *msg,               /* message structure            */ 
    IN    cmdPARAM_TYPE type,              /* parameter type               */ 
    IN    const void   *value,             /* parameter value              */ 
    IN    vltLOGICAL   newPar);            /* first value of new parameter */ 

/* Get the status of an exposure */
ccsCOMPL_STAT ccdGetExpStatus( 
    INOUT vltINT32 *expId,          /* ID of exposure                        */
    OUT   vltINT32 *expStatus,      /* status of exposure                    */
    OUT   ccsERROR *error);         /* error structure                       */

/* Set the status of an exposure */
ccsCOMPL_STAT ccdSetExpStatus( 
    INOUT vltINT32 *expId,          /* ID of exposure                        */
    IN    vltINT32 *expStatus,      /* status of exposure                    */
    OUT   ccsERROR *error);         /* error structure                       */

/* Write in database new operational state for CCD WS sw */
ccsCOMPL_STAT ccdSetOpState(
    IN    ccdWsSTATE   opState,      /* operational state code           */
    OUT   ccsERROR     *error);      /* error structure                  */

/* Get ID of last exposure started */
ccsCOMPL_STAT ccdGetLastExpId(
    OUT vltINT32      *expId,              /* exposure ID                    */
    OUT ccsERROR      *error);             /* error structure                */

/* get name of this program */
void ccdGetProgName( 
    IN  const char   *argv0,                 /* runtime parameters        */
    OUT ccsPROCNAME   progName);             /* pointer program name      */

/* get name of CCD process */
void ccdGetProcName( 
    IN  const ccdCAMERANAME cameraName,      /* name of the camera used   */
    IN  const ccsPROCNAME   progName,        /* name of the program       */
    OUT       ccsPROCNAME   procName);       /* pointer to process name   */

/* Send a command to Midas and return its reply */
ccsCOMPL_STAT ccdMidasCmd(
    IN  char *midasId,            /* midas Unit ID string                  */
    IN  char *midasHost,          /* host where Midas is running           */
    IN  char *midasCmd,           /* Command for Midas                     */
    OUT msgLENGTH *rplyLen,       /* length of last reply from Midas       */
    OUT char **rplyBody,          /* pointer to Midas reply body           */
    OUT ccsERROR *error);         /* error structure                       */

/* Get path for midas procedure usable from within Midas and image directory */
ccsCOMPL_STAT ccdObjGetMidasProc(
    IN  char  *imgPath,           /* image file full path                 */
    IN  char  *procName,          /* name of Midas procedure              */
    OUT char  **imgDir,           /* image directory name                 */
    OUT char  **imgFile,          /* image file name                      */
    OUT char  *procPath,          /* path of procedure found              */
    OUT ccsERROR *error);         /* error structure                      */

/* execute command DEBUG */
ccsCOMPL_STAT ccdDebug( 
    IN  const ccdMSG   *inpCmd,     /* received command structure            */
    OUT ccdMSG         *destMsg,    /* destination message structure         */
    OUT ccsERROR       *error);      /* error structure                      */

#ifdef __cplusplus
}
#endif

#endif /* ! CCDWSINTERNAL_H */
