#ifndef CCS_MONITOR_H
#define CCS_MONITOR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccsMonitor.h,v 2.166 2005/10/14 09:12:47 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* tebert    12/11/02  created
* tebert    15/03/03  removed TREND data structures
* tebert    09/07/03  added semaphore.h include for SUN
* rrollfin  04/05/05  added quotes to extern "C"
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* 
 * The following piece of code alternates the linkage type to C for all 
 * functions declared within the braces, which is necessary to use the 
 * functions in C++-code.
 */

#ifdef SUN_COMP 
#include <semaphore.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*
 * command of ccsMonitor
 */
#define ccsMONITOR_EXIT_CMD       "MONEXIT"
#define ccsMONITOR_STOP_CMD       "MONSTOP"
#define ccsMONITOR_START_CMD      "MONSTRT"
#define ccsMONITOR_LEVEL_CMD      "MONLEVL"
#define ccsMONITOR_ENABLE_CMD     "MONENAB"
#define ccsMONITOR_DISABLE_CMD    "MONDISA"
#define ccsMONITOR_PINGST_CMD     "MONPIST"
#define ccsMONITOR_PINGSP_CMD     "MONPISP"
#define ccsMONITOR_PINGGT_CMD     "MONPIGT"
#define ccsMONITOR_LOADST_CMD     "MONLDST"
#define ccsMONITOR_LOADSP_CMD     "MONLDSP"
#define ccsMONITOR_LOADGT_CMD     "MONLDGT"
#define ccsMONITOR_MEMST_CMD      "MONMEST"
#define ccsMONITOR_MEMSP_CMD      "MONMESP"
#define ccsMONITOR_MEMGT_CMD      "MONMEGT"
#define ccsMONITOR_PERIOD_CMD     "MONLOGP"

#define ccsMONITOR_LOGGER        "ccsMonLogger"
#define ccsMONITOR_MAIN          "ccsMonitor"
#define ccsMONITOR_PING          "ccsPing"
#define ccsMONITOR_TOP           "ccsMonTop"

#define ccsMONITOR_QUEUE_WARNING 50
#define ccsMONITOR_QUEUE_SERIOUS 80
#define ccsMONITOR_MAX_CMDS      64

#define ccsMONITOR_DIR           "monitoring"

#define ccsMONITOR_LEVEL0        "L0"
#define ccsMONITOR_LEVEL1        "L1"
#define ccsMONITOR_LEVEL2        "L2"

#define ccsMONITOR_PERIOD1        60      /* logging period for message system  */
#define ccsMONITOR_PERIOD2        12      /* logging period for qsemu           */
#define ccsMONITOR_PERIOD3        14      /* default logging period for ccsPing */
#define ccsMONITOR_PERIOD4        16      /* default period for db system       */
#define ccsMONITOR_PERIOD5        18      /* default period for db system       */

#define ccsMONITOR_MAX_REMOTE     50

#define ccsMONITOR_TREND_CLASS    63999
#define ccsMONITOR_TREND_ATTR     "trendValues"

#define ccsMONITOR_MAX_CE         30
#define ccsMONITOR_MAX_USERS      512

typedef enum
    {
    ccsMONITOR_L0 = 0,
    ccsMONITOR_L1,
    ccsMONITOR_L2
    } ccsMONITOR_LEVEL;

typedef enum
    {
    ccsMONITOR_QUEUE_EXT = 0,
    ccsMONITOR_QUEUE_INT1,
    ccsMONITOR_QUEUE_INT2
    } ccsMONITOR_QUEUE_TYPE;

typedef enum
    {
    ccsMONITOR_DB_QUERY = 1,
    ccsMONITOR_DB_CONTROL,
    ccsMONITOR_DB_CONFIG
    } ccsMONITOR_DB_TYPE;

typedef enum
    {
    ccsMONITOR_NOWD = 1,
    ccsMONITOR_RESTART,
    ccsMONITOR_KILL,
    ccsMONITRO_KILLE,
    ccsMONITOR_KILLA,
    ccsMONITOR_CLEAN,
    ccsMONITOR_EXEC
    } ccsMONITOR_WD_TYPE;

typedef struct
    {
    ccsMONITOR_WD_TYPE type;
    vltUINT8           cmdIndex;
    vltINT32           value;
    } ccsMONITOR_WD;

typedef struct
    {
    vltLOGICAL valid;
    vltLOGICAL active;
    rtAin      ain;
    rtPlin     plin;
    vltUINT32  limVal;
    } ccsMONITOR_ALARM;

typedef struct
    {
    ccsENVNAME env;
    vltUINT32  sent;
    vltUINT32  bytesSent;
    vltUINT32  recv;
    vltUINT32  bytesRecv;
    vltUINT32  reconnect;
    vltUINT32  attempts;
    } ccsMONITOR_MSG_QSEMU;

typedef struct
    {
    vltINT32  pid;
    vltUINT32 memSize;
    vltUINT32 resSize;
    vltUINT32 startTime;
    vltFLOAT  cpuLoad;
    } ccsMONITOR_CCS_PROC;

typedef struct
    {
    vltLOGICAL          enabled;
    ccsMONITOR_CCS_PROC procs[ccsMAX_PROCS];
    } ccsMONITOR_CCS;

typedef struct
    {
    ccsMONITOR_LEVEL level;
    ccsMONITOR_ALARM alarm;
    ccsMONITOR_WD    queueLoad;
#ifndef SUN_COMP 
    int              sem;
#else
    sem_t            sem;
#endif
    vltUINT32        sentResponse;
    vltUINT32        sentRespond;
    vltUINT32        sentOther;
    vltUINT32        sentBytes;
    vltUINT32        sentTotal;
    vltUINT32        sentBytesTotal;
    vltUINT32        recvResponse;
    vltUINT32        recvRespond;
    vltUINT32        recvOther;
    vltUINT32        recvBytes;
    vltUINT32        recvTotal;
    vltUINT32        recvBytesTotal;
    vltUINT32        bytesQueue;
    vltUINT32        qOneBytes;
    vltUINT32        qOneResponse;
    vltUINT32        qOneRespond;
    vltUINT32        qOneOther;   
    vltUINT32        qTwoBytes;
    vltUINT32        qTwoResponse;
    vltUINT32        qTwoRespond;
    vltUINT32        qTwoOther;   
    } ccsMONITOR_MSG_PROC;

typedef struct
    {
    vltLOGICAL           enabled;
#ifndef SUN_COMP 
    int                  alarmSem;
#else
    sem_t                alarmSem;
#endif
    vltINT32             warningLim;
    vltINT32             seriousLim;
    ccsMONITOR_MSG_PROC  procs[ccsMAX_PROCS];
    ccsMONITOR_MSG_QSEMU remote[ccsMONITOR_MAX_REMOTE];
    } ccsMONITOR_MSG;

typedef struct
    {
    ccsMONITOR_LEVEL level;
    vltUINT32        attachments;
    vltUINT32        notifications;
    vltUINT32        attachTotal;
    vltUINT32        notifyTotal;
    } ccsMONITOR_EVT_PROC;

typedef struct
    {
    vltLOGICAL          enabled;
    vltUINT32           eventsAttached;
    vltUINT32           notifySent;
    ccsMONITOR_EVT_PROC procs[ccsMAX_PROCS];
    } ccsMONITOR_EVT;

typedef struct
    {
    ccsMONITOR_LEVEL level;
    vltUINT32        multiReadCnt;
    vltUINT32        multiReadTotal;
    vltUINT32        multiWriteCnt;
    vltUINT32        multiWriteTotal;
    vltUINT32        readCnt;
    vltUINT32        readTotal;
    vltUINT32        writeCnt;
    vltUINT32        writeTotal;
    vltUINT32        queryCnt;
    vltUINT32        queryTotal;
    vltUINT32        controlCnt;
    vltUINT32        controlTotal;
    vltUINT32        configCnt;
    vltUINT32        configTotal;
    vltUINT32        bytesRead;
    vltUINT32        bytesReadTotal;
    vltUINT32        bytesWritten;
    vltUINT32        bytesWrittenTotal;
    } ccsMONITOR_DB_PROC;

typedef struct
    {
    char       name[32];
    vltUINT32  callCnt;
    } ccsMONITOR_CE_FCTS;

typedef struct
    {
    vltLOGICAL         enabled;
    vltUINT32          ceCalls;
    vltUINT32          eventsAttach;
    vltUINT32          eventsDetach;
    vltUINT32          eventsSent;
    ccsMONITOR_DB_PROC procs[ccsMAX_PROCS];
    ccsMONITOR_CE_FCTS fcts[ccsMONITOR_MAX_CE];
    } ccsMONITOR_DB;

typedef struct
    {
    vltBYTES12  name;
    int         uid;
    } ccsMONITOR_USER;

typedef struct
    {
    vltLOGICAL      enabled;
    vltLOGICAL      initDone;
    ccsMONITOR_MSG  msg;
    ccsMONITOR_DB   db;
    ccsMONITOR_CCS  ccs;
    ccsMONITOR_EVT  evt;
    vltUINT32       startTime;
    vltUINT32       stopTime;
    vltUINT32       userCnt;
    ccsMONITOR_USER users[ccsMONITOR_MAX_USERS];
    vltBYTES128     cmds[ccsMONITOR_MAX_CMDS];
    } ccsMONITOR_DATA;

typedef struct
    {
    vltBYTES32  sysName;
    int         pid;
    } ccsLOGGER;


/*
 * prototypes
 */
void             ccsMonFirstInit         (ccsGLOBAL_SHM *ccsShm, msgPROCESSID *id);
void             ccsMonAttach            (ccsGLOBAL_SHM *ccsShm, msgPROCESSID *id);
ccsCOMPL_STAT    ccsMonGlobalInitialize  (ccsERROR *error);
ccsCOMPL_STAT    ccsMonEnable            ();
void             ccsMonDisable           ();
ccsCOMPL_STAT    ccsMonDir               (ccsERROR *error);
void             ccsMonLogSend           (rtMsgHeader *header, int sender, 
                                          int receiver, int numBytes);
void             ccsMonLogRecv           (rtMsgHeader *header, int receiver);
ccsMONITOR_DATA *ccsMonGetPtr            ();
void             ccsMonSetLevel          (vltBYTES32 subSystem, vltBYTES4 level, 
                                          vltUINT32 procNum, ccsERROR *error);
void             ccsMonInitMsgProc       (ccsMONITOR_MSG *msg, int index);
void             ccsMonInitDbProc        (ccsMONITOR_DB  *msg, int index);
void             ccsMonInitEvtProc       (ccsMONITOR_EVT *evt, int index);
void             ccsMonEnableSubsystem   (char *sys, ccsERROR *error);
void             ccsMonDisableSubsystem  (char *sys, ccsERROR *error);
char            *ccsMonGetPath           ();
int              ccsMonGetEnvIndex       (ccsENVNAME env);
void             ccsMonLogRemoteRecv     (rtMsgHeader *header, int index);
void             ccsMonLogRemoteSent     (rtMsgHeader *header, int index);
void             ccsMonLogSend2          (rtMsgHeader *header, int sender, int numBytes);
void             ccsMonLogNewConnect     (int index);
void             ccsMonLogConnectAttempts(int index);
int              ccsMonStartProcess      (char *cmd);
ccsCOMPL_STAT    ccsMonFillProcessData   (int index, vltINT32 pid, vltUINT32 memSize,
                                          vltUINT32 resSize, vltUINT32 startTime, 
                                          vltFLOAT  cpuLoad);
void             ccsMonLogCQC            (ccsMONITOR_DB_TYPE type, rtDbCq *data);
void             ccsMonLogDbRead         (rtDbReadWrite *rw);
void             ccsMonLogDbWrite        (rtDbReadWrite *rw);
void             ccsMonLogMultiRead      (vltUINT32 blkCnt, rtDbReadWrite *rw);
void             ccsMonLogMultiWrite     (vltUINT32 blkCnt, rtDbReadWrite *rw);
void             ccsMonInitMQDBM         ();
void             ccsMonSetCeName         (int index, char *name);
void             ccsMonIncrCeCnt         (int index);
void             ccsMonIncrCeRun         ();
void             ccsMonIncrEvtAttach     ();
void             ccsMonIncrEvtDetach     ();
void             ccsMonIncrEvtSent       (int cnt);
void             ccsMonInitEvtGlobal     ();
void             ccsMonLogAttach         (rtEventId *eventId);
void             ccsMonLogNotifySent     (rtEventHeader *eventHeader, msgPROCESSID *procId);
void             ccsMonAttachCnt         (int i,  msgPROCESSID *procId, rtDbAddress *addr,
                                          rtEventId *id, vltUINT8 trigger);
void             ccsMonInitEvtEventsAttached(vltUINT32 cnt);
void             ccsMonInitEvtProcAttach (ccsPROCNUM procnum, vltUINT32 procCnt);
void             ccsMonLogIntQueueAdd    (int size, rtMsgHeader *header, ccsMONITOR_QUEUE_TYPE type);
void             ccsMonLogIntQueueRem    (int size, rtMsgHeader *header, ccsMONITOR_QUEUE_TYPE type);
vltINT32         ccsMonStatFs();
void             ccsMonDisableDetailed   (int percentage, int lim);

#ifndef SUN_COMP 
void             ccsMonCleanResources    ();
#endif

#ifdef __cplusplus
}
#endif


#endif /*!CCS_MONITOR_H*/
