#ifndef CCSPTHREAD_H
#define CCSPTHREAD_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccsPthread.h,v 2.166 2005/10/14 09:12:47 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* bgilli    2005-10-14  corrected extern C in extern "C"
* vltuser1  2005-02-16  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* The following piece of code alternates the linkage type to C for all
functions declared within the braces, which is necessary to use the
functions in C++-code.
*/
#ifdef CCS_PTHREADS

#include "log.h"
#include "err.h"
#include "db.h"
#include "msg.h"
#include "msgPrivate.h"
#include "alrm.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
    {
    int  mtype;
    char mtext[msgMAXLEN];
    } ccsMSGSYS;

/*
 * management structure
 * NOTE: <msg> must be the last element in the structure
 */
typedef struct ccsMSG_LIST
    {
    struct ccsMSG_LIST *previous;
    struct ccsMSG_LIST *next;
    int                 size;
    int                 mtype;
    char                msg;
    } ccsMSG_LIST;

typedef struct
    {
    pthread_mutex_t     threadMutex;
    pthread_mutex_t     queueMutex;
    pthread_cond_t      queueCond;
    pthread_mutexattr_t threadAttr;
    pthread_t           threadId;
    int                 msgQueueId;
    msgCMDID            msgId;
    msgPROCESSID        procId;
    msgPROCESSID        ccsProcId;
    ccsMSGSYS           msgRecv;
    vltUINT32           obiStatus;
    ccsPROCNAME         procName;
    int                 origPid;
    char                pName[32];
    ccsPROCNAME         myName;
    ccsENVNAME          ccsName;
    int                 janitorStarted;
    int                 sema;
    ccsMSG_LIST        *ccsQueueStart; /* first element in list */
    ccsMSG_LIST        *ccsQueueEnd;   /* last element in list  */
    ccsMSG_LIST        *ccsQueueDel;   /* entry to be removed   */
    vltUINT32           numbLinks;
    dbLinkStatus        linkTable[dbMAX_CONNECTIONS];
    int                 msgFdMonitor;
    msgMSG_LIST        *msgQueueStart;   /* first element in list */
    msgMSG_LIST        *msgQueueEnd;     /* last element in list  */
    msgMSG_LIST        *msgQueueDel;     /* entry to be removed   */
    ccsPROCNAME         rdbServer;
    rtASConnection_T    connId;
    ccsLIST             alrmConLst;
    } ccsPTHREAD_MSG;

typedef struct
    {
    rtMsgHeader   *header;
    char          *offset;
    rtInt          number;
    int            index;
    rtAlarmInfo_T  alarmInfo;
    unsigned short connBits;
    } ccsPTHREADS_ALRM;

typedef struct
    {
    ccsOLD_ERROR       ccsErrno;
    char               name[ccsPROCNAME_LEN+2];
    char               message[rtMSGSIZE];
    rtProcessId        newProcessId;
    ccsPTHREAD_MSG    *msg;
    void              *argPtr;
    void             (*startRoutine)(void *);
    char              *VLTROOT;
    char              *INTROOT;
    char              *VLTDATA;
    rtQueryAlphaAttrs *queryResult;
    vltUINT8           ainStorage[dbMAX_ATTR_CNT];
    dbATTRIBUTE       *nameStorage;
    dbSYMADDRESS      *nameS;
    char               strPad[sizeof(vltBYTES256)];
    rtTimerId          timerId;
    rtEventId          eventId;
    char               cmdGBuffer[msgBODYMAXLEN];
    msgRECEIVEFILTER   dbFilter;
    ccsPTHREADS_ALRM   alrm;
    char               mess[rtMSGSIZE];
    } ccsPTHREAD_DATA;


ccsPTHREAD_DATA *ccsGetPptr();
ccsCOMPL_STAT    ccsMutexLock     (ccsPTHREAD_DATA *pPtr, pthread_mutex_t *mutex);
ccsCOMPL_STAT    ccsMutexUnlock   (ccsPTHREAD_DATA *pPtr, pthread_mutex_t *mutex);
ccsCOMPL_STAT    ccsGlobalLock    ();
ccsCOMPL_STAT    ccsGlobalUnlock  ();
ccsCOMPL_STAT    ccsStartMsgThread(ccsPTHREAD_DATA *pPtr);
rtMsgHeader     *ccsPthreadGetMsg (ccsPTHREAD_DATA *pPtr, int *type, int timeOut);
ccsCOMPL_STAT    ccsAddEntry      (ccsPTHREAD_MSG *msg, ccsMSGSYS *message, int size);
ccsMSG_LIST     *ccsFindEntry     (ccsPTHREAD_MSG *msg, int *mtype);
ccsCOMPL_STAT    ccsPthreadInit   (ccsPROCNAME procName, vltUINT32 obituary,
                                   void (*breakHandler)(int), void (*killHandler)(int));
ccsCOMPL_STAT    ccsPthreadCreate (pthread_t *threadId, pthread_attr_t *attr,
                                   void (*routine)(void*), void *arg, ccsERROR *error);
ccsCOMPL_STAT    ccsMPthreadCreate(pthread_t *threadId, const ccsPROCNAME procName,
                                   vltUINT32 obituary, pthread_attr_t *attr,
                                   void (*routine)(void*), void *arg, ccsERROR *error);

#ifdef __cplusplus
}
#endif

#else
/*
 * just do nothing
 */
#endif

#endif /*!CCSPTHREAD_H*/
