/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: chkc.h,v 4.97.1.1 2005/10/24 13:55:42 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  18/01/00  added parameter void to break and kill handlers to 
*                     avoid compiler warnings
* bgustafs  16/09/96  modified file layout
* imunoz    15/07/96  added prototype for chkcConvertReply
* imunoz    10/07/96  in chkcCOMMAND changed type of field replyParams
*                     to type chkcPARAM, allows to use the same functions
*                     on params and replyParams fields.
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
* bgustafs  07/03/96  modified for lcc 3.1
*
*/

#ifndef CHKC_H
#define CHKC_H
/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software             SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  2.1          31/01/95        P.Rochez        First version
*  3.1          24/11/95        P.Creten        Modified type chkcCIT_ENTRY
*                                               Modified type chkcCOMMAND
*------------------------------------------------------------------------------
*  SCCS Id:  chkc.h  3.1  05 Mar 1996
*----------------------------------------------------------------------------*/

/************************************************************************
 * COMMAND PARSING AND VERIFICATION - Interface File
 */

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * header files
 */
#include "ccs.h"


/*
 * constants
 */

/*
 * Command Interpreter blocked states
 */
#define chkcCIT_BLOCK_OFF         0x01   /* Block commands when OFF        */
#define chkcCIT_BLOCK_LOADED      0x02   /* Block commands when LOADED     */
#define chkcCIT_BLOCK_STANDBY     0x04   /* Block commands when STANDBY    */
#define chkcCIT_BLOCK_STANDALONE  0x08   /* Block commands when STANDALONE */
#define chkcCIT_BLOCK_ONLINE      0x10   /* Block commands when ONLINE     */


/*
 * data types
 */

typedef enum {                          /* Environment where tables     */
                                        /* are located                  */
    chkcINTROOT = 1,
    chkcVLTROOT
    } chkcENV_TYPE;


typedef enum                            /* Parameter types              */
    {
    chkcPARAM_TYPE_NOT_DEFINED = 1,
    chkcPARAM_TYPE_INTEGER,
    chkcPARAM_TYPE_REAL,
    chkcPARAM_TYPE_STRING,
    chkcPARAM_TYPE_LOGICAL
    } chkcPARAM_TYPE;


typedef enum                            /* Parameter formats            */
    {
    chkcPARAM_FORMAT_NOT_DEFINED = 1,
    chkcPARAM_FORMAT_ASCII,
    chkcPARAM_FORMAT_BINARY,
    chkcPARAM_FORMAT_STRUCTURED_BINARY
    } chkcPARAM_FORMAT;


typedef enum                            /* Range types                  */
    {
    chkcRANGE_TYPE_NOT_DEFINED = 1,
    chkcRANGE_TYPE_MIN_MAX,
    chkcRANGE_TYPE_ENUM
    } chkcRANGE_TYPE;


typedef enum                            /* Command types                */
    {
    chkcCOMMAND_TYPE_NOT_DEFINED = 1,
    chkcCOMMAND_TYPE_PUBLIC,
    chkcCOMMAND_TYPE_MAINTENANCE,
    chkcCOMMAND_TYPE_TEST
    } chkcCOMMAND_TYPE;


typedef struct                          /* Range description            */
    {
    chkcRANGE_TYPE      type;           /* Range type                   */
    vltUINT32           valueNb;        /* Number of values in range    */
    char                **values;       /* Array of values in ASCII     */
    } chkcRANGE;


typedef struct                          /* Parameter/Reply parameter    */
                                        /* common data                  */
    {
    char                *name;          /* Parameter name               */
    char                *unit;          /* Parameter unit               */
    chkcPARAM_TYPE      type;           /* Parameter type               */
    char                *defaultValue;  /* Default value in ASCII       */
    vltUINT16           repFactor;      /* Repetition factor            */
    } chkcCOMMON_PARAM;


typedef struct                          /* Parameter description        */
    {
    chkcCOMMON_PARAM    common;         /* Parameter common data        */
    vltUINT16           maxRep;         /* Maximum number of repetitions*/
    chkcRANGE           range;          /* Parameter range description  */
    vltLOGICAL          optional;       /* Is parameter optional ?      */
    } chkcPARAM;


typedef chkcCOMMON_PARAM  chkcREPLY_PARAM;
                                        /* Reply parameter description  */


typedef struct                          /* Command description          */
    {
    char                *name;          /* Command name                 */
    chkcCOMMAND_TYPE    type;           /* Command type                 */
    vltINT32            timeOut;        /* Timeout in seconds,          */
                                        /* -1 stands for no timeout     */
    vltUINT32           synonymNb;      /* Number of synonyms           */
    char                **synonyms;     /* Array of synonyms            */
    chkcPARAM_FORMAT    format;         /* Parameters format            */
    vltUINT32           parameterNb;    /* Number of parameters         */
    chkcPARAM           *parameters;    /* Array of parameters          */
    chkcPARAM_FORMAT    replyType;      /* Reply format                 */
    vltUINT32           replyParamNb;   /* Number of reply parameters   */

  /*chkcREPLY_PARAM     *replyParams;*/ /* Array of reply parameters    */
    chkcPARAM           *replyParams;
    vltUINT32           replyLength;    /* Length of reply              */
    char                *displayFormat; /* Display format string        */
    char                *helpText;      /* Help text                    */
    } chkcCOMMAND;


typedef struct                          /* Fast name scan structure     */
    {                                   
    char                *name;          /* Name (command or synonym)    */
    chkcCOMMAND         *command;       /* Pointer to associated command*/
    } chkcNAME_ITEM;


typedef struct                          /* Process description          */
    {
    char                *name;          /* Process name                 */
    chkcENV_TYPE        location;       /* Location of definition table */
    vltUINT32           commandNb;      /* number of commands           */
    chkcCOMMAND         *commands;      /* Array of command descriptions*/
                                        /* Last three fields are used   */
                                        /* for fast command name search:*/
    vltUINT32           nameNb;         /* Number of defined names      */
    vltUINT32           namesSize;      /* Current size of names array  */
    chkcNAME_ITEM       *names;         /* Array of all command names   */
                                        /* and synonyms                 */
    } chkcPROCESS;


 
typedef struct                          /* Environment description      */
    {
    char                *name;          /* Environment name             */
    chkcENV_TYPE        location;       /* Location of definition file  */
    vltUINT32           processNb;      /* Number of processes          */
    chkcPROCESS         *processes;     /* Array of process descriptions*/
    } chkcENV;


typedef struct                          /* All the environments         */
    {
    vltUINT32           envNb;          /* Number of environments       */
    chkcENV             *env;           /* Environment descriptions     */
    } chkcCHKC;
 

typedef struct                          /* Description of parameters    */
                                        /* found in parsed commands     */
    {
    chkcPARAM_TYPE      type;           /* Parameter type               */
    vltUINT16           valueNb;        /* Number of values             */
    char                **text;         /* Array of values in ASCII     */
    vltLOGICAL          flag;           /* Flag value (only for LOGICAL */
                                        /* parameters)                  */
    } chkcPARAM_LIST_ITEM;


typedef struct                          /* Description parsed command   */
    {
    vltUINT32           parameterNb;    /* Number of parameters         */
    chkcPARAM_LIST_ITEM *parameters;    /* Array of parameters          */
    } chkcPARAM_LIST;

typedef struct                          /* Description of parameters    */
                                        /* found in parsed commands     */
    {
    chkcPARAM_TYPE      type;           /* Parameter type               */
    vltUINT16           valueNb;        /* Number of values             */
    char                **text;         /* Array of values in ASCII     */
    vltLOGICAL          flag;           /* Flag value (only for LOGICAL */
                                        /* parameters)                  */
    } chkcREPLY_PARAM_LIST_ITEM;


typedef struct                          /* Description parsed command   */
    {
    vltUINT32           parameterNb;    /* Number of parameters         */
    chkcREPLY_PARAM_LIST_ITEM *parameters;  /* Array of parameters          */
    } chkcREPLY_PARAM_LIST;

typedef enum                            /* CIT entry types              */
    {
    chkcENTRY_TYPE_NOT_DEFINED = 1,
    chkcENTRY_TYPE_DUMMY ,
    chkcENTRY_TYPE_FUNCTION ,
    chkcENTRY_TYPE_TASK
    } chkcCIT_ENTRY_TYPE;


typedef struct                          /* CIT entry description        */
    {
    msgCMD              commandName;    /* Command name                 */
    char                *routineName;   /* Routine name                 */
    char                *entryPoint;    /* Pointer to routine           */
    void                (*breakHandler)(int);/* Pointer to break handler */
    void                (*killHandler)(int); /* Pointer to kill handler */
    chkcCIT_ENTRY_TYPE  type;           /* Entry type                   */
    char                *taskName;      /* Task name (for tasks)        */
    vltUINT32           taskPriority;   /* Task priority (for tasks)    */
    vltUINT32           taskRTOptions;  /* Task options (for tasks)     */
    vltUINT32           taskStackSize;  /* Task stack size (for tasks)  */
    vltLOGICAL          toRegister;     /* Register to message system   */
                                        /* (for tasks)                  */
    vltLOGICAL          rawMode;        /* Parameters should be passed  */
                                        /* without any processing       */
    vltLOGICAL          setId;          /* Use process id of command    */
                                        /* sender during processing     */
    vltUINT8            badStates;      /* Bitfield indicating LCU      */
                                        /* states for which command is  */
                                        /* rejected                     */
    } chkcCIT_ENTRY;


typedef struct                          /* All the entries of a CIT     */
    {
    vltUINT32           entryNb;        /* Number of entries            */
    chkcCIT_ENTRY       *entries;       /* Array of entry descriptions  */
    } chkcCIT;


/*
 * function prototypes
 */

ccsCOMPL_STAT chkcCDTLoadProcess(       /* Load commands of a process   */
                                        /* from CDT file                */
        const char      *fileName,      /* CDT file name                */
        chkcPROCESS     *processTable,  /* Where to put loaded commands */
        ccsERROR        *error          /* Returned error structure     */
        );


void chkcCDTDeleteProcess(              /* Delete all CDT command       */
                                        /* descriptions for a process   */
        chkcPROCESS *processTable       /* Process description          */
        );


ccsCOMPL_STAT chkcGetCmdInProcess(      /* Find a command description   */
                                        /* in a process                 */
        const char      *commandName,   /* Command name or synonym      */
        const chkcPROCESS *processTable,/* Where to look for            */
        chkcCOMMAND     **commandData,  /* Returned command description */
        ccsERROR        *error          /* Returned error structure     */
        );


ccsCOMPL_STAT chkcCheckCmd(             /* Check an ASCII parameter list*/
        const char      *paramList,     /* ASCII parameter list         */
        vltINT32        length,         /* Length of parameters string  */
        const chkcCOMMAND *commandData, /* Command description          */
        chkcPARAM_LIST  *foundParams,   /* Returned params description  */
        vltUINT32       *size,          /* Returned length of conversion*/
                                        /* buffer required for          */
                                        /* chkcConvertCmd               */ 
        ccsERROR        *error          /* Returned error structure     */
        );

ccsCOMPL_STAT chkcCheckReply(           /* Check an ASCII parameter list*/
        const char      *paramList,     /* ASCII parameter list         */
        vltINT32        length,         /* Length of parameters string  */
        const chkcCOMMAND *commandData, /* Command description          */
        chkcPARAM_LIST  *foundParams,   /* Returned params description  */
        vltUINT32       *size,          /* Returned length of conversion*/
                                        /* buffer required for          */
                                        /* chkcConvertCmd               */ 
        ccsERROR        *error          /* Returned error structure     */
        );

ccsCOMPL_STAT chkcConvertCmd(           /* Convert an ASCII parameter   */
                                        /* description list into a      */
                                        /* packed byte stream           */
        const chkcCOMMAND *commandData, /* Command description          */
        const chkcPARAM_LIST *foundParams,
                                        /* Parameters description list  */
        vltUINT32       streamMaxLen,   /* Size of output byte stream   */
        vltUINT32       *streamFinalLen,/* Used part in the stream      */
        char            *stream,        /* Where to write byte stream   */
        ccsERROR        *error          /* Returned error structure     */
        );


ccsCOMPL_STAT chkcConvertReply(         /* Convert an ASCII parameter   */
                                        /* description list into a      */
                                        /* packed byte stream           */
        const chkcCOMMAND *commandData, /* Command description          */
        const chkcPARAM_LIST *foundParams,
                                        /* Parameters description list  */
        vltUINT32       streamMaxLen,   /* Size of output byte stream   */
        vltUINT32       *streamFinalLen,/* Used part in the stream      */
        char            *stream,        /* Where to write byte stream   */
        ccsERROR        *error          /* Returned error structure     */
        );

void chkcListDelete(                    /* Delete parsed parameter list */
        chkcPARAM_LIST  *list           /* List to delete               */
        );


ccsCOMPL_STAT chkcCITLoad(              /* Load all entries from a CIT  */
                                        /* file                         */
        const char      *fileName,      /* CIT file name                */
        chkcCIT         *table,         /* Where to put loaded entries  */
        ccsERROR        *error          /* Returned error structure     */
        );


void chkcCITDelete(                     /* Delete all CIT entries       */
        chkcCIT         *table          /* Table to delete              */
        );


ccsCOMPL_STAT chkcCDTLoadAll(           /* Load all environments        */
        chkcCHKC        *chkcTable,     /* Where to put environments    */
        ccsERROR        *error          /* Returned error structure     */
        );


void chkcExpandCommandLine(             /* Attempts to complete the     */
                                        /* current command line with    */
                                        /* longest possible keyword     */
        const char      *commandLine,   /* Current command line         */
        vltINT32        lineLen,        /* Length of parameters string  */
        const chkcPROCESS *process,     /* Process table                */
        char            **keyword,      /* Pointer to keyword that      */
                                        /* completes line               */
        vltINT32        *length,        /* Completion length            */
        vltLOGICAL      *isComplete     /* Indicates that line is       */
                                        /* uniquely completed with      */
                                        /* keyword                      */
        );


vltINT32 chkcCompareName(               /* Perform case insensitive     */
                                        /* string comparison            */
        const char      *name1,
        const char      *name2,
        vltINT32        len             /* len = -1 -> like strcmp      */
                                        /* len != -1 -> like strncmp    */
        );


/* 
 * message definitions
 */

/* 
 * error definitions
 */

#ifdef MAKE_VXWORKS
#include "chkcLCC.h"
#else
#include "chkcCCS.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /*!CHKC_H*/

