/**************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: cmdCCS.h,v 2.31 2005/07/27 12:26:24 vltsccm Exp $" 
*
* <cmd.h>  -  CCS cmd Interface File
*
* who        when      what
* ---------  --------  ----------------------------------------------
* T.Herlin   13/10/94  Created
* M.COMIN    21/11/94  Add definitions for error mnemonics
* T.Herlin   27/01/95  Change CDT database point
* T.Herlin   27/01/95  Added in  cmdCheckCmd return of cmd in case of
*                      synonyms.
* T.Herlin   31/01/95  Added const declarations. 
* T.Herlin   22/05/95  Added parameter list structures
* T.Herlin   28/09/95  Extended parameter list structure
* T.Herlin   30/10/95  Added cmdGetCmdType
* T.Herlin   09/04/96  Added cmdParamDefault and cmdParamDelete
* T.Herlin   09/04/96  Added cmdParamSet and cmdParamGet convenience functions
* T.Herlin   08/07/96  Added new reply functions and moved internal functions
*                      to cmdPrivate.h
* B.Gilli    09/10/01  SPR 20010583; **bufPtr is now const char in cmdSimulation proto
* tebert     02/02/02  added function cmdSwapBinReply for LINUX
* B.Gilli    24/05/02  corrected endif termination comment for gcc 3.0.4
* T.Ebert    29/11/04  cmdSwapBinCmd()
************************************************************************
*/

#ifndef CMDCCS_H
#define CMDCCS_H


/*
 *  Header Files
 */

#include "msg.h"
#include "ccs.h"
#include "err.h"

/*
 * Definitions of error mnemonics
 */

#include "cmdErrors.h"

/*
 * local structures
 */

typedef enum                            /* Parameter types              */
    {
    cmdPARAM_TYPE_NOT_DEFINED = 1,
    cmdPARAM_TYPE_INTEGER,
    cmdPARAM_TYPE_REAL,
    cmdPARAM_TYPE_STRING,
    cmdPARAM_TYPE_LOGICAL
    } cmdPARAM_TYPE;

typedef enum                            /* Command types                */
    {
    cmdCOMMAND_TYPE_NOT_DEFINED = 1,
    cmdCOMMAND_TYPE_PUBLIC,
    cmdCOMMAND_TYPE_MAINTENANCE,
    cmdCOMMAND_TYPE_TEST
    } cmdCOMMAND_TYPE;


/* parameter list item */
typedef struct
{
    cmdPARAM_TYPE     type;      /* Parameter type               */
    vltUINT16         valueNb;   /* Number of values */
    char              **text;    /* Array of values in ASCII */
    vltLOGICAL        flag;      /* Flag value (only for LOGICAL parameters) */
} cmdPARAM_LIST_ITEM;

/* list of parameter items */
typedef struct
{
    vltUINT32            parameterNb; /* Number of parameters */
    cmdPARAM_LIST_ITEM  *parameters;  /* Array of parameters  */
    msgCMD               command;     /* Command name         */
    ccsPROCNAME          processName; /* Name of process      */
    vltLOGICAL           replyParam;  /* Flag indicating reply parameters  */
} cmdPARAM_LIST;


/*
 * Functions reserved for message system usage only:
 */

/* check command parameters and expand defaults */
ccsCOMPL_STAT cmdCheckCmd(const ccsPROCNAME   processName,
			  const char          *command,
                          const char          *param,
                          msgLENGTH           paramLen,
			  msgCMD              cmd,
                          char                **checkParam,
                          msgLENGTH           *checkParamLen,
                          ccsERROR            *error);

/* check if process is in simulation */
ccsCOMPL_STAT cmdSimulation(const ccsENVNAME  env,
			    const ccsPROCNAME process,
			    const msgCMD      command,
			    vltLOGICAL       *simulate,
			    const char      **bufPtr,
			    vltUINT32        *bufSize,
			    ccsERROR         *error);

/* format structured binary reply to ascii reply */
ccsCOMPL_STAT cmdFormatReply(const ccsPROCNAME processName,
			     const char        *command,
			     const char        *reply,
			           msgLENGTH   replyLen,
			           char        *formatedReply,
			           ccsERROR    *error);
/*
 * 
 * Public functions of general use in CCS
 * 
 */
/* command parameter parsing */
ccsCOMPL_STAT cmdParamList(const char         *command,
			   const char         *param,
                           const vltINT32     paramLen,
			   cmdPARAM_LIST      *paramList,
                           ccsERROR           *error);

/* reply buffer parsing */
ccsCOMPL_STAT cmdReplyList(const ccsPROCNAME   processName,
			    const char         *command,
			    const char         *reply,
                            const vltINT32      replyLen,
			    cmdPARAM_LIST      *replyList,
			    ccsERROR           *error);

/* parameter list de-allocation */
ccsCOMPL_STAT cmdParamFree(cmdPARAM_LIST *paramList,
                           ccsERROR      *error);

/* parameter value retrieval - by name */
ccsCOMPL_STAT cmdParamGetByName(const cmdPARAM_LIST *paramList,
                                const char          *paramName,
                                      vltINT32      *paramNb,
                                      cmdPARAM_TYPE *paramType,
                                      void          **paramValues,
                                      ccsERROR      *error);

ccsCOMPL_STAT cmdParamGetLogicalByName(const cmdPARAM_LIST *paramList,
				       const char          *paramName,
					     vltLOGICAL    *value,
					     ccsERROR      *error);


ccsCOMPL_STAT cmdParamGetIntByName(const cmdPARAM_LIST *paramList,
				   const char          *paramName,
				         vltINT32      *paramNb,
					 vltINT32      **value,
					 ccsERROR      *error);

ccsCOMPL_STAT cmdParamGetDoubleByName(const cmdPARAM_LIST *paramList,
				      const char          *paramName,
				            vltINT32      *paramNb,
				            vltDOUBLE     **value,
				            ccsERROR      *error);

ccsCOMPL_STAT cmdParamGetStringByName(const cmdPARAM_LIST *paramList,
				      const char          *paramName,
				            vltINT32      *paramNb,
				            char          **value,
				            ccsERROR      *error);


ccsCOMPL_STAT cmdParamReadByName(const cmdPARAM_LIST *paramList,
				 const char          *paramName,
                          	 const cmdPARAM_TYPE paramType,
                                       vltINT32      *paramNb,
                                       void          *paramValues,
                                       ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadLogicalByName(const cmdPARAM_LIST *paramList,
				 	const char          *paramName,
				       	      vltLOGICAL    *value,
				       	      ccsERROR      *error);


ccsCOMPL_STAT cmdParamReadIntByName(const cmdPARAM_LIST *paramList,
				    const char          *paramName,
				          vltINT32      *paramNb,
				          vltINT32      *value,
				          ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadDoubleByName(const cmdPARAM_LIST *paramList,
				       const char          *paramName,
					     vltINT32      *paramNb,
					     vltDOUBLE     *value,
					     ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadStringByName(const cmdPARAM_LIST *paramList,
				       const char          *paramName,
					     vltINT32      *paramNb,
				       const char          **value,
					     ccsERROR      *error);


/* parameter value setting - by name */
ccsCOMPL_STAT cmdParamSetByName(cmdPARAM_LIST *paramList,
                                char          *paramName,
				vltINT32      paramNb,
                                void          *paramValues,
                                ccsERROR      *error);


ccsCOMPL_STAT cmdParamSetLogicalByName(cmdPARAM_LIST *paramList,
				       char          *paramName,
				       vltLOGICAL    value,
				       ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetIntByName(cmdPARAM_LIST *paramList,
				   char          *paramName,
				   vltINT32      paramNb,
				   const vltINT32      *value,
				   ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetDoubleByName(cmdPARAM_LIST *paramList,
				      char          *paramName,
				      vltINT32      paramNb,
				      const vltDOUBLE     *value,
				      ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetStringByName(cmdPARAM_LIST *paramList,
				      char          *paramName,
				      vltINT32      paramNb,
				      char          **stringArray,
				      ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetSingleStringByName(cmdPARAM_LIST *paramList,
					    char          *paramName,
					    const char          *string,
					    ccsERROR      *error);

/* parameter value retrieval - by index */
ccsCOMPL_STAT cmdParamGetByIndex(const cmdPARAM_LIST *paramList,
                                 const vltINT32      index,
                                       vltINT32      *paramNb,
                                       cmdPARAM_TYPE *paramType,
                                       void          **paramValues,
                                       ccsERROR      *error);


ccsCOMPL_STAT cmdParamGetLogicalByIndex(const cmdPARAM_LIST *paramList,
					const vltINT32      index,
					      vltLOGICAL    *value,
					      ccsERROR      *error);


ccsCOMPL_STAT cmdParamGetIntByIndex(const cmdPARAM_LIST *paramList,
				    const vltINT32      index,
				          vltINT32      *paramNb,
					  vltINT32      **value,
					  ccsERROR      *error);

ccsCOMPL_STAT cmdParamGetDoubleByIndex(const cmdPARAM_LIST *paramList,
				       const vltINT32      index,
				             vltINT32      *paramNb,
				             vltDOUBLE     **value,
				             ccsERROR      *error);

ccsCOMPL_STAT cmdParamGetStringByIndex(const cmdPARAM_LIST *paramList,
				       const vltINT32      index,
				             vltINT32      *paramNb,
				             char          **value,
				             ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadByIndex(const cmdPARAM_LIST *paramList,
                          	  const vltINT32      index,
                          	  const cmdPARAM_TYPE paramType,
                                	vltINT32      *paramNb,
                                	void          *paramValues,
                                	ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadLogicalByIndex(const cmdPARAM_LIST *paramList,
				 	 const vltINT32      index,
				       	       vltLOGICAL    *value,
				       	       ccsERROR      *error);


ccsCOMPL_STAT cmdParamReadIntByIndex(const cmdPARAM_LIST *paramList,
			     	     const vltINT32      index,
				           vltINT32      *paramNb,
				           vltINT32      *value,
				           ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadDoubleByIndex(const cmdPARAM_LIST *paramList,
					const vltINT32      index,
					      vltINT32      *paramNb,
					      vltDOUBLE     *value,
					      ccsERROR      *error);

ccsCOMPL_STAT cmdParamReadStringByIndex(const cmdPARAM_LIST *paramList,
					const vltINT32      index,
					      vltINT32      *paramNb,
					const char          **value,
					      ccsERROR      *error);



/* parameter value setting - by index */
ccsCOMPL_STAT cmdParamSetByIndex(cmdPARAM_LIST *paramList,
                                 vltINT32      index,
                                 vltINT32      paramNb,
                                 void          *paramValues,
                                 ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetLogicalByIndex(cmdPARAM_LIST *paramList,
					vltINT32      index,
					vltLOGICAL    value,
					ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetIntByIndex(cmdPARAM_LIST *paramList,
				    vltINT32      index,
				    vltINT32      paramNb,
				    vltINT32      *value,
				    ccsERROR      *error);

ccsCOMPL_STAT cmdParamSetDoubleByIndex(cmdPARAM_LIST *paramList,
				       vltINT32      index,
				       vltINT32      paramNb,
				       vltDOUBLE     *value,
				       ccsERROR      *error);


ccsCOMPL_STAT cmdParamSetStringByIndex(cmdPARAM_LIST *paramList,
				       vltINT32      index,
				       vltINT32      paramNb,
				       char          **stringArray,
				       ccsERROR      *error);


ccsCOMPL_STAT cmdParamSetSingleStringByIndex(cmdPARAM_LIST *paramList,
					     vltINT32      index,
					     char          *string,
					     ccsERROR      *error);

/* format parameter list structure to comma seperated ascii string */
ccsCOMPL_STAT cmdParamToMsg(const cmdPARAM_LIST *paramList,
			          char          *param,
                                  vltINT32      *paramLen,
                                  ccsERROR      *error);

/* format parameter list structure to quoated ascii string */
ccsCOMPL_STAT cmdParamToForwardMsg(const cmdPARAM_LIST *paramList,
				         char          *param,
				         vltINT32      *paramLen,
				         ccsERROR      *error);

ccsCOMPL_STAT cmdForwardToMsg(char          *param,
			      ccsERROR      *error);

/* return command type of command */
ccsCOMPL_STAT cmdGetCmdType(const ccsPROCNAME     process,
                            const char            *command,
                                  cmdCOMMAND_TYPE *cmdType,
                                  ccsERROR        *error);

/* log parameter list structure */
ccsCOMPL_STAT cmdParamLog(const cmdPARAM_LIST *paramList,
			        ccsERROR      *error);


/* construct parameter list structure with default (if present) values */
ccsCOMPL_STAT cmdParamDefault(const ccsPROCNAME   processName,
                              const char          *command,
			            cmdPARAM_LIST *paramList,
                                    ccsERROR      *error);

/* construct reply  list structure with default (if present) values */
ccsCOMPL_STAT cmdReplyDefault(const ccsPROCNAME   processName,
                              const char          *command,
			            cmdPARAM_LIST *replyList,
                                    ccsERROR      *error);

/* delete single parameter from parameter list */
ccsCOMPL_STAT cmdParamDelete(cmdPARAM_LIST *paramList,
			     vltUINT32     index,
			     ccsERROR      *error);

#if defined(i386) || defined(__i386)
/* format structured binary reply under LINUX */
void cmdSwapBinReply (const ccsPROCNAME processName,
		      const char        *command,
		            char        *reply,
                               ccsERROR *error);

void cmdSwapBinCmd   (const ccsPROCNAME  processName, 
                             const char *cmd, 
                             char       *buffer, 
		             ccsERROR   *error);
#endif

#endif  /* CMDCCS_H */
/* --o0o-- */
