/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: dbv44reg.h,v 2.140 2005/10/17 15:17:04 vltsccm Exp $"
*
* who       when      what
* abalestr  06/07/00  Copied from original dbv44reg.h
*************************************************************************/

/************************************************************************

  File: dbv44reg.h

  Date: 13.11.97

  Register definition of the VME/DSP Adapter Board 


************************************************************************/

#ifndef _dbv44reg_h
#define _dbv44reg_h

/* Define default signal to send to user and maximum single transfer size */
#define DBV44_GPISIG	SIGUSR1
#define DBV44_MINPHYS	(8 * 1024)

/* This structure defines the parts of the VME map used for communicating with
 * the LIA.
 */
struct lia_regs {
	/* Four LIA data registers (one for each TIM site) */
	u_long	c_data;
	u_long	d_data;
	u_long	b_data;
	u_long	a_data;

	/* General control/status and interrupt registers */
	short	status;		/* General status (R) and GPI clear (W)	*/
	short	isr;		/* Interrupt identify (R) and select (W)*/
	short	id;		/* Status/ID register (RW)		*/
	short	ier;		/* Interrupt enable(R)			*/
	short	status_ref;	/* Reflections of above regs		*/
	short	isr_ref;
	short	id_ref;
	short	ier_ref;
};

/* This structure defines the whole VME address map for DBV44. It includes the
 * LIA structure defined above
 */
struct dbv44_regs {
	/* First 128 bytes are TBC registers, although only 24 words are used */
	short		tbc_reg[64];
	struct lia_regs	lia;	/* LIA registers		*/
	struct lia_regs	lia_ref;/* LIA register reflections	*/
	short		waste;	/* Waste 16-bit word before CSR	*/
	short		csr;	/* DBV44 board CSR		*/
	short		csr_ref[6];	/* CSR reflections	*/

    /* Registers of Com Port Distributor added by jb, 22.09.97 */

        short           cpd_ctrl_reg;      
        short           cpd_counter_hi_reg;
        short           cpd_counter_lo_reg;
        short           cpd_fifo_stat_reg;
    
        short           muell[21];
};

/*****************************************************************************
 * Constants for masking bits out of registers
 */

/*
 * DBV44 Board Control Register
 */
#define DBV44_GLOBAL_RESET	0x0010	/* Global Reset bit for DBV44 	*/
#define DBV44_IRQ_SHIFT		5	/* Number of bits to shift VMEbus
					   interrupt level when ORing into
					   DBV44 board control register	*/
#define DBV44_IRQ_MASK		0x00e0	/* Mask for interrupt level part of
					   DBV44 board control register	*/
/*
 * LIA Status Register
 */
#define DBV44_LED_C		0x0001	/* LED C status			*/
#define DBV44_LED_D		0x0002	/* LED D status			*/
#define DBV44_SENSE_C		0x0004	/* Site C sense flag		*/
#define DBV44_SENSE_D		0x0008	/* Site D sense flag		*/
#define DBV44_GLOBAL_RESET	0x0010	/* GLOBAL_RESET line status	*/
#define DBV44_LED_B		0x0100	/* LED B status			*/
#define DBV44_LED_A		0x0200	/* LED A status			*/
#define DBV44_SENSE_A		0x0400	/* Site A sense flag		*/
#define DBV44_SENSE_B		0x0800	/* Site B sense flag		*/
#define DBV44_CONFIG		0x1000	/* Board-wide CONFIG line status*/

/*
 * LIA Interrupt [Select|Status] Register. The _GP constants also apply
 * to the GP Interrupt Clear Register.
 */
#define DBV44_C_RDF		0x0001	/* Port C Receive Data Reg. full*/
#define DBV44_C_TDE		0x0002	/* Port C Transmit Data Reg. empty*/
#define DBV44_D_RDF		0x0004	/* Port D Receive Data Reg. full*/
#define DBV44_D_TDE		0x0008	/* Port D Transmit Data Reg. empty*/
#define DBV44_C_GPI		0x0010	/* Port C GP Interrupt		*/
#define DBV44_D_GPI		0x0020	/* Port D GP Interrupt		*/
#define DBV44_B_RDF		0x0100	/* Port B Receive Data Reg. full*/
#define DBV44_B_TDE		0x0200	/* Port B Transmit Data Reg. empty*/
#define DBV44_A_RDF		0x0400	/* Port A Receive Data Reg. full*/
#define DBV44_A_TDE		0x0800	/* Port A Transmit Data Reg. empty*/
#define DBV44_B_GPI		0x1000	/* Port B GP Interrupt		*/
#define DBV44_A_GPI		0x2000	/* Port A GP Interrupt		*/

#define DBV44_ALL_RDF	(DBV44_A_RDF | DBV44_B_RDF | DBV44_C_RDF | DBV44_D_RDF)
#define DBV44_ALL_TDE	(DBV44_A_TDE | DBV44_B_TDE | DBV44_C_TDE | DBV44_D_TDE)
#define DBV44_ALL_GPI	(DBV44_A_GPI | DBV44_B_GPI | DBV44_C_GPI | DBV44_D_GPI)
#define DBV44_A_ALL	(DBV44_A_RDF | DBV44_A_TDE | DBV44_A_GPI)
#define DBV44_B_ALL	(DBV44_B_RDF | DBV44_B_TDE | DBV44_B_GPI)
#define DBV44_C_ALL	(DBV44_C_RDF | DBV44_C_TDE | DBV44_C_GPI)
#define DBV44_D_ALL	(DBV44_D_RDF | DBV44_D_TDE | DBV44_D_GPI)
#define DBV44_ALL_ALL	(DBV44_ALL_RDF | DBV44_ALL_TDE | DBV44_ALL_GPI)

/*
 * LIA Interrupt Enable Register
 */
#define DBV44_CD_ENABLE		0x0002	/* Enable IRQs from sites C & D	*/
#define DBV44_AB_ENABLE		0x0200	/* Enable IRQs from sites A & B	*/
#define DBV44_ALL_ENABLE	(DBV44_CD_ENABLE | DBV44_AB_ENABLE)

/*****************************************************************************
 * Device driver definitions possibly of use to user programs
 */
#define DBV44_REG_SIZE		(sizeof(struct dbv44_regs))
#define DBV44_NT_BOARD		"lsi-dsp:c40:board"
#define DBV44_NT_LIA		"lsi-dsp:c40:lia"

#endif /* _dbv44reg_h */
