#ifndef eccs_H
#define eccs_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccs.h,v 1.134 2005/04/08 08:34:12 vltsccm Exp $" 
*
* eccs.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  10/02/97  Added prototype for new eccsParseTarget() function
* gchiozzi  16/09/94  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
* This include file is the EasyCCS interface to CCS base module
* and must be included by any application using EasyCCS
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

extern "C" {

#include "ccs.h"        /* Standard ccs include */

}

#include <eccsErrors.h>

#ifndef TRUE
    #define TRUE  1
    #define FALSE 0
#endif

/* Usefull macros for defining what parameters     */
/* are in INPUT or OUTPUT in function declarations */
#ifndef IN
#ifndef OUT
    #define IN
    #define OUT
#endif
#endif

/* The macro __FILE_LINE__ produces a string in the format: "FileName:Line" */
/* with the name and line of the file where it has been called.             */
/* It is usefull in error messages                                          */
/* (the other macros defined here are just for support, since I have not    */
/* found a cleaner way to define __FILE_LINE__ macro                        */

#define _eccs_tostr(a) #a
#define _eccs_tostr_pass2(a) _eccs_tostr(a) 

#ifndef __FILE_LINE__ 
     #define __FILE_LINE__   __FILE__ ":" _eccs_tostr_pass2(__LINE__)
#endif

/* The macro USE(var) can be used to prevent the GCC compiler from  */
/* complaining about an unused variable. Specially with objects,    */
/* can happens that the compiler thinks that a variable is not used */
/* and issues a warning. This macro prevents the warning            */
#ifndef USE
  #define USE(var)  static const void *use_##var = ((const void)&use_##var, (const void *) &var);
#endif


/* 
 * Class eccsERROR for error handling.
 * The true definition is in eerr.h
 */
struct eccsERROR;

/* This is the global eccs error stack */
extern eccsERROR *stdErr;


/****************************************/
/* Prototypes for general CCS functions */
/****************************************/

ccsCOMPL_STAT ccsExit ();      /* Release CCS resources used by application */

ccsCOMPL_STAT ccsGetMyProcId ( /* Retrieves the Rtap process identifier */
   OUT ccsENVNAME    envName,
   OUT ccsPROCNUM   *procNum,
   OUT ccsPROCNAME   procName);

ccsCOMPL_STAT ccsInit (        /* Initialize the CCS resources     */
   IN  const ccsPROCNAME procName,  
   IN  void        (*breakHandler)(int signal)     = NULL,
   IN  void        (*killHandler)(int signal)      = NULL);

ccsCOMPL_STAT ccsOpenFile (    /* Open file located in standard search path */
   IN  const char   *fileName,
   IN  const char   * mode,
   OUT FILE        **fileHANDLE);


/* Error handling must always be included when eccs is used */
#ifndef eccsErr_H
  #include "eccsErr.h"
#endif

/****************************************/
/* Prototypes for new ECCS functions    */
/****************************************/

ccsCOMPL_STAT eccsParseTarget(OUT ccsENVNAME   destEnv, 
			      OUT ccsPROCNAME  destProc, 
			      IN  const char  *target);

/* Include the code for inline methods */
#include "eccs.icc"

#include "eccsTIMEVAL.h"

#endif /* ! ECCS_H */
