#ifndef eccsDB_ATTR_H
#define eccsDB_ATTR_H
/****************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: eccsDB_ATTR.h,v 1.134 2005/04/08 08:34:14 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgilli   2002-12-03 (const void *) rather than (void *): SPR 20020531
* gchiozzi 2002-08-01 Added using namespace std
* gchiozzi 2002-08-01 gcc 3.1: changed <xxx.h> sys includes with <xxx>
* gchiozzi  27/08/97  Added option parameter to check buffer size on read/write
* rkarban   03/07/97  Added INT8 and UINT8 classes
* gchiozzi  10/06/97  Removed DbRead() and DbWrite() with index 
* vltsw     23/07/96  Derived public virtual from eccsERROR_CLASS
* mnastvog  23/04/96  Added classes for UINT32 and UINT16
* gchiozzi  20/11/95  Added new DbWrite methods for attribute details specifications
* gchiozzi  15/11/95  Moved here RecordLen() method
* rabuter   27/10/95  Added DbRead DbWrite by index, RecordCnt()
* gchiozzi  18/08/95  Added define eccsDB_STRING_MAX_SIZE
* gchiozzi  24/07/95  Added assignement operators
* gchiozzi  20/07/95  Modified class eccsDB_STRING
* gchiozzi  26/04/95  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include <iostream>
using namespace std;

#ifndef eccsErr_H
  #include "eccsErr.h"
#endif
#ifndef eccsDb_H
  #include "eccsDb.h"
#endif

class eccsDB_ATTR : virtual public eccsERROR_CLASS
{
  public:
    eccsDB_ATTR(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_ATTR(const eccsDB_ATTR &attr);
    virtual ~eccsDB_ATTR();

    virtual ccsCOMPL_STAT GetValue(void *buffer, vltINT32 bsize = 0);
    virtual ccsCOMPL_STAT SetValue(const void *buffer, vltINT32 bsize = 0);

    virtual vltUINT32   RecordCnt();
    virtual vltUINT32   RecordLen();

    virtual void        PrintOn( ostream& outputStream)const;

  protected:
    virtual ccsCOMPL_STAT CopyDataType(const dbTYPE *newDataType);
    virtual ccsCOMPL_STAT DbRead(void *buffer, vltINT32 bsize = 0);
    virtual ccsCOMPL_STAT DbRead(char *attrSpec, vltUINT16 nrec,
				 void *buffer, vltINT32 bsize = 0);
    virtual ccsCOMPL_STAT DbWrite(const void *buffer, vltINT32 bsize = 0);
    virtual ccsCOMPL_STAT DbWrite(char *attrSpec, vltUINT16 nrec,
				  const void *buffer, vltINT32 bsize = 0);
    virtual ccsCOMPL_STAT LoadInfo();

    dbSYMADDRESS  symAddr;
    dbTYPE       *dataType;
    vltINT32      size;
    vltUINT16     recordCnt;
    vltUINT32     recordLen;
    dbATTRTYPE    attrType;

  private:
    eccsDB_ATTR &operator=(const eccsDB_ATTR &);

};

class eccsDB_LOGICAL : public eccsDB_ATTR
{
  public:
    eccsDB_LOGICAL(const dbSYMADDRESS pointName, const char *attrName=dbEMPTY);
    eccsDB_LOGICAL(const eccsDB_LOGICAL &attr);

    eccsDB_LOGICAL &operator=(vltLOGICAL newValue);
    eccsDB_LOGICAL &operator=(eccsDB_LOGICAL &from);
    operator vltLOGICAL();

};

class eccsDB_INT8 : public eccsDB_ATTR
{
  public:
    eccsDB_INT8(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_INT8(const eccsDB_INT8 &attr);

    eccsDB_INT8 &operator=(vltINT8 newValue);
    eccsDB_INT8 &operator=(eccsDB_INT8 &from);
    operator vltINT8();

};

class eccsDB_UINT8 : public eccsDB_ATTR
{
  public:
    eccsDB_UINT8(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_UINT8(const eccsDB_UINT8 &attr);

    eccsDB_UINT8 &operator=(vltUINT8 newValue);
    eccsDB_UINT8 &operator=(eccsDB_UINT8 &from);
    operator vltUINT8();

};

class eccsDB_INT16 : public eccsDB_ATTR
{
  public:
    eccsDB_INT16(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_INT16(const eccsDB_INT16 &attr);

    eccsDB_INT16 &operator=(vltINT16 newValue);
    eccsDB_INT16 &operator=(eccsDB_INT16 &from);
    operator vltINT16();

};

class eccsDB_UINT16 : public eccsDB_ATTR
{
  public:
    eccsDB_UINT16(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_UINT16(const eccsDB_UINT16 &attr);

    eccsDB_UINT16 &operator=(vltUINT16 newValue);
    eccsDB_UINT16 &operator=(eccsDB_UINT16 &from);
    operator vltUINT16();

};

class eccsDB_INT32 : public eccsDB_ATTR
{
  public:
    eccsDB_INT32(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_INT32(const eccsDB_INT32 &attr);

    eccsDB_INT32 &operator=(vltINT32 newValue);
    eccsDB_INT32 &operator=(eccsDB_INT32 &from);
    operator vltINT32();

};

class eccsDB_UINT32 : public eccsDB_ATTR
{
  public:
    eccsDB_UINT32(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_UINT32(const eccsDB_UINT32 &attr);

    eccsDB_UINT32 &operator=(vltUINT32 newValue);
    eccsDB_UINT32 &operator=(eccsDB_UINT32 &from);
    operator vltUINT32();

};

class eccsDB_FLOAT : public eccsDB_ATTR
{
  public:
    eccsDB_FLOAT(const dbSYMADDRESS pointName, const char *attrName = dbEMPTY);
    eccsDB_FLOAT(const eccsDB_FLOAT &attr);

    eccsDB_FLOAT &operator=(vltFLOAT newValue);
    eccsDB_FLOAT &operator=(eccsDB_FLOAT &from);
    operator vltFLOAT();

};

class eccsDB_DOUBLE : public eccsDB_ATTR
{
  public:
    eccsDB_DOUBLE(const dbSYMADDRESS pointName,const char *attrName = dbEMPTY);
    eccsDB_DOUBLE(const eccsDB_DOUBLE &attr);

    eccsDB_DOUBLE &operator=(vltDOUBLE newValue);
    eccsDB_DOUBLE &operator=(eccsDB_DOUBLE &from);
    operator vltDOUBLE();

};

#define eccsDB_STRING_MAX_SIZE 256

class eccsDB_STRING : public eccsDB_ATTR
{
  public:
    eccsDB_STRING(const dbSYMADDRESS pointName,const char *attrName = dbEMPTY);
    eccsDB_STRING(const eccsDB_STRING &attr);
    virtual ~eccsDB_STRING();

    eccsDB_STRING &operator=(const char *newValue);
    eccsDB_STRING &operator=(eccsDB_STRING &from);
    operator const char *();
  private:
    char *string;

    static char buffer[eccsDB_STRING_MAX_SIZE];
};

#include "eccsDB_ATTR.icc"

#endif /*!eccsDB_ATTR_H*/
