#ifndef eccsDB_TABLE_H
#define eccsDB_TABLE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: eccsDB_TABLE.h,v 1.134 2005/04/08 08:34:14 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2002-06-05 Added class-key to frien declaration (gcc 3.0)
* gchiozzi  03/03/97  Removed include of dbPrivate.h
* gchiozzi  09/01/97  Added const to char*
* rabuter   27/10/95  created
*/

/************************************************************************
* 
*------------------------------------------------------------------------
*/


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _POSIX_SOURCE 1
#include "vltPort.h"

/* 
 * System Headers 
 */
#ifndef _STDLIB_INCLUDED
  #include <stdlib.h>
#endif
#ifndef _STDIO_INCLUDED
  #include <stdio.h>
#endif
#ifndef _STRING_INCLUDED
  #include <string.h>
#endif
#ifndef _STDARG_INCLUDED
  #include <stdarg.h>
#endif



/*
 * Local Headers 
 */

#ifndef err_H
  #include "err.h"
#endif
#ifndef msg_H
  #include "msg.h"
#endif
#ifndef db_H
  #include "db.h"
#endif
#ifndef eccsDB_ATTR_H
  #include "eccsDB_ATTR.h"
#endif
#ifndef eccsTestTools_H
  #include "eccsTestTools.h"
#endif


/****************************/
/*   Classes for TABLES     */
/****************************/


template<class T> class eccsDB_TABLE;


/*
 * Base class for table record classes
 */

class eccsDB_TABLE_RECORD
{
  public:
    virtual ccsCOMPL_STAT Pack( void * buffer )=0;
    virtual ccsCOMPL_STAT Unpack( void * buffer )=0;
};


/*
 * Support template class used to handle tables
 */

template<class T>
struct eccsDB_TABLE_HELPER
{
    T                 value;
    vltUINT32         index;
    const char        *name;
    eccsDB_TABLE<T>   *ptr;
    
    eccsDB_TABLE_HELPER& operator =(T newVal) {
	if(name != NULL)
	    ptr->SetValue(name,newVal);
	else
	    ptr->SetValue(index,newVal);
	return *this;
    }
    operator const T&() { 
	if(name != NULL)
	    ptr->GetValue(name,&value);
	else
	    ptr->GetValue(index,&value);
	return value; 
    }
};

/*
 *   Template class for handling tables
 */


template<class T>
class eccsDB_TABLE  : public eccsDB_ATTR
{
    friend class eccsDB_TABLE_HELPER<T>;

  public:
    eccsDB_TABLE(const dbSYMADDRESS pointName, 
		  const char *attrName = dbEMPTY);

    virtual ccsCOMPL_STAT GetValue(T *buffer);
    virtual ccsCOMPL_STAT SetValue(T *buffer);
    virtual ccsCOMPL_STAT GetValue(vltUINT32 index, T *buffer);
    virtual ccsCOMPL_STAT SetValue(vltUINT32 index, T &buffer);
    virtual ccsCOMPL_STAT GetValue(const char *name, T *buffer);
    virtual ccsCOMPL_STAT SetValue(const char *name, T &buffer);

    eccsDB_TABLE_HELPER<T>& operator[](int index) {
	helper.index = index;
	helper.name  = NULL;
	return helper;
    };

    eccsDB_TABLE_HELPER<T>& operator[](const char *name) {
	helper.name  = name;
	return helper;
    };


  protected:
    eccsDB_TABLE_HELPER<T> helper;
};

/* Include code for inline methods */
#include "eccsDB_TABLE.icc"

#endif
