#ifndef eccsDB_VECTOR_H
#define eccsDB_VECTOR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: eccsDB_VECTOR.h,v 1.134 2005/04/08 08:34:15 vltsccm Exp $"
*
* who       when      what
* --------  --------  -----------------------------------------------------------
* gchiozzi 2003-12-09 Use now nice compile time template selection from Alexandrescu
* gchiozzi 2002-06-05 Added class-key to frien declaration (gcc 3.0)
* gchiozzi  31/08/99  Reintroduced const in cast operator.
* rkarban   13/08/99  remove const from cast operator
* rkarban   12/08/99  cast away const-ness newVal in operator=
* gchiozzi  10/06/97  Added handling of table columns as vectors
* gchiozzi  10/06/97  Removed template  eccsDB_VECTOR_BYTES<T>.
* gchiozzi  10/06/97  In eccsDB_VECTOR_HELPER replaced DbXXX XXXValue
* gchiozzi  03/03/97  Removed include of dbPrivate.h
* gchiozzi  15/11/95  Changed macros, cleaned up and made OK on GCC 2.7
* rabuter   27/10/95  Factorize DbReadOne, DbWriteOne, RecordCnt up in eccsDB_ATTR
*                     add error handling, templates access macros, man pages. 
* gchiozzi  18/10/95  created
*/

/************************************************************************
* 
*------------------------------------------------------------------------
*/


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _POSIX_SOURCE 1
#ifndef vltPort_H
  #include "vltPort.h"
#endif


/* 
 * System Headers 
 */
#ifndef _STDLIB_INCLUDED
  #include <stdlib.h>
#endif
#ifndef _STDIO_INCLUDED
  #include <stdio.h>
#endif
#ifndef _STRING_INCLUDED
  #include <string.h>
#endif
#ifndef _STDARG_INCLUDED
  #include <stdarg.h>
#endif


/*
 * Local Headers 
 */

#ifndef err_H
  #include "err.h"
#endif
#ifndef msg_H
  #include "msg.h"
#endif
#ifndef db_H
  #include "db.h"
#endif
#ifndef eccsDB_ATTR_H
  #include "eccsDB_ATTR.h"
#endif
#ifndef eccsTestTools_H
  #include "eccsTestTools.h"
#endif



/************************************/
/*   Support class                  */
/************************************/

/*
 * GCH 2003-12-03
 * Vectors of byte-arrays have to be defined by explicit template
 * specialisation because in operator= I cannot simply assign
 * arrays but I have to explicitly copy them.
 * Actually GCC allows and supports arrays copy, but this is not ANSI
 * standard and for portability it is better to use memcpy.
 * This code uses a nice technique described in Alexandrescu,
 * "Modern C++ Design" to select alternative implementation 
 * of methods at compile time based on a template parameter.
 */
template<class T, bool needsMemcpy> class eccsDB_VECTOR;

template <int v>
struct eccsDB_VECTOR_INT2TYPE
{
    enum { value = v };
};

template<class T, bool needsMemcpy = false>
class eccsDB_VECTOR_HELPER
{
  private:
    void assign(const T &newVal, eccsDB_VECTOR_INT2TYPE<false>)
    { 
	value = const_cast<T &>(newVal);  
    }

    void assign(const T &newVal, eccsDB_VECTOR_INT2TYPE<true>)
    { 
	memcpy(value, newVal, sizeof(value));  
    }

  public:
    T                 value;
    vltUINT32         index;
    eccsDB_VECTOR<T, needsMemcpy> *ptr;
    ccsCOMPL_STAT stat; 
    eccsDB_VECTOR_HELPER& operator =(const T &newVal)
    { 
	assign(newVal, eccsDB_VECTOR_INT2TYPE<needsMemcpy>() );  
	stat = ptr->SetValue(index,value);
        // Error already logged: cannot add anything usefull here
	return *this;
    }

    operator const T&()
    { 
	stat = ptr->GetValue(index,&value);
	// Error already logged: cannot add anything usefull here
	return value; 
    }

};


/*******************************************/
/*   Class for vectors of standard types   */
/*******************************************/

enum eccsDB_VECTOR_TYPE { eccsDB_TBL_COL };

template<class T, bool needsMemcpy = false>
class eccsDB_VECTOR  : public eccsDB_ATTR
{
    friend class eccsDB_VECTOR_HELPER<T>;

  public:
    eccsDB_VECTOR(const dbSYMADDRESS pointName, 
		  const char *attrName = dbEMPTY);
    eccsDB_VECTOR(const eccsDB_VECTOR_TYPE type,
		  const dbSYMADDRESS pointName, 
		  const char *fieldName);
    ~eccsDB_VECTOR();

    virtual ccsCOMPL_STAT GetValue(T *buffer);
    virtual ccsCOMPL_STAT SetValue(T *buffer);

    virtual ccsCOMPL_STAT GetValue(vltUINT32 index, T *v);
    virtual ccsCOMPL_STAT SetValue(vltUINT32 index, T &v);

    eccsDB_VECTOR_HELPER<T, needsMemcpy>& operator[](int index);

  protected:
    eccsDB_VECTOR_HELPER<T, needsMemcpy> helper;

    virtual ccsCOMPL_STAT LoadInfo();

  private:
    char *tblField;
};

/*
 * Access Macros for templates of allowed types
 */
#define eccsDB_VECTOR_vltINT8         eccsDB_VECTOR<vltINT8>    
#define eccsDB_VECTOR_vltUINT8        eccsDB_VECTOR<vltUINT8>   
#define eccsDB_VECTOR_vltINT16        eccsDB_VECTOR<vltINT16>   
#define eccsDB_VECTOR_vltUINT16       eccsDB_VECTOR<vltUINT16>   
#define eccsDB_VECTOR_vltINT32        eccsDB_VECTOR<vltINT32>   
#define eccsDB_VECTOR_vltUINT32       eccsDB_VECTOR<vltUINT32>   
#define eccsDB_VECTOR_vltLOGICAL      eccsDB_VECTOR<vltLOGICAL> 
#define eccsDB_VECTOR_vltDOUBLE       eccsDB_VECTOR<vltDOUBLE>    
#define eccsDB_VECTOR_vltFLOAT        eccsDB_VECTOR<vltFLOAT>     

#define eccsDB_VECTOR_vltBYTES4       eccsDB_VECTOR<vltBYTES4,true>     
#define eccsDB_VECTOR_vltBYTES8       eccsDB_VECTOR<vltBYTES8,true>     
#define eccsDB_VECTOR_vltBYTES12      eccsDB_VECTOR<vltBYTES12,true>    
#define eccsDB_VECTOR_vltBYTES16      eccsDB_VECTOR<vltBYTES16,true>    
#define eccsDB_VECTOR_vltBYTES20      eccsDB_VECTOR<vltBYTES20,true>    
#define eccsDB_VECTOR_vltBYTES32      eccsDB_VECTOR<vltBYTES32,true>    
#define eccsDB_VECTOR_vltBYTES48      eccsDB_VECTOR<vltBYTES48,true>    
#define eccsDB_VECTOR_vltBYTES64      eccsDB_VECTOR<vltBYTES64,true>    
#define eccsDB_VECTOR_vltBYTES80      eccsDB_VECTOR<vltBYTES80,true>    
#define eccsDB_VECTOR_vltBYTES128     eccsDB_VECTOR<vltBYTES128,true>   
#define eccsDB_VECTOR_vltBYTES256     eccsDB_VECTOR<vltBYTES256,true>   

#include "eccsDB_VECTOR.icc"

#endif
