#ifndef eccsDb_H
#define eccsDb_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsDb.h,v 1.134 2005/04/08 08:34:12 vltsccm Exp $" 
*
* eccsDb.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  16/09/94  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef eccs_H
  #include "eccs.h"
#endif

extern "C" {
   #include "db.h"
}


/* General class to handle buffers of a given size */
/* The class does not hide completely the true     */
/* buffer and is responsibility of the user to not */
/* corrupt it and not to go out of boundaries      */

class eccsBUFFER
{
  public:
    eccsBUFFER(int bytes);
   ~eccsBUFFER();
    void *Buffer();
    int   Size() const;
    
    operator const int() const;
    operator void*();

  private:
    void *buffer;
    int   size;
};


/**********************************/
/* General CCS database functions */
/**********************************/

ccsCOMPL_STAT dbExit (       /* Close a specific database connection or all */
  IN  const ccsENVNAME  envName  = dbALL_ENV);
 
ccsCOMPL_STAT dbOpen(         /* Establishes a connection to a database  */
  IN  const ccsENVNAME  envName  = ccsLOCAL_ENV);

const char *eccsBuildSymaddress(const dbSYMADDRESS pointName, const char *attrName);

/*****************************************/
/* CCS database READ and WRITE functions */
/*****************************************/

/* The typedef are automatically used for conversions from-to VLT  */
/* standad types, since more VLT type can be mapped on the same    */
/* C-language basic type                                           */

/* WARNING:  attributes of type:                                   */
/* -------                      vltLOGICAL,                        */
/*                              vltINT8                            */
/*                              vltUINT8                           */
/*           are treated in a special way and not transparently by */
/*           edbRead() / edbWrite() function.                      */
/*           Look forward for special functions                    */
/*           This is due to intrinsical problems in type definition*/
/*           The "clean" solution is in special ATTIBUTE classes   */

/*******************************************************************/
/* edbRead() functions: they are defined using basic C types       */

/* The following overloaded functions are able to read types:      */
/*       Parms:                   Types:                           */
/*         short *                  vltINT16                       */ 
/*         u short *                vltUINT16                      */ 
/*         int   *                  vltINT32                       */ 
/*         u int *                  vltUINT32                      */ 
/*         double *                 vltDOUBLE                      */ 
/*         float *                  vltFLOAT                       */ 
/*         vltPOLAR *               vltPOLAR                       */ 
/*         vltRECTANGULAR *         vltRECTANGULAR                 */ 
/* Types derived by the basic char * type must be handled in a     */
/* special way (unfortunately this includes vltLOGICAL, vltINT8    */
/* and vltUINT8).                                                  */
/* This special handling is necessary in order to have a check     */
/* over buffer sizes and to be symmetrical with write functions    */

ccsCOMPL_STAT dbRead(
         short          *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         unsigned short *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         int            *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         unsigned int   *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         float          *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         double         *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         vltPOLAR       *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbRead(
         vltRECTANGULAR *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

/* This read function uses an   eccsBUFFER   object */
ccsCOMPL_STAT dbRead(
         eccsBUFFER     *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

/* These functions are used to read buffers of type BYTES... */
/* The macro  VLTBYTES(buff[]) can be used to pass only one  */
/* parameter instead of buffer and size.                     */
/* This MUST BE AN ARRAY and cannot be a simple pointer      */

ccsCOMPL_STAT dbRead(
         unsigned char  *value,
	 vltINT32        buffSize,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

/*************************************************************/
/* edbWrite() functions                                      */
/*                                                           */
/* Write functions are more complex than Read, because it is */
/* necessary to know exactly the RTAP type of the data to be */
/* written and not only the buffer sizes                     */

ccsCOMPL_STAT dbWrite(
         short          *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         unsigned short *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         int             *value,
   const dbSYMADDRESS     pointName,
   const char            *attrName  = dbEMPTY,
         ccsERROR        *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         unsigned int    *value,
   const dbSYMADDRESS     pointName,
   const char            *attrName  = dbEMPTY,
         ccsERROR        *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         float           *value,
   const dbSYMADDRESS     pointName,
   const char            *attrName  = dbEMPTY,
         ccsERROR        *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         double         *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         vltPOLAR       *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

ccsCOMPL_STAT dbWrite(
         vltRECTANGULAR *value,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );


#if 0

/* The functions commented out by the #if 0 .... #endif section  */
/* have not been implemented.                                    */
/* The main problem, that makes not trivial their implementation */
/* is that it is mecessary to know the deTYPE of what we want    */
/* to write. In the case of vltBYTES... it is not a problem.     */
/* More complex is the case of general eccsBUFFER. A solution    */
/* could be to query the database, but this has an impact on     */
/* performances.                                                 */
/* Once again: the clean solution is in the ATTRIBUTE class      */

ccsCOMPL_STAT dbWrite(
         eccsBUFFER      *value,
   const dbSYMADDRESS     pointName,
   const char            *attrName  = dbEMPTY,
         ccsERROR        *error     = NULL     );

#endif

ccsCOMPL_STAT dbWrite(
         unsigned char  *value,
	 vltINT32        buffSize,
   const dbSYMADDRESS    pointName,
   const char           *attrName  = dbEMPTY,
         ccsERROR       *error     = NULL     );

/************************************************/
/* CCS Database: macros used for READ and WRITE */
/************************************************/

/* This macro is used for buffers of type vltBYTES....   */
/* It split the parameter in two parameters suitable     */
/* for the specific implementations of the dbRead() and  */
/* dbWrite() functions: the buffer itself and its size   */

#define VLTBYTES(a)   (a),sizeof(a)

/* Include the code for inline methods */
#include "eccsDb.icc"

#endif /* ! ECCSDB_H */
