#ifndef eccsErr_H
#define eccsErr_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsErr.h,v 1.134 2005/04/08 08:34:12 vltsccm Exp $" 
*
* eccsErr.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2003-12-09 Updated documentation 
* gchiozzi 2003-11-28 Replaced errAdd and related functions with argument counting.
* gchiozzi 2002-06-06 Added isNullPointer() template function and changed macros to compare with NULL
* rkarban  13/08/99  cast NULL in errClassNullPtr macros to (void *)
* gchiozzi  21/07/98  Added errIsReason functions and IsReason methods. 
* gchiozzi  12/06/97  In eccsERROR_CLASS made const all methods.
* gchiozzi  18/10/95  Added handling of "object status"
* gchiozzi  26/07/95  Added new macros to complete the set
* gchiozzi  26/07/95  Macro errClassReturnNullPtr return FAILURE
* gchiozzi  25/05/95  Added new methods in eccsERROR_CLASS
* gchiozzi  15/05/95  Add macro errFlag(). Corrected bug in errReturnGeneric()
* gchiozzi  16/09/94  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef _STDARG_INCLUDED
  #include <stdarg.h>
#endif

#ifndef eccs_H
  #include "eccs.h"
#endif

extern "C" {
   #include "err.h"
}

/************************************************/
/* Mapping of some standard CCS error functions */
/* to use standard error stack                  */
/************************************************/

ccsCOMPL_STAT  errAddArgsCount(vltINT32 nargs, /* TOTAL number of arguments */
			       ccsMODULEID moduleId, 
			       vltINT16 errorNumber, 
			       ccsLOC_ID locId, 
			       ...);

ccsCOMPL_STAT  errAdd_v( vltINT32 nargs,   /* TOTAL number of arguments counting va_pars */
			 ccsMODULEID moduleId, 
			 vltINT16 errorNumber,
			 ccsLOC_ID locId, 
			 va_list va_parList);

ccsCOMPL_STAT  errSysAddArgsCount(vltINT32 nargs, /* TOTAL number of arguments */
				  ccsMODULEID moduleId, 
				  vltINT16 errorNumber, 
				  ccsLOC_ID locId,
				  char *sysModule=modRTAP, 
				  ...);

ccsCOMPL_STAT  errSysAdd_v( vltINT32 nargs,   /* TOTAL number of arguments counting va_pars */
			    ccsMODULEID moduleId, 
			    vltINT16 errorNumber, 
			    ccsLOC_ID locId, 
			    char *sysModule, 
			    va_list va_parList);

ccsCOMPL_STAT  errCloseStack();
ccsCOMPL_STAT  errResetStack();
ccsCOMPL_STAT  errPrint();
ccsCOMPL_STAT  errDisplay(vltLOGICAL syncFlag=FALSE);

ccsCOMPL_STAT  errIsReason(const ccsMODULEID moduleId,
			   vltINT16   errorNumber,
			   vltLOGICAL *errFound,
			   ccsSTACK_ELEM **errDescription = NULL);

/***************************************************************/
/* Defines for checking error handling functions               */
/* inside error classes: ccsERROR and eccsERROR_CLASS          */
/***************************************************************/

#define ErrAdd(x...) \
        ErrAddArgsCount (__errformatguard_counter(x)+1, ## x)


#define ErrSysAdd(x...) \
        ErrSysAddArgsCount (__errformatguard_counter(x)+1, ## x)

/***************************************************************/
/* Class eccsERROR for error handling.                         */
/* It is a subclass of ccsERROR, the standard CCS error class  */
/***************************************************************/

struct eccsERROR: public ccsERROR
{
    eccsERROR();
    ~eccsERROR();
    
    ccsCOMPL_STAT  ErrAddArgsCount(vltINT32 nargs, /* TOTAL number of arguments */
				   ccsMODULEID moduleId, 
				   vltINT16 errorNumber, 
				   ccsLOC_ID locId, 
				   ...);
    
    ccsCOMPL_STAT  ErrAdd_v(vltINT32 nargs, /* TOTAL number of arguments counting va_pars */
			    ccsMODULEID moduleId, 
			    vltINT16 errorNumber, 
			    ccsLOC_ID locId, 
			    va_list va_parList);
    
    ccsCOMPL_STAT  ErrSysAddArgsCount(vltINT32 nargs, /* TOTAL number of arguments */
				      ccsMODULEID moduleId, 
				      vltINT16 errorNumber, 
				      ccsLOC_ID locId, 
				      char *sysModule=modRTAP, 
				      ...);
    
    ccsCOMPL_STAT  ErrSysAdd_v(vltINT32 nargs, /* TOTAL number of arguments counting va_pars */
			       ccsMODULEID moduleId, 
			       vltINT16 errorNumber,
			       ccsLOC_ID locId, 
			       char *sysModule, 
			       va_list va_parList);

    ccsCOMPL_STAT  Close();
    ccsCOMPL_STAT  Display(vltLOGICAL syncFlag=FALSE);
    ccsCOMPL_STAT  Print();
    ccsCOMPL_STAT  Reset();
    
    ccsCOMPL_STAT  IsReason(const ccsMODULEID moduleId,
			    vltINT16          errorNumber,
			    vltLOGICAL       *errFound,
			    ccsSTACK_ELEM   **errDescription = NULL);
    
    vltLOGICAL     IsReason(const ccsMODULEID moduleId,
			    vltINT16   errorNumber);
    
    eccsERROR &operator =(const ccsERROR &e);
    
    /* This static section is used to handle default eccs error stack */
    static void DefErrorStack(eccsERROR *err = NULL);
};



/********************************************************************/
/* Class eccsERROR_CLASS for objects than can have error conditions */
/* This has to be used as a base class                              */
/********************************************************************/

class eccsERROR_CLASS
{
  public:

    eccsERROR_CLASS(eccsERROR *s = stdErr);

    ccsCOMPL_STAT  ErrStatus() const;
    eccsERROR     &ErrStack()  const;

    ccsCOMPL_STAT  ErrStackClose() const;
    ccsCOMPL_STAT  ErrStackReset() const;

    ccsCOMPL_STAT  ObjStatus() const;

    operator ccsCOMPL_STAT() const;
    operator eccsERROR &() const;

  protected:
    const eccsERROR_CLASS &ErrStatus(const ccsCOMPL_STAT s) const;
    const eccsERROR_CLASS &ErrReset() const;
    const eccsERROR_CLASS &ObjStatus(const ccsCOMPL_STAT s) const;

    const eccsERROR_CLASS &operator =(const ccsCOMPL_STAT s) const;

    ccsCOMPL_STAT  ErrAddArgsCount(vltINT32 nargs,  /* TOTAL number of arguments */
				   ccsMODULEID moduleId, 
				   vltINT16 errorNumber, 
				   ccsLOC_ID locId, 
				   ...) const;

    ccsCOMPL_STAT  ErrAdd_v(vltINT32 nargs, /* TOTAL number of arguments counting va_pars */
			    ccsMODULEID moduleId, 
			    vltINT16 errorNumber, 
			    ccsLOC_ID locId, 
			    va_list va_parList) const;

    ccsCOMPL_STAT  ErrSysAddArgsCount(vltINT32 nargs,  /* TOTAL number of arguments */
				      ccsMODULEID moduleId, 
				      vltINT16 errorNumber, 
				      ccsLOC_ID locId, 
				      char *sysModule=modRTAP, 
				      ...) const;

    ccsCOMPL_STAT  ErrSysAdd_v(vltINT32 nargs, /* TOTAL number of arguments counting va_pars */
			       ccsMODULEID moduleId, 
			       vltINT16 errorNumber,
			       ccsLOC_ID locId, 
			       char *sysModule, 
			       va_list va_parList) const;

  private:

    ccsCOMPL_STAT   status;
    ccsCOMPL_STAT   objStatus;
    eccsERROR      *stack;
};


/*******************************************************/
/* This macros makes easier handling error conditions  */
/*******************************************************/

#define errFlag(a,b)         { (a) = ((b)==FAILURE) ? FAILURE : (a); }


#define errAddGeneric(a)     if( (a) == FAILURE )                  \
                                {                                  \
                                errAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
                                }					   
#define errReturnGeneric(a)  if( (a) == FAILURE )                  \
                                {                                  \
                                errAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
                                return(FAILURE);                   \
                                }
#define errExitGeneric(a)    if( (a) == FAILURE )                  \
                                {                                  \
                                errAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
 	                        errCloseStack(stdErr);             \
                                exit(1);                           \
                                }               

#define errClassAddGeneric(a) if( (a) == FAILURE )                 \
                                {                                  \
                                ErrAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
                                }					   
#define errClassReturnGeneric(a)  if( (a) == FAILURE )             \
                                {                                  \
                                ErrAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
                                return(FAILURE);                   \
                                }
#define errClassExitGeneric(a) if( (a) == FAILURE )                \
                                {                                  \
                                ErrAdd(stdErr,"eccs",              \
                                       eccsERR_GENERIC,            \
                                       __FILE_LINE__,"");          \
 	                        ErrStackClose(stdErr);             \
                                exit(1);                           \
                                }               

/**
 * This template function compares a pointer to
 * any type with a NULL "of the same type"
 * gcc 2.95, 2.96 and 3 behave all in a different
 * way when comparing pointers to objects and object methods
 * with a NULL and this solution works on all 3 versions
 * of the compiler
 */
template <class T> bool isNullPointer(T f) 
{ 
 T   nullPointer = NULL; 

 return (f == nullPointer); 
} 

#define errNullPtr(a)       if( isNullPointer((a)) )               \
                                {                                  \
                                errAdd(stdErr,"eccs",              \
                                       eccsERR_NULL_PTR,           \
                                       __FILE_LINE__,"",#a);       \
                                }

#define errReturnNullPtr(a) if( isNullPointer((a)) )               \
                                {                                  \
                                errAdd(stdErr,"eccs",              \
                                       eccsERR_NULL_PTR,           \
                                       __FILE_LINE__,"",#a);       \
                                return(FAILURE);                   \
                                }

#define errClassNullPtr(a)       if( isNullPointer((a)) )          \
                                  {                                \
 	                           ErrAdd("eccs",                  \
                                         eccsERR_NULL_PTR,         \
                                         __FILE_LINE__,#a);        \
                                  }				   

#define errClassReturnNullPtr(a) if( isNullPointer((a)) )          \
                                  {                                \
 	                           ErrAdd("eccs",                  \
                                         eccsERR_NULL_PTR,         \
                                         __FILE_LINE__,#a);        \
                                   return(FAILURE);                \
                                  }


/* Include the code for inline methods */
#include "eccsErr.icc"

#endif /* ! ECCSERR_H */
