#ifndef eccsEvt_H
#define eccsEvt_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsEvt.h,v 1.134 2005/04/08 08:34:13 vltsccm Exp $" 
*
* eccsEvt.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  22/08/97  Removed LogString(), added TraceMsgString()
* gchiozzi  28/04/97  evtdict now a pointer.
* vltsw  23/07/96  Derived public virtual from eccsERROR_CLASS
* gchiozzi  05/01/96  LogString is a protected method
* rabuter   20/11/95  added LogString 
* gchiozzi  04/01/95  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef eccs_H
  #include "eccs.h"
#endif
#ifndef eccsMsg_H
  #include "eccsMsg.h"
#endif

extern "C" {
   #include "evt.h"
}

#include "eccsEvtIdMap.h"

#define  evtEVENT_CMD "EVENT"


class  evtEVENT: virtual public eccsERROR_CLASS  {

  public:
    
    evtEVENT(const dbSYMADDRESS attr = NULL, int attach = FALSE);
    evtEVENT(const evtEVENT &source);
    virtual ~evtEVENT();

    virtual int           Status(); // TRUE if attached, FALSE if not
    virtual ccsCOMPL_STAT Attach();
    virtual ccsCOMPL_STAT Detach();
    virtual ccsCOMPL_STAT Disable();
    virtual ccsCOMPL_STAT Enable();

     /* Methods to access safely the values of parameters */
     const char         *AttrName() const;
     evtEVENT           &AttrName(const dbSYMADDRESS newName);

     evtFILTER          Filter() const;
     evtEVENT           &Filter(evtFILTER newFilter);

     const char         *Env() const;
     evtEVENT           &Env(const ccsENVNAME newEnv);

     const char         *Proc() const;
     evtEVENT           &Proc(const ccsPROCNAME newProc);

     const char         *UserMsg() const;
     evtEVENT           &UserMsg(const ccsPROCNAME newUserMsg);

     const evtEVENT_ID  &EventId() const;

     msgCMDID            CommandId() const throw(vltINT16) ;

     evtEVENT &operator =(const evtEVENT &e);

   private:

    dbSYMADDRESS        attrName;  /* Attribute name                       */
    evtFILTER           filter;    /* Condition to generate the event      */
    ccsENVNAME          env;       /* Event receiver process environment   */
    ccsPROCNAME         proc;      /* Event receiver process               */
    char               *userMsg;   /* User message attached to event       */
    evtEVENT_ID         eventId;   /* Returned event identifier            */

    friend class evtEVENT_MSG;
    friend class evtEVENT_INIT;

    static evtID_MAP *evtdict;

    void Init();
    void Status(int value); // TRUE if attached, FALSE if not

    
}; /* end class evtEVENT */


class  evtEVENT_MSG: public msgRAW_MESSAGE {
  public:
     evtEVENT_MSG(const msgHEADER *msg = NULL);
     evtEVENT_MSG(const evtEVENT_MSG &source);
    ~evtEVENT_MSG();

     ccsCOMPL_STAT Parse(const msgHEADER *msg);
     ccsCOMPL_STAT Receive(msgTIMEOUT              timeout = msgNO_TIMEOUT,
			   const msgRECEIVEFILTER *filter  = NULL);

     /* Methods to access safely the values of parameters */
     vltUINT8            Type() const;
     const char         *Command() const;
     msgCMDID            CommandId() const;

     const evtEVENT_ID  &EventId() const;
     vltUINT8            Trigger() const;
     const msgPROCESSID &TriggerProc() const;
     const char         *AttrName() const;
     dbATTRTYPE          AttrType() const;
     const evtDATA      &EventInfo() const;
     const char         *UserMsg() const;

     evtEVENT_MSG &operator =(const evtEVENT_MSG &e);

  protected:
    const char         *TraceMsgString(char *buf, vltLOGICAL recv=TRUE) const;

  private:

    evtEVENT_ID    event;         /* Event Identifier                     */
    vltUINT8       trigger;       /* Event triggering condition           */
    msgPROCESSID   triggerProc;   /* Process that triggered the event     */
    dbSYMADDRESS   attrName;      /* DB Element where the event occurred  */
    dbATTRTYPE     attrType;      /* Attribute Type                       */
    evtDATA        eventInfo;     /* returned event information           */
    char          *userMsg;       /* Attached user message, if any        */
     
    void Init();

}; /* end class evtEVENT_MSG */

/* Include the code for inline methods */
#include "eccsEvt.icc"

#endif /* ! ECCSEVT_H */
