#ifndef eccsLog_H
#define eccsLog_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsLog.h,v 1.134 2005/04/08 08:34:13 vltsccm Exp $" 
*
* eccsLog.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2003-11-28 Added __attribute__ check for printf-like formatting
* mcomin  08/04/98  Modification to ParArray class
* tcsmgr  25/03/98  Created logFITS class
* gchiozzi  16/09/94  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef eccs_H
  #include "eccs.h"
#endif

extern "C" {
   #include "log.h"
}

ccsCOMPL_STAT logData(ccsMODULEID mod, const char *logText, ...)
    __attribute__ ((format (printf, 2, 3)));

ccsCOMPL_STAT logData_v(ccsMODULEID mod,const char *logText,
			va_list va_parList);

/* 
 * Basic FITS logging class 
 */
class  logFITS: virtual public eccsERROR_CLASS {

  public:
    logFITS(const ccsMODULEID   module   = "",     /* Module reporting the log.          */
	    const logDICTIONARY fitsDict = "");    /* Dictionary where to check the log. */

    virtual ~logFITS() {};

    logFITS       &Module(const ccsMODULEID module);
    const char    *Module();

    logFITS       &FitsDict(const logDICTIONARY mask);
    const char    *FitsDict();

    logFITS       &Level(vltINT32 level);
    vltINT32       Level();

    static void           Mask(const logMASK mask);
    static const char    *Mask();

    /* 
     * FITS logging methods
     */
    virtual ccsCOMPL_STAT Comment (const char *who,             /* Who generated the log         */
				   const char *text,            /* String describing the comment */
				   vltINT32    level = 1);
    virtual ccsCOMPL_STAT Event(logFITS_TYPE  fitsType,         /* Log type : FITS_UNFORSEEN, FITS_RECOVERY */
				const char   *text,            /* String describing the condition          */
				vltINT32      level = 1);
    virtual ccsCOMPL_STAT Action(const logCATEGORY fitsCat,    /* Category                          */
				 const logFITS_KEY fitsSys,    /* System : identifies a component   */
				 const logACTION   fitsAction, /* Verb describing the action        */
				 vltINT32          level = 1);

    virtual ccsCOMPL_STAT ParRecord(const logCATEGORY fitsCat,   /* Category                        */
				    const logFITS_KEY fitsSys,   /* System : identifies a component */
				    const logFITS_KEY fitsParam, /* System Parameter                */
				    const vltLOGICAL  value,     /* Current value for Parameter     */
				    vltINT32          level = 1);
    virtual ccsCOMPL_STAT ParRecord(const logCATEGORY fitsCat,   
				    const logFITS_KEY fitsSys,   
				    const logFITS_KEY fitsParam, 
				    const vltINT32    value,     
				    vltINT32          level = 1);
    virtual ccsCOMPL_STAT ParRecord(const logCATEGORY fitsCat,   
				    const logFITS_KEY fitsSys,   
				    const logFITS_KEY fitsParam, 
				    const vltDOUBLE   value,     
				    vltINT32          level = 1);
    virtual ccsCOMPL_STAT ParRecord(const logCATEGORY fitsCat,   
				    const logFITS_KEY fitsSys,   
				    const logFITS_KEY fitsParam, 
				    const char       *value,     
				    vltINT32          level = 1);

    virtual ccsCOMPL_STAT ParRecord(const logCATEGORY    fitsCat,   
				    const logFITS_KEY    fitsSys,   
				    const logFITS_KEY    fitsParam, 
				    const unsigned char *value,     
				    vltINT32             level = 1);

    virtual ccsCOMPL_STAT ParArray(const logCATEGORY fitsCat,   /* Category                           */
				   const logFITS_KEY fitsSys,   /* System : identifies a component    */
				   const logFITS_KEY fitsParam, /* System Parameter                   */
				   vltINT16          first,     /* Index of the first element         */
				   vltINT16          last,      /* Index of the last element          */
				   vltINT16          elemSize,  /* Array element size                 */
				   const void       *array,     /* Pointer to the first element       */
				   vltINT32          level = 1);
    
  protected:

  private:    
    ccsMODULEID    module;     /* Module reporting the log.          */
    logDICTIONARY  fitsDict;   /* Dictionary where to check the log. */
    vltINT32       level;      /* Current logging level.             */

    static logMASK mask;       /* Mask used for the FITS logs.       */

}; /* end class logFITS */

/* Include the code for inline methods */
#include "eccsLog.icc"

#endif /* ! ECCSLOG_H */
