#ifndef eccsMsg_H
#define eccsMsg_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsMsg.h,v 1.134 2005/04/08 08:34:13 vltsccm Exp $" 
*
* eccsMsg.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2000-05-24 Removed const from ccsERROR *replyErrorStack;
* gchiozzi  22/08/97  Added to OrgName() long/short flag
* gchiozzi  22/08/97  Changed Trace() functionality
* gchiozzi  24/09/96  Added replyErrorStack member
* gchiozzi  24/09/96  Added emptyErrorStack member
* gchiozzi  04/09/96  Major changes to adapt to new error system
* vltsw  23/07/96  Derived public virtual from eccsERROR_CLASS
* gchiozzi  05/01/96  LogString is a protected method
* gchiozzi  07/07/95  Added class handling of command send checking
* gchiozzi  12/06/95  Added defines for "null" command
* gchiozzi  16/09/94  created
* gchiozzi  05/01/94  added the new class msgROUGH_MESSAGE
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*                     Renamed class msgROUGH_MESSAGE in msgRAW_MESSAGE
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef eccs_H
  #include "eccs.h"
#endif

#ifndef fndNAME_MAP_H
  #include "fndNAME_MAP.h"
#endif

extern "C" {
   #include "msg.h"
}

/********************************/
/* CCS message system functions */
/********************************/

/* Defines for a "null" command */
#define msgNULL_CMD        ""
#define msgNULL_CMD_LEN    (strlen(NULL_CMD)+1)


/* Basic message class */
class  msgRAW_MESSAGE: virtual public eccsERROR_CLASS {

  public:
    virtual ~msgRAW_MESSAGE() {};

    /* Methods to access safely the values of parameters */
    virtual vltUINT8            Type() const      = 0;
    virtual msgCMDID            CommandId() const = 0;
    virtual const char         *Command() const   = 0;

    const char                 *TypeName() const;

    static ccsCOMPL_STAT  Trace(vltLOGICAL mode = TRUE);
    static fndNAME_MAP   &MsgTypeMap();

  protected:
    virtual ccsCOMPL_STAT    TraceMsg(vltLOGICAL recv=TRUE) const;
    virtual const char      *TraceMsgString(char *buf, vltLOGICAL recv=TRUE) const;

    static vltLOGICAL        TraceStatus();

  private:    
    static vltLOGICAL       trace;

    static fndNAME_AND_INDEX msgTypeNames[];
    static fndNAME_MAP       msgTypeMap;

}; /* end class msgRAW_MESSAGE */



class  msgMESSAGE: public msgRAW_MESSAGE {
  public:
    msgMESSAGE();
    msgMESSAGE(const msgHEADER *msg);
    msgMESSAGE(const msgMESSAGE &source);
    ~msgMESSAGE();

    ccsCOMPL_STAT Parse(const msgHEADER *msg);
    ccsCOMPL_STAT Receive(msgTIMEOUT              timeout = msgNO_TIMEOUT,
			  const msgRECEIVEFILTER *filter  = NULL);

    ccsCOMPL_STAT SendCommand();
    ccsCOMPL_STAT SendCommand(msgCHECKFLAG flag);

    ccsCOMPL_STAT SendReply(); 
    ccsCOMPL_STAT SendReply(const char *newBuffer, 
			    msgLENGTH   newBuflen=0,
			    vltLOGICAL  lastReply=ccsTRUE);
    ccsCOMPL_STAT SendReply(const ccsERROR &errmsg);


    /* Methods to access safely the values of parameters */
    vltUINT8            Type() const;
    msgMESSAGE         &Type(vltUINT8 newType);

    const char         *Command() const;
    msgMESSAGE         &Command(const msgCMD newCommand);

    msgCMDID            CommandId() const;
    msgMESSAGE         &CommandId(msgCMDID newCommandId);

    const char         *Destenv() const;
    msgMESSAGE         &Destenv(const ccsENVNAME newDestenv);

    const char         *Destproc() const;
    msgMESSAGE         &Destproc(const ccsPROCNAME newDestProc);

    const msgPROCESSID &OrgId() const;
    msgMESSAGE         &OrgId(const  msgPROCESSID &newOrgId);

    const char         *OrgName(vltLOGICAL longForm=TRUE) const;

    vltLOGICAL          LastReply() const;
    msgMESSAGE         &LastReply(vltLOGICAL newLastReply);

    const char         *Buffer() const;
    msgLENGTH           Buflen() const;
    msgMESSAGE         &Buffer(const char *newBuffer, msgLENGTH newBuflen=0);

    ccsERROR           &Errmsg();
    msgMESSAGE         &Errmsg(const ccsERROR &newErrmsg);

    msgCHECKFLAG        Check() const;
    msgMESSAGE         &Check(msgCHECKFLAG flag);

    msgMESSAGE         &operator =(const msgMESSAGE &source);

    static  ccsCOMPL_STAT GlobalCheckOn(msgCHECKFLAG flag);
    static  ccsCOMPL_STAT GlobalCheckOff();
    
  protected:
    ccsCOMPL_STAT       SetBuffer(const char *buffer,msgLENGTH newlen);

    const char         *TraceMsgString(char *buf, vltLOGICAL recv=TRUE) const;
    ccsCOMPL_STAT       TraceReply() const;

  private:
    vltUINT8     type;
    msgCMD       command;
    msgCMDID     commandId;
    ccsENVNAME   destenv;
    ccsPROCNAME  destproc;
    msgPROCESSID orgId;
    vltLOGICAL   lastReply;
    char        *buffer;
    msgLENGTH    buflen;

    ccsERROR *replyErrorStack;

    msgCHECKFLAG check;

    msgLENGTH    fullBuflen;
    vltLOGICAL   autoCommandId;

    static int            cmdProg;
    static eccsERROR      emptyErrorStack;
    static vltLOGICAL     globalCheck;
    static msgCHECKFLAG   globalCheckBitmask;

}; /* end class msgMESSAGE */


ccsCOMPL_STAT msgParseMsg(msgMESSAGE            *msg,
			  msgHEADER             *input);

ccsCOMPL_STAT msgRecvMsg(msgMESSAGE             *msg,
			 msgTIMEOUT              timeout = msgNO_TIMEOUT,
			 const msgRECEIVEFILTER *filter  = NULL);
			  
ccsCOMPL_STAT msgRecvMsg(msgHEADER             **msg,
			 msgTIMEOUT              timeout = msgNO_TIMEOUT,
			 const msgRECEIVEFILTER *filter  = NULL);
			  
ccsCOMPL_STAT msgSendCommand(msgMESSAGE         *msg);

ccsCOMPL_STAT msgSendCommand(msgMESSAGE         *msg,
			     msgCHECKFLAG        flag);

ccsCOMPL_STAT msgSendReply(msgMESSAGE           *msg);



/* Include the code for inline methods */
#include "eccsMsg.icc"


#endif /* ! ECCSMSG_H */
