#ifndef eccsTestTools_H
#define eccsTestTools_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: eccsTestTools.h,v 1.134 2005/04/08 08:34:14 vltsccm Exp $" 
*
* eccsTestTools.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2003-12-03 Moved instantiation of eccsEmptyLog in .C
* gchiozzi 2003-11-28 Added __attribute__ directive to check printf-like format 
* gchiozzi  03/01/00  Added default value 0 for eccsLOG_LEVEL define
* gchiozzi  18/08/95  In eccsErrLog() replaced ResetStack() with CloseStack()
* gchiozzi  04/05/95  Added macros for debugging logs
* gchiozzi  18/10/94  created
* gchiozzi  18/01/95  cleanup and changed file names to be compliant with
*                     VLT rules. Moved inline code in .icc files
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/* Tools for developing test programs */

#ifndef eccs_H
  #include "eccs.h"
#endif
#ifndef eccsLog_H
  #include "eccsLog.h"
#endif
#ifndef eccsErr_H
  #include "eccsErr.h"
#endif

/*
 * Empty log (i.e. "") defined as a constant string
 * to avoid that the eccsLogComment printf style check complains
 * about receiving an empty format
 */
extern const char *eccsEmptyLog;

ccsCOMPL_STAT eccsLogComment(const char *logstr, ...)
    __attribute__ ((format (printf, 1, 2)));

ccsCOMPL_STAT eccsLogError(const ccsERROR *error_p, 	
			   const char * errstr, ...)
    __attribute__ ((format (printf, 2, 3)));

void eccsLogSave(const int value);

/*******************************************************/
/* These macros makes easier handling error conditions  */
/*******************************************************/

#define eccsErrExit(a,b)     if( (a) == FAILURE )                  \
                                {                                  \
                                eccsLogError(stdErr, (b));         \
 	                        errCloseStack(stdErr);             \
                                exit(1);                           \
                                }               

#define eccsErrLog(a,b)      if( (a) == FAILURE )                  \
                                {                                  \
                                eccsLogError(stdErr, (b));         \
 	                        errCloseStack(stdErr);             \
                                }               

/*************************************************************************/
/* These macros can be used to log messages (on standard output AND/OR   */
/* on the log system) depending on the value of a control variable:      */
/*                      eccsLOG_LEVEL (between 0 to 3)                   */
/* if it is not defined or if its value is 0, only level 0 are logged    */
/*************************************************************************/

#define eccsLOG_0(args)  eccsLogComment args
#define eccsLOG_1(args)
#define eccsLOG_2(args)
#define eccsLOG_3(args)

#ifndef eccsLOG_LEVEL
#define eccsLOG_LEVEL 0
#endif

#if eccsLOG_LEVEL >= 1
     #undef  eccsLOG_1
     #define eccsLOG_1(args)  eccsLogComment args
#endif
#if eccsLOG_LEVEL >= 2
     #undef  eccsLOG_2
     #define eccsLOG_2(args)  eccsLogComment args
#endif
#if eccsLOG_LEVEL >= 3
     #undef  eccsLOG_3
     #define eccsLOG_3(args)  eccsLogComment args
#endif

/* Include the code for inline methods */
#include "eccsTestTools.icc"


#endif /* ! ECCSTESTTOOLS_H */
