/****************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: err.h,v 2.36 2005/07/27 12:48:47 vltsccm Exp $" 
*
*
* <err.h> - CCS ERROR SYSTEM Interface File
*
* who        when      what
* ---------  --------  ----------------------------------------------
* B.GILLI    29/04/93  Preliminary version
* B.GILLI    27/05/93  for PDR
* B.GILLI    08/07/93  add param to routine errAdd()
*                      suppress param from routine errLogError()
*                      vlt types definitions introduced.
* B.GILLI    20/07/93  added    loc_id to   routine errAdd(), 
*                      suppress loc_id from errLogError().
*                      logLOC_ID is now a ccsLOC_ID.
*                      param increased to 256 bytes.
* M.COMIN    16/08/93  Modified errAdd() to include a variable number of
*                      run time parameters.
*
* M.COMIN    16/08/93  Add errSysAdd() to log system errors.
* M.COMIN    26/10/93  Add routine errConvRtapErr()
*
* M.COMIN    10/11/93  Change type of second parameter from msgHEADER 
*                      to msgMSG
*
* M.COMIN    12/01/94  Modifications for CCS 1.0
*                      - errDisplay() replaced with errCloseStack
*
* M.COMIN    28/09/94  taken away "char locId[100]"
*
* M.COMIN    27/10/94  Add routine errDisplay, errSetProcInfo
* M.COMIN    17/11/94  Add errPrint* 
* B.GUSTAFSSON 28/11/94 Including specific CCS/LCC include files
* B.GUSTAFSSON 01/12/94 Added C++ support
* ---------------------
* M.COMIN    11/01/95   Add prototypes for errAdd_v() and errSysAdd_v()
* ---------------------
* M.COMIN    13/03/96   In errERRSTRT_REPLY_HEADER changed type of errorNumber
*                       from vltUINT16 to vltINT16
* M.COMIN    19/03/96   Add errPack/errUnPack routines
*            30/04/96   Modification to errGetFromStack and errGetSize
*
* B.GILLI    08/04/03   Added errPrintStderr SPR 20030100
*****************************************************************************/

#ifndef ERR_H
#define ERR_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * header files
 */

#include <stdarg.h>
#include "ccs.h"
#include "msg.h"

/*
 ************************************ 
 *  Error System  definitions       *
 ************************************
 */
/* 
 * Constants 
 */

#define errPARAM_DELIM   ","
#define errTOP    "top"
#define errBOTTOM "bottom"

/*
 * Message formats
 */

typedef struct {                    /* Reply for ERRSTRT command        */
    ccsTIMEVAL      timeStamp;      /* Time stamp (UTC) of the error    */
    ccsSTACK_ID     stackId;        /* Error stack identifier           */
    vltUINT16       seqNumber;      /* Sequence number in the stack	*/
    ccsPROCNAME     processName;    /* Name of process that found error */
    ccsMODULEID     moduleId;       /* Name of module that found error  */
    ccsLOC_ID       locId;	    /* Name of function where error was */
                                    /* found				*/
    vltINT16        errorNumber;    /* Error number			*/
    ccsPROCNUM      procNum;	    /* Procnum of process reporting error */
    vltUINT8        fill[1];        /* Filling characters to have same  */
                                    /* structure length on WS and LCU   */
} errERRSTRT_REPLY_HEADER;


#define __errformatguard_counter(y...) __errformatguard_count1 ( , ##y)

#define __errformatguard_count1(y...) __errformatguard_count2 (y, 99,98,97,96,95,94,93,92,91,90,89,88,87, 86,85,84,83,82,81,80,79,78,77,76,75,74,73,72, 71,70,69,68,67,66,65,64,63,62,61,60,59,58,57, 56,55,54,53,52,51,50,49,48,47,46,45,44,43,42, 41,40,39,38,37,36,35,34,33,32,31,30,29,28,27, 26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0)

#define __errformatguard_count2(_,x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12, x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23, x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34, x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45, x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56, x57,x58,x59,x60,x61,x62,x63,x64,x65,x66,x67, x68,x69,x70,x71,x72,x73,x74,x75,x76,x77,x78, x79,x80,x81,x82,x83,x84,x85,x86,x87,x88,x89, x90,x91,x92,x93,x94,x95,x96,x97,x98, n,ys...) n


/*
 * These macros add in from of the call one parameter
 * with the total number of arguments passed, including
 * the added argument itself.
 */

#define errAdd(x...) \
        errAddArgsCount (__errformatguard_counter(x)+1, ## x)


#define errSysAdd(x...) \
        errSysAddArgsCount (__errformatguard_counter(x)+1, ## x)




#ifdef errLOCAL_TEST

/* Here for backwards compatibility during the first test ... to be removed */

ccsCOMPL_STAT errAddProtected ( 
    const char        *fn,         /* Function name */ 
    vltINT32           nargs,      /* Number of arguments */
    ccsERROR          *error,
    const ccsMODULEID  moduleId,
    vltINT16           errorNumber,
    const ccsLOC_ID    locId, 
    char              *format,
    ...
    );

ccsCOMPL_STAT errSysAddProtected ( 
    const char        *fn,         /* Function name */ 
    vltINT32           nargs,      /* Number of arguments */
    ccsERROR          *error,
    const ccsMODULEID  moduleId,
    vltINT16           errorNumber,
    const ccsLOC_ID    locId, 
    char              *format,
    char              *sysModule,
    ... );

#endif



ccsCOMPL_STAT errAdd_v ( 
    vltINT32             nargs,  /* TOTAL Number of arguments */
    ccsERROR            *error,
    const ccsMODULEID    moduleId,
    vltINT16             errorNumber,                       
    const ccsLOC_ID      locId,
    char                *format,
    va_list va_parList
    );


ccsCOMPL_STAT errAddArgsCount ( 
    vltINT32           nargs,      /* TOTAL Number of arguments */
    ccsERROR          *error,
    const ccsMODULEID  moduleId,
    vltINT16           errorNumber,
    const ccsLOC_ID    locId, 
    char              *format,
    ...
    );


ccsCOMPL_STAT errSysAdd_v ( 
    vltINT32             nargs,   /* TOTAL Number of arguments */
    ccsERROR            *error,
    const ccsMODULEID    moduleId,
    vltINT16             errorNumber,                       
    const ccsLOC_ID      locId,
    char                *format,
    char                *sysModule,
    va_list va_parList
    );


ccsCOMPL_STAT errSysAddArgsCount ( 
    vltINT32           nargs,      /* TOTAL Number of arguments */
    ccsERROR          *error,
    const ccsMODULEID  moduleId,
    vltINT16           errorNumber,
    const ccsLOC_ID    locId, 
    char              *format,
    char              *sysModule,
    ... );


/*
 *******************************************************************
 * ERROR SYSTEM  : Application  Programmer  Routines               *
 *******************************************************************
 */


/* 
 *  error conditions are propagated at the higher level using the ccsERROR 
 *  structure.  This structure contains information related to :
 *
 *  - error STACK   :  Describes the status of the current error stack.
 *  - error CONTEXT :  Describes the kind of error occurred in the routine.
 */

ccsCOMPL_STAT errClear (        /* Initialize the error structure */
    ccsERROR        *error
    );

void errCopy (                 /* Used to copy error structures */
     ccsERROR *errDest,
     ccsERROR *errSrc 
     );

ccsCOMPL_STAT errMergeStack (  /* Merge the error stack of two structures */
    ccsERROR   *errDest,       /* Destination error stack                 */
    ccsERROR   *errSrc,        /* Source error stack                      */
    char       *flag           /* Defines how to merge : TOP or BOTTOM    */
    );

ccsCOMPL_STAT errPrint (        /* Prints the error structure on the screen */
    ccsERROR        *error
    );


ccsCOMPL_STAT errPrintStderr (  /* Prints the error structure on the stderr */
    ccsERROR        *error
    );


/* Search for a specific error in the stack  */

ccsCOMPL_STAT errIsReason (
   ccsERROR          *error,          /* Error Stack    */
   const ccsMODULEID  moduleId,       /* Module         */       
   vltINT16           errorNumber,    /* Error Number to be sarched   */
   vltLOGICAL        *errFound,       /* Flag indicating if error was found */
   ccsSTACK_ELEM    **errDescription  /* Pointer to error description */
   );

vltLOGICAL errInStack (    /* Check if the stack contains a specific error */
   ccsERROR          *error,          /* Error Stack    */
   const ccsMODULEID  moduleId,       /* Module         */       
   vltINT16           errorNumber     /* Error Number to be sarched   */
   );


/*
 *  The following routines are for internal use of other CCS modules
 */
 
ccsCOMPL_STAT errPutInStack(    /* Stores the error into the stack */
    ccsERROR *error             /* Current error */
   );


ccsCOMPL_STAT errSetStack (    /* Set the local error structure to the  */ 
                               /* error reported by the message system  */
    ccsERROR   *error,         /* The stack to be modified              */
    msgMSG     *message        /* The message containing the new status */
    );


vltINT32 errBuildReply (       /* Build the body of error reply         */ 
    char *msgBody,
    ccsERROR *error            /* Current error */        
    );


ccsCOMPL_STAT errUnPack ( /* Extract the error structure from buffer        */
   ccsERROR  *error,   /* Returned error structure                          */
   char      *buffer,  /* Pointer to beginning of error structure in buffer */
   char      **ptrNext /* Returned pointer to beginning of next structure   */
                       /* in buffer                                         */
   );

ccsCOMPL_STAT errPack (    /* Pack the error information into a buffer    */
   const ccsERROR  *error, /* Error structure to be packed                */
   char      *buffer,   /* Pointer to the 'insert' position in the buffer */
   char      **ptrNext  /* Returned pointer to the next 'insert' position */
   );


/* 
 *  error conditions are propagated at the higher level using the ccsERROR 
 *  structure.  This structure contains information related to :
 *
 *  - error STACK   :  Describes the status of the current error stack.
 *  - error CONTEXT :  Describes the kind of error occurred in the routine.
 */



ccsCOMPL_STAT errResetStack (      /* Reset the error structure  */
                                   /* and initiate a new stack   */
    ccsERROR        *error
    );


ccsCOMPL_STAT  errCloseStack (  /* Logs the error and resets the stack */
   ccsERROR       *error        /* current error structure to be displayed  */
   );

vltINT8 errGetStackSize (       /* Returns the current stack size */
   ccsERROR       *error        /* current error structure to be displayed  */
   );


ccsCOMPL_STAT errGetFromStack ( /* Returns the error structure stored */
			        /* in a given position of the stack   */
    vltINT8    pos,             /* Position in the stack     */
    ccsERROR   *error,          /* Current error structure   */                
    ccsERROR   *errRet          /* Returned error structure  */
   );


#ifdef MAKE_VXWORKS
#include "errLCC.h"
#else
#include "errCCS.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /*!ERR_H*/
