#ifndef evhDB_CMD_SEND_H
#define evhDB_CMD_SEND_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: evhDB_CMD_SEND.h,v 1.135 2005/07/21 12:41:48 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban 2001-02-11 add methods Msg, ResetWaitingId
* gchiozzi  31/07/98  Added proper fndStdObject...() calls
* gchiozzi  12/02/97  Added new ActualSendCommand protected method
* gchiozzi  15/03/96  Added protected access methods for user installed callbacks
* gchiozzi  04/07/95  Added inst. cb for completion, errors and timeout
* gchiozzi  02/05/95  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef db_H
  #include "db.h"
#endif
#ifndef evhDB_TASK_H
  #include "evhDB_TASK.h"
#endif
#ifndef evhDB_COMMAND_H
  #include "evhDB_COMMAND.h"
#endif
#ifndef evhDB_COMMAND_H
  #include "evhDB_COMMAND.h"
#endif

#ifndef fndLIST_H
  #include "fndLIST.h"
#endif
#ifndef evhDB_SEND_ID_H
  #include "evhDB_SEND_ID.h"
#endif

#define evhCMD_POINT "cmd"

typedef classType        evhDB_COMMAND_TYPE;

class evhDB_CMD_SEND : public evhDB_TASK
{
  public:
    /* Static variable used as default when no wait id is requested */
    static evhDB_SEND_ID noWaitId;

    evhDB_CMD_SEND(const dbSYMADDRESS  dbPoint, evhDB_COMMAND_TYPE type);
   ~evhDB_CMD_SEND();

    virtual ccsCOMPL_STAT     Init();
    virtual ccsCOMPL_STAT     Reset();
    virtual ccsCOMPL_STAT     Send(const msgCMD command, 
				   const char *newBuffer, 
				   msgLENGTH newBuflen=0,
				   evhDB_SEND_ID *sendId=NULL, 
				   evhDB_SEND_ID  waitId=
				        evhDB_CMD_SEND::noWaitId);
    virtual ccsCOMPL_STAT     Timeout(int timeout);

    virtual ccsCOMPL_STAT   Completed(evhCALLBACK &cb);
    virtual ccsCOMPL_STAT   Error(evhCALLBACK &cb);
    virtual ccsCOMPL_STAT   Timeout(evhCALLBACK &cb);

    const char *Destination();    

    /* Should not be necessary to have these public: protected should be OK */
    /* I think that here GCC has some problems                              */
    ccsCOMPL_STAT           Register(evhDB_CMD_SEND *who, vltUINT32 id); 
    ccsCOMPL_STAT           UnRegister(evhDB_CMD_SEND *who, vltUINT32 id); 
    virtual ccsCOMPL_STAT   SendCommand(evhDB_SEND_ID &);

    fndStdObjectDef(evhDB_CMD_SEND, evhDB_TASK);

  protected:

    
    virtual ccsCOMPL_STAT     Notify(); 
    ccsCOMPL_STAT             SetCommand(const msgCMD command, 
				 const char *newBuffer, msgLENGTH newBuflen=0);
    virtual ccsCOMPL_STAT             ActualSendCommand();

    /* Hook functions to implement a user defined behaviour on completion */
    virtual evhCB_COMPL_STAT  CompletedCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  ErrorCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  TimeoutCB(msgMESSAGE &msg, void *udata);

    /* Real callbacks. Call hooks and do internal administration */
    virtual evhCB_COMPL_STAT  EventCompletedCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  EventErrorCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  EventTimeoutCB(msgMESSAGE &msg, void *udata);

    virtual evhCALLBACK *CompletedUserInstCB();
    virtual evhCALLBACK *ErrorUserInstCB();
    virtual evhCALLBACK *TimeoutUserInstCB();

    vltUINT32      Id();
    ccsCOMPL_STAT  SetId();
    fndLIST       &WaitingList();
    msgMESSAGE    &Msg();
    void          ResetWaitingId();

    virtual evhDB_COMMAND *Cmd();
    evhDB_COMMAND *NewCmd(const dbSYMADDRESS  dbPoint,evhDB_COMMAND_TYPE type);

    evhDB_CMD_SEND(); // NOT to be used!!!


private:
    evhDB_COMMAND  *cmd;
    vltUINT32        id;

    evhCALLBACK    *completed;
    evhCALLBACK    *error;
    evhCALLBACK    *timeout;

    fndLIST         waitingList;
    msgMESSAGE      msg;
    evhDB_SEND_ID   waitingFor;

    static vltUINT32 lastId;
};


#include "evhDB_CMD_SEND.icc"

#endif /*!evhDB_CMD_SEND_H*/
