#ifndef evhSIGNAL_H
#define evhSIGNAL_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: evhSIGNAL.h,v 1.135 2005/07/21 12:41:47 vltsccm Exp $" 
*
* evhSignal.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* vltsw  23/07/96  Derived public virtual from eccsERROR_CLASS
* jknudstr  09/01/95  created
*/

/************************************************************************
* Contains definition of evhSIGNAL class for the CCS Event Handler 
* module. Used to handle installation and deinstallation of the
* EVH signal handler.
*----------------------------------------------------------------------
*/

#ifndef eccs_H
  #include "eccs.h"
#endif
#ifndef eccsMsg_H
  #include "eccsMsg.h"
#endif


#define evhSIG_DATA_SIZE 64

#define evhSIGNAL_CMD        "EVHSIG"

// The Signal Class to administrate the adding and deleting of 
// signal handlers
class evhSIGNAL: virtual public eccsERROR_CLASS
{
  public:
    evhSIGNAL(int sig);
    virtual ~evhSIGNAL();
    
    // To install the EVH signal handler for the specified signal
    virtual ccsCOMPL_STAT Install();

    // To desintall the EVH signal handler, and to reinstall the
    // default signal handler
    virtual ccsCOMPL_STAT Deinstall();

    // Returns the signal registered for this object
    int  Sig() const;

    // Indicates whether a signal handler is installed for this obejct or not 
    vltLOGICAL Status() const;

    // Builds a 'fake' message if the event was a signal
    static ccsCOMPL_STAT BuildMsg(int sig, msgMSG *signalMsg);

    // Methods to get information about the last signal catched
    // Gets the type of the last handled signal
    static int         LastSignal();
    static void        ResetSignal();
    static const char *SignalData();
    static int         SignalErrStatus();

  private:

    int sig;   // signal handled in this instance

    // This row contains a flag indicating if a signal handler has
    // already been installed for this signal: 
    //    SIG_FREE       = nothing installed,
    //    SIG_REGISTERED = signal handler registerd, but not installed, 
    //    SIG_INSTALLED  = signal handler installed
    enum SIG_STATUS { SIG_FREE, SIG_REGISTERED, SIG_INSTALLED };
    static SIG_STATUS signalsInst[evhNO_OF_SIGNALS];

    // Static variables used for handling the signal
    // and set by the signal handler
    static int   signalHandle;         // Signal id
    static char  signalData[evhSIG_DATA_SIZE];  // Buffer to pass back data
    static int   signalErrStatus;      // Return status from the signal handler

    // The EVH signal handler
    static void SignalHandler(int sig);
};

#endif /*!evhSIGNAL_H*/
