/**************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: evt.h,v 2.53 2005/09/22 21:51:54 vltsccm Exp $" 
*
* <evt.h>  -  CCS/ON-LINE DATABASE EVENT Interface File
*
* who        when      what
* ---------  --------  ----------------------------------------------
* T.Herlin   13/09/94  Created
* T.Herlin   21/09/94  Changed filter type defines to enum evtFILTER
*                      Type changed on 'message' of evtParseMsg.
* T.Herlin   23/09/94  Removed evtPROC_INFO structure and replaced usage
*                      with msgPROCESSID.
* T.Herlin   27/09/94  Added evtAttachExt() and extended evtParseMsg() with
*                      the return of a userMsg.
* T.Herlin   29/09/94  Change evtEVENT to evtEVENT_ID due to LCC compliance.
* T.Herlin   28/10/94  Added LCU event message type
* M.COMIN    21/11/94  Add definitions for error mnemonics
* B.GUSTAFSSON 28/11/94 Including specific CCS/LCC include files
* B.GUSTAFSSON 01/12/94 Added C++ support
* T.Herlin   27/01/94  Changed misleading filter to reserved in evtEVENT_ID.
* B.GUSTAFSSON 06/02/95 Changed evtEVTREQ_REPLY_HEADER to evtEVTREQ_REPLY
*                       Members oldStatus and newStatus of evtEVTREQA_REPLY 
*                       changed to array of 2.
*---------
* gfilippi  05/03/95  do not include dbCCS.h if RTAP is not available.
* T.Herlin  17/11/95  Alignment of evtSCALAR_INFO to 8 byte boundary SPR950530
* bgustafs  24/11/95  Alignment problem of evtEVTREQ_REPLY
* b.Gilli   25/03/96  Modification to handle proper msgProcessId from LCUs
* T.Ebert   29/04/04  Added new filter type evtQUALITY_CHANGES
* 
************************************************************************
*/

#ifndef EVT_H
#define EVT_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  Header Files
 */

#include "ccs.h"
#include "db.h"
#include "msg.h"


/*
 ***********************************************************************
 *  Database Event Generation                                          *
 ***********************************************************************
 */
/*
 * Message constants
 */

#define evtPERIOD_10MS_STRING	"10ms"
#define evtPERIOD_100MS_STRING	"100ms"
#define evtPERIOD_SECOND_STRING	"Sec"

#define evtMSG_PRIORITY 100 /* same as message system */
/* 
 * The event filter mask defines how the event is triggered.  
 * For SCALARS the event can be triggered by comparing old and new values 
 * according to the event mask. 
 *
 * For VECTORS and TABLES an event is triggered at every write
 *
 */
typedef enum {
    evtNO_EVENT = 0,		    /* Not supported on LCU	      */
    evtLESS,	   	            /* Not supported on LCU	      */
    evtEQUAL,			    /* Not supported on LCU	      */
    evtLESS_OR_EQUAL,		    /* Not supported on LCU	      */
    evtGREATER,			    /* Not supported on LCU	      */
    evtNOT_EQUAL,		    /* Generate event if value change */
    evtGREATER_OR_EQUAL,	    /* Not supported on LCU	      */
    evtANY_WRITE,                   /* Generate event on any write    */
    evtDEAD_BAND,                   /* Generate event if value change */
                                    /* more than deadband             */
    evtQUALITY_CHANGE = 16          /* Not supported on LCU           */
} evtFILTER;



typedef struct {
   ccsENVNAME  envName;   /* Environment where the event occurred */
   vltINT32    eventNum;  /* Event Number                         */
   vltUINT8    reserved;  /* Event filter mask (not used on LCU)  */
   vltUINT8    fill[3];   /* padding bytes                        */
} evtEVENT_ID;



typedef struct {
   evtEVENT_ID  eventId;  /* Event Identifier                     */
   ccsPROCNAME  procName; /* Process requesting the event         */
   evtFILTER    filter;   /* Condition to generate the event      */
   vltUINT8     status;   /* Current status : enabled/disabled    */
} evtEVENT_DESCR;      
   


typedef struct {          /* Information returned with an event   */
                          /* associated to a SCALAR attribute     */
    dbTYPE       dataType;
    vltUINT8     oldQuality;
    vltUINT8     newQuality;
    vltUINT8     fill;
    vltUINT32    dblAlign; /* alignment to 8 byte boundary SPR950530 */
    char         oldValue[256];   
    char         newValue[256];  
} evtSCALAR_INFO;   


typedef struct {          /* Information returned with an event   */
                          /* associated to a VECTOR attribute     */
   vltUINT16  startElem;
   vltUINT16  endElem;
   } evtVECTOR_INFO;
   
   

typedef struct {          /* Information returned with an event   */
                          /* associated to a TABLE attribute      */
   vltUINT16  startElem;
   vltUINT16  endElem;
   vltUINT8   startField;
   vltUINT8   endField;
   } evtTABLE_INFO;


typedef union {           /* General structure with the event     */
                          /* information                          */
   evtSCALAR_INFO  scalar;
   evtVECTOR_INFO  vector;
   evtTABLE_INFO   table;
   } evtDATA;

typedef enum {                  /* Definition of alarm status   */
    alarmLOWLOW = 1,
    alarmLOW,
    alarmNORMAL,                /* No alarm                     */
    alarmHIGH,
    alarmHIGHHIGH
} alarmSTATUS;


/*
 * Message formats
 */

typedef struct {                    /* Reply for EVTATT and EVTREQP    */
                                    /* commands                         */
    evtEVENT_ID     eventId;        /* Event identifier                 */
    evtFILTER       trigCond;       /* Condition that generated the     */
                                    /* event                            */
    msgPROCESSID    process;        /* Process that generated the event */
    vltUINT8        fill1[3];
    dbSYMADDRESS    attrName;       /* Name of the attribute that       */
                                    /* generated the event              */
    dbATTRTYPE      type;           /* Attribute type                   */
    vltUINT16       fill;
    evtDATA         eventInfo;      /* Event info                       */
} evtEVTREQ_REPLY;


typedef struct {                    /* Reply to EVTREQA command         */
    evtEVENT_ID     eventId;        /* Event identifier                 */
    evtFILTER       trigCond;       /* Condition that generated event   */
    msgPROCESSID    process;        /* Process that generated the event */
    vltUINT8        fill1[3];
    dbSYMADDRESS    attrName;       /* Name of the attribute for which  */
                                    /* event is generated               */
    dbTYPE          type;           /* Type of attribute                */
    vltUINT16       alarmNumber;    /* Alarm number                     */
    alarmSTATUS     oldStatus[2];   /* Old alarm status                 */
    alarmSTATUS     newStatus[2];   /* New alarm status                 */
    char            oldValue[16];   /* 16 bytes is large enough to hold */
    char            newValue[16];   /* the biggest data types for which */
                                    /* alarms can be requested:         */
                                    /* vltPOLAR or vltRECTANGULAR       */
} evtEVTREQA_REPLY;


/*
 *   Event Interface (Function Prototypes)
 */

ccsCOMPL_STAT evtAttach(           /* Configure the database to generate   */
                                   /* an event when a process accesses a   */
                                   /* specific database attribute.         */
    dbSYMADDRESS        attrName,  /* Attribute name                       */
    evtFILTER           filter,    /* Condition to generate the event      */
    evtEVENT_ID         *eventId,  /* Returned event identifier            */
    ccsERROR            *error     /* returned error structure             */
    );

ccsCOMPL_STAT evtDetach(           /* Remove the event definition          */
     evtEVENT_ID   *eventId,       /* Event identifier                     */
     ccsERROR      *error          /* Returned error structure             */
     );

ccsCOMPL_STAT evtSingleEnable(     /* Enable event notification            */
     evtEVENT_ID   *eventId,       /* Event identifier                     */
     ccsERROR      *error          /* Returned error structure             */
     );

ccsCOMPL_STAT evtSingleDisable(    /* Disable event notification           */
     evtEVENT_ID   *eventId,       /* Event identifier                     */
     ccsERROR      *error          /* Returned error structure             */
     );

ccsCOMPL_STAT evtParseMsg(         /* Parse the message received as a      */
                                   /* consequence of an event notification */
    msgHEADER      *message,       /* Pointer to Message notification      */
    evtEVENT_ID    *event,         /* Event Identifier                     */
    vltUINT8       *trigger,       /* Event triggering condition           */
    msgPROCESSID   *triggerProc,   /* Process that triggered the event     */
    dbSYMADDRESS   attrName,       /* Name of attribute for which event    */
				   /* is generated			   */
    dbATTRTYPE     *attrType,      /* Attribute Type                       */
    evtDATA        *eventInfo,     /* returned event information           */
    char           **userMsg,      /* Attached user message, if any        */
    ccsERROR       *error          /* Returned error structure             */
    );



#ifdef MAKE_VXWORKS
#include "evtLCC.h"
#else

#ifdef CCS_LIGHT
#include "evtLite.h"
#endif

/*
 * normally the following is included, but in a non-RTAP installation (LCU only),
 * the CCS part would produce errors, because it uses RTAP include files
 * Executable that have to run on both cases, like lcc<xx> shall 
 * define the flag RTAP_EXCLUDE
 */
#ifndef RTAP_EXCLUDE
#include "evtCCS.h"
#endif

#endif/* MAKE_VXWORKS */

#ifdef __cplusplus
}
#endif

#endif /*!EVT_H*/











