/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcd.h,v 2.140 2005/10/17 15:16:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* ronalds   20/09/97  changed MAXOUTPUT to MAXOUTCHIP for mode config
*                     since all chips are enabled the same
*
* rdonalds  26/09/97  Extensive modification to fcdCONFIG added dimension
*		      of image pixels and descriptons of serial registers
*
* rdonalds  27/09/97  added PULPO1 and PULPO2 as shutter types
* ccumani   21/10/97  include fcdObj.h deleted
* ccumani   22/10/97  fcdEXP_DRIFT added in fcdEXPTYPE
* rdonalds  02/11/97  Added the definitions required for IPLLX, IPLLY ..
*                     to support sub-windows for image processing
* ccumani   11/11/97  unused macros deleted
* rdonalds  22/04/98  Updated PROCESS and SETUP structures in preparation
*                     for multi step exposures and centroiding
* rdonalds  25/05/98  Added itSync field to setup 
* rdonalds  06/06/98  Added IP_NO_USER_FUN IP_IQE_USER_FUN
* rdonalds  32/07/98  Added double buffer and clock speed attibutes to CONFIG
* rdonalds  05/08/98  Added absolute time fields to mode definition
* rdonalds  27/08/98  Added read noise per port to mode
* rdonalds  28/10/98  Added maxFrames to detector to indicate the max # frames
*                     which are permissible for one exposure (AO)
*                     Added nFrames to Setup for the number of frames to 
*                     acquire (AO)
* rdonalds  27/01/99  Added DET.READ.NFRAM as setup parameter
* abalestr  19/03/99  Added WAIT parameters
* ccumani   08/03/02  definition of FDEBUG corrected (it was generating
*                     a coredump on Linux, if FIERADEBUG env. var.
*                     was not defined
* abalestr  17/07/03  Macro fcdFILENAME_LEN set to slxFILENAME_LEN; also 
*                     fcdMAXLENFILE set to fcdFILENAME_LEN
*                     fileNameUnComp and fileNameComp set accordingly
*                     see SPR no. VLTSW20030275
* abalestr  18/09/03  Added field rampUp to fcdCONFMODE structure
* abalestr  27/10/03  Added keyword DET.EXP.SYNC for external sgnal sync
* abalestr  21/01/04  Added syncMaster field to conf structure
* abalestr  21/01/04  Added macro for max exposure time accepted by Pulpo
* abalestr  02/09/05  Added macro and variable for taking care of open time 
                      when at2 parameter for START is used, i.e. when a local 
		      sync on the shutter open time is required 
*/
/*************************************************************************
*  This file contains all definitions used by external modules using the 
*  FIERA sw
*************************************************************************/

#ifndef FCD_H
#define FCD_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCS & LCC header files
 */
#include "CCS.h"
#include "cai.h"
#include "slx.h"

/*
 * FIERA header files
 */
#include "fcdDbPublic.h"
#include "fcdErrors.h"

/* FIERA module name */
#define fcdMODULE       "fcd"

/* FIERA DEBUG */
#define FDEBUG ((getenv("FIERADEBUG") != 0) ? \
	(strcmp(getenv("FIERADEBUG"),"ON")==0) : 0)

/* useful file name type */
#define fcdFILENAME_LEN slxFILENAME_LEN
typedef char fcdFILENAME[fcdFILENAME_LEN+1];
/* general string length */
#define fcdSTRINGNAME_LEN 128
typedef char fcdSTRINGNAME[fcdSTRINGNAME_LEN+1];

/* max length of a voltage table name */
#define fcdVOLTTABLENAME_LEN 64
typedef char fcdVOLTTABLENAME[fcdVOLTTABLENAME_LEN+1];

/* max length of a sequence name */
#define fcdSEQUENCENAME_LEN 64
typedef char fcdSEQUENCENAME[fcdSEQUENCENAME_LEN+1];

/* max length of a useequence name */
#define fcdUSEQUENCENAME_LEN 64
typedef char fcdUSEQUENCENAME[fcdUSEQUENCENAME_LEN+1];

/* max length of a useequence name */
#define fcdPATTERNNAME_LEN 64
typedef char fcdUPATTERNNAME[fcdPATTERNNAME_LEN+1];

/* maximum number of images which can be generated by one exposure     */
/* one image is generated per window, also possibly one image per chip */
/* this is TBD                                                         */
#define fcdMAX_IMAGES_PER_EXP (fcdMAXWINDOW * fcdMAXCHIPS)

/* Maximum integration time supported by PULPO */
#define FCD_MAX_PULPO_EXPTIME 16777.0

/*
 * Constants
 */
#define fcdENV_LENV       "CCDLENV"   /* name of env. var. for SLCU env.     */
#define fcdENV_DID        "CCDDID"    /* name of env. var. for Dictionary    */
#define fcdDID            "CCDDCS"    /* name of default CCD Data Dictionary */
#define fcdENV_FDID       "FCDDID"    /* name of env. var. for FIERA Dic.    */
#define fcdFDID           "FCDDCS"    /* name of default FIERA Data Dic.     */
#define fcdCAT            slxDETECTOR /* name of FITS category               */

#define fcdEXP_NEXT       -1          /* ID for next exposure to start       */
#define fcdEXP_LAST       0           /* ID for last started exposure        */
#define fcdCAMERANAME_LEN 8           /* maximum length of camera name       */
#define fcdMAXLENFILE fcdFILENAME_LEN /* maximum length of FITS file names   */
#define fcdMAXBINX        8           /* maximum binning allowed along x     */
#define fcdMAXBINY        8           /* maximum binning allowed along y     */
#define fcdFITSHIERSUFF   "det"       /* suffix for FITS HIER DET file       */
#define fcdDCSHEADLINES   20      /* lines written by CCD DCS in FITS header */
#define fcdDCSXTNDLINES   21      /* lines written by CCD DCS in FITS header */
                                  /* XTENSION part                           */

/* commands implemented */
#define fcdCMD_ABORT       "ABORT"
#define fcdCMD_CONTINUE    "CONT"
#define fcdCMD_DISPLAY     "DISPLAY"
#define fcdCMD_DISABLE     "DISABLE"
#define fcdCMD_DUMP        "DUMP"
#define fcdCMD_END         "END"
#define fcdCMD_ENABLE      "ENABLE"
#define fcdCMD_EXIT        "EXIT"
#define fcdCMD_GRAB        "GRAB"
#define fcdCMD_INIT        "INIT"
#define fcdCMD_OFF         "OFF"
#define fcdCMD_ONLINE      "ONLINE"
#define fcdCMD_OPERATE     "OPERATE"
#define fcdCMD_PAUSE       "PAUSE"
#define fcdCMD_PDOWN       "PDOWN"
#define fcdCMD_SENDIP      "SENDIP"
#define fcdCMD_SETUP       "SETUP"
#define fcdCMD_STAND_ALONE "STANDAL"
#define fcdCMD_STANDBY     "STANDBY"
#define fcdCMD_START       "START"
#define fcdCMD_START_WIPE  "STARTWP"
#define fcdCMD_STARTTL     "STARTTL"
#define fcdCMD_STARTTM     "STARTTM"
#define fcdCMD_STATUS      "STATUS"
#define fcdCMD_STOP        "STOP"
#define fcdCMD_STOP_WIPE   "STOPWP"
#define fcdCMD_STOPTL      "STOPTL"
#define fcdCMD_STOPTM      "STOPTM"
#define fcdCMD_VERS        "VERSION"
#define fcdCMD_WAIT        "WAIT"

#define fcdCMD_STARTLP     "STARTLP"

#define fcdCMD_LOADSEQ     "LOADSQ"

/* commands implemented in WS simulation only (not supported by SLCU yet ) */
#define fcdCMD_SELFTST     "SELFTST"
#define fcdCMD_VERBOSE     "VERBOSE"

/* commands not yet implemented */
#define fcdCMD_CONFIG      "CONFIG"
#define fcdCMD_SIM         "SIM"
#define fcdCMD_STOPSIM     "STOPSIM"

/* parameters */
/* command fcdCMD_STATUS */
#define fcdPAR_STATUS_EXPOID   "expoId"          /* exposure ID parameter name  */
#define fcdPAR_STATUS_FUNCTION "function"        /* function parameter name     */

/* paremeters currently used for fcdPAR_STATUS_FUNCTION */
#define fcdPAR_ALL            "all"
#define fcdPAR_GLOBAL         "global"

#define fcdKEY_OPMODE         "DET.MODE"         /* see also fcdOPMODE          */
#define fcdKEY_OPSTATE        "DET.STATE"        /* see also fcdSTATE           */
#define fcdKEY_SHTSTAT        "DET.SHUT"         /* see also fcdSHT_STATUS      */
#define fcdKEY_TMPSTAT        "DET.TEMP"         /* Temperature values          */
#define fcdKEY_TELSTAT        "DET.TELE"         /* Telemetry values            */

/* reply parameters for fcdCMD_STATUS */
#define fcdPAR_STATUS_EXPSTATUS  "expStatus"     /* exposure status par. name   */
				                 /* see also fcdEXP_xxx         */
#define fcdPAR_STATUS_FUNCSTATUS "funcStatus"    /* function status par. name   */

/* command fcdCMD_SETUP */
#define fcdPAR_SETUP_EXPOID   "expoId"           /* exposure ID parameter name  */
#define fcdPAR_SETUP_FILE     "file"             /* setup file parameter name   */
#define fcdPAR_SETUP_FUNCTION "function"         /* setup function name         */

/* paremeters currently used for fcdPAR_SETUP_FUNCTION */
#define fcdEXP_TYPE           "DET.EXP.TYPE"     /* exposure type               */
				                 /* see also fcdEXPTYPE         */
#define fcdEXP_STEP_DEF       "DET.WIN1.NDIT"    /* Number of steps for MULTI   */
#define fcdTIME_DEF_IDEM      "DET.WIN1.ASUIT"  /* integration time as UIT1    */
#define fcdTIME_DEF_BASE      "DET.WIN1.UIT"
#define fcdTIME_DEFi          "DET.WIN1.UIT%i"   /* integration time           */
#define fcdTIME_DEF           "DET.WIN1.UIT1"    /* integration time            */
#define fcdREPEAT_DEF         "DET.EXP.NREP"     /* exp. repetition count       */
				                 /* see also fcdREPEAT_FOREVER  */
#define fcdTIME_START         "DET.EXP.START"    /* UTC of exposure start       */
#define fcdTIME_SHUT_ABS      "DET.EXP.SHTMABS"  /* Time absolute for local sync on shutter opent time */
#define fcdTIME_ABS           "DET.EXP.TIMEABS"  /* Times Absolute/Relative     */
#define fcdEXP_SYNC           "DET.EXP.SYNC"     /* Exposure syncronised on an 
						    external signal */
#define fcdTIME_REP           "DET.EXP.TIMEREP"  /* Interval between 2 exposures*/
#define fcdFILE_UNC           "DET.FRAM.FITSUNC" /* file for uncompr. image data*/
#define fcdDISK_SAVE          "DET.FRAM.FITSMTD" /* disk save method            */
				                 /* see also fcdDISKSAVE        */
#define fcdXFER_SYNC          "DET.FRAM.XFERSYN" /* image transfer sync */
#define fcdSAMPLE             "DET.FRAM.SAMPLE"  /* transfer sampling rate      */
#define fcdDISPLAY            "DET.DISPLAY"      /* real-time display frame Id  */
#define fcdREAD_GAIN          "DET.OUT1.GAININD"
/* readout gain is no longer an independant parameter, this is maintained */
/* purely for compatibility with the existing S/W                         */
				                 /* see also fcdNO_DISPLAY      */
#define fcdREAD_MODE          "DET.READ.CLKIND"  /* mode index                  */
#define fcdREAD_NFRAM         "DET.READ.NFRAM"   /* # frames in one read */
#define fcdREAD_FRAME_TYPE    "DET.FRAM.TYPE"    /* frame type                  */
				                 /* see also fcdFRAME_xxx       */
#define fcdREAD_SHIFT_TYPE    "DET.READ.SHIFTYP" /* line shift type for MULTI  */
#define fcdREAD_SHIFT_BASE    "DET.READ.SHIFT"   /* line shift                  */
#define fcdREAD_SHIFTi        "DET.READ.SHIFT%i" /* line shift step #i          */

/* command fcdCMD_WAIT */
#define fcdPAR_WAIT_EXPOID   "expoId"            /* exposure ID parameter name  */
#define fcdPAR_WAIT_MODE     "waitMode"          /* wait mode */

/*
 * AB 971005 indexed windows
 */

/*===========*/
/* WINDOW #i */
/*===========*/
#define fcdREAD_WINi_BIN_X    "DET.WIN%d.BINX"    /* readout x binning    */
#define fcdREAD_WINi_BIN_Y    "DET.WIN%d.BINY"    /* readout y binning    */
#define fcdREAD_WINi_ENAB     "DET.WIN%d.ST"      /* enabled flag         */
#define fcdREAD_WINi_FIR_X    "DET.WIN%d.STRX"    /* first x pixel        */
#define fcdREAD_WINi_DIM_X    "DET.WIN%d.NX"      /* x dimensions         */
#define fcdREAD_WINi_FIR_Y    "DET.WIN%d.STRY"    /* first y pixel        */
#define fcdREAD_WINi_DIM_Y    "DET.WIN%d.NY"      /* y dimensions         */
#define fcdPROC_WINi_MINMAX   "DET.WIN%d.MINMAX"  /* search image extrema */
#define fcdPROC_WINi_RMS      "DET.WIN%d.RMS"     /* RMS Calc on image    */
#define fcdPROC_WINi_IPLLX    "DET.WIN%d.IPLLX"   /* proc subWin offsets   */
#define fcdPROC_WINi_IPLLY    "DET.WIN%d.IPLLY"   /* proc subWin offsets   */
#define fcdPROC_WINi_IPURX    "DET.WIN%d.IPURX"   /* proc subWin offsets   */
#define fcdPROC_WINi_IPURY    "DET.WIN%d.IPURY"   /* proc subWin offsets   */

/*
 * AB 971005
 * The following definitions for window 1 and window 2 are mantained for 
 * backward compatibility
 */

/*===========*/
/* WINDOW #1 */
/*===========*/
#define fcdREAD_WIN1_BIN_X    "DET.WIN1.BINX"    /* readout x binning           */
#define fcdREAD_WIN1_BIN_Y    "DET.WIN1.BINY"    /* readout y binning           */
#define fcdREAD_WIN1_ENAB     "DET.WIN1.ST"      /* enabled flag                */
#define fcdREAD_WIN1_FIR_X    "DET.WIN1.STRX"    /* first x pixel               */
#define fcdREAD_WIN1_DIM_X    "DET.WIN1.NX"      /* x dimensions                */
#define fcdREAD_WIN1_FIR_Y    "DET.WIN1.STRY"    /* first y pixel               */
#define fcdREAD_WIN1_DIM_Y    "DET.WIN1.NY"      /* y dimensions                */
#define fcdPROC_WIN1_MINMAX   "DET.WIN1.MINMAX"  /* search image extrema        */
#define fcdPROC_WIN1_RMS      "DET.WIN1.RMS"     /* RMS Calc on image        */

/* the ones below are retained for compatibility */
#define fcdPROC_WIN1_CENTROID "DET.WIN1.CENTROID" /* Centroiding        */

#define fcdPROC_WIN1_REF_X    "DET.WIN1.REFX"    /* X ref. star for centroiding */
#define fcdPROC_WIN1_REF_Y    "DET.WIN1.REFY"    /* Y ref. star for centroiding */
#define fcdPROC_WIN1_USERFUNC "DET.WIN1.IPFUNC"  /* User Function Name */
#define fcdPROC_WIN1_THRMIN   "DET.WIN1.THRMIN"  /* threshold for min intensity */
#define fcdPROC_WIN1_THRMAX   "DET.WIN1.THRMAX"  /* threshold for max intensity */
#define fcdPROC_WIN1_BACKGND  "DET.WIN1.BACKGND" /* sky background          */
#define fcdPROC_WIN1_IPLLX    "DET.WIN1.IPLLX"   /* proc subWin offsets   */
#define fcdPROC_WIN1_IPLLY    "DET.WIN1.IPLLY"   /* proc subWin offsets   */
#define fcdPROC_WIN1_IPURX    "DET.WIN1.IPURX"   /* proc subWin offsets   */
#define fcdPROC_WIN1_IPURY    "DET.WIN1.IPURY"   /* proc subWin offsets   */

/*===========*/
/* WINDOW #2 */
/*===========*/
#define fcdREAD_WIN2_BIN_X    "DET.WIN2.BINX"    /* readout x binning           */
#define fcdREAD_WIN2_BIN_Y    "DET.WIN2.BINY"    /* readout y binning           */
#define fcdREAD_WIN2_ENAB     "DET.WIN2.ST"      /* enabled flag                */
#define fcdREAD_WIN2_FIR_X    "DET.WIN2.STRX"    /* first x pixel               */
#define fcdREAD_WIN2_DIM_X    "DET.WIN2.NX"      /* x dimensions                */
#define fcdREAD_WIN2_FIR_Y    "DET.WIN2.STRY"    /* first y pixel               */
#define fcdREAD_WIN2_DIM_Y    "DET.WIN2.NY"      /* y dimensions                */
#define fcdPROC_WIN2_MINMAX   "DET.WIN2.MINMAX"  /* search image extrema        */
#define fcdPROC_WIN2_RMS      "DET.WIN2.RMS"     /* RMS Calc on image        */
/* the ones below are retained for compatibility */
#define fcdPROC_WIN2_CENTROID "DET.WIN2.CENTROID" /* Centroiding        */

#define fcdPROC_WIN2_USERFUNC "DET.WIN2.IPFUNC"  /* User Function Name */
#define fcdPROC_WIN2_REF_X    "DET.WIN2.REFX"    /* X ref. star for centroiding */
#define fcdPROC_WIN2_REF_Y    "DET.WIN2.REFY"    /* Y ref. star for centroiding */
#define fcdPROC_WIN2_THRMIN   "DET.WIN2.THRMIN"  /* threshold for min intensity */
#define fcdPROC_WIN2_THRMAX   "DET.WIN2.THRMAX"  /* threshold for max intensity */
#define fcdPROC_WIN2_BACKGND  "DET.WIN2.BACKGND" /* sky background          */
#define fcdPROC_WIN2_IPLLX    "DET.WIN2.IPLLX"   /* proc subWin offsets   */
#define fcdPROC_WIN2_IPLLY    "DET.WIN2.IPLLY"   /* proc subWin offsets   */
#define fcdPROC_WIN2_IPURX    "DET.WIN2.IPURX"   /* proc subWin offsets   */
#define fcdPROC_WIN2_IPURY    "DET.WIN2.IPURY"   /* proc subWin offsets   */

/* paremeters foreseen but not used (the list is not finalized) */
#define fcdREADPIX            "DET.READ.NO"      /* pixel readout times (skip)  */
#define fcdMPPMODE            "DET.READ.MODE"    /* mpp mode                    */
#define fcdFILE_COM           "DET.FRAM.FITSCMP" /* file for compr. image data  */

/* command fcdCMD_START */
#define fcdPAR_START_EXPOID   "expoId"           /* exposure ID parameter name  */

/* command fcdCMD_VERS */
#define fcdPAR_VERS_CCDVERS   "fcdVersion"       /* version of CCD software     */

/* parameters for fcdCMD_STAND_ALONE */
#define fcdPAR_STANDAL_ON     "on"               /* stand-alone flag            */
#define fcdPAR_STANDAL_ARC    "archive"          /* archive images              */

/* 
 * exposure status (fcdPAR_STATUS_EXPSTATUS). Bit-field
 */
#define fcdEXP_NONE             0
#define fcdEXP_INACTIVE         1
#define fcdEXP_PENDING          2
#define fcdEXP_INTEGRATING      4
#define fcdEXP_PAUSED           8
#define fcdEXP_READING         16
#define fcdEXP_PROCESSING      32
#define fcdEXP_TRANSFERING     64
#define fcdEXP_COMPLETED      128
#define fcdEXP_FAILED         256
#define fcdEXP_ABORTED        512
#define fcdEXP_LOOP_FINITE   1024
#define fcdEXP_LOOP_INFINITE 2048
#define fcdEXP_WIPING        4096  /* added RHD */

#define fcdSIMEXP_NONE          0 /* AB 970723 used only by ws simulation */
#define fcdSIMEXP_STOPPED       1 /* AB 970723 used only by ws simulation */
#define fcdSIMEXP_ENDED         2 /* AB 970723 used only by ws simulation */
#define fcdSIMEXP_ABORTED       4 /* AB 970723 used only by ws simulation */
#define fcdSIMEXP_PAUSED        8 /* AB 970723 used only by ws simulation */
#define fcdSIMEXP_STARTED      16 /* AB 970723 used only by ws simulation */

#define fcdEXP_DONE \
       (fcdEXP_COMPLETED | fcdEXP_FAILED | fcdEXP_ABORTED)
#define fcdEXP_RUNNING \
       (fcdEXP_PENDING | fcdEXP_WIPING | fcdEXP_INTEGRATING | fcdEXP_PAUSED | \
	fcdEXP_READING | fcdEXP_PROCESSING | fcdEXP_TRANSFERING )

/* type for an exposure status. made up of bits fcdEXP_* */
typedef vltINT32 fcdEXP_STATUS;

/*
 * Infinite loop (fcdREPEAT_DEF)
 */
#define fcdREPEAT_FOREVER 0

/*
 * Display types
 */
#define fcdNO_DISPLAY	-1
#define fcdNO_BUILDUP	3

/*
 * Wait modes
 */
#define fcdWAIT_SINGLE 0
#define fcdWAIT_GLOBAL 1

#define fcdWAIT_SINGLE_STR "Single"
#define fcdWAIT_GLOBAL_STR "Global" 

/*
 * New types definitions
 */

/* type for user IP function */
typedef enum
    {
    fcdIMAGE_CHAR = 0,               /* 8 bit unsigned                     */  
    fcdIMAGE_INT,                    /* 16 bit signed                      */
    fcdIMAGE_FLOAT                   /* 32 bit floating point              */
    } fcdIMAGE_REP;

/* fcdEXP_TYPE */
typedef enum
    {
    fcdEXP_DARK = 0,
    fcdEXP_NORMAL,
    fcdEXP_LED,
    fcdEXP_LEDSHUT,
    fcdEXP_MULTI,
    fcdEXP_DRIFT
    } fcdEXPTYPE;
#define fcdEXP_DARK_STR    "Dark"
#define fcdEXP_NORMAL_STR  "Normal"
#define fcdEXP_LED_STR     "Led"
#define fcdEXP_LEDSHUT_STR "Ledshut"
#define fcdEXP_MULTI_STR   "Multiple"        /* not implemented yet */
#define fcdEXP_DRIFT_STR   "Driftscan"       /* not implemented yet */

typedef char fcdCAMERANAME[fcdCAMERANAME_LEN + 1];   /* name of CCD camera  */

/* operational modes */
typedef enum                       /* possible operational modes           */
    {
    fcdNOT_AVAILABLE = 1,          /* FIERA sw is not available              */
    fcdNORMAL,                     /* FIERA sw and hw available              */
    fcdSIM_WS,                     /* SLCU FIERA sw is simulated at WS level */
    fcdSIM_SLCU,                   /* FIERA hw is simulated at SLCU level    */
    } fcdOPMODE;

/*
 * operational states
 */
typedef enum {                       /* possible states of CCD DCS           */
    fcdUNKNOWN   = ccsSTATE_UNK,     /* unknown                              */
    fcdOFF       = ccsSTATE_OFF,     /* terminating                          */
    fcdLOADED    = ccsSTATE_LOADED,  /* need to be initialised               */
    fcdSTANDBY   = ccsSTATE_STANDBY, /* stand-by                             */
    fcdONLINE    = ccsSTATE_ONLINE,  /* ready for exposures                  */
} fcdSTATE;

/*
 * RMS Calculation types
 */
typedef enum {                     /* possible RMS Calculation types       */
    fcdRMS_NONE = 0,
    fcdRMS_WHOLE,  
    fcdRMS_CENTRAL, 
    fcdRMS_WINDOW,   
} fcdRMS_TYPE;


/*
 * Multi-Step Integration time
 */
#define fcdEXPTIME_IDEM ccsTRUE  /* use DET.WIN1.UIT1 for all steps */
#define fcdEXPTIME_LIST ccsFALSE /* use DET.WIN1.UITi as defined    */

/*
 * Line Shift type
 */
    /* Setup values for Line Shift type */
typedef enum 
    {
    fcdLINE_SHIFT_ALT = -1,      /* use DET.READ.SHIFT1 alt. pos & neg*/
                                 /* Alternate exposure mode           */
    fcdLINE_SHIFT_IDEM = 0,      /* use DET.READ.SHIFT1 for all steps */
                                 /* Focus & Sky Coherence modes       */
    fcdLINE_SHIFT_LIST = 1       /* use DET.READ.SHIFTi as defined    */
    } fcdLINESHIFT_TYPE;

#define fcdLINE_SHIFT_ALT_STR  "alternate"
#define fcdLINE_SHIFT_IDEM_STR "idem"
#define fcdLINE_SHIFT_LIST_STR "list"


/*
 * Centroiding type (see setup param. fcdIP_CENTROID)
 */
typedef enum
    {
    fcdCENTROID_NONE = 0,
    fcdCENTROID_STANDARD,
    fcdCENTROID_THRESHOLD
    } fcdCENTYPE;

#define fcdCEN_NONE_STR      "none"       /* no centroiding calculation      */
#define fcdCEN_STANDARD_STR  "standard"   /* standard centroiding calc.      */
#define fcdCEN_THRESHOLD_STR "threshold"  /* standard centroiding calc.      */

    /* 
     * Maximum Threshold :
     * ===================
     * - Ignored in the current implementation
     */

    /* 
     * Minimum Threshold :
     * ===================
     * => value >= 0 : use this value
     * => value within the integer range [-1;-9] : 
     *    the threshold min is preset to N StdDev of Intensity distribution 
     *    over this window
     * => value within the integer range [-11;-19] : 
     *    the threshold min is preset to N StdDev of Intensity distribution 
     *    over the first window
     *    This setting is only valid for the second window
     */
    /* The following macros are standard settings */
#define fcdTHRMIN_3SIGMA_WINDOW_SELF  -3
#define fcdTHRMIN_3SIGMA_WINDOW_PREV -13
#define fcdTHRMIN_5SIGMA_WINDOW_SELF  -5
#define fcdTHRMIN_5SIGMA_WINDOW_PREV -15

    /* 
     * BackGround :
     * ============
     * => value within the >= 0 : use this value
     * => value set to -1 : 
     *    the BackGround is preset to the Average Intensity 
     *    over this window
     * => value set to -11 : 
     *    the BackGround is preset to the Average Intensity 
     *    over the first window
     *   This setting is only valid for the second window
     */
    /* The following macros are standard settings */
#define fcdBCKGND_FLUX_WINDOW_SELF    -1
#define fcdBCKGND_FLUX_WINDOW_PREV   -11

/* real-time image processing user function definitions */
#define fcdIP_NO_USER_FUN   "None"        /* no user function to be called   */
#define fcdIP_NO_USER_BUF   "None"        /* user function has no buffer     */

#define fcdIP_IQE_USER_FUN   "IQE"        /* Image quality analysys */


/*
 * shutter status definition
 */
typedef enum
    {
    fcdSHT_ERROR = -1,
    fcdSHT_CLOSED,
    fcdSHT_OPENED,
    fcdSHT_CLOSING,
    fcdSHT_OPENING
    } fcdSHT_STATUS;

/* Various ways of saving an image in disk file(s) */
typedef enum
    {
    fcdDISK_NONE = 0,
    fcdDISK_COMPRESS,               /* not implemented yet */
    fcdDISK_UNCOMPRESS,
    fcdDISK_BOTH                    /* not implemented yet */     
    } fcdDISKSAVE;

#define fcdDISK_NONE_STR       "None"
#define fcdDISK_COMPRESS_STR   "Compress"
#define fcdDISK_UNCOMPRESS_STR "Uncompress"
#define fcdDISK_BOTH_STR       "Both"

/* configuration of a chips serial register */
typedef struct
    {
    vltINT32 yLocation;                   /* y position of serial */
    vltINT32 yOverscan;                   /* pre and over scan in y */
    vltINT32 yPrescan;
    vltINT32 nOuts;                  /* number of outputs in serial reguster */
    vltINT32 output[fcdMAXOUTCHIP]; /* indexes of outputs */
    } fcdCONFSERIAL;

/* info about chip configuration */
typedef struct 
    {                     
    vltLOGICAL available;            /* available                          */
    vltINT32   xLocation;            /* x location in mosaic               */
    vltINT32   yLocation;            /* y location in mosaic               */
    vltDOUBLE  xGap;                 /* xGap                               */
    vltDOUBLE  yGap;                 /* yGap                               */
    vltLOGICAL flipped;              /* is chip flipped in mosaic          */
    } fcdCONFCHIP;

/* info about output configuration */
typedef struct 
    {
	vltLOGICAL available;          /* available                          */
	vltINT32   chipIndex;          /* index of chip it belongs to        */
	vltINT32   xLocation;          /* x location in mosaic               */
	vltINT32   yLocation;          /* y location in mosaic               */
	vltDOUBLE  crpix1;             /* crpix value includes gaps          */
	vltDOUBLE  crpix2;            
	vltINT32   leftToRight;        /* horizontal readout direction 0,1   */
	vltINT32   downToUp;           /* vertical readout direction 0,1     */
	vltINT32   xPrescan;           /* prescan pixels along x             */
	vltINT32   yPrescan;           /* prescan pixels along y             */
	/* some chipe (e.g. FORS have a funny number of pre-scan pixels in 
	   X or Y, therefore we provide these fields as a mechanism to dump
	   pixels or lines before thinking about the pre-scan region */
	vltINT32   xPredump;
	vltINT32   yPredump;
	vltINT32   xPix;               /* active pixels along x              */
	vltINT32   yPix;               /* active pixels along y              */
	vltINT32   xOverscan;          /* overscan pixels along x            */
	vltINT32   yOverscan;          /* overscan pixels along y            */
	vltBYTES16  name;              /* name of the output                 */
    } fcdCONFOUTPUT;

/*
 * fcdModeUseqConfig
 *
 * The usequence definitions which go together to make a "mode" i.e. all the
 * usequences and speeds which make out a "FAST" readout.
 */
typedef struct
{
  vltINT32  parShiftSpeed;
  vltINT32  parDumpSpeed;
  vltINT32  serShiftSpeed;
  vltINT32  serDumpSpeed;
  vltINT32  serReadSpeed;
} fcdModeUseqConfig;

/*
 * fcdMode
 *
 * Information which defines a readout "mode".
 */

/*
 *# Different readout sequences can be defined for different binning
 *# factors. This allows optimized versions rather than S/W generated
 *# ones.
 */
typedef struct {
  vltINT32          binFactX; 
  vltINT32          binFactY; 
  fcdSEQUENCENAME   readSeq;
  fcdVOLTTABLENAME  readVolts;
} fcdREAD_SEQ_CONFIG;

/*
 *# fcdABS_TIME_MODE
 *#
 *# Either schedule by the SPARC, or synchronise with the TIM module
 */
typedef enum
{
    fcdABS_TIME_START_TIME = 1,    /* just let the SPARC schedule */
    fcdABS_TIME_SEQ_TIME           /* sychronise the start of a sequence */
} fcdABS_TIME_MODE;

/*
 *# fcdABS_TIME_WHICH_SEQ
 *#
 *# Which sequence is to be synchronised in the case of fcdABS_TIME_SEQ_TIME
 */
typedef enum
{
    fcdABS_TIME_WIPE_SEQ = 1,   
    fcdABS_TIME_INT_SEQ,   
    fcdABS_TIME_DURING_INT_SEQ,
    fcdABS_TIME_READ_SEQ
} fcdABS_TIME_WHICH_SEQ;

typedef struct
{
  vltLOGICAL valid;                /* is this a valid mode */

  vltLOGICAL maintOnly;            /* only valid in maintainence mode */

  vltBYTES64 description;          /* description of mode  */
  vltBYTES16 speed;                /* descriptive field for speed  */

  vltINT32   firstOutput;          /* output generating first pixel value*/
  vltLOGICAL outEnab[fcdMAXOUTPUT];/* outputs enabled over the mosaic    */

  double      gain[fcdMAXOUTPUT]; 
  double      elAdu[fcdMAXOUTPUT]; /* electrons per adu */
  double      ron[fcdMAXOUTPUT];   /* Read Noise in e   */

  fcdVOLTTABLENAME  wipeVolts; /* voltage tables to be applied       */
  fcdVOLTTABLENAME  intVolts;
  fcdVOLTTABLENAME  readVolts;

  fcdSEQUENCENAME       wipeSeq;   /* sequences to use for this mode      */
  fcdSEQUENCENAME       intSeq;
  fcdSEQUENCENAME       duringIntSeq;
  fcdREAD_SEQ_CONFIG    readSeqs[fcdMAXREADSEQ];
  vltLOGICAL            wipeAlways;  /* always wipe, even in loop */
  vltLOGICAL            rampUp;      /* ramp up electronics (default is yes) */
  vltLOGICAL            unordData;   /* don't reorder the data */

  /* absolute time stuff */
  vltINT32              absTimeMode;     /* one of fcdABS_TIME_MODE */
  vltINT32              absTimeWhichSeq; /* one of fcdABS_TIME_WHICH_SEQ */
  vltDOUBLE             absTimeOffset;   /* seconds by which to offset time */

  /* usequences to use for this mode which will be used to create */
  /* sequences depending upon the setup (windowing etc)           */
  fcdModeUseqConfig     useqs;
} fcdCONFMODE;

/* types of shutter available */
typedef enum
{
  fcdSHUT_PULPO1 = 1,
  fcdSHUT_PULPO2,
  fcdSHUT_DIGITAL,
  fcdSHUT_EXERNAL,
  fcdSHUT_PULPO1and2 /* VIMOS shutter uses two PULPOs */
} fcdSHUTTER_TYPE;

/* info about camera shutter          */
typedef struct 
{
  vltLOGICAL     available;            /* available                          */

  vltINT32       typeId;               /* one of fcdSHUTTER_TYPE             */

  /* peripheral information for control (not implemented yet)*/
  vltINT32       boardId;
  vltINT32       periphId;
  vltINT32       openValue;
  vltINT32       closeValue;
} fcdCONFSHUTTER;

/* info about camera configuration    */
typedef struct 
    {                     
    fcdCAMERANAME name;              /* camera name                        */
    ccsENVNAME    envName;           /* environment name                   */
    } fcdCAMERA;

/* info about camera configuration  */
typedef struct 
    {                       
    vltLOGICAL     frameTransfer;       /* frame transfer CCD               */
    vltLOGICAL     skipper;             /* skipper CCD                      */
    vltLOGICAL     mpp;                 /* mpp mode supported               */
    vltINT32       xPixels;             /* Number of columns                */
    vltINT32       yPixels;             /* Number of rows                   */
    vltINT32       bitsPixel;           /* ADC resolution                   */

    vltINT32       xTotalChipPixels;    /* maximum for one chip with pre/ovr */
    vltINT32       yTotalChipPixels;    /* maximum for one chip with pre/ovr */

    vltINT32       xTotalImagePixels;   /* total X image dim of all chips   */
    vltINT32       yTotalImagePixels;   /* total Y image dim of all chips   */
    vltINT32       xTotalPixels;        /* total X including all Pre+Over   */
    vltINT32       yTotalPixels;        /* total Y including all Pre+Over   */

    vltINT32       nChips;              /* number of chips present          */
    vltINT32       xChips;              /* chips in X direction in mosaic   */
    vltINT32       yChips;              /* chips in Y direction in mosaic   */

    fcdCONFCHIP    chip[fcdMAXCHIPS];   /* single chip description          */
    vltINT32       numOutputs;          /* Number of outputs in mosaic      */
    fcdCONFOUTPUT  output[fcdMAXOUTPUT];/* single output description over 
					 whole mosaic */
    vltINT32       nOutChip;            /* number of outputs present on chip */
    fcdCONFOUTPUT  chipOutput[fcdMAXOUTCHIP]; /* outputs defined for chip  */

    vltINT32       nSerial;             /* number of serial registers        */
    fcdCONFSERIAL  serial[fcdMAXOUTCHIP]; /* configuration of serial regs   */

    fcdCONFMODE    mode[fcdMAXMODES];   /* single mode pattern information */
    fcdCONFSHUTTER shutter;             /* shutter information              */

    vltDOUBLE      clockSpeed;          /* speed of clock in Mhz            */
    vltLOGICAL     aoDetector;          /* is this an AO application        */
    vltLOGICAL     doubleBuffered;      /* does the detector need two exps  */
	                                /* in memory at the same time       */
    vltLOGICAL     syncMaster;          /* is the detector a synchronization 
					   master? (default no) 	    */
    vltINT32       maxFrames;           /* can the detector have multiple   */
	                                /* frames in the same exposure, for */
	                                /* AO work                          */
    
    } fcdCONFIG;

/* window readout definition */
typedef struct
    { 
    vltLOGICAL enabled;             /* this window is enabled for readout  */
    vltLOGICAL idem;                /* same binning as Window #1           */
    vltINT32   xBinning;            /* horizontal binning factor           */
    vltINT32   yBinning;            /* vertical binning factor             */
    vltINT32   xFirst;              /* first column                        */
    vltINT32   xDim;                /* number of columns                   */
    vltINT32   yFirst;              /* first row                           */
    vltINT32   yDim;                /* number of rows                      */
    } fcdWINDOW;

/* window process definition */
typedef struct
    { 
    vltLOGICAL   minMax;            /* do MinMax                         */
    vltINT32     rms;               /* do RMS Calculation                */
	
	/* the following are not currently supported */
    vltLOGICAL   bias;              /* do Bias correction                */
    vltLOGICAL   dark;              /* do Dark correction                */
    vltLOGICAL   flat;              /* do Flat Field correction          */
    vltINT32     averageN;          /* do average over N frames          */
    vltBYTES32   userBuffName;      /* User-Defined buffer address       */
	/* end unsupported items */

    vltBYTES32   userFuncName;      /* User-Defined function name        */
	                            /* Only "IQE" currently supported    */
    vltINT32     centroiding;       /* do Centroid calculation (mode)    */
    vltDOUBLE    centrRefX;         /* Reference Point X                 */
    vltDOUBLE    centrRefY;         /* Reference Point Y                 */
    vltDOUBLE    centrThreshMin;    /* Centroid Threshold Min            */
    vltDOUBLE    centrThreshMax;    /* Centroid Threshold Max            */
    vltDOUBLE    backGround;        /* Sky BackGround                    */
    vltINT32     procLLX,procLLY;   /* Lower Left & Upper Right corners  */
    vltINT32     procURX,procURY;   /* of Processing window (offset from */
	                            /* physical window)                  */
    } fcdPROCESS;

/* exposure setup parameters */
typedef struct 
    {                 
    fcdEXPTYPE   expType;        /* exposure type                           */
    vltINT32     stepsDef;       /* number of steps (MULTI)                 */
    vltINT32     repeatDef;      /* repetition number (0 is forever)        */
    vltLOGICAL   timeDefIdem;    /* exposure time UITi as UIT1 ?            */
    vltDOUBLE    timeDef;        /* exposure time (step 1)                  */
    vltDOUBLE    timeDefMulti[fcdMAXINTEGR];
                                 /* exposure time (steps 2+ for MULTI)      */
    vltLOGICAL   timeAbsolute;   /* absolute (T) or relative (F) time       */
    vltLOGICAL   timeShutAbsolute; /* absolute (T) or relative (F) time for 
				      local shutter sync  */
    vltDOUBLE    timeStart;      /* exposure start time (MJD)               */
    vltDOUBLE    timeRepeat;     /* sec between repeated exposures          */
    vltLOGICAL   expSync;        /* exposure sync on external signal        */
    vltLOGICAL   syncMaster;     /* We are master of the sync exposure      */
    vltLOGICAL   fitsInfoCollect;/* Collect info for FITS header            */
    vltLOGICAL   tempExp;        /* Store tmp. periodically during exposure */
    vltINT32     tempSecs;       /* sec between temperature checks          */
    vltLOGICAL   teleExp;        /* Store tel. periodically during exposure */
    vltINT32     teleSecs;       /* sec between telemetry checks            */

	/* IT  : */
    fcdDISKSAVE  diskSave;       /*       disk save method                  */
    fcdFILENAME   fileNameUnComp; 
                                 /*       uncompressed FITS file name       */
    fcdFILENAME   fileNameComp;   
                                 /*       compressed FITS file name         */
    vltINT32     pixRepr;        /*       # bit for representation          */
    vltINT32     bitShift;       /*       # bit shift for RTD               */
    vltINT32     compression;    /*       compression type                  */
    vltINT32     sampling;       /*       1 image transferred out of N read */
    vltLOGICAL   itSync;         /*       image transfer synchronous        */

	/* RTD */
    vltINT32     frameId;        /*  real-time display frame Id        */

	/* IP */
    fcdPROCESS   process[fcdMAXWINDOW]; /* IP  : single window setup        */

	/* RDT */
    vltINT32     linesToShift;         /* # lines to be shifted step 1      */ 
    vltINT32     linesToShiftMulti[fcdMAXINTEGR];
    fcdLINESHIFT_TYPE linesToShiftType;
    
    vltINT32     readPixelN;     /*       # times the same pixel is read    */ 
    vltLOGICAL   mpp;            /*       MPP mode flag                     */
    vltINT32     modeIndex;      /*       readout clock to use              */
    vltBYTES16   readSpeed;      /*       copied from the mode definition   */

    vltINT32     nFrames;        /* number of frames to acquire for one image */

    fcdWINDOW    window[fcdMAXWINDOW];/*  single window setup               */
    } fcdSETUP;

typedef struct
{ 
  vltLOGICAL changed;                /* some setup values have been changed */
  vltLOGICAL doTransfer;             /* Image transfer required             */
  vltLOGICAL doProcess;              /* Image processing required           */
 
  vltLOGICAL fullFrame;              /* is this a full frame readout        */
  vltINT32   numWindows;             /* number of enabled windows           */
  fcdWINDOW  window[fcdMAXWINDOW];   /*  single window setup                */
  vltINT32   totalPixels;            /* number of pixels in total           */
 
  vltDOUBLE  wipeTime;               /* time required by the sequences      */
  vltDOUBLE  readTime;
} fcdSETUPRES;

/*
 * function prototypes
 */
/* Get information about a CCD camera configuration */
ccsCOMPL_STAT fcdGetConf    ( fcdCAMERA *camera, fcdCONFIG *config,
			      ccsERROR  *error);
/* Check a given camera configuration */
ccsCOMPL_STAT fcdCheckConf    ( fcdCAMERA *camera, fcdCONFIG *config,
			      ccsERROR  *error);

/* Check a simulated exposure setup */
ccsCOMPL_STAT fcdCheckSimSetup ( fcdCONFIG *config, fcdSETUP *setup,
                              fcdSETUPRES *results, ccsERROR *error);

/* Check a complete exposure setup */
ccsCOMPL_STAT fcdCheckSetup ( fcdCONFIG *config, fcdSETUP *setup,
                              fcdSETUPRES *results, ccsERROR *error);

/* Check the setup for a simulated windowed readout */
ccsCOMPL_STAT fcdCheckSimSetupWindow ( fcdCONFIG *config, fcdSETUP *setup,
                                    fcdSETUPRES *results, ccsERROR *error); 

/* Check the setup for a windowed readout */
ccsCOMPL_STAT fcdCheckSetupWindow ( fcdCONFIG *config, fcdSETUP *setup,
                                    fcdSETUPRES *results, ccsERROR *error); 

/* Get information about a CCD camera configuration */
void fcdGetCIName           ( fcdCAMERANAME camera, ccsPROCNAME procName );

/* Extract index of DB point describing an exposure from the exposure ID     */
vltINT32 fcdGetIndexFromId  ( vltINT32 expId );



#ifdef __cplusplus
}
#endif

#endif /* ! CCD_H */
