#ifndef FCDBOARD_CONFIG_H
#define FCDBOARD_CONFIG_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdBOARD_CONFIG.h,v 2.140 2005/10/17 15:16:56 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  23/05/96  created
* rdonalds  11/06/99  added second preamp
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Include files for standard C++ library components
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

// Normal fcd includes
#include "fcdDLListPtr.h"
#include "fcd.h"
#include "fcdlIntf.h"
#include "fcdC40.h"


/*
 *# fcdSIMM
 *#
 *# Simms are defined by their position (SIMM0->N), type and user supplied name
 */
typedef struct
{
    vltINT32		number;	// SIMM position SIMM0->N
    enum fcdlCLK_TYPE	type;	// Type of SIMM installed
                                // VACANT, BILEVEL, MULTILEVEL
                                // Use the same type as the C40 code
    string		name;	// Name the user wants to refer to e.g. RESET
} fcdSIMM;

/*
 *# fcdBOARD_MAP_ENTRY
 *
 *# entries which go into a map recording where the patterns are loaded in
 *# the boards memory
 */
typedef struct
{
    vltINT32	address;
    vltINT32	length;
} fcdBOARD_MAP_ENTRY;

/*
 *# fcdBOARD_MAP
 *#
 *# Class to maintain a record of where patterns have been placed in memory
 *# and provide member functions which indicate where new patterns may be
 *# placed.
 */
class fcdBOARD_MAP
{
  public:
    fcdBOARD_MAP();
    ~fcdBOARD_MAP();

    void Init(vltINT32 max);

    ccsCOMPL_STAT FreeAt (vltINT32 address, vltINT32 space);
    ccsCOMPL_STAT FirstFreeAfter (vltINT32 &address, vltINT32 space);
    ccsCOMPL_STAT Allocate (vltINT32 address, vltINT32 space);

  private:
    fcdDLListPtr        entryList; /* list of BOARD_MAP_ENTRY * */
 
    vltINT32            maxAddress;

    /* does the board have a pattern loaded overlapping this address */
    fcdBOARD_MAP_ENTRY *Contains (vltINT32 address);

    /* what is the first free address following this one */
    fcdBOARD_MAP_ENTRY *Following (vltINT32 address);
    void ClearList();
};
/*
 *# fcdBOARD
 *#
 *# Board definition 
 */
class fcdBOARD
{
friend class fcdBOARD_PARSER;
public:
    /*
     *# Creator and destructors
     */
    fcdBOARD();
    ~fcdBOARD();

    /*
     *# Accessor functions
     */
    fcdSIMM		*Simm (const string &name);
    string		&Name();
    vltINT32		Id();
    vltINT32            HasPreamp();
    vltINT32            HasPreamp2();
    vltINT32            BrdRev();

    // Access simms by index
    vltINT32		NumSimms();
    fcdSIMM		*SimmAtIx(vltINT32);
    fcdSIMM		*SimmWithNumber(vltINT32);

    vltLOGICAL	SimmExists (const string &name);
    vltLOGICAL	SimmExists (const vltINT32 number);
    vltINT32	SimmIndex (const string &name);
    vltINT32    NumMultilevelSimms();

    /* access the Board Map */
    ccsCOMPL_STAT FreeAt (vltINT32 address, vltINT32 space);
    ccsCOMPL_STAT FirstFreeAfter (vltINT32 &address, vltINT32 space);
    ccsCOMPL_STAT Allocate (vltINT32 address, vltINT32 space);

    /* set the used space on the board to 0 */
  ccsCOMPL_STAT ReleaseSpace();
private:
    string		name;	// "BRD_CLKDRV1" etc
    vltINT32		id;	// BRD_CLKDRV1 etc
    vltINT32		initialSpace;	// Amount of memory on the board

    fcdBOARD_MAP        boardMap;

    // Has the board got a preamp or two .. valid only for Video boards
    vltINT32            hasPreamp;
    vltINT32            hasPreamp2;
    // Brd revision
    vltINT32            brdRev;

    // Initialise self after parsing
    void InitValues();

    // Simm list and functions
    vector <fcdSIMM>	simms;
    ccsCOMPL_STAT SimmAdd (const fcdSIMM &simm);
    ccsCOMPL_STAT SimmAdd (const vltINT32       number,
			   const enum fcdlCLK_TYPE  type,
			   const string   &name);
};
/*
 *# fcdBOARD_CONFIG
 *#
 *# Overall Board definition 
 */
class fcdBOARD_CONFIG
{
friend class fcdBOARD_CONFIG_PARSER;
public:
    fcdBOARD_CONFIG();
    ~fcdBOARD_CONFIG();

    /*
     *# InitFromDir
     *
     *# Initialise the board config from given directory
     */
    ccsCOMPL_STAT GetFromDir (const string &dirName);

    /*
     *# Uninit
     *#
     *# free any resources allocated
     */
    ccsCOMPL_STAT Uninit();

    /*
     *# Download the board cnfiguration using the gC40 object
     */
    ccsCOMPL_STAT Download(fcdC40 *gC40);

    /*
     *# Find a board with a particular name
     */
    fcdBOARD      *FindBoard(const string &);

    /*
     *# Add a new board
     */
    ccsCOMPL_STAT Add(fcdBOARD *p);

    /*
     *# Number of boards
     */
    vltINT32 NumBoards();

    /*
     *# Set the used space on all board to 0
     */
    ccsCOMPL_STAT ReleaseSpace();
private:
    fcdDLListPtr     list;
    void             FreeList();
};

#endif /*!FCDBOARD_CONFIG_H*/
