#ifndef fcdC40_H
#define fcdC40_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdC40.h,v 2.140 2005/10/17 15:16:55 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  12/06/96  created
* rdonalds  05/08/98  Added ExecSeqAtTime functions for Abs time start
* rdonalds  28/10/98  Added timeout on WaitAsynchSeq to avoid hanging
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _REENTRANT    /* basic 2-lines for threads */
#include <pthread.h>

#include "ECCS.h"
#include "fcdlIntf.h"
#include "fcdc40srvif.h"
#include "fcdc40Errors.h"
#include "fcdTIM.h"
#include "fcdSUX.h"

class fcdC40
{
public:
  // Initialise the interface object
  fcdC40();
  ~fcdC40();

    /* intialise and uninitialise the interface object */
  ccsCOMPL_STAT Init(vltLOGICAL simHW);
  ccsCOMPL_STAT Uninit();

  /*
   *# Return simulation flag
   */
  vltLOGICAL Simulated() { return simC40;};

  /************************************************************************
   * Methods for invoking C40 functions
   ************************************************************************/
  
  /* Ping the S/W to see if there */
  ccsCOMPL_STAT Ping(void);

  /* download the configuration and voltage tables. This MUST be done */
  /* before the S/W is Selftested ot Online                           */
  ccsCOMPL_STAT SetConfig(fcdlSetConfigParams *config);
  ccsCOMPL_STAT SetVoltageTable( fcdlSetVoltTableParams	*voltTable);

  /* Initialise the detector head using the predefined voltages and */
  /* and configuration                                              */
  ccsCOMPL_STAT InitDetector(void);

  /* Selftest the S/W without outputing live voltages */
  ccsCOMPL_STAT Selftest(void);

  /* subset of the selftest which only checks the telemetry */
  ccsCOMPL_STAT CheckTelemetry(void);

  /* Put the S/W online and output voltages to the Chip */
  ccsCOMPL_STAT Online(void);

  /* Open the output relays */
  ccsCOMPL_STAT Standby(void);

  /* Switch off the S/W and open the output relays*/
  ccsCOMPL_STAT Off(void);


  /* Sequence and pattern loading */
  ccsCOMPL_STAT DefineBilevelPattern(fcdlDefineBilevelPatternParams	*p);
  ccsCOMPL_STAT DefineMultilevelPattern(
			fcdlDefineMultilevelPatternParams	*pattern);
  ccsCOMPL_STAT DefineUSeq(fcdlDefineUseqParams	*useq);
  ccsCOMPL_STAT DefineSeq(fcdlDefineSeqParams *seq);

  /*****************************************************************
     sequence execution and stopping. A semaphore may be used
     to ensure that no two processes can be sequencing at 
     the same time. If the seqSemId is < 0 no coordination is 
     performed 
     
     ExecSequence
        Executes sequence and waits for completion
     ExecAsynchMultiSequence
     ExecAsynchSequence
     WaitAsynchSequence
        Execute a sequence without waiting, and then wait for reply.
	It is ESSENTIAL that the wait routine is called
  *****************************************************************/

  ccsCOMPL_STAT ExecSequence (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId);
  ccsCOMPL_STAT ExecAsynchMultiSequence (
					  fcdlExecNSeqsParams *params,
					  vltINT32 coordSemId
					 );
  ccsCOMPL_STAT ExecAsynchSequence (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId);
  ccsCOMPL_STAT ExecSequenceAtTime (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId,
			      fcdtimsTIME *time,
			      fcdTIM     *tim);
  ccsCOMPL_STAT ExecAsynchSequenceAtTime (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId,
			      fcdtimsTIME *time,
			      fcdTIM     *tim);
  ccsCOMPL_STAT ExecSequenceAtSignal (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId,
			      fcdtimsTIME *time,
			      fcdTIM     *tim,
			      vltLOGICAL syncMaster,
			      vltINT32 syncTimeout);
  ccsCOMPL_STAT ExecAsynchSequenceAtSignal (
			      vltINT32 seqId,
			      vltINT32 voltTableId,
			      vltINT32 numIterations,
			      vltINT32 interIterationTime,
			      vltINT32 coordSemId,
			      vltLOGICAL syncMaster,
			      vltINT32 syncTimeout);

  ccsCOMPL_STAT WaitAsynchSequence (vltINT32 tmo, vltINT32 coordSemId);
  ccsCOMPL_STAT StopSequence(vltINT32 coordSemId);

    /* directly affect a peripheral */
  ccsCOMPL_STAT SetOneVoltage(
			      fcdlBoardId		board,
			      fcdlPeriphId		periph,
			      fcdlPeriphValue	value
			      );

  /* read a peripheral */
  ccsCOMPL_STAT GetOneVoltage(
			      fcdlBoardId		board,
			      fcdlPeriphId		periph,
			      fcdlPeriphValue	*value
			      );

  /* pass an EXEC_TEST Command */
  ccsCOMPL_STAT ExecTest (vltINT32 testNum, vltINT32 param, 
			  vltINT32 nparams2,
			  vltINT32 *params2);
  ccsCOMPL_STAT ExecAsynchTest (vltINT32 testNum, vltINT32 param, 
				vltINT32 nparams2, vltINT32 *params2,
			       vltINT32 coordSemId);
  ccsCOMPL_STAT WaitAsynchTest (vltINT32 tmo, vltINT32 coordSemId);

  /* read and write arbitrary addresses on camera head */
  ccsCOMPL_STAT Peek (vltINT32 address, vltINT32 *value);
  ccsCOMPL_STAT Poke (vltINT32 address, vltINT32 value);

    /* open and close shutter */
  ccsCOMPL_STAT OpenShutter(void);
  ccsCOMPL_STAT CloseShutter(void);

    /* set the logging level of the C40 code big numbers make more info */
  ccsCOMPL_STAT SetLogLevel(vltINT32 logLevel);

  /* set test video sequencer clock mode. does not enable individual */
  /* text video channels. Must be done with SetOneVoltage with       */
  /* BRD_VIDBRDX, VID_TESTVID_CHANX                                  */
  ccsCOMPL_STAT SetTestVideoMode (vltINT32 mode);
private:
  vltLOGICAL	initialised;	// Has the object been initialised yet
  
  vltLOGICAL simC40;

  int suxFileDesc; // Will be used for sync on signal only

  struct thread_data {
    int suxFileDesc;
    int timeout;
  };

  static void *SyncThread(void *);

};

#endif /*!fcdC40_H*/
