#ifndef FCDCSETUP_H
#define FCDCSETUP_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdCSETUP.h,v 2.140 2005/10/17 15:16:56 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  9/4/97    created
* rdonalds  28/10/98  Added accessors for nFrames
* abalestr  18/03/03  Added rampUp field and accessor
* abalestr  21/01/04  Added sync stuff
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Need to forward declare detector
class fcdDETECTOR;

// Include files for standard C++ library components
#include <string>
#include <iostream>
#include <fstream>

// Normal fcd includes
#include "fcdINTERNAL.h"
#include "fcdDBDIRECT.h"      // for direct DB access
#include "fcdFIERA_CONFIG.h"  // used to access the sequence definitions
#include "fcdEXPOSURE_DATA_DESCR.h"  // describe the data for an exposure
#include "fcdC40.h"           // C40 interface object

/* results of setup preparation on SLCU */
typedef struct 
{                     
    vltINT32   wipeSeqId;            /* sequence ids for various exp phases */
    vltINT32   intSeqId;
    vltINT32   duringIntSeqId;       /* executed after shutter open */
    vltINT32   duringIntSeqTicks;    /* number of ticks it takes */
    vltINT32   readSeqId;
    vltINT32   shiftSeqId;           /* focus exposures shift sequence */

    vltINT32   wipeVoltTable;        /* tables to be used */
    vltINT32   intVoltTable;
    vltINT32   readVoltTable;

    vltLOGICAL wipeAlways;          /* wipe when not start of sequence ?
				       derived from mode */
    vltLOGICAL rampUp;              /* ramp up the electronics?
				       derived from mode default is yes */
    vltLOGICAL syncMaster;          /* Are we synchronization masters?
				       default is no */
    fcdABS_TIME_MODE       absTimeMode;         /* type of ABS time synch */
    fcdABS_TIME_WHICH_SEQ  absTimeWhichSeq;     /* which sequence if 
						   ABS_TIME_SEQ synchronised */
    vltDOUBLE              absTimeOffset;       /* offset for scheduling */
  

    vltLOGICAL unordData;          /* flag indicating that data should not be
				      ordered when read out */

    vltINT32   nFrames;            /* number of frames to acquire for
				      one image transfer */

    fcdREORDER_PIXEL_ORDER pixelOrder;   /* the order of pixels for this */
    fcdEXPOSURE_DATA_DESCR expDataDescr; /* desription of the exposure data */
} fcdSLCUSETUPRES;

/*
 *# T_ENG_SETUP
 *#
 *# Engineering parameters which can be set to override the standard
 *# setup
 */
#define fcdENG_WIPESEQ "ENG.WIPESEQ"
#define fcdENG_INTSEQ  "ENG.INTSEQ"
#define fcdENG_READSEQ "ENG.READSEQ"
typedef struct 
{
    fcdSEQUENCENAME wipeSeq;
    fcdSEQUENCENAME intSeq;
    fcdSEQUENCENAME duringIntSeq;
    fcdSEQUENCENAME readSeq;
} fcdT_ENG_SETUP;

/*
 * T_SETUP
 *
 * The setup information which goes into shared memory as part of the T_EXPOSURE
 * structure for a particular detector
 *
 */
typedef struct
{
    // Has this setup been statically checked and prepared?
    vltLOGICAL  checked; 
    vltLOGICAL  prepared; 

    // Engineering parameters which can be set
    fcdT_ENG_SETUP engSetup;

    // The setup buffer
    fcdSETUP    setup;

    // The result of checking setup using public function
    fcdSETUPRES     setupRes;
    fcdSLCUSETUPRES slcuSetupRes;

  // The root point of the exposure which we are setting up
  // <alias>CAMERA: exposure_[1..N] or exposure_next
    dbSYMADDRESS dbExpRootPt;
} fcdT_SETUP;
/*
 * fcdCSETUP
 *
 * Class which holds all setup information required for an exposure. And has the
 * methods required to store and retrieve setup from the database.
 */
class fcdCSETUP
{
    friend class fcdSEQUENCE_CREATOR;
  public:
    fcdCSETUP();
    ~fcdCSETUP();

    /***********************************************************************
      INITIALISATION / UNINITIALISATION
      ***********************************************************************/
    ccsCOMPL_STAT Create(fcdDETECTOR *det,
			 fcdT_SETUP    *setup,
			 dbSYMADDRESS   dbExpRootPt);
    ccsCOMPL_STAT Attach (fcdDETECTOR *det,
			  fcdT_SETUP  *setup);
    ccsCOMPL_STAT Detach();

    /**********************************************************************
      ASSIGNMENT
      **********************************************************************/

    /* assignment. assigns all the values and updates the target setup in OLDB */
    /* used when copying exposure_next.setup to exposure_n.setup               */
    fcdCSETUP &operator= (fcdCSETUP &s);

    /* used when receiving a command to set a parameter value */
    ccsCOMPL_STAT SetParam (char *name, char *value);

    ccsCOMPL_STAT ChangeIntTime (double newTime);

    /**********************************************************************
      ACCESSORS
      **********************************************************************/

    vltLOGICAL FullFrame() {return (shSetup->setupRes.fullFrame);}
    vltINT32   NFrames()   {return (shSetup->slcuSetupRes.nFrames);}
    void       NFrames(vltINT32 n)   {shSetup->slcuSetupRes.nFrames = n;}
    vltINT32 NumWindows();   /* number of windows */
    vltINT32 XBin (vltINT32 winNum);     /* bin window */
    vltINT32 YBin (vltINT32 winNum);     /* bin window */
    vltINT32 XDim (vltINT32 winNum);     /* cols of window */
    vltINT32 YDim (vltINT32 winNum);     /* rows of window */
    vltINT32 UnbinnedXDim (vltINT32 winNum);     /* cols of window w/o bin */
    vltINT32 UnbinnedYDim (vltINT32 winNum);     /* rows of window w/o bin */
    vltINT32 XFirst(vltINT32 winNum);    /* start X */
    vltINT32 YFirst(vltINT32 winNum);    /* start Y */
    vltINT32 Pixels (vltINT32 winNum);     /* number of pixels */
    vltINT32 PixelBytes (vltINT32 winNum); /* number of bytes  */
    vltLOGICAL IpRMS(vltINT32 winNum);   /* Image process RMS */
    vltLOGICAL IpMinMax(vltINT32 winNum); /* Image process MinMax */
    vltLOGICAL DoProcess()  {return (shSetup->setupRes.doProcess);}

    vltINT32 TotalPixels ();       /* number of pixels (total) */
    vltINT32 TotalPixelBytes ();   /* number of bytes (total)  */
    
    char     *FileNameUnComp() {return ((char *)shSetup->setup.fileNameUnComp);}
    vltINT32  FrameId()        {return (shSetup->setup.frameId);}
    vltINT32  Sampling()        {return (shSetup->setup.sampling);}
    vltLOGICAL ItSync()        {return (shSetup->setup.itSync);}
    vltLOGICAL ExpSync()        {return (shSetup->setup.expSync);}

    vltDOUBLE   ExpTime()  {return (shSetup->setup.timeDef);} 
    vltDOUBLE   ExpTime(vltINT32 step) 
                     {return (shSetup->setup.timeDefMulti[step]);} 
    fcdEXPTYPE  ExpType() {return ((fcdEXPTYPE)shSetup->setup.expType);}
    vltINT32    RepeatDef() {return (shSetup->setup.repeatDef);}
    vltDOUBLE   TimeRepeat() {return (shSetup->setup.timeRepeat);}
    fcdDISKSAVE DiskSave()  {return ((fcdDISKSAVE)shSetup->setup.diskSave);}
    vltINT32    StepsDef()  {return ((fcdDISKSAVE)shSetup->setup.stepsDef);}
    vltLOGICAL  DoTransfer()  {return (shSetup->setupRes.doTransfer);}

    vltINT32    ModeIndex() {return (shSetup->setup.modeIndex);}
    vltDOUBLE   ReadTime()  {return (shSetup->setupRes.readTime);}

    /* return the sequence ids from the slcuSetupRes */
    vltLOGICAL WipeAlways()     {return (shSetup->slcuSetupRes.wipeAlways);}
    vltLOGICAL RampUp()     {return (shSetup->slcuSetupRes.rampUp);}
    vltINT32 WipeSeqId()     {return (shSetup->slcuSetupRes.wipeSeqId);}
    vltINT32 IntSeqId()      {return (shSetup->slcuSetupRes.intSeqId);}
    vltINT32 DuringIntSeqId()      {return (shSetup->slcuSetupRes.duringIntSeqId);}
    vltINT32 DuringIntSeqTicks()      {return (shSetup->slcuSetupRes.duringIntSeqTicks);}

    /* ABS time definitions from mode */
    fcdABS_TIME_MODE AbsTimeMode() {return shSetup->slcuSetupRes.absTimeMode;}
    fcdABS_TIME_WHICH_SEQ AbsTimeWhichSeq() 
             {return shSetup->slcuSetupRes.absTimeWhichSeq;}
    vltDOUBLE AbsTimeOffset() {return shSetup->slcuSetupRes.absTimeOffset;}
    
    vltINT32 ShiftSeqId()      {return (shSetup->slcuSetupRes.shiftSeqId);}
    vltINT32 ReadSeqId()     {return (shSetup->slcuSetupRes.readSeqId);}
    /* return the voltage table ids from the slcuSetupRes */
    vltINT32 WipeVoltTable() {return (shSetup->slcuSetupRes.wipeVoltTable);}
    vltINT32 IntVoltTable()  {return (shSetup->slcuSetupRes.intVoltTable);}
    vltINT32 ReadVoltTable() {return (shSetup->slcuSetupRes.readVoltTable);}

    /* unordered data flag */
    vltLOGICAL UnordData() {return (shSetup->slcuSetupRes.unordData);}
    void UnordData(vltLOGICAL u) {shSetup->slcuSetupRes.unordData = u;}

    /* return the shared pixel order data */
    fcdREORDER_PIXEL_ORDER  &PixelOrder() 
    {return (shSetup->slcuSetupRes.pixelOrder);}
    fcdEXPOSURE_DATA_DESCR  &ExpDataDescr() 
    {return (shSetup->slcuSetupRes.expDataDescr);}
  
    /* return a complete setup buffer */
    fcdSETUP &Setup() {return (shSetup->setup);}

    /**********************************************************************
      MISC
      **********************************************************************/

    /* perform a static check of the setup performing any calculations */
    /* necessary                                                       */
    ccsCOMPL_STAT Check();
    vltLOGICAL    Checked() {return (shSetup->checked);}
  
    /* ensure that the sequences necessary are downloaded or download */
    /* and generate them if necessary                                 */
    ccsCOMPL_STAT Prepare(fcdFIERA_CONFIG *config, fcdC40 *c40);
    vltLOGICAL    Prepared() {return (shSetup->prepared);}
  
    /* performs a check and prepare of the setup */
    ccsCOMPL_STAT CheckAndPrepare (fcdFIERA_CONFIG *config, fcdC40 *c40);
  private:
    /* store/retrieve fields for a setup from the OLDB */
    ccsCOMPL_STAT WriteToDB ();
    ccsCOMPL_STAT ReadFromDB ();

    /* used by the Create/Attach routines */
    ccsCOMPL_STAT Init();

    /*****************************************************************
      Methods used during setup preparation
      *****************************************************************/
    ccsCOMPL_STAT PrepareWithMode(
				  fcdCONFMODE     *mode,
				  fcdFIERA_CONFIG *config, 
				  fcdC40          *c40
				  );
    ccsCOMPL_STAT PrepareWipe (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    ccsCOMPL_STAT PrepareInt (
			      fcdCONFMODE     *mode,
			      fcdFIERA_CONFIG *config, 
			      fcdC40          *c40
			      );
    ccsCOMPL_STAT PrepareShift (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    ccsCOMPL_STAT PrepareRead (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    ccsCOMPL_STAT PrepareFullFrameRead (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    ccsCOMPL_STAT PrepareWindowedRead (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    ccsCOMPL_STAT PrepareWindowedReadSeq (
			       fcdCONFMODE     *mode,
			       fcdFIERA_CONFIG *config, 
			       fcdC40          *c40
			       );
    
    /* store information extracted during Prepare */
    void WipeAlways(vltLOGICAL a) {shSetup->slcuSetupRes.wipeAlways = a;}
    void RampUp(vltLOGICAL a) {shSetup->slcuSetupRes.rampUp = a;}
    void WipeSeqId(vltINT32 seqId) {shSetup->slcuSetupRes.wipeSeqId = seqId;}
    void IntSeqId(vltINT32 seqId)  {shSetup->slcuSetupRes.intSeqId = seqId;}
    void DuringIntSeqId(vltINT32 seqId)  {shSetup->slcuSetupRes.duringIntSeqId = seqId;}
    void DuringIntSeqTicks(vltINT32 ticks)  {shSetup->slcuSetupRes.duringIntSeqTicks = ticks;}
    void ReadSeqId(vltINT32 seqId) {shSetup->slcuSetupRes.readSeqId = seqId;}
    void ShiftSeqId(vltINT32 seqId) {shSetup->slcuSetupRes.shiftSeqId = seqId;}
    void WipeVoltTable(vltINT32 t) {shSetup->slcuSetupRes.wipeVoltTable = t;}
    void IntVoltTable(vltINT32 t)  {shSetup->slcuSetupRes.intVoltTable = t;}
    void ReadVoltTable(vltINT32 t) {shSetup->slcuSetupRes.readVoltTable = t;}
    void WipeTime (vltDOUBLE t)    {shSetup->setupRes.wipeTime = t;}
    void ReadTime (vltDOUBLE t)    {shSetup->setupRes.readTime = t;}
    void PixelOrder (fcdREORDER_PIXEL_ORDER &order)
    {shSetup->slcuSetupRes.pixelOrder = order;}      

    /*
     *# Store the windowing information extracted from a read seq
     */
    void SetReadSeqWindow (vltINT32 xDim, vltINT32 yDim);

    /*
     *# LOCAL INFORMATION
     */
    fcdDETECTOR   *det;          // pointer to the Det we are for
    string        dbRootPt;      // root point

    fcdDBDIRECT_LIST dbDirect;   // list of DB direct addresses
  
    /* the structure holding the setup information in shared memory */
    fcdT_SETUP   *shSetup;
};

/************************************************************************
    INLINE FUNCTIONS
************************************************************************/

inline vltLOGICAL fcdCSETUP::IpRMS(vltINT32 window)
{
    return (shSetup->setup.process[window].rms);
}
inline vltLOGICAL fcdCSETUP::IpMinMax(vltINT32 window)
{
    return (shSetup->setup.process[window].minMax);
}

#endif /*!FCDSETUP_H */
