#ifndef FCDDBDIRECT_H
#define FCDDBDIRECT_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdDBDIRECT.h,v 2.140 2005/10/17 15:16:57 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  14/04/97  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _POSIX_SOURCE 1

#include "vltPort.h"
#include "fcdINTERNAL.h"
#include "fcdDLListPtr.h"
#include <string>

/* 
 *# fcdDBDIRECT
 *#
 *# Class for handling direct access to the on-line DB. Holds the address of 
 *# a variable in memory and the OL-DB direct address with which it is
 *# associated
 */
class fcdDBDIRECT
{
public:
    fcdDBDIRECT();
    ~fcdDBDIRECT();
  
    /*
     *# Init for a given database point
     */
    ccsCOMPL_STAT Init (
			const char    *pointName,    // Point name
			const char    *attrName,     // Attribute name
			const void    *attrAddr      // Attribute address
			);
    ccsCOMPL_STAT Uninit();

    /*
     *# DB Transfer functions Read/Write
     */
    ccsCOMPL_STAT ReadFromDB();
    ccsCOMPL_STAT WriteToDB();
    ccsCOMPL_STAT WriteValueToDB(const void *value);

    /*
     *# return the record count
     */
    vltINT32 RecCnt() {return (recCnt);}
private:
    dbDIRADDRESS attrDbAddr;    // Direct address of object
    vltUINT32     attrSize;     // Size of object (based on type)
    vltUINT16     recCnt;       // Number of records
    void        *attrAddr;      // Address of object variable
    dbTYPE       attrType[2];   // Attribute type (0 terminated)
    dbATTRTYPE   scalType;      // dbSCALAR or dbVECTOR supported
    string       attrFullName;  // Full name of attribute only when debugging
};

class fcdDBDIRECT_LIST
{
public:
  fcdDBDIRECT_LIST();
  ~fcdDBDIRECT_LIST();

  /* intialise and clean up functions */
  ccsCOMPL_STAT Init();
  ccsCOMPL_STAT Uninit();

  /*
   *# Add a new entry to the list
   */
  ccsCOMPL_STAT Add (
		      const char    *pointName,    // Point name
		      const char    *attrName,     // Attribute name
		      const char    *attrAddr      // Attribute address
		      );

  /*
   *# Read/Write functions
   */
  ccsCOMPL_STAT WriteToDB();
  ccsCOMPL_STAT ReadFromDB();
private:
  /* the list itself */   
  fcdDLListPtr  list;
  
  /*
   *# Function to free all members allocated
   */
  void FreeList();
};


/*****************************************************************
    INLINE METHODS
*****************************************************************/

#endif /*!_H*/
