/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdDETECTOR.h,v 2.140 2005/10/17 15:16:57 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  04/04/97  created 
* rdonalds  28/10/98  Added accessors for DoubleBuffer and MaxFrames
* rdonalds  20/01/99  Added accessors for AODetector and StopRead
* abalestr  21/01/04  Added accessors for SyncMaster
* abortolu  17/01/05  Now fcdDETECTOR::SimSLCU() to detects also WS sim
*/

/************************************************************************
*   NAME
*   
* 
*   SYNOPSIS
*
*   
*   PARENT CLASS
*
* 
*   DESCRIPTION
*
*
*   PUBLIC METHODS
*
*
*   PUBLIC DATA MEMBERS
*
*
*   PROTECTED METHODS
*
*
*   PROTECTED DATA MEMBERS
*
*
*   PRIVATE METHODS
*
*
*   PRIVATE DATA MEMBERS
*
*
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/
#ifndef FCDDETECTOR_H
#define FCDDETECTOR_H

#include "vltPort.h"

#include "fcdINTERNAL.h"
#include "fcdSlcuInternal.h"
#include "fcdDETID.h"
#include "fcdDETCONFIG.h"
#include "fcdDETTIMES.h"
#include "fcdDBDIRECT.h"
#include "fcdEXPOSURE.h"
#include "fcdTELEMETRY.h"

/*
 * fcdT_DETECTOR
 *
 * Sructure which goes into shared memory holding the detector information
 */
typedef struct {
    fcdCAMERANAME  name;          /* cameras name */
    vltINT32       detIndex;      /* index in the global structure */
  
    fcdOPMODE      opMode;        /* opmode (read from DB) SIM_WS SIM_SLCU ... */

    fcdSHT_STATUS  shtState;      /* shutter status OPENING .. ccdSHT_STATUS */
  
    vltINT32       state;         /* fcdSTATE off, standby, etc */
                                  /* mapped to .opStateSlcu     */
    vltINT32       failed;        /* boolean indication of failure */
                                  /* mapped to .failureSlcu     */

    vltINT32       rdtStatus;     /* Status of the last readout of this det 
				     used only for display purposed */

    vltINT32       rdtState;      /* fcdrdtONLINE_IDLE, fcdrdtONLINE_WIPE 
				     not realy a function of readout, but this
				     matches the original definition 
				     updated by WipeStart(), WipeEnd(),
				     ReadStart() ReadEnd() */

    vltINT32       readPercent;   /* percentage of readout complete */
    vltINT32       tranPercent;   /* percentage of transfer complete */


    vltINT32       nextExpId;     /* next exposure id to generate */

    fcdT_DETTIMES  detTimes;      /* recorded times for various events */

    fcdT_SETUP     nextSetup;     /* the next setup to be executed */
 
    fcdEXP_STATUS  expStatus;     /* overall status of all exposures */

    fcdT_EXPOSURE  exposures[fcdMAXEXP]; /* the shared exposure information */

    fcdT_TELEMETRY telemetry;     /* telemetry for detector */

    vltLOGICAL     stopRead;      /* flag indicating that the read  */
                                  /* should stop */

    /***********************************************************************
      CONFIGURATION INFORMATION FROM DATABASE
      ***********************************************************************/

    fcdFILENAME    imagePath;     /* path for detector images */
    vltINT32       numFITSBlocks; /* num of FITS header blocks for FITS files */

    /* attribute images:transfer:transferMethod */
    vltLOGICAL     diskSaveNFS;
    /* attribute images:display:displayLocal */
    vltLOGICAL     displayLocal;

    vltDOUBLE       wipePeriod;    /* time between wipes */
    fcdSEQUENCENAME wipeSeq;       /* sequence name */
    fcdSEQUENCENAME frameTransferSeq;  /* sequence name */

} fcdT_DETECTOR;

class fcdGLOBAL;

/*
 * fcdDETECTOR
 *
 * Global configuration and dynamic information for a DETECTOR
 */
class fcdDETECTOR
{
    friend class fcdGLOBAL;
  public:
    /*
     *# Reload the configuration from the database
     */
    ccsCOMPL_STAT ReloadConfig();

    /* return the name of the detector */
    fcdCAMERANAME &Name() {return shDet->name;}

    /* return the detectors index in the global structure */
    vltINT32      DetIndex() {return shDet->detIndex;}

    /*********************************************************************
                            LOCKING FUNCTIONS (via GLOBAL)
  *********************************************************************/
    ccsCOMPL_STAT CheckExclusive();
    ccsCOMPL_STAT CheckSetExclusive();
    ccsCOMPL_STAT WaitSetExclusive();
    ccsCOMPL_STAT ReleaseExclusive();

  /*********************************************************************
                            DYNAMIC STATE 
  *********************************************************************/

  /* Set and retrieve exposure state of Detector */
    fcdEXP_STATUS ExpStatus() {return (shDet->expStatus);}
    ccsCOMPL_STAT ExpStatus (fcdEXP_STATUS);

    /* set and retrieve the flag indicating that the read should stop */
    vltLOGICAL    StopRead() {return (shDet->stopRead);}
    void          StopRead (vltLOGICAL v) {shDet->stopRead = v;};


  /* return op Mode */
    fcdOPMODE OpMode(){return shDet->opMode;}
    /* indicate if local simulation (no communication with DSP) */
    vltLOGICAL SimSLCU() {return (OpMode() == fcdSIM_WS || OpMode() == fcdSIM_SLCU);}

    /* is save to disk being done on NFS disk */
    vltLOGICAL DiskSaveNFS() {return (shDet->diskSaveNFS);}
    /* is there a local RTD process being used */
    vltLOGICAL DisplayLocal() {return (shDet->displayLocal);}

    /* Set and retrieve shutter status */
    fcdSHT_STATUS ShtState() {return shDet->shtState;}
    ccsCOMPL_STAT ShtState (fcdSHT_STATUS);

  /* update the state of the detector off, online etc */
    ccsCOMPL_STAT State (fcdSTATE state);
    fcdSTATE State() {return (fcdSTATE)shDet->state;}
    ccsCOMPL_STAT Failed (vltINT32 failed);
    vltINT32 Failed() {return shDet->failed;}

    /* update the state of the ONLINE_WIPE ONLINE_READ state */
    ccsCOMPL_STAT RdtState (fcdrdtSTATE state);
    fcdrdtSTATE RdtState() {return (fcdrdtSTATE)shDet->rdtState;}
    ccsCOMPL_STAT RdtStatus (fcdrdtSTATUS status);

  /*********************************************************************
                            FRAME TRANSFER 
  *********************************************************************/
    fcdSEQUENCENAME &FrameTransferSequence () {return shDet->frameTransferSeq;}
    vltLOGICAL       FrameTransfer() {return detConfig.FrameTransfer();}

    /*********************************************************************
                            WIPE 
  *********************************************************************/
    fcdSEQUENCENAME &WipeSequence () {return shDet->wipeSeq;}
    vltDOUBLE       WipePeriod () {return shDet->wipePeriod;}
  
    // Record wipe timing and status
    ccsCOMPL_STAT   WipeStart();
    ccsCOMPL_STAT   WipeEnd();
  
  // Read timing and status
    ccsCOMPL_STAT   ReadStart();
    ccsCOMPL_STAT   ReadPercent (vltINT32 pc);
    ccsCOMPL_STAT   ReadEnd(vltDOUBLE readTime);
    ccsCOMPL_STAT   ReadFailed();
    ccsCOMPL_STAT   ReadAborted();

    // Transfer timing and status
    ccsCOMPL_STAT   TranStart();
    ccsCOMPL_STAT   TranPercent (vltINT32 pc);
    ccsCOMPL_STAT   TranEnd(vltDOUBLE tranTime);


    /*********************************************************************
                     EXPOSURE ID HANDLING
  *********************************************************************/
  /* Set and retrieve nextExpId */
    vltINT32 NextExpId() {return shDet->nextExpId;}
    ccsCOMPL_STAT NextExpId (vltINT32);

  /*********************************************************************
                     TIMING
  *********************************************************************/
  /* retrieve the times field */
    fcdDETTIMES &DetTimes() {return (detTimes);}

    /*********************************************************************
                     CONFIGURATION
  *********************************************************************/

    /* retrieve the detId field */
    fcdDETID &DetId() {return (detId);}

    /* retrieve log Mask */
    char     *LogMask() {return (logMask);}

    /* return image path */
    fcdFILENAME &ImagePath() {return (shDet->imagePath);}
    vltINT32    NumFITSBlocks() {return (shDet->numFITSBlocks);}

    /* retrieve the detConfig fields */
    fcdDETCONFIG &DetConfig()   {return (detConfig);}
    fcdCONFIG    &Config()   {return (detConfig.Config());}
    vltINT32	TotalPixels() {return (detConfig.TotalPixels());}
    vltINT32	XTotalPixels()    {return (detConfig.XTotalPixels());}
    vltINT32	YTotalPixels()    {return (detConfig.YTotalPixels());}
    vltINT32	TotalPixelBytes()    {return (detConfig.TotalPixelBytes());}
    vltINT32	BitsPixel()   {return (detConfig.BitsPixel());}
    vltINT32	BytesPixel()  {return (detConfig.BytesPixel());}
    vltLOGICAL  DoubleBuffered() {return detConfig.DoubleBuffered();}
    vltLOGICAL  SyncMaster() {return detConfig.SyncMaster();}
    vltLOGICAL  AODetector() {return detConfig.AODetector();}
    vltINT32    MaxFrames() {return detConfig.MaxFrames();}

    /*********************************************************************
                     EXPOSURE and NEXT SETUP access
  *********************************************************************/
  /* return the exposure pointer */
    fcdEXPOSURE   *Exposure(vltINT32 expIndex);
  
    /* commands for manipulating setup */
    fcdCSETUP     &NextSetup() {return (nextSetup);}
  
    /* commands for accessing tm  */
    fcdTELEMETRY  &Telemetry() {return (telemetry);}


    /*********************************************************************
                     ACCESS TO GLOBAL FOR COORDINATION
  *********************************************************************/
    fcdGLOBAL     *Global() {return (global);}

  private:
    /* c'tor d'tor are private because this object is only accessed by */
    /* indexing the fcdGLOBAL structure                                */
    fcdDETECTOR();
    ~fcdDETECTOR();

  /*
   *# Create, attach, detach are private. They should be accessed 
   *# through the fcdGLOBAL object
   */
    ccsCOMPL_STAT Create (fcdGLOBAL *, fcdT_DETECTOR *,
			  fcdCAMERANAME, vltINT32 index);
    ccsCOMPL_STAT Attach (fcdGLOBAL *, fcdT_DETECTOR *);
    ccsCOMPL_STAT Detach();

    /*
   *# Initialise function common to create and attach
   *# which inialises the fields local to this object
   *# (not in shared memory)
   */
    ccsCOMPL_STAT InitLocal();

    /* Initialises the T_DETECTOR structure in memory (for Create())  */
    ccsCOMPL_STAT CreateT_DETECTOR();

    /*
   *###########################################
   *# information which is local to this object
   *###########################################
   */
    fcdDETID         detId;          /* name and environment for CAMERA       */
    fcdDETCONFIG     detConfig;      /* configuration of CHIPs etc from OL-DB */
    logMASK          logMask;        /* log mask for FITS logging */

  /*
   *###########################################
   *# Used to map from memory into the DB
   *###########################################
   */
    fcdDBDIRECT      mapShtState;     /* map shutter status */
    fcdDBDIRECT      mapState;        /* map detector state */
    fcdDBDIRECT      mapFailed;       /* map detector failed state */
    fcdDBDIRECT      mapRdtState;     /* map rdt state, used for WIPE status */
    fcdDBDIRECT      mapRdtStatus;    /* map last rdt status, used for display */

    fcdDBDIRECT      mapReadPercent;  /* Pc of readout complete */
    fcdDBDIRECT      mapTranPercent;  /* Pc of transfer complete */

  /*
   *###########################################
   *# information which is in shared memory
   *###########################################
   */
  /* next setup buffer */
    fcdCSETUP         nextSetup;

    /* timing information for this detector (last wipe etc) */
    fcdDETTIMES      detTimes;

    /* the exposures */
    fcdEXPOSURES      exposures;

    /* telemetry */
    fcdTELEMETRY      telemetry;

    /* shared detector information */
    fcdT_DETECTOR    *shDet;

    /* needed so we can access any of the global data */
    fcdGLOBAL        *global;
};
/*****************************************************************************
  INLINE METHODS
*****************************************************************************/

#endif /* FCDDETECTOR_H */
/*___oOo___*/
