/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdDatabase.h,v 2.140 2005/10/17 15:16:45 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  04/04/97  created 
* rdonalds  19/03/98  Added FITSLOGMASK
* rdonalds  20/04/98  Changed the definition of CON_OUTNAME from "description"
*                     to "id".
* rdonalds  23/07/98  Added clock speed and double buffer attributes
* rdonalds  27/08/98  Adde ron to mode
* abalestr  03/03/99  moved fcdDB_WCS_XREFPIX and fcdDB_WCS_YREFPIX to public 
*                     part of database - see spr no. VLTSW990082
* abalestr  25/03/99  moved simExp from public part of db here
* abalestr  25/03/99  added definition for WAIT status (ws simulation only)
* abalestr  21/06/99  moved form here to the public part of the db the 
*                     definition of primary and secondary FITS header size
* ccumani   14/03/01  comments corrected
* ccumani   26/04/01  "binning factors of readout windows" macros
*                     deleted (already in fcdDbPublic.h)
* abaruffo  23/11/01  Added definition of buffer size for disk writes
* abalestr  18/09/03  Added macro for rampUp point in the db
* abalestr  21/01/04  Added macro for syncMaster point in the db
* abalestr  05/03/04  Added macro for lcuLittleEndian point in the db
* abalestr  05/03/04  Added macro for wsLittleEndian point in the db
*/

/************************************************************************
*   NAME
*   
* 
*   SYNOPSIS
*
*   
*   PARENT CLASS
*
* 
*   DESCRIPTION
*
*
*   PUBLIC METHODS
*
*
*   PUBLIC DATA MEMBERS
*
*
*   PROTECTED METHODS
*
*
*   PROTECTED DATA MEMBERS
*
*
*   PRIVATE METHODS
*
*
*   PRIVATE DATA MEMBERS
*
*
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/
#ifndef FCDDATABASE_H
#define FCDDATABASE_H


/*
 * Database points
 */

#define fcdDB_OBS_POINT       "observations:observation_1"

#define fcdDB_POINT_DET       "detector"
#define fcdDB_POINT_PRC       "process"
#define fcdDB_POINT_PRC_WINi  "process:window_%i"
#define fcdDB_POINT_XFR       "transfer"
#define fcdDB_POINT_IMG       "images"
#define fcdDB_POINT_RTD       "display"
#define fcdDB_POINT_RDT       "readout"
#define fcdDB_POINT_RDT_WINi  "readout:window_%i"
#define fcdDB_POINT_EXP       "exposures"
#define fcdDB_POINT_EXP_NEXT  "exposures:exposure_next"
/*#define fcdDB_POINT_EXP_EXPi  "exposures:exposure_%i"*/
#define fcdDB_POINT_SHT       "shutter"
#define fcdDB_POINT_TEL       "telemetry"
#define fcdDB_POINT_TMP       "temperature"

/*
 * Configuration Attributes
 */

/* Is the ws a big endian (sparc) or a little endian (intel) ? */
#define fcdDB_CON_WSARCH "wsLittleEndian"
 
/* Is the lcu a big endian (sparc) or a little endian (intel) ? */
#define fcdDB_CON_LCUARCH "lcuLittleEndian"
 
/* 
 * exposure coordination 
 */
 
/* FITS log mask */
#define fcdDB_CON_FITSLOGMASK ".logMask"

/* frame transfer CCD */
#define fcdDB_CON_FRAMETRAN   "detector.frameTransfer"

/* skipper CCD */
#define fcdDB_CON_SKIPPER     "detector.skipper"

/* CCD supporting mpp mode */
#define fcdDB_CON_MPP         "detector.mpp"

/* number of outputs */
#define fcdDB_CON_NOUTS       "detector.nOutputs"

/* pixel x dimension */
#define fcdDB_CON_XPIXSZ      "detector.xPixelSize"

/* pixel y dimension */
#define fcdDB_CON_YPIXSZ      "detector.yPixelSize"

/* clock speed of detector electronics (MHz) */
#define fcdDB_CON_CLOCK_SPEED        "detector.clockSpeed"

/* do we need to support two exposures in memory ? */
#define fcdDB_CON_DOUBLE_BUFFERED        "detector.doubleBuffered"

/* Are we synchronization masters? */
#define fcdDB_CON_SYNCMASTER "detector.syncMaster"

/* table of output amplifiers */
#define fcdDB_CHIP_OUT        "detector.outputAmplifier"
#define fcdDB_CHIP_OUT2        "detector.outputAmplifier2"

/* chip availabilty - under fcdDB_CHIP_INST */

#define fcdDB_CON_CHIPAVAIL   "available"

/* chip x location - under fcdDB_CHIP_INST */
#define fcdDB_CON_CHIPXLOC   "xLocation"

/* chip y location - under fcdDB_CHIP_INST */
#define fcdDB_CON_CHIPYLOC   "yLocation"

/* chip xGap - under fcdDB_CHIP_INST */
#define fcdDB_CON_CHIPXGAP   "xGap"

/* chip yGap - under fcdDB_CHIP_INST */
#define fcdDB_CON_CHIPYGAP   "yGap"

/* chip flipped - under fcdDB_CHIP_INST */
#define fcdDB_CON_CHIPFLIPPED   "flipped"

/* chip x location - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTXLOC   "xLocation"

/* chip y location - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTYLOC   "yLocation"

/* output hor direction - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTHDIR   "leftToRight"

/* output ver direction - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTVDIR   "downToUp"

/* output x prescan - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTXPRE   "xPrescan"

/* output y prescan - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTYPRE   "yPrescan"

/* output x prescan - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTXPRED   "xPredump"

/* output y prescan - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTYPRED   "yPredump"

/* output x active pixels - under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTXPIX   "xPix"

/* output y active pixels under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTYPIX   "yPix"

/* output x overscan under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTXOVE   "xOverscan"

/* output y overscan under fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTYOVE   "yOverscan"

/* output identification fcdDB_CHIP_OUT */
#define fcdDB_CON_OUTNAME   "id"


/* 
 * readout and wiping 
 */

/* bits per pixel */
#define fcdDB_CON_BITSPIX     "readout.bitsPixel"

/* wipe periode */
#define fcdDB_CON_WINT   "readout.wipePeriod"

/* wipe sequence */
#define fcdDB_CON_WSEQ      "readout.wipeSeq"
#define fcdDB_CON_FTRANSEQ  "readout.frameTransferSeq"

/* DCL prefix */
#define fcdDB_CON_PREFIX "readout.hwPrefix"

/* mode description - under fcdDB_POINT_RDT_MODE_<index> */

#define fcdDB_CON_MAINT     "maintOnly"
#define fcdDB_CON_OUTPUT    "outEnab"
#define fcdDB_CON_OUTFIRST  "firstOutput"
#define fcdDB_CON_GAIN      "gain"

#define fcdDB_CON_WIPEV     "wipeVolts"
#define fcdDB_CON_INTV      "intVolts"
#define fcdDB_CON_READV     "readVolts"

#define fcdDB_CON_WIPESEQ    "wipeSeq"
#define fcdDB_CON_INTSEQ     "intSeq"
#define fcdDB_CON_DURINGINTSEQ     "duringIntSeq"

#define fcdDB_CON_READSEQNAME         "readSeq"
#define fcdDB_CON_READSEQFACTX        "binFactX"
#define fcdDB_CON_READSEQFACTY        "binFactY"

#define fcdDB_CON_WIPEALWAYS "wipeAlways"
#define fcdDB_CON_RAMPUP "rampUp"
#define fcdDB_CON_UNORDDATA  "unordData"

#define fcdDB_CON_ABSTIMEMODE      "absTimeMode"
#define fcdDB_CON_ABSTIMEWHICHSEQ  "absTimeWhichSeq"
#define fcdDB_CON_ABSTIMEOFFSET    "absTimeOffset"

/* useqs definitions */
#define fcdDB_CON_USEQ_PDMPSPD        "useqs.parDumpSpeed"
#define fcdDB_CON_USEQ_PSHFSPD        "useqs.parShiftSpeed"
#define fcdDB_CON_USEQ_SDMPSPD        "useqs.serDumpSpeed"
#define fcdDB_CON_USEQ_SSHFSPD        "useqs.serShiftSpeed"
#define fcdDB_CON_USEQ_SREASPD        "useqs.serReadSpeed"


/* 
 * image transfer 
 */

/* transfer method  */
/* 0 == dfx, 1 == NFS */
#define fcdDB_CON_TRANSF    "images:transfer.transferMethod"

/* are image extensions used? */
/* 0 is no, 1 is yes */
#define fcdDB_CON_IMGEXT    "images:transfer.imageExtensions"

/* dxf packet Size  */
#define fcdDB_CON_DXFSIZE   "images:transfer.itPacketSize"

/* buffer size for disk writes */
#define fcdDB_CON_WRITEBUFSIZE   "images:transfer.itBufferSize"

/* buffer size for disk writes */
#define fcdDB_CON_WRITEBUFSTEP   "images:transfer.itBufferStep"

/* display local ? */
/* o == NO, 1 == YES */
#define fcdDB_CON_DISPLOCAL "images:display.localDisplay"

/* 
 *  telemetry
 */

/* default sampling period for TEL/TMP */
#define fcdMONITOR_PERIOD    5           

/* default logging period for TEL/TMP (10 min.) */
#define fcdLOG_PERIOD    600           

/*  */
#define fcdDB_CON_TELAVAIL  "telemetry.available"

/* 
 * temperature 
 */

/*  */
#define fcdDB_CON_TMPAVAIL  "temperature.available"

/* 
 * shutter 
 */

/*  */
#define fcdDB_CON_SHTAVAIL  "shutter.available"

/*  */
#define fcdDB_CON_SHTTYPE   "shutter.type"

/*  discriminator for PULPO, DIGITAL, EXTERNAL */
#define fcdDB_CON_SHTTYPEID "shutter.typeId"

/*  */
#define fcdDB_CON_SHTDESCR  "shutter.description"

/*  */
#define fcdDB_CON_SHTHWDESC "shutter.hwDescription"

/*  */
#define fcdDB_CON_SHTTIMOUT "shutter.timeout"


/* 
 * DCL's 
 */

/* DCL device name */
#define fcdDB_CON_DCLDEV    "ace.devName"

/* DCL device timeout */
#define fcdDB_CON_DCLTOUT   "ace.devTimeout"


/*
 * Setup Attributes
 */
 
/* 
 * EXPOSURE COORDINATION 
 */

/* defined exposure time start */
#define fcdDB_SET_TIMESTART "exposures:exposure_next.timeStart"

/* times are absolute/relative */
#define fcdDB_SET_TIMEABS   "exposures:exposure_next.timeAbsolute"

/* defined exposure repetitions */
#define fcdDB_SET_FRMTYPE   "exposures:exposure_next.frameType"


/* 
 * IMAGE TRANSFER 
 */

/* transfer synchronous or not */
#define fcdDB_SET_XFERSYNC  "exposures:exposure_next:transfer.itSync"

/* disk save method */
#define fcdDB_SET_DISKSAVE  "exposures:exposure_next:transfer.diskSaveMethod"

/* name of uncompressed FITS file */
#define fcdDB_SET_FILEUNC   "exposures:exposure_next:transfer.fileNameUnComp"
#define fcdDB_SET_FILEUNC_1 "exposures:exposure_1:transfer.fileNameUnComp"

/* name of compressed FITS file */
#define fcdDB_SET_FILECOM   "exposures:exposure_next:transfer.fileNameComp"

/* pixel representation */
#define fcdDB_SET_PIXREPR   "exposures:exposure_next:transfer.itPixRepr"

/* bit shift */
#define fcdDB_SET_BITSHIFT  "exposures:exposure_next:transfer.itBitShift"

/* compression mode */
#define fcdDB_SET_COMPRESS  "exposures:exposure_next:transfer.itCompression"

/* transfer sampling */
#define fcdDB_SET_SAMPLING  "exposures:exposure_next:transfer.itSampling"

/* 
 * REAL TIME DISPLAY 
 */

/* real-time display flag */
#define fcdDB_SET_DISPLAY   "exposures:exposure_next:display.frameId"

/* 
 * IMAGE PROCESSING
 */

/* Search for extrema */
#define fcdDB_SET_WIN1_MINMAX    "exposures:exposure_next:process:window_1.ipMinMax"
#define fcdDB_SET_WIN2_MINMAX    "exposures:exposure_next:process:window_2.ipMinMax"

#define fcdDB_SET_WINi_MINMAX    "exposures:exposure_next:process:window_%d.ipMinMax"

/* Search for extrema */
#define fcdDB_SET_WIN1_RMS    "exposures:exposure_next:process:window_1.ipRMS"
#define fcdDB_SET_WIN2_RMS    "exposures:exposure_next:process:window_2.ipRMS"

#define fcdDB_SET_WINi_RMS    "exposures:exposure_next:process:window_%d.ipRMS"

/* 
 * READOUT & WIPE
 */
 
/* number of lines to shift */
#define fcdDB_SET_LINESHIFT "exposures:exposure_next:readout.linesToShift"

/* number of times the same pixel is read (skipper CCDs only) */
#define fcdDB_SET_READPIX   "exposures:exposure_next:readout.readPixelN"

/* MPP mode */
#define fcdDB_SET_MPPMODE   "exposures:exposure_next:readout.mpp"

/* read-out clock to use */
#define fcdDB_SET_CLOCK     "exposures:exposure_next:readout.clockIndex"

/* read-out gain index */
#define fcdDB_SET_GAIN      "exposures:exposure_next:readout.gainIndex"

/* read-out mode index */
#define fcdDB_SET_MODEINDEX "exposures:exposure_next:readout.modeIndex"

/* readout windows */
#define fcdDB_SET_WIN1_ENAB "exposures:exposure_next:readout:window_1.enabled"
#define fcdDB_SET_WIN2_ENAB "exposures:exposure_next:readout:window_2.enabled"
#define fcdDB_SET_WIN1_ENAB_1 "exposures:exposure_1:readout:window_1.enabled"
#define fcdDB_SET_WIN2_ENAB_1 "exposures:exposure_1:readout:window_2.enabled"

#define fcdDB_SET_WINi_ENAB "exposures:exposure_next:readout:window_%d.enabled"
#define fcdDB_SET_WINi_ENAB_1 "exposures:exposure_1:readout:window_%d.enabled"

/* windows first and last pixels */
#define fcdDB_SET_WIN1_XFIR "exposures:exposure_next:readout:window_1.xFirst"
#define fcdDB_SET_WIN1_XDIM "exposures:exposure_next:readout:window_1.xDim"
#define fcdDB_SET_WIN1_YFIR "exposures:exposure_next:readout:window_1.yFirst"
#define fcdDB_SET_WIN1_YDIM "exposures:exposure_next:readout:window_1.yDim"
#define fcdDB_SET_WIN2_XFIR "exposures:exposure_next:readout:window_2.xFirst"
#define fcdDB_SET_WIN2_XDIM "exposures:exposure_next:readout:window_2.xDim"
#define fcdDB_SET_WIN2_YFIR "exposures:exposure_next:readout:window_2.yFirst"
#define fcdDB_SET_WIN2_YDIM "exposures:exposure_next:readout:window_2.yDim"
#define fcdDB_SET_WIN1_XFIR_1 "exposures:exposure_1:readout:window_1.xFirst"
#define fcdDB_SET_WIN1_XDIM_1 "exposures:exposure_1:readout:window_1.xDim"
#define fcdDB_SET_WIN1_YFIR_1 "exposures:exposure_1:readout:window_1.yFirst"
#define fcdDB_SET_WIN1_YDIM_1 "exposures:exposure_1:readout:window_1.yDim"
#define fcdDB_SET_WIN2_XFIR_1 "exposures:exposure_1:readout:window_2.xFirst"
#define fcdDB_SET_WIN2_XDIM_1 "exposures:exposure_1:readout:window_2.xDim"
#define fcdDB_SET_WIN2_YFIR_1 "exposures:exposure_1:readout:window_2.yFirst"
#define fcdDB_SET_WIN2_YDIM_1 "exposures:exposure_1:readout:window_2.yDim"

#define fcdDB_SET_WINi_XFIR "exposures:exposure_next:readout:window_%d.xFirst"
#define fcdDB_SET_WINi_XDIM "exposures:exposure_next:readout:window_%d.xDim"
#define fcdDB_SET_WINi_YFIR "exposures:exposure_next:readout:window_%d.yFirst"
#define fcdDB_SET_WINi_YDIM "exposures:exposure_next:readout:window_%d.yDim"
#define fcdDB_SET_WINi_XFIR_1 "exposures:exposure_1:readout:window_%d.xFirst"
#define fcdDB_SET_WINi_XDIM_1 "exposures:exposure_1:readout:window_%d.xDim"
#define fcdDB_SET_WINi_YFIR_1 "exposures:exposure_1:readout:window_%d.yFirst"
#define fcdDB_SET_WINi_YDIM_1 "exposures:exposure_1:readout:window_%d.yDim"

/* output enabled flags - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_SET_OUTPUTEN  "readout.readOutputEn"

/* 
 * shutter 
 */
/* none */


/*
 * Status Attributes
 */

/* 
 * exposure coordination 
 */
/* last exposure started ID */
#define fcdDB_STA_LASTEXPID "exposures.lastExpId"          

/* CCD sw WS state */
#define fcdDB_STA_WS     ".opStateWs"

/* CCD sw SLCU state */
#define fcdDB_STA_LCU     ".opStateSlcu"

/* CCD sw SLCU failure state (nominaly 0) */
#define fcdDB_FAIL_LCU     ".failureSlcu"

/* All SLCU processes have a point under <alias>CCDNAME:slcuProcs" */
#define fcdDB_STA_SLCUPROCS "slcuProcs"

/* exposure type - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_EXPTYPE   "expType"

/* single exposure status - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_SINGLEEXP   "singleExpStatus"

/* elapsed exposure time for each step - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMEELA    "timeEla"   

/* elapsed exposure time for multi step - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMEELAMULTI    "timeElaMulti"   

/* elapsed dark current time for each step - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMEDARK   "timeDark"

/* start of first step integration start - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMESTART  "timeStarted"

/* end of last step integration time - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMEENDED  "timeEnded" 

/* exposure repetition count - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_EXPREPS    "expRepeatCur" 

/* ws simulation: simulated exposure status*/
#define fcdDB_STA_SIMEXP    "simExpStatus"

/* ws simulation: simulated wait status i.e. someone is waiting... */
#define fcdDB_STA_SIMWAIT   "simExpWait"

/* 
 * readout and wiping 
 */
 
/* wiping start time */
#define fcdDB_STA_WIPESTART  "readout.wipeStarted"

/* time needed for one wiping */
#define fcdDB_STA_WIPEEND    "readout.wipeTime"

/* readout module state */
#define fcdDB_STA_MSTA    "readout.opState"

/* completion status   */
#define fcdDB_STA_CSTA    "readout.status"

/* percentage of image read     */
#define fcdDB_STA_PERCENT "readout.percent"

/* gain factor for selected speed - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_GAIN    "readout.gainIndex"

/* ADC time for sel. speed - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_ADCTIME "readout.readAdcTime" 

/* information about outputs - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_ELEADU  "readout.readEleAdu" 

/* readout clock index - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_CLOCK   "readout.clockIndex"

/* number of times the same pixel is read - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_READPIX    "readout.readPixelN"

/* MPP mode - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_MPPMODE    "readout.mpp"

/* output enable - under fcdDB_EXP_POINT_<clockIndex> */
#define fcdDB_STA_OUTPUT    "readout.outEnab"

/* last line read */ 
#define fcdDB_STA_LINE    "readout.last"

/* read-out start time (mjd) */ 
#define fcdDB_STA_START   "readout.started"

/* read-out duration */ 
#define fcdDB_STA_TIME    "readout.time"

/* frame representation - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_REPR    "frameRepr"   

/* readout windows */
#define fcdDB_STA_ENAB "enabled"

/* binning factors of readout windows */
#define fcdDB_STA_XBIN "xBinning"
#define fcdDB_STA_YBIN "yBinning"

/* windows first and last pixels */
#define fcdDB_STA_XFIR "xFirst"
#define fcdDB_STA_XDIM "xDim"
#define fcdDB_STA_YFIR "yFirst"
#define fcdDB_STA_YDIM "yDim"

/* 
 * image transfer 
 */

/* transfer status */
#define fcdDB_IT_STA_TSTA    "images:transfer.opState"

/* transfer Compl Status */
#define fcdDB_IT_STA_CSTA    "images:transfer.status"

/* info for real time dis. */
#define fcdDB_IT_RTD         "images:display.frame"

/* transfer start time */
#define fcdDB_IT_STA_START   "images:transfer.started"

/* transfer duration */
#define fcdDB_IT_STA_TIME    "images:transfer.time"

/* image list */
#define fcdDB_STA_IMAGELST   "images.frame"

/* 
 * shutter 
 */

/* time when open action started */
#define fcdDB_STA_OPENSTART     "shutter.openStarted"

/* time needed to open */
#define fcdDB_STA_OPENTIME      "shutter.openTime"

/* time when close action started */
#define fcdDB_STA_CLOSESTART    "shutter.closeStarted"

/* time needed to close */
#define fcdDB_STA_CLOSETIME     "shutter.closeTime"

/*
 * database paths for common attributes 
 */

/* unknown attribute */
#define fcdDB_OBS_UNKNOWN ""

/* new data for Archive (used when in stand-alone) */
#define fcdDB_NEW_DATA_POINT    "<alias>newData"
#define fcdDB_NEW_DATA_FILE_UNC "newDataFileName"

/* attributes for WCS handling */
#define fcdDB_POINT_WCS    "wcs"
#define fcdDB_WCS_SECPIX   "wcs.secPix"
#define fcdDB_WCS_ROTATE   "wcs.rotate"
#define fcdDB_WCS_EQUINOX  "wcs.equinox"
#define fcdDB_WCS_EPOCH    "wcs.epoch"
#define fcdDB_WCS_PROJ     "wcs.proj"

#endif
/*___oOo___*/
