/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdDbPublic.h,v 2.140 2005/10/17 15:16:45 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  04/04/97  created
* rdonalds  xx/xx/97  changed IP1_MEANVAL etc to IP_MEANVAL_FMT 
*                     to allow scope for more windows
* ccumani   11/11/97  unused macros deleted
* rdonalds  25/05/98  Added centroiding 
* rdonalds  05/01/99  Increased MAXINTEGR to 100
* abalestr  03/03/99  moved here fcdDB_WCS_XREFPIX and fcdDB_WCS_YREFPIX from 
*                     private part of database - see spr no. VLTSW990082
* abalestr  21/06/99  moved here definition of primary and secondary FITS
*                     header size
* abalestr  04/09/00  fcdMAXDETECTORS from 2 to 4
* ccumani   14/03/01  comments corrected
* ccumani   14/03/01  fcdDB_TM_CURRENT and fcdDB_TM_OPSTATE added
* ccumani   26/04/01  fcdMAXMODES changed from 20 to 40
* abalestr  12/02/04  Added points for SNR & SD on background and fwhm when
*                     requested in image processing
* abalestr  25/02/05  Added points for centroiding error vector (xErr and yErr)
* ccumani   29/03/05  fcdMAXTELSRC increased from 100 top 120
*/

/*************************************************************************
*  This file contains all definitions for FIERA database public attributes
*************************************************************************/

#ifndef FCDDBPUBLIC_H
#define FCDDBPUBLIC_H

/* 
 * Definition of database vector and table dimensions. Keep consistant !
 * NOTE: This file is used by dbLoader as well. Nothing else than define
 *       instructions or comment lines are possible here.
 */
/* These constants are actually not used directly in the database definition
 * files, but the database itself supports configuration for these values
 * only. To increase capabilities, modifications in the class definition
 * files are needed as well, in particular fcdDETECTOR and fcdEXPOSURES
 */
#define fcdMAXCHIPS   32       /* maximum number of chips in one mosaic      */
#define fcdMAXWINDOW  2       /* maximum number of windows for one readout  */
#define fcdMAXMODES  40       /* maximum number of mode            */
#define fcdMAXCLOCKS fcdMAXMODES
#define fcdMAXEXP     2       /* maximum number of expositions              */

/* These constants are used directly in the database definition files */
#define fcdMAXOUTCHIP 16      /* maximum number of outputs in one chip      */
#define fcdMAXOUTPUT  512
			      /* maximum number of outputs in mosaic 
				 (fcdMAXOUTCHIP * fcdMAXCHIPS)        */
#define fcdMAXOBJECTS 100     /* maximum number of objects in image proc.   */
#define fcdMAXINTEGR  100     /* maximum number of integrations in one exp. */
#define fcdMAXIMAGES  100     /* maximum number of images in memory         */
#define fcdMAXTELSRC  120     /* maximum number of telemetry sources        */
#define fcdMAXTMPSRC  10      /* maximum number of temperature sensors      */
#define fcdMAXREADSEQ  8      /* maximum number of readout sequences / binning  
				 factors      */

/* Public part of the CCD database */

/* operational mode */
#define fcdDB_CON_OPMODE      ".opMode"

/* system state */
#define fcdDB_STA_SYSTEM      ".opState"

/* system errors */
#define fcdDB_STA_FAILURE_WS  ".failureWs"
#define fcdDB_STA_FAILURE_LCU ".failureSlcu"

/* default image file directory  */
#define fcdDB_CON_IMGPATH     "images.imageDirectory"

/* running exposures prefix  */
#define fcdDB_EXP_POINT       "exposures:exposure"

/* exposure status - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_EXPOSURE    "expStatus"

/* exposure ID - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_EXPID       "expId"

/* remaining exposure time for each step - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_STA_TIMEREM     "timeRem"  

/* name of uncompressed FITS file - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_EXP_FILEUNC     "transfer.fileNameUnComp"

/* name of compressed FITS file - under fcdDB_EXP_POINT_<expIndex> */
#define fcdDB_EXP_FILECOM     "transfer.fileNameComp"

/* Centroiding Error vector */
#define fcdDB_IP_CEN_XERRV    "images:process:window_%d.ipXerrv"
#define fcdDB_IP_CEN_YERRV    "images:process:window_%d.ipYerrv"

/* Intensity of image center, as computed from centroiding algorythm */
#define fcdDB_IP_XCEN_FMT    "images:process:window_%d.ipXCen"
#define fcdDB_IP_YCEN_FMT    "images:process:window_%d.ipYCen"
#define fcdDB_IP_CENVAL_FMT  "images:process:window_%d.ipCenVal"

/* SNR */
#define fcdDB_IP_SNR_FMT  "images:process:window_%d.ipSNR"
/* background level */
#define fcdDB_IP_BACKGRND_FMT  "images:process:window_%d.ipBackGnd"
#define fcdDB_IP_BACKGRND_SD_FMT  "images:process:window_%d.ipBackGnd_SD"
/* Full Width Half Max */
#define fcdDB_IP_X_FWHM_FMT  "images:process:window_%d.ipXFWHM"
#define fcdDB_IP_X_FWHM_SD_FMT  "images:process:window_%d.ipXFWHM_SD"
#define fcdDB_IP_Y_FWHM_FMT  "images:process:window_%d.ipYFWHM"
#define fcdDB_IP_Y_FWHM_SD_FMT  "images:process:window_%d.ipYFWHM_SD"
/* number of pixels within threshold */
#define fcdDB_IP_NUMPIX_FMT  "images:process:window_%d.ipNumPix"

/* Minimum/Maximum/Mean value of image */
#define fcdDB_IP_MINVAL_FMT   "images:process:window_%d.ipMinVal"
#define fcdDB_IP_XMIN_FMT     "images:process:window_%d.ipXMin"
#define fcdDB_IP_YMIN_FMT     "images:process:window_%d.ipYMin"

#define fcdDB_IP_MAXVAL_FMT   "images:process:window_%d.ipMaxVal"
#define fcdDB_IP_XMAX_FMT     "images:process:window_%d.ipXMax"
#define fcdDB_IP_YMAX_FMT     "images:process:window_%d.ipYMax"

/* Mean */
#define fcdDB_IP_MEANVAL_FMT  "images:process:window_%d.ipMean"

/* RMS Calculation */
#define fcdDB_IP_RMS_FMT       "images:process:window_%d.ipRMS"

/* IP Computation completed */
#define fcdDB_IP_COMPLETED     "images:process.completed"

/* % of image transf. */
#define fcdDB_IT_STA_PERC     "images:transfer.percent"

/* primary and secondary header size */
#define fcdDB_CON_FITSSIZE	"images:transfer.itHeaderSize"
#define fcdDB_CON_SECFITSSIZE	"images:transfer.itSecHeaderSize"

/* last line transf. */
#define fcdDB_IT_STA_LINE     "images:transfer.last"

/* shutter status */
#define fcdDB_STA_SHTSTATUS   "shutter.status"

/* attributes for WCS handling */
#define fcdDB_WCS_RA          "wcs.ra"
#define fcdDB_WCS_DEC         "wcs.dec"
#define fcdDB_WCS_XREFPIX  "wcs.xRefPix"
#define fcdDB_WCS_YREFPIX  "wcs.yRefPix"

#define fcdMAXDETECTORS 4  /* max detectors supported by 1 FIERA */

/* read-out mode index */
#define fcdDB_SET_MODEINDEX "exposures:exposure_next:readout.modeIndex"

/* readout windows */
#define fcdDB_SET_WIN1_ENAB "exposures:exposure_next:readout:window_1.enabled"
#define fcdDB_SET_WIN2_ENAB "exposures:exposure_next:readout:window_2.enabled"
#define fcdDB_SET_WIN1_ENAB_1 "exposures:exposure_1:readout:window_1.enabled"
#define fcdDB_SET_WIN2_ENAB_1 "exposures:exposure_1:readout:window_2.enabled"

#define fcdDB_SET_WINi_ENAB "exposures:exposure_next:readout:window_%d.enabled"
#define fcdDB_SET_WINi_ENAB_1 "exposures:exposure_1:readout:window_%d.enabled"

/* binning factors of readout windows */
#define fcdDB_SET_WIN1_XBIN "exposures:exposure_next:readout:window_1.xBinning"
#define fcdDB_SET_WIN1_YBIN "exposures:exposure_next:readout:window_1.yBinning"
#define fcdDB_SET_WIN2_XBIN "exposures:exposure_next:readout:window_2.xBinning"
#define fcdDB_SET_WIN2_YBIN "exposures:exposure_next:readout:window_2.yBinning"
#define fcdDB_SET_WIN1_XBIN_1 "exposures:exposure_1:readout:window_1.xBinning"
#define fcdDB_SET_WIN1_YBIN_1 "exposures:exposure_1:readout:window_1.yBinning"
#define fcdDB_SET_WIN2_XBIN_1 "exposures:exposure_1:readout:window_2.xBinning"
#define fcdDB_SET_WIN2_YBIN_1 "exposures:exposure_1:readout:window_2.yBinning"

#define fcdDB_SET_WINi_XBIN "exposures:exposure_next:readout:window_%d.xBinning"
#define fcdDB_SET_WINi_YBIN "exposures:exposure_next:readout:window_%d.yBinning"
#define fcdDB_SET_WINi_XBIN_1 "exposures:exposure_1:readout:window_%d.xBinning"
#define fcdDB_SET_WINi_YBIN_1 "exposures:exposure_1:readout:window_%d.yBinning"

/* defined exposure time */
#define fcdDB_SET_TIMEDEF   "exposures:exposure_next.timeDef"
#define fcdDB_SET_TIMEDEF_1 "exposures:exposure_1.timeDef"

/* defined exposure time repeat */
#define fcdDB_SET_TIMEREP   "exposures:exposure_next.timeRepeat"
#define fcdDB_SET_TIMEREP_1 "exposures:exposure_1.timeRepeat"

/* exposure type */
#define fcdDB_SET_EXPTYPE   "exposures:exposure_next.expType"
#define fcdDB_SET_EXPTYPE_1 "exposures:exposure_1.expType"

/* defined exposure repetitions */
#define fcdDB_SET_EXPREPS   "exposures:exposure_next.expRepeatDef"
#define fcdDB_SET_EXPREPS_1 "exposures:exposure_1.expRepeatDef"

/* readout mode prefix  */
#define fcdDB_POINT_RDT_MODE  "readout:mode"

/* mode description - under fcdDB_POINT_RDT_MODE_<index> */
#define fcdDB_CON_ELADU          "elAdu"
#define fcdDB_CON_RON            "ron"
#define fcdDB_CON_SPEED          "speed"
#define fcdDB_CON_READSEQTABLE   "readSeqs"
#define fcdDB_CON_MODEDES        "description"

/* table of chip Instances */
#define fcdDB_CHIP_INST       "detector.chipInstances"

/* detector description */
#define fcdDB_DET_DESCR       "detector.description"

/* number of columns on chip */
#define fcdDB_CON_XPIX        "detector.xPixels"

/* number of rows on chip */
#define fcdDB_CON_YPIX        "detector.yPixels"

/* setup files dir. */
#define fcdDB_OBS_STP_DIR "observations.setupDirectory"

/* telemetry refresh period */
#define fcdDB_TM_PERIOD "telemetry.period"

/* telemetry logging period */
#define fcdDB_TM_LOGPERIOD "telemetry.logPeriod"

/* curent telemetry values */
#define fcdDB_TM_CURRENT  "telemetry.current"

/* state of telemetry monitoring */

/* crpix defaults */
#define fcdNO_DBCRPIX 99999. // Computed values for crpixes will be used


#endif /* ! FCDDBPUBLIC_H */
