
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdEXPOSURE.h,v 2.140 2005/10/17 15:16:58 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  15/04/96  created
* rdonalds  31/01/98  added TotalImagePixelBytes
* rdonalds  23/07/98  Added double buffer and ItSync methods
* rdonalds  12/10/98  Added ipResult
* rdonalds  28/10/98  Added manipulators for PrepareFrame, CurFrame, MaxFrame
* abalestr  27/10/03  Added fields/accessors for exposure started on external 
*                     signal in setup structure
*/

#ifndef FCDEXPOSURE_H
#define FCDEXPOSURE_H

/*
 * FCD header files
 */
#include "fcdINTERNAL.h"
#include "fcdCSETUP.h"
#include "fcdSHARED_DATA.h"
#include "fcdEXPOSURE_DATA.h"
#include "fcdSHUTTER.h"
#include "fcdtimsTIME.h"

class fcdEXPOSURES;
class fcdEXPOSURE;
class fcdEXPOSURE_DATA;
class fcdGLOBAL;

/*
 *# fcdT_EXP_TIMES
 *#
 *# Structure holding timing for exposure
 */
typedef struct
{
    fcdtimsTIME darkStart;         /* start of dark current integration time */
    vltDOUBLE   darkTime;          /* total dark time during integration     */
    fcdtimsTIME cycleStart;        /* start of cycle integration time        */
    vltDOUBLE   cycleEla;          /* elapsed cycle integration time         */
    fcdtimsTIME cycleEnd;          /* cycle end time                         */

    vltINT32    stepNum;           /* step number for MULTI exposures */

    vltDOUBLE   stepDef;           /* defined step integration time          */
    vltDOUBLE   stepDefMulti[fcdMAXINTEGR];
    vltDOUBLE   stepEla;           /* elapsed step integration time          */
    vltDOUBLE   stepElaMulti[fcdMAXINTEGR];
    vltDOUBLE   stepRem;           /* remaining step integration time        */
    vltDOUBLE   monitorStepEla;    /* the step values which are displayed    */
    vltDOUBLE   monitorStepRem;    /* and may not be completely accurate,    */
    
    vltDOUBLE   shutOpenTime;      /* shutter times */
    vltDOUBLE   shutCloseTime;
    /* the real values are update at cycle end */

    fcdtimsTIME expStart;          /* exposure  start time                    */
    fcdtimsTIME expEnd;            /* exposure  start time                    */

    fcdtimsTIME readStart;         /* readout  start time                    */
    fcdtimsTIME readEnd;           /* readout  end time                    */
    vltDOUBLE   readDuration;      /* readout  duration                      */
    fcdtimsTIME procStart;         /* process  start time                    */
    fcdtimsTIME procEnd;           /* process  end time                    */
    vltDOUBLE   procDuration;      /* process  duration                      */
    fcdtimsTIME tranStart;         /* transfer start time                    */
    fcdtimsTIME tranEnd;           /* transfer end time                    */
    vltDOUBLE   tranDuration;      /* transfer duration                      */

} fcdT_EXP_TIMES;

/*
 *# The IPRESULT result structure needs to be stored in the exposure
 *# so that the it process can send the result as part of the exposure
 *# header.
 */
typedef struct 
    {  
    /*
     * The result structure is filled in according to the setup
     * The first part (STATISTICS) is automatically updated 
     * whenever the centroid calculation or a user-function is
     * invoked. 
     * Since the two advanced image processing functions 
     * fcdipAG(3) and fcdipIQE(3) have the same interface (fcdIPUSERFUNC),
     * it is possible to design a more complex function complying the
     * interface and invoking these functions in the processing sequence.
     * However, the designer has to considere the computation times which
     * are involved (for 20x20 window) :
     *       fcdipAG  : <20ms
     *       fcdipIQE : 800ms
     */
	/* STATISTICS : fcdipStatistics() */
    vltDOUBLE    max;                 /* First Maximum Intensity             */
    vltINT32     xMax,yMax;           /* Location                            */
    vltDOUBLE    min;                 /* First Minimum Intensity             */
    vltINT32     xMin,yMin;           /* Location                            */
    vltDOUBLE    flux;                /* Raw Average ADC counts / pixel      */
    vltDOUBLE    sigma;               /* Standard-Deviation on the flux      */
    vltDOUBLE    SNR;                 /* Signal-to-Noise Ratio               */
    vltINT32     numPix;              /* Number of pixels between thresholds */
	/* CENTROID : fcdipCentroid() or fcdipAG() */
    vltDOUBLE    cen;                 /* Centroid Intensity (closest pixel)  */
    vltDOUBLE    xCen,yCen;           /* Location                            */
    vltDOUBLE    xErr,yErr;           /* Error vector (relative to Ref Point)*/
	/* ANALYSIS : fcdipIQE() */
    vltDOUBLE    xFWHM,xFWHM_SD;      /* X FWHM on object       + Std Dev    */
    vltDOUBLE    yFWHM,yFWHM_SD;      /* Y FWHM on object       + Std Dev    */
    vltDOUBLE    ObjAngle,ObjAngle_SD;/* Object position angle  + Std Dev    */
    vltDOUBLE    BckGnd,BckGnd_SD;    /* Back Ground estimation + Std Dev    */
    } fcdIPRESULT;

/*
 *# fcdT_EXPOSURE
 *#
 *# Holds all of the information about the exposure in shared memory
 */
typedef struct {
    dbSYMADDRESS      dbExpRootPt; /* root point in the oldb */
    fcdEXP_STATUS     expStatus;   /* status of the exposure           */
                                   /* a bit field see fcdWsCommon.h    */
    vltINT32	      expId;	   /* id of the exposure               */
    vltINT32          expIndex;    /* index of the exposure */

    fcdT_SETUP	      expSetup;	   /* Setup of the exposure            */
    vltINT32          repeatDef;   /* number of exposures in this seq  */
    vltINT32          expSeqNum;   /* the number of this exposure in seq */

    vltINT32          maxFrame;    /* for AO multi frame acquisition */
    vltINT32          curFrame;

    vltLOGICAL        fastExposure; /* indicates if the exposure is "fast"
					hence is done as local to exposure
					task */
    vltLOGICAL        doubleBuffered; /* use EXP(0) and EXP(1) */
    vltLOGICAL        aoExposure;     /* is this an adaptive optics exposure */

    fcdT_EXP_TIMES     expTimes;    /* timing information about exposure */
    vltINT32           readPercent; /* amount of readout complete. Updated
				       by NEW_READ_EXP. and used by the
				       it task to upate the DB */
    fcdT_EXPOSURE_DATA expData;
    
    fcdIPRESULT        ipResult[fcdMAXWINDOW]; /* image process results */

    vltLOGICAL         usingShutter; /* does this exposure use the shutter */
    vltINT32           frameTransferSeqId; /* sequence for frame transfer */

    vltLOGICAL         testImageLoaded; /* has a test image been loaded using
					  LOADIMG */

    fcdtimsTIME       nextAbsTime;     /* next ABS time to use if AbsTime() */

} fcdT_EXPOSURE;

/*
 *# fcdEXPOSURE
 *# 
 *# Class used to hide the implementation of the shared exposure structure
 *# 
 */
class fcdEXPOSURE {
    friend class fcdEXPOSURES;
  public:

    /*
     *# Method to initialise the Setup  and 
     *# for each image forming the exposure, and method called
     *# by processes which are attaching to the exposure
     *# 
     *# PrepareLoop is called once
     *# Prepare     is called for each iteration
     *#
     *# The methods with parameters are called by fcdexp to initialise
     *# the values of the exposure. The others are called by the
     *# processes which attach to the exposure.
     */
    ccsCOMPL_STAT	PrepareLoop (
				     fcdCSETUP       *setup,
				     fcdFIERA_CONFIG *fieraConfig
				     );
    ccsCOMPL_STAT	PrepareLoop ();

    ccsCOMPL_STAT	Prepare (
				 vltINT32	      expId,
				 vltINT32             expSeqNum
				 );
    ccsCOMPL_STAT	Prepare ();

    ccsCOMPL_STAT	PrepareFrame (
				 vltINT32	      frameSeq
				 );
    ccsCOMPL_STAT       PrepareReadFrame(vltINT32 frameSeq);

    /*
     *# Called when the exposure if finished, so we can mark the image
     *# buffers as no longer used.
     *# XXXX Can we call this directly from the ExpStatus() method
     */
    ccsCOMPL_STAT Completed();

    // Get and set the exposure status
    fcdEXP_STATUS	ExpStatus() {return (shExp->expStatus);}
    ccsCOMPL_STAT	ExpStatus(fcdEXP_STATUS newStatus);
    // Functions to set and unset individual bits
    ccsCOMPL_STAT	ExpAddStatus(fcdEXP_STATUS newStatus);
    ccsCOMPL_STAT	ExpRemoveStatus(fcdEXP_STATUS newStatus);
    // Update the status "scanned" to the WS. This normally
    // simply shows the exposure status. Unless the exposure
    // is in a loop in which case only Loop status is shown
    ccsCOMPL_STAT	ExpDisplayStatus(fcdEXP_STATUS newStatus);

    // Get and set the exposure id
    vltINT32	ExpId() {return (shExp->expId);}
    ccsCOMPL_STAT	ExpId(vltINT32 newId);
    vltINT32	ExpIndex() {return (shExp->expIndex);}

    /* which detector are we associated with and which global obkect */
    fcdDETECTOR   *Det() {return (det);}
    fcdGLOBAL     *Global();
  
    /* return exposures shared memory id */
    int           ShmId() {return (expData.ShmId());}

    /*******************************************************************
      MISC  ACCESSORS
      *******************************************************************/
    vltLOGICAL UsingShutter() {return (shExp->usingShutter);}
    void UsingShutter(vltLOGICAL v) {shExp->usingShutter = v;}
    vltINT32 FrameTransferSeqId() {return (shExp->frameTransferSeqId);}
    void     FrameTransferSeqId(vltINT32 s) {shExp->frameTransferSeqId = s;}

    vltLOGICAL FastExposure() {return (shExp->fastExposure);}
    vltLOGICAL DoubleBuffered() {return (shExp->doubleBuffered);}
    void       DoubleBuffered(vltLOGICAL v) {shExp->doubleBuffered = v;}
    vltLOGICAL AOExposure() {return (shExp->aoExposure);}
    void       AOExposure(vltLOGICAL v) {shExp->aoExposure = v;}

    fcdT_EXP_TIMES &ExpTimes() {return (shExp->expTimes);}
    fcdIPRESULT   &IpResult(vltINT32 i) {return (shExp->ipResult[i]);}
    vltLOGICAL    TestImageLoaded() { return (shExp->testImageLoaded);}
    void          TestImageLoaded(vltLOGICAL v) { shExp->testImageLoaded = v;}

    fcdtimsTIME   &NextAbsTime(){return shExp->nextAbsTime;}

    /*******************************************************************
      SETUP  ACCESSORS
      *******************************************************************/
    fcdCSETUP	&Setup() {return setup;}    // Setup
    vltLOGICAL    ItSync()        {return (setup.ItSync());}
    vltINT32      WipeAlways(){return (setup.WipeAlways());}
    vltINT32      WipeSeqId(){return (setup.WipeSeqId());}
    vltINT32      IntSeqId() {return (setup.IntSeqId());}
    vltINT32      DuringIntSeqId() {return (setup.DuringIntSeqId());}
    vltINT32      DuringIntSeqTicks() {return (setup.DuringIntSeqTicks());}
    vltINT32      ShiftSeqId() {return (setup.ShiftSeqId());}
    vltINT32      ReadSeqId(){return (setup.ReadSeqId());}
    vltINT32      WipeVoltTable(){return (setup.WipeVoltTable());}
    vltINT32      IntVoltTable(){return (setup.IntVoltTable());}
    vltINT32      ReadVoltTable(){return (setup.ReadVoltTable());}
    vltDOUBLE	ExpTime() {return (setup.ExpTime());}
    vltINT32      NumWindows() {return (setup.NumWindows());}
    vltDOUBLE     ReadTime() {return (setup.ReadTime());}
    vltINT32	ReadSeconds();
    fcdABS_TIME_MODE AbsTimeMode() {return setup.AbsTimeMode();}
    fcdABS_TIME_WHICH_SEQ AbsTimeWhichSeq() {return setup.AbsTimeWhichSeq();}
    vltDOUBLE     AbsTimeOffset() {return setup.AbsTimeOffset();}

    fcdREORDER_PIXEL_ORDER  &PixelOrder() {return (setup.PixelOrder());};

    vltINT32      XDim(int winNum) {return (setup.XDim(winNum));}
    vltINT32      YDim(int winNum) {return (setup.YDim(winNum));}

    fcdEXPTYPE    ExpType() {return (setup.ExpType());}
    vltLOGICAL    AbsTime() {return (setup.Setup().timeAbsolute);}
    vltLOGICAL    ShutAbsTime() {return (setup.Setup().timeShutAbsolute);}
    vltDOUBLE     TimeStart() {return (setup.Setup().timeStart);}
    vltDOUBLE     TimeRepeat() {return (setup.Setup().timeRepeat);}

    vltLOGICAL    ExpSync(){return setup.ExpSync();}

    // DoTransfer means display or save
    vltLOGICAL    DoTransfer() { return (setup.DoTransfer());}

    
    /*******************************************************************
      LOOP ACCESSORS
      *******************************************************************/
    vltINT32      RepeatDef() { return (shExp->repeatDef);}
    vltINT32      ExpSeqNum() { return (shExp->expSeqNum);}

    vltINT32      CurFrame() {return (shExp->curFrame);}
    vltINT32      MaxFrame() {return (shExp->maxFrame);}

    /*******************************************************************
      DATA  ACCESSORS
      *******************************************************************/
    fcdEXPOSURE_WINDOW *Window(vltINT32 winNum)
    {return (expData.Window(winNum));}  // Return window data
    fcdEXPOSURE_DATA   *ExpData()  // Return window data
    {return (&expData);}          // Return whole exposure data
    char          *Data(vltINT32 win) {return (Window(win)->Data());}

    /* total number of pixels including spaces between windows */
    vltINT32      TotalPixelBytes() {return (expData.TotalPixelBytes());}
    /* Total number of bytes in all windows */
    vltINT32      TotalWindowPixelBytes() 
        {return (expData.TotalWindowPixelBytes());}
    /* Total number of bytes in all images, not allways the same as */
    /* the amount of data in the windows, when binned windows cross */
    /* image boundaries                                             */
    vltINT32      TotalImagePixelBytes() 
        {return (expData.TotalImagePixelBytes());}

       /*********************************************************************
	 GENERAL TIMING MANIPULATION
	 *********************************************************************/
    /* update at "start" of exposure */
    ccsCOMPL_STAT ExpStart();
    ccsCOMPL_STAT ExpEnd();
    fcdtimsTIME   &ExpStartTime () {return (shExp->expTimes.expStart);}
    fcdtimsTIME   &ExpEndTime () {return (shExp->expTimes.expEnd);}
  
    /*********************************************************************
      INTEGRATION MANIPULATORS
      *********************************************************************/

    /* current step and number of steps defined */
    vltINT32      StepsDef() {return (setup.StepsDef());}
    vltINT32      StepNum() {return (shExp->expTimes.stepNum);}
    void          StepNum(vltINT32 v) {shExp->expTimes.stepNum = v;}

    /* update during integration */
    ccsCOMPL_STAT IntStart();

    /* change the stepDef time */
    ccsCOMPL_STAT IntChangeTime(vltDOUBLE newTime);

    /* methods called during PAUSE/CONTNUE cycles */
    ccsCOMPL_STAT IntCycleStart();
    ccsCOMPL_STAT IntCycleEnd();
    ccsCOMPL_STAT IntCycleUpdate(vltLOGICAL paused, vltDOUBLE *timeRem);
    /* IntCycleUpdate should be called before InTimeRemaining */
    vltDOUBLE     IntTimeRemaining();
    /* called at the end of the integration to fix up the times */
    ccsCOMPL_STAT IntEnd(vltINT32 endState, fcdSLCU_SHUTTER_TYPE shutType);
    /* the actual integration time which occurred */
    vltDOUBLE     ActualIntTime();
  

    /*********************************************************************
      READOUT MANIPULATORS
      *********************************************************************/
   
    /* update read start/end times and status*/
    ccsCOMPL_STAT ReadStart();
    ccsCOMPL_STAT ReadEnd();
    ccsCOMPL_STAT ReadFailed();
    ccsCOMPL_STAT ReadAborted();

    /* update and return percentage read */
    void ReadPercent (vltINT32 pc)   {shExp->readPercent = pc;}
    vltINT32      ReadPercent (void) {return shExp->readPercent;}

    /*********************************************************************
      PROCESSING MANIPULATORS
      *********************************************************************/

    /* update processing start/end times */
    ccsCOMPL_STAT ProcStart();
    ccsCOMPL_STAT ProcPercent(vltINT32 pc);
    ccsCOMPL_STAT ProcEnd();

    /*********************************************************************
      TRANSFER MANIPULATORS
      *********************************************************************/
    /* update transfer start/end time */
    ccsCOMPL_STAT TransStart();
    ccsCOMPL_STAT TranPercent(vltINT32 pc);
    ccsCOMPL_STAT TransEnd();

    // Frame counting for AO stuff
    void MaxFrame (vltINT32 m) {shExp->maxFrame = m;};

  private:
    /* c'tor, d'tor */
    fcdEXPOSURE();
    ~fcdEXPOSURE();

    /* Create, attach, detach */
    ccsCOMPL_STAT Create(fcdDETECTOR *det,
			 fcdT_EXPOSURE *exp,
			 dbSYMADDRESS   dbExpRootPt,
			 vltINT32       index);
    ccsCOMPL_STAT Attach(fcdDETECTOR *det,
			 fcdT_EXPOSURE *exp);
    ccsCOMPL_STAT Detach ();

    // Some objects used to map values from memory into
    // the on-line database
    fcdDBDIRECT    mapExpId;
    fcdDBDIRECT    mapExpStatus;
    fcdDBDIRECT    mapExpRepeatCur;
    fcdDBDIRECT    mapTimeEla;
    fcdDBDIRECT    mapTimeRem;

    // Initialise the mappings
    ccsCOMPL_STAT  InitMappings();

    // The shared exposure information
    fcdT_EXPOSURE *shExp;

    // The Detector we are associated with
    fcdDETECTOR   *det;

    // Setup buffer for this exposure
    fcdCSETUP      setup;

    // Images which are stored for this exposure (1 per window)
    // managed by an expData object
    fcdEXPOSURE_DATA expData;

    // Update exp times in OL-DB
    ccsCOMPL_STAT WriteExpTimesToDB();
  
    // Update sequence number, called by Prepare
    ccsCOMPL_STAT ExpSeqNum(vltINT32);
    
    void CurFrame (vltINT32 c) {shExp->curFrame = c;};

};
/*
 *# fcdEXPOSURES
 *# 
 *# Class used to hide the implementation of the shared exposures structure
 *# 
 */
class fcdEXPOSURES {
  public:
    /* c'tor, d'tor */
    fcdEXPOSURES();
    ~fcdEXPOSURES();

    /* Create, attach, detach */
    ccsCOMPL_STAT Create(fcdDETECTOR *det, fcdT_EXPOSURE *exposures);
    ccsCOMPL_STAT Attach(fcdDETECTOR *det, fcdT_EXPOSURE *exposures);
    ccsCOMPL_STAT Detach ();

    /* operator by which individual exposures are accessed */
    fcdEXPOSURE *operator[](vltINT32);
    vltINT32 NumExposures() {return (fcdMAXEXP);}
  private:
    // The exposure objects
    fcdEXPOSURE     exposures[fcdMAXEXP];
};

#endif /* ! FCDEXPOSURE_H */
