
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdEXPOSURE_DATA.h,v 2.140 2005/10/17 15:16:57 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  15/04/96  created
* rdonalds  30/01/98  added TotalImagePixelBytes
* rdonalds  28/10/98  Added methods for preparing frames
*/

#ifndef FCDEXPOSURE_DATA_H
#define FCDEXPOSURE_DATA_H

/*
 * FCD header files
 */
#include "fcdINTERNAL.h"
#include "fcdCSETUP.h"
#include "fcdSHARED_DATA.h"
#include "fcdEXPOSURE_DATA_DESCR.h"
#include "fcdtimsTIME.h"

class fcdEXPOSURE;
class fcdEXPOSURE_DATA;

/*
 *# fcdEXPOSURE_IMAGE
 *#
 *# Class which handles individual images for an exposure
 */
class fcdEXPOSURE_IMAGE
{
public:
  /* c'tor, d'tor */
  fcdEXPOSURE_IMAGE();
  ~fcdEXPOSURE_IMAGE();

  /* Create, attach, detach */
  ccsCOMPL_STAT Create(fcdEXPOSURE_DATA *exp, fcdT_EXPOSURE_IMAGE *data);
  ccsCOMPL_STAT Attach(fcdEXPOSURE_DATA *exp, fcdT_EXPOSURE_IMAGE *data);
  ccsCOMPL_STAT Detach ();


  ccsCOMPL_STAT Prepare(vltLOGICAL initialise);
  ccsCOMPL_STAT PrepareLoop(vltLOGICAL initialise);
  ccsCOMPL_STAT PrepareFrame (vltINT32 frameSeq);

  /* 
     Data Access
     it is VERY IMPORTANT to realise that the data 
     will not be contiguous if there is more than one image. The
     data is layed out in such a way that it can be displayed. In which
     case you should go through the window line by line
     */
  char *Data() {return (data);}
  char *Line(vltINT32 lineNo);

  /*
   *# Return the detector we are part of
   */
  fcdDETECTOR *Det();

  /*
   *# Accessors
   */
  vltINT32 NumPixels() {return (shExpIm->numPixels);}
  vltINT32 PixelBytes() {return (shExpIm->pixelBytes);}
  vltINT32 Lines() {return (shExpIm->imLines);}
  vltINT32 LineBytes() {return (shExpIm->imLineBytes);}
  vltINT32 Cols() {return (shExpIm->imCols);}
  vltINT32 XFillDir() {return (shExpIm->xFillDir);}
  vltINT32 YFillDir() {return (shExpIm->yFillDir);}
  vltINT32 ImYOff() {return (shExpIm->imYOff);}
  vltINT32 ImYs() {return (shExpIm->imYs);}
  vltINT32 ImXs() {return (shExpIm->imXs);}
  vltINT32 WinNum() {return (shExpIm->winNum);}
  vltINT32 PortNum() {return (shExpIm->portNum);}
  vltINT32 PortChipNum() {return (shExpIm->portChipNum);}
  vltINT32 ChipIdx() {return (shExpIm->chipIdx);}

  vltDOUBLE Crpix1() {return (shExpIm->crpix1);}
  vltDOUBLE Crpix2() {return (shExpIm->crpix2);}
  vltINT32 Prscx() {return (shExpIm->prscx);}
  vltINT32 Prscy() {return (shExpIm->prscy);}
  vltINT32 Ovscx() {return (shExpIm->ovscx);}
  vltINT32 Ovscy() {return (shExpIm->ovscy);}
  vltINT32 Impx() {return (shExpIm->impx);}
  vltINT32 Impy() {return (shExpIm->impy);}

    /*
      The current line number read out
      updated during readout by the reorder class
      */
  vltINT32 YCur() {return (shExpIm->yCur);}
    
  /*
   *# Set the yCur, used by REORDER during readout
   */
  void     YCur(vltINT32 y) {shExpIm->yCur = y;}
private:
  // The shared exposure data information
  fcdT_EXPOSURE_IMAGE *shExpIm;


  // The exposure data object we are associated with
  fcdEXPOSURE_DATA     *expData;

  // The start address of window data
  char                 *data;
};
/*
 *# fcdEXPOSURE_WINDOW
 *#
 *# Class which handles individual windows for an exposure
 */
class fcdEXPOSURE_WINDOW
{
public:
  /* c'tor, d'tor */
  fcdEXPOSURE_WINDOW();
  ~fcdEXPOSURE_WINDOW();

  /* Create, attach, detach */
  ccsCOMPL_STAT Create(fcdEXPOSURE_DATA *exp, fcdT_EXPOSURE_WINDOW *data);
  ccsCOMPL_STAT Attach(fcdEXPOSURE_DATA *exp, fcdT_EXPOSURE_WINDOW *data);
  ccsCOMPL_STAT Detach ();

  ccsCOMPL_STAT Prepare(vltLOGICAL initialise);
  ccsCOMPL_STAT PrepareLoop(vltLOGICAL initialise);
  ccsCOMPL_STAT PrepareFrame (vltINT32 frameSeq);

  /* 
     Data Access
     it is VERY IMPORTANT to realise that the data 
     will not be contiguous if there is more than one window. The
     data is layed out in such a way that it can be displayed. In which
     case you should go through the window line by line
     */
  char *Data() {return (data);}
  char *Line(vltINT32 lineNo);

  /*
   *# Accessors
   */
  vltINT32 NumPixels() {return (shExpWindow->numPixels);}
  vltINT32 PixelBytes() {return (shExpWindow->pixelBytes);}
  vltINT32 Lines() {return (shExpWindow->winLines);}
  vltINT32 LineBytes() {return (shExpWindow->winLineBytes);}
  vltINT32 Cols() {return (shExpWindow->winCols);}
private:
  // The shared exposure data information
  fcdT_EXPOSURE_WINDOW *shExpWindow;

  // The exposure data object we are associated with
  fcdEXPOSURE_DATA     *expData;

  // The start address of window data
  char                 *data;
};

/*
 *# fcdT_EXPOSURE_DATA
 *#
 *# Holds a description of the data which images and windows are defined
 *# and the information required to attach to the shared memory segment
 *# where the image is stored
 */
typedef struct
{
    fcdT_EXPOSURE_DATA_DESCR descr;
    vltINT32                 size;
    vltINT32                 shmId;
} fcdT_EXPOSURE_DATA;
/*
 *# fcdEXPOSURE_DATA
 *#
 *# CLass which manages the storage of the data for an exposure
 */
class fcdEXPOSURE_DATA
{
  public:
    /* c'tor, d'tor */
    fcdEXPOSURE_DATA();
    ~fcdEXPOSURE_DATA();

    /* Create, attach, detach */
    ccsCOMPL_STAT Create(fcdEXPOSURE *exp, fcdT_EXPOSURE_DATA *data);
    ccsCOMPL_STAT Attach(fcdEXPOSURE *exp, fcdT_EXPOSURE_DATA *data);
    ccsCOMPL_STAT Detach ();

    /* return the maximum possible amount of data which may be stored */
    vltINT32 MaxPixelBytes() {return (shExpData->size);}

    /*
     *# Method to initialise the Setup and FITS data and 
     *# for each image forming the exposure, and method called
     *# by processes which are attaching to the exposure
     *#
     *# PrepareLoop (descr) is called by fcdexp
     *# Prepare (ccsTRUE)  is called by fcdexp
     */
    ccsCOMPL_STAT Prepare(vltLOGICAL initShValues);
    ccsCOMPL_STAT PrepareLoop (fcdEXPOSURE_DATA_DESCR	*descr);
    ccsCOMPL_STAT PrepareLoop (vltLOGICAL initShValues = ccsFALSE);
    ccsCOMPL_STAT PrepareFrame (vltINT32 frameSeq);

    /*
     *# Accessor functions for window number and window
     */
    vltINT32      NumWindows() {return (shExpData->descr.numWindows);}
    vltINT32      NumImages() {return (shExpData->descr.numImages);}
    fcdEXPOSURE_WINDOW *Window(vltINT32 num);
    fcdEXPOSURE_IMAGE  *Image(vltINT32 num);

    /*
     *# Access the data
     */
    char *Data(){return ((char *)shData.Data());}

    /*
     *# Return the overall dimension of the exposure (all windows)
     */
    vltINT32 TotalLines() {return (shExpData->descr.yDim);}
    vltINT32 TotalCols() {return (shExpData->descr.xDim);}
    vltINT32 TotalXs() {return (shExpData->descr.xs);}
    vltINT32 TotalYs() {return (shExpData->descr.ys);}
    vltINT32 TotalLineBytes() {return (shExpData->descr.totalLineBytes);}
    vltINT32 TotalPixelBytes() 
         {return (TotalLineBytes() * TotalLines());}

    /* total number of pixels in all windows and all images    */
    /* these two numbers are not identical when binned windows */
    /* cross image boundaries                                  */
    vltINT32 TotalWindowPixelBytes();
    vltINT32 TotalImagePixelBytes();

    /* return and set the number of pixels currently read out    */
    /* this is updated by the REORDER class and used by the it   */
    /* process. This is only realy necessary for readouts which  */
    /* do not reorder their data, otherwise you can use the YCur */
    /* fields of the window or image                             */
    vltINT32 ReadImageBytes() {return (shExpData->descr.readImageBytes);}
    void  ReadImageBytes(vltINT32 r) {shExpData->descr.readImageBytes = r;}
 
    /*
     *# Return the detector we are part of
     */
    fcdDETECTOR *Det();
  
    /*
     *# Return the shared memory identifuer
     */
    int         ShmId() {return (shData.ShmId());}
  private:  // The object managing the allocated shared data segment
    fcdSHARED_DATA      shData;
  
    // The shared exposure data information
    fcdT_EXPOSURE_DATA *shExpData;

    // The exposure we are associated with
    fcdEXPOSURE   *shExp;

    // Window and image objects for this exposure data
    fcdEXPOSURE_WINDOW  windows[fcdMAXWINDOW];
    fcdEXPOSURE_IMAGE   images[fcdMAXIMAGES];
};


#endif /* ! FCDEXPOSURE_DATA_H */
