
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdEXPOSURE_DATA_DESCR.h,v 2.140 2005/10/17 15:17:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  15/04/96  created
*/

#ifndef FCDEXPOSURE_DATA_DESCRH
#define FCDEXPOSURE_DATA_DESCRH

/*
 * FCD header files
 */
#include "fcdINTERNAL.h"

class fcdSEQUENCE;
class fcdCSETUP;

/*
 *# fcdT_EXPOSURE_WINDOW
 *#
 *# The descriptor for the shared window information
 */
typedef struct
{
    vltINT32          winNum;      /* window number */

    vltINT32          numPixels;      /* number of pixels in window */
    vltINT32          pixelBytes;  /* number of bytes of pixel in window */
    vltINT32          winLineBytes; /* length of one line (bytes)      */
    vltINT32          winLines;    /* number of lines                 */
    vltINT32          winCols;     /* number of columns */

    vltINT32          winDataOffset; /* offset of the first pixel (butes) */
} fcdT_EXPOSURE_WINDOW;

typedef struct
{
    vltINT32          winNum;      /* window indexed from 0 */
    vltINT32          portNum;     /* port  indexed from 0 over whole mosaic */
    vltINT32          portChipNum; /* port indexed from 0 for chip */
    vltINT32          chipIdx;     /* chip from which it comes */

    vltINT32          numPixels;   /* number of pixels in image */
    vltINT32          pixelBytes;  /* number of bytes of pixel in window */
    vltINT32          imLines;     /* number of lines                 */
    vltINT32          imCols;      /* number of columns */
    vltINT32          imLineBytes; /* length of one line (bytes)      */

    vltINT32          imYOff;       /* offset of the first line readout 
				       in mosaic coordinates            */

    vltINT32          imDataOffset; /* offset of the first pixel in the
				       exposure image (bytes) */
    vltINT32          imXs;         /* offset of the first pixel in the */
    vltINT32          imYs;         /* exposure image in pixels         */
    
    vltINT32          xFillDir;     /* fill direction */
    vltINT32          yFillDir;

    vltINT32          chipXs;       /* start (l-left) coordinates of the   */
    vltINT32          chipYs;       /* image w/reference to this chip      */
    
    vltINT32          yCur;         /* # lines read (updated by REORDER) */

    /* information required for building FITS header */
    vltDOUBLE         crpix1;      /* location of first pixel */
    vltDOUBLE         crpix2;
    vltINT32          impx;        /* image pixels */
    vltINT32          impy;
    vltINT32          prscx;       /* prescan pixels */
    vltINT32          prscy;       
    vltINT32          ovscx;       /* overscan */
    vltINT32          ovscy;       

} fcdT_EXPOSURE_IMAGE;

/*
 *# fcdT_EXPOSURE_DATA
 *#
 *# The descriptor for the shared data segment for this exposure
 */
typedef struct
{
    vltINT32           numWindows; /* num windows in used at the moment */
    vltINT32           numImages; /* num windows in used at the moment */
    
    vltINT32           xs;          /* the start x point of the first win */
    vltINT32           ys;          /* the start y point of the first win */
    vltINT32           xDim;        /* maximum X coordinate of all windows */
    vltINT32           yDim;        /* maximum Y coordinate of all windows */
    vltINT32           totalLineBytes; /* number of bytes in one whole line */

    /* number of image bytes readout so far */
    vltINT32           readImageBytes; 

    fcdT_EXPOSURE_IMAGE images[fcdMAXIMAGES];   /* image data */
    fcdT_EXPOSURE_WINDOW windows[fcdMAXWINDOW]; /* window data */
} fcdT_EXPOSURE_DATA_DESCR;

/*
 *# fcdEXPOSURE_DATA_DESCR
 *#
 *# Class for holding a description of exposure data and creating
 */
class fcdEXPOSURE_DATA_DESCR
{
  public:
    ccsCOMPL_STAT            CreateFromSeq (
					    fcdCSETUP *setup,
					    fcdSEQUENCE *seq, 
					    fcdDETECTOR *det);
    ccsCOMPL_STAT            CreateWindowed (fcdCSETUP *setup, 
					    fcdDETECTOR *det);
    ccsCOMPL_STAT            CreateFullFrame (fcdCSETUP *setup, 
					    fcdDETECTOR *det);
    fcdT_EXPOSURE_DATA_DESCR &ExpDataDescr() {return (expDataDescr);}

    /* return total image pixel bytes */
    vltINT32                  TotalImagePixels();

  private:
    fcdT_EXPOSURE_DATA_DESCR  expDataDescr;
};

#endif /* ! FCDEXPOSURE_DATA_DESCRH */
