#ifndef fcdFIERA_CONFIG_H
#define fcdFIERA_CONFIG_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdFIERA_CONFIG.h,v 2.140 2005/10/17 15:16:58 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  10/05/97  created
* rdonalds  23/07/98  Added FindSeq(vltINT32) method
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "fcdINTERNAL.h"

#include "fcdBOARD_CONFIG.h"
#include "fcdSEQUENCE_CONFIG.h"
#include "fcdVOLTAGES.h"
/*
 *# fcdFIERA_CONFIG
 *#
 *# Holds the fiera specific configuration classes, sequences, boards, voltages
 */
class fcdFIERA_CONFIG
{
friend class fcdSEQUENCE_CREATOR;
public:
  fcdFIERA_CONFIG();
  ~fcdFIERA_CONFIG();

  /*
   *# Free resources
   */
  ccsCOMPL_STAT Uninit();

  /*
   *# Read the configuration from a particular directory
   */
  ccsCOMPL_STAT        GetFromDir (char *dirName);
  
  /*
   *# Download the configuration using a c40 interface object
   */
  ccsCOMPL_STAT        Download (fcdC40 *c40);

  /*
   *# Reload the configuration (again) used for engineering purposes
   *# to reload the sequences during editing
   */
  ccsCOMPL_STAT        Reload (fcdC40 *c40);

  /*
   *# Find a sequence of given name or identifier
   */
  fcdSEQUENCE          *FindSequence (const string &name);
  fcdSEQUENCE          *FindSequence (vltINT32 id);
    
  /*
   *# Find the id of a voltage table of a given name
   */
  vltINT32             FindVoltTable (char *name);

  /*
   *# Set a given voltage
   */
  ccsCOMPL_STAT        SetVoltage(fcdC40          *c40,
				  char            *boardName,
				  char            *periphName,
				  fcdlPeriphValue  value);
  ccsCOMPL_STAT        GetVoltage(fcdC40          *c40,
				  char            *boardName,
				  char            *periphName,
				  fcdlPeriphValue  *value);
				  

  fcdSEQUENCE_CONFIG      &SequenceConfig() {return sequenceConfig;}
private:
  fcdBOARD_CONFIG      boardConfig;
  fcdSEQUENCE_CONFIG   sequenceConfig;
  fcdVOLTAGES          voltageConfig;
};


#endif /*!_H*/
