/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdFITSFuncs.h,v 2.140 2005/10/17 15:16:56 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  12/08/94  created
* rdonalds  1/07/97   ported to FIERA
*/

/*************************************************************************
*       Functions used for manipulating FITS files on the SLCU
*************************************************************************/

#ifndef FCDFITSFUNCS_H
#define FCDFITSFUNCS_H

#ifndef __cplusplus
#error "This is a C++ file"
#endif

#include "fcdINTERNAL.h"
#include "fcdDETECTOR.h"
#include "fcdEXPOSURE.h"

#define fcditFILENAME_EMPTY	"NoName"

typedef enum {                    /* order of basic keywords in FITS header */
    fcdSIMPLE = 0,                /* SIMPLE                                 */
    fcdBITPIX,                    /* BITPIX                                 */
    fcdNAXIS,                     /* NAXIS                                  */
    fcdNAXIS1,                    /* NAXIS1                                 */
    fcdNAXIS2,                    /* NAXIS2                                 */
    fcdORIGIN,                    /* ORIGIN                                 */
    fcdDATE,                      /* DATE                                   */
    fcdCRVAL1,                    /* CRVAL1                                 */
    fcdCRPIX1,                    /* CRPIX1                                 */
    fcdCDELT1,                    /* CDELT1                                 */
    fcdCTYPE1,                    /* CTYPE1                                 */
    fcdCRVAL2,                    /* CRVAL2                                 */
    fcdCRPIX2,                    /* CRPIX2                                 */
    fcdCDELT2,                    /* CDELT2                                 */
    fcdCTYPE2,                    /* CTYPE2                                 */
    fcdBSCALE,                    /* BSCALE                                 */
    fcdBZERO,                     /* BZERO                                  */
    fcdMJDOBS,                    /* MJD-OBS                                */
    fcdEXPTIME,                   /* EXPTIME                                */
    fcdMAXBASIC,                  /* maximum number of basic keywords used  */
} fcdFITSBASICORDER;

typedef struct {                              /* image file information    */
    int               fd;                     /* file descriptor           */
    char              path[fcdMAXPATHLEN];    /* full path                 */
    vltLOGICAL        newFile;                /* file new created          */
    vltINT32          offset;                 /* offset applied to data    */
    vltDOUBLE         timeStart;              /* image transfer start time */
    int               fdHier;                 /* file descriptor for HIER  */
} fcditIMFILE;

typedef struct {                              /* image file information    */
    int               fd;                     /* file descriptor           */
    char              file[fcdMAXPATHLEN];    /* full path                 */
} fcdNFSIMFILE;

extern ccsCOMPL_STAT fcdFITSOpenFile(
    fcdDETECTOR  *det,
    fcditIMFILE  *file,        /* info about file to write into    */
    fcdFITSBASIC *fitsBasic);

extern ccsCOMPL_STAT fcdFITSCloseFile(
    fcditIMFILE   *file,        /* info about file to write into    */
    ccsCOMPL_STAT status);       /* image transfer completion status */


extern ccsCOMPL_STAT fcdFITSImHeadWrite(
    fcditIMFILE  *file,         /* info about file to write into    */
    fcdFITSBASIC *fitsBasic,    /* info about image                 */
    vltINT32      imIdx         /* index of image being written */
    );

extern ccsCOMPL_STAT fcdFITSImDataWrite(
    fcditIMFILE  *file,         /* info about file to write into    */
    fcdFITSBASIC *fitsBasic, /* info about image                 */
    char         *imData,
    vltINT32     numBytes);

extern ccsCOMPL_STAT fcdFITSImDataEnd(
    fcditIMFILE  *file,         /* info about file to write into    */
    fcdFITSBASIC *fitsBasic
    );

ccsCOMPL_STAT fcdReadNFSFitsExposure ( char *fileName,
				       fcdEXPOSURE *exp);
ccsCOMPL_STAT fcdOpenFits (fcdNFSIMFILE *desc);
ccsCOMPL_STAT fcdReadFitsHeader  ( fcdNFSIMFILE *desc,
				   fcdEXPOSURE *exp,
				   fcdFITSBASIC *fitsbasic);
ccsCOMPL_STAT fcdReadFitsData ( fcdNFSIMFILE *desc,
				fcdEXPOSURE *exp,
				fcdFITSBASIC *fitsbasic);
ccsCOMPL_STAT fcdReadFitsTrailer ( fcdNFSIMFILE *desc );
ccsCOMPL_STAT fcdCloseFits ( fcdNFSIMFILE *desc);

#endif /* ! _H */
