#ifndef FCDINTEGRATE_H
#define FCDINTEGRATE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdINTEGRATE.h,v 2.140 2005/10/17 15:17:01 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  25/04/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "vltPort.h"

// Include for the basic class
#include "fcdEXPOSURE.h"
#include "fcdSHUTTER.h"

class fcdINTEGRATE
{
  public:
    fcdINTEGRATE();
    ~fcdINTEGRATE();
  
    ccsCOMPL_STAT       Init(fcdDETECTOR *det, 
			     fcdC40 *c40, 
			     fcdPULPO_CONFIG *gp);

    ccsCOMPL_STAT       Uninit();

  /*
   *# Preparation which has to be done at the start of the loop
   *# of exposures and for each individual exposure
   */
    ccsCOMPL_STAT       PrepareLoop(fcdEXPOSURE *exp);
    ccsCOMPL_STAT       PrepareExposure(fcdEXPOSURE *exp);

  /*
   *# Perform an integration and return the end state of it
   *# endState may be STOP or ABORT if the pollForCommand is set
   *#
   *# exp
   *#    The exposure which is being integrated
   *#
   *# endState
   *#    Indication of EVT_ENDED, EVT_ABORTED, EVT_FAILED etc
   *#
   *# pollForCommand
   *#    Flag controling msgRecv during integration. Looks for
   *#    fcdCMD_STOP, ABORT, CONTINUE, PAUSE and cycles pause/continues
   *#    by opening and closing the shutter
   *#
   *# coordSemId
   *#    If the semid is >=0 it will sem_wait(coordSemId) before
   *#    executing the integration and frame transfer sequences
   */
    ccsCOMPL_STAT       Integrate(
				  fcdEXPOSURE *exp,
				  vltINT32    *endState,
				  vltLOGICAL   pollForCommand,
				  vltINT32     coordSemId
				  );
  private:
    fcdDETECTOR *gDet;
    fcdC40      *gC40;
  
    /*
     *# The shutter control object and an indication of the
     *# type of shutter it is
     */
    fcdSHUTTER           shutter;
    fcdSLCU_SHUTTER_TYPE gShutterType;

    vltLOGICAL          initialised;

    vltLOGICAL  pollMode; /* flag used during Integrate to see if we need to
			     look for commands */

    // Get a command sent during Integration
    ccsCOMPL_STAT	CmdPoll(double s, vltINT32 *cmd, fcdEXPOSURE *exp);
    ccsCOMPL_STAT	MonitorIntegration( vltINT32 *state, fcdEXPOSURE *exp);
    ccsCOMPL_STAT	ControlIntStep( fcdEXPOSURE *exp,
					vltINT32    *state, 
					vltINT32     coordSemId);

    // Change exposure time during exposure
    ccsCOMPL_STAT       SetExpTime(char *buffer, fcdEXPOSURE *exp);

    // Open and close the shutter we need to use different calls for the
    // different reasons, because PULPO needs this information to control
    // its own shutter timing. The reasons are based on the state of
    // the exposure. 
    //        fcdEVT_PAUSED
    //        fcdEVT_CONTINUED
    //        fcdEVT_ABORT
    //        fcdEVT_STOP
    //
    // The INTEGRATE class can be used without shutter control. This could
    // be the case for Dark exposures or Frame Transfer devices
    vltLOGICAL          canUseShutter;
    vltLOGICAL          shutterOpen;
    ccsCOMPL_STAT	OpenShutter(fcdEXPOSURE *exp, vltINT32 reason);
    ccsCOMPL_STAT	CloseShutter(fcdEXPOSURE *exp,
				     vltINT32 lastState,
				     vltINT32 newState);
};

#endif /*!FCDINTEGRATE_H*/
