#ifndef FCDIP_H
#define FCDIP_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdIP.h,v 2.140 2005/10/17 15:17:01 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  22/04/98  created
* rdonalds  23/07/98  Now passes exposure to Prepare() to cope with
*                     double buffered loops
* rdonalds  28/10/98  Removed IPRESULT structure, moved to exposure.h
*                     because it is stored in the exposure itself
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Include files for standard C++ library components
#include <iostream>
#include <fstream>
#include <string>

// Normal fcd includes
#include "fcdINTERNAL.h"
#include "fcdEXPOSURE.h"
#include "fcdDETECTOR.h"
#include "fcdIpDefines.h"

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/*
 * IMAGE PROCESSING DATA STRUCTURES
 * --------------------------------
 * The coordinates are, unless specified, absolute to the lower left corner
 * of the detector. This pixel has the coordinates (1,1).
 */

typedef struct 
    {            
    /*
     * The exposure structure provides general information on the
     * exposure.
     */
    vltDOUBLE    expTime;             /* Effective exposure time             */
    } fcdIPEXPOSURE;

typedef struct 
    {            
    /*
     * The window structure describes the current window passed to the
     * IP function. Besides the information on the window position on
     * the detector and its dimensions, available are also the window
     * index (starting from 0 as first window defined)
     *
     * There is also a pointer to the window data and an lineIncr which
     * indicates by how much the pointer needs to be incremented to get
     * to the next line. This is not just the xDim of the window, because
     * multiple windows are stored in the same data matrix.
     */
    vltINT32     winNum;              /* Window index                        */
    fcdIMAGE_REP pixelRep;            /* Pixel value representation          */
    vltINT32     xPos,yPos;           /* Location of lower left corner       */
    vltINT32     xDim,yDim;           /* Window dimension                    */

    fcdEXPOSURE_WINDOW *expWin;       /* pointer to exposure window */
    vltUINT16    *data;               /* first pixel                */
    vltINT32     lineIncr;             /* increment to next line     */
    } fcdIPWINDOW;

typedef struct 
    {            
    /*
     * The setup structure contains the setup parameters used by the 
     * image processing functions.
     */
    vltINT32     llx,lly;             /* Offset to Lower Left  corner        */
    vltINT32     urx,ury;             /* Offset to Upper Right corner        */
    vltDOUBLE    xRef,yRef;           /* Reference point                     */
    vltDOUBLE    minThr,maxThr;       /* Thresholds                          */
    vltDOUBLE    bckGnd;              /* Estimation of Sky Back Ground       */
    } fcdIPSETUP;

typedef struct 
    {    
    fcdIPEXPOSURE exposure;
    fcdIPWINDOW   window;
    fcdIPSETUP    setup[fcdMAXWINDOW];
    fcdIPRESULT   result[fcdMAXWINDOW];
    } fcdIPIMAGE;

/*
 *# Structure definition for Image Quality functions
 */
typedef struct
{
    vltDOUBLE xMPos,xMPos_SD;                /* mean X position       */
    vltDOUBLE yMPos,yMPos_SD;                /* mean Y position       */
    vltDOUBLE xFWHM,xFWHM_SD;                /* gauss fit X FWHM      */
    vltDOUBLE yFWHM,yFWHM_SD;                /* gauss fit Y FWHM      */
    vltDOUBLE ObjAngle,ObjAngle_SD;          /* orientation [degrees] */
    vltDOUBLE ObjPeak,ObjPeak_SD;            /* peak intensity        */
    vltDOUBLE BGnd,BGnd_SD;                  /* back ground level     */
} fcdipIQEBUF;

/*
 *# fcdIP
 *#
 *# Class for encapsulating the image processing functionality
 */
class fcdIP
{
  public:
    fcdIP();
    ~fcdIP();
  
    ccsCOMPL_STAT Init(fcdDETECTOR *det);
    ccsCOMPL_STAT Uninit();
  
    /*
     *# Preparation for a loop and for the individual exposure
     */
    ccsCOMPL_STAT PrepareLoop (fcdEXPOSURE *exp);
    ccsCOMPL_STAT PrepareExposure (fcdEXPOSURE *exp);
    ccsCOMPL_STAT IP();
  
  private:
    vltLOGICAL  initialised;
    
    /* flag indicating that there is something to be done */
    vltLOGICAL  ipToBeDone;

    /* Counter of IP iterations */
    vltUINT32 completed;
    
    /* exposure being processed */
    fcdEXPOSURE *curExposure;

    /* perform calculation on window */
    ccsCOMPL_STAT IPWindow (vltINT32 winNum);

    /* processing structures for each window */
    fcdIPIMAGE  ipWindows[fcdMAXWINDOW];

    /* Database pointers for writing the results */
    fcdDBDIRECT_LIST dbDirect;

    /* Database pointer to signal IP ready */
    fcdDBDIRECT IPCompleted;
};
/*****************************************************************************
*	INLINE METHODS
*****************************************************************************/

#endif /*!FCDPP_H */
