/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdInternal.h,v 2.140 2005/10/17 15:16:45 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  4/04/97  created
*
*
* Changed length of dateObs in FITSBASIC to allow 4 digit year
* rdonalds  9/10/97  chaged ACT_FMT to miss out the Act. The name is now
*                    fcdit_ccdblah
* rdonalds  2/11/97  Added CMD_GET for get voltage functionality
* rdonalds  18/03/98 Added include of fcdarchDefines
* abalestr  27/08/98 Added ron (readout noise) field to FITS structure
* abalestr  28/10/98 Added include files for compilation with CCSLITE
* abalestr  08/02/99 Added definition for maximun muner of image extensions
*		     actually transferred
* abalestr  13/01/00  Added prototype for fcdPrintSetup function
*/

/*************************************************************************
*  This file contains all definitions for CCD database public attributes
*************************************************************************/

#ifndef FCDINTERNAL_H
#define FCDINTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

/* definitions */
#include "fcd.h"

/* VLT header files */
#include "dxf.h"

/* logging functions */
/*#include "fcdLOG.h"*/

/* database definitions */
#include "fcdDatabase.h"

/* CCD version definitions */
#include "fcdarchDefines.h"

/* Missing function prototypes */
/*extern int strcasecmp (char *, char *);*/

/* system includes (for CCS_LITE) */
#include <signal.h>
#include <unistd.h>

/*************************************************************************
  FILE NAMES
*************************************************************************/
/*
 *# The file containing the names of the cameras for a FIERA
 */
#define fcdFNAME_CAMERA_NAMES "cameras"

#define fcdFNAME_VOLTAGES     "volttable.def"


/*
 * Constants
 */
#define IN 
#define OUT
#define INOUT

#define ILLEGAL 0                     /* illegal value for most type enum  */
#define NO_ERROR 0
#define FD_INVALID -1                 /* invalid file descriptor number    */
#define fcdPID_ILLEGAL 0              /* invalid process id                */

#define fcdCAMERANAME_LEN 8           /* maximum lenght of camera name     */
#define fcdMAXFILELEN 32              /* maximum lenght of image file name */
#define fcdMAXPATHLEN 256             /* maximum lenght of full path name  */
#define fcdLAST_IMAGE_ID -1           /* Id of last image read             */

#define fcdTASK_NAME_ACT_FMT "%s_%s"  /* format for SLCU action task name*/
#define fcditMODULE       "fcdit"     /* name of fcdit module              */
  
/*
 * initialization modes 
 */
typedef enum
    {
    fcdINIT_SW = 1,
    fcdINIT_HW,
    fcdINIT_BOTH
    } fcdINITMODE;
    
/*
 * image transfer completion status
 */
typedef enum
    {
    fcditNONE = 1,                 /* no transfer completed yet */
    fcditSUCCESS,                  /* last transfer completed successfully */
    fcditFAILURE,                  /* last transfer stopped with error */
    fcditABORTED,                  /* last transfer aborted by command */
    } fcditCOMPSTATUS;

/*
 * state of readout task
 */
typedef enum
    {
    fcdrdtOFF         = fcdOFF,
    fcdrdtLOADED      = fcdLOADED,
    fcdrdtSTANDBY     = fcdSTANDBY,
    fcdrdtONLINE_IDLE = fcdONLINE,     /* on-line but nothing to do         */
    fcdrdtONLINE_WIPE,                 /* chip wiping active                */
    fcdrdtONLINE_READY,                /* read-out ready                    */
    fcdrdtONLINE_READ                  /* read-out active                   */
    } fcdrdtSTATE;

/* 
 * commands to SLCU
 */
#define fcdCMD_BREAK      "BREAK"
#define fcdCMD_KILL       "KILL"
#define fcdCMD_SET        "SET"
#define fcdCMD_GET        "GET"

/* command parameters */

/*
 * New types definitions
 */

/*
 * Note: the file name, although not modified by the SLCU sw, better "follows"
 * the exposure path as all other setup parameters. If it stops at WS level, 
 * problems of merging information coming from the same exposure between 
 * SLCU and WS, specially when more exposures are running simultaneously.
 */
typedef struct {                      /* FITS info for each window          */
    vltLOGICAL on;                    /* Window enabled                     */
    vltINT16   strx;                  /* X window start                     */
    vltINT16   stry;                  /* Y window start                     */
    vltINT16   nx;                    /* X window size                      */
    vltINT16   ny;                    /* Y window size                      */
    vltINT8    binx;                  /* X binning factor                   */
    vltINT8    biny;                  /* Y binning factor                   */
} fcdFITSWIND;

typedef struct {                       /* FITS file header basic information */
    /* This part is used to build the FITS header(s) */
    vltINT32    bitPix;                /* bits per pixel                     */
    vltINT32    windowN;               /* number of windows used             */
    fcdFITSWIND winInfo[fcdMAXWINDOW]; /* info for each used window          */
    vltBYTES16  dateObs;               /* date when exposure started         */
    vltDOUBLE   bscale;                /* real = file * BSCALE + BZERO       */
    vltDOUBLE   bzero;                 /* real = file * BSCALE + BZERO       */
    vltDOUBLE   mjd_start;             /* modified julian date of start      */
    vltDOUBLE   expTime;               /* actual exposure time               */

    /* This part is used by the WS part */
    vltINT32    expId;                 /* ID of associated exposure          */

    /* This part is used by the WS part for FITS */
    char        fileUnc[fcdMAXFILELEN];/* name of file for image uncompressed*/
    vltINT32    diskSaveMethod;        /* disk save method                   */

    /* This part is used by the WS part for RTD */
    vltINT32    frameId;               /* real-time display frame ID         */
} fcdFITSBASIC;

typedef struct {                      /* FITS info for each output used     */
    vltINT32   index;                 /* index of output on chip            */
    vltFLOAT   elAdu;                 /* conversion factor e- to ADU        */
    vltFLOAT   ron;                   /* readout noise                      */
    vltFLOAT   gain;                  /* gain used                          */
    vltINT32   xPrescan;              /* prescan pixels */
    vltINT32   xOverscan;             /* overscan pixels */
    vltINT32   xPix;                  /* image pixels */
    vltINT32   yPix;
} fcdFITSOUTP;

typedef struct {                      /* FITS info for each integration     */
    vltDOUBLE  uit;                   /* defined by the user                */
    vltDOUBLE  dit;                   /* elapsed                            */
    vltDOUBLE  dark;                  /* dark integration time              */
} fcdFITSINTE;

typedef struct {                      /* info for each temper. sensor used  */
    vltINT32   index;                 /* index of sensor                    */
    vltFLOAT   start;                 /* temp. value at start of exposure   */
    vltFLOAT   stop;                  /* temp. value at end of exposure     */
    vltFLOAT   mean;                  /* temp. average during exposure      */
    vltFLOAT   drift;                 /* temp. variance during exposure     */
} fcdFITSTEMP;

typedef struct {                      /* info for each telem. source used   */
    vltINT32   index;                 /* index of source                    */
    vltFLOAT   start;                 /* tel. value at start of exposure    */
    vltFLOAT   stop;                  /* tel. value at end of exposure      */
    vltFLOAT   mean;                  /* tel. average during exposure       */
    vltFLOAT   drift;                 /* tel. variance during exposure      */
} fcdFITSTELE;

typedef struct {                       /* FITS hierarchical information      */
    vltINT32    readPixelN;            /* Times same pixel has been read     */
    vltBYTES16  readMode;              /* readout mode used                  */
    vltBYTES16  readSpeed;             /* readout speed used                 */
    vltBYTES16  readClock;             /* clock pattern used for readout     */
    vltINT32    readShift;             /* number of lines shifted / intgr    */
    vltINT32    expType;               /* type of exposure                   */
    vltINT32    outputN;               /* number of outputs used             */
    fcdFITSOUTP outInfo[fcdMAXOUTPUT]; /* info for each used output          */
    vltINT32    outRef;                /* refer. output (image orientation)  */
    vltINT32    frmId;                 /* image sequential number            */
    vltINT32    frmType;               /* type of frame                      */
    vltINT32    frmNaver;              /* number of frames averaged          */
    vltLOGICAL  frmBias;               /* bias substraction done             */
    vltLOGICAL  frmFlatf;              /* flatF division done                */
    vltINT32    integrN;               /* number of integrations             */
    fcdFITSINTE intTime[fcdMAXINTEGR]; /* integration time information       */
    vltDOUBLE   startReadTime;         /* mjd when readout has started       */
    vltDOUBLE   readTime;              /* image readout time [s]             */
    vltDOUBLE   startProcTime;         /* mjd when process has started       */
    vltDOUBLE   procTime;              /* image processing time [s]          */
    vltDOUBLE   startTranTime;         /* mjd when transfer has started      */
    vltDOUBLE   tranTime;              /* image transfer time [s]            */
    vltDOUBLE   stopWipeTime;          /* mjd last wipe done before readout  */
    vltDOUBLE   darkTime;              /* total dark time [s]                */
    vltDOUBLE   shutOpenTime;          /* shutter open  time [s]             */
    vltDOUBLE   shutCloseTime;         /* shutter close time [s]             */
    vltDOUBLE   telPeriod;             /* sampling period [s]                */
    vltINT32    telSrcN;               /* number of tel. sources enabled     */
    fcdFITSTELE telSrc[fcdMAXTELSRC];  /* info on enabled tel. sources       */
} fcdFITSHIERARCH;

/* data structure for sensors (telemetry & temperature) */
typedef struct {                     /* Mapping of DB sensor table         */
    vltLOGICAL      readOnly;        /* Parameter is Read Only             */
    vltLOGICAL      available;       /* Signal is available                */
    vltLOGICAL      enabled;         /*                                    */
    vltLOGICAL      reserved;        /* Reserved for align / future use    */
    vltBYTES16      description;     /* Sensor type description            */
    vltBYTES16      id;              /* Sensor Id                          */
    vltBYTES8       unit;            /* Unit of physical value             */
    vltDOUBLE       low;             /* Lowest tolerated value             */
    vltDOUBLE       high;            /* Highest tolerated value            */
    vltDOUBLE       reference;       /* Reference value                    */
} fcdSENSOR;

/* data structure for sensors parameters (telemetry & temperature) */
typedef struct {
    vltBYTES8       parName;         /* Parameter name                     */
    vltDOUBLE       parValue;        /* Parameter value                    */
} fcdPARAMETER;

/*****************************************************************
  FUNCTION PROTOS
*****************************************************************/
extern ccsCOMPL_STAT fcdReadAttr(
				 char         *fcdName,
				 dbSYMADDRESS  attrName,
				 void         *buffer,
				 ccsERROR     *error
				 );
extern ccsCOMPL_STAT fcdWriteAttr(
				  char *fcdName, dbSYMADDRESS attrName, 
                                  void *buffer, ccsERROR *error );

extern ccsCOMPL_STAT fcdSetIndexAttr(
    IN  dbSYMADDRESS    attrPath,        /* path (rel. or abs.) before index */
    IN  vltINT32        index,           /* index                            */
    IN  dbSYMADDRESS    attrName,        /* attribute name                   */
    OUT dbSYMADDRESS    attrFullName);   /* returned full path               */

/* returns number of pixels taking into account binning factor */
extern vltINT32 fcdNumPixels(
    IN vltINT32 dim,             /* dimension in pixels                      */
    IN vltINT32 binning);        /* binning factor                           */

/* calculate total number of pixels selected windows */
extern ccsCOMPL_STAT fcdGetImageDim (
    IN  fcdCONFIG       *config,   /* configuration of  camera used          */
    IN  fcdSETUP        *setup,    /* setup information                      */
    IN  fcdSETUPRES     *setupRes, /* setup information                      */
    OUT vltINT32        *winDim,   /* window dimension                       */
    OUT ccsERROR        *error);   /* error structure                        */

/* calculate number of pixels/direction selected windows */
extern ccsCOMPL_STAT fcdGetImageXYDim (
    IN  fcdCONFIG       *config,   /* configuration of  camera used          */
    IN  fcdSETUP        *setup,    /* setup information                      */
    IN  fcdSETUPRES     *setupRes, /* setup information                      */
    OUT vltINT32        *winXDim,  /* window X dimension                     */
    OUT vltINT32        *winYDim,  /* window Y dimension                     */
    OUT ccsERROR        *error);   /* error structure                        */

/* print setup for debugging purposes */
void fcdPrintSetup ( 
    IN  fcdSETUP *setup);          /* setup structure                        */

/* FITS header/trailer Pack/Unpack functions */
extern ccsCOMPL_STAT fcdPackFitsHeader ( 
    IN  fcdFITSBASIC *header,      /* FITS header to pack to byte stream     */
    OUT char         *buffer,      /* packed FITS header                     */
    OUT ccsERROR     *error );     /* error structure                        */
extern ccsCOMPL_STAT fcdUnpackFitsHeader ( 
    IN  char         *buffer,      /* byte stream containing the packed hdr  */
    OUT fcdFITSBASIC *header,      /* unpacked FITS header                   */
    OUT ccsERROR     *error );     /* error structure                        */
extern ccsCOMPL_STAT fcdPackFitsTrailer ( 
    IN  fcdFITSHIERARCH *trailer,  /* FITS trailer to pack to byte stream    */
    OUT char            *buffer,   /* packed FITS trailer                    */
    OUT ccsERROR        *error );  /* error structure                        */
extern ccsCOMPL_STAT fcdUnpackFitsTrailer ( 
    IN  char            *buffer,   /* byte stream containing the packed trl  */
    OUT fcdFITSHIERARCH *trailer,  /* unpacked FITS trailer                  */
    OUT ccsERROR        *error );  /* error structure                        */

/* FITS Logging functions */
extern ccsCOMPL_STAT fcdLogFitsComment    ( const char *msg );
extern ccsCOMPL_STAT fcdLogFitsAction     ( const logFITS_KEY  sub,
				     const logFITS_KEY  key, 
				     const logACTION    act );
extern ccsCOMPL_STAT fcdLogFitsParameter  ( const logFITS_KEY  sub, 
				     const logFITS_KEY  key, 
				     const logFITS_KEY  par,
				     const void        *val );
extern ccsCOMPL_STAT fcdLogFitsUnforeseen ( const char *msg );
extern ccsCOMPL_STAT fcdLogFitsRecovery   ( const char *msg );


#ifdef __cplusplus
}
#endif

#endif /* ! FCDINTERNAL_H */
