/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdPULPO.h,v 2.140 2005/10/17 15:17:01 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  16/06/97  created
* abalestr  08/08/01  added handling of led exposures: added exptype parameter
*                     to Start method
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef FCDPULPO_H
#define FCDPULPO_H

/*
 * FCD header files
 */
#include "fcdINTERNAL.h"
#include "fcdtimsTIME.h"

/*
 *# fcdPULPO
 *#
 *# Class for manipulating PULPO interface
 *#
 *# Most of the functions which perform an operation on the shutter
 *# are expected to return an indication of the amount of time taken to
 *# perform the operation e.g. The Open function returns the amount
 *# of time (seconds) which it took the shutter to open
 */
class fcdPULPO
{
  public:
    /* ctor dtor */
    fcdPULPO();
    ~fcdPULPO();

  /*
   *# Return H/W simulation flag
   */
    vltLOGICAL Simulated() { return simulated;};

    /*
     *# Init
     *#
     *# Initialise the PULPO interface with the given unit number
     */
    ccsCOMPL_STAT Init(vltINT32 unit, char *cfgPath, 
		       vltLOGICAL simulate);
    ccsCOMPL_STAT Uninit();
  
    /* Ping the System to see if there */
    ccsCOMPL_STAT Ping(void);

    /*
     *# initialise the shutter function of the controler
     */
    ccsCOMPL_STAT ShutterInit();

    /*
     *# Get a given sensor value
     */
    ccsCOMPL_STAT GetSensor (vltINT32 sId, vltDOUBLE *value);

    /*
     *# Start 
     *#
     *# Open the shutter for a given duration or forever if duration is < 0
     */
    ccsCOMPL_STAT Start (vltDOUBLE duration, fcdEXPTYPE mode, vltDOUBLE *openTime);

    /*
     *# ChangeTime
     *#
     *# Change the PULPO open time
     */
    ccsCOMPL_STAT ChangeTime (vltDOUBLE duration);

    /*
     *# End
     *#
     *# Wait for the end of the exposure
     */
    ccsCOMPL_STAT End (vltDOUBLE *closeTime);

    /*
     *# Pause 
     *#
     *# Pause the shutter timing in PULPO and close shutter
     */
    ccsCOMPL_STAT Pause (vltDOUBLE *pauseTime);

    /*
     *# Continue
     *#
     *# Continue the shutter timing in PULPO and open the shutter
     */
    ccsCOMPL_STAT Continue (vltDOUBLE *contTime);

    /*
     *# Abort
     *#
     *# Abort the current exposure
     */
    ccsCOMPL_STAT Abort (vltDOUBLE *pauseTime);


  private:
    vltLOGICAL simulated;
    vltLOGICAL initialised;
    vltLOGICAL paused;
    vltINT32   unit;
    vltDOUBLE  lastDuration;
};

#endif /*!_H*/
