/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdREAD_EXP.h,v 2.140 2005/10/17 15:17:05 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* abalestr  21/08/03  created
*/

/*************************************************************************
 *  This file contains Internal definitions for C++ modules
*************************************************************************/

#ifndef FCDREAD_EXP_H
#define FCDREAD_EXP_H

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _REENTRANT    /* basic 2-lines for threads */
#include <pthread.h>

#include "fcdREORDER.h"
#include "fcdC40.h"
#include "fcdDETECTOR.h"
#include "fcdSUX.h"

/*
 *# Wrapper around the fcdSUX method for actually perform DMA
 *# It is done like this in order to avoid compiling problems, because
 *# the compiler is looking for a void *(*) (void *) function in the 
 *# pthread_create call. This means that the function must be outside the 
 *# namespace.
 */
class fcdREAD_EXP
{
public:
  fcdREAD_EXP();
  ~fcdREAD_EXP();

  /*
   *# Initialise/unitialise the exposure reading object
   */
  ccsCOMPL_STAT Init(fcdDETECTOR *det, fcdC40 *c40);
  ccsCOMPL_STAT Uninit();

  /*
   *# Preparation which need to be done at the start of the loop
   *# of exposures and for each individual exposure
   */
  ccsCOMPL_STAT PrepareLoop(fcdEXPOSURE *exp);
  ccsCOMPL_STAT PrepareExposure(fcdEXPOSURE *exp);

  /*
   *# Read
   *#
   *# Read using a semaphore as coordination with other processes
   *# which are using the C40. If the value < 0, then no coordination
   *# is performed
   *#
   *# The contMode switch means that the C40 is in a loop and the
   *# exposure is reading continuously (drift mode, AO)
   *#
   */
  ccsCOMPL_STAT Read(vltINT32 coordSemId, vltLOGICAL contMode);

private:
  vltLOGICAL    initialised;
  vltLOGICAL    simC40;

  vltINT32      totalBytes;     // current total size
  vltINT32      oldTotalBytes;  // previous current total size

  fcdEXPOSURE   *curExposure;  // exposure being read

  fcdREORDER    reorder;       // object to reorder the data

  // SUX Functions
  ccsCOMPL_STAT InitSux();    // Initialise device
  ccsCOMPL_STAT UninitSux();  // Uninitialise device (and free resource)

  // Pointer to the C40 interface
  fcdC40        *gC40;

  // SUX object
  fcdSUX        sux;

  // Pointer to the detector object
  fcdDETECTOR	*pDet;

  unsigned short *recvData;
  //char recvData[260000000];

  vltINT32 dspTimeout;

  struct thread_data {
    fcdSUX *sux;
    fcdC40 *gC40;
    fcdEXPOSURE *curExposure;
    fcdDETECTOR *pDet;
    vltINT32 coordSemId;
    vltINT32 dspTimeout;
  };

  static void *WaitForBuffer(void *);
  static void *StartDSP(void *);
};


#endif /* ! _H */
