#ifndef fcdREORDER_H
#define fcdREORDER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdREORDER.h,v 2.140 2005/10/17 15:17:02 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  14/05/97  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "fcdINTERNAL.h"
#include "fcdREORDER_PIXEL_ORDER.h"

class fcdEXPOSURE;
class fcdEXPOSURE_IMAGE;

/*
 *# fcdREORDER_IMAGE
 *#
 #* Data needed to reorder a single image
 */
typedef struct
{
    vltUINT16  *pOrigin;
    vltUINT16  *pCur;     // pointer to current pixel
    vltINT32   pXDim;     // X dimensions for this readout
    
    vltINT32   pXDir;     // X direction +1, -1
    vltINT32   pYDir;     // Y direction +1, -1
    
    vltINT32   pXCur;     // X current position 
    vltINT32   pYCur;     // Y current position

    fcdEXPOSURE_IMAGE *im; // the image we we are ordering
} fcdREORDER_IMAGE;


/*
 *# fcdREORDER
 *#
 *# Class for managing reordering of image
 */
class fcdREORDER
{
public:
  fcdREORDER();
  ~fcdREORDER();
  
  /*
   *# Init 
   *#
   *# Initialise the static part of reorder
   */
  ccsCOMPL_STAT Init ();
  
  /*
   *# Prepare 
   *#
   *# Prepare exposure at start of loop and for readout
   */
  ccsCOMPL_STAT PrepareLoop (
			 fcdEXPOSURE    *exp,
			 vltLOGICAL     forceReorder
			 );
  ccsCOMPL_STAT Prepare (
			 fcdEXPOSURE    *exp
			 );

  /*
   *# Reorder 
   *#
   *# Reorder exposure pixels
   */
  ccsCOMPL_STAT Reorder (
			 char           *newData,
			 vltINT32        numBytes
			 );

private:
    /*
     *# Optimised reordering algorithms
     *# See the header of fcdREORDER.C for a description of when
     *# each is used
     */
    vltLOGICAL unordReorder;
    vltLOGICAL optReorder;
    vltLOGICAL singleBlockReorder;

    /*
     *# Reordering information
     */
    fcdREORDER_DATA      reorderData;
    
    /*
     *# Variables holding the current position in the incoming data
     */
    vltINT32        curBlock; 
    vltINT32        curBlockIter;
    vltINT32        curEntry;
    vltINT32        curEntryOffset;
    
    /*
     *# Data for reordering each image
     */
    vltINT32         imageXDim;
    vltINT32         numImages;
    fcdREORDER_IMAGE reorderImage[fcdMAXIMAGES];

    /*
     *# Exposure being reordered
     */
    fcdEXPOSURE      *curExp;

};
#endif /*!_H*/
