#ifndef fcdREORDER_PIXEL_ORDER_H
#define fcdREORDER_PIXEL_ORDER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdREORDER_PIXEL_ORDER.h,v 2.140 2005/10/17 15:17:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  14/05/97  created
* abaruffo  11/10/00  Raised limit in fcdMAX_REORDER_ENTRY to support up to
*                     16 output ports.
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "fcdINTERNAL.h"

/*
 *#################################################################
 *# Reordering structures
 *#
 *# The data delivered by a sequence is defined as coming from
 *# a particular port for a particular window.
 *#
 *# This reordering information is placed in shared memory
 *# hence it must have a defined Maximum size
 *#################################################################
 */

#define fcdMAX_REORDER_BLOCK 10
#define fcdMAX_REORDER_ENTRY 16

typedef struct 
{
    vltINT32 imNum;     // image the data is for
    vltINT32 numPixels; // number of pixels
} fcdREORDER_ENTRY;

typedef struct
{
    vltINT32 numEntries;             // number of entries in this block
    vltINT32 numIter;                // number of iterations
    
    fcdREORDER_ENTRY  entry[fcdMAX_REORDER_ENTRY];
} fcdREORDER_BLOCK;

typedef struct
{
    vltINT32 numBlocks;             // number of blocks in whole sequence
    
    fcdREORDER_BLOCK block[fcdMAX_REORDER_BLOCK];
} fcdREORDER_DATA;


/*
 *# fcdREORDER_PIXEL_ORDER
 *#
 *# Information required to reorder the pixels
 *# use a string for this information at the moment because
 *# we don't realy know what it will be
 *#
 *# This class is stored in shared memory so that the exposure
 *# coordination class can create it and the readout class can
 *# use it
 */
class fcdREORDER_PIXEL_ORDER {
public:
    fcdREORDER_PIXEL_ORDER();
    ~fcdREORDER_PIXEL_ORDER();

    /*
     *# Set the order from the information retrieved from 
     *# a sequence
     */
    ccsCOMPL_STAT SetOrder (const string &order, vltINT32 xDim, vltINT32 yDim);

    /*
     *# The following methods are used to build the order
     *# while defining a readout sequence
     */
    void Init();
    void BlockStart (vltINT32 repeat);
    void BlockEnd();
    void Entry (vltINT32 image, vltINT32 numPixels);
    void End();
    void SetOutput (vltINT32 outIdx);
	
    /*
     *# Debugging method to print contents
     */
    void PrintOrder();

    /*
     *# Accessors
     */
    vltINT32    NumOutputs() {return (numOutputs);}
    vltINT32    Output(vltINT32 i) {return (outputs[i]);}

    vltINT32 NumBlocks() {return reorderData.numBlocks;}
    
    fcdREORDER_DATA &ReorderData() {return reorderData;}

private:
    ccsCOMPL_STAT ColonSetOrder (const string &order, 
				 vltINT32 xDim, vltINT32 yDim);
    ccsCOMPL_STAT OldSetOrder (const string &order, 
			       vltINT32 xDim, vltINT32 yDim);

    vltINT32         numOutputs;
    vltINT32         outputs[fcdMAXOUTPUT];
    fcdREORDER_DATA  reorderData;

    /* private variables used when constructing a pixel order */
    vltINT32         blockIdx;
    vltINT32         entryIdx;
};
#endif /*!_H*/
