#ifndef FCDSEQUENCE_CONFIG_H
#define FCDSEQUENCE_CONFIG_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdSEQUENCE_CONFIG.h,v 2.140 2005/10/17 15:16:59 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  23/05/96  created
* rdonalds  23/07/98  Added FindSeq(vltINT32)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Include files for standard C++ library components
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

// Normal fcd includes
#include "fcdDLListPtr.h"
#include "fcd.h"
#include "fcdlIntf.h"
#include "fcdBOARD_CONFIG.h"
#include "fcdC40.h"
#include "fcdREORDER_PIXEL_ORDER.h"


/*************************************/
/* Definition for the fcdSEQ class   */
/*************************************/

class fcdUSEQUENCE;
class fcdPATTERN;

/*
 *# fcdSEQUENCE
 *#
 *# 
 */
class fcdSEQUENCE
{
friend class fcdSEQUENCE_PARSER;
friend class fcdSEQUENCE_CONFIG;
friend class fcdSEQUENCE_CREATOR;
public:
    fcdSEQUENCE();
    ~fcdSEQUENCE();

    /*
     *# Accessors
     */
    vltINT32 		Id();		// return binary id of seq
    vltLOGICAL 		Loaded();	// return loaded flag
    void 		Loaded(vltLOGICAL l) {loaded = l;}
    string 		&Name();	// return name of seq
    vltDOUBLE           Time();         // Return the time for seq

    
    /*
     *# Return the fields input by the user if this is
     *# a hand crafted sequence
     */
    vltINT32            TotalPixels();
    vltINT32            TotalTicks() {return totalTicksSoFar;};
    vltINT32            PixelXDim();
    vltINT32            PixelYDim();
    fcdREORDER_PIXEL_ORDER	&PixelOrder();
    
    /*
     *# Download
     */
    ccsCOMPL_STAT	Download(fcdC40 *);	// download to C40

    /*
     *# DumpDef
     *# Dump the definition in ASCII to an output file
     */
    ccsCOMPL_STAT	DumpDef (ostream &stream);

    /*
     *# This method should not realy be public. But it is used from
     *# some static functions in the fcdCREATE_SEQUENCE class
     */
    ccsCOMPL_STAT             AddStatement (fcdlSequenceStatement &stat,
					    fcdSEQUENCE_CONFIG    *seqConfig);

    /*
     *# Accessor functions
     */
    vltINT32                   NumStatements();
    fcdlSequenceStatement     *Statement(vltINT32 index);

    
    /* add a reference to a useq. Used by the SEQUENCE_PARSER and
       SEQUENCE_CREATOR classes. This adds to the list of usequences
       refereced, these will be downloaded when the sequence is
       downloaded */
    ccsCOMPL_STAT             AddUseqRef (fcdUSEQUENCE *useq);

    fcdREORDER_PIXEL_ORDER	pixelOrder;
private:
    static vltINT32     nextId; // used to generate unique ids
    vltLOGICAL		loaded;		// is it on the C40

    // Initialise fields after creation
    ccsCOMPL_STAT InitValues();
    ccsCOMPL_STAT InitValuesWithId(vltINT32 id);

    // a list of all the usquences referenced by this sequence
    fcdDLListPtr usequences;
    // Method to download the useqs associated with this sequence
    ccsCOMPL_STAT DownloadUseqs(fcdC40 *);

    // name as read from the definition
    string		name;		// name of sequence

    // Reordering and dimension information
    // as input by the user in a sequence definition
    vltINT32                    pixelXDim;
    vltINT32                    pixelYDim;
    vltINT32                    totalPixels;

    // Time taken to execute the sequence
    vltDOUBLE           time;
    // the sequence itself (in the form needed for download)
    fcdlDefineSeqParams     *statements;

    // When adding statements we need to keep a track of where the
    // repeat statements begin and end. We do this in repeatIndexes, the
    // current repeat level is indicated by repeatLevel.
    vltINT32                  repeatIndexes[fcdlMAX_SEQ_NEST];
    vltINT32                  repeatLevel;
  
    // Used to record the number of ticks accumulated at the
    // beginning of each repeat loop
    vltINT32                  repeatTicks[fcdlMAX_SEQ_NEST];
    // The total number of ticks in the sequence, can be used with the
    // DSP clock speed to determin the execution time of a sequence
    vltINT32                  totalTicksSoFar;
};

/*
 *# fcdPATTERN
 *#
 *# Patterns are associated with a board and have both Bilevel and Multi level
 *# contents
 */
class fcdPATTERN
{
friend class fcdPATTERN_PARSER;
friend class fcdUSEQUENCE_PARSER;
friend class fcdSEQUENCE_CONFIG;
public:
    fcdPATTERN();
    ~fcdPATTERN();

    vltINT32	Length();
    string	&Name();
    fcdBOARD 	*Board();
    vltINT32	Id();

    // download the pattern to the given address
    ccsCOMPL_STAT Download(fcdC40 *C40, vltINT32 toAddress);

    /*
     *# DumpDef
     *# Dump the definition in ASCII to an output file
     */
    ccsCOMPL_STAT	DumpDef (ostream &stream);

private:
    static vltINT32     nextId; // used to generate unique ids
    vltINT32	id; // internaly allocated id

    // Initialise values after creation
    void InitValues();

    string	name;		// name of pattern
    vltINT32	length;		// length of pattern
    fcdBOARD	*board;		// pointer to the board this pattern goes to

    // The final contents of the pattern
    fcdlDefineBilevelPatternParams *bilevelConts;
    fcdlDefineMultilevelPatternParams *multilevelConts;
};

/*
 *# fcdUSEQUENCE
 *#
 *# Micro sequences perform a single operation on multiple boards
 *#
 *# The name of the micro sequence should encode its behaviour if it
 *# is to be used to build a sequence. 
 *#       e.g shift_line:A-B:chipIdx:binFact:camera
 *#
 */
class fcdUSEQUENCE
{
friend class fcdUSEQUENCE_PARSER;
friend class fcdSEQUENCE_CONFIG;
public:
    fcdUSEQUENCE();
    ~fcdUSEQUENCE();

    vltINT32	Length();
    string	&Name();
    vltINT32    Id();

    vltLOGICAL  Loaded();

    // Download the usequence onto the C40. which implies downloading all of
    // the patterns
    ccsCOMPL_STAT Download(fcdC40 *);
    /*
     *# DumpDef
     *# Dump the definition in ASCII to an output file
     */
    ccsCOMPL_STAT	DumpDef (ostream &stream);
private:
    static vltINT32     nextId;     // used to generate unique ids
    vltINT32		id;	    // internally generated identifier
    vltLOGICAL	        loaded;     // loaded on C40 ?
    vltINT32            address;    // address at which loaded on C40

    // Find a free address to download self into
    ccsCOMPL_STAT       GetLoadableAddress (vltINT32 &address);
    // Download the patterns required for the useq
    ccsCOMPL_STAT       DownloadPatterns (fcdC40 *C40, vltINT32 &address);

    // Initialise self after creation
    void InitValues();

    string		name;		// name of the USEQUENCE
    vltINT32		length;		// length of each pattern
    fcdDLListPtr	patterns;	// list of patterns

    fcdlDefineUseqParams *useqParams;  // parameters required for download
};


/*
 *# fcdSEQUENCE_CONFIG_LIST
 *#
 *# Maintains a list of objects defined in the system
 */
template <class T>
class fcdSEQUENCE_CONFIG_LIST
{
public:
    // Ctor Dtor
    fcdSEQUENCE_CONFIG_LIST() : list() {};
    ~fcdSEQUENCE_CONFIG_LIST() {FreeList();};

    // Accessor
    T		*Find (const string &name)
			{
			    T *p;
			    for (Pix pix = list.first(); pix != 0;
						 list.next(pix))
				{
				p = (T *)(list(pix));
				if (p->Name() == name)
				    return (p);
				}
			    return (0);
			}
    T		*Find (vltINT32 id)
			{
			    T *p;
			    for (Pix pix = list.first(); pix != 0;
						 list.next(pix))
				{
				p = (T *)(list(pix));
				if (p->Id() == id)
				    return (p);
				}
			    return (0);
			}
    T		*First (Pix &pix)
			{
			    pix = list.first();
			    if (pix)
				return (T *)(list(pix));
			    else
				return (0);
			}
    T          *Next (Pix &pix)
                        {
			    list.next(pix);
			    if (pix)
				return (T *)(list(pix));
			    else
				return (0);
			}

    // Free all the board ptrs
    void		FreeList()
			{
			    T *p;
			    while ((p = (T *)list.remove_front()) != 0)
				delete p;
			};


    // The list of boards themselves
    fcdDLListPtr	list;

    // Add new board
    ccsCOMPL_STAT	Add(T *p)
			{
			    list.append(p);
			    return (SUCCESS);
			}
};

/*
 *# fcdSEQUENCE_CONFIG
 *#
 *# Class containing all of the config parameters
 */
class fcdSEQUENCE_CONFIG
{
friend class fcdSEQUENCE_CONFIG_PARSER;
friend class fcdSEQUENCE_CREATOR;
public:
    fcdSEQUENCE_CONFIG();
    ~fcdSEQUENCE_CONFIG();

    /*
     *# InitFromDir
     *
     *# Initialise the sys config from given directory
     */
    ccsCOMPL_STAT GetFromDir (const string &dirName, fcdBOARD_CONFIG *gBoard);

    /*
     *# Uninit
     *#
     *# free any resources allocated
     */
    ccsCOMPL_STAT Uninit();

    /*
     *# Download the complete system configuration
     */
    ccsCOMPL_STAT Download(fcdC40 *, fcdBOARD_CONFIG *);

    /*
     *# Accessors by name or id
     */
    fcdSEQUENCE		*FindSequence(const string &);
    fcdSEQUENCE		*FindSequence(vltINT32 seqId);
    fcdUSEQUENCE	*FindUsequence(const string &);
    fcdUSEQUENCE	*FindUsequence(vltINT32 useqId);
    fcdUSEQUENCE	*FirstUsequence(Pix &pix) 
                            {return usequences.First(pix);}
    fcdUSEQUENCE	*NextUsequence(Pix &pix) 
                            {return usequences.Next(pix);}
    fcdPATTERN		*FindPattern(const string &);

    /*
     *# Number of boards
     */
    vltINT32 NumBoards();

    /*
     *# Maximum Sequence Id generated. This is used when we are generating
     *# sequences on the fly. We generate readout sequences for detectors
     *# based on MaxSeqId() + DetectorIndex. Assuming we only have one S/W
     *# genarated sequence for each detector
     */
    vltINT32 MaxSeqId();
private:
    fcdSEQUENCE_CONFIG_LIST<fcdPATTERN>   patterns;
    fcdSEQUENCE_CONFIG_LIST<fcdUSEQUENCE> usequences;
    fcdSEQUENCE_CONFIG_LIST<fcdSEQUENCE>  sequences;
};

#endif /*!FCDSEQUENCE_H*/
