#ifndef FCDSEQUENCE_PARSERH
#define FCDSEQUENCE_PARSERH
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdSEQUENCE_PARSER.h,v 2.140 2005/10/17 15:16:59 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  23/05/96  created
* abalestr  25/08/99  eliminated CreateCV method for gcc 2.95 (OCT99) 
*                     compatibility (ansi c++ does not allow to call creators 
*                     explicitly)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Include files for standard C++ library components
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

// Normal fcd includes
#include "fcd.h"

#include "fcdTOKENISER.h"	// for Tokenising the ASCII files
#include "fcdSEQUENCE_CONFIG.h"	// Definition of sequence,etc

/*************************************/
/*************************************/

/*
 *# fcdSEQUENCE_PARSER
 *#
 */
class fcdSEQUENCE_PARSER
{
public:
    /*
     *# ParseDef
     *# Get an ASCII representation from file using tokeniser
     */
    ccsCOMPL_STAT	ParseDef (fcdSEQUENCE  *,
				fcdTOKENISER &tok, fcdTOKEN &token);

private:
    // The pattern being parsed
    fcdSEQUENCE	*seq;

    // Check the sequence for some simple design rules
    ccsCOMPL_STAT CheckSequence();

    // Parse the statements which make up a sequence
    ccsCOMPL_STAT ParseSequenceW (string &str);
    ccsCOMPL_STAT ParseSequenceParameter (fcdTOKENISER &tok,
				     fcdTOKEN &token);
    ccsCOMPL_STAT ParseSequenceStatements (fcdTOKENISER &tok, fcdTOKEN &token);
    ccsCOMPL_STAT ParseSequenceStatement (fcdlSequenceStatement &,
                                     fcdTOKENISER &tok, fcdTOKEN &token);
    ccsCOMPL_STAT ParseWaitStatement (fcdlWaitStatement &,
				     fcdTOKENISER &tok, fcdTOKEN &token); 
    ccsCOMPL_STAT ParseUSeqStatement (fcdlUSeqStatement &,
                                     fcdTOKENISER &tok, fcdTOKEN &token); 
    ccsCOMPL_STAT ParseRepeatStatement (fcdlRepeatStatement &,
                                     fcdTOKENISER &tok, fcdTOKEN &token); 
    ccsCOMPL_STAT ParseEndRepeatStatement (fcdlEndRepeatStatement &,
                                     fcdTOKENISER &tok, fcdTOKEN &token); 
};
/*
 *# fcdUSEQUENCE_PARSER
 *#
 */
class fcdUSEQUENCE_PARSER
{
public:
    /*
     *# ParseDef
     *# Get an ASCII representation from file using tokeniser
     */
    ccsCOMPL_STAT	ParseDef (fcdUSEQUENCE *,
				fcdTOKENISER &tok, fcdTOKEN &token);

private:
    // The useq being parsed
    fcdUSEQUENCE	*useq;

    // Verify that a pattern can be added to this useq correctly
    // lengt is correct etc.
    ccsCOMPL_STAT	VerifyPattern (fcdPATTERN *,
				fcdTOKENISER &tok, fcdTOKEN &token);
  
  // Check the completed USEQ
  ccsCOMPL_STAT       CheckUseq();
  // Check to ensure no more than one video board is output enabled at the
  // same time
  ccsCOMPL_STAT       CheckVideoOE();
  ccsCOMPL_STAT       CheckVideoOEVP(
				     vltINT32 numPatterns,
				     fcdPATTERN *patterns[]
				     );

};
/*
 *# fcdMULTILEVEL
 *#
 *# structure to hold the pair number, contents for a Multi level SIMM
 */
typedef class fcdMULTILEVEL{
public:
    vector<unsigned char>	simmContents;	// contents
    class fcdMULTILEVEL &operator=(const class fcdMULTILEVEL &other);
} ;
inline fcdMULTILEVEL &fcdMULTILEVEL::operator=(const class fcdMULTILEVEL &other)
{
    this->simmContents = other.simmContents;
    return (*this);
}

/*
 *# fcdMULTILEVEL_CONTS
 *#
 *# There is an array of multi level contents for each SIMM.
 *# this array is indexed by the SIMMs index on the board
 */
typedef vector <fcdMULTILEVEL> fcdMULTILEVEL_CONTS;

/*
 *# fcdBILEVEL_CONTS
 *# 
 *# The Bilevel contents are simply an array of shorts
 */
typedef vector<short> fcdBILEVEL_CONTS;

/*
 *# fcdPATTERN_PARSER
 *#
 */
class fcdPATTERN_PARSER
{
public:
    /*
     *# ParseDef
     *# Get an ASCII representation from file using tokeniser
     */
    ccsCOMPL_STAT	ParseDef (fcdPATTERN *,
				fcdTOKENISER &tok, fcdTOKEN &token);
private:
    // The pattern being parsed
    fcdPATTERN	*pattern;

    // Map of SIMM to value
    typedef vector <vltINT32> SimmValues;
    ccsCOMPL_STAT	SetBilevelValue (vltINT16 &, fcdSIMM *, short);
    ccsCOMPL_STAT	AddCVToPattern (SimmValues &);

    ccsCOMPL_STAT	ParseEntries (fcdTOKENISER &tok, fcdTOKEN &token);
    ccsCOMPL_STAT	ParseEntry (SimmValues &, 
				fcdTOKENISER &tok, fcdTOKEN  &token);
    ccsCOMPL_STAT	ParseValue (SimmValues &, 
				fcdTOKENISER &tok, fcdTOKEN &token);
    ccsCOMPL_STAT	AssignValue (SimmValues &,
				string &name, vltDOUBLE value);
    
    ccsCOMPL_STAT       CreateDownloadablePattern();
    ccsCOMPL_STAT       CreateMultilevelConts();
    ccsCOMPL_STAT       CreateBilevelConts();

    /* objects used to collect multilevel and bilevel values while */
    /* parsing the definition. Discarded when finished             */
    fcdMULTILEVEL_CONTS multilevels;
    fcdBILEVEL_CONTS    bilevels;

    // Check function
    ccsCOMPL_STAT       CheckPattern();
};

/*
 *# fcdSEQUENCE_CONFIG_LIST_PARSER
 *#
 *# Parses a list of objects defined in the system
 */
template <class T, class TP>
class fcdSEQUENCE_CONFIG_LIST_PARSER
{
public:
    /*
     *# GetFromDir
     *#
     *# Get an ASCII representation from files in a given directory
     */
    ccsCOMPL_STAT	GetFromDir(fcdSEQUENCE_CONFIG_LIST<T> *,
				const string &dirName, const string &suffix);

private:
    fcdSEQUENCE_CONFIG_LIST<T>	*list;

    ccsCOMPL_STAT	GetFromFile(const string &fileName);
};
/*
 *# fcdSEQUENCE_CONFIG_PARSER
 *#
 *# Parser for complete system configuration
 */
class fcdSEQUENCE_CONFIG_PARSER
{
public:
    /*
     *# GetFromFile
     *#
     *# Get an ASCII representation from files in a givedn directory
     */
    ccsCOMPL_STAT	GetFromDir(
			      fcdSEQUENCE_CONFIG *,
			      const string    &dirName,
			      fcdBOARD_CONFIG *gBoardConfig
			      );
private:
    fcdSEQUENCE_CONFIG 	*config;
};

#include "fcdSEQUENCE_CONFIG_PARSER.icc"

#endif /*!FCDSEQUENCE_PARSERH*/
