/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdSHUTTER.h,v 2.140 2005/10/17 15:17:01 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  16/06/97  created
* abalestr  08/08/01  added handling of led exposures: added exptype parameter
*                     to Start method
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#ifndef FCDSHUTTER_H
#define FCDSHUTTER_H

/*
 * FCD header files
 */
#include "fcdINTERNAL.h"
#include "fcdC40.h"
#include "fcdPULPO_CONFIG.h"
#include "fcdtimsTIME.h"

typedef enum 
{
    fcdSLCU_SHUT_DIGITAL,   /* digital shutter through anabias board */
    fcdSLCU_SHUT_PULPO,     /* pulpo controled PULPO1 or PULPO2      */
    fcdSLCU_SHUT_NONE       /* no shutter                            */
} fcdSLCU_SHUTTER_TYPE;

/*
 *# T_SHUTTER_TIME
 *#
 *# Stores time for a shutter operation (open, close)
 */
typedef struct
{
    fcdtimsTIME start;     /* shutter  open  times                   */
    fcdtimsTIME middle;
    fcdtimsTIME end;
    vltDOUBLE   duration;
} fcdT_SHUTTER_TIME;

class fcdDETECTOR;
/*
 *# fcdSHUTTER
 *#
 *# Class for manipulating shutters
 */
class fcdSHUTTER
{
  public:
    /* ctor, dtor */
    fcdSHUTTER();
    ~fcdSHUTTER();

    /*
     *# Init
     *#
     *# Initialise the shutter with the interface objects
     */
    ccsCOMPL_STAT Init(fcdDETECTOR *, fcdC40 *, fcdPULPO_CONFIG *gP,
		       fcdSLCU_SHUTTER_TYPE *shutType);
    ccsCOMPL_STAT Uninit();

    /*
     *# Start 
     *#
     *# Open the shutter for a given duration or forever if duration is < 0
     */
    ccsCOMPL_STAT Start (vltDOUBLE duration, fcdEXPTYPE mode);

    /*
     *# ChangeTime 
     *#
     *# Change the shutter open time
     */
    ccsCOMPL_STAT ChangeTime (vltDOUBLE duration);

    /*
     *# End
     *#
     *# End of an exposure, wait for the shutter to be closed (or close it
     *# for a digital shutter)
     */
    ccsCOMPL_STAT End ();

    /*
     *# Pause 
     *#
     *# Pause the execution of shutter timing (does not make sense if
     *# PULPO or C40 is not controling the exposure time)
     *# Shutter is closed
     */
    ccsCOMPL_STAT Pause ();

    /*
     *# Continue
     *#
     *# Continue execution of exposure timing shutter is opened
     */
    ccsCOMPL_STAT Continue ();

    /*
     *# Abort
     *#
     *# end exposure timing and close shutter
     */
    ccsCOMPL_STAT Abort ();

  private:
    /* shutter configuration from DB */
    fcdCONFSHUTTER  gConf;
    /* indication of shutter availability */
    vltLOGICAL      gAvailable;
    /* the last duration for which opened */
    vltDOUBLE       gDuration;
    /* detector for which it's being controlled */
    fcdDETECTOR    *gDet;
  
    /* interfaces with which to communicate with shutter */
    fcdSLCU_SHUTTER_TYPE shutterType;
    vltINT32             shutterUnit;
    fcdC40          *gC40;
    fcdPULPO        *gPULPO;
    fcdPULPO        *gPULPO2;

    /* pulpo internal functions */
    ccsCOMPL_STAT PulpoStart (vltDOUBLE duration, fcdEXPTYPE mode, 
	                      fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT PulpoEnd (fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT PulpoPause (fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT PulpoContinue (fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT PulpoAbort (fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT PulpoChangeTime (vltDOUBLE duration);

    /* C40 Digital Shutter  internal functions */
    ccsCOMPL_STAT DigOpen (fcdT_SHUTTER_TIME *t);
    ccsCOMPL_STAT DigClose (fcdT_SHUTTER_TIME *t);


    /* initialised flag */
    vltLOGICAL initialised;
};

#endif /*!_H*/
