#ifndef fcdSLCUINTERNAL_H
#define fcdSLCUINTERNAL_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdSlcuInternal.h,v 2.140 2005/10/17 15:17:00 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  03/06/97  created
* rdonalds  02/11/97  Added the GETV command
* ccumani   26/07/05  fcdLONG_TIMEOUT and fcdLONG_LONG_TIMEOUT added
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is C++
#endif

#include "vltPort.h"
#include "fcdInternal.h"

#define fcdSLCU_FNAME_PULPO_CFG "pulpo.cfg"

/*
 *# Command sent from con process to exp, to indicate that the
 *# con process is loaded
 */
#define fcdSLCU_CMD_LOADED    "LOADED"

/*
 *# Set the test video mode
 */
#define fcdSLCU_CMD_TVID      "TVID"

/*
 *# Set and Get voltage values
 */
#define fcdSLCU_CMD_SETVOLTAGE      "SETV"
#define fcdSLCU_CMD_GETVOLTAGE      "GETV"

/*
 *# Set exposure time of running exposure
 */
#define fcdSLCU_CMD_SETEXPTIME     "EXPT"

/*
 *# Commands to initiate actions of exposure. These commands have the
 *# detector id appended so we end up with commands that look like TRAN:1
 *# so the maximum allowed is ccsCMD_LEN-2 == 5
 */
#define fcdSLCU_CMD_INTEGRATE      "INT"
#define fcdSLCU_CMD_WIPE           "WIPE"
#define fcdSLCU_CMD_READOUT        "READ"
#define fcdSLCU_CMD_TRANSFER       "TRAN"
#define fcdSLCU_CMD_PREP_TRANSFER  "TPEP"
#define fcdSLCU_CMD_DEBUG          "DBUG"

/*
 *# Environment variable defining the CCD name
 */
#define fcdENV_CCDNAME     "CCDNAME"


/* exposure events */
#define fcdEVT_EXPIRED     1
#define fcdEVT_ENDED       2
#define fcdEVT_ABORTED     4
#define fcdEVT_PAUSED      8
#define fcdEVT_CONTINUED  16
#define fcdEVT_ERROR      32
#define fcdEVT_STOP      256

/*
 *# Status of a readout. Stored in the db as 
 *# "<alias>ccdname:readout.status"
 *# used only for status display
 */
typedef enum
    {
    fcdrdtNONE = 1,                 /* no readout completed yet */
    fcdrdtSUCCESS,                  /* last readout completed successfully */
    fcdrdtFAILURE,                  /* last readout stopped with error */
    fcdrdtABORTED                   /* last readout aborted by command */
    } fcdrdtSTATUS;

/*
 *# Exposure coordination task
 */
#define fcdEXPCOORD_TASK     "fcdexp"
#define fcdREADOUT_TASK      "fcdrdt"
#define fcdINTEGRATION_TASK  "fcdint"

/* 
 *# Timeout in seconds to allow the system to initialise
 *# used as a timeout in the RPC 
 */
#define fcdINIT_TIMEOUT  30.0

/*
 *# Normal command timeouts
 */
#define fcdSHORT_TIMEOUT      10.0
#define fcdLONG_TIMEOUT       90.0
#define fcdLONG_LONG_TIMEOUT 150.0

/*
 *# Board and Peripheral name lookup functions
 */
extern int fcdc40BoardNameToId (const char	*BoardName);
extern int fcdc40PeriphNameToId (const char	*PeriphName);

/*
 *# fcdSlcuGetConfigDir
 *#
 *# Establish the config directory for a given camera name
 *# and return the list of cameras which are controlled by the
 *# same system
 */
class fcdCAMERA_NAMES;
extern ccsCOMPL_STAT fcdSlcuGetCameraConfigDir (
					  fcdCAMERANAME cameraName,
					  fcdFILENAME   configDir,
					  fcdCAMERA_NAMES *detNames
					  );

/*
 *# fcdSlcuGetDefaultImageDir
 *#
 *# Establish the default image directory for given name
 *# this can be overriden in the database
 */
extern ccsCOMPL_STAT fcdSlcuGetDefaultImageDir (
					  fcdCAMERANAME cameraName,
					  fcdFILENAME   imageDir
					  );

/*
 *# fcdmSleep 
 *#
 *# Sleep for a given number of milliseconds
 */
extern vltINT32 fcdmSleep (
			   vltINT32 milliseconds
			   );

/*
 *# fcduSleep 
 *#
 *# Sleep for a given number of microseconds
 */
extern vltINT32 fcduSleep (
			   vltINT32 useconds
			   );

/*
 *# fcdReadAttr 
 *#
 *# 
 */
extern ccsCOMPL_STAT fcdSlcuReadAttr (
                           char         *ccdName,
                           dbSYMADDRESS  attrName,
                           void         *buffer
                          );

/*
 *#  fcdSlcuCcdLenvEqRtapEnv
 *#
 *# 
 */
vltLOGICAL fcdSlcuCcdLenvEqRtapEnv(void);

/*
 *#  fcdIsSlcuRunningOnIws
 *#
 *# 
 */
vltLOGICAL fcdIsSlcuRunningOnIws(void);


#ifdef __cplusplus
#endif

#endif /*!_H*/
