/************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdSockFns.h,v 2.14 2004/01/14 15:12:46 vltsccm Exp $"
************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif
/* @(#) sock_fns.h Release 1.8 10/19/94 */
/***************************** module header ***********************************
*   ESTEC/WGE                              *
*                                          *
*   ****    *****   *****   *****  *****   *   Project: EGSE Kernel
*   *   *   *       *       *      *       *
*   *   *   *       *       *      *       *
*   *   *   *****   * ***   *****  *****   *   Filename: sock_fns.h
*   *   *   *       *   *       *  *       *
*   *   *   *       *   *       *  *       *
 *   ****    *****   *****   *****  *****  *
*                                          *
*   Distributed EGSE                       *
********************************************************************************
** Description:**

	C header file for utility socket functions, these functions provide
	a simpler way to perform the usual socket, connect, listen etc

********************************************************************************
** Update record:**  <date> <name> <description...> 

Fri Mar  4 09:27:47 MET 1994
	R. Donaldson
	Original

******************************** end of module header *************************/

#ifndef SOCK_FNS_h
#define SOCK_FNS_h

#ifndef _INCLUDE_HPUX_SOURCE
#include <sys/select.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>

/******************************************************************************
			CONNECTION ESTABLISHMENT ROUTINES
******************************************************************************/

/*
 * SockClose
 *
 * Close a socket which has been accepted or created
 */
extern int SockClose (int SockFd);

/*
 * SockSrvCreateUnix
 *
 * Create a unix domain socket, bind to address and listen and return the socket
 * id
 *
 * Can be used in conjunction with SockCliConnectUnix
 */
extern int SockSrvCreateUnix (char *UnixPath);

/*
 * SockSrvCreateUnixInTmp
 *
 * This is an safer version of the one above. It creates a UNIX domain socket
 * in the  SOCK_TMP_SUB_DIR sub-directory, attempting to ensure that when
 * a different user later attempt to unlink the socket the attempt will
 * be successfull. This is done by ensuring that the SOCK_TMP_SUB_DIR
 * has mode 0777.
 *
 * This function is provided because often the "/tmp/" directory has the
 * sticky bit set which disallows users other than the creator from
 * removing files.
 *
 * The full path name of the UNIX domain socket thus produced is
 * SOCK_TMP_SUB_DIR/UnixPath.
 *
 *
 * This routine should be used in conjunction with SockCliConnectUnixInTmp
 */
#define SOCK_TMP_SUB_DIR	"/tmp/unix_sockets"
extern int SockSrvCreateUnixInTmp (char const *UnixPath);

/*
 * SockSrvAcceptUnix
 *
 * Accept a connection on socket created with SockSrvCreateUnix
 */
extern int SockSrvAcceptUnix (int SockFd);

/*
 * SockCliConnectUnix
 *
 * Client connection routine, create a socket, and connect to the
 * specified Unix port returning a new socket id
 *
 * The routine attempts the connection NumTries times sleeping
 * one second between each in the case of connection failure
 */
extern int SockCliConnectUnix (char *UnixPath, int NumTries);

/*
 * SockCliConnectUnixInTmp
 *
 * Wrapper for SockCliConnectUnix which works out the full path name
 * of the UNIX domain socket, based on the SOCK_TMP_SUB_DIR. Should
 * be used in conjunction with SockSrvCreateUnixInTmp()
 */
extern int SockCliConnectUnixInTmp (char const *UnixPath, int NumTries);

/*
 * SockAddrCreateIn
 *
 * Create an internet address
 */
extern int SockAddrCreateInet (char *Host, int Port, struct sockaddr_in *addr);

/*
 * SockSrvCreateInet
 *
 * Create a internet domain socket, bind to address and listen returning
 * the new socket id
 */
extern int SockSrvCreateInet (int Port);

/*
 * SockSrvAcceptInet
 *
 * Accept a connection on socket created with SockSrvCreateInet and return the
 * new socket id
 */
extern int SockSrvAcceptInet (int SockFd);

/*
 * SockCliConnectInet
 *
 * Client connection routine, create a socket, and connect to the
 * specified host/port and return the new socket id
 *
 * The routine attempts the connection NumTries times sleeping
 * one second between each in the case of connection failure
 */
extern int SockCliConnectInet (char *Host, int Port, int NumTries);

/*
 * SockSrvCreateUDP
 *
 * Create a socket bound to a port which can be used to accept UDP datagrams
 * using SockReadDG and send datagrams using SockSendDG
 */
extern int SockSrvCreateUDP (int Port);

/*
 * SockCliCreateUDP
 *
 * Create a UDP Socket 
 */
extern int SockCliCreateUDP(void);


/******************************************************************************
			DATA TRANSFER ROUTINES
******************************************************************************/

/*
 * SockWaitForData
 *
 * Wait for for data to arrive on given socket in time
 *
 * returns 0 if not ready 1 if ready
 */
extern int SockWaitForData (int SockFd, long Seconds, long Microseconds);

/*
 * SockSendData
 * SockReadData
 *
 * Uninterruptable send function shich splits the data block into chunks
 * that the socket can manage to be read by SockReadData
 */
extern int SockSendData(int SockFd,void *BufPointer,int ByteCount);
extern int SockReadData(int SockFd, void *BufPointer, int ByteCount) ;


/*
 * SockSendDataAndHeader
 *
 * Uninterruptable send function shich splits the data block into chunks
 * Send the header before the data using a writev call
 */
extern int SockSendDataAndHeader(int SockFd,
				 void *HdrPointer,
				 long HdrByteCount,
				 void *BufPointer,
				 long ByteCount
				 );

/*
 * SockSendProtoBuff
 * SockReadProtoBuff
 *
 * Send a buffer of data preceded by its length, allowing the corresponding
 * SockReadProtoBuff to read the data at the other side
 */
extern int SockSendProtoBuff (int SockFd, void *ProtoBuff, int ProtoLen);
extern int SockReadProtoBuff (int SockFd, void *ProtoBuff, int MaxProtoSize);

/*
 * Routines for sending and receiving datagrams
 */
extern int SockReadUDPDG (
			int		SockFD,
			int		MaxAddrSize,
			struct sockaddr	*FromAddr,
			int		*FromAddrSize,
			int		MaxDataSize,
			void		*Data
			);

extern int SockSendUDPDG (
			int		SockFD,
			int		ToAddrSize,
			struct sockaddr	*ToAddr,
			int		DataSize,
			void		*Data
			);



/******************************************************************************
			SOCKET SET ROUTINES
******************************************************************************/

typedef struct
{
    fd_set	sock_set;
    int		max_fd;
} SockSet;


/*
 * SockSetClr
 *
 * Initialise a socket set to be empty
 */
extern void SockSetClr (SockSet *SockSet);

/*
 * SockSetSetFd
 *
 * Set a file descriptor in a socket set
 */
extern void SockSetSetFd (SockSet *SockSet, int fd);

/*
 * SockSetFdIsSet
 *
 * Determin if a file descriptor in a socket is set
 */
extern int SockSetFdIsSet (SockSet *SockSet, int fd);

/*
 * SockSetFdClr
 *
 * Clear a given fd from the set
 */
extern void SockSetFdClr (SockSet *SockSet, int fd);

/*
 * SockSetFirstFdSetAfter
 *
 * Return the first fd which is set after fd or <0 if none are set
 */
extern int SockSetFirstFdSetAfter (SockSet *SockSet, int fd);


/*
 * SockWaitForData
 *
 * Wait for the for data to arrive on given socket within seconds and microsecs
 *
 * returns 0 if not ready 1 if ready and updates the RetSet with the ready
 * file descriptors
 */
extern int SockSetWaitForData (
			SockSet		*TestSet,
		     	SockSet		*RetSet,
			long		Seconds,
			long		Microseconds
			);

/*
 * SockSetNumSet
 *
 * Return the number of Fds set
 *
 */
extern int SockSetNumSet (
			SockSet		*TestSet
			);


#endif


#ifdef __cplusplus
}
#endif
