#ifndef FCDTOKENISER_H
#define FCDTOKENISER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdTOKENISER.h,v 2.140 2005/10/17 15:17:00 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  23/05/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


// Include files for standard C++ library components
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

// Normal fcd includes
#include "ccs.h"

/*
 *# Entry in the token lookup table
 */
typedef struct {
    char	*name;
    vltINT32	token;
} fcdTOKEN_ENTRY;

/*
 *# Standard tokens
 */
enum {
	fcdTOKEN_NONE = 1,
	fcdTOKEN_NAME,
	fcdTOKEN_NUMBER,
	fcdTOKEN_SEMICOLON,
	fcdTOKEN_COMMA,
	fcdTOKEN_EQUALS,
	fcdTOKEN_EOF,
	fcdTOKEN_FIRST_USER,	// this token should be used as the value
				// for the first user token
};

/* 
 *# fcdTOKEN
 *# 
 *# Object returned from tokenising functions
 */
typedef struct {
    vltINT32	token;
    string	name;
    double	number;
} fcdTOKEN;

/*
 *# fcdTOKENISER
 *#
 *# Class for performing basic tokenising functions
 */
class fcdTOKENISER {
public:
    fcdTOKENISER();
    ~fcdTOKENISER();

    ccsCOMPL_STAT	Open (const string &fileName);
    ccsCOMPL_STAT	Close ();

    /*
     *# Get the next token from the input stream using the lookup
     *# table supplied. The table should be terminated by a token with
     *# a zero length "name" field.
     */
    ccsCOMPL_STAT	NextToken(
				fcdTOKEN_ENTRY	*tokenTable,
				fcdTOKEN	&token
				);
    /*
     *# Put back the token such that it will be the next token returned
     *# with a call to NextToken
     */
    ccsCOMPL_STAT	PutbackToken(
				fcdTOKEN	&token
				);

    /*
     *# Return the current line number in the file being parsed
     */
    vltINT32		LineNum();

private:
    ifstream	stream;
    vltINT32	lineNum;
    vltLOGICAL	open;

    vltLOGICAL	putback;	// flag indicating we have a putback token
    				// to return
    fcdTOKEN	putbackToken;	// the token which was put back

    ccsCOMPL_STAT	EatWhite();
    ccsCOMPL_STAT	EatComment();
    ccsCOMPL_STAT	GetName(string &);
    ccsCOMPL_STAT	GetQuotedName(string &);
    ccsCOMPL_STAT	GetNumber(vltDOUBLE &, string&);

    ccsCOMPL_STAT	LookupToken (
				fcdTOKEN_ENTRY	*tokenTable,
				fcdTOKEN	&token,
                                string          &name);
	
};

/*****************************************************************************
*	INLINE METHODS
*****************************************************************************/

#endif /*!FCDTOKENISER_H */
