#ifndef FCDPERIPH_VOLTS_H
#define FCDPERIPH_VOLTS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdVOLTAGES.h,v 2.140 2005/10/17 15:17:00 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  26/04/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "vltPort.h"

/* 
 * System Headers
 */

/* 
 * Local Headers
 */

// Include files for ECCS and Evh Toolkit
#include "fcd.h"
#include "fcdC40.h"
#include "fcdlIntf.h"

/*
 * Voltage Table Identifiers
 */
#define fcdDEFAULT_VOLT_TABLE      0

/*
 * fcdVOLT_DEF
 *
 * Structure holding all the information for one voltage definition
 */
typedef struct
{
    fcdlBoardId	        boardId;
    fcdlPeriphId        periphId;
    fcdlPeriphValue     tolerance;
    fcdlPeriphValue	highValue;
    fcdlPeriphValue	lowValue;
    fcdlPeriphValue     initValue;
} fcdVOLT_DEF;

/*
 * fcdVOLT_DEF_TABLE
 *
 * Class for handling the table of voltage definitions
 */
class fcdVOLT_DEF_TABLE
{
    friend class fcdVOLTAGES;
    friend  class fcdVOLT_TABLE;
 public:
    fcdVOLT_DEF_TABLE();
    ~fcdVOLT_DEF_TABLE();

    ccsCOMPL_STAT       Init();
    ccsCOMPL_STAT       Uninit();

    fcdVOLT_DEF 	*Find (fcdlBoardId, fcdlPeriphId);

    ccsCOMPL_STAT       InitFromFile(const char *fname);

  private:
    fcdVOLT_DEF    	*voltDefs;
    vltINT32		numVolts;

    // Extract the values from a text line
    ccsCOMPL_STAT	ParseLine (char *line,
				fcdlBoardId		*BoardId,
				fcdlPeriphId		*PeriphId,
				fcdlPeriphValue	*LowValue,
				fcdlPeriphValue	*HighValue,
				fcdlPeriphValue	*Tolerance,
				fcdlPeriphValue	*InitValue);

    // Add a peripheral fails if already in list
    ccsCOMPL_STAT	AddPeriph (
				fcdlBoardId		BoardId,
				fcdlPeriphId		PeriphId,
				fcdlPeriphValue	LowValue,
				fcdlPeriphValue	HighValue,
				fcdlPeriphValue	Tolerance,
				fcdlPeriphValue	InitValue
				);
    
};
/*
 * fcdVOLT_TABLE
 *
 * Class for handling one of the voltage tables
 */
class fcdVOLT_TABLE
{
    friend class fcdVOLTAGES;
  public:
    fcdVOLT_TABLE();
    ~fcdVOLT_TABLE();

    ccsCOMPL_STAT       Init();
    ccsCOMPL_STAT       Uninit();
    
    /*
     *# return the name of the table
     */
    char *Name() {return tableName;}
    vltINT32 Id() {return voltDefs->voltTableId;}

    /*
     * Update an entry in one of the tables and redownload
     */
    ccsCOMPL_STAT UpdateTableEntry (fcdC40 *,fcdlBoardId, fcdlPeriphId,
				    fcdlPeriphValue tol, fcdlPeriphValue value);

    /*
     * The voltage tables come from two sources, either the default table which gets
     * its values from the VOLT_DEF_TABLE or from separate files which are used
     * for the extra voltage tables used during sequence execution
     *
     * When the table is read from a file. The table name is taken from
     * the filename
     */
    ccsCOMPL_STAT InitFromFile (vltINT32 tableId, const char *fileName,
				fcdVOLT_DEF_TABLE *defTable);
    ccsCOMPL_STAT InitFromDefTable (vltINT32 tableId, fcdVOLT_DEF_TABLE *t);

    /*
     * Download the given table
     */
    ccsCOMPL_STAT Download (fcdC40 *gC40);

  private:
    ccsCOMPL_STAT	ParseLine (char *line,
				fcdlBoardId *, fcdlPeriphId *,
                                fcdlPeriphValue *tol, fcdlPeriphValue *init);

    ccsCOMPL_STAT AddTableEntry (fcdlBoardId, fcdlPeriphId,
				 fcdlPeriphValue tol, fcdlPeriphValue value,
				   fcdVOLT_DEF_TABLE *defTable);

    fcdlVoltTableEntry        *Find (fcdlBoardId, fcdlPeriphId);

    char                      tableName[fcdVOLTTABLENAME_LEN+1];
    fcdlSetVoltTableParams    *voltDefs;
};
/*
 *# Class which holds all voltage tables
 */
class fcdVOLTAGES
{
public:
    fcdVOLTAGES();
    ~fcdVOLTAGES();

    ccsCOMPL_STAT       Init();
    ccsCOMPL_STAT       Uninit();

    // Read the configurations from files in given directory
    ccsCOMPL_STAT	GetFromDir (char *dirName);

    // Download the complete configuration to the c40
    ccsCOMPL_STAT	Download (fcdC40 *);

    // Reload the non default voltages used during engineering 
    // with the Reload button
    ccsCOMPL_STAT	Reload (fcdC40 *);

    // Set a voltage and download it (may fail if out of range)
    ccsCOMPL_STAT	SetVoltage(
				fcdC40                  *gC40,
				fcdlBoardId		BoardId,
				fcdlPeriphId		PeriphId,
				fcdlPeriphValue	Value
				);

    // Same as above but use board names and periph names
    ccsCOMPL_STAT	SetVoltage(
				fcdC40                  *gC40,
				char			*BoardName,
				char			*PeriphName,
				fcdlPeriphValue	Value
				);

    // Communicate with the C40 to read a current peripheral voltage
    ccsCOMPL_STAT	GetVoltage(
				fcdC40                  *gC40,
				fcdlBoardId		BoardId,
				fcdlPeriphId		PeriphId,
				fcdlPeriphValue	*Value
				);

    ccsCOMPL_STAT       GetVoltage (
				fcdC40                  *gC40,
				char			*BoardName,
				char			*PeriphName,
				fcdlPeriphValue	*Value
                                );

    /*
     *# Return the table id. or -1 if no table exists with that name
     *# as a special case the default table is returned if the name
     *# "" is used
     */
    vltINT32            FindTable(char *tableName);

  private:
    // Table defining the voltages in the system
    fcdVOLT_DEF_TABLE  	voltDefs;

    // The default voltage table is index 0
    // Additional voltage tables to be used during sequence execution
    vltINT32            numVoltTables;
    fcdVOLT_TABLE       voltTables[fcdlMAX_VOLTAGE_TABLE_ID];
};

#endif /*!FCDPERIPHVOLTS*/
