#ifndef FCDWS_PROCESS_H
#define FCDWS_PROCESS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdWS_PROCESS.h,v 2.140 2005/10/17 15:16:47 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* abalestr  08/06/98  created (copied from SLCU part)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _POSIX_SOURCE 1

#include "vltPort.h"
#include "fcdCSTATE.h"

/* class which holds process information */
class fcdWS_PROCESS
{
public:
  fcdWS_PROCESS();
  ~fcdWS_PROCESS();

  /*
   *# Register 
   *#
   *# Register the process with CCS
   *#
   *# For processes which are not camera specific we use just the process
   *# name. For those which are camera specific we build a name lie
   *# <process_name>_<camera_name>. This allows more than one instance
   *# in a given environment
   *#
   *# dbStateAddress is the database point where the processes
   *# state should be reflected
   */
  ccsCOMPL_STAT Register (int argc, char **argv);

  ccsCOMPL_STAT State(fcdSTATE newState);	// Set state
  fcdSTATE	State(void);			// Get state
  
  /*
   *# Exit performed before exiting the process. This will be done 
   *# automomaticaly if not called directly
   */
  ccsCOMPL_STAT Uninit();

  /*
   *# Accessor functions
   */
  ccsPROCNAME     &Name() {return (procName);}
  ccsPROCNAME     &BaseName() {return (baseProcName);}
  ccsENVNAME      &Env()  {return (envName);}
  fcdCAMERANAME   &CameraName()  {return (cameraName);}
private:
  vltLOGICAL      registered; 
  
  fcdCSTATE       cState;          /* object to handle state */

  ccsENVNAME      envName;         /* environment name */

  ccsPROCNUM      procId;          /* identifier of the process calling */
  ccsPROCNAME     procName;        /* the name of the process       */
  ccsPROCNAME     baseProcName;    /* the base name of the process 
				      (without camera)      */

  fcdCAMERANAME   cameraName;      /* camera for which invoked */
  /* extract the name from the arg list */
  ccsCOMPL_STAT   GetName (int argc, char **argv);
};


/*****************************************************************
    INLINE METHODS
*****************************************************************/
inline fcdSTATE fcdWS_PROCESS::State ()
{
    return (cState.State());
}
inline ccsCOMPL_STAT fcdWS_PROCESS::State (fcdSTATE state)
{
    return (cState.State(state));
}

#endif /*!_H*/
