/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdWsInternal.h,v 2.140 2005/10/17 15:16:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  12/08/94  created
* alongino  24/07/95  commands moved to fcd.h
*                     removed IMREADY. Obsolete
* alongino  14/08/95  added definition of fcdPARAM
* alongino  17/08/95  changed structure of fcdENVVAR to include fcdCAMERA
* alongino  29/01/96  fcdPARAM replaced with cmdPARAM_LIST
* alongino  15/11/96  added fcdMidasCmd and fcdObjGetMidasProc prototypes
* pduhoux   31/01/97  added param <revision> to fcdLog functions
* pduhoux   31/01/97  fixed DB/dataType mismatch in fcdFITSHIERARCH_WS
*
* rdonalds  19/03/98  Added logMask to envVar
* abalestr  13/01/00  Added prototype for fcdGetOutputDim for compatibility 
*                     with ancient layers of sw... 
* abortolu  21/07/05  Modified fcdFITSTELE_WS structure to support changes
*                     in telemetry database
*/

/*************************************************************************
*  This file contains the definitions for the WS part of the FIERA sw.
*  Common file Used by all WS FIERA modules
*************************************************************************/

#ifndef CCDWSINTERNAL_H
#define CCDWSINTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * System header files
 */
#include <stdlib.h>

/*
 * VLT include files
 */
#include "slx.h"

/*
 * CCD include files
 */
#include "fcdInternal.h"               /* common definitions for WS and SLCU */

 /*
 * Constants
 */
#define fcdTIMEOUT_DEFAULT 25000      /* default timeout for replies 25 sec  */
                                      /* AL941214 10 sec not enough for wipe */
#define fcdSETUP_VERS_PAR "VERSION"   /* keyword in setup file for version   */
#define fcdMAX_LINE        80         /* maximum size of a setup line        */
#define fcdMAX_PARAM       10         /* maximum number of command parameters*/

/* commands */
/* command fcdCMD_VERS */
#define fcdPAR_START_MODE   "imageSave"    /* image save paramneter name     */

/* log IDs */
enum fcdLogId {
    fcdLOG_START = logAUTO_RANGE + 1,
    fcdLOG_OPMODE,
    fcdLOG_SIGNAL,
    fcdLOG_SETUP_SAVE,
    fcdLOG_SETUP_RESTORE,
    fcdLOG_STOP_LOG_REPLY_ERROR,
    fcdLOG_END
};

/*
 * database paths for common attributes 
 */

/*
 * New types definitions
 */

/*
 * operational states of FIERA WS part
 */
typedef enum {                     /* possible states of CCD DCS WS part   */
    fcdWsOFF         = fcdOFF,         /* terminating                      */
    fcdWsLOADED      = fcdLOADED,      /* need to be initialised           */
    fcdWsSTANDBY     = fcdSTANDBY,     /* stand-by                         */
    fcdWsONLINE      = fcdONLINE,   /* ready for commands               */
} fcdWsSTATE;

/*
 * FITS for CCD WS part
 */
typedef struct {                     /* basic FITS info                    */
    fcdFITSBASIC    slcu;            /* basic info from SLCU               */
    vltBYTES16      dateFile;        /* date the file has been written     */
} fcdFITSBASIC_WS;

typedef struct {                     /* cryostat FITS info                 */
    vltBYTES16      description;     /* type                               */
    vltUINT8        esoNumber;       /* eso sequencial number              */
} fcdFITSCRYO_WS;

typedef struct {                     /* FITS info on camera shutter        */
    vltBYTES16      type;            /* shutter type                       */
    vltBYTES16      description;     /* unique identifier                  */
} fcdFITSSHUT_WS;

typedef struct {                     /* FITS info on telemetry sensors     */
    vltBYTES16      id;              /* id as from manufacturer            */
    vltBYTES64      description;     /* unique identifier                  */
} fcdFITSTELE_WS;

typedef struct {                     /* FITS info on chips in the mosaic   */
    vltBYTES16      id;              /* chip description                   */
    vltBYTES16      description;     /* chip identifier                    */
    vltBYTES16      date;            /* chip installation date             */
    vltDOUBLE       xGap;            /* right gap to next chip [um]        */
    vltDOUBLE       yGap;            /* top   gap to next chip [um]        */
} fcdFITSCHIP_WS;

typedef struct {                     /* FITS info on outputs               */
    vltBYTES16      id;              /* id as from manufacturer            */
    vltBYTES16      description;     /* name of the output                 */
} fcdFITSOUTP_WS;

typedef struct {                     /* FITS info on chip mosaic           */
    vltBYTES16      description;     /* chip type                          */
} fcdFITSMOSAIC_WS;

typedef struct {                     /* hierarchical FITS info             */
    fcdFITSHIERARCH    slcu;                /* info from SLCU              */
    vltBYTES32         id;                  /* detector system identifier  */
    vltBYTES32         name;                /* name of detector system     */
    vltBYTES16         date;                /* detector commissioning date */
    vltBYTES32         installed;           /* software installation date  */
    vltDOUBLE          tranTime;            /* transfer time [s]           */
    vltDOUBLE          xPixSize;            /* pixel size in X [um]        */
    vltDOUBLE          yPixSize;            /* pixel size in Y [um]        */
    vltINT32           dumdit;              /* # dummy rdt / exposure      */
    fcdFITSCRYO_WS     cryo;                /* cryostat info               */
    fcdFITSSHUT_WS     shut;                /* shutter  info               */
    fcdFITSTELE_WS     tele[fcdMAXTELSRC];  /* telemetry info              */
    fcdFITSCHIP_WS     chip[fcdMAXCHIPS];   /* chip info                   */
    fcdFITSOUTP_WS     outp[fcdMAXOUTPUT];  /* output info                 */
    fcdFITSMOSAIC_WS   mosaic;              /* mosaic info                 */
} fcdFITSHIERARCH_WS;

typedef struct {                     /* complete DCS FITS info             */
    fcdFITSBASIC_WS    basic;        /* basic info                         */
    fcdFITSHIERARCH_WS hierarch;     /* hierarchical info                  */
} fcdFITS_WS;

typedef struct {                              /* Environment information   */
    fcdCAMERA         camera;                 /* camera info               */  
    ccsPROCNUM        myProcId;               /* my process ID             */
    ccsPROCNAME       myProcName;             /* my process name           */
    ccsPROCNAME       destProcName;           /* destination process name  */
    ccsENVNAME        destEnv;                /* destination CCS enviroment*/
    dbSYMADDRESS      myRootPoint;            /* name of my DB root point  */
    dbSYMADDRESS      destRootPoint;          /* name of dest. DB root point */
    fcdCONFIG         config;                 /* configuration information */
    slxDICTIONARY     ccdDictionary;          /* CCD dictionary name       */
    slxDICTIONARY     fieraDictionary;        /* FIERA dictionary name     */
    logMASK           logMask;                /* FITS log mask             */
} fcdENVVAR;

typedef struct {                  /* information associated to any message */
    vltUINT8          type;                   /* type of message           */
    msgCMD            name;                   /* name                      */
    msgCMDID          id;                     /* ID                        */
    msgPROCESSID      orgId;                  /* originator of message     */
    ccsPROCNAME       proc;                   /* destination process name  */
    ccsENVNAME        env;                    /* destination CCS enviroment*/
    char              body[msgMAXLEN];        /* body of message           */
    msgLENGTH         length;                 /* length of message body    */
    cmdPARAM_LIST     param;                  /* parameters structure      */
    msgTIMEOUT        timeout;                /* timeout to wait for reply */
    msgCHECKFLAG      check;                  /* check flag                */
    vltLOGICAL        sendReply;              /* reply must be sent        */
    vltLOGICAL        lastReply;              /* flag for last reply       */
    ccsERROR          error;                  /* error returned in reply   */
} fcdMSG;

typedef struct fcdMsgLst *fcdMSGLSTPTR;       /* pointer to fcdMSGLST      */
/* 
   Note: better to define variables as (fcdMSGLST *) rather than fcdMSGLSTPTR
   because some debugger, e.g. gdb, shows the contents of the structure only
   in the former case 
*/

typedef struct fcdMsgLst{ /* list of msg whose execution must be completed */
    fcdMSG            msg;                    /* message info              */
    fcdMSGLSTPTR      orgCmd;                 /* entry of original command */
    fcdMSGLSTPTR      prev;                   /* previous entry in the list*/
    fcdMSGLSTPTR      next;                   /* next entry in the list    */
} fcdMSGLST;

/* 
 * Macros 
 */

/*
 * function prototypes
 */

/* log that a CCD program started */
void fcdLogStart( 
    IN  ccsPROCNAME  procName,              /* process name               */
    IN  char        *revision);             /* RCS code revision string   */

/* log that a CCD program terminated */
void fcdLogTerminate( 
    IN  ccsPROCNAME  procName,              /* process name               */
    IN  char        *revision);             /* RCS code revision string   */

/* log that a CCD program runs in a certain operational mode */
void fcdLogOpMode( 
    IN  const char  *opModeDescr);        /* operational mode description */

/* add error to stack, print error info on stdout and close stack */
void fcdPrintErr( 
    IN    ccsPROCNAME   procName,            /* process name              */
    IN    char          *reason,             /* reason for error          */
    IN    ccsERROR      *error);             /* error structure           */

/* do startup actions for any WS CCD program */
ccsCOMPL_STAT fcdProcStart( 
    IN  int argc,                         /* number of runtime parameters */
    IN  char *argv[],                     /* runtime parameters           */
    IN  ccsPROCNAME destProc,             /* name of commands destination */
    IN  char        *revision,            /* RCS code revision string     */
    OUT fcdENVVAR *envInfo);              /* pointer to env. var. values  */

/* terminate CCD process */
void fcdProcTerminate( 
    IN    ccsPROCNAME   procName,            /* process name              */
    IN    char         *revision,            /* RCS code revision string   */
    IN    vltLOGICAL    ccsRegistered,       /* registered already to CCS */
    IN    char          *reason,             /* reason for error          */
    IN    ccsERROR      *error);             /* error structure           */

/* check operational mode */
ccsCOMPL_STAT fcdCheckOpMode( 
    IN  fcdOPMODE opMode,                    /* operational mode          */
    OUT ccsERROR  *error);                   /* error structure           */

/* receive and parse next message */
ccsCOMPL_STAT fcdRecvMsg( 
    INOUT fcdMSGLST         **msg,           /* pointer to message entry  */
    OUT   vltLOGICAL        *logErr,         /* log the error             */
    INOUT ccsERROR          *error);         /* error structure           */

/* Send command to other process and add expected reply to message list */
ccsCOMPL_STAT fcdSendCmd(
    INOUT fcdMSGLST    **msg,             /* entry to list                */
    IN    fcdMSG       *cmd,              /* command info                 */
    IN    vltLOGICAL   link,              /* link to associated command   */
    OUT   ccsERROR     *error);           /* error structure              */

/* send all pending replies in the pending messages list */
ccsCOMPL_STAT fcdSendReply( 
    INOUT fcdMSGLST    **msg,             /* pointer to message entry     */
    OUT   vltLOGICAL   *exit,             /* last reply to EXIT sent      */
    INOUT ccsERROR     *error);           /* error structure              */

/* add suffux to file name */
void fcdFileNameAddSuffix( 
    IN     char       *header,         /* header of original pathname */
    IN     char       *suffix,         /* suffix of original pathname */
    IN     int        index,           /* index to be added as suffix */
    OUT    char       *path);          /* new pathname                */

/* separate suffix from header in a file name */
void fcdFileNameSepSuffix( 
    IN     char       *path,           /* original pathname           */
    OUT    char       *header,         /* file name header            */
    OUT    char       *suffix);        /* file name suffix            */

/* get env. information needed by any CCD WS program */
ccsCOMPL_STAT fcdGetEnv( 
    INOUT fcdENVVAR    *envInfo,           /* pointer to env. var. values */
    OUT   fcdOPMODE    *opMode,            /* operational mode            */
    OUT   ccsERROR     *error);            /* error structure             */

/* get CCD sw operational mode */
ccsCOMPL_STAT fcdGetOpMode (
    OUT fcdOPMODE     *opMode,             /* operational mode            */
    OUT ccsERROR      *error);             /* error structure             */

/* convert operational mode from code to string */
char *fcdOpModeEnumToStr(
    IN fcdOPMODE      opMode);             /* operational mode            */

void fcdHandleBreak(             
    IN  int           sigNum);             /* signal Number               */

/* add new entry in the list of pending messages for any CCD WS program */
ccsCOMPL_STAT fcdMsgLstAdd(
    INOUT fcdMSGLST    **msg,              /* pointer to new entry in list */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a command entry in the list of pending messages */
ccsCOMPL_STAT fcdMsgLstCmdAdd(
    INOUT fcdMSGLST    **msg,              /* pointer to entry in list     */
    IN    msgCMD       *cmdName,           /* command name                 */
    IN    msgCMDID     cmdId,              /* command ID                   */
    IN    msgPROCESSID *orgId,             /* originator ID                */
    IN    char         *body,              /* command body                 */
    IN    msgLENGTH    length,             /* command body length          */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a command entry in the list of pending messages */
ccsCOMPL_STAT fcdMsgLstRplyAdd(
    INOUT fcdMSGLST    **msg,              /* entry to list                */
    IN    vltLOGICAL   link,               /* link to associated command   */
    IN    fcdMSG       *cmd,               /* command info                 */
    OUT   ccsERROR     *error);            /* error structure              */

/* fill in a reply entry in the list of pending messages */
ccsCOMPL_STAT fcdMsgLstRplyFill(
    IN    fcdMSGLST    *rply,              /* reply entry in list          */
    IN    vltUINT8     type,               /* reply type                   */
    IN    const char   *command,           /* associated command           */
    IN    msgPROCESSID *orgId,             /* reply originator ID          */
    IN    char         *body,              /* pointer to reply body        */
    IN    msgLENGTH    length,             /* command length               */
    IN    vltLOGICAL   last,               /* last reply                   */
    IN    ccsERROR     *error);            /* reply error                  */

/* delete one entry in the list of pending messages for CCD WS program*/
ccsCOMPL_STAT fcdMsgLstDel(
    INOUT fcdMSGLST    **msg,              /* pointer to next entry in list*/
    OUT   ccsERROR     *error);            /* error structure              */

/* query list of pending messages for CCD WS program */
ccsCOMPL_STAT fcdMsgLstSearch(
    INOUT fcdMSGLST    **msg,              /* pointer to new entry in list */
    IN    vltUINT8     type,               /* message type                 */
    IN    msgCMD       *cmdName,           /* command name                 */
    IN    msgCMDID     cmdId,              /* command ID                   */
    IN    msgPROCESSID *orgId,             /* originator ID                */
    OUT   vltLOGICAL   *logErr,            /* if TRUE, log error           */
    OUT   ccsERROR     *error);            /* error structure              */

/* add reply information for pending command */
void fcdMsgLstRplyBuild(
    IN    fcdMSGLST    *cmd,               /* entry to command associated  */ 
    IN    char         *reply,             /* reply body                   */
    IN    msgLENGTH    length,             /* reply length                 */
    IN    vltLOGICAL   last,               /* last reply                   */
    IN    ccsERROR     *error);            /* error reply                  */

/* Delete all links to a command in the pending messages list */
void fcdMsgLstCmdLinkDel(
    IN    fcdMSGLST    *cmd);              /* entry to command associated  */ 

/* Check if there are links to a command in the pending messages list */
vltLOGICAL fcdMsgLstCmdLinkCheck(
    IN    fcdMSGLST    *cmd);              /* entry to command associated  */ 

/* Check if there are pending commands ready for reply */
vltLOGICAL fcdMsgLstCmdReady(
    INOUT    fcdMSGLST   **cmd);           /* pointer to entry to command  */

/* Return timeout value for next message receive and associated message */
void fcdMsgLstTimeout(
    OUT      msgTIMEOUT   *timeout,        /* timeout value               */
    INOUT    fcdMSGLST    **msg);          /* pointer to message entry    */

/* Initialise message structure */
void fcdMsgInit(
    INOUT fcdMSG       *msg);              /* message structure to init   */ 

/* Add one value to the message body */
void fcdMsgAddValue(
    INOUT fcdMSG       *msg,               /* message structure            */ 
    IN    cmdPARAM_TYPE type,              /* parameter type               */ 
    IN    const void   *value,             /* parameter value              */ 
    IN    vltLOGICAL   newPar);            /* first value of new parameter */ 

/* Get the status of an exposure */
ccsCOMPL_STAT fcdGetExpStatus( 
    INOUT vltINT32 *expId,          /* ID of exposure                        */
    OUT   vltINT32 *expStatus,      /* status of exposure                    */
    OUT   ccsERROR *error);         /* error structure                       */

/* Set the status of an exposure */
ccsCOMPL_STAT fcdSetExpStatus( 
    INOUT vltINT32 *expId,          /* ID of exposure                        */
    IN    vltINT32 *expStatus,      /* status of exposure                    */
    OUT   ccsERROR *error);         /* error structure                       */

/* Write in database new operational state for CCD WS sw */
ccsCOMPL_STAT fcdSetOpState(
    IN    fcdWsSTATE   opState,      /* operational state code           */
    OUT   ccsERROR     *error);      /* error structure                  */

/* Get ID of last exposure started */
ccsCOMPL_STAT fcdGetLastExpId(
    OUT vltINT32      *expId,              /* exposure ID                    */
    OUT ccsERROR      *error);             /* error structure                */

/* get name of this program */
void fcdGetProgName( 
    IN  char          *argv0,                /* runtime parameters        */
    OUT ccsPROCNAME   progName);             /* pointer program name      */

/* get name of CCD process */
void fcdGetProcName( 
    IN  fcdCAMERANAME cameraName,            /* name of the camera used   */
    IN  ccsPROCNAME   progName,              /* name of the program       */
    OUT ccsPROCNAME   procName);             /* pointer to process name   */

/* Send a command to Midas and return its reply */
ccsCOMPL_STAT fcdMidasCmd(
    IN  char *midasId,            /* midas Unit ID string                  */
    IN  char *midasHost,          /* host where Midas is running           */
    IN  char *midasCmd,           /* Command for Midas                     */
    OUT msgLENGTH *rplyLen,       /* length of last reply from Midas       */
    OUT char **rplyBody,          /* pointer to Midas reply body           */
    OUT ccsERROR *error);         /* error structure                       */

/* Get path for midas procedure usable from within Midas and image directory */
ccsCOMPL_STAT fcdObjGetMidasProc(
    IN  char  *imgPath,           /* image file full path                 */
    IN  char  *procName,          /* name of Midas procedure              */
    OUT char  **imgDir,           /* image directory name                 */
    OUT char  **imgFile,          /* image file name                      */
    OUT char  *procPath,          /* path of procedure found              */
    OUT ccsERROR *error);         /* error structure                      */

/* Do we use image extensions ? */
ccsCOMPL_STAT fcdGetImExt(
   INOUT vltLOGICAL *doWeUseImageExtensions);

/* For compatibility with ancient layers of sw... */
ccsCOMPL_STAT fcdGetOutputDim (
    IN fcdCONFIG    *config,       /* configuration of camera used           */
    IN vltINT32     clkUsd,        /* index of clock pattern used            */
    IN vltINT32     outUsd,        /* index of output requested              */
    OUT vltINT32    *xDim,         /* window x dimension for specified output*/
    OUT vltINT32    *yDim,         /* window y dimension for specified output*/
    INOUT ccsERROR  *error);       /* error structure                        */

#ifdef __cplusplus
}
#endif

#endif /* ! CCDWSINTERNAL_H */
