#ifndef FCDITIF_H
#define FCDITIF_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcditIf.h,v 2.56 2005/07/29 13:03:10 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* abalestr  27/05/98  created
* rdonalds  22/06/98  Added OUT ID field to structure
*                     Changed cdelt1+2 to double
* rdonalds  06/07/98  Added unordData to the header for images which
*                     are not ordered. All data is transferred as extension 0
* abalestr  27/08/98  added ron field to FITS header
* rdonalds  09/10/98  Added the IP_RESULT sctructure (used by WFOV)
* rdonalds  13/01/99  Modified the definition of MAXINTEGR to 100
* rdonalds  22/02/99  Added resetRTDToZero
* abalestr  24/09/02  Aligned structures: the problem is that on sparc/motorola
                      doubles are aligned on 8 bytes boundary while on x86 &
		      Linux they are aligned on 4 bytes.
******************************************************************************
* BE VERY CAREFUL IN MODIFYING THE STRUCTURES!!!!!!!!!!!!! TAKE ALWAYS INTO  *
* ACCOUNT DIFFERENT ALIGNMENTS ON MOTOROLA AND INTEL!!!!!!!!!!!!!!!!!!!!!!!!!*
******************************************************************************
*/

/*
 *# We may have to reproduce some FIERA definitions here, because
 *# when using this interface from the NoVltIf there is no access to
 *# normal FIERA types
 */
#ifndef fcdMAXINTEGR
#define fcdMAXINTEGR 100
#define fcdMAXWINDOW 2
#define fcdMAXTELSRC 100
#define fcdMAXOUTPUT 128
#endif

#define fcditPORTNUMBER 5333
#define fcditMAXNUMEXT 32

typedef enum {fcdRIGHT2LEFT, fcdLEFT2RIGHT} xDirection; 
typedef enum {fcdDOWN2UP, fcdUP2DOWN} yDirection;

/*
 *# These structures are meant for FITS data coming from the SLCU to
 *# the INS WS. Some of these data may be also computated by the INS WS but,
 *# in order to keep these computations to a minimum, the SLCU is in charge
 *# of them.
 *# This is NOT a list of keywords that will be written to the FITS file. Thes
 *# structures only help the transmission of information
 *# As the name of variables is kept (as far as possible) equal to the
 *# correspondent FITS keyword, refer to the CCD and FCD dictionaries
 *# for the meaning.
 */

/*
 *# FITS for main header
 */

/*
 *# Basic FITS info
 */
typedef struct {
  vltINT32	bitpix;			/* BITPIX value			   */
  vltINT32	windowN;		/* number of windows used	   */
  vltBYTES32	dateObs;		/* date when exposure started	   */
  vltDOUBLE	mjd_start;		/* modified julian date of start   */
  vltDOUBLE	expTime;		/* actual exposure time	  	   */
} fcdFITS_BASIC;

/*
 *# Hierarchical FITS info
 */


/*
 *# Hierarchical keywords - window
 */
typedef struct {
  vltLOGICAL st;		/* DET WINi ST value			   */
  vltINT8    binx;		/* DET WINi BINX value			   */
  vltINT8    biny;		/* DET WINi BINY value			   */
  vltINT16   strx;		/* DET WINi STRX value			   */
  vltINT16   stry;		/* DET WINi STRY value			   */
  vltINT16   nx;		/* DET WINi NX value			   */
  vltINT16   ny;		/* DET WINi NY value			   */
  vltINT32   ndit;		/* DET WINi NDIT value			   */
  vltDOUBLE  uit[fcdMAXINTEGR];	/* DET WINi UIT value			   */
  vltDOUBLE  dit[fcdMAXINTEGR];	/* DET WINi DIT value			   */
  vltDOUBLE  dktm;		/* DET WINi DKTM value			   */
} fcdFITS_WINDOW;

/*
 *# Hierarchical keywords - telemetry
 */
typedef struct {
  char        dummy[4];         /* alignment				   */
  vltINT32    dbIndex;		/* Database index of telemetry item	   */
  vltDOUBLE   start;		/* DET TLMi START value			   */
  vltDOUBLE   end;		/* DET TLMi END value			   */
  vltDOUBLE   max;		/* DET TLMi MAX value			   */
  vltDOUBLE   min;		/* DET TLMi MIN value			   */
  vltDOUBLE   mean;		/* DET TLMi MEAN value			   */
  vltDOUBLE   var;		/* DET TLMi VAR value			   */
} fcdFITS_TELEMETRY;

/*
 *# Hierarchical keywords - all
 */
typedef struct {
  /* 
   *# detector hierarchy 
   */
  vltINT32    bits;		/* DET BITS value			   */
  vltINT32    outputs;		/* DET OUTPUTS value			   */
  vltINT32    outref;		/* DET OUTREF value			   */
  /* detector read hierarchy */
  vltINT32    shift;		/* DET READ SHIFT value			   */
  vltBYTES16  mode;		/* DET READ MODE value			   */
  vltBYTES16  speed;		/* DET READ SPEED value			   */
  vltBYTES32  clock;		/* DET READ CLOCK value			   */
  /* 
   *# detector exposure hierarchy 
   */
  vltINT32    id;		/* DET EXP ID value			   */
  vltINT32    expType;		/* DET EXP TYPE value			   */
  vltDOUBLE   rdttime;		/* DET EXP RDTTIME value		   */
  vltDOUBLE   proctim;		/* DET EXP PROCTIM value		   */
  vltDOUBLE   xfertim;		/* DET EXP XFERTIM value		   */
  /* 
   *# detector frame hierarchy 
   */
  vltINT32    frmId;		/* DET FRAM ID value			   */
  vltINT32    frmType;		/* DET FRAM TYPE value			   */
  /* 
   *# detector shutter hierarchy 
   */
  vltDOUBLE   tmopen;		/* DET SHUT TMOPEN value		   */
  vltDOUBLE   tmclos;		/* DET SHUT TMCLOS value		   */
  /* 
   *# detector telemetry hierarchy 
   */
  vltINT32    nValidTel;	/* number of valid telemetry items  	   */
  char	      dummy[4];         /* alignment */
  vltDOUBLE   telPeriod;        /* telemetry sampling period in seconds    */
  fcdFITS_TELEMETRY	teleInfo[fcdMAXTELSRC];
} fcdFITS_HIERARCH;


/*
 *# FITS information for extensions
 *# All the information come from the SLCU
 */

/*
 *# Basic
 */
typedef struct {
  vltINT32	naxis1;		/* NAXIS1 value				   */
  vltINT32	naxis2;		/* NAXIS2 value				   */
  vltDOUBLE	cdelt1;		/* CDELT1 value				   */
  vltDOUBLE	cdelt2;		/* CDELT2 value				   */
  vltDOUBLE	crval1;		/* CRVAL1 value				   */
  vltDOUBLE	crval2;		/* CRVAL2 value				   */
  vltDOUBLE	crpix1;		/* CRPIX1 value				   */
  vltDOUBLE	crpix2;		/* CRPIX2 value				   */
  vltDOUBLE	bscale;		/* BSCALE value				   */
  vltDOUBLE	bzero;		/* BZERO value				   */
  vltBYTES32	extname;	/* EXTNAME value                           */
} fcdEXT_FITS_BASIC;

/*
 *# Hierarchical keywords - chip
 */
typedef struct {
  char dummy[4];		/* alignment                               */
  vltINT32 index;		/* CHIP INDEX value			   */
  vltINT32 x;			/* CHIP X coordinate			   */
  vltINT32 y;			/* CHIP Y coordinate			   */
  vltINT32 nx;			/* CHIP NX value			   */
  vltINT32 ny;			/* CHIP NY value			   */
  vltDOUBLE pszx;		/* CHIP PSZX value			   */
  vltDOUBLE pszy;		/* CHIP PSZY value			   */
  vltDOUBLE xGap;		/* CHIP XGAP value			   */
  vltDOUBLE yGap;		/* CHIP YGAP value			   */
  vltBYTES16 id;		/* CHIP ID value			   */
  vltBYTES16 name;		/* CHIP NAME value			   */
  vltBYTES16 date;		/* CHIP DATE value			   */
} fcdEXT_FITS_CHIP;

/*
 *# Hierarchical keywords - output
 */
typedef struct {
  vltBYTES8  id;		/* OUT ID value			           */
  vltBYTES16 name;		/* OUT NAME value			   */
  vltINT32 index;		/* OUT INDEX value			   */
  vltINT32 chip;		/* OUT CHIP value			   */
  vltINT32 x;			/* OUT X coordinate			   */
  vltINT32 y;			/* OUT Y coordinate			   */
  vltINT32 nx;			/* OUT NX value				   */
  vltINT32 ny;			/* OUT NY value				   */
  vltINT32 prscx;		/* OUT PRSCX value			   */
  vltINT32 prscy;		/* OUT PRSCY value			   */
  vltINT32 ovscx;		/* OUT OVSCX value			   */
  vltINT32 ovscy;		/* OUT OVSCY value			   */
  vltDOUBLE conad;		/* OUT CONAD value			   */
  vltDOUBLE ron;		/* OUT RON value			   */
  vltDOUBLE gain;		/* OUT GAIN value			   */
} fcdEXT_FITS_OUT;

/*
 *# END OF FITS STRUCTURES
 */

/*
 *# EXTENSION_INFO
 *#
 *# Stores info for extensions (coming from SLCU)
 */
typedef struct {
  vltINT32 xStart;	/* X,Y offsets first line transmitted */
  vltINT32 yStart;	/* in "tiled" coordinates             */
  xDirection xDir;	/* x readout direction                */
  yDirection yDir;	/* y readout direction                */

  /* Structures for FITS info */ 
  fcdEXT_FITS_BASIC	basicExtFits;
  fcdEXT_FITS_CHIP	chipFits;
  fcdEXT_FITS_OUT	outFits;

} fcditEXTENSION_INFO;

/*
 *# IMAGE_INFO
 *#
 *# Stores general info for image (coming from SLCU)
 */
typedef struct {
  vltLOGICAL resetRTDToZero;    /* reset the RTD memory to zeros before    */
                                /* acquisition                             */
  vltINT32 size;		/* image size (in bytes)		   */
  vltINT32 xTotDim;		/* total X/Y dimensions in tiled coords    */
  vltINT32 yTotDim;		
  vltINT32 nExt;		/* number of image extensions in image     */
  vltINT32 nWin;		/* number of windows in image		   */
  vltINT32 diskSaveMethod;	/* disk save method			   */
  vltINT32 frameId;		/* real-time display frame ID		   */
  vltINT32 expId;		/* exposure ID		   		   */
  vltINT32 expSeqNum;           /* number in sequence                      */
  vltINT32 nFrame;              /* number of frames read                   */
  slxFILENAME fileUnc;		/* name of file for image uncompressed	   */
} fcditIMAGE_INFO;

/*
 *# Results of image processing passed back for centroiding with the WFOV
 */
typedef struct {
    char      dummy[4];		/* alignment                               */
    /* setup for window and IP */
    vltINT32  xFirst;        /* start coordinate of window */
    vltINT32  yFirst;
    vltINT32  nx;            /* window dimension */
    vltINT32  ny;

    vltINT32  ipLLX;         /* offset in window for processing */
    vltINT32  ipLLY;
    vltINT32  ipURX;
    vltINT32  ipURY;
    vltINT32  ipNumPix;
    vltDOUBLE ipCentrRefX;   /* centre reference point */
    vltDOUBLE ipCentrRefY;

    vltDOUBLE ipXCen;        /* centre coordinate */
    vltDOUBLE ipYCen;
    vltDOUBLE ipXErr;        /* erorr from centre to reference */
    vltDOUBLE ipYErr;
    vltDOUBLE ipCenVal;      /* value at centre */
    vltDOUBLE ipXFWHM;
    vltDOUBLE ipYFWHM;
    vltDOUBLE ipBackGnd;
} fcditIP_RESULT;

/*
 *# HEADER_INFO
 *#
 *# Stores all info (general + extensions) for image (coming from SLCU)
 */
typedef struct {
    char                dummy[4];   /* alignment */
    vltLOGICAL          hasTrailer; /* indicates if TRAILER_INFO is 
				       transmitted */
    vltINT32            reserved;   /* reserved as future discriminator */
    fcditIMAGE_INFO	imageInfo;
    fcdFITS_BASIC	basicFits;
    fcdFITS_WINDOW	winInfo[fcdMAXWINDOW];
    fcditEXTENSION_INFO	extInfo[fcditMAXNUMEXT];
    fcditIP_RESULT      ipResult;
} fcditHEADER_INFO;

/*
 *# CHUNK_INFO
 *#
 *# Header for each chunk of data
 */
typedef struct {
  vltINT32	extIndex;	/* extension index                         */
  vltINT32	numOfBytes;	/* number of incoming bytes                */
} fcditCHUNK_INFO;

/*
 *# TRAILER_INFO
 *#
 *# Header for trailer data
 */
typedef struct {
  fcdFITS_HIERARCH	hierFits;
} fcditTRAILER_INFO;

#endif /*!_H*/
