#ifndef FCDITWSIMAGE_H
#define FCDITWSIMAGE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcditWs_IMAGE.h,v 1.88 2005/10/06 14:47:35 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* abalestr  03/12/97  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#define _POSIX_SOURCE 1

#include <vltPort.h>
#include <fcditWs_INTERNAL.h>
#include <fcditWs_RTD.h>
#include <fcditWs_FITS.h>

/*
 * byte swapping macros (courtesy of CCS Lite)
 */

/* most significant byte of 2-byte integer */
#define FCD_MSB_INT16(x)        (((x) >>  8) & 0xff)
/* least significant byte of 2-byte integer */
#define FCD_LSB_INT16(x)         ((x)        & 0xff)
/* most significant word of 2-word integer */
#define FCD_MSW_INT32(x) (((x) >> 16) & 0xffff)
/* least signifcant byte of 2-word integer */
#define FCD_LSW_INT32(x)  ((x)        & 0xffff)

/* swap the MSB with the LSB of a 16 bit integer */
#define FCD_SWAP_INT16(x) (FCD_MSB_INT16(x) | (FCD_LSB_INT16(x) << 8))

/* swap all bytes of a 32 bit integer */
#define FCD_SWAP_INT32(x) (FCD_SWAP_INT16(FCD_LSW_INT32(x)) << 16) | \
                             FCD_SWAP_INT16(FCD_MSW_INT32(x))
/*void ccs_swap_2int32(int32_t* i1, int32_t* i2)
  { int32_t i=*i1; *i1=*i2; *i2=i; };*/
                          
/* swap all bytes of two consecutive 32 bit integers */
#define FCD_SWAP_2INT32(x_ptr,y_ptr) {int32_t i=*x_ptr;\
                                     *x_ptr=*y_ptr;\
                                     *y_ptr=i;}\
                           *x_ptr = FCD_SWAP_INT32(*x_ptr);         \
                           *y_ptr = FCD_SWAP_INT32(*y_ptr) 
/* courtesy alongino */			   
#define FCD_SWAP_FLOAT(v)  {vltINT32 *i1; i1=(vltINT32 *)&(v); \
                            *i1 = FCD_SWAP_INT32(*i1);} 
#define FCD_SWAP_DOUBLE(v) {vltINT32 *i1,*i2; i1=(vltINT32 *)&(v); \
                            i2=i1+1;FCD_SWAP_2INT32(i1,i2);} 


void fcdSwapHeaderInfo(fcditHEADER_INFO *headerInfo);
void fcdSwapChunkInfo(fcditCHUNK_INFO *chunkInfo);
void fcdSwapTrailerInfo(fcditTRAILER_INFO *trailerInfo);

/* class which holds image information */
class fcditWs_IMAGE
{
public:
  fcditWs_IMAGE();
  ~fcditWs_IMAGE();

  /*
   *# Get and extract info from header
   *# 
   */
  ccsCOMPL_STAT Init(const vltINT32 connectionDescriptor, 
		     fcdDETID detectorId, fcdDETCONFIG detectorConfiguration);

  /*
   *# Get and extract info from header
   *# 
   */
  ccsCOMPL_STAT GetHeader(void);

  /*
   *# Get data
   *# 
   */
  ccsCOMPL_STAT GetData(void);

  /*
   *# Get and extract info from trailer
   *# 
   */
  ccsCOMPL_STAT GetTrailer(void);

  /*
   *# Process image: show and/or save
   *# 
   */
  ccsCOMPL_STAT Close(void);

  /*
   *# Set default values
   *# 
   */
  void SetDefaults(void);

  /*
   *# Set standalone mode
   *# 
   */
  ccsCOMPL_STAT SetStandAlone(vltLOGICAL);

  /*
   *# Accessor functions
   */
  vltLOGICAL GetErrorFlag() {return errorFlag;}

  /*
  ** Assess status of connection
  */
  vltLOGICAL ConnectionReset(void) { return connectionReset; }
  void ClearConnectionReset(void) { connectionReset = ccsFALSE; }

private:
  vltLOGICAL		updateRtd;
  vltLOGICAL		doBuildUp;
  vltLOGICAL		doRtd;
  vltLOGICAL		saveData;
  vltLOGICAL		isInited;
  vltLOGICAL		isHeaderDone;
  vltLOGICAL		isDataDone;
  vltLOGICAL		isTrailerDone;
  vltLOGICAL		standAlone;
  vltLOGICAL		errorFlag;
  vltLOGICAL            connectionReset; // keeps track of connection resets
  vltLOGICAL		noMoreTransLog;
  vltLOGICAL		lcuLittleEndian; // Is lcu little endian?
  vltLOGICAL		wsLittleEndian;  // Is ws little endian?
  vltLOGICAL		doSwap;  	 // Shall we swap?
  
  vltINT32		sockDesc;
  vltINT32		expIndex;
  vltINT32		totImageBytes;
  
  vltDOUBLE		timeStart;
  vltDOUBLE		timeDuration;

  fcdDETID		locDetId;

  fcdOUTEXTDESC		outExt[fcditMAXNUMEXT];

  fcditWs_RTD		locRtd;
  fcditWs_FITS		locFits;

  const char *addrArray[fcditMAXNUMEXT];
  
  // Database points
  dbSYMADDRESS		dbItOpState;	/* Transfer status		*/
  dbSYMADDRESS		dbItStatus;	/* Transfer completion status	*/
  dbSYMADDRESS		dbItRtdFrame;	/* Info for real time display	*/
  dbSYMADDRESS		dbItTimeStart;	/* Transfer start time		*/
  dbSYMADDRESS		dbItTime;	/* Transfer duration		*/
  dbSYMADDRESS		dbItPerc;	/* Percentage completed		*/
  
  // Structures for image transfer
  fcditHEADER_INFO	headerInfo;
  fcditTRAILER_INFO	trailerInfo;
  fcditCHUNK_INFO	chunkInfo;

  // Structures for FITS keywords
  fcdFITS_BASIC    	fitsBasicSlcu;
  
  oslxDICTIONARY	oslxDict;

  ccsCOMPL_STAT GetBuffer(const int, void *);
  ccsCOMPL_STAT	SetUTCStart(void);
  ccsCOMPL_STAT	SetUTCDuration(void);
  ccsCOMPL_STAT	GetFileName(slxFILENAME &);
};

// External library to perform operations on image and/or file
ccsCOMPL_STAT fcditNewData(void *data, rtdIMAGE_INFO *rtdImageInfo);
ccsCOMPL_STAT fcditNewFileData(const slxFILENAME &fileName);

/*****************************************************************
    INLINE METHODS
*****************************************************************/

#endif /*!_H*/
