#ifndef FCDLINTF_H
#define FCDLINTF_H

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdlIntf.h,v 3.50 2005/04/13 09:14:23 vltsccm Exp $"
*
* fcdlIntf.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  17/05/96  created as fcdlih.h
* ccumani   20/05/96  - header added
*                     - DSP destination process IDs added
*                     - command destination added
* rdonalds  22/05/96  Addition of DEFINE_SEQ and associated parameters
* rdonalds  10/06/96  - Extensive modification to the Sequence, Pattern and 
*                       Usequence definition.
*                     - Removal of READ, WIPE, INTEGRATE commands replaced 
*                       by a generic EXEC_SEQ command which sets a number of
*                       voltages and executes a previously downloded sequence
*                     - Addition of id's to voltage table so they can be 
*                       referenced from
* ccumani   16/07/96  - Access Code (bits A21-A20) macros added
*                     - AnaBias Board (bits A04-A00) macros added
* rdonalds  23/7/96   - Modified the definitions of ANB_PRESET_VOLT_XX so they
*                       do not start with 0. Otherwise compiler interprets 
*                       them as octal
* ccumani   10/01/97  - renamed as fcdlInterface.h and moved in the
*                       fcdl module
*                     - inclusion of general.h and fiera.h deleted
*                     - old macros from fiera.h substituted with values
*                     - ANB_NUM_DACS macro added
* ccumani   27/02/97  - new Board Ids
* ccumani   21/04/97  - renamed as fcdlIntf.h (8 characters, for MSDOS
*                       compatibility)
*                     - usage of "fcdl" prename
* ccumani   23/05/97  - fcdlCMD_OFF added in fcdlCommandId
*                     - fcdlCMD_CHANGE_LOG_LEVEL added in fcdlCommandId
*                     - typedef enum fcdlTestId definition added
* ccumani   02/06/97  - fcdlCMD_CHECK_VOLTAGE -> fcdlCMD_CHECK_VOLTAGES
* ccumani   06/06/97  - fcdlMAX_SEQ_ID changed from 20 to 40
*                     - fcdlMAX_USEQ_ID changed from 50 to 70
* ccumani   03/07/97  - file ordered in sections
*                     - macro fcdlMAX_SEQ_NEST added
* ccumani   16/07/97  - fcdlCLK_TYPE typedef'd
* ccumani   13/08/97  - definition of VID_ENABLE_MASK moved from fcdlLocl.h
* rdonalds  16/08/97  - board revision number added in fcdlBrdConfig
* ccumani   17/08/97  - fcdlCLK_NUM_SIMMS set equal to 14
* ccumani   13/10/97  - fcdlMAX_SEQ_ID changed from 40 to 200
*                     - fcdlMAX_USEQ_ID changed from 70 to 200
* ccumani   17/12/97  - fcdlMAX_VOLTAGE_TABLE_ID changed from 6 to 100
* ccumani   12/05/98  - BRD_ANABIAS2 and BRD_ANABIAS3 macros added
*                     - BRD_IS_ANABIAS and ANABIAS_TO_INDEX macros updated
* ccumani   03/08/98  - fcdlCMD_EXEC_SEQ_AT_SIG command added
* ccumani   09/02/99  - new test Ids added in fcdlTestId for sending
*                       data from DSP2
* ccumani   23/02/99  - fcdlMIN_TICKS_TO_WAIT definition added
* ccumani   06/05/99  - fcdlCMD_EXEC_N_SEQS command added
*                     - fcdlExecNSeqsParams struct added
* ccumani   14/06/99  - VID_GAIN3_CHAN[0-3] macros added, for Video 
*                     - hasPreamp2 added in fcdlBrdConfig, for Video 
*                       Preamp 2
* ccumani   13/07/99  - fcdlTestAO[x] added in fcdlTestId typedef
* ccumani   10/08/99  - paramas2 element added in fcdlDoTestParams
*                       structure
*                     - fcdlTestAO6 added in fcdlTestId typedef, for
*                       new NAOS general test mode
* ccumani   14/11/00  - new 16 Voltage definitions added for Anabias
*                       boards (Rev. 3 has 32 Voltages)
*                     - Anabias hw addresses moved into fcdlAnb.h
*/

/************************************************************************
 *
 *    This file describes the interface between the c40 and the SPARC in the
 *    FIERA system.  The dsp sw is passive. All commands are initiated from
 *    the SPARC with a message type of fcdlCOMMAND. The c40 can then respond
 *    with messages of type fcdlREPLY and when the operation is complete
 *    with a reply of type fcdlLAST_REPLY.
 *
 *    The commands are sent using a fcdlMsgHdr followed by a number
 *    of parameter bytes. The format of the parameters is described for
 *    each c40CommandId. Replies are formated in the same way but with the
 *    destination and source identifiers reversed.
 *
 *    A return type of fcdlERROR_REPLY is possible in which case the reply
 *    buffer should contain a vltINT32 with an error code defined in the
 *    fcdErrors.h file.
 *
 *----------------------------------------------------------------------
 */


#include "ccs.h"

/*****************************************************************************
 *# DSP destination processes IDs
 *# -----------------------------
 */
#define DSP_PRIPH   0x10001    /* Periph Control Process  */
#define DSP_DSP2    0x10002    /* DSP2 Control Process    */


/*****************************************************************************
 *# Hardware Addresses
 *# ------------------
 */

/*
 *# Access Code: bits A21-A20
 *  if new access codes are added the routine
 *  fcdlAccCodeNameToId should be updated
 */
#define ACC_ADDRTOPMASK          0x300000L
#define ACC_SEQ_STATUS           0x000000L
#define ACC_SEQ_BRDCST           0x100000L
#define ACC_SEQ_SRAM             0x200000L
#define ACC_PERIPHERAL           0x300000L
extern vltINT32 fcdlAccCodeNameToId (const char *name);


/*
 *# Board ID: bits A19-A16
 *  Used as values for fcdlBoardId
 *  if new ids are added the routine
 *  fcdlBoardNameToId should be updated
 */
#define BRD_BRDCAST            0x000000L
#define BRD_COMMBRD            0x010000L
#define BRD_ANABIAS0           0x020000L
#define BRD_ANABIAS1           0x030000L
#define BRD_CLKDRV0            0x040000L
#define BRD_CLKDRV1            0x050000L
#define BRD_CLKDRV2            0x060000L
#define BRD_CLKDRV3            0x070000L
#define BRD_VIDBRD0            0x080000L
#define BRD_VIDBRD1            0x090000L
#define BRD_VIDBRD2            0x0a0000L
#define BRD_VIDBRD3            0x0b0000L
#define BRD_ANABIAS2           0x0e0000L
#define BRD_ANABIAS3           0x0f0000L
extern vltINT32 fcdlBoardNameToId (const char *name);

/*
 *# macros for board id manipulation
 */
#define BRD_IS_VIDBRD(board) \
        (board == BRD_VIDBRD0 || board == BRD_VIDBRD1 || \
        board == BRD_VIDBRD2 || board == BRD_VIDBRD3)

#define BRD_IS_CLKDRV(board) \
        (board == BRD_CLKDRV0 || board == BRD_CLKDRV1 || \
        board == BRD_CLKDRV2 || board == BRD_CLKDRV3)

#define BRD_IS_ANABIAS(board) \
        (board == BRD_ANABIAS0 || board == BRD_ANABIAS1 || \
        board == BRD_ANABIAS2 || board == BRD_ANABIAS3)

#define BRD_IS_COMMBRD(board) (board == BRD_COMMBRD)

#define VIDBRD_TO_INDEX(board) ((board - BRD_VIDBRD0)>> 16)

#define ANABIAS_TO_INDEX(board) ((board == BRD_ANABIAS0) ? 0 : \
                                ((board == BRD_ANABIAS1) ? 1 : \
                                ((board == BRD_ANABIAS2) ? 2 : \
                                ((board == BRD_ANABIAS3) ? 3 : -1))))

#define CLKDRV_TO_INDEX(board) ((board == BRD_CLKDRV0) ? 0 : \
                               ((board == BRD_CLKDRV1) ? 1 : \
                               ((board == BRD_CLKDRV2) ? 2 : \
                               ((board == BRD_CLKDRV3) ? 3 : -1))))

/* 
 *# maximum board number
 *  this is due to addressing constraints 
 */
#define fcdlMAX_BOARDS 15


/*****************************************************************************
 *# AnaBias Board Periphs
 *# ---------------------
 */

/* Number of DACS per analogue bias board */
#define ANB_NUM_DACS         32

/* Preset Voltages IDs, note that these ids are used as DAC indexes */
#define ANB_PRESET_VOLT_A     0
#define ANB_PRESET_VOLT_B     1
#define ANB_PRESET_VOLT_C     2
#define ANB_PRESET_VOLT_D     3
#define ANB_PRESET_VOLT_E     4
#define ANB_PRESET_VOLT_F     5
#define ANB_PRESET_VOLT_G     6
#define ANB_PRESET_VOLT_H     7
#define ANB_PRESET_VOLT_I     8
#define ANB_PRESET_VOLT_J     9
#define ANB_PRESET_VOLT_K    10
#define ANB_PRESET_VOLT_L    11
#define ANB_PRESET_VOLT_M    12
#define ANB_PRESET_VOLT_N    13
#define ANB_PRESET_VOLT_O    14
#define ANB_PRESET_VOLT_P    15

/* Anabias Rev. 3 has 32 Voltages */
#define ANB_PRESET_VOLT_AA   16
#define ANB_PRESET_VOLT_AB   17
#define ANB_PRESET_VOLT_AC   18
#define ANB_PRESET_VOLT_AD   19
#define ANB_PRESET_VOLT_AE   20
#define ANB_PRESET_VOLT_AF   21
#define ANB_PRESET_VOLT_AG   22
#define ANB_PRESET_VOLT_AH   23
#define ANB_PRESET_VOLT_AI   24
#define ANB_PRESET_VOLT_AJ   25
#define ANB_PRESET_VOLT_AK   26
#define ANB_PRESET_VOLT_AL   27
#define ANB_PRESET_VOLT_AM   28
#define ANB_PRESET_VOLT_AN   29
#define ANB_PRESET_VOLT_AO   30
#define ANB_PRESET_VOLT_AP   31

#define ANB_OPTOOUT          32

extern vltINT32 fcdlPeriphNameToId (const char *name);


/*****************************************************************************
 *# VideoBoard peripherals
 *# ----------------------
 */

#define VID_GAIN1_CHAN0       0
#define VID_GAIN1_CHAN1       1
#define VID_GAIN1_CHAN2       2
#define VID_GAIN1_CHAN3       3

#define VID_GAIN2_CHAN0       4
#define VID_GAIN2_CHAN1       5
#define VID_GAIN2_CHAN2       6
#define VID_GAIN2_CHAN3       7

#define VID_FILT_CHAN0        8
#define VID_FILT_CHAN1        9
#define VID_FILT_CHAN2       10
#define VID_FILT_CHAN3       11

#define VID_OFFSET_CHAN0     12
#define VID_OFFSET_CHAN1     13
#define VID_OFFSET_CHAN2     14
#define VID_OFFSET_CHAN3     15

#define VID_TESTVID_CHAN0    16
#define VID_TESTVID_CHAN1    17
#define VID_TESTVID_CHAN2    18
#define VID_TESTVID_CHAN3    19

/* for Video Preamp 2 - CCU 14/06/99 */
#define VID_GAIN3_CHAN0      20
#define VID_GAIN3_CHAN1      21
#define VID_GAIN3_CHAN2      22
#define VID_GAIN3_CHAN3      23

/* 
 * Mask of all ENABLE bits on video
 * this is needed to invert the OUTPUT ENABLE bits which are active low
 */
#define VID_ENABLE_MASK      0x0000f000

/*****************************************************************************
 *# ClkDrv peripherals
 *# ------------------
 *
 *  NOTE. The fact that all VSS are even and VDD are odd 
 *        is used in fcdlClk.c. 
 *        Please check before modifying these definitions
 */
#define CLKDRV_DAC0_VSS        0
#define CLKDRV_DAC0_VDD        1
#define CLKDRV_DAC1_VSS        2
#define CLKDRV_DAC1_VDD        3
#define CLKDRV_DAC2_VSS        4
#define CLKDRV_DAC2_VDD        5
#define CLKDRV_DAC3_VSS        6
#define CLKDRV_DAC3_VDD        7
#define CLKDRV_DAC4_VSS        8
#define CLKDRV_DAC4_VDD        9
#define CLKDRV_DAC5_VSS       10
#define CLKDRV_DAC5_VDD       11
#define CLKDRV_DAC6_VSS       12
#define CLKDRV_DAC6_VDD       13
#define CLKDRV_DAC7_VSS       14
#define CLKDRV_DAC7_VDD       15
#define CLKDRV_DAC8_VSS       16
#define CLKDRV_DAC8_VDD       17
#define CLKDRV_DAC9_VSS       18
#define CLKDRV_DAC9_VDD       19
#define CLKDRV_DAC10_VSS      20
#define CLKDRV_DAC10_VDD      21
#define CLKDRV_DAC11_VSS      22
#define CLKDRV_DAC11_VDD      23
#define CLKDRV_DAC12_VSS      24
#define CLKDRV_DAC12_VDD      25
#define CLKDRV_DAC13_VSS      26
#define CLKDRV_DAC13_VDD      27
#define CLKDRV_DAC14_VSS      28
#define CLKDRV_DAC14_VDD      29
#define CLKDRV_DAC15_VSS      30
#define CLKDRV_DAC15_VDD      31

/*
 *# max number of Simms / clock driver board 
 */
#define fcdlCLK_NUM_SIMMS     14

/*
 *# SIMM Types
 */
typedef enum {VACANT, BILEVEL, MULTILEVEL} fcdlCLK_TYPE;  


/*****************************************************************************
 *# Boards Configuration
 *# --------------------
 */

/* 
 *# fcdlBrdConfig
 */
typedef struct
{
    /* CLKDRV<X>, etc */
    vltINT32  boardId;

    /* it's realy only the clock drivers which have variable configurations
     * the simms are indexed by SIMM number 0-15
     */
    fcdlCLK_TYPE simm[fcdlCLK_NUM_SIMMS];

    /* this only applies to VIDEO boards, indicates that there is a preamp 
     * to be controlled via an i2c bus
     */
    vltINT32 hasPreamp;

    /* this only applies to VIDEO boards, indicates that there is a second 
     * preamp to be controlled via an i2c bus
     */
    vltINT32 hasPreamp2;

    /* 
     * Generic board revision number. Normaly 0
     */
    vltINT32 brdRev;
} fcdlBrdConfig;


/*
 *# fcdlSetConfigParams
 *  Configuration of the Detector Head
 *  Board configuration contains the SIMM definitions for each board
 */
typedef struct
{
    vltINT32           numBoards;
    fcdlBrdConfig      boardConfig[fcdlMAX_BOARDS];
} fcdlSetConfigParams;

/*
 *# fcdlBoardId
 *  Board indentifier should be one of the defines above
 */
typedef vltINT32       fcdlBoardId;

/*
 *# fcdlPeriphId
 *  Periph Id should be one of the defines above
 */
typedef vltINT32       fcdlPeriphId;

/*
 *# fcdlPeriphValue
 *  Some periphs will not use all 32 bits of the value
 */
typedef vltINT32       fcdlPeriphValue;



/*****************************************************************************
 *# Voltages
 *# --------
 */

/*
 *# Floating point voltage scale
 *  voltages are passed as 0.001V when used as integers
 */
#define fcdlFIXED_TO_FLOAT_VOLT(fixed_volt)  (((double)fixed_volt) / 1000.0)
#define fcdlFLOAT_TO_FIXED_VOLT(float_volt)  ((int)(float_volt * 1000.0))

/*
 *# fcdlVoltTableEntry
 */
typedef struct
{
    fcdlBoardId      boardId;
    fcdlPeriphId     periphId;
    fcdlPeriphValue  tolerance;
    fcdlPeriphValue  value;
                     /* More parameters probably required */
    } fcdlVoltTableEntry;

/*
 *# fcdlSetVoltTableParams
 *  Parameters to the SET_VOLTAGE_TABLE command.
 *  A number of different voltage tables may be defined, the 
 *  DEFAULT_VOLTAGE_TABLE is the one which is used to initialise the system. 
 *  The others are used for different modes of operation (READ, WIPE, 
 *  INTEGRATE etc).
 */
#define fcdlDEFAULT_VOLTAGE_TABLE 0
#define fcdlMAX_VOLTAGE_TABLE_ID  100
typedef struct
{
    vltINT32             voltTableId; /* id of the table being defined  */
    vltINT32             numVolts;    /* number of entries in the table */
    fcdlVoltTableEntry   volts[1];    /* indexable up to numVolts       */
} fcdlSetVoltTableParams;

/*
 *# fcdlVoltSetting
 *  One entry in the SetVoltageParams
 */
typedef struct
{
    fcdlBoardId	        boardId;
    fcdlPeriphId	periphId;
    fcdlPeriphValue     value;
} fcdlVoltSetting;

/*
 *# fcdlSetVoltageParams
 *  Parameter to the SET_VOLTAGE command used to set a number of
 *  peripheral voltages
 */
typedef struct
{
    vltINT32	      numVolts;
    fcdlVoltSetting   volts[1];
					/* indexable up to NumVolts */
} fcdlSetVoltageParams;

/*
 *# Structure which uniquely identifies a voltage
 */
typedef struct
{
    fcdlBoardId	   boardId;
    fcdlPeriphId   periphId;
} fcdlVoltId;

/*
 *# fcdlGetVoltageParams
 *#
 *# Parameter to the GET_VOLTAGE command used to get a number of
 *# peripheral voltages
 */
typedef struct
{
    vltINT32	numVolts;
    fcdlVoltId	volts[1];
					/* indexable up to NumVolts */
} fcdlGetVoltageParams;



/*****************************************************************************
 *# Patterns
 *# --------
 */

/*
 *# minimum length of a pattern
 *
 * Currently we have a restriction on the minimum length of a pattern to
 * allow the DSP code enough time to load the next one
 */
#define fcdlMIN_PATTERN_LENGTH 6

/*
 *# fcdlDefineBilevelPatternParams
 *  Parameter to the DEFINE_BILEVEL_PATTERN which loads the Bilevel simm on a 
 *  board at a particular address
 */
typedef struct
{
    vltINT32  boardId;  /* id of the board to which the pat. is sent        */
    vltINT32  address;  /* address at which the pattern is loaded           */
    vltINT32  numData;  /* number of data words                             */
    vltINT32  data[1];  /* patt. data, 1 16bit word in each 32bit data word */
} fcdlDefineBilevelPatternParams;

/*
 *# fcdlReadBilevelPatternParams
 *# fcdlReadBilevelPatternResult
 *
 * Parameter and result for the READ_BILEVEL_PATTERN which
 * reads the Bilevel simm on a  board at a particular address
 */
typedef struct
{
    vltINT32   boardId;   /* id of the board from which the pat. is read */
    vltINT32   address;   /* address from which the pattern is read      */
    vltINT32   numData;   /* number of data words                        */
} fcdlReadBilevelPatternParams;

typedef struct
{
    vltINT32   boardId;  /* id of the b. from which the patt. has been read  */
    vltINT32   address;  /* address from which the pattern has been loaded   */
    vltINT32   numData;  /* number of data words                             */
    vltINT32   data[1];  /* patt. data, 1 16bit word in each 32bit data word */
} fcdlReadBilevelPatternResult;

/*
 *# fcdlMAX_SIMMS
 *  max total number of SIMMS
 */
#define fcdlMAX_SIMMS 32

/*
 *# fcdlDefineMultilevelPatternParams
 *  Parameter to the DEFINE_MULTILEVEL_PATTERN which loads 
 *  the multilevel simms on a board at a particular address
 */
typedef struct
{
    vltINT32   boardId;       /* id of the board to which the patt. is sent */
    vltINT32   address;       /* address at which the pattern is loaded     */
    vltINT32   numSimms;      /* number of multilevel simms being loaded    */
    vltINT32   simms[fcdlMAX_SIMMS];  /* simm numbers (num. from 0)         */
    vltINT32   numData;           /* number of data words (per simm)        */
    vltINT32   data[1];       /* pattern data (numSimms*numData words long) */
} fcdlDefineMultilevelPatternParams;


/*****************************************************************************
 *# Microsequences
 *# --------------
 */

/*
 *# MSE_MAX_LOOP MSE_MAX_LENGTH
 *  Some microsequencer limitations because of the size
 *  of the bit fields above
 */
#define MSE_MAX_LOOP   0x3FF  /* MSE_SEQCMD.loop   */
#define MSE_MAX_LENGTH 0x3FF  /* MSE_SEQCMD.length */

/*
 *# fcdlMAX_USEQ_ID
 *  maximum indentifier for a USEQ 
 *  (this is so we can use a table in the DSP code)
 */
#define fcdlMAX_USEQ_ID 200

/*
 *# fcdlDefineUseqParams
 *  Parameter to the DEFINE_USEQ command.
 *  One microsequence addressing multiple micro sequencers. 
 *  The patterns which are to be executaed will be loaded in 
 *  advance at <address> using the DEFINE_<XXX>LEVEL_PATTERN command.
 */
typedef struct
{
    vltINT32	useqId;	    /* which is used in sequence definitions         */
    vltINT32	address;    /* address which should be sent to the microseq. */
    vltINT32    length;     /* the length of the patterns                    */
    vltINT32	numBoards;  /* number of boards to be addressed              */
    vltINT32    boards[1];  /* id's of the boards to be addressed            */
} fcdlDefineUseqParams;


/*****************************************************************************
 *# Sequences
 *# ---------
 */

/* Minimum wait time (in DSP tics) between the repetition of a sequence
 * to allow the ABORT of the loop
 */
#define fcdlMIN_TICKS_TO_WAIT 2100

/*
 *# fcdlUSeqStatement
 *  Invoke a Useq repCount times at clockSpeed
 */
typedef struct {
    vltINT32		useqId;
    vltINT32		repCount;
    vltINT32		clockSpeed;
} fcdlUSeqStatement;

/*
 *# fcdlWaitStatement
 *  Invoke a wait for waitTime micoseconds
 */
typedef struct {
    vltINT32		waitMicrosecs;
} fcdlWaitStatement;

/*
 *# fcdlRepeatStatement
 *  Repeat a statement list <numStatements-1> long repCount times. 
 *  The index of the associated END statement is 
 *  <currentIndex> + <numStatements>
 */
typedef struct {
    vltINT32		repCount;
    vltINT32		numStatements;
} fcdlRepeatStatement;

/*
 *# fcdlEndRepeatStatement
 *  Closure for a repeat statement. The associated REPEAT may be found at
 *  <currentIndex>-<numStatements>
 */
typedef struct {
    vltINT32		numStatements;
} fcdlEndRepeatStatement;

/*
 *# fcdSTATEMENT_TYPE
 *  Enumeration for the different types of statement which are 
 *  possible in a sequence
 */
typedef enum {
	USEQ_STATEMENT,
	WAIT_STATEMENT,
	REPEAT_STATEMENT,
	END_REPEAT_STATEMENT
} fcdlSTATEMENT_TYPE;

/*
 *# define fcdlMAX_SEQ_NEST
 *  maximum number of nested REPEAT_STATEMENTs
 */
#define fcdlMAX_SEQ_NEST 20

/*
 *# fcdSequenceStatement
 *  A union to hold on of the various statement types listed above. The 
 *  descrimentant tells you which one it is.
 */
typedef struct
{
    fcdlSTATEMENT_TYPE	statementType;	/* statement discriminant  */
    union {
        fcdlUSeqStatement       useq;         /* if USEQ_STATEMENT       */
        fcdlWaitStatement       wait;         /* if WAIT_STATEMENT       */
        fcdlRepeatStatement     repeat;       /* if REPEAT_STATEMENT     */
        fcdlEndRepeatStatement  endRepeat;    /* if END_REPEAT_STATEMENT */
	} contents;
} fcdlSequenceStatement;

/*
 *# define fcdlMAX_SEQ_ID
 *  maximum indentifier for a SEQ 
 *  (this is so we can use a table in the DSP code)
 */
#define fcdlMAX_SEQ_ID 200

/*
 *# fcdlDefineSeqParams
 *  Parameter to the DEFINE_SEQ command. Defines the list of sequence 
 *  statements making up the sequence
 */
typedef struct 
{
    vltINT32                sequenceId;     /* sequence id                 */
    vltINT32                numStatements;  /* number of statmnts in array */
    fcdlSequenceStatement   statements[1];  /* array indexed up to 
                                               numStatements               */
} fcdlDefineSeqParams;

/*
 *# fcdlExecSeqParams
 *  Parameters to the EXEC_SEQ command
 */
typedef struct
{
    vltINT32	seqId;               /* sequence to be executed on the C40 */
    vltINT32	voltTableId;	     /* id of the voltage table to be used */
    vltINT32	numIterations;	     /* 0 means infinite	           */
    vltINT32	interIterationTime;  /* In microseconds                    */
} fcdlExecSeqParams;

/*
 *# fcdlExecNSeqsParams
 *  Parameters to the EXEC_N_SEQS command
 */
typedef struct
{
    vltINT32           numIterations; /* how many times the sequence set  *
                                       * has to be executed               *
                                       * (0 means infinite)               */
    vltINT32           interIterationTime;   /* In microseconds           */
    vltINT32           numSequences;  /* number of sequences in array     */
    fcdlExecSeqParams  sequences[1];  /* array indexed up to numSeq       */
} fcdlExecNSeqsParams;


/*****************************************************************************
 *# Commands
 *# --------
 */

/*
 *# fcdlPeekParams fcdlPeekReply
 *  Command and reply formats for the PEEK command allowing a low level read
 *  access to the C40 peripheral addresses.
 */
typedef struct
{
    vltINT32                   address;
} fcdlPeekParams;

typedef struct
{
    vltINT32                   address;
    vltINT32                   value;
} fcdlPeekReply;

/*
 *# fcdlTestId
 *
 * Defines the enumerations used for fcdlCMD_DO_TEST command
 */
typedef enum
    {
    fcdlTestVideo,
    fcdlTestVideoFromCommBoard,
    fcdlTestFiberWithDSP2a,
    fcdlTestFiberWithDSP2b,
    fcdlTestFiberWithDSP2c,
    fcdlTestAO1,
    fcdlTestAO2,
    fcdlTestAO3,
    fcdlTestAO4,
    fcdlTestAO5,
    fcdlTestAO6
    } fcdlTestId;

/*
 *# fcdlDoTestParams
 *  Paramaters to the arbitrary test function. This should identify a test
 *  function which has been linked with the C40 code.
 */
typedef struct
{
    fcdlTestId                 testIdent;
    vltINT32                   params[16];
    vltINT32                   params2[256];
} fcdlDoTestParams;


/*
 *# fcdlOpenShutterParams
 *  Paramaters to the OpenShutter function.
 */
typedef struct
{
   vltINT32                    totYetDefined;
} fcdlOpenShutterParams;


/*
 *# fcdlPokeParams
 *  Command and reply formats for the POKE command allowing a low level read
 *  access to the C40 peripheral addresses.
 */
typedef struct
{
    vltINT32                   address;
    vltINT32                   value;
} fcdlPokeParams;

/*
 *# fcdlMAX_PARAM_LEN 
 *  Maximum size of the parameter buffer in words
 */
#define fcdlMAX_PARAM_LEN 3000

/*
 *# fcdlCommandId
 *  Defines the enumerations used for Command field of a fcdlMsgHdr
 */
typedef enum
    {
	/* PING
	 *
	 * NOOP operation used to test the presence of the c40
	 *
	 * Destination:		both Exp. and Periph. processes
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_PING = 1,

	/* SET_CONFIG
	 *
	 * Download the Configuration structure to the C40
	 * describing the H/W configuration currently loaded
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlSetConfigParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_SET_CONFIG,
	
	/* GET_CONFIG
	 *
	 * Upload the Configuration structure from the C40
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	type fcdlConfig
	 */
    fcdlCMD_GET_CONFIG,

	/* SET_VOLTAGE_TABLE
	 *
	 * Download a voltage table to the c40, This table may be one of the
	 * tables to be used during a sequence execution or the 
	 * DEFAULT_VOLTAGE_TABLE to be used for initialisation.
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlSetVoltTableParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_SET_VOLTAGE_TABLE,
	
	/* GET_VOLTAGE_TABLE
	 *
	 * Upload the Voltage table from the c40
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	voltTableId
	 *
	 * Reply Format:	type fcdlSetVoltTableParams
	 */
    fcdlCMD_GET_VOLTAGE_TABLE,

         /*
          * INIT
          *
	  * Perform initialisation using the configuration 
	  * already downloaded
	  *
	  * Destination:	Peripheral process
	  *
	  * Message Format:	Empty
	  *
	  * Reply Format:	Empty
	  *
	  */
    fcdlCMD_INIT,

         /*
          * SELFTEST
          *
	  * Perform complete selftest of C40 H/W using the voltages
	  * and configuration already downloaded
	  *
	  * Destination:	Peripheral process
	  *
	  * Message Format:	Empty
	  *
	  * Reply Format:	Empty
	  *
	  */
    fcdlCMD_SELFTEST,


	/* ONLINE
	 *
	 * Set the C40 S/W into ONLINE mode
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_ONLINE,

	/* STANDBY
	 *
	 * Set the C40 S/W into STANDBY mode
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_STANDBY,

	/* DEFINE_BILEVEL_PATTERN
	 *
	 * Define the bilevel SIMM contents for a pattern to be loaded onto 
	 * a board at a particular address
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlDefineBilevelPatternParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_DEFINE_BILEVEL_PATTERN,

	/* READ_BILEVEL_PATTERN
	 *
	 * Read the bilevel SIMM contents for a pattern from
	 * a board at a particular address
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlReadBilevelPatternParams
	 *
	 * Reply Format:	type fcdlReadBilevelPatternResult
	 */
    fcdlCMD_READ_BILEVEL_PATTERN,

	/* DEFINE_MULTILEVEL_PATTERN
	 *
	 * Define the multilevel SIMM contents for a pattern to be loaded onto 
	 * a board at a particular address
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlDefineMultilevelPatternParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_DEFINE_MULTILEVEL_PATTERN,

	/* DEFINE_USEQ
	 *
	 * Define a micro sequence which should be performed using a number
	 * of sequencers
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlDefineUSeqParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_DEFINE_USEQ,

	/* DEFINE_SEQ
	 *
	 * Define a sequence which comprises a number of patterns and
	 * sequences. The command is acknowledged immediately
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	type fcdlDefineSeqParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_DEFINE_SEQ,

	/* SET_VOLTAGE
	 *
	 * Set a  set of C40 periferal voltages
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	fcdlSetVoltageParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_SET_VOLTAGE,

	/* GET_VOLTAGE
	 *
	 * Get a set of C40 periferal voltages
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	fcdlGetVoltageParams
	 *
	 * Reply Format:	fcdlSetVoltageParams
	 */
    fcdlCMD_GET_VOLTAGE,

	/* CHECK__VOLTAGE
	 *
	 * Check the set of the C40 periferal voltages
	 *
	 * Destination:		Periph. process
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_CHECK_VOLTAGES,

	/* EXEC_SEQ
	 *
	 * Perform a sequence on the C40 after setting the voltages to the 
	 * values defined in the parameters. The sequence can be executed 
	 * once or many times with an optional interval between executions.
	 *
	 * Only one sequence can be executing at the same time.
	 *
	 * Destination:		Peripheral process
	 *
	 * Message Format:	fcdExecSeqParams
	 *
	 * Reply Format:	Empty
	 */

    fcdlCMD_EXEC_SEQ,

	/* STOP_SEQ
	 *
	 * Abort a sequence which has been started by EXEC_SEQ
	 *
	 * Destination:		Peripheral process
	 *
	 * Message Format:	Empty
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_STOP_SEQ,

         /*
          * OPEN, CLOSE_SHUTTER
          *
          * Shutter control commands
	  *
	  * Destination:	Peripheral process
	  *
	  * Message Format:	fcdlOpenShutterParams
	  *
	  * Reply Format:	Empty
          */
    fcdlCMD_OPEN_SHUTTER,
    fcdlCMD_CLOSE_SHUTTER,

          /*
           * PEEK
	   *
	   * Low level memory access on C40
	   *
	   * Destination:	Peripheral process
	   *
	   * Message Format:	fcdlPeekParams
	   *
	   * Reply Format:	fcdlPeekReply
	   *
	   */
    fcdlCMD_PEEK,

          /*
           * POKE
	   *
	   * Low level memory access on C40
	   *
	   * Destination:	Peripheral process
	   *
	   * Message Format:	fcdlPokeParams
	   *
	   * Reply Format:	Empty
	   *
	   */
    fcdlCMD_POKE,

          /*
           * DO_TEST
	   *
	   * Perform The test function identified in the parameters. This
	   * test function must be linked into the C40 code.
	   *
	   * Destination:	Peripheral process
	   *
	   * Message Format:	fcdlDoTestParams
	   *
	   * Reply Format:	Empty
	   *
	   */
    fcdlCMD_DO_TEST,

         /*
          * OFF
          *
	  * Set the C40 S/W into OFF mode
	  *
	  * Destination:	Peripheral process
	  *
	  * Message Format:	Empty
	  *
	  * Reply Format:	Empty
	  *
	  */
    fcdlCMD_OFF,

         /*
          * CHANGE_LOG_LEVEL
          *
	  * Change the Logging level
	  *
	  * Destination:	Peripheral process
	  *
	  * Message Format:	vltINT32
	  *
	  * Reply Format:	Empty
	  *
	  */
    fcdlCMD_CHANGE_LOG_LEVEL,

	/* EXEC_SEQ_AT_SIG
	 *
	 * Perform a sequence on the C40 after setting the voltages to the 
	 * values defined in the parameters.
         * The sequence has to be exeuted after a signal from the
         * P2 backplane (TIM module) has been received.
         * The sequence can be executed once or many times with
	 * an optional interval between executions.
	 *
	 * Only one sequence can be executing at the same time.
	 *
	 * Destination:		Peripheral process
	 *
	 * Message Format:	fcdExecSeqParams
	 *
	 * Reply Format:	Empty
	 */

    fcdlCMD_EXEC_SEQ_AT_SIG,

	/* EXEC_N_SEQS
	 *
	 * Perform a set of sequences on the C40 after setting the voltages
         * to the values defined in the parameters.
         * The set of sequences can be executed  once or many times
	 * with an optional interval between executions.
	 *
	 * Destination:		Peripheral process
	 *
	 * Message Format:	fcdExecNSeqsParams
	 *
	 * Reply Format:	Empty
	 */
    fcdlCMD_EXEC_N_SEQS,

        /*
	 * Dummy command to mark the last legal command value
         */
    fcdlCMD_LAST
    } fcdlCommandId;
    

/*****************************************************************************
 *# Messages
 *# --------
 */

/*
 *# fcdlMsgType
 *#
 *# Defines the enumarations used for COMMAND and REPLY to be put into
 *# the Type field of a fcdlMsgHdr
 */
typedef enum
    {
    fcdlCOMMAND		= 0,
			/* all other values are intermediate replies */
    fcdlLAST_REPLY		= 32767,
			/* for an error reply the body of the message	*/
			/* should contain one 32 bit value which is an	*/
			/* error code from fcdErrors.h			*/
    fcdlERROR_REPLY
    } fcdlMsgType;

/*
 *# fcdlMsgHdr
 *  Fixed portion of a message going accross the fcdl link.
 *  This message is immediately followed by "Length" words of
 *  Parameter data
 */
typedef struct
    {
    vltINT32	Length;		/* total length of message in words	*/
    vltINT32	Dest;		/* destination channel on the fcdl
				   or fd on SPARC depending on dir      */
    vltINT32	Src;		/* source fd on SPARC or channel on c40
				   depending on direction               */
    vltINT32	Command;	/* cmd ID one of c40CommandId		*/
    vltINT32	Type;		/* fcdlCOMMAND, fcdlREPLY, 		*/
				/* fcdlLAST_REPLY			*/
    /*
     * Data follows
     */
    } fcdlMsgHdr;
/*
 *# fcdlMSG_HDR_LEN
 *  Returns the number of bytes on the SPARC and the number of 32bit words
 *  on the fcdl of the message header structure.
 */
#define fcdlMSG_HDR_LEN (sizeof(fcdlMsgHdr))

/*
 *# fcdlMSG_DATA_LEN
 *  Returns the number of bytes on the SPARC and the number of 
 *  32bit words on the c40 of the data section of the message given 
 *  the value of the Length field of the message. This works both on 
 *  the c40 and the SPARC
 */
#define fcdlMSG_DATA_LEN(Len) ((Len)*sizeof(vltINT32) - sizeof(fcdlMsgHdr))

/*
 *# fcdlMSG_TOTAL_LEN
 *  Gives the number which must be put into the "Length" field of the 
 *  message header given that there are ParamLen words of parameter
 *  data.
 */
#define fcdlMSG_TOTAL_LEN(ParamLen) \
                      ((ParamLen) + sizeof(fcdlMsgHdr)/sizeof(vltINT32))


#ifdef __cplusplus
}
#endif


#endif /* !FCDLINTF_H */
