#ifndef FCDP_COMMAND_H
#define FCDP_COMMAND_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdpCommand.h,v 1.59 2005/10/24 11:39:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* hpieper   14/07/97  created
* ccumani   23/04/98  - pulpo shutter identifiers added
*                     - new pulpo error replies added
* ccumani   18/09/01  - FCDP_CMD_MAX_ERR set to 26
* ccumani   30/08/04  - old pulpo1 commands added
*                     - new pulpo2 commands added
* ccumani   03/12/04  - new pulpo2 SelfRecovery command added
* ccumani   08/04/05  - new pulpo2 EnableSensor command added
* ccumani   29/06/05  - new pulpo2 VacuumIdentifier command added
*                     - new pulpo2 LogtolocalFile command added
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */


/*-----------------------------------------------------------------------
 * pulpo command tags
 *----------------------------------------------------------------------*/

/* WARNING: these definitions must be alligned with the command
 *          fcdpulpoCmd[] table in fcdp/src/fcdpCommand.c
 */
#define FCDP_START_EXPOSURE         0
#define FCDP_STOP_EXPOSURE          1
#define FCDP_SET_ALARM_MODE         2
#define FCDP_GET_ALARM_MODE         3
#define FCDP_SET_AUTO_TUNER_MODE    4
#define FCDP_GET_AUTO_TUNER_MODE    5
#define FCDP_GET_SENSOR_ADUSTDDEV   6
#define FCDP_GET_SENSOR_ADU         7
#define FCDP_GET_SENSOR_STDDEV      8
#define FCDP_SET_BIAS_POWER         9
#define FCDP_GET_BIAS_POWER         10
#define FCDP_GET_CLOSE_DELAY        11
#define FCDP_CTRL_MODE              12
#define FCDP_DEBUG_MODE             13
#define FCDP_DISABLE_LEDS           14
#define FCDP_SET_DEWAR_MODEL        15
#define FCDP_GET_DEWAR_MODEL        16
#define FCDP_ECHO_MODE              17
#define FCDP_SET_HEATER_PID_MODE    18
#define FCDP_GET_HEATER_PID_MODE    19
#define FCDP_SET_HEATER_MODE        20
#define FCDP_GET_HEATER_MODE        21
#define FCDP_SET_HEATER_RESIST      22
#define FCDP_GET_HEATER_RESIST      23
#define FCDP_SET_HEATER_SENSOR      24
#define FCDP_GET_HEATER_SENSOR      25
#define FCDP_HW_TEST                26
#define FCDP_SAFE_INIT              27
#define FCDP_SET_PID_KD             28
#define FCDP_GET_PID_KD             29
#define FCDP_SET_PID_KI             30
#define FCDP_GET_PID_KI             31
#define FCDP_SET_PID_KP             32
#define FCDP_GET_PID_KP             33
#define FCDP_LOG_T                  34
#define FCDP_LOG_P                  35
#define FCDP_LOG_V                  36
#define FCDP_LOG_BEGIN              37
#define FCDP_LOG_CHECK              38
#define FCDP_LOG_DUMP               39
#define FCDP_LOG_STOP               40
#define FCDP_GET_OPEN_DELAY         41
#define FCDP_OPEN_SHUTTER           42
#define FCDP_PAUSE_EXPOSURE         43
#define FCDP_SET_HEATER_POWER       44
#define FCDP_GET_HEATER_POWER       45
#define FCDP_RESET_SHUTTER          46
#define FCDP_GET_STATUS             47
#define FCDP_CLOSE_SHUTTER          48
#define FCDP_SET_CLOCK_DATE         49
#define FCDP_GET_CLOCK_DATE         50
#define FCDP_GET_SENSOR_VAL         51
#define FCDP_SET_SHUTTER_TYPE       52
#define FCDP_GET_SHUTTER_TYPE       53
#define FCDP_SET_SHUTTER_LOGIC      54
#define FCDP_GET_SHUTTER_LOGIC      55
#define FCDP_SET_SHUTTER_MODE       56
#define FCDP_GET_SHUTTER_MODE       57
#define FCDP_SET_REF_TEMP           58
#define FCDP_GET_REF_TEMP           59
#define FCDP_SET_CLOCK_TIME         60
#define FCDP_GET_CLOCK_TIME         61
#define FCDP_TERMINAL_MODE          62
#define FCDP_SET_TEMP_SLOPE         63
#define FCDP_GET_TEMP_SLOPE         64
#define FCDP_SET_VACUUM_POWER       65
#define FCDP_GET_VACUUM_POWER       66
#define FCDP_GET_SW_VERSION         67
#define FCDP_SET_EXP_TIME           68
#define FCDP_GET_EXP_TIME           69
#define FCDP_INCR_EXP_TIME          70
#define FCDP_DECR_EXP_TIME          71

/* CCU 30/08/04 */
#define FCDP_GET_ALARMS             72
#define FCDP_SET_ENABLE_MUX         73
#define FCDP_GET_ENABLE_MUX         74
#define FCDP_SET_FILTER_VALUE       75
#define FCDP_GET_FILTER_VALUE       76
#define FCDP_SET_AL_LOW_LIMIT       77
#define FCDP_GET_AL_LOW_LIMIT       78
#define FCDP_LOG_RESTART            79
#define FCDP_GET_RESET_BITS         80
#define FCDP_RESET_HEATERS          81
#define FCDP_RESET_OVERCURRENT      82
#define FCDP_RESET_TEMP_SENSORS     83
#define FCDP_RESET_VAC_SENSORS      84
#define FCDP_SHOW_TRIGG_ALARMS      85
#define FCDP_SET_GLOBAL_T_ALARM     86
#define FCDP_GET_GLOBAL_T_ALARM     87
#define FCDP_SET_DAC_TRIP_CURR      88
#define FCDP_GET_DAC_TRIP_CURR      89
#define FCDP_SET_TRIP_POINT         90
#define FCDP_GET_TRIP_POINT         91
#define FCDP_SET_VAC_ALARM_LIM      92
#define FCDP_GET_VAC_ALARM_LIM      93
#define FCDP_EXPOSURE_DELAY         94

/* CCU 03/12/04 */
#define FCDP_SET_SELFRECOVERY       95
#define FCDP_GET_SELFRECOVERY       96
#define FCDP_SET_SELFREC_SENSOR     97
#define FCDP_GET_SELFREC_SENSOR     98
#define FCDP_SET_SELFREC_VALUE      99
#define FCDP_GET_SELFREC_VALUE     100

/* CCU 08/04/05 */
#define FCDP_SET_ENABLE_SENSOR     101
#define FCDP_GET_ENABLE_SENSOR     102

/* CCU 29/06/05 */
#define FCDP_SET_VACUUM_TYPE       103
#define FCDP_GET_VACUUM_TYPE       104
#define FCDP_SET_LOCAL_LOG         105
#define FCDP_GET_LOCAL_LOG         106

#define FCDP_NOF_CMD               107 /* total number of supported CMDs */

/*-----------------------------------------------------------------------
 * user command tags
 *----------------------------------------------------------------------*/

#define FCDPU_GET_VAL                0
#define FCDPU_GET_INIT_VAL           1
#define FCDPU_GET_VAL_BASE           2
#define FCDPU_GET_VAL_RANGE          3
#define FCDPU_SET_VAL                4
#define FCDPU_SAVE_CFG               5
#define FCDPU_SAVE_CFG_FILE          6
#define FCDPU_LOAD_CFG               7
#define FCDPU_LOAD_CFG_FILE          8        
#define FCDPU_GET_UNIT_TABLE         9
#define FCDPU_NOF_CMD                10

/*-----------------------------------------------------------------------
 * pulpo shutter identifiers
 *----------------------------------------------------------------------*/

#define FCDP_SHT_NO_SHUTTER_BOARD       0
#define FCDP_SHT_BOARD1_NO_STATUS       1
#define FCDP_SHT_IRIS_NO_STATUS         2
#define FCDP_SHT_IRIS_OPEN_NOCLOSE      3
#define FCDP_SHT_IRIS_OPEN_CLOSE        4
#define FCDP_SHT_IRIS_CLOSE_NOPEN       5
#define FCDP_SHT_TWOBLADES_OPEN_NOCLOSE 6
#define FCDP_SHT_SESO                   7
#define FCDP_SHT_EMMI                   8

/*
 * NOTES form Nicolas Haddad:
 *
 * For the shutter routines to work correctly the Shutter Identifier
 * has to be properly set ( serial command SI )
 * 
 * - BOARD1_NO_STATUS is handy for light projectors that need an opto
 *   isolated output control and were you don't have any status
 *   available
 * 
 * - IRIS_NO_STATUS is meant for IRIS type of shutters without status
 *   but needs a shutter module driver inside PULPO (Shutter Board 0)
 *   which is different from the one mounted by default.
 * 
 * - The other IRIS_ derivatives are easy to understand ...
 * 
 * - TWOBLADES_OPEN_NOCLOSE is used for SUSI2
 * 
 * - SESO is for any instrument using SESO shutter drive, like the VLTTC
 * 
 * - EMMI is for the NTT EMMI shutter driver.
 */

/*-----------------------------------------------------------------------
 * pulpo error replies
 *----------------------------------------------------------------------*/

#define FCDP_CMD_MAX_ERR            26

#include "fcdpTTY.h"

#ifdef  __cplusplus
extern "C" {
#endif
    
/*-----------------------------------------------------------------------
 * pulpo command structure to compose command telegrams
 *----------------------------------------------------------------------*/
typedef ccsCOMPL_STAT (*PCALL)(void);

/*-----------------------------------------------------------------------
 * pulpo command + reply structure to compose command telegrams
 *----------------------------------------------------------------------*/

typedef struct fcdpPulpoCmdStruct
{
    PCALL cmdCall;
    vltINT16 cmdTag;
    vltINT8  *pulpoCmd;
    vltINT8  *fmt;
    vltINT16 argc;
} fcdpULPO_CMD;

typedef struct fcdpPulpoReplyStruct 
{
    vltINT16 cmdTag;
    vltINT8  **argv;
    vltINT8  *fmt;
    vltINT8  argc;
} fcdpULPO_REPLY;

/*-----------------------------------------------------------------------
 * user command + reply structure to compose user command telegrams
 *----------------------------------------------------------------------*/
typedef struct fcdpUserCmdStruct
{
    PCALL cmdCall;
    vltINT16 cmdTag;
    vltINT8  *userCmd;
    vltINT8  *fmt;
    vltINT16 argc;
} fcdpUSER_CMD;

typedef struct fcdpUserReplyStruct 
{
    vltINT16 cmdTag;
    vltINT8  **argv;
    vltINT8  *fmt;
    vltINT8  argc;
} fcdpUSER_REPLY;

extern fcdpULPO_REPLY fcdpulpoReply[];
extern fcdpULPO_CMD   fcdpulpoCmd[];
extern fcdpUSER_REPLY fcdpUserReply[];
extern fcdpUSER_CMD   fcdpUserCmd[];
/*-----------------------------------------------------------------------
 * pulpo basic operations MISC
 *----------------------------------------------------------------------*/

ccsCOMPL_STAT fcdpCreateCmd  ( vltINT32 cmdTag, 
			       vltINT8 *args,
			       vltINT8 *buf,	 
			       vltINT32 *cmdLen, 
			       ccsERROR *pErr);
ccsCOMPL_STAT fcdpIsUserCmd  ( vltINT8 *cmdStr, 
			       vltINT32 *cmdTag, 
			       ccsERROR *pErr);
ccsCOMPL_STAT fcdpIsPulpoCmd ( vltINT8 *cmdStr, 
			       vltINT32 *cmdTag, 
			       ccsERROR *pErr);
ccsCOMPL_STAT fcdpExecuteCmd ( vltINT32 pulpoID,
			       vltINT8 *cmdStr, 
			       vltINT32 cmdTag,
			       vltINT8 *reply,
			       vltINT32 delay,
			       ccsERROR *pErr);
ccsCOMPL_STAT fcdpExecuteUserCmd ( 
				  vltINT32 pulpoID,
				  vltINT8 *cmdStr,
				  vltINT32 cmdTag,
				  vltINT8 *reply,
				  ccsERROR *pErr
				  );
ccsCOMPL_STAT fcdpCheckError ( vltINT8 *reply, 
			       ccsERROR *pErr,
			       vltINT32 *pErrCode);		

#ifdef  __cplusplus
}
#endif


#endif /*!FCDP_COMMAND_H*/
