#ifndef FCDPCONFIG_H
#define FCDPCONFIG_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdpConfig.h,v 1.59 2005/10/24 11:39:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* hpieper  15/09/97  created
* rdonalds 20/01/98  Added definition of epxosure active bit
* abalestr 08/01/01  Added definition of shutter failure bit
* abalestr 13/01/01  Added definition of shutter open bit
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#include "fcdpTTY.h"

/************************************************************************
 * the PULPO configuration table
 * each CFG parameter can be adressed via a pulpoID and a peripheral ID
 *----------------------------------------------------------------------
 */

/* Pulpo ID's                                                                  */

#define FCDP_PULPO_1             1
#define FCDP_PULPO_2             2

#define FCDP_NOF_PULPOS          2
/* Peripheral Addresses                                                        */
/*                     in case there are two functions, one to read, and one   */
/*                     to write a value, then the Peripheral ID is set to the  */
/*                     GET function. Then in the respective Get or Set function*/
/*                     the other (GET or SET) is filtered respectively         */
#define FCDP_DEWAR_MODEL         (FCDP_GET_DEWAR_MODEL << 8) +1
#define FCDP_TEMP_SLOPE          (FCDP_GET_TEMP_SLOPE << 8) +1
#define FCDP_BIAS_POWER          (FCDP_GET_BIAS_POWER << 8) +1
#define FCDP_AUTO_TUNER_MODE     (FCDP_GET_AUTO_TUNER_MODE << 8) +1
#define FCDP_SOFTWARE_VERSION    (FCDP_GET_SW_VERSION << 8) +1
#define FCDP_VG_POWER            (FCDP_GET_VACUUM_POWER << 8) +1
#define FCDP_SHUTTER_LOGIC       (FCDP_GET_SHUTTER_LOGIC << 8) +1
#define FCDP_SHUTTER_MODE        (FCDP_GET_SHUTTER_MODE << 8) +1
#define FCDP_SHUTTER_TYPE        (FCDP_GET_SHUTTER_TYPE << 8) +1
#define FCDP_HEATER1_POWER       (FCDP_GET_HEATER_POWER << 8) +1
#define FCDP_HEATER1_RESISTANCE  (FCDP_GET_HEATER_RESIST << 8)+1
#define FCDP_HEATER1_SETPOINT    (FCDP_GET_REF_TEMP << 8)+1
#define FCDP_HEATER1_SENSOR      (FCDP_GET_HEATER_SENSOR << 8)+1
#define FCDP_HEATER1_KP          (FCDP_GET_PID_KP << 8)+1
#define FCDP_HEATER1_KI          (FCDP_GET_PID_KI << 8)+1
#define FCDP_HEATER1_KD          (FCDP_GET_PID_KD << 8)+1
#define FCDP_HEATER1_PID_MODE    (FCDP_GET_HEATER_PID_MODE << 8)+1 /* enabled/disabled */
#define FCDP_HEATER1_MODE        (FCDP_GET_HEATER_MODE << 8)+1 /* duty cycle           */
#define FCDP_HEATER2_POWER       (FCDP_GET_HEATER_POWER << 8) +2
#define FCDP_HEATER2_RESISTANCE  (FCDP_GET_HEATER_RESIST << 8)+2
#define FCDP_HEATER2_SETPOINT    (FCDP_GET_REF_TEMP << 8)+2
#define FCDP_HEATER2_SENSOR      (FCDP_GET_HEATER_SENSOR << 8)+2
#define FCDP_HEATER2_KP          (FCDP_GET_PID_KP << 8)+2
#define FCDP_HEATER2_KI          (FCDP_GET_PID_KI << 8)+2
#define FCDP_HEATER2_KD          (FCDP_GET_PID_KD << 8)+2
#define FCDP_HEATER2_PID_MODE    (FCDP_GET_HEATER_PID_MODE << 8)+2
#define FCDP_HEATER2_MODE        (FCDP_GET_HEATER_MODE << 8)+2
#define FCDP_HEATER3_POWER       (FCDP_GET_HEATER_POWER << 8) +3
#define FCDP_HEATER3_RESISTANCE  (FCDP_GET_HEATER_RESIST << 8)+3
#define FCDP_HEATER3_SETPOINT    (FCDP_GET_REF_TEMP << 8)+3
#define FCDP_HEATER3_SENSOR      (FCDP_GET_HEATER_SENSOR << 8)+3
#define FCDP_HEATER3_KP          (FCDP_GET_PID_KP << 8)+3
#define FCDP_HEATER3_KI          (FCDP_GET_PID_KI << 8)+3
#define FCDP_HEATER3_KD          (FCDP_GET_PID_KD << 8)+3
#define FCDP_HEATER3_PID_MODE    (FCDP_GET_HEATER_PID_MODE << 8)+3
#define FCDP_HEATER3_MODE        (FCDP_GET_HEATER_MODE << 8)+3
#define FCDP_STATUS_1            (FCDP_GET_STATUS << 8)+1
#define FCDP_STATUS_2            (FCDP_GET_STATUS << 8)+2
#define FCDP_STATUS_3            (FCDP_GET_STATUS << 8)+3
#define FCDP_STATUS_4            (FCDP_GET_STATUS << 8)+4
#define FCDP_STATUS_5            (FCDP_GET_STATUS << 8)+4
#define FCDP_NOF_PERIPHERALS     40     

/* bit in FCDP_STATUS_2 which indicates shutter open */
#define FCDP_SHUT_OPEN_BIT       1
/* bit in FCDP_STATUS_2 which indicates exposure active */
#define FCDP_EXP_ACTIVE_BIT      4
/* bit in FCDP_STATUS_2 which indicates shutter error */
#define FCDP_SHUT_FAILURE_BIT    128


#ifdef __cplusplus
extern "C" {
#endif

typedef vltINT32 fcdpPERIPHVAL; 

typedef struct fcdpPulpoLookup
{
    vltINT8 *pulpoName;
    vltINT32 pulpoID;
} fcdpPULPO_LOOKUP;

typedef struct fcdpPeriphLookup
{
    vltINT8 *periphName;
    vltINT32 periphID;
} fcdpPERIPH_LOOKUP;

typedef struct fcdpConfigStruct 
{
    vltINT32 pulpoID;
    vltINT32 periphID;
    fcdpPERIPHVAL base;
    fcdpPERIPHVAL highVal;
    fcdpPERIPHVAL lowVal;
    fcdpPERIPHVAL initVal;
    fcdpPERIPHVAL curVal;

} fcdpCONFIG;


ccsCOMPL_STAT fcdpCfgSetAll (ccsERROR *pErr);
fcdpCONFIG   *fcdpCfgGetAll ( 
			     vltINT32 pulpoID, 
			     vltINT32 *n,
			     ccsERROR *pErr
			     );
ccsCOMPL_STAT fcdpCfgGetVal ( 
			     vltINT32 pulpoID, 
			     vltINT32 periphID,
			     vltINT32 *val, 
			     ccsERROR *pErr
			     );
ccsCOMPL_STAT fcdpCfgGetInitVal(
			     vltINT32 pulpoID, 
			     vltINT32 periphID,
			     vltINT32 *val, 
			     ccsERROR *pErr
			     );
ccsCOMPL_STAT fcdpCfgGetValRange ( 
				  vltINT32 pulpoID, 
				  vltINT32 periphID,
				  vltINT32 *lval,
				  vltINT32 *hval,
				  ccsERROR *pErr
				  );
ccsCOMPL_STAT fcdpCfgGetValBase ( 
				  vltINT32 pulpoID, 
				  vltINT32 periphID,
				  vltINT32 *base,
				  ccsERROR *pErr
				  );
ccsCOMPL_STAT fcdpCfgSetVal ( 
			     vltINT32 pulpoID, 
			     vltINT32 periphID,
			     vltINT32 val, 
			     ccsERROR *pErr
			     );
fcdpCONFIG *fcdpFindPULPO_TABLE_DEF( 
				    vltINT32 pulpoID, 
				    vltINT32 periphID
				    );

vltINT32 fcdpPulpoNameToID (
			    vltINT8 *pName,
			    ccsERROR *pErr
			    );
vltINT8 *fcdpPulpoIDToName (
			    vltINT32 id,
			    ccsERROR *pErr
			    );
ccsCOMPL_STAT fcdpGetPulpoIDName( 
				 vltINT32 n,
				 vltINT32 *id,
				 vltINT8  *name
				 );
ccsCOMPL_STAT fcdpGetPeriphIDName( 
				  vltINT32 n,
				  vltINT32 *id,
				  vltINT8  *name
				  );
vltINT32 fcdpPeriphNameToID (
			     vltINT8 *pName,
			     ccsERROR *pErr
			     );
vltINT8 *fcdpPeriphIDToName (
			    vltINT32 id,
			    ccsERROR *pErr
			    );
ccsCOMPL_STAT fcdpSetPULPO_TABLE_DEF ( 
				      vltINT32 pulpoID,
				      vltINT32 periphID,
				      fcdpPERIPHVAL base,
				      fcdpPERIPHVAL lowVal,
				      fcdpPERIPHVAL highVal,
				      fcdpPERIPHVAL initVal,
				      ccsERROR *pErr
				      );
ccsCOMPL_STAT fcdpCfgDelete ( ccsERROR *pErr);
fcdpCONFIG *fcdpCfgRead     ( vltINT8 *cfgFilename, ccsERROR *pErr);
ccsCOMPL_STAT fcdpCfgReadStd( ccsERROR *pErr);
ccsCOMPL_STAT fcdpCfgWrite  ( vltINT8 *cfgFilename, ccsERROR *pErr);
ccsCOMPL_STAT fcdpCfgWriteStd( ccsERROR *pErr);
ccsCOMPL_STAT fcdpCfgPrint( ccsERROR *pErr);

#ifdef __cplusplus
}
#endif

#endif /*!FCDPCONFIG_H*/
