#ifndef fcdpTTY_H
#define fcdpTTY_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdpTTY.h,v 1.59 2005/10/24 11:39:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* hpieper  26/05/97  created
* abalestr 17/01/00  For FEB2000: 
*		     - set 3rd parameter of fcdpGetPacket from int16 to uint32
* ccumani  29/09/05  ttyDELAY and ttyLONGDELAY macros added
*/
/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifdef  __cplusplus
extern "C" {
    
#endif
    
#include "CCS.h"
#include "err.h"
#include "vltPort.h"
#include "semaph.h"    
#include <termios.h>  
#include <string.h> 
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/stat.h>
#if defined (SUN_COMP)
#include <sys/select.h>
#endif
#include <signal.h>
#include <ctype.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "fcdpErrors.h"

#define  FCDP_MAXOPEN_TTYS         2
#define  FCDP_CMDBUF_MAX_LEN      256  /* maximum protocol buffer  */

#define  FCDP_READ_TIMEOUT   1    /* 1 second     */
#define  FCDP_CHAR_TIMEOUT   1 /* 100 millisecs */
    
#define  FCDP_TERM_CHAR          '\r'
    
#define  FCDP_EXP_DARK_EXP       0
#define  FCDP_EXP_REAL_EXP       1
#define  FCDP_NEXP_BIAS_LED      2
#define  FCDP_EXP_BIAS_LED       3
#define  FCDP_EXP_NOF_MODES      3


#define ttyDELAY     40  /* at least 7, for startup */
#define ttyLONGDELAY 40


#define  fcdpLog(m) fcdpL1Log m;
    

#define fcdpTTYLog(m) 
    

    
/*-----------------------------------------------------------------------
 * tty descriptor
 *----------------------------------------------------------------------*/
    
typedef struct fcd_tty_info
    {
	struct termios  saveTIO;
	vltINT32    unitID;
	vltINT32    ttyID;     /* the ID of the TTY being used                */
	vltLOGICAL  ttyInit;   /* indicates whether TTY has been initialized  */
	vltINT8     ttyName[256];
	vltUINT32   telegramID;  
	vltINT32    fcdpSEM_KEY;
	vltINT32    fcdpSEM_ID;
    /* and already runs a communication                                   */  
} fcdpTTYINFO;


extern fcdpTTYINFO *fcdpTTY;
extern vltINT32   fcdpMAXOPEN_TTYS;
extern vltINT32   fcdpMAX_UNIT;
extern vltLOGICAL fcdpSIM_MODE;

/*-----------------------------------------------------------------------
 * logging functions
 *----------------------------------------------------------------------*/
void fcdpL2Log (  
		const char *format,
		...);

void fcdpL1Log ( 
	       const char *format, 
	       ...
	       );
 
/*-----------------------------------------------------------------------
 * operations on TTY level
 *----------------------------------------------------------------------*/

/*----------------------------------------------------------------------
 *
 * SERVER SIDE INIT Function to physically open close the device
 *
 *----------------------------------------------------------------------*/

ccsCOMPL_STAT fcdpOpenTTY     ( const vltINT32 ttyID, ccsERROR *pErr);
ccsCOMPL_STAT fcdpCloseTTY    ( const vltINT32 ttyID, ccsERROR *pErr);

/*----------------------------------------------------------------------
 *
 * CLIENT SIDE INIT Function to physically open close the device
 *
 *----------------------------------------------------------------------*/

ccsCOMPL_STAT fcdpUseTTY     ( const vltINT32 ttyID, ccsERROR *pErr);
ccsCOMPL_STAT fcdpReleaseTTY ( const vltINT32 ttyID, ccsERROR *pErr);

/*----------------------------------------------------------------------
 *
 * Generic Functions to read/write and get info on TTY
 *
 *----------------------------------------------------------------------*/

fcdpTTYINFO  *fcdpGetTTY      ( vltINT32 unitID);
vltINT8 *fcdpGetPacket       ( const vltINT32 ttyID, 
                                vltINT32 seconds, 
				vltUINT32 expLength, 
				ccsERROR *pErr);

ccsCOMPL_STAT fcdpulpoIsTTY   ( const char *ttyName, vltINT32 *pid);

ccsCOMPL_STAT fcdpGetCommID   ( const vltINT32 ttyID, vltUINT32 *telCnt); 

ccsCOMPL_STAT fcdpReadTTY     ( const vltINT32 ttyID, vltINT8 *rbuf, 
				vltINT32 *count, 
				ccsERROR *pErr);
ccsCOMPL_STAT fcdpReadTimeoutTTY ( const vltINT32 ttyID, 
				   vltINT32 seconds,
				   vltINT8 *rbuf, 
				   vltINT32 *count, 
				   ccsERROR *pErr);
ccsCOMPL_STAT fcdpWriteTTY    ( const vltINT32 ttyID, 
				vltINT8 *wbuf, 
				ccsERROR *pErr);
ccsCOMPL_STAT fcdpWriteTTYNCR ( const vltINT32 ttyID, 
				vltINT8 *wbuf, 
				ccsERROR *pErr);
vltINT8      *fcdpSafeString  ( vltINT8 *str); 
				
ccsCOMPL_STAT fcdpExpectPacket( const vltINT32 ttyID, 
				vltINT8 *str, 
				vltINT32 timeout, 
				ccsERROR *pErr);

#ifdef  __cplusplus
}
#endif


#endif /*!fcdpTTY_H*/
