#ifndef FCDTIM_H
#define FCDTIM_H

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdtim.h,v 1.6 2004/01/19 09:25:53 vltsccm Exp $" 
*
* fcdtim.h  - TIM Interface file for the FIERA.
*             It contains the declarations of the constants, data types,
*             variables and function available to the users.
*
* who        when      what
* ---------- --------  ----------------------------------------------
* ccumani    26/05/98  ported from tim.h
* ccumani    04/08/98  - added function prototype for fcdtimGetDeviceMode
*                        and fcdtimSetTime
* ccumani    05/08/98  C++ ifdef switches added
*/

#include "ccs.h"
#include "fcdErrors.h"      /* Include TIM error codes and literals  */


/*
 * Version number
 */
#define fcdtimVERSION	"@(#) fcdtim $Revision: 1.6 $"

/*
 * Device modes
 */
#define fcdtimDEV_LOCAL_MODE  0
#define fcdtimDEV_UTC_MODE    1

/*
 * Timers
 * NOTE: only Timer 0 and Timer 1 are accessible by the DSP
 *       Board through the VME P2 connectors:
 *          A7  : frequency output 1
 *          A8  : frequency output 0
 *          A13 : Timer 1 output
 *          A14 : Timer 0 output
 *       All other timers are rejected by fcdtimConfigTimer.c
 */
#define fcdtimTIMER0      0
#define fcdtimTIMER1      1
#define fcdtimMAX_TIMER   fcdtimTIMER1

/*
 * Timer software modes
 */
#define fcdtimMODE_ONCE_NOW             1
#define fcdtimMODE_ONCE_NEXT_100        2
#define fcdtimMODE_ONCE_SIGNAL          3
#define fcdtimMODE_PERIODIC_NOW         4
#define fcdtimMODE_PERIODIC_NEXT_100    5
#define fcdtimMODE_PERIODIC_SIGNAL      6
#define fcdtimMODE_UTC                  7
#define fcdtimMODE_LAST                 fcdtimMODE_UTC

/*
 * Timer clock rates
 */
#define fcdtimCLK_RATE_1Hz           0
#define fcdtimCLK_RATE_10Hz          1
#define fcdtimCLK_RATE_100Hz         2
#define fcdtimCLK_RATE_1KHz          3
#define fcdtimCLK_RATE_10KHz         4
#define fcdtimCLK_RATE_100KHz        5
#define fcdtimCLK_RATE_1MHz          6
#define fcdtimCLK_RATE_0Hz           7
#define fcdtimCLK_RATE_LAST          fcdtimCLK_RATE_0Hz

/*
 * Function Prototypes
 */

extern ccsCOMPL_STAT fcdtimOpen
    (
    void 
    );

extern ccsCOMPL_STAT fcdtimSetDeviceMode
    (
    int mode
    );

extern int fcdtimGetDeviceMode (
    void
    );

extern ccsCOMPL_STAT fcdtimGetTime
    (
    ccsTIMEVAL *time
    );

extern ccsCOMPL_STAT fcdtimSetTime
    (
    const ccsTIMEVAL *time
    );

extern ccsCOMPL_STAT fcdtimConfigTimer
    (
    int           number,
    int           mode,
    vltUINT16     count,
    int           rate,
    ccsTIMEVAL   *utc
    );

extern ccsCOMPL_STAT fcdtimDisableTimer
    (
    int number
    );

extern ccsCOMPL_STAT fcdtimReset
    (
    void
    );

extern ccsCOMPL_STAT fcdtimClose
    (
    void
    );

#ifdef __cplusplus
}
#endif

#endif /* !FCDTIM_H */
