#ifndef fndABST_ARRAY_H
#define fndABST_ARRAY_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fndABST_ARRAY.h,v 1.46 2005/04/08 07:41:45 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  04/01/00  Renamed destroy as destroyFlag to avoid shadowing
* gchiozzi  14/11/95  At line 89 added cast to unsigned
* G.Chiozzi 18/01/95  Created
* 
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef fndPort_H
  #include <fndPort.h>
#endif
#ifndef _LIMITS_INCLUDED
  #include <limits.h>
#endif
#ifndef fndOBJECT_H
  #include <fndOBJECT.h>
#endif
#ifndef fndCOLLECTION_H
  #include <fndCOLLECTION.h>
#endif

#define fndABST_ARRAY_CLASS      (classType)fndABST_ARRAY::CreateNew


class fndABST_ARRAY:  public fndCOLLECTION
{
  public:
    fndABST_ARRAY( int upper, int lower = 0, sizeType aDelta = 0 );
    fndABST_ARRAY();
    virtual ~fndABST_ARRAY();

    int             LowerBound() const { return lowerbound; }
    int             UpperBound() const { return upperbound; }
    sizeType        ArraySize() const;

    virtual fndCONTAINER_ITERATOR& InitIterator() const;

    virtual void            Add( fndOBJECT& ) = 0;
    virtual void            Destroy( const fndOBJECT& o ) { Detach( o, 1 ); }
    virtual void            Destroy( int pos ) { Detach( pos, 1 ); }
    virtual void            Detach( int pos, int deleteToo = 0 );

    virtual void            Detach( const fndOBJECT&obj, int deleteToo = 0 );
    virtual void            Empty( int destroyFlag = FALSE );
    
    virtual int             IsEqual( const fndOBJECT& ) const;

    virtual void            PrintContentsOn( ostream& ) const;

    fndOBJECT& operator []( int atIndex ) const { return ObjectAt( atIndex ); };

  protected:
    fndOBJECT     &ObjectAt( int i ) const
	{ return *theArray[ i - lowerbound ]; }
    fndOBJECT     *PtrAt( int i ) const
	{ return theArray[ ZeroBase(i) ]; }
    void            SetData( int, fndOBJECT * );

    void            InsertEntry( int );
    void            RemoveEntry( int );
    void            SqueezeEntry( int );
    int             Find( const fndOBJECT & );

    void            Reallocate( sizeType );

    sizeType        delta;
    int             lowerbound;
    int             upperbound;
    int             whereToAdd;

    fndOBJECT     **theArray;

    friend  class fndARRAY_ITERATOR;

  private:
    int ZeroBase( int loc ) const {
	if( loc >= lowerbound && loc <= upperbound )
	    return loc - lowerbound;
	else
	    return -1;
	};

    int BoundBase( unsigned loc ) const {
	if( loc == UINT_MAX || loc <= (unsigned)(upperbound - lowerbound) )
	    return loc == UINT_MAX ? INT_MAX : loc + lowerbound;
	else
	    return -1;
	};
    
    fndStdObjectDef(fndABST_ARRAY, fndCOLLECTION)
};


class fndARRAY_ITERATOR:  public fndCONTAINER_ITERATOR
{
  public:
    fndARRAY_ITERATOR( const fndABST_ARRAY& );
    virtual                ~fndARRAY_ITERATOR();

    virtual operator        int();
    virtual operator        fndOBJECT&();
    virtual fndOBJECT      &GetCurrent();

    virtual fndOBJECT&      operator ++(int);
    virtual fndOBJECT&      operator ++();
    virtual void            Restart();

  private:
          int                 currentIndex;
    const fndABST_ARRAY      &beingIterated;
};



/* Inline methods */

inline sizeType fndABST_ARRAY::ArraySize() const
{
        return sizeType( upperbound - lowerbound + 1 );
}



inline fndARRAY_ITERATOR::fndARRAY_ITERATOR( const fndABST_ARRAY& toIterate ):
     currentIndex( toIterate.lowerbound ),beingIterated( toIterate )
{
}

inline  fndOBJECT &fndARRAY_ITERATOR::GetCurrent()
{
    if ( currentIndex <= beingIterated.upperbound )
	return ( (fndOBJECT&)( beingIterated.ObjectAt( currentIndex ) ) );
    else // no more elements in the array.
	return NOOBJECT;
}

#endif

/* __o0o__ */
