#ifndef fndLIST_H
#define fndLIST_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fndLIST.h,v 1.46 2005/04/08 07:41:47 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2002-07-31 gcc 3.1: Replace <xxx.h> sys includes with <xxx>a
* G.Chiozzi 18/01/95  Created
* 
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef fndPort_H
  #include <fndPort.h>
#endif
#include <iostream>

#ifndef fndCOLLECTION_H
  #include <fndCOLLECTION.h>
#endif
#ifndef fndLIST_ELEMENT_H
  #include <fndLIST_ELEMENT.h>
#endif

#define fndLIST_CLASS      (classType)fndLIST::CreateNew

class fndLIST:public fndCOLLECTION {

  public:

    fndLIST();
    fndLIST( fndLIST &fromList);
    virtual ~fndLIST();

    virtual  void         Add( fndOBJECT& item);
    virtual  void         AddAtHead( fndOBJECT& item);
    virtual  void         AddAtTail( fndOBJECT& item);

    virtual  void         DestroyFromHead( const fndOBJECT& item);
    virtual  void         DestroyFromTail( const fndOBJECT& item);
    virtual  void         Detach( const fndOBJECT& item, int deleteToo = 0 );
    virtual  void         Detach( int   pos, int deleteToo = 0 );
    virtual  void         DetachFromHead( const fndOBJECT&, int deleteToo = 0 );
    virtual  void         DetachFromTail( const fndOBJECT&, int deleteToo = 0 );

    virtual void          Empty( int deleteToo = FALSE );


    virtual  fndCONTAINER_ITERATOR& InitIterator() const;
    virtual  fndCONTAINER_ITERATOR& InitReverseIterator() const;

    virtual  fndOBJECT  &PeekAtHead() const;
    virtual  fndOBJECT  &PeekAtTail() const;

  private:
    fndLIST_ELEMENT* head;
    fndLIST_ELEMENT* tail;
    friend class fndLIST_ITERATOR;

    fndStdObjectDef(fndLIST, fndCOLLECTION)

}; /* end class fndLIST */



class fndLIST_ITERATOR: public fndCONTAINER_ITERATOR {

  public:

    fndLIST_ITERATOR() {};
    fndLIST_ITERATOR( const fndLIST&, int atHead = 1 );

    virtual ~fndLIST_ITERATOR();

    virtual  fndOBJECT    &GetCurrent();
    virtual  operator int();
    virtual  operator fndOBJECT&();

    virtual fndOBJECT& operator ++(int);
    virtual fndOBJECT& operator --(int);
    virtual fndOBJECT& operator ++();
    virtual fndOBJECT& operator --();

    virtual void Restart();

  private:
    fndLIST_ELEMENT* currentElement;
    fndLIST_ELEMENT* startingElement;
};



/* Inline methods */

/* 
   Destroys an object from the head of a double list.  We remove
   the object from the list and call that object's destructor.
   The parameter:
      toDestroy
   is the object we are to search for and Destroy from the fndLIST.
   
   Warnings:
   No error condition is generated if the object which was specified
   isn't on the double list. 
*/
inline void fndLIST::DestroyFromHead( const fndOBJECT& toDestroy )
{
    DetachFromHead( toDestroy, 1 );
}


/*
   Destroys an object from the tail of a double list.  We remove
   the object from the list and call that object's destructor.
   The parameter:
       toDestroy
   is the object we are to search for and destroy from the fndLIST.
   
   Warnings:
   No error condition is generated if the object which was specified
   isn't on the double list. 
*/
inline void fndLIST::DestroyFromTail( const fndOBJECT& toDestroy )
{
    DetachFromTail( toDestroy, 1 );
}


/*      
   Constructor for a list iterator object.
   The parameters:
        toIterate
   is the double list we are to iterat;.
        atHead
   indicates whether we are to start at the head of the list.  If this
   is not 1, we will start at the tail. 
*/
inline fndLIST_ITERATOR::fndLIST_ITERATOR( const fndLIST& toIterate,
                                               int atHead )


{
    if ( atHead == 1 )
    {
        startingElement = currentElement = toIterate.head;
    }
    else
    {
        startingElement = currentElement = toIterate.tail;
    }
}


#endif

/* __o0o__ */
