#ifndef fndOBJECT_H
#define fndOBJECT_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fndOBJECT.h,v 1.46 2005/04/08 07:41:47 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2002-07-31 gcc 3.1: Added using namespace std
* gchiozzi 2002-07-31 gcc 1.3: Replace <xxx.h> sys includes with <xxx>
* gchiozzi  03/01/00  Copy constructor get a const object.
* gchiozzi  29/03/96  Relational operators now return bool instead of int
* G.Chiozzi 18/01/95  Created
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef fndPort_H
  #include <fndPort.h>
#endif

#include <iostream>
#ifndef _STDDEF_INCLUDED
  #include <stddef.h>
#endif
#ifndef _STDIO_INCLUDED
  #include <stdio.h>
#endif

using namespace std;

/***********/
/* Defines */
/***********/

class fndOBJECT;

typedef char *caddr_t;

typedef unsigned int hashValueType; /* Defines a returned hash value.        */
typedef unsigned int sizeType;      /* Defines the size of a storage area.   */
typedef int countType;              /* Defines a container for counting      */
                                    /* things in the class librery.          */
typedef void(*iterFuncType )( fndOBJECT&, void* );
                                    /* Defines a pointer to an iteration     */
                                    /* function.The iteration function takes */
                                    /* an fndOBJECT reference and a point   */
                                    /* to a list of parameters to the        */
                                    /* function. The parameter list pointer  */
                                    /* may be NULL.                          */
typedef int(*condFuncType )( const fndOBJECT&, void* );
                                    /* Defines a pointer to a function which */
                                    /* implements a conditional test and     */
                                    /* returns 0 if the condition was met,   */
                                    /* non-zero otherwise. The non-zero      */
                                    /* values are defined by the individual  */
                                    /* function.                             */
typedef char Boolean;

typedef fndOBJECT *(*creator )( );

typedef fndOBJECT* func_type();

enum ObjectInit { objectInit };

#define NOOBJECT        *(fndOBJECT::ZERO)

/* Defines the type of a class */
#define classType       caddr_t

#define fndOBJECT_CLASS          (classType)fndOBJECT::CreateNew

class  fndOBJECT {
    
    /*********************/
    /* Public section    */
    /*********************/
    
  public:

    static fndOBJECT *ZERO;
    
    fndOBJECT();
    fndOBJECT( const fndOBJECT& );
    
    virtual ~fndOBJECT();

    virtual void          ForEach( iterFuncType, void* ) const;
    virtual hashValueType HashValue() const;
    
    virtual int           IsSortable()const;
    virtual int           IsAssociation()const;

    virtual int           IsEqual( const fndOBJECT& )const;
    
    virtual const fndOBJECT &FirstThat( condFuncType, void* )const;
    virtual const fndOBJECT &LastThat( condFuncType, void* )const;
    
    void *operator new( size_t s );
    void  operator delete( void* );
    
    /*******************************************************/
    /* These methods MUST be overloaded for each sub-class */
    /*******************************************************/
    static   fndOBJECT *CreateNew();
    
    virtual classType   IsA() const;
    virtual int         IsSubclassOf(classType toTest) const;
    virtual char       *NameOf() const;
    
    /*******************************************************/
    /* These methods MUST be overloaded for each sub-class */
    /* only if they are used for the subclass itself       */
    /*******************************************************/
    virtual void        PrintOn( ostream& outputStream)const;
    
    /*********************/
    /* Protected section */
    /*********************/
    
  protected:
    
    friend inline ostream& operator <<( ostream&, const fndOBJECT& );

  private:

    fndOBJECT( ObjectInit flag);

}; /* end class fndOBJECT */


/* Macro user in fndOBJECT sublasses header files (at end of      */
/* class definition in order to automatically build standard       */
/* methods:  CreateNew(), IsA(), IfSubclassOf(), NameOf()          */
#define fndStdObjectDef(Class, ParentClass)                                 \
          public:                                                            \
            static   Class      *CreateNew();                                \
            virtual  classType   IsA() const                                 \
                      {return ((classType)(Class::CreateNew));};             \
            virtual  int         IsSubclassOf(classType toTest) const        \
                      {                                                      \
                       if( Class::IsA() == toTest ) return TRUE;             \
                       return (ParentClass::IsSubclassOf(toTest));           \
                      };                                                     \
            virtual  char       *NameOf() const { return #Class; };

/* Macro user in fndOBJECT sublasses source files in order        */
/* to build the standard form of CreateNew() method body           */
#define fndStdObjectSource(Class, ParentClass)                              \
            Class *Class::CreateNew() { return(new Class); }


/******************/
/* Inline methods */
/******************/

inline hashValueType fndOBJECT::HashValue() const 
{ 
    return (hashValueType)this;
}

inline int fndOBJECT::IsSubclassOf(classType toTest) const 
{
    if( fndOBJECT::IsA() == toTest ) return TRUE;
    return FALSE;
}

inline void fndOBJECT::PrintOn( ostream& outputStream)const 
{
    outputStream << NameOf();
}

inline ostream& operator <<( ostream& outputStream,
                               const fndOBJECT& anfndOBJECT )
{
  anfndOBJECT.PrintOn( outputStream );
  return outputStream;
}

bool operator ==( const fndOBJECT& test1, const fndOBJECT& test2 );

inline bool operator !=( const fndOBJECT& test1, const fndOBJECT& test2 )

  /* Determines whether the given object is not equal to this. We just
     reverse the condition returned from operator ==.

     Parameters: test1 - the first object we are testing;
                 test2 - the second object we are testing. */

{
  return( !(test1 == test2 ) );
}


#endif

/* __o0o__ */
