#ifndef fndSTRING_H
#define fndSTRING_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fndSTRING.h,v 1.46 2005/04/08 07:41:48 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  03/01/00  Replace str with a String
* gchiozzi  29/03/96  Relational operators now return bool instead of int
* gchiozzi  09/08/95  Added new methods
* gchiozzi  18/01/95  Created
* 
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef fndPort_H
  #include <fndPort.h>
#endif
#ifndef fndString_H
  #include <fndString.h>
#endif
#ifndef fndSORTABLE_H
  #include <fndSORTABLE.h>
#endif

#define fndSTRING_CLASS          (classType)fndSTRING::CreateNew

class fndSTRING:  public fndSORTABLE {

  public:
    fndSTRING( const char* = NULL );
    fndSTRING( const fndSTRING& );
    fndSTRING(char fillCh, unsigned int count);

    virtual ~fndSTRING();

    virtual hashValueType HashValue() const;
    virtual int           IsEqual( const fndOBJECT& ) const;
    virtual int           IsLessThan( const fndOBJECT& ) const;
    virtual void          PrintOn( ostream& ) const;

    // Number of allocated bytes (including '\0')
    sizeType      GetLen() const;
    
    fndSTRING& operator           =( const fndSTRING& );
    operator const char*() const;
    
    // concatenation methods
    friend fndSTRING operator + (const fndSTRING & str1, const fndSTRING & str2);
    void operator += (const  fndSTRING& str);

    // comparison methods
    bool operator <  (const fndSTRING & str);
    bool operator >  (const fndSTRING & str);
    bool operator <= (const fndSTRING & str);
    bool operator >= (const fndSTRING & str);
    bool operator == (const fndSTRING & str);
    bool operator != (const fndSTRING & str);

    int Compare(const fndSTRING & str, fndSTR_CMP_MODE caseChk= fndSTR_IGNORE);

    // substring search methods
    int Find(const fndSTRING & str, unsigned int *pos,
	     fndSTR_CMP_MODE caseChk = fndSTR_IGNORE);

    // substring deletion method
    void Delete(unsigned int pos, unsigned int count);

    // substring insertion methods
    void Insert(unsigned int pos, char ch);
    void Insert(unsigned int pos, const fndSTRING & str);

    // substring retrieval method
    fndSTRING SubStr(unsigned int start, unsigned int count);

    // character retrieval method
    char operator [] (unsigned int pos);

    // case-modification methods
    fndSTRING ToUpper();
    fndSTRING ToLower();

    fndStdObjectDef(fndSTRING,fndSORTABLE)

  private:
    sizeType  len;
    char     *theString;

};

/***********************************************************************/
/* Constructs a fndSTRING object from a given character string.       */
/*                                                                     */
/* We assign the string's length to len, then allocate space into      */
/* which we will store the string's characters.  You may construct     */
/* fndSTRING objects out of local character strings.                  */
/*                                                                     */
/* Parameters: aPtr - Pointer to the characters out of which we are to */
/*                    construct a fndSTRING object.                   */
/***********************************************************************/

inline fndSTRING::fndSTRING( const char* aPtr )
{
    /* if ( aPtr && *aPtr )                    */
    /* 02/11/93: the previous was the original */
    if ( aPtr )
     {
      len = (unsigned int)strlen( aPtr ) + 1;
      theString = new char[ len ];
      (void)strcpy( theString, aPtr );
     }
    else  /* make a null string fndSTRING object.*/
     {
      len = 0;
      theString = 0;
     }
}


/******************/
/* Inline methods */
/******************/
inline sizeType fndSTRING::GetLen() const  { return len; }
    
inline fndSTRING::operator const char*() const { return theString; }

// comparison methods
inline bool fndSTRING::operator <  (const fndSTRING & aString)
    {
    return (Compare(aString) == fndSTR_LESS);
    }

inline bool fndSTRING::operator >  (const fndSTRING & aString)
    {
    return (Compare(aString) == fndSTR_GREATER);
    }

inline bool fndSTRING::operator <= (const fndSTRING & aString)
    {
    return (Compare(aString) != fndSTR_GREATER);
    }

inline bool fndSTRING::operator >= (const fndSTRING & aString)
    {
    return (Compare(aString) != fndSTR_LESS);
    }

inline bool fndSTRING::operator == (const fndSTRING & aString)
    {
    return (Compare(aString) == fndSTR_EQUAL);
    }

inline bool fndSTRING::operator != (const fndSTRING & aString)
    {
    return (Compare(aString) != fndSTR_EQUAL);
    }

// character retrieval method
inline char fndSTRING::operator [] (unsigned int pos)
    {
    if (pos >= len)
        return '\x00';
    else
        return theString[pos];
    }

/********************/
/* Inline operators */
/********************/

inline bool operator ==( const fndSTRING& aString, const fndSTRING& str2 )
{
 return( (const fndOBJECT &)aString == (const fndOBJECT &)str2 );
}

inline bool operator ==( const fndSTRING& aString, const char *ch )
{
 return( strcmp( ch, aString )==0 ? TRUE : FALSE );
}

inline bool operator ==(  const char *ch , const fndSTRING& aString)
{
 return( strcmp( ch, aString )==0 ? TRUE : FALSE );
}

inline bool operator !=( const fndSTRING& aString, const fndSTRING& str2 )
{
 return( (const fndOBJECT &)aString != (const fndOBJECT &)str2 );
}

inline bool operator !=( const fndSTRING& aString, const char *ch )
{
 return( strcmp( ch, aString )!=0 ? TRUE : FALSE );
}

inline bool operator !=(  const char *ch , const fndSTRING& aString)
{
 return( strcmp( ch, aString )!=0 ? TRUE : FALSE );
}





#endif

/* __o0o__ */
