#ifndef fntpubDEFINES_H
#define fntpubDEFINES_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fntpubDefines.h,v 1.16 2004/08/12 10:20:13 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott      23/03/03  Added definition fntpubPERF_DATA_DEF_SRC_SPOINT
* jott      17/03/03  Added database addresses of simulation flags for all
*                     other sub-systems in addition to detector.
*                     Set fntpubFNT_DEF_DBPOINT to ":Appl_data:DLCS"
* jott      10/02/03  Added "Performance data public sources"
*                     Modified and partly added "Performance data public
*                     destinations"
* jott      17/01/03  adapted definition fntpubDETECTOR_SIM_FLAG_ADDR related
*                     to simulation of whole LCUs to the simulation of
*                     individual LCU applications
* jott      17/12/02  removed definition fntpubFINITO_ENVNAME
* jott      19/11/02  added named data items for FINITO public data
* jott      05/08/02  adapted to be used for first integration test in
*                     ISS without including the dlif branch 1.31.1 into the
*                     dlif main trunc.
* jott      29/05/02  created
*/

/************************************************************************
 * Header file containing general public definitions of FINITO
 *----------------------------------------------------------------------
 */
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * Module name
 */
#define fntpubMOD                   "fntpub"


/*
 * Default environment name for FINITO workstation environment
 */
#define fntpubFINITO_DEF_ENV        "wdline"

/*
 * Default database root point for FINITO workstation environment
 */
#define fntpubFNT_DEF_DBPOINT       ":Appl_data:DLCS"

/*
 * Default database root point for FINITO coordination process
 */
#define fntpubFNTCOORD_DEF_DBPOINT  ":Appl_data:DLCS:fnt:coord"

/*
 * Flag indicating whether a FINITO public data base branch shall be inserted
 * to FINITO WS SW database in order to be able to run the test program
 * fntpubTestPerformance within the DLCS environment.
 */
#define FNT_ADD_PUB_BRANCH          1


/*
 * Environment variable containing the name of the database root point for
 * FINITO. This is the point where the database class fntFINITO_WS is added
 * as point 'fnt'. If not set the default "@wdline:Appl_data:DLCS" is used
 */
#define fntpubFNT_DBPOINT           "FNT_BASE_POINT"


/*
 * Definitions for FITS file name length, absolute path length and subdirectory
 * containing the image file
 */
#define fntpubFITS_FILE_NAME_LEN      64
#define fntpubABSOLUTE_PATH_LEN       256    /* maximum: msgBODYMAXLEN  */
#define fntpubFITS_FILE_SUBDIR        "DETDATA"

/*
 * Database addresses of simulation flags for sub-systems
 */
#define fntpubTRANSLAT_SIM_FLAG_ADDR   ":fnt:coord:simulation:acuTst.simulation"
#define fntpubTIPTILT_SIM_FLAG_ADDR    ":fnt:coord:simulation:acuTtp.simulation"
#define fntpubDETECTOR_SIM_FLAG_ADDR   ":fnt:coord:simulation:aduDet.simulation"
#define fntpubMODULATOR_SIM_FLAG_ADDR  ":fnt:coord:simulation:fmuMod.simulation"
#define fntpubIRACE_SIM_FLAG_ADDR      ":fnt:coord:simulation:irace.simulation"


/*
 * Definitions for named data items
 */

/* Global status */
#define fntpubDATA_FNT_STATE                "dlifDATA_FNT_STATE"
#define fntpubDATA_FNT_SUBSTATE             "dlifDATA_FNT_SUBSTATE"
#define fntpubDATA_FNT_OP_MODE              "dlifDATA_FNT_OP_MODE"

/* FITS header */
#define fntpubDATA_FNT_TRK_MODE             "dlifDATA_FNT_TRK_MODE"
#define fntpubDATA_FNT_CH_SEL               "dlifDATA_FNT_CH_SEL"
#define fntpubDATA_FNT_TARG_HMAG            "dlifDATA_FNT_TARG_HMAG"
#define fntpubDATA_FNT_TRK_WLEN             "dlifDATA_FNT_TARG_WLEN"
#define fntpubDATA_FNT_VISIBILITY           "dlifDATA_FNT_VISIBILITY"
#define fntpubDATA_FNT_LADC_MODE            "dlifDATA_FNT_LADC_MODE"
#define fntpubDATA_FNT_TADC_MODE            "dlifDATA_FNT_TADC_MODE"


/*
 * Definitions for symbolic addresses of named data items
 */

/* Global status */
#define fntpubDATA_FNT_STATE_ADDR           "fnt.fntState"
#define fntpubDATA_FNT_SUBSTATE_ADDR        "fnt.fntSubstate"
#define fntpubDATA_FNT_OP_MODE_ADD          "fnt:coord.opMode"

/* FITS header */
#define fntpubDATA_FNT_TRK_MODE_ADDR        "fnt:coord:setup.trkMode"
#define fntpubDATA_FNT_CH_SEL_ADDR          "fnt:coord:setup.chSel"
#define fntpubDATA_FNT_TARG_HMAG_ADDR       "fnt:coord:setup.targHmag"
#define fntpubDATA_FNT_TRK_WLEN_ADDR        "fnt:coord:setup.trkWlength"
#define fntpubDATA_FNT_VISIBILITY_ADDR      "fnt:coord:setup.visibility"
#define fntpubDATA_FNT_LADC_MODE_ADDR       "fnt:adc:data.ladcMode"
#define fntpubDATA_FNT_TADC_MODE_ADDR       "fnt:adc:data.tadcMode"


/*
 * FITS performance data DESTINATIONS:
 * These attributes are used by the library function fntGetPerformanceData()
 * to store the read performance data after an ENDEX command to fntCoordination
 * process is finished with an OK reply.
 */
#define fntpubPERF_DATA_DEF_ENVNAME         "wvgvlti"
#define fntpubPERF_DATA_DEF_DB_POINT        ":Appl_data:VLTI:issprs:perfData"
#define fntpubPERF_DATA_DEF_SRC_SPOINT      ":Appl_data:DLCS:fnt:coord:perfData"
#define fntpubDATA_PERF_VALID_DEST          ".perfDataValid"
#define fntpubDATA_FNT_PHA_M_CH1_DEST       ".opdPhaseMeanCh1"
#define fntpubDATA_FNT_PHA_M_CH2_DEST       ".opdPhaseMeanCh2"
#define fntpubDATA_FNT_PHA_RMS_CH1_DEST     ".opdPhaseRmsCh1"
#define fntpubDATA_FNT_PHA_RMS_CH2_DEST     ".opdPhaseRmsCh2"
#define fntpubDATA_FNT_COH_M_CH1_DEST       ".opdCoherenceMeanCh1"
#define fntpubDATA_FNT_COH_M_CH2_DEST       ".opdCoherenceMeanCh2"
#define fntpubDATA_FNT_COH_RMS_CH1_DEST     ".opdCoherenceRmsCh1"
#define fntpubDATA_FNT_COH_RMS_CH2_DEST     ".opdCoherenceRmsCh2"
#define fntpubDATA_FNT_VIS_M_CH1_DEST       ".visibilityMeanCh1"
#define fntpubDATA_FNT_VIS_M_CH2_DEST       ".visibilityMeanCh2"
#define fntpubDATA_FNT_VIS_RMS_CH1_DEST     ".visibilityRmsCh1"
#define fntpubDATA_FNT_VIS_RMS_CH2_DEST     ".visibilityRmsCh2"


/*
 * Performance data public SOURCES:
 * These attributes are used by the fntCoordination process to store the
 * read performance data after an ENDEX command to the detector
 * process is finished with an OK reply.
 */

#define fntpubDATA_PERF_VALID_SRC           ".perfDataValid"
#define fntpubDATA_FNT_PHA_M_CH1_SRC        ".opdPhaseMeanCh1"
#define fntpubDATA_FNT_PHA_M_CH2_SRC        ".opdPhaseMeanCh2"
#define fntpubDATA_FNT_PHA_RMS_CH1_SRC      ".opdPhaseRmsCh1"
#define fntpubDATA_FNT_PHA_RMS_CH2_SRC      ".opdPhaseRmsCh2"
#define fntpubDATA_FNT_COH_M_CH1_SRC        ".opdCoherenceMeanCh1"
#define fntpubDATA_FNT_COH_M_CH2_SRC        ".opdCoherenceMeanCh2"
#define fntpubDATA_FNT_COH_RMS_CH1_SRC      ".opdCoherenceRmsCh1"
#define fntpubDATA_FNT_COH_RMS_CH2_SRC      ".opdCoherenceRmsCh2"
#define fntpubDATA_FNT_VIS_M_CH1_SRC        ".visibilityMeanCh1"
#define fntpubDATA_FNT_VIS_M_CH2_SRC        ".visibilityMeanCh2"
#define fntpubDATA_FNT_VIS_RMS_CH1_SRC      ".visibilityRmsCh1"
#define fntpubDATA_FNT_VIS_RMS_CH2_SRC      ".visibilityRmsCh2"


/*
 * Definitions for fringe tracking modes
 */
#define fntpubFR_TRK_MODE_UNKNOWN              0
#define fntpubFR_TRK_MODE_STANDARD             1
#define fntpubFR_TRK_MODE_SLOW                 2
#define fntpubFR_TRK_MODE_FAST                 3

#define fntpubFR_TRK_MODE_UNKNOWN_STR          "UNKNOWN"
#define fntpubFR_TRK_MODE_STANDARD_STR         "STD"
#define fntpubFR_TRK_MODE_SLOW_STR             "SLOW"
#define fntpubFR_TRK_MODE_FAST_STR             "FAST"

/*
 * Definitions for channel selections
 * !! ATTENTION:
 * !! Do not arbitrarily change the numbers because they are used in
 * !! a bitmask.
 */
#define fntpubSELECT_UNKNOWN                   0
#define fntpubSELECT_CHANNEL1                  1
#define fntpubSELECT_CHANNEL2                  2
#define fntpubSELECT_BOTH_CHANNELS             3

#define fntpubSELECT_UNKNOWN_STR               "UNKNOWN"
#define fntpubSELECT_CHANNEL1_STR              "CH1"
#define fntpubSELECT_CHANNEL2_STR              "CH2"
#define fntpubSELECT_BOTH_CHANNELS_STR         "BOTH"

#endif /* ! spubDEFINES_H */
