#ifndef HIS_H
#define HIS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: his.h,v 1.33 2004/09/23 09:25:12 vltsccm Exp $"
*
* <his.h> - Public Interface of the CCS History System.
*
* who       when      what
* --------  --------  ----------------------------------------------
* tebert    09/05/01  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/*
 * Historian configuration point alias
 */
#define rtHISTORIAN_PT            "<alias>historian config"

/*
 * Historian configuration point attribute name definition
 */
#define rtDH_HISTORY_PTS           "history Points"

/*
 * 'historian Point':  attribute ID numbers
 */
#define rtDH_HISTORY_CONFIG_AIN    0
#define rtDH_SETUP_AIN	           1
#define rtDH_HISTORY_AIN           2
#define rtDH_STATISTIC_AIN         3
#define rtDH_HIST_CONFIG_ATTR_NAME "history config"
#define rtDH_SETUPS_ATTR_NAME	   "setups"
#define rtDH_HISTORY_ATTR_NAME	   "history"

#define rtDH_HC_DATA_START_FIELD   15
#define rtDH_HC_DATA_END_FIELD     134
#define rtDH_SOURCE_MAX_ITEMS      (rtDH_HC_DATA_END_FIELD - rtDH_HC_DATA_START_FIELD + 1)

#define rtDH_HC_TABLE_STATE_START_FIELD 0
#define rtDH_HC_TABLE_STATE_END_FIELD   5

/*
 * History Tables:  field number definitions
 */
#define rtDH_HIST_TIME_STAMP_FIELD        0
#define rtDH_HIST_EVENT_STAMP_FIELD       1
#define rtDH_HIST_SAMPLE_STATUS_FIELD	  2
#define rtDH_HIST_FIELD_OFFSET            3

/*
 * Enable/disable state definitions.
 */
#define rtDH_DISABLE_STATE          (rtLogical) 0
#define rtDH_ENABLE_STATE           (rtLogical) 1
typedef rtLogical	            rtDhState;

/*
 * History table status definitions.
 */
#define rtDH_TABLE_DISABLED     0
#define rtDH_TABLE_IDLE         1
#define rtDH_TABLE_ARMED        2
#define rtDH_TABLE_RUNNING      3
typedef rtUInt8                 rtDhTableStatus;

/*
 * Setup start conditions.
 */
#define rtDH_SETUP_START_ARMED	0
#define rtDH_SETUP_START_TIMED	1
#define rtDH_SETUP_START_EVENT	2
typedef rtUInt8                 rtDhStartCondition;

/*
 * Event trigger states. These are bit field definitions for an rtUINT8.
 */
#define rtDH_EV_TRIG_ALL            0x01
#define rtDH_EV_TRIG_ADDR_START     0x02
#define rtDH_EV_LESS_THAN_START	    0x04
#define rtDH_EV_EQUAL_TO_START	    0x08
#define rtDH_EV_GREATER_THAN_START  0x10
#define rtDH_EV_TRIG_ADDR_STOP      0xe0
#define rtDH_EV_LESS_THAN_STOP	    0x20
#define rtDH_EV_EQUAL_TO_STOP       0x40
#define rtDH_EV_GREATER_THAN_STOP   0x80

#define rtDH_EV_START_BITS(x)       (rtUInt8)(((rtUInt8)(x) >> 2) & (0x07))
#define rtDH_EV_STOP_BITS(x)        (rtUInt8)(((rtUInt8)(x) >> 5) & (0x07))
typedef rtUInt8		            rtDhEvTrigState;

typedef rtStartTime	            rtDhConfigStartTime;

/*
 * Setup interval sample type definitions.
 */
#define rtDH_SAMPLE_DAYS        0
#define rtDH_SAMPLE_HOURS	1
#define rtDH_SAMPLE_MINUTES	2
#define rtDH_SAMPLE_SECONDS	3
typedef rtUInt8	                rtDhSampleType;

/*
 * Setup stop condition definitions.
 */
#define rtDH_STOP_NONE	        0
#define rtDH_STOP_TIMED	        1
#define rtDH_STOP_EVENT	        2
#define rtDH_STOP_COUNT	        3
typedef rtUInt8			rtDhStopCondition;

/*
 * Setup stop type definitions.
 */
#define rtDH_STOP_DAYS          0
#define rtDH_STOP_HOURS	        1
#define rtDH_STOP_MINUTES       2
#define rtDH_STOP_SECONDS       3
typedef rtUInt8			rtDhStopType;

/*
 * Statistic type definitions.
 */
#define rtDH_STAT_DAYS	        0
#define rtDH_STAT_HOURS	        1
#define rtDH_STAT_MINUTES       2
#define rtDH_STAT_SECONDS       3
typedef rtUInt8			rtDhStatType;

/*
 * Statistics enable/disable status. These are bit field definitions for an
 * rtUINT8.
 */
#define rtDH_STATS_MIN_MAX	0x01
#define rtDH_STATS_CNT_AVG	0x02
#define rtDH_STATS_TIME_AVG	0x04
#define rtDH_STATS_MIN_MAX_TBL	0x08
typedef rtUInt8			rtDhStatsState;

/*
 * Define the setup configuration structure.
 */
typedef struct 	
    {
    rtDhState            setupEnable;      /* setup enable      */
    rtDhTableStatus      setupStatus;      /* setup status      */
    rtDhStartCondition   startCondition;   /* start condition   */
    rtDhEvTrigState      triggerStates;    /* trigger bit field */
                                           /* includes <=> defs */
                                           /* for start & stop  */
    rtDhConfigStartTime  startTime;        /* start time        */
    rtDbXref             startTrigAddr;	   /* start trig addr   */
    rtDhSampleType       sampleType;       /* sample type       */
    rtUInt8              reserved1;        /* reserved byte     */
    rtInt16              samplePeriod;     /* time betw'n samp  */
    rtDhStopCondition    stopCondition;    /* stop condition    */
    rtDhStopType         stopType;         /* stop time type    */
    rtInt16	         stopPeriod;       /* stop period       */
    rtDbXref             stopTrigAddr;     /* stop trig addr    */
    rtUInt16             stopCount;        /* stop count        */
    rtDhState            autoReArm;        /* auto re-arm       */
    rtUInt8              reserved2;        /* reserved byte     */
    } rtDhSetup;

/*
 * Define the configuration table status tructure.
 */
typedef struct
    {
    rtUInt16        nextRecord;       /* next record 0 - 65534  */
    rtUInt16        remainingSamples; /* remaining 0 - 65534    */
    rtUInt32        lapCounter;       /* lap ctr 0 - 4294967295 */
    rtUInt16        clearCounter;     /* clear ctr 0 - 65534    */
    rtDhTableStatus tableStatus;      /* dis/idle/armed/running */
    rtUInt8         reserved;         /* reserved byte          */
    } rtDhConfigTableState;

/*
 * Define the statistics configuration structure.
 */
typedef struct 	
    {
    rtUInt32          avgTime;     /* avg time 0 - 4294967295  */
    rtUInt16          avgCount;    /* avg count 0 - 65534      */
    rtDhStatsState    state;       /* bit field for states     */
    rtDhStatType      statType;    /* days/hours/minutes/secs  */
    } rtDhConfigStatistics;

/*
 * Define the output trigger configuration structure.
 */
typedef struct	
    {
    rtDbXref    trigAddr;   /* alarm db address         */
    rtDhState   trigState;  /* disabled/enabled         */
    rtUInt8     reserved;   /* reserved byte            */
    } rtDhConfigOutputTrigger;

/*
 * Define the history item data structure.
 */
typedef struct
    {
    rtDbXref        item[rtDH_SOURCE_MAX_ITEMS]; /* db refs */
    } rtDhConfigRecordData;

/*
 * Define the configuration table structure.
 */
typedef struct
    {
    rtDhConfigTableState    tableState;     /* table state      */
    rtDhConfigStatistics    statistics;     /* stats config     */
    rtDhConfigOutputTrigger outputTrigger;  /* output trig cnfg */
    rtDhState               autoArm;        /* enable/disable   */
    rtDhState               autoWrapEnable; /* auto wrap enable */
    rtDhConfigRecordData    recordData;     /* db addrs         */
    } rtDhConfigTable;

#endif /*!HIS_H*/
