#ifndef ibacLog_H
#define ibacLog_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ibacLog.h,v 1.33 2005/09/01 15:18:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* zins      15/05/98  created
* martiny   18/05/99  declared ATTOL class as friend to be able to access
*                     private and protected members for unit test purposes
* zins      10/06/99  added SetPrintDate() and SetPrintFileLine() methods
* dumont    16/09/99  introduction of NOCCS flag used for LCU development
* gzins     22/11/99  for OCT99, removed #pragma macro used for ATTOL 
*                     (not supported by the new GNU C preprocessor)
* epozna    01/09/05  VLTSW20050208 
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * VLT Software Headers
 */

#include "ccs.h"
#include "log.h"
#include "db.h"

/* This macro is also defined in eccs.h, and do */
    
#ifndef __cplusplus
#ifndef __FILE_LINE__
#define ibacIToStr(a) #a
#define ibacIToStr2(a) ibacIToStr(a) 
#define __FILE_LINE__ __FILE__ ":" ibacIToStr2(__LINE__)
#endif /*!__FILE_LINE__*/
#endif

/*
 * Logging level constants
 */

#define ibacQUIET   0
#define ibacWARNING 1
#define ibacINFO    2
#define ibacTEST    3
#define ibacDEBUG   4
#define ibacEXTDBG  5

/*
 * Log/Verbose/Action Logging Functions
 */
    
extern ccsCOMPL_STAT ibacPrintLog(ccsMODULEID mod, vltINT32 level,
                                  const char *fileLine,
                                  const char *logText, ...);

extern ccsCOMPL_STAT ibacPrintActionLog(vltINT32 level,
                                        const char *logText, ...);

extern ccsCOMPL_STAT ibacSetLogLevel(vltINT32 level);

extern vltINT32      ibacGetLogLevel(void);

extern ccsCOMPL_STAT ibacSetPrintDate(vltLOGICAL flag);
extern ccsCOMPL_STAT ibacSetPrintFileLine(vltLOGICAL flag);

extern ccsCOMPL_STAT ibacSetLog(vltLOGICAL flag);
extern vltLOGICAL    ibacIsLog();
   
extern ccsCOMPL_STAT ibacSetVerboseLevel(vltINT32 level);

extern vltINT32      ibacGetVerboseLevel(void);

extern ccsCOMPL_STAT ibacSetVerbose(vltLOGICAL flag);
extern vltLOGICAL    ibacIsVerbose();

extern ccsCOMPL_STAT ibacSetActionLevel(vltINT32 level);

extern vltINT32      ibacGetActionLevel(void);
    
extern ccsCOMPL_STAT ibacSetActionDbPoint(const dbSYMADDRESS dbPoint); 

/*
 * Convenience macros
 */

/* Logging and Verbose */

#define WarningLog(format, arg...) \
    ibacPrintLog(APP_MODULE_ID, ibacWARNING, __FILE_LINE__, format, ##arg)

#define InfoLog(format, arg...) \
    ibacPrintLog(APP_MODULE_ID, ibacINFO, __FILE_LINE__, format, ##arg)

#define TestLog(format, arg...) \
    ibacPrintLog(APP_MODULE_ID, ibacTEST, __FILE_LINE__, format, ##arg)

#define DebugLog(format, arg...) \
    ibacPrintLog(APP_MODULE_ID, ibacDEBUG, __FILE_LINE__, format, ##arg)

#define ExtDbgLog(format, arg...) \
    ibacPrintLog(APP_MODULE_ID, ibacEXTDBG, __FILE_LINE__, format, ##arg)

/* Action logs */
    
#define ActionLog(level, format, arg...) \
    ibacPrintActionLog(level, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, level, __FILE_LINE__, format, ##arg)

#define WarningActionLog(format, arg...) \
    ibacPrintActionLog(ibacWARNING, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, ibacWARNING, __FILE_LINE__, format, ##arg)

#define InfoActionLog(format, arg...) \
    ibacPrintActionLog(ibacINFO, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, ibacINFO, __FILE_LINE__, format, ##arg)

#define TestActionLog(format, arg...) \
    ibacPrintActionLog(ibacTEST, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, ibacTEST, __FILE_LINE__, format, ##arg)

#define DebugActionLog(format, arg...) \
    ibacPrintActionLog(ibacDEBUG, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, ibacDEBUG, __FILE_LINE__, format, ##arg)

#define ExtDbgActionLog(format, arg...) \
    ibacPrintActionLog(ibacEXTDBG, format, ##arg); \
    ibacPrintLog(APP_MODULE_ID, ibacEXTDBG, __FILE_LINE__, format, ##arg)

#ifdef __cplusplus
};
#endif
  
#endif /*!ibacLog_H*/
