#ifndef ic0HISTORIAN_H_H
#define ic0HISTORIAN_H_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ic0HISTORIAN.h,v 2.14 2005/09/22 07:54:27 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  17/09/01  Created  
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* 
 * System Headers
 */

/* 
 * VLT Headers
 */
#include "ECCS.h" // eccsERROR_CLASS

/* 
 * Local Headers
 */
#include "ic0DEBUG.h"

#define ic0HISTORIAN_INDEX_INVALID -1 // invalid index in historian table

/*
 * Class definition
 */
class ic0HISTORIAN : public eccsERROR_CLASS
{
  public: 
    ic0HISTORIAN();
    virtual ~ic0HISTORIAN();
    virtual ccsCOMPL_STAT Init(const char *fieldName, const dbSYMADDRESS dbRoot);
    virtual vltLOGICAL Available() { return available; }
    virtual ccsCOMPL_STAT SetStart();
    virtual ccsCOMPL_STAT SetStop();
    virtual vltINT32 ReadValues(vltINT32 start = ic0HISTORIAN_INDEX_INVALID, 
                                vltINT32 stop = ic0HISTORIAN_INDEX_INVALID);
    virtual vltINT32  GetSize() { return nTot; }
    virtual vltDOUBLE *GetValues() { return values; }
    virtual ccsCOMPL_STAT Statistics(vltINT32 buflen, vltDOUBLE *buffer);
    virtual vltDOUBLE GetStartVal() { return startVal; }
    virtual vltDOUBLE GetStopVal() { return stopVal; }
    virtual vltDOUBLE Min() { return min; }
    virtual vltDOUBLE Max() { return max; }
    virtual vltDOUBLE Mean() { return average; }
    virtual vltDOUBLE Sum() { return sum; }
    virtual vltDOUBLE Rms() { return rms; }
    virtual vltDOUBLE TMean() { return mean; }
    virtual vltDOUBLE Grad() { return grad; }
    virtual vltDOUBLE LinRegConstant() { return lrconst; }
    virtual vltDOUBLE LinRegRms() { return lrrms; }
    virtual vltDOUBLE DetCoef() { return detcoef; }
    virtual vltDOUBLE Period() { return period; }
    virtual dbSYMADDRESS &Field() { return field; }

  protected:

  private:
    static vltDOUBLE *values; // buffer containing sampled values (static !!)
    static vltINT32 size;     // size of buffer containing sampled values (static !!)
    ic0DEBUG debug;      // used for ic0LOG
    dbSYMADDRESS point;  // historian table OLDB point name
    dbSYMADDRESS statAttr;  // statistics table OLDB name
    dbSYMADDRESS field;  // field containing values
    dbSYMADDRESS dbAttr; // working attribute name
    vltLOGICAL available; // historian table available
    vltINT32 tblSize;    // size of historian table
    vltINT32 statRec;    // statistics table record index
    vltINT32 startIndex; // historian pointer at exposure start
    vltINT32 stopIndex;  // historian pointer at exposure end
    vltINT32 nTot;       // number of samples used for statistics
    vltINT32 period;     // sampling period (sec)
    vltDOUBLE min;       // minimum value
    vltDOUBLE max;       // maximum value
    vltDOUBLE average;   // average
    vltDOUBLE sum;       // sum of samples (average * nTot)
    vltDOUBLE rms;       // root mean square
    vltDOUBLE mean;      // mean value
    vltDOUBLE grad;      // linear regression gradient
    vltDOUBLE lrconst;   // linear regression constant
    vltDOUBLE lrrms;     // linear regression RMS
    vltDOUBLE detcoef;   // linear regression coefficient of determination
    vltDOUBLE startVal;  // first sample value
    vltDOUBLE stopVal;   // last sample value

    vltINT32 Index(); // get pointer to current value in historian
    ccsCOMPL_STAT State(vltLOGICAL state, const char *timeField);
};


#endif /*!ic0HISTORIAN_H_H*/
