#ifndef ic0INS_DEVICE_H
#define ic0INS_DEVICE_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: ic0INS_DEVICE.h,v 2.14 2005/09/22 07:54:25 vltsccm Exp $" 
*
* ic0INS_DEVICE.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  23/08/02  VLTSW20020507
* alongino  02/02/01  VLTSW20000565 VLTSW20000564 VLTSW20000506 VLTSW20000503
* alongino  11/12/99  created
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "oslx.h"            // oslxDICTIONARY
#include "ctooCONFIG.h"
#include "ic0COMPONENT.h"    // parent class
#include "ic0TRIGGER_OBJ.h"  // ic0TRIGGER_OBJ

/*
 * Constants
 */
#define MAX_KEYS 20

class ic0INS_MODEL; // to avoid compilation errors with mutual dependencies
/*
 * Type definitions
 */
typedef struct 
    {            
    char         keyword[32];
    char         devType[32];
    char         devName[32];
    char         devId[32];
    dbSYMADDRESS dbRef;
    vltINT32     lcuNo;
    ic0HANDLER   *handler;
    ic0INS_MODEL *model;
    } ic0INS_DEVICE_CONF;


class ic0INS_DEVICE: public ic0COMPONENT
{
  public:

    ic0INS_DEVICE();
    virtual ~ic0INS_DEVICE();

    fndStdObjectDef(ic0INS_DEVICE, ic0COMPONENT)

    char *Name() const { return (char *)conf.devName; }
    virtual ccsCOMPL_STAT Init(ic0INS_DEVICE_CONF &device);
    vltLOGICAL IsAssembly() { return assembly; }  
    vltLOGICAL FitsHeader() { return fitsHeader; }
    virtual char* Value(vltINT32 index = 0);
    virtual char* Key(vltINT32 index = 0);
    virtual vltINT32 NValue() { return nval; }
    vltLOGICAL Fallback() { return fallback; }
    vltINT32   Lcu() { return conf.lcuNo; }
    void SetupRunning(vltLOGICAL flag) { setupRunning = flag; }
    vltLOGICAL SetupPostProc() { return setupPostProc; }

  protected:
    ic0TRIGGER_OBJ    trigger;
    ic0INS_DEVICE_CONF conf;   
    ctooCONFIG        *cfg;           // pointer to instrument configuration
    oslxDICTIONARY    *dictionary;
    oslxALIAS         *alias;
    vltLOGICAL        simulation;
    dbSYMADDRESS      dbAttr;
    ic0KEYW_KEY       key;
    vltINT32          setupKeyN;       // number of setup keywords
    slxKEYW_LINE      setupKeys[MAX_KEYS]; // array of keywords

    virtual ccsCOMPL_STAT DevTrigger(ic0KEYWORD &keyword, void*);
    virtual ccsCOMPL_STAT AddKeyword(ic0KEYWORD &keyword, vltLOGICAL alloc);
    virtual ccsCOMPL_STAT CheckKeyword(ic0KEYWORD &keyword);
    virtual ccsCOMPL_STAT Value(const char *val, vltINT32 index = 0);
    virtual ccsCOMPL_STAT Key(const char *val, vltINT32 index = 0);
    virtual ccsCOMPL_STAT NValue(vltINT32 n);
    void IsAssembly(vltLOGICAL flag) { assembly = flag; } 
    void FitsHeader(vltLOGICAL flag) { fitsHeader = flag; } 
    void Fallback(vltLOGICAL flag) { fallback = flag; }
    vltLOGICAL Header();   
    vltLOGICAL SetupRunning() { return setupRunning; }
    void SetupPostProc(vltLOGICAL flag) { setupPostProc = flag; }
   
  private:
    slxKEYW_LINE      *values;
    slxKEYW_LINE      *keys;
    vltINT32          nval;
    vltLOGICAL        assembly;
    vltLOGICAL        fitsHeader;
    vltLOGICAL        hasValue;
    vltLOGICAL        fallback;
    vltLOGICAL        setupRunning;
    vltLOGICAL        setupPostProc;

}; /* end class ic0INS_DEVICE */

#endif /*!ic0INS_DEVICE_H*/











