#ifndef ic0INS_MODEL_H
#define ic0INS_MODEL_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: ic0INS_MODEL.h,v 2.14 2005/09/22 07:54:21 vltsccm Exp $" 
*
* ic0INS_MODEL.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  13/02/01  VLTSW20010114
* alongino  02/02/01  VLTSW20000565 VLTSW20000564 VLTSW20000506 VLTSW20000503
* alongino  11/12/99  introduced in ic0SERVER for multiple LCUs
* therlin   10/01/96  Created
*/

/************************************************************************
*
*----------------------------------------------------------------------
*/

#include <fndARRAY.h>
#include <fndSTRING.h>
#include <fndPOINTER.h>
#include "ECCS.h"
#include "oslx.h"              // oslxDICTIONARY
#include "ctooCONFIG.h"
#include "ic0COMPONENT.h"
#include "ic0DEBUG.h"
#include "ic0HANDLER.h"
#include "ic0INS_DEVICE.h"

class ic0SERVER; // to avoid compilation errors with mutual dependencies

class ic0INS_MODEL: public fndARRAY, public eccsERROR_CLASS
{
  public:
    static  ic0INS_MODEL  *model;  
    oslxDICTIONARY dictionary;     // dictionary used
    slxDICTIONARY  dicFile;        // name of dictionary file
    oslxALIAS      alias;          // alias used
    ic0SERVER      *server;        // pointer to WS server
    ctooCONFIG     *cfg;           // pointer to instrument configuration

                          ic0INS_MODEL();
    virtual               ~ic0INS_MODEL();
    fndStdObjectDef(ic0INS_MODEL, fndARRAY)
    virtual ccsCOMPL_STAT Init(const dbSYMADDRESS dbRef, 
		               ctooCONFIG &insCfg,
		               char *dicName = NULL,
	                       char *aliasName = NULL,
                               void *server = NULL);
    virtual void          Add( ic0COMPONENT& );
    virtual ccsCOMPL_STAT Add( ic0INS_DEVICE_CONF&); // VLTSW20040376 return ccsCOMPL_STAT
    virtual void          Delete( ic0COMPONENT& );
    virtual vltINT32      GetCompNo();
    virtual ic0COMPONENT  &GetComponent(char *id); 
    virtual ic0INS_DEVICE &GetDev(vltINT32 i); 
    virtual ic0INS_DEVICE &SearchDev(char *id); 
            ccsCOMPL_STAT ClearValidFlags();
            ccsCOMPL_STAT ClearFlags();
    virtual ccsCOMPL_STAT Test();
            ic0HANDLER    &Handler();
	    void          PreProcessing();
	    void          PostProcessing();
    virtual ccsCOMPL_STAT AddKeyword(ic0KEYWORD &keyword, 
                                     vltLOGICAL alloc = ccsFALSE);
    virtual ccsCOMPL_STAT AddNextStepKeyword(ic0KEYWORD &keyword, 
                                             vltLOGICAL alloc = ccsFALSE);
    virtual ccsCOMPL_STAT AddWsKeyword(ic0KEYWORD &keyword, 
                                       vltLOGICAL alloc = ccsFALSE);
    ccsCOMPL_STAT         NewKeywordList(ic0KEYWORD_LIST &keyList);
    ccsCOMPL_STAT         NextStepKeywordList(ic0KEYWORD_LIST &keyList);
    ccsCOMPL_STAT         WsKeywordList(ic0KEYWORD_LIST &keyList);
    ccsCOMPL_STAT         Command(ic0KEYWORD_LIST &keyList, msgCMD cmd);

    vltLOGICAL            Simulation() { return(simulation); }
    vltLOGICAL            FirstCmd();          // check if first command in a multi-step SETUP
    void                  FirstCmd(vltLOGICAL flag); // set if first command in a multi-step SETUP
    void                  Header(vltLOGICAL value) { header = value; };
    vltLOGICAL            Header() {  return (header); };
    const char            *DbRoot() {  return (icsDbRoot); };
    char                  *Category() { return (category); }
    char                  *CfgCategory() { return (cfgCat); }

  protected:
    ic0HANDLER            *handler;
    vltLOGICAL            preProcFlag;
    vltLOGICAL            simulation;
    ic0DEBUG              debug;        // debug object
    virtual ccsCOMPL_STAT CreateDevices(const dbSYMADDRESS dbRef);

  private:
    ic0KEYWORD_LIST       newList;      // global keyword list
    ic0KEYWORD_LIST       nextStepList; // list used for multi-step SETUP with assemblies
    ic0KEYWORD_LIST       wsList;       // list used for STATUS with assemblies
    vltLOGICAL            firstCmd;     // first command in multi-step SETUP
    vltLOGICAL            header;       // header requested
    dbSYMADDRESS          icsDbRoot;    // ICS OLDB root point
    slxKEYW_LINE          category;     // setup/status keywords default category
    slxKEYW_LINE          cfgCat;       // configuration category

    virtual ccsCOMPL_STAT AddKeyword2List(ic0KEYWORD_LIST &keyList,
                                          ic0KEYWORD &keyword, 
                                          vltLOGICAL alloc = ccsFALSE);
    ccsCOMPL_STAT         CopyKeywordList(ic0KEYWORD_LIST &toList,
                                          ic0KEYWORD_LIST &fromList);
    vltINT32              GetLcu(vltINT32 index);

}; /* end class ic0INS_MODEL */


inline vltINT32 ic0INS_MODEL::GetCompNo()
{
    return (GetItemsInContainer());
}

inline void ic0INS_MODEL::PreProcessing()
{
    preProcFlag = ccsTRUE;
}

inline void ic0INS_MODEL::PostProcessing()
{
    preProcFlag = ccsFALSE;
}

inline ic0HANDLER &ic0INS_MODEL::Handler() 
{
    return *handler;
}

#endif /*!ic0INS_MODEL_H*/











