#ifndef IC0_SERVER_H
#define IC0_SERVER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ic0SERVER.h,v 2.14 2005/09/22 07:54:22 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  09/01/02  added fallback init treatment (VLTSW20010770)
* alongino  02/02/01  VLTSW20000565 VLTSW20000564 VLTSW20000506 VLTSW20000503
* alongino  11/12/99  added INS_MODEL
* alongino  06/10/99  added multiReplyEmerg (VLTSW990400)
* alo/rsc   19/07/99  OCT99: virtual added to GeneralCmdCB.
* alongino  27/08/97  Added method CheckAllParam
* therlin   17/09/96  Added new commands
* therlin   30/08/95  Created  
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#include <fndDICTIONARY.h>
#include "ic0.h"
#include "ic0chkINTERFACE.h"
#include "ic0simINTERFACE.h"
#include "ic0lcuINTERFACE.h"
#include "ic0STATUS.h"
#include "ic0SCAN.h"
#include "ic0DEBUG.h"
#include "ic0INS_MODEL.h"
#include "ctooCONFIG.h"  // ctooCONFIG
#include "evhSTD_COMMANDS.h"
#include "evhCOMMAND.h"
#include "cmd.h"

class ic0SERVER : public evhSTD_COMMANDS
{
  public:
    ic0genINTERFACE  *lcuCmd;       // Virtual interface pointer

    ic0SERVER();
    ic0SERVER(const dbSYMADDRESS dbRef,
	            ccsPROCNAME  chkServiceName,
	            ccsPROCNAME  simServerName,
	            vltINT32     lcuNum,
	            ic0lcuSERVER *lcuServers,
	            char         *dictionary,
	            char         *alias,
	            ic0chkType   chkInterfaceGhost = ic0chkINTERFACE_CLASS,
                    ic0simType   simInterfaceGhost = ic0simINTERFACE_CLASS,
	            ic0lcuType   lcuInterfaceGhost = ic0lcuINTERFACE_CLASS);   
    ic0SERVER(const dbSYMADDRESS dbRef,
	            ctooCONFIG   &cfg,
	            ccsPROCNAME  simServerName,
	            vltINT32     lcuNum,
	            ic0lcuSERVER *lcuServers,
	            char         *dictionary,
	            char         *alias);   
   ~ic0SERVER();
    fndStdObjectDef(ic0SERVER, evhSTD_COMMANDS)
    virtual ccsCOMPL_STAT InitServers();
    // Class callbacks
    virtual evhCB_COMPL_STAT CommentCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT SetModeCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT SetupCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT SimCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT SimulatCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StopSimCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StateCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT ExpCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT VersionCB(msgMESSAGE &msg, void *udata);
    // Maintenance callbacks
    virtual evhCB_COMPL_STAT DebugCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT MaintenanceCB(msgMESSAGE &msg, void *udata);

    // State commands callbacks
    // All state commands OFF, STANDBY, ONLINE essentially use StateCmdCB
    // but the functions below are kept i.o. to be overloadable
    virtual evhCB_COMPL_STAT OffCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StandByCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StateCmdCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StateCmdReplyCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT StateCmdErrorCB(msgMESSAGE &msg, void *udata);

    // State transition methods
    virtual ccsCOMPL_STAT Loaded2Standby(void *udata);
    virtual ccsCOMPL_STAT Standby2Online(void *udata);
    virtual ccsCOMPL_STAT Online2Standby(void *udata);
    virtual ccsCOMPL_STAT Standby2Loaded(void *udata);
    virtual ccsCOMPL_STAT Online2Loaded(void *udata);

    // state & substate support methods
    virtual ccsCOMPL_STAT CheckState(const char *command);
    virtual ccsCOMPL_STAT CheckSubState(const char *command);
    virtual ccsCOMPL_STAT ResetSubState(const char *command);
    virtual char          *SubStateName();
    virtual ccsCOMPL_STAT LcuState();

    // Init related methods
    virtual evhCB_COMPL_STAT InitReplyCB(msgMESSAGE &msg, void *udata);
    // Setup related methods
    virtual evhCB_COMPL_STAT SetupErrorCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT SetupReplyCB(msgMESSAGE &msg, void *udata);
    // Status related methods
    virtual evhCB_COMPL_STAT StatusReplyCB(msgMESSAGE &msg, void *udata);

    // Keyword related methods
    virtual ccsCOMPL_STAT KeywordRangeCheck(msgCMD         cmd,
					   ic0KEYWORD_LIST *keyword);
    virtual ccsCOMPL_STAT KeywordPreProcessing(msgCMD         cmd,
					       ic0KEYWORD_LIST *keyword);
    virtual ccsCOMPL_STAT KeywordPostProcessing(msgCMD         cmd,
						ic0KEYWORD_LIST *keyword);
    virtual ccsCOMPL_STAT CheckFunctions(const msgCMD cmd, 
					 vltINT32 no, 
					 char **keywArray);

    virtual evhCB_COMPL_STAT TestCB(msgMESSAGE &msg, void *udata);

    // Standard function callback - which maps commands with notation:
    // COMMAND -function <keywords>
    virtual evhCB_COMPL_STAT GeneralCmdCB(msgMESSAGE &msg, void *udata);

    // "Dynamic" Callbacks
    virtual evhCB_COMPL_STAT ReplyCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT ErrorCB(msgMESSAGE &msg, void *udata);
    virtual ccsCOMPL_STAT    ErrorAnalysis(const char *command);
    virtual evhCB_COMPL_STAT TimeoutCB(msgMESSAGE &msg, void *udata);
    virtual ccsCOMPL_STAT    CreateModel(const dbSYMADDRESS dbRef,
		                         ctooCONFIG   &insCfg,
		                         char         *dictionary,
		                         char         *alias);
    virtual ccsCOMPL_STAT    AddFixedKeywords(ic0KEYWORD_LIST *keyList);

    ccsCOMPL_STAT AddCallbacks();

    // Misc data structures
    ic0DEBUG *Debug();
    void Version(const char *version);
    const char *Version();
    vltINT32 LcuNum() { return lcuTot; }
    void SetLcu(vltINT32 lcuTot, vltINT32 *lcuIndex);
    vltINT32 GetLcu(vltINT32 lcu);
    char *Category()  { return (category); }
    char *CfgCategory() { return (cfgCat); }

  protected:
    ic0chkINTERFACE  *chkService;   // Interface to parameter check server 
    ic0simINTERFACE  *simServer;    // Interface to command simulation server
    ic0lcuINTERFACE  *lcuServer;    // Interface to ICS lcu server  
    cmdPARAM_LIST    paramList;     // Command parameter list
    ic0STATUS        *status;       // STATUS object
    ic0DEBUG         debug;         // debug object
    vltINT32         multiReply;    // syncronization value for multiple LCU's
    ic0KEYWORD_LIST  keywordList;   // list to pass keywords between objects
    ic0KEYWORD_LIST  nextStepList;  // list used for multi-step SETUP with assemblies
    ic0KEYWORD_LIST  wsList;        // list used for STATUS with assemblies
    vltINT32         emergencyCmd;  // counter for emergency commands running
    vltINT32         multiReplyEmerg; // syncronization value for emergency cmd
    ic0INS_MODEL     *model;        // instrument model
    ctooCONFIG       *cfg;          // instrument configuration
    fndDICTIONARY    fitsHead;      // dictionary of fits headers

    // database values
    eccsDB_INT32     substate;  // substate (usage reserved for sub-classes)
    eccsDB_INT32     opMode;    // ics operational mode
    eccsDB_INT32     error;     // error indication (usage reserved for sub-classes)
    eccsDB_INT32     timeout;   // timeout value in millisec
    eccsDB_LOGICAL   simulation;// flag for simulation mode
    eccsDB_LOGICAL   maintMode; // flag for maintenance mode
    eccsDB_LOGICAL   extFitsHeader; // flag for dumping extended fits header

    virtual evhCB_COMPL_STAT ErrorReply(msgMESSAGE &msg,
					vltLOGICAL reset = ccsTRUE);

    virtual evhCB_COMPL_STAT Reply(msgMESSAGE &msg,char *what);

    virtual evhCB_COMPL_STAT OkReply(msgMESSAGE &msg);

    virtual ccsCOMPL_STAT WsStatus(msgMESSAGE &msg);
    // Placeholder for requests to attach to OLDB events
    virtual ccsCOMPL_STAT AttachDbEvents();
    // Placeholder for requests to start timers
    virtual ccsCOMPL_STAT AddTimers();
    virtual ccsCOMPL_STAT FallbackInit();
    // used within ReplyCB and SetupReplyCB
    virtual vltLOGICAL ReplyReceived(msgMESSAGE &msg);
    virtual evhCB_COMPL_STAT SendReply(vltLOGICAL lastReply, msgMESSAGE &msg, void *udata);

    vltLOGICAL CheckAllParam(char *param);

    vltLOGICAL LastCmd();           // check if last command in a multi-step SETUP
    void LastCmd(vltLOGICAL flag);  // set if last command in a multi-step SETUP
    vltLOGICAL FirstCmd();          // check if first command in a multi-step SETUP
    void FirstCmd(vltLOGICAL flag); // set if first command in a multi-step SETUP

    void ClearLists();
    ccsCOMPL_STAT RegFallbackDev(char *name, vltINT32 lcu = 0); // register a falback device
    virtual ccsCOMPL_STAT RegFallbackDevs(); // register all fallback devices
    ic0KEYWORD_LIST *AllDevices();
    ic0KEYWORD_LIST *SetupPostProcDevices();
    void ClearSetup();

  private:
    const char       *version;      // version string
    vltLOGICAL       lastCmd;       // last command in multi-step SETUP
    vltLOGICAL       firstCmd;      // first command in multi-step SETUP
    vltINT32         lcuTot;        // number of total lcus defined
    vltINT32         *lcuIndex;     // index associated to each used LCU
    ic0SCAN          *scan;         // pointer to object handling ICS OLDB scan
    fndARRAY         fbackDevs;     // array of devices using fallback initialisation
    evtEVENT         *fbackEvt;     // list of events for fallback initialisation
    slxKEYW_LINE     category;      // keywords default category
    slxKEYW_LINE     cfgCat;        // configuration category
    vltINT32         sumReply;      // sum of replies to last command received from LCUs 

    void Init(const dbSYMADDRESS dbRef,
	      ccsPROCNAME  chkServiceName,
	      ccsPROCNAME  simServerName,
	      vltINT32     lcuNum,
	      ic0lcuSERVER *lcuServers,
	      char         *dictionary,
	      char         *alias, 
	      ic0chkType   chkInterfaceGhost = ic0chkINTERFACE_CLASS,
              ic0simType   simInterfaceGhost = ic0simINTERFACE_CLASS,
	      ic0lcuType   lcuInterfaceGhost = ic0lcuINTERFACE_CLASS);

    ccsCOMPL_STAT         CopyKeywordList(ic0KEYWORD_LIST &toList,
                                          ic0KEYWORD_LIST &fromList);
    ccsCOMPL_STAT AddDbScan(); // scan OLDB attributes from WS/LCU to LCU
    evhCB_COMPL_STAT FallbackCB(evtEVENT_MSG &msg, void *udata);
    ccsCOMPL_STAT GetFallbackPos(fndASSOCIATION &device);
    ccsCOMPL_STAT SetFallbackPos();
};




// case unsensitive check if the parameter passed is ic0ALL
inline vltLOGICAL ic0SERVER::CheckAllParam(char *param)
{
    unsigned int length = strlen(ic0ALL);

    if (strlen(param) == length)
	{
        char paramUC[length+1];
        strcpy(paramUC, param);
        slxStrToUpper(paramUC);
        char allUC[length+1];
        strcpy(allUC, ic0ALL);
        slxStrToUpper(allUC);
        if (strcmp(paramUC, allUC) == 0) return TRUE;
	}
    return FALSE;
}
#endif /*!IC0_SERVER_H*/
