#ifndef IC0_STATUS_H
#define IC0_STATUS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ic0STATUS.h,v 2.14 2005/09/22 07:54:22 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* therlin   02/05/96  Created  
*/

/************************************************************************
*   NAME
*   ic0STATUS - class holding data related for the STATUS command.
*
*   SYNOPSIS
*   #include "ic0STATUS.h"
*
*   ic0STATUS status;

*   PARENT CLASS
*
*   DESCRIPTION
*   The ic0STATUS class holds the data information of an STATUS command.
*
*   The ic0KEYWORD_LIST statusList points to the list of keywords currently 
*   handled. 
*   
*   When a STATUS command is received a ic0STATUS instance is created.
*   After the STATUS command is processed the STATUS object should be destroyed.
*
*   PUBLIC METHODS:
*
*    ic0STATUS();
*          Constructor.
*
*   ~ic0STATUS();
*          Destructor.
*
*    ccsCOMPL_STAT     FitsFile(char *fileName);
*    char              *FitsFile();
*          Set and retrieve the FITS file name of the object.
*
*    ccsCOMPL_STAT     SetupFile(char *fileName);
*    char              *SetupFile();
*          Set and retrieve the setup file name of the object.
*
*    void              AllFunctions(vltLOGICAL value);
*    vltLOGICAL        AllFunctions();
*          Set and retrieve the all functions flag.
*
*    ic0KEYWORD_LIST   &StatusList();
*
*    void              Clear();
*    
*   PROTECTED DATA MEMBERS:
* 
*    ic0KEYWORD_LIST   statusList;     // list of keywords handled in this session
*    char              *fitsFile;      // name of the FITS file the status must be
*                                      // stored in 
*    char              *setupFile;     // name of the setup file the status is 
*                                      // stored in
*    vltLOGICAL        allFunctions;   // flag if status of all functions requested 
*
*   RETURN VALUES
*
* 
*------------------------------------------------------------------------
*/


#include <fndOBJECT.h>
#include "ic0.h"

/*
 * Class definition
 */
class ic0STATUS : public fndOBJECT
{
  public: 
    ic0STATUS();
   ~ic0STATUS();
    ccsCOMPL_STAT     FitsFile(char *fileName);
    char              *FitsFile()    { return (fitsFile); };
    ccsCOMPL_STAT     SetupFile(char *fileName);
    char              *SetupFile()   { return (setupFile); };
    void              AllFunctions(vltLOGICAL value) { allFunctions = value; };
    vltLOGICAL        AllFunctions() {  return (allFunctions); };
    ic0KEYWORD_LIST   *StatusList()  { return (& statusList); };
    void              Header(vltLOGICAL value) { header = value; };
    vltLOGICAL        Header() {  return (header); };
    void              Conflog(vltLOGICAL value) { conflog = value; };
    vltLOGICAL        Conflog() {  return (conflog); };
    void              Opslog(vltLOGICAL value) { opslog = value; };
    vltLOGICAL        Opslog() {  return (conflog); };
    void              Condlog(vltLOGICAL value) {  condlog = value; };
    vltLOGICAL        Condlog() {  return (conflog); };

    void              Clear();

  protected:    
    ic0KEYWORD_LIST   statusList;     // list of keywords handled in this session
    char              fitsFile[256];      // name of the FITS file the status must be
                                      // stored in 
    char              setupFile[256];     // name of the setup file the status is 
                                      // stored in
    vltLOGICAL        allFunctions;   // flag if status of all functions requested
    vltLOGICAL        header;         // header requested
    vltLOGICAL        conflog;        // conflog requested
    vltLOGICAL        opslog;         // opslog requested
    vltLOGICAL        condlog;        // condlog requested

 
  private:
};

inline void ic0STATUS::Clear() 
{ 
  memset(fitsFile,'\0',sizeof(fitsFile));
  memset(setupFile,'\0',sizeof(setupFile));
  allFunctions = ccsFALSE;
  header = ccsFALSE; 
  conflog = ccsFALSE;
  opslog = ccsFALSE; 
  condlog = ccsFALSE;

  if (statusList.GetItemsInContainer() > 0)
      {
	statusList.Empty(ccsTRUE);
      }
}

inline ic0STATUS::ic0STATUS () 
{
    Clear();
}

inline ic0STATUS::~ic0STATUS () 
{
    Clear();
}


inline ccsCOMPL_STAT ic0STATUS::FitsFile(char *fileName) 
{ 
  memset(fitsFile,'\0',sizeof(fitsFile));
  
  if (strlen(fileName))
    strcpy(fitsFile,fileName);

  return SUCCESS;
}


inline ccsCOMPL_STAT ic0STATUS::SetupFile(char *fileName) 
{
  memset(setupFile,'\0',sizeof(setupFile));
  
  if (strlen(fileName))
    strcpy(setupFile,fileName);

    return SUCCESS;
}

#endif /*!IC0_STATUS_H*/
