#ifndef IC0GEN_INTERFACE_H
#define IC0GEN_INTERFACE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ic0genINTERFACE.h,v 2.14 2005/09/22 07:54:23 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* therlin   30/08/95  Created  
* therlin   17/09/96  Added default callbacks references
* alongino  06/10/99  added allIdsEmerg (VLTSW990400)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */


#include "cmd.h"
#include "eccsMsg.h"
#include "evhOBJ_CALLBACK.h"
#include "evhCOMMAND.h"

#include "ic0.h"
#include "ic0KEYWORD_LIST.h"
#include "ic0DEBUG.h"

#define ic0genMAX_LCU 4
/*
 * Class definition
 */
class ic0genFORWARD
{
  public: 
    ic0genFORWARD(msgMESSAGE &msg);
   ~ic0genFORWARD() {};

    msgMESSAGE   &ReplyMsg();
    evhCOMMAND   &SendCmd(int no = 0);
    
  private:
    msgMESSAGE              replyMsg;
    evhCOMMAND              sendCmd[ic0genMAX_LCU];

}; /* end class ic0genFORWARD */

inline ic0genFORWARD::ic0genFORWARD(msgMESSAGE &msg)
{
    replyMsg = msg;
}

inline msgMESSAGE   &ic0genFORWARD::ReplyMsg()
{
   return (replyMsg);
}

inline  evhCOMMAND  &ic0genFORWARD::SendCmd(int no)
{
   if (no >= ic0genMAX_LCU) no = 0;
   return (sendCmd[no]);
}

/*
 * Class definition
 */
class ic0genINTERFACE : public msgMESSAGE
{
  public: 
    ic0genINTERFACE();
   ~ic0genINTERFACE();
    void SetMaster(fndOBJECT *newMaster);

    virtual ccsCOMPL_STAT   ComParameters(ccsPROCNAME serverName,
					  ccsENVNAME  lcuName = "");

    virtual ccsCOMPL_STAT   InitCommunication();

    // synchronous sending of commands 
    virtual ccsCOMPL_STAT   Send();
    virtual ccsCOMPL_STAT   Send(cmdPARAM_LIST   *paramList,
                                 ic0KEYWORD_LIST *keywordList); // VLTSW980686
    // asyncronous forwarding of commands
    virtual ccsCOMPL_STAT   Forward(const char      *command,
				    cmdPARAM_LIST   *paramList,
				    ic0KEYWORD_LIST *keywordList,
				    ic0genFORWARD   *forwardObj,
				    evhCB_METHOD    replyCB = 0,
				    evhCB_METHOD    errorCB = 0,
				    evhCB_METHOD    timeoutCB = 0);

    //
    virtual ccsCOMPL_STAT   Forward(const char      *command,
				    char            *message,
				    ic0genFORWARD   *forwardObj,
				    evhCB_METHOD    replyCB = 0,
				    evhCB_METHOD    errorCB = 0,
				    evhCB_METHOD    timeoutCB = 0);

    virtual msgMESSAGE      &Parameters(cmdPARAM_LIST *paramList, 
					vltINT32      start = 0);

    // return reply buffer
    virtual const char *Reply(vltINT32 lcuNo = 0);
    virtual ccsCOMPL_STAT     NextReply(vltINT32 lcuNo = 0);
    virtual vltLOGICAL        LastReply(vltINT32 lcuNo = 0);
    virtual vltLOGICAL        ReplyFlag(vltINT32 lcuNo = 0);
    virtual ccsCOMPL_STAT     ReplyFlag(vltLOGICAL flag, 
                                        vltINT32 lcuNo = 0);

    virtual vltLOGICAL      ParametersPresent(cmdPARAM_LIST *paramList);
    virtual ccsCOMPL_STAT   RemoveParameters(cmdPARAM_LIST *paramList, 
					     vltUINT32     index);

    virtual ccsCOMPL_STAT   ParseReply(const char      *replyBuf, 
				       ic0KEYWORD_LIST *keywordList,
				       vltLOGICAL      prependNo = ccsFALSE);
    virtual ccsPROCNAME     *ServerName();
    virtual ccsENVNAME      *LcuName();
    virtual vltINT32        AllIds(const char *command = NULL);
    virtual ccsCOMPL_STAT   Timeout(vltINT32 msec);

    virtual ccsCOMPL_STAT   SetDefaultReplies(evhCB_METHOD reply = 0,
					      evhCB_METHOD error = 0,
					      evhCB_METHOD timeout = 0);

    virtual vltINT32        LcuNum() { return 1; };
  protected:    
    msgRECEIVEFILTER        recvFilter;
    vltINT32                timeout;
    ccsTIMEVAL              timeoutInterval;
    fndOBJECT               *master;
    ic0DEBUG                debug;
    vltINT32                allIds;
    vltINT32                allIdsEmerg; // for emergency commands
    evhCB_METHOD            defaultCB; 
    evhCB_METHOD            defaultErrorCB;
    evhCB_METHOD            defaultTimeoutCB;

  private:
    vltLOGICAL              replyInBuffer;
    evhCB_COMPL_STAT        ForwardClean(msgMESSAGE &msg, void *);

};

inline vltINT32 ic0genINTERFACE::AllIds(const char *command)
{
    if (command != NULL)
      if (strcmp(command,ic0STOP_CMD) == 0) return allIdsEmerg; // STOP command
    return allIds;  
}

// parameter needed to allow overloading in ic0lcuINTERFACE for more LCUs
inline ccsPROCNAME *ic0genINTERFACE::ServerName()
{
    return (ccsPROCNAME *)Destproc();
}

inline ccsENVNAME *ic0genINTERFACE::LcuName()
{
    return (ccsENVNAME *)Destenv();
}
#endif /*!IC0GEN_INTERFACE_H*/
