#ifndef IC0SIM_SERVER_H
#define IC0SIM_SERVER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ic0simSERVER.h,v 2.14 2005/09/22 07:54:24 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* therlin   30/08/95  Created  
* therlin   17/03/97  Added user procedure for simulation on SETUP.
* therlin   12/08/97  Added missing dbFillBuff for ic0simDATAREF_TABLE
* therlin   13/08/97  Added missing dbFillBuff for ic0simDATAREF_TABLE
* alongino  27/08/97  Added method CheckAllParam
* rschmutz  08/08/99  OCT99: egcs-2.91 warnings removed.
* alongino  06/09/00  added missing commands for the ic0lcuServer CDT
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/*
 * CCS includes
 */
#include "evhSTD_COMMANDS.h"
#include "evhCOMMAND.h"
#include "eccsDB_TABLE.h"
#include "cmd.h"
#include "ctooCONFIG.h"  // ctooCONFIG

/*
 * Local includes
 */
#include "ic0.h"
#include "ic0DEBUG.h"
#include "ic0KEYWORD_LIST.h"
#include "ic0STATUS.h"
#include "ic0INS_MODEL.h"

/* Database references for simulation tables */
#define ic0simREPLY_ATTR "replies"
#define ic0simDATAREF_ATTR "dataref"

/* Commands used by ic0lcuServer and not defined anywhere else */
#define ic0simCHECK_CMD   "CHECK"
#define ic0simFIXPOS_CMD  "FIXPOS"
#define ic0simGETCONF_CMD "GETCONF"
#define ic0simGETDEBU_CMD "GETDEBU"
#define ic0simHANDSET_CMD "HANDSET"
#define ic0simHSETOFF_CMD "HSETOFF"
#define ic0simLOGCONF_CMD "LOGCONF"
#define ic0simSET_CMD     "SET"
#define ic0simWAIT_CMD    "WAIT"

class ic0simREPLY_TABLE : public eccsDB_TABLE_RECORD
{
  public:
    vltBYTES8   command;
    vltBYTES256 reply;
    ccsCOMPL_STAT Unpack( void *buffer )
    {
	memcpy( &command, buffer,sizeof(command) ); 
	buffer = (char*)buffer+sizeof(command);
	memcpy( &reply, buffer,sizeof(reply)); 
	return(SUCCESS);
    }
    ccsCOMPL_STAT Pack( void *buffer )
    {
	int size;
	size = 0;
        dbFillBuf( (char**)&buffer, (char*)&command, &size, dbBYTES8);
        dbFillBuf( (char**)&buffer, (char*)&reply,   &size, dbBYTES256);
	return(SUCCESS);
    }
};

class ic0simDATAREF_TABLE : public eccsDB_TABLE_RECORD
{
  public:
    vltBYTES32  keyword;
    vltBYTES256 dbReference;
    vltINT32    type;
    ccsCOMPL_STAT Unpack( void *buffer )
    {
	memcpy( &keyword, buffer,sizeof(keyword) ); 
	buffer = (char*)buffer+sizeof(keyword);
	memcpy( &dbReference, buffer,sizeof(dbReference)); 
	buffer = (char*)buffer+sizeof(dbReference);
	memcpy( &type, buffer,sizeof(type)); 
	return(SUCCESS);
    }
    ccsCOMPL_STAT Pack( void *buffer )
    {
	int size;
	size = 0;
        dbFillBuf( (char**)&buffer, (char*)&keyword,     &size, dbBYTES32);
        dbFillBuf( (char**)&buffer, (char*)&dbReference, &size, dbBYTES256);
        dbFillBuf( (char**)&buffer, (char*)&type,        &size, dbINT32);
	return(SUCCESS);
    }
};

class ic0simSERVER : public evhSTD_COMMANDS
{
  public:
    ic0simSERVER();
    ic0simSERVER(const dbSYMADDRESS dbRef);   
    ic0simSERVER(const dbSYMADDRESS dbRef, ctooCONFIG &cfg);   
   ~ic0simSERVER();
    fndStdObjectDef(ic0simSERVER, evhSTD_COMMANDS)

    // 
    virtual evhCB_COMPL_STAT  TestCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  SimCommandCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  SimReadDbReply(msgCMD command);
    virtual evhCB_COMPL_STAT  StateCmdCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  StatusCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  SetupCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  SelfTestCB(msgMESSAGE &msg, void *udata);
    virtual evhCB_COMPL_STAT  StopCB(msgMESSAGE &msg, void *udata);
    virtual ccsCOMPL_STAT     CreateModel(const dbSYMADDRESS dbRef,
		                          ctooCONFIG   &insCfg);
    // 

    ic0DEBUG *Debug();

  protected:  
    ic0INS_MODEL     *model;        // instrument model
    ctooCONFIG       *cfg;          // instrument configuration

    virtual ccsCOMPL_STAT    WriteSimKeyword(char          *dbAddr,
				             unsigned char *value,
					     vltINT32      dbType,
					     vltLOGICAL    relative = ccsFALSE);

    virtual ccsCOMPL_STAT    ReadSimKeyword(char          *dbAddr,
					    unsigned char *value,
					    vltINT32      dbType);

    // Overloadable dummies
    virtual ccsCOMPL_STAT    SimSetupProc(char *keyword, 
					  char *value,
					  const vltLOGICAL noMove = ccsFALSE);
    virtual ic0KEYWORD      *SimSetupKeyword(vltINT32 index);

    virtual ccsCOMPL_STAT    SimStatusProc(char *keyword, char *value);
    virtual ccsCOMPL_STAT    SimStatusPrepare(char *keyword);
    virtual vltLOGICAL       SimStatusCompare(char *keyInp, 
                                              char *keyTbl,
                                              vltLOGICAL &breakLoop);

    virtual ccsCOMPL_STAT    CompleteDataRefTbl(); // complete data ref. table
    
    // Format the setup value
    virtual ccsCOMPL_STAT    SetupFormat(char       *keyword, 
					 char       *value, 
					 vltLOGICAL *relative);

    // Format the status reply data
    virtual vltINT32         FormatData(unsigned char *keyword,
					unsigned char *value, 
					vltINT32 type,
					char     *buffer);
	
    virtual evhCB_COMPL_STAT ErrorReply(msgMESSAGE *msg); 

    dbSYMADDRESS    simServerDb;
    dbSYMADDRESS    simDataRepliesDb;
    dbSYMADDRESS    simDataRefDb;
    cmdPARAM_LIST   paramList;     // Command parameter list
    ic0DEBUG        debug;
    vltINT32        simDelay;      // reply delay simulation (in msec).
    ic0STATUS       status;
    ic0KEYWORD      setupKey;
    // table with database replies
    eccsDB_TABLE<ic0simREPLY_TABLE>   *simReplyTbl;  
    // cross reference table for keyword values 
    eccsDB_TABLE<ic0simDATAREF_TABLE> *simDataRefTbl;
    vltLOGICAL CheckAllParam(char *param);

  private:
    void Init(const dbSYMADDRESS dbRef);
};



// case unsensitive check if the parameter passed is ic0ALL
inline vltLOGICAL ic0simSERVER::CheckAllParam(char *param)
{
    unsigned int length = strlen(ic0ALL);

    if (strlen(param) == length)
	{
        char paramUC[length+1];
        strcpy(paramUC, param);
        slxStrToUpper(paramUC);
        char allUC[length+1];
        strcpy(allUC, ic0ALL);
        slxStrToUpper(allUC);
        if (strcmp(paramUC, allUC) == 0) return TRUE;
	}
    return FALSE;
}
#endif /*!IC0SIM_SERVER_H*/
