#ifndef iracqEVH_H
#define iracqEVH_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: iracqEVH.h,v 1.113 2005/09/13 07:27:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  28/05/99  created
* jstegmei  10/12/99  iracqSRV class is no longer public in iracqEVH
* jstegmei  13/12/99  support multiple server instances
*/

/************************************************************************
 *
 *  iracqEVH.h - acquisition control server class definition
 *
 *----------------------------------------------------------------------
 */


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "ECCS.h"

#include "evhHANDLER.h"
#include "evhTASK.h"
#include "evhFILEIO.h"
#include "evhTIMER.h"

#include "db.h"
#include "slx.h"

#include "iracqSRV.h"

#include "iracqErrors.h"

/*
 *  database points
 */
#define iracqDB_POINT         dbPoint_
#define iracqDB_IRACE         dbIrace_
#define iracqDB_EXPOSURE      dbExposure_
#define iracqDB_DETECTOR      dbDetector_
#define iracqDB_RTD           dbRtd_
#define iracqDB_CHOPPER       dbChopper_
   
/*
 *  maximum number of automatic database mirrors
 */
#define iracqDB_MAX_MIRRORS   1024

/*
 *  maximum number of WAIT requestors
 */
#define iracqMAX_WAIT   4

typedef struct
{
  vltBYTES64   nameLow;
  vltBYTES64   nameHigh;
  vltFLOAT     voltageLow;
  vltFLOAT     voltageHigh;
  vltINT32     dacChLow;
  vltINT32     dacChHigh;
  vltINT32     telChLow;
  vltINT32     telChHigh;
  vltFLOAT     range1Low;
  vltFLOAT     range1High;
  vltFLOAT     range2Low;
  vltFLOAT     range2High;
} iracqVOLT_CLK;

typedef struct
{
  vltBYTES64   name;
  vltFLOAT     voltage;
  vltINT32     dacCh;
  vltINT32     telCh;
  vltFLOAT     range1;
  vltFLOAT     range2;
} iracqVOLT_DC;

typedef struct
{
  vltBYTES64   name;
  vltINT32     id;
} iracqRM_RECORD;

typedef struct
{
  vltBYTES64   name;
  vltINT32     generate;
  vltINT32     store;
  vltINT32     disp;
  vltINT32     breakCnt;
} iracqFRM_RECORD;

typedef struct
{
  vltBYTES32   name;
  vltBYTES32   value;
} iracqPAR_RECORD;

typedef struct
{
  dbTYPE         type;
  vltINT32       size;
  void          *var;
  char           mirror[256];
  dbDIRADDRESS   addr;
  iracqSTAT2STR  map;
  dbATTRIBUTE    attrName;
} iracqDB_MIRROR;

typedef struct
{
  dbTYPE         type;
  char           paramName[32];
  dbATTRIBUTE    attrName;
  dbDIRADDRESS   addr;
} iracqDB_MAP;


class iracqEVH: public evhTASK, private iracqSRV
{
  public:
  iracqEVH();           // constructor  
  virtual ~iracqEVH();  // destructor 

  void                  PrintUsage(const char *);
  const char           *Instance();
  ccsCOMPL_STAT         ParseInputPars(int, char **);
  ccsCOMPL_STAT         ReplaceEnv(char *);
  int                   InitTest();

  // verbose callback
  void                  Verbose(const char *, ...);

  // server state callback
  void                  StateHandler(int, int);

  // data transfer task event callback
  void                  DataHandler(iracqDATA_EVT *);

  // exit callback
  void                  ExitHandler(int);

  // initialization routine
  ccsCOMPL_STAT         Init();
  void                  AbortServer();

  // command callback handling
  void                  EnterCB(msgMESSAGE &msg, vltINT32 updateSys);
  void                  EnterCB(msgMESSAGE &msg); 
  void                  ExitCB(msgMESSAGE &msg);
  ccsCOMPL_STAT         ExitCB();

  // command extension callbacks
  int                   ExtOnline(char *);
  int                   ExtStandby(char *);
  int                   ExtAbort(char *);
  virtual ccsCOMPL_STAT Online();
  virtual ccsCOMPL_STAT Standby();
  virtual ccsCOMPL_STAT Abort();

  // low-level IRACE command
  ccsCOMPL_STAT         IraceCmd(const char *, const char *);

  // frame control
  ccsCOMPL_STAT         FrameDefault(const char *, vltLOGICAL,
				     vltLOGICAL, vltLOGICAL);
  ccsCOMPL_STAT         AddFrame(vltINT32, const char *, vltLOGICAL, 
                                 vltLOGICAL, vltLOGICAL, char *);
  ccsCOMPL_STAT         BreakCnt(const char *, vltINT32);
  ccsCOMPL_STAT         FrameGen(const char *, vltLOGICAL);
  ccsCOMPL_STAT         FrameStore(const char *, vltLOGICAL);

  // irace program loops
  ccsCOMPL_STAT         ClearClkp();
  ccsCOMPL_STAT         LoadClkp(vltINT32, vltINT32 *, vltINT32);
  ccsCOMPL_STAT         LoadSeqProg(vltINT32 *, vltINT32);
  ccsCOMPL_STAT         FrameTime(vltINT32, vltFLOAT *);

  // sequencer mode
  ccsCOMPL_STAT         SeqMode(vltINT32);
  
  // exposure status
  vltLOGICAL            ExpActive();
  int                   ExpStatus();

  // state mapping
  void                  Stat2Str(char *, vltINT32, iracqSTAT2STR);
  
  // parameter interface
  ccsCOMPL_STAT         SetSysParam(char **, vltINT32);
  ccsCOMPL_STAT         SetSysParam(const char *, const char *, vltLOGICAL *);
  ccsCOMPL_STAT         SetSysParam(char *);
  ccsCOMPL_STAT         GetSysParam(const char *, char *);
  ccsCOMPL_STAT         GetSysParam(const char *, char *, vltINT32 *);
  ccsCOMPL_STAT         AddSysParam(const char *, vltINT32, void *, 
				    vltINT32, vltINT32);
  ccsCOMPL_STAT         ParamDefault();
  ccsCOMPL_STAT         ParamStore();
  ccsCOMPL_STAT         ParamRestore();
  ccsCOMPL_STAT         Param2Db(const char *, const dbSYMADDRESS, 
				 const dbATTRIBUTE);
  ccsCOMPL_STAT         AcqParam(const char *);
  char                 *AcqParam();
  ccsCOMPL_STAT         UsrCmd(const char *, char *);


  // parameter callbacks
  int                   ParamCallBack(const char *, const char *, 
				      int *, char *);
  int                   ParamStatCallBack(const char *, char *, int *, 
					 int *, char *);
  virtual ccsCOMPL_STAT SetParam(const char *, const char *, vltLOGICAL *);
  virtual ccsCOMPL_STAT GetParam(const char *, char *, vltLOGICAL *);
  virtual ccsCOMPL_STAT GetParam(const char *, char *, vltINT32 *, 
				 vltLOGICAL *);

  // sequencer program handler
  virtual ccsCOMPL_STAT SeqHandler(const char *, vltINT32 *, vltINT32 *);
  int                   SeqHandlerGen(const char *, int *, int *, char *);

  // alarm issue
  ccsCOMPL_STAT         Alarm(const char *);

  // FITS header
  ccsCOMPL_STAT         AddFits(const char *, int *, const char *);
  ccsCOMPL_STAT         AddFits(const char *, float *, const char *);
  ccsCOMPL_STAT         AddFits(const char *, double *, const char *);
  ccsCOMPL_STAT         AddFits(const char *, char *, const char *);
  ccsCOMPL_STAT         AddFits(const char *);
  ccsCOMPL_STAT         AddFitsDynPar(const char *, const char *);

  // download data file
  ccsCOMPL_STAT         DownloadData(vltINT32, vltINT32, const char *);
  ccsCOMPL_STAT         DownloadDataRaw(vltINT32, vltINT32, char *, vltINT32);

  // image post processing
  ccsCOMPL_STAT         ImgOp(vltINT32);

  // chopper 
  ccsCOMPL_STAT         InitChopper();

  protected:

  private:
  // callback structure
  evhOBJ_CALLBACK       cb_;
  void                 *paramValues_;
  cmdPARAM_LIST         paramList_;
  cmdPARAM_TYPE         paramType_;
  vltINT32              paramNb_;  
  char                  replyBuffer_[iracqMAX_PARAM_SIZE];
  vltBYTES64            orgName_;

  // file i/o handling
  evhFILEIO            *fioSrvCmd_;
  evhFILEIO            *fioSrvEvt_;
  evhFILEIO            *fioStdIn_;
  evhFILEIO            *fioAcqCmd_[iracqMAX_ACQ_DEV];
  evhFILEIO            *fioAcqErr_[iracqMAX_ACQ_DEV];
  evhFILEIO            *fioCtrl_;
  evhFILEIO            *fioData_;
  vltINT32              numAcq_;

   // direct database addresses
  dbDIRADDRESS          dbAddrDcVolt_;
  dbDIRADDRESS          dbAddrClkVolt_;
  dbDIRADDRESS          dbAddrActionLog_;
  dbDIRADDRESS          dbAddrFileHistory_;
  dbDIRADDRESS          dbAddrUpdate_;
  dbDIRADDRESS          dbAddrAlarm_;
  dbDIRADDRESS          dbAddrExpCountDown_;
  dbDIRADDRESS          dbAddrOffset_;

  // database mirror
  iracqDB_MIRROR        mirrorList_[iracqDB_MAX_MIRRORS];
  vltINT32              mirrorLen_;
  vltINT32              mirrorEleSubState_;
  vltINT32              mirrorEleSubStateN_;
  vltINT32              mirrorEleExpState_;
  vltINT32              mirrorEleExpStateN_;
  vltINT32              mirrorEleDtState_;
  iracqDB_MAP           dbMap_[iracqMAX_PARAM];
  vltINT32              dbMapLen_;
  char                  dbPoint_[64];
  char                  dbIrace_[64];
  char                  dbExposure_[64];
  char                  dbDetector_[64];
  char                  dbRtd_[64];
  char                  dbChopper_[64];

  // global flags
  vltINT32              srvInitDone_;
  vltINT32              dbInitDone_;
  vltINT32              logOnStdout_;
  vltINT32              actionLogOn_;
  vltINT32              subState_;
  vltINT32              updateSys_;

  // exposure countdown
  ccsTIMEVAL            expStartTime_;
  vltINT32              expCountDown_;
  vltINT32              expCountDownStart_;
  evhTIMER             *expCountDownTimerCB_;
  vltINT32              expStatus_;

  // status polling
  vltLOGICAL            pollStatEnabled_;
  evhTIMER             *pollStatTimerCB_;
  evhTIMER             *pollPerfTimerCB_;
  evhCB_COMPL_STAT      PollStatCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      PollPerfCB_(msgMESSAGE &msg, void *udata);
  evhTIMER             *PollStart_(vltINT32, evhCB_METHOD2);
  ccsCOMPL_STAT         PollStop_(evhTIMER *);

  // dynamic parameters
  vltINT32              numDynPar_;
  char                  emptyString_[4];

  // WAIT requestors
  msgMESSAGE            waitMsg_[iracqMAX_WAIT];
  vltINT32              numWaitMsg_;

  // file i/o callbacks
  ccsCOMPL_STAT         UpdateFd_();
  ccsCOMPL_STAT         UpdateCtrl_(char, int);
  ccsCOMPL_STAT         UpdateData_(char, int);
  ccsCOMPL_STAT         UpdateAcqCmd_(char, int, int);
  ccsCOMPL_STAT         UpdateAcqErr_(char, int, int);
  evhCB_COMPL_STAT      SrvCmdCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      SrvEvtCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      SockCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      EvtCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      DataCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      CtrlCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      AcqCmdCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      AcqErrCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      StdinCB_(msgMESSAGE &msg, void *udata);

  // command callbacks
  void                  DiscardDetIdx_(char *);
  ccsCOMPL_STAT         CmdCallback_(const char *, evhCB_METHOD2);
  ccsCOMPL_STAT         AddCmdCallbacks_();
  evhCB_COMPL_STAT      OnlineCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      StandbyCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      ResetCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      OffCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      StartCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      WaitCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      AbortCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      EndCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      PauseCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      ContCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      SimulatCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      StopSimCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      SeqCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      CldcCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      SetupCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      StatusCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      CommentCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      MiscCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      NcCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      VersionCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      FrameCB_(msgMESSAGE &msg, void *udata);
  evhCB_COMPL_STAT      ImgOpCB_(msgMESSAGE &msg, void *udata);

  // irace low-level callbacks
  evhCB_COMPL_STAT      IraceCB_(msgMESSAGE &msg, void *udata);

  // exposure countdown
  evhCB_COMPL_STAT      ExpCountDownCB_(msgMESSAGE &msg, void *udata);
  ccsCOMPL_STAT         StartExpCountDown_();
  ccsCOMPL_STAT         StopExpCountDown_();
  ccsCOMPL_STAT         CheckExpCountDown_();
  
  // database mirror
  ccsCOMPL_STAT         DbInitAddr_();
  ccsCOMPL_STAT         DbInitMirror_();
  vltINT32              DbMirror_(const char *, const char *, void *);
  vltINT32              DbMirror_(const char *, const char *, void *, 
				  iracqSTAT2STR);
  vltINT32              DbMirror_(const char *, void *);
  vltINT32              DbMirror_(const char *, void *, iracqSTAT2STR);
  ccsCOMPL_STAT         DbMirrorUpdateEle_(vltINT32);
  ccsCOMPL_STAT         DbMirrorUpdate_();
  ccsCOMPL_STAT         DbUpdateAdc_();
  ccsCOMPL_STAT         DbUpdateVoltAll_();
  ccsCOMPL_STAT         DbUpdateRm_();
  ccsCOMPL_STAT         DbUpdateParam_();
  ccsCOMPL_STAT         DbUpdateFrames_();
  ccsCOMPL_STAT         DbUpdateSubState_();
  ccsCOMPL_STAT         DbUpdateExpState_();
  ccsCOMPL_STAT         DbFileHistory_(const char *);
  ccsCOMPL_STAT         DbActionLog_(const char *);
  ccsCOMPL_STAT         DbOffset_(double, double);
  ccsCOMPL_STAT         DbUpdateCfg_(const char *);
  ccsCOMPL_STAT         UpdateSystem_(vltINT32, const char *);
  ccsCOMPL_STAT         DbRecover_();
  ccsCOMPL_STAT         DbDirectWrite_(dbDIRADDRESS *, vltINT32);
  ccsCOMPL_STAT         DbDirectWrite_(dbDIRADDRESS *, vltFLOAT);
  ccsCOMPL_STAT         DbAlarm_(const char *);
  ccsCOMPL_STAT         DbDynParam_(const char *, const dbSYMADDRESS, 
				    const dbATTRIBUTE);
  vltLOGICAL            dbLocked_;
};


#endif /*!iracqEVH_H*/
