#ifndef issCALCULATOR_H
#define issCALCULATOR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issCALCULATOR.h,v 2.21 2005/06/03 15:38:25 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2003-10-20 added methods ComputeTadcShift() and GetTelId()
* jott     2003-09-19 added file header comment
* rrollf    02/04/02  created
*/

/************************************************************************
 * Header file for issCALCULATOR class which can be commonly used by all
 * ISS processes to compute often used parameters of general interest
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "eccsErr.h"

#include "evhDB_TASK.h"
#include "evhCOMMAND.h"
#include "tcs.h"
#include "issCalc.h"

class issCALCULATOR : public eccsERROR_CLASS
    {
    public:
        issCALCULATOR();

        ~issCALCULATOR();

        ccsCOMPL_STAT   Init(dbSYMADDRESS vltiBase = ":Appl_data:VLTI",
                             vltDOUBLE    latitude  = -0.429792176,
                             vltDOUBLE    longitude =  1.228750578);

        ccsCOMPL_STAT   GetAltAz(vltDOUBLE   *alt,
                                 vltDOUBLE   *az);

        ccsCOMPL_STAT   GetAltAz(ccsTIMEVAL   utc,
                                 vltDOUBLE    alpha,
                                 vltDOUBLE    delta,
                                 vltDOUBLE   *alt,
                                 vltDOUBLE   *az);

        ccsCOMPL_STAT   GetDlPos(issOPL_DL   *oplDl);

        ccsCOMPL_STAT   GetDlPos(vltINT32     refDl,
                                 vltDOUBLE    refOpl,
                                 issOPL_DL   *oplDl);

        ccsCOMPL_STAT   NewConfigTable(void){mNewConfigTable = ccsTRUE; return SUCCESS;};

        ccsCOMPL_STAT   ComputeTadcShift(issCALC_TADC_IN_DATA *input,
                                         vltDOUBLE            *dispMagArcsec);

        ccsCOMPL_STAT   GetTelId(vltINT32 beam, int *telId);


    private:

        /* Private methods */
        ccsCOMPL_STAT       ReadConfigTable(void);

        /* Constants */
        vltDOUBLE   mLatitude;
        vltDOUBLE   mLongitude;
        vltDOUBLE   mRadToDeg;     /* convert radians to degrees */


        /* Input values */
        ccsTIMEVAL  mUtc;                       /* actual time  */
        vltDOUBLE   mAlpha, mDelta;             /* RA and DEC   */

        vltINT32    mRefDl;                     /* reference delay line */
        vltDOUBLE   mRefOpl;                    /* reference OPL        */

        /* Computed values */
        vltDOUBLE   mHourAngle;                 /* hour angle in HMS format */
        vltDOUBLE   mAltRad;
        vltDOUBLE   mAzRad;

        issOPL_DL   mOplDL;


        /* Support */
        dbSYMADDRESS        mVltiBasePoint;
        issVLTI_CFG_TABLE   mVltiCfgTable;
        dbSYMADDRESS        mCfgTabSymAddr;
        dbDIRADDRESS        mCfgTabDirAddr;


        /* Flags */
        vltLOGICAL  mInitDone;
        vltLOGICAL  mAltAzOk;
        vltLOGICAL  mDlPosOk;
        vltLOGICAL  mNewConfigTable;

    };

#endif /* !issCALCULATOR_H */
