#ifndef issSBS_DB_DEFINES_H
#define issSBS_DB_DEFINES_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issSbsDbDefines.h,v 2.21 2005/06/03 15:38:26 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-02-18 Added issSBS_VALID_MACAO_DATA (SPR VLTSW20050036)
* jott     2005-01-19 Added definitions issATx_NFD_POS and issSBS_NFD_POS
* jott     2004-06-23 Added definitions iss<TEL>_CHOPGUIDE
*                     (SPR VLTSW20040164)
* jott     2004-05-14 Added definitions for OPL offsets, issSBS_DLCS_OPLOFFS1
*                     etc. (SPR VLTSW20040089)
* jott     2004-02-12 Added support for delay lines 4 to 6 (SPR VLTSW20030502)
* rrollf    06/08/03  Moved issNUM_OF_TEL_ATTR and issNUM_OF_DEL_ATTR to module
*                     isstrkws.
* rrollf    06/05/03  Changed definition of issSBS_DLCS_REF_MODE from
*                     "dlcsState.dummyRefMode" to "dlcsState.refMode"
* jott      02/04/03  Added definitions for M16 positions to solve SPR
*                     VLTSW20020683
*                     Set issNUM_OF_DEL_ATTR to 16 to solve SPR VLTSW20030106
* rrollf    10/09/02  Added defines for actual position for nasmyth device:
*                     "subSystems:wtXtcs:beacon.adfnPos"
* jott      07/08/02  added definitons issFNT_PUB_DATA_xxx for FINITO public
*                     database point
* rrollf    26/07/02  Added a check of tif version for the defines
*                     issSBS_COORD_RA2000_LOCAL and issSBS_COORD_DEC2000_LOCAL
*                     (see the Makefile of issprs)
* rrollf    18/07/02  Added defines for actual position (right ascension and
*                     declination) used by the REMEMB command
*                     (see SPR VLTSW20020299)
* jott      11/06/02  adapted to APR2003 and gcc 3.0.4
* rrollf    16/01/02  Added defines for UT flags:
*                     "subSystems:wtXtcs:guide.status",
*                     "subSystems:wtXtcs:guide.mode" and
*                     "subSystems:wtXtcs:ao.cyclaoM1M2Ok"
* jott      15/01/02  Added definition issSBS_STRAP_FILTER to solve
*                     SPR VLTSW20010783
* rrollf    12/11/01  Added defines: issDLCS_TEL1_XYZ .. issDLCS_TEL4_XYZ
*                     Set issNUM_OF_DEL_ATTR to 15
* rrollf    25/10/01  Set issNUM_OF_DEL_ATTR to 11 (+ ra2000 & dec2000)
* jott      05/10/01  completed definitions for issDLCS_DEC2000,
*                     issDLCS_RA2000, issDLCS_OPL_OFFSPRE, issSBS_DLCS_RA2000,
*                     issSBS_DLCS_DEC2000, issSBS_DLCS_OPL_OFFSPRE, and
*                     issSBS_DLCS_OPL_OFFSEXT
*
* jott      21/09/01  added definitons for
* rrollf    04/09/01  added prltic, dvelra, dveldec, probeInPos
*                     changed issNUM_OF_TEL_ATTR to 30
*                     changed issSBS_CGS_STATUS to issSBS_GUIDING_STATE
* jott      30/07/01  added definitions issCOUx_FOCUS, issUTx_FOCUS, and
*                     issATx_LOCATION and first time referenced defintions
* rrollf    09/01/01  created
*/

/************************************************************************
 * Definitions of subSystem branches of VLTI ISS
 *----------------------------------------------------------------------
 */
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/* Includes */
#include "iss.h"

/*
 * Definitions for database attribute addresses of subsystem branch
 * needed by isstrkws, issprs
 */

#define issSUB_SYSTEMS  "subSystems"    /* Name of subSystem point */

#define issSBS_NONE     ""
#define issSBS_SID1     "watsid1"       /* Name of watsid1 (SID1)   */
#define issSBS_SID2     "watsid2"       /* Name of watsid2 (SID2)   */
#define issSBS_UT1      "wt1tcs"        /* Name of wt1tcs  (UT1)    */
#define issSBS_UT2      "wt2tcs"        /* Name of wt2tcs  (UT2)    */
#define issSBS_UT3      "wt3tcs"        /* Name of wt3tcs  (UT3)    */
#define issSBS_UT4      "wt4tcs"        /* Name of wt4tcs  (UT4)    */
#define issSBS_AT1      "wat1tcs"       /* Name of wat1tcs (AT1)    */
#define issSBS_AT2      "wat2tcs"       /* Name of wat2tcs (AT2)    */
#define issSBS_AT3      "wat3tcs"       /* Name of wat3tcs (AT3)    */
#define issSBS_DLCS     "wdline"        /* Name of wdline  (DLCS)   */
#define issSBS_COU1     "wop1"          /* Name of wop1    (COU1)   */
#define issSBS_COU2     "wop2"          /* Name of wop2    (COU2)   */
#define issSBS_COU3     "wop3"          /* Name of wop3    (COU3)   */
#define issSBS_COU4     "wop4"          /* Name of wop4    (COU4)   */

/* Attribute names of subSystem data */
#define issSBS_TIMES_LST            "times.lst"
#define issSBS_TIMES_UTC            "times.utc"
#define issSBS_COORD_HA             "coord.ha"
#define issSBS_COORD_RA             "coord.ra"
#define issSBS_COORD_DEC            "coord.dec"
#define issSBS_COORD_RA2000         "coord.ra2000"
#define issSBS_COORD_DEC2000        "coord.dec2000"

/*
 * Actual position for UT's
 * Since tif version 1.300 the actual position for UT's
 * is stored in 'raLocalTel2000' and 'decLocalTel2000'
 * The defines TIF_VER_HI and TIF_VER_LO are set in the
 * Makefile of issprs, which includes issSbsDbDefines.h
 */
#define issSBS_COORD_RA2000_LOCAL   issSBS_COORD_RA2000
#define issSBS_COORD_DEC2000_LOCAL  issSBS_COORD_DEC2000
#ifdef TIF_VER_HI
  #if TIF_VER_HI > 1
    #undef issSBS_COORD_RA2000_LOCAL
    #undef issSBS_COORD_DEC2000_LOCAL
    #define issSBS_COORD_RA2000_LOCAL   "coord.raLocalTel2000"
    #define issSBS_COORD_DEC2000_LOCAL  "coord.decLocalTel2000"
  #endif
#endif
#ifdef TIF_VER_LO
  #if TIF_VER_LO > 129
    #undef issSBS_COORD_RA2000_LOCAL
    #undef issSBS_COORD_DEC2000_LOCAL
    #define issSBS_COORD_RA2000_LOCAL   "coord.raLocalTel2000"
    #define issSBS_COORD_DEC2000_LOCAL  "coord.decLocalTel2000"
  #endif
#endif

/*
 * Actual position for SID's
 * Since sidtif version 1.8 the actual position for SID's
 * is stored in 'raLocalTel2000' and 'decLocalTel2000'
 * The defines SIDTIF_VER_HI and SIDTIF_VER_LO are set in the
 * Makefile of issprs, which includes issSbsDbDefines.h
 */
#define issSBS_COORD_RA2000_LOCAL_SID   issSBS_COORD_RA2000
#define issSBS_COORD_DEC2000_LOCAL_SID  issSBS_COORD_DEC2000
#ifdef SIDTIF_VER_HI
  #if SIDTIF_VER_HI > 1
    #undef issSBS_COORD_RA2000_LOCAL_SID
    #undef issSBS_COORD_DEC2000_LOCAL_SID
    #define issSBS_COORD_RA2000_LOCAL_SID   "coord.raLocalTel2000"
    #define issSBS_COORD_DEC2000_LOCAL_SID  "coord.decLocalTel2000"
  #endif
#endif
#ifdef SIDTIF_VER_LO
  #if SIDTIF_VER_LO > 8
    #undef issSBS_COORD_RA2000_LOCAL_SID
    #undef issSBS_COORD_DEC2000_LOCAL_SID
    #define issSBS_COORD_RA2000_LOCAL_SID   "coord.raLocalTel2000"
    #define issSBS_COORD_DEC2000_LOCAL_SID  "coord.decLocalTel2000"
  #endif
#endif

#define issSBS_COORD_RA2000DEG      "coord.ra2000deg"
#define issSBS_COORD_DEC2000DEG     "coord.dec2000deg"
#define issSBS_COORD_ALT            "coord.alt"
#define issSBS_COORD_AZ             "coord.az"
#define issSBS_COORD_PRLTIC         "coord.prltic"
#define issSBS_TRACK_REMTIME        "track.remtime"
#define issSBS_TRACK_AIRMASS        "track.airmass"
#define issSBS_TRACK_DVELRA         "track.dvelra"
#define issSBS_TRACK_DVELDEC        "track.dveldec"
#define issSBS_CGS_RA2000           "coude.probeRA"
#define issSBS_CGS_DEC2000          "coude.probeDEC"
#define issSBS_CGS_MODE             "coude.guidingMode"
#define issSBS_CGS_MAG              "coude.mag"
#define issSBS_CGS_WAVELEN          "coude.wavelen"
#define issSBS_CGS_PIP              "coude.probeInPos"
#define issSBS_CHOP_FREQ            "chop.freq"
#define issSBS_CHOP_STATUS          "chop.status"
#define issSBS_CHOP_STARTTIME       "chop.startTime"
#define issSBS_CHOP_THROW           "chop.throw"
#define issSBS_CHOP_PVR             "chop.peakValleyRatio"
#define issSBS_CHOP_POSANG          "chop.posAng"
#define issSBS_CHOP_GUIDE           "chop.guide"
#define issSBS_GUIDING_STATE        "coude.guidingState"
#define issSBS_TCSSTATE_TRACK       "tcsState.track"
#define issSBS_STRAP_FILTER         "macv:strapStatus.ndFilter"
#define issSBS_VALID_MACAO_DATA     "macv:rtcStatus:observation.validData"


/* Special attributes of UTs  */
#define issSBS_NAS_GS_RA2000        "guide.ra"
#define issSBS_NAS_GS_DEC2000       "guide.dec"
#define issSBS_NAS_STATUS           "guide.status"
#define issSBS_NAS_MODE             "guide.mode"
#define issSBS_AO_CYCLM1M2          "ao.cyclaoM1M2Ok"
#define issSBS_TCSSTATE_FOCUS       "tcsState.focus"
#define issSBS_BEACON_ADFNPOS       "beacon.adfnPos"

/* Special attributes of ATs */
#define issSBS_TCSSTATE_LOCATION    "site.station"
#define issSBS_NFD_POS              "tcsState.nfdPosName"

/* Special attributes of DLCS */
#define issSBS_DLCS_TRK_STATE       "dlcsState.trkOkFlag"
#define issSBS_DLCS_STATE           "dlcsState.dlcsState"
#define issSBS_DLCS_SUB_STATE       "dlcsState.dlcsSubstate"
#define issSBS_DLCS_REM_TRK_TIME    "dlcsState.remTrkTime"
#define issSBS_DLCS_MOTION_PHASE    "dlcsState.motionPhase"
#define issSBS_DLCS_REF_DL          "dlcsState.refDl"
#define issSBS_DLCS_REF_OPL         "dlcsState.refOpl"
#define issSBS_DLCS_REF_MODE        "dlcsState.refMode"
#define issSBS_DLCS_RA2000          "coord.ra2000"
#define issSBS_DLCS_DEC2000         "coord.dec2000"
#define issSBS_DLCS_OPL1            "dl1.measPos"
#define issSBS_DLCS_OPL2            "dl2.measPos"
#define issSBS_DLCS_OPL3            "dl3.measPos"
#define issSBS_DLCS_OPL4            "dl4.measPos"
#define issSBS_DLCS_OPL5            "dl5.measPos"
#define issSBS_DLCS_OPL6            "dl6.measPos"
#define issSBS_DLCS_OPLOFFS1        "coord.opl1Offset"
#define issSBS_DLCS_OPLOFFS2        "coord.opl2Offset"
#define issSBS_DLCS_OPLOFFS3        "coord.opl3Offset"
#define issSBS_DLCS_OPLOFFS4        "coord.opl4Offset"
#define issSBS_DLCS_OPLOFFS5        "coord.opl5Offset"
#define issSBS_DLCS_OPLOFFS6        "coord.opl6Offset"



#define issSBS_DLCS_REFBEAM         "dlcsState.refDl"
#define issSBS_DLCS_TEL1_XYZ        "position.dld1xyz"
#define issSBS_DLCS_TEL2_XYZ        "position.dld2xyz"
#define issSBS_DLCS_TEL3_XYZ        "position.dld3xyz"
#define issSBS_DLCS_TEL4_XYZ        "position.dld4xyz"
#define issSBS_DLCS_TEL5_XYZ        "position.dld5xyz"
#define issSBS_DLCS_TEL6_XYZ        "position.dld6xyz"

#define issSBS_DLCS_OPL_OFFSPRE     "coord.opl"
#define issSBS_DLCS_OPL_OFFSEXT     "Offset"


/* Help function */
#define zcat(x,y) x y

/* DB address of subSystem point (:Appl_data:VLTI:subSystems)*/
#define issSBS   zcat(zcat(tcsVLTI_DBPOINT_DEF,":"),issSUB_SYSTEMS)

/* DB address of the telescopes */
#define issSID1 zcat(zcat(issSBS,":"),issSBS_SID1)
#define issSID2 zcat(zcat(issSBS,":"),issSBS_SID2)
#define issUT1  zcat(zcat(issSBS,":"),issSBS_UT1)
#define issUT2  zcat(zcat(issSBS,":"),issSBS_UT2)
#define issUT3  zcat(zcat(issSBS,":"),issSBS_UT3)
#define issUT4  zcat(zcat(issSBS,":"),issSBS_UT4)
#define issAT1  zcat(zcat(issSBS,":"),issSBS_AT1)
#define issAT2  zcat(zcat(issSBS,":"),issSBS_AT2)
#define issAT3  zcat(zcat(issSBS,":"),issSBS_AT3)
#define issCOU1 zcat(zcat(issSBS,":"),issSBS_COU1)
#define issCOU2 zcat(zcat(issSBS,":"),issSBS_COU2)
#define issCOU3 zcat(zcat(issSBS,":"),issSBS_COU3)
#define issCOU4 zcat(zcat(issSBS,":"),issSBS_COU4)
#define issDLCS zcat(zcat(issSBS,":"),issSBS_DLCS)


/* SID1 */
#define issSID1_LST             zcat(zcat(issSID1,":"),issSBS_TIMES_LST)
#define issSID1_UTC             zcat(zcat(issSID1,":"),issSBS_TIMES_UTC)
#define issSID1_HA              zcat(zcat(issSID1,":"),issSBS_COORD_HA)
#define issSID1_RA              zcat(zcat(issSID1,":"),issSBS_COORD_RA)
#define issSID1_DEC             zcat(zcat(issSID1,":"),issSBS_COORD_DEC)
#define issSID1_RA2000          zcat(zcat(issSID1,":"),issSBS_COORD_RA2000)
#define issSID1_DEC2000         zcat(zcat(issSID1,":"),issSBS_COORD_DEC2000)
#define issSID1_RA2000DEG       zcat(zcat(issSID1,":"),issSBS_COORD_RA2000DEG)
#define issSID1_DEC2000DEG      zcat(zcat(issSID1,":"),issSBS_COORD_DEC2000DEG)
#define issSID1_ALT             zcat(zcat(issSID1,":"),issSBS_COORD_ALT)
#define issSID1_AZ              zcat(zcat(issSID1,":"),issSBS_COORD_AZ)
#define issSID1_PRLTIC          zcat(zcat(issSID1,":"),issSBS_COORD_PRLTIC)
#define issSID1_REMTIME         zcat(zcat(issSID1,":"),issSBS_TRACK_REMTIME)
#define issSID1_AIRMASS         zcat(zcat(issSID1,":"),issSBS_TRACK_AIRMASS)
#define issSID1_DVELRA          zcat(zcat(issSID1,":"),issSBS_TRACK_DVELRA)
#define issSID1_DVELDEC         zcat(zcat(issSID1,":"),issSBS_TRACK_DVELDEC)
#define issSID1_CGS_RA2000      zcat(zcat(issSID1,":"),issSBS_CGS_RA2000)
#define issSID1_CGS_DEC2000     zcat(zcat(issSID1,":"),issSBS_CGS_DEC2000)
#define issSID1_CGS_MODE        zcat(zcat(issSID1,":"),issSBS_CGS_MODE)
#define issSID1_CGS_MAG         zcat(zcat(issSID1,":"),issSBS_CGS_MAG)
#define issSID1_CGS_WAVELEN     zcat(zcat(issSID1,":"),issSBS_CGS_WAVELEN)
#define issSID1_CGS_PIP         zcat(zcat(issSID1,":"),issSBS_CGS_PIP)
#define issSID1_FREQ            zcat(zcat(issSID1,":"),issSBS_CHOP_FREQ)
#define issSID1_STATUS          zcat(zcat(issSID1,":"),issSBS_CHOP_STATUS)
#define issSID1_STARTTIME       zcat(zcat(issSID1,":"),issSBS_CHOP_STARTTIME)
#define issSID1_THROW           zcat(zcat(issSID1,":"),issSBS_CHOP_THROW)
#define issSID1_PVR             zcat(zcat(issSID1,":"),issSBS_CHOP_PVR)
#define issSID1_POSANG          zcat(zcat(issSID1,":"),issSBS_CHOP_POSANG)
#define issSID1_CHOPGUIDE       zcat(zcat(issSID1,":"),issSBS_CHOP_GUIDE)
#define issSID1_GUIDING_STATE   zcat(zcat(issSID1,":"),issSBS_GUIDING_STATE)
#define issSID1_TRACKSTATE      zcat(zcat(issSID1,":"),issSBS_TCSSTATE_TRACK)

/* SID2 */
#define issSID2_LST             zcat(zcat(issSID2,":"),issSBS_TIMES_LST)
#define issSID2_UTC             zcat(zcat(issSID2,":"),issSBS_TIMES_UTC)
#define issSID2_HA              zcat(zcat(issSID2,":"),issSBS_COORD_HA)
#define issSID2_RA              zcat(zcat(issSID2,":"),issSBS_COORD_RA)
#define issSID2_DEC             zcat(zcat(issSID2,":"),issSBS_COORD_DEC)
#define issSID2_RA2000          zcat(zcat(issSID2,":"),issSBS_COORD_RA2000)
#define issSID2_DEC2000         zcat(zcat(issSID2,":"),issSBS_COORD_DEC2000)
#define issSID2_RA2000DEG       zcat(zcat(issSID2,":"),issSBS_COORD_RA2000DEG)
#define issSID2_DEC2000DEG      zcat(zcat(issSID2,":"),issSBS_COORD_DEC2000DEG)
#define issSID2_ALT             zcat(zcat(issSID2,":"),issSBS_COORD_ALT)
#define issSID2_AZ              zcat(zcat(issSID2,":"),issSBS_COORD_AZ)
#define issSID2_PRLTIC          zcat(zcat(issSID2,":"),issSBS_COORD_PRLTIC)
#define issSID2_REMTIME         zcat(zcat(issSID2,":"),issSBS_TRACK_REMTIME)
#define issSID2_AIRMASS         zcat(zcat(issSID2,":"),issSBS_TRACK_AIRMASS)
#define issSID2_DVELRA          zcat(zcat(issSID2,":"),issSBS_TRACK_DVELRA)
#define issSID2_DVELDEC         zcat(zcat(issSID2,":"),issSBS_TRACK_DVELDEC)
#define issSID2_CGS_RA2000      zcat(zcat(issSID2,":"),issSBS_CGS_RA2000)
#define issSID2_CGS_DEC2000     zcat(zcat(issSID2,":"),issSBS_CGS_DEC2000)
#define issSID2_CGS_MODE        zcat(zcat(issSID2,":"),issSBS_CGS_MODE)
#define issSID2_CGS_MAG         zcat(zcat(issSID2,":"),issSBS_CGS_MAG)
#define issSID2_CGS_WAVELEN     zcat(zcat(issSID2,":"),issSBS_CGS_WAVELEN)
#define issSID2_CGS_PIP         zcat(zcat(issSID2,":"),issSBS_CGS_PIP)
#define issSID2_FREQ            zcat(zcat(issSID2,":"),issSBS_CHOP_FREQ)
#define issSID2_STATUS          zcat(zcat(issSID2,":"),issSBS_CHOP_STATUS)
#define issSID2_STARTTIME       zcat(zcat(issSID2,":"),issSBS_CHOP_STARTTIME)
#define issSID2_THROW           zcat(zcat(issSID2,":"),issSBS_CHOP_THROW)
#define issSID2_PVR             zcat(zcat(issSID2,":"),issSBS_CHOP_PVR)
#define issSID2_POSANG          zcat(zcat(issSID2,":"),issSBS_CHOP_POSANG)
#define issSID2_CHOPGUIDE       zcat(zcat(issSID2,":"),issSBS_CHOP_GUIDE)
#define issSID2_GUIDING_STATE   zcat(zcat(issSID2,":"),issSBS_GUIDING_STATE)
#define issSID2_TRACKSTATE      zcat(zcat(issSID2,":"),issSBS_TCSSTATE_TRACK)

/* UT1 */
#define issUT1_LST              zcat(zcat(issUT1,":"),issSBS_TIMES_LST)
#define issUT1_UTC              zcat(zcat(issUT1,":"),issSBS_TIMES_UTC)
#define issUT1_HA               zcat(zcat(issUT1,":"),issSBS_COORD_HA)
#define issUT1_RA               zcat(zcat(issUT1,":"),issSBS_COORD_RA)
#define issUT1_DEC              zcat(zcat(issUT1,":"),issSBS_COORD_DEC)
#define issUT1_RA2000           zcat(zcat(issUT1,":"),issSBS_COORD_RA2000)
#define issUT1_DEC2000          zcat(zcat(issUT1,":"),issSBS_COORD_DEC2000)
#define issUT1_RA2000DEG        zcat(zcat(issUT1,":"),issSBS_COORD_RA2000DEG)
#define issUT1_DEC2000DEG       zcat(zcat(issUT1,":"),issSBS_COORD_DEC2000DEG)
#define issUT1_ALT              zcat(zcat(issUT1,":"),issSBS_COORD_ALT)
#define issUT1_AZ               zcat(zcat(issUT1,":"),issSBS_COORD_AZ)
#define issUT1_PRLTIC           zcat(zcat(issUT1,":"),issSBS_COORD_PRLTIC)
#define issUT1_REMTIME          zcat(zcat(issUT1,":"),issSBS_TRACK_REMTIME)
#define issUT1_AIRMASS          zcat(zcat(issUT1,":"),issSBS_TRACK_AIRMASS)
#define issUT1_DVELRA           zcat(zcat(issUT1,":"),issSBS_TRACK_DVELRA)
#define issUT1_DVELDEC          zcat(zcat(issUT1,":"),issSBS_TRACK_DVELDEC)
#define issUT1_CGS_RA2000       zcat(zcat(issUT1,":"),issSBS_CGS_RA2000)
#define issUT1_CGS_DEC2000      zcat(zcat(issUT1,":"),issSBS_CGS_DEC2000)
#define issUT1_CGS_MODE         zcat(zcat(issUT1,":"),issSBS_CGS_MODE)
#define issUT1_CGS_MAG          zcat(zcat(issUT1,":"),issSBS_CGS_MAG)
#define issUT1_CGS_WAVELEN      zcat(zcat(issUT1,":"),issSBS_CGS_WAVELEN)
#define issUT1_CGS_PIP          zcat(zcat(issUT1,":"),issSBS_CGS_PIP)
#define issUT1_FREQ             zcat(zcat(issUT1,":"),issSBS_CHOP_FREQ)
#define issUT1_STATUS           zcat(zcat(issUT1,":"),issSBS_CHOP_STATUS)
#define issUT1_STARTTIME        zcat(zcat(issUT1,":"),issSBS_CHOP_STARTTIME)
#define issUT1_THROW            zcat(zcat(issUT1,":"),issSBS_CHOP_THROW)
#define issUT1_PVR              zcat(zcat(issUT1,":"),issSBS_CHOP_PVR)
#define issUT1_POSANG           zcat(zcat(issUT1,":"),issSBS_CHOP_POSANG)
#define issUT1_CHOPGUIDE        zcat(zcat(issUT1,":"),issSBS_CHOP_GUIDE)
#define issUT1_GUIDING_STATE    zcat(zcat(issUT1,":"),issSBS_GUIDING_STATE)
#define issUT1_TRACKSTATE       zcat(zcat(issUT1,":"),issSBS_TCSSTATE_TRACK)

/* special for UTs */
#define issUT1_NAS_GS_RA2000    zcat(zcat(issUT1,":"),issSBS_NAS_GS_RA2000)
#define issUT1_NAS_GS_DEC2000   zcat(zcat(issUT1,":"),issSBS_NAS_GS_DEC2000)
#define issUT1_NAS_STATUS       zcat(zcat(issUT1,":"),issSBS_NAS_STATUS)
#define issUT1_NAS_MODE         zcat(zcat(issUT1,":"),issSBS_NAS_MODE)
#define issUT1_AO_CYCLM1M2      zcat(zcat(issUT1,":"),issSBS_AO_CYCLM1M2)
#define issUT1_FOCUS            zcat(zcat(issUT1,":"),issSBS_TCSSTATE_FOCUS)
#define issUT1_BEACON_ADFNPOS   zcat(zcat(issUT1,":"),issSBS_BEACON_ADFNPOS)

/* UT2 */
#define issUT2_LST              zcat(zcat(issUT2,":"),issSBS_TIMES_LST)
#define issUT2_UTC              zcat(zcat(issUT2,":"),issSBS_TIMES_UTC)
#define issUT2_HA               zcat(zcat(issUT2,":"),issSBS_COORD_HA)
#define issUT2_RA               zcat(zcat(issUT2,":"),issSBS_COORD_RA)
#define issUT2_DEC              zcat(zcat(issUT2,":"),issSBS_COORD_DEC)
#define issUT2_RA2000           zcat(zcat(issUT2,":"),issSBS_COORD_RA2000)
#define issUT2_DEC2000          zcat(zcat(issUT2,":"),issSBS_COORD_DEC2000)
#define issUT2_RA2000DEG        zcat(zcat(issUT2,":"),issSBS_COORD_RA2000DEG)
#define issUT2_DEC2000DEG       zcat(zcat(issUT2,":"),issSBS_COORD_DEC2000DEG)
#define issUT2_ALT              zcat(zcat(issUT2,":"),issSBS_COORD_ALT)
#define issUT2_AZ               zcat(zcat(issUT2,":"),issSBS_COORD_AZ)
#define issUT2_PRLTIC           zcat(zcat(issUT2,":"),issSBS_COORD_PRLTIC)
#define issUT2_REMTIME          zcat(zcat(issUT2,":"),issSBS_TRACK_REMTIME)
#define issUT2_AIRMASS          zcat(zcat(issUT2,":"),issSBS_TRACK_AIRMASS)
#define issUT2_DVELRA           zcat(zcat(issUT2,":"),issSBS_TRACK_DVELRA)
#define issUT2_DVELDEC          zcat(zcat(issUT2,":"),issSBS_TRACK_DVELDEC)
#define issUT2_CGS_RA2000       zcat(zcat(issUT2,":"),issSBS_CGS_RA2000)
#define issUT2_CGS_DEC2000      zcat(zcat(issUT2,":"),issSBS_CGS_DEC2000)
#define issUT2_CGS_MODE         zcat(zcat(issUT2,":"),issSBS_CGS_MODE)
#define issUT2_CGS_MAG          zcat(zcat(issUT2,":"),issSBS_CGS_MAG)
#define issUT2_CGS_WAVELEN      zcat(zcat(issUT2,":"),issSBS_CGS_WAVELEN)
#define issUT2_CGS_PIP          zcat(zcat(issUT2,":"),issSBS_CGS_PIP)
#define issUT2_FREQ             zcat(zcat(issUT2,":"),issSBS_CHOP_FREQ)
#define issUT2_STATUS           zcat(zcat(issUT2,":"),issSBS_CHOP_STATUS)
#define issUT2_STARTTIME        zcat(zcat(issUT2,":"),issSBS_CHOP_STARTTIME)
#define issUT2_THROW            zcat(zcat(issUT2,":"),issSBS_CHOP_THROW)
#define issUT2_PVR              zcat(zcat(issUT2,":"),issSBS_CHOP_PVR)
#define issUT2_POSANG           zcat(zcat(issUT2,":"),issSBS_CHOP_POSANG)
#define issUT2_CHOPGUIDE        zcat(zcat(issUT2,":"),issSBS_CHOP_GUIDE)
#define issUT2_GUIDING_STATE    zcat(zcat(issUT2,":"),issSBS_GUIDING_STATE)
#define issUT2_TRACKSTATE       zcat(zcat(issUT2,":"),issSBS_TCSSTATE_TRACK)

/* special for UTs */
#define issUT2_NAS_GS_RA2000    zcat(zcat(issUT2,":"),issSBS_NAS_GS_RA2000)
#define issUT2_NAS_GS_DEC2000   zcat(zcat(issUT2,":"),issSBS_NAS_GS_DEC2000)
#define issUT2_NAS_STATUS       zcat(zcat(issUT2,":"),issSBS_NAS_STATUS)
#define issUT2_NAS_MODE         zcat(zcat(issUT2,":"),issSBS_NAS_MODE)
#define issUT2_AO_CYCLM1M2      zcat(zcat(issUT2,":"),issSBS_AO_CYCLM1M2)
#define issUT2_FOCUS            zcat(zcat(issUT2,":"),issSBS_TCSSTATE_FOCUS)
#define issUT2_BEACON_ADFNPOS   zcat(zcat(issUT2,":"),issSBS_BEACON_ADFNPOS)

/* UT3 */
#define issUT3_LST              zcat(zcat(issUT3,":"),issSBS_TIMES_LST)
#define issUT3_UTC              zcat(zcat(issUT3,":"),issSBS_TIMES_UTC)
#define issUT3_HA               zcat(zcat(issUT3,":"),issSBS_COORD_HA)
#define issUT3_RA               zcat(zcat(issUT3,":"),issSBS_COORD_RA)
#define issUT3_DEC              zcat(zcat(issUT3,":"),issSBS_COORD_DEC)
#define issUT3_RA2000           zcat(zcat(issUT3,":"),issSBS_COORD_RA2000)
#define issUT3_DEC2000          zcat(zcat(issUT3,":"),issSBS_COORD_DEC2000)
#define issUT3_RA2000DEG        zcat(zcat(issUT3,":"),issSBS_COORD_RA2000DEG)
#define issUT3_DEC2000DEG       zcat(zcat(issUT3,":"),issSBS_COORD_DEC2000DEG)
#define issUT3_ALT              zcat(zcat(issUT3,":"),issSBS_COORD_ALT)
#define issUT3_AZ               zcat(zcat(issUT3,":"),issSBS_COORD_AZ)
#define issUT3_PRLTIC           zcat(zcat(issUT3,":"),issSBS_COORD_PRLTIC)
#define issUT3_REMTIME          zcat(zcat(issUT3,":"),issSBS_TRACK_REMTIME)
#define issUT3_AIRMASS          zcat(zcat(issUT3,":"),issSBS_TRACK_AIRMASS)
#define issUT3_DVELRA           zcat(zcat(issUT3,":"),issSBS_TRACK_DVELRA)
#define issUT3_DVELDEC          zcat(zcat(issUT3,":"),issSBS_TRACK_DVELDEC)
#define issUT3_CGS_RA2000       zcat(zcat(issUT3,":"),issSBS_CGS_RA2000)
#define issUT3_CGS_DEC2000      zcat(zcat(issUT3,":"),issSBS_CGS_DEC2000)
#define issUT3_CGS_MODE         zcat(zcat(issUT3,":"),issSBS_CGS_MODE)
#define issUT3_CGS_MAG          zcat(zcat(issUT3,":"),issSBS_CGS_MAG)
#define issUT3_CGS_WAVELEN      zcat(zcat(issUT3,":"),issSBS_CGS_WAVELEN)
#define issUT3_CGS_PIP          zcat(zcat(issUT3,":"),issSBS_CGS_PIP)
#define issUT3_FREQ             zcat(zcat(issUT3,":"),issSBS_CHOP_FREQ)
#define issUT3_STATUS           zcat(zcat(issUT3,":"),issSBS_CHOP_STATUS)
#define issUT3_STARTTIME        zcat(zcat(issUT3,":"),issSBS_CHOP_STARTTIME)
#define issUT3_THROW            zcat(zcat(issUT3,":"),issSBS_CHOP_THROW)
#define issUT3_PVR              zcat(zcat(issUT3,":"),issSBS_CHOP_PVR)
#define issUT3_POSANG           zcat(zcat(issUT3,":"),issSBS_CHOP_POSANG)
#define issUT3_CHOPGUIDE        zcat(zcat(issUT3,":"),issSBS_CHOP_GUIDE)
#define issUT3_GUIDING_STATE    zcat(zcat(issUT3,":"),issSBS_GUIDING_STATE)
#define issUT3_TRACKSTATE       zcat(zcat(issUT3,":"),issSBS_TCSSTATE_TRACK)

/* special for UTs */
#define issUT3_NAS_GS_RA2000    zcat(zcat(issUT3,":"),issSBS_NAS_GS_RA2000)
#define issUT3_NAS_GS_DEC2000   zcat(zcat(issUT3,":"),issSBS_NAS_GS_DEC2000)
#define issUT3_NAS_STATUS       zcat(zcat(issUT3,":"),issSBS_NAS_STATUS)
#define issUT3_NAS_MODE         zcat(zcat(issUT3,":"),issSBS_NAS_MODE)
#define issUT3_AO_CYCLM1M2      zcat(zcat(issUT3,":"),issSBS_AO_CYCLM1M2)
#define issUT3_FOCUS            zcat(zcat(issUT3,":"),issSBS_TCSSTATE_FOCUS)
#define issUT3_BEACON_ADFNPOS   zcat(zcat(issUT3,":"),issSBS_BEACON_ADFNPOS)

/* UT4 */
#define issUT4_LST              zcat(zcat(issUT4,":"),issSBS_TIMES_LST)
#define issUT4_UTC              zcat(zcat(issUT4,":"),issSBS_TIMES_UTC)
#define issUT4_HA               zcat(zcat(issUT4,":"),issSBS_COORD_HA)
#define issUT4_RA               zcat(zcat(issUT4,":"),issSBS_COORD_RA)
#define issUT4_DEC              zcat(zcat(issUT4,":"),issSBS_COORD_DEC)
#define issUT4_RA2000           zcat(zcat(issUT4,":"),issSBS_COORD_RA2000)
#define issUT4_DEC2000          zcat(zcat(issUT4,":"),issSBS_COORD_DEC2000)
#define issUT4_RA2000DEG        zcat(zcat(issUT4,":"),issSBS_COORD_RA2000DEG)
#define issUT4_DEC2000DEG       zcat(zcat(issUT4,":"),issSBS_COORD_DEC2000DEG)
#define issUT4_ALT              zcat(zcat(issUT4,":"),issSBS_COORD_ALT)
#define issUT4_AZ               zcat(zcat(issUT4,":"),issSBS_COORD_AZ)
#define issUT4_PRLTIC           zcat(zcat(issUT4,":"),issSBS_COORD_PRLTIC)
#define issUT4_REMTIME          zcat(zcat(issUT4,":"),issSBS_TRACK_REMTIME)
#define issUT4_AIRMASS          zcat(zcat(issUT4,":"),issSBS_TRACK_AIRMASS)
#define issUT4_DVELRA           zcat(zcat(issUT4,":"),issSBS_TRACK_DVELRA)
#define issUT4_DVELDEC          zcat(zcat(issUT4,":"),issSBS_TRACK_DVELDEC)
#define issUT4_CGS_RA2000       zcat(zcat(issUT4,":"),issSBS_CGS_RA2000)
#define issUT4_CGS_DEC2000      zcat(zcat(issUT4,":"),issSBS_CGS_DEC2000)
#define issUT4_CGS_MODE         zcat(zcat(issUT4,":"),issSBS_CGS_MODE)
#define issUT4_CGS_MAG          zcat(zcat(issUT4,":"),issSBS_CGS_MAG)
#define issUT4_CGS_WAVELEN      zcat(zcat(issUT4,":"),issSBS_CGS_WAVELEN)
#define issUT4_CGS_PIP          zcat(zcat(issUT4,":"),issSBS_CGS_PIP)
#define issUT4_FREQ             zcat(zcat(issUT4,":"),issSBS_CHOP_FREQ)
#define issUT4_STATUS           zcat(zcat(issUT4,":"),issSBS_CHOP_STATUS)
#define issUT4_STARTTIME        zcat(zcat(issUT4,":"),issSBS_CHOP_STARTTIME)
#define issUT4_THROW            zcat(zcat(issUT4,":"),issSBS_CHOP_THROW)
#define issUT4_PVR              zcat(zcat(issUT4,":"),issSBS_CHOP_PVR)
#define issUT4_POSANG           zcat(zcat(issUT4,":"),issSBS_CHOP_POSANG)
#define issUT4_CHOPGUIDE        zcat(zcat(issUT4,":"),issSBS_CHOP_GUIDE)
#define issUT4_GUIDING_STATE    zcat(zcat(issUT4,":"),issSBS_GUIDING_STATE)
#define issUT4_TRACKSTATE       zcat(zcat(issUT4,":"),issSBS_TCSSTATE_TRACK)

/* special for UTs */
#define issUT4_NAS_GS_RA2000    zcat(zcat(issUT4,":"),issSBS_NAS_GS_RA2000)
#define issUT4_NAS_GS_DEC2000   zcat(zcat(issUT4,":"),issSBS_NAS_GS_DEC2000)
#define issUT4_NAS_STATUS       zcat(zcat(issUT4,":"),issSBS_NAS_STATUS)
#define issUT4_NAS_MODE         zcat(zcat(issUT4,":"),issSBS_NAS_MODE)
#define issUT4_AO_CYCLM1M2      zcat(zcat(issUT4,":"),issSBS_AO_CYCLM1M2)
#define issUT4_FOCUS            zcat(zcat(issUT4,":"),issSBS_TCSSTATE_FOCUS)
#define issUT4_BEACON_ADFNPOS   zcat(zcat(issUT4,":"),issSBS_BEACON_ADFNPOS)

/* AT1 */
#define issAT1_LST              zcat(zcat(issAT1,":"),issSBS_TIMES_LST)
#define issAT1_UTC              zcat(zcat(issAT1,":"),issSBS_TIMES_UTC)
#define issAT1_HA               zcat(zcat(issAT1,":"),issSBS_COORD_HA)
#define issAT1_RA               zcat(zcat(issAT1,":"),issSBS_COORD_RA)
#define issAT1_DEC              zcat(zcat(issAT1,":"),issSBS_COORD_DEC)
#define issAT1_RA2000           zcat(zcat(issAT1,":"),issSBS_COORD_RA2000)
#define issAT1_DEC2000          zcat(zcat(issAT1,":"),issSBS_COORD_DEC2000)
#define issAT1_RA2000DEG        zcat(zcat(issAT1,":"),issSBS_COORD_RA2000DEG)
#define issAT1_DEC2000DEG       zcat(zcat(issAT1,":"),issSBS_COORD_DEC2000DEG)
#define issAT1_ALT              zcat(zcat(issAT1,":"),issSBS_COORD_ALT)
#define issAT1_AZ               zcat(zcat(issAT1,":"),issSBS_COORD_AZ)
#define issAT1_PRLTIC           zcat(zcat(issAT1,":"),issSBS_COORD_PRLTIC)
#define issAT1_REMTIME          zcat(zcat(issAT1,":"),issSBS_TRACK_REMTIME)
#define issAT1_AIRMASS          zcat(zcat(issAT1,":"),issSBS_TRACK_AIRMASS)
#define issAT1_DVELRA           zcat(zcat(issAT1,":"),issSBS_TRACK_DVELRA)
#define issAT1_DVELDEC          zcat(zcat(issAT1,":"),issSBS_TRACK_DVELDEC)
#define issAT1_CGS_RA2000       zcat(zcat(issAT1,":"),issSBS_CGS_RA2000)
#define issAT1_CGS_DEC2000      zcat(zcat(issAT1,":"),issSBS_CGS_DEC2000)
#define issAT1_CGS_MODE         zcat(zcat(issAT1,":"),issSBS_CGS_MODE)
#define issAT1_CGS_MAG          zcat(zcat(issAT1,":"),issSBS_CGS_MAG)
#define issAT1_CGS_WAVELEN      zcat(zcat(issAT1,":"),issSBS_CGS_WAVELEN)
#define issAT1_CGS_PIP          zcat(zcat(issAT1,":"),issSBS_CGS_PIP)
#define issAT1_FREQ             zcat(zcat(issAT1,":"),issSBS_CHOP_FREQ)
#define issAT1_STATUS           zcat(zcat(issAT1,":"),issSBS_CHOP_STATUS)
#define issAT1_STARTTIME        zcat(zcat(issAT1,":"),issSBS_CHOP_STARTTIME)
#define issAT1_THROW            zcat(zcat(issAT1,":"),issSBS_CHOP_THROW)
#define issAT1_PVR              zcat(zcat(issAT1,":"),issSBS_CHOP_PVR)
#define issAT1_POSANG           zcat(zcat(issAT1,":"),issSBS_CHOP_POSANG)
#define issAT1_CHOPGUIDE        zcat(zcat(issAT1,":"),issSBS_CHOP_GUIDE)
#define issAT1_GUIDING_STATE    zcat(zcat(issAT1,":"),issSBS_GUIDING_STATE)
#define issAT1_TRACKSTATE       zcat(zcat(issAT1,":"),issSBS_TCSSTATE_TRACK)

/* special for ATs */
#define issAT1_LOCATION         zcat(zcat(issAT1,":"),issSBS_TCSSTATE_LOCATION)
#define issAT1_NFD_POS          zcat(zcat(issAT1,":"),issSBS_NFD_POS)

/* AT2 */
#define issAT2_LST              zcat(zcat(issAT2,":"),issSBS_TIMES_LST)
#define issAT2_UTC              zcat(zcat(issAT2,":"),issSBS_TIMES_UTC)
#define issAT2_HA               zcat(zcat(issAT2,":"),issSBS_COORD_HA)
#define issAT2_RA               zcat(zcat(issAT2,":"),issSBS_COORD_RA)
#define issAT2_DEC              zcat(zcat(issAT2,":"),issSBS_COORD_DEC)
#define issAT2_RA2000           zcat(zcat(issAT2,":"),issSBS_COORD_RA2000)
#define issAT2_DEC2000          zcat(zcat(issAT2,":"),issSBS_COORD_DEC2000)
#define issAT2_RA2000DEG        zcat(zcat(issAT2,":"),issSBS_COORD_RA2000DEG)
#define issAT2_DEC2000DEG       zcat(zcat(issAT2,":"),issSBS_COORD_DEC2000DEG)
#define issAT2_ALT              zcat(zcat(issAT2,":"),issSBS_COORD_ALT)
#define issAT2_AZ               zcat(zcat(issAT2,":"),issSBS_COORD_AZ)
#define issAT2_PRLTIC           zcat(zcat(issAT2,":"),issSBS_COORD_PRLTIC)
#define issAT2_REMTIME          zcat(zcat(issAT2,":"),issSBS_TRACK_REMTIME)
#define issAT2_AIRMASS          zcat(zcat(issAT2,":"),issSBS_TRACK_AIRMASS)
#define issAT2_DVELRA           zcat(zcat(issAT2,":"),issSBS_TRACK_DVELRA)
#define issAT2_DVELDEC          zcat(zcat(issAT2,":"),issSBS_TRACK_DVELDEC)
#define issAT2_CGS_RA2000       zcat(zcat(issAT2,":"),issSBS_CGS_RA2000)
#define issAT2_CGS_DEC2000      zcat(zcat(issAT2,":"),issSBS_CGS_DEC2000)
#define issAT2_CGS_MODE         zcat(zcat(issAT2,":"),issSBS_CGS_MODE)
#define issAT2_CGS_MAG          zcat(zcat(issAT2,":"),issSBS_CGS_MAG)
#define issAT2_CGS_WAVELEN      zcat(zcat(issAT2,":"),issSBS_CGS_WAVELEN)
#define issAT2_CGS_PIP          zcat(zcat(issAT2,":"),issSBS_CGS_PIP)
#define issAT2_FREQ             zcat(zcat(issAT2,":"),issSBS_CHOP_FREQ)
#define issAT2_STATUS           zcat(zcat(issAT2,":"),issSBS_CHOP_STATUS)
#define issAT2_STARTTIME        zcat(zcat(issAT2,":"),issSBS_CHOP_STARTTIME)
#define issAT2_THROW            zcat(zcat(issAT2,":"),issSBS_CHOP_THROW)
#define issAT2_PVR              zcat(zcat(issAT2,":"),issSBS_CHOP_PVR)
#define issAT2_POSANG           zcat(zcat(issAT2,":"),issSBS_CHOP_POSANG)
#define issAT2_CHOPGUIDE        zcat(zcat(issAT2,":"),issSBS_CHOP_GUIDE)
#define issAT2_GUIDING_STATE    zcat(zcat(issAT2,":"),issSBS_GUIDING_STATE)
#define issAT2_TRACKSTATE       zcat(zcat(issAT2,":"),issSBS_TCSSTATE_TRACK)

/* special for ATs */
#define issAT2_LOCATION         zcat(zcat(issAT2,":"),issSBS_TCSSTATE_LOCATION)
#define issAT2_NFD_POS          zcat(zcat(issAT2,":"),issSBS_NFD_POS)

/* AT3 */
#define issAT3_LST              zcat(zcat(issAT3,":"),issSBS_TIMES_LST)
#define issAT3_UTC              zcat(zcat(issAT3,":"),issSBS_TIMES_UTC)
#define issAT3_HA               zcat(zcat(issAT3,":"),issSBS_COORD_HA)
#define issAT3_RA               zcat(zcat(issAT3,":"),issSBS_COORD_RA)
#define issAT3_DEC              zcat(zcat(issAT3,":"),issSBS_COORD_DEC)
#define issAT3_RA2000           zcat(zcat(issAT3,":"),issSBS_COORD_RA2000)
#define issAT3_DEC2000          zcat(zcat(issAT3,":"),issSBS_COORD_DEC2000)
#define issAT3_RA2000DEG        zcat(zcat(issAT3,":"),issSBS_COORD_RA2000DEG)
#define issAT3_DEC2000DEG       zcat(zcat(issAT3,":"),issSBS_COORD_DEC2000DEG)
#define issAT3_ALT              zcat(zcat(issAT3,":"),issSBS_COORD_ALT)
#define issAT3_AZ               zcat(zcat(issAT3,":"),issSBS_COORD_AZ)
#define issAT3_PRLTIC           zcat(zcat(issAT3,":"),issSBS_COORD_PRLTIC)
#define issAT3_REMTIME          zcat(zcat(issAT3,":"),issSBS_TRACK_REMTIME)
#define issAT3_AIRMASS          zcat(zcat(issAT3,":"),issSBS_TRACK_AIRMASS)
#define issAT3_DVELRA           zcat(zcat(issAT3,":"),issSBS_TRACK_DVELRA)
#define issAT3_DVELDEC          zcat(zcat(issAT3,":"),issSBS_TRACK_DVELDEC)
#define issAT3_CGS_RA2000       zcat(zcat(issAT3,":"),issSBS_CGS_RA2000)
#define issAT3_CGS_DEC2000      zcat(zcat(issAT3,":"),issSBS_CGS_DEC2000)
#define issAT3_CGS_MODE         zcat(zcat(issAT3,":"),issSBS_CGS_MODE)
#define issAT3_CGS_MAG          zcat(zcat(issAT3,":"),issSBS_CGS_MAG)
#define issAT3_CGS_WAVELEN      zcat(zcat(issAT3,":"),issSBS_CGS_WAVELEN)
#define issAT3_CGS_PIP          zcat(zcat(issAT3,":"),issSBS_CGS_PIP)
#define issAT3_FREQ             zcat(zcat(issAT3,":"),issSBS_CHOP_FREQ)
#define issAT3_STATUS           zcat(zcat(issAT3,":"),issSBS_CHOP_STATUS)
#define issAT3_STARTTIME        zcat(zcat(issAT3,":"),issSBS_CHOP_STARTTIME)
#define issAT3_THROW            zcat(zcat(issAT3,":"),issSBS_CHOP_THROW)
#define issAT3_PVR              zcat(zcat(issAT3,":"),issSBS_CHOP_PVR)
#define issAT3_POSANG           zcat(zcat(issAT3,":"),issSBS_CHOP_POSANG)
#define issAT3_CHOPGUIDE        zcat(zcat(issAT3,":"),issSBS_CHOP_GUIDE)
#define issAT3_GUIDING_STATE    zcat(zcat(issAT3,":"),issSBS_GUIDING_STATE)
#define issAT3_TRACKSTATE       zcat(zcat(issAT3,":"),issSBS_TCSSTATE_TRACK)

/* special for ATs */
#define issAT3_LOCATION         zcat(zcat(issAT3,":"),issSBS_TCSSTATE_LOCATION)
#define issAT3_NFD_POS          zcat(zcat(issAT3,":"),issSBS_NFD_POS)

/* COU1 */
#define issCOU1_LST             zcat(zcat(issCOU1,":"),issSBS_TIMES_LST)
#define issCOU1_UTC             zcat(zcat(issCOU1,":"),issSBS_TIMES_UTC)
#define issCOU1_HA              zcat(zcat(issCOU1,":"),issSBS_COORD_HA)
#define issCOU1_RA              zcat(zcat(issCOU1,":"),issSBS_COORD_RA)
#define issCOU1_DEC             zcat(zcat(issCOU1,":"),issSBS_COORD_DEC)
#define issCOU1_RA2000          zcat(zcat(issCOU1,":"),issSBS_COORD_RA2000)
#define issCOU1_DEC2000         zcat(zcat(issCOU1,":"),issSBS_COORD_DEC2000)
#define issCOU1_RA2000DEG       zcat(zcat(issCOU1,":"),issSBS_COORD_RA2000DEG)
#define issCOU1_DEC2000DEG      zcat(zcat(issCOU1,":"),issSBS_COORD_DEC2000DEG)
#define issCOU1_ALT             zcat(zcat(issCOU1,":"),issSBS_COORD_ALT)
#define issCOU1_AZ              zcat(zcat(issCOU1,":"),issSBS_COORD_AZ)
#define issCOU1_PRLTIC          zcat(zcat(issCOU1,":"),issSBS_COORD_PRLTIC)
#define issCOU1_REMTIME         zcat(zcat(issCOU1,":"),issSBS_TRACK_REMTIME)
#define issCOU1_AIRMASS         zcat(zcat(issCOU1,":"),issSBS_TRACK_AIRMASS)
#define issCOU1_DVELRA          zcat(zcat(issCOU1,":"),issSBS_TRACK_DVELRA)
#define issCOU1_DVELDEC         zcat(zcat(issCOU1,":"),issSBS_TRACK_DVELDEC)
#define issCOU1_CGS_RA2000      zcat(zcat(issCOU1,":"),issSBS_CGS_RA2000)
#define issCOU1_CGS_DEC2000     zcat(zcat(issCOU1,":"),issSBS_CGS_DEC2000)
#define issCOU1_CGS_MODE        zcat(zcat(issCOU1,":"),issSBS_CGS_MODE)
#define issCOU1_CGS_MAG         zcat(zcat(issCOU1,":"),issSBS_CGS_MAG)
#define issCOU1_CGS_WAVELEN     zcat(zcat(issCOU1,":"),issSBS_CGS_WAVELEN)
#define issCOU1_CGS_PIP         zcat(zcat(issCOU1,":"),issSBS_CGS_PIP)
#define issCOU1_FREQ            zcat(zcat(issCOU1,":"),issSBS_CHOP_FREQ)
#define issCOU1_STATUS          zcat(zcat(issCOU1,":"),issSBS_CHOP_STATUS)
#define issCOU1_STARTTIME       zcat(zcat(issCOU1,":"),issSBS_CHOP_STARTTIME)
#define issCOU1_THROW           zcat(zcat(issCOU1,":"),issSBS_CHOP_THROW)
#define issCOU1_PVR             zcat(zcat(issCOU1,":"),issSBS_CHOP_PVR)
#define issCOU1_POSANG          zcat(zcat(issCOU1,":"),issSBS_CHOP_POSANG)
#define issCOU1_CHOPGUIDE       zcat(zcat(issCOU1,":"),issSBS_CHOP_GUIDE)
#define issCOU1_GUIDING_STATE   zcat(zcat(issCOU1,":"),issSBS_GUIDING_STATE)
#define issCOU1_TRACKSTATE      zcat(zcat(issCOU1,":"),issSBS_TCSSTATE_TRACK)

/* special for COUs */
#define issCOU1_FOCUS           zcat(zcat(issCOU1,":"),issSBS_TCSSTATE_FOCUS)

/* COU2 */
#define issCOU2_LST             zcat(zcat(issCOU2,":"),issSBS_TIMES_LST)
#define issCOU2_UTC             zcat(zcat(issCOU2,":"),issSBS_TIMES_UTC)
#define issCOU2_HA              zcat(zcat(issCOU2,":"),issSBS_COORD_HA)
#define issCOU2_RA              zcat(zcat(issCOU2,":"),issSBS_COORD_RA)
#define issCOU2_DEC             zcat(zcat(issCOU2,":"),issSBS_COORD_DEC)
#define issCOU2_RA2000          zcat(zcat(issCOU2,":"),issSBS_COORD_RA2000)
#define issCOU2_DEC2000         zcat(zcat(issCOU2,":"),issSBS_COORD_DEC2000)
#define issCOU2_RA2000DEG       zcat(zcat(issCOU2,":"),issSBS_COORD_RA2000DEG)
#define issCOU2_DEC2000DEG      zcat(zcat(issCOU2,":"),issSBS_COORD_DEC2000DEG)
#define issCOU2_ALT             zcat(zcat(issCOU2,":"),issSBS_COORD_ALT)
#define issCOU2_AZ              zcat(zcat(issCOU2,":"),issSBS_COORD_AZ)
#define issCOU2_PRLTIC          zcat(zcat(issCOU2,":"),issSBS_COORD_PRLTIC)
#define issCOU2_REMTIME         zcat(zcat(issCOU2,":"),issSBS_TRACK_REMTIME)
#define issCOU2_AIRMASS         zcat(zcat(issCOU2,":"),issSBS_TRACK_AIRMASS)
#define issCOU2_DVELRA          zcat(zcat(issCOU2,":"),issSBS_TRACK_DVELRA)
#define issCOU2_DVELDEC         zcat(zcat(issCOU2,":"),issSBS_TRACK_DVELDEC)
#define issCOU2_CGS_RA2000      zcat(zcat(issCOU2,":"),issSBS_CGS_RA2000)
#define issCOU2_CGS_DEC2000     zcat(zcat(issCOU2,":"),issSBS_CGS_DEC2000)
#define issCOU2_CGS_MODE        zcat(zcat(issCOU2,":"),issSBS_CGS_MODE)
#define issCOU2_CGS_MAG         zcat(zcat(issCOU2,":"),issSBS_CGS_MAG)
#define issCOU2_CGS_WAVELEN     zcat(zcat(issCOU2,":"),issSBS_CGS_WAVELEN)
#define issCOU2_CGS_PIP         zcat(zcat(issCOU2,":"),issSBS_CGS_PIP)
#define issCOU2_FREQ            zcat(zcat(issCOU2,":"),issSBS_CHOP_FREQ)
#define issCOU2_STATUS          zcat(zcat(issCOU2,":"),issSBS_CHOP_STATUS)
#define issCOU2_STARTTIME       zcat(zcat(issCOU2,":"),issSBS_CHOP_STARTTIME)
#define issCOU2_THROW           zcat(zcat(issCOU2,":"),issSBS_CHOP_THROW)
#define issCOU2_PVR             zcat(zcat(issCOU2,":"),issSBS_CHOP_PVR)
#define issCOU2_POSANG          zcat(zcat(issCOU2,":"),issSBS_CHOP_POSANG)
#define issCOU2_CHOPGUIDE       zcat(zcat(issCOU2,":"),issSBS_CHOP_GUIDE)
#define issCOU2_GUIDING_STATE   zcat(zcat(issCOU2,":"),issSBS_GUIDING_STATE)
#define issCOU2_TRACKSTATE      zcat(zcat(issCOU2,":"),issSBS_TCSSTATE_TRACK)

/* special for COUs */
#define issCOU2_FOCUS           zcat(zcat(issCOU2,":"),issSBS_TCSSTATE_FOCUS)

/* COU3 */
#define issCOU3_LST             zcat(zcat(issCOU3,":"),issSBS_TIMES_LST)
#define issCOU3_UTC             zcat(zcat(issCOU3,":"),issSBS_TIMES_UTC)
#define issCOU3_HA              zcat(zcat(issCOU3,":"),issSBS_COORD_HA)
#define issCOU3_RA              zcat(zcat(issCOU3,":"),issSBS_COORD_RA)
#define issCOU3_DEC             zcat(zcat(issCOU3,":"),issSBS_COORD_DEC)
#define issCOU3_RA2000          zcat(zcat(issCOU3,":"),issSBS_COORD_RA2000)
#define issCOU3_DEC2000         zcat(zcat(issCOU3,":"),issSBS_COORD_DEC2000)
#define issCOU3_RA2000DEG       zcat(zcat(issCOU3,":"),issSBS_COORD_RA2000DEG)
#define issCOU3_DEC2000DEG      zcat(zcat(issCOU3,":"),issSBS_COORD_DEC2000DEG)
#define issCOU3_ALT             zcat(zcat(issCOU3,":"),issSBS_COORD_ALT)
#define issCOU3_AZ              zcat(zcat(issCOU3,":"),issSBS_COORD_AZ)
#define issCOU3_PRLTIC          zcat(zcat(issCOU3,":"),issSBS_COORD_PRLTIC)
#define issCOU3_REMTIME         zcat(zcat(issCOU3,":"),issSBS_TRACK_REMTIME)
#define issCOU3_AIRMASS         zcat(zcat(issCOU3,":"),issSBS_TRACK_AIRMASS)
#define issCOU3_DVELRA          zcat(zcat(issCOU3,":"),issSBS_TRACK_DVELRA)
#define issCOU3_DVELDEC         zcat(zcat(issCOU3,":"),issSBS_TRACK_DVELDEC)
#define issCOU3_CGS_RA2000      zcat(zcat(issCOU3,":"),issSBS_CGS_RA2000)
#define issCOU3_CGS_DEC2000     zcat(zcat(issCOU3,":"),issSBS_CGS_DEC2000)
#define issCOU3_CGS_MODE        zcat(zcat(issCOU3,":"),issSBS_CGS_MODE)
#define issCOU3_CGS_MAG         zcat(zcat(issCOU3,":"),issSBS_CGS_MAG)
#define issCOU3_CGS_WAVELEN     zcat(zcat(issCOU3,":"),issSBS_CGS_WAVELEN)
#define issCOU3_CGS_PIP         zcat(zcat(issCOU3,":"),issSBS_CGS_PIP)
#define issCOU3_FREQ            zcat(zcat(issCOU3,":"),issSBS_CHOP_FREQ)
#define issCOU3_STATUS          zcat(zcat(issCOU3,":"),issSBS_CHOP_STATUS)
#define issCOU3_STARTTIME       zcat(zcat(issCOU3,":"),issSBS_CHOP_STARTTIME)
#define issCOU3_THROW           zcat(zcat(issCOU3,":"),issSBS_CHOP_THROW)
#define issCOU3_PVR             zcat(zcat(issCOU3,":"),issSBS_CHOP_PVR)
#define issCOU3_POSANG          zcat(zcat(issCOU3,":"),issSBS_CHOP_POSANG)
#define issCOU3_CHOPGUIDE       zcat(zcat(issCOU3,":"),issSBS_CHOP_GUIDE)
#define issCOU3_GUIDING_STATE   zcat(zcat(issCOU3,":"),issSBS_GUIDING_STATE)
#define issCOU3_TRACKSTATE      zcat(zcat(issCOU3,":"),issSBS_TCSSTATE_TRACK)

/* special for COUs */
#define issCOU3_FOCUS           zcat(zcat(issCOU3,":"),issSBS_TCSSTATE_FOCUS)

/* COU4 */
#define issCOU4_LST             zcat(zcat(issCOU4,":"),issSBS_TIMES_LST)
#define issCOU4_UTC             zcat(zcat(issCOU4,":"),issSBS_TIMES_UTC)
#define issCOU4_HA              zcat(zcat(issCOU4,":"),issSBS_COORD_HA)
#define issCOU4_RA              zcat(zcat(issCOU4,":"),issSBS_COORD_RA)
#define issCOU4_DEC             zcat(zcat(issCOU4,":"),issSBS_COORD_DEC)
#define issCOU4_RA2000          zcat(zcat(issCOU4,":"),issSBS_COORD_RA2000)
#define issCOU4_DEC2000         zcat(zcat(issCOU4,":"),issSBS_COORD_DEC2000)
#define issCOU4_RA2000DEG       zcat(zcat(issCOU4,":"),issSBS_COORD_RA2000DEG)
#define issCOU4_DEC2000DEG      zcat(zcat(issCOU4,":"),issSBS_COORD_DEC2000DEG)
#define issCOU4_ALT             zcat(zcat(issCOU4,":"),issSBS_COORD_ALT)
#define issCOU4_AZ              zcat(zcat(issCOU4,":"),issSBS_COORD_AZ)
#define issCOU4_PRLTIC          zcat(zcat(issCOU4,":"),issSBS_COORD_PRLTIC)
#define issCOU4_REMTIME         zcat(zcat(issCOU4,":"),issSBS_TRACK_REMTIME)
#define issCOU4_AIRMASS         zcat(zcat(issCOU4,":"),issSBS_TRACK_AIRMASS)
#define issCOU4_DVELRA          zcat(zcat(issCOU4,":"),issSBS_TRACK_DVELRA)
#define issCOU4_DVELDEC         zcat(zcat(issCOU4,":"),issSBS_TRACK_DVELDEC)
#define issCOU4_CGS_RA2000      zcat(zcat(issCOU4,":"),issSBS_CGS_RA2000)
#define issCOU4_CGS_DEC2000     zcat(zcat(issCOU4,":"),issSBS_CGS_DEC2000)
#define issCOU4_CGS_MODE        zcat(zcat(issCOU4,":"),issSBS_CGS_MODE)
#define issCOU4_CGS_MAG         zcat(zcat(issCOU4,":"),issSBS_CGS_MAG)
#define issCOU4_CGS_WAVELEN     zcat(zcat(issCOU4,":"),issSBS_CGS_WAVELEN)
#define issCOU4_CGS_PIP         zcat(zcat(issCOU4,":"),issSBS_CGS_PIP)
#define issCOU4_FREQ            zcat(zcat(issCOU4,":"),issSBS_CHOP_FREQ)
#define issCOU4_STATUS          zcat(zcat(issCOU4,":"),issSBS_CHOP_STATUS)
#define issCOU4_STARTTIME       zcat(zcat(issCOU4,":"),issSBS_CHOP_STARTTIME)
#define issCOU4_THROW           zcat(zcat(issCOU4,":"),issSBS_CHOP_THROW)
#define issCOU4_PVR             zcat(zcat(issCOU4,":"),issSBS_CHOP_PVR)
#define issCOU4_POSANG          zcat(zcat(issCOU4,":"),issSBS_CHOP_POSANG)
#define issCOU4_CHOPGUIDE       zcat(zcat(issCOU4,":"),issSBS_CHOP_GUIDE)
#define issCOU4_GUIDING_STATE   zcat(zcat(issCOU4,":"),issSBS_GUIDING_STATE)
#define issCOU4_TRACKSTATE      zcat(zcat(issCOU4,":"),issSBS_TCSSTATE_TRACK)

/* special for COUs */
#define issCOU4_FOCUS           zcat(zcat(issCOU4,":"),issSBS_TCSSTATE_FOCUS)

/* ":Appl_data:VLTI:subSystems:wdline:dlcsState.trkOkFlag" et al. */
#define issDLCS_TRACKSTATE      zcat(zcat(issDLCS,":"),issSBS_DLCS_TRK_STATE)
#define issDLCS_STATE           zcat(zcat(issDLCS,":"),issSBS_DLCS_STATE)
#define issDLCS_SUB_STATE       zcat(zcat(issDLCS,":"),issSBS_DLCS_SUB_STATE)
#define issDLCS_REM_TRK_TIME    zcat(zcat(issDLCS,":"),issSBS_DLCS_REM_TRK_TIME)
#define issDLCS_MOTION_PHASE    zcat(zcat(issDLCS,":"),issSBS_DLCS_MOTION_PHASE)
#define issDLCS_REF_DL          zcat(zcat(issDLCS,":"),issSBS_DLCS_REF_DL)
#define issDLCS_REF_OPL         zcat(zcat(issDLCS,":"),issSBS_DLCS_REF_OPL)
#define issDLCS_REF_MODE        zcat(zcat(issDLCS,":"),issSBS_DLCS_REF_MODE)
#define issDLCS_RA2000          zcat(zcat(issDLCS,":"),issSBS_DLCS_RA2000)
#define issDLCS_DEC2000         zcat(zcat(issDLCS,":"),issSBS_DLCS_DEC2000)
#define issDLCS_OPL1            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL1)
#define issDLCS_OPL2            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL2)
#define issDLCS_OPL3            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL3)
#define issDLCS_OPL4            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL4)
#define issDLCS_OPL5            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL5)
#define issDLCS_OPL6            zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL6)
#define issDLCS_OPLOFFS1        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS1)
#define issDLCS_OPLOFFS2        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS2)
#define issDLCS_OPLOFFS3        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS3)
#define issDLCS_OPLOFFS4        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS4)
#define issDLCS_OPLOFFS5        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS5)
#define issDLCS_OPLOFFS6        zcat(zcat(issDLCS,":"),issSBS_DLCS_OPLOFFS6)


#define issDLCS_REFBEAM         zcat(zcat(issDLCS,":"),issSBS_DLCS_REFBEAM)
#define issDLCS_TEL1_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL1_XYZ)
#define issDLCS_TEL2_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL2_XYZ)
#define issDLCS_TEL3_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL3_XYZ)
#define issDLCS_TEL4_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL4_XYZ)
#define issDLCS_TEL5_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL5_XYZ)
#define issDLCS_TEL6_XYZ        zcat(zcat(issDLCS,":"),issSBS_DLCS_TEL6_XYZ)

#define issDLCS_OPL_OFFSPRE     zcat(zcat(issDLCS,":"),issSBS_DLCS_OPL_OFFSPRE)


/* database root point for FINITO public data */
#define issFNT_PUB_DATA_POINT    "fntpub"
#define issFNT_PUB_DATA_PATH     zcat(zcat(issDLCS,":"),issFNT_PUB_DATA_POINT)


/* M16 positions of physical delay lines */
#define issM16_PUB_DATA_POINT    "m16"
#define issM16_PUB_DATA_PATH     zcat(zcat(issDLCS,":"),issM16_PUB_DATA_POINT)
#define issDB_DL1_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL1")
#define issDB_DL2_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL2")
#define issDB_DL3_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL3")
#define issDB_DL4_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL4")
#define issDB_DL5_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL5")
#define issDB_DL6_M16_POS        zcat(issM16_PUB_DATA_PATH,".m16posDL6")


#endif /* issSBS_DB_DEFINES_H */


