#ifndef issalignCOMMON_H
#define issalignCOMMON_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issalignCOMMON.h,v 3.17 2005/06/08 13:37:01 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* jott      2005-05-13  Added method GetRefpixTableStatus() to improve verbose
*                       STATUS command reply.
* jott      2005-05-03  Added method OpslogAlignmentParameter(), SPR VLTSW20050147
* jott      2005-02-28  Added members mTccdBakTablesInitialized and
*                       mIrisBakTablesInitialized
* jott      2005-02-23  Provided backup tables separately for IRIS and TCCD
*                       for correct recovering when IAS changes
* jott      2005-01-28  Added method ZeroAllCompRefTables() to handle IAS==NONE
*                       and OpslogReferencePixels() to solve SPR VLTSW20050019
* jott      2004-12-13  Added members mVltiTracking, mTrackingList,
*                       added public method CheckChipLimits();
* jott      2004-12-07  Added optional parameter 'mIgnoreTableStateChanges' to method
*                       SetRefPixTableStatus()
*                       added method ClearTableStateIgnoreFlag()
*                       added member  mIgnoreTableStateChanges
* jott      2004-11-04  Comments from 28-Oct-2004 to SPR VLTSW20040230:
*                       added methods RestoreOldPixelTables(), AdjustWindow(),
*                       BackupCurrentPixelTables(), SetRefPixTableStatus(),
*                       InitZeroBackupPixelTables(), InitOtherBackupPixelTables(),
*                       CheckChipLimitsForRefPix()
*                       added members mTccdZeroRefPixelsBak[],
*                       mIrisZeroRefPixelsBak[], mInsPtOffsetBak[],
*                       mInsPtOffsetPixBak[], mDynamicZPointPixBak[],
*                       mCurrZPointPixBak[], mIrisAccuOffsetsVWBak[],
*                       mIirisAccuOffsetsPixBak[], mDeltaTadcBak[], mIasIdBak,
*                       mRefPixTableStatus, mTccdZeroRefPixels[],
*                       mIrisZeroRefPixels[], mInsPtOffset[], mInsPtOffsetPix[],
*                       mDdynamicZPointPix[], mCurrZPointPix[],
*                       mIrisAccuOffsetsVW[], mIrisAccuOffsetsPix[],
*                       mDeltaTadc[], mFinalRefPixels[], mFinalRefPixelsBak[]
* jott      2004-10-25  added methods InstrumentChangedCB(),
*                       GetTadcZeroWaveLen()
*                       added members mInstrumentChanged, mInstrConfValid,
*                       mUseWlen, mInstrKeywordValid, mInvalidInstKeyword
* jott      2004-10-05  created
*/

/************************************************************************
 * Header file for the common methods library class of the ISS alignment module
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * Common VLTSW headers
 */
#include "eccsEvt.h"
#include "evhTASK.h"

/*
 * VLTI headers
 */
#include "issCALCULATOR.h"
#include "issalign.h"


class issalignCOMMON : public evhTASK
    {
    public:
        issalignCOMMON(const dbSYMADDRESS   dbBasePoint,
                       const dbSYMADDRESS   dbVltiPoint);
        ~issalignCOMMON(void);

        ccsCOMPL_STAT     Init(void);
        ccsCOMPL_STAT     ComputeReferencePixels(issalignREF_PIX_CONF  *configuration);
        ccsCOMPL_STAT     UpdateConfigTable(void){return mGenCalc.NewConfigTable();};
        ccsCOMPL_STAT     ConvertXy2Vw(vltINT32   iaSensorId,
                                       vltDOUBLE  posX,       vltDOUBLE  posY,
                                       vltDOUBLE *offsV,      vltDOUBLE *offsW);
        ccsCOMPL_STAT     ConvertXyError2VwErrorNorm(vltINT32   iaSensorId,
                                                     vltDOUBLE  errX, vltDOUBLE  errY,
                                                     vltDOUBLE *errVw);
        ccsCOMPL_STAT     ConvertVwNorm2XyNorm(vltINT32   iaSensorId,
                                               vltDOUBLE  normVw,
                                               vltDOUBLE *normXy);
        ccsCOMPL_STAT     RestoreOldPixelTables(void);
        ccsCOMPL_STAT     BackupCurrentPixelTables(vltINT32 iaSensorId);
        ccsCOMPL_STAT     InitZeroBackupPixelTables(vltINT32  iaSensorId);
        ccsCOMPL_STAT     InitOtherBackupPixelTables(vltINT32 iaSensorId);

        ccsCOMPL_STAT     SetRefPixTableStatus(issalignREF_PIX_TAB_STATUS status,
                                               vltLOGICAL                 ignoreFlag = 0);
        ccsCOMPL_STAT     ClearTableStateIgnoreFlag(void) {mIgnoreTableStateChanges = ccsFALSE; return SUCCESS;};
        vltLOGICAL        CheckChipLimits(vltINT32  sensorId,
                                          vltDOUBLE posX, vltDOUBLE posY);
        ccsCOMPL_STAT     OpslogReferencePixels(vltINT32             sensorId,
                                                vltINT32             beam,
                                                issalignOPSLOGPIX   *conf);
        ccsCOMPL_STAT     OpslogAlignmentParameter(vltINT32  sensorId,
                                                   int       numBeams);


        /*
         * Event callbacks
         */
        evhCB_COMPL_STAT  InstrumentChangedCB(evtEVENT_MSG &evtMsg, void *udata);

        /*
         * auxiliary methods mainly for online diagnostics
         */
        issalignTADC_USE_ZERO_WLEN  GetUsedWlenId(void) {return mUseWlen;};
        issalignREF_PIX_TAB_STATUS  GetRefpixTableStatus(void) {return mRefPixTableStatus;};


    protected:
        ccsCOMPL_STAT     CalculateTadcShift(vltINT32   beam,     vltINT32   sensorId,
                                             vltDOUBLE  pixScale, vltDOUBLE  theta,
                                             vltDOUBLE  epsilon,  vltDOUBLE *deltaX,
                                             vltDOUBLE *deltaY);
        ccsCOMPL_STAT     GetTadcZeroWaveLen(void);
        ccsCOMPL_STAT     AdjustWindow(vltINT32   sensorId);
        vltLOGICAL        CheckChipLimitsForRefPix(vltINT32  sensorId);
        ccsCOMPL_STAT     ZeroAllCompRefTables(void);

    private:
        ccsCOMPL_STAT     PrepareDatabaseLists(void);

        dbSYMADDRESS                mDbVltiPoint;

        vltDOUBLE                   mTccdPixScale;
        vltDOUBLE                   mIrisPixScale;
        vltDOUBLE                   mTccdRotAngle;
        vltDOUBLE                   mIrisRotAngle;
        vltDOUBLE                   mTccdAxialSym;
        vltDOUBLE                   mIrisAxialSym;

        vltBYTES32                  mSelectedIns;
        vltLOGICAL                  mInstrValid;
        vltLOGICAL                  mInstrKeywordValid;
        vltBYTES64                  mInvalidInstKeyword;
        vltLOGICAL                  mIgnoreTableStateChanges;
        issalignTADC_USE_ZERO_WLEN  mUseWlen;

        issalignREF_PIX_TAB_STATUS  mRefPixTableStatus;
        issalignREF_PIXEL_REC       mTccdZeroRefPixels[issalignMAX_NUM_ARMS];
        issalignREF_PIX_IRIS_REC    mIrisZeroRefPixels[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mInsPtOffset[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mInsPtOffsetPix[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mDynamicZPointPix[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mCurrZPointPix[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisAccuOffsetsVW[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisAccuOffsetsPix[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mDeltaTadc[issalignMAX_NUM_ARMS];
        issalignREF_PIXEL_REC       mFinalRefPixels[issalignMAX_NUM_ARMS];

        vltINT32                    mIasIdBak;
        issalignREF_PIXEL_REC       mTccdZeroRefPixelsBak[issalignMAX_NUM_ARMS];
        issalignREF_PIX_IRIS_REC    mIrisZeroRefPixelsBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mInsPtOffsetBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mInsPtOffsetPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mTccdZPointPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisZPointPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mTccdCurrZPointPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisCurrZPointPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisAccuOffsetsVWBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisAccuOffsetsPixBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mTccdDeltaTadcBak[issalignMAX_NUM_ARMS];
        issalignREF_OFFSET_REC      mIrisDeltaTadcBak[issalignMAX_NUM_ARMS];
        issalignREF_PIXEL_REC       mTccdFinalRefPixelsBak[issalignMAX_NUM_ARMS];
        issalignREF_PIXEL_REC       mIrisFinalRefPixelsBak[issalignMAX_NUM_ARMS];
        vltBYTES8                   mIrisFilter;

        issalignTADC_IN_DATA        mTadcInput;
        vltLOGICAL                  mVltiTracking;

        issCALCULATOR               mGenCalc;

        dbLISTID                    mRefPixInputList;
        dbLISTID                    mTadcInputList;
        dbLISTID                    mTrackingList;

        /*
         * Object storing the configuration of database attachments
         */
        evtEVENT                    mInstrumentChanged;

        vltLOGICAL                  mTccdBakTablesInitialized;
        vltLOGICAL                  mIrisBakTablesInitialized;
    };


#endif /*!issalignCOMMON_H*/

