#ifndef issalignIMAGE_H
#define issalignIMAGE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issalignIMAGE.h,v 3.17 2005/06/08 13:37:02 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-05-13 Added methods IsLagActive() and GetRefpixTableStatus()
*                     to extend verbose STATUS reply
* jott     2005-05-03 SPR VLTSW20050147: Added method CmdLogalgnCB()
* jott     2005-02-23 Added maintenance method GetUsedWlenId()
* jott     2005-01-25 SPRs VLTSW20050019 and VLTSW20040225:
*                     Added methods  CmdChrefpxCB(), CmdSettadcCB(),
*                     removed mTadcChanged event
*                     added member mLagStarted
* jott     2005-01-18 SPR VLTSW20040225 (allow IAS to be NONE)
*                     Added method GetIasId();
* jott     2004-12-08 Added method UpdateGuvScale()
* jott     2004-12-02 SPR VLTSW20040228:
*                     replaced mLagActive flag by command processing state
*                     issalignIMAGE_EXE_IRIS_LG_ACTIVE,
*                     added method GetStartupResult(),
*                     added member mStartupStatus
* jott     2004-11-04 SPR VLTSW20040228:
*                     added method IrisLagChangedCB()
*                     moved method AdjustWindow() into issalignCOMMON class
*                     added members mIrisLagChanged and mLagActive
* jott     2004-10-14 SPR VLTSW20040230/VLTSW20040228:
*                     added parameter 'allBeams' to method
*                     AlignCmdStateLookupReference() and parameter modStateProc
*                     to method SetAlignmentDoneCb() and renamed this method
*                     to SetAlignmenCbs()
*                     added members mTadcChanged, mModSubStateCb, mTadcFlag,
*                     mLagMsg
*                     added method GetProcStatus()
* rrollf   2004-10-11 SPR VLTSW20040230:
*                     added method IasChangedCB()
*                     added member mIasChanged
* jott     2004-10-08 SPR VLTSW20040230:
*                     added members mCurrIasId, mIris, mLib, mIrisInfo
*                     added methods UpdateAlignSensor(), IrisCompletedCB()
*                     moved CalculateTadcShift() and (content of)
*                     PrepareTadcInputList() into issalignCOMMON class
*                     moved members mTadcInputList, mTadcInput, and mGenCalc
*                     into issalignCOMMON class
*                     renamed CmdGetrefCB() to CmdLgOffsCB()
*                     removed mGuvScale and mThetaTccdVw (handled now within
*                     issalignCOMMON class)
* rrollf   2004-09-07 SPR VLTSW20040228: ISS to support IRIS - start/stop lab...
*                     added callbacks CmdStrtlagCB() and CmdStoplagCB()
* jott     2004-08-27 SPR VLTSW20040229:
*                     added method CmdGetrefCB()
* jott     2004-03-26 added parameter 'updateRefPoints' to method
*                     Initialize() to continue with SPR VLTSW20030462
* jott     2004-03-05 continued with port to APR2004 release
* jott     2004-02-26 ported to APR2004 release (renamed Init() method to
*                     Initialize())
* jott     2003-11-10 update refPointTable also onINIT command
*                     (SPR VLTSW20030462)
* jott     2003-10-15 added methods SetPointingOffsets(), AdjustWindow(),
*                     CalculateTadcShift(), PrepareTadcInputList(),
*                     and members  mGuvScale, mThetaTccdVw, mAralTccdWaveLen,
*                     mTadcInputList, mDbVltiPoint, mGenCalc
*                     to handle instrument specifc pointing offsets and TADC
*                     (SPR VLTSW20030425)
* jott      23/10/02  added method CheckAralStatus()
* jott      21/10/02  added member variables mOffsetV, mOffsetW, and
*                     mStoreError
*                     added methods SaveErrorStack() and RestoreErrorStack()
*                     removed member mErrOrigin
* jott      08/10/02  SPR VLTSW20020554:
*                     modified mErrOrigin to type issalignIMG_STATE
*                     added method AlignCmdAbort()
*                     added CmdChkimgCB() callback and members mTolerance,
*                     mMaxLoops, and mLoopCounter
*                     added parameter aligned to method AlignCmdStateReadError()
* jott      07/10/02  added  mExpStatus and mErrOrigin[128] for
*                     modified exception course for IMGALGN command
*                     (restart exposure loop)
* jott      13/08/02  added mWaitPicMsg to solve SPR VLTSW20020436
* jott      17/07/02  created
*/

/************************************************************************
 * Header file for the image aligning object of the ISS alignment process
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * Local definitions
 */

/* Size of reply buffer used by the issalignControl process */
#define issalignIMAGE_REPLY_BUFFER_SIZE  256
#define issalignIMAGE_SEND_BUFFER_SIZE   512


/*
 * Include parent class
 */
#include "db.h"
#include "eccsMsg.h"
#include "eccsEvt.h"
#include "evhSTD_COMMANDS.h"
#include "evhTASK.h"
#include "oslx.h"
#include "evhOBJ_CALLBACK.h"
#include "evhCOMMAND.h"
#include "evhTIMEOUT.h"
#include "issCALCULATOR.h"
#include "issalign.h"
#include "issalignIRIS.h"
#include "issalignCOMMON.h"


class issalignIMAGE : public evhTASK
    {
    public:
        issalignIMAGE(const dbSYMADDRESS   dbBasePoint,
                      const dbSYMADDRESS   dbVltiPoint,
                      issalignCOMMON       &commonLib);
        ~issalignIMAGE(void);

        /*
         * Command callbacks
         */
        evhCB_COMPL_STAT    CmdChkimgCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdCntimgCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdImalgnCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdSetupCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdWaitpicCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdLgOffsCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdStrtlagCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdStoplagCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdChrefpxCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdSettadcCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdLogalgnCB(msgMESSAGE &msg, void *udata);

        /*
         * Event callbacks
         */
        evhCB_COMPL_STAT    ArrayConfigChangedCB(evtEVENT_MSG  &msg,
                                                 void          *udata);
        evhCB_COMPL_STAT    IasChangedCB(evtEVENT_MSG  &msg,
                                         void          *udata);
        evhCB_COMPL_STAT    IrisLagChangedCB(evtEVENT_MSG  &msg,
                                             void          *udata);


        /*
         * Timeout callbacks
         */
        evhCB_COMPL_STAT    TimeoutWaitForSetupCB(evtEVENT_MSG  &msg,
                                                  void          *udata);

        /*
         * Callbacks for replies or timeouts to sent commands
         */
        evhCB_COMPL_STAT    ReplyCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    ErrorCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    TimeoutCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT       Initialize(vltLOGICAL updateRefPoints = ccsTRUE);
        ccsCOMPL_STAT       Stop(void);
        ccsCOMPL_STAT       SetAlignmentCbs(fndOBJECT       *object,
                                            evhCB_METHOD     alignDoneProc,
                                            evhCB_METHOD     modStateProc);
        ccsCOMPL_STAT       SetPointingOffsets(msgMESSAGE &msg);

        /*
         * Callback called when IRIS handler has finished its work
         */
        evhCB_COMPL_STAT    IrisCompletedCB(msgMESSAGE &msg, void *udata);
        ccsCOMPL_STAT       GetStartupResult(void) {return mStartupStatus;};

        /*
         * Maintenance methods to provide more detailed status information
         * for STATUS command
         */
        vltLOGICAL                   IrisLagUpdateActive(void) {return mIris.IsLagUpdateActive();};
        issalignIMG_STATE            GetCmdProcessStatus(void) {return mCmdProcState;};
        issalignTADC_USE_ZERO_WLEN   GetUsedWlenId(void) {return mLib.GetUsedWlenId();};
        vltLOGICAL                   IsLagActive(void)  {return mLagStarted;};
        issalignREF_PIX_TAB_STATUS   GetRefpixTableStatus(void) {return mLib.GetRefpixTableStatus();};


        /*
         * Public method returning the currently selected image alignment sensor
         */
        vltINT32            GetIasId(void) {return mCurrIasId;};

    protected:
        ccsCOMPL_STAT       AlignCmdStateMachine(issalignIMG_STATE  cmdState);
        ccsCOMPL_STAT       AlignCmdProcessed(ccsCOMPL_STAT result);
        ccsCOMPL_STAT       AlignCmdStateCheckAral(void);
        ccsCOMPL_STAT       AlignCmdStateCheckGuidingSystem(void);
        ccsCOMPL_STAT       AlignCmdStateStopExposure(void);
        ccsCOMPL_STAT       AlignCmdStateLookupReference(vltLOGICAL ignorePntOffs = ccsFALSE,
                                                         vltLOGICAL ignoreTadc    = ccsFALSE,
                                                         vltLOGICAL allBeams      = ccsTRUE);
        ccsCOMPL_STAT       AlignCmdStateSetup(void);
        ccsCOMPL_STAT       AlignCmdStateCalcError(void);
        ccsCOMPL_STAT       AlignCmdStateReadError(vltLOGICAL *aligned);
        ccsCOMPL_STAT       AlignCmdAbort(void);
        ccsCOMPL_STAT       SendCmdTo(msgCMD       command,
                                      char        *cmdBuffer,
                                      dbSYMADDRESS receiverAddr,
                                      dbSYMADDRESS timeoutAddr);
        ccsCOMPL_STAT       UpdateBeamGuidSysMapping(void);
        ccsCOMPL_STAT       CheckAralStatus(msgMESSAGE &msg);
        ccsCOMPL_STAT       UpdateAlignSensor(void);
        ccsCOMPL_STAT       GetProcStatus(vltINT32 *state, vltINT32 *subState);
	    ccsCOMPL_STAT       UpdateGuvScale(void);

        cmdPARAM_LIST       mParamList;
        oslxDICTIONARY      mDictionary;
        oslxSETUP           mSetup;
        oslxSETUP           mSendSetup;


    private:
        ccsCOMPL_STAT           SaveErrorStack(void);
        ccsCOMPL_STAT           RestoreErrorStack(void);

        dbSYMADDRESS            mDbBasePoint;
        dbSYMADDRESS            mDbVltiPoint;
        char                    mReplyBuffer[issalignIMAGE_REPLY_BUFFER_SIZE];
        char                    mSendBuffer[issalignIMAGE_SEND_BUFFER_SIZE];
        double                  mErrorV;
        double                  mErrorW;
        double                  mOffsetV;
        double                  mOffsetW;
        issalignIMG_STATE       mCmdProcState;
        int                     mCurrBeam;
        issalignREF_PIXEL_REC   mCurrRefPixels;
        vltLOGICAL              mArrayConfigChanged;
        vltLOGICAL              mReadyForCntimg;
        char                    mLastSendCmd[ccsCMD_LEN + 1];
        vltINT32                mExpStatus;
        vltINT32                mMaxLoops;
        vltINT32                mLoopCounter;
        vltDOUBLE               mTolerance;
        vltLOGICAL              mTadcFlag;
        ccsCOMPL_STAT           mStartupStatus;
        vltLOGICAL              mLagStarted;

        /*
         * Objects storing the configuration of database attachments
         */
        evtEVENT            mArrayConfTableChange;
        evtEVENT            mIasChanged;
        evtEVENT            mIrisLagChanged;

        /*
         * Callback to be executed when image alignment is finished
         */
        evhOBJ_CALLBACK     mImageAlignDoneCb;
        evhOBJ_CALLBACK     mModSubStateCb;

        /*
         * Objects storing the received commands
         */
        msgMESSAGE          mImageAlignMsg;
        msgMESSAGE          mLagMsg;
        msgMESSAGE          mWaitPicMsg;

        /*
         * Command handlers handling commands to other processes
         */
        evhCOMMAND          mImageAlignSender;

        /*
         * Mapping of guiding system to ISS beam (input channel)
         */
        issalignBeamGuideSysMap    mBeam2GuideSys[issMAX_NUM_ARMS];

        /*
         * Timeout and timer objects
         */
        evhTIMEOUT          *mWaitForSetupTimeout;

        /*
         * Error stack saving alignment errors over more than one callback
         */
        eccsERROR           mStoreError;

        /*
         * Identifier for currently used image alignment sensor
         */
        vltINT32            mCurrIasId;

        /*
         * Object handling IRIS specific stuff
         */
        issalignIRIS        mIris;

        /*
         * Common image alignment library object
         */
        issalignCOMMON      &mLib;

        /*
         * Contains information needed by the IRIS handler object
         */
        issalignIRIS_INFO   mIrisInfo;
    };


#endif /*!issalignIMAGE_H*/
