#ifndef issalignIRIS_H
#define issalignIRIS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issalignIRIS.h,v 3.17 2005/06/08 13:37:02 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* jott      2005-05-10  Added member mReplyFlag,
*                       added methods ChangeIasToIris(), CheckArrayForIris()
* jott      2005-02-21  SPR VLTSW20050019:
*                       Added methods ChangeZeroRefPoints(),
*                       DownloadNewZeroPixels(), ZeroRefDownloadCB(),
*                       ChangeTadcMode()
*                       added members mDlNewZeroPoints, mDlOnceMsg,
*                       mDownloadOnce, mDlOnceInfo, mOldTadcFlag
* jott      2004-12-01  SPR VLTSW20040228:
*                       Added parameter keepUVWoffsets to method ExeStartLag()
*                       Added members  mExeLagCompleted, mGotLagOkReply,
*                       mGotOkEvent, mExpLagState, mWaitForLagTimeout,
*                       mLagStateEvt, mStartupStatus, mUpdatesPerLag,
*                       Added methods IrisLagStateEvtCB(), GetStartupResult(),
*                       IsLagUpdateActive(), InitLagInfo()
* jott      2004-11-04  SPR VLTSW20040228:
*                       Renamed mTadcLabGuiding to mCycRefUpdate
*                       removed member array 'mRefPoints[]', because now
*                       contained within member 'mRefPixConfiguration',
*                       added methods SetCyclicUpdateFlag() and
*                       DownloadRefPix()
*                       SPR VLTSW20040230:
*                       Replaced parameter 'beam' with  parameter 'info' in
*                       method GetReadyForCntimgFlag()
*                       added member mCenterQuadrant
* jott      2004-10-18  SPR VLTSW20040228:
*                       added methods ReturnLagControl(), StartTadcLag(),
*                       IrisLagCmdCB(), ProcessImgalgnReply(), TadcUpdateRefCB(),
*                       UpdateAllIrisQuadrants(), StopTadcLag()
*                       added members mLagMsg, mLagCmdId, mLagCmds,
*                       mLagCmdCounter, mLagCmdEvt, mLagInfo, mAbortLagRequested,
*                       mLagSetrefActive, mLagSkippedUpdates, mLagNumErrors
*                       renamed mIrisCmds, mIrisCmdCounter and mIrisCmdEvt to
*                       mAlignCmds, mAlignCmdCounter and mAlignCmdEvt, mLgoffsBeam
*                       renamed IrisCmdCB() to IrisAlignCmdCB()
*                       added parameter 'info' to method GetQuadrantsForBeam()
*
*                       SPR VLTSW20040229:
*                       added method ForwardUvwOffsets(), IrisLgoffsCmdCB()
*                       added members mLgoffsCmds, mLgoffsCmdCounter,
*                       mLgoffsCmdEvt, mLgoffsMsg, mParamList
*                       renamed method ReturnAlignControl() to ReturnControl()
*
*                       SPR VLTSW20040230:
*                       added methods ProcessChkimgReply(), GetReadyForCntimgFlag()
*                       SetReadyForCntimgFlag()
*                       and filled methods ExeChkimg(), ExeCntimg(),
*                       added member mReadyForCntimg
* jott      2004-10-04  created
*/

/************************************************************************
 * Header file for the object of the ISS alignment process which handles
 * specific features of the image alignment sensor IRIS
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * Common VLTSW headers
 */
#include "evhTASK.h"
#include "evhOBJ_CALLBACK.h"
#include "evhDB_CMD_SEND.h"
#include "evhDB_CMD_SERIAL.h"
#include "evhTIMER.h"


/*
 * VLTI headers
 */
#include "issalign.h"
#include "issalignCOMMON.h"


class issalignIRIS : public evhTASK
    {
    public:
        issalignIRIS(const dbSYMADDRESS   dbBasePoint,
                     const dbSYMADDRESS   dbVltiPoint,
                     issalignCOMMON       &commonLib);
        ~issalignIRIS(void);

        ccsCOMPL_STAT       Init(void);

        /*
         * Command reply and event callbacks
         */
        evhCB_COMPL_STAT    IrisAlignCmdCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    IrisLagCmdCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    IrisLgoffsCmdCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    TadcUpdateRefCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    IrisLagStateEvtCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    DownloadOnceCB(msgMESSAGE &msg, void *udata);

        /*
         * Define callback of genaral image alignment callback to be executed
         * when IRIS alignment sequences are finished
         */
        ccsCOMPL_STAT       SetAlignmentDoneCb(fndOBJECT       *object,
                                               evhCB_METHOD     procedure);

        /*
         * transfers image alignment control to IRIS
         */
        ccsCOMPL_STAT       ExecuteAlign(msgMESSAGE &msg,
                                         issalignIRIS_INFO *info);

        /*
         * Start and stop laboratory guiding
         */
        ccsCOMPL_STAT       ExeStartLag(msgMESSAGE         &msg,
                                        issalignIRIS_INFO  *info,
                                        vltLOGICAL          keepUVWoffsets = ccsFALSE);
        ccsCOMPL_STAT       ExeStopLag(msgMESSAGE &msg);

        /*
         * Clear accumulated OFFSGUV offset table
         */
        ccsCOMPL_STAT       ResetAccuOffsguv(void);

        /*
         * Start/stop cyclic update of reference pixels
         */
        ccsCOMPL_STAT       StartTadcLag(issalignIRIS_INFO  *info = NULL);
        ccsCOMPL_STAT       StopTadcLag(void);

        /*
         * Start offseting IRIS in course of an OFFGUV command to ISS
         */
        ccsCOMPL_STAT       ForwardUvwOffsets(msgMESSAGE         &msg,
                                              issalignIRIS_INFO  &irisInfo);

        /*
         * Download reference pixels once
         */
        ccsCOMPL_STAT       DownloadRefPix(issalignREF_PIX_DL_TRIGGER  event,
                                           issalignREF_PIX_CONF       *configuration);

        /*
         * Methods to get and set the ready for CNTIMG flags for IRIS
         */
        vltLOGICAL          GetReadyForCntimgFlag(issalignIRIS_INFO &info,
                                                  vltINT32          *quadrant);
        ccsCOMPL_STAT       SetReadyForCntimgFlag(vltINT32 quadrant, vltLOGICAL value);

        /*
         * Return result of initialization during start-up
         */
        ccsCOMPL_STAT       GetStartupResult(void) {return mStartupStatus;};

        /*
         * Returns whether currently waiting for reply to SETREF command of
         * cyclic ref. pixel update
         */
        vltLOGICAL          IsLagUpdateActive(void) {return mLagSetrefActive;};

        /*
         * Provide information for cyclic update of reference pixels
         */
        ccsCOMPL_STAT       InitLagInfo(issalignIRIS_INFO  *lagInfo);

        /*
         * Check whether current array configuration is compliant for use of IRIS
         */
        ccsCOMPL_STAT       CheckArrayForIris(issalignIRIS_INFO  &arrayInfo);

        /*
         * Change IRIS Zero Reference Points
         */
        ccsCOMPL_STAT       ChangeZeroRefPoints(msgMESSAGE                &msg,
                                                issalignIRIS_INFO          info,
                                                issalignREF_PIX_IRIS_REC  *pixels,
                                                vltLOGICAL                 updateRefPixels);
        /*
         * Change TADC mode for reference pixel computation
         */
        ccsCOMPL_STAT       ChangeTadcMode(msgMESSAGE                &msg,
                                           issalignIRIS_INFO          info,
                                           vltLOGICAL                 tadcOld);
        /*
         * IRIS becomes the new selected image alignment sensor
         */
        ccsCOMPL_STAT       ChangeIasToIris(msgMESSAGE         &msg,
                                            issalignIRIS_INFO  &info);


    protected:
        /*
         * Methods to start execution of the different image alignment commands
         */
        ccsCOMPL_STAT       ExeImgalgn(void);
        ccsCOMPL_STAT       ExeCntimg(void);
        ccsCOMPL_STAT       ExeChkimg(void);
        ccsCOMPL_STAT       ProcessImgalgnReply(msgMESSAGE &msg, void *udata,
                                                char *replyBuffer);
        ccsCOMPL_STAT       ProcessChkimgReply(msgMESSAGE &msg, void *udata,
                                               char *replyBuffer);
        ccsCOMPL_STAT       ReturnControl(msgMESSAGE              &msg,
                                          evtEVENT                &event,
                                          evhDB_CMD_SERIAL        &cmdHandler,
                                          vltINT32                *counter,
                                          issalignIRIS_COMPL_STAT  status,
                                          char                    *replyBuffer = NULL);
        ccsCOMPL_STAT       ReturnLagControl(issalignIRIS_COMPL_STAT   status,
                                             char                     *replyBuffer = NULL);
        ccsCOMPL_STAT       CheckLabGuidingStatus(vltLOGICAL active);
        ccsCOMPL_STAT       GetQuadrantsForBeam(vltINT32           beam,
                                                issalignIRIS_INFO &info,
                                                vltINT32          *quadrants);
        ccsCOMPL_STAT       UpdateAllIrisQuadrants(evhDB_CMD_SEND     &cmds,
                                                   vltINT32           &counter,
                                                   issalignIRIS_INFO  &info);
        ccsCOMPL_STAT       SetCyclicUpdateFlag(vltINT32 value);



    private:
        dbSYMADDRESS            mDbBasePoint;
        dbSYMADDRESS            mDbVltiPoint;
        cmdPARAM_LIST           mParamList;

        msgMESSAGE              mAlignMsg;
        vltINT32                mAlignCmdId;
        msgMESSAGE              mLagMsg;
        vltINT32                mLagCmdId;
        vltLOGICAL              mCycRefUpdate;
        vltLOGICAL              mAbortLagRequested;
        vltLOGICAL              mLagSetrefActive;
        vltINT32                mLagSkippedUpdates;
        vltINT32                mLagNumErrors;
        vltUINT32               mUpdatesPerLag;
        vltINT32                mQuadrantOfChannel[issalignNUM_INPUT_CH_FOR_IRIS];
        msgMESSAGE              mLgoffsMsg;
        vltLOGICAL              mReadyForCntimg[issalignNUM_IRIS_QUADRANTS];
        vltINT32                mCenterQuadrant;
        vltLOGICAL              mExeLagCompleted;
        vltLOGICAL              mGotLagOkReply;
        vltLOGICAL              mGotOkEvent;
        vltLOGICAL              mExpLagState;
        ccsCOMPL_STAT           mStartupStatus;
        msgMESSAGE              mDlOnceMsg;
        vltINT32                mNewZeroPixBeam;
        vltLOGICAL              mOldTadcFlag;
        vltLOGICAL              mReplyFlag;


        /*
         * Timeouts
         */
        vltINT32                mShortTimeout;
        vltINT32                mMediumTimeout;
        vltINT32                mLongTimeout;

        /*
         * Callback to be executed when image alignment by IRIS is finished
         */
        evhOBJ_CALLBACK         mImageAlignDoneCb;

        /*
         * IRIS Command handling members
         */
        evhDB_CMD_SERIAL        mAlignCmds;
        vltINT32                mAlignCmdCounter;
        evhDB_CMD_SERIAL        mLagCmds;
        vltINT32                mLagCmdCounter;
        evhDB_CMD_SERIAL        mLgoffsCmds;
        vltINT32                mLgoffsCmdCounter;
        vltINT32                mLgoffsBeam;
        evhDB_CMD_SEND          mDownloadOnce;



        /*
         * Timer for cyclic update of reference pixels and timeout to wait
         * for change of laboratory guiding state
         */
        evhTIMER               *mRefPixTimer;
        evhTIMEOUT             *mWaitForLagTimeout;

        /*
         * Events observing the ERROR state of evhDB_CMD_SERIAL objects
         * or scanned database attributes
         */
        evtEVENT                mAlignCmdEvt;
        evtEVENT                mLagCmdEvt;
        evtEVENT                mLgoffsCmdEvt;
        evtEVENT                mLagStateEvt;

        /*
         * Common image alignment library object
         */
        issalignCOMMON          &mLib;

        /*
         * Current reference pixel configuration and alignment information
         */
        issalignREF_PIX_CONF    mRefPixConfiguration;
        issalignIRIS_INFO       mAlignInfo;
        issalignIRIS_INFO       mLagInfo;
        issalignIRIS_INFO       mDlOnceInfo;
    };


#endif /*!issalignIRIS_H*/
