#ifndef issalignMAIN_TASK_H
#define issalignMAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issalignMAIN_TASK.h,v 3.17 2005/06/08 13:37:02 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-05-11 Added member mWaitForInitFlag
* jott     2004-12-14 SPR VLTSW20040230 (issmsw sub-state ALIGNING):
*                     Added members mIssmswSender, mIssmswMsg, mAlignMsg,
*                     added method IssmswCB()
*                     added parameters 'msg' and 'udata' to method
*                     ImageCmdCompletedCB()
* jott     2004-10-21 Added method SetSubStateCB()
* jott     2004-03-26 Added parameter 'vltiTracking' to Init() method to handle
*                     update of refPointTable on INIT command in dependency
*                     of VLTI tracking status (SPR VLTSW20030462)
* jott     2003-10-15 Added method CmdPntOffsCB() to support instrument
*                     specific pointing offsets and TADC (SPR VLTSW20030425)
*                     added VerboseCB() callback to dynamically switch trace
*                     logging on or off
* rrollf    30/01/03  Revisited implementation of reference pixel manager
* rka/awa  2002-11-18 include refpixMngr
* jott      16/07/02  created
*/

/************************************************************************
 * Header file for the main object of the ISS alignment process
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * Local definitions
 */

/* Size of reply buffer used by the fntCoordination process */
#define issalignMAIN_REPLY_BUFFER_SIZE  256


/*
 * Include parent class
 */
#include "db.h"
#include "msg.h"
#include "eccsMsg.h"
#include "eccsEvt.h"
#include "evhSTD_COMMANDS.h"
#include "evhDB_COMMAND.h"
#include "issalignIMAGE.h"
#include "issalignREFPIX_MNGR.h"


class issalignMAIN_TASK : public evhSTD_COMMANDS
    {
    public:
        issalignMAIN_TASK(const dbSYMADDRESS    dbBasePoint,
                          issalignIMAGE        &imageAligner);
        ~issalignMAIN_TASK(void);
        ccsCOMPL_STAT  Recover(void);

        /*
         * Command callbacks
         */
        evhCB_COMPL_STAT    InitCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    StatusCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    StopCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    ExitCB(msgMESSAGE &msg, void *udata);

        evhCB_COMPL_STAT    CmdCntimgCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdImalgnCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdSetupCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdWaitpicCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    CmdPntOffsCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    VerboseCB(msgMESSAGE &msg, void *udata);

        /*
         * Callbacks to be executed when alignment sequences are finished
         * or a an instance of another class wants to modify the process
         * sub-state
         */
        evhCB_COMPL_STAT    ImageCmdCompletedCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    SetSubStateCB(msgMESSAGE &msg, void *udata = NULL);
        evhCB_COMPL_STAT    IssmswCB(msgMESSAGE &msg, void *udata);

    protected:
        ccsCOMPL_STAT       Init(vltLOGICAL vltiTracking);
        vltINT32            GetSubState(void);
        ccsCOMPL_STAT       SetSubState(vltINT32 subState);
        ccsCOMPL_STAT       RejectIfNotInStateList(msgMESSAGE &msg,
                                                   int        *listOfAllowedStates,
                                                   int         numberOfStates);
        ccsCOMPL_STAT       AcceptIfNotInSubStateList(msgMESSAGE &msg,
                                                      int        *listOfSubStates,
                                                      int         numberOfSubStates);
        /*
         * Objects for specific tasks
         */
        issalignIMAGE       &mImageAligner;

        /*
         * Command handler for mode switching commands
         */
        evhCOMMAND          mIssmswSender;

    private:
        dbSYMADDRESS        mDbBasePoint;
        char                mReplyBuffer[issalignMAIN_REPLY_BUFFER_SIZE];
        issalignREFPIX_MNGR mRefpixMngr;
        msgMESSAGE          mIssmswMsg;
        msgMESSAGE          mAlignMsg;
        vltLOGICAL          mWaitForInitFlag;
    };

#endif /*!issalignMAIN_TASK_H*/
