#ifndef ISSARRAY_H
#define ISSARRAY_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issarray.h,v 2.26 2005/03/21 13:03:12 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-02-02 SPR VLTSW20050031:
*                     Extendend structure issarrayTUNNEL_RECORD to support
*                     two light duct covers
*                     added external variable issarrayOldVersionMap
*                     added external methods GetLightCoversForStation(),
*                     GetM12RecordForStation(),
*                     added enumeration issarrayLIGHT_DUCT_COVER_ID
* jott     2005-01-31 SPR VLTSW20050031:
*                     Added struvture issarrayTUNNEL_RECORD.
* jott     2005-01-13 SPR VLTSW20040396
*                     added method GetInputChForM16Pos()
* jott     2005-01-11 SPR VLTSW20040399:
*                     Added structure issarrayTEST_CONF
* jott     2004-11-02 SPR VLTSW20040235:
*                     Added structure issarrayIRIS_BEAM_CONF to provide
*                     information about telescope and quadrant associated to
*                     each beam
* jott     2004-08-18 SPR VLTSW20040225:
*                     added enumeration issarrayIRIS_ACTION
*                     added structure issarrayIRIS_QUAD_CONFIG
*                     added members 'ISSARRAY_CMDPROC_SETUP_IRIS_xxx' to
*                     enumeration issarrayCOMMAND_STATE,
*                     added enumeration issarrayIRIS_CFG_STATUS
* jott     2004-04-20 Added structure issarrayTEL_POSITION to optimize database
*                     access (SPR VLTSW20030515)
* jott     2004-02-06 Added methods IsEasternDl(), IsWesternDl(),
*                     IsEasternStation() and IsWesternStation() to solve
*                     SPR VLTSW20030501
* jott      11/08/03  Take into account input channel for pupil out computation
*                     added input channel to structure issarrayPUPIL_INFO
*                     (VLTSW20030322)
* jott      14/10/02  mapped issarrayCONFIG_COMPLETE to issVLTI_CFG_FIELDS2,
*                     adding pupilIn0 and pupilOut0 to Array Configuration
*                     table (SPR VLTSW20020429)
*                     added oplZero to structure issarraySETDELD_PARAM
* jott      01/08/02  extended structure issarraySETDELD_PARAM and added
*                     structure issarrayPUPIL_INFO to solve SPR
*                     VLTSW20020429
* jott      08/04/02  moved declaration of Array Configuration Table structure to
*                     module iss and map definition issarrayCONFIG_COMPLETE to it.
* jott      06/11/01  extendet enum issarrayCOMMAND_STATE with value
*                     ISSARRAY_CMDPROC_SETDEL_SET, ISSARRAY_CMDPROC_SETDEL_INIT,
*                     ISSARRAY_CMDPROC_SETDEL_STARTUP due to SPR VLTSW20010628
* jott      20/07/01  added definitions ISSARRAY_CMDPROC_INITIALIZE
*                     and ISSARRAY_CMDPROC_STARTUP to solve SPR VLTSW20010322
* jott      28/06/01  code reviewed
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      20/02/01  second version
* jott      20/12/00  first version
* jott      11/12/00  created
*/

/************************************************************************
 * Header file containing common type definitions, enumeration values etc.
 * which are commonly used by all issarray classes
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include <string.h>

#include "fndNAME_MAP.h"
#include "issConfigTable.h"

/*
 * Depth of interferometer arm specification
 */
typedef enum
    {
    ISSARRAY_CONF_LEVEL_DISABLE,
    ISSARRAY_CONF_LEVEL_STATION,
    ISSARRAY_CONF_LEVEL_COMPLETE
    } issarrayCONFIG_LEVEL;

/*
 * Structure describing a complete configuration table record
 */
typedef issVLTI_CFG_FIELDS2 issarrayCONFIG_COMPLETE;

/*
 * Structure describing a complete test configuration table record
 */
typedef struct
    {
    vltINT32            hpDummy;  /* Alignment: Not needed under Linux, but needed for HP */
    vltINT32            telescopeName;
    vltBYTES20          location;
    vltINT32            physDelayLine;
    vltINT32            inputChannel;
    vltINT32            swYardConfig;
    vltINT32            secInputChannel;
    vltINT32            secSwYardConfig;
    vltINT32            opticsMidi;
    vltINT32            opticsVinci;
    vltINT32            opticsFinito;
    vltINT32            opticsPrimaFsuA;
    vltINT32            opticsPrimaFsuB;
    vltINT32            opticsAmber;
    vltDOUBLE           oplZero;
    vltDOUBLE           pupIn0;
    vltDOUBLE           pupOut0;
    } issarrayTEST_CONF;

/*
 * Structure containing the telescope position in UVW coordinates (needed for
 * SETDELD command
 */
typedef struct
    {
    vltDOUBLE        posU;
    vltDOUBLE        posV;
    vltDOUBLE        posW;
    } issarrayTEL_POSITION;

/*
 * Structure describing the first 3 fields configuration table record
 */
typedef struct
    {
    vltINT32            telescopeNum;
    vltINT32            telescopeName;
    vltBYTES20          location;
    } issarrayCONFIG_SHORT;

/*
 * Structure describing a configuration table record as far as needed
 * for the OPL0 calculation
 */
typedef struct
    {
    vltINT32            telescopeNum;
    vltINT32            telescopeName;
    vltBYTES20          location;
    vltINT32            physDelayLine;
    vltINT32            inputChannel;
    vltINT32            swYardConfig;
    } issarrayCONFIG_OPL0;

/*
 * Structure describing a set of subsystems which is
 * requested by a SETCONF/SETARR command
 */
typedef struct
    {
    vltINT32      telescopeId;
    vltINT32      stationId;
    vltINT32      delayLineId;
    vltINT32      inputChannel;
    } issarrayNEW_SUBSYSTEMS;

/*
 * Definitions for the command processing states
 * used by the command processing state machine
 */
typedef enum
    {
    /* general states */
    ISSARRAY_CMDPROC_ERROR,
    ISSARRAY_CMDPROC_IDLE,              /* reserved for future use */
    ISSARRAY_CMDPROC_CONFCMD_COMPLETED,
    ISSARRAY_CMDPROC_CONFCMD_NO_REPLY,
    /* state for starting up                          */
    ISSARRAY_CMDPROC_STARTUP,
    ISSARRAY_CMDPROC_SETDEL_STARTUP,
    /* state for processing a INIT/ONLINE command     */
    ISSARRAY_CMDPROC_INITIALIZE,
    ISSARRAY_CMDPROC_SETDEL_INIT,
    /* states for processing a DISARM command         */
    ISSARRAY_CMDPROC_DISARM_ARM,
    ISSARRAY_CMDPROC_SETDEL_DISARM,
    /* states for processing a SETARR/SETCONF command */
    ISSARRAY_CMDPROC_PING_DESTINATIONS_COU,
    ISSARRAY_CMDPROC_PING_DESTINATIONS_TEL,
    ISSARRAY_CMDPROC_CHANGE_MSWTABLE,
    ISSARRAY_CMDPROC_RECONF_COMPONENTS, /* reserved for future use */
    ISSARRAY_CMDPROC_SETDELD_SET,
    ISSARRAY_CMDPROC_SETDEL_SET,
    /* states related to IRIS configuration */
    ISSARRAY_CMDPROC_SETUP_IRIS_STARTUP
    } issarrayCOMMAND_STATE;

/*
 * Structure containing all information needed
 * to process the replies of a configuration command
 */
typedef struct
    {
    int                      commandNameId;
    issarrayCONFIG_COMPLETE  armConfiguration;
    vltUINT16                currMsgId;
    issarrayCOMMAND_STATE    processState;
    int                      sentCmdId;
    } issarrayCONF_CMD_DESCR;


/*
 * Structure containing all the values which
 * are parameters of the SETDELD command to the delay line
 * control system
 */
typedef struct
    {
    vltINT32         delId;
    vltINT32         telName;
    vltBYTES20       station;
    vltDOUBLE        posU;
    vltDOUBLE        posV;
    vltDOUBLE        posW;
    vltINT32         inputCh;
    vltDOUBLE        oplZero;
    vltDOUBLE        pupilIn;
    vltDOUBLE        pupilOut;
    } issarraySETDELD_PARAM;

/*
 * Structure containing a module name and its related
 * destination process as given in the mode switching table
 */
typedef struct
    {
    vltBYTES32       modName;
    vltBYTES32       destination;
    } issarrayMODULE_DEST;

/*
 * Structure containing information which was obtained during the course of
 * calculatiing OPL0 and is needed again when calculating the pupil positions
 */
typedef struct
    {
    vltINT32         delId;
    vltINT32         telName;
    vltDOUBLE        dlMechZeroU;
    vltDOUBLE        stationU;
    vltDOUBLE        stationdU;
    vltDOUBLE        stationV;
    vltDOUBLE        stationdV;
    vltDOUBLE        dlMechZeroVIn;
    vltDOUBLE        dlMechZeroVOut;
    vltINT32         inputChannel;
    } issarrayPUPIL_INFO;


/*
 * Enumeration for actions started by any object
 */
typedef enum
    {
    ISSARRAY_IDLE_LOOP,
    ISSARRAY_CONFIGURE_STARTUP,
    ISSARRAY_CONFIGURE_INIT,
    ISSARRAY_CONFIGURE_SETIAS,
    ISSARRAY_CONFIGURE_SETCONF,
    ISSARRAY_CONFIGURE_SETARR,
    ISSARRAY_CONFIGURE_DISARM,
    } issarrayIRIS_ACTION;

/*
 * IRIS quadrant configuration
 */
typedef struct issarrayIRIS_QUAD_CONFIG
    {
    vltINT32       inputChannel;
    vltINT32       quadrant;
    vltINT32       active;      // used INT32 instead of LOGICAL to avoid problems when reading whole table in a single db access
    vltBYTES32     guidingMode;
    vltBYTES32     feed;
    } issarrayIRIS_QUAD_CONFIG;
vltLOGICAL  operator==(issarrayIRIS_QUAD_CONFIG X, issarrayIRIS_QUAD_CONFIG Y);

/*
 * IRIS - VLTI beam composed configuration
 */
typedef struct issarrayIRIS_VLTI_CONFIG
    {
    issarrayIRIS_QUAD_CONFIG    iris;
    vltBYTES32                  telescope;
    vltBYTES32                  station;
    } issarrayIRIS_VLTI_CONFIG;
vltLOGICAL  operator==(issarrayIRIS_VLTI_CONFIG X, issarrayIRIS_VLTI_CONFIG Y);

/*
 * Enumeration for IRIS configuration of a VLTI beam
 */
typedef enum
    {
    ISSARRAY_BEAM_CFG_NOT_USED         = 0,
    ISSARRAY_BEAM_CFG_ERROR,          // 1
    ISSARRAY_BEAM_CFG_OK              // 2
    } issarrayIRIS_CFG_STATUS;

/*
 * Structure containing beam number, telescope name and IRIS quadrant
 */
typedef struct
    {
    vltINT32    beamNum;
    vltBYTES32  telescope;
    vltINT32    quadrant;
    vltINT32    secQuadrant;
    vltINT32    quality;
    } issarrayIRIS_BEAM_CONF;


/*
 * Structure containing a M12 and light cover table record
 */
typedef struct
    {
    vltBYTES20    LightDuctName;
    vltINT32      M12_DL_installed[8];
    vltINT32      S_Cover_installed;
    vltINT32      N_Cover_installed;
    } issarrayTUNNEL_RECORD;


/*
 * Identifiers for types of light duct covers
 */
typedef enum
    {
    issarrayLIGHT_DUCT_COVER_UNDEFINED_ID,
    issarrayLIGHT_DUCT_COVER_SOUTH_ID,
    issarrayLIGHT_DUCT_COVER_NORTH_ID
    } issarrayLIGHT_DUCT_COVER_ID;


/*
 * mapped names
 */
extern fndNAME_MAP  issarraySubStateNameMap;
extern fndNAME_MAP  issarrayCommandNameMap;
extern fndNAME_MAP  issarrayTelescopeNameMap;
extern fndNAME_MAP  issarrayCoudeNameMap;
extern fndNAME_MAP  issarrayStationNameMap;
extern fndNAME_MAP  issarrayDelayLineNameMap;
extern fndNAME_MAP  issarraySwitchyardConfigMap;
extern fndNAME_MAP  issarrayFeedOpticsConfigMap;
extern fndNAME_MAP  issarrayIrisActionMap;
extern fndNAME_MAP  issarrayOldVersionMap;


vltLOGICAL IsEasternDl(int delId);
vltLOGICAL IsWesternDl(int delId);
vltLOGICAL IsEasternStation(int stationId);
vltLOGICAL IsWesternStation(int stationId);
vltLOGICAL IsPrimaryInput(vltINT32 channel);
vltLOGICAL IsSecondaryInput(vltINT32 channel);
vltINT32   GetInputChForM16Pos(char *m16Pos);
issarrayLIGHT_DUCT_COVER_ID   GetLightCoversForStation(vltINT32 stationId);
int GetM12RecordForStation(vltINT32 stationId);

#include "issarray.icc"

#endif /* !ISSARRAY_H */

/* __oOo__  */
