#ifndef issarrayIRIS_H
#define issarrayIRIS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issarrayIRIS.h,v 2.26 2005/03/21 13:03:14 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* jott      2005-01-17  Added method GetCurrIas() (SPR VLTSW20040225)
* jott      2004-11-26  Added methods UpdatePubIrisConfTable(),
*                       UpdateInput2QuadTable(), and IsBusy()
* jott      2004-10-29  SPRs VLTSW20040225, VLTSW20040235:
*                       Added methods StopLagging(), StopLagReplyCB(),
*                       added members 'mAlignControl', 'mIrisUsedBak',
*                       'mBeamTelQuad[]'
* jott      2004-09-21  added member array mIrisConfStatus, mIrisConfAlreadySet,
*                       mIrisBeamStatusDbAddr, mStoredOldConfBeams
*                       added methods UpdateIrisStatus(), BackupIasData(void),
*                       RestoreIasData(void)
* jott      2004-08-18  Created (see SPR VLTSW20040225)
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


/*
 * System headers
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*
 * VLTWS headers
 */
#include "db.h"
#include "eccsDb.h"
#include "eccsEvt.h"
#include "eccsTestTools.h"
#include "evhStates.h"
#include "evhTASK.h"
#include "evhDB_CMD_SERIAL.h"
#include "evhCOMMAND.h"

/*
 * Application headers
 */
#include "issarrayErrors.h"
#include "issarrayDefines.h"
#include "issarray.h"


class issarrayIRIS : public evhTASK
    {
    public:

        issarrayIRIS(const dbSYMADDRESS  processDbPoint,
                     const dbSYMADDRESS  irisSenderDbPoint);
        ~issarrayIRIS(void);

        ccsCOMPL_STAT     Init(void);
        ccsCOMPL_STAT     Reset(void);
        ccsCOMPL_STAT     SetAction(issarrayIRIS_ACTION action, msgMESSAGE &msg);
        ccsCOMPL_STAT     SetNewVltiConfiguration(issarrayCONFIG_COMPLETE *configuration = NULL);

        /* Command callbacks */
        evhCB_COMPL_STAT  CmdGetIasCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT  CmdSetIasCB(msgMESSAGE &msg, void *udata);


        /* Install callback for complation of IRIS configuration */
        ccsCOMPL_STAT   Install(evhCALLBACK &completionCb);

        /* Backup and restore Image Alignment Sensor configuration */
        ccsCOMPL_STAT   BackupIasData(void);
        ccsCOMPL_STAT   RestoreIasData(void);
        vltLOGICAL      IsBusy(void);
        char           *GetCurrIas(void) { return (char *)mCurrIASensor; };

    protected:
        evhCB_COMPL_STAT  OkReplyCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT  ErrorCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT  TimeoutCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT  StopLagReplyCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT     UpdateIris(void);
        ccsCOMPL_STAT     UpdateIrisStatus(void);
        ccsCOMPL_STAT     UpdatePubIrisConfTable(void);
        ccsCOMPL_STAT     UpdateInput2QuadTable(void);
        ccsCOMPL_STAT     IrisConfDone(msgMESSAGE  retMsg, void *udata);
        ccsCOMPL_STAT     StopLagging(void);

    private:
        dbSYMADDRESS                mProcessDbPoint;
        dbSYMADDRESS                mIASensorDbAddr;
        dbSYMADDRESS                mChannelConfDbAddr;
        dbSYMADDRESS                mIrisBeamStatusDbAddr;
        issarrayIRIS_ACTION         mCurrAction;
        vltBYTES20                  mCurrIASensor;
        cmdPARAM_LIST               mParamList;
        vltLOGICAL                  mIrisUsed;
        vltLOGICAL                  mIrisUsedBak;
        issarrayCONFIG_COMPLETE     mStoredVltiArray[issMAX_NUM_ARMS];
        vltINT32                    mStoredOldConfBeams[issMAX_NUM_ARMS];
        vltLOGICAL                  mNoBeamConfigured;

        /* IRIS quadrant configuration in dependency of input channels */
        issarrayIRIS_QUAD_CONFIG    mInput2Quadrant[issarrayIRIS_MAX_USED_INPUT_CH];
        issarrayIRIS_VLTI_CONFIG    mIrisVltiConf[issMAX_NUM_ARMS];
        issarrayIRIS_BEAM_CONF      mBeamTelQuad[issMAX_NUM_ARMS];
        vltINT32                    mIrisConfStatus[issMAX_NUM_ARMS];
        vltLOGICAL                  mIrisConfAlreadySet[issMAX_NUM_ARMS];

        /* Timeouts */
        vltINT32                    mShortTimeout;
        vltINT32                    mMediumTimeout;
        vltINT32                    mLongTimeout;

        /* Command handling objects */
        evhCOMMAND                  mAlignControl;
        evhDB_CMD_SERIAL            mIrisCmds;
        vltINT32                    mIrisCmdCounter;

        /* Events observing the ERROR state evhDB_CMD_SERIAL object */
        evtEVENT                    mIrisCmdEvt;

        /* Callback to install */
        evhCALLBACK                *mCompletedCB;

        /* Stored Messages */
        msgMESSAGE                  mIasMsg;
        msgMESSAGE                  mActionMsg;
    };


#endif /* !issarrayIRIS_H */

