#ifndef issarrayMAIN_TASK_H
#define issarrayMAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issarrayMAIN_TASK.h,v 2.26 2005/03/21 13:03:13 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-01-31 SPR VLTSW20050031:
*                     Prepared for later check of M12 data by adding
*                     a reference to issarrayTUNNEL_TABLE_MNGR object
*                     added member mTunnelMngr
* jott     2005-01-13 SPR VLTSW20040396:
*                     Added parameter telescopeName to CheckM16Position()
* jott     2005-01-11 SPR VLTSW20040399:
*                     Added method CmdTstConfigCB()
* jott     2004-11-29 SPR VLTSW20040225:
*                     Added member mStartUpWarning
*                     Added CheckIrisLabGuiding()
* jott     2004-11-10 added member mConfIiris to suppress sending of commands
*                     to IRIS during start-up and INIT command
* jott     2004-08-19 SPR VLTSW20040225:
*                     Added reference to IRIS handling object,
*                     added members mIrisMngr, mSendReplyAfterIrisConf,
*                     mNextIrisAction
*                     added method IrisFinishedCB()
* jott     2004-04-19 Added member 'mRestoredArrayFromFile' to optimize database
*                     access (SPR VLTSW20030515)
* jott     2004-02-06 Added method CheckTelDelCombination() to check consistency
*                     of telescope and used delay line (SPR VLTSW20030501)
* jott     2003-09-22 Added parameter 'commandId' to method CheckM16Position()
*                     to solve SPR VLTSW20030397
* jott      02/04/03  Added method CheckM16Position() and M16 related variables
*                     to solve SPR VLTSW20020683
* rrollf    05/03/03  Use VERBOSE command to switch on / off trace logging
*                     Added mSynchMngr object (see SPR VLTSW20020539)
* rrollf    28/01/03  Adapted to VLTSW Release APR2003
* jott      15/10/02  added callback CmdGetPupilPosCB() for GETPUP command
* jott      21/02/02  added member mLastPointModel to solve SPR VLTSW20020013
* jott      06/11/01  added methods GetDelayLines() and
*                     CmdProcessInSetDelSetState()
* jott      19/07/01  added OnlineCB(), CmdProcessInInitState() and
*                     functionality to synchronize with DLCS when going
*                     ONLINE (SPR VLTSW20010322)
* jott      28/06/01  code reviewed
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      20/02/01  second version
* jott      20/12/00  first version
* jott      12/12/00  created
*/

/************************************************************************
 * Header file for class issarrayMAIN_TASK
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * Include error messages
 */
#include "eccsTestTools.h"
#include "issarrayErrors.h"

/*
 * Include parent classes
 */
#include "evhSTD_COMMANDS.h"
#include "evhCOMMAND.h"
#include "issarray.h"
#include "issarrayTABLE_MNGR.h"
#include "issarraySYNCH_MNGR.h"
#include "issarrayIRIS.h"
#include "issarrayTUNNEL_TABLE_MNGR.h"

using namespace std;

class issarrayMAIN_TASK : public evhSTD_COMMANDS
    {
    public:
        issarrayMAIN_TASK(const dbSYMADDRESS         dbControlPoint,
                          const dbSYMADDRESS         dbBasePoint,
                          const dbSYMADDRESS         dbSynchMngrPoint,
                          issarrayTABLE_MNGR        &pTableMngr,
                          issarrayIRIS              &irisMngr,
                          issarrayTUNNEL_TABLE_MNGR &tunnelMngr);
        ~issarrayMAIN_TASK(void);

        evhCB_COMPL_STAT CmdGetArrayCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdGetConfigCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdSetArrayCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdSetConfigCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdTstConfigCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdGetOptPathLenCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdGetPupilPosCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT CmdDisarmCB(msgMESSAGE &msg, void *udata);

        evhCB_COMPL_STAT ReplyCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT ErrReplyCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT TimeoutCB(msgMESSAGE &msg, void *udata);

        evhCB_COMPL_STAT IrisFinishedCB(msgMESSAGE &msg, void *udata);

        /*
         * overriding methods for issarray specific reactions to
         * standard commands
         */
        evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT    Recover();

    protected:
        ccsCOMPL_STAT             SetSubState(vltINT32 newState);
        ccsCOMPL_STAT             GetSubState(vltUINT32 *subState);
        ccsCOMPL_STAT             Init(int commandId);
        ccsCOMPL_STAT             RejectIfNotInStateList(msgMESSAGE &msg,
                                                         int        *listOfAllowedStates,
                                                         int         numberOfStates,
                                                         vltLOGICAL *rejected);
        ccsCOMPL_STAT             ConvertParamStringToStruct(issarrayCONFIG_LEVEL level,
                                                             char *buffer,
                                                             issarrayCONFIG_COMPLETE *newConfiguration);
        ccsCOMPL_STAT             GetRequiredParamString(issarrayCONFIG_LEVEL level,
                                                         issarrayCONFIG_COMPLETE *newConfiguration,
                                                         issarrayCONFIG_COMPLETE *oldConfiguration,
                                                         char *paramBuffer,
                                                         int paramBufferSize);
        ccsCOMPL_STAT             AddCommandToList(issarrayCONF_CMD_DESCR *newCmd);
        issarrayCONF_CMD_DESCR   *FindCommandInList(vltUINT16 currMsgId);
        ccsCOMPL_STAT             DeleteCommandFromList(vltUINT16 currMsgId);
        ccsCOMPL_STAT             GetCommandDestination(const char *subsystem,
                                                        ccsENVNAME *destEnv,
                                                        ccsPROCNAME  *destProc);

        ccsCOMPL_STAT             GetDelayLines(char *buffer, int bufLen);
        ccsCOMPL_STAT             CheckM16Position(vltINT32 dlId,
                                                   vltINT32 inputCh,
                                                   int      commandId,
                                                   vltINT32 telescopeName);
        ccsCOMPL_STAT             CheckTelDelCombination(issarrayCONFIG_COMPLETE *configuration,
                                                         int                      commandId);
        ccsCOMPL_STAT             CheckIrisLabGuiding(vltLOGICAL reqStatus);


        /*
         * methods representing the internal state machine
         */
        ccsCOMPL_STAT             CmdStateMachine(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInErrorState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInIdleState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInDisarmArmState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInSetDelDisarmState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInSetDelSetState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInPingDestinationsStateCou(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInPingDestinationsStateTel(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInChangeMswTableState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInReconfComponentsState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInSetDeldSetState(issarrayCONF_CMD_DESCR  *cmdDesc);
        ccsCOMPL_STAT             CmdProcessInInitState(issarrayCONF_CMD_DESCR  *cmdDesc);

    private:
        issarraySYNCH_MNGR         mSynchMngr;
        issarrayIRIS              &mIrisMngr;
        dbSYMADDRESS               mDbBasePoint;
        vltINT32                   mSubState;
        msgMESSAGE                 mMsg;
        msgMESSAGE                 mReplyMsg;
        msgMESSAGE                 mInfoMsg;
        evhCOMMAND                 mArrayControl;
        issarrayTABLE_MNGR        &mTableMngr;
        issarrayTUNNEL_TABLE_MNGR &mTunnelMngr;
        issarrayCONF_CMD_DESCR    *mCmdInProgress[issarrayMAX_CMD_IN_PROGRESS];
        issarrayMODULE_DEST        mTarget[issarrayMAX_NUM_PING];
        int                        mInitNextArm;
        vltLOGICAL                 mSetdeldForLastArm;
        char                       mLastPointModel[16];
#ifndef issarraySUPPORT_AT_M16
        vltINT32                   mM16VltiInputChMap[issNUM_INPUT_CHANNELS + 1];
#endif
        dbSYMADDRESS               mM16PositionAddr[issNUMBER_OF_DELAY_LINE_IDS];
        vltLOGICAL                 mRestoredArrayFromFile;
        vltLOGICAL                 mSendReplyAfterIrisConf;
        issarrayIRIS_ACTION        mNextIrisAction;
        vltLOGICAL                 mConfIris;
        vltLOGICAL                 mStartUpWarning;
    };


#endif /* !issarrayMAIN_TASK_H */

/* __oOo__ */

