#ifndef issarrayTABLE_MNGR_H
#define issarrayTABLE_MNGR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issarrayTABLE_MNGR.h,v 2.26 2005/03/21 13:03:13 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-01-12 Removed methods IsPrimaryBeamInput() and
*                     IsSecondaryBeamInput() in order to use common
*                     IsPrimaryInput() and IsSecondaryInput() methods
*
*                     SPR VLTSW20040399:
*                     Added optional parameter 'testConfiguration' to method
*                     CalculateOPL()
* jott     2004-04-06 Added methods UpdateConfigTable() and GetSetdeldParams(),
*                     added members mTelPositions[] and mConfTableArm[]
*                     to optimize database write access (SPR VLTSW20030515)
*                     added parameter 'vltLOGICAL useBackupFile' to method
*                     InitFromConfigTable()
* jott      15/10/02  changed interface of CalculateOPL() method
* jott      01/08/02  added method CalculatePupilPos() to solve SPR
*                     VLTSW20020429
* jott      30/07/02  added parameter to method EnforceBeamOrder() to solve
*                     SPR VLTSW20020290
* jott      07/05/02  added methods IsPrimaryBeamInput() and
*                     IsSecondaryBeamInput() to enforce proper assignment of
*                     input channels to instrumentation.
*                     added method EnforceBeamOrder() to solve SPR VLTSW20020290
* jott      21/02/02  added opdModel as parameter to BackupConfigTable() and
*                     RestoreConfigTable() to solve SPR VLTSW20020013
* jott      06/11/01  added method SetdelToBeSend() in course of SPR
*                     VLTSW20010628 (Sending SETDEL command if necessary)
* jott      19/07/01  added IsConfigured() to solve SPR VLTSW20010322
* jott      28/06/01  code reviewed and method CleanReservedResourced() added
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  initermediate version
* jott      20/02/01  first version
* jott      31/01/01  created
*/

/************************************************************************
 * Header file for class issarrayTABLE_MNGR
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * Include error messages
 */
#include "eccsTestTools.h"
#include "issarrayErrors.h"

/*
 * Include parent classes
 */
#include "evhDB_TASK.h"
#include "issarray.h"

/*
 * Name mappings
 */
extern fndNAME_MAP  telesopeNames;
extern fndNAME_MAP  stationNames;
extern fndNAME_MAP  delayLineNames;
extern fndNAME_MAP  switchyardConfig;
extern fndNAME_MAP  feedOpticsConfig;


class issarrayTABLE_MNGR : public evhDB_TASK
    {
    public:
        issarrayTABLE_MNGR(const dbSYMADDRESS  dbObjectPoint,
                           const dbSYMADDRESS  dbBasePoint);
        ~issarrayTABLE_MNGR(void);

        ccsCOMPL_STAT   CalculateOPL(int                      arm,
                                     issarraySETDELD_PARAM    *dlParams,
                                     issarrayCONFIG_COMPLETE  *testConfiguration = NULL);
        ccsCOMPL_STAT   ReadArrayConfigToString(issarrayCONFIG_LEVEL level,
                                                char *buffer, int bufferSize);
        ccsCOMPL_STAT   GetArmConfiguration(int arm,
                                            issarrayCONFIG_COMPLETE *configuration);
        ccsCOMPL_STAT   ReserveNewResources(issarrayCONFIG_COMPLETE *newConfiguration);
        ccsCOMPL_STAT   FreeOldResources(issarrayCONFIG_COMPLETE *newConfiguration);
        ccsCOMPL_STAT   UpdateArmInConfigTable(issarrayCONFIG_LEVEL level,
                                               issarrayCONFIG_COMPLETE *newConfiguration);
        ccsCOMPL_STAT   UpdateConfigTable(void);
        vltUINT32       dlUsed(void);
        ccsCOMPL_STAT   delayLineNamesInUse(char *buffer, int bufferSize);
        ccsCOMPL_STAT   InitFromConfigTable(vltLOGICAL useBackupFile);
        ccsCOMPL_STAT   BackupConfigTable(char *opdModel);
        ccsCOMPL_STAT   RestoreConfigTable(char *opdModel);
        vltLOGICAL      IsConfigured(int arm);
        vltLOGICAL      SetdelToBeSend(char *replyBuffer,
                                       char *sendSetdelBuffer);
        ccsCOMPL_STAT   GetSetdeldParams(vltINT32                beam,
                                         issarraySETDELD_PARAM  *dlParams);

    protected:
        ccsCOMPL_STAT   CleanReservedResources(void);
        ccsCOMPL_STAT   EnforceBeamOrder(issarrayCONFIG_COMPLETE  *oldConf);
        ccsCOMPL_STAT   CalculatePupilPos(issarrayPUPIL_INFO *pupilInfo,
                                          vltDOUBLE          *pupilIn,
                                          vltDOUBLE          *pupilOut);

    private:
        dbSYMADDRESS                mDbObjectPoint;
        dbSYMADDRESS                mDbBasePoint;
        issarrayCONFIG_COMPLETE     mConfTableArm[issarrayMAX_NUM_ARMS];
        issarrayTEL_POSITION        mTelPositions[issarrayMAX_NUM_ARMS];
    };


#endif /* !issarrayTABLE_MNGR_H */

/* ___oOo___ */
