#ifndef ISSIF_H
#define ISSIF_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issif.h,v 2.35 2005/06/07 14:02:25 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott      03/09/01  Moved definitions to issifDefines.h
* jott      02/08/01  renamed isstrkwsVLTI_CFG_FIELDS to issifVLTI_CFG_FIELDS
* jott      25/07/01  added definitions issifDATA_DLREF and issifDATA_DLREFOPL
* jott      14/05/01  Definition issifMAX_NUM_ARMS replaced by issMAX_NUM_ARMS
*                     Added definitions issifDATA_PRI_FSUP_FREQ,
*                     issifDATA_PRI_FSUS_FREQ, and issifDATA_PRI_MET_STATUS for
*                     PRIMA FITS header keywords
*                     Moved functions issifFitsStartCollBinData() and
*                     issifFitsStopCollBinData() ti issifLibs.h
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      20/02/01  second version
* jott      20/12/00  first version
* jott      09/11/00  created
*/

/************************************************************************
 * Public interface definition of the VLTI ISS Interface (issif)
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "ccs.h"
#include "evt.h"
#include "msg.h"

#include "eccsTestTools.h"
#include "iss.h"
#include "issifDefines.h"
#include "tif.h"

/*
 * Typedefs with the definition of storage type
 * for data and event identification names
 */
typedef vltBYTES32  issifNAME;
typedef vltBYTES256 issifDB_ADDR;

/*
 * Interferometer Position Data Structure (actual position)
 */
struct issifPOSITION
    {
    vltDOUBLE HA;    /* hour angle */
    vltDOUBLE RA;    /* right ascension due to mean place of J2000 */
    vltDOUBLE dec;   /* declination due to mean place of J2000 */
    vltDOUBLE alt;   /* altitude */
    vltDOUBLE az;    /* azimuth */
    };

/*
 * Module States Data Structure
 */
struct issifSTATES
    {
    vltINT32 vltiState;      /* VLTI ISS global state */
    vltINT32 vltiSubstate;   /* VLTI ISS sub-state    */
    };

/*
 *  Coordinates for VLTI telescopes
 */
struct issifTEL_COORD
    {
    vltDOUBLE   X;
    vltDOUBLE   Y;
    vltDOUBLE   Z;
    };

struct issifVLTI_CFG_FIELDS
    {
    vltINT32    telescopeNum;
    vltINT32    telescopeName;
    vltBYTES20  location;
    };


/*
 * Common VLTI ISS Library init function
 */
ccsCOMPL_STAT issifInitLibs(ccsERROR *error = NULL);

/*
 * VLTI ISS Data Query Library functions
 */
ccsCOMPL_STAT issifGetPosition(issifPOSITION *position, ccsERROR *error = NULL);
ccsCOMPL_STAT issifGetStates(issifSTATES *states, ccsERROR *error = NULL);
ccsCOMPL_STAT issifGetBaselines(issifTEL_COORD *tel1, issifTEL_COORD *tel2,
                                issifTEL_COORD *tel3, ccsERROR *error = NULL);
ccsCOMPL_STAT issifGetArray(issifVLTI_CFG_FIELDS vltiArms[issMAX_NUM_ARMS], ccsERROR *error = NULL);
ccsCOMPL_STAT issifGetByName(char *nameIndex, char *buffer, dbTYPE *type = NULL,
                             ccsERROR *error = NULL);

/*
 * VLTI ISS FITS Library functions
 */
ccsCOMPL_STAT issifGetFitsStart(const char *imageFile, ccsERROR *error = NULL);
ccsCOMPL_STAT issifGetFitsEnd(const char *imageFile, ccsERROR *error = NULL);

/*
 * VLIT ISS Event Library functions
 */
ccsCOMPL_STAT issifAttachEvent(char *nameIndex, evtEVENT_ID *eventId, ccsERROR *error = NULL);
ccsCOMPL_STAT issifDetachEvent(evtEVENT_ID *eventId, ccsERROR *error = NULL);


#endif /* !ISSIF_H */

/* __oOo__ */

