#ifndef issmsw_H
#define issmsw_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issmsw.h,v 2.17 2005/05/04 06:41:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2004-06-17 added issmswSubStateNameMap (SPR VLTSW20040134)
* jott     2004-01-12 added issmswTelIdNameMap and structure
*                     issmswTEL_FOCUS_RECORD to solve remarks from
*                     02-Dec-2003 and 08-Dec-2003 to SPR VLTSW20020475
* jott     2003-10-13 replaced issmswCONFIG_ISSPRS with issmswCONFIG_ISSALIGN
* jott      07/08/03  added enumeration value issmswTEL_TYPE
* jott      03/07/03  added enum issmswINS_CMD_HANDLER_ID and structures
*                     issmswGUIDING_SYS, issmswDELAY_SYS, issmswMODULE_TYPE,
*                     issmswMODULE_CONF, issmswINS_CONFIGURATION
*                     to solve SPR VLTSW20020475 (SETINS suit of commands)
* jott      27/08/02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "mswINS_DATA_RECORD.h"
#include "iss.h"
#include "issmswDefines.h"

/*
 * Types of modules in the mode switching table.
 * These types are introduced to provide a most flexible base for handling
 * the SETINS suite of commands, even if presently only one type is used!
 */
typedef enum
    {
    issmswSEL_ALL,          /* all modules beeing part of mode switching
                               table */
    issmswSEL_TELESCOPE,    /* physical telescopes (not necessarily complete
                               guiding systems) */
    issmswSEL_GUIDING_SYST, /* add-ons to UTs which upgrade them to complete
                               guiding systems */
    issmswSEL_TEL_GUIDE,    /* all physical telescopes and add-ons to UTs */
    issmswSEL_ISS           /* modules which are internally to ISS */
    } issmswMOD_SELECTION;

/*
 * Structures describing ISS instrument configuration data.
 */
typedef struct
    {
    mswINSID      insId;
    vltDOUBLE     pupOffs1U;
    vltDOUBLE     pupOffs1V;
    vltDOUBLE     pupOffs1W;
    vltDOUBLE     pupOffs2U;
    vltDOUBLE     pupOffs2V;
    vltDOUBLE     pupOffs2W;
    vltDOUBLE     pupOffs3U;
    vltDOUBLE     pupOffs3V;
    vltDOUBLE     pupOffs3W;
    vltDOUBLE     oplOffs1;
    vltDOUBLE     oplOffs2;
    vltDOUBLE     oplOffs3;
    vltDOUBLE     focOffs1;
    vltDOUBLE     focOffs2;
    vltDOUBLE     focOffs3;
    vltDOUBLE     pointOffs1V;
    vltDOUBLE     pointOffs1W;
    vltDOUBLE     pointOffs2V;
    vltDOUBLE     pointOffs2W;
    vltDOUBLE     pointOffs3V;
    vltDOUBLE     pointOffs3W;
    } issmswINS_DATA_RECORD;

typedef struct
    {
    mswINSID      insId;
    vltDOUBLE     pupOffsU[issMAX_NUM_ARMS];
    vltDOUBLE     pupOffsV[issMAX_NUM_ARMS];
    vltDOUBLE     pupOffsW[issMAX_NUM_ARMS];
    vltDOUBLE     oplOffs[issMAX_NUM_ARMS];
    vltDOUBLE     focOffs[issMAX_NUM_ARMS];
    vltDOUBLE     pointOffsV[issMAX_NUM_ARMS];
    vltDOUBLE     pointOffsW[issMAX_NUM_ARMS];
    } issmswINS_CONFIGURATION;


/*
 * Index for instrument configuration command handlers
 */
typedef enum
    {
    issmswCONFIG_SID1  = 0,
    issmswCONFIG_SID2,                  /* 1  */
    issmswCONFIG_UT1,                   /* 2  */
    issmswCONFIG_UT2,                   /* 3  */
    issmswCONFIG_UT3,                   /* 4  */
    issmswCONFIG_UT4,                   /* 5  */
    issmswCONFIG_AT1,                   /* 6  */
    issmswCONFIG_AT2,                   /* 7  */
    issmswCONFIG_AT3,                   /* 8  */
    issmswCONFIG_DLCS,                  /* 9  */
    issmswCONFIG_COU1,                  /* 10 */
    issmswCONFIG_COU2,                  /* 12 */
    issmswCONFIG_COU3,                  /* 13 */
    issmswCONFIG_COU4,                  /* 14 */
    issmswCONFIG_ISSALIGN               /* 15 */
    } issmswINS_CMD_HANDLER_ID;

/*
 * Structures containing mapping information between issmsw modules and array
 * configuration (intentionally done this way to be prepared for future
 * extensions)
 */
typedef struct
    {
    int           beam;
    } issmswGUIDING_SYS;

typedef struct
    {
    int           dlId[issMAX_NUM_ARMS];
    } issmswDELAY_SYS;

typedef union
    {
    issmswGUIDING_SYS  guidSys;
    issmswDELAY_SYS    dlcs;
    vltLOGICAL         issComponent;
    } issmswMODULE_TYPE;

typedef struct
    {
    issmswMODULE_TYPE   type;
    vltLOGICAL          active;
    } issmswMODULE_CONF;

/*
 * Enumeration values for telescope types
 */
typedef enum
    {
    issmswTEL_TYPE_UNDEF   = 0,
    issmswTEL_TYPE_SID,
    issmswTEL_TYPE_UT,
    issmswTEL_TYPE_AT
    } issmswTEL_TYPE;

/*
 * Telescope nominal adapter focus record
 */
typedef struct
    {
    vltBYTES32   telescope;
    vltDOUBLE    nomFocus;
    vltDOUBLE    nomFocusAO;
    } issmswTEL_FOCUS_RECORD;

/*
 * Mapping of issmsw specific names and indices
 */
extern fndNAME_MAP issmswTelIdNameMap;
extern fndNAME_MAP issmswSubStateNameMap;

#endif /*!issmsw_H*/
