#ifndef issmswCONTROL_MAIN_H
#define issmswCONTROL_MAIN_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issmswCONTROL_MAIN.h,v 2.17 2005/05/04 06:41:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2004-06-17 Added CalibrationCB() (SPR VLTSW20040134)
* jott      24/07/03  Added method StoreTelescopeConfData() and members
*                     mTelConfTable, mDbPoint, mWaitForConfigEnd to solve
*                     SPR VLTSW20020475
* jott      27/03/03  Removed not used member mFirstReply,
*                     added members mSetInsData, mInsConfCompleted, mDummyMsg,
*                     mInsConfMngr, mNoTelescopeCmd, mSelInsConf and
*                     mSetInsName,
*                     added methods ProcessSelInsDReply(), VerboseCB(),  and
*                     ConfigFinishedEventCB()
*                     to solve SPR VLTSW20020475 (SETINS suit of commands)
* rrollf    07/03/03  Added callback for CLRREQ command (see SPR VLTSW20020539)
* jott      27/08/02  added mCurrSelInstrument to modify handling of GETINS,
*                     GETINSD, SELINS, and SETINSD commands (SPR VLTSW20020475)
* jott      21/11/01  added handling of commands GETINS, GETINSD, SELINS, and
*                     SETINSD
* jott      10/07/01  code reviewed
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      20/01/01  second version
* jott      20/12/00  first version
* jott      04/12/00  created
*/

/************************************************************************
 * Header file for class issmswCONTROL_MAIN
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * Include local headers
 */
#include "eccsTestTools.h"
#include "evhHANDLER.h"
#include "mswCONTROL_MAIN.h"
#include "issmsw.h"
#include "issmswMOD_LIST_MANAGER.h"
#include "issmswINS_CONF_MGR.h"
#include "mswINS_CONFIG_TABLE.h"


/*
 * Class declaration
 */
class issmswCONTROL_MAIN: public mswCONTROL_MAIN
    {
    public:
        issmswCONTROL_MAIN(const dbSYMADDRESS      dbPoint,
                           issmswMOD_LIST_MANAGER &listManager,
		                   mswINS_CONFIG_TABLE    &insTable,
                           mswEVAL_STATE          &eval,
                           mswCHANGE_FOCUS        &changeFoc,
                           issmswINS_CONF_MGR     &insConfMngr);
        ~issmswCONTROL_MAIN(void);

        /*
         * Command callbacks
         */
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT SetRequiredFieldCB(msgMESSAGE &msg,
                                                    void       *udata);
        virtual evhCB_COMPL_STAT ClrReqCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT GetInsCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT GetInsDataCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT SetInsDataCB(msgMESSAGE &msg, void *udata);

        /* Callback handling SELINS command */
        virtual evhCB_COMPL_STAT TcsChangeFocusCB(msgMESSAGE &msg, void *udata);

        /* Callback handling calibration commands */
        virtual evhCB_COMPL_STAT CalibrationCB(msgMESSAGE &msg, void *udata);

        /* Callbacks overloading those of parent classes */
        virtual evhCB_COMPL_STAT TcsPresetCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT TcsAlignCB(msgMESSAGE &msg, void *udata);

        /*
         * Callbacks for database events
         */
        evhCB_COMPL_STAT    ConfigFinishedEventCB(evtEVENT_MSG &msg,
                                                  void *udata);
        /*
         * Callbacks replies to commands
         */
        evhCB_COMPL_STAT ReplyInsCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT ErrReplyInsCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT TimeoutInsCB(msgMESSAGE &msg, void *udata);



    protected:
        virtual evhCB_COMPL_STAT MswTableChangedCB(msgMESSAGE &msg, void *udata);
        virtual ccsCOMPL_STAT    ResetInsCmdObjects(void);
        virtual ccsCOMPL_STAT    SendInsCommand(const char          *cmd,
                                                const char          *buffer,
                                                issmswMOD_SELECTION  selection);
        virtual ccsCOMPL_STAT    ProcessGetInsReply(msgMESSAGE &msg);
        virtual ccsCOMPL_STAT    ProcessSelInsReply(msgMESSAGE &msg);
        virtual ccsCOMPL_STAT    ProcessSetInsDReply(msgMESSAGE &msg);
        virtual ccsCOMPL_STAT    StoreTelescopeConfData(char *destination,
                                                        char *msgBuffer);

    private:
        dbSYMADDRESS             mDbPoint;
        msgMESSAGE               mMsg;
        msgMESSAGE               mDummyMsg;
        issmswMOD_LIST_MANAGER  &mModListMngr;
        issmswINS_CONF_MGR      &mInsConfMngr;
        mswINS_CONFIG_TABLE     &mTelConfTable;
        char                     mReferenceBuffer[issmswREF_BUFFER_SIZE];
        evhCOMMAND               mSender[issmswMAX_NUM_TEL_SYSTEMS];
        msgMESSAGE               mSendMsg[issmswMAX_NUM_TEL_SYSTEMS];
        int                      mCmdId[issmswMAX_NUM_TEL_SYSTEMS];
        int                      mNoOfRepliesToWait;
        int                      mCmdProcState;
        vltLOGICAL               mReplyCheckFlag;
        evtEVENT                 mEvt;
        evtEVENT                 mInsConfCompleted;
        vltBYTES32               mCurrSelInstrument;
        cmdPARAM_LIST            mParamList;
        issmswINS_DATA_RECORD    mSetInsData;
        issmswINS_CONFIGURATION  mSelInsConf;
        vltBYTES32               mSetInsName;
        vltLOGICAL               mNoTelescopeCmd;
        vltLOGICAL               mWaitForConfigEnd;
    };

#endif /* !issmswCONTROL_MAIN_H */
