#ifndef issmswINS_CONF_MGR_H
#define issmswINS_CONF_MGR_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issmswINS_CONF_MGR.h,v 2.17 2005/05/04 06:41:10 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* jott      2005-04-28  SPR VLTSW20050163:
*                       added method OpsLogInstrumentData()
*                       added member mDbVltiPoint
* jott      2004-01-12  added method ReadNominalFocus() to solve remarks
*                       from 02-Dec-2003 and 08-Dec-2003 to SPR VLTSW20020475
* jott      2003-08-06  added mTelConfData, mDbBase, mSkipSelins and modified
*                       constructor
*                       added GetConfigCmdForIssprs()
* jott      2003-07-02  created to solve SPR VLTSW20020475 (SETINS suit of
*                       commands
*/

/************************************************************************
 * Header file of the instrument configuration manager of the issmsw module
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * Include local headers
 */
#include "db.h"
#include "evhTASK.h"
#include "evhDB_CMD_SERIAL.h"
#include "tcsDB_SYNC_ERR_CHECK.h"
#include "mswCONTROL_MAIN.h"
#include "mswINS_CONFIG_DATA.h"
#include "issConfigTable.h"
#include "issmswDefines.h"
#include "issmsw.h"

/*
 * Class declaration
 */
class issmswINS_CONF_MGR : public evhTASK
    {
    public:
        issmswINS_CONF_MGR(const dbSYMADDRESS  dbVltiPoint,
                           const dbSYMADDRESS  dbPoint,
                           const dbSYMADDRESS  dbBase,
                           mswINS_CONFIG_DATA &telescopeData);
        ~issmswINS_CONF_MGR(void);

        evhCB_COMPL_STAT    ArrayTableEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    InsConfigOkCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    InsConfigErrorCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    InsConfigTimeoutCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    InsConfigTimeoutModCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT       Initialize(void);
        ccsCOMPL_STAT       ConfigureInstrument(issmswINS_CONFIGURATION *insData,
                                                vltLOGICAL skipSelins = ccsFALSE);
        ccsCOMPL_STAT       MapInsConfDb2Int(issmswINS_DATA_RECORD      *dbConf,
                                             issmswINS_CONFIGURATION    *intConf);
        eccsERROR&          GetError(void);

        ccsCOMPL_STAT       OpsLogInstrumentData(issmswINS_CONFIGURATION *selInsConf);


    private:
        ccsCOMPL_STAT   SetCompletionStatus(vltINT32 result);
        ccsCOMPL_STAT   SendToModule(evhDB_CMD_SEND *sender, msgCMD command,
                                     char          *msgBuffer,
                                     vltLOGICAL    *modActive);
        ccsCOMPL_STAT   UpdateModuleConfiguration(void);
        ccsCOMPL_STAT   GetConfigCmdForModule(issmswINS_CONFIGURATION *insData,
                                              issmswINS_CMD_HANDLER_ID modId,
                                              msgCMD                   command,
                                              char                    *msgBuffer,
                                              int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForSids(issmswINS_CONFIGURATION *insData,
                                            issmswINS_CMD_HANDLER_ID modId,
                                            msgCMD                   command,
                                            char                    *msgBuffer,
                                            int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForUts(issmswINS_CONFIGURATION *insData,
                                           issmswINS_CMD_HANDLER_ID modId,
                                           msgCMD                   command,
                                           char                    *msgBuffer,
                                           int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForAts(issmswINS_CONFIGURATION *insData,
                                           issmswINS_CMD_HANDLER_ID modId,
                                           msgCMD                   command,
                                           char                    *msgBuffer,
                                           int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForDlcs(issmswINS_CONFIGURATION *insData,
                                            issmswINS_CMD_HANDLER_ID modId,
                                            msgCMD                   command,
                                            char                    *msgBuffer,
                                            int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForCous(issmswINS_CONFIGURATION *insData,
                                            issmswINS_CMD_HANDLER_ID modId,
                                            msgCMD                   command,
                                            char                    *msgBuffer,
                                            int                     *numCmds);
        ccsCOMPL_STAT   GetConfigCmdForIssprs(issmswINS_CONFIGURATION  *insData,
                                              issmswINS_CMD_HANDLER_ID  modId,
                                              msgCMD                    command,
                                              char                     *msgBuffer,
                                              int                      *numCmds);

        ccsCOMPL_STAT   ReadNominalFocus(issmswINS_CMD_HANDLER_ID   modId,
                                         vltLOGICAL                 adaptiveOptics,
                                         vltDOUBLE                 *nomPos);
        ccsCOMPL_STAT   ResetCmdHandlers(void);
        ccsCOMPL_STAT   SetError(eccsERROR *err, ccsMODULEID moduleId,
                                 vltINT16 errorNumber, ccsLOC_ID locId,
                                 char *string);
        ccsCOMPL_STAT   ResetError(void);


        /*
         * Database point of issmsw instrument configuration manager and
         * issmswControl process
         */
        dbSYMADDRESS            mDbVltiPoint;
        dbSYMADDRESS            mDbPoint;
        dbSYMADDRESS            mDbBase;

        /*
         *  Read access to telescope specific instrument configuration data
         */
        mswINS_CONFIG_DATA     &mTelConfData;

        /*
         * Command handling objects for all configurable sub-systems
         */
        evhDB_CMD_SERIAL       *mConfSender[issmswNUM_MOD4INS];
        tcsDB_SYNC_ERR_CHECK    mWaitConfigReady;

        /*
         * Cache change of array configuration table
         */
        vltLOGICAL              mArrayConfChanged;
        evtEVENT                mArrayTableEvent;

        /*
         * Counter for total number of sent configuration messages
         */
        vltINT32                mConfMsgCounter;

        /*
         * Mapping information between issmsw modules and array
         * configuration
         */
        issmswMODULE_CONF       mModConfiguration[issmswNUM_MOD4INS];

        /*
         * Cache VLTI array configuration table
         */
        issVLTI_CFG_FIELDS2     mVltiConf[issMAX_NUM_ARMS];

        /*
         * Private member saving errors which shall be displayed later
         */
        eccsERROR               mErrorStack;
        int                     mErrorCnt;

        /*
         * In case of SETINSD to currently selected instrument no SELINS
         * configuration command is required
         */
        vltLOGICAL              mSkipSelins;

    };

#include "issmswINS_CONF_MGR.icc"

#endif /*! issmswINS_CONF_MGR_H*/
