#ifndef issmswMOD_LIST_MANAGER_H
#define issmswMOD_LIST_MANAGER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issmswMOD_LIST_MANAGER.h,v 2.17 2005/05/04 06:41:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott      07/08/03  Added method GetTelType() and members mTelType to solve
*                     SPR VVLTSW20020475
* jott      30/06/03  Added methods GetFirstFreeRecNo() and GetInsRecNo()
*                     to solve SPR VVLTSW20020475 (SETINS suite of commands)
* rrollf    07/03/03  Added ClearRequiredFlag (see SPR VLTSW20020539)
* jott      27/08/02  changed handling of GETINS, GETINSD, SELINS,
*                     and SETINSD commands (SPR VVLTSW20020475)
* jott      21/11/01  added support for handling of commands GETINS, GETINSD,
*                     SELINS, and SETINSD
* jott      12/07/01  code reviewed
*                     added methods BackupMswTable() and RestoreMswTable()
*                     to solve problem of safer VLTI configuration
*                     (SPR  VLTSW20010322)
* jott      30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      20/02/01  second version
* jott      20/12/00  first version
* jott      07/12/00  created
*/

/************************************************************************
 * Header file for the class issmswMOD_LIST_MANAGER
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * Include local headers
 */
#include "db.h"
#include "eccsTestTools.h"
#include "evhHANDLER.h"
#include "mswMOD_LIST_MANAGER.h"
#include "mswINS_DATA_RECORD.h"
#include "issmswErrors.h"
#include "issmsw.h"


typedef struct
    {
    vltLOGICAL  required;
    vltLOGICAL  ignore;
    } issmswMswTableExcerpt;

typedef struct
    {
    vltBYTES32  modName;
    vltBYTES32  modId;
    vltBYTES256 modSymbAddr;
    vltINT8     startPhase;
    vltINT8     stopPhase;
    vltINT8     tracking;
    vltINT8     focus;
    vltLOGICAL  required;
    vltLOGICAL  ignore;
    vltBYTES256 userField;
    } issmswMswTableRecord;

/*
 * Class declaration
 */
class issmswMOD_LIST_MANAGER : public mswMOD_LIST_MANAGER
    {
    public:
        issmswMOD_LIST_MANAGER(const dbSYMADDRESS  dbPoint,
                               const dbSYMADDRESS  dbRoot);
        ~issmswMOD_LIST_MANAGER();

        ccsCOMPL_STAT      InitMgr(void);
        ccsCOMPL_STAT      SetRequiredFlag(const char *command,
                                           const char *buffer,
                                           int bufLen);
        ccsCOMPL_STAT      ClearRequiredFlag(void);
        ccsCOMPL_STAT      BackupMswTable(void);
        ccsCOMPL_STAT      RestoreMswTable(void);
        ccsCOMPL_STAT      SelectReceiverSystems(issmswMOD_SELECTION selection);
        int                GetNoOfReceivSystems(void);
        char              *GetDestenv(int i);
        char              *GetDestproc(int i);
        issmswTEL_TYPE     GetTelType(int i);
        ccsCOMPL_STAT      CheckFocusUT(void);
        ccsCOMPL_STAT      GetInsId(mswINSID insId);
        ccsCOMPL_STAT      SetInsId(mswINSID insId);
        ccsCOMPL_STAT      GetInsData(mswINSID                  insId,
                                      issmswINS_DATA_RECORD    *data);
        ccsCOMPL_STAT      SetInsData(issmswINS_DATA_RECORD    *data);

        protected:
        virtual evhCB_COMPL_STAT ListChangedCB(evtEVENT_MSG &msg, void *udata);
        virtual ccsCOMPL_STAT    GetInsRecNo(mswINSID  insId, int *record);
        virtual ccsCOMPL_STAT    GetFirstFreeRecNo(int *record);

    private:
        ccsCOMPL_STAT      UpdateSystemList(void);
        ccsCOMPL_STAT      AddToReceiverLists(int mswIndex);
        dbSYMADDRESS       mModListPoint;
        int                mConfSystems;
        ccsENVNAME         mEnvs[issmswMAX_NUM_TEL_SYSTEMS];
        ccsPROCNAME        mProcs[issmswMAX_NUM_TEL_SYSTEMS];
        vltLOGICAL         mConfUT[issmswNUM_UT];
        issmswTEL_TYPE     mTelType[issmswMAX_NUM_TEL_SYSTEMS];
        vltLOGICAL         mMswTableChangedFlag;
        dbSYMADDRESS       mDbPoint;
    };


#include "issmswMOD_LIST_MANAGER.icc"


#endif /* !issmswMOD_LIST_MANAGER_H */

/* __oOo__ */
