#ifndef ISSPRS_H
#define ISSPRS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issprs.h,v 2.56 2005/06/08 13:29:41 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-05-13 Added member issprsOFFSADG_IGNORED to enum
*                     issprsOFFSADG_STATES to fix bug
*                     SPR VLTSW20040255:
*                     - added member nasGuidUtBeam to struct issprsOFFSADG_DATA,
*                     - added member issprsBR_CMD_OFFSGP to enum issprsBR_CMD
*                     - added type issprsOFFSGP_DATA
*                     - added enumeration issprsOFFSGP_STATES
* jott     2005-01-19 Added external variable issprsAtDbPosNameMap
*                     (SPR VLTSW20050012)
* jott     2004-03-11 Added issprsSbsNasGuideStateNameMap to accept special
*                     case of Nasmyth guiding for OFFSGUV commands to UTs
*                     (SPR VLTSW2004055)
* jott     2003-10-22 Added enums issprsBR_CMD_STARTAO and issprsBR_CMD_STOPAO
* jott     2003-09-23 Added enums issprsGOBACK_WAIT_SBS_TRACKING and
*                     issprsGOBACK_WAIT_SBS_TRACK_EVT to ensure
*                     waiting until all sub-systems are tracking in the
*                     course of a GOBACK command (aligned to UC 10.11 of ISS DD)
* rrollf    22/07/03  Removed check of Nasmyth Guiding and Active optics
*                     corrections made (see SPR VLTSW20030237)
* rrollf    02/07/03  Added issprsGuidingModeNameMap
* rrollf    20/06/03  Added issprsOFFSADG_ABORT to enum issprsOFFSADG_STATES
* jott      03/06/03  Added commands FNDBEAM, OFFSFTS, SETFNTM, and ZEROOPD
*                     (SPRs VLTSW20030208 and VLTSW20030214)
* rrollf    19/05/03  Added OPTFLUX command (see SPR VLTSW20030203)
* rrollf    30/10/02  Added issprsSBS_DATA struct and changed type of teleSys
*                     and guideSys in issprsOFFSADG_DATA to that struct
* rrollf    29/10/02  Removed guideSys from issprsOFFSGUV_DATA, not used
* rrollf    11/10/02  Serialized OFFSGUV (see SPR VLTSW20020551):
*                     Expanded beam specific data for  OFFSGUV,
*                     Added issprsMAX_OFFS_STEPS for maximum allowed offsets
* rrollf    18/09/02  Added issprsOFFSADG_DATA and issprsOFFSGUV_DATA
* rrollf    10/09/02  Added issprsUtDbPosNameMap
* rrollf    03/09/02  Added types for PRSNAME "AUTOCOLL" | "FREE"
* rrollf    06/05/02  Added issprsFtSensorNameMap
* rrollf    03/05/02  Added types for issprsAction
* rrollf    08/04/02  Include issConfigTable.h instead of issarray.h
* rrollf    13/02/02  Now using the complete config table from issarray
* rrollf    11/02/02  Renamed issprsGOBACK_WAIT_RECOGUV to
*                     issprsGOBACK_ACCEPT_RECOGUV
* jott      16/01/02  to solve SPR VLTSW20010787 and SPR VLTSW20020019:
*                     Changed enum issprsGOBACK_WAIT_STRCAG_REPLY to
*                     issprsGOBACK_WAIT_FOR_RECOGUV
*                     removed enum issprsGOBACK_WAIT_OFFSGUV_REPLY
*                     added enum issprsGOBACK_WAIT_RECOGUV
* jott      15/01/02  Added member ndFilter to struct issprsRMB_TELTAB to
*                     solve SPR VLTSW20010783
* jott      06/12/01  Added processing state enum
*                     issprsGOBACK_WAIT_STOPCAG_REPLY to send a STOPCAG
*                     command before sending a SETUP command
*                     (SPR VLTSW20010746)
* rrollf    25/10/01  Added issprsBR_CMD_OPTDLCS to issprsBR_CMD
* jott      11/10/01  added enum issprsGOBACK_STEPS
* rrollf    09/10/01  added enums issprsBR_CMD_OFFSADG_SUSPEND and
*                     issprsBR_CMD_OFFSADG_RESUME
* jott      21/09/01  modified structures issprsREMEMB, issprdRMB_TELTAB and
*                     added structure issprsRMB_DLTAB
* rrollf    18/09/01  added issprsStationNameMap
* rrollf    10/09/01  added issprsBR_CMD_OPTCGS to issprsBR_CMD
* jott      07/09/01  modified structure issprsRMB_TELTAB due to
*                     UVW offsets.
* rrollf    30/08/01  new fields of configTable (issarray) used
*                     (inputChannel and swYardConfig)
* rrollf    05/07/01  added issprsDelayLineNameMap used by PRSOPL
* rrollf    11/06/01  expanded the used configTabel to the physDelayLine field
* rrollf    30/04/01  third version (Version 1 of ISS)
* rrollf    28/03/01  second version
* jott      19/02/01  first version
* jott      12/01/01  created
*/

/************************************************************************
 *
 *   General include file for the "issprs" module.
 *   It contains common definitions used in every component of the module
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "eccsTestTools.h"

/*
 *  Defines for the target telescope VLT
 */

#include "tcs.h"
#include "fndNAME_MAP.h"
#include "oslx.h"
#include "mswMOD_LIST.h"
#include "iss.h"
#include "issConfigTable.h"
#include "issmswDefines.h"
#include "issprsErrors.h"
#include "issprsDefines.h"

/*
 *  Value/Name conversion tables:
 *  The defines for both values and names are in prsDefines.h
 *  Other applications sending or receiving these values/string
 *  can include this file and link the file prs.C with the code
 *  for the mapping functions.
 *  We have to think if there is a better place where to put these
 *  common things.
 */

extern fndNAME_MAP issprsStateNameMap;
extern fndNAME_MAP issprsTargetNameMap;
extern fndNAME_MAP issprsTargetDelayNameMap;
extern fndNAME_MAP issprsTelIdNameMap;
extern fndNAME_MAP issprsTelGuideIdNameMap;
extern fndNAME_MAP issprsSbsGuideStateNameMap;
extern fndNAME_MAP issprsSbsNasGuideStateNameMap;
extern fndNAME_MAP issprsDbTelescopeNameMap;
extern fndNAME_MAP issprsDelayLineNameMap;
extern fndNAME_MAP issprsMemorySlotNameMap;
extern fndNAME_MAP issprsStationNameMap;
extern fndNAME_MAP issprsGobackStepNameMap;
extern fndNAME_MAP issprsTelTypeNameMap;
extern fndNAME_MAP issprsFtSensorNameMap;
extern fndNAME_MAP issprsUtDbPosNameMap;
extern fndNAME_MAP issprsAtDbPosNameMap;
extern fndNAME_MAP issprsGbKwdNameMap;
extern fndNAME_MAP issprsGuidingModeNameMap;
extern vltUINT8    issprsCfgToMsw[issNUM_PHYSICAL_TEL][3];

/* Use the complete config table from issarray (see issarray.h) */
typedef issVLTI_CFG_FIELDS issprsVLTI_CFG_FIELDS;

/* Number of used fields of issarray configTable */
#define issprsFIELDS_CFG_TABLE  issFIELDS_CFG_TABLE

/*
 * VLTI config table
 */
typedef struct {
    issprsVLTI_CFG_FIELDS  arm[issMAX_NUM_ARMS];
} issprsVLTI_CFG_TABLE;

/*
 * mode switching table from issmsw
 */
typedef mswMOD_DB_RECORD    issprsMSW_MOD_TABLE[issmswNO_OF_MODULES];

/*
 * Table of FITS binary producer
 */
typedef struct {
    vltBYTES32  modName;        /* identifier for module */
    vltBYTES32  modId;          /* destination process   */
} issprsFITS_BIN_PROD;

typedef issprsFITS_BIN_PROD issprsFBP_TABLE[issmswNO_OF_MODULES];

/*
 * Enum to specify a broadcast command
 * Used in issprsBR_MAIN_TASK::SetTargetList()
 */
typedef enum
    {
    /* NO_COMMAND                                               */
    issprsBR_CMD_NONE,

    /* DETCGS:  Detect best object and take it as guide star    */
    issprsBR_CMD_DETCGS,

    /* OPTCGS:  Optimize Coud Guiding Loop                     */
    issprsBR_CMD_OPTCGS,

    /* OFFSADG: SUSPEND to all UT's                             */
    issprsBR_CMD_OFFSADG_SUSPEND,

    /* OFFSADG: Apply an ADG offset on all telescopes           */
    issprsBR_CMD_OFFSADG,

    /* OFFSADG: RESUME to all UT's                              */
    issprsBR_CMD_OFFSADG_RESUME,

    /* OFFSOPD: Apply an OPD offset to delay line               */
    issprsBR_CMD_OFFSOPD,

    /* OFFSFNT: Apply an OPD offset to FINITO beam              */
    issprsBR_CMD_OFFSFNT,

    /* OFFSGUV: Apply an UVW offset for a single telescope      */
    issprsBR_CMD_OFFSGUV,

    /* PRSNAME: Preset all telescopes to named position         */
    issprsBR_CMD_PRSNAME,

    /* PRSNAME: Preset all telescopes to AUTOCOLL position      */
    issprsBR_CMD_PRSNAME_AUTOCOLL,

    /* PRSNAME: Preset all telescopes to FREE position          */
    issprsBR_CMD_PRSNAME_FREE,

    /* PRSOPL:  Preset delay line to given OPL position         */
    issprsBR_CMD_PRSOPL,

    /* STRTCAG: Start Coude auto guiding                        */
    issprsBR_CMD_STRTCAG,

    /* STOPCAG: Stop Coude auto guiding                         */
    issprsBR_CMD_STOPCAG,

    /* STRTCHP: Start synchronized chopping                     */
    issprsBR_CMD_STRTCHP,

    /* STOPCHP: Stop synchronized chopping                      */
    issprsBR_CMD_STOPCHP,

    /* STRTFTK: Start fringe tracking                           */
    issprsBR_CMD_STRTFTK,

    /* STOPFTK: Stop fringe tracking                            */
    issprsBR_CMD_STOPFTK,

    /* STARTEX: Start data collection in FITS binary tables     */
    issprsBR_CMD_STARTEX,

    /* ENDEX:   Stop data collection in FITS binary tables      */
    issprsBR_CMD_ENDEX,

    /* CLRSTP:  Clear ready SETUP file resetting default values */
    issprsBR_CMD_CLRSTP,

    /* OPTDLCS:  Optimize DLCS                                  */
    issprsBR_CMD_OPTDLCS,

    /* OPTFLUX:  Optimize FLUX                                  */
    issprsBR_CMD_OPTFLUX,

    /* FNDBEAM:  Find beam                                      */
    issprsBR_CMD_FNDBEAM,

    /* OFFSFTS:  Offset FINITO translation stages               */
    issprsBR_CMD_OFFSFTS,

    /* SETFNTM:  Set FINITO operational mode                    */
    issprsBR_CMD_SETFNTM,

    /* ZEROOPD:  Store current positions as Zero OPD positions  */
    issprsBR_CMD_ZEROOPD,

    /* STARTAO: Start active optics on UTs and ATs              */
    issprsBR_CMD_STARTAO,

    /* STOPAO: Stop active optics on UTs and ATs                */
    issprsBR_CMD_STOPAO,

    /* OFFSGP: Offset guiding probe in ra and dec               */
    issprsBR_CMD_OFFSGP,

    } issprsBR_CMD;

/*
 * structs for issprsReacquire
 */
typedef struct {
    vltDOUBLE   ra2000;
    vltDOUBLE   dec2000;
    vltDOUBLE   nasGsRa2000;
    vltDOUBLE   nasGsDec2000;
    vltDOUBLE   coGsRa2000;
    vltDOUBLE   coGsDec2000;
    vltDOUBLE   offsU;
    vltDOUBLE   offsV;
    vltDOUBLE   offsW;
    vltDOUBLE   mag;
    vltINT32    ndFilter;
    vltINT32    guidingMode;
//    vltINT32    reserved;
} issprsRMB_TELTAB;

typedef struct {
    vltDOUBLE           ra2000;
    vltDOUBLE           dec2000;
    /* currently only data four 3 delay lines are provided in public database */
    vltDOUBLE           oplOffs[issNUM_CURR_SUPP_DL];
    vltBYTES20          refDl;
    vltDOUBLE           refOpl;
    vltBYTES20          refMode;
    } issprsDLCS_READ_DATA;

typedef struct {
    vltLOGICAL              dataOk;
    vltABS_TIME             timeStamp;
    issprsRMB_TELTAB        tele[issMAX_NUM_ARMS];
    issprsDLCS_READ_DATA    delay;
} issprsREMEMB;


typedef enum {
    issprsGOBACK_IDLE,
    issprsGOBACK_WAIT_PRESET_REPLY,
    issprsGOBACK_WAIT_PRSNAME_REPLY,
    issprsGOBACK_WAIT_STOPCAG_REPLY,
    issprsGOBACK_WAIT_SETUP_REPLY,
    issprsGOBACK_WAIT_OFFSOPD_REPLY,
    issprsGOBACK_WAIT_SBS_TRACKING,
    issprsGOBACK_WAIT_SBS_TRACK_EVT,
    issprsGOBACK_SEND_EPRESET_REPLY,
    issprsGOBACK_WAIT_EPRESET_REPLY,
    issprsGOBACK_ACCEPT_RECOGUV
} issprsGOBACK_STEPS;

/*
 * Enumeration values for GOBACK SETUP keywords
 * (used in issprsGbKwdNameMap -> issprs.C)
 */
typedef enum {
    issprsGB_KWD_UNKNOWN,
    issprsGB_KWD_TEL_TARG_TYPE,
    issprsGB_KWD_TEL_TARG_ALPHA,
    issprsGB_KWD_TEL_TARG_DELTA,
    issprsGB_KWD_TEL_AG_GUIDESTAR,
    issprsGB_KWD_TEL_GS1_ALPHA,
    issprsGB_KWD_TEL_GS1_DELTA,
    issprsGB_KWD_TEL_AG_TYPE,
    issprsGB_KWD_TEL_AG_OBJCNT,
    issprsGB_KWD_TEL_AG_START,
    issprsGB_KWD_TEL_AO_START,
    issprsGB_KWD_COU_AG_GSSOURCE,
    issprsGB_KWD_COU_AG_ALPHA,
    issprsGB_KWD_COU_AG_DELTA,
    issprsGB_KWD_COU_STP_NDFIDX,
    issprsGB_KWD_COU_GUID_RA,
    issprsGB_KWD_COU_GUID_DEC
} issprsGB_KEYWORDS;

/*
 * Types for issprsBroadcast
 */
typedef vltDOUBLE   issprsTRANSF_MAT[3][3];

typedef struct {
    vltDOUBLE   offsU;
    vltDOUBLE   offsV;
    vltDOUBLE   offsW;
} issprsUVW_OFFS;

typedef issprsUVW_OFFS  issprsUVW_OFFS_TAB[issNUM_PHYSICAL_TEL];

/* UVW Coordinates of a telescope (location) */
typedef struct {
    vltDOUBLE   u;
    vltDOUBLE   v;
    vltDOUBLE   w;
} issprsUVW;


/*
 * Types for issprsAction
 */
typedef struct {
    vltBYTES4   telescopeType;
    vltBYTES20  guideMode;
} issprsGUIDE_TYPE_RECORD;

typedef issprsGUIDE_TYPE_RECORD issprsGUIDE_TYPE_TABLE[issprsNUM_TEL_TYPES];


/*
 * Types for PRSNAME "AUTOCOLL" | "FREE"
 */

/* Telescope description for autocollimation */
typedef struct
    {
    vltINT32    telId;
    vltBYTES32  modId;
    } issprsBR_AC_TEL;

/* List of telescopes for autocollimation */
typedef issprsBR_AC_TEL issprsBR_AC_TEL_LST[issMAX_NUM_ARMS];

/* One row of the nasmythDevice table */
typedef struct
    {
    vltBYTES8  telescope;
    vltDOUBLE  freePos;
    vltDOUBLE  mirrorPos;
    } issprsNAS_DEV_TABLE_REC;

/* Complete nasmythDevice table */
typedef issprsNAS_DEV_TABLE_REC issprsNAS_DEV_TABLE[issMAX_NUM_NASMYTH_DEVICE];

/* States of the OFFSADG command sequence */
typedef enum
    {
    issprsOFFSADG_IDLE = 0,
    issprsOFFSADG_SUSPEND,
    issprsOFFSADG_OFFSET,
    issprsOFFSADG_RESUME,
    issprsOFFSADG_ABORT,
    issprsOFFSADG_IGNORED
    } issprsOFFSADG_STATES;

/* States of the OFFSGP command sequence */
typedef enum
    {
    issprsOFFSGP_IDLE = 0,
    issprsOFFSGP_OFFSGP,
    issprsOFFSGP_ABORT,
    issprsOFFSGP_IGNORED
    } issprsOFFSGP_STATES;

/* Data about a subsystem */
typedef struct
    {
    vltBYTES32              dest;       /* Destination process  */
    vltLOGICAL              ignored;    /* Ignore flag          */
    vltLOGICAL              replyOk;    /* Reply Ok flag        */
    } issprsSBS_DATA;

/* Data for OFFSADG command */
typedef struct
    {
    issprsOFFSADG_STATES    state;
    vltLOGICAL              foundUT;
    vltLOGICAL              singleBeam;
    vltDOUBLE               raOffset;
    vltDOUBLE               decOffset;
    issprsSBS_DATA          teleSys;
    issprsSBS_DATA          guideSys;
    vltLOGICAL              nasGuidUtBeam;
    } issprsOFFSADG_DATA;

/* Data for OFFSGP command */
typedef struct
    {
    issprsOFFSGP_STATES     state;
    vltLOGICAL              foundUT;
    vltLOGICAL              singleBeam;
    vltDOUBLE               raOffset;
    vltDOUBLE               decOffset;
    issprsSBS_DATA          teleSys;
    issprsSBS_DATA          guideSys;
    vltLOGICAL              nasGuidUtBeam;
    } issprsOFFSGP_DATA;

/* Data for OFFSGUV command */
typedef struct
    {
    vltLOGICAL  dataOk;
    vltINT32    loop;
    vltDOUBLE   rotOffsetU;
    vltDOUBLE   rotOffsetV;
    vltDOUBLE   rotOffsetW;
    vltDOUBLE   remOffsetU;     /* remaining offset U (serialized OFFSGUV) */
    vltDOUBLE   remOffsetV;     /* remaining offset V (serialized OFFSGUV) */
    vltDOUBLE   remOffsetW;     /* remaining offset W (serialized OFFSGUV) */
    vltINT32    teleId;
    } issprsOFFSGUV_DATA;


typedef struct
    {
    vltBYTES8   telescopeType;
    vltDOUBLE   maxOffset;
    }   issprsMAX_OFFS_STEP_REC;

typedef issprsMAX_OFFS_STEP_REC issprsMAX_OFFS_STEPS[3];


#endif /*!ISSPRS_H*/
