#ifndef issprsACQ_MAIN_TASK_H
#define issprsACQ_MAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issprsACQ_MAIN_TASK.h,v 2.56 2005/06/08 13:29:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-03-21 added StatusCB() to allow verbose STATUS reply (in course
*                     of SPR VLTSW20040251)
* jott     2005-03-02 Added member mParamList (SPR VLTSW20030508)
* jott     2003-12-12 Added mContRecoGuvFlag to solve SPR VLTSW20030509
* jott     2003-12-04 Added mIssRa2000Deg and mIssDec2000Deg
* jott     2003-10-15 Removed method WriteAppliedGuvOffsets()
*                     and members mAppliedGuvOffsSymAddr, mAppliedGuvOffsDirAddr,
*                     because handling of instrument pointing offsets now is
*                     done by issalign module.
*                     (SPR VLTSW20030425)
* jott     2003-09-23 Added methods GobackProcessTrackEvt(), RecoguvAbort()
*                     and members mEvtVltiTrackGoback, mWaitTrackTimeout,
*                     mVltiTrackGobackTimer to handle wait for all subsystems
*                     are tracking in the course of a GOBACK command
*                     (aligned to UC 10.11 of ISS DD)
* rrollf    30/07/03  Added support for flag of appliedGUVoffsets
*                     (see SPR VLTSW20020475)
* rrollf    22/07/03  Removed check of Nasmyth Guiding and Active optics
*                     corrections made (see SPR VLTSW20030237)
* rrollf    05/05/03  Removed gobackFlags (see SPR VLTSW20020411)
* rrollf    12/02/03  Write runGoBackFlags (see SPR VLTSW20020411)
* rrollf    28/01/03  Adapted to VLTSW Release APR2003
* rrollf    17/10/02  Tuning of command timeouts (see SPR VLTSW20020347)
* rrollf    10/09/02  Added GobackSendPrsname to send PRSNAME "FREE" to
*                     issprsBroadcast while processing a GOBACK command
* rrollf    08/04/02  Use the library issCalc for computation of predicted
*                     positions (VLTSW20020095)
* rrollf    28/02/02  Added support for waitForUT attribute (VLTSW20020093)
* rrollf    11/02/02  Added mRecoGuvBeam for RECOGUV command with second
*                     parameter <beam> (SPR VLTSW20020096)
* jott      16/01/02  to solve SPR VLTSW20010787 and SPR VLTSW20020019:
*                     added RecoguvCB(), RecoguvOK(), RecoguvSendOffset() ,
*                     removed GobackSendStartCag(),
*                     added mWaitForNasGuidAndActOpt, mUTConfigured, mDataType,
*                     mRecoguvMsg, mRecoGuvSender, GobackSetNasGuideEvents(),
*                     and RecoguvUseCalibOffsets()
* jott      06/12/01  added method GobackSendStopCag() and GobackSendSetup()
*                     to solve SPR VLTSW20010746
* jott      12/10/01  added GoBackStepsCB(), GobackSendStartCag(),
*                     GobackSendOffset(void), mLocalProcId, mLocalProcName,
*                     mDlcsEnvName, mDlcsProcName, mGobackStepSender,
*                     mDlcsIgnore, mCurrArm, and mGobackState
* jott      09/10/01  added mDlcsDbReadList
* rrollf    21/06/01  created
*/

/************************************************************************
 * Header file for class issprsACQ_MAIN_TASK
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "cmd.h"
#include "issprs.h"
#include "evhDB_TASK.h"
#include "evhCOMMAND.h"
#include "fndSTRING.h"
#include "evhTIMEOUT.h"

#include "issprsACQ_STD_CMD.h"
#include "issprsACQ_SEND_CMD.h"
#include "tcsDB_SYNC_ERR_CHECK.h"
#include "issCALCULATOR.h"


class issprsACQ_MAIN_TASK : public evhDB_TASK
    {
    public:

        issprsACQ_MAIN_TASK(const dbSYMADDRESS            dbPoint,
                                  issprsACQ_STD_CMD      &stdCmds,
                                  issprsACQ_SEND_CMD    **sendCmds);
        ~issprsACQ_MAIN_TASK();

        /*
         * Callbacks for standard commands
         */
        virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);

        /*
         * Callbacks for public commands
         */
        virtual evhCB_COMPL_STAT RemembCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT GoBackCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT RecoguvCB(msgMESSAGE &msg, void *udata);

        /*
         * Callbacks for database events
         */
        evhCB_COMPL_STAT    DbCfgTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    DbMswTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    VltiGoesTrackingEventCB(evtEVENT_MSG &msg,
                                                    void *udata);

        ccsCOMPL_STAT Init();
        ccsCOMPL_STAT Recover();


    protected:
        virtual evhCB_COMPL_STAT  ExePresetCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  GoBackStepsCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  RecoguvOK(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT  ReadyCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT  StopOKCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorStopCB(msgMESSAGE &msg, void *udata);

        virtual ccsCOMPL_STAT     PresetAbort(vltINT32 errorNumber);
        virtual ccsCOMPL_STAT     RecoguvAbort(vltINT32 errorNumber);
        virtual ccsCOMPL_STAT     PresetAbortPermitted(vltLOGICAL &permitted,
                                                       fndSTRING  &reason);
        virtual ccsCOMPL_STAT     Reset();

        issprsACQ_STD_CMD        &mStdCmds;
        issprsACQ_SEND_CMD      **mSendCmds;     /* List of send objects */

        evhDB_COMMAND             mSwitchMode;
        evhDB_COMMAND             mSwitchModeEnd;
        tcsDB_SYNC_ERR_CHECK      mWaitAllReady;
        tcsDB_SYNC_ERR_CHECK      mWaitStopReady;
        evhCOMMAND                mGobackStepSender;
        evhCOMMAND                mRecoGuvSender;
        issCALCULATOR             mCalc;

    private:
        ccsCOMPL_STAT       PrepareDbAccess(void);
        ccsCOMPL_STAT       ComputePredictedPos(vltUINT8 slotId);

        ccsCOMPL_STAT       SendSetupCmd(vltUINT8 telId, vltUINT8 sendId,
                                         vltUINT8 *arm);
        ccsCOMPL_STAT       ReadMswTable(void);     /* Mode switching table */
        ccsCOMPL_STAT       ReadConfigTable(void);  /* VLTI configuration   */
        ccsCOMPL_STAT       SelectSubsystems(void); /* Select conf. systems */
        ccsCOMPL_STAT       WriteRefRaDec(void);
        ccsCOMPL_STAT       WritePredPos(void);
        ccsCOMPL_STAT       WriteCurrentSlot(const char *currentSlot);

        /*
         * Reading of current preset values from scanned subsystem branch
         */
        ccsCOMPL_STAT       ReadData(vltUINT8 slotNameId);

        /*
         * Writing the read preset values into own database slot
         */
        ccsCOMPL_STAT       WriteData(vltUINT8 slotNameId);

        /*
         * Miscellaneous methods
         */
        ccsCOMPL_STAT       SendErrorReply(msgMESSAGE &msg);
        ccsCOMPL_STAT       GetParentName(dbSYMADDRESS parentName,
                                          dbSYMADDRESS pointName);
        ccsCOMPL_STAT       GobackSendPrsname(void);
        ccsCOMPL_STAT       GobackSendStopCag(void);
        ccsCOMPL_STAT       GobackSendSetup(void);
        ccsCOMPL_STAT       GobackSendOffset(void);
        ccsCOMPL_STAT       GobackProcessTrackEvt(msgMESSAGE &msg);
        ccsCOMPL_STAT       RecoguvSendOffset(int dataType);
        ccsCOMPL_STAT       RecoguvUseCalibOffsets(char *buffer);

        /*
         * VLTI base point (:Appl_data:VLTI)
         */
        dbSYMADDRESS              mVltiBasePoint;

        /*
         * Base point of issprs module (:Appl_data:VLTI:issprs)
         */
        dbSYMADDRESS              mDbBasePoint;

        /*
         * Control point of issprsReacquire
         * (:Appl_data:VLTI:issprs:reacquire)
         */
        dbSYMADDRESS             mDbControlPoint;

        /*
         * Variables for the three data slots
         */
        int                     mActualSlotNr;
        issprsREMEMB            mSlot[3];
        issprsDLCS_READ_DATA    mDlcsData;
        dbLISTID                mSlotDbReadList[3];
        dbLISTID                mSlotDbWriteList[3];
        dbLISTID                mDataOkList;
        dbLISTID                mDlcsDbReadList;

        /*
         * structure used to retrieve command parameters
         */
        cmdPARAM_LIST           mParamList;

        /*
         *  Used subsystems (SID, UT, COU, AT)
         */
        int         mUsedSubSys[issprsACQ_NO_OF_SEND_OBJ];
        vltUINT8    mNumUsedSubSys;

        /*
         *  Used telescopes (SID, UT, AT)
         */
        vltUINT8    mNumUsedTele;

        /*
         * Config table and mode switching table
         */
        issprsVLTI_CFG_TABLE    mVltiCfgTable;
        issprsMSW_MOD_TABLE     mMswTable;

        /*
         * Global variables for PRESET command
         */
        vltDOUBLE               mIssRa;
        vltDOUBLE               mIssDec;
        vltDOUBLE               mIssRa2000Deg;
        vltDOUBLE               mIssDec2000Deg;
        vltDOUBLE               mIssRefRa;
        vltDOUBLE               mIssRefDec;
        vltDOUBLE               mPredAlt;
        vltDOUBLE               mPredAz;
        issOPL_DL               mPredOplDL;

        /*
         * DB listIDs
         */
        dbLISTID                mIssRaDecListId;
        dbLISTID                mIssPredPosListId;

        /*
         * Symbolic and direct addresses
         */
        dbSYMADDRESS            mMswTabSymAddr;
        dbSYMADDRESS            mCfgTabSymAddr;
        dbSYMADDRESS            mCurrSlotSymAddr;
        dbDIRADDRESS            mMswTabDirAddr;
        dbDIRADDRESS            mCfgTabDirAddr;
        dbDIRADDRESS            mCurrSlotDirAddr;

        /*
         * Events
         */
        evtEVENT                mEvtCfgTab;
        evtEVENT                mEvtMswTab;
        evtEVENT                mEvtVltiGoesTracking;
        evtEVENT                mEvtVltiTrackGoback;

        /*
         * Timer
         */
        evhTIMEOUT              *mVltiTrackGobackTimer;

        /*
         * Messages
         */
        msgMESSAGE              mPresetMsg;
        msgMESSAGE              mObjectMsg;
        msgMESSAGE              mGobackMsg;
        msgMESSAGE              mRecoguvMsg;

        /*
         * Timeouts
         */
        vltUINT32               mReadyTimeout;
        vltUINT32               mModeSwitchTimeout;
        vltUINT32               mBrShortTimeout;
        vltUINT32               mBrLongTimeout;
        vltUINT32               mBrVeryLongTimeout;
        vltUINT32               mWaitTrackTimeout;

        vltUINT8                mNumUsedSendObj;

        /*
         * Flags
         */
        vltLOGICAL              mNewMswTable;
        vltLOGICAL              mNewCfgTable;
        vltLOGICAL              mDlcsIgnore;
        vltLOGICAL              mLogRaDec;

        /*
         * States during processing the GOBACK command
         */
        issprsGOBACK_STEPS      mGobackState;

        /*
         * Process and environment names to be used during the course
         * of a GOBACK command
         */
        msgPROCESSID            mLocalProcId;
        ccsPROCNAME             mLocalProcName;
        ccsENVNAME              mDlcsEnvName;
        ccsPROCNAME             mDlcsProcName;

        /*
         * Number of currently processed record in array config table, used
         * in the course of a GOBACK command
         */
        int                     mCurrArm;

        /*
         * Type of data extraction for the OFFSGUV parameter in the course
         * of a RECOGUV command
         */
        int                     mDataType;

        /*
         * made as a member to check performance
         */
        ccsTIMEVAL              mGobackStartTime;

        /*
         * Global variables for RECOGUV command
         * (used by RecoguvCB, RecoguvSendOffset)
         */
        vltINT32                mRecoGuvBeam;
        vltLOGICAL              mContRecoGuvFlag;

        /*
         * Error codes
         */
        vltINT32                mSwitchModeError;
        vltINT32                mSwitchModeToErr;
        vltINT32                mGoBackError;
        vltINT32                mGoBackToErr;
        vltINT32                mRecoGuvError;
        vltINT32                mWaitError;
        vltINT32                mWaitToErr;
        vltINT32                mSetupError;
        vltINT32                mSetupToErr;
        vltINT32                mStopError;
        vltINT32                mStopToErr;

    };


#endif /* !issprsACQ_MAIN_TASK_H */
