#ifndef issprsAH_MAIN_TASK_H
#define issprsAH_MAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issprsAH_MAIN_TASK.h,v 2.56 2005/06/08 13:29:42 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-04-06 added method SendEpreset() and added optional parameter
*                     'epreset' to methods SendErrorReply() and PresetAbort()
*                     in course of SPR VLTSW20040251
* jott     2005-03-21 added StatusCB() to allow verbose STATUS reply (in course
*                     of SPR VLTSW20040251)
* rrollfin 2005-02-23 SPR VLTSW20050043: ISS should not send PRESET to a UT that
*                     is not in Coude focus
* rrollf   2004-09-07 SPR VLTSW20040226: ISS to support IRIS - setup
*                     Added callback SendStopLagCB and objects mStopLagCmd,
*                     mAlignSensorSymAddr, mAlignSensorDirAddr and mAlignSensor
* jott     2003-12-04 Added mIssRa2000Deg and mIssDec2000Deg
* jott     2003-10-15 Removed support for flag of appliedGUVoffsets because the
*                     instrument pointing offsets are now handled by issalign
*                     module (SPR VLTSW20030425)
* jott     2003-10-10 Added members mStopOptTimeout and mStopOptCmd,
*                     and method SendStopOptCB
* rrollf    30/07/03  Added support for flag of appliedGUVoffsets
*                     (see SPR VLTSW20020475)
* rrollf    22/07/03  Removed check of Active optics corrections made
*                     (see SPR VLTSW20030237)
* rrollf    05/05/03  Removed issprsKEYWD_RESET object (see SPR VLTSW20020411)
* rrollf    14/02/02  Added handler for GOBACK keywords (see SPR VLTSW20020411)
* rrollf    28/01/03  Adapted to VLTSW Release APR2003
* rrollf    21/10/02  Get reference coordinates of target (SPR VLTSW20020567)
* rrollf    15/10/02  Introduced two timeouts for issprsBroadcast
*                     (see SPR VLTSW20020347)
* rrollf    09/09/02  Renamend mBroadcastCmd to mStopCagCmd and added
*                     mPrsnameCmd
* rrollf    31/07/02  Added callback for CLRSTP command (see SPR VLTSW20020411)
* rrollf    30/07/02  Added ReplaceAgTypeDefault()
* rrollf    22/07/02  Added mAralCs (VLTSW20020433)
* rrollf    03/05/02  Added members for guide type table (VLTSW20020287)
* rrollf    08/04/02  Use the library issCalc for computation of predicted
*                     positions (VLTSW20020095)
* rrollf    19/02/02  Added computation of predicted positions (VLTSW20020095)
* rrollf    08/02/02  Added support for waitForUT attribute (VLTSW20020093)
* rrollf    16/01/02  Added new callback and some member variables for
*                     SPR VLTSW20020019 (Check AG and AO for UT SETUP's)
* rrollf    05/12/01  Added mBroadcastCmd, mBroadcastTimeout and GoPresetCB
*                     for STOPCAG command (see SPR VLTSW20010746)
* rrollf    15/10/01  Added new event and callback (VltiGoesTracking)
*                     Added global variables for SETUP command
* rrollf    25/06/01  added DbCfgTabEventCB to set mNewCfgTable
* rrollf    30/04/01  third version (Version 1 of ISS)
* rrollf    19/03/01  second version
* jott      20/02/01  first version
* jott      17/01/01  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "issprs.h"
#include "evhDB_TASK.h"
#include "evhCOMMAND.h"
#include "fndSTRING.h"

#include "issprsAH_STD_COMMANDS.h"
#include "issprsAH_TELE.h"
#include "issprsAH_GUIDE.h"
#include "issprsAH_DELAY.h"
#include "issprsARAL.h"
#include "tcsDB_SYNC_ERR_CHECK.h"
#include "issCALCULATOR.h"

class issprsAH_MAIN_TASK : public evhDB_TASK
    {
    public:
        issprsAH_MAIN_TASK(const dbSYMADDRESS            dbPoint,
                                 issprsAH_STD_COMMANDS  &stdCmds,
                                 issprsAH_TELE          &tele,
                                 issprsAH_GUIDE         &guide,
                                 issprsAH_DELAY         &delay,
                                 issprsAH_STD_MOD       &ao,
                                 issprsAH_STD_MOD       &prima,
                                 issprsARAL             &aral
                           );

        ~issprsAH_MAIN_TASK();

        virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT ClrStpCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT PresetCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);

        /* Callbacks for database events */
        evhCB_COMPL_STAT    DbCfgTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    DbMswTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    UtFocusChangedEvtCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    VltiGoesTrackingEventCB(evtEVENT_MSG &msg,
                                                    void *udata);

        ccsCOMPL_STAT Init();
        ccsCOMPL_STAT Recover();

    protected:
        virtual evhCB_COMPL_STAT  SendStopCagCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  SendStopLagCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  GoPresetCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ExePresetCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  SendStopOptCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorSwitchModeCB(msgMESSAGE &msg,
                                                    void       *udata);

        virtual evhCB_COMPL_STAT  PresetReadyCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT  StopOKCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorStopCB(msgMESSAGE &msg, void *udata);

        virtual ccsCOMPL_STAT     PresetAbort(vltINT32   errorNumber,
                                              vltLOGICAL epreset = ccsTRUE);
        virtual ccsCOMPL_STAT     PresetAbortPermitted(vltLOGICAL &permitted,
                                                       fndSTRING  &reason);
        virtual ccsCOMPL_STAT     Reset();

        issprsAH_STD_COMMANDS   &mStdCmds;
        issprsAH_TELE           &mTeleCs;
        issprsAH_GUIDE          &mGuideCs;
        issprsAH_DELAY          &mDelayCs;
        issprsAH_STD_MOD        &mAoCs;
        issprsAH_STD_MOD        &mPrimaCs;
        issprsARAL              &mAralCs;

        evhDB_COMMAND            mStopCagCmd;
        evhDB_COMMAND            mStopLagCmd;
        evhDB_COMMAND            mPrsnameCmd;
        evhDB_COMMAND            mSwitchMode;
        evhDB_COMMAND            mSwitchModeEnd;
        evhDB_COMMAND            mStopOptCmd;
        tcsDB_SYNC_ERR_CHECK     mWaitPresetReady;
        tcsDB_SYNC_ERR_CHECK     mWaitStopReady;

        oslxALIAS               *mAlias;
        oslxDICTIONARY          *mDictionary;
        oslxSETUP               *mCurrentSetupFile;

        issCALCULATOR           mCalc;

    private:

        /*
         * Private member functions
         */
        ccsCOMPL_STAT       PrepareDbAccess(void);
        ccsCOMPL_STAT       UpdateIssRefRaDec(void);
        ccsCOMPL_STAT       ComputePredictedPos(void);

        ccsCOMPL_STAT       ReadUtFocus(vltUINT8 utID, vltLOGICAL &isFocusCoude);
        ccsCOMPL_STAT       ReadMswTable(void);     /* Mode switching table */
        ccsCOMPL_STAT       ReadConfigTable(void);  /* VLTI configuration   */
        ccsCOMPL_STAT       ReadGuideTypeTable(void);
        ccsCOMPL_STAT       WriteRefRaDec(void);
        ccsCOMPL_STAT       WritePredPos(void);
        ccsCOMPL_STAT       WriteCurrentSlot(const char *currentSlot);


        ccsCOMPL_STAT       SendErrorReply(msgMESSAGE &msg,
                                           vltLOGICAL epreset = ccsTRUE);
        ccsCOMPL_STAT       SendEpreset(void);
        ccsCOMPL_STAT       GetParentName(dbSYMADDRESS parentName,
                                          dbSYMADDRESS pointName);

        /* Functions to extract setup keyword from the current setup file */
        ccsCOMPL_STAT       GetKeywordString(char** string,
                                             const char* keywordName);
        ccsCOMPL_STAT       GetKeywordDouble(vltDOUBLE* value,
                                             const char* keywordName);

        /* Functions to handle some special operation on setup file */
        ccsCOMPL_STAT       ReplaceAgTypeDefault(void);

        /* VLTI base point   */
        /* ":Appl_data:VLTI" */
        dbSYMADDRESS             mVltiBasePoint;

        /* base point of prs module */
        /* ":Appl_data:VLTI:issprs" */
        dbSYMADDRESS             mDbBasePoint;

        /* control point of issprsAction      */
        /* ":Appl_data:VLTI:issprs:action"    */
        dbSYMADDRESS             mDbControlPoint;

        /*
         * Config table ans mode switching table
         */
        issprsVLTI_CFG_TABLE    mVltiCfgTable;
        issprsMSW_MOD_TABLE     mMswModTable;

        /*
         * Guide type table
         * (contains the best guide type for a telescope type)
         */
        issprsGUIDE_TYPE_TABLE  mGuideTypeTable;

        /*
         * Global variables for PRESET command
         */
        vltDOUBLE               mIssRa;
        vltDOUBLE               mIssDec;
        vltDOUBLE               mIssRa2000Deg;
        vltDOUBLE               mIssDec2000Deg;
        vltDOUBLE               mIssRefRa;
        vltDOUBLE               mIssRefDec;
        vltDOUBLE               mPredAlt;
        vltDOUBLE               mPredAz;
        issOPL_DL               mPredOplDL;

        int                     mTelTargType;
        vltBYTES20              mAlignSensor;

        /*
         * DB listIDs
         */
        dbLISTID                mIssRaDecListId;
        dbLISTID                mIssPredPosListId;

        /*
         * Symbolic and direct addresses
         */
        dbSYMADDRESS            mMswTabSymAddr;
        dbSYMADDRESS            mCfgTabSymAddr;
        dbSYMADDRESS            mUtFocusSymAddr[issNUM_UT];
        dbSYMADDRESS            mGuideTypeTabSymAddr;
        dbSYMADDRESS            mCurrSlotSymAddr;
        dbSYMADDRESS            mAlignSensorSymAddr;
        dbDIRADDRESS            mMswTabDirAddr;
        dbDIRADDRESS            mCfgTabDirAddr;
        dbDIRADDRESS            mUtFocusDirAddr[issNUM_UT];
        dbDIRADDRESS            mGuideTypeTabDirAddr;
        dbDIRADDRESS            mCurrSlotDirAddr;
        dbDIRADDRESS            mAlignSensorDirAddr;

        /*
         * Events
         */
        evtEVENT                mEvtCfgTab;
        evtEVENT                mEvtMswTab;
        evtEVENT                mEvtUtFocus[issNUM_UT];
        evtEVENT                mEvtVltiGoesTracking;

        /*
         * Messages
         */
        msgMESSAGE              mPresetMsg;
        msgMESSAGE              mObjectMsg;

        vltBYTES64              mCurrentSetup;

        /*
         * Timeout values
         */
        vltUINT32               mPresetReadyTimeout;
        vltUINT32               mBrShortTimeout;
        vltUINT32               mBrLongTimeout;
        vltUINT32               mModeSwitchTimeout;
        vltUINT32               mStopOptTimeout;

        /*
         * Error codes for callbacks
         */
        vltINT32                mWaitError;
        vltINT32                mWaitToErr;
        vltINT32                mBroadcastError;
        vltINT32                mBroadcastToErr;
        vltINT32                mSwitchModeError;
        vltINT32                mSwitchModeToErr;
        vltINT32                mStopError;
        vltINT32                mStopToErr;
        vltINT32                mPresetSeqErr;

        /*
         * flags
         */
        vltLOGICAL              mNewMswTable;
        vltLOGICAL              mNewCfgTable;
        vltLOGICAL              mUtFocusChanged;
        vltLOGICAL              mLogRaDec;
};


#endif /* !issprsAH_MAIN_TASK_H */
