#ifndef issprsBR_MAIN_TASK_H
#define issprsBR_MAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issprsBR_MAIN_TASK.h,v 2.56 2005/06/08 13:29:43 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-04-12 SPR VLTSW20040255:
*                     added method OffsgpCB()
*                     added members mOffsgpActive, mOffsgpBeam,
*                     mOffsgpData[]
* jott     2005-04-05 removed use of definition SPR_VLTSW20050036 because it is
*                     now standard
* jott     2005-03-21 added StatusCB() to allow verbose STATUS reply (in course
*                     of SPR VLTSW20040251)
* jott     2005-03-01 SPR VLTSW20050043:
*                     Added member arrays  mEvtUtFocus[],  mUtFocusSymAddr[],
*                     mUtFocusDirAddr[], mUtFocusChanged
*                     added methods CheckUtFocus(), UtFocusChangedEvtCB()
* rrollfin 2005-02-18 SPR VLTSW20050036: Added mWaitMacvPerfDataReady,
*                     mWaitForMacao and SetTargetForXEx()
* rrollfin 2005-01-27 SPR VLTSW20040416: Added TrkOkFlagCB, mOffsOpdCommandId,
*                     mDbAdrDlXTrkOkFlag, mWaitForDlXTrkOkFlag
* rrolff   2004-09-30 SPR VLTSW20040229: ISS to support IRIS - OFFSGUV
*                     Added objects: mLabGuidingSymAddr, mLabGuidingDirAddr,
*                     mAlignSensor, mIsIrisSelected, mLabGuiding
*                     Added methods: ReadIAS(), ReadLabGuid()
* rrollf   2004-09-07 SPR VLTSW20040227: ISS to support IRIS - chopping
*                     Added objects mAlignSensorSymAddr and mAlignSensorDirAddr
* jott     2004-05-18 SPR VLTSW20040134: Support calibrating of FINITO detector
*                     algorithms:
*                     Added CalFntCB() mDetCalibrator(), CalibrationCompleted(),
*                     mDetCalibFlag, mCalibMsg
* jott     2004-03-10 Added extension for OFFSGUV command in case of UT and not
*                     Coude guiding - methods ConvertUVW2RaDec() and
*                     SendOffsgpCmd() (SPR VLTSW2004055)
*                     Removed no longer used variables mStrtCagActive,
*                     mStrtCagMsg, mInstrOffsSend (remnants of SPR VLT20020475)
*                     to tauten code.
* jott     2003-11-26 Added member mAlignMsg to fix bug in handling subsequent
*                     beam aligning commands
* jott     2003-11-04 Added member mChpStopTime to modify stop chopping
*                     Renamed StrtChpOkCB to ChopOkCB
* jott     2003-10-22 Added callbacks StartAoCB() and StopAoCB() (SPR
*                     VLTSW20030424)
* jott     2003-10-03 Removed methods PntOffsCB(), ReadInstrGuvOffsets(),
*                     WriteInstrGuvOffsets(), SendInstrOffs(),
*                     ReadAppliedGuvOffsets(), WriteAppliedGuvOffsets(),
*                     and members mInstrGuvOffsSymAddr, mInstrGuvOffsDirAddr,
*                     mInstrGuvOffs[6], mAppliedGuvOffsSymAddr,
*                     mAppliedGuvOffsDirAddr, mGuvOffsetsApplied,
*                     because handling of pointing offsets now is done by
*                     module issalign (SPR VLTSW20030425)
* jott     2003-10-02 Added flag mBeamAlignFlag and removed mOptfluxTimeout and
*                     mFndBeamTimeout
*                     Added method BeamAlignCompleted() and removed FndBeamCB()
*                     (SPR VLTSW20030410)
* rrollf    24/07/03  Added new command PNTOFFS to store the insturment GUV
*                     offsets. This offsets are used by STRTCAG command.
*                     (see SPR VLTSW20020475)
* rrollf    23/06/03  Changes concerning the error handling of OFFSADG and
*                     OFFSGUV commands (see SPR VLTSW20030202)
* jott      03/06/03  Added callbacks for commands FNDBEAM, OFFSFTS, SETFNTM,
*                     and ZEROOPD
*                     Added members mFndBeamTimeout, mFntShortTimeout, and
*                     mFntLongTimeout
* rrollf    19/05/03  Added OPTFLUX command (see SPR VLTSW20030203)
* rrollf    13/02/03  Added timeout for OPTCGS (see SPR VLTSW20030048)
* rrollf    30/10/02  Duplicated number of send objects for OFFSADG
*                     (to send offset to UT and COU)
* rrollf    11/10/02  Serialized OFFSGUV (see SPR VLTSW20020551)
* rrollf    26/09/02  Commands OFFSGUV and OFFSADG now supports parallel
*                     processing for different beams. (see SPR VLTSW20020289)
* rrollf    01/08/02  Added OFFSTEL command (see SPR VLTSW20020463)
* rrollf    24/07/02  Added AbortStartChopping(void)
* rrollf    19/07/02  Added mPrsoplTimeout, mPrsnameTimeout and mOptdlcsTimeout
*                     (see SPR VLTSW20020347)
* rrollf    17/07/02  Added ReadInpChFntBeamMap(), mInpChFntBeamMapSymAddr,
*                     mInpChFntBeamMapDirAddr, mInpChFntBeamMap
*                     (see SPR VLTSW20020210)
* rrollf    17/01/02  Support for SPR VLTSW20010790:
*                     OFFSGUV values stored before transformation
* rrollf    25/10/01  Added OptDlcsCB callback for command OPTDLCS
* rrollf    15/10/01  Added global variables for OFFSGUV command
* rrollf    09/10/01  Added some global variables for OFFSADG
* rrollf    08/10/01  Added mWaitSuspended, mWaitOffsAdgReady and
*                     mWaitResumed for synchronization
* rrollf    18/09/01  Using one transformation matrix
* rrollf    10/09/01  added OptCgsCB callback of OPTCGS command
* rrollf    06/09/01  updated OFFSGUV command
* rrollf    08/06/01  added PRSNAME command (SPR VLTSW20010305)
*                     (awalland 2001-05-11 Added PRSNAME command)
*                     added PRSOPL command (SPR VLTSW20010306)
* rrollf    30/04/01  third version (Version 1 of ISS)
* jott      29/03/01  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "evhDB_TASK.h"
#include "evhDB_COMMAND.h"
#include "tcsDB_SYNC_ERR_CHECK.h"
#include "issprs.h"
#include "issprsBR_COMMAND.h"
#include "issprsBR_STD_COMMANDS.h"
#include "issprsBR_OFFSTEL.h"
#include "issprsBR_AUTOCOLL.h"
#include "issprsBR_ALIGN.h"
#include "issprsBR_CALIB.h"


class issprsBR_MAIN_TASK : public evhDB_TASK
    {
    public:
        issprsBR_MAIN_TASK(const dbSYMADDRESS           dbPointName,
                                 issprsBR_STD_COMMANDS &stdCmds,
                                 issprsBR_COMMAND      &broadcastControl,
                                 issprsBR_OFFSTEL      &offstelControl,
                                 issprsBR_AUTOCOLL     &autocollControl,
                                 issprsBR_ALIGN        &beamAligner,
                                 issprsBR_CALIB        &detCalibrator);
        ~issprsBR_MAIN_TASK();

        /*
         * Callbacks for public commands
         */
        virtual evhCB_COMPL_STAT DetCgsCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OptCgsCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT OffsAdgCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffsgpCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffsTelCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffsOpdCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffsGuvCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT PrsNameCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT PrsOplCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT StartCagCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopCagCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT StartChopCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopChopCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT StartAoCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopAoCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT StartFtkCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopFtkCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT StartExCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT EndExCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT ClrStpCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT OptDlcsCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT OptFluxCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffsFtsCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT SetFntMCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT ZeroOpdCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT CalFntCB(msgMESSAGE &msg, void *udata);


        /*
         * Overloading of inherited commands
         */
        virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);


        /*
         * Callbacks for database events
         */
        evhCB_COMPL_STAT    DbCfgTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    DbMswTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    UtFocusChangedEvtCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    ResetUvwOffsEventCB(evtEVENT_MSG &msg, void *udata);


        /*
         * Callback to get information about completion of beam alignment
         */
        evhCB_COMPL_STAT    BeamAlignCompleted(msgMESSAGE &msg, void *udata);

        /*
         * Callback to get information about completion of FINITO detector
         * calibration
         */
        evhCB_COMPL_STAT    CalibrationCompleted(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT    Init();
        ccsCOMPL_STAT Recover();

    protected:
        virtual evhCB_COMPL_STAT  ReplyOkCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  OffsetOkCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  OffsAdgExecCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  OffsAdgOkCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ChopOkCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  ErrorCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT  TimeoutCB(msgMESSAGE &msg, void *udata);

        /* Introduced for VLTSW20040416 */
        virtual evhCB_COMPL_STAT  TrkOkFlagCB(msgMESSAGE &msg, void *);

        virtual ccsCOMPL_STAT     Reset();

        issprsBR_STD_COMMANDS   &mStdCmds;
        issprsBR_COMMAND        &mBroadcastControl;
        evhCOMMAND              mOffsetCommand[2 * issMAX_NUM_ARMS];

        /*
         *  Object for OFFSTEL command
         */
        issprsBR_OFFSTEL        &mOffstelHandler;

        /*
         * Object to handle autocollimation
         */
        issprsBR_AUTOCOLL       &mAutocollHandler;

        /*
         * Object to handle beam alignment commands
         */
        issprsBR_ALIGN          &mBeamAligner;

        /*
         * Object to handle beam alignment commands
         */
        issprsBR_CALIB          &mDetCalibrator;

        tcsDB_SYNC_ERR_CHECK    mWaitAllReady;
        tcsDB_SYNC_ERR_CHECK    mWaitSuspended;
        tcsDB_SYNC_ERR_CHECK    mWaitOffsAdgReady;
        tcsDB_SYNC_ERR_CHECK    mWaitResumed;
        tcsDB_SYNC_ERR_CHECK    mWaitChopReady;

    private:

        /*
         * Private member functions
         */
        ccsCOMPL_STAT       PrepareDbAccess(void);

        /* Read Mode switching table */
        ccsCOMPL_STAT       ReadMswTable(void);

        /* Read VLTI configuration */
        ccsCOMPL_STAT       ReadConfigTable(void);

        /* Check focus of UT's */
        ccsCOMPL_STAT       CheckUtFocus(vltUINT8 utID, vltLOGICAL &isFocusCoude);

        /* Read FITS bin producer */
        ccsCOMPL_STAT       ReadFbpTable(void);

        /* Read Input Channel Finito Beam Map */
        ccsCOMPL_STAT       ReadInpChFntBeamMap(void);

        /* Read the current image alignment sensor */
        ccsCOMPL_STAT       ReadIAS(void);

        /* Read Laboratory Guiding Status */
        ccsCOMPL_STAT       ReadLabGuid(void);

        /* Read transformation matrix for OFFSGUV command */
        ccsCOMPL_STAT       ReadTransMat(vltLOGICAL  bcFlag,
                                         vltLOGICAL  location,
                                         vltDOUBLE  *paramX);

        /* Read the maximum allowed offset step */
        ccsCOMPL_STAT       GetMaxOffsStep(vltBYTES8  telescopeType,
                                           vltDOUBLE *maxOffsetStep);

        /* Read and write accumulated U'V'W' offsets for OFFSGUV command */
        ccsCOMPL_STAT       ReadUVW(issprsUVW_OFFS_TAB *accumulator);
        ccsCOMPL_STAT       WriteUVW(issprsUVW_OFFS_TAB *accumulator);

        ccsCOMPL_STAT       SetTargetList(issprsBR_CMD brCmd);
        ccsCOMPL_STAT       SetTargetForXEx(issprsBR_CMD brCmd,          // IN
                                            vltINT32     telId,          // IN
                                            vltLOGICAL   &addModId,      // OUT
                                            vltLOGICAL   &dataValidFlag, // OUT
                                            vltINT32     &couId);        // OUT
        ccsCOMPL_STAT       CheckForUT(vltINT32 beam, vltLOGICAL *foundUT);

        ccsCOMPL_STAT       SendErrorReply(msgMESSAGE &msg,
                                           vltLOGICAL setState = ccsTRUE);

        ccsCOMPL_STAT       SendOffsgpCmd(vltINT32  beam,
                                          vltINT32  telePos,
                                          vltDOUBLE offsAlpha,
                                          vltDOUBLE offsDelta);

        ccsCOMPL_STAT       SplitModId(const vltBYTES32  modId,
                                             ccsENVNAME  env,
                                             ccsPROCNAME proc);

        ccsCOMPL_STAT       GetLocation(vltLOGICAL  &location,
                                        char        *station);

        ccsCOMPL_STAT       AbortStartChopping(void);

        ccsCOMPL_STAT       AbortBroadcast(vltLOGICAL error = ccsFALSE);

        ccsCOMPL_STAT       AbortOffsGuv(vltINT32 beam);

        ccsCOMPL_STAT       GetParentName(dbSYMADDRESS parentName,
                                          dbSYMADDRESS pointName);

        ccsCOMPL_STAT       ConvertUVW2RaDec(vltINT32       beam,
                                             vltINT32       ut,
                                             vltDOUBLE      offsU,
                                             vltDOUBLE      offsV,
                                             vltDOUBLE      offsW,
                                             vltDOUBLE     *offsRa,
                                             vltDOUBLE     *offsDec);

        /* List of command parameters */
        cmdPARAM_LIST            mParamList;

        /* VLTI base point   */
        /* ":Appl_data:VLTI" */
        dbSYMADDRESS             mVltiBasePoint;

        /* base point of prs module */
        /* ":Appl_data:VLTI:issprs" */
        dbSYMADDRESS             mDbBasePoint;

        /* control point of issprsBroadcast   */
        /* ":Appl_data:VLTI:issprs:broadcast" */
        dbSYMADDRESS             mDbControlPoint;

        /*
         * config table, mode switching table, table of
         * fitsBinProducer and transformation matrix
         */
        issprsVLTI_CFG_TABLE    mVltiCfgTable;
        issprsMSW_MOD_TABLE     mMswModTable;
        issprsFBP_TABLE         mFbpModTable;
        issprsTRANSF_MAT        mTransfMat;
        issprsMAX_OFFS_STEPS    mMaxOffsSteps;


        /*
         * symbolic addresses
         */
        dbSYMADDRESS            mMswTabSymAddr;
        dbSYMADDRESS            mCfgTabSymAddr;
        dbSYMADDRESS            mUtFocusSymAddr[issNUM_UT];
        dbSYMADDRESS            mFbpTabSymAddr;
        dbSYMADDRESS            mTransfMatSymAddr;
        dbSYMADDRESS            mMaxOffsSymAddr;
        dbSYMADDRESS            mUvwOffsSymAddr;
        dbSYMADDRESS            mInpChFntBeamMapSymAddr;
        dbSYMADDRESS            mAlignSensorSymAddr;
        dbSYMADDRESS            mLabGuidingSymAddr;

        /*
         * direct addresses
         */
        dbDIRADDRESS            mMswTabDirAddr;
        dbDIRADDRESS            mCfgTabDirAddr;
        dbDIRADDRESS            mUtFocusDirAddr[issNUM_UT];
        dbDIRADDRESS            mFbpTabDirAddr;
        dbDIRADDRESS            mTransfMatDirAddr;
        dbDIRADDRESS            mMaxOffsDirAddr;
        dbDIRADDRESS            mUvwOffsDirAddr;
        dbDIRADDRESS            mInpChFntBeamMapDirAddr;
        dbDIRADDRESS            mAlignSensorDirAddr;
        dbDIRADDRESS            mLabGuidingDirAddr;

        /*
         * events
         */
        evtEVENT                mEvtCfgTab;
        evtEVENT                mEvtMswTab;
        evtEVENT                mEvtUtFocus[issNUM_UT];
        evtEVENT                mEvtResetUvwOffs;

        /*
         * Broadcast message and timeout
         */
        msgMESSAGE              mBroadcastMsg;
        msgMESSAGE              mObjectMsg;
        msgMESSAGE              mOffsetMsg[issMAX_NUM_ARMS];
        msgMESSAGE              mAlignMsg;
        msgMESSAGE              mCalibMsg;
        vltINT32                mOffsetSendId[2 * issMAX_NUM_ARMS];
        vltUINT32               mStdTimeout;
        vltUINT32               mPrsoplTimeout;
        vltUINT32               mPrsnameTimeout;
        vltUINT32               mOptdlcsTimeout;
        vltUINT32               mOptcgsTimeout;
        vltUINT32               mFntShortTimeout;
        vltUINT32               mFntLongTimeout;

        /*
         * Global variable for STRTCHP/STOPCHP commands
         */
        vltBYTES32              mChpStrtTime;
        vltBYTES32              mChpStopTime;

        /*
         * Global variables for OFFSADG command
         * (used by OffsAdgCB, OffsAdgExecCB, OffsAdgOkCB)
         */
        vltINT32                mOffsAdgBeam;
        issprsOFFSADG_DATA      mOffsAdgData[issMAX_NUM_ARMS];

        /*
         * Global variables for OFFSGP command
         * (used by OffsgpCB and SetTargetList)
         */
        vltINT32                mOffsgpBeam;
        issprsOFFSGP_DATA       mOffsgpData[issMAX_NUM_ARMS];


        /*
         * Global variables for OFFSGUV command
         * (used by OffsGuvCB, ReplyOkCB)
         */
        issprsOFFSGUV_DATA      mOffsGuvData[issMAX_NUM_ARMS];
        vltBYTES20              mAlignSensor;
        vltLOGICAL              mIsIrisSelected;
        vltLOGICAL              mLabGuiding;

        /*
         * Introduced for VLTSW20040416
         *
         * Global variables for OFFSOPD command
         * (used by OffsOpdCB, ReplyOkCB)
         */
        issprsBR_CMD            mOffsOpdCommandId;
        dbSYMADDRESS            mDbAdrDlXTrkOkFlag;
        evhDB_SYNC              mWaitForDlXTrkOkFlag;

        /*
         * Introduced for VLTSW20050036
         *
         * Global variables for ENDEX command
         * (used by SetTargetList, ReplyOkCB)
         */
        evhCB_COMPL_STAT        EndexOkCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT        EndexToCB(msgMESSAGE &msg, void *udata);
        vltLOGICAL              mWaitMacvPerfDataReady;
        evhDB_SYNC              mWaitForMacao;

        /*
         *  Global VLTI parameters
         */
        vltINT32    mInpChFntBeamMap[issNUMBER_OF_INPUT_CHANNEL_IDS];

        /*
         * List of telescope descriptions for autocollimation
         */
        issprsBR_AC_TEL_LST     mAutocollList;

        /*
         * Error codes for callbacks
         */
        vltINT32                mBroadcastError;
        vltINT32                mBroadcastToErr;

        /*
         * flags and auxiliary members
         */
        vltLOGICAL              mNewMswTable;
        vltLOGICAL              mNewCfgTable;
        vltLOGICAL              mUtFocusChanged;
        vltLOGICAL              mOffsGuvActive;
        vltLOGICAL              mOffsAdgActive;
        vltLOGICAL              mOffsgpActive;
        vltLOGICAL              mOffsBeam[issMAX_NUM_ARMS];
        vltLOGICAL              mBeamAlignFlag;
        vltLOGICAL              mDetCalibFlag;
    };


#endif /*!issprsBR_MAIN_TASK_H*/

/* __oOo__ */

