#ifndef issprsSEQ_MAIN_TASK_H
#define issprsSEQ_MAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: issprsSEQ_MAIN_TASK.h,v 2.56 2005/06/08 13:29:43 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-03-21 Moved StatusCB() from issprsSEQ_STD_COMMNANDS to here
*                     to allow verbose reply (SPR VLTSW20040251)
* rrollf    05/05/03  Removed issprsKEYWD_RESET object (see SPR VLTSW20020411)
* rrollf    14/02/02  Added handler for GOBACK keywords (see SPR VLTSW20020411)
* rrollf    09/10/02  Special handling of ARAL setup (see SPR VLTSW20020550)
* rrollf    31/07/02  Added SendErrorReply()
* rrollf    17/07/02  Added mDbBasePoint and GetParentName()
* rrollf    08/06/01  removed PRSOPL command (SPR VLTSW20010306)
* rrollf    14/05/01  callbacks GobackCB and RemembCB removed
*                     moved to the fourth process of issprs module
* rrollf    30/04/01  third version (Version 1 of ISS)
* jott      05/04/01  intermediate version
* jott      14/03/01  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "evhDB_TASK.h"
#include "evhDB_COMMAND.h"
#include "evhDB_CMD_SERIAL.h"
#include "tcsDB_SYNC_ERR_CHECK.h"
#include "issprs.h"
#include "issprsARAL.h"
#include "issprsSEQ_STD_COMMANDS.h"

/* Defines */

#define  issprsNOMOVE                 0
#define  issprsMOVE                   1

class issprsSEQ_MAIN_TASK : public evhDB_TASK
    {
    public:
        issprsSEQ_MAIN_TASK(const dbSYMADDRESS      dbPointName,
                            issprsSEQ_STD_COMMANDS &stdCmds,
                            evhDB_CMD_SERIAL       &actionCmd,
                            evhDB_CMD_SERIAL       &broadcastCmd,
                            evhDB_CMD_SERIAL       &reacquireCmd,
                            issprsARAL             &aral         );
        ~issprsSEQ_MAIN_TASK();

        /*
         * Callbacks for public commands
         */
        virtual evhCB_COMPL_STAT ClrStpCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT SetupCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT PrsCoorCB(msgMESSAGE &msg, void *udata);

        /*
         * Overloading of inherited commands
         */
        virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StopCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT VerboseCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT    Init();
        ccsCOMPL_STAT Recover();

    protected:
        virtual evhCB_COMPL_STAT Execute(msgMESSAGE &msg,
                                         vltLOGICAL moveFlag  = issprsMOVE,
                                         vltLOGICAL aralSetup = ccsFALSE);
        virtual evhCB_COMPL_STAT ExeReadyCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT ExeErrorCB(msgMESSAGE &msg, void *udata);

        virtual evhCB_COMPL_STAT ReplyOKCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT ReplyErrorCB(msgMESSAGE &msg, void *udata);



        ccsCOMPL_STAT            RejectIfNotInStateList(msgMESSAGE &msg,
                                                int        *listOfAllowedStates,
                                                int         numberOfStates,
                                                vltLOGICAL *rejected);


        oslxDICTIONARY         *mGlobalDictionary;
        oslxALIAS              *mGlobalAlias;
        oslxSETUP              *mReadySetupFile;
        oslxSETUP              *mCheckSetupFile;

        issprsSEQ_STD_COMMANDS &mStdCmds;
        evhDB_CMD_SERIAL       &mActionCmd;
        evhDB_CMD_SERIAL       &mBroadcastCmd;
        evhDB_CMD_SERIAL       &mReacquireCmd;
        evhDB_COMMAND           mPresetCmd;
        issprsARAL             &mAralCs;
        tcsDB_SYNC_ERR_CHECK    mWaitReady;


    private:

        ccsCOMPL_STAT       SendErrorReply(msgMESSAGE &msg);
        ccsCOMPL_STAT       GetParentName(dbSYMADDRESS parentName,
                                          dbSYMADDRESS pointName);

        msgMESSAGE      mStdCmdMsg;
        msgMESSAGE      mSetupMsg;

        /*
         * Configuration parameters (read from DB) in Init():
         */
        vltBYTES64      mReadySetup;
        vltBYTES64      mDefaultSetup;
        vltBYTES64      mSetupDictionary;
        vltBYTES64      mSetupAliasTable;
        vltUINT32       mReadyTimeout;
        vltUINT32       mAralTimeout;
        vltUINT32       mMsgTimeout;

        /* base point of prs module */
        /* ":Appl_data:VLTI:issprs" */
        dbSYMADDRESS             mDbBasePoint;

        /* control point of issprsAction      */
        /* ":Appl_data:VLTI:issprs:action"    */
        dbSYMADDRESS             mDbControlPoint;

    };


#endif /* !issprsSEQ_MAIN_TASK_H */

/* __oOo__ */
