#ifndef isstrkwsMAIN_TASK_H
#define isstrkwsMAIN_TASK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: isstrkwsMAIN_TASK.h,v 2.23 2005/06/18 20:24:54 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jott     2005-06-06 SPR VLTSW20040333:
*                     Added method ComputeProjBaselineAngles(),
*                     added arithmeticError' class
* jott     2005-04-20 In frmaework of SPR VLTSW20050128:
*                     Added methods StateCB(), StateChangeReplyCB(),
*                     ForwardStateChange()
*                     Added members mChangeQueryState, mLastStateCmd
* jott     2005-02-14 SPR VLTSW20050036:
*                     Added member mMacao and reference to MACAO object in
*                     constructor,
*                     added ExitCB() and modified StatusCB()
* jott     2004-04-21 Added methods UpdateUtNasGuidFlags(), and
*                     CreateNasmythGuideList()
*                     added members mCoudeGuideState[], mUtNasGuideState[],
*                     mUtNasGuideMode[], and mNasmythCheckList to continue
*                     solving SPR VLTSW20040055 (see comments on this SPR dated
*                     20-Apr-2004)
* jott     2004-03-18 Added method CheckUtNasmythGuiding() to continue solving
*                     SPR VLTSW20040055
*                     Included check of adaptive optics into method
*                     CheckSubSystems() to avoid additional database read access
*                     -> removed method UpdateGaoData()
* jott     2004-03-17 Added member variables mUtNasGuideFlags[] to allow
*                     independant handling of UT Nasmyth guiding
*                     (SPR VLTSW20040055)
* jott      15/09/03  Made site coordinates to member variables, because
*                     latitude is also needed to calculate parallactic angle
*                     (SPR VLTSW20030154). Removed member mPrlticOffset.
* rrollf    14/08/03  Added computation of image and pupil rotation
*                     (see SPR VLTSW20030273)
* rrollf    08/08/03  Added function to update guidingOk and opticsOk flags
*                     which should be used by isstcl via issif public database
*                     to check nasmyth guiding and active optics for all UTs
*                     (see SPR VLTSW20030237)
* rrollf    12/12/01  Added substate
* rrolff    20/11/01  Added variables for baseline calculation
* rrollf    14/11/01  Added vltLOGICAL mTestOn and ccsTIMEVAL mTestUtc
* rrollf    12/11/01  Removed mHaOffset, not more needed
* rrollf    09/11/01  Removed mDec2000degOffset, not more needed
* rrollf    06/11/01  Added mPhDelay[issMAX_NUM_ARMS]
*                     Use of issMAX_NUM_ARMS instead of isstrkwsMAX_NUM_ARMS
* rrollf    01/10/01  Added ChkHmsNAttr() and ChkDegN90Attr()
* rrollf    21/09/01  Added new methods: ChkXXXAttr()
* rrollf    20/09/01  Added mDlcsIgnored flag
* rrollf    19/09/01  Use of isstrkwsINFO structure (mInfo)
* rrollf    13/09/01  New event callback when VLTI goes IDLE
* rrollf    30/08/01  Avoiding of error flooding for time conversion
*                     Added mTimeToNextError and mErrorWaitTime
* rrollf    30/04/01  Second version (Version 1 of ISS)
* rrollf    20/02/01  First version
* rrollf    31/01/01  Created
*/

/************************************************************************
 * Header file of the isstrkwsMAIN_TASK class which tracks changes of
 * sub-system data provides public data representing VLTI as an entity
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/* Set to '1' for more debug informations */
#define isstrkwsDEBUG 0

/* Include error messages */
#include "eccsErr.h"
#include "eccsTestTools.h"
#include "isstrkwsErrors.h"

/* Include parent classes */
#include "evhTIMER.h"
#include "evhHANDLER.h"
#include "evhSTD_COMMANDS.h"
#include "evhCOMMAND.h"
#include "db.h"
#include "isstrkws.h"
#include "isstrkwsMACAO.h"


class isstrkwsMAIN_TASK : public evhSTD_COMMANDS
    {
    public:
        isstrkwsMAIN_TASK(const dbSYMADDRESS  dbControlPoint,
                          const dbSYMADDRESS  dbBasePoint,
                          isstrkwsMACAO       &macao);
        ~isstrkwsMAIN_TASK(void);


        ccsCOMPL_STAT       Init(void);

        /* Callbacks for receiving commands */
        virtual evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OffCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT OnlineCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StandByCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT TestCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StateCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT StatusCB(msgMESSAGE &msg, void *udata);
        virtual evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *udata);

        /* Callback for command replies */
        evhCB_COMPL_STAT    StateChangeReplyCB(msgMESSAGE &msg, void *udata);

        /* Callbacks for database events */
        evhCB_COMPL_STAT    DbMswTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    DbCfgTabEventCB(evtEVENT_MSG &msg, void *udata);
        evhCB_COMPL_STAT    VltiGoesTrackingEventCB(evtEVENT_MSG &msg,
                                                    void *udata);

        ccsCOMPL_STAT       Recover(void);

        /*
         * Error class
         */
        class arithmeticError {};

    protected:

        ccsCOMPL_STAT    SubState(vltINT32 subState);
        vltINT32         SubState();

        /* for debugging only */
        long             mLoop;
        #if isstrkwsDEBUG
        ccsCOMPL_STAT    ShowMswData(void);
        ccsCOMPL_STAT    ShowConfigData(void);
        ccsCOMPL_STAT    ShowTelescopeData(isstrkwsTEL_DATA telData);
        ccsCOMPL_STAT    ShowDelayLineData(void);
        #endif

    private:

        /*
         * private methods
         */

        evhCB_COMPL_STAT    PollingCycleCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    ShadowingCycleCB(msgMESSAGE &msg, void *udata);
        evhCB_COMPL_STAT    ShadowingOneShootCB(msgMESSAGE &msg, void *udata);

        ccsCOMPL_STAT       ReadTelData(vltUINT8 telNum);
        ccsCOMPL_STAT       ReadDelayLineData(void);
        ccsCOMPL_STAT       CalcOutputData(void);
        ccsCOMPL_STAT       CalcImagePupilRot(void);
        ccsCOMPL_STAT       CheckSubSysData(void);
        ccsCOMPL_STAT       UpdateUtNasGuidFlags(void);
        ccsCOMPL_STAT       CheckUtNasmythGuiding(vltLOGICAL   acceptNasGuiding,
                                                  int          activeTelIndex,
                                                  vltLOGICAL  *newNasGuideFlag);
        ccsCOMPL_STAT       ReadMswTable(void);
        ccsCOMPL_STAT       ReadConfigTable(void);
        ccsCOMPL_STAT       ReadIsstrkwsConfig(void);
        ccsCOMPL_STAT       WriteIsstrkwsData(void);
        ccsCOMPL_STAT       WriteShadowData(void);
        ccsCOMPL_STAT       CreateDataList(void);
        ccsCOMPL_STAT       CreateNasmythGuideList(void);
        ccsCOMPL_STAT       CreateShadowDataList(void);
        ccsCOMPL_STAT       CreateConfigList(void);
        ccsCOMPL_STAT       CreateTeleDataList(vltUINT8 telNum);
        ccsCOMPL_STAT       CreateDelayDataList(void);

        ccsCOMPL_STAT       MinMaxHms(vltDOUBLE *value,
                                      vltDOUBLE *minValue,
                                      vltDOUBLE *maxValue,
                                      vltDOUBLE  input,
                                      vltDOUBLE  mean,
                                      vltDOUBLE  range,
                                      vltDOUBLE  offset);

        ccsCOMPL_STAT       MinMaxDms(vltDOUBLE *value,
                                      vltDOUBLE *minValue,
                                      vltDOUBLE *maxValue,
                                      vltDOUBLE  input,
                                      vltDOUBLE  mean,
                                      vltDOUBLE  range);

        /*
         * ChkXxxAttr() methods to check the ranges of telescope data
         */

        /* HMS: [0.0, +240000.0] */
        ccsCOMPL_STAT   ChkHmsAttr(vltDOUBLE value, char *name, char *tel);

        /* HMS: [-240000.0, +240000.0] */
        ccsCOMPL_STAT   ChkHmsNAttr(vltDOUBLE value, char *name, char *tel);

        /* DMS: [-900000.0, +900000.0] */
        ccsCOMPL_STAT   ChkDmsAttr(vltDOUBLE value, char *name, char *tel);

        /* deg: [0.0, +360.0] */
        ccsCOMPL_STAT   ChkDeg360Attr(vltDOUBLE value, char *name, char *tel);

        /* deg: [0.0, +90.0] */
        ccsCOMPL_STAT   ChkDeg90Attr(vltDOUBLE value, char *name, char *tel);

        /* deg: [-90.0, +90.0] */
        ccsCOMPL_STAT   ChkDegN90Attr(vltDOUBLE value, char *name, char *tel);

        /* deg: [-180.0, +360.0] */
        ccsCOMPL_STAT   ChkDegN180Attr(vltDOUBLE value, char *name, char *tel);


        /*
         * Function to read int32 value from db and check
         * for bad quality errors on stack
         */
        ccsCOMPL_STAT   ReadIntCheckQuality(vltINT32 *int32,
                                            const char *attrName);

        /* Compute image and pupil rotation */
        ccsCOMPL_STAT   GetImgPupRot(isstrkwsIMG_PUP_ROT_DATA* data);

        /* forward state changing commands to other isstrkws processes */
        ccsCOMPL_STAT   ForwardStateChange(msgMESSAGE msg);

        /* compute projected baseline angles with vector algebraic approach */
        ccsCOMPL_STAT   ComputeProjBaselineAngles(void);

        /*
         * private data
         */

        /* like ":Appl_data:VLTI" */
        dbSYMADDRESS             mVltiBasePoint;

        /* like ":Appl_data:VLTI:isstrkws" */
        dbSYMADDRESS             mDbBasePoint;

        /* Substate for isstrkws */
        dbSYMADDRESS             mSubStateSymAddr;
        dbDIRADDRESS             mSubStateDirAddr;

        /* Converts physical DL's to logical */
        vltUINT8                 mLogDelay[issNUM_PHYSICAL_DL];

        /* Converts logical DL's to physical */
        vltUINT8                 mPhDelay[issMAX_NUM_ARMS];

        /* Data of DLCS */
        isstrkwsDELAY_DATA       mDelayData;

        /* Actual shadow value of ISS */
        vltDOUBLE                mShadow;

        /* Remaining tracking time before shadowing */
        vltINT32                 mRemtimeShadow;

        /* Mode switching table of issmsw */
        isstrkwsMSW_MOD_TABLE    mMswTable;

        /* Part of configTable of issarray */
        isstrkwsVLTI_CFG_TABLE   mVltiConfigTable;

        /* Message object */
        msgMESSAGE               mMsg;

        /* Event object for mode switching table */
        evtEVENT                 mEvtMswTab;

        /* Event object for VLTI configuration table */
        evtEVENT                 mEvtCfgTab;

        /* Event object for VLTI is going to trackingstate TRACKING */
        evtEVENT                 mEvtVltiGoesTracking;

        /* Timer objects */
        evhTIMER                *mPollTimer;
        evhTIMER                *mShadowTimer;
        evhTIMER                *mShadowOneShootTimer;

        /* Avoid flooding of error logs */
        vltINT32                mTimeToNextError;
        vltINT32                mErrorWaitTime;

        /* Number of configured arms */
        vltUINT8                 mConfiguredArms;

        /* List of configured arms */
        vltINT8                  mListOfConfArms[issMAX_NUM_ARMS];

        /* Number of active telescopes */
        vltUINT8                 mActiveTelescopes;

        /* List of active telescopes */
        vltUINT8                 mListOfActiveTele[isstrkwsMAX_NUM_TELESCOPES];

        /* Info structure (badQuality flags and number of used attributes) */
        isstrkwsINFO             mInfo;

        /* Data of configured telescopes */
        isstrkwsTEL_DATA         mTelData[isstrkwsMAX_NUM_TELESCOPES];

        /* DB lists of telescope attributes */
        dbLISTID                 mTeleDataList[isstrkwsMAX_NUM_TELESCOPES];

        /* DB list to handle Nasmyth guiding flags for UT (SPR VLTSW20040055) */
        dbLISTID                 mNasmythCheckList;

        /* Data for "isstrkws:data" */
        isstrkwsDATA             mData;

        /* Data for "isstrkws:chop" */
        isstrkwsCHOP             mChop;

        /* Data for "isstrkws:cfg" */
        isstrkwsCFG              mConfig;

        /* Offsets for shifting data into the middle of the valid range */
        vltDOUBLE                mLstOffset;
        vltDOUBLE                mRaOffset;
        vltDOUBLE                mRa2000Offset;
        vltDOUBLE                mCgsRa2000Offset;
        vltDOUBLE                mRa2000degOffset;
        vltDOUBLE                mAzOffset;

        /* Guiding state (is true if VLTI is guiding) */
        vltLOGICAL               mGuidingState;

        /* Tracking state (is true if VLTI is tracking) */
        vltLOGICAL               mTrackingState;

        /*
         * Guiding Ok flag
         *   Is true when all configured and not ignored UT's are Nasmyth
         *   Guding or no UT is configured or all configured UT's are ignored.
         */
        vltLOGICAL               mGuidingOk;

        /*
         * Optics Ok flag
         *   Is true when for all configured and not ignored UT's the active
         *   optics correction is done. If no UT is configured or all UT's are
         *   ignored then this flag is also true.
         */
        vltLOGICAL               mOpticsOk;

        /* Handles of attribute lists for database access */
        dbLISTID                 mIsstrkwsData;
        dbLISTID                 mIsstrkwsShadowData;
        dbLISTID                 mConfigData;
        dbLISTID                 mDelayDataList;

        dbSYMADDRESS             mMswTabSymAddr;
        dbSYMADDRESS             mCfgTabSymAddr;
        dbDIRADDRESS             mMswTabDirAddr;
        dbDIRADDRESS             mCfgTabDirAddr;

        /* UVW Coordinates of configured telescopes */
        isstrkwsUvwCoords        mUvwCoords[issMAX_NUM_ARMS];

        /* Baseline data T1T2, T1T3, T2T3 */
        vltDOUBLE                mBaseLineT1T2;
        vltDOUBLE                mBaseLineT1T3;
        vltDOUBLE                mBaseLineT2T3;
        vltDOUBLE                mBaseLineT1T2Sqr;
        vltDOUBLE                mBaseLineT1T3Sqr;
        vltDOUBLE                mBaseLineT2T3Sqr;

        /* Reference angles for az = 0 */
        vltDOUBLE                mAngAz0T1T2;
        vltDOUBLE                mAngAz0T1T3;
        vltDOUBLE                mAngAz0T2T3;

        /* Specified UTC for tat tests */
        ccsTIMEVAL               mTestUtc;
        char                     mTestUtcString[32];

        /* Flags */
        vltLOGICAL               mNewMswTable;
        vltLOGICAL               mNewVltiConfigTable;
        vltLOGICAL               mDlcsIgnored;
        vltLOGICAL               mArmConfigured[issMAX_NUM_ARMS];
        vltLOGICAL               mTestOn;

        /* Site coordinates */
        vltDOUBLE                mLongitude;
        vltDOUBLE                mLatitude;

        /* Nasmyth guiding variables for UTs */
        vltINT32                 mCoudeGuideState[4];
        vltINT32                 mUtNasGuideState[4];
        vltINT32                 mUtNasGuideMode[4];
        vltLOGICAL               mUtNasGuideFlags[4];
        vltBYTES32               mGuideStarSource;

        /* Object handling mapping and updating of MACAO quality control data */
        isstrkwsMACAO           &mMacao;

        /* Object forwarding state change commands to isstrkwsQuery process */
        evhCOMMAND               mChangeQueryState;

        /* last state changing command */
        msgMESSAGE               mLastStateCmd;
    };


#endif /*!isstrkwsMAIN_TASK_H*/
