#ifndef ixacCMD_PARAM_H
#define ixacCMD_PARAM_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ixacCMD_PARAM.h,v 1.49 2005/04/08 12:44:31 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* zins      02/12/98  created
* zins      19/05/99  added CmdParamDefault() and InitIdxCDT2ParamPos() methods,
*                     and declared parameter 'param' of constructor as optional.
* martiny   20/05/99  declared ATTOL class as friend to be able to access
*                     private and protected members for unit test purposes
* rondeaux  11/07/00  added all CmdReplyParamXXXXX methods.
* gzins     12/07/00  changed private members to protected
* gzins     17/11/00  removed ATTOL macros
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif
/*
 * VLT Software
 */
#include "fndOBJECT.h"
#include "cmd.h"
#include "chkc.h"
#include "slxDynBuf.h"
#include "eccs.h"

#define ixacMAXNBPARAM  99 // Max. number of parameters of a command.

class ixacCMD_PARAM  : public fndOBJECT, public eccsERROR_CLASS
{
public:
    ixacCMD_PARAM(const char *cmdName, const char *paramPtr = NULL );
    virtual ~ixacCMD_PARAM();

    virtual const char *GetCommand();
    virtual const char *GetParam();

    virtual ccsCOMPL_STAT CmdParamParse(char *paramPtr = NULL );

    virtual ccsCOMPL_STAT CmdParamDefault(const ccsPROCNAME processName);

    virtual ccsCOMPL_STAT CmdParamGetLogical
        (const vltINT32 index, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdParamGetInt
        (const vltINT32 index, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetDouble
        (const vltINT32 index, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetString
        (const vltINT32 index, vltINT32 *paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetLogical
        (const char *paramName, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdParamGetInt
        (const char *paramName, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetDouble
        (const char *paramName, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdParamGetString
        (const char *paramName, vltINT32 *paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetLogical
        (const vltINT32 index, vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamSetInt
        (const vltINT32 index, vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetDouble
        (const vltINT32 index, vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetString
        (const vltINT32 index, vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetSingleString
        (const vltINT32 index, char *value);

    virtual ccsCOMPL_STAT CmdParamSetLogical
        (char *paramName, vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamSetInt
        (char *paramName, vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetDouble
        (char *paramName, vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamSetString
        (char *paramName, vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamSetSingleString
        (char *paramName, char *value);

    virtual ccsCOMPL_STAT CmdParamAddLogical(vltLOGICAL value);

    virtual ccsCOMPL_STAT CmdParamAddInt
        (vltINT32 paramNb, vltINT32 *paramValues);

    virtual ccsCOMPL_STAT CmdParamAddDouble
        (vltINT32 paramNb, vltDOUBLE *paramValues);

    virtual ccsCOMPL_STAT CmdParamAddString
        (vltINT32 paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdParamAddSingleString(char *value);

    virtual ccsCOMPL_STAT CmdParamDelete(const vltINT32 index);

    virtual ccsCOMPL_STAT CmdParamDelete(const char *paramName);

    virtual ccsCOMPL_STAT MakeMsgBuf();
    virtual const char* GetMsgBuf();

    // Methods for reply parameters
    virtual ccsCOMPL_STAT CmdReplyParamParse(char *paramPtr = NULL);

    virtual ccsCOMPL_STAT CmdReplyParamGetLogical
        (const vltINT32 index, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdReplyParamGetLogical
        (const char *paramName, vltLOGICAL *value);

    virtual ccsCOMPL_STAT CmdReplyParamGetInt
        (const vltINT32 index, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdReplyParamGetInt
        (const char *paramName, vltINT32 *paramNb, vltINT32 **paramValues);

    virtual ccsCOMPL_STAT CmdReplyParamGetDouble
        (const vltINT32 index, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdReplyParamGetDouble
        (const char *paramName, vltINT32 *paramNb, vltDOUBLE **paramValues);

    virtual ccsCOMPL_STAT CmdReplyParamGetString
        (const vltINT32 index, vltINT32 *paramNb, char **paramValues);

    virtual ccsCOMPL_STAT CmdReplyParamGetString
        (const char *paramName, vltINT32 *paramNb, char **paramValues);

protected:
    cmdPARAM_LIST paramList;
    char          *command;
    char          *param;

    vltLOGICAL    parsed;
    slxDYN_BUF    msgBuf;

    // The following table gives the parameter position in the parameter list
    // deduced from the parameter index given both the CDT and parameter name.
    // This suppresses the problem of shift of the parameter position when a
    // parameter is deleted (see cmdParamDelete(3)). This table is updated each
    // time a parameter is deleted.
    virtual ccsCOMPL_STAT InitIdxCDT2ParamPos();
    vltUINT32       idxCDT2ParamPos[ixacMAXNBPARAM];

private:
    // Declaration of copy constructor and assignment operator as private
    // methods, in order to hide them from the users.
    ixacCMD_PARAM& operator=(const ixacCMD_PARAM&);
    ixacCMD_PARAM (const ixacCMD_PARAM&);

};

#endif /*!ixacCMD_PARAM_H*/
