#ifndef ixacKEY_DB_MAP_H
#define ixacKEY_DB_MAP_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ixacKEY_DB_MAP.h,v 1.49 2005/04/08 12:44:32 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  14/02/99  created
* martiny   20/05/99  declared ATTOL class as friend to be able to access
*                     private and protected members for unit test purposes
* gzins     12/07/00  changed private members to protected
* gzins     17/11/00  removed ATTOL macros
* sskole    11/07/03  Added assignDbValue & assignKeyValue (PPRS 10103)
*
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "eccsErr.h"
#include "slx.h"
#include "oslx.h"
#include "ixacHASH.h"

#define ixacMAX_POINTS  1000


class ixacKEY_DB_MAP : public eccsERROR_CLASS
{
public:
    ixacKEY_DB_MAP();
    ~ixacKEY_DB_MAP();

    ccsCOMPL_STAT Reset();
    ccsCOMPL_STAT LoadMapFile(const slxFILENAME  mapFile,
			      const vltLOGICAL   merge = FALSE);
    ccsCOMPL_STAT SetDictionary(oslxDICTIONARY  *dictionary);

    vltLOGICAL IsInKeyDbMap(const slxKEYW_LINE  keyword);
    vltLOGICAL IsRemoteAttr(const slxKEYW_LINE  keyword);
    ccsCOMPL_STAT GetDbAddress(const oslxKEYWORD &keyword,
                               dbSYMADDRESS addr);

    ccsCOMPL_STAT MakeMsgBuffer(oslxSHORT_FITS &keywordList,
                            oslxDYN_BUF &dynBuf,
                            oslxKEYW_FILTER *filter = (oslxKEYW_FILTER *)NULL);

    ccsCOMPL_STAT WriteKeywListToDb(oslxSHORT_FITS &keywordList,
                            vltLOGICAL logChange,
                            oslxKEYW_FILTER *filter = (oslxKEYW_FILTER *)NULL);
    
    // Methods to write an attribute associated to a key
    ccsCOMPL_STAT WriteKeyToDb(oslxKEYWORD &keyword, vltLOGICAL log = ccsFALSE);

    ccsCOMPL_STAT WriteDb(const slxKEYW_LINE  keyword,
			  const vltINT32      value);
    ccsCOMPL_STAT WriteDb(const slxKEYW_LINE  keyword,
			  const char          *value);
    ccsCOMPL_STAT WriteDb(const slxKEYW_LINE  keyword,
			  const vltLOGICAL    value);    
    ccsCOMPL_STAT WriteDb(const slxKEYW_LINE  keyword,
			  const vltDOUBLE     value);
    ccsCOMPL_STAT WriteDb(const slxKEYW_LINE  keyword,
			  const vltFLOAT      value);

    // Functions to read a keyword from the DB

    ccsCOMPL_STAT ReadKeyFromDb(oslxKEYWORD &keyword);
    
    ccsCOMPL_STAT ReadDb(const slxKEYW_LINE  keyword,
			 vltINT32            *value);
    ccsCOMPL_STAT ReadDb(const slxKEYW_LINE  keyword,
			 char*               value,
			 vltINT32            valueSize = slxKEYW_LINE_LEN);
    ccsCOMPL_STAT ReadDb(const slxKEYW_LINE  keyword,
			 vltLOGICAL          *value);
    ccsCOMPL_STAT ReadDb(const slxKEYW_LINE  keyword,
			 vltFLOAT            *value);  
    ccsCOMPL_STAT ReadDb(const slxKEYW_LINE  keyword,
			 vltDOUBLE           *value);  

    // Functions to read a keyword from the DB return the value directly
    // These functions should be used with cautious
    vltINT32    RdDbI(const slxKEYW_LINE  keyword);
    char       *RdDbS(const slxKEYW_LINE  keyword);
    vltLOGICAL  RdDbL(const slxKEYW_LINE  keyword);
    vltFLOAT    RdDbF(const slxKEYW_LINE  keyword);
    vltDOUBLE   RdDbD(const slxKEYW_LINE  keyword);

    // Extract keywords from the DB and store them in a oslxSHORT_FITS
    ccsCOMPL_STAT SnapShotFromDb(oslxSHORT_FITS     &targetObj,
                                 oslxKEYW_FILTER    *filter = NULL);

protected:
    char            *dbAccessTbl_[ixacMAX_POINTS];
    char            *dbAttrTbl_[ixacMAX_POINTS];
    vltLOGICAL      dbRemoteAccessTbl_[ixacMAX_POINTS];
    ixacHASH        key2AttrTbl_;
    vltINT32        pointCount_;
    oslxDICTIONARY  *dictionary_;

private:
    // Declaration of copy constructor and assignment operator as private
    // methods, in order to hide them from the users.
    ixacKEY_DB_MAP& operator=(const ixacKEY_DB_MAP&);
    ixacKEY_DB_MAP (const ixacKEY_DB_MAP&);

    ccsCOMPL_STAT WriteDb_(const slxKEYW_LINE  keyword,
			   const void *keyValue,
			   slxPAR_TYPE keyValueType);

    ccsCOMPL_STAT ReadDb_(const slxKEYW_LINE  keyword,
			  const void *keyBuf,
			  slxPAR_TYPE keyBufType,
			  vltINT32 keyBufSize);

    ccsCOMPL_STAT assignDbValue(const void *dbValue, 
				dbTYPE dbValueType,
				const void *keyBuf,
				slxPAR_TYPE keyType,
				vltINT32 keyBufSize);

    ccsCOMPL_STAT assignKeyValue(const void *keyValue,
				 slxPAR_TYPE keyValueType,
				 const void *dbBuf, 
				 dbTYPE dbBufType);
};


#endif /*!ixacKEY_DB_MAP_H*/

