/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: motDefines.h,v 2.36 2005/08/23 14:13:46 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* P.Duhoux  07/07/95  Created  for Release 1.3
* P.Duhoux  25/07/95  Added new literals for optimized unit conversions
*                     motBOARD_UNIT_... motUNIT_... [POS/SPD/CUR/REF]
* P.Duhoux  04/12/95  Added new literal for motion to HOME position
* P.Duhoux  26/03/96  Added macros for position index with section (SPR960183)
* P.Duhoux  26/03/96  Added macros motHALF_TURN_ABS/REL (SPR960512)
* P.Duhoux  13/12/96  Added macros motACTION_DELAY
*/

#ifndef MOT_DEFINES_H
#define MOT_DEFINES_H

/******************************************************************************
 * motDefines.h - MCM definition file
 *
 * This file contains all definitions needed for MCM as #define macros. 
 *
 *-----------------------------------------------------------------------------
 */
/* formats for dbcfg files */
#define motDBCFG_FMT       "%s.dbcfg"
#define motDBCFG_CWP_FMT   "%s=@%s<alias>%s"
#define motDBCFG_CWP       "<CWP>: "

/* literals for event types */
#define motEVENT_MOTION_END		0
#define motEVENT_DRIVE_FAULT       	1
#define motEVENT_EMERGENCY_STOP    	2
#define motEVENT_OVER_TEMPERATURE       3
#define motEVENT_OVER_CURRENT       	4
#define motEVENT_POSITIONING_ERROR 	5
#define motEVENT_ON_LIMIT          	6
#define motEVENT_NORMAL_STOP            7
#define motEVENT_INTERLOCK_ACTIVE       8
#define motEVENT_NEW_MOTION             9
#define motEVENT_BOARD_INIT_READY      10

/* Public events (mot<event> = 2 << motEVENT_<event>) */
#define motMOTION_END		 1
#define motDRIVE_FAULT       	 2
#define motEMERGENCY_STOP    	 4
#define motOVER_TEMPERATURE      8
#define motOVER_CURRENT       	16
#define motPOSITIONING_ERROR 	32
#define motON_LIMIT          	64
#define motNORMAL_STOP         128
#define motINTERLOCK_ACTIVE    256
#define motNEW_MOTION          512
#define motBOARD_INIT_READY   1024

/* logical flags */
#define motNOT_SUPPORTED  0
#define motNOT_APPLICABLE 0
#define motAVAILABLE      1
#define motSET            1

/* empty string */
#define motEMPTY_STRING   ""

/* interlock status strings */
#define motUNLOCKED_STRING "Unlocked"
#define motLOCKED_STRING   "Locked"

/* brake status strings */
#define motUNCLAMPED_STRING "Unclamped"
#define motCLAMPED_STRING   "Clamped"

/* axis status strings */
#define motDISABLED_STRING  "Disabled"
#define motENABLED_STRING   "Enabled"

/* special values */
#define motINVALID -1
#define motNONE     0

/*  8 bit signed extrema */
#define motMIN8  -128
#define motMAX8   127

/* 16 bit signed extrema */
#define motMIN16 -32768
#define motMAX16  32767

/* literal for maximum number of special positions */
#define motMAX_SPECIAL_POSITION  7

/* literal for maximum number of motors to be controlled */
#define motMAX_MOTOR	        32

/* literal for maximum number of control boards */
#define motMAX_BOARD	         3

/* literal for maximum number of events */
#define motMAX_EVENT            32

/* literal for maximum number of digital IO signals */
#define motMAX_SIGNAL           4

/* literal for minimum number of conversion methods */
#define motMIN_CONV_METHOD      4

/* literal for maximum number of steps in a motion sequence */
#define motMAX_MOTION_SEQ       10

/* literal for minimum polling rate in milli-seconds */
#define motMIN_POLL_RATE       100
#define motMIN_MOVE_POLL_FAR   100
#define motMIN_MOVE_POLL_NEAR   20

/* literal for maximum number of interpolation points */
#define motMAX_INTERPOL_POINTS  50

/* literal for maximum length of motor name/alias */
#define motMAX_MOTORNAME_LENGTH 12

/* definition of access modes */ 
#define motUNUSED     0
#define motREAD_ONLY  1
#define motEXCLUSIVE  2
#define motEMERGENCY  3
      
#define motUNUSED_STRING     "Unused"
#define motREAD_ONLY_STRING  "Read-Only"
#define motEXCLUSIVE_STRING  "Exclusive"
#define motEMERGENCY_STRING  "Emergency"
      
/* operational modes */ 
#define motDETACHED       1
#define motNOT_AVAILABLE  2
#define motFIXED_POSITION 3
#define motSIMULATION     4
#define motNORMAL         5
#define motHANDSET        6

#define motDETACHED_STRING       "Detached"
#define motNOT_AVAILABLE_STRING  "NotAvailable"
#define motFIXED_POSITION_STRING "FixedPosition"
#define motSIMULATION_STRING     "Simulation"
#define motNORMAL_STRING         "Normal"
#define motHANDSET_STRING        "Handset"

/* axis types */
#define motAXIS_OFF          motNONE
#define motAXIS_LINEAR       1
#define motAXIS_CIRCULAR     2
#define motAXIS_CIRCULAR_OPT 3

#define motAXIS_OFF_STRING          "Off"
#define motAXIS_LINEAR_STRING       "Linear"
#define motAXIS_CIRCULAR_STRING     "Circular"
#define motAXIS_CIRCULAR_OPT_STRING "CircularOptimized"

/* encoder types */
#define motENC_NONE     0
#define motENC_INC      1
#define motENC_ABS      2
#define motENC_ABS_REL  3

#define motENC_NONE_STRING     "None"
#define motENC_INC_STRING      "Incremental"
#define motENC_ABS_STRING      "Absolute"
#define motENC_ABS_REL_STRING  "Absolute in relative mode"

/* position/speed definition types */ 
#define motBY_DEFAULT 1
#define motBY_VALUE   2
#define motBY_NAME    3
#define motBY_INDEX   4

#define motBY_DEFAULT_STRING "ByDefault"
#define motBY_VALUE_STRING   "ByValue"
#define motBY_NAME_STRING    "ByName"
#define motBY_INDEX_STRING   "ByIndex"

#define motNAMED_POSITION_DUMMY "DUMMY"

/* section types */
#define motNO_SECTION   motNONE
#define motOBSERVATION  1
#define motMAINTENANCE  2
#define motUSER_DEFINED 3
#define motALL_SECTIONS 4

#define motNO_SECTION_STRING   "None"
#define motOBSERVATION_STRING  "Observation"
#define motMAINTENANCE_STRING  "Maintenance"
#define motUSER_DEFINED_STRING "UserDefined"
#define motALL_SECTIONS_STRING "AllSections"

/* section with index */
#define motSET_OBSERVATION_INDEX(i) \
    (vltUINT32)((i & 0x00FFFFFF) | (vltUINT32)motOBSERVATION << 24)
#define motSET_MAINTENANCE_INDEX(i) \
    (vltUINT32)((i & 0x00FFFFFF) | (vltUINT32)motMAINTENANCE << 24)
#define motSET_USERDEFINED_INDEX(i) \
    (vltUINT32)((i & 0x00FFFFFF) | (vltUINT32)motUSER_DEFINED << 24)

/* position types */
#define motABSOLUTE          1
#define motRELATIVE          2
#define motLOWER_LIMIT       3
#define motUPPER_LIMIT       4
#define motREFERENCE_SWITCH  5
#define motINDEX_PULSE       6
#define motHOME_POSITION     7
#define motBY_SPEED          8
#define motTWO_STEP_ABS      9
#define motTWO_STEP_REL     10
#define motHALF_TURN_ABS    11 
#define motHALF_TURN_REL    12 

#define motOFF_POSITION    0x100
#define motOFF_REFERENCE_SWITCH (motOFF_POSITION | motREFERENCE_SWITCH)

#define motABSOLUTE_STRING         "Absolute"
#define motRELATIVE_STRING         "Relative"
#define motLOWER_LIMIT_STRING      "LowerLimit"
#define motUPPER_LIMIT_STRING      "UpperLimit"
#define motREFERENCE_SWITCH_STRING "ReferenceSwitch"
#define motINDEX_PULSE_STRING      "IndexPulse"
#define motHOME_POSITION_STRING    "HomePosition"
#define motBY_SPEED_STRING         "BySpeed"
#define motTWO_STEP_ABS_STRING     "TwoStepAbsolute"
#define motTWO_STEP_REL_STRING     "TwoStepRelative"
#define motHALF_TURN_ABS_STRING    "HalfTurnAbsolute"
#define motHALF_TURN_REL_STRING    "HalfTurnRelative"

/* motion modes */
#define motINVALID_MOTION_MODE         1
#define motDIRECT_MODE                 2
#define motPOSITION_MODE               3
#define motTRACKING_MODE               4
#define motSPEED_MODE                  5

#define motINVALID_MOTION_MODE_STRING "Invalid"
#define motDIRECT_MODE_STRING         "Direct"
#define motPOSITION_MODE_STRING       "Position"
#define motTRACKING_MODE_STRING       "Tracking"
#define motSPEED_MODE_STRING          "Speed"

/* electrical connection types for amplifier boards */
#define motDISCONNECTED 0
#define motPOSITIVE     1
#define motNEGATIVE     2
#define motBOTH         3

#define motDISCONNECTED_STRING "Disconnected"
#define motPOSITIVE_STRING     "Positive"
#define motNEGATIVE_STRING     "Negative"
#define motBOTH_STRING         "Both connected"

/* board types */
#define motNO_BOARD   0
#define motAMPLIFIER  1
#define motCONTROLLER 2
#define motDIGITAL_IO 3
#define motCPU_BOARD  4
#define motEXT_BOARD  5

#define motNO_BOARD_STRING   "No Board"
#define motAMPLIFIER_STRING  "Amplifier"
#define motCONTROLLER_STRING "Controller"
#define motDIGITAL_IO_STRING "Digital I/O"
#define motCPU_BOARD_STRING  "CPU Board"
#define motEXT_BOARD_STRING  "External Board"

/* board status */
#define motBOARD_ERROR ERROR
#define motBOARD_OK    OK
#define motBOARD_RESET 1
#define motBOARD_NONE  2

/* limits types */
#define motNOT_ON_LIMIT 0
#define motSW_LOWER     1
#define motSW_UPPER     2
#define motSW_BOTH      3
#define motHW_LOWER     4
#define motHW_UPPER     5
#define motHW_BOTH      6
 
#define motNOT_ON_LIMIT_STRING "Not on Limit"
#define motSW_LOWER_STRING     "Lower SW Limit"
#define motSW_UPPER_STRING     "Upper SW Limit"
#define motSW_BOTH_STRING      "Both SW Limit"
#define motHW_LOWER_STRING     "Lower HW Limit"
#define motHW_UPPER_STRING     "Upper HW Limit"
#define motHW_BOTH_STRING      "Both HW Limit"
 
/* special positions indices */
#define motSPECIAL_POS_LSW 0
#define motSPECIAL_POS_USW 1
#define motSPECIAL_POS_LHW 2
#define motSPECIAL_POS_UHW 3
#define motSPECIAL_POS_REF 4
#define motSPECIAL_POS_IND 5
#define motSPECIAL_POS_HOM 6

/* motion status */
#define motSTANDING  1
#define motMOVING    2
#define motTIMEOUT   3
#define motABORTED   4
#define motREDEFINED 5
    
#define motSTANDING_STRING  "Standing"
#define motMOVING_STRING    "Moving"
#define motTIMEOUT_STRING   "Timeout"
#define motABORTED_STRING   "Aborted"
#define motREDEFINED_STRING "Redefined"
    
/* initialization status */
#define motNOT_INIT    1
#define motSW_INIT     2
#define motHW_INIT_RUN 3
#define motINIT_DONE   4
    
#define motNOT_INIT_STRING    "Not Initialized"
#define motSW_INIT_STRING     "S/W Initialized"
#define motHW_INIT_RUN_STRING "H/W Initialization Running"
#define motINIT_DONE_STRING   "Initialization Done"
    
/* conversion methods types */
#define motEXTERN        1
#define motLINEAR        2
#define motINTERPOLATION 3

/* types for user function */ 
#define motUSER_INIT 0
#define motUSER_CONV 1

/* unit types */
#define motUNIT_POS    0
#define motUNIT_SPD    1
#define motUNIT_CUR    2
#define motUNIT_REF    3

/* board units */
#define motBOARD_UNIT_POS    "Enc"
#define motBOARD_UNIT_SPD    "Enc/ms"
#define motBOARD_UNIT_CUR    "Dig"
#define motBOARD_UNIT_REF    "Ref"

/* motor types */
#define motDC_MOTOR      1
#define motSTEPPER_MOTOR 2

/* HW initialization actions */
#define motNO_ACTION               motNONE
#define motACTION_END              1
#define motACTION_MOVE             2
#define motACTION_SET_LOW_SW_LIMIT 3
#define motACTION_SET_UP_SW_LIMIT  4
#define motACTION_SET_ENC_VALUE    5
#define motACTION_CLAMP_BRAKE      6
#define motACTION_DISCONNECT       7
#define motACTION_USERFUNCTION     8
#define motACTION_BOARD_PROCEDURE  9
#define motACTION_DELAY           10
#define motACTION_CONNECT         11
#define motACTION_UNCLAMP_BRAKE   12

/* encoder coding types */
#define motCODE_BINARY 1
#define motCODE_BCD    2
#define motCODE_GRAY   3

/* events handling types */
#define motNO_EVENT        motNONE
#define motEVENT_INTERRUPT 1
#define motEVENT_SIGNAL    2
#define motEVENT_SOFTWARE  3

/* digital IO signal types */
#define motNO_SIGNAL           -1
#define motSIGNAL_BRAKE_ACTION  0
#define motSIGNAL_BRAKE_STATUS  1
#define motSIGNAL_INTERLOCK     2
#define motSIGNAL_AMPL_HOME     3

#define motSIGNAL_BRAKE_ACTION_NAME  "brakeAction"
#define motSIGNAL_BRAKE_STATUS_NAME  "brakeStatus"
#define motSIGNAL_INTERLOCK_NAME     "interlock"
#define motSIGNAL_AMPL_HOME_NAME     "electrHome"

#define motSIGNAL_ATTRIBUTE_NAME     "value"

#define motSIGNAL_OFF                0
#define motSIGNAL_ACTIVE_LOW         1
#define motSIGNAL_ACTIVE_HIGH        2

/*
 * index of motor status database attributes for cai Access
 */
#define motDB_STATUS_OP_MODE           0
#define motDB_STATUS_MOTION_MODE       1
#define motDB_STATUS_MOTION_STATUS     2 
#define motDB_STATUS_MOTION_STEP       3
#define motDB_STATUS_INIT_STATUS       4
#define motDB_STATUS_INIT_STEP         5
#define motDB_STATUS_BRAKE_CLAMPED     6
#define motDB_STATUS_AXIS_ENABLED      7
#define motDB_STATUS_EVENT_MASK        8
#define motDB_STATUS_AMPL_STATUS       9
#define motDB_STATUS_POWER_STATUS     10
#define motDB_STATUS_OVER_CURRENT     11
#define motDB_STATUS_OVER_TEMPERATURE 12
#define motDB_STATUS_MCON_STATUS      13
#define motDB_STATUS_DRIVE_FAULT      14
#define motDB_STATUS_EMERGENCY_STOP   15
#define motDB_STATUS_POS_ERROR        16
#define motDB_STATUS_IN_POS           17
#define motDB_STATUS_ON_LIMIT         18
#define motDB_STATUS_AMPL_ON_LIMIT    19
#define motDB_STATUS_AMPL_ON_HOME     20
#define motDB_STATUS_MCON_ON_LIMIT    21
#define motDB_STATUS_INTERLOCK        22
#define motDB_STATUS_SPEED            23
#define motDB_STATUS_SPEED_UNIT       24
#define motDB_STATUS_CURRENT          25
#define motDB_STATUS_CURRENT_UNIT     26
#define motDB_STATUS_POS_ENC          27
#define motDB_STATUS_POS_USER         28
#define motDB_STATUS_POS_UNIT         29
#define motDB_STATUS_POS_NAME         30
#define motDB_STATUS_POS_SECTION      31
#define motDB_STATUS_POS_INDEX        32
#define motDB_STATUS_NEXT_POS_ENC     33
#define motDB_STATUS_NEXT_POS_USER    34
#define motDB_STATUS_NEXT_POS_NAME    35
#define motDB_STATUS_NEXT_POS_SECTION 36
#define motDB_STATUS_NEXT_POS_INDEX   37
#define motDB_STATUS_FOLLOWING_ERR    38
#define motDB_STATUS_TIME_BEG         39
#define motDB_STATUS_TIME_END         40

/*
 * index of motor configuration database attributes for cai Access
 */
#define motDB_CONF_MOTOR               0
#define motDB_CONF_BOARDS              1
#define motDB_CONF_EVENTS              2
#define motDB_CONF_SIGNALS             3
#define motDB_CONF_AXIS                4
#define motDB_CONF_ENCODER             5
#define motDB_CONF_OFFSETS             6
#define motDB_CONF_SPECIAL_POS         7
#define motDB_CONF_NAMED_POSITIONS     8
#define motDB_CONF_NAMED_SPEEDS        9
#define motDB_CONF_TWO_STEP_OFFSET    10
#define motDB_CONF_INDEX_PULSE_SPEED  11
#define motDB_CONF_DEFAULT_SPEED      12
#define motDB_INIT_ACTIONS            13
#define motDB_UNIT_DEFAULT            14
#define motDB_UNIT_METHODS            15

/*************************************************************************/
/* THE FOLLOWING MACROS REMAIN AVAILABLE BUT ARE NOT USED ANYMORE        */
/*  SINCE THE DIMENSION OF THE TABLES IS FREE                            */
/*************************************************************************/
/* literal for maximum number of named positions */
#define motMAX_NAMED_POSITION   15

/* literal for maximum number of named speeds */
#define motMAX_NAMED_SPEED      15

/* literal for maximum number of HW init actions */
#define motMAX_INIT_ACTION      12

/* literal for maximum number of conversion methods */
#define motMAX_CONV_METHOD      15

#endif /* if !MOT_DEFINES_H */

/*************************************************************************/

/*___oOo___*/
