/******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: msg.h,v 2.41 2005/07/28 04:07:01 vltsccm Exp $" 
*
* msg.h
*
* who         when      what
* ---------   --------  -------------------------------------------------------
* B.GILLI     29/04/93  Preliminary version
* B.GILLI     27/05/93  for PDR
* A.WALLANDER 21/07/93  error handling, prototypes
* A.WALLANDER 20/08/93  many changes and clarifications after CDR
* J.AYASSE    19/11/93  Declaration of msgStartMon and msgStopMon.
* B.GILLI     10/02/94  Added qsemu.h according to SPR 09/02/94 from Birger.
* B.GILLI     13/09/94  Added msgBODYMAXLEN SPR 940032
* B.GILLI     23/09/94  Added new equivalent Rtap message types.
* T.HERLIN    18/10/94  Added defines for checkflag
* T.HERLIN    21/10/94  Added msgTYPE_SIMREPLY
* B.GILLI     12/11/94  Added msgNO_CHECK
* B.GILLI     14/11/94  Modified union decl. for message structure.SPR 940032
* M.COMIN     21/11/94  Add include file to define error mnemonics
* B.GUSTAFSSON 29/11/94 Including specific CCS/LCC include files
* B.GUSTAFSSON 01/12/94 Added C++ support
* T.EBERT     30/01/95  Added new filter types msgREPLY_FOR_CMD and
*                       msgREPLY_FOR_CMDID;
*                       Changed data type of msgRECEIVEFILTER according
*                       new filtering features;
*                       Changed prototype of msgSendCommand() function
*                       (msgCMD command -> char *command)
* T.EBERT     08/08/95  Modified definition of msgMAXLEN for CCS_LIGHT on SUN
* T.EBERT     22/01/95  Added message type msgTYPE_BOOK_REQUEST
* T.EBERT     12/02/96  Added msgTYPE_NEXT_FREE/msgTYPE_SCAN_OFFSET
* T.EBERT     28/03/96  Added msgSOCK_DESC structure
*                       msgOpenConnection(), msgAcceptConnection(),
*                       msgSendData(), msgReadData()
* B.GILLI     18/08/98  Added msgExhaustQueue to be used only in ccsExit.
* B.GILLI     23/11/99  Cleaned warnings revealed by new gcc compiler.
* B.GILLI     10/01/00  modify msgExhaustQueue to removed unused param.
* B.GILLI     18/01/00  More warning cleaning: (void) for func without args
* B.GILLI     07/04/00  SPR20000001, sup. const for *errmsg in msgSendReply
* B.GILLI     25/04/00  removed ref to book, obsolete with 200011
* B.GILLI     28/06/01  msgMAXSIZE is only 4080 on Linux Redhat 6.2
* T.EBERT     06/09/01  msgMAXLEN reset to standard (8192) for LINUX
*                       (requires KERNEL modification on REDHAT)
*----------------------------------------------------------------------------*/
/*
 * <msg.h> - CCS/MESSAGE SYSTEM - Interface File
 */
#ifndef MSG_H
#define MSG_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * Header files
 */
#include "ccs.h"
  /* #include "book.h" */    

/* 
 * Constants 
 * max. size of a complete message
 */
#define msgMAXLEN            8192

/*
 *    CCS message types, mapped in the user domain of the Rtap range.
 */
#define msgTYPE_COMMAND       129    
#define msgTYPE_REPLY         130
#define msgTYPE_ERROR_REPLY   131
#define msgTYPE_REQUEUED      132
#define msgTYPE_SIMREPLY      133
/* #define msgTYPE_BOOK_REQUEST  134 */
#define msgTYPE_SCAN_OFFSET   139
#define msgTYPE_NEXT_FREE     150
/*
 *    CCS message types equivalent to existing Rtap types.
 *    list to be upgraded according to recognized needs.
 */
#define msgTYPE_TIMER          65
#define msgTYPE_EVENT          66
#define msgTYPE_OBITUARY       67
#define msgTYPE_ALARM          75
#define msgTYPE_ALARM_CONN     76

#define msgNO_TIMEOUT           0       /* wait forever for msg              */
#define msgNO_WAIT             -1       /* don't wait for msg                */

/*
 * definition of available filter types
 */
#define msgANY_MESSAGE          0       /* list of filter types              */
#define msgANY_REPLY            1
#define msgREPLIES_FROM         2
#define msgREPLY_FOR_CMD        3
#define msgREPLY_FOR_CMDID      4

/* 
 * CCS check flag types of msgCHECKFLAG bitmask
 */
#define msgNO_CHECK             0x00
  /* #define msgCHECK_PREBOOK        0x01 */
#define msgCHECK_CMD            0x02
#define msgCHECK_ALL            0xFF

/* 
 * Data types
 */
typedef char      msgCMD[ccsCMD_LEN+1]; /* Command name: UPPER CASE ALWAYS!  */
typedef vltUINT16 msgCMDID;             /* command identifier                */ 
typedef vltINT32  msgPRIORITY;          /* message priority                  */
typedef vltINT32  msgLENGTH;            /* message length                    */
typedef vltUINT8  msgCHECKFLAG;         /* 8 bits mask defines checks to be  */
                                        /* performed.                        */
typedef vltINT32  msgTIMEOUT;            
typedef vltUINT8  msgFILTERTYPE;
				
typedef struct { 
    ccsENVNAME    envname;
    ccsPROCNUM    procnum;      
    vltUINT8      reserved;
} msgPROCESSID;                         /* Identifies a unique process       */

typedef struct {
    msgFILTERTYPE accept;
    msgCMDID      commandId;            /* only if accept = msgREPLY_FOR_CMDID */
    msgCMD        command;              /* only if accept = msgREPLY_FOR_CMD   */
    msgPROCESSID  source;               /* Meaningful only if accept         */
                                        /* = msgREPLIES_FROM                 */
} msgRECEIVEFILTER;

typedef struct {
    msgPRIORITY  priority;              /* Internal use only                 */
    msgPROCESSID source;                /* Sender of the message             */
    msgPROCESSID dest;                  /* Recipient                         */
    vltINT32     sourceUId;             /* User Id                           */
    msgCMDID     msgId;                 /* To distinguish identical commands */
    vltUINT8     msgType;               /* Defined by message system         */
    vltUINT8     responseFlag;          /* Command/Reply                     */
    vltINT32     typeHdrSize;    
    vltINT32     msgBodySize;
} msgHEADER;                            /* 1st part of the message header    */
                                        /* (fixed)                           */ 

/*
 * Depending on msgType, a second header, having the size indicated in 
 * typeHdrSize follows. It has the following composition, according to      
 * msgType :                                              
 */

/* 
 *  msgType = msgTYPE_COMMAND  :  Normal commands
 */
 
typedef struct {
/*    bookUCODE      bookingId; */      /* Added by message system when      */
                                        /* sending. Obsolete with 200011     */
    vltUINT32      unused;              /* Replacement fot bookingId         */
    msgCMD         command;             /* As provided by the user           */
    ccsTIMEVAL     timeStamp;           /* ~Unix timeval=8 bytes             */
    msgTIMEOUT     timeout;             /* Number of seconds, future         */
                                        /* extension                         */
} msgCOMMAND;

/*
 *  msgType = msgTYPE_REPLY  : Normal replies
 */
typedef struct {
    msgCMD         command;             /* The commandId is in header_1      */
    ccsTIMEVAL     timeStamp;           /* ~Unix timeval=8 bytes             */
    vltLOGICAL     lastReply;           /* Flag TRUE if it is last reply     */
} msgREPLY;

/*
 *  msgType = msgTYPE_ERROR_REPLY  : Error replies
 */
typedef struct {
    msgCMD         command;             /* The commandId is in header_1      */
    ccsTIMEVAL     timeStamp;           /* ~Unix timeval=8 bytes             */
} msgERROR_REPLY;

/*               
 *   Note: the message body shall transport the error context, ccsERROR
 */
/*
 *  msgType = msgTYPE_REQUEUED : Requeued message
 */
 
typedef struct {
    ccsTIMEVAL     timeStamp;           /* Time of requeuing                 */
} msgREQUEUED;

/*
 *  msgType = msgTYPE_OBITUARY : RTAP obituary message
 */
typedef struct {
    vltINT32       exitStatus;          /* how it terminated                 */
    msgPROCESSID   deceased;            /* process that died                 */
                                        /* deceased proc_num : positive if   */
                                        /* process obituary, 0 if environment*/
                                        /* obituary                          */
    ccsPROCNAME    procName;            /* process Name of the dead process  */
    vltINT8        restart;             /* will it be restarted              */
    vltINT8        debug;               /* did it register for debug         */
    vltUINT8       reserved[4];
} msgOBITUARY;

typedef union {
        msgCOMMAND      command;
        msgREPLY        reply;
        msgERROR_REPLY  errorReply;
        msgREQUEUED     requeued;
        msgOBITUARY     obi;
    } msgHEADER2;                /* Type dependant header   */


#define msgBODYMAXLEN  (msgMAXLEN - sizeof(msgHEADER) - sizeof(msgHEADER2) - 1) 

typedef struct {                        /* The complete ccs message          */
    msgHEADER  msgHeader;
    msgHEADER2 type;
    char       body [msgMAXLEN - sizeof(msgHEADER) - sizeof(msgHEADER2)];
} msgMSG;

/* structures and defines for socket connection */
#define msgSOCK_SIZE     32768
#define msgCONNECT_CMD   "MSGOCON" 

/*
 * REMARK: MAXHOSTNAMELEN should be used instead of 256, but
 *         for SUN/HP and VxWorks this literal is defined in 
 *         different header files
 */ 
typedef struct {
    char         hostname[256];
    int          ipAddr;        
    msgPROCESSID procId;         
    int          fd;
} msgSOCK_DESC;

/* 
 * Functions prototypes 
 */
ccsCOMPL_STAT msgSendCommand(       /* Used to send command messages    */
    const char        *command,     /* Command to be sent               */
    msgCMDID           commandId,   /* User supplied identifier or      */
                                    /* defaulted                        */
    const ccsENVNAME   destenv,     /* Where to send the command        */
    const ccsPROCNAME  destproc,    /* To whom : process name           */
    const char        *buffer,      /* Buffer with command parameters   */
    msgLENGTH          buflen,      /* Buffer Length : max len = 8 kB   */
    msgCHECKFLAG       flag,        /* Flag specifying checks to be done*/
    ccsERROR          *error       /* Returned error structure         */
    );

ccsCOMPL_STAT msgSendReply(         /* Used to send one reply to a given*/
                                    /* command. Most of the arguments   */
                                    /* are found in the header of the   */
                                    /* command using the msgParseMsg    */
                                    /* function.                        */
    const msgPROCESSID *destId,     /* Destination of the reply, as     */
                                    /* found in the origin field of the */
                                    /* command                          */
    msgCMDID            commandId,  /* As found in the command          */
    const msgCMD        command,    /* As found in the command          */
    const char         *buffer,     /* Buffer containing the reply      */
    msgLENGTH           buflen,     /* Buffer length. Max len = 8kB.    */
    vltLOGICAL          lastReply,  /* Flag to indicate if another      */
                                    /* reply follows                    */
    ccsERROR           *errmsg,     /* Error structure returned in msg  */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT msgRequeueMsg(        /* Put a message already received   */
                                    /* back into the input queue. A new */
                                    /* header is added.                 */
     const msgHEADER   *message,    /* Message to be requeued           */
     ccsERROR          *error       /* Returned error structure         */
     );

ccsCOMPL_STAT msgParseMsg(          /* Used to parse a received message */
    const msgHEADER    *message,    /* Pointer to message to parse      */
    vltUINT8           *msgType,    /* Message type                     */
    msgCMD            **command,    /* Pointer to command               */
    msgCMDID           *commandId,  /* CommandId                        */
    msgPROCESSID      **orgId,      /* Pointer to originator            */
    vltLOGICAL         *lastReply,  /* Last reply flag (only for        */
                                    /* msgREPLY message type)           */
    char              **buffer,     /* Pointer to buffer                */
    msgLENGTH          *buflen,     /* Length of buffer                 */
    ccsERROR           *errmsg,     /* Error structure in message       */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT msgScheduleProcess (   /* Schedule a process              */
    const ccsENVNAME    envName,     /* Where to schedule               */
    const char         *runString,   /* Shell run string                */
    const ccsRUNOPT    *runOptions,  /* VxWorks run-time options        */
    ccsERROR           *error        /* Returned error structure        */
    );

ccsCOMPL_STAT msgRecvMsg  (            /* Receive a message                  */
    const msgRECEIVEFILTER  *filter,   /* Provides filtering information     */
    msgTIMEOUT               timeout,  /* Max. waiting time in milliseconds  */
    msgHEADER              **message,  /* pointer to the message             */
    ccsERROR                *error     /* Returned error structure           */
    );

ccsCOMPL_STAT msgEnProcessMon(      /* Enable monitoring of messages    */
                                    /* to/from a given process          */
    const ccsPROCNAME  procName,    /* Process name                     */
    vltLOGICAL         sendfl,      /* TRUE = monitor sent messages     */
    vltLOGICAL         receive,     /* TRUE = monitor received messages */
    ccsERROR          *error        /* Returned error structure         */
    );

ccsCOMPL_STAT msgDisProcessMon(     /* Disable monitoring of messages   */
                                    /* to/from a given process          */
    const ccsPROCNAME  procName,    /* Process name                     */
    ccsERROR          *error        /* Returned error structure         */
    );

ccsCOMPL_STAT msgEnGlobalMon (         /* Enable global message monitoring   */
    ccsERROR         *error            /* Returned error structure           */
    );
    
ccsCOMPL_STAT msgDisGlobalMon (        /* Disable global message monitoring  */
    ccsERROR         *error            /* Returned error structure           */
    );

ccsCOMPL_STAT msgStartMon (            /* Start monitoring of messages       */
    msgCMDID          cmdId,           /* Command Id for the reply           */
    const msgCMD      cmdName,         /* Command Name for the reply         */
    ccsERROR         *error            /* Returned error structure           */
    );

ccsCOMPL_STAT msgStopMon (             /* Stop monitoring of messages        */
    ccsERROR         *error            /* Returned error structure           */
    );

ccsCOMPL_STAT msgSetFilter(	    /* Setup a receive filter		*/
    const ccsENVNAME   envname,	    /* Source environment name		*/
    const ccsPROCNAME  procName,    /* Source process name		*/
    msgFILTERTYPE      filterType,  /* Filter type			*/
    msgCMDID           commandId,   /* command id of requested reply            */
    msgCMD             command,     /* command corresponding to requested reply */
    msgRECEIVEFILTER  *filter,      /* Provides filtering information   */
    ccsERROR          *error        /* Returned error structure         */
    );

int msgGetPidByName (
                    ccsPROCNAME  procName,  /* process name */
                    ccsERROR    *error      /* error structure */
                    );

ccsCOMPL_STAT msgAcceptConnection(msgPROCESSID *orgId,
                                  char         *buffer,
                                  msgSOCK_DESC *sockDesc,
                                  ccsERROR     *error
                                  );

ccsCOMPL_STAT msgOpenConnection(char         *hostname, 
                                ccsENVNAME    envName, 
                                ccsPROCNAME   procName,
                                msgTIMEOUT    timeout,
                                msgSOCK_DESC *sockDesc,
                                ccsERROR     *error
                                );

ccsCOMPL_STAT msgRecvData (msgSOCK_DESC *sockDesc,
                           msgTIMEOUT    timeout,
                           char        **buffer,
                           vltUINT32     bytesToRead,
                           vltUINT32    *bytesRead,
                           vltINT16     *errorCode,
                           ccsERROR     *error
                           );

ccsCOMPL_STAT msgSendData (msgSOCK_DESC *sockDesc,
                           char         *buffer,
                           vltUINT32     buflen,
                           vltINT16      errorCode,
                           ccsERROR     *error
                           );

ccsCOMPL_STAT msgExhaustQueue (void);

#ifdef MAKE_VXWORKS
#include "msgLCC.h"
#else
#include "msgCCS.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /*!MSG_H*/
