#ifndef MSGPRIVATE_H
#define MSGPRIVATE_H
/*************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: msgPrivate.h,v 2.41 2005/07/28 04:07:01 vltsccm Exp $" 
*
* msgPrivate.h
*
* who         when      what
* ---------   --------  ----------------------------------------------
* A.Wallander 14.7.93   First version
* T.Ebert     11/01/95  Added msgMSG_LIST data structure 
* T.Ebert     19/01/95  Added definitions for handling of runstring
*                       in msgScheduleProcess()
*                       Reorganized header files
* T.Ebert     26/05/95  Modified for CCS Light
* T.Ebert     10/11/95  Added msgInternalReply()
* T.Ebert     08/03/96  Added msgGetSourceUid()
* T.Ebert     28/03/96  Added definition MSGOCON - command
* T.Ebert     22/04/96  Added msgGetMsgPtr()
* T.Ebert     20/05/96  Added msgSemGive()
* rabuter     13/01/97  Add a timeout for msgRecvMsg , to wait for reply on MSGSCHP 
*                       command
* bgilli      07/01/00  Removed unused flag from msgAddEntry proto
*/
/************************************************************************
*  msgPrivate.h - Private include file for msg system
*                 Defines constants and prototypes for private routines
*               
*------------------------------------------------------------------------
*/
/*
 * System Headers
 */
#include <stdlib.h>
#include <stdlib.h>           
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>

#include "ccs.h"
#include "ccsRtap.h"
#include "msg.h"
#include "cmd.h"
#include "log.h"
#include "err.h"
#include "tims.h"

/*
 * defines
 */
#define msgSERVER1         "msgServer1"
#define msgCMD_MANAGER     "cmdManager"

#define msgMINPRIO         rtMINPRIO    /* minimum priority for msg*/
#define msgRESPOND         rtRESPOND    /* acknowledgement desired */
#define msgNO_RESPOND      rtNO_RESPOND /* don't acknowledge       */
#define msgRESPONSE        rtRESPONSE   /* reply or ack message    */

/*Timeout for msgScheduleProcess*/
#define msgSCHEDULE_TIMEOUT 9000  /* milliseconds to wait for reply on a MSGSCHP command*/

/*
 * data structure for linked list of messages
 * NOTE: <msg> MUST be the last element in this structure
 */
typedef struct msgMSG_LIST
    {
    struct msgMSG_LIST *previous; /* pointer to element before in list */
    struct msgMSG_LIST *next;     /* pointer to next element in list   */
    int                 size; 
    vltINT32            flag;     /* flag -> only for CCS_LITE         */
    char                msg;      /* start of message buffer           */
    } msgMSG_LIST;
    
/*
 * functions prototypes
 */
void msgSetAutoLog (                        /* To turn auto log on or off */
                   const vltLOGICAL status
                   );
void msgLogMessage (                        /* to produce log of a message */
                   msgHEADER  *msg,         /* message to be logged        */
                   char       *operation    /* "sent" or "received"        */
                   );

ccsCOMPL_STAT msgReplyTo (
                         msgHEADER    *message,    /* message to reply to */
                         char         *buffer,     /* same as msgSendReply */
                         msgLENGTH     buflen,     /* for other args.     */
                         vltLOGICAL    lastreply,
                         ccsERROR     *errmsg,
                         ccsERROR     *error
                         );

ccsCOMPL_STAT msgInternalReply (msgHEADER *msgHeader,
                                char      *buffer,
                                msgLENGTH  buflen,
                                ccsERROR  *errmsg,
                                ccsERROR  *error);

ccsCOMPL_STAT msgGetSourceUid (
                              msgHEADER   *msg,
                              vltINT32    *sourceUid,
                              ccsERROR    *error
                              );

/*
 * functions to handle internal queue
 */
ccsCOMPL_STAT  msgAddEntry      (msgMSG *msg, ccsERROR *error);
msgMSG        *msgFindEntry     (const msgRECEIVEFILTER *filter);
void           msgDeleteEntry   (void);
vltLOGICAL     msgCheckForEntry (void);

/*
 * misc
 */
msgMSG *msgGetMsgPtr(void);

#endif /*!MSGPRIVATE_H*/

