#ifndef mswDefines_H
#define mswDefines_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: mswDefines.h,v 1.126 2005/09/26 14:24:02 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* awalland  25/06/99  Added define for thermal module
* pduhoux   02/11/98  Added macros mswMODNAME_xxx for the modules' names of MSW table
* pduhoux   28/10/98  Added macro mswONFOCUS_STRING for UserField selection
* pduhoux   23/10/98  Added macro mswINITCHFOC_STRING for UserField selection
* gchiozzi  13/02/98  Increased mswNO_OF_MODULES from 32 to 64.
* gchiozzi  13/11/97  Changed VLT default focus from NA to CA.
* gchiozzi  02/09/97  Removed definition of TARGET_TCS
* rkarban   15/03/97  remove mswSUB_STATE(_STR)_TRACKING
* gchiozzi  17/02/97  Added new defines for commands GETINS, GETINSD, SELINS, SETINSD
* gchiozzi  11/02/97  Added new defines for instrument data
* gchiozzi  10/01/97  Added define for m3 position attribute
* nttmgr    16/12/96  Renamed AO commands GETPERM and RELPERM
* nttmgr    06/12/96  Changed name of command to M3: now MOVFOCU
* gchiozzi  31/10/96  Removed DEC95 sections
* gchiozzi  31/10/96  New define mswDB_FORCE_SETTING
* gchiozzi  30/10/96  Added defines for PERMSET and RELSET cmds
* gchiozzi  05/07/96  Changed to C style comments
* gchiozzi  10/05/96  Porting to JUN96 (activated commented sections)
* mverola   03/04/96  Added focus names and codes.
*                     DB point names now defined here.
* gchiozzi  07/09/95  Changed state ids to have them unique.
* awalland  14/06/95  created
*/

/************************************************************************
 *
 * This file contains common defines used both by C/C++ files and 
 * dbLoader files
 *----------------------------------------------------------------------
 */

/* 
 *  Defines for the target telescope.
 *  The default target is VLT
 *  Up to now only VLT and NTT
 */
#define VLT_TCS   1
#define NTT_TCS   2

#include "tcsDefines.h"

/*
 * Defines for states values and names
 * These enum and string vector are used in the whole module
 * to identify states and substates.
 * The values must be coordinated with the standard states defined in 
 * evhStates.h
 * LCU states must have the same enum
 */

#include "evhStates.h"
#include "evhSTD_COMMANDS_NAMES.h"

/* Module name */
#define mswMOD "msw"

/* Module Names */
#define mswMODNAME_ACTWS        "actws"
#define mswMODNAME_AGWS         "agws"
#define mswMODNAME_FSWS         "agws"
#define mswMODNAME_CHOPWS       "chopws"
#define mswMODNAME_M1           "m1"
#define mswMODNAME_M2           "m2"
#define mswMODNAME_M2TC         "m2tc"
#define mswMODNAME_M3           "m3"
#define mswMODNAME_M4           "m4"
#define mswMODNAME_LADC         "ladc"
#define mswMODNAME_ALTAXIS      "altAxis"
#define mswMODNAME_AZAXIS       "azAxis"
#define mswMODNAME_TRKWS        "trkws"
#define mswMODNAME_TRKALT       "trkalt"
#define mswMODNAME_TRKAZ        "trkaz"
#define mswMODNAME_TCSMON       "tcsmon"
#define mswMODNAME_ECSWS        "ecsws"
#define mswMODNAME_PRS          "prs"
#define mswMODNAME_HB           "hb"
#define mswMODNAME_ENCL         "encl"
#define mswMODNAME_ADAPTER      "adapter"
#define mswMODNAME_TRKROT       "trkrot"
#define mswMODNAME_TCCD_AG      "tccd_ag"
#define mswMODNAME_ACT          "act"
#define mswMODNAME_AG           "ag"
#define mswMODNAME_ADREF        "adref"
#define mswMODNAME_THERMAL      "thermal"
#define mswMODNAME_SPARE        "spare"

/* Local remapping of standard states (just for convenience) */
#define mswSTATE_UNK             evhSTATE_UNK
#define mswSTATE_OFF             evhSTATE_OFF
#define mswSTATE_LOADED          evhSTATE_LOADED  /* Not used by msw */
#define mswSTATE_STANDBY         evhSTATE_STANDBY
#define mswSTATE_ONLINE          evhSTATE_ONLINE
#define mswSTATE_ERROR           evhSTATE_ERROR

/* msw substate definitions */
#define mswSUB_STATE_UNK            mswSTATE_UNK   /*MVE added */
#define mswSUB_STATE_ERROR          19
#define mswSUB_STATE_STEADY         20  /*MVE not used anymore */
#define mswSUB_STATE_IDLE           21  
#define mswSUB_STATE_COLDSTARTING   22
#define mswSUB_STATE_SHUTTINGDOWN   23
#define mswSUB_STATE_PAUSED         24
#define mswSUB_STATE_PAUSING        25
#define mswSUB_STATE_STARTING       26
#define mswSUB_STATE_STOPPING       27
#define mswSUB_STATE_PRESETTING     29
#define mswSUB_STATE_CHANGINGFOCUS  30
#define mswSUB_STATE_ALIGNING       31

/* Defines for name states */
#define mswSTATE_STR_UNK            evhSTATE_STR_UNK 
#define mswSTATE_STR_OFF            evhSTATE_STR_OFF 
#define mswSTATE_STR_LOADED         evhSTATE_STR_LOADED /* not used by msw */
#define mswSTATE_STR_STANDBY        evhSTATE_STR_STANDBY 
#define mswSTATE_STR_ONLINE         evhSTATE_STR_ONLINE 

#define mswSUB_STATE_STR_ERROR          "ERROR"
#define mswSUB_STATE_STR_STEADY         "STEADY"  /*MVE not used anymore */
#define mswSUB_STATE_STR_IDLE           "IDLE"
#define mswSUB_STATE_STR_COLDSTARTING   "COLD STARTING"
#define mswSUB_STATE_STR_SHUTTINGDOWN   "SHUTTING DOWN"
#define mswSUB_STATE_STR_PAUSED         "PAUSED"
#define mswSUB_STATE_STR_PAUSING        "PAUSING"
#define mswSUB_STATE_STR_STARTING       "STARTING"
#define mswSUB_STATE_STR_STOPPING       "STOPPING"
#define mswSUB_STATE_STR_PRESETTING     "PRESETTING"
#define mswSUB_STATE_STR_CHANGINGFOCUS  "CHANGING FOCUS"
#define mswSUB_STATE_STR_ALIGNING       "ALIGNING"

/*
 * Defines for command names
 */

/* User commands */
#define mswCSTART_CMD         "CSTART"
#define mswSTART_CMD          "START"
#define mswSHDOWN_CMD         "SHDOWN"
#define mswPAUSE_CMD          "PAUSE"
#define mswCHFOCUS_CMD        "CHFOCUS"
#define mswGETINS_CMD         "GETINS"
#define mswGETINSD_CMD        "GETINSD"
#define mswSELINS_CMD         "SELINS"
#define mswSETINSD_CMD        "SETINSD"

/* From prs module */
#define mswPRESET_CMD         "PRESET"
#define mswEPRESET_CMD        "EPRESET"

/* From iss module */
#define mswALIGN_CMD          "ALIGN"
#define mswEALIGN_CMD         "EALIGN"

/* From active optics module */
#define mswGETPERM_CMD        "GETPERM"
#define mswRELPERM_CMD        "RELPERM"

/* Overloaded standard commands */
#define mswVERSION_CMD        evhVERSION_CMD
#define mswSTATUS_CMD         evhSTATUS_CMD
#define mswOFF_CMD            evhOFF_CMD
#define mswSTANDBY_CMD        evhSTANDBY_CMD
#define mswONLINE_CMD         evhONLINE_CMD
#define mswSTOP_CMD           evhSTOP_CMD
#define mswINIT_CMD           evhINIT_CMD

/* Commands to "broadcast" */
#define mswINIT_ALL_CMD       mswINIT_CMD 
#define mswSTANDBY_ALL_CMD    mswSTANDBY_CMD 
#define mswONLINE_ALL_CMD     mswONLINE_CMD
#define mswOFF_ALL_CMD        mswOFF_CMD
#define mswSTOP_ALL_CMD       mswSTOP_CMD

/* maximum number of modules in module table */
#define mswNO_OF_MODULES       64

/* strings to match in userField of module table */
#define mswGUIDING_STRING       "GUIDING"
#define mswONFOCUS_STRING       "ONFOCUS"
#define mswINITCHFOC_STRING     "INITCHFOC"

/* special values of table state to trigger evaluation */
/*MVE In msw they are not used! Who needs them? */
#define mswTABLE_CHANGED       98
#define mswSTATE_CHANGED       99

/*
 * Defines for instrument configuration handling
 */
#define mswNO_OF_INSTRUMENTS   24

/* Database symbolic addresses */
/* --------------------------- */

/* Module entry point (path to append after mainHandler.DbRoot()) */
#define mswDB_ENTRY_POINT        "msw"

/* Relative to the mswDB_ENTRY_POINT */
#define mswDB_SUBSTATE_ATTR      "substate"
#define mswDB_READY_TIMEOUT_ATTR "readyTimeout"
#define mswDB_STOP_TIMEOUT_ATTR  "stopTimeout"
#define mswDB_LIST_POINT         "list"
#define mswDB_INS_DATA_POINT     "insData"
#define mswDB_FOCUS_POINT        "foc"
#define mswDB_FORCE_SETTING      "forceSetting"

/* Relative to mswDB_LIST_POINT */
#define mswDB_MODULES_ATTR       "modules"
#define mswDB_LIST_CHANGED_ATTR  "listchanged"
#define mswDB_UNIT_CHANGED_ATTR  "unitchanged"

/* Relative to mswDB_INS_DATA_POINT */
#define mswDB_INS_ID_ATTR         "selectedIns"
#define mswDB_INS_DATA_ATTR       "data"

/*MVE Min and max value to assess if altitude is within "ZENITH range" */
#define mswMIN_ZENITH           88.0
#define mswMAX_ZENITH           90.1
#define mswIS_ZENITH(alt)       (((alt)>=mswMIN_ZENITH)&&((alt)<=mswMAX_ZENITH))

/*MVE Defines for focus station numbers and names */
#define mswUNDEFINED_FOCUS      0
#define mswALL_FOCUS            1 /* Not meaningful to define the focus station    */
				  /* but compliant with the msw table focus codes  */
#define mswNASMITH_A_FOCUS      2
#define mswNASMITH_B_FOCUS      3
#define mswCASSEGRAIN_FOCUS     4
#define mswCOUDE_FOCUS          5
#define mswINTERMEDIATE_D_FOCUS 6

#define mswUNDEFINED_STR_FOCUS      "UNDEFINED"
#define mswALL_STR_FOCUS            "ALL"
#define mswNASMITH_A_STR_FOCUS      "NA"
#define mswNASMITH_B_STR_FOCUS      "NB"
#define mswCASSEGRAIN_STR_FOCUS     "CA"
#define mswCOUDE_STR_FOCUS          "CO"
#define mswINTERMEDIATE_D_STR_FOCUS "IM"

#endif /*!mswDefines_H*/
