#ifndef mswINS_DATA_RECORD_H
#define mswINS_DATA_RECORD_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: mswINS_DATA_RECORD.h,v 1.126 2005/09/26 14:24:05 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban  06/08/99  add virtual destructor
* gchiozzi  17/07/98  Added fields for detector reference pixel and adapter focus offset.
* gchiozzi  02/09/97  Modified table format.
* gchiozzi  11/02/97  Created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "msw.h"
#include "eccsErr.h"
#include "evhDB_CMD_SEND.h"
#include "fndARRAY.h"
#include "fndOBJECT.h"
#include "fndSTRING.h"
#include "eccsDB_TABLE.h"

// support classes mswINS_DATA_RECORD et al.
// -----------------------------------------------------
// These support class is used to hold and manipulate data from
// the instrument data description table

typedef vltBYTES32 mswINSID;
typedef vltBYTES32 mswINSMODE;

class  mswINS_DATA_RECORD: public eccsDB_TABLE_RECORD, public eccsERROR_CLASS
{
  public:
    mswINS_DATA_RECORD();
    mswINS_DATA_RECORD( const mswINS_DATA_RECORD& );
    mswINS_DATA_RECORD& operator           =( const mswINS_DATA_RECORD& );
    virtual ~mswINS_DATA_RECORD();

    // Pack() and Unpack() methods used while accessing the database table
    // to convert from RTAP internal format to C class members
    virtual ccsCOMPL_STAT Pack( void *buffer );
    virtual ccsCOMPL_STAT Unpack( void *buffer );

    // Methods to retrieve/set the instrument data configuration
    // in the format of a command message buffer
    virtual const char   *Data();
    virtual ccsCOMPL_STAT Data(const char *msgBuffer);

    // Empties the record
    virtual ccsCOMPL_STAT CleanUp();

    // Checks if a given mswINSID correspond to this record
    const vltLOGICAL IsInsId(const mswINSID &insId) const;

    // Data member read access methods
    const mswINSID   &InsId() const;
    const mswINSMODE &InsMode() const;
    const vltINT32    FocusStation() const;
    const char       *FocusStationName() const;
    const vltDOUBLE   RotOffset() const;
    const vltDOUBLE   FocusOffset() const;
    const vltDOUBLE   AdFocusPos() const;

    const vltDOUBLE   PixelSizeX() const;
    const vltDOUBLE   PixelSizeY() const;
    const vltDOUBLE   RefX() const;
    const vltDOUBLE   RefY() const;

    const vltDOUBLE   VigAreaSizeX() const;
    const vltDOUBLE   VigAreaSizeY() const;
    const vltDOUBLE   VigAreaCntX() const;
    const vltDOUBLE   VigAreaCntY() const;

    const vltDOUBLE   PointAxisOffX() const;
    const vltDOUBLE   PointAxisOffY() const;

    const char       *DetectorLCU() const;

  private:
    mswINSID    insId;           // identifier for a pair (ins,detector)
    mswINSID    insMode;         // identifier for instrument mode
    vltINT32    focusStation;    // instrument focus station (defines in mswDefines.h)

    vltDOUBLE   rotOffset;       // rotator offset (deg)
    vltDOUBLE   focusOffset;     // optimal focus offset (mm)
    vltDOUBLE   adFocusPos;      // optimal adapter focus Pos (mm)

    vltDOUBLE   pixelSizeX;      // detector pixel size in X (arcsec/pixel)
    vltDOUBLE   pixelSizeY;      // detector pixel size in Y (arcsec/pixel)
    vltDOUBLE   refX;            // detector reference pixel X (pixel)
    vltDOUBLE   refY;            // detector reference pixel Y (pixel)

    vltDOUBLE   vigAreaSizeX;    // vignetting area: X size (arcsec)
    vltDOUBLE   vigAreaSizeY;    // vignetting area: Y size (arcsec)
    vltDOUBLE   vigAreaCntX;     // vignetting area: X offset of center 
                                 // with respect to center (arcsec)
    vltDOUBLE   vigAreaCntY;     // vignetting area: Y offset of center
                                 // with respect to center (arcsec)

    vltDOUBLE   pointAxisOffX;   // X pointing axis offset (arcsec)
    vltDOUBLE   pointAxisOffY;   // Y pointing axis offset (arcsec)

    vltBYTES256 detectorLCU;     // environment and process name

    fndSTRING cmdString;
};


// include inline code for support classes
#include "mswINS_DATA_RECORD.icc"

#endif /*!mswINS_DATA_RECORD_H*/
