#ifndef mswMOD_LIST_H
#define mswMOD_LIST_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: mswMOD_LIST.h,v 1.126 2005/09/26 14:24:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rabuter 2000-07-12 Added method UpdateTrkModList
* gchiozzi  13/02/98  BUG: modTable was sized with hard coded 32 instead of mswNO_OF_MODULES
* gchiozzi  04/04/97  Removed _CACHE subclasses
* gchiozzi  24/05/96  Protected selectList and added access method
* awalland  23/05/96  Added methods SelectGuidingModules, ParseUserField
* awalland  22/05/96  Replaced GetAddrList with Select*List
* mverola   03/04/96  Added StoreTable, UpdateReqModList
*                     Changed GetStateList into GetAddrList
* gchiozzi  30/08/95  Removed many not useful parameters
* awalland  04/08/95  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "msw.h"
#include "eccsErr.h"
#include "evhDB_CMD_SEND.h"
#include "fndARRAY.h"
#include "fndOBJECT.h"

// support classes mswMOD_RECORD and mswMOD_RECORD_ARRAY
// -----------------------------------------------------
// These support classes are used to hold and manipulate data from
// the module table

// structure to hold a record of module configuration
typedef struct 
{
    vltBYTES32  modName;           // identifier for module
    vltBYTES32  modId;             // destination process
    vltBYTES256 modSymbAddr;       // state attribute in database
    vltINT8     startPhase;        // start phase for the module, 0..n
    vltINT8     stopPhase;         // stop phase for the module, 0..n
    vltINT8     tracking;          // true if module has to receive tracking commands
    vltINT8     focus;             // module used for focus x
    vltLOGICAL  required;          // module currently required in operation
    vltLOGICAL  ignore;            // ignore this module (maintenance operation)
    vltBYTES256 userField;         // free field to be used for private filtering
} mswMOD_DB_RECORD;
    

class  mswMOD_RECORD: public fndOBJECT, public mswMOD_DB_RECORD
{
  public:
    mswMOD_RECORD();
    mswMOD_RECORD( const mswMOD_DB_RECORD& );
    mswMOD_RECORD& operator           =( const mswMOD_DB_RECORD& );

    vltLOGICAL   IsMod(const vltBYTES32  modName) const;

    const char  *ModName()     const;
    const char  *ModId()       const;
    const char  *ModSymbAddr() const;
    const char  *ModBaseAddr() const;
    vltINT8      StartPhase()  const;
    vltINT8      StopPhase()   const;
    vltINT8      Tracking()    const;
    vltINT8      Focus()       const;
    const char  *FocusName()   const;
    vltLOGICAL   Required()    const;
    vltLOGICAL   Ignore()      const;
    const char  *UserField()   const;
    
  private:
    vltBYTES256 modBaseAddr;       // Base root point for the unit
                                   // assumed to be the parent point for the
                                   // state attribute
};


class  mswMOD_RECORD_ARRAY:  public fndARRAY
{
  public:
    mswMOD_RECORD_ARRAY();
    mswMOD_RECORD& operator []( int atIndex ) const; 
    mswMOD_RECORD& operator []( const vltBYTES32 modName) const; 

};


class mswMOD_LIST : public eccsERROR_CLASS
{
  public:
    mswMOD_LIST(const dbSYMADDRESS dbPoint = dbEMPTY);
    virtual ~mswMOD_LIST();

    // SelectList returns the list after selection
    const mswMOD_RECORD_ARRAY   &SelectList() const; 

    // Get<x>() methods read everytime from DB table

    virtual ccsCOMPL_STAT  SelectRequiredModules();
    virtual ccsCOMPL_STAT  SelectTrackingModules();
    virtual ccsCOMPL_STAT  SelectGuidingModules();
    virtual ccsCOMPL_STAT  SelectOnFocusModules(vltINT8 focus);
    virtual ccsCOMPL_STAT  SelectInitChFocModules();
    
    // Loads the table from DB into the modTable vector
    virtual ccsCOMPL_STAT  LoadTable();


    ccsCOMPL_STAT  DbPoint(const dbSYMADDRESS point);
    
    ccsCOMPL_STAT  UpdateReqModList(vltINT8 oldFocus, vltINT8 newFocus);
    ccsCOMPL_STAT  UpdateTrkModList(vltINT8 index, vltINT8 istracking );
  protected:
    dbSYMADDRESS    modPoint;

    // selectList contains the list after selection
    mswMOD_RECORD            modTable[mswNO_OF_MODULES];
    mswMOD_RECORD_ARRAY      selectList; 
    
    vltLOGICAL      ParseUserField(const char *matchString, 
                                   const char *userField);
    ccsCOMPL_STAT  _SelectRequiredModules();
    ccsCOMPL_STAT  _SelectUsrFieldModules(char *usrField);

    ccsCOMPL_STAT  StoreTable();
    
  private:
    vltINT8 focus;
};

// include inline code for support classes
#include "mswMOD_LIST.icc"

#endif /*!mswMOD_LIST_H*/
