#ifndef PCO_H
#define PCO_H
/******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: pco.h,v 1.32 2005/06/03 13:45:50 vltsccm Exp $" 
*
* pco.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  18/08/94  Created 1. version
*/

/************************************************************************
*  Header file for the Protocol Converter - PCO
*
*----------------------------------------------------------------------
*/
#ifdef __cplusplus
extern "C" {
#endif

#include "ccs.h"
#include "msg.h"
#include "evt.h"
#include "pcoMidas.h"

/* Definition of constants
 */
#define pcoNOT_USED              "-"
#define pcoUNKNOWN_PROC          "unknown"
#define pcoMODULE_ID             "pco"
#define pcoQUIT                  "pCoQuIt"
#define pcoMAX_CCS_BODY          msgBODYMAXLEN
#define pcoMAX_MSG_LEN           (pcoMAX_CCS_BODY - 1024)
#define pcoPCO_PROC_NAME         "pco" 
#define pcoRTAP_ENV_VAR          "RTAPENV"
#define pcoQUIT_NAME             "pcoQuit"
#define pcoPING                  "pCoPiNg"
#define pcoLOCAL_ENV             "\0"
#define pcoMAX_VAL_MSG_COUNT     65000
#define pcoMAX_FAIL_EXT_PROC     5
#define pcoGEN_KEYW_LEN          32
#define pcoPROTOCOL_ID_LEN       32
#define pcoPROCESS_ID_LEN        32
#define pcoHOST_NAME_LEN         32
#define pcoMIDAS_KEY_NAME_LEN    64
#define pcoATTRIB_NAME_LEN       512
#define pcoINTEGER_STR_LEN       10
#define pcoREAL_STR_LEN          20
#define pcoDOUBLE_STR_LEN        32

/* PCO commands as strings
 */
#define pcoATTACH_COM_STR     "ATTACH"
#define pcoDISCON_COM_STR     "DISCON"
#define pcoDETACH_COM_STR     "DETACH"
#define pcoEXIT_COM_STR       "EXIT"
#define pcoFORWARD_COM_STR    "FORWMSG"
#define pcoNO_COMMAND_STR     "NOCOM"
#define pcoPING_COM_STR       "PING"
#define pcoREAD_KEY_COM_STR   "READKEY"
#define pcoWRITE_KEY_COM_STR  "WRITKEY"

/* Protocols as strings
 */
#define pcoUNKNOWN_PROTOCOL   "UNKNOWN"
#define pcoMIDAS_PROTOCOL     "MIDAS"
#define pcoARCHIVE_PROTOCOL   "ARCHIVE"

/* MIDAS types as strings
 */
#define pcoMIDAS_INTEGER_STR  "INTEGER"
#define pcoMIDAS_REAL_STR     "REAL"
#define pcoMIDAS_DOUBLE_STR   "DOUBLE"
#define pcoMIDAS_STRING_STR   "STRING"

/* Macro definitions
 */
#define INFO(info) if (verbose) printf("\npco> %s", info)

/* Type definitions
 */
typedef char       pcoMSG[pcoMAX_MSG_LEN];
typedef vltUINT32  pcoMSG_LEN;

/* Enumerated type definitions
 */
typedef enum {
    pcoUNKNOWN = 1,
    pcoMIDAS,
    pcoARCHIVE
} pcoPROTOCOL;

typedef enum {
    pcoNO_CONNECTION = 1,
    pcoCONNECTION
} pcoCON_STAT; 

typedef enum {
    pcoCOM_NOT_DEF = 1,
    pcoUNKNOWN_COM,
    pcoPING_COM,
    pcoFORWARD_MSG_COM,
    pcoREAD_KEY_COM,
    pcoWRITE_KEY_COM,
    pcoATTACH_COM,
    pcoDETACH_COM,
    pcoEXIT_COM,
    pcoDISCON_COM
} pcoCOMMAND_TYPE;

typedef enum {
    pcoMIDAS_UNKNOWN = 1,
    pcoMIDAS_INTEGER,
    pcoMIDAS_REAL,
    pcoMIDAS_DOUBLE,
    pcoMIDAS_STRING
} pcoMIDAS_KEY_TYPE;


/* Type definitions of Structures
 */
typedef struct pcoPROC {
    struct pcoPROC     *next;
    struct pcoPROC     *prev;
    pcoCON_STAT        conStatus;
    pcoPROTOCOL        protocol;
    char               processID[pcoPROCESS_ID_LEN];
    char               hostName[pcoHOST_NAME_LEN];
    evtEVENT_ID        eventID;
    char               attributeName[pcoATTRIB_NAME_LEN];
    int                conID;
    int                sockFd;
    vltLOGICAL         message;
    vltLOGICAL         waitingReply;
    vltLOGICAL         waitingEventReply;
    msgPROCESSID       ccsProcID;
    msgCMD             command;
    msgCMDID           commandID;
    vltUINT16          count;
} pcoPROC_HANDLE;

typedef struct {
    msgPROCESSID  srcID;
    vltUINT16     msgID;
} pcoMSG_HANDLE;


/* Function prototyping
 */
ccsCOMPL_STAT pcoPackMsg(pcoPROTOCOL   protocol,
                         char          *processID,
                         char          *hostName,
                         char          *sendBuf,
                         msgLENGTH     *sendBufLen,
                         ...);

ccsCOMPL_STAT pcoPing(ccsENVNAME  pcoEnvName, 
                      msgTIMEOUT  timeOut, 
                      ccsERROR    *error);

ccsCOMPL_STAT pcoReadMidasKey(msgTIMEOUT         timeOut,
                              ccsENVNAME         pcoEnvName,
                              vltUINT8           midasUnitID,
                              char               *hostName,
                              char               *keyName,
                              pcoMIDAS_KEY_TYPE  keyType,
                              vltUINT16          elSize,
                              vltUINT16          firstEl,
                              vltUINT16          numberOfEls,
                              void               *keyArray,
                              vltUINT16          *elsRead,
                              ccsERROR           *error);

ccsCOMPL_STAT  pcoWriteMidasKey(msgTIMEOUT         timeOut,
                                ccsENVNAME         pcoEnvName,
                                vltUINT8           midasUnitID,
                                char               *hostName,
                                char               *keyName,
                                pcoMIDAS_KEY_TYPE  keyType,
                                vltUINT16          elSize,
                                vltUINT16          firstEl,
                                vltUINT16          numberOfEls,
                                void               *elArray,
                                ccsERROR           *error);

#ifdef __cplusplus
}
#endif

#endif /*!PCO_H*/





