#ifndef CCS_PLOT_H
#define CCS_PLOT_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: plot.h,v 2.40 2005/08/09 07:50:33 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* mcomin    23/05/00  created
* tebert    21/11/01  new command in plotServer to schedule plotTkDisplay
*/

#include "cpgplot.h"
#include "plotErrors.h"

#define ccsPLOT              "plot"
#define plotMAX_CURVES       8
#define plotSECS_IN_DAY      86400
#define plotDATA_MAX         32768
#define plotPOLL_TEXT_LEN    20
#define plotUNDEFINED        -1

#define firstXREF_ITEM_FIELD  15
#define firstHIST_DATA_FIELD  4

#define plotPERCENT_SCALE     0.5
/*
 * Define point class numbers : these numbers are used to find
 * all the history and the plot points in the database
 */

#define ccsHIST_PT_CLASS    65411
#define ccsPLOT_PT_CLASS    65501

/* 
 * Color indexes from 0 to 15 are reserved to a set of common colors 
 * Define indexes for the main colors.
 */

#define plotBASE_COLOR_INDEX  15
#define plotPEN_COLORS         4
#define plotIDX_BLACK          0
#define plotIDX_WHITE          1

/* Define Type of Trait  */

#define plotTRAIT_OK            0
#define plotTRAIT_SUSPECT       1
#define plotTRAIT_ERROR         2
#define plotTRAIT_DISABLE       3
#define plotTRAIT_SRC_DELETED   5
#define plotTRAIT_SRC_READ      6
#define plotTRAIT_SRC_COERCE    7
#define plotTRAIT_SRC_DE        8
#define plotTRAIT_MODIFIED      9
#define plotTRAIT_QUEST        10

/* Define PGPLOT Units */

#define plotDEVICE_UNITS   0
#define plotINCH_UNITS     1
#define plotMM_UNITS       2
#define plotPIXEL_UNITS    3
#define plotWORLD_UNITS    4

/* Define Commands */
#define plotCMD_POLL         "ATRPOLL"
#define plotCMD_POLL_HIST    "HSTPOLL"
#define plotCMD_POLL_SAMP    "SAMPOLL"
#define plotCMD_DISP_ATTACH  "DSLOT"
#define plotCMD_DISP_DETACH  "RDSLOT"
#define plotCMD_STOP         "STOP"
#define plotCMD_GETINFO      "GETINFO"
#define plotCMD_DINFO        "DINFO"
#define plotCMD_PINFO        "PINFO"
#define plotCMD_START_TK     "PLSTART"
#define plotCMD_PLKILL       "PLKILL"

/* Define Plot Read Mode */
#define plotREAD_UNDEF    0
#define plotREAD_HIST     1
#define plotREAD_SAMP     2
#define plotREAD_ATTR     3
#define plotREAD_FILE     4
#define plotREAD_CONFIG   5

#define plotREAD_SAMP_REFRESH  20 /* Each time read table from the beginning */

#define plotDISP_ATTACH   6
#define plotDISP_DETACH   7

/* Define Frame status */

#define plotFRAME_PLOT     0
#define plotFRAME_FREEZE   1
#define plotFRAME_ZOOM     2


/* Define Re-Configuration Flag */
#define plotCONFIG_DATA     1
#define plotCONFIG_DISPLAY  2

/* Define Axis Type */
#define plotX_AXIS  1
#define plotY_AXIS  2

/* Define scroll Type */
#define plotADJUST_MIN    1
#define plotADJUST_MAX    2
#define plotADJUST_RANGE  3
#define plotADJUST_AUTO   4

/* Type Of Plot */
#define plotTYPE_XY      1
#define plotTYPE_TIME    2

/* Define Scaling Mode for X and Y axis */
#define plotSCAL_MANUAL    1
#define plotSCAL_EXPAND    2
#define plotSCAL_AUTO      3
#define plotSCAL_SCROLL    4
#define plotSCAL_ASCROLL   5

/* Define Styles ... */
#define plotSTYLE_PLAIN    8960
#define plotSTYLE_LOG      8832
#define plotSTYLE_LINE2    8976
#define plotSTYLE_LINE3    8992
#define plotSTYLE_LINE4    9008
#define plotSTYLE_LINE5    9024
#define plotSTYLE_LINE6    8978
#define plotSTYLE_LINE8    8979
#define plotSTYLE_LINE10   8980

/* Define how the data source for the curve is specified */
#define plotDATA_HIST_SOURCE   1
#define plotDATA_HIST_FIELD    2
#define plotDATA_XY            3
#define plotDATA_FILE          4

#define plotDATA_HIST_SOURCE1  17
#define plotDATA_HIST_FIELD1   18
#define plotDATA_REFRESH       19


/* Define names for indexes and main fields */

#define plotY_ATTR     "yAttrOrSourceAddr"
#define plotX_ATTR     "xAttrOrHistTable"
#define plotFIRST_IDX  "firstIndexerAddr"
#define plotOLD_IDX    "oldestIndexerAddr"
#define plotNEXT_IDX   "nextIndexerAddr"


#define plotERR_ADD(error,errId,params...)    \
        errAdd(error,ccsPLOT,errId,__FILE__,"", ##params);

#define plotERR_RETURN(error,errId,params...)    \
        errAdd(error,ccsPLOT,errId,__FILE__,"", ##params); \
        return(FAILURE);

#define plotLOG_ERROR(error,errText,params...)  \
        sprintf(plotLogText,errText,##params); \
        errAdd(error,ccsPLOT,plotERR_GENERIC,__FILE__,"",plotLogText); \
        errCloseStack(error); \
        continue;

#define plotERR_RET_GENERIC(error,errText,params...)    \
        sprintf(plotLogText,errText,##params); \
        errAdd(error,ccsPLOT,plotERR_GENERIC,__FILE__,"",plotLogText); \
        return(FAILURE);

#define plotERR_ADD_GENERIC(error,errText,params...)    \
        sprintf(plotLogText,errText,##params); \
        errAdd(error,ccsPLOT,plotERR_GENERIC,__FILE__,"",plotLogText);


#define pollTIMER(destId,time,cmd,buffer,timerId,error)    \
        timerSendReplyNV(timerSCHEDULE_ONCE,NULL,time,destId, 0,cmd,buffer,strlen(buffer)+1,ccsTRUE,timerId,error);

#define plotLOG_TXT(text)        \
         logData("PLOT",0,text)


#define plotLOG(format,params...)        \
         logData1("PLOT",0,format,params)

#define plotLOG_V(format,params...)        \
      if (plotRoot.verbose)                \
         {                                 \
         logData1("PLOT",0,format,params); \
         }

#define plotLOG_DV(format,params...)       \
      if (plotInfo->verbose)               \
         {                                 \
         logData1("PLOT",0,format,params); \
         }

#define plotLOG_DV2(format,params...)            \
      if (plotInfo->verbose || plotRoot.verbose) \
         {                                 \
         logData1("PLOT",0,format,params); \
         }

#define plotLOG_DBG(level,format,params...)  \
      if (plotRoot.dbgLevel >= level)         \
         {                                   \
         logData1("PLOT",0,format,params);   \
         }




typedef struct {
    char          pollCmd[plotPOLL_TEXT_LEN];
    char          pollParams[plotPOLL_TEXT_LEN];
    ccsTIMEVAL    pollDelay;
    vltUINT32     timerId;
} plotPOLL_INFO;


typedef struct {
    vltDBXREF     xrefAddr;
    dbDIRADDRESS  dirAddr;
    dbSYMADDRESS  alias;
    dbTYPE        dataType[dbMAX_FIELD_CNT+1];
    dbATTRTYPE    attrType;
    vltUINT8      fieldCnt;
    vltUINT16     recCnt;
    vltUINT32     recSize;
    char          strAddress[40];
} plotDB_INFO;


typedef struct {
    vltUINT16     first;
    vltUINT16     oldest;   
    vltUINT16     next;
    plotDB_INFO   firstRec;      /* First filled element       */
    plotDB_INFO   oldestRec;
    plotDB_INFO   nextRec;       /* Next free element          */
    plotDB_INFO   plotPoint;
    plotDB_INFO   dataTable;
    /*
     * The following members are used by the plot API 
     */
    vltLOGICAL    usedTable;   /* Flag specifying if the table is free */
    vltINT16      numbFields;  /* Number of fields to be written by the user appl. */
    dbTYPE        fieldDataType[plotMAX_CURVES + 1];/* User data types */
    vltDOUBLE     *dataBuf;
} plotTABLE_INFO;


typedef struct {
    vltUINT16       plin;
    vltINT32        readMode;
    vltINT32        numPens;
    vltINT32        evtIdx;
    vltINT32        penId[plotMAX_CURVES];
    plotPOLL_INFO   pollInfo;
   /* Indexes describing the plot/history table */
    vltUINT16       dbOldest;
    vltUINT16       dbNext;
} plotSAMP_HIST_INFO;


typedef struct {
    vltBYTES32 windowTitle;
    vltBYTES80 title;
    vltLOGICAL xAxisVisible;
    vltBYTES80 xAxisLabel;
    vltINT32   xAxisStyle;
    vltINT32   xAxisScalingMode;
    vltINT32   xAxisTextFormat;
    vltDOUBLE  xAxisMinima;
    vltDOUBLE  xAxisMaxima;
    vltINT32   xAxisDivisions;
    /*
     * Variables holding previous configuration parameters and used 
     * when the plot is re-configured 
     */
    vltINT32   oldScalingMode;
    vltFLOAT   oldScrollTime;
} plotX_CONFIG;


typedef struct {
    vltINT32         frameId;
    vltINT32         readMode;
    vltLOGICAL       visible;
    vltLOGICAL       leadingPen;
    vltUINT32        sampleRate;
    vltINT32         yAxisSelect;
    vltINT8          yAxisVisible;
    vltBYTES80       yAxisLabel;
    vltINT32         yAxisStyle;
    vltINT32         yAxisScalingMode;
    vltDOUBLE        yAxisMinima;
    vltDOUBLE        yAxisMaxima;
    vltINT32         yAxisDivisions;
    vltBYTES16       yAxisColor;
    vltINT8          dataSourceType;
    vltINT32         gridStyle;
    vltLOGICAL       gridVisible;
    vltLOGICAL       internalClock;
    vltLOGICAL        xTimeAxis;      /* Flag indicating wether the X axis */
                                      /* is time scaled                    */
    /* 
     * Subset of the trait record 
     */
    vltBYTES16       lineColor;
    /* 
     * Other configuration parameters 
     */
    char          fileName[256];
    plotDB_INFO   xAttrOrHistTable;   /* Description of X DB Attributes */
    plotDB_INFO   yAttrOrSourceAddr;  /* Description of Y DB Attributes */
    plotDB_INFO   firstIndexerAddr;   /* First filled element       */
    plotDB_INFO   oldestIndexerAddr;  /* Last yet-not-read element  */
    plotDB_INFO   nextIndexerAddr;    /* Next free element          */
    plotDB_INFO   histConfig;
    plotDB_INFO   histTable;          /* Address of the Hist. table */
    /* 
     * Indexes to the color look-up table 
     */
    vltINT32         yColorIdx;
    vltINT32         lineColorIdx; 
    /*
     * Variables holding previous configuration parameters and used 
     * when the plot is re-configured 
     */
    vltLOGICAL       refreshFrame;
    vltLOGICAL       oldVisible;
    vltINT32         oldFrame;
    vltINT32         oldScalingMode;
    vltINT32         reConfigFlag;
    /*
     * Local Variables
     */
    ccsTIMEVAL       t0;
    plotPOLL_INFO    pollInfo;
    vltINT32         evtIdx;
    vltINT32         hstIdx;       /* Index of the hist/samp table */
    vltUINT8         histTbField;  /* Id of the history table field to read data from */
    vltUINT8         histCfgField; /* Id of the history config XREF field */
} plotY_CONFIG;


typedef struct {
   /* These two data are used by the server only !!! */
   vltFLOAT      *xbuf;
   vltFLOAT      *ybuf;
   /* Indexes describing the plotting buffer */
   vltINT32       plotFirst;  /* First record taken in account after re-draw */
   vltINT32       plotOldest; /* First 'not-yet-read' record    */
   vltINT32       plotNext;   /* Next free record First         */
   vltINT32       plotMaxRec;
   vltLOGICAL     appendRecs;
   vltLOGICAL     wrapBuffer;
   /* Indexes describing the database table */
   vltUINT16      dbLastRead;
} plotDATA_INFO;


typedef struct {
    vltINT32     ph;         /* Handle to PGPLOT device */
    vltINT32     numPens;
    vltINT32     rowId; 
    vltINT32     colId;
    vltINT32     penId[plotMAX_CURVES];
    vltLOGICAL   initialized;
    vltLOGICAL   used;
    vltLOGICAL   internalClock;
    vltLOGICAL   scaleX;
    vltLOGICAL   scaleY;
    vltINT8      plotMode;
/* Flag used to alert plotAdjustY that a X re-scale occurred */
    vltLOGICAL   reScale; 
/* Repeated fields for optimization */
    vltLOGICAL   oneShotPlot;
    vltINT32     yAxisScalingMode;
/* Current plot X axis min,max values */
    vltFLOAT     xPlotMin;
    vltFLOAT     xPlotMax;
    vltFLOAT     xPlotDelta;
    vltFLOAT     xTickSize;
/* Current plot Y axis min,max values */
    vltFLOAT     yPlotMin;
    vltFLOAT     yPlotMax;
    vltFLOAT     yPlotDelta;
    vltFLOAT     yTickSize;
/* Current graphic setting */
    vltFLOAT     fontSize;
    vltFLOAT     viewLeft;
    vltFLOAT     viewRight;
    vltFLOAT     viewTop;
    vltFLOAT     viewBot;
} plotFRAME;


typedef struct {
   vltLOGICAL    textVisible;
   vltINT8       location;
   vltINT8       horizontalAlignment;
   vltINT8       verticalAlignment;
   vltBYTES16    textColor;
   vltBYTES256   text; 
} plotAPPL_TEXT;

typedef struct {
   vltINT32      trait;
   vltINT8       curve;
   vltLOGICAL    markerVisible;
   vltINT32      markerStyle;
   vltBYTES16    markerColor;
   vltLOGICAL    lineVisible;
   vltINT32      lineStyle;
   vltBYTES16    lineColor;
} plotTRAIT;

typedef struct {
    vltLOGICAL          used;
    vltLOGICAL          verbose;
    vltINT32            numFrames;
    vltINT32            numHistOrSamp;
    vltINT32            numEvents;
    plotFRAME           frameConfig[plotMAX_CURVES];
    plotY_CONFIG        yconfig[plotMAX_CURVES];    /* Single pen configuration       */
    plotX_CONFIG        xconfig;                    /* Common X configuration params  */
    plotDATA_INFO       data[plotMAX_CURVES];       /* Array of pointer to the Y data */
    evtEVENT_ID         evtId[plotMAX_CURVES];
    evtEVENT_ID         evtReadConfig;              /* Event Id for re-configuration */
    plotSAMP_HIST_INFO  infoSampOrHist[plotMAX_CURVES];
    char                plotDataSource[10];
   /* 
    *   Internal paramaters describing the status of the plot
    */
    ccsTIMEVAL         t0;
    vltLOGICAL         plotReDraw;
    vltLOGICAL         configFlag;
    dbSYMADDRESS       plotPoint;      /* Plot Table alias or symb. address */
    vltUINT16          numCurves;      /* Number of recs. in table 'curves' */
    vltINT32           subx;
    vltINT32           suby;
    vltINT32           numPenInFile;   /* Number of pens used to plot a file */
    vltFLOAT           scrollTime;
    vltFLOAT           scrollYAxis;
    vltFLOAT           sizeWinX;
    vltFLOAT           sizeWinY;
    vltINT32           buffSize;
    int                processId;
} plotDISP_INFO;


typedef struct {
    vltINT32        buffSize;
    vltINT32        buffRecs;
    msgPROCESSID    plotProcId;
    vltINT8         dbgLevel;
    vltLOGICAL      verbose;

    /* Plot shared memory area */
    vltINT32        plotShMemSize;
    vltINT32        plotMax;
    int             plotShMemId;
   /* 
    * Pointer to shared memory sections 
    */
    char           *plotShmPtr;
    plotDISP_INFO  *plDispInfo;
    vltFLOAT       *plDataArea;
    vltFLOAT       *logBuf;

    /* Used for bytes alignment */
/*
    char           dummy[4];
*/
} plotROOT;

#endif /*!CCS_PLOT_H*/

