#ifndef prsDefines_H
#define prsDefines_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: prsDefines.h,v 1.117 2005/08/24 12:35:06 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  14/10/99  Removed FindGuideStar. FINDGS command obsolete.
* ram/awa   09/08/99  Added define for DETGS
* kwirenst  04/01/99  Added define for STOPAG
* pduhoux   23/10/98  added m2STOPCHOP_CMD macro
* rkarban   24/06/98  change start command for ag to STARTAG
* rkarban   09/05/98  add UPDSKY def for M2
* gchiozzi  04/09/97  Removed MARK command
* gchiozzi  15/05/97  Replaced actSTPWAIC with actSTPWAIT
* rkarban 23/03/97 change STPWAIC to STPWAIT
* nttmgr  14/12/96  Added new defines for commands to trkws
* nttmgr  07/12/96  Added define for ag PRTOGS command
* gchiozzi  22/05/96  Removed defines for obsolete keywords
* gchiozzi  22/05/96  Removed defines for obsolete commands
* gchiozzi  09/01/96  Added define for STOPTRK command
* gchiozzi  18/08/95  Corrected bug: prsMGS_BUFF_ALL replaced with prsMSG...
* gchiozzi  01/08/95  Added include to new tcsDefines.h
* gchiozzi  23/05/95  created
*/

/************************************************************************
 *
 * This file contains common defines used both by C/C++ files and 
 * dbLoader files
 *----------------------------------------------------------------------
 */

/* 
 *  Defines for the target telescope.
 *  The default target is VLT
 *  Up to now only VLT and NTT
 */

#include "tcsDefines.h"


/* Module name */
#define prsMOD "prs"

/* Contents of message buffer for standard commands used globally */
/* and to be propagated to prsAction                              */
#define prsMSG_BUFF_ALL tcsMSG_BUFF_ALL

/*
 * Defines for states values and names
 * These enum and string vector are used in the whole module
 * to identify states and substates.
 * The values must be coordinated with the standard states defined in 
 * evhStates.h
 */

#include "evhStates.h"

/* Local remapping of some standard states (just for convenience) */
#define prsSTATE_OFF             evhSTATE_OFF
#define prsSTATE_LOADED          evhSTATE_LOADED
#define prsSTATE_STANDBY         evhSTATE_STANDBY
#define prsSTATE_ONLINE          evhSTATE_ONLINE
#define prsSTATE_ERROR           evhSTATE_ERROR
#define prsSTATE_READY           evhSTATE_READY
#define prsSTATE_IDLE            evhSTATE_IDLE

/* New states. I start from 16 to be shure not to clash */
#define prsSTATE_PRESETTING     17
#define prsSTATE_STOPPING       18

/* Defines for name states */
#define    prsSTATE_STR_OFF        evhSTATE_STR_OFF 
#define    prsSTATE_STR_LOADED     evhSTATE_STR_LOADED 
#define    prsSTATE_STR_STANDBY    evhSTATE_STR_STANDBY 
#define    prsSTATE_STR_ONLINE     evhSTATE_STR_ONLINE 
#define    prsSTATE_STR_READY      evhSTATE_STR_READY
#define    prsSTATE_STR_ERROR      evhSTATE_STR_ERROR 
#define    prsSTATE_STR_IDLE       evhSTATE_STR_IDLE 
#define    prsSTATE_STR_PRESETTING "PRESETTING"
#define    prsSTATE_STR_STOPPING   "STOPPING"

/*
 * Defines for command names
 */


/* New commands accepted by prsAction process */
#define prsPRESET_CMD    "PRESET"

/* New commands accepted by prsControl process */
#define prsSETUP_CMD     "SETUP"
#define prsPRSALAZ_CMD   "PRSALAZ"
#define prsPRSCOOR_CMD   "PRSCOOR"
#define prsPRSNAME_CMD   "PRSNAME"
#define prsCNTOBJ_CMD    "CNTOBJ"

#define prsCLRSTP_CMD    "CLRSTP"
#define prsSAVCSTP_CMD   "SAVCSTP"
#define prsSAVRSTP_CMD   "SAVRSTP"
#define prsSTOPTRK_CMD   "STOPTRK"

/* Commands send to other modules     */
/* (interface must be defined better) */

/* General commands                   */
#define tcsSETUP_CMD     "SETUP"


/* To Mode Switch:                    */
#define mswPRESET_CMD    "PRESET"
#define mswEPRESET_CMD   "EPRESET"

/* To active optics */
#define actONECAL_CMD  "ONECAL"
#define actCYCLAO_CMD  "CYCLAO"
#define actSTPWAIT_CMD "STPWAIT"

/* To auto guide */
#define agPRTOGS_CMD "PRTOGS"
#define agOBJCNT_CMD "OBJCNT"
#define agGSCNTPR_CMD  "PR2STAR"
#define agGSCNTEL_CMD  "STAR2PR"
#define agSTART_CMD  "STARTAG"
#define agSTOPRG_CMD  "STOPRG"
#define agSTOP_CMD  "STOPAG"
#define agSRCGS_CMD  "SRCGS"
#define agDETGS_CMD  "DETGS"

/* To Mirror 2 */
#define m2RESET_CMD "RESET"
#define m2UPDSKY_CMD "UPDSKY"
#define m2CHOPPOS_CMD "CHOPPOS"
#define m2STOPCHOP_CMD "STOPCHP"
#define m2SETUP_CMD     "SETUP"


/* To Mirror 1 */
#define m1PSDEFPOS_CMD "MOVMIAP"


/* To tracking module */
#define trkOBJFIX_CMD   "OBJFIX"
#define trkOBJNAME_CMD  "OBJNAME"
#define trkOBJROT_CMD   "OBJROT"
#define trkOBJSTAR_CMD  "OBJSTAR"
#define trkOFFSAD_CMD   "OFFSAD"
#define trkOFFSADG_CMD  "OFFSADG"
#define trkSETAV_CMD    "SETAV"
#define trkSETLAM_CMD   "SETLAM"
#define trkSTOP_CMD     "STOP"


/* To adc module */
#define adcCOOR_CMD     "ADCCOOR"
/* Standard OK message */
#define prsOK_MSG            "OK"
#define prsOK_ABORT_REPLY   "OK, superseded by valid equivalent command"


/*
 *  Target types
 */
#define prsTARG_NONE        0
#define prsTARG_ALAZ        1
#define prsTARG_COOR        2
#define prsTARG_NAME        3

#define prsTARG_STR_NONE        "NO_TARGET"
#define prsTARG_STR_ALAZ        "ALT_AZ"
#define prsTARG_STR_COOR        "COORDINATE"
#define prsTARG_STR_NAME        "NAMED_POS"

/*
 *  Preset types
 */
#define prsPRS_ERR         0
#define prsPRS_ASTRO       1
#define prsPRS_ENG         2

#define prsPRS_STR_ERR     "ERROR"
#define prsPRS_STR_ASTRO   "ASTRONOMICAL"
#define prsPRS_STR_ENG     "ENGINEERING"

/*
 *   Auto guide type
 */
#define prsAG_NONE              0
#define prsAG_AUTO_GUIDE        1
#define prsAG_FIELD_STAB        2

#define prsAG_STR_NONE              "NONE"
#define prsAG_STR_AUTO_GUIDE        "AUTO_GUIDE"
#define prsAG_STR_FIELD_STAB        "FIELD_STAB"

/*
 *   Where are guidestars (can be SETUPFILE, CATALOGUE or NONE)
 */
#define prsGSTAR_NONE         "NONE"

/*
 *  ADC types
 */
#define prsADC_NONE        0
#define prsADC_ACTUAL      1
#define prsADC_COORD       2
#define prsADC_ZENITH      3
#define prsADC_ABSPOS      4

#define prsADC_STR_NONE    "NONE"
#define prsADC_STR_ACTUAL  "ACTUAL"
#define prsADC_STR_COORD   "COORD"
#define prsADC_STR_ZENITH  "ZENITH"
#define prsADC_STR_ABSPOS  "ABSPOS"

/*
 *  Defines for setup file keywords
 */ 

/* Setup file category */
#define prsSK_CATEGORY    "TEL"

/* Setup file subsystems */
#define prsSK_SUBS_PRS    "PRS"
#define prsSK_SUBS_TARG   "TARG"
#define prsSK_SUBS_ROT    "ROT"
#define prsSK_SUBS_AG     "AG"
#define prsSK_SUBS_FS     "FS"
#define prsSK_SUBS_M2     "M2"
#define prsSK_SUBS_ADC    "ADC"
#define prsSK_SUBS_AO     "AO"

/* Setup file keywords */
#define prsSK_TYPE        "TYPE"
#define prsSK_ALPHA       "ALPHA"
#define prsSK_DELTA       "DELTA"
#define prsSK_EPOCHSYSTEM "EPOCHSYSTEM"
#define prsSK_EPOCH       "EPOCH"
#define prsSK_EQUINOX     "EQUINOX"
#define prsSK_PMA         "PMA"
#define prsSK_PMD         "PMD"
#define prsSK_RADVEL      "RADVEL"
#define prsSK_PARALLAX    "PARALLAX"
#define prsSK_COORDTYPE   "COORDTYPE"
#define prsSK_ALT         "ALT"
#define prsSK_AZ          "AZ"
#define prsSK_NAME        "NAME"
#define prsSK_WLENGTH     "WLENGTH"
#define prsSK_OFFSETALPHA "OFFSETALPHA"
#define prsSK_OFFSETDELTA "OFFSETDELTA"
#define prsSK_ADDVELALPHA "ADDVELALPHA"
#define prsSK_ADDVELDELTA "ADDVELDELTA"
#define prsSK_PRSCRIT     "PRSCRIT"
#define prsSK_OFFANGLE    "OFFANGLE"
#define prsSK_ENABLED     "ENABLED"
#define prsSK_ALTAZTRACK  "ALTAZTRACK"
#define prsSK_AGTYPE      "TYPE"
#define prsSK_START       "START"
#define prsSK_OBJCNT      "OBJCNT"
#define prsSK_AGMINMAG    "MINMAG"
#define prsSK_AGMAXMAG    "MAXMAG"
#define prsSK_AGVIGLIM    "VIGLIM"
#define prsSK_RESET       "RESET"
#define prsSK_ZENITH      "ZENITH"
#define prsSK_ABSPOS      "ABSPOS"
#define prsSK_DTIME       "DTIME"
#define prsSK_GSCAT       "CAT"
#define prsSK_ADCTYPE     "TYPE"
#define prsSK_GUIDESTAR   "GUIDESTAR"
#define prsSK_DTIME	  "DTIME"
#define prsSK_LAMRED      "LAMRED"
#define prsSK_LAMBLUE     "LAMBLUE"

#endif /*!prsDefines_H*/
