#ifndef QSEMU_ENVTBL_H
#define QSEMU_ENVTBL_H

/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: qsemu.h,v 3.57 2005/10/03 20:22:15 vltsccm Exp $" 
*
* qsemuEnvTbl.h - Include file for environment table 
*                 Defines structure for table and
*                 prototypes for functions to manipulate the table
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* tebert     08.05.98   First Version
* bgilli     05/10/99   Added MAXHOSTNAMELEN for LINUX
* bgilli     21/10/99   increased hash table size from 16 to 64Kb
* bgilli     01/03/00   Added Bytes swapping macros from T. Ebert.
* bgilli     26/04/00   HP-UX 11 => common thread handling.
* tebert     15/10/04   new element sendObi in structure
*/
#define SELCAST(X)    (X) /* used in `select', but not defined in SUN-headers*/

#ifndef __EXTENSIONS__
#define __EXTENSIONS__
#endif
#ifndef _REENTRANT
#define _REENTRANT
#endif

#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <pwd.h>
#include <limits.h>
#include <pthread.h>
#include <netdb.h>
#include <sys/time.h>


/*
 * defines
 */
#if defined(i386) || defined(__i386)
/*
 * byte swapping macros
 */

/* most significant byte of 2-byte integer */
#define QSEMU_MSB_INT16(x)        (((x) >>  8) & 0xff)
/* least significant byte of 2-byte integer */
#define QSEMU_LSB_INT16(x)         ((x)        & 0xff)
/* most significant word of 2-word integer */
#define QSEMU_MSW_INT32(x) (((x) >> 16) & 0xffff)
/* least signifcant byte of 2-word integer */
#define QSEMU_LSW_INT32(x)  ((x)        & 0xffff)

/* swap the MSB with the LSB of a 16 bit integer */
#define QSEMU_SWAP_INT16(x) (QSEMU_MSB_INT16(x) | (QSEMU_LSB_INT16(x) << 8))

/* swap all bytes of a 32 bit integer */
#define QSEMU_SWAP_INT32(x) (QSEMU_SWAP_INT16(QSEMU_LSW_INT32(x)) << 16) | \
                             QSEMU_SWAP_INT16(QSEMU_MSW_INT32(x))
/* swap all bytes of two consecutive 32 bit integers */
#define QSEMU_SWAP_2INT32(x_ptr,y_ptr) {int32_t i=*x_ptr;\
                                     *x_ptr=*y_ptr;\
                                     *y_ptr=i;}\
                           *x_ptr = QSEMU_SWAP_INT32(*x_ptr);         \
                           *y_ptr = QSEMU_SWAP_INT32(*y_ptr) 
/*

inline void qsemu_swap_2int32(int32_t* i1, int32_t* i2)
                           { int32_t i=*i1; *i1=*i2; *i2=i; };
			   
swap all bytes of two consecutive 32 bit integers 
#define QSEMU_SWAP_2INT32(x_ptr,y_ptr) qsemu_swap_2int32(x_ptr, y_ptr); \
                           *x_ptr = QSEMU_SWAP_INT32(*x_ptr);         \
                           *y_ptr = QSEMU_SWAP_INT32(*y_ptr) 
*/
#endif
#ifdef LINUX
    #ifndef MAXHOSTNAMELEN
        #define        MAXHOSTNAMELEN  64
    #endif
#endif
#define qsemuCMD_PRINT_TBL              "QPRTBL"

#define qsemuPROTOCOL_VERSION           3   /* default qsemu protocolVersion */
#define qsemuPROTOCOL_VERSION_RTAP_6_6  3   /* identifies Rtap rev. >= 6.6   */
#define qsemuMAX_ENVS                   100
#define qsemuMAX_CONNECT_FAILURES       4
#define qsemuDISCONNECT_WAIT_TIME       18  /* seconds */
#define qsemuRETRY_TIMEOUT              3   /* seconds */
#define qsemuHASH_SIZE                  65536
#define qsemuTIMEOUT                    9

/*
 * typedefs
 */
typedef struct
    {
    int  mtype;
    char mtext[msgMAXLEN];
    } msgQUEUE;

typedef struct
    {
    vltINT32        netPacketSize;
    char            userName[9];
    vltUINT8        reserved;
    vltUINT16       msgSeqNum;
    } qsemuNET_HEADER;
 
typedef struct
    {
    qsemuNET_HEADER         netHeader;
    ccsMSG_ACK              msg;
    } qsemuNET_PACKET;
 
typedef qsemuNET_PACKET *qsemuNET_PACKETP;


typedef enum 
    {
    qsemuDISCONNECTED = 0,
    qsemuCONNECTING,
    qsemuCONNECTED
    } qsemuSTATE;

typedef struct
    {
    rtProcessId     procId;
    vltUINT8        protocolVersion;
    } qsemuHELLO_MSG;
 
typedef struct
    {
    qsemuNET_HEADER netHeader;
    qsemuHELLO_MSG  helloMsg;
    } qsemuNET_PACKET_ID;

typedef struct
    {
    vltBYTES12 name;
    int        uid;
    } qsemuNAME_STRUCT;

typedef struct
    {
    ccsENVNAME          env;
    char                host[MAXHOSTNAMELEN];
    int                 tcpPort;
    char                netSuffix[8];
    int                 fd;
    int                 protocolVersion;
    int                 pingCtr;
    int                 waitPingResponse;
    int                 connectFailures;
    int                 monIndex;
    time_t              disconnectTime;
    struct sockaddr_in  s;
    pthread_t           threadId;
    vltLOGICAL          sendObi;
    qsemuSTATE          state;
    qsemuNET_PACKET     qsMsg;
    } qsemuENV_LIST;


/*
 * externals
 */
extern int               qsemuTimeout;
extern char              qsemuEnvTbl[MAX_INPUT];
extern ccsENVNAME        qsemuEnvName;
extern qsemuENV_LIST    *qsemuEnvList;
extern int               qsemuEnvCnt;
extern char              qsemuUser[9];
extern int               qsemuFdReceive;
extern int               qsemuPid;
extern int               qsemuProtocolVersion;
extern int               qsemuMsgSeqNum;
extern pthread_t         qsemuThread;
extern pthread_mutex_t   qsemuMutex;
extern void             *qsemuHash;
extern qsemuNAME_STRUCT *qsemuNames;
extern vltLOGICAL        qsemuLog;
extern int               qsemuCnt;
extern vltLOGICAL        qsemuRObi;

/*
 * functions
 */
ccsCOMPL_STAT  qsemuParseEnvList();
ccsCOMPL_STAT  qsemuCreateSocket(ccsERROR *error);
void           qsemuGetFromLocal(ccsERROR *error);
void           qsemuGetFromRemote();
void           qsemuSendNackToLocal(rtMsgHeader *header, ccsSINGLE_ERROR *singleError);
ccsCOMPL_STAT  qsemuSendToLocal(msgQUEUE *msg, int size, int *errVal, ccsSINGLE_ERROR *singleError);
qsemuENV_LIST *qsemuGetTblEntry(char *env);
ccsCOMPL_STAT  qsemuConnectRemote(qsemuENV_LIST *entry, ccsSINGLE_ERROR *singleError, ccsERROR *error);
ccsCOMPL_STAT  qsemuSendHello(int fd, ccsSINGLE_ERROR *singleError);
ccsCOMPL_STAT  qsemuReadHello(int fd, qsemuENV_LIST *entry, ccsSINGLE_ERROR *singleError);

int            qsemuReadSocket(int fd, char *buf, int len);
ccsCOMPL_STAT  qsemuSendToRemote(qsemuENV_LIST *entry, qsemuNET_PACKET *qsMsg, 
                                 ccsSINGLE_ERROR *singleError);
void           qsemuCloseEnv(qsemuENV_LIST *entry);
ccsCOMPL_STAT  qsemuGetHelloAccept(int fd, qsemuENV_LIST **entry, void *sockaddr);
ccsCOMPL_STAT  qsemuSendNackToRemote(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                                     ccsSINGLE_ERROR *singleError, vltUINT8 type);
 
ccsCOMPL_STAT  qsemuSendAckToRemote(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                                    ccsSINGLE_ERROR *singleError);
void           qsemuSendObituary(rtMsgHeader *msg);
void           qsemuEnvThread(qsemuENV_LIST *entry);
ccsCOMPL_STAT  qsemuSendPingResponse(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                                     ccsSINGLE_ERROR *singleError);
ccsCOMPL_STAT  qsemuSendProcnum(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                                ccsPROCNUM procnum, ccsSINGLE_ERROR *singleError);
ccsCOMPL_STAT  qsemuSendProcname(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                                 ccsPROCNAME name, ccsSINGLE_ERROR *singleError);
ccsCOMPL_STAT  qsemuSendPid(qsemuENV_LIST *entry, vltUINT16 msgSeqNum, rtMsgHeader *header, 
                            rtInt pid, ccsSINGLE_ERROR *singleError);

void           qsemuPrintNetHeader (qsemuNET_HEADER * netHeader);
void           qsemuPrintMsgHeader (rtMsgHeader * msgHeader);
void           qsemuLogDebug (char *fmt, int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);
void           qsemuPrintNack (ccsACKNOWLEDGE *ack);
msgLENGTH      qsemuSendTblToLocal(msgMSG *header, msgQUEUE *buf, ccsSINGLE_ERROR *singleError, ccsERROR *error);
ccsCOMPL_STAT  qsemuQueryEnvTbl(char *printBuffer, int *printSize);

ccsCOMPL_STAT  qsemuSendPingRequest(qsemuENV_LIST *entry,  ccsSINGLE_ERROR *singleError);
int            qsemuCompare(const void *name1, const void *name2);

#if defined(i386) || defined(__i386)
    void       qsemuMsgMassage (char *header, vltLOGICAL remote);
#endif

#ifndef ERROR
#define ERROR -1
#endif

#ifndef OK
#define OK 0
#endif

/* #define NULL 0 */
/* typedef unsigned int size_t; */
 
 
/*  Structure of hashtable in memory:
 *
 *    size_t         size;          // maximum number of entries
 *    size_t         insert_count;  // current number of entries
 *    Hashtable_item table[size];   // array of entries
 *
 *  Remaining memory is ununsed.
 */
 
#define HASH_STRING_LENGTH 10
 
struct Hashtable_item
{
   char      key[HASH_STRING_LENGTH];   /* string for identification */
   char     *value;                     /* data combined with key    */
   int8_t    used;                      /* status of entry in array: */
                                        /*    0: not used up to now  */
                                        /*    1: used actually       */
                                        /*   -1: used before, but    */
                                        /*       not actually        */
};
 
 
/*  prototypes
 */
 
int symAdd
    (
    void         *symTblId,    /* symbol table to add symbol to       */
    char         *key,         /* pointer to symbol key string        */
    char         *value        /* symbol address                      */
    );
 
int symFindByName
    (
    void         *symTblId,    /* symbol table to add symbol to       */
    char         *key,         /* pointer to symbol key string        */
    char         **pValue      /* symbol address                      */
        );
 
int symRemove /* caution: no merging of empty space */
    (
    void         *symTblId,    /* symbol tbl to remove symbol from    */
    char         *key          /* key of symbol to remove             */
        );
 
int memPartAddToPool
    (
    void         *partId,      /* partition to initialize             */
    char         *pPool,       /* pointer to memory block             */
    unsigned     poolSize      /* block size in bytes                 */
        );
 
int symHashCreate
    (
    int          tableSize,    /* max. size of hash table in bytes    */
    void         *symPartId    /* start of free memory for hash table */
        );
 
int fetch_hash_index(
    void         *symTblId,    /* symbol table to fetch symbol from   */
    char         *search_key,  /* pointer to symbol key string        */
    size_t       *hash_index   /* returned index, if search_key found */
        );
 
size_t hash_function(
    void         *symTblId,    /* symbol table to fetch symbol from   */
    char         *key          /* pointer to symbol key string        */
        );
 
int hashtable_print_qsemu(
    void         *symTblId     /* symbol table                        */
        );
 
int hashtable_statistics(
    void         *symTblId     /* symbol table                        */
        );

#endif
