#ifndef SCAN_LITE_INTERNAL_H
#define SCAN_LITE_INTERNAL_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: scanLiteInternal.h,v 2.60 2005/07/31 20:47:24 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* tebert    16/07/99  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "scanLite.h"

#define SSCONN        "SSCONN"

#define scanDEFAULT_SSMA_SIZE   12400
#define scanOPEN                 1000
#define scanCLOSE                1001
#define scanDIRECT               1002

/*
 * 'scan config' point: alias
 */
#define rtSCAN_CONFIG_PT        "scan config"

#define rtCP_COMM_MODE          ".communication mode"
#define rtCP_DEV_TYPE           ".device type"
#define rtCP_CONTROL            ".control"
#define rtCP_STATUS             ".status"

#define rtSD_SI_CNTRL           ".scan input control"
#define rtSD_SI_STATUS          ".scan input status"
#define rtSD_SI_FAIL_THRSHLD    ".failure threshold"
#define rtSD_SCAN_INPUT         ".scan input"
#define rtSD_SO_CNTRL           ".scan output control"
#define rtSD_SO_STATUS          ".scan output status"
#define rtSD_SCAN_OUTPUT        ".scan output"
#define rtSD_FAIL_THRSHLD       ".dev fail threshold"
#define rtSD_CONFIG_OPT         ".config optimization"
#define rtSD_SSMA_UPDATE_LIM    ".SSMA update limit"
#define rtSD_DEBUG_LEVEL        ".debug level"

#define rtSI_POINT_CONTROL      "point control"
#define rtSI_POINT_STATUS       "point status"
#define rtSI_POLL_TYPE_MASK     "poll type mask"
#define rtSI_ST_BUFFER_SIZE     "st buffer size"
#define rtSI_SSMA_OFFSET        "SSMA offset"
#define rtSI_NO_OF_ELEMENTS     "# of elements"
#define rtSI_RTAP_DE_TYPE       "RTAP DE type"
#define rtSI_DEADBAND_TYPE      "deadband type"
#define rtSI_DEADBAND           "deadband"
#define rtSI_LINK_ADDRESS       "link address"
#define rtSI_RESERVED_FLD       "reserved1"
#define rtSI_FIRST_FIELD_NO     0
#define rtSI_LAST_FIELD_NO      10

#define rtCP_POLL_TYPES         ".poll types"
#define rtCP_PRBX_TYPES         ".PRBX types"

#define rtSC_SCAN_TASKS         ".scan tasks"
#define rtST_SCAN_TASK_NAME     "scan task name"

#define rtE_ST_FUNC_UNSUPPORTED   0x00100011
#define rtE_SS_CONFIG_ERR         0x00200022
#define rtE_ST_TIMEOUT            0x00300033
#define rtE_ST_DCD_ERROR          0x00400044
#define rtE_ST_CTS_ERROR          0x00500055
#define rtE_ST_BAD_CRC            0x00600066

/*
 * scan system scan task capability & communication port modes
 */
#define rtPOLLED_PRBX           0       /* capability and mode */
#define rtPOLLED_PRBX_SRBX      1       /* capability and mode */
#define rtSRBX                  2       /* SRBX only mode */

#define scanMAX_PLINS_IN_LIST   255

typedef struct
    {
    ccsNODE        node;
    msgPROCESSID   procId;
    rtSsConnection handle;
    } scanHANDLE;

typedef struct 
    {
    rtInt8    startDay;
    rtInt8    startHour;
    rtInt8    startMin;
    rtInt8    startSec;
    rtUInt8   intervalDays;
    rtUInt8   intervalHours;
    rtUInt16  intervalMins;
    rtUInt32  intervalSecs;
    rtUInt16  intervalMSecs;
    } rtPTypesRecord;


typedef struct
    {
    int ssmaSize;
    int ssmaOffset;
    } scanSSMA;

typedef struct
    {
    vltUINT32      command;
    vltUINT32      counter;
    rtSsConnection handle;
    } scanTYPE_HDR;

typedef struct
    {
    msgHEADER     msgHeader;
    scanTYPE_HDR  scanHeader;
    char          body [msgMAXLEN - sizeof(msgHEADER) - sizeof(scanTYPE_HDR)];
    } scanCOMMAND;

/*
 * hooks
 */
void  rtCpInitHook        (void);
void  rtCpShutdownHook    (void);
rtInt rtDepMsgHook        (rtMsgHeader *message);
void  rtPostCntrlHook     (void);
void  rtPostDirCmdHook    (void);
void  rtPostInitDvceHook  (rtPlin scanDevicePlin);
void  rtPostPollHook      (void);
void  rtPostPollTypeHook  (void);
void  rtPostPrbxHook      (void); 
void  rtPostPrbxTypeHook  (void);
void  rtPostReadHook      (void); 
void  rtPostRequestHook   (void); 
void  rtPostWriteHook     (void); 
void  rtPreCntrlHook      (void); 
void  rtPreDirCmdHook     (void); 
void  rtPreInitDvceHook   (rtPlin scanDevicePlin); 
void  rtPrePollHook       (void); 
void  rtPrePollTypeHook   (void);
void  rtPrePrbxHook       (void); 
void  rtPrePrbxTypeHook   (void);
void  rtPreReadHook       (void); 
void  rtPreRequestHook    (void);
void  rtPreWriteHook      (void); 

rtInt rtRequestCapability (void);
rtInt rtSetCwp            (rtPlin plin);
rtInt rtUpdateInputStatus (rtInt record, rtSsState status);
void  rtUpdatePlinList    (rtPlin plin);
rtInt rtUpdateSsma        (rtScanInputRecord *scanRecord, rtUInt8 *buffer, rtUInt32 size,
                           rtUInt16 recnum);
rtInt rtSetNewStatus      (rtScanInputRecord *scanRecord, rtUInt16 recnum, 
                           rtSsState newStatus);
rtInt rtGetSiTableSize    (void);
rtInt rtEnableDeviceInput (void);
rtInt rtFailDeviceInput   (void);
void  rtMoveMemA          (void *destination, const void *source, rtUInt32 count);
rtInt rtGetDbCwp          (rtDbConnection connection);
rtSsState rtGetScanInputStatus (void);

rtInt32 rtDevPtDisabled   (rtPointId *pointId, rtInt scanInputPts);
rtInt32 rtDevPtEnabled    (rtPointId *pointId, rtInt scanInputPts);
rtInt32 rtDirectCmd       (rtUInt8 *writeCommand, rtUInt16 commandLength,
                           rtUInt16 timeOut, rtSsDcAction commandAction, 
                           rtUInt8 *replyBuf, rtUInt16 maxReplyBufSize);
void    rtDisableCommPort (void);
rtInt32 rtInitDevice      (rtPlin scanDevicePlin, rtUInt8 retries, rtUInt16 timeout);
FILE   *rtInitCommPort    (void);
rtInt32 rtDisableDevice   (rtSsCqAction action, rtPlin scanDevicePlin, rtUInt8 retry,
                           rtUInt16 timeOut);
rtInt32 rtRestartDevice   (rtPlin scanDevicePlin, rtUInt8 retry, rtUInt16 timeout,
                           rtLogical rst_type);


#endif /*!SCAN_LITE_INTERNAL_H*/
