/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: scanPrivate.h,v 2.60 2005/07/31 20:47:24 vltsccm Exp $"
*
* <scanPrivate.h> - CCS/SCAN SYSTEM private interface file
*
* who        when      what
* ---------  --------  ---------------------------------------------- 
* B.Gilli    06/11/95  added field for PRBX intervalMSecs for Rtap 6.6
* B.Gilli    25/02/97  removed reference to dbPrivate include.
* B.Gilli    26/04/00  removed refs to booking, obsolete with 200011
*
******************************************************************************
*/
#ifndef SCAN_PRIVATE_H
#define SCAN_PRIVATE_H

#include "ccs.h"
#include "db.h"
#include "tims.h"
#include "timer.h"
#include "err.h"
#include "log.h"
#include "msg.h"
#include "evt.h"

#ifndef MAKE_VXWORKS
#include "ccsRtap.h"
#include "scanCCS.h"
#ifndef CCS_LIGHT
#   include <rtap/scan.h>
#   define  scanOTHER_MANAGER  "scanMngr"
#else
#   define  scanOTHER_MANAGER  "RtapScanMngr"
#   include "scanLiteInternal.h"
#endif
/* #   include "bookPrivate.h"    Obsolete */
#endif

/*
 * DEFINITIONS
 */
#define scanOUT_TASK_NAME         "scanOut"      /* name of scanOut task                */
#define scanOUT_TABLE             "scan output"  /* sym. name of scan output table      */
#define scanOUT_TABLE_NO_FIELDS   16             /* scan output table: number of fields */
#define scanMAX_PRBX_TYPES        32             /* max. number of polling periods      */
#define scanINPUT_TBL_AIN          6             /* AIN of scan input table              */
#define scanENV_AIN               14             /* AIN of scanOut environment attribute */
#define scanPROCNUM_AIN           15             /* AIN of scanOut environment name      */
#define scanOUT_EVENT_DELAY       250000         /* max. delay for sending SRBX data     */
#define scanINPUT_TBL_FIELDS      17

/*
 * comm-port defintions
 */
#define scanLAN_POINT       "<alias>LAN"   /* point where to fing debug information attribute */

/*
 * LCU 'scan device' Point: device dependent Attribute names
 */
#define SD_DEVICE_STATS       ".device stats"
#define SD_DEBUG_MODE         ".debug mode"
#define SD_SCANOUT_PROCNUM    ".scanOut ProcNum"
#define SD_SCANOUT_ENV        ".scanOut environment"
#define SD_RESTART_FLAG       ".restart flag"
#define SD_RESTART_DELAY      ".restart delay"

#define SI_NO_OF_LCU_FIELDS     7

/*
 * Definitions for communications statistics table
 * Field numbers in "device stats" Attribute of scan device Point
 */
#define TRANS_ATTEMPTS          0
#define TRANS_SUCCESS           1
#define TIMEOUT_ERR             2
#define CONFIG_ERR              3
#define LCU_ERR                 4
#define DEVICE_STATS_SIZE       5      /* used for dimensioning arrays */

/*
 * Rtap message numbers used for scan system : 10 numbers starting with :
 */                   
#define  scanMESSAGE       msgTYPE_SCAN_OFFSET
#define  scanREP           (scanMESSAGE + 1)
#define  scanBUF           (scanMESSAGE + 2)
#define  scanCONTROL       (scanMESSAGE + 3)
#define  scanCONTROL_REPLY (scanMESSAGE + 4)

/*
 * ScanOut operations :
 */
#define  scanENABLE          1
#define  scanGO              2
#define  scanDISABLE         3
#define  scanDEL             5
#define  scanENABLE_PT       9
#define  scanDISABLE_PT     10

/*
 * command used for timerSendCommand()
 */
#define scanTIMER_CMD  "SCANPO"
#define scanEVENT_SEND "SCANEV"
#define scanDES_CMD    "SCANDES"

/*
 * data acquisition modes
 */
#define  POLL           1
#define  SRBX           2
    
/* 
 * scan output table record;
 * if something is changed in this structure 
 * compare with definition of scanINPUT_NBR_FIELD
 */
typedef struct 
    {                   
    vltLOGICAL      point_control;
    vltUINT8        point_status;
    vltUINT16       reserved1;
    vltUINT16       element_count;
    vltUINT16       start_address;
    vltUINT8        org_id;
    vltUINT8        dbnr;
    vltUINT8        bit_position;
    vltUINT8        deadband_type;
    vltUINT32       eventId;
    vltDOUBLE       deadband;
    vltBYTES4       poll_type_mask;
    vltUINT16       scanIn_num;
    vltUINT16       reserved2;
    vltDBXREF       link_address;
    vltUINT32       reserved3;
    } scanOUTPUT_TABLE_REC;
    
/* scan reply message header */
typedef struct 
    {
    vltUINT16    operation;     /* Returns the requested operation.   */
    vltUINT16    entries;       /* Number of following reply messages */
    vltUINT16    devicePlin;    /* PLIN of scan device point */
    } scanHEADER;

/* header for scan data messages */    
typedef struct
    {
    ccsTIMEVAL     timestamp;       /* times-tamp */
    vltUINT16      number;          /* number of parameters */
    vltUINT16      devicePlin;      /* PLIN of scan device point */
    vltUINT16      recMin;          
    vltUINT16      recMax;
    vltINT32       startByte;       /* start byte if multiple messages for one parameter */
    vltLOGICAL     multiple;        /* indicates if multiple messages for one parameter  */
    vltUINT8       reserved[3];    
    } scanBUFHEADER;

typedef struct 
    {
    msgHEADER   msgHeader;
    union 
        {
        scanBUFHEADER bufHeader;
        scanHEADER    request;
        } type;
    char body[msgMAXLEN - sizeof(msgHEADER) - sizeof(scanBUFHEADER) - 1];
    } scanMSG;

/* message body for a SRBX */
typedef struct 
    {                  
    vltDOUBLE    deadband;          
    vltUINT8     deadband_type;
    vltUINT8     reserved[7];    
    } srbxBODY;

/* body for ADD/DEL commands */
typedef struct 
    {
    dbSYMADDRESS     attr_name;  /* Name of attribute to be scanned. */
    vltUINT16        mode;       /* SRBX or POLL as defined above    */
    vltUINT16        entry_in;   /* Index in the scan input table.   */
    vltUINT16        entry_out;  /* Index in the scan output table.  */
    vltUINT16        bufSize;    /* size of buffer allocated         */
    union {
          vltBYTES4  period;     /* Poll mask defining polling period. */
          srbxBODY   srbxBody;
          } body;
    } scanBODY;               
    
/* 
 * replies
 */    
typedef struct 
    {
    vltUINT16    entry_in;          /* Index in the scan input table.    */
    vltUINT16    entry_out;         /* Index in the scan output table.   */
    vltUINT16    bufSize;           /* number of bytes to be transferred */
    vltUINT8     result;            /* FAILED or SUCCESS                 */
    } scanREPBODY;
    
/* 
 * message body for POLL
 * (this structure must be identical to rtPTypesRecord) 
 * with Rtap 6.6 this structure was not more aligned and 
 * we add two bytes.
 */
typedef struct 
    {
    vltINT8   startDay;
    vltINT8   startHour;
    vltINT8   startMinute;
    vltINT8   startSecond;
    vltUINT8  intervalDays;
    vltUINT8  intervalHours;
    vltUINT16 intervalMins;
    vltUINT32 intervalSecs;
    vltUINT16 intervalMSecs; 
    vltUINT16 reserved;
    } scan_prbxBODY;

#ifndef MAKE_VXWORKS
/*
 * The 'scan input' record structure MUST be maintained such that its size
 * is a multiple of 8 bytes.  This is required to preserve proper alignment
 * on the Series 800.  The fixed portion is also required to meet these
 * criteria.
 */
typedef struct	
    {
    rtScanInputRecord   fixed;
    rtUInt16            element_count;
    rtUInt16            scanOut_num;
    rtUInt8             lcu_deadband_type;
    rtUInt8             reserved2;
    rtUInt16            reserved3;
    rtDouble            lcu_deadband;
    rtBytes256          remote_name;
    } SiLcu;

/*
 * internal used functions  (->interface)
 */
ccsCOMPL_STAT  scanEnaDisAttr (rtSsCqAction  action, dbSYMADDRESS  attribute[],
                               vltUINT16     attrNbr, ccsERROR     *error);
#endif

#endif  /* !SCAN_PRIVATE_H */
