#ifndef SGSSYSIC
#define SGSSYSIC

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>

/*
**  - - - - - - - - -
**   s g s s y s . h
**  - - - - - - - - -
**
**  Simple Graphics System function prototypes, macros, global data
**  include file.
**
**  !!! Version for use with plotting via Tcl/Tk !!!
**
**  Last revision:   27 January 2000
**
**  Copyright 2000 P.T.Wallace.  All rights reserved.
*/

/* Function prototypes: public + internal */
#include "sgs.h"

/*
**  Plotting mode:  flag which distinguishes between the "standard"
**  and "alternative" plotting modes, and if the latter shows
**  the plotting status:  modep=0 signifies standard plotting (separate
**  alphanumeric and graphics display surfaces);  modep=1 signifies
**  alternative plotting (single graphics terminal with switchable,
**  non-overlaid alpha and graphics screens), currently in alpha mode;
**  modep=2 signifies alternative plotting currently in graphics mode.
*/
   extern int modep;

/* Default name of hardcopy file */
#define LHCF 120
   extern char hcfile [ LHCF ];

/* Name of Tcl/Tk shell program */
#define LWISH 120
   extern char wish [ LWISH ];

/* Colour codes (element zero is the current pen) */
#define NCOL 18
#define LCOL 32
   extern char pens [ NCOL + 1 ] [ LCOL ];
   extern char bgcol [ LCOL ];

/* Pen recommendations */
   extern int npenrf, npenrc, npenra, npenrl, npenrp, npenro;

/*
** Device table:  - one item in each array per device ID
**                - device IDs start at 1
**                - first element of each array belongs to device ID 1
*/

/* Maximum number of graphics devices that may be opened */
#define MXWK 16

/* Stream for sending Tcl/Tk commands for each device */
   extern FILE *plotfp [ MXWK ];

/* Supplementary Tcl/Tk setup commands */
#define MXTCL 500
#define LTCL 180
   extern char tcltk [ MXTCL ] [ LTCL ];

/* Default display surface width and height (pixels) */
   extern int nxds, nyds;

/* Display surface extent in pixels for each device */
   extern double wsdsp [ MXWK ] [ 4 ];

/* Display surface size in normalized device coordinates for each device */
   extern double wsdsn [ MXWK ] [ 2 ];

/*
** Zone table:  - one item in each array per zone
**              - zone IDs start at 1
**              - first element of each array belongs to zone ID 1
*/

/* Maximum number of zones */
#define MXZ 32

/* ID of zone last selected */
   extern int iszid;

/* Graphics device ID ( 0 = zone not allocated, -ve = base zone) */
   extern int iztw [ MXZ + 1 ];

/* Window (WC) */
   extern double ztw [ MXZ + 1 ] [ 4 ];

/* Viewport (NDC) */
   extern double ztv [ MXZ + 1 ] [ 4 ];

/* Current pen */
   extern int npc;

/* Text font */
   extern int nftx;

/* Text precision */
   extern int nptx;

/* Text height */
   extern double htx;

/* Text aspect ratio */
   extern double artx;

/* Text up vector */
   extern double xuptx, yuptx;
/* (n.b. up vector is unit length) */

/* Text spacing */
   extern double sptx;

/* Text justification */
   extern char ctxj [ 3 ];

/* Text buffer */
#define LTEXT 120
   extern char ctext [ LTEXT + 1 ];

/* Length of current string */
   extern int ntext;

/* Plotting position for current string */
   extern double xtext, ytext;

/* Current polyline */
#define LPOLY 100
   extern double xpoly [ LPOLY ], ypoly [ LPOLY ];

/* Length of current polyline */
   extern int npoly;

/* Current pen position (device coordinates) */
   extern double xcurd, ycurd;

#ifdef __cplusplus
}
#endif

#endif
