#ifndef tcsCMD_PARALLEL_H
#define tcsCMD_PARALLEL_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcsCMD_PARALLEL.h,v 1.153 2005/08/02 12:52:45 vltsccm Exp $"
*
* who       when      what
* --------  --------  --------------------------------------------------------
* rkarban 2002-02-03 created
*/

/******************************************************************************
 * Declaration of the tcsCMD_PARALLEL class. This class serves to send a list
 * of commands in parallel (various commands to various targets) and collect
 * all the replies.
 *-----------------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

/*
 * eccs and evh includes
 */
#include <eccsErr.h>
#include <evhOBJ_CALLBACK.h>
#include <evhCOMMAND.h>
#include <evhTASK.h>
#include <fndLIST.h>

/*
 * module includes
 */
#include "tcs.h"
#include "tcsCMD_HANDLER.h"

class tcsCMD_PARALLEL : public evhSIMPLE_TASK
{
  public:
    tcsCMD_PARALLEL();
    virtual ~tcsCMD_PARALLEL();

    virtual ccsCOMPL_STAT    SetCompletedCB(evhCALLBACK &cb);
    virtual ccsCOMPL_STAT    SetErrorCB(evhCALLBACK &cb);

    virtual ccsCOMPL_STAT    SetTimeoutCB(evhCALLBACK &cb, const ccsTIMEVAL &timeout);
    virtual ccsCOMPL_STAT    SetTimeout(const ccsTIMEVAL &timeout);
    virtual ccsTIMEVAL       GetTimeout() const;

    virtual ccsCOMPL_STAT    AddMessage(const msgMESSAGE &msg);
    virtual ccsCOMPL_STAT    Send();

    virtual ccsCOMPL_STAT    Reset();
    virtual ccsCOMPL_STAT    Init();
    virtual const vltLOGICAL GetStatus() const;

    virtual const vltINT32   GetNumCmds() const;
    virtual const vltINT32   GetNumWait() const;

  protected:
    virtual const fndLIST    &GetCmdList() const;
    virtual ccsCOMPL_STAT    RunCompleted(const msgMESSAGE &);
    virtual ccsCOMPL_STAT    RunError(const msgMESSAGE &);

    virtual evhCB_COMPL_STAT ReplyCB(msgMESSAGE &, void *);
    virtual evhCB_COMPL_STAT ErrReplyCB(msgMESSAGE &, void *);
    virtual evhCB_COMPL_STAT TimeoutCB(msgMESSAGE &, void *);

  private:
    vltINT32     numWait;        // number of replies still pending
    ccsTIMEVAL   timeout;        // overall timeout on waiting for replies

    vltINT32     numCmds;        // number of commands handled in parallel
    fndLIST      cmdList;        // list of commands handled in parallel
    fndLIST      msgList;        // list of msg handled in parallel

    evhCALLBACK *completedCB;    // user defined reply handler
    evhCALLBACK *errorCB;        // user defined error reply handler
    evhCALLBACK *timeoutCB;      // user defined timeout handler
};

#endif // !tcsCMD_PARALLEL_H
