#ifndef tcsDefines_H
#define tcsDefines_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcsDefines.h,v 1.153 2005/08/02 12:52:41 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban 2001-03-26 change TCS dictionary, alias to ALL
* rkarban 2000-03-02 additional TCSID, TARGET_TCS definitions
* gchiozzi  03/06/97  Added macros for compile time ID
* gchiozzi   03/06/97  Added definitions for ASM_TCS and AUX_TCS
* kwirenst  27/04/97  Added defines for telescope number
* rkarban   17/04/97  add define tcsTRK_DUT_BACKUP_FILE
* gchiozzi  21/02/97  Added defines for TCS env vars
* rkarban   12/02/97  add define tcsTRK_PC_BACKUP_FILE
* nttmgr  04/12/96  Added defines for SETUP FILE tcs global Dictionary and Alias table
* gchiozzi  08/11/96  Moved telescope site constant in tcs.db
* gchiozzi  01/10/96  Added telescope site constants
* gchiozzi  18/08/95  Fixed bug in define name
* gchiozzi  20/07/95  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* 
 *  Defines for the target telescope.
 *  The default target is VLT
 *  Up to now VLT, NTT, ASM, ATS, SID, TP6
 */

#define VLT_TCS   1
#define NTT_TCS   2
#define ASM_TCS   3
#define ATS_TCS   4
#define SID_TCS   5
#define TP6_TCS   6 /* 3.6M telescope */
#define VST_TCS   7 /* VLT Survey Telescope */
#define VISTA_TCS 8

#ifndef TARGET_TCS
#warning TARGET_TCS not defined. Using default value VLT_TCS 
#define  TARGET_TCS VLT_TCS
#endif

/* 
 *  Defines for telescope number (to be used for var. TCSID).
 *  The default is VLT Control Model
 */

#define VLT_CM   0
#define VLT_UT1  1
#define VLT_UT2  2
#define VLT_UT3  3
#define VLT_UT4  4

#define ASM_PL 1 /* ASM Paranal */
#define ASM_LS 2 /* ASM La Silla */

#define VLTI_AT_CM 0 /* VLTI Auxiliary Telscopes */
#define VLTI_AT1   1
#define VLTI_AT2   2
#define VLTI_AT3   3

#define VLTI_SID_CM 0 /* VLTI Siderostats */
#define VLTI_SID1   1
#define VLTI_SID2   2

#define VLT_ST_CM 0 /* VLT Survey Telescope */
#define VLT_ST1   1

#ifndef TCSID
#define  TCSID VLT_CM
#endif

#if TARGET_TCS == ATS_TCS

#define PARTOP 0
#define PARMMB 1
#define LIEGE  2

#ifndef AT_LOCALE
#define AT_LOCALE PARTOP
#endif

#endif

/*
 * Macro to free an object and invalidate its pointer
 */
#define DELETE_OBJECT(object) do                        \
                                  {                     \
                                  if (object != NULL)   \
                                      {                 \
                                      delete object;    \
                                      object = NULL;    \
                                      }                 \
                                  } while (0)

/*
 * the maximum buffer size for printf buffers used in tcs
 * used to check for overflows
 */

#define tcsPRINTF_BUFFER_SIZE 1024

/*
 * Macros used to build the targetTCS string Id
 */

/* Support macros to handle parameters substitution */
#define tcsREPLACE_IT(date,time,targ) "@(#) $TCS Compile Id - Date: " date " Time: " time " Target: " #targ " $"
#define tcsTARGET_IT(date,time,targ) tcsREPLACE_IT(date,time,targ)

/* Real macro to be used in the code */
/* 
 * It MUST be called in the following way:
 *      tcsTCS_RCSID("@(#) $Id: tcsDefines.h,v 1.153 2005/08/02 12:52:41 vltsccm Exp $")
 * to be concatenated with rcsId and with version strings
 */
#define tcsTCS_RCSID(id)                                                                         \
   static const char *rcsId = id tcsTARGET_IT(__DATE__, __TIME__, TARGET_TCS);                         \
   static const void *use_rcsId = ((const void)&use_rcsId,(const void *) &rcsId);

/*
 * Module name
 */
#define tcsMOD "tcs"

/* Defines for TCS environment variables and default values */
/* for their contents                                       */

/* Root point for TCS database branch */
#define tcsDBPOINT     "TCS_DBPOINT"
#define tcsDBPOINT_DEF ":Appl_data:TCS"

/* Environment where TCS is running */
#define tcsENVNAME     "TCS_ENVNAME"

/* Contents of message buffer for standard commands used globally */
/* and to be propagated to prsAction                              */
#define tcsMSG_BUFF_ALL "ALL"

/* Standard OK message for replies */
#define tcsOK_MSG        "OK"

/* File names for SETUP FILE tcs global Dictionary and Alias table */
#define tcsSETUP_FILE_DICTIONARY "ALL"
#define tcsSETUP_FILE_ALIAS "ALL"

/* 
 *a constant shared by trk and trkws; this has to be moved to a common
 * trk module some time !!!!
 */

#define tcsTRK_PC_BACKUP_FILE "trkPcData.dat"
#define tcsTRK_DUT_BACKUP_FILE "trkDut.dat"
#define tcsTRK_SETROT_BACKUP_FILE "trkSetrot.dat"

#define tcsCMD_LOGFITS "LOGFITS"

#define tcsDB_TRANSFER_DEST_ENV            "destEnv"
#define tcsDB_TRANSFER_LCU_SERVER          "lcuServer"
#define tcsDB_TRANSFER_TIMEOUT             "timeout"
#define tcsDB_TRANSFER_CMDS                "commands"

#endif /*!tcsDefines_H*/
