#ifndef tcsVcc_H
#define tcsVcc_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcsVcc.h,v 1.153 2005/08/02 12:52:43 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  30/09/96  Removed tcsH2a_d and tcsH2f_d functions (Paranal's latitude was embedded)
* gchiozzi  30/09/96  Added new check functions for HHMMSS.TTT and DDMMSS.TTT formats
* vltsccm   29/08/94  created
*/

/************************************************************************
*  Conversion functions for geometrical representations 
*----------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#include "slamac.h"
#include "ccs.h"

/*
 * function prototypes
 */

#ifndef FALSE
#define FALSE 0
#define TRUE  1
#endif

typedef  enum {tcsEpochB1950, tcsEpochJ2000} tcsEPOCH_SYSTEM;
typedef  enum {tcsCoordMean, tcsCoordApparent} tcsCOORD_TYPE;


void tcsH2a(double p,double h,double d,double *al,double *az);
void tcsH2f(double ph,double h,double d,double *p);

void tcsSplitHms(double angle,double *h,double *m,double *s);
void tcsSplitSec(double sec,double *h,double *m,double *s);

int    tcsCheckHms(double angle);
int    tcsCheckDms(double angle);

double tcsHms2rad (double angle);
double tcsDms2rad (double angle);
double tcsRad2Hms (double angle);
double tcsRad2Dms (double angle);
double tcsD2r (double deg);
double tcsR2d (double rad);

void tcsJ2B (double alphaIn,
	     double deltaIn,
	     double epochIn,
	     double equinoxIn,
	     double pmaIn,
	     double pmdIn,
	     double radvelIn,
	     double parallaxIn,
	     double epochObserved,
	     double equinoxObserved,
	     double *alphaOut,
	     double *deltaOut,
	     double *pmaOut,
	     double *pmdOut,
	     double *radVelOut,
	     double *parallaxOut);

void tcsB2J (double alphain,
	     double deltaIn,
	     double epochIn,
	     double equinoxIn,
	     double pmaIn,
	     double pmdIn,
	     double radvelIn,
	     double parallaxIn,
	     double epochObserved,
	     double equinoxObserved,
	     double *alphaMean,
	     double *deltaMean,
	     double *pmaOut,
	     double *pmdOut,
	     double *radVelOut,
	     double *parallaxOut);

void tcsCoord2AltAz (double alpha,
		     double delta,
		     tcsEPOCH_SYSTEM epochSystem,
		     double epoch,
		     double equinox,
		     double pma,
		     double pmd,
		     double radvel,
		     double parallax,
		     tcsCOORD_TYPE coordType,
		     double envTemp,
		     double envPressure,
		     double envHumidity,
		     double envLapseRate,
		     double siteLat,
		     double siteLong,
		     double siteHeight,
		     double sitePolmx,
		     double sitePolmy,
		     double timeDut1,
		     double wavelength,
		     double mjd,
		     double *alt,
		     double *az);

void  tcsAltAz2Coord (double alt, 
		      double az, 
		      tcsEPOCH_SYSTEM epochSystem,
		      double epoch,
		      double equinox,
		      double pma,
		      double pmd,
		      double radvel,
		      double parallax,
		      tcsCOORD_TYPE coordType,
		      double envTemp,
		      double envPressure,
		      double envHumidity,
		      double envLapseRate,
		      double siteLat,
		      double siteLong,
		      double siteHeight,
		      double sitePolmx,
		      double sitePolmy,
		      double timeDut1,
		      double wavelength,
		      double mjd,
		      double *alphaOut,
		      double *deltaOut);

#ifdef __cplusplus
}
#endif


#endif /*!tcsVcc_H*/
