#ifndef TIF_H
#define TIF_H

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tif.h,v 1.153 2005/07/25 11:18:53 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jargomed 2005-03-30 added tifDATA_IA_FWHM define
* jargomed 2002-09-05 added new asm data (by name)
* rkarban 2001-11-20 add data item OBSWAVELEN
* gchiozzi 2000-06-26 Added defines for new entries
* gchiozzi  14/10/99  Added tifDATA_ASM_PRESSURE
* rkarban  04/02/99  align with tcsPUBLIC db
* rabuter  13/12/98  Added windspeed and winddir
* gchiozzi  27/04/98  Added handling of angle on sky for guide probe technical CCD.
* gchiozzi  16/04/98  Added RA2000 and dec2000 in degrees (spr. 970502)
* gchiozzi  16/04/98  Added define for tifDATA_ADA_ANGLE
* awalland  03/11/97  Added foculen, focuscale and installed (SPR970397)
* gchiozzi  10/09/97  Changed the prototype of tifGetByName() and tifAttachEvent()
* gchiozzi  10/09/97  Replaced structures with named data with string defines

* tphan     31/05/97  Added new items: ADA_POSANG
*                     Use measured status value, not setup (awa)
* gchiozzi  23/04/97  Updated units in comments
* gchiozzi  09/04/97  Added new items: ROT_ANGLE, PROBE_X and PROBE_Y
* awalland  13/03/97  Added NAMED data values required for FITS header
* awalland  12/03/97  Added inclusion of tcs and msw conversion functions
* awalland  11/03/97  Added tifGetFitsStart and tifGetFitsEnd prototypes
* gchiozzi  16/01/97  Moved here from tifEventLib definition of values for tifEventInfo array
* gchiozzi  16/01/97  Moved here from tifDataQueryLib definition of values for tifDataInfo array
* gchiozzi  06/11/96  Added tifDATA_FOCUS to retrieve current focus station
* gchiozzi  05/11/96  Error stack made optional. Added check for multiple calls.
* nfiebig   11/01/96  changed comments for RA and dec "... for J2000"
* gchiozzi  06/12/95  Added some comments
* gchiozzi  01/12/95  Added inclusion of new file tifDefines.h
* nfiebig   07/08/95  created
*/

/************************************************************************
 * Public interface definition of the TCS Interface (tif)
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include <ccs.h>
#include <evt.h>

#include "eccsTestTools.h"
#include "tcs.h"
#include "tcsVcc.h"
#include "msw.h"
#include "tifDefines.h"

/*
 * Typedes with the definition of storage type
 * for data and event identification names
 */
typedef vltBYTES32  tifNAME;
typedef vltBYTES256 tifDB_ADDR;

/*
 * Defines for named data items (see tifGetByName)
 * These defines allow to select data items to be read by name.
 * A database table contains the mapping from names to database
 * symbolic address and data types.
 */
#define tifDATA_LST            "tifDATA_LST"
#define tifDATA_UTC            "tifDATA_UTC"
#define tifDATA_HA             "tifDATA_HA"
#define tifDATA_RA             "tifDATA_RA"
#define tifDATA_DEC            "tifDATA_DEC"
#define tifDATA_RA2000         "tifDATA_RA2000"
#define tifDATA_DEC2000        "tifDATA_DEC2000"
#define tifDATA_RA2000DEG      "tifDATA_RA2000DEG"
#define tifDATA_DEC2000DEG     "tifDATA_DEC2000DEG"
#define tifDATA_OBSWAVELEN     "tifDATA_OBSWAVELEN"
#define tifDATA_ALT            "tifDATA_ALT"
#define tifDATA_AZ             "tifDATA_AZ"
#define tifDATA_ALT_REF        "tifDATA_ALT_REF"
#define tifDATA_AZ_REF         "tifDATA_AZ_REF"
#define tifDATA_AIRMASS        "tifDATA_AIRMASS"
#define tifDATA_PRLTIC         "tifDATA_PRLTIC"
#define tifDATA_DVELRA         "tifDATA_DVELRA"
#define tifDATA_DVELDEC        "tifDATA_DVELDEC"
#define tifDATA_LONGI          "tifDATA_LONGI"
#define tifDATA_LATI           "tifDATA_LATI"
#define tifDATA_LEVEL          "tifDATA_LEVEL"
#define tifDATA_REMTIME        "tifDATA_REMTIME"
#define tifDATA_REMLIMIT       "tifDATA_REMLIMIT"
#define tifDATA_TRACK          "tifDATA_TRACK"
#define tifDATA_FOCUS          "tifDATA_FOCUS"
#define tifDATA_TELESCOP       "tifDATA_TELESCOP"
#define tifDATA_VERSION        "tifDATA_VERSION"
#define tifDATA_INSTALLED      "tifDATA_INSTALLED"
#define tifDATA_OPER           "tifDATA_OPER"
#define tifDATA_FOCULEN_NA     "tifDATA_FOCULEN_NA"
#define tifDATA_FOCULEN_CA     "tifDATA_FOCULEN_CA"
#define tifDATA_FOCULEN_CO     "tifDATA_FOCULEN_CO"
#define tifDATA_FOCUSCALE_NA   "tifDATA_FOCUSCALE_NA"
#define tifDATA_FOCUSCALE_CA   "tifDATA_FOCUSCALE_CA"
#define tifDATA_FOCUSCALE_CO   "tifDATA_FOCUSCALE_CO"
#define tifDATA_FOCU_VALUE     "tifDATA_FOCU_VALUE"
#define tifDATA_DOME_STATUS    "tifDATA_DOME_STATUS"
#define tifDATA_AG_STATUS      "tifDATA_AG_STATUS"
#define tifDATA_AG_RA          "tifDATA_AG_RA"
#define tifDATA_AG_DEC         "tifDATA_AG_DEC"
#define tifDATA_ROT_ANGLE      "tifDATA_ROT_ANGLE"
#define tifDATA_PROBE_X        "tifDATA_PROBE_X"
#define tifDATA_PROBE_Y        "tifDATA_PROBE_Y"
#define tifDATA_PROBE_RA       "tifDATA_PROBE_RA"
#define tifDATA_PROBE_DEC      "tifDATA_PROBE_DEC"
#define tifDATA_PROBE_IN_POS   "tifDATA_PROBE_IN_POS"
#define tifDATA_PROBE_PPOS     "tifDATA_PROBE_PPOS"
#define tifDATA_ADA_POSANG     "tifDATA_ADA_POSANG"
#define tifDATA_ADA_ANGLE      "tifDATA_ADA_ANGLE"
#define tifDATA_CCD_ON_SKY     "tifDATA_CCD_ON_SKY"
#define tifDATA_ASM_SEEING     "tifDATA_ASM_SEEING"
#define tifDATA_ASM_TAU0       "tifDATA_ASM_TAU0"
#define tifDATA_ASM_THETA0     "tifDATA_ASM_THETA0"
#define tifDATA_ASM_RFLRMS     "tifDATA_ASM_RFLRMS"
#define tifDATA_ADC_SEP        "tifDATA_ADC_SEP"
#define tifDATA_TEMP_SER_1     "tifDATA_TEMP_SER_1"
#define tifDATA_TEMP_SER_2     "tifDATA_TEMP_SER_2"
#define tifDATA_TEMP_SER_3     "tifDATA_TEMP_SER_3"
#define tifDATA_TEMP_SER_4     "tifDATA_TEMP_SER_4"
#define tifDATA_TEMP_CNT_1     "tifDATA_TEMP_CNT_1"
#define tifDATA_TEMP_CNT_2     "tifDATA_TEMP_CNT_2"
#define tifDATA_TEMP_FLX_1     "tifDATA_TEMP_FLX_1"
#define tifDATA_TEMP_FLX_2     "tifDATA_TEMP_FLX_2"
#define tifDATA_TEMP_FLX_3     "tifDATA_TEMP_FLX_3"
#define tifDATA_TEMP_FLX_4     "tifDATA_TEMP_FLX_4"
#define tifDATA_ASM_WINDSPEED  "tifDATA_ASM_WINDSPEED"
#define tifDATA_ASM_WINDDIR    "tifDATA_ASM_WINDDIR"
#define tifDATA_ASM_PRESSURE   "tifDATA_ASM_PRESSURE"
#define tifDATA_M1_CORRECTED   "tifDATA_M1_CORRECTED"
#define tifDATA_M2_CORRECTED   "tifDATA_M2_CORRECTED"
#define tifDATA_AO_CMD_STATE   "tifDATA_AO_CMD_STATE"
#define tifDATA_AO_TARGET      "tifDATA_AO_TARGET"
#define tifDATA_AO_CUR_IND_AVG "tifDATA_AO_CUR_IND_AVG"
#define tifDATA_CHOP_STATUS    "tifDATA_CHOP_STATUS"
#define tifDATA_CHOP_STRT_TIME "tifDATA_CHOP_STRT_TIME"
#define tifDATA_CHOP_STOP_TIME "tifDATA_CHOP_STOP_TIME"
#define tifDATA_CHOP_FREQ      "tifDATA_CHOP_FREQ"
#define tifDATA_CHOP_THROW     "tifDATA_CHOP_THROW"
#define tifDATA_CHOP_POI       "tifDATA_CHOP_POI"
#define tifDATA_CHOP_PVR       "tifDATA_CHOP_PVR"
#define tifDATA_CHOP_TPA       "tifDATA_CHOP_TPA"
#define tifDATA_CHOP_POSANG    "tifDATA_CHOP_POSANG"
#define tifDATA_CHOP_OFFSET    "tifDATA_CHOP_OFFSET"
#define tifDATA_ASM_WINDSPEED  "tifDATA_ASM_WINDSPEED"
#define tifDATA_ASM_WINDDIR    "tifDATA_ASM_WINDDIR"
#define tifDATA_ASM_RHUM       "tifDATA_ASM_RHUM"
#define tifDATA_ASM_TEMP       "tifDATA_ASM_TEMP"
#define tifDATA_MOON_RA        "tifDATA_MOON_RA"
#define tifDATA_MOON_DEC       "tifDATA_MOON_DEC"
#define tifDATA_M1_TEMP        "tifDATA_M1_TEMP"
#define tifDATA_AO_FOCUS_TERM  "tifDATA_AO_FOCUS_TERM"
#define tifDATA_TARG_ALPHA       "tifDATA_TARG_ALPHA"
#define tifDATA_TARG_DELTA       "tifDATA_TARG_DELTA"
#define tifDATA_TARG_EPOCH       "tifDATA_TARG_EPOCH"
#define tifDATA_TARG_EPOCHSYSTEM "tifDATA_TARG_EPOCHSYSTEM"
#define tifDATA_TARG_EQUINOX     "tifDATA_TARG_EQUINOX"
#define tifDATA_TARG_PMA         "tifDATA_TARG_PMA"
#define tifDATA_TARG_PMD         "tifDATA_TARG_PMD"
#define tifDATA_TARG_RADVEL      "tifDATA_TARG_RADVEL"
#define tifDATA_TARG_PARALLAX    "tifDATA_TARG_PARALLAX"
#define tifDATA_TARG_COORDTYPE   "tifDATA_TARG_COORDTYPE"
#define tifDATA_IA_FWHM        "tifDATA_IA_FWHM"

/*
 * Defines for named event items (see tifAttachEvent)
 * These defines allow to select data items to be read by name.
 * A database table contains the mapping from names to database
 * symbolic address and event types.
 */
#define tifEVENT_TRACKING_LOST "tifEVENT_TRACKING_LOST"
#define tifEVENT_GUIDING_LOST  "tifEVENT_GUIDING_LOST"
#define tifEVENT_REMTRACK_LOW  "tifEVENT_REMTRACK_LOW"
#define tifEVENT_AO_FOCUS_TERM_CHANGED  "tifEVENT_AO_FOCUS_TERM_CHANGED"

/*
 * Telescope Position Data Structure (actual position)
 */
struct tifPOSITION 
    {
    vltDOUBLE HA;    // hour angle
    vltDOUBLE RA;    // right ascension due to mean place of J2000
    vltDOUBLE dec;   // declination due to mean place of J2000
    vltDOUBLE alt;   // altitude
    vltDOUBLE az;    // azimuth
    };
    
/*
 * Module States Data Structure
 */
struct tifSTATES
    {
    vltINT32 tcsState;      // TCS global state
    vltINT32 tcsSubstate;   // TCS sub-state
    };
    
/*
 * Telescope Status Data Structure
 */
struct tifSTATUS
    {
    // TBD
    };
    
/*
 * Common TCS Library init function
 */
ccsCOMPL_STAT tifInitLibs(ccsERROR *error = NULL);

/*
 * TCS Data Query Library functions
 */
ccsCOMPL_STAT tifGetPosition(tifPOSITION *position, ccsERROR *error = NULL);
ccsCOMPL_STAT tifGetStates(tifSTATES *states, ccsERROR *error = NULL);
ccsCOMPL_STAT tifGetStatus(tifSTATUS *status, ccsERROR *error = NULL);
ccsCOMPL_STAT tifGetByName(char *nameIndex, char *buffer, dbTYPE *type = NULL,
                           ccsERROR *error = NULL);
ccsCOMPL_STAT tifGetFitsStart(const char *imageFile, ccsERROR *error = NULL);
ccsCOMPL_STAT tifGetFitsEnd(const char *imageFile, ccsERROR *error = NULL);

/*
 * TCS Event Library functions
 */
ccsCOMPL_STAT tifAttachEvent(char *nameIndex, evtEVENT_ID *eventId,
                             ccsERROR *error = NULL);
ccsCOMPL_STAT tifDetachEvent(evtEVENT_ID *eventId, ccsERROR *error = NULL);

#endif /*!TIF_H*/
