#ifndef TIF_INTERNAL_H
#define TIF_INTERNAL_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tifInternal.h,v 1.153 2005/07/25 11:18:53 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi 2000-06-23 Added Recover method
* gchiozzi  21/10/97  Added new column for destination command name
* gchiozzi  21/04/97  Removed temporary ObjStatus()
* nfiebig   07/08/95  created
*/

/************************************************************************
 * Internal interface definition of the TCS Interface (tif)
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

// system includes

#include "fndSTRING.h"
#include "fndASSOCIATION.h"
#include "fndINT.h"
#include "fndDICTIONARY.h"

// includes for ECCS and Evh Toolkit
#include "ECCS.h"
#include "ECCS.h"
#include "eccsTestTools.h"
#include "evhHANDLER.h"
#include "evhCOMMAND.h"
#include "evhSTD_COMMANDS.h"
#include "evhTIMEOUT.h"
#include "evhErrors.h"
#include "evhStates.h"
#include "err.h"

// local includes
#include "tif.h"
#include "tifErrors.h"


// ******************************************
// * Constants and Literals
// ******************************************

// ******************************************
// * Definition of the Routing Table Entry
// ******************************************

struct tifROUTING
    {
    msgCMD      command;    // command name
    msgCMD      newCommand; // new remapped command name
    ccsENVNAME  env;        // destination environment name
    ccsPROCNAME process;    // destination process name
    vltINT32    timeout;    // reply timeout in seconds
    };

// ******************************************
// * Definition of the tifMAIN_TASK class
// ******************************************

class tifMAIN_TASK : public evhSTD_COMMANDS
    {
    public:
        tifMAIN_TASK();
        ~tifMAIN_TASK();
        evhCB_COMPL_STAT InitCB(msgMESSAGE &msg, void *userData);
        evhCB_COMPL_STAT ExitCB(msgMESSAGE &msg, void *userData);
        evhCB_COMPL_STAT RouteCommand(msgMESSAGE &msg, tifROUTING *route);
        evhCB_COMPL_STAT GenericCB(msgMESSAGE &msg, void *userData);
        evhCB_COMPL_STAT PrintTable(msgMESSAGE &msg, void *userData);
        evhCB_COMPL_STAT GetPendCB(msgMESSAGE &msg, void *userData);

        ccsCOMPL_STAT Init();
        ccsCOMPL_STAT Recover();

    private:
        tifROUTING    *table;                 // routing table
        vltINT32      numCommands;            // no. entries in routing table
        ccsCOMPL_STAT LoadCommands();
    };

// ******************************************
// * Definition of the tifCOMMAND class
// ******************************************

class tifCOMMAND : public evhCOMMAND
    {
    public:
        tifCOMMAND (msgMESSAGE &cmd, tifROUTING *route);
        ~tifCOMMAND();
        evhCB_COMPL_STAT HandleReply      (msgMESSAGE &reply, tifROUTING *route);
        evhCB_COMPL_STAT HandleErrorReply (msgMESSAGE &reply, tifROUTING *route);
        evhCB_COMPL_STAT HandleTimeout    (msgMESSAGE &reply, tifROUTING *route);
    private:
        msgMESSAGE userCommand;     // original command received from user
    };

// ******************************************
// * Global Variables
// ******************************************

extern dbSYMADDRESS tifDbRoot;
extern eccsDB_INT32 *tifCmdsInProgress;

// ******************************************
// * Prototypes
// ******************************************

#endif /* !TIF_INTERNAL_H */
