#ifndef TIMER_H
#define TIMER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* timer.h
*
* "@(#) $Id: timer.h,v 2.35 2005/09/13 16:35:11 vltsccm Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* B.GILLI    29/04/93  Preliminary version
* B.GILLI    27/11/93  Elaborated version for ccs design document.
* B.GILLI    08/12/93  modified timsSleep in timerSleep, and timerSendMessage
*                      in timerSendCommand and timerSendReply
* T.HERLIN   03/08/94  splitted tims.h into tims.h and timer.h
* B.GILLI    23/09/94  moved here #include msg.h from tims.h.
* B.GUSTAFSSON 28/11/94 Including specific CCS/LCC include files
* B.GILLI    29/11/94  modified timerScheduleTask as msgScheduleProcess
* B.GUSTAFSSON 01/12/94 Added C++ support
* B.GUSTAFSSON 02/12/94 Changed timerScheduleTask to timerScheduleProcess 
* B.GILLI    06/04/00  Added proto of non blocking version of timerSendReplyNV for log. 
*/


/************************************************************************
 * CCS/TIME SYSTEM - TIMER Interface File
 */

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * header files
 */
#include "tims.h"
#include "msg.h"

/* 
 * constants 
 */
/*none*/

/*
 * Message constants
 */

#define timerFREE_STRING	           "Free"
#define timerBUSY_STRING	           "Busy"

#define timerSTARTED_STRING	           "Started"
#define timerNOT_STARTED_STRING	           "Not started"

#define timerONCE_IMMEDIATE_STRING         "Once Immediate"
#define timerONCE_AFTER_TIME_GATE_STRING   "Once after Time"
#define timerONCE_AFTER_HW_GATE_STRING     "Once after Gate"
#define timerPER_IMMEDIATE_STRING          "Periodic Immediate"
#define timerPER_AFTER_TIME_GATE_STRING    "Periodic after Time"
#define timerPER_AFTER_HW_GATE_STRING      "Periodic after Get"
#define timerAFTER_TIME_GATE_STRING        "After Time"
#define timerAFTER_HW_GATE_STRING          "After Gate"
#define timerUTC_STRING                    "On UTC"
#define timerPER_AFTER_UTC_STRING          "Periodic after UTC"

#define timerSCHEDULE_ONCE_STRING          "Once"
#define timerSCHEDULE_PER_STRING           "Periodic"
#define timerSCHEDULE_UTC_STRING           "UTC"
#define timerSCHEDULE_PER_AFTER_UTC_STRING "Periodic after UTC"


/* 
 * data types
 */

typedef enum {                      /* Mode of activation for task      */
                                    /* scheduling or message sending    */
    timerSCHEDULE_ONCE = 1,         /* One shot activation after delay  */
    timerSCHEDULE_PER,              /* Periodic activation after delay  */
    timerSCHEDULE_UTC,              /* One shot activation after UTC    */
    timerSCHEDULE_PER_AFTER_UTC     /* Periodic activation after UTC    */
} timerSCHEDULE_MODE;

/* 
 * -------- end of data type definitions 
 */
                       
/* 
 * macros 
 */

/* 
 * functions prototypes 
 */
/*  Interface subroutines to the "Time Scheduler"  :  */

ccsCOMPL_STAT  timerSleep (        /* Sleep, unit microseconds, resolution ? */
   vltUINT32          howlong,     /* Number of microseconds to SLEEP */
   ccsERROR           *error
   );

ccsCOMPL_STAT timerScheduleProcess( /* Schedule a task synchronised to  */
                                    /* Time Reference System            */
    const ccsENVNAME   envName,     /* Where to schedule                */
    const char         *runString,  /* Task entry name and parameters   */
    const ccsRUNOPT    *runOptions, /* VxWorks run-time options         */
    timerSCHEDULE_MODE mode,        /* Mode of activation               */
    const ccsTIMEVAL   *startTime,  /* Start time                       */
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT timerSendCommand(	    /* Send command to a given process  */
				    /* Time Reference System		*/
    timerSCHEDULE_MODE mode,        /* Mode of activation		*/
    const ccsTIMEVAL   *startTime,  /* Start time			*/
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    const msgCMD       command,     /* Command to be sent		*/
    msgCMDID           commandId,   /* User supplied identifier		*/
    const ccsENVNAME   destenv,     /* Where to send the command	*/
    const ccsPROCNAME  destproc,    /* To whom : process name		*/
    const char         *buffer,     /* Buffer				*/
    msgLENGTH          buflen,      /* Message length			*/
    msgCHECKFLAG       checkFlag,   /* Flag specifying checks to be done*/
    vltUINT32          *requestId,  /* Request identifier		*/
    ccsERROR           *error	    /* Returned error structure		*/
    );


ccsCOMPL_STAT timerSendReply(	    /* Send reply synchronised to	*/
				    /* Time Reference System		*/
    timerSCHEDULE_MODE mode,        /* Mode of activation		*/
    const ccsTIMEVAL   *startTime,  /* Start time			*/
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    const msgPROCESSID *destId,     /* Destination of the reply         */
    msgCMDID           commandId,   /* Command Id replied to	        */
    const msgCMD       command,     /* Command replied to 		*/
    const char         *buffer,     /* Buffer containing the reply      */
    msgLENGTH          buflen,      /* Buffer length. Max len = 8kB.    */
    vltLOGICAL         lastReply,   /* Flag to indicate if another      */
                                    /* reply follows                    */
    vltUINT32          *requestId,  /* Request identifier		*/
    ccsERROR           *error	    /* Returned error structure		*/
    );
/*
 * The next one is a "private" version to be used only by the logging system.
 */
ccsCOMPL_STAT timerSendReplyNV(	    /* Send reply synchronised to	*/
				    /* Time Reference System		*/
    timerSCHEDULE_MODE mode,        /* Mode of activation		*/
    const ccsTIMEVAL   *startTime,  /* Start time			*/
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    const msgPROCESSID *destId,     /* Destination of the reply         */
    msgCMDID           commandId,   /* Command Id replied to	        */
    const msgCMD       command,     /* Command replied to 		*/
    const char         *buffer,     /* Buffer containing the reply      */
    msgLENGTH          buflen,      /* Buffer length. Max len = 8kB.    */
    vltLOGICAL         lastReply,   /* Flag to indicate if another      */
                                    /* reply follows                    */
    vltUINT32          *requestId,  /* Request identifier		*/
    ccsERROR           *error	    /* Returned error structure		*/
    );

ccsCOMPL_STAT timerCancelRequest(  /* cancel an existing request  */
   vltUINT32          requestId,   /*   Timer request to cancel  */
   ccsERROR           *error
   );
ccsCOMPL_STAT timerGetReport(      /* Get list of pending timer requests */
   char               **buffer,    /* where to put report    */
   vltUINT32          *size,       /* size of buffer used for report */
   ccsERROR           *error
   );

/* 
 * message definitions
 */


/* 
 * error definitions
 */

#ifdef MAKE_VXWORKS
#include "timerLCC.h"
#else

#ifdef CCS_LIGHT
#include "timerLite.h"
#endif

#include "timerCCS.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /*!TIMER_H*/



