#ifndef TPTSYSIC
#define TPTSYSIC

#ifdef __cplusplus
extern "C" {
#endif

/*
**  - - - - - - - - -
**   t p t s y s . h
**  - - - - - - - - -
**
**  TPOINT functions, macros and global data include file.
**
**  Last revision:   13 September 2000
**
**  Copyright 2000 P.T.Wallace.  All rights reserved.
*/

/*
**  -------------------
**  Function prototypes
**  -------------------
*/

#include "tpoint.h"

/*
**  ------------------
**  PLATFORM-DEPENDENT  -  sizes of TPOINT data arrays etc
**  ------------------
**
*/

#include "tptpfd.h"

/*
**  ------
**  CONSTS  -  constants
**  ------
*/

#define DPI   3.1415926535897932384626433832795028841971693993751
#define D2PI  6.2831853071795864769252867665590057683943387987502
#define DPI2  1.5707963267948966192313216916397514420985846996876
#define DH2R  0.26179938779914943653855361527329190701643078328126
#define DS2R  7.2722052166430399038487115353692196393452995355905E-5
#define DD2R  0.017453292519943295769236907684886127134428718885417
#define DAS2R 4.8481368110953599358991410235794797595635330237270E-6

/*
**  --
**  FP  -  file pointers
**  --
*/

/* Observation data input */
   extern FILE *fpinf;

/* Observation data output */
   extern FILE *fpoutf;

/* Report (log) file */
   extern FILE *fprept;

/* Command input */
   extern FILE *fpcmd;

/* Message output */
   extern FILE *fpmes;

/* Procedure library */
   extern FILE *fpprol;

/* Star catalogue */
   extern FILE *fpstar;

/* HELP library */
   extern FILE *fphl;

/*
**  -----
**  FILES  -  file names
**  -----
*/

/* Default procedure library */
   extern char iprol [ LSTRNG ];

/* Default star catalogue */
   extern char istarc [ LSTRNG ];

/* Root HELP library */
   extern char hlname [ LHL ];

/*
**  -------
**  MESSAGE  -  interrupt, message and report handling
**  -------
*/

/* Pointer to console interrupt function */
   extern int ( *abortf ) ( int );

/* Pointer to message handler function */
   extern int ( *mesrep ) ( FILE*, FILE*, int, int, int, char* );

/* Message suppression codes for message and report devices */
   extern int mgagm, mgagr;

/*
**  -----
**  DLIST  -  list of pointing observations
**  -----
*/

/* Append/replace flag:  TRUE = append */
   extern int apprep;

/*
** Header
*/

/* Flag (-1 = null list) */
   extern int ifflag;

/* Telescope latitude (corrected for polar motion) and functions */
   extern double telphi, sinphi, cosphi;

/* Caption */
   extern char capt [ LCAPT ];

/* Mount type */
   extern int mount;
                        /* 0 = unspecified        */
#define EQUAT 1         /* 1 = equatorial         */
#define ALTAZ 2         /* 2 = altazimuth         */
#define GIMBAL 3        /* 3 = generalized gimbal */

/* Rotation matrix, observed [az,el] to nominal mount [roll,pitch] */
   extern double ae2nm [ 3 ] [ 3 ];

/*
** List
*/

/* INDAT filename (as specified). */
   extern char inname [ LSTRNG ];

/* Observation status flag (-1 = EOF, 0 = inactive record, +1 = active) */
   extern int idflag [ LISTN ];

/* "Below pole" flag (TRUE = below pole) */
   extern int ibp [ LISTN ];

/* Star observed HA/Dec (i.e. as affected by refraction) */
   extern double soh [ LISTN ], sod [ LISTN ];

/* Star adjusted HA/Dec */
   extern double sah [ LISTN ], sad [ LISTN ];

/* Telescope adjusted HA/Dec */
   extern double tah [ LISTN ], tad [ LISTN ];

/* Telescope raw HA/Dec */
   extern double trh [ LISTN ], trd [ LISTN ];

/* Auxiliary readings (if relevant) */
   extern double aux [ LISTN ] [ 3 ];

/*
**  ---
**  CMD  -  command and status
**  ---
*/

/* Command line nearly raw */
   extern char cmdraw [ LSTRNG ];

/* Command line with leading spaces etc removed */
   extern char cmdlc [ LSTRNG ];

/* Same but converted to uppercase */
   extern char cmd [ LSTRNG ];

/* Status returned from command (' ' = OK) */
   extern int iokf;

/* Report length option (0 = normal, 1 = short, 2 = none) */
   extern int lrep;

/* Echo flag (true = echo procedure commands to screen) */
   extern int verify;

/*
**  -----
**  PROCS  -  procedures
**  -----
*/

/* Command lines */
   extern char cmndls [ LINES ] [ LSTRNG ];

/* Link stack pointer (current top; -1 = empty) */
   extern int lstakp;

/* Link stack:  lstak[lstakp] is current line */
   extern int lstak [ 20 ];

/*
**  -----
**  STARS  -  catalogue of pointing calibration stars
**  -----
*/

/* Star records (first 6 characters are star ID;  endmark is 'END') */
   extern char stars [ NSTMAX ] [ LSTAR ];

/*
**  -----
**  MODEL  -  pointing model
**  -----
*/

/* List of term numbers defining model (0 = end) */
   extern int model [ MAXTRM ];

/* Coefficient values */
   extern double coeffv [ MAXTRM ];

/* Float/fix flags (true = floating) */
   extern int cfloat [ MAXTRM ];

/* Chained/parallel flags (true = chained) */
   extern int cchain [ MAXTRM ];

/* Latest individual standard deviations (-ve = not available) */
   extern double coeffsd [ MAXTRM ];

/* Latest overall RMS and PSD (radians) */
   extern double skyrms, skypsd;

/* Ill-conditioning flag:  true if last fit required intervention */
   extern int illcon;

/* Refraction constants last used (radians) */
   extern double refa, refb;

/* Number of locally-defined terms */
   extern int nterml;

/* Number of non-generic (explicitly implemented) terms (local+standard) */
   extern int ntermx;

/* Number of terms in current repertoire (local+standard+generics) */
   extern int nterms;

/* Repertoire of term names (local then standard then generics) */
   extern char coeffn [ NTROOM ] [ 9 ];

/* Tokenized form of generic term name (same index as coeffn table) */
   extern char coform [ NTROOM ] [ 9 ];

/* Method:  1 = adjust telescope, 2 = adjust star */
   extern int method;

/* SVD w-matrix rejection tolerance */
   extern double svdtol;

/*
**  ------
**  GRAFIX  -  SGS graphics
**  ------
*/

/* Aspect ratio and default world coordinates of plotting zone */
#define AR 1.333333333333333333
#define XWL 0.0
#define XWH 20.0
#define YWL 0.0
#define YWH 15.0

/* Default name */
   extern char defwsn [ LWSN ];

/* Default pens, fonts, text precision */
   extern int npfrad, npcapd, npaxed, nplabd, npptsd, npoffd,
              nfcapd, nflabd,
              nprecd;

/* Default graphics hardcopy filename */
   extern char ghcfn [ LSTRNG ];

/* Current number of graphics devices open (index to zone lists) */
   extern int iwk;

/* Zone lists: */

/* Graphics device names, IDs of base zones and plotting zones */
   extern char sgswsn [ MAXWK ] [ LWSN ];
   extern int idbz [ MAXWK ], idz [ MAXWK ];

/* Something plotted flags */
   extern int plotd [ MAXWK ];

/* SGS pens: frame, caption, axes, labels, points, offscale */
   extern int npfra [ MAXWK ], npcap [ MAXWK ], npaxe [ MAXWK ],
              nplab [ MAXWK ], nppts [ MAXWK ], npoff [ MAXWK ];

/* SGS fonts: caption, labels */
   extern int nfcap [ MAXWK ], nflab [ MAXWK ];

/* SGS text precision */
   extern int nprec [ MAXWK ];

/* Bounds of plotting zone;  display surface is from (0,0) to (1,1) */
   extern double xlo, xhi, ylo, yhi;

/* Marker half height */
   extern double hhmkr;

/* Flag:  false = don't plot frame */
   extern int pltfra;

/* Flag:  false = don't plot caption */
   extern int pltcap;

/* Zone erasure option: -1 = screen, 0 = no erase, 1 = zone */
   extern int jclrng;

/*
**  -----
**  ALPHA  -  alphanumeric screen
**  -----
*/

/* Flag:  true = VT100-compatible screen in use */
   extern int vt100;

/* Top and bottom line numbers for scrolling region */
   extern int ltop, lbot;

/* Lines of interactive output this screenful */
   extern int lintop;

/* Interactive output enable/disable */
   extern int intop;

/*
**  ------
**  WORKSP  -  workspace
**  ------
*/

/* Scratch string */
   extern char scratch [ LSTRNG ];

#ifdef __cplusplus
}
#endif

#endif
