#ifndef trkwsDefines_H
#define trkwsDefines_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: trkwsDefines.h,v 1.166 2005/07/25 15:44:48 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban 2001-08-13 add autoguiding correction
* kwirenst  18/12/98  Added defines for _ALPHA_APP and _DELTA_APP
* rkarban  03/03/98  add ROTONSKY define
* rkarban  19/08/97  remove alias defines
* kwirenst  29/05/97  Added ":times.utc"
* rkarban   23/05/97  add setpolm definition
* gchiozzi  17/04/97  Added define for focus check time
* rkarban   15/03/97  add ALPHA_CURRENT_ATTRNAME & DELTA_CURRENT_NAME
* rkarban   14/02/97  add definition for pc backup file handling
* kwirenst  13/12/96  Added #define trkwsSETUP_ROTANGLE_ATTRNAME
* kwirenst  13/12/96  Added commands OFFSROT and OBJROT
* kwirenst  13/12/96  Added command SETROT
* kwirenst  24/09/96  Re-arranged sequence of defines and modified comments.
*                     No data changed!!
* gchiozzi  20/08/96  Added defines for other LCU axis attributes used
* gchiozzi  20/08/96  Added defines for actual RA/dec encoder position
* gchiozzi  13/08/96  Added defines for limit range on alt and az
* gchiozzi  29/05/96  Removed c++ style comments
* nfiebig   25/03/96  SETUP attribute adcSeparation replaced by 
*                     attribute imageShift in pointing branch
* nfiebig   20/03/96  SETUP attribute names addVelAlpha/Delta added
* kwirenst  07/06/95  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif


#define VLT_TCS   1
#define NTT_TCS   2

#ifndef TARGET_TCS
#define TARGET_TCS VLT_TCS
#endif

/* Module name */
#define trkwsMOD "trkws"

/*
 * maximum file line when reading files
 */
#define trkwsMAX_FILE_LINE 256

#define trkwsFOCUS_STRING_MAX_LEN 32

#define trkwsALARM_WARNING 1
#define trkwsALARM_NORMAL 0


/* maximum msg len in msg.h */
#define trkwsMAX_MSG_LEN (msgMAXLEN - sizeof(msgHEADER) - sizeof(msgHEADER2) - 1)

/* Max counter for repeated error      */
/* Some errors are not logged any more */
/* is repeated mor than this times     */
#define trkwsMAX_ERROR_COUNT 10

/* Limits for alt and az values (in radians) */
/* This should be checked with NTT           */   
#define trkwsMIN_ALT    0.0
#define trkwsMAX_ALT    M_PI_2
#define trkwsMIN_AZ     -M_PI
#define trkwsMAX_AZ     2*M_PI

/*
 * Defines for states values and names
 * These enum and string vector are used in the whole module
 * to identify states and substates.
 * The values must be coordinated with the standard states defined in 
 * evhtates.h
 */

#include "evhStates.h"

/* Local remapping of some standard states (just for convenience) */
#define trkwsSTATE_OFF             evhSTATE_OFF
#define trkwsSTATE_LOADED          evhSTATE_LOADED
#define trkwsSTATE_STANDBY         evhSTATE_STANDBY
#define trkwsSTATE_ONLINE          evhSTATE_ONLINE
#define trkwsSTATE_ERROR           evhSTATE_ERROR
#define trkwsSTATE_READY           evhSTATE_READY
#define trkwsSTATE_IDLE            evhSTATE_IDLE

/* New states. I start from 100 to be shure not to clash */
#define trkwsSTATE_STOPPING       101
#define trkwsSTATE_PRESETTING     103
#define trkwsSTATE_TRACKING       104

/* Defines for name states */
#define    trkwsSTATE_STR_OFF        evhSTATE_STR_OFF 
#define    trkwsSTATE_STR_LOADED     evhSTATE_STR_LOADED 
#define    trkwsSTATE_STR_STANDBY    evhSTATE_STR_STANDBY 
#define    trkwsSTATE_STR_ONLINE     evhSTATE_STR_ONLINE 
#define    trkwsSTATE_STR_READY      evhSTATE_STR_READY
#define    trkwsSTATE_STR_ERROR      evhSTATE_STR_ERROR 
#define    trkwsSTATE_STR_IDLE       evhSTATE_STR_IDLE

#define    trkwsSTATE_STR_STOPPING   "STOPPING"
#define    trkwsSTATE_STR_PRESETTING "PRESETTING"
#define    trkwsSTATE_STR_TRACKING   "TRACKING"

/* Error conditions for the position calculation routine */
#define trkwsPOS_OK            0
#define trkwsPOS_DB_ERROR      1
#define trkwsPOS_SYNC_ERROR    2
#define trkwsPOS_RANGE_ERROR   3
#define trkwsPOS_EMPTY_BUFFER  4

/*
 * Defines for command names
 */


/* Commands accepted by trkwsControl process */
#define trkwsSTOP_CMD      "STOP"
#define trkwsSTOPTRK_CMD   "STOP"
#define trkwsOFFSAD_CMD    "OFFSAD"
#define trkwsOFFSAA_CMD    "OFFSAA"
#define trkwsOFFSXY_CMD    "OFFSXY"
#define trkwsOFFSDEF_CMD   "OFFSDEF"
#define trkwsOFFSADG_CMD   "OFFSADG"
#define trkwsOBJSTAR_CMD   "OBJSTAR"
#define trkwsOBJFIX_CMD    "OBJFIX"
#define trkwsOBJNAME_CMD   "OBJNAME"
#define trkwsSETPMC_CMD    "SETPMC"
#define trkwsSTRTMC_CMD    "STRTMC"
#define trkwsSTOPMC_CMD    "STOPMC"
#define trkwsSETAV_CMD     "SETAV"
#define trkwsSETLAM_CMD    "SETLAM"
#define trkwsSETADC_CMD    "SETADC"
#define trkwsSETDUT_CMD    "SETDUT"
#define trkwsSETTERM_CMD    "SETTERM"
#define trkwsENATERM_CMD    "ENATERM"
#define trkwsDISTERM_CMD    "DISTERM"
#define trkwsNEWPCM_CMD     "NEWPCM"
#define trkwsSETUP_CMD      "SETUP"
#define trkwsMARK_CMD       "MARK"
#define trkwsREADASM_CMD    "READASM"
#define trkwsREADPOS_CMD    "READPOS"
#define trkwsREADSID_CMD    "READSID"
#define trkwsGETREM_CMD     "GETREM"
#define trkwsSETRLIM_CMD    "SETRLIM"
#define trkwsSETFOC_CMD     "SETFOC"
#define trkwsSETOAX_CMD     "SETOAX"
#define trkwsSETROT_CMD     "SETROT"
#define trkwsOFFSROT_CMD    "OFFSROT"
#define trkwsOBJROT_CMD     "OBJROT"
#define trkwsSETPOLM_CMD    "SETPOLM"
#define trkwsMKCIRC_CMD     "MKCIRC"
#define trkwsSETSITE_CMD    "SETSITE"

/* Commands send to other modules     */
/* (interface must be defined better) */

/* To Autoguiding                   */
#define agaPROBESTEP_CMD     "OFFSAD"

/*
 *    Standard OK reply
 *    Othere replies to commands
 */

#define trkwsOK_REPLY             "OK"
#define trkwsABORT_REPLY          "aborted"
#define trkwsREJECTED_REPLY       "rejected"
#define trkwsPRS_OK_ABORT_REPLY   "OK, superseded by valid equivalent command"
#define trkwsOFF_OK_ABORT_REPLY   "OK, superseded by valid equivalent command"
#define trkwsINIT_OK_ABORT_REPLY   "OK, superseded by valid equivalent command"


/*
**************************************************************************
 *    Database attribute names and aliases.
 *    ------------------------------------
 */

/*
 *     .... general
 */
#define trkwsDB_ENTRY_POINT           "trk"
#define trkwsDB_STATE                 "state"
#define trkwsDB_SUBSTATE              "substate"
#define trkwsDB_RTTLIMITALARM         "remTrkTimeAlarm"
#define trkwsDB_TARGETS_ADDR_ATTRNAME "targetsAddr"

#define trkwsDB_CONTROL_POINT         "control"
#define trkwsDB_CONTROL_STATE         "state"


#define trkwsDB_DATA_POINT      "data"

/*
 *    NOTE:  the following attributes are relative to  trk:data:
 */

/*
 *    .... reference position
 */
#define trkwsMAX_TIME_DIFF_ATTRNAME  ":position.maxTimeDiff"

#define trkwsREF_ALPHA_ATTRNAME  ":position:refPos.alpha"
#define trkwsREF_DELTA_ATTRNAME  ":position:refPos.delta"
#define trkwsREF_EPSYST_ATTRNAME  ":position:refPos.epochSystem"
#define trkwsREF_EPOCH_ATTRNAME  ":position:refPos.epoch"
#define trkwsREF_EQUINOX_ATTRNAME  ":position:refPos.equinox"
#define trkwsREF_PMA_ATTRNAME  ":position:refPos.pma"
#define trkwsREF_PMD_ATTRNAME  ":position:refPos.pmd"
#define trkwsREF_RVEL_ATTRNAME  ":position:refPos.radvel"
#define trkwsREF_PLX_ATTRNAME  ":position:refPos.parallax"
#define trkwsREF_CTYPE_ATTRNAME  ":position:refPos.coordType"
#define trkwsREF_ALT_ATTRNAME  ":position:refPos.alt"
#define trkwsREF_AZ_ATTRNAME  ":position:refPos.az"
#define trkwsREF_NAME_ATTRNAME  ":position:refPos.objName"


#define trkwsACT_POS_STATUS_ATTRNAME  ":position:actPos.posStatus"
#define trkwsACT_ALPHA_ATTRNAME  ":position:actPos.RA"
#define trkwsACT_DELTA_ATTRNAME  ":position:actPos.dec"
#define trkwsACT_ALPHA_ENC_ATTRNAME  ":position:actPos.RAEncoder"
#define trkwsACT_DELTA_ENC_ATTRNAME  ":position:actPos.decEncoder"
#define trkwsACT_ALPHA_LT2000_ATTRNAME  ":position:actPos.RALocalTel2000"
#define trkwsACT_DELTA_LT2000_ATTRNAME  ":position:actPos.decLocalTel2000"
#define trkwsACT_HA_ATTRNAME     ":position:actPos.HA"
#define trkwsACT_ALPHA2000_ATTRNAME  ":position:actPos.RA2000"
#define trkwsACT_DELTA2000_ATTRNAME  ":position:actPos.dec2000"
#define trkwsACT_ALPHA2000DEG_ATTRNAME  ":position:actPos.RA2000Deg"
#define trkwsACT_DELTA2000DEG_ATTRNAME  ":position:actPos.dec2000Deg"
#define trkwsACT_ALPHA_CURRENT_ATTRNAME  ":position:actPos.RACurrent"
#define trkwsACT_DELTA_CURRENT_ATTRNAME  ":position:actPos.decCurrent"
#define trkwsACT_ALT_ATTRNAME  ":position:actPos.alt"
#define trkwsACT_AZ_ATTRNAME  ":position:actPos.az"
#define trkwsACT_ROTRAD_ATTRNAME  ":position:actPos.rotRad"
#define trkwsACT_ROTDEG_ATTRNAME  ":position:actPos.rotDeg"
#define trkwsACT_ADRAD_ATTRNAME  ":position:actPos.adRad"
#define trkwsACT_ADDEG_ATTRNAME  ":position:actPos.adDeg"
#define trkwsACT_TIMESTAMP_ATTRNAME  ":position:actPos.timestamp"
#define trkwsACT_ALT_ADDRPTR  ":position:actPos.altAddr"
#define trkwsACT_AZ_ADDRPTR  ":position:actPos.azAddr"
#define trkwsACT_ROT_ADDRPTR  ":position:actPos.rotAddr"
#define trkwsACT_AD_ADDRPTR  ":position:actPos.adAddr"
#define trkwsACT_DEC_DEVIATION_ATTRNAME ":position:actPos.decDeviation"
#define trkwsACT_RA_DEVIATION_ATTRNAME ":position:actPos.RADeviation"
#define trkwsACT_ROTOFFSET_ATTRNAME ":position:actPos.rotOffset"
#define trkwsACT_ROTONSKY_ATTRNAME ":position:actPos.rotOnSky"
#define trkwsACT_POINTAXISOFFX_ATTRNAME ":position:actPos.pointAxisOffX"
#define trkwsACT_POINTAXISOFFY_ATTRNAME ":position:actPos.pointAxisOffY"

/* 
 * 
 */
#define trkwsACT_REMTRKTIME_ATTRNAME  ":position:actPos.remTrack"

/*
 *    ...... setup data
 */
#define trkwsDB_SETUP_POINT                 "setup"
#define trkwsDB_SETUP_LAMBDA_ATTRNAME          ":setup.wavelength"
#define trkwsDB_SETUP_ADDVELALPHA_ATTRNAME     ":setup.addVelAlpha"
#define trkwsDB_SETUP_ADDVELDELTA_ATTRNAME     ":setup.addVelDelta"
#define trkwsDB_SETUP_REMTRKTIMELIMIT_ATTRNAME ":setup.remTrkTimeLimit"
#define trkwsDB_SETUP_ROTANGLE_ATTRNAME        ":setup.rotDiffAngle"

/*
 *    ...... pointing correction data
 */
#define trkwsIMAGE_SHIFT_ATTRNAME ":pointing.imageShift"

/*
 *    ...... times data
 */

#define trkwsTIMES_DUT1_ATTRNAME ":times.dut1"
#define trkwsTIMES_SID_ATTRNAME  ":times.lst"
#define trkwsTIMES_UTC_ATTRNAME  ":times.utc"

/*
 *    ...... site monitor data
 */

#define trkwsENV_TEMP_ATTRNAME      ":environment.temperature"
#define trkwsENV_PRES_ATTRNAME      ":environment.pressure"
#define trkwsENV_HUM_ATTRNAME       ":environment.humidity"
#define trkwsENV_LAPSERATE_ATTRNAME ":environment.lapseRate"

/*
 *    ...... site constants data
 */

#define trkwsSITE_LATITUDE_ATTRNAME  ":siteConstants.latitude"
#define trkwsSITE_LONGITUDE_ATTRNAME ":siteConstants.longitude"
#define trkwsSITE_HEIGHT_ATTRNAME    ":siteConstants.height"
#define trkwsSITE_POLMX_ATTRNAME    ":siteConstants.xPolarMotion"
#define trkwsSITE_POLMY_ATTRNAME    ":siteConstants.yPolarMotion"

/*
 *    ...... rotator data
 */

#define trkwsROTATORS_ROTPOS_ATTRNAME  ":focusData.rotPos"
#define trkwsDB_LOAD_ON_INIT           ":focusData.loadOnInit"
#define trkwsFOCUS_CHECK_TIME          ":focusData.checkTime"

/*
 *    ...... file data
 */
#define trkwsLCU_DUT_BACKUP_FILE       ":files.backupDutLCU"
#define trkwsLCU_SETROT_BACKUP_FILE    ":files.backupSetrotLCU"

#define trkwsLCU_PC_BACKUP_FILE        ":files.backupPcLCU"
#define trkwsLCU_PC_BACKUP_BRANCH      ":files.backupPcLCUbranch"

/*
 *    ...... alarm data
 */

#define trkwsNEWPCM_ALARM        ":alarms.newpcmAlarm"
#define trkwsSETFOC_ALARM        ":alarms.setfocAlarm"
#define trkwsSETROT_ALARM        ":alarms.setrotAlarm"
#define trkwsSETOAX_ALARM        ":alarms.setoaxAlarm"

/*
 *    ...... limit data
 */

#define trkwsLIMIT_CHECK_LIMIT  ":limits.checkLimit"
#define trkwsLIMIT_ALT_ADDRPTR  ":limits.altLimitAddr"

/*
 *    NOTE:  the following attributes are relative to  trk part of 
 *           LCU branch for each axis
 */


#define trkwsAXIS_REMTRKTIME_ATTRNAME   ".remTrkTime"
#define trkwsAXIS_LIMITFLAG_ATTRNAME    ".limitFlag"
#define trkwsAXIS_SUBSTATE_ATTRNAME     ".substate"
#define trkwsAXIS_TIMESTAMP_ATTRNAME    ".timestamp"
#define trkwsAXIS_POSITION_ATTRNAME     ".position"
#define trkwsAXIS_POSENCODER_ATTRNAME   ".posEncoder"
#define trkwsAXIS_UPPER_LIMIT_ATTRNAME  ".upperLimit"
#define trkwsAXIS_LOWER_LIMIT_ATTRNAME  ".lowerLimit"
#define trkwsAXIS_ALPHA_AGC_ATTRNAME    ".alphaAGC"
#define trkwsAXIS_DELTA_AGC_ATTRNAME    ".deltaAGC"
#define trkwsAXIS_ROT_OFFSET_OBJ        ":intCoord.rotOffsetObj"
#define trkwsAXIS_ROT_OFFSET_STEP_ACCU  ":intCoord.rotOffsetStepAccu"
#define trkwsAXIS_ALPHA_APP  ":intCoord.alphaApp"
#define trkwsAXIS_DELTA_APP  ":intCoord.deltaApp"

/*
 *        end of database attribute names
 ****************************************************************
 */


/********************/
/* ASM data defines */
/********************/

/* Command sent to LCUs */
#define trkwsSETASM_CMD  "SETASM"

/* Contents of message buffer for standard commands used globally */
/* and to be propagated to trkwsSetup                              */
#define trkwsMSG_BUFF_ALL "ALL"

#endif /*!trkwsDefines_H*/
