#ifndef trkwsFOCUS_DATA_H
#define trkwsFOCUS_DATA_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: trkwsFOCUS_DATA.h,v 1.166 2005/07/25 15:44:51 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  17/04/97  Added attribute for focus check time
* rkarban  22/01/97  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#define _POSIX_SOURCE 1
#include "vltPort.h"

#include "trkws.h"
#include "ECCS.h"
#include "eccsDB_ATTR.h"
#include "eccsDB_TABLE.h"
#include "evhTIMER.h"
#include "trkwsSELF_CMD.h"
#include "mswFOCUS.h"
#include "mswINS_DATA_MONITOR.h"

class trkwsFOCUS_RECORD : public eccsDB_TABLE_RECORD {
  public:
    vltBYTES20  focusName;
    vltBYTES256 rotAddress;
    vltBYTES256 adAddress;
    vltBYTES256 pomFile;

    ccsCOMPL_STAT Unpack( void *buffer );
    ccsCOMPL_STAT Pack( void *buffer );
};

class trkwsFOCUS_DATA : public fndOBJECT, public eccsERROR_CLASS
{
  public:
    // Class constructor: here all the basic event handlers are installed
    trkwsFOCUS_DATA(const dbSYMADDRESS rootDbPoint, mswFOCUS &focus);

    // Class destructor: here everything is cleaned up
    ~trkwsFOCUS_DATA();

    ccsCOMPL_STAT  Init(const char *focusString, vltLOGICAL pollFocusIfNotReady = ccsTRUE);
    ccsCOMPL_STAT  SetReplyHandler(evhCALLBACK &cb);
    ccsCOMPL_STAT  SetErrorHandler(evhCALLBACK &cb);
    evhCALLBACK  *GetReplyHandler();
    evhCALLBACK  *GetErrorHandler();

    void ResetUserCBs ();
    void Reset ();
    ccsCOMPL_STAT UpdateFocus(evhCALLBACK *replyCB = NULL, evhCALLBACK *errorCB = NULL);
    ccsCOMPL_STAT UpdateRotator(evhCALLBACK *replyCB = NULL, evhCALLBACK *errorCB = NULL);
    ccsCOMPL_STAT UpdateOffAxisTracking(evhCALLBACK *replyCB = NULL, evhCALLBACK *errorCB = NULL);
    void ResetUpdateFocus();
    void ResetUpdateRotator();
    void ResetUpdateOax();

  protected:
    eccsDB_LOGICAL                   loadOnInit;
    eccsDB_TABLE<trkwsFOCUS_RECORD>  rotTable;
    eccsDB_STRING                    rotPosAddr;
    eccsDB_STRING                    adPosAddr;
    eccsDB_UINT32                    newpcmAlarm;
    eccsDB_UINT32                    setfocAlarm;
    eccsDB_UINT32                    setrotAlarm;
    eccsDB_UINT32                    setoaxAlarm;
    eccsDB_INT32                     checkTime;
    mswFOCUS                         &trkwsFocus;

  private:
    ccsCOMPL_STAT    _Init(const char *focusString);
    ccsCOMPL_STAT  ReadPOMFile (const char *fileName);
    evhCB_COMPL_STAT  NewpcmOKCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  SetrotOKCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  NewpcmErrorCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  SetfocErrorCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  SetrotErrorCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  SetoaxErrorCB(msgMESSAGE &, void*);
    evhCB_COMPL_STAT  CheckFocusCB(msgMESSAGE &msg, void *);
    evhCB_COMPL_STAT  InsDataChangedCB(msgMESSAGE &msg, void *);

    mswINS_DATA_MONITOR insConfigData;
    char               pcmCmd[trkwsMAX_MSG_LEN];
    char               setfocCmd[trkwsMAX_MSG_LEN];
    char               setrotCmd[trkwsMAX_MSG_LEN];
    char               setoaxCmd[trkwsMAX_MSG_LEN];
    trkwsFOCUS_RECORD  focusFound;
    evhTIMER          *timer;
    char               oldFocusString[trkwsFOCUS_STRING_MAX_LEN];
    vltLOGICAL         checkFocus;
    trkwsSELF_CMD      newpcm;
    trkwsSELF_CMD      setfoc;
    trkwsSELF_CMD      setrot;
    trkwsSELF_CMD      setoax;
    evhCALLBACK       *userReplyCB;
    evhCALLBACK       *userErrorCB;
};

/* end class trkwsFOCUS_DATA */

#endif



