#ifndef trkwsPOS_CALC_H
#define trkwsPOS_CALC_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: trkwsPOS_CALC.h,v 1.166 2005/07/25 15:44:49 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban 2001-08-13 add DB members for autoguiding correction
* ssandroc 2000-02-12 SPR 990471: ASM attrs float -> double
* kwirenst  18/12/98  Added alphaAppAct and deltaAppAct
* rkarban  15/04/98  add adDegPosActual attribute
* rkarban  03/03/98  add rotOffset, rotOnSky attributes
* kwirenst 29/05/97  Added declaration of 'utc'.
* rkarban  15/03/97  add attributes alphaCurrentReadback & deltaCurrentReadback
* rkarban  23/01/97  add attribute rotPosActual that keeps the last read rotator position
* gchiozzi  24/10/96  Added maxTimeDifference for time stamps
* gchiozzi  20/08/96  Added new attributes
* gchiozzi  19/08/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "evhHANDLER.h"
#include "eccsDB_ATTR.h"
#include "trkws.h"
#include "trkwsPOS_READ_LIST.h"


class trkwsPOS_CALC: public fndOBJECT, public eccsERROR_CLASS
{
  public:
    trkwsPOS_CALC(const dbSYMADDRESS dbPoint);
    virtual ~trkwsPOS_CALC();

    ccsCOMPL_STAT Enable();
    ccsCOMPL_STAT Disable();

    virtual evhCB_COMPL_STAT ActPosDbEventCB(evtEVENT_MSG &msg, void *udata);

  protected:
    ccsCOMPL_STAT SetInvalid(vltINT32 code, vltDOUBLE time = 0);
    ccsCOMPL_STAT SetValid();

  private:
    /* Symbolic address for the root point */
    dbSYMADDRESS    dbPoint;

    /* Objects to handle the LCU images, dynamically created */
    trkwsPOS_READ_LIST *altLcu;
    trkwsPOS_READ_LIST *azLcu;
    eccsDB_DOUBLE *rotOffsetObj;
    eccsDB_DOUBLE *rotOffsetStepAccu;
    eccsDB_DOUBLE *alphaAppAct;
    eccsDB_DOUBLE *deltaAppAct;

    /* 
     * Database attributes to be accessed 
     */

    /* Attributes accessed in write mode for update */
    eccsDB_INT32  posStatus;

    eccsDB_DOUBLE syncTimestamp;
    vltDOUBLE altitude;
    vltDOUBLE azimuth;
    vltDOUBLE alpha2000Readback;
    vltDOUBLE delta2000Readback;
    vltDOUBLE alpha2000DegReadback;
    vltDOUBLE delta2000DegReadback;
    vltDOUBLE alphaCurrentReadback;
    vltDOUBLE deltaCurrentReadback;
    vltDOUBLE alphaReadback;
    vltDOUBLE deltaReadback;
    vltDOUBLE alphaEncoder;
    vltDOUBLE deltaEncoder;
    vltDOUBLE alphaLT2000;
    vltDOUBLE deltaLT2000;
    vltDOUBLE siderealTime;
    vltDOUBLE utc;
    vltDOUBLE hourAngle;
    eccsDB_DOUBLE rotRadPosActual;
    eccsDB_DOUBLE rotDegPosActual;
    eccsDB_DOUBLE adRadPosActual;
    eccsDB_DOUBLE adDegPosActual;
    vltDOUBLE decDeviation;
    vltDOUBLE raDeviation;
    eccsDB_DOUBLE refAlpha;
    eccsDB_DOUBLE refDelta;

    /* Attributes accessed in read mode for calculations */
    eccsDB_DOUBLE maxTimeDifference;

    eccsDB_DOUBLE epoch;
    eccsDB_DOUBLE equinox;
    eccsDB_STRING epochSystem;
    eccsDB_DOUBLE pma;
    eccsDB_DOUBLE pmd;
    eccsDB_DOUBLE parallax;
    eccsDB_DOUBLE radvel;
    eccsDB_DOUBLE dut1;
    eccsDB_DOUBLE wavelen;
    eccsDB_DOUBLE temperature;
    eccsDB_DOUBLE pressure;
    eccsDB_DOUBLE humidity;
    eccsDB_DOUBLE lapseRate;
    eccsDB_DOUBLE longitude;
    eccsDB_DOUBLE latitude;
    eccsDB_DOUBLE height;
    eccsDB_DOUBLE polmx;
    eccsDB_DOUBLE polmy;
    eccsDB_DOUBLE pointAxisOffX;
    eccsDB_DOUBLE pointAxisOffY;
    eccsDB_DOUBLE rotOffset;
    eccsDB_DOUBLE rotOnSky;
    eccsDB_STRING rotPosAddr;
    eccsDB_STRING adPosAddr;

    /* 
     * Internal flags and variables
     */
    vltLOGICAL enabled;

    vltINT32   errorCounter;
    vltINT32   errorCounterRotatorAddr;
    vltINT32   errorBonus;
    vltDOUBLE  lastTimeStamp;
    vltLOGICAL alreadyInvalid;

    dbLISTID         listId;
};

#endif /*!trkwsPOS_CALC_H*/
