#******************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: aioxISignal.tcl,v 1.34 2005/09/26 12:28:57 vltsccm Exp $" 
#
# aioxISignal.tcl
# 
#  who      when     what
# -------- -------- ------------------
# imunoz  10/01/95 created for cmm
# imunoz  20/04/96 itcl2.0 port
# imunoz  27.03.97 MAY97 integration

#************************************************************************
#   NAME
#   
#       AioxISignal - Class description for an input signal of an analog
#                        I/O device.
#
#   SYNOPSIS
#
#       AioxISignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       AioxISignal inherits from the base class AioxSignal general
#       attributes common to I/O signals and defines particular
#       attributes of input signals of an analog I/O device. 
#       This class is intended to be used only as a base class. Derived
#       classes must take care of the interaction with the user and call the 
#       methods provided here to comunicate with the device's input line.  
#      
#       An object of this class is created by:
#
#		AioxISignal <object-name> -<Option> <Value> ...
#
#	which is typically done by the constructor of the derived class.
#       
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The are no special options for this class.
#
#
#   METHODS
#	Methods are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	All the methods in this class are currently intended to be used from 
#       derived classes.
#
#       setinputGain     -  sets the gain for the input signal.
#                           It is set to the current value of the global 
#                           var ${this}(inputGain)
#
#       readSignalGain   -  stores in the global var ${this}(inputGain) the
#                           gain value of the signal.
#
#       readValue        -  read the value in volts of the input signal. 
#                           It is stored in the global var ${this}(vLevel) .
#
#       updateSignal     -  this method performs all the read operations  
#                           listed above plus `readSignalConfig'.
#
#	loadData	 -  set internal state from data list (saved
#			    with a Save Panel).
#
#	configLcc	 - set internal state from data list with LCC
#                          configuration for analog I/O signals.
#
#       This class inherits methods from the base clas AioxSignal.
#
#   CAUTIONS
#	The class uses common variables.
#       The class asumes the existence of the global array ${this}, defined in
#       the base class and extends it.
#	All signals must belong to the same device.
#	In this implementation it is not possible to have signals
#	that belong to different devices at the same time.
#
#   EXAMPLES
#
#   SEE ALSO
#	aioxInducer(1),
#	InducerClass(n), InducerDevice(n), AioxDevice(n)
#	AioxSignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

class AioxISignal {
    inherit AioxSignal
    
    constructor {args} { }
    
    #---------------------------------
    # public symbols
    #---------------------------------
    public variable signalGain 1 {
	global ${itk_interior}
	
	set ${itk_interior}(signalGain) $signalGain
    }
    # added by mathar 27 July 1999; default DIFFERENTIAL
    public variable signalP2 0x100 {
	global ${itk_interior}
	
	set ${itk_interior}(signalP2) $signalP2
	set ${itk_interior}(signalP2Literal) [$itk_interior p2ToLiteral $signalP2 ]
    }
    
    #---------------------------------
    # Range Configuration
    #---------------------------------
    method readSignalConfig { } { }
    method setSignalConfig { configLiteral } { }
    # next line added by mathar July 27 1999
    method setSignalP2 { p2Literal } { }
    method setSignalConfig2 { } { }
    
    #--------------------------
    # Gain 
    #--------------------------
    #---------------------
    # Conversion Routines
    #---------------------
    method gainToLiteral { gain } { }
    method literalToGain { literal } {  }
    method readSignalGain { } { }
    method setSignalGain { gainLiteral } {  }
    method setSignalGain2 { } { }
    # next 2 lines added by mathar 27 July 99
    method literalToP2 { literal } {  }
    method p2ToLiteral { p2 } { }
    

    #----------------------------------
    # Read/Update Signal Value/Status
    #----------------------------------
    method readSignalValue {} {}
    method updateSignal { } { }
    method resumeConnection { } { }

    #------------------------------
    # Report/Load
    #------------------------------
    method report { } { }
    method loadData {data} { }
    
    #---------------------------------
    # Config from LCC
    #---------------------------------
    method configLcc { cnfString } { }
    
    #---------------------
    # protected symbols
    #---------------------
    protected variable gainList { "x1" "x10" "x100" "x500" }
    # next line added by mathar 27 July 99
    # don't change the labels, they are used below in the code!
    protected variable p2List { "diff" "single" }
}

body AioxISignal::constructor {args} {
    global ::${itk_interior}
    
    set ${itk_interior}(signalGain) 1
    # next line added by mathar 27 July 99
    set ${itk_interior}(signalP2) 0x100

    eval itk_initialize $args
}
#---------------------------------
# Range Configuration
#---------------------------------

body AioxISignal::readSignalConfig { } {
    global ::${itk_interior}
    debugPrint
    AioxSignal::readSignalConfig GET_INPUT_CONFIG

    # Split into Range and P2 with bit-masks by mathar 27 July 1999
    #set ${itk_interior}(signalConfigLiteral) \
    #       [$itk_interior configToLiteral [set ${itk_interior}(signalConfig)]]
    set range [ expr {[set ${itk_interior}(signalConfig)]} & 7 ]
    set ${itk_interior}(signalConfigLiteral) [$itk_interior configToLiteral $range ]
    # the following information already masked out/deleted by the aio driver
    # aioGetInputConfig.c 1.18 and not obtainable here...
    #set p2 [ expr [set ${itk_interior}(signalConfig)] & 0x100 ]
    #set ${itk_interior}(signalP2Literal) [$itk_interior p2ToLiteral $p2 ]
    
}

body AioxISignal::setSignalConfig { configLiteral } {
    global ::${itk_interior}
    
    # bits set selectively mathar 27 July 1999
    # set ${itk_interior}(signalConfig) [$itk_interior literalToConfig $configLiteral]
    set conf [$itk_interior literalToConfig $configLiteral]
    set ${itk_interior}(signalP2) [ literalToP2 [set ${itk_interior}(signalP2Literal)] ]
    set ${itk_interior}(signalConfig) [ expr {[set ${itk_interior}(signalP2)]} | {$conf}]

    debugPrint "Config $configLiteral - [set ${itk_interior}(signalConfig)]"
    AioxSignal::setSignalConfig SET_INPUT_CONFIG
    readSignalConfig
}

# added function by mathar 27 July 1999
body AioxISignal::setSignalP2 { p2Literal } {
    global ::${itk_interior}
    
    set p2 [$itk_interior literalToP2 $p2Literal]
    set ${itk_interior}(signalConfig) \
	[ expr {[set ${itk_interior}(signalConfig)]} & 7 | {$p2} ]
    debugPrint "Config $p2Literal - [set ${itk_interior}(signalConfig)]"
    AioxSignal::setSignalConfig SET_INPUT_CONFIG
    readSignalConfig
}

body AioxISignal::setSignalConfig2 { } {
    global ${itk_interior}
    AioxSignal::setSignalConfig SET_INPUT_CONFIG
    readSignalConfig
}

#--------------------------
# Gain 
#--------------------------
#---------------------
# Conversion Routines
#---------------------
body AioxISignal::gainToLiteral { gain } {
    return "x$gain"
}

body AioxISignal::literalToGain { literal } { 
    return [string range $literal 1 end]
}

# added mathar 27 July 99; see aioCommand.h: 0x100 or 0x200
body AioxISignal::literalToP2 { literal } { 
    if { [string compare $literal "diff"] == 0 } {
	return 0x100 } else { return 0x200 }
}
body AioxISignal::p2ToLiteral { p2 } { 
    if { $p2 == 0x100 } {
	return "diff"
    } else { 
	return "single"
    }
}
#------------------
# R/W Routines
#------------------
body AioxISignal::readSignalGain { } {
    debugPrint
    global ${itk_interior}
    set sNum $itk_option(-sNum)

    set args "$sNum;0;[set ${itk_interior}(signalGain)];0.0"
    set r [sendIoctlTrap GET_GAIN $args]
    scan [lindex $r 0] "%d;%d;%d;%lf" n n ${itk_interior}(signalGain) n

    set ${itk_interior}(signalGainLiteral) \
	    [$itk_interior gainToLiteral [set ${itk_interior}(signalGain)]]
}

body AioxISignal::setSignalGain { gainLiteral } {
    global ${itk_interior}
    
    set ${itk_interior}(signalGain) [$itk_interior literalToGain $gainLiteral]
    debugPrint "Gain $gainLiteral - [set ${itk_interior}(signalGain)]"
    set sNum $itk_option(-sNum)

    set args "$sNum;0;[set ${itk_interior}(signalGain)];0.0"
    sendIoctlTrap SET_GAIN "$args"
    readSignalGain
}

body AioxISignal::setSignalGain2 { } {
    global ${itk_interior}
    set sNum $itk_option(-sNum)    

    set args "$sNum;0;[set ${itk_interior}(signalGain)];0.0"
    sendIoctlTrap SET_GAIN "$args"
    readSignalGain
}
#----------------------------------
# Read/Update Signal Value/Status
#----------------------------------
body AioxISignal::readSignalValue {} {
    debugPrint
    global ${itk_interior}
    set n 0
    set sNum $itk_option(-sNum)

    set args "$sNum;0;0;0.0"
    set r [sendIoctlTrap READ_VALUE $args]
    scan [lindex $r 0] "%d;%d;%d;%lf" n n n vLevel
    set ${itk_interior}(vLevel) $vLevel
    debugPrint "V : $vLevel"
}

body AioxISignal::updateSignal { } {
    readSignalGain
    readSignalConfig
    readSignalValue
}

body AioxISignal::resumeConnection { } {
    $itk_interior setSignalConfig2
    $itk_interior setSignalGain2
}

#------------------------------
# Report/Load
# added P2 support mathar July 27 1999
#------------------------------
body AioxISignal::report { } {
    global ::${itk_interior}
    
    set number  "$itk_option(-sNum)"
    set name    "[list [set ${itk_interior}(signalName)]]"
    set config  "[list [set ${itk_interior}(signalConfigLiteral)]]"
    set gain    "[list [set ${itk_interior}(signalGainLiteral)]]"
    set p2    "[list [set ${itk_interior}(signalP2Literal)]]"

    return "$number $name $config $gain $p2"
}

body AioxISignal::loadData {data} {
    global ::${itk_interior}

    # Configure default bg color
    $itk_component(nr) configure -bg grey80
    #$this configure -sNum [lindex $data 0]
    #$this configure -signalName [lindex $data 1]
    set ${itk_interior}(signalName) [lindex $data 1]
    set conf [lindex $data 2]
    if { [lsearch $configList $conf] >= 0 } {
	set ${itk_interior}(signalConfigLiteral) $conf
	set ${itk_interior}(signalConfig) \
		[$itk_interior literalToConfig [set ${itk_interior}(signalConfigLiteral)]]
    }

    set gain [lindex $data 3]
    if {[lsearch $gainList $gain] >= 0 } {
	set ${itk_interior}(signalGainLiteral) $gain
	set ${itk_interior}(signalGain) \
		[$itk_interior literalToGain [set ${itk_interior}(signalGainLiteral)]]
    }

    # added P2 support mathar July 27 1999
    set p2 [lindex $data 4]
    if {[lsearch $p2List $p2] >= 0 } {
	set ${itk_interior}(signalP2Literal) $p2
	set ${itk_interior}(signalP2) \
		[$itk_interior literalToP2 [set ${itk_interior}(signalP2Literal)]]
    }
}

#---------------------------------
# Config from LCC
# adapted to the P2 settings by adding 0x100 mathar 27 July 99
#---------------------------------
body AioxISignal::configLcc { cnfString } {
    global ${itk_interior}

    #-----------------------
    # Assign config values
    #-----------------------
    set name  [lindex $cnfString [expr {[llength $cnfString] -1}] ]
    $itk_interior configure -signalName $name
    set factor [lindex $cnfString 3]
    set lowerRange [expr {round([lindex $cnfString 4]/$factor)}]
    set higherRange [expr {round( [lindex $cnfString 5]/$factor)}]

    #---------------------
    # Range checking
    #---------------------
    if {$lowerRange == -10 && $higherRange==10}  {
	# set ${itk_interior}(signalConfig) 1
	set ${itk_interior}(signalConfig) 0x101
    } elseif { $lowerRange == 0 && $higherRange==10} { 
	# set ${itk_interior}(signalConfig) 2
	set ${itk_interior}(signalConfig) 0x102
    } elseif {$lowerRange == -5 && $higherRange==5} {
	# set ${itk_interior}(signalConfig)  3
	set ${itk_interior}(signalConfig)  0x103
    } elseif {$lowerRange == 0 && $higherRange==5} {
	# set ${itk_interior}(signalConfig) 4
	set ${itk_interior}(signalConfig) 0x104
    } else {
	set ${itk_interior}(signalConfig) 0
    }
    
    set gain [lindex $cnfString 6]
    if {$gain == 1  || $gain == 10 || $gain == 100 || $gain == 500} {
	set ${itk_interior}(signalGain) $gain
    } 
}
