#******************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: aiox_InputSignal.tcl,v 1.34 2005/09/26 12:28:58 vltsccm Exp $" 
#
#  who      when     what
# -------- -------- ------------------
# imunoz 10/01/95 created for cmm
#

#*********************************************************************************
#   NAME
#   
#       Aiox_InputSignal - Class description for an interface object
#       to an input line of an analog I/O device.
#
#   SYNOPSIS
#
#       Aiox_InputSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       Aiox_Inputsignal defines the class for an interface object to
#       an  input line of an analog I/O device. 
#       Objects of this class interacts with the user and calls methods
#       of its base class to interact with the "aio" driver.
#      
#       An object of this class is created by:
#
#		Aioox_InputSignal <object-name> -<Option> <Value> ...
#
#	which is typically done by the constructor of the Device object.
#       
#
#	No public interface defined for this class.
#
#   CAUTIONS
#       The class asumes the existence of the global array ${this}, defined in
#       the base class.
#
#   EXAMPLES
#
#   SEE ALSO
#	aioxInducer(1),
#	InducerClass(n), InducerDevice(n), AioxDevice(n)
#       AioxSignal(n), AioxISignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

usual UifOptionButton { }
usual Aiox_InputSignal { keep -borderwidth -relief}


class Aiox_InputSignal {
    inherit AioxISignal
    
    constructor {args} { }
    
}

body Aiox_InputSignal::constructor {args} { 
    global ${itk_interior}
    
    itk_option add hull.relief
    itk_option add hull.borderwidth

    set ${itk_interior}(signalGainLiteral) "N/A"
    set ${itk_interior}(signalP2Literal) "N/A" ;# added by mathar 27 July 1999
    eval itk_initialize $args
    
    # create the widgets for this class:
    # -- Top Label
    itk_component add nr {
	label $itk_interior.nr -text "Input $itk_option(-sNum)"
    }
    pack $itk_component(nr) -side top -fill x -expand yes

    # signal name
    itk_component add sn {
#	iwidgets::entryfield $itk_interior.sn -labeltext "Signal Name:" 
	iwidgets::entryfield $itk_interior.sn -labeltext "" \
		-textvariable ${itk_interior}(signalName)
    } 
    setShortHelp $itk_component(sn) "Signal Name"
#   pack $itk_component(sn) -side top -anchor w -expand yes
    pack $itk_component(sn) -side top -expand yes
    
    itk_component add vl {
#	iwidgets::entryfield $itk_interior.vl -labeltext "Level:" 
	iwidgets::entryfield $itk_interior.vl -labeltext "" \
		-textvariable ${itk_interior}(vLevel)
    }
    setShortHelp $itk_component(vl) "Voltage Level"
#   pack $itk_component(vl)  -side top -anchor e -expand yes
    pack $itk_component(vl)  -side top -expand yes
    
    # Signal Range Configuration
    itk_component add fr2 {
	frame $itk_interior.fr2  -bd 2  
    }
#    itk_component add fr2.1 {
#	label $itk_component(fr2).1 -width 0 -anchor w -padx 2 -text "Range:"
#    }
    # shortened length by mathar 27 July 99 (was 6)
    itk_component add fr2.2 {
	UifOptionButton $itk_component(fr2).2 -length 3 \
		-variable ${itk_interior}(signalConfigLiteral) \
		-itemList $configList \
		-command [code $this setSignalConfig] 
    }
    setShortHelp $itk_component(fr2.2)\
	    "Set Input Range. Fixed by jumper settings, not programmable."
    
#   pack $itk_component(fr2) $itk_component(fr2.1) 
    pack $itk_component(fr2) -side left -anchor w -fill x -expand yes
    pack $itk_component(fr2.2) -side left 

    # Gain Factor
#    itk_component add fr2.3 {
#	label $itk_component(fr2).3 -width 0 -anchor w -text "Gain:"
#    }
    # shortened length by mathar 27 July 99 (was 5)
    itk_component add fr2.4 {
	UifOptionButton $itk_component(fr2).4 -length 2 \
		-variable ${itk_interior}(signalGainLiteral) \
		-itemList $gainList \
		-command [code $this setSignalGain]
    }
    setShortHelp $itk_component(fr2).4 "Set Gain Factor (x1 - x10 - x100 - x500)"

#   pack $itk_component(fr2).3 -side left -anchor w -fill x -expand yes 
    pack $itk_component(fr2).4 -side left

    # rest of lines added by mathar 27 July 99
    # VMEbus P2 connector cables
#    itk_component add fr2.5 {
#        label $itk_component(fr2).5 -width 0 -anchor w -text "P2:"
#    }
    itk_component add fr2.6 {
        UifOptionButton $itk_component(fr2).6 -length 3 \
                -variable ${itk_interior}(signalP2Literal) \
                -itemList $p2List \
                -command [code $this setSignalP2]
    }
    setShortHelp $itk_component(fr2).6 \
        "P2 Connector Line Configuration (differential - single-ended). Not programmable."

#   pack $itk_component(fr2).5 -side left -anchor w -fill x -expand yes 
    pack $itk_component(fr2).6 -side left
    
}
