#********************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: aiox_OutputSignal.tcl,v 1.34 2005/09/26 12:28:58 vltsccm Exp $" 
#
# aiox_OutputSignal.tcl"
# 
#  who      when     what
# -------- -------- ------------------
# imunoz 10/01/95 created for cmm
# imunoz 21/04/96 itcl2.0 port
#

#************************************************************************
#   NAME
#   
#       Aiox_OutputSignal - Class description for an interface object
#       to an output line of an analog I/O device.
#
#   SYNOPSIS
#
#       Aiox_OutputSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       Aiox_Outputsignal defines the class for an interface object to
#       an  output line of an analog I/O device. 
#       Objects of this class interacts with the user and calls methods
#       of its base class to interact with the "aio" driver.
#      
#       An object of this class is created by:
#
#		Aiox_OutputSignal <object-name> -<Option> <Value> ...
#
#	which is typically done by the constructor of the Device object.
#       
#
#	No public interface defined for this class.
#
#   CAUTIONS
#       The class asumes the existence of the global array ${this}, defined in
#       base classes.
#
#   EXAMPLES
#
#   SEE ALSO
#	aioxInducer(1),
#	InducerClass(n), InducerDevice(n), AioxDevice(n)
#       AioxSignal(n), AioxOSignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

usual Aiox_OutputSignal { }


class Aiox_OutputSignal {
    inherit AioxOSignal

    constructor {args} {}
}

body Aiox_OutputSignal::constructor {args} { 
    global ::${itk_interior} 
    
    itk_option add hull.relief
    itk_option add hull.borderwidth

    eval itk_initialize $args
    # -- Top Label
    itk_component add nr {
	label $itk_interior.nr -text "Output $itk_option(-sNum)"
    }
    pack $itk_component(nr) -side top -fill x -expand yes
    
    # "SIGNAL NAME" FRAME:
    itk_component add sn {
	iwidgets::entryfield $itk_interior.sn -labeltext "Signal Name:" \
		-textvariable ${itk_interior}(signalName)
    }
    pack $itk_component(sn) -side top  -anchor w -fill x -expand yes
    
    itk_component add vl {
	iwidgets::entryfield $itk_interior.vl -labeltext "Level:" \
		-textvariable ${itk_interior}(vLevel) \
		-command [code $this writeValue]
	
    }
    setShortHelp $itk_component(vl) \
	    "Type a Voltage level. Press <Enter> to set the value"
    pack $itk_component(vl)  -side top -anchor e -expand yes
    

    # Signal Range Configuration
    itk_component add fr2 {
	frame $itk_interior.fr2  -bd 2  
    }
    itk_component add fr2.1 {
	label $itk_component(fr2).1 -width 5 -anchor w -padx 2 -text "Range:"
    }
    itk_component add fr2.2 {
	UifOptionButton $itk_component(fr2).2 -length 6 \
		-variable ${itk_interior}(signalConfigLiteral) \
		-itemList $configList \
		-command [code $this setSignalConfig] 
    }
    setShortHelp $itk_component(fr2.2) \
	    "Set Input Range. Fixed by jumper settings, not programmable."
    pack $itk_component(fr2) $itk_component(fr2.1) \
	    -side left -anchor w -fill x -expand yes
    pack $itk_component(fr2.2) -side left 

    # Write mode selection
    # Float or Dec ==> 
    # WRITE_VALUE or WRITE_BIN_VALUE
    # will be used.
    itk_component add fr2.3 {
	UifRadioButton $itk_component(fr2).3 -label "Float"\
		-variable ${itk_interior}(writeCmd) \
		-onVal "WRITE_VALUE" 
    }
    itk_component add fr2.4 {
	UifRadioButton $itk_component(fr2).4 -label "Int"  \
		-variable ${itk_interior}(writeCmd) \
		-onVal "WRITE_BIN_VALUE"
    }
    pack $itk_component(fr2).3 -side left -anchor w -fill x -expand yes 
    pack $itk_component(fr2).4 -side left
    
}
