#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: alrmGuiProcs.tcl,v 1.115 2005/09/05 09:04:28 vltsccm Exp $" 
#
#
# who       when      what
# --------  --------  ----------------------------------------------
# bgilli   2005-04-04 Replaced usage of snack by standard audio module.
# bgilli   2003-01-17 Sound package version required= 2.2
#************************************************************************

proc alrmMessage { name type message } {
   global gvar

   set c .$name
   set flag -1
   set gvar(alarmMessage) 0

   toplevel $c

   frame $c.middle         -relief raised -border 1
   frame $c.bot            -relief raised -border 1
   frame $c.middle.center  -relief groove -border 1

   button  $c.bot.ok       -text "Ok" -width 10 -command \
                           "set flag 1; set gvar(alarmMessage) 1; destroy $c"

   if { $type == "QUESTION" } {
       button  $c.bot.cancel -text "Cancel" -width 10 -command \
                             "set flag 1; destroy $c"
   } elseif { $type == "ASK" } {
       button  $c.bot.fix -text "Fix" -width 10 -command \
                          "set flag 1; set gvar(alarmMessage) 2; destroy $c"
   }

   message $c.middle.center.msg  -justify left -text "$message" \
           -font "-Adobe-courier-bold-r-normal--14*" -width 15c

   if { $type == "ERROR" } {
       label $c.middle.left -bitmap warning
   } elseif { $type == "QUESTION" } {
       label $c.middle.left -bitmap questhead
   } elseif { $type == "ASK" } {
       label $c.middle.left -bitmap question
   } else {
       label $c.middle.left -bitmap info
   }

   $c.middle.left config  -foreground red -borderwidth 5 -height 40 -width 30

   pack $c.middle $c.bot -fill x
   if { $type == "QUESTION" } {
       pack $c.bot.ok $c.bot.cancel -side left -pady 15 -expand 1
   } elseif { $type == "ASK" } {
       pack $c.bot.ok $c.bot.fix -side left -pady 15 -expand 1
   } else {
       pack $c.bot.ok -side left -pady 15 -expand 1
   }

   pack $c.middle.center -side right -padx 10 -fill both y 

   pack $c.middle.left -side left -expand 1 -fill y -padx 1 -pady 1
   pack $c.middle.center.msg  -side right -expand 1 -fill y -padx 10 -pady 10

   wm geometry $c +400+400
   wm title $c "CCS ALARM System Info"

   tkwait visibility $c
   grab $c
   vwait flag;      # Stops until variable as been modified
   }

proc alrmAck {} {
    alrmMessage message INFO "This feature is currently NOT supported"
}

proc alrmAbout { } {
    global gvar seq_ccsEnvName RCS RTAPENV env
 
    if {[catch {set user $env(USER)}]} {set user dummy}
    regexp {Id: (.*) Exp} $RCS -> version

    set output "\n         About CCS ALARM DISPLAY\n\
                \n$version\n\
                \n           started by $user\n\
                \n    at $gvar(startTime)\n\
                \n          in environment $RTAPENV\n\
                \n   attached to environment $gvar(alrmEnv)\n"

    alrmMessage message INFO $output          
}

proc alrmHide { } {
    global gvar

    set cnt [llength $gvar(alarmColumns)]
    for {set i 0} {$i<$cnt} {incr i} {
        if { $gvar([lindex $gvar(alarmColumns) $i]) == 1 } {
            $gvar(tbl) columnconfigure $i -hide 0
	} else {
            $gvar(tbl) columnconfigure $i -hide 1
	}
    }
}


proc alrmSetMode { val } {
    global gvar color

    if { $gvar(alrmInit) == 0 } return

    if { $val == 3 } {
        set mode $gvar(alrmMode)
    } else {
        set mode $val
    }

    set tmpList [list]
    if { $mode == 1 } {
        if { $gvar(alrmTimerVar) == 0 } {
            wm title $gvar(top) [format "CCS ALARM DISPLAY - %s - (Summary Mode)" $gvar(alrmEnv)]
	}
        set cnt [llength $gvar(alrmListS)]
        for {set i 0} {$i<$cnt} {incr i} {
            if { $gvar([lindex [lindex $gvar(alrmListS) $i] 4]) == 1} {
                lappend tmpList [lindex $gvar(alrmListS) $i]
            }
	}
    } elseif { $mode == 0 } {
        wm title $gvar(top) [format "CCS ALARM DISPLAY - %s - (History Mode)" $gvar(alrmEnv)]
        set cnt [llength $gvar(alrmListH)]
        for {set i 0} {$i<$cnt} {incr i} {
            if { $gvar([lindex [lindex $gvar(alrmListH) $i] 4]) == 1} {
                lappend tmpList [lindex $gvar(alrmListH) $i]
            }
	}
    }

    set gvar(alrmList) $tmpList
    set cnt [llength $tmpList]
    for {set i 0} {$i<$cnt} {incr i} {
        set aktVal [$gvar(tbl) rowcget $i -fg]
        set newVal $color([lindex [lindex $tmpList $i] 4])
        if { $aktVal != $newVal } {
            $gvar(tbl) rowconfigure $i -fg $newVal
	}
    }
    set gvar(alrmMode) $mode
    unset tmpList
}

proc alrmStop { } {
    global gvar

    if { $gvar(alrmTimerVar) == 1} {
        set gvar(alrmTimerVar)  0
        after cancel $gvar(alrmTimer)    
        wm title $gvar(top) [format "CCS ALARM DISPLAY - %s - (Summary Mode)" $gvar(alrmEnv)]
    }
}

proc alrmBrowser { } {
    global gvar

    set sel [$gvar(tbl) curselection]
    if { [llength $sel] > 0 } {
        set index [lindex $sel 0]
        set attribute [format "@%s%s" $gvar(alrmEnv) [lindex [$gvar(tbl) get $index] 2]]
        if { [seq_catch {seq_dbReadSymbolic $attribute bad} result] } {
        } else {
            set output [format "Current value of %s is %s" $attribute $result]
            alrmMessage message INFO $output
        }
    } else {
        alrmMessage message ERROR "No item selected"
    }
}

proc alrmHelpFile { } {
    global gvar seq_ccsEnvName

    set sel [$gvar(tbl) curselection]
    if { [llength $sel] > 0 } {
        set index [lindex $sel 0]
        set attribute [lindex [$gvar(tbl) get $index] 2]
        if { [catch {seq_msgSendCommand $gvar(alrmEnv) alrmServer ALHEF $attribute} cmdId] } {
            alrmMessage message ERROR $cmdId
        } else {
            set result [seq_msgRecvReply $cmdId -all  err last]
            if { $result != "NONE" } {
                if { [seq_catch { exec alrmDisplayFile $result & } er] } {
                    alrmMessage message ERROR $er
                }
            } else {
                alrmMessage message INFO [format "No HELP file available for %s" $attribute]
            }
        }
    } else {
        alrmMessage message ERROR "No item selected"
    }
}


proc alrmInitSound { reinit } {
    global gvar soundFile

    if {[string equal [info command ::audio::audioPlay] {}]} {
       package require Audio
       auto_load audio::audioPlay
    }
    set cnt [llength $gvar(alarmFilters)]
    for {set i 0} { $i < $cnt } {incr i} {
        set severity [lindex $gvar(alarmFilters) $i]
        if {[catch {seq_findFile sounds/$severity.wav} soundFile($severity)]} {
            set soundFile($severity) "NONE"
        }
    }
}

proc alrmSaveTable { } {
    global gvar

    if {[set filePath [panSelectFile [pwd] *]] == ""} {
        return 
    } else {
         if {[lempty $gvar(alrmList)]} { return }
         set fh [open $filePath w ] ;  # Open and create a new file
         foreach item $gvar(alrmList) {
             puts $fh $item
         }
         close $fh
    } 
}

proc alrmNextSound { } {
    global gvar soundObj soundOn soundLevel soundNext soundActive soundFile

    if { $soundNext == "NONE" } {
        set soundActive "NONE"
    } else {
        set soundActive $soundNext
        set soundNext   "NONE"
	if {[catch {::audio::audioPlay  -volume 80 $soundFile($soundActive)} msg]} {
	    error "audio(Server) problem: $msg"
	}
        after 2000 alrmNextSound
    }
}

proc alrmNewAlarm { msg severity } {
    global gvar color soundFile soundObj soundOn soundLevel soundNext soundActive

    if { $gvar(alrmInit) == 0 } return
  
    set tim       [string trimright [string range $msg  0  18]]
    set scope     [string trimright [string range $msg 21  27]]
    set attribute [string trim      [string range $msg 28  68]]
    set value     [string trimright [string range $msg 72  91]]
    set help      [string trimright [string range $msg 110 end]]

    lappend newElem $tim $scope $attribute $value $severity $help

    if { $gvar(sound) == 1 } {
        if { $soundFile($severity) != "NONE" } {
            if { $soundOn($severity) == 1 } {
                if { $soundActive == "NONE" } {
                    set soundActive $severity
		    if {[catch {::audio::audioPlay  -volume 80 $soundFile($severity)} msg]} {
			error "audio(Server) problem: $msg"
		    }
                    after 2000 alrmNextSound
		} elseif { $soundLevel($soundActive) < $soundLevel($severity) } {
                    if { $soundNext == "NONE" } {
                        set soundNext $severity
                    } else {
                        if { $soundLevel($severity) > $soundLevel($soundNext) } {
                            set soundNext $severity
                        }
		    }
		}
	    }
	}
    }


    if { $gvar(alrmMode) == 1 } {
        # Summary Mode
        set cnt [llength $gvar(alrmList)]
	for {set i 0} {$i<$cnt} {incr i} {
	    if { [lindex [lindex $gvar(alrmList) $i] 2] == $attribute } {
                $gvar(tbl) delete $i
                break
	    }
	}
        if { $gvar($severity) == 1 } {
            $gvar(tbl) insert 0 $newElem
            $gvar(tbl) rowconfigure 0 -fg $color($severity)
            if { $severity != "normal" } {
                wm deiconify $gvar(top)
                raise  $gvar(top)
            }
	}
    } elseif { $gvar(alrmMode) == 0 } {
        # History Mode
        if { $gvar($severity) == 1 } {
            $gvar(tbl) insert 0 $newElem
            $gvar(tbl) rowconfigure 0 -fg $color($severity)
            if { $severity != "normal" } {
                wm deiconify $gvar(top)
                raise  $gvar(top)
            }
        }
    }

    # handle summary list        
    set cnt [llength $gvar(alrmListS)]
    for {set i 0} {$i<$cnt} {incr i} {
	if { [lindex [lindex $gvar(alrmListS) $i] 2] == $attribute } {
            set gvar(alrmListS) [lreplace $gvar(alrmListS) $i $i]
            break;
        }
    }
    set gvar(alrmListS) [linsert $gvar(alrmListS) 0 $newElem]

    # handle history list
    set gvar(alrmListH) [linsert $gvar(alrmListH) 0 $newElem]
}

proc alrmStartAll { } {
    global gvar seq_ccsEnvName

    set gvar(alrmInit) 1

    scan $gvar(alarmConn) {alarm%d} connId
    if { [catch {seq_msgSendCommand -nocheck $gvar(alrmEnv) alrmServer ALALL $connId} cmdId] } {
        alrmMessage message ERROR $cmdId
    } else {
        if { [catch {seq_msgRecvReply $cmdId -last error} result] } {
             alrmMessage message ERROR $result
	}
        alrmSetMode 1
        alrmStop
        if { $gvar(soundOnCmd) == 1 } {
            alrmChangeSound
        }
    }
}

proc alrmChange { } {
    global gvar

    wm title $gvar(top) [format "%s ." [wm title $gvar(top)]]
    set gvar(alrmTimer) [after 2000 alrmChange]
}

proc alrmChangeSound { } {
    global gvar

    if { $gvar(soundError) == 0 } {
        if { $gvar(alrmTimerVar) == 0 } {
            if { $gvar(sound) == 1 } {
                set gvar(sound) 0
                $gvar(top).soundIcon configure -image $gvar(soundIconOff)
            } else {
                set gvar(sound) 1
                $gvar(top).soundIcon configure -image $gvar(soundIconOn)
            }
        }
    } else {
        set gvar(sound) 0
        $gvar(top).soundIcon configure -image $gvar(soundIconOff)
    }
}
