#########################################################################
#
# E.S.O. - VLT project
#
# "@(#) $Id: arifPanProcs.tcl,v 1.14 2005/02/09 12:35:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# ahuxley   02/08/02  created
#
#########################################################################

#########################################################################
#
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#########################################################################

#  Handler for when 'START' button is pressed
proc arifPanStartButtonHandler { } {

    #puts "arifPanStartButtonHandler: DEBUG: sof"

    #  Lock critical call, to prevent 'STOP' or
    #  'START' while starting up.
    if { [arifPanLock] } {
        return
    }
    catch { arifPanStartLoop }
    arifPanUnlock

}

#  Handler for when 'STOP' button is pressed
proc arifPanStopButtonHandler { } {

    #puts "arifPanStopButtonHandler: DEBUG: sof"

    if { [arifPanLock] } {
        return
    }
    #  Lock critical call, to prevent 'STOP' or
    #  'START' while stopping.
    catch { arifPanStopLoop }
    arifPanUnlock

}

#  Handler for when 'SET' button is pressed
proc arifPanSetButtonHandler { } {
    global gvar cv

    #puts "arifPanSetButtonHandler: DEBUG: sof"

    set loopFinite l1024
    set loopInfinite 2048
    set integrating 4

    if { ([info exists gvar(arifPanTCCDWidget_expTime)] == 0) || ($gvar(arifPanTCCDWidget_expTime) == "") } {
    	tk_messageBox -icon error -message "Enter a valid exposure time first."
        return
    } 

    #
    #  This function needs to know the name of the database point where the
    #  exposure status is stored so that it can check if now is a valid time
    #  to change the exposure time with the 'SET' button.
    #
    #  Of course, aropanControl should look in the 'waral' environment's
    #  database for this information, and ISS should look in the ARAL
    #  branch of the ISS database.
    # 
    #  This requires that is function know what the current working point
    #  (the db 'hook' linking this UIF class into the parent panel).
    #  The way to do this is to use 'cget'. ('getMyCwd' isn't implemented
    #  yet.)
    #
    set cwp [$cv.arifPanTCCDWidget cget -cwp]
    #puts "cwp = $cwp"

    #  Determine if an exposure is currently running
    set expStatus [seq_dbReadSymbolic [format "%s.expStatus" $cwp]]
    #puts "expStatus = $expStatus"

    #  If an expose is running change the exposure time of the last exposure
    if { ($expStatus == $loopFinite) || ($expStatus == $loopInfinite) || ($expStatus == $integrating) } {; # current exposure
        panSendCommand waral ccdconCI_tccd SETUP -last 15000 "-expoId 0 -function DET.WIN1.UIT1 $gvar(arifPanTCCDWidget_expTime)"

    #  Otherwise change the exposure time of the next exposure
    } else {
        panSendCommand waral ccdconCI_tccd SETUP -last 15000 "-expoId -1 -function DET.WIN1.UIT1 $gvar(arifPanTCCDWidget_expTime)"
    }
}

proc arifPanStartLoop { } {
    #puts "arifPanStartLoop: DEBUG: sof"

    #  Start the exposure
    panSendCommand waral ccdconCI_tccd STARTLP -last 15000 ""
}

proc arifPanStopLoop { } {
    #puts "arifPanStopLoop: DEBUG: sof"

    #  Stop the exposure
    panSendCommand waral ccdconCI_tccd STPWAIT -last 30000 ""
}

proc arifPanLock { } {
    global gvar

    #puts "arifPanLock: DEBUG: sof"

    set active $gvar(arifPanButtonLock)
    if { $active } {
    	tk_messageBox -icon error -message "command in progress:\n\nPlease try again in a moment."
    } else {
        set gvar(arifPanButtonLock) 1
    }

    return $active
}

proc arifPanUnlock { } {
    global gvar

    #puts "arifPanUnlock: DEBUG: sof"

    set gvar(arifPanButtonLock) 0
}

proc arifPanInit { } {

    #  Unlocking effectively creates/initialises the lock
    arifPanUnlock
}
