#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplot2Entries.tcl,v 1.4 2005/07/07 10:46:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/06/04  created

#************************************************************************
#   NAME
#   Bltplot2Entries - itcl class to handle 2 entry widgets
# 
#   SYNOPSIS
#   Bltplot2Entries <widget-name> ?widget options?
# 
#   DESCRIPTION
#   Bltplot2Entries handles 2 entry widgets with the following properties:
#   - entry types are 'real'
#
#   SEE ALSO
#   LabelEntry(n) - Rtd tclutil package
#
#   BUGS     
#
#------------------------------------------------------------------------
itk::usual Bltplot2Entries {}

class Bltplot2Entries {
    inherit BltplotBase

    constructor {args} {
        eval itk_initialize $args

	make_entry v0
	make_entry v1
    }

    private method make_entry { compo } {
	set w $w_.$compo
	itk_component add $compo {
	    LabelEntry $w -valuewidth 7 -validate real \
		-command [code $this changed]
	}
	pack forget [$w component label]
	pack [$w component entry] -padx 0 -ipadx 0 -pady 0 -ipady 0
	pack $w -side left -expand 1 -fill x -pady 0 -ipady 0
    }

    # Cut entry value(s) changed

    private method changed { val } {
	set v0 [$itk_component(v0) get]
	set v1 [$itk_component(v1) get]

	if { "$v0" == "" && "$v1" == "" } { return }

	if { "$v0" == "" } { set v0 $v0_ }
	if { "$v1" == "" } { set v1 $v1_ }

	# swap values if max/min values were entered
	if { $v0 > $v1 } {
	    lassign "$v0 $v1" v1 v0
	}

	if { ! [lempty $command] } {
	    eval $command $v0 $v1
	}
    }

    # set values; callback from trace (see bltplotCtrl.tcl)

    public method set_values { idx array name opt } {
	global $array
	set v$idx\_ [set ${array}($name)]
	$itk_component(v$idx) config -value [set ${array}($name)]
    }

    # -- private vars --

    # actual values (min, max)
    private variable v0_ 0.0
    private variable v1_ 1.0

    # -- public vars --

    # the command for <Return> in one of the the entries, called with the new values
    public variable command {}
}


# ___bltplot___
