#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotBase.tcl,v 1.4 2005/07/07 10:46:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/06/01  created
#

#************************************************************************
#   NAME
#   BltplotBase - bltplot base class
#
#   SYNOPSIS
#   BltplotBase <widget-name> ?widget options?
# 
#   DESCRIPTION
#   Base class for bltplot
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotBase {}

class BltplotBase {
    inherit UifWidget

    constructor {args} {
	global shArea
	set w_ [namespace tail $this]
	eval itk_initialize $args
	config -helparea $shArea  ; # for UifWidget
    }

    # add helptext

    protected method shelp { wdg txt } {
	UifShortHelp::add [cget -helparea] $wdg $txt
    }

    # return command for iwidgets::panedwindow

    protected method panedwindowCmd {  } {
	if { "$panedwindowCmd_" != "" } {
	    return $panedwindowCmd_
	}
	foreach name "panedwindow ipanedwindow Panedwindow Ipanedwindow" {
	    if { ! [catch {iwidgets::$name $w_.try}] } {
		destroy $w_.try
		set panedwindowCmd_ iwidgets::$name
		return $panedwindowCmd_
	    }
        }
	puts stderr "Could not find command for iwidgets::panedwindow"
	exit -1
    }


    # execute command 'trace add variable <name> write <command>
    # - 'var' must be a member of the global array 'globy'

    public proc trace_addvar { var command { globy bltplot } } {
	global tcl_version $globy

	if { $tcl_version > 8.3 } {
	    trace add variable ${globy}($var) write $command
	} else {
	    trace variable ${globy}($var) w $command
	}
    }

    # remove trace variable

    public proc trace_remvar { var command { globy bltplot } } {
	global tcl_version $globy

	if { $tcl_version > 8.3 } {
	    trace remove variable ${globy}($var) write $command
	} else {
	    trace vdelete ${globy}($var) w $command
	}
    }

    # set trace variable

    public proc trace_setvar { var value { globy bltplot } } {
	global $globy
	set ${globy}($var) $value
    }

    # -- private vars --
    private variable panedwindowCmd_ {}     ; # panedwindow command

    # -- protected vars --

    protected variable image_ {} ; # name of internal rtdimage object
    protected variable w_ {}     ; # shorter name for $itk_component(hull)

    # -- public variables --

    # main RtdImage widget (set by caller)
    public variable image {} {
	if { "[cget -image]" != "" } {
	    set image_ [[cget -image] get_image]
	}
    }
    # background of plot area
    public variable plotbg {midnightblue}

    # default: show cross
    public variable show_cross {0}

    # default: show grid
    public variable show_grid {0}

    # default: show pixel marker
    public variable show_pixmark {0}

    # default: with scale widgets and scrollbars
    public variable with_scale {1}

    # with x axis tick labels
    public variable with_xticks {0}

    # with y axis tick labels
    public variable with_yticks {0}

    # object Id
    public variable id {0}

    # scaline width (0 = width of image)
    public variable swidth {0}

    # line colors
    public variable element_colors {
	grey60 grey40 black red green blue cyan brown magenta purple orange yellow
    }

    # widget is activated, bool
    public variable activated {0}

    # widget is visible, bool
    public variable visible {1}
}

#
# ___bltplot___
