#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotCtrl.tcl,v 1.4 2005/07/07 10:46:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/06/04  created
#

#************************************************************************
#   NAME
#   BltplotCtrl - Plot control class
# 
#   SYNOPSIS
#   BltplotCtrl <widget-name> ?widget options?
# 
#   DESCRIPTION
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotCtrl {}

class BltplotCtrl {
    inherit util::TopLevelWidget

    constructor {args} {
        wm protocol $w_ WM_DELETE_WINDOW "$this quit"
	wm transient $w_ [winfo parent $w_]
	eval itk_initialize $args
    }

    # method init is called after the window was created.

    protected method init { } {
        wm title $w_ "Plot Control"

	# add menu and controls for the scans
	make_menu

	loop gr 0 [llength $groupObjs_] {
	    make_controls $gr
	}
	make_buttons

	wm resizable $w_ 0 0
	center_window
   }

    # REDEFINED: Center this window on the screen. 

    public method center_window {} {
	set parent [winfo parent $w_]
	set x [expr {[winfo screenwidth $w_]/2 - [winfo reqwidth $w_]/2 \
			 - [winfo vrootx $parent]}]
	set y [expr {[winfo screenheight $w_]/2 - [winfo reqheight $w_]/2 \
			 - [winfo vrooty $parent]}]
	after 0 "wm geometry $w_ +$x+$y"
    }

    # add the menubar and short help

    private method make_menu {} {
	global $w_ bltplot

	set allSobjs [get_scobjs]  ; # get all scan objects

	add_menubar
        pack $itk_component(menubar) \
            -side top -fill x -ipady 2

	# add the File menu
	set m [add_menubutton File]
	add_menuitem $m command "Close" {} -command [code $this quit]

	set m [add_menubutton View]
	add_menuitem $m checkbutton "Pixel marker" {} -variable bltplot(pixmark)
	add_menuitem $m checkbutton "Crosshair"    {} -variable bltplot(cross)
	add_menuitem $m checkbutton "Grid"         {} -variable bltplot(grid) ; # useful?

	foreach var "cross pixmark grid" {
	    BltplotBase::trace_setvar $var [cget -show_$var]
	}

	$m add separator
	add_menuitem $m cascade "Hide..." {} -menu [menu $m.hide]

	# hide/show for all groups and scans
	#
	set n [llength $groupObjs_]
	loop l 0 $n {
	    set grObj [lindex $groupObjs_ $l]

	    $m.hide add checkbutton \
		-label "Group [expr {$l + 1}]" \
		-variable bltplot(hideGroup$l)

	    # hide/show for all scans
	    #
	    foreach scan [$grObj get_scanObjs] {
		set id [$scan cget -id]
		$m.hide add checkbutton -label "$id" \
		    -variable bltplot(hideScan$id)
	    }
	    if { [expr {$l + 1}] < $n } { $m.hide add separator }
	}

	#
	# colors for all scans
	#
	add_menuitem $m cascade "Colors..." {} -menu [menu $m.colors]

	set l 0
	foreach sobj $allSobjs {
	    set Id [$sobj cget -id]
	    set ml "$m.colors[incr l]"
	    add_menuitem $m.colors cascade [$sobj cget -id] {} -menu [menu $ml]
	    
	    # set colors in menu
	    #
	    foreach c [$sobj cget -element_colors] {
		$ml add command -background $c \
		    -command "[code $sobj config -color $c]; [code $w_ component sclab$Id config -fg $c ]"
	    }
	}

	set m [add_menubutton Tools]
	#
	# print for all scans
	#
	add_menuitem $m cascade "Print..." {} -menu [menu $m.print]
	foreach sobj $allSobjs {
	    $m.print add command -label [$sobj cget -id] -command [code $sobj print]
	}
    }

    # create control window for a group

    private method make_controls { gr } {
	set row 0
	set grframe $w_.f$gr
	set groupObj [lindex $groupObjs_ $gr]

	pack [frame $grframe] -fill both -expand 1

	# heading for scan lines
	set Id [$groupObj cget -id]
	label $grframe.title2 \
	    -text "$Id: Cut levels"

	blt::table $grframe \
	    $grframe.title2  $row,1 -columnspan 2

	foreach sobj [$groupObj get_scanObjs] {
	    add_scan_control $sobj $grframe [incr row] $Id
	}
	incr row

	make_group_control $groupObj $grframe $row
    }

    # add control widgets for a group

    private method make_group_control { obj grframe row } {
	set Id [$obj cget -id]

	label $grframe.title0 \
	    -anchor   c \
	    -fg       white \
	    -bg       [$obj cget -plotbg] \
	    -relief   sunken

	$grframe.title0 config -bitmap arrowboth -width 32
	shelp $grframe.title0 \
	    "Scale range (left, right) of scan (enter pixel numbers)"

	label $grframe.title1 \
	    -text     "$Id: Scale"

	# Entries for scaling
	Bltplot2Entries $grframe.scale \
	    -command [code $obj set_scale]

	BltplotBase::trace_addvar scalex$Id [code $grframe.scale set_values 0]
	BltplotBase::trace_addvar scaley$Id [code $grframe.scale set_values 1]

	shelp $grframe.scale \
	    "$Id: Enter left and right x-values (pixel number) for scale"

	# add button widgets
	button $grframe.reset \
	    -width    2 \
	    -text     "R" \
	    -command  [code $obj reset_scale]
	shelp $grframe.reset \
            "Reset all scans in this group ($Id) to full scale"

	button $grframe.acut \
	    -width    2 \
	    -text     "A" \
	    -command  "BltplotBase::trace_setvar groupAcut$Id 1"
        shelp $grframe.acut "Autocut all scans in this group ($Id)"

	if { ! $with_scale } { return }

	blt::table $grframe \
	    $grframe.title1  $row,1 -columnspan 2 \

	incr row
	blt::table $grframe \
	    $grframe.title0  $row,0 \
	    $grframe.scale   $row,1 -columnspan 2 -fill x \
	    $grframe.reset   $row,3 \
	    $grframe.acut    $row,4 
    }

    # add control widgets for a scanline

    private method add_scan_control { sobj scframe row groupId } {
	set Id [$sobj cget -id]

	# scanline id
	itk_component add sclab$Id {
	    label $scframe.id$row \
		-width   5 \
		-text    $Id \
		-fg      [$sobj cget -color] \
		-relief  sunken
	}
	$itk_component(sclab$Id) config -bg [$sobj cget -plotbg]
	shelp $scframe.id$row "$Id"

	# Entries for entering the cut levels
	Bltplot2Entries $scframe.cuts$row \
	    -command [code $sobj set_cuts]
	shelp $scframe.cuts$row "$Id: Enter cut levels"

	BltplotBase::trace_addvar lcut$Id [code $scframe.cuts$row set_values 0]
	BltplotBase::trace_addvar hcut$Id [code $scframe.cuts$row set_values 1]

	# add button widgets
	button $scframe.set$row \
	    -width       2 \
	    -text       "S" \
	    -command    "BltplotBase::trace_setvar groupCuts$groupId $sobj"
	shelp $scframe.set$row \
            "Apply the cut levels of $Id to all plots in this group"

	button $scframe.acut$row \
	    -width      2 \
	    -text      "A" \
	    -command   [code $sobj auto_cut]
        shelp $scframe.acut$row "Set autocut levels for this plot ($Id)"

	blt::table $scframe \
	    $scframe.id$row    $row,0 \
	    $scframe.cuts$row  $row,1 -columnspan 2 -fill x \
	    $scframe.set$row   $row,3 \
	    $scframe.acut$row  $row,4
    }
    

    # create the buttons

    private method make_buttons { } {
	set butf $w_.butf
	pack [frame $butf] -fill x -side bottom -pady 1 

	button $butf.acuts \
	    -text      "Auto Cuts" \
	    -command   "BltplotBase::trace_setvar acuts 1"
	shelp $butf.acuts "Auto cut all scanlines"

	button $butf.reset \
	    -text      "Reset" \
	    -command   [code $this reset_plot]
	shelp $butf.reset "Reset plot to defaults"

	button $butf.quit \
	    -command   [code $this quit] \
	    -text      "Close"
	shelp $butf.quit "Close this window"

	foreach but "acuts reset quit" {
	    $butf.$but config -relief raised 
	}

	blt::table $butf \
	    $butf.acuts  0,0 -fill x \
	    $butf.reset  0,1 -fill x \
	    $butf.quit   0,2 -fill x
    }
    
    # reset plot window: set to defaults

    private method reset_plot { } {
	if { [llength $groupObjs_] } {
	    destroy $itk_component(menubar)
	    make_menu
	    pack $itk_component(menubar) -before $w_.f0
	}
	BltplotBase::trace_setvar resetPlot 1
    }
    
    # get all scan objects

    private method get_scobjs { } {
	set allSobjs {}
	foreach grObj $groupObjs_ {
	    foreach sobj [$grObj get_scanObjs] {
		lappend allSobjs $sobj
	    }
	}
	return $allSobjs
    }

    # clients can call addShortHelp to add their short help text

    private method shelp { wdg txt } {
	global shArea
	UifShortHelp::add $shArea $wdg $txt
    }

    # -- private varibales --
    private variable groupObjs_ {}   ; # group objects

    # -- public varibales --

    # default: show cross
    public variable show_cross {0}

    # default: show grid
    public variable show_grid {0}

    # default: show pixel marker
    public variable show_pixmark {0}

    # default: with scale widgets and scrollbars
    public variable with_scale {1}

    # widget handling the plots
    public variable plotWdg {} {
	set groupObjs_ [[cget -plotWdg] get_groupObjs]
    }
}

#
# ___bltplot___
