#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotPlot.tcl,v 1.4 2005/07/07 10:46:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/05/04  created
#

#************************************************************************
#   NAME
#   BltplotPlot - Plot widget class for module bltplot
# 
#   SYNOPSIS
#   BltplotPlot <widget-name> ?widget options?
# 
#   DESCRIPTION
#
#   SEE ALSO
#   iwidgets(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotPlot {}

class BltplotPlot {
    inherit BltplotBase

    constructor {args} {
	package require Iwidgets

	# main plot frame (paned window)
	#
	set plt_ $w_.plt
        [panedwindowCmd] $plt_ \
	    -sashcursor  sb_v_double_arrow \
	    -thickness   2 \
	    -orient      horizontal \
	    -width       $width \
	    -height      $height

	eval itk_initialize $args
	pack $plt_ -expand 1 -fill both

	# make plot group widgets (paned windows)
	#
	set idx 0
        loop gr 0 [llength $plotGroupList] {
	    if { [$w_ make_group $plt_ $gr $idx] != 0 } {
		trace_addvar hideGroup$idx [code $this hide_group $idx]
		incr idx
	    }
	}

	# set equal height for scans
	if { [llength $plotGroupList] > 1 } {
	    set ntot [llength [join [call_objs get_scanObjs]]]
	    set cmd "$plt_ fraction "
	    foreach gr $groupObjs_ {
		set f [round [expr {double ([llength [$gr get_scanObjs]]) * 100.0 / $ntot}]]
		append cmd "$f "
	    }
	    catch {eval $cmd}
	}
    }

    # make a plot group widget

    private method make_group { w gr idx } {
	set plotGroup [lindex $plotGroupList $gr]
	set plotScans [lindex $plotScanList $gr]

	# check group spec
	#
	set pval [lindex $plotScans 0]
	if { "$plotGroup" == "" || "$pval" == "" } { return 0 }

	if { [llength $plotGroup] < 2 || \
		 [catch {expr {[lindex $plotGroup 1] + 1}}] } {
	    puts stderr "invalid spec of plot group: $plotGroup"
	    return 0
	}
	
	# w is the paned window (see package iwidgets)
	$w add $idx \
	    -margin 2 \
	    -minimum 5

	set groupObj [$w childsite $idx].group$idx

	BltplotGroup $groupObj \
	    -image         $image \
	    -id            [lindex $plotGroup 0] \
	    -swidth        [lindex $plotGroup 1] \
	    -plotScanList  $plotScans \
	    -with_scale    $with_scale \
	    -show_cross    $show_cross \
	    -show_pixmark  $show_pixmark \
	    -with_xticks   $with_xticks \
	    -with_yticks   $with_yticks \
	    -plotbg        $plotbg

	if { [llength $plotGroup] >= 4 } {
	    $groupObj set_xrange 1 [lindex $plotGroup 2] [lindex $plotGroup 3]
	}

	lappend groupObjs_ $groupObj
	return 1
    }

    # show/hide this widget

    public method show_plot { {flag 1} } {
	eval pack [expr {$flag ? "$w_ -expand 1 -fill both" : "forget $w_"}]
	call_objs config -activated $flag
    }
    
    # call methods of objects belonging to the scan group

    private method call_objs { args } {
	foreach gr $groupObjs_ {
	    lappend ret [eval $gr $args]
	}
	return $ret
    }

    # return group objects

    public method get_groupObjs { } {
	return $groupObjs_
    }

    # hide/show a group

    private method hide_group { idx array name args } {
	global $array
	set bool [set ${array}($name)]
	set groupObj [lindex $groupObjs_ $idx]
	if { $bool != [$groupObj cget -visible] } { return }

	if { $bool } {
	    $plt_ hide $idx
	    $groupObj config -visible 0
	} else {
	    $plt_ show $idx
	    $groupObj config -visible 1
	}
    }

    # -- private variables --

    private variable plt_            ; # name of plot frame widget
    private variable groupObjs_ {}   ; # itcl objects for handling the scan groups

    # -- public variables --

    # width and height
    public variable width {400} {
	$plt_ config -width [cget -width]
    }
    public variable height {400} {
	$plt_ config -height [cget -height]
    }
    # spec of group list and scan list
    public variable plotGroupList {}
    public variable plotScanList {}
}

#
# ___bltplot___
