#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotRtd.tcl,v 1.4 2005/07/07 10:46:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/05/04  created
#

#************************************************************************
#   NAME
#   BltplotRtd - Rtd widget class for module bltplot
# 
#   SYNOPSIS
#   BltplotRtd <widget-name> ?widget options?
# 
#   DESCRIPTION
#   This class loads and initializes an RtdImage widget.
#   - The image coordinates and values are displayed in the help
#     area when the mouse moves over the image.
# 
#   METHODS
#   public method show_rtd { {flag 1} }
#     show/hide this widget, bool
# 
#   OPTIONS
#   public variable scrollbars {0}
#     show scrollbars, bool
#
#   public variable width {10}
#     width of Rtd Image (in number of screen bins)
#
#   public variable height {10}
#     width of Rtd Image (in number of screen bins)
#
#   SEE ALSO
#   RtdImage(n), rtdimage(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotRtd {}

class BltplotRtd {
    inherit BltplotBase

    constructor {args} {
	set whelp_ [cget -helparea]

	# create, configure and pack RtdImage widget
	#
	catch {package forget Rtd}  ; # pending bug in seqWish
	set vers [package require Rtd]
	if { $vers < 2.71 } {
	    error_dialog "Need rtd version >= 2.71"
	}

	config -image [rtd::RtdImage $w_.rtd -with_warp 1]
	[$image get_canvas] config -background black

	eval itk_initialize $args
	pack $image -fill both -expand 1

	$w_.rtd config -cameraPostCmd [code "trace_setvar imgEvt 1;"]
    }

    # show/hide this widget and add traces for cursor movements over the image

    public method show_rtd { {flag 1} } {
	if { $flag } {
	    pack $w_ -fill both -expand 1
	    trace_addvar X [code $this update_xyv] $image_
	} else {
	    pack forget $w_
	    trace_remvar X [code $this update_xyv] $image_
	}
    }

    # update x/y/value 

    protected method update_xyv { args } {
	global $image_
	set val [set ${image_}(VALUE)]

	if { "$val" != "" } {
	    $whelp_ displayText \
		"X = [set ${image_}(X)], Y = [set ${image_}(Y)], VALUE = $val"
	} else {
	    $whelp_ displayText {}
	}
    }

    # -- private variables --

    private variable whelp_ {}         ; # help widget

    # -- public variables --

    # show scrollbars
    public variable scrollbars {0} {
	$image config -scrollbars [cget -scrollbars]
    }
    # width and hight of Rtd Image
    public variable width {10} {
	[$image get_canvas] config -width [cget -width]
    }
    public variable height {10} {
	[$image get_canvas] config -height [cget -height]
    }
}

#
# ___bltplot___
