#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: canioxAnalogInputSignal.tcl,v 1.3 2005/09/26 15:16:55 vltsccm Exp $"
#
# Makefile of ........
#
# who       when      what
# --------  --------  ----------------------------------------------
# vltuser5  06/02/02  created
#

#*********************************************************************************
#   NAME
#   
#       CanioxAnalogInputSignal - Class description for an interface object
#       to an input line of an analog I/O device.
#
#   SYNOPSIS
#
#       CanioxAnalogInputSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       CanioxAnalogInputSignal defines the class for an interface object to
#       an  input line of an analog I/O device. 
#       Objects of this class interacts with the user and calls methods
#       of its base class to interact with the "canio" driver.
#      
#       An object of this class is created by:
#
#       Aioox_InputSignal <object-name> -<Option> <Value> ...
#
#   which is typically done by the constructor of the Device object.
#       
#
#   No public interface defined for this class.
#
#   CAUTIONS
#       The class asumes the existence of the global array ${this}, defined in
#       the base class.
#
#   EXAMPLES
#
#   SEE ALSO
#   aioxInducer(1),
#   InducerClass(n), InducerDevice(n), AioxDevice(n)
#       AioxSignal(n), AioxISignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

usual UifOptionButton { }
usual CanioxAnalogInputSignal { keep -borderwidth -relief}


class CanioxAnalogInputSignal {
    inherit CanioxAnalogISignal
    
    constructor {args} { }
    
}

body CanioxAnalogInputSignal::constructor {args} { 
    global ${itk_interior}
    
    itk_option add hull.relief
    itk_option add hull.borderwidth

    set ${itk_interior}(signalGainLiteral) "N/A"
#    set ${itk_interior}(signalP2Literal) "N/A" ;# added by mathar 27 July 1999
    eval itk_initialize $args
    
    # create the widgets for this class:
    # -- Top Label
    itk_component add nr {
    label $itk_interior.nr -text "In $itk_option(-signalNumber)" 
    }
    pack $itk_component(nr) -side top -fill x -expand yes

    $itk_component(nr) configure \
            -foreground white -background #8080FF \
            -activeforeground white -activebackground #8080FF

    # signal name
    itk_component add sn {
    iwidgets::entryfield $itk_interior.sn -labeltext "" \
        -textvariable ${itk_interior}(signalName) -width 18
    } 
    setShortHelp $itk_component(sn) "Signal Name"
    pack $itk_component(sn) -side top -expand yes -padx 5 -pady 5
    
    itk_component add vl {
    iwidgets::entryfield $itk_interior.vl -labeltext "" \
        -textvariable ${itk_interior}(vLevel) -width 18
    }
    setShortHelp $itk_component(vl) "Voltage Level"
    pack $itk_component(vl)  -side top -expand yes  -padx 5 -pady 5
    
    # Signal Range Configuration
#    itk_component add fr2 {
#    frame $itk_interior.fr2  -bd 2  
#    }
#    itk_component add fr2.2 {
#    UifOptionButton $itk_component(fr2).2 -length 3 \
#        -variable ${itk_interior}(signalConfigLiteral) \
#        -itemList $configList \
#        -command [code $this setSignalConfig] 
#    }
#    setShortHelp $itk_component(fr2.2)\
#        "Set Input Range."
    
#    pack $itk_component(fr2) -side left -anchor w -fill x -expand yes -pady 5
#    pack $itk_component(fr2.2) -side left -padx 5

    # Gain Factor
#    itk_component add fr2.4 {
#    UifOptionButton $itk_component(fr2).4 -length 2 \
#        -variable ${itk_interior}(signalGainLiteral) \
#        -itemList $gainList \
#        -command [code $this setSignalGain]
#    }
#    setShortHelp $itk_component(fr2).4 "Set Gain Factor (x1 - x10 - x100 - x500)"

#    pack $itk_component(fr2).4 -side left -padx 5

    # rest of lines added by mathar 27 July 99
    # VMEbus P2 connector cables
#    itk_component add fr2.5 {
#        label $itk_component(fr2).5 -width 0 -anchor w -text "P2:"
#    }
#    itk_component add fr2.6 {
#        UifOptionButton $itk_component(fr2).6 -length 3 \
#                -variable ${itk_interior}(signalP2Literal) \
#                -itemList $p2List \
#                -command [code $this setSignalP2]
#    }
#    setShortHelp $itk_component(fr2).6 \
#        "P2 Connector Line Configuration (differential - single-ended). Not programmable."

#   pack $itk_component(fr2).5 -side left -anchor w -fill x -expand yes 
#    pack $itk_component(fr2).6 -side left
    
}
