#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: canioxAnalogOutputSignal.tcl,v 1.3 2005/09/26 15:16:56 vltsccm Exp $"
#
# Makefile of ........
#
# who       when      what
# --------  --------  ----------------------------------------------
# vltuser5  06/02/02  created
#

#************************************************************************
#   NAME
#   
#       CanioxAnalogOutputSignal - Class description for an interface object
#       to an output line of an analog I/O device.
#
#   SYNOPSIS
#
#       CanioxAnalogOutputSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       Aiox_Outputsignal defines the class for an interface object to
#       an  output line of an analog I/O device. 
#       Objects of this class interacts with the user and calls methods
#       of its base class to interact with the "aio" driver.
#      
#       An object of this class is created by:
#
#       CanioxAnalogOutputSignal <object-name> -<Option> <Value> ...
#
#   which is typically done by the constructor of the Device object.
#       
#
#   No public interface defined for this class.
#
#   CAUTIONS
#       The class asumes the existence of the global array ${this}, defined in
#       base classes.
#
#   EXAMPLES
#
#   SEE ALSO
#   aioxInducer(1),
#   InducerClass(n), InducerDevice(n), AioxDevice(n)
#       canioxAnalogSignal(n), AioxOSignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

usual CanioxAnalogOutputSignal { }


class CanioxAnalogOutputSignal {
    inherit CanioxAnalogOSignal

    constructor {args} {}
}

body CanioxAnalogOutputSignal::constructor {args} { 
    global ::${itk_interior} 
    
    itk_option add hull.relief
    itk_option add hull.borderwidth

    eval itk_initialize $args
    # -- Top Label
    itk_component add nr {
    label $itk_interior.nr -text "Out $itk_option(-signalNumber)"
    }
    pack $itk_component(nr) -side top -fill x -expand yes
    
    $itk_component(nr) configure \
            -foreground white -background #8080FF \
            -activeforeground white -activebackground #8080FF
 

    # "SIGNAL NAME" FRAME:
    itk_component add sn {
#   iwidgets::entryfield $itk_interior.sn -labeltext "Signal Name:"
    iwidgets::entryfield $itk_interior.sn -labeltext "" \
        -textvariable ${itk_interior}(signalName) -width 18
    }
#    pack $itk_component(sn) -side top  -anchor w -fill x -expand yes
    pack $itk_component(sn) -side top -expand yes -padx 5 -pady 5
    
    itk_component add vl {
#   iwidgets::entryfield $itk_interior.vl -labeltext "Level:"
    iwidgets::entryfield $itk_interior.vl -labeltext "" \
        -textvariable ${itk_interior}(vLevel) \
        -command [code $this writeValue] -width 18
    
    }
    setShortHelp $itk_component(vl) \
        "Type a Voltage level. Press <Enter> to set the value"
#    pack $itk_component(vl)  -side top -anchor e -expand yes
    pack $itk_component(vl)  -side top -expand yes -padx 5 -pady 0
    

    # Signal Range Configuration
    itk_component add fr2 {
    frame $itk_interior.fr2  -bd 2  
    }
#    itk_component add fr2.1 {
#   label $itk_component(fr2).1 -width 5 -anchor w -padx 2 -text "Range:"
#    }
#    itk_component add fr2.2 {
#    UifOptionButton $itk_component(fr2).2 -length 3 \
#        -variable ${itk_interior}(signalConfigLiteral) \
#        -itemList $configList \
#        -command [code $this setSignalConfig] 
#    }
#    setShortHelp $itk_component(fr2.2) \
#        "Set Output Range. Not supported."
#    pack $itk_component(fr2) $itk_component(fr2.1)
#    pack $itk_component(fr2) \
#        -side left -anchor w -fill x -expand yes -pady 5
#    pack $itk_component(fr2.2) -side left -padx 5
     pack $itk_component(fr2) \
        -side left -anchor w -fill x -expand yes -pady 5      

    # Write mode selection
    # Float or Dec ==> 
    # WRITE_VALUE or WRITE_BIN_VALUE
    # will be used.
    itk_component add fr2.3 {
    UifRadioButton $itk_component(fr2).3 -label "Float"\
        -variable ${itk_interior}(writeCmd) \
        -onVal "WRITE_VALUE" 
    }
    itk_component add fr2.4 {
    UifRadioButton $itk_component(fr2).4 -label "Int"  \
        -variable ${itk_interior}(writeCmd) \
        -onVal "WRITE_BIN_VALUE"
    }
#    pack $itk_component(fr2).3 -side left -anchor w -fill x -expand yes 
    pack $itk_component(fr2).3 -side left 
    pack $itk_component(fr2).4 -side left
    
}
