#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdConfigRdt.tcl,v 2.7 2005/05/11 03:10:43 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  06/03/97  created
#

#************************************************************************
#   NAME 
#   ccdReadoutInit, ccdFormatRdt, ccdEnableReadout, ccdClearReadout, 
#   ccdSaveReadout, ccdRdtBoxToEdit, ccdRdtModify, ccdRdtDelete
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the ccdConfig panel to handle the
#   readout modes parameters
#
#------------------------------------------------------------------------
#

#
# Initialise variables for readout list box
proc ccdReadoutInit { } {

   global cv gvar
   global rdtRows rdtCol rdtDbAttr rdtWdg rdtFormatProc
   global rdtIndex rdtNames rdtRec rdtButton
   global rdtTable rdtDisp 

   # set names of edit part variables
   set rdtNames(0) ccdRdtRec
   set rdtNames(1) ccdRdtDesc
   set rdtNames(2) ccdRdtSpeed
   set rdtNames(3) ccdRdtGain
   set rdtNames(4) ccdRdtOut1
   set rdtNames(5) ccdRdtEl1
   set rdtNames(6) ccdRdtOut2
   set rdtNames(7) ccdRdtEl2
   set rdtNames(8) ccdRdtOut3
   set rdtNames(9) ccdRdtEl3
   set rdtNames(10) ccdRdtOut4
   set rdtNames(11) ccdRdtEl4
   set rdtNames(12) ccdRdtFirst
   set rdtNames(13) ccdRdtWind
   set rdtNames(14) ccdRdtTime

   # set indexes in a row corresponding to ccdName
   for { set i 0 } { $i < 15 } {incr i } {
     set rdtIndex($i) $i
   }

   # define name of buttons
   set rdtButton(0) ccdRdtAdd
   set rdtButton(1) ccdRdtDel

   # define record templates
   set rdtRec(clean) {0 "" "" 0 0 0 0 0 0 0 0 0 0 0 0}; # when removing records
   set rdtRec(mask)  {0 "" "" 0 0 0 0 0 0 0 0 0 0 0 0}; # when adding records


  set rdtRows 10;  # maximum number of rows, as from DB
  set rdtCol 15;   # maximum number of columns, as from DB
  set rdtWdg $cv.ccdRdtModes
  set rdtFormatProc ccdFormatRdt

  # set path to DB values
  set rdtDbAttr(avail) readout.available
  set gvar($rdtDbAttr(avail)) 1; # readout is always enabled
  for { set i 0 } { $i < $rdtRows } {incr i } {
      set rdtDbAttr($i,availInd) 1; # available flag is at position 2 in record
      set rdtDbAttr($i,availVal) ""; # not available value
      set k [expr {$i+1}]
      set rdtDbAttr($i,1)  readout:clock_$k.description
      set rdtDbAttr($i,2)  readout:clock_$k.speed
      set rdtDbAttr($i,3)  readout:clock_$k.gainIndex
      set rdtDbAttr($i,4)  readout:clock_$k.outEnab(0)
      set rdtDbAttr($i,5)  readout:clock_$k.elAdu(0)
      set rdtDbAttr($i,6)  readout:clock_$k.outEnab(1)
      set rdtDbAttr($i,7)  readout:clock_$k.elAdu(1)
      set rdtDbAttr($i,8)  readout:clock_$k.outEnab(2)
      set rdtDbAttr($i,9)  readout:clock_$k.elAdu(2)
      set rdtDbAttr($i,10) readout:clock_$k.outEnab(3)
      set rdtDbAttr($i,11) readout:clock_$k.elAdu(3)
      set rdtDbAttr($i,12) readout:clock_$k.firstOutput
      set rdtDbAttr($i,13) readout:clock_$k.nWindows
      set rdtDbAttr($i,14) readout:clock_$k.adcTime
  }

  # attach call to selection of one row
  $rdtWdg setBind <B1-ButtonRelease> {ccdRdtBoxToEdit}
}



# formats one row in the readout mode list box
proc ccdFormatRdt { text } {

  global rdtIndex
 
  for { set i 1 } { $i < 5 } {incr i } {
      set pos [expr {2+2*$i}]
      set out$i ""
      set el$i  ""
      if {[lindex $text $pos] == 1} { 
	  set out$i "Y" 
	  incr pos
	  set el$i [lindex $text $pos]
      }
  }
  set recInd   $rdtIndex(0)
  set descInd  $rdtIndex(1)
  set speedInd $rdtIndex(2)
  set gainInd  $rdtIndex(3)
  set firstInd $rdtIndex(12)
  set windInd  $rdtIndex(13)
  set timeInd  $rdtIndex(14)
   
  return [format " %-3s %-13s %-8s %-4s %-1s %-5s %-1s %-5s %-1s %-5s %-1s %-5s %-5s %-4s %-2s" \
         [lindex $text $recInd] [lindex $text $descInd] \
         [lindex $text $speedInd] [lindex $text $gainInd] \
         $out1 $el1 $out2 $el2 $out3 $el3 $out4 $el4 \
         [lindex $text $firstInd] [lindex $text $windInd] \
         [lindex $text $timeInd]]
}



#
# Enable/disable readout modes in the panel ccdConfig
proc ccdEnableReadout {  } {

  global rdtRows rdtCol rdtDbAttr rdtWdg rdtFormatProc
  global rdtButton
  global rdtTable rdtDisp 

  ccdEnableBox rdtTable rdtDisp rdtButton $rdtRows $rdtCol rdtDbAttr $rdtWdg $rdtFormatProc

  # determine camera type and update option buttons accordingly
  ccdRdtEditOpt rdtTable rdtDbAttr
}


#
# Reset the readout modes list box and optionally clean table
proc ccdClearReadout { } {
   global rdtTable rdtWdg rdtRec

   ccdClearBox $rdtWdg rdtTable $rdtRec(clean)
}


#
# Stores the current readout modes table into the database
proc ccdSaveReadout { } {
 
  global rdtTable rdtRows rdtCol rdtDbAttr rdtRec
  
  ccdSaveBoxToDb rdtTable $rdtRows $rdtCol rdtDbAttr $rdtRec(clean)
}


#
# Show in the edit section the selected record from the readout modes list
proc ccdRdtBoxToEdit { } {

  global rdtNames rdtIndex rdtTable rdtWdg

  ccdBoxToEdit rdtNames rdtIndex rdtTable $rdtWdg
}



#
#
# Add or modify one entry in the readout modes list box
proc ccdRdtModify { } {

  global rdtWdg
  global rdtIndex rdtNames rdtRec
  global rdtTable rdtDisp rdtFormatProc

  ccdEditToBox rdtNames rdtIndex rdtTable rdtDisp $rdtWdg $rdtFormatProc rdtRec 1
}



#
# Delete one entry in the readout modes list box
proc ccdRdtDelete { } {

  global rdtWdg
  global rdtIndex rdtNames rdtRec
  global rdtTable rdtDisp rdtFormatProc

  ccdEditToBox rdtNames rdtIndex rdtTable rdtDisp $rdtWdg $rdtFormatProc rdtRec 0
}





# Determine camera type and Update standard speed and gain option buttons 
# in edit part
proc ccdRdtEditOpt { table dbAttr } {

   global cv gvar
   global ccdRdtStdSpeed
   global ccdTecGain ccdSciGain

   upvar $table  ccdTable 
   upvar $dbAttr ccdDbAttr 

   # determine if technical or scientific camera
   set name [lindex $ccdTable(0) $ccdDbAttr(0,availInd)]
   if {[string match ccd*.clk $name]} {
       set gvar(ccdCamType) "Sci"
       set gvar(ccdCtrlDev) "bxvi"
   } else {   
       set gvar(ccdCamType) "Tec"
       if {[string match acetec*.clk $name]} {
	   set gvar(ccdCtrlDev) "bxvi"
       } 
       if {[string match acetec*.lod $name]} {
	   set gvar(ccdCtrlDev) "arcdrv"
       }
   }    

   # update option buttons 
   if {$gvar(ccdCamType) == "Tec"} {
       set gainList $ccdTecGain
   } 
   if {$gvar(ccdCamType) == "Sci"} {
       set gainList $ccdSciGain
   }

   $cv.ccdRdtNSpeed configure -itemList $ccdRdtStdSpeed
   $cv.ccdRdtNGain  configure -itemList $gainList
}


#
# ___oOo___
