#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdCtrlMulti.tcl,v 2.7 2005/05/11 03:10:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  13/10/97  created
#

#************************************************************************
#   NAME 
#   ccdMstInit, ccdMstFormat, ccdMstClear, ccdMstSave, ccdMstBoxToEdit,
#   ccdMstModify, ccdMstDelete
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the ccdCtrl panel to handle the
#   multi-step exposure setup table
#
#------------------------------------------------------------------------
#

#
# Initialise variables for setup list box
proc ccdMstInit { } {

   global cv gvar
   global mstRows mstCol mstWdg mstFormatProc
   global mstIndex mstNames mstRec mstButton
   global mstTable

   # set names of edit part variables
   set mstNames(0) ccdExpMulti_multiRec
   set mstNames(1) ccdExpMulti_multiTim
   set mstNames(2) ccdExpMulti_multiRow

   # set indexes in a row corresponding to ccdName
   for { set i 0 } { $i < 3 } {incr i } {
     set mstIndex($i) $i
   }

   # define name of buttons
   set mstButton(0) multiMod
   set mstButton(1) multiDel

   # define record templates
   set mstRec(clean) {}; # when removing records
   set mstRec(mask)  {0 0 0}; # when adding records

   set mstRows 10;  # maximum number of rows
   set mstCol 3;   # maximum number of columns
   set mstFormatProc ccdMstFormat; # formatting procedure

   # initialise internal table
   for { set i 1 } { $i < $mstRows } {incr i } {
     set mstTable($i) {}
   }

   # attach call to selection of one row
   set widget [$cv.ccdExpMulti getClassPath]
   set mstWdg $widget.cv.multiSetup
   $mstWdg setBind <B1-ButtonRelease> {ccdMstBoxToEdit}
}



# formats one row in the readout mode list box
proc ccdMstFormat { text } {

  global mstIndex
 
  set recInd $mstIndex(0)
  set timInd $mstIndex(1)
  set rowInd $mstIndex(2)
   
  return [format "%2s %-5s %-3s" \
         [lindex $text $recInd] [lindex $text $timInd] \
         [lindex $text $rowInd]]
}

#
# Reset the readout modes list box and optionally clean table
proc ccdMstClear { } {
   global mstTable mstWdg mstRec

   ccdClearBox $mstWdg mstTable $mstRec(clean)
}


#
# Stores the current readout modes table into the database
proc ccdMstSave { } {
 
  global mstTable mstRows mstCol mstDbAttr mstRec
  
#  ccdSaveBox mstTable $mstRows $mstCol mstDbAttr $mstRec(clean)
}


#
# Show in the edit section the selected record from the readout modes list
proc ccdMstBoxToEdit { } {

  global mstNames mstIndex mstTable mstWdg

  ccdBoxToEdit mstNames mstIndex mstTable $mstWdg
}



#
#
# Add or modify one entry in the readout modes list box
proc ccdMstModify { } {

  global mstWdg
  global mstIndex mstNames mstRec
  global mstTable mstDisp mstFormatProc

  ccdEditToBox mstNames mstIndex mstTable mstDisp $mstWdg $mstFormatProc mstRec 1
}



#
# Delete one entry in the readout modes list box
proc ccdMstDelete { } {

  global mstWdg
  global mstIndex mstNames mstRec
  global mstTable mstDisp mstFormatProc

  ccdEditToBox mstNames mstIndex mstTable mstDisp $mstWdg $mstFormatProc mstRec 0
}



#
# Add to oslx buffer information about sub-integrations
proc ccdMstAddToBuf { oslxBuf } {
  global gvar
  global mstRows mstCol mstTable

  # only add info if really multi-step exposure is selected
  if {![string compare $gvar(ccdExpOpt_setup.0.DET.EXP.TYPE) Multiple]} {
    set ndit $gvar(ccdExpMulti_setup.1141.DET.WIN1.NDIT)
    set asuit $gvar(ccdExpMulti_setup.1142.DET.WIN1.ASUIT1)
    set shiftype $gvar(ccdExpMulti_setup.1143.DET.READ.SHIFTYP)
    for { set i 1 } { $i < $mstRows } {incr i } {
      # add infor only for valid entries. Ignore empty entries
      set record [split $mstTable($i)]
      if {[llength $record] == $mstCol} {
        set rec [lindex $record 0]
	if {$rec <= $ndit} {
	  if {($rec == 1) || (![string compare $asuit F])} {
            set tim [lindex $record 1]
            set oslxBuf "$oslxBuf DET.WIN1.UIT$rec $tim"
          }
	  if {($rec == 1) || (![string compare $shiftype list])} {
            set row [lindex $record 2]
            set oslxBuf "$oslxBuf DET.READ.SHIFT$rec $row"
          }
        }
      }
    }
  }
  return $oslxBuf
}



#
# Enter into sub-integration box information from oslx buffer
proc ccdMstBufToBox { oslxBuf } {
   global mstTable mstCol mstWdg mstFormatProc mstDisp mstDbAttr

  ccdMstClear
  set keyL [split $oslxBuf ","]; # split string in a list
  set keyL [lreplace $keyL 0 0]; # remove first element (number of keywords)
  foreach key $keyL {;           # for each pair name + value ...
    set keyNV [split $key];      # split name and value
    set keyN [lindex $keyNV 0]
    set keyV [lindex $keyNV 1]
    set index 0
    if {[string match DET.WIN1.UIT* $keyN]} {
      set sub 12
      set index 1
    } elseif {([string match DET.READ.SHIFT* $keyN]) && \
              (![string match DET.READ.SHIFTYP $keyN])} {
      set sub 14
      set index 2
    }
    if {$index > 0} {
      set rec [string range $keyN $sub end]
      if {![info exists mstTable($rec)]} { set mstTable($rec) {} }
      set record [split $mstTable($rec)]
      if {[llength $record] != $mstCol} {
        set record "$rec 0 0"
      }
      set record [lreplace $record $index $index $keyV]
      set mstTable($rec) [join $record]
    }
  }
  ccdTableToDisplay mstTable mstDisp mstDbAttr $mstWdg $mstFormatProc
}



#
# ___oOo___
