
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdExpCtrlUpdate.tcl,v 2.7 2005/05/11 03:10:37 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/09/95  created
# alongino  05/07/96  updated with new expStatus values 1024 1nd 2048
# alongino  08/07/96  introduced ccdGetOpState
# alongino  30/10/96  added check if panel is still up (SPR 960660)
# alongino  10/03/97  added disabling telemetry button
# alongino  13/08/97  operational state values taken from ccdUsrGlobalData
#

#************************************************************************
#   NAME
#   ccdExpCtrlUpdate - update en./dis. buttons according to exposure status
# 
#   SYNOPSIS
#   ccdExpCtrlUpdate
# 
#   DESCRIPTION
#   Tcl procedure used within CCD OS GUI Exposure Control panels to 
#   update enabling/disabling buttons according to exposure status
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc ccdExpCtrlUpdate { } {               

   global gvar ccdExpStatusAttr ccdReadStateAttr
   global ccdLoopFlag

# SPR 960660. If the panel is unloaded from the panel editor, then an
#             error would occur,
   if {[info exists gvar(ccdContext)] == 0} { return }

   ccdGetOpState

   if {$gvar(ccdContext)} {;            # context sensitive
     set expStatus [dbRead $ccdExpStatusAttr]
     set readState [dbRead $ccdReadStateAttr]
     set gvar(sysFail) [dbRead ".failureLcu"]

     set gvar(ccdExpCtrl_running)   0  ;                 # default idle
     set gvar(ccdExpCtrl_notRunning) 1
     set gvar(ccdExpCtrl_notPaused) 0
     set gvar(ccdExpCtrl_paused)    0
     set gvar(ccdExpCtrl_loopRunning) 0
     set gvar(ccdExpCtrl_anyRunning) 0
     set gvar(ccdExpCtrl_ccdTel) 1
     if {$gvar(opState) != $gvar(ccdSTATE_ONLINE)} {;  # not online
        set expStatus 0
        set gvar(ccdExpCtrl_notRunning) 0
        set gvar(ccdExpCtrl_ccdTel) 0
     }
     if {(($expStatus >= 2) && ($expStatus < 128)) || ($expStatus >= 1024)} {;  # running
       set gvar(ccdExpCtrl_notRunning) 0
       if {($expStatus >=4) && (($expStatus < 16) || ($expStatus >= 1024))} {; # still integrating
         if {$expStatus & 4} {;                       # integrating
           if {($gvar(shutAvai) == 1) && !($expStatus & 8)} {; # not paused 
             set gvar(ccdExpCtrl_notPaused) 1
           }
         }
         if {$expStatus & 8} {;                       # paused
           set gvar(ccdExpCtrl_paused) 1
         }
         if {[info exists ccdLoopFlag] == 0} { set ccdLoopFlag 0 }
         set gvar(ccdExpCtrl_anyRunning) 1
         if {($expStatus >= 1024) && ($ccdLoopFlag == 0)} {;  # loop running
           set gvar(ccdExpCtrl_loopRunning) 1
         } else {; # abort and stop integr active only if loop stopped
           set gvar(ccdExpCtrl_running) 1
         }
       }
     }
     if {$gvar(ccdExpCtrl_notRunning) == 1} {
       set ccdLoopFlag 0
       set gvar(ccdExpCtrl_wiping) 0
       set gvar(ccdExpCtrl_notWiping) 1
       if {$readState == $gvar(ccdSTATE_ONLINE_WIPING)} {      
         set gvar(ccdExpCtrl_wiping) 1
         set gvar(ccdExpCtrl_notWiping) 0
         set gvar(ccdExpCtrl_notRunning) 0
       } 
     } else {;                 # exposure active
       set gvar(ccdExpCtrl_wiping) 0
       set gvar(ccdExpCtrl_notWiping) 0       
     }
   } else {;                   # buttons always enabled
     set gvar(ccdExpCtrl_wiping) 1
     set gvar(ccdExpCtrl_notWiping) 1     
     set gvar(ccdExpCtrl_notRunning) 1
     set gvar(sysFail) 1
     set gvar(ccdExpCtrl_running) 1
     set gvar(ccdExpCtrl_notPaused) 1
     set gvar(ccdExpCtrl_paused) 1
     set gvar(ccdExpCtrl_loopRunning) 1
     set gvar(ccdExpCtrl_anyRunning) 1
     set gvar(ccdExpCtrl_ccdTel) 1
   }

   update idletasks;           # update buttons status

# do update of buttons periodically every 2 secs
   after 2000 ccdExpCtrlUpdate
}

#
# ___oOo___


