
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdExpSim.tcl,v 2.7 2005/05/11 03:10:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/09/95  created
# alongino  05/07/96  updated with new expStatus values 1024 1nd 2048
# alongino  08/07/96  introduced ccdGetOpState
# alongino  26/02/97  introduced second window readout and image processing
# alongino  20/05/97  added logic for change exposure time
#

#************************************************************************
#   NAME
#   ccdExpSim - simulate an exposure within the CCD stand-alone panel
# 
#   SYNOPSIS
#   ccdExpSim <dump> <step>
# 
#   DESCRIPTION
#   Tcl procedure used within CCD GUI panel ccdCtrl to simulate
#   the execution of an exposure
#   <dump> flag indicating if only image dump is requested
#   <step> index of the current exposure step
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc ccdExpSim { dump step } {
   global env gvar simul ccdExpStatusAttr ccdExpStatus
   global ccdTimeDef ccdTimeRem ccdNumRep ccdNumTot
   global ccdAbortFlag ccdStopFlag ccdChangeFlag ccdChangeRate ccdPauseFlag ccdLoopFlag
   global ccdCentro ccdMinMax ccdMin ccdMax ccdErrX ccdErrY
   global ccdMaxWin

# simulate an exposure
  set attrShut "shutter.status"
  set attrFile "exposures:exposure_1:transfer.fileNameUnComp"
  set attrTDef "exposures:exposure_1.timeDef"
  set attrTime "exposures:exposure_1.timeRem"
  set attrRDef "exposures:exposure_1.expRepeatDef"
  set attrRCur "exposures:exposure_1.expRepeatCur"
  set attrTran "images:transfer.percent"
  set attrTrtm "images:transfer.time"
  for { set i 1} { $i <= $ccdMaxWin } {incr i} {
    set attrPMin($i) "images:process:window_${i}.ipMinVal"
    set attrPMax($i) "images:process:window_${i}.ipMaxVal"
    set attrPXCe($i) "images:process:window_${i}.ipXCen"
    set attrPYCe($i) "images:process:window_${i}.ipYCen"
  }
  set exitStatus 128; # default success
  set delay 1; # default no delay
  set pollRate 2;  # 2 sec polling rate
  if {[info exist ccdAbortFlag]} {
     if {$ccdAbortFlag == 1} {
#puts "abort flag set"
        set step 4
        set exitStatus 512
        set ccdAbortFlag 0
        set ccdPauseFlag 0
     }
  }
  if {[info exist ccdStopFlag]} {
     if {$ccdStopFlag == 1} {
#puts "stop flag set"
        set step 2
        set ccdTimeRem $pollRate
        set ccdStopFlag 0
        set ccdPauseFlag 0
     }
  } 
  if {[info exist ccdChangeFlag]} {
     if {$ccdChangeFlag == 1} {
#puts "change time flag set"
        set step 2
        set ccdTimeNew $gvar(setup.1145.DET.WIN1.UIT1)
        set ccdTimeAdd [expr {$ccdTimeNew - $ccdTimeDef}]
	set ccdTimeRem [expr {$ccdTimeRem + $ccdTimeAdd}]
	if {$ccdTimeRem < $pollRate} {set ccdTimeRem $pollRate}
        set ccdTimeDef $ccdTimeNew
        set ccdChangeFlag 0
     }
  } 
  if {[info exist ccdPauseFlag]} {
     if {$ccdPauseFlag == 1} {
#puts "pause flag set"
        set step 2
        set ccdTimeRem [expr {$ccdTimeRem + $pollRate}]
     }
  }
  if {[info exist ccdLoopFlag]} {
     if {$ccdLoopFlag == 1} {
#puts "stop loop flag set"
        set ccdNumTot $ccdNumRep
        set ccdLoopFlag 0
     }
  }
  if {$step == 0} {;   # extract setup information
     set ccdTimeDef $gvar(setup.1145.DET.WIN1.UIT1)
     set ccdNumTot $gvar(setup.2.DET.EXP.NREP)
     for { set i 1} { $i <= $ccdMaxWin } {incr i} { 
       set ccdMinMax($i) $gvar(ccdSetupIp${i}_ipMinMax)
       if {![string compare $ccdMinMax($i) F]} {set ccdMinMax($i) 0
       } else {set ccdMinMax($i) 1}
       set ccdCentro($i) $gvar(ccdSetupIp${i}_ipCentroid)
       if {![string compare $ccdCentro($i) none]} {set ccdCentro($i) 0
       } else {set ccdCentro($i) 1}
       set ccdMin($i) 100
       set ccdMax($i) 10000
       set ccdErrX($i) 0
       set ccdErrY($i) -3
     }
     set ccdNumRep 0
     panDbWrite $attrTDef $ccdTimeDef
     panDbWrite $attrRDef $ccdNumTot
     panDbWrite $attrFile $gvar(setup.8.DET.FRAM.FITSUNC)
  } elseif {$step == 1} {;   # start integration
     if {$dump != 1} {
       set ccdPauseFlag 0
       set ccdTimeRem $ccdTimeDef
       incr ccdNumRep
       panDbWrite $attrTime $ccdTimeRem
       panDbWrite $attrRCur $ccdNumRep
       panDbWrite $attrTran 0;        # reset transfer percentage
       if {$ccdNumTot == 1} {
           set ccdExpStatus 4;     # integrating
       } elseif {$ccdNumTot == 0} {
           set ccdExpStatus 2048;  # infinite loop
       } elseif {$ccdNumTot > 0} {
           set ccdExpStatus 1024;  # finite loop
       }
       panDbWrite $ccdExpStatusAttr $ccdExpStatus;
       panDbWrite $attrShut 1;        # shutter open
       set delay [expr {1000 * $pollRate}];
     }
  } elseif {$step == 2} {; # integrating
      if {$dump !=1} {
	  set ccdTimeRem [expr {$ccdTimeRem - $pollRate}]
	  if {$ccdTimeRem > 0} {
	      if {$pollRate > $ccdTimeRem} {
		  set pollRate [expr {int($ccdTimeRem)}]
		  if { $pollRate < 1 } { set pollRate 1 }
	      }
	      set delay [expr {1000 * $pollRate}];
	      incr step -1
	  } else {
	      set ccdTimeRem 0
	  }
	  panDbWrite $attrTime $ccdTimeRem;
	  if {$ccdPauseFlag == 0} {
	      if {$ccdNumTot == 1} {set ccdExpStatus 4}
	      if {$ccdTimeRem > 0} {panDbWrite $attrShut 1};   # shutter open
	  } else {
	      if {$ccdNumTot == 1} {set ccdExpStatus 8}
	      panDbWrite $attrShut 0;        # shutter close
	  }
	  panDbWrite $ccdExpStatusAttr $ccdExpStatus
      }
  } elseif {$step == 3} {; # read and transfer
     panDbWrite $attrTime 0;        # no remaining time
     panDbWrite $attrShut 0;        # shutter closed
     if {$ccdNumTot == 1} {
         set ccdExpStatus 80
         if {$dump == 1} {set ccddExpStatus 64}
     }
     panDbWrite $ccdExpStatusAttr $ccdExpStatus; # read & transfer
     panDbWrite $attrTran 20;       # 20 percent transfered
     set delay 2000
  } elseif {$step == 4} {; # exposure completed
     panDbWrite $attrTran 100;      # whole image transfered
     panDbWrite $attrTrtm 2.5;      # time needed for transfer
     panDbWrite $attrTime 0;        # time remaining
     panDbWrite $attrShut 0;        # shutter close
     for { set i 1} { $i <= $ccdMaxWin } {incr i} { 
       if {$ccdCentro($i)} {
         incr ccdErrX($i)
         if {$ccdErrX($i) > 3} {set ccdErrX($i) -3}
         panDbWrite $attrPXCe($i) $ccdErrX($i)
         incr ccdErrY($i)
         if {$ccdErrY($i) > 3} {set ccdErrY($i) -3}
         panDbWrite $attrPYCe($i) $ccdErrY($i)
       }
       if {$ccdMinMax($i)} {
         incr ccdMin($i)
         if {$ccdMin($i) > 110} {set ccdMin($i) 90}
         panDbWrite $attrPMin($i) $ccdMin($i)
         incr ccdMax($i)
         if {$ccdMax($i) > 10010} {set ccdMax($i) 9990}
         panDbWrite $attrPMax($i) $ccdMax($i)
       }
     }
     if {$ccdNumTot != 0} {
	 set numTot $ccdNumTot
     } else {
	 set numTot [expr {$ccdNumRep + 1}]
     }
     if {$ccdNumRep == $numTot} {
         panDbWrite $ccdExpStatusAttr $exitStatus;# file transfer complete
     }
  }
  if {$step < 4} {; # go to next step
     incr step
     after $delay ccdExpSim $dump $step
  } elseif {($dump != 1) && ($numTot > $ccdNumRep)} {
#puts "$ccdNumTot $numTot $ccdNumRep"
     after $delay ccdExpSim $dump 1 
  }
}


#
# ___oOo___


