#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdIpPlot.tcl,v 2.7 2005/05/11 03:10:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  30/10/97  created
#

#************************************************************************
#   NAME 
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the ccdIpPlot panel to handle the 
#   plotting of CCD real-time image processing results
#
#------------------------------------------------------------------------
#

#
# Initialise variables and widgets
proc ccdIpPlotInitialize { } {

    global gvar camera lcuenv
    
    # define name of table to be used for plotting
    catch {seq_dbGetAlias "\<alias\>$camera:images:process:plot"} plotAlias
    set gvar(plotTable) [string range $plotAlias 7 end]

    catch {seq_dbGetCwp $lcuenv} curCwp
    catch {seq_dbSetCwp "@$lcuenv\<alias\>$camera"}
    catch {seq_dbGetCwp $lcuenv} ccdPath
    catch {seq_dbSetCwp $curCwp}
    set ccdPath1 [format "%s:images:process:window_1" $ccdPath]
    set ccdPath2 [format "%s:images:process:window_2" $ccdPath]

    # define all possible attributes which can be sampled
    set gvar(options) [list \
      [list ""  ""        ""]                                                 \
      [list "1" "Minimum"          "$ccdPath1.ipMinVal"]\
      [list "1" "Min. Pos. X"      "$ccdPath1.ipXMin" ] \
      [list "1" "Min. Pos. Y"      "$ccdPath1.ipYMin" ] \
      [list "1" "Maximum"          "$ccdPath1.ipMaxVal"]\
      [list "1" "Max. Pos. X"      "$ccdPath1.ipXMax" ] \
      [list "1" "Max. Pos. Y"      "$ccdPath1.ipYMax" ] \
      [list "1" "Average Flux"     "$ccdPath1.ipFlux" ] \
      [list "1" "Flux Std Dev"     "$ccdPath1.ipStdDev"]\
      [list "1" "Background"       "$ccdPath1.ipBGnd" ] \
      [list "1" "BGnd Std Dev"     "$ccdPath1.ipBGnd_SD"]\
      [list "1" "SNR"              "$ccdPath1.ipSNR" ]  \
      [list "1" "Centr. error X"   "$ccdPath1.ipXCen" ] \
      [list "1" "Centr. error Y"   "$ccdPath1.ipYCen" ] \
      [list "1" "Centr. intensity" "$ccdPath1.ipCenVal"]\
      [list "1" "Valid Pixels"     "$ccdPath1.ipNumPix"]\
      [list "1" "X FWHM est."      "$ccdPath1.ipFWHM_X"]\
      [list "1" "Y FWHM est."      "$ccdPath1.ipFWHM_Y"]\
      [list "2" "Minimum"          "$ccdPath2.ipMinVal"]\
      [list "2" "Min. Pos. X"      "$ccdPath2.ipXMin" ] \
      [list "2" "Min. Pos. Y"      "$ccdPath2.ipYMin" ] \
      [list "2" "Maximum"          "$ccdPath2.ipMaxVal"]\
      [list "2" "Max. Pos. X"      "$ccdPath2.ipXMax" ] \
      [list "2" "Max. Pos. Y"      "$ccdPath2.ipYMax" ] \
      [list "2" "Average Flux"     "$ccdPath2.ipFlux" ] \
      [list "2" "Flux Std Dev"     "$ccdPath2.ipStdDev"]\
      [list "2" "Background"       "$ccdPath2.ipBGnd" ] \
      [list "2" "BGnd Std Dev"     "$ccdPath2.ipBGnd_SD"]\
      [list "2" "SNR"              "$ccdPath2.ipSNR" ]  \
      [list "2" "Centr. error X"   "$ccdPath2.ipXCen" ] \
      [list "2" "Centr. error Y"   "$ccdPath2.ipYCen" ] \
      [list "2" "Centr. intensity" "$ccdPath2.ipCenVal"]\
      [list "2" "Valid Pixels"     "$ccdPath2.ipNumPix"]\
      [list "2" "X FWHM est."      "$ccdPath2.ipFWHM_X"]\
      [list "2" "Y FWHM est."      "$ccdPath2.ipFWHM_Y"]\
    ]
    # configure option buttons
    ccdIpConfigButtons $gvar(options)
    # define image directory
    set gvar(imageDir) [dbRead "\<alias\>$camera:images.imageDirectory"]
}

# Configure option buttons
proc ccdIpConfigButtons { options } {
    global cv gvar
    
    set optList {}
    foreach rec $options {
	set currOpt [lindex $rec 1] 
	if {[lsearch -exact $optList $currOpt] == -1} {
	    lappend optList $currOpt
	}
    }
    for { set i 1} { $i <= 4 } {incr i} {
	$cv.ipSamp$i configure -itemList $optList
    }
}

# start sampling image processing results
proc ccdIpSampleStart { } {
    global gvar camera lcuenv env
    
    set attr ""
    for { set i 1} { $i <= 4 } {incr i} {
	set window $gvar(ipWind$i)
	set param  $gvar(ipSamp$i)
	set dbAttr [ccdIpAttrFromLabel $gvar(options) $window $param ]
	if {[string length $dbAttr] > 0 } {
# 	    set attr "$attr \"$dbAttr\" -l \"$param$window\""
	    set attr "$attr \"$dbAttr\""
	}
    }
    if {[string length $attr] == 0} {
	puts "no element selected for plotting"
    } else {
	if {$gvar(saveInFile) == 0} {
	    set fileName "none"
	} else {
	    set isoTimeStamp [exec date -u +"%Y-%m-%dT%X"]
	    set fileName "$gvar(imageDir)/${camera}_ip.samp.${isoTimeStamp}"
	}
	eval exec sampCtrl_start.sh $gvar(saveInFile) $fileName "$gvar(plotTable)" $lcuenv LCU $gvar(rate) 3 "$attr" &;
    }
}

# get attribute name from label
proc ccdIpAttrFromLabel { options window param} {
    global gvar
    
    set attr ""
    foreach rec $options {
	set currWindow [lindex $rec 0]
	set currParam [lindex $rec 1]
	if {([string compare $currWindow $window] == 0) && \
		([string compare $currParam $param] == 0) } {
	    set attr [lindex $rec 2] 
	    break
	}
    }
    return $attr
}

# stop sampling image processing results
proc ccdIpSampleStop { } {
    global gvar lcuenv
    
    exec sampCtrl_stop.sh "$gvar(plotTable)" $lcuenv LCU &;
}

#
# ___oOo___
