#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdListBox.tcl,v 2.7 2005/05/11 03:10:43 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/03/97  created
# alongino  20/03/97  ccdDisplayBox updated
# alongino  15/10/97  merged with ccdConfigListBox
#

#************************************************************************
#   NAME
#   ccdDbToTable, ccdTableAvail, ccdDisplayBox, ccdBoxToEdit, ccdEditToBox,
#   ccdEditToRec, ccdSaveBoxToDb, ccdClearBox, ccdClearTable, ccdLoadBoxFromDb,
#   ccdEnableBox, ccdTableToDisplay
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the CCD panels to handle list boxes
#
#------------------------------------------------------------------------
#

#
# Read data from OLDB and build internal table
proc ccdDbToTable { table rows columns dbAttr } {

   upvar $table  ccdTable 
   upvar $dbAttr ccdDbAttr 

   # Read element by element from OLDB and set internal table
   for { set i 0 } { $i < $rows } {incr i } {
     set curRow $i
     if {[info exists ccdDbAttr($i)]} {; # row can be treated in one go
       set tmp [dbRead $ccdDbAttr($i)]
       set curRow [concat $curRow $tmp]
     } else {
       for { set j 1 } { $j < $columns } {incr j } {
         set tmp [dbRead $ccdDbAttr($i,$j)]
         set curRow [lappend curRow $tmp]
       } 
     } 
     set ccdTable($i) $curRow
   }
}




#
# Extract from internal table only entries which are available
proc ccdTableAvail { table disp dbAttr} {

  upvar $table ccdTable
  upvar $disp ccdDisp
  upvar $dbAttr ccdDbAttr

  if {[info exists ccdTable] == 0 } { return }
  if {[info exists ccdDisp]} { unset ccdDisp }
   
  # Build display list from internal table
  set elList [array names ccdTable]
  if {[llength $elList] > 0} {
    foreach rec $elList {
      set add 0
      if {[info exists ccdDbAttr($rec,availInd)]} {
        if {[llength $ccdTable($rec)] > $ccdDbAttr($rec,availInd)} {
          if {[lindex $ccdTable($rec) $ccdDbAttr($rec,availInd)] \
             != $ccdDbAttr($rec,availVal)} {; # available set
            set add 1
          }
        }
      } elseif {[llength $ccdTable($rec)] > 0} {
        set add 1
      }
      if {$add} {
        set ccdDisp($rec) $ccdTable($rec)
      }
    }
  }
}





#
# Display the data stored in the internal table
proc ccdDisplayBox { disp wdg formatProc } {

  upvar $disp ccdDisp

  # now update list box widget
  set posBar [$wdg component uifListBox yview]
  set yPos [lindex $posBar 0]; # get current position of cross bar
  $wdg deleteItem 0 end ; # Reset list box    
  if {[info exists ccdDisp]} {;   #AL960214 added check if DB empty
     $wdg updateSortList -increasing -integer ccdDisp $formatProc
     $wdg component uifListBox yview moveto $yPos
  }
}





# Show in the edit section the record selected from the list
proc ccdBoxToEdit { name index table wdg } {

   global gvar

   upvar $name ccdName
   upvar $index ccdIndex
   upvar $table ccdTable

   if {[set tmp [$wdg getSelection]] != "" } {
#      if {[lindex $tmp 0] == "N/A"} {   
#          set tmp [lrange $tmp 1 end]
#      }
      set idx [lindex $tmp 0]
      set ccdRecord $ccdTable($idx)
      set col [array size ccdName]
      for { set i 0 } { $i < $col } {incr i } {
	  set ind $ccdIndex($i)
	  if {$ind >= 0} {; # simple value
	      set val [lindex $ccdRecord $ind]
	  } else {; # combined value   e.g. clock_3_H
	      set ind [expr {-1 * $ind}]
	      set piece [split [lindex $ccdRecord $ind] _]; # use _ as separator
	      set val [lindex $piece 1];  # extract number in string
	      set piece [lreplace $piece 1 1]; # take out number
	      set gvar($ccdName($i)) [join $piece _]; # build string with the rest
	      incr i; # set next value
	  }
	  set gvar($ccdName($i)) $val
      }
      #      return
  } 
   
#   if {[info exists ccdRecord] == 0 } { 
#      set ccdRecord ""
#      if {[info exists ccdTable]} {
#         set tmp          [lsort -integer [array names ccdTable]]
#         set idx          [lindex $tmp 0]
#         set ccdRecord $ccdTable($idx)
#      }
#   }
}






#
# Changes the contents of a list box with the contents of the edit section
proc ccdEditToBox { name index table disp wdg formatProc templ {modify 1}} {
   
   global gvar

   upvar $name ccdName
   upvar $index ccdIndex
   upvar $table ccdTable
   upvar $disp ccdDisp
   upvar $templ ccdTempl

   set rec $gvar($ccdName(0))
   if {[info exists ccdDisp]} {; #AL960214 added check if DB empty
      set tmp [lsort -integer [array names ccdDisp]]
   } else {
      set tmp [list];                     # empty list
   }
   set len [llength $tmp]

   set idx [lsearch $tmp $rec]
   if {$idx != -1 } {;                    # entry already exists; modify
      $wdg deleteItem $idx
      unset ccdDisp($rec)
   } elseif {$rec <= [lindex $tmp 0]} {;  # search for right place to add
       set idx 0   
   } elseif {$rec >= [lindex $tmp [expr {$len - 1}]]} {
       set idx $len
   } else {
       for {set i 0} { $i < $len } { incr i } {
	   set min [lindex $tmp $i]
	   set max [lindex $tmp [expr {$i + 1} ]]
	   if {$min <= $rec && $rec < $max} { set idx [expr {$i +1} ]; break }
       }
   }

   if {$modify} {; # add record to lixt box
     set val [ccdEditToRec ccdName ccdIndex $ccdTempl(mask)]
     $wdg addItem $idx [$formatProc $val]
     set ccdDisp($rec) $val
   } else {; # clean complete record, included available flags
     set val $ccdTempl(clean)   
   }
   set ccdTable($rec) $val  
}





#
# Update record with contents of the edit section
proc ccdEditToRec { name index record } {

   global gvar 

   upvar $name ccdName
   upvar $index ccdIndex

   set ccdRecord $record
   set col [array size ccdName]
   for { set i 0 } { $i < $col } {incr i } {
       set ind $ccdIndex($i)
       if {$ind >= 0} {; # simple value
	   set val $gvar($ccdName($i))
       } else {; # combined value e.g. clock_2_L
	   set ind [expr {-1 * $ind}]
	   set piece [split $gvar($ccdName($i)) _]; # split string into pieces
	   incr i; # now get index
	   if {[llength $piece] > 0} { 
	       set piece [linsert $piece 1 $gvar($ccdName($i))]
	   }
	   set val [join $piece _]
       }
       set ccdRecord [lreplace $ccdRecord $ind $ind $val]
   }
   return $ccdRecord
}



#
# Store a generic table from a list box into the database
proc ccdSaveBoxToDb { table rows columns dbAttr cleanRec } {

   global gvar
  
   upvar $table  ccdTable
   upvar $dbAttr ccdDbAttr

   if {!($gvar($ccdDbAttr(avail)))} {; # disabled; clean table 
     ccdClearTable ccdTable $cleanRec     
   }
   for { set i 0 } { $i < $rows } {incr i } {
     if {[info exists ccdTable($i)]} {; # data entered in box at least once
       if {[info exists ccdDbAttr($i)]} {; # row can be treated in one go
         panDbWrite $ccdDbAttr($i) [lrange $ccdTable($i) 1 end]
       } else {
         for { set j 1 } { $j < $columns } {incr j } {; #j=0 record index: skip
           panDbWrite $ccdDbAttr($i,$j) [lindex $ccdTable($i) $j]
         } 
       }
      update idletasks; # the whole screen may block if not done
     } 
   } 
}






#
# Reset the list box and optionally clean table
proc ccdClearBox { wdg {table ""} {cleanRec ""} } {

   upvar $table ccdTable

   $wdg deleteItem 0 end ; # Reset list box
   ccdClearTable ccdTable $cleanRec
}




#
# Clean table
proc ccdClearTable { table cleanRec } {

   upvar $table ccdTable

   set rows [array size ccdTable]
   for { set i 0 } { $i < $rows } {incr i } {
     set ccdTable($i) $cleanRec
   }
}




#
# Read from OLDB, build internal table and load it in the listbox
proc ccdLoadBoxFromDb { table disp rows columns dbAttr wdg formatProc } {

   upvar $table  ccdTable 
   upvar $disp   ccdDisp 
   upvar $dbAttr ccdDbAttr 

   # Read element by element from OLDB and set internal table
   ccdDbToTable ccdTable $rows $columns ccdDbAttr
 
   # now extract from the table only those rows which should be displayed
   ccdTableToDisplay ccdTable ccdDisp ccdDbAttr $wdg $formatProc
}




#
# Enable/disable a generic list box
proc ccdEnableBox { table disp button rows columns dbAttr wdg formatProc } {

  global gvar

  upvar $table  ccdTable
  upvar $disp   ccdDisp
  upvar $button ccdButton
  upvar $dbAttr ccdDbAttr

  if {($gvar($ccdDbAttr(avail)))} {; #enabled
     ccdLoadBoxFromDb ccdTable ccdDisp $rows $columns ccdDbAttr $wdg $formatProc
     set butVal 1
   } else {
     ccdClearBox $wdg; # reset list box but NOT the table, for next enable
     set butVal 0
   }
   # enable/disable edit buttons
   foreach rec [array names ccdButton] { 
     set gvar($ccdButton($rec)) $butVal
   }
}




#
# Display the data stored in the internal table
proc ccdTableToDisplay { table disp dbAttr wdg formatProc } {

  upvar $table ccdTable
  upvar $disp ccdDisp
  upvar $dbAttr ccdDbAttr

  # Build display list from internal table
  ccdTableAvail ccdTable ccdDisp ccdDbAttr

  # now update list box widget
  ccdDisplayBox ccdDisp $wdg $formatProc
}





#
# ___oOo___
