
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdLoadSetup.tcl,v 2.7 2005/05/11 03:10:39 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  04/09/95  created
# alongino  24/10/95  added check if file exists before trying to load
# alongino  18/02/96  updated to new panel ccdCtrl.pan
# alongino  20/05/96  ported to new library slxtcl
# alongino  04/07/96  added DET.FRAM.SAMPLE DET.USER.FUNC and DET.USER.BUFF
# alongino  02/09/96  new interface for ccdEnableWindowing
# alongino  24/02/97  new way of defining readout mode
#                     two windows readout supported
# alongino  26/02/97  ccdSaveSetup moved from ccdCtrl.pan
# alongino  20/03/97  setup file type "Detector Setup" according to oslx MAY97
# alongino  05/05/97  updated to new slxtcl (no oslxServer)
# alongino  10/06/97  added THRMIN BACKGND IPLLX IPLLY IPURX IPURY
# alongino  13/10/97  added SIMIMG SHIFTYP SHIFT NDIT ASUIT 
# alongino  12/05/98  added XFERSYN
#

#************************************************************************
#   NAME
#   ccdLoadSetup, ccdSaveSetup - interface panel to setup files
# 
#   SYNOPSIS
#   ccdLoadSetup <file>
#   ccdSaveSetup <file>
# 
#   DESCRIPTION
#   Tcl procedures used within CCD OSS GUI panels to load in a panel the
#   contents of a setup file and to save into a setup file the contents
#   of a panel
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdLoadSetup $insroot/SYSTEM/COMMON/SETUPFILES/DET/mySetup.det
#   ccdSaveSetup $insroot/SYSTEM/COMMON/SETUPFILES/DET/mySetup.det
#
#   SEE ALSO
#
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc ccdFilterDetSetup { inFile outFile } {

    set status "OK"
    set ifd [open $inFile r]
    if {[lempty $ifd]} {
        puts "ERROR : Could not open $inFile"
        return "FAILURE"
    }
    set ofd [open $outFile w+]
    if {[lempty $ofd]} {
	close $ifd
        puts "ERROR : Could not create $outFile"
        return "FAILURE"
    }

    # read inFile until EOF
    # replace DETi by DET
    # write outFile
    while { [gets $ifd inLine] != -1 } {
        # skip empty lines
	if {[string length $inLine] == 0} continue
	# skip comment lines
	if {[string index $inLine 0] == "#"} continue
	if {[regsub {DET[0-9].} $inLine "DET." outLine] == 0} {
	    set outLine $inLine 
	}
	puts $ofd $outLine
    }

    close $ifd ; close $ofd
    if {$status == "FAILURE"} {
	file delete $outFile
    }
    return $status
}

# load in a panel the contents of a setup file
proc ccdLoadSetup { file } {  

   global gvar cv
   global ccdMaxWin

   set status "OK"
   # select file
   set file [ccdSetupFilePath $file]

   # load setup file in panel
   if {$file != ""} {
     if {[file exists $file]} {
       set tmpFile "/tmp/ccdLoadSetup_[pid]"
       set status [ccdFilterDetSetup $file $tmpFile]
       if {$status == "FAILURE"} { return $status}
       set status [slxtclReadSetup $tmpFile $gvar(oslxId)]
       file delete $tmpFile
       if {$status != "FAILURE"} {
         set oslxBuf $status
         slxtclBufToPan $oslxBuf; # update panel widgets
         ccdMstBufToBox $oslxBuf; # update multi-step exposure table
       }
     }
   } else {;          # no file to be loaded
     return $status
   }
   if {$status == "FAILURE"} {return $status}

   # update widgets not directly associated to setup variables
   # exposure setup options
   set gvar(ccdExpOpt_nrep) $gvar(setup.2.DET.EXP.NREP); # repeated exposures
   if {$gvar(ccdExpOpt_nrep) == 1} {
     set gvar(ccdExpOpt_rep) 0
   } else {set gvar(ccdExpOpt_rep) 1}
   if {$gvar(ccdExpOpt_nrep) >= 1} {
     set gvar(ccdExpOpt_expNrep) $gvar(ccdExpOpt_nrep)
     set gvar(ccdExpOpt_nrep) 2
   }

   # image setup
   set rtdFrame $gvar(setup.9.DET.DISPLAY)
   set rtd 0
   set rtdRf 0
   if {$rtdFrame >= 0} {
     set rtd 1
     if {$rtdFrame > 0} {
       set rtdRf 1
     }
   }
   set gvar(rtd) $rtd
   set gvar(imageOpt_rtdRf) $rtdRf

   # readout setup
   set rdtMode [ccdRdtIndex2Mode $gvar(setup.5.DET.READ.CLKIND)]
   set gvar(ccdRdtSpeed)  [lindex $rdtMode 0]
   set gvar(ccdRdtOutput) [lindex $rdtMode 1]
   set gvar(setup.6.DET.OUT1.GAININD)  [lindex $rdtMode 2]
   ccdSetRdtSpeed $gvar(ccdRdtSpeed); # needed to update outputs enabled
   for { set i 1} { $i <= $ccdMaxWin } {incr i} {; # window readout and ip
     set gvar(ccdWin${i}_Strx) $gvar(setup.${i}11.DET.WIN$i.STRX) 
     set gvar(ccdWin${i}_Nx)   $gvar(setup.${i}12.DET.WIN$i.NX)   
     set gvar(ccdWin${i}_Stry) $gvar(setup.${i}13.DET.WIN$i.STRY) 
     set gvar(ccdWin${i}_Ny)   $gvar(setup.${i}14.DET.WIN$i.NY)

     set gvar(ccdSetupIp${i}_ipOffLLX)   $gvar(setup.${i}31.DET.WIN$i.IPLLX)
     set gvar(ccdSetupIp${i}_ipOffLLY)   $gvar(setup.${i}32.DET.WIN$i.IPLLY)
     set gvar(ccdSetupIp${i}_ipOffURX)   $gvar(setup.${i}33.DET.WIN$i.IPURX)
     set gvar(ccdSetupIp${i}_ipOffURY)   $gvar(setup.${i}34.DET.WIN$i.IPURY)
     set gvar(ccdSetupIp${i}_ipBias)     $gvar(setup.${i}27.DET.WIN${i}.BIAS)
     set gvar(ccdSetupIp${i}_ipFF)       $gvar(setup.${i}28.DET.WIN${i}.FLATF)
     set gvar(ccdSetupIp${i}_ipMinMax)   $gvar(setup.${i}23.DET.WIN${i}.MINMAX)
     set gvar(ccdSetupIp${i}_ipCentroid) $gvar(setup.${i}20.DET.WIN${i}.CENTROID)
     set backVal $gvar(setup.${i}29.DET.WIN${i}.BACKGND)
     if {$backVal > 0} {
       set gvar(ccdSetupIp${i}_ipBackMode) 0
       set gvar(ccdSetupIp${i}_ipBackVal) $backVal
     } else {
       set gvar(ccdSetupIp${i}_ipBackMode) [expr {int($backVal)}]
     }
     set threshVal $gvar(setup.${i}30.DET.WIN${i}.THRMIN)
     if {$threshVal > 0} {
       set gvar(ccdSetupIp${i}_ipThreshMode) 0
       set gvar(ccdSetupIp${i}_ipThreshVal) $threshVal
     } else {
       set gvar(ccdSetupIp${i}_ipThreshMode) [expr {int($threshVal)}]
     }
     set gvar(ccdSetupIp${i}_ipRefX)     $gvar(setup.${i}21.DET.WIN${i}.REFX)
     set gvar(ccdSetupIp${i}_ipRefY)     $gvar(setup.${i}22.DET.WIN${i}.REFY)
     if {![string compare $gvar(setup.${i}25.DET.WIN${i}.IPFUNC) None]} {
       set gvar(ccdSetupIp${i}_ipUsrFlag) 0
     } else {;                            # user function used
       set gvar(ccdSetupIp${i}_ipUsrFlag) 1
       set gvar(ccdSetupIp${i}_ipUsrBuff) $gvar(setup.${i}26.DET.WIN${i}.IPBUFF)
       set gvar(ccdSetupIp${i}_ipUsrFunc) $gvar(setup.${i}25.DET.WIN${i}.IPFUNC)
     }
   }
   # enable windowing
   ccdEnableWindowing

   return $status
}


# save in a setup file the contents of a panel
proc ccdSaveSetup {file} {
   global cv
   global $cv.ccdSetupIp1.ckShow $cv.ccdSetupIp2.ckShow 
   global $cv.ccdWindowFrame1.ckShow $cv.ccdWindowFrame2.ckShow
   global $cv.ccdExpOpt.ckShow $cv.ccdExpMulti.ckShow $cv.imageOpt.ckShow
   global gvar insroot insrootSetupDet
   global vltroot introot did
   global ccdMaxWin

   # get complete file path
   set file [ccdSetupFilePath $file]

   # set variable values according to class container button status
   for { set i 1} { $i <= $ccdMaxWin } {incr i} {
     set gvar(setup.${i}31.DET.WIN$i.IPLLX) $gvar(ccdSetupIp${i}_ipOffLLX);
     set gvar(setup.${i}32.DET.WIN$i.IPLLY) $gvar(ccdSetupIp${i}_ipOffLLY);
     set gvar(setup.${i}33.DET.WIN$i.IPURX) $gvar(ccdSetupIp${i}_ipOffURX);
     set gvar(setup.${i}34.DET.WIN$i.IPURY) $gvar(ccdSetupIp${i}_ipOffURY);
     set gvar(setup.${i}25.DET.WIN${i}.IPFUNC) None
     set gvar(setup.${i}26.DET.WIN${i}.IPBUFF) None
     set gvar(setup.${i}21.DET.WIN${i}.REFX) $gvar(ccdSetupIp${i}_ipRefX)
     set gvar(setup.${i}22.DET.WIN${i}.REFY) $gvar(ccdSetupIp${i}_ipRefY)
     if {$gvar(ccdSetupIp${i}_ipThreshMode) == 0} {
       set threshVal $gvar(ccdSetupIp${i}_ipThreshVal)
     } else {
       set threshVal $gvar(ccdSetupIp${i}_ipThreshMode)
     }
     set gvar(setup.${i}30.DET.WIN${i}.THRMIN) $threshVal
     if {$gvar(ccdSetupIp${i}_ipBackMode) == 0} {
       set backVal $gvar(ccdSetupIp${i}_ipBackVal)
     } else {
       set backVal $gvar(ccdSetupIp${i}_ipBackMode)
     }
     set gvar(setup.${i}29.DET.WIN${i}.BACKGND) $backVal
     set status [set $cv.ccdSetupIp${i}.ckShow];
     if {$status == 0} {
       set gvar(setup.${i}23.DET.WIN${i}.MINMAX) F
       set gvar(setup.${i}20.DET.WIN${i}.CENTROID) none
       set gvar(setup.${i}27.DET.WIN${i}.BIAS) F
       set gvar(setup.${i}28.DET.WIN${i}.FLATF) F
     } else {
       set gvar(setup.${i}23.DET.WIN${i}.MINMAX) $gvar(ccdSetupIp${i}_ipMinMax)
       set gvar(setup.${i}20.DET.WIN${i}.CENTROID) $gvar(ccdSetupIp${i}_ipCentroid)
       set gvar(setup.${i}27.DET.WIN${i}.BIAS)  $gvar(ccdSetupIp${i}_ipBias)
       set gvar(setup.${i}28.DET.WIN${i}.FLATF) $gvar(ccdSetupIp${i}_ipFF)
       
       if {$gvar(ccdSetupIp${i}_ipUsrFlag)} {; # user function used
         set gvar(setup.${i}25.DET.WIN${i}.IPFUNC) $gvar(ccdSetupIp${i}_ipUsrFunc)
         set gvar(setup.${i}26.DET.WIN${i}.IPBUFF) $gvar(ccdSetupIp${i}_ipUsrBuff)
       }
     }
     set status [set $cv.ccdWindowFrame$i.ckShow]
     if {$status == 0} {
       set gvar(setup.${i}10.DET.WIN$i.ST) F
     } else {
       set gvar(setup.${i}10.DET.WIN$i.ST) T
     }
     set gvar(setup.${i}11.DET.WIN$i.STRX) $gvar(ccdWin${i}_Strx)
     set gvar(setup.${i}12.DET.WIN$i.NX)   $gvar(ccdWin${i}_Nx)
     set gvar(setup.${i}13.DET.WIN$i.STRY) $gvar(ccdWin${i}_Stry)
     set gvar(setup.${i}14.DET.WIN$i.NY)   $gvar(ccdWin${i}_Ny)
   }

   # exposure setup options
   set status [set $cv.ccdExpOpt.ckShow];
   if {$status == 0} {
     set gvar(ccdExpOpt_setup.0.DET.EXP.TYPE) Normal;
     set gvar(ccdExpOpt_setup.61.DET.EXP.WIPETIM) 0;
     set gvar(ccdExpOpt_rep) 0
     set gvar(ccdExpOpt_setup.40.DET.EXP.TIMEREP) 0;
   }
   if {$gvar(ccdExpOpt_rep) == 0} {set nrep 1
   } elseif {$gvar(ccdExpOpt_nrep) == 2} {set nrep $gvar(ccdExpOpt_expNrep)
   } else {set nrep $gvar(ccdExpOpt_nrep)}  
   set gvar(setup.2.DET.EXP.NREP) $nrep

   # multi-step exposure setup options
   set status [set $cv.ccdExpMulti.ckShow];
   if {$status == 0} {
     set gvar(ccdExpMulti_setup.1141.DET.WIN1.NDIT) 1;
     set gvar(ccdExpMulti_setup.1142.DET.WIN1.ASUIT1) T;
     set gvar(ccdExpMulti_setup.1143.DET.READ.SHIFTYP) list;
     set gvar(ccdExpMulti_setup.1144.DET.READ.SHIFT1) 0;
   }

   # image setup options
   set status [set $cv.imageOpt.ckShow];
   if {$status == 0} {
     set gvar(imageOpt_setup.30.DET.FRAM.TYPE) Normal;
     set gvar(imageOpt_setup.24.DET.FRAM.SAMPLE) 1;
     set gvar(imageOpt_setup.60.DET.FRAM.XFERSYN) T;
     set gvar(imageOpt_rtdRf) 0
   }
   if {$gvar(rtd)} {
     if {$gvar(imageOpt_rtdRf)} {
       set gvar(setup.9.DET.DISPLAY) 4;  # rapid frame
     } else {
       set gvar(setup.9.DET.DISPLAY) 0;  # main frame
     }
   } else {
     set gvar(setup.9.DET.DISPLAY) -1;   # no display
   }

   # save setup
   if {$file != ""} {
      set type "Detector Setup"
      set oslxBuf [slxtclPanToBuf]
      set oslxBuf [ccdMstAddToBuf $oslxBuf]
      set status [slxtclWriteSetup $oslxBuf $file $type $gvar(oslxId)]
   }

   #
   # Return result
   #
   return $status
}



proc ccdSetupDefaults {} {
   global cv gvar
   global ccdMaxWin

   panSetDefaults $cv
   for { set i 1} { $i <= $ccdMaxWin } {incr i} {
     # no windowed readout
     set gvar(setup.${i}10.DET.WIN$i.ST) F

     # no image processing
     set gvar(ccdSetupIp${i}_ipMinMax) F
     set gvar(ccdSetupIp${i}_ipBias) F
     set gvar(ccdSetupIp${i}_ipCentroid) none
     set gvar(ccdSetupIp${i}_ipFF) F
     set gvar(ccdSetupIp${i}_ipUsrFlag) 0
   }
   ccdSetRdtSpeed $gvar(ccdRdtSpeed); # needed to set default gain
   ccdEnableWindowing
}
#
# ___oOo___


