
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdTelTemSim.tcl,v 2.7 2005/05/11 03:10:43 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  07/03/97  created
#

#************************************************************************
#   NAME
#   ccdTelTemSim - simulate a telemetry and temperature monitoring loop
# 
#   SYNOPSIS
#   ccdTelTemSim
# 
#   DESCRIPTION
#   Tcl procedure used within CCD GUI panel ccdTelemetry to simulate
#   the monitoring of all available and enabled telemetry and
#   temperature values
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc ccdTelTemSim { cat step } {
    
   global env gvar simul
   global ccdTelStopFlag ccdTemStopFlag
   global ccdTelStateDb ccdTemStateDb
   global ccdTelCurVal ccdTemCurVal
   global ccdTelRefVal ccdTemRefVal

  if {![string compare $cat "Tel"]} {; # telemetry
    set attrStatus $ccdTelStateDb
    set attrCurrent "telemetry.current"
    set attrSensor "telemetry.sensor"
    set curSize 100
    set stopFlag $ccdTelStopFlag
  } else {;                           # temperature
    set attrStatus $ccdTemStateDb
    set attrCurrent "temperature.current"
    set attrSensor "temperature.sensor"
    set curSize 10
    set stopFlag $ccdTemStopFlag
  }
  if {$stopFlag} {; # loop stopped
    panDbWrite $attrStatus $gvar(ccdSTATE_ONLINE_IDLE)
    return
  }
  set name ccd${cat}CurVal
  set ref ccd${cat}RefVal
  if {$step == 0} {;   # extract setup information
    panDbWrite $attrStatus $gvar(ccdSTATE_ONLINE_ACTIVE); # loop active
    # Initialize list with reference values
    set $ref {} 
    for { set i 0 } { $i < $curSize } {incr i } {
	set val [dbRead ${attrSensor}($i,reference)]
      set $ref [lappend $ref $val]
    }
    set $name [set $ref]
  }
  # simulate updating of values
  # for the moment do it only for record 0. 
  # Eventually all enabled values should be updated
  set curList [set $name]
  set refList [set $ref]
  for { set i 0 } { $i < $curSize } {incr i } {
      set refVal [lindex $refList $i]
      # sinus with amplitude 10% of reference value and period 10*(i+1) steps
      set ampl [expr {$refVal/10}]
      set period [expr {10*($i+1)}]
      set phase [expr {2*3.14*$step/$period}]
      set newVal [expr {$refVal+$ampl*sin($phase)}]; # temporary
      set curList [lreplace $curList $i $i $newVal]
  }
  panDbWrite $attrCurrent $curList
  set $name $curList

  update idletasks; # needed to be able to handle pending messages
  # call again this procedure for next update
  incr step
  after 2000 ccdTelTemSim $cat $step
}


#
# ___oOo___


