#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdTelemetry.tcl,v 2.7 2005/05/11 03:10:43 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/03/97  created
#

#************************************************************************
#   NAME 
#   ccdTelTemInit, ccdFormatTelTemStatus, ccdTelTemAddCurr,
#   ccdTelTemDisplay, ccdTemperatureInitialise, ccdTelemetryInitialise,
#   ccdTelPlotStart, ccdTelPlotStop, ccdTemPlotStart, ccdTemPlotStop
#   ccdFormatTelStatus, ccdFormatTemStatus
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the ccdTelemetry panel to handle the 
#   telemetry parameters
#
#------------------------------------------------------------------------
#

#
# Initialise variables for telemetry and temperature list boxes
proc ccdTelTemInit { } {
    global gvar telIndex camera absCcdPath

    set telIndex(0) 6;  # id
    set telIndex(1) 7;  # units
    set telIndex(2) 10; # reference
    set telIndex(3) 9;  # current, replacing high level, unused in this context

    catch {seq_dbGetAlias "\<alias\>$camera:telemetry:plot"} plotAlias
    set gvar(plotTelTable) [string range $plotAlias 7 end]
    catch {seq_dbGetAlias "\<alias\>$camera:temperature:plot"} plotAlias
    set gvar(plotTemTable) [string range $plotAlias 7 end]
    catch {seq_dbGetCwp} curCwp
    catch {seq_dbSetCwp "\<alias\>$camera"}
    catch {seq_dbGetCwp} absCcdPath
    catch {seq_dbSetCwp $curCwp}
}

# formats on row in the telemetry or temperature list box
proc ccdFormatTelTemStatus { index text } {
    upvar $index ccdIndex
    
    set idIdx   $ccdIndex(0)
    set unitIdx $ccdIndex(1)
    set refIdx  $ccdIndex(2)
    set curIdx  $ccdIndex(3)
    
    return [format "%-11s %1s %5.1f %5.1f" \
	    [lindex $text $idIdx]  [lindex $text $unitIdx] \
	    [lindex $text $refIdx] [lindex $text $curIdx]]
}

# add current value to table of available elements
proc ccdTelTemAddCurr { avail ccdCurr } {
    global telIndex
    
    upvar $avail ccdAvail
    
    set elList [array names ccdAvail]
    if {[llength $elList] > 0} {
	# high level (unused in this context) replaced with current    
	set pos $telIndex(3)
	foreach el $elList {
	    set idx [lindex $ccdAvail($el) 0]
	    set val [lindex $ccdCurr $idx]
	    set ccdAvail($el) [lreplace $ccdAvail($el) $pos $pos $val]
	}
    }
}

#
# read current telemetry or temperature values and display in list box
proc ccdTelTemDisplay { dbCurr avail wdg formatProc } {
    upvar $avail ccdAvail
    
    # temporarily read here current value. Should be done on trace from 
    # telemetry.current(0)
    set current [dbRead $dbCurr]
    
    # merge current values in table
    ccdTelTemAddCurr ccdAvail $current
    
    # display list of telemetry status
    ccdDisplayBox ccdAvail $wdg $formatProc
}

#
# Initialise telemetry variables
proc ccdTelemetryInitialise { } {
    global gvar
    global telAvail
    global telDbCurr telWdg telFormatProc

    # set default polling rate
    set gvar(polltel) 60

    # read from db and build table
    ccdTelInitialise
    ccdTelTemInit
    
    # read current values and display in list box
    ccdTelTemDisplay $telDbCurr telAvail $telWdg $telFormatProc
    
    # update buttons
    ccdTelUpdateButtons
}

#
# Initialise temperature variables
proc ccdTemperatureInitialise { } {
    global gvar
    global temAvail
    global temDbCurr temWdg temFormatProc
    
    # set default polling rate
    set gvar(polltmp) 60
    
    # read from db and build table
    ccdTemInitialise
    ccdTelTemInit

    # read current values and display in list box
    ccdTelTemDisplay $temDbCurr temAvail $temWdg $temFormatProc

    # update buttons
    ccdTemUpdateButtons
}

# formats one row in the telemetry status list box
proc ccdFormatTelStatus { text } {
    global telIndex
    ccdFormatTelTemStatus telIndex $text
}

# formats one row in the telemetry status list box
proc ccdFormatTemStatus { text } {
    global telIndex
    ccdFormatTelTemStatus telIndex $text
}

# Enable/Disable telemetry buttons
proc ccdTelUpdateButtons { {opState -1} } {
    global cv gvar
    global telAvail telDbAttr
    
    set gvar(ccdTelIdle) 0
    set gvar(ccdTelActive) 0
    if {$opState == -1} {; #invalid value. Read state from DB
	set opState [dbRead "telemetry.opState"]
    }
    if {$opState == $gvar(ccdSTATE_ONLINE_IDLE)} {
	set gvar(ccdTelIdle) 1
    } elseif {$opState == $gvar(ccdSTATE_ONLINE_ACTIVE)} {
	set gvar(ccdTelActive) 1
    }
    
    # enter options for plotting
    set ccdList [ccdTelTemGetList telAvail telDbAttr]
    set ccdList [linsert $ccdList 0 ""]; # empty entry
    for { set i 1} { $i <= 4 } {incr i} {
	$cv.ccdTelPlot$i configure -itemList $ccdList
    }
}

# Enable/Disable temperature buttons
proc ccdTemUpdateButtons { {opState -1} } {
    global cv gvar
    global temAvail temDbAttr
    
    set gvar(ccdTemIdle) 0
    set gvar(ccdTemActive) 0
    if {$opState == -1} {; #invalid value. Read state from DB
	set opState [dbRead "temperature.opState"]
    }
    if {$opState == $gvar(ccdSTATE_ONLINE_IDLE)} {
	set gvar(ccdTemIdle) 1
    } elseif {$opState == $gvar(ccdSTATE_ONLINE_ACTIVE)} {
	set gvar(ccdTemActive) 1
    }
    
    # enter options for plotting
    set ccdList [ccdTelTemGetList temAvail temDbAttr]
    set ccdList [linsert $ccdList 0 ""]; # empty entry
    for { set i 1} { $i <= 2 } {incr i} {
	$cv.ccdTemPlot$i configure -itemList $ccdList
    }
}

# set or reset variables to be traced within ccdTelemetry panel
proc ccdTelemetryTrace {op} {
    global gvar
    
    # Trace change of state to enable/disable buttons
    trace $op gvar(telemetry.opState) w ccdExecTelTrace
    trace $op gvar(temperature.opState) w ccdExecTelTrace
    # Trace operations needed because of renaming of variables in classes
    trace $op gvar(telemetry.current(0)) w ccdExecTelTrace
    trace $op gvar(temperature.current(0)) w ccdExecTelTrace
}

# trace procedure for ccdCtrl panel
proc ccdExecTelTrace {name element op} {
    global telDbCurr telAvail telWdg telFormatProc
    global temDbCurr temAvail temWdg temFormatProc
    global gvar
    
    if {![string compare $element "telemetry.opState"]} {
	ccdTelUpdateButtons $gvar($element)
    } elseif {![string compare $element "temperature.opState"]} {
	ccdTemUpdateButtons $gvar($element)
    } elseif {![string compare $element "telemetry.current(0)"]} {
	ccdTelTemDisplay $telDbCurr telAvail $telWdg $telFormatProc
    } elseif {![string compare $element "temperature.current(0)"]} {
	ccdTelTemDisplay $temDbCurr temAvail $temWdg $temFormatProc
    } 
}

# start plotting telemetry or temperature values
proc ccdTelTemPlotStart { table attr title } {
    global wsenv

    if {[string length $attr] > 0} {
	eval exec sampCtrl_start.sh 0 "none" $table $wsenv WS 1 5 $attr &;
    } else {
	puts "no element selected for plotting"
    }
}

# start plotting telemetry values
proc ccdTelPlotStart {} {
    global gvar camera telAvail absCcdPath
    
    set attr ""
    for { set i 1} { $i <= 4 } {incr i} {
	set selected $gvar(ccdTelPlot$i)
	set rec [ccdTelTemGetListRec $selected telAvail]
	if {$rec >= 0} {
	    set index [lindex $telAvail($rec)  0]
	    set dbAttr "$absCcdPath:telemetry.current($index)"
	    set attr "$attr \"$dbAttr\" -l \"$selected\""
	}
    }
    
    ccdTelTemPlotStart $gvar(plotTelTable) $attr "$camera telemetry"
}

# start plotting temperature values
proc ccdTemPlotStart {} {
    global gvar wsenv camera temAvail absCcdPath
    
    set attr ""
    for { set i 1} { $i <= 2 } {incr i} {
	set selected $gvar(ccdTemPlot$i)
	set rec [ccdTelTemGetListRec $selected temAvail]
	if {$rec >= 0} {
	    set index [lindex $temAvail($rec)  0]
	    set dbAttr "$absCcdPath:temperature.current($index)"
	    set attr "$attr \"$dbAttr\" -l \"$selected\""
	}
    } 
    ccdTelTemPlotStart $gvar(plotTemTable) $attr "$camera temperature"
}

# stop plotting telemetry or temperature values
proc ccdTelTemPlotStop { table } {
    global wsenv
    eval exec sampCtrl_stop.sh "$table" $wsenv WS &;
}

# stop plotting telemetry values
proc ccdTelPlotStop {} {
    global gvar
    ccdTelTemPlotStop $gvar(plotTelTable)
}

# stop plotting temperature values
proc ccdTemPlotStop {} {
    global gvar
    ccdTelTemPlotStop $gvar(plotTemTable) 
}

#
# Get list of parameters enabled
proc ccdTelTemGetList { avail dbAttr } {
    upvar $avail ccdAvail
    upvar $dbAttr ccdDbAttr
    
    set ccdList {}
    set available [dbRead $ccdDbAttr(avail)]
    if {$available} {; # telemetry system is available
	set elList [array names ccdAvail]
	set elList [lsort -integer $elList]
	if {[llength $elList]} {
	    foreach rec $elList {
		set newEntry [lindex $ccdAvail($rec) 6]
		set ccdList [lappend ccdList $newEntry]
	    }
	}
    }
    return $ccdList
}

#
# Get record of an element in the list of parameters enabled
proc ccdTelTemGetListRec { entry avail } {
    upvar $avail ccdAvail
    
    set found 0
    set elList [array names ccdAvail]
    if {[llength $elList]} {
	# look for selected option in the list of available entries
	foreach rec $elList {
	    set val [lindex $ccdAvail($rec) 6]
	    if {![string compare $entry $val]} { 
		set found 1
		break 
	    }
	}
    }
    if {$found == 0} {set rec -1}
    return $rec
}

#
# ___oOo___
