
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdUsrGlobalData.tcl,v 2.7 2005/05/11 03:10:42 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/05/95  created
# alongino  25/07/95  added env. var. INS_ROOT, INTROOT, RTAPENV, CCDLENV
#                     added simul
# alongino  30/07/95  added insrootSetupDet
# alongino  22/02/96  added CCDDID env. variable
# alongino  06/03/96  added temporarily cwpTmp for patch to panel 2.57 bug
# alongino  20/05/96  added CCDOLSX_SERVER
# alongino  05/06/96  removed cwp "undefined" if no camera name defined
# alongino  24/02/97  added standard speeds, gains and maximum wind. definition
# alongino  05/05/97  oslxServer not used any more. Removed
# alongino  20/05/97  added various definitions
# alongino  13/08/97  added gvar(ccdSTATE_XXX)
#

#************************************************************************
#   NAME
#   ccdUsrGlobalData - assign a default value to common global variables
# 
#   SYNOPSIS
#   ccdUsrGlobalData
# 
#   DESCRIPTION
#   Tcl procedure, used within CCD GUI panels, to assign a default value to
#   common global variables, in particular CWP.
#
#   FILES
#
#   ENVIRONMENT
#   CCDNAME     name of camera used
#   RTAPENV     name of WS environment where CCD sw runs
#   CCDLENV     name of LCU environment where CCD sw runs
#   INS_ROOT    root directory for instrument data
#   INTROOT     integration root directory
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   ccdUsrGlobalData
#
#   SEE ALSO
#   panUsrGlobalData
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ccdUsrGlobalData { } {        

 # Declare any variable as global

   global cwp gvar env camera
   global insroot simul wsenv lcuenv insrootSetupDet
   global vltroot introot did
   global cwpTmp
   global ccdRdtStdSpeed
   global ccdTecGain ccdSciGain
   global ccdMaxWin

 # Variable Initialization
   set camera "undefined"
   if {[info exists env(CCDNAME)] == 1} {
       set camera $env(CCDNAME)
       set cwp "<alias>$camera" ;      # cwp normally passed as runtime par.
   }
   set gvar(camera) $camera

   if {[info exists cwpTmp] == 1} {;   # temporary for bug in panel 2.57
       set cwp $cwpTmp ;
   }

   set wsenv "undefined"
   if {[info exists env(RTAPENV)] == 1} {
       set wsenv "$env(RTAPENV)" ;           # WS CCD environment
   }
   set lcuenv "undefined"
   if {[info exists env(CCDLENV)] == 1} {
       set lcuenv "$env(CCDLENV)" ;          # LCU CCD environment
   }

   set insroot "undefined"
   if {[info exists env(INS_ROOT)] == 1} {
       set insroot "$env(INS_ROOT)" ;   # Instrument root directory
   }

   set vltroot "$env(VLTROOT)"
   set introot "undefined"
   if {[info exists env(INTROOT)] == 1} {
       set introot "$env(INTROOT)" ;    # Integration directory
   } elseif {[info exists env(VLTROOT)] == 1} {
       set introot $vltroot ;
   }

   set did CCDDCS;                      # default
   if {[info exists env(CCDDID)] == 1} {
       set did "$env(CCDDID)" ;         # Data Dictionary name
   }

   set simul 0;                         # default: no actions simulation
   set insrootSetupDet $insroot/SYSTEM/COMMON/SETUPFILES/DET

   set ccdRdtStdSpeed {vfast fast normal slow vslow}; # standard speeds
   set ccdTecGain {1 2 4 8};         # possible gains for TCCDS
   set ccdSciGain {0 1 2 3 4 5 6 7}; # possible gains for SCCDS

   set ccdMaxWin 2;  # maximum 2 windows supported
   set gvar(ccdEXP_NEXT) -1
   set gvar(ccdEXP_LAST) 0

   # define operational modes here
   set gvar(ccdMODE_INVALID_LOW)    0
   set gvar(ccdMODE_NOT_AVAIL)      1
   set gvar(ccdMODE_NORMAL)         2
   set gvar(ccdMODE_SIM_WS)         3
   set gvar(ccdMODE_SIM_LCU)        4
   set gvar(ccdMODE_SIM_ACE)        5
   set gvar(ccdMODE_INVALID_HIGH)   6

   # define operational states here
   set gvar(ccdSTATE_INVALID_LOW)   0
   set gvar(ccdSTATE_OFF)           1
   set gvar(ccdSTATE_LOADED)        2
   set gvar(ccdSTATE_STANDBY)       3
   set gvar(ccdSTATE_ONLINE)        4
   set gvar(ccdSTATE_ONLINE_IDLE)   $gvar(ccdSTATE_ONLINE)
   set gvar(ccdSTATE_ONLINE_ACTIVE) 5
   set gvar(ccdSTATE_ONLINE_WIPING) $gvar(ccdSTATE_ONLINE_ACTIVE)
   set gvar(ccdSTATE_INVALID_HIGH)  5
#puts "cwp     $cwp"
#puts "wsenv   $wsenv"
#puts "lcuenv  $lcuenv"
#puts "insroot $insroot"
#puts "did     $did"
}


#
# ___oOo___


