#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdVerifyProcs.tcl,v 2.7 2005/05/11 03:10:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   21/10/97  created
# alongino  14/05/98  a few adaptations to Paranal configuration
#
#************************************************************************
#   NAME
#       ccdVerify - Verify CCD environment
#
#   SYNOPSIS
#       ccdVerify
#
#   DESCRIPTION
#       This script checks step by step the correctness of the
#       environment for the CCD Control Software to operate.
#
#	1 - Check Environment Variables
#	2 - Check INS_ROOT
#	3 - Check WS
#	4 - Check LCU
#	5 - Scan system
#

#
# procedure to puts -nonewline <args> to stdout or logfile
#
proc putsnonl args { 
    global gvar 
    if {[llength $args] > 1} {
	set str [lindex [join $args " "] 0]
    } else {
	set str [lindex $args 0]
    }
    if {$gvar(ccdVerLogFd) != 0} {
	puts -nonewline $gvar(ccdVerLogFd) $str
    } else {
	puts -nonewline $str
	flush stdout
    }
}

#
# procedure to puts <args> to stdout or logfile
#
proc putsnl args { 
    global gvar 
    if {[llength $args] > 1} {
	set str [lindex [join $args " "] 0]
    } else {
	set str [lindex $args 0]
    }
    if {$gvar(ccdVerLogFd) != 0} {
	puts $gvar(ccdVerLogFd) $str
	flush $gvar(ccdVerLogFd)
    } else {
	puts $str
    }
}

#
# procedures to puts the output messages ERR/WRN/HLP/DGN
#
proc putsERR args {
    putsnonl "ERROR   : "
    putsnl [lindex $args 0]
}
proc putsHLP args {  
    putsnonl "HELP   -> "
    putsnl [lindex $args 0]
}
proc putsWRN args {  
    putsnonl "WARNING : "
    putsnl [lindex $args 0]
}
proc putsDGN args {  
    putsnonl "          "
    putsnl [lindex $args 0]
}

#
# procedure to update a GUI variable and update the panel
#
proc ccdVerifyUpdate { var val } {
    global gvar

    if {[string length $var] != 0 } {
	set gvar($var) $val
	update idletasks
    }
}
    
#
# procedure to preset the global variables
#
proc ccdVerifyPreset { } {
    global gvar ccdOpState ccdOpMode ccdVerifyStatus
    global UKN NOP ERR RUN WRN OK

    set UKN -3
    set NOP -2
    set ERR -1
    set RUN  0
    set WRN  1
    set OK   2

    set ccdVerifyStatus     $RUN

    set gvar(ccdVerMessage) ""

    set ccdOpState(LOADED)  2
    set ccdOpState(STANDBY) 3
    set ccdOpState(ONLINE)  4
    set ccdOpState(ACTIVE)  5

    set ccdOpMode(LCU_SIM)  3
    set ccdOpMode(ACE_SIM)  4
    set ccdOpMode(HW_SIM)   5
    set ccdOpMode(NORMAL)   2

    if {$gvar(ccdVerLogFd) == 0} {
	set gvar(ccdVerDoWsEnv)   1
	set gvar(ccdVerDoCcdEnv)  1
	set gvar(ccdVerDoRtapEnv) 1
	set gvar(ccdVerDoLcuSim)  1
	set gvar(ccdVerDoLcuEnv)  1
	set gvar(ccdVerDoScanSys) 1
	set gvar(ccdVerDoAceSim)  1
	set gvar(ccdVerDoAceEnv)  1
	set gvar(ccdVerDoHwSim)   1
	set gvar(ccdVerDoNoSim)   1
    }

    update idletasks
}

#
# procedure to issue the HLP messages
#
proc ccdVerifyHelp { help { msg "none" } } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    if {[string length $help] == 0} return

    if { $help == "UNIXENV" } {
	if { $gvar(ccdVerEnvVar) != $OK } {
	    putsDGN "Some environment variables have not been defined"
	    putsHLP "See CCD User Manual"
	} else {
	    putsHLP "Report to the system manager"
	}
    } elseif { $help == "VLTROOT" } {
	putsDGN "VLT file system is not correct"
	putsHLP "Report to the VLT manager"
    } elseif { $help == "INSTALL" } {
	putsDGN "CCD intance $ccdname has not been correctly installed"
	putsHLP "Invoke script ccdInstall.sh"
    } elseif { $help == "RTAPENV" } {
	if { $gvar(ccdVerRtapEnv) != $OK } {
	    putsDGN "RTAP environment $rtapenv is not running correctly"
	    putsHLP "Invoke vccEnvInit/Start -e $rtapenv -h $host -t RTAP"
	} elseif { $gvar(ccdVerRtapCcs) != $OK } {
	    ;
	} elseif { $gvar(ccdVerRtapDbf) != $OK } {
	    putsDGN "CCD branch has probably not been correctly generated"
	    putsHLP "Check $envdir/$rtapenv/dbl/DATABASE.db"
	    putsHLP "Regenerate RTAP DB and restart RTAP environment"
	    putsHLP "Invoke vccEnvStop, vccEnvInit & vccEnvStart"
	} elseif { $gvar(ccdVerRtapDba) != $OK } {
	    putsHLP "Check CCDNAME=$ccdname with CCD DB alias"
	    putsHLP "Check also with ccseiDb"
	} elseif { $gvar(ccdVerLogMaskWS) == "NOT SET" } {
	    putsHLP "Invoke ccdDcsSetLogMask.sh"
	} elseif { $gvar(ccdVerCcdCdt) != $OK} {
	    putsHLP "Check CCD SW installation"
	} 
    } elseif { $help == "RTDSRV" } {
	putsDGN "You may need the Real-Time Server for display"
	putsHLP "Ask your system manager to start rtdServer"
	putsDGN "   or start it from your Shell: \$rtdServer &"
    } elseif { $help == "CCDLENV" } {
	if { $gvar(ccdVerLcuEnv) != $OK } {
	    putsHLP "Check network connection to LCU"
	    putsDGN "Check environment generation procedure"
	    putsHLP "Invoke vccEnvCreate to create LCU environment"
	} elseif { $gvar(ccdVerLcuDbf) != $OK } {
	    putsDGN "CCD branch has probably not been correctly generated"
	    putsHLP "Check $envdir/$ccdlenv/dbl/DATABASE.db"
	    putsHLP "Regenerate LCU DB and reboot LCU"
	} elseif { $gvar(ccdVerLcuBs) != $OK } {
	    putsDGN "Check environment generation procedure"
	    putsHLP "Invoke vccConfigLcu for bootScript check"
	} elseif { $gvar(ccdVerLcuBoot) != $OK } {
	    putsHLP "Reboot via vccConfigLcu for detailed diagnostic"
	    putsDGN "Check LCU Hardware / Power Supply / Cables"
	} elseif { $gvar(ccdVerUtcTime) != $OK } {
	    putsDGN "Check logFile for details"
	    putsHLP "Check WS time, LCU bootScript file"
	} elseif { $gvar(ccdVerLcuDba) != $OK } {
	    putsHLP "Check CCDNAME=$ccdname with CCD DB alias"
	    putsHLP "Check also with ccseiDb"
	} elseif { $gvar(ccdVerLcuNfs) != $OK } {
	    putsHLP "Report to your system manager"
	} elseif { $gvar(ccdVerLogMaskLCU) == "NOT SET" } {
	    putsHLP "Invoke ccdDcsSetLogMask.sh"
	}
    } elseif { $help == "CCSSCAN" } {
	putsHLP "Check \$VLTDATA/ENVIRONMENTS/$rtapenv/RtapEnvTable.normal"
    } elseif { $help == "LCCSCAN" } {
	putsHLP "Check \$VLTDATA/ENVIRONMENTS/$ccdlenv/lcubootLogFile"
    } elseif { $help == "DCSSCAN" } {
	putsHLP "Invoke ccdDcsScan.sh"
    } elseif { $help == "OSSCAN" } {
	putsHLP "Invoke ccdosScan.sh"
    } elseif { $help == "CCDACE" } {
	if { $gvar(ccdVerDriver) != $OK } {
	    putsDGN "The B016 or ARC board has no device on the system" 
	    putsHLP "Check LCU Hardware configuration"
	    putsHLP "B016 or ARC board is missing or not correctly plugged in"
	} elseif { $gvar(ccdVerAceCom) != $OK } {
	    putsHLP "Check LCU-ACE Hardware connections"
	    putsHLP "ACE Powered on, Fiber links, LAB board"
	}
    } elseif { $help == "CCDHEAD" } {
	putsHLP "Check Hardware connections to CCD head"
    }
}

#
# procedure to issue the ERR messages
#
proc ccdVerifyError { var msg ret hlp } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    ccdVerifyUpdate $var $ERR
    ccdVerifyUpdate ccdVerMessage $msg
    putsERR $msg
    if {[string length $ret] != 0} { putsDGN "-> $ret" }
    ccdVerifyHelp $hlp
}

#
# procedure to issue the WRN messages
#
proc ccdVerifyWarning { var msg ret hlp } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    ccdVerifyUpdate $var $WRN
    ccdVerifyUpdate ccdVerMessage $msg
    putsWRN $msg
    if {[string length $ret] != 0} { putsDGN "-> $ret" }
    ccdVerifyHelp $hlp
}

#
# procedure to Read the database attribute <attr> into the value <val>
#
proc ccdDbRead { attr val } {
    global NOP ERR RUN WRN OK

    upvar $val v

    if {[catch {set v [seq_dbReadSymbolic $attr]} ret] != 0 } {
	ccdVerifyError "" "Database read attribute $attr" $ret ""
	return $ERR
    } 
    return $OK
}

#
# procedure to Write the value <val> into the database attribute <attr>
#
proc ccdDbWrite { attr val } {
    global NOP ERR RUN WRN OK

    if {[catch {seq_dbWriteSymbolic $attr $val} ret] != 0 } { 
	ccdVerifyError "" "Database write attribute $attr = $val" $ret ""
	return $ERR
    } 
    return $OK
}

#
# procedure to Send a command <cmd> with parameters <par> to
# the process <procName> on environment <envName>
# The command Id is returned in <cmdId>
#
proc ccdMsgSend { envName procName cmd cmdId {par ""}} {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    upvar $cmdId c

    putsnonl "Send command $cmd to $envName:$procName ... "
    if {$cmd == "PING"} {
	set x [catch {set c [seq_msgSendCommand -nocheck $envName $procName PING]} ret]
    } elseif { $par == "" } {
	set x [catch {set c [seq_msgSendCommand $envName $procName $cmd]} ret]
    } else {
	set x [catch {set c [seq_msgSendCommand $envName $procName $cmd $par]} ret]
    }

    if {$x != 0} {
	putsnl "Failed"
	ccdVerifyError "" "Send command $cmd to $envName:$procName" $ret ""
	return $ERR
    }
    putsnl "done"
    return $OK
}

#
# procedure to receive the replies <reply> associated to the command Id <cmdId>
#
proc ccdMsgRecv { cmdId reply } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    upvar $reply rep
    putsnonl "Receive reply ... "
    if {[catch {set rep [seq_msgRecvReply -all -30000 $cmdId en lr]} ret] != 0} {
	putsnl "Failed"
	ccdVerifyError "" "Error receiving reply from cmdId $cmdId" $ret ""
	return $ERR
    }
    if {$en != 0} {
	putsnl "Failed"
	ccdVerifyError "" "Received ERROR reply - ErrorNumber=$en" $rep ""
	return $ERR
    }
    putsnl "done"
    return $OK
}

#
# procedure to Send/Recv a command <cmd> with parameters <par> to
# the process <procName> on environment <envName>
# The replies are returned in <reply>
#
proc ccdSendCommandAndRecvReply { envName procName cmd reply {par ""}} {
    global NOP ERR RUN WRN OK
    
    upvar $reply rep

    if {[ccdMsgSend $envName $procName $cmd cmdId $par] != $OK} { return $ERR }
    if {[ccdMsgRecv $cmdId rep] != $OK} { return $ERR }
    if { ($cmd == "PING") || ($cmd == "CHKDRV") || ($cmd == "CHKACE") } {
	if {$rep != "OK"} { return $ERR }
    }
    return $OK
}

#
# procedure to check the database accesses on the environment <environ>
# - RW access to CCS attributes
# - RO access to CCD attributes
#
proc ccdCheckDatabaseAccess { environ } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    set attr "@$environ:PARAMS:SCALARS.scalar_int32"
    if {[ccdDbRead $attr dbScalarInt32] != $OK} { 
	ccdVerifyError "" \
		"CCS Database branch may not be loaded" "" ""
	return $ERR 
    }
    putsnl "Read  Attribute $attr = $dbScalarInt32"

    set val32 123456789
    putsnl "Write Attribute $attr = $val32"
    if {[ccdDbWrite $attr $val32] != $OK} { return $ERR }
    if {[ccdDbRead $attr dbVal32] != $OK} { return $ERR }
    putsnl "Read  Attribute $attr = $dbVal32"
    if { $dbVal32 != $val32 } {
	ccdVerifyError "" \
		"CCS Database Read/Write error" "Read=$dbVal32 - Expected=$val32" ""
	return $ERR
    }
    if {[ccdDbWrite $attr $dbScalarInt32] != $OK} { return $ERR }

    putsnl ""
    set attr "@$environ<alias>$ccdname.opMode"
    if {[ccdDbRead $attr dbOpMode] != $OK} {
	ccdVerifyError "" "CCD Database branch may not be loaded" "" ""
	return $ERR 
    }
    putsnl "Read  Attribute $attr = $dbOpMode"

    putsnl ""
    set attr1 "@$environ<alias>$ccdname:telemetry.message"
    set attr2 "@$environ<alias>$ccdname:temperature.message"
    set attrList [list $attr1 $attr2]
    foreach attr $attrList {
	if {[ccdDbRead $attr message] != $OK} { 
	    ccdVerifyError "" \
		    "CCD Database branch may not be complete" "" ""
	    return $ERR 
	}
	putsnl "Read  Attribute $attr :"
	putsnl "   ->$message"
    }
    return $OK
}

#
# procedure to check if the CCD processes are running on the WS
#
proc ccdCheckCcdProcessesWS { } {
    global gvar
    global rtapenv ccdname
    global NOP ERR RUN WRN OK

    putsnl ""
    putsnl "Check CCD processes on WS"
    set ccdProcList [list ccdconCI ccditWs]
    foreach ccdProc $ccdProcList {
	set procName "$ccdProc\_$ccdname"
	set psLine "ps -aef | grep $ccdProc | grep $ccdname | grep -v grep"
	if {[catch {set line [exec $psLine]} ret]} continue
	if {[ccdSendCommandAndRecvReply $rtapenv $procName PING reply] != $OK} \
	    continue
	ccdVerifyWarning "ccdVerRtapCcs" \
		"CCD process $procName is running" "" RTAPENV
	set pid [lindex [split [string trim $line] ] 1]
	if {[ccdSendCommandAndRecvReply $rtapenv $procName EXIT reply] != $OK} {
	    ccdVerifyError "ccdVerRtapCcs" "Could not terminate the process $procName" "Kill UNIX process $pid" RTAPENV
	    catch {exec kill $pid}
	}
    }
    return $OK
}

#
# procedure to check if the CCD processes are running on the LCU
#
proc ccdCheckCcdProcessesLCU { } {
    global gvar
    global ccdlenv ccdname
    global NOP ERR RUN WRN OK

    set gvar(ccdconRunLCU) $NOP
    set gvar(ccditRunLCU)  $NOP
    set gvar(ccdipRunLCU)  $NOP
    set gvar(ccdrdtRunLCU) $NOP
    set gvar(ccdshtRunLCU) $NOP
    set gvar(ccdtelRunLCU) $NOP
    set gvar(ccdtmpRunLCU) $NOP

    set ccdProcList [list ccdcon ccdit ccdip ccdrdt ccdsht ccdtel ccdtmp]
    foreach ccdProc $ccdProcList {
	set procName "$ccdProc\_$ccdname"
	if {[ccdSendCommandAndRecvReply $ccdlenv $procName PING reply] == $OK} {
	    set gvar(ccdVerMessage) "CCD process $ccdProc is running"
	    set var "$ccdProc"
	    append var "RunLCU"
	    set gvar($var) $RUN
	}
    }
    return $OK
}

proc ccdVerifyUTC { } {
    global gvar
    global ccdlenv
    global NOP ERR RUN WRN OK

    putsnl ""
    putsnl "Check UTC time alignment"
    putsnl ""
    ccdVerifyUpdate ccdVerUtcTime $RUN

    if {[ccdSendCommandAndRecvReply $ccdlenv lccServer TIMGUTC lcuTime] != $OK} {
	ccdVerifyError ccdVerUtcTime "Could not retrieve LCU time" "" CCDLENV
	return $ERR
    }
    putsnl "Get LCU time ... done"

    set wsTime [seq_isoTime]
    putsnonl "Get WS  time ... done"

    set llcuTime [split $lcuTime]
    set secLCU [expr {round([lindex $llcuTime 1])}]
    set timMod [lindex $llcuTime 4]
    
    set secWS  [seq_isoTimeToClock $wsTime]

    set secDiff [expr {abs($secWS - $secLCU)}]

    putsnl "UTC time is $wsTime"
    putsnl "  LCU time = $secLCU s - TIM in $timMod mode"
    putsnl "  WS  time = $secWS  s"
    putsnl "  -> Difference = $secDiff seconds"
    if {$secDiff > 1.0} { 
	ccdVerifyWarning ccdVerUtcTime "UTC times are not aligned" "" CCDLENV
	return $WRN 
    }
    putsnl ""
    putsnl "UTC times are aligned"
    ccdVerifyUpdate ccdVerUtcTime $OK
    return $OK
}

#
# procedure to perform all WS checks
#
proc ccdVerifyWS { } {
    global gvar env cv
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global display
    global NOP ERR RUN WRN OK

    if {$gvar(ccdVerDoWsEnv) == 1 } {

	set noIntRoot 0
	#
	# 1 - Check Environment Variables
	#
	putsnl ""
	putsnl " 1 - Check Environment Variables"
	putsnl "================================"
	
	putsnl ""
	putsnl "1a - Hosts"
	putsnl "----------"
	ccdVerifyUpdate ccdVerEnvHost $RUN
	set ENVHST [list HOST INS_HOST]
	foreach var $ENVHST {
	    if {![info exists env($var)]} {
		if { $var != "INS_HOST" } {
		    ccdVerifyError ccdVerEnvHost \
			    "$var is not defined. Please fix it." "" UNIXENV
		    return $ERR
		} else {
		    putsnonl "$var is not defined. "
		    putsnl "Defaulted to \$HOST=$env(HOST)"
		    set inshost $env(HOST)
		}
	    } else {
		if { $var ==     "HOST" } { set host    $env(HOST)     }
		if { $var == "INS_HOST" } { set inshost $env(INS_HOST) }
		putsnl "$var defined as $env($var)"
	    }
	}
	set gvar(ccdHOST) $host
	ccdVerifyUpdate ccdVerEnvHost $OK
	putsnl ""
	
	putsnl "1b - Environment variables"
	putsnl "--------------------------"
	ccdVerifyUpdate ccdVerEnvVar $RUN
	set ENVVAR [list VLTROOT VLTDATA INTROOT RTAPENV INS_ROOT CCDNAME CCDLENV]
	foreach var $ENVVAR {
	    if {![info exists env($var)]} {
		if { $var == "INTROOT" } {
		    set noIntRoot 1
		    putsnonl "$var is not defined. "
		    putsnl "Defaulted to \$VLTROOT=$env(VLTROOT)"
		} else { 
		    ccdVerifyError ccdVerEnvVar \
			    "$var is not defined. Please fix it." "" UNIXENV
		    return $ERR
		}
	    } else {
		putsnl "$var defined as $env($var)"
	    }
	}
    } else {
	set noIntRoot 1
	if {[info exists env(INTROOT)]} { set noIntRoot 0 }
    }
    
    set host    $env(HOST)
    if {[info exists env(INS_HOST)]} { 
	set inshost $env(INS_HOST) 
    } else {
	set inshost $host
    }
    set vltroot $env(VLTROOT)
    if { $noIntRoot == 0 } {
	set introot $env(INTROOT)
    } else {
	set introot $vltroot
    }
    set vltdata $env(VLTDATA)
    set insroot $env(INS_ROOT)
    set rtapenv $env(RTAPENV)
    set ccdname $env(CCDNAME)
    set ccdlenv $env(CCDLENV)
    set envdir $vltdata/ENVIRONMENTS

    set gvar(ccdCCDNAME) $ccdname
    set gvar(ccdRTAPENV) $rtapenv
    set gvar(ccdCCDLENV) $ccdlenv

    if {$gvar(ccdVerDoWsEnv) == 1 } {
	ccdVerifyUpdate ccdVerEnvVar $OK

	putsnl ""
	putsnl "1c - Directories"
	putsnl "----------------"
	ccdVerifyUpdate ccdVerEnvDir $RUN
	set ENVDIR [list VLTROOT VLTDATA INS_ROOT]
	if { $noIntRoot == 0 } {
	    set ENVDIR [concat $ENVDIR INTROOT]
	}
	foreach var $ENVDIR {
	    if {![file isdirectory $env($var)]} {
		if {$var == "INTROOT"} {
		    set noIntRoot 0 
		    putsnl "INTROOT is not a directory ... ignored"
		} else {
		    ccdVerifyError ccdVerEnvDir \
			    "$var is not a directory. Please fix it." "" VLTROOT
		    return $ERR
		}
	    } else {
		putsnl "$var is a directory"
	    }
	}
	ccdVerifyUpdate ccdVerEnvDir $OK
    }

    # 
    # 2 - Check INS_ROOT
    # 	- dbcfg files
    # 	- Dictionary
    # 	- std setup files
    # 
    if {$gvar(ccdVerDoCcdEnv) == 1 } {
	putsnl ""
	putsnl " 2 - Check INS_ROOT"
	putsnl "==================="
	
	putsnl ""
	putsnl "2a - Directory structure"
	putsnl "------------------------"
	
	ccdVerifyUpdate ccdVerInsDir $RUN
	
	if {![info exists env(INS_USER)]} {
	    set insuser "SYSTEM"
	    putsnl "INS_USER is not defined. Use default SYSTEM"
	} else {
	    set insuser $env(INS_USER)
	    putsnl "INS_USER is defined as $insuser"
	}
    } else {
	if {![info exists env(INS_USER)]} {
	    set insuser "SYSTEM"
	} else {
	    set insuser $env(INS_USER)
	}
    }
    
    set dicdir $insuser/Dictionary
    set comdir $insuser/COMMON
    set cfgdir $comdir/CONFIGFILES
    set setdir $comdir/SETUPFILES
    set detdir $setdir/DET
    set refdir $setdir/REF
    set imgdir $insuser/DETDATA

    if {$gvar(ccdVerDoCcdEnv) == 1 } {
	set SUBDIR [list $dicdir $comdir $cfgdir $setdir $detdir $refdir $imgdir]
	if { ! [ file isdirectory $insroot/$insuser ] } {
	    ccdVerifyError ccdVerInsDir \
		    "\$INS_ROOT/$insuser is not a directory." "" INSTALL
	    return $ERR
	}
	foreach subdir $SUBDIR {
	    if { [ file isdirectory $insroot/$subdir ] } {
		putsnl "\$INS_ROOT/$subdir ... OK"
	    } else {
		ccdVerifyError ccdVerInsDir \
			"\$INS_ROOT/$subdir is not a directory." "" INSTALL
		return $ERR
	    }
	}
	ccdVerifyUpdate ccdVerInsDir $OK
	
	putsnl ""
	putsnl "2b - Dictionary"
	putsnl "---------------"
	if {[info exists cv]} { $cv.ccdDictionary configure -color Black }
	ccdVerifyUpdate ccdCCDDID "-"
	if {![info exists env(CCDDID)]} {
	    set ccddid "CCDDCS"
	    putsnl "CCDDID is not defined. Use default CCDDCS"
	} else {
	    set ccddid $env(CCDDID)
	    putsnl "CCDDID is defined as $ccddid"
	}
	if {![ file exists $insroot/$dicdir/ESO-VLT-DIC.$ccddid ]} {
	    ccdVerifyError "" \
		    "Dictionary ESO-VLT-DIC.$ccddid not found under \$INS_ROOT/$dicdir" \
		    "" INSTALL
	    ccdVerifyUpdate ccdCCDDID "ERROR"
	    if {[info exists cv]} { $cv.ccdDictionary configure -color Red }
	    return $ERR
	}
	putsnl "Dictionary ESO-VLT-DIC.$ccddid ... OK"
	ccdVerifyUpdate ccdCCDDID $ccddid
	
	putsnl ""
	putsnl "2c - Configuration files"
	putsnl "------------------------"
	ccdVerifyUpdate ccdVerInsCfg $RUN
	set e "dbcfg"
	set files [list ccdDcsFixed $ccdname]
	foreach f $files {
	    if {![ file exists $insroot/$cfgdir/$f.$e ]} {
		ccdVerifyError ccdVerInsCfg \
			"Configuration file \$INS_ROOT/$cfgdir/$f.$e not found" \
			"" INSTALL
		return $ERR
	    }
	    putsnl "Configuration file $f.$e ... OK"
	}
	
	foreach f {ccdTecTemplate ccdSciTemplate} {
	    if {[ file exists $insroot/$cfgdir/$f.$e ]} {
		putsnl "Template file $f.$e ... OK"
	    } else {
		ccdVerifyWarning ccdVerInsCfg \
			"Template file \$INS_ROOT/$cfgdir/$f.$e not found" \
			"" INSTALL
	    }
	}
	if {$gvar(ccdVerInsCfg) != $WRN } {
	    set gvar(ccdVerInsCfg) $OK
	}
	update idletasks
	
	putsnl ""
	putsnl "2d - Setup files"
	putsnl "----------------"
	ccdVerifyUpdate ccdVerInsSet $RUN
	set files [list ccdSetupComplete ccdCmdBias ccdCmdDisplay ccdCmdFlat ccdCmdStartAg ccdCmdStartLp]
	foreach f $files {
	    if {[ file exists $insroot/$detdir/$f.det ]} {
		putsnl "Setup file $f.det ... OK"
	    } else {
		ccdVerifyWarning ccdVerInsSet \
			"Setup file \$INS_ROOT/$detdir/$f.det not found" \
			"" INSTALL
	    }
	}
	if {$gvar(ccdVerInsSet) != $WRN } {
	    set gvar(ccdVerInsSet) $OK
	}
	update idletasks
    }
    set dicdir $insroot/$insuser/Dictionary
    set cfgdir $insroot/$insuser/COMMON/CONFIGFILES
    set detdir $insroot/$insuser/COMMON/SETUPFILES/DET
    set imgdir $insroot/$insuser/DETDATA

    # 
    # 3 - Check RTAP
    # 	- Running
    # 	- check logLCU.CONFIG
    # 	- DB access
    # 	- CCD branch is there (Read)
    # 
    if {$gvar(ccdVerDoRtapEnv) == 1 } {
	putsnl ""
	putsnl " 3 - Check RTAP Environment"
	putsnl "==========================="
	
	ccdVerifyUpdate ccdVerRtapEnv $RUN
	putsnl ""
	putsnl "3a - RTAP Environment"
	putsnl "---------------------"
	if {[catch {set line [exec vccEnvCheck -e $rtapenv -h $host]} ret] != 0} {
	    ccdVerifyError ccdVerRtapEnv "" $ret RTAPENV
	    return $ERR
	}
	putsnl $line
	putsnl "RTAP environment $rtapenv ... OK"
	set envdir $vltdata/ENVIRONMENTS
	set gvar(ccdRTAPENV) "$rtapenv@$host"
	ccdVerifyUpdate ccdVerRtapEnv $OK
	
	putsnl ""
	putsnl "3b - CCS Init"
	putsnl "-------------"
	ccdVerifyUpdate ccdVerRtapCcs $RUN
	putsnonl "CCS Init ... "
	if {[catch seq_ccsInit ret] != 0} {
	    putsnl "failed"
	    ccdVerifyError ccdVerRtapCcs "Could not register to CCS" $ret RTAPENV
	    return $ERR
	}
	putsnl "OK"
	if {[ccdCheckCcdProcessesWS] != $OK} { 
	    ccdVerifyUpdate ccdVerRtapCcs $ERR
	    return $ERR 
	}
	ccdVerifyUpdate ccdVerRtapCcs $OK
	
	putsnl ""
	putsnl "3c - Database branch"
	putsnl "--------------------"
	ccdVerifyUpdate ccdVerRtapDbf $RUN
	set dbdir $vltdata/ENVIRONMENTS/$rtapenv/DB
	if {[catch {set dbf [exec find $dbdir -print]} ret] != 0} {
	    ccdVerifyError ccdVerRtapDbf "Could not find database files" \
		    "" RTAPENV
	    return $ERR
	}
	foreach f $dbf {
	    if {[catch {set fnd [exec grep Alias $f]} ret] != 0} { continue }
	    if {[lsearch -exact $fnd $ccdname] == -1} { continue }
	    set l [string length $dbdir]
	    set y [expr {[string last "/" $f] - 1}]
	    set x [split [string range $f $l $y] "/"]
	    set p [lindex [join $x ":"] 0]
	    putsnonl "DB path to CCD branch $ccdname is : "
	    putsnl "@$rtapenv<alias>$ccdname = $p"
	    set fnd $OK
	    break
	}
	if {$fnd != $OK} {
	    ccdVerifyError ccdVerRtapDbf "Could not find CCD branch Alias file" \
		    "" RTAPENV
	    return $ERR
	}
	ccdVerifyUpdate ccdVerRtapDbf $OK
	
	putsnl ""
	putsnl "3d - Database access"
	putsnl "--------------------"
	ccdVerifyUpdate ccdVerRtapDba $RUN
	if {[ccdCheckDatabaseAccess $rtapenv] != $OK} {
	    ccdVerifyError ccdVerRtapDba "RTAP Database access checks failed" \
		    "" RTAPENV
	    return $ERR
	}
	putsnl ""
	putsnl "RTAP Database access ... OK"
	ccdVerifyUpdate ccdVerRtapDba $OK
	
	putsnl ""
	putsnl "3e - Read FITS logs mask"
	putsnl "------------------------"
	if {[info exists cv]} { $cv.ccdLogMaskWS configure -color Black }
	set gvar(ccdVerLogMaskWS) "-"
	if {[ccdDbRead "@$rtapenv<alias>$ccdname.logMask" logMaskWS] != $OK} { 
	    set gvar(ccdVerLogMaskWS) "ERROR"
	    if {[info exists cv]} { cv.ccdLogMaskWS configure -color Red }
	    return $ERR 
	}
	if {[string length $logMaskWS] != 0} {
	    putsnl "FITS log mask is set to $logMaskWS"
	    set gvar(ccdVerLogMaskWS) $logMaskWS
	} else {
	    set gvar(ccdVerLogMaskWS) "NOT SET"
	    ccdVerifyWarning "" "FITS log mask is not set" "" RTAPENV
	    if {[info exists cv]} { $cv.ccdLogMaskWS configure -color Orange }
	}
	update idletasks
	
	putsnl ""
	putsnl "3f - Check CDT of CCD processes"
	putsnl "-------------------------------"
	putsnl ""
	ccdVerifyUpdate ccdVerCcdCdt $RUN
	foreach p {ccdconCI ccditWs} {
	    putsnl ""
	    if {[ccdSendCommandAndRecvReply $rtapenv cmdManager CMDLIST cdt $p] != $OK} {
		ccdVerifyError ccdVerCcdCdt "Retrieving $p CDT" "" RTAPENV
		return $ERR
	    }
	    set cdtList [split $cdt ","]
	    putsnl "  > CDT $p@$rtapenv : [lindex $cdtList 0] commands"
	    putsnl ""
	}
	putsnl "CCD CDT@$rtapenv ... OK"
	ccdVerifyUpdate ccdVerCcdCdt $OK
	
	putsnl ""
	putsnl "3g - Check RTD server running"
	putsnl "-----------------------------"
	set gvar(ccdVerRtdSrv) $RUN
	if {[catch {set line [exec ps -aef | grep rtdServer | grep -v grep]} ret] != 0} {
	    ccdVerifyWarning ccdVerRtdSrv "RTD server is not running" "" RTDSRV
	    set display -1
	} else {
	    set display 0
	    putsnl "RTD server is running ... OK"
	    set gvar(ccdVerRtdSrv) $OK
	}
    } elseif {[catch seq_ccsInit ret] != 0} {
	ccdVerifyError "" "Could not register to CCS" $ret RTAPENV
	return $ERR
    }

    if { ($gvar(ccdVerDoWsEnv)   == 1) || \
	 ($gvar(ccdVerDoCcdEnv)  == 1) || \
	 ($gvar(ccdVerDoRtapEnv) == 1) } {    
        putsnl ""
        putsnl ""
        putsnl "=========================="
	putsnl "CCD WS part is OK"
	putsnl "=========================="
	putsnl ""
	putsnl ""
    }
    return $OK
}

#
# procedure to perform all LCU checks
#
proc ccdVerifyLCU { } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global UKN NOP ERR RUN WRN OK

    # 
    # 4 - Check LCU
    # 	- /etc/ping
    # 	- message to lccServer (eg LCCGVER)
    # 	- DB access
    # 	- CCD branch is there (Read)
    # 	- check NFS access to INS_ROOT on INS_HOST
    # 	- check B016/ARC channels
    # 	- check ACE connection (->acecom0)
    # 
    if {$gvar(ccdVerDoLcuEnv) == 1 } {
	putsnl ""
	putsnl " 4 - Check LCU Environment"
	putsnl "=========================="
	
	putsnl "4a - LCU Environment"
	putsnl "---------------------"
	ccdVerifyUpdate ccdVerLcuEnv $RUN
        # At UT1 the /etc/exports file does not contain the list of LCUs
        # but the environment is OK. In order to let the check pass over there
        # I comment out the check here. To be taken back ONLY when we are
        # sure that at Paranal it works
	#if {[catch {set line [exec vccEnvCheck -e $ccdlenv -h $host]} ret] != 0} {
	#    ccdVerifyError ccdVerLcuEnv ""	$ret CCDLENV
	#    return $ERR
	#}
	#putsnl $line
	putsnl "LCU environment $ccdlenv ... OK"
	ccdVerifyUpdate ccdVerLcuEnv $OK
	
	putsnl ""
	putsnl "4b - Database branch"
	putsnl "--------------------"
	ccdVerifyUpdate ccdVerLcuDbf $RUN
	set dbdir $envdir/$ccdlenv/DB
	if {[catch {set dbf [exec find $dbdir -print]} ret] != 0} {
	    ccdVerifyError ccdVerLcuDbf "Could not find database files" \
		    "" CCDLENV
	    return $ERR
	}
	foreach f $dbf {
	    if {[catch {set fnd [exec grep Alias $f]} ret] != 0} { continue }
	    if {[lsearch -exact $fnd $ccdname] == -1} { continue }
	    set l [string length $dbdir]
	    set y [expr {[string last "/" $f] - 1}]
	    set x [split [string range $f $l $y] "/"]
	    set p [lindex [join $x ":"] 0]
	    putsnonl "DB path to CCD branch $ccdname is : "
	    putsnl "@$ccdlenv<alias>$ccdname = $p"
	    set fnd $OK
	    break
	}
	if {$fnd != $OK} {
	    ccdVerifyError ccdVerLcuDbf "Could not find CCD branch Alias file" \
		    "" CCDLENV
	    return $ERR
	}
	ccdVerifyUpdate ccdVerLcuDbf $OK
	
	putsnl ""
	putsnl "4c - Boot files & BootScript"
	putsnl "----------------------------"
	ccdVerifyUpdate ccdVerLcuBf $RUN
	set p "$envdir/$ccdlenv"
	set files [list bootScript devicesFile lccSnapshot.db lcubootLogFile logfile rebootFile]
	foreach f $files {
	    if {[file exists $p/$f]} {
		putsnl "Boot file : $f ... OK"
	    } else {
		ccdVerifyError ccdVerLcuBf \
			"Boot file \$VLTDATA/ENVIRONMENTS/$ccdlenv/$f not found" \
			"" CCDLENV
		return $ERR
	    }
	}
	ccdVerifyUpdate ccdVerLcuBf $OK
	
	ccdVerifyUpdate ccdVerLcuBs $RUN
	set bs "$envdir/$ccdlenv/bootScript"
	
	if {[catch {set line [exec grep vLcuHost: $bs]} ret] != 0} {
	    ccdVerifyError ccdVerLcuBs \
		    "Could not find string vLcuHost: in $bs" \
		    "" CCDLENV
	    return $ERR
	} else {
	    set ccdlcu [lrange $line end end]
	}
	ccdVerifyUpdate ccdCCDLENV "$ccdlenv@$ccdlcu"
	
	if {[catch {set line [exec grep vVltHost: $bs]} ret] != 0} {
	    ccdVerifyError ccdVerLcuBs \
		    "Could not find string vVltHost: in $bs" \
		    "" CCDLENV
	    return $ERR
	} else {
	    set boothost [lrange $line end end]
	}
	if {[catch {set line [exec grep BOOTHOME: $bs]} ret] != 0} {
	    ccdVerifyError ccdVerLcuBs "Could not find string BOOTHOME: in $bs" \
		    "" CCDLENV
	    return $ERR
	} else {
	    set boothome [lrange $line end end]
	}
	
	putsnl "LCU is $ccdlcu booting from $boothost:$boothome"
	putsnl ""
	
	if {[catch {set line [exec grep vSysmodlist: $bs]} ret] != 0} {
	    ccdVerifyError ccdVerLcuBs \
		    "Could not find string vSysmodlist: in $bs" \
		    "" CCDLENV
	    return $ERR
	} else {
	    set sysmodList [lrange $line 2 end]
	}
	if {[catch {set line [exec grep vUsrmodlist: $bs]} ret] != 0} {
	    ccdVerifyError ccdVerLcuBs \
		    "Could not find string vUsrmodlist: in $bs" \
		    "" CCDLENV
	    return $ERR
	} else {
	    set usrmodList [lrange $line 2 end]
	}

	set modList [concat $sysmodList $usrmodList]
	set wrnMissingList {}
	set errMissingList {}
	putsnonl "System and User module list ... "
	set expmodList [list lcudrv lculog lqs tim lcc cai scan ccd ccdipUSR b016 arcdrv]
	set fndDRV 0
	foreach m $expmodList {
	    set fnd 0
	    foreach x $modList {
		if { $x == $m } {
		    putsnonl "$m "
		    set fnd 1
		    if { ($m == "b016") || ($m == "arcdrv") } { set fndDRV 1 }
		    break
		}
	    }
	    if { $fnd == 0 } { 
		if { ($m == "b016") || ($m == "arcdrv") } {
		    set fnd 0
		} elseif { ($m == "ntp") || ($m == "tim") } {
		    set wrnMissingList [concat $wrnMissingList "'$m'"]
		} else {
		    set errMissingList [concat $errMissingList "'$m'"]
		}
	    }
	}
	if {$fndDRV == 0} {
	    set errMissingList [concat $errMissingList "'b016' or 'arcdrv'"]
	}
	set n [llength $errMissingList]
	if {$n > 0} {
	    putsnl "... ERROR"
	    if {$n == 1} { 
		set msg "Module $errMissingList is"
	    } else {
		set msg "Modules $errMissingList are"
	    }
	    set msg "$msg missing"
	    ccdVerifyError ccdVerLcuBs $msg "" CCDLENV
	    return $ERR
	} else {
	    ccdVerifyUpdate ccdVerLcuBs $OK
	    putsnl "... OK"
	    set n [llength $wrnMissingList]
	    if {$n > 0} {
		if {$n == 1} { 
		    set msg "Module $wrnMissingList is"
		} else {
		    set msg "Modules $wrnMissingList are"
		}
		set msg "$msg missing"
		ccdVerifyWarning ccdVerLcuBs $msg "" CCDLENV
	    }
	}
	
	putsnl ""
	putsnl "4d - Reboot"
	putsnl "-----------"
	putsnl ""
	ccdVerifyUpdate ccdVerLcuBoot $RUN
	putsnonl "Reboot LCU $ccdlcu ... Please wait! ... "
	if {[catch {set y [exec lccBoot $ccdlenv 300000 ]} ret] != 0} {
	    putsnl "Failed"
	    ccdVerifyError ccdVerLcuBoot "Could not reboot LCU $ccdlcu" \
		    $ret CCDLENV
	    return $ERR
	}
	set x [catch {set y [exec scanDevOn $ccdlenv]} ret]
	putsnl "done"
        set utcCheck [ccdVerifyUTC]
	ccdVerifyUpdate ccdVerLcuBoot $utcCheck
        if {$utcCheck == $ERR} { return $ERR }
	
	putsnl ""
	putsnl "4e - Database access"
	putsnl "--------------------"
	ccdVerifyUpdate ccdVerLcuDba $RUN
	if {[ccdCheckDatabaseAccess $ccdlenv] != $OK} {
	    ccdVerifyError ccdVerLcuDba "LCU Database access checks failed" \
		    "" CCDLENV
	    return $ERR
	}
	putsnl ""
	putsnl "LCU Database access ... OK"
	ccdVerifyUpdate ccdVerLcuDba $OK
	
	putsnl ""
	putsnl "4f - NFS access to INS_ROOT"
	putsnl "---------------------------"
	putsnl ""
	ccdVerifyUpdate ccdVerLcuNfs $RUN
	
	if {$inshost != $host } {
	    putsnl "\$INS_ROOT is located on \$INS_HOST=$inshost"
	    putsnl "Check is postponed to next startup ... "
	    ccdVerifyUpdate ccdVerLcuNfs $UKN
	} else {
	    set lcuBootHost $host
	    set insRootDisk [lindex [split $insroot "/"] 1]
	    if {[cequal [file type "/$insRootDisk"] "link"]} {
		set insRootLink [file readlink "/$insRootDisk"]
		putsnl "\$INS_ROOT is a link to $insRootLink"
		set insRootDisk [lindex [split $insRootLink "/"] 1]
	    } 
	    set insRootDisk "/$insRootDisk"
	    putsnl "\$INS_ROOT file system is $inshost:$insRootDisk"
	    putsnonl "Search $insRootDisk in /etc/exports ... "
	    if {[catch {set exp [exec grep $insRootDisk /etc/exports]} ret] != 0} {
		putsnl "Failed"
		putsnl "Check is postponed to next startup ... "
		ccdVerifyWarning ccdVerLcuNfs \
			"$insRootDisk was not found in /etc/exports" \
			"" CCDLENV
	    } else {
		putsnl "done"
		set exportList [lrange [split $exp ":"] 1 end] 
		if {[lsearch -exact $exportList $ccdlcu] == -1} {
		    putsnl "Check is postponed to next startup ... "
                    # At Paranal /etc/exports does not contain list of LCU
                    # I take out the warning here
		    ccdVerifyUpdate ccdVerLcuNfs $OK
		    #ccdVerifyWarning ccdVerLcuNfs \
		    #"$ccdlcu was not found in clients list of $insRootDisk" \
                    #"" CCDLENV
		} else {
		    putsnl "LCU $ccdlcu is declared in $insRootDisk export list"
		    putsnl "$insRootDisk can be exported to $ccdlcu via NFS"
		    putsnl ""
		    putsnl "Export INS_ROOT file system to $ccdlcu via NFS ... OK"
		    ccdVerifyUpdate ccdVerLcuNfs $OK
		}
	    }
	}
    }	

    # 
    # 5 - Scan system
    # 	- check running
    # 	- DB write on LCU
    # 	- DB read  on WS  + Check
    # 
    if {$gvar(ccdVerDoScanSys) == 1 } {
	putsnl ""
	putsnl " 5 - Scan system"
	putsnl "================"
	set pathWS  "@$rtapenv<alias>$ccdname:"
	set pathLCU "@$ccdlenv<alias>$ccdname:"
	
	putsnl ""
	putsnl "5a - Check scan system running"
	putsnl "------------------------------"
	ccdVerifyUpdate ccdVerCcsScan $RUN
	putsnl ""
	if {[ccdSendCommandAndRecvReply $rtapenv ccsScan PING ping] != $OK} { 
	    ccdVerifyError ccdVerCcsScan \
		    "CCS ccsScan process is not running" "" CCSSCAN
	    return $ERR
	}
	putsnl ""
	putsnl "CCS ccsScan process running ... OK"
	putsnl ""
	if {[ccdSendCommandAndRecvReply $ccdlenv scanOut PING ping] != $OK} { 
	    ccdVerifyError ccdVerCcsScan \
		    "LCU scanOut task is not running" "" LCCSCAN
	    return $ERR
	}
	putsnl ""
	putsnl "LCC scanOut task running ... OK"
	ccdVerifyUpdate ccdVerCcsScan $OK
	
	putsnl ""
	putsnl "5b - Check DCS scan links"
	putsnl "-------------------------"
	ccdVerifyUpdate ccdVerDcsScan $RUN
	set attrDCS "exposures:exposure_1.expStatus"
	if {[ccdDbRead "$pathLCU$attrDCS" valorg] != $OK} { 
	    ccdVerifyError ccdVerDcsScan "" "" DCSSCAN
	    return $ERR
	}
	if {[ccdDbWrite "$pathLCU$attrDCS" 255] != $OK} { 
	    ccdVerifyError ccdVerDcsScan "" "" DCSSCAN
	    return $ERR
	}
	after 5000
	if {[ccdDbRead "$pathWS$attrDCS" valmod] != $OK} { 
	    ccdVerifyError ccdVerDcsScan "" "" DCSSCAN
	    return $ERR
	}
	if { $valmod != 255 } {
	    ccdVerifyError ccdVerDcsScan "DCS Scan links disabled" "" DCSSCAN
	    return $ERR
	} else {
	    putsnl "DCS scan system ... OK"
	    if {[ccdDbWrite "$pathLCU$attrDCS" $valorg] != $OK} { 
		ccdVerifyError ccdVerDcsScan "" "" DCSSCAN
		return $ERR
	    }
	    ccdVerifyUpdate ccdVerDcsScan $OK
	}
	
	putsnl ""
	putsnl "5c - Check OS scan links"
	putsnl "------------------------"
	ccdVerifyUpdate ccdVerOsScan $RUN
	set attrOS "exposures:exposure_1:readout.gainIndex"
	if {[ccdDbRead  "$pathLCU$attrOS" valorg] != $OK} { 
	    ccdVerifyError ccdVerOsScan "" "" OSSCAN
	    return $ERR
	}
	if {[ccdDbWrite "$pathLCU$attrOS" 255] != $OK} { 
	    ccdVerifyError ccdVerOsScan "" "" OSSCAN
	    return $ERR
	}
	after 5000
	if {[ccdDbRead "$pathWS$attrOS" valmod] != $OK} { 
	    ccdVerifyError ccdVerOsScan "" "" OSSCAN
	    return $ERR
	}
	if { $valmod != 255 } {
	    ccdVerifyWarning ccdVerOsScan "Stand-Alone Scan links disabled" "" OSSCAN
	} else {
	    if {[ccdDbWrite "$pathLCU$attrOS" $valorg] != $OK} { 
		ccdVerifyError ccdVerOsScan "" "" OSSCAN
		return $ERR
	    }
	    putsnl "OS  scan system ... OK"
	    ccdVerifyUpdate ccdVerOsScan $OK
	}
    }
    putsnl ""
    putsnl ""
    putsnl "=========================="
    putsnl "CCD LCU part is OK"
    putsnl "=========================="
    putsnl ""
    putsnl ""
    return $OK
}

#
# procedure to perform all ACE checks
#
proc ccdVerifyACE { } {
    global gvar env
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global NOP ERR RUN WRN OK

    # 
    # 6 - ACE system
    # 	- check B016 devices
    # 	- Check LCU-ACE communication
    # 
    putsnl ""
    putsnl " 6 - ACE system"
    putsnl "==============="
    putsnl ""
    putsnonl "ccdDcsStart.sh ... "
    set x [catch {set y [exec ccdDcsStart.sh >/dev/null]} ret]
    putsnl "done"

    if {[ccdDbRead "@$rtapenv<alias>$ccdname:ace.devName" ch] != $OK} {
	return $ERR
    }
    ccdVerifyUpdate ccdDRV $ch

    if {[string match $ch "/bxvi0"]} {
	putsnl "6a - Check B016 device"
	putsnl "----------------------"
	ccdVerifyUpdate ccdVerDriver $RUN
	if {[ccdSendCommandAndRecvReply $ccdlenv ccdcon_$ccdname CHKDRV reply $ch] != $OK} {
	    ccdVerifyError ccdVerDriver "B016 driver device '$ch' check failed" "" CCDACE
	    return $ERR
	}
	ccdVerifyUpdate ccdVerDriver $OK
    } else {
	putsnl "6a - Check ARC device"
	putsnl "---------------------"
	ccdVerifyUpdate ccdVerDriver $RUN
	if {[ccdSendCommandAndRecvReply $ccdlenv ccdcon_$ccdname CHKDRV reply $ch] != $OK} {
	    ccdVerifyError ccdVerDriver "ARC driver device '$ch' check failed" "" CCDACE
	    return $ERR
	}
	ccdVerifyUpdate ccdVerDriver $OK
    }
    
    putsnl ""
    putsnl "6b - Check LCU-ACE communication"
    putsnl "--------------------------------"
    ccdVerifyUpdate ccdVerAceCom $RUN
    if {[ccdSendCommandAndRecvReply $ccdlenv ccdcon_$ccdname CHKACE reply $ch] != $OK} {
	ccdVerifyError ccdVerAceCom "LCU-ACE communication check failed" "" CCDACE
	return $ERR
    }
    ccdVerifyUpdate ccdVerAceCom $OK
    
    putsnl ""
    putsnl ""
    putsnl "=========================="
    putsnl "CCD ACE part is OK"
    putsnl "=========================="
    putsnl ""
    putsnl ""
    return  $OK
}

#
# procedure to check TMP sub-system
#
proc ccdVerifyTMP { } {
    global gvar
    global ccdlenv ccdname ccdOpState
    global NOP ERR RUN WRN OK

    putsnl ""
    putsnl "Check temperature sub-system"
    putsnl "----------------------------"
    set attr "@$ccdlenv<alias>$ccdname:temperature.available"
    if {[ccdDbRead $attr tmpAvail] != $OK} { return $ERR }
    if { $tmpAvail } {
	set p "ccdtmp_$ccdname"
	if {[ccdSendCommandAndRecvReply $ccdlenv $p STATUS reply] != $OK} { 
	    return $ERR
	}
	putsnl "  > $reply"
	putsnl ""
	set opState [lindex [split [lindex [split $reply ","] 1] " "] 1]
	if {$opState < $ccdOpState(ONLINE)} {
	    ccdVerifyWarning "" "Temperature sub-system is not On-Line" \
		    "" CCDHEAD
	    return $WRN
	}
	set attrMsg "@$ccdlenv<alias>$ccdname:temperature.message"
	if {[ccdDbRead $attrMsg tmpMsg] != $OK} { return $ERR }
	putsnl "TMP : $tmpMsg"
	if {$tmpMsg != "Chip is cooling down ... Please wait (60s)" } {
	    ccdVerifyWarning "" "Peltier element is not cooling" \
		    "$tmpMsg" CCDHEAD
	    return $WRN
	} 
	set cmd "GETPAR"
	if {$opState == $ccdOpState(ACTIVE)} { set cmd "CONT" } 
	set attrVal "@$ccdlenv<alias>$ccdname:temperature.current(0:1)"
	for {set t 0} {$t < 8} {incr t} {
	    if {[ccdSendCommandAndRecvReply $ccdlenv $p $cmd reply] != $OK} { 
		return $ERR
	    }
	    if {[ccdDbRead $attrVal tmpVal] != $OK} { return $ERR }
	    lassign $tmpVal HV CV
	    set fmt "TMP : House = %6.2f K - Chip = %6.2f K"
	    putsnl [format $fmt $HV $CV]
	    if {[ccdDbRead $attrMsg tmpMsg] != $OK} { return $ERR }
	    if {$tmpMsg == "All temperature values OK" } { break }
	    after 10000
	}
	if {$tmpMsg != "All temperature values OK" } {
	    ccdVerifyWarning "" "$tmpMsg" "" CCDHEAD
	}
	putsnl "TMP : $tmpMsg"
    }
    return $OK
}

#
# procedure to check TEL sub-system
#
proc ccdVerifyTEL { } {
    global gvar
    global ccdlenv ccdname ccdOpState
    global NOP ERR RUN WRN OK

    putsnl ""
    putsnl "Check telemetry sub-system"
    putsnl "--------------------------"
    set attr "@$ccdlenv<alias>$ccdname:telemetry.available"
    if {[ccdDbRead $attr telAvail] != $OK} { return $ERR }
    if { $telAvail } {
	set p "ccdtel_$ccdname"
	if {[ccdSendCommandAndRecvReply $ccdlenv $p STATUS reply] != $OK} { 
	    return $ERR
	}
	putsnl "  > $reply"
	putsnl ""
	set opState [lindex [split [lindex [split $reply ","] 1] " "] 1]
	if {$opState < $ccdOpState(ONLINE)} {
	    ccdVerifyWarning "" "Telemetry sub-system is not On-Line" \
		    "" CCDHEAD
	    return $WRN
	}
	set cmd "GETPAR"
	if {$opState == $ccdOpState(ACTIVE)} { set cmd "CONT" } 
	if {[ccdSendCommandAndRecvReply $ccdlenv $p $cmd reply] != $OK} { 
	    return $ERR
	}
	set attr "@$ccdlenv<alias>$ccdname:telemetry.message"
	if {[ccdDbRead $attr telMsg] != $OK} { return $ERR }
	putsnl "TEL : $telMsg"
	if {$telMsg != "All telemetry values OK" } {
	    ccdVerifyWarning "" "$telMsg" "" CCDHEAD
	}
    }
    return $OK
}

#
# procedure to check the FITS log mask
#
proc ccdVerifyLogMask { opMode } {
    global gvar cv
    global rtapenv ccdname ccdlenv
    global ccdOpMode
    global NOP ERR RUN WRN OK

    putsnl ""
    putsnl "Read FITS logs mask"
    putsnl "-------------------"
    if {[info exists cv]} { $cv.ccdLogMaskWS  configure -color Black }
    if {[info exists cv]} { $cv.ccdLogMaskLCU configure -color Black }
    set gvar(ccdVerLogMaskLCU) "-"
    if {[ccdDbRead "@$rtapenv<alias>$ccdname.logMask" logMaskWS] != $OK} { 
	set gvar(ccdVerLogMaskWS) "ERROR"
	if {[info exists cv]} { $cv.ccdLogMaskWS configure -color Red }
	return $ERR
    }
    if {[string length $logMaskWS] == 0} {
	set logMaskWS "NOT SET"
	if {[info exists cv]} { $cv.ccdLogMaskWS  configure -color Orange }
    }
    set gvar(ccdVerLogMaskWS) $logMaskWS

    if {[ccdDbRead "@$ccdlenv<alias>$ccdname.logMask" logMaskLCU] != $OK} { 
	set gvar(ccdVerLogMaskLCU) "ERROR"
	if {[info exists cv]} { $cv.ccdLogMaskLCU configure -color Red }
	return $ERR
    }
    if {[string length $logMaskLCU] != 0} {
	putsnl "FITS log mask is set to $logMaskLCU"
    } else {
	set logMaskLCU "NOT SET"
    } 
    set gvar(ccdVerLogMaskLCU) $logMaskLCU

    if { $logMaskLCU != $logMaskWS } {
	ccdVerifyWarning "" \
		"The FITS log masks are different" \
		"logMaskWS=$logMaskWS - logMaskLCU=$logMaskLCU" CCDLENV
	if {[info exists cv]} { $cv.ccdLogMaskWS  configure -color Red }
	if {[info exists cv]} { $cv.ccdLogMaskLCU configure -color Red }
    } elseif { $logMaskLCU == "NOT SET" } {
	ccdVerifyWarning "" "FITS log mask is not set" "" CCDLENV
	if {[info exists cv]} { $cv.ccdLogMaskWS  configure -color Orange }
	if {[info exists cv]} { $cv.ccdLogMaskLCU configure -color Orange }
    }
    return $OK
}

#
# procedure to check the CCD system in the given opMode <mode>
# - startup
# - STANDBY INIT ONLINE
# - SETUP START WAIT EXIT
#
proc ccdVerifyEXP { { mode "LCU_SIM" } } {
    global gvar
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global display 
    global ccdOpMode
    global NOP ERR RUN WRN OK

    set opMode $ccdOpMode($mode)
    putsnl "-> Start a session in $mode operational mode"
    if {[ccdDbWrite "@$rtapenv<alias>$ccdname.opMode" $opMode] != $OK} {
	return $ERR
    }

    putsnl "Startup the CCD system"
    putsnl "----------------------"
    
    putsnonl "ccdDcsDbSave.sh ... "
    set x [catch {set y [exec ccdDcsDbSave.sh]} ret]
    putsnl "done"
    putsnonl "ccdDcsStart.sh ... "
    set x [catch {set y [exec ccdDcsStart.sh >/dev/null]} ret]
    putsnl "done"
    if {$opMode == $ccdOpMode(LCU_SIM)} {
	set waitStatus  0
	putsnonl "scanDevOff $ccdlenv ... "
	set x [catch {set y [exec scanDevOff $ccdlenv]} ret]
	putsnl "done"
    } else {
	set waitStatus 128
    }

    set status $OK
    set env $rtapenv
    set prc "ccdconCI_$ccdname"
    set cmdList [list STANDBY INIT ONLINE SETUP START WAIT EXIT]
    foreach cmd $cmdList {
	if {($status == $ERR) && ($cmd != "EXIT")} { continue }

	set par ""
	if { $cmd == "SETUP" } {
	    if {![info exists display]} { set d -1 } else { set d $display }
	    set par "-file ccdSetupComplete.det -function DET.DISPLAY $d"
	} 
	if { $cmd == "START" } {
	    putsnl "Take one single exposure using the default setup"
	    putsnl "------------------------------------------------"
	}

	if {[ccdSendCommandAndRecvReply $env $prc $cmd rep $par] != $OK} { 
	    set status $ERR
	    break
	}
	putsnl "  > $rep"
	putsnl ""
	if {$opMode == $ccdOpMode(NORMAL) && $cmd == "ONLINE"} {
	    set tmpStatus [ccdVerifyTMP]
	    if {$tmpStatus != $OK} { set status $tmpStatus }
	    set telStatus [ccdVerifyTEL]
	    if {$telStatus != $OK} { set status $tmpStatus }
	}
	if { $cmd == "START" } {
	    if { [expr {$rep % 10}] != 1 } {
		putsnl "Exposure Id = $rep - Expected 1"
		putsnl "-> A failure occured - Check logs"
		set status $ERR
	    }
	}
	if { $cmd == "WAIT" && $rep != $waitStatus } {
	    putsnl "Exposure status = $rep - Expected $waitStatus"
	    putsnl "-> A failure occured - Check logs"
	    set status $ERR
	}
    }
    set ccdVerSession $status

    putsnl ""
    if {$opMode == $ccdOpMode(LCU_SIM)} {
	putsnonl "scanDevOn $ccdlenv ... "
	set x [catch {set y [exec scanDevOn $ccdlenv]} ret]
	putsnl "done"
    }

    if {$opMode == $ccdOpMode(ACE_SIM)} { 
	set logMaskStatus [ccdVerifyLogMask $opMode]
	if {$logMaskStatus != $OK } { 
	    set ccdVerSession $logMaskStatus
	} 
    }

    return $ccdVerSession
}

proc ccdVerifySetStatus { var val } {
    global gvar
    global NOP ERR RUN WRN OK
    global ccdVerifyStatus

    ccdVerifyUpdate $var $val
    if { $val != $OK } { set ccdVerifyStatus $val }
}
    
#
# main procedure to check the CCD system
#
proc ccdVerifyCCD { {fd 0} } {
    global gvar env
    global seq_ccsCmdCheck seq_ccsEnvName
    
    global host inshost
    global vltroot introot insroot insuser vltdata 
    global rtapenv ccdlenv ccdname
    global cfgdir detdir imgdir envdir
    global display
    global ccdOpState ccdOpMode
    global NOP ERR RUN WRN OK
    global ccdVerifyStatus

    catch [seq_ccsExit]

    set gvar(ccdVerLogFd) $fd

    ccdVerifyPreset

    set seq_ccsCmdCheck CHECK_ALL
    
    set ccdVerifyStatus $NOP

    putsnl ""
    putsnl "================================"
    putsnl " CCD Environment Verification   "
    putsnl "================================"
    
    if { ($gvar(ccdVerDoWsEnv)   == 1) || \
	 ($gvar(ccdVerDoCcdEnv)  == 1) || \
	 ($gvar(ccdVerDoRtapEnv) == 1) } {    
        ccdVerifyUpdate ccdVerWS $RUN
        ccdVerifySetStatus ccdVerWS [ccdVerifyWS]
        if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    } elseif { [ccdVerifyWS] == $ERR } { 
	seq_ccsExit
	return $ERR
    }
	
    if {$gvar(ccdVerDoLcuSim) == 1 } {
	ccdVerifyUpdate ccdVerSessionLCU_SIM $RUN
	ccdVerifySetStatus ccdVerSessionLCU_SIM [ccdVerifyEXP LCU_SIM]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }
    
    if { ($gvar(ccdVerDoLcuEnv)  == 1) || ($gvar(ccdVerDoScanSys) == 1) } {
	ccdVerifyUpdate ccdVerLCU $RUN
	ccdVerifySetStatus ccdVerLCU [ccdVerifyLCU]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }

    if {$gvar(ccdVerDoAceSim) == 1 } {
	ccdVerifyUpdate ccdVerSessionACE_SIM $RUN
	ccdVerifySetStatus ccdVerSessionACE_SIM [ccdVerifyEXP ACE_SIM]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }

    if {$gvar(ccdVerDoAceEnv) == 1 } {
	ccdVerifyUpdate ccdVerACE $RUN
	ccdVerifySetStatus ccdVerACE [ccdVerifyACE]
	catch [exec ccdDcsStop.sh >/dev/null]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }

    if {$gvar(ccdVerDoHwSim) == 1 } {
	ccdVerifyUpdate ccdVerSessionHW_SIM $RUN
	ccdVerifySetStatus ccdVerSessionHW_SIM [ccdVerifyEXP HW_SIM]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }
    
    if {$ccdVerifyStatus != $NOP} {
	putsnl ""
	putsnl ""
	putsnl "=========================="
	putsnl "CCD Control System is OK"
	putsnl "=========================="
	putsnl ""
	putsnl ""
    }

    if {$gvar(ccdVerDoNoSim) == 1 } {
	ccdVerifyUpdate ccdVerSessionNORMAL $RUN
	ccdVerifySetStatus ccdVerSessionNORMAL [ccdVerifyEXP NORMAL]
	if {$ccdVerifyStatus == $ERR} { 
	    seq_ccsExit
	    return $ERR
	}
    }

    putsnl ""
    putsnl ""
    putsnl "======================================="
    putsnl "All checks of CCD Control System are OK"
    putsnl "======================================="
    putsnl ""
    
    if {($ccdVerifyStatus == $OK) || ($ccdVerifyStatus == $WRN)} { 
	ccdVerifyUpdate ccdVerMessage "CCD Control System is ready to use ... "
	putsnl $gvar(ccdVerMessage)
    }

    seq_ccsExit
    return 0
}

#
# ___oOo___
#
