#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccsMonInstall.tcl,v 0.31 2005/10/14 10:40:03 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin  2005-08-10  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ccsMonGetFileFromEso { name } {
    global gvar VLTSWR VLTROOT INTROOT

    if { $VLTSWR == "" } {
        ccsMonMessage message ERROR "Environment Variable VLTSW_RELEASE undefined !"
        return ""
    }

    # get reference file
    set orgDir [pwd]
    set destDir [format "/tmp/%s" [pid]]
    exec rm -rf $destDir

    if {[catch {exec mkdir $destDir} msg]} {
        ccsMonMessage message ERROR $msg
        return ""
    }

    cd $destDir

    set ftpCon  [format "user %s \\nbinary\\ncd %s \\nget %s\\n" $gvar(userCmd) $gvar(ftpPath) $name]

    if {[catch {exec echo "$ftpCon" | ftp -n -i $gvar(ftpDest) } msg] }  {
        ccsMonMessage message ERROR $msg
        cd $orgDir
        return ""
    }

    cd $orgDir

    set source [format "%s/%s" $destDir $name]

    if { [file exists $source] == 0 } {
        ccsMonMessage message ERROR "failed to transfer reference file ($source) from ESO"
        exec rm -rf $destDir
        return ""     
    }

    return $destDir
}

proc ccsMonSetupVer { } {
    global gvar VLTSWR
    # window for output of version check

    toplevel .ccsMonVer 
    set gvar(version) .ccsMonVer

    wm protocol $gvar(version) WM_DELETE_WINDOW "destroy $gvar(version)"
    wm title    $gvar(version) [format "CCS Monitor System - Version Check (%s)" $VLTSWR]

    # Define frame where the menu bar will be created
    set topMenu [frame $gvar(version).menuBar]
    pack $topMenu -fill x

    set menuBar [ makeMenuFrame $topMenu ]

    #  File Menu
    
    set m [addMenu $menuBar  File  0];

    $m add command -label "Save" -underline 0 -accel Ctrl-q -command ccsMonVerSave
    $m add separator
    $m add command -label "Quit" -underline 0 -accel Ctrl-q -command {destroy $gvar(version)}

    # Define frame where all the other widget are placed

    set wfr1 [frame $gvar(version).fr1]
    set wfr2 [frame $gvar(version).fr2]

    pack $wfr1 -fill both -expand 1
    pack $wfr2 -fill x

    set gvar(verTbl)  $wfr1.tbl
    set gvar(verRes)  ""

    set vsb [scrollbar $wfr1.vsb -orient vertical   -command [list $gvar(verTbl) yview] ]
    set hsb [scrollbar $wfr2.hsb -orient horizontal -command [list $gvar(verTbl) xview] ]

    pack $hsb  -fill x  -anchor w

    tablelist::tablelist $gvar(verTbl) \
	-columns {0 "Module" 0 "needed Version" 0 "installed in $VLTROOT" 0 "installed in $INTROOT" } \
	-labelcommand tablelist::sortByColumn -background grey96 -selectbackground navy     \
	-stripebg {#e0e8f0} -yscrollcommand [list $vsb set] -xscrollcommand [list $hsb set] \
	-selectforeground white -height 10 -width 80 -setgrid yes -stretch all             \
	-listvariable gvar(verRes) -fg black -labelfont "-Adobe-courier-bold-r-normal--14*"

    $gvar(verTbl) configure -showseparators yes

    foreach col {0 2} {
        $gvar(verTbl) columnconfigure $col -background {#e0e8f0}
    }

    $gvar(verTbl) columnconfigure 0 -font "-Adobe-courier-bold-r-normal--14*"
    $gvar(verTbl) columnconfigure 1 -font "-Adobe-courier-medium-r-normal--14*"

    pack $gvar(verTbl)  -side left  -fill both -anchor w -expand yes
    pack $vsb   -side right -fill y

    return
}

proc ccsMonCheckCoreVltsw { } {
    global gvar VLTSWR VLTROOT INTROOT MODROOT

    set destDir [ccsMonGetFileFromEso buildFromArchive]

    if { [cequal $destDir ""] } {
        return
    }

    ccsMonSetupVer

    if {[file exists $VLTROOT/bin/ccsMonGetModules.awk]} {
        set awkFile [format "%s/bin/ccsMonGetModules.awk" $VLTROOT]
    }
    if {[file exists $INTROOT/bin/ccsMonGetModules.awk]} {
        set awkFile [format "%s/bin/ccsMonGetModules.awk" $INTROOT]
    }
    if {[file exists $MODROOT/bin/ccsMonGetModules.awk]} {
        set awkFile [format "%s/bin/ccsMonGetModules.awk" $MODROOT]
    }

    set source [format "%s/%s" $destDir buildFromArchive]

    set fileId [open "| cat $source | gawk -f $awkFile" RDONLY]

    while {[gets $fileId line] >= 0} {
        set num [llength $line]
        if { $num == 3 } {
            scan $line "%s %s %s" mod ver spec
            set refV($mod) $ver
            set refS($mod) $spec
        } elseif { $num == 2 } {
            scan $line "%s %s" mod ver
            set refV($mod) $ver
            set refS($mod) "Equal"
        }
    }

    close $fileId
    exec rm -rf $destDir

    ccsMonDoCheck refV refS
    return
}

proc ccsMonCheckRefFile { } {
    global gvar INTROOT VLTROOT

    if { [cequal $INTROOT ""] } {
        set dirStart $VLTROOT
    } else {
        set dirStart $INTROOT
    }

    set type {{{Version Files} {.versions}}}
    set fileName [tk_getOpenFile -filetypes $type]
    if { "$fileName" == "" } {
        return
    }

    if {[catch {open $fileName r} fileId]} {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }

    while {[gets $fileId line] >= 0} {
        set num [llength $line]
        if { $num == 3 } {
            scan $line "%s %s %s" mod ver spec
            set refV($mod) $ver
            set refS($mod) $spec
        } elseif { $num == 2 } {
            scan $line "%s %s" mod ver
            set refV($mod) $ver
            set refS($mod) "Equal"
        }
    }
    close $fileId

    ccsMonSetupVer

    ccsMonDoCheck refV refS
    return
}


proc ccsMonVerSave { } {
    global gvar

    set type {{{Log Files} {.log}}}

    set fileName [tk_getSaveFile -filetypes $type]
    if { "$fileName" == "" } {
        return
    }

    if { [catch {open $fileName w} fileId] } {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }
    set len [llength $gvar(verRes)]
    for {set i 0} {$i<$len} {incr i} {
        set tmp [lindex $gvar(verRes) $i]
        puts $fileId [format "%s\t\t%s\t\t%s\t\t%s" [lindex $tmp 0] [lindex $tmp 1]\
                                                    [lindex $tmp 2] [lindex $tmp 3]]
    }
    close $fileId
}


proc ccsMonDoCheck { refV refS } {
    global gvar VLTSWR VLTROOT INTROOT
    upvar $refV refVer
    upvar $refS refSpec
    
    # read log file of vltMakeInstall.config
    set instFile [format "%s/%s" $VLTROOT vltMakeInstall.config]
    if { [catch {open $instFile r} fileId] } {
        ccsMonMessage message ERROR [format "failed to open file vltMakeInstall.config: %s" $fileId]
        return
    }
    
    while {[gets $fileId line] >= 0} {
        set num [llength $line]
        set line [string trim $line]
        if { [string index $line 0] != "#" } {
            if { $num > 5 } {
                scan $line "%s %s" mod ver 
                set instVlt($mod) $ver
                set instVltUser($mod) [lindex $line 7]
                set instVltTime($mod) [lindex $line 8]
            } elseif { $num > 3 } {
                scan $line "%s" mod
                set instVlt($mod) "UNKNOWN"
                set instVltUser($mod) [lindex $line 3]
                set instVltTime($mod) [lindex $line 4]
	    }
        }
    }
    close $fileId

    if { [cequal $INTROOT ""] == 0 } {
        set instFile [format "%s/%s" $INTROOT vltMakeInstall.config]

        if { [catch {open $instFile r} fileId] == 0} {

            while {[gets $fileId line] >= 0} {
                set num [llength $line]
                set line [string trim $line]
                if { [string index $line 0] != "#" } {
                    if { $num > 5 } {
                        scan $line "%s %s" mod ver
                        set instInt($mod) $ver
                        set instIntUser($mod) [lindex $line 7]
                        set instIntTime($mod) [lindex $line 8]
                    } elseif { $num > 3 } {
                        scan $line "%s" mod
                        set instInt($mod) "UNKNOWN"
                        set instIntUser($mod) [lindex $line 3]
                        set instIntTime($mod) [lindex $line 4]
                    }
		}
            }
            close $fileId
        }
    }

    set vltCol ""
    set intCol ""
    foreach index [array names refVer] {
        set modules ""
        if {[info exists instVlt($index)]} {
            lappend modules $index
        } elseif {[info exists instInt($index)]} {
            lappend modules $index
        }

        if {[info exists instVlt($index-ws)]} {
            lappend modules $index-ws
        } elseif {[info exists instInt($index-ws)]} {
            lappend modules $index-ws
        }

        if {[info exists instVlt($index-lcu)]} {
            lappend modules $index-lcu
        } elseif {[info exists instInt($index-lcu)]} {
            lappend modules $index-lcu
        }

        set cnt [llength $modules]
        if { $cnt == 0 } {
            lappend modules $index
            set cnt 1
        }

        for {set i 0} {$i<$cnt} {incr i} {
            set mod [lindex $modules $i]

            if {[info exists instVlt($mod)]} {
                set version     [split $instVlt($mod) .]
                set instMod     $instVlt($mod)
                set instModUser $instVltUser($mod)
                set instModTime $instVltTime($mod)
	    } else {
                set version     ""
                set instMod     ""
                set instModUser ""
                set instModTime ""
	    }

            if {[info exists instInt($mod)]} {
                set intVersion [split $instInt($mod) .]
                set intMod     $instInt($mod)
                set intModUser $instIntUser($mod)
                set intModTime $instIntTime($mod)
	    } else {
                set intVersion ""
                set intMod     ""
                set intModUser ""
                set intModTime ""
	    }

            set refNum [split $refVer($index) .]

            if { [cequal $intVersion ""] == 0 } {
                switch -exact -- $refSpec($index) {
                    Equal        {set result [ccsMonE  $intVersion $refNum]}
                    Greater      {set result [ccsMonG  $intVersion $refNum]}
                    GreaterEqual {set result [ccsMonGE $intVersion $refNum]}
	            LessEqual    {set result [ccsMonLE $intVersion $refNum]}
                    Less         {set result [ccsMonL  $intVersion $refNum]}
                    default      {set result 0}
                    }

                if { $result == 0 } {
                    if { [ccsMonG  $intVersion $refNum] == 0 || "$intVersion" == "UNKNOWN" } {
                        lappend intCol "red"
                    } else {
                        lappend intCol "blue"
		    }
                } else {
#                    lappend intCol {#6b8e23}
                    lappend intCol green
                }
	    } else {
                set intMod     "-----"
                lappend intCol "green"
            }

            set verStr "UNKNOWN"
            switch -exact -- $refSpec($index) {
                Equal        {set verStr [format "   %s" $refVer($index)]}
                Greater      {set verStr [format ">  %s" $refVer($index)]}
                GreaterEqual {set verStr [format ">= %s" $refVer($index)]}
	        LessEqual    {set verStr [format "<= %s" $refVer($index)]}
                Less         {set verStr [format "<  %s" $refVer($index)]}
                default      {set verStr "UNKNOWN"}
            }

            if { [cequal $version ""] == 0 } {
                switch -exact -- $refSpec($index) {
                    Equal        {set result [ccsMonE  $version $refNum]}
                    Greater      {set result [ccsMonG  $version $refNum]}
                    GreaterEqual {set result [ccsMonGE $version $refNum]}
	            LessEqual    {set result [ccsMonLE $version $refNum]}
                    Less         {set result [ccsMonL  $version $refNum]}
                    default      {set result 0}
                    }

                if { $result == 0 } {
                    lappend vltCol "red"
                } else {
#                    lappend vltCol {#6b8e23}
                    lappend vltCol green
                }

	        if { "$intModUser" == "" } {
                    lappend gvar(verRes) [format "%s {%s} {%s by %s on %s} {%s}" \
                      $mod $verStr $instMod $instModUser $instModTime $intMod]
                } else {
                    lappend gvar(verRes) [format "%s {%s} {%s by %s on %s} {%s by %s on %s}" \
                      $mod $verStr $instMod $instModUser $instModTime $intMod $intModUser $intModTime]
	        }

	    } else {
                lappend vltCol "red"
                if { [cequal $intModUser ""] } {
                    lappend gvar(verRes) [format "%s {%s} %s {%s}" $mod $verStr {"-----"} $intMod]
                } else {
                    lappend gvar(verRes) [format "%s {%s} %s {%s by %s on %s}" \
                                          $mod $verStr {"-----"} $intMod $intModUser $intModTime]
	        }
	    }
	}
    }

    set len [llength $gvar(verRes)]
    for {set i 0} {$i<$len} {incr i} {
	$gvar(verTbl) cellconfigure $i,2 -fg [lindex $vltCol $i]
 	$gvar(verTbl) cellconfigure $i,3 -fg [lindex $intCol $i]
    }

    return
}


proc ccsMonE { ver ref } {
    if { "$ver" != "$ref" } {
        return 0
    }
    return 1
}


proc ccsMonG { ver ref } {
    set verLen [llength $ver]
    set refLen [llength $ref]
    for {set i 0} {$i<$refLen} {incr i} {
        if { $i > $verLen } {
            return 1
        }
        set refVal [lindex $ref $i]
        set verVal [lindex $ver $i]
        if { $verVal < $refVal } {
            return 0
	} elseif { $verVal > $refVal } {
            return 1
        } 
    }
    if { $verLen > $refLen } {
        return 1
    }
    return 0
}


proc ccsMonGE { ver ref } {
    set verLen [llength $ver]
    set refLen [llength $ref]
    for {set i 0} {$i<$refLen} {incr i} {
        if { $i > $verLen } {
            return 1
        }
        set refVal [lindex $ref $i]
        set verVal [lindex $ver $i]
        if { $verVal < $refVal } {
            return 0
	} elseif { $verVal > $refVal } {
            return 1
        } 
    }
    if { $verLen > $refLen } {
        return 1
    }
    return 1
}


proc ccsMonL { ver ref } {
    set verLen [llength $ver]
    set refLen [llength $ref]
    for {set i 0} {$i<$refLen} {incr i} {
        if { $i > $verLen } {
            return 0
        }
        set refVal [lindex $ref $i]
        set verVal [lindex $ver $i]
        if { $verVal > $refVal } {
            return 0
	} elseif { $verVal < $refVal } {
            return 1
        } 
    }
    if { $verLen > $refLen } {
        return 0
    }
    return 0
}


proc ccsMonLE { ver ref } {
    set verLen [llength $ver]
    set refLen [llength $ref]
    for {set i 0} {$i<$refLen} {incr i} {
        if { $i > $verLen } {
            return 0
        }
        set refVal [lindex $ref $i]
        set verVal [lindex $ver $i]
        if { $verVal > $refVal } {
            return 0
	} elseif { $verVal < $refVal } {
            return 1
        } 
    }
    if { $verLen > $refLen } {
        return 0
    }
    return 1
}

proc ccsMonPatchSave { } {
    global gvar

    set type {{{Log Files} {.log}}}

    set fileName [tk_getSaveFile -filetypes $type]
    if { "$fileName" == "" } {
        return
    }

    if { [catch {open $fileName w} fileId] } {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }
    set len [llength $gvar(patRes)]
    for {set i 0} {$i<$len} {incr i} {
        set tmp [lindex $gvar(patRes) $i]
        puts $fileId [format "%s\t\t%s\t\t%s" [lindex $tmp 0] [lindex $tmp 1] [lindex $tmp 2]]
    }
    close $fileId
}

proc ccsMonSetupPatch { } {
    global gvar VLTSWR
    # window for output of version check

    toplevel .ccsMonPatch 
    set gvar(patch) .ccsMonPatch

    wm protocol $gvar(patch) WM_DELETE_WINDOW "destroy $gvar(patch)"
    wm title    $gvar(patch) [format "CCS Monitor System - Patch Level Check (%s)" $VLTSWR]

    # Define frame where the menu bar will be created
    set topMenu [frame $gvar(patch).menuBar]
    pack $topMenu -fill x

    set menuBar [ makeMenuFrame $topMenu ]

    #  File Menu
    
    set m [addMenu $menuBar  File  0];

    $m add command -label "Save" -underline 0 -accel Ctrl-q -command ccsMonPatchSave
    $m add separator
    $m add command -label "Quit" -underline 0 -accel Ctrl-q -command {destroy $gvar(patch)}

    # Define frame where all the other widget are placed

    set wfr1 [frame $gvar(patch).fr1]
    set wfr2 [frame $gvar(patch).fr2]

    pack $wfr1 -fill both -expand 1
    pack $wfr2 -fill x

    set gvar(patTbl)  $wfr1.tbl
    set gvar(patRes)  ""

    set vsb [scrollbar $wfr1.vsb -orient vertical   -command [list $gvar(patTbl) yview] ]
    set hsb [scrollbar $wfr2.hsb -orient horizontal -command [list $gvar(patTbl) xview] ]

    pack $hsb  -fill x  -anchor w

    tablelist::tablelist $gvar(patTbl) \
	-columns {0 "File-Type" 0 "File" 0 "Patch Level"} \
	-labelcommand tablelist::sortByColumn -background grey96 -selectbackground navy     \
	-stripebg {#e0e8f0} -yscrollcommand [list $vsb set] -xscrollcommand [list $hsb set] \
	-selectforeground white -height 10 -width 80 -setgrid yes -stretch all             \
	-listvariable gvar(patRes) -fg black -labelfont "-Adobe-courier-bold-r-normal--14*"

    $gvar(patTbl) configure -showseparators yes

    foreach col {0 2} {
        $gvar(patTbl) columnconfigure $col -background {#e0e8f0}
    }

    $gvar(patTbl) columnconfigure 0 -font "-Adobe-courier-bold-r-normal--14*"
    $gvar(patTbl) columnconfigure 1 -font "-Adobe-courier-medium-r-normal--14*"

    pack $gvar(patTbl)  -side left  -fill both -anchor w -expand yes
    pack $vsb   -side right -fill y

    return
}

proc ccsMonCheckPatches { } {
    global gvar VLTSWR VLTROOT INTROOT

    ccsMonSetupPatch

    set source [format "%s/%s" $VLTROOT vltpatch.log]

    set fileId [open $source RDONLY]

    while {[gets $fileId line] >= 0} {
         set num [llength $line]
         if { $num == 3 } {
             scan $line "%s %s %s" type fil lev
             if { ![cequal $type "module"] } {
                 lappend gvar(patRes) [format "{%s} {%s} {%s}" \
                          $type $fil $lev]
             }
         }
     }

    close $fileId

    return
}


#
# ___oOo___
