#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccsMonVerProcs.tcl,v 0.31 2005/10/14 10:40:02 vltsccm Exp $" 
#


proc ccsMonOpen { } {
    global gvar

    set type {{{Version Files} {.versions}}}
    set fileName [tk_getOpenFile -filetypes $type]

    if { [cequal $fileName ""] } {
        return
    }

    if {[catch {open $fileName r} fileId]} {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }

    set gvar(ccsMonData) ""
    while {[gets $fileId line] >= 0} {
        set num [llength $line]
        if { $num == 3 } {
            scan $line "%s %s %s" mod ver spec
            lappend gvar(ccsMonData) [format "%s %s %s" $mod $ver $spec]
        } elseif { $num == 2 } {
            scan $line "%s %s" mod ver
            lappend gvar(ccsMonData) [format "%s %s %s" $mod $ver "Equal"]
        }
    }
    close $fileId
 
    set gvar(file) $fileName
    $gvar(men)  entryconfigure 3 -state active
    $gvar(inst) entryconfigure 1 -state active
    $gvar(inst) entryconfigure 2 -state active
    return
}

#-------------------------------------------------------------

proc ccsMonDelete { } {
    global gvar
    $gvar(list).tbl delete [$gvar(list).tbl curselection]
}


#-------------------------------------------------------------

proc ccsMonDrawLine { cv x0 y0 h txtLength textVal } {

    set w [expr {$txtLength + 10}]

    $cv create line [expr {$x0 + 10}] $y0 \
	$x0 $y0 \
	$x0 [expr {$y0 + $h}] \
	[expr {$x0 + $w}] [expr {$y0 + $h}]\
	[expr {$x0 + $w}] $y0 \
	[expr {$x0 + $txtLength}] $y0 \
	-fill Blue -width 1

    $cv create text [expr {$x0 + 20}]  [expr {$y0 + 5}] -anchor w -text $textVal \
              -fill black -font -Adobe-helvetica-bold-r-normal--14* 
    return
}


#-------------------------------------------------------------

proc ccsMonInstallInt { } {
    global gvar INTROOT VLTROOT

    if { "$INTROOT" == "" } {
        ccsMonMessage message ERROR "INTROOT undefined"
    }

    if { "$gvar(file)" == "" } {
        ccsMonMessage message ERROR "no file defined"
    }

    if { [catch [file copy $gvar(file) "$INTROOT/config"] msg]}  {
        ccsMonMessage message ERROR $msg
    } else {
        ccsMonMessage message INFO "file successfully installed in INTROOT"
    }
    return
}

#-------------------------------------------------------------

proc ccsMonInstallVlt { } {
    global gvar INTROOT VLTROOT

    if { "$VLTROOT" == "" } {
        ccsMonMessage message ERROR "VLTROOT undefined"
    }

    if { "$gvar(file)" == "" } {
        ccsMonMessage message ERROR "no file defined"
    }

    if {[catch [file copy $gvar(file) "$VLTROOT/config"] msg]} {
        ccsMonMessage message ERROR $msg
    } else {
        ccsMonMessage message INFO "file successfully installed in VLTROOT"
    }
    return
}

#-------------------------------------------------------------

proc ccsMonVerify { } {
    global gvar

    if {[catch {exec cmmCheckVersion $gvar(module) $gvar(version)} msg]}  {
        ccsMonMessage message ERROR [format "version %s of module %s is not existing" $gvar(version) $gvar(module)]
    } else {
        ccsMonMessage message INFO [format "version %s of module %s is existing" $gvar(version) $gvar(module)]
    }
    return
}


#-------------------------------------------------------------

proc ccsMonSave { } {
    global gvar env

    set type {{{Version Files} {.versions}}}

    set len [string length $gvar(file)]
    set index [string last "/" $gvar(file)]
    if { $index == -1 } {
        set dir  $env(PWD)
        set file $gvar(file)
    } else {
        set dir  [string range $gvar(file) 0 $index]
        set file [string range $gvar(file) [incr index] $len]
    }
    set fileName [tk_getSaveFile -filetypes $type -initialfile $file -initialdir $dir]
    if { "$fileName" == "" } {
        return
    }
    set gvar(file) $fileName

    if {[catch {open $fileName w} fileId]} {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }
    set len [llength $gvar(ccsMonData)]
    for {set i 0} {$i<$len} {incr i} {
        puts $fileId [lindex $gvar(ccsMonData) $i]
    }
    close $fileId

    return
}

#-------------------------------------------------------------

proc ccsMonSaveAs { } {
    global gvar

    set type {{{Version Files} {.versions}}}
    set fileName [tk_getSaveFile -filetypes $type]

    if { "$fileName" == "" } {
        return
    }

    if {[catch {open $fileName w} fileId]} {
        ccsMonMessage message ERROR [format "failed to open file: %s" $fileId]
        return
    }
    set len [llength $gvar(ccsMonData)]
    for {set i 0} {$i<$len} {incr i} {
        puts $fileId [lindex $gvar(ccsMonData) $i]
    }
    close $fileId

    set gvar(file) $fileName
    $gvar(men)  entryconfigure 3 -state active
    $gvar(inst) entryconfigure 1 -state active
    $gvar(inst) entryconfigure 2 -state active
    return
}

#-------------------------------------------------------------

proc ccsMonAdd { } {
    global gvar

    set gvar(module) [string trim "$gvar(module)"]
    if { "$gvar(module)" == "" } {
        ccsMonMessage message ERROR "Modul undefined"
        return
    }
    set gvar(version) [string trim "$gvar(version)"]
    if { "$gvar(version)" == "" } {
        ccsMonMessage message ERROR "Version undefined"
        return
    }
    set len [string length $gvar(version)]
    set dots 0
    for {set i 0} {$i <$len} {incr i} {
        set val [string index $gvar(version) $i]
        if { [string is digit $val] || $val == "." } {
            continue
        }
        ccsMonMessage message ERROR "invalid version string"
        return
    }

    set len [llength $gvar(ccsMonData)]
    for {set i 0} {$i<$len} {incr i} {
	set entry  [lindex $gvar(ccsMonData) $i]
        set module [lindex $entry 0]
        if { "$module" == "$gvar(module)" } {
            ccsMonMessage message ERROR [format "module <%s> already in list" $module]
            return
        }
    }
    set rule [$gvar(rule) get]
    lappend gvar(ccsMonData) [format "%s %s %s" $gvar(module) $gvar(version) $rule]

    return
}

#-------------------------------------------------------------

proc ccsMonTblSel { cell } {
    global gvar

    set lis [split $cell ", "]
    set row [lindex $lis 0]
    set col [lindex $lis 1]
    if { $row == -1 } {
        return
    }
    set data [$gvar(tbl) rowcget $row -text]

    set gvar(module)  [lindex $data 0]
    set gvar(version) [lindex $data 1]
    $gvar(rule) select [lindex $data 2]
}

