#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccsei.tcl,v 3.11 2005/09/14 14:11:14 vltsccm Exp $" 
#
# ccsei.tcl   -   Common part of the CCS Engineering User Interface
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 02.01.1995 Created
#	     13.01.1995 Moved extensive parts to libraries
#	     22.01.1995 Added support for synonyms in ccseiMsgSendCommand
#	     25.01.1995 Moved parts to ccseiLib.tcl
#
#************************************************************************
#   NAME
#	ccsei - The CCS Engineering User Interface
#   SYNOPSIS
#	ccsei &
#   DESCRIPTION
#	The CCS Engineering User Interface is a framework of several
#	applications. It consists of:
#
#	- the Starter-Panel (ccseiStarter, or usually invoked with ccsei)
#	- the command interface to the message-system (ccseiMsg)
#	- the Database-Monitor (ccseiDb) to display and change DB value
#   FILES
#	The following files are searched in the directories given by MODROOT, 
#	INTROOT, and then VLTROOT, wherever they are found first:
#
#	    config/ccseiStarter.config - configuration of the starter window
#	    config/ccsei.config - common environment configuration (optional)
#
#	The following file can be placed in the HOME directory and is
#	sourced during start-up after ccsei.config, when present:
#
#	    ~/.ccsei - secondary common environment configuration
#
#	Specific applications may require additional files.
#   ENVIRONMENT
#	All ccsei applications need at least the following shell-variables:
#
#       VLTROOT		Path to the global VLT area
#	INTROOT		Path to a public integration area (optional)
#	MODROOT		Path to a private module area (optional)
#
#	Specific applications may require additional shell-variables.
#   VARIABLES
#	seq_ccsEnvName	Name of the local environment on the host workstation
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	ccseiMsg(1), ccseiDb(1), ccsei.config(5), ccseiStarter.config(5)
#   BUGS
#
#   TO DO
#
#------------------------------------------------------------------------
#

###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules:
#	- All global variable-names start with the module-name (ccsei)
#
# Names ending with "_X" have a special meaning:
#	_c	Constant
#	_e	Derived from environment variable
#	_n	Names defined in configuration file
#	_v	Variable
#	_w	Widget name
#
# Abbreviations (as part of names):
#	Ses	Session
#	Env	Environment
#	Pro	Process
#	Com	Command
#	Par	Parameter
#
###############################################################################


proc ccseiKeyDefinitions {} {
   global ccsei_c 

   # Derive version from RCS
   set Id Dummy
   set myVersion "@(#) $Id: ccsei.tcl,v 3.11 2005/09/14 14:11:14 vltsccm Exp $"
   set ccsei_c(version) "[lindex $myVersion 3]"

   # Definition of electric keys: all bindings shall use these variables
   # to have a common behaviour. Adjust them here to your needs.
   # (Format must be as accepted by the Tcl `bind' command)

   set ccsei_c(keyEnter) <Return>	;# the big Enter
   set ccsei_c(keyCompl) <Tab>		;# completion key
   set ccsei_c(keyRevCompl) <Control-Tab> ;# the opposite ???

   #	<Shift-Tab> does not work, probably a bug in the keyboard-mapping!

   set ccsei_c(keyTerm)  <space>	;# used in the ccseiMsg command-box
   set ccsei_c(keyFocus) <Escape>	;# to move the focus between entries
   set ccsei_c(keyNuke)  <Delete>	;# to clear an entry
   set ccsei_c(keyHelp)  <F1>		;# not used yet
}

#_oOo_
