#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiComplLib.tcl,v 3.11 2005/09/14 14:11:14 vltsccm Exp $" 
#
# ccseiComplLib.tcl - Library for internal completion-management of ccsei
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.01.1995 Created
# S.Sandrock 09.10.1995 Obsolete
#

#************************************************************************
#   NAME
#	- OBSOLETE FILE! -
#	All functions in this file are no longer supported.
#	Use `vccCompletion' class instead.
#------------------------------------------------------------------------

#************************************************************************
#   NAME
#	ccseiComplLib,
#	ccseiCompletion - Tcl procedure for completion operations
#
#   SYNOPSIS
#	ccseiCompletion Operation Tag Items
#
#   DESCRIPTION
#	Completion of words based on search-lists.
#
#	This procedure provides a word completion mechanism that can behave
#	differently depending on a given tag, thereby enabling context-
#	dependent completion. Several modes - including completion on both
#	ends of a word - can be selected for each tag individually.
#	There may be any number of tags.
#
#   PARAMETERS
#	Operation - specifies one of the following operations:
#
#	  mode	Set completion mode for `Tag'. `Items' can be a combination of:
#		    enabled	- no operation for this Tag if not set
#		    ignorecase	- ignore case for completion
#		    atfirst	- complete at beginning of item
#		    atlast	- complete at end of item
#		Default is: { enabled ignorecase atfirst atlast }
#
#	  new	Create new list for `Tag' consisting of the word(s) `Items', 
#		overwrite if already existing
#
#	  add	Add word(s) `Items' to the list of `Tag'
#
#	  get	Return a unique completion for word `Items' from list of `Tag'
#
#	  all	Return all possible completions for word `Items' from list 
#		`Tag', return them as a list
#
#	  info	Return information on internal state as a string, 
#		`Items' and `Tag' ignored.
#
#	Tag	- selects one (of possibly many) search-lists
#
#	Items	- single search-item or list, depends on Operation, see there
#
#   RETURNS
#	depends on Operation, see there
#
#   GLOBAL VARIABLES
#	Exported: none
#	Imported: none
#	Private:
#		ccseiCompletionList($Tag) - array of search lists
#		ccseiCompletionMode($Tag) - array of completion modes
#
#   CAUTIONS
#
#   EXAMPLES
#	ccseiCompletion mode Env { enabled ignorecase atfirst atlast }
#	ccseiCompletion new Env $envlist
#	ccseiCompletion new Com $comlist
#	set completion [ccseiCompletion get Env $field]
#	set comlist [ccseiCompletion all Com $Command ]
#
#   SEE ALSO
#	ccsei(1),
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#   TO DO
#	- partial completion if there is no unique completion,
#	  what only makes sense for single-ended completion.
#
#------------------------------------------------------------------------


###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################

global ccseiCompletionList()	;# array of search-lists
global ccseiCompletionMode()	;# array of completion-mode settings



###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################


proc ccseiCompletion { Operation Tag Items args } {
    # args: intended only for internal use, for recursive/iterative calls

    global ccseiCompletionList ccseiCompletionMode

    # initialize mode settings, set to default if not already existing:
    if {! [info exists ccseiCompletionMode($Tag)]} {
	set ccseiCompletionMode($Tag) { enabled ignorecase atfirst atlast }
    }
    set m $ccseiCompletionMode($Tag)

    # first check if completion is at all enabled for this Tag:
    if {$Operation != "mode" && [lsearch $m enabled] == -1} { return "" }

    # set flags for easier use:
    set m_ignorecase [expr {([lsearch $m ignorecase] != -1) ? 1 : 0}]
    set m_atfirst    [expr {([lsearch $m atfirst   ] != -1) ? 1 : 0}]
    set m_atlast     [expr {([lsearch $m atlast    ] != -1) ? 1 : 0}]

    # create empty search-list if not already existing:
    if {! [info exists ccseiCompletionList($Tag)]} {
	set ccseiCompletionList($Tag) {}
    }

    switch $Operation {
	mode {
	    set ccseiCompletionMode($Tag) $Items
	    ccseiPuts "ccseiCompletion mode($Tag) = {$Items}"
	    return ""
	}
	new {
	    set ccseiCompletionList($Tag) {}
	    ccseiCompletion add $Tag $Items
	    return ""
	}
	add {
	    foreach item $Items {
		if {[lsearch $ccseiCompletionList($Tag) $item] == -1} {
		    lappend ccseiCompletionList($Tag) $item
		}
	    }
	    ccseiPuts "ccseiCompletion add {$Items} to list($Tag) \
		    ==> {$ccseiCompletionList($Tag)}"
	    return ""
	}
	get {
	    set completions [ccseiCompletion all $Tag $Items]
	    if {[llength $completions] > 1} {return ""}
	    return $completions
	}
	all {
	    set completions {}
	    set item [lindex $Items 0]
	    set pattern $item
	    if {$m_ignorecase} {set pattern [string tolower $pattern]}
	    if {$m_atfirst}    {set pattern "*$pattern"}
	    if {$m_atlast }    {set pattern "$pattern*"}
	    set clist $ccseiCompletionList($Tag)
	    set wlist $clist
	    if {$m_ignorecase} {set wlist [string tolower $wlist]}
	    set j 0 ;# start-pos of sub-list in original list
	    set i [lsearch -glob $wlist $pattern] ;# pos of item in sub-list
	    while {$i >= 0} {
		lappend completions [lindex $clist [expr {$j+$i}]]
		set wlist [lrange $wlist [incr i] end] ;# remaining sub-list
		incr j $i
		set i [lsearch -glob $wlist $pattern]
	    }
	    ccseiPuts "ccseiCompletion $Operation {$item} from list($Tag)\
		    ==> {$completions}"
	    return $completions
	}
	info {
	    set infostr "[array size ccseiCompletionList] completion lists:"
	    set names [array names ccseiCompletionList]
	    foreach name $names {
		set s "\t$name: [llength $ccseiCompletionList($name)] entries"
		set infostr "$infostr\n$s"
	    }
	    ccseiPuts $infostr
	    return $infostr
	}
	sole_unsupported {
	    set completions [ccseiCompletion all $Tag $Items]
	    if {[llength $completions] <= 1} {return $Items}
	    if {[llength $completions] > 1 && \
		    [lsearch $completions $Items] != -1} {return $Items}
	    return ""
	}
	default {
	    ccseiPuts "ccseiCompletion: invalid operation ($Operation)"
	}
    } ;#switch
}

#_oOo_
