
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiDbFileProcs.tcl,v 3.11 2005/09/14 14:11:18 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin  2004-11-11  created
#

#************************************************************************
#   NAME
#
#   SYNOPSIS
#
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#

proc ccseiDbFileLoadAttrList { {fileName ""} } {
    global ccseiDb_v cv gvar env ccseiDbVersion
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable    
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    if {[cequal $fileName ""]} {
	set fileName [panSelectFile . *.dbm]
    }

    if {[cequal $fileName ""]} {
	return
    }

    # open file to check the format by the first line:
    set fh [open $fileName r]

    ccseiGuiPuts "ccseiDbFileLoadAttrList: opening \"$fileName\" fh = $fh"

    switch -- [lindex [gets $fh] 0] {
	NODE -
	ENVIRONMENT -
	"#" {
	    # --> new format
	}
	default {
	    # --> old format
	    close $fh
	    return [ccseiDbFileLoadAttrList_OldFormat $fileName]
	}
    }
    seek $fh 0 ;# go back to the start of the file

    # process the file contents:
    set node ""
    set attList {}
    set valList {}

    while {[gets $fh line] >= 0} {
	set keyword [string trim [ctoken line :]]
	set value   [string trim [crange $line 1 end]]

	switch -- $keyword {
	    ENVIRONMENT	 {set ccseiDb_v(Env) $value}
	    NODE	 {set ccseiDb_v(Env) $value; set node @$value}
	    PERIOD	 {
		if {$value < 1 || $value > 60} {
		    set gvar(dbUpdatePeriod) 1
		} else {
		    set gvar(dbUpdatePeriod) $value
		}
	    }
	    PARAMETER    {lappend attList $node$value}
	    VALUE	 {lappend valList $value}
	}
    }

    close $fh

    set lenAtt [llength $attList]
    set lenVal [llength $valList]
    
    if {$lenAtt < $lenVal} {
	ccseiPuts "ccseiDbFileLoadAttrList: shorten ValList: $lenVal->$lenAtt"
    } elseif {$lenAtt > $lenVal} {
	ccseiPuts "ccseiDbFileLoadAttrList: extend ValList: $lenVal->$lenAtt"
	for {set i $lenVal} {$i < $lenAtt} {incr i} {
	    lappend valList UNKNOWN
	}
    }
    
    # Delete current content of the table
    $wdgMonTable delete 0 end 

    # Add attributes to table
    
    set recId 0
    foreach dbName $attList {
	set dbInfo [ccseiDbAttrInfo $dbName ""]
	lassign $dbInfo dbAttrType dbTypeList numRecs numFields
    
	set dbType [ccseiGetDataType $dbTypeList]

	# Add selected attribute to monitoring table
	$wdgMonTable insert end "${dbName} $dbAttrType $dbType OK [lindex $valList $recId]"
	incr recId
    }

    ccseiUpdateAttrList $wdgMonTable all 1
}

proc ccseiDbFileSaveAttrList { {fileName ""} } {
    global ccseiDb_v cv gvar env ccseiDbVersion
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    if {[cequal $fileName ""]} {
	set fileName [panSelectFile . *.dbm]
    }

    if {[cequal $fileName ""]} {
	return
    }

    ccseiPuts "ccseiDbFileSaveAttrList: opening \"$fileName\""

    set f [open $fileName w]

    puts $f "# File generated by ccseiDb $ccseiDbVersion - DO NOT EDIT"
    puts $f "ENVIRONMENT : $ccseiDb_v(Env)"
    puts $f "PERIOD : $gvar(dbUpdatePeriod)"

    set len [$wdgMonTable size]

    for {set recId 0} {$recId < $len} {incr recId} {

	set dbName  [$wdgMonTable cellcget ${recId},${colDbName}  -text]
	set dbValue [$wdgMonTable cellcget ${recId},${colDbValue} -text]

	puts $f "PARAMETER : $dbName"
	puts $f "    VALUE : $dbValue"
    }
    close $f
}


proc ccseiDbOpenFile {} {
    global ccseiDb_v gvar cv env dbfh wdgMonTable

    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    set gvar(dbMonMessage) ""
    
    if {![info exists dbfh]} { 
	set dbfh ""
    } else {
	if {![cequal $dbfh ""]} {
	    catch {close $dbfh }
	    set gvar(dbfh) ""
	}
    }
    
    if {[info exists env(VLTDATA)]} {
	set tmpDir $env(VLTDATA)/ENVIRONMENTS/$ccseiDb_v(Env)
    } else {
	if {[info exists env(HOME)]} {
	    set tmpDir $env(HOME) 
	} else {
	    set tmpDir "." ;# Current directory
	}
    }
    
    if {$gvar(dbAutoName)} {
	set gvar(ccseiDbFile) [fmtclock [clock seconds] "%Y-%m-%dT%T"].dblog
    } else {
	if {[cequal $gvar(ccseiDbFile) ""]} {
	    tk_dialog .d "Dialog" "Name of data file is an empty string" info 0 OK   
	    return
	}
    }
    
    set fname "${tmpDir}/$gvar(ccseiDbFile)"
    
    if {[file exists $fname]} {
	set tmp "File $gvar(ccseiDbFile) Already exists"
	
	if {[tk_dialog .d "Dialog" $tmp question 1 "Continue" "Abort Action"]} {
	    set gvar(enableRecord) 0
	    return
	}
    }
    
    if { [catch {set dbfh [open $fname "w+"] } ]} {
	tk_dialog .d "Dialog" "Cannot open file \n$fname" info 0 OK   
	return
    }
    
    set gvar(ccseiDbFile) $fname

    puts $dbfh "#"
    puts $dbfh "#=============================================="
    puts $dbfh "#List of sampled attributes :\n#"
    
    foreach item [$wdgMonTable getcolumn $colDbName] {
	puts $dbfh "#  $item"
    }
    
    puts $dbfh "#\n#=============================================="  
    puts $dbfh "# WS  start time: (GMT) [exec date -u +%c]"
    puts $dbfh "#                 (Loc) [exec date +%c]   "
    
    flush $dbfh
    
    set gvar(dbRecordOnFile) 1

    return $dbfh
}

proc ccseiDbMonitorOnFile {} {
    global gvar wdgMonTable dbfh

    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    if {$gvar(dbRecordOnFile) == 0 || [cequal $dbfh ""]} {return}

    set dbValues [fmtclock [clock seconds] "%T"]

    foreach value [$wdgMonTable getcolumn $colDbValue] {
	# The lrange is necessary to get rid of the BAD quality flag
	lappend dbValues $value
    }
    
    puts  $dbfh $dbValues
    flush $dbfh
}















#-------------------------------------------------------------------
#  Obsolete procedures 
#-------------------------------------------------------------------

# called by `ccseiDbFileLoadAttrList' to process old file format:

proc ccseiDbFileLoadAttrList_OldFormat { {fileName ""} } {
    global ccseiDb_v

    if {$fileName == ""} {set filename [panSelectFile . *]}
    if {$fileName != ""} {
	ccseiPuts "ccseiDbFileLoadAttrList_OldFormat: opening \"$fileName\""
	set f [open $fileName r]
	gets $f ccseiDb_v(AttList)
	gets $f ccseiDb_v(ValList)
	close $f
	ccseiDbUpdateLists
    }
}


# save old format is not used anymore:
proc ccseiDbFileSaveAttrList_OldFormat { {File ""} } {
    global ccseiDb_v
    set filename $File
    if {$filename == ""} {set filename [panSelectFile . *]}
    if {$filename != ""} {
	ccseiPuts "ccseiDbFileSaveAttrList: opening \"$filename\""
	set f [open $filename w]
	ccseiPuts "ccseiDbFileSaveAttrList: writing {$ccseiDb_v(AttList)}"
	puts $f $ccseiDb_v(AttList)
	ccseiPuts "ccseiDbFileSaveAttrList: writing {$ccseiDb_v(ValList)}"
	puts $f $ccseiDb_v(ValList)
	close $f
    }
}

#___oOo___
