#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiDbMonitor.tcl,v 3.11 2005/09/14 14:11:18 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  31/07/01  created
#


proc ccseiDbUpdateList { } {
    global wdgMonTable
    global ccseiDb_v ccseiDb_w gvar

    if {$gvar(dbMonActive) == 0 } { return }

    set gvar(dbUpdateLed) 1
    ccseiUpdateAttrList $wdgMonTable all 1
    ccseiDbMonitorOnFile ;# Write values on file
    
    set delay [expr {$gvar(dbUpdatePeriod) * 1000}]
    set ccseiDb_v(dbMonAfterId) [after $delay ccseiDbUpdateList]
}


proc ccseiDbMoveToList { recNumber } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global ccseiDb_v

    ccseiCallDbMonitor

    set tmp [$wdgAttrTable get $recNumber]

    set attrName [lindex $tmp $colDbName]
    set attrType [lindex $tmp $colDbAttrType]
    set dbName   "@$ccseiDb_v(Env)$ccseiDb_v(Cwp).${attrName}"

    set dbInfo  [ccseiDbAttrInfo  $dbName ""]
    lassign $dbInfo dbAttrType dbType numRecs numFields
    
    incr numRecs   -1
    incr numFields -1

    if {[cequal $attrType VECTOR]} {
	append dbName "(0:$numRecs)"
    } elseif {[cequal $attrType TABLE]} {
	append dbName "(0:$numRecs,0:$numFields)"
    }

    # Add selected attribute to monitoring table
    $wdgMonTable insert end "\"${dbName}\" [lrange $tmp $colDbAttrType $colDbQuality]"

    set idx [$wdgMonTable size]; incr idx -1
    # Read Current Value
    ccseiDbUpdateValue $wdgMonTable $idx $dbName 1
}


proc ccseiDbMonAction { what } {

    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global ccseiDb_v gvar dbModifyIDX

    set recId [$wdgMonTable  curselection]
    
    if {[cequal $recId ""]} {return }

    set recInfo [$wdgMonTable get $recId]

    if {[cequal $what "MOVE_UP"]} {

	if {$recId != 0} {
	    $wdgMonTable delete $recId
	    incr recId -1
	    $wdgMonTable insert $recId $recInfo
	}

	$wdgMonTable selection set $recId $recId

    } elseif {[cequal $what "MOVE_DOWN"]} {
	set maxIndex  [$wdgMonTable size]
	incr maxIndex -1

	if {$recId < $maxIndex} {
	    $wdgMonTable delete $recId
	    incr recId +1
	    $wdgMonTable insert $recId $recInfo
	}

	$wdgMonTable selection set $recId $recId

    } elseif {[cequal $what "REMOVE"]} {
	$wdgMonTable delete $recId
    } elseif {[cequal $what "MODIFY"]} {
	# Keep track of the modified record number
	set dbModifyIDX $recId
	# Move selected attribute to entry field to be modified
	set gvar(dbModifyAttribute) [$wdgMonTable cellcget ${recId},${colDbName}  -text]
	set gvar(dbModifyValue)     [$wdgMonTable cellcget ${recId},${colDbValue} -text]
    }
}



proc ccseiDbModifyAction {} {
    global gvar dbModifyIDX wdgMonTable

    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    set dbName $gvar(dbModifyAttribute)

    if {$gvar(dbModifyMode) == 1} {

	# ACTION : write
	panDbWrite $dbName "$gvar(dbModifyValue)"
	
	# If monitoring is not active ... force update
	
	if {$gvar(dbMonActive) == 0 } {
	    set gvar(dbUpdateLed) 1
	    ccseiUpdateAttrList $wdgMonTable $dbModifyIDX 1
	}

    } elseif {$gvar(dbModifyMode) == 2} {

	# ACTION : modify attribute name
	if {![cequal $dbName ""] && $dbModifyIDX >= 0 } {
	    # Move the modifyed attribute back to table
	    $wdgMonTable cellconfigure ${dbModifyIDX},${colDbName} -text $dbName
	}
    }

    # Update value in the entry field 
    set val [ccseiDbReadSymbolic $dbName 1]
    set gvar(dbModifyValue) [lindex $val 1]

    # Update values in the list
    set gvar(dbUpdateLed) 1
    ccseiUpdateAttrList $wdgMonTable $dbModifyIDX 1
}

proc ccseiDbMonExit { w } {
    global ccseiDb_v

    catch { after cancel $ccseiDb_v(dbMonAfterId) }
    destroy $w
}


proc ccseiCallDbMonitor {} {
    global ccseiDb_v cv gvar env dbListClassNames  dbListClassID

    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable
    
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    global attrTblName monTblName dbModifyIDX

    set wroot .wdbm

    if {[winfo exists $wroot]} {
	wm deico $wroot
	raise    $wroot
	return
    }

    set gvar(dbMonMessage) ""
    set dbModifyIDX -1
    set wtop [toplevel $wroot]

    wm title     $wtop "CCS DB Monitor"
    wm resizable $wtop 1 1     ;# Set window as resizable
    wm protocol  $wtop WM_DELETE_WINDOW "ccseiDbMonExit $wtop"
    wm deico     $wtop

    # Split panel into frames

    set topMenu [frame $wtop.topMenu]
    set f1 [frame $wtop.f1 ]
    set f2 [frame $wtop.f2 ]
    set f3 [frame $wtop.f3 ]
    set f4 [frame $wtop.f4 ]
    set f5 [frame $wtop.f5 ]
    
    pack $topMenu $f1 -side top -fill x
    pack $f2   -fill both -expand yes
    pack $f3 $f4 $f5 -fill both


    #############################################################
    #        Create DB Monitoring window
    #############################################################
    

    set wdgMonTable [ccseiCreateScrollTable $f2 $monTblName  expand]

    # Configure Table

    set clist [list 70 "Attribute Path" 0 "Attr. Type" 0 "DB Type" 0 "Quality" 0 "Value" ] 

    $wdgMonTable configure -columns $clist -labelfont $ccseiDb_v(cour12B)

    $wdgMonTable columnconfigure $colDbValue    -font $ccseiDb_v(cour14)
    $wdgMonTable columnconfigure $colDbAttrType -hide 0
    $wdgMonTable columnconfigure $colDbValue    -width 32

    # Configure the name of the procedure called when a cell
    # is edited

    $wdgMonTable configure -editendcommand   {} 
    $wdgMonTable configure -editstartcommand {}

    # Set default options
    ccseiDbOption $monTblName POINT_INFO
    ccseiDbOption $monTblName HIDE_DB_TYPE
    ccseiDbOption $monTblName HIDE_ATTR_TYPE
    ccseiDbOption $monTblName HIDE_QFLAG

    set wdgMonitorCmd $f3.dbMonitorCmd
    ccseiDbMonCommands_uifClass $wdgMonitorCmd -relief flat -library libccseiLib -globalName 1
    $wdgMonitorCmd display
    
    pack $wdgMonitorCmd -fill both
    
    # Define bindings

    set cv [string trim [$wdgMonitorCmd getClassPath ] ":"]
    set w  [$cv.wdgModifyAttribute component uifEntry]
    bind $w <Return> ccseiDbModifyAction

    set w  [$cv.wdgModifyValue component uifEntry]
    bind $w <Return> ccseiDbModifyAction


    #############################################################
    #                   Create Menu Bar
    #############################################################
    
    set menuBar [ makeMenuFrame $topMenu ]
    
    set mb [addMenu $menuBar  Help  1 right];      # Help Menu

    addMenuCmd $mb "Info..."          -1 "ccseiDbHelpInfo"
    addMenuCmd $mb "Extended Help..." -1 "ccseiDbHelpExtHelp"
    
    #  File Menu
    
    set mb [addMenu $menuBar  File  1];
    addMenuCmd $mb "Load Attribute List" 1 {ccseiDbFileLoadAttrList}
    addMenuCmd $mb "Save Attribute List" 1 {ccseiDbFileSaveAttrList}
    $mb add separator
    addMenuCmd $mb  "Quit"  1  "ccseiDbMonExit $wtop"
    
    #  Options
    
    set mb [addMenu $menuBar  Options  1]; 

    addMenuCk $mb "Hide Data Type"        -1 "ccseiDbOption $monTblName HIDE_DB_TYPE"   ccseiDb_v(${monTblName}_hideDbType)
    addMenuCk $mb "Hide Attribute Type "  -1 "ccseiDbOption $monTblName HIDE_ATTR_TYPE" ccseiDb_v(${monTblName}_hideAttrType)
    addMenuCk $mb "Hide Quality Flag "    -1 "ccseiDbOption $monTblName HIDE_QFLAG"     ccseiDb_v(${monTblName}_hideQFlag)
    $mb add separator
    addMenuCmd $mb "Remove All Attributes" 1 {$wdgMonTable delete 0 end}

    return
}

#
# ___oOo___
