#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiDbNavigation.tcl,v 3.11 2005/09/14 14:11:19 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin  2004-10-07  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#-----------------------------------------------------------------
#                  Database Navigation Procedures
#-----------------------------------------------------------------


proc ccseiDbOption { wdgTableName  what } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView wdgMonTable
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    global ccseiDb_v attrTblName monTblName

    if {[cequal $wdgTableName $attrTblName]} {
	set w $wdgAttrTable
    } else {
	set w $wdgMonTable
    }

    if {[cequal $what POINT_INFO]} {

	if {$ccseiDb_v(ptInfo)} {
	    pack $wdgPtInfo -after $wdgPtView -fill both
	    $wdgPtInfo display
	} else {
	    pack forget $wdgPtInfo
	}
    }

    if {[cequal $what HIDE_DB_TYPE]} {
	set val [set ccseiDb_v(${wdgTableName}_hideDbType)]
	$w columnconfigure $colDbDataType -hide $val

    } elseif {[cequal $what HIDE_ATTR_TYPE]} {
	set val [set ccseiDb_v(${wdgTableName}_hideAttrType)]
	$w columnconfigure $colDbAttrType -hide $val

    } elseif {[cequal $what HIDE_QFLAG]} {
	set val [set ccseiDb_v(${wdgTableName}_hideQFlag)]
	$w columnconfigure $colDbQuality -hide $val
    }

}

proc ccseiDbGetParentNodes { nid } {
    global dbTreeName

    set nodeList {}

    while {$nid > 0} {
	set nid [$dbTreeName parent $nid]
	lappend nodeList $nid
    }
    return $nodeList
}




proc ccseiDbSetView { {what "" } } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView ccseiDb_v
    global ccseiDb_v gvar dbTreeName

    if {[cequal $what all]} {
	$wdgDbTree show all
	return
    }

    set nodeId   [$wdgDbTree curselection]
    set nodeName [$dbTreeName label $nodeId]
    set nodeList [list $nodeId]

    # Get all ancestors of the current node.
    # The root node has nodeId = 0.

    set nodeList [ccseiDbGetParentNodes $nodeId]

    # Get all nodes below the one choosen by the user.
    set l1 [$dbTreeName find $nodeId -path  -regexp "*${nodeName}*"]
    
    if {[llength $l1] != 0} {
	eval lappend nodeList $l1
    } 

    # Hide all nodes and then show only the selected list
    $wdgDbTree hide all
    eval $wdgDbTree show $nodeList
    return
}




# Procedure to be invoked when a node (db point) is opened.

proc ccseiDbOpenNode { nodeName nodePath nodeId } {
    global wdgDbTree dbTreeName gvar ccseiDb_v

    set ccseiDb_v(Message) "" ;# Clear previous message

    ccseiGuiPuts "ccseiDbOpenNode ID = $nodeId  nodeName $nodeName"
    ccseiGuiPuts "                     $nodePath"
    
    if {[cequal $nodePath ":"]} { return } 

    set dbPath ":${nodePath}"

    # Check if the children of this DB points have already been added
    # to the DB tree.

    set numChildren [$wdgDbTree entry size $nodeId]

    ccseiGuiPuts "Num. Children for node $nodeId = $numChildren"
    if {$numChildren == 0} {
	# Add new children
	ccseiDbAddChildren $dbPath $nodeId
	update idletasks
    }

    # Notify that the point has been added
#    set ccseiDb_v(notify) 1

    update idletasks
    return
}



proc ccseiDbSelectPoint { nodePath nodeId } {
    global wdgDbTree dbTreeName gvar ccseiDb_v env

    set nodeName [$dbTreeName label $nodeId]

    if {[info exist ccseiDb_v(dbNodeId)] && [$dbTreeName exists $ccseiDb_v(dbNodeId)]} {
	$wdgDbTree entry configure $ccseiDb_v(dbNodeId) -foreground Black \
	    -font $ccseiDb_v(cour12)
	update idletasks
    }

    # Record current selected node 
    set ccseiDb_v(dbNodeId) $nodeId
    
    # Change forgound color to indicate the current selected node
    $wdgDbTree entry configure $nodeId  -font $ccseiDb_v(cour12B)

    if {[cequal $nodePath ":"]} {
	set dbPath ":${nodeName}"
    } else {
	set dbPath "${nodePath}:${nodeName}"
    }

    set ccseiDb_v(Cwp)   $dbPath
    set dbName           "@$ccseiDb_v(Env)${dbPath}"
    
    set ptInfo [ccseiDbGetPtInfo $dbName]
    
    lassign $ptInfo gvar(dbAliasName) gvar(dbClassName) gvar(dbClassID) \
	gvar(dbBaseClassName) gvar(dbPointPLIN)
    
    if {[cequal $gvar(dbPointPLIN) ""]} {
	set gvar(dbPointPLIN) $ccseiDb_v(na)
    }
    
    set gvar(dbPointName) [$dbTreeName label $nodeId]
    set gvar(dbPointID)   $nodeId

    update

    ccseiDbShowAttributes $dbName $nodeId
}



proc ccseiDbShowAttributes { dbPath nodeId } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView 
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global gvar ccseiDb_v

    # Clear previous list
    $wdgAttrTable delete 0 end

    ccseiGuiPuts "ccseiDbShowAttributes dbPath $dbPath  nodeId $nodeId"

    # The dbPath already contains the name of the environment.
    set slp   [lsort [ccseiDbAttrNames "$dbPath" ]]
 
    set idx 0
    foreach attrName $slp {

	# Skip attribute class.
	if {[cequal $attrName class] && $ccseiDb_v(guiDebug) == 0} {
	    continue
	}

	set dbInfo [ccseiDbAttrInfo "$dbPath" $attrName]

	lassign $dbInfo attrType dbType numRecs numFields

	# Convert numeric database data type into string
	set dbType [ ccseiGetDataType $dbType]

	# Attribute names are enclosed in quotes because they can contain blanks !!!

	if {[cequal $attrType SCALAR]} {
	    set tmp "\"$attrName\" SCALAR $dbType OK  0"
	} elseif {[cequal $attrType VECTOR]} {
	    set tmp "\"$attrName\" VECTOR $dbType OK  VECTOR($numRecs)"
	} elseif {[cequal $attrType TABLE]} {
	    set tmp "\"$attrName\" TABLE \"$dbType\"  OK   TABLE($numRecs,$numFields)"
	} else {
	    set tmp "\"$attrName\" UNKNOWN UNKNOWN  UNKNOWN UNKNOWN"
	}

	# Configure as editable the cells associated to SCALAR values

	$wdgAttrTable insert end $tmp
	update idletasks

	incr idx
    }

    # Configure editable cells according to user's choice
    ccseiDbEnableAttrEdit

    # Read current DB values. 
    ccseiUpdateAttrList $wdgAttrTable all 0

}




# Procedure to add new db points 

proc ccseiDbAddChildren { nodePath  nodeId } {
    global wdgDbTree ccseiDb_v dbTreeName

   # Get current tree level
    set ccseiDb_v(dbLevel) [$dbTreeName depth $nodeId]

    ccseiGuiPuts "ccseiDbAddChildren  nodePath  $nodePath   levelID $ccseiDb_v(dbLevel)"

    set lp  [seq_dbGetFamilyNames "@$ccseiDb_v(Env)${nodePath}" ABSOLUTE ]

    # Remove first element of the list which indicated the parent
    # and sort the remaining names
    set slp  [lsort [lrange $lp 1 end]]

    ccseiGuiPuts "ccseiDbAddChildren  nodePath  $nodePath \n  $slp"

    foreach pname $slp {
	# Get alias of the point This are used to tag the node
	# The alias returns the name prefixed with the string "<alias>"
	# which should be removed from the tag.
	
	set aliasPoint [ccseiDbGetAlias "@$ccseiDb_v(Env)${pname}"]
	set id         [$wdgDbTree insert end $pname]
	
	$wdgDbTree tag add $aliasPoint  $id
	$wdgDbTree tag add "level$ccseiDb_v(dbLevel)" $id
	$wdgDbTree entry configure $id -font $ccseiDb_v(cour12) -foreground Black

	$wdgDbTree bind $id <Double-ButtonPress-1> "ccseiDbSelectPoint \"${nodePath}\" $id"

	update idletasks
	ccseiGuiPuts "Add Point $pname  alias $aliasPoint  ID $id"
    }
}


proc ccseiDbGetPath { nodeId } {
    global wdgDbTree gvar dbTreeName

    ccseiGuiPuts "ccseiDbGetPath NodeID = $nodeId"

    # From nodeId we get the node path and we convert it
    # into database full database path
    set nodePath [$dbTreeName path $nodeId]

    # The node path is a list of names. Since there are 
    # some names with " " (blanks) inside we cannot use
    # regsub, therefore we have to build the DB path 
    # using a for loop.

    set dbPath ""
    foreach name $nodePath {
	append dbPath ":"
	append dbPath [string trim $name "{}"]
    }

    ccseiGuiPuts "ccseiDbGetPath  nodePath $dbPath"

    return $dbPath
}

proc ccseiDbCloseNode { nodeName nodePath nodeId } {
    global wdgDbTree dbTreeName gvar ccseiDb_v

    if {[cequal $nodeId ""] } { return }

    # Change forgound color back to 'normal'
    $wdgDbTree entry configure $ccseiDb_v(dbNodeId) -foreground Black

    return
}


proc ccseiDbGetClassInfo { className idx } {
    global dbListClassNames  dbListClassID dbListSuperClass ccseiDb_v

    set classID   [lindex $dbListClassID    $idx] 
    set sclassID  [lindex $dbListSuperClass $idx]

    if {$sclassID == 0} {
	set baseClass $className
    } else {
	set idx       [lsearch $dbListClassID $sclassID]
	set baseClass [lindex $dbListClassNames $idx]
    }

    ccseiGuiPuts "ccseiDbGetClassInfo $className idx $idx classID $classID  sclassID $sclassID "

    return [list $classID $baseClass]
}



proc ccseiDbGetPtInfo { dbPath } {
    global dbListClassNames  dbListClassID dbListSuperClass ccseiDb_v

    ccseiGuiPuts "ccseiDbGetPtInfo dbPath $dbPath"

    set className ""
    set classID   ""
    set baseClass ""
    set aliasName [ccseiDbGetAlias   "$dbPath"]
    set idx       [string last ":" $dbPath]; incr idx
    set ptName    [string range $dbPath $idx end]
    set ptPlin    [ccseiDbGetDirAddr "$dbPath"]

    # Get attribute list

    set alist [seq_dbGetAttrNames "$dbPath"]

    ccseiGuiPuts "dbGetAttrNames $alist"

    # Check if the name of the points can be identified as class in itself

    set idx1  [lsearch $dbListClassNames "$ptName"]
    set idx2  [lsearch $dbListClassNames "<${ptName}>"]

    if {$idx1 != -1 || $idx2 != -1} {

	if {$idx1 >=0 } { 
	    set idx $idx1 
	} else {
	    set idx $idx2
	}

	set className [lindex $dbListClassNames $idx]
        lassign [ccseiDbGetClassInfo $className $idx] classID baseClass

    } else {

	# Check if there is the attribute 'class'

	if {[lsearch $alist "class"] != -1} {
	    
	    set className [string range [seq_dbReadSymbolic "${dbPath}.class" ] 7 end]
	    set idx       [lsearch $dbListClassNames $className]
	    
	    ccseiGuiPuts "ccseiDbGetPtInfo className $className  idx = $idx"

	    if {$idx >= 0} {
		lassign [ccseiDbGetClassInfo $className $idx] classID baseClass
	    } else {
		return [list $aliasName $className ?? UNKNOWN]
	    }   
	}	   
    }

    ccseiGuiPuts "ccseiDbGetPtInfo returns [list $aliasName $className $classID $baseClass $ptPlin]"

    return [list $aliasName $className $classID $baseClass $ptPlin]
}


proc ccseiDbOpenNextLevel { } {
    global wdgDbTree gvar dbTreeName ccseiDb_v 

    if { ![$dbTreeName exists $gvar(dbNavNodeId)]} {
	return
    }
    
    # Re-define procedure to be called when a node is opened
    # such that the attributes are not displayed

    set ccseiDb_v(selectNode) 0

    $wdgDbTree configure -opencommand { ccseiDbOpenNode {%p} {%P} %# }

    $wdgDbTree open -recurse $gvar(dbNavNodeId)

    # Restore original configuration
    $wdgDbTree configure -opencommand { ccseiDbOpenNode {%p} {%P} %# }
}



proc ccseiDbShowPoint { } {
    global wdgDbTree gvar dbTreeName ccseiDb_v env

    set ccseiDb_v(selectNode) 0 
    set gvar(dbNavNodeId) [$wdgDbTree curselection]
    set ptInfo ""

    ccseiGuiPuts "ccseiDbShowPoint  $gvar(dbNavNodeId)"

    if {![cequal $gvar(dbNavNodeId) ""]} {
	set dbPath [ccseiDbGetPath $gvar(dbNavNodeId)]
	set ptInfo [ccseiDbGetPtInfo "@$ccseiDb_v(Env)${dbPath}"]

	lassign $ptInfo gvar(dbNavAlias) gvar(dbNavClassName) classID className gvar(dbNavPLIN)
    }

    if {[cequal $gvar(dbNavPLIN) ""]} {
	set gvar(dbNavPLIN)  $ccseiDb_v(na)
    }

    ccseiGuiPuts "ccseiDbShowPoint return  $ptInfo"
    update
}


# This routine is used to read all the class names and ID

proc ccseiDbReadListClass {} {
    global dbListClassNames  dbListClassID dbListSuperClass ccseiDb_v

    set dbListClassNames {}
    set dbListClassID    {}
    set dbListSuperClass {}

    set attrName "@$ccseiDb_v(Env)<alias>class config.names"

    if {[catch {ccseiDbAttrInfo $attrName "" } attrInfo]} {
	return
    }

    lassign $attrInfo attrType dbTypes maxRecs maxFields

    incr maxRecs -1

    if {[catch {seq_dbReadSymbolic "${attrName}(0:$maxRecs,0)" } dbListClassID]} {
	set tmp "ccseiDb Cannot read list of class IDs from attribute $attrName"
	set ccseiDb_v(Message) $tmp
	return
    }

    if {[catch {seq_dbReadSymbolic "${attrName}(0:$maxRecs,1)" } dbListClassNames]} {
	set tmp "ccseiDb Cannot read list of class Names from attribute $attrName"
	set ccseiDb_v(Message) $tmp
	return
    }

    if {[catch {seq_dbReadSymbolic "${attrName}(0:$maxRecs,2)" } dbListSuperClass]} {
	set tmp "ccseiDb Cannot read list of 'super class' from attribute $attrName"
	set ccseiDb_v(Message) $tmp
	return
    }
}


#
# ___oOo___
