#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiDbSearch.tcl,v 3.11 2005/09/14 14:11:19 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin  2004-10-07  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


proc ccseiDbSearchDialog { } {
    global ccseiDb_v gvar wdgDbSearch
 
    # Some aliases contain blancks, which must be removed.
    set w .dbSearchDialog

    if {[winfo exists $w]} {
	raise $w
        return
    }

    set wtop [toplevel $w]
    wm title $wtop "DB Search Dialog"
    wm resizable $wtop 0 0
	
    set f1 [frame $wtop.f1 -relief flat]
    set f2 [frame $wtop.f2 -relief flat]
    set f3 [frame $wtop.f3 -relief flat]

    pack $f1      -fill x -padx 5 -pady 5
    pack $f2 $f3  -fill x -padx 5

    UifEntry  $f1.searchKey -label "Keyword "  -width 32 \
	-labelFont $ccseiDb_v(cour12B) -variable ccseiDb_v(dbSearchKey)

    UifButton $f1.dbPtSearch  -label  "Search"  -width 10  -command  {ccseiDbSearch}

    pack $f1.searchKey $f1.dbPtSearch -side left -padx 10

    label $f2.lblName -text "Search Result" -font $ccseiDb_v(cour12B)
   
    set wdgDbSearch [UifText $f2.wdgSearch -height 3 -width 65 -wrap char \
			 -font $ccseiDb_v(cour12B) -foreground Blue -relief sunken ]

    pack forget [$wdgDbSearch component scrollv]
    pack forget [$wdgDbSearch component scrollh]

    [$wdgDbSearch component uifText] configure -state disabled

    pack $f2.lblName $wdgDbSearch  -anchor w -padx 10

    UifButton $f3.dbPtAdd    -label  "Show Point"  -width 12  -command  {ccseiDbAddToDB}

    UifButton $f3.dbCancel   -label  "Cancel"      -width 12 \
	-command  "ccseiDbSearchCancel $wtop"

    pack $f3.dbPtAdd  -side left  -pady 10 -padx 5
    pack $f3.dbCancel -side right -pady 10 -padx 5

    wm protocol  $wtop WM_DELETE_WINDOW "ccseiDbSearchCancel $wtop"
    return

}


proc ccseiDbAddToDB {} {
    global wdgDbTree dbTreeName gvar ccseiDb_v wdgDbSearch
    
    if {![info exist ccseiDb_v(dbFoundName)] || [cequal $ccseiDb_v(dbFoundName) ""] } {
	return
    }
    
    set dbListName [split [string trim $ccseiDb_v(dbFoundName) ":"] ":"]
    
    # Open all nodes 
    
    set j 0
    set maxNodes [llength $dbListName]
    set dbName   [lindex $dbListName $j]
    
    while { $j < $maxNodes} {
	set idx [$wdgDbTree find -full "$dbName"];
	$wdgDbTree open $idx;
	incr j;
	append dbName ":[lindex $dbListName $j]"
    }

    $wdgDbTree selection clearall
    $wdgDbTree selection set $idx

    return
}


proc ccseiDbSearchClear {} {
    global ccseiDb_v wdgDbTree

    if {[info exists ccseiDb_v(dbSearchList)]} {
	set idx 0

	foreach nodeId $ccseiDb_v(dbSearchList) {
	    set fcolor  [lindex $ccseiDb_v(dbSearchFg) $idx]
	    $wdgDbTree entry configure $nodeId -foreground $fcolor
	    incr idx
	} 
    }
}

proc ccseiDbSearchCancel { wtop } {
    global wdgDbTree dbTreeName gvar ccseiDb_v wdgDbSearch

    ccseiDbSearchClear
    if {[winfo exists $wtop]} { destroy $wtop }
}


proc ccseiDbSearch {} {
    global wdgDbTree dbTreeName gvar ccseiDb_v wdgDbSearch

    ccseiDbSearchClear

    set keyword [string trim $ccseiDb_v(dbSearchKey) " "]

    [$wdgDbSearch component uifText] configure -state normal
    $wdgDbSearch deleteText 0.0 end

    # Look for nodes with this keyword

    set nodeList {}

    if { [ catch {$wdgDbTree find -name $keyword} tmp] == 0 } {
	set nodeList [concat $nodeList $tmp]
    }

    if { [ catch {$dbTreeName tag nodes $keyword} tmp] == 0 } {
	# Merge the two lists and remove duplicates
	set nodeList [lsort -unique [concat $nodeList $tmp]]
    }


    if {[llength $nodeList] == 0} {

	# Check if this is an alias of the databas.
	# In that case get its absolute address and report it.

	set dbName "@$ccseiDb_v(Env)<alias>${keyword}"
	set ccseiDb_v(dbFoundName) ""

	if {[catch {ccseiDbGetDirAddr $dbName} dirAddr ]} { 
	    set searchText "No Point found for keyword = '$keyword'"
	} else {
	    if {![cequal $dirAddr ""]} {
		# Convert direct address into ABSOLUTE symbolic name
		set ccseiDb_v(dbFoundName) [seq_dbDirAddrToName "${dirAddr}" ABSOLUTE]
		set searchText "'$keyword' found as alias of $ccseiDb_v(dbFoundName)"
	    }
	}

    } else {

        set numNodes [llength $nodeList]
	set searchText "Found #$numNodes points matching keyword"

	set ccseiDb_v(dbSearchList) {}
	set ccseiDb_v(dbSearchFg)   {}

	set tmp {}
	foreach nid $nodeList {
	    # Get parents of all nodes in nodeList
	    set tmp [concat $tmp [ccseiDbGetParentNodes $nid] $nid]
	}

	# Remove duplicates and open all nodes
	foreach nodeId [lsort -unique $tmp] {
	    $wdgDbTree open $nodeId
	}

	# Identify target nodes.
	foreach nodeId $nodeList {
	    lappend ccseiDb_v(dbSearchList) $nodeId
	    lappend ccseiDb_v(dbSearchFg)   [$wdgDbTree entry cget $nodeId -foreground]
	    $wdgDbTree entry configure $nodeId -foreground magenta
	}
    }

    $wdgDbSearch insertText 0.0 $searchText
    [$wdgDbSearch component uifText] configure -state disabled
}




#
# ___oOo___
