#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiDbViewPoint.tcl,v 3.11 2005/09/14 14:11:19 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin  2004-10-07  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


# This procedure replaces the default 'endcommand' procedure, therefore it
# also takes care of properly configuring the cell and replace the old value
# with the new one.

proc ccseiDbEditEnd { wtable row col text } {
    global ccseiDb_v

    if {$ccseiDb_v(guiDebug)} {
	puts "ccseiDbEditEnd $wtable $row $col '$text'"
    }

    # The return value of this procedure is automatically used
    # as content of the entry widget created to edit the cell.

    return $text
}



proc ccseiDbWriteCell { w rowIdx  colIdx  newVal } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView ccseiDb_v 
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global gvar

    set attrName  [$wdgAttrTable cellcget ${rowIdx},${colDbName} -text]
    set dbName    "@$ccseiDb_v(Env)$ccseiDb_v(Cwp).${attrName}"

    if {$ccseiDb_v(guiDebug)} {
	puts "ccseiDbWriteCell $dbName    $w $colIdx newVal = '$newVal'"
    }

    if {[catch {seq_dbWriteSymbolic $dbName "$newVal" } err]} {
	set tmp "Failed writing attribute $attrName"
	tk_dialog .dialog "Error On Write" $tmp info 0 OK

	# restore old value
	set val    $ccseiDb_v(oldCell)
	set msgTxt "Failing to write DB attribute -->  $gvar(dbPointName).${attrName}"
    } else {

	set msgTxt "DB Write :  $gvar(dbPointName).${attrName} = '$newVal'"
	update

	# Replace cell value of cell with its current DB value.
	set val [seq_dbReadSymbolic $dbName]
    }

    set ccseiDb_v(Message) $msgTxt

    # The return value of this procedure is automatically used
    # as content of the entry widget created to edit the cell.

    return $val
}




proc ccseiDbEditStart { wtable row col text } {
    global ccseiDb_v

    # Get text from the table cell
    set txt [$wtable cellcget ${row},${col} -text]
    set ccseiDb_v(oldCell) $txt
    set w [$wtable editwinpath]

    if {$ccseiDb_v(guiDebug)} {
	puts "ccseiDbEditStart $wtable $row $col $txt"
	puts "cell widget name editwinpath [$wtable editwinpath]"
	puts "cell widget name entrypath   [$wtable entrypath]"
    }
   
    # The return value of this procedure is automatically used
    # as content of the entry widget created to edit the cell.

    return $txt
}



proc ccseiDbCE { what w dbName } {
    global ccseiDb_v

    # If this Set/Get CE is not avaliable in this version of seq.
    # just return and do NOTHING
    if {[cequal $ccseiDb_v(dbCE) ""] } {
	return
    }

    # Remove quotes added to dela with attribute names
    # containing spaces
    set dbName [string trim $dbName "\""]

    if {[cequal $what READ]} {

	$w deleteText 0.0 end

	if {[catch {seq_dbGetCe "${dbName}" } dbCE] } {
	    set tmp "Cannot get CE for \n $dbName"
	    seq_errDisplay
	    return
	}

	$w insertText 0.0 $dbCE

    } else {

	# Remove "\n" character
	set dbCE [string trim [$w getText 0.0 end] "\n"]

	# Report erro only if this function is available

	if {[catch {seq_dbSetCe "${dbName}" ${dbCE} }]} {
	    set tmp "Cannot set CE for \n $dbName"
	    seq_errDisplay
	    return
	}
    }
}



proc ccseiDbViewCmd { recId } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView ccseiDb_v gvar

    if {$ccseiDb_v(moveToList)} {
	ccseiDbMoveToList    $recId
    } else {
	ccseiDbViewAttribute $recId
    }
}


proc ccseiDbConfigView { wvalue wname } {
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global gvar ccseiDb_v

    $wvalue delete 0 end

    set tmp [$wname getText 0.0 end]
    # Remove carriage return
    regsub -all "\n" $tmp "" dbName

    $wname deleteText 0.0 end
    $wname insertText 0.0 $dbName

    # This call returns exactly the number of records and fields 
    # as specified by the attributes range.

    set dbInfo  [seq_dbGetAttrInfo  "${dbName}"]
    lassign $dbInfo dbAttrType dbTypeList numRecs numFields

    if {[cequal $dbAttrType SCALAR]} {
	return [list 1 1]
    }

    set dbInfo [ccseiDbGetRange "${dbName}" $dbAttrType $numRecs $numFields]
    lassign $dbInfo dbName1 rec1 rec2 field1 field2
	
    if {[expr {$rec2 - $rec1} ] < 15 } {
	set wheight [expr {$rec2 - $rec1 + 3}]
    } else {
	set wheight 15
    } 

    if {[cequal $dbAttrType VECTOR]} {

	set dbType [ccseiGetDataType $dbTypeList]
	$wvalue configure -columns [list 0 "" 15 "( $dbType )"] -height $wheight
	
    } else {

	set fieldNames [lrange  [ccseiDbGetFieldNames "${dbName}" $numFields] $field1 $field2]
	set clist [list 0 ""]
	set fieldLen 10
	
	set idx 0
	foreach field $fieldNames {
	    
	    # Convert data type numbers into data type strings
	    set dbType     [lindex $dbTypeList $idx]
	    set dbTypeName [ccseiGetDataType $dbType]
	    # Calculate field index
	    set fdx        [expr {$rec1 + $idx}]
	    lappend clist 0 "$field\n($fdx - ${dbTypeName})"
	    
	    # Here we also calculate the length of all the field labels
	    # to get an approx. estimate of the size of the table.
	    # If this is less than 100, we configure the table to automatically
	    # stretch to see all the columns.
	    
	    incr fieldLen [string length $field]
	    incr idx
	}

	$wvalue configure -columns $clist -width 0 -height $wheight
    }
    
    set idx 1
    while {$idx < [$wvalue columncount]} {
	$wvalue columnconfigure $idx -labelfont $ccseiDb_v(cour12B) \
	    -editable 1 -font $ccseiDb_v(cour14)
	incr idx
    }

    # Force tcl to finish the creation of the dialog in order to 
    # be able to get its dimensions

    update idletask

    if {[winfo width $wvalue] > 800 } {
	$wvalue configure -width 100
    }
  
    update idletask
#    puts "ccseiDbConfigView $dbName  numRecs $numRecs numFields $numFields"
    return [list $numRecs $numFields]
}

 

proc ccseiDbViewAttribute { recNumber } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView 
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global gvar ccseiDb_v env

    if {[cequal $recNumber ""]} { return }

    ccseiGuiPuts "ccseiDbViewAttribute $recNumber "

    set recInfo    [$wdgAttrTable get $recNumber]
    set dbAttrName [lindex $recInfo $colDbName]
    set dbName     "@$ccseiDb_v(Env)$ccseiDb_v(Cwp).${dbAttrName}"

    # The name of the dialog must be unique, therefore it is 
    # formed by the concatenation of alias and attribute name.

    # Some aliases contain blancks, which must be removed.
    regsub -all " " ".pt$gvar(dbAliasName)_${dbAttrName}" "_" w

    if {[winfo exists $w]} {
        raise $w
	return
    }
    
    ccseiGuiPuts "ccseiDbViewAttribute $dbName $gvar(dbPointName) $gvar(dbAliasName)" 

    set dbInfo  [seq_dbGetAttrInfo  $dbName]
    lassign $dbInfo dbAttrType dbType numRecs numFields

    # Some database attributes, namely VECTORS and TABLES, can be defined
    # with NO RECORDS (numRecs = 0). In this case the view dialog should
    # not appear.
 
    if {$numRecs <= 0 || $numFields <= 0} {
	tk_dialog .d "Dialog" "VECTOR or TABLE does not have RECORDS or FIELDS" info 0 OK
    }

    set dialogName [string trimleft $w "."]
    set wtop       [toplevel $w]
    wm title $wtop "$gvar(dbPointName).${dbAttrName}"
    
    set f1 [frame $wtop.f1 -relief flat]
    set f2 [frame $wtop.f2 -relief flat]
    set f3 [frame $wtop.f3 -relief flat]
    set f4 [frame $wtop.f4 -relief flat]
    set f5 [frame $wtop.f5 -relief flat]
    set f6 [frame $wtop.f6 -relief flat]
#   set f7 [frame $wtop.f7 -relief flat]
    set f8 [frame $wtop.f8 -relief flat]

    pack $f1              -fill x    -padx 5 -pady 5
    pack $f2 $f3 $f4 $f5  -fill x    -padx 5
    pack $f6              -fill both -padx 5 -expand yes
    pack $f8              -fill x    -padx 5

    # Parse attribute name to use point and attribute name as pre-fixes
    # for variables.

    set width [string length $dbName]; incr width +2

    label $f1.lblName -text "Attribute Name" -font $ccseiDb_v(cour12B)

    set wname $f1.viewName
   
    UifText $wname -height 3 -width 60 -wrap char -font $ccseiDb_v(cour12B) \
	            -foreground Blue -relief sunken

    pack forget [$wname component scrollv]
    pack forget [$wname component scrollh]

    $wname insertText 0.0 $dbName

    pack $f1.lblName $wname  -anchor w -padx 10

    if {[cequal $dbAttrType SCALAR]} {
	uplevel #0 set dbType_${dialogName} [ccseiGetDataType $dbType]
	
	UifOutput $f2.viewDbType -label "Data Type :" -variable dbType_${dialogName} \
	    -color Blue -width 15 -font $ccseiDb_v(cour12)
	
	pack $f2.viewDbType -side left -pady 3
	set dirAddr [ccseiDbGetDirAddr $dbName];
	wm resizable $wtop 0 0
    } else {

	UifOutput $f2.viewRecs   -label "# Recs.  : " -variable nrecs_${dialogName} \
	    -color Blue -width 6 -font $ccseiDb_v(cour12)

	UifOutput $f2.viewField -label "# Fields. : " -variable nfields_${dialogName} \
	    -color Blue -width 6 -font $ccseiDb_v(cour12)

	uplevel #0 set nrecs_${dialogName}    $numRecs
	uplevel #0 set nfields_${dialogName}  $numFields

	pack $f2.viewRecs $f2.viewField  -side left -padx 5

	set dirAddr "n/a"
	if {$numRecs != 0 && $numFields != 0 } {
	    set maxRec   [expr {$numRecs -1}]
	    set maxField [expr {$numFields -1}]

	    # Adding elements to the tablelist widget is rather slow
	    # therefore for very big tables we take by default the first 1000
	    # elements and we let the user choose a proper range later on.
	    
	    if {$maxRec > 1000} {
		set recMax 999
	    } else {
		set recMax $maxRec
	    }
	    
	    if {[cequal $dbAttrType VECTOR ]} {
		set elemRange "(0:$recMax)"
		set dirAddr [ccseiDbGetDirAddr ${dbName}${elemRange}];
		wm resizable $wtop 0 1
	    } else {
		set elemRange "(0:$recMax,0:$maxField)"
		set dirAddr [ccseiDbGetDirAddr ${dbName}${elemRange}]
		
		wm resizable $wtop 1 1
	    }
	    
	    $wname insertText end  $elemRange
	}

    }
	
    # Remove squared parenthesis otherwise we get problems with tcl interpreter
	
    if {[cequal $dirAddr ""]} {
	set tmp "$ccseiDb_v(na)"
	uplevel #0 set diradd_${dialogName} "${tmp}"
    } else {
	set tmp [string trim $dirAddr "\[\]@"]
	uplevel #0 set diradd_${dialogName}  "@\\\[${tmp}\\\]"
	set width [string length $tmp]; incr width +4
    }
    
    set width [string length $tmp]; incr width +4
    set wdirAddr $f2.viewDirAddr
    
    UifOutput $wdirAddr   -label "Dir. Addr. : " -variable diradd_${dialogName} \
	-color Blue -width $width -font $ccseiDb_v(cour12)
    
    pack $wdirAddr  -side left -padx 5
    
    set wapply    $f8.dbApply
    set wrefresh  $f8.dbRefresh 
    set wclear    $f8.dbClear  
    set wcancel   $f8.dbCancel 
    
    UifButton $wapply   -label  "Apply"   -width 8  -command  {}
    UifButton $wcancel  -label  "Cancel"  -width 8  -command  "destroy $w"
    UifButton $wrefresh -label  "Refresh" -width 8  -command  {}
    UifButton $wclear   -label  "Clear"   -width 8  -command  {}
    
    pack $wrefresh $wapply $wclear -side left  -pady 10 -padx 5
    pack $wcancel                  -side right -pady 10 -padx 5
    
    # CE definitions are not supported in TABLES and for all attributes of an LCU
    
    set envType [string index $ccseiDb_v(Env) 0]
    
    if {![cequal $dbAttrType TABLE] && ![cequal $envType "l"]} {

	set wce       $f5.viewCE
	label   $f4.lblName -text "Calculation Engine Formula" -font $ccseiDb_v(cour12B)
	
	UifButton $f4.ceWrite  -label  "Write CE" -width 8  \
	    -command  "ccseiDbCE WRITE $wce \"${dbName}\""
	UifButton $f4.ceRead   -label  "Read  CE" -width 8 \
	    -command  "ccseiDbCE READ  $wce \"${dbName}\""
	UifButton $f4.ceClear  -label  "Clear CE" -width 8 \
	    -command  "$wce deleteText 0.0 end"
	
	UifText $wce -height 3 -width 55 -wrap char -font $ccseiDb_v(cour12) \
	    -foreground Blue -relief sunken
	
	pack forget [$wce component scrollv]
	pack $wce  -side left -padx 10
	ccseiDbCE READ  $wce \"${dbName}\"
	pack $f4.lblName $f4.ceRead  -side left  -pady 10 -padx 5
	
	if {  [info exists env(WITH_CCS_SETCE)] &&
	      [cequal $env(RTAPENV) $ccseiDb_v(Env)] } {
	    
	    # Add button to modify CE if the env. configuration is properly configured
	    pack $f4.ceWrite $f4.ceClear  -side left  -pady 10 -padx 5
	}
    }
    
    set wframe $f6
    set wvalue $wframe.viewValue
    
    # Read current value of the attribute
    set attrName [$wname getText 0.0 end]
    
    if {[cequal $dbAttrType SCALAR]} {
	
	label $wframe.lblVALUE -text "Value" -font $ccseiDb_v(cour12B)
	
	UifText $wvalue -height 2 -width 55 -font $ccseiDb_v(cour12)

	pack forget [$wvalue component scrollv]
	pack $wframe.lblVALUE $wvalue -anchor w -padx 15 -pady 3

    } else {

        label $wframe.lblVALUE -text "List Of Values"

        set tframe [frame $wframe.ftable]

        pack $wframe.lblVALUE -padx 10 -pady 5
        pack $tframe          -padx 10 -pady 1 -fill both -expand yes

        set wvalue  [ccseiCreateScrollTable $tframe viewValue expand ]
        set range   [ccseiDbConfigView $wvalue $wname ]

        lassign $range numRecs numFields

	$wvalue configure -editendcommand   ccseiDbEditEnd
	$wvalue configure -editstartcommand ccseiDbEditStart

	set w [$wname component uifText]
	bind $w <Return> "ccseiDbConfigView $wvalue $wname ; ccseiDbViewRefresh $wvalue $wname 0"
    }

    update idletasks

    if {$numRecs != 0 && $numFields != 0} {

	# Configure buttons for apply/ refresh commands

	$wapply   configure -command "ccseiDbViewApply   $wvalue $wname $recNumber"
	$wrefresh configure -command "ccseiDbViewRefresh $wvalue $wname $recNumber"
	$wclear   configure -command "ccseiDbViewClear   $wvalue $wname"

	ccseiDbViewRefresh $wvalue $wname $recNumber
    }

    return
}


# This procedure returns the 'clear' value to be assigned to the 
# attributes according to the data type

proc ccseiDbClearValue { dbType } {
    if {$dbType >= 1 && $dbType <= 9} {
	return 0
    } elseif  {$dbType == 28} {
	return "@\[\]"
    } elseif  {$dbType == 10} {
	# Polar coordinates
	return "{0.0,0.0d}"
    } elseif  {$dbType == 11} {
	# Rectangular coordinates
	return "{0.0,0.0j}"

    } else {
	return ""
    }
}



proc ccseiDbViewClear { wvalue wname } {

    set dbName  [$wname getText 0.0 end]
    set dbInfo  [seq_dbGetAttrInfo  $dbName]
    lassign $dbInfo dbAttrType dbTypeList numRecs numFields

    if {[cequal $dbAttrType SCALAR]} {
	$wvalue deleteText 0.0 end
	$wvalue insertText  0.0 [ccseiDbClearValue [lindex $dbTypeList 0]]
    } else  {

	set tmp [ccseiDbGetRange $dbName $dbAttrType $numRecs $numFields]
	lassign $tmp dbName1 rec1 rec2 field1 field2

	set dbTypeList [lrange $dbTypeList $field1 $field2]

	set recId 0
	set maxRec [expr {$rec2 - $rec1 + 1}]
	while { $recId < $maxRec} {
	    
	    # The first columns contains just an index.. it should not be cleared
	    set colId 1

	    foreach dbType $dbTypeList {
		# Set cell with the proper 'clear' value according to the type of data
		$wvalue cellconfigure $recId,$colId -text [ccseiDbClearValue $dbType]
		incr colId
	    }
	    incr recId
	}
    }

    return
}

# This procedure updates the database attribute value displayed 
# in the 'view' panel.


proc ccseiDbViewRefresh { wvalue wname recId } {
    global wdgAttrTable ccseiDb_v

    set dbName  [$wname getText 0.0 end]

    if {$ccseiDb_v(guiDebug)} {
       puts "ccseiDbViewRefresh $wvalue wname $wname recId $recId   dbName $dbName"
    }

    set dbInfo  [seq_dbGetAttrInfo  $dbName]

    lassign $dbInfo dbAttrType dbTypeList numRecs numFields

    if {[cequal $dbAttrType SCALAR]} {
	$wvalue deleteText 0.0 end

	# Read Current Value
	set valList [ccseiDbReadSymbolic $dbName]
	$wvalue insertText 0.0 [lindex $valList 1]

	# Update corresponding value in the list of attributes
	# This applies only to SCALARS.
	ccseiUpdateAttrList $wdgAttrTable $recId 0
 
    } else {

	$wvalue delete 0 end

	set tmp [ccseiDbGetRange $dbName $dbAttrType $numRecs $numFields]
	lassign $tmp dbName1 rec1 rec2 field1 field2
    
#	puts "ccseiDbViewRefresh recs = $rec1 $rec2   fields = $field1 $field2"

	while { $rec1 <= $rec2 } {

	    if {[cequal $dbAttrType TABLE]} {
		set attrName "${dbName1}($rec1,${field1}:${field2})"
	    } else {
		set attrName "${dbName1}($rec1)"
	    }

            set valRec [seq_dbReadSymbolic "${attrName}"]

            # This check is needed if we are displaying a very big table
            # and the user 'cancel' the action before the whole table
            # has been updated

            if {![winfo exists $wvalue]} { return }

	    $wvalue insert end "$rec1 $valRec"
            update

	    incr rec1 +1
	}
    }

#    puts "DBG> Refresh DONE !!"
    return
}



proc ccseiDbGetRange { dbName dbAttrType numRecs numFields } {
    global wdgAttrTable

    if {[cequal $dbAttrType SCALAR]} {
	return [list $dbName 1 1 1 1]
    }

    set rec1   0
    if {$numRecs == 0} {
	set rec2   0
    } else {
	set rec2   [expr {$numRecs -1}]
    }

    set field1 0
    if {$numFields == 0} {
	set field2 0
    } else {
	set field2 [expr {$numFields -1}]
    }

    set fieldRange ""
    set idx        [string last "(" $dbName];

    if {$idx == -1 } {
	set elemRange "${rec1}:${rec2},${field1}:${field2}"
	set dbName1    $dbName
    } else {
	set tmp        [string range $dbName $idx end]
	set elemRange  [string trim $tmp "()\n" ]
	set dbName1    [string range $dbName 0 [expr {$idx - 1}]]
    }

    if {[cequal $dbAttrType TABLE] && $idx != -1 } {
	# Get record range
	set jdx [string last "," $elemRange]
	
	if {$jdx >= 0} {
	    incr jdx +1
	    set fieldRange [string range $elemRange $jdx end]

	    set nargs [scan $fieldRange "%d:%d" field1 field2]
    
	    if {$nargs == 1} { set field1 $field2 }

	    incr jdx -2
	    set recRange   [string range $elemRange 0 $jdx]
	}
    } else {
	set recRange  $elemRange
    }
  
    if {$idx != -1} {
	set nargs [scan $recRange "%d:%d" rec1 rec2]
    
	if {$nargs == 1} { set rec2 $rec1 }
    }

    return [list $dbName1 $rec1 $rec2 $field1 $field2 ]
}


proc ccseiDbViewApply { wvalue wname recId } {
    global ccseiDb_v

    if {$ccseiDb_v(guiDebug)} {
       puts "ccseiDbViewApply  $wvalue $wname  attribute index = $recId"
    }

    set dbName  [$wname getText 0.0 end]

    if {$ccseiDb_v(guiDebug)} {
       puts "ccseiDbViewApply  seq_dbGetAttrInfo $dbName"
    }

    set dbInfo  [seq_dbGetAttrInfo  $dbName]
    set valList ""
    lassign $dbInfo dbAttrType dbTypeList numRecs numFields

    if {[cequal $dbAttrType SCALAR]} {
	set valList [string trim [$wvalue getText 0.0 end] "\n"]
	set attrName $dbName 
    } else {

	set tmp [ccseiDbGetRange $dbName $dbAttrType $numRecs $numFields]
	lassign $tmp dbName1 rec1 rec2 field1 field2

	if {[cequal $dbAttrType VECTOR]} {
	    set attrName "${dbName1}(${rec1}:${rec2})"

	    # Collect all values into a single buffer
	    set valList [$wvalue getcolumns 1]

	} else {

	    set attrName "${dbName1}(${rec1}:${rec2},${field1}:${field2})"

	    # Collect all values into a single buffer
	    set recId 0
	    set maxRec [expr {$rec2 - $rec1 +1}]
	    
            if {$maxRec == 1} {
		set valList "[lrange [$wvalue get $recId] 1 end]"
	    } else {
		# If several records are written, each must be 
		# surrounded by curly brackets

		while { $recId < $maxRec } {
		    lappend valList "[lrange [$wvalue get $recId] 1 end]"
		    incr recId
		}
	    }
	}
    }

    if {[catch {ccseiDbWriteSymbolic $attrName $valList  1}]} {
	return
    }

    ccseiDbViewRefresh $wvalue $wname $recId

#    puts "DBG> Apply DONE !!"

    return
}

proc ccseiDbEnableAttrEdit {} {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView 
    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue
    global gvar ccseiDb_v

    set idx 0
    set listAttrType [$wdgAttrTable getcolumn $colDbAttrType $colDbAttrType]

    ccseiGuiPuts "ccseiDbEnableAttrEdit editFlag = $ccseiDb_v(editAttrList)"

    foreach attrType [lindex $listAttrType 0] {

	if {[cequal $attrType SCALAR] } {
	    # Direct editing of SCALARS follows user's choice
	    set editFlag $ccseiDb_v(editAttrList)
	} else {
	    # Direct editing is disabled for all non SCALAR attributes
	    set editFlag 0
	}

	$wdgAttrTable cellconfigure ${idx},${colDbValue} -editable $editFlag
#        $wdgAttrTable configure -finishediting    ccseiDbEditFinish


	incr idx
    }
}




proc ccseiUpdateAttrList { wtable elemIdx displayAllTypes } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView ccseiDb_v

    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    ccseiGuiPuts "ccseiUpdateAttrList $wtable $elemIdx $displayAllTypes  table size [$wtable size]"

    update idletasks

    if {[cequal $elemIdx all]} {
	set recId      0
	set maxNumbElem [$wtable size]
    } else {
	set recId $elemIdx
	set maxNumbElem [expr {$elemIdx +1} ]
    }
 
    while {$recId >= 0 && $recId < $maxNumbElem } {

	set tmp   [$wtable get $recId]
	set atype [lindex $tmp $colDbAttrType]
	set aname [lindex $tmp $colDbName]
	set cstr  [string index $aname 0]

	if {[cequal $cstr "@"]} {
	    # The attribute name already specifies the whole path.
	    set dbName $aname
	} else {
	    set dbName "@$ccseiDb_v(Env)$ccseiDb_v(Cwp).${aname}"
	}

	# This check is done beacuse some database attributes are
	# (VECTORS) are created with 0 data records.

	set dbInfo [ccseiDbAttrInfo  "$dbName" ""]
        if {[lindex $dbInfo 2] == 0 || [lindex $dbInfo 3] == 0} {
	    $wtable rowconfigure $recId -foreground red  ;# Bad quality
	    incr recId +1
	    continue;
	}

	if {[cequal $atype SCALAR]} {
	    ccseiDbUpdateValue $wtable $recId $dbName 1
	} else {

	    # Check if the database name already contains rec/field specification

	    set cidx [string first "(" ${dbName} ]

	    if {$cidx == -1} {
		# Append dummy element just to be able to read the
		# quality flag

		if {[cequal $atype VECTOR]} {
		    ccseiDbUpdateValue $wtable $recId ${dbName}(0)   $displayAllTypes
		} elseif { [cequal $atype TABLE]} {
		    ccseiDbUpdateValue $wtable $recId ${dbName}(0,0) $displayAllTypes
		}

	    } else {
		ccseiDbUpdateValue $wtable $recId ${dbName} $displayAllTypes
	    }
	}

        incr recId +1
    }
}

proc ccseiDbUpdateValue { wtable recId dbName displayAllTypes } {
    # List of global widget names
    global wdgDbTree wdgPtInfo wdgAttrTable wdgPtView ccseiDb_v 

    # List of global names for tablelist columns
    global colDbName colDbAttrType colDbDataType colDbQuality colDbValue

    if {$recId < 0} { return }

    set tmp [ccseiDbReadSymbolic $dbName ]

    # This flag is used when this call is applied to the list of point's attributes
    # In that such a case we read the first element of the attributes in order to
    # get the 'quality flag. The actual values for VECTORS and TABLES are
    # not displayed.
    
    if {$displayAllTypes} {
	$wtable cellconfigure $recId,$colDbValue -text "[lindex $tmp 1]"
    }

    if {[lindex $tmp 0] > 0} {
	$wtable rowconfigure $recId -foreground red  ;# Bad quality
	$wtable cellconfigure $recId,$colDbQuality -text BAD
    } else {
	$wtable rowconfigure $recId -foreground Black
	$wtable cellconfigure $recId,$colDbQuality -text OK
    }

#    puts "DBG> ccseiDbUpdateValue DONE !!"
}

#
# ___oOo___

