#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiLib.tcl,v 3.11 2005/09/14 14:11:15 vltsccm Exp $" 
#
# ccseiLib.tcl - General library for the common part of ccsei
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 25.01.1995 Created
#	     04.02.1995 Added variable to skip dialog-boxes.
# mcomin     21/04/97   Added option -fbin to seq_msgRecvReply
#

#************************************************************************
#   NAME
#	ccseiLib,
#	ccseiPuts, ccseiDialog, ccseiTBD,
#	ccseiMsgRecvReply, ccseiMsgRecvReply
#	- Small wrapper procs for all ccsei applications
#
#   DESCRIPTION
#	Same function as the basic procs, but with additional error-checking
#	and/or debugging options.
#
#	ccseiPuts - put a string to stdout, if debug-option is on
#
#	ccseiDialog - invoke pop-up box with a message and a single OK button
#
#	ccseiTBD - invoke pop-up box to apologize for a non-implemented feature
#		This is done only once for each calling procedure.
#
#	ccseiMsgSendCommand - send a command-message, with debug-option.
#	    Synonyms are automatically translated into Commands before sending.
#
#	ccseiMsgRecvReply - receive a reply-message, with debug-option
#
#   PARAMETERS
#	Same as basic procs.
#
#   RETURNS
#	Same as basic procs.
#
#   GLOBAL VARIABLES
#	ccsei_v(debug)	 - enable flag for debugging outputs, default = off
#	ccsei_v(dialog)	 - enable flag for dialog-boxes, default = on
#	ccsei_v(timeout) - timeout in ms for seq_msgRecvReply, default = 10000
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------


###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see ccsei.tcl
#
###############################################################################


# Variables that can be changed during run-time:
# ---------
if {! [info exists ccsei_v(debug)]} {
    set ccsei_v(debug)	0		;# enable flag for debugging outputs
}
if {! [info exists ccsei_v(dialog)]} {
    set ccsei_v(dialog) 1		;# enable flag for dialog-boxes
}
if {! [info exists ccsei_v(timeout)]} {
    set ccsei_v(timeout) 10000		;# timeout in ms for seq_msgRecvReply
}



###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
#
###############################################################################

proc ccseiDebugControl { {OnOff ""} } {
    global ccsei_v
    if {$OnOff == ""} {set OnOff $ccsei_v(debug)}
    set ccsei_v(debug)  $OnOff	;# ccsei global debug enable flag
    VccDebug::Control $OnOff	;# propagate to vcc debug class
}

proc ccseiPuts { Arg } {
    global ccsei_v
    if {$ccsei_v(debug)} {
	puts $Arg
    }
}

proc ccseiDialog { Message } {
    global ccsei_v
    if {$ccsei_v(dialog)} {
	tk_dialog .dialog "ccsei: Confirm" $Message info 0 OK
    }
}

proc ccseiTBD { {What ""} } {
    global ccseiTBDflag	;# array of What's that have already appeared here

    # if no `What' is given explicitly, determine the calling proc-name:
    if {$What == ""} {
	set What [lindex [info level -1] 0]
    }

    # expose a pop-up window, but only if not already done once before:
    if {! [info exists ccseiTBDflag($What)]} {
	ccseiDialog "Note: $What is not implemented yet"
	set ccseiTBDflag($What) 1
    }
}

proc ccseiMsgSendCommand { Env Pro Com Par } {

    # a synonym may ge given, so first translate the argument into a command:
    set Com [ccseiTranslateSynonym $Pro $Com]

    set cmdId [seq_msgSendCommand $Env $Pro $Com $Par]
    ccseiPuts "seq_msgSendCommand $Env $Pro $Com $Par ==> $cmdId"

    return $cmdId
}

proc ccseiMsgRecvReply { CmdId ErrorFlag LastFlag } {
    global ccsei_v
    upvar $ErrorFlag error
    upvar $LastFlag last

    if {[set t $ccsei_v(timeout)] < 1} {
	set reply [seq_msgRecvReply $CmdId error last -fbin ]
    } else {
	set reply [seq_msgRecvReply $CmdId error last $t -fbin]
    }

    ccseiPuts "seq_msgRecvReply $CmdId $t ==> \"$reply\" err=$error last=$last"
    return $reply
}

proc ccseiMsgEvtAttach { CmdId Script } {
    global ccsei_v
    set evtId [seq_evtAttach $CmdId $ccsei_v(timeout) $Script]
    ccseiPuts "seq_evtAttach $CmdId $ccsei_v(timeout) (...) ==> $evtId"
    return $evtId
}

#_oOo_





