#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMenuLib.tcl,v 3.11 2005/09/14 14:11:16 vltsccm Exp $" 
#
# ccseiMenuLib.tcl - Library for internal menu-management of ccsei
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.01.1995 Created
#	     25.01.1995 Added `-noprefix' option
# S.Sandrock 09.10.1995 Obsolete
#

#************************************************************************
#   NAME
#	- OBSOLETE FILE! -
#	All functions in this file are no longer supported.
#	Use `vccSelectButton' widget instead.
#------------------------------------------------------------------------

#************************************************************************
#   NAME
#	ccseiMenuLib,
#	ccseiMakeMenu - Tcl proc to build a menu from a list representation
#
#   SYNOPSIS
#	ccseiMakeMenu Menu List Callback ?Prefix?
#
#   DESCRIPTION
#	Assembles a menu from the elements of a list.
#	The text of the menu-items correspond to the elements of the list.
#	Leaf-menu-items invoke a callback-procedure when selected, the text
#	of the menu-items is passed to this procedure as parameter.
#	If the number of entries exceed a maximum then a continuation-menu
#	is automatically appended ("more...").
#
#   PARAMETERS
#	Menu	 is the widget-name of the target menu (must already exist).
#		 All entries of this menu - if any - are deleted before.
#		 Any cascaded sub-menus are automatically created.
#
#	List	 contains a list of elements to make the menu from.
#		 If an element is itself a list, then the first element
#		 of this sub-list denotes the text of a cascade-entry, while
#		 the rest of the list denotes the elements of the corresponding
#		 sub-menu.
#
#	Callback is the name of the procedure to be called when a leaf-item of 
#		 the menu is selected. The text of the item is passed as
#		 parameter to the procedure (plus optional Prefix).
#		 Cascade-entries do not have callbacks.
#
#	Prefix	 is prefixed to the parameter of the Callback. This is useful
#		 for cascaded menus and normally only necessary for recursive
#		 calls but not for user calls, except for the
#		 special value `-noprefix', which inhibits the propagation of
#		 callback-parameters to the child-entries.
#		 The argument is optional, default is an empty string.
#
#   RETURNS
#	Total number of leafs incl. in all sub-menus
#
#   GLOBAL VARIABLES
#	Exported: ccsei_c(maxMenuLength)
#	Imported: none
#
#   CAUTIONS
#	All entries of the top-level menu - if any - are deleted.
#	Any already existing sub-menu is destroyed.
#	The menu given as parameter must already exist.
#	Menus having sub-menu should be `destroy'ed before re-generating them,
#	otherwise unused sub-menus may occupy memory.
#	Sub-menus are built by recursive calls of this procedure.
#
#   EXAMPLES
#	set dblist { :PARAMS\
#	    { :SCALARS .scalar_int32 .scalar_float .scalar_string20 }\
#	    { :VECTORS .vector_int32 .vector_float .vector_string20 }\
#	    { :TABLES  .full_table } }
#	set dbmenu [menu .database.button.menu]
#	ccseiMakeMenu $dblist $dbmenu procDbRead
#
#   BUGS
#	Labels of leafs must not contain spaces or other white-space chars.
#	This is due to the problem to recognize if a list-element is a list.
#	How to do that properly?
#
#
#   SEE ALSO
#	ccsei(1),
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#------------------------------------------------------------------------


###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################

# Widget configuration constants:
# (adjust values when needed)
set ccsei_c(maxMenuLength) 25	;# max.number of menu-entries on one level
				;# above that a "more..." sub-menu is appended


# Widget-names of menus to be accessible from procedures:
# The menus are built when needed the first time. Once built they are re-used
# each time they are accessed.
#global ccseiMenuEnv_w()	;# Widget-name: menu of environments(session)
				;# (whatever `session' should stand for).
#global ccseiMenuPro_w()	;# Widget-name: menu of processes(environment),
				;# e.g. ...Pro(lte21)
#global ccseiMenuCom_w()	;# Widget-name: menu of commands(process),
				;# e.g. ...Com(lccServer)
				;# Note: a process has always the same commands
				;#       even on different environments.
#global ccseiMenuPar_w()	;# Widget-name: menu of parameters(proc,com),
				;# e.g. ...Par(lccServer,LCCGVER)
				;# Note: 2-dim arrays are not supported by Tcl.
				;#       The ',' is part of the index-string!



###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################


proc ccseiMakeMenu { Menu List Callback {Prefix ""}} {
    global ccsei_c

    set wMenu $Menu	;# widget-name of the current working menu
    set nLeafs 0	;# total number of leafs in the hierarchy
    set nSubs  0	;# number of sub-menus on the current level
    set nItems 0	;# number of entries in the current menu
    set maxItems $ccsei_c(maxMenuLength)

    $wMenu delete 0 last
    foreach item $List {
	if {$nItems == 0} {
	    # On the top create a cascade-entry for a continuation-sub-menu.
	    # This entry is later removed if a cont-menu is not needed.
	    set moreMenu $wMenu.more	;# new widget-name
	    catch {destroy $moreMenu}	;# destroy it if already existing
	    menu $moreMenu		;# and (re-)create it
	    $wMenu add cascade -label "more ..." -menu $moreMenu
	}
	if {[llength $item] == 1} {
	    # add a normal leaf-entry:
	    set param $item
	    if {$Prefix != "-noprefix"} {set param "$Prefix$item"}
	    $wMenu add command -label "$item" -command [list $Callback $param]
	    incr nLeafs
	} else {
	    # create a sub-menu and connect it to the "more..."-entry:
	    set topLabel [lindex $item 0]	;# first element is the label
	    set subList [lrange $item 1 end]	;# the rest forms the sub-menu
	    set subMenu $wMenu.[incr nSubs]	;# new widget-name
	    catch {destroy $subMenu}	;# destroy it if already existing
	    menu $subMenu		;# and (re-)create it
	    $wMenu add cascade -label $topLabel -menu $subMenu
	    # call myself recursively for the sub-menu:
	    set subPrefix $Prefix
	    if {$Prefix != "-noprefix"} {append subPrefix $topLabel}
	    incr nLeafs [ccseiMakeMenu $subMenu $subList $Callback $subPrefix]
	}
	if {[incr nItems] >= $maxItems} {
	    # create a continuation-sub-menu where all further entries go to:
	    set wMenu $moreMenu	;# from now on use this sub-menu instead
	    set nItems 0	;# re-start counting the entries
	}
    }

    $wMenu delete 0	;# remove the preventively created "more..." again

    return $nLeafs
}



# The following version creates the "more..." entry at the bottom of the menu.
# This is easier to program, but less convenient to use!

proc ccseiMakeMenu1 { Menu List Callback {Prefix ""}} {
    global ccsei_c

    set wMenu $Menu	;# widget-name of the current working menu
    set nLeafs 0	;# total number of leafs in the hierarchy
    set nSubs  0	;# number of sub-menus on the current level
    set nItems 0	;# number of entries in the current menu
    set maxItems $ccsei_c(maxMenuLength)

    $wMenu delete 0 last
    foreach item $List {
	if {[incr nItems] > $maxItems} {
	    # create a continuation-sub-menu where all further entries go to:
	    set moreMenu $wMenu.more	;# new widget-name
	    catch {destroy $moreMenu}	;# destroy it if already existing
	    menu $moreMenu		;# and (re-)create it
	    $wMenu add cascade -label "more ..." -menu $moreMenu
	    set wMenu $moreMenu	;# from now on use this sub-menu instead
	    set nItems 0	;# re-start counting the entries
	}
	if {[llength $item] == 1} {
	    # add a normal leaf-entry:
	    $wMenu add command -label "$item" \
		    -command [list $Callback "$Prefix$item"]
	    incr nLeafs
	} else {
	    # add a cascade-entry and create a sub-menu:
	    set topLabel [lindex $item 0]	;# first element is the label
	    set subList [lrange $item 1 end]	;# the rest forms the sub-menu
	    set subMenu $wMenu.[incr nSubs]	;# new widget-name
	    catch {destroy $subMenu}	;# destroy it if already existing
	    menu $subMenu		;# and (re-)create it
	    $wMenu add cascade -label $topLabel -menu $subMenu
	    # call myself recursively for the sub-menu:
	    incr nLeafs \
		[ccseiMakeMenu $subMenu $subList $Callback $Prefix$topLabel]
	}
    }
    return $nLeafs
}



# simple version without cascaded menus:
proc ccseiMakeMenu2 { Menu List Callback } {
    $Menu delete 0 last
    foreach item $List {
	$Menu add command -label "$item" -command "$Callback \"$item\""
    }
}


#_oOo_
