#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMsgCdt.tcl,v 3.11 2005/09/14 14:11:16 vltsccm Exp $" 
#
# ccseiMsgCdt.tcl - CDT-File-Setup Window of the ccsei Message-Commander
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    21/10/94  Created (cmdTool.tcl)
# S.Sandrock 12.01.1995 extracted parts modified for ccsei
# S.Sandrock 26.06.1995 UIF modified due to changes in CCS/cmd 1.12
# S.Sandrock 12.04.1996 Tk4 porting
#

#
# NOTE: this file is not conform with the naming-rules described in ccsei.tcl
#

#************************************************************************
#   NAME
#
#   SYNOPSIS
#
#   DESCRIPTION
#
#   PARAMETERS
#
#   RETURNS
#
#   GLOBAL VARIABLES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   BUGS
#
#   SEE ALSO
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#------------------------------------------------------------------------


proc ccseiMsgCdtCreateWindow { Root } {
    global ccseiMsg_v ccseiMsgCdt_v ccseiMsgCdt_w

    set ccseiMsgCdt_w(root) $Root
    set w $Root

    catch {destroy $w}
    toplevel $w -class Dialog
    wm title $w "[wm iconname .]: CDT File Setup"
    wm iconname $w "[wm iconname .]: CDT"
    wm iconbitmap $w [wm iconbitmap .]

    if {$ccseiMsgCdt_v(File) == ""} {
	set ccseiMsgCdt_v(File) $ccseiMsg_v(defPro)
    }

    frame $w.fr  -relief raised -border 2
    frame $w.act -relief raised -border 2
    frame $w.act.btn
    frame $w.act.sh 
    pack $w.act.sh $w.act.btn -fill x 
    pack $w.fr $w.act -fill both ;#-expand yes
    #
    # file area
    set ffr [frame $w.fr.file -relief groove -bd 2]
    label $ffr.lbl -text "CDT File:" -anchor w 
    entry $ffr.entry -width 40 -relief sunken -bd 2 \
	    -textvariable ccseiMsgCdt_v(File)
    button $ffr.browse -text "Browse..." -width 8 \
	    -command {set ccseiMsgCdt_v(File) [panSelectFile ../CDT *.cdt]}
    button $ffr.edit -text "Edit..." -width 8 \
	    -command {exec $env(EDITOR) $ccseiMsgCdt_v(File) &}
    #
    #
    set ofr [frame $w.fr.out -relief groove -bd 2]
    label $ofr.lbl -text "Output:" -anchor w
    text  $ofr.text -bd 2 -yscrollcommand "$ofr.yscroll set" \
	    -height 12 -width 60 -relief sunken -bd 2
    set ccseiMsgCdt_w(Text) $ofr.text
    scrollbar $ofr.yscroll -relief sunken -command "$ofr.text yview"
 
    # pack main frames
    pack $ffr $ofr -side top -anchor w -fill x -expand yes \
          -padx 3 -pady 3
    pack $ffr.lbl $ffr.entry $ffr.browse $ffr.edit \
         -side left -padx 4 -pady 4 
    pack $ofr.lbl -side top  -fill x
    pack $ofr.text -side left -expand yes  -fill both
    pack $ofr.yscroll -side right -fill y
    # 
    # Action buttons:
    #
    button $w.act.btn.load -text "Load CDT" -width 10 -command {
	$ccseiMsgCdt_w(Text) delete 1.0 end
	$ccseiMsgCdt_w(Text) insert end \
		"Loading CDT from \"$ccseiMsgCdt_v(File)\" ...\n\n"
	update idletasks
	$ccseiMsgCdt_w(Text) insert end [ccseiCmdLoadCDT $ccseiMsgCdt_v(File)]
	ccseiOptionOnAllLists clear ;# flush caches that may contain old CDTs
    }
    button $w.act.btn.check -text "Check CDT" -width 10 -command {
	$ccseiMsgCdt_w(Text) delete 1.0 end
	$ccseiMsgCdt_w(Text) insert end \
		"Checking CDT in \"$ccseiMsgCdt_v(File)\" ...\n\n"
	update idletasks
	$ccseiMsgCdt_w(Text) insert end [ccseiCmdCheckCDT $ccseiMsgCdt_v(File)]
    }
    button $w.act.btn.list -text "List CDTs" -width 10 -command {
	$ccseiMsgCdt_w(Text) delete 1.0 end; update idletasks
	$ccseiMsgCdt_w(Text) insert end [ccseiCmdListCDTs]
    }
    
    button $w.act.btn.dismiss -text "Dismiss" -width 10 -command "destroy $w"

    pack $w.act.btn.list $w.act.btn.check $w.act.btn.load $w.act.btn.dismiss \
	    -side left -expand yes \
	    -padx 10 -pady 5

    #
    # Add Short Help (first) and all other bindings
    #       
    set shArea $w.act.sh.help 
    pack [UifShortHelp $shArea] -side bottom -fill x -expand 1
    UifShortHelp::add $shArea  $w.act.btn.load \
	    "Load specified CDT and flush internal caches"
    UifShortHelp::add $shArea  $w.act.btn.check \
	    "Check specified CDT for correctness"
    UifShortHelp::add $shArea  $w.act.btn.list \
	    "Show the list of currently loaded CDTs"
    UifShortHelp::add $shArea  $w.act.btn.dismiss \
	    "Close and go back to main panel"
    UifShortHelp::add $shArea  $ffr.entry \
	    "Enter CDT filename for single load or syntax check" 
    UifShortHelp::add $shArea  $ffr.browse \
	    "Filebrowser to select specific CDT file"
    UifShortHelp::add $shArea  $ffr.edit "Call editor for specified CDT file"
    UifShortHelp::add $shArea  $ofr.text "CDT Operations output area"   
    focus $w
}


proc ccseiMsgCdtRefresh { } {
    # nothing to be done.
}

#_oOo_
