#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMsgHelp.tcl,v 3.11 2005/09/14 14:11:16 vltsccm Exp $" 
#
# ccseiMsgHelp.tcl - Help-window of the ccsei Message-Commander
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    21/10/94  Created (cmdTool.tcl)
# S.Sandrock 11.01.1995 extracted parts modified for ccsei
# S.Sandrock 15.10.1995 revision for vcc module
# S.Sandrock 12.04.1996 Tk4 porting
#
#************************************************************************
#   NAME
#	ccseiMsgHelpCreateWindow,
#	ccseiMsgHelpShow,
#	ccseiMsgHelpRefresh
#	- Operation on ccseiMsg help-window
#
#   SYNOPSIS
#	ccseiMsgHelpCreateWindow Root
#	ccseiMsgHelpRefresh
#	ccseiMsgHelpShow Process Command
#
#   DESCRIPTION
#	ccseiMsgHelpCreateWindow - Created the ccsei Msg-Monitor help-window
#		The help-window is created as an independent toplevel.
#		If already existing, it is destroyed and re-generated.
#
#	ccseiMsgHelpShow - display the CDT help-text for a specified command
#		Displays the help for a specific command of a process
#		on the help-window (if it is active).
#
#	ccseiMsgHelpRefresh - refresh the contents of the help-window
#
#   PARAMETERS
#	Root - widget-name for the root-window
#	Process - name of the process on which to show help
#	Command - name of the command on which to show help; may be 
#		  incomplete, in which case all matching commands are show.
#
#   RETURNS
#	None.
#
#   GLOBALS
#	ccseiMsgHelp_w(root)	root for all widgets
#	ccseiMsgHelp_v(Pro)	variable for "process name" entry
#
#   CAUTIONS
#	Several widget-names below the toplevel are referenced directly
#	by the event-handlers below.
#
#   EXAMPLES
#
#------------------------------------------------------------------------

proc ccseiMsgHelpCreateWindow { Root } {
    global ccsei_c ccseiMsgHelp_v ccseiMsgHelp_w

    set ccseiMsgHelp_w(root) $Root	;# name of help window
    set ccseiMsgHelp_v(lastPro) ""	;# name of last displayed process

    # destroy help-window if already exposed:
    set W $Root
    catch {destroy $W}
    set ccseiMsgHelp_v(Pro) ""

    toplevel $W
    wm title $W "[wm iconname .]: Help On Commands"
    wm iconname $W "[wm iconname .]: Help"
    wm iconbitmap $W [wm iconbitmap .]
    wm geometry $W +740+0 ;# pop-up-position

    frame $W.form -width 20c -height 20c

    #
    # Define process area
    #
    frame $W.form.proc            -relief groove -bd 2 
    set ccseiMsgHelp_w(selPro) [\
	    VccSelectorPro $W.form.proc.sel	\
	    -menuFilter SIM			\
	    -command ccseiMsgHelpEvhForSelectPro\
	    -variable ccseiMsgHelp_v(Pro)	\
	    -userMenuEnable 0			\
	    ]
    button $W.form.proc.refresh \
	    -text Refresh -width 10 -command ccseiMsgRefresh

    #
    # Define Command area
    #
    frame     $W.form.cmd            -relief groove -bd 2
    frame     $W.form.cmd.fr 
    label     $W.form.cmd.fr.clbl    -text "Commands:"
    scrollbar $W.form.cmd.fr.yscroll -relief sunken \
	    -command "$W.form.cmd.fr.lb yview"
    listbox   $W.form.cmd.fr.lb      -width 20 -height 10 \
	    -yscroll [list $W.form.cmd.fr.yscroll set] \
	    -relief sunken -setgrid 1
    #@Tk3: tk_listboxSingleSelect $W.form.cmd.fr.lb
    frame     $W.form.cmd.fr2
    label     $W.form.cmd.fr2.plbl    -text "Parameters:" 
    scrollbar $W.form.cmd.fr2.yscroll -relief sunken -command \
	    "$W.form.cmd.fr2.lb yview"
    listbox   $W.form.cmd.fr2.lb -width 20 -height 10 \
	    -yscroll [list $W.form.cmd.fr2.yscroll set] \
	    -relief sunken -setgrid 1
    #@Tk3: tk_listboxSingleSelect $W.form.cmd.fr2.lb
    #
    # Define Param
    #
    frame $W.form.options -relief groove -bd 2 -height 3c
    label $W.form.options.label -text "Not defined yet:" 
    #
    # Define Output area
    #
    frame $W.form.info -relief groove -bd 2 ;#-height 3c
    label $W.form.info.label -text "Parameter Info:" 
    text $W.form.info.text -relief sunken -bd 2 -setgrid 1 \
	    -yscrollcommand "$W.form.info.yscroll set" \
	    -height 31 -width 80 \
	    ;#-background Grey80
    ## -font -adobe-courier-bold-r-normal--14*
    scrollbar $W.form.info.yscroll -relief sunken \
	    -command "$W.form.info.text yview"
    #
    # Help text
    #
    frame $W.form.hlp
    # 
    # PACK 
    #
    pack $W.form -side top -fill both -expand 1
    pack $W.form.proc $W.form.cmd -side top -padx 4 -pady 4 -fill x
    pack $W.form.hlp -side bottom -padx 4 -pady 4 -fill x
    pack $W.form.info -side top -padx 4 -pady 4 -fill both -expand 1
    pack $W.form.proc.sel -side left -fill x
    pack $W.form.proc.refresh -side right
    pack $W.form.cmd.fr $W.form.cmd.fr2 -side left -padx 4 -fill x -expand 1
    #pack $W.form.cmd.fr.yscroll  -side right -fill y
    pack $W.form.cmd.fr.clbl -side top -anchor w
    pack $W.form.cmd.fr.yscroll -side left -fill y
    pack $W.form.cmd.fr.lb -side left -fill x -expand 1
    pack $W.form.cmd.fr2.plbl  -side top -anchor w
    pack $W.form.cmd.fr2.yscroll -side left -fill y
    pack $W.form.cmd.fr2.lb -side left -fill x -expand 1
    #!pack $W.form.options.label -side top -anchor w
    pack $W.form.info.label -side top -anchor w
    pack $W.form.info.yscroll -side left -fill y
    pack $W.form.info.text -side left -fill both -expand 1
    #
    # ADD SHORT HELP
    #   
    set shArea $W.form.hlp.sh
    pack [UifShortHelp $shArea] -side bottom -fill x -expand 1
    UifShortHelp::add $shArea $W.form.proc.sel \
	    "Selector for Processes with loaded CDTs"
    UifShortHelp::add $shArea  $W.form.proc.refresh \
	    "Refresh panel"
    UifShortHelp::add $shArea  $W.form.cmd.fr.lb \
	    "Click to display information, Double-click to copy to command-box"
    UifShortHelp::add $shArea  $W.form.cmd.fr2.lb \
	    "List of parameter names, Double-click to copy to command-box"
    UifShortHelp::add $shArea  $W.form.info.text "Detailed CDT Information"
    #
    # BINDINGS
    #
    # commands box:
    bind $W.form.cmd.fr.lb <ButtonRelease-1> \
	    { ccseiMsgHelpEvhForSelectCom [selection get] }
    bind $W.form.cmd.fr.lb <Double-1> \
	    { $ccseiMsg_w(boxCom) putField Com [selection get] }
    # parameters box:
    bind $W.form.cmd.fr2.lb <ButtonRelease-1> \
	    { ccseiMsgHelpEvhForSelectPar [selection get] }
    bind $W.form.cmd.fr2.lb <Double-1> \
	    { $ccseiMsg_w(boxCom) appendField Par " -[selection get] " }

    # do an initial refresh of the window:
    ccseiMsgHelpRefresh
}


proc ccseiMsgHelpRefresh { } {
    global ccseiMsg_v ccseiMsgHelp_v ccseiMsgHelp_w

    set ccseiMsgHelp_v(lastPro) ""	;# clear name of last displayed process

    # query list of loaded CDTs and attach them to the menu:
    if {[winfo exists $ccseiMsgHelp_w(root)]} {
	$ccseiMsgHelp_w(selPro) MakeMenu [ccseiGetListOfLoadedCDTs]
	ccseiMsgHelpShow $ccseiMsg_v(defPro) ""
    }
}


proc ccseiMsgHelpShow { Process Command } {
    global ccseiMsgHelp_w
    if {[winfo exists $ccseiMsgHelp_w(root)]} {
	# this should execute in background because it can take a second,
	# but tests with `after' failed so far...
	if {$Process != ""} {ccseiMsgHelpEvhForSelectPro $Process}
	if {$Command != ""} {ccseiMsgHelpEvhForSelectCom $Command}
    }
}



#************************************************************************
#   NAME
#	ccseiMsgHelpEvhForSelectPro, ccseiMsgHelpEvhForSelectCom
#	- event handlers for selection of process and command
# 
#   DESCRIPTION
#	These callback-procedures are invoked when an item is selected in the
#	corresponding menu resp. when <Return> is pressed in the entry.
#
#   PARAMETERS
#	The text of the selected entry is passed to the procedure
#
#   RETURNS
#	None
#
#   GLOBALS
#	ccseiMsgHelp_v()		current selection
#	ccseiMsgHelp_w()		widget-names of the menus
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------

proc ccseiMsgHelpEvhForSelectPro { Process } {
    global ccseiMsgHelp_v ccseiMsgHelp_w
    set ccseiMsgHelp_v(Pro) $Process
    # for speed reasons only do something if the selection has changed:
    if {$Process != "" && $Process != $ccseiMsgHelp_v(lastPro)} {
	# query the list of commands and put them into the listbox:
	set ccseiMsgHelp_v(lastPro) $Process
	set comlist [ccseiGetListOfCommands $Process]
	set W $ccseiMsgHelp_w(root).form.cmd.fr
	$W.clbl configure -text "[llength $comlist] Commands:"
	$W.lb delete 0 end
	foreach i $comlist {$W.lb insert end $i}
    }
    return $Process ;# necessary for VccSelector to return proper value
}

## problem when for a "sole but not unique" completion:
#... to be solved
proc ccseiMsgHelpEvhForSelectCom { Command } {
    global ccseiMsgHelp_v ccseiMsgHelp_w ccsei_w

    # if accidentally no process is given, return:
    if {$ccseiMsgHelp_v(Pro) == ""} return

    # first check if the Command is complete (not necessarily unique).
    # if yes, then display the parameters and the help-text,
    # if not, then display all the matching commands.
    set comlist [$ccsei_w(complCom) Completion all $Command]
    set n [llength $comlist]
    if {$n == 1 || [lsearch $comlist $Command] >= 0} {
	set W $ccseiMsgHelp_w(root).form
	set parlist [ccseiGetListOfParameters $ccseiMsgHelp_v(Pro) $Command ]
	$W.cmd.fr2.plbl configure -text "[llength $parlist] Parameters:"
	$W.cmd.fr2.lb delete 0 end
	foreach i $parlist {
	    $W.cmd.fr2.lb insert end $i
	}
	#    cmdRetValues

	$W.info.text delete 1.0 end
	$W.info.label configure -text \
		"$ccseiMsgHelp_v(Pro)/$Command Parameter Info:"
	set helptext [ccseiGetHelpText $ccseiMsgHelp_v(Pro) $Command]
	$W.info.text insert end $helptext
    } elseif {$n > 1} {
	# put the list of matching commands  into the listbox:
	set W $ccseiMsgHelp_w(root).form.cmd.fr
	$W.clbl configure -text "[llength $comlist] Commands/Matches:"
	$W.lb delete 0 end
	foreach i $comlist {$W.lb insert end $i}
    }
}
# slightly different realisation. Don't know yet which is better.
proc ccseiMsgHelpEvhForSelectCom_1 { Command } {
    global ccseiMsgHelp_v ccseiMsgHelp_w ccsei_w

    # first check if the Command is known, i.e. it's a sole completion;
    # if yes, then display the parameters and the help-text,
    # if not, then display all the matching commands.
    set parlist [ccseiGetListOfParameters $ccseiMsgHelp_v(Pro) $Command ]
    if {$parlist != -1} {
	set W $ccseiMsgHelp_w(root).form
	$W.cmd.fr2.plbl configure -text "[llength $parlist] Parameters:"
	$W.cmd.fr2.lb delete 0 end
	foreach i $parlist {
	    $W.cmd.fr2.lb insert end $i
	}
	#    cmdRetValues

	$W.info.text delete 1.0 end
	$W.info.label configure -text "$Command Parameter Info:"
	set helptext [ccseiGetHelpText $ccseiMsgHelp_v(Pro) $Command]
	$W.info.text insert end $helptext
    } else {
	set comlist [$ccsei_w(complCom) Completion all $Command]
	set n [llength $comlist]
	if {$n >= 1} {
	    # put the list of matching commands  into the listbox:
	    set W $ccseiMsgHelp_w(root).form.cmd.fr
	    $W.clbl configure -text "[llength $comlist] Matches:"
	    $W.lb delete 0 end
	    foreach i $comlist {$W.lb insert end $i}
	}
    }
}


proc ccseiMsgHelpEvhForSelectPar { Parameter } {
    # TBD
    # scroll the text with selected parameter on top:
    # $W.form.info.text yview ...
}

#_oOo_
