#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMsgRmon.tcl,v 3.11 2005/09/14 14:11:17 vltsccm Exp $" 
#
# ccseiMsgRmon.tcl - Reply-Monitor-Window of the ccsei Message-Commander
#
# who        when       what
# ---------  ---------- ----------------------------------------------
# S.Sandrock 20.01.1995 Created
# S.Sandrock 07.10.1996 Extended for error-stack
# mcomin     03/11/97   Adjust geomwtry for laptop
#            21/10/98   Re-added external window where to see the reply.
#************************************************************************
#   NAME
#	ccseiMsgRmonCreateWindow,
#	ccseiMsgRmonMonitor,
#	ccseiMsgRmonShowReply,
#	ccseiMsgRmonRefresh
#	- Operation on ccseiMsg Rmon-window
#
#   SYNOPSIS
#	ccseiMsgRmonCreateWindow Root
#	ccseiMsgRmonMonitor
#	ccseiMsgRmonRefresh
#	ccseiMsgRmonShowReply Prefix Reply
#
#   DESCRIPTION
#	ccseiMsgRmonCreateWindow - Created the ccseiMsg Reply-Monitor window
#		The Rmon-window is created as an independent toplevel.
#		If already existing, it is destroyed and re-generated.
#
#	ccseiMsgRmonShowReply - display an reply in the "last reply" box
#		on the Rmon-window (if it is active).
#
#	ccseiMsgRmonRefresh - refresh the contents of the Rmon-window
#
#	ccseiMsgRmonMonitor - performs a continous refresh every second as
#		background process.
#
#   PARAMETERS
#	Root - widget-name for the root-window
#	Prefix - ident of the reply, any string e.g. "REPLY 2/45"
#	Reply  - the reply text
#
#   RETURNS
#	None.
#
#   GLOBALS
#	ccseiMsgRmon_w()
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------

proc ccseiMsgRmonSetNames {} {
    global ccseiMsgRmon_v ccseiMsgRmon_w cv

    # only raise if window is already there:

    # variables:
    set ccseiMsgRmon_v(prefix) "Reply/Error Display"
    set ccseiMsgRmon_v(reply) ""
    set ccseiMsgRmon_v(stack) ""
    set ccseiMsgRmon_v(queue) ""

    # widgets:
    set ccseiMsgRmon_w(root) .extRep
    set ccseiMsgRmon_w(boxQue)  $ccseiMsgRmon_w(oldBoxQue) 
    set ccseiMsgRmon_w(boxStk)  $ccseiMsgRmon_w(oldBoxStk)
}

proc ccseiMsgRmonExtWindow {} {
    global ccseiMsgRmon_v ccseiMsgRmon_w

    # only raise if window is already there:
    if {[winfo exists $ccseiMsgRmon_w(root)]} {
	raise $ccseiMsgRmon_w(root)
	return
    }

    # variables:
    set ccseiMsgRmon_v(prefix) "Reply/Error Display"
    set ccseiMsgRmon_v(reply) ""
    set ccseiMsgRmon_v(stack) ""
    set ccseiMsgRmon_v(queue) ""

    # Clear Error stack on main window
    $ccseiMsgRmon_w(boxStk) delete 1.0 end

    # destroy Rmon-window if already exposed:
    set W $ccseiMsgRmon_w(root)
    catch {destroy $W}

    toplevel $W
    wm title $W "[wm iconname .]: Reply Monitor"
    wm iconname $W "[wm iconname .]: Rmon"
    wm iconbitmap $W [wm iconbitmap .]
    wm minsize $W 400 150
    wm geometry $W 700x200+20+500

    set W1 [frame $W.rpl -relief groove -bd 2]
    frame $W1.header
    label $W1.header.label -textvariable ccseiMsgRmon_v(prefix)
    set ccseiMsgRmon_w(boxRpl) [text $W1.rpl -height 2 -relief sunken -bd 2]
    set ccseiMsgRmon_w(boxStk) [text $W1.stk -height 2 -relief sunken -bd 2]
    set ccseiMsgRmon_w(boxQue) [text $W1.que -height 2 -relief sunken -bd 2]

    pack $W1 -expand yes -fill both -padx 1m -pady 1m
    pack $W1.header -fill x
    pack $W1.header.label  -side left  -anchor w
    pack $ccseiMsgRmon_w(boxRpl) -expand yes -fill both
    pack $ccseiMsgRmon_w(boxStk) -expand no -fill both
    pack $ccseiMsgRmon_w(boxQue) -expand no -fill both

    # do an initial refresh of the window:
#    ccseiMsgRmonMonitor

    # Set proper binding to re-set some variables and continue to use
    # the main panel.

    bind $W1 <Destroy> ccseiSetRmonVariables
}

proc ccseiSetRmonVariables {} {
   after cancel ccseiMsgRmonMonitor
   ccseiMsgRmonSetNames
   ccseiMsgRmonMonitor
}

proc ccseiMsgRmonMonitor { } {
    ccseiMsgRmonRefresh
    after 2000 ccseiMsgRmonMonitor
}


proc ccseiMsgRmonRefresh { } {
    global ccseiMsgRmon_v ccseiMsgRmon_w

    # display reply on separate window

    if {[winfo exists $ccseiMsgRmon_w(root)]} {
       $ccseiMsgRmon_w(boxRpl) delete 1.0 end
       $ccseiMsgRmon_w(boxRpl) insert 1.0 $ccseiMsgRmon_v(reply)
    }

    # display error-stack:

    if {[winfo exists $ccseiMsgRmon_w(boxStk)]} {

       $ccseiMsgRmon_w(boxStk) delete 1.0 end

       if {[winfo exists $ccseiMsgRmon_w(root)]} {
          set size [expr {2 * [llength $ccseiMsgRmon_v(stack)] + 1}]
          $ccseiMsgRmon_w(boxStk) configure -height $size
       }

       if {[lempty $ccseiMsgRmon_v(stack)]} {
	   set ccseiMsgRmon_v(stack) ""
           $ccseiMsgRmon_w(boxStk) insert end "No error-stack"
       } else {
           foreach line $ccseiMsgRmon_v(stack) {
	      set text [lvarpop line end] ;# separate the error-text
              $ccseiMsgRmon_w(boxStk) insert end ${line}\n\t${text}\n
           }
       }
    }

    # display pending replies:
    set ccseiMsgRmon_v(queue) [seq_msgList]

    if {[lempty $ccseiMsgRmon_v(queue)]} {
	set ccseiMsgRmon_v(queue) {"No pending replies on commands"}
    }

    if {[winfo exists $ccseiMsgRmon_w(root)]} {
        $ccseiMsgRmon_w(boxQue) delete 0.0 end
        foreach i $ccseiMsgRmon_v(queue) {
	   $ccseiMsgRmon_w(boxQue) insert end "$i\n"
        }
    }

    $ccseiMsgRmon_w(oldBoxQue) deleteItem 0 end
    foreach i $ccseiMsgRmon_v(queue) {
	   $ccseiMsgRmon_w(oldBoxQue) addItem end $i
    }
}


proc ccseiMsgRmonShowReply { Prefix Reply Stack } {
    global ccseiMsgRmon_v ccseiMsgRmon_w

    set ccseiMsgRmon_v(prefix) $Prefix
    set ccseiMsgRmon_v(reply) $Reply
    set ccseiMsgRmon_v(stack) $Stack

    ccseiMsgRmonRefresh

    if {[winfo exists $ccseiMsgRmon_w(root)]} {
       raise $ccseiMsgRmon_w(root)
    }
}



#_oOo_
