#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMsgSim.tcl,v 3.11 2005/09/14 14:11:17 vltsccm Exp $" 
#
# ccseiMsgSim.tcl - Simulation-Setup Window of the ccsei Message-Commander
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    21/10/94  Created (cmdTool.tcl)
# S.Sandrock 23.01.1995 extracted parts modified for ccsei
# S.Sandrock 15.10.1995 major revision for vcc module
# S.Sandrock 12.04.1996 Tk4 porting
#


#
# NOTE: this file is not conform with the naming-rules described in ccsei.tcl
#

#************************************************************************
#   NAME
#
#   SYNOPSIS
#
#   DESCRIPTION
#
#   PARAMETERS
#
#   RETURNS
#
#   GLOBAL VARIABLES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   BUGS
#
#   SEE ALSO
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#------------------------------------------------------------------------


proc ccseiMsgSimCreateWindow { Root } {
    ##    global command procname simenv replyvalue dbpoint simdlg
    global ccseiMsgSim_v ccseiMsgSim_w ccseiMsg_v

    set ccseiMsgSim_w(root) $Root

    # take the default selections from the main-window:
    set ccseiMsgSim_v(Env) $ccseiMsg_v(defEnv)
    set ccseiMsgSim_v(Pro) $ccseiMsg_v(defPro)
    set ccseiMsgSim_v(Com) $ccseiMsg_v(defCom)
    set ccseiMsgSim_v(Par) $ccseiMsg_v(defPar)

    set w $Root

    catch {destroy $w}
    toplevel $w -class Dialog
    wm title $w "[wm iconname .]: Simulation Setup"
    wm iconname $w "[wm iconname .]: Sim"
    wm iconbitmap $w [wm iconbitmap .]
    
    frame $w.fr  -relief raised -border 1
    frame $w.act -relief raised -border 2
    frame $w.act.sh
    frame $w.act.btn 
        
    pack $w.fr $w.act -fill both ;#-expand yes
    pack $w.act.sh $w.act.btn -fill x
    
    # simulation frame
    frame $w.fr.sim  -relief groove -bd 2 
    # entry area

    # Environment Selector:
    set ccseiMsgSim_w(selEnv) [\
	    VccSelector $w.fr.sim.selEnv \
	    -labelText Environment: -entryWidth 8 -labelWidth 14 \
	    -command ccseiMsgSimEvhSelectEnv \
	    -variable ccseiMsgSim_v(Env) \
	    -userMenuEnable 1 \
	    ]

    # Process Selector:
    set ccseiMsgSim_w(selPro) [\
	    VccSelector $w.fr.sim.selPro \
	    -labelText Process: -entryWidth 20 -labelWidth 14 \
	    -command ccseiMsgSimEvhSelectPro \
	    -variable ccseiMsgSim_v(Pro) \
	    -userMenuEnable 1 \
	    ]
    set ccseiMsgSim_w(menuPro) [$ccseiMsgSim_w(selPro) MakeMenu {}]

    # Command Selector:
    set ccseiMsgSim_w(selCom) [\
	    VccSelector $w.fr.sim.selCom \
	    -labelText Command: -entryWidth 60 -labelWidth 14 \
	    -command ccseiMsgSimEvhSelectCom \
	    -variable ccseiMsgSim_v(Com) \
	    -menuType command \
	    ]

    # Parameter Selector:
    set ccseiMsgSim_w(selPar) [\
	    VccSelector $w.fr.sim.selPar \
	    -labelText Parameters: -entryWidth 60 -labelWidth 14 \
	    -command ccseiMsgSimEvhSelectPar \
	    -variable ccseiMsgSim_v(Par) \
	    -menuType command \
	    ]

    #
    # Mode area
    set mfr [frame $w.fr.mode -relief groove -bd 2]
    label $mfr.lbl -text "Process Mode:" -anchor e -width 14
#    radiobutton $mfr.n -text "NORMAL" -variable ccsstate -value NORMAL \
#	    -command { set env(CCSSTATE) $ccsstate }
#    radiobutton $mfr.s    -text "SIMULATION" -variable ccsstate \
#	    -value SIMULATION -command { set env(CCSSTATE) $ccsstate }
    radiobutton $mfr.n -text "NORMAL" \
	    -variable ccseiMsgSim_v(Mode) -value NORMAL \
	    -command {
	ccseiCmdSetNormalMode $ccseiMsgSim_v(Pro)
	ccseiMsgModifyProcessMenu $ccseiMsgSim_w(menuPro)
    }
    radiobutton $mfr.s -text "SIMULATION" \
	    -variable ccseiMsgSim_v(Mode) -value SIMULATION \
	    -command {
	ccseiCmdSetSimulationMode $ccseiMsgSim_v(Pro) 
	ccseiMsgSimRefresh
    }
    label $mfr.gslbl -text "CCSSTATE:"
    #label $mfr.state -textvariable env(CCSSTATE) -foreground Red
    set ccseiMsgSim_w(optState) [\
	    UifOptionButton $mfr.state -length 12 \
	    -variable env(CCSSTATE) -itemList {NORMAL SIMULATION} \
	    -command ccseiMsgSimEvhCCSSTATE \
	    ]
    button $mfr.refresh -text Refresh -width 8 -command ccseiMsgRefresh

    #
    # Reply area
    #
    set rfr [frame $w.fr.reply -relief groove -bd 2]
    set ccseiMsgSim_w(selRep) [\
	    VccSelector $rfr.1 \
	    -labelText "Reply Par Name:" -entryWidth 60 -labelWidth 14 \
	    -command ccseiMsgSimEvhSelectReplyPar \
	    -variable ccseiMsgSim_v(replyPar) \
	    -entryState disabled \
	    ]
    frame $rfr.2
    frame $rfr.3

    label $rfr.2.lbl -text "Reply Par Value:" -anchor e -width 14
    label $rfr.2.out -relief sunken -bd 2 \
	    -textvariable ccseiMsgSim_v(replyVal) -width 62 -anchor w
    label $rfr.3.lbl -text "DB Ref Value:" -anchor e -width 14
    set ccseiMsgSim_w(entryDbVal) $rfr.3.entry
    entry $ccseiMsgSim_w(entryDbVal) -relief sunken -bd 2 \
	    -textvariable ccseiMsgSim_v(replyDbVal) -width 62

    #
    # Output area
    frame $w.fr.out -relief groove -bd 2
    label $w.fr.out.lbl -text "Output:" -anchor w
    set ofr [frame $w.fr.out.fr]
    text  $ofr.text -bd 2 -yscrollcommand "$ofr.yscroll set" \
	    -height 10 -width 60 -relief sunken -bd 2
#!	    -font -adobe-courier-bold-r-normal--14*
    scrollbar $ofr.yscroll -relief sunken -command "$ofr.text yview" 
    #
    # pack widgets in simulation frame
    # pack main frames
    pack $w.fr.sim $mfr $w.fr.reply $w.fr.out \
	    -side top -anchor w -fill x -expand yes -padx 3 -pady 3
    pack    $ccseiMsgSim_w(selEnv) \
	    $ccseiMsgSim_w(selPro) \
	    $ccseiMsgSim_w(selCom) \
	    $ccseiMsgSim_w(selPar) \
	    -side top -anchor w -expand yes

    pack $rfr -side top -anchor w -expand yes -padx 3 -pady 3
    pack $rfr.1 $rfr.2 $rfr.3 -side top -expand yes
    pack $rfr.2.lbl $rfr.2.out -side left -padx 2 -pady 2
    pack $rfr.3.lbl $rfr.3.entry -side left -padx 2 -pady 2

    pack $mfr.lbl $mfr.n $mfr.s -side left
    pack $mfr.gslbl $mfr.state $mfr.refresh -side left -padx 3 -pady 3
    pack $w.fr.out.lbl $ofr -side top -fill x
    pack $ofr.yscroll -side right -fill y
    pack $ofr.text -side left -expand yes -fill both
    
    # 
    # Dialog buttons
    #
    label $w.act.btn.swLabel -text "msgSend Options:" -anchor w
    entry $w.act.btn.swEntry -width 10 -relief sunken -bd 2 \
	    -textvariable ccseiMsgSim_v(msgSendOptions)
    button $w.act.btn.sim -text "Send Command" \
	    -command "ccseiMsgSimSend $ofr.text"
    button $w.act.btn.dismiss -text "Dismiss" -width 10 -command  "destroy $w"
      
    button $w.act.btn.clear -text "Clear" -width 10 -command {
	set ccseiMsgSim_v(Par) ""
	$ccseiMsgSim_w(root).fr.out.fr.text delete 1.0 end
    }
    pack $w.act.btn.swLabel $w.act.btn.swEntry -side left
    pack $w.act.btn.sim $w.act.btn.clear -side left  -padx 10 -pady 5
    pack $w.act.btn.dismiss -side right -padx 10 -pady 5
    #
    # Select reply  type 
    ccseiMsgSimSetupReply 
#!    if { $ccseiMsgSim_v(DBref) != "" } { $rfr.db select }
#!    if { $ccseiMsgSim_v(Reply) != "" } { $rfr.ascii select }

    #
    # Add Short Help (first) and all other bindings
    #   
    
    set shArea $w.act.sh.help 
    pack [UifShortHelp $shArea] -side bottom -fill x -expand 1
    UifShortHelp::add $shArea  $w.act.btn.sim	\
	    "Send selected Command for test, using the utility msgSend"
    UifShortHelp::add $shArea  $w.act.btn.dismiss \
	    "Dismiss Simulation Setup Window"
    UifShortHelp::add $shArea  $w.act.btn.clear "Clear input/output fields"

    UifShortHelp::add $shArea $ccseiMsgSim_w(selEnv) \
	    "Simulation Environment Selector"
    UifShortHelp::add $shArea $ccseiMsgSim_w(selPro) \
	    "Simulation Process Selector"
    UifShortHelp::add $shArea $ccseiMsgSim_w(selCom) \
	    "Simulation Command Selector"
    UifShortHelp::add $shArea $ccseiMsgSim_w(selPar) \
	    "Simulation Parameter Selector"

    UifShortHelp::add $shArea  $mfr.n	\
	"Process is registered for Normal Mode - click to change selection"
    UifShortHelp::add $shArea  $mfr.s	\
	"Process is registered for Simulation Mode - click to change selection"
    UifShortHelp::add $shArea  $mfr.state	\
	    "Activate or disable Simulation Mode"
    UifShortHelp::add $shArea  $mfr.refresh	\
	    "Refresh Panels"

    UifShortHelp::add $shArea  $rfr.1 \
	    "Selector of formal name of reply-parameter"
    UifShortHelp::add $shArea  $rfr.2.out	\
	    "Default value defined in CDT for selected reply-parameter"
    UifShortHelp::add $shArea  $rfr.3.entry	\
	    "Value of connected DB attribute. Type <Enter> to write value"
    UifShortHelp::add $shArea  $ofr.text      "msgSend Output area"
    #
    # Bindings
    bind $ccseiMsgSim_w(entryDbVal) <Return> {
	if { $ccseiMsgSim_v(replyVal) != "" } {
	    seq_dbWriteSymbolic $ccseiMsgSim_v(replyVal) \
		    [seq_dbGetAttrInfo  $ccseiMsgSim_v(replyVal)] \
		    "$ccseiMsgSim_v(replyDbVal)"
	    set ccseiMsgSim_v(replyDbVal) \
		    [seq_dbReadSymbolic $ccseiMsgSim_v(replyVal)]
	}
    }

    # initial refresh:
    ccseiMsgSimRefresh
}


proc ccseiMsgSimRefresh { args } {
    global ccseiMsg_c ccseiMsgSim_v ccseiMsgSim_w env
    if {[winfo exists $ccseiMsgSim_w(root)]} {
	ccseiMsgSimEvhSelectSes A
	# re-invoke all default selections:
	foreach tag $ccseiMsg_c(tags) {
	    if {$ccseiMsgSim_v($tag) != ""} {
		ccseiMsgSimEvhSelect$tag $ccseiMsgSim_v($tag)
	    }
	}
    }
}


#************************************************************************
#   NAME
#	ccseiMsgSimEvhSelect... - Event-handlers for simulation selections
# 
#   DESCRIPTION
#	These callback-procedures are invoked when an item is selected in the
#	corresponding menu resp. when <Return> is pressed in the entry.
#	(ccseiMsgSimEvhSelectSes is mainly for "symmetry" reasons, no menu 
#	 is associated with it.)
#
#   PARAMETERS
#	The text of the selected menu-item is passed to the procedure
#
#   RETURNS
#	None
#
#   GLOBALS
#	ccseiMsgSim_v(X)	simulation selection
#	ccseiMsgSim_w(menuX)	widget-names of the menus
#				where X corresponds to Env/Pro/Com/Par
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------
proc ccseiMsgSimEvhSelectSes { Session } {
    global ccseiMsgSim_v ccseiMsgSim_w
    set ccseiMsgSim_v(Ses) $Session ;# save selected value in a global variable
    set envlist [ccseiGetListOfEnvironments $ccseiMsgSim_v(Ses)]
    $ccseiMsgSim_w(selEnv) MakeMenu $envlist
    return $Session ;# necessary for VccSelector to return proper value
}

proc ccseiMsgSimEvhSelectEnv { Environment } {
    global ccseiMsgSim_v ccseiMsgSim_w
    set ccseiMsgSim_v(Env) $Environment ;# save selected value in a global var.
    $ccseiMsgSim_w(selPro) MakeMenu [ccseiGetListOfProcesses $Environment]
    ccseiMsgModifyProcessMenu $ccseiMsgSim_w(menuPro) ;# mark simulated Pro's
    return $Environment ;# necessary for VccSelector to return proper value
}

proc ccseiMsgSimEvhSelectPro { Process } {
    global ccseiMsgSim_v ccseiMsgSim_w
    set ccseiMsgSim_v(Pro) $Process ;# save selected value in a global variable
    $ccseiMsgSim_w(selCom) MakeMenu [ccseiGetListOfCommands $Process]
    ccseiMsgHelpShow $Process "" ;# show help for all commands of this process

    # determine if this Process is in the list of simulated processes:
    if {[lsearch [ccseiGetListOfProcessesInSimulation] $Process] == -1} {
	set ccseiMsgSim_v(Mode) NORMAL
    } else {
	set ccseiMsgSim_v(Mode) SIMULATION
    }

    return $Process ;# necessary for VccSelector to return proper value
}

proc ccseiMsgSimEvhSelectCom { Command } {
    global ccseiMsgSim_v ccseiMsgSim_w
    set ccseiMsgSim_v(Com) $Command ;# save selected value in a global variable
    set valtree [ccseiGetTreeOfParamValues $ccseiMsgSim_v(Pro) $Command]
    $ccseiMsgSim_w(selPar) MakeMenu $valtree

    ccseiMsgHelpShow $ccseiMsgSim_v(Pro) $Command ;# show help for this command

    ccseiMsgSimSetupReply ;# query the replyParList and its values
    $ccseiMsgSim_w(selRep) MakeMenu $ccseiMsgSim_v(replyParList)
    return $Command ;# necessary for VccSelector to return proper value
}

proc ccseiMsgSimEvhSelectPar { Parameter } {
    global ccseiMsgSim_v ccseiMsgSim_w
    set ccseiMsgSim_v(Par) $Parameter ;# save selected value in a global var.
    return $Parameter ;# necessary for VccSelector to return proper value
}


proc ccseiMsgSimEvhSelectReplyPar { ReplyParameter } {
    global ccseiMsgSim_v ccseiMsgSim_w

    if {$ReplyParameter == ""} {
	set ccseiMsgSim_v(replyPar) "<none>"
	set ccseiMsgSim_v(replyVal) "<none>"
	set ccseiMsgSim_v(replyDbVal) "<none>"
	$ccseiMsgSim_w(entryDbVal) configure -state disabled
    } else {
	set ccseiMsgSim_v(replyPar) $ReplyParameter ;# save selected value
	set ccseiMsgSim_v(replyVal) $ccseiMsgSim_v(replyVal,$ReplyParameter)

	set value $ccseiMsgSim_v(replyVal)
	# check if it is a DB reference:
	if { [regexp {^[ ]*:.+|^[ ]*@.+|^[ ]*<.+} [lindex $value 0]+] == 1 } {
	    # yes, read the value and enable the entry for change:
	    $ccseiMsgSim_w(entryDbVal) configure -state normal
	    set ccseiMsgSim_v(replyDbVal) [seq_dbReadSymbolic $value]
	} else {
	    # no, disable the entry:
	    $ccseiMsgSim_w(entryDbVal) configure -state disabled
	    set ccseiMsgSim_v(replyDbVal) "<not applicable>"
	}
    }
    return $ReplyParameter ;# necessary for VccSelector to return proper value
}


proc ccseiMsgSimEvhCCSSTATE { State } {
    global env ccseiMsg_v
    set env(CCSSTATE) $State	;# NORMAL or SIMULATION
    set ccseiMsg_v(simMode) [expr {$env(CCSSTATE) == "SIMULATION"}]
    ccseiMsgRefresh
    return $State
}


############################################################
#                  PROCEDURE    DEFINITION                 #
############################################################

proc ccseiMsgSimSetupReply {} {
    global env ccseiMsgSim_v ccseiMsgSim_w

    set ccseiMsgSim_v(replyParList) {}

    # get return values
    if { $ccseiMsgSim_v(Pro) == "" || $ccseiMsgSim_v(Com) == "" } {
	set ccseiMsgSim_v(replyPar) ""
	set ccseiMsgSim_v(replyVal) ""
	set ccseiMsgSim_v(replyDbVal) ""
    } else {
	set parlist [ccseiCmdQueryReplyParams \
		$ccseiMsgSim_v(Pro) $ccseiMsgSim_v(Com)]

	foreach typlist $parlist {
	    set parname  [lindex $typlist 0]
	    set parvalue [lindex $typlist 4]
	    set ccseiMsgSim_v(replyVal,$parname) $parvalue
	    lappend ccseiMsgSim_v(replyParList) $parname
	}

	if {[llength $ccseiMsgSim_v(replyParList)] > 0} {
	   ccseiMsgSimEvhSelectReplyPar [lindex $ccseiMsgSim_v(replyParList) 0]
	} else {
	    ccseiMsgSimEvhSelectReplyPar ""
	}
    }
}



proc ccseiMsgSimSend { w } {
    global env ccseiMsgSim_v

    # get the list of Prosesses which are selected for Simulation Mode:
    set simlist [ccseiGetListOfProcessesInSimulation]

    set f_simModeEnabled 0
    if {[info exists env(CCSSTATE)]} {
	if {$env(CCSSTATE) == "SIMULATION"} {
	    set f_simModeEnabled 1
	}
    }
    set f_simModeProcess \
	[expr {[lsearch $simlist $ccseiMsgSim_v(Pro)] != -1}]

    # If Simulation Mode is enabled,
    # check if Process is also selected for Simulation Mode:
#    if {$f_simModeEnabled && ! $f_simModeProcess} {
#	ccseiDialog "Simulation Mode is enabled, but Process\
#		\"$ccseiMsgSim_v(Pro)\" is not registered for Simulation!"
#	return
#    }

    # If Simulation Mode is not enabled,
    # check if Process is also not selected for Simulation Mode:
#    if {! $f_simModeEnabled && $f_simModeProcess} {
#	ccseiDialog "Process \"$ccseiMsgSim_v(Pro)\" is registered for\
#		Simulation, but Simulation Mode is not enabled!"
#	return
#    }

    # protect quoted strings 
    #regsub -all \" $ccseiMsgSim_v(Par) \\\" sendparam

    # the given Command may be a Synonym, so translate it before:
    set command [ccseiTranslateSynonym $ccseiMsgSim_v(Pro) $ccseiMsgSim_v(Com)]

    # assemble the command-line:
    set cmd "msgSend $ccseiMsgSim_v(msgSendOptions) $ccseiMsgSim_v(Env)\
	    $ccseiMsgSim_v(Pro) $command \"$ccseiMsgSim_v(Par)\""

    # execute the command-line and display the result:
    $w delete 1.0 end
    $w insert 1.0 "$cmd\n"
    $w insert 2.0 "EXECUTING ..."
    update ;# display it so far
    catch {exec sh -c $cmd} outp
    $w delete 2.0 end
    $w insert 2.0 $outp
}


#_oOo_
