#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ctooConfigArchive.tcl,v 1.52 2005/09/06 23:52:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2003-01-01  archive always under the manager account.
# rschmutz  2003-01-01  support archiving directories and misc. files.
# rschmutz  2003-01-01  support archiving without user intervention.
# rschmutz  2001-07-06  update the ChangeLog file.
# rschmutz  2000-09-16  support multiple CONFIG.ARCHIVEi.* configs.
# rschmutz  2000-09-04  skip password check, if no account specified.
# rschmutz  2000-09-02  created (based in uvinsArchiveCfg).
#

#************************************************************************
#   NAME
#      ctooConfigArchive.tcl
#
#   DESCRIPTION
#      ctooConfigArchive panel code.
#
#------------------------------------------------------------------------
#

#************************************************************************
# proc ctooConfigArchiveInitVars
#------------------------------------------------------------------------

proc ctooConfigArchiveInitVars {} {
    global cv gvar cfg EDITING argv

    #*# 1. load config.
    ctooCCONFIG cfg
    set ari 1
    set name ""
    set user ""
    set comment ""
    if { !$EDITING } {
        while { $argv != "" } {
	    set opt [lvarpop argv]
	    switch -- $opt {
	        "-config"	{ set name [lvarpop argv] }
		"-user"		{ set user [lvarpop argv] }
		"-comment"	{ set comment [lvarpop argv] }
		default	{
    	            tk_messageBox -type ok -icon error \
	                -message "Unknown command option $opt."
	            exit 1
		}
	    }
	}
	if { $name == "" } {
    	    catch { set name [cfg Get CONFIG.ARCHIVE$ari.NAME] }
	    if { $name == "" } {
    	        tk_messageBox -type ok -icon error \
	            -message "ctooConfigArchive not configured."
	        exit 1
            }
	} else {
	    while {1} {
                try_eval {
    	            set name1 [cfg Get CONFIG.ARCHIVE$ari.NAME]
                } {
	    	    tk_messageBox -type ok -icon error \
		        -message "ctooConfigArchive configuration $name not found."
		    exit 1
	        }
		if { $name1 == $name } {
		    break
		}
		incr ari
   	    } 
	}
    }
    set gvar(ctooArchiveIndex) $ari
    set gvar(user)             $user
    set gvar(comment)          $comment
    set gvar(passwd)           ""

    #*# 2. init. widgets
    if { !$EDITING } {
        $cv itemconfigure ctooConfigArchive-item1 \
    	    -text "Archive $name Configuration"
    }
    [$cv.passwd component uifEntry] configure -show "*"

    #*# 3. if name, user and comment provided: archive
    if { $EDITING == 0 && $name != "" && $user != "" && $comment != "" } {
    	set mgraccount ""
	set curaccount [exec whoami]
    	catch { set mgraccount [cfg Get CONFIG.ARCHIVE$ari.USER] }
	if { $mgraccount == "" || $curaccount == $mgraccount } {
	    try_eval {
	    	ctooConfigArchiveButton2
		sleep 1
		exit 0
	    } {
	    	tk_messageBox -type ok -icon error -message $errorResult
	        ctooShowHelptext ""
	    }
	}
    }
}


#************************************************************************
# proc ctooConfigArchiveButton
#------------------------------------------------------------------------

proc ctooConfigArchiveButton {} {
    global gvar cfg

    #*# 1. catch any errors
    try_eval {
    	ctooConfigArchiveButton1
        set ari $gvar(ctooArchiveIndex)
	set txt "[cfg Get CONFIG.ARCHIVE$ari.NAME] configuration archived."
        tk_messageBox -type ok -icon info -message $txt
        ctooShowHelptext ""
        panClosePanel
    } {
    	tk_messageBox -type ok -icon error -message $errorResult
        ctooShowHelptext ""
    }
}

proc ctooConfigArchiveButton1 {} {
    global gvar cfg

    #*# 1. check that data is available
    if { $gvar(user) == "" } {
        error "Enter your user initials."
    }
    if { $gvar(comment) == "" } {
        error "Enter a comment."
    }

    #*# 2. check password
    set mgraccount ""
    set curaccount [exec whoami]
    set ari $gvar(ctooArchiveIndex)
    catch { set mgraccount [cfg Get CONFIG.ARCHIVE$ari.USER] }
    if { $mgraccount != "" && $curaccount != $mgraccount } {
        ctooShowHelptext "Checking manager password ..."
        set fd [open "|ctooCheckPasswd $mgraccount" "w+"]
        puts $fd $gvar(passwd)
        flush $fd
        set result [string trim [read $fd]]
        if { $result != "OK" } {
    	    error "Wrong manager password."
        }
    }

    #*# 3. if not running as manager, restart as manager
    #      (otherwise continue below)
    if { $mgraccount != "" && $curaccount != $mgraccount } {
        ctooShowHelptext "Re-starting panel under user $mgraccount ..."
    	set name [cfg Get CONFIG.ARCHIVE$ari.NAME]
        set fd [open "|ctooArchAsUser $mgraccount -config $name -user $gvar(user) -comment \"$gvar(comment)\"" "w+"]
        puts $fd $gvar(passwd)
        flush $fd
	sleep 5
	panClosePanel
    } else {
    	ctooConfigArchiveButton2
    }
}

proc ctooConfigArchiveButton2 {} {
    global gvar cfg env

    #*# 1. retrieve config. module
    set ari $gvar(ctooArchiveIndex)
    set cfgmodule [cfg Get CONFIG.ARCHIVE$ari.MODULE]
    ctooShowHelptext "Retrieving module $cfgmodule ..."
    cd $env(HOME)
    file mkdir "tmp"
    cd "tmp"
    if {[file exists $cfgmodule]} {
    	error "Directory ~/tmp/$cfgmodule already exists.  Delete it."
    }
    file delete ctooConfigArchive.out
    file delete ctooConfigArchive.err
    set ret [system cmmModify $cfgmodule > ctooConfigArchive.log 2> ctooConfigArchive.err]
    if { $ret != 0 } {
    	error "Error retrieving module $cfgmodule.  Check log files in ~/tmp."
    }

    #*# 2. save configuration files
    ctooShowHelptext "Saving configuration files ..."
    set i 1
    set filename ""
    catch { set filename [cfg Get CONFIG.ARCHIVE$ari.FILE$i] }
    while { $filename != "" } {
    	set fd [open ctooConfigArchive.log "a"]
	puts $fd "Saving files: '$filename'"
	close $fd

    	# glob filename
	if { [cindex $filename 0] == "\$" } {
	    regsub -- {\$INTROOT}  $filename $env(INTROOT)  filename
	    regsub -- {\$VLTDATA}  $filename $env(VLTDATA)  filename
	    regsub -- {\$INS_ROOT} $filename $env(INS_ROOT) filename
	}
	if { [file pathtype $filename] == "absolute" } {
	    set filenames [glob $filename]
	} else {
	    set filenames [glob [file join $env(INS_ROOT) SYSTEM COMMON CONFIGFILES $filename]]
	}

	# copy filenames
	foreach fromfile $filenames {
	    if { [file isdirectory $fromfile] } {
		file delete -force [file join $cfgmodule config [file tail $fromfile]]
	        set tofile [file join $cfgmodule config]
	    } else {
	        set tofile [file join $cfgmodule config [file tail $fromfile]]
	    }
	    file copy -force $fromfile $tofile
	}

    	# get next filename
        incr i
        set filename ""
        catch { set filename [cfg Get CONFIG.ARCHIVE$ari.FILE$i] }
    }

    #*# 3. execute user plugin
    if { $i != 1 } {
        set plugin ""
    } else {
        set plugin "make save"
    }
    catch { set plugin [cfg Get CONFIG.ARCHIVE$ari.PLUGIN] }
    if { $plugin != "" } {
    	set fd [open ctooConfigArchive.log "a"]
	puts $fd "Executing user plugin: $plugin"
	close $fd
    	set plugin "cd $cfgmodule/src ; $plugin"
        set ret [system $plugin >> ../../ctooConfigArchive.log 2>> ../../ctooConfigArchive.err]
        if { $ret != 0 } {
	    error "Error running user plugin.  Check log files in ~/tmp."
        }
    }

    #*# 4. update ChangeLog file
    set txt "$gvar(user) - $gvar(comment)"
    set chgLogFile [file join $cfgmodule ChangeLog]
    if { [file isfile $chgLogFile] } {
        set lineList [split [read_file -nonewline $chgLogFile] "\n"]
	set lineList [linsert $lineList 1 "" "\t* $txt" ""]
	write_file $chgLogFile [join $lineList "\n"]
    }

    #*# 5. archive config. module
    ctooShowHelptext "Archiving module $cfgmodule ..."
    set ret [system cmmArchive $cfgmodule \"$txt\" >> ctooConfigArchive.log 2>> ctooConfigArchive.err]
    # \""
    if { $ret != 0 } {
        error "Error archiving module $cfgmodule.  Check log files in ~/tmp."
    }

    #*# 6. cleanup
    ctooShowHelptext "Cleaning up ..."
    file delete -force $cfgmodule
    file delete ctooConfigArchive.log
    file delete ctooConfigArchive.err
    cd $env(HOME)
    catch { file delete tmp }
    ctooShowHelptext "Done."
    sleep 1
}

#
# ___oOo___
