#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbSTACK.tcl,v 1.8 2001/06/06 08:48:11 vltsccm Exp $" 
#
#
# STACK
# ----------------------------------------------------------------------
# Implements a stack of strings.
#
#   PUBLIC ATTRIBUTES:
#
#   METHODS:
#
#	push ..... put a string on the top of the stack
#       pop ...... remove the top element of the stack and return
#		   its value or an error if the stack is empty
#       top ...... return the value of the top element of the stack
#		   or an error if the stack is empty
#	empty? ... returns 1 if the stack is empty, 0 otherwise
#
#   COMMON ATTRIBUTES:
#
#   PROTECTED ATTRIBUTES:
#
#	stack .... a list of strings
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Computerized Processes Unlimited, Inc.
#            Phone: (504) 889-2784
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when      what
# --------------  --------  ----------------------------------------------
# Erik Allaert    08/03/96  adapted to [incr Tcl] 2.0
# ======================================================================

# This will fool the auto_mkindex procedure into sourcing this file
# so that this class can be loaded.
proc STACK {} {;}; rename STACK {}

class STACK {
	protected variable stack {}

	# ------------------------------------------------------------------
	#  CONSTRUCTOR
	# ------------------------------------------------------------------
	constructor {} {}

	# ------------------------------------------------------------------
	#  DESTRUCTOR
	# ------------------------------------------------------------------
	destructor {}


	# ------------------------------------------------------------------
	#  METHOD:  push - put a string on the top of the stack
	# ------------------------------------------------------------------
	method push {str} {
		set stack [linsert $stack 0 $str]
	}

	# ------------------------------------------------------------------
	#  METHOD:  pop - remove the top element of the stack and return
	#		  its value or an error if the stack is empty
	# ------------------------------------------------------------------
	method pop {} {
		if {$stack == ""} {error "method 'pop' failed: stack '$this' empty"}
		lvarpop stack
	}

	# ------------------------------------------------------------------
	#  METHOD:  top - return the value of the top element of the stack
	#		  or an error if the stack is empty
	# ------------------------------------------------------------------
	method top {} {
		if {$stack == ""} {error "method 'top' failed: stack '$this' empty"}
		lindex $stack 0
	}

	# ------------------------------------------------------------------
	#  METHOD:  empty? - returns 1 if the stack is empty, 0 otherwise
	# ------------------------------------------------------------------
	method empty? {} {
		expr {($stack == "") ? 1 : 0}
	}
}
