#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dxfPrivate.tcl,v 2.51 2004/11/22 13:55:31 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  29/01/02  created
#

#************************************************************************
#   NAME
#    dxfPrivate - Tcl library used by dxf tcl scripts
#     library.
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   RETURN VALUES
#
#------------------------------------------------------------------------
#
proc dxfDebugMsg {msg} {
    global gvar

    if {[info exist gvar(debug)]} {
        if {$gvar(debug)} {
	    puts "$msg"
	}
    }
}

proc dxfPing {envName procName} {
    
    return [catch {seq_msgSendCommand $envName $procName PING ""} ]
}

proc dxfSendCommand {envName command arg {procName ""} } {
    global seq_ccsCmdCheck gvar
    
    set seq_ccsCmdCheck NO_CHECK
    if { [string length $procName] == 0 } { set procName $gvar(locServer) }
    dxfDebugMsg "Sending command $command parameters $arg to process $procName environment $envName"
    set cmdHandle [seq_msgSendCommand $envName $procName $command $arg]
    if {[string first "Error:" $cmdHandle] != -1 } {
        # Something went wrong sending the command
        return $cmdHandle
    }

    set error ""
    set reply [seq_msgRecvReply $cmdHandle error -$gvar(timeOut)]
    if { $error != 0 } {
        dxfDebugMsg "Received error: $error reply: $reply"
    } else {
        dxfDebugMsg "Received reply $reply"
    }
    return $reply
}

proc dxfCompareFiles { refFile outFile } {
    dxfDebugMsg "Check Results"
    catch {exec diff $refFile $outFile} msg
    if {[string length $msg] > 0} {
       dxfDebugMsg "$msg"
    }
}

proc dxfRemoveFiles { fileMask } {
    catch {exec sh -c "rm -f $fileMask"}
}

proc dxfCopyFiles { fileMask destDir } {
    catch {exec sh -c "cp -f $fileMask $destDir"}
}

proc dxfVxShellCmd { Cmd } {
    global gvar

    puts $gvar(lcuPipe) $Cmd; flush $gvar(lcuPipe); after 1000
}
	
proc dxfProcStart { } {
    global gvar env

    if { [dxfPing $gvar(locEnv) $gvar(locServer)] } {
        dxfDebugMsg "Starting program $gvar(locServer) in environment $gvar(locEnv)"
	exec msgSchedule $gvar(locEnv) $gvar(locServer)
    }
    if { [dxfPing $gvar(remEnv) $gvar(remServer)] } {
        dxfDebugMsg "Starting program $gvar(remServer) in environment $gvar(remEnv)"
        set runstring $gvar(remServer)
        if { $gvar(remWs) } {
            if { [string compare $gvar(remHost) $gvar(locHost)] } {
                set runstring "sh -c /tmp/dxf/ws/bin/$gvar(remServer)"; # see dxfTestRemWsMake
	    } else {; # remote environment on local host
                set runstring "sh -c $gvar(remServer)"
	    }
        }
	exec msgSchedule $gvar(remEnv) $runstring $gvar(lcuPri) $gvar(lcuOpt) $gvar(lcuStk)
    }
    sleep 5

    # wait till locServer is registered to CCS
    set counter 0
    set env(NO_ERR_DISPLAY) true
    while { [dxfPing $gvar(locEnv) $gvar(locServer)] } {
	sleep 1
        if {[incr counter] == 20} {
            puts "Timeout. Process $gvar(locServer) not registered in env. $gvar(locEnv)"
            dxfExit 
        }
    }
    dxfDebugMsg "$gvar(locServer) is registered in environemnt $gvar(locEnv)"

    # wait till remServer is registered to CCS
    set counter 0
    while { [dxfPing $gvar(remEnv) $gvar(remServer)] } {
	sleep 1
        if {[incr counter] == 20} { 
            puts "Timeout. Process $gvar(remServer) not registered in env. $gvar(remEnv)"
            dxfExit 
        }
    }
    dxfDebugMsg "$gvar(remServer) is registered in environment $gvar(remEnv)"

    set gvar(serversRunning) 1
}
	
proc dxfExit { } {
    global gvar

    if { $gvar(serversRunning) } {
        catch { dxfSendCommand $gvar(locEnv) EXIT "" $gvar(locServer) }
        catch { dxfSendCommand $gvar(remEnv) EXIT "" $gvar(remServer) }
        set gvar(serversRunning) 0
    }
    seq_ccsExit
    exit
}

proc dxfInit { } {
    global gvar env

    set gvar(lcuPri) ""
    set gvar(lcuOpt) ""
    set gvar(lcuStk) ""
    if {![info exist env(WSLOC)]} { set env(WSLOC) $env(RTAPENV) }
    set env(RTAPENV) $env(WSLOC)
    if {![info exist env(WSREM)]} { set env(WSREM) $env(WSLOC) }
    set gvar(locEnv) $env(WSLOC)
    if {$gvar(remWs)} {
        set gvar(remEnv) $env(WSREM)
    } else {
        set gvar(remEnv) $env(LCUENV)
        set gvar(lcuPri) 100
        set gvar(lcuOpt) 24
        set gvar(lcuStk) 50000
    }
    set gvar(locHost) $env(HOST)
    set gvar(remHost) $env(HOST)
    if {[info exist env(RHOST)]}   { set gvar(remHost) $env(RHOST) }
    if {[info exist env(RHOSTQS)]} { set gvar(remHost) $env(RHOSTQS) }

    dxfDebugMsg "Local host: $gvar(locHost)"
    dxfDebugMsg "Local environment: $gvar(locEnv)"
    dxfDebugMsg "Remote host: $gvar(remHost)"
    dxfDebugMsg "Remote environment: $gvar(remEnv)"

    set gvar(timeOut)        30000

    # register to CCS
    seq_ccsInit

    # start processes if requested
    if { !$gvar(serversRunning) } {
        dxfProcStart
    }
} 

# --- oOo ---
