#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdConfigTel.tcl,v 2.83 2005/09/30 08:26:37 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    05/02/96  created
# alongino  06/03/97  telemetry only
# rdonalds  2/06/98   ported
#

#************************************************************************
#   NAME 
#   fcdConfigTelLibInit, fcdFormatTel, fcdEnableTelemetry, fcdClearTelemetry, 
#   fcdSaveTelemetry, fcdTelBoxToEdit, fcdTelModify, fcdTelDelete
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the fcdConfig panel to handle the
#   telemetry parameters
#
#------------------------------------------------------------------------
#

#
# Initialise variables for telemetry list box
proc fcdConfigTelLibInit { } {

  global cv
  global telRows telCol telDbAttr telWdg telFormatProc
  global telIndex telNames telRec telButton
  global telTable telDisp 

  # set common stuff for telemetry and temperature
  fcdConfigTelTemInit Tel telIndex telNames telRec telButton

  set telRows 100; # maximum number of rows, as from DB
  set telCol 10;   # maximum number of columns, as from DB
  set telWdg $cv.fcdTelemetry
  set telFormatProc fcdFormatTel

  # set path to DB values
  fcdTelDbInit $telRows telDbAttr

  # attach call to selection of one row
  $telWdg setBind <B1-ButtonRelease> {fcdTelBoxToEdit}
}

# Close the Telemetry config window at user request
proc fcdConfigTelUserClose { } {
    fcdSaveTelemetry
    panClosePanel
}

# refresh the data from DB
proc fcdConfigTelUserRefresh { } {
    
    fcdEnableTelemetry

}

# write data to DB
proc fcdConfigTelUserApply { } {
    global cv

    panDbWriteWidget $cv all
    fcdSaveTelemetry

}



# formats one row in the telemetry list box
proc fcdFormatTel { text } {

  global telIndex
 
  fcdConfigFormatTelTem telIndex $text
}



#
# Enable/disable telemetry in the panel fcdConfig
proc fcdEnableTelemetry {  } {

  global telRows telCol telDbAttr telWdg telFormatProc
  global telButton
  global telTable telDisp 

  fcdEnableBox telTable telDisp telButton $telRows $telCol telDbAttr $telWdg $telFormatProc
}


#
# Reset the telemetry list box and optionally clean table
proc fcdClearTelemetry { } {
   global telTable telWdg telRec

   fcdClearBox $telWdg telTable $telRec(clean)
}


#
# Stores the current telemetry table into the database
proc fcdSaveTelemetry { } {
 
  global telTable telRows telCol telDbAttr telRec
  
  fcdSaveBoxToDb telTable $telRows $telCol telDbAttr $telRec(clean)
}


#
# Show in the edit section the selected record from the telemetry list
proc fcdTelBoxToEdit { } {

  global telNames telIndex telTable telWdg

  fcdBoxToEdit telNames telIndex telTable $telWdg
}



#
#
# Add or modify one entry in the telemetry list box
proc fcdTelModify { } {

  global telWdg
  global telIndex telNames telRec
  global telTable telDisp telFormatProc

  fcdEditToBox telNames telIndex telTable telDisp $telWdg $telFormatProc telRec 1
}



#
# Delete one entry in the telemetry list box
proc fcdTelDelete { } {

  global telWdg
  global telIndex telNames telRec
  global telTable telDisp telFormatProc

  fcdEditToBox telNames telIndex telTable telDisp $telWdg $telFormatProc telRec 0
}

#
# ___oOo___
