#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdCtrl.tcl,v 2.83 2005/09/30 08:26:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  09/07/97  created (ported from CCD software)
# abalestr  20/04/99  added fcdCtrlGetMosaicConf procedure to get mosaic 
#                     configuration and display it through the main panel
#                     see spr no. VLTSW980463
# ccumani   14/01/04  {} added to "expr" calls
# abortolu  13/09/05  Modified timeout in fcdStart to 100 seconds
#

#************************************************************************
#   NAME fcdRdtModesUpdate, fcdRdtIndex2Mode, fcdRdtMode2Index, 
#        fcdSetRdtOutputs, fcdSetRdtMode, fcdEnableRdtWindow,
#        fcdEnableImgProcess, fcdEnableExpOpt, fcdEnableImgOpt, 
#        fcdEnableWindowing, fcdChangeWidgets, fcdDrawBorders, 
#        fcdSetDefaults, fcdSetOpState, fcdInitialize, fcdStartup, 
#        fcdShutdown, fcdStart, fcdPerWipe, fcdAbort, fcdStopWipe,
#        fcdPause, fcdContinue, fcdStop, fcdChangeTime, fcdWipe,
#        fcdDump, fcdStopLoop, fcdStartDisplay, fcdSetTrace, fcdTrace
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#        Tcl procedures used within fcdCtrl panel
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# procedure to set default values in the fcdCtrl panel
proc fcdSetDefaults { } {
  global gvar
  global fcdMaxWin

 # Determine if sprBrowse can run in this environment
 # AL960412 catch added (see SPR 960221)
   catch {exec which sprBrowse} fileName
   set gvar(sprBrowse) [file executable $fileName]

 # set default values for numeric variables
 set gvar(fcdMainCtrl_fcdContext) 1;
 set gvar(fcdMainCtrl_fcdFits) 0;
 set gvar(fcdMainCtrl_fcdArchive) 0;
 set gvar(fcdMainCtrl_startupFast) 0;
 set gvar(setup.1.DET.WIN1.UIT1) 1;
 set gvar(setup.2.DET.EXP.NREP) 1;
 set gvar(setup.9.DET.DISPLAY) 0;
 set gvar(setup.5.DET.READ.CLKIND) 1;
 set gvar(setup.6.DET.OUT1.GAININD) 1;

 # set default values for variables from imported classes
 set gvar(fcdMainCtrl_maint) $gvar(maint)

 set gvar(fcdExpOpt_setup.0.DET.EXP.TYPE) Normal;
 set gvar(fcdExpOpt_setup.40.DET.EXP.TIMEREP) 0;
 set gvar(imageOpt_setup.30.DET.FRAM.TYPE) Normal;
 set gvar(imageOpt_setup.24.DET.FRAM.SAMPLE) 1;

 for { set i 1} { $i <= $fcdMaxWin } {incr i} {
   set gvar(setup.${i}10.DET.WIN$i.ST) F;
   set gvar(setup.${i}11.DET.WIN$i.STRX) 1;
   set gvar(setup.${i}12.DET.WIN$i.NX)  20;
   set gvar(setup.${i}13.DET.WIN$i.STRY) 1;
   set gvar(setup.${i}14.DET.WIN$i.NY)  20;

   set gvar(setup.${i}23.DET.WIN$i.MINMAX)  F;
   set gvar(setup.${i}20.DET.WIN$i.CENTROID)  none;
   set gvar(setup.${i}27.DET.WIN$i.RMS)  0;
   set gvar(setup.${i}21.DET.WIN$i.REFX)  1;
   set gvar(setup.${i}22.DET.WIN$i.REFY)  1;
   set gvar(setup.${i}25.DET.WIN$i.IPFUNC) None;
 }

 # work-around bug JUN96 with scale widgets
 set gvar(setup.15.DET.WIN1.BINX) 1;
 set gvar(setup.16.DET.WIN1.BINY) 1;
}




# procedure to draw borders in the fcdCtrl panel
proc fcdCtrlDrawBorders { } {
 global cv

 # Rectangle for GENERAL
   $cv create rectangle    10  1 1015  125  -outline Blue -width 2

 # Rectangle for SETUP
   $cv create rectangle    10 135 1015 505 -outline Blue -width 2
 # Rectangle for EXPOSURE
   $cv create rectangle    15 170 350 300 -outline Blue
 # Rectangle for IMAGE
   $cv create rectangle    15 305 350 465 -outline Blue
 # Rectangle for READOUT
   $cv create rectangle    355 170 1010 500 -outline Blue

 # Rectangle for CONTROL
   $cv create rectangle    10 510 290 905 -outline Blue -width 2

 # Rectangle for STATUS
   $cv create rectangle    295 510 1015 905 -outline Blue -width 2
}





# procedure to be called after any change in the OLDB CCD configuration
proc fcdRdtModesUpdate { {minMode {0}} {maxMode {19}} } {

  global cv gvar
  global fcdRdtModeList
  global fcdRdtModeIndexList

  set fcdRdtModeList {} ;    # List of available modes
  set showList {}

# Get from OLDB the list of readout modes defined and related characteristics
  for { set i $minMode} { $i <= $maxMode } {incr i} { 
    set tmp [dbRead "readout:mode_${i}.description"]
     
    if { ([string trim $tmp " "] != "") } {
      lappend fcdRdtModeList $tmp
      set tmpspeed [dbRead "readout:mode_${i}.speed"]
      lappend fcdRdtModeIndexList $i
      lappend showList "$tmp ($tmpspeed)"
    }
  }
  if {$showList != {}} {
    set gvar(fcdRdtMode) 0
    $cv.fcdRdtMode configure -itemList $showList -itemValue $fcdRdtModeIndexList
  } else {
    $cv.fcdRdtMode configure -itemList {none}
    $cv.fcdRdtMode disable
  }
}



# enable/disable windowed readout container widget
proc fcdEnableRdtWindow { } {
   global cv gvar
   global fcdMaxWin

   # window setup
   for { set i 1} { $i <= $fcdMaxWin } {incr i} { 
     if {[info exists gvar(setup.${i}10.DET.WIN$i.ST)]} {
        set status $gvar(setup.${i}10.DET.WIN$i.ST)
        if {[string compare $status F]} {set status 1
           } else {set status 0}
        $cv.fcdWindowFrame$i configure -show $status
     }
   }
}




# enable/disable image processing container widget
proc fcdEnableImgProcess { } {
   global cv gvar
   global fcdMaxWin

   for { set i 1} { $i <= $fcdMaxWin } {incr i} { 
     set status 1
     set value $gvar(fcdSetupIp${i}_ipCentroid)
     if {![string compare $value none]} {
         set value $gvar(fcdSetupIp${i}_ipMinMax)
         if {![string compare $value F]} {
           set value $gvar(fcdSetupIp${i}_ipRMS)
           if {![string compare $value 0]} {set status 0}
         }
     }
     $cv.fcdSetupIp$i configure -show $status
   }
}




# enable/disable exposure options container widget
proc fcdEnableExpOpt { } {
   global cv gvar

   set status 1
   # do not show if no option selected
   # period makes sense ONLY for repeated exposures
   if {![string compare $gvar(fcdExpOpt_setup.0.DET.EXP.TYPE) Normal]} {
       if {$gvar(fcdExpOpt_rep) == 0} {
           set status 0
       }
   }
   $cv.fcdExpOpt configure -show $status
}




# enable/disable image options container widget
proc fcdEnableImgOpt { } {
   global cv gvar

   set status 1
   # do not show if no option selected
   if {![string compare $gvar(imageOpt_setup.30.DET.FRAM.TYPE) Normal]} {
       if {$gvar(imageOpt_setup.24.DET.FRAM.SAMPLE) == 1} {
           if {$gvar(imageOpt_rtdBld) == 0} {
               set status 0
           }
       }
   }
   $cv.imageOpt configure -show $status
}




# enable/disable windowed readout and image processing container widgets
proc fcdEnableWindowing { } {
   global cv gvar

   # exposure setup options
   fcdEnableExpOpt
   # image setup options
   fcdEnableImgOpt
   # window setup
   fcdEnableRdtWindow
   # image process setup
   fcdEnableImgProcess
}



# enable/disable widgets
proc fcdChangeWidgets {} {
   global gvar cv
   global $cv.fcdExpOpt.ckShow

   # disable Dark exposure type option if no shutter available
   set widget [$cv.fcdExpOpt getClassPath]
   set shutAvai [dbRead "shutter.available"]
   if {$shutAvai == 0} {;  # no shutter
     $widget.cv.fcdExpType disableEntry 1
   } else {
     $widget.cv.fcdExpType enableEntry 1
   }

   # Update supported readout modes
   fcdRdtModesUpdate
}




# set operational state. Used in simulation
proc fcdSetOpState {opState} {

 # Declare global variables

   global gvar

 # User Initialization
   panDbWrite ".opStateWs" $opState;       # set operational state
}



# startup FIERA stand-alone control processes
proc fcdInitialize {} {
  global gvar
  global camera wsenv simul

  set status 0
  if {$simul != 1} {;        # no simulation
      set timeout 60000;     # 60 secs timeout
      # send command STANDBY
      set status [panSendCommand $wsenv "fcdconCI_$camera" "STANDBY" 0 $timeout]
      if {$status != "FAILURE"} {
        # send command ONLINE
        set status [panSendCommand $wsenv "fcdconCI_$camera" "ONLINE" 0 $timeout]
      }
      if {$status != "FAILURE"} {
        # send command STANDAL
        set fits ""
        set archive ""
	if {$gvar(fcdMainCtrl_fcdFits)} { set fits "-on" }
	if {$gvar(fcdMainCtrl_fcdArchive)} { set archive "-archive" }
        set status [panSendCommand $wsenv "fcdconCI_$camera" "STANDAL" 0 $timeout "$fits $archive"]
      }
  } else {;                   # simulation
      fcdSetOpState 4 ;       # state ONLINE
  }

  if {$status == "FAILURE"} {
    set status 1
  } else {
    set status 0
  }
  return $status
}




# startup FIERA stand-alone control processes
proc fcdStartup {} {
  global gvar camera wsenv slcuenv insroot simul

  set status 0
  # execute startup script
  if {$simul != 1} {;           # no simulation
    set mode [dbRead ".opMode"]
    set ws [fcdWsEnv $wsenv $mode]
    set slcu [fcdSlcuEnv $slcuenv $mode]
    set pid [exec fcdDcsWarmStart.sh $camera $ws $slcu &]
    set status [lindex [wait $pid] 2]; # wait for children to finish
  } else {;                     # simulation
    fcdSetOpState 4 ;           # set operational state to OPERATING
  }

  # initialize
  if {$status == 0} {
    set status [fcdInitialize]
  }

  # print final result of procedure
  fcdPrintResult $status "Startup for camera $camera"
}



# shutdown FIERA stand-alone control processes
proc fcdShutdown {} {

  global gvar simul camera wsenv slcuenv

  set status 0
  # execute shutdown script
  if {$simul != 1} {;       # no simulation
      # If something is going on, ask for confirmation
      set msg ""
      if {$gvar(fcdMainCtrl_fcdContext)} {;            # context sensitive
        if {$gvar(fcdExpCtrl_wiping)} {
          set msg "CCD chip wiping active"
        } elseif {(!($gvar(fcdExpCtrl_notRunning))) && ($gvar(opState) == 5)} {
          set msg "Exposure active"
        }
      }
      if {[string length $msg] > 0} {
       set status [tk_dialog .dialog "WARNING" $msg question 1 Continue cancel]
       if {$status} { return }
      }
      set mode [dbRead ".opMode"]
      set ws [fcdWsEnv $wsenv $mode]
      set slcu [fcdSlcuEnv $slcuenv $mode]
      set pid [exec fcdDcsStop.sh $camera $ws $slcu &]
      set status [lindex [wait $pid] 2]; # wait for children to finish
  } else {;                       # simulation
      fcdSetOpState 1 ;           # operational state OFF
  }

  # print final result of procedure
  fcdPrintResult $status "Shutdown for camera $camera"
}



# start an exposure
proc fcdStart {} {
 global camera wsenv gvar simul
 global fcdAbortFlag

 set dir [dbRead "observations.setupDirectory"]
 set setupfile "fcdSetupCurrent_$camera.det"
 # save current setup
 set status [fcdSaveSetup $setupfile]
 if {$status != "FAILURE"} {
   if {$simul != 1} {;               # no simulation
     set setupfile $dir/$setupfile
     set process "fcdconCI_$camera"
     set expId $gvar(fcdEXP_NEXT);   # next exposure
     set timeout 100000;              # 20 sec timeout
     set param "-expoId $expId -file $setupfile"
     set status [panSendCommand $wsenv $process "SETUP" 0 $timeout "$param"]
     if {$status != "FAILURE"} {
       set status [panSendCommand $wsenv $process "START" 0 $timeout]
     }
   } else {;                         # simulation
     set fcdAbortFlag 0
     after 1 fcdExpSim 0 0
   }
 }
}




proc fcdPerWipe {} {
   global camera wsenv gvar simul fcdReadStateAttr

   if {$simul != 1} {;    # no simulation
       set timeout 60000; # 60 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "STARTWP" 0 $timeout "1"]
   } else {;                         # simulation
       panDbWrite $fcdReadStateAttr 6
   }
}




proc fcdAbort {} {
   global camera wsenv gvar simul
   global fcdAbortFlag

   if {$simul != 1} {;     # no simulation
       set timeout 10000;  # 10 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "ABORT" 0 $timeout]
   } else {;               # simulation
       set fcdAbortFlag 1
   }
}




proc fcdStopWipe {} {
   global camera wsenv gvar simul fcdReadStateAttr

   if {$simul != 1} {;          # no simulation
       set timeout 20000;       # 20 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "STOPWP" 0 $timeout]
   } else {;                         # simulation
       panDbWrite $fcdReadStateAttr 0
   }
}




proc fcdPause {} {
   global camera wsenv gvar simul
   global fcdPauseFlag

   if {$simul != 1} {;        # no simulation
       set timeout 10000;     # 10 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "PAUSE" 0 $timeout]
   } else {;                  # simulation
       set fcdPauseFlag 1
   }
}




proc fcdContinue {} {
   global camera wsenv gvar simul
   global fcdPauseFlag

   if {$simul != 1} {;        # no simulation
       set timeout 10000;     # 10 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "CONT" 0 $timeout]
   } else {;                  # simulation
       set fcdPauseFlag 0
   }
}




proc fcdStop {} {

   global camera wsenv gvar simul
   global fcdStopFlag

   if {$simul != 1} {;     # no simulation
       set timeout 10000;  # 10 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "END" 0 $timeout]
   } else {;               # simulation
       set fcdStopFlag 1
   }
}




proc fcdChangeTime {} {

  global camera wsenv gvar simul
  global fcdChangeFlag

  if {$simul != 1} {;     # no simulation
    set timeout 10000;  # 10 sec timeout
    set process "fcdconCI_$camera"
    set newTime $gvar(setup.1.DET.WIN1.UIT1); # taken from setup area  
    set param "-expoId $gvar(fcdEXP_LAST) -function DET.WIN1.UIT1 $newTime"
    set status [panSendCommand $wsenv $process "SETUP" 0 $timeout "$param"]
  } else {;               # simulation
    set fcdChangeFlag 1
  }
}




proc fcdWipe {} {
   global camera wsenv gvar simul

   if {$simul != 1} {;     # no simulation
       set timeout 60000;  # 60 sec timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "STARTWP" 0 $timeout "0"]
   }
}




proc fcdDump {} {
   global camera wsenv gvar simul

   if {$simul != 1} {;     # no simulation
       set timeout 20000;  # 20 secs timeout
       set status [panSendCommand $wsenv "fcdconCI_$camera" "DUMP" 0 $timeout]
   } else {;               # simulation
     after 1 fcdExpSim 1 0
   }
}




proc fcdStopLoop {} {
   global camera wsenv gvar simul
   global fcdLoopFlag

   if {$simul != 1} {;     # no simulation
       set timeout 40000;  # 40 secs timeout (20 not enough, see SPR 960285)
       set status [panSendCommand $wsenv "fcdconCI_$camera" "STOP" 0 $timeout]
       if {$status != "FAILURE"} { set fcdLoopFlag 1 }
   } else {;               # simulation
     set fcdLoopFlag 1
   }
}




proc fcdStartDisplay {} {
   global camera imageDir

   exec sh -c "cd $imageDir; rtd -camera $camera" &
}





# set or reset variables to be traced within fcdCtrl
proc fcdSetTrace {op} {

 global gvar

 # Trace operations needed because of renaming of variables in classes
   trace $op gvar(fcdMainCtrl_maint) w fcdTrace

 # Trace operations needed for interaction among widgets
   trace $op gvar(fcdExpOpt_expNrep) w fcdTrace
   trace $op gvar(fcdExpOpt_nrep) w fcdTrace
   trace $op gvar(imageOpt_rtdBld) w fcdTrace
   trace $op gvar(rtd) w fcdTrace
   trace $op gvar(.id) w fcdTrace
   trace $op gvar(setup.15.DET.WIN1.BINX) w fcdTrace
   trace $op gvar(setup.16.DET.WIN1.BINY) w fcdTrace
}






# trace procedure for fcdCtrl panel
proc fcdTrace {name element op} {

  global cv gvar oldId
  global fcdMaxWin

  if {![string compare $element "fcdMainCtrl_maint"]} {
    set gvar(maint) $gvar($element)
  } elseif {(![string compare $element "fcdExpOpt_expNrep"])
           || (![string compare $element "fcdExpOpt_nrep"])} {
    set isInt [regexp {^[0-9]+$} $gvar(fcdExpOpt_expNrep)]
    if {($gvar(fcdExpOpt_nrep) == 0) || 
        (($gvar(fcdExpOpt_expNrep) != 1) && ($isInt))} {
      set gvar(fcdExpOpt_rep) 1
      # AL970521 Added to reduce risk of filling disk for infinite loop
      if {$gvar(fcdExpOpt_nrep) == 0} {
        set gvar(setup.7.DET.FRAM.FITSMTD) 0
      }
    } else {
      set gvar(fcdExpOpt_rep) 0
    }
    if {![string compare $element "fcdExpOpt_expNrep"]} {
      set gvar(fcdExpOpt_nrep) 2
    }
  } elseif {![string compare $element "imageOpt_rtdBld"]} {
    if {$gvar($element)} {set gvar(rtd) 1}
  } elseif {![string compare $element "rtd"]} {
    if {!($gvar($element))} {set gvar(imageOpt_rtdBld) 0}
  } elseif {![string compare $element ".id"]} {
    if {![info exists oldId]} { set oldId "" }
    after 1000 fcdChangeWidgets
    set oldId $gvar($element)
  }
}


proc fcdCtrlGetMosaicConf {} {

global gvar

set nChip 0
set xDim [dbRead "detector.xPixels"]
set yDim [dbRead "detector.yPixels"]
set nXChip 0
set nYChip 0

for {set i 0} { $i < $gvar(maxChip)} {incr i} {
  if {[dbRead "detector.chipInstances($i,available)"] == 1} {
    if {[dbRead "detector.chipInstances($i,xLocation)"] == 0} {
      incr nYChip
    }
    if {[dbRead "detector.chipInstances($i,yLocation)"] == 0} {
      incr nXChip
    }
  }
}

set gvar(totXDim) [expr {$nXChip * $xDim}]
set gvar(totYDim) [expr {$nYChip * $yDim}]
}

#
# ___oOo___
