
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdExpCtrlUpdate.tcl,v 2.83 2005/09/30 08:26:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  12/06/97  created (ported from CCD software)
# abalestr  22/09/99  fixed spr no. VLTSW990362 ("Wipe" buttons)


#

#************************************************************************
#   NAME
#   fcdExpCtrlUpdate - update en./dis. buttons according to exposure status
# 
#   SYNOPSIS
#   fcdExpCtrlUpdate
# 
#   DESCRIPTION
#   Tcl procedure used within CCD OS GUI Exposure Control panels to 
#   update enabling/disabling buttons according to exposure status
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc fcdExpCtrlUpdate { } {               

   global gvar fcdExpStatusAttr fcdReadStateAttr
   global fcdLoopFlag

# SPR 960660. If the panel is unloaded from the panel editor, then an
#             error would occur,
   if {[info exists gvar(fcdMainCtrl_fcdContext)] == 0} { return }

   fcdGetOpState

   if {$gvar(fcdMainCtrl_fcdContext)} {;            # context sensitive
     set expStatus [dbRead $fcdExpStatusAttr]
     set readState [dbRead $fcdReadStateAttr]
     set gvar(sysFail) [dbRead ".failureSlcu"]

     # AB function not yet implemented
     # set gvar(fcdExpCtrl_dummy) 0

     set gvar(fcdExpCtrl_running)   0  ;                 # default idle
     set gvar(fcdExpCtrl_notRunning) 1
     set gvar(fcdExpCtrl_notPaused) 0
     set gvar(fcdExpCtrl_paused)    0
     set gvar(fcdExpCtrl_loopRunning) 0
     set gvar(fcdExpCtrl_fcdTel) 0 ;                  # AB not implemented yet
     if {$gvar(opState) != 4} {;                         # not online
        set expStatus 0
        set gvar(fcdExpCtrl_notRunning) 0
        set gvar(fcdExpCtrl_fcdTel) 0
     }
     if {(($expStatus >= 2) && ($expStatus < 128)) || ($expStatus >= 1024)} {;  # running
       set gvar(fcdExpCtrl_notRunning) 0
       if {($expStatus >=4) && (($expStatus < 16) || ($expStatus >= 1024))} {; # still integrating
         if {$expStatus & 4} {;                       # integrating
           if {!($expStatus & 8)} {;                  # not paused 
             set gvar(fcdExpCtrl_notPaused) 1
           }
         }
         if {$expStatus & 8} {;                       # paused
           set gvar(fcdExpCtrl_paused) 1
         }
         if {[info exists fcdLoopFlag] == 0} { set fcdLoopFlag 0 }
         if {($expStatus >= 1024) && ($fcdLoopFlag == 0)} {;  # loop running
           set gvar(fcdExpCtrl_loopRunning) 1
         } else {; # abort and stop integr active only if loop stopped
           set gvar(fcdExpCtrl_running) 1
         }
       }
     }
     if {$gvar(fcdExpCtrl_notRunning) == 1} {
       set fcdLoopFlag 0
       set gvar(fcdExpCtrl_wiping) 0
       set gvar(fcdExpCtrl_notWiping) 1
       if {$readState == 5} {      
         set gvar(fcdExpCtrl_wiping) 1
         set gvar(fcdExpCtrl_notWiping) 0
         set gvar(fcdExpCtrl_notRunning) 0
       } 
     } else {;                 # exposure active
       set gvar(fcdExpCtrl_wiping) 0
       set gvar(fcdExpCtrl_notWiping) 0       
     }
   } else {;                   # buttons always enabled
     set gvar(fcdExpCtrl_wiping) 1
     set gvar(fcdExpCtrl_notWiping) 1     
     set gvar(fcdExpCtrl_notRunning) 1
     set gvar(sysFail) 1
     set gvar(fcdExpCtrl_running) 1
     set gvar(fcdExpCtrl_notPaused) 1
     set gvar(fcdExpCtrl_paused) 1
     set gvar(fcdExpCtrl_loopRunning) 1
   }

   update idletasks;           # update buttons status

# do update of buttons periodically every 2 secs
   after 2000 fcdExpCtrlUpdate
}

#
# ___oOo___


