
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdExpSim.tcl,v 2.83 2005/09/30 08:26:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/09/95  created
# alongino  05/07/96  updated with new expStatus values 1024 1nd 2048
# alongino  08/07/96  introduced fcdGetOpState
# alongino  26/02/97  introduced second window readout and image processing
# alongino  20/05/97  added logic for change exposure time
# ccumani   14/01/04  {} added to "expr" calls
#

#************************************************************************
#   NAME
#   fcdExpSim - simulate an exposure within the CCD stand-alone panel
# 
#   SYNOPSIS
#   fcdExpSim <dump> <step>
# 
#   DESCRIPTION
#   Tcl procedure used within CCD GUI panel fcdCtrl to simulate
#   the execution of an exposure
#   <dump> flag indicating if only image dump is requested
#   <step> index of the current exposure step
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc fcdExpSim { dump step } {
   global env gvar simul fcdExpStatusAttr fcdExpStatus
   global fcdTimeDef fcdTimeRem fcdNumRep fcdNumTot
   global fcdAbortFlag fcdStopFlag fcdChangeFlag fcdPauseFlag fcdLoopFlag
   global fcdCentro fcdMinMax fcdMin fcdMax fcdErrX fcdErrY
   global fcdMaxWin

# simulate an exposure
  set attrShut "shutter.status"
  set attrFile "exposures:exposure_1:transfer.fileNameUnComp"
  set attrTDef "exposures:exposure_1.timeDef"
  set attrTime "exposures:exposure_1.timeRem"
  set attrRDef "exposures:exposure_1.expRepeatDef"
  set attrRCur "exposures:exposure_1.expRepeatCur"
  set attrTran "images:transfer.percent"
  set attrTrtm "images:transfer.time"
  for { set i 1} { $i <= $fcdMaxWin } {incr i} {
    set attrPMin($i) "images:process:window_${i}.ipMinVal"
    set attrPMax($i) "images:process:window_${i}.ipMaxVal"
    set attrPRMS($i) "images:process:window_${i}.ipRMS"
    set attrPXCe($i) "images:process:window_${i}.ipXCen"
    set attrPYCe($i) "images:process:window_${i}.ipYCen"
  }
  set exitStatus 128; # default success
  set delay 1; # default no delay
  set pollRate 2;  # 2 sec polling rate
  if {[info exist fcdAbortFlag]} {
     if {$fcdAbortFlag == 1} {
#puts "abort flag set"
        set step 4
        set exitStatus 512
        set fcdAbortFlag 0
        set fcdPauseFlag 0
     }
  }
  if {[info exist fcdStopFlag]} {
     if {$fcdStopFlag == 1} {
#puts "stop flag set"
        set step 2
        set fcdTimeRem $pollRate
        set fcdStopFlag 0
        set fcdPauseFlag 0
     }
  } 
  if {[info exist fcdChangeFlag]} {
     if {$fcdChangeFlag == 1} {
#puts "change time flag set"
        set step 2
        set fcdTimeNew $gvar(setup.1.DET.WIN1.UIT1)
        set fcdTimeAdd [expr {$fcdTimeNew - $fcdTimeDef}]
        set fcdTimeRem [expr {$fcdTimeRem + $fcdTimeAdd}]
	if {$fcdTimeRem < $pollRate} {set fcdTimeRem $pollRate}
        set fcdTimeDef $fcdTimeNew
        set fcdChangeFlag 0
     }
  } 
  if {[info exist fcdPauseFlag]} {
     if {$fcdPauseFlag == 1} {
#puts "pause flag set"
        set step 2
        set fcdTimeRem [expr {$fcdTimeRem + $pollRate}]
     }
  }
  if {[info exist fcdLoopFlag]} {
     if {$fcdLoopFlag == 1} {
#puts "stop loop flag set"
        set fcdNumTot $fcdNumRep
        set fcdLoopFlag 0
     }
  }
  if {$step == 0} {;   # extract setup information
     set fcdTimeDef $gvar(setup.1.DET.WIN1.UIT1)
     set fcdNumTot $gvar(setup.2.DET.EXP.NREP)
     for { set i 1} { $i <= $fcdMaxWin } {incr i} { 
       set fcdMinMax($i) $gvar(fcdSetupIp${i}_ipMinMax)
       if {![string compare $fcdMinMax($i) F]} {set fcdMinMax($i) 0
       } else {set fcdMinMax($i) 1}
       set fcdCentro($i) $gvar(fcdSetupIp${i}_ipCentroid)
       if {![string compare $fcdCentro($i) none]} {set fcdCentro($i) 0
       } else {set fcdCentro($i) 1}
       set fcdMin($i) 100
       set fcdMax($i) 10000
       set fcdErrX($i) 0
       set fcdErrY($i) -3
     }
     set fcdNumRep 0
     panDbWrite $attrTDef $fcdTimeDef
     panDbWrite $attrRDef $fcdNumTot
     panDbWrite $attrFile $gvar(setup.8.DET.FRAM.FITSUNC)
  } elseif {$step == 1} {;   # start integration
     if {$dump != 1} {
       set fcdPauseFlag 0
       set fcdTimeRem $fcdTimeDef
       incr fcdNumRep
       panDbWrite $attrTime $fcdTimeRem
       panDbWrite $attrRCur $fcdNumRep
       panDbWrite $attrTran 0;        # reset transfer percentage
       if {$fcdNumTot == 1} {
           set fcdExpStatus 4;     # integrating
       } elseif {$fcdNumTot == 0} {
           set fcdExpStatus 2048;  # infinite loop
       } elseif {$fcdNumTot > 0} {
           set fcdExpStatus 1024;  # finite loop
       }
       panDbWrite $fcdExpStatusAttr $fcdExpStatus;
       panDbWrite $attrShut 1;        # shutter open
       set delay [expr {1000 * $pollRate}];
     }
  } elseif {$step == 2} {; # integrating
     if {$dump !=1} {
       set fcdTimeRem [expr {$fcdTimeRem - $pollRate}]
       if {$fcdTimeRem > 0} {
         if {$pollRate > $fcdTimeRem} {
           set pollRate [expr {int($fcdTimeRem)}]
	   if { $pollRate < 1 } { set pollRate 1 }
         }
         set delay [expr {1000 * $pollRate}];
         incr step -1
       } else {
          set fcdTimeRem 0
       }
       panDbWrite $attrTime $fcdTimeRem;
       if {$fcdPauseFlag == 0} {
           if {$fcdNumTot == 1} {set fcdExpStatus 4}
	   if {$fcdTimeRem > 0} {panDbWrite $attrShut 1};   # shutter open
       } else {
	   if {$fcdNumTot == 1} {set fcdExpStatus 8}
           panDbWrite $attrShut 0;        # shutter close
       }
       panDbWrite $fcdExpStatusAttr $fcdExpStatus
     }
  } elseif {$step == 3} {; # read and transfer
     panDbWrite $attrTime 0;        # no remaining time
     panDbWrite $attrShut 0;        # shutter closed
     if {$fcdNumTot == 1} {
         set fcdExpStatus 80
         if {$dump == 1} {set ccddExpStatus 64}
     }
     panDbWrite $fcdExpStatusAttr $fcdExpStatus; # read & transfer
     panDbWrite $attrTran 20;       # 20 percent transfered
     set delay 2000
  } elseif {$step == 4} {; # exposure completed
     panDbWrite $attrTran 100;      # whole image transfered
     panDbWrite $attrTrtm 2.5;      # time needed for transfer
     panDbWrite $attrTime 0;        # time remaining
     panDbWrite $attrShut 0;        # shutter close
     for { set i 1} { $i <= $fcdMaxWin } {incr i} { 
       if {$fcdCentro($i)} {
         incr fcdErrX($i)
         if {$fcdErrX($i) > 3} {set fcdErrX($i) -3}
         panDbWrite $attrPXCe($i) $fcdErrX($i)
         incr fcdErrY($i)
         if {$fcdErrY($i) > 3} {set fcdErrY($i) -3}
         panDbWrite $attrPYCe($i) $fcdErrY($i)
       }
       if {$fcdMinMax($i)} {
         incr fcdMin($i)
         if {$fcdMin($i) > 110} {set fcdMin($i) 90}
         panDbWrite $attrPMin($i) $fcdMin($i)
         incr fcdMax($i)
         if {$fcdMax($i) > 10010} {set fcdMax($i) 9990}
         panDbWrite $attrPMax($i) $fcdMax($i)
       }
     }
     if {$fcdNumTot != 0} {
        set numTot $fcdNumTot
     } else {
        set numTot [expr {$fcdNumRep + 1}]
     }
     if {$fcdNumRep == $numTot} {
         panDbWrite $fcdExpStatusAttr $exitStatus;# file transfer complete
     }
  }
  if {$step < 4} {; # go to next step
     incr step
     after $delay fcdExpSim $dump $step
  } elseif {($dump != 1) && ($numTot > $fcdNumRep)} {
#puts "$fcdNumTot $numTot $fcdNumRep"
     after $delay fcdExpSim $dump 1 
  }
}


#
# ___oOo___


