#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdListBox.tcl,v 2.83 2005/09/30 08:26:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/03/97  created
# alongino  20/03/97  fcdDisplayBox updated
# rdonalds  22/05/98  Added new functions required by Telemetry from ccd s/w
# rdonalds  25/08/98  Made SaveBoxToDB not clear the box if disabled
# ccumani   14/01/04  - {} added to "expr" calls
#                     - duplication of fcdDisplayBox proc deleted
#

#************************************************************************
#   NAME
#   fcdDbToTable, fcdTableToDisp
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the fcdConfig, fcdTelSet and
#   fcdTelemetry panels to handle list boxes
#
#------------------------------------------------------------------------
#

#
# Read data from OLDB and build internal table
proc fcdDbToTable { table rows columns dbAttr } {

   upvar $table  fcdTable 
   upvar $dbAttr fcdDbAttr 

   # Read element by element from OLDB and set internal table
   for { set i 0 } { $i < $rows } {incr i } {
     set curRow $i
     if {[info exists fcdDbAttr($i)]} {; # row can be treated in one go
       set tmp [dbRead $fcdDbAttr($i)]
       set curRow [concat $curRow $tmp]
     } else {
       for { set j 1 } { $j < $columns } {incr j } {
         set tmp [dbRead $fcdDbAttr($i,$j)]
         set curRow [lappend curRow $tmp]
       } 
     } 
     set fcdTable($i) $curRow
   }
}




#
# Extract from internal table only entries which are available
proc fcdTableAvail { table disp dbAttr } {

  upvar $table fcdTable
  upvar $disp fcdDisp
  upvar $dbAttr fcdDbAttr

  if {[info exists fcdTable] == 0 } { return }
  if {[info exists fcdDisp]} { unset fcdDisp }
   
  # Build display list from internal table
  set elList [array names fcdTable]
  if {[llength $elList] > 0} {
    foreach rec $elList {
      if {[llength $fcdTable($rec)] > $fcdDbAttr($rec,availInd)} {
        if {[lindex $fcdTable($rec) $fcdDbAttr($rec,availInd)] \
           != $fcdDbAttr($rec,availVal)} {; # available set
          set fcdDisp($rec) $fcdTable($rec)
        }
      }
    }
  }
}





#
# Display the data stored in the internal table
proc fcdDisplayBox { disp wdg formatProc } {

  upvar $disp fcdDisp

  # now update list box widget
  set posBar [$wdg component uifListBox yview]
  set yPos [lindex $posBar 0]; # get current position of cross bar
  $wdg deleteItem 0 end ; # Reset list box    
  if {[info exists fcdDisp]} {;   #AL960214 added check if DB empty
     $wdg updateSortList -increasing -integer fcdDisp $formatProc
     $wdg component uifListBox yview moveto $yPos
  }
}



# Show in the edit section the record selected from the list
proc fcdBoxToEdit { name index table wdg } {

   global gvar

   upvar $name fcdName
   upvar $index fcdIndex
   upvar $table fcdTable

   if {[set tmp [$wdg getSelection]] != "" } {
#      if {[lindex $tmp 0] == "N/A"} {   
#          set tmp [lrange $tmp 1 end]
#      }
      set idx [lindex $tmp 0]
      set fcdRecord $fcdTable($idx)
      set col [array size fcdName]
      for { set i 0 } { $i < $col } {incr i } {
        set ind $fcdIndex($i)
        if {$ind >= 0} {; # simple value
          set val [lindex $fcdRecord $ind]
        } else {; # combined value   e.g. clock_3_H
          set ind [expr {-$ind}]
          set piece [split [lindex $fcdRecord $ind] _]; # use _ as separator
          set val [lindex $piece 1];  # extract number in string
          set piece [lreplace $piece 1 1]; # take out number
          set gvar($fcdName($i)) [join $piece _]; # build string with the rest
          incr i; # set next value
        }
        set gvar($fcdName($i)) $val
      }
#      return
   } 
   
#   if {[info exists fcdRecord] == 0 } { 
#      set fcdRecord ""
#      if {[info exists fcdTable]} {
#         set tmp          [lsort -integer [array names fcdTable]]
#         set idx          [lindex $tmp 0]
#         set fcdRecord $fcdTable($idx)
#      }
#   }
}






#
# Changes the contents of a list box with the contents of the edit section
proc fcdEditToBox { name index table disp wdg formatProc templ {modify 1}} {
   
   global gvar

   upvar $name fcdName
   upvar $index fcdIndex
   upvar $table fcdTable
   upvar $disp fcdDisp
   upvar $templ fcdTempl

   set rec $gvar($fcdName(0))
   if {[info exists fcdDisp]} {; #AL960214 added check if DB empty
      set tmp [lsort -integer [array names fcdDisp]]
   } else {
      set tmp [list];                     # empty list
   }
   set len [llength $tmp]

   set idx [lsearch $tmp $rec]
   if {$idx != -1 } {;                    # entry already exists; modify
      $wdg deleteItem $idx
      unset fcdDisp($rec)
   } elseif {$rec <= [lindex $tmp 0]} {;  # search for right place to add
      set idx 0   } elseif {$rec >= [lindex $tmp [expr {$len -1}]]} {
      set idx $len
   } else {

      for {set i 0} { $i < $len } { incr i } {
         set min [lindex $tmp $i]
         set max [lindex $tmp [{expr $i +1} ]]
	 if {$min <= $rec && $rec < $max} { set idx [expr {$i +1} ]; break }
      }
   }

   if {$modify} {; # add record to lixt box
     set val [fcdEditToRec fcdName fcdIndex $fcdTempl(mask)]
     $wdg addItem $idx [$formatProc $val]
     set fcdDisp($rec) $val
   } else {; # clean complete record, included available flags
     set val $fcdTempl(clean)   
   }
   set fcdTable($rec) $val  
}





#
# Update record with contents of the edit section
proc fcdEditToRec { name index record } {

   global gvar 

   upvar $name fcdName
   upvar $index fcdIndex

   set fcdRecord $record
   set col [array size fcdName]
   for { set i 0 } { $i < $col } {incr i } {
     set ind $fcdIndex($i)
     if {$ind >= 0} {; # simple value
       set val $gvar($fcdName($i))
     } else {; # combined value e.g. clock_2_L
       set ind [expr {-$ind}]
       set piece [split $gvar($fcdName($i)) _]; # split string into pieces
       incr i; # now get index
       if {[llength $piece] > 0} { 
         set piece [linsert $piece 1 $gvar($fcdName($i))]
       }
       set val [join $piece _]
     }
     set fcdRecord [lreplace $fcdRecord $ind $ind $val]
   }
   return $fcdRecord
}



#
# Store a generic table from a list box into the database
proc fcdSaveBoxToDb { table rows columns dbAttr cleanRec } {

   global gvar
  
   upvar $table  fcdTable
   upvar $dbAttr fcdDbAttr

   for { set i 0 } { $i < $rows } {incr i } {
     if {[info exists fcdTable($i)]} {; # data entered in box at least once
       if {[info exists fcdDbAttr($i)]} {; # row can be treated in one go
         panDbWrite $fcdDbAttr($i) [lrange $fcdTable($i) 1 end]
       } else {
         for { set j 1 } { $j < $columns } {incr j } {; #j=0 record index: skip
           panDbWrite $fcdDbAttr($i,$j) [lindex $fcdTable($i) $j]
         } 
       }
      update idletasks; # the whole screen may block if not done
     } 
   } 
}






#
# Reset the list box and optionally clean table
proc fcdClearBox { wdg {table ""} {cleanRec ""} } {

   upvar $table fcdTable

   $wdg deleteItem 0 end ; # Reset list box
   fcdClearTable fcdTable $cleanRec
}




#
# Clean table
proc fcdClearTable { table cleanRec } {

   upvar $table fcdTable

   set rows [array size fcdTable]
   for { set i 0 } { $i < $rows } {incr i } {
     set fcdTable($i) $cleanRec
   }
}




#
# Read from OLDB, build internal table and load it in the listbox
proc fcdLoadBoxFromDb { table disp rows columns dbAttr wdg formatProc } {

   upvar $table  fcdTable 
   upvar $disp   fcdDisp 
   upvar $dbAttr fcdDbAttr 

   # Read element by element from OLDB and set internal table
   fcdDbToTable fcdTable $rows $columns fcdDbAttr
 
   # now extract from the table only those rows which should be displayed
   fcdTableToDisplay fcdTable fcdDisp fcdDbAttr $wdg $formatProc
}




#
# Enable/disable a generic list box
proc fcdEnableBox { table disp button rows columns dbAttr wdg formatProc } {

  global gvar

  upvar $table  fcdTable
  upvar $disp   fcdDisp
  upvar $button fcdButton
  upvar $dbAttr fcdDbAttr

  if {($gvar($fcdDbAttr(avail)))} {; #enabled
     fcdLoadBoxFromDb fcdTable fcdDisp $rows $columns fcdDbAttr $wdg $formatProc
     set butVal 1
   } else {
     fcdClearBox $wdg; # reset list box but NOT the table, for next enable
     set butVal 0
   }
   # enable/disable edit buttons
   foreach rec [array names fcdButton] { 
     set gvar($fcdButton($rec)) $butVal
   }
}




#
# Display the data stored in the internal table
proc fcdTableToDisplay { table disp dbAttr wdg formatProc } {

  upvar $table fcdTable
  upvar $disp fcdDisp
  upvar $dbAttr fcdDbAttr

  # Build display list from internal table
  fcdTableAvail fcdTable fcdDisp fcdDbAttr

  # now update list box widget
  fcdDisplayBox fcdDisp $wdg $formatProc
}








#
# ___oOo___
