
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdLoadSetup.tcl,v 2.83 2005/09/30 08:26:30 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  09/07/97  created (ported from CCD software)
#

#************************************************************************
#   NAME
#   fcdLoadSetup, fcdSaveSetup - interface panel to setup files
# 
#   SYNOPSIS
#   fcdLoadSetup <file>
#   fcdSaveSetup <file>
# 
#   DESCRIPTION
#   Tcl procedures used within CCD OSS GUI panels to load in a panel the
#   contents of a setup file and to save into a setup file the contents
#   of a panel
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   fcdLoadSetup $insroot/SYSTEM/COMMON/SETUPFILES/DET/mySetup.det
#   fcdSaveSetup $insroot/SYSTEM/COMMON/SETUPFILES/DET/mySetup.det
#
#   SEE ALSO
#
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# load in a panel the contents of a setup file
proc fcdLoadSetup { file } {  

   global gvar cv
   global fcdMaxWin

   set status "OK"
   # select file
   set file [fcdSetupFilePath $file]

   # load setup file in panel
   if {$file != ""} {
     if {[file exists $file]} {
       set status [slxtclLoadSetup $file $gvar(oslxId)]
     }
   } else {;          # no file to be loaded
     return $status
   }
   if {$status == "FAILURE"} {return $status}

   # update widgets not directly associated to setup variables
   # exposure setup options
   set gvar(fcdExpOpt_nrep) $gvar(setup.2.DET.EXP.NREP); # repeated exposures
   if {$gvar(fcdExpOpt_nrep) == 1} {
     set gvar(fcdExpOpt_rep) 0
   } else {set gvar(fcdExpOpt_rep) 1}
   if {$gvar(fcdExpOpt_nrep) >= 1} {
     set gvar(fcdExpOpt_expNrep) $gvar(fcdExpOpt_nrep)
     set gvar(fcdExpOpt_nrep) 2
   }

   # image setup
   set rtdFrame $gvar(setup.9.DET.DISPLAY)
   set rtd 0
   set rtdBld 0
   if {$rtdFrame >= 0} {
     set rtd 1
     if {$rtdFrame > 0} {
       set rtdBld 1
     }
   }
   set gvar(rtd) $rtd
   set gvar(imageOpt_rtdBld) $rtdBld

   # readout setup
   for { set i 1} { $i <= $fcdMaxWin } {incr i} {; # window readout and ip
     set gvar(fcdWin${i}_Strx) $gvar(setup.${i}11.DET.WIN$i.STRX) 
     set gvar(fcdWin${i}_Nx)   $gvar(setup.${i}12.DET.WIN$i.NX)   
     set gvar(fcdWin${i}_Stry) $gvar(setup.${i}13.DET.WIN$i.STRY) 
     set gvar(fcdWin${i}_Ny)   $gvar(setup.${i}14.DET.WIN$i.NY)

     set gvar(fcdSetupIp${i}_ipMinMax)   $gvar(setup.${i}23.DET.WIN${i}.MINMAX)
     set gvar(fcdSetupIp${i}_ipRMS)  $gvar(setup.${i}27.DET.WIN${i}.RMS)
     set gvar(fcdSetupIp${i}_ipCentroid) $gvar(setup.${i}20.DET.WIN${i}.CENTROID)
     set gvar(fcdSetupIp${i}_ipRefX) $gvar(setup.${i}21.DET.WIN${i}.REFX)
     set gvar(fcdSetupIp${i}_ipRefY) $gvar(setup.${i}22.DET.WIN${i}.REFY)
   }
   # enable windowing
   fcdEnableWindowing

   return $status
}


# save in a setup file the contents of a panel
proc fcdSaveSetup {file} {
   global cv
   global $cv.fcdSetupIp1.ckShow $cv.fcdSetupIp2.ckShow 
   global $cv.fcdWindowFrame1.ckShow $cv.fcdWindowFrame2.ckShow
   global $cv.fcdExpOpt.ckShow $cv.imageOpt.ckShow
   global gvar insroot insrootSetupDet
   global vltroot introot did
   global fcdMaxWin

   # get complete file path
   set file [fcdSetupFilePath $file]

   # set variable values according to class container button status
   for { set i 1} { $i <= $fcdMaxWin } {incr i} {
     set gvar(setup.${i}21.DET.WIN${i}.REFX) $gvar(fcdSetupIp${i}_ipRefX)
     set gvar(setup.${i}22.DET.WIN${i}.REFY) $gvar(fcdSetupIp${i}_ipRefY)
     set status [set $cv.fcdSetupIp${i}.ckShow];
     if {$status == 0} {
       set gvar(setup.${i}23.DET.WIN${i}.MINMAX) F
       set gvar(setup.${i}20.DET.WIN${i}.CENTROID) none
       set gvar(setup.${i}27.DET.WIN${i}.RMS) 0
       set gvar(setup.${i}25.DET.WIN${i}.IPFUNC) None
     } else {
       set gvar(setup.${i}23.DET.WIN${i}.MINMAX) $gvar(fcdSetupIp${i}_ipMinMax)
       set gvar(setup.${i}20.DET.WIN${i}.CENTROID) $gvar(fcdSetupIp${i}_ipCentroid)
       set gvar(setup.${i}27.DET.WIN${i}.RMS) $gvar(fcdSetupIp${i}_ipRMS)
       if {$gvar(fcdSetupIp${i}_ipUsrFlag)} {; # user function used
         set gvar(setup.${i}25.DET.WIN${i}.IPFUNC) $gvar(fcdSetupIp${i}_ipUsrFunc)
	 }
     }
     set status [set $cv.fcdWindowFrame$i.ckShow]
     if {$status == 0} {
       set gvar(setup.${i}10.DET.WIN$i.ST) F
     } else {
       set gvar(setup.${i}10.DET.WIN$i.ST) T
     }
     set gvar(setup.${i}11.DET.WIN$i.STRX) $gvar(fcdWin${i}_Strx)
     set gvar(setup.${i}12.DET.WIN$i.NX)   $gvar(fcdWin${i}_Nx)
     set gvar(setup.${i}13.DET.WIN$i.STRY) $gvar(fcdWin${i}_Stry)
     set gvar(setup.${i}14.DET.WIN$i.NY)   $gvar(fcdWin${i}_Ny)
   }

   # exposure setup options
   set status [set $cv.fcdExpOpt.ckShow];
   if {$status == 0} {
     set gvar(fcdExpOpt_setup.0.DET.EXP.TYPE) Normal;
     set gvar(fcdExpOpt_rep) 0
     set gvar(fcdExpOpt_setup.40.DET.EXP.TIMEREP) 0;
   }
   if {$gvar(fcdExpOpt_rep) == 0} {set nrep 1
   } elseif {$gvar(fcdExpOpt_nrep) == 2} {set nrep $gvar(fcdExpOpt_expNrep)
   } else {set nrep $gvar(fcdExpOpt_nrep)}  
   set gvar(setup.2.DET.EXP.NREP) $nrep

   # image setup options
   set status [set $cv.imageOpt.ckShow];
   if {$status == 0} {
     set gvar(imageOpt_setup.30.DET.FRAM.TYPE) Normal;
     set gvar(imageOpt_setup.24.DET.FRAM.SAMPLE) 1;
     set gvar(imageOpt_rtdBld) 0
   }
   if {$gvar(rtd)} {
     if {$gvar(imageOpt_rtdBld)} {
       set gvar(setup.9.DET.DISPLAY) 3;  # no build up effect
     } else {
       set gvar(setup.9.DET.DISPLAY) 0;  # main frame
     }
   } else {
     set gvar(setup.9.DET.DISPLAY) -1;   # no display
   }
   # save setup

   if {$file != ""} {
      set status [slxtclSaveSetup $file "Detector Setup" $gvar(oslxId)]
   }

   #
   # Return result
   #
   return $status
}



proc fcdSetupDefaults {} {
   global cv gvar
   global fcdMaxWin

   panSetDefaults $cv
   for { set i 1} { $i <= $fcdMaxWin } {incr i} {
     # no windowed readout
     set gvar(setup.${i}10.DET.WIN$i.ST) F

     # no image processing
     set gvar(fcdSetupIp${i}_ipMinMax) F
     set gvar(fcdSetupIp${i}_ipRMS) 0
     set gvar(fcdSetupIp${i}_ipCentroid) none
   }
   fcdEnableWindowing
}
#
# ___oOo___


