#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdTelemetry.tcl,v 2.83 2005/09/30 08:26:37 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  10/03/97  created
# rdonalds  22/05/98  ported from ccd S/W
# abalestr  26/03/03  ported plotting to ccs_lite environment 
#                     (see SPR no. VLTSW20030103)
# abortolu  01/04/05  Adapted to the new sizes of the OLDB attributes 
# abortolu  09/05/05  Changed telemetry default polling rate 
#

#************************************************************************
#   NAME 
#   fcdTelTemInit, fcdFormatTelTemStatus, fcdTelTemAddCurr,
#   fcdTelTemDisplay, fcdTemperatureInitialise, fcdTelemetryInitialise,
#   fcdTelPlotStart, fcdTelPlotStop, fcdTemPlotStart, fcdTemPlotStop
#   fcdFormatTelStatus, fcdFormatTemStatus
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   These procedures are used within the fcdTelemetry panel to handle the 
#   telemetry parameters
#
#------------------------------------------------------------------------
#

#
# Initialise variables for telemetry and temperature list boxes
proc fcdTelTemInit { } {

   global telIndex

   set telIndex(0) 6;  # id
   set telIndex(1) 5;  # description
   set telIndex(2) 7;  # units
   set telIndex(3) 10; # reference
   set telIndex(4) 9;  # current, replacing high level, unused in this context
}




# formats on row in the telemetry or temperature list box
proc fcdFormatTelTemStatus { index text } {

   upvar $index fcdIndex

   set idInd   $fcdIndex(0)
   set descInd $fcdIndex(1)
   set unitInd $fcdIndex(2)
   set refInd  $fcdIndex(3)
   set currInd $fcdIndex(4)

   set indStr  [lindex $text $idInd]
   set descStr [lindex $text $descInd]
   set unitStr [lindex $text $unitInd]
   set refVal  [lindex $text $refInd]
   set currVal [lindex $text $currInd]

   if { ([expr {abs($refVal)}]  > 0.1 || $refVal  == 0) &&
        ([expr {abs($currVal)}] > 0.1 || $currVal == 0) } {
       set formatStr "%-16s %-64s %-16s %7.1f  %7.1f"
   } elseif { ([expr {abs($refVal)}] > 0.1 || $refVal == 0) &&
               [expr {abs($currVal)}] < 0.1 } {
       set formatStr "%-16s %-64s %-16s %7.1f  %7.0e"
   } elseif { [expr {abs($refVal)}] < 0.1 &&
             ([expr {abs($currVal)}] > 0.1 || $currVal == 0) } {
       set formatStr "%-16s %-64s %-16s %7.0e  %7.1f"
   } else {
       set formatStr "%-16s %-64s %-16s %7.0e  %7.0e"
   }

   return [format $formatStr $indStr $descStr $unitStr $refVal $currVal]
}


# add current value to table of available elements
proc fcdTelTemAddCurr { avail fcdCurr } {

  global telIndex

  upvar $avail fcdAvail

  set elList [array names fcdAvail]
  if {[llength $elList] > 0} {
    # high level (unused in this context) replaced with current    
    set pos $telIndex(4)
    foreach rec $elList {
      set ind [lindex $fcdAvail($rec) 0]
      set val [lindex $fcdCurr $ind]
      set fcdAvail($rec) [lreplace $fcdAvail($rec) $pos $pos $val]
    }
  }
}





#
# read current telemetry or temperature values and display in list box
proc fcdTelTemDisplay { dbCurr avail wdg formatProc } {

  upvar $avail fcdAvail

  # temporarily read here current value. Should be done on trace from 
  # telemetry.current(0)
  set current [dbRead $dbCurr]

  # merge current values in table
  fcdTelTemAddCurr fcdAvail $current

  # display list of telemetry status
  fcdDisplayBox fcdAvail $wdg $formatProc
}




#
# Initialise telemetry variables
proc fcdTelemetryInitialise { } {

  global gvar
  global telAvail
  global telDbCurr telWdg telFormatProc

  # set default polling rate
  set gvar(polltel) 900

  # read from db and build table
  fcdTelInitialise

  # declare indexes of elements in table
  fcdTelTemInit

  # read current values and display in list box
  fcdTelTemDisplay $telDbCurr telAvail $telWdg $telFormatProc

  # update buttons
  fcdTelUpdateButtons
}


# formats one row in the telemetry status list box
proc fcdFormatTelStatus { text } {

  global telIndex
 
  fcdFormatTelTemStatus telIndex $text
}


# Enable/Disable telemetry buttons
proc fcdTelUpdateButtons { {opState -1} } {

  global cv gvar
  global telAvail telDbAttr

  set gvar(fcdTelIdle) 0
  set gvar(fcdTelActive) 0
  if {$opState == -1} {; #invalid value. Read state from DB
    set opState [dbRead "telemetry.opState"]
  }
  if {$opState == 4} {set gvar(fcdTelIdle) 1
  } elseif {$opState == 5} {
    set gvar(fcdTelActive) 1
  }

  # enter options for plotting
  set fcdList [fcdTelTemGetList telAvail telDbAttr]
  set fcdList [linsert $fcdList 0 ""]; # empty entry
  for { set i 1} { $i <= 4 } {incr i} {
    $cv.fcdTelPlot$i configure -itemList $fcdList
  }
}

# set or reset variables to be traced within fcdTelemetry panel
proc fcdTelemetryTrace {op} {

   global gvar

 # Trace change of state to enable/disable buttons
   trace $op gvar(telemetry.opState) w fcdExecTelTrace
 # Trace operations needed because of renaming of variables in classes
   trace $op gvar(telemetry.current(0)) w fcdExecTelTrace
}


# trace procedure for fcdCtrl panel
proc fcdExecTelTrace {name element op} {

  global telDbCurr telAvail telWdg telFormatProc
  global temDbCurr temAvail temWdg temFormatProc
  global gvar

  if {![string compare $element "telemetry.opState"]} {
    fcdTelUpdateButtons $gvar($element)
  } elseif {![string compare $element "telemetry.current(0)"]} {
    fcdTelTemDisplay $telDbCurr telAvail $telWdg $telFormatProc
  } 
}


# start plotting telemetry or temperature values
proc fcdTelTemPlotStart { table attr title } {
  global wsenv

  eval exec sampCtrl_stop.sh $table $wsenv WS
  sleep 1
  eval exec sampCtrl_start.sh 0 \"\" $table $wsenv WS 1 1 $attr
}


# start plotting telemetry values
proc fcdTelPlotStart {} {
  global gvar camera telAvail

  set attr ""
  for { set i 1} { $i <= 4 } {incr i} {
    set selected $gvar(fcdTelPlot$i)
    set rec [fcdTelTemGetListRec $selected telAvail]
    if {$rec >= 0} {
      set index [lindex $telAvail($rec)  0]
      set dbAttr "$camera:telemetry.current($index)"
      set attr "$attr \"$dbAttr\""
    }
  }
  fcdTelTemPlotStart "$camera:telemetry:plot" $attr "$camera telemetry"
}

# stop plotting telemetry or temperature values
proc fcdTelTemPlotStop {table} {
  global wsenv

  eval exec sampCtrl_stop.sh $table $wsenv WS
}


# stop plotting telemetry values
proc fcdTelPlotStop {} {
  global camera

  fcdTelTemPlotStop "$camera:telemetry:plot"
}


#
# Get list of parameters enabled
proc fcdTelTemGetList { avail dbAttr } {

  upvar $avail fcdAvail
  upvar $dbAttr fcdDbAttr

  set fcdList {}
  set available [dbRead $fcdDbAttr(avail)]
  if {$available} {; # telemetry system is available
    set elList [array names fcdAvail]
    set elList [lsort -integer $elList]
    if {[llength $elList]} {
      foreach rec $elList {
        set newEntry [lindex $fcdAvail($rec) 6]
        set fcdList [lappend fcdList $newEntry]
      }
    }
  }
  return $fcdList
}

#
# Get record of an element in the list of parameters enabled
proc fcdTelTemGetListRec { entry avail } {

  upvar $avail fcdAvail

  set found 0
  set elList [array names fcdAvail]
  if {[llength $elList]} {
  # look for selected option in the list of available entries
    foreach rec $elList {
      set val [lindex $fcdAvail($rec) 6]
      if {![string compare $entry $val]} { 
        set found 1
        break 
      }
    }
  }
  if {$found == 0} {set rec -1}
  return $rec
}

#
# ___oOo___
