
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdUsrGlobalData.tcl,v 2.83 2005/09/30 08:26:32 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  09/07/97  created (ported from CCD software)
# rdonalds  22/05/98  Added gvar(fcdcon) set to fcdcon or fcdconCI depending
#                     upon whether we are on a SPARC or HP
#

#************************************************************************
#   NAME
#   fcdUsrGlobalData - assign a default value to common global variables
# 
#   SYNOPSIS
#   fcdUsrGlobalData
# 
#   DESCRIPTION
#   Tcl procedure, used within FIERA GUI panels, to assign a default value to
#   common global variables, in particular CWP.
#
#   FILES
#
#   ENVIRONMENT
#   CCDNAME     name of camera used
#   RTAPENV     name of WS environment where FIERA sw runs
#   CCDLENV     name of SLCU environment where FIERA sw runs
#   INS_ROOT    root directory for instrument data
#   INTROOT     integration root directory
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#   fcdUsrGlobalData
#
#   SEE ALSO
#   panUsrGlobalData
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc fcdUsrGlobalData { } {        

 # Declare any variable as global

   global cwp gvar env camera
   global insroot simul wsenv slcuenv insrootSetupDet
   global vltroot introot did
   global cwpTmp
   global fcdMaxWin

 # Variable Initialization
   set camera "undefined"
   if {[info exists env(CCDNAME)] == 1} {
       set camera $env(CCDNAME)
       set cwp "<alias>$camera" ;      # cwp normally passed as runtime par.
   }
   set gvar(camera) $camera

   if {[info exists cwpTmp] == 1} {;   # temporary for bug in panel 2.57
       set cwp $cwpTmp ;
   }

   set wsenv "undefined"
   if {[info exists env(RTAPENV)] == 1} {
       set wsenv "$env(RTAPENV)" ;           # WS FIERA environment
   }
   set slcuenv "undefined"
   if {[info exists env(CCDLENV)] == 1} {
       set slcuenv "$env(CCDLENV)" ;          # SLCU FIERA environment
   }

   set insroot "undefined"
   if {[info exists env(INS_ROOT)] == 1} {
       set insroot "$env(INS_ROOT)" ;   # Instrument root directory
   }

   set vltroot "$env(VLTROOT)"
   set introot "undefined"
   if {[info exists env(INTROOT)] == 1} {
       set introot "$env(INTROOT)" ;    # Integration directory
   } elseif {[info exists env(VLTROOT)] == 1} {
       set introot $vltroot ;
   }

   set did CCDDCS;                      # default
   if {[info exists env(CCDDID)] == 1} {
       set did "$env(CCDDID)" ;         # Data Dictionary name
   }
   
   set did "$did FCDDCS";               # default
   if {[info exists env(FCDDID)] == 1} {
       set did "$did $env(FCDDID)" ;    # Data Dictionary name
   }

   set simul 0;                         # default: no actions simulation
   set insrootSetupDet $insroot/SYSTEM/COMMON/SETUPFILES/DET

   set fcdMaxWin 2;  # maximum 2 windows supported
   set gvar(fcdEXP_NEXT) -1
   set gvar(fcdEXP_LAST) 0
   
   # are we on the SPARC or the HP ?
   set os [exec uname]
   if { $os == "SunOS" } {
       set gvar(fcdcon) fcdcon_$camera
   } else {
       set gvar(fcdcon) fcdconCI_$camera
   }
       
}

set gvar(maxChip) 32

#
# ___oOo___


