#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdUifMisc.tcl,v 2.23 2005/09/12 13:07:41 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rdonalds  27/10/97  created
# rdonalds  05/11/98  Added CallEngPanel needed because the cwp was being
#                     changed when the Engineering panel was called
# rdonalds  20/11/98  Set CENTROID to none
# abalestr  22/07/04  Timeout on ONLINE command doubled in order to cope with
#                     omegacam case
# ccumani   19/08/05  all parameters are passed to fcdSlcuStartupCamera
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
# Send command
#
proc fcdSendCmd {cmd  {arguments ""} {tmo 10000} } {
    global env gvar

    set result [panSendCommand $env(RTAPENV) $gvar(fcdcon) $cmd $gvar(checkCommands) $tmo $arguments]

    return $result
}
#
# fcdChangeTime
#
proc fcdChangeTime {} {
    global gvar env

    set result [fcdSendCmd SETUP "-expoId 0 -function DET.WIN1.UIT1 $gvar(fcdExpSetup_intTime)" 20000]

    return $result
}
# fcdUserChangeTime
#
# Called when the user hits the Change Time button
#
proc fcdUserChangeTime {} {
    global gvar env

    # Busy now
    fcdBusyCursor

    # Send a set command to FCDCON
    fcdChangeTime

    # Not Busy now
    fcdNormalCursor
}
# fcdUserWipe
#
# Called when the user hits the Wipe button
#
proc fcdUserWipe {} {
    global gvar env

    # Busy now
    fcdBusyCursor

    # Send a set command to FCDCON
    # Send STARTWP command Allways assume periodic wipe
    fcdSendCmd STARTWP "-periodic 1"

    # Not Busy now
    fcdNormalCursor
}

proc fcdStopWipe {} {
    global gvar env

    fcdSendCmd STOPWP
}

proc fcdUserStopWipe {} {
    global gvar env

    fcdBusyCursor
    fcdStopWipe
    fcdNormalCursor
}
proc fcdInit {} {
    global gvar env

    # Get the env var which says that we are debugging
    if {[catch {set DEBUGGING_C40 $env(DEBUGGING_C40)}]} {set DEBUGGING_C40 0}

    # Use that var to determine the timeout -- no timeout when debugging
    if {$DEBUGGING_C40 != "0"} {
	set timeout 4000000000
    } else {
	set timeout 60000
    }
    # puts "Calling INIT from panel with timeout $timeout"
    fcdSendCmd INIT  "0" $timeout

    # Record the fact that an INIT command has been done
    set gvar(fcdInitSent) 1;
}
proc fcdUserInit {} {

    global gvar env

    fcdBusyCursor

    fcdInit

    fcdNormalCursor
}
#
# fcdExit
#
# Exit called from internal level
#
proc fcdExit {} {
    global gvar env

    fcdSendCmd EXIT "" 20000
}
#
# fcdUserExit
#
# Exit called from user level
#
proc fcdUserExit {} {
    global gvar env

    fcdBusyCursor
    # Make sure everything is stopped before exit
    fcdExit
    fcdNormalCursor
}
#
# fcdShutdown
#
# Shutdown internal call
#
proc fcdShutdown {} {
    # Make sure everything is stopped before exit

    # Now call the shutdown script
    #exec xterm -e fstopfcd
    fcdExit
}
#
# fcdUserShutdown
#
# Shutdown called from user level
#
proc fcdUserShutdown {} {
    global gvar env

    # Create a message which indicates the severity of the
    # action
    messagedialog .pd -modality global -bitmap error \
	-text "Are you sure you want to shutdown"
    

    # Center the dialoge on the screen
    .pd center

    # We use only the OK and Cancel buttons
    .pd hide Help
    .pd hide Apply

    # Change OK text 
    .pd buttonconfigure OK -text Confirm
    # Default is to cancel
    .pd default Cancel

    # See what the Result Is
    set res [.pd activate]
    switch $res {
	1 {
	fcdBusyCursor

	# Perform shutdown
	fcdShutdown

	fcdNormalCursor
	}
    }
    destroy .pd
}

proc fcdStartup {} {
    global gvar env

    # Now call the startup script for the camera
    # with a default mode of 2 which can be overriden with env var
    # OP_MODE
    if {[catch {set OP_MODE $env(OP_MODE)}]} {set OP_MODE 2}
    if {[catch {set INS_USER $env(INS_USER)}]} {set INS_USER SYSTEM}
    if {[catch {set CCDLENV $env(CCDLENV)}]} {set CCDLENV 0}
    if {[catch {set CCDID $env(CCDID)}]} {set CCDID CCDDCS}
    if {[catch {set FCDID $env(FCDID)}]} {set FCDID FCDDCS}
    if {[catch {set FCD_DSP_EXTERNAL_RESET $env(FCD_DSP_EXTERNAL_RESET)}]} {set FCD_DSP_EXTERNAL_RESET ""}
    # puts "FCD_DSP_EXTERNAL_RESET is $FCD_DSP_EXTERNAL_RESET"
    exec xterm -e fcdSlcuStartupCamera $gvar(CCDNAME) $OP_MODE $env(RTAPENV) $env(INS_HOST) $INS_USER $CCDLENV $env(INS_ROOT) $CCDID $FCDID $FCD_DSP_EXTERNAL_RESET
}

proc fcdUserStartup {} {
    global gvar env

    set state  [dbRead $gvar(cwp).opStateSlcu]
    # puts "state is $state"

    #
    # If the system says that it is off then do the action
    # otherwise do a dialogue
    #
    if {$state <= 1} {
	set performStartup 1
    } else {
	# Create a message which indicates the severity of the
	# action
	messagedialog .pd -modality global -bitmap error \
	    -text "Startup will destroy any existing processes"
	

	# Center the dialoge on the screen
	.pd center

	# We use only the OK and Cancel buttons
	.pd hide Help
	.pd hide Apply

	# Default is to cancel
	.pd default Cancel
	.pd buttonconfigure OK -text Continue

	# See what the Result Is
	set performStartup [.pd activate]

	destroy .pd
    }

    #
    # Now what we do depends upon the result
    if {$performStartup == "1"} {
	fcdBusyCursor

	# Perform startup
	fcdStartup

	fcdNormalCursor
    }
}
#
#
proc fcdStart {} {
    global gvar env

    # No absolute time or epxosure id passed by default
    set startParams ""

    # Set the ABS time as a string. This is not the normal way of
    # doing this. It should be done as an MJD, therefore we use
    # a special SET command to do this
    # Only send the start time if the start is absolute
    if {$gvar(fcdExpSetup_startTimeAbs) == "T"} {
	set startParams "-at $gvar(fcdExpSetup_startTimeStr)"
    }

    set startResult [fcdSendCmd START $startParams 20000]
    
    puts "Start returns $startResult"
}
#
# fcdUserStart
#
# Start an exposure called by a user
#
proc fcdUserStart {} {
    global gvar env

    fcdBusyCursor

    #
    # Perform the setup and check the result
    #
    set status [fcdSetup] 
    if { $status == "FAILURE"} {
	fcdNormalCursor
	return FAILURE
    }

    #
    # Perform the engineering setup, allowing the sequences
    # defined in the mode to be overridden
    #
    set status [fcdEngSetup] 
    if { $status == "FAILURE"} {
	fcdNormalCursor
	return FAILURE
    }
    
    fcdStart

    fcdNormalCursor
}

#
# Wait
#
# mode can be either Single or Global
#
proc fcdWait { mode } {
    global gvar env

    # Wait limited to 4 minutes
    fcdSendCmd WAIT "-waitMode $mode" 240000
}
#
# fcdUserWait
#
# Wait for exposure to complete (either single or global)
#
proc fcdUserWait {} {
    global gvar env

    fcdBusyCursor

    #
    # Perform the Wait
    #
    fcdWait Single

    fcdNormalCursor
}

proc fcdPause {} {
    fcdSendCmd PAUSE
}
proc fcdUserPause {} {
    global gvar env

    fcdBusyCursor
    fcdPause
    fcdNormalCursor
}

proc fcdContinue {} {
    fcdSendCmd CONT
}
proc fcdUserContinue {} {
    global gvar env

    fcdBusyCursor
    fcdContinue
    fcdNormalCursor
}

proc fcdStop {} {
    global gvar env

    fcdSendCmd STOP "" 20000
}

proc fcdUserStop {} {
    global gvar env

    fcdBusyCursor
    fcdStop
    fcdNormalCursor
}

proc fcdUserStartCamera {} {
    global gvar env

    fcdBusyCursor

    # Store the PID from the execution so we can Kill it later if required
    set gvar(rtdPID) [exec rtd -camera $gvar(CCDNAME) &]
    fcdNormalCursor
}

proc fcdAbort {} {
    fcdSendCmd ABORT "" 20000
}
proc fcdUserAbort {} {
    global gvar env

    fcdBusyCursor
    fcdAbort
    fcdNormalCursor
}

proc fcdEnd {} {
    fcdSendCmd END "" 20000
}
proc fcdUserEnd {} {
    global gvar env

    fcdBusyCursor
    fcdEnd
    fcdNormalCursor
}
#
# Engineering parameters which are used to override the mode selection
#
proc fcdEngSetup { } {                              
    global env gvar
    
    if {[string length $gvar(fcdEngSetup_wipeSeq)] != 0} {
	fcdSendCmd SET "-setupSet ENG.WIPESEQ $gvar(fcdEngSetup_wipeSeq)"
    }
    if {[string length $gvar(fcdEngSetup_intSeq)] != 0} {
	fcdSendCmd SET "-setupSet ENG.INTSEQ $gvar(fcdEngSetup_intSeq)"
    }
    if {[string length $gvar(fcdEngSetup_readSeq)] != 0} {
	fcdSendCmd SET "-setupSet ENG.READSEQ $gvar(fcdEngSetup_readSeq)"
    }
}

#########################################################################
# # Manipulate the setup buffer #
#########################################################################
proc fcdSetupBufReset {} {
    global gvar

    set gvar(setupBuffer) "-expoId -1 -function "
}
proc fcdSetupBufAppend { args } {
    global gvar

    set gvar(setupBuffer) [concat $gvar(setupBuffer) [join $args]]
}
proc fcdSetupBufAppendLogical { name value } {
    global gvar

    if { $value == 1 } {
	set log T
    } else {
	set log F
    }
    set gvar(setupBuffer) [concat $gvar(setupBuffer) [join $name] [join $log]]
}
proc fcdSetup { } {                                  
    global gvar env

    # Initialise the setup buffer
    fcdSetupBufReset;

    # Set the mode index
    fcdSetupBufAppend "DET.READ.CLKIND $gvar(fcdReadMode_readMode)"

    # Integration time
    fcdSetupBufAppend "DET.WIN1.UIT1 $gvar(fcdExpSetup_intTime)"

    # Binning (only square at the moment)

    # Set the DISK Save type
    fcdSetupBufAppend "DET.FRAM.FITSMTD $gvar(fcdExpSetup_saveType)"

    # Set exposure type
    fcdSetupBufAppend "DET.EXP.TYPE $gvar(fcdExpSetup_expType)"

    fcdSetupBufAppend "DET.FRAM.FITSUNC $gvar(fcdExpSetup_saveFileName)"
    fcdSetupBufAppend "DET.FRAM.SAMPLE $gvar(fcdExpSetup_sampling)"

    fcdSetupBufAppendLogical DET.WIN1.ST  $gvar(fcdExpSetup_win1_winEnabled)
    fcdSetupBufAppend "DET.WIN1.BINX $gvar(fcdExpSetup_xBinning)"
    fcdSetupBufAppend "DET.WIN1.BINY $gvar(fcdExpSetup_yBinning)"
    fcdSetupBufAppend "DET.WIN1.STRX $gvar(fcdExpSetup_win1_xFirst)"
    fcdSetupBufAppend "DET.WIN1.STRY $gvar(fcdExpSetup_win1_yFirst)"
    fcdSetupBufAppend "DET.WIN1.NX $gvar(fcdExpSetup_win1_xDim)"
    fcdSetupBufAppend "DET.WIN1.NY $gvar(fcdExpSetup_win1_yDim)"

    fcdSetupBufAppendLogical DET.WIN2.ST  $gvar(fcdExpSetup_win2_winEnabled)
    fcdSetupBufAppend "DET.WIN2.BINX $gvar(fcdExpSetup_xBinning)"
    fcdSetupBufAppend "DET.WIN2.BINY $gvar(fcdExpSetup_yBinning)"
    fcdSetupBufAppend "DET.WIN2.STRX $gvar(fcdExpSetup_win2_xFirst)"
    fcdSetupBufAppend "DET.WIN2.STRY $gvar(fcdExpSetup_win2_yFirst)"
    fcdSetupBufAppend "DET.WIN2.NX $gvar(fcdExpSetup_win2_xDim)"
    fcdSetupBufAppend "DET.WIN2.NY $gvar(fcdExpSetup_win2_yDim)"

    # The exposure repetition count is used as part of status display
    # so we copy it into a global variable
    if {$gvar(fcdExpSetup_repType) == "2"} {
	set gvar(fcdExpRepCount) $gvar(fcdExpSetup_repCount)
    } else {
	set gvar(fcdExpRepCount) $gvar(fcdExpSetup_repType)
    }
    fcdSetupBufAppend "DET.EXP.NREP    $gvar(fcdExpRepCount)"
    fcdSetupBufAppend "DET.EXP.TIMEREP $gvar(fcdExpSetup_loopTime)"
    fcdSetupBufAppend "DET.EXP.TIMEABS $gvar(fcdExpSetup_startTimeAbs)"

    # Dont do min/max 
    fcdSetupBufAppendLogical DET.WIN1.MINMAX 0
    fcdSetupBufAppend DET.WIN1.CENTROID none
    fcdSetupBufAppendLogical DET.WIN2.MINMAX 0
    fcdSetupBufAppend DET.WIN1.CENTROID none

    if {$gvar(fcdExpSetup_win1_sigmaCalc) == "None"} {
	fcdSetupBufAppend "DET.WIN1.RMS 0"
    }
    if {$gvar(fcdExpSetup_win1_sigmaCalc) == "Whole"} {
	fcdSetupBufAppend "DET.WIN1.RMS 1"
    }
    if {$gvar(fcdExpSetup_win1_sigmaCalc) == "Central"} {
	fcdSetupBufAppend "DET.WIN1.RMS 2"
    }
    if {$gvar(fcdExpSetup_win1_sigmaCalc) == "Window"} {
	fcdSetupBufAppend "DET.WIN1.RMS 3"
    }
    if {$gvar(fcdExpSetup_win2_sigmaCalc) == "None"} {
	fcdSetupBufAppend "DET.WIN2.RMS 0"
    }
    if {$gvar(fcdExpSetup_win2_sigmaCalc) == "Whole"} {
	fcdSetupBufAppend "DET.WIN2.RMS 1"
    }
    if {$gvar(fcdExpSetup_win2_sigmaCalc) == "Central"} {
	fcdSetupBufAppend "DET.WIN2.RMS 2"
    }
    if {$gvar(fcdExpSetup_win2_sigmaCalc) == "Window"} {
	fcdSetupBufAppend "DET.WIN2.RMS 3"
    }
    fcdSetupBufAppend "DET.WIN1.IPLLX $gvar(fcdExpSetup_win1_procLLX)"
    fcdSetupBufAppend "DET.WIN1.IPLLY $gvar(fcdExpSetup_win1_procLLY)"
    fcdSetupBufAppend "DET.WIN2.IPLLX $gvar(fcdExpSetup_win2_procLLX)"
    fcdSetupBufAppend "DET.WIN2.IPLLY $gvar(fcdExpSetup_win2_procLLY)"
    #
    # The UR coordinates are defined as being an offset from the upper
    # right. This is not a good number to ask the user to input. 
    #
    # So we have cheated by interpreting negative coordinates as a window
    # delta from the LLX LLY
    # 
    set tmpur $gvar(fcdExpSetup_win1_procURX)
    fcdSetupBufAppend "DET.WIN1.IPURX -$tmpur"

    set tmpur $gvar(fcdExpSetup_win1_procURY)
    fcdSetupBufAppend "DET.WIN1.IPURY -$tmpur"

    set tmpur $gvar(fcdExpSetup_win2_procURX)
    fcdSetupBufAppend "DET.WIN2.IPURX -$tmpur"

    set tmpur $gvar(fcdExpSetup_win2_procURY)
    fcdSetupBufAppend "DET.WIN2.IPURY -$tmpur"

    fcdSetupBufAppend "DET.DISPLAY $gvar(fcdExpSetup_rtdDisplay)"
    
    #puts "Send buffer $gvar(setupBuffer)"
    #puts "Length is [string length $gvar(setupBuffer)]"

    set result [fcdSendCmd SETUP $gvar(setupBuffer) 20000]

    if { $result == "FAILURE" } {
	return $result
    }

    return $result
}

proc fcdSet { value } {                              
    global env gvar

    fcdSendCmd SET $value
}

proc fcdUserGetAnaVoltage { } {
    global env gvar

    set board $gvar(fcdAnaVoltControl_boardName);
    set periph $gvar(fcdAnaVoltControl_periphName);

    set result [fcdGetVoltage $board $periph];

    set gvar(fcdAnaVoltControl_periphValue) $result;
}

proc fcdUserGetVidVoltage { } {
    global env gvar

    set board $gvar(fcdVidVoltControl_boardName);
    set periph $gvar(fcdVidVoltControl_periphName);

    set result [fcdGetVoltage $board $periph];

    set gvar(fcdVidVoltControl_periphValue) $result;
}

#
# Get the values to display in the new video control panel
#
proc fcdNVidVltControlGet { string } {
    global env gvar

    set board $gvar(fcdNVidVltControl_boardName);

    set result [fcdGetVoltage $board VID_GAIN1_CHAN$gvar(fcdNVidVltControl_channel)];
    set gvar(fcdNVidVltControl_gain1) $result;
    set result [fcdGetVoltage $board VID_GAIN2_CHAN$gvar(fcdNVidVltControl_channel)];
    set gvar(fcdNVidVltControl_gain2) $result;
    set result [fcdGetVoltage $board VID_FILT_CHAN$gvar(fcdNVidVltControl_channel)];
    set gvar(fcdNVidVltControl_filter) $result;
    set result [fcdGetVoltage $board VID_OFFSET_CHAN$gvar(fcdNVidVltControl_channel)];
    set gvar(fcdNVidVltControl_offset) $result;
}

proc fcdNVidVltControlSet { } {
    global env gvar

    set board $gvar(fcdNVidVltControl_boardName);

    if {$gvar(fcdNVidVltControl_filter) == ""} {
        set gvar(fcdNVidVltControl_filter) 0
    }
    if {$gvar(fcdNVidVltControl_offset) == ""} {
        set gvar(fcdNVidVltControl_offset) 0
    }

    fcdSetVoltage $board VID_GAIN1_CHAN$gvar(fcdNVidVltControl_channel) $gvar(fcdNVidVltControl_gain1);
    fcdSetVoltage $board VID_GAIN2_CHAN$gvar(fcdNVidVltControl_channel) $gvar(fcdNVidVltControl_gain2);
    fcdSetVoltage $board VID_FILT_CHAN$gvar(fcdNVidVltControl_channel) $gvar(fcdNVidVltControl_filter);
    fcdSetVoltage $board VID_OFFSET_CHAN$gvar(fcdNVidVltControl_channel) $gvar(fcdNVidVltControl_offset);
}

proc fcdUserGetClkVoltage { } {
    global env gvar

    set board $gvar(fcdClkVoltControl_boardName);
    set periph $gvar(fcdClkVoltControl_periphName);

    set result [fcdGetVoltage $board $periph];

    set gvar(fcdClkVoltControl_periphValue) $result;
}

proc fcdGetVoltage { board periph} {                              
    global env gvar

    set result [fcdSendCmd GET "-voltage $board $periph"]

    return $result
}

proc fcdUserSetAnaVoltage { } {
    global env gvar

    set board $gvar(fcdAnaVoltControl_boardName);
    set periph $gvar(fcdAnaVoltControl_periphName);
    set value $gvar(fcdAnaVoltControl_periphValue);

    fcdSetVoltage $board $periph $value;
}

proc fcdUserSetVidVoltage { } {
    global env gvar

    set board $gvar(fcdVidVoltControl_boardName);
    set periph $gvar(fcdVidVoltControl_periphName);
    set value $gvar(fcdVidVoltControl_periphValue);

    fcdSetVoltage $board $periph $value;
}

proc fcdUserVidBrdStatusRefresh { } {
    global env gvar

    set board $gvar(fcdVidBrdStatus_vidBrd);

    set result [fcdGetVoltage $board VID_GAIN1_CHAN0];
    set gvar(fcdVidBrdStatus_vidChan0Gain1) $result;
    set result [fcdGetVoltage $board VID_GAIN2_CHAN0];
    set gvar(fcdVidBrdStatus_vidChan0Gain2) $result;
    set result [fcdGetVoltage $board VID_FILTER_CHAN0];
    set gvar(fcdVidBrdStatus_vidChan0Filter) $result;
    set result [fcdGetVoltage $board VID_OFFSET_CHAN0];
    set gvar(fcdVidBrdStatus_vidChan0Offset) $result;
}

#
# Put the system into operate mode: timeout takes into account cases like 
# omegacam
proc fcdOnline {} {
    global env gvar

    fcdSendCmd ONLINE "" 120000
}

#
# Send a command to the TM process which will close the output relays
proc fcdUserOnline { } {
    global env gvar

    # Busy now
    fcdBusyCursor

    fcdOnline

    # Not Busy now
    fcdNormalCursor
}


#
# Send a command to the TM process which will open the output relays
proc fcdUserStandby { } {
    global env gvar

    # Busy now
    fcdBusyCursor

    fcdSendCmd STANDBY "" 60000

    # Not Busy now
    fcdNormalCursor
}


proc fcdUserSetClkVoltage { } {
    global env gvar

    set board $gvar(fcdClkVoltControl_boardName);
    set periph $gvar(fcdClkVoltControl_periphName);
    set value $gvar(fcdClkVoltControl_periphValue);

    fcdSetVoltage $board $periph $value;
}

proc fcdSetVoltage { board periph value} {                              
    global env gvar

    # puts "SetVoltage $board $periph $value"

    fcdSendCmd SET "-voltage $board $periph $value"
}

proc fcdUserReadAddress { } {
    global env gvar

    set address $gvar(fcdReadWriteAddr_readAddress);

    set result [fcdReadAddress $address];

    set gvar(fcdReadWriteAddr_readValue) $result;
}

proc fcdReadAddress { address } {                              
    global env gvar


    set result [panSendCommand $env(RTAPENV) $gvar(fcdcon) PEEK NO_CHECK 10000 $address]

    return $result
}

proc fcdUserWriteAddress { } {
    global env gvar

    set address $gvar(fcdReadWriteAddr_writeAddress);
    set value $gvar(fcdReadWriteAddr_writeValue);

    set result [fcdWriteAddress $address $value];

    set gvar(fcdReadWriteAddr_writeValue) $result;
}

proc fcdWriteAddress {address value} {                              
    global env gvar


    set result [panSendCommand $env(RTAPENV) $gvar(fcdcon) POKE NO_CHECK 10000 $address $value]

    return $result
}

proc fcdUserSetDebugLevel { } {
    global env gvar

    set value $gvar(fcdDebugLevel_debugLevel);

    fcdSetDebugLevel $value
}

proc fcdSetDebugLevel {level} {                              
    global env gvar


    set result [fcdSendCmd SET "-debugLevel $level"]

    return $result
}

proc fcdUserSetTestVideoMode { } {
    global env gvar

    set mode $gvar(fcdTestVideoMode_testVideoMode);

    set result [fcdSetTestVideoMode $mode];
}


proc fcdSetTestVideoMode {mode} {                              
    global env gvar


    set result [fcdSendCmd SET "-testVideo $mode"]

    return $result
}

proc fcdUserExecTest { } {
    global env gvar

    set testNum $gvar(fcdExecTest_testNum);

    set result [fcdExecTest $testNum];

    set gvar(fcdExecTest_testResult) $result;
}


proc fcdExecTest {testNum} {                              
    global env gvar


    set result [panSendCommand $env(RTAPENV) $gvar(fcdcon) EXECT NO_CHECK 10000 $testNum]

    return $result
}

proc fcdUserCheckTelemetry { } {
    global env gvar

    set gvar(fcdCheckTelemetry_checkTelemetryResult) "wait";

    set result [fcdCheckTelemetry];

    set gvar(fcdCheckTelemetry_checkTelemetryResult) $result;
}


proc fcdCheckTelemetry {} {                              
    global env gvar


    set result [panSendCommand $env(RTAPENV) $gvar(fcdcon) CHECKTM NO_CHECK 10000]

    return $result
}

proc fcdUserReloadSeq { } {
    global env gvar

    set gvar(fcdReloadSeq_reloadSeqResult) "wait";

    set dir $gvar(fcdReloadSeq_reloadSeqDir);

    set result [fcdReloadSeq $dir];

    set gvar(fcdReloadSeq_reloadSeqResult) $result;
}


proc fcdReloadSeq {dir} {
    global env gvar

    set result [fcdSendCmd LOADSQ "" 40000]

    return $result
}

proc fcdBusyCursor { } {      
    global gvar
     
    . config -cursor watch
}


proc fcdNormalCursor { } {         
    global gvar env

    . config -cursor left_ptr
}
#
# Find the description and sequences for the given read out mode
proc fcdReadModeUpdate { mode } {         
    global gvar env

    # Make sure the mode is OK
    if {$mode < 0} {
	set descr ""
    } else {
	set var $gvar(cwp):readout:mode_$mode.description
	set descr  [dbRead $var]
    }
    
    # Don't accept modes which are undefined
    if {[string length $descr] == 0} {
	set gvar(fcdReadMode_readModeDescription) $descr
	set gvar(fcdReadMode_readModeIntSeq) ""
	set gvar(fcdReadMode_readModeWipeSeq) ""
	set gvar(fcdReadMode_readModeReadSeq) ""
	set gvar(fcdReadMode_readModeWipeAlways) 0
	return 
    }
    set gvar(fcdReadMode_readModeDescription) $descr
    set v [dbRead $gvar(cwp):readout:mode_$mode.readSeqs(0,readSeq)]
    set gvar(fcdReadMode_readModeReadSeq) $v
    set v [dbRead $gvar(cwp):readout:mode_$mode.wipeSeq]
    set gvar(fcdReadMode_readModeWipeSeq) $v
    set v [dbRead $gvar(cwp):readout:mode_$mode.intSeq]
    set gvar(fcdReadMode_readModeIntSeq) $v
    set v [dbRead $gvar(cwp):readout:mode_$mode.wipeAlways]
    set gvar(fcdReadMode_readModeWipeAlways) $v
}
#
# Enable or disable window mode
proc fcdWinEnabled { } {         
    global gvar env cv

    # Switchin on or off window mode ?
    if {$gvar(fcdExpSetup_win1_winEnabled) == 0} {
	set enable disable
    } else {
	set enable enable
    }
    $cv.fcdExpSetup.cv.win1_xFirst $enable
    $cv.fcdExpSetup.cv.win1_yFirst $enable
    $cv.fcdExpSetup.cv.win1_xDim $enable
    $cv.fcdExpSetup.cv.win1_yDim $enable

    if {$gvar(fcdExpSetup_win2_winEnabled) == 0} {
	set enable disable
    } else {
	set enable enable
    }
    $cv.fcdExpSetup.cv.win2_xFirst $enable
    $cv.fcdExpSetup.cv.win2_yFirst $enable
    $cv.fcdExpSetup.cv.win2_xDim $enable
    $cv.fcdExpSetup.cv.win2_yDim $enable
    return 
}
#
# fcdInitValues
#
# useModes
#    Indicates if the panel specifies a mode index or uses individual
#    sequence names. If it uses a mode index then the sequence names
#    should be read out of the database
#
proc fcdInitValues { useModes } {         

    global gvar env  pollRate cv

    # Record the use modes flag
    set gvar(useModes) $useModes

    # Set process timeout
    set gvar(procTimeout) 240000

    # Set the value passed as the check parameter to panSendCommand
    set gvar(checkCommands) "CHECK_ALL"

    # store the CCDNAME in the gvar vector
    set gvar(CCDNAME) $env(CCDNAME)

    # db root
    set gvar(cwp) "<alias>$env(CCDNAME)"

    # con process
    set gvar(fcdcon) "fcdcon_$env(CCDNAME)"
    
    # Exposure setup
    #
    
    # Disable the window and update the panel
    set gvar(fcdExpSetup_winEnabled) 0;

    set gvar(fcdExpSetup_xFirst) 0;
    set gvar(fcdExpSetup_yFirst) 0;
    set gvar(fcdExpSetup_xDim) 100;
    set gvar(fcdExpSetup_yDim) 100;
    set gvar(fcdExpSetup_xBinning) 1;
    set gvar(fcdExpSetup_yBinning) 1;
    set gvar(fcdExpSetup_xBinning) 1;
    set gvar(fcdExpSetup_yBinning) 1;

    set gvar(fcdExpSetup_intTime) 1.0;
    set gvar(fcdExpSetup_expType) Normal;

    set gvar(fcdExpSetup_repType) 1;
    set gvar(fcdExpSetup_repCount) 10;
    set gvar(fcdExpSetup_startTimeAbs) F;
    set gvar(fcdExpSetup_startTimeStr) "yyyy-mm-ddThh:mm:ss.uuuu";
    set gvar(fcdExpSetup_loopTime) 0;

    # PP setup
    #
    set gvar(fcdExpSetup_win1_sigmaCalc) None;
    set gvar(fcdExpSetup_win2_sigmaCalc) None;
    set gvar(fcdExpSetup_win1_procLLX) 0;
    set gvar(fcdExpSetup_win1_procLLY) 0;
    set gvar(fcdExpSetup_win1_procURX) 0;
    set gvar(fcdExpSetup_win1_procURY) 0;
    set gvar(fcdExpSetup_win2_procLLX) 0;
    set gvar(fcdExpSetup_win2_procLLY) 0;
    set gvar(fcdExpSetup_win2_procURX) 0;
    set gvar(fcdExpSetup_win2_procURY) 0;

    set gvar(fcdExpSetup_saveFileName) "default";
    set gvar(fcdExpSetup_rtdDisplay) 0;
    set gvar(fcdExpSetup_saveType) 0;
    set gvar(fcdExpSetup_transfer) 1;

    # Wipe control variables
    #
    set gvar(fcdWipeSetup_wipePeriod) 1.0;
    set gvar(fcdWipeSetup_wipeNum) 20;
    set gvar(fcdWipeSetup_wipeType) 1;

    # Voltage control values
    #
    set gvar(fcdAnaVoltControl_periphValue) 0;
    set gvar(fcdAnaVoltControl_boardName) BRD_ANABIAS1;
    set gvar(fcdAnaVoltControl_periphName) ANB_PRESET_VOLT_A;

    set gvar(fcdVidVoltControl_periphValue) 0;
    set gvar(fcdVidVoltControl_boardName) BRD_VIDBRD1;
    set gvar(fcdVidVoltControl_periphName) VID_GAIN1_CHAN0;

    set gvar(fcdNVidVltControl_channel) 0;
    set gvar(fcdNVidVltControl_boardName) BRD_VIDBRD1;
    set gvar(fcdNVidVltControl_gain1) 0;
    set gvar(fcdNVidVltControl_gain2) 0;
    set gvar(fcdNVidVltControl_offset) 0;
    set gvar(fcdNVidVltControl_filter) 0;

    # Variable which holds the id of the rtd process
    # when started by us
    set gvar(rtdPID) 0;

    set gvar(fcdReadMode_readMode) "0";
    set gvar(fcdReadMode_readModeDescription) "";
    set gvar(fcdReadMode_readModeReadSeq) "";
    set gvar(fcdReadMode_readModeIntSeq) "";
    set gvar(fcdReadMode_readModeWipeSeq) "";
    set gvar(fcdReadMode_readModeWipeAlways) 0;
    
    set gvar(fcdEngSetup_wipeSeq) "";
    set gvar(fcdEngSetup_intSeq) "";
    set gvar(fcdEngSetup_readSeq) "";

    # Test Video
    set gvar(fcdTestVideoMode_testVideoMode) 0;
    
    #
    # Has the system been initialised by an INIT command
    set gvar(fcdInitSent) 0;

    # Rate at which db values are updated on display
    # does not seem to work
    #set pollRate 1

    # puts [array names gvar];

}
#
#
proc fcdInitPanel { } {         
    global gvar env  cv

    fcdWinEnabled;

    fcdReadModeUpdate $gvar(fcdReadMode_readMode);
}
#
# fcdResetPanelValues
# 
# Used when some of the values in the panel have been screwed, this 
# shows up as oslx Failed to handle setup buffer.
#
# We just reinitialisa all the setup values to their original states
#
proc fcdResetPanelValues {} {

    global gvar env  pollRate cv

    # Reinitialise
    fcdInitValues $gvar(useModes);
}
#
# Call the engineering panel
#
proc fcdUserCallEngPanel {} {

    global gvar env  pollRate cv

    fcdBusyCursor; 

    puts "import panel"
    panImportPanel fcdSlcuEngCon "cwp=<alias>$env(CCDNAME)"; 

    # db root

    fcdNormalCursor
}
