#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: FileViewer.tcl,v 1.20 2004/01/16 15:28:34 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  10/07/97  adapted from Micheal McLennan Example
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
# ----------------------------------------------------------------------
#  EXAMPLE: Fileviewer mega-widget
# ----------------------------------------------------------------------
#     BOOK: Tcl/Tk Extensions for O'Reilly & Associates
#   AUTHOR: Michael J. McLennan, AT&T Bell Laboratories
# ======================================================================
#               Copyright (c) 1996  AT&T Bell Laboratories
# ======================================================================

option add *Fileviewer.width 2i widgetDefault
option add *Fileviewer.height 3i widgetDefault
option add *Fileviewer.scrollbar auto widgetDefault
option add *Fileviewer.cursor center_ptr widgetDefault

class Fileviewer {
    inherit itk::Widget

    constructor {args} {
        itk_option add hull.width hull.height
        itk_option remove hull.cursor

        itk_component add scrollbar {
            scrollbar $itk_interior.sbar -orient vertical \
                -command [code $itk_interior.canv yview]
        } {
            usual
            ignore -cursor
        }
        pack $itk_component(scrollbar) -side right -fill y

        itk_component add display {
            canvas $itk_interior.canv -borderwidth 2 \
                -relief sunken -background white \
                -yscrollcommand [code $this fixScrollbar] \
                -width 1 -height 1
        } {
            keep -cursor
            keep -highlightcolor -highlightthickness
            rename -highlightbackground -background background Background
        }
        pack $itk_component(display) -side left -expand yes -fill both

        eval itk_initialize $args
        component hull configure -cursor ""

        pack propagate $itk_component(hull) 0
    }

    itk_option define -selectcommand selectCommand Command ""

    itk_option define -scrollbar scrollbar Scrollbar "on" {
        switch -- $itk_option(-scrollbar) {
            on - off - auto {
                fixScrollbar
            }
            default {
                error "bad value \"$itk_option(-scollbar)\""
            }
        }
    }

    private variable root ""

    public method display {dir}
    public method select {node}

    private method createNode {dir}
    private proc cmpTree {option obj1 obj2}
    private method fixScrollbar {args}
    private variable sbvisible 1
}

usual Fileviewer {
    keep -activebackground -activerelief
    keep -background -cursor
    keep -highlightcolor -highlightthickness
    keep -troughcolor
}

proc fileviewer {pathName args} {
    uplevel Fileviewer $pathName $args
}

# ----------------------------------------------------------------------

body Fileviewer::display {dir} {
    if {$root != ""} {
        delete object $root
    }
    set root [createNode $dir]
    $root configure -state open
    $root refresh
}

body Fileviewer::select {node} {
    set name [$node cget -name]
    regsub -all {%n} $itk_option(-selectcommand) $name cmd
    uplevel #0 $cmd
}

body Fileviewer::createNode {fname} {
    set obj [VisualFileTree ::#auto $fname $itk_component(display)]

    $obj configure -name $fname \
        -sort [code cmpTree -name] \
        -procreate [code $this createNode] \
        -selectcommand [code $this select %o]

    if {[file isdirectory $fname]} {
        $obj configure -icon dirIcon
    } elseif {[file executable $fname]} {
        $obj configure -icon programIcon
    } else {
        $obj configure -icon fileIcon
    }
    $obj configure -title [file tail $fname]

    return $obj
}

body Fileviewer::cmpTree {option obj1 obj2} {
    set val1 [$obj1 cget $option]
    set val2 [$obj2 cget $option]
    if {$val1 < $val2} {
        return -1
    } elseif {$val1 > $val2} {
        return 1
    }
    return 0
}

body Fileviewer::fixScrollbar {args} {
    switch $itk_option(-scrollbar) {
        on  { set sbstate 1 }
        off { set sbstate 0 }

        auto {
            if {[$itk_component(display) yview] == "0 1"} {
                set sbstate 0
            } else {
                set sbstate 1
            }
        }
    }
    if {$sbstate != $sbvisible} {
        if {$sbstate} {
            pack $itk_component(scrollbar) -side right -fill y
        } else {
            pack forget $itk_component(scrollbar)
        }
        set sbvisible $sbstate
    }

    if {$args != ""} {
        eval $itk_component(scrollbar) set $args
    }
}
