#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: VisualFileTree.tcl,v 1.20 2004/01/16 15:28:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  10/07/97  adapted from Micheal McLennan Example
# ccumani  01/03/98  bitmap files search corrected
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
# ----------------------------------------------------------------------
#  EXAMPLE: VisualFileTree class
# ----------------------------------------------------------------------
#     BOOK: Tcl/Tk Extensions for O'Reilly & Associates
#   AUTHOR: Michael J. McLennan, AT&T Bell Laboratories
# ======================================================================
#               Copyright (c) 1996  AT&T Bell Laboratories
# ======================================================================


class VisualFileTree {
    inherit FileTree VisualRep

    public variable state "closed"
    public variable selectcommand ""

    constructor {file cwin args} {
        FileTree::constructor $file
        VisualRep::constructor $cwin
    } {
        eval configure $args
    }

    public method select {}
    public method toggle {}

    public method draw {ulVar midVar}
    public method refresh {}
}

body VisualFileTree::select {} {
    $canvas itemconfigure hilite -fill ""
    $canvas itemconfigure $this-hilite -fill LightGray
    regsub -all {%o} $selectcommand $this cmd
    uplevel #0 $cmd
}

body VisualFileTree::toggle {} {
    if {$state == "open"} {
        set state "closed"
    } else {
        set state "open"
    }
    refresh
}

body VisualFileTree::draw {ulVar midVar} {
    upvar $ulVar ul
    upvar $midVar mid

    VisualRep::draw ul mid
    $canvas bind $this <ButtonPress-1> "$this select"
    $canvas bind $this <Double-ButtonPress-1> "$this toggle"

    set lr(x) [expr $ul(x) + 2*($mid(x)-$ul(x))]
    set lr(y) $ul(y)

    if {$state == "open"} {
        foreach obj [contents] {
            $obj draw lr mid2
            set id [$canvas create line \
                $mid(x) $mid(y) $mid(x) $mid2(y) $mid2(x) $mid2(y) \
                -fill black]
            $canvas lower $id
        }
    }
    set ul(y) $lr(y)
}

body VisualFileTree::refresh {} {
    set root $this
    while {[$root back] != ""} {
        set root [$root back]
    }

    set oldcursor [$canvas cget -cursor]
    $canvas configure -cursor watch
    update

    $canvas delete all

    set ul(x) 5
    set ul(y) 5
    $root draw ul mid
    set bbox [$canvas bbox all]
    $canvas configure -cursor $oldcursor -scrollregion $bbox
}

configbody VisualFileTree::state {
    if {$state != "open" && $state != "closed"} {
        error "bad value \"$state\": should be open or closed"
    }
    refresh
}

image create photo dirIcon -file [fcdpmsFindVLT bitmaps/dir2.gif]
image create photo fileIcon -file [fcdpmsFindVLT bitmaps/file.gif]
image create photo programIcon -file [fcdpmsFindVLT bitmaps/program.gif]
