#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: VisualRep.tcl,v 1.20 2004/01/16 15:28:34 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  10/07/97  adapted from Michael McLennan Example
# ccumani  01/03/98  bitmap files search corrected
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# ----------------------------------------------------------------------
#  EXAMPLE: VisualRep base class
# ----------------------------------------------------------------------
#     BOOK: Tcl/Tk Extensions for O'Reilly & Associates
#   AUTHOR: Michael J. McLennan, AT&T Bell Laboratories
# ======================================================================
#               Copyright (c) 1996  AT&T Bell Laboratories
# ======================================================================


image create photo defaultIcon -file [fcdpmsFindVLT bitmaps/default.gif]

class VisualRep {
    public variable icon "defaultIcon"
    public variable title ""

    protected variable canvas ""

    constructor {cwin args} {
        set canvas $cwin
        if {![info exists selectedObjs($canvas)]} {
            set selectedObjs($canvas) ""
        }
        eval configure $args
    }
    destructor {
        deselect
        $canvas delete $this
    }

    public method draw {ulVar midVar}
    public method select {}
    public method deselect {}

    public method canvas {args}

    private common selectedObjs
    public proc clear {canv}
    public proc selected {canv}
}

body VisualRep::draw {ulVar midVar} {
    upvar $ulVar ul
    upvar $midVar mid

    if {[catch {image type $icon}] == 0} {
        set imh $icon
    } else {
        set imh defaultIcon
    }

    $canvas create image $ul(x) $ul(y) -image $imh \
        -anchor nw -tags [list $this $this-icon]

    set mid(x) [expr $ul(x)+[image width $imh]/2]
    set mid(y) [expr $ul(y)+[image height $imh]/2]
    set x1 [expr $ul(x)+[image width $imh]+2]
    set y1 [expr $ul(y)+[image height $imh]+2]

    if {[lsearch $selectedObjs($canvas) $this] >= 0} {
        set fill LightGray
    } else {
        set fill ""
    }
    $canvas create rectangle $x1 $ul(y) 1500 $y1 \
        -outline "" -fill $fill -tags [list $this hilite $this-hilite]

    $canvas create text $x1 $mid(y) -text $title \
        -anchor w -tags [list $this $this-text]

    set ul(y) $y1
}

body VisualRep::select {} {
    $canvas itemconfigure $this-hilite -fill LightGray

    if {[lsearch $selectedObjs($canvas) $this] < 0} {
        lappend selectedObjs($canvas) $this
    }
}

body VisualRep::deselect {} {
    $canvas itemconfigure $this-hilite -fill ""

    set i [lsearch $selectedObjs($canvas) $this]
    if {$i >= 0} {
        set selectedObjs($canvas) [lreplace $selectedObjs($canvas) $i $i]
    }
}

body VisualRep::canvas {args} {
    if {[llength $args] == 0} {
        return $canvas
    }
    regsub -all {%o} $args $this args
    eval $canvas $args
}

body VisualRep::clear {canv} {
    if {[info exists selectedObjs($canv)]} {
        foreach obj $selectedObjs($canv) {
            $obj deselect
        }
    }
}

body VisualRep::selected {canv} {
    if {[info exists selectedObjs($canv)]} {
        return $selectedObjs($canv)
    }
    return ""
}
