#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdpmsSensor.tcl,v 1.20 2004/01/16 15:28:27 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  15/09/97  created
# rdonalds 17/09/09  Modified to log date and time with values to file
#                    now logs every sensor value
#                    Allows multiple sensors to be logged to one file

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#



itk::usual fcdpmsSensor {
    keep -short_help
}

class fcdpmsSensor {
    inherit itk::Widget fcdpmsPHW

    itk_option define       -name  name Name ""   
    itk_option define       -short_help short_help Short_help  ""

    constructor             {unit_ sensorid_ title_ args} {}
    destructor                     {}

    private  variable      logfile        ""
    private  variable      data           

    private  variable      gcount         "0"
    private  variable      count          "0"
    private  variable      sensorID       ""
    private  variable      logtitle          ""
    #methods for logging the sensor value    
    public   method logSensor         {}
    public   method logConfigure      {}

};#fcdpmsSensor

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# fcdpmsSensor::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body fcdpmsSensor::constructor {unit_ sensorid_ title_ args} {
    set unit $unit_
    set sensorID $sensorid_
    set logtitle $title_
    
    set data(0) -1

    if {[llength $args] != 0 } {
	eval itk_initialize $args
    }
    itk_component add sensor {
	fcdpmsBulletLog $itk_interior.sensor \
		-logproc [code $this logSensor] \
		-logconfigure [code $this logConfigure] \
		-name "Sensor $sensorID" \
		-orient horiz \
		-valwidth 10 \
		-valheight 2 \
		-short_help $itk_option(-short_help)
    } {
	usual
	keep -name
    }

   
    
    pack $itk_component(sensor) -padx 2 -side left -fill x
    $itk_component(sensor) log
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# fcdpmsSensor::destructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body fcdpmsSensor::destructor {} {
    
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# fcdpmsSensor::logConfigure
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body fcdpmsSensor::logConfigure {} {
    global env
    
    if {[set par [sensor_log_dialog \
	    [expr [$itk_component(sensor) getInterv]/1000] \
	    $logfile \
	    $logtitle\
	    "Logging Properties for $itk_option(-name)"]] == ""} {
	return
    }
    
    if [$itk_component(sensor) logActive] {
	if [confirm_dialog "Stop Logging of Sensor $itk_option(-name)"] {
	        $itk_component(sensor) stopLog

	} else {
	    info_dialog "Cannot configure a running log-process"
	    return
	}
    } 
    
    set logfile [lindex $par 0]
    $itk_component(sensor) setInterv [expr [lindex $par 1]*1000]
    set logtitle [lindex $par 2]


    if ![util::fexists $logfile] {
	if [confirm_dialog "Create New Logfile [util::fname $logfile]"] {
	    util::fcreate $logfile
	} else {
	    warning_dialog "Logfile [util::fname $logfile] NOT created"\
		    "Sensor data is not logged to file"
	    set logfile ""
	    return
	}
    } else {
	if ![confirm_dialog "Append to File [util::fname $logfile]"] {
	    info_dialog "Go back to Properties Popup Dialog\n\
		    and select other filename"
	    set logfile ""
	    return
	}
    }
    set gcount 0
    if {[set rc [util::fopenr $logfile fd]] != "OK"} {
	error_dialog $rc 
	set logfile ""
	return
    }    
    fclose $fd
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# fcdpmsSensor::logSensor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body fcdpmsSensor::logSensor {} {
    set data(0) [getSensor $sensorID]

    #dump values to file
    if {$logfile != ""} {
	if {[set rc [util::fopena $logfile fd]] != "OK"} {
	    error_dialog "Logproc $rc"
	} else {
	    set date_string [exec date "+%d/%m/%y %H:%M:%S"]
	    puts $fd "$date_string \"$logtitle\" $data(0)"
	    util::fclose $fd
	}
    }
    return $data(0)
}
#
# ___oOo___
