#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdpmsUIFUtil.tcl,v 1.20 2004/01/16 15:28:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  15/04/97  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
#
#
#     VALIDATION FUNCTIONS
#
#

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forcePosInt
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forcePosInt {P l h} {
    set rc 0
    if {[regexp {[0-9]+$} $P]} {
	if {($P <= $h) && ($P >= $l)} { 
	    set rc 1
	}
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceInt
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceInt {P l h} {
    set rc 0
    if {[regexp {^[-+]?[0-9]*$} $P]} {
	if {($P <= $h) && ($P >= $l)} { 
	    set rc 1
	}
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceReal
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceReal {P l h} {
    set rc 0
    if {[regexp {^[-+]?[0-9]*\.?[0-9]*([0-9]\.?e[-+]?[0-9]*)?$} $P]} {
	if {($P <= $h) && ($P >= $l)} { 
	    set rc 1
	}
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceHex
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceHex {P l h} {
    set rc 1
    if {[regexp {^(0[xX])?[0-9a-fA-F]*$} $P]} {
	set pi [format "%i" $P]
	if {($pi >= $l) && ($pi <= $h)} {
	    set rc 1
	}
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceRegExp
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceRegexp {P regexp} {
    set rc 0
    if {[regexp $regexp $P]} {
	set rc 1
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceAlpha
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceAlpha {P} {
    set rc 0
    if {[regexp {^[a-zA-Z]*$} $P]} {
	set rc 1
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# forceLen
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc forceLen {P len} {
    set rc 0
    if {[string length $P] <= $len} {
	set rc 1
    }
    return $rc
}

#
#
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# useful UIF functions to LOCK/UNLOCK the UIF
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#######################################################################
# Author:       C. Zerbst
# Description:  this procedure is used to lock the interface while 
#               some processes are working
#      17/02/1997 adapted by hpieper
#
proc BusyXXX {message} {
    global ::WES
    set message [string trim $message]
    
    set b .busy     
    if {[set ww [expr 3 + [string length $message]]] < 25} {
	set ww 25
    }
    if {![winfo exists $b] && ![string match end $message]} {
	. configure -cursor watch
	set WES(status) $message
	toplevel $b
	wm override $b 1
	wm withdraw $b
	wm positionfrom $b program
	set x [expr int([winfo x .] + 0.5 * [winfo width .])-100]
	set y [expr int([winfo y .] + 0.5 * [winfo height .])-50]       
	wm transient $b .
	pack [label $b.l -highlightt 0 \
		-text "WES\n$message" \
		-relief raised -bd 1 -background Grey90 \
		-width $ww \
		-height 3] -expand true
	wm geometry $b +$x+$y
	wm deiconify $b
	raise $b
	grab $b
	update idletask
    } elseif {[string match end $message] && [winfo exists $b]} {
	. configure -cursor top_left_arrow
	grab release $b
	destroy $b  
	set WES(status) ""
    } elseif [winfo exists $b] {
        set WES(status) $message
	$b.l configure -width $ww -text "WES\n$message"
	update idletask         
    }
}
proc Busy {message} {
    global ::WES
    set message [string trim $message]
   
    if {![string match end $message]} {
	freezeEvents
	. configure -cursor watch
	set WES(status) $message
	update idletask
    } else {
	unfreezeEvents
	. configure -cursor top_left_arrow
	set WES(status) ""
    }
}

proc applyBindtag {w} {
    set oldtags [bindtags $w]
    if {[lsearch $oldtags "FreezeEvents"] == -1} {
	set bindtags [concat FreezeEvents $oldtags]
	bindtags $w $bindtags
    }
    foreach child [winfo children $w] {
	applyBindtag $child
    }
}

proc freezeEvents {} {
    foreach event [list KeyPress KeyRelease \
	    ButtonPress ButtonRelease] {
	bind FreezeEvents <Any-$event> {break}
    }
    . configure -cursor watch
}

proc unfreezeEvents {} {
    foreach event [list KeyPress KeyRelease \
	    ButtonPress ButtonRelease] {
	bind FreezeEvents <Any-$event> {}
    }
    . configure -cursor top_left_arrow

}
#-----------------------------------------------------------------------
# the following routines have been taken from bobFindLib, Module BOB
# which was implemented by eallaert 30/11/1996
#
proc fcdpmsFindVLT {relPath} {
    foreach area [namespace eval ::fcdpms set searchList2] {
	set absPath ${area}/$relPath
	if [file readable $absPath] {
	    #puts "found it: $absPath"
	    return $absPath		;# found in given area
	}
    }
    return {}	
}

proc fcdpmsRel2Abs {path} {
    if {[file isfile $path]} {
	set dir [file dirname $path]
	set file /[file tail $path]
    } elseif {[file isdirectory $path]} {
	set dir $path
	set file {}
    } else {
	wesError "file $path does not exist"
    }
    set cwd [pwd]
    cd $dir
    set path [pwd]$file
    cd $cwd
    unset cwd
    return $path
}

namespace eval fcdpms {
    global ::env searchList1 searchList2
    
    set searchList1 "[pwd] [fcdpmsRel2Abs [pwd]/..]"
    set searchList2 $searchList1
   
    if {[info exists env(INS_ROOT)]} {
	if {[info exists env(INS_USER)] && ![cequal $env(INS_USER) SYSTEM]} {
	    lappend searchList1 $env(INS_ROOT)/$env(INS_USER)
	}
	lappend searchList1 $env(INS_ROOT)/SYSTEM
    }

    if {[info exists env(INTROOT)]} {
	lappend searchList2 $env(INTROOT)
    }
    if {[info exists env(VLTROOT)]} {
	lappend searchList2 $env(VLTROOT)
    }
}    


#
# ___oOo___
