# dialog.tcl - general purpose dialogs
#
# Copyright (C) 1994 Allan Brighton (abrighto@eso.org)
# "@(#) $Id: udialog.tcl,v 1.20 2004/01/16 15:28:31 vltsccm Exp $"


# get a file name from the user and return it
# or the empty string

proc filename_dialog {{dir "."} {filter "*"} {parent ""}} {
    set w .fs
    if {![winfo exists $w]} {
	FileSelect $w -dir $dir -filter $filter -transient 1 -withdraw 1
 	#Fileselectiondialog $w -directory $dir -modality application
   } else {
	#$w config -dir $dir -filter $filter
	#$w config -directory $dir
    }
    if {"$parent" != ""} {
	wm transient $w [winfo toplevel $parent]
    }
    if {[$w activate]} {
        return [$w get]
    }
}


# error message routine with exit button

proc errexit_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.errexit_dialog
    catch {destroy $w}
    set d [util::DialogWidget $w \
	       -title Error \
	       -text "Error: $msg" \
	       -bitmap error \
	       -transient 1 \
	       -buttons {Continue Exit}]
    if {[$d activate] == 1} {
	exit 1
    }
}


# error  message routine

proc error_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.error_dialog
    catch {destroy $w} 
    [util::DialogWidget $w \
	 -title Error \
	 -text "Error: $msg" \
	 -transient 1 \
	 -bitmap error] activate
}


# warning message routine

proc warning_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.warning_dialog
    catch {destroy $w}
    [util::DialogWidget $w \
	 -title Warning \
	 -text "Warning: $msg" \
	 -transient 1 \
	 -bitmap warning] activate
}


# info message routine

proc info_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.info_dialog
    catch {destroy $w}
    [util::DialogWidget $w \
	 -title Information \
	 -text "$msg" \
	 -transient 1 \
	 -bitmap info] activate
}


# Ask the user to do something of press cancel.
# If the answer is OK, return 1, otherwise 0

proc action_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.action_dialog
    catch {destroy $w}
    set d [util::DialogWidget $w \
	       -title Action \
	       -text $msg \
	       -bitmap info \
	       -transient 1 \
	       -default 0 \
	       -buttons {OK Cancel}]
    return [expr {[$d activate] == 0}]
}


# Question the user and get the answer
# If the answer is yes, return 1, otherwise 0

proc confirm_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.confirm_dialog
    catch {destroy $w}
    set d [util::DialogWidget $w \
	       -title Confirm \
	       -text $msg \
	       -bitmap questhead \
	       -transient 1 \
	       -default 1 \
	       -buttons {Yes Cancel}]
    return [expr {[$d activate] == 0}]
}


# Get some text input from the user and return it
# or an empty string (in case the user cancels the operation)

proc input_dialog {msg {parent ""}} {
    if {"$parent" != ""} {
	if {"[set parent [winfo toplevel $parent]]" == "."} {
	    set parent ""
	}
    }
    set w $parent.input_dialog
    catch {destroy $w}
    set d [InputDialog $w \
	       -title Input \
	       -text $msg \
	       -bitmap questhead \
	       -transient 1 \
	       -messagewidth 4i \
	       -default 0 \
	       -buttons {OK Cancel}]
    return [$d activate]
}

