#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: hisConfigDataProcs.tcl,v 1.33 2004/09/23 09:25:07 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  07/11/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc hisWriteDataConfig {} {
    global gvar
    global cv gvar numHistData
    global hdPlin  histBaseIdx

    if {[hisUserConfirm ] == 0} { return }

    # Get attribute info to check if the data type matches
    # the one defined in the history table

    set idx 0

    while {$idx < $numHistData} {
	lappend val $hdPlin($idx) ; incr idx
    }

    set max       [expr $numHistData - 1 + $histBaseIdx]
    set attrName "$gvar(hisPoint).history config(0,$histBaseIdx:$max)"

    set ret [panDbWrite $attrName $val]

    if {[cequal $ret FAILURE]} {
	set msg "Cannot update $attrName"
	tk_dialog .dialog "Configuration Error" $msg info 0 OK 
	return FAILURE
    } 

    # Re-read the table to update the GUI
    hisGetAllData
    return
}


proc hisCheckDataType { attrName } {
    global gvar

    # Get attribute info to check if the data type matches
    # the one defined in the history table

    if {[catch {seq_dbGetAttrInfo $attrName} info ]} {
	puts "Cannot get 'info' for \"attrName\""
	return FAILURE
    }

    set dbType [hisGetDataType [lindex $info 1]]

    if {![cequal $dbType $gvar(dbType)]} {
	set msg "Attribute has wrong data type $dbType, expected $gvar(dbType)"
	tk_dialog .dialog "Configuration Error" $msg info 0 OK 
	return FAILURE
    }
}


proc histConfigDataEntry {} {
    global cv gvar histBaseIdx
    global hdPlin hdAlias hdAbsolute hdDataType

    # Delete current polling

    set idx [expr $gvar(fieldId) - $histBaseIdx]

    if {[cequal $gvar(dataLink) ""]} {
	set hdPlin($idx)     "@\[\]"
	set hdAlias($idx)    ""
	set hdAbsolute($idx) ""
	set hdDataType($idx) $gvar(dbType)

    } else {

	set ret [hisGetPLIN] ;                    if {[cequal $ret FAILURE]} { return }
	set tmp [hisGetPtNames $gvar(dataPLIN)] ; if {[cequal $tmp ""]} { return }

	lassign $tmp nameAbsAddr nameAlias

	set ret [hisCheckDataType $nameAlias];    if {[cequal $ret FAILURE]} {return }

	set hdPlin($idx)     $gvar(dataPLIN)
	set hdAlias($idx)    $nameAlias
	set hdAbsolute($idx) $nameAbsAddr
	set hdDataType($idx) $gvar(dbType)
    }

    hisDisplayData $cv.wdgHistDataList  
}


proc histSelectData {} {
    global cv gvar histBaseIdx

    # Delete current polling

    set idx [$cv.wdgHistDataList getIndex]

    set gvar(fieldId) [expr $idx + $histBaseIdx]

    set tmp [$cv.wdgHistDataList getItem $idx]
    lassign $tmp gvar(dbType) gvar(dataPLIN) gvar(dataLink) 

    $cv.wdgHistDataList setSelection $idx
}


proc hisGetPLIN {} {
    global gvar

    set gvar(dataPLIN) ""

    if {[cequal $gvar(dataLink) ""]} { return }
	
    if {[catch {seq_dbGetDirAddr "$gvar(dataLink)"} tmp]} {
	puts "Cannot get 'direct address' for \"$gvar(dataLink)\""
	return FAILURE
    }

    set gvar(dataPLIN) $tmp
    return [hisCheckDataType $gvar(dataLink)]
}


proc hisGetDataType { dt } {
    switch $dt {
	2 { return INT8 }
	4 { return INT16 }
	6 { return INT32 }
	8 { return FLOAT }
	9 { return DOUBLE }
	3 { return UINT8  }
	5 { return UINT16 }
	7 { return UINT32 }
	default { return UNKNOWN }
    }
}



proc hisGetAllData {} {
    global cv gvar numHistData
    global hdPlin hdAlias hdAbsolute hdDataType histBaseIdx

    set hp           "$gvar(hisPoint)"
    set numHistData  0

    if {[cequal $hp ""]} { return }

    # Get attr info on "history config" and "history table"

    if {[catch {seq_dbGetAttrInfo "$hp.history config"} info ]} {
	puts "Cannot get 'info' for \"$hp.history config\""
	return
    }

    lassign $info d1 d2 d3 numFields

    set hisMaxData [expr $numFields - 15]

    if {[catch {seq_dbGetAttrInfo "$hp.history"} info ]} {
	puts "Cannot get 'info' for \"$hp.history\""
	return
    }

    lassign $info d1 dataType d2 d3


    # Get list of data

    if {[cequal [panDbRead "${hp}.history config" tmp] FAILURE]} {
	puts "Cannot read attribute \"${hp}.history config\""
	return
    }

    set dataList [lrange $tmp $histBaseIdx end]

    # Retrieve alias and absolute names for each history data

    if {[info exists hdPlin]}     { unset hdPlin}
    if {[info exists hdAlias]}    { unset hdAlias}
    if {[info exists hdAbsolute]} { unset hdAbsolute}
    if {[info exists hdDataType]} { unset hdDataType}

    set idx 0

    foreach hpDirAddr $dataList {

        set nameAbsAddr ""
        set nameAlias   ""

        if {[cequal $hpDirAddr "@\[0\]"]} { set hpDirAddr "@\[\]"}

	# Loop until the first 'empty' address is found
	if {![cequal $hpDirAddr "@\[\]"] } {

            set tmp [hisGetPtNames $hpDirAddr]
            if {[cequal $tmp ""]} { return }

            lassign $tmp nameAbsAddr nameAlias
	}

	set hdPlin($idx)     $hpDirAddr
	set hdAlias($idx)    $nameAlias
	set hdAbsolute($idx) $nameAbsAddr

        # Calculate exact index to fetch the data type
        set dtIdx  [expr ($idx * 2) + 4 ]

	set hdDataType($idx) [hisGetDataType [lindex $dataType $dtIdx]]

	incr idx
    }

    set numHistData $idx

    hisDisplayData $cv.wdgHistDataList

}

proc hisDisplayData {w} {
    global gvar numHistData
    global hdPlin hdAlias hdAbsolute hdDataType

    $w deleteItem 0 end

    if {[cequal $gvar(hisPoint) ""] || $numHistData == 0 } { return }


    if {[cequal $gvar(hpSymbAddr) alias]} {

	for {set idx 0} {$idx < $numHistData} {incr idx} {
	    $w addItem end [format "%-10s %-16s %s" $hdDataType($idx) $hdPlin($idx) $hdAlias($idx)]
	}

    } else {
	for {set idx 0} {$idx < $numHistData} {incr idx} {
	    $w addItem end [format "%-10s %-16s %s"  $hdDataType($idx) $hdPlin($idx) $hdAbsolute($idx)]
	}
    }

}

#
# ___oOo___
