#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: hisSetupProcs.tcl,v 1.33 2004/09/23 09:25:07 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  07/11/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc hisShowTrigger {} {
    global gvar

    set     triggerStr "              Trigger Basic Conditions \n\n"
    append  triggerStr " index   Description \n"
    append  triggerStr "------------------------------------------------ \n"
    append  triggerStr "  1      (ALL)     At any write on attribute \n"
    append  triggerStr "  2      (ADDR)    At any Write On Trig. Address \n"
    append  triggerStr "  4      ( <  )    Value less than the previous \n" 
    append  triggerStr "  8      ( =  )    Value equal to the previous  \n"
    append  triggerStr "  16     ( >  )    Value greater than the previous \n\n"
    append  triggerStr "  The final trigger state is the sum of the single trigger options.\n"
    append  triggerStr "\nExamples : \n\n"
    append  triggerStr "  24     ( >= )    Value greater than or equal to the previous \n"
    append  triggerStr "  12     ( <= )    If value written is less than or equal to the previous\n"
    append  triggerStr "  20     ( <> )    If value written is not equal \n"

    panTerm "Trigger Basic Conditions" $triggerStr
}


proc checkStartCondition { value } {
    global cv

    if {$value == 1} {
	$cv.wdgStartTime enable
    } else {
	$cv.wdgStartTime disable
    }
}

proc checkStopCondition { value } {
    global cv

    if {$value == 3} {
	$cv.wdgStopAfterSamples enable
    } else {
	$cv.wdgStopAfterSamples disable
    }
}

proc checkDirAddr {dirAddr} {
    switch $dirAddr {
	"@\[\]"  {return ""}
	"@\[0\]" {return ""}
	default  {return $dirAddr}
    }
}


proc hisSendSetup {} {
    global gvar
    
    if {[hisUserConfirm ] == 0} { return }

    if {$gvar(startCondition) != 1} {
	# Set start time to NOW !! .. otherwise hisDHMngr complains 
	set now [expr  [getclock] + 10]
	set gvar(startTime) [fmtclock $now "%Y-%m-%d %T"]
    }

    switch $gvar(startCondition) {
	0       { set startCondition armed }
	1       { set startCondition time  }
	2       { set startCondition event }
	default { set startCondition ""}
    }


    switch $gvar(sampleType) {
	0       { set sampleType days}
	1       { set sampleType hours}
	2       { set sampleType minutes}
	3       { set sampleType seconds}
	default { set sampleType ""}
    }

    switch $gvar(stopCondition) {
	0       { set stopCondition disarmed}
	1       { set stopCondition time}
	2       { set stopCondition event}
	3       { set stopCondition count}
	default { set stopCondition ""}
    }

    switch $gvar(stopTimeType) {
	0       { set stopTimeType days}
	1       { set stopTimeType hours}
	2       { set stopTimeType minutes}
	3       { set stopTimeType seconds}
	default { set stopTimeType ""}
    }

    set tmp    "$gvar(hisPoint),$gvar(hisSetupId),$gvar(setupEnable),$startCondition"
    append tmp ",$gvar(startTime),$gvar(triggerState),$gvar(startTrigAddr),$gvar(sampleInterval)"
    append tmp ",$sampleType,$stopCondition,$gvar(stopTimePeriod),$stopTimeType"
    append tmp ",,$gvar(stopTrigAddr),$gvar(stopAfterSamples),$gvar(autoRearm)"

#    puts "setup = $tmp"
    set ret [panSendCommand "" hisDHMngr CFGSTUP "" 60000 $tmp]

    if {[cequal $ret FAILURE]} {
	set msg "Cannot send current setup to history manager"
	tk_dialog .dialog "Configuration Error" $msg info 0 OK
	return   
    }
}


proc hisReadSetup {} {
    global gvar hisSetup

    if {[cequal $gvar(hisPoint) ""]}  { return}

    panDbRead $hisSetup setupRecs

    set value [lindex $setupRecs $gvar(hisSetupId)]

#    puts "setupRecs = $value"
    # Assign fields to proper entry widget

    lassign $value setupEnable gvar(setupState) gvar(startCondition) \
	    gvar(triggerState) startYY startMM startDD \
	    startHH startMIN startSS startUSEC gvar(startTrigAddr) \
	    gvar(sampleType) dummy1 gvar(sampleInterval) gvar(stopCondition) \
	    gvar(stopTimeType) gvar(stopTimePeriod) gvar(stopTrigAddr) \
	    gvar(stopAfterSamples) autoReArm dummy2

    if {$setupEnable == 0} {
	set gvar(setupEnable) FALSE
    } else {
	set gvar(setupEnable) TRUE
    }

    if {$autoReArm == 0} {
	set gvar(autoReArm) FALSE
    } else {
	set gvar(autoReArm) TRUE
    }
    
    set gvar(stopTrigAddr)  [checkDirAddr $gvar(stopTrigAddr)] 
    set gvar(startTrigAddr) [checkDirAddr $gvar(startTrigAddr)] 

    set gvar(startTime) [format "%.4d-%.2d-%.2d %.2d:%.2d:%.2d" ${startYY} ${startMM} ${startDD} ${startHH} ${startMIN} ${startSS}]
}

proc hisAcceptSetupPt {} {
    global gvar hisSetupEvt hisSetup

    if {[cequal $gvar(hisPoint) ""]} { return}
    if {![cequal $hisSetupEvt ""]}   { seq_evtDetach $hisSetupEvt}

    set hisSetup $gvar(hisPoint).setups

    if {[catch {seq_evtAttach $hisSetup "w" hisReadSetup } ret] } {
	set msg "Cannot attach event on $hisSetup"
	tk_dialog .dialog "Configuration Error" $msg info 0 OK
	return
    }

    hisReadSetup

    checkStartCondition $gvar(startCondition)
    checkStopCondition  $gvar(stopCondition)
    update idletasks
}


#
# ___oOo___
