#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ibacLog.tcl,v 1.33 2005/09/01 15:18:04 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# zins  17/04/00  created
#

#************************************************************************
#   NAME
#   ibacLog - Functions to handle standard and verbose logs
#   DESCRIPTION
#   proc ibacSetLogLevel {level}
#   proc ibacGetLogLevel {}
#     Set/Get the log level. By default, the log level is set to 1.
#
#   proc ibacLog {level message {newline 1} }
#     Log data into the standard VLT log file according the current log level.
#     The current log level can be set using ibacSetLogLevel function or
#     can be retreived using ibacGetLogLevel function. The logging level ranges
#     from 1 to 5, where the lowest number means the highest priority. By
#     default, the level is set to 1.
#
#   proc ibacWarningLog {message {newline 1} }
#     Log information about errors or abnormal events for application. The
#     logging level is fixed to 1.
#
#   proc ibacInfoLog {message {newline 1} }
#     Log information about major events. For example, when operational mode is
#     modified. The logging level is fixed to 2.
#
#   proc ibacTestLog {message {newline 1} }
#     Log relevant information used for the software test activities. The
#     logging level is fixed to 3.
#
#   proc ibacDebugLog {message {newline 1} }
#     Log debugging information. The logging level is fixed to 4.
#
#   proc ibacExtDbLog {message {newline 1} }
#     Log more detailed debugging information. The logging level is fixed
#
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

##### Functions to handle standard and verbose logs
proc ibacSetLogLevel {level} {
    global gvar

    # Set new log level
    set gvar(ibacLogLevel) $level
}

proc ibacGetLogLevel {} {
    global gvar

    # If the log level is not yet defined
    if { ! [info exists gvar(ibacLogLevel)] } {
        # Set to the default value
        set gvar(ibacLogLevel) 1
    }

    # Return the current log level
    return $gvar(ibacLogLevel)
}

proc ibacLog {level message {newline 1} } {
    global gvar

    # If the specified level is less than or egal to the logging level
    if { $level <= [ibacGetLogLevel] } {
        # Check if function is called from 'BOB' context or seqWish
        if { [info commands BobTPL::tplLog] != "" } {
            BobTPL::tplLog $message
        } else {
            if { $newline == 1} {
                puts $message
            } else {
                puts -nonewline $message
            }
        }
    }
}
    
proc ibacWarningLog {message {newline 1} } {

    ibacLog 1 $message $newline
}

proc ibacInfoLog {message {newline 1} } {

    ibacLog 2 $message $newline

    
}

proc ibacTestLog {message {newline 1} } {
    
    ibacLog 3 $message $newline

}

proc ibacDebugLog {message {newline 1} } {
    
    ibacLog 4 $message $newline
}

proc ibacExtDbLog {message {newline 1} } {

    ibacLog 5 $message $newline

}

#
# ___oOo___
