#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ibacSendCommand.tcl,v 1.33 2005/09/01 15:18:04 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# gzins     17/04/00  created
# gzins     09/06/00  added 'displayError' parameter
#

#************************************************************************
#   NAME
#   ibacSendCommand - send CCS message
# 
#   DESCRIPTION
#   proc ibacSendCommand {envName procName command params {timeout 10000} 
#                        {check yes} {displayError 1} }
#     Send <command> with <params> as argument to the process <procName> running
#     in the environment <envName>.
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc ibacSendCommand {envName procName command {params ""} \
                      {timeout 10000} {check yes} {displayError 1} } {

    # Command checking
    global seq_ccsCmdCheck
    if {[string compare $check "no"] == 0} {
        set seq_ccsCmdCheck NO_CHECK
    } else {
        set seq_ccsCmdCheck CHECK_CMD
    }
    
    # Send the command
    if {[catch {seq_msgSendCommand $envName $procName $command $params} \
        cmdId]} {
        seq_msgFlush all -nowait
	    if {$displayError} {
            seq_errDisplay
        } else {
		    seq_errCloseStack
        }
	    return -1
    }

    # Wait for replies
    set errorNumber ""
    set lastReply 0
    while {!$lastReply} {
        if {[catch {seq_msgRecvReply $timeout -ascii $cmdId \
             errorNumber lastReply} \
            reply ]} {
            return $cmdId
        } 
    }

    # Check for error and display it if any
    if {$errorNumber != 0 } {  
	    if {$displayError} {
            seq_errDisplay
        } else {
		    seq_errCloseStack
        }
        return -1
    }

    # Return reply
    return $reply
}

#
# ___oOo___
