#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ibacUtils.tcl,v 1.33 2005/09/01 15:18:05 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# gzins     10/02/00  created
#

#************************************************************************
#   NAME
#   ibacUtils, ibacBusy, ibacReleaseBusy - miscellaneous utility functions
#
#   SYNOPSIS
#   ibacBusy          <busy> ?timeout? ?windows?
#   ibacReleaseBusy
#
#   DESCRIPTION
#   The procedure ibacBusy block/unblock keyboard, button, and pointer events
#   from windows (by default lock event from main window; i.e. .). Events are
#   unlocked when timeout is expired (by default after 3 min.).
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# enable/disable all children of main window apart from log buttons
proc ibacBusy {busy {timeout 180000} {windows "."}} {
    global gvar cv

    # If no tk application return
    if {! [info exists cv]} { return }

    # Cancel possible pending timeout
    catch {after cancel $gvar(ibacAfterBusyId)}


    if {$busy} {
        # Change cursor to watch
	    . config -cursor watch
	    update
        
        # Blocks events for all specified windows
        foreach window $windows {
            blt::busy hold $window
        }
        
        # Release cursor after timeout (default 3 min.)
        set gvar(ibacBusy) 1
        set gvar(ibacAfterBusyId) [after $timeout ibacReleaseBusy] ; 
    } else {
        ibacReleaseBusy
    }
}

# return to normal cursor
proc ibacReleaseBusy {} {
    global gvar
    . config -cursor left_ptr
    set gvar(ibacBusy) 0
    eval blt::busy release [blt::busy windows]
}

#
# ___oOo___
