#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ic0CONFIG2DBCFG.tcl,v 2.14 2005/09/22 07:54:40 vltsccm Exp $"
#
# ic0CONFIG2DBCFG.tcl
#
# who       when        what
# --------  ----------  ---------------------------------------------
# rschmutz  2005-09-08  VLTSW20050270: INS.SHUTi.POSNAMEi support added.
# rschmutz  2005-07-16  VLTSW20050248: INS.LAMPi.COOLDOWN support added.
# rschmutz  2005-07-14  VLTSW20050242: INS.ADC|DROT.PERIOD: allow msec.
# rschmutz  2004-11-23  VLTSW20040379: check/truncate string lengths.
# alongino  31/07/03  VLTSW20030334: multiple ICSs in same instrument
# rschmutz  09/04/03  VLTSW20030120: timeout1, timeout2 swapped.
# rschmutz  17/03/03  VLTSW20020572: INS.SENSORi.MIN/MAX support added.
# rschmutz  15/02/03  VLTSW20030052: INS.ADCi.SPDMIN/SPDMAX support added.
# rschmutz  31/12/02  VLTSW20020689: *.POSENCi and *.ALIGN support added.
# rschmutz  18/11/02  VLTSW20020626: get format, units from dictionary.
# rschmutz  11/11/02  SYNC lines aligned with ic0dev 1.90 dbl files.
# rschmutz  09/11/02  INS.*.INISWFCT support added.
# rschmutz  09/11/02  INS.*.PREINIFCT/POSTINIFCT support added.
# rschmutz  12/05/02  support for new category added (VLTSW20020181).
# rschmutz  12/05/02  INS.*.ENABLED not supported: deleted.
# rschmutz  12/05/02  INS.*.TWOSTEP support added.
# rschmutz  12/05/02  INS.*.MOVETOUT, INS.*.INITTOUT support added.
# rschmutz  11/05/02  place *.ENC in image header according to dic.
# rschmutz  11/05/02  accept optional INS.TIMi.PERIOD.
# rschmutz  22/12/01  initialize moveOrder attr.
# rschmutz  21/12/01  if ENABLED, AVAIL used: issue warning.
# rschmutz  19/12/01  icbTIMER support added.
# rschmutz  09/12/01  make sure that MCM configs. contain '<alias>NAME'.
# rschmutz  07/12/01  INS.*.INIALONE support added.
# rschmutz  07/12/01  get INS.POSi.POS format from ICS dictionary.
# rschmutz  25/10/01  INS.*.PREFCT/POSTFCT support added.
# rschmutz  03/09/01  icbSEN_CN77000, icbSEN_ESTERS support added.
# rschmutz  15/08/01  Bug in ConfigSetSENSOR_DIGITAL fixed.
# rschmutz  11/05/01  INS.POSi.PARAM/UNITCONV support added.
# rschmutz  29/04/01  icbSEN_EXPM support added.
# rschmutz  28/04/01  INS.POSi/TILTi/GRATi.TEMPREF/TEMPRAMP support added.
# rschmutz  28/04/01  INS.GRATi.ORDERMAX support added.
# rschmutz  11/12/00  INS.GRATi.TEMPREF,TEMPRAMP support added.
# rschmutz  10/12/00  icbMOT_ADC support added.
# rschmutz  09/12/00  insid parameter added.
# rschmutz  21/06/00  individual signal config. fixed for lamps and shutters.
# rschmutz  25/05/00  INS.LAMPi.MAXON support added.
# rschmutz  25/05/00  all sensors: *.LOGi support added.
# rschmutz  25/05/00  all sensors: set ic0senflag = 3 (SPR 20000186).
# rschmutz  25/05/00  index was missing in nearly all *.HEADERi reads.
# rschmutz  01/02/00  icbMOT_POS support added.
# alongino  18/01/00  prefix changed for sensors
# rschmutz  09/12/99  various details fixed.
# rschmutz  18/11/99  shutter standbyO signal is output.
# tphan     31/08/99  ConfigSetSensorADAM:fixed bug of typeText
# rschmutz  21/08/99  INS.CON.CONFIGSET processing added.
# rschmutz  17/08/99  error in DROT method fixed.
# rschmutz  06/08/99  created
#

#************************************************************************
#   NAME
#	ic0CONFIG2DBCFG - class to transfer the PAF config to *.dbcfg
#
#   SYNOPSIS
#	ic0CONFIG2DBCFG cfg2db $cfgname
#
#   PARENT CLASS
#	ctooDBCFG <cfg set id>
#
#   DESCRIPTION
#	ctooDBCFG provides methods to create a *.dbcfg file.
#
#   PUBLIC METHODS
#	ConfigSet env
#	    Writes the $VLTDATA/config/<env>.dbcfg file
#	    according to the loaded configuration.
#
#	GetLcuEnvs
#	    Returns a list of known LCU environments.
#
#   PROTECTED METHODS
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<name>.dic
#       $INTROOT/config/<name>.dic
#       $VLTROOT/config/<name>.dic
#           File containing the list of config. dictionaries.
#
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<name>.cfg
#       $INTROOT/config/<name>.cfg
#       $VLTROOT/config/<name>.cfg
#           Main configuration file.
#
#	$VLTDATA/config/<env>.dbcfg
#	    Created LCU configuration file.
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   RETURN VALUES
#
#   CAUTIONS
#       INS_ROOT resp. INTROOT must be defined to be able to read
#       a configuration stored in the corresponding location.
#
#	Whenever any OLDB value or location is updated in the ICS
#	OLDB, this class has to be updated accordingly!
#	Otherwise, the created *.dbcfg file will place the data
#	in the wrong location.
#
#   EXAMPLES
#	ic0CONFIG2DBCFG cfg2db XXXX
#	cfg2db ConfigSet lxxics1
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

#************************************************************************
#*# class: ic0CONFIG2DBCFG
#------------------------------------------------------------------------

class ic0CONFIG2DBCFG {
    inherit ctooDBCFG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { cfgname } {
	ctooCONFIG cfg $cfgname

	#
	# 1. get category
	#
	#    On request of ALO, we are differentiating here
	#    two categories:
	#     cfgCatg: category used for config. keywords
	#     insCatg: category used by the instrument,
	#	       i.e. SETUP, STATUS, logs.
	#
	set cfgCatg [cfg Category]
	if { $cfgCatg == "" } {
	    set cfgCatg "INS"
	}
	set cfgCatg1 [cconcat $cfgCatg "."]
	set cfgCatg1Len [clength $cfgCatg1]

	set insCatg $cfgCatg
	catch { set insCatg [cfg Get "$cfgCatg.CON.CATEGORY"] }
	set insCatg1 [cconcat $insCatg "."]
	set insCatg1Len [clength $insCatg1]

	# slxCATEGORY_LEN = logCATEGORY_LEN = 3,
	# i.e. accept max. 3 chars. before the first ".".
	set tmp $insCatg
	set insCatg2 [ctoken tmp "."]
	if { [clength $insCatg2] > 3 } {
	    error "Category '$insCatg2' too long (max. 3 chars.)."
	}

	#
	# 2. get instrument identifier, number of LCUs
	#
	set insid [cfg Get "$cfgCatg.CON.ID"]
	set lcunumKey "$cfgCatg.CON.LCUNUM"
	if {[catch { cfg Get $lcunumKey }]} {
	    set lcunumKey ""
	}
	set lcuEnvs [GetKeyList "$cfgCatg.CON.LCUENV" "$lcunumKey"]

	#
	# 3. load ICS dictionary
	#
	set insdictName [cconcat $insid "_ICS"]
	ctooDICT $this.insdict -dict $insdictName
	set insdict "$this.insdict"

	#
	# 4. set ICS OLDB root point
	#
	set icsDbRoot "<alias>$insid:ICS"
    }


    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {
	delete object $insdict
	delete object cfg
    }


    #****************************************************************
    #*# method: ConfigSet
    #----------------------------------------------------------------

    method ConfigSet { envName } {
	global env

	#
	# 1. search lcu id.
	#
	set lcuid [GetKeyIndex $lcuEnvs $envName]
	if { $lcuid == 0 } {
	    error "Environment $envName not known."
	}

	#
	# 2. open *.dbcfg file
	#
	Open "$env(VLTDATA)/config/[set envName].dbcfg"
	PutsCwp "@[set envName]:"

	#
	# 3. process common config.
	#
	ConfigSetCommon $lcuid

	#
	# 4. process all devices of the LCU
	#
	set order 0
	foreach device [GetDevices $lcuid] {
	    ConfigSetDevice $device $order
	    incr order
	}

	#
	# 5. process last common config.
	#
	ConfigSetCommonEnd $lcuid

	#
	# 6. close *.dbcfg file
	#
	Close

	#
	# 7. call user process
	#
	set userConfigSet ""
	catch { set userConfigSet [cfg Get "$cfgCatg.CON.CONFIGSET"] }
	if { "$userConfigSet" != "" } {
	    set msg [exec $userConfigSet $envName]
	    if { "$msg" != "" } {
		error "$msg."
	    }
	}
    }


    #****************************************************************
    #*# method: GetLcuEnvs
    #----------------------------------------------------------------

    method GetLcuEnvs {} {
	return $lcuEnvs
    }


    #****************************************************************
    #*# methods: ConfigSetCommon, ConfigSetCommonEnd
    #----------------------------------------------------------------

    protected method ConfigSetCommon { lcuid } {
	#
	# 1. configure ic0lcuServer
	#
	set lcuPoint "$icsDbRoot:PROCESSES:LCU[set lcuid]"
	PutsPoint "[set lcuPoint]:icsServer:frontend"
	PutsScalar prefix $insCatg
	set cfgList "
	    $cfgCatg.CON.CMDTOUT  timeout1    15000 -
	    $cfgCatg.CON.MOVETDIF interval        1 -
	    $cfgCatg.CON.MOVEMAX  maxParallel   100 -
	    $cfgCatg.CON.ONLINE   autoOnline      T L2N
	"
	PutsScalars $cfgList

	# init. ic0lcuServer move cmds. timeout
	set maxTimeout 120000
	catch { set maxTimeout [cfg Get "$cfgCatg.CON.MOVETOUT"] }

	#
	# 2. configure ic0devServer
	#
	PutsPoint "[set lcuPoint]:icsDevServer"
	PutsScalar prefix $insCatg
	set cfgList "
	    $cfgCatg.CON.MONTIME  monitorOnline  60000 -
	    $cfgCatg.CON.MONTIME  monitorStandby 60000 -
	"
	PutsScalars $cfgList

	#
	# 3. clear the keyword routing table
	#
	set routekeys ""
    }

    protected method ConfigSetCommonEnd { lcuid } {
	#
	# 1. set ic0lcuServer move cmds. timeout
	#
	set lcuPoint "$icsDbRoot:PROCESSES:LCU[set lcuid]"
	PutsPoint "[set lcuPoint]:icsServer:frontend"
	PutsScalar timeout2 $maxTimeout

	#
	# 2. set the keyword routing table
	#
	set routekeysSize [llength $routekeys]
	if { $routekeysSize != 0 } {
	    PutsScalar keywordsSize $routekeysSize
	    PutsTable keywords $routekeys
	}
    }

    #****************************************************************
    #*# method: GetDevices
    #
    # return the devices of a given LCU, or all devices (lcuid=0).
    #----------------------------------------------------------------

    protected method GetDevices { {lcuid 0} } {
	set devnumKey "$cfgCatg.CON.DEVNUM"
	if {[catch { cfg Get $devnumKey }]} {
	    set devnumKey ""
	}
	set allDevs [GetKeyList "$cfgCatg.CON.DEVICE" "$devnumKey"]
	set devList ""
	loop i 0 [llength $allDevs] {
	    set device [lindex $allDevs $i]
	    if { $lcuid == 0 } {
		lappend devList $device
	    } else {
		if {[catch { set devlcu [cfg Get [cconcat $device .LCUID] ] }]} {
		    set devlcu 1
		}
		if { $devlcu == $lcuid } {
		    lappend devList $device
		}
	    }
	}

	return $devList
    }


    #****************************************************************
    #*# method: ConfigSetDevice
    #----------------------------------------------------------------

    protected method ConfigSetDevice { device order } {
	#
	# 1. check device category
	#
	if { [csubstr $device 0 $cfgCatg1Len] != $cfgCatg1 } {
	    error "Device name '$device' does not start with category '$cfgCatg'."
	}

	#
	# 2. retrieve subsystem and optional DEVTYPE
	#
	#    default device type implicit in subsystem name
	#    default may be explicitely overwritten through .DEVTYPE
	#    For example, a piezo mirror may have prefix INS.MIRR1, but
	#    it is NOT a standard motorised MIRR device:
	#    it is indeed a special device, with different OLDB structure
	#    and different control hw (analog I/O board instead of MACCON).
	#
	regexp {[a-zA-Z]+} [lindex [split $device "."] 1] subSys
	set devType ""
	catch { set devType [cfg Get [cconcat $device .DEVTYPE]] }

	#
	# 3. check if the subsystem,devType combination is known
	#    (if it is not known, then assume special device)
	#
	if { $devType != "" } {
	    set name [cconcat $subSys "_" $devType]
	} else {
	    set name $subSys
	}
	set nameList {
	   LAMP SHUT ADC DPOR DROT FILT GRAT MIRR OPTI POS SLIT_LEN2
	   SLIT_WID2 SLIT_WHEEL TILT SENSOR_ADAM SENSOR_BAROMETER
	   SENSOR_CN76000 SENSOR_COOLING SENSOR_DIGITAL SENSOR_HUMIDITY
	   SENSOR_QUAT10 DET_EXPM SENSOR_CN77000 SENSOR_ESTERS TIM
	}
	if { [lsearch $nameList $name] == -1 } {
	    set devCmd PutsDevCommon
	} else {
	    set devCmd [cconcat ConfigSet $name]
	}

	#
	# 4. exec. device type config.
	#
	$devCmd $device $order
    }


    #****************************************************************
    #*# method: ConfigSetLAMP - lamp
    #----------------------------------------------------------------

    protected method ConfigSetLAMP { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order

	#
	# 2. add lamp specific config.
	#
	set cfgList "
	    [cconcat $device .ID]	id 	   - 15
	    [cconcat $device .NAME]	name	   - 15
	    [cconcat $device .CHGABLE]	useCfgElem F L2N
	    [cconcat $device .TIMEOUT]	delay 	2000 -
	"
	PutsScalars $cfgList

	# warmup time
	set val 0
	catch { set val [cfg Get [cconcat $device .WARMUP]] }
	set valTable [list [list "WarmUpTime" -1 1 1 $val "sec" -1]]
	PutsTable values $valTable 0

	# wait time
	set val 60
	catch { set val [cfg Get [cconcat $device .WAIT]] }
	set valTable [list [list "WaitTime" -1 1 1 $val "sec" -1]]
	PutsTable values $valTable 3

	# maxon
	set val 0
	catch { set val [cfg Get [cconcat $device .MAXON]] }
	set valTable [list [list "MaxOnTime" -1 1 1 $val "sec" -1]]
	PutsTable values $valTable 5

	# cooldown time
	set val 0
	catch { set val [cfg Get [cconcat $device .COOLDOWN]] }
	set valTable [list [list "CoolDownTime" -1 1 1 $val "sec" -1]]
	PutsTable values $valTable 6

	#
	# 3. add signals config.
	#

	# ioUsed
	set ioUsed 0
	catch { set ioUsed [cfg Get [cconcat $device .SIGUSED]] }
	if { $ioUsed == 0 } {
	    # ic0digLAMP_SIGNAL_DEFAULT
	    set ioUsed 45
	}
	# if ignoreFault: set ignoreFault flag in ioUsed.
	set ignoreFault 0
	catch { set ignoreFault [L2N [cfg Get [cconcat $device .IGFAULT]]] }
	if { $ignoreFault != 0 } {
	    incr ioUsed 1024
	}
	PutsScalar ioUsed $ioUsed

	# process signals keywords
	set sigDev [cfg Get [cconcat $device .SIGDEV]]
	set sigBit1 [cfg Get [cconcat $device .SIGBIT1]]
	loop j 1 7 {
	    # The signals have to be processed in the hardware default
	    # order, so that sigBit() is correctly configured in the
	    # default case (ioUsed = 0).
	    # Note that standbyO and standbyI are not used.
	    set i [lindex {0 1 3 4 6 2 5} $j]
	    if {![IsBitSet $ioUsed $i]} {
		set sigBit($i) 0
		set sigLow($i) 0
		set sigUsed($i) 0
		continue
	    }
	    if {[catch {
		set sigBit($i) [cfg Get [cconcat $device .SIGBIT $i]]
	    }]} {
		set sigBit($i) $sigBit1
	    }
	    if {[catch {
		set sigLow($i) [L2N [cfg Get [cconcat $device .SIGLOW $i]]]
	    }]} {
		set sigLow($i) [lindex {0 1 1 0 0 0 0} $i]
	    }
	    set sigUsed($i) 1
	    incr sigBit1
	}
	set tbl ""
	lappend tbl [list ".lampSwitchO" $sigDev $sigBit(1) $sigUsed(1) 1 $sigLow(1) 0]
	lappend tbl [list ".standbyO"    $sigDev $sigBit(2) $sigUsed(2) 1 $sigLow(2) 0]
	lappend tbl [list ".localI"      $sigDev $sigBit(3) $sigUsed(3) 0 $sigLow(3) 0]
	lappend tbl [list ".lampOnI"     $sigDev $sigBit(4) $sigUsed(4) 0 $sigLow(4) 0]
	lappend tbl [list ".standbyI"    $sigDev $sigBit(5) $sigUsed(5) 0 $sigLow(5) 0]
	lappend tbl [list ".faultI"      $sigDev $sigBit(6) $sigUsed(6) 0 $sigLow(6) 0]
	PutsTable signals $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSHUT - shutter
    #----------------------------------------------------------------

    protected method ConfigSetSHUT { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order

	#
	# 2. add shutter specific config.
	#
	set cfgList "
	    [cconcat $device .ID]	id 	   - 15
	    [cconcat $device .NAME]	name	   - 15
	    [cconcat $device .TIMEOUT]	delay 	5000 -
	    [cconcat $device .POSNAME1]	stClosed  {} 15
	    [cconcat $device .POSNAME2]	stOpen    {} 15
	"
	PutsScalars $cfgList

	#
	# 3. add signals config.
	#

	# ioUsed
	set ioUsed 0
	catch { set ioUsed [cfg Get [cconcat $device .SIGUSED]] }
	if { $ioUsed == 0 } {
	    # ic0digSHUTTER_SIGNAL_DEFAULT
	    set ioUsed 27
	}
	# if ignoreFault: set ignoreFault flag in ioUsed.
	set ignoreFault 0
	catch { set ignoreFault [L2N [cfg Get [cconcat $device .IGFAULT]]] }
	if { $ignoreFault != 0 } {
	    incr ioUsed 1024
	}
	PutsScalar ioUsed $ioUsed

	# process signals keywords
	set sigDev [cfg Get [cconcat $device .SIGDEV]]
	set sigBit1 [cfg Get [cconcat $device .SIGBIT1]]
	loop j 1 6 {
	    # The signals have to be processed in the hardware default
	    # order, so that sigBit() is correctly configured in the
	    # default case (ioUsed = 0).
	    # Note that shutClosedI is not used.
	    set i [lindex {0 1 4 2 5 3} $j]
	    if {![IsBitSet $ioUsed $i]} {
		set sigBit($i) 0
		set sigLow($i) 0
		set sigUsed($i) 0
		continue
	    }
	    if {[catch {
		set sigBit($i) [cfg Get [cconcat $device .SIGBIT $i]]
	    }]} {
		set sigBit($i) $sigBit1
	    }
	    if {[catch {
		set sigLow($i) [L2N [cfg Get [cconcat $device .SIGLOW $i]]]
	    }]} {
		set sigLow($i) [lindex {0 1 0 0 0 0} $i]
	    }
	    set sigUsed($i) 1
	    incr sigBit1
	}
	set tbl ""
	lappend tbl [list ".shutSwitchO" $sigDev $sigBit(1) $sigUsed(1) 1 $sigLow(1) 0]
	lappend tbl [list ".shutOpenI"   $sigDev $sigBit(2) $sigUsed(2) 0 $sigLow(2) 0]
	lappend tbl [list ".shutClosedI" $sigDev $sigBit(3) $sigUsed(3) 0 $sigLow(3) 0]
	lappend tbl [list ".localI"      $sigDev $sigBit(4) $sigUsed(4) 0 $sigLow(4) 0]
	lappend tbl [list ".faultI"      $sigDev $sigBit(5) $sigUsed(5) 0 $sigLow(5) 0]
	PutsTable signals $tbl
    }


    #****************************************************************
    #*# method: ConfigSetADC - ADC
    #----------------------------------------------------------------

    protected method ConfigSetADC { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order 1

	#
	# 2. specific config.
	#
	set cfgList {
	    .PERIOD   period   15000
	    .MOTSIGN  motSgn1  -1
	    .MOTSIGN2 motSgn2  -1
	    .REFENC   refEnc   0
	    .FORMULA  formula  1
	    .MINELEV  minElev  27.64
	    .POSOFFST posOffst 90
	    .COFFSET  cOffset  1.7387
	    .PSLOPE   pSlope   0.0023
	    .POFFSET  pOffset  743
	    .TSLOPE   tSlope   -0.0061
	    .TOFFSET  tOffset  12
	    .DROTFACT drotFact 0
	    .AFACTOR  aFactor  3.32
	    .SPDMIN   spdMin   0.01
	    .SPDMAX   spdMax   60
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    # default for motSgn2 is motSgn1
	    if { $var == "motSgn2" } {
		set $var $motSgn1
	    }
	    catch { set $var [cfg Get [cconcat $device $key]] }
	}
	set period [expr {int($period) / 1000.0}]
	set tbl ""
	lappend tbl [list "Range"        -1 2 1 0         "deg" -1]
	lappend tbl [list ""             -1 1 1 360       "deg" -1]
	lappend tbl [list "Tolerance"    -1 1 1 0.02      "deg" -1]
	lappend tbl [list "TaskWait"     -1 1 1 $period   "sec" -1]
	lappend tbl [list "UtcOffset"    -1 1 1 1         "sec" -1]
	lappend tbl [list "MinSpeed"     -1 1 1 $spdMin   "Enc/ms" -1]
	lappend tbl [list "MaxSpeed"     -1 1 1 $spdMax   "Enc/ms" -1]
	lappend tbl [list "Equinox"      -1 1 1 0         ""    -1]
	lappend tbl [list "PosAngSign"   -1 1 1 $motSgn2  ""    -1]
	lappend tbl [list "EncResolSign" -1 1 1 $motSgn1  ""    -1]
	lappend tbl [list "Reference"    -1 2 1 $refEnc   "Enc" -1]
	lappend tbl [list ""             -1 1 1 $refEnc   "Enc" -1]
	lappend tbl [list "Formula"      -1 1 1 $formula  ""    -1]
	lappend tbl [list "MinElev"      -1 1 1 $minElev  "deg" -1]
	lappend tbl [list "PosOffset"    -1 1 1 $posOffst "deg" -1]
	lappend tbl [list "Coffset"      -1 1 1 $cOffset  ""    -1]
	lappend tbl [list "Pslope"       -1 1 1 $pSlope   ""    -1]
	lappend tbl [list "Poffset"      -1 1 1 $pOffset  ""    -1]
	lappend tbl [list "Tslope"       -1 1 1 $tSlope   ""    -1]
	lappend tbl [list "Toffset"      -1 1 1 $tOffset  ""    -1]
	lappend tbl [list "DrotFactor"   -1 1 1 $drotFact ""    -1]
	lappend tbl [list "Afactor"      -1 1 1 $aFactor  ""    -1]
	PutsTable values $tbl 1

	#
	# 3. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0motAdc.class (actions table).
	PutsMotorTwoStep $device 0 2 6

	#
	# 4. add motor config.
	#
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetDPOR - depolarizer wheel
    #----------------------------------------------------------------

    protected method ConfigSetDPOR { device order } {
	PutsDevCommon $device $order 1
	PutsMotorCFcts $device
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotDpor.class (actions table).
	PutsMotorTwoStep $device 0 4 8
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetDROT - derotator
    #----------------------------------------------------------------

    protected method ConfigSetDROT { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order 1

	#
	# 2. specific config.
	#
	set cfgList {
	    .ANGMIN   angMin   0
	    .ANGMAX   angMax   360
	    .TOLERANC toleranc 0.02
	    .PERIOD   period   5000
	    .SPDMIN   spdMin   0.01
	    .SPDMAX   spdMax   5
	    .ANGSGN   angSgn   1
	    .RESOLSGN resolSgn -1
	    .STATENC  statEnc  0
	    .SKYENC   skyEnc   0
	    .ELEVENC  elevEnc  0
	    .STATENC1 statEnc1 0
	    .STATENC2 statEnc2 0
	    .STATENC3 statEnc3 0
	    .STATENC4 statEnc4 0
	    .STATENC5 statEnc5 0
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	}
	set period [expr {int($period) / 1000.0}]
	set tbl ""
	lappend tbl [list "Range"        -1 2 1 $angMin   "deg" -1]
	lappend tbl [list ""             -1 1 1 $angMax   "deg" -1]
	lappend tbl [list "Tolerance"    -1 1 1 $toleranc "deg" -1]
	lappend tbl [list "TaskWait"     -1 1 1 $period   "sec" -1]
	lappend tbl [list "UtcOffset"    -1 1 1 1         "sec" -1]
	lappend tbl [list "MinSpeed"     -1 1 1 $spdMin   "Enc/ms" -1]
	lappend tbl [list "MaxSpeed"     -1 1 1 $spdMax   "Enc/ms" -1]
	lappend tbl [list "Equinox"      -1 1 1 0         ""    -1]
	lappend tbl [list "PosAngSign"   -1 1 1 $angSgn   ""    -1]
	lappend tbl [list "EncResolSign" -1 1 1 $resolSgn ""    -1]
	lappend tbl [list "Reference"    -1 3 1 $statEnc  "Enc" -1]
	lappend tbl [list ""             -1 1 1 $skyEnc   "Enc" -1]
	lappend tbl [list ""             -1 1 1 $elevEnc  "Enc" -1]
	lappend tbl [list "StatPos"      -1 5 1 $statEnc1 "Enc" -1]
	lappend tbl [list ""             -1 1 1 $statEnc2 "Enc" -1]
	lappend tbl [list ""             -1 1 1 $statEnc3 "Enc" -1]
	lappend tbl [list ""             -1 1 1 $statEnc4 "Enc" -1]
	lappend tbl [list ""             -1 1 1 $statEnc5 "Enc" -1]
	PutsTable values $tbl 1

	#
	# 3. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0motDrot.class (actions table).
	PutsMotorTwoStep $device 0 2 6

	#
	# 4. add motor config.
	#
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetFILT - filter wheel
    #----------------------------------------------------------------

    protected method ConfigSetFILT { device order {encPos 4} } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set useCfgElem 0
	catch { set useCfgElem [L2N [cfg Get [cconcat $device .CHGABLE]] 3] }
	PutsScalar useCfgElem $useCfgElem
	set align 0
	catch { set align [cfg Get [cconcat $device .ALIGN]] }

	#
	# 3. add filter elements
	#
	set ids [GetKeyList [cconcat $device .ID]]
	set posNum [cfg Get [cconcat $device .POSNUM]]
	incr posNum
	PutsScalar positionsSize $posNum
	set posTable ""
	set encposList ""
	set rec {{#DUMMY#} "" "" 0 0 0 0 0 0 0}
	lappend posTable $rec
	loop i 1 $posNum {
	    set id [cfg Get [cconcat $device .POSID $i]]
	    set j [GetKeyIndex $ids $id]
	    if { $j == 0 } {
		error "Device $device: element $id not found."
	    }
	    set name $id
	    set id [TruncLen [cconcat $device .POSID $i] $id 15]
	    catch { set name  [cfg Get [cconcat $device .NAME $j]] }
	    set name [TruncLen [cconcat $device .NAME $j] $name 15]
	    set type FILTER
	    catch { set type  [cfg Get [cconcat $device .TYPE $j]] }
	    set type [TruncLen [cconcat $device .TYPE $j] $type 15]
	    set focus 0
	    catch { set focus [cfg Get [cconcat $device .FOCUS $j]] }
	    lappend posTable [list $id $name $type 0 $focus 0 0 0 0 0]

	    # POSENC, ALIGN are associated to the POSition,
	    # not the assigned element -> use $i
	    set encpos ""
	    catch { set encpos [cfg Get [cconcat $device .POSENC $i]] }
	    set aligni 0
	    catch { set aligni [cfg Get [cconcat $device .ALIGN $i]] }
	    if { $encpos == "" } {
		if { $align != 0 } {
		    error "Device $device: All values [cconcat $device .POSENCi] must be specified if [cconcat $device .ALIGN] is used ([cconcat $device .POSENC $i] is missing)."
		}
		if { $aligni != 0 } {
		    error "Device $device: [cconcat $device .POSENC $i] must be specified if [cconcat $device .ALIGN $i] is used."
		}
	    } else {
		set encpos [expr {$encpos + $aligni + $align}]
	    }
	    lappend encposList $encpos
	}
	PutsTable positions $posTable
	PutsMotorCFcts $device

	#
	# 4. report ENC in image header
	#
	set keyl [$insdict GetEntry [cconcat $prefix ".ENC"]]
	if {[lcontain [lindex $keyl 0] "header"]} {
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotFilter.class (see the file for details).
	    PutsTableAttr parameters $encPos 2 7
	} else {
	    PutsTableAttr parameters $encPos 2 5
	}

	#
	# 5. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILES:
	# ic0devMotFilter.class, ic0devMotOpti.class (actions table).
	PutsMotorTwoStep $device 0 {0 6} {10 14}

	#
	# 6. add motor config.
	#
	PutsMotor $device

	#
	# 7. set named positions encoder values
	#
	PutsMotorEncPos $device $encposList
    }


    #****************************************************************
    #*# method: ConfigSetGRAT - gratings wheel
    #----------------------------------------------------------------

    protected method ConfigSetGRAT { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set cfgList {
	    .TOLERANC toleranc  ""
	    .ALIGNMIN alignMin  1000
	    .ALIGNMAX alignMax -1000
	    .RESOL    resol     -
	    .ROT      rotAngle  -
	    .TEMPREF  tempRef   12
	    .CHGABLE  useCfgElem F
	    .UNITFCT  unitFunc  ""
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}
	# retrieve format from ICS dictionary
	lassign [GetDictEntry $prefix "WLEN" "nm" $toleranc] format unit toleranc

	PutsTableAttr parameters 4 4 $format
	set tbl [list [list "Tolerance"    -1 1 1 $toleranc "nm" -1]]
	PutsTable values $tbl 3
	set tbl ""
	lappend tbl [list "AlignRange"   -1 2 1 $alignMin "Enc" -1]
	lappend tbl [list ""             -1 1 0 $alignMax "Enc" -1]
	PutsTable values $tbl 7
	set tbl [list [list "parEncResol" -1 1 1 $resol   "Enc" -1]]
	PutsTable values $tbl 15
	set tbl ""
	lappend tbl [list "parTeta"     -1 1 1 $rotAngle "deg" -1]
	lappend tbl [list "parTemp0"    -1 1 1 $tempRef  "C"   -1]
	PutsTable values $tbl 17
	#set tbl ""
	#lappend tbl [list "TempRange"   -1 2 1 $tempMin "C" -1]
	#lappend tbl [list ""            -1 1 0 $tempMax "C" -1]
	#PutsTable values $tbl 20
	set useCfgElem [L2N $useCfgElem 7]
	PutsScalar useCfgElem $useCfgElem

	#
	# 3. add grating elements
	#
	set ids [GetKeyList [cconcat $device .ID]]
	set posNum 2
	catch { set posNum [cfg Get [cconcat $device .POSNUM]] }
	incr posNum
	PutsScalar positionsSize $posNum
	set posTable ""
	set rec {{#DUMMY#} "" "" 0 0 0 0 0 0 0}
	lappend posTable $rec
	set orderMax 0
	loop i 1 $posNum {
	    set id [cfg Get [cconcat $device .POSID $i]]
	    set j [GetKeyIndex $ids $id]
	    if { $j == 0 } {
		error "Device $device: element $id not found."
	    }
	    set name $id
	    set id [TruncLen [cconcat $device .POSID $i] $id 15]
	    catch { set name  [cfg Get [cconcat $device .NAME $j]] }
	    set name [TruncLen [cconcat $device .NAME $j] $name 15]
	    set type GRATING
	    catch { set type  [cfg Get [cconcat $device .TYPE $j]] }
	    set type [TruncLen [cconcat $device .TYPE $j] $type 15]
	    set wlenMin [cfg Get [cconcat $device .WLENMIN $j]]
	    set wlenMax [cfg Get [cconcat $device .WLENMAX $j]]
	    catch {
		set tmp ""
		set tmp [cfg Get [cconcat $device .ORDERMAX $j]]
		if { $tmp > $orderMax } {
		    set orderMax $tmp
		}
	    }
	    set zorder  [cfg Get [cconcat $device .ZORDER $j]]
	    set align 0
	    catch { set align [cfg Get [cconcat $device .ALIGN $i]] }
	    set grooves [cfg Get [cconcat $device .GROOVES $j]]
	    set disp 0
	    catch { set disp  [cfg Get [cconcat $device .DISP $j]] }
	    set focus 0
	    catch { set focus [cfg Get [cconcat $device .FOCUS $j]] }
	    set tempRamp 0
	    catch { set tempRamp [cfg Get [cconcat $device .TEMPRAMP $j]] }
	    if { $focus != 0 && $tempRamp != 0 } {
		error "[cconcat $device ".*" $j]: simultaneous use of FOCUS and TEMPRAMP is not supported."
	    }
	    if { $tempRamp != 0 } {
		set focus $tempRamp
	    }
	    lappend posTable [list $id $name $type 0 $zorder $grooves \
				   $wlenMin $wlenMax $focus $align]
	}
	PutsTable positions $posTable
	# if ORDERMAX is supplied: enable ORDER keyword
	if { $orderMax == 0 } {
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotGrating2.class (see the file for details).
	    set tbl [list [list "#" "ORDER" 1 3 "%d" "" 22 0 -1 0 -2 -2 -2 -2 " 1" 0 " 1" 0]]
	    PutsTable parameters $tbl 5
	} else {
	    set tbl [list [list "#" "ORDER" 7 3 "%d" "" 22 0 -1 0 0 0 -2 -2 " 1" 0 " 1" 0]]
	    PutsTable parameters $tbl 5

	    set tbl [list [list "" -1 1 0 $orderMax "" -1]]
	    PutsTable values $tbl 23
	}
	if { $tempRamp != 0 } {
	    # report TEMP in image header
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotGrating2.class (see the file for details).
	    PutsTableAttr parameters 6 2 7
	} else {
	    PutsTableAttr parameters 6 2 5
	}
	if { $unitFunc != "" } {
	    PutsScalar userUnitFct "ic0devMotUsr2Enc"
	    PutsScalar userUnitConvFct $unitFunc
	}
	PutsMotorCFcts $device

	#
	# 4. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotGrating2.class (actions table).
	PutsMotorTwoStep $device 1 {0 7} 11

	#
	# 5. add motor config.
	#
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetMIRR - mirror wheel
    #----------------------------------------------------------------

    protected method ConfigSetMIRR { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set align 0
	catch { set align [cfg Get [cconcat $device .ALIGN]] }

	#
	# 3. add filter elements
	#
	set ids [GetKeyList [cconcat $device .ID]]
	set posNum [cfg Get [cconcat $device .POSNUM]]
	incr posNum
	PutsScalar positionsSize $posNum
	set encposList ""
	set posTable ""
	set rec {{#DUMMY#} "" "" 0 0 0 0 0 0 0}
	lappend posTable $rec
	loop i 1 $posNum {
	    set id [cfg Get [cconcat $device .POSID $i]]
	    set j [GetKeyIndex $ids $id]
	    if { $j == 0 } {
		error "Device $device: element $id not found."
	    }
	    set name $id
	    set id [TruncLen [cconcat $device .POSID $i] $id 15]
	    catch { set name  [cfg Get [cconcat $device .NAME $j]] }
	    set name [TruncLen [cconcat $device .NAME $j] $name 15]
	    set type MIRROR
	    catch { set type  [cfg Get [cconcat $device .TYPE $j]] }
	    set type [TruncLen [cconcat $device .TYPE $j] $type 15]
	    lappend posTable [list $id $name $type 0 0 0 0 0 0 0]

	    # POSENC, ALIGN are associated to the POSition,
	    # not the assigned element -> use $i
	    set encpos ""
	    catch { set encpos [cfg Get [cconcat $device .POSENC $i]] }
	    set aligni 0
	    catch { set aligni [cfg Get [cconcat $device .ALIGN $i]] }
	    if { $encpos == "" } {
		if { $align != 0 } {
		    error "Device $device: All values [cconcat $device .POSENCi] must be specified if [cconcat $device .ALIGN] is used ([cconcat $device .POSENC $i] is missing)."
		}
		if { $aligni != 0 } {
		    error "Device $device: [cconcat $device .POSENC $i] must be specified if [cconcat $device .ALIGN $i] is used."
		}
	    } else {
		set encpos [expr {$encpos + $aligni + $align}]
	    }
	    lappend encposList $encpos
	}
	PutsTable positions $posTable
	PutsMotorCFcts $device

	#
	# 4. report ENC in image header
	#
	set keyl [$insdict GetEntry [cconcat $prefix ".ENC"]]
	if {[lcontain [lindex $keyl 0] "header"]} {
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotMirror.class (see the file for details).
	    PutsTableAttr parameters 4 2 7
	} else {
	    PutsTableAttr parameters 4 2 5
	}

	#
	# 5. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotMirror.class (actions table).
	PutsMotorTwoStep $device 0 {0 6} {10 14}

	#
	# 6. add motor config (xxmcfg<NAME>M).
	#
	PutsMotor $device

	#
	# 7. set named positions encoder values
	#
	PutsMotorEncPos $device $encposList
    }


    #****************************************************************
    #*# method: ConfigSetOPTI - wheel with optical elements
    #----------------------------------------------------------------

    protected method ConfigSetOPTI { device order } {
	ConfigSetFILT $device $order 5
    }


    #****************************************************************
    #*# method: ConfigSetPOS - continuous position function
    #----------------------------------------------------------------

    protected method ConfigSetPOS { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set cfgList {
	    .POSMIN   posMin   -
	    .POSMAX   posMax   -
	    .TOLERANC toleranc ""
	    .OFFSET   offset   0
	    .RAMP     ramp     1
	    .TEMPRAMP tempRamp 0
	    .TEMPREF  tempRef  12
	    .UNIT     unit     ""
	    .PARAM    param    "POS"
	    .UNITFCT  unitFunc ""
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}

	# retrieve the <op_prefix>.POS unit and format from the ICS dictionary
	lassign [GetDictEntry $prefix $param $unit $toleranc] format unit toleranc

	set tbl [list [list "#" "$param" 32775 36 "$format" "$unit" 1 1 -1 32 0 0 -2 -2 "" 0 "" 0]]
	PutsTable parameters $tbl 0
	if { $tempRamp != 0 } {
	    # report TEMP in image header
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotPos.class (see the file for details).
	    PutsTableAttr parameters 1 2 7
	} else {
	    PutsTableAttr parameters 1 2 5
	}
	set tbl ""
	lappend tbl [list "Range"       -1 2 1 $posMin   "$unit" -1]
	lappend tbl [list ""            -1 1 0 $posMax   "$unit" -1]
	lappend tbl [list "Tolerance"   -1 1 1 $toleranc "$unit" -1]
	PutsTable values $tbl 1
	set tbl ""
	lappend tbl [list "parOffset"   -1 1 1 $offset   "Enc"   -1]
	lappend tbl [list "parRamp"     -1 1 1 $ramp     "Enc"   -1]
	lappend tbl [list "parTemp0"    -1 1 1 $tempRef  "C"     -1]
	lappend tbl [list "parTemp1"    -1 1 1 $tempRamp "Enc/C" -1]
	#lappend tbl [list "TempRange"   -1 2 1 $tempMin  "C"     -1]
	#lappend tbl [list ""            -1 1 0 $tempMax  "C"     -1]
	PutsTable values $tbl 7
	if { $unitFunc != "" } {
	    PutsScalar userUnitFct "ic0devMotUsr2Enc"
	    PutsScalar userUnitConvFct $unitFunc
	}
	PutsMotorCFcts $device

	#
	# 3. report ENC in image header
	#
	set keyl [$insdict GetEntry [cconcat $prefix ".ENC"]]
	if {[lcontain [lindex $keyl 0] "header"]} {
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotPos.class (see the file for details).
	    PutsTableAttr parameters 2 2 7
	} else {
	    PutsTableAttr parameters 2 2 5
	}

	#
	# 4. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotPos.class (actions table).
	PutsMotorTwoStep $device 1 {0 5} 9

	#
	# 5. add motor config.
	#
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetSLIT_LEN2 - length slits
    #----------------------------------------------------------------

    protected method ConfigSetSLIT_LEN2 { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set cfgList {
	    .LENMIN   lenMin   -
	    .LENMAX   lenMax   -
	    .TOLERANC toleranc ""
	    .X1LENMIN x1lenMin -
	    .X1LENMAX x1lenMax -
	    .X2LENMIN x2lenMin -
	    .X2LENMAX x2lenMax -
	    .X1OFFSET x1offset -
	    .X2OFFSET x2offset -
	    .X1RESOL  x1resol  -
	    .X2RESOL  x2resol  -
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}
	# retrieve format, unit and tolerance from ICS dictionary
	lassign [GetDictEntry $prefix "LEN" "" $toleranc] format unit toleranc

	PutsTableAttr parameters 0 4 $format
	PutsTableAttr parameters 0 5 $unit
	PutsTableAttr parameters 1 4 $format
	PutsTableAttr parameters 1 5 $unit
	PutsTableAttr parameters 2 4 $format
	PutsTableAttr parameters 2 5 $unit
	set tbl ""
	lappend tbl [list "Range"      -1 2 1 $lenMin   "$unit" -1]
	lappend tbl [list ""           -1 1 1 $lenMax   "$unit" -1]
	lappend tbl [list "Tolerance"  -1 1 1 $toleranc "$unit" -1]
	lappend tbl [list "Range1"     -1 2 1 $x1lenMin "$unit" -1]
	lappend tbl [list ""           -1 1 0 $x1lenMax "$unit" -1]
	lappend tbl [list "Range2"     -1 2 1 $x2lenMin "$unit" -1]
	lappend tbl [list ""           -1 1 0 $x2lenMax "$unit" -1]
	PutsTable values $tbl 1
	set tbl ""
	lappend tbl [list "parOffset"   -1 2 1 $x1offset "Enc"     -1]
	lappend tbl [list ""            -1 1 0 $x2offset "Enc"     -1]
	lappend tbl [list "parEncResol" -1 2 1 $x1resol  "Enc/deg" -1]
	lappend tbl [list ""            -1 1 0 $x2resol  "Enc/deg" -1]
	PutsTable values $tbl 12
	PutsMotorCFcts $device 0

	#
	# 3. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotSlit2Len.class (actions table).
	PutsMotorTwoStep $device 1 {0 3 18 29} {9 12 24 35}

	#
	# 4. add motor config.
	#
	PutsMotor $device 1
	PutsMotor $device 2
    }


    #****************************************************************
    #*# method: ConfigSetSLIT_WID2 - width slits
    #----------------------------------------------------------------

    protected method ConfigSetSLIT_WID2 { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set cfgList {
	    .WIDMIN   widMin   -
	    .WIDMAX   widMax   -
	    .TOLERANC toleranc ""
	    .Y1WIDMIN y1widMin -
	    .Y1WIDMAX y1widMax -
	    .Y2WIDMIN y2widMin -
	    .Y2WIDMAX y2widMax -
	    .Y1OFFSET y1offset -
	    .Y2OFFSET y2offset -
	    .Y1RESOL  y1resol  -
	    .Y2RESOL  y2resol  -
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}
	# retrieve format, unit and tolerance from ICS dictionary
	lassign [GetDictEntry $prefix "WID" "" $toleranc] format unit toleranc

	PutsTableAttr parameters 0 4 $format
	PutsTableAttr parameters 0 5 $unit
	PutsTableAttr parameters 1 4 $format
	PutsTableAttr parameters 1 5 $unit
	PutsTableAttr parameters 2 4 $format
	PutsTableAttr parameters 2 5 $unit
	set tbl ""
	lappend tbl [list "Range"      -1 2 1 $widMin   "$unit" -1]
	lappend tbl [list ""           -1 1 1 $widMax   "$unit" -1]
	lappend tbl [list "Tolerance"  -1 1 1 $toleranc "$unit" -1]
	lappend tbl [list "Range1"     -1 2 1 $y1widMin "$unit" -1]
	lappend tbl [list ""           -1 1 0 $y1widMax "$unit" -1]
	lappend tbl [list "Range2"     -1 2 1 $y2widMin "$unit" -1]
	lappend tbl [list ""           -1 1 0 $y2widMax "$unit" -1]
	PutsTable values $tbl 1
	set tbl ""
	lappend tbl [list "parOffset"   -1 2 1 $y1offset "Enc"     -1]
	lappend tbl [list ""            -1 1 0 $y2offset "Enc"     -1]
	lappend tbl [list "parEncResol" -1 2 1 $y1resol  "Enc/deg" -1]
	lappend tbl [list ""            -1 1 0 $y2resol  "Enc/deg" -1]
	PutsTable values $tbl 12
	PutsMotorCFcts $device 0

	#
	# 3. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotSlit2Wid.class (actions table).
	PutsMotorTwoStep $device 1 {0 3 18 29} {9 12 24 35}

	#
	# 4. add motor config.
	#
	PutsMotor $device 1
	PutsMotor $device 2
    }


    #****************************************************************
    #*# method: ConfigSetSLIT_WHEEL - slit wheel
    #----------------------------------------------------------------

    protected method ConfigSetSLIT_WHEEL { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set chgable 0
	set useCfgElem 0
	catch { set useCfgElem [L2N [cfg Get [cconcat $device .CHGABLE]] 3] }
	PutsScalar useCfgElem $useCfgElem
	set align 0
	catch { set align [cfg Get [cconcat $device .ALIGN]] }

	#
	# 3. process each position
	#
	set ids [GetKeyList [cconcat $device .ID]]
	set posNum [cfg Get [cconcat $device .POSNUM]]
	incr posNum
	PutsScalar positionsSize $posNum
	set posTable ""
	set rec {{#DUMMY#} "" "" 0 0 0 0 0 0 0}
	lappend posTable $rec
	loop i 1 $posNum {
	    set id [cfg Get [cconcat $device .POSID $i]]
	    set j [GetKeyIndex $ids $id]
	    if { $j == 0 } {
		error "Device $device: element $id not found."
	    }
	    set name $id
	    set id [TruncLen [cconcat $device .POSID $i] $id 15]
	    catch { set name  [cfg Get [cconcat $device .NAME $j]] }
	    set name [TruncLen [cconcat $device .NAME $j] $name 15]
	    set wid [cfg Get [cconcat $device .WID $j]]
	    set len [cfg Get [cconcat $device .LEN $j]]
	    set align 0
	    catch { set align [cfg Get [cconcat $device .ALIGN $j]] }
	    lappend posTable [list $id $name SLIT 0 $wid $len 0 0 0 $align]

	    # POSENC, ALIGN are associated to the POSition,
	    # not the assigned element -> use $i
	    set encpos ""
	    catch { set encpos [cfg Get [cconcat $device .POSENC $i]] }
	    set aligni 0
	    catch { set aligni [cfg Get [cconcat $device .ALIGN $i]] }
	    if { $encpos == "" } {
		if { $align != 0 } {
		    error "Device $device: All values [cconcat $device .POSENCi] must be specified if [cconcat $device .ALIGN] is used ([cconcat $device .POSENC $i] is missing)."
		}
		if { $aligni != 0 } {
		    error "Device $device: [cconcat $device .POSENC $i] must be specified if [cconcat $device .ALIGN $i] is used."
		}
	    } else {
		set encpos [expr {$encpos + $aligni + $align}]
	    }
	    lappend encposList $encpos
	}
	PutsTable positions $posTable
	PutsMotorCFcts $device

	#
	# 4. report ENC in image header
	#
	set keyl [$insdict GetEntry [cconcat $prefix ".ENC"]]
	if {[lcontain [lindex $keyl 0] "header"]} {
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotSlits.class (see the file for details).
	    PutsTableAttr parameters 6 2 7
	} else {
	    PutsTableAttr parameters 6 2 5
	}

	#
	# 5. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotSlits.class (actions table).
	PutsMotorTwoStep $device 0 {0 6} {10 14}

	#
	# 6. add motor config.
	#
	PutsMotor $device

	#
	# 7. set named positions encoder values
	#
	PutsMotorEncPos $device $encposList
    }


    #****************************************************************
    #*# method: ConfigSetTILT - camera tilt
    #----------------------------------------------------------------

    protected method ConfigSetTILT { device order } {
	#
	# 1. common config.
	#
	set prefix [PutsDevCommon $device $order 1]

	#
	# 2. specific config.
	#
	set cfgList {
	    .POSMIN   posMin   -
	    .POSMAX   posMax   -
	    .TOLERANC toleranc ""
	    .OFFSET   offset   0
	    .RESOL    resol    -
	    .TEMPRAMP tempRamp 0
	    .TEMPREF  tempRef  12
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}
	# retrieve format, unit and tolerance from ICS dictionary
	lassign [GetDictEntry $prefix "POS" "" $toleranc] format unit toleranc

	PutsTableAttr parameters 0 4 $format
	PutsTableAttr parameters 0 5 $unit
	set tbl ""
	lappend tbl [list "Range"       -1 2 1 $posMin   "$unit" -1]
	lappend tbl [list ""            -1 1 0 $posMax   "$unit" -1]
	lappend tbl [list "Tolerance"   -1 1 1 $toleranc "$unit" -1]
	PutsTable values $tbl 1
	set tbl ""
	lappend tbl [list "parOffset"   -1 1 1 $offset   "Enc"   -1]
	lappend tbl [list "parEncResol" -1 1 1 $resol    "Enc"   -1]
	lappend tbl [list "parTemp0"    -1 1 1 $tempRef  "C"     -1]
	lappend tbl [list "parTemp1"    -1 1 1 $tempRamp "Enc/C" -1]
	#lappend tbl [list "TempRange"   -1 2 1 $tempMin  "C"     -1]
	#lappend tbl [list ""            -1 1 0 $tempMax  "C"     -1]
	PutsTable values $tbl 7
	if { $tempRamp != 0 } {
	    # report TEMP in image header
	    # THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	    # ic0devMotTilt.class (see the file for details).
	    PutsTableAttr parameters 1 2 7
	} else {
	    PutsTableAttr parameters 1 2 5
	}
	PutsMotorCFcts $device

	#
	# 3. set two step/no two step movements
	#
	# THE FOLLOWING LINE MUST BE KEPT IN SYNC WITH FILE:
	# ic0devMotTilt.class (actions table).
	PutsMotorTwoStep $device 1 {0 5} 9

	#
	# 4. add motor config.
	#
	PutsMotor $device
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_ADAM - ADAM analog sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_ADAM { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit ""
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    #lassign [GetDictEntry $insCatg $fits $unit] dummy unit
	    set addr [cfg Get [cconcat $device .SENADDR $i]]
	    set typeText [cfg Get [cconcat $device .SENTYPE $i]]
	    # allow an integer to be able to use an undocumented type
	    # (see ic0senAdam.class)
	    if { [string is digit -strict $typeText] } {
		set type $typeText
	    } else {
		set type [lsearch {ANALOG TRUE_FALSE ON_OFF OPEN_CLOSED \
				   OPEN_CLOSED2 PRESSURE} $typeText]
		incr type
		if { $type == 0 } {
		    error "Key [cconcat $device .SENTYPE $i]: wrong type $typeText."
		}
	    }
	    set ref 0
	    catch { set ref  [cfg Get [cconcat $device .SENREF $i]] }
	    set ramp 1
	    catch { set ramp [cfg Get [cconcat $device .SENRAMP $i]] }
	    lappend tbl [list $name $desc $flags $fits $unit $addr \
			      $type $ref $ramp]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_BAROMETER - Barometer analog sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_BAROMETER { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set tbl ""
	set name BARO
	catch { set name [cfg Get [cconcat $device .NAME 1]] }
	set name [TruncLen [cconcat $device .NAME 1] $name $sensorNameLen 1]
	set desc "Barometer pressure"
	catch { set desc [cfg Get [cconcat $device .DESC 1]] }
	set desc [TruncLen [cconcat $device .DESC 1] $desc 31 2]
	set header 0
	catch { set header [L2N [cfg Get [cconcat $device .HEADER 1]]] }
	set log 0
	catch { set log [L2N [cfg Get [cconcat $device .LOG 1]]] }
	set flags [expr {2 * $log + $header}]
	set fits [cfg Get [cconcat $device .FITS 1]]
	CheckLen [cconcat $device .FITS 1] $fits 31
	if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
	    error "[cconcat $device .FITS 1] = '$fits' does not start with category '$insCatg'."
	}
	set fits [csubstr $fits $insCatg1Len end]
	set tbl [list [list $name $desc $flags $fits mbar]]
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_CN76000 - Omega CN76000 analog sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_CN76000 { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit ""
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    #lassign [GetDictEntry $insCatg $fits $unit] dummy unit
	    set addr [cfg Get [cconcat $device .SENADDR $i]]
	    lappend tbl [list $name $desc $flags $fits $unit $addr]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_COOLING - ESO cabinet cooling controller
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_COOLING { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    set addr [cfg Get [cconcat $device .SENADDR $i]]
	    switch $addr {
		1 {
		    set desc "Cooling outlet temp."
		    set unit "C"
		}
		2 {
		    set desc "Cooling Inlet temp."
		    set unit "C"
		}
		3 {
		    set desc "Cabinet temp."
		    set unit "C"
		}
		4 {
		    set desc "Ambient temp."
		    set unit "C"
		}
		5 {
		    set desc "Flow rate 1"
		    set unit "l/min"
		}
		6 {
		    set desc "Flow rate 2"
		    set unit "l/min"
		}
		7 {
		    set desc "Flow rate 3"
		    set unit "l/min"
		}
		8 {
		    set desc "Cabinet door open alarm"
		    set unit "l/min"
		}
		9 {
		    set desc "CSTAT1 register"
		    set unit ""
		}
		10 {
		    set desc "CSTAT2 register"
		    set unit ""
		}
		11 {
		    set desc "FEPROM register"
		    set unit ""
		}
		12 {
		    set desc "Communication register"
		    set unit ""
		}
		default {
		    error "Key [cconcat $device .SENADDR $i]: Wrong value $addr."
		}
	    }
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    lappend tbl [list $name $desc $flags $fits $unit $addr]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_DIGITAL - Digital sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_DIGITAL { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port, period
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 19
	PutsScalar acroDevice $port
	set period 10000
	catch { set period [cfg Get [cconcat $device .PERIOD]] }
	PutsScalar period $period

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set startBit [cfg Get [cconcat $device .SIGBIT $i]]
	    set width    [cfg Get [cconcat $device .SIGWIDTH $i]]
	    set low      [L2N [cfg Get [cconcat $device .SIGLOW $i]]]
	    set val0 ""
	    set key [cconcat $device .SIGVAL [format "%02d" $i] 0]
	    catch { set val0 [cfg Get $key] }
	    set val0 [TruncLen $key $val0 15]
	    set val1 ""
	    set key [cconcat $device .SIGVAL [format "%02d" $i] 1]
	    catch { set val1 [cfg Get $key] }
	    set val1 [TruncLen $key $val1 15]
	    set val2 ""
	    set key [cconcat $device .SIGVAL [format "%02d" $i] 2]
	    catch { set val2 [cfg Get $key] }
	    set val2 [TruncLen $key $val2 15]
	    set val3 ""
	    set key [cconcat $device .SIGVAL [format "%02d" $i] 3]
	    catch { set val3 [cfg Get $key] }
	    set val3 [TruncLen $key $val3 15]
	    set simValue 0
	    catch { set simValue [cfg Get [cconcat $device .SIGSIM $i]] }
	    lappend tbl [list $name $desc $flags $fits $startBit \
			      $width $low $val0 $val1 $val2 $val3 \
			      $simValue]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_HUMIDITY - Humidity sensor
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_HUMIDITY { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum 3
	set tbl ""
	loop i 1 4 {
	    set name [lindex { "" "EHST" "EHRH" "EHDP" } $i]
	    catch { set name [cfg Get [cconcat $device .NAME $i]] }
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    set desc [lindex { ""
		"Humidity sensor temp."
		"Relative humidity"
		"Dew point" } $i]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit [lindex { "" "C" "%" "C" } $i]
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    set unit [TruncLen [cconcat $device .SENUNIT $i] $unit 7]
	    #lassign [GetDictEntry $insCatg $fits $unit] dummy unit
	    lappend tbl [list $name $desc $flags $fits $unit]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_QUAT10 - HERAEUS QUAT10 analog sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_QUAT10 { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit "C"
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    set unit [TruncLen [cconcat $device .SENUNIT $i] $unit 7]
	    lappend tbl [list $name $desc $flags $fits $unit]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetDET_EXPM - EXPM
    #----------------------------------------------------------------

    protected method ConfigSetDET_EXPM { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order

	#
	# 2. specific config.
	#
	set cfgList {
	    .PORT     port     -
	    .UIT      uit      1
	    .OFFDRK   offdrk   0
	    .OFFSKY   offsky   0
	    .SIMVALUE simvalue 0
	    .LOG      log      F
	}
	foreach { key var def } $cfgList {
	    set $var $def
	    catch { set $var [cfg Get [cconcat $device $key]] }
	    if { "[set $var]" == "-" } {
		error "Key [cconcat $device $key] not found."
	    }
	}
	CheckLen [cconcat $device .PORT] $port 31
	PutsScalar comDevice $port
	PutsScalar offdrk    $offdrk
	PutsScalar offsky    $offsky
	# Only values lower than minlog are logged.
	# I.e. we can disable logging, placing minlog very high.
	if { [L2N $log] } {
	    set minlog 10
	} else {
	    set minlog 1000000000
	}
	PutsScalar ctMinLog $minlog
	set tbl ""
	lappend tbl [list "DefaultUIT"   -1 1 1 $uit      "" -1]
	lappend tbl [list "SimValue"     -1 1 1 $simvalue "" -1]
	PutsTable values $tbl 1
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_CN77000 - Omega CN77000 analog sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_CN77000 { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit ""
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    set unit [TruncLen [cconcat $device .SENUNIT $i] $unit 7]
	    #lassign [GetDictEntry $insCatg $fits $unit] dummy unit
	    set addr [cfg Get [cconcat $device .SENADDR $i]]
	    set refmin ""
	    catch { set refmin [cfg Get [cconcat $device .MIN $i]] }
	    set refmax ""
	    catch { set refmax [cfg Get [cconcat $device .MAX $i]] }
	    if { $refmin != "" && $refmax == "" } {
		error "[cconcat $device .MIN $i] specified, but [cconcat $device .MAX $i] is missing."
	    }
	    if { $refmin == "" && $refmax != "" } {
		error "[cconcat $device .MAX $i] specified, but [cconcat $device .MIN $i] is missing."
	    }
	    if { $refmin != "" && $refmax != "" } {
		# enable SETUP
		if { [expr {$addr % 10}] != 1 } {
		    error "[cconcat $device .MIN/MAX $i] specified, but this is not a controller set value."
		}
		if { $refmin > $refmax } {
		    error "[cconcat $device .MIN $i] > [cconcat $device .MAX $i]."
		}
		incr flags 4
		lappend tbl [list $name $desc $flags $fits $unit $addr $refmin $refmax]

		# tell ic0lcuServer to route the sensor keyword to this device
		lappend routekeys [list $fits $devName 0 0]
	    } else {
		lappend tbl [list $name $desc $flags $fits $unit $addr 0 0]
	    }
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetSENSOR_ESTERS - Esters DC24 temperature sensors
    #----------------------------------------------------------------

    protected method ConfigSetSENSOR_ESTERS { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order
	PutsScalar sensorsPrefix $insCatg
	PutsScalar ic0senflag 3

	#
	# 2. port
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	set port [cfg Get [cconcat $device .PORT]]
	CheckLen [cconcat $device .PORT] $port 15
	PutsPoint "<alias>[string toupper $devName]:communication"
	PutsScalar device $port
	PutsPoint "<alias>[string toupper $devName]"

	#
	# 3. list of sensors
	#
	set sensorsNum [cfg Get [cconcat $device .NUM]]
	PutsScalar sensorsSize $sensorsNum
	set tbl ""
	incr sensorsNum
	loop i 1 $sensorsNum {
	    set name [cfg Get [cconcat $device .NAME $i]]
	    set desc $name
	    set name [TruncLen [cconcat $device .NAME $i] $name $sensorNameLen 1]
	    catch { set desc [cfg Get [cconcat $device .DESC $i]] }
	    set desc [TruncLen [cconcat $device .DESC $i] $desc 31 2]
	    set header 0
	    catch { set header [L2N [cfg Get [cconcat $device .HEADER $i]]] }
	    set log 0
	    catch { set log [L2N [cfg Get [cconcat $device .LOG $i]]] }
	    set flags [expr {2 * $log + $header}]
	    set fits [cfg Get [cconcat $device .FITS $i]]
	    CheckLen [cconcat $device .FITS $i] $fits 31
	    if { [csubstr $fits 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .FITS $i] = '$fits' does not start with category '$insCatg'."
	    }
	    set fits [csubstr $fits $insCatg1Len end]
	    set unit "C"
	    catch { set unit [cfg Get [cconcat $device .SENUNIT $i]] }
	    set unit [TruncLen [cconcat $device .SENUNIT $i] $unit 7]
	    set addr [cfg Get [cconcat $device .SENADDR $i]]
	    lappend tbl [list $name $desc $flags $fits $unit $addr]
	}
	PutsTable sensors $tbl
    }


    #****************************************************************
    #*# method: ConfigSetTIM - TIMER
    #----------------------------------------------------------------

    protected method ConfigSetTIM { device order } {
	#
	# 1. common config.
	#
	PutsDevCommon $device $order

	#
	# 2. specific config.
	#
	set timer  [cfg Get [cconcat $device .TIMER]]
	set period 0
	catch { set period [cfg Get [cconcat $device .PERIOD]] }
	set period [expr {int($period * 1000)}]
	PutsScalar timer  $timer
	PutsScalar period $period
    }


    #****************************************************************
    #*# method: PutsDevCommon - device common config.
    #----------------------------------------------------------------

    protected method PutsDevCommon { device order {isMotor 0} } {
	#
	# 1. set OLDB point
	#
	set devName [cfg Get [cconcat $device .DEVNAME]]
	if { [string first { } $devName] != -1 } {
	    error "[cconcat $device .DEVNAME] = '$devName': spaces not allowed."
	}
	CheckLen [cconcat $device .DEVNAME] $devName 19
	PutsPoint "<alias>[string toupper $devName]"
	#puts "Configuring $devName . . ."

	#
	# 2. set device name, prefix, description, LCU id. and moveOrder
	#
	PutsScalar device $devName
	set val $device
	catch { set val [cfg Get [cconcat $device .DEVDESC]] }
	set val [TruncLen [cconcat $device .DEVDESC] $val 19 2]
	PutsScalar description $val
	set val $device
	if {![catch { set val [cfg Get [cconcat $device .PREFIX]] }]} {
	    if { [csubstr $val 0 $insCatg1Len] != $insCatg1 } {
		error "[cconcat $device .PREFIX] = '$val' does not start with category '$insCatg'."
	    }
	} else {
	    if { $cfgCatg != $insCatg } {
		set val [csubstr $val $cfgCatg1Len end]
		set val [cconcat $insCatg1 $val]
	    }
	}
	if { [clength $val] > 15 } {
	    error "$device prefix = '$val' too long (max. 15 chars.)."
	}
	set prefix $val
	PutsScalar prefix $prefix
	if {[catch { set val [cfg Get [cconcat $device .LCUID] ] }]} { set val 1 }
	PutsScalar lcuId $val
	PutsScalar moveOrder $order

	#
	# 3. set optional flags
	#
	set val "F"
	catch { set val [cfg Get [cconcat $device .SWSIM]] }
	PutsScalar simulation [L2N $val]
	set val "F"
	catch { set val [cfg Get [cconcat $device .INIALONE]] }
	if { $val == "T" } {
	    PutsScalar initParallel 1
	}
	if { $isMotor } {
	    set val 90000
	    catch { set val [cfg Get [cconcat $device .MOVETOUT]] }
	    PutsScalar moveTimeout $val
	    if { $val > $maxTimeout } {
		set maxTimeout $val
	    }
	    set val 120000
	    catch { set val [cfg Get [cconcat $device .INITTOUT]] }
	    PutsScalar initTimeout $val
	    if { $val > $maxTimeout } {
		set maxTimeout $val
	    }
	}
	catch {
	    set val [cfg Get [cconcat $device .AVAIL]]
	    puts "WARNING: [cconcat $device .AVAIL] keyword not supported: ignored."
	}

	return $prefix
    }


    #****************************************************************
    #*# method: PutsScalars
    #
    #   cfgList must be a list, where each 4 entries correspond
    #   to one config. key:
    #	  key : FITS name of the config. parameter
    #     attr: attribute name where to store the config. value
    #     def : "-" or the default value, if the key is optional.
    #	  conv: "" or "L2N" if logical FITS must be converted
    #		to a number (0 or 1).
    #	  conv: <maxlen>: max. allowed string length
    #----------------------------------------------------------------

    protected method PutsScalars { cfgList } {

	foreach { key attr default conv } $cfgList {
	    if {[catch { set val [cfg Get $key] }]} {
		if { "$default" == "-" } {
		    error "Key $key not found."
		}
		set val $default
	    }

	    if { "$conv" == "L2N" } {
		set val [L2N $val]
	    } elseif { [ctype digit $conv] } {
		set val [TruncLen $key $val $conv]
	    }
	    PutsScalar $attr $val
	}

    }


    #****************************************************************
    #*# method: PutsMotorCFcts
    #----------------------------------------------------------------

    protected method PutsMotorCFcts { device {setValue 1} } {

	foreach { key attr } {
	    ".INISWFCT"	  "userInitSwFct"
	    ".PREINIFCT"  "userPreInitFct"
	    ".POSTINIFCT" "userPostInitFct"
	    ".PREFCT"	  "userPreMoveFct"
	    ".POSTFCT"	  "userPostMoveFct"
	} {
	    set val ""
	    if { $setValue } {
		catch { set val [cfg Get [cconcat $device $key]] }
	    }
	    PutsScalar $attr $val
	}
    }


    #****************************************************************
    #*# method: PutsMotorTwoStep
    #
    #   Config. the motor to use two step movements or not.
    #   In the motor device OLDB actions table, place in the
    #   <abs> list of records: motTWO_STEP_ABS or motABSOLUTE values,
    #   <rel> list of records: motTWO_STEP_REL or motRELATIVE values.
    #----------------------------------------------------------------

    protected method PutsMotorTwoStep { device defval absRecs relRecs } {

	#
	# 1. read <cfg_prefix>.TWOSTEP keyword
	#
	set val $defval
	catch { set val [L2N [cfg Get [cconcat $device .TWOSTEP]]] }
	if { $val } {
	    # motTWO_STEP_ABS, motTWO_STEP_REL
	    lassign {9 10} absMove relMove
	} else {
	    # motABSOLUTE, motRELATIVE
	    lassign {1 2} absMove relMove
	}

	#
	# 2. set absolute movements
	#
	foreach rec $absRecs {
	    PutsTableAttr actions $rec 11 $absMove
	}

	#
	# 3. set relative movements
	#
	foreach rec $relRecs {
	    PutsTableAttr actions $rec 11 $relMove
	}
    }


    #****************************************************************
    #*# method: PutsMotor
    #
    #   adds file <xx>mcfg<NAME><num>M.dbcfg.
    #   Strings "@<env>:" are replaced with
    #   the env. where the device is located.
    #----------------------------------------------------------------

    protected method PutsMotor { device {num ""} } {

	set modPrefix [cfg Get "$cfgCatg.CON.PREFIX"]
	set devName [string toupper [cfg Get [cconcat $device .DEVNAME]]]
	set motName [cconcat $devName $num "M"]
	if {[catch { set lcuId [cfg Get [cconcat $device .LCUID] ] }]} { set lcuId 1 }
	set lcuEnv [lindex $lcuEnvs [incr lcuId -1]]
	set lcuEnvText [cconcat @ $lcuEnv :]
	AddFile [cconcat $modPrefix mcfg $motName] \
		{@[a-zA-Z0-9_]+:} $lcuEnvText      \
		":Appl_data:.+:$devName:" ":<alias>$devName:"
	PutsCwp $lcuEnvText
    }


    #****************************************************************
    #*# method: PutsMotorEncPos
    #----------------------------------------------------------------

    protected method PutsMotorEncPos { device encposList } {

	set cwp 0
	set i 1
	foreach encpos $encposList {
	    if { $encpos != "" } {
		if { $cwp == 0 } {
		    set devName [cfg Get [cconcat $device .DEVNAME]]
		    PutsPoint "<alias>[string toupper $devName]M:SERVER:CONF"
		    set cwp 1
		}
		PutsTableAttr namedPositions $i 2 $encpos
	    }
	    incr i
	}
    }

    #****************************************************************
    #*# method: GetKeyList
    #
    #   Retrieve the values of all "$key$i" keys found
    #   in the configuration and return them in a list.
    #   If numkey is supplied, then it indicates the number
    #   of keys to retrieve.
    #   E.g.: to retrieve all LCU envs.:
    #	  GetKeyList "INS.CON.LCUENV" "INS.CON.LCUNUM"
    #----------------------------------------------------------------

    protected method GetKeyList { key {numkey ""} } {

	set vals {}
	if { "$numkey" != "" } {
	    set num [cfg Get $numkey]
	    incr num
	    loop i 1 $num {
		lappend vals [cfg Get [cconcat $key $i]]
	    }
	} else {
	    set i 1
	    while {1} {
		if {[catch {
		    set val [cfg Get [cconcat $key $i]]
		}]} {
		    break
		}
		lappend vals $val
		incr i
	    }
	}

	return $vals
    }


    #****************************************************************
    #*# method: GetKeyIndex
    #
    #   Receives a list made by GetKeyList and a value.
    #   The value is searched in the list, and the index i
    #   corresponding to the key "$key$i" is returned.
    #----------------------------------------------------------------

    protected method GetKeyIndex { list val } {
	set i [lsearch -exact $list $val]
	incr i
	return $i
    }


    #****************************************************************
    #*# method: GetDictEntry
    #
    #   Retrieve the key <prefix>.<param> from the ICS dictionary.
    #   Return format and unit found.
    #   Return the tolerance, adjusted according to the format,
    #   if it was initially empty.
    #----------------------------------------------------------------

    protected method GetDictEntry { prefix param {unit ""} {toleranc ""} } {

	# 1. obtain the <prefix>.<param> unit and format from the ICS dictionary
	set format ""
	set key [cconcat $prefix "." $param]
	set keyl [$insdict GetEntry $key]
	set format [lindex $keyl 2]
	set unit1  [lindex $keyl 3]
	if { [clength $format] > 7 } {
	    error "The $key format '$format' is too long (max. 7 chars allowed, found in the $insdictName dictionary)."
	}

	# 2. check that unit is present and consistent
	if { $unit1 != "" } {
	    if { $unit != "" && $unit != $unit1 } {
		error "Wrong $key unit '$unit1' found in the $insdictName dictionary (expected '$unit')."
	    }
	    set unit $unit1
	}
	if { $unit == "" } {
	    error "Unit for $key not found in $insdictName dictionary."
	}
	if { [clength $unit] > 7 } {
	    error "The $key unit '$unit' is too long (max. 7 chars allowed, found in the $insdictName dictionary)."
	}

	# 3. set tolerance according to format
	if { $toleranc == "" } {
	    set val 1
	    regexp {\.([0-9]+)} $format dummy val
	    if { $val == 0 } {
		set toleranc 1.0
	    } else {
		set toleranc [cconcat "0." [replicate "0" $val] "5"]
	    }
	}

	return [list $format $unit $toleranc]
    }


    #****************************************************************
    #*# method: L2N
    #----------------------------------------------------------------

    protected method L2N { val {num 1} } {
	if { "[cindex $val 0]" == "T" } {
	    set val $num
	} else {
	    set val 0
	}
	return $val
    }


    #****************************************************************
    #*# method: IsBitSet
    #----------------------------------------------------------------

    protected method IsBitSet { field bit } {
	if { [expr {$field & [int [pow 2 "$bit - 1"]]}] != 0 } {
	    return 1
	} else {
	    return 0
	}
    }


    #****************************************************************
    #*# method: TruncLen
    #
    # type = 0: normal behaviour: check if val is longer than maxlen,
    #		truncate it if it is too long.
    # type = 1: retrieve only the first word, ignore the rest.
    # type = 2: if val is too long, then ignore everything past
    #           the first ' (' occurrence, and check again.
    #
    # type = 1 is to be used by INS.SENSORi.NAMEi keywords:
    # The first word is the sensor id. passed to LCU, the rest
    # may be used by icbpan.
    #
    # type = 2 is to be used by any description: again a longer
    # string may be passed to icbpan, than the one that can be
    # stored in the LCU OLDB.
    #----------------------------------------------------------------

    protected method TruncLen { key val maxlen {type 0} } {

	# if type = 1: truncate everything past the first word
	if { $type == 1 } {
	    set i [string first { } $val]
	    if { $i != -1 && $i <= $maxlen } {
		set val [csubstr $val 0 $i]
	    }
	}

	# check length
	if { [clength $val] > $maxlen } {
	    set i [string first { (} $val]
	    if { $type == 2 && $i != -1 && $i <= $maxlen } {
		set val [csubstr $val 0 $i]
	    } else {
		set val [csubstr $val 0 $maxlen]
		puts "WARNING: $key truncated to '$val' (max. $maxlen chars.)"
	    }
	}

	return $val
    }


    #****************************************************************
    #*# method: CheckLen
    #----------------------------------------------------------------

    protected method CheckLen { key val maxlen } {
	if { [clength $val] > $maxlen } {
	    error "$key = '$val' too long (max. $maxlen chars.)"
	}

	return $val
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    protected variable cfg
    protected variable cfgCatg
    protected variable cfgCatg1
    protected variable cfgCatg1Len
    protected variable insCatg
    protected variable insCatg1
    protected variable insCatg1Len
    protected variable lcuEnvs
    protected variable insdict ""
    protected variable insdictName ""
    protected variable maxTimeout 0
    protected variable routekeys ""
    protected variable insid
    protected variable icsDbRoot

    protected variable sensorNameLen 5 ; # 15
}

#************************************************************************
# ___oOo___
