#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanICSCmd.tcl,v 1.47 2005/07/01 13:25:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dimarcan  16/11/00  created
#

#************************************************************************
#   NAME
#    icbpanICSCmd - wrapper class for icbpanICSCmd_uifClass
# 
#   SYNOPSIS
#    icbpanICSCmd .w \
#               -background Grey90   \
#               -relief raised       \
#               -cwp "<alias>${insName}:ICS:PROCESSES:WS:icsControl" \
#               -procName $procName]
#
#
#     where procName is the name of the ICS control process
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICS control process. The corresponding
#    ibcpanICSCmd_uifClass contains a set of option button
#    and one entry widget which can be used to select the command
#    + parameters to be sent to the ICS WS process.
#    The button "Send" takes care to send the selected cmds to ICS. 
#
#   PUBLIC METHODS
#    icbpanICSControl  xxx
#       constructor. Calls "initFunction" which initialize the option
#       buttons with the list of valid ICS cmds and the corresponding
#       list of valid parameters. Both lists are extracted from CDT
#       by quering the cmdManager process. The Send button uses the
#       internal method "sendParString" to actual send the constructed
#       list of command(s) to the ICS process.
#
#   PUBLIC VARIABLES
#    procName
#       name of the front-end WS ICS process 
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8), icbpanLib
#
#   BUGS     
#
#------------------------------------------------------------------------
#

class icbpanICSCmd {
   inherit icbpanICSCmd_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {
      set commands [icbpanCmdQueryCommands $procName]
      if {$commands == 1} {return $commands}
      $cv.cmdList configure -itemList $commands -default "" \
                            -command [code $this updateParms]

# configure Send button
      $cv.icbpanCmdSend configure -command [code $this sendParString]

# configure param list option button
      $cv.parList configure -command [code $this updateCmdLine]

   }


   method updateParms { cmdName } {
      global ::gvar
  
# initialize par stringvariable
      set [$cv.parString cget -variable] ""
      set cmdToSend $cmdName

      set parameters [icbpanCmdQueryParameters $procName $cmdName]
      $cv.parList configure -itemList $parameters -default ""
   }


   method updateCmdLine { parms } {
      global ::gvar

# add selected parameter to the current parameter string
      upvar [$cv.parString cget -variable] var
      set [$cv.parString cget -variable] "$var -${parms}"
   }


   method sendParString { } {
      global ::gvar ::env

      upvar [$cv.parString cget -variable] var

      if { [cequal $cmdToSend ""] } {
         tk_dialog .dialog "Send Command" \
                     "ERROR: No command specified." error 0 OK
      } else {
         set timeout 30000; # 30 sec timeout
         icbpanCmd $env(RTAPENV) $procName $cmdToSend $var $timeout 2
      }   
   }

   public variable procName ""
   private variable cmdToSend ""

}


#
# ___oOo___
