#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanPmt.tcl,v 1.47 2005/07/01 13:25:40 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  13/08/01  created
#

#************************************************************************
#   NAME
#    icbpanPmt - wrapper class for icbpanPmt_uifClass
# 
#   SYNOPSIS
#    icbpanPmt .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICB PMT (photomultiplier) device. The corresponding
#    ibcpanPmt_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, keyword used to identify the
#    device, the current simulation level (N=normal,S=simulate), the current
#    device state, handset mode (H, if active), the current and the cumulated counts.
#    Finally a checkbutton is used to indicate if the corresponding device is 
#    selected for some actions.
#
#   PUBLIC METHODS
#    icbpanPmt  xxx
#       constructor. Calls "initFunction" which initialize the option
#       button with appropriate parameters. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    getKeyw
#       returns the associated setup keyw. + value. Dummy
#    setKeyw val
#       This method is called usually during a reading of a previously 
#       saved setup file. Dummy
#    
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanPmt {
   inherit icbpanPmt_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
   }

# return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


# set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }

# return associated setup keyword. Dummy
   public method getKeyw { } { return "" }

# set setup keyword+value. Dummy
   public method setKeyw { val } { }

   public variable devType ""
   public variable cfgObj  ""

}


#
# ___oOo___
