#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanTabset.tcl,v 1.47 2005/07/01 13:25:41 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  16/07/04  VLTSW20040176
# alongino  14/08/03  created from iwidgets::Tabset
#

#************************************************************************
#   NAME
#    icbpanTabset - tabset mega-widget with multiple tiers
# 
#   SYNOPSIS
#    icbpan::Icbpantabset <args>
# 
#   DESCRIPTION
#    This itcl class is an extension of the iwidgets:Tabset.
#    Options and methods are the same except for the new option -tiers,
#    which allows to specify the maximum number of tiers tabs can be
#    stacked on.
#
#   PUBLIC METHODS
#    see iwidgets::Tabset
#
#   PRIVATE METHOD
#    see iwidgets::Tabset
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   SEE ALSO
#    iwidgets::Tabset
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#
# Default resources.
#
option add *Icbpantabset.width          0            widgetDefault
option add *Icbpantabset.height         0            widgetDefault
option add *Icbpantabset.equalTabs      true         widgetDefault
option add *Icbpantabset.tabPos         s            widgetDefault
option add *Icbpantabset.raiseSelect    false        widgetDefault
option add *Icbpantabset.start          4            widgetDefault
option add *Icbpantabset.margin         5            widgetDefault
option add *Icbpantabset.tabBorders     true         widgetDefault
option add *Icbpantabset.bevelAmount    0            widgetDefault
option add *Icbpantabset.padX           0            widgetDefault
option add *Icbpantabset.padY           0            widgetDefault
option add *Icbpantabset.gap            overlap      widgetDefault
option add *Icbpantabset.angle          20           widgetDefault
option add *Icbpantabset.font           fixed        widgetDefault
option add *Icbpantabset.state          normal       widgetDefault
option add *Icbpantabset.disabledForeground #a3a3a3  widgetDefault
option add *Icbpantabset.foreground     black        widgetDefault
option add *Icbpantabset.background     #d9d9d9      widgetDefault
option add *Icbpantabset.selectForeground black      widgetDefault
option add *Icbpantabset.selectBackground #ececec    widgetDefault

#
# Usual options.
#
itk::usual Icbpantabset {
    keep -backdrop -background -cursor -disabledforeground -font -foreground \
     -selectbackground -selectforeground
}

itcl::class icbpan::Icbpantabset {
    inherit itk::Widget
    
    constructor {args} {} 
    destructor {}
    
    itk_option define -width width Width  0 
    itk_option define -equaltabs equalTabs EqualTabs true 
    itk_option define -height height Height  0
    itk_option define -tabpos tabPos TabPos  s
    itk_option define -raiseselect raiseSelect RaiseSelect false
    itk_option define -start start Start 4
    itk_option define -margin margin Margin 5
    itk_option define -tabborders tabBorders TabBorders true
    itk_option define -bevelamount bevelAmount BevelAmount 0
    itk_option define -padx padX PadX 4
    itk_option define -pady padY PadY 4
    itk_option define -gap gap Gap overlap
    itk_option define -angle angle Angle 20
    itk_option define -font font Font fixed
    itk_option define -state state State normal
    itk_option define \
        -disabledforeground disabledForeground DisabledForeground #a3a3a3
    itk_option define -foreground foreground Foreground black
    itk_option define -background background Background #d9d9d9 
    itk_option define -selectforeground selectForeground Background black
    itk_option define -backdrop backdrop Backdrop white 
    itk_option define -selectbackground selectBackground Foreground #ececec
    itk_option define -command command Command {} 
    itk_option define -tiers tiers Tiers 1
    
    public method configure {args} 
    public method add {args} 
    public method delete {args} 
    public method index {index} 
    public method insert {index args} 
    public method prev {} 
    public method next {} 
    public method select {index} 
    public method tabcget {index args} 
    public method tabconfigure {index args} 
    public method bbox {}
    
    protected method _selectName {tabName} 
    
    private method _createTab {args} 
    private method _deleteTabs {fromTab toTab} 
    private method _index {pathList index select} 
    private method _tabConfigure {args} 
    private method _relayoutTabs {} 
    private method _drawBevelBorder {} 
    private method _calcNextTabOffset {tabName} 
    private method _tabBounds {} 
    private method _recalcCanvasGeom {} 
    private method _canvasReconfigure {width height} 
    private method _startMove {x y} 
    private method _moveTabs {x y} 
    private method _endMove {x y} 
    private method _configRelayout {} 
    
    private variable _width 0          ;# Width of the canvas in screen units
    private variable _height 0         ;# Height of the canvas in screen units
    private variable _selectedTop 0    ;# top edge of tab + a margin
    private variable _deselectedTop 0  ;# top edge of tab + a margin&raiseamt
    private variable _selectedLeft 0   ;# left edge of tab + a margin
    private variable _deselectedLeft 0 ;# left edge of tab + a margin&raiseamt
    private variable _tabs {}          ;# our internal list of tabs
    private variable _currTab -1       ;# numerical index # of selected tab
    private variable _uniqueID 0       ;# used to create unique names
    private variable _cmdStr  {}       ;# holds value of itk_option(-command)
    ;# do not know why I need this!
    private variable _canvasWidth 0    ;# set by canvasReconfigure, is can wid
    private variable _canvasHeight 0   ;# set by canvasReconfigure, is can hgt
    
    private variable _anchorX 0        ;# used by mouse scrolling methods
    private variable _anchorY 0        ;# used by mouse scrolling methods
    
    private variable _margin 0         ;# -margin in screen units
    private variable _start  0         ;# -start in screen units
    private variable _gap overlap      ;# -gap in screen units
    
    private variable _relayout false   ;# flag tripped to tell whether to
                                       ;# relayout tabs after the configure
    private variable _skipRelayout false ;# flag that tells whether to skip
                                       ;# relayouting out the tabs. used by
                                       ;# _endMove.
    private variable _tiers 1             ;# number of teors for tabs
}

#
# Provide a lowercase access method for the Icbpantabset class
#
proc ::icbpan::icbpantabset {pathName args} {
    uplevel ::icbpan::Icbpantabset $pathName $args
}

# ----------------------------------------------------------------------
#                              CONSTRUCTOR
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::constructor {args} {
    global tcl_platform

    #
    # Create the canvas that holds the tabs
    #
    itk_component add canvas {
    canvas $itk_interior.canvas -highlightthickness 0
    } {
    keep -cursor -width -height
    }
    pack $itk_component(canvas) -fill both -expand yes -anchor nw
    
    # ... This gives us a chance to redraw our bevel borders, etc when
    # the size of our canvas changes...
    bind $itk_component(canvas) <Configure> \
        [itcl::code $this _canvasReconfigure %w %h]
    bind $itk_component(canvas) <Map> \
           [itcl::code $this _relayoutTabs]

    
    # ... Allow button 2 scrolling as in label widget.
    if {$tcl_platform(os) != "HP-UX"} {
    bind $itk_component(canvas) <2>               \
        [itcl::code $this _startMove %x %y]
    bind $itk_component(canvas) <B2-Motion>       \
        [itcl::code $this _moveTabs %x %y]
    bind $itk_component(canvas) <ButtonRelease-2> \
        [itcl::code $this _endMove %x %y]
    }
    
    # @@@ 
    # @@@ Is there a better way?
    # @@@

    bind $itk_component(hull) <Tab> [itcl::code $this next]
    bind $itk_component(hull) <Shift-Tab> [itcl::code $this prev]
    
    eval itk_initialize $args
    
    _configRelayout 
    
    _recalcCanvasGeom
    
}

itcl::body icbpan::Icbpantabset::destructor {} {
    foreach tab $_tabs {
    itcl::delete object $tab
    }
}

# ----------------------------------------------------------------------
#                              OPTIONS
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# OPTION -width
#
# Sets the width explicitly for the canvas of the tabset
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::width {
    # bug in iwidgets APR2004 ?
    #if {$itk_option(-width) != {}} {
    #}
    set _width [winfo pixels $itk_interior $itk_option(-width)]
}

# ----------------------------------------------------------------------
# OPTION -equaltabs
#
# If set to true, causes horizontal tabs to be equal in
# in width and vertical tabs to equal in height.
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::equaltabs {
    if {$itk_option(-equaltabs) != {}} {
    set _relayout true
    }
}

# ----------------------------------------------------------------------
# OPTION -height
#
# Sets the height explicitly for the canvas of the tabset
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::height {
    set _height [winfo pixels $itk_interior $itk_option(-height)]
}

# ----------------------------------------------------------------------
# OPTION -tabpos
#
# Sets the tab position of tabs, n, s, e, w
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::tabpos {
    if {$itk_option(-tabpos) != {}} {
    switch $itk_option(-tabpos) {
        n {
        _tabConfigure -invert true -orient horizontal
        }
        s {
        _tabConfigure -invert false -orient horizontal
        }
        w {
        _tabConfigure -invert false -orient vertical
        }
        e {
        _tabConfigure -invert true -orient vertical
        }
        default {
        error "bad anchor position\
            \"$itk_option(-tabpos)\" must be n, s, e, or w"
        }
    }
    }
}

# ----------------------------------------------------------------------
# OPTION -raiseselect
#
# Sets whether to raise selected tabs slightly
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::raiseselect {
    if {$itk_option(-raiseselect) != {}} {
    set _relayout true
    }
}

# ----------------------------------------------------------------------
# OPTION -start
#
# Sets the offset to start of tab set
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::start {
    if {$itk_option(-start) != {}} {
    set _start [winfo pixels $itk_interior $itk_option(-start)]
    set _relayout true
    } else {
    set _start 4
    }
}

# ----------------------------------------------------------------------
# OPTION -margin
#
# Sets the margin used above n tabs, below s tabs, left of e
# tabs, right of w tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::margin {
    if {$itk_option(-margin) != {}} {
    set _margin [winfo pixels $itk_interior $itk_option(-margin)]
    set _relayout true
    } else {
    set _margin 5
    }
}

# ----------------------------------------------------------------------
# OPTION -tabborders
#
# Boolean that specifies whether to draw the borders of
# the unselected tabs (tabs in background)
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::tabborders {
    if {$itk_option(-tabborders) != {}} {
    _tabConfigure -tabborders $itk_option(-tabborders)
    }
}

# ----------------------------------------------------------------------
# OPTION -bevelamount
#
# Specifies pixel size of tab corners. 0 means no corners.
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::bevelamount {
    if {$itk_option(-bevelamount) != {}} {
    _tabConfigure -bevelamount $itk_option(-bevelamount)
    }
}

# ----------------------------------------------------------------------
# OPTION -padx
#
# Sets the padding in each tab to the left and right of label
# I don't convert for fpixels, since Tab does it for me.
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::padx {
    if {$itk_option(-padx) != {}} {
    _tabConfigure -padx $itk_option(-padx)
    }
}

# ----------------------------------------------------------------------
# OPTION -pady
#
# Sets the padding in each tab to the left and right of label
# I don't convert for fpixels, since Tab does it for me.
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::pady {
    if {$itk_option(-pady) != {}} {
    _tabConfigure -pady $itk_option(-pady)
    }
}

# ----------------------------------------------------------------------
# OPTION -gap
#
# Sets the amount of spacing between tabs in pixels
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::gap {
    if {$itk_option(-gap) != {}} {
    if {$itk_option(-gap) != "overlap"} {
        set _gap [winfo pixels $itk_interior $itk_option(-gap)]
    } else {
        set _gap overlap
    }
    set _relayout true 
    } else {
    set _gap overlap
    }
}

# ----------------------------------------------------------------------
# OPTION -angle
#
# Sets the angle of the tab's sides
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::angle {
    if {$itk_option(-angle) != {}} {
    _tabConfigure -angle $itk_option(-angle)
    }
}

# ----------------------------------------------------------------------
# OPTION -font
#
# Sets the font of the tab (SELECTED and UNSELECTED)
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::font {
    if {$itk_option(-font) != {}} {
    _tabConfigure -font $itk_option(-font)
    }
}

# ----------------------------------------------------------------------
# OPTION -state
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::state {
    if {$itk_option(-state) != {}} {
    _tabConfigure -state $itk_option(-state)
    }
}

# ----------------------------------------------------------------------
# OPTION -disabledforeground
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::disabledforeground {
    if {$itk_option(-disabledforeground) != {}} {
    _tabConfigure \
        -disabledforeground $itk_option(-disabledforeground)
    }
}

# ----------------------------------------------------------------------
# OPTION -foreground
#
# Sets the foreground label color of UNSELECTED tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::foreground {
    _tabConfigure -foreground $itk_option(-foreground)
}

# ----------------------------------------------------------------------
# OPTION -background
#
# Sets the background color of UNSELECTED tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::background {
    if {$itk_option(-background) != {}} {
    _tabConfigure -background $itk_option(-background)
    } else {
    _tabConfigure -background \
        [$itk_component(canvas) cget -background]
    }
}

# ----------------------------------------------------------------------
# OPTION -selectforeground
#
# Sets the foreground label color of SELECTED tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::selectforeground {
    _tabConfigure -selectforeground $itk_option(-selectforeground)
}

# ----------------------------------------------------------------------
# OPTION -backdrop
#
# Sets the background color of the Tabset backdrop (behind the tabs)
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::backdrop {
    if {$itk_option(-backdrop) != {}} {
    $itk_component(canvas) configure \
        -background $itk_option(-backdrop)
    }
}

# ----------------------------------------------------------------------
# OPTION -selectbackground
#
# Sets the background color of SELECTED tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::selectbackground {
    if {$itk_option(-selectbackground) != {}} {
    } else {
    #set _selectBackground \
        [$itk_component(canvas) cget -background]
    }
    _tabConfigure -selectbackground $itk_option(-selectbackground)
}

# ----------------------------------------------------------------------
# OPTION -command
#
# The command to invoke when a tab is hit.
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::command {
    if {$itk_option(-command) != {}} {
    set _cmdStr $itk_option(-command)
    }
}

# ----------------------------------------------------------------------
# OPTION -tiers
#
# Specifies the maximum number of tiers for the tabs
# ----------------------------------------------------------------------
itcl::configbody icbpan::Icbpantabset::tiers {
    
    if {$itk_option(-tiers) != {}} {
        set _tiers $itk_option(-tiers)
    }
}

# ----------------------------------------------------------------------
# METHOD: add ?option value...?
#
# Creates a tab and appends it to the list of tabs.
# processes tabconfigure for the tab added.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::add {args} {
    set tabName [eval _createTab $args]
    lappend _tabs $tabName
    
    _relayoutTabs
    
    return $tabName
}

# ----------------------------------------------------------------------
# METHOD: configure ?option? ?value option value...?
#
# Acts as an addendum to the itk::Widget::configure method.
#
# Checks the _relayout flag to see if after configures are done
# we need to relayout the tabs.
#
# _skipRelayout is set in the MB2 scroll methods, to avoid constant
# relayout of tabs while dragging the mouse.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::configure {args} {
    set result [eval itk::Archetype::configure $args]

    _configRelayout

    return $result
}

itcl::body icbpan::Icbpantabset::_configRelayout {} {
    # then relayout tabs if necessary
    if { $_relayout } {
    if { $_skipRelayout } {
    } else {
        _relayoutTabs
    }
    set _relayout false
    }
}

# ----------------------------------------------------------------------
# METHOD: delete index1 ?index2?
#
# Deletes a tab or range of tabs from the tabset
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::delete {args} {
    if { $_tabs == {} } {
    error "can't delete tabs,\
        no tabs in the tabset named $itk_component(hull)"
    }
    
    set len [llength $args]
    switch $len {
    0 {
        error "wrong # args: should be\
            \"$itk_component(hull) delete index1 ?index2?\""
    }
    
    1 {
        set fromTab [index [lindex $args 0]]
        if { $fromTab == -1 } {
        error "bad value for index1:\
            [lindex $args 0] in call to delete"
        }
        set toTab $fromTab
        _deleteTabs $fromTab $toTab
    }
    
    2 {
        set fromTab [index [lindex $args 0]]
        if { $fromTab == -1 } {
        error "bad value for index1:\
            [lindex $args 0] in call to delete"
        }
        set toTab [index [lindex $args 1]]
        
        if { $toTab == -1 } {
        error "bad value for index2:\
            [lindex $args 1] in call to delete"
        }
        _deleteTabs $fromTab $toTab
    }
    
    default {
        error "wrong # args: should be\
            \"$itk_component(hull) delete index1 ?index2?\""
    }
    }
}

# ----------------------------------------------------------------------
# METHOD: index index
#
# Given an index identifier returns the numeric index of the tab
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::index {index} {
    return [_index $_tabs $index $_currTab]
}

# ----------------------------------------------------------------------
# METHOD: insert index ?option value...?
#
# Inserts a tab before a index. The before tab may
# be specified as a label or a tab position.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::insert {index args} {
    if { $_tabs == {} } {
    error "no tab to insert before,\
        tabset '$itk_component(hull)' is empty"
    }
    
    # get the tab
    set tab [index $index]
    
    # catch bad value for before tab.
    if { $tab < 0 || $tab >= [llength $_tabs] } {
    error "bad value $tab for index:\
        should be between 0 and [expr {[llength $_tabs] - 1}]"
    }
    
    # create the new tab and get its name...
    set tabName [eval _createTab $args]
    
    # grab the name of the tab currently selected. (to keep in sync)
    set currTabName [lindex $_tabs $_currTab]
    
    # insert tabName before $tab
    set _tabs [linsert $_tabs $tab $tabName]
    
    # keep the _currTab in sync with the insert.
    set _currTab [lsearch -exact $_tabs $currTabName]
    
    _relayoutTabs
    
    return $tabName
}

# ----------------------------------------------------------------------
# METHOD: prev
#
# Selects the prev tab. Wraps at first back to last tab.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::prev {} {
    if { $_tabs == {} } {
    error "can't goto previous tab,\
        no tabs in the tabset: $itk_component(hull)"
    }
    
    # bump to the previous tab and wrap if necessary
    set prev [expr {$_currTab - 1}]
    if { $prev < 0 } {
    set prev [expr {[llength $_tabs] - 1}]
    }
    
    select $prev
    
}

# ----------------------------------------------------------------------
# METHOD: next
#
# Selects the next tab. Wraps at last back to first tab.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::next {} {
    if { $_tabs == {} } {
    error "can't goto next tab,\
        no tabs in the tabset: $itk_component(hull)"
    }
    
    # bump to the next tab and wrap if necessary
    set next [expr {$_currTab + 1}]
    if { $next >= [llength $_tabs] } {
    set next 0
    }
    
    select $next
}

# ----------------------------------------------------------------------
# METHOD: select index
#
# Select a tab by index
#
# Lowers the last _currTab if it existed.
# Then raises the new one if it exists.
#
# Returns numeric index of selection, -1 if failed.
# -------------------------------------------------------------
itcl::body icbpan::Icbpantabset::select {index} {
    if { $_tabs == {} } {
    error "can't activate a tab,\
        no tabs in the tabset: $itk_component(hull)"
    }
    
    # if there is not current selection just ignore trying this selection
    if { $index == "select" && $_currTab == -1 } {
    return -1
    }
    
    # is selection request in range ? 
    set reqTab [index $index]
    if { $reqTab == -1 } {
    error "bad value $index for index:\
        should be from 0 to [expr {[llength $_tabs] - 1}]"
    }
    
    # If already selected then ignore and return...
    if { $reqTab == $_currTab } {
    return $reqTab
    }
    
    # ---- Deselect
    if { $_currTab != -1 } {
    set currTabName [lindex $_tabs $_currTab]
    $currTabName deselect
    
    # handle different orientations and multiple tiers
    set currTabTop [$currTabName cget -top]
    if { $itk_option(-tabpos) == "n" || $itk_option(-tabpos) == "s"} {
        $currTabName configure -top [expr {$currTabTop + $_deselectedTop - $_selectedTop}]
    } else {
        $currTabName configure -left [expr {$currTabTop + $_deselectedLeft - $_selectedLeft}]
    }
    }
    
    # get the stacking order correct...
    foreach tab $_tabs {
    $tab lower
    }
    
    # set this now so that the -command cmd can do an 'index select'
    # to operate on this tab.
    set _currTab $reqTab
    
    # ---- Select
    set reqTabName [lindex $_tabs $reqTab]
    $reqTabName select
    # handle different orientations and multiple tiers
    set reqTabTop [$reqTabName cget -top]
    if { $itk_option(-tabpos) == "n" || $itk_option(-tabpos) == "s"} {
        $reqTabName configure -top [expr {$reqTabTop + $_selectedTop - $_deselectedTop}]
    } else {
        $reqTabName configure -top [expr {$reqTabTop + $_selectedLeft - $_deselectedLeft}]
        $reqTabName configure -left $_selectedLeft
    }
    
    set _currTab $reqTab
    
    # invoke any user command string, appended with tab index number
    if { $_cmdStr != {} } {
    set newCmd $_cmdStr
    eval [lappend newCmd $reqTab]
    }
    
    return $reqTab
}

# ----------------------------------------------------------------------
# METHOD: tabcget index ?option? 
#
# Returns the value for the option setting of the tab at index $index.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::tabcget {index args} {
    return [lindex [eval tabconfigure $index $args] 2]
}

# ----------------------------------------------------------------------
# METHOD: tabconfigure index ?option? ?value option value?
#
# tabconfigure index : returns configuration list
# tabconfigure index -option : returns option values
# tabconfigure index ?option value option value ...? sets options
#   and returns empty string.
#
# Performs configure on a given tab denoted by index.
#
# Index may be a tab number or a pattern matching the label
# associated with a tab.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::tabconfigure {index args} {
    # convert index to numeric
    set tab [index $index]
    
    if { $tab == -1 } {
    error "bad index value:\
        $index for $itk_component(hull) tabconfigure"
    }
    
    set tabName [lindex $_tabs $tab]
    
    set len [llength $args]
    switch $len {
    0 {
        return [eval $tabName configure]
    }
    1 {
        return [eval $tabName configure $args]
    }
    default {
        eval $tabName configure $args
        _relayoutTabs
        select select
    }
    }
    return ""
}

# ----------------------------------------------------------------------
# METHOD: bbox
# 
# calculates the bounding box that will completely enclose 
# all the tabs.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::bbox {} {
    return [_tabBounds]
}

# ----------------------------------------------------------------------
# PROTECTED METHOD: _selectName
#
# internal method to allow selection by internal tab name 
# rather than index. This is used by the bind methods
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_selectName {tabName} {
    # if the tab is disabled, then ignore this selection...
    if { [$tabName cget -state] == "disabled" } {
    return
    }
    
    set tab [lsearch -exact $_tabs $tabName]
    select $tab
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _createTab
#
# Creates a tab, using unique tab naming, propagates background
# and keeps unique id up to date.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_createTab {args} {
    #
    # create an internal name for the tab: tab0, tab1, etc.
    # these are one-up numbers they do not 
    # correspond to the position the tab is located in.
    #
    set tabName $this-tab$_uniqueID
    
    switch $itk_option(-tabpos) {
    n {
        set invert true
        set orient horizontal
        set x 0
        set y [expr {$_margin + 1}]
    }
    s {
        set invert false
        set orient horizontal
        set x 0
        set y 0
    }
    w {
        set invert false
        set orient vertical
        set x 0 
        set y 0
    }
    e {
        set invert true
        set orient vertical
        set x [expr {$_margin + 1}]
        set y 0
    }
    default {
        error "bad anchor position\
            \"$itk_option(-tabpos)\" must be n, s, e, or w"
    }
    }
    
    eval iwidgets::Tab $tabName $itk_component(canvas) \
        -left             $x \
        -top              $y \
        -font             [list $itk_option(-font)] \
        -background       $itk_option(-background) \
        -foreground       $itk_option(-foreground) \
        -selectforeground $itk_option(-selectforeground) \
        -disabledforeground $itk_option(-disabledforeground) \
        -selectbackground $itk_option(-selectbackground) \
        -angle            $itk_option(-angle) \
        -padx             $itk_option(-padx) \
        -pady             $itk_option(-pady) \
        -bevelamount      $itk_option(-bevelamount) \
        -state            $itk_option(-state) \
        -tabborders       $itk_option(-tabborders) \
        -invert           $invert \
        -orient           $orient \
        $args
    
    $tabName lower
    
    $itk_component(canvas) \
        bind $tabName <Button-1> [itcl::code $this _selectName $tabName]
    
    incr _uniqueID
    
    return $tabName
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _deleteTabs
#
# Deletes tabs from $fromTab to $toTab.
#
# Operates in two passes, destroys all the widgets
# Then removes the pathName from the tab list
#
# Also keeps the current selection in bounds.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_deleteTabs {fromTab toTab} {
    for { set tab $fromTab } { $tab <= $toTab } { incr tab } {
    set tabName [lindex $_tabs $tab]
    
    # unbind Button-1 from this window name
    $itk_component(canvas) bind $tabName <Button-1> {}
    
    # Destroy the Tab class...
    itcl::delete object $tabName 
    }
    
    # physically remove the tab
    set _tabs [lreplace $_tabs $fromTab $toTab]
    
    # If we deleted a selected tab set our selection to none
    if { $_currTab >= $fromTab && $_currTab <= $toTab } {
    set _currTab -1
    _drawBevelBorder
    }
    
    # make sure _currTab stays in sync with new numbering...
    if { $_tabs == {} } {
    # if deleted only remaining tab,
    # reset current tab to undefined
    set _currTab -1
    
    # or if the current tab was the last tab, it needs come back
    } elseif { $_currTab >= [llength $_tabs] } {
    incr _currTab -1
    if { $_currTab < 0 } {
        # but only to zero
        set _currTab 0
    }
    }
    
    _relayoutTabs
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _index
#
# pathList : list of path names to search thru if index is a label
# index    : either number, 'select', 'end', or pattern
# select   : current selection
#
# _index takes takes the value $index converts it to
# a numeric identifier. If the value is not already
# an integer it looks it up in the $pathList array.
# If it fails it returns -1
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_index {pathList index select} {
    switch $index {
    select {
        set number $select
    }
    end {
        set number [expr {[llength $pathList] -1}]
    }
    default {
        # is it an number already?
        if { [regexp {^[0-9]+$} $index] } {
        set number $index
        if { $number < 0 || $number >= [llength $pathList] } {
            set number -1
        }
        
        # otherwise it is a label
        } else {
        # look thru the pathList of pathNames and
        # get each label and compare with index.
        # if we get a match then set number to postion in $pathList
        # and break out.
        # otherwise number is still -1
        set i 0
        set number -1
        foreach pathName $pathList {
            set label [$pathName cget -label]
            if { $label == $index } {
            set number $i
            break
            }
            incr i
        }
        }
    }
    }
    
    return $number
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _tabConfigure
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_tabConfigure {args} {
    foreach tab $_tabs {
    eval $tab configure $args
    }
    
    set _relayout true
    
    if { $_tabs != {} } {
    select select
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _relayoutTabs
# 
# relays out the tabs with correct spacing...
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_relayoutTabs {} {
    if { [llength $_tabs] == 0 || ![winfo viewable $itk_component(hull)]} {
    return
    }
    
    # get the max width for fixed width tabs...
    set maxWidth 0
    foreach tab $_tabs {
    set width [$tab labelwidth]
    if { $width > $maxWidth } {
        set maxWidth $width
    }
    }
    
    # get the max height for fixed height tabs...
    set maxHeight 0
    foreach tab $_tabs {
    set height [$tab labelheight]
    if { $height > $maxHeight } {
        set maxHeight $height
    }
    }
    
    # get curr tab's name
    set currTabName [lindex $_tabs $_currTab]
    
    # Start with our margin offset in pixels...
    set tabStart $_start
    
    if { $itk_option(-raiseselect) } {
    set raiseAmt 2
    } else {
    set raiseAmt 0
    }

    set nTiers 1    
    #
    # Depending on the tab layout: n, s, e, or w place the tabs
    # according to orientation, raise, margins, etc.
    #
    switch $itk_option(-tabpos) {
    n {
        set _selectedTop [expr {$_margin + 1}]
        set _deselectedTop [expr {$_selectedTop + $raiseAmt}]
        
        if { $itk_option(-equaltabs) } {
        set tabWidth $maxWidth
        } else {
        set tabWidth 0
        }
        
        # Implement multiple tiers. Only for north tabs for the time being
        # First calculate how many tiers are needed
        set i 1
        foreach tab $_tabs {
            set tabStartNext [expr {$tabStart + [_calcNextTabOffset $tab]}]
            if { $tabStartNext > $_width } {
                if { $nTiers < $_tiers } {
                    set tabStart $_start
                    set tabStartNext [expr {$tabStart + [_calcNextTabOffset $tab]}]
                    incr nTiers
		}
	    }
            set tabTier($i) $nTiers
            set tabBegin($i) $tabStart
            set tabStart $tabStartNext
            incr i
	}
        # now configure tabs
        set i 1
        foreach tab $_tabs {
            set tabStart $tabBegin($i)
            set topSet $_deselectedTop
	    if { $tab == $currTabName } { set topSet $_selectedTop }
            set tabHeight [expr {$maxHeight + 2 * $itk_option(-pady) + 3}]
            set tabTop [expr {$topSet + ($nTiers - $tabTier($i)) * $tabHeight}]
            $tab configure -left $tabStart -top $tabTop \
                           -height $maxHeight -width $tabWidth -anchor c
            incr i
        }
        
    }
    s {
        set _selectedTop 0
        set _deselectedTop [expr {$_selectedTop - $raiseAmt}]
        
        if { $itk_option(-equaltabs) } {
        set tabWidth $maxWidth
        } else {
        set tabWidth 0
        }
        
        foreach tab $_tabs {
        if { $tab == $currTabName } {
            $tab configure -left $tabStart -top $_selectedTop \
                -height $maxHeight -width $tabWidth -anchor c
        } else {
            $tab configure -left $tabStart -top $_deselectedTop \
                -height $maxHeight -width $tabWidth -anchor c
        }
        set tabStart [expr {$tabStart + [_calcNextTabOffset $tab]}]
        }
        
    }
    w {
        set _selectedLeft [expr {$_margin + 1}]
        set _deselectedLeft [expr {$_selectedLeft + $raiseAmt}]
        
        if { $itk_option(-equaltabs) } {
        set tabHeight $maxHeight
        } else {
        set tabHeight 0
        }
        
        foreach tab $_tabs {
        # selected
        if { $tab == $currTabName } {
            $tab configure -top $tabStart -left $_selectedLeft \
                -height $tabHeight -width $maxWidth -anchor e
            # deselected
        } else {
            $tab configure -top $tabStart -left $_deselectedLeft \
                -height $tabHeight -width $maxWidth -anchor e
        }
        set tabStart [expr {$tabStart + [_calcNextTabOffset $tab]}]
        }
        
    }
    e {
        set _selectedLeft 0
        set _deselectedLeft [expr {$_selectedLeft - $raiseAmt}]
        
        if { $itk_option(-equaltabs) } {
        set tabHeight $maxHeight
        } else {
        set tabHeight 0
        }
        
        foreach tab $_tabs {
        # selected
        if { $tab == $currTabName } {
            $tab configure -top $tabStart -left $_selectedLeft \
                -height $tabHeight -width $maxWidth -anchor w
            # deselected
        } else {
            $tab configure -top $tabStart -left $_deselectedLeft \
                -height $tabHeight -width $maxWidth -anchor w
        }
        set tabStart [expr {$tabStart + [_calcNextTabOffset $tab]}]
        }
        
    }
    default {
        error "bad anchor position\
            \"$itk_option(-tabpos)\" must be n, s, e, or w"
    }
    }
    
    # put border on & calc our new canvas size...
    _drawBevelBorder
    _recalcCanvasGeom
    
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _drawBevelBorder
# 
# draws the bevel border along tab edge (below selected tab)
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_drawBevelBorder {} {
    $itk_component(canvas) delete bevelBorder
    
    switch $itk_option(-tabpos) {
    n {
        $itk_component(canvas) create line \
            0 [expr {$_canvasHeight - 1}] \
            $_canvasWidth [expr {$_canvasHeight - 1}] \
            -fill [iwidgets::colors::topShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
        $itk_component(canvas) create line \
            0 $_canvasHeight \
            $_canvasWidth $_canvasHeight \
            -fill [iwidgets::colors::topShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
    }
    s {
        $itk_component(canvas) create line \
            0 0 \
            $_canvasWidth 0 \
            -fill [iwidgets::colors::bottomShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
        $itk_component(canvas) create line \
            0 1 \
            $_canvasWidth 1 \
            -fill black \
            -tags bevelBorder
    }
    w {
        $itk_component(canvas) create line \
            $_canvasWidth 0 \
            $_canvasWidth [expr {$_canvasHeight - 1}] \
            -fill [iwidgets::colors::topShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
        $itk_component(canvas) create line \
            [expr {$_canvasWidth - 1}] 0 \
            [expr {$_canvasWidth - 1}] [expr {$_canvasHeight - 1}] \
            -fill [iwidgets::colors::topShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
        
    }
    e {
        $itk_component(canvas) create line \
            0 0 \
            0 [expr {$_canvasHeight - 1}] \
            -fill black \
            -tags bevelBorder
        $itk_component(canvas) create line \
            1 0 \
            1 [expr {$_canvasHeight - 1}] \
            -fill [iwidgets::colors::bottomShadow $itk_option(-selectbackground)] \
            -tags bevelBorder
        
    }
    }
    
    $itk_component(canvas) raise bevelBorder
    if { $_currTab != -1 } {
    set currTabName [lindex $_tabs $_currTab]
    $currTabName raise
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _calcNextTabOffset
# 
# given $tabName, determines the offset in pixels to place
# the next tab's start edge at.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_calcNextTabOffset {tabName} {
    if { $_gap == "overlap" } {
    return [$tabName offset]
    } else {
    return [expr {[$tabName majordim] + $_gap}]
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _tabBounds
# 
# calculates the bounding box that will completely enclose 
# all the tabs.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_tabBounds {} {
    set bbox { 100000 100000 -10000 -10000 }
    foreach tab $_tabs {
    set tabBBox [$tab bbox]
    # if this left is less use it
    if { [lindex $tabBBox 0] < [lindex $bbox 0] } {
        set bbox [lreplace $bbox 0 0 [lindex $tabBBox 0]]
    }
    # if this top is greater use it
    if { [lindex $tabBBox 1] < [lindex $bbox 1] } {
        set bbox [lreplace $bbox 1 1 [lindex $tabBBox 1]]
    }
    # if this right is less use it
    if { [lindex $tabBBox 2] > [lindex $bbox 2] } {
        set bbox [lreplace $bbox 2 2 [lindex $tabBBox 2]]
    }
    # if this bottom is greater use it
    if { [lindex $tabBBox 3] > [lindex $bbox 3] } {
        set bbox [lreplace $bbox 3 3 [lindex $tabBBox 3]]
    }
    
    }
    return $bbox
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _recalcCanvasGeom
# 
# Based on size of tabs, recalculates the canvas geometry that
# will hold the tabs.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_recalcCanvasGeom {} {
    if { [llength $_tabs] == 0 } {
    return
    }
    
    set bbox [_tabBounds]
    
    set width [lindex [_tabBounds] 2]
    set height [lindex [_tabBounds] 3]
    
    # now we have the dimensions of all the tabs in the canvas.
    
    
    switch $itk_option(-tabpos) {
    n {
        # height already includes margin
        $itk_component(canvas) configure \
            -width $width \
            -height $height
    }
    s {
        $itk_component(canvas) configure \
            -width $width \
            -height [expr {$height + $_margin}]
    }
    w {
        # width already includes margin
        $itk_component(canvas) configure \
            -width $width \
            -height [expr {$height + 1}]
    }
    e {
        $itk_component(canvas) configure \
            -width [expr {$width + $_margin}] \
            -height [expr {$height + 1}]
    }
    default {
    }
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _canvasReconfigure
# 
# Bound to the reconfigure notify event of a canvas, this 
# method resets canvas's correct width (since we are fill x)
# and redraws the beveled edge border.
# will hold the tabs.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_canvasReconfigure {width height} {
    set _canvasWidth $width
    set _canvasHeight $height
    
    if { [llength $_tabs] > 0 } {
    _drawBevelBorder
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _startMove
# 
# This method is bound to the MB2 down in the canvas area of the
# tab set. This starts animated scrolling of the tabs along their
# major axis.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_startMove {x y} {
    if { $itk_option(-tabpos) == "n" || $itk_option(-tabpos) == "s" } {
    set _anchorX $x
    } else {
    set _anchorY $y
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _moveTabs
# 
# This method is bound to the MB2 motion in the canvas area of the
# tab set. This causes the tabset to move with the mouse.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_moveTabs {x y} {
    if { $itk_option(-tabpos) == "n" || $itk_option(-tabpos) == "s" } {
    set startX [expr {$_start + $x - $_anchorX}]
    foreach tab $_tabs {
        $tab configure -left $startX 
        set startX [expr {$startX + [_calcNextTabOffset $tab]}]
    }
    } else {
    set startY [expr {$_start + $y - $_anchorY}]
    foreach tab $_tabs {
        $tab configure -top $startY 
        set startY [expr {$startY + [_calcNextTabOffset $tab]}]
    }
    }
}

# ----------------------------------------------------------------------
# PRIVATE METHOD: _endMove
# 
# This method is bound to the MB2 release in the canvas area of the
# tab set. This causes the tabset to end moving tabs.
# ----------------------------------------------------------------------
itcl::body icbpan::Icbpantabset::_endMove {x y} {
    if { $itk_option(-tabpos) == "n" || $itk_option(-tabpos) == "s" } {
    set startX [expr {$_start + $x - $_anchorX}]
    set _skipRelayout true
    configure -start $startX
    set _skipRelayout false
    } else {
    set startY [expr {$_start + $y - $_anchorY}]
    set _skipRelayout true
    configure -start $startY
    set _skipRelayout false
    }
}


#
# ___oOo___
