#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerClass.tcl,v 1.65 2003/12/17 19:26:25 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 18.03.1995 created
# S.Sandrock 15.10.1995 revised, dependent on vcc
# imunoz     08.04.1995 ported to itcl2.0
#

#************************************************************************
#   NAME
#
#	InducerClass - Abstract class for inducer widgets
# 
#   SYNOPSIS
#
#	inherit InducerClass
# 
#   DESCRIPTION
#
#	InducerClass is used to inherit special features for the 
#	implementation of objects to be used in "Inducer Panels".
#	All OPTIONS and METHODS of this class are then also available
#	in the objects that inherit from this class.
#
#   INHERITANCE
#	VccDebug
#
#   PUBLIC OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following options are publicly available:
#
#	-wHelparea	Object-name of short-help.
#			By default the panel-editor defined area is used.
#			(Note: this is a direct reference which may change!)
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	The following methods are publicly available:
#
#	debugPrint ?<string>?
#		This can be called from the body of a method or config-body
#		of a public variable to print debugging information.
#		If the option `debugFlag' is enabled, then a line in the form:
#
#			(<object-name>) <method-name> <args>: <string>
#
#		is printed to standard-output, where "<object-name>" is the
#		name of the object and "<method-name> <args>" is the name of
#		the method from which the call was made and it's arguments. 
#		If the call was not made from a method, this field is empty.
#		An optional "<string>" can also be printed.
#		NOTE: debugPrint can not be called from the body of a proc!
#
#	More methods: see VccDebug(n)
#
#   PROCS
#
#   EXAMPLES
#
#	itcl_class AcroxSignal {
#	    inherit FrameWidget InducerClass
#
#	    #*# constructor
#	    constructor { config } {
#		...
#	    }
#	    method abc {
#		debugPrint "..."
#	    }
#		...
#	}
#
#
#   SEE ALSO
#	VccDebug(n)
#
#   BUGS
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: InducerClass

class InducerClass {
    inherit UifWidget VccDebug
    
    #*# constructor
    constructor { args } {
	global shArea
	configure -helparea $shArea

	eval itk_initialize $args
    }
    
    
    #--------------------------------------------------------------------
    # Short Help
    #--------------------------------------------------------------------

    #::global ::shArea		       ;# default: as defined by panel-editor
    #public variable wHelparea $shArea  ;# object-name of short-help
    
    protected method setShortHelp {widget text} { }
    #--------------------------------------------------------------------
    # Debugging Options
    #--------------------------------------------------------------------
    # (inherited from VccDebug)
}

body InducerClass::setShortHelp {widget text} {
 	UifShortHelp::add $itk_option(-helparea) $widget $text
}

#___oOo___
