#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerEvhLib.tcl,v 1.65 2003/12/17 19:26:25 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.03.1995 created
# imunoz     20.04.1996 icl2.0 port
#

#************************************************************************
#   NAME
#	inducerEvhLib,
#	inducerEvhSelectEnv, inducerEvhSelectDev, 
#	inducerEvhMode, inducerEvhModeCheck, inducerEvhResetDevice,
#	inducerEvhQuit, inducerEvhLoadPanel, inducerEvhSavePanel,
#	inducerEvhSetupPanel, inducerEvhEnableDebug,
#	inducerEvhVxWorks, inducerEvhDriver,
#	inducerEvhInfo, inducerEvhHelpOnDevice, 
#	inducerEvhExtendedHelp, inducerEvhPersonalNotes
#	- Inducer Event-Handlers
# 
#   SYNOPSIS
#	inducerEvhSelectEnv	Environment
#	inducerEvhSelectDev	Device
#	inducerEvhMode		TEST | EXCLUSIVE | SHARED | READONLY
#	inducerEvhModeCheck	Modes
#	inducerEvhResetDevice
#	inducerEvhQuit
#	inducerEvhLoadPanel
#	inducerEvhSavePanel
#	inducerEvhSetupPanel
#	inducerEvhEnableDebug
#	inducerEvhVxWorks	?Function?
#	inducerEvhDriver	?Function?
#	inducerEvhInfo
#	inducerEvhHelpOnDevice
#	inducerEvhExtendedHelp
#	inducerEvhPersonalNotes
#
#   DESCRIPTION
#	These event-handler are called on user-request, i.e. when ...
#
#	inducerEvhSelectEnv	- an environment is selected.
#	inducerEvhSelectDev	- a device is selected
#	inducerEvhMode		- the `Mode' option-button is pressed
#	inducerEvhModeCheck	- (internal use to check if one of the modes
#				  in `Modes' is active. Pops up dialog box
#				  and returns 1 if so, otherwise returns 0.)
#	inducerEvhResetDevice	- the `Reset Device' button is pressed
#	inducerEvhQuit		- `Quit' is selected in the File menu
#	inducerEvhLoadPanel	- `Load Panel...' is selected in File menu
#	inducerEvhSavePanel	- `Save Panel...' is selected in File menu
#	inducerEvhSetupPanel	- `Setup...' is selected in the Options menu
#	inducerEvhEnableDebug	- `Enable Debug' is selected in Options menu
#	inducerEvhVxWorks	- an entry in the `VxWorks' menu is selected
#				  `Function' is the function-name to be called.
#	inducerEvhDriver	- an entry in the `Driver' menu is selected
#				  `Function' is the function-name to be called.
#	inducerEvhInfo		- `Info...' is selected in the Help menu
#	inducerEvhExtendedHelp	- `Extended Help...' is selected in Help menu
#	inducerEvhPersonalNotes	- `Personal Notes...' is selected in Help menu
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#	None.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

global inducer(vEnv)
global inducer(vDev)
global inducer(vDevFilter)
global inducer(vMode)
global inducer(fOnline)
global inducer(vTickRate)


#
# Event-Handler for Buttons:
#

proc inducerEvhSelectEnv { Environment } {
    global inducer

    if {$Environment == ""} {return ""}

    # drop current device selection and menu:
    inducerEvhSelectDev ""
    set inducer(vDev) ""

    # check if server process on Environment is OK:
    if {[catch {InducerLine::Send $Environment VERSION} msg]} {
	inducerDialog "Environment is not responding:\n$msg"
	set inducer(vMessage) "Connection to $Environment refused."
	return ""
    }
    set inducer(vServerVersion) $msg

    set inducer(vMessage) \
	    "Querying $inducer(vDevFilter) devices in $Environment ..."

    # save selected value in global variable of Environment-entry:
    set inducer(vEnv) $Environment

    # get the list of devices and make a menu of it:
    set devlist [inducerGetDevices $Environment $inducer(vDevFilter)]
    $inducer(wSelDev) MakeMenu $devlist

    # tell Device object it's Environment:
    $inducer(wDev) configure -vEnv $Environment

    set inducer(vMessage) \
	 "Got [llength $devlist] $inducer(vDevFilter) devices in $Environment."

    # get sysClkRate (ticks per second) to display it:
    set inducer(vTickRate) [inducerGetTickRate $Environment]

    # get the list of tools for the devices and make the `Driver' menu of it:
    set tools [inducerGetTools $devlist]
    $inducer(wSelDrv) MakeMenu $tools "" $inducer(wMenuDrv)

    # return validated selection as required by vccSelector:
    return $Environment
}


proc inducerEvhSelectDev { Device } {
    global inducer

    # make device-specific clean-up:
    set inducer(vMessage) "Disconnecting device $inducer(vDev) ..."
    if {$inducer(vDev) != ""} {$inducer(wDev) ExitDevice}

    set inducer(fOnline) 0

    set inducer(vMessage) "Device $inducer(vDev) disconnected."

    # return now if no new device shall be selected:
    if {$Device == ""} {return ""}

    # save selected value in global variable of Device-entry:
    set inducer(vDev) $Device

    set inducer(vMessage) "Connecting to device $Device ..."

    # query the ioctl-commands for the selected device (i.e. driver):
    set inducer(vCommands) [inducerGetIoctlCommands $Device]

    # tell Device object it's Device-name and openMode:
    $inducer(wDev) configure -vDev $Device -vMode $inducer(vMode)
    # call `InitDevice' to make device-specific preparations:
    if {[$inducer(wDev) InitDevice] != ""} {
	set inducer(vMessage) "Connection to device $Device refused."
	return ""
    }

    # call `GetDeviceId' to display it:
    set inducer(vDeviceId) [$inducer(wDev) GetDeviceId]

    set inducer(vMessage) "$Device selected in $inducer(vMode) mode."

    set inducer(fOnline) 1

    # return validated selection as required by vccSelector:
    return $Device
}


proc inducerEvhMode { Mode } {
    global inducer

    set inducer(vMode) $Mode
    if {$inducer(vDev) != ""} {
	inducerEvhSelectDev $inducer(vDev)
    }
}


proc inducerEvhModeCheck { Modes } {
    global inducer
    if {[lsearch $Modes $inducer(vMode)] >= 0} {
	inducerDialog "This operation is not allowed in $inducer(vMode) mode!"
	return 1
    } else {
	return 0
    }
}


proc inducerEvhResetDevice { } {
    global inducer

    # don't allow this in READONLY mode:
    if {[inducerEvhModeCheck READONLY]} return

    # ask for confirmation:
    if {[tk_dialog .dialog "[wm title .]: Confirm" \
	    "Do you really want to reset the device?" \
	    question 1 Continue Cancel]} {
						;# No  ==> do nothing
    } else {  
	$inducer(wDev) ResetDevice		;# Yes ==> do reset
    }
}


proc inducerEvhOnline { } {
    global inducer

    inducerEvhSelectDev $inducer(vDev)
}

proc inducerEvhOffline { } {
    global inducer

    inducerEvhSelectDev ""
}


proc inducerEvhLogo { } {
    inducerDialog "WARNING:\n\
	    \nInducer Test Panels operate at driver level.\n\
	    \nAll your inputs are usually immediately forwarded\
	    to the device, without further checks!\n\
	    \nUSE FOR TEST AND MAINTENANCE ONLY!"
}


#
# Event-Handler for `File' Menu:
#

proc inducerEvhQuit {  } {
    global inducer
    catch {$inducer(wDev) ExitDevice}	;# close any opened device channel
    exit 0				;# exit application
    # generic Quit procedure (panClosePanel) is not included here!
}


proc inducerEvhLoadPanel {  } {
    global inducer

    # open a file-browser to select a file, abort when empty:
    set filename [panSelectFile . *.ipd]
    if {$filename == ""} return

    # open and read file contents:
    set f [open $filename r]
    set versionString [gets $f]
    set data [read $f]
    close $f

    # check if the version saved in the file is the same as the current:
    set devVer [$inducer(wDev) GetVersion]
    if {$versionString != $devVer} {
	inducerDialog "WARNING: This file was saved with another tool version.\
		It might be incompatible!"
    }

    # disconnect the panel from the hardware device:
    $inducer(wDev) Disconnect

    # execute the device-specific load method:
    $inducer(wDev) LoadDevice $data

    # ask for confirmation (for READONLY simply reselect):
    if {$inducer(vMode) == "READONLY" || \
	    [tk_dialog .dialog "[wm title .]: Confirm" \
	    "Do you want to send the new panel-contents to the Device?" \
	    question 1 Continue Cancel]} {
	inducerEvhSelectDev $inducer(vDev)	;# No  ==> reselect the device
    } else {  
	$inducer(wDev) Resume			;# Yes ==> resume connection
    }
}


proc inducerEvhSavePanel {  } {
    global inducer

    # open a file-browser to select a file, abort when empty:
    set filename [panSelectFile . *.ipd]
    if {$filename == ""} return

    # disconnect the panel from the hardware device:
    $inducer(wDev) Disconnect

    # execute the device-specific method:
    set data [$inducer(wDev) SaveDevice]

    # open and write file contents:
    set f [open $filename w]
    puts $f [$inducer(wDev) GetVersion]
    puts $f $data
    close $f

    inducerDialog "Panel saved in \"$filename\"."	;# done message
    inducerEvhSelectDev $inducer(vDev)			;# reselect the device
}


proc inducerEvhBeginMacro { } {
    global inducer

    $inducer(wDev) Macro begin
    set inducer(vMessage) "Recording Macro ..."
    $inducer(wEndMacro) configure -state normal
}

proc inducerEvhEndMacro { } {
    global inducer

    $inducer(wDev) Macro end
    set inducer(vMessage) "Macro recorded."
    $inducer(wEndMacro) configure -state disabled
    $inducer(wExecMacro) configure -state normal
    $inducer(wSaveMacro) configure -state normal
}

proc inducerEvhExecMacro { } {
    global inducer

    set inducer(vMessage) "Executing Macro ..."
    update idletasks
    vccWinBusy {
	set nLines [$inducer(wDev) Macro exec inducerEvhExecMacroFlash]
    }
    set inducer(vMessage) "Macro executed ($nLines lines)."
}
proc inducerEvhExecMacroFlash { args } {
    global inducer
    # !! Note: The new uifLibrary 1.39 doesn't provide any more
    # the flash method for buttons (at least it was present for
    # uifActionButtons). Use getComponent path instead to get acces to the
    # button and makeit flash.
    set b [$inducer(wExecMacro) getComponentPath uifActionButton]
    $b flash ;# flash action-button
}

proc inducerEvhLoadMacro { } {
    global inducer
    set inducer(vMessage) "Macro loaded from \"[$inducer(wDev) Macro load]\"."
    $inducer(wEndMacro) configure -state disabled
    $inducer(wExecMacro) configure -state normal
    $inducer(wSaveMacro) configure -state normal
}

proc inducerEvhSaveMacro { } {
    global inducer
    set inducer(vMessage) "Macro saved to \"[$inducer(wDev) Macro save]\"."
}


#
# Event-Handler for `Options' Menu:
#

proc inducerEvhLoadLccConfig {  } {
    global inducer

    # get LCC info for this device, check for empty:
    if {[$inducer(wDev) GetSignalConfig] != ""} {
	$inducer(wDev) Disconnect  ;# disconnect the panel from the HW device
	$inducer(wDev) LoadFromLCC ;# Execute device specific LoadFromLCC

	# ask for confirmation (for READONLY simply reselect):
	if {$inducer(vMode) == "READONLY" || \
		[tk_dialog .dialog "[wm title .]: Confirm" \
		"Data Loaded From LCC.\
		\nDo you want to send the new panel-contents to the Device?" \
		question 1 Continue Cancel]} {
	    inducerEvhSelectDev $inducer(vDev)	;# No  ==> reselect the device
	} else {
	    $inducer(wDev) Resume		;# Yes ==> resume connection
	}
	return ""
    }
    inducerDialog "No LCC Info Present"
}

proc inducerEvhMacroWithDelay { } {
    global inducer

    $inducer(wDev) Macro keepDelay $inducer(fMacroWithDelay)
    set what [expr {$inducer(fMacroWithDelay) ? "with" : "without"}]
    set inducer(vMessage) "Macros to be exec'ed $what recorded delay."
}

#!! Not used anymore:
proc inducerEvhSetupPanel { } {
    global inducer

    inducerTBD
    return

    # temporary solution:
    catch {destroy .setup}
    toplevel .setup
    wm title .setup "[wm title .] Setup"
    inducerOption_uifClass .setup.1
    pack .setup.1
    .setup.1 init
    .setup.1 draw
}

proc inducerEvhEnableDebug { } {
    global inducer
    VccDebug::Control $inducer(fDebug)
}


#
# Event-Handler for `VxWorks' and `Driver' Menus:
#

proc inducerEvhVxWorks { {Function ""} } {
    global inducer
    inducerExec $inducer(vEnv) $Function
}

proc inducerEvhDriver { {Function ""} } {
    global inducer
    set arg [inducerGetToolArg $inducer(vDev) $Function]
    inducerExec $inducer(vEnv) "$Function,$arg"
}


#
# Event-Handler for `Help' Menu:
#

proc inducerEvhInfo { } {
    global inducer
    inducerDialog "[$inducer(wDev) GetVersion]\
	    \nInducer Test Access Panel\
	    \nConnected with: $inducer(vEnv) $inducer(vDev)\
	    \n\
	    \nClient Kernel Version: $inducer(version)\
	    \nServer Version:\n$inducer(vServerVersion)"
}


proc inducerEvhHelpOnDevice { } {
    global argv0
    # display man-page of "xxx" section 4 (Devices)
    regsub "xInducer" [file tail $argv0] "" name
    exec vltMan $name.4 &
}


proc inducerEvhExtendedHelp { } {
    global argv0
    # display man-page of "xxxInducer" section 1 (User Commands)
    exec vltMan [file tail $argv0].1 &
}

proc inducerEvhPersonalNotes { } {
    global argv0
    # call Personal Notes editor
    exec inducerNotes ~/.[file tail $argv0].notes &
}

# ___oOo___
