#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerExecLib.tcl,v 1.65 2003/12/17 19:26:26 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 23.06.1995 created (moved from inducerEvhLib)
#

#************************************************************************
#   NAME
#	inducerExecLib,
#	inducerExec
#	- Inducer Library Procedures for Function Execution on LCUs
# 
#   SYNOPSIS
#	inducerExec		Environment Command
#
#   DESCRIPTION
#	inducerExec	- to execute a `Command' on `Environment'
#			  and show its standard output in a pop-up window.
#			  `Command' must be a valid parameter-list for 
#			  the EXEC command of `inducerServer'.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#	None.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	inducerServer(1)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# exec `Cmd' (= function,parameters) and show result in toplevel window:
proc inducerExec { Env Cmd } {
    set W .exec	;# window-name of the exec-panel
    
    if {$Env == "" || $Cmd == ""} return
    
    # process command-line with server EXEC command:
    set text "\n-> EXEC $Cmd\n"
    set reply [InducerLine::Send $Env EXEC $Cmd]
    
    # separate function's stdout and return-value, indicated by a new-line:
    if {[set pos [string last \n $reply]] == -1} {
	append text "$reply\n"
    } else {
	set stdout [string range $reply 0 [expr {$pos-1}]]
	set result [string range $reply [expr {$pos+1}] end]
	append text "$stdout\nvalue = $result = 0x[format %x $result]\n"
    }

    # If exec-panel exists already, then only put it on top and return:
    if {! [winfo exists $W]} {
	# Build new toplevel window:
	toplevel $W
	wm title $W "[wm title .]: Exec Output"
	wm iconbitmap $W [wm iconbitmap .]
	wm minsize $W 1 1
	# !! -bd and -borderwidth options are not reognized in
	# UifWidgets
	UifText $W.text -relief groove -width 80 -height 25 -wrap none
	frame $W.but
	UifButton $W.but.clear -label Clear -width 10 \
		-command [list $W.text deleteText 1.0 end]
	UifButton $W.but.close -label Close -width 10 \
		-command [list destroy $W]
	pack $W.text -side top -expand yes -fill both -padx 1m -pady 1m
	#!! NOTE: the next line is a workaround (geometry bug in UifText ?):
	#pack $W.text.txt -expand yes -fill both
	pack $W.but -side bottom -anchor s -fill x -padx 1m -pady 1m
	pack $W.but.clear -side left  -anchor w
	pack $W.but.close -side right -anchor e
	$W.text insertText end $text	;# show the text
    } else {
	wm deiconify $W			;# make sure that it's not an icon
	#!! NOTE: the `raise' causes sometimes panel blocking. Don't know why!
	#!! raise $W			;# put window on top
	$W.text insertText end $text	;# show the text

	#!! NOTE: the following bypasses the class's callable interface
	#         uses direct access to the TK text widget inside
	#         UifText
	set textWdg [$W.text getComponentPath uifText]
	$textWdg mark set insert end	;# move cursor to the very end
	$textWdg yview -pickplace end ;# make sure that cursor is visible
    }
}

# ___oOo___
