# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: isstclISS.tcl,v 1.19 2005/05/04 10:48:20 vltsccm Exp $"
#
# isstclISS.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# jott     2005-05-03 Added method LogAlign{} (SPR VLTSW20050147),
#                     modified method WaitReply not to overwrite global
#                     default timeout, set global default timeout to
#                     40 seconds and provided method individual default
#                     timeouts for all methods sending commands with longer
#                     timeouts in the issif routing table 
# jott     2005-02-23 Reset error stack in WaitReply to avoid error flooding
# jott     2005-02-22 Added optional parameter gsFwhm for COU GS FWHM to method
#                     PresetStart{} (SPR VLTSW20040339)
# rrollfin 2005-01-11 Added StartLabGuiding and StopLabGuiding (VLTSW20040337)
# rrollfin 2005-01-11 Added checkAbortFlag to WaitForTracking, WaitForGuiding,
#                     WaitForOkFlags, WaitForFringes and WaitReply
#                     (see SPR VLTSW20040418)
# jott     2004-12-21 Include simulation of ISS to procedure ImageAlignAll
# jott     2004-12-17 Commented out tplLog in method GetIas{} and formatted
#                     returned reference pixels in method GetRefPixel{} to
#                     two digits after comma
# awalland 2004-07-01 Added methods OptFluxWait and CalibFinitoWait
# jott     2004-06-23 Added method CalibFinito{} (SPR VLTSW20040134)
# jott     2004-04-07 Added method SetLogToFile{} to finish
#                     SPR VLTSW20030115
# jott     2004-02-16 Ported to APR2004 release
# awa/alo  2003-12-16 Corrected bug in getting guvScale in constructor
# jott     2003-11-20 Fixed bug in issGetArray{} and issGetConfiguration{}
#                     methods in case of access to ISS is simulated
# jott     2003-11-19 Method GetGUVScale now reads from database and added
#                     methods issGetArray{} and issGetConfiguration{}
# awalland 2003-10-29 Correct DB ref for GetRefPixel (VLTSW20030425)
# jott     2003-10-23 Added methods StartActOptics{} and  StopActOptics{}
# rrollf   2003-09-11 Moved WaitForOkFlags into WaitForTracking
# rrollf   2003-07-03 Added issGetInputChannels and issGetBaselines
#                     (see SPR VLTSW20030276)
# awalland 2003-05-13 Added CheckForFringe and OffsOpd methods
# awalland 2003-05-13 Added StrtFtk and StopFtk methods
# awalland 2003-05-13 Added OptFlux method
# awalland 2003-05-13 GetFringeSensor, SetFringeSensor, FinitoSetup and WaitForFringes
#                     methods added (see SPR VLTSW20030189)
# rrollf   2003-05-07 Added IWS file logging (see SPR VLTSW20030115)
# awalland 2002-11-18 Swapped sin and cos in ChopNod method after skytest
# awalland 2002-11-13 Changed chop setup parameters to support chop-nod
# mkiekebu 2002-11-02 method GetCurrSlot added
# mkiekebu 2002-10-14 method GetRefPixel added
# mkiekebu 2002-10-11 In case of ISS not used, do not wait for tracking
# mkiekebu 2002-10-02 WaitPick, CentreImage and ImageAlign methods added
# awalland 2002-04-20 Added equinox par in PresetStart
#                     Added chop methods, added PresetOPL method
# awalland 2002-04-19 Removed OffsetAdg and OffsetTelStart
# awalland 2002-04-11 Derived from vnoseqISS.tcl
#
#************************************************************************
#   NAME
#       isstclISS - ISS class interface for templates.
#
#   SYNOPSIS
#   isstclISS iss <obj_name> <issDbAlias> <osDbAlias> <osSubsysName> <insId>
#
#   DESCRIPTION
#   VLTI Supervisor (ISS) interface for BOB templates.
#
#   PUBLIC METHODS
#       isstclISS  <obj_name> <issDbAlias> <osDbAlias> <osSubsysName> <insId>
#           obj_name    name of the new object
#           issDbAlias alias of the ISS point in the Instrument Environment
#           osDbAlias  alias of the OS point in the Instrument Environment
#           osSubsysName  name of ISS subsystem as known to OS
#           insId      ID of the instrument as known to ISS
#       IsUsed
#           Returns 1 if ISS is available, 0 otherwise.
#       EnvName
#           Returns the ISS environment, e.g. 'wvgvlti'.
#       ProcName
#           Returns the ISS interface process, e.g. 'issifControl'.
#       PresetStart <ra> <dec> <wlen> <dlMode> <dlRef> <dlRefOpl>
#                   <couType> <couMag> <pma> <pmd> <couSource> <couAlpha> <couDelta>
#                   <equinox> <addvela> <addveld> <gsFwhm>
#           Initiates a VLTI preset using SETUP command with TEL.TARG.TYPE COORDINATES.
#           Parameters corresponds to SETUP keyword
#           The caller has to check for the reply
#       PresetOPL <beam> <opl>
#           Preset delay line corresponding to <beam> to achieve a fixed total OPL of <opl>
#           The caller has to check for the reply
#       Remember <scical>
#           Send command REMEMBER to ISS. Parameter scical must be SCI, CAL1 or CAL2.
#           The caller has to check for the reply
#       Goback <scical>
#           Send command GOBACK to ISS. Parameter scical must be SCI, CAL1 or CAL2.
#           The caller has to check for the reply
#       WaitPick <timeout>
#           Send command WAITPIC to ISS
#           Default timeout 3 minutes (180000)
#           The caller has to check for the reply
#       CentreImage <beam> <timeout>
#           Send command CNTIMG to ISS to centre the image on the stellar beam selected
#           Default timeout 3 minutes (180000)
#           The caller has to check for the reply
#       ImageALign <beam> <tolerance> <maxloop> <timeout>
#           Default timeout 3 minutes (180000)
#           Send comamd IMGALGN to ISS to align automatically the stellar beam selected
#           The caller has to check for the reply
#       WaitForTracking [<timeout>]
#           Returns when ISS has completed preset and is tracking.
#           Default timeout is 300000 (5 minutes).
#       WaitForGuiding [<timeout>]
#           Returns when ISS is Coude' guiding (and tracking).
#           Default timeout is 300000 (5 minutes).
#       WaitForOkFlags [<timeout>]
#           Returns when flags OpticsOk and GuidingOk are true.
#           Default timeout is 600000 (10 minutes).
#       WaitForFringes [<timeout>]
#           Returns when DLCS is fringe tracking
#           Default timeout is 600000 (10 minutes)
#       CheckForFringes
#           Returns 1 when DLCS is fringe tracking
#       SendGuvOffset <beam> <offsetU> <offsetV> <offsetW>
#           Send UVW offset to VLTI (command OFFSGUV)
#           The caller has to check for the reply
#       SendAdgOffset <offsetAlpha> <offsetDelta> [<beam>]
#           Send ADG offset to VLTI (command OFFSADG)
#           The caller has to check for the reply
#       ChopSetup <freq> <throw> <posang> <offschop> [<pvratio>]
#           Setup chop parameters (but without starting chopping)
#           Should be called before PresetStart
#       StartChop [<atTime>]
#           Start synchronized chopping with parameters set by ChopSetup method
#           The caller has to check for the reply
#       StopChop [<atTime>]
#           Stop synchronized chopping
#           The caller has to check for the reply
#       ChopNodSetup <freq> <throw> <posang> <pvratio>
#           Setup chop nod parameters (but without starting chopping)
#           Should be called before PresetStart
#       ChopNod <throw> <posang>
#           Start a chop nod sequence
#           Returns when sequence completed
#       ChopNCycles <freq> <cycles>
#           Start a fixed cycle of chops
#           The caller has to check for the reply
#       GetFringeSensor
#           Returns currently selected fringe sensor
#       SetFringeSensor <fringeSensor>
#           Sets required fringe sensor using setup keyword, e.g.
#           fringe sensor is actually set after PresetStart
#           The caller has to check for the reply
#       FinitoSetup <trkMode> <targHmag> <trkWlength> <visibility> <ch_sel>
#           Setup Finito parameters (but without starting Finito)
#           Should be called before PresetStart
#       OptFlux
#           Start optimizing Finito injection
#           The caller has to check for the reply
#       OptFluxWait <cmdList>
#           Returns when Finito injection optimization has completed
#       CalibFinito
#           Start calibrating Finito
#           The caller has to check for the reply
#       CalibFinitoWait <cmdList>
#           Returns when Finito calibration has completed
#       StrtFtk
#           Start searching for fringes
#           The caller has to check for the reply
#       StopFtk
#           Stop searching for fringes
#           The caller has to check for the reply
#       OffsOpd <beam> <offset>
#           Apply a OPD offset on beam
#           The caller has to check for the reply
#       StartActOptics <flag>
#           Start active optics on UTs and ATs. The flag can have the
#           enumeration values 'REF' or 'STAR'.
#       StopActOptics
#           Stop active optics on UTs and ATs.
#       GetCurrSlot
#           Get actual target (NONE, SCI, CAL1, CAL2)
#       WaitReply <cmdList>
#           Wait the (last) repli(es) of all commands in cmdList.
#       issGetArray{}
#           Get VLTI Array Telescope configuration (telescope name and station).
#       issGetConfiguration{}
#           Get VLTI Array complete configuration (complete record of Array
#           configuration table).
#       SetLogToFile <on/off>
#           Enables or disables logging into file
#           $env(VLT_LOG_FILES)/bob.$envName.log
#       LogAlign{}
#           Triggers ISS to FITS log the image alignment parameters. Shall be
#           called by the instrument template after the image alignment itself
#           has been successfully completed.
#           NOTE that the logging is done only if the current image alignment
#           sensor is IRIS.
#
#   SEE ALSO
#
#   CAUTIONS
#
#   EXAMPLES
#
#   BUGS
#
#------------------------------------------------------------------------
#

#########################################################################
# ISS procedures
#
class isstclISS {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { issDbAlias osDbAlias osSubsysName insId } {
        global env

        set ccsEnv $env(RTAPENV)
        set issAlias $issDbAlias
        set osAlias $osDbAlias
        set subsysId $osSubsysName
        set insID $insId
        set envName $env(VLTI_ENVNAME)
        set procName "issifControl"

        #
        # check if ISS is used
        #
        set access [seq_dbReadSymbolic "<alias>$osAlias:subsystems:$subsysId.access"]
        if { $access == 0 } {
            set isUsed 1
        } else {
            set isUsed 0
        }
# rrollfin, 2004-12-06: Removed guv scal from constructor
#        #
#        # Get guv scale
#        set guvScale [seq_dbReadSymbolic "<alias>$issAlias:align.guvScale"]

        # retain up to 20 Kb of previous logs in /vltdata/tmp
        set lfname $env(VLT_LOG_FILES)/bob.$envName.log
        if {[ file exists $lfname ]} {
            exec isstclFileTrunc 20000 $lfname
        }

        # Open the log file
        set logf [open $lfname {CREAT RDWR APPEND} 0664]

    }


    #--------------------------------------------------------------------
    # destructor
    #
    destructor {

        # Close the log file
        close $logf

    }


    #--------------------------------------------------------------------
    # method IsUsed
    #
    method IsUsed {} {
        return $isUsed
    }


    #--------------------------------------------------------------------
    # method EnvName
    #
    method EnvName {} {
        return $envName
    }


    #--------------------------------------------------------------------
    # method ProcName
    #
    method ProcName {} {
        return $procName
    }


    #--------------------------------------------------------------------
    # method Timeout
    #
    method Timeout {} {
        return $timeout
    }


    #--------------------------------------------------------------------
    # method logtmp
    #
    method logtmp {cmdName msgbuf} {
        if { $logActive } {
            set now [clock seconds]
            set tstamp [clock format $now -format "%Y-%m-%dT%H:%M:%S"]
            puts $logf "# $tstamp"
            puts $logf "$cmdName $msgbuf"
            flush $logf
        }
    }

    #--------------------------------------------------------------------
    # method PresetStart
    #
    method PresetStart { ra dec wlen {dlMode NO_AUTO} {dlRef 0} {dlRefOpl 0} {couType NONE} {couMag 7} {pma 0} {pmd 0} {couSource SCIENCE} {couAlpha 0} {couDelta 0} {equinox 2000} {addvela 0} {addveld 0} {gsFwhm 0} {timeout 36300000} } {

        set cmdList {}
        set cmdName SETUP
        set par ",,TEL.TARG.TYPE COORDINATE \
                   TEL.TARG.ALPHA $ra \
                   TEL.TARG.DELTA $dec \
                   TEL.TARG.WLENGTH $wlen \
                   TEL.TARG.ADDVELALPHA $addvela \
                   TEL.TARG.ADDVELDELTA $addveld \
                   TEL.TARG.PMA $pma \
                   TEL.TARG.PMD $pmd \
                   TEL.TARG.EQUINOX $equinox \
                   COU.AG.TYPE $couType \
                   COU.AG.GSSOURCE $couSource \
                   COU.AG.ALPHA $couAlpha \
                   COU.AG.DELTA $couDelta \
                   COU.GS.MAG $couMag \
                   COU.GS.FWHM $gsFwhm \
                   DEL.REF.MODE $dlMode"
        if {$dlRef > 0} {
            set par "$par \
                     DEL.REF.NAME   dl$dlRef \
                     DEL.REF.OPL    $dlRefOpl"
        }
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }

        return $cmdList
    }

    #--------------------------------------------------------------------
    # method PresetOPL
    #
    method PresetOPL { beam opl {timeout 220000} } {

        set cmdList {}
        set cmdName PRSOPL
        set par "$beam,$opl"

        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
    }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method Remember
    #
    method Remember { scical } {

        set cmdList {}
        set cmdName REMEMB
        set par "$scical"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method Goback
    #
    method Goback { scical  {timeout 880000} } {

        set cmdList {}
        set cmdName GOBACK
        set par "$scical"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }

        return $cmdList
    }


    #--------------------------------------------------------------------
    # method WaitPick
    #
    # This method send the comand WAITPIC to the issif process and this
    # command waits for a pick-object in the aral tcscam rtd.
    #
    # Default timeout 2 minutes (120000)
    #
    method WaitPick { {timeout 120000} } {
        EnterLog

        set cmdList {}
        set cmdName WAITPIC
        set par {}
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        LeaveLog
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method CentreImage
    #
    # This method send the comand CNTIMG to the issif process and this
    # command centre the image of the selected stellar beam to the VLTI
    # reference point. It requires a previus WAITPIC command to know the
    # real coordinates of the object.
    #
    # Default timeout 3 minutes (180000)
    #
    method CentreImage { beam {timeout 180000}} {

        set cmdList {}
        set cmdName CNTIMG
        set par $beam
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method ImageAlign
    #
    # This method send the comand IMGALGN to the issif process and this
    # command does the alignment of the selected stellar beam to the VLTI
    # reference point by measuring the error on the ARAL tccd.
    #
    # Default timeout 5 minutes (300000)
    #
    method ImageAlign { beam tolerance maxloop {timeout 300000} } {

        set cmdList {}
        set cmdName IMGALGN
        set par "$beam,$tolerance,$maxloop"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StartLabGuiding
    #
    # This method send the comand STRTLAG to the issif process and this
    # command starts the laboratoy guiding for IRIS.
    #
    # Default timeout 35 seconds (35000)
    #
    method StartLabGuiding { {mode SLOW} {timeout 35000} } {

        set currIAS [GetIas]
        if { "$currIAS" == "IRIS" } {
            set cmdList {}
            set cmdName STRTLAG
            set par "$mode"
            if {$isUsed} {
                tplLog "Send command $cmdName $par to process $procName env. $envName"
                logtmp $cmdName $par
                set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
                lappend cmdList [list $cmdId $cmdName $timeout]
            } else {
                tplLog "Simulate command $cmdName $par to process $procName env. $envName"
            }
        } else {
            error "StartLabGuiding is only allowed for IRIS, but IAS = $currIAS"
        }

        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StopLabGuiding
    #
    # This method send the comand STOPLAG to the issif process and this
    # command stops the laboratoy guiding for IRIS.
    #
    # Default timeout 35 seconds (35000)
    #
    method StopLabGuiding { {timeout 35000} } {

        set currIAS [GetIas]
        if { "$currIAS" == "IRIS" } {
            set cmdList {}
            set cmdName STOPLAG
            set par ""
            if {$isUsed} {
                tplLog "Send command $cmdName $par to process $procName env. $envName"
                logtmp $cmdName $par
                set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
                lappend cmdList [list $cmdId $cmdName $timeout]
            } else {
                tplLog "Simulate command $cmdName $par to process $procName env. $envName"
            }
        } else {
            error "StopLabGuiding is only allowed for IRIS, but IAS = $currIAS"
        }

        return $cmdList
    }


    #--------------------------------------------------------------------
    # method GetIas
    #
    method GetIas { } {
        #
        # Get IAS
        set currIAS [seq_dbReadSymbolic "<alias>$issAlias:align.sensor"]
        #tplLog "Current Image Alignment Sensor is $currIAS"
        return $currIAS
    }


    #--------------------------------------------------------------------
    # method ImageAlignAll
    #
    method ImageAlignAll { tolerance maxloop {timeout 180000} } {

        set currIAS [GetIas]
        if { "$currIAS" == "IRIS" } {
            set cmdList [ImageAlign 0 $tolerance $maxloop ]
        } else {
            error "ImageAlignAll is only allowed for IRIS, but IAS = $currIAS"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method GetGUVScale
    #
    method GetGUVScale {} {

        # Get guv scale
        return [seq_dbReadSymbolic "<alias>$issAlias:align.guvScale"]

    }


    #--------------------------------------------------------------------
    # method GetRefPixel
    #
    method GetRefPixel {beam} {
        #- This information should be in the vlti public dbl but for the -#
        #- moment I will read them directly from wvgvlti                 -#
        set index [expr {$beam - 1}]
        set refX [seq_dbReadSymbolic "@$envName:Appl_data:VLTI:issalign:imageAlign.refPointTable($index,1)"]
        set refY [seq_dbReadSymbolic "@$envName:Appl_data:VLTI:issalign:imageAlign.refPointTable($index,2)"]
        set retRefX [format %.2f $refX]
        set retRefY [format %.2f $refY]
        return "$retRefX $retRefY"
    }


    #--------------------------------------------------------------------
    # method issGetInputChannels
    #
    method issGetInputChannels { } {
        # <alias>ISS:array.T1input
        # <alias>ISS:array.T2input
        # <alias>ISS:array.T3input

        set ntel [seq_dbReadSymbolic "<alias>$issAlias:array.ntel"]
        set retStr {}
        for { set i 0 } { $i < $ntel } { incr i } {
            set j [expr {$i + 1}]
            lappend retStr [seq_dbReadSymbolic "<alias>$issAlias:array.T${j}input"]
        }
        return $retStr
    }


    #--------------------------------------------------------------------
    # method issGetBaselines
    #
    method issGetBaselines { } {
        #   <alias>ISS:array.T1X    <alias>ISS:array.T2X    <alias>ISS:array.T3X
        #   <alias>ISS:array.T1Y    <alias>ISS:array.T2Y    <alias>ISS:array.T3Y
        #   <alias>ISS:array.T1Z    <alias>ISS:array.T2Z    <alias>ISS:array.T3Z

        set ntel [seq_dbReadSymbolic "<alias>$issAlias:array.ntel"]
        set retStr "There is only one telescope configured!"
        for { set i 0 } { $i < $ntel } { incr i } {
            set j [expr {$i + 1}]
            set X${j} [seq_dbReadSymbolic "<alias>$issAlias:array.T${j}X"]
            set Y${j} [seq_dbReadSymbolic "<alias>$issAlias:array.T${j}Y"]
            set Z${j} [seq_dbReadSymbolic "<alias>$issAlias:array.T${j}Z"]
        }

        if { $ntel > 1 } {
            # baseline T1/T2
            set baseline [expr {sqrt(($X1 - $X2) * ($X1 - $X2) + \
                                     ($Y1 - $Y2) * ($Y1 - $Y2) + \
                                     ($Z1 - $Z2) * ($Z1 - $Z2))}]
            set retStr $baseline
        }
        if { $ntel > 2 } {
            # baseline T1/T3
            set baseline [expr {sqrt(($X1 - $X3) * ($X1 - $X3) + \
                                     ($Y1 - $Y3) * ($Y1 - $Y3) + \
                                     ($Z1 - $Z3) * ($Z1 - $Z3))}]
            lappend retStr $baseline

            # baseline T2/T3
            set baseline [expr {sqrt(($X2 - $X3) * ($X2 - $X3) + \
                                     ($Y2 - $Y3) * ($Y2 - $Y3) + \
                                     ($Z2 - $Z3) * ($Z2 - $Z3))}]
            lappend retStr $baseline
        }

        return $retStr

    }


    #--------------------------------------------------------------------
    # method WaitForTracking
    #
    method WaitForTracking { {timeout 300000} } {
        EnterLog

        # check VLTI is tracking and substate IDLE (=21) (preset completed)
        set dbAttrSubstate "<alias>$issAlias:vltiState.vltiSubstate"
        set dbAttrTrack "<alias>$issAlias:vltiState.track"

        tplLog "Waiting for VLTI to complete preset and track, timeout $timeout"
        set elapsed 0
        set delay 1000
        set cont 1
        if {$isUsed} {
            while {$cont} {
                set substate [seq_dbReadSymbolic "$dbAttrSubstate"]
                set track [seq_dbReadSymbolic "$dbAttrTrack"]
                if {($track == 1) && ($substate == 21)} { set cont 0 }
                if {$cont} {
                    # check for abort flag
                    checkAbortFlag

                    update
                    after $delay
                    incr elapsed $delay

                }
                if {$elapsed >= $timeout} {
                    error "VLTI did not complete preset and start track after $timeout msec"
                    set cont 0
                }
            }

        # Check for guidingOk and opticsOk flags
        WaitForOkFlags

        }
        LeaveLog
    }


    #--------------------------------------------------------------------
    # method WaitForGuiding
    #
    method WaitForGuiding { {timeout 300000} } {
        EnterLog

        # check VLTI is Coude guiding (and tracking)
        set dbAttrGuide "<alias>$issAlias:guide.status"
        set dbAttrTrack "<alias>$issAlias:vltiState.track"

        tplLog "Waiting for VLTI to start Coude' guiding, timeout $timeout"
        bell
        set elapsed 0
        set delay 1000
        set cont 1
        if {$isUsed} {
            while {$cont} {
                set guide [seq_dbReadSymbolic "$dbAttrGuide"]
                set track [seq_dbReadSymbolic "$dbAttrTrack"]
                if {($guide == 1) && ($track == 1)} { set cont 0 }
                if {$cont} {
                    # check for abort flag
                    checkAbortFlag

                    update
                    after $delay
                    incr elapsed $delay
                }
                if {$elapsed >= $timeout} {
                    error "VLTI did not start Coude' guiding after $timeout msec"
                    set cont 0
                }
            }
        }
        LeaveLog
    }


    #--------------------------------------------------------------------
    # method WaitForOkFlags
    #
    method WaitForOkFlags { {timeout 600000} } {
        EnterLog

        # Check VLTI guidingOk and opticsOk flags
        set dbAttrGuidingOk "<alias>$issAlias:vltiState.guidingOk"
        set dbAttrOpticsOk  "<alias>$issAlias:vltiState.opticsOk"

        # First check if the flags are currently true
        set guidingOk [seq_dbReadSymbolic "$dbAttrGuidingOk"]
        set opticsOk [seq_dbReadSymbolic "$dbAttrOpticsOk"]
        if {($guidingOk == 1) && ($opticsOk == 1)} {
            LeaveLog
            return
        }

        tplLog "Waiting for Nasmyth Guiding and Active Optics, timeout $timeout"
        set elapsed 0
        set delay 1000
        set cont 1
        if {$isUsed} {
            while {$cont} {
                set guidingOk [seq_dbReadSymbolic "$dbAttrGuidingOk"]
                set opticsOk [seq_dbReadSymbolic "$dbAttrOpticsOk"]

                if {($guidingOk == 1) && ($opticsOk == 1)} { set cont 0 }
                if {$cont} {
                    # check for abort flag
                    checkAbortFlag

                    update
                    after $delay
                    incr elapsed $delay
                }
                if {$elapsed >= $timeout} {
                    if {($guidingOk == 0) && ($opticsOk == 0)} {
                        error "Nasmyth Guiding and Active Optics not started after $timeout msec"
                    } else {
                        if {($guidingOk == 0)} {
                            error "Nasmyth Guiding not started after $timeout msec"
                        } else {
                            error "Active Optics not started after $timeout msec"
                        }
                    }
                    set cont 0
                }
            }
        }
        LeaveLog
    }


    #--------------------------------------------------------------------
    # method WaitForFringes
    #
    method WaitForFringes { {timeout 600000} } {
        EnterLog

        # check VLTI is fringe tracking
        set dbAttrFringe "<alias>$issAlias:dl.motionPhase"

        tplLog "Waiting for VLTI to find fringes, timeout $timeout"
        bell
        set elapsed 0
        set delay 1000
        set cont 1
        set oldFringe 0
        # motionPhase is 6 when fringe tracking
        # we require two consecutive loops with motionPhase equal 6
        # to avoid triggering on spurious spikes
        if {$isUsed} {
            while {$cont} {
                set fringe [seq_dbReadSymbolic "$dbAttrFringe"]
                if {($fringe == 6) && ($oldFringe == 6)} { set cont 0 }
                if {$cont} {
                    # check for abort flag
                    checkAbortFlag

                    update
                    after $delay
                    incr elapsed $delay
                    set oldFringe $fringe
                }
                if {$elapsed >= $timeout} {
                    error "VLTI did not find fringes after $timeout msec"
                    set cont 0
                }
            }
        }
        LeaveLog
    }


    #--------------------------------------------------------------------
    # method CheckForFringes
    #
    method CheckForFringes {} {

        set vltiFringe 0
        # check VLTI is fringe tracking
        set dbAttrFringe "<alias>$issAlias:dl.motionPhase"

        # motionPhase is 6 when fringe tracking
        # we require two consecutive reads with motionPhase equal 6
        # to avoid triggering on spurious spikes
        if {$isUsed} {
            set oldFringe [seq_dbReadSymbolic "$dbAttrFringe"]
            after 1000
            set fringe [seq_dbReadSymbolic "$dbAttrFringe"]
            if {($fringe == 6) && ($oldFringe == 6)} {
                set vltiFringe 1
            }
        }
        return $vltiFringe
    }


    #--------------------------------------------------------------------
    # method SendGuvOffset
    #
    # Send command OFFSGUV to VLTI.
    #
    method SendGuvOffset { beam offsetU offsetV offsetW } {

        set cmdList {}
        set cmdName OFFSGUV
        set par "$beam,$offsetU,$offsetV,$offsetW"

        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method SendAdgOffset
    #
    # Send command OFFSADG to VLTI.
    #
    method SendAdgOffset { offsetAlpha offsetDelta {beam 0} } {

        set cmdList {}
        set cmdName OFFSADG
        if {$beam == 0} {
            set par "$offsetAlpha,$offsetDelta"
        } else {
            set par "$offsetAlpha,$offsetDelta,$beam"
        }

        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method ChopSetup
    #
    method ChopSetup { freq throw posang {pvratio 1} } {

        set cmdList {}
        set cmdName SETUP
        set offschop [ expr {$throw / 2.0}]
        set par ",,TEL.TARG.TYPE NO_TARGET \
                   TEL.CHOP.FREQ $freq \
                   TEL.CHOP.THROW $throw \
                   TEL.CHOP.PVRATIO $pvratio \
                   TEL.CHOP.POSANG $posang \
                   TEL.CHOP.OFFSCHOP $offschop \
                   TEL.CHOP.GUIDE A"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StartChop
    #
    method StartChop { {atTime ""} } {

        set cmdList {}
        set cmdName STRTCHP
        set par $atTime
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StopChop
    #
    method StopChop { {atTime ""} } {

        set cmdList {}
        set cmdName STOPCHP
        set par $atTime
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method ChopNod
    #
    method ChopNod { throw posang } {

        set cmdList {}
        set cmdName OFFSTEL
        # compute offset corresponding to chop

        # convert posang from degrees to radians
        set posangRad [ expr {$posang * 0.017453292}]
        set ra [expr {$throw * sin($posangRad)}]
        set dec [expr {$throw * cos($posangRad) * (-1)}]
        set par "$ra,$dec"

        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method ChopNodSetup
    #
    method ChopNodSetup { freq throw posang pvratio } {

        set cmdList {}
        set cmdName SETUP
        set pvratio [ expr {1 / $pvratio}]
        set offschop [ expr {$throw / 2.0}]
        set par ",,TEL.TARG.TYPE NO_TARGET \
                   TEL.CHOP.FREQ $freq \
                   TEL.CHOP.THROW $throw \
                   TEL.CHOP.PVRATIO $pvratio \
                   TEL.CHOP.POSANG $posang \
                   TEL.CHOP.OFFSCHOP $offschop \
                   TEL.CHOP.GUIDE B"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method ChopNCycles
    #
    method ChopNCycles { freq N } {

        set cmdList {}
        set cmdName STRTCHP
        # compute offset corresponding to chop

        # get current time and increment 5 seconds, stop N*freq seconds later (N chop cycles)
        set chopTime [ expr {$N / $freq}]
        set clock [getclock]
        set startClock [expr {int($clock + 5)}]
        set stopClock [expr {int($clock + 5 + $chopTime)}]
        set startTime [fmtclock $startClock "%Y-%m-%dT%H:%M:%S.000000"]
        set stopTime [fmtclock $stopClock "%Y-%m-%dT%H:%M:%S.000000"]

        if {$isUsed} {
            tplLog "Send command $cmdName $startTime to process $procName env. $envName"
            logtmp $cmdName $startTime
            set cmdId [seq_msgSendCommand $envName $procName $cmdName $startTime]
            lappend cmdList [list $cmdId $cmdName $timeout]
            WaitReply $cmdList
        } else {
            tplLog "Simulate command $cmdName $startTime to process $procName env. $envName"
        }

        set cmdList {}
        set cmdName STOPCHP
        if {$isUsed} {
            tplLog "Send command $cmdName $stopTime to process $procName env. $envName"
            logtmp $cmdName $stopTime
            set cmdId [seq_msgSendCommand $envName $procName $cmdName $stopTime]
            lappend cmdList [list $cmdId $cmdName $timeout]
            WaitReply $cmdList
        } else {
            tplLog "Simulate command $cmdName $stopTime to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method GetFringeSensor
    #
    method GetFringeSensor {} {

        set fringeSensor [seq_dbReadSymbolic "<alias>$issAlias:dl.fringeSensor"]
        return "$fringeSensor"
    }


    #--------------------------------------------------------------------
    # method SetFringeSensor
    #
    method SetFringeSensor { fringeSensor } {

        set cmdList {}
        set cmdName SETUP
        set par ",,TEL.TARG.TYPE NO_TARGET         \
                   DEL.FT.SENSOR $fringeSensor"

        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method FinitoSetup
    #
    method FinitoSetup { trkMode targHmag trkWlength visibility {ch_sel CH2} } {

        set cmdList {}
        set cmdName SETUP
        set par ",,TEL.TARG.TYPE NO_TARGET         \
                   DEL.FNT.TRK_MODE $trkMode       \
                   DEL.FNT.TARG_HMAG $targHmag     \
                   DEL.FNT.TRK_WLENGTH $trkWlength \
                   DEL.FNT.VISIBILITY $visibility  \
                   DEL.FNT.CH_SEL $ch_sel"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method OptFlux
    #
    method OptFlux { {timeout 86600000} } {

        set cmdList {}
        set cmdName OPTFLUX
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method OptFluxWait
    #
    method OptFluxWait { cmdList } {

        if {$isUsed} {
            WaitReply $cmdList
        }
    }

    #--------------------------------------------------------------------
    # method StrtFtk
    #
    method StrtFtk {} {

        set cmdList {}
        set cmdName STRTFTK
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StopFtk
    #
    method StopFtk {} {

        set cmdList {}
        set cmdName STOPFTK
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method OffsOpd
    #
    method OffsOpd { beam offset } {

        set cmdList {}
        set cmdName OFFSOPD
        set par "$beam,$offset"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method GetCurrSlot
    #
    method GetCurrSlot {} {
        return [seq_dbReadSymbolic "<alias>$issAlias:coord.currSlot"]
    }


    #--------------------------------------------------------------------
    # method WaitReply
    #
    method WaitReply { cmdList } {
        EnterLog

        global seq_errLogging

        set reply ""
        set cmdNum [llength $cmdList]
        for { set i 0 } { $i < $cmdNum } { incr i } {
            set cmd  [lindex $cmdList $i]
            set id      [lindex $cmd 0]
            set name    [lindex $cmd 1]
            set waitTimeout [lindex $cmd 2]

            tplLog "Waiting (timeout $waitTimeout) for pending replies to command $name id $id"

# rrollfin, 11.01.2005  SPR VLTSW20040418: Call seq_msgRecvReply periodically
#                       with -nowait option. So we can check for the abort flag.
#
#            set reply [seq_msgRecvReply -last $waitTimeout $id errFlag]
#            if { $errFlag != 0 } {
#                bell
#                bell
#                bell
#                error "ERROR performing command $name ($reply)."
#            }

            set elapsed 0
            set delay 1000

            # Switch to manual error logging mode
            set backup_seq_errLogging $seq_errLogging
            set seq_errLogging manual
            while {1} {
                if { [catch {seq_msgRecvReply -last -nowait $id errFlag} reply] } {
                    if { "$reply" == "message queue empty" } {
                        seq_errResetStack -internal
                        if { $elapsed >= $waitTimeout } {
                            bell
                            bell
                            bell

                            set errMsg "seqERR_REPLY_TIMEOUT: reply on $id ($name) timed out after $waitTimeout ms"
                            seq_errAdd -reply "$errMsg" seqMsgRecvReply.c

                            # Close error stack and restore logging mode
                            seq_errCloseStack -reply
                            set seq_errLogging $backup_seq_errLogging
                            error "reply timed out"
                        }

                        # check for abort flag
                        checkAbortFlag

                        update
                        after $delay
                        incr elapsed $delay
                        continue
                    }
                    break
                } else {
                    if { $errFlag != 0 } {
                        bell
                        bell
                        bell
                        error "ERROR performing command $name ($reply)."
                    }
                    break
                }
            }; # end while

            # Close error stack and restore logging mode
            seq_errCloseStack -reply
            set seq_errLogging $backup_seq_errLogging

            tplLog "Last reply to command $name id $id received"
        }

        LeaveLog
        return "$reply"
    }


    #--------------------------------------------------------------------
    # method StartActOptics
    #
    method StartActOptics { {flag "STAR"} } {

        set cmdList {}
        set cmdName STARTAO
        set par "$flag"
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand -check $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method StoptActOptics
    #
    method StopActOptics { } {

        set cmdList {}
        set cmdName STOPAO
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand -check $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }

    #--------------------------------------------------------------------
    # method issGetArray
    #   Get VLTI Array Telescope configuration (telescope name and station).
    #
    method issGetArray { {timeout 70000} } {
        set cmdList {}
        set cmdName GETARR
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
            set retStr [WaitReply $cmdList]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
            set retStr ""
        }
        return $retStr
    }


    #--------------------------------------------------------------------
    # method issGetConfiguration{}
    #   Get VLTI Array complete configuration (complete record of Array
    #   configuration table).
    #
    method issGetConfiguration { {timeout 70000} } {
        set cmdList {}
        set cmdName GETCONF
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
            set retStr [WaitReply $cmdList]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
            set retStr ""
        }
        return $retStr
    }

    #--------------------------------------------------------------------
    # method SetLogToFile <on/off>
    #    Enables or disables logging into file $env(VLT_LOG_FILES)/bob.$envName.log
    #
    method SetLogToFile { flag } {
        if { $flag == "ON" } {
            set logActive 1
        } else {
            set logActive 0
        }
    }


    #--------------------------------------------------------------------
    # method CalibFinito
    #
    method CalibFinito { {timeout 86600000} } {

        set cmdList {}
        set cmdName CALFNT
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # method CalibFinitoWait
    #
    method CalibFinitoWait { cmdList } {
        if {$isUsed} {
            WaitReply $cmdList
        }
    }


    #--------------------------------------------------------------------
    # method EnterLog
    #
    method EnterLog { } {
        if { $debug } {
            tplLog "=> entering [info level [expr {[info level] - 1}]]"
        }
    }

    #--------------------------------------------------------------------
    # method LeaveLog
    #
    method LeaveLog { } {
        if { $debug } {
            tplLog "=> leaving [info level [expr {[info level] - 1}]]"
        }
    }


    #--------------------------------------------------------------------
    # method LogAlign
    #
    # This method sends the comand LOGALGN to the issif process. It effects
    # that the parameters of the last image alignment are FITS logged (only in
    # case of IRIS is the current image alignment sensor).
    #
    # Default timeout 15 seconds (15000)
    #
    method LogAlign {{timeout 15000} } {

        set cmdList {}
        set cmdName LOGALGN
        set par ""
        if {$isUsed} {
            tplLog "Send command $cmdName $par to process $procName env. $envName"
            logtmp $cmdName $par
            set cmdId [seq_msgSendCommand $envName $procName $cmdName "$par"]
            lappend cmdList [list $cmdId $cmdName $timeout]
        } else {
            tplLog "Simulate command $cmdName $par to process $procName env. $envName"
        }
        return $cmdList
    }


    #--------------------------------------------------------------------
    # variables
    #
    public  variable issAlias   ""
    public  variable osAlias    ""
    public  variable subsysId   ""
    public  variable logf       0
    public  variable logActive  1       ;# 0 means no logging in /vltdata/tmp;
                                         # nonzero to activate
    private variable ccsEnv     ""
    private variable envName    ""
    private variable insID      ""
    private variable procName   ""
    private variable isUsed     0
    private variable debug      0       ;# Enable debug logging
    private variable timeout    40000

}

#########################################################################
# ___oOo___
