#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istRtd.tcl,v 1.49 2004/01/28 09:35:30 vltsccm Exp $" 
#
# istRtd.tcl
#
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    15/05/96  Created
# pbiereic   30/07/99  Uses istNamespaceTail
#
#************************************************************************
#   NAME
#
#   istRtd - panel editor wrapper of the RtdImageCtrl widget
#
#   SYNOPSIS
#
#   istRtd  <widget-name>
#
#   DESCRIPTION
#
#   istRtd is a mega widget which includes a fully useable rtd application.
#
#
#   This widget is only intended to be a small wrapper for panel editor 
#   usage of the RtdImageCtrl widget located in /usr/local/lib/rtd.
#
#   METHODS
#
#   imageCtrl  - returns the handle of the RtdImageCtrl widget. See man page
#                for more documentation.
# 
#   Example:
#   eval [<widget-name> imageCtrl] open
#  
#
#   PUBLIC VARIABLES
#
#   EXAMPLES
#
#     Create an instance
#
#     istRtd   .myImage
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#
#--------------------------------------------------------------
itk::usual istRtd {
    keep -relief -borderwidth -background -foreground
}

class istRtd { 
    
    inherit UifInteractiveWidget

    constructor {args} {
	global ::env ::rtd_library ::auto_path
	
	# the environment variable $RTD_LIBRARY should point to the directory
	# containing the Tcl library files
	if {[info exists env(RTD_LIBRARY)]} {
	    set rtd_library $env(RTD_LIBRARY)
	} else {
	    set rtd_library /usr/local/lib/rtd/library
	}
	
	# make sure the tcl library files are found
	set auto_path "$rtd_library $auto_path"
	
	set w_    [istNamespaceTail $this]
	
	eval itk_initialize 

	# 
	setXdefaults



	# create the rtd image widget and exit on errors, such as no more 
	# colors, no more memory...
	if {[catch {make_rtd} msg]} {
	    global ::errorInfo  
	    set info $errorInfo
	    #error_dialog $msg
	    puts "Error: $msg"
	    if {$itk_option(-verbose)} {
		puts stderr "$info"
	    }
	    exit 1
	}

	eval itk_initialize $args

	pack $itk_component(image) -fill both -expand 1
    }

    
    method setXdefaults {} {
	tk appname Rtd

        #option add *foreground Black 
        #option add *background #B2B2B2 
        #option add *Listbox.font -*-courier-medium-r-*-*-*-140-*-*-*-*-iso8859-* 
        #option add *Entry.font -*-courier-medium-r-*-*-*-140-*-*-*-*-iso8859-* 
        #option add *titleFont -*-courier-Bold-r-*-*-*-140-*-*-*-*-iso8859-* 
        # these are Tk defaults, but HP overrides them if we don't do this
	set labelFont -Adobe-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-*
	#option add *Button.Font $labelFont
	#option add *Label.Font $labelFont
	#option add *Menu.Font $labelFont
	#option add *Menubutton.Font $labelFont
	#option add *Message.Font $labelFont
	#option add *Scale.Font $labelFont
        #option add *Text.Font "-*-Courier-Medium-R-Normal--*-140-*-*-*-*-*-*"
	#option add *Text.foreground Black 
	#option add *Listbox.foreground Black 
	#option add *Entry.foreground Black 

	#option add *Text.background #D3B5B5 
	#option add *Listbox.background #D3B5B5 
	#option add *Entry.background #D3B5B5 

	#option add *selectForeground #C0C0C0 
	#option add *selectBackground Black 
	
	#option add *highlightThickness 0 
	#option add *Text.highlightThickness 2  
	#option add *Entry.highlightThickness 2 
	#option add *highlightBackground #B2B2B2
	#option add *highlightColor Black 

	#option add *shelp*Background  #B2B2B2
	#option add *shelp*Font -Adobe-helvetica-medium-r-normal--14*

	option add *plotBackground #C0C0C0 

	# bindings

	# make delete and backspace work like they should
	#bind Entry <Delete> {tkEntryBackspace %W}
	#bind Entry <BackSpace> {tkEntryBackspace %W}
	#bind Text <Delete> {tkEntryBackspace %W}
	#bind Text <BackSpace> {tkEntryBackspace %W}
    }


    # create the rtd image widget

    method make_rtd {} {
	set image_ $w_.image
	itk_component add image {
	    RtdImageCtrl $image_ \
		-file $itk_option(-file) \
		-usexshm $itk_option(-usexshm) \
		-verbose $itk_option(-verbose) \
		-shm_header $itk_option(-shm_header) \
		-shm_data $itk_option(-shm_data) \
		-drag_scroll $itk_option(-drag_scroll) \
		-subsample $itk_option(-subsample) \
		-xscale $itk_option(-xscale) \
		-yscale $itk_option(-yscale) \
		-use_zoom_view $itk_option(-use_zoom_view) \
		-zoom_view_propagate $itk_option(-zoom_view_propagate) \
		-with_zoom_window $itk_option(-with_zoom_window) \
		-dozoom $itk_option(-dozoom) \
		-with_pan_window $itk_option(-with_pan_window) \
		-zoom_factor $itk_option(-zoom_factor) \
		-zoom_width $itk_option(-zoom_width) \
		-zoom_height $itk_option(-zoom_height) \
		-pan_width $itk_option(-pan_width) \
		-pan_height $itk_option(-pan_height) \
		-colorramp_height $itk_option(-colorramp_height) \
		-default_cmap $itk_option(-default_cmap) \
		-default_itt $itk_option(-default_itt) \
		-with_colorramp $itk_option(-with_colorramp) \
		-rapid_frame_command [code $this rapid_frame_command] 
	}
    }
    
    # returns the image control handle
    method imageCtrl {} {
	return $image_
    }

    # -- public variables (also program options) -- 
    # image file to display
    itk_option define -file file File {} {
	$image_ configure -file $itk_option(-file)
    }

    # flag: if true, try to use X shared memory for images
    itk_option define -usexshm useXshm UseXshm {1} {
	#$image_ configure -usexshm $itk_option(-usexshm)
    }

    # these 2 flags control whether the FITS image header and/or data is kept in 
    # shared memory (see the rtdRemote interface for use of this)
    itk_option define -shm_header shm_header Shm_header 0 {
	#$image_ configure -shm_header $itk_option(-shm_header)
    }
    itk_option define -shm_data shm_data Shm_data 0 {
	#$image_ configure -shm_data $itk_option(-shm_data)
    }

    # flag: if true, print diagnostic messages
    itk_option define -verbose verbose Verbose {0} {
	#$image_ configure -verbose $itk_option(-verbose)
    }

    # flag: if true, use faster subsampling algorithm when shrinking images,
    # otherwise use max pixel algorithm
    itk_option define -subsample subsample Subsample {1} {
	#$image_ configure -subsample $itk_option(-subsample)
    }

    # default (midas) colormap
    itk_option define -default_cmap default_cmap Default_cmap {real} {
	#$image_ configure -default_cmap  $itk_option(-default_cmap)
    }

    # default (midas) intensity transfer table
    itk_option define -default_itt default_itt Default_itt {ramp} {
	#$image_ configure -default_itt  $itk_option(-default_itt)
    }

    # default scaling factor
    itk_option define -xscale xscale Xscale 1 {
	#$image_ configure -xscale  $itk_option(-xscale)
    }
    itk_option define -yscale yscale Yscale 1 {
	#$image_ configure -yscale  $itk_option(-yscale)
    }

    # zooming factor
    itk_option define -zoom_factor zoom_factor Zoom_factor 4 {
	#$image_ configure -zoom_factor  $itk_option(-zoom_factor)
    }

    # size of zoom window
    itk_option define -zoom_width zoom_width Zoom_width 152 {
	#$image_ configure -zoom_width  $itk_option(-zoom_width)
    }
    itk_option define -zoom_height zoom_height Zoom_height 152 {
	#$image_ configure -zoom_height  $itk_option(-zoom_height)
    }

    # height of the colorramp subwindow
    itk_option define -colorramp_height colorramp_height Colorramp_height 12  {
	#$image_ configure -colorramp_height  $itk_option(-colorramp_height)
    }

    # size of panning window
    itk_option define -pan_width pan_width Pan_width 152 {
	#$image_ configure -pan_width  $itk_option(-pan_width)
    }

    itk_option define -pan_height pan_height Pan_height 152 {
	#$image_ configure -pan_height  $itk_option(-pan_height)
    }

    # flag: if true (default), show the color ramp window
    itk_option define -with_colorramp with_colorramp With_colorramp 1 {
	#$image_ configure -with_colorramp  $itk_option(-with_colorramp)
    }

    # flag: if true, use a "view" of the main image for the zoom window
    # otherwise zoom directly from the X display.
    # The advantage of the first approach (-use_zoom_view 1) is that the zoom 
    # is accurate even when the main image is shrunken. 
    # The second (-use_zoom_view 0) is faster and allows more accurate positioning.
    itk_option define -use_zoom_view use_zoom_view Use_zoom_view 1 {
	#$image_ configure -use_zoom_view  $itk_option(-use_zoom_view)
    }

    # flag: if true, changes in main image scale will propagate to the zoom window,
    # otherwise controls are displayed so the user can manually change it (ZoomView only)
    itk_option define -zoom_view_propagate zoom_view_propagate Zoom_view_propagate 1 {
	#$image_ configure -zoom_view_propagate  $itk_option(-zoom_view_propagate)
    }

    # flag: if true (default) make a zoom window 
    itk_option define -with_zoom_window with_zoom_window With_zoom_window 1 {
	#$image_ configure -with_zoom_window  $itk_option(-with_zoom_window)
    }

    # flag: if true (default) make a panning window
    itk_option define -with_pan_window with_pan_window With_pan_window 1 {
	#$image_ configure -with_pan_window  $itk_option(-with_pan_window)
    }

    # flag: if true, turn on zoom window
    itk_option define -dozoom dozoom Dozoom 1 {
	#$image_ configure -dozoom  $itk_option(-dozoom)
    }
    
    # flag: if true, set bindings to scroll with the middle mouse button
    itk_option define -drag_scroll drag_scroll Drag_scroll 0 {
	#$image_ configure -drag_scroll  $itk_option(-drag_scroll)
    }

    
    # -- protected variables -- 

    # name of main image (class RtdImageCtrl or a derived class)
    protected variable image_
    protected variable w_
}  

# __o0o__
