#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: lcustat.tcl,v 1.11 2003/11/11 13:37:00 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  23/07/96  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# taken from vccRemPing, just don't fire a tcl error if host unreachable
# array of hosts that have responded to ping:
vccInit
global vccRemHosts
seq_ccsAsyncInput open
  
#
# D: 
# General panel initialization, configure widgets, set global variables and
# initializes display.
#
proc lcustatInitPanel { } {
    global cv bcv gvar seq_ccsEnvName EDITING env
    
    if {$EDITING} return
    
    seq_ccsAsyncInput open
    # configure rebootButton
    $bcv.rebootButton configure -background red -foreground yellow
    
    # reads database, set's gvar(lcuEnvList)
    if {[lcustatDbInit] == "FAILURE"} {
	puts "Error, lcustat database is empty or non-existent"
	puts "Please run lcustatSetup, or update the WS database"
	exit
    }
    set gvar(lcuEnvDisplay) $gvar(lcuEnvList)
    # set event handler
    lcustatDbAttachEvents {lcustatProcessEvt %D %T } 
    
    set gvar(lcuName) {}
    #------------------------------
    # CONFIGURE WIDGETS
    #configure LCU Selector
    lcustatConfLcuSelector
    $cv.aliveListWdg setBind <B1-ButtonRelease> [list lcustatSetLcuName\
	    $cv.aliveListWdg]
    $cv.deadListWdg  setBind <B1-ButtonRelease> [list lcustatSetLcuName \
	    $cv.deadListWdg]

    #D:
    # check that lcustatCk is running or start it
    checkLcustatCk
    #get init status
    foreach el $gvar(lcuEnvDisplay) {
	lcustatReadStatus $el
    }

}

# D: 
# Updates LCU environment selector.
proc lcustatConfLcuSelector { } {
    global gvar cv
    $cv.lcuSelector MakeMenu $gvar(lcuEnvList)
}

# D: 
# Read status for LCU <lcu> from DB, updates panel display.
proc lcustatReadStatus {lcu} {
    global gvar lcustatDb

    if {! [info exist lcustatDb($lcu)] } return

    set stat [seq_dbReadSymbolic $lcustatDb(path)($lcustatDb($lcu))]
    lcustatStatus $stat
    return $stat
}

#
# D: 
# Adds a <lcu> LCU environment from the listboxes which displays
# LCU's status
#
proc lcustatAddDisplay { lcu } {
    global gvar

    if {[lsearch $gvar(lcuEnvList) $lcu] < 0} return; # unknown environment
    if {[lsearch $gvar(lcuEnvDisplay) $lcu] >= 0} return; # already displayed
    
    lappend gvar(lcuEnvDisplay) $lcu
    lcustatReadStatus $lcu
}

#
# D: 
# Delete a LCU environment <lcu> from the listboxes which displays
# LCU's status
#
proc lcustatDelDisplay { lcu } {
    global gvar cv

    if {[lsearch $gvar(lcuEnvList) $lcu] < 0} return; # unknown environment
    set idx [lsearch $gvar(lcuEnvDisplay) $lcu]
    if { $idx < 0} return; # already not displayed
    
    lvarpop gvar(lcuEnvDisplay) $idx; #delete lcu from display list
    # looks lcu in alive list
    set alive [$cv.aliveListWdg getList]
    set idx [lsearch $alive $lcu]
    if {$idx >= 0} {
	$cv.aliveListWdg deleteItem $idx 
	return
    }
    # delete item from deadlist
    set idx [lsearch [$cv.deadListWdg getList]  "[lindex $lcu 0] *"]
    if {$idx >=0} {
	$cv.deadListWdg deleteItem $idx
    }	
}

#
# D: 
# Start application <app> , stolen from ssandroc's ccseiStartApp
#
proc lcustatStartApp { app } {
    global env

    set timeout [expr 4*30] ;# wait max. 30 sec for startup
    vccWinBusy "          Starting \"$app\" ...          " {
	set pid [eval exec $app &]
	while {[incr timeout -1] > 0} {
	    if {! [catch {exec sh -c "ps -e | grep '^ *$pid '"}]} {
		after 3000 ;# give another 2 seconds to initialize
		break
	    }
	    after 2500
	}
    }
}

#
# D: 
# called from lcustatCk application to add 'ALIVE' LCU <lcu>.
proc lcustatAddAlive { lcu } {
    global gvar cv
    
    # check if this node have been rebooted and it
    # boots from a remote WS
    if {[info exist gvar(showLog,$lcu)]} {
	#show remote logFile
	if { [seq_catch \
		[list vccRemOpen "" $gvar(showLog,$lcu) r] \
		fd [info script]]} {
	    tk_dialog .dialog "Error" $fd {} 1 OK
	    return
	}
	lcustatShow "\nEnvironment $lcu is back\n\n"
	lcustatShow [read $fd]
	vccRemClose $fd
	unset gvar(showLog,$lcu)
    }
    set alive [$cv.aliveListWdg getList]
    set idx [lsearch $alive $lcu]
    if {$idx < 0} {
	$cv.aliveListWdg addItem end $lcu
    }
    
    # delete item from deadlist
    set idx [lsearch [$cv.deadListWdg getList] "[lindex $lcu 0] *"]
    if {$idx >=0} {
	$cv.deadListWdg deleteItem $idx
    }	
}

#
# D: 
# called from lcustatCk application to add a 'DEAD' LCU <lcu>.
proc lcustatAddDead { lcu } {
  global gvar cv

    # delete item from alive list
    set idx [lsearch [$cv.aliveListWdg getList] [lindex $lcu 0]]
    if {$idx >=0} {
	$cv.aliveListWdg deleteItem $idx
    }
    
    set dead [$cv.deadListWdg getList]
    set idx [lsearch $dead "[lindex $lcu 0] *"]
    
    if {$idx < 0} {
	$cv.deadListWdg addItem end $lcu
    } else {
	$cv.deadListWdg deleteItem $idx 
	$cv.deadListWdg addItem $idx $lcu
    }

}

# D: 
# Clean LCU's status display list boxes.
proc lcustatCleanListBox { } {
    global cv 

    $cv.deadListWdg  deleteItem 0 end
    $cv.aliveListWdg deleteItem 0 end
    
}

# D: 
# Shows in a toplevel window the passed string <text>
# The toplevel window has  Close and Clear buttons.
#
proc lcustatShow { text } {
    set W .lcustatShow	;# window-name of the exec-panel
    
    # If exec-panel exists already, then only put it on top and return:
    if {! [winfo exists $W]} {
	# Build new toplevel window:
	toplevel $W
	wm title $W "[wm title .]: Exec Output"
	wm iconbitmap $W [wm iconbitmap .]
	wm minsize $W 1 1
	# !! -bd and -borderwidth options are not recognized in
	# UifWidgets
	UifText $W.text -relief groove -width 80 -height 25
	frame $W.but
	UifButton $W.but.clear -label "Clear" -width 10 \
		-command [list $W.text deleteText 1.0 end]
	UifButton $W.but.close -label "Close" -width 10 \
		-command [list destroy $W]
	pack $W.text -side top  -padx 1m -pady 1m
	#!! NOTE: the next line is a workaround (geometry bug in UifText ?):
	#pack $W.text.txt -expand yes -fill both
	pack $W.but -side bottom -anchor s -fill x -padx 1m -pady 1m
	pack $W.but.clear -side left  -anchor w
	pack $W.but.close -side right -anchor e
	$W.text insertText end $text	;# show the text
    } else {
	wm deiconify $W			;# make sure that it's not an icon
	#!! NOTE: the `raise' causes sometimes panel blocking. Don't know why!
	#raise $W			;# put window on top
	$W.text insertText end $text	;# show the text

	#!! NOTE: the following bypasses the class's callable interface
	#         uses direct access to the TK text widget inside
	#         UifText
	set textWdg [$W.text getComponentPath uifText]
	$textWdg mark set insert end	;# move cursor to the very end
	$textWdg yview -pickplace end ;# make sure that cursor is visible
    }
}


# D: 
# Send command  <Cmd> to environment <lcu> with Parameterss <Par>.
# returns all replys in one string or error message
proc lcustatCmd { lcu Cmd {Par ""} } {
    
    if { [lempty $lcu] } return    
    set host [vccInfo GetByEnv $lcu hostName]
    
    if { [seq_catch \
	    { set cmdId [seq_msgSendCommand $lcu lccServer $Cmd $Par] } \
	    msg [info script] ]} {
	return  "Error:  $msg"
	return
    }
    
    set err 0;	set lastReply 0; 
    if { [seq_catch \
	    {set reply [seq_msgRecvReply $cmdId errorNr lastReply -all]} \
	    msg [info script]]} {
	return ">>> $reply\n$msg"
    }
    return $reply
}

#-----------
# D: 
# Event handler to display the status of LCU's
proc lcustatProcessEvt {data type} {
    global lcustatDb gvar

    if {$type != "TABLE"} {
	puts "Error data type is not a table"
	return
    }

    set start [lindex $data 0]
    set end   [lindex $data 1]
    set elemList [seq_dbReadSymbolic $lcustatDb(path)($start:$end)]

    set len [expr $end-$start]
    
    if {$len==0} {
	lcustatStatus $elemList
    } else {
	while {![lempty $elemList]} {
	    lcustatStatus [lvarpop elemList]
	}
    }
}

# D: 
# Read LCU's status from DB, uodates panel display
proc lcustatStatus { lcuStatus } {
    global gvar lcustatDb
    
    set lcu [lindex $lcuStatus 0]
    if {[lsearch $gvar(lcuEnvDisplay) $lcu ] < 0} return
    set wd  [lindex $lcuStatus 1]
    set stat  [lindex $lcuStatus 2]

    switch $stat {
	0 {
	    set msg "$lcu host down"
	    lcustatAddDead $msg
	}
	1 {
	    set msg "$lcu network ok"
	    lcustatAddDead $msg
	}
	2 {
	    set msg  $lcu
	    lcustatAddAlive $msg
	}
    }
}

# D: 
# When a LCU is selected in a listbox updates
# the LCU name: entry in the selector
#
proc lcustatSetLcuName {wdg} {
    global gvar cv
    
    # gvar(lcuName) is set every time an item is selectwd in any
    # listbox
    set lcuStat  [$wdg getSelection]
    set gvar(lcuName) [lindex $lcuStat 0]
    
    # set state of action buttons
    if {[llength $lcuStat] > 1} {
				# comes from dead list
	lcustatActionButtons $gvar(lcuName) 
    } else {
	lcustatActionButtons $gvar(lcuName) 
    }
}

# D:
# Configure action buttons according to the status of the selected 
# LCU environment
proc lcustatActionButtons {lcu} {
    global gvar bcv cv lcustatDb

    if {[lsearch $gvar(lcuEnvList) $lcu] < 0} return; # unknown environment
    #if {[lsearch $gvar(lcuEnvDisplay) $lcu] < 0} return; # not displayed

    if {! [info exist lcustatDb($lcu)] } return
    set stat [lindex [seq_dbReadSymbolic $lcustatDb(path)($lcustatDb($lcu))] 2]
    
    # stat: 0 down 1 networkO= OK  2 LCC OK
    $bcv.configure    configure -state normal
    $bcv.showLog      configure -state normal

    if {$stat == 0 || $stat == 1} {
	$bcv.nfsState     configure -state disabled
	$bcv.rebootButton configure -state disabled
	$bcv.showEnvs     configure -state disabled
	$bcv.showProcs    configure -state disabled
    } elseif {$stat == 2} {
	$bcv.nfsState     configure -state normal
	$bcv.rebootButton configure -state normal
	$bcv.showEnvs     configure -state normal
	$bcv.showProcs    configure -state normal
    }
}

# D: 
# Check if lcustatCheck is running, if not starts it.
proc checkLcustatCk { } {
    global seq_ccsEnvName

    if {[seq_catch \
	    {seq_msgSendCommand $seq_ccsEnvName lcustatCk PING ""} cmd] } {
	# start lcustatCk server
	lcustatStartApp [findVLTFile bin/lcustatCheck]
    } else {
	seq_catch {seq_msgRecvReply $cmd err}
    }
}

# ---------- PANEL BUTTONS CALBACKS ---------------
#

# show lcuenvs / callback for panel action button
proc lcustatShowEnvs {lcu} {

    if {[lempty $lcu]} return;
    lcustatShow "\n--- Cmd LCCPET in env $lcu ----\n"
    lcustatShow [lcustatCmd $lcu LCCPET]
    lcustatShow "\n--- Cmd LCCPET Done ----\n\n"
}

#
# show process registered in lcu env, callback for panel action button
#
proc lcustatShowProcs { lcu } {
    if {[lempty $lcu]} return;

    lcustatShow "\n--- Cmd LCCPLT in env $lcu ----\n"
    lcustatShow [lcustatCmd $lcu LCCPLT]
    lcustatShow "\n--- Cmd LCCPLT Done ----\n\n"
}

# execute vccConfigLcu on lcu / callback for panel action button.
proc lcustatConfigLcu {lcu} {

    if {[lempty $lcu]} return

    lcustatStartApp "vccConfigLcu $lcu"
}

# nfs check button
proc lcustatNfsCheck { lcu } {
    if {[lempty $lcu]} return
    
    lcustatShow "\n--- Cmd LCCNFS in env $lcu ----\n"
    lcustatShow [lcustatCmd $lcu LCCNFS]
    lcustatShow "\n--- Cmd LCCNFS Done ----\n\n"
    
}
# show log file button
proc lcustatShowLog { lcuEnv } {
    global env

    if {[lempty $lcuEnv]} return

    set bootEnv   [vccInfo GetBootEnvByEnv $lcuEnv]
    set localHost [vccInfo GetByEnv $env(RTAPENV) hostName]
    set bootHost  [vccInfo GetByEnv $bootEnv  hostName]

    if {$localHost != $bootHost} {
	#resolve remote path 
	set path [vccRemPath $bootHost \$VLTDATA]
	if {[lempty $path]} {
	    error "could not resolve VLTDATA in Host $Host"
	    return ""
	}
	set path "$bootHost:$path/ENVIRONMENTS/$lcuEnv/lcubootLogFile"
    } else {
	set path $env(VLTDATA)/ENVIRONMENTS/$lcuEnv/lcubootLogFile
    }
    #show logFile
    set fd [vccRemOpen "" $path]
    lcustatShow "\nLogFile for Environment $lcuEnv\n\n"
    lcustatShow [read $fd]
    vccRemClose $fd
}

# reboot button
proc lcustatReboot {} {
    global gvar

    if {[lempty $gvar(lcuName)]} return;

    set lcu     $gvar(lcuName)
    set gvar(lcuName) {}
    lcubootWatchBoot $lcu
}

# ---> stolen from moteiSendProcs.tcl
# ---> Sends command Cmd to Env/Proc
# ---> returns reply, 
# ---> sets err, cmdId and last
#
proc sendServer { Env Proc Com {Par ""} {err ""} {cmdid ""} {last ""}} {

    if {$err != ""} {
	upvar err errorNr
    }
    set errorNr 0
    if {$cmdid != ""} {
	upvar $cmdid cmdId
    }
    set cmdId unknown
    if {$last != ""} {
	upvar $last lastReply
    }
    set   lastReply 0

    if { [seq_catch \
	    { set cmdId [seq_msgSendCommand $Env $Proc $Com $Par] } \
	    msg [info script] ]} {
	set reply $msg
	# puts ">>> $reply"
	if {![lempty $err]} {
	    set errorNr 1
	    return $reply
	}
	# backward compatibility
	return "FAILURE"
    }
    if { [seq_catch \
	    {set reply [seq_msgRecvReply $cmdId errorNr lastReply]} \
	    msg [info script]]} {
#	puts ">>> $reply\n$msg"
	if {$err != ""} {
	    set errorNr 1
	    return "$reply\n$msg"
	}
	# backward compatibility
	return "FAILURE"
    } elseif {$errorNr != 0} {
	set msg "Error/reply from $Env/$Proc response to\
		\"$Com $Par\":\n$reply"
#	puts $msg
	if {$err != ""} {
	    set errorNr 1
	    return $msg
	}
	# backward compatibility
	return "FAILURE"
    }
    return $reply
}

proc lcustatUpdateConfig {} {
 global gvar 
 
    puts "Cleaning list box"
    lcustatCleanListBox
    puts "updating"
    lcustatDbInit
    set gvar(lcuEnvDisplay) $gvar(lcuEnvList)
    
    set gvar(lcuName) {}
    
    foreach el $gvar(lcuEnvDisplay) {
	lcustatReadStatus $el
    }
    
}
#
# ___oOo___
