#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: logGenFiles.tcl,v 4.26 2005/10/24 07:59:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  05/10/95  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#-----------------------------------------------------------
#   Procedure   logGenFiles
#
#
#  The procedure generates three files :
#
#  - <mod>Logs.h    which contains the '#define' statements
#                   to be included in C-source files
#
#  - <mod>_LOGS     which contains the error definitions
#
#  - <mod>LOGS.IDX  which contains the indexes to access the 
#                   proper error definition
#
#  Return  none.
#
#-----------------------------------------------------------

proc logGenFiles { module } {

   global INTROOT editMode  mainShArea saveFlag
   
#   if { $editMode != "r+" } {   
#       set tmp  "Not Allowed Operation"
#       tk_dialog .dialog "Dialog Box" $tmp info 0 OK     
#       return 
#   }  
    
#   sortErrors   
#   genLogDefFile      $module 
   genLogIndexFile    $module 
   genLogIncludeFile  $module
   
#   set saveFlag      0  
#   SetShortHelpArea $mainShArea "Error Configuration Files Generated " 
}

#-----------------------------------------------------------
#   Procedure   genLogIndexFile
#
#  The procedure generates the Index File for fast access 
#  to the definition file
#
#  Return  none
#
#-----------------------------------------------------------
 
 
proc genLogIndexFile { module } {
 
   global recList logTotal byteCnt
   global log_id log_type log_memo log_format  
   
    
   set myFile  "${module}LOGS.IDX";   # File name

   if { [ set filePath [ findModuleFile "LOGS/$myFile" ]] == "" } {
       set filePath  [getRelPath $module "LOGS" ]
       append filePath "/$myFile"       
   }
    
   set fh [open $filePath w ] ;      # Open and create a new file 

   writeFileHeader $module "Error Index File" $fh "TCL"
 
   set i 0
   
   foreach item [array names recList]  {     
      set myList   $recList($item)      
      set logId    [lindex $myList $log_id]
      set logType  [lindex $myList $log_type]

      puts $fh "$logId  $logType [lindex $byteCnt $i]"; incr i     
   } 

   puts  $fh "#\n# \"@(#) \$Id\$\" \n";  # Write cmm identifier and the EOF
   close $fh;                            # Close the file
}
 
#-----------------------------------------------------------
#   Procedure   genIncludeFile
#
#  The procedure generates the Include File
#
#  Return  none
#
#-----------------------------------------------------------
 
 
proc genLogIncludeFile { module } {
 
   global recList logTotal byteCnt
   global log_id log_type log_memo log_format  

   
   set myFile  "${module}Logs.h";   # File name

   if { [ set filePath [ findModuleFile "include/$myFile" ]] == "" } {
       set filePath  [getRelPath $module "include" ]
       append filePath "/$myFile"       
   }
     
   set fh [open $filePath w ] ;      # Open and create a new file 

   writeFileHeader $module "Error Include File" $fh "C" 

   puts $fh "#ifndef ${module}LOGS_H"
   puts $fh "#define ${module}LOGS_H \n"  

   #
   #  Calculate the maximum length of the mnemonics
   #

   set maxLen 0
 
   foreach item [array names recList] {       
      set len    [string length [lindex $recList($item) $log_memo]]        
      if {$len > $maxLen} { set maxLen $len }
   }
   
 
   set spaces  "                                " 
   set maxLine [expr {9 + $maxLen}] 
   
   foreach item [array names recList] {
   
      set myList   $recList($item)
      set mnemonic [lindex $myList $log_memo]
      set logId    [lindex $myList $log_id]

      set  line  "#define  $mnemonic" 
      set  len   [string length $line]
      
      if { $len < $maxLine } {
	 set padLen  [expr {$maxLine - $len - 1}]
         append line [string range $spaces 0 $padLen]      
      }
      
      append line  "    "  $logId                
   
      puts $fh $line;          # Write each line in the file      
   } 
   
   puts  $fh "\n#endif"  
   puts  $fh "\n";           # Write the EOF of the file 
   close $fh;                # Close the fill
}

 
 
 
 
 
#-----------------------------------------------------------
#   Procedure   writeFileHeader
#
#  The procedure write an header on top of the generated files
#
#  Return  Offset String
#
#-----------------------------------------------------------

proc writeFileHeader  {module description  fh  language} {

   set date [exec date]

   set date [lreplace $date 4 4]
   set date [lreplace $date 0 0]

   if {$language == "C"} {
      puts $fh "/***********************************************************"
      puts $fh "# \"@(#) \$Id\$\" \n#";  # Write cmm Identifier
   } else {
      puts $fh "#\n# Version Id. is specified at the end of the file \n#"
   }    
      
   set line1 "# $description    Created on  $date"

   puts $fh $line1
   puts $fh "# "     
   puts $fh "# This file has been generated by a utility"
   puts $fh "# "
   puts $fh "#  !!!!!!!!!!!  DO NOT MANUALLY EDIT THIS FILE  !!!!!!!!!!!"
   puts $fh "# "
     
   if {$language == "C"} {
      puts $fh "***********************************************************/"
   }        
}



#
# ___oOo___
