#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: logMonitorAwk.tcl,v 4.26 2005/10/24 07:59:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/03/96  created
#           20/08/98  Add INSPECT of the last n-th lines
# tebert    17/07/03  removed processing with awk
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#
# Setup AWK program and file corresponding to selections
#
proc logSetupAwk {} {
    global disAll wname selectBox dispsw filterVal

    # Initialize other filter parameters

    set t1 $filterVal(date1)
    set t2 $filterVal(date2)

    # setup time strings for correct comparison
    set index [string first . $filterVal(date1)]
    if { $index == -1 } {
        set tmpStr $filterVal(date1)
    } else {
        set tmpStr [string range $filterVal(date1) 0 [expr {$index - 1}] ]
    }

    if { [catch {set secFormat [convertclock $tmpStr]} secFormat]} {
        $shArea displayText $secFormat
        set filterVal(date1new) "00-01-01 00:00:00"
    } else {
        set filterVal(date1new) [clock format $secFormat -format "%Y-%m-%d %H:%M:%S"]
    }

    set index [string first . $filterVal(date2)]
    if { $index == -1 } {
        set tmpStr $filterVal(date2)
    } else {
        set tmpStr [string range $filterVal(date2) 0 [expr {$index - 1}] ]
    }

    if { [catch {set secFormat [convertclock $tmpStr]} secFormat]} {
        $shArea displayText $secFormat
        set filterVal(date2new) "30-01-01 23:59:59"
    } else {
        set filterVal(date2new) [clock format $secFormat -format "%Y-%m-%d %H:%M:%S"]
    }
}


proc logCheckListbox { fitsLogFlag args } {
    global wname filterBox filtmn  CheckListEnable f_menu

    # Return if the routine is actually processing data

    if {$fitsLogFlag} {
	# For FITS logs only the env. name applies
	set nameList "env"
    } else {
	set nameList  [array names filterBox] 
    }

    foreach name $nameList {
	set tmp $filterBox($name)

	set colIdx [logGetColIdx $name]

	if {[lindex $tmp $f_menu] == 0 } { continue } ;# if menu is disabled

	set item [lindex $args $colIdx]

       # Check if the entry does not exists yet in the menu

	if {[tkMenuFindName $filtmn($name) $item] != "" } { continue }
        
	# Add new entry to the menu
	$filtmn($name) add command -label "$item" -command "set filterVal($name) \"$item\""
    }
}


proc logClearFilterMenu {} {
   global filtmn
   foreach name [array names filtmn] {
       $filtmn($name) delete 0 end   ;# Delete the menu
   }
}

#
# Filter log-file with awk and store in output file:
#
proc logFilterOnFile { userLogFile  {lines ""} } {
    global wname env monType awkOutFile ckErrOnly filtsw filterVal ckFitsFilter filterFlag
    global logDialog

    if {$ckErrOnly && $ckFitsFilter != 0} {
	tk_dialog .dialog "Inspect File" "Filters 'FITS Filter' and 'Errors' Only \ncannot be both active at the same time !!" info 0 OK
        return
    }
    
    if {[file exists $userLogFile] == 0 }  { return }

    $logDialog configure -refValue 100 -text "Pre-processing log file"

    update

    if { ![cequal $lines ""] } {

	# Store selected lines into temporary file
	# This trick is needed otherwise a tail from the end of the file
	# is limied to a maximum 20k of data.

	set tmp [lindex [exec wc -l $userLogFile] 0]
	if {[set tmp [expr {$tmp - $lines}]] < 1} { set tmp 1}
	
	# Read file from the tmp-th line
	set fileName $env(VLT_LOG_FILES)/logTailFile_[pid]
	exec tail +$tmp $userLogFile > $fileName
	exec chmod 777 $fileName
    } else {
	# Just read the whole file
	set fileName $userLogFile
    }

    # Set filtering criteria that should be applied at script level

    set filterCmd ""

    if {$filterFlag} {

	if { $ckErrOnly } {
	    set filterCmd "\"errorsOnly=on\" "
	}
	
	foreach name [list mod env proc pid] {
	    if {$filtsw($name) && ![cequal $filterVal($name) ""] } {
		append filterCmd " \"$name=$filterVal($name)\" "
	    }
	}
    }
    
    update
    
    if { $ckFitsFilter == 1 } {
	exec logFilterFile.awk "fitsOnly=on"  "env=$filterVal(env)" $fileName > $awkOutFile
    } elseif {$ckFitsFilter == 2 } {

#	puts " exec logFilterFile.awk \"noFits=on\" $filterCmd \"env=$filterVal(env)\" $fileName > $awkOutFile"
	eval exec logFilterFile.awk \"noFits=on\" $filterCmd \"env=$filterVal(env)\" $fileName > $awkOutFile
    } else {
#	puts "exec logFilterFile.awk $filterCmd $fileName > $awkOutFile"
	eval exec logFilterFile.awk  $filterCmd $fileName > $awkOutFile
    }

    update

    if {[file exists $awkOutFile]} {
	exec chmod 777 $awkOutFile
	set max [lindex [exec wc -l $awkOutFile] 0]
    } else {
	set max 0
    }

#    puts "$awkOutFile   max = $max"
    if {$max == 0} {
	if {[winfo exists $logDialog]} { destroy $logDialog }
	tk_dialog .dialog "File Failure" "No LOGS Retrieved !!" info 0 OK
	return
    }

#    puts "awkOutFile   $awkOutFile  $max"

    # Step represent the percentage step at which the UifWaitDialog
    # is updated.

    if {$max < 2000} {
	set step 10
    } elseif {$max >= 2000 && $max < 4000} {
	set step 30
    } else {
	set step 50
    }
    
    # Add dialog to tell the user that it is processing
    
    $logDialog configure -refValue $max \
	-text "Reading file '[file tail $userLogFile]' total of $max logs"
    update

    set fh [open $awkOutFile r ]
    set numLogs 0;

    while {[gets $fh line] >= 0 && ![cequal $monType STOP] } {

	if {[expr {fmod($numLogs,$step)}] == 0.0} {
	    $logDialog updatePercentage $numLogs
	    update
	}
	
	logAddLineFromFile $line ; incr numLogs;
    }

    close $fh
    destroy $logDialog ;

    if {$numLogs > 1000} {
	tk_dialog .dialog "Loading Logs" "$numLogs Logs .. LOADED from file $userLogFile !!" info 0 OK
    }

    $wname(LOGS) see end
    exec rm $awkOutFile
}

#
# ___oOo___


