#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: logMonitorBindings.tcl,v 4.26 2005/10/24 07:59:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/03/96  created
#           13/05/96  Add global var. in logClear
#           30/07/96  Add check on multiple selections for error stack
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#
# Set default fields to display
#

proc logSetFields { what } {
    global selectBox dispsw
        
    foreach i $selectBox {
	set name  [lindex $i 0]

	if {[cequal $what all]} {
	    set dispsw($name) 1
	} else {
	    set dispsw($name) [lindex $i 2]
	}
    }
    
    logCheckAllFields   
}



proc logCheckAllFields { } {
    global dispsw
    foreach name [array names dispsw] {
	logCheckField $name
    }
}

#
# This script is executed whenever the user selects a field
# to be displayed 
#

proc logGetColIdx { name } {
    global dispsw wname

    switch $name {
	"date"    { return 0 }
	"time"    { return 1 }
	"env"     { return 2 }
	"mod"     { return 3 }
	"proc"    { return 4 }
	"pid"     { return 5 }
	"logid"   { return 6 }
	"text"    { return 7 }
        default   { return -1 }
    }
}

proc logCheckField { name } {
    global dispsw wname

    set colIdx [ logGetColIdx $name ]

    # Hide the columns of the table which are not visible

    if {$dispsw($name)} {
	$wname(LOGS) columnconfigure $colIdx -hide 0
    } else {
	$wname(LOGS) columnconfigure $colIdx -hide 1
    }
}


proc logSetListFont {} {
   global wname logFont

   switch $logFont {
     small    {set val "*-fixed-medium-r-normal--10-*-*-*-*-*-*-*"; set y 400}
     medium   {set val "*-fixed-medium-r-normal--13-*-*-*-*-*-*-*"; set y 440}
     large    {set val "*-fixed-medium-r-normal--15-*-*-*-*-*-*-*"; set y 470}
     default  { return }
   }

   $wname(LOGS) configure -font $val
}


proc logApplyFilter {} {
    global wname filterStatus filtwi filterBox monType filterFlag ckFitsFilter ckErrOnly

    if {$filterFlag} {
	set filterStatus "IN USE"
	set fgColor Blue
    } else {
	set filterStatus "NOT USED"
	set fgColor      Grey90
	set ckFitsFilter 0
	set ckErrOnly    0
    }
 
   update

   logSetupAwk

   foreach name [array names filterBox] {
      if {[winfo exists $filtwi($name)]} {
         $filtwi($name) configure -fg $fgColor
      }
   }
}

#
# Clear log listbox
#

proc logClear {} {
   global wname

   $wname(LOGS) delete 0 end
   update
}

#
# Display complete error stack
#

proc logShowErrStack {} {
    global wname dispsw msgFile shArea
    
    if {[set idx [$wname(LOGS) curselection]] == ""} { 
	$shArea displayText "NO Log SELECTED"
	return 
    }
    
    if {[set last [llength $idx]] > 1} {
	incr last -1
	$wname(LOGS) selection clear [lindex $idx 0] [lindex $idx $last]
	$shArea displayText "SELECT a SINGLE log"
	return 
    }

    # Get error text directly from the table cell where it is contained

    set logText [$wname(LOGS) cellcget $idx,7 -text]

    if {[scan $logText {%s %d %d %d %s %s %n} procName errNumb stack seqNumb errSev errMemo si] < 7} {
	tk_dialog .dialog "Failure" "Cannot Retrieve Error Stack : NOT ERROR LOG" info 0 OK
	return

    }

    exec errShowStack $stack [file tail $msgFile] &
    return
}

#
# Copy Date/Time to filter as starting Date/Time
#

proc copyDateTime { what } {
    global  filterFlag filtsw filterVal filtwi wname
    
    if {!$filterFlag || !$filtsw(date1) } { return }
        
    # Get only first line of the selection
    if {[set idx [lindex [$wname(LOGS) curselection] 0]] == ""} { 
	return 
    }
    
    # Get date and time
    set colIdx  [logGetColIdx "date"]
    set logDate [$wname(LOGS) cellcget $idx,$colIdx -text]

    set colIdx  [logGetColIdx "time"]
    set logTime [$wname(LOGS) cellcget $idx,$colIdx -text]
    set logTime  [string range $logTime 0 7]
    
    if {[cequal $what "start"]} {
	set  filterVal(date1)  "$logDate $logTime"
	  
	if {[winfo exists $filtwi(date1)]} {
	    eval $filtwi(date1) flash
	}
    } else {
	
	set  filterVal(date2)  "$logDate $logTime"
	
	if {[winfo exists $filtwi(date2)]} {
	    eval $filtwi(date2) flash
	}
    }
}

#
# Preset the filter entries from the selection in the listbox
#
proc logPresetFilterFromSelection {} {
   global selectBox dispsw  filterVal filtsw filtwi wname

   # Get only first line of the selection
   if {[set idx [lindex [$wname(LOGS) curselection] 0]] == ""} { return }

   foreach i $selectBox {
       set d_name      [lindex $i 0]
       set d_subfields [lindex $i 3]
       set f_name      [lindex $i 5]

       # For unused filters: f_name == NIL
       
       if { $f_name == "NIL" || $f_name == "" } { continue }

       if {[info exists filtsw($f_name)] == 0 || $filtsw($f_name) == 0} { 
	   continue 
       }

       set colIdx  [logGetColIdx $d_name]
              
       if { $dispsw($d_name) && $colIdx >= 0 } {
	   set filterVal($f_name) [$wname(LOGS) cellcget $idx,$colIdx -text]
	   eval $filtwi($d_name) flash
       }
   }
}


#
# ___oOo___
