#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: logMonitorPanel.tcl,v 4.26 2005/10/24 07:59:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    20/03/96  created
#           30/07/96  Changed selectmode to extended
#           28/08/96  Add menu check button to enable/disable scrolling
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#
# Generate sequence of check buttons to setup the proper filter
#

proc createFilterBox { filtFrame shArea {mainFlag 1} } {
   global filterBox filtsw filterVal filtwi  filtmn filterStatus
   global f_lbl f_oper f_len f_ckDef f_Def f_row f_menu f_hlp f_main

   # Create variables with dummy values, when the routine is called by the main panel

   if {$mainFlag} {
      foreach name [array names filterBox] {
         set tmp $filterBox($name)
         set filterVal($name) [lindex $tmp $f_Def]   ;# Def. for entry
         set filtsw($name)    [lindex $tmp $f_ckDef] ;# Def. for checkbutton
         set filtwi($name)    ""                     ;# Set Widget name
      }
   } 

   for {set row 1} {$row <= 4} { incr row } {
       foreach name [lsort [array names filterBox]] {

          set buf $filterBox($name)

          # Check if it appears on main panel

	  if {$mainFlag} {
              if {![lindex $buf $f_main]} { continue }
          } else {
              if {[lindex $buf $f_main]}  { continue }
          }

          # Check if widget should be packed on current row
          if {[lindex $buf $f_row] != $row} { continue }

          set hlpText  [lindex $buf $f_hlp]

          set w             $filtFrame.r${row}.c$name
          set filtwi($name) $w
          set tmp           "[lindex $buf $f_lbl] [lindex $buf $f_oper]"

          checkbutton $w -text $tmp -variable filtsw($name)\
                         -anchor w -fg Blue -command logSetupAwk 

          if {!$mainFlag} { $w configure -width 10 } ;# To keep alignement

          UifShortHelp::add $shArea $w "Enable/Disable Filter ' $tmp '"

          pack $w -side left;

          # Create associated entry widget

          set w $filtFrame.r${row}.e$name
      
          entry $w -textvariable filterVal($name) -width [lindex $buf $f_len]
          bind  $w <Return> {set  filterFlag 1; logApplyFilter}

          UifShortHelp::add $shArea $w "Filter Text for $hlpText"

          pack $w -side left -anchor w -padx 3 -pady 3

          ##############################################################
          #   This adds a pop-up menu of collected values to the entry
          ##############################################################

          if { [lindex $buf $f_menu] > 0 } {
	     set w $filtFrame.r$row.m$name
             menubutton $w -menu $w.menu -text "+" -relief raised -bd 1 -padx 2
             set filtmn($name) [menu $w.menu]

             pack $w -side left 

             UifShortHelp::add $shArea $w \
                               "Menu of collected Values for $hlpText"
             UifShortHelp::add $shArea $filtmn($name) \
                               "Copy selected Value to the Entry"
	  }
       } ; # End foreach ...    
   } ; # End foreach on row ...

 
    logApplyFilter ;# Apply filter according to the value of filterFlag
}


proc logSetType { type } {
   global wname logType msgName env USER_LOG_PATH monType

   if {$type == "Normal"} {
      set logType "Normal Logs"
      set msgName logFile
   } elseif {$type == "Automatic"} {
      set logType "Automatic Logs"
      set msgName logAuto
   } else {

      if {[set tmp [panSelectFile $USER_LOG_PATH "log*" -quit]] == ""} {
	  return 
      }

      # Redefine user directory according to the last selection 
      set USER_LOG_PATH [file dirname $tmp]

      set msgName [file tail $tmp]
      set logType "$msgName"\

      # Force monitoring type to 'inspect all'
      set monType INSPALL
   }

   $wname(LOGS) delete 0 end

   logUpdate
   logCheckSyslog
}


proc logStoreFile {} {
    global wname USER_LOG_PATH
    
    # We should first get the selected logs, otherwise the selection
    # is lost when choosing the file

    set itemList [$wname(LOGS) curselection]

    if {[set sfile [panSelectFile [pwd] *]] == ""} { return }

    if {[lempty $itemList]} {
	set firstLog 0
	set lastLog  [$wname(LOGS) size]
    } else {
	set firstLog [lindex $itemList 0]
	set lastLog  [lindex $itemList end]
    }
    
    set sh [open $sfile w ] ;  # Open and create a new file
    
    puts "firstLog  $firstLog  lastLog  $lastLog   $sfile"
    # Highlights selected items
    
    for {set idx $firstLog} {$idx <= $lastLog} { incr idx } {
	puts $sh "[$wname(LOGS) get $idx]"
    }
    
    close $sh;
    $wname(LOGS) selection clear 0 end

    set numLogs [expr {$lastLog - $firstLog + 1}]
    tk_dialog .dialog "Store Logs" "$numLogs LOGS stored in $sfile" info 0 OK
}


proc logMonitorPanel {} {
   global shArea logRelease CheckListEnable logFont env inspectLines
   global wname selectBox dispsw dspOffset fileReadMode

   if {[info exists env(HOST)]} {
      set tmp "VLT Log Monitor - @$env(HOST) - Release $logRelease"
   } else {
      set tmp "VLT Log Monitor - Release $logRelease"
   }
   wm title . $tmp
   wm resizable . 1 1     ;# Set window as resizable
#   wm minsize   . 780 420 ;# Set window minimum dimensions

   # Split panel into frames

   frame .topMenu ; frame .f1 ; frame .f2 ; frame .f3 ; 
   frame .f4 ;      frame .f5 ; frame .f6 -relief raised -bd 2 ; frame .sh

   pack .topMenu .f1 -fill x
   pack .f3          -fill both -expand 1
   pack .f5  -fill x
   pack .f6  -fill both
   pack .sh  -fill x
 
   set wname(LOGS)      ".f3.logBox"
   set wname(SCROLLX)   ".sx"
   set wname(SCROLLY)   ".f3.sy"

   #------------------------------------------
   #         Create Short Help Area
   #------------------------------------------

   set shArea .sh.help
   UifShortHelp $shArea
   pack $shArea -fill x

   #------------------------------------------
   #            Create Menu Bar
   #------------------------------------------

   logCreateMenuBar .topMenu

   #------------------------------------------
   #           Fill frame .f1
   #------------------------------------------

    set rbSTOP      .f1.rbSTOP
    set rbMONITOR   .f1.rbMONITOR
    set rbINSPALL   .f1.rbINSPALL
    set rbINSPECT   .f1.rbINSPECT
    set btREADFILE  .f1.readFile
    set wdgLINE     .f1.fileLines
    set wdgLOG_TYPE .f1.logType

    UifRadioButton $rbSTOP    -label STOP            -variable monType -onVal STOP    -command {after 500 logUpdate }
    UifRadioButton $rbMONITOR -label MONITOR         -variable monType -onVal MONITOR -command {after 500 logUpdate }
    UifRadioButton $rbINSPALL -label "INSPECT ALL"   -variable monType -onVal INSPALL -command {after 500 logUpdate }
    UifRadioButton $rbINSPECT -label "INSPECT  last" -variable monType -onVal INSPECT -command {after 500 logUpdate }
    UifEntry       $wdgLINE   -label "lines"         -variable inspectLines  -width 6 

#    UifOptionButton $btREADFILE -itemValue { 1 2 3 4 } -default 3 \
#          -itemList {"Read Errors Only" "Read FITS Only" "Read No Filter" "Read With Filter"} \
#          -variable fileReadMode -selectMode value -length 14\
#          -command {logStartReadFile}
#
    UifOutput $wdgLOG_TYPE -label "logFile : " -variable logType -color Blue -width 30 -font CourR12 \
              -frame 0 -labelFont HelvR12 -anchor w


    UifShortHelp::add $shArea $rbSTOP    "Stop reading new logs" 
    UifShortHelp::add $shArea $rbMONITOR "Display the logs currently generated by the logging system"
    UifShortHelp::add $shArea $rbINSPECT "Read a logFile and display all the logs"
    
    pack $rbSTOP $rbMONITOR    -side left -fill x -padx 5   -pady 5
    pack $rbINSPALL            -side left -fill x -padx 15  -pady 5
    pack $rbINSPECT            -side left -fill x -padx 5  -pady 5
    pack $wdgLINE              -side left -padx 2 -pady 5
#    pack $btREADFILE           -side left -fill x -padx 10  -pady 5
    pack $wdgLOG_TYPE          -side left -padx 20
    
    #------------------------------------------
    #           Fill frame .f2
    #------------------------------------------
    
    # The listbox LOG_TITLE is used as a trick to create a label
    # which can be scrolled with the listbox LOGS.
    
    scrollbar $wname(SCROLLY) -orient vertical   -command [list $wname(LOGS) yview]
    scrollbar $wname(SCROLLX) -orient horizontal -command [list $wname(LOGS) xview]

    set clist [list 0 "Date" 0 "Time" 0 "Env. Name" 0 "Module" 0 "Proc. Name" \
		   0 "Proc. ID"  0  "Log. ID" 0  "Log Text" ] 

    tablelist::tablelist $wname(LOGS) -columns $clist \
                          -labelcommand tablelist::sortByColumn        \
                          -selectbackground navy     \
                          -yscrollcommand [list $wname(SCROLLY) set]   \
                          -xscrollcommand [list $wname(SCROLLX) set]   \
                          -setgrid yes -stretch 7 -showseparators yes  \
                          -stripebg Gray80 \
                          -fg Blue -font "-Adobe-courier-medium-r-normal--12*-iso8859-1" 

    for {set cidx 0} {$cidx < 8} {incr cidx} {
	$wname(LOGS) columnconfigure $cidx -foreground navy
    }

    logCheckAllFields ;# Hide default columns

    pack $wname(LOGS)      -side left  -fill both -anchor w -expand yes
    
    pack $wname(SCROLLY)   -side right -fill y
    pack $wname(SCROLLX)   -fill x    -anchor w -after .f3

    UifShortHelp::add $shArea $wname(LOGS)  "Copy-selection-to-filter:\
	<1> Select  <Dbl-1> Copy-start-date <Dbl-2> Copy-end-date\
	<Dbl-3> Copy-rest"
    
    # Define specific bindings

    $wname(LOGS) configure -selectmode extended

    bind [$wname(LOGS) bodypath] <Double-Button-1> "copyDateTime start; break"
    bind [$wname(LOGS) bodypath] <Double-Button-2> "copyDateTime final; break"
    bind [$wname(LOGS) bodypath] <Double-Button-3> "logPresetFilterFromSelection ;break"

    #------------------------------------------
    #            Create Button Panel
    #------------------------------------------

    set filtFrame [frame .f5.filtFrame ]
    pack $filtFrame -padx 5 -pady 5 -fill x
    
    #------------------------------------------
    #            Filter Frame
    #------------------------------------------
    
    set f0 [frame $filtFrame.r0]
    set f1 [frame $filtFrame.f1 -relief groove -bd 2]
    
    pack  $f0 $f1 -fill x
    
    button $f0.clear        -text "Clear Screen"      -width 12 -command logClear
    button $f0.showErr      -text "Show Error Stack"  -width 16 -command logShowErrStack
#    button $f0.extractFits  -text "Extract FITS ..."     -command {exec logFitsDisplay &}

    checkbutton $f0.notFITS  -text "NO FITS"   -variable ckFitsFilter \
       -offvalue 0 -onvalue 2 -command {logFilterFits }
        
    pack $f0.clear $f0.showErr   -side left -pady 5 -padx 3

#    pack $f0.clear $f0.showErr $f0.extractFits   -side left -pady 5 -padx 3
    pack $f0.notFITS -side left -pady 5 -padx 7
    
    checkbutton $f0.autoScroll -text "Auto. Scroll" -variable logAutoScroll
    checkbutton $f0.filter     -text "Filter"       -variable filterFlag \
                -command {after 200 logApplyFilter }
    label $f0.status -textvariable filterStatus -width 9

    pack $f0.autoScroll $f0.filter -side left -padx 15
    pack $f0.status -side left -padx 1

    set r0 [frame $f1.r0]
    set r1 [frame $f1.r1]
    
    pack $r0 $r1 -fill x
    
    label $r0.lbl -text "Filter Selection :  "
    pack  $r0.lbl -side left -padx 5
    
    createFilterBox  $f1 $shArea
   
    set logFont medium; logSetListFont

    # Set Short Helps


}

proc logEnableStrip {} {
    global logStripFlag wname

    set w $wname(LOGS)

    if {[winfo exist $w]} {
	if {$logStripFlag} {
	    $w configure -stripebg lightyellow -background white
	} else {
	    $w configure -stripebg Gray80 -background Gray80
	}
    }
}


proc createHelpOnError { helpFrame } {
   global errMemo wname

   set top [frame $helpFrame.top] ; pack $helpFrame.top -fill both -pady 5
   set f1  [frame $top.f1]
   set f2  [frame $top.f2]
   set wname(HELP_TEXT) $top.f2.txt
   set wdgHelpFile $f1.fileName 

   pack $f1 -pady 3 -fill x
   pack $f2 -pady 3 -fill both

   label $f1.lbl1 -anchor nw -foreground black -text "On-Line Help On Error :"
   label $f1.fileName -textvariable errMemo

   pack $f1.lbl1  $f1.fileName -side left -padx 5 

   # Create text widget with scroll bars

   text $wname(HELP_TEXT) -width 80 -height 8 -setgrid false -wrap none \
      -borderwidth 2 -relief sunken
   
   scrollbar $f2.scrollv -orient v -borderwidth 2 -width 10\
             -command "$wname(HELP_TEXT) yview" -relief sunken

   scrollbar $top.scrollh -orient h -borderwidth 2 -width 10\
                -command "$wname(HELP_TEXT) xview" -relief sunken

   $wname(HELP_TEXT) configure -yscrollcommand "$f2.scrollv set" \
	             -xscrollcommand "$top.scrollh set"

   pack $wname(HELP_TEXT) -side left -fill both -expand 1  
   pack $f2.scrollv  -side left -fill y
   pack $top.scrollh -fill x
}

proc checkHelpOnError { helpFrame } {
    global logHelpOnError wname
    
    if {$logHelpOnError} {
	if {![winfo exists $helpFrame.top]} {
	    createHelpOnError $helpFrame
	    bind [$wname(LOGS) bodypath] <ButtonRelease-1> "logDisplayErrHelp"
	}
	#      wm minsize   . 780 610
	pack $helpFrame -after .f5 -fill x
    } else {
	#      wm minsize   . 780 420
	if {[winfo exists $helpFrame.top]} { 
	    pack forget $helpFrame
	    bind [$wname(LOGS) bodypath] <ButtonRelease-1> ""
	}
    }
    
   update idletasks
}

proc logDisplayErrHelp {} {
    global wname logHelpOnError errMemo textIdx

    if {!$logHelpOnError} { return }

    $wname(HELP_TEXT) delete 0.0 end
    
    set idx [$wname(LOGS) curselection]
    
    # Get error text directly from the table cell where it is contained

    set logText [$wname(LOGS) cellcget $idx,7 -text]

    if {[scan $logText {%s %d %d %d %s %s %n} procName errNumb stack seqNumb errSev errMemo si] < 7} {
	tk_dialog .dialog "Failure" "Cannot Retrieve Error Stack : NOT ERROR LOG" info 0 OK
	return
    }

    if {![cequal $errMemo ""] } {
	set buf [::internal::errReadHelpFile ${errMemo}.hlp "r"]
	set buf [string trim $buf " \n"]
    
	if {![cequal $buf ""]} {
	    $wname(HELP_TEXT) insert 0.0 $buf
	}
    }
}







# ___oOo___
