#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: logMonitorProcs.tcl,v 4.26 2005/10/24 07:59:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/03/96  created
#           13/05/96  Add check if file.bck exists
#           30/07/96  Fixed bug in checking when to re-open log file
#           01/08/96  Add logSetFilter
#           19/09/97  Add check on max. indentation value
#           30/10/97  Changed from awk to gawk and add space at the end of
#                     the log line otherwise we loose last character on SUN
#           19/05/99  Better implementation to check is a backup took place
#                     in order to re-open the logFile.
# bgilli    25/02/00  Removed trailing r "option" when collecting last logs.
# tebert    17/07/03  removed processing with awk for each line
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc logFilterErrorOnly {} {
    global ckErrOnly filterFlag ckFitsFilter

    if {$ckErrOnly} { 
	set filterFlag   1
	set ckFitsFilter 0
	logApplyFilter
    }
}



proc logFilterFits {} {
    global ckFitsFilter filterFlag ckErrOnly

    if { $ckFitsFilter == 2 } {
	# FITS Filter "NO FITS"
	set ckErrOnly  0
	set filterFlag 1;
	logApplyFilter

    } elseif {$ckFitsFilter == 1} {

	# FITS Filter "FITS ONLY"
	set ckErrOnly  0
	set filterFlag 1;
	logApplyFilter

    } 
}

#
# Update all selections and modes
#

proc logUpdate {} {
    global wname monType msgName CheckListIndex USER_LOG_PATH 
    global logMonitorInit inspectLines HOME env logDialog
    
    set logDialog .logInspectDialog

    logDeleteTailFile ;# Delete temporary tail file !!
    
    logCloseFile
    logSetupAwk
        
    if {[cequal $monType MONITOR]} {
	set startFlag 2
	logOpenFile $env(VLT_LOG_FILES)/$msgName
	logCheckSyslog

    } else {

	# Process queue of X events and check if user clicked on "STOP"
	if {[cequal $monType STOP]} { 
	    update
	    return 
	}

	set logMonitorInit 0

	if {[winfo exists $logDialog]} { destroy $logDialog}

	UifWaitDialog $logDialog -refValue 100 -length 300 -step 5\
	    -text "              Cleaning  logMonitor ..."
	
	logClear
	logClearFilterMenu
	logOpenFile     "$USER_LOG_PATH/$msgName"
	
	if {[cequal $monType INSPALL]} {
	    # Read the complete file
	    logFilterOnFile "$USER_LOG_PATH/$msgName"
	} else {
	    # Read the last inspectLines of the file
	    logFilterOnFile "$USER_LOG_PATH/$msgName" $inspectLines
	}
    }
    
    set CheckListIndex 0
}


proc logDeleteTailFile {} {
   global env

   set tmp $env(VLT_LOG_FILES)/logTailFile_[pid] 
   if {[file exists $tmp]} { exec rm -f $tmp }
}

#
# Close current syslog file
#

proc logCloseFile {} {
   global msgHandle debug

   if {$debug} { puts "DBG> logCloseFile msgHandle = $msgHandle" }
   catch {close $msgHandle};  #  Close file
   set msgHandle ""
}


#
# Open current syslog file
#

proc logOpenFile { fileName {option ""} } {
    global wname shArea  monType logMonitorInit USER_LOG_PATH
    global msgHandle msgFile debug logFileSize

    set msgFile $fileName

#    puts "DBG> logOpenFile msgFile = $msgFile  option = $option" 
#    puts "DBG> USER_LOG_PATH = $USER_LOG_PATH  fileName = $fileName" 

    logCloseFile ;# Force closing of the file !!!

    if {[catch {set msgHandle [open $msgFile r]}]} {
        puts "DBG> Could not open file : $msgFile"
	$shArea displayText "Could not open file : $msgFile"
	return
    }

    # Only at the beginning go to the end of the file.
    # In case of backup ... start from the beginning.

    if {! [cequal $monType MONITOR]} { return }

    if {$logMonitorInit == 0 } {

       set logFileSize [file size $msgFile]

       if {$debug} { puts "DBG> Collect last n-th lines" }
       # Collect the last 'height' logs

#       puts "logOpenFile   exec tail -[$wname(LOGS) cget -height] $msgFile "
       set tmpBuf  [exec tail -[$wname(LOGS) cget -height] $msgFile] 

       foreach line [split $tmpBuf "\n"] {
             logAddLine $line 
       }

       set logMonitorInit 1
    }

    if {![cequal $msgHandle ""]} {
       if {[cequal $option START] } {
          seek $msgHandle 0 start ;# Set bytes offset to the beginning
       } else {
          seek $msgHandle 0 end ;# Set bytes offset to the end of the file
       }
    }

    return
}



#
# check if there is anything to read from msgHandle
# Runs in the background as a lightweight process
#
proc logCheckSyslog { } {
    global monType msgHandle msgFile logFileSize msgName logMonitorInit gawkOpen
    global env

#  puts "logCheckSyslog  msgHandle($msgHandle)  $msgFile"

    # Check if we are MONITORing the log file
    if { [cequal $monType MONITOR] == 0 } { update; return }

    # Check if the file handler is valie
    if { [cequal $msgHandle ""]}          { update; return }

    set tmp [file size $msgFile]

    # If size is less than before , then a backup took place ..
    # therefore re-open the logFile

    if { $tmp < $logFileSize } {

       # If the logFile has been saved into the backup copy, re-open the file.
       logOpenFile $env(VLT_LOG_FILES)/$msgName START
#      puts "DBG > Re-Open logFile offset([tell $msgHandle])" 
    }

    set logFileSize $tmp
    set nc [gets $msgHandle line]

    while {$nc > 0 } {
       # Process the line and display in the listbox

	if { [cequal $monType MONITOR] } { logAddLine $line }

       # Now update the screen to display lines one by one,
       # otherwise no action would be possible until all new lines 
       # are read!

       update  ;# dont use 'update idletasks'

       if {![cequal $msgHandle ""]} {
           set nc [gets $msgHandle line]
       } else {
           set nc -1
       }
    }

#   puts "DBG > CheckSyslog Done  File Size $logFileSize "
    after 2000 logCheckSyslog ;    # re-execute later ..
    update 
    return
}

# This procedure takes a new log line and process it according to the current
# filtering criteria. If the current conditions are fulfilled the new log is 
# added to the listbox.

proc logAddLine { origLine } {
    global wname ckErrOnly logAutoScroll monType env
    global selectBox dispsw filterVal filtsw filterFlag ckFitsFilter

#    puts "logAddLine $origLine"

    # Strip off the first fields of the log which are added by the system

    ctoken origLine " " ;# Strip month
    ctoken origLine " " ;# Strip day
    ctoken origLine " " ;# Strip time
    ctoken origLine " " ;# Strip host
    ctoken origLine " " ;# Strip 'syslog:' or equivalent keyword

    if { $env(UNAMES) == "SunOS" } {
        if { $env(UNAMER) >= "5.8" } {
	    # Strip [ID ....] string added by syslog of Solaris 5.8  (JOT. 15-Mar-2002)
            ctoken origLine "\]"
            ctoken origLine " "  ;# Strip last closing brace
        }
    }

    # Check if line is a Fits log : in this case reformat the log
    # according to all others by inserting dummy values '-' or '0'

    set fitsTime    "HH:MM:SS"
    set si          end 

    scan $origLine {%s %9s %n} wsEnvName fitsTime numChars 
    update

    set part(0) ""; set part(1) "" ; set part(2) ""; set part(3) ""
    set part(4) ""; set part(5) "" ; set part(6) ""; set part(7) ""

    if {[regexp {([0-9]+):([0-9]+):([0-9]+)>} $fitsTime all] } {

        # Fits timestamp can have two formats :
        # 18:29:49>-ACTION CAT
        # 18:32:36> CAT SYSTEM INT16

	set fitsLogFlag 1

	# If the user DOES NOT wants to see FITS logs .. do not add log to table
	if {$ckFitsFilter == 2} { return ""} 

        set ft      [string trimright $fitsTime " >" ]
        set logLine "FITS_LOG $ft $wsEnvName - - 0 0 [string range $origLine $numChars end]"
        scan $logLine {%s %s %s %s %s %s %s %n} part(0) part(1) part(2) part(3) \
	           part(4) part(5) part(6) si
        set part(7) [string range $logLine $si end]

    } else {

	# If the user wants to see only FITS logs .. do not add log to table
	if {$ckFitsFilter == 1} { return ""} 

	set fitsLogFlag 0
	set logLine     $origLine
        scan $logLine   {%s %s %s %s %s %s %s %n} part(0) part(1) part(2) part(3) \
	                 part(4) part(5) part(6) si

        set part(7) [string range $logLine $si end]
    }

    # Filter current log line according to the filtering criteria set by the user

    if {$filterFlag} {

	# The following filters do not apply to FITS logs

	if {$fitsLogFlag == 0} {

            set datetime [format "%s %s" $part(0) $part(1)]
#	    puts "$datetime    --> filterVal(date1new)=$filterVal(date1new)"

	    if { $filtsw(date1) && $filterVal(date1new) != "" } {
                if { "$datetime" <= "$filterVal(date1new)" } {
                    return ""
		}
	    }

#	    puts "$datetime    --> filterVal(date2new)=$filterVal(date2new)"
	    if { $filtsw(date2) && $filterVal(date2new) != "" } {
                if { "$datetime" >= "$filterVal(date2new)" } {
                    return ""
		}
	    }
	}

#	puts "filtsw(env)= $filtsw(env)  filterVal(env)=$filterVal(env) val=$part(2)"
	if { $filtsw(env) && $filterVal(env) != "" } {
	    if { "$filterVal(env)" !=  "$part(2)" } {
		return ""
	    }
	}

#	puts "filtsw(mod)= $filtsw(mod) val=$part(3)"
	if { $filtsw(mod) && $filterVal(mod) != "" } {
	    if { "$filterVal(mod)" !=  "$part(3)" } {
		return ""
	    }
	}

#	puts "filtsw(proc)= $filtsw(proc) val=$part(4)"
	if { $filtsw(proc) && $filterVal(proc) != "" } {
	    if { "$filterVal(proc)" !=  "$part(4)" } {
		return ""
	    }
	}

#	puts "filtsw(pid)= $filtsw(pid)  filterVal(pid)=$filterVal(pid) val=$part(5)"
	if { $filtsw(pid) && $filterVal(pid) != "" } {
	    if { "$filterVal(pid)" !=  "$part(5)" } {
		return ""
	    }
	}

#	puts "filtsw(text)= $filtsw(text)  filterVal(text)=$filterVal(text)"

	if { $filtsw(text) && $filterVal(text) != "" } {
	    set val [string first "$filterVal(text)" "$logLine"]
	    if  { $val == -1 } {
		return ""
	    }
	}
	if { $filtsw(textn) && $filterVal(textn) != "" } {
	    set val [string first "$filterVal(textn)" "$logLine"]
	    if  { $val != -1 } {
		return ""
	    }
	}
    }

    logAddLineToTable $fitsLogFlag $part(0) $part(1) $part(2) \
	              $part(3) $part(4) $part(5) $part(6) $part(7)

    return
}


# This procedure process a log read from a file produces by the "INSPECT" function

proc logAddLineFromFile { origLine } {
    global wname ckErrOnly logAutoScroll monType env filterFlag
    global selectBox dispsw filterVal filtsw

#    puts "logAddLineFromFile $origLine"

    # The line produced by the awk script is already formatted in such
    # a way that it is ready to be added to the table list.

    set part(7) ""
    set si end

    set numTokens [ scan $origLine {%s %s %s %s %s %s %s %n} part(0) part(1) part(2) part(3) \
			part(4) part(5) part(6) si ]

    if {$numTokens < 7} {
	for {set i $numTokens} {$i < 7} {incr i} {
            set part($i) ""
	}
    } else {
	set part(7) [string range $origLine $si end]
    }

    if {[cequal $part(0) "FITS_LOG"]} {
	set fitsLogFlag 1
    } else {
	set fitsLogFlag 0
    }
    
    if {$filterFlag} {
	
	# The following filters do not apply to FITS logs
	
	if {$fitsLogFlag == 0} {
	    
            set datetime [format "%s %s" $part(0) $part(1)]
	    #	    puts "$datetime    --> filterVal(date1new)=$filterVal(date1new)"
	    
	    if { $filtsw(date1) && $filterVal(date1new) != "" } {
                if { "$datetime" <= "$filterVal(date1new)" } {
                    return ""
		}
	    }
	    
	    #	    puts "$datetime    --> filterVal(date2new)=$filterVal(date2new)"
	    if { $filtsw(date2) && $filterVal(date2new) != "" } {
                if { "$datetime" >= "$filterVal(date2new)" } {
                    return ""
		}
	    }
	}
	
	#	puts "filtsw(text)= $filtsw(text)  filterVal(text)=$filterVal(text)"
	
	if { $filtsw(text) && $filterVal(text) != "" } {
	    set val [string first "$filterVal(text)" "$origLine"]
	    if  { $val == -1 } {
		return ""
	    }
	}
	
	if { $filtsw(textn) && $filterVal(textn) != "" } {
	    set val [string first "$filterVal(textn)" "$origLine"]
	    if  { $val != -1 } {
		return ""
	    }
	}
    }

    logAddLineToTable $fitsLogFlag $part(0) $part(1) $part(2) \
	              $part(3) $part(4) $part(5) $part(6) $part(7)

    return
}



proc logAddLineToTable { fitsLogFlag arg0 arg1 arg2 arg3 arg4 arg5 arg6 logText} {

    global wname ckErrOnly monType logAutoScroll filtsw filterVal CheckListEnable

#    puts "$fitsLogFlag $arg0 $arg1 $arg2 $arg3 $arg4 $arg5 $arg6 $logText"
    
    set redFont 0

    if {$fitsLogFlag} {
	if {$ckErrOnly} { return "" }
    } else {
	set seqNumb ""

	# Check if this is an error log

	if {[scan $logText {%s %d %d %d %n} procName errNumb stack seqNumb si] < 5} {
	    set seqNumb ""
	} else {
	    set errorLog [string range $logText $si end]
	}

	if {$ckErrOnly && [cequal $seqNumb ""]} { return "" }

	if {[cequal $seqNumb ""] == 0} {

	    if {$seqNumb == 1 } { set redFont 1 }

	    # This part performs an indentation of the error logs according
	    # to the error sequence number -i.e.- the bigger the sequence number
	    # the more indented the error text is.
	    
	    if {$seqNumb > 30} { set seqNumb 1 }
	    
	    set    logText [format "%-.80s %4d %8d" $procName $errNumb $stack]
	    append logText [replicate "  " $seqNumb]
	    append logText "$seqNumb $errorLog"
	}
    }

    # Concatenate tokens to get the final formatted log

    lappend line $arg0 $arg1 $arg2 $arg3 $arg4 $arg5 $arg6 $logText

    $wname(LOGS) insert end $line

    if {$redFont} {
	$wname(LOGS) rowconfigure end -foreground indianRed
    }

    if {$CheckListEnable} {
	logCheckListbox $fitsLogFlag $arg0 $arg1 $arg2 $arg3 $arg4 $arg5 $arg6 $logText
    }

    if {[cequal $monType MONITOR]} {
	# Delete previous lines only if we are 'MONITORing' the logFile
        # Otherwise we display the whole 'INSPECTing' file
       
	if {[set tmp [$wname(LOGS) size]] > 200} {
	    $wname(LOGS) delete 0 [expr {$tmp - 100}]  
	}

	if {$logAutoScroll} { $wname(LOGS) see end }
    }

    return
}


proc logSetFilter { key value } {
   global filtsw filter

   if {[info exists filtsw($key)] && [info exists filter($key)]} {
       set filtsw($key) 1
       set filter($key) $value
   }
}

#
# ___oOo___












