#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCHwInit.tcl,v 2.43 2005/04/19 12:32:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    13/03/97  defined gvar(relPath) to support dbTableDump
# imunoz    04/03/97  uses moteiDbFancyTableWdg to interact with the DB,
#                     lot of code saved (and deleted).
# imunoz    02/05/96  changed panel name to moteiCHIni
# imunoz    30/04/96  itcl2.0 port
# imunoz    11/12/95  Changed twoStepDelta to 8 due to new posType HOME
# imunoz    11/23/95  created for cmm
#

#************************************************************************
#   NAME
#       moteiCHwInit - configure the HW Initialization sequence
#
#   SYNOPSIS
#       moteiCHwInit <args>
#
#   DESCRIPTION
#       The panel moteiCHwInit.pan contains a set of widgets allowing the 
#       configuration of the HW Initialization sequence. 
#       The file moteiCHwInit.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(l), moteiConfig(l)
#       moteiCHwInit(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
#-----------------------------------
# Init, widgets & DB procedures
#-----------------------------------
proc moteiCHwInitReadConfig { {mode 0} } {
    global gvar moteiConfigParams
    global moteiIA

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set moteiIA $moteiConfigParams(actions)
    }
}

proc moteiCHwInitWriteConfig { } {
    global gvar moteiIA moteiIAL

    set i "$gvar(dbPath):SERVER:INIT"

    set ian [llength $moteiIAL] ; set attr "actions"
    for {set n 0} {$n < $ian} {incr n} {
	set ia [lindex $moteiIA $n]
	if {[catch {seq_dbWriteSymbolic "$i.${attr}($n)" $ia} ret] != 0} {
	    seq_errDisplay -suspend
	}
    }
    moteiConfigReinstall
}

proc moteiCHwInitConfigWidgets { } {
    global gvar cv moteiConfigParams moteiIA moteiIAL moteiIAIdx 
    global actionFrame

    set moteiIAIdx 0
    moteiCHwInitUpdate
    moteiCHwInitSelect -1
}
    
proc moteiCHwInitInitPanel { } {

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:INIT
    moteiCHwInitReadConfig

    # configure widgets
    moteiCHwInitConfigWidgets
}


#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCHwInitApply { } {
    global moteiIAIdx

    moteiCHwInitApplyRow
    moteiCHwInitWriteConfig
    moteiCHwInitSelect [expr {$moteiIAIdx + 1}]
}

proc moteiCHwInitReset { } {
    global moteiConfigBackup moteiIA

    set moteiIA $moteiConfigBackup(actions)

    moteiCHwInitConfigWidgets
    moteiCHwInitWriteConfig
}

proc moteiCHwInitRefresh { } {
    moteiCHwInitReadConfig 1
    moteiCHwInitConfigWidgets
}

#-----------------------------------
# Action selection procedures
#-----------------------------------
proc moteiCHwInitSelectAction { actionType } {
    global gvar cv actionFrame moteiConfigParams

    if {$moteiConfigParams(axisType) == 1} {
	$cv.moteiCHwInit-AT enableEntry 1
	$cv.moteiCHwInit-AT enableEntry 2
    } else {
	$cv.moteiCHwInit-AT disableEntry 1
	$cv.moteiCHwInit-AT disableEntry 2
	if {$actionType == 3 || $actionType == 4} { set actionType 5 }
    }
    if {$moteiConfigParams(amplifier) == 1} {
	$cv.moteiCHwInit-AT enableEntry  6
	$cv.moteiCHwInit-AT enableEntry  7
    } else {
	$cv.moteiCHwInit-AT disableEntry 6
	$cv.moteiCHwInit-AT disableEntry 7
	if {$actionType == 7 || $actionType == 11 } { set actionType 10 }
    }
    if {$moteiConfigParams(brakeAction) == 1 && \
	$moteiConfigParams(digitalIO) == 1 } {
	$cv.moteiCHwInit-AT enableEntry  4
	$cv.moteiCHwInit-AT enableEntry  5
    } else {
	$cv.moteiCHwInit-AT disableEntry 4
	$cv.moteiCHwInit-AT disableEntry 5
	if {$actionType == 6 || $actionType == 12 } { set actionType 10 }
    }
    set gvar(actionType) $actionType

    set show 1
    switch $actionType {
	 2 { set actionFrame 0 }
	 3 -
	 4 -
	 5 { set actionFrame 1 }
	 6 -
	 12 -
	 7 -
	 11 { set actionFrame 2 }
	 8 { set actionFrame 3 }
	 9 { set actionFrame 4 }
	10 { set actionFrame 5 }
	default {
	    set show 0
	}
    }
    $cv.actionFrame configure -show $show
    $cv.actionFrame show

    set w 512 ; set xoff 2; set yoff 16
    set x0 $xoff; set x1 [expr {$xoff + $w}]
    set W [$cv.actionFrame getClassPath]
    switch $actionFrame {
	0 { # Motion
	    set h 44; incr x0 38; incr x1 -254
	    drawSquare $W.cv $x0 $x1 $w $h $xoff $yoff Blue 1
	    set h 88; set yoff 80
	    drawSquare $W.cv $x0 $x1 $w $h $xoff $yoff Blue 1
	    set yoff 184; incr x1 56

	    moteiCHwInitSelectMotionMode $gvar(motionMode)
	}
	1 { # Position
	    set h 128; set yoff 48; incr x0 38; incr x1 -118
	    if {$actionType == 5} {
		set gvar(actionClass_text) "Calibrate the Axis"
	    } else {
		if {$actionType == 3} { set lim "Low" } else { set lim "Upp" }
		set gvar(actionClass_text) "Set the ${lim}er SW Limit"
	    }
	    moteiCHwInitSelectPosType $gvar(posType)
	}
	2 { # Action 
	    set h 28; incr x0 22; incr x1 -22
	    if {$actionType == 6} {
		incr x0 64; incr x1 -64
		set gvar(actionClass_text) "Clamp the Brake"
	    } elseif {$actionType == 7} {
		incr x0 24; incr x1 -24
		set gvar(actionClass_text) "Disconnect the Power Relays"
	    } elseif {$actionType == 11} {
		incr x0 30; incr x1 -30
		set gvar(actionClass_text) "Connect the Power Relays"
	    } elseif {$actionType == 12} {
		incr x0 58; incr x1 -58
		set gvar(actionClass_text) "Unclamp the Brake"
	    }
	}
	3 { # User Function
	    set h 40; incr x0 58; incr x1 -62
	    set gvar(actionClass_function) $gvar(function)
	}
	4 { # On-Board Proc
	    set h 40; incr x0 22; incr x1 -22
	    set gvar(actionClass_initCode) $gvar(initCode)
	}
	5 { # Delay
	    set h 40; incr x0 102; incr x1 -102
	    set gvar(actionClass_delay) $gvar(initCode)
	}
    }    
    drawSquare $W.cv $x0 $x1 $w $h $xoff $yoff Blue 1
    update idletasks
}

proc moteiCHwInitSelectMotionMode { motionMode } {
    global gvar cv moteiConfigParams actionFrame

    if {$actionFrame != 0} return

    set amp $moteiConfigParams(amplifier)
    set ivl [list 3 5]
    set w [$cv.actionFrame getClassPath]
    if {($moteiConfigParams(axisType) == 1) && ($amp == 1)} { 
	$w.cv.moteiInitMotion-MM enableEntry 0
	lappend ivl 2
    } else {
	if {$motionMode == 2} { set motionMode 5 }
	$w.cv.moteiInitMotion-MM disableEntry 0
    }
    if {[lsearch $ivl $motionMode] == -1} { set motionMode 5 }

    set gvar(motionMode)             $motionMode
    set gvar(actionClass_motionMode) $motionMode
    moteiCHwInitSelectPosType        $gvar(posType)
}

proc moteiCHwInitSelectPosType { posType } {
    global gvar cv moteiConfigParams actionFrame

    if {$actionFrame != 0 && $actionFrame != 1} return

    set W [$cv.actionFrame getClassPath]
    if {$actionFrame == 0} {
	# Motion
	set mm $gvar(motionMode)
	set ptl {} ; set ivl {}
	if {$mm == 3} {
	    lappend ptl "to Absolute position" "relative to current position"
	    lappend ivl 1 2
	    if {$moteiConfigParams(axisType) == 1} { 
		lappend ptl "relative to Lower HW limit" \
			    "relative to Upper HW limit"
		lappend ivl 3 4
	    } 
	    lappend ptl "relative to Reference Switch" \
		        "relative to Index Pulse" \
		        "relative to Home Position" \
		        "in Two Step Absolute" \
		        "in Two Step Relative" 
	    lappend ivl 5 6 7 9 10
	    if {$moteiConfigParams(axisType) != 1} { 
		lappend ptl "Half-Turn Absolute" "Half-Turn Relative"
		lappend ivl 11 12
	    } 
	    lappend ptl "Off Reference Switch" 
	    lappend ivl 261
	} else {
	    if {$moteiConfigParams(axisType) == 1} { 
		lappend ptl "to Lower HW limit" "to Upper HW limit"
		lappend ivl 3 4
	    } 
	    if {$mm == 5} {
		lappend ptl "to Reference Switch" \
			    "to Index Pulse" \
			    "to Home Position"
		lappend ivl 5 6 7
	    }
	}
	$W.cv.moteiInitMotion-PT configure -itemList $ptl -itemValue $ivl
    } else { 
	# Position
	set ptl {} ; set ivl {}
	lappend ptl "absolute" "relative to current position"
	lappend ivl 1 2
	if {$moteiConfigParams(axisType) == 1} { 
	    lappend ptl "relative to Lower HW limit" \
		        "relative to Upper HW limit"
	    lappend ivl 3 4
	} 
	lappend ptl "relative to Reference Switch" \
		    "relative to Index Pulse" \
		    "relative to Home Position"
	lappend ivl 5 6 7
	
	$W.cv.moteiInitPosition-PT configure -itemList $ptl -itemValue $ivl
    } 
    if {[lsearch $ivl $posType] == -1} {
	set posType [lindex $ivl 0]
    }

    set gvar(posType)             $posType
    set gvar(actionClass_posType) $posType

    if {$actionFrame == 0} {
	# Motion
	moteiCHwInitSelectSpdHow $gvar(spdHow)
	moteiCHwInitSelectPosHow $gvar(posHow)
    } else { 
	# Position
	moteiCHwInitSelectPosHow $gvar(posHow)
    } 
}

proc moteiCHwInitSelectSpdHow { spdHow } {
    global gvar cv moteiConfigParams 
    global actionFrame speedFrame

    if {$actionFrame != 0} return
    set W [$cv.actionFrame getClassPath]

    set NSL $moteiConfigParams(namedSpdList)
    set nsLen [llength $NSL]
    set ISL $moteiConfigParams(indexSpdList)
    set SUL $moteiConfigParams(spdUnit)
    set DSI $moteiConfigParams(defaultSpeed)
    set DSN [lindex $NSL $moteiConfigParams(defaultSpeed)]

    if {[lsearch [list 1 2 3 4] $spdHow] == -1} { set spdHow 2 }

    if {$moteiConfigParams(defaultSpeed) == -1} {
	$W.cv.moteiInitMotion-SpdHow disableEntry 0
    } else {
	$W.cv.moteiInitMotion-SpdHow enableEntry  0
    }
    $W.cv.moteiInitMotion-SpdHow enableEntry 1
    if {$nsLen == 0} {
	$W.cv.moteiInitMotion-SpdHow disableEntry 0
	$W.cv.moteiInitMotion-SpdHow disableEntry 2
	$W.cv.moteiInitMotion-SpdHow disableEntry 3
	set spdHow 2
    } else {
	$W.cv.moteiInitMotion-SpdHow enableEntry  0
	$W.cv.moteiInitMotion-SpdHow enableEntry  2
	$W.cv.moteiInitMotion-SpdHow enableEntry  3
    }
    $W.cv.moteiInitMotion-SpdHow enable
    $W.cv.moteiInitMotion-Speed  configure -show 1

    set gvar(actionClass_spdHow) $spdHow
    set gvar(spdHow) $spdHow

    set speedFrame [expr {$spdHow - 1}]
    set w [$W.cv.moteiInitMotion-Speed getClassPath]
    # Configure option buttons with corresponding lists
    switch $spdHow {
	1 { # Configure default speed
	    set gvar(speedClass_defaultSpdIndex) $DSI
	    set gvar(speedClass_defaultSpdName)  $DSN
	    set DSV $moteiConfigParams(spdDefault)
	    set DSU $moteiConfigParams(spdDefaultUnit)
	    set gvar(speedClass_defaultSpdValue) "$DSV $DSU"
	}
	2 { # Configure list of units
	    $w.cv.valueUnit configure -itemList $SUL
	    if {[lsearch $SUL $gvar(spdUnit)] == -1} {
		set gvar(spdUnit) [lindex $SUL 0]
	    }
	    set gvar(speedClass_valueUnit) $gvar(spdUnit)
	    set gvar(speedClass_value)     $gvar(spdValue)
	}
	3 { # Configure list of names
	    $w.cv.nameList configure -itemList $NSL
	    if {[lsearch $NSL $gvar(spdName)] == -1} {
		set gvar(spdName) [lindex $NSL 0]
	    }
	    set gvar(speedClass_nameValue) $gvar(spdName)
	}
	4 { # Configure list of index
	    $w.cv.indexList configure -itemList $ISL
	    if {[lsearch $ISL $gvar(spdIndex)] == -1} {
		set gvar(spdIndex) [lindex $ISL 0]
	    }
	    set gvar(speedClass_indexValue) $gvar(spdIndex)
	}
    }
    $W.cv.moteiInitMotion-Speed show
}

proc moteiCHwInitSelectPosHow { posHow } {
    global gvar cv moteiConfigParams 
    global actionFrame positionFrame offsetFrame

    if {$actionFrame != 0 && $actionFrame != 1} return

    set W [$cv.actionFrame getClassPath]

    if {$actionFrame == 0} {
	set what "Motion"
    } else {
	set what "Position"
    }

    set NPL $moteiConfigParams(namedPosList) ; set NP0 [lindex $NPL 0]
    set npLen [llength $NPL]
    set IPL $moteiConfigParams(indexPosList) ; set IP0 [lindex $IPL 0]
    set PUL $moteiConfigParams(posUnit) ; set PU [lindex $PUL 0]

    if {[lsearch [list 2 3 4 5 6] $posHow] == -1} { set posHow 2 }
    if {$actionFrame == 0 && $gvar(motionMode) != 3} {
	$W.cv.moteiInit${what}-PosHow disable
	$W.cv.moteiInit${what}-Target configure -show 0
	$W.cv.moteiInit${what}-Offset configure -show 0
    } else {
	$W.cv.moteiInit${what}-PosHow enableEntry 0
	$W.cv.moteiInit${what}-PosHow enable
	$W.cv.moteiInit${what}-Target configure -show 1
    }

    if {$gvar(posType) != 1 && $gvar(posType) != 9} { 
	$W.cv.moteiInit${what}-PosHow disableEntry 1
	$W.cv.moteiInit${what}-PosHow disableEntry 2
	$W.cv.moteiInit${what}-PosHow disableEntry 3
	$W.cv.moteiInit${what}-PosHow disableEntry 4
	set posHow 2
    } else {
	$W.cv.moteiInit${what}-PosHow enableEntry  1
	$W.cv.moteiInit${what}-PosHow enableEntry  2
	$W.cv.moteiInit${what}-PosHow enableEntry  3
	$W.cv.moteiInit${what}-PosHow enableEntry  4
    }
    set gvar(actionClass_posHow) $posHow
    set gvar(posHow)             $posHow

    # Tell which frame to display
    switch $posHow {
	2     {set positionFrame 0 }
	3 - 5 {set positionFrame 1 }
	4 - 6 {set positionFrame 2 }
    }
    # Tell second frame to display or hide offset sub-panel
    if {$posHow == 5 || $posHow == 6} {
	$W.cv.moteiInit${what}-Offset configure -show 1
	# Configure list of units
	set offsetFrame 0
	set w [$W.cv.moteiInit${what}-Offset getClassPath]
	$w.cv.valueUnit configure -itemList $PUL
	if {[lsearch $PUL $gvar(posUnit)] == -1} {
	    set gvar(posUnit) $PU
	}
	set gvar(offsetClass_valueUnit) $gvar(posUnit)
	set gvar(offsetClass_value)     $gvar(posValue)
    } else {
	$W.cv.moteiInit${what}-Offset configure -show 0
    }
    $W.cv.moteiInit${what}-Offset show

    # Configure option buttons with corresponding lists
    set w [$W.cv.moteiInit${what}-Target getClassPath]
    switch $positionFrame {
	0 {
	    # Configure list of units
	    $w.cv.valueUnit configure -itemList $PUL
	    if {[lsearch $PUL $gvar(posUnit)] == -1} {
		set gvar(posUnit) $PU
	    }
	    set gvar(positionClass_value)     $gvar(posValue)
	    set gvar(positionClass_valueUnit) $gvar(posUnit)
	}
	1 {
	    # Configure list of names
	    $w.cv.nameList configure -itemList $NPL
	    if {[lsearch $NPL $gvar(posName)] == -1} {
		set gvar(posName) $NP0
	    }
	    if {[lsearch [list 1 2 3]  $gvar(posSection)] == -1} {
		set gvar(posSection) 1
	    }
	    set gvar(positionClass_nameValue)    $gvar(posName)
	    set gvar(positionClass_sectionValue) $gvar(posSection)
	}
	2 {
	    # Configure list of index
	    $w.cv.indexList configure -itemList $IPL
	    if {[lsearch $IPL $gvar(posIndex)] == -1} {
		set gvar(posIndex) $IP0
	    }
	    if {[lsearch [list 1 2 3]  $gvar(posSection)] == -1} {
		set gvar(posSection) 1
	    }
	    set gvar(positionClass_indexValue)   $gvar(posIndex)
	    set gvar(positionClass_sectionValue) $gvar(posSection)
	}
    }
    $W.cv.moteiInit${what}-Target show
}

#-----------------------------------
# Action labels
#-----------------------------------
proc moteiCHwInitMakeActionLabel { ia } {

    lassign $ia actionType function dummy initCode motionMode \
	    posHow posValue posUnit posName posSection posIndex \
	    posType spdHow spdValue spdUnit spdName spdIndex far near end
    switch $actionType {
	 1 { set name "END" }
	 2 { 
	     if {$motionMode == 3} {
		 set name "Move " 
		 switch $posType {
		     1 { append name "ABS"  }
		     2 { append name "REL"  }
		     3 { append name "LHW+" }
		     4 { append name "UHW+" }
		     5 { append name "REF+" }
		     6 { append name "IND+" }
		     7 { append name "HOM+" }
		     9 { append name "2 Step ABS"}
		    10 { append name "2 Step REL"}
		    11 { append name "Half-Turn ABS"}
		    12 { append name "Half-Turn REL"}
		   261 { append name "Off REF"}
		 }
	     } else {
		 set name "Find "
		 switch $posType {
		     3 { append name "LHW" }
		     4 { append name "UHW" }
		     5 { append name "REF" }
		     6 { append name "IND" }
		     7 { append name "HOM" }
		 }
	     }
	 }
	 3 -
	 4 -
	 5 { 
	     if {$actionType == 3} { 
		 set name "Set LSW = " 
	     } elseif {$actionType == 4} { 
		 set name "Set USW = "
	     } else { 
		 set name "Calibrate " 
	     }
	     switch $posType {
		 1 { append name "ABS"  }
		 2 { append name "REL"  }
		 3 { append name "LHW+" }
		 4 { append name "UHW+" }
		 5 { append name "REF+" }
		 6 { append name "IND+" }
		 7 { append name "HOM+" }
	     }
	 }
	 6 { set name "Clamp Brake" }
	 12 { set name "UnClamp Brake" }
	 7 { set name "Disconnect Power Relays" }
	 11 { set name "Connect Power Relays" }
	 8 { set name "Function '$function'" }
	 9 { set name "Exec On-Board Proc #$initCode" }
	10 { set name "Wait $initCode ms" }
	default { set name "<empty>" }
    }
    return $name
}


#-----------------------------------
# List handling procedures
#-----------------------------------
proc moteiCHwInitUpdate {  } {
    global cv moteiIA moteiIAIdx moteiIAL

    set ial {}
    foreach ia $moteiIA {
	set name [moteiCHwInitMakeActionLabel $ia]
	lappend ial $name
    }
    set moteiIAL $ial
    set w $cv.moteiCHwInit-IAL
    $w configure -itemList $ial

    set iend 0
    foreach ia $moteiIA {
	if {[lindex $ia 0] == 1} break
	incr iend
    }
    set last [expr {[llength $ial] - 1}]
    if {$moteiIAIdx == 0 || [lindex [lindex $moteiIA $moteiIAIdx] 0] == 1} {
	$cv.moteiCHwInit-MUP disable
    } else {
	$cv.moteiCHwInit-MUP enable
    }
    if {$moteiIAIdx == $iend || [lindex [lindex $moteiIA [expr {$moteiIAIdx + 1}]] 0] == 1} {
	$cv.moteiCHwInit-MDN disable
    } else {
	$cv.moteiCHwInit-MDN enable
    }
    if {![cequal [lindex $ial $last] "<empty>"]} {
	$cv.moteiCHwInit-INS disable
    } else {
	$cv.moteiCHwInit-INS enable
    }
    if {[cequal [lindex $ial 0] "<empty>"] || \
	[cequal [lindex $ial $moteiIAIdx] "<empty>"] || \
        [cequal [lindex $ial $moteiIAIdx] "END"]} {
	$cv.moteiCHwInit-DEL disable
    } else {
	$cv.moteiCHwInit-DEL enable
    }
}

proc moteiCHwInitSelect { {row 0} } {
    global gvar cv moteiIA moteiIAIdx moteiConfigParams

    set w $cv.moteiCHwInit-IAL
    if {$row == -1} {
	set idx 0
    } elseif {$row == 0} {
	set idx [$w getIndex]
	moteiCHwInitApplyRow
    } else {
	set idx [expr {$row - 1}]
    }
    set moteiIAIdx  $idx
    moteiCHwInitUpdate
    $w setSelection $idx
    set ia [lindex $moteiIA $idx]
    lassign $ia gvar(actionType) \
	    gvar(function) dummy \
	    gvar(initCode) \
	    gvar(motionMode) \
	    gvar(posHow) gvar(posValue) gvar(posUnit) \
	    gvar(posName) gvar(posSection) gvar(posIndex) \
	    gvar(posType)\
	    gvar(spdHow) gvar(spdValue) gvar(spdUnit) \
	    gvar(spdName) gvar(spdIndex) gvar(far) gvar(near) gvar(end)

    if {[lsearch $moteiConfigParams(posUnit) $gvar(posUnit)] == -1} {
	set gvar(posUnit) [lindex $moteiConfigParams(posUnit) 0]
    }
    if {[lsearch $moteiConfigParams(spdUnit) $gvar(spdUnit)] == -1} {
	set gvar(spdUnit) [lindex $moteiConfigParams(spdUnit) 0]
    }
    set ptl [list 1 2 5 6 7 9 10 261]
    if {$moteiConfigParams(axisType) == 1} { 
	lappend ptl 3 4
    } else { 
	lappend ptl 11 12
    }
    if {[lsearch $ptl $gvar(posType)] == -1} {
	set gvar(posType) 1
    }
    moteiCHwInitSelectAction $gvar(actionType)

    update idletasks
}

proc moteiCHwInitApplyRow {} {
    global gvar moteiIA moteiIAIdx 
    global actionFrame speedFrame positionFrame offsetFrame

    set gvar(posHow)     0
    set gvar(posValue)   0
    set gvar(posUnit)    ""
    set gvar(posName)    ""
    set gvar(posSection) 0
    set gvar(posIndex)   0

    set gvar(spdHow)     0
    set gvar(spdValue)   0
    set gvar(spdUnit)    ""
    set gvar(spdName)    ""
    set gvar(spdIndex)   0
    set gvar(far)        1000
    set gvar(near)       200
    set gvar(end)        1000

    switch $actionFrame {
	0 {
	    set gvar(motionMode) $gvar(actionClass_motionMode)
	    set gvar(posType)    $gvar(actionClass_posType)
	    set gvar(spdHow)     $gvar(actionClass_spdHow)
	    switch $gvar(spdHow) {
		1 { # by default
		}
		2 { # by value
		    set gvar(spdValue) $gvar(speedClass_value)
		    set gvar(spdUnit)  $gvar(speedClass_valueUnit)
		}
		3 { # by Name
		    set gvar(spdName)  $gvar(speedClass_nameValue)
		}
		4 { # by Index
		    set gvar(spdIndex) $gvar(speedClass_indexValue)
		}
	    }
	    if {$gvar(motionMode) == 3} {
		switch $gvar(actionClass_posHow) {
		    2 { # by value
			set gvar(posHow)     2
			set gvar(posValue)   $gvar(positionClass_value)
			set gvar(posUnit)    $gvar(positionClass_valueUnit)
		    }
		    3 { # by Name
			set gvar(posHow)     3
			set gvar(posName)    $gvar(positionClass_nameValue)
			set gvar(posSection) $gvar(positionClass_sectionValue)
		    }
		    4 { # by Index
			set gvar(posHow)     4
			set gvar(posSection) $gvar(positionClass_sectionValue)
			set gvar(posIndex)   $gvar(positionClass_indexValue)
		    }
		    5 { # by Name + offset
			set gvar(posHow)     3
			set gvar(posValue)   $gvar(offsetClass_value)
			set gvar(posUnit)    $gvar(offsetClass_valueUnit)
			set gvar(posName)    $gvar(positionClass_nameValue)
			set gvar(posSection) $gvar(positionClass_sectionValue)
		    }
		    6 { # by Index + offset
			set gvar(posHow)     4
			set gvar(posValue)   $gvar(offsetClass_value)
			set gvar(posUnit)    $gvar(offsetClass_valueUnit)
			set gvar(posSection) $gvar(positionClass_sectionValue)
			set gvar(posIndex)   $gvar(positionClass_indexValue)
		    }
		}
	    } 
	}
	1 {
	    set gvar(posType)   $gvar(actionClass_posType)
	    switch $gvar(actionClass_posHow) {
		2 { # by value
		    set gvar(posHow)     2
		    set gvar(posValue)   $gvar(positionClass_value)
		    set gvar(posUnit)    $gvar(positionClass_valueUnit)
		}
		3 { # by Name
		    set gvar(posHow)     3
		    set gvar(posName)    $gvar(positionClass_nameValue)
		    set gvar(posSection) $gvar(positionClass_sectionValue)
		}
		4 { # by Index
		    set gvar(posHow)     4
		    set gvar(posSection) $gvar(positionClass_sectionValue)
		    set gvar(posIndex)   $gvar(positionClass_indexValue)
		}
		5 { # by Name + offset
		    set gvar(posHow)     3
		    set gvar(posValue)   $gvar(offsetClass_value)
		    set gvar(posUnit)    $gvar(offsetClass_valueUnit)
		    set gvar(posName)    $gvar(positionClass_nameValue)
		    set gvar(posSection) $gvar(positionClass_sectionValue)
		}
		6 { # by Index + offset
		    set gvar(posHow)     4
		    set gvar(posValue)   $gvar(offsetClass_value)
		    set gvar(posUnit)    $gvar(offsetClass_valueUnit)
		    set gvar(posSection) $gvar(positionClass_sectionValue)
		    set gvar(posIndex)   $gvar(positionClass_indexValue)
		}
	    }
	}
	3 {
	    set gvar(function)  $gvar(actionClass_function)
	}
	4 {
	    set gvar(initCode)  $gvar(actionClass_initCode)
	}
	5 {
	    set gvar(initCode)  $gvar(actionClass_delay)
	}
    }
    set idx $moteiIAIdx
    set ia [list $gvar(actionType) $gvar(function) 0 $gvar(initCode) \
	    $gvar(motionMode) $gvar(posHow) $gvar(posValue) $gvar(posUnit) \
	    $gvar(posName) $gvar(posSection) $gvar(posIndex) \
	    $gvar(posType) $gvar(spdHow) $gvar(spdValue) $gvar(spdUnit) \
	    $gvar(spdName) $gvar(spdIndex) $gvar(far) $gvar(near) $gvar(end)]
    set moteiIA [lreplace $moteiIA $idx $idx $ia]
}

proc moteiCHwInitInsert { } {
    global gvar cv moteiIA moteiIAIdx moteiIAL

    set idx $moteiIAIdx
    moteiCHwInitApplyRow
    set ilast [expr {[llength $moteiIAL] - 1}]
    set moteiIA [lreplace $moteiIA $ilast $ilast]
    set new [list 1 "" 0 0 0 0 0 "" "" 0 0 0 0 0 "" "" 0 0 0 0]
    set moteiIA [linsert $moteiIA $idx $new]
    moteiCHwInitUpdate
    moteiCHwInitSelect [expr {$idx + 1}]
}

proc moteiCHwInitMoveUp { } {
    global gvar cv moteiIA moteiIAIdx moteiIAL

    set idx $moteiIAIdx
    moteiCHwInitApplyRow
    if {$idx == 0} return
    set iprev [expr {$idx - 1}]
    set ia  [lindex $moteiIA $idx]
    set iap [lindex $moteiIA $iprev]
    set moteiIA [lreplace $moteiIA $iprev $idx $ia $iap]
    moteiCHwInitUpdate
    moteiCHwInitSelect [expr {$iprev + 1}]
}

proc moteiCHwInitMoveDn { } {
    global gvar cv moteiIA moteiIAIdx moteiIAL
   
    set idx $moteiIAIdx
    moteiCHwInitApplyRow
    if {$idx == [llength $moteiIAL]} return
    if {$gvar(actionType) == 1} return
    set inext [expr {$idx + 1}]
    set ia  [lindex $moteiIA $idx]
    set ian [lindex $moteiIA $inext]
    set moteiIA [lreplace $moteiIA $idx $inext $ian $ia]
    moteiCHwInitUpdate
    moteiCHwInitSelect [expr {$inext + 1}]
}

proc moteiCHwInitDelete { } {
    global gvar cv moteiIA moteiIAIdx moteiIAL

    set idx $moteiIAIdx
    set moteiIA [lreplace $moteiIA $idx $idx]
    set new [list 0 "" 0 0 0 0 0 "" "" 0 0 0 0 0 "" "" 0 0 0 0]
    lappend moteiIA $new
    moteiCHwInitUpdate
    moteiCHwInitSelect [expr {$idx + 1}]
}

#
# Registration Procedures
#
proc moteiCHwInitExitPanel { } {
}


#
# ___oOo___

