#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCIndSpd.tcl,v 2.43 2005/04/19 12:32:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  27/10/95  created
#

#************************************************************************
#   NAME
#       moteiCIndSpd - configure the speed for Index Pulse search 
#
#   SYNOPSIS
#       moteiCIndSpd <args>
# 
#   DESCRIPTION
#       The panel moteiCIndSpd.pan contains a set of widgets allowing the 
#       configuration of the Speed to apply for searching the encoder 
#       Index Pulse. 
#       The file moteiCIndSpd.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCIndSpd(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCIndSpdReadConfig { {mode 0} } {
    global gvar moteiConfigParams moteiSetupParams moteiStatusParams 
    global moteiIS moteiBkpIS

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set c "$gvar(dbPath):SERVER:CONF"
	if {[catch {seq_dbReadSymbolic "$c.indexPulseSpeed"} is]} {
	    seq_errDisplay -suspend
	    return "FAILURE"
	}
	set moteiIS    $is
	set moteiBkpIS $is
    }
}

proc moteiCIndSpdWriteConfig { } {
    global gvar moteiIS

    set c "$gvar(dbPath):SERVER:CONF"
    if {[catch {seq_dbWriteSymbolic "$c.indexPulseSpeed" $moteiIS}] != 0} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

proc moteiCIndSpdConfigWidgets { } {
    global gvar cv speedFrame
    global moteiConfigParams moteiSetupParams moteiStatusParams moteiIS

    #
    # for all widgets, configure as from DB
    #
    lassign $moteiIS spdHow \
	    gvar(spdVal) gvar(spdUnit) gvar(spdName) gvar(spdIdx) \
	    gvar(spdFar) gvar(spdNear) gvar(spdEnd)

    if {[lsearch [list 1 2 3 4] $spdHow] == -1} {
	set spdHow 2
	set gvar(spdVal) 0.0
	set gvar(spdUnit) $moteiConfigParams(spdUnit)
    }
    if {[lsearch $moteiConfigParams(spdUnit) $gvar(spdUnit)] == -1} {
	set gvar(spdUnit) $moteiConfigParams(spdUnit)
    }
    if {[lsearch $moteiConfigParams(namedSpdList) $gvar(spdName)] == -1} {
	set gvar(spdName) [lindex $moteiConfigParams(namedSpdList) 0]
    }
    if {[lsearch $moteiConfigParams(indexSpdList) $gvar(spdIdx)] == -1} {
	set gvar(spdIdx) [lindex $moteiConfigParams(indexSpdList) 0]
    }
    moteiCIndSpdUpdateVelocityGui $spdHow
}

proc moteiCIndSpdUpdateVelocityGui { guiMode } {
    global cv gvar speedFrame moteiSetupParams moteiConfigParams

    set speedFrame [expr {$guiMode - 1}]
    set gvar(speedHow) $guiMode

    # Configure option buttons with corresponding lists
    switch $speedFrame {
	0 {
	    # Configure default speed
	    set gvar(speedClass_defaultSpdIndex) $moteiConfigParams(defaultSpeed)
	    set gvar(speedClass_defaultSpdName) [lindex $moteiConfigParams(namedSpdList) $moteiConfigParams(defaultSpeed)]
	    if {![cequal $moteiConfigParams(spdUnit) "Enc/ms"]} {
		set DSV $moteiConfigParams(spdDefault)
		set DSU $moteiConfigParams(spdDefaultUnit)
	    } else {
		set DSV $moteiConfigParams(spdDefaultEnc)
		set DSU "Enc/ms"
	    }
	    set gvar(speedClass_defaultSpdValue) "$DSV $DSU"
	}
	1 {
	    # Configure list of units
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.valueUnit configure \
		    -itemList $moteiConfigParams(spdUnit) \
		    -default  $moteiConfigParams(spdUnit)
	    set gvar(speedClass_value) $gvar(spdVal)
	    set gvar(speedClass_valueUnit) $gvar(spdUnit)
	    moteiConvertVelocityRange
	}
	2 {
	    # Configure list of names
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.nameList configure \
		    -itemList $moteiConfigParams(namedSpdList) \
		    -default  [lindex $moteiConfigParams(namedSpdList) 0]
	    set gvar(speedClass_nameValue) $gvar(spdName)
	}
	3 {
	    # Configure list of index
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.indexList configure \
		    -itemList $moteiConfigParams(indexSpdList)  \
		    -default  [lindex $moteiConfigParams(indexSpdList) 0]
	    set gvar(speedClass_indexValue) $gvar(spdIdx)
	}
    }
    $cv.speedFrame show
}

proc  moteiCIndSpdInitPanel { } {
    global gvar cv

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCIndSpdReadConfig

    # configure widgets
    moteiCIndSpdConfigWidgets
}

#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCIndSpdApply { } {
    global gvar moteiIS

    set moteiIS $gvar(speedHow)
    switch $gvar(speedHow) {
	1 { # by Default
	    lappend moteiIS 0.0 "Enc/ms" "" 0
	}
	2 { # by Value
	    lappend moteiIS $gvar(speedClass_value) $gvar(speedClass_valueUnit) "" 0 
	}
	3 { # by Name
	    lappend moteiIS 0.0 "Enc/ms" $gvar(speedClass_nameValue) 0 
	}
	4 { # by Index
	    lappend moteiIS 0.0 "Enc/ms" "" $gvar(speedClass_indexValue) 
	}
    }
    lappend moteiIS $gvar(spdFar) $gvar(spdNear) $gvar(spdEnd) 0

    moteiCIndSpdWriteConfig
}

proc moteiCIndSpdReset { } {
    global moteiBkpIS moteiIS

    set moteiIS $moteiBkpIS

    moteiCIndSpdWriteConfig
    moteiCIndSpdConfigWidgets
}

proc moteiCIndSpdRefresh { } {
    moteiCIndSpdReadConfig 1
    moteiCIndSpdConfigWidgets
}

proc moteiCIndSpdExitPanel { } {
}


#
# ___oOo___
