#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCMAC4.tcl,v 2.43 2005/04/19 12:32:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux  07/06/95  created
# imunoz   18.03.97  Added dump of data in dbl format 
#

#************************************************************************
#   NAME
#       moteiCMAC4 - configure the Motion Controller MACCON MAC4
# 
#   SYNOPSIS
#       moteiCMAC4 <args>
# 
#   DESCRIPTION
#       The panel moteiCMAC4.pan contains a set of widgets allowing the
#       configuration of the Motion Controller MACCON MAC4. 
#       The file moteiCMAC4.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCMAC4(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
#
# Preset Parameters Mnemonics
#----------------------------
proc moteiCMAC4_setMnemonics { } {
    global mac4Idx mac4Par
    
    set mac4Par [list sa sd pa pd ca cd ia id ha hd fa fd\
	    da ax cr ln lp et eb ec eo es sc ga ze ki is po lt\
	    ull lll rsl dfl msr ssf bst crd swt\
	    pol wd mf ed tr tt sf lv iev]

    set i 0
    foreach el $mac4Par { 
	set mac4Idx($el) $i
	incr i
    }
}

#
# Read Motor Main & Motion Controller Configuration
#--------------------------------------------------
proc moteiCMAC4_readConfig { {mode 0} } {
    global gvar 
    global mac4Cfg mac4Bkp mac4Par mac4Idx mac4Dev
    global motCfg

    #
    # Read motor main config
    #
    set p "$gvar(dbPath):SERVER:CONF"
    set motCfg(motor)   [seq_dbReadSymbolic "$p.motor"]
    set motCfg(encoder) [seq_dbReadSymbolic "$p.encoder"]
    set motCfg(axis)    [seq_dbReadSymbolic "$p.axis"]
    set motCfg(lsw)     [seq_dbReadSymbolic "$p.specialPositions(0)"]
    set motCfg(usw)     [seq_dbReadSymbolic "$p.specialPositions(1)"]
    if {$mode != 0} return

    #
    # Read motion controller config
    #
    set p "$gvar(dbPath):SDL:MAC4:CFG"
    if {[catch {seq_dbReadSymbolic "$p.configValues"} mac4Cfg]} {
	seq_errDisplay -suspend
	return
    }
    if {[catch {seq_dbReadSymbolic "$p.device"} mac4Dev]} {
	seq_errDisplay -suspend
	return
    }
    lassign $mac4Dev dummy gvar(Channel)
    set mac4Dev [lreplace $mac4Dev 0 0 $gvar(Driver)]
    moteiDeviceExtractChannel

    #
    # Copy EO
    #
    if {[llength $motCfg(encoder)] == 10} {
	lassign $motCfg(encoder) et ei cc ea eb ec cr es sc eo
	lreplace $mac4Cfg 20 20 $eo
    }

    #
    # Make a backup
    #
    set mac4Bkp $mac4Cfg
}

#
# Write Motion Controller Configuration
#--------------------------------------
proc moteiCMAC4_writeConfig { } {
    global gvar mac4Cfg mac4Dev

    set p "$gvar(dbPath):SDL:MAC4:CFG"
    if {[catch {seq_dbWriteSymbolic "$p.configValues" $mac4Cfg}]} {
	seq_errDisplay -suspend
    }
    if {[catch {seq_dbWriteSymbolic "$p.device" $mac4Dev}]} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

#
# Configure widgets
#------------------
proc moteiCMAC4_setEncoderType { mt at et ei } {

    if {$et == 0 && $mt == 2} {
	# STP with internal counting (no encoder)
	return 0
    }
    if {$et == 1} {
	# Incremental encoder
	if {$ei != 2} {
	    # External
	    return 3
	}
	return 1
    } 
    if {$et == 2} {
	# Absolute encoder
	if {$ei != 2} {
	    # External
	    return 4
	}
	return 2
    }
    if {$et == 3} {
	# Absolute encoder in relative mode
	if {$ei != 2} {
	    # External
	    return 3
	}
	return 1
    }
}

proc moteiCMAC4_setSwLim { lim } {

    lassign $lim flag posType limValue limUnit
    if {$flag == 0} { return "N/A" }
    set swLimit "" ; set ref "" ; set sign ""
    if {$limValue >= 0} { set sign "+" }
    switch $posType {
	2 { set ref "REL" }
	3 { set ref "LHW" }
	4 { set ref "UHW" }
	5 { set ref "REF" }
	6 { set ref "IND" }
	7 { set ref "HOM" }
    }
    append swLimit $ref $sign $limValue $limUnit
    return $swLimit
}

proc moteiCMAC4_configWidgets { } {
    global gvar cv rangeFrame
    global mac4Cfg mac4Par mac4Idx
    global motCfg

    #
    # Assign widget values with motion controller config
    #
    set i 0
    foreach el $mac4Par { 
	set gvar($el) [lindex $mac4Cfg $mac4Idx($el)]
	incr i
    }

    #
    # Overload widget values with motor main config
    #
    set at $motCfg(axis) ; set gvar(at) $at
    set mt [lindex $motCfg(motor) 0]
    if {[llength $motCfg(encoder)] == 10} {
	lassign $motCfg(encoder) et ei cc ea eb ec cr es sc eo
    } else {
	set eo $gvar(eo)
	lassign $motCfg(encoder) et ei cc ea eb ec cr es sc
    }
    set gvar(et) [moteiCMAC4_setEncoderType $mt $at $et $ei]

    if {$mt == 2} {
	set gvar(tUnit) "ms"
	set gvar(pUnit) "Step"
	set gvar(vUnit) "Hz"
	set gvar(aUnit) "Hz/SP"
    } else {
	set gvar(tUnit) "ms"
	set gvar(pUnit) "Enc"
	set gvar(vUnit) "Enc/SP"
	set gvar(aUnit) "Enc/SP^2"
    }
    if {$et == 0} {
	if {$mt == 2} {
	    set cr [lindex $mac4Cfg $mac4Idx(msr)]
	}
    } elseif {$et == 1} {
	if {$mt == 1} {
	    set gvar(ln) -1000000000
	    set gvar(lp)  1000000000
	} else {
	    set gvar(ln) -10000000
	    set gvar(lp)  10000000
	}
    } else {
	set gvar(ln)  0
	set gvar(lp)  [expr {$ec - 1}]
    }
    if {$et > 0 && $at == 1} { set cr $ec }

    set gvar(eb)      $eb
    set gvar(eb_str)  $eb
    set gvar(ec)      $ec
    set gvar(ec_str) [format "%d %s" $ec $gvar(pUnit)]
    set gvar(cr)      $cr
    set gvar(cr_str) [format "%d %s" $cr $gvar(pUnit)]
    set gvar(eo)      $eo
    set gvar(eo_str) [format "%d %s" $eo $gvar(pUnit)]
    
    if {$mt == 2 && $et != 0} {
	set gvar(es) $es
	set gvar(es_str) [format "%d %s" $es $gvar(pUnit)]
    }
    set gvar(sc) 0
    set gvar(sc_str) "N/A"
    
    if {$et == 0} {
	set gvar(eb) 15
	set gvar(eb_str) "N/A"
	set gvar(ec) 32768
	set gvar(ec_str) "N/A"
	set gvar(es) 1000
	set gvar(es_str) "N/A"
	set gvar(eo) 0
	set gvar(eo_str) "N/A"
    }

    if {$et == 2 || $et == 3} {
	set gvar(sc)      $sc
	set gvar(sc_str)  $sc
    }

    if {$at == 1} {
	set rangeFrame 0
	set gvar(rangeClass_LN) [moteiCMAC4_setSwLim $motCfg(lsw)]
	set gvar(rangeClass_LP) [moteiCMAC4_setSwLim $motCfg(usw)]
    } else {
	set rangeFrame 1
	set gvar(rangeClass_CR) [format "%d %s" $cr $gvar(pUnit)]
    }
    $cv.rangeClass configure -show 1
    $cv.rangeClass show

    # 
    # configure widgets
    #
    if {$mt == 1} {
	set gvar(SP) 10.0
	# DC
	$cv.moteiCMAC4-wdgMSR  disable
	$cv.moteiCMAC4-wdgSSF  disable
	$cv.moteiCMAC4-wdgBST  disable
	$cv.moteiCMAC4-wdgSWT  disable
	$cv.moteiCMAC4-wdgCRD  disable
	$cv.moteiCMAC4-wdgSTR  disable
	$cv.moteiCMAC4-wdgGA    enable
	$cv.moteiCMAC4-wdgZE    enable
	$cv.moteiCMAC4-wdgPO    enable
	$cv.moteiCMAC4-wdgIG    enable
	$cv.moteiCMAC4-wdgIS    enable

    } else {
	set gvar(SP) 12.5
	# Stepper
	$cv.moteiCMAC4-wdgMSR   enable
	$cv.moteiCMAC4-wdgSSF   enable
	$cv.moteiCMAC4-wdgBST   enable
	$cv.moteiCMAC4-wdgSWT   enable
	$cv.moteiCMAC4-wdgCRD   enable
	$cv.moteiCMAC4-wdgSTR   enable
	$cv.moteiCMAC4-wdgGA   disable
	$cv.moteiCMAC4-wdgZE   disable
	$cv.moteiCMAC4-wdgPO   disable
	$cv.moteiCMAC4-wdgIG   disable
	$cv.moteiCMAC4-wdgIS   disable
    }

    if { $et == 1 && $ei == 2} {
	$cv.moteiCMAC4-wdgCA    enable
	$cv.moteiCMAC4-wdgCD    enable
	$cv.moteiCMAC4-wdgHA    enable
	$cv.moteiCMAC4-wdgHD    enable
	$cv.moteiCMAC4-wdgIA    enable
	$cv.moteiCMAC4-wdgID    enable
	$cv.moteiCMAC4-wdgLVIC  enable
	$cv.moteiCMAC4-wdgLVIF  enable
	$cv.moteiCMAC4-wdgLVH   enable
    } else {
	$cv.moteiCMAC4-wdgCA    disable
	$cv.moteiCMAC4-wdgCD    disable
	$cv.moteiCMAC4-wdgHA    disable
	$cv.moteiCMAC4-wdgHD    disable
	$cv.moteiCMAC4-wdgIA    disable
	$cv.moteiCMAC4-wdgID    disable
	$cv.moteiCMAC4-wdgLVIC  disable
	$cv.moteiCMAC4-wdgLVIF  disable
	$cv.moteiCMAC4-wdgLVH   disable
    }
    moteiCMAC4_showPOL

    update idletasks
}

proc moteiCMAC4InitPanel { } {
    global gvar cv EDITING mac4Par mac4Idx mac4Bkp mac4Cfg
    
    moteiCMAC4_setMnemonics

    if {$EDITING} return

    seq_ccsAsyncInput open 

    # install close handler
    vccStdMain shutDown

    moteiCMAC4_readConfig
    moteiCMAC4_configWidgets

    #
    moteiCMAC4_stateMachine
}

#
# Configure POL widget label
#---------------------------
proc moteiCMAC4_showPOL { } {
    global gvar cv

    if {$gvar(pol) == 0} {
	$cv.moteiCMAC4-wdgPOL configure -label Normal
    } else {
        $cv.moteiCMAC4-wdgPOL configure -label Reverse
    }
}

#
# Switch Polarity
#----------------
proc moteiCMAC4_switchPOL { } {
    global gvar

    set gvar(pol) [expr {!$gvar(pol)}]
    moteiCMAC4_showPOL
}

#
# State Machine for Axis Type
#----------------------------
proc moteiCMAC4_stateMachineAX { } {

    global gvar cv

}

#
# State Machine for Encoder Type
#-------------------------------
proc moteiCMAC4_stateMachineET { {label ""} } {
    global gvar
    
}

proc moteiCMAC4_stateMachineET0 { {label ""} } {
    moteiCMAC4_stateMachineET
}

proc moteiCMAC4_stateMachineET1 { {label ""} } {
    moteiCMAC4_stateMachineET
}

#
# State Machine
#--------------
proc moteiCMAC4_stateMachine { } {                              
    global gvar 
    
    moteiCMAC4_stateMachineAX
    moteiCMAC4_stateMachineET
}

#
# Action Button APPLY
#---------------------
proc moteiCMAC4Apply { } {
    global gvar cv
    global mac4Cfg mac4Bkp mac4Par mac4Idx mac4Dev
    global motCfg

    #
    # Assign widget values with motion controller config
    #
    set mac4Dev [list $gvar(Driver) $gvar(Channel)]

    foreach el $mac4Par { 
	set idx $mac4Idx($el)
	set val $gvar($el)
	if {$val != "N/A"} { 
	    set mac4Cfg [lreplace $mac4Cfg $idx $idx $val]
	}
    }

    #
    # Write motion controller config
    #
    moteiCMAC4_writeConfig
}

#
# Action Button RESET
#---------------------
proc moteiCMAC4Reset { } {
    global gvar
    global mac4Cfg mac4Bkp mac4Dev
    
    #
    # Restore backup configuration
    #
    set mac4Cfg $mac4Bkp
    lassign $mac4Dev dummy gvar(Channel)
    moteiDeviceExtractChannel

    #
    # Write motion controller config
    #
    moteiCMAC4_writeConfig

    #
    # Refresh panel
    #
    moteiCMAC4_configWidgets
    moteiCMAC4_stateMachine
}

#
# Action Button REFRESH
#----------------------
proc moteiCMAC4Refresh { } {
    moteiCMAC4_readConfig 1
    moteiCMAC4_configWidgets
    moteiCMAC4_stateMachine
}

proc moteiCMAC4ExitPanel { } {
}

#
# ___oOo___
