#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCNamPos.tcl,v 2.43 2005/04/19 12:32:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    13/03/97  defined gvar(relPath) to support dbTableDump
# imunoz    04/03/97  uses moteiDbTableWdg to interact with the DB, 
#                     lot of code saved (and deleted).
# imunoz    30/04/96  itcl2.0 port
# imunoz    11/23/95  created 
#

#************************************************************************
#   NAME
#       moteiCNamPos - configure the Named Positions
# 
#   SYNOPSIS
#       moteiCNamPos <args>
# 
#   DESCRIPTION
#       The panel moteiCNamPos.pan contains a set of widgets allowing the 
#       configuration of the Named Positions. 
#       The file moteiCNamPos.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCNamPos(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCNamPosReadConfig { {mode 0} } {
    global gvar moteiConfigParams
    global moteiNP moteiPO

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set moteiPO $moteiConfigParams(offsets)
	set moteiNP $moteiConfigParams(namedPositions)
    }
}

proc moteiCNamPosWriteConfig { } {
    global gvar moteiNP moteiNPL moteiPO

    set c "$gvar(dbPath):SERVER:CONF"

    set moteiPO [list $gvar(globMntOff) $gvar(globUsrOff) $gvar(globOffUnit) 0 0]

    if {[catch {seq_dbWriteSymbolic "$c.offsets" $moteiPO}] != 0} {
	seq_errDisplay -suspend
    }
    set npn [llength $moteiNPL] ; set attr "namedPositions"
    for {set n 0} {$n < $npn} {incr n} {
	set np [lindex $moteiNP $n]
	if {[catch {seq_dbWriteSymbolic "$c.${attr}($n)" $np}] != 0} {
	    seq_errDisplay -suspend
	}
    }
    moteiConfigReinstall
}

proc moteiCNamPosConfigWidgets { } {
    global gvar cv moteiConfigParams moteiPO moteiNP moteiNPL moteiNPIdx 

    set moteiNPIdx 0
    moteiCNamPosUpdateList
    moteiCNamPosSelectRow -1

    set gvar(globMntOff)  [lindex $moteiPO 0]
    set gvar(globUsrOff)  [lindex $moteiPO 1]
    set gvar(globOffUnit) [lindex $moteiPO 2]

    $cv.moteiCNamPos-Unit configure -itemList $moteiConfigParams(posUnit)

    if {$moteiConfigParams(axisType) == 1} {
	$cv.moteiCNamPos-Type enableEntry 1
	$cv.moteiCNamPos-Type enableEntry 2
    } else {
	$cv.moteiCNamPos-Type disableEntry 1
	$cv.moteiCNamPos-Type disableEntry 2
    }
}

proc moteiCNamPosInitPanel { } {

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCNamPosReadConfig

    # configure widgets
    moteiCNamPosConfigWidgets
}

proc moteiCNamPosUpdateList {  } {
    global cv moteiNP moteiNPIdx moteiNPL

    set npl {}
    foreach np $moteiNP {
        set name [lindex $np 0]
        if {[cequal $name ""]} { set name "<empty>" }
	lappend npl $name
    }
    set moteiNPL $npl
    set w $cv.moteiCNamPos-NPL
    $w configure -itemList $npl

    if {$moteiNPIdx == 0} {
	$cv.moteiCNamPos-MUP disable
    } else {
	$cv.moteiCNamPos-MUP enable
    }
    if {$moteiNPIdx == [expr {[llength $npl] - 1}]} {
	$cv.moteiCNamPos-MDN disable
    } else {
	$cv.moteiCNamPos-MDN enable
    }
    set last [expr {[llength $npl] - 1}]
    if {$moteiNPIdx == $last} {
	$cv.moteiCNamPos-MDN disable
    } else {
	$cv.moteiCNamPos-MDN enable
    }
    if {![cequal [lindex $npl $last] "<empty>"]} {
	$cv.moteiCNamPos-INS disable
    } else {
	$cv.moteiCNamPos-INS enable
    }
    if {[cequal [lindex $npl 0] "<empty>"] || \
	[cequal [lindex $npl $moteiNPIdx] "<empty>"]} {
	$cv.moteiCNamPos-DEL disable
    } else {
	$cv.moteiCNamPos-DEL enable
    }
}

proc moteiCNamPosSelectRow { {row 0} } {
    global gvar cv moteiNP moteiNPIdx moteiConfigParams

    set w $cv.moteiCNamPos-NPL
    if {$row == -1} {
	set idx 0
    } elseif {$row == 0} {
	set idx [$w getIndex]
	moteiCNamPosApplyRow
    } else {
	set idx [expr {$row - 1}]
    }
    set moteiNPIdx  $idx
    moteiCNamPosUpdateList
    $w setSelection $idx
    set np [lindex $moteiNP $idx]
    lassign $np gvar(posName) gvar(posType) gvar(obsPos) \
	    gvar(lowRange) gvar(upRange) \
	    gvar(mntOffSet) gvar(mntOff) \
	    gvar(usrOffSet) gvar(usrOff) gvar(posUnit)
    if {[lsearch $moteiConfigParams(posUnit) $gvar(posUnit)] == -1} {
	set gvar(posUnit) [lindex $moteiConfigParams(posUnit) 0]
    }
    set ptl [list 1 5 6 7]
    if {$moteiConfigParams(axisType) == 1} { lappend ptl 3 4 } 
    if {[lsearch $ptl $gvar(posType)] == -1} {
	set gvar(posType) 1
    }
    set gvar(lowRange) [abs $gvar(lowRange)]
    set gvar(upRange)  [abs $gvar(upRange)]

    if {[cequal $gvar(posName) ""]} {
	set gvar(mntOff)    0
	set gvar(mntOffSet) 0
	set gvar(usrOff)    0
	set gvar(usrOffSet) 0
    }
    $cv.moteiCNamPos-PO configure -show 0
    $cv.moteiCNamPos-PO show
    update idletasks
}

proc moteiCNamPosApplyRow {} {
    global gvar moteiNP moteiNPIdx moteiNPL

    set idx $moteiNPIdx
    set npi [lsearch $moteiNPL $gvar(posName)]
    if {$npi != -1 && $npi != $idx} {
	set gvar(posName) ""
    }
    set gvar(lowRange) [abs $gvar(lowRange)]
    set gvar(upRange)  [abs $gvar(upRange)]
    set np [list $gvar(posName) $gvar(posType) $gvar(obsPos) \
	    $gvar(lowRange) $gvar(upRange) \
	    $gvar(mntOffSet) $gvar(mntOff) \
	    $gvar(usrOffSet) $gvar(usrOff) $gvar(posUnit)]
    lappend np 0 0 0 0 0 0 0 0 0 0
    set moteiNP [lreplace $moteiNP $idx $idx $np]
}

proc moteiCNamPosApply { } {
    global moteiNPIdx

    moteiCNamPosApplyRow
    moteiCNamPosWriteConfig
    moteiCNamPosSelectRow [expr {$moteiNPIdx + 1}]
}

proc moteiCNamPosReset { } {
    global moteiConfigBackup moteiNP moteiPO

    set moteiNP $moteiConfigBackup(namedPositions)
    set moteiPO $moteiConfigBackup(offsets)

    moteiCNamPosConfigWidgets
    moteiCNamPosWriteConfig
}

proc moteiCNamPosRefresh { } {
    moteiCNamPosReadConfig 1
    moteiCNamPosConfigWidgets
}

proc moteiCNamPosInsertAbove { } {
    global gvar cv moteiNP moteiNPIdx moteiNPL

    set idx $moteiNPIdx
    moteiCNamPosApplyRow
    set ilast [expr {[llength $moteiNPL] - 1}]
    set moteiNP [lreplace $moteiNP $ilast $ilast]
    set new [list "position#$idx" 1 $idx 1 1 0 0 0 0 "Enc" 0 0 0 0 0 0 0 0 0 0]
    set moteiNP [linsert $moteiNP $idx $new]
    moteiCNamPosUpdateList
    moteiCNamPosSelectRow [expr {$idx + 1}]
}

proc moteiCNamPosMoveUp { } {
    global gvar cv moteiNP moteiNPIdx moteiNPL

    set idx $moteiNPIdx
    moteiCNamPosApplyRow
    if {$idx == 0} return
    set iprev [expr {$idx - 1}]
    set np  [lindex $moteiNP $idx]
    set npp [lindex $moteiNP $iprev]
    set moteiNP [lreplace $moteiNP $iprev $idx $np $npp]
    moteiCNamPosUpdateList
    moteiCNamPosSelectRow [expr {$iprev + 1}]
}

proc moteiCNamPosMoveDn { } {
    global gvar cv moteiNP moteiNPIdx moteiNPL
   
    set idx $moteiNPIdx
    moteiCNamPosApplyRow
    if {$idx == [llength $moteiNPL]} return
    set inext [expr {$idx + 1}]
    set np  [lindex $moteiNP $idx]
    set npn [lindex $moteiNP $inext]
    set moteiNP [lreplace $moteiNP $idx $inext $npn $np]
    moteiCNamPosUpdateList
    moteiCNamPosSelectRow [expr {$inext + 1}]
}

proc moteiCNamPosDeleteRow { } {
    global gvar cv moteiNP moteiNPIdx moteiNPL

    set idx $moteiNPIdx
    set moteiNP [lreplace $moteiNP $idx $idx]
    set new [list "" 0 0 0 0 0 0 0 0 "" 0 0 0 0 0 0 0 0 0 0]
    lappend moteiNP $new
    moteiCNamPosUpdateList
    moteiCNamPosSelectRow [expr {$idx + 1}]
}

proc moteiCNamPosConfigOffsets { } {
    global gvar cv moteiConfigParams

    $cv.moteiCNamPos-PO configure -show 1
    $cv.moteiCNamPos-PO show

    set w [$cv.moteiCNamPos-PO getClassPath]
    $w.cv.moteiPosOff-GPU configure -itemList $moteiConfigParams(posUnit) \
	                            -variable  gvar(globOffUnit) \
	                            -default  $gvar(globOffUnit)

    $w.cv.moteiPosOff-GMO configure -variable gvar(globMntOff) 
    $w.cv.moteiPosOff-GUO configure -variable gvar(globUsrOff) 

    $w.cv.moteiPosOff-MOS configure -variable gvar(mntOffSet) 
    $w.cv.moteiPosOff-MO  configure -variable gvar(mntOff) 
    $w.cv.moteiPosOff-UOS configure -variable gvar(usrOffSet) 
    $w.cv.moteiPosOff-UO  configure -variable gvar(usrOff) 

    set gvar(posOffClass_offUnit) $gvar(posUnit)

    update idletasks
}

proc moteiCNamPosExitPanel { } {
}


#
# ___oOo___

