#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCNamSpd.tcl,v 2.43 2005/04/19 12:32:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    13/03/97  defined gvar(relPath) to support dbTableDump
# imunoz    04/03/97  uses moteiDbTableWdg to interact with the DB, 
#                     lot of code saved (and deleted).
# imunoz    30/04/96  itcl2.0 port
# imunoz    11/23/95  created 
#

#************************************************************************
#   NAME
#       moteiCNamSpd - configure the Named Speeds
# 
#   SYNOPSIS
#       moteiCNamSpd <args>
# 
#   DESCRIPTION
#       The panel moteiCNamSpd.pan contains a set of widgets allowing the 
#       configuration of the Named Speeds. 
#       The file moteiCNamSpd.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCNamSpd(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCNamSpdReadConfig { {mode 0} } {
    global gvar moteiConfigParams
    global moteiNS moteiDS

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set moteiNS $moteiConfigParams(namedSpeeds)
	set moteiDS $moteiConfigParams(defaultSpeed)
    }
}

proc moteiCNamSpdWriteConfig { } {
    global gvar moteiNS moteiNSL moteiDS

    set c "$gvar(dbPath):SERVER:CONF"

    set nsn [llength $moteiNSL] ; set attr "namedSpeeds"
    for {set n 0} {$n < $nsn} {incr n} {
	set ns [lindex $moteiNS $n]
	if {[catch {seq_dbWriteSymbolic "$c.${attr}($n)" $ns}] != 0} {
	    seq_errDisplay -suspend
	}
    }
    if {[catch {seq_dbWriteSymbolic "$c.defaultSpeed" $moteiDS}] != 0} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

proc moteiCNamSpdConfigWidgets { } {
    global gvar cv moteiConfigParams moteiDS moteiNS moteiNSL moteiNSIdx 

    set moteiNSIdx 0
    moteiCNamSpdUpdateList
    moteiCNamSpdSelectRow -1

    set spdUnitList $moteiConfigParams(spdUnit)
    if {[lsearch $spdUnitList "Ref"   ] == -1} { lappend spdUnitList "Ref"    }
    if {[lsearch $spdUnitList "Enc/ms"] == -1} { lappend spdUnitList "Enc/ms" }
    $cv.moteiCNamSpd-Unit configure -itemList $spdUnitList

    set ds [lindex $moteiNS $moteiDS]
    lassign $ds gvar(spdNameDefault) vd ud
    set gvar(spdDefault) "$vd $ud"
}

proc moteiCNamSpdInitPanel { } {

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCNamSpdReadConfig

    # configure widgets
    moteiCNamSpdConfigWidgets
}

proc moteiCNamSpdUpdateList {  } {
    global cv moteiNS moteiNSIdx moteiNSL

    set nsl {}
    foreach ns $moteiNS {
        set name [lindex $ns 0]
        if {[cequal $name ""]} { set name "<empty>" }
	lappend nsl $name
    }
    set moteiNSL $nsl
    set w $cv.moteiCNamSpd-NSL
    $w configure -itemList $nsl

    if {$moteiNSIdx == 0} {
	$cv.moteiCNamSpd-MUP disable
    } else {
	$cv.moteiCNamSpd-MUP enable
    }
    if {$moteiNSIdx == [expr {[llength $nsl] - 1}]} {
	$cv.moteiCNamSpd-MDN disable
    } else {
	$cv.moteiCNamSpd-MDN enable
    }
    set last [expr {[llength $nsl] - 1}]
    if {$moteiNSIdx == $last} {
	$cv.moteiCNamSpd-MDN disable
    } else {
	$cv.moteiCNamSpd-MDN enable
    }
    if {![cequal [lindex $nsl $last] "<empty>"]} {
	$cv.moteiCNamSpd-INS disable
    } else {
	$cv.moteiCNamSpd-INS enable
    }
    if {[cequal [lindex $nsl 0] "<empty>"] || \
	[cequal [lindex $nsl $moteiNSIdx] "<empty>"]} {
	$cv.moteiCNamSpd-DEL disable
    } else {
	$cv.moteiCNamSpd-DEL enable
    }
}

proc moteiCNamSpdSelectRow { {row 0} } {
    global gvar cv moteiNS moteiNSIdx moteiConfigParams

    set idx 0
    set w $cv.moteiCNamSpd-NSL
    if {$row == -1} {
	set idx 0
    } elseif {$row == 0} {
	set idx [$w getIndex]
	moteiCNamSpdApplyRow
    } else {
	set idx [expr {$row - 1}]
    }
    set moteiNSIdx  $idx
    moteiCNamSpdUpdateList
    $w setSelection $idx
    set ns [lindex $moteiNS $idx]
    lassign $ns gvar(spdName) gvar(spdValue) gvar(spdUnit) \
	        gvar(far) gvar(near) gvar(end)

    if {[lsearch $moteiConfigParams(spdUnit) $gvar(spdUnit)] == -1 && \
	![cequal $gvar(spdUnit) "Ref"] } {
	set gvar(spdUnit) [lindex $moteiConfigParams(spdUnit) 0]
    }

    if {[cequal $gvar(spdName) ""]} {
	set gvar(far)  1000
	set gvar(near)  200
	set gvar(end)  1000 
    }
	
    $cv.moteiCNamSpd-MMT configure -show 0
    $cv.moteiCNamSpd-MMT show
    update idletasks
}

proc moteiCNamSpdApplyRow {} {
    global gvar moteiNS moteiNSIdx moteiNSL

    set idx $moteiNSIdx
    set nsi [lsearch $moteiNSL $gvar(spdName)]
    if {$nsi != -1 && $nsi != $idx} {
	set gvar(spdName) ""
    }

    set gvar(far)  [abs $gvar(far)]
    set gvar(near) [abs $gvar(near)]
    set gvar(end)  [abs $gvar(end)]
    if {$gvar(far)  == 0} { set gvar(far)  1000 }
    if {$gvar(near) == 0} { set gvar(near)  200 }
    if {$gvar(end)  == 0} { set gvar(end)  1000 }

    set ns [list $gvar(spdName) $gvar(spdValue) $gvar(spdUnit) \
	    $gvar(far) $gvar(near) $gvar(end) 0.0]
    set moteiNS [lreplace $moteiNS $idx $idx $ns]
}

proc moteiCNamSpdMakeDefault { } {
    global gvar moteiDS moteiNS moteiNSIdx

    moteiCNamSpdApplyRow
    set moteiDS $moteiNSIdx
    set gvar(spdNameDefault) $gvar(spdName)
    set gvar(spdDefault)  "$gvar(spdValue) $gvar(spdUnit)"
}

proc moteiCNamSpdApply { } {
    global moteiNSIdx
    moteiCNamSpdApplyRow
    moteiCNamSpdWriteConfig
    moteiCNamSpdSelectRow [expr {$moteiNSIdx + 1}]
}

proc moteiCNamSpdReset { } {
    global moteiConfigBackup moteiNS moteiDS

    set moteiNS $moteiConfigBackup(namedSpeeds)
    set moteiDS $moteiConfigBackup(defaultSpeed)

    moteiCNamSpdWriteConfig
    moteiCNamSpdConfigWidgets
}

proc moteiCNamSpdRefresh { } {
    moteiCNamSpdReadConfig 1
    moteiCNamSpdConfigWidgets
}

proc moteiCNamSpdInsertAbove { } {
    global gvar cv moteiNS moteiNSIdx moteiNSL

    set idx $moteiNSIdx
    moteiCNamSpdApplyRow
    set ilast [expr {[llength $moteiNSL] - 1}]
    set moteiNS [lreplace $moteiNS $ilast $ilast]
    set new [list "speed#$idx" $idx "Enc/ms" 1000 200 1000 0.0]
    set moteiNS [linsert $moteiNS $idx $new]
    moteiCNamSpdUpdateList
    moteiCNamSpdSelectRow [expr {$idx + 1}]
}

proc moteiCNamSpdMoveUp { } {
    global gvar cv moteiNS moteiNSIdx moteiNSL

    set idx $moteiNSIdx
    moteiCNamSpdApplyRow
    if {$idx == 0} return
    set iprev [expr {$idx - 1}]
    set ns  [lindex $moteiNS $idx]
    set nsp [lindex $moteiNS $iprev]
    set moteiNS [lreplace $moteiNS $iprev $idx $ns $nsp]
    moteiCNamSpdUpdateList
    moteiCNamSpdSelectRow [expr {$iprev + 1}]
}

proc moteiCNamSpdMoveDn { } {
    global gvar cv moteiNS moteiNSIdx moteiNSL
   
    set idx $moteiNSIdx
    moteiCNamSpdApplyRow
    if {$idx == [llength $moteiNSL]} return
    set inext [expr {$idx + 1}]
    set ns  [lindex $moteiNS $idx]
    set nsn [lindex $moteiNS $inext]
    set moteiNS [lreplace $moteiNS $idx $inext $nsn $ns]
    moteiCNamSpdUpdateList
    moteiCNamSpdSelectRow [expr {$inext + 1}]
}

proc moteiCNamSpdDeleteRow { } {
    global gvar cv moteiNS moteiNSIdx moteiNSL

    set idx $moteiNSIdx
    set moteiNS [lreplace $moteiNS $idx $idx]
    set new [list "" 0 "" 0 0 0 0.0]
    lappend moteiNS $new
    moteiCNamSpdUpdateList
    moteiCNamSpdSelectRow [expr {$idx + 1}]
}

proc moteiCNamSpdConfigMonTim { } {
    global gvar cv

    if {$gvar(far)  <= 0} { set gvar(far)  1000 }
    if {$gvar(near) <= 0} { set gvar(near)  200 }
    if {$gvar(end)  <= 0} { set gvar(end)  1000 }

    $cv.moteiCNamSpd-MMT configure -show 1
    $cv.moteiCNamSpd-MMT show

    set w [$cv.moteiCNamSpd-MMT getClassPath]
    $w.cv.moteiMonTim-far  configure -variable gvar(far) 
    $w.cv.moteiMonTim-near configure -variable gvar(near) 
    $w.cv.moteiMonTim-end  configure -variable gvar(end) 

    update idletasks
}

proc moteiCNamSpdExitPanel { } {
}


#
# ___oOo___

