#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCSetAdr.tcl,v 2.43 2005/04/19 12:32:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   28/01/98  created 
#

#************************************************************************
#   NAME
#       moteiCSetAdr - set the buffer addresses 
# 
#   SYNOPSIS
#       moteiCSetAdr <args>
# 
#   DESCRIPTION
#       The panel moteiCSetAdr.pan contains a set of widgets allowing the 
#       setting the addresses of the buffers used by the user-defined external 
#       functions for HW Init and Unit Conversion. 
#       The file moteiCSetAdr.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the control panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiControl(7)
#       moteiControl(7) via the MenuBar Help->Extended Help button of the 
#           Control Panel
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCSetAdrReadConfig { } {
    global gvar moteiUF

    if {$gvar(addrMode) == 0} {
	# INIT
	set i "$gvar(dbPath):SERVER:INIT"
	if {[catch {seq_dbReadSymbolic "$i.actions"} ial]} { 
	    seq_errDisplay -suspend
	    exit 1
	}
	set lst {}
	foreach ia $ial {
	    if {[lindex $ia 0] == 8} {
		lappend lst [lrange $ia 1 2]
	    }
	}
    } else {
	# UNIT
	set u "$gvar(dbPath):SERVER:UNIT"
	if {[catch {seq_dbReadSymbolic "$u.unitConversion"} ucl]} { 
	    seq_errDisplay -suspend
	    exit 1
	}
	set lst {}
	foreach uc $ucl {
	    if {[lindex $uc 1] == 1} {
		lappend lst [lrange $uc 2 3]
	    }
	}
    }
    set moteiUF $lst
}

proc moteiCSetAdrConfigWidgets { } {
    global cv moteiUF moteiUFIdx 

    if {[llength $moteiUF] == 0} {
	set w $cv.moteiCSetAdr-UFL
	$w configure -itemList {}
	return
    }
    set moteiUFIdx 0
    moteiCSetAdrUpdate
    moteiCSetAdrSelect -1
}

proc moteiCSetAdrInitPanel { } {
    global gvar mControl

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:INIT/UNIT
    moteiCSetAdrReadConfig

    # configure widgets
    moteiCSetAdrConfigWidgets

    #------------------------------------------------
    # Create control instance
    #
    set mControl "$gvar(lcuEnv):$gvar(motorName)"
    moteiControlSend [list createControl $gvar(lcuEnv) $gvar(motorName)]
}

proc moteiCSetAdrUpdate { } {
    global cv moteiUF moteiUFIdx moteiUFL

    set ufl {}
    foreach uf $moteiUF {
	lassign $uf name addr
        if {![cequal $name ""]} { 
	    set line [format "%-32s          %08X" $name $addr]
	    lappend ufl $line
	}
    }
    set moteiUFL $ufl
    set w $cv.moteiCSetAdr-UFL
    $w configure -itemList $ufl
}

proc moteiCSetAdrSelect { {row 0} } {
    global gvar cv moteiUF moteiUFIdx

    set w $cv.moteiCSetAdr-UFL
    if {$row == -1} {
	set idx 0
    } else {
	set idx [$w getIndex]
    } 
    set moteiUFIdx  $idx
    moteiCSetAdrUpdate
    $w setSelection $idx

    lassign [lindex $moteiUF $idx] gvar(func) dec_addr
    set gvar(addr) [format "%08X" $dec_addr]
}

proc moteiCSetAdrApply { } {
    global gvar mControl moteiUF moteiUFIdx

    if {$gvar(addrMode) == 0} {
	set fct setInitAddr
    } else {
	set fct setConvAddr
    }
    scan $gvar(addr) "%x" dec_addr
    set command [list motorCommand $mControl $fct $gvar(func) 0 $dec_addr]
    moteiControlSend $command

    set uf [list $gvar(func) $dec_addr]
    set moteiUF [lreplace $moteiUF $moteiUFIdx $moteiUFIdx $uf]

    moteiCSetAdrUpdate
}

proc moteiCSetAdrExitPanel {} {

}

