#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCSetMon.tcl,v 2.43 2005/04/19 12:32:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   30/01/98  created
#

#************************************************************************
#   NAME
#       moteiCSetMon - set the motion monitoring times
# 
#   SYNOPSIS
#       moteiCSetMon <top>
# 
#   DESCRIPTION
#       This panel contains a set of widgets allowing the setting
#       of the motion monitoring times. 
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the control panel.
#       It is built using ITCL code, therefore is not editable with the 
#       panel editor.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiControl(7)
#       moteiControl(7) via the MenuBar Help->Extended Help button of the 
#           Control Panel
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiCSetMon { top } {
   global gvar

   if {[winfo exists $top]} { wm deiconify $top ; raise $top ; return }

   toplevel $top

   set f0 [frame $top.f0 ]
   set f1 [frame $top.f1 ]    
   set f2 [frame $top.f2 ]

   pack $f0 $f1 $f2 -pady 5 -fill both

   # Create/Pack  widgets

   label $f0.title -text "Motion Monitoring Times" \
                   -font "-Adobe-helvetica-bold-r-normal--24*"

   pack $f0.title -expand 1


   UifScale $f1.far  -length 150 -maxVal 5000 -minVal 100 -variable gvar(far)\
                     -label "Far Phase (ms)"
   UifScale $f1.near -length 150 -maxVal 2000 -minVal 20 -variable gvar(near)\
                     -label "Near Phase (ms)"
   UifScale $f1.stable -length 150 -maxVal 5000 -minVal 100 \
                       -variable gvar(stable) -label "End Phase (ms)"

   UifArrowsButton $f1.arr1 -variable gvar(far) -stateVar gvar(dfVar)
   UifArrowsButton $f1.arr2 -variable gvar(near) -stateVar gvar(dnVar)
   UifArrowsButton $f1.arr3 -variable gvar(stable) -stateVar gvar(ieVar)

   pack $f1.far  -side left -padx 10 ;   pack $f1.arr1 -side left
   pack $f1.near -side left -padx 10 ;   pack $f1.arr2 -side left
   pack $f1.stable -side left -padx 10 ; pack $f1.arr3 -side left


   UifButton $f2.close -label Close -command "destroy $top"
   pack $f2.close -expand 1 

   # Variable Initialization

   if {![info exist gvar(far)]}    { set gvar(far)    1000 }
   if {![info exist gvar(near)]}   { set gvar(near)    200 }
   if {![info exist gvar(stable)]} { set gvar(stable) 1000 }

   if {$gvar(far)    < 100} { set gvar(far)    1000 }
   if {$gvar(near)   <  20} { set gvar(near)    200 }
   if {$gvar(stable) < 100} { set gvar(stable) 1000 }

}
#
# ___oOo___
