#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCSignal.tcl,v 2.43 2005/04/19 12:32:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux    FEB 98   created 
#

#************************************************************************
#   NAME
#       moteiCSignal - configure the Digital I/O Signals
# 
#   SYNOPSIS
#       moteiCSignal <args>
# 
#   DESCRIPTION
#       The panel moteiCSignal.pan contains a set of widgets allowing the 
#       configuration of the Digital I/O Signals for Brake and Interlock
#       handling. 
#       The file moteiCSignal.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCSignal(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiCSignalReadConfig { } {
    global gvar 
    global moteiDIOS moteiBkpDIOS moteiBaB moteiBkpBaB

    set c "$gvar(dbPath):SERVER:CONF"
    set moteiDIOS [seq_dbReadSymbolic "$c.signals"]
    set moteiBkpDIOS $moteiDIOS
    set moteiBaB [seq_dbReadSymbolic "$c.brakeAtBoot"]
    set moteiBkpBaB $moteiBaB
}

proc moteiCSignalWriteConfig { } {
    global gvar moteiDIOS moteiBaB

    set c "$gvar(dbPath):SERVER:CONF"
    if {[catch {seq_dbWriteSymbolic "$c.signals" $moteiDIOS}] != 0} {
	seq_errDisplay -suspend
    }
    if {[catch {seq_dbWriteSymbolic "$c.brakeAtBoot" $moteiBaB}] != 0} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

proc moteiCSignalConfigWidgets { } {
    global gvar cv
    global moteiDIOS moteiBaB

    #
    # preset signals
    #
    set gvar(brakeActionSignal) -1
    set gvar(brakeStatusSignal) -1
    set gvar(interlockSignal)   -1
    set gvar(electrHomeSignal)  -1
    set gvar(brakeActionDevice) ""
    set gvar(brakeStatusDevice) ""
    set gvar(interlockDevice)   ""
    set gvar(electrHomeDevice)  ""
    set gvar(brakeActionBit)    0
    set gvar(brakeStatusBit)    0
    set gvar(interlockBit)      0
    set gvar(electrHomeBit)     0
    set gvar(brakeActionLogic)  0
    set gvar(brakeStatusLogic)  0
    set gvar(interlockLogic)    0
    set gvar(electrHomeLogic)   0
    set gvar(brakeAtBoot)       $moteiBaB

    #
    # for all widgets, configure as from DB
    #
    for {set i 0} {$i < 4} {incr i} {
	set dio [lindex $moteiDIOS $i]
	lassign $dio signal device startBit level
        if {$signal != -1} {
	    switch $signal {
		0 { set what brakeAction }
		1 { set what brakeStatus }
		2 { set what interlock   }
		3 { set what electrHome  }
	    }
	    set gvar(${what}Signal) $signal
	    set gvar(${what}Device) $device
	    set gvar(${what}Bit)    [expr {$startBit + 1}]
	    set gvar(${what}Logic)  $level
	} 
    }
    for {set i 0} {$i < 4} {incr i} {
	switch $i {
	    0 { set what brakeAction }
	    1 { set what brakeStatus }
	    2 { set what interlock   }
	    3 { set what electrHome  }
	}
    }
    moteiCSignalUpdatePanel 0
}

proc moteiCSignalUpdatePanel { {baLogic 0} } {
    global gvar cv
    
    set wdg $cv.moteiCSignal-BaB
    if {$gvar(brakeActionLogic)} {
	$wdg enable
    } else {
	$wdg disable
    }
}

proc moteiCSignalInitPanel { } {
    global gvar cv
    global moteiDIOS moteiBkpDIOS moteiBaB moteiBkpBaB

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCSignalReadConfig

    # configure widgets
    moteiCSignalConfigWidgets
}

#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCSignalApply { } {
    global gvar moteiDIOS moteiBaB

    set ba "Brake Action" ; set bs "Brake Status" ; set is "Interlock"
    set bab $gvar(brakeActionBit)
    set bsb $gvar(brakeStatusBit)
    set isb $gvar(interlockBit)
    set ehb $gvar(electrHomeBit)

    for {set i 0} {$i < 4} {incr i} {
	switch $i {
	    0 { set what brakeAction ; set signal "Brake Action" }
	    1 { set what brakeStatus ; set signal "Brake Status" }
	    2 { set what interlock   ; set signal "Interlock"    }
	    3 { set what electrHome  ; set signal "Electr. Home" }
	}
	if {($gvar(${what}Device) == "") || ($gvar(${what}Logic) == 0)} {
	    set gvar(${what}Signal) -1
	} else {
	    set gvar(${what}Signal) $i
	}
    }

    for {set i 0} {$i < 4} {incr i} {
	switch $i {
	    0 { set wi brakeAction ; set si "Brake Action" }
	    1 { set wi brakeStatus ; set si "Brake Status" }
	    2 { set wi interlock   ; set si "Interlock"    }
	    3 { set wi electrHome  ; set si "Electr. Home" }
	}
	if {$gvar(${wi}Signal) == -1} continue
	set bit $gvar(${wi}Bit)
	if {$bit < 1 || $bit > 64} {
	    set msg [format "%s\n\t%s" "Signal '$si' Invalid bit #$bit" \
		    "Out of range 1 to 64"]
	    moteiDialogBox "ERROR" $msg
	    return
	}
	for {set j [expr {$i+1}]} {$j < 4} {incr j} {
	    switch $j {
		1 { set wj brakeStatus ; set sj "Brake Status" }
		2 { set wj interlock   ; set sj "Interlock"    }
		3 { set wj electrHome  ; set sj "Electr. Home" }
	    }
	    if {$gvar(${wj}Signal) == -1} continue
	    if {$gvar(${wj}Bit) == $gvar(${wi}Bit)} {
		moteiDialogBox "ERROR" "Signals '$si' and '$sj' mapped on same bit #$gvar(${wi}Bit)"
		return
	    }
	}
    }

    for {set i 0} {$i < 4} {incr i} {
	switch $i {
	    0 { set wi brakeAction }
	    1 { set wi brakeStatus }
	    2 { set wi interlock   }
	    3 { set wi electrHome  }
	}
	set dev $gvar(${wi}Device)
	set bit [expr {$gvar(${wi}Bit) - 1}]
	set log $gvar(${wi}Logic)
	set dio$i [list $gvar(${wi}Signal) $dev $bit $log]
    }

    set moteiDIOS [list $dio0 $dio1 $dio2 $dio3]

    set moteiBaB $gvar(brakeAtBoot)

    moteiCSignalWriteConfig
}

proc moteiCSignalReset { } {
    global moteiBkpDIOS moteiDIOS moteiBaB moteiBkpBaB

    set moteiDIOS $moteiBkpDIOS
    set moteiBaB $moteiBkpBaB

    moteiCSignalWriteConfig
    moteiCSignalConfigWidgets
}

proc moteiCSignalRefresh { } {
    #
    # read DB configuration
    #
    moteiCSignalReadConfig

    # configure widgets
    moteiCSignalConfigWidgets
}

proc moteiCSignalHelp { } {
}

proc moteiCSignalExitPanel { } {
}

#
# ___oOo___


