#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCSpePos.tcl,v 2.43 2005/04/19 12:32:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    13/03/97  defined gvar(relPath) to support dbTableDump
# imunoz    04/03/97  uses moteiDbFancyTableWdg to interact with the DB,
#                     lot of code saved (and deleted).
# imunoz  08/05/96  disabled posType SPD in moteiPosTypeUob widget
# imunoz  11/23/95  created 
#

#************************************************************************
#   NAME
#       moteiCSpePos - configure the HW Special Positions
# 
#   SYNOPSIS
#       moteiCSpePos <args>
# 
#   DESCRIPTION
#       The panel moteiCSpePos.pan contains a set of widgets allowing the 
#       configuration of the HW Special Positions. 
#       The file moteiCSpePos.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCSpePos(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiCSpePosReadConfig { {mode 0} } {
    global gvar cv
    global moteiAT moteiET moteiPU moteiSP moteiBkpSP

    set c "$gvar(dbPath):SERVER:CONF"
    set u "$gvar(dbPath):SERVER:UNIT"
    set moteiAT [seq_dbReadSymbolic "$c.axis"]
    set moteiET [seq_dbReadSymbolic "$c.encoder(0,type)"]
    set moteiPU [seq_dbReadSymbolic "$u.defaultUnits(0,position)"]
    if {$moteiPU != "Enc"} { lappend moteiPU "Enc" }

    if {$mode == 0} {
	set moteiSP [seq_dbReadSymbolic "$c.specialPositions"]
	set moteiBkpSP $moteiSP
    }
}

proc moteiCSpePosWriteConfig { } {
    global gvar moteiSP

    set c "$gvar(dbPath):SERVER:CONF"
    if {[catch {seq_dbWriteSymbolic "$c.specialPositions" $moteiSP}] != 0} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

proc moteiCSpePosConfigWidgets { } {
    global gvar cv
    global moteiAT moteiET moteiPU moteiSP

    set LST [list LHW UHW REF IND HOM]
    set Lst [list Lhw Uhw Ref Ind Hom]

    set at $moteiAT
    set et $moteiET
    #
    # for all widgets, configure as from DB
    #
    for {set i 0} {$i < 5} {incr i} {
	set spc [lindex $moteiSP [expr {$i + 2}]]
	set Sp [lindex $Lst $i]
	set SP [lindex $LST $i]
	set v "spePos${Sp}Class_"
	lassign $spc gvar(${v}flag) gvar(${v}type) gvar(${v}value) gvar(${v}unit)
	set w [$cv.moteiCSpePos-${SP} getClassPath]
	$w.cv.unit configure -itemList $moteiPU
	if {[lsearch $moteiPU $gvar(${v}unit)] == -1} {
	    set gvar(${v}unit) [lindex $moteiPU 0]
	}
	if {[lsearch [list 1 2 3 4 5 6 7] $gvar(${v}type)] == -1} {
	    set gvar(${v}type) 1
	}

	#
	# for LIN axis, enable  posType LHW & UHW
	# for CIR axis, disable posType LHW & UHW
	#
	if {$at == 1} {
	    $w.cv.type enableEntry  2
	    $w.cv.type enableEntry  3
	} else {
	    $w.cv.type disableEntry 2
	    $w.cv.type disableEntry 3
	}
	#
	# for no encoder, disable posType IND & HOM
	# else enable posType IND & HOM
	#
	if {$et == 0} {
	    $w.cv.type disableEntry 5
	    $w.cv.type disableEntry 6
	} else {
	    $w.cv.type enableEntry  5
	    $w.cv.type enableEntry  6
	}
	#
	# disable its own posType
	#
	$w.cv.type disableEntry [expr {$i + 2}]
    }
    
    #
    # for LIN axis, enable widgets LHW & UHW
    # else disable them
    #
    for {set i 0} {$i < 2} {incr i} {
	set Sp [lindex $Lst $i]
	set SP [lindex $LST $i]
	set w [$cv.moteiCSpePos-${SP} getClassPath]
	if {$at == 1} {
	    set wdgFlag normal
	} else {
	    set gvar(spePos${Sp}Class_flag) 0
	    set wdgFlag disabled
	}
	$w.cv.flag  configure -state $wdgFlag
	$w.cv.type  configure -state $wdgFlag
	$w.cv.value configure -state $wdgFlag
	$w.cv.unit  configure -state $wdgFlag
    }
    #
    # For no encoder, disable widgets IND & HOM
    # else enable them
    #
    for {set i 3} {$i < 5} {incr i} {
	set Sp [lindex $Lst $i]
	set SP [lindex $LST $i]
	set w [$cv.moteiCSpePos-${SP} getClassPath]
	if {$et != 0} {
	    set wdgFlag normal
	} else {
	    set gvar(spePos${Sp}Class_flag) 0
	    set wdgFlag disabled
	}
	$w.cv.flag  configure -state $wdgFlag
	$w.cv.type  configure -state $wdgFlag
	$w.cv.value configure -state $wdgFlag
	$w.cv.unit  configure -state $wdgFlag
    }

    #
    # refresh widgets
    #
    foreach SP $LST {
	$cv.moteiCSpePos-${SP} show
    }
}

proc moteiCSpePosInitPanel { } {
    global gvar cv
    global moteiAT moteiSP moteiBkpSP

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCSpePosReadConfig

    # configure widgets
    moteiCSpePosConfigWidgets
}

#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCSpePosApply { } {
    global gvar moteiSP

    set LST [list LHW UHW REF IND HOM]
    set Lst [list Lhw Uhw Ref Ind Hom]

    for {set i 0} {$i < 5} {incr i} {
	set Sp [lindex $Lst $i]
	set w "spePos${Sp}Class_"
	set spc [list $gvar(${w}flag) $gvar(${w}type) $gvar(${w}value) $gvar(${w}unit)]
	set j [expr {$i + 2}]
	set moteiSP [lreplace $moteiSP $j $j $spc]
    }
    moteiCSpePosWriteConfig
}

proc moteiCSpePosReset { } {
    global moteiBkpSP moteiSP

    set moteiSP $moteiBkpSP

    moteiCSpePosWriteConfig
    moteiCSpePosConfigWidgets
}

proc moteiCSpePosRefresh { } {
    moteiCSpePosReadConfig 1
    moteiCSpePosConfigWidgets
    
}

proc moteiCSpePosHelp { } {
    
}

proc moteiCSpePosExitPanel { } {
}

#
# ___oOo___


