#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCTwoStp.tcl,v 2.43 2005/04/19 12:32:56 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  27/10/95  created
#

#************************************************************************
#   NAME
#       moteiCTwoStp - configure the Offset & Speed for Two-Step motions
# 
#   SYNOPSIS
#       moteiCTwoStp <args>
#       The panel moteiCTwoStp.pan contains a set of widgets allowing the 
#       configuration of the Speed and Offset used for Two-Step motions. 
#       The file moteiCTwoStp.tcl contains all associated procedures.
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCTwoStp(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiCTwoStpReadConfig { {mode 0} } {
    global gvar moteiConfigParams 
    global moteiTSO moteiBkpTSO

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set c "$gvar(dbPath):SERVER:CONF"
	if {[catch {seq_dbReadSymbolic "$c.twoStepOffset"} tso]} {
	    seq_errDisplay -suspend
	    return "FAILURE"
	}
	set moteiTSO    $tso
	set moteiBkpTSO $tso
    }
}

proc moteiCTwoStpWriteConfig { } {
    global gvar moteiTSO

    set c "$gvar(dbPath):SERVER:CONF"
    if {[catch {seq_dbWriteSymbolic "$c.twoStepOffset" $moteiTSO}] != 0} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

proc moteiCTwoStpConfigWidgets { } {
    global gvar cv speedFrame
    global moteiConfigParams moteiTSO
    
    set DPU [lindex $moteiConfigParams(posUnit) 0]
    set DSU [lindex $moteiConfigParams(spdUnit) 0]

    #
    # for all widgets, configure as from DB
    #
    lassign $moteiTSO gvar(offset) gvar(posUnit) spdHow \
	    gvar(spdVal) gvar(spdUnit) gvar(spdName) gvar(spdIdx) \
	    gvar(spdFar) gvar(spdNear) gvar(spdEnd)

    if {[lsearch $moteiConfigParams(posUnit) $gvar(posUnit)] == -1} {
	set gvar(posUnit) $DPU
    }
    $cv.moteiCTwoStp-Unit configure -itemList $moteiConfigParams(posUnit) -default $gvar(posUnit)

    if {[lsearch [list 1 2 3 4] $spdHow] == -1} {
	set spdHow 2
	set gvar(spdVal) 0.0
	set gvar(spdUnit) $DSU
    }
    if {[lsearch $moteiConfigParams(spdUnit) $gvar(spdUnit)] == -1} {
	set gvar(spdUnit) $DSU
    }
    if {[lsearch $moteiConfigParams(namedSpdList) $gvar(spdName)] == -1} {
	set gvar(spdName) [lindex $moteiConfigParams(namedSpdList) 0]
    }
    if {[lsearch $moteiConfigParams(indexSpdList) $gvar(spdIdx)] == -1} {
	set gvar(spdIdx) [lindex $moteiConfigParams(indexSpdList) 0]
    }
    moteiCTwoStpUpdateVelocityGui $spdHow
}

proc moteiCTwoStpUpdateVelocityGui { guiMode } {
    global cv gvar speedFrame moteiConfigParams

    set speedFrame [expr {$guiMode - 1}]
    set gvar(speedHow) $guiMode

    # Configure option buttons with corresponding lists
    switch $speedFrame {
	0 {
	    # Configure default speed
	    set gvar(speedClass_defaultSpdIndex) $moteiConfigParams(defaultSpeed)
	    set gvar(speedClass_defaultSpdName) [lindex $moteiConfigParams(namedSpdList) $moteiConfigParams(defaultSpeed)]
	    set DSV $moteiConfigParams(spdDefault)
	    set DSU $moteiConfigParams(spdDefaultUnit)
	    set gvar(speedClass_defaultSpdValue) "$DSV $DSU"
	}
	1 {
	    # Configure list of units
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.valueUnit configure \
		    -itemList $moteiConfigParams(spdUnit) \
		    -default  $moteiConfigParams(spdDefaultUnit)
	    set gvar(speedClass_value) $gvar(spdVal)
	    set gvar(speedClass_valueUnit) $gvar(spdUnit)
	    moteiConvertVelocityRange
	}
	2 {
	    # Configure list of names
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.nameList configure \
		    -itemList $moteiConfigParams(namedSpdList) \
		    -default  [lindex $moteiConfigParams(namedSpdList) 0]
	    set gvar(speedClass_nameValue) $gvar(spdName)
	}
	3 {
	    # Configure list of index
	    set w [$cv.speedFrame getClassPath]
	    $w.cv.indexList configure \
		    -itemList $moteiConfigParams(indexSpdList)  \
		    -default  [lindex $moteiConfigParams(indexSpdList) 0]
	    set gvar(speedClass_indexValue) $gvar(spdIdx)
	}
    }
    $cv.speedFrame show
}

proc  moteiCTwoStpInitPanel { } {

    # make this application known to motei
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown

    # read database configuration SERVER:CONF
    moteiCTwoStpReadConfig

    # configure widgets
    moteiCTwoStpConfigWidgets
}

#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCTwoStpApply { } {
    global gvar moteiTSO

    set moteiTSO [list $gvar(offset) $gvar(posUnit) $gvar(speedHow)]
    switch $gvar(speedHow) {
	1 { # by Default
	    lappend moteiTSO 0.0 "Enc/ms" "" 0 
	}
	2 { # by Value
	    lappend moteiTSO $gvar(speedClass_value) $gvar(speedClass_valueUnit) "" 0 
	}
	3 { # by Name
	    lappend moteiTSO 0.0 "Enc/ms" $gvar(speedClass_nameValue) 0
	}
	4 { # by Index
	    lappend moteiTSO 0.0 "Enc/ms" "" $gvar(speedClass_indexValue) 
	}
    }
    lappend moteiTSO $gvar(spdFar) $gvar(spdNear) $gvar(spdEnd) 0 0.0

    moteiCTwoStpWriteConfig
}

proc moteiCTwoStpReset { } {
    global moteiBkpTSO moteiTSO

    set moteiTSO $moteiBkpTSO

    moteiCTwoStpWriteConfig
    moteiCTwoStpConfigWidgets
}

proc moteiCTwoStpRefresh { } {
    moteiCTwoStpReadConfig 1
    moteiCTwoStpConfigWidgets
}

proc moteiCTwoStpExitPanel { } {
}

#
# ___oOo___
