#*************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCUnit.tcl,v 2.43 2005/04/19 12:32:56 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    13/03/97  defined gvar(relPath) to support dbTableDump
# imunoz    11/23/95  created for cmm
# imunoz    04.03.97  uses moteiDbTableWdg to interact with the DB,
#                     lot of code saved (and deleted).
#
#************************************************************************
#   NAME
#       moteiCUnit - configure the Unit conversion methods
# 
#   SYNOPSIS
#       moteiCUnit <args>
# 
#   DESCRIPTION
#       The panel moteiCUnit.pan contains a set of widgets allowing the 
#       configuration of the Unit conversion methods. 
#       The file moteiCUnit.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCUnit(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCUnitReadConfig { {mode 0} } {
    global gvar moteiConfigParams
    global moteiUC moteiDU

    moteiConfigReadDbConfig

    if {$mode == 0} {
	set moteiDU $moteiConfigParams(defaultUnits)
	set moteiUC $moteiConfigParams(unitConversion)
    }
}

proc moteiCUnitWriteConfig { } {
    global gvar moteiUC moteiUCL moteiDU

    set u "$gvar(dbPath):SERVER:UNIT"

    set moteiDU [list $gvar(defPosUnit) $gvar(defSpdUnit) $gvar(defCurUnit)]

    if {[catch {seq_dbWriteSymbolic "$u.defaultUnits" $moteiDU}] != 0} {
	seq_errDisplay -suspend
    }
    set ucn [llength $moteiUCL] ; set attr "unitConversion"
    for {set n 0} {$n < $ucn} {incr n} {
	set uc [lindex $moteiUC $n]
	if {[catch {seq_dbWriteSymbolic "$u.${attr}($n)" $uc}] != 0} {
	    seq_errDisplay -suspend
	}
    }
    moteiConfigReinstall
}

proc moteiCUnitConfigWidgets { } {
    global gvar cv moteiConfigParams moteiDU moteiUC moteiUCL moteiUCIdx 

    set moteiUCIdx 0
    moteiCUnitUpdateList
    moteiCUnitSelectRow -1

    set gvar(defPosUnit) [lindex $moteiDU 0]
    set gvar(defSpdUnit) [lindex $moteiDU 1]
    set gvar(defCurUnit) [lindex $moteiDU 2]
}

proc moteiCUnitInitPanel { } {

    #enable ext commands
    seq_ccsAsyncInput open

    # install close handler
    vccStdMain shutDown
    
    # read database configuration SERVER:CONF
    moteiCUnitReadConfig

    # configure widgets
    moteiCUnitConfigWidgets
}

#---------------------------------------
# Internal buttons
#---------------------------------------
proc moteiCUnitMakeDefaultUnit { } {
    global gvar cv

    if {$gvar(toUnitSel) == "Other"} return
    switch -- $gvar(toUnitSel) {
	"Enc" {
	    set gvar(defPosUnit) $gvar(fromUnit)
	}
	"Enc/ms" -
	"Ref" {
	    set gvar(defSpdUnit) $gvar(fromUnit)
	}
	"Dig" {
	    set gvar(defCurUnit) $gvar(fromUnit)
	}
    }
}

proc moteiCUnitClearEntry { } {
    global gvar cv

    if {[cequal $gvar(fromUnit) $gvar(defPosUnit)]} {
	set gvar(defPosUnit) "Enc"
    }
    if {[cequal $gvar(fromUnit) $gvar(defSpdUnit)]} {
	set gvar(defSpdUnit) "Enc/ms"
    }
    if {[cequal $gvar(fromUnit) $gvar(defCurUnit)]} {
	set gvar(defCurUnit) "Dig"
    }
	
    set gvar(fromUnit)  ""  
    set gvar(toUnit)    ""
    set gvar(toUnitSel) "Other"
    set gvar(otherUnit) ""
    set gvar(convMethod) 0
    set gvar(convName) "" 
    $cv.other enable
    $cv.moteiCUnit-MkDef disable
    moteiCUnitApplyRow
    moteiCUnitUpdateList
}

proc moteiCUnitSetBoardUnits { } {
    global gvar

    set gvar(defPosUnit) "Enc"
    set gvar(defSpdUnit) "Enc/ms"
    set gvar(defCurUnit) "Dig"
}

proc moteiCUnitSelectToUnit { {unit ""} } {
    global gvar cv

    set stdUnitList [list Enc "Enc/ms" Ref Dig]
    if {[cequal $unit ""]} {
	set u $gvar(toUnit)
    } else {
	set u $unit
    }
    if {[lsearch $stdUnitList $u] == -1} {
	set gvar(toUnitSel) "Other"
	set gvar(otherUnit) $u
	$cv.other enable
	$cv.moteiCUnit-MkDef disable
    } else {
	set gvar(toUnit)    $u
	set gvar(toUnitSel) $u
	set gvar(otherUnit) ""
	$cv.other disable
	$cv.moteiCUnit-MkDef enable
    }
}

proc moteiCUnitSelectMethod { {method 1} } {
    global cv gvar methodFrame moteiIP moteiIPIdx

    set gvar(convMethod) $method
    set gvar(moteiCUnit-Method) $method
    set methodFrame [expr {$method - 1}]
    set w [$cv.methodFrame getClassPath]
    switch $method {
	1 { # EXTERN
	    $w.cv.moteiConvExtern-FN configure -variable gvar(function)
	}
	2 { # LINEAR
	    if {$gvar(slope) == 0} {
		set gvar(slope)  1.0
		set gvar(offset) 0.0
	    }
	    set fmt "%s"
	    set gvar(methodClass_Title) \
		    "<Y $gvar(toUnit)> = <X $gvar(fromUnit)> * slope + offset"
	    if {[string first "/" $gvar(toUnit)] != -1} {
		set fmt "\[%s\]"
	    }
	    append fmt " / "
	    if {[string first "/" $gvar(fromUnit)] != -1} {
		append fmt "\[%s\]"
	    } else {
		append fmt "%s"
	    }

	    set gvar(methodClass_XUnit) [format $fmt $gvar(toUnit) $gvar(fromUnit)]
	    set gvar(methodClass_toUnit) $gvar(toUnit)
	    $w.cv.moteiConvLinear-SL configure -variable gvar(slope)
	    $w.cv.moteiConvLinear-OS configure -variable gvar(offset)
	}
	3 { # INTERP
	    if {$gvar(numInterpPoints) < 2} {
		set gvar(numInterpPoints) 2
                set moteiIP [lreplace $moteiIP 0 1 {0 0} {1 1}]
	    }
	    set iw $w.cv.moteiConvInterp-IPL
	    $iw setBind <B1-ButtonRelease> moteiConvInterpSelect
	    set gvar(methodClass_fromUnit) $gvar(fromUnit)
	    set gvar(methodClass_toUnit)   $gvar(toUnit)
	    set gvar(methodClass_NPT)      $gvar(numInterpPoints)
	    set moteiIPIdx 0
	    moteiConvInterpUpdate
	    moteiConvInterpSelect -1
	}
    }
    $cv.methodFrame show
}

#---------------------------------------
# Interpolation
#---------------------------------------
proc moteiConvInterpUpdate { } {
    global gvar cv moteiIP moteiIPIdx moteiIPL

    set ipl {}
    for {set i 0} {$i < $gvar(numInterpPoints)} {incr i} {
	lassign [lindex $moteiIP $i] X Y
	set line [format "%2d %8.2f %8.2f" [expr {$i + 1}] $X $Y]
	lappend ipl $line
    }
    set moteiIPL $ipl

    set gvar(methodClass_NPT) $gvar(numInterpPoints)
    set w [$cv.methodFrame getClassPath]
    set iw $w.cv.moteiConvInterp-IPL
    $iw configure -itemList $ipl
    $cv.methodFrame show
    
    if {$gvar(numInterpPoints) <= 2} {
	$w.cv.moteiConvInterp-DEL disable
    } else {
	$w.cv.moteiConvInterp-DEL enable
    }
    if {$gvar(numInterpPoints) == 50} {
	$w.cv.moteiConvInterp-INS disable
    } else {
	$w.cv.moteiConvInterp-INS enable
    }
    if {$moteiIPIdx == 0} {
	$w.cv.moteiConvInterp-MUP disable
    } else {
	$w.cv.moteiConvInterp-MUP enable
    }
    if {$moteiIPIdx == [expr {$gvar(numInterpPoints) - 1}]} {
	$w.cv.moteiConvInterp-MDN disable
    } else {
	$w.cv.moteiConvInterp-MDN enable
    }
}

proc moteiConvInterpSelect { {row 0} } {
    global gvar cv moteiIP moteiIPIdx

    set w [$cv.methodFrame getClassPath]
    set iw $w.cv.moteiConvInterp-IPL
    if {$row == -1} {
	set idx 0
    } elseif {$row == 0} {
	set idx [$iw getIndex]
	moteiConvInterpApply
    } else {
	set idx [expr {$row - 1}]
    }
    set moteiIPIdx   $idx
    moteiConvInterpUpdate
    $iw setSelection $idx

    lassign [lindex $moteiIP $idx] gvar(methodClass_X) gvar(methodClass_Y)
    set gvar(methodClass_NP) [expr {$idx + 1}]
}

proc moteiConvInterpApply { } {
    global gvar moteiIP moteiIPIdx

    set idx $moteiIPIdx
    set XY [list $gvar(methodClass_X) $gvar(methodClass_Y)]
    set moteiIP [lreplace $moteiIP $idx $idx $XY]

}

proc moteiConvInterpInsert { } {
    global gvar cv moteiIP moteiIPIdx

    set idx $moteiIPIdx
    moteiConvInterpApply
    if {$idx >= $gvar(numInterpPoints)} return
    incr gvar(numInterpPoints) 1
    set moteiIP [lreplace $moteiIP 49 49]
    set moteiIP [linsert $moteiIP $idx [list 0 0]]
    moteiConvInterpUpdate
    moteiConvInterpSelect [expr {$idx + 1}]
}

proc moteiConvInterpMoveUp { } {
    global gvar cv moteiIP moteiIPIdx

    set idx $moteiIPIdx
    moteiConvInterpApply
    if {$idx == 0} return
    set iprev [expr {$idx - 1}]
    set ip  [lindex $moteiIP $idx]
    set ipp [lindex $moteiIP $iprev]
    set moteiIP [lreplace $moteiIP $iprev $idx $ip $ipp]
    moteiConvInterpUpdate
    moteiConvInterpSelect [expr {$iprev + 1}]
}

proc moteiConvInterpMoveDn { } {
    global gvar cv moteiIP moteiIPIdx moteiIPL
   
    set idx $moteiIPIdx
    moteiConvInterpApply
    set inext [expr {$idx + 1}]
    if {$inext >= $gvar(numInterpPoints)} return
    set ip  [lindex $moteiIP $idx]
    set ipn [lindex $moteiIP $inext]
    set moteiIP [lreplace $moteiIP $idx $inext $ipn $ip]
    moteiConvInterpUpdate
    moteiConvInterpSelect [expr {$inext + 1}]
}

proc moteiConvInterpDelete { } {
    global gvar cv moteiIP moteiIPIdx moteiIPL

    set idx $moteiIPIdx
    if {$idx >= $gvar(numInterpPoints)} return
    if {$gvar(numInterpPoints) == 2} return
    incr gvar(numInterpPoints) -1
    set moteiIP [lreplace $moteiIP $idx $idx]
    lappend moteiIP [list 0 0]
    if {$gvar(methodClass_NP) >= $gvar(numInterpPoints)} {
	incr gvar(methodClass_NP) -1
	incr idx -1
    }
    moteiConvInterpUpdate
    moteiConvInterpSelect [expr {$idx + 1}]
}

#---------------------------------------
# List handling
#---------------------------------------
proc moteiCUnitUpdateList { } {
    global cv moteiUC moteiUCL

    set ucl {}
    foreach uc $moteiUC {
        set name [lindex $uc 0]
        if {[cequal $name ""]} { set name "<empty>" }
	lappend ucl $name
    }
    set moteiUCL $ucl
    set w $cv.moteiCUnit-UCL
    $w configure -itemList $ucl
}

proc moteiCUnitSelectRow { {row 0} } {
    global gvar cv moteiUC moteiUCIdx moteiIP moteiConfigParams

    set w $cv.moteiCUnit-UCL
    if {$row == -1} {
	set idx 0
    } elseif {$row == 0} {
	set idx [$w getIndex]
	moteiCUnitApplyRow
    } else {
	set idx [expr {$row - 1}]
    }
    set moteiUCIdx  $idx
    moteiCUnitUpdateList
    $w setSelection $idx
    set uc [lindex $moteiUC $idx]
    set ucb [lrange $uc 0 1]
    lassign $ucb gvar(convName) gvar(convMethod) 
    set i [string first "To" $gvar(convName)]
    set gvar(fromUnit) [string range $gvar(convName) 0 [expr {$i - 1}]]
    set gvar(toUnit)   [string range $gvar(convName) [expr {$i + 2}] end]

    # EXTERN
    set ucx [lrange $uc 2 3]
    lassign $ucx gvar(function) dummy
    # LINEAR
    set ucl [lrange $uc 4 5]
    lassign $ucl gvar(offset) gvar(slope)
    # INTERP
    set gvar(numInterpPoints) [lindex $uc 6]
    set uci [lrange $uc 7 106]
    set moteiIP {}
    for {set i 0} {$i < 50} {incr i} {
	set ix [expr {2 * $i}]
	set XY [lrange $uci $ix [expr {$ix + 1}]]
	lappend moteiIP $XY
    }
	    
    moteiCUnitSelectToUnit
    moteiCUnitSelectMethod $gvar(convMethod)
}

proc moteiCUnitApplyRow {} {
    global gvar moteiUC moteiUCIdx moteiUCL moteiIP

    if {[cequal $gvar(toUnitSel) "Other"]} {
	set convName "$gvar(fromUnit)To$gvar(otherUnit)"
	if { [cequal $convName "To"] } { set convName "" }
    } else {
	set convName "$gvar(fromUnit)To$gvar(toUnit)"
    }
    set idx $moteiUCIdx

    # check if conversion method is unique
    set uci [lsearch $moteiUCL $convName]
    if {$uci != -1 && $uci != $idx} {
        moteiDialogBox "ERROR" "Conversion method '$convName' not unique"
        return
    }
    set gvar(convName) $convName
    set uc [list $gvar(convName) $gvar(convMethod)]
    switch $gvar(convMethod) {
	0 { # Unset
	    lappend uc "" 0 0 0 0
	}
	1 { # EXTERN
	    lappend uc $gvar(function) 0 0 0 0
	}
	2 { # LINEAR
	    lappend uc "" 0 $gvar(offset) $gvar(slope) 0
	}
	3 { # INTERP
	    lappend uc "" 0 0 0 $gvar(numInterpPoints)
	}
    }
    if {$gvar(convMethod) == 3} {
	for {set i 0} {$i < $gvar(numInterpPoints)} {incr i} {
	    lappend uc [join [lindex $moteiIP $i] " "]
	}
	for {set i $gvar(numInterpPoints)} {$i < 50} {incr i} {
	    lappend uc 0 0
	}
	set uc [join $uc " "]
    } else {
	for {set i 0} {$i < 50} {incr i} {
	    lappend uc 0 0
	}
    }
    lappend uc 0

    set moteiUC [lreplace $moteiUC $idx $idx $uc]
}

#-----------------------------------
# Action Buttons procedures
#-----------------------------------
proc moteiCUnitApply { } {
    global moteiUCIdx

    moteiCUnitApplyRow
    moteiCUnitWriteConfig
    moteiCUnitSelectRow [expr {$moteiUCIdx + 1}]
}

proc moteiCUnitReset { } {
    global moteiConfigBackup moteiUC moteiDU

    set moteiDU $moteiConfigBackup(defaultUnits)
    set moteiUC $moteiConfigBackup(unitConversion)

    moteiCUnitConfigWidgets
    moteiCUnitWriteConfig
}

proc moteiCUnitRefresh { } {
    moteiCUnitReadConfig 1
    moteiCUnitConfigWidgets
}

proc moteiCUnitExitPanel { } {
}

#
# ___oOo___
