#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiCVME4SA.tcl,v 2.43 2005/04/19 12:32:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  26/09/95  created
#

#************************************************************************
#   NAME
#       moteiCVME4SA - configure the Servo Amplifier ESO VME4SA
# 
#   SYNOPSIS
#       moteiCVME4SA <args>
# 
#   DESCRIPTION
#       The panel moteiCVME4SA.pan contains a set of widgets allowing the 
#       configuration of the Servo Amplifier ESO VME4SA. 
#       The file moteiCVME4SA.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main configuration panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiConfig(7)
#       moteiCVME4SA(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#
# Preset Parameters Mnemonics
#----------------------------
proc moteiCVME4SA_setMnemonics { } {
    global vme4saIdx vme4saPar
    
    set vme4saPar [list do]

    set i 0
    foreach el $vme4saPar { 
	set vme4saIdx($el) $i
	incr i
    }
}

#
# Read Motor Main & Servo Amplifier Configuration
#--------------------------------------------------
proc moteiCVME4SA_readConfig { {mode 0} } {
    global gvar 
    global vme4saCfg vme4saBkp vme4saPar vme4saIdx vme4saDev
    global motCfg

    #
    # Read motor main config
    #
    set p "$gvar(dbPath):SERVER:CONF"

    if {$mode != 0} return
    #
    # Read servo amplifier config
    #
    set p "$gvar(dbPath):SDL:VME4SA:CFG"
    if {[catch {seq_dbReadSymbolic "$p.configValues"} vme4saCfg]} {
	seq_errDisplay -suspend
	return
    }
    if {[catch {seq_dbReadSymbolic "$p.device"} vme4saDev]} {
	seq_errDisplay -suspend
	return
    }
    lassign $vme4saDev dummy gvar(Channel)
    set vme4saDev [lreplace $vme4saDev 0 0 $gvar(Driver)]
    moteiDeviceExtractChannel
    
    #
    # Make a backup
    #
    set vme4saBkp $vme4saCfg
    
}

#
# Write Servo Amplifier Configuration
#--------------------------------------------------
proc moteiCVME4SA_writeConfig { } {
    global gvar vme4saCfg vme4saDev

    set p "$gvar(dbPath):SDL:VME4SA:CFG"
    if {[catch {seq_dbWriteSymbolic "$p.configValues" $vme4saCfg}]} {
	seq_errDisplay -suspend
    }
    if {[catch {seq_dbWriteSymbolic "$p.device" $vme4saDev}]} {
	seq_errDisplay -suspend
    }
    moteiConfigReinstall
}

#
# Configure widgets
#------------------
proc moteiCVME4SA_configWidgets { } {
    global gvar cv 
    global vme4saCfg vme4saPar vme4saIdx
    global motCfg

    #
    # Assign widget values with servo amplifier config
    #
    set i 0
    foreach el $vme4saPar { 
	set gvar($el) [lindex $vme4saCfg $vme4saIdx($el)]
	incr i
    }

    #
    # Overload widget values with motor main config
    #

    # 
    # configure widgets
    #

    moteiCVME4SA_stateMachine    
}

proc moteiCVME4SAInitPanel { } {
    global gvar cv EDITING vme4saPar vme4saIdx vme4saBkp vme4saCfg
    
    moteiCVME4SA_setMnemonics

    if {$EDITING} return

    seq_ccsAsyncInput open 

    # install close handler
    vccStdMain shutDown

    moteiCVME4SA_readConfig
    moteiCVME4SA_configWidgets
}

#
# State Machine
#--------------
proc moteiCVME4SA_stateMachine { } {                              
}

#
# Action Button APPLY
#---------------------
proc moteiCVME4SAApply { } {
    global gvar cv
    global vme4saCfg vme4saBkp vme4saPar vme4saIdx vme4saDev
    global motCfg

    #
    # Assign widget values with motion controller config
    #
    set vme4saDev [list $gvar(Driver) $gvar(Channel)]
    foreach el $vme4saPar { 
	set idx $vme4saIdx($el)
	set val $gvar($el)
	if {$val == "N.A."} { set val 0 }

	set vme4saCfg [lreplace $vme4saCfg $idx $idx $val]
    }

    #
    # Write motion controller config
    #
    moteiCVME4SA_writeConfig
}

#
# Action Button RESET
#---------------------
proc moteiCVME4SAReset { } {
    global gvar
    global vme4saCfg vme4saBkp vme4saDev
    
    #
    # Restore backup configuration
    #
    set vme4saCfg $vme4saBkp
    lassign $vme4saDev dummy gvar(Channel)
    moteiDeviceExtractChannel

    #
    # Write motion controller config
    #
    moteiCVME4SA_writeConfig

    #
    # Refresh panel
    #
    moteiCVME4SA_configWidgets
}   

#
# Action Button REFRESH
#---------------------
proc moteiCVME4SARefresh { } {
    moteiCVME4SA_readConfig 1
    moteiCVME4SA_configWidgets
}
    
proc moteiCVME4SAExitPanel { } {    
}

#
# ___oOo___
