#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiConfig.tcl,v 2.43 2005/04/19 12:32:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  10/10/96  uses seq_dbGetAlias to get motor's alias (name).
# imunoz  07/05/96  added handling of communication with moteiControlServer
#                   process to automatically detach, deinstall, install 
#                   and attach while aplying a new configuration.
# pduhoux  16/05/95  created
#

#************************************************************************
#   NAME
#       moteiConfig - main configuration panel
#
#   SYNOPSIS
#       moteiConfig <args>
#
#   DESCRIPTION
#       The panel moteiConfig.pan contains a set of widgets allowing the basic
#       configuration of a motor. 
#       In addition, advanced configuration panels are accessible
#       via the MenuBar.
#       The file moteiConfig.tcl contains all associated procedures and common
#       procedures to all configuration sub-panels.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main panel.
#       
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7)
#       moteiConfig(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiCheckLinearAxis { what state } {
    global gvar cv

    set w1 $cv.${what}PosType
    set w2 $cv.${what}Value 
    set w3 $cv.${what}Unit 

    if {$state == 0 } {
	set gvar(${what}Flag) 0
	$w1 disable
	$w2 disable    
	$w3 disable    
    } else {
	$w1 enable
	$w2 enable
	$w3 enable
	if {$gvar(${what}PosType) == 0} { set gvar(${what}PosType) 2 }
    }
}

proc moteiSetAxisType {{axisType ""}} {
    global gvar cv

    if {$gvar(MC_Flag) == 0} {
	# force disable all
        moteiCheckLinearAxis lsw 0
        moteiCheckLinearAxis usw 0
        $cv.ckLSW disable
        $cv.ckUSW disable
        $cv.circularRange disable
	$cv.axisType disableEntry 2
	if {$gvar(axisType) == 1} {
	    $cv.moteiConfig-wdgTIMELIM enable
	} else {
	    if {$gvar(axisType) == 3} { set gvar(axisType) 2}
	    $cv.moteiConfig-wdgTIMELIM disable
	}
	return
    } else {
	$cv.axisType enableEntry 2
    }


    if {$gvar(axisType) == 1} {
        # Enable lsw/usw setting according to the current
        # axis configuration
        moteiCheckLinearAxis lsw $gvar(lswFlag)
        moteiCheckLinearAxis usw $gvar(uswFlag)
        $cv.ckLSW enable
        $cv.ckUSW enable

        # Disable circular range
        $cv.circularRange disable

        # Enable timeLim scale
        $cv.moteiConfig-wdgTIMELIM enable
    } else {
        # Disable lsw/usw setting and the associated checkButtons
        moteiCheckLinearAxis lsw 0
        moteiCheckLinearAxis usw 0
        $cv.ckLSW disable
        $cv.ckUSW disable

        # Enable circular range
        $cv.circularRange enable
	if {$gvar(circularRange) == 0} {
	    if {$gvar(MC_Flag) != 0} {
		set gvar(circularRange) 86400
	    } else {
		set gvar(circularRange) 1
	    }
	}

        # Disable timeLim scale
        $cv.moteiConfig-wdgTIMELIM disable
    }
}

proc moteiSetEncoderType {{encType ""}} {
    global gvar cv

    if {$gvar(MC_Flag) == 0} {
	#
	# disable Encoder widgets
	#
	set gvar(encoderType) 0
        $cv.encoderType disable
    } else {
	if {$gvar(motorType) == 2} {
	    $cv.encoderType enableEntry 0
	} else {
	    if {$gvar(encoderType) == 0} { set gvar(encoderType) 1 }
	    $cv.encoderType disableEntry 0
	}
        $cv.encoderType enable
    }

    if {$gvar(encoderType) == 0} {
        # Disable all widgets
        $cv.countRange disable
        $cv.es         disable
        $cv.sc         disable
        $cv.eo         disable
        $cv.validBits  disable
        $cv.code       disable
        $cv.interface  disable
        $cv.address    disable

        # Disable buttons to increment/decrement validBits
        set gvar(bitsFlag) 0

        return
    } 

    # Enable all basic widgets
    # Disable buttons to increment/decrement validBits
    set gvar(bitsFlag) 1
    $cv.countRange enable
    $cv.validBits  enable
    $cv.code       enable
    $cv.interface  enable
    $cv.eo         enable

    if {($gvar(encoderType) == 0 && $gvar(motorType) == 2) || $gvar(encoderType) == 1} {
	if {$gvar(interface) == 2} {
	    set gvar(countRange) 32768
	    $cv.countRange       disable
	    set gvar(code)       1
	    $cv.code             disable
	    set gvar(validBits)  15
	    $cv.validBits        disable
	    set gvar(bitsFlag)   0
	}
    }

    # Enable widgets specific to a given mode.
    if {$gvar(encoderType) >= 2} {
        $cv.sc enable
    } else {
        $cv.sc disable
    }
    if {$gvar(motorType) == 2} {
	$cv.es enable
    } else {
        $cv.es disable
    }

    # update Interface
    moteiSetInterface
}


proc moteiSetInterface {{type {}}} {
   global gvar cv

    if {$gvar(interface) == 2} {
	# Internal
	$cv.address disable
	if {$gvar(encoderType) == 1} {
	    panEnableMenuEntry Speeds "Index Search ..."
	    set gvar(bitsFlag) 0
	    $cv.sc         disable
	    $cv.validBits  disable
	    set gvar(countRange) 32768
	    $cv.countRange disable
	} else {
	    panDisableMenuEntry Speeds "Index Search ..."
	    set gvar(bitsFlag) 1
	    $cv.sc         enable
	    $cv.validBits  enable
	    $cv.countRange enable
	}
    } else {
	# 4 = CPU
	# 5 = External
	panDisableMenuEntry Speeds "Index Search ..."

# RFR: SPR VLTSW 20050144
# If a configuration file is loaded, it is not allowed
# to overwrite this configuration!
#
#	if {$gvar(encoderType) >= 2} { 
#	    set gvar(validBits) 24 
#	} else {
#	    set gvar(validBits) 15
#	}

	set gvar(bitsFlag) 1
	$cv.sc         enable
	$cv.eo         enable
	$cv.validBits  enable
	$cv.address    enable
	$cv.countRange enable
    }
}

proc moteiConfig_stateMachineSA { {flag ""} } {
    global gvar cv

    #
    # Update menu bar
    #
    if {$gvar(SA_Flag) == 1} {
	# enable menu entry: Hardware->Amplifier
	panEnableMenuEntry Hardware "Amplifier ..."
	set gvar(SA_Prefix) "vme4sa"
    } else {
	# disable menu entry: Hardware->Amplifier
	panDisableMenuEntry Hardware "Amplifier ..."
	set gvar(SA_Prefix) ""
    }
    if {$gvar(motorType) == 2} {
	$cv.moteiConfig-wdgSA disableEntry 0
	set gvar(SA_Flag)   2
	set gvar(SA_Prefix) ""
    } else {
	$cv.moteiConfig-wdgSA enableEntry 0
    }
}

proc moteiConfig_stateMachineMC { {flag ""} } {
    global gvar cv

    #
    # Update menu bar
    #
    if {$gvar(MC_Flag) == 0} {
	# disable menu entry: Initialization->Special Positions
	panDisableMenuEntry Initialization "Special Positions ..."
	# disable menu entry: Speeds->Index Search
	panDisableMenuEntry Speeds "Index Search ..."
	# disable menu entry: Positions
	[panGetMbPath Positions] configure -state disabled
	# disable menu entry: Hardware->Controller
	panDisableMenuEntry Hardware "Controller ..."
	set gvar(MC_Prefix) ""
	if {$gvar(circularRange) == 0 && $gvar(axisType) != 1} {
	    set gvar(circularRange) 1
	}
    } else {
	# enable menu entry: Initialization->Special Positions
	panEnableMenuEntry Initialization "Special Positions ..."
	# enable menu entry: Speeds->Index Search
	panEnableMenuEntry Speeds "Index Search ..."
	# enable menu entry: Positions
	[panGetMbPath Positions] configure -state normal
	
	# enable menu entry: Hardware->Controller
	panEnableMenuEntry Hardware "Controller ..."
	if {$gvar(MC_Flag) == 1} { 
	    set gvar(MC_Prefix) "mac4"
	} else {
	    set gvar(MC_Prefix) "sen4"
	}
	if {$gvar(circularRange) == 0 && $gvar(axisType) != 1} {
	    set gvar(circularRange) 86400
	}
    }
}

proc moteiConfig_stateMachineDIO { {flag ""} } {
    global gvar cv

    if {$gvar(DIO_Flag) == 1} {
	panEnableMenuEntry Hardware "Brake & Interlock ..."
    } else {
	panDisableMenuEntry Hardware "Brake & Interlock ..."
    }
}

#-----------------------------------
# panel state machine
#
proc moteiConfig_stateMachine { args } {
    global gvar cv

    #
    # Enable all widgets
    #
    if {!$gvar(context_sensitive)} {
	#
	# Menu
	#
	panEnableMenuEntry Initialization "Special Positions ..."
	panEnableMenuEntry Speeds "Index Search ..."
	[panGetMbPath Positions] configure -state normal
	panEnableMenuEntry Hardware "Amplifier ..."
	panEnableMenuEntry Hardware "Controller ..."
	panEnableMenuEntry Hardware "Brake & Interlock ..."
	#
	# Boards
	#
	$cv.moteiConfig-wdgSA enableEntry 0
	#
	# Axis
	#
        $cv.axisType enable
	$cv.axisType enableEntry 2
        moteiCheckLinearAxis lsw 1
        moteiCheckLinearAxis usw 1
        $cv.ckLSW enable
        $cv.ckUSW enable
        $cv.circularRange enable
	#
	# Encoder
	#
        $cv.encoderType enable
        $cv.countRange enable
        $cv.validBits enable
        $cv.interface enable
        $cv.code enable
        $cv.address enable
        $cv.sc enable
        $cv.eo enable
        $cv.es enable
        set gvar(bitsFlag) 1
	#
	# Times
	#
        $cv.moteiConfig-wdgTIMELIM enable

	return
    }

    moteiConfig_stateMachineDIO
    moteiConfig_stateMachineSA
    moteiConfig_stateMachineMC
    moteiSetAxisType
    moteiSetEncoderType
}

#-----------------------------------
# panel state machine
#
proc moteiConfigCfgWidgets { } {
    global gvar cv moteiConfigParams

    # SERVER:CONF.motor
    lassign $moteiConfigParams(motor) gvar(motorType) \
	    gvar(timeout) gvar(timelim) timemon gvar(opMode)
    set gvar(chkHwLimit) [lindex $moteiConfigParams(motor) 10]
    # convert times to seconds
    set gvar(timeout) [expr {$gvar(timeout) / 1000}]
    set gvar(timelim) [expr {$gvar(timelim) / 1000}]
    # for timeMon, special formula
    if {$timemon == 10} {
	set gvar(timemon) 0
    } elseif {$timemon < 1000} {
	set gvar(timemon) [expr {round(-1000.0 / double($timemon))}]
	if {$gvar(timemon) < -10} { set gvar(timemon) -10 }
    } else {
	set gvar(timemon) [expr {round(double($timemon) / 1000.0)}]
	if {$gvar(timemon) > 30} { set gvar(timemon) 30 }
    }
    # SERVER:CONF.boards
    set gvar(SA_Flag)  1
    set gvar(SA_Prefix) "vme4sa"
    set gvar(MC_Flag)  1
    set gvar(MC_Prefix) "mac4"
    set gvar(DIO_Flag) 0
    for {set i 0} {$i < 3} {incr i} {
	set b [lindex $moteiConfigParams(boards) $i]
	lassign $b typ flg pfx
	switch $typ {
	    1 { # Amplifier
		set gvar(SA_Flag)   1
		set gvar(SA_Prefix) $pfx
		if {[string length $pfx] == 0} {
		    set gvar(SA_Flag) 2
		} 
	    }
	    2 { # Controller
		set gvar(MC_Flag)   $flg
		set gvar(MC_Prefix) $pfx
	    }
	    3 { # Digital IO board
		set gvar(DIO_Flag)  $flg
	    }
	}
    }

    # SERVER:CONF.axis
    set gvar(axisType) $moteiConfigParams(axisType)

    # SERVER:CONF.encoder
    if {[llength $moteiConfigParams(encoder)] == 10} {
	lassign $moteiConfigParams(encoder) \
		gvar(encoderType) gvar(interface) gvar(code) \
		gvar(address) gvar(validBits) gvar(countRange) \
		gvar(circularRange) gvar(es) gvar(sc) gvar(eo)
    } else {
	lassign $moteiConfigParams(encoder) \
		gvar(encoderType) gvar(interface) gvar(code) \
		gvar(address) gvar(validBits) gvar(countRange) \
		gvar(circularRange) gvar(es) gvar(sc)
	set gvar(eo) 0
    }

    # convert encoder address to Hex representaion		
    set gvar(address) [format "0x%08X" $gvar(address)]

    # SERVER:UNIT.defaultUnits
    set posDefUnit $moteiConfigParams(posDefUnit) 
    $cv.lswUnit configure -itemList $moteiConfigParams(posUnit) \
	    -default $posDefUnit
    $cv.uswUnit configure -itemList $moteiConfigParams(posUnit) \
	    -default $posDefUnit

    # SERVER:CONF.specialPositions (LSW & USW)
    set lsw [lindex $moteiConfigParams(specialPositions) 0]
    set usw [lindex $moteiConfigParams(specialPositions) 1]
    lassign $lsw gvar(lswFlag) gvar(lswPosType) gvar(lswValue) gvar(lswUnit)
    lassign $usw gvar(uswFlag) gvar(uswPosType) gvar(uswValue) gvar(uswUnit)
    if {$gvar(lswPosType) == 0} { 
	set gvar(lswFlag) 0
	set gvar(lswPosType) 3
	set gvar(lswValue) 1
    }
    if {$gvar(uswPosType) == 0} {
	set gvar(uswFlag) 0
	set gvar(uswPosType) 4
	set gvar(uswValue) -1
    }

    # if Units are empty set them to Enc
    if {[lempty $gvar(lswUnit)]} { set gvar(lswUnit) "Enc" }
    if {[lempty $gvar(uswUnit)]} { set gvar(uswUnit) "Enc" }

    moteiAdjustTimeout 0

    moteiConfig_stateMachine
}

proc moteiAdjustTimeout { incr } {
    global gvar cv
    
    set timeout $gvar(timeout)

    if {$incr > 0 && $timeout < 300} {
	incr timeout  1
    } elseif {$incr < 0 && $timeout >= 0} {
	incr timeout -1
    }
    $cv.moteiConfig-wdgTIMEOUT setValue $timeout
    set gvar(timeout) $timeout
}

proc moteiConfigReadDbConfig { {mode 0} } {
    global gvar moteiConfigParams

    #
    # SERVER:CONF
    #
    set p "$gvar(dbPath):SERVER:CONF"
    set moteiConfigParams(motor) [seq_dbReadSymbolic "$p.motor"]
    set moteiConfigParams(boards) [seq_dbReadSymbolic "$p.boards"]
    
    set boards $moteiConfigParams(boards)
    set nb [llength $boards]
    for {set i 0} {$i < $nb} {incr i} {
	set b [lindex $boards $i]
	lassign $b bt ba bp
	if {$bt != 3 && $bp == ""} { set ba 0 }
	switch $bt {
	    1 { set what amplifier  }
	    2 { set what controller }
	    3 { set what digitalIO  }
	}
	set moteiConfigParams($what) $ba
    }

    set moteiConfigParams(events) [seq_dbReadSymbolic "$p.events"]
    set moteiConfigParams(signals) [seq_dbReadSymbolic "$p.signals"]
	    
    set signals $moteiConfigParams(signals) 
    set ns [llength $signals]
    set moteiConfigParams(brakeAction) 0
    set moteiConfigParams(brakeStatus) 0
    set moteiConfigParams(interlock) 0
    for {set i 0} {$i < $ns} {incr i} {
	set s [lindex $signals $i]
	lassign $s st sd sb sl
	if {$st != -1} {
	    if {$sd == "" || $sb < 0 || $sb > 63} { set sl 0 }
	    if {$sl > 0} { set sl 1 }
	    switch $st {
		0 { set what brakeAction }
		1 { set what brakeStatus }
		2 { set what interlock   }
	    }
	    set moteiConfigParams($what) $sl
	} 
    }

    set moteiConfigParams(axisType) [seq_dbReadSymbolic "$p.axis"]
    set moteiConfigParams(encoder) [seq_dbReadSymbolic "$p.encoder"]
    set moteiConfigParams(specialPositions) [seq_dbReadSymbolic "$p.specialPositions"] 
    set moteiConfigParams(offsets) [seq_dbReadSymbolic "$p.offsets"]
 
    set attr "namedPositions"
    set npAttrInfo [seq_dbGetAttrInfo "$p.$attr"]
    set npn [lindex $npAttrInfo 2]
    set moteiConfigParams($attr) {}
    for { set n 0 } { $n < $npn } { incr n } {
	lappend moteiConfigParams($attr) [seq_dbReadSymbolic "$p.${attr}($n)"] 
    }
    set moteiConfigParams(namedPosList) {}
    set moteiConfigParams(indexPosList) {}
    set idx 0
    foreach np $moteiConfigParams(namedPositions) {
	set name [lindex $np 0]
	if {! [cequal $name ""]} { 
	    lappend moteiConfigParams(namedPosList) $name
	    lappend moteiConfigParams(indexPosList) $idx
	    incr idx
	}
    }

    set attr "namedSpeeds"
    set nsAttrInfo [seq_dbGetAttrInfo "$p.$attr"]
    set nsn [lindex $nsAttrInfo 2]
    set moteiConfigParams($attr) {}
    for { set n 0 } { $n < $nsn } { incr n } {
	lappend moteiConfigParams($attr) [seq_dbReadSymbolic "$p.${attr}($n)"] 
    }
    set moteiConfigParams(namedSpdList) {}
    set moteiConfigParams(indexSpdList) {}

    set idx 0
    foreach ns $moteiConfigParams(namedSpeeds) {
	set name [lindex $ns 0]
	if {! [cequal $name ""]} { 
	    lappend moteiConfigParams(namedSpdList) $name
	    lappend moteiConfigParams(indexSpdList) $idx
	    incr idx
	}
    }

    set moteiConfigParams(twoStepOffset) [seq_dbReadSymbolic "$p.twoStepOffset"] 
    set moteiConfigParams(indexPulseSpeed) [seq_dbReadSymbolic "$p.indexPulseSpeed"] 
    set moteiConfigParams(defaultSpeed) [seq_dbReadSymbolic "$p.defaultSpeed"] 

    set moteiConfigParams(spdDefault)    0
    set moteiConfigParams(spdDefaultEnc) 0
    set moteiConfigParams(spdDefaultUnit) "Enc/ms"
    set idx $moteiConfigParams(defaultSpeed)
    if {$idx >= 0} {
	set defSpd  [lindex $moteiConfigParams(namedSpeeds) $idx]
	set name    [lindex $defSpd 0]
	set vel_usr [lindex $defSpd 1]
	set unit    [lindex $defSpd 2]
	set vel_enc [lindex $defSpd 6]
	if {[string length $name] > 0} {
	    set moteiConfigParams(spdDefault)     $vel_usr
	    set moteiConfigParams(spdDefaultEnc)  $vel_enc
	    set moteiConfigParams(spdDefaultUnit) $unit
	    set gvar(far)    [lindex $defSpd 3]
	    set gvar(near)   [lindex $defSpd 4]
	    set gvar(end)    [lindex $defSpd 5]
	}
    }


    #
    # SERVER:UNIT
    #
    set p "$gvar(dbPath):SERVER:UNIT"
    set moteiConfigParams(defaultUnits) {}
    if {[catch {seq_dbReadSymbolic $p.defaultUnits} lst]} { 
	seq_errDisplay -suspend
    } else {
	foreach item $lst {
	    if {! [cequal $item ""]} { 
		lappend moteiConfigParams(defaultUnits) $item
	    } else {
		lappend moteiConfigParams(defaultUnits) {}
	    }
	}
    }
    set posDefUnit [lindex $moteiConfigParams(defaultUnits) 0]
    set moteiConfigParams(posDefUnit) $posDefUnit 
    set moteiConfigParams(posUnit)    $posDefUnit 
    if {$posDefUnit != "Enc"} {
	lappend moteiConfigParams(posUnit) "Enc"
    }
    set spdDefUnit [lindex $moteiConfigParams(defaultUnits) 1]
    set moteiConfigParams(spdDefUnit) $spdDefUnit 
    set moteiConfigParams(spdUnit)    $spdDefUnit 
    if {$spdDefUnit != "Enc/ms" && $spdDefUnit != "Ref"} {
	lappend moteiConfigParams(spdUnit) "Enc/ms"
    }
    set curDefUnit [lindex $moteiConfigParams(defaultUnits) 2]
    set moteiConfigParams(curDefUnit) $curDefUnit
    set moteiConfigParams(curUnit)    $curDefUnit
    if {$curDefUnit != "Dig"} {
	lappend moteiConfigParams(curUnit) "Dig" 
    }

    set attr "unitConversion"
    set ucAttrInfo [seq_dbGetAttrInfo "$p.$attr"]
    set nuc [lindex $ucAttrInfo 2]
    set moteiConfigParams($attr) {}
    for { set n 0 } { $n < $nuc } { incr n } {
	lappend moteiConfigParams($attr) [seq_dbReadSymbolic "$p.${attr}($n)"] 
    }

    #
    # SERVER:INIT
    #
    set p "$gvar(dbPath):SERVER:INIT"
    set attr "actions"
    set iaAttrInfo [seq_dbGetAttrInfo "$p.$attr"]
    set nia [lindex $iaAttrInfo 2]
    set moteiConfigParams($attr) {}
    for { set n 0 } { $n < $nia } { incr n } {
	lappend moteiConfigParams($attr) [seq_dbReadSymbolic "$p.${attr}($n)"] 
    }

    if {$mode == 0} {
	moteiConfigMakeDbBackup
    }
}

proc moteiConfigMakeDbBackup { } {
    global moteiConfigParams moteiConfigBackup
    
    #
    # SERVER:CONF
    #
    set moteiConfigBackup(motor)    $moteiConfigParams(motor) 
    set moteiConfigBackup(boards)   $moteiConfigParams(boards) 
    set moteiConfigBackup(events)   $moteiConfigParams(events) 
    set moteiConfigBackup(signals)  $moteiConfigParams(signals) 
    set moteiConfigBackup(axisType) $moteiConfigParams(axisType) 
    set moteiConfigBackup(encoder)  $moteiConfigParams(encoder) 
    set moteiConfigBackup(specialPositions) \
	                            $moteiConfigParams(specialPositions) 
    set moteiConfigBackup(offsets)         $moteiConfigParams(offsets) 
    set moteiConfigBackup(namedPositions)  $moteiConfigParams(namedPositions) 
    set moteiConfigBackup(namedSpeeds)     $moteiConfigParams(namedSpeeds)
    set moteiConfigBackup(twoStepOffset)   $moteiConfigParams(twoStepOffset)
    set moteiConfigBackup(indexPulseSpeed) $moteiConfigParams(indexPulseSpeed)
    set moteiConfigBackup(defaultSpeed)    $moteiConfigParams(defaultSpeed) 

    #
    # SERVER:UNIT
    #
    set moteiConfigBackup(defaultUnits)    $moteiConfigParams(defaultUnits)
    set moteiConfigBackup(unitConversion)  $moteiConfigParams(unitConversion)

    #
    # SERVER:INIT
    #
    set moteiConfigBackup(actions)         $moteiConfigParams(actions)
}

proc moteiConfigInitPanel { } {                     
    # Declare global variables 
    global gvar

    # check if motor accessible
    if {![info exist gvar(controlMotor)]} {
	set gvar(controlMotor) 1
    }

    #
    # read DB configuration
    #
    moteiConfigReadDbConfig

    # configure widgets
    moteiConfigCfgWidgets
}


proc moteiConfigSave { } {                 
    global gvar

    moteiConfigApply

    moteiSaveDatabase "$gvar(dbPath)"
}

proc moteiConfigReinstall { } {
    global gvar

    if {$gvar(controlMotor) == 0 } return

    # creates a link to the controlServer
    set mControl "$gvar(lcuEnv):$gvar(motorName)"
    moteiControlSend [list createControl $gvar(lcuEnv) $gvar(motorName)]
    
    #-- detach if attached deinstall, install ad attach again
    set attached [seq_dbReadSymbolic $gvar(dbPath):STATUS.opMode]
    if {$attached != 1} { 
	set reply [moteiControlSend [list motorCommand $mControl detach]]
	if {$reply == "FAILURE"} { 
	    moteiControlSend [list detachControl $mControl]
	    return "FAILURE" 
	}
    }
    set reply [moteiControlSend [list initMotor $mControl]]
    if {$reply == "FAILURE"} { 
	moteiControlSend [list detachControl $mControl]
	return "FAILURE" 
    }
    if {$attached != 1} { 
	set reply [moteiControlSend [list motorCommand $mControl attach]]
	if {$reply == "FAILURE"} { 
	    moteiControlSend [list detachControl $mControl]
	    return "FAILURE" 
	}
    }
    
    # releases connection to the moteiServer
    moteiControlSend [list detachControl $mControl]
}

proc moteiConfigApply { } {                 
    # Declare global variables 
    global gvar moteiConfigParams

    #
    # SERVER:CONF
    #
    set p "$gvar(dbPath):SERVER:CONF"
    
    #
    # SERVER:CONF.motor
    #
    # convert times to ms
    set timeout [expr {$gvar(timeout) * 1000}]
    set timelim [expr {$gvar(timelim) * 1000}]
    if {$gvar(timemon) > 0} {
	set timemon [expr {$gvar(timemon) * 1000}]
    } elseif {$gvar(timemon) == 0} {
	set timemon 10
    } else {
	set timemon [expr {round(-1000.0 / double($gvar(timemon)))}]
    }
    set moteiConfigParams(motor) [lreplace $moteiConfigParams(motor) 0 4 \
	    $gvar(motorType) $timeout $timelim $timemon $gvar(opMode)]
    set chkHwLimit $gvar(chkHwLimit)
    set moteiConfigParams(motor) [lreplace $moteiConfigParams(motor) 10 10 \
	    $chkHwLimit]
    seq_dbWriteSymbolic "$p.motor" $moteiConfigParams(motor)

    #
    # SERVER:CONF.boards
    #
    set SA_Flag 1
    if {$gvar(SA_Flag) == 1} { set SA_Prefix "vme4sa" }
    if {$gvar(SA_Flag) == 2} { set SA_Prefix "" }
    set b1 [list 1 $SA_Flag $SA_Prefix]
    if {$gvar(MC_Flag) == 1} { set MC_Prefix "mac4" }
    if {$gvar(MC_Flag) == 2} { set MC_Prefix "sen4" }
    set b2 [list 2 $gvar(MC_Flag) $gvar(MC_Prefix)]
    set DIO_Prefix ""
    if {$gvar(DIO_Flag) == 1} { set DIO_Prefix "acro" } 
    set b3 [list 3 $gvar(DIO_Flag) $DIO_Prefix]

    set moteiConfigParams(boards) [list $b1 $b2 $b3]
    seq_dbWriteSymbolic "$p.boards" $moteiConfigParams(boards)

    #
    # SERVER:CONF.encoder
    #
    set gvar(ea) [format "%u" $gvar(address)]
    if {[llength $moteiConfigParams(encoder)] == 10} {
	set moteiConfigParams(encoder) [list $gvar(encoderType) \
		$gvar(interface) $gvar(code) $gvar(ea) $gvar(validBits) \
		$gvar(countRange) $gvar(circularRange) \
		$gvar(es) $gvar(sc) $gvar(eo)]
	if {$gvar(MC_Flag) == 1} {
	    set pmc "$gvar(dbPath):SDL:MAC4:CFG"
	    seq_dbWriteSymbolic "$pmc.configValues(20)" $gvar(eo)
	}
    } else {
	set moteiConfigParams(encoder) [list $gvar(encoderType) \
		$gvar(interface) $gvar(code) $gvar(ea) $gvar(validBits) \
		$gvar(countRange) $gvar(circularRange) \
		$gvar(es) $gvar(sc)]
    }
    seq_dbWriteSymbolic "$p.encoder" $moteiConfigParams(encoder)

    if {$gvar(MC_Flag) == 1} {
	set pmc "$gvar(dbPath):SDL:MAC4:CFG"
	seq_dbWriteSymbolic "$pmc.configValues(21)" $gvar(es)
    }

    #
    # SERVER:CONF.axis
    #
    set moteiConfigParams(axisType) $gvar(axisType)
    seq_dbWriteSymbolic "$p.axis"  $moteiConfigParams(axisType)

    #
    # SERVER:CONF.specialPositions(LSW & USW)
    #
    set lsw [list \
	    $gvar(lswFlag) $gvar(lswPosType) $gvar(lswValue) $gvar(lswUnit)]
    set usw [list \
	    $gvar(uswFlag) $gvar(uswPosType) $gvar(uswValue) $gvar(uswUnit)]
    set moteiConfigParams(specialPositions) \
	    [lreplace $moteiConfigParams(specialPositions) 0 1 $lsw $usw]
    seq_dbWriteSymbolic "$p.specialPositions(0:1)" [list $lsw $usw]
    
    #
    # If control selected,
    #   detach, deinstall & install, attach motor
    #
    moteiConfigReinstall
}

proc moteiConfigReset { } {
    global moteiConfigParams moteiConfigBackup
    
    #
    # SERVER:CONF
    #
    set moteiConfigParams(motor)    $moteiConfigBackup(motor) 
    set moteiConfigParams(boards)   $moteiConfigBackup(boards) 
    set moteiConfigParams(events)   $moteiConfigBackup(events) 
    set moteiConfigParams(signals)  $moteiConfigBackup(signals) 
    set moteiConfigParams(axisType) $moteiConfigBackup(axisType) 
    set moteiConfigParams(encoder)  $moteiConfigBackup(encoder) 
    set moteiConfigParams(specialPositions) \
	                            $moteiConfigBackup(specialPositions) 
    set moteiConfigParams(offsets)         $moteiConfigBackup(offsets) 
    set moteiConfigParams(namedPositions)  $moteiConfigBackup(namedPositions) 
    set moteiConfigParams(namedSpeeds)     $moteiConfigBackup(namedSpeeds)
    set moteiConfigParams(twoStepOffset)   $moteiConfigBackup(twoStepOffset)
    set moteiConfigParams(indexPulseSpeed) $moteiConfigBackup(indexPulseSpeed)
    set moteiConfigParams(defaultSpeed)    $moteiConfigBackup(defaultSpeed) 

    #
    # SERVER:UNIT
    #
    set moteiConfigParams(defaultUnits)    $moteiConfigBackup(defaultUnits)
    set moteiConfigParams(unitConversion)  $moteiConfigBackup(unitConversion)

    #
    # SERVER:INIT
    #
    set moteiConfigParams(actions)         $moteiConfigBackup(actions)
    
    moteiConfigCfgWidgets
    moteiConfigApply
}

proc moteiConfigRefresh { } {
    global moteiConfigParams moteiConfigBackup

    #
    # read DB configuration
    #
    moteiConfigReadDbConfig 1

    # configure widgets
    moteiConfigCfgWidgets
}

#
# Termination Procedure
#
proc moteiConfigExitPanel { } {
    global moteiConfigAppList
    moteiCloseExternalApp moteiConfigAppList 
}

#
#
# ___oOo___
