#************************************************************************
# e.S.O. - VLT project
#
# "@(#) $Id: moteiControl.tcl,v 2.43 2005/04/19 12:32:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  06/11/95  created
# imunoz  07/05/96  always deInstall and Install motor before attach it.
# imunoz  10/10/96  uses seq_dbGetAlias to get motor's name
# pduhoux 05/12/97  redesign
#

#************************************************************************
#   NAME
#       moteiControl - motor control panel
# 
#   SYNOPSIS
#       moteiControl <args>
# 
#   DESCRIPTION   
#       The panel moteiControl.pan contains a set of widgets allowing the 
#       full control of a motor, and a basic set of status widgets showing the 
#       main operational status of a motor.
#       In addition, advanced actions are accessible via the MenuBar.
#       The file moteiControl.tcl contains all associated procedures and common
#       procedures to all control sub-panels.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7), moteiServer(1)
#       moteiControl(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiControlReadDbConfig { } {
    global gvar moteiConfigParams moteiSetupParams moteiStatusParams 

    set c "$gvar(dbPath):SERVER:CONF"
    set u "$gvar(dbPath):SERVER:UNIT"

    set moteiConfigParams(timemon)         0
    if {[catch {seq_dbReadSymbolic "$c.motor"} motor]} {
	seq_errDisplay -suspend
	return
    } else {
	set timemon [lindex $motor 3]

	set moteiConfigParams(timemon) $timemon
	if {$timemon >= 1000} {
	    set gvar(samplingRate) [expr {$timemon / 1000}]
	} elseif {$timemon == 0} {
	    set gvar(samplingRate) -1
	} else {
	    set gvar(samplingRate) [expr {-1000 / $timemon}]
	}
    }

    set moteiConfigParams(signals)     {}
    set moteiConfigParams(brakeAction) 0
    set moteiConfigParams(brakeStatus) 0
    set moteiConfigParams(interlock)   0
    if {[catch {seq_dbReadSymbolic "$c.signals"} signals]} {
	seq_errDisplay -suspend
	return
    } else {
	set moteiConfigParams(brakeAction) 0
	set moteiConfigParams(brakeStatus) 0
	set moteiConfigParams(interlock)   0
	set moteiConfigParams(signals) $signals
	set ns [llength $signals]
	for {set i 0} {$i < $ns} {incr i} {
	    set s [lindex $signals $i]
	    lassign $s st sd sb sl
	    if {$st != -1} {
		if {$sd == "" || $sb < 0 || $sb > 63} { set sl 0 }
		if {$sl > 0} { set sl 1 }
		switch $st {
		    0 { set what brakeAction }
		    1 { set what brakeStatus }
		    2 { set what interlock   }
		}
		set moteiConfigParams($what) $sl
	    }
	}
    }

    set moteiConfigParams(boards)     {}
    set moteiConfigParams(amplifier)  0
    set moteiConfigParams(controller) 0
    set moteiConfigParams(digitalIO)  0
    if {[catch {seq_dbReadSymbolic "$c.boards"} boards]} {
	seq_errDisplay -suspend
	return
    } else {
	set moteiConfigParams(boards) $boards
	set nb [llength $boards]
	for {set i 0} {$i < $nb} {incr i} {
	    set b [lindex $boards $i]
	    lassign $b bt ba bp
	    switch $bt {
		1 { set what amplifier  }
		2 { set what controller }
		3 { set what digitalIO  }
	    }
	    set moteiConfigParams($what) $ba
	}
    }

    set moteiConfigParams(axisType) 1
    if {[catch {seq_dbReadSymbolic "$c.axis"} at]} {
	seq_errDisplay -suspend
	return
    } else {
	set moteiConfigParams(axisType) $at
    }

    set moteiConfigParams(encoderType) 1
    if {[catch {seq_dbReadSymbolic "$c.encoder(0,0)"} et]} {
	seq_errDisplay -suspend
	return
    } else {
	set moteiConfigParams(encoderType) $et
    }

    set moteiConfigParams(namedPosList) {}
    set moteiConfigParams(indexPosList) {}
    if {[catch {seq_dbReadSymbolic "$c.namedPositions(0:$,0)"} lst]} {
	seq_errDisplay -suspend
    } else {
	set idx 0
	foreach item $lst {
	    if {! [cequal $item ""]} { 
		lappend moteiConfigParams(namedPosList) $item
		lappend moteiConfigParams(indexPosList) $idx
		incr idx
	    }
	}
    }
    
    set moteiConfigParams(namedSpdList) {}
    set moteiConfigParams(indexSpdList) {}
    if {[catch {seq_dbReadSymbolic "$c.namedSpeeds(0:$,0)"} lst]} {
	seq_errDisplay -suspend
    } else {
	set idx 0
	foreach item $lst {
	    if {! [cequal $item ""]} { 
		lappend moteiConfigParams(namedSpdList) $item
		lappend moteiConfigParams(indexSpdList) $idx
		incr idx
	    }
	}
    }

    set moteiConfigParams(defaultSpeed) -1
    set moteiConfigParams(spdDefault)    0
    set moteiConfigParams(spdDefaultEnc) 0

    if {[catch {seq_dbReadSymbolic "$c.defaultSpeed"} idx]} {
	seq_errDisplay -suspend
    } elseif {$idx >= 0} {
	if {[catch {seq_dbReadSymbolic "$c.namedSpeeds($idx,0:$)"} defSpd]} {
	    seq_errDisplay -suspend
	} else {
	    set name    [lindex $defSpd 0]
	    set vel_usr [lindex $defSpd 1]
	    set unit    [lindex $defSpd 2]
	    set vel_enc [lindex $defSpd 6]
	}
	#    puts "moteiConfigParams(defSpdVal) = $defSpd"
	if {([string length $name] > 0) && ($vel_enc != 0.0) } {
	    set moteiConfigParams(defaultSpeed)  $idx
	    set moteiConfigParams(spdDefault)    $vel_usr
	    set moteiConfigParams(spdDefaultEnc) $vel_enc
	    set moteiConfigParams(spdDefaultUnit) $unit
	    set gvar(far)    [lindex $defSpd 3]
	    set gvar(near)   [lindex $defSpd 4]
	    set gvar(stable) [lindex $defSpd 5]
	}
    }

    set moteiConfigParams(defaultUnits) {}
    if {[catch {seq_dbReadSymbolic "$u.defaultUnits"} lst]} { 
	seq_errDisplay -suspend
    } else {
	foreach item $lst {
	    if {! [cequal $item ""]} { 
		lappend moteiConfigParams(defaultUnits) $item
	    } else {
		lappend moteiConfigParams(defaultUnits) {}
	    }
	}
    }

    set posDefUnit [lindex $moteiConfigParams(defaultUnits) 0]
    set moteiConfigParams(posDefUnit) $posDefUnit 
    set moteiConfigParams(posUnit)    $posDefUnit 
    set moteiSetupParams(posUnit)     $posDefUnit 
    if {$posDefUnit != "Enc"} {
	lappend moteiConfigParams(posUnit) "Enc"
    }

    set spdDefUnit [lindex $moteiConfigParams(defaultUnits) 1]
    set moteiConfigParams(spdDefUnit) $spdDefUnit
    set moteiConfigParams(spdUnit)    $spdDefUnit
    set moteiSetupParams(spdUnit)     $spdDefUnit
    if {$spdDefUnit != "Enc/ms" && $spdDefUnit != "Ref"} {
	lappend moteiConfigParams(spdUnit) "Enc/ms" 
    }

    set curDefUnit [lindex $moteiConfigParams(defaultUnits) 2]
    set moteiConfigParams(curDefUnit) $curDefUnit
    set moteiConfigParams(curUnit)    $curDefUnit
    set moteiSetupParams(curUnit)     $curDefUnit
    if {$curDefUnit != "Dig"} {
	lappend moteiConfigParams(curUnit) "Dig" 
    }

    set na "   N/A    "
    set moteiStatusParams(posMini)    $na
    set moteiStatusParams(posMiniEnc) $na
    set moteiStatusParams(posMaxi)    $na
    set moteiStatusParams(posMaxiEnc) $na
    set moteiStatusParams(spdMini)    $na
    set moteiStatusParams(spdMiniEnc) $na
    set moteiStatusParams(spdMaxi)    $na
    set moteiStatusParams(spdMaxiEnc) $na

    set moteiStatusParams(lowerLimit)      $na
    set moteiStatusParams(upperLimit)      $na
    set moteiStatusParams(lowerRange)      $na
    set moteiStatusParams(upperRange)      $na
    set moteiStatusParams(referenceSwitch) $na
    set moteiStatusParams(indexPulse)      $na
    set moteiStatusParams(homePosition)    $na

    set i "$gvar(dbPath):SERVER:INIT"
    if {[catch {seq_dbReadSymbolic "$i.actions(0:$,actionType)"} ial]} { 
	seq_errDisplay -suspend
	exit 1
    }
    set moteiConfigParams(IAXF) 0
    foreach ia $ial {
	if {$ia == 8} { incr moteiConfigParams(IAXF) }
    }
    set u "$gvar(dbPath):SERVER:UNIT"
    if {[catch {seq_dbReadSymbolic "$u.unitConversion(0:$,method)"} ucl]} { 
	seq_errDisplay -suspend
	exit 1
    }
    set moteiConfigParams(UCXF) 0
    foreach uc $ucl {
	if {$uc == 1} { incr moteiConfigParams(UCXF) }
    }
}

proc moteiSetPositionGui {guiMode} {
    global cv gvar positionFrame offsetFrame
    global moteiConfigParams moteiSetupParams moteiStatusParams

    # Tell which frame to display
    switch $guiMode {
	2     {set positionFrame 0 }
	3 - 5 {set positionFrame 1 }
	4 - 6 {set positionFrame 2 }
    }
    # Tell second frame to display or hide offset sub-panel
    if {$guiMode == 5 || $guiMode == 6} {
	$cv.posOffset configure -show 1
	# Configure list of units
	set offsetFrame 0
	set w [$cv.posOffset getClassPath]
	$w.cv.valueUnit configure -itemList $moteiConfigParams(posUnit) -default $moteiSetupParams(posUnit)
    } elseif {$gvar(context_sensitive)} {
	$cv.posOffset configure -show 0
    } else {
	$cv.posOffset configure -show 1
    }
    $cv.posOffset show

    # Configure option buttons with corresponding lists
    set w [$cv.posFrame getClassPath]
    switch $positionFrame {
	0 {
	    # Configure list of units
	    $w.cv.valueUnit configure -itemList $moteiConfigParams(posUnit)  -default $moteiSetupParams(posUnit)
	    moteiConvertPositionRange
	}
	1 {
	    # Configure list of names
	    $w.cv.nameList configure -itemList $moteiConfigParams(namedPosList)  -default [lindex $moteiConfigParams(namedPosList) 0]
	}
	2 {
	    # Configure list of index
	    $w.cv.indexList configure -itemList $moteiConfigParams(indexPosList)  -default [lindex $moteiConfigParams(indexPosList) 0]
	}
    }
    $cv.posFrame show
}


proc moteiSetVelocityGui {guiMode} {
    global cv gvar speedFrame moteiSetupParams moteiConfigParams

    set gvar(speedHow) $guiMode
    set speedFrame [expr {$guiMode - 1}]
    set w [$cv.speedFrame getClassPath]

    # Configure option buttons with corresponding lists
    switch $speedFrame {
	0 {
	    # Configure default speed
	    set gvar(speedClass_defaultSpdIndex) $moteiConfigParams(defaultSpeed)
	    set gvar(speedClass_defaultSpdName) [lindex $moteiConfigParams(namedSpdList) $moteiConfigParams(defaultSpeed)]
	    if {![cequal $moteiSetupParams(spdUnit) "Enc/ms"]} {
		set DSV $moteiConfigParams(spdDefault)
		set DSU $moteiConfigParams(spdDefaultUnit)
	    } else {
		set DSV $moteiConfigParams(spdDefaultEnc)
		set DSU "Enc/ms"
	    }
	    set gvar(speedClass_defaultSpdValue) "$DSV $DSU"
	}
	1 {
	    if { [lsearch $moteiConfigParams(spdUnit) $moteiSetupParams(spdUnit)] == -1 } {
		set moteiSetupParams(spdUnit) [lindex $moteiConfigParams(spdUnit) 0]
	    }
	    # Configure list of units
	    $w.cv.valueUnit configure -itemList $moteiConfigParams(spdUnit)  -default $moteiSetupParams(spdUnit)
	    moteiConvertVelocityRange
	}
	2 {
	    # Configure list of names
	    $w.cv.nameList configure -itemList $moteiConfigParams(namedSpdList)  -default [lindex $moteiConfigParams(namedSpdList) 0]
	}
	3 {
	    # Configure list of index
	    $w.cv.indexList configure -itemList $moteiConfigParams(indexSpdList)  -default [lindex $moteiConfigParams(indexSpdList) 0]
	}
    }
    $cv.speedFrame show
}

#--------------------------------------------------
# update velocity range
#
proc moteiUpdateVelocityRange { } {
    global gvar cv mControl moteiConfigParams moteiStatusParams

    set na "   N/A    "
    set spdMini $na
    set spdMaxi $na
    set moteiStatusParams(spdMiniEnc) $spdMini
    set moteiStatusParams(spdMaxiEnc) $spdMaxi

    set unit $moteiConfigParams(spdDefUnit)
    if {![cequal $unit "Enc/ms"] && [info exist mControl]} {
	if {![cequal $spdMini $na]} {
	    set command [list motorCommand $mControl Enc2Vel $spdMini $unit]
	    set reply [moteiControlSend $command]
	    if {$reply != "FAILURE"} {
		set spdMini [format "%10.3f" [lindex $reply 2]]
	    }
	}
	
	if {![cequal $spdMaxi $na]} {
	    set command [list motorCommand $mControl Enc2Vel $spdMaxi $unit]
	    set reply [moteiControlSend $command]
	    if {$reply != "FAILURE"} {
		set spdMaxi [format "%10.3f" [lindex $reply 2]]
	    }
	}
    }
    set moteiStatusParams(spdMini) $spdMini
    set moteiStatusParams(spdMaxi) $spdMaxi

    moteiConvertVelocityRange
}

#--------------------------------------------------
# update velocity widgets
#
proc moteiUpdateVelocityWidgets { } {
    global gvar cv mControl 
    global speedFrame 
    global moteiConfigParams moteiSetupParams moteiStatusParams

    if {$speedFrame != 1} return

    if {[cequal $moteiSetupParams(spdUnit) "Enc/ms"]} {
	set spdDflt $moteiConfigParams(spdDefaultEnc)
    } else {
	set spdDflt $moteiConfigParams(spdDefault)
    }

    if {![info exist gvar(speedClass_value)]} {
	set gvar(speedClass_value) $spdDflt
    } elseif {[cequal $gvar(speedClass_value) ""]} {
	set gvar(speedClass_value) $spdDflt
    }
}

#--------------------------------------------------
# update special positions
#
proc moteiUpdateSpecialPositions { } {
    global gvar moteiConfigParams moteiStatusParams

    set na "   N/A    "
    set moteiStatusParams(lowerLimit)      $na
    set moteiStatusParams(upperLimit)      $na
    set moteiStatusParams(lowerRange)      $na
    set moteiStatusParams(upperRange)      $na
    set moteiStatusParams(referenceSwitch) $na
    set moteiStatusParams(indexPulse)      $na
    set moteiStatusParams(homePosition)    $na

    set attr "$gvar(dbPath):SERVER:INTERN.specialPositions(0:$,0:$)"
    if {[catch {seq_dbReadSymbolic $attr} spi] != 0} {
	seq_errDisplay -suspend
	return
    } 

    if {$moteiConfigParams(axisType) == 1} {
	set lsw [lindex $spi 0]
	set lhw [lindex $spi 2]
	if {[lindex $lsw 0] == 1} {
	    set moteiStatusParams(lowerLimit) [lindex $lsw 1]
	} elseif {[lindex $lhw 0] == 1 } {
	    set moteiStatusParams(lowerLimit) [lindex $lhw 1]
	}
	set usw [lindex $spi 1]
	set uhw [lindex $spi 3]
	if {[lindex $usw 0] == 1} {
	    set moteiStatusParams(upperLimit) [lindex $usw 1]
	} elseif {[lindex $uhw 0] == 1 } {
	    set moteiStatusParams(upperLimit) [lindex $uhw 1]
	}
    }

    set ref [lindex $spi 4]
    if {[lindex $ref 0] == 1} {
	set moteiStatusParams(referenceSwitch) [lindex $ref 1]
    }
    set ind [lindex $spi 5]
    if {[lindex $ind 0] == 1} {
	set moteiStatusParams(indexPulse) [lindex $ind 1]
    }
    set hom [lindex $spi 6]
    if {[lindex $hom 0] == 1} {
	set moteiStatusParams(homePosition) [lindex $hom 1]
    }
}

#--------------------------------------------------
# update position range
#
proc moteiUpdatePositionRange { } {
    global gvar cv mControl moteiConfigParams moteiStatusParams

    moteiUpdateSpecialPositions

    set attr "$gvar(dbPath):STATUS.posEnc"
    if {[catch {seq_dbReadSymbolic $attr} posEnc]} {
	seq_errDisplay -suspend
	return
    } 
    set na "   N/A    "
    if {$moteiConfigParams(axisType) == 1} {
	if {![cequal $moteiStatusParams(lowerLimit) $na] } {
	    set moteiStatusParams(lowerRange) \
		    [expr {$moteiStatusParams(lowerLimit) - $posEnc}]
	} else {
	    set moteiStatusParams(lowerRange) $na
	}
	if {![cequal $moteiStatusParams(upperLimit) $na] } {
	    set moteiStatusParams(upperRange) \
		    [expr {$moteiStatusParams(upperLimit) - $posEnc}]
	} else {
	    set moteiStatusParams(upperRange) $na
	}
    }

    set posMini $na
    set posMaxi $na
    if {$moteiConfigParams(axisType) == 1} {
	switch $gvar(posType) {
	    1 { # ABS
		set posMini $moteiStatusParams(lowerLimit)
		set posMaxi $moteiStatusParams(upperLimit)
	    } 
	    2 { # REL
		set posMini $moteiStatusParams(lowerRange)
		set posMaxi $moteiStatusParams(upperRange)
	    } 
	    3 { # LHW
		set posMini 0
		if {![cequal $moteiStatusParams(lowerLimit) $na] && \
		    ![cequal $moteiStatusParams(upperLimit) $na] } {
		    set posMaxi [expr {$moteiStatusParams(upperLimit) - \
			               $moteiStatusParams(lowerLimit)}]
		}
	    }
	    4 { # UHW
		if {![cequal $moteiStatusParams(lowerLimit) $na] && \
		    ![cequal $moteiStatusParams(upperLimit) $na]} {
		    set posMini [expr {$moteiStatusParams(lowerLimit) - \
			               $moteiStatusParams(upperLimit)}]
		}
		set posMaxi 0
	    }
	    5 { # REF
		if {![cequal $moteiStatusParams(referenceSwitch) $na]} {
		    if {![cequal $moteiStatusParams(lowerLimit) $na] } {
			set posMini [expr {$moteiStatusParams(lowerLimit) - \
				           $moteiStatusParams(referenceSwitch)}]
		    }
		    if {![cequal $moteiStatusParams(upperLimit) $na] } {
			set posMaxi [expr {$moteiStatusParams(upperLimit) -  \
				           $moteiStatusParams(referenceSwitch)}]
		    }
		}
	    }
	    6 { # IND
		if {![cequal $moteiStatusParams(indexPulse) $na]} {
		    if {![cequal $moteiStatusParams(lowerLimit) $na] } {
			set posMini [expr {$moteiStatusParams(lowerLimit) - \
				           $moteiStatusParams(indexPulse)}]
		    }
		    if {![cequal $moteiStatusParams(upperLimit) $na] } {
			set posMaxi [expr {$moteiStatusParams(upperLimit) - \
				           $moteiStatusParams(indexPulse)}]
		    }
		}
	    }
	    7 { # HOM
		if {![cequal $moteiStatusParams(homePosition) $na]} {
		    if {![cequal $moteiStatusParams(lowerLimit) $na] } {
			set posMini [expr {$moteiStatusParams(lowerLimit) - \
				           $moteiStatusParams(homePosition)}]
		    }
		    if {![cequal $moteiStatusParams(upperLimit) $na] } {
			set posMaxi [expr {$moteiStatusParams(upperLimit) - \
				           $moteiStatusParams(homePosition)}]
		    }
		}
	    }
	}
    }
    set moteiStatusParams(posMiniEnc) $posMini
    set moteiStatusParams(posMaxiEnc) $posMaxi

    set unit $moteiConfigParams(posDefUnit)
    if { $gvar(initStatus) < 2 } {
	set posMini $na
	set posMaxi $na
    } elseif {![cequal $unit "Enc"] && [info exist mControl]} {
	if {![cequal $posMini $na]} {
	    set command [list motorCommand $mControl Enc2Pos $posMini $unit]
	    set reply [moteiControlSend $command]
	    if {$reply != "FAILURE"} {
		if {[llength $reply] >= 2} {
		    set posMini [format "%10.3f" [lindex $reply 2]] 
		} else {
		    set posMini $moteiStatusParams(posMini)
		}
	    } else {
		set posMini $na
	    }
	}
	
	if {![cequal $posMaxi $na]} {
	    set command [list motorCommand $mControl Enc2Pos $posMaxi $unit]
	    set reply [moteiControlSend $command]
	    if {$reply != "FAILURE"} {
		if {[llength $reply] >= 2} {
		    set posMaxi [format "%10.3f" [lindex $reply 2]] 
		} else {
		    set posMaxi $moteiStatusParams(posMaxi)
		}
	    } else {
		set posMaxi $na
	    }
	}
    }
    set moteiStatusParams(posMini) $posMini
    set moteiStatusParams(posMaxi) $posMaxi

    moteiConvertPositionRange
}

#--------------------------------------------------
# update position widgets (posType)
#
proc moteiUpdatePositionTypeWidgets { posType } {
    global cv gvar positionFrame moteiConfigParams

    # for motionMode = Direct or Speed : do nothing
    if {$gvar(motionMode) == 2 || $gvar(motionMode) == 5} return

    if {$posType != 1 && $posType != 2 && $gvar(context_sensitive)} {
	$cv.twoStep disable
	$cv.halfTurn disable
    } else {
	$cv.twoStep  enable
	if {$moteiConfigParams(axisType) != 1} {
	    $cv.halfTurn enable
	}
    }

    $cv.posFrame configure -show 1
    if {$posType != 1 && $gvar(context_sensitive)} {
	# RELATIVE position -> Only by Value
	set positionFrame 0
	set gvar(posHow) 2
	set gvar(positionMode) 2
	moteiSetPositionGui 2
	# Enable/disable entries in Position Type
	set w $cv.positionMode
	for {set i 1} {$i < 5} { incr i } {
	    $w disableEntry $i
	}
	$cv.posOffset configure -show 0
	$cv.posOffset show
    } else {
	set w $cv.positionMode
	for {set i 0} {$i < 5} { incr i } {
	    $w enableEntry $i
	}
    }
    $cv.posFrame show
}
	
#--------------------------------------------------
# update position widgets
#
proc moteiUpdatePositionModeWidgets { motionMode } {
    global cv gvar moteiConfigParams moteiSetupParams moteiStatusParams

    $cv.halfTurn disable
    $cv.twoStep  disable
    set na "   N/A    "
    set enableList {}
    if {$moteiConfigParams(axisType) == 1} {
	switch $motionMode {
	    2 { # DIRECT
		lappend enableList 3 4 8 
	    }
	    3 { # POSITION
		$cv.twoStep  enable
		lappend enableList 1 2
		if {![cequal $moteiStatusParams(lowerLimit) $na]} {
		    lappend enableList 3
		}
		if {![cequal $moteiStatusParams(upperLimit) $na]} {
		    lappend enableList 4
		}
		if {![cequal $moteiStatusParams(referenceSwitch) $na]} {
		    lappend enableList 5
		}
		if {![cequal $moteiStatusParams(indexPulse) $na]} {
		    lappend enableList 6
		}
		if {![cequal $moteiStatusParams(homePosition) $na]} {
		    lappend enableList 7
		}
	    }
	    4 { # TRACKING
		$cv.twoStep  enable
		lappend enableList 1 2 8 
	    }
	    5 { # SPEED
		lappend enableList 3 4 5 6 7 8 
	    }
	}
    } else {
	if {$moteiConfigParams(encoderType) != 0} {
	    switch $motionMode {
		2 { lappend enableList 8 }
		3 { 
		    $cv.twoStep  enable
		    $cv.halfTurn enable
		    lappend enableList 1 2 5 6 7 
		}
		4 { 
		    $cv.twoStep  enable
		    $cv.halfTurn enable
		    lappend enableList 1 2 8 
		}
		5 { lappend enableList 5 6 7 8 }
	    }
	} else {
	    switch $motionMode {
		2 { lappend enableList 8 }
		3 { lappend enableList 1 2 5 }
		4 { lappend enableList 1 2 8 }
		5 { lappend enableList 5 8 }
	    }
	}
    }

    # Enable/disable entries in Position Type
    set w $cv.posType
    for {set i 0} {$i < 8} { incr i } {
	if {[lsearch $enableList [expr {$i + 1}]] == -1 } {
	    $w disableEntry $i
	} else {
	    $w enableEntry $i
	}
    }

    # Check current value and force a valid one
    if {[lsearch $enableList $gvar(posType)] == -1 } {
	switch $motionMode {
	    2 - 5   { set gvar(posType) 8}
	    default { set gvar(posType) 1}
	}
    }

    # Enable/Disable Position input
    if {$motionMode == 3 || $motionMode == 4} {
	# Position/Tracking
	$cv.posFrame configure -show 1
	$cv.positionMode enable

	# preset Position/Tracking to actual
	set p "$gvar(dbPath):STATUS"
	if {[cequal $moteiSetupParams(posUnit) "Enc"]} {
	    set pos [seq_dbReadSymbolic "$p.posEnc"]
	} else {
	    set pos [format "%10.3f" [seq_dbReadSymbolic "$p.posUser"]]
	}

	if {![info exist gvar(posClass_value)]} {
	    set gvar(posClass_value) $pos
	} elseif {[cequal $gvar(posClass_value) ""]} {
	    set gvar(posClass_value) $pos
	}

    } else {
	# Direct/Speed
	$cv.posFrame configure -show 0
	$cv.positionMode disable
    }

    if {!$gvar(context_sensitive)} {
	$cv.posFrame configure -show 1
	$cv.positionMode enable
	$cv.twoStep      enable
	$cv.halfTurn     enable
	set w $cv.posType
	for {set i 0} {$i < 8} { incr i } {
	    $w enableEntry $i
	}
    }
    $cv.posFrame show
}

proc moteiControlUpdOpMode { val } {
    global gvar ; set gvar(opMode) [lindex $val 2]
}

proc moteiControlUpdMode { {var ""} {val 0}} {
    global gvar
    #
    # store changed value
    #
    if {$var == ""} return
    set gvar($var) [lindex $val 2]
}

proc moteiControlUpdWidgets { {var ""} {val 0}} {
    global gvar cv bcv 
    global mControl moteiConfigParams
    global positionFrame speedFrame
    global moteiControlLost

    #
    # store changed value
    #
    if {$var != ""} {
	set gvar($var) [lindex $val 2]
    }

    #
    # update MenuBar, INIT & MOVE Buttons 
    #
    set show_pos 0
    set show_spd 0
    set show_off 0
    set mm   disable
    set pt   disable
    set pm   disable
    set sm   disable
    set ie   disable
    set me   disable
    set mb   normal
    set ht   disable
    set ts   disable
    switch $gvar(initStatus) {
	1 {
	    #
	    # enable  INIT button 
	    # disable MOVE button while not InitDone
	    #
	    set ie enable
	    set me disable
	}
	2 {
	    #
	    # enable  INIT button 
	    # disable MOVE button while not InitDone
	    #
	    set ie enable
	    set me disable
	}
	3 {
	    #
	    # disable INIT button while InitRun
	    # disable MOVE button while not InitDone
	    #
	    set mb disabled
	    set ie disable
	    set me disable
	} 
	4 {
	    #
	    # enable  INIT / MOVE buttons while InitDone
	    # disable INIT & MOVE buttons while Moving
	    #
	    set en enable
            if { $gvar(motionStatus) == 2 } { # MOVING
		set mb disabled
		set en disable 
	    } elseif {$gvar(motionStatus) == 1} { # STANDING
		moteiUpdatePositionRange
		moteiUpdatePositionModeWidgets $gvar(motionMode)
		moteiUpdatePositionTypeWidgets $gvar(posType)
		moteiUpdateVelocityWidgets
	    }
	    set ie $en
	    set me $en
	    set show_spd 1
	    set mm enable
	    set sm enable
	    set pt enable
	    if {$gvar(motionMode) == 3 || $gvar(motionMode) == 4} {
		set pm enable
		set show_pos 1
		if {($gvar(positionMode) == 5) || ($gvar(positionMode) == 6)} {
		    set show_off 1
		} 
		set ts enable
		if {$moteiConfigParams(axisType) != 1} { set ht enable }
	    }
	}
    }
    if {![info exist mControl] || $moteiControlLost} { set mb disabled }

    if { $gvar(initStatus) == 4 } {
	panEnableMenuEntry Hardware "Sampling ..."
    } else {
	panDisableMenuEntry Hardware "Sampling ..."
    }

    if { $gvar(initStatus) == 4 && $gvar(motionStatus) == 1 } {
	#
	# according to configuration enable/disable speed/motion how entries
	#
	if {$gvar(context_sensitive)} {
	    if {$moteiConfigParams(defaultSpeed) == -1} {
		$cv.speedMode disableEntry 0
		if {$speedFrame == 0} {
		    set speedFrame 1
		    set gvar(speedMode) 1
		    moteiSetVelocityGui 1
		}
	    } 
	    if {[llength $moteiConfigParams(namedSpdList)] == 0} {
		$cv.speedMode disableEntry 2
		$cv.speedMode disableEntry 3
		if {$speedFrame == 2 || $speedFrame == 3} {
		    set speedFrame 1
		    set gvar(speedMode) 1
		    moteiSetVelocityGui 1
		}
	    }
	    if {[llength $moteiConfigParams(namedPosList)] == 0} {
		$cv.positionMode disableEntry 1
		$cv.positionMode disableEntry 2
		$cv.positionMode disableEntry 3
		$cv.positionMode disableEntry 4
		set positionFrame 0
		set gvar(positionMode) 2
		moteiSetPositionGui 2
		set show_off 0
	    }
	} else {
	    $cv.speedMode    enableEntry 0
	    $cv.speedMode    enableEntry 2
	    $cv.speedMode    enableEntry 3
	    $cv.positionMode enableEntry 1
	    $cv.positionMode enableEntry 2
	    $cv.positionMode enableEntry 3
	    $cv.positionMode enableEntry 4
	}
    }

    #
    # force enable/show on context free
    #
    if {!$gvar(context_sensitive)} {
	set show_pos 1
	set show_spd 1
	set show_off 1
	set mm enable
	set pt enable
	set pm enable
	set sm enable
	set ie enable
	set me enable
	set mb normal
	set ht enable
	set ts enable
	panEnableMenuEntry Control "Power Control"
	panEnableMenuEntry Control "Control Loop"
	panEnableMenuEntry Control "Brake Control"
	panEnableMenuEntry Hardware "Sampling ..."
	panEnableMenuEntry Hardware Amplifier
	panEnableMenuEntry Hardware Controller
	panEnableMenuEntry Address "HW Initialization ..."
	panEnableMenuEntry Address "Unit Conversion ..."
    }

    [panGetMbPath OpMode] configure -state $mb
    [panGetMbPath Control] configure -state $mb
    [panGetMbPath Initialization] configure -state $mb
    [panGetMbPath Hardware] configure -state $mb
    [panGetMbPath Address] configure -state $mb

    $bcv.moteiControl-INIT $ie
    $bcv.moteiControl-MOVE $me

    $cv.motionMode   $mm
    $cv.posType      $pt
    $cv.positionMode $pm
    $cv.speedMode    $sm
    $cv.twoStep      $ts
    $cv.halfTurn     $ht

    $cv.posFrame   configure -show $show_pos
    $cv.speedFrame configure -show $show_spd
    $cv.posOffset  configure -show $show_off
    
    $cv.posFrame    show
    $cv.posOffset   show
    $cv.speedFrame  show

    update idletasks
}

proc moteiControlCfgWidgets { } {
    global gvar cv positionFrame offsetFrame speedFrame
    global moteiConfigParams moteiSetupParams moteiControlLost

    #------------------------------------------------
    # configure colors of emergency stop button
    $cv.emergencyStop configure -foreground Red -activeforeground Red 

    #------------------------------------------------
    # configure position widgets
    if {!$moteiControlLost } moteiUpdatePositionRange
    set positionFrame 0
    set w [$cv.posFrame getClassPath]
    $w.cv.valueUnit configure -itemList $moteiConfigParams(posUnit) \
	                      -default  $moteiSetupParams(posUnit)
    $cv.posFrame show

    set offsetFrame 0 
    set w [$cv.posOffset getClassPath]
    $w.cv.valueUnit configure -itemList $moteiConfigParams(posUnit) \
	                      -default  $moteiSetupParams(posUnit)
    $cv.posOffset show

    #------------------------------------------------
    # configure velocity widgets
    if {!$moteiControlLost } moteiUpdateVelocityRange
    set speedFrame 1
    set w [$cv.speedFrame getClassPath]
    if { [lsearch $moteiConfigParams(spdUnit) $moteiSetupParams(spdUnit)] == -1 } {
	set moteiSetupParams(spdUnit) [lindex $moteiConfigParams(spdUnit) 0]
    }
    $w.cv.valueUnit configure -itemList $moteiConfigParams(spdUnit) \
 	                      -default  $moteiSetupParams(spdUnit)

    $cv.speedFrame show

    set wA Enable
    set wa enable
    set wB Enable
    set wb enable
    set wC Enable
    set wc enable
    if {$gvar(context_sensitive)} {
	if {!$moteiConfigParams(amplifier)} { 
	    set wA Disable
	    set wa disable
	    if {$gvar(motionMode) == 2} { set gvar(motionMode) 5 }
	} 
	if {!$moteiConfigParams(brakeAction) || \
	    !$moteiConfigParams(digitalIO)} { set wB Disable }
	if {!$moteiConfigParams(controller)}  { 
	    set wC Disable
	    set wc disable
	    set gvar(motionMode) 2
	} 
    }
    pan${wA}MenuEntry Control "Power Control"
    pan${wA}MenuEntry Hardware Amplifier
    pan${wB}MenuEntry Control "Brake Control"
    pan${wC}MenuEntry Control "Control Loop"
    pan${wC}MenuEntry Hardware Monitoring
    pan${wC}MenuEntry Hardware Controller

    $cv.motionMode ${wa}Entry 1
    $cv.motionMode ${wc}Entry 2
    $cv.motionMode ${wc}Entry 4

    set mb normal ; set pi Enable ; set pu Enable
    if {$moteiConfigParams(IAXF) == 0} { set pi Disable }
    if {$moteiConfigParams(UCXF) == 0} { set pu Disable }
    if {$moteiConfigParams(UCXF) == 0 && $moteiConfigParams(IAXF) == 0} {
	set mb disabled
    } 
    [panGetMbPath Address] configure -state $mb
    pan${pi}MenuEntry Address "HW Initialization ..."
    pan${pu}MenuEntry Address "Unit Conversion ..."

    moteiControlUpdWidgets
}

#
# Termination Procedures
#
proc moteiControlExitPanel {  } {
    global gvar evtList mControl moteiControlLost

    #
    # detach motor
    #
    if {[info exists mControl]} {
	if {$moteiControlLost != 1} {
	    catch {set r [moteiControlSend [list motorCommand $mControl stop]]}
	    set r [moteiControlSend [list motorCommand $mControl detach]]
	} 
	set gvar(attached) 0
	set gvar(initStatus) 1
	set gvar(opMode)     1
	moteiControlSend [list detachControl $mControl]
    }

    #
    # detach DB events
    #
    if {$moteiControlLost != 1} {
	foreach e $evtList {
	    catch {seq_evtDetach $e}
	}
    }

    moteiCloseExternalApp moteiControlAppList
}

proc moteiControlInitPanel { } {
    global gvar EDITING
    global mControl

    if {$EDITING} return

    #------------------------------------------------
    # read configuration & configure widgets
    if {[moteiControlReadDbConfig] == "FAILURE" } { return "FAILURE" }
    moteiControlCfgWidgets

    #------------------------------------------------
    # Create control instance
    #
    set mControl "$gvar(lcuEnv):$gvar(motorName)"
    moteiControlSend [list createControl $gvar(lcuEnv) $gvar(motorName)]
    
    #------------------------------------------------
    # DeInstall+Install and Attach the motor 
    #
    set gvar(attached) 0
    if {[moteiControlInstall] == "FAILURE"} { 
	moteiControlSend [list detachControl $mControl]
    } 
}   

proc moteiControlUpdCtrl { } {
    global gvar evtList mControl moteiControlLost
    
    if {![cequal [moteiMonitorSend [list queryMonitor $gvar(lcuEnv)]] "OK"]} { 
	if {$moteiControlLost == 0} {
	    set moteiControlLost 1
	    set gvar(initStatus) 1
	    set gvar(opMode)     1
	    set gvar(attached)   0
	    set evtList {}
	    moteiControlSend [list detachControl $mControl]
	    moteiControlCfgWidgets
	}
    } elseif {$moteiControlLost == 1} {
	set moteiControlLost 0
	moteiControlDbEvt
	moteiControlSend [list createControl $gvar(lcuEnv) $gvar(motorName)]
	moteiControlInstall
	moteiControlCfgWidgets
    }
    after 1000 moteiControlUpdCtrl
}

proc moteiControlDbEvt { } {
    global gvar evtList

    #
    # attach events to attributes for dynamic update of buttons
    #
    set p "$gvar(dbPath):STATUS"
    set attrList [list opMode motionStatus initStatus]
    lappend evtList [seq_evtAttach $p.opMode       NE  { moteiControlUpdOpMode %D }]
    lappend evtList [seq_evtAttach $p.motionStatus NE  { moteiControlUpdWidgets motionStatus %D }]
    lappend evtList [seq_evtAttach $p.initStatus   NE  { moteiControlUpdWidgets initStatus %D }]

    #
    # read DB values and update widgets & menubar
    #
    foreach a $attrList {
	set gvar($a) [seq_dbReadSymbolic $p.$a]
    }

    set attr "$gvar(dbPath):SERVER:CONF.motor(0,4)"
    set gvar(opModeSet)    [seq_dbReadSymbolic $attr]
}

#--------------------------------------------------
# Send command to moteiServer ...
#
proc moteiControlCommand { cmd } {
    global gvar mControl

    return [moteiControlSend [list motorCommand $mControl $cmd]]
}

#--------------------------------------------------
# Install motor ...
#
proc moteiControlInstall { } {
    global gvar

    set gvar(attached) 0
    if {[moteiControlCommand deinstall] != "FAILURE"} { 
	moteiControlCommand install
    }
    if {[moteiControlCommand attach] != "FAILURE"} { 
	set gvar(attached) 1 
    }

    #------------------------------------------------
    # read configuration & configure widgets
    moteiControlReadDbConfig
    moteiControlCfgWidgets

    return "OK"
}

#--------------------------------------------------
# Menu Command Control->Update Configuration ...
#
proc moteiControlReinstall { } {
    global gvar 
    
    if {$gvar(attached) == 1} {
	if {[moteiControlCommand detach] == "FAILURE"} { return "FAILURE" }
	set gvar(attached) 0
    }
    return [moteiControlInstall]
}

#--------------------------------------------------
# Menu Command Control->Detach Control ...
#
proc moteiControlDetach { } {
    global gvar mControl
    
    if {[moteiControlCommand detach] == "FAILURE"} { return "FAILURE" }
    set gvar(attached) 0
    return "OK"
}

#--------------------------------------------------
# Menu Command Control->Clear Control ...
#
proc moteiControlClear { } {
    global gvar mControl
    
    if {[moteiControlCommand clear] == "FAILURE"} { return "FAILURE" }
    set gvar(attached) 0
    return [moteiControlInstall]
}

#--------------------------------------------------
# Menu Command OpMode ...
#
proc moteiSetOpMode { } {
    global mControl cv bcv gvar

    if {$gvar(opModeSet) == $gvar(opMode)} return

    set command [list motorCommand $mControl setOpMode $gvar(opModeSet)]
    set reply [moteiControlSend $command]
    if {$reply == "FAILURE"} {
	set gvar(opModeSet) $gvar(opMode)
	return
    }

    set opMode $gvar(opModeSet)
    if {$opMode == 4 || $opMode == 5} {
	$cv.motionMode enable
	$cv.posType    enable
	moteiControlUpdWidgets
    } else {
	$cv.motionMode disable
	$cv.posType disable
	$bcv.moteiControl-INIT disable
	$bcv.moteiControl-MOVE disable
    }
    if {!$gvar(context_sensitive)} {
	$cv.motionMode enable
	$cv.posType enable
	$bcv.moteiControl-INIT enable
	$bcv.moteiControl-MOVE enable
    }
}

proc moteiSetMotionMode {motionMode {access ""}} {
    global gvar cv mControl speedFrame
    global moteiSetupParams moteiConfigParams

    if {[cequal $access "Force"]} {
	set attr "$gvar(dbPath):STATUS.motionMode"
	set motionModeSave [seq_dbReadSymbolic $attr]
	set command [list motorCommand $mControl setMotionMode $motionMode]
	if {[moteiControlSend $command] == "FAILURE"} {
	    set gvar(motionMode) $motionModeSave
	    return "FAILURE"
	}
    }

    moteiUpdateSpecialPositions
    moteiUpdatePositionModeWidgets $motionMode

    set spdDefUnit [lindex $moteiConfigParams(defaultUnits) 1]
    set moteiConfigParams(spdUnit) $spdDefUnit
    if {$motionMode == 2} {
	# Direct mode
	if {$spdDefUnit != "Enc/ms" && $spdDefUnit != "Ref"} {
	    lappend moteiConfigParams(spdUnit) "Ref"
	}
	if {$moteiSetupParams(spdUnit) == "Enc/ms"} {
	    set moteiSetupParams(spdUnit) "Ref"
	}
    } else {
	# other modes
	if {$spdDefUnit != "Enc/ms" && $spdDefUnit != "Ref"} {
	    lappend moteiConfigParams(spdUnit) "Enc/ms"
	}
	if {$moteiSetupParams(spdUnit) == "Ref"} {
	    set moteiSetupParams(spdUnit) "Enc/ms"
	}
    }
    if {$speedFrame != 1} return

    set w [$cv.speedFrame getClassPath]
    $w.cv.valueUnit configure -itemList $moteiConfigParams(spdUnit) -default $moteiSetupParams(spdUnit)
    $cv.speedFrame show
    moteiConvertVelocityRange
}


proc moteiSetPosType {posType} {
    moteiUpdatePositionTypeWidgets $posType
    moteiUpdatePositionRange
}

#--------------------------------------------------
# Menu CheckButton Context Sensitive
#
proc moteiSetContext { } {
    global cv bcv gvar

    #
    # force enable/show on context free
    #
    $cv.motionMode enable
    $cv.posType    enable

    moteiControlCfgWidgets
    moteiControlUpdWidgets
}

#--------------------------------------------------
# INIT action button
#
proc moteiInitMotor { initCmd } {
    global gvar mControl moteiConfigParams
    
    if {![info exist mControl]} return

    set command [list motorCommand $mControl $initCmd]
    set reply [moteiControlSend $command]
    return $reply
}

#--------------------------------------------------
# MOVE action button
#
proc moteiSetSpdDef { } {
    global gvar speedFrame moteiSetupParams

    set spd {}
    set gvar(speedHow) [expr {$speedFrame + 1}]
    lappend spd $gvar(speedHow)
    switch $gvar(speedHow) {
	1 { # By Default
	    lappend spd "<none>,0,0.0,<none>"
	}
	2 { # By Value
	    lappend spd "<none>,0"
	    lappend spd $gvar(speedClass_value)
	    lappend spd $gvar(speedClass_valueUnit)
	    set moteiSetupParams(spdUnit) $gvar(speedClass_valueUnit)
	}
	3 { # By Name
	    lappend spd $gvar(speedClass_nameValue)
	    lappend spd "0,0.0,<none>"
	}
	4 { # By Index
	    lappend spd "<none>"
	    lappend spd $gvar(speedClass_indexValue)
	    lappend spd "0.0,<none>"
	}
    }
    lappend spd "$gvar(far),$gvar(near),$gvar(stable)"
    return [join $spd ,]
}

proc moteiSetPosDef { } {
    global gvar positionFrame moteiSetupParams

    set pos {}

    set gvar(posHow) [expr {$positionFrame + 2}]
    if {$gvar(posType) == 1 || $gvar(posType) == 2 } {
	set posHow $gvar(posHow)
	lappend pos $posHow
	switch $posHow {
	    2 { # By Value
		lappend pos $gvar(posClass_value)
		lappend pos $gvar(posClass_valueUnit)
		set moteiSetupParams(posUnit) $gvar(posClass_valueUnit)
		lappend pos "<none>,0,0"
	    }
	    3 { # By Name
		if {$gvar(positionMode) == 5 || $gvar(positionMode) == 6} {
		    lappend pos $gvar(offsetClass_value)
		    lappend pos $gvar(offsetClass_valueUnit)
		    set moteiSetupParams(posUnit) $gvar(offsetClass_valueUnit)
		} else {
		    lappend pos "0.0,Enc"
		}
		lappend pos $gvar(posClass_nameValue)
		lappend pos $gvar(posClass_sectionValue)
		lappend pos "0"
	    }
	    4 { # By Index
		if {$gvar(positionMode) == 5 || $gvar(positionMode) == 6} {
		    lappend pos $gvar(offsetClass_value)
		    lappend pos $gvar(offsetClass_valueUnit)
		    set moteiSetupParams(posUnit) $gvar(offsetClass_valueUnit)
		} else {
		    lappend pos "0.0,Enc"
		}
		lappend pos "<none>"
		lappend pos $gvar(posClass_sectionValue)
		lappend pos $gvar(posClass_indexValue)
	    }
	}
	lappend pos $gvar(ts_ht)
    } else {
	lappend pos $gvar(posClass_value)
	lappend pos $gvar(posClass_valueUnit)
	set moteiSetupParams(posUnit) $gvar(posClass_valueUnit)
    }
    return [join $pos ,]
}

proc moteiMoveMotor { } {
    global gvar mControl

    if {![info exist mControl]} return

    set pos {} ; set spd {}

    if {[moteiSetMotionMode $gvar(motionMode) "Force"] == "FAILURE"} return

    if {$gvar(motionMode) == 3 || $gvar(motionMode) == 4} {
	# Position/Tracking
	set pos [moteiSetPosDef]
    }

    set spd [moteiSetSpdDef]
    if {$gvar(posType) != 8} {
	set command [list motorCommand $mControl defSpd $spd]
	set reply [moteiControlSend $command]
	if {$reply == "FAILURE"} return
    }

    set par $pos
    switch $gvar(posType) {
	1 { set mt "ABS" }
	2 { set mt "REL" }
	3 { set mt "LHW" }
	4 { set mt "UHW" }
	5 { set mt "REF" }
	6 { set mt "IND" }
	7 { set mt "HOM" }
	8 { set mt "SPD" ; set par $spd }
    }
    set cmd move$mt
    set command [list motorCommand $mControl $cmd $par]
    set reply [moteiControlSend $command]
    return $reply
}

#--------------------------------------------------
# STOP action button
#
proc moteiStopMotion { } {
    global mControl

    if {![info exist mControl]} return
    set command [list motorCommand $mControl stop ]
    set reply [moteiControlSend $command]
    return $reply
}

#--------------------------------------------------
# WAIT action button
#
proc moteiWaitMotion { {what "Move"} } {
    global mControl

    if {![info exist mControl]} return
    set command [list motorCommand $mControl wait$what ]
    set reply [moteiControlSend $command]
    return $reply
}

#--------------------------------------------------
# EMERGENCY STOP button
#
proc moteiEmergencyStop { } {
    global mControl

    if {![info exist mControl]} return
    set command [list motorCommand $mControl emergencyStop ]
    set reply [moteiControlSend $command]
    return $reply
}

#--------------------------------------------------
# Status Sampling
#
proc moteiInitSampling { } {
    global gvar mControl
    
    if {![info exist mControl]} return
    set command [list motorCommand $mControl initSam $gvar(samplingRate)]
    return [moteiControlSend $command]
}

#--------------------------------------------------
# 
#
proc boardmoteiParWindow {b} {
    global mControl gvar
    
    #    moteiDialogBox "WARNING" "Sorry - Not implemented yet"
    #    return ""
    #    if {$b == "controller"} {
    #	moteiMac4ParamWindow $mControl
    #    } else {
    #	amplParamWindow $mControl
    #    }
    if { [winfo exists .parW]} {
	raise .parW
	return
    }
    # -- set default board MCON
    set gvar(boardType) 2

    toplevel .parW
    label .parW.l -text "Retrieve/Load board parameters"
    pack .parW.l -side top
    
    frame .parW.f
    UifRadioButton .parW.f.ampl -label AMPL -variable gvar(boardType) -onVal 1
    UifRadioButton .parW.f.mcon -label MCON -variable gvar(boardType) -onVal 2
    pack .parW.f -side top
    pack .parW.f.ampl .parW.f.mcon -side left
    
    iwidgets::entryfield .parW.par -labeltext PAR -width 6 \
	    -textvariable gvar(boardPar)
    iwidgets::entryfield .parW.val -labeltext VAL -width 6 \
	    -textvariable gvar(boardVal)
    pack .parW.par .parW.val -side top
    frame .parW.f1
    button .parW.f1.load -text Load -command loadPar
    button .parW.f1.retr -text Retrieve -command retrPar
    pack .parW.f1 -side top
    pack .parW.f1.load .parW.f1.retr -side left -anchor w -padx 10
}

#--------------------------------------------------
# LOAD Board Parameter
#
proc loadPar { } { 
    global gvar mControl

    set reply [moteiControlSend [list motorCommand  $mControl \
	    loadPar $gvar(boardType) $gvar(boardPar) $gvar(boardVal)]]
}

#--------------------------------------------------
# RETRIEVE Board Parameter
#
proc retrPar { } {
    global gvar mControl
    
    set reply [moteiControlSend [list motorCommand  $mControl \
	    retrPar $gvar(boardType) $gvar(boardPar)]]
    if { $reply != "FAILURE"} {
	set gvar(boardVal) [lindex $reply end]
    }
}

#
#
# ___oOo___
