#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiControlSend.tcl,v 2.43 2005/04/19 12:32:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  22/05/96  modified moteiControlSend to wait for last reply
# imunoz  10/05/96  added parameter errNr to proc sendServer
# imunoz  06/05/96  created
#

#************************************************************************
#   NAME
#       moteiControlSend - send a command script to the Control Server
#
#   SYNOPSIS
#       moteiControlSend <script>
#       pingServer <environment> <server>
#       sendServer <args>
#
#   DESCRIPTION
#       The procedure moteiControlSend accepts a script as argument.
#       The script is a list that tell the Control Server the actions
#       to perform. 3 commands are implemented:
#         createControl : create an instance of the class moteiMotorControl
#         motorCommand  : send a command to motci@<lcuEnv> 
#         detachControl : delete the instance of the class moteiMotorControl
#
#       The server process is called moteiServer_<id>, where <id> is the
#       3 first characters of the user name (see whoami).
#
#       The procedure pingServer returns the state of the Control Server:
#       ie running or not.
#
#       The procedure sendServer is the internal procedure that communicates
#       with the Control Server, it is only called by moteiControlSend.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#       moteiControlServer(l), moteiControlMotor(l)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#--------------------------------------------------
# methods for message system interface 
#
proc moteiControlSend { script } {
    global seq_ccsEnvName gvar moteiWhoami
    
    set Env  $seq_ccsEnvName
    set Proc "moteiServer_$moteiWhoami"
    set reply [sendServer $Env $Proc SCRIPT $script err cmdId lastReply]
    if {$err != 0} { return "FAILURE" } 

    if {$lastReply == 0} {
	set timeOut 20000
	set Cmd [string range [lindex [split $script ] 2] 0 3]
	if {[string compare $Cmd "init"] == 0} {
	    set timeOut 300000
	}
	if {[string compare $Cmd "move"] == 0} {
	    set timeOut 300000
	}
	if { [seq_catch \
		{set reply [seq_msgRecvReply -$timeOut $cmdId errorNr -last -ascii]} \
		msg [info script]]} {
	    return "FAILURE"
	} 
	if {$errorNr != 0} {
	    set msg "Error/reply from $Env/$Proc response to\
		    \"SCRIPT $script\":\n$reply"
	    return "FAILURE"
	}
    }
    return $reply
}

proc moteiMonitorSend { script } {
    global seq_ccsEnvName gvar moteiWhoami
    
    set Env  $seq_ccsEnvName
    set Proc "moteiMonitor_$moteiWhoami"
    set reply [sendServer $Env $Proc SCRIPT $script err cmdId lastReply]
    if {$err != 0} { return "FAILURE" } 

    if {$lastReply == 0} {
	if { [seq_catch \
		{set reply [seq_msgRecvReply $cmdId errorNr -last -ascii]} \
		msg [info script]]} {
	    return "FAILURE"
	} 
	if {$errorNr != 0} {
	    set msg "Error/reply from $Env/$Proc response to\
		    \"SCRIPT $script\":\n$reply"
	    return "FAILURE"
	}
    }
    return $reply
}

proc pingServer { Env Server } {
    global seq_errLogging

    # disable error logging
    set seq_errLogging "manual"

    # send PING to process and get reply
    if { [catch "seq_msgSendCommand $Env $Server PING" cmdId] } {
	set reply "FAILURE"
    } else {
	set reply [seq_msgRecvReply $cmdId errNr -ascii]
    }

    # discard the error stack
    seq_errResetStack -internal

    # enable normal error logging
    set seq_errLogging "auto"

    return $reply
}

proc sendServer { Env Proc Cmd {Par ""} {err ""} {cmdid ""} {last ""}} {

    if {$err   != ""} { upvar $err   en }
    if {$cmdid != ""} { upvar $cmdid id }
    if {$last  != ""} { upvar $last  lr }

    if { [seq_catch \
	    { seq_msgSendCommand $Env $Proc $Cmd $Par } id [info script] ]} {
	set reply $id
	if {![lempty $err]} {
	    set en 1
	    return $reply
	}
	# backward compatibility
	return "FAILURE"
    } 
    set timeOut 20000
    set command [lindex [split $Cmd ] 0]
    if {[string compare $command "WAITINI"] == 0} {
	set timeOut 300000
    }
    if {[string compare $command "WAITMOV"] == 0} {
	set timeOut 300000
    }
    if { [seq_catch \
	    {seq_msgRecvReply -$timeOut $id en lr} reply [info script]]} {
	if {$err != ""} {
	    set en 1
	    return "$reply"
	}
	# backward compatibility
	return "FAILURE"
    } 
    if {$en != 0} {
	set msg "Error/reply from $Env/$Proc response to\
		\"$Cmd $Par\":\n$reply"
	if {$err != ""} {
	    set en 1
	    return $msg
	}
	# backward compatibility
	return "FAILURE"
    }
    if {$reply == ""} { set reply "OK" }
    return $reply
}
#
# ___oOo___
