#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiHelp.tcl,v 2.43 2005/04/19 12:32:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   02/02/98  created
#

#************************************************************************
#   NAME
#       moteiHelp - display an extended help text
# 
#   SYNOPSIS
#       moteiHelp <top>
# 
#   DESCRIPTION
#       This panel contains a set of widgets dedicated to the display of
#       on-line extended help text.
#       The help file is taken from man/man7/$par(MY_NAME).7
#       where $par(MY_NAME) is the name of the calling panel.
#       The text is displayed in a window with vertical scrolling.
#       The button 'Close' destroys this window.
#
#   FILES
#       man/man7/motei*.7
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       It is assumed that the corresponding help file is located in the
#       man/man7 directory under $VLTROOT or $INTROOT.
#       It is built using ITCL code, therefore is not editable with the 
#       panel editor.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiHelp { {top ""} } {
    global par
    set name $par(MY_NAME)
    exec xterm -geometry 85x24 -font 7x13 -title "$name - Extended Help"\
	    -e moteiHelpWindow $name & 
}

proc moteiHelpOld { {top ""} } {
    global cv gvar par

    if {[winfo exists $top]} { wm deiconify $top ; raise $top ; return }

    toplevel $top

    moteiHelpInit $par(MY_NAME) name title dateTime helpText

    set f0 [frame $top.f0 ]
    set f1 [frame $top.f1 ]    
    set f2 [frame $top.f2 ]
    set fc [frame $top.fc ]
    
    pack $f0 $f1 $f2 $fc -pady 5 -fill both
    
    # Create/Pack widgets
    
    label $f0.name -text $name -font "-Adobe-courier-bold-r-normal--12*"
    label $f0.date -text $dateTime -font "-Adobe-courier-bold-r-normal--12*"
    pack $f0.name -side left -padx 10 ; pack $f0.date -side right -padx 10
    
    label $f1.title -text $title -font "-Adobe-courier-bold-r-normal--24*"
    pack $f1.title -expand 1

    UifHelpText $f2.text -canvasAnchor nw
    pack $f2.text -expand 1
    
    UifButton $fc.close -label Close -command "destroy $top"
    pack $fc.close -expand 1 
    
    # Variable Initialization
    $f2.text insertText $helpText
}

proc moteiHelpInit { pan name title date help } {
    upvar $name  nam
    upvar $title ttl
    upvar $date  tim
    upvar $help  hlp

    set fn "man/man7/${pan}.7"
    if {[catch {findVLTFile $fn} hfn]} return

    set fd [open $hfn "r"]
    if {$fd == -1} {
	moteiDialogBox "ERROR" "Unable to open file '$hfn'"
	return
    }

    set text {}
    while {[gets $fd line] != -1} {
	lappend text $line
    }
    close $fd

    set thNameFound 0
    set shNameFound 0
    set shHelpFound 0
    for { set n 0 } { $n < [llength $text] } { incr n } {
	set line [string trim [lindex $text $n]]
	set lst  [split $line]
	if {[string compare [lindex $lst 0] ".TH"] == 0 && \
		$thNameFound == 0} {
	    set nam "[lindex $lst 1](7)"
	    set tim [string trim [lindex $lst 4] \"]
	    set thNameFound 1
	}
	
	if {[string compare $line ".SH NAME"] == 0 && \
		$shNameFound == 0} {
	    incr n
	    set line [lindex $text $n]
	    set i [expr {[string first " - " $line] + 3}]
	    set ttl [string range $line $i end]
	    set ttl [string toupper [string trim $ttl]]
	    set shNameFound 1
	} 

	if {[string compare $line ".SH EXTENDED HELP"] == 0 && \
		$shHelpFound == 0} {
	    incr n 3
	    set m [expr {$n + 1}] ; set s "___oOo___"
	    while {[string compare $s [string trim [lindex $text $m]]] != 0} {
		incr m
	    }
	    set lst [lrange $text $n $m]
	    set len [llength $lst]
	    set hlp [join $lst \n]
	    set shHelpFound 1
	    break
	}
    }
}

#
# ___oOo___
