#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiMain.tcl,v 2.43 2005/04/19 12:33:02 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    09/09/97  Get motors from file $VLTDATA/ENVIRONMENTS/$lcu/MOTORS
# imunoz    31/05/96  commented out prompt on moteiQuit
# imunoz    27/11/95  handling of configuration file .moteiRc added
# pduhoux   16/05/95  created
#

#************************************************************************
#   NAME
#       moteiMain - Main panel for Environment/Motor and action selection
#
#   SYNOPSIS
#       moteiMain ["gvar(lcuEnv)=<environment>" ["gvar(motorName)=<motorName>"]]
# 
#   DESCRIPTION
#       The panel moteiMain.pan contains 2 entry selectors for Environment and
#       Motor selection, and 4 buttons allowing:
#        "Load Config ..." : to load the database with a .dbcfg file
#        "Configure ..."   : to tune the configuration of a motor via 
#                            dedicated panels.
#        "Control ..."     : to perform motions with a motor via the control
#                            panel.
#        "Status ..."      : to show the motor status on the status panel.
#
#       The file moteiMain.tcl contains all associated procedures.
#
#   FILES
#
#   ENVIRONMENT
#       
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main script motei(1).
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1)
#       moteiMain(7) via the MenuBar Help->Extended Help button
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc moteiSetCwp { mcmEnv mot } { 
    global gvar

    set gvar(dbPath) ""
    if {[cequal $mcmEnv "<none>"] || [cequal $mcmEnv ""]} return
    if {[cequal $mot    "<none>"] || [cequal $mot    ""]} return

    if {[catch {seq_dbSetCwp "@$mcmEnv<alias>$mot"}]} {
	return FAILURE
    } else {
	set gvar(dbPath) "@$mcmEnv<alias>$mot"
    }
}

proc moteiMain_StateMachine { {motor ""} } {
    global gvar cv bcv
    
    if {![info exists gvar(motorName)]} { 
	set motor "<none>" 
    } elseif {[cequal $motor ""]} { 
	set motor $gvar(motorName) 
    }

    $cv.wMotorSelector MakeMenu $gvar(moteiMotorList)

    if {![cequal $motor "<none>"]} {
        $cv.wdgLoadDatabase   enable
        $cv.wdgConfMotor      enable
	if { $gvar(wsDatabase) == 1} {
	    $cv.wdgCtrlMotor      disable
	    $cv.wdgShowStatus     disable
	} else {
	    $cv.wdgCtrlMotor      enable
	    $cv.wdgShowStatus     enable
	}
    } else {
        $cv.wdgLoadDatabase   disable
        $cv.wdgConfMotor      disable
        $cv.wdgCtrlMotor      disable
        $cv.wdgShowStatus     disable
    }
}

#
# Install default set of motors
#
proc moteiInstallTemplateMotors { lcu } {      
    global gvar
    
    set errMsg {}
    set errorSaved 0

    if {[catch {set ptList [seq_dbGetFamilyNames "@$lcu:" absolute]}]} {
	seq_errDisplay -internal -suspend
	return
    }
    if {[lsearch -exact $ptList ":MOTEI"] == -1} return
    if {[catch {set dbMoteiList [seq_dbGetFamilyNames "@$lcu:MOTEI" alias]}]} {
	seq_errResetStack -internal
	return
    }
    if {[llength $dbMoteiList] == 1} return
    set motList [lrange $dbMoteiList 1 end]
    set gap [string length <alias>]
    foreach mot $motList {
	set m [string range $mot $gap end]
	set err 0
	sendServer $lcu motServer INSTALL $m err
	if {$err == 1} {
	    set errorSaved 1
	    set errMsg "$errMsg Failed to install motor: $m\n"
	}
    }
    if {$errorSaved} { moteiDialogBox "ERROR" $errMsg }
}

#
# Callback for Environment Selector.
proc moteiEnvironmentCB { mcmEnv } {
    global gvar env cv seq_errLogging

    set seq_errLogging "manual"
    if {[catch {exec vccEnvCheck -e $mcmEnv -h $env(HOST) 2>/dev/null}]} {
	$cv.wMotorSelector configure -state disabled
	moteiDialogBox "ERROR" "Environment $mcmEnv : failed to retrieve information"

	if {[string index $mcmEnv 0] != "w"} {
            if {![cequal [pingServer $mcmEnv lccServer] "OK"]} {
		moteiDialogBox "ERROR" "Environment $mcmEnv : Invalid or not active"
		set gvar(lcuEnv) "<none>"
		return FAILURE
	    }
            if {![cequal [pingServer $mcmEnv motServer] "OK"]} {
		moteiDialogBox "ERROR" "Could not PING $mcmEnv/motServer"
		set gvar(lcuEnv) "<none>"
		return FAILURE
	    }
	}
    } 

    $cv.wMotorSelector configure -state normal

    if {[string index $mcmEnv 0] == "w"} {
	set gvar(wsEnv)  $mcmEnv
    } else {
	set gvar(lcuEnv) $mcmEnv
    }
    set envStatus [moteiStartMonitor]


    if {![cequal $envStatus "FAILURE"]} {
	moteiGetMotorList
	
	if { $gvar(motorName) == "<none>" } {
	    if {[llength $gvar(moteiMotorList)] > 0} {
		set gvar(motorName) [lindex $gvar(moteiMotorList) 0]
	    } else {
		set gvar(motorName) "<none>"
	    }
	}
    } else {
	set gvar(motorName) "<none>"
    }
    moteiMotorCB $gvar(motorName)
}

#
# Callback for Motor Selector.
# Set CWP in the lcu database to the path indicated
# by <alias>motor.
#
proc moteiMotorCB { motor } {
    global gvar
    
    if {$motor == "<none>" || $gvar(lcuEnv) == "<none>"} {
	moteiMain_StateMachine $motor
	return
    }

    #env is WS
    if { $gvar(wsDatabase) == 1} {
	set wsMoteiPoint @$gvar(lcuEnv)<alias>$motor
	if {[moteiSetCwp $gvar(lcuEnv) $motor] == "FAILURE"} {
	    seq_errResetStack -internal
	    moteiDialogBox "ERROR" "Unresolved motor alias @$gvar(lcuEnv)<alias>$motor"
	    set gvar(motorName) "<none>"
	    moteiMain_StateMachine 
	    return
	} elseif {[catch {set mlist [seq_dbGetFamilyNames $wsMoteiPoint ABSOLUTE]}]} {
	    seq_errResetStack -internal
	    moteiDialogBox "ERROR" "Invalid point @$gvar(lcuEnv)<alias>$motor"
	    set gvar(motorName) "<none>"
	    moteiMain_StateMachine 
	    return
	}
	#check that mlist contains SDL SERVER and STATUS
	if { ![string match *:SDL* $mlist] || \
		![string match *:SERVER* $mlist] || \
		![string match *:STATUS* $mlist] } {
	    set msg [format "%s\n%s" "Invalid motor point @$gvar(lcuEnv)<alias>$motor" \
		    "Sub-points SDL, SERVER and STATUS not found"]
	    moteiDialogBox "ERROR" $msg
	    set gvar(motorName) "<none>"
	    moteiMain_StateMachine 
	    return
	}
	if {[lsearch $gvar(moteiMotorList) $gvar(motorName)] == -1} {
	    set gvar(moteiMotorList) [linsert $gvar(moteiMotorList) 0 $gvar(motorName)]
	}
	moteiMain_StateMachine $gvar(motorName)
	return
    } 

    # env is LCU
    # 
    if {[lsearch $gvar(moteiMotorList) $motor] == -1} {
	set err 0
	set reply [sendServer $gvar(lcuEnv) motServer INSTALL $motor err]
	if {[set err]} {
	    moteiDialogBox "ERROR" "Failed to install motor: $motor"
	    set gvar(motorName) "<none>"
	    return 
	} 
	moteiGetMotorList
    }
    set gvar(motorName) $motor
    if {[moteiSetCwp $gvar(lcuEnv) $gvar(motorName)] == "FAILURE"} {
	moteiDialogBox "ERROR" "Unresolved motor alias @$gvar(lcuEnv)<alias>$motor"
	seq_errResetStack -internal
	set gvar(motorName) "<none>"
    }
    moteiMain_StateMachine $gvar(motorName)
}

#
# Get motors list on LCU
# Callback for LCU env selector
proc moteiGetMotorList { } {      
    global gvar cv env

    set mcmEnv $gvar(lcuEnv)
   
    if {[cequal $mcmEnv "<none>"] || [cequal $mcmEnv ""]} return
    
    # Get registered motors on LCU
    set gvar(moteiMotorList) {}
    
    # env is a LCU
    if {[string index $mcmEnv 0] == "l"} {
	moteiGetMotorListLCU $mcmEnv
    } else {
	moteiGetMotorListWS  $mcmEnv
    }

    if {[lempty $gvar(moteiMotorList)]} {
	if {![info exists gvar(motorName)]} {
	    set gvar(motorName) "<none>" 
	} elseif {[cequal $gvar(motorName) ""]} {
	    set gvar(motorName) "<none>"
	} 
    } else {
	if {![info exists gvar(motorName)]} {
	    set gvar(motorName) [lindex $gvar(moteiMotorList) 0]
	} elseif {[cequal $gvar(motorName) ""]} {
	    set gvar(motorName) "<none>"
	} elseif {[lsearch $gvar(moteiMotorList) $gvar(motorName)] == -1} {
	    set gvar(motorName) [lindex $gvar(moteiMotorList) 0]
	}
	if {[moteiSetCwp $mcmEnv $gvar(motorName)] == "FAILURE"} {
	    seq_errDisplay -internal -suspend
	}
    }
    
    # Refresh panel
    moteiMain_StateMachine
}

proc moteiGetMotorListLCU { lcuEnv } {
    global gvar 

    set gvar(wsDatabase) 0
    
    if {![cequal [pingServer $lcuEnv motServer] "OK"]} {
	moteiDialogBox "ERROR" "Could not PING $lcuEnv/motServer"
	return
    }
    set gvar(moteiMotorList) {}
    puts -nonewline "Retrieving installed motors on $lcuEnv : "
    vccWinBusy . "Retrieving installed motors on $lcuEnv" {
 	set mList [sendServer $lcuEnv motServer MOTORS "" err]
	if {! [set err] } {	
	    if {[lindex [split $mList ,] 0] == 0} {
  		set reply [moteiInstallTemplateMotors $lcuEnv]
		set mList [sendServer $lcuEnv motServer MOTORS "" err]
	    }
	    if {[lindex $mList 0] != 0 && ![set err]} {
		set ml [lrange [split $mList ,] 1 end]
		set gvar(moteiMotorList) $ml
		puts $ml		
	    } else {
		puts "None"
	    }
	} else {
	    puts "failed"
	    moteiDialogBox "ERROR" "Error Reply: $mList"    
	}
    }
}

proc moteiGetMotorListWS { wsEnv } {
    global gvar

    set gvar(moteiMotorList) {}
    set wsMoteiPoint "@$wsEnv:Appl_data:MOTEI"

    set gvar(wsDatabase) 1
    puts -nonewline "Retrieving motors in environment $wsEnv : "
    if {[catch {seq_dbGetAlias $wsMoteiPoint}]} {
	seq_errResetStack -internal
	puts "None"
	return
    }
    vccWinBusy . "Retrieving motor list in environment $wsEnv" {
	if {[catch {set mlist [seq_dbGetFamilyNames $wsMoteiPoint ABSOLUTE]}]} {
	    seq_errResetStack -internal
	    puts "None"
	    return
	}
	lvarpop mlist
	foreach el $mlist {
	    set tmp [ seq_dbGetAlias $el]
	    regsub -nocase {<alias>} $tmp "" el
	    lappend gvar(moteiMotorList) $el
	}
	puts $gvar(moteiMotorList)
    }
}

#
# Call moteiShowStatus panel
#
proc moteiShowStatus { } {                                        
    global moteiMainAppList
    vccWinBusy . "Starting Status Panel" {
	moteiCallExternalPanel moteiMainAppList "moteiStatus"
    }
}

#
# Call motei control panel, as an external application.
#
proc moteiCtrlMotor { } {                                           
    global moteiMainAppList
    
    # moteiServer is needed, start it (if not yet)
    moteiStartServer

    vccWinBusy . "Starting Control Panel" {
	moteiCallExternalPanel moteiMainAppList "moteiControl"
    }
}

#
# Call motei configure motor as an external application.
#
proc moteiConfMotor { } {                           
    global gvar moteiMainAppList

    # moteiServer is needed, start it (if not yet)
    if {! [set gvar(wsDatabase)]} {
	moteiStartServer
    }

    set appPar "gvar(controlMotor)=[expr {!$gvar(wsDatabase)}]"

    vccWinBusy . "Starting Configuration Panel" {
	moteiCallExternalPanel moteiMainAppList "moteiConfig" $appPar
    }
}

proc moteiStartServer { } {
    global gvar seq_ccsEnvName moteiServer moteiWhoami

    set server "moteiServer_$moteiWhoami"
    #
    # check CCS server process alive
    #
    if {![cequal [pingServer $seq_ccsEnvName $server] "OK"]} {
	seq_errResetStack -internal
	puts "Starting Server $server ... "
	set moteiServer [moteiStartExternalApp "Process" moteiServer]
    }
}

proc moteiStopServer { } {
    global gvar seq_ccsEnvName moteiServer moteiWhoami

    if {![info exists seq_ccsEnvName]} return
    set server  "moteiServer_$moteiWhoami"

    #
    # check CCS process alive
    #
    if {[cequal [pingServer $seq_ccsEnvName $server] "OK"]} {
	puts "Terminate Process $server ..."
	#
	# terminate the process in a clean way -> command exitServer
	#
	set reply [catch {moteiControlSend [list exitServer]} ret]
	#
	# check for pid active every 500ms up to 5s
	# passed this time, kill the process
	#
	set t 0 ; set pid [lindex [split $moteiServer "_"] 1]
	while {![catch {exec sh -c "ps | grep '^ *$pid '"}]} {
	    if {[incr t] > 10} {
		exec kill $pid &
		break
	    } 
	    after 500
	}
	puts "Process $server terminated"
    }
    set moteiServer 0
}

proc moteiStartMonitor { } {
    global gvar seq_ccsEnvName moteiMonitor moteiWhoami

    set monitor "moteiMonitor_$moteiWhoami"

    if {[string index $gvar(lcuEnv) 0] == "l"} {
	#
	# check CCS process alive
	#
	if {![cequal [pingServer $seq_ccsEnvName $monitor] "OK"]} {
	    seq_errResetStack -internal
	    puts "Starting Monitor $monitor ... "
	    set moteiMonitor [moteiStartExternalApp "Process" moteiMonitor $gvar(lcuEnv)]
	} else {
	    set moteiMonitor -1
	    puts "Monitor already running ... "
	}
	moteiMonitorSend [list createMonitor $gvar(lcuEnv)]
	moteiMonitorSend [list checkMonitor  $gvar(lcuEnv)]
	
	if {[cequal [moteiMonitorSend [list queryMonitor $gvar(lcuEnv)]] "OK"]} {
	    return "LCU"
	} else {
	    return "FAILURE"
	}
    } else {
	return "WS"
    }
}

proc moteiStopMonitor { } {
    global gvar seq_ccsEnvName moteiMonitor moteiWhoami

    if {![info exists seq_ccsEnvName]} return

    if {$moteiMonitor == -1} return
    #
    # check CCS monitor process alive
    #
    set monitor "moteiMonitor_$moteiWhoami"
    if {[cequal [pingServer $seq_ccsEnvName $monitor] "OK"]} {
	puts "Terminate Process $monitor ..."
	#
	# check for pid active every 500ms up to 5s
	# passed this time, kill the process
	#
	set pid [lindex [split $moteiMonitor "_"] 1]
	exec kill $pid &
	puts "Process $monitor terminated"
	set moteiMonitor 0
    }
}


#
#
# ___oOo___
