#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiSMAC4.tcl,v 2.43 2005/04/19 12:33:03 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   10/02/98  created
#

#************************************************************************
#   NAME
#       moteiSMAC4 - Maccon MAC4 Motion Controller status panel
# 
#   SYNOPSIS
#       moteiSMAC4 <args>
#
#   DESCRIPTION
#       The panel moteiSMAC4.pan contains a full set of output widgets 
#       showing the complete status of the Maccon MAC4 Motion Controller.
#       All widgets updated on database change. 
#       The file moteiSMAC4.tcl contains all associated procedures.
#       
#   FILES
#       
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       moteiStatus(7) via the MenuBar Help->Extended Help button
#
#   BUGS
#
#------------------------------------------------------------------------
#

proc moteiSMAC4_CV { what data } {
    global gvar
    set gvar($what) [lindex $data 2]
}

proc moteiSMAC4_USR { data } {
    global gvar

    set usr [lindex $data 2]
    set gvar(USR) [format "%08X" $usr]

    set gvar(MOD) [expr { $usr & 0x000000FF}]
    set gvar(EOM) [expr {($usr & 0x00000C00) >> 10}]

    set den       [expr {($usr & 0x00000100) >>  8}]
    set ini       [expr {($usr & 0x00001000) >> 12}]
    set gvar(AXS) [expr {($den << 1) | $ini}]

    set gvar(OUT) [expr {($usr & 0x00000200) >>  9}]

    set gvar(ERR) 0
    set gvar(ERR_MSG1) ""
    set gvar(ERR_MSG2) ""
    set err       [expr {($usr & 0x0007E000) >> 13}]
    if {$err} {
	set gvar(ERR) 1

	set ukn       [expr { $err & 0x01}]
	set rng       [expr {($err & 0x02) >> 1}]
	set uxc       [expr {($err & 0x04) >> 2}]
	set ecf       [expr {($err & 0x08) >> 3}]
	set drf       [expr {($err & 0x10) >> 4}]
	set fer       [expr {($err & 0x20) >> 5}]

	if {$ecf == 1} { set gvar(ERR_MSG1) "Encoder Fault" }
	if {$drf == 1} { set gvar(ERR_MSG1) "Drive Fault" }
	if {$fer == 1} { set gvar(ERR_MSG1) "Following Error" }
	if {$ukn == 1} { set gvar(ERR_MSG2) "Unknown Command" }
	if {$rng == 1} { set gvar(ERR_MSG2) "Range Error" }
	if {$uxc == 1} { set gvar(ERR_MSG2) "Unexpected Command" }
    } 
    set gvar(LIM) [expr {($usr & 0x00F80000) >> 19}]

    #    set gvar(FLY) [expr {($usr & 0x02000000) >> 25}]
    #    set gvar(SSI) [expr {($usr & 0x08000000) >> 27}]
    
}

proc moteiSMAC4_SYS { data } {
    global gvar cv

    set sys [expr {[lindex $data 2] & 0x00FFFFFF}]
    set gvar(SYS) [format "%06X" $sys]

    set ph1_4     [expr {($sys & 0x0000001E) >>  1}]
    set ph5       [expr {($sys & 0x00020000) >> 17}]
    set ph6       [expr {($sys & 0x00080000) >> 19}]
    
    set phase [expr {($ph6 << 5) | ($ph5 << 4) | $ph1_4}]
    for {set b 0; set phs 0} {$b < 6} {incr b} {
	if {[expr {($phase & (1 << $b)) >> $b}] == 1} {
	    incr phs [expr {0x1 << (4*$b)}]
	}
    }
    set gvar(PHASE) [format "%06X" $phs]

    set boostOn     [expr {($sys & 0x00000800) >> 11}]
    set boostEnable [expr {($sys & 0x00001000) >> 12}]
    set boostLock   [expr {($sys & 0x00040000) >> 18}]
    set gvar(BOOST) [expr {($boostLock << 2) | ($boostEnable << 1) | $boostOn}]
    set gvar(TURBO) [expr {($sys & 0x00002000) >> 13}]

    set ledP      [expr {($sys & 0x00008000) >> 15}]
    set ledF      [expr {($sys & 0x00010000) >> 16}]
    set x 244 ; set y 212
    if {$ledP == 1} {
        drawCircle $cv $x $y 6 Red
    } else {
        drawCircle $cv $x $y 6
    }
    set x 316
    if {$ledF == 1} {
        drawCircle $cv $x $y 6 Red
    } else {
        drawCircle $cv $x $y 6
    }

#    set gvar(RDY) [expr  $sys & 0x00000001]
#    set gvar(ZPD) [expr ($sys & 0x00000020) >>  5]
#    set gvar(SZP) [expr ($sys & 0x00000040) >>  6]
#    set gvar(ENC) [expr ($sys & 0x00000200) >>  9]
#    set gvar(REV) [expr ($sys & 0x00000400) >> 10]
#    set gvar(STP) [expr ($sys & 0x00004000) >> 14]
#    set gvar(BIL) [expr ($sys & 0x00100000) >> 20]

}

proc moteiSMAC4_COM { data } {
    global gvar

    set com [expr {[lindex $data 2] & 0x00FF}]
    set gvar(COM) [format "%02X" $com]

    set emr       [expr { $com & 0x01}]
    set ian       [expr {($com & 0x02) >> 1}]
    set wde       [expr {($com & 0x04) >> 2}]
    set gvar(COM_ERR) [expr {($ian << 2) |($wde << 1) | $emr}]
    
    set ba        [expr {($com & 0x38) >> 3}]
    for {set b 0; set adr 0} {$b < 3} {incr b} {
	if {[expr {($ba & (1 << $b)) >> $b}] == 1} {
	    incr adr [expr {0x1 << (4*$b)}]
	}
    }
    set gvar(ADR) [format "%04X" $adr]

    #    set gvar(USE) [expr {($com & 0x80) >> 7}]
}

#
# ___oOo___











