#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiSPlot.tcl,v 2.43 2005/04/19 12:33:04 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  24/11/97  created
#

#************************************************************************
#   NAME 
#       moteiSPlot - Plotting facility
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#       The panel moteiSPlot.pan 
#       The file moteiSPlot.tcl contains all associated procedures used 
#       by the panel to handle the sampling and plotting of motor parameters.
#
#------------------------------------------------------------------------
#

#
# Termination Procedure
#
proc moteiSPlotExitPanel { } {
    moteiSPlotStop
}

# Configure option buttons
proc moteiSPlotConfigButtons { } {
    global cv gvar

    # set motor parameters list
    
    set optList {}
    foreach rec $gvar(motParam) {
	set currOpt [lindex $rec 0]
	if {[lsearch -exact $optList $currOpt] == -1} {
	    lappend optList $currOpt
	}
    }
    for { set i 1} { $i <= 4 } {incr i} {
	$cv.param$i configure -itemList $optList
	# set default option
	if {[lsearch -exact $optList $gvar(defParam$i)] != -1} {
	    set gvar(param$i) $gvar(defParam$i)
	}
    }
    
    # set default value for save in file
    set gvar(saveInFile) $gvar(defSave)
    # set default sampling rate
    set gvar(rate) $gvar(defRate)
}

proc moteiSPlotUpdateButtons { {x "" } } {

    # Declare global variables
    global gvar bcv

    set gvar(paramList) {}
    for { set i 1} { $i <= 4 } {incr i} {
	set param $gvar(param$i)
	set dbAttr [moteiSPlotAttrFromLabel $gvar(motParam) $param ]
	if {[string length $dbAttr] > 0 } {
	    lappend gvar(paramList) $dbAttr
	}
    }

    if {$gvar(plotRun) == 1} {
	$bcv.moteiSPlot-START configure -state disabled
	$bcv.moteiSPlot-STOP  configure -state normal
    } else {
	$bcv.moteiSPlot-STOP  configure -state disabled
	if {[llength $gvar(paramList)] > 0} {
	    $bcv.moteiSPlot-START configure -state normal
	} else {
	    $bcv.moteiSPlot-START configure -state disabled
	}
    }
}


proc moteiSPlotBuildFileName { {x "" } } {
    global env gvar

    if {$gvar(saveInFile) == 0} {
	set gvar(fName) ""
	set gvar(pName) ""
	set gvar(fileName) ""
    } else {
	if {[info exists env(INS_ROOT)]} {
	    set dir "$env(INS_ROOT)/SYSTEM/DETDATA"
	    set pName "INS_ROOT/SYSTEM/DETDATA"
	} else {
	    set dir "/tmp"
	    set pName $dir
	}
	set gvar(pName) "under : $pName"
	set isoTimeStamp [exec date -u +"%Y-%m-%dT%X"]
	set gvar(fName) "$gvar(motorName).samp.${isoTimeStamp}"
	set gvar(fileName) "$dir/$gvar(fName)"
    }
}


# get attribute name from label
proc moteiSPlotAttrFromLabel { options param } {
    global gvar
    
    set attr ""
    foreach rec $options {
	set currParam [lindex $rec 0]
	if {([string compare $currParam $param] == 0) } {
	    set attr [lindex $rec 1]
	    break
	}
    }
    return $attr
}

# start sampling motor parameters
proc moteiSPlotStart { } {
    global gvar bcv
    
    set status [moteiSampStart $gvar(lcuEnv) $gvar(motorName) \
	                       $gvar(paramList) $gvar(rate) $gvar(fileName)]
    if {$status == "FAILURE"} { 
	seq_errDisplay -suspend 
	return
    } 
    set gvar(plotRun) 1
    $bcv.moteiSPlot-START configure -state disabled
    $bcv.moteiSPlot-STOP  configure -state normal
}


# stop sampling motor parameters
proc moteiSPlotStop { } {
    global gvar bcv
    
    moteiSampStop $gvar(lcuEnv) $gvar(motorName)
    set gvar(plotRun) 0
    $bcv.moteiSPlot-STOP  configure -state disabled
    $bcv.moteiSPlot-START configure -state normal
}

#************************************************************************
#   NAME
#   moteiSampStart, moteiSampStop - Tcl procedures used to sample LCU data
# 
#   SYNOPSIS
#   moteiSampStart <lcu> <motor> <paramList> <rate> [<fileName>]
#   moteiSampStop <lcu> <motor>
# 
#   DESCRIPTION
#   These procedures are used by MCM EI to sample and plot motor parameters
#   moteiSampStart starts the sampling the specified OLDB attributes for
#                the spoecified motor with the specified rate (Hz) . 
#                The results are plotted.
#                If filename is specified, they are also written in a 
#                ASCII file
#                The point used to support the plotting is defined in the 
#                global variable gvar(plotTable). See moteiGlobalData.
#   moteiSampStop  stops sampling data for the specified motor. The plotting
#                is also stopped
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#   SUCCESS or FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#
#   set file $env(INS_ROOT)/SYSTEM/MISC/INSDATA/RFILM_current.samp
#   set stat [moteiSampStart luvics1 RFILM "STATUS.current" 10 $file]
#
#   set file $env(INS_ROOT)/SYSTEM/MISC/INSDATA/RFILM_currAndErr.samp
#   set attr "STATUS.current STATUS.followingErr"
#   set stat [moteiSampStart luvics1 RFILM $attr 1 $file]
#
#   moteiSampStop luvics1 RFILM
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# start sampling motor parameters
proc moteiSampStart { lcu motor paramList rate {fileName ""} } {
    global gvar env
    
    if {[string length $motor] == 0} {
	#seq_errAdd "no motor name specified" moteiSampStart
	error "no motor name specified"
	return "FAILURE"
    }
    set nParam [llength $paramList]
    if {$nParam == 0} {
	#seq_errAdd "no parameter specified for sampling" moteiSampStart
	error "no parameter specified for sampling"
	return "FAILURE"
    }
    
    set paramStr ""
    for { set i 0} { $i < $nParam} {incr i} {
	set param [lindex $paramList $i]
	set paramStr "$paramStr \"$param\""
    }
    if {[cequal $fileName ""]} { 
	set saveInFile 0 
	set fileName "none"
    } else { 
	set saveInFile 1 
    }

    # start sampling and at the same time plotting sampled data
    eval exec sampCtrl_start.sh $saveInFile $fileName "$gvar(plotTable)" \
            $lcu LCU $rate 3 "$paramStr" &;
    
    return "OK"
}


# stop sampling motor parameters
proc moteiSampStop { lcu motor } {
    global gvar

    # stop sampling
    eval exec sampCtrl_stop.sh "$gvar(plotTable)" $lcu LCU &;
}

#
# ___oOo___
