#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiStatus.tcl,v 2.43 2005/04/19 12:33:06 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz  08/05/96  added interactive setting of pollRate
# pduhoux  21/03/95  created
#

#************************************************************************
#   NAME
#       moteiStatus - motor status panel
# 
#   SYNOPSIS
#       moteiStatus <args>
#
#   DESCRIPTION
#       The panel moteiStatus.pan contains a full set of output widgets 
#       showing the complete status of a motor. 
#       All widgets but position, speed and current are updated on database 
#       change. The latter are updated by polling with a period 
#       to be set from the panel.
#       The file moteiStatus.tcl contains all associated procedures.
#       
#   FILES
#       
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       This panel is not intended to be invoked from the UNIX Shell 
#       but only from the main panel.
#
#   EXAMPLES
#
#   SEE ALSO
#       motei(1), moteiMain(7)
#       moteiStatus(7) via the MenuBar Help->Extended Help button
#
#   BUGS
#
#------------------------------------------------------------------------
#
proc moteiStatusInitDbEvents { } {
    global gvar evtList

    set p "$gvar(dbPath):STATUS"

    set attrList [list opMode \
	    motionMode motionStatus motionStep inPos \
	    initStatus initStep \
	    axisEnabled powerStatus brakeClamped interlock \
	    mconStatus mconOnLimit mconOnInd mconOnRef \
            amplStatus amplOnLimit amplOnHome \
	    nextPosUser nextPosEnc nextPosName nextPosSection \
	    posName posSection posUnit speedUnit currentUnit]

    foreach a $attrList {
	set gvar($a) [seq_dbReadSymbolic $p.$a]
    }	
    if {$gvar(amplAvail) == 0} {
	set gvar(amplOnSwitch) -1
    }
    if {$gvar(mconAvail) == 0} {
	set gvar(mconOnSwitch) -1
    }
    lappend evtList [seq_evtAttach $p.opMode         NE {update_opMode         %D}]
    lappend evtList [seq_evtAttach $p.motionMode     NE {update_motionMode     %D}]
    lappend evtList [seq_evtAttach $p.motionStatus   NE {update_motionStatus   %D}]
    lappend evtList [seq_evtAttach $p.motionStep     NE {update_motionStep     %D}]
    lappend evtList [seq_evtAttach $p.inPos          NE {update_inPos          %D}]
    lappend evtList [seq_evtAttach $p.initStatus     NE {update_initStatus     %D}]
    lappend evtList [seq_evtAttach $p.initStep       NE {update_initStep       %D}]
    lappend evtList [seq_evtAttach $p.axisEnabled    NE {update_axisEnabled    %D}]
    lappend evtList [seq_evtAttach $p.powerStatus    NE {update_powerStatus    %D}]
    lappend evtList [seq_evtAttach $p.brakeClamped   NE {update_brakeClamped   %D}]
    lappend evtList [seq_evtAttach $p.interlock      NE {update_interlock      %D}]
    lappend evtList [seq_evtAttach $p.mconStatus     NE {update_mconStatus     %D}]
    lappend evtList [seq_evtAttach $p.mconOnLimit    NE {update_mconOnLimit    %D}]
    lappend evtList [seq_evtAttach $p.mconOnInd      NE {update_mconOnInd      %D}]
    lappend evtList [seq_evtAttach $p.mconOnRef      NE {update_mconOnRef      %D}]
    lappend evtList [seq_evtAttach $p.amplStatus     NE {update_amplStatus     %D}]
    lappend evtList [seq_evtAttach $p.amplOnLimit    NE {update_amplOnLimit    %D}]
    lappend evtList [seq_evtAttach $p.amplOnHome     NE {update_amplOnHome     %D}]
    lappend evtList [seq_evtAttach $p.nextPosUser    NE {update_nextPosUser    %D}]
    lappend evtList [seq_evtAttach $p.nextPosEnc     NE {update_nextPosEnc     %D}]
    lappend evtList [seq_evtAttach $p.nextPosName    NE {update_nextPosName    %D}]
    lappend evtList [seq_evtAttach $p.nextPosSection NE {update_nextPosSection %D}]
    lappend evtList [seq_evtAttach $p.posName        NE {update_posName        %D}]
    lappend evtList [seq_evtAttach $p.posSection     NE {update_posSection     %D}]
    lappend evtList [seq_evtAttach $p.posUnit        NE {update_posUnit        %D}]
    lappend evtList [seq_evtAttach $p.speedUnit      NE {update_speedUnit      %D}]
    lappend evtList [seq_evtAttach $p.currentUnit    NE {update_currentUnit    %D}]
}

proc update_opMode { data } {
    global gvar ; set gvar(opMode) [lindex $data 2]
}
proc update_motionMode { data } {
    global gvar ; set gvar(motionMode) [lindex $data 2]
}
proc update_motionStatus { data } {
    global gvar ; set gvar(motionStatus) [lindex $data 2]
}
proc update_motionStep { data } {
    global gvar ; set gvar(motionStep) [lindex $data 2]
}
proc update_inPos { data } {
    global gvar ; set gvar(inPos) [lindex $data 2]
}
proc update_initStatus { data } {
    global gvar ; set gvar(initStatus) [lindex $data 2]
}
proc update_initStep { data } {
    global gvar ; set gvar(initStep) [lindex $data 2]
}
proc update_axisEnabled { data } {
    global gvar ; set gvar(axisEnabled) [lindex $data 2]
}
proc update_powerStatus { data } {
    global gvar ; set gvar(powerStatus) [lindex $data 2]
}
proc update_brakeClamped { data } {
    global gvar ; set gvar(brakeClamped) [lindex $data 2]
}
proc update_interlock { data } {
    global gvar ; set gvar(interlock) [lindex $data 2]
}
proc update_mconStatus { data } {
    global gvar ; set gvar(mconStatus) [lindex $data 2]
}
proc update_mconOnLimit { data } {
    global gvar ; set gvar(mconOnLimit) [lindex $data 2] 
    moteiStatusMcon
}
proc update_mconOnInd { data } {
    global gvar ; set gvar(mconOnInd) [lindex $data 2]
    moteiStatusMcon
}
proc update_mconOnRef { data } {
    global gvar ; set gvar(mconOnRef) [lindex $data 2]
    moteiStatusMcon
}
proc update_amplStatus { data } {
    global gvar ; set gvar(amplStatus) [lindex $data 2]
}
proc update_amplOnLimit { data } {
    global gvar ; set gvar(amplOnLimit) [lindex $data 2] 
    moteiStatusAmpl
}
proc update_amplOnHome { data } {
    global gvar ; set gvar(amplOnHome) [lindex $data 2] 
    moteiStatusAmpl
}
proc update_nextPosUser { data } {
    format_output nextPosUser [lindex $data 2]
}
proc update_nextPosEnc { data } {
    global gvar ; set gvar(nextPosEnc) [lindex $data 2]
}
proc update_nextPosName { data } {
    global gvar ; set gvar(nextPosName) [lindex $data 2]
}
proc update_nextPosSection { data } {
    global gvar ; set gvar(nextPosSection) [lindex $data 2]
}
proc update_posName { data } {
    global gvar ; set gvar(posName) [lindex $data 2]
}
proc update_posSection { data } {
    global gvar ; set gvar(posSection) [lindex $data 2]
}
proc update_posUnit { data } {
    global gvar ; set gvar(posUnit) [lindex $data 2]
}
proc update_speedUnit { data } {
    global gvar ; set gvar(speedUnit) [lindex $data 2]
}
proc update_currentUnit { data } {
    global gvar ; set gvar(currentUnit) [lindex $data 2]
}

proc format_output { what value } {
    global gvar

    set absval [expr {abs($value)}]
    if {$absval < 1e+7} {
	set fmt "%+12.3f"
    } elseif {$absval < 1e+08} {
	set fmt "%+12.2f"
    } elseif {$absval < 1e+09} {
	set fmt "%+12.1f"
    } elseif {$absval < 1e+10} {
	set fmt "%+12.0f"
    } else {
	set fmt "%+g"
    }
    set gvar($what) [format $fmt $value]
}

proc updatePoll { } {
    global gvar
    
    set p "$gvar(dbPath):STATUS"

    if {[moteiMonitorSend [list queryMonitor $gvar(lcuEnv)]] == "OK"} { 
	set gvar(posEnc)           [seq_dbReadSymbolic $p.posEnc]
	format_output posUser      [seq_dbReadSymbolic $p.posUser]
	format_output followingErr [seq_dbReadSymbolic $p.followingErr]
	format_output speed        [seq_dbReadSymbolic $p.speed]
	format_output current      [seq_dbReadSymbolic $p.current]
    } 
    after $gvar(pollRate)   updatePoll
}

#------------------------------------------------------------------------------
# PROCEDURE : moteiStatusAmpl
#------------------------------------------------------------------------------
proc moteiStatusAmpl { } {
    global gvar

    set amplOnSwitchSave $gvar(amplOnSwitch)

    if {$gvar(amplAvail) == 0} {
	set gvar(amplOnSwitch) -1
    } else {
	if {$gvar(amplIsStandAlone)} {
	    set gvar(amplOnSwitch) -1
	    if {$gvar(amplOnHome)} {
		set gvar(amplOnSwitch) 7
	    }
	} else {
	    if {$gvar(amplOnLimit)} {
		set gvar(amplOnSwitch) $gvar(amplOnLimit)
	    } else {
		set gvar(amplOnSwitch) 0
		if {$gvar(amplOnHome)} {
		    set gvar(amplOnSwitch) 7
		}
	    }
	}
    }

    if {$amplOnSwitchSave != $gvar(amplOnSwitch)} {
	update idletasks
	after 10
    }
}

#------------------------------------------------------------------------------
# PROCEDURE : moteiStatusMcon
#------------------------------------------------------------------------------
proc moteiStatusMcon { } {
    global gvar

    set mconOnSwitchSave $gvar(mconOnSwitch)
    if {$gvar(mconOnLimit)} {
	set gvar(mconOnSwitch) $gvar(mconOnLimit)
    } else {
	set gvar(mconOnSwitch) 0
	if {$gvar(mconOnRef)} {
	    if {$gvar(mconOnInd)} { 
		set gvar(mconOnSwitch) 9 
	    } else {
		set gvar(mconOnSwitch) 7
	    }
	} else {
	    if {$gvar(mconOnInd)} { 
 		set gvar(mconOnSwitch) 8
	    }
	}
    }
    if {$gvar(mconAvail) == 0} {
	set gvar(mconOnSwitch) -1
    }
    if {$mconOnSwitchSave != $gvar(mconOnSwitch)} {
	update idletasks
	after 10
    }
}

#------------------------------------------------------------------------------
# PROCEDURE : moteiStatusTime
#------------------------------------------------------------------------------
proc moteiStatusTime { } {
    global gvar
 
    set attr "$gvar(dbPath):STATUS.time"
    set moteiMotionTimes [seq_dbReadSymbolic $attr]

    # convert STATUS.time to UTC formatted strings
    set fmt "%02d:%02d:%02d.%03d"
    set time0 [lindex $moteiMotionTimes 0]
    set time1 [lindex $moteiMotionTimes 1]
    set time0sec  [expr { [lindex $time0 0] % 86400}]
    set time0msec [expr {([lindex $time0 1] + 500) / 1000}]
    set time1sec  [expr { [lindex $time1 0] % 86400}]
    set time1msec [expr {([lindex $time1 1] + 500) / 1000}]

    set h [expr {$time0sec / 3600}]
    set r [expr {$time0sec % 3600}]
    set m [expr {$r / 60}]
    set s [expr {$r % 60}]
    set gvar(moteiMotionBegTime) [format $fmt $h $m $s $time0msec]

    set h [expr {$time1sec / 3600}]
    set r [expr {$time1sec % 3600}]
    set m [expr {$r / 60}]
    set s [expr {$r % 60}]
    set gvar(moteiMotionEndTime) [format $fmt $h $m $s $time1msec]

    # compute motionDuration 
    set s  [expr {[lindex $time1 0] - [lindex $time0 0]}]
    set us [expr {[lindex $time1 1] - [lindex $time0 1]}]
    set ms [expr {($us + 500) / 1000}]
    while {$ms < 0} {
	incr ms 1000
	incr s  -1
    }
    if {$s < 0} {
	set gvar(moteiMotionDuration) "Undetermined"
    } else {
	set gvar(moteiMotionDuration) [format "%d.%03d" $s $ms]
    }
}

#------------------------------------------------------------------------------
# PROCEDURE : moteiStatusMask
#------------------------------------------------------------------------------
proc moteiStatusMask { {data {}} } {

    global gvar cv

    panGetCvNames moteiStatus cv bcv

    if {[llength $data] == 0} {
	set mask 0
    } else {
	set mask [lindex $data 2]
    }

    if {$mask == $gvar(moteiEvtMaskSave)} return
    set gvar(moteiEvtMaskSave) $mask
    set gvar(moteiEventList) {}
    $cv.moteiStatus-wdgEvtList deleteItem  0 end
    if {$mask == 0} return

    set evtLst [list "Motion End"       "Drive Fault" \
	             "Emergency Stop"   "Over Temperature" \
		     "Over Current"     "Positioning Error" \
		     "On Limit"         "Normal Stop" \
		     "Interlock Active" "New Motion" \
		     "Board Init Ready" "" ]

    set i 0 ; set bit 1
    while {$i < 32} {
	if {[expr {($mask & $bit)}] != 0} {
	    lappend gvar(moteiEventList) [lindex $evtLst $i]
	}
	set bit [expr {$bit * 2}]
	incr i
    }
    $cv.moteiStatus-wdgEvtList addItemList 0 $gvar(moteiEventList)
    update idletasks
    after 100
}

proc moteiChangePollRate { {delta 0} } {
    global gvar

    if {$gvar(pollRate) <= 500 && $delta <= 0} return
    incr gvar(pollRate) $delta
}

#
# Termination Procedure
#
proc moteiStatusExitPanel { } {
    global moteiStatusAppList
    moteiCloseExternalApp moteiStatusAppList
}

#
