#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiTools.tcl,v 2.43 2005/04/19 12:33:06 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux  28/03/95  created
#

#************************************************************************
#   NAME
#       moteiCallExternalPanel, moteiCloseExternalApp, moteiStartExternalApp, moteiDialogBox - tool procedures
#
#   SYNOPSIS
#       moteiCallExternalPanel <panel> <extPar>
#       moteiCloseExternalApp 
#       moteiStartExternalApp <what> <app> <appPar>
#       moteiDialogBox <Message>
#
#   DESCRIPTION
#       The procedure moteiCallExternalPanel invokes the panel named <panel>
#       and passes it the list of arguments <extPar> and the standard argument
#       list ["gvar(dbPath)=$gvar(dbPath)" "gvar(lcuEnv)=$gvar(lcuEnv)" \
#             "gvar(motorName)=$gvar(motorName)" ]
#       It invokes the procedure moteiStartExternalApp for a panel
#       The panel <pid> is stored in a list.
#
#       The procedure moteiCloseExternalApp terminates all applications 
#       launched from this process via the list. The processes are killed
#       (SIGKILL).
#
#       The procedure moteiStartExternalApp launches the specified process
#       and waits until the latter has registered under CCS. Finally it 
#       returns its PID.
#
#       The procedure moteiDialogBox pops up a tk_dialog window displaying the 
#       specified message. The window is closed on confirmation button click.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
   
proc moteiCallExternalPanel { appList panel {extPar ""} } {
    global gvar moteiWhoami

    upvar $appList al

    set panPar [list "gvar(dbPath)=$gvar(dbPath)" \
	             "gvar(lcuEnv)=$gvar(lcuEnv)" \
                     "gvar(motorName)=$gvar(motorName)" \
                     "moteiWhoami=$moteiWhoami" \
                     $extPar ]
    #
    # For all configuration panels, pass gvar(controlMotor)
    #
    if {[string match "moteiC\[A-Z\]*" $panel] && \
       ![string match "moteiCSet*"   $panel]} {
	if {[info exists gvar(controlMotor)]} {
	    set cm $gvar(controlMotor)
	    lappend panPar "gvar(controlMotor)=$cm"
	}
    }

    set pid [moteiStartExternalApp "Panel" $panel $panPar]
    if {[lempty pid]} {
	promptDialog "ERROR : Could not start Panel:$panel"
	return
    }
    lappend al $pid
}

proc moteiCloseExternalApp { appList } {
    global gvar seq_ccsEnvName

    upvar $appList al
    if {![info exists al]} return

    #
    # for all sub-panels
    #
    while { [llength $al] } {
	set app [lindex $al 0]
	#
	# check CCS process alive
	#
	set pr [pingServer $seq_ccsEnvName $app]
	if {![cequal $pr "OK"]} {
	    set pid [lindex [split $app "_"] 1]
	    if {![catch {exec sh -c "ps -e | grep '^ *$pid '"}]} {
		exec kill -9 $pid &
	    }
	    set al [lreplace $al 0 0]
	    continue
	}
	puts "Closing panel:$app ... "
	#
	# terminate the panel in a clean way -> message SCRIPT shutdown
	#
        set pnf [format "%sExit" [lindex [split $app "_"] 0]]
	vccWinBusy . "Closing panel: $app" {
	    if {[seq_catch {set cmdId [seq_msgSendCommand $seq_ccsEnvName $app \
		    SCRIPT [list shutDown]]} msg $pnf] != 0 } {
		#
		# on error, kill the process
		#
		set pid [lindex [split $app "_"] 1]
		exec kill -9 $pid &
		continue
	    } 
	}
	
	#
	# check for pid active every 500ms up to 30s
	# passed this time, kill the process
	#
	set t 0 ; set pid [lindex [split $app "_"] 1]
	while {![catch {exec sh -c "ps -e | grep '^ *$pid '"}]} {
	    if {[incr t] > 60} {
		exec kill -9 $pid &
		break
	    } 
	    after 500
	}

	puts "Panel $app closed"
	set idx [lsearch $al $app]
	set al [lreplace $al $idx $idx]
    }
}
	
#-----------------------------------------------------------
# Stolen from S.Sandrock's ccseiStartApp
# start application while displaying a busy window
# NOTE: the delays used here are not intended to be precise!
proc moteiStartExternalApp { what app {appPar {}}} {

    set timeout [expr 4*30] ;# wait max. 30 sec for startup
    set appx [vccFindFile bin/$app]
    if {[lempty $appx]} { puts "$what not found $app"; return }
    vccWinBusy "          Starting $what:$app ...          " {
	puts "Starting $what: $app ... "
	set pid [eval exec $appx $appPar &]
	while {[incr timeout -1] > 0} {
	    if {! [catch {exec sh -c "ps -e | grep '^ *$pid '"}]} {
		after 2000 
		break
	    }
	    after 250
	}
    }
    set ppid "$app\_$pid"
    puts "$what started : $ppid"
    return $ppid
}

proc moteiHoldPanel { trg } {
    foreach el $trg {
	blt::busy hold $trg
    }
    update
}

proc moteiReleasePanel { trg } {
    foreach el $trg {
	blt::busy release $trg
    }
}

proc moteiDialogBox {Label Message} {
    tk_dialog .dialog "[wm title .]: $Label" $Message info 0 OK
}

proc shutDown { } {
    #
    # close panel
    #
    panClosePanel
}

#
# ___oOo___
