#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiConvert_uifTools.tcl,v 2.43 2005/04/19 12:32:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   05/02/98  created
#

#************************************************************************
#   NAME
#       moteiConvertPositionRange,moteiConvertVelocityRange - Conversion routines for position and speed extrema 
#
#   SYNOPSIS
#       moteiConvertPositionRange <args>
#       moteiConvertVelocityRange <args>
#
#   DESCRIPTION
#       These procedures are belonging to the UIF classes moteiSetTarget and
#       moteiSetSpeed. They are triggered by the Unit selector so as to update
#       the extrema fields accordingly.
#       They can however be invoked from any other procedure assumed instances
#       of the 2 associated UIF classes exist.
#       The extrema are stored in the global array moteiStatusParams
#         posMini/posMiniEnc, posMaxi/posMaxiEnc,
#         spdMini/spdMiniEnc, spdMaxi/spdMaxiEnc
#       If the global array is not available, the extrema are set to "N/A"
#
#       The extrema are global and stored into:
#         gvar(posClass_minValue)   / gvar(posClass_maxValue)
#         gvar(speedClass_minValue) / gvar(speedClass_maxValue)
#
#       The procedures apply only for positionFrame = 0 (ie position by value)
#       and speedFrame = 1 (ie speed by value).
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#--------------------------------------------------
# convert position range
#
proc moteiConvertPositionRange { args } {
    global gvar positionFrame moteiStatusParams

    if {![info exists positionFrame]} return
    if {$positionFrame != 0} return

    if {![info exists moteiStatusParams]} {
	set na "   N/A    "
	set gvar(posClass_minValue) $na
	set gvar(posClass_maxValue) $na
	return
    }

    if {$gvar(posClass_valueUnit) == "Enc"} {
	set posMini $moteiStatusParams(posMiniEnc)
	set posMaxi $moteiStatusParams(posMaxiEnc)
    } else {
	set posMini $moteiStatusParams(posMini)
	set posMaxi $moteiStatusParams(posMaxi)
    }
    set gvar(posClass_minValue) $posMini
    set gvar(posClass_maxValue) $posMaxi

    update idletasks
}

#--------------------------------------------------
# convert speed range
#
proc moteiConvertVelocityRange { args } {
    global gvar speedFrame moteiStatusParams moteiSetupParams

    if {![info exists speedFrame]} return
    if {$speedFrame != 1} return

    if {![info exists moteiStatusParams]} {
	set na "   N/A    "
	set gvar(speedClass_minValue) $na
	set gvar(speedClass_maxValue) $na
	return
    }
	
    if {($gvar(speedClass_valueUnit) == "Enc/ms") || \
	($gvar(speedClass_valueUnit) == "Ref") } {
	set spdMini $moteiStatusParams(spdMiniEnc)
	set spdMaxi $moteiStatusParams(spdMaxiEnc)
    } else {
	set spdMini $moteiStatusParams(spdMini)
	set spdMaxi $moteiStatusParams(spdMaxi)
    }
	    
    set gvar(speedClass_minValue) $spdMini
    set gvar(speedClass_maxValue) $spdMaxi
    
    set moteiSetupParams(spdUnit) $gvar(speedClass_valueUnit)

    update idletasks
}

#
# ___oOo___
