#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiDevice_uifTools.tcl,v 2.43 2005/04/19 12:32:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pduhoux   05/02/98  created
#

#************************************************************************
#   NAME
#       moteiDeviceComputeChannel,moteiDeviceExtractChannel - Compute device channel
#
#   SYNOPSIS
#       moteiDeviceComputeChannel <args>
#       moteiDeviceExtractChannel <args>
#
#   DESCRIPTION
#       These procedures belong to the UIF class moteiDevice
#
#       moteiDeviceComputeChannel is triggered by the Board & Channel selectors
#         so as to update the effective device number accordingly.
#         It can however be invoked from any other procedure assumed an 
#         instance of the associated UIF class exist.
#         It computes the device as:
#           d = 4*(b-1) + (c-1)
#
#       moteiDeviceExtractChannel shall be invoked when instanciating the 
#         moteiDevice class so as to assign proper values to the selectors.
#         It computes the board & channel as:
#           b = (d / 4) + 1
#           c = (d % 4) + 1
#
#       where <d> is the effective device number (from 0 to 23)
#             <b> is the board number (from 1 to 6)
#             <c> is the channel number on the board (from 1 to 4)
#
#       The values are global and stored into:
#         gvar(deviceClass_board) is <b>
#         gvar(deviceClass_canal) is <c> 
#         gvar(Channel)           is <d>
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#--------------------------------------------------
# compute channel
#
proc moteiDeviceComputeChannel { args } {
    global gvar 

    set bn $gvar(deviceClass_board)
    set bc $gvar(deviceClass_canal)

    set gvar(Channel) [expr {($bn - 1) * 4 + ($bc - 1)}]
}

proc moteiDeviceExtractChannel { args } {
    global gvar 

    set ch $gvar(Channel)

    set gvar(deviceClass_board) [expr {($ch / 4) + 1}]
    set gvar(deviceClass_canal) [expr {($ch % 4) + 1}]
}

#
# ___oOo___
