#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiDrawSquare_uifTools.tcl,v 2.43 2005/04/19 12:32:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    16/12/97  Created - Library of panel dedicated utilities
#

#************************************************************************
#   NAME
#       drawSquare - draw a box on the panel
#
#   SYNOPSIS
#       drawSquare <cv> <x0> <x1> <w> <h> <xoff> <yoff> <color> <width>
#
#   DESCRIPTION
#       This procedure draws a line of color <color> and width <width> 
#       starting at (x0,yoff) to (xoff,yoff), (xoff,yoff+h), (xoff+w,yoff+h),
#       (xoff+w,yoff) and (x1,yoff).
#       
# yoff> --------- title --------------- ^
#       |       ^       ^             | |   
#       |      x0       x1            | h   
#       |                             | |   
#       |                             | |   
#       ------------------------------- v
#       <-------------w--------------->
#       ^
#      xoff
#
#       The parameters <width> is defaulted to 2, <color> to Blue.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc drawSquare { cv x0 x1 w h xoff yoff {color Blue} {width 2} } {

   $cv create line $x0 $yoff \
	           $xoff $yoff \
                   $xoff [expr {$yoff + $h}] \
                   [expr {$xoff + $w}] [expr {$yoff + $h}]\
                   [expr {$xoff + $w}] $yoff \
		   $x1 $yoff \
                   -fill $color -width $width
}

proc drawCircle { cv x y r {color "backGround"} } {

    set r45 [expr {round($r * 0.707)}]
    if {![cequal $color "backGround"]} {
	$cv create oval [expr {$x - $r45}] [expr {$y - $r45}] \
		        [expr {$x + $r45}] [expr {$y + $r45}] \
                        -fill $color -outline $color
    } else {
	set color #d9d9d9
	$cv create oval [expr {$x - $r45}] [expr {$y - $r45}] \
		        [expr {$x + $r45}] [expr {$y + $r45}] \
                        -fill $color
    }
}

#
# ___oOo___
#
