#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifArrowsButton.tcl,v 2.43 2005/04/19 12:33:06 vltsccm Exp $" 
#
# uifArrowsButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    17/03/94   created 
# M.COMIN    31/07/95   Use args instead of config
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
#
#************************************************************************
#   NAME
#
#     UifArrowsButton - Class for VLT frame
# 
#   SYNOPSIS
#
#     UifArrowsButton  <widget-name>
# 
#   DESCRIPTION
#
#     UifArrowsButton defines the class for the VLT frame widget.
#     The user create a widget of this class (called instance) by :
#
#         UifArrowsButton <widget-name> <config-parameters>
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifArrowsButton - up/down arrow widget.
#
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Inherits options from UifWidget class and supports the followings :
#
#     height  <integer>     Determine the height of the widget
#     width   <integer>     Determine the widget length
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifArrowsButton .w  -width 200 -height 20 
#
#     Re-configure the widget
#
#     .w  configure -width 150
#
#     set width  [.w cget -width]
#
#   SEE ALSO
#     UifWidget
#
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual UifArrowsButton { }

class UifArrowsButton {
    inherit  UifWidget

    constructor {args} {}

    itk_option define -variable variable Variable    ""
    itk_option define -incrStep  incrStep  IncrStep  1  
    itk_option define -decrStep  decrStep  DecrStep  -1
    itk_option define -script    script    Script    ""

    public method step { what }

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifArrowsButton::constructor {args} {

   set b1 [findVLTFile bitmaps/uifUpArrow.xbm]  
   set b2 [findVLTFile bitmaps/uifDownArrow.xbm]

   itk_component add upArrow  {
      UifButton $itk_interior.upArrow -bitmap "@${b1}"
   } {
     keep -stateVar -command
   }

   itk_component add downArrow  {
      UifButton $itk_interior.downArrow -bitmap "@${b2}"
   } {
     keep -stateVar -command
   }

   pack $itk_component(upArrow) $itk_component(downArrow) -pady 1
   eval itk_initialize

   set initialized 1

   # Set default configuration

   $itk_component(upArrow)   configure -command "$this step up"
   $itk_component(downArrow) configure -command "$this step down"

   eval configure $args
}

#---------------------------------------------------------------
# OPTION : script - Tcl procedure invoked when the button is pushed
#                   The associated variable is increased/decreased before.
#---------------------------------------------------------------

configbody UifArrowsButton::script {
   
   if {! $initialized } { return }

   if {[cequal $itk_option(-script) ""]} {
      $itk_component(upArrow)   configure -command "$this step up"
      $itk_component(downArrow) configure -command "$this step down"
   } else {
      $itk_component(upArrow)   configure -command "$this step up ; $itk_option(-script)"
      $itk_component(downArrow) configure -command "$this step down ; $itk_option(-script)"
   }
}

#---------------------------------------------------------------
# METHOD : step - 
#---------------------------------------------------------------

body UifArrowsButton::step { what } {
   
    if { ![cequal $itk_option(-variable) ""] &&
          [uplevel #0 info exists $itk_option(-variable)] } {

      if {[cequal $what up]} {
         uplevel #0 incr $itk_option(-variable) $itk_option(-incrStep)
      } else {
         uplevel #0 incr $itk_option(-variable) $itk_option(-decrStep)
      }
   }
}

