#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifHelpText.tcl,v 2.43 2005/04/19 12:33:06 vltsccm Exp $" 
#
# who         when      what
# --------   --------   ----------------------------------------------
# pduhoux    03/02/98   created 
#
#************************************************************************
#   NAME
#     UifHelpText - Class for Help Text Widgets
#   SYNOPSIS
#     UifHelpText <widget-name>
#   DESCRIPTION
#
#     UifHelpText defines the class for the VLT help text widgets.
#     The user create a widget of this class (called instance) by :
#
#         UifHelpText  <widget-name> <config-parameters>
#
#   COMPONENTS
#
#     The itk 'component' method returns the component's full name.
#
#     scrollv - Vertical scroll bar
#     uifHelpText - Help Text widget
#
#   METHODS
#
#     Inherits methods from UifIntercativeWidget class and supports 
#     the following method :
#
#     insertText <text>
#        Insert text at the beginning of the window
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget class and supports :
#
#     font    <font name>
#     height  <integer>     Determine the height of the widget expressed
#                           in number of rows.
#     width   <integer>     Determine the widget length (in characters)
#
#   SPECIFIC BINDINGS
#
#     <Delete>       Deletes character left to the insertion cursor.
#                    It also deletes selected part of the text.
#     <BackSpace>    Deletes character left to the insertion cursor
#                    It also deletes selected part of the text.
#     <Arrow-Left>   Moves the insertion cursor leftwards
#     <Arrow-Right>  Moves the insertion cursor rightwards
#     <Arrow-Up>     Moves the insertion cursor one line upwards
#     <Arrow-Down>   Moves the insertion cursor one line downwards
#     <B1-Motion>    Drag the mouse left-button to select part of the text.
#
#     <Button-2>     Done by clicking the mouse middle-button.
#                    The selected part of the text is inserted in the position
#                    where the cursor is when the user clicks on <Button-2>.
# 
#     <Control-d>    Deletes the character at the left of the cursor current
#                    position.
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     UifIntercativeWidget  UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#

option add *UifHelpText.relief sunken widgetDefault
option add *UifHelpText.borderwidth 2 widgetDefault

itk::usual UifHelpText { }

class UifHelpText {
    inherit UifInteractiveWidget

    constructor {args} {}
        
    public method insertText { text } { 
      $itk_component(helpText) insert "0.0" $text
      return [$itk_component(helpText) configure -state disabled]
    }

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifHelpText::constructor {args} { 

   itk_component add f1 {
      frame $itk_interior.f1 
   }

   pack $itk_component(f1)  -fill both -expand 1

   itk_component add helpText {
      text $itk_component(f1).helpText -setgrid false 
   } { 
      keep -relief -width -height -font
      keep -background -foreground -wrap
   }

   itk_component add scrollv {
       scrollbar $itk_component(f1).scrollv \
	       -orient v -borderwidth 2 -width 10 -relief sunken \
	       -command [code $itk_component(helpText) yview]
                
   } {
      keep -activebackground
   }

   pack $itk_component(helpText) -side left -fill both -expand 1      
   pack $itk_component(scrollv)  -side left -fill y

   eval itk_initialize 

   # Set default configuration

   $itk_component(helpText) configure \
	   -relief sunken -borderwidth 2 -wrap word -height 24 -width 80 \
	   -font "-Adobe-courier-medium-r-normal--14*" \
	   -yscrollcommand [code $itk_component(scrollv) set]

   eval configure $args
}    

#
# ___oOo___
