#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginCONFIG.tcl,v 1.105 2005/10/12 19:47:31 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-10-04  ignore INSTALL.PACKAGE.TARGETS from appended files.
# rschmutz  2004-10-04  if "*INSTALL_VLTSW*.cfg" file not found:
#			just issue a warning and continue.
# rschmutz  2004-08-24  CfgAppend, CfgAppendFromModule: search appended
#			files also in 'appended' modules. For the time
#			being nested CfgAppendFromModule calls are not
#			supported.
# rschmutz  2004-08-23  CfgAppendFromModule added.
# rschmutz  2004-06-07  CfgAppend: allow nested INSTALL.APPENDi.FILEs.
# rschmutz  2004-06-02  CfgAppend: try parameter added.
# rschmutz  2004-06-02  use bash instead of csh.
# rschmutz  20/01/03    VLTSW20030023: seq_errLogging manual -> off.
# rschmutz  16/01/03    accept *.tar and *.tar.gz package files.
# rschmutz  11/05/01    accept append empty config. files.
# rschmutz  24/12/00    CfgAppend did not handle empty values correctly.
# rschmutz  24/12/00    retrieve config. file from tar file bug fixed.
# rschmutz  09/12/00    function VLT-RELEASE added.
# rschmutz  08/12/00    functions PLATFORM and NUM2X added.
# rschmutz  05/11/00    handling of expression INDEX added.
# rschmutz  17/09/00    allow arg1 to be an absolute path (for pkginVerify).
# rschmutz  2000-07-09  SaveAs fixed.
# rschmutz  2000-07-04  CfgAppend: allow values with spaces.
# rschmutz  2000-06-26  method Save added.
# rschmutz  2000-05-13  renamed, updated, man page added.
# sskole    2000-02-25  resolveString: added resolving of shell variables
# rschmutz  2000-01-20  INSTALL.DETi.* keywords added. 
# sskole    16/12/99  Added the method getModCPU.
# sskole    02/11/99  resolveString: fixed bug regarding variable resolving
# rschmutz  16/10/99  USER and USERMGR not longer mandatory.
# rschmutz  14/10/99  search config and scripts in INTROOT and VLTROOT.
# rschmutz  12/10/99  resolveString: speedup: if no '$', return.
# sskole    07/10/99  Fixed bug concerning missing command line cfg. file
# rschmutz  01/10/99  minor -cfg processing updates.
# sskole    26/08/99  Added INSTALL.FLAG.DELETEDIR
# sskole    25/08/99  Added getScanScript and getInitialScript
# rschmutz  20/08/99  simplified initPAF and getKeyword.
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginCONFIG - configuration file interface for pkgin
# 
#   SYNOPSIS
#	pkginCONFIG cfg arg1
# 
#   DESCRIPTION
#	Interface to the package configuration.
#	The constructor argument arg1 can be either:
#	1) an absolute path to the config. file
#	2) the package name: the config. file is searched
#	   in ./<package>/config/<package>INSTALL.cfg
#	3) a tar file: the config. file is extracted
#	   from the tar file, from location:
#	   ./INSTALL/INSTALL.cfg
#
#   PARENT
#	ctooOBJ_BASE
#
#   PUBLIC METHODS
#	CfgAppend name
#	    Appends the contents of the configuration file
#	    <name> to the current loaded configuration.
#
#	CfgAppendFromModule <name> <module> <version>
#	    Appends the contents of the configuration file
#	    <name> to the current loaded configuration.
#	    The configuration file is searched in module
#	    <module>,<version> located in the current
#	    directory.  If needed, the module is retrieved
#	    from CMM.
#
#	Get <parameter>
#	    Returns the value of a configuration parameter.
#
#	Save
#	    Saves the configuration.
#
#	SaveAs <name>
#	    Saves the configuration in file <name>.
#
#	Set <parameter> <value>
#	    Set a parameter value.
#
#   FILES
#	./<package>/config/<package>INSTALL.cfg - package config.
#	./INSTALL/INSTALL.cfg - config. file inside tar file
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	ctooCONFIG
#
#   BUGS     
#
#------------------------------------------------------------------------
#

package require SeqOslx


#************************************************************************
#*# class: pkginCONFIG
#------------------------------------------------------------------------

class pkginCONFIG {
    inherit ctooOBJ_BASE pkginLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { arg1 } {
        ctooOBJ_BASE::constructor -objType CONFIG -name PACKAGE

	#*# 1. check arg1
	if { "$arg1" == "" || "[cindex $arg1 0]" == "-" } {
	    error "Wrong argument $arg1."
	}

	#*# 2. load dictionary
	seq_oslxCmd CONFIG -loadDic $name PKGIN

	#*# 3. according to arg1 type:
	if { [crange $arg1 end-6 end] != ".tar.gz" && \
	     [crange $arg1 end-3 end] != ".tar" } {

	    #*# 4. load absolute path or package config. file
	    if { [cindex $arg1 0] == "/" } {
	        set cfgfile $arg1
	    } else {
	        set cfgfile [file join $arg1 "config" [cconcat $arg1 "INSTALL.cfg"]]
	    	set cfgfile "[pwd]/$cfgfile"
		set appendPath "[pwd]/$arg1"
	        set appendPaths "[pwd]/ [pwd]/ $appendPath/config/ $appendPath/config/"
	    }
	    if {![file exists $cfgfile]} {
	        error "Configuration file $cfgfile not found."
	    }
	    seq_oslxCmd CONFIG -load $name $cfgfile

	} else {

	    #*# 5. tar config. file: extract config. file
	    if {![file exists $arg1]} {
	        error "File $arg1 not found."
	    }
	    # "./" is needed, otherwise the file is not found (!!).
	    set cfgFile "./INSTALL/INSTALL.cfg"
	    exec pkginUnziptar $arg1 $cfgFile
	    if {![file exists $cfgFile]} {
	        error "Error extracting config. file from $arg1 (file $cfgFile not found)."
	    }

	    #*# 6. load config. file
	    seq_oslxCmd CONFIG -load $name "[pwd]/INSTALL/INSTALL.cfg"

	    #*# 7. delete config. file
	    file delete $cfgFile
	}
    }


    #****************************************************************
    #*# method: CfgAppend
    #----------------------------------------------------------------

    public method CfgAppend { appendName {try 0} {nesting 0} {log 1} } {
        global env

	#*# 1. check nesting level
	incr nesting
	if { $nesting >= 4 } {
	    error "Configuration file $appendName nested too deep."
	}

	#*# 2. search for <appendName>
	set found 0
	foreach path $appendPaths ext  { "" ".cfg" "" ".cfg" "" ".cfg" } {
	    set cfgFile [cconcat $path $appendName $ext]
	    if {[file isfile $cfgFile]} {
		set found 1
		break
	    }
	}
	if { $found == 0 } {
	    # if try and not found: return without error
	    if { $try } {
	    	return 0
	    }

	    # if loading a "*INSTALL_VLTSW*" file: return without error
	    if { [string first "INSTALL_VLTSW" $appendName] != -1 } {
	        Info "       Skipped $appendName (not found)."
	    	return 0
	    }
	    error "Configuration file $appendName not found."
	}

	#*# 3. log
	if { $log } {
	    Info "       Reading $appendName."
	}

	#*# 3. load dictionary to aux. buffer
	seq_oslxCmd CONFIG -loadDic pkginAux PKGIN

	#*# 4. load configuration to aux. buffer
	seq_oslxCmd CONFIG -load pkginAux $cfgFile

	#*# 5. retrive aux. buffer and store it in $name
	set appendFileList ""
	if { [seq_oslxCmd CONFIG -noOfKeywords pkginAux] } {
	    set buf [seq_oslxCmd CONFIG -getKeywordBuf pkginAux]
	    foreach keyval [lrange [split $buf ","] 1 end] {
	        lassign $keyval key val
		if { [string first "INSTALL.APPEND" $key] == 0 } {
		    if { ![regexp {^INSTALL\.APPEND[0-9]+\.FILE$} $key] } {
		        if { [regexp {^INSTALL\.APPEND[0-9]+\.MODULE$} $key] } {
		    	    error "INSTALL.APPENDi.MODULE in appended config. files not supported. Load module in the main config. file."
		        } elseif { [regexp {^INSTALL\.APPEND[0-9]+\.VERSION$} $key] } {
		    	    error "INSTALL.APPENDi.VERSION in appended config. files not supported. Load module in the main config. file."
		        } else {
		    	    error "Wrong keyword $key."
			}
		    }
		    if { "[csubstr $val 0 1]" == "\"" && "[csubstr $val end 1]" == "\"" } {
		        set val [csubstr $val 1 [expr {[clength $val] - 2}]]
		    }
		    lappend appendFileList $val
		} else {
		    # ignore INSTALL.PACKAGE.TARGETS from appended files
		    if { $key != "INSTALL.PACKAGE.TARGETS" } {
	                seq_oslxCmd CONFIG -storeKeyword $name $key "\"$val\""
		    }
		}
	    }
	}

	#*# 6. clear aux. buffer
	seq_oslxCmd CONFIG -clear pkginAux

	#*# 7. load nested INSTALL.APPENDi.FILEs
	foreach appendFile $appendFileList {
	    set appendFile [ResolveString $appendFile]
	    CfgAppend $appendFile 0 $nesting
	}

	return 1
    }


    #****************************************************************
    #*# method: CfgAppendFromModule
    #----------------------------------------------------------------

    public method CfgAppendFromModule { appendName modName modVersion } {

	Info "       Reading $appendName from module $modName $modVersion:"

        #*# 1. get maxbak value
	set maxbak 2
	set txt F
	catch { set txt [Get INSTALL.FLAG.DELETEDIR] }
	if { $txt } {
	    set maxbak 0
	}
	catch { set maxbak [Get INSTALL.CON.MAXBAK] }

	#*# 2. retrieve module
        pkginMODULE module -name $modName -version $modVersion \
			   -maxbak $maxbak
        module Retrieve
        delete object module

        #*# 3. load config. file
        set appendPath "[pwd]/$modName"
	lappend appendPaths "$appendPath/config/" "$appendPath/config/"
        CfgAppend $appendName 0 0 0
    }


    #****************************************************************
    #*# method: Get
    #----------------------------------------------------------------

    public method Get { param } {
	global seq_errLogging

	set prevErrLogging $seq_errLogging
	set seq_errLogging off
	set val [seq_oslxCmd CONFIG -getValue $name $param]
	if { "[csubstr $val 0 1]" == "\"" && "[csubstr $val end 1]" == "\"" } {
	    set val [csubstr $val 1 [expr {[clength $val] - 2}]]
	}
	set seq_errLogging $prevErrLogging

	return [ResolveString $val]
    }


    #****************************************************************
    #*# method: Set
    #----------------------------------------------------------------

    public method Set { param val } {
	if { "[csubstr $val 0 1]" == "\"" && "[csubstr $val end 1]" == "\"" } {
	    set val [csubstr $val 1 [expr {[clength $val] - 2}]]
	}
	seq_oslxCmd CONFIG -storeKeyword $name $param "\"$val\""
    }


    #****************************************************************
    #*# method: Save
    #----------------------------------------------------------------

    public method Save {} {
    	SaveAs $cfgfile
    }


    #****************************************************************
    #*# method: SaveAs
    #----------------------------------------------------------------

    public method SaveAs { filename } {
	if {[file exists $filename]} {
    	    if {![file writable $filename]} {
	        error "File $filename is read-only.  Unable to save."
	    }
	}
	seq_oslxCmd CONFIG -save $name fileName $filename
    }


    #****************************************************************
    #*# method: ResolveString
    #----------------------------------------------------------------

    protected method ResolveString { txt } {
        global env


        #*# 1. Resolve cfg. file keywords and env. vars.
        while { [string first "\$" $txt ] != -1 } {
	    # extract text before var., var. name and after var.
	    if {[regexp {([^$]*)\$\(?([A-Za-z0-9_\.]+)\)?(.*)} $txt dummy txt1 varname txt3]} {
		# if found: search config. var. or env. var.
		try_eval {
	            set txt2 [Get $varname]
	            set txt [cconcat $txt1 $txt2 $txt3]
		} {
		    try_eval { 
	    	        set txt2 $env($varname)
	        	set txt [cconcat $txt1 $txt2 $txt3]
			if { $varname == "HOST" } {
			    set hostUsed 1
			}
		    } {
			break
		    }
		}
	    } else {
		break
	    }
        }

	#*# 2. recursive resolve '[...]'
	while {[regexp {(.*)\[([^][]+)\](.*)} $txt dummy txt1 expr txt3]} {
	    set txt2 [ResolveExpr $expr]
	    set txt [cconcat $txt1 $txt2 $txt3]
	}
        
        #*# 3. Resolve shell variables  
        if { [string first "~" $txt ] != -1 } {
	    catch {exec bash -c "echo $txt"} txt
        } 

        return [string trim $txt " "]
    }


    #****************************************************************
    #*# method: ResolveExpr
    #----------------------------------------------------------------

    protected method ResolveExpr { txt } {
        global env

        #*# 1. search cmd.
        set cmd [ctooStrFirst $txt]
        set arg [ctooStrTail $txt]
        switch $cmd {
            INDEX { 
        	set index [ctooStrFirst $arg]
        	set ilist [ctooStrTail $arg]
		if { $index < 0 || $index >= [llength $ilist] } {
		    Exit "ERROR: Configuration error in '\[$txt\]': $index out of range (0..[expr {[llength $ilist]-1}])."
		}
	        return [lindex $ilist $index]
	    }
            EXPR  {
                if { [csubstr $arg 0 1] == "\"" && [csubstr $arg end 1] == "\"" } {
                    set arg [csubstr $arg 1 [expr {[clength $arg] - 2}]]
                }
                set val [expr {$arg}]
                return [expr {round(100.0*$val)/100.0}]
            }
	    NUM2X {
	    	regsub {[0-9]+} $arg x ret
	    	return $ret
	    }
	    PLATFORM {
	    	set platformUsed 1
		return [Platform $arg]
	    }
	    VLT-RELEASE {
	        try_eval {
	            set relist [file split $env(VLTROOT)]
		} {
		    return "NONE"
		}
		set lenlist [llength $relist]
		return [join [lrange $relist [expr {$lenlist-2}] [expr {$lenlist-1}]] "_"]
	    }
            default {
		error "Unknown command $cmd."
            }
        }

    }

    #****************************************************************
    #*# method: Platform
    #----------------------------------------------------------------

    public method Platform { arg } {
	set ruse 0
	if { "[exec uname]" == "Linux" } {
	    set rpos [lsearch $arg "-r"]
	    set rfile "/etc/redhat-release"
	    if { $rpos != -1 && [file isfile $rfile] && [file readable $rfile] } {
	    	set rval [lindex [read_file $rfile] 4]
		set ruse 1
	    }
	}
	if { $ruse } {
	    set ret [eval exec uname $arg]
	    set ret [translit " " "_" [lreplace $ret $rpos $rpos $rval]]
	} else {
    	    set ret [translit " " "_" [eval exec uname $arg]]
	}
	return $ret
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable hostUsed     0
    public variable platformUsed 0

    protected variable appendPaths ""
    protected variable cfgfile     ""
}

#
# ___oOo___
