#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginENV_BASE.tcl,v 1.105 2005/10/12 19:47:34 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-10-12  Nslookup: use host_info to avoid problems with Linux
#			(suggested/proposed by Erik).
# alongino  25/08/04  Method Backup: added $VLTDATA/config/<env name>.dbcfg file
# rschmutz  18/02/03  EvaluateCmd: replaced vccEnvStart with pkginEnvStart.
# rschmutz  12/02/03  for Solaris: search nslookup in /usr/sbin.
# rschmutz  19/01/03  call pkginEnvStart (instead of vccEnvStart).
# rschmutz  16/01/03  call vccEnvCreate (pkginEnvCreate deleted).
# rschmutz  06/01/03  Init: pass args. to pkginEnvInit.
# rschmutz  04/01/03  Nslookup: use host or nslookup, search for both.
# rschmutz  06/01/02  package require msqltcl added (SPR 20010719).
# rschmutz  16/10/01  always create env. in local host.
# rschmutz  14/05/01  use rsh/remsh according to operating system.
# rschmutz  14/05/01  dummy ForceSnap added.
# rschmutz  24/03/01  detect errors if remsh is used.
# rschmutz  07/01/01  Nslookup added.
# rschmutz  24/12/00  bug fix: wrong quote deleted.
# rschmutz  17/09/00  Verify method added.
# rschmutz  05/07/00  AccSelect added (to get the env. type).
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  28/05/00  renamed, updated, man page added.
# sskole    22/02/00  stop: vccEnvStop only issued twice for LCU envs.
# sskole    18/02/00  createScanLinks: Moved scan script part to top of loop. 
# sskole    31/01/00  evaluateCmd: Use -d option instead of -h when compressing.
#                     Due to the bug of -d, this is currently commented out.
# sskole    31/01/00  Shell script return values are now considered 
# rschmutz  2000-01-20  logs adjusted to include detector scanlinks.
# sskole    16/12/99  evaluateCmd: Use -h option when compressing.
# sskole    02/11/99  evaluateCmd: additional condition for remsh added.
# sskole    02/11/99  install: removed the change owner statement.
# sskole    20/10/99  evaluateCmd: added remsh
# rschmutz  16/10/99  scanLinks: check if both env. are built.
# rschmutz  11/10/99  getType added.
# sskole    27/09/99  Implemented the methods createScanLinks and install
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginENV_BASE - environment base class
# 
#   SYNOPSIS
#	inherit pkginENV_BASE
# 
#   DESCRIPTION
#	pkginENV_BASE is the base class for environments.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    Has to be overloaded.
#
#	Create
#	    Create the environment calling vccEnvCreate.
#
#	Init
#	    Initialize the environment calling vccEnvInit.
#
#	Start
#	    Start the environment calling vccEnvStart.
#
#	Stop
#	    Stop the environment calling vccEnvStop.
#
#	Backup
#	    Make a backup of the environment directory.
#
#   PUBLIC DATA
#	name:	environment name.
#	tplsrc: environment template directory (default: none).
#	user:	user that runs the environment (default: current user).
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#

package require msqltcl

#************************************************************************
#*# class: pkginENV_BASE
#------------------------------------------------------------------------

class pkginENV_BASE {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {
    
	#*# 1. user
	set txt ""
	catch { set txt [$cfg Get INSTALL.LOGINNAME.USER] }
	configure -user $txt

	#*# 2. maxbak
	set txt F
	catch { set txt [$cfg Get INSTALL.FLAG.DELETEDIR] }
	set txt [expr {2 * (1 - [Logical2Num $txt])}]
	catch { set txt [$cfg Get INSTALL.CON.MAXBAK] }
	configure -maxbak $txt
    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create { {argList ""} } {

    	#*# 1. backup or delete current directory
	Backup

        #*# 2. set VLTDATA instead of using the 'obsolete' -d option.
	set oldVltdata [GetEnv VLTDATA]
	set vltdata $oldVltdata
	catch { set vltdata [[[Repository] LookupName VLTDATA ROOT 2] cget -dir] }
        SetEnv VLTDATA $vltdata
	try_eval {

	    #*# 3. check that the env. is registered in ACC
	    set desthost ""
	    catch { set desthost [AccSelect "SELECT station_name FROM prog_environment WHERE progenv_name = '$name'"] }
	    if { "$desthost" == "" } {
	        error "Environment $name not registered in ACC (or unable to access ACC)."
	    }

	    #*# 4. force env. to be built always on local host
	    append argList " -d [GetEnv HOST]:"

	    #*# 5. append template
	    if { "$tplsrc" != "" } {
	        append argList " -s $tplsrc"
	    }

	    #*# 6. create
    	    EvaluateCmd vccEnvCreate $argList
	} {
	    error $errorResult $errorCode $errorInfo
	} {
	    #*# 7. reset VLTDATA
            SetEnv VLTDATA $oldVltdata
	}
    }

    #****************************************************************
    #*# method: Init
    #----------------------------------------------------------------

    public method Init { {argList ""} } {
    	EvaluateCmd pkginEnvInit $argList
    }

    
    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start { {argList ""} } {

    	# If processes residing in INTROOT are to be started, 
    	# $INTROOT/bin has to be in the PATH environment variable
    	set oldPath [GetEnv PATH]
	SetEnv PATH "[GetEnv INTROOT]/bin:$oldPath"
	try_eval {
	    EvaluateCmd pkginEnvStart $argList
	} {
	    error $errorResult $errorCode $errorInfo
	} {
	    SetEnv PATH $oldPath
	}
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
    	# ignore errors from stopping an environment
    	catch { EvaluateCmd vccEnvStop }
    }

    #****************************************************************
    #*# method: ForceSnap
    #----------------------------------------------------------------

    public method ForceSnap {} {
    	# default: nothing to do (only WS envs. need to make snapshots).
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {
    	#EvaluateCmd vccEnvCheck
    }

    #****************************************************************
    #*# method: Backup
    #----------------------------------------------------------------

    public method Backup {} {

	set vltdata [GetEnv VLTDATA]
	catch { set vltdata [[[Repository] LookupName VLTDATA ROOT 2] cget -dir] }
	set dir [file join $vltdata ENVIRONMENTS $name]
    	if {[file exists $dir]} {
            if {$maxbak != 0 && [string first "/COMPRESS/" $dir] == -1 } {
	    	BackupFile $dir $maxbak
            	Log "       Environment $name: renamed $dir."
            } else {
	        file delete -force $dir
            	Log "       Environment $name: deleted $dir."
	    }
	}
        # ALO: do the same for the .dbcfg file in $VLTDATA/config (see icbConfigSet)
	set dbcfgfile [file join $vltdata config $name.dbcfg]
    	if {[file exists $dbcfgfile]} {
            if {$maxbak != 0} {
	    	BackupFile $dbcfgfile $maxbak
            	Log "       Environment $name: renamed $dbcfgfile."
            } else {
	        file delete -force $dbcfgfile
            	Log "       Environment $name: deleted $dbcfgfile."
	    }
	}
    }

    #****************************************************************
    #*# method: EvaluateCmd
    #----------------------------------------------------------------

    public method EvaluateCmd {cmd {argList ""} } {
    
	#*# 1. build command
        set cmdStr "$cmd -v -e $name $argList"
	
	#*# 2. always start/stop under the user account
    	if { "$user" != "" && "[GetEnv USER]" != "$user" &&
	     ("$cmd" == "pkginEnvStart" || "$cmd" == "vccEnvStop") } {
	    if { "[exec uname]" == "HP-UX" } {
	    	set cmdStr "remsh [GetEnv HOST] -l $user -n '$cmdStr'"
	    } else {
	    	set cmdStr "rsh [GetEnv HOST] -l $user -n '$cmdStr'"
	    }
	}

	#*# 3. execute command
	#
	#      (Note: remsh does not propagate the return code
	#       of the called command: errors must be detected
	#       using stderr and stdout).
	#
	System $cmdStr {
	    foreach txt { "FAILED" "rror" "ommand not found" } {
	        if { [string first $txt "%e"] != -1 } {
	            error "Error running %c: %e."
	        }
	        if { [string first $txt "%c"] != -1 } {
	            error "Error running %c: %c."
	        }
	    }
        }
    }

    #****************************************************************
    #*# proc AccSelect
    #
    #   Apply an SQL SELECT instruction on the ACC database.
    #   Return the result set.
    #----------------------------------------------------------------

    protected method AccSelect { select } {

        #*# 1. connect to ACC database
        set ret ""
        set acc [msqlconnect [GetEnv ACC_HOST]]
        msqluse $acc acc

        #*# 2. execute select
        set n [msqlsel $acc $select]

        #*# 3. return result set
        loop i 0 $n {
	    lappend ret [msqlnext $acc]
        }
	msqlclose $acc

        return $ret
    }

    #****************************************************************
    #*# method: GrepFile
    #----------------------------------------------------------------

    protected method GrepFile { file grep } {

	#*# search file
	set var ""
	for_file line $file {
	    if {[regexp -- $grep $line]} {
	    	return 1
	    }
	}
	return 0
    }

    #****************************************************************
    #*# method: Nslookup - return 1 if host found, 0 otherwise
    #----------------------------------------------------------------

    protected method Nslookup { host } {
        return [expr {![catch { host_info official_name $host }]}]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable tplsrc  ""
    public variable type    ""
    public variable user    ""
    public variable maxbak  2
    public variable procnames ""
}


#************************************************************************
#___oOo___

