#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginINSROOT.tcl,v 1.105 2005/10/12 19:47:34 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-04-09  Backup: backup complete directory.
#			(volac now starts to look up from the new
#			directory automatically).
# rschmutz  2005-04-08  SPR-VLTSW20050112: permissions updated.
# rschmutz  2005-01-28  SPR VLTSW20050032: remove trailing '/'.
# rschmutz  2004-12-19  SPR VLTSW20040292: chmod +t added.
# alongino  25/08/04  Backup: if maxbak = 0, just delete
# rschmutz  27/03/03  Create: force arg. added.
# rschmutz  14/05/01  DiskSpace method added.
# rschmutz  11/10/00  Verify: do not complain if no INS_ROOT found.
# rschmutz  17/09/00  Verify method added.
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    18/02/00  install: Set the correct group write permissions
# sskole    27/09/99  Implemented the method install.
# rschmutz  20/08/99  some missing directories added.
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginINSROOT - integration root
# 
#   SYNOPSIS
#	pkginINSROOT name
# 
#   DESCRIPTION
#	pkginINSROOT is used to build and install software.
#
#   PARENT
#	ctooOBJ_BASE pkginBASE
#
#   PUBLIC METHODS
#	Create
#	    Creates the INS_ROOT.
#
#	Backup
#	    Makes a backup the INS_ROOT directory (renames it).
#
#	Verify
#	    Checks the files installed in INS_ROOT against the
#	    files located in $INTROOT/config/INS_ROOT.
#
#   PUBLIC DATA
#	dir:	INSROOT directory
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginINSROOT
#------------------------------------------------------------------------

class pkginINSROOT {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {

	set txt ""
	catch { set txt [GetEnv INS_ROOT] }
	if { $txt != "" } {
		# remove trailing '/'
        set txt [eval file join [file split $txt]]
	}
	configure -objType ROOT -name INSROOT -dir $txt
    	eval configure $args

	#   directory                   	mode	secure
	set dirList {
	    SYSTEM/Alias			mgr	F
	    SYSTEM/Dictionary			mgr	F
	    SYSTEM/DETDATA			user	F
	    SYSTEM/ARCDATA			user	F
	    SYSTEM/BADDATA 			user	F
	    SYSTEM/REFDATA 			user	F
	    SYSTEM/COMMON/CONFIGFILES		user	T
	    SYSTEM/COMMON/SETUPFILES/REF	user	T
	    SYSTEM/COMMON/SETUPFILES/DET	user	T
	    SYSTEM/COMMON/SETUPFILES/INS	user	T
	    SYSTEM/COMMON/SETUPFILES/TARG	user	T
	    SYSTEM/COMMON/TEMPLATES/TSF		mgr	F
	    SYSTEM/COMMON/TEMPLATES/SEQ		mgr	F
	    SYSTEM/COMMON/TEMPLATES/OBD		user	T
	    SYSTEM/COMMON/MOBS			user	T
	    SYSTEM/COMMON/MIDAS/PROCS		mgr	F
	    SYSTEM/COMMON/MIDAS/REF		user	T
	    SYSTEM/MISC				user	T
	}
    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create { {force 0} } {

    	#*# 1. if creation forced: backup/delete current INS_ROOT
	if { $force } {
	    Backup
	}

	#*# 2. create/check INS_ROOT
    	if {[file exists $dir]} {
            Info "       INS_ROOT: using $dir: " 0
    	} else {
            Info "       INS_ROOT: creating $dir ... " 0
	}
	try_eval {
	    file mkdir $dir
	    foreach {name mode secure} $dirList {
	    	set path [file join $dir $name]
	    	file mkdir $path
		if { "$mode" == "mgr" } {
		    chmod 0755 $path 
		} else {
		    chmod 0775 $path 
		}
		if { $secure } {
		    chmod +t $path
		}
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Backup
    #----------------------------------------------------------------

    public method Backup {} {

	if { [file exists $dir] } {
	   try_eval {
		if { $maxbak != 0 } {
                    Info "       INS_ROOT: renaming $dir ... " 0
		    BackupFile $dir $maxbak
		} else {
                    Info "       INS_ROOT: deleting $dir ... " 0
		    file delete -force $dir
		}
		Info "OK."
	    } {
		LogErr $errorResult
		Info "FAILURE."
		error $errorResult $errorCode $errorInfo
	    }
	}
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

    	try_eval {
	    #*# check all files in $INTROOT/config/INS_ROOT
            Info "       INS_ROOT: verifying files ...              " 0
	    set errlist ""
	    set cr "\n"
	    set srcdir [file join [GetEnv INTROOT] config INS_ROOT]
	    if { [file exists $srcdir] } {
	        set cp [expr {[clength $srcdir] + 1}]
	        for_recursive_glob srcfile $srcdir "*" {
	    	    set file [crange $srcfile $cp end]
	            set destfile [file join [GetEnv INS_ROOT] $file]
	            if { ![file exists $destfile] } {
    		        Info "$cr       File not found: $file: FAILURE."
		        append errlist "ERROR: INS_ROOT file not found: $file\n"
		        set cr ""
	            }
	        }
	    }
	} {
	    LogErr $errorResult
	    if { $cr != "" } {
	    	Info "FAILURE."
	    }
	    error $errorResult $errorCode $errorInfo
	}
	if { $errlist != "" } {
	    LogErr $errlist
	    error $errlist
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: DiskSpace
    #----------------------------------------------------------------

    public method DiskSpace {} {
    	return [GetDiskSpace [file dirname $dir]]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable dir ""
    public variable maxbak 2
    public variable size 0

    protected variable dirList
}


#************************************************************************
#___oOo___

