#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginINSTALL.tcl,v 1.105 2005/10/12 19:47:30 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-10-04  INSTALL.MODULE.NAME added.
# rschmutz  2004-10-04  hook defaultPath fixed ("src" added).
# rschmutz  2004-09-04  initScriptPath added.
# rschmutz  2004-08-23  support for CfgAppendFromModule added.
# rschmutz  2004-06-03  search the initial script also in the config dir.
# rschmutz  2004-06-02  read TARGET configuration file.
# rschmutz  2004-06-02  use bash instead of csh.
# rschmutz  13/02/03  add a day to diffTime to have a correct output.
# rschmutz  17/01/03  method Install renamed to Unpack (for pkginUnpack).
# rschmutz  17/01/03  method Pack added (for pkginPack).
# rschmutz  16/01/03  accept *.tar and *.tar.gz package files.
# rschmutz  06/01/03  method Test added (for pkginTest).
# rschmutz  05/01/03  added support to set config. keyw. on the cmd. line.
# rschmutz  04/01/03  -cfg file has precedence: re-read it after the rest.
# rschmutz  25/06/01  LogEnd: print logEndInfo contents.
# rschmutz  22/06/01  consider all commands in method Script.
# rschmutz  13/05/01  pkginInstall: accept -fromstep,-step,-tostep options.
# rschmutz  12/05/01  print elapsed time in LogEnd.
# rschmutz  25/12/00  support for -target option added.
# rschmutz  24/12/00  INSTALL.APPENDi.FILE,OPTIONS support added.
# rschmutz  08/12/00  INSTALL.CON.APPENDi support added.
# rschmutz  08/12/00  host platform check added.
# rschmutz  23/11/00  VLTROOT version check added.
# rschmutz  11/10/00  search init. and Tcl script also in INTROOT.
# rschmutz  17/09/00  pkginVerify support added.
# rschmutz  26/06/00  method Versions added (for pkginVersions).
# rschmutz  12/05/00  created.
#

#************************************************************************
#   NAME
#	pkginINSTALL - build/install/verify an application
# 
#   SYNOPSIS
#	pkginINSTALL myScript -config <name> [-loglevel <n>] [-debug]
#	myScript Build
# 
#   DESCRIPTION
#	This class implements the initial step to build and
#	install a VLT package.
#
#	The build/install is performed as follows:
#
#	1) pkginINSTALL creates a repository (ctooOBJ_REPOSITORY)
#	   and opens the config. file.
#	2) The pkginType array is initialized with the class
#	   types needed to build/install a VLT package.
#       3) If a particular package needs some special processing
#	   not considered by the standard classes, then a Tcl
#	   script INSTALL.CON.TCLFILE can be supplied.
#	   This Tcl script should define alternative class types
#	   and register them in the pkginType array.
#	4) A INSTALL object is created and stored in the repository
#          (the INSTALL type is: $pgkinType(INSTALL)
#       5) The Config method of the INSTALL object is called.
#	   This Config method is expected to read the package
#	   configuration and to store it in the repository.
#	6) The Build, Pack, Unpack, Verify, Test resp. Clean method
#	   of the INSTALL object is called.  This method is expected
#	   to perform the corresponding action.
#
#   PUBLIC METHODS
#	Build <args>
#	    Build a VLT package as specified above.
#	    The -config option specifies the config. file to use,
#	    all other options are passed to the INSTALL object
#	    in the repository.
#
#	Pack <args>
#	    Pack a VLT package as specified above.
#
#	Unpack <args>
#	    Install a VLT package as specified above.
#	    The -config option specifies the config. file to use,
#	    all other options are passed to the INSTALL object
#	    in the repository.
#
#	Verify <args>
#	    Verify a VLT package as specified above.
#
#	Test <args>
#	    Test a VLT package, i.e. execute 'make test' on all
#	    modules.
#
#	Clean <args>
#	    Clean a VLT package as specified above.
#	    The -config option specifies the config. file to use,
#	    all other options are passed to the INSTALL object
#	    in the repository.
#
#   FILES
#	<package>INSTALL.cfg - package configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#	      ../config
#
#   SEE ALSO
#	pkginBuild pkginClean
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: pkginINSTALL
#------------------------------------------------------------------------

class pkginINSTALL {
    inherit pkginBASE

    #****************************************************************
    #*# method Build
    #----------------------------------------------------------------

    public method Build { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginBuild"

    	#*# 2. search -debug flag, call Build
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Build $param
	} else {
	    try_eval { Script Build $param } {
	    	LogEnd
		Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Pack
    #
    #   This just calls Build, but logs to pkginPack.log/.err.
    #----------------------------------------------------------------

    public method Pack { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginPack"

    	#*# 2. search -debug flag, call Build
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Build "$param -step PACK"
	} else {
	    try_eval { Script Build "$param -step PACK" } {
	    	LogEnd
		Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Unpack
    #----------------------------------------------------------------

    public method Unpack { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginUnpack"

    	#*# 2. search -debug flag, call Install
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Install $param
	} else {
	    try_eval { Script Install $param } {
	    	LogEnd
	    	Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Verify
    #----------------------------------------------------------------

    public method Verify { param } {

        #*# 1. set log directory and filename
	file mkdir "[GetEnv HOME]/VERIFY"
	configure -logdir "[GetEnv HOME]/VERIFY" -logname "pkginVerify"

    	#*# 2. search -debug flag, call Verify
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Verify $param
	} else {
	    try_eval { Script Verify $param } {
	    	LogEnd
	    	Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Test
    #
    #   This just calls Build, but logs to pkginTest.log/.err.
    #----------------------------------------------------------------

    public method Test { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginTest"

    	#*# 2. search -debug flag, call Build
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Build $param
	} else {
	    try_eval { Script Build $param } {
	    	LogEnd
		Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Clean
    #----------------------------------------------------------------

    public method Clean { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginClean"

    	#*# 2. search -debug flag, call Clean
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Clean $param
	} else {
	    try_eval { Script Clean $param } {
	    	LogEnd
	    	Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Versions
    #----------------------------------------------------------------

    public method Versions { param } {

        #*# 1. set log directory and filename
	configure -logdir "./INSTALL" -logname "pkginVersions"

    	#*# 2. search -debug flag, call Clean
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Versions $param
	} else {
	    try_eval { Script Versions $param } {
	    	LogEnd
	    	Exit "ERROR: $errorResult"
	    }
	}
    }

    #****************************************************************
    #*# method Script
    #
    #  1. init.
    #  2. if specific install module version requested: retrieve it.
    #  3. create repository
    #  4. open config. file
    #  5. check VLTROOT version.
    #  6. delete old logs
    #  7. source env. vars.
    #  8. source init. script
    #  9. load class types
    # 10. source the user extensions
    # 11. create the INSTALL object
    # 12. call INSTALL::Config
    # 13. process debug options -objList, -objShow
    # 14. call INSTALL::<cmd>
    #----------------------------------------------------------------

    protected method Script { cmd param } {
        global pkginType

    	#*# 1. init.
	#
	#   - log start
	#   - get debug level
	#   - set NO_ERR_DISPLAY
	#   - set MAKE_NO_WARNINGS
	LogStart $param
	set i [lsearch $param "-loglevel"]
	if { $i != -1 } {
	    lvarpop param $i
	    configure -loglevel [lvarpop param $i]
	}

	#   In some situations dialog boxes can emerge waiting for 
	#   user interaction, and this is not desired.
	SetEnv NO_ERR_DISPLAY T

	#   Make sure that no warnings are generated.
	#   It seems that with OCT99 all modules will generate warnings.
	SetEnv MAKE_NO_WARNINGS on

    	#*# 2. if specific install module version requested: retrieve it.
	if { $cmd == "Build" } {
	    set i [lsearch $param "-ver"]
	    if { $i != -1 } {
	    	incr i
		set arg1 [lindex $param 0]
	    	set version [lindex $param $i]
    	        pkginMODULE insmod -name $arg1 -version $version
	        try_eval {
	    	    insmod Retrieve
	        } {
		    error $errorResult $errorCode $errorInfo
	        } {
	            delete object insmod
	        }
	    }
	}

    	#*# 3. create repository
	ctooOBJ_REPOSITORY rep -name PKGIN

    	#*# 4. open config. files
	#
	#   If pkginBuild,
	#      pkginClean,
	#      pkginVersions: retrieve config. from integration module
	#   If pkginUnpack:  retrieve config. from tar file
	#                     (if not installed yet), or from
	#		      the current INTROOT.
	#   If pkginVerify:   retrieve config. from INTROOT.
	#
	set moduleName ""
	switch $cmd {
	    "Install" {
	        set arg1 [lindex $param 0]
		set moduleName $arg1
		set appendFiles 0

		# If tar file already installed, them get config. from INTROOT.
		set i [lsearch $param "-fromstep"]
		if { $i == -1 } {
		    set i [lsearch $param "-step"]
		}
		if { $i != -1 } {
		    incr i
		    set tmp [lindex $param $i]
		    if { ![lcontain {SAVE_CONFIG BEFORE_INSTALL STOP_ENV INSTALL_ROOTS} $tmp] } {
	    		set arg1 "[GetEnv INTROOT]/INSTALL/INSTALL.cfg"
		    }
		}

		# if no tar file specified: error
		if { $arg1 == "*" } {
		    puts "\npkginUnpack <path name>"
		    puts "            \[-cfg <cfg. file>\] \[<cfgkeyw>=\"<value>\" ...\]"
		    puts "            \[-fromstep | -step <step name> \[<env. name/det./scan link>\]\]"
		    puts "            \[-tostep <step name>\] \[-env <env. name> ...\]"
		    puts "ERROR: Path <path name> of the package file missing."
		    Exit
		}
	    }
	    "Verify" {
	    	set arg1 "[GetEnv INTROOT]/INSTALL/INSTALL.cfg"
		set appendFiles 0
	    }
	    default {
	        set arg1 [lindex $param 0]
		set moduleName $arg1
		set appendFiles 1
	    }
	}
	Info "INIT : Reading $arg1 configuration."
	set cfg [pkginCONFIG $this.cfg $arg1]
	if { $moduleName != "" } {
	    $cfg Set INSTALL.MODULE.NAME $arg1
	}
	#
	# append -cfg config. file (if any)
	# to load any INSTALL.APPENDi.FILE entries
	#
	set cfgUserFile ""
	set i [lsearch $param "-cfg"]
	if { $i != -1 } {
	    lvarpop param $i
	    set cfgUserFile [lvarpop param $i]
	    Info "       Reading $cfgUserFile."
	    $cfg CfgAppend $cfgUserFile
	}
	#
	# append INSTALL.APPENDi.* config. files
	#
	# The config. file in the tar file and in INTROOT contain
	# already the files specified with INSTALL.APPENDi.*,
	# therefore in these cases it is not necessary (nor possible)
	# to append the INSTALL.APPENDi.* files.
	# The INSTALL.APPENDi.* entries have to be processed only
	# when the config. file from the integration module is loaded,
	# i.e. when pkginBuild, pkginClean or pkginVersions are used.
	# In this case, the target <targetId> config. file is also loaded
	# (if it exists).
	#
	if { $appendFiles } {

	    # load INSTALL.APPENDi.FILEs
	    pkginPLUGIN  dummyPlugin
	    pkginPKGFILE dummyPkgfile
	    loop i 1 21 {
	        if { ![catch { set cfgAppendFile [$cfg Get "INSTALL.APPEND$i.FILE"] }]} {
	            if { ![catch { set cfgAppendModule [$cfg Get "INSTALL.APPEND$i.MODULE"] }]} {
		    	# load config. file from specified module
			set cfgAppendVersion "LAST"
	            	catch { set cfgAppendVersion [$cfg Get "INSTALL.APPEND$i.VERSION"] }
		    	$cfg CfgAppendFromModule $cfgAppendFile $cfgAppendModule $cfgAppendVersion

			# add module to plugin search path
			set pluginPath [dummyPlugin cget -defaultPath]
			lappend pluginPath [file join [pwd] $cfgAppendModule "src"]
			dummyPlugin configure -defaultPath $pluginPath

			# add module to initScript search path
			lappend initScriptPath [file join [pwd] $cfgAppendModule "src"] \
					       [file join [pwd] $cfgAppendModule "config"]
			dummyPkgfile configure -initScriptPath $initScriptPath
		    } else {
	    	        $cfg CfgAppend $cfgAppendFile
		    }
	        }
	    }

	    # load TARGET config. file
	    set targetList ""
	    catch { set targetList [$cfg Get "INSTALL.PACKAGE.TARGETS"] }
	    if { $targetList != "" } {
	    	set targetId ""
	    	catch { set targetId [GetEnv TARGET] }
	    	set i [lsearch $param "-target"]
	    	if { $i != -1 } {
	            lvarpop param $i
	    	    set targetId [lvarpop param $i]
	    	}
	    	if { $targetId == "" } {
		    set targetId [lindex $targetList 0]
	    	}
		CheckOptions "TARGET" $targetId $targetList
	        Info "       TARGET= $targetId"
	    	SetEnv TARGET $targetId
	    	set cfgAppendFile [cconcat $arg1 "TARGET_" $targetId ".cfg"]
	    	$cfg CfgAppend $cfgAppendFile 1
	    } else {
	    	set i [lsearch $param "-target"]
		if { $i != -1 } {
		    error "No targets known: option -target not allowed."
		}
	    }
	}
	#
	# re-read $cfgUserFile to make sure that its contents has precedence
	#
	if { $cfgUserFile != "" } {
	    $cfg CfgAppend $cfgUserFile
	}	
	rep AddObj $cfg

	#
	# load command-line config. keywords
	#
	set newParam ""
	foreach par $param {
	    if { [string first "=" $par] > 0 } {
		if { [regexp {([^=]+)=([^=]+)} $par dummy key val] == 0 } {
		    error "Wrong parameter $par."
		}
		try_eval {
		    $cfg Set $key $val
		} {
		    error "Unknown config. parameter '$key' (check command line)."
		}
	    } else {
	    	lappend newParam $par
	    }
	}
	set param $newParam

        #*# 5. check platform and VLTROOT version.
	set txt1 ""
	catch { set txt1 [$cfg Get INSTALL.CON.PLATFORM] }
	set txt2 [$cfg Platform "-s -r"]
	if { $txt1 != "" && $txt1 != $txt2 } {
	    error "Wrong host platform: $txt1 != $txt2."
	}
	$cfg Set INSTALL.CON.PLATFORM $txt2

	set txt1 ""
	catch { set txt1 [$cfg Get INSTALL.CON.VLTROOT] }
	set txt2 [GetEnv VLTROOT]
	if { $txt1 != "" && $txt1 != $txt2 } {
	    error "Wrong VLTROOT version: $txt1 != $txt2."
	}
	$cfg Set INSTALL.CON.VLTROOT $txt2

	#*# 6. delete old logs
	set maxbaks 2
	catch { set maxbaks [$cfg Get INSTALL.CON.MAXBAK] }
	if { $maxbaks < 2 } {
	    set maxbaks 2
	}
	DeleteBackup [file join $logdir [cconcat $logname ".log"]] $maxbaks
	DeleteBackup [file join $logdir [cconcat $logname ".err"]] $maxbaks

	#*# 7. source env. vars.
	##SourceEnvVars

	#*# 8. source init. script
	SourceInitScript $cfg $arg1

        #*# 9. load class types
	#
	#      The pkginINSTALLATION::ItclTypes procedure loads
	#      the pkginType array with the class types
	#      that can be overloaded by the user extensions.
	#
	#      This array is used to create other classes,
	#      E.g.: to create a MODULE object, call:
	#        $pkginType(MODULE) module
	#
	pkginINSTALLATION::ItclTypes

    	#*# 10. source the user extensions
	#
	#      The user tcl file should usually update the pkginType
	#      array, according to the available user types.
	#
	#      E.g.: setting pkginType(MODULE) = myMODULE
	#            will create an object myMODULE to build/install
	#	     a software module, instead of the default 
	#	     pkginMODULE.  Of course, the user tcl file should
	#	     supply this myMODULE class.
	#
	SourceTclFile $cfg $arg1

    	#*# 11. create the INSTALL object
	if {[catch { set install [$pkginType(INSTALL) $this.install] }]} {
	    error "Wrong package type class $pkginType(INSTALL)."
	}
	rep AddObj $install
	$install configure -debug $debug

    	#*# 12. call INSTALL::Config<cmd>
	$install Config$cmd [rep Contents CONFIG] $param

	#*# 13. process debug options -objList, -objShow
	rep Debug $param 

    	#*# 14. call INSTALL::<cmd>
	$install $cmd

	#*# 15. log end
	LogEnd
    }

    #****************************************************************
    #*# method: LogStart
    #----------------------------------------------------------------

    protected method LogStart { param } {

    	set rtapRoot ""
    	set vltRoot  ""
    	set vxRoot   ""
    	set intRoot  ""
    	set insRoot  ""
    	catch { set rtapRoot [GetEnv RTAPROOT] }
    	catch { set vltRoot  [GetEnv VLTROOT]  }
    	catch { set vxRoot   [GetEnv VXROOT]   }
    	catch { set intRoot  [GetEnv INTROOT]  }
    	catch { set insRoot  [GetEnv INS_ROOT] }
	set startTime [clock seconds]
    	Info "$logSep2"
	Info "$logname started at [clock format $startTime]"
	if { "$logname" == "pkginBuild"   || \
	     "$logname" == "pkginPack" || \
	     "$logname" == "pkginUnpack" || \
	     "$logname" == "pkginTest" } {
    	    Info "Arguments = '$param'"
    	    Info "Host      = '[exec uname -a]'"
	    Info "PWD       = '[pwd]'"
	    if { $rtapRoot != "" } {
    	        Info "RTAPROOT  = '$rtapRoot'"
	    }
	    if { $vxRoot != "" } {
    	    	Info "VXROOT    = '$vxRoot'"
	    }
    	    Info "VLTROOT   = '$vltRoot'"
    	    Info "INTROOT   = '$intRoot'"
    	    Info "INS_ROOT  = '$insRoot'"
    	    Info "$logSep1"
    	    Info "For error and status messages please refer to the files:"
    	    Info "[pwd]/INSTALL/$logname.log"
    	    Info "[pwd]/INSTALL/$logname.err"
	} elseif { "$logname" == "pkginVerify" } {
    	    Info "INTROOT   = $intRoot"
    	    Info "INS_ROOT  = $insRoot"
    	    Info "$logSep1"
    	    Info "For error and status messages please refer to the files:"
    	    Info "~/VERIFY/$logname.log"
    	    Info "~/VERIFY/$logname.err"
	} else {
	    Info "PWD       = [pwd]"
	}
    	Info "$logSep2"
    }

    #****************************************************************
    #*# method: LogEnd
    #----------------------------------------------------------------

    protected method LogEnd {} {
	catch { Info "$logSep1" }
	set showHostWarning 0
	set showPlatformWarning 0
	catch { set showHostWarning [[rep Contents CONFIG] cget -hostUsed] }
	catch { set showPlatformWarning [[rep Contents CONFIG] cget -platformUsed] }
	if { $showHostWarning } {
	    catch { Info "WARNING: Env. var. \$HOST used:   The package can't be built off-line." }
	}
	if { $showPlatformWarning } {
	    catch { Info "WARNING: Function PLATFORM used: The package is not platform independent." }
	}
	if { $showHostWarning || $showPlatformWarning } {
	    catch { Info "$logSep1" }
	}
	set stopTime [clock seconds]
	set diffTime 0
	catch { set diffTime [expr {$stopTime - $startTime}] }
	incr diffTime [clock scan "01/01/2000"]
	catch { Info "$logname ended at [clock format [clock seconds]] ([clock format $diffTime -format {%H:%M:%S}])" }
	catch { Info "$logSep2" }
	if { $logEndInfo != "" } {
	    puts ""
	    puts "$logEndInfo"
	}
    }

    #****************************************************************
    #*# method: SourceInitScript
    #----------------------------------------------------------------

    protected method SourceInitScript { cfg arg1 } {

	set initScript ""
	catch { set initScript [$cfg Get INSTALL.INITIAL.SCRIPT] }
	if { "$initScript" != "" } {

	    #*# 1. if tar file: extract init. script
	    set isTarFile 0
	    if { [crange $arg1 end-6 end] == ".tar.gz" || \
	    	 [crange $arg1 end-3 end] == ".tar" } {
		set isTarFile 1
	    	set initScriptArg $initScript
		ctoken initScriptArg " \t"
		set initScript1 "./INSTALL/INSTALL.script"
		set initScript "$initScript1 $initScriptArg"
	    	exec pkginUnziptar $arg1 $initScript1
		if {![file isfile $initScript1]} {
		    error "Error extracting init. script from $arg1 (file $initScript1 not found)."
		}
	    } else {
	    	set initScript [SearchFile $initScript "$arg1/src $arg1/config $initScriptPath \$INTROOT/bin \$VLTROOT/bin"]
	    }

	    #*# 2. source init. script
	    #
            #      Not only must the script be sourced but defined 
            #      environment variables in the script must also be
	    #      defined in this script's array of environment 
	    #      variables. 
	    #
            Info "Sourcing the initial script '$initScript' ..."
            set cmd "exec bash -c \". $initScript; echo `env`\""
            catch {eval $cmd} result
            foreach envVar $result {
	        SetEnv [ctoken envVar =] [ctoken envVar =]
            }
            Info ""

	    #*# 3. if tar file: delete init. script
	    if { $isTarFile } {
		file delete $initScript1
	    }
	}
    }

    #****************************************************************
    #*# method: SourceTclFile
    #----------------------------------------------------------------

    protected method SourceTclFile { cfg arg1 } {

	set userTclFile ""
	catch { set userTclFile [$cfg Get INSTALL.CON.TCLFILE] }
	if { "$userTclFile" != "" } {

	    #*# 1. if tar.gz file: extract TCL file
	    set isTarFile 0
	    if { [crange $arg1 end-6 end] == ".tar.gz" || \
	    	 [crange $arg1 end-3 end] == ".tar" } {
		set isTarFile 1
	    	set userTclFile "./INSTALL/INSTALL.tcl"
	    	exec pkginUnziptar $arg1 $userTclFile
		if {![file isfile $userTclFile]} {
		    error "Error extracting TCL script from $arg1 (file $userTclFile not found)."
		}
	    } else {
	        try_eval {
	    	    set userTclFile [SearchFile $userTclFile "$arg1/src \$INTROOT/bin \$VLTROOT/bin"]
		} {
	    	    set userTclFile [SearchFile [cconcat $userTclFile ".tcl"] "$arg1/src \$INTROOT/bin \$VLTROOT/bin"]
		}
	    }

	    #*# 2. source TCL file
	    if { $debug } {
	        uplevel #0 source $userTclFile
	    } else {
	    	try_eval {
	            uplevel #0 source $userTclFile
		} {
		    set txt "ERROR loading $userTclFile: $errorResult."
		    error $errorResult $errorCode $errorInfo
		}
	    }

	    #*# 3. if tar.gz file: delete TCL file
	    if { $isTarFile } {
		file delete $userTclFile
	    }
	}
    }

    #****************************************************************
    # variables
    #****************************************************************

    protected variable initScriptPath ""
    protected variable startTime ""
    protected variable debug 0
}


#___oOo___
