#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginINSTALLATION.tcl,v 1.105 2005/10/12 19:47:32 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-09-12  SPR VLTSW20050295: run ctooConfigInstall.
# rschmutz  2004-12-19  SPR VLTSW20040405: IsOfflineBuild relaxed.
# rschmutz  2004-10-06  hook BEFORE_BUILD_ENV added.
# rschmutz  2004-10-06  option -introot added for pkginVersions.
# alongino  2004-08-30  fixed bug in buildOptionsStop: hook_AFTER_STOP_ENV
# alongino  2004-08-25  hook BEFORE_CLEAN added. Skip _CLEAN plugins if not step CLEAN
# rschmutz  2004-08-24  step CLEAN added.
# rschmutz  2004-08-23  retrieving integration module not supported.
# rschmutz  2004-08-23  hook AFTER_STOP_ENV added.
# rschmutz  2004-06-02  place TARGET env. var. in pkgin-apps-all.env.
# rschmutz  2003-11-23  SPR VLTSW20030382: additional check in IsOfflineBuild.
# rschmutz  27/03/03  INSTALL.CON.BACKUP support added.
# rschmutz  19/01/03  support added to boot LCUs in parallel.
# rschmutz  17/01/03  BuildModules: set WIND_BASE, MIDASHOME, MIDVERS.
# rschmutz  17/01/03  CheckAuthorization: allow users to run START_LCUENV,
#			CREATE_SCAN steps.
# rschmutz  17/01/03  INSTALL.UNPACK.KEEPROOT support added.
# rschmutz  17/01/03  last default install step: AFTER_INSTALL.
# rschmutz  07/01/03  -compress option deleted, COMPRESS -> PACK.
# rschmutz  04/01/03  SPR VLTSW20010789: 10 sec. delay added at the end
#		      of CreateScanLinks.
# rschmutz  30/10/01  -envonly option support added.
# rschmutz  29/10/01  Off-line build support added.
# rschmutz  29/10/01  if compress and BUILD_MOD: do not re-create INTROOT.
# rschmutz  14/10/01  CheckAuthorization: do not check rsh to same user.
# rschmutz  27/09/01  back-compatiblity: VLTSW_CCSTYPE may not exist.
# rschmutz  24/07/01  bug fix: start compress with step RETRIEVE.
# rschmutz  22/07/01  verify file sets added.
# rschmutz  26/06/01  step INSTALL_FILES added, pkginFILESET type added.
# rschmutz  25/06/01  steps SAVE_CONFIG, AFTER_INSTALL added.
# rschmutz  24/06/01  create pkgin-apps-all.env.
# rschmutz  24/06/01  SourceEnvVars added.
# rschmutz  14/05/01  use rsh/remsh according to operating system.
# rschmutz  14/05/01  ConfigInstall: check disk space: register root sizes.
# rschmutz  14/05/01  check that rsh works ok.
# rschmutz  14/05/01  check root area disk space.
# rschmutz  14/05/01  CheckEnvVars added.
# rschmutz  13/05/01  END_INSTALL step added.
# rschmutz  13/05/01  pkginInstall: accept -fromstep,-step,-tostep options.
# rschmutz  12/05/01  pkginInstall: always stop/backup all WS environments.
# rschmutz  23/11/00  oslx sets INTROOT = "NOT-USED" when not defined.
# rschmutz  17/09/00  pkginVerify support added.
# rschmutz  16/09/00  if -step BUILD_ENV <env>: only STOP_ENV <env>.
# rschmutz  02/09/00  set config. file permissions (ctooConfigInstall).
# rschmutz  30/08/00  hook BEFORE_BUILD_MOD added.
# rschmutz  07/08/00  ConfigPackage: config. integration module.
# rschmutz  09/07/00  man page updated.
# rschmutz  26/06/00  methods ConfigVersions and Versions added.
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  13/05/00  created.
#

#************************************************************************
#   NAME
#	pkginINSTALLATION - build/install VLT software
#
#   SYNOPSIS
#	pkginINSTALLATION name
#
#   DESCRIPTION
#   	This class contains the main procedures for the pkgin utilities.
#
#   	Execution is split into 2 main phases:
#   	1) The config. file and command line arguments are processed
#   	   to create an internal tree of objects (that can be seen
#   	   with command line option -objList).
#	2) The command line options are processed, to define
#	   what steps to execute.
#   	3) The build or installation steps are executed.
#
#	pkginBuild calls:
#	    ConfigBuild
#	    Build
#
#	pkginUnpack calls:
#	    ConfigInstall
#	    Install
#
#	pkginVerify calls:
#	    ConfigVerify
#	    Verify
#
#	pkginClean calls:
#	    ConfigClean
#	    Clean
#
#   	Methods Config* and LoadSteps implement the config. phase.
#   	Method Config is called by Config<name> to build an internal
#   	representation of the instrument based in the config. file.
#   	Method LoadSteps finds out the build or installation steps
#   	to be executed, according to the command line arguments.
#
#   	Methods Build, Install, Verify and Clean execute the
#	individual build/installation steps.
#
#   	The individual build/installation steps are implemented
#   	with methods:
#   	RetrieveModules, BeforeInstall, StopEnvs, CreateRoots,
#   	BuildModules, BuildEnvs, InstallRoots,
#   	StartWsEnvs, StartLcuEnvs, CreateScanLinks
#
#   	Usually each step just has to execute a certain method
#   	of a certain set of objects (e.g. the StopEnvs step has
#   	to call the Stop method of all WS environments).
#   	This is done with the helper methods:
#   	StepBegin, StepExec, StepEnd (and ExitIfErrors, if necessary).
#
#   	The local variables listed at the end of the class, list
#   	the known steps, their execution order and specify if the
#   	steps accept additional arguments.  See their individual
#   	descriptions for details.
#
#   	If this class is overloaded, then normally the methods
#   	implementing the indivual build/installation steps are
#   	overloaded and/or the local variables are updated with
#   	additional steps.
#
#   PARENT CLASS
#       pkginINSTALL_BASE
#
#   PUBLIC METHODS
#	ConfigBuild <cfg> <param>
#	ConfigInstall <cfg> <param>
#	ConfigVerify <cfg> <param>
#	ConfigClean <cfg> <param>
#	ConfigVersions <cfg> <param>
#	    Reads the package configuration and command line
#	    parameters for commands pkginBuild, pkginUnpack,
#	    pkginVerify, pkginClean and pkginVersions.
#
#	Build
#	Install
#	Verify
#	Clean
#	Versions
#	    Build, install or clean a VLT package
#	    This methods are called after ConfigBuild,
#	    ConfigInstall, ConfigVerify, ConfigClean,
#	    resp. ConfigVersions.
#
#    	RetrieveModules [<modules>]
#	BeforeInstall
#	StopEnvs [<envs>]
#	CleanDirs
#	CreateRoots
#	BuildModules [<modules>]
#	BuildEnvs [<envs>]
#	InstallRoots
#	StartWsEnvs [<envs>]
#	InstallCcds [<ccds>]
#	StartLcuEnvs [<envs>]
#	CreateScanLinks [<scanlinks>]
#	VerifyRoots
#	VerifyEnvs [<envs>]
#	VerifyScanLinks [<scanlinks>]
#	    Build/install/Verify steps called by Build, Install
#	    and Verify.
#
#   PUBLIC PROCEDURES
#	ItclTypes
#	    Registers the Itcl types.
#
#   SEE ALSO
#	pkginINSTALL_BASE
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: pkginINSTALLATION
#------------------------------------------------------------------------

class pkginINSTALLATION {
    inherit pkginINSTALL_BASE

    #****************************************************************
    #*# proc ItclTypes
    #----------------------------------------------------------------

    proc ItclTypes {} {
	global pkginType

	#*# 1. init. the Itcl types
	set pkginType(INSTALL)	pkginINSTALLATION
	set pkginType(PACKAGE)	pkginPACKAGE
	set pkginType(MODULE)	pkginMODULE
	set pkginType(LIBRARY)	pkginLIBRARY
	set pkginType(PLUGIN)	pkginPLUGIN
	set pkginType(RTAPENV)	pkginRTAPENV
	set pkginType(LCUENV)	pkginLCUENV
	set pkginType(DCS)	pkginDCS
	set pkginType(SCANLINK)	pkginSCANLINK
	set pkginType(FILESET)  pkginFILESET
	set pkginType(INSROOT)	pkginINSROOT
	set pkginType(INTROOT)	pkginINTROOT
	set pkginType(VLTDATA)	pkginVLTDATA
	set pkginType(STEP)	pkginSTEP
    }

    #****************************************************************
    #*# method ConfigBuild
    #----------------------------------------------------------------

    public method ConfigBuild { cfg params } {
	global pkginType

	#*# 1. config. first
	ConfigFirst $cfg

	#*# 2. get command line options
	configure -module [lvarpop params]
	GetOptions {-ver <version> -env <envlist> \
		    -fromstep <step> -step <step> -tostep <step> \
		    -envonly -clean} \
		   $params
	if { "$fromstep" != "" && "$step" != "" } {
	    error "Options -fromstep and -step can't be used simultaneously."
	}
	if { "$tostep" != "" && "$step" != "" } {
	    error "Options -step and -tostep can't be used simultaneously."
	}

	#*# 3. load INTROOT, INS_ROOT, VLTDATA
	#      (default types: pkginINTROOT, pkginINSROOT, pkginVLTDATA)
	set container [ctooOBJ_CONTAINER $this.roots -objType ROOTS -name VLT]
	AddObj $container
	set obj [$pkginType(INTROOT) $this.introot -maxbak $maxbak]
	$container AddObj $obj
	$obj SetEnvPaths
	set obj [$pkginType(INSROOT) $this.insroot -maxbak $maxbak]
	$container AddObj $obj
	set obj [$pkginType(VLTDATA) $this.vltdata]
	$container AddObj $obj

	#*# 4. load package config.
	ConfigPackage $cfg

	#*# 5. load build steps
	#      (process -fromstep, -step, -tostep).
	ConfigBuildSteps "Build"

	#*# 6. load tar file
	set pkgname [[Contents PACKAGE] Name]
	set obj [pkginPKGFILE $this.pkgfile -name $pkgname \
			      -module $module -maxbak $maxbak]
	AddObj $obj

	#*# 7. get backup flag
	set txt "F"
	if { $clean } {
	    set txt "F"
	} elseif { $envonly == 0 } {
	    catch { set txt [$cfg Get INSTALL.CON.BACKUP] }
	}
	configure -backup [Logical2Num $txt]
    }

    #****************************************************************
    #*# method ConfigInstall
    #----------------------------------------------------------------

    public method ConfigInstall { cfg params } {
	global pkginType

	#*# 1. config. first
	ConfigFirst $cfg

	#*# 2. get command line options
	set txt [lvarpop params]
	if { $txt == "*" } {
	    set txt ""
	}
	configure -tarfile $txt
	GetOptions {-env <envlist> -fromstep <step> -step <step> \
		    -tostep <step>} $params
	if { "$fromstep" != "" && "$step" != "" } {
	    error "Options -fromstep and -step can't be used simultaneously."
	}
	if { "$tostep" != "" && "$step" != "" } {
	    error "Options -step and -tostep can't be used simultaneously."
	}

	#*# 3. load INTROOT, INS_ROOT
	#      (default types: pkginINTROOT, pkginINSROOT)
	#      oslx sets INTROOT = "NOT-USED" when it is not defined ...
	set package [$cfg Get INSTALL.PACKAGE.NAME]
	if { ![ExistsEnv INTROOT] || [GetEnv INTROOT] == "NOT-USED" } {
	    SetEnv INTROOT "/vlt/$package/INTROOT"
	}
	if { ![ExistsEnv INS_ROOT] || [GetEnv INS_ROOT] == "NOT-USED" } {
	    SetEnv INS_ROOT "/data/$package/INS_ROOT"
	}
	set container [ctooOBJ_CONTAINER $this.roots \
		      -objType ROOTS -name VLT]
	AddObj $container
	set minsize 0
	catch { set minsize [$cfg Get INSTALL.INTROOT.SIZE] }
	set obj [$pkginType(INTROOT) $this.introot -maxbak $maxbak -size $minsize]
	$container AddObj $obj
	$obj SetEnvPaths
	set minsize 0
	catch { set minsize [$cfg Get INSTALL.INSROOT.SIZE] }
	set obj [$pkginType(INSROOT) $this.insroot -maxbak $maxbak -size $minsize]
	$container AddObj $obj
	set minsize 0
	catch { set minsize [$cfg Get INSTALL.VLTDATA.SIZE] }
	set obj [$pkginType(VLTDATA) $this.vltdata -size $minsize]
	$container AddObj $obj

	#*# 4. load package config.
	ConfigPackage $cfg

	#*# 5. load install steps
	#      (process -fromstep, -step, -tostep).
	ConfigBuildSteps "Install"

	#*# 6. load tar file
	if { $tarfile != "" } {
	    set tarsize 0
	    catch { set tarsize [$cfg Get INSTALL.INTROOT.SIZE] }
	    set obj [pkginPKGFILE $this.pkgfile -name $tarfile -size $tarsize]
	    AddObj $obj
	}

	#*# 7. get keeproots
	set txt F
	catch { set txt [$cfg Get INSTALL.UNPACK.KEEPROOT] }
	configure -keeproots [Logical2Num $txt]
    }

    #****************************************************************
    #*# method ConfigVerify
    #----------------------------------------------------------------

    public method ConfigVerify { cfg params } {
	global pkginType

	#*# 1. config. first
	ConfigFirst $cfg

	#*# 2. get command line options
	GetOptions {-build -env <modules>} $params
	set onlyEnvs 0
	if { $env != "" } {
	    set onlyEnvs 1
	    if { $env == "ALL" } {
		configure -env ""
	    }
	}

	#*# 3. load INTROOT, INS_ROOT, VLTDATA
	#      (default types: pkginINTROOT, pkginINSROOT, pkginVLTDATA)
	set container [ctooOBJ_CONTAINER $this.roots -objType ROOTS -name VLT]
	AddObj $container
	set obj [$pkginType(INTROOT) $this.introot -maxbak $maxbak]
	$container AddObj $obj
	$obj SetEnvPaths
	set obj [$pkginType(INSROOT) $this.insroot -maxbak $maxbak]
	$container AddObj $obj
	set obj [$pkginType(VLTDATA) $this.vltdata]
	$container AddObj $obj

	#*# 4. load package config.
	ConfigPackage $cfg

	#*# 5. load verify steps, check environments
	set step1 0
	set step2 [expr {[llength $verifyOrder] - 1}]
	set optStep ""
	set optArg ""
	if { $build != 0 && $onlyEnvs == 0 } {
	    set step2 [lsearch $verifyOrder "hook_AFTER_VERIFY_ROOTS"]
	}
	if { $build == 0 && $onlyEnvs != 0 } {
	    set step1 [lsearch $verifyOrder "step_VERIFY_ENV"]
	}
	if { $env != "" } {
	    set optStep "step_VERIFY_ENV"
	    set package [Contents PACKAGE]
	    set container [$package Contents ENVIRONMENTS]
	    set optObjList [$container Contents]
	    set optNameList [$container ContentsByName]
	    foreach envName $env {
		set i [lsearch $optNameList $envName]
		if { $i == -1 } {
		    error "Wrong -env: unknown argument $envName."
		}
		lappend optArg [lindex $optObjList $i]
	    }
	    set optObjList ""
	    set optNameList ""
	    Debug "OptStep='$optStep'"
	    Debug "OptArg ='$optArg'"
	}
	LoadSteps $verifyOrder $step1 $step2 $optStep $optArg
    }

    #****************************************************************
    #*# method ConfigClean
    #----------------------------------------------------------------

    public method ConfigClean { cfg params } {

	#*# 1. config. first
	ConfigFirst $cfg

	#*# 2. get command line options
	configure -module [lvarpop params]
	GetOptions {-mod <modules>} $params

	#*# 3. load package config.
	ConfigPackage $cfg
    }

    #****************************************************************
    #*# method ConfigVersions
    #----------------------------------------------------------------

    public method ConfigVersions { cfg params } {

	#*# 1. config. first
	ConfigFirst $cfg

	#*# 2. get command line options
	configure -module [lvarpop params]
	GetOptions {-introot -update -mod <modules>} $params

	#*# 3. load package config.
	ConfigPackage $cfg
    }

    #****************************************************************
    #*# method ConfigFirst
    #----------------------------------------------------------------

    protected method ConfigFirst { cfg } {

	#*# 1. set loglevel
	set txt [cget -loglevel]
	if { $txt == 0 } {
	    catch { set txt [$cfg Get INSTALL.CON.LOGLEVEL] }
	    configure -loglevel $txt
	}

	#*# 2. maxbak
	set txt F
	catch { set txt [$cfg Get INSTALL.FLAG.DELETEDIR] }
	set txt [expr {2 * (1 - [Logical2Num $txt])}]
	catch { set txt [$cfg Get INSTALL.CON.MAXBAK] }
	configure -maxbak $txt
    }

    #****************************************************************
    #*# method ConfigPackage
    #----------------------------------------------------------------

    protected method ConfigPackage { cfg } {
	global pkginType

	#*# 1. load the package configuration
	#      (default type: pkginPACKAGE)
	set package [$pkginType(PACKAGE) $this.package]
	AddObj $package
	$package Config $cfg

	#*# 2. if install module not present in the package, add it
	#      otherwise set install module version, if supplied.
	if { "$module" != "" } {
	    set container [$package Contents MODULES]
	    set obj [$container LookupName $module]
	    if { "$obj" == "" } {
		if { "$mod" == "" || [lsearch $mod $module] != -1 } {
		    set obj [$pkginType(MODULE) $this.insmod \
			     -name $module -version $ver -maxbak $maxbak]
		    $container AddObj $obj
		    $obj Config $cfg
		}
	    } elseif { "$ver" != "" } {
		$obj configure -version $ver
	    }
	}
    }

    #****************************************************************
    #*# method ConfigBuildSteps
    #
    #   Processes values fromstep, step and tostep.
    #----------------------------------------------------------------

    protected method ConfigBuildSteps { cmd } {

	#*# 1. load build/install steps
	#
	# process -fromstep, -step, -tostep.
	set offline [IsOfflineBuild]
	if { $offline } {
	    if { $cmd == "Install" } {
		Info "WARN : This is not the target (WS environments are not local)."
	    } else {
		Info "WARN : Making an OFF-LINE build (WS environments are not local)."
	    }
	}
	# allow 'pkginBuild -step PACK': set off-line = 1
	if { $cmd == "Build" && $step == "PACK" } {
	    set offline 1
	}
	if { $cmd == "Install" } {
	    if { $offline } {
		set step1 [lsearch $buildOptions $firstInstallOfflineStep]
		set step2 [lsearch $buildOptions $lastInstallOfflineStep]
	    } else {
		set step1 [lsearch $buildOptions $firstInstallStep]
		set step2 [lsearch $buildOptions $lastInstallStep]
	    }
	} else {
	    if { $offline } {
		set step1 [lsearch $buildOptions $firstOfflineStep]
		set step2 [lsearch $buildOptions $lastOfflineStep]
	    } else {
		set step1 0
		set step2 [lsearch $buildOptions $lastBuildStep]
	    }
	}
	set optStepName ""
	set optStep ""
	set optType ""
	set optArg ""

	#*# 2. process -fromstep
	#
	#      -fromstep <step> [<optArg>]
	#      step = "<step> <optArg>"
	#      Extract the step name from $fromstep and check that
	#      it is a known step name (against table buildOptions).
	#      If <optArg> is present, then check:
	#      - that the step accepts an argument
	#        (against table buildOptionsObj).
	#      - that only _one_ argument is present
	#
	if { "$fromstep" != "" } {
	    set fromstep1 [lindex $fromstep 0]
	    set step1 [lsearch $buildOptions $fromstep1]
	    if { $step1 == -1 } {
		error "Wrong -fromstep: $fromstep."
	    }
	    if { "$fromstep1" != "$fromstep" } {
		set optType [lindex $buildOptionsObj $step1]
		if { "$optType" == "" } {
		    error "Wrong -fromstep (only step name accepted): $fromstep."
		}
		if { [llength $fromstep] != 2 } {
		    error "Wrong -fromstep (only one arg. after step name accepted): $fromstep."
		}
		set optStepName $fromstep1
		set optStep $step1
		set optArg [lindex $fromstep 1]
	    }
	    if { $fromstep1 == "CLEAN" } {
		set clean 1
	    }
	}

	#*# 3. process -step
	#
	#      similar as -fromstep above
	#
	if { "$step" != "" } {
	    set stepw1 [lindex $step 0]
	    set step1 [lsearch $buildOptions $stepw1]
	    if { $step1 == -1 } {
		error "Wrong -step: $step."
	    }
	    if { "$stepw1" != "$step" } {
		set optType [lindex $buildOptionsObj $step1]
		if { "$optType" == "" } {
		    error "Wrong -step (only step name accepted): $step."
		}
		if { [llength $step] != 2 } {
		    error "Wrong -step (only one arg. after step name accepted): $step."
		}
		set optStepName $stepw1
		set optStep $step1
		set optArg [lindex $step 1]
	    }
	    if { $stepw1 == "CLEAN" } {
		set clean 1
	    }
	    set step2 $step1
	}

	#*# 4. process -tostep
	#
	#      as -fromstep, -step above, but no <optArg> is accepted.
	#
	if { "$tostep" != "" } {
	    set tostep1 [lindex $tostep 0]
	    set step2 [lsearch $buildOptions $tostep1]
	    if { $step2 == -1 } {
		error "Wrong -tostep: $tostep."
	    }
	    if { "$tostep1" != "$tostep" } {
		error "Wrong -tostep (only step name accepted): $tostep."
	    }
	    if { $tostep == "CLEAN" } {
		set clean 1
	    }
	}

	#*# 5. process <optArg>
	#
	#      <optArg> specifies from what module/env. the <optStep>
	#      has to start processing.  Instead of passing just the
	#      first module/env. to the <optStep> method, the complete
	#      list of modules/envs. that should be processed by the
	#      <optStep> step is passed to the <optStep> method.
	#      This is done as follows:
	#
	#      1. Table buildOptionsObj specifies if the <optArg> value
	#         is a module or environment (<optType>).  According to
	#         this, load the list of all available modules/envs.
	#	      objects and all modules/envs. names.
	#      2. Check that <optArg> is a valid value.
	#      3. Return a list of all modules/env. objects present
	#         after the <optArg> value (including the <optArg>
	#         object itself), e.g. if the module list is:
	#         'xxidev xxilcu xxipan xxo xxopan xxins' and <optArg>
	#         is 'xxo', then return the list of objects that
	#         represent modules 'xxo xxopan xxins'.
	#         This list is passed to the <optStep> method.
	#
	if { "$optArg" != "" } {
	    # 5.1 retrieve modules/envs.
	    set package [Contents PACKAGE]
	    set container [$package Contents $optType]
	    if { "$optStepName" == "START_WSENV" } {
		set optType "RTAPENV"
	    } elseif { "$optStepName" == "START_LCUENV" } {
		set optType "LCUENV"
	    } else {
		set optType ""
	    }
	    set optObjList [$container Contents $optType]
	    set optNameList [$container ContentsByName $optType]

	    # 5.2  check that <optArg> is a valid value.
	    set optIndex [lsearch $optNameList $optArg]
	    if { $optIndex == -1 } {
		error "Wrong -fromstep or -step: unknown argument $optArg. Should be one of: $optNameList."
	    }

	    # 5.3 return modules/envs. list
	    set optStep [lindex $buildOptionsObjStep $optStep]
	    if { "$step" != "" } {
		set optArg [lrange $optObjList $optIndex $optIndex]
	    } else {
		set optArg [lrange $optObjList $optIndex end]
	    }
	    set optObjList ""
	    set optNameList ""
	    Debug "OptStep='$optStep'"
	    Debug "OptArg ='$optArg'"
	}

	#*# 6. load all steps according to tables
	#      buildOrder, offlineOrder, installOrder,
	#      buildOptionsStart and buildOptionsStop.
	if { $cmd == "Install" } {
	    set step1 [lsearch $installOrder [lindex $buildOptionsStart $step1]]
	    set step2 [lsearch $installOrder [lindex $buildOptionsStop  $step2]]
	    if { $step1 == -1 || $step2 == -1 } {
		error "Wrong step specified in -fromstep, -step or -tostep."
	    }
	    LoadSteps $installOrder $step1 $step2 $optStep $optArg

	} elseif { $offline == 0 } {
	    set step1 [lsearch $buildOrder [lindex $buildOptionsStart $step1]]
	    set step2 [lsearch $buildOrder [lindex $buildOptionsStop  $step2]]
	    if { $step1 == -1 || $step2 == -1 } {
		error "Wrong step specified in -fromstep, -step or -tostep."
	    }
	    LoadSteps $buildOrder $step1 $step2 $optStep $optArg

	} else {
	    set step1 [lsearch $offlineOrder [lindex $buildOptionsStart $step1]]
	    set step2 [lsearch $offlineOrder [lindex $buildOptionsStop  $step2]]
	    if { $step1 == -1 || $step2 == -1 } {
		error "Wrong step specified in -fromstep, -step or -tostep."
	    }
	    LoadSteps $offlineOrder $step1 $step2 $optStep $optArg

	}
    }

    #****************************************************************
    #*# method LoadSteps
    #
    #   Create a STEPS container and add all steps and hooks
    #   to it that are listed in the list <list>, from index
    #   <from> to index <to> (default: all the list).
    #
    #   If a step is called <optStep>, then call it's method
    #   passing argument <optArg>.  This is used to pass
    #   arguments -fromstep, -step to the step methods.
    #----------------------------------------------------------------

    protected method LoadSteps { list {from 0} {to ""} {optStep ""} {optArg ""} } {
	global pkginType

	#*# 1. create steps container
	set container [ctooOBJ_CONTAINER $this.steps \
		      -objType STEPS -name VLT]
	AddObj $container

	#*# 2. add steps and plugins
	if { "$to" == "" } {
	    set to [llength $list]
	} else {
	    incr to
	}
	loop i $from $to {
	    set txt [lindex $list $i]
	    set stepType [csubstr $txt 0 5]
	    set stepName [crange $txt 5 end]
	    if { $stepType == "step_" } {

		#*# 3. if first step is BUILD_ENV, then first add step STOP_ENV
		if { $i == $from && "$stepName" == "BUILD_ENV"  } {
		    set stepMethod [lindex $stepMethods [lsearch $steps "STOP_ENV"]]
		    # optArg contains the environments that will be build.
		    # Stop only these environments.
		    set obj [$pkginType(STEP) $this.stepa \
			-name STOP_ENV -command [code $this $stepMethod $optArg]]
		    $container AddObj $obj
		}

		#*# 4. add step
		set stepMethod [lindex $stepMethods [lsearch $steps $stepName]]
		if { "$txt" == "$optStep" } {
		    set obj [$pkginType(STEP) $this.step$i \
			-name $stepName -command [code $this $stepMethod $optArg]]
		} else {
		    set obj [$pkginType(STEP) $this.step$i \
			-name $stepName -command [code $this $stepMethod]]
		}
		$container AddObj $obj
	    } else {

		#*# 5. if optStep is BUILD_MOD and optArg is not empty,
		#      then skip the BEFORE_BUILD_MOD hooks.
		if { "$stepName" == "BEFORE_BUILD_MOD" && \
		     "$optStep" == "step_BUILD_MOD" && "$optArg" != "" } {
		    continue
		}

		#*# 6. add plugins
		loop j 1 100 {
		    set obj [LookupName ${stepName}_$j PLUGIN $maxdepth]
		    if { "$obj" != "" } {
			# ALO: if we are not in step clean, skip BEFORE_CLEAN and AFTER_CLEAN
			if { ($clean == 0) && [string match *_CLEAN $stepName] } { continue }
			$container AddObj $obj
		    } else {
			break
		    }
		}
	    }
	}
    }

    #****************************************************************
    #*# method IsOfflineBuild
    #
    #   Make an off-line build if WS environments are not local.
    #----------------------------------------------------------------

    public method IsOfflineBuild {} {

	#*# 1. retrieve the available RTAP environments
	set container [[Contents PACKAGE] Contents AVAIL_ENVIRONMENTS]
	set objs [$container Contents RTAPENV]

	#*# 2. check if all environments are local
	set localEnvs ""
	set remoteEnvs ""
	set isOffline 0
	foreach obj $objs {
	    if { [$obj IsLocal] } {
		lappend localEnvs [$obj cget -name]
	    } else {
		lappend remoteEnvs [$obj cget -name]
		set isOffline 1
	    }
	}

	#*# 3. if there is one local env., then all must be local
	if { $localEnvs != "" && $remoteEnvs != "" && $step != "RETRIEVE" } {
	    error "Invalid configuration: all WS environments must be local to the target WS (local: $localEnvs, remote: $remoteEnvs)."
	}

	return $isOffline
    }

    #****************************************************************
    #*# method Build
    #----------------------------------------------------------------

    public method Build {} {

	#*# 1. check authorization
	CheckAuthorization BUILD

	#*# 2. check env. vars.
	CheckEnvVars

	#*# 3. execute all steps
	set errList ""
	foreach obj [[Contents STEPS] Contents] {
	    $obj Exec
	}
	ExitIfErrors
    }

    #****************************************************************
    #*# method Install
    #----------------------------------------------------------------

    public method Install {} {

	#*# 1. check authorization
	CheckAuthorization INSTALL

	#*# 2. check env. vars.
	CheckEnvVars

	#*# 3. execute all steps
	set errList ""
	foreach obj [[Contents STEPS] Contents] {
	    $obj Exec
	}
	ExitIfErrors
    }

    #****************************************************************
    #*# method Verify
    #----------------------------------------------------------------

    public method Verify {} {

	#*# 2. execute all steps
	set errList ""
	foreach obj [[Contents STEPS] Contents] {
	    $obj Exec
	}
	ExitIfErrors
    }

    #****************************************************************
    #*# method Clean
    #----------------------------------------------------------------

    public method Clean {} {
	set errList ""
	StepBegin CLEAN "Cleaning modules"
	StepExec  MODULES "" "" Clean "No modules to clean."
	StepEnd   CLEAN
	ExitIfErrors 0
    }

    #****************************************************************
    #*# method Versions
    #----------------------------------------------------------------

    public method Versions {} {

	#*# 1. show versions
	set errList ""
	StepBegin INFO "Module versions"
	Info "Module   Source/INTROOT   Last      Config    -> Update    Comments"
	Info "----------------------------------------------------------------------------"
	StepExec  MODULES "" "" "ShowVersions $introot $update" "No modules."

	#*# 2. update versions
	if { $update } {
	    set cfg [[Repository] Contents CONFIG]
	    try_eval {
		Info "----------------------------------------------------------------------------"
		Info "Updating config. file ... " 0
		$cfg Save
		Info "OK."
	    } {
		append errList "ERROR: $errorResult\n"
		Info "FAILURE."
	    }
	}
	StepEnd INFO
	ExitIfErrors 0
    }

    #****************************************************************
    #*# method CheckAuthorization
    #----------------------------------------------------------------

    public method CheckAuthorization { phase } {

	#*# 1. retrieve user and usermgr
	set package [Contents PACKAGE]
	set user    [$package cget -user]
	set usermgr [$package cget -usermgr]

	#*# 2. Always allow to run START_LCUENV and CREATE_SCAN.
	set steps [[Contents STEPS] ContentsByName]
	set steps2 {START_LCUENV CREATE_SCAN}
	if { [lindex [intersect3 $steps $steps2] 2] != {} } {

	    #*# 3. BUILD phase:
	    #      The main user is not allowed to install the software,
	    #      specially on the target machine.
	    if { "$phase" == "BUILD" } {
		if { "$user" != "" && "$user" == "[GetEnv USER]" && "$user" != "$usermgr" } {
		    error "User '$user' is not allowed to build the software."
		}

	    #*# 4. INSTALL phase:
	    #      If a userMgr is registered, then he is the only one allowed
	    #      to install the software.
	    } elseif { "$phase" == "INSTALL" } {
		if { "$usermgr" != "" && "$usermgr" != "[GetEnv USER]" } {
		    error "Only user '$usermgr' is allowed to install software."
		}

	    #*# 5. wrong phase:
	    } else {
		error "Authorization check: unknown phase $phase."
	    }
	}

	#*# 6. if we need to rsh/remsh to the user account,
	#      then check that we can do so.
	#      (we only check when needed, because this
	#      takes a 'long' time).
	#
	set steps2 {STOP_ENV START_WSENV START_LCUENV CREATE_SCAN}
	if { "$user" != "" && "$user" != "[GetEnv USER]" && \
	     [intersect $steps $steps2] != {} } {
	    if { [exec uname] == "HP-UX" } {
		set rsh "remsh"
	    } else {
		set rsh "rsh"
	    }
	    try_eval {
		System "$rsh [GetEnv HOST] -l $user -n echo OK"
	    } {
		error "User [GetEnv USER] is unable to rsh to user $user."
	    }
	}
    }

    #****************************************************************
    #*# method CheckEnvVars
    #----------------------------------------------------------------

    public method CheckEnvVars {} {

	#*# 1. if no envvars: return
	set envvars [[Contents PACKAGE] cget -envvars]
	if { $envvars == "" } {
	    return
	}

	#*# 2. process each env. variable
	set errList ""
	StepBegin INIT "Checking environment variables"
	foreach envvar $envvars {
	    try_eval {
		# extract "varname[=value]"
		set isequal [expr {[string first "=" $envvar] != -1}]
		set varname [string trim [ctoken envvar "="]]
		set val1    [string trim [ctoken envvar "="]]
		if {[catch {
		    set val2 [GetEnv $varname]
		}]} {
		    Info "       [format %-10s $varname]= <not defined>."
		    error "Env. variable $varname not defined."
		} else {
		    # env. var. set: check value.
		    if { $isequal == 1 && "$val1" != "$val2" } {
			Info "       [format %-10s $varname]= '$val2' (should be '$val1')"
			error "Env. variable $varname = '$val2' (should be '$val1')."
		    } else {
			Info "       [format %-10s $varname]= '$val2'"
		    }
		}
	    } {
		append errList "ERROR: $errorResult\n"
	    }
	}
	StepEnd INIT
	ExitIfErrors
    }

    #****************************************************************
    #*# method SaveConfig
    #----------------------------------------------------------------

    public method SaveConfig {} {

	#*# 1. init.
	StepBegin CFG "Saving application configuration"

	#*# 2. save config. only when available, INTROOT and INS_ROOT exist
	try_eval {
	    set cfgname [[Contents PACKAGE] cget -cfgname]
	    if { $cfgname == "" } {
		Info "       No application config. to be saved:   OK."
	    } elseif { [file isdirectory [GetEnv INTROOT]] == 0 ||
		       [file isdirectory [GetEnv INS_ROOT]] == 0 } {
		Info "       INS_ROOT and/or INTROOT do not exist: SKIPPED."
	    } elseif { [GetEnv DISPLAY] == "" } {
		error "DISPLAY env. variable is missing."
	    } else {
		Info "       Saving config. $cfgname (if not needed: press Cancel) ... " 0
		System "ctooConfigArchive $cfgname"
		Info "OK."
	    }
	} {
	    append errList "ERROR: $errorResult\n"
	}

	#*# 3. end
	StepEnd CFG
	ExitIfErrors
    }

    #****************************************************************
    #*# method RetrieveModules
    #----------------------------------------------------------------

    public method RetrieveModules { {objs ""} } {

	#*# 1. init.
	StepBegin CMM "Retrieving modules"

	#*# 2. retrieve all module objs, except the installation module
	if { "$objs" == "" } {
	    set container [[Contents PACKAGE] Contents MODULES]
	    set objs [$container Contents]
	    loop i 0 [llength $objs] {
		if { "[[lindex $objs $i] Name]" == "$module" } {
		    lvarpop objs $i
		    break
		}
	    }
	} else {
	    loop i 0 [llength $objs] {
		if { "[[lindex $objs $i] Name]" == "$module" } {
		    error "RETRIEVE of the integration module is not supported."
		}
	    }
	}

	#*# 3. retrieve modules from Archive
	StepExec  MODULES "" $objs Retrieve "No modules to retrieve."

	#*# 4. report status
	StepEnd   CMM
    }

    #****************************************************************
    #*# method BeforeInstall
    #----------------------------------------------------------------

    public method BeforeInstall {} {

	#*# 1. if tarfile: unpack plugins
	if { "$tarfile" != "" } {
	    set container [[Contents PACKAGE] Contents PLUGINS]
	    set obj [$container LookupName BEFORE_INSTALL_1]
	    if { "$obj" != "" } {
		exec pkginUnziptar $tarfile "./INSTALL/PLUGINS"
		if {![file isfile "./INSTALL/PLUGINS/PLUGIN_BEFORE_INSTALL_1"]} {
		    error "Error extracting plugins from $tarfile (file INSTALL/PLUGINS/PLUGIN_BEFORE_INSTALL_1 not found)."
		}
	    }
	}
    }

    #****************************************************************
    #*# method StopEnvs
    #----------------------------------------------------------------

    public method StopEnvs { {objs ""} } {

	#*# 1. if running pkginUnpack: stop all environments
	#      (because all envs. will be extracted from the tar file).
	set container ENVIRONMENTS
	if { $logname == "pkginUnpack" } {
	    set objs ""
	    set container ALL_ENVIRONMENTS
	}

	#*# 2. remove any LCU environments
	if { "$objs" != "" } {
	    set wsObjs ""
	    foreach obj $objs {
		if { "[$obj Type]" != "LCUENV" } {
		    lvarpush wsObjs $obj
		}
	    }
	    set objs $wsObjs
	    if { $objs == "" } {
		return
	    }
	}

	#*# 3. run step
	StepBegin STOP "Stopping WS environments"
	StepExec  $container RTAPENV $objs Stop "No environments to stop."
	StepEnd   STOP
    }

    #****************************************************************
    #*# method CleanDirs
    #----------------------------------------------------------------

    public method CleanDirs {} {

	#*# 1. if no -clean: skip
	if { $clean == 0 } {
	    return
	}

	#*# 2. clean environments
	StepBegin CLEAN "Cleaning directories"
	StepExec  ALL_ENVIRONMENTS "" "" Clean "No environments to clean."

	#*# 3. clean directories
	set container [Contents ROOTS]
	set objs [$container Contents]
	StepExec  "" "" $objs Backup "No root areas to clean."
	StepEnd   CLEAN
    }

    #****************************************************************
    #*# method CreateRoots
    #----------------------------------------------------------------

    public method CreateRoots {} {
	StepBegin BUILD "Preparing root areas"
	set container [Contents ROOTS]
	set objs [$container Contents]
	StepExec  "" "" $objs "Create $backup" "No directories to prepare."
	StepEnd   BUILD
    }

    #****************************************************************
    #*# method BuildModules
    #----------------------------------------------------------------

    public method BuildModules { {objs ""} } {

	#*# 1. build modules
	StepBegin BUILD "Building modules"
	StepExec  MODULES "" $objs "Build $envonly" "No modules to build." 1

	#try_eval {
	    #*# 2. create INSTALL/INSTALL.cfg file
	    #Info "       Saving  INSTALL.cfg file ...                                " 0
	    set filedir [file join [GetEnv INTROOT] INSTALL]
	    file mkdir $filedir
	    set filename [file join $filedir INSTALL.cfg]
	    set cfg [[Repository] Contents CONFIG]
	    $cfg SaveAs $filename

	    #*# 3. create pkgin-apps-all.env file
	    set vltswRelease ""
	    set vltswCcstype ""
	    set windBase     ""
	    set midasHome    ""
	    set midasVersion ""
	    set targetId     ""
	    catch { set vltswRelease [GetEnv VLTSW_RELEASE] }
	    catch { set vltswCcstype [GetEnv VLTSW_CCSTYPE] }
	    catch { set windBase     [GetEnv WIND_BASE] }
	    catch { set midasHome    [GetEnv MIDASHOME] }
	    catch { set midasVersion [GetEnv MIDVERS] }
	    catch { set targetId     [GetEnv TARGET] }
	    if { $vltswRelease != "" || $vltswCcstype != "" || \
		 $windBase != "" || $midasHome != "" || $midasVersion != "" || \
		 $targetId != "" } {
		set filename [file join [GetEnv INTROOT] config pkgin-apps-all.env]
		file delete $filename
		set txt \
"#*******************************************************************************
# E.S.O. - VLT project
#
# \"@(#) \$Id\$\"
#
# This file is used to specify the VLTSW release and type,
# VxWorks version and MIDAS version used to build the
# executables in this INTROOT.
# Login scripts may use these values to select the correct
# VLTROOT.
#
#-------------------------------------------------------------------------------
#
"
		if { $vltswRelease != "" } {
		    append txt "export VLTSW_RELEASE=$vltswRelease\n"
		}
		if { $vltswCcstype != "" } {
		    append txt "export VLTSW_CCSTYPE=$vltswCcstype\n"
		}
		if { $windBase != "" } {
		    append txt "if \[ X\${WIND_BASE} != X \]; then\n    export WIND_BASE=$windBase\nfi\n"
		}
		if { $midasHome != "" } {
		    append txt "if \[ X\${MIDASHOME} != X \]; then\n    export MIDASHOME=$midasHome\nfi\n"
		}
		if { $midasVersion != "" } {
		    append txt "if \[ X\${MIDVERS} != X \]; then\n    export MIDVERS=$midasVersion\nfi\n"
		}
		if { $targetId != "" } {
		    append txt "export TARGET=$targetId\n"
		}
		append txt "\n#___oOo___\n"
		write_file $filename $txt
	    }

	#} {
	#    LogErr $errorResult
	#    Info "FAILURE."
	#    if { $debug } {
	#        error $errorResult $errorCode $errorInfo
	#    } else {
	#        if {[string first "ERROR: " $errorResult] == 0} {
	#    	    append errList $errorResult
	#        } else {
	#    	    append errList "ERROR: $errorResult\n"
	#	}
	#    }
	#}
	#Info "OK."

	#*# 4. set new env. vars.
	set envvars [SourceEnvVars]
	if { $envvars != "" && $offline == 0 } {
	    Info ""
	    Info "       WARNING: You will have to logout, login to set new env. variables"
	    Info "                (after this script has completed)."
	    Info ""
	    # variable logEndInfo will be logged by LogEnd().
	    append logEndInfo "
WARNING: You should now logout and login from this shell, to set new
	 environment variables needed by modules installed in INTROOT:
	 $envvars.
"
	}

	#*# 5. set config. file permissions
	catch { system "ctooConfigInstall 2> /dev/null" }

	StepEnd BUILD
    }

    #****************************************************************
    #*# method BuildEnvs
    #----------------------------------------------------------------

    public method BuildEnvs { {objs ""} } {
	StepBegin BUILD "Building environments"
	StepExec  ENVIRONMENTS "" $objs Create "No environments to build." 1
	StepEnd   BUILD
    }

    #****************************************************************
    #*# method Pack
    #----------------------------------------------------------------

    public method Pack {} {
	ExitIfErrors
	StepBegin PACK "Creating package file"
	[Contents PKGFILE] Pack
	StepEnd   PACK
    }

    #****************************************************************
    #*# method InstallRoots
    #----------------------------------------------------------------

    public method InstallRoots {} {

	#*# 1. backup INTROOT, INS_ROOT
	StepBegin UNPCK "Unpacking package file"
	if { $keeproots == 0 } {
	    set container [Contents ROOTS]
	    set rootObjs [$container Contents]
	    StepExec "" "" $rootObjs Backup "No directories to backup."
	} else {
	    Info "       Keeping current INTROOT and INS_ROOT (no backup)."
	}

	#*# 2. backup environments
	try_eval {
	    Info "       VLTDATA:  renaming environments ...        " 0
	    Log ""
	    set container [[Contents PACKAGE] Contents ALL_ENVIRONMENTS]
	    set objs [$container Contents]
	    if { "$objs" != "" } {
		StepExec "" "" $objs Backup ""
	    }
	    Info "OK."
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}

	#*# 3. check/register disk space
	Log "\n       Disk space before installing new root areas:"
	foreach obj $rootObjs {
	    set dirname  [$obj cget -name]
	    set dirpath  [$obj cget -dir]
	    set dirspace [$obj DiskSpace]
	    set minsize  [$obj cget -size]
	    if { $minsize != 0 } {
		Log "       $dirname: avail=[format %6s $dirspace]K, min=[format %6s $minsize]K ($dirpath)."
		# always ask for 1Mb more
		incr minsize 1000
		if { $dirspace < $minsize } {
		    error "Insufficient disk space for $dirname: need=[set minsize]K, avail=[set dirspace]K."
		}
	    } else {
		Log "       $dirname: avail=[format %6s $dirspace]K              ($dirpath)."
	    }
	}
	Log ""

	#*# 4. unpack package file
	[Contents PKGFILE] Unpack

	#*# 5. set config. file permissions
	catch { system "ctooConfigInstall 2> /dev/null" }

	#*# 7. register new disk space
	Log "\n       Disk space after installing new root areas:"
	foreach obj $rootObjs {
	    set dirname  [$obj cget -name]
	    set dirpath  [$obj cget -dir]
	    set dirspace [$obj DiskSpace]
	    Log "       $dirname: avail=[format %6s $dirspace]K              ($dirpath)."
	}
	Log ""
	StepEnd UNPCK
    }

    #****************************************************************
    #*# method InstallFiles
    #----------------------------------------------------------------

    public method InstallFiles { {objs ""} } {

	#*# 1. if envonly: skip
	if { $envonly } {
	    return
	}

	#*# 2. install files
	StepBegin BUILD "Installing additional files"
	StepExec  FILESETS "" $objs Install "No additional files to install."

	#*# 3. set config. file permissions
	catch { system "ctooConfigInstall 2> /dev/null" }

	StepEnd   BUILD
    }

    #****************************************************************
    #*# method AfterInstall
    #----------------------------------------------------------------

    public method AfterInstall {} {

	#*# 1. if envonly: skip
	if { $envonly } {
	    return
	}

	#*# 2. if no application config.: nothing to do
	set cfgname [[Contents PACKAGE] cget -cfgname]
	if { $cfgname == "" } {
	    return
	}

	#*# 3. retrieve related config. module
	StepBegin BUILD "Installing latest application configuration"
	try_eval {
	    set cfgname [[Contents PACKAGE] cget -cfgname]
	    ctooCCONFIG appcfg
	    set i 1
	    while {1} {
		try_eval {
		    set cfgname1 [appcfg Get CONFIG.ARCHIVE$i.NAME]
		} {
		    delete object appcfg
		    error "Application config. $cfgname not found (see INSTALL.PACKAGE.CFGNAME)."
		}
		if { $cfgname1 == $cfgname } {
		    break
		}
		incr i
	    }
	    set module [appcfg Get CONFIG.ARCHIVE$i.MODULE]
	    set obj [[[Contents PACKAGE] Contents MODULES] LookupName $module]
	    if { $obj == "" } {
		error "Config. module $module not known (INSTALL.PACKAGE.CFGNAME=$cfgname)."
	    }
	    if { [llength $obj] > 1 } {
		error "More than one module $module registered.  Check configuration."
	    }

	    #*# 4. get config. module versions
	    set currVersion [$obj GetIntrootVersion]
	    set lastVersion [lindex [$obj GetLastVersion] 0]
	    set cfgVersion  [$obj cget -cfgvers]

	    #*# 5. if correct version installed: nothing to do
	    if { $currVersion == $lastVersion } {
		Info "       Module $module $currVersion is already installed: OK."
	    } elseif { $currVersion != "" && $cfgVersion != "LAST" } {
		Info "       WARNING: Using config. module $currVersion (last=$lastVersion)."

	    #*# 6. install latest config. module
	    } else {
		if { $cfgVersion == "LAST" } {
		    $obj configure -version $lastVersion
		} else {
		    $obj configure -version $cfgVersion
		}
		$obj Retrieve
		$obj Build
	    }
	} {
	    if {[string first "ERROR: " $errorResult] == 0} {
		append errList $errorResult
	    } else {
		append errList "ERROR: $errorResult\n"
	    }
	}
	delete object appcfg
	StepEnd BUILD
    }

    #****************************************************************
    #*# method StartWsEnvs
    #----------------------------------------------------------------

    public method StartWsEnvs { {objs ""} } {
	ExitIfErrors
	StepBegin START "Starting WS environments"
	StepExec  ENVIRONMENTS RTAPENV $objs "Start 1" "No WS  environments to start."
	StepEnd   START
    }

    #****************************************************************
    #*# method InstallCcds
    #----------------------------------------------------------------

    public method InstallCcds { {objs ""} } {

	#*# 1. if -env is given, then limit detectors to used envs.
	StepBegin START "Installing detectors"
	if { "$env" != "" && "$objs" == "" } {
	    set container [[Contents PACKAGE] Contents DETECTORS]
	    set objs ""
	    foreach obj [$container Contents] {
		$obj SetConfig
		set envname [$obj cget -envname]
		set ccdlenv [$obj cget -ccdlenv]
		if { [lsearch $env $envname] != -1 } {
		    lappend objs $obj
		} elseif { [lsearch $env $ccdlenv] != -1 } {
		    lappend objs $obj
		}
	    }
	}

	#*# 2. run step
	StepExec  DETECTORS "" $objs Install "No detectors to install."
	StepEnd   START
    }

    #****************************************************************
    #*# method StartLcuEnvs
    #----------------------------------------------------------------

    public method StartLcuEnvs { {objs ""} } {
	StepBegin START "Starting LCU environments"
	StepExec  ENVIRONMENTS LCUENV $objs "Start 1" "No LCU environments to start."
	StepExec  ENVIRONMENTS LCUENV $objs StartWait ""
	StepEnd   START
    }

    #****************************************************************
    #*# method CreateScanLinks
    #----------------------------------------------------------------

    public method CreateScanLinks { {objs ""} } {
	StepBegin START "Create scanlinks"
	StepExec  SCANLINKS "" $objs Install "No scanlinks to create." 0 10
	StepEnd   START
    }

    #****************************************************************
    #*# method EndInstall
    #----------------------------------------------------------------

    public method EndInstall {} {
	StepBegin START "End Install (OLDB snapshots)"
	StepExec  ENVIRONMENTS RTAPENV "" ForceSnap "No WS environments."
	StepEnd   START
    }

    #****************************************************************
    #*# method VerifyRoots
    #----------------------------------------------------------------

    public method VerifyRoots { {objs ""} } {
	StepBegin CHECK "Verify installed modules"
	StepExec  ROOTS "" [[Contents ROOTS] LookupName INTROOT] Verify ""
	StepEnd   CHECK
	StepBegin CHECK "Verify INS_ROOT files"
	StepExec  ROOTS "" [[Contents ROOTS] LookupName INSROOT] Verify ""
	StepEnd   CHECK
	StepBegin CHECK "Verify additional files"
	StepExec  FILESETS "" "" Verify "No additional files to verify."
	StepEnd   CHECK
    }

    #****************************************************************
    #*# method VerifyEnv
    #----------------------------------------------------------------

    public method VerifyEnv { {objs ""} } {
	StepBegin CHECK "Verify environments"
	StepExec  ENVIRONMENTS "" $objs Verify "No environments to verify."
	StepEnd   CHECK
    }

    #****************************************************************
    #*# method VerifyScanLinks
    #----------------------------------------------------------------

    public method VerifyScanLinks { {objs ""} } {
	StepBegin CHECK "Verify scanlinks"
	StepExec  SCANLINKS "" $objs Verify "No scanlinks to verify."
	StepEnd   CHECK
    }

    #****************************************************************
    #*# method StepBegin
    #----------------------------------------------------------------

    protected method StepBegin { id title } {

	#*# 1. init.
	Info $logSep1
	Info "[format %-5s $id]: $title:"
	set errListSize [clength $errList]
    }

    #****************************************************************
    #*# method StepExec
    #----------------------------------------------------------------

    protected method StepExec { container type objs method noObjMsg {sepObj 0} {delay 0} } {

	#*# 1. retrieve objects
	if { "$objs" == "" } {
	    set container [[Contents PACKAGE] Contents $container]
	    set objs [$container Contents $type]
	}

	#*# 2. exec. method on each object
	if { "$objs" == "" } {
	    Info "       $noObjMsg"
	}
	foreach obj $objs {
	    if { $debug } {
		eval $obj $method
	    } else {
		try_eval {
		    eval $obj $method
		} {
		    if {[string first "ERROR: " $errorResult] == 0} {
			append errList $errorResult
		    } else {
			append errList "ERROR: $errorResult\n"
		    }
		}
	    }
	    if { $sepObj } {
		Log $logSep1
	    }
	}

	#*# 3. delay before continuing
	if { "$objs" != "" && $delay != 0 } {
	    sleep $delay
	}
    }

    #****************************************************************
    #*# method StepEnd
    #----------------------------------------------------------------

    protected method StepEnd { id } {

	#*# 1. report status
	if { [clength $errList] != $errListSize } {
	    Info "[format %-5s $id]: FAILURE."
	} else {
	    Info "[format %-5s $id]: OK."
	}
    }

    #****************************************************************
    #*# method ExitIfErrors
    #----------------------------------------------------------------

    protected method ExitIfErrors { {seeErrLog 1} } {
	if { "$errList" != "" } {
	    if { $seeErrLog } {
		append errList "       See error log file for details.\n"
	    }
	    error $errList
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable module   ""
    public variable ver      ""
    public variable tarfile  ""
    public variable env      ""
    public variable fromstep ""
    public variable step     ""
    public variable tostep   ""
    public variable envonly  0
    public variable clean    0
    public variable backup   0
    public variable maxbak   2
    public variable mod      ""
    public variable maxdepth 10
    public variable debug    0
    public variable update   0
    public variable introot  0
    public variable build    0
    public variable keeproots 0

    protected variable offline  0

    protected variable errList ""
    protected variable errListSize 0

    # all build/install/verify step names known to pkginINSTALLATION
    protected variable steps {
	SAVE_CONFIG   RETRIEVE      BEFORE_INSTALL STOP_ENV
	CLEAN	      CREATE_ROOTS  BUILD_MOD      BUILD_ENV
	PACK 	      INSTALL_ROOTS INSTALL_FILES  AFTER_INSTALL
	START_WSENV   INSTALL_CCD   START_LCUENV   CREATE_SCAN
	END_INSTALL   VERIFY_ROOTS  VERIFY_ENV     VERIFY_SCAN
    }

    # Methods assigned to each build/install step listed above.
    # E.g. method StopEnvs executes step STOP_ENV.
    protected variable stepMethods {
	SaveConfig   RetrieveModules BeforeInstall   StopEnvs
	CleanDirs    CreateRoots     BuildModules    BuildEnvs
	Pack         InstallRoots    InstallFiles    AfterInstall
	StartWsEnvs  InstallCcds     StartLcuEnvs    CreateScanLinks
	EndInstall   VerifyRoots     VerifyEnv       VerifyScanLinks
    }

    # hook names accepted by INSTALL.HOOKi.NAME
    public variable hooks {
	AFTER_SAVE_CONFIG   BEFORE_RETRIEVE     AFTER_RETRIEVE
	BEFORE_INSTALL      AFTER_STOP_ENV      BEFORE_CLEAN
	AFTER_CLEAN         AFTER_CREATE_ROOTS  BEFORE_BUILD_MOD
	AFTER_BUILD_MOD     BEFORE_BUILD_ENV	AFTER_BUILD_ENV
	AFTER_INSTALL_FILES AFTER_INSTALL       AFTER_START_WSENV
	AFTER_INSTALL_CCD   AFTER_START_LCUENV  AFTER_CREATE_SCAN
	BEFORE_END_INSTALL  AFTER_VERIFY_ROOTS  AFTER_VERIFY_ENV
	AFTER_VERIFY_SCAN
    }

    # Order how steps and hooks are executed when pkginBuild
    # is called under normal conditions, i.e. not off-line.
    protected variable buildOrder {
	step_SAVE_CONFIG      hook_AFTER_SAVE_CONFIG
	hook_BEFORE_RETRIEVE
	step_RETRIEVE         hook_AFTER_RETRIEVE
	step_BEFORE_INSTALL   hook_BEFORE_INSTALL
	step_STOP_ENV         hook_AFTER_STOP_ENV
	hook_BEFORE_CLEAN
	step_CLEAN            hook_AFTER_CLEAN
	step_CREATE_ROOTS     hook_AFTER_CREATE_ROOTS
	hook_BEFORE_BUILD_MOD
	step_BUILD_MOD        hook_AFTER_BUILD_MOD
	hook_BEFORE_BUILD_ENV
	step_BUILD_ENV        hook_AFTER_BUILD_ENV
	step_INSTALL_FILES    hook_AFTER_INSTALL_FILES
	step_AFTER_INSTALL    hook_AFTER_INSTALL
	step_START_WSENV      hook_AFTER_START_WSENV
	step_INSTALL_CCD      hook_AFTER_INSTALL_CCD
	step_START_LCUENV     hook_AFTER_START_LCUENV
	step_CREATE_SCAN      hook_AFTER_CREATE_SCAN
	hook_BEFORE_END_INSTALL step_END_INSTALL
	step_VERIFY_ENV	      hook_AFTER_VERIFY_ENV
	step_VERIFY_SCAN      hook_AFTER_VERIFY_SCAN
    }
    protected variable lastBuildStep VERIFY_SCAN

    # Order how steps and hooks are executed when pkginBuild
    # is called during off-line build
    protected variable offlineOrder {
	hook_BEFORE_RETRIEVE
	step_RETRIEVE        hook_AFTER_RETRIEVE
	hook_BEFORE_CLEAN
	step_CLEAN           hook_AFTER_CLEAN
	step_CREATE_ROOTS    hook_AFTER_CREATE_ROOTS
	hook_BEFORE_BUILD_MOD
	step_BUILD_MOD       hook_AFTER_BUILD_MOD
	hook_BEFORE_BUILD_ENV
	step_BUILD_ENV       hook_AFTER_BUILD_ENV
	step_PACK
    }
    protected variable firstOfflineStep  RETRIEVE
    protected variable lastOfflineStep   BUILD_ENV

    # Order how steps and hooks are executed by pkginUnpack.
    protected variable installOrder {
	step_SAVE_CONFIG     hook_AFTER_SAVE_CONFIG
	step_BEFORE_INSTALL  hook_BEFORE_INSTALL
	step_STOP_ENV        hook_AFTER_STOP_ENV
	step_INSTALL_ROOTS
	step_INSTALL_FILES   hook_AFTER_INSTALL_FILES
	step_AFTER_INSTALL   hook_AFTER_INSTALL
	step_START_WSENV     hook_AFTER_START_WSENV
	step_INSTALL_CCD     hook_AFTER_INSTALL_CCD
	step_START_LCUENV    hook_AFTER_START_LCUENV
	step_CREATE_SCAN     hook_AFTER_CREATE_SCAN
	hook_BEFORE_END_INSTALL step_END_INSTALL
	step_VERIFY_ROOTS    hook_AFTER_VERIFY_ROOTS
	step_VERIFY_ENV	     hook_AFTER_VERIFY_ENV
	step_VERIFY_SCAN     hook_AFTER_VERIFY_SCAN
    }
    protected variable firstInstallStep SAVE_CONFIG
    protected variable lastInstallStep  AFTER_INSTALL
    protected variable firstInstallOfflineStep INSTALL_ROOTS
    protected variable lastInstallOfflineStep  INSTALL_ROOTS

    # Order how steps and hooks are executed by pkginVerify.
    protected variable verifyOrder {
	step_VERIFY_ROOTS    hook_AFTER_VERIFY_ROOTS
	step_VERIFY_ENV	     hook_AFTER_VERIFY_ENV
	step_VERIFY_SCAN     hook_AFTER_VERIFY_SCAN
    }

    # Names that are allowed to be used as argument
    # to pkginBuild options -fromstep, -step and -tostep.
    protected variable buildOptions {
	SAVE_CONFIG 		RETRIEVE
	BEFORE_INSTALL 		STOP_ENV
	CLEAN 			CREATE_ROOTS
	BUILD_MOD		BUILD_ENV
	INSTALL_ROOTS		INSTALL_FILES
	AFTER_INSTALL		START_WSENV
	INSTALL_CCD		START_LCUENV
	CREATE_SCAN		END_INSTALL
	VERIFY_ENV		VERIFY_SCAN
	PACK
    }

    # The following two tables list the first and last step
    # (or hook) that are executed for a given step.
    #
    # E.g. if pkginBuild receives option '-step RETRIEVE'
    # then hook BEFORE_RETRIEVE (in table buildOptionsStart'
    # to hook AFTER_RETRIEVE (in table buildOptionsStop)
    # are executed, according to the order specified in
    # table buildOrder. I.e. when option '-step RETRIEVE'
    # is used, the following steps are executed:
    #   hook BEFORE_RETRIEVE
    #   step RETRIEVE
    #   hook AFTER_RETRIEVE
    #
    # If options '-fromstep BEFORE_INSTALL -tostep BUILD_MOD'
    # are used, then the steps between step BEFORE_INSTALL and
    # hook AFTER_BUILD_MOD are executed (see corresponding
    # table entries below).
    #
    protected variable buildOptionsStart {
	step_SAVE_CONFIG 	hook_BEFORE_RETRIEVE
	step_BEFORE_INSTALL 	step_STOP_ENV
	hook_BEFORE_CLEAN       step_CREATE_ROOTS
	step_CREATE_ROOTS       hook_BEFORE_BUILD_ENV
	step_INSTALL_ROOTS      step_INSTALL_FILES
	step_AFTER_INSTALL	step_START_WSENV
	step_INSTALL_CCD	step_START_LCUENV
	step_CREATE_SCAN	hook_BEFORE_END_INSTALL
	step_VERIFY_ENV  	step_VERIFY_SCAN
	step_PACK
    }
    protected variable buildOptionsStop {
	hook_AFTER_SAVE_CONFIG	hook_AFTER_RETRIEVE
	hook_BEFORE_INSTALL 	hook_AFTER_STOP_ENV
	hook_AFTER_CLEAN	hook_AFTER_CREATE_ROOTS
	hook_AFTER_BUILD_MOD	hook_AFTER_BUILD_ENV
	step_INSTALL_ROOTS	hook_AFTER_INSTALL_FILES
	hook_AFTER_INSTALL	hook_AFTER_START_WSENV
	hook_AFTER_INSTALL_CCD	hook_AFTER_START_LCUENV
	hook_AFTER_CREATE_SCAN	step_END_INSTALL
	hook_AFTER_VERIFY_ENV	hook_AFTER_VERIFY_SCAN
	step_PACK
    }

    # The pkginBuild options -fromstep and -step accept
    # apart from a step name, an optional value that can
    # be either a module, environment, detector or scanlink
    # name. The table buildOptionsObj specifies the accepted
    # value type ("": no value accepted) for each step name
    # listed in table buildOptions.
    #
    # Table buildOptionsObjStep specifies the step method
    # that should receive the value as argument.
    #
    protected variable buildOptionsObj {
	"" 			MODULES
	"" 			""
	""			""
	MODULES			ENVIRONMENTS
	""			FILESETS
	""			ENVIRONMENTS
	DETECTORS		ENVIRONMENTS
	SCANLINKS		""
	ENVIRONMENTS		SCANLINKS
	""
    }
    protected variable buildOptionsObjStep {
	"" 			step_RETRIEVE
	"" 			""
	""			""
	step_BUILD_MOD		step_BUILD_ENV
	""			step_INSTALL_FILES
	""			step_START_WSENV
	step_INSTALL_CCD	step_START_LCUENV
	step_CREATE_SCAN	""
	step_VERIFY_ENV		step_VERIFY_SCAN
	""
    }
}


#___oOo___
