#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginINSTALL_BASE.tcl,v 1.105 2005/10/12 19:47:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  17/09/00  pkginVerify support added.
# rschmutz  13/05/00  created.
#

#************************************************************************
#   NAME
#	pkginINSTALL_BASE - base to build/install software
# 
#   SYNOPSIS
#	pkginINSTALL_BASE name
# 
#   DESCRIPTION
#	This is a base class for the main object responsible for
#	building/installing VLT software with pkgin.
#	The basic procedure is as follows:
#	1) The package configuration is read, creating the
#	   corresponding package owned objects (Config)
#	2) The command line options are processed, to define
#	   what steps to execute.
#	3) The individual steps are executed.
#
#   PUBLIC METHODS
#	ConfigBuild <cfg>
#	ConfigInstall <cfg>
#	ConfigVerify <cfg>
#	ConfigClean <cfg>
#	    Reads the package configuration and stores it then
#	    repository (for the corresponding action).
#
#	Build <options>
#	    Builds the software.
#
#	Install <options>
#	    Installs the software.
#
#	Verify <options>
#	    Verify the software.
#
#	Clean <options>
#	    Cleans the software.
#
#   FILES
#	<package>INSTALL.cfg - configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	pkginBuild pkginPack pkginUnpack pkginClean
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: pkginINSTALL_BASE
#------------------------------------------------------------------------

class pkginINSTALL_BASE {
    inherit ctooOBJ_CONTAINER pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	configure -objType INSTALL -name VLT
    	eval configure $args
    }

    #****************************************************************
    #*# proc ItclTypes
    #----------------------------------------------------------------

    proc ItclTypes {} {
    	global pkginType

	# pkginType(INSTALL) should be set by the derived class
	set pkginType(INSTALL)    pkginINSTALL_BASE
    }

    #****************************************************************
    #*# method ConfigBuild
    #----------------------------------------------------------------

    public method ConfigBuild { cfg params } {
    	# must be overloaded

	Info "BUILD: [exec date]"
	Info "ARGS : $params."
    }

    #****************************************************************
    #*# method ConfigInstall
    #----------------------------------------------------------------

    public method ConfigInstall { cfg params } {
    	# must be overloaded

	Info "INSTALL: [exec date]"
	Info "ARGS : $params."
    }

    #****************************************************************
    #*# method ConfigVerify
    #----------------------------------------------------------------

    public method ConfigVerify { cfg params } {
    	# must be overloaded

	Info "VERIFY: [exec date]"
	Info "ARGS : $params."
    }

    #****************************************************************
    #*# method ConfigClean
    #----------------------------------------------------------------

    public method ConfigClean { cfg params } {
    	# must be overloaded

	Info "CLEAN: [exec date]"
	Info "ARGS : $params."
    }

    #****************************************************************
    #*# method Build
    #----------------------------------------------------------------

    public method Build {} {
    	# must be overloaded

	Info "BUILD: Not implemented."

	Info "END  : Build end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method Install
    #----------------------------------------------------------------

    public method Install {} {
    	# must be overloaded

	Info "INSTALL: Not implemented."

	Info "END  : Install end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method Verify
    #----------------------------------------------------------------

    public method Verify {} {
    	# must be overloaded

	Info "VERIFY: Not implemented."

	Info "END  : Install end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method Clean
    #----------------------------------------------------------------

    public method Clean {} {
    	# must be overloaded

	Info "CLEAN: Not implemented."

	Info "END  : Install end."
	Info "END  : [exec date]"
    }

}


#___oOo___
