#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginINTROOT.tcl,v 1.105 2005/10/12 19:47:33 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-01-28  SPR VLTSW20050032: remove trailing '/'.
# rschmutz  2004-09-07  SPR VLTSW20040292: chmod -R go-w added.
# rschmutz  14/05/01  DiskSpace method added.
# rschmutz  13/05/01  Verify: if version=LOCAL: do not complain.
# rschmutz  23/11/00  if INTROOT not defined: issue error.
# rschmutz  17/09/00  Verify method added.
# rschmutz  08/07/00  Create: always check INTROOT.
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    18/02/00  create: The initial script is copied to $INTROOT/config
# sskole    19/11/99  create: Plugin copy removed. Now task of modules.
# sskole    18/11/99  create: Cfg. files/plugins copied to $INTROOT/config
# sskole    02/11/99  install: removed the change owner statement
# sskole    27/09/99  Implemented the method install
# rschmutz  20/08/99  constructor: PPC604 directories added.
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginINTROOT - integration root
#
#   SYNOPSIS
#	pkginINTROOT name
#
#   DESCRIPTION
#	pkginINTROOT is used to build and install software.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Create
#	    Creates the INTROOT.
#
#	Backup
#	    Makes a backup the INTROOT directory (renames it).
#
#	Verify
#	    Checks the software modules installed in INTROOT.
#
#   PUBLIC DATA
#	dir:	INTROOT directory
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginINTROOT
#------------------------------------------------------------------------

class pkginINTROOT {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	global env

	set txt ""
	catch { set txt $env(INTROOT) }
	# oslx sets INTROOT = "NOT-USED" when it is not defined ... no comments.
	if { $txt == "" || $txt == "NOT-USED" } {
	    error "No INTROOT defined."
	}
	if { $txt != "" } {
		    # remove trailing '/'
	    set txt [eval file join [file split $txt]]
	}
	configure -objType ROOT -name INTROOT -dir $txt
	eval configure $args
    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create { {force 0} } {

	#*# 1. if creation forced: backup/delete current INTROOT
	if { $force } {
	    Backup
	}

	#*# 2. create/check INTROOT
	if {[file exists $dir]} {
	    Info "       INTROOT:  using $dir: " 0
	} else {
	    Info "       INTROOT:  creating $dir ... " 0
	    file mkdir $dir
	}
	try_eval {
	    System "getTemplateForDirectory INTROOT $dir"
	    System "chmod -R go-w $dir"
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."

    }

    #****************************************************************
    #*# method: Backup
    #----------------------------------------------------------------

    public method Backup {} {

	if { [file exists $dir] } {
	   try_eval {
		if { $maxbak != 0 } {
		    Info "       INTROOT:  renaming $dir ... " 0
		    BackupFile $dir $maxbak
		} else {
		    Info "       INTROOT:  deleting $dir ... " 0
		    file delete -force $dir
		}
		Info "OK."
	    } {
		LogErr $errorResult
		Info "FAILURE."
		error $errorResult $errorCode $errorInfo
	    }
	}
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

	#*# 1. process INTROOT vltMakeInstall.config file:
	#      (fill modlist, verlist)
	set errlist ""
	set modlist ""
	set verlist ""
	set filename [file join [GetEnv INTROOT] vltMakeInstall.config]
	for_file line $filename {

	    #*# 2. skip comments and empty lines
	    set line [string trim $line]
	    if { $line == "" || [cindex $line 0] == "#" } {
		continue
	    }

	    #*# 3. get module and version
	    set modname [ctoken line " \t"]
	    if { [crange $modname end-3 end] == "-lcu" } {
		set modname [crange $modname 0 end-4]
	    } elseif { [crange $modname end-2 end] == "-ws" } {
		set modname [crange $modname 0 end-3]
	    }
	    set modvers [ctoken line " \t"]
	    if { ![lcontain $modlist $modname] } {
		lappend modlist $modname
		lappend verlist $modvers
	    }
	}

	#*# 4. process all package modules
	foreach obj [[[[Parent] Parent] Contents MODULES 2] Contents] {

	    #*# 4.1 get version and module name (or all sub-modules)
	    set modvers [$obj cget -version]
	    if { $modvers == "LAST" } {
		set modvers [lindex [$obj GetLastVersion] 0]
	    }
	    set modname [$obj cget -name]
	    set modnames [$obj cget -modules]
	    if { $modnames == "" } {
		set modnames $modname
		set modtitle "Module"
	    } else {
		Info "       Module $modname:"
		set modtitle "      "
	    }

	    #*# 4.2 process all module names:
	    foreach modname $modnames {
		Info "       $modtitle [format %-7s $modname] " 0
		set i [lsearch $modlist $modname]
		if { $i != -1 } {
		    set modvers2 [lindex $verlist $i]
		    if { "$modvers" == "$modvers2" || "$modvers" == "LOCAL" } {
			Info "[format %-10s $modvers2] installed:               OK."
		    } else {
			set txt "[format %-10s $modvers] wrong version [format %-10s $modvers2]"
			Info "[set txt] FAILURE."
			append errlist "ERROR: Module [format %-7s $modname] $txt\n"
		    }
		    lvarpop modlist $i
		    lvarpop verlist $i
		} else {
		    set txt "[format %-10s $modvers] not installed"
		    Info "$txt:           FAILURE."
		    append errlist "ERROR: Module [format %-7s $modname] $txt\n"
		}
	    }
	}

	#*# 5. list all additional modules
	if { $modlist != "" } {
	    Info "\n       Additional modules found in INTROOT:"
	    foreach modname $modlist modvers $verlist {
		Info "       Module [format %-7s $modname] [format %-10s $modvers]"
	    }
	}

	#*# 6. report errors
	if { $errlist != "" } {
	    LogErr $errlist
	    error $errlist
	}
    }

    #****************************************************************
    #*# method: SetEnvPaths
    #----------------------------------------------------------------

    public method SetEnvPaths {} {

	set txt [GetEnv PATH]
	if { [string first $dir $txt] == -1 } {
	    SetEnv PATH "$dir/bin:$txt"
	}
	if {[ExistsEnv LD_LIBRARY_PATH]} {
	    set txt [GetEnv LD_LIBRARY_PATH]
	    if { [string first $dir $txt] == -1 } {
		SetEnv LD_LIBRARY_PATH "$dir/lib:$txt"
	    }
	}
	if {[ExistsEnv LPATH]} {
	    set txt [GetEnv LPATH]
	    if { [string first $dir $txt] == -1 } {
		SetEnv LPATH "$dir/lib:$txt"
	    }
	}
	if {[ExistsEnv SHLIB_PATH]} {
	    set txt [GetEnv SHLIB_PATH]
	    if { [string first $dir $txt] == -1 } {
		SetEnv SHLIB_PATH "$dir/lib:$txt"
	    }
	}
    }

    #****************************************************************
    #*# method: DiskSpace
    #----------------------------------------------------------------

    public method DiskSpace {} {
	return [GetDiskSpace [file dirname $dir]]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable dir ""
    public variable maxbak 2
    public variable size 0
}


#************************************************************************
#___oOo___

