#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginMODULE.tcl,v 1.105 2005/10/12 19:47:33 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-10-06  method ShowVersions: -introot support added.
# rschmutz  2004-10-06  method GetIntrootVersion: consider sub-modules.
# alongino  2004-08-26  method Clean: skip module running tat
# rschmutz  2004-06-16  all procheck warnings added.
# rschmutz  2004-06-04  better procheck warning detection.
# rschmutz  2004-05-23  MakeError: check for procheck warnings "(warn"
# rschmutz  2003-11-24  ShowVersions: show also INTROOT version.
# rschmutz  05/01/03  MakeError: check for "No test found".
# rschmutz  05/01/03  ShowVersions: show "-" when module not in Archive.
# rschmutz  05/01/03  support for INSTALL.MODULE.[MAKE|OPTIONS] added.
# rschmutz  05/01/03  VLTSW20010761: use "ws","lcu" to select WS,LCU options.
# rschmutz  30/01/01  method Build: installOnly support added (for -envonly).
# rschmutz  28/10/01  if not -compress and module modified: do not retrieve.
# rschmutz  25/06/01  GetIntrootVersion added (for AfterInstall step).
# rschmutz  25/06/01  public cfgvers added (for AfterInstall step).
# rschmutz  22/06/01  SPR 20010364: ShowVersions: display if last is modified.
# rschmutz  22/06/01  allow CPU=MC68000.
# rschmutz  11/05/01  check options.
# rschmutz  30/01/01  SPR 20010053: options INSTNOVX_LCU and VXWORKS_LCU added.
# rschmutz  10/12/00  SPR 20000545: log module in error file.
# rschmutz  08/12/00  SKIP, WS_ONLY, LCU_ONLY, TEST_ONLY options added.
# rschmutz  06/12/00  IGNORE_WS/LCU_ERRORS/WARNINGS options added.
# rschmutz  30/09/00  WS code build: ignore warnings.
# rschmutz  17/09/00  Verify method added.
# rschmutz  07/08/00  Config: config. only -cpu when no i is specified.
# rschmutz  08/07/00  GetCurrentVersion: do not return '+' in version.
# rschmutz  26/06/00  method ShowVersions added.
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  22/06/00  use MAKE_VERBOSE=ON, if loglevel >= 2 (SPR 20000075).
# rschmutz  22/06/00  added IGNORE_WARNINGS option (SPR 20000074).
# rschmutz  22/06/00  allow a comment in version.
# rschmutz  22/06/00  support for version = "LOCAL" added.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    27/02/00  Simplified buildTargets.
# sskole    27/02/00  DoMake: making all targets together. SPR VLTSW20000037
# sskole    25/02/00  Removed the getCopy "compress" arg. 
# sskole    25/02/00  Added setIndex used by the integration module.
# sskole    25/02/00  Fixed bug in getCopy.
# sskole    25/02/00  constructor: Path specification for int. module
# sskole    25/02/00  constructor: sub pkg full path allowed.  
# sskole    18/02/00  isLcuModule: If no CPU type is specified in the
#                     cfg. file, $CPU is used.
# sskole    18/02/00  The module version is now retrieved in getCopy.
# sskole    16/12/99  Added the methods isLcuModule and doMake.
# sskole    16/12/99  getCopy: fixed bug when retrieving modified mod.
# rschmutz  17/10/99  IGNORE_ERRORS option added (for FORS).
# sskole    07/10/99  Currently modified modules are only applicable 
#                     when not using -compress.
# sskole    26/08/99  getCopy: updated m_version in case the module is 
#                     being modified.
#                     Added warning check in isMakeError.
#                     Added clean-after-build option in buildTargets.
# sskole    25/08/99  Added status to the stdout text in getCopy
# rschmutz  22/08/99  removed INS_ROOT copy (should be placed in the
#		      module Makefile).
# rschmutz  20/08/99  removed 'test' from default build directories,
#		      options TEST and TESTONLY added.
# rschmutz  20/08/99  getLastVersion: always retrieve main trunk version.
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginMODULE - software module
# 
#   SYNOPSIS
#	pkginMODULE name -name <name>
# 
#   DESCRIPTION
#	pkginMODULE is used to build and install a software module.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.MODULE<index>.NAME    -> name
#	    	INSTALL.MODULE<index>.VERSION -> version, comment (opt.)
#	        INSTALL.MODULE<index>.MAKE    -> make    (optional)
#	        INSTALL.MODULE<index>.SUBPKG  -> subpkg  (optional)
#		INSTALL.MODULE<index>.CPU     -> cpu     (optional)
#		INSTALL.MODULE<index>.OPTIONS -> options (optional)
#		INSTALL.MODULE<index>.MODULES -> modules (optional)
#		INSTALL.CON.MAXBAK	      -> maxbak  (optional)
#
#	    The INSTALL.MODULE<index>.VERSION keyword may contain
#	    the module version and a comment, as follows:
#	        "<version> [<comment>]"
#	    The version may be a version number, or the words
#	    "LAST" or "LOCAL".
#
#	Retrieve [<compress>]
#	    Retrieves the module from Archive.
#	    An existing module is renamed, if it does not have the
#	    correct version (or if it is under modification when
#	    -compress is used).
#
#	Build
#	    Build the module.
#
#	Clean
#	    Clean the module.
#
#	ShowVersions [<introot>] [<update>]
#	    Shows the current, last and configured versions of the
#	    module.  Shows what module version should be used.
#	    Optionally sets the version to the one that should be
#	    used (usually the last version in the Archive, unless
#	    the current version is "LOCAL" or if comment starts 
#	    with "LOCK" or "FORCE".
#
#   PUBLIC DATA
#	name:	    module name
#	version:    module version (default: empty = current or last
#		    module version, depending if the module is already
#		    retrieved from Archive or not).
#	make:	    make targets and options
#		    (default: "clean all man install").
#	subpkg:	    subsdirectory where the module should be retrieved
#		    (default: empty = current directory).
#	cpu:	    LCU CPU types (default: "MC68040 PPC604").
#		    This value is only used if the module contains
#		    VxWorks code.
#	options:    module options (default: no options).
#		    The following options (separated by spaces) are
#		    recognized:
#		    FULLCCS:  Build only for Full CCS.
#		    CCSLITE:  Build only for CCS-Lite.
#		    VXWORKS:  Build only if VxWorks is available.
#		    INSTNOVX: Install only if VxWorks is not available.
#		    SKIP:     Skip the complete module (no build).
#		    WS_ONLY:  Build only WS code.
#		    LCU_ONLY: Build only LCU code.
#		    TEST:     Relative to <module root> make files will
#			      be searched in:
#		              src ws/src lcu/src test ws/test lcu/test
#		    TEST_ONLY: Relative to <module root> make files will
#			      be searched in: test ws/test lcu/test
#		    IGNORE_WARNINGS: Ignore warnings.  This option is
#			      given to allow to build and install
#			      software that normally produces warnings.
#		    IGNORE_ERRORS: Ignore errors.  This option is given
#			      to allow to build and install software that
#			      is still under development.
#		              This option should normally not be present
#			      in the final release of the software.
#		    CLEAN:    After building all module targets, the 
#			      target clean is made.
#		    NOTEST:   do not include default test Makefile rule.
#	modules:    Software modules installed by the Makefile of this
#		    module.  Parameter only used by pkginINTROOT::Verify.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginMODULE
#------------------------------------------------------------------------

class pkginMODULE {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType MODULE
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
    	#*# 0. configure only -cpu if i is missing
	if { $i == "" } {
	    set txt {MC68040 PC604}
	    catch { set txt [GetEnv CPU] }
	    catch { set txt [$cfg Get INSTALL.MODULE.CPU] }
	    if {[lcontain $cpu ALL]} {
	        set txt {MC68040 PPC604}
 	    }
	    CheckOptions "INSTALL.MODULE.CPU" $txt {MC68040 MC68000 PPC604}
	    configure -cpu $txt
	    return
	}

	#*# 1. module name and index
	configure -name [$cfg Get INSTALL.MODULE$i.NAME]
	configure -index $i

	#*# 2. module version and comments
	#      "<version> [<comment>]"
	set txt "LAST"
	catch { set txt [$cfg Get INSTALL.MODULE$i.VERSION] }
	set txt2 [lvarpop txt]
	configure -cfgvers $txt2
	configure -version $txt2
	configure -comment $txt

	#*# 3. make options
	set txt "clean all man install"
	catch { set txt [$cfg Get INSTALL.MODULE$i.MAKE] }
	catch { set txt [$cfg Get INSTALL.MODULE.MAKE] }
	configure -make $txt

	#*# 4. sub-package
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE$i.SUBPKG] }
	configure -subpkg $txt

	#*# 5. LCU CPU type
	set txt {MC68040 PPC604}
	catch { set txt [GetEnv CPU] }
	catch { set txt [$cfg Get INSTALL.MODULE.CPU] }
	catch { set txt [$cfg Get INSTALL.MODULE$i.CPU] }
	if {[lcontain $cpu ALL]} {
	    set txt {MC68040 PPC604}
 	}
	CheckOptions "INSTALL.MODULE$i.CPU" $txt {MC68040 MC68000 PPC604}
	configure -cpu $txt

	#*# 6. module options
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE.OPTIONS] }
	CheckOptions "INSTALL.MODULE.OPTIONS" $txt {
	    FULLCCS CCSLITE VXWORKS VXWORKS_LCU INSTNOVX INSTNOVX_LCU
	    TEST TEST_ONLY WS_ONLY LCU_ONLY SKIP IGNORE_WARNINGS
	    IGNORE_WS_WARNINGS IGNORE_LCU_WARNINGS IGNORE_ERRORS
	    IGNORE_WS_ERRORS IGNORE_LCU_ERRORS CLEAN
	}
	catch { append txt " " [$cfg Get INSTALL.MODULE$i.OPTIONS] }
	CheckOptions "INSTALL.MODULE$i.OPTIONS" $txt {
	    FULLCCS CCSLITE VXWORKS VXWORKS_LCU INSTNOVX INSTNOVX_LCU
	    TEST TEST_ONLY WS_ONLY LCU_ONLY SKIP IGNORE_WARNINGS
	    IGNORE_WS_WARNINGS IGNORE_LCU_WARNINGS IGNORE_ERRORS
	    IGNORE_WS_ERRORS IGNORE_LCU_ERRORS CLEAN
	}
	configure -options $txt

	#*# 7. modules (optional)
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE$i.MODULES] }
	configure -modules $txt

	#*# 8. maxbak
	set txt F
	catch { set txt [$cfg Get INSTALL.FLAG.DELETEDIR] }
	set txt [expr {2 * (1 - [Logical2Num $txt])}]
	catch { set txt [$cfg Get INSTALL.CON.MAXBAK] }
	configure -maxbak $txt
    }

    #****************************************************************
    #*# method: Retrieve
    #----------------------------------------------------------------

    public method Retrieve { {compress 0} } {

    	try_eval {
	    Info "       Module  [format %-7s $name] " 0

	    #*# 1. if no version specified:
	    #      retrieve current or last version
	    set curVer   [GetCurrentVersion]
	    set modified [lindex $curVer 1]
	    set curVer   [lindex $curVer 0]
	    if { "$version" == "LOCAL" } {
	    	if { $compress != 0 } {
	            Info "[format %-10s $version] not allowed:          " 0
		    error "Module $name: local module version not allowed with option -compress."
		}
	        set path [file join $subpkg $name]
	        if {![file isdirectory $path]} {
	            Info "[format %-10s $version] $path not found: " 0
		    error "Module $name: local module version $path not found."
		}
	  	Info "[format %-10s $version] local version [format %-7s $curVer:] OK."
		return
	    } elseif { "$version" == "" || "$version" == "LAST" } {
	    	try_eval {
	            configure -version [lindex [GetLastVersion] 0]
		} {
	  	    Info "[format %-10s $version] may not exist:        " 0
		    error $errorResult $errorCode $errorInfo
	        }
	    }

    	    #*# 2. Only retrieve from the archive when the module is not already 
    	    #      existing in the parent directory, or the version is not correct.
	    #      If -compress, then also retrieve module, if current one is being
	    #      modified.
	    if { ($modified == 0 && $curVer != $version) ||
	         ($modified == 1 && $compress != 0) } {
	        Info "[format %-10s $version] retrieving ...        " 0

    	        #*# 3. if subpkg dir. does not exist: create it
	        if { "$subpkg" != "" } {
	            file mkdir $subpkg
	        }

	        #*# 4. if module exists: rename or delete it
	        set dir [file join $subpkg $name]
	        if {[file exists $dir]} {
                    if {$maxbak != 0 } {
		        if { $modified } {
		    	    set dirver "$dir-${curVer}+"
		        } else {
		    	    set dirver "$dir-$curVer"
		        }
			BackupFile $dir $maxbak $dirver
                    } else {
	                file delete -force $dir
	            }
	        }

	        #*# 5. cmmCopy
	        set curDir [pwd]
	        if { "$subpkg" != "" } {
	            cd $subpkg
	        }
	        if {[catch {exec cmmCopy $name $version} result]} {
	    	    cd $curDir
	            set result [lindex [split $result "\n"] 0]
	            error "Error retrieving module $name ($result)."
	        }
	        cd $curDir
	        Log $result

	    } elseif { $modified } {
	    	set version $curVer
	        Info "[format %-10s "$version+"] is being modified:    " 0
	    } else {
	        Info "[format %-10s $version] is already available: " 0
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: GetLastVersion
    #
    #   returns "<version> <modified_flag>"
    #----------------------------------------------------------------

    public method GetLastVersion {} {

	# get the version of the main trunk:
	# if there are branches, then this is the last line
	if {[catch {exec cmmLast $name | tail -1} result]} {
	    set result [lindex [split $result "\n"] 0]
	    error "Error retrieving version of module $name ($result)."
	}
    
        set lastVer [lindex $result 3]
	if { [lindex $result 4] == "modified" } {
	    set result [list $lastVer 1]
	} else {
	    set result [list $lastVer 0]
	}

	return $result
    }

    #****************************************************************
    #*# method: GetCurrentVersion
    #
    #   retrieve the version of the current module
    #   returns "<version> <modified_flag>"
    #----------------------------------------------------------------

    public method GetCurrentVersion {} {

        #*# 1. search version inside a Makefile
        set result ""
    	set fileList "src/Makefile ws/src/Makefile lcu/src/Makefile test/Makefile"
        foreach file $fileList {
	    set path [file join $subpkg $name $file]
	    if {[file isfile $path]} {
	        if {[catch {set result [exec ident $path]}]} {
		    set result "0 0 0 0"
		}
	        break
	    }
	}
	set curVer [lindex $result 3]

	#*# 2. separate "+": return "<version> <modified_flag>"
	set tmp $curVer
	set tmpCurVer ""
	while {"[set token [ctoken tmp +]]" != ""} {
	    append tmpCurVer $token 
	}
	if { "$curVer" != "$tmpCurVer" } {
	    set result [list $tmpCurVer 1]
	} else {
	    set result [list $curVer 0]
	}

	return $result
    }

    #****************************************************************
    #*# method: GetIntrootVersion
    #----------------------------------------------------------------

    public method GetIntrootVersion {} {

	#*# 1. search INTROOT vltMakeInstall.config file:
	set ret ""
	set filename [file join [GetEnv INTROOT] vltMakeInstall.config]
	if { ![file exists $filename] } {
	    return $ret
	}
	if { $modules == "" } {
	    set checkname $name
	} else {
	    set checkname [lindex $modules 0]
	}
	for_file line $filename {

	    #*# 2. skip comments and empty lines
	    set line [string trim $line]
	    if { $line == "" || [cindex $line 0] == "#" } {
		continue
	    }

	    #*# 3. get module and version
	    set modname [ctoken line " \t"]
	    if { [crange $modname end-3 end] == "-lcu" } {
		set modname [crange $modname 0 end-4]
	    } elseif { [crange $modname end-2 end] == "-ws" } {
		set modname [crange $modname 0 end-3]
	    }

	    #*# 4. if module found: return its version
	    if { $modname == $checkname } {
	        set modvers [ctoken line " \t"]
		if { $modvers == "installed" } {
		    set modvers "0"
		}
	    	set ret $modvers
		break
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: Build
    #----------------------------------------------------------------

    public method Build { {installOnly 0} } {

        #*# 1. if installOnly:
	#        if module not under modification and already installed: skip it.
	set buildMake $make
	set buildCpu  $cpu
        set ignoreWsWarnings  0
        set ignoreWsErrors    0
        set ignoreLcuWarnings 0
        set ignoreLcuErrors   0
	if { $installOnly } {
	    lassign [GetCurrentVersion] ver mod
	    if { $mod == 0 && [GetIntrootVersion] == $ver } {
	    	Info "       Skipping [format %-24s $name:] [format %-30s "already installed."] OK."
	    	return
	    }
	    # install ignoring warnings
	    set buildMake install
	    set buildCpu  [lindex $buildCpu 0]
 	    set ignoreWsWarnings  1
	    set ignoreWsErrors    1
 	    set ignoreLcuWarnings 1
	    set ignoreLcuErrors   1
	}

	#*# 2. init: retrieve all options
	if { "$buildMake" == "" } {
	    set buildMake "clean all man install"
	}
        set buildDir {src ws/src lcu/src}
        set cleanAfterBuild   0
	set buildWs           1
	set buildLcu          1
	set buildWsInfo       0
	set buildLcuInfo      0
        foreach option $options {
	    switch -- $option {
	        "CCSLITE" {
		    if {![ExistsEnv CCSLITE]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with CCSLITE. "]"
			return
		    }
		}
	        "FULLCCS" {
		    if {[ExistsEnv CCSLITE]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with FULLCCS. "]"
			return
		    }
		}
	        "VXWORKS" {
		    if {![ExistsEnv VXROOT]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with VXWORKS. "]"
			return
		    }
		}
	        "VXWORKS_LCU" {
		    if {![ExistsEnv VXROOT]} {
		    	set buildLcu 0
		    }
		}
	        "INSTNOVX" {
		    if {![ExistsEnv VXROOT]} {
		    	set buildMake install    
		    }
		}
	        "INSTNOVX_LCU" {
		    if {![ExistsEnv VXROOT]} {
		    	set buildLcu 2
		    }
		}
	        "WS_ONLY" {
		    set buildLcu 0
		}    
	        "LCU_ONLY" {
		    set buildWs 0
		}    
	        "TEST" {
		    set buildDir {src ws/src lcu/src test ws/test lcu/test}
		}    
	        "TEST_ONLY" {
		    set buildDir {test ws/test lcu/test}
		}    
	        "TESTONLY" {
		    set buildDir {test ws/test lcu/test}
		}    
	        "CLEAN" {
		    set cleanAfterBuild 1
		}
	        "IGNORE_WARNINGS" {
		    set ignoreWsWarnings  1
		    set ignoreLcuWarnings 1
		}
	        "IGNORE_ERRORS" {
		    set ignoreWsErrors  1
		    set ignoreLcuErrors 1
		}
	        "IGNORE_WS_WARNINGS" {
		    set ignoreWsWarnings  1
		}
	        "IGNORE_WS_ERRORS" {
		    set ignoreWsErrors  1
		}
	        "IGNORE_LCU_WARNINGS" {
		    set ignoreLcuWarnings 1
		}
	        "IGNORE_LCU_ERRORS" {
		    set ignoreLcuErrors 1
		}
	        "SKIP" {
	    	    Info "       Skipping [format %-24s $name:] [format %-30s "SKIP option set."] OK."
		    return
		}
	        default  {
	    	    Info "       Skipping [format %-24s $name:] [format %-30s "wrong option '$option': "] FAILURE."
		    set txt "ERROR: Module $name: Wrong build option '$option'.\n       Use: CCSLITE FULLCCS VXWORKS INSTNOVX TEST TESTONLY CLEAN IGNORE_ERRORS."
		    LogErr $txt
		    error "$txt\n"
		}
	    }
        }

	#*# 3. set targets list
	set targetsList [list $buildMake]
	if { $cleanAfterBuild } {
	    lappend targetsList clean
	}

	#*# 4. build module
	set errList ""
	set curDir [pwd]
	set build 0
	try_eval {

	    #*# 5. foreach target:
	    foreach targets $targetsList {

	    	#*# 6. foreach Makefile:
	        foreach subDir $buildDir {
		    set dir [file join $curDir $subpkg $name $subDir]
		    set makeFile [file join $dir Makefile]
		    if { ![file isfile $makeFile] } {
		        continue
		    }
	   	    set build 1

		    #*# 7. skip "ws" or "lcu" type directories
		    cd $dir
		    set isLcuDir [IsLcuModule "Makefile"]
		    set subDirType [file dirname $subDir]
		    if { [lsearch {"ws" "lcu"} $subDirType] == -1 } {
			if { $isLcuDir } {
			    set subDirType "lcu"
			} else {
			    set subDirType "ws"
			}
		    }
		    if { $subDirType == "ws" && $buildWs == 0 } {
			if { $buildWsInfo == 0 } {
			    Info "       Skipping [format %-24s "$name WS directories:"] [format %-30s "LCU_ONLY option set."] OK."
			    set buildWsInfo 1
			}
			continue
		    }
		    if { $subDirType == "lcu" && $buildLcu == 0 } {
			if { $buildLcuInfo == 0 } {
			    Info "       Skipping [format %-24s "$name LCU directories:"] [format %-30s "WS_ONLY or VXWORKS_LCU set."] OK."
			    set buildLcuInfo 1
			}
		    	continue
		    }

		    #*# 8. if LCU module: build for each LCU architecture
		    #      if WS  module: build once
		    if { $subDirType == "lcu" } {
		    	set ignoreWarnings $ignoreLcuWarnings
		    	set ignoreErrors   $ignoreLcuErrors
		    } else {
		    	set ignoreWarnings $ignoreWsWarnings
		    	set ignoreErrors   $ignoreWsErrors
		    }
		    if { $isLcuDir } {
			set lcuTargets $targets
			if { $buildLcu == 2 && $lcuTargets != "clean" } {
			    set lcuTargets "install"
			}
		        foreach cpuType $buildCpu {
			    SetEnv CPU $cpuType
			    set txt "Building [format %-24s "$name/$subDir, CPU=[format %-8s $cpuType:]"] [format %-30s "make $lcuTargets ..."] "
			    Info "       $txt" 0
			    Log  ""
			    LogErr $txt
			    try_eval {
			    	Make $makeFile $lcuTargets $ignoreWarnings $ignoreErrors
			    } {
			        append errList "ERROR: $errorResult\n"
			    }
			}
		    } else {
		        set txt "Building [format %-24s "$name/$subDir:"] [format %-30s "make $targets ..."] "
			Info "       $txt" 0
		        Log  ""
			LogErr $txt
			try_eval {
		            Make $makeFile $targets $ignoreWarnings $ignoreErrors
			} {
			    append errList "ERROR: $errorResult\n"
			}
		    }
	        }
	    }
	} {
	    error $errorResult $errorCode $errorInfo
	} {
	    cd $curDir
	}

	#*# 9. return errors
	if { !$build } {
	    Info "       Skipping [format %-24s $name:] [format %-30s "no Makefile found:"] FAILURE."
	    set txt "ERROR: Module $name: no Makefile found."
	    LogErr $txt
	    append errList "$txt\n"
	}
	if { "$errList" != "" } {
	    error $errList
	}
    }

    #****************************************************************
    #*# method: Clean
    #----------------------------------------------------------------

    public method Clean {} {

	#*# 1. clean module
	set errList ""
	set clean 0
	set buildDir {src ws/src lcu/src test ws/test lcu/test}
	set curDir [pwd]
	try_eval {

	    #*# 2. foreach Makefile: clean
	    foreach subDir $buildDir {
		set dir [file join $curDir $subpkg $name $subDir]
		set makeFile [file join $dir Makefile]
		if {![file isfile $makeFile] || ![file isdirectory $dir]} {
		    continue
		}
		cd $dir
                # ALO: If we are running tat, a make clean would break it
                if { [file isdirectory tatlogs] } {
		    continue
		}
		Info "       Cleaning [format %-24s "$name/$subDir:"] make clean ...     " 0
		Log  ""
		try_eval {
		    Make $makeFile clean
		} {
		    append errList "ERROR: $errorResult\n"
		}
		set clean 1
	    }
	} {
	    error $errorResult $errorCode $errorInfo
	} {
	    cd $curDir
	}

	#*# 3. return errors
	if { !$clean } {
	    Info "       Skipping [format %-24s $name:] no Makefile found: FAILURE."
	    set txt "ERROR: Module $name: no Makefile found."
	    LogErr $txt
	    append errList "$txt\n"
	}
	if { "$errList" != "" } {
	    error $errList
	}
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {
	    Info "       Verifying [format %-24s $name:] OK."
    }

    #****************************************************************
    #*# method: Make
    #----------------------------------------------------------------

    protected method Make { makeFile targets {ignoreWarnings 0} {ignoreErrors 0} } {

    	#*# 1. execute make and check for errors
	try_eval {
	    set opt ""
	    if { $loglevel >= 2 } {
	        append opt " MAKE_VERBOSE=ON"
	    }
	    set makeComment ""
	    System "make $opt $targets" "$this MakeError %e $ignoreWarnings"
            Info "OK$makeComment."
	} {
	    if { $ignoreErrors } {
	        Log  "ERROR: Errors occurred building $makeFile:"
		Log  "       $errorResult"
		Log  "       The errors are ignored due to the IGNORE_ERRORS flag."
		Info "OK (errors ignored)."
	    } else {
	        Log  "ERROR: Errors occurred building $makeFile:"
		Log  "       $errorResult"
		Log  "       The detailed log has been placed in the error log file."
		Info "FAILURE."
	    	error "Module $name: $errorResult" $errorCode $errorInfo
	    }
	}
    }

    #****************************************************************
    #*# method: MakeError
    #
    #   Scans the make output for errors.
    #   If an error is found, then an error is raised
    #	with the first line that contains an error.
    #----------------------------------------------------------------

    public method MakeError { makeOutput ignoreWarnings } {

    	#*# 1. scan all output lines:
	set errLine ""
	set procheckWarns {"(warn" "(bad" "(non" "(num" "(arg" "(array" "(invalid" \
			   "(mismatch" "(noExpr" "(noS" "(obsoleteCmd" "(parse" \
			   "(proc" "(tooMany" "(win" "(blt::" "(coreTcl::" "(coreTk::" \
			   "(expect::" "(incrTcl::" "(oratcl::" "(tclX::" "(xmlAct::" }
        while {1} {
	    set line [ctoken makeOutput "\n"]

	    #*# 2. When the strings "make" and "Error" occur in the same line and the 
	    #      make result is not ignored (ignored), a make error has occurred.
	    set ignoredIndex [string first "(ignored)" $line]
	    set errorIndex [string first "Error" $line]
	    set makeIndex [string first "make" $line]
	    set warningIndex [string first ": warning:" $line]
	    foreach txt $procheckWarns {
	        if { $warningIndex != -1 } {
		    break
		}
	        set warningIndex [string first $txt $line]
	    }
	    if {$line == ""} {
	    	break
	    } elseif { $ignoredIndex == -1 && $errorIndex != -1 && $makeIndex != -1} {
	    	set errLine $line
		break
	    } elseif { $ignoreWarnings == 0 && $warningIndex != -1 } {
	    	set errLine $line
	    } elseif { $ignoreWarnings != 0 && $warningIndex != -1 } {
	    	set makeComment " (warnings ignored)"
	    }

	    #*# 3. for test rule: check for "No test found." 
	    if { $makeComment == "" && [string first "No test found" $line] != -1 } {
	    	set makeComment " (no test)"
	    }
        }

	if { "$errLine" != "" } {
	    error $errLine
	}
    }

    #****************************************************************
    #*# method: IsLcuModule
    #----------------------------------------------------------------

    public method IsLcuModule { makeFile } {

	set trimmedLine ""
        for_file line $makeFile {
	    set token [ctoken line " "]
	    if {"[string index $token 0]" != "#"} {
	        if {[string first "MAKE_VXWORKS" "$token$line"] != -1} {
		    append trimmedLine $token
		    while {[set token [ctoken line " "]] != ""} {
		        append trimmedLine $token
		    }
		    if {[string first "MAKE_VXWORKS=on" $trimmedLine] != -1} {
		        return 1
		    }
	        }
	    }
        }

    	return 0
    }

    #****************************************************************
    #*# method: ShowVersions
    #----------------------------------------------------------------

    public method ShowVersions { {introot 0} {update 0} } {

	#*# 1. retrieve config., current, INTROOT and last versions
	if { "$version" == "" } {
	    set vers "LAST"
	} else {
	    set vers $version
	}
	lassign [GetCurrentVersion] currvers currmod
	if { "$currvers" == "" } {
	    set currvers "-"
	}
	lassign [GetIntrootVersion] introotvers introotmod
	if { "$introotvers" == "" } {
	    set introotvers "-"
	}
	if { "$introotmod" == "" } {
	    set introotmod 0
	}
	try_eval {
	    lassign [GetLastVersion] lastvers lastmod
	} {
	    set lastvers "-"
	    set lastmod 0
	}

	#*# 2. new version
	#
	# If the configured version is LOCAL: do not modify it.
	#
	# If compare with INTROOT: then modify the module version
	# if INTROOT and configured versions do not match.
	#
	# Otherwise compare with the Archive version:
	# If the version is not FIXED or LOCAL, and there is a last
	# version in the Archive, then modify the module version;
	# otherwise keep the current version.
	#
	if { [string first "LOCK" $comment] == 0 || \
	     [string first "FORCE" $comment] == 0 } {
	    set isfixed 1
	} else {
	    set isfixed 0
	}
	set updatevers $vers
	if { "$vers" != "LOCAL" } {
	    if { $introot == 0 } {
	        if { $isfixed == 0 && "$lastvers" != "-" } {
	            set updatevers $lastvers
	        }
	    } elseif { "$introotvers" != "-" } {
	        set updatevers $introotvers
	        if { "$vers" == "LAST" && "$introotvers" == "$lastvers" } {
	    	    set updatevers "LAST"
	    	}
	    }
	}

	#*# 3. set comment; show arrow, if version is updated.
	set txt $comment
	set retrieve 0
	if { ("$vers" == "LAST" && "$lastvers" != "$currvers") || \
	     ("$vers" != "LAST" && "$vers" != "LOCAL" && "$vers" != "$currvers") } {
	    if { "$lastvers" != "-" } {
		set retrieve 1
	    }
	}
	if { $introot == 0 && $retrieve == 0 && \
	     (($introotvers != $currvers && $introotvers != $lastvers) || \
	      $introotvers == "-") } {
	    set txt "INSTALL $txt"
	}
	if { $retrieve } {
	    set txt "RETRIEVE $txt"
	}
	if { "$vers" != "$updatevers" } {
	    set arrow "->"
	    set txt "UPDATE $txt"
	} else {
	    set arrow "  "
	}
	if { ("$currvers" == "-" && "$lastvers" == "-") || \
	     ("$vers" == "LAST"  && "$lastvers" == "-") || \
	     ("$vers" == "LOCAL" && "$currvers" == "-") } {
	    set txt "??? $txt"
	}

	#*# 4. show versions
	if { $currmod } {
	    append currvers "+"
	}
	if { $introotmod } {
	    append introotvers "+"
	}
	if { $currvers != $introotvers } {
	    set currvers "$currvers/$introotvers"
	}
	if { $lastmod } {
	    append lastvers "+"
	}
	Info "[format %-8s $name] [format %-16s $currvers] [format %-9s $lastvers] [format %-9s $vers] $arrow [format %-9s $updatevers] $txt"

	#*# 5. update
	if { $update && $index != 0 } {
	    set cfg [[Repository] Contents CONFIG]
	    $cfg Set INSTALL.MODULE$index.VERSION $updatevers
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable cfgvers ""
    public variable version ""
    public variable make    ""
    public variable subpkg  ""
    public variable cpu     ""
    public variable options ""
    public variable modules ""
    public variable maxbak  2
    public variable comment ""
    public variable index   0

    protected variable makeComment ""
}


#************************************************************************
#___oOo___

