#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginPACKAGE.tcl,v 1.105 2005/10/12 19:47:32 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-10-04  INSTALL.ENVS.BUILD added.
# rschmutz  2004-10-04  skip empty INSTALL.*.NAME entries.
# rschmutz  2004-06-05  INSTALL.ENVS.AVAIL support added.
# rschmutz  2004-06-02  Library support added.
# rschmutz  04/01/03  index added to scanlink name (to be able to diff.).
# rschmutz  26/03/02  inherit from pkginBASE (instead of pkginLOG).
# rschmutz  26/06/01  file sets added.
# rschmutz  14/05/01  envvars attr. added.
# rschmutz  12/05/01  ALL_ENVIRONMENTS container added for pkginInstall.
# rschmutz  08/12/00  check INSTALL.DETi.TYPE instead of *.CCDNAME.
# rschmutz  23/11/00  limits increased to 30.
# rschmutz  15/09/00  DCS CFGFILE -> CFGSET.
# rschmutz  13/05/00  created.
#

#************************************************************************
#   NAME
#	pkginPACKAGE - VLT software package
# 
#   SYNOPSIS
#	pkginPACKAGE name
# 
#   DESCRIPTION
#	pkginPACKAGE describes a VLT software package.
#
#   PARENT CLASS
#	ctooOBJ_CONTAINER, pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Configure the package.
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: pkginPACKAGE
#------------------------------------------------------------------------

class pkginPACKAGE {
    inherit ctooOBJ_CONTAINER pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	configure -objType PACKAGE
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {
        global pkginType

	#*# 1. name
	configure -name [$cfg Get INSTALL.PACKAGE.NAME]

	#*# 2. required env. variables
	catch { configure -envvars [$cfg Get INSTALL.PACKAGE.ENVVARS] }

	#*# 3. config. archive name and module
	catch { configure -cfgname [$cfg Get INSTALL.PACKAGE.CFGNAME] }

	#*# 4. usermgr, user
	catch { configure -usermgr [$cfg Get INSTALL.LOGINNAME.USERMGR] }
	catch { configure -user    [$cfg Get INSTALL.LOGINNAME.USER] }

	#*# 5. initscript
	catch { configure -initscript [$cfg Get INSTALL.INITIAL.SCRIPT] }

	#*# 6. load modules
	#      (default types: pkginMODULE, pkginLIBRARY)
	set modlist [[[Repository] Contents INSTALL] cget -mod]
	set container [ctooOBJ_CONTAINER $this.modules \
		      -objType MODULES -name $name]
	AddObj $container
	set i 1
	set lastIndex $maxModules
	while { $i <= $maxModules || $i <= $lastIndex } {
	    set modname ""
	    catch { set modname [$cfg Get INSTALL.MODULE$i.NAME] }
	    if { $modname != "" } {
	    	if { "$modlist" == "" || [lsearch $modlist $modname] != -1 } {
		    set modType "MODULE"
		    if { ![catch { set modOptions [$cfg Get INSTALL.MODULE$i.OPTIONS] }]} {
		    	if { [lsearch $modOptions "LIBRARY"] != -1 } {
			    set modType "LIBRARY"
			}
		    }
	    	    set obj [$pkginType($modType) $this.module$i]
		    $container AddObj $obj
		    $obj Config $cfg $i
		    set lastIndex [expr {$i + 50}]
		}
	    }
	    incr i
	}

	#*# 7. load environments
	#      (default types: pkginRTAPENV, pkginLCUENV)
	#
	#  All environments are registered in the container
	#  ALL_ENVIRONMENTS (this is needed to backup all
	#  environments before untarring a package).
	#
	#  All available environments are registered in the
	#  AVAIL_ENVIRONMENTS container.
	#
	#  Only register in the ENVIRONMENTS container the environments
	#  to be built:
	#  If INSTALL.ENVS.AVAIL (or command-line option -env) is not
	#  empty, then only register in the ENVIRONMENTS container the 
	#  environments listed.
	#
	set availlist ""
	catch { set availlist [$cfg Get INSTALL.ENVS.AVAIL] }
	set envlist [[[Repository] Contents INSTALL] cget -env]
	if { $envlist == "" } {
	    set envlist $availlist
	}
	set all_container [ctooOBJ_CONTAINER $this.all_envs \
		      -objType ALL_ENVIRONMENTS -name $name]
	AddObj $all_container
	set avail_container [ctooOBJ_CONTAINER $this.avail_envs \
		      -objType AVAIL_ENVIRONMENTS -name $name]
	AddObj $avail_container
	set container [ctooOBJ_CONTAINER $this.envs \
		      -objType ENVIRONMENTS -name $name]
	AddObj $container
	loop i 1 [expr {$maxEnvs + 1}] {
	    set envname ""
	    catch { set envname [$cfg Get INSTALL.RTAPENV$i.NAME] }
	    if { $envname != "" } {
	    	set obj [$pkginType(RTAPENV) $this.rtapenv$i]
		$all_container AddObj $obj
		if { "$availlist" == "" || [lsearch $availlist $envname] != -1 } {
		    $avail_container AddObj $obj
		}
		if { "$envlist" == "" || [lsearch $envlist $envname] != -1 } {
		    $container AddObj $obj
		}
		$obj Config $cfg $i
	    }
	}
	loop i 1 [expr {$maxEnvs + 1}] {
	    set envname ""
	    catch { set envname [$cfg Get INSTALL.LCUENV$i.NAME] }
	    if { $envname != "" } {
	    	set obj [$pkginType(LCUENV) $this.lcuenv$i]
		$all_container AddObj $obj
		if { "$availlist" == "" || [lsearch $availlist $envname] != -1 } {
		    $avail_container AddObj $obj
		}
		if { "$envlist" == "" || [lsearch $envlist $envname] != -1 } {
		    $container AddObj $obj
		}
		$obj Config $cfg $i
	    }
	}
	if { $availlist == "" } {
	    $cfg Set INSTALL.ENVS.AVAIL [$avail_container ContentsByName]
	}
	$cfg Set INSTALL.ENVS.BUILD [$container ContentsByName]

	#*# 8. load plugins
	#      (default type: pkginPLUGIN)
	set container [ctooOBJ_CONTAINER $this.hooks \
		      -objType PLUGINS -name $name]
	AddObj $container
	loop i 1 [expr {$maxHooks + 1}] {
	    set hookname ""
	    catch { set hookname [$cfg Get INSTALL.HOOK$i.NAME] }
	    if { $hookname != "" } {
	    	set obj [$pkginType(PLUGIN) $this.hook$i]
		$container AddObj $obj
		$obj Config $cfg $i
	    }
	}

	#*# 9. load detectors
	#      (default type: pkginDCS)
	set container [ctooOBJ_CONTAINER $this.detectors \
		      -objType DETECTORS -name $name]
	AddObj $container
	loop i 1 [expr {$maxDcs + 1}] {
	    set detname ""
	    catch { set detname [$cfg Get INSTALL.DET$i.NAME] }
	    if { $detname != "" } {
	        set i1 [catch { $cfg Get INSTALL.DET$i.CFGSET }]
	        set i2 [catch { $cfg Get INSTALL.DET$i.TYPE }]
	        if { $i1 == 0 && $i2 == 0 } {
	    	    error "Either INSTALL.DET$i.CFGSET or INSTALL.DET$i.TYPE may be used.  Not both."
	        }
	    	set obj [$pkginType(DCS) $this.dcs$i]
		$container AddObj $obj
		$obj Config $cfg $i
	    }
	}

	#*# 10. load scanlinks
	#      (default type: pkginSCANLINKS)
	set container [ctooOBJ_CONTAINER $this.scanlinks \
		      -objType SCANLINKS -name $name]
	AddObj $container
	loop i 1 [expr {$maxScanlinks + 1}] {
	    set envname ""
	    catch { set envname [$cfg Get INSTALL.SCAN$i.REMOTEENV] }
	    if { $envname != "" } {
                # register scanlink only if the remote
                # environment is listed in envlist.
		if { "$envlist" == "" || ($envname != "" && [lsearch $envlist $envname] != -1) } {
	    	    set obj [$pkginType(SCANLINK) $this.scanlink$i]
		    $container AddObj $obj
		    $obj Config $cfg $i
		}
	    }
	}

	#*# 11. load file sets
	#      (default type: pkginFILESET)
	set container [ctooOBJ_CONTAINER $this.filesets \
		      -objType FILESETS -name $name]
	AddObj $container
	loop i 1 [expr {$maxFileSets + 1}] {
	    set fsetname ""
	    catch { set fsetname [$cfg Get INSTALL.FILES$i.NAME] }
	    if { $fsetname != "" } {
	    	set obj [$pkginType(FILESET) $this.fileset$i]
		$obj Config $cfg $i
		# register file set only if SRCHOST is empty
		# or equal to the current host.
		set srchost [$obj cget -srchost]
	    	if { $srchost == "" || $srchost == [GetEnv HOST] } {
		    $container AddObj $obj
		} else {
		    delete object $obj
		}
	    }
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable envvars    ""
    public variable cfgname    ""
    public variable usermgr    ""
    public variable user       ""
    public variable initscript ""

    public variable maxModules  100
    public variable maxHooks     30
    public variable maxEnvs      30
    public variable maxDcs       30
    public variable maxScanlinks 30
    public variable maxFileSets  30

}


#___oOo___
