#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginVLTDATA.tcl,v 1.105 2005/10/12 19:47:34 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-01-28  SPR VLTSW20050032: remove trailing '/'.
# rschmutz  14/05/01  DiskSpace method added.
# rschmutz  17/09/00  Verify method added.
# rschmutz  28/05/00  created.
#

#************************************************************************
#   NAME
#	pkginVLTDATA - VLTDATA root
#
#   SYNOPSIS
#	pkginVLTDATA name
#
#   DESCRIPTION
#	pkginVLTDATA contains the environment directories a.o.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Create
#	    Creates a VLTDATA directory that contains only the
#	    ENVIRONMENT and config subdirectories (used by
#	    'pkginBuild -compress' only).
#
#	Backup
#	    Dummy method.
#
#	Verify
#	    Dummy method.
#
#   PUBLIC DATA
#	dir:	VLTDATA directory
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginVLTDATA
#------------------------------------------------------------------------

class pkginVLTDATA {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	global env

	set txt ""
	catch { set txt $env(VLTDATA) }
	if { $txt != "" } {
		    # remove trailing '/'
	    set txt [eval file join [file split $txt]]
	}
	configure -objType ROOT -name VLTDATA -dir $txt
	eval configure $args
    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create { {dummy 0} } {

	if {[file exists $dir]} {
	    return
	} else {
	    Info "       VLTDATA:  creating $dir ... " 0
	    try_eval {
		file mkdir $dir
		file mkdir [file join $dir ENVIRONMENTS]
		file mkdir [file join $dir config]
	    } {
		LogErr $errorResult
		Info "FAILURE."
		error $errorResult $errorCode $errorInfo
	    }
	    Info "OK."
	}
    }

    #****************************************************************
    #*# method: Backup
    #----------------------------------------------------------------

    public method Backup {} {
	# nothing to do: each environment backups itself.
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {
	# nothing to do: each environment checks itself.
    }

    #****************************************************************
    #*# method: DiskSpace
    #----------------------------------------------------------------

    public method DiskSpace {} {
	return [GetDiskSpace $dir]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable dir ""
    public variable maxbak 2
    public variable size 0
}


#************************************************************************
#___oOo___

