#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: plotMisc.tcl,v 2.40 2005/08/09 07:50:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  03/05/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# This procedure notifies RtapPlotDisplay that some plot configuration
# parameters have changed. RtapPlotDisplay will then update the display
# according to the new values.

proc plotNotifyChange {} {
   global gvar plotRTAPENV plotTABLE

   panDbWrite "@$gvar(envName)${plotTABLE}.updateDisplay" 0

   return 
}


proc plotDisableScalModes { w } {
   $w disableEntry 1
   $w disableEntry 4
}


proc plotSetEnvironment {} {
   global plotConfigAttr plotPenAttributes plotENV plotTraitFields \
          plotRTAPENV plotTable plotTRAITS plotTABLE traitRecId penId \
          plotUPDATE

   set c [string index $plotTable 0]

   if {![cequal $c ":"] && ![cequal $c "<"]} {
      set plotTABLE "<alias>${plotTable}"
   } else {
      set plotTABLE  "${plotTable}"
   }

   set plotTRAITS "$plotTABLE.traits"
   set plotUPDATE "$plotTABLE.updateDisplay"

   set plotConfigAttr [ list windowTitle title xAxisVisible xAxisLabel \
                        xAxisStyle xAxisScalingMode xAxisMinima xAxisMaxima \
                        xAxisDivisions ]

   set plotPenAttributes [ list visible sampleRate gridStyle gridVisible \
                           yAxisSelect yAxisVisible yAxisLabel yAxisColor \
                           yAxisStyle yAxisScalingMode yAxisMinima \
                           yAxisMaxima yAxisDivisions traitAttr yAxisSelect \
                           yAttrOrSourceAddr dataSourceType xAttrOrHistTable ]

   set plotTraitFields  [list markerVisible markerStyle markerColor \
                              lineVisible  lineStyle lineColor ]

}
 
proc plotSetScalingModeX {mode} {
   global cv gvar

   set w $cv.wdgTimeSpan
   if {$mode != 1} {
      # Enable Manual X range specification
      $cv.wdgMinAxis  disable
      $cv.wdgMaxAxis  disable
      if {[winfo exists $w]} { $w  enable }
   } else {
      $cv.wdgMinAxis  enable
      $cv.wdgMaxAxis  enable
      if {[winfo exists $w]} { $w  disable }
   }
}

proc plotSetScalingModeY {mode} {
   global cv gvar

   if {$mode != 1} {
      $cv.wdgMinimumY  disable
      $cv.wdgMaximumY  disable
   } else {
      $cv.wdgMinimumY  enable
      $cv.wdgMaximumY  enable
   }
}

proc plotSetUpdateMode {mode} {
   global cv gvar

   if {![winfo exists $cv.wdgSamplingRate]} { return }

   if {$mode == 0} {
      # Enable Manual X range specification
      $cv.wdgSamplingRate disable
   } else {
      $cv.wdgSamplingRate enable
   }
}

proc plotSetGridStyle {style} {
   global cv gvar

   if {$style == 0} {
      set gvar(gridVisible) 0
   } else {
      set gvar(gridVisible) 1 
   }
}

# This procedure sets all the color attributes of the curve
# to the same value.

proc plotSetColor { color } {
   global cv gvar

   $cv.wdgPenColor configure -background $color

#   set w [$cv.wdgPenId component uifOutput]
#   $w configure -foreground $gvar(yAxisColor)

   set gvar(gridColor)   $color
   set gvar(markerColor) $color
   set gvar(lineColor)   $color
   set gvar(textColor)   $color
}

# Configure pen colors

proc plotSetPenConfig {} {
   global plotTABLE cv gvar

   # Get number of active PENs

   set tableInfo [seq_dbGetAttrInfo "@$gvar(envName)${plotTABLE}.curves"]
   set numbPen   [lindex $tableInfo 2]
   set recCnt    [expr {$numbPen -1}]

   panDbRead "@$gvar(envName)${plotTABLE}.curves(0:$recCnt,yAxisColor)"  yAxisColor
   panDbRead "@$gvar(envName)${plotTABLE}.curves(0:$recCnt,visible)"     visibleFlags
   panDbRead "@$gvar(envName)${plotTABLE}.curves(0:$recCnt,yAxisSelect)" yAxisSelect

#   puts "numbPen      :  $numbPen"
#   puts "yAxisColor   :  $yAxisColor"
#   puts "visibleFlags :  $visibleFlags"
#   puts "yAxisSelect  :  $yAxisSelect"

   set j 0

   while {$j < $numbPen} {
      set w "$cv.wdgPen${j}"
      set fg [plotGetTraitAttr $j lineColor 0]
      $w configure -font "-Adobe-helvetica-bold-r-normal--14*"  -foreground $fg
      set gvar(yAxisSelect${j}) [lindex $yAxisSelect $j]
      incr j
   }

   # Disable NOT-USED PENs

   for {set i 0} {$i < 8} {incr i} {
       if {[expr {$i + 1}] <= $numbPen} {
         set gvar(enablePen${i}) 1       ;# Enable Pen
         $cv.wdgVisible${i} configure    -state normal
         $cv.wdgAxisSelect${i} configure -state normal
      } else {
         set gvar(enablePen${i}) 0       ;# Disable Pen
         $cv.wdgVisible${i} configure    -state disabled
         $cv.wdgAxisSelect${i} configure -state disabled
      }
   }

   # Set Visible PENs.

   set i 0

   foreach val $visibleFlags {
#      puts "plotSetPenColors gvar(visibleFlag${i}) $val"
      set gvar(visibleFlag${i}) $val
      incr i
   }
}

proc plotSetUpdateEvent { procName } {
   global gvar plotUPDATE plotUPDATE_EVT

   if {[catch {seq_evtAttach "@$gvar(envName)$plotUPDATE" "w" $procName } ret] } {
      puts "Cannot Attach Event : $ret"
   }
   set plotUPDATE_EVT $ret
}

proc plotSelectPlotTable {} {
   global gvar plotTABLE plotTRAITS plotUPDATE plotUPDATE_EVT

   if {[catch {seq_evtDetach $plotUPDATE_EVT}] } {
      puts "Cannot Detach Attach Event : $plotUPDATE_EVT"
   }

   set c [string index $gvar(currentPlotTable) 0]

   if {![cequal $c ":"] && ![cequal $c "<"]} {
       set gvar(currentPlotTable) "<alias>$gvar(currentPlotTable)"
   }

   set plotTABLE  "$gvar(currentPlotTable)"
   set plotTRAITS "$plotTABLE.traits"
   set plotUPDATE "$plotTABLE.updateDisplay"

   plotSetUpdateEvent plotReadTPConfig
   plotReadTPConfig
}

proc plotReadPen {penId} {
    global plotRTAPENV plotTable gvar traitRecId plotTABLE

    if {[cequal $penId ""]} { set penId  [expr {$gvar(penId) -1}] }
    
    set traitRecId [plotGetTraitRecord $penId]
    
    plotReadPenConfig $penId
    
    set gvar(penId) [expr {$penId + 1}]
}

##############################################################
#       Set of simple procs common to several panels 
##############################################################

proc setNoShareAxis {} {
   global gvar

   for {set i 0} {$i < 8} { incr i} {
       set gvar(yAxisSelect${i}) -1
   }
}

proc setAllPenVisible {} {
   global gvar

   for {set i 1} {$i <= 8} { incr i} {
       set gvar(visibleFlag${i}) 1
   }
}


proc plotSetPenScaling { plotPoint mode } {
   global gvar

    if {[string match "<alias>*" $plotPoint] == 0} {
       set plotPoint "<alias>$plotPoint"
    }

   set tableInfo [seq_dbGetAttrInfo "${plotPoint}.curves"]
   set numbPen   [lindex $tableInfo 2]

    for {set i 0} {$i < $numbPen} { incr i} {
	panDbWrite "${plotPoint}.curves($i,yAxisScalingMode)" $mode
   }

   panDbWrite "${plotPoint}.updateDisplay" 0
}


proc plotSetNoShareAxis { plotPoint } {
   global gvar

    if {[string match "<alias>*" $plotPoint] == 0} {
       set plotPoint "<alias>$plotPoint"
    }

    set tableInfo [seq_dbGetAttrInfo "${plotPoint}.curves"]
    set numbPen   [lindex $tableInfo 2]

    for {set i 0} {$i < $numbPen} { incr i} {
	panDbWrite "${plotPoint}.curves($i,yAxisSelect)" -1
    }

   panDbWrite "${plotPoint}.updateDisplay" 0
}




##############################################################
#           procs for FFT configuration
##############################################################


proc plotSetFFTDest { pt } {

    if {[cequal $pt ""]} { return SUCCESS }

    set c [string index $pt 0]

    if {![cequal $c "<"] && ![cequal $c ":"]} {
	set pt "<alias>$pt"
    }

    # Check if point exists

    if {[catch {seq_dbGetAttrInfo "$pt.curves"} attrInfo]} {
       seq_logData "Plot Point $pt does not exist"
       return FAILURE
    }

    set numPENS [lindex $attrInfo 2]

    if {$numPENS < 4} {
       seq_logData "Plot Point $pt must have at least 4 PENS : actual $numPENS"
       return FAILURE
    } 

    # Check if point has the right properties

    if {[catch {seq_dbGetAttrInfo "$pt:plot_table.data"} attrInfo]} {
       seq_logData "Plot Point $pt:plot_table.data does not exist"
       return FAILURE
    }

    set numFIELDS [lindex $attrInfo 3]
    set numRECS   [lindex $attrInfo 2]

    if {$numFIELDS < 6} {
       seq_logData "Plot Table $pt:plot_table.data must have at least 6 fields : actual $numFIELD"
       continue
    } 

    # Get direct addresses for plot table indexes

    set attrName "${pt}:plot_table"

    if {[catch {seq_dbGetDirAddr ${attrName}.firstRecord} firstIdx]} {
	set firstIdx "@\[\]"
	seq_logData "Cannot Get direct address for ${attrName}.firstRecord"
	return FAILURE
    }

    if {[catch {seq_dbGetDirAddr ${attrName}.nextRecord} nextIdx]} {
	set oldestIdx "@\[\]"
	seq_logData "Cannot Get direct address for ${attrName}.nextRecord"
	return FAILURE
    }

    if {[catch {seq_dbGetDirAddr ${attrName}.oldestRecord1} oldestIdx]} {
	set nextIdx "@\[\]"
	seq_logData "Cannot Get direct address for ${attrName}.oldestRecord1"
        return FAILURE
    }

    set idx 0
    while {$idx < 4} {
	panDbWrite  "${pt}.curves($idx,firstIndexerAddr)"   $firstIdx
	panDbWrite  "${pt}.curves($idx,oldestIndexerAddr)"  $oldestIdx
	panDbWrite  "${pt}.curves($idx,nextIndexerAddr)"    $nextIdx
	panDbWrite  "${pt}.curves($idx,visible)"            1
        incr idx
    }

    # Set direct addresses for the plot curves

    incr numRECS -1
    set idx 0

    while {$idx < 6} {
        set attrName "${pt}:plot_table.data(0:$numRECS,$idx)"
	if {[catch {seq_dbGetDirAddr $attrName} dirAddr($idx)]} {
            set dirAddr($idx) "@\[\]"
	    puts "Cannot Get direct address for $attrName"
	    return FAILURE
	}
        incr idx
    }
   
    # PEN 1 displays the input signal
    panDbWrite ${pt}.curves(0,xAttrOrHistTable)  $dirAddr(0)
    panDbWrite ${pt}.curves(0,yAttrOrSourceAddr) $dirAddr(1)

    # PEN 2 displays FFT Magnitude
    panDbWrite ${pt}.curves(1,xAttrOrHistTable)  $dirAddr(2)
    panDbWrite ${pt}.curves(1,yAttrOrSourceAddr) $dirAddr(3)

    # PEN 3 displays FFT Spectrum
    panDbWrite ${pt}.curves(2,xAttrOrHistTable)  $dirAddr(2)
    panDbWrite ${pt}.curves(2,yAttrOrSourceAddr) $dirAddr(4)

    # PEN 4 displays FFT Phase
    panDbWrite ${pt}.curves(3,xAttrOrHistTable)  $dirAddr(2)
    panDbWrite ${pt}.curves(3,yAttrOrSourceAddr) $dirAddr(5)

    return SUCCESS
}

#
# ___oOo___
