#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: plotRWConfiguration.tcl,v 2.40 2005/08/09 07:50:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  03/05/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# This procedure reads the PEN configuration parameters and for some
# of them calls an additional procedure to properly set the apperance
# of the GUI

proc plotReadPenConfig { penId } {
   global gvar plotPenAttributes plotRTAPENV plotTraitFields traitRecId \
          plotTABLE plotTRAITS

   if {$traitRecId != -1} {

     # Read Current Trait Configuration

     foreach item $plotTraitFields {
        panDbRead "@$gvar(envName)${plotTRAITS}($traitRecId,$item)" value
        set gvar($item) $value
     }
   }

   plotSetColor $gvar(lineColor)

   foreach item $plotPenAttributes  {
       panDbRead "@$gvar(envName)$plotTABLE.curves($penId,$item)" value
#       puts "@$gvar(envName)$plotTABLE.curves($penId,$item) $value"

       if {[cequal $item sampleRate]} {

          if {$value == 0} {
             set tmp 0
          } else {
             set tmp 1
          }

          set gvar(plotUpdatingMode) $tmp ; plotSetUpdateMode $tmp

       } elseif { [cequal $item yAxisScalingMode]} {

	   plotSetScalingModeY $value 

       } elseif { [cequal $item dataSourceType]} {

	   if {$value == 2 } {
              set gvar(yDataSource) 0
           } else {
              set gvar(yDataSource) $value
           }

       } elseif { [cequal $item yAttrOrSourceAddr]} {
 
	   if {![cequal $value "@\[\]" ]} {
              set tmp [seq_dbDirAddrToName $value alias]
           } else {
              set tmp ""
           }
           set gvar(ySourceName) $tmp

       } elseif { [cequal $item xAttrOrHistTable]} {

	   if {![cequal $value "@\[\]" ]} {
              set tmp [seq_dbDirAddrToName $value alias]
           } else {
              set tmp ""
           }

           set gvar(xSourceName) $tmp

       } elseif { [cequal $item XXX] } {
           set do nothing
       }

#       puts "plotReadPenConfig @$gvar(envName)${plotTABLE}.curves($penId,$item) $value"
       set gvar($item) $value
   }
}

proc plotGetY_PLIN {} {
    global gvar
    set tmp [seq_dbGetDirAddr $gvar(ySourceName)]
    set gvar(yAttrOrSourceAddr) $tmp
}


proc plotGetX_PLIN {} {
    global gvar

    if {[cequal $gvar(xSourceName) ""] } {
       set gvar(xAttrOrHistTable) "@\[\]"
    } else {
       set tmp [seq_dbGetDirAddr $gvar(xSourceName)]
       set gvar(xAttrOrHistTable) $tmp
    }
}


proc plotWritePenConfig { penId } {
   global gvar plotPenAttributes plotRTAPENV plotTraitFields traitRecId \
          plotTABLE plotTRAITS

    # Check if this is a valid color
    if {[catch {winfo rgb . $gvar(lineColor)}] } {
	set msg "Invalid color $gvar(lineColor)"
	tk_dialog .dialog "Configuration Error" $msg info 0 OK
	return
    }

    # Force PLIN reading

    if { [cequal $gvar(ySourceName) ""]} {
       set gvar(yAttrOrSourceAddr) "@\[\]"
    } else {
       if {[catch {seq_dbGetDirAddr $gvar(ySourceName)} tmp]} {
          set msg "Cannot get direct address for $gvar(xSourceName)"
          tk_dialog .dialog "Configuration Error" $msg info 0 OK
          return "FAILURE"
       }
       set gvar(yAttrOrSourceAddr) $tmp
    }

    if { [cequal $gvar(xSourceName) ""]} {
       set gvar(xAttrOrHistTable) "@\[\]"
    } else {
       if {[catch {seq_dbGetDirAddr $gvar(xSourceName)} tmp]} {
          set msg "Cannot get direct address for $gvar(xSourceName)"
          tk_dialog .dialog "Configuration Error" $msg info 0 OK
          return "FAILURE"
       }
       set gvar(xAttrOrHistTable) $tmp
    }

   # Update local configuration values
   set gvar(dataSourceType) $gvar(yDataSource)

   # Range checking

   if {$gvar(yAxisMinima) >= $gvar(yAxisMaxima)} {
      set msg "Not Valid Y axis min/max values : yAxisMinima >= yAxisMaxima"
      tk_dialog .dialog "Configuration Error" $msg info 0 OK
      return "FAILURE"
   }

   # Add additional Read-Only parameters
   set attrList [concat $plotPenAttributes gridColor]

   if {$traitRecId != -1} {

     # Write Current Trait Configuration

     foreach item $plotTraitFields {
        set attr "@$gvar(envName)${plotTRAITS}($traitRecId,$item)" 
        puts "plotWritePenConfig $attr $gvar($item)"
        panDbWrite $attr $gvar($item)
     }
   }

   foreach item $attrList {

       if {[cequal $item sampleRate]} {
	   if {$gvar(plotUpdatingMode) == 0} {
              set gvar(sampleRate) 0
           }          
       }
       set attr "@$gvar(envName)${plotTABLE}.curves($penId,$item)"
       puts "plotWritePenConfig $attr $gvar($item)"
       panDbWrite $attr $gvar($item)
   }
}


proc plotReadTPConfig {} {
   global gvar cv plotConfigAttr plotRTAPENV plotTABLE tbTraits

   # Read table of traits
   panDbRead "@$gvar(envName)${plotTABLE}.traits" tbTraits

   foreach item $plotConfigAttr {
       panDbRead "@$gvar(envName)${plotTABLE}.${item}" value

#       puts "@$gvar(envName)${plotTABLE}.${item}  $value"

       if {[cequal $item xAxisMinima] || [cequal $item xAxisMaxima]} {

	  if {[cequal $item xAxisMinima]} { set t1 $value }
	  if {[cequal $item xAxisMaxima]} { set t2 $value }

          # Covert dates from 'number of sec' into string
          set time [expr {int($value)}]
          set value [fmtclock $time "%y-%m-%d %T"]
       }

       set gvar($item) $value
   }

   set w $cv.wdgTimeSpan

   if {$gvar(xAxisScalingMode) != 1} {

       set tmp [expr {$t2 - $t1}]

       if {$tmp >= 86400} {
	   set gvar(timeSpan) [expr {$tmp/86400.0}]
	   set gvar(timeSpanUnit) 3
       } elseif { $tmp >= 3600} {
	   set gvar(timeSpan) [expr {$tmp/3600.0}]
	   set gvar(timeSpanUnit) 2
       } elseif { $tmp >= 300 } {
	   set gvar(timeSpan) [expr {$tmp/60.0}]
	   set gvar(timeSpanUnit) 1
       } else {
	   set gvar(timeSpan) $tmp
	   set gvar(timeSpanUnit) 0
       }
       
       if {[winfo exists $w]} { $w enable }
   } else {

       if {$gvar(xAxisScalingMode) == 2} { set gvar(xAxisScalingMode) 3}
       if {$gvar(xAxisScalingMode) == 5} { set gvar(xAxisScalingMode) 4}

       if {[winfo exists $w]} { $w disable }
       set gvar(timeSpan) ""
   }

   plotSetPenConfig ; # Set Global PEN configuration
}

# Get line color from the trais table
# Each record has the following fields :
# - trait  
# - curveId 
# - markerVisible markerStyle markerColor
# - lineVisible   lineStyle   lineColor

proc plotGetTraitAttr { curveId name what} {
   global tbTraits

   set idx   0

   foreach tr  $tbTraits {
      lassign $tr type cid mv ms mc lv ls lc

      if {$cid == $curveId && $type == $what} {
	  switch $name {
	      lineColor     { return $lc}
              lineStyle     { return $ls}
              lineVisible   { return $lv}
 	      markerColor   { return $mc}
              markerStyle   { return $ms}
              markerVisible { return $mv}
          }
      }
      incr idx
   }

}

proc plotWriteTPConfig {} {
   global gvar plotConfigAttr plotRTAPENV plotTABLE

   # Process timeSpan
   if {$gvar(xAxisScalingMode) != 1} {

       if {[cequal $gvar(timeSpan) ""]} { 
          set gvar(timeSpan) 60
          set gvar(timeSpanUnit) 0
       }

       if {$gvar(timeSpanUnit) == 1}  {
           set numSecs [expr {$gvar(timeSpan) * 60}]
       } elseif {$gvar(timeSpanUnit) == 2}  {
           set numSecs [expr {$gvar(timeSpan) * 3600}]
       } elseif {$gvar(timeSpanUnit) == 3}  {
           set numSecs [expr {$gvar(timeSpan) * 86400}]
       } else {
           set numSecs $gvar(timeSpan)
       }

       set numSecs [expr {int($numSecs)}]
       set time    [dateToValue $gvar(xAxisMinima)]
       set time    [expr {$time + $numSecs}]
       set gvar(xAxisMaxima) [fmtclock $time "%y-%m-%d %T"]
   }

   foreach item $plotConfigAttr {
       
       if {[cequal $item xAxisMinima] || [cequal $item xAxisMaxima]} {
          # Covert dates to 'number of seconds'
          set value [dateToValue $gvar($item)]
       } else {
          set value $gvar($item)
       }

       panDbWrite "@$gvar(envName)${plotTABLE}.${item}" $value
   }

   # Write status of visible PENs

   set tableInfo [seq_dbGetAttrInfo "@$gvar(envName)${plotTABLE}.curves"]
   set numbPen   [lindex $tableInfo 2]

   for {set i 0} {$i < $numbPen} { incr i} {
       panDbWrite "@$gvar(envName)${plotTABLE}.curves($i,visible)"     $gvar(visibleFlag${i})

#       puts "panDbWrite $i yAxisSelect = $gvar(yAxisSelect${i})"
       panDbWrite "@$gvar(envName)${plotTABLE}.curves($i,yAxisSelect)" $gvar(yAxisSelect${i})
   }
}

proc dateToValue { tnow } {
   scan $tnow "%s %s" day time
   regsub -all --  "\-" $day { } newDay
   scan $newDay "%s %s %s" yy mm dd
   set value [convertclock "$time $mm/$dd/$yy"]
    return [expr {int($value)}]
}


# Retrieve the associated trait record number
# If several trait records have the same curve Id., only the first one 
# is used to configure the curve.

proc plotGetTraitRecord {penId} {
   global gvar plotTABLE plotTRAITS

   # Get all the curve Ids of the trait table

   set info [seq_dbGetAttrInfo "@$gvar(envName)${plotTABLE}.traits"]
   set recs [lindex $info 2] ; incr recs -1

  panDbRead "@$gvar(envName)${plotTRAITS}(0:$recs,curve)" curveId
  panDbRead "@$gvar(envName)${plotTRAITS}(0:$recs,trait)" traitType

#   puts "@$gvar(envName)${plotTRAITS}(0:$recs,curve)  $curveId"
#   puts "@$gvar(envName)${plotTRAITS}(0:$recs,trait)  $traitType"

   set recId 0
   foreach id $curveId {
#      puts "id = $id  recId $recId  traitType [lindex $traitType 0]"
      if {$id == $penId && [lindex $traitType 0] == 0} { return $recId }
      incr recId
   }

   puts "WARNING : No Trait Record Found"
   return -1
}

#
# ___oOo___
