#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: plotSelectFFT.tcl,v 2.40 2005/08/09 07:50:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  07/11/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc fftFormatDisplay { txt } {
    lassign $txt f1 f2 f3 f4 f5 f6 f7

    if {$f7 == 0} {
	set f7 DISABLED
    } else {
	set f7 ENABLED
    }

    if {$f6 == 0} {
	set f6 "-"
    } else {
	set f6 "OVERLAP"
    }


    set tmp  [format "%-28s  %-28s %-4d %-4d %-12s %s" \
	    $f1 $f2 $f3 $f5 $f6 $f7]

    return $tmp
}


proc fftReadConfigTbl { fftTbl } {
    global cv gvar fftCREC fftRecNum

    if {[info exist fftCREC]} { unset fftCREC}

    $cv.wdgConfigTable deleteItem 0 end
    panDbRead $fftTbl.setup recList

    set fftRecNum 0
    foreach item $recList {
	lassign $item s d

	if {[cequal $s ""]} { set item [lreplace $item 0 0 "none"]}
	if {[cequal $d ""]} { set item [lreplace $item 1 1 "none"]}

	# Take away dummy field or fields which are not used
	set fftCREC($fftRecNum) [lrange $item 0 6]

	incr fftRecNum
    }
    
    $cv.wdgConfigTable updateSortList -increasing -integer fftCREC fftFormatDisplay
}


proc fftWriteConfigTbl {fftTbl} {
    global gvar cv fftCREC

    set max [$cv.wdgConfigTable getSize]

    for {set idx 0} {$idx < $max} {incr idx} {

        set tmp $fftCREC($idx)

        lassign $tmp fftSource fftDest

        if {[cequal $fftSource none]} { set tmp [lreplace $tmp 0 0 ""] }

        if {[cequal $fftDest   none]} { 
	    set tmp [lreplace $tmp 1 1 ""] 
        } else {
	    set ret [plotSetFFTDest $fftDest]

	    if {[cequal $ret FAILURE]} {
		set tmp "Cannot configure FFT Plot Point $fftDest"
		tk_dialog .dialog "Configuration Error" $tmp info 0 OK
                fftReadConfigTbl $fftTbl
                return
	    }
        }

	panDbWrite $fftTbl.setup($idx,0:6) $tmp
    }

    fftReadConfigTbl $fftTbl
}


proc fftEraseRec {} {
    global gvar 
    set gvar(fftSource)     "" ; set gvar(fftDest)       ""
    set gvar(fftSourceItem) 0  ; set gvar(fftDestItem)   0
    set gvar(fftLength)     0  ; set gvar(fftOverlap)    0
    set gvar(fftEnable)     0  ; set gvar(fftSourcePLIN) ""
    set gvar(fftDestPLIN)   "" ; set gvar(calcType)   0
}


proc fftConfigRec {fftTbl} {
    global gvar cv fftCREC

    set idx       $gvar(fftRecNum)
    set fftSource $gvar(fftSource)
    set fftDest   $gvar(fftDest)

    if {[cequal $gvar(fftSource) ""]} { set fftSource none}
    if {[cequal $gvar(fftDest) ""]}   { set fftDest none}

    set fftCREC($idx) [list $fftSource $fftDest $gvar(fftSourceItem) \
	    $gvar(fftDestItem) $gvar(fftLength) $gvar(fftOverlap) \
	    $gvar(fftEnable) ]

    $cv.wdgConfigTable deleteItem 0 end
    $cv.wdgConfigTable updateSortList -increasing -integer fftCREC fftFormatDisplay
}



proc fftSelectRecord {} {
    global cv gvar

    fftEraseRec

    set gvar(fftRecNum) [$cv.wdgConfigTable getIndex]
    set tmp             [$cv.wdgConfigTable getSelection]

    lassign $tmp gvar(fftSource) gvar(fftDest) gvar(fftSourceItem) \
	    gvar(fftLength) fftOverlap fftEnable

    if {[cequal $fftOverlap "OVERLAP"]} {
        set gvar(fftOverlap) 1
    } else {
        set gvar(fftOverlap) 0
    }

    if {[cequal $fftEnable "ENABLED"]} {
        set gvar(fftEnable) 1
    } else {
        set gvar(fftEnable) 0
    }

    if {! [cequal $gvar(fftSource) "none"]} {

	if {[catch {seq_dbGetDirAddr $gvar(fftSource)} tmp]} {
	    puts "Cannot get 'direct address' for $gvar(fftSource)"
	    return
	}

	set gvar(fftSourcePLIN) $tmp
    }


    if {! [cequal $gvar(fftDest) "none"]} {

	if {[catch {seq_dbGetDirAddr $gvar(fftDest)} tmp]} {
	    puts "Cannot get 'direct address' for $gvar(fftDest)"
	    return
	}
	
	set gvar(fftDestPLIN) $tmp
    }
}

#
# ___oOo___
