#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: plotSelectPoint.tcl,v 2.40 2005/08/09 07:50:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  07/11/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc plotSelectPoint {} {
    global cv gvar ppIndex ppAlias ppPlin seq_errLogging

    set ppIndex [$cv.wdgPlotPointList getIndex]
    set gvar(ppAliasName) $ppAlias($ppIndex)
    set gvar(ppPLIN)      $ppPlin($ppIndex)
    $cv.wdgPlotPointList setSelection $ppIndex

    set gvar(plotNumPens) 0
    set gvar(plotNumRecs) 0
    set gvar(ptPLIN)      ""
    set gvar(ptAliasName) ""

    # Get info on plot table 

    set ptable "<alias>$gvar(ppAliasName):plot_table"

    set seq_errLogging off
    if {[catch {seq_dbGetAttrInfo "${ptable}.data"} attrInfo]} {
	set seq_errLogging auto ; return
    }

    set seq_errLogging auto 

    set gvar(plotNumPens) [expr {[lindex $attrInfo 3] -1}]
    set gvar(plotNumRecs) [lindex $attrInfo 2]

    if {[catch {seq_dbGetDirAddr $ptable} tmp]} {
	puts "Cannot get 'direct address' for $ptable"
	return
    }

    set gvar(ptPLIN)  $tmp    

    if {[catch {seq_dbGetAlias $ptable } tmp]} {
	puts "Cannot get 'Alias' for $ptable"
	return
    }

    set len [string last ">" $tmp] ; incr len
    set gvar(ptAliasName)    [string range $tmp $len end]
}


proc getAllPlotPoints {} {
    global ppList plotPoints numPlotPoints plotPtSize ppInfoList
    global ppPlin ppAlias ppAbsolute gvar

    # Get list of plot points

    if {[cequal [panDbRead ${plotPoints} ppList] FAILURE]} {
	puts "Cannot read attribute $plotPoints"
	return
    }

    # Retrieve alias and absolute names for each history point

    if {[info exists ppPlin]}     { unset ppPlin}
    if {[info exists ppAlias]}    { unset ppAlias}
    if {[info exists ppAbsolute]} { unset ppAbsolute}

    set idx 0

    foreach ppDirAddr $ppList {

	# Skip CCS history table template
	if {[cequal $ppDirAddr "@\[6\]"]} { continue }

	# Loop until the first 'empty' address is found
	if {[cequal $ppDirAddr "@\[\]"]} { break }

	# Get alias
	if {[catch {seq_dbDirAddrToName $ppDirAddr alias} nameAlias]} {
	    puts "Cannot get 'alias' for $ppDirAddr"
	    return
	}

	# Get absolute address
	if {[catch {seq_dbDirAddrToName $ppDirAddr absolute} nameAbsAddr]} {
	    puts "Cannot get 'absolute name' for $ppDirAddr"
	    return
	}

	set len [string last ">" $nameAlias] ; incr len

	set ppPlin($idx)     $ppDirAddr
	set ppAlias($idx)    [string range $nameAlias $len end]
	set ppAbsolute($idx) $nameAbsAddr

	incr idx
    }

    set numPlotPoints $idx
}



proc listPlotPoints {w} {
    global gvar numPlotPoints
    global ppPlin ppAlias ppAbsolute

    $w deleteItem 0 end

    if {[cequal $gvar(symAddrType) ALIAS] } {

	for {set idx 0} {$idx < $numPlotPoints} {incr idx} {
	    $w addItem end [format "%-8s %s"  $ppPlin($idx) $ppAlias($idx)]
	}

    } else {
	for {set idx 0} {$idx < $numPlotPoints} {incr idx} {
	    $w addItem end [format "%-8s %s"  $ppPlin($idx)  $ppAbsolute($idx)]
	}
    }

}




##############################################################
#           procs for plotServer Monitor GUI
##############################################################

proc plotGetDispInfo {} {
    global cv env gvar
    global ppPID ppALIAS ppABSOLUTE ppUSED

    set ret [panSendCommand $env(RTAPENV) plotServer GETINFO "" 5000]

    if {[cequal $ret FAILURE]} { return 0}

    set tmpList [split $ret ","]
    set len     [llength $tmpList]

    set gvar(shmID)       [lindex $tmpList 0]
    set gvar(shmSIZE)     [lindex $tmpList 1]
    set gvar(maxNumPlots) [lindex $tmpList 2]
    set gvar(maxBuffRec)  [lindex $tmpList 3]


    # Retrieve alias and absolute names for each history point

    if {[info exists ppPID]}      { unset ppPID}
    if {[info exists ppALIAS]}    { unset ppALIAS}
    if {[info exists ppABSOLUTE]} { unset ppABSOLUTE}
    if {[info exists ppUSED]}     { unset ppUSED}
    if {[info exists ppRUN]}      { unset ppRUN}

    set idx 0
    while { $idx < $gvar(maxNumPlots) } {
        set ppALIAS($idx) [lindex $tmpList [expr {4 + $idx*3} ]]
        set ppUSED($idx)  [lindex $tmpList [expr {4 + $idx*3 + 1}]]
        set ppPID($idx)   [lindex $tmpList [expr {4 + $idx*3 + 2}]]

        # Get absolute address for valid plot points

        if {[cequal $ppALIAS($idx) "none"]} {
            set ppABSOLUTE($idx) "none"
        } else {

	    # Get direct address
	    if {[catch {seq_dbGetDirAddr $ppALIAS($idx)} dirAddr]} {
		puts "Cannot get 'direct address' for $ppALIAS($idx)"
		return
	    }

	    # Get absolute address
	    if {[catch {seq_dbDirAddrToName $dirAddr absolute} ppABSOLUTE($idx)]} {
		puts "Cannot get 'absolute name' for $dirAddr"
		return
	    }
	}

        incr idx
    }

    listPlotNames $cv.wdgServer
    selectPlotName $cv.wdgServer $gvar(dispID)

    return $idx
}



proc listPlotNames { w } {
    global env gvar

    $w deleteItem 0 end

    for {set idx 0} {$idx < $gvar(maxNumPlots) } {incr idx} {
       $w addItem end [formatPlotName $idx]
    }
}


proc formatPlotName { idx } {
    global env gvar
    global ppPID ppALIAS ppABSOLUTE ppUSED ppRUN

    if { $ppUSED($idx) } {
	set f1 "IN USE"
    } else {
	set f1 "FREE"
    }

    if { [cequal $gvar(symAddrType) ALIAS] } {
	set pname $ppALIAS($idx)
    } else {
	set pname $ppABSOLUTE($idx)
    }

    set ppRUN($idx) [checkPlotRunning $ppPID($idx) ]

    return [format "%-6s  %-13s %-9s  %s" $f1 $ppRUN($idx) $ppPID($idx) $pname]

}


proc selectPlotName { w { idx "" } } {
    global cv gvar
    global ppPID ppALIAS ppABSOLUTE ppUSED ppRUN

    if {[cequal $idx ""]} { set idx  [$w getIndex]}

    set gvar(dispID)      $idx
    set gvar(ppAliasName) $ppALIAS($idx)
    set gvar(ppPID)       $ppPID($idx)

    $w setSelection $idx

    set gvar(pdSTATE) [checkPlotRunning $gvar(ppPID)]

    # plotDisplay state changed ... update its state in the list box

    $w deleteItem $idx
    $w addItem $idx [formatPlotName $idx]
}


proc checkPlotRunning { pid } {

    if { $pid == 0 } { return "NOT RUNNING" }

    if { [catch {exec ps -aef | grep $pid | grep -v grep} ] } {
	return "NOT RUNNING"
    } else {
	return "RUNNING"
    }
}









#
# ___oOo___
