# vim: set filetype=tcl: #
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdscopeLib.tcl,v 1.0 2005/06/17 09:07:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# pbiereic  2004-04-14  created
# ahuxley   2004-06-30  tailored to rtdscope panel
# ahuxley   2004-07-14  many updates for new widgets in panel to access
#                       new methods provided by bltplot to control 
#                       plot widget.
#

set verbose 2
proc rtdscopeDebug { level mess } {
    global verbose

    if {$verbose >= $level} {
        puts stderr "rtdscopeLib: DEBUG[$level]: $mess"
    }
}
    
proc rtdscopeLibInitialize { } {
    global cv

    rtdscopeDebug 10 "rtdscopeLibInitialize: start of function"

    # 
    #  Call plot widget methods for things which are not accessible via
    #  public instantiation-time variables.
    #

    $cv.rtdscopeGuiWdgPlot attach 1 RTDSCOPE
    $cv.rtdscopeGuiWdgPlot show_plot 1

    #
    #  Set the graph related widgets to the correct values
    #  for graph 0.
    #

    rtdscopeDebug 10 "rtdscopeLibInitialize: configuring graph-related panel widgets for graph 0 ..."
    rtdscopeLibUpdateGraphWidgets 0

    #
    #  Set the non-graph related widgets to the correct values
    #

    rtdscopeDebug 10 "rtdscopeLibInitialize: configuring non-graph-related panel widgets ..."
    rtdscopeLibUpdateGraphWidgets

    #
    #  Update the plot widget itself to the values in the database.
    #

    rtdscopeDebug 10 "rtdscopeLibInitialize: getting plot widget to reconfigure itself according to db attrs ..."
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibSetTransfer { transfer } {
    panDbWrite <alias>rtdscope:data.desiredSdmaStatus $transfer
}

proc rtdscopeLibSetMaxX { maxx } {
    global rtdscopeLcu rtdscopeServer
    set timeout 2000

    #
    #  Tell the LCU to change the width of images it sends us.
    #

    panSendCommand $rtdscopeLcu $rtdscopeServer SETFSX "" $timeout $maxx

    #
    #  The above command usually (except on failure) results in a 
    #  database change, so after it completes we re-read the database.
    #

    rtdscopeLibUpdateGraphWidgets
}

proc rtdscopeLibSetBlock { block } {
    global rtdscopeLcu rtdscopeServer
    set timeout 2000

    #
    #  Write the value to the database. rtdscopeDart attaches
    #  to this and will see it.
    #

    #panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.block $block
    panSendCommand $rtdscopeLcu $rtdscopeServer SETBLK "" $timeout $block

    #
    #  The above command usually (except on failure) results in a 
    #  database change, so after it completes we re-read the database.
    #

    rtdscopeLibUpdateGraphWidgets
}

proc rtdscopeLibSetSecsPerSamp { secspersamp } {
    global rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.secs_per_samp $secspersamp
    #  Since the above command bypasses the server on the LCU, and writes straight to its
    #  database there should be no way it can fail, and so no need to update the non-graph
    #  widgets in the GUI, but just in case the db read can fail, let's do it anyway.
    rtdscopeLibUpdateGraphWidgets
    #  Redraw the plot so the new scale is visible.
    rtdscopePlotWidgetReconfigure
}

#
#  This maps field names in an OLDB table to indexes required to read them.
#

set attr_indices_graphConfig(scanid)    0
set attr_indices_graphConfig(enabled)   1
set attr_indices_graphConfig(colour)    2
set attr_indices_graphConfig(min)       3
set attr_indices_graphConfig(max)       4
set attr_indices_graphConfig(imgline)   5
set attr_indices_graphConfig(grouped)   6
set attr_indices_graphConfig(title)     7

proc rtdscopePlotWidgetReconfigure { } {
    global cv gvar attr_indices_graphConfig rtdscopeLcu

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: start of function"
     
    #
    #  Set tac block output and line colour
    #

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: constructing '-plotScanList' option list ..."
    set config_opts ""
    set redo_assignments 0
    for {set i 0} {$i < 10} {incr i} {
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(scanid))    scanid
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(imgline))   imgline
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(colour))    colour
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(title))     title
        rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: scanid=$scanid, imgline=$imgline, colour=$colour, title=$title"
        if {[string compare $title unset] == 0} {
            set title [format "T%d" [expr {$i + 1}]]
        } else {
            set title [format "T%d ($title)" [expr {$i + 1}]]
        }
        set config_opts "$config_opts -plotScanList$i \"$scanid $imgline $colour {$title}\""
    }
    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: effecting '-plotScanList' option list with '$cv.rtdscopeGuiWdgPlot config $config_opts' ..."
    $cv.rtdscopeGuiWdgPlot config $config_opts

    #  
    #  For each scan line, assign it to a scan group. (In the process count how
    #  many scan groups there are.)
    #

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: assigning scan lines to scan groups ..."
    set groupindex -1
    set groupid "Group0"
    for {set i 0} {$i < 10} {incr i} {
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(scanid))  scanid
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(enabled)) enabled
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(grouped)) grouped
        if {$enabled && ($i == 0 || !$grouped)} {
            incr groupindex
            set groupid "Group$groupindex"
        }
        $cv.rtdscopeGuiWdgPlot assign_scan $scanid $groupid end
    }

    #
    #  Enable or disable scan groups.
    #

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: enabling/disabling scan groups ..."
    for {set i 0} {$i < 10} {incr i} {
        set enabled [expr {($i <= $groupindex) ? 1 : 0}]
        set groupid "Group$i"
        $cv.rtdscopeGuiWdgPlot show_group $groupid $enabled
    }
    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: done enabling/disabling scan groups"

    #
    #  Enable or disable scan lines
    #

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: enabling/disabling scan lines ..."
    for {set i 0} {$i < 10} {incr i} {
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(scanid))  scanid
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(enabled)) enabled
        rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: calling '$cv.rtdscopeGuiWdgPlot show_scan $scanid $enabled' ..."
        $cv.rtdscopeGuiWdgPlot show_scan $scanid $enabled
    }
    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: done enabling/disabling scan lines"

    #
    #  Set minimum and maximum values for Y axis
    #

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: setting cut levels ..."
    for {set i 0} {$i < 10} {incr i} {
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(scanid)) scanid
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(min))    min
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($i,$attr_indices_graphConfig(max))    max
        if {[string compare $min unset] == 0 || [string compare $max unset] == 0} {
            continue
        }
        $cv.rtdscopeGuiWdgPlot set_cuts $scanid $min $max
    }

    #
    #  Set the X axis labelling for all plot groups which are to be displayed
    #  to be the same
    #

    panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.secs_per_samp sampint
    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: groupindex=$groupindex"
    for {set i 0} {$i <= $groupindex} {incr i} {
        set groupid "Group$i"
        rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: setting x range for group $i ..."
        $cv.rtdscopeGuiWdgPlot set_xrange $groupid 1 0.0 $sampint
    }

    rtdscopeDebug 10 "rtdscopePlotWidgetReconfigure: end of function"
}

proc rtdscopeHideGraph { graphid hide } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(enabled)) $hide
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeGroupGraph { graphid grouped } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(grouped)) $grouped
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibGraphMinY { graphid min } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(min)) $min
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibGraphMaxY { graphid max } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(max)) $max
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibSetAutoScaling { graphid } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(max)) unset
    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(min)) unset
    rtdscopePlotWidgetReconfigure
    #  We also need to remove any numbers currently written in the max and min slots in the GUI
    rtdscopeLibUpdateGraphWidgets $graphid
}

proc rtdscopeLibClearTitle { graphid } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(title)) unset
    rtdscopePlotWidgetReconfigure
    #  We also need to remove any title currently written in the title slot in the GUI
    rtdscopeLibUpdateGraphWidgets $graphid
}

proc rtdscopeBlockGraph { graphid imgline } {
    global gvar attr_indices_graphConfig rtdscopeLcu
    
    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(imgline)) $imgline
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibGraphColour { graphid colour } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(colour)) $colour
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibPrint { graphid } {
    global cv

    set scanid [format "T%d" [expr {$graphid + 1}]]

    $cv.rtdscopeGuiWdgPlot print $scanid
}

proc rtdscopeLibSetGraphTitle { graphid title } {
    global gvar attr_indices_graphConfig rtdscopeLcu

    panDbWrite @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(title)) $title
    rtdscopePlotWidgetReconfigure
}

proc rtdscopeLibUpdateGraphWidgets { {graphid -1} } {
    global cv gvar rtdscopeLcu attr_indices_graphConfig

    rtdscopeDebug 10 "rtdscopeLibUpdateGraphWidgets: start of function (graphid = $graphid)"

    #
    #  if a graph id is supplied then update the widgets for that graphi
    #  otherwise update the general widgets at the top of the panel.
    #   

    if {$graphid >= 0} {
        rtdscopeDebug 10 "rtdscopeLibUpdateGraphWidgets: copying graph-related db attrs to panel ..."
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(enabled)) enabled
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(colour)) colour
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(min)) min
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(max)) max
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(imgline)) imgline
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(grouped)) grouped
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.graphConfig($graphid,$attr_indices_graphConfig(title)) title

        if {[string compare $min unset] == 0} {
            set min ""
        }
        if {[string compare $max unset] == 0} {
            set max ""
        }
        if {[string compare $title unset] == 0} {
            set title ""
        }

        set gvar(rtdscopeGuiEnabled)             $enabled
        set gvar(rtdscopeGuiGrouped)             $grouped
        set gvar(rtdscopeGui_var_miny)           $min
        set gvar(rtdscopeGui_var_maxy)           $max
        set gvar(rtdscopeGui_var_tacblockoutput) $imgline
        set gvar(rtdscopeGui_var_colour)         $colour 
        set gvar(rtdscopeGui_var_title)          $title 

    } else {
        rtdscopeDebug 10 "rtdscopeLibUpdateGraphWidgets: copying non-graph-related db attrs to panel ..."
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.block         gvar(rtdscopeGui_var_block)
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.frame_size_x  gvar(rtdscopeGui_var_maxx)	     
        panDbRead <alias>rtdscope:data.lcuenv                                gvar(rtdscopeGui_var_lcuenv)	     
        panDbRead <alias>rtdscope:data.desiredSdmaStatus                     gvar(rtdscopeGui_var_transfer)	     
        panDbRead @$rtdscopeLcu<alias>rtdscope:control:misccfg.secs_per_samp gvar(rtdscopeGui_var_sampint)	     
    }
}

proc rtdscopeLibStateCallback { hook cmd } {
    global gvar

    #  Don't do anything before the state has changed.
    if {[string compare $hook PRE] == 0} {
        return
    }

    #  Update the various GUI widgets.
    rtdscopeLibUpdateGraphWidgets $gvar(graphid)
    rtdscopeLibUpdateGraphWidgets
    rtdscopePlotWidgetReconfigure
}
