#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: sampMisc.tcl,v 2.45 2005/10/18 14:33:17 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  04/05/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc sampGetPID { procName } {
    global gvar env

    set awkp     "{print \$2}"
    set userName [exec whoami]

    if {[catch {exec ps -aef | grep $procName | grep $userName | grep -v grep | grep -v emacs | awk $awkp} pid] } {
	set pid 0
    }

    return $pid
}


proc sampGetPlotProcId { plotPoint } {
    global gvar env

    if {[catch {seq_dbGetAlias $plotPoint } plALIAS]} {
	puts "Cannot get 'Alias' for $plotPoint"
	return
    }

    set tmp [string range $plALIAS 7 end]
    set fname "/tmp/plotDisplay_$env(RTAPENV)_${tmp}.lock"

    if {[file exists $fname]} {
	set plotPID [exec cat $fname]
    } else {
	set plotPID 0
    }

#    puts "sampGetPlotProcId  plotPoint $plotPoint  plotPID = $plotPID"
    return $plotPID
}

proc sampCtrlSet {cmd flag } {
    global gvar env

    set pid [sampGetPID sampDb]

    if {$pid > 0} {
	panSendCommand $env(RTAPENV) sam${pid} $cmd "NO_CHECK" 5000 $flag
    }
}


proc sampSelectAll {} {
   global gvar env numbValidPen

   set idx 1

   while {$idx <= $numbValidPen} {
      set gvar(cb${idx}) 1
      incr idx
   }
}

proc sampCtrlStart {} {
    global gvar numbValidPen env sampVerbose sampDebug

    # Check attributes list
    sampSetEnvType ""
    sampCheckTable

    set numAttr 0

    # Check attributes

    for {set idx 1} {$idx <= 8} { incr idx} {
	set tmp [string trim $gvar(attr${idx}) " "]

	if {![cequal $tmp ""]} {

	    set item "@$gvar(envName)<>${tmp}"
	    if {[catch {seq_dbGetAttrInfo $item } attrInfo] } {
		set tmp "Attribute $idx does not exist\n ... samp did not start !!"
		tk_dialog .d "Dialog" $tmp info 0 OK
		return
	    }

	    incr numAttr
	} else {

	    # Attribute is empty ... disable it

	    set gvar(cb${idx}) 0
	}
    }

    update

    if {$numAttr == 0} {
	tk_dialog .d "Dialog" "NO Attributes Specified" question 0 OK
	return
    }

    if {$gvar(sampFileAutoName)} {
	set gvar(dataFileName) [sampGetDefDataFile]
    }

    if {[file exists $gvar(dataFileName)] &&  $gvar(writeToFile) } {
	set     msg "Data File already exists\n"
	append  msg "Do you want to overwrite it ?"

	if {[tk_dialog .d "Dialog" $msg question 1 Continue Cancel]} {
	    return
	}
    }

    # Check if there is another instance of samp 
    # running on the same plot table

    set tmp [seq_dbGetAlias "$gvar(plotTable)"]
    set idx [string last ">" $tmp ]; incr idx;
    set plotAlias [string range $tmp $idx end]

    # Perform checks to see which processes are still running

    set fname   "/tmp/plotDisplay_$env(RTAPENV)_${plotAlias}.lock"
    set ret     [catch {exec ps -aef | grep plotDisplay | grep $plotAlias}]

    if { [file exists $fname] || $ret == 0 } {
	set     msg "Another plotDisplay is running\n"
	append  msg "on the same plot point ${plotAlias}"

	if {[tk_dialog .d "Dialog" $msg question 1 "Re-Start Samp" "Abort Action"]} {
	    return
	}
    }

    update 

    # Construct command line based on the number of 'enabled' items

    set cmdLine {}

    for {set idx 1} {$idx <= 8} { incr idx} {

	if {$gvar(cb${idx}) == 0} continue;

	append cmdLine " {$gvar(attr${idx})}  "

	set tmp [string trim $gvar(lbl${idx}) " "]

	if {![cequal $tmp ""]} { 
	    append cmdLine " -l {$tmp}"
	}
    }

    if {$sampVerbose} {
	set verboseFlag "-v"
    } else {
	set verboseFlag ""
    }


    if {$sampDebug} {
	set dbgFlag "-d"
    } else {
	set dbgFlag ""
    }

    eval exec sampStart $verboseFlag $dbgFlag $gvar(writeToFile) $gvar(dataFileName) \
	       \"$gvar(plotTable)\"  $gvar(envName) $gvar(envType)  $gvar(sampleRate) \
	       $gvar(reportRate) $cmdLine &

    set cnt 10
    set ret 1
    while {$cnt > 0 && $ret != 0} {
	after 300 ; incr cnt -1
	# Check if sampDb is really running
	set ret [catch {exec ps -aef | grep sampDb | grep -v grep | grep $plotAlias}]
    }

    if {$ret == 0} {
	set gvar(sampCtrlStatus) 1
    } else {
	set gvar(sampCtrlStatus) 0
    }

    if {$gvar(plotLblReSize)} {

       set cnt 0
       set ret -1

       while {$ret < 0 && $cnt < 5} {
          set ret [panSendCommand $env(RTAPENV) plotServer USEPLOT "" 5000 $plotAlias] 
          after 1000
          incr cnt +1
       }          
          
       if { $ret >= 0 } {
           panSendCommand $env(RTAPENV) plotServer VIEWRS "" 30000 "$plotAlias,1,-0.1,0.0"
       }
    }

    return
}

proc sampClearList {  what } {
    global gvar

    for {set i 1} {$i <= 8} {incr i} {

	if {![cequal $what labels]} {
	    set gvar(attr${i}) ""
	    set gvar(cb${i})   1
	}

	set gvar(lbl${i})  ""
    }
}

proc sampSetDefDir {} {
    global gvar env

    if {[info exists env(VLTDATA)]} {
	set tmpDir $env(VLTDATA)/ENVIRONMENTS/$gvar(envName)
    } else {
	if {[info exists env(HOME)]} {
	    set tmpDir $env(HOME) 
	} else {
	    set tmpDir [pwd] ;# Current directory
	}
    }

    return $tmpDir
}

proc sampGetDefDataFile {} {
    global gvar

    set time [fmtclock [clock seconds] "%Y-%m-%dT%T"]
    set tdir [file dirname $gvar(dataFileName)]
    set gvar(dataFileName) "$tdir/$gvar(envName).${time}.samp"
}


proc sampSetDefAttrFile {} {
    global gvar env 

    set tdir [sampSetDefDir]
    set gvar(attrFileName)  $tdir/.sampAttrList
}

proc sampAttrOnFile { what } {
    global env gvar

    if {[cequal $what store]} {
	set mode "w+"

	if {[file exists $gvar(attrFileName)]} {
	    set     msg "Attribute File already exists\n"
	    append  msg "Do you want to overwrite it ?"
	    
	    if {[tk_dialog .d "Dialog" $msg question 1 Continue Cancel]} {
		return
	    }
	}

    } else {
	set mode "r"
    }

    if { [catch {set fh [open $gvar(attrFileName) $mode] } ]} {
	# puts "Cannot open attribute list file $gvar(attrFileName)"
	return
    }

    if {[cequal $what store]} {
	puts $fh "$gvar(envName) ;# environment name"
	puts $fh "$gvar(envType) ;# environment type WS or LCU"
	puts $fh "\"$gvar(plotTable)\" ;# alias of the plot point"
	puts $fh "$gvar(writeToFile) ;# flag write to file"
	puts $fh "$gvar(sampFileAutoName) ;# flag Auto re-name"
	puts $fh "$gvar(reportRate) ;# report Rate"
	puts $fh "$gvar(sampleRate) ;# sample rate"
    } else {
	sampClearList attributes
	gets $fh tmp ; lassign $tmp gvar(envName) xx
	gets $fh tmp ; lassign $tmp gvar(envType) xx

	# This is for backward compatibility

	gets $fh tmp ; set idx [string last ";#" $tmp ];

	if {$idx == -1} {
	    set gvar(plotTable) $tmp
	} else {
	    lassign $tmp gvar(plotTable) xx
	}

	gets $fh tmp ; lassign $tmp gvar(writeToFile) xx
	gets $fh tmp ; lassign $tmp gvar(sampFileAutoName) xx
	gets $fh tmp ; lassign $tmp gvar(reportRate) xx
	gets $fh tmp ; lassign $tmp gvar(sampleRate) xx
    }

    for {set i 1} {$i <= 8} {incr i} {
	if {[cequal $what store]} {
            if {[cequal $gvar(attr${i}) ""]} {
	       set attrName empty
	    } else {
	       set attrName "$gvar(attr${i})"
	    }

	    puts $fh "{$attrName}  $gvar(cb${i}) \"$gvar(lbl${i})\" " 

	} else {

	    gets $fh tmp

#	    puts "$i tmp    $tmp"
	    lassign $tmp attrName ckFlag attrLabel

	    if {[cequal $ckFlag   ""]}    { set ckFlag 1 }
	    if {[cequal $attrName empty]} { set attrName "" }

	    set gvar(attr${i}) [string trim $attrName "\{\}"]

	    if {[cequal $gvar(attr${i}) ""]} {
		set gvar(cb${i}) 0
	    } else {
		set gvar(cb${i}) $ckFlag
	    }

	    set gvar(lbl${i})  $attrLabel
	}
    }

    if {[cequal $what read]} {
	sampCheckTable
        sampSetEnvType ""
    }

    close $fh
}


proc sampCheckTable {} {
    global cv gvar numbValidPen

    # Enable a number of attributes equal to the plot table size

    if {[cequal $gvar(plotTable) ""]} { return }

    if {[catch {seq_dbGetAttrInfo $gvar(plotTable).curves} tableInfo]} {
	set numbPen 0
    } else {
	set  numbPen   [lindex $tableInfo 2]
    }

    #   panDbRead "$gvar(plotTable).curves(,yAttrOrSourceAddr)" yref
    set  numbValidPen $numbPen
    incr numbPen ;

    # Enable used PENs
    set i   1
    while { $i < $numbPen} {
	${cv}.wdgAttr${i}  enable ; # Disable not used pens
	${cv}.wdgLbl${i}   enable 
	${cv}.wdgCheck${i} enable
	incr i;
    }

    # Disable Remaining PENs
    while { $numbPen <= 8} {
	${cv}.wdgAttr${numbPen}  disable ; # Disable not used pens
	${cv}.wdgLbl${numbPen}   disable
	${cv}.wdgCheck${numbPen} disable
	incr numbPen;
    }
}

proc sampSetEnvType { env } {
    global cv gvar

    if {[cequal $env ""]} {
	set c [string index $gvar(envName) 0]

	if {[cequal $c "w"]} {
	    set gvar(envType) WS
	} else {
	    set gvar(envType) LCU
	}
    }

    if {[cequal $gvar(envType) WS]} {
	$cv.wdgSamplingRate   disable
	#      $cv.wdgSamplingRate disable
    } else {
	$cv.wdgSamplingRate   enable
	#      $cv.wdgSamplingRate enable
    } 
}


proc sampGetNumCurves { ptName } {
    #   puts "sampGetNumCurves  $ptName"

    if {[catch {seq_dbGetAttrInfo ${ptName}.curves}  attrInfo]} {
	set n 0
    } else {
	set n [lindex $attrInfo 2]
    }
    return $n
}

proc sampLogData { text } {
    seq_logData $text
    puts $text
}


proc sampSetLabels { plotPoint dbList lblList nattrib} {

    # Clear table

    set totCurves [sampGetNumCurves $plotPoint]
    set curve     ${plotPoint}.curves

    if { $totCurves == 0 } {
	sampLogData "Cannot access plot table or invalid number of curves"
	return 0
    }

    set i 0

    while { $i < $nattrib } {

	seq_dbWriteSymbolic "${curve}(${i},visible)"  1 
   
	set myLabel [lindex $lblList $i]

	if {[cequal $myLabel ""]} {
	    set myLabel [lindex $dbList $i]
	}
	seq_dbWriteSymbolic "${curve}(${i},yAxisLabel)"  $myLabel
	seq_dbWriteSymbolic "${curve}(${i},sampleRate)"  0           ;# Set update on event

	incr i
    }
    
    #
    # Configure not-used curves
    #

    while { $i < $totCurves } {
	seq_dbWriteSymbolic "${curve}(${i},visible)"     0
	seq_dbWriteSymbolic "${curve}(${i},yAxisLabel)"  ""

	# Set update on event
	seq_dbWriteSymbolic "${curve}(${i},sampleRate)"  0
	incr i
    }

    return $nattrib
}

proc sampInitTable { plotPoint totCurves } {

    #  Initialize record indexes
    seq_dbWriteSymbolic ${plotPoint}:plot_table.firstRecord 0
    seq_dbWriteSymbolic ${plotPoint}:plot_table.nextRecord  0

    set attrList [seq_dbGetAttrNames  "${plotPoint}:plot_table"]

    # Initialize indexes 'oldestRecordXX'
    foreach item [lmatch -glob $attrList "oldest*"] {
	seq_dbWriteSymbolic ${plotPoint}:plot_table.${item}  0
    }

    seq_dbWriteSymbolic "${plotPoint}.xAxisScalingMode" 4 ;# Set X scrolling mode

    ############################################################
    #      Initialize table attribute plotPoint.curves()
    ###########################################################

    set i 0 ; set idx 1

    set dbInfo [seq_dbGetAttrInfo "$plotPoint:plot_table.data"]
    lassign $dbInfo attrType dataType numRecs numFields

    set maxRec [expr {$numRecs - 1}]
    set curve  ${plotPoint}.curves

    while { $i < $totCurves } {

	#
	#.curves(0:n,xAttrOrHistTable) = @[plot_table.data(0:599,1)]
	#
	set dirAddr [seq_dbGetDirAddr "$plotPoint:plot_table.data(0:${maxRec},0)"]
	seq_dbWriteSymbolic "${curve}(${i},xAttrOrHistTable)" $dirAddr

	#
	#.curves(0:n,yAttrOrSourceAddr) = @[plot_table.data(0:599,1:${totCurves})]
	#
	set dirAddr [seq_dbGetDirAddr "$plotPoint:plot_table.data(0:${maxRec},$idx)"]
	seq_dbWriteSymbolic "${curve}(${i},yAttrOrSourceAddr)" $dirAddr

	#
	#.curves(0:n,firstIndexerAddr) = @[plot_table.firstRecord]
	#
	set dirAddr [seq_dbGetDirAddr "$plotPoint:plot_table.firstRecord"]
	seq_dbWriteSymbolic "${curve}(${i},firstIndexerAddr)" $dirAddr

	#
	#.curves(0:n,oldestIndexerAddr) = @[plot_table.oldestRecord1-4]
	#
	set dirAddr  [seq_dbGetDirAddr "$plotPoint:plot_table.oldestRecord${idx}"]
	seq_dbWriteSymbolic "${curve}(${i},oldestIndexerAddr)" $dirAddr

	#
	#.curves(0:3,nextIndexerAddr) = @[plot_table.nextRecord]
	#
	set dirAddr [seq_dbGetDirAddr "$plotPoint:plot_table.nextRecord"]
	seq_dbWriteSymbolic "${curve}(${i},nextIndexerAddr)" $dirAddr 

	#
	#.curves(0:3,dataSourceType) = 3  for X versus Y rather than history.
	#
	seq_dbWriteSymbolic "${curve}(${i},dataSourceType)"  3

	incr i; incr idx
    }

    return
}





#
# ___oOo___
