#************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqErrLog.tcl,v 2.97 2005/09/14 09:47:25 vltsccm Exp $" 
#
# 
# who       when      what 
# --------  --------  ---------------------------------------------- 
# eallaert  14/09/94  created 
# 
# seqErrLog.tcl
 
#************************************************************************
#   NAME 
#   seq_errLog - log a single Sequencer error
# 
#   SYNOPSIS 
#   seq_errLog <error> <srcId>
#
#   DESCRIPTION 
#   seq_errLog will handle the logging of a single error for the Sequencer, 
#   i.e. it will first reset the error stack, then log <error> and finally 
#   close (and thereby reset) the error stack.
#   
#   <error> : string containing a description of the error
#   <srcId> : string containing the source identifier
# 
#   ENVIRONMENT 
#   no environment nor Tcl variables are read nor set 
#
#   RETURN VALUES  
#   OK return if no errors occurred.  The result string is empty.
#   ERROR return if seq_errAdd fails, with a corresponding error message as
#   generated by seq_errAdd.
#
#   CAUTIONS  
#   none
#
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   ..... 
#   seqWish>seq_errLog "Who the hell deleted this file??" "file_proc"
#   seqWish>..... 
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
#   seq_errResetStack(n), seq_errAdd(n), seq_errCloseStack(n)
#
#------------------------------------------------------------------------ 

proc seq_errLog {error sourceId} {
	seq_errResetStack; seq_errAdd $error $sourceId; seq_errCloseStack
	return
}

