#************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqLogFits.tcl,v 2.97 2005/09/14 09:47:33 vltsccm Exp $" 
#
# 
# who       when        what 
# --------  ----------  ---------------------------------------------- 
# eallaert  2001-04-20  created 
# 
# seqLogFits.tcl
 
#************************************************************************
#   NAME 
#   seq_logStringParRecord, seq_logIntParRecord, seq_logRealParRecord - 
#   FITS parameter logging for strings, integers and floats
# 
#   SYNOPSIS 
#   seq_logStringParRecord <dictionary> <category> <subsys> <parameter> <value>
#
#   seq_logIntParRecord    <dictionary> <category> <subsys> <parameter> <value>
#
#   seq_logRealParRecord   <dictionary> <category> <subsys> <parameter> <value>
#
#   DESCRIPTION 
#   These commands are all simple wrappers around seq_logFitsParRecord, ensuring
#   that the latter is called with the right option (-string, -int or -real).
# 
#   ENVIRONMENT 
#   no environment nor Tcl variables are read nor set 
#
#   RETURN VALUES  
#   OK return if no errors occurred.  The result string is empty.
#   ERROR return if argument count is wrong or if seq_logFitsParRecord fails, 
#   with a corresponding error message.
#
#   CAUTIONS  
#   none
#
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   ..... 
#   seqWish>seq_logStringRecord OBS OBS "" NAME "this is my name"
#   seqWish>..... 
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
#   seq_logFitsParRecord(n), logFitsParRecord(3)
#
#------------------------------------------------------------------------ 

# None of these procs print debugging info themselves, as they call
# seq_logFitsParRecord anyway.

proc seq_logStringParRecord {dictionary category subsys parameter value} {
    seq_logFitsParRecord -string $dictionary $category $subsys $parameter $value
    return
}


proc seq_logIntParRecord {dictionary category subsys parameter value} {
    seq_logFitsParRecord -int $dictionary $category $subsys $parameter $value
    return
}

proc seq_logRealParRecord {dictionary category subsys parameter value} {
    seq_logFitsParRecord -real $dictionary $category $subsys $parameter $value
    return
}

