#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooAPPLICATION.tcl,v 1.47 2005/08/23 08:02:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  27/05/02  skip applic. start if access=FORBIDDEN.
# rschmutz  17/02/00  created.
#

#************************************************************************
#   NAME
#	stooAPPLICATION - start/stop an application
# 
#   SYNOPSIS
#	stooAPPLICATION app
# 
#   DESCRIPTION
#	stooAPPLICATION is the base class for applications 
#	started/stopped by a main VLT application.
#
#   PUBLIC METHODS
#	Config <rep> <prefix>
#	    Configure the application.
#
#	Start
#	    Start the application.
#	    All processes directly owned by this object are started.
#
#	Stop
#	    Stop the application.
#	    All processes directly owned by this object are stopped.
#
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	    Returns 1 if all processes directly owned by this object
#	    are active.
#	    
#
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	    Returns 1 if all processes directly owned by this object 
#	    are stopped.
#
#	Standby
#	    The application is placed in state Standby.
#	    (default: do nothing).
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooAPPLICATION
#------------------------------------------------------------------------

class stooAPPLICATION {
    inherit ctooOBJ_CONTAINER stooBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg index } {
    	# default: nothing.
    }

    #****************************************************************
    #*# method: Start
    #
    # 1. if application is already running: do nothing.
    # 2. start processes
    #----------------------------------------------------------------

    public method Start {} {
    	
	#*# 1. if application has no processes: do nothing
	set objList [ContentsR stooPROCESS]
	if { "$objList" == "" } {
	    return
	}

	#*# 2. if application is already running: do nothing.
	if { [IsActive] } {
	    Info "INFO : [Desc] is already running."
	    return
	} 
	if { "$access" == "FORBIDDEN" } {
	    Info "INFO : [Desc] not started.  Access = $access."
	    return
	}
	if { $wsSimulation } {
	    Info "START: Starting [Desc] (WS SIMULATION) . . ."
	} else {
	    Info "START: Starting [Desc] . . ."
	}

    	#*# 3. start processes
	foreach obj $objList {
	    if { ![$obj IsActive] } {
	    	$obj Start
	    }
	}	
	Info "       [Desc] started."
    }

    #****************************************************************
    #*# method: Stop
    #
    # 1. check if stopped
    # 2. stop processes
    #----------------------------------------------------------------

    public method Stop {} {

	#*# 1. if application has no processes: do nothing
	set objList [Contents stooPROCESS]
	if { "$objList" == "" } {
	    return
	}

	#*# 2. check if stopped
	if { [IsStopped] } {
	    Info "STOP : [Desc] seems to be stopped.  Stopping anyway."
	} else {
	    Info "STOP : Stopping [Desc] . . ."
	}

    	#*# 3. stop processes
	foreach obj $objList {
	    if { [$obj IsActive] } {
	    	$obj Stop
	    }
	}	
	Info "       [Desc] stopped."

    }

    #****************************************************************
    #*# method: IsActive
    #
    # 1. check processes
    #----------------------------------------------------------------

    public method IsActive {} {

    	#*# 1. check processes
	set ret 1
	set objList [Contents stooPROCESS]
	foreach obj $objList {
	    if { ![$obj IsActive] } {
	    	set ret 0
		break
	    }
	}	

	return $ret
    }

    #****************************************************************
    #*# method: IsStopped
    #
    # 1. check processes
    #----------------------------------------------------------------

    public method IsStopped {} {

    	#*# 1. check processes
	set ret 1
	set objList [Contents stooPROCESS]
	foreach obj $objList {
	    if { [$obj IsActive] } {
	    	set ret 0
		break
	    }
	}	

	return $ret
    }

    #****************************************************************
    #*# method: Standby
    #----------------------------------------------------------------

    public method Standby {} {
    	# default: nothing.
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable default 1
    public variable stop    1
    public variable startWithPanel 0
    public variable wsSimulation 0
    public variable access "NORMAL"
}


#___oOo___
