#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooBOB.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  08/11/02  START.BOB.STOP support added.
# rschmutz  26/05/02  START.BOB.CFGFILE support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooBOB - start/stop BOB
# 
#   SYNOPSIS
#	stooBOB bob
# 
#   DESCRIPTION
#	stooBOB allows to start/stop BOB.
#
#   PARENT
#	stooPANEL
#
#   PUBLIC METHODS
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#
#   FILES
#	$VLTDATA/tmp/<envName>.bob.pid  --  lock file
#
#   SEE ALSO
#	stooPANEL
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooBOB
#------------------------------------------------------------------------

class stooBOB {
    inherit stooPANEL

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	configure -name BOB -execCmd bob -desc "$infoHeader BOB"
    	eval configure $args
	Debug "BOB $name." 5
    }

    #****************************************************************
    #*# method Config
    #----------------------------------------------------------------

    public method Config { cfg } {
    	global stooInsMulti env

	#*# 1. if multiple instruments: add instr. name to BOB's name
	if { $stooInsMulti } {
	    configure -name [cconcat [[Parent] Name] "_BOB"]
	    configure -desc "[[Parent] Name] BOB"
	}

	#*# 2. set BOB configfile
	set txt ""
	catch { set txt [$cfg Get START.BOB.CFGFILE] }
	if { $txt != "" && [file pathtype $txt] != "absolute" } {
	    set txt [file join $env(INTROOT) config $txt]
	}
	configure -configFile $txt
	Debug "BOB $name: configFile = '$configFile'" 5

	#*# 3. set stop flag
	set txt "T"
	catch { set txt [$cfg Get START.BOB.STOP] }
	configure -stop [Logical2Num $txt]
	Debug "BOB $name: stop       = '$multi'" 5

	#*# 4. set default start
	set txt "F"
	catch { set txt [$cfg Get START.BOB.DEFAULT] }
	configure -default [Logical2Num $txt]
	Debug "BOB $name: default    = '$default'" 5
    }

    #****************************************************************
    #*# method SetExecCmd
    #----------------------------------------------------------------

    protected method SetExecCmd {} {
        set cmd bob
	if { "$password" != "" } {
	    append cmd " -password $password"
	}
	if { "$configFile" != "" } {
	    append cmd " -configFile $configFile"
	}
	configure -execCmd $cmd
    }

    #****************************************************************
    #*# attributes
    #----------------------------------------------------------------

    public variable password "" {
    	SetExecCmd
    }
    public variable configFile "" {
    	SetExecCmd
    }
    public variable multi 0
}


#___oOo___
