#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooDCS_IRACE.tcl,v 1.47 2005/08/23 08:02:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  16/08/05  SPR VLTSW20050277 - pass -cfg option to stooIracqCtrl
# dpopovic  28/07/04  Curly braces in expr.
# rschmutz  21/07/02  RTD name simplified.
# rschmutz  08/05/01  set hwSimulation fixed.
# rschmutz  30/04/01  if HW_SIM: pass -sim to iracqStart.
# rschmutz  30/04/01  CheckOption added.
# rschmutz  08/01/01  missing number after -verbose added.
# rschmutz  10/10/00  local flag support dropped.
# rschmutz  25/05/00  OCS.DETi.DEFAULT, OCS.DETi.STOP support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooDCS_IRACE - start/stop detector software
# 
#   SYNOPSIS
#	stooDCS_IRACE dcs
# 
#   DESCRIPTION
#	stooDCS_IRACE allows to start/stop standard detector software.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure the application.
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooDCS_IRACE
#------------------------------------------------------------------------

class stooDCS_IRACE {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        configure -objType DCS
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
        global stooInsMulti env

	#*# 1. name, desc, type, CCD name
	set txt [$cfg Get OCS.DET$i.NAME]
	if { $stooInsMulti } {
	    set rtdName "[$cfg Get INS.CON.ID]_"
	    configure -name [cconcat $rtdName $txt]
	} else {
	    set rtdName ""
	    configure -name $txt
	}
	if { $txt != "DCS" } {
	    append rtdName $txt "_RTD"
	    configure -desc "[$cfg Get INS.CON.ID] DCS $txt"
	} else {
	    append rtdName "RTD"
	    configure -desc "[$cfg Get INS.CON.ID] DCS"
	}
	configure -type [$cfg Get OCS.DET$i.TYPE]
	CheckOption OCS.DET$i.TYPE $type {IRACE}

	#*# 2. process name, server and instance
	set txt "iracqServer"
	catch { set txt [$cfg Get OCS.DET$i.PROCNAME] }
	if { "$txt" == "" } {
	    error "Detector $name process name not known."
	}
	configure -procName $txt -server $txt -inst ""
	regexp {(.+)_([0-9])} $txt dummy server inst

	#*# 3. environment name
	set txt [$cfg Get INS.CON.WSENV]
	catch { set txt [$cfg Get OCS.DET$i.ENVNAME] }
	configure -envName $txt

	#*# 4. SDMAHOST, SYSCFG
	set txt $env(HOST)
 	catch { set txt [$cfg Get OCS.DET$i.SDMAHOST] }
 	configure -sdmaHost $txt
 	configure -sysCfg [$cfg Get OCS.DET$i.SYSCFG]

	#*# 5. TCOMSRV, SDMACMD, SDMADATA, RTDPORT
	set txt 8021
 	catch { set txt [$cfg Get OCS.DET$i.TCOMSRV] }
 	configure -tcomSrv $txt
	set txt [expr {$tcomSrv+1}]
 	catch { set txt [$cfg Get OCS.DET$i.SDMACMD] }
 	configure -sdmaCmd $txt
	set txt [expr {$sdmaCmd+1}]
 	catch { set txt [$cfg Get OCS.DET$i.SDMADATA] }
 	configure -sdmaData $txt
	configure -sdmaErr ""	
	set txt [expr {$sdmaData+1}]
 	catch { set txt [$cfg Get OCS.DET$i.RTDPORT] }
 	configure -rtdPort $txt

	#*# 6. DTT, UIFMOD
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.DTT] }
 	configure -dtt $txt
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.UIFMOD] }
 	configure -uifModule $txt

	#*# 7. DETCFG, TCOMDEV, SDMADEV, RTDCAM, OPTIONS
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.DETCFG] }
 	configure -detCfg $txt
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.TCOMDEV] }
 	configure -tcomDev $txt
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.SDMADEV] }
 	configure -sdmaDev $txt
	set txt [cconcat $name "_RTD"]
 	catch { set txt [$cfg Get OCS.DET$i.RTDCAM] }
 	configure -rtdCamera $txt
	set txt ""
 	catch { set txt [$cfg Get OCS.DET$i.OPTIONS] }
 	configure -options $txt

	#*# 8. access
	set txt "NORMAL"
	catch { set txt [$cfg Get OCS.DET$i.ACCESS] }
	CheckOption OCS.DET$i.ACCESS $txt {NORMAL FORBIDDEN IGNORE} 
	configure -access $txt

	#*# 9. simulation
	set txt "NONE"
	catch { set txt [$cfg Get OCS.DET$i.SWSIM] }
	CheckOption OCS.DET$i.SWSIM $txt {NONE LCU_SIM HW_SIM} 
	configure -simulation $txt
	if { "$simulation" == "LCU_SIM" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}

	#*# 10. stop
	set txt "T"
	catch { set txt [$cfg Get OCS.DET$i.STOP] }
	configure -stop [Logical2Num $txt]

	#*# 11. start application
	set txt "T"
	catch { set txt [$cfg Get OCS.DET$i.DEFAULT] }
	configure -default [Logical2Num $txt]

	#*# 12. start UIF
	set txt "F"
	catch { set txt [$cfg Get OCS.DET$i.STRTUIF] }
	set startUif [Logical2Num $txt]

	#*# 13. start RTD
	set txt "F"
	catch { set txt [$cfg Get OCS.DET$i.STRTRTD] }
	set startRtd [Logical2Num $txt]

	#*# 14. add main process
	#      (only used by method IsActive)
	set obj [stooPROCESS $this.$procName \
		 -name    $procName \
		 -envName $envName \
		 -desc    "[Desc] process $procName"]
	AddObj $obj

	#*# 15. add stand-alone panel
	if { $wsSimulation } {
	    set sdmaHost1 $env(HOST)
	} else {
	    set sdmaHost1 $sdmaHost
	}
	set execCmd "stooIracqCtrl $name $tcomSrv \"$tcomDev\""
	append execCmd " \"$sdmaHost1\" \"$sdmaDev\" $sdmaCmd $sdmaData \"$sdmaErr\""
	append execCmd " \"$rtdCamera\" $rtdPort -host $env(HOST)"
	if { $uifModule != "" } {
	    append execCmd " -load $uifModule"
	}
	if { $server != "iracqServer" } {
	    append execCmd " -server $server"
	}
	if { $inst != "" } {
	    append execCmd " -inst $inst"
	}
	if { $loglevel < 3 } {
	    append execName " > /dev/null"
	} else {
	    append execCmd " -verbose $loglevel"
	}
	# SPR VLTSW20050277 - pass -cfg option
	if { $sysCfg != "" } {
	    append execCmd " -cfg $sysCfg"
	}
	set obj [stooPANEL $this.standalone \
		 -name $name \
	    	 -execCmd $execCmd \
	    	 -execName "racqCtrl" -grepFilter $name \
	    	 -procName [cconcat "iracqCtrl" $name] \
	    	 -envName $envName \
	    	 -desc "[Desc] Stand-alone Panel" \
		 -default $startUif]
	AddObj $obj

	#*# 16. add RTD panel
	set obj [stooPANEL $this.rtd \
		 -name $rtdName \
	    	 -execCmd "stooIrtdStart \"$sdmaHost\" $sdmaData \"$rtdCamera\" $rtdPort $rtdName" \
	    	 -execName "-noKill" \
	    	 -procName $rtdName \
	    	 -envName $envName \
		 -desc "[Desc] RTD Panel" \
		 -stop 1 \
		 -default $startRtd]
	AddObj $obj
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

        #*# 1. if DCS is active: do nothing
	if {[IsActive]} {
	    Info "INFO : [Desc] already running."
	    return
	}
	if { "$access" != "NORMAL" } {
	    Info "INFO : [Desc] not started.  Access = $access."
	    return
	}
	if { "$wsSimulation" } {
	    Info "START: Starting [Desc] (WS SIMULATION) . . ."
	} else {
	    Info "START: Starting [Desc] . . ."
	}

	#*# 2. build start command
	if { $wsSimulation } {
	    set sdmaHost1 $env(HOST)
	} else {
	    set sdmaHost1 $sdmaHost
	}
	set execCmd "stooIracqStart $tcomSrv \"$tcomDev\""
	append execCmd " \"$sdmaHost1\" \"$sdmaDev\" $sdmaCmd $sdmaData \"$sdmaErr\""
	append execCmd " \"$rtdCamera\" $rtdPort"
	if { $sysCfg != "" } {
	    append execCmd " -cfg $sysCfg"
	}
	if { $detCfg != "" } {
	    append execCmd " -det $detCfg"
	}
	if { $server != "iracqServer" } {
	    append execCmd " -server $server"
	}
	if { $dtt != "" } {
	    append execCmd " -dtt $dtt"
	}
	if { $inst != "" } {
	    append execCmd " -inst $inst"
	}
	if { $simulation != "NONE" } {
	    append execCmd " -sim"
	}
	if { $loglevel > 0 } {
	    append execCmd " -verbose $loglevel"
	}
	if { $options != "" } {
	    append execCmd " $options"
	}
	if { $loglevel < 3 } {
	    append execCmd " > /dev/null"
	} 

	#*# 3. start DCS
	Debug "[Desc]: start command='$execCmd'" 2
	if {[ExecCmd [Desc] $execCmd]} {
	    Exit
	}

	#*# 4. check startup
	set objList [Contents stooPROCESS]
	set obj [lvarpop objList]
	if { ![$obj IsActive 90000] } {
	    Exit "Problems starting the [Desc]. Process [$obj Name] not started."
	}
	foreach obj $objList {
	    if { ![$obj IsActive 10000] } {
	        Exit "Problems starting the [Desc]. Process [$obj Name] not started."
	    }
	}
	#if { [seq_dbReadSymbolic "<alias>$ccdName.opState"] != 2 } {
	#    Exit "Problem starting the [Desc]. The state is not LOADED."
	#}

	Info "       [Desc] started."
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
        global env

    	#*# 1. info
	if { [IsStopped] } {
	    Info "STOP : [Desc] seems to be stopped.  Stopping anyway."
	} else {
	    Info "STOP : Stopping [Desc] . . ."
	}

	#*# 2. build stop command
	if { $wsSimulation } {
	    set sdmaHost1 $env(HOST)
	} else {
	    set sdmaHost1 $sdmaHost
	}
	set execCmd "stooIracqStop $tcomSrv \"$tcomDev\""
	append execCmd " \"$sdmaHost1\" \"$sdmaDev\" $sdmaCmd $sdmaData \"$sdmaErr\""
	append execCmd " \"$rtdCamera\" $rtdPort"
	if { $server != "iracqServer" } {
	    append execCmd " -server $server"
	}
	if { $inst != "" } {
	    append execCmd " -inst $inst"
	}
	if { $loglevel > 0 } {
	    append execCmd " -verbose $loglevel"
	}
	if { $loglevel < 3 } {
	    append execCmd " > /dev/null"
	}

	#*# 3. stop DCS
	Debug "[Desc]: stop command='$execCmd'" 2
	if {[ExecCmd [Desc] $execCmd]} {
	    Warning "ERROR: Error stopping [Desc]."
	}
	Info "       [Desc] stopped."
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable type
    public variable procName
    public variable envName
    public variable sdmaHost
    public variable tcomDev
    public variable tcomSrv
    public variable sdmaDev
    public variable sdmaCmd
    public variable sdmaData
    public variable sdmaErr
    public variable rtdPort
    public variable rtdCamera
    public variable server
    public variable dtt
    public variable inst
    public variable uifModule
    public variable sysCfg
    public variable detCfg
    public variable options
    public variable simulation {
    	if { "$simulation" != "NONE" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}
    }
    public variable startUif
    public variable startRtd
}


#___oOo___
