#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooINS.tcl,v 1.47 2005/08/23 08:02:48 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  23/08/05  Removed additional stooType(ICS2-10). 
#                     Taken Ricardo's approach. There is no limit to the number of ICSs.
# dpopovic  09/08/05  VLTSW20050111: Addition of stooType(ICS2-10)
# rschmutz  27/05/02  reset stooInsMulti only if not set.
# rschmutz  09/01/01  START.CON.STBYAPP support added.
# rschmutz  08/12/00  START.CON.STARTAPP support added.
# rschmutz  07/12/00  stooDCS replaced with individual DCS types.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooINS - start/stop instrumentation software
# 
#   SYNOPSIS
#	stooINS ins
# 
#   DESCRIPTION
#	This is the main class responsible for starting/stopping
#	standard instrumentation software.
#	The startups is performed according to the following steps:
#	1) The instrument configuration is read, creating the
#	   corresponding startup instrument objects (Init)
#	2) The command line options are processed, to define
#	   what processes and panels shall be started/stopped.
#	3) The processes and panels are started/stopped.
#
#   PARENT CLASS
#       stooSIMPLE
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Reads the instrument configuration and stores it
#	    in the repository.
#
#   PUBLIC PROCEDURES
#	ItclTypes
#	    Registers the Itcl types used by the instrument.
#
#   SEE ALSO
#	stooSIMPLE
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooINS
#------------------------------------------------------------------------

class stooINS {
    inherit stooSIMPLE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	configure -objType STARTUP -name INS
    	eval configure $args
    }

    #****************************************************************
    #*# proc ItclTypes
    #----------------------------------------------------------------
    
    proc ItclTypes {} {
    	global stooType stooInsMulti

	#*# 1. init. the Itcl types
	set stooType(STARTUP)    stooINS
	set stooType(INSTRUMENT) stooINSTRUMENT
	set stooType(OS)	 stooOS
	set stooType(TCS)	 stooINS_TCS
	set stooType(ICS)	 stooICS
	set stooType(DCS_ACE)	 stooDCS_ACE
	set stooType(DCS_FIERA)	 stooDCS_FIERA
	set stooType(DCS_IRACE)	 stooDCS_IRACE
	set stooType(MIDAS)	 stooMIDAS
	set stooType(BOB)	 stooBOB
	set stooType(ALARM)	 stooALARM

	#*# 2. normally we do not start multiple instruments
	if { ![info exists stooInsMulti] } {
	    set stooInsMulti 0
	}
    }

    #****************************************************************
    #*# method Config
    #----------------------------------------------------------------

    public method Config { cfg } {
        global stooType stooInsMulti

	#*# 1. set loglevel
	set txt [cget -loglevel]
	if { $txt == 0 } {
	    catch { set txt [$cfg Get OCS.CON.LOGLEVEL] }
	    configure -loglevel $txt
	}

	#*# 2. init. startApp, stbyApp
	set txt "T"
	catch { set txt [$cfg Get START.CON.STARTAPP] }
	configure -startApp [Logical2Num $txt]
	set txt "T"
	catch { set txt [$cfg Get START.CON.STBYAPP] }
	configure -stbyApp [Logical2Num $txt]

	#*# 3. load the instrument configuration (default type: stooINSTRUMENT)
	set instrument [$stooType(INSTRUMENT) $this.instrument]
	AddObj $instrument
	$instrument Config $cfg
    }

}


#___oOo___
