#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooINS_TCS.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  30/04/01  CheckOption added.
# rschmutz  10/10/00  OCS.TEL.TCSID replaced with OCS.TEL.NAME.
# rschmutz  10/10/00  simulation and timeout support dropped.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooINS_TCS - start/stop TCS
# 
#   SYNOPSIS
#	stooTCS tcs
# 
#   DESCRIPTION
#	stooTCS is currently a dummy.
#	It is assumed that the scanlinks to TCS have been enabled
#	during installation and do not need to be disabled during
#	operation.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Configure the application.
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooINS_TCS
#------------------------------------------------------------------------

class stooINS_TCS {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType TCS
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {
    	global env


    	#*# 1. name, desc, tcsid
	set txt [$cfg Get OCS.TEL.NAME]
	configure -name $txt -desc "TCS $txt"
	set txt 0
	if { [ctype digit [cindex $name 2]] } {
	    set txt [cindex $name 2]
	}
	configure -tcsid $txt
	Debug "Subsystem TCS $name:" 5

    	#*# 2. focus
	set txt ""
	catch { set txt [$cfg Get OCS.TEL.FOCUS] }
	configure -focus $txt
	Debug "TCS: focus      = '$focus'." 5

    	#*# 3. procName
    	set txt [cconcat "tif" $focus]
	catch { set txt [$cfg Get OCS.TEL.PROCNAME] }
	configure -procName $txt
	Debug "TCS: procName   = '$procName'." 5

    	#*# 4. envName
    	set txt [cconcat "wt" $tcsid "tcs"]
	catch { set txt [$cfg Get OCS.TEL.ENVNAME] }
	configure -envName $txt
	Debug "TCS: envName    = '$envName'." 5

    	#*# 5. access
	set txt "NORMAL"
	catch { set txt [$cfg Get OCS.TEL.ACCESS] }
	CheckOption OCS.TEL.ACCESS $txt {NORMAL FORBIDDEN IGNORE} 
	configure -access $txt
	Debug "TCS: access  = '$access'." 5

	#*# 6. check
	if { "$access" == "NORMAL" } {
	    if {[catch { set env(TCS_ENVNAME) }]} {
	        error "Env. var. TCS_ENVNAME is not set."
	    }
	    if { "$env(TCS_ENVNAME)" != "$envName" } {
	        error "The env. var. TCS_ENVNAME is not set correctly to '$envName'."
	    }
	    if {![catch { set env(TCSENV) }]} {
	        if { "$env(TCSENV)" != "$envName" } { 
	            error "The env. var. TCSENV is not set correctly to '$envName'."
	        }
	    }
	}
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

	#*# 1. Show TCS_ENVNAME
	Debug "TCS_ENVNAME is $env(TCS_ENVNAME)" 2

	#*# 2. Configure TCS scan links
set comment1 {
	if { "$access" == "NORMAL" } {
	    StartTcsScanLinks $tcsName
	} else {
	    StopTcsScanLinks $tcsName
	}
}
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
    	# nothing to do.
    }

    #****************************************************************
    #*# method: IsActive
    #----------------------------------------------------------------

    public method IsActive {} {
	return 1
    }

    #****************************************************************
    #*# method: IsStopped
    #----------------------------------------------------------------

    public method IsStopped {} {
	return 0
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable tcsid 0 {
    	configure -envName [cconcat "wt" $tcsid "tcs"]
    }
    public variable focus "CA" {
    	configure -procName [cconcat "tif" $focus]
    }
    public variable procName	"tifCA"
    public variable envName	"wt0tcs"
}


#___oOo___
