#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooSTARTUP_BASE.tcl,v 1.47 2005/08/23 08:02:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooSTARTUP_BASE - base for objects that start/stop software
# 
#   SYNOPSIS
#	stooSTARTUP_BASE name
# 
#   DESCRIPTION
#	This is a base class for the main object responsible for
#	starting/stopping some type of software.
#	The startups is performed according to the following steps:
#	1) The instrument configuration is read, creating the
#	   corresponding startup instrument objects (Config)
#	2) The command line options are processed, to define
#	   what processes and panels shall be started/stopped.
#	3) The processes and panels are started/stopped.
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Reads the instrument configuration and stores it then
#	    repository.
#
#	Start <options>
#	    Starts the software.
#
#	Stop <options>
#	    Stops the software.
#
#   FILES
#	<name>.cfg - configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooSTARTUP_BASE
#------------------------------------------------------------------------

class stooSTARTUP_BASE {
    inherit ctooOBJ_CONTAINER stooBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	configure -objType STARTUP -name NONAME
    	eval configure $args
    }

    #****************************************************************
    #*# proc ItclTypes
    #----------------------------------------------------------------

    proc ItclTypes {} {
    	global stooType

	# stooType(STARTUP) should be set by the derived class
	set stooType(STARTUP)    stooSTARTUP_BASE
    }

    #****************************************************************
    #*# method Config
    #----------------------------------------------------------------

    public method Config { cfg } {
    	# must be overloaded
    }

    #****************************************************************
    #*# method Start
    #----------------------------------------------------------------

    public method Start { {options ""} } {
    	# must be overloaded

	Info "START: [exec date]"
	Info "ARGS : $options."

	Info "START: Not implemented."

	Info "END  : Start end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method Stop
    #----------------------------------------------------------------

    public method Stop { {options ""} } {
    	# must be overloaded

    	#*# 1. if the environment is not running, skip everything
	Info "STOP : [exec date]"
	Info "ARGS : $options."
	if { ![IsWsEnvActive] } {
	    Exit "Environment is not running."
	}

	Info "STOP : Not implemented."

	Info "END  : Stop end."
	Info "END  : [exec date]"
    }

}


#___oOo___
