#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooSosStartup.tcl,v 1.47 2005/08/23 08:02:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  09/12/03  created


#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#****************************************************************
#*# proc: stooSosStartupInitVars
#----------------------------------------------------------------

proc stooSosStartupInitVars {} {
    global gvar cv stooType argv argv0 insid EDITING panWidgetHlp ::cfgSos
 
    #*# 1. get instrument name, detectors and user extensions file
    #
    #   stooSosStartup.pan instrument widget reads gvar(insid)
    #   stooSosStartup<x>_uifClass.tcl read gvar(cfg_detName<n>)
    #	userTclFile may contain a user uifClass.
    #

    # Save the SOS name into insid
    # Somewhere we need gvar(insid) instead of insid
    set insid       $argv
    set gvar(insid) $insid


    #
    # Get the list of sub-OS's.
    # The argument passed to stooSosStartup is the instrument identifier. 
    # For example: 
    #   stooSosStartup FLAMES     i.e. argv = FLAMES
    #

    # Create in instance of ctooCONFIG in order to
    # read the instrument configuration.
    catch { delete object ::cfgSos }
    ctooCONFIG ::cfgSos $insid


    # Get the instrument prefix. The keyword must exist!
    set gvar(insPrefix) [cfgSos Get INS.CON.PREFIX]	


    # Get the number of sub-OS's that run in the same 
    # environment as the SOS. Store their names as well.
    # WARNING: It is assumed that the sub-OS's are listed in such
    # an order that the ones that run in the same environment as 
    # the SOS are listed first and then the others.
    # For example, in case of FLAMES (running in wflames):
    # OS1 = FP      (wflames)  - shown on panel
    # OS2 = GIRAFFE (wflames)  - shown on panel
    # OS3 = UVES    (wuves)    - shown on panel
    set osNumUsed  0
    set osNumTotal [cfgSos Get OCS.OS.NUM]
    if { $osNumTotal == 0 } {
	tk_dialog .dialog "ERROR" "Instrument $insid is not an SOS" error 0 OK
	exit 1
    }
    set sosEnv     [cfgSos Get INS.CON.WSENV]

    loop i 1 [ expr {$osNumTotal+1} ] {
	set osEnv [cfgSos Get OCS.OS$i.ENVNAME]
	if { [ string compare $osEnv $sosEnv] == 0 } {
	    set  osName($i) [cfgSos Get OCS.OS$i.NAME]
	    incr osNumUsed
	} 
    }
    set gvar(osNum) $osNumUsed



    # Get name of the 'real' frame widget
    set frameWidget [$cv.cfg component uifFrame]

    # Set the size of the canvas to match the size of the frame

    # WARNING Scroll does not work! It should be 'canvas' without scroll.
    set canvasName [panCreateScrollCanvas $frameWidget 444 354 0 0 "0 0 444 354"]

    $canvasName create text 20 15 -text "  \#    Name       Available   OS Startup Panel" -anchor nw \
	-font -Adobe-courier-bold-r-normal--14*-iso8859-1

    loop i 1 [expr { $gvar(osNum) + 1}] {
	
	# Set widget instance name
	set iname osData${i}
	set w $canvasName.$iname

	stooSosStartup_uifClass $w -cwp "" -relief raised -library libstooTcl

        # Display widget onto canvas
	set ypos [expr {35 + ($i - 1) * 50}]

	$canvasName create window 5 $ypos -window $w  -anchor nw

	${w} display

	# Set default values
	set gvar(${iname}_osNameVar)     $osName($i)
	set gvar(${iname}_osNumVar)      OS$i

	${w}.cv.ignoreFlag configure -variable gvar(cfg__OCS.OS${i}.ACCESS)
	${w}.cv.osStartup  configure -command  "panCallPanel stooStartup $osName($i)"
    }
	
    # Add miscellaneous class and read config keywords
    set w $canvasName.wdgMisc
    stooStartupMisc_uifClass $w -relief flat -library libstooTcl.tcl
    set y 240
    $canvasName create window 0 $y -window $w -anchor nw
    $w display
    $w.cv.bob    configure -variable gvar(cfg__START.BOB.DEFAULT)
    $w.cv.osctrl configure -variable gvar(cfg__START.OSCTRL.DEFAULT)
    $w.cv.alarm  configure -variable gvar(cfg__START.ALARM.DEFAULT)

    # Default values
    loop i 1 [expr { $gvar(osNum) + 1}] {
	set gvar(cfg__OCS.OS${i}.ACCESS) "NORMAL"
	set gvar(cfg__OCS.OS${i}.ACCESS) [cfgSos Get OCS.OS${i}.ACCESS] 
    }
    set gvar(cfg__START.BOB.DEFAULT)    "F"
    set gvar(cfg__START.OSCTRL.DEFAULT) "F"
    set gvar(cfg__START.ALARM.DEFAULT)  "F"
    set gvar(cfg__OCS.CON.LOGLEVEL)     0

    # Read keyword values set in the START file. 
    # These keywords should be in START file. Otherwise, they will be saved
    # in the INS file, which is not exactly what we would like.
    catch { set gvar(cfg__START.BOB.DEFAULT)    [cfgSos Get START.BOB.DEFAULT] }
    catch { set gvar(cfg__START.OSCTRL.DEFAULT) [cfgSos Get START.OSCTRL.DEFAULT] }
    catch { set gvar(cfg__START.ALARM.DEFAULT)  [cfgSos Get START.ALARM.DEFAULT] }
    catch { set gvar(cfg__OCS.CON.LOGLEVEL)     [cfgSos Get OCS.CON.LOGLEVEL] }
}



#****************************************************************
#*# proc: stooSosSave
#----------------------------------------------------------------

proc stooSosSave {} {
    global gvar ::cfgSos

    # Set ACCESS flags
    loop i 1 [expr { $gvar(osNum) + 1}] {
	cfgSos Set OCS.OS${i}.ACCESS $gvar(cfg__OCS.OS${i}.ACCESS)
    }
    
    # Set Misc flags
    cfgSos Set START.BOB.DEFAULT    $gvar(cfg__START.BOB.DEFAULT)
    cfgSos Set START.OSCTRL.DEFAULT $gvar(cfg__START.OSCTRL.DEFAULT)
    cfgSos Set START.ALARM.DEFAULT  $gvar(cfg__START.ALARM.DEFAULT)
    cfgSos Set OCS.CON.LOGLEVEL     $gvar(cfg__OCS.CON.LOGLEVEL)


    if {[catch { cfgSos Save } msg]} {
	set text "Error executing cmd.: $cmd\n $msg."
	tk_dialog .dialog "Dialog Box" $text info 0 OK	
	return
    }
}


#****************************************************************
#*# proc: stooSosRestart
#----------------------------------------------------------------

proc stooSosRestart {} {
    global gvar cv wdList insid ::cfgSos

    #*# 1. Update configuration
    stooSosSave

    #*# 2. Call stooStart
    set cmd "stooStart $insid -log -restart -nopopup"
    if {[catch { exec stooStart $insid -log -restart -nopopup & } msg]} {
	tk_dialog .dialog "Dialog Box" "Error executing cmd.: $cmd\n $msg" info 0 OK	
	return
    }
   
    #*# 3. Close the panel
    sleep 5
    panClosePanel
}


#****************************************************************
#*# proc: stooSosStartupExitVars
#----------------------------------------------------------------

proc stooSosStartupExitVars {} {
    global gvar cfg

}



#
# ___oOo___
