#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooStartup.tcl,v 1.47 2005/08/23 08:02:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  25/10/04  Hidden DCS if STOOSHOW != T.
# dpopovic  28/07/04  Missing 'else' added to eliminate tcl warnings.
# alongino  09/09/03  VLTSW20030352
# rschmutz  30/04/01  tell the user when settings are modified
# rschmutz  09/12/00  name parameter added.
# mkiekebu  03/11/99  Changed attribute for FIERA simulation
# mkiekebu  14/01/98  added FIER and Init LCUs
# mkiekebu  22/12/98  adapted to FORS from TC


#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#****************************************************************
#*# proc: stooStartupInitVars
#----------------------------------------------------------------

proc stooStartupInitVars {} {
    global gvar cv stooType argv argv0 insid EDITING
 
    #*# 1. get instrument name, detectors and user extensions file
    #
    #   stooStartup.pan instrument widget reads gvar(insid)
    #   stooStartup<x>_uifClass.tcl read gvar(cfg_detName<n>)
    #	userTclFile may contain a user uifClass.
    #
    set userTclFile ""
    set detNum 1
    set telNum 1
    set icsNum 1


    # Check which DCS should not be created on the GUI
    # Keyword OCS.DETi.STOOSHOW defines this.
    # By default OCS.DETi.STOOSHOW is T.
    set detNumShow 0

    if { $EDITING == 0 } {
        if { $argv == "" } {
    	    puts "Usage: $argv0 <name>"
	    exit
        }
        set insid [lvarpop argv]
        ctooCONFIG inscfg $insid

	# Check if OS is available, i.e. if it exists
	set osAvail "T"
	catch { set osAvail [inscfg Get OCS.CON.AVAIL] }
	

        set gvar(insid) [inscfg Get INS.CON.ID]
        set detNum [inscfg Get OCS.DET.NUM]
        loop i 1 [expr {$detNum + 1}] {
    	    set gvar(cfg_detName$i) [inscfg Get OCS.DET$i.NAME]

	    # Check if this DCS should be shown on the GUI
	    if { [catch { set gvar(showDCS${i}) [inscfg Get OCS.DET$i.STOOSHOW] }] } { 
		set gvar(showDCS${i}) "T"
	    }
	    if { $gvar(showDCS${i}) == "T" } { incr detNumShow }
        }
        if { ![catch { set icsNum [inscfg Get OCS.INS.NUM] }] } {
            loop i 1 [expr {$icsNum +1} ] {
                if { [catch { set gvar(cfg_icsName$i) [inscfg Get OCS.INS$i.NAME] }] } { 
                    set gvar(cfg_icsName$i) ICS$i
                }
	    }
        }
        if { [catch { set gvar(cfg_telName) [inscfg Get OCS.TEL.NAME] }] } { set telNum 0 }
        catch { set userTclFile [inscfg Get START.CON.TCLFILE] }
        delete object inscfg

    }
    #*# 2. source the user extensions, get user uifClass
    #
    #   The user tcl file should usually update the stooType array.
    #   Value stooType(UIF_CLASS) may set the uifClass used by
    #   stooStartup.
    #
    if { "$userTclFile" != "" } {
	set userTclFile [seq_findFile [cconcat "bin/" $userTclFile]]
	if {[catch { uplevel #0 source $userTclFile } msg]} {
	    tk_dialog .dialog "Dialog Box" "Error loading $userTclFile: $msg" info 0 OK	
	    exit
	}
    }

    #*# 3. create canvas
    #

    # Compute geometry
    set top [$cv.cfg component uifFrame]
    set xsize [$cv.cfg cget -width]
    set ysize [$cv.cfg cget -height]
    set guiGeometry [split [wm geometry .] x+]
    set guiWidth  [lindex $guiGeometry 0]
    set guiHeight [lindex $guiGeometry 1]
    set guiX      [lindex $guiGeometry 2]
    set guiY      [lindex $guiGeometry 3]
    set guiXOffset [expr {$guiWidth - $xsize}]
    set guiYOffset [expr {$guiHeight - $ysize}]

    # Create frame canvas
    set geometry [list 0 0 $xsize $ysize]
    set scanvas $top.diagCanvas
    canvas $scanvas -width $xsize -height $ysize
    pack $scanvas -side left -fill both -expand true

    #*# 4. show the right cfg. class
    #
    #   if there is a user uifClass: load it,
    #   else add as many TCS, ICS and DCS widgets as from the configuration
    #

    set userUifClass ""
    catch { set userUifClass $stooType(UIF_CLASS) }
    if { "$userUifClass" != "" } {
        # Add user provided class
        set w $scanvas.cfg
        $userUifClass $w -relief flat
        $scanvas create window 0 0 -window $w -anchor nw
        $w display
    } else {
        if {$telNum > 0} {
            # Add TCS labels
            $scanvas create text 20 35 -text "Available" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            # Add TCS class
            set w $scanvas.wdgTcs
            stooStartupTCS_uifClass $w -relief flat -library libstooTcl.tcl
            $scanvas create window 80 0 -window $w -anchor nw
            $w display
            $w.cv.name    configure -variable gvar(cfg_telName)
            $w.cv.access  configure -variable gvar(cfg__OCS.TEL.ACCESS)
            set width  [$w.cv cget -width]
            if {$width > $xsize} { set xsize $width }
            set height [$w.cv cget -height]
            if {$height > $ysize} { set ysize $height }
	}
        if {$icsNum > 0} {
            # Add ICS labels
            $scanvas create text 180 35 -text "Available" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            $scanvas create text 150 60 -text "WS Simulation" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            $scanvas create text 150 85 -text "User Interface" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            loop i 1 [expr {$icsNum + 1}] {
                # Add ICS class
                set w $scanvas.wdgIcs$i
                stooStartupICS_uifClass $w -relief flat -library libstooTcl.tcl
                set width [$w.cv cget -width]
                set x [expr {150 + $width * $i}]
                $scanvas create window $x 0 -window $w -anchor nw
                $w display
                $w.cv.name    configure -variable gvar(cfg_icsName$i)
                $w.cv.access  configure -variable gvar(cfg__OCS.INS$i.ACCESS)
                if {$i == 1} { set j ""
                } else { set j $i }
                $w.cv.swsim   configure -variable gvar(cfg__INS$j.CON.OPMODE)
                $w.cv.strtuif configure -variable gvar(cfg__OCS.INS$i.STRTUIF)
		set width [expr {$x + $width}]
                if {$width > $xsize} { set xsize $width }
	    }
	}
        if {$detNumShow > 0} {
            # Add DCS labels
            $scanvas create text 50 145 -text "Available" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            $scanvas create text 20 170 -text "WS Simulation" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            $scanvas create text 20 195 -text "User Interface" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*
            $scanvas create text 50 220 -text "Start RTD" -anchor nw \
                     -font -Adobe-helvetica-medium-r-normal--14*

	    # DCS widgets are created if STOOSHOW==T
	    # The numDcsDisplayed variable is used to determine the x-position 
	    # of the DCS widget. It is incremented only if the widget is 
	    # created in order to avoid horizontal gaps between DCS widgets.
	    set numDcsDisplayed 1
            loop i 1 [expr {$detNum + 1}] {
		# Skip the widget creation if STOOSHOW != T
		if { $gvar(showDCS${i}) != "T" } { 
		    continue
		}
		
		# Add DCS class
		
		set w $scanvas.wdgDcs$i
		stooStartupDCS_uifClass $w -relief flat -library libstooTcl.tcl
		# The width of the DCS widget
		set width [$w.cv cget -width]
		# The x-position of the DCS widget.
		set x [expr {30 + $width * $numDcsDisplayed}]
		$scanvas create window $x 110 -window $w -anchor nw
		$w display
		$w.cv.name    configure -variable gvar(cfg_detName$i)
		$w.cv.access  configure -variable gvar(cfg__OCS.DET$i.ACCESS)
		$w.cv.swsim   configure -variable gvar(cfg__OCS.DET$i.SWSIM)
		$w.cv.strtuif configure -variable gvar(cfg__OCS.DET$i.STRTUIF)
		$w.cv.strtrtd configure -variable gvar(cfg__OCS.DET$i.STRTRTD)
		set width [expr {$x + $width}]
		if {$width > $xsize} { set xsize $width }
		incr numDcsDisplayed
	    }
	}
	
        # Add miscellaneous class
	# There are two classes, one with OS and the other without it.
	# osctrl check button exists with OS.
        set w $scanvas.wdgMisc
	if { "$osAvail" == "T" } {
	    stooStartupMisc_uifClass $w -relief flat -library libstooTcl.tcl
	} else {
	    stooStartupMiscNoOS_uifClass $w -relief flat -library libstooTcl.tcl
	}
        set y [expr {$ysize - [$w.cv cget -height]}]
        $scanvas create window 0 $y -window $w -anchor nw
        $w display
        $w.cv.bob    configure -variable gvar(cfg__START.BOB.DEFAULT)
        $w.cv.alarm  configure -variable gvar(cfg__START.ALARM.DEFAULT)
	if { "$osAvail" == "T" } {
	    $w.cv.osctrl configure -variable gvar(cfg__START.OSCTRL.DEFAULT)
	}
    }

    # Resize canvas and panel
    $scanvas configure -width  $xsize -height $ysize
    incr xsize $guiXOffset
    incr ysize $guiYOffset
    wm geometry . ${xsize}x${ysize}+${guiX}+${guiY}

    #*# 5. load START.cfg
    if { $EDITING == 0 } {
        if { [catch { ctooCONFIG::UifCreate [cconcat $insid "_START"] }] } {
            ctooCONFIG::UifCreate $insid; # ALO: if XXXX_START set not found, try with XXXX set
	}
    }
    set gvar(telNum) $telNum; # for backward compatibility, pass parameter as global
    set gvar(icsNum) $icsNum; # for backward compatibility, pass parameter as global
    stooConfigNormalize $detNum

}


#****************************************************************
#*# proc: stooConfigNormalize
#----------------------------------------------------------------

proc stooConfigNormalize { detNum } {
    global gvar

    set txt ""

    #*# 1. TCS ACCESS: only NORMAL and IGNORE
    if {[info exist gvar(telNum)]} { set telNum $gvar(telNum)
    } else { set telNum 1 }
    loop i 1 [expr {$telNum + 1}] {
        set name TCS
        set key OCS.TEL.ACCESS
	set val $gvar(cfg__$key)
	if { [lcontain {NORMAL FORBIDDEN IGNORE} $val] } {
            if { $val == "FORBIDDEN" } {
	        append txt "$name access was FORBIDDEN.  Set to IGNORE.\n"
    	        set gvar(cfg__$key) "IGNORE"
	    }
	} else {
	    append txt "Wrong $name access = $val.  Set to IGNORE.\n"
    	    set gvar(cfg__$key) "IGNORE"
	}
    }

    #*# 2. all ICSi ACCESS: only NORMAL and IGNORE
    #      all ICSi SWSIM:  only NORMAL and LCU_SIM
    if {[info exist gvar(icsNum)]} { 
	set icsNum $gvar(icsNum)
    } else { 
	set icsNum 1 
    }
    loop i 1 [expr {$icsNum + 1}] {
        if {$i > 1 } { set suffix $i 
        } else { set suffix "" }
        set name ICS$suffix
        set key OCS.INS$i.ACCESS
	set val $gvar(cfg__$key)
	if { [lcontain {NORMAL FORBIDDEN IGNORE} $val] } {
            if { $val == "FORBIDDEN" } {
	        append txt "$name access was FORBIDDEN.  Set to IGNORE.\n"
    	        set gvar(cfg__$key) "IGNORE"
	    }
	} else {
	    append txt "Wrong $name access = $val.  Set to IGNORE.\n"
    	    set gvar(cfg__$key) "IGNORE"
	}
        set val $gvar(cfg__INS$suffix.CON.OPMODE)
        if { ![lcontain {NORMAL LCU_SIM} $val] } {
	    append txt "Wrong $name simulation = $val.  Set to LCU_SIM.\n"
    	    set gvar(cfg__INS$suffix.CON.OPMODE) "LCU_SIM"
        }
    }

    #*# 3. all DETi ACCESS: only NORMAL and FORBIDDEN
    #      all DETi SWSIM:  only NONE   and LCU_SIM
    loop i 1 [expr {$detNum + 1}] {
	# Skip the widget if it does not exist
	if {[catch {set val $gvar(cfg__OCS.DET${i}.ACCESS)}]} {
	    continue
	}
	    
	set val $gvar(cfg__OCS.DET${i}.ACCESS)
	if { [lcontain {NORMAL FORBIDDEN IGNORE} $val] } {
            if { $val == "IGNORE" } {
	        append txt "DET$i access was IGNORE.  Set to FORBIDDEN.\n"
    	        set gvar(cfg__OCS.DET${i}.ACCESS) "FORBIDDEN"
	    }
	} else {
	    append txt "Wrong DET$i access = $val.  Set to FORBIDDEN.\n"
    	    set gvar(cfg__OCS.DET${i}.ACCESS) "FORBIDDEN"
	}
	set val $gvar(cfg__OCS.DET${i}.SWSIM)
	if { [lcontain {NONE HW_SIM LCU_SIM} $val] } {
            if { $val == "HW_SIM" } {
	        append txt "DET$i simulation was HW_SIM.  Set to LCU_SIM.\n"
    	        set gvar(cfg__OCS.DET${i}.SWSIM) "LCU_SIM"
	    }
	} else {
	    append txt "Wrong DET$i simulation = $val.  Set to LCU_SIM.\n"
    	    set gvar(cfg__OCS.DET${i}.SWSIM) "LCU_SIM"
        }
    }

    #*# 4. tell user about changes
    if { $txt != "" } {
    	append txt "\nIf you need different settings, then please use the engineering interface."
	tk_messageBox -icon warning -message $txt
    }
}


#****************************************************************
#*# proc: stooRestart
#----------------------------------------------------------------

proc stooRestart {} {
    global gvar cv wdList insid

    #*# 1. Update configuration
    ctooCONFIG::UifSave

    #*# 2. Call stooStart
    set cmd "stooStart $insid -log -restart -nopopup"
    if {[catch { exec stooStart $insid -log -restart -nopopup & } msg]} {
	tk_dialog .dialog "Dialog Box" "Error executing cmd.: $cmd\n $msg" info 0 OK	
	return
    }
   
    #*# 3. Close the panel
    sleep 5
    panClosePanel
}


#****************************************************************
#*# proc: stooStartupExitVars
#----------------------------------------------------------------

proc stooStartupExitVars {} {
    global gvar cfg

    ctooCONFIG::UifQuit
    delete object cfg
}




#****************************************************************
#*# proc: stooIsSos
#----------------------------------------------------------------

# Check if this is an OS or SOS
# If it is an OS,  return 0
# If it is an SOS, return 1
proc stooIsSos {} {
    global argc argv argv0 insid EDITING
    
    
    if { $EDITING == 0 } {
        if { $argv == "" } {
    	    puts "Usage: $argv0 <name>"
	    exit
        }
	
	# Create in instance of ctooCONFIG in order to
	# read the instrument configuration.
        set insid $argv
	catch { delete object inscfgSOS }
	if { [catch { ctooCONFIG inscfgSOS $insid } msg] } {
	    set text1 "$msg\n\nCannot start $insid Startup Panel."
	    tk_dialog .dialog "WARNING" "$text1" info 0 OK	
	    exit
	}
	
	#
	# Check if this is an SOS or OS
	# If it is an SOS, OCS.OS.NUM should be greater than 0.
	#
	set osNumTotal [inscfgSOS Get OCS.OS.NUM]
	if { $osNumTotal > 0 } {
	    # This is an SOS
	    return 1
	} else {
	    # This is an OS
	    return 0
	}
    }
}


#
# ___oOo___
