#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tacGuiPublic.tcl,v 1.55 2005/10/03 17:37:52 vltsccm Exp $"
#
# who      when      what
# -------  --------  ----------------------------------------------
# bbauvir  31/12/02  Modify the RECORD command parameters
# bbauvir  12/07/01  Created
#

#************************************************************************
#   NAME
#     tacState - 
#
#   SYNOPSIS
#     tacSendStandardCommand $cmd
#
#   DESCRIPTION
#     The procedure tacSendStandardCommand sends a standard command to the
#     LCU server '$gvar(modName)Server' with the parameter $gvar(modName).
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc tacSendStandardCommand { env timeout cmd } {
    global gvar 

    panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout
}

proc tacSendRecordCommand { env file block sample timeout cmd } {
    global gvar

    if {$cmd == "RECORD"} {
        panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout $file,$block,$sample
    } else {
        panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout
    }
}

proc tacSendConfigCommand { env file timeout cmd } {
    global gvar

    panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout $file

    tacUpdateBlockList
    tacUpdateLinkList
}

proc tacSendBlockCommand { env type name p0 p1 p2 p3 p4 p5 p6 p7 p8 p9 timeout cmd } {
    global gvar

    set paramIndex 0
    set paramList "$p0 $p1 $p2 $p3 $p4 $p5 $p6 $p7 $p8 $p9"
    set paramBuffer ""

    while {($paramIndex < 10) && ([lindex $paramList $paramIndex] != "")} {
	set paramBuffer [format "%s,%s" $paramBuffer [lindex $paramList $paramIndex]]
	incr paramIndex
    }

    if {$cmd == "ADDBLCK"} {
	set cmdBuffer "$type,$name"
	set cmdBuffer [format "%s%s" $cmdBuffer $paramBuffer]
    } else {
    if {$cmd == "MODBLCK"} {
	set cmdBuffer "$name"
	lappend cmdBuffer $paramBuffer
	set cmdBuffer
    } else {
    if {$cmd == "DELBLCK"} {
	set cmdBuffer "$name"
    } } } 

    panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout $cmdBuffer

    tacUpdateBlockList
}

proc tacSendLinkCommand { env name srcName srcPort dstName dstPort timeout cmd } {
    global gvar

    if {$cmd == "ADDLINK"} {
        panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout $name,$srcName,$srcPort,$dstName,$dstPort
    } else {
    if {$cmd == "DELLINK"} {
        panSendCommand $env [format "%sServer" $gvar(modName)] $cmd "" $timeout $name
    } }

    tacUpdateLinkList
}

proc tacCheckLcuEnv {} {
    global env lcuEnv argv 

    # Hardcoded default
    set lcuEnv lted

    # ENV variable -> Override
    if {[catch {set lcuEnv $env(TACENV)}]} {
        puts "TACENV variable not set\n"
    }

    if { $argv != "" } {
        if { [lindex $argv 0] == "-e" } {
            set lcuEnv [lindex $argv 1]
        } else {
            if { $lcuEnv == "lted" } {
                puts "usage: tacgui \[-e <lcuEnv>\] - Start TAC engineering panel"
                puts "option: -e Specifies the name of the LCU environment"
                puts "           running TAC, i.e. lte45m"
                puts "The name of the LCU environment may be omitted provided that"
                puts "TACENV shell variable has been defined to a valid LCU environment"
                puts "name."

                exit
	    }
	}
    } else {
        if { $lcuEnv == "lted" } {
            puts "usage: tacgui \[-e <lcuEnv>\] - Start TAC engineering panel"
            puts "option: -e Specifies the name of the LCU environment"
            puts "           running TAC, i.e. lte45m"
            puts "The name of the LCU environment may be omitted provided that"
            puts "TACENV shell variable has been defined to a valid LCU environment"
            puts "name."

            exit
	}   
    }
}

proc tacUpdateBlockList {} {
    global gvar lcuEnv cv

    set blockNumber_attr "@$lcuEnv<alias>tac:data:rtc:block.number"
    set rc [catch { seq_dbReadSymbolic $blockNumber_attr } number]

    if { $rc == 0 } {
    } else {
        return
    }

    #  Collect the names
    set nameList ""
    set valueList ""
    set default 0

    for {set i 0} {$i < $number} {incr i} {
        panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,1)" $i] name

        if { $name != "" } {
            set nameList [lappend nameList $name]
            set valueList [lappend valueList $i]
        }
    }

    #  Put the list onto the button
    $cv.tacguiWdgBlockList configure -itemList $nameList -itemValue $valueList -default $default

    set gvar(updBlock) 0
}

proc tacFillBlockWidget { blockIndex } {
    global gvar lcuEnv cv

    set blockNumber_attr "@$lcuEnv<alias>tac:data:rtc:block.number"
    set rc [catch { seq_dbReadSymbolic $blockNumber_attr } number]

    if { $rc == 0 } {
    } else {
    }

    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,1)" $blockIndex] gvar(blockName)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,0)" $blockIndex] gvar(blockType)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,2)" $blockIndex] gvar(param0)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,3)" $blockIndex] gvar(param1)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,4)" $blockIndex] gvar(param2)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,5)" $blockIndex] gvar(param3)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,6)" $blockIndex] gvar(param4)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,7)" $blockIndex] gvar(param5)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,8)" $blockIndex] gvar(param6)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,9)" $blockIndex] gvar(param7)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,10)" $blockIndex] gvar(param8)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:block.list(%d,11)" $blockIndex] gvar(param9)
}

proc tacUpdateLinkList {} {
    global gvar lcuEnv cv

    set linkNumber_attr "@$lcuEnv<alias>tac:data:rtc:link.number"
    set rc [catch { seq_dbReadSymbolic $linkNumber_attr } number]

    if { $rc == 0 } {
    } else {
        return
    }

    #  Collect the names
    set nameList ""
    set valueList ""
    set default 0

    for {set i 0} {$i < $number} {incr i} {
        panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,0)" $i] name

        if { $name != "" } {
            set nameList [lappend nameList $name]
            set valueList [lappend valueList $i]
        }
    }

    #  Put the list onto the button
    $cv.tacguiWdgLinkList configure -itemList $nameList -itemValue $valueList -default $default

    set gvar(updLink) 0
}

proc tacFillLinkWidget { linkIndex } {
    global gvar lcuEnv cv

    set linkNumber_attr "@$lcuEnv<alias>tac:data:rtc:link.number"
    set rc [catch { seq_dbReadSymbolic $linkNumber_attr } number]

    if { $rc == 0 } {
    } else {
    }

    #  Compare the names
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,0)" $linkIndex] gvar(netName)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,1)" $linkIndex] gvar(srcName)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,2)" $linkIndex] gvar(srcPort)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,3)" $linkIndex] gvar(dstName)
    panDbRead [format "@$lcuEnv<alias>tac:data:rtc:link.list(%d,4)" $linkIndex] gvar(dstPort)
}


#
# ___oOo___
