# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplACE.tcl,v 1.64 2005/10/07 22:29:46 vltsccm Exp $"
#
# tplACE.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  24/10/00  created
#
#************************************************************************
#   NAME
#   	tplACE - ACE detector class interface for templates.
# 
#   SYNOPSIS
#	tplACE dcs <dbAlias> <subsystem> [<detIndex>] [<imageScale>]
# 
#   DESCRIPTION
#	ACE detector interface for BOB templates.
#
#   PARENT
#	tplDCS
#
#   PUBLIC METHODS
#	see tplDCS man page.
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#	tplDCS tplFIERA tplMIDAS
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# Technical CCD procedures
#
class tplACE {
    inherit tplDCS

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { dbAlias subsys {index ""} {scale -1} } {
	tplDCS::constructor $dbAlias $subsys $index $scale
    } {
	#
	# 1. init tables
	#
    	set modes {gain_1_fast gain_1_slow \
		   gain_2_fast gain_2_slow \
		   gain_4_fast gain_4_slow \
		   gain_8_fast gain_8_slow}
	set clkinds { 1 2 1 2 1 2 1 2 }
	set gains   { 1 1 2 2 3 3 4 4 }
    }


    #--------------------------------------------------------------------
    # method SetupMode
    #
    # return the ACE mode according to the supplied FITS keywords.
    #
    method SetupMode { { mode ""} } {
	upvar $det $det

	#
	# 1. get ACE mode and return it
	#
	set value [Mode $mode]
	return "$det.READ.CLKIND [lindex $value 0] \
		$det.OUT1.GAININD [lindex $value 1]"
    }
    

    #--------------------------------------------------------------------
    # method Mode
    #
    # return the FIERA mode according to the supplied FITS keywords.
    #
    method Mode { { mode ""} } {
	upvar $det $det

	#
	# 1. default: get DET(READ.SPEED)
	#
        if { $mode == "" } {
	    set mode [set ${det}(READ.SPEED)]
	}

	#
	# 2. search ACE mode and gain
	#
	set index [lsearch $modes $mode]
	if { $index == -1 } {
	    error "$det.FRAM.SPEED $mode not supported."
	}
	set aceMode [lindex $clkinds $index]
	set aceGain [lindex $gains $index]

	return "$aceMode $aceGain"
    }
    

    #--------------------------------------------------------------------
    # method IsLoopSupported
    #
    method IsLoopSupported {} {
        set supported 1
        if {[IsWsOnlyUsed]} { set supported 0 }
        return $supported
    }

    #--------------------------------------------------------------------
    # method IsFileNameFixed
    #
    method IsFileNameFixed {} {
        set fixed 0
        if {[IsWsOnlyUsed]} { set fixed 1 }
        return $fixed
    }

    #--------------------------------------------------------------------
    # variables
    #

    protected variable gains {}
}


#########################################################################
# ___oOo___

