# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplIRACE.tcl,v 1.64 2005/10/07 22:29:46 vltsccm Exp $"
#
# tplIRACE.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  24/10/00  created
#
#************************************************************************
#   NAME
#   	tplIRACE - IRACE detector class interface for templates.
# 
#   SYNOPSIS
#	tplIRACE dcs <dbAlias> <subsystem> [<detIndex>] [<imageScale>]
# 
#   DESCRIPTION
#	IRACE detector interface for BOB templates.
#
#   PARENT
#	tplDCS
#
#   PUBLIC METHODS
#	see tplDCS man page.
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#	tplACE tplDCS tplMIDAS
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# IRACE CCD procedures
#
class tplIRACE {
    inherit tplDCS

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { dbAlias subsys {index ""} {scale -2} } {
	tplDCS::constructor $dbAlias $subsys $index $scale
    } {
        set dbExpStatus "<alias>$detName:exposure.expStatus"
    }


    #--------------------------------------------------------------------
    # method SetupMode
    #
    # return the IRACE mode according to the supplied FITS keywords.
    # Dummy for the time being
    #
    method SetupMode { { mode ""} } {

        return ""
    }


    #--------------------------------------------------------------------
    # method ExpTime
    #
    # set the exposure time and return the corresponding FITS keywords.
    #
    method ExpTime { {expTimeSec -1} {ndit -1} } {
	upvar $det $det
        
        if {$expTimeSec >= 0} { set ${det}(DIT) $expTimeSec }
        if {$ndit >= 0} { set ${det}(NDIT) $ndit }
        set buffer "$det.DIT [set ${det}(DIT)]"
        if {[info exists ${det}(NDIT)]} { append buffer " $det.NDIT [set ${det}(NDIT)]" }
	return "$buffer"
    }
    

    #--------------------------------------------------------------------
    # method Mode
    #
    # return the detector mode according to the supplied FITS keywords.
    # Dummy for the time being
    #
    method Mode { { mode ""} } {

	return ""
    }
    

    #--------------------------------------------------------------------
    # method Filename
    #
    method Filename {} {
	return [seq_dbReadSymbolic "<alias>$detName:exposure.newDataFileName"]
    }


    #--------------------------------------------------------------------
    # method IsImageReal
    #
    method IsImageReal {} {
        set fromchip 1
	set simulation [seq_dbReadSymbolic "<alias>$detName:irace.opMode"]
        if {$simulation == 2} { set fromchip 0 }
	return $fromchip
    }

    #--------------------------------------------------------------------
    # method IsWsOnlyUsed
    #
    method IsWsOnlyUsed {} {
        set wsonly 0
	set simulation [seq_dbReadSymbolic "<alias>$detName:irace.opMode"]
        if {$simulation == 2} { set wsonly 1 }
	return $wsonly
    }

}

#########################################################################
# ___oOo___

