#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplLogFits.tcl,v 1.64 2005/10/07 22:29:49 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-02-05  created
#

#************************************************************************
#   NAME
#	tplLogFitsDictionary, tplLogFitsLogical, tplLogFitsInt,
#	tplLogFitsReal, tplLogFitsString, tplLogFitsBegin,
#	tplLogFitsEnd - log FITS values
# 
#   SYNOPSIS
#	tplLogFitsDictionary <dict>
#	tplLogFitsLogical <key> <value> [<dict>]
#	tplLogFitsInt <key> <value> [<dict>]
#	tplLogFitsReal <key> <value> [<dict>]
#	tplLogFitsString <key> <value> [<dict>]
#	tplLogFitsBegin <id>
#	tplLogFitsEnd <id>
#
#   DESCRIPTION
#     Functions to log FITS values in the logMonitor.
#
#     tplLogFitsDictionary:
#	Set the default dictionary, where the to be logged 
#	FITS keywords are described.
#
#     tplLogFitsLogical:
#     tplLogFitsInt:
#     tplLogFitsReal:
#     tplLogFitsString:
#	Log the specified <key> with value <value>.
#
#     tplLogFitsBegin:
#     tplLogFitsEnd:
#	If a template logs are result consisting of several
#	FITS keyword values, then it is recommended to place
#	these keywords inside a BEGIN - END block.
#	The <id> is a keyword identifying the block.
#
#   RETURN VALUE
#	-
#
#   EXAMPLES
#
#   SEE ALSO
#	tplFITS
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#********************************************************************
# proc tplLogFits* - log FITS values
#
proc tplLogFitsDictionary { dict } {
    set ::BobTPL::tplLogFitsDict $dict
}

proc tplLogFitsLogical { key val {dict ""} } {
    if { $dict == "" } {
	set dict $::BobTPL::tplLogFitsDict
    }
    if { $val } {
	set txt "T"
    } else {
	set txt "F"
    }
    exec logAddFits P NO_CHECK DUMMY $key $txt
}

proc tplLogFitsInt { key val {dict ""} } {
    if { $dict == "" } {
	set dict $::BobTPL::tplLogFitsDict
    }
    set keys [split $key "."]
    set key1 [lindex $keys 0]
    set key2 [lrange $keys 1 end-1]
    set key3 [lindex $keys end]
    seq_logIntParRecord $dict $key1 $key2 $key3 $val
}

proc tplLogFitsReal { key val {dict ""} } {
    if { $dict == "" } {
	set dict $::BobTPL::tplLogFitsDict
    }
    set keys [split $key "."]
    set key1 [lindex $keys 0]
    set key2 [lrange $keys 1 end-1]
    set key3 [lindex $keys end]
    seq_logRealParRecord $dict $key1 $key2 $key3 $val
}

proc tplLogFitsString { key val {dict ""} } {
    if { $dict == "" } {
	set dict $::BobTPL::tplLogFitsDict
    }
    set keys [split $key "."]
    set key1 [lindex $keys 0]
    set key2 [lrange $keys 1 end-1]
    set key3 [lindex $keys end]
    seq_logStringParRecord $dict $key1 $key2 $key3 $val
}

proc tplLogFitsBegin { txt } {
    seq_logFitsComment "BEGIN $txt"
}

proc tplLogFitsEnd { txt } {
    seq_logFitsComment "END $txt"
}


# ___oOo___
