#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplBOOKING.tcl,v 1.15 2005/09/12 02:25:21 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-09-11  converted to class tplBOOKING.
# rschmutz  2005-05-08  created
#

#************************************************************************
#
#   NAME
#       tplBOOKING - OS booking class for templates
#
#   SYNOPSIS
#	tplBOOKING bk
#
#   DESCRIPTION
#       Booking/resource handling methods for templates.
#
#       This is only useful for instruments that allow the
#       execution of templates in parallel (e.g. FLAMES), where
#       we must make sure, that templates do not interfere with
#       each-other, i.e. use the same instrument resources.
#       For the time being this is not implemented, and the
#       operators are responsible of not making something wrong.
#
#   PUBLIC METHODS
#     Begin [<resources>]
#       Begin a new observation.
#       Unless specific resources are specified, all instrument
#       resources are assigned to the observation.  This method
#       should be called at the beginning of a template.
#     End
#       End the observation.
#       Unlock all assigned resources.
#       Note that this method is automatically called at the
#       end of the template execution (see CAUTION below).
#
#     Lock <resources> ...
#       Lock resources.
#     Unlock <resources> ...
#       Unlock resources.
#
#   PROTECTED METHODS
#
#   CAUTION
#       Booking/Resource handling is not implemented.
#
#       It is assumed, that only a single instance of this
#       class is instantiated within a template.  When this
#       class is instantiated, it registers a callback (with
#       BOBs setCallBack function) to automatically call its
#       End method at the end of the template execution.
#
#   SEE ALSO
#	tplOS
#
#   BUGS
#
#------------------------------------------------------------------------
#


#--------------------------------------------------------------------
# class tplBOOKING
#
class tplBOOKING {

    #----------------------------------------------------------------
    # constructor
    #
    constructor {} {
	global tplOsInstance

	#= 1. register in end callback
	set regInTplOsInst 0
	if { $tplOsInstance == "" } {
	    set tplOsInstance $this
	    set regInTplOsInst 1
	    setCallBack tplOsEnd
	}
    }


    #----------------------------------------------------------------
    # destructor
    #
    destructor {
	global tplOsInstance

	#= 1. reset observing flag
	set observing 0

	#= 2. unregister from end callback
	if { $regInTplOsInst } {
	    set tplOsInstance ""
	}
    }


    #----------------------------------------------------------------
    # method Begin - begin an observation
    #
    method Begin { {resources ""} } {

	#= 1. check that no other observation is running.
	if { $observing } {
	    error "Another observation is already running."
	}
	set observing 1
    }


    #----------------------------------------------------------------
    # method End - end an observation
    #
    method End {} {
	#= 1. reset observing flag
	set observing 0
    }


    #------------------------------------------------------------
    # method Lock - lock resources
    #
    method Lock { args } {
	# not implemented
    }


    #------------------------------------------------------------
    # method Unlock - unlock resources
    #
    method Unlock { args } {
	# not implemented
    }


    #------------------------------------------------------------
    # variables
    #
    protected variable regInTplOsInst 0
    protected variable observing 0
}


#___oOo___
