#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplOS_Obs.tcl,v 1.15 2005/09/12 02:25:20 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-05-08  created
#

#************************************************************************
#
#   NAME
#       tplOS - OS interface class for templates
#
#   DESCRIPTION
#       Observation related methods for tplOS.
#       See man page in file tplOS.tcl.
#
#------------------------------------------------------------------------
#


#--------------------------------------------------------------------
# method InsMode - return current INS.MODE
#
body tplOS::InsMode {} {
    return [$this.obs InsMode]
}


#--------------------------------------------------------------------
# method ImgName - return the image filename
#
body tplOS::ImgName { {dprType ""} } {
    upvar DPR DPR
    upvar TPL TPL
    return [$this.obs ImgName $TPL(ID) $dprType]
}


#--------------------------------------------------------------------
# method Setup - setup an exposure
#
# Let's try that the caller does not need to care any
# longer about 'first SETUP needs a INS.MODE entry',
# -noExposure, -expoId, etc. ...
#
body tplOS::Setup { buffer } {

    #= 1. check that an observation is running
    if { $observing == 0 } {
	error "No observation is running."
    }

    #= 2. add -noExposure ?
    #
    #     If no exposure is active (expoId = 0) and
    #     no INS.MODE value is specified, then add
    #     option -noExposure to the buffer, to have
    #     OS accept the SETUP command.
    #
    if { [$this.obs ExpoId] == 0 && \
	    [lcontain $buffer "INS.MODE"] == 0 && \
	    [lcontain $buffer "-noExposure"] == 0 } {
	set buffer "-noExposure $buffer"
    }

    #= 3. execute Setup
    #
    #     If this is a -noExposure SETUP, then
    #     send it directly to OS, otherwise go
    #     through the tplOBS object.
    #
    if { [lcontain $buffer "-noExposure"] } {
	set timeout [$this.obs SetupTimeout]
	set ret [sendCmd $timeout SETUP $buffer]
    } else {
	set ret [$this.obs Setup $buffer]
    }

    return $ret
}


#--------------------------------------------------------------------
# method AddFits - add header FITS keywords
#
body tplOS::AddFits { buffer {detId ""} } {

    #= 1. check that an observation is running
    if { $observing == 0 } {
	error "No observation is running."
    }

    #= 2. execute AddFits
    return [$this.obs AddFits $buffer $detId]
}


#--------------------------------------------------------------------
# method Start - start an exposure
#
body tplOS::Start { {detId ""} } {

    #= 1. check that an observation is running
    if { $observing == 0 } {
	error "No observation is running."
    }

    #= 2. execute Start
    checkAbortFlag
    return [$this.obs Start $detId]
}


#--------------------------------------------------------------------
# method Wait - wait for the end of exposure
#
body tplOS::Wait { args } {

    #= 1. check that an observation is running
    if { $observing == 0 } {
	error "No observation is running."
    }

    #= 2. execute Wait
    return [eval $this.obs Wait $args]
}


#___oOo___
