#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplOS_Tcs.tcl,v 1.15 2005/09/12 02:25:20 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-05-08  created
#

#************************************************************************
#
#   NAME
#       tplOS - OS interface class for templates
#
#   DESCRIPTION
#       TCS related methods for tplOS.
#       See man page in file tplOS.tcl.
#
#------------------------------------------------------------------------
#


#--------------------------------------------------------------------
# method Offset
#
# coords = SKY:      offset1=alpha, offset2=delta
# coords = DETECTOR: offset1=x,     offset2=y
#
body tplOS::Offset { coords offset1 offset2 } {
    error "Offset not implemented."
}


#--------------------------------------------------------------------
# method OffsetReturn - offset, execute code and return offset
#
# coords = SKY:      offset1=alpha, offset2=delta
# coords = DETECTOR: offset1=x,     offset2=y
#
body tplOS::OffsetReturn { coords offset1 offset2 code } {
    tplSeeVars

    #= 1. init
    set retOffset1 0.0
    set retOffset2 0.0
    set haveMacao [HaveChild macao]
    set isLoopClosed 0
    if { $haveMacao } {
	set isLoopClosed [$this.macao IsLoopClosed]
    }
    checkAbortFlag

    tplTry {
	#= 2. offset
	Offset $coords $offset1 $offset2
	tplIncr retOffset1 $offset1
	tplIncr retOffset2 $offset2

	#= 3. execute code
	uplevel 1 [list eval $code]
    } {
	set ret 1
	catch { set ret $SEQ(RETURN) }
	if { $ret } {

	    #= 4. if AO loop was initially closed: close it again
	    if { $isLoopClosed && [$this.macao IsLoopClosed] == 0 } {
		if { [$this.macao IsLoopSuspended] } {
		    $this.macao Resume
		} else {
		    error "AO loop was initially closed: now it is open (not suspended)."
		    #$this.macao CloseLoop
		}
	    }

	    #= 5. return offset
	    Offset $coords [expr -$retOffset1] [expr -$retOffset2]
	}
    }
}


#___oOo___
