#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooFMT_HTML.tcl,v 1.15 2005/09/12 02:25:18 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mkiekebu  2005-05-02  created
#

#************************************************************************
#   NAME
#       tpltooFMT_HTML - tpltoo output format for TSF report.
# 
#   SYNOPSIS
#       tpltooFMT_HTML <obj>
# 
#   DESCRIPTION
#       This class provides the functionality to obtain reports of TSF
#       files in a fixed HTML format. 
#
#   PARENT CLASS
#	tpltooFMT
#
#   PUBLIC METHODS
#	addHeader <instrument> 
#              Add report header information in a fixed HTML format.
#
#       addFooter <msg>
#              Add report footer information in a fixed HTML format.
#    
#       createTable <title>
#              Create a table in a fixed html format with the TSF 
#              information divided in keywords to be specified and 
#              keywords fixed.
#       
#       addTableRowVar <key hide range initval shortd>
#              Increment the array of table rows for keywords to be 
#              specified in fixed html format.
#
#       addTableRowFix <key hide value shortd>
#              Increment the array of table rows for fixed keywords 
#              in fixed html format. 
#
#   FILES
#	*.tsf:	template signature files
#
#   SEE ALSO
#	
#
#   BUGS     
#
#------------------------------------------------------------------------
#
class tpltooFMT_HTML {
    inherit tpltooFMT

    constructor {args} {
	eval configure $args
	set format "html"
	$this setFile 
    }
    
     public method addHeader {instrument version} {		
         puts $fd "<html>"
         puts $fd "<head>"
         puts $fd "<title>$instrument $version</title>"
	 puts $fd "<meta http-equiv=\"content-type\" "
	 puts $fd "content=\"text/html; charset=ISO-8859-1\">"
         puts $fd "</head>"
         puts $fd "<body>"
    }

    public method addFooter { msg } {
	puts $fd "</body></html>"
    }

     public method createTable {title} {
	 $this addTableVar $title
	 foreach entry $varKeyList {
	     puts $fd $entry
	 }
	 $this addTableFix
	 foreach entry $fixKeyList {
	     puts $fd $entry
	 }
	 $this endTable
	 $this emptyTblList
     }

     public method addTableRowVar {key hide range initval shortd} {	

	 lappend varKeyList "<tr><td valign=\"left\" width=\"25%\"><small><b>$key</b><br></td><td valign=\"left\" width=\"5%\">$hide<br></td><td valign=\"left\" width=\"25%\">${range} <i>(${initval})</i><br></td><td valign=\"left\" width=\"45%\">$shortd<br></small></td></tr>"	

    }

     public method addTableRowFix {key hide value shortd} {	
	lappend fixKeyList "<tr><td valign=\"left\" width=\"25%\"><small><b>$key</b><br></td><td valign=\"left\" width=\"5%\">$hide<br></td><td valign=\"left\" width=\"25%\">$value<br></td><td valign=\"left\" width=\"45%\">$shortd<br></small></td></tr>" 
    }

    private method addTableVar {title} {
	puts $fd "<table cellpadding=\"0\" cellspacing=\"0\" border=\"1\" width=\"100%\">"
	puts $fd "<tbody>"
	puts $fd "<tr align=\"center\">"
	puts $fd "<td valign=\"top\" colspan=\"4\"><b>$title</b><br>"
	puts $fd "</td>"
	puts $fd "</tr>"
	puts $fd "<tr>"
	puts $fd "<td valign=\"top\" colspan=\"4\"><i>To be specified:</i><br>"
	puts $fd "</td>"
	puts $fd "</tr>"
	puts $fd "<tr>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"25%\">Parameter<br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"5%\">Hide<br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"25%\">Range <i>(Default)</i><br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" width=\"45%\" bgcolor=\"#99ff99\">Label<br>"
	puts $fd "</td>"
	puts $fd "</tr>"
    }     

    private method addTableFix { } {
	puts $fd "<tr>"
	puts $fd "<td valign=\"top\" colspan=\"4\"><i>Fixed Values:</i><br>"
	puts $fd "</td>"
	puts $fd "</tr>"
	puts $fd "<tr>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"25%\">Parameter<br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"5%\">Hide<br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"25%\">Value<br>"
	puts $fd "</td>"
	puts $fd "<td valign=\"middle\" bgcolor=\"#99ff99\" width=\"45%\">Label<br>"
	puts $fd "</td>"
	puts $fd "</tr>"
    }
    
    private method endTable {} {	
	puts $fd "</tbody>"
	puts $fd "</table>"
	puts $fd "<br>"
	puts $fd "<br>"
	puts $fd "<br>"
	puts $fd "<hr width=\"100%\" size=\"1\"><br>"
    }

}


#
# ___oOo___














