#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooFMT_LATEX.tcl,v 1.15 2005/09/12 02:25:18 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mkiekebu  2005-05-02  created
#

#************************************************************************
#   NAME
#       tpltooFMT_LATEX - latex output format for TSF report.
# 
#   SYNOPSIS
#       tpltooFMT_LATEX <obj>
# 
#   DESCRIPTION
#       This class provides the functionality to obtain reports of TSF
#       files in a latex fixed format. The latex format has been borrowed
#       from tsf2tab tool by A.Kaufer.
#
#   PARENT CLASS
#	tpltooFMT
#
#   PUBLIC METHODS
#	addHeader <instrument> 
#              Add report header information in a latex fixed format.
#
#       addFooter <msg>
#              Add report footer information in a latex fixed format.
#    
#       createTable <title>
#              Create a table in a fixed latex format with the TSF 
#              information divided in keywords to be specified and 
#              keywords fixed.
#       
#       addTableRowVar <key hide range initval shortd>
#              Increment the array of table rows for keywords to be 
#              specified in latex fixed format.
#
#       addTableRowFix <key hide value shortd>
#              Increment the array of table rows for fixed keywords 
#              in latex fixed format. 
#              
#   FILES
#	*.tsf:	template signature files
#
#   SEE ALSO
#	tpltooFMT
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class tpltooFMT_LATEX {
    inherit tpltooFMT
    
     constructor { args } {	   
	eval configure $args
	set format "latex"
	$this setFile 
    }

     public method addHeader { instrument version} {	
	 puts $fd "\\documentclass\[10pt\]\{article\}"
	 puts $fd "\\usepackage\{times,psfig,latexsym\}"
	 puts $fd "\\topmargin-3cm"
	 puts $fd "\\oddsidemargin.2cm"
	 puts $fd "\\textwidth16cm"
	 puts $fd "\\textheight24cm"
	 puts $fd "\\parindent0cm"
	 puts $fd "\n"

	 puts $fd "%\\renewcommand\{\\tabcolsep\}\{2mm\}"
	 puts $fd "%\\renewcommand\{\\arraystretch\}\{1.3\}"
	 puts $fd "\n\n"

	 puts $fd "\\begin\{document\}"
	 puts $fd "\\title\{\\huge\\bf $instrument template status report\}"
	 puts $fd "\\author\{\\Large [exec whoami]\}"
	 puts $fd "\\maketitle"
	 puts $fd "\n"

	 puts $fd "\\vspace*\{4cm\}"
	 puts $fd "\n"
	 puts $fd "\\begin\{center\}"
	 puts $fd "\\fbox\{\\parbox\{8cm\}\{\\large"
	 puts $fd "Warning: \\\\

             The contents of this document have been extracted and formatted automatically 
             from the $instrument tsf-files.\n\n
             \\begin\{center\}IP: ${instrument}-$version \\end\{center\}"
      	     	     
	 puts $fd "\}\}"
         puts $fd "\\end\{center\}"
	 	 	 	 	 
     }
    
  
    public method addFooter { msg } {	 
         puts $fd "\\end\{document\}"
    }

    public method createTable {title} {
	$this addTableVar $title
	foreach entry $varKeyList {
	    puts $fd $entry
	}
	$this addTableFix
	foreach entry $fixKeyList {
	    puts $fd $entry
	}
	$this endTable
	$this emptyTblList
    }

     public method addTableRowVar {key hide range initval shortd} {
	#- Add scape character to underscore -#
	regsub -all _ $key {\_} key
	regsub -all _ $range {\_} range
	regsub -all _ $initval {\_} initval
	regsub -all _ $shortd {\_} shortd

	lappend varKeyList "\\parbox\[t\]\{5cm\}\{\\bf $key\} & \\parbox\[t]\{0.5cm\}\{$hide\} & \\parbox\[t\]\{5cm\}\{ $range \{\\it ($initval\})\} & \\parbox\[t\]\{5cm\}\{ $shortd\}\\\\"

    }

     public method addTableRowFix {key hide value shortd} {
	 regsub -all _ $key {\_} key
	 regsub -all _ $value {\_} value	 
	 regsub -all _ $shortd {\_} shortd

	 lappend fixKeyList "\\parbox\[t\]\{5cm\}\{\\bf $key\} & \\parbox\[t]\{0.5cm\}\{$hide\} & \\parbox\[t\]\{5cm\}\{ $value\} & \\parbox\[t\]\{5cm\}\{ $shortd\}\\\\"	
    }

    private method addTableVar {title} {
	#- Add scape character to underscore -#
	regsub -all _ $title {\_} title

	puts $fd "\\newpage"
	puts $fd "\\begin{tabular}{|l|l|l|l|}\\hline"
        puts $fd "\\multicolumn\{4\}\{|c|\}\{\\bf $title\} \\\\ \\hline\\hline"
        puts $fd "\\multicolumn\{4\}\{|l|\}\{\\it To be specified:\} \\\\ \\hline"
        puts $fd "Parameter  & Hide & Range \{\\it (Default)\} & Label \\\\ \\hline"
    }     

    private method addTableFix { } {
	puts $fd "\\hline\\multicolumn{4}{|l|}{\\it Fixed values:} \\\\ \\hline"
	puts $fd "Parameter  & Hide & Value & Label\\\\ \\hline"
    }

    
    private method endTable {} {	
	puts $fd "\\hline"
	puts $fd "\\hline"
	puts $fd "\\end\{tabular\}"
    }

    
      
}

#
# ___oOo___








