#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooFMT_TXT.tcl,v 1.15 2005/09/12 02:25:18 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mkiekebu  2005-05-02  created
#

#************************************************************************
#   NAME
#       tpltooFMT_TXT - tpltoo output format for TSF report.
# 
#   SYNOPSIS
#       tpltooFMT <obj>
# 
#   DESCRIPTION
#       This class provides the functionality to obtain reports of TSF
#       files in simple text format with columns separated by commas to 
#       provide a easy export to a more sophisticated formats. 
#
#   PARENT CLASS
#	tpltooFMT
#
#   PUBLIC METHODS
#	addHeader <instrument> 
#              Add report header information in a simple text format.
#
#       addFooter <msg>
#              Add report footer information in a simple text format.
#    
#       createTable <title>
#              Create a table in a fixed text format with the TSF 
#              information divided in keywords to be specified and 
#              keywords fixed.
#       
#       addTableRowVar <key hide range initval shortd>
#              Increment the array of table rows for keywords to be 
#              specified in text fixed format.
#
#       addTableRowFix <key hide value shortd>
#              Increment the array of table rows for fixed keywords 
#              in text fixed format. 
#
#   FILES
#	*.tsf:	template signature files
#
#   SEE ALSO
#	tpltooFMT
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class tpltooFMT_TXT {
    inherit tpltooFMT

    constructor {args} {
	eval configure $args
	set format "txt"
	$this setFile
    }
    

   public method addHeader { instrument version} {
       puts $fd "$instrument template status report"
       puts $fd ""
       puts $fd "IP: ${instrument}-$version"
       puts $fd ""
   }
  
    public method addFooter { msg } {
	puts $fd "END of report"
    }

    public method createTable {title} {
	$this addTableVar $title
	foreach entry $varKeyList {
	    puts $fd $entry
	}
	$this addTableFix
	foreach entry $fixKeyList {
	    puts $fd $entry
	}
	$this endTable
	$this emptyTblList
    }

     public method addTableRowVar {key hide range initval shortd} {
	lappend varKeyList "$key, $hide, $range"
    }

     public method addTableRowFix {key hide value shortd} {
	 lappend fixKeyList "$key,$hide,$value"	
    }

    private method addTableVar {title} {		
	puts $fd "$title"
	puts $fd "To be specified:"
        puts $fd "Parameter,Hide,Range"
    }     

    private method addTableFix { } {
	puts $fd "Fixed values:"
	puts $fd "Parameter,Hide,Value"
    }

    private method endTable {} {		
	puts $fd "\n\n"
    }
}



#
# ___oOo___


















