#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooLOG.tcl,v 1.15 2005/09/12 02:25:16 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  07/06/01  created.
#

#************************************************************************
#   NAME
#	tpltooLOG - log utilities
# 
#   SYNOPSIS
#	tpltooLOG name [-infoHeader <text>]
#		       [-infoWidget <widget>]
#		       [-loglevel <level>]
# 
#   DESCRIPTION
#	tpltooLOG provides log utilities for the tpltoo scripts.
#
#   PARENT CLASS
#	-
#
#   PUBLIC PROCEDURES
#	SetWidget <widget>
#	    Sets the widget where the logs should appear.
#
#   PUBLIC DATA
#	infoHeader: header printed by method Info.
#	infoWidget: widget used by method Info.
#	loglevel:   loglevel used by method Debug.
#
#   PROTECTED METHODS
#	Exit [<text>]
#	    Prints the text (if supplied) and terminates the script.
#
#	Debug <text> [<level>]
#	    Prints text using the Info method if the loglevel
#	    is greater or equal level. The default level is 1.
#
#	Info <text>
#	    Prints the text to stdout.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: tpltooLOG
#------------------------------------------------------------------------

class tpltooLOG {

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: SetWidget
    #----------------------------------------------------------------

    public proc SetWidget { widget } {
    	set infoWidget $widget
    }

    #****************************************************************
    #*# method: Exit
    #----------------------------------------------------------------

    protected method Exit { {msg ""} } {
	global cv

	#*# 1. remove duplicate "ERROR:"
	regsub "ERROR: ERROR: " $msg "ERROR: " msg

    	#*# 2. print message
	if { "$msg" != "" } {
	    Info $msg
	}

	#*# 3. exit or issue an error (if we are in a panel)
	if { ![info exists cv] } {
	    exit
	} else {
	    error $msg
	}
    }

    #****************************************************************
    #*# method: Debug
    #----------------------------------------------------------------

    protected method Debug { msg {level 1} } {

    	#*# 1. if loglevel: print message
    	if { $loglevel >= $level } {
	    Info "       $msg"
	}

    }

    #****************************************************************
    #*# method: Info
    #----------------------------------------------------------------

    protected method Info { msg {newline 1} } {

    	#*# 1. print message
	if { "$infoWidget" == "" } {
	    if { $newline } {
	        puts "$infoHeader$msg"
	    } else {
	        puts -nonewline "$infoHeader$msg"
		flush stdout
	    }
	} else {
	    if { $newline } {
	        $infoWidget addItem end "$infoHeader$msg"
	    } else {
	    	set txt [$infoWidget getItem end]
		append txt $msg
		$infoWidget deleteItem end
	        $infoWidget addItem end $txt
	    }
	    $infoWidget setCursor end
	    update
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common loglevel   0
    public common infoWidget ""
    public common infoHeader ""

    protected common logSep1 [replicate "-" 80]
    protected common logSep2 [replicate "=" 80]
}




#
# ___oOo___
