#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooOBS.tcl,v 1.15 2005/09/12 02:25:17 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  08/06/01  created (from calobOBS.tcl).
#

#************************************************************************
#   NAME
#	tpltooOBS - OB observation interface
# 
#   SYNOPSIS
#	tpltooOBS obs
# 
#   DESCRIPTION
#	Itcl interface to an OB observation.
#
#   PARENT CLASS
#	tpltooPAF
#
#   PUBLIC METHODS
#	See tpltooPAF.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#
#   CAUTION
#
#   EXAMPLES
#
#   SEE ALSO
#	tpltooPAF(8)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: tpltooOBS
#------------------------------------------------------------------------

class tpltooOBS {
    inherit tpltooPAF

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        tpltooPAF::constructor -objType OBS
	eval configure $args
    }

    #****************************************************************
    #*# method: Load
    #----------------------------------------------------------------

    public method Load { {fileName ""} } {
    	error "Not applicable."
    }

    #****************************************************************
    #*# method: Puts
    #----------------------------------------------------------------

    public method Puts { {exclist "PAF CALIB"} } {
    	tpltooPAF::Puts $exclist
    }

    #****************************************************************
    #*# method: Merge
    #
    # Merge two observations, appending values to the keylists.
    # It is assumed, that each observation is executed SEQ.NEXPO
    # times.
    #----------------------------------------------------------------

    public method Merge { obs keylist } {

    	#*# 1. check that other observation has same TPL.ID
	set tplId1 [Get1 TPL.ID]
	set tplId2 [$obs Get1 TPL.ID]
	if { $tplId1 != $tplId2 } {
	    error "tpltooOBS::Merge: Observations have different TPL.IDs: '$tplId1' and '$tplId2'."
	}

	#*# 2. set length of local keylists
	if { ![array exists val1] } {
	    foreach key $keylist {
	        set val1($key) ""
	    }
	    MergeObs $this $keylist
	}

	#*# 3. merge other observation
	MergeObs $obs $keylist

	#*# 4. set all local keylists
	foreach key $keylist {
	    set val [lrmdups $val1($key)]
	    if { [llength $val] == 1 } {
		Set $key $val
	    } else {
	        Set $key $val1($key)
	    }
	}
	Set SEQ.NEXPO [llength $val1([lindex $keylist 0])]

	#*# 5. align lists for easier viewing in BOB
	set len 1
	foreach key $keylist {
	    set valist [Get1 $key]
	    if { [llength $valist] != 1 } {
	    	foreach val $valist {
		    if { [clength $val] > $len } {
		    	set len [clength $val]
		    }
		}
	    }
	}
	foreach key $keylist {
	    set valist [Get1 $key]
	    if { [llength $valist] != 1 } {
	    	set valist2 ""
	    	foreach val $valist {
		    append valist2 "[format "%[set len]s" $val] "
		    #"
		}
		Set $key $valist2
	    }
	}
    }

    #****************************************************************
    #*# method: MergeObs
    #----------------------------------------------------------------

    protected method MergeObs { obs keylist } {

    	#*# 1. retrieve all list keywords
	foreach key $keylist {
	    set val2($key) [$obs Get $key]
	    set len2($key) [llength $val2($key)]
	    set index2($key) 0
	}

	#*# 2. 'execute' SEQ.NEXPO loops
	loop i 0 [$obs Get SEQ.NEXPO] {
	    foreach key $keylist {
	    	lappend val1($key) [lindex $val2($key) $index2($key)]
		incr index2($key)
		if { $index2($key) >= $len2($key) } {
		    set index2($key) 0
		}
	    }
	}

    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public    variable sortkey ""
    protected variable val1
}

#
# ___oOo___
