#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooTSF.tcl,v 1.15 2005/09/12 02:25:17 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  15/05/03  SPR VLTSW20030188: truncate PAF.DESC to 47 chars.
# rschmutz  28/05/02  <param>.HIDE keyword support added.
# rschmutz  08/06/01  created (from calobTPL.tcl).
#

#************************************************************************
#   NAME
#	tpltooTSF - Template signature file interface
# 
#   SYNOPSIS
#	tpltooTSF tsf
# 
#   DESCRIPTION
#	Itcl interface to a TSF.
#
#   PARENT
#	tpltooPAF
#
#   PUBLIC METHODS
#	see tpltooPAF.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#
#   CAUTION
#
#   EXAMPLES
#
#   SEE ALSO
#	tpltooPAF(8)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: tpltooTSF
#------------------------------------------------------------------------

class tpltooTSF {
    inherit tpltooPAF

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        tpltooPAF::constructor -objType TSF
	eval configure $args

	#*# 1. add PAF entries
	set now [clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S.00"]
	Set PAF.HDR.START	""
	Set PAF.TYPE            "Template Signature"
	Set PAF.ID              ""
	Set PAF.NAME            ""
	Set PAF.DESC            ""
	Set PAF.CRTE.NAME       "tpltooTplKeySort"
	Set PAF.CRTE.DAYTIM     "$now"
	Set PAF.LCHG.NAME       ""
	Set PAF.LCHG.DAYTIM     "$now"
	Set PAF.CHCK.NAME       ""
	Set PAF.CHCK.DAYTIM     ""
	Set PAF.CHCK.CHECKSUM   ""
	Set PAF.HDR.END         ""

	Set TPL.INSTRUM         ""
	Set TPL.MODE            ""
	Set TPL.VERSION         "1.0"
	Set TPL.REFSUP          ""
	Set TPL.PRESEQ          ""
	Set TPL.GUI             ""
	Set TPL.TYPE            "test"
	Set TPL.EXECTIME        "computed"
	Set TPL.RESOURCES       ""

	#*# 2. load file
	Load
    }

    #****************************************************************
    #*# method: Load
    #----------------------------------------------------------------

    public method Load { {fileName ""} } {
        global env

    	if { "$fileName" != "" } {
    	    configure -file $fileName
	}
	set fileName $file
	if { "$file" != "" } {
	    if { ![file exists $file] && [file tail $file] == $file } {
	        configure -file "$env(INS_ROOT)/SYSTEM/COMMON/TEMPLATES/TSF/$file"
	        if { ![file exists $file] } {
	            configure -file "$env(INTROOT)/config/INS_ROOT/SYSTEM/COMMON/TEMPLATES/TSF/$fileName"
	        }
	    }
	    if { ![file exists $file] } {
	        error "File $fileName not found."
	    }
	    tpltooPAF::Load
	}
    }

    #****************************************************************
    #*# method: Puts
    #----------------------------------------------------------------

    public method Puts {} {
        global env

    	#*# 1. set PAF header
	set user  $env(USER)
	set today [clock format [clock seconds] -format "%Y-%m-%d"]
	set now   [clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S.00"]
	set pafDesc [Get1 PAF.DESC]
	if { [clength $pafDesc] > 47 } {
	    set pafDesc [csubstr $pafDesc 0 47]
	    Info "  PAF.DESC is longer 47 chars.\n  Truncated to: '$pafDesc'."
	}
    	set txt \
"#*******************************************************************************
# E.S.O. - VLT project
#
# \"@(#) \$Id\$\"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# [format %-8s $user]  $today  created.
#

PAF.HDR.START;
PAF.TYPE                [format %-26s "\"[Get1 PAF.TYPE]\";"	   ] # Type of PAF
PAF.ID                  [format %-26s "\"[Get1 PAF.ID]\";"	   ] # ID of PAF
PAF.NAME                [format %-26s "\"[Get1 PAF.NAME]\";"	   ] # Name of PAF
PAF.DESC                [format %-26s "\"$pafDesc\";"		   ] # Short description of PAF
PAF.CRTE.NAME           [format %-26s "\"[Get1 PAF.CRTE.NAME]\";"  ] # Name of creator
PAF.CRTE.DAYTIM         [format %-26s "\"[Get1 PAF.CRTE.DAYTIM]\";"] # Civil time for creation
PAF.LCHG.NAME           [format %-26s "\"[set user]\";"		   ] # Name of person/appl. chg.
PAF.LCHG.DAYTIM         [format %-26s "\"[set now]\";"		   ] # Timestamp of last change
PAF.CHCK.NAME           [format %-26s "\"[Get1 PAF.CHCK.NAME]\";"  ] # Name of appl. checking
PAF.CHCK.DAYTIM         [format %-26s "\"[Get1 PAF.CHCK.DAYTIM]\";"] # Time for checking
PAF.CHCK.CHECKSUM       [format %-26s "\"[Get1 PAF.CHCK.CHECKSUM]\";"] # Checksum for the PAF
PAF.HDR.END;

# --------------------------------------------------------------------
TPL.INSTRUM             [format %-26s "\"[Get1 TPL.INSTRUM]\";"	  ] # Instrument this template is for
TPL.MODE                [format %-26s "\"[Get1 TPL.MODE]\";"	  ] # Mode of observation for this template
TPL.VERSION             [format %-26s "\"[Get1 TPL.VERSION]\";"	  ] # Version of the template
TPL.REFSUP              [format %-26s "\"[Get1 TPL.REFSUP]\";"	  ] # Reference Setup File
TPL.PRESEQ              [format %-26s "\"[Get1 TPL.PRESEQ]\";"	  ] # Sequencer script
TPL.GUI                 [format %-26s "\"[Get1 TPL.GUI]\";"	  ] # Template GUI panel
TPL.TYPE                [format %-26s "\"[Get1 TPL.TYPE]\";"	  ] # Type of template
TPL.EXECTIME            [format %-26s "\"[Get1 TPL.EXECTIME]\";"  ] # Expected execution time
TPL.RESOURCES           [format %-26s "\"[Get1 TPL.RESOURCES]\";" ] # Required resources
"

    	#*# 2. add all parameters
	set keywList [lmatch [GetKeywords] {*.TYPE}]
	set keywList [lindex [intersect3 $keywList {PAF.TYPE TPL.TYPE}] 0]
	regsub -all {.TYPE } $keywList " " keywList
	regsub {.TYPE$} $keywList "" keywList
	set keywList [SortKeywords $keywList]
	foreach keyw $keywList {
	    append txt "\n# --------------------------------------------------------------------\n"
	    append txt "TPL.PARAM\t\t[format %-26s "\"$keyw\";"] # Next template parameter\n"
	    foreach param {TYPE RANGE DEFAULT VALUE LABEL MINIHELP HIDE} \
	    	    comment { "Keyword type" "Valid range" "Default value" \
		    	      "Allocated value" "Label used in P2PP" \
			      "Short Help for keyword" "Hide keyword in OHS" } {
	        catch { append txt "[format %-23s $keyw.$param] [format %-26s "\"[Get1 $keyw.$param]\";"] # $comment\n" }
	    }
	}

	#*# 3. trailer
	append txt "\n# --- oOo ---\n"

	return $txt
    }

}

#
# ___oOo___
