#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vcc.tcl,v 2.116 2005/10/21 14:15:07 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.10.1995 created
# fcarbogn   08.09.1998 use of netstat -i to find the net Interface (SPR980407)
# psivera    23.05.2001 added ACSROOT and ACE_ROOT_VW
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
#

#************************************************************************
#   NAME
#	vcc - common VCC initialization part
# 
#   SYNOPSIS
#	vcc
# 
#   DESCRIPTION
#	Sets global variables of the vcc module.
#
#   FILES
#
#   ENVIRONMENT
#	DEBUG - if defined, enables debugging outputs
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	VccCompletion(n), VccDebug(n), VccInfo(n), 
#	VccSelectButton(n), VccSelector(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc vcc {} {
    # dummy proc used by other library files to make sure that the globals
    # defined herein will be sourced before
}


#---------------------------------------------
# Global Variables
#---------------------------------------------
# NOTE: vccSetDef defines the variable only if not already defined before!

global vcc vcc_c vcc_n tcl_platform


# VccDebug:
vccSetDef vcc(debug) [info exists env(DEBUG)]


# Name definitions:
# -----------------
vccSetDef vcc_n(areaList)	{MODROOT INTROOT VLTDATA VLTROOT ACSROOT ACE_ROOT_VW}
						;# areas in order of priority
vccSetDef vcc_n(bitmapsDir)	bitmaps		;# name of bitmaps sub-dir.
vccSetDef vcc_n(bitmapsExt)	.xbm		;# extension for bitmaps files
vccSetDef vcc_n(pixmapsDir)	bitmaps		;# name of pixmaps sub-dir.
vccSetDef vcc_n(pixmapsExt)	.xpm		;# extension for pixmaps files
vccSetDef vcc_n(bootDir)	vw/bin		;# name of boot sub-dir.
vccSetDef vcc_n(bootExt)	.boot		;# extension for boot files
vccSetDef vcc_n(configDir)	config		;# name of config sub-directory
vccSetDef vcc_n(configExt)	.config		;# extension for config files
vccSetDef vcc_n(envDir)		ENVIRONMENTS	;# subdirname with environments
vccSetDef vcc_n(proFile)	PROCESSES	;# file-name with processes
vccSetDef vcc_n(tmpDir)		/tmp		;# dir for temporary files
#vccSetDef vcc_n(infoObj)	vccInfo		;# name for VccInfo instance
#	shouldn't be configurable!

# File definitions:
# -----------------
set vcc_n(uname-s) $tcl_platform(os)
set vcc_n(uname-r) $tcl_platform(osVersion)
set vcc_n(isSUN)   [string match *sun* [string tolower $vcc_n(uname-s)]]
set vcc_n(isHP)    [string match *hp*  [string tolower $vcc_n(uname-s)]]
set vcc_n(isLINUX) [string match *linux*  [string tolower $vcc_n(uname-s)]]

vccSetDef vcc_n(exports) [expr {$vcc_n(isSUN)?"/etc/dfs/dfstab":"/etc/exports"}]
if {[info exists env(RTAPROOT)]} {
    if {($vcc_n(isHP)  && $vcc_n(uname-r) >= "B.10") || \
	($vcc_n(isSUN) && $vcc_n(uname-r) >= "5.5") } {
	# for Rtap >= 6.7 on HP-UX >= 10 or SunOS >= 5.5:
	vccSetDef vcc_n(RtapEnvList)	/etc$env(RTAPROOT)/RtapEnvList
    } else {
	# for Rtap <= 6.6 on HP-UX <= 9 or SunOS <= 5.4:
	vccSetDef vcc_n(RtapEnvList)	$env(RTAPROOT)/etc/RtapEnvList
    }
}

# set correct remote shell and ifconfig commands for HP and SUN:
set vcc_n(netmask) ffffff00 ;# ESO default
if {$vcc_n(isSUN)} {
    set vcc_n(posixsh) /bin/sh
    set vcc_n(remsh) rsh
    set vcc_n(ping) /usr/sbin/ping
    if {[catch {exec /usr/5bin/netstat -i | grep $env(HOST) | awk "{print \$1}"} Interface ]} {
        set Interface le0
        }
    catch {set vcc_n(netmask) [exec \
	    /sbin/ifconfig $Interface | grep netmask | awk "{print \$4}" ] }
} elseif {$vcc_n(isLINUX)} {
    set vcc_n(posixsh) /bin/sh
    set vcc_n(remsh) rsh
    set vcc_n(ping) /bin/ping
    if {[catch {exec /bin/netstat -i | grep $env(HOST) | awk "{print \$1}"} Interface ]} {
        set Interface lan0
        }
    catch {set vcc_n(netmask) [exec \
            /sbin/ifconfig $Interface | grep netmask | awk "{print \$4}" ] }
} else {
    set vcc_n(posixsh) /bin/posix/sh
    set vcc_n(remsh) remsh
    set vcc_n(ping) /etc/ping
    if {[catch {exec /usr/bin/netstat -i | grep $env(HOST) | awk "{print \$1}"} Interface ]} {
        set Interface lan0
        }
    catch {set vcc_n(netmask) [exec \
	    /etc/ifconfig $Interface | grep netmask | awk "{print \$4}" ] }
}

if { $vcc_n(netmask) == "" } { set vcc_n(netmask) ffffff00 }

# VccSelectButton/VccSelector:
if {[info command option] != ""} {
    option add *VccSelector.menuMaxLen   25			widgetDefault
    option add *VccSelector.evtInvoke    <Return>		widgetDefault
    option add *VccSelector.evtComplete  {<Escape> <Tab>}	widgetDefault
    option add *VccSelector.evtFocusNext {}			widgetDefault
    option add *VccSelector.evtFocusPrev {}			widgetDefault
}

# vccConfigLcuLib:
vccSetDef vcc_c(bootScriptEndMarker) ___END_OF_BOOT_SCRIPT___

# ___oOo___
