#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccConfigLcuLib.tcl,v 2.116 2005/10/21 14:15:11 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.03.1996 created, partly adapted from vccConfigLcu.tcl
# imunoz     24.01.97   SPR 960542, added vInetBackplane.
# imunoz     24.01.97   SPR 960386, added automatic setting of gateway 
#                       when host and LCU are in different subnets.
# mkiekebu   14/06/98   Added vccGetBootDevice and removed ei hardcode
# fcarbogn   23/03/99   Corrected vccGetBootDevice to consider bspName like
#			mv167_m2-p1-s8 (SPR990042)
# fcarbogn   23/03/99   Changes for PPC slave CPU (SPR990018)
# fcarbogn   14/07/99   added vccConfigLcuGetNFS to find NFShost and real
#			path, (SPRs 960308 and 960055)
# fcarbogn   10/08/99   scan module also loaded in case of CCS-Lite boot env
# psivera    02/06/00   replaced /INTROOT with real path to the introot dir
#                       (SPR 20000208)
# fcarbogn   14/06/00   Added mv162 board for sm anchor definition
# psivera    18/12/00   merged with branch 2.46.1 and added mv1603 bsp support
#                       (SPR 20000552). Implemented SPR 20000513 (removed 
#                       all references to lcc if lcc is not in the mod list)
# psivera    11/01/01   changed nfsMount to lcubootNfsMount (SPR 20000566)
# psivera    21/01/01   changed lcubootAutoLoad to lcubootAutoLoadNoAbort (SPR 20000566)
# psivera    07/02/01   proc vccConfigLcuGetNFS: better handling of MODROOT definition
# psivera    10/05/01   restored "putenv MODROOT" (probably removed by mistake)
# psivera    23/05/01   added ACSROOT and ACE_ROOT_VW 
# psivera    05/12/01   added backplane_name stuff (SPR 20010665)
# psivera    07/12/01   avoid using remote commands to query the acc db
# psivera    11/12/01   added warning when ACC_HOST is not used about the missing 
#			LCU IP address in the .bootChange.* files
# psivera    11/07/02   implemented SPRS VLTSW 20020364 and 20020365
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
# psivera    2002-10-16 SPR 20020277: read usr and sys mod list from the template bootscript
# psivera    2003-06-02 SPR 20030225: in case of ACSROOT lqs and lcc only are loaded
# psivera    2003-06-05 SPR 20030225: syslist of mods is forced to be "lqs and lcc";
#				      usrlist of mods is forces to be empty
# psivera    2003-06-30 SPR 20030204: SPR 20030204: the gateway is taken into account 
#				      only if host and gateway are on different subnets
# psivera    2003-07-03 SPR 20020712: exit with error message if the home dir is not NFS exported
# psivera    2003-07-04 SPR VLTSW20030081: if no information on backplane the operation is aborted
#                     with an error message
# psivera    2003-07-04 the two cases: old accDataBase and no connection to the acc are distinguished.
#			additional bit to the SPR VLTSW20030081
# psivera    2003-07-07 bug fixing: changed variable res to res_conn
# psivera    2003-07-03 SPR 20020712: the exit has been changed with warning messages;
#                       the proc vccConfigLcuGetNFS has been 'catched'.
# psivera    2003-11-21 SPR 20030412: replaced 0x18 by 0x8
# psivera    2003-12-12 SPR 20030471: for slave lcus, e is set to "" and b to the IP address
# sfeyrin    2005-03-11 SPR 20050123: modified vccConfigLcuGetNFS to manage case Path without /home 
# eallaert   2005-07-21 make arg SrcTempl of vccConfigLcuMakeBootScript optional
# sfeyrin    2005-09-16 SPR 20050187 :suppress tab after vSysmodlist: and vUsrmodlist: in vccConfigLcuGenerateBootScript
# bgustafs   2005-10-19 Changed attach of ethernet on slave CPU, changed shared memory address 
#                       in boot device of slave CPU 
#

#************************************************************************
#   NAME
#	vccConfigLcuLib - LCU configuration library
# 
#   SYNOPSIS
#	vccConfigLcuGet ConfigArray_ref LcuEnv \
#			?BwsEnv? ?VltHost? ?IntHost? ?ModHost? ?LcuHost?
#
#	vccConfigLcuParseBootScript ConfigArray_ref BootScript
#
#	vccConfigLcuGenerateBootScript ConfigArray_ref \
#			?LcubootSource? ?UserScriptEnable?
#
#	vccConfigLcuMakeBootScript ConfigArray_ref Modules FileName LcuEnv \
#			?BwsEnv? ?VltHost? ?IntHost? ?ModHost? ?LcuHost? \
#			?LcubootSource? ?UserScriptEnable?
#
#	vccConfigLcuGenerateBootChange ConfigArray_ref ?BootScript?
#
#	vccConfigLcuEvalBspName bspname ?proc? ?size?
#
#     	vccConfigLcuGetNFS Path ModHost ?NfsHost? ?NfsPath?
#
#   DESCRIPTION
#     vccConfigLcuGet - Queries the configuration from the database
#
#	ConfigArray_ref	<i/o>	array containing configuration options
#	LcuEnv		<in>	name of LCU environment
#	BwsEnv		<in>	name of WS boot environment
#	VltHost		<in>	host where VLTROOT or ACSROOT resides
#	IntHost		<in>	host where INTROOT resides
#	ModHost		<in>	host where MODROOT resides
#	LcuHost		<in>	host assigned to the LCU
#
#	The configuration array is filled from the database.
#	All options are also returned as a keyed list.
#	Omitted optional arguments default to the database values. 
#
#     vccConfigLcuParseBootScript - Interprets the boot-script
#
#	ConfigArray_ref	<i/o>	array containing configuration options
#	BootScript	<in>	contents of bootScript as a string
#
#	The configuration array is filled from the contents of BootScript.
#	All options are also returned as a keyed list.
#
#     vccConfigLcuGenerateBootScript - Generates the boot-script as a string
#
#	ConfigArray_ref	 <in>	array containing configuration options
#	LcubootSource	 <in>	where lcuboot module shall be loaded from,
#				either VLTROOT or INTROOT
#	UserScriptEnable <in>	flag whether userScript shall be executed.
#
#	A bootScript corresponding to the configuration array is generated
#	and returned as a string.
#
#     vccConfigLcuMakeBootScript - Makes a standard boot-script file
#
#	ConfigArray_ref	<i/o>	array containing configuration options
#	FilePath <in>	full path where bootScript shall be written;
#			if empty, then the default BOOTHOME is used.
#	Modules  <in>	is the list of modules to be loaded; 
#			additionally the type of bootScript to produce
#			can be specified by a special keyword in the list:
#			"minimum": only LCC and lqs, no drivers etc.
#			"maximum": all common modules shall be loaded
#			otherwise: exactly as specified in database
#	Other arguments are as above.
#
#	A standard bootScript is produced and written to a file.
#	This is a combination of `vccConfigLcuGet' and 
#	`vccConfigLcuGenerateBootScript'.
#
#     vccConfigLcuGenerateBootChange - generate `bootChange' sequence
#
#	ConfigArray_ref	 <in>	array containing configuration options
#	BootScript	 <in>	full path of bootScript to be set,
#				by default use no bootScript.
#
#	Generates a script to be sourced by LCU to configure the boot-line
#	and returns it as a string.
#
#     vccConfigLcuEvalBspName - analyse or construct BSP name 
#			based on size and proc convention
#
#     vccConfigLcuGetNFS - retrieve real path and NFS host from /etc/auto_home 
#			   for NFS mouted users
#   ENVIRONMENT
#	VLTDATA		path to VLT data area
#	VLTROOT		path to VLT code area
#	INTROOT		path to private integration code area (optional)
#	MODROOT		path to private module area (optional)
#	VXROOT		path to VxWorks files
#
#	Other variables see VccInfo(n).
#
#   FILES
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   DIAGNOSTICS
#
#   BUGS  
#	vccConfigLcuGenerateBootScript: the generated lines are not checked
#	for a maximum length. This can lead to problems when the bootScript
#	is sourced under the VxWorks shell, since here max. 128 chars or so
#	are permitted. If this is exceeded, the boot will fail.
#
#   SEE ALSO
#	bootScript(5), userScript(5), module.boot(5),
#	VccInfo(n),
#	lcuboot(1)
#
#------------------------------------------------------------------------
#

vccSourceConfigFile vccConfigLcu

#
# Queries the configuration from the database
#
proc vccConfigLcuGet {
    ConfigArray_ref LcuEnv 
    {BwsEnv ""} {VltHost ""} {IntHost ""} {ModHost ""} {LcuHost ""}
} {
    global vcc_n env tcl_platform

    # NOTE: "gvar" is NOT global here!
    # All elements of gvar are passed to the given array "ConfigArray_ref".
    # All what is an empty string in gvar is NOT passed (left untouched).
    # All what is only white space is passed as empty string.

    if {![lempty $ConfigArray_ref]} {
	upvar $ConfigArray_ref ConfigArray ;# bind given array name to local
    }

    # query vcc info for selected LCU-environment:
    set gvar(vLcuEnv) $LcuEnv

    if { [catch { set acc_host  [set env(ACC_HOST)] }] } {
	puts ""
        puts "WARNING: ACC_HOST not defined, we can't automatically gather information on the LCU host IP address!"
        puts "Please, add by hand this information in the .bootChange.start and .bootChange.stop!"
        puts "We remind you that the use of the ACC_HOST+acc database is strongly recommended"
	puts ""
    }

    # get the LCU hostname
    if {$LcuHost == ""} {set LcuHost [vccInfo GetByEnv $LcuEnv hostName]}
    set gvar(vLcuHost) $LcuHost

    # get the parameters for the booting interface
    lassign [vccInfo GetByEnv $LcuEnv tcpPort] gvar(vLcuTCPport)

    # get the parameters for the network interfaces
    lassign [vccInfo GetByEnv $LcuEnv {hostName0 hostName1 hostName2}] \
	    gvar(vLcuHost0) gvar(vLcuHost1) gvar(vLcuHost2)

    # clear fields for additional network interfaces:
    if {$gvar(vLcuHost0) == ""} {set gvar(vLcuHost0) " "}
    if {$gvar(vLcuHost1) == ""} {set gvar(vLcuHost1) " "}
    if {$gvar(vLcuHost2) == ""} {set gvar(vLcuHost2) " "}

    # get parameters for these hosts:
    lassign [vccInfo GetByHost $gvar(vLcuHost) \
	    {ipAddr cpuName  hostType bspName}] \
	    gvar(vLcuIPaddr) gvar(vLcuCpu) gvar(vLcuType) gvar(vLcuBsp)

    # if the main network interface is empty, then assume the boot interface
    if {[string trim $gvar(vLcuHost0)] != ""} {
	set gvar(vLcuIPaddr0) [vccInfo GetByHost $gvar(vLcuHost0) ipAddr]
    } else {
	set gvar(vLcuHost0)   $gvar(vLcuHost)
	set gvar(vLcuIPaddr0) $gvar(vLcuIPaddr)
    }

    if {[string trim $gvar(vLcuHost1)] != ""} {
	set gvar(vLcuIPaddr1) [vccInfo GetByHost $gvar(vLcuHost1) ipAddr]
    } else {
	set gvar(vLcuIPaddr1) " "
    }

    if {[string trim $gvar(vLcuHost2)] != ""} {
	set gvar(vLcuIPaddr2) [vccInfo GetByHost $gvar(vLcuHost2) ipAddr]
    } else {
	set gvar(vLcuIPaddr2) " "
    }

    # note: "vccInfo GetByEnv $LcuEnv bootEnv" does not work with ACCDB (or?)
    set gvar(vBwsEnv) [vccInfo GetBootEnvByEnv $LcuEnv]

    # note: the TBDs should in future be added to the database!
    # TBD: set gvar(vSysmodlist) [vccInfo GetByEnv $LcuEnv sysModlist]
    # TBD: set gvar(vUsrmodlist) [vccInfo GetByEnv $LcuEnv usrModlist]
    # TBD: set gvar(vDevlist)    [vccInfo GetByEnv $LcuEnv devs]
    set gvar(vProlist)    [vccInfo GetProcsByEnv $LcuEnv]

    # query vcc info for selected boot-environment:
    if {[lempty $BwsEnv]} {set BwsEnv $gvar(vBwsEnv)}
    set gvar(vBwsEnv) $BwsEnv

    lassign [vccInfo GetByEnv $gvar(vBwsEnv) {hostName tcpPort}] \
	    gvar(vBwsHost) gvar(vBwsTCPport)

    lassign [vccInfo GetByHost $gvar(vBwsHost) {ipAddr cpuName hostType}] \
	    gvar(vBwsIPaddr) gvar(vBwsCpu) gvar(vBwsType)

    # query the root-variables on boot host:
    set gvar(VXROOT)  [vccRemPath $gvar(vBwsHost) \$VXROOT]
    set gvar(BOOTROOT) [vccRemPath $gvar(vBwsHost) \$VLTDATA]
    set gvar(BOOTHOME) $gvar(BOOTROOT)/$vcc_n(envDir)/$gvar(vLcuEnv)

    # query info for VLTROOT host:
    if {[lempty $VltHost]} {set VltHost $gvar(vBwsHost)}
    set gvar(vVltHost) $VltHost
    set gvar(VLTROOT) [vccRemPath $VltHost \$VLTROOT]
    set gvar(ACSROOT) [vccRemPath $VltHost \$ACSROOT]
    set gvar(ACE_ROOT_VW) [vccRemPath $VltHost \$ACE_ROOT_VW]
    set gvar(vVltIPaddr) [vccInfo GetByHost $VltHost ipAddr]

    # query info for INTROOT host:
    if {[lempty $IntHost]} {set IntHost $gvar(vBwsHost)}
    set gvar(vIntHost) $IntHost
    set gvar(INTROOT) [vccRemPath $IntHost \$INTROOT]
    if { ! [ catch { exec ypcat -k auto.home } result_ypcat ] && [regexp /home/(\[a-z0-9\]*) $gvar(INTROOT) dummy searchKey] == 1 } {
	catch { lassign [vccConfigLcuGetNFS $gvar(INTROOT) $IntHost] IntHost gvar(INTROOT) } nfs_result
    }
    set gvar(vIntIPaddr) [vccInfo GetByHost $IntHost ipAddr]
    set gvar(vIntHost) $IntHost

    # query info for MODROOT host:
    if {[lempty $ModHost]} {set ModHost $gvar(vBwsHost)}
    set gvar(MODROOT) ""
    catch { set modroot [exec $vcc_n(posixsh) -c "echo \$MODROOT"]} output
#    if {! [lempty $modroot]} {lassign [vccConfigLcuGetNFS $modroot $ModHost] ModHost gvar(MODROOT) }

    if {! [lempty $modroot]} {
        catch { lassign [vccConfigLcuGetNFS $modroot $ModHost] ModHost gvar(MODROOT) } nfs_result
    }


    set gvar(vModHost) $ModHost
    set gvar(vModIPaddr) [vccInfo GetByHost $ModHost ipAddr]

    # Init network vars:
    set gvar(bootUser)  vx
    set gvar(vPassword) " "
    set gvar(vGateway)  " "
    set gvar(vInetBackplane) " " ;# Default value for the backplane ip address: is empty

    # At the moment, the possibility to pass the information about the backplane 
    # from command line is not implemented. So the ACC_HOST is needed. 

    if { [catch { set acc_host  [set env(ACC_HOST)] }] } {
        puts "Warning for backplane lcus users: ACC_HOST not defined, we can't gather information on the backplane!"

    } else {

        # if the ACC_HOST is defined, we can query the database to see whether it has 
        # been updated with the backplane or not. 

	# first check whether it is possible to connect to the acc database
	set res_conn [ catch { exec msqlrelshow -h $acc_host } check_acc_conn ]
	if { $res_conn != 0 } {
	    puts "Cannot connect to acc Database. Check the value of ACC_HOST and if the msqld is running."
	    puts "The operation is aborted with return code 67"
	    exit 67
	}

	# then we try to find information about the backplane:
	set res [ catch { exec msqlrelshow -h $acc_host acc station | grep backplane } check_new_acc ]

	if { $res == 0 } {

	    # 1. if the acc db has been updated, we can check whether the lcu environment has got a backplane.

	    # lindex is necessary because whenever the backplane_name field is empty
            # the command vccInfo ... used in the following line
            # returns a {}
            set bkplaneName [lindex [vccInfo GetByHost $gvar(vLcuHost) backplaneName] 0]

            # we eliminate trailing characters and white spaces...
            set bkplaneName [string trim $bkplaneName]
            # ... then it is safe to do the check as in 3.

	    # 2. then check which Tornado version is used

            set tornado_dir [set env(WIND_BASE)]
            set tornado_list [split $tornado_dir /]
            set num_of_elements [llength $tornado_list]
            set last_el_index [expr {$num_of_elements-1}]
            set tornado_version [lindex $tornado_list $last_el_index]
            set new_list [split $tornado_version w]
            set number [lindex $new_list 1]

            # 3. If both 1. and 2. are verified, look in the ACC db for the ip address of the backplane

            if { $bkplaneName != "" && $number >= 5.4 } {
                set bkplaneIp [lindex [vccInfo GetByHost $bkplaneName ipAddr] 0]
                set gvar(vInetBackplane) $bkplaneIp
            }

        } else {

            # SPR VLTSW20030081 BEGIN
            puts "Cannot gather backplane information from the ACC database."
            puts "The accData.sql has not been updated."
            puts "The operation is aborted with return code 77."
            exit 77
            # SPR VLTSW20030081 END

        }
    }

    set gvar(vSubnet)   $vcc_n(netmask)

    # check if host and LCU are in the same subnet
    set wSubnet [vccInfo GetByHost $gvar(vBwsHost) subnetName]
    set lSubnet [vccInfo GetByHost $gvar(vLcuHost) subnetName]
    if {$wSubnet != $lSubnet} {
	set gvar(vGateway) [vccInfo GetRouterIpBySubnet $lSubnet]
    }


    set gvar(nfsUser) $gvar(bootUser)
    catch {id convert user $gvar(nfsUser)} gvar(nfsUserId)
    set gvar(nfsGroup) [id group]
    catch {id convert group $gvar(nfsGroup)} gvar(nfsGroupId)

    # pass array elements to caller and return them as a list:
    foreach name [array names gvar] {
	if {$ConfigArray_ref != ""} {
	    # set only those items that are not empty in DB or are not yet set:
	    if {$gvar($name) != "" || ! [info exists ConfigArray($name)]} {
		set ConfigArray($name) [string trim $gvar($name)]
	    } else {
		# (commented, too much output!)
		#puts "warning:\
		#	item $name not found in VCC database;\
		#	using instead: \"$ConfigArray($name)\""
	    }
	}
	lappend config [list $name [string trim $gvar($name)]]
    }
    return $config
}

#
# Interprets the boot-script
#
proc vccConfigLcuParseBootScript { ConfigArray_ref BootScript } {

    if {![lempty $ConfigArray_ref]} {
	upvar $ConfigArray_ref ConfigArray ;# bind given array name to local
    }

    set key {#::: +([A-Z_]+)} ;# regexp to extract keyword
    set keyword ""

    foreach line [split $BootScript \n] {

	set m ""; set a ""; set b ""; set c ""
	set newkey [regexp $key $line m keyword]
	switch $keyword {
	    VCCDATA {
		if {[regexp {^# +([^:]+):[ 	]+(.*)} $line m a b]} {
		    vccPuts "$keyword: $a = $b"

		    # check for backward compatibility:
		    switch $a {
			vSysmodlist {
			    set i [lsearch -exact $b scanOut]
			    if {$i > 0} {
				set b [lreplace $b $i $i scan]
				puts {vccConfigLcuParseBootScript:\
					"scanOut" obsolete, replaced by "scan"}
			    }
			}
		    }

		    # put value into config-array:
		    set gvar($a) [string trim $b] ;# drop any trash-blanks!
		}
	    }
	    ROOT {
		# no action anymore; removed in 1.73.
	    }
	    NETWORK {
		if {[regexp {^nfsAuthUnixSet +[^,]+, *([0-9]+) *, *([0-9]+)}  $line m a b]} {
		    set gvar(nfsUserId)  $a
		    set gvar(nfsGroupId) $b
		    # translated IDs are optionally stored as a comment:
		    regexp {/\* +([^ ,]+) *, *([^ ]+) +\*/} $line m a b
		    set gvar(nfsUser)  $a
		    set gvar(nfsGroup) $b
		    vccPuts "$keyword: $gvar(nfsUserId),$gvar(nfsGroupId)"
		}
	    }
	    ENVIRONMENT {
		# parse "Boot Environment":
		if {[regexp {^putenv +"HOSTENV=([^""]+)"} $line m a]} {
		    vccPuts "$keyword: $a = $b"
		    set gvar(vBwsEnv) $a
		    #NOTE: this will NOT set the hostname + IP address!
		}
	    }
	    MODULES {
		# no action anymore; removed in 1.73.
	    }
	    DEVICES {
		# no action anymore; removed in 1.73.
	    }
	    USER {
		if {$newkey} {
		    set gvar(vUserScript) 0
		}
		if {[regexp {^< +userScript} $line]} {
		    set gvar(vUserScript) 1
		}
	    }
	}
    }

    # pass array elements to caller and return them as a list:
    foreach name [array names gvar] {
	if {![lempty $ConfigArray_ref]} {set ConfigArray($name) $gvar($name)}
	lappend config [list $name $gvar($name)]
    }
    return $config
}

#
# Generates the boot-script as a string
#
proc vccConfigLcuGenerateBootScript { 
    ConfigArray_ref {LcubootSource VLTROOT} {UserScriptEnable 1}
} {
    global argv0 vcc_c env tcl_platform

    upvar $ConfigArray_ref gvar ;# bind given array name to local `gvar'

    set x {} ;# the generated text will be appended here

    # general variables used in the algorithm for SPR VLTSW 20020365
    # just look for 20020365
    set lSubnet [vccInfo GetByHost $gvar(vLcuHost) subnetName]
    # SPR 20030204: the following line is commented
    #set gvar(vGateway) [vccInfo GetRouterIpBySubnet $lSubnet]
    set vltSubnet dummy
    set intSubnet dummy
    set modSubnet dummy

    # write header:
    set revision [lindex {$Revision: 2.116 $} 1]
    append x "#! VxWorks
#******************************************************************************
# E.S.O. - VLT project
#
# \"@(#) \$Id\$\"
#
# THIS IS A GENERATED FILE - DO NOT EDIT (unless you know how)
#
# Created by
#   User: $tcl_platform(user)
#   Host: [set host {}; if {[info exists env(HOSTNAME)]} {set host $env(HOSTNAME) \
    } elseif {[info exists env(HOST)]} {set host $env(HOST)} ; set host]
#   Date: [clock format [clock seconds]]
#   Tool: [file tail $argv0]
#   Proc: vccConfigLcuGenerateBootScript $revision
#
"

    # write section VCCDATA:
    #
    # Note: there may be problems with the maximum line-length under VxWorks!
    # The boot will fail if this length (~ 128 chars) is exceeded.
    # SPR 960402: The vProlist is no longer output here due to this reason.
    #
    append x "\n#:::   VCCDATA   :::
# vLcuEnv:	$gvar(vLcuEnv)
# vLcuHost:	$gvar(vLcuHost)
# vLcuIPaddr:	$gvar(vLcuIPaddr)
# vLcuHost0:	$gvar(vLcuHost0)
# vLcuIPaddr0:	$gvar(vLcuIPaddr0)
# vLcuHost1:	$gvar(vLcuHost1)
# vLcuIPaddr1:	$gvar(vLcuIPaddr1)
# vLcuHost2:	$gvar(vLcuHost2)
# vLcuIPaddr2:	$gvar(vLcuIPaddr2)
# vLcuTCPport:	$gvar(vLcuTCPport)
# vLcuCpu:	$gvar(vLcuCpu)
# vLcuType:	$gvar(vLcuType)
# vLcuBsp:	$gvar(vLcuBsp)
# vBwsEnv:	$gvar(vBwsEnv)
# vBwsHost:	$gvar(vBwsHost)
# vBwsIPaddr:	$gvar(vBwsIPaddr)
# vBwsTCPport:	$gvar(vBwsTCPport)
# vBwsCpu:	$gvar(vBwsCpu)
# vBwsType:	$gvar(vBwsType)
# vGateway:	$gvar(vGateway)
# BOOTROOT:	$gvar(BOOTROOT)
# BOOTHOME:	$gvar(BOOTHOME)
# VXROOT:	$gvar(VXROOT)
# vVltHost:	$gvar(vVltHost)
# vVltIPaddr:	$gvar(vVltIPaddr)
# VLTROOT:	$gvar(VLTROOT)
# ACSROOT:	$gvar(ACSROOT)
# ACE_ROOT_VW:	$gvar(ACE_ROOT_VW)
# vIntHost:	$gvar(vIntHost)
# vIntIPaddr:	$gvar(vIntIPaddr)
# INTROOT:	$gvar(INTROOT)
# vModHost:	$gvar(vModHost)
# vModIPaddr:	$gvar(vModIPaddr)
# MODROOT:	$gvar(MODROOT)
# vSysmodlist: $gvar(vSysmodlist)
# vUsrmodlist: $gvar(vUsrmodlist)
# vDevlist:	$gvar(vDevlist)
#
"

    # write section ROOT:
    append x "\n#:::   ROOT   :::\n"
    append x "putenv \"BOOTROOT=$gvar(BOOTROOT)\"\n"
    append x "putenv \"BOOTHOME=$gvar(BOOTHOME)\"\n"
    append x "putenv \"VLTROOT=/VLTROOT\"\n"
    if {$gvar(INTROOT) == ""} {append x #}
    #append x "putenv \"INTROOT=/INTROOT\"\n"
    append x "putenv \"INTROOT=$gvar(INTROOT)\"\n"
    if {$gvar(MODROOT) == ""} {append x #}
    append x "putenv \"MODROOT=/MODROOT\"\n"
    if {$gvar(ACSROOT) == ""} {append x #}
    append x "putenv \"ACSROOT=/ACSROOT\"\n"
    if {$gvar(ACE_ROOT_VW) == ""} {append x #}
    append x "putenv \"ACE_ROOT_VW=/ACE_ROOT_VW\"\n"


    # write section LCUBOOT:
    append x "\n#:::   LCUBOOT   :::\n"
    append x "cd \"$gvar(VLTROOT)\"\n"
    if {$gvar(vLcuCpu) == ""} {
	append x "ld 0,1,\"vw/bin/lcuboot\"\n"
    } else {
	append x "ld 0,1,\"vw/bin/$gvar(vLcuCpu)/lcuboot\"\n"
    }
    # IvO 1996-10-04 16:39 EDT, init lcuboot to log messages
    append x "lcubootLogInit\n"

    # write section NETWORK:
    append x "\n#:::   NETWORK   :::\n"

    # for a slave CPU we have to attach the ethernet explicitly:
    lassign [vccConfigLcuEvalBspName $gvar(vLcuBsp)] bspname procnum
	# Attach on-board ethernet I/F of slave CPU MV167 or MV2604
	#
	# ipAttach (int unit, char *bootDev)
	#
	#
    if { $procnum > 0} {

	# Check which Tornado version is used

	set tornado_dir [set env(WIND_BASE)]
	set tornado_list [split $tornado_dir /]
	set num_of_elements [llength $tornado_list]
	set last_el_index [expr {$num_of_elements-1}]
	set tornado_version [lindex $tornado_list $last_el_index]
	set new_list [split $tornado_version w]
	set number [lindex $new_list 1]
	
    switch -regexp -- $bspname {
	"mv167.*" {
	    if { $number <= 5.4 } {
		append x "eiattach 0,0x40+7,0x6c,-1,0,0\n"
	    } else {
		append x "ipAttach 0,\"ei\"\n"
	    }
	    append x "ifMaskSet \"[vccGetBootDevice $bspname]0\",0x$gvar(vSubnet)\n"
	    append x "ifAddrSet \"[vccGetBootDevice $bspname]0\",\"$gvar(vLcuIPaddr0)\"\n"
        }
	"mv2604.*" {
	    if { $number <= 5.4 } {
		append x "dcattach 0, 0xfe020000, 18, 18, -1, 0, -1, 0, 0x08\n"
	    } else {
		append x "ipAttach 0,\"dc\"\n"
	    }
	    append x "ifMaskSet \"[vccGetBootDevice $bspname]0\",0x$gvar(vSubnet)\n"
	    append x "ifAddrSet \"[vccGetBootDevice $bspname]0\",\"$gvar(vLcuIPaddr0)\"\n"
        } 
	"mv2700.*" {
	    if { $number <= 5.4 } {
	    append x "dcattach 0, 0xfe020000, 18, 18, -1, 0, -1, 0, 0x08\n"
	    } else {
		append x "ipAttach 0,\"dc\"\n"
	    }
	    append x "ifMaskSet \"[vccGetBootDevice $bspname]0\",0x$gvar(vSubnet)\n"
	    append x "ifAddrSet \"[vccGetBootDevice $bspname]0\",\"$gvar(vLcuIPaddr0)\"\n"
        }
	"mv1603.*" {
	    if { $number <= 5.4 } {
		append x "dcattach 0, 0xfe020000, 18, 18, -1, 0, -1, 0, 0x08\n"
	    } else {
		append x "ipAttach 0,\"dc\"\n"
	    }
	    append x "ifMaskSet \"[vccGetBootDevice $bspname]0\",0x$gvar(vSubnet)\n"
	    append x "ifAddrSet \"[vccGetBootDevice $bspname]0\",\"$gvar(vLcuIPaddr0)\"\n"
        }
      }
    }
   
    append x "nfsIdSet $gvar(nfsUserId) /* $gvar(nfsUser) */\n"

    # NFS-config for boot-host:
    append x "nfsAuthUnixSet \"$gvar(vBwsHost)\",$gvar(nfsUserId),$gvar(nfsGroupId) /* $gvar(nfsUser),$gvar(nfsGroup) */\n"

    # SPR 970034: explicit BOOTROOT mount
    # NOTE: BOOTROOT must currently be mounted under the same local name!
    append x "lcubootNfsMount \"$gvar(vBwsHost)\",\"$gvar(BOOTROOT)\",0\n"

    # implicit NFS-config for VLTROOT-INTROOT-MODROOT-ACSROOT-ACE_ROOT_VW (as far as defined):
    # NOTE that the LCU-local filenames are fixed to "/xxxROOT"!
    if {$gvar(VLTROOT) != ""} {

	# SPR 20020365
	# look for which subnet the vlt host belongs to
        set vltSubnet [vccInfo GetByHost $gvar(vVltHost) subnetName]
	# if the subnet for vlt host is different from the lcu subnet:
        if {$vltSubnet != $lSubnet} {
	    # find the router for the subnet of the vlt host and the network for the vlt host
	    # (to find the network, substitute 0 with the 4 byte if the ip address of the vlt host)

	    # SPR 20030204
	    set gvar(vGateway) [vccInfo GetRouterIpBySubnet $lSubnet]
	    # end SPR 20030204

            set vltHostGw [vccInfo GetRouterIpBySubnet $vltSubnet]
            set vltHostlist [split $vltHostGw {.}]
            set vltHostNetlist [lreplace $vltHostlist 3 3 0]
            set vltHostNet  [join $vltHostNetlist {.}]
	    # add the router to the network in the bootscript
            append x "routeAdd \"$vltHostNet\",\"$gvar(vGateway)\"\n"
        }

      append x "hostAdd \"$gvar(vVltHost)\",\"$gvar(vVltIPaddr)\"\n"
      append x "nfsAuthUnixSet \"$gvar(vVltHost)\",$gvar(nfsUserId),$gvar(nfsGroupId) /* $gvar(nfsUser),$gvar(nfsGroup) */\n"
      append x "lcubootNfsMount \"$gvar(vVltHost)\",\"$gvar(VLTROOT)\",\"/VLTROOT\"\n"
    }
    if {$gvar(INTROOT) != ""} {

	# SPR 20020365: same algorithm as for VLTROOT
        set intSubnet [vccInfo GetByHost $gvar(vIntHost) subnetName]
	if {$intSubnet != $lSubnet && $intSubnet != $vltSubnet} {

	    # SPR 20030204
	    set gvar(vGateway) [vccInfo GetRouterIpBySubnet $lSubnet]
	    # end SPR 20030204

	    set intHostGw [vccInfo GetRouterIpBySubnet $intSubnet]
            set intHostlist [split $intHostGw {.}]
            set intHostNetlist [lreplace $intHostlist 3 3 0]
            set intHostNet  [join $intHostNetlist {.}]
            append x "routeAdd \"$intHostNet\",\"$gvar(vGateway)\"\n"
         }

      append x "hostAdd \"$gvar(vIntHost)\",\"$gvar(vIntIPaddr)\"\n"
      append x "nfsAuthUnixSet \"$gvar(vIntHost)\",$gvar(nfsUserId),$gvar(nfsGroupId) /* $gvar(nfsUser),$gvar(nfsGroup) */\n"
      #append x "lcubootNfsMount \"$gvar(vIntHost)\",\"$gvar(INTROOT)\",\"/INTROOT\"\n"
      append x "lcubootNfsMount \"$gvar(vIntHost)\",\"$gvar(INTROOT)\"\n"
    }

    if {$gvar(MODROOT) != ""} {

	# SPR 20020365: same algorithm as for VLTROOT
        set modSubnet [vccInfo GetByHost $gvar(vModHost) subnetName]
        if {$modSubnet != $lSubnet && $modSubnet != $vltSubnet || $modSubnet != $lSubnet && $modSubnet != $intSubnet} {

	    # SPR 20030204
	    set gvar(vGateway) [vccInfo GetRouterIpBySubnet $lSubnet]
	    # end SPR 20030204

            set modHostGw [vccInfo GetRouterIpBySubnet $modSubnet]
            set modHostlist [split $modHostGw {.}]
            set modHostNetlist [lreplace $modHostlist 3 3 0]
            set modHostNet  [join $modHostNetlist {.}]
            append x "routeAdd \"$modHostNet\",\"$gvar(vGateway)\"\n"
        }

      append x "hostAdd \"$gvar(vModHost)\",\"$gvar(vModIPaddr)\"\n"
      append x "nfsAuthUnixSet \"$gvar(vModHost)\",$gvar(nfsUserId),$gvar(nfsGroupId) /* $gvar(nfsUser),$gvar(nfsGroup) */\n"
      append x "lcubootNfsMount \"$gvar(vModHost)\",\"$gvar(MODROOT)\",\"/MODROOT\"\n"
    }
    if {$gvar(ACSROOT) != ""} {
      append x "lcubootNfsMount \"$gvar(vVltHost)\",\"$gvar(ACSROOT)\",\"/ACSROOT\"\n"
    }
    if {$gvar(ACE_ROOT_VW) != ""} {
      append x "lcubootNfsMount \"$gvar(vVltHost)\",\"$gvar(ACE_ROOT_VW)\",\"/ACE_ROOT_VW\"\n"
    }

    # write section ENVIRONMENT:
    append x "\n#:::   ENVIRONMENT   :::\n"
    if {$gvar(vLcuCpu) == ""} {append x #}
    append x "putenv \"CPU=$gvar(vLcuCpu)\"\n"
    append x "putenv \"LOCALENV=$gvar(vLcuEnv)\"\n"
    append x "putenv \"LOCALTCPPORT=$gvar(vLcuTCPport)\"\n"
    append x "putenv \"HOSTENV=$gvar(vBwsEnv)\"\n"
    append x "putenv \"HOSTTCPPORT=$gvar(vBwsTCPport)\"\n"

    # host/address of the booting interface
    append x "putenv \"LOCALHOST=$gvar(vLcuHost)\"\n"
    append x "putenv \"LOCALIPADDRESS=$gvar(vLcuIPaddr)\"\n"

    # host/address of the additional network interface (main i/f not relevant)
    if {$gvar(vLcuHost1) == ""} {append x #}
    append x "putenv \"LOCALHOST_1=$gvar(vLcuHost1)\"\n"
    if {$gvar(vLcuIPaddr1) == ""} {append x #}
    append x "putenv \"LOCALIPADDRESS_1=$gvar(vLcuIPaddr1)\"\n"

    if {$gvar(vLcuHost2) == ""} {append x #}
    append x "putenv \"LOCALHOST_2=$gvar(vLcuHost2)\"\n"
    if {$gvar(vLcuIPaddr2) == ""} {append x #}
    append x "putenv \"LOCALIPADDRESS_2=$gvar(vLcuIPaddr2)\"\n"

    append x "lcubootAutoEnvInit /* set rest of variables automatically */\n"

    # write section MODULES:
    append x "\n#:::   MODULES   :::\n"
    # first load all system- and user-modules:
#    append x "#:SYSTEM: $gvar(vSysmodlist)\n"
#    append x "#:USER: $gvar(vUsrmodlist)\n"
    set drvAuto {} ;# drivers to be installed automatically
    foreach dev $gvar(vDevlist) {
	set cnt [lindex $dev 0] ;# expected device-count, "?" means "auto"
	if {$cnt == "?" || $cnt == 0 || $cnt == -1 || $cnt == ""} {
	    lappend drvAuto [string trim [lindex $dev 1] /0123456789]
	}
    }
    if {$gvar(ACSROOT) != ""} {
	set gvar(vSysmodlist) "lqs lcc"
    }
    foreach mod $gvar(vSysmodlist) {
	# defer loading for auto-installed drivers to their sub-boot-script:
	if {[lsearch $drvAuto $mod] == -1} {
	    append x "lcubootAutoLoadNoAbort 1,\"${mod}\",0\n"
	} else {
	    append x "# auto-install with defered loading for driver: $mod\n"
	}
    }
    foreach mod $gvar(vUsrmodlist) {
        append x "lcubootAutoLoadNoAbort 1,\"${mod}\",0\n"
    }
    append x "# Initialize System-Modules:\n"
    foreach mod $gvar(vSysmodlist) {
	append x "lcubootAutoCdBoot 1,\"${mod}.boot\"\n"
	append x "< ${mod}.boot\n"
    }
    # Note: Initialization of User-Modules is done in section USER.

    
    set tot_mod_list [concat $gvar(vSysmodlist) $gvar(vUsrmodlist)]
    
    # write section DEVICES:
    append x "\n#:::   DEVICES   :::\n"
    foreach dev $gvar(vDevlist) {
        set num  [lindex $dev 0]
        set name [lindex $dev 1]
        if {$num == "" || $num == "?"} {set num -1; append x #}
        append x "lcubootAutoDevCheck \"$name\",$num\n"
    }

    # write section USER:
    append x "\n#:::   USER   :::\n"
    append x "# Initialize User-Modules:\n"
    foreach mod $gvar(vUsrmodlist) {
	append x "lcubootAutoCdBoot 1,\"${mod}.boot\"\n"
	append x "< ${mod}.boot\n"
    }
    append x "# Execute User-Script:\n"
    append x "cd getenv(\"BOOTHOME\")\n"
    if {! $UserScriptEnable} {append x #}
    append x "< userScript\n"

    # write section TERMINATION:
    append x "\n#:::   TERMINATION   :::\n"
    append x "taskDelay 100\n"
    set found [lsearch -exact $tot_mod_list lcc]
    if { $found >= 0 } {
        append x "lcubootAutoSpawn 1,\"tLccResumeInit\",80,0x8,40000,\"lccResumeInit\"\n"
        append x "lcubootAutoExec 1,\"lccWaitInit\"\n"
    }
    # IvO, 1996-10-04 16:40 EDT, finish lcuboot log messages
    append x "lcubootLogFinish\n"
 
    append x "\n# $vcc_c(bootScriptEndMarker)\n"

    return $x
}

#
# Makes a standard boot-script file
#
proc vccConfigLcuMakeBootScript {
    ConfigArray_ref FilePath Modules
    LcuEnv {BwsEnv ""} {VltHost ""} {IntHost ""} {ModHost ""} {LcuHost ""} {SrcTempl ""}
    {LcubootSource VLTROOT} {UserScriptEnable 1}
} {
    global env vccConfigLcu_c

    upvar $ConfigArray_ref gvar ;# bind given array name to local `gvar'

    vccSetDef gvar(vSysmodlist) {}
    vccSetDef gvar(vUsrmodlist) {}
    vccSetDef gvar(vDevlist)    {}
    vccSetDef gvar(vProlist)    {}

    # get standard config:
    vccConfigLcuGet gvar $LcuEnv $BwsEnv $VltHost $IntHost $ModHost $LcuHost

    # modify BOOTHOME directory, if explicit path to bootScript is given,
    # otherwise use default BOOTHOME as path:
    if {$FilePath == ""} {
	set FilePath $gvar(BOOTHOME)
    } else {
	set gvar(BOOTHOME) [vccRemPath $gvar(vBwsHost) $FilePath]
    }

    # extract keywords from Modules:
    set minmax [intersect $Modules {minimum maximum}]

    # modify for minimum/maximum flags:
    set minmax [string tolower $minmax]
    switch -exact -- $minmax {
	minimum -
	maximum {
	    # set items from configuration file definitions, or empty if none:
	    foreach i {vSysmodlist vUsrmodlist vDevlist vProlist} {
		set gvar($i) {}
		catch {set gvar($i) $vccConfigLcu_c($i,$minmax)}
	    }
	}
    }


    # assemble full module lists, divide into system and user modules:
    foreach mod $Modules {
	if {[lsearch {minimum maximum} $mod] >= 0} {
						;# drop keywords
	} elseif {[lsearch $vccConfigLcu_c(sysMods) $mod] >= 0} {
						;# drop keywords
	    if {[lsearch $gvar(vSysmodlist) $mod] < 0} {
		lappend gvar(vSysmodlist) $mod	;# regarded as system-module
	    }
	} else {
						;# drop keywords
	    if {[lsearch $gvar(vUsrmodlist) $mod] < 0} {
		lappend gvar(vUsrmodlist) $mod	;# regarded as user-module
	    }
	}
    }

    # SPR 20020277: if a bootScript template is specified, 
    # read and set the vSysmodlist and vUsrmodlist from the template file.
    catch {set SrcTempl} out
    if { $out != "" && [file exists ${FilePath}/bootScript]} {

    if {[catch {open ${FilePath}/bootScript r} fileId]} {
	puts "Cannot open ${FilePath}/bootScript"
    } else {
	foreach line [split [read $fileId] \n] {
	    puts $line
	    if {[string match *vSysmodlist* $line]} {
		set gvar(vSysmodlist) [lindex [split $line :] 1]
	    }
	    if {[string match *vUsrmodlist* $line]} {
		set gvar(vUsrmodlist) [lindex [split $line :] 1]
	    }
	    # SPR 20030225
	    if {[info exists env(ACSROOT)]} {set gvar(vSysmodlist) "lqs lcc"}
	    if {[info exists env(ACSROOT)]} {set gvar(vUsrmodlist) {}}
	}
    }
    close $fileId

    }
    # END SPR 20020277


    # write file:
    set fd [open ${FilePath}/bootScript w]
    puts $fd \
	[vccConfigLcuGenerateBootScript gvar $LcubootSource $UserScriptEnable]
    close $fd

}


#
# generate `bootChange' sequence to be sourced by LCU
#
proc vccConfigLcuGenerateBootChange { ConfigArray_ref {BootScript ""} } {

    upvar $ConfigArray_ref gvar ;# bind given array name to local `gvar'
    global tcl_platform env

    set x {} ;# the generated text will be appended here

    set bootdev [vccGetBootDevice $gvar(vLcuBsp)] ;# the boot device
    
    set procnum 0		;# processor number on the backplane (0..n)
    set hostname $gvar(vBwsHost) ;# the name of the boot host
    set filename $gvar(VXROOT)/config/$gvar(vLcuBsp)/vxWorks
				;# vxWorks image file to be booted
    set subnet $gvar(vSubnet)	;# Subnet-mask in the form "ffffff00"
    set e $gvar(vLcuIPaddr):$subnet ;# Internet address  of the Ethernet I/F
    set b $gvar(vInetBackplane) ;# Internet address of the backplane I/F
    if {$b != ""} {
	set bkplane_subnet "ffffffff"
	set b $gvar(vInetBackplane):$bkplane_subnet
    }

    set h $gvar(vBwsIPaddr)	;# Internet address of the boot host
    set g $gvar(vGateway)	;# Internet address of the gateway to boot host
    set u $gvar(bootUser)	;# a valid user name on the boot host
    set pw $gvar(vPassword)	;# the password for the user on the host
    set f 0x8			;# configuration flags: no countdown
    set tn $gvar(vLcuHost)	;# the name of the system being booted
    set s $BootScript		;# file to be executed as a start-up script
    # record user and date of configuration in field not used by VxWorks:
    # (no spaces accepted!)
    set o $tcl_platform(user):[clock format [clock seconds] -format %Y/%m/%d-%H:%M:%S] 

    # overload differences for master/slave CPUs:
    lassign [vccConfigLcuEvalBspName $gvar(vLcuBsp)] bspname procnum memsize
    if {$procnum <= 0} {
	set p 0
    } elseif {$procnum != 0} {
	# for slave CPU:
	set p $procnum	               ;# slave processor number (> 0)
	set f 0x8		      ;# configuration flags: proxy client

	# These fields will be automatically determined by VxWorks:
	# (The following VxWorks kernel options are assumed to be set:
	#  INCLUDE_SM_SEQ_ADDR, INCLUDE_PROXY_DEFAULT_ADDR)
	set b ""	;# Internet address of the backplane I/F
	set g ""	;# Internet address of the gateway to boot host

	# The booting inet address is NOT the ethernet address!
	# For mv167, the main ethernet will be attached in the bootScript.
	# Anyway we set it here, although it may be ignored.
	if {[string length $gvar(vLcuIPaddr0)] > 8} {
	    set e $gvar(vLcuIPaddr0):$subnet ;# Inet address of Ethernet I/F
	} else {
	    set e ""
	}

	# the shared memory anchor shall be at this address:
    	switch -regexp -- $bspname {
	   "mv167.*"  {
	    # Check which Tornado version is used

            set tornado_dir [set env(WIND_BASE)]
            set tornado_list [split $tornado_dir /]
            set num_of_elements [llength $tornado_list]
            set last_el_index [expr {$num_of_elements-1}]
            set tornado_version [lindex $tornado_list $last_el_index]
            set new_list [split $tornado_version w]
            set number [lindex $new_list 1]

            if { $number <= 5.4 } {
		if {$memsize < 0} {set memsize 8} ;# 8MB default size
		set smstr [format 0x%08x [expr {2 * $memsize * 0x100000 + 0x600}]]
		set bootdev "sm=$smstr"	;# boot over shared memory
	    } else {
		set bootdev "sm=0x02000600"      ;# boot over shared memory
	    }
	    }
	    "mv2604.*" {
		set bootdev "sm=0x040004100"	;# boot over shared memory
	    }
            "mv2700.*" {
                set bootdev "sm=0x040004100"    ;# boot over shared memory
            }
            "mv1603.*" {
                set bootdev "sm=0x010004100"    ;# boot over shared memory
            }
            "mv162.*" {
                set bootdev "sm=0x02000600"      ;# boot over shared memory
		set f 0x8
            }
	}
        # SPR VLTSW20030471: for a slave LCU, the inet on ethernet should be empty, while the inet on backplane should show the IP addr
	set e ""
	set b $gvar(vLcuIPaddr):$subnet
    }

    set bootline [list $bootdev $p $hostname $filename $e $b $h $g $u $pw $f $tn $s $o]

    append x "bootChange\n"

    foreach i $bootline {
      if {[lempty $i]} {set i .} ;# according to VxWorks bootChange convention
      append x "$i\n"
    }

    return $x
}

#
# analyse or construct BSP name based on size and proc convention
#
proc vccConfigLcuEvalBspName { bspname {proc -1} {size -1} } {

    if {$proc < 0} {
	regexp -nocase -- {-p([0-9]+)} $bspname dummy proc
	set procstr ""
    } else {
	set procstr -p$proc
    }

    if {$size < 0} {
	regexp -nocase -- {-s([0-9]+)} $bspname dummy size
	set sizestr ""
    } else {
	set sizestr -s$size
    }

    if {![regsub -nocase -- {-p[0-9]+} $bspname $procstr bspname]} {
	append bspname $procstr
    }

    if {![regsub -nocase -- {-s[0-9]+} $bspname $sizestr bspname]} {
	append bspname $sizestr
    }

    regexp -nocase -- {-p([0-9]+)} $bspname dummy proc
    regexp -nocase -- {-s([0-9]+)} $bspname dummy size

    return [list $bspname $proc $size]
}

proc vccGetBootDevice { bspname } { 
    switch -regexp -- $bspname {
	"mv167.*"    {return "ei"}
	"pcore604.*" {return "frcDc"}
	"mv2604.*"   {return "dc"}
	"mv2700.*"   {return "dc"}
	"mv1603.*"   {return "dc"}
    }
}
    
proc vccConfigLcuGetNFS { Path Host } {

    global tcl_platform
# retrieve real path and NFS host from auto_home if Path contains 
# /home/ (so NFS mounted directory is assumed )
# Works only with the NFS conventions:
# /home/<user>

# Path is the modroot as set by the user (or by tat). 
# The real path has to be found, 
# to let the modroot be mounted by the lcu.
# Host is the local host; the physical host where the home dir 
# of the user is has to be found.

# default: 
#	returned path = input path
#	returned host = local host

    set nfsPath ""
    set nfsHost $Host

    # For the following line: if $Path is found in /home/(\[a-z0-9\]* the result is 1; if not the result is 0
    if { [regexp /home/(\[a-z0-9\]*) $Path dummy searchKey] == 1 } {
	# For the following line: the if statement is verified by the value 1; it is not verified by the value 0.
	# if ypcat exists and does not fail, catch returns 0 and if is not verified;
	# if ypcat is not used (NIS is not used), the command fails, catch returns 1 and if is verified
        if {[catch { exec ypcat -k auto.home | grep $searchKey | grep -v ^# } outstring ]} {
                catch { exec grep "^$searchKey" /etc/auto_home | grep -v ^# } outstring
        }
	# At the end of the previous if statement, in both case NIS and no NIS I have outstring 
	# containing the couple: username - real path to the home directory of username

	# If the following line is verified, it gives 1, it means that NIS is not used and the 
	# string related to the user in /etc/auto_home has the path to the home dir 
	# ending with :&
	# dummy will contain something like "vltmgr    wgsops:/diskc:&"
	# and slicePath will contain "/diskc"
        if { [regexp .*:(\/.*):\& $outstring dummy slicePath ] == 1 } {
		# In case then not a real disk is found in outstring but homelink:
                if { [ string compare $slicePath "/homelink" ] == 0 } {
			catch { exec ls -al /homelink | grep "$searchKey " } out
			# After the following command, slicePath contains the real path to the home dir of the user
			# for example: slicePath = /diska/home;
			# dummy contains /diska/home/username
			# out, coming from the previous command contains something like: 
			# username@ -> /diska/home/username
			regexp -- "-> (.*)/$searchKey" $out dummy slicePath 
                } 
		if { [string first $slicePath $Path] != 0 } {
			regsub /home $Path $slicePath nfsPath
	        }
# SPR VLTSW 20020364
# the following piece of code is replaced using shell commands; they work better!
#	if { [regexp "$searchKey.*\[\ \	\]+(\[a-z0-9\]*):" $outstring dummy outHost] == 1 } {
#                if { [ string compare $outHost "localhost" ] != 0 } {
#                        set nfsHost $outHost
#                }
#        }

    		set userName $tcl_platform(user)
    		set outHost ""
    		if {[file exists /etc/auto_home]} {
        		catch { set outHost [exec grep $userName /etc/auto_home | grep -v ^# | awk "{print \$2}"  | awk -F: "{print \$1}" ] }
    		} else {
        		puts "Error: you are not using NIS and /etc/auto_home does not exist." 
        		puts "I don't know how to find the home dir location for $userName. I exit."
        		exit 5
    		}

    		#SPR VLTSW20020712 BEGIN
    		if { $outHost == "" } {
			puts ""
			puts "Warning: Could not find the home dir location for $userName in /etc/auto_home"
			puts "Warning: Home directory not NFS exported or /etc/auto_home not used."
			if { $nfsPath != "" } {
	    		    puts "Warning: Continuing with the default: host = $nfsHost; path to be mounted = $nfsPath"
	    		    puts "Warning: the bootScript will fail if $nfsPath is not exported."
			} else {
	    		    puts "Warning: Continuing with the default: host = $nfsHost; path to be mounted = $Path"
	    		    puts "Warning: the bootScript will fail if $Path is not exported."
			}
			puts ""
#			exit 3
    		}
    		#SPR VLTSW20020712 END

    		if { [ string compare $outHost "localhost" ] != 0 && $outHost != "" } {
			set nfsHost $outHost
    		}


	# Case of NIS; but I check again, to be sure
	} elseif { ! [ catch { exec ypcat -k auto.home } result_ypcat ] } {
	    set line [lindex $outstring 1]
	    set sline [split $line :]
	    set nfsHost [lindex $sline 0]
	    set mountDir [lindex $sline 1]
	    set userName $tcl_platform(user)
	    set userlen [string length $userName]
	    set dirname [string range $Path [expr {[string first $userName $Path] + $userlen}] end]
	    set nfsPath [concat $mountDir$dirname]


	} else {
	    puts "Error: the /etc/auto_home file is not standard and you don't use NIS."
	    puts "I don't know how to find the real path to the home dir"
	    exit 6
        }
    } 


    if { $nfsPath == "" } {
    	    set parent [ lindex [split $Path /] 1 ]
    	    if { $tcl_platform(os) == "SunOS" } {
    		    set export_file /etc/dfs/dfstab
    	    } else {
    		    set export_file /etc/exports
    	    }
    	    if { [ catch { exec sh -c "grep $parent $export_file" } result ] && [ catch { exec sh -c "grep $parent /etc/auto_home | grep $searchKey" } res2 ] } {
    		    puts "Warning: cannot determine whether the modroot is an exported directory."
    		    puts "Warning: modroot parent dir not found neither in /etc/exports nor /etc/auto_home"
    		    puts "Warning: The bootScript will fail if $Path is not exported"
    	    }
    	    # Here the default is used:
    	    set nfsPath $Path
    
    }
    
    return [list $nfsHost $nfsPath]

}

# ___oOo___
