#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccLib.tcl,v 2.116 2005/10/21 14:15:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# ssandroc  19/05/95    created
# eallaert  1999-02-25  made compatible with both Tcl 7.6 and Tcl 8.0
# eallaert  2002-07-15  replaced file access via "exec" by "file" cmds
# eallaert  2005-09-28  added proc vccWait
#

#************************************************************************
#   NAME
#	vccLib,
#	vccInit, vccExit,
#	vccSetDef,
#	vccPuts
#	- VLT Central Configuration General Library
#
#   SYNOPSIS
#	vccLib
#	vccInit	?args?			==> <fileId> | ""
#	vccExit				==> ""
#	vccSetDef <Var> <Val>		==> <value of Var>
#	vccPuts <String>
#
#   DESCRIPTION
#	vccLib	- is a dummy proc that forces autoloading of this library.
#		  Only internally used.
#
#	vccInit	- creates an object named `.vccInfo' as 
#		  instance of the class `VccInfo'.
#		  All args are passed to the VccInfo constructor.
#
#	vccExit	- destroys all vcc-specific objects.
#
#	vccSetDef - Sets a global variable if not defined before.
#		    Returns the value of the variable.
#
#	vccPuts	- Prints a debugging string to stdout if the global variable 
#		  `vcc(debug)' is set. The printed line has the form:
#		  <calling proc> <args>: <String>
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	VccInfo(n),
#	seq_ccsInit(n), seq_ccsExit(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc vccLib { } {
    # dummy proc to force auto-loading
}


proc vccSetDef { Var Val } {
    set var $Var
    global [ctoken var \(]	;# for arrays only use basename before "(..)"

    if {![info exists $Var]} {
	return [set $Var $Val]	;# not yet defined: set and return new value
    } else {
	return [set $Var]	;# was already defined: return old value
    }
}

proc vccInit { args } {
    # create the object named vccInfo, if not already done:
    set objects [itcl::find objects vccInfo]
    if {[lempty $objects]} { eval VccInfo vccInfo $args }

    return ""
}

proc vccExit { } {
    # remove the object named vccInfo, if not already done:
    catch {destroy .vccInfo}
    # this does not work (?):
    #if {[itcl_info objects vccInfo] != {}} {
	#destroy vccInfo
    #}
}


proc vccPuts { String } {
    global vcc
    if {$vcc(debug)} {
	if {[catch {info level -1} procname]} {set procname ""}
	puts "$procname: $String"
    }
}

proc vccWait {ms} {
    set i 1
    while {[info exists ::vccTimeoutFlag$i]} {
	incr i;		# should build in a limit
	if {$i > 1000} {
	    puts stderr "Bug? More than 1000 timeout flags in use"
	    exit 1
	}
    }
    set ::vccTimeoutFlag$i 0
    after $ms "set ::vccTimeoutFlag$i 1"
    vwait ::vccTimeoutFlag$i
    unset ::vccTimeoutFlag$i
}

# ___oOo___
