#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelectorEnv.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 22.08.1995 created
# S.Sandrock 19.11.1995 added validate selection
# imunoz     04.04.1996 ported to itcl2.0
#
#

#************************************************************************
#   NAME
#	VccSelectorEnv - class for a environment selection widget
#
#   SYNOPSIS
#	VccSelectorEnv <pathName> ?-<option> <value> ...?
#	<pathName> configure ?-<option> <value> ...?
#	<pathName> <method> ?<arg> <arg> ...?
#
#   DESCRIPTION
#	VccSelectorEnv is an environment selector widget.
#
#   INHERITANCE
#	It has the same properties as the VccSelectorAll base class, 
#	except for some specific deviations listed below.
#
#   PUBLIC OPTIONS
#	The following options are different from VccSelectorAll:
#
#	-labelText	Default: "Environment:"
#
#	-entryWidth	Default: 8
#
#	-menuFilter	A glob-style filter expression that all menu entries
#			must match. It can also be a list of filters, which 
#			are then or-combined.
#			Default: *	(all environments)
#			Examples:
#				l*	includes only LCU environments
#				w*	includes only WS environments
#
#   METHODS
#	validate {Env}	Local method to check `Env' for validity. This is
#			done by sending a PING command to the msgServer.
#
#   COMMON PROCEDURES
#
#   DEFAULT BINDINGS
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#
#   CAUTIONS
#
#   BUGS
#
#   SEE ALSO
#	VccSelectorAll(n)
#
#------------------------------------------------------------------------
#

option add *VccSelectorEnv.labelText  Environment:          widgetDefault
option add *VccSelectorEnv.entryWidth 10                    widgetDefault
option add *VccSelectorEnv.menuFilter *                     widgetDefault

class VccSelectorEnv {
    
    inherit VccSelectorAll
    
    constructor { args } {
	set infoMethod GetEnvs
	VccSelectorAll::constructor 
    } {
	debugEnter
	
	set infoMethod GetEnvs	;# preset protected VccInfo method to be used
	eval itk_initialize $args
	$this configure -menuFilter $itk_option(-menuFilter)
	debugReturn
    }
    
    private method validate { Env } {}
    protected variable validatePro msgServer	;# process to be used for validate
    protected variable validateCom PING		;# command to be used for validate
}

body VccSelectorEnv::validate { Env } {
    debugEnter
    set pro $validatePro
    set com $validateCom

    # send test-command to process:
    if {[catch {seq_msgSendCommand $Env $pro $com} cmdId]} {
	set ret "$Env validation failed:\n"
	# append CCS error, e.g. "Environment xxx not active":
	append ret [lindex [seq_errPrint] 5]	;# "..." ==> fail
    } else {
	debugPrint [seq_msgRecvReply $cmdId err last -all] ;# discard reply
	set ret ""					;# ""    ==> OK
    }
    debugReturn $ret
}

# ___oOo___
