#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelectorHost.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 22.08.1995 created
# S.Sandrock 19.11.1995 added validate selection
# imunoz     04.04.1996 port to itcl2.0
#

#************************************************************************
#   NAME
#	VccSelectorHost - class for a host selection widget
#
#   SYNOPSIS
#	VccSelectorHost <pathName> ?-<option> <value> ...?
#	<pathName> configure ?-<option> <value> ...?
#	<pathName> <method> ?<arg> <arg> ...?
#
#   DESCRIPTION
#	VccSelectorHost is a host selector widget.
#
#   INHERITANCE
#	It has the same properties as the VccSelectorAll base class, 
#	except for some specific deviations listed below.
#
#   PUBLIC OPTIONS
#	The following options are different from VccSelectorAll:
#
#	-labelText	Default: "Host:"
#
#	-entryWidth	Default: 16
#
#	-menuFilter	A glob-style filter expression that all menu entries
#			must match. It can also be a list of filters, which 
#			are then or-combined.
#			Default: *	(all hosts)
#
#   METHODS
#	validate {Host}
#		Local method to check `Host' for validity. 
#		This is done via the "ping" utility.
#		A sample OK output of ping on HP is:
#
#		    8 bytes from 134.171.12.176: icmp_seq=0. 
#		    
#		    ----te41 PING Statistics----
#		    1 packets transmitted, 1 packets received, 0% packet loss
#
#		Validation is considered as failed if:
#		- ping's exit status is not OK, as on SUN, if host not alive
#		- ping reports an exit error, such as "unknown host ..."
#		- the first character in the output is not "8"
#
#   COMMON PROCEDURES
#
#   DEFAULT BINDINGS
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#
#   CAUTIONS
#
#   BUGS
#
#   SEE ALSO
#	VccSelectorAll(n)
#
#------------------------------------------------------------------------
#
option add *VccSelectorHost.labelText  Host:                widgetDefault
option add *VccSelectorEnv.entryWidth  16                   widgetDefault
option add *VccSelectorEnv.menuFilter  *                    widgetDefault

class VccSelectorHost {  
    inherit VccSelectorAll
    
    constructor { args } { }
    method validate { Host } {}
}

body VccSelectorHost::constructor { args } {
    set infoMethod GetHosts	;# preset protected VccInfo method to be used
    eval itk_initialize $args
    $this configure -menuFilter $itk_option(-menuFilter)
}

body VccSelectorHost::validate { Host } {
    global ::vcc_n

    debugEnter
    if {[catch {exec $vcc_n(ping) $Host 8 1} msg]} {
	set ret $msg				;# host unknown
    } elseif {! [string match *alive* $msg] && [string index $msg 0] != "8"} {
	debugPrint $msg
	set ret "no answer from $Host"		;# host unreachable
    } else {
	debugPrint $msg
	set ret ""					;# ""    ==> OK
    }
    debugReturn $ret
}

# ___oOo___
