#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelectorPro.tcl,v 2.116 2005/10/21 14:15:10 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 06.09.1995 created
# S.Sandrock 19.11.1995 added validate selection
# imunoz     04.04.1996 ported to itcl2.0
#

#************************************************************************
#   NAME
#	VccSelectorPro - class for a process selection widget
#
#   SYNOPSIS
#	VccSelectorPro <pathName> ?-<option> <value> ...?
#	<pathName> configure ?-<option> <value> ...?
#	<pathName> <method> ?<arg> <arg> ...?
#
#   DESCRIPTION
#	VccSelectorPro is an process selector widget.
#
#   INHERITANCE
#	It has the same properties as the VccSelectorAll base class, 
#	except for some specific deviations listed below.
#
#   PUBLIC OPTIONS
#	The following options are different from VccSelectorAll:
#
#	-labelText	Default: "Process:"
#
#	-entryWidth	Default: 20
#
#	-menuFilter	The environment for which to query the process-list
#			Default: none
#
#   METHODS
#	validate {Pro}	Local method to check `Pro' for validity. This is
#			done by sending a PING command to the process.
#
#   COMMON PROCEDURES
#
#   DEFAULT BINDINGS
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#
#   CAUTIONS
#
#   BUGS
#
#   SEE ALSO
#	VccSelectorAll(n)
#
#------------------------------------------------------------------------
#

option add *VccSelectorPro.labelText  Process:              widgetDefault
option add *VccSelectorPro.entryWidth 16                    widgetDefault
option add *VccSelectorPro.menuFilter ""                    widgetDefault

class VccSelectorPro {
    inherit VccSelectorAll
    
    constructor { args } { }
    destructor {}
    
    method    validate {Pro} { }
    protected variable validateCom PING   ;# command to be used for validate
}

body VccSelectorPro::constructor { args } {
 
    set infoMethod GetProcsByEnv	;# preset protected VccInfo method
    eval itk_initialize $args
    $this configure -menuFilter $itk_option(-menuFilter)
}

body VccSelectorPro::validate {Pro} {
    debugEnter
    set env $itk_option(-menuFilter)
    set com $validateCom
    
    if {[catch {seq_msgSendCommand $env $Pro $com} cmdId]} {
	set ret "$env/$Pro validation failed:\n"
	# append CCS error, e.g. "Cannot get information on process":
	append ret [lindex [seq_errPrint] 5]	;# "..." ==> fail
    } else {
	debugPrint [seq_msgRecvReply $cmdId err last -all] ;# discard reply
	set ret ""					;# ""    ==> OK
    }
    debugReturn $ret
}

# ___oOo___
