#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSourceLib.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 31.08.1995 created
#

#************************************************************************
#   NAME
#	vccSourceLib,
#	vccSourceConfigFile, vccSourceRcFile, vccSourceConfigAndRc
#	- VLT Central Configuration Source File Library
# 
#   SYNOPSIS
#	vccSourceConfigFile ?<Name>?	==> <script-return-value> | ""
#	vccSourceRcFile ?<Name>?	==> <script-return-value> | ""
#	vccSourceConfigAndRc ?<ConfigName> <RcName>?
#					==> { <script1-value> <script2-value> }
#
#   DESCRIPTION
#     The functions contained herein are build on top of vccFindLib.
#     They not only locate a script file, but also immediately source it.
#     If a suitable script is found, then - before sourcing it - a line
#     indicating the script's name is output to stdout.
#
#     vccSourceConfigFile	- Locate and source a config file
#
#	This function should be used to source a config file at run time.
#	It calls `vccFindConfigFile' and sources it when found.
#	The return value of the sourced script is returned,
#	or an empty string if none was found.
#
#
#     vccSourceRcFile		- Locate and source a user-config file
#
#	This function should be used to source a user-config file at run time.
#	It calls `vccFindRcFile' and sources it when found.
#	The return value of the sourced script is returned,
#	or an empty string if none was found.
#
#
#     vccSourceConfigAndRc 	- Locate and source both config files
#
#	This function should be used to source both config files at run time.
#	It first calls `vccFindConfigFile' and sources it when found.
#	Then it calls `vccSourceRcFile' and sources it when found.
#	The return values of both sourced scripts are returned as a list,
#	or an empty string (for either script) if none was found.
#
#   RETURN VALUES
#	As stated in DESCRIPTION.
#
#   FILES
#	See vccFindLib.
#
#   ENVIRONMENT
#	See vccFindLib.
#
#   CAUTIONS
#	All scripts are sourced at global level.
#
#   EXAMPLES
#	The usual case for an application is that it sources its configuration
#	scripts before entering the event loop. This can be achived by only
#	one statement placed at the end of the application script:
#
#		vccSourceConfigAndRc
#
#	This will source automatically the configuration scripts whose names
#	are derived from the program name (argv0).
#
#   SEE ALSO
#	vccFindLib(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc vccSourceLib { } {
    # dummy proc to force auto-loading
}


#
#    vccSourceConfigFile ?<Name>?	==> <script-return-value> | ""
#
proc vccSourceConfigFile { {Name ""} } {
    global argv0

    if {[set absPath [vccFindConfigFile $Name]] != ""} {
	puts "[file tail $argv0]: source \"$absPath\""
	return [uplevel #0 source $absPath]
    }
    return ""					;# nothing found
}

#
#    vccSourceRcFile ?<Name>?	==> <script-return-value> | ""
#
proc vccSourceRcFile { {Name ""} } {
    global argv0

    if {[set absPath [vccFindRcFile $Name]] != ""} {
	puts "[file tail $argv0]: source \"$absPath\""
	return [uplevel #0 source $absPath]
    }
    return ""					;# nothing found
}

#
#    vccSourceConfigAndRc ?<ConfigName> <RcName>?
#				==> { <script1-value> <script2-value> }
#
proc vccSourceConfigAndRc { {ConfigName ""} {RcName ""} } {
    lappend retVal [vccSourceConfigFile $ConfigName]
    lappend retVal [vccSourceRcFile $RcName]
    return $retVal
}


# ___oOo___
