#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccStdLib.tcl,v 2.116 2005/10/21 14:15:10 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 09.09.1995 created
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
#

#************************************************************************
#   NAME
#	vccStdLib,
#	vccStdMain, vccStdToplevel
#	- VLT Central Configuration Standardized Windows Library
# 
#   SYNOPSIS
#	vccStdMain ?<CloseHandler>?
#	vccStdToplevel <Window> ?<Title>?
#
#   DESCRIPTION
#
#     vccStdMain - Configure a main panel in a standard way
#
#	This function should be used to perform a standard panel config
#	procedure. It should be called before entering the event-loop.
#	It sets up the following window-manager attributes:
#
#	title      - derived from existing title, prefixed with "prog@host"
#	iconname   - set to "prog@host"
#	iconbitmap - loaded from file with vccFindBitmapFile(n)
#	client     - set to "host", to tell the WM where the prog is running
#	command    - set to "$argv0 $argv", to tell the WM how to start it
#
#	Then it calls the routine vccSourceConfigAndRc to execute the 
#	config and rc files, if present.
#
#	If `CloseHandler' is given, then it is executed when the window is 
#	closed from the window-manager.
#
#     vccStdToplevel - Configure a toplevel sub-window in a standard way
#
#	This function should be used to perform a standard toplevel  
#	sub-window config procedure. It sets up the window-manager 
#	attributes derived from those of the main window.
#	If `Title' is not given, then the window will be marked as transient
#	and placed (roughly) on top of the main window. The actual title
#	is derived from the main window and `Window' then.
#
#   RETURN VALUES
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   EXAMPLES
#
#   CREDITS
#
#   SEE ALSO
#	vcc(n), vccWinLib(n), vccFindBitmapFile(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

vcc	;# make sure to source the global variable definitions before


proc vccStdLib { } {
    # dummy proc to force auto-loading
}


proc vccStdMain { {CloseHandler ""} } {
    global argv0 argv

    set prog [file tail $argv0]		;# default program name
    set host {};			;# hostname may not be available
    if {[info exists env(HOSTNAME)]} {
	set host $env(HOSTNAME)
    } elseif {[info exists env(HOST)]} {
	set host $env(HOST)
    }
    set bitmapfile [vccFindBitmapFile]	;# file for bitmap
    #set pixmapfile [vccFindPixmapFile]	;# file for pixmap

    if {$host == ""} {
	set name ${prog}		;# the program-name shall be the name
    } else {
	set name ${prog}@${host}	;# add hostname if available
    }

    # replace any leading "$prog:" or trailing "@$host" with proper strings:
    regsub " *-? *@${host} *\$" [wm title .] "" title
    if {$title == "" || $title == $prog} {
	set title $name
    } else {
	set title "$name: $title"
    }

    wm title      . $title		;# title of normal window
    wm iconname   . $name		;# title of icon

    if {$bitmapfile != ""} {
	wm iconbitmap . @$bitmapfile	;# b/w pictogram to be used
    }

    # (not working OK)
    #if {$pixmapfile != ""} {
	#toplevel .icon
	#pack [label .icon.i -image [image create pixmap -file $pixmapfile]]
	#wm iconwindow . .icon		;# color pictogram to be used
	#wm withdraw . ; wm deiconify .	;# (otherwise the icon is not taken!)
    #}

    wm client     . ${host}		;# machine on which program is running
    wm command    . "$argv0 $argv"	;# how to start this program

    if {$CloseHandler != ""} {
	wm protocol . WM_DELETE_WINDOW $CloseHandler	;# for clean shutdown
    }

    vccSourceConfigAndRc		;# source config files
}


proc vccStdToplevel { Window {Title ""} } {
    if {$Title == ""} {
	wm transient $Window .		;# this should be a temporary window
	scan [wm geometry .] %dx%d+%d+%d w h x y ;# place it over main window
	if {$x > 900} {incr x -400} ;# ... but keep away from right margin
	wm geometry $Window +[expr {$x+64}]+[expr {$y+64}] ;# with an offset
	catch {wm title  $Window "[wm title .]: [string range $Window 1 end]"}
    } else {
	catch {wm title  $Window "[wm title .]: $Title"}
    }
    wm iconname   $Window $Title
    wm iconbitmap $Window [wm iconbitmap .]
    #wm iconwindow $Window [wm iconwindow .]
}


# ___oOo___
