#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccWinLib.tcl,v 2.116 2005/10/21 14:15:10 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 26.09.1995 created
# imunoz     03.04.1996 port to itcl2.0
# mcomin     06.03.1997 blt_busy replaced with ::blt::busy
#

#************************************************************************
#   NAME
#	vccWinLib,
#	vccWinBusy
#	- VLT Central Configuration Window Library
# 
#   SYNOPSIS
#	vccWinBusy ?<Window>? ?<Message>? <Script>
#
#   DESCRIPTION
#
#     vccWinBusy - execute a script while showing busy cursor
#
#	This function displays the busy-cursor for `Window' while `Script'
#	is executed. Any error occuring in Script is propagated to the caller.
#	If `Message' is given then it is displayed in a pop-up busy-window.
#	If `Window' is not given then the main window "." is assumed.
#
#   RETURN VALUES
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   EXAMPLES
#	::vccWinBusy $this "Completions: $completions" {sleep 2}
#	vccWinBusy "Scanning ..." {xxxScan}
#
#   CREDITS
#	Part of vccWinBusy is taken from A.Brighton's FrameWidget class.
#
#   SEE ALSO
#	vcc(n), vccStdLib(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

vcc	;# make sure to source the global variable definitions before


proc vccWinLib { } {
    # dummy proc to force auto-loading
}


proc vccWinBusy { Window {Message ""} {Script ""} } {
    ::global errorInfo errorCode
    ::global vccWinBusy_v ;# global array with # of invocations for each Window

    # rearrange arguments:
    if {$Script == "" && $Message == ""} {
	set Script $Window
	set Message ""
	set Window .
    } elseif {$Script == ""} {
	set Script $Message
	set Message $Window
	set Window .
    }

    # internally used variables:
    set busyWindow .busy	;# widget name of toplevel busy window
    if {![info exists vccWinBusy_v($Window)]} {
	set vccWinBusy_v($Window) 0	;# count up each invocation for Window
    }

    if {[incr vccWinBusy_v($Window)] == 1} {
	# display busy cursor:
	catch {focus $Window}
	::blt::busy hold $Window
	update idletasks
	# pop-up busy window:
	if {$Message != ""} {
	    set W .busy
	    if {![winfo exists $busyWindow]} {
		vccStdToplevel [toplevel $busyWindow]
		set font [option get . font Font] ;# use default font
		if {[lempty $font]} {set font fixed}
		pack [message $busyWindow.msg -font $font -aspect 800] \
			-padx 1c -pady 1c
	    }
	    $busyWindow.msg configure -text $Message
	    wm deiconify $busyWindow
	    update
	}
    }

    # save any errors and report them later
    if {[set code [catch [list uplevel $Script] msg]]} {
	set info $errorInfo
    } 

    if {[incr vccWinBusy_v($Window) -1] == 0} {
	# remove busy cursor:
	::blt::busy release $Window
	catch {focus -lastfor $Window}
	# remove busy window:
	if {[winfo exists $busyWindow]} {
	    wm withdraw $busyWindow
	}
    }

    if {$code} {
	uplevel [list error $msg $info $code]
    }
}


# the next proc is PRELIMINARY!
proc vccWinWait { Message } {
    global vccWinWait

    set W .wait
    if {$Message == ""} {
	catch {destroy $W}
	unset vccWinWait(act)
	return
    }

    if {[info exists vccWinWait(act)]} {
	if {$Message == "."} {
	    append vccWinWait(ppp) .
	    update idletasks
	    after 100 vccWinWait "."
	} else {
	    set vccWinWait(msg) $Message
	    set vccWinWait(ppp) ""
	}
    } else {
	if {$Message != "."} {
	    vccStdToplevel [toplevel $W]
	    wm geometry $W +100+100
	    set vccWinWait(msg) $Message
	    set vccWinWait(ppp) ""
	    pack [label $W.msg -textvariable vccWinWait(msg)]
	    pack [label $W.ppp -textvariable vccWinWait(ppp) -width 20]
	    set vccWinWait(act) 1
	    vccWinWait "."
	}
    }
}

# ___oOo___
