#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccmakeLib.tcl,v 2.12 2005/10/14 14:49:32 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 17.05.1995 created
# S.Sandrock 22.12.1995 extended
# fcarbogn   23.09.1995 vccmakeBackup: backup files saved under /tmp/vccmake
#

#************************************************************************
#   NAME
#	vccmakeLib - library procs for vccMake programs
#
#   SYNOPSIS
#	vccmakeParseOptions				==> options
#	vccmakeEcho		String
#	vccmakeBackup		Host File		==> backupTarget
#	vccmakeBegin		Default			==> target
#	vccmakePuts		String
#	vccmakeEnd
# 
#   DESCRIPTION
#	For internal used by vccmake programs.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

vcc ;# define global vars

#
# Constants:
#
set vccmake_c(PROGRAM)	[file tail $argv0]
set vccmake_c(VERSION)	{$Revision: 2.12 $}
set vccmake_c(USAGE)	{[-du(mmy)] [-i(nteractive)] [-log <dir>] [-no(backup)] [-pu(rge)] [-q(uiet)] [-v(erbose)] <host> [<path>]}
set vccmake_c(BEGIN)	#vccmake::BEGIN
set vccmake_c(END)	#vccmake::END
set vccmake_c(HEADER) "#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# E.S.O. - VLT project
#
# \"@(#) \$Id\$\"
#
# THIS IS PARTLY A GENERATED FILE
# DO NOT EDIT the part between the vccmake BEGIN and END markers!
# Manual changes may only be added before or after these markers.
#	
#	!!!!!!!!!!!! >>>>>  WARNING  <<<< !!!!!!!!!!!!!!
# WE ARE CURRENTLY INTRODUCING THE VCCMAKE UTILITY ABLE TO UPGRADE 
# AUTOMATICLY THIS FILE FROM ACC DATABASE,  PLEASE REPORT TO VLTMGR ANY STRANGE
# BEHAVIOUR THAT CAN BE RELATED TO THE CONFIGURATION SETUP
#
# This part created by
#   User: [exec whoami]
#   Host: [exec hostname]
#   Date: [exec date]
#   Tool: $vccmake_c(PROGRAM) $vccmake_c(VERSION)
#"
set vccmake_c(FOOTER)  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#
# command line options:
#
set vccmake_o(dummy)	0	;# target files are not written
set vccmake_o(interactive) 0	;# ask for confirmation before overwriting
set vccmake_o(log) $vcc_n(tmpDir) ;# directory for log output files
set vccmake_o(nobackup)	0	;# original file will not be backuped
set vccmake_o(purge)	0	;# purge old backup files
set vccmake_o(quiet)	0	;# inhibits output to stdout
set vccmake_o(verbose)	0	;# outputs more information than normally

#
# Variables:
#
set vccmake_v(host)	{}	;# target host name
set vccmake_v(file)	{}	;# target file name (as seen on target host)
set vccmake_v(fd)	{}	;# opened file descriptor
set vccmake_v(prolog)	{}	;# part before vccmake-BEGIN marker
set vccmake_v(trailer)	{}	;# part behind vccmake-END marker
set vccmake_v(length)	0	;# number of written bytes


proc vccmakeParseOptions { } {
    global argv vccmake_c vccmake_o

    set options {}
    while {![lempty $argv]} {
	set arg [lvarpop argv]
	switch -glob -- $arg {
	    -du* {set x dummy;       set vccmake_o($x) 1; lappend options -$x}
	    -i*  {set x interactive; set vccmake_o($x) 1; lappend options -$x}
	    -log {set x log;         set vccmake_o($x) [lvarpop argv];\
		    lappend options -$x $vccmake_o($x)}
	    -no* {set x nobackup;    set vccmake_o($x) 1; lappend options -$x}
	    -pu* {set x purge;       set vccmake_o($x) 1; lappend options -$x}
	    -q*  {set x quiet;       set vccmake_o($x) 1; lappend options -$x;\
		    dup [open /dev/null w] stdout}
	    -v*  {set x verbose;     set vccmake_o($x) 1; lappend options -$x}
	    default {
		if {[cindex $arg 0] == "-"} {
		    puts stderr "error: illegal option: $arg"
		    puts stderr "usage: $vccmake_c(USAGE)"
		    exit 1
		}
		set argv [linsert $argv 0 $arg]
		break
	    }
	}
    }
    vccmakeEcho "options: $options"
    return $options
}


proc vccmakeEcho { String } {
    global vccmake_c
    puts stdout "$vccmake_c(PROGRAM): $String"
}


proc vccmakeBackup { Host File } {
    global vccmake_o

    # create backup directory if not existing
    if {![file exists /tmp/vccmake]} {
	catch {exec mkdir /tmp/vccmake}
    }
    # find file name using basename
    catch {set Filename [exec basename ${File}] }

    # backups are marked with the suffix "~<number>", number is incremented

    # find old backups and purge them if requested:
    set maxNumber 0	;# number of latest backup
    set oldBackups {}	;# list of backups
    catch {set oldBackups [split [vccRemExec $Host "ls /tmp/vccmake/${Filename}~?*"] \n]}
    if {$vccmake_o(purge) && ! [lempty $oldBackups]} {
	vccmakeEcho "purge old backups of: \"${Host}:${File}\""
	vccRemExec $Host "rm -f $oldBackups"
    }

    # determine the highest existing number:
    foreach f $oldBackups {
	set number [lindex [split $f ~] 1]
	if {$number >= 0} {
	    set maxNumber [max $maxNumber $number]
	}
    }

    # copy file to the next higher backup name:
    set backupname /tmp/vccmake/${Filename}~[incr maxNumber]
    vccmakeEcho "make backup: \"${Host}:${backupname}\""
    if {! $vccmake_o(dummy)} {
	vccRemExec $Host "cp -p $File $backupname" ;# yields error on failure
    }

    return ${Host}:${backupname}
}


proc vccmakeBegin { Default } {
    global argc argv0 argv vccmake_c vccmake_o vccmake_v

    # check parameters:
    vccmakeParseOptions
    set Host [lindex $argv 0]
    set File [lindex $argv 1]

    if {[file isfile $File]} {
	# if only filename is given then append directory from default:
	set File [file dirname $Default]/$File
    } elseif {[file isdirectory $File]} {
	# if only directory is given then append filename from default:
	append File /[file tail $Default]
    } else {
	# if nothing is given then use complete default:
	set File $Default
    }

    # check for wrong arguments:
    if {$argc < 1 || $Host == "" || $File == ""} {
	puts stderr "usage: $vccmake_c(USAGE)"
	exit 1
    }

    # get expanded physical path:
    set path [vccRemPath $Host [file dirname $File]]
    if {$path == ""} {
	puts stderr "error: \"[file dirname $File]\" does not exists on $Host"
	exit 1
    }
    set File ${path}/[file tail $File]

    # set global variables:
    set vccmake_v(host) $Host
    set vccmake_v(file) $File

    # initialize VccInfo engine:
    vccInit
    vccInfo configure -queryModes {fromaccdb fromvccdb} ;# NO file-based query

    # read remote file:
    if {[catch {set fd [vccRemOpen $Host $File]} msg]} {
	set contents {}
	vccmakeEcho "warning: \"${Host}:${File}\" unreadable or not existing!"
	if {$vccmake_o(verbose)} {vccmakeEcho $msg}
    } else {
	if {! $vccmake_o(nobackup)} {
	    vccmakeBackup $Host $File ;# copy to backup, unless inhibited
	}
	set contents [read $fd]
	vccRemClose $fd
	vccmakeEcho "read \"${Host}:${File}\" ([clength $contents] bytes)"
    }

    # find BEGIN and END markers, extract prolog and trailer:
    if {[set pos1 [string first $vccmake_c(BEGIN) $contents]] >= 0} {
	set vccmake_v(prolog) [csubstr $contents 0 $pos1]  ;# part before BEGIN
    } else {
	vccmakeEcho "warning: no vccmake-BEGIN marker found!"
	set vccmake_v(prolog) $contents
    }
    if {[set pos2 [string last  $vccmake_c(END)   $contents]] >= 0} {
	incr pos2 [clength $vccmake_c(END)] ;# move behind END
	set vccmake_v(trailer) [crange $contents $pos2 end] ;# part behind END
    } else {
	vccmakeEcho "warning: no vccmake-END marker found!"
	set vccmake_v(trailer) {}
    }

    # open remote file for writing, output prolog and header:
    set openOption {}
    if {$vccmake_o(interactive)} {set openOption -dialog}
    set vccmake_v(length) 0
    vccmakeEcho "preparing \"${Host}:${File}\""
    if {! $vccmake_o(dummy)} {
	set vccmake_v(fd) [vccRemOpen $Host $File w $openOption]
    }
    vccmakePuts $vccmake_v(prolog)
    vccmakePuts $vccmake_c(BEGIN)\n
    vccmakePuts $vccmake_c(HEADER)\n

    return ${Host}:${File}
}

proc vccmakePuts { String } {
    global vccmake_v
    if {[lempty $vccmake_v(fd)]} return
    puts -nonewline $vccmake_v(fd) $String
    incr vccmake_v(length) [clength $String]
}

proc vccmakeEnd { } {
    global vccmake_c vccmake_v

    # write trailer and close remote file:
    vccmakePuts $vccmake_c(FOOTER)\n
    vccmakePuts $vccmake_c(END)
    vccmakePuts $vccmake_v(trailer)
    if {[lempty $vccmake_v(fd)]} {
	vccmakeEcho "file not written."
    } else {
	vccRemClose $vccmake_v(fd)
	vccmakeEcho "file completed:\
		$vccmake_v(length) bytes\
		(prolog=[clength $vccmake_v(prolog)],\
		trailer=[clength $vccmake_v(trailer)])"
    }
    vccExit
}

# ___oOo___
