#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: wesMain.tcl,v 4.5 2004/01/16 15:25:13 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  05/11/96  created
# ccumani  02/03/98  bitmap files search corrected
# abalestr 27/07/00  added "eval" command to some widget creation so to let 
#                    tcl/tk to interpret correctly variables as lists of items
#                    and not as single items i.e. "pippo pluto" is 2 items and
#                    not one.
# abalestr 16/01/04  modified panedwindow to ipanedwindow  - see SPR no. 
#                    VLTSW20040021
#************************************************************************
#   NAME
#    wesMain  - class for WES Editor widget
#   SYNOPSIS
#    wesMain <widget_name>     
#   DESCRIPTION
#    wesMain defines the class for the WES Editor widget. The Editor 
#    comprises a textual input panel and a graphical input panel
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   WIDGET OPTIONS 
#
#   EXAMPLES
#    create an instance
#    wesMain .wesMain 
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#     

itk::usual wesMain {}
class wesMain {
    inherit itk::Widget ClassWESCommand
    constructor   {args} {}
    destructor           {}   
    itk_option define -toplevel toplevel Toplevel "."

    private variable  viewMODE "2";# 1 : GRAPHIC, 2 : TEXT
    
    private  variable  wesIniFile       "~/.wesFieraCfg"
    private  variable  waveData         ""
    private  variable  displData        ""
    private  variable  displMode        "0"
    private  variable  dbgMODE          "1"
    private  variable  versionStr       "WES \$Revision: 4.5 $"
    private  method    defbindings     {}  {}
    public   method    zoomInHoriz     {}  {} 
    public   method    zoomOutHoriz    {}  {}
    public   method    zoomInVert      {}  {} 
    public   method    zoomOutVert     {}  {}
    public   method    viewGraphic     {}  {}
    public   method    viewText        {}  {}
    public   method    viewDual        {}  {}
    public   method    viewDiag        {}  {}
    public   method    createNew       {cancel}  {} 
    public   method    onNew           {}  {}
    public   method    save            {{obj ""}}  {}
    public   method    saveAs          {{obj ""}}  {}
    public   method    onFileSelect    {{flag 0}}  {}
    public   method    onDirSelect     {} {}
    public   method    print           {}     {}
    public   method    progressInit    {} {}
    public   method    progressStep    {s} {}
    public   method    progressRemove  {} {}
    public   method    onExit          {flag} {}
    public   method    about           {}     {}
    public   method    getHelp         {{subj ""}} {}
    public   method    markDirty       {} {}
    public   method    checkForSave    {{obj ""}} {}
    public   method    showProgress    {varname index op} {}
    public   method    adjustWL        {} {}
    public   method    insertTicks     {} {}
    public   method    preferences     {} {}
    public   method    insertProfile   {type} {}
    public   method    selPatList      {}
    public   method    loadIniFile     {}
    public   method    saveIniFile     {}
    public   method    dbg             {m}
    public   method    undo            {} {}
    public   method    redo            {} {}
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::constructor {args} {
    global ::WES

    set waveData [ClassWESWaveIF #auto]
    set displData [ClassWESDisplData #auto]

    #create the sub-frames that make the mainframe 
    itk_component add menuFrame {
	frame $itk_interior.mf -relief raised
    } {
	ignore -background
    }
    itk_component add topFrame1 {
	frame $itk_interior.tf1 -relief flat
    } {
	usual
	keep  -width 
	ignore -background
    }
    itk_component add topFrame2 {
	frame $itk_interior.tf2 -relief flat
    } {
	usual
	keep   -width
	ignore -background
    }  

    #the centerframe stores the text & graphic display
    itk_component add pw {
	ipanedwindow $itk_interior.pw
    } {
	ignore -background
    }
    itk_component add bottomFrame {
	frame $itk_interior.bf1 -relief flat
    } {
	usual
	keep   -width
	ignore -background
    }
    
    $itk_component(pw) add top
    $itk_component(pw) add bottom

    set bw [$itk_component(pw) childsite bottom]
    itk_component add wesDbg {
	scrolledtext  $bw.wesDbg\
		-labelpos nw -labeltext "Diagnostic Messages"\
		-width 960 -height 100\
		-background Grey75\
		-foreground Black\
		-vscrollmode static -hscrollmode static \
		-textfont "-Adobe-helvetica-medium-r-normal--12*" \
		-wrap none
    }

    set tw [$itk_component(pw) childsite top]

    itk_component add wesEditor {
	ClassWESEditor $tw.wesEditor -waveData [code $waveData] \
		-displData [code $displData] \
		-debug $itk_component(wesDbg)
    }

    itk_component add statusBar {
	label $itk_component(bottomFrame).sbar -textvariable ::WES(status) \
		-relief raised -bd 3 -anchor nw
    }
    #the current file entry display box
    itk_component add pSelBox {
	ClassWESFileSel $itk_interior.psb
    }
    set fnewIMG [image create photo -file [wesFindVLT bitmaps/filenew.gif]]
    itk_component add fileNew {
	button $itk_component(topFrame1).fileNew \
		-image $fnewIMG \
		-padx 5 -command [code $this onNew]
    } {
	ignore -background
    }
    set fopenIMG [image create photo -file [wesFindVLT bitmaps/fopen.gif]]
    itk_component add pathSel {
	button $itk_component(topFrame1).pathSel \
		-image $fopenIMG \
		-padx 5 -command [code $this onFileSelect]
    } {
	ignore -background
    }
    set printIMG [image create photo -file "[wesFindVLT bitmaps/print.gif]"]
    itk_component add prBut {
	button $itk_component(topFrame1).prBut \
		-image $printIMG \
		-padx 5 -command [code $this print]
    } {
	ignore -background
    }
    set fsaveIMG [image create photo -file "[wesFindVLT bitmaps/fsave.gif]"]
    itk_component add saveBut {
	button $itk_component(topFrame1).saveBut \
		-image $fsaveIMG \
		-padx 5 -command [code $this save]
    } {
	ignore -background
    }
    set fsaveasIMG [image create photo -file "[wesFindVLT bitmaps/fsaveas.gif]"]
    itk_component add saveAsBut {
	button $itk_component(topFrame1).saveAsBut \
		-image $fsaveasIMG \
		-padx 5 -command [code $this saveAs]
    } {
	ignore -background
    }
    set exitIMG [image create photo -file "[wesFindVLT bitmaps/fexit.gif]"]
    itk_component add exBut {
	button $itk_component(topFrame1).exBut \
		-image $exitIMG \
		-padx 5 -command [code $this onExit 1]
    } {
	ignore -background
    }
    set helpIMG [image create photo -file "[wesFindVLT bitmaps/help.gif]"]
    itk_component add helpBut {
	button $itk_component(topFrame1).helpBut \
		-image $helpIMG \
		-padx 5 -command [code $this getHelp]
    } {
	ignore -background
    }
    set zvimIMG [image create photo -file "[wesFindVLT bitmaps/zoomin.gif]"]
    itk_component add zvPBut {
	button $itk_component(topFrame1).zvPBut \
		-image $zvimIMG \
		-padx 5 -command \
		[code [$itk_component(wesEditor) component GRAPHIC] zoomVert 1]
    } {
	ignore -background
    }
    set zvoutIMG [image create photo -file "[wesFindVLT bitmaps/zoomout.gif]"]
    itk_component add zvMBut {
	button $itk_component(topFrame1).zvMBut \
		-image $zvoutIMG \
		-padx 5 -command \
		[code [$itk_component(wesEditor) component GRAPHIC] zoomVert -1]
    } {
	ignore -background
    }
    set zhinIMG [image create photo -file "[wesFindVLT bitmaps/zoomin.gif]"]
    itk_component add zhPBut {
	button $itk_component(topFrame1).zhPBut \
		-image $zhinIMG \
		-padx 5 -command \
		[code [$itk_component(wesEditor) component GRAPHIC] zoomHoriz 1]
    } {
	ignore -background
    }
    set zhoutIMG [image create photo -file "[wesFindVLT bitmaps/zoomout.gif]"]
    itk_component add zhMBut {
	button $itk_component(topFrame1).zhMBut \
		-image $zhoutIMG \
		-padx 5 -command \
		[code [$itk_component(wesEditor) component GRAPHIC] zoomHoriz -1]
    } {
	ignore -background
    }
    set infoIMG [image create photo -file "[wesFindVLT bitmaps/info.gif]"]
    itk_component add infoBut {
	button $itk_component(topFrame1).infoBut \
		-image $infoIMG \
		-padx 5 -command {}
    } {
	ignore -background
    }
 
    itk_component add mb1 {
	menubutton $itk_component(menuFrame).mb1 -text "File"\
		-underline 0
    } {
	ignore -background
	ignore -foreground
    }
    itk_component add mb2 {
	menubutton $itk_component(menuFrame).mb2 -text "Edit"\
		-underline 0
    } {
	ignore -background
	ignore -foreground
    }
    itk_component add mb3 {
	menubutton $itk_component(menuFrame).mb3 -text "Option"\
		-underline 0
    } {
	ignore -background
	ignore -foreground
    }
    itk_component add mb4 {
	menubutton $itk_component(menuFrame).mb4 -text "View"\
		-underline 0
    } {
	ignore -background
	ignore -foreground
    }
    itk_component add mb5 {
	menubutton $itk_component(menuFrame).mb5 -text "Help"\
		-underline 0
    } {
	ignore -background
	ignore -foreground
    }
    $itk_component(mb1) config -menu $itk_component(mb1).menu
    $itk_component(mb2) config -menu $itk_component(mb2).menu
    $itk_component(mb3) config -menu $itk_component(mb3).menu
    $itk_component(mb4) config -menu $itk_component(mb4).menu
    $itk_component(mb5) config -menu $itk_component(mb5).menu
    set menu1 [menu $itk_component(mb1).menu -tearoff 0]	
    set menu2 [menu $itk_component(mb2).menu -tearoff 0]
    set menu2Sub1 [menu $itk_component(mb2).menu.sub1 -tearoff 0]
    set menu3 [menu $itk_component(mb3).menu -tearoff 0]
    set menu4 [menu $itk_component(mb4).menu -tearoff 0]
    set menu5 [menu $itk_component(mb5).menu -tearoff 0]
    $menu1 add command -label "New"\
	    -command [code $this onNew]
    $menu1 add command -label "Load"\
	    -command [code $this onFileSelect 0]
    $menu1 add command -label "Save"\
	    -command [code $this save]
    $menu1 add command -label "Save As"\
	    -command [code $this saveAs]
    $menu1 add separator
    $menu1 add command -label "Import FIERA database"\
	    -command [code $this onFileSelect 1]
    $menu1 add separator
    $menu1 add command -label "Print"\
	    -command [code $this print]
    $menu1 add separator
    $menu1 add command -label "Exit"\
	    -command [code $this onExit {1}]
    $menu2 add command -label "Undo"\
	    -command [code $this undo]
    $menu2 add command -label "Redo"\
	    -command [code $this redo]
    $menu2 add separator
    $menu2 add command -label "Select"\
	    -command {}
    $menu2 add separator
    $menu2 add command -label "Adjust Waveform Length"\
	    -command [code $this adjustWL]
    $menu2 add command -label "Insert Ticks" \
	    -command [code $this insertTicks]
    $menu2 add separator
    $menu2 add command -label "Properties"\
	    -command {}
    $menu2 add cascade -label "Multilevel" -menu $menu2Sub1
    $menu2Sub1 add command -label "Exponential 1"\
	    -command [code $this insertProfile exp1]
    $menu2Sub1 add command -label "Exponential 2"\
	    -command [code $this insertProfile exp2]
    $menu2Sub1 add command -label "Ramp1"\
	    -command [code $this insertProfile ramp1]
    $menu2Sub1 add command -label "Ramp2"\
	    -command [code $this insertProfile ramp2]
    $menu3 add command -label "Preferences"\
	    -command [code $this preferences]

    set WES(grMODE) 0
    set WES(textMODE) 1
    set WES(dbgMODE) 1

    $menu4 add check -label "Graphic" -variable WES(grMODE)\
	    -command [code $this viewGraphic] \
	    -selectcolor red \
	    -indicatoron 1
    $menu4 add check -label "Text" -variable WES(textMODE)\
	    -command [code $this viewText] \
	    -selectcolor red \
	    -indicatoron 1
    $menu4 add check -label "Diagnostic Messages" -variable WES(dbgMODE)\
	    -command [code $this viewDiag] \
	    -selectcolor red \
	    -indicatoron 1
    $menu4 add command -label "Refresh"\
	    -command [code "$itk_component(wesEditor) display"]
    $menu5 add command -label "Help on WES"\
	    -command [code $this getHelp]
    $menu5 add command -label "About"\
	    -command [code $this about]

    itk_component add newDlg {
	dialog $itk_interior.newDlg -title "Create New Waveform" -modality application
    } 
    $itk_component(newDlg) hide Help 
    $itk_component(newDlg) hide Apply
   
    set cs [$itk_component(newDlg) childsite]
    
    tabnotebook $cs.tn -tabpos n -angle 15 -width 400 -height 200
    $itk_component(newDlg) buttonconfigure OK -text Create \
	    -command [code $this createNew 0]
    $itk_component(newDlg) buttonconfigure Cancel -text Cancel \
	    -command [code $this createNew 1]
    pack $cs.tn -side top -fill both -expand yes -padx 10 -pady 10
    $cs.tn add -label "Pattern"
    $cs.tn add -label "USequence"
    $cs.tn add -label "Sequence"
    set nPatFrame [frame [$cs.tn childsite "Pattern"].nPat]
    set nUSeqFrame [frame [$cs.tn childsite "USequence"].nUSeq]
    set nSeqFrame [frame [$cs.tn childsite "Sequence"].nSeq]
    entryfield $nPatFrame.name \
	    -labeltext "Pattern Name:" -textvariable ::WES(patName) 
    entryfield $nPatFrame.ticks \
	    -labeltext "Number of Ticks:" -textvariable ::WES(ticks) \
	    -validate {forcePosInt %P 1 4096}
    combobox $nPatFrame.cb -labeltext "Boards" -editable false\
	    -textvariable ::WES(board)
    entryfield $nUSeqFrame.name \
	    -labeltext "USequence Name:" -textvariable ::WES(useqName)
    set WES(patSel) {}
    scrolledlistbox $nUSeqFrame.lb -vscrollmode static \
	    -hscrollmode dynamic -selectioncommand [code $this selPatList] \
	    -selectmode multiple \
	    -scrollmargin 5 -labelpos n -labeltext "Choose Member Patterns" 
    entryfield $nSeqFrame.name \
	    -labeltext "Sequence Name:" -textvariable ::WES(seqName) 
    entryfield $nSeqFrame.px \
	    -labeltext "Pixel X:" -textvariable ::WES(seqPx) \
	    -validate {forcePosInt %P 1 10240}
    entryfield $nSeqFrame.py \
	    -labeltext "Pixel Y:" -textvariable ::WES(seqPy) \
	    -validate {forcePosInt %P 1 10240}
    entryfield $nSeqFrame.po \
	    -labeltext "Pixel Order:" -textvariable ::WES(seqPo)
::iwidgets::Labeledwidget::alignlabels $nPatFrame.name $nPatFrame.ticks $nPatFrame.cb
::iwidgets::Labeledwidget::alignlabels $nSeqFrame.name $nSeqFrame.px $nSeqFrame.py $nSeqFrame.po    
    $cs.tn select 0
    
    eval itk_initialize $args 
    if {[llength $args] != 0 } {eval configure $args}
   

    pack $nPatFrame.name -padx 5 -pady 5 -fill x -expand true
    pack $nPatFrame.ticks -padx 5 -pady 5 -fill x -expand true
    pack $nPatFrame.cb -padx 5 -pady 5 -fill x -expand true
    pack $nPatFrame -side left -fill both -expand true
    pack $nUSeqFrame.name -padx 5 -pady 5 -fill x -expand true
    pack $nUSeqFrame.lb -padx 5 -pady 5 -fill x -expand true
    pack $nUSeqFrame -side left -fill both -expand true
    pack $nSeqFrame.name -padx 5 -pady 5 -fill x -expand true
    pack $nSeqFrame.px -padx 5 -pady 5 -fill x -expand true
    pack $nSeqFrame.py -padx 5 -pady 5 -fill x -expand true
    pack $nSeqFrame.po -padx 5 -pady 5 -fill x -expand true
    pack $nSeqFrame -side left -fill both -expand true

    pack $itk_component(mb1) -side left
    pack $itk_component(mb2) -side left
    pack $itk_component(mb3) -side left
    pack $itk_component(mb4) -side left
    pack $itk_component(mb5) -side right

    pack $itk_component(menuFrame) -side top -fill x
    pack $itk_component(fileNew) -side left -padx 3
    pack $itk_component(pathSel) -side left -padx 3
    pack $itk_component(saveBut) -side left -padx 3
    pack $itk_component(saveAsBut) -side left -padx 3
    pack $itk_component(prBut) -side left -padx 3
    pack $itk_component(infoBut) -side left -padx 3
    pack $itk_component(exBut) -side left -padx 3
    pack $itk_component(helpBut) -side right -padx 3
    #pack $itk_component(path) -side left -padx 2 -fill x
    $itk_component(saveBut) configure -state disabled
    $itk_component(saveAsBut) configure -state disabled
    $itk_component(prBut) configure -state disabled
    $itk_component(infoBut) configure -state disabled
    balloonhelp $itk_component(fileNew) "create new waveform"
    balloonhelp $itk_component(pathSel) "open File"
    balloonhelp $itk_component(saveBut) "save file to disk"
    balloonhelp $itk_component(saveAsBut) "save file with new name"
    balloonhelp $itk_component(prBut) "print current waveform"
    balloonhelp $itk_component(exBut) "exit application"
    balloonhelp $itk_component(helpBut) "run help"
    balloonhelp $itk_component(zvPBut) "zoomin vertically"
    balloonhelp $itk_component(zvMBut) "zoomout vertically"
    balloonhelp $itk_component(zhPBut) "zoomin horizontally"
    balloonhelp $itk_component(zhMBut) "zoomout horizontally"
    balloonhelp $itk_component(infoBut) "show properties" 
    pack $itk_component(topFrame1) -side top -fill x -anchor w
    pack $itk_component(topFrame2) -side top -fill x -anchor w
    pack $itk_component(wesEditor) -side top -fill both -expand true
    pack $itk_component(wesDbg) -side left -fill both -expand true
    $itk_component(pw) fraction 80 20 
    #$itk_component(pw) hide bottom
    pack $itk_component(pw) -fill both -expand true
    pack $itk_component(statusBar) -fill x -anchor sw -padx 3 -pady 3
    pack $itk_component(bottomFrame) -fill x -anchor sw -padx 1 -pady 1
    set WES(gage) ready
    set WES(dirtyBuf) 0
    wm protocol $itk_option(-toplevel) WM_DELETE_WINDOW [code $this onExit 1]
    set sw [winfo screenwidth .]
    set sh [winfo screenheight .]
    set sx [expr 3*$sw/4]
    set sy [expr 2*$sh/3]
    set px [expr $sw/8]
    set py [expr $sh/6] 
    set g ""
    append g $sx x $sy + $px + $py
    wm geometry $itk_option(-toplevel) $g
    wm title $itk_option(-toplevel) "$versionStr"
    
    $waveData configure -displData [code $displData] -debug $itk_component(wesDbg)
    set WES(filter) "*"
    loadIniFile
    defbindings
    $itk_component(saveBut) configure -state normal
    $itk_component(saveAsBut) configure -state normal
    $itk_component(prBut) configure -state normal
    $itk_component(infoBut) configure -state normal
    
    set WES(baseClkF) "50"
    set WES(patName) ""
    set WES(useqName) ""
    set WES(seqName) ""
    set WES(patList) {}
    set WES(ticks) ""
    set WES(board) ""
    set WES(seqPx) 0
    set WES(seqPy) 0
    set WES(seqPo) ""
    applyBindtag $itk_option(-toplevel)

    freezeEvents
};#wesMain::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::undo
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::undo {} {
    ClassWESCommand::undo 
    $itk_component(wesEditor) undo
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::redo
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::redo {} {
    ClassWESCommand::redo
    $itk_component(wesEditor) redo
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::loadIniFile
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::loadIniFile {} {
    set parser [ClassWESParser #auto]
    if {[set rc [$parser setInputFile $wesIniFile]] != 0} {    
	if {[$parser expectT "BOARD_PATH"]} {
	    set rc 1
	    set t [$parser getToken]
	    set p $t
	    while {($t != "") && ($t != ";")} {
		if {[file isdirectory $p] == 0} {
		    set t [$parser getToken]
		    set p "$p:$t"
		} else {
		    $itk_component(wesEditor) configure -path $p
		    break
		}
	    }
	    if {$p == ""} {
		bell
		dbg "<<ERROR>> WES INI File\
			\"$wesIniFile\" is corrupt."
		dbg "                expected <path_name> but found \"\"."
	    }
	} else {
	    bell
	    dbg "<<ERROR>> WES INI File \"$wesIniFile\" is corrupt."
	    dbg "                expected \"BOARD_PATH\" but found ??."
	}
    } else {
	dbg "<<WARNING>> WES INI File \"$wesIniFile\" does not exist."
    }
    delete object $parser
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::saveIniFile
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::saveIniFile {} {
    global env
    dbg "saving WES INI File \"$wesIniFile\"..\n"
    
    set path [$itk_component(wesEditor) cget -path]
    
    if {[info exists env(HOME)]} {
        regsub ^$env(HOME)/ $path ~/ path
    }
    set parser [ClassWESParser #auto]
    if {[set rc [$parser setOutputFile $wesIniFile]] != 0} {
	
	$parser putToken "BOARD_PATH = $path;"
	$parser putToken " "
	$parser closeOutput
    } else {
	bell
	dbg "<<ERROR>> WES INI File \"$wesIniFile\"\n\
		could not be opened for writing."
    }
    dbg "done.\n"
    delete object $parser 
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::defbindings
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::defbindings {} {
    
    #bind $itk_option(-toplevel) <Alt-f> [$itk_component(mb1) invoke]
    #bind $itk_option(-toplevel) <Alt-e> [$itk_component(mb2) invoke]
    #bind $itk_option(-toplevel) <Alt-o> [$itk_component(mb3) invoke]
    #bind $itk_option(-toplevel) <Alt-v> [$itk_component(mb4) invoke]
    #bind $itk_option(-toplevel) <Alt-h> [$itk_component(mb5) invoke]
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wesMain::dbg
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::dbg {m} {
    $itk_component(wesDbg) insert end $m
    $itk_component(wesDbg) see end
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# showProgress
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::showProgress {varname index op} {
    global ::WES
    switch $op {
	w {
	    $itk_component(statusBar) delete all
	    $itk_component(statusBar) create text 10 10\
		-anchor w -text $WES(gage) -fill blue
	    after 200 [code $this progressStep ""]
	}

    }
};#wesMain::showProgress
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::progressStep
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::progressStep {s} {
    
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# preferences
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::preferences {} {
    global ::WES
    
    viewText
    update
    if {![checkForSave]} {
	return
    }
   
    $displData clear
    $waveData clearWave
    clearCommandStack
    
    $itk_component(wesEditor) configure -object ""
    wm title $itk_option(-toplevel) "$versionStr <untitled>"
    $itk_component(wesEditor) clear
    set pdlg [dialog $itk_interior.prefDlg \
		-title "WES Preferences" -modality application]
    $pdlg hide Apply
    set cs [$pdlg childsite]
    set WES(boardPath) [$itk_component(wesEditor) cget -path]
    entryfield $cs.boardCFGPath \
	    -labeltext "directory containing BOARD Data :" \
	    -textvariable ::WES(boardPath)
    entryfield $cs.baseClk \
	    -labeltext "System Clock Frequency (MHz)    :" \
	    -textvariable ::WES(baseClkF) \
	    -validate {forcePosInt %P 1 100}
    set cse [$cs.boardCFGPath childsite]
    button $cse.pathSelBut -text "browse" -command [code $this onDirSelect] 
    pack $cse.pathSelBut -padx 5 -pady 5 
    pack $cs.boardCFGPath  -padx 5 -pady 5 -side top -fill x -expand true
    pack $cs.baseClk  -padx 5 -pady 5 -side top -fill x -expand true
    $cs.boardCFGPath xview end
    $pdlg buttonconfigure OK -text Ok \
	    -command [code "$pdlg deactivate Ok"]
    $pdlg buttonconfigure Cancel -text Cancel \
	    -command [code "$pdlg deactivate Cancel"]
    $pdlg buttonconfigure Help -text Help \
	    -command [code "$pdlg deactivate Help"]
    $pdlg center
    unfreezeEvents
   
    switch [$pdlg activate] {
	Ok {
	    $pdlg deactivate
	    if {($WES(baseClkF) == 0) || ($WES(baseClkF) == "")} {
		wesInfo WES_ERROR "Invalid clock speed divider,\n\
			assuming 50Mhz as default"
		set WES(baseClkF) 50
	    } 
	    if {[file isdirectory $WES(boardPath)] == 0} {
		wesError WES-ERROR "invalid directory:\n\"$WES(boardPath)\"."
	    } else {
		$itk_component(wesEditor) configure -path $WES(boardPath) \
			-freq [expr $WES(baseClkF)*1e+6]
		$itk_component(wesEditor) loadConfig
		$itk_component(wesEditor) saveDisplayList
		saveIniFile
	    }
	}
	Cancel {
	    $pdlg deactivate
	}
	Help {
	    getHelp manipulate
	}
	
    }
    delete object $pdlg
    unfreezeEvents
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# insertProfile
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::insertProfile {type} {
    global ::WES
    set obj [$itk_component(wesEditor) cget -object]
    if {$obj == ""} {
	wesError $FAILED "No Waveform loaded"
	return 
    } elseif {($obj != "") && [$obj noModify]} {
	wesError $FAILED "This Command cannot be used with\n\
		type \"[$obj cget -type]\" objects"
	return
    }
    set ticks [$waveData getTicks]
    #set waveData [scope "::.wesApp.wd"]
    set tmp [$obj getBoard]
    set brds {}
    foreach e $tmp {
	if {[$e getMLSIMMS] != {}} {
	    lappend brds [$e cget -name]
	}
    }
    if {$brds == {}} {
	wesError $FAILED "Current Waveform does not contain \n\
		MULTI-LEVEL SIMMs"
	return
    }
    freezeEvents
    set profdlg [dialog $itk_interior.profDlg \
		-title "WES Create Profile $type" -modality application]
    $profdlg hide Apply
    set cs [$profdlg childsite]
    set WES(board) ""
    
    set WES(wd) [code $waveData]
    combobox $cs.cbSIMM -labeltext "Select SIMM" -editable false\
	    -textvariable ::WES(simm)
    set WES(cbs) "$cs.cbSIMM"
    combobox $cs.cbBRD -labeltext "Select Board" -editable false\
	    -textvariable ::WES(board) \
	    -selectioncommand {$WES(cbs) insert list end [$WES(wd) getMLSimms $WES(board)]}
    
    $cs.cbBRD insert list end $brds

    entryfield $cs.starttick \
	-labeltext "start at Tick:" \
	    -textvariable ::WES(stick) \
	    -validate "forcePosInt %P 0 $ticks"
    entryfield $cs.endtick \
	    -labeltext "end at Tick:" \
	    -textvariable ::WES(etick) \
	    -validate "forcePosInt %P 0 $ticks"
::iwidgets::Labeledwidget::alignlabels $cs.cbBRD $cs.cbSIMM $cs.starttick $cs.endtick
    pack $cs.cbBRD -padx 5 -pady 3 -side top -fill x -expand true
    pack $cs.cbSIMM -padx 5 -pady 3 -side top -fill x -expand true
    pack $cs.starttick -padx 5 -pady 3 -side top -fill x -expand true
    pack $cs.endtick -padx 5 -pady 3 -side top -fill x -expand true
    $profdlg buttonconfigure OK -text Ok \
	    -command [code "$profdlg deactivate Ok"]
    $profdlg buttonconfigure Cancel -text Cancel \
	    -command [code "$profdlg deactivate Cancel"]
    $profdlg buttonconfigure Help -text Help \
	    -command [code $this getHelp manipulate]
    $profdlg center
    switch [$profdlg activate] {
	Ok {
	    $profdlg deactivate
	    delete object $profdlg
	    if {($WES(simm) != "") && ($WES(board) != "") \
		    && ($WES(stick) < $WES(etick)) \
		    && ($WES(etick) <= $ticks)} {
		
		$waveData createProfile $WES(board) \
			$WES(simm) $WES(stick) \
			$WES(etick) 0 0 $type
		$obj markDirty 1 GRAPHIC
		$itk_component(wesEditor) display
	    } else {
		wesError $FAILED "Invalid Parameter Choice"
	    }
	    
	}
	Cancel {
	    $profdlg deactivate
	    delete object $profdlg
	}

    }
    unfreezeEvents
    set WES(simm) ""
    set WES(board) ""
    #set WES(stick) ""
    #set WES(etick) ""
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# adjustWL
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::adjustWL {} {
    global ::WES

    set obj [$itk_component(wesEditor) cget -object]
    if {$obj == ""} {
	wesError $FAILED "No Waveform loaded"
	return 
    } elseif {($obj != "") && [$obj noModify]} {
	wesError $FAILED "This Command cannot be used with\n\
		type \"[$obj cget -type]\" objects"
	return
    }
    freezeEvents
    set type [$obj cget -type]
    set ticks [$waveData getTicks]
    set WES(ticks) [expr $ticks + 1]
    set adlg [dialog $itk_interior.adjustDlg \
		-title "Adjust Waveform Length" -modality application]
    $adlg hide Apply
    set cs [$adlg childsite]
    label $cs.curVal -text "$type : [$obj cget -name]"
    
    entryfield $cs.ticks \
	    -labeltext "new value :" \
	    -textvariable ::WES(ticks)\
	    -validate "forcePosInt %P 1 4096"
    pack $cs.curVal  -padx 5 -pady 5 -side top -fill x -expand true
    pack $cs.ticks -padx 5 -pady 5 -side top -fill x -expand true

    $adlg buttonconfigure OK -text OK \
	    -command [code "$adlg deactivate Ok"]
    $adlg buttonconfigure Cancel -text Cancel \
	    -command [code "$adlg deactivate Cancel"]
    $adlg buttonconfigure Help -text Help \
	    -command [code $this getHelp manipulate]
    $adlg center
    switch [$adlg activate] {
	Ok {
	    if {$WES(ticks) > $MIN_PAT_LENGTH} {
		pushUndoStack "[code $waveData] setTick $ticks"
		set rc [$waveData resize [expr $WES(ticks)-1]]
		$obj markDirty 1 GRAPHIC
		$obj setLength "$WES(ticks) ticks"
		$itk_component(wesEditor) display
	    } else {
		wesError $SYNTAX_ERROR "<ticks> must be greater than $MIN_PAT_LENGTH"
	    }
	    
	}
    
	Cancel {
	    set WES(ticks) ""
	}

    }
    #incr WES(ticks) -1
    delete object $adlg
    unfreezeEvents
};#wesMain::adjustWL
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::insertTicks
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::insertTicks {} {
    global ::WES

    set obj [$itk_component(wesEditor) cget -object]
    if {$obj == ""} {
	wesError $FAILED "No Waveform loaded"
	return 
    } elseif {($obj != "") && [$obj noModify]} {
	wesError $FAILED "This Command cannot be used with\n\
		type \"[$obj cget -type]\" objects"
	return
    }
    freezeEvents
    set ticks [$waveData getTicks]
    set type [$obj cget -type]

    set WES(ticks) 1
    set WES(atTick) 0
    set idlg [dialog $itk_interior.insertDlg \
		-title "InsertTicks" -modality application]
    $idlg hide Apply
    set cs [$idlg childsite]
    label $cs.curVal -text "current Waveform : [$obj cget -name]"
    
    entryfield $cs.at \
	    -labeltext "insert at tick :" \
	    -textvariable ::WES(atTick)\
	    -validate "forcePosInt %P 0 $ticks"
    entryfield $cs.ticks \
	    -labeltext "number of ticks :" \
	    -textvariable ::WES(ticks)\
	    -validate "forcePosInt %P 1 4096"
::iwidgets::Labeledwidget::alignlabels $cs.at $cs.ticks 
    pack $cs.curVal  -padx 5 -pady 5 -side top -fill x -expand true
    pack $cs.at -padx 5 -pady 5 -side top -fill x -expand true
    pack $cs.ticks -padx 5 -pady 5 -side top -fill x -expand true

    $idlg buttonconfigure OK -text OK \
	    -command [code "$idlg deactivate Ok"]
    $idlg buttonconfigure Cancel -text Cancel \
	    -command [code "$idlg deactivate Cancel"]
    $idlg buttonconfigure Help -text Help \
	    -command [code $this getHelp manipulate]
    $idlg center
    switch [$idlg activate] {
	Ok {
	    if {($WES(atTick) <= $ticks) && ($WES(ticks) >= 0) \
		&& ($WES(ticks) != "") && ($WES(atTick) != "")} {
		if ![$obj insertTicks $WES(atTick) $WES(ticks)] {
		    wesError $FAILED "Cannot insert ticks \n\
			    into \"[$obj cget -type]\"\
			    [$obj cget -name]"
		} else {
		    pushUndoStack "[code $waveData] setTick $ticks"
		    set ticks [expr $ticks + $WES(ticks)]
		    $waveData setTick $ticks
		    $obj setLength "[expr $ticks + 1] ticks"
		    $obj markDirty 1 GRAPHIC
		    $itk_component(wesEditor) display
		    
		} 
	    }
	}
	default {
	   
	}
    }
    delete object $idlg
    unfreezeEvents
};#wesMain::insertTicks
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::checkForSave
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::checkForSave {{o ""}} {
    global ::WES

    Busy "Checking"
    set WES(gage) "checking.."

    set mdlg [messagedialog $itk_interior.m -bitmap questhead\
	    -title "WES Save Confirm"\
	    -text "Current Waveform not saved" \
	    -modality application]
    $mdlg show Apply
    $mdlg hide Help
    $mdlg default Cancel
    $mdlg center
    
    $mdlg buttonconfigure OK -text Save \
	    -command [code "$mdlg deactivate save"]
    $mdlg buttonconfigure Apply -text Ignore \
	    -command [code "$mdlg deactivate ignore"]
    $mdlg buttonconfigure Cancel -text Cancel \
	    -command [code "$mdlg deactivate cancel"]
    if {[set obj [$itk_component(wesEditor) cget -object]] != ""} {
	if {[$obj isDirty]} {
	    $mdlg configure -text "\"[$obj cget -name]\" not saved!"
	    switch [$mdlg activate] {
		save {
		    # save
		    save $obj
		}
		ignore {
		    # ignore
		}
		cancel {
		    # cancel
		    delete object $mdlg
		    unset WES(gage)
		    Busy end
		    return 0
		}
	    }
	}
    } 

    delete object $mdlg
    set WES(gage) ready
    Busy end
    return 1
};#wesMain::CheckForSave
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::markDirty
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::markDirty {} {
    global ::WES
    if {[info exists WES(cur)]} {
	set obj $WES(cur)
	set waveData [code "::.wesApp.wd"]
	$waveData markDirty $obj 1
	
    } else {
	$itk_component(saveBut) configure -state normal
	$itk_component(saveAsBut) configure -state normal
	$itk_component(prBut) configure -state normal
	$itk_component(infoBut) configure -state normal
	
	set WES(dirtyBuf) 1 
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::selPatList 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::selPatList {} {
    global ::WES
    set WES(patSel) [[[$itk_component(newDlg) childsite].tn \
            childsite USequence].nUSeq.lb getcurselection]
    #puts $WES(patSel)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::onNew
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::onNew {} {
    global ::WES
    viewText
    set WES(patName) ""
    set WES(useqName) ""
    set WES(seqName) ""
    set WES(patSel) {}
    set WES(ticks) [expr $MIN_PAT_LENGTH + 1]
    set WES(board) ""
    set brds [$waveData getBoardNames]
    set pats [$waveData getPatInPath]
    if {$brds == {}} {
	wesError $SYNTAX_ERROR "BOARD configuration not found !\n\
		No \"*.brd\" configuration file in Path \
		\"[$itk_component(wesEditor) cget -path]\""
    } else {
	if {![checkForSave]} {
	    return
	}
	eval [[$itk_component(newDlg) childsite].tn \
		childsite "Pattern"].nPat.cb insert list end $brds
	eval [[$itk_component(newDlg) childsite].tn \
		childsite "USequence"].nUSeq.lb insert end $pats
	$itk_component(newDlg) configure -modality application
	$itk_component(newDlg) center
	$itk_component(newDlg) activate
    }
};#wesMain::onNew
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::createNew
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::createNew {cancel} {
    global ::WES
    set obj ""
    set wantedObj ""
    set wantedType ""
    $itk_component(newDlg) deactivate
    if {$cancel == 0} {
	$displData clear
	$waveData clearWave
	clearCommandStack
	
	$itk_component(wesEditor) configure -object ""
	update
	$itk_component(wesDbg) delete 0.0 end
	set what [[$itk_component(newDlg) childsite].tn view]
	update idletasks
	switch -exact $what {
	    0 {
		if {[set WES(patName) [string trim $WES(patName)]] == ""} {
		    wesError $SYNTAX_ERROR "<pattern name> is empty." 
		    return
		}
		if {$WES(ticks) <= $MIN_PAT_LENGTH} {
		    wesError $SYNTAX_ERROR "<ticks> must be greater than $MIN_PAT_LENGTH"
		    return
		}
		set wantedType PATTERN
		set wantedObj $WES(patName)
		if {[set obj [$waveData createPattern \
			$WES(patName) $WES(board)]] != $FAILED} {
		    $waveData setTick [expr $WES(ticks) -1]
		}
	    }
	    1 { 
		if {[set WES(useqName) [string trim $WES(useqName)]] == ""} {
		    wesError $SYNTAX_ERROR "<usequence name> is empty." 
		    return
		}
		if {$WES(patSel) == {}} {
		    wesError $SYNTAX_ERROR "No member PATTERNS selected." 
		    return
		}
		set wantedType USEQUENCE
		set wantedObj $WES(useqName)
		if {[set obj [$waveData createUSeq $WES(useqName)]] != $FAILED} {
		    foreach p $WES(patSel) {
			set pName "[$itk_component(wesEditor) cget -path]/$p"
			if {[set patObj [$waveData load $pName]] != $FAILED} {
			    if ![$obj insertPattern $patObj] {
				set msg "Cannot create USequence Pattern \n\
					[$patObj cget -name] in $pName."
				dbg "<<ERROR>> $msg"
				wesError $SYNTAX_ERROR "$msg"
				return
			    }
			} else {
			    set msg "Cannot load Pattern \n\
				    in $pName."
			    dbg "<<ERROR>> $msg"
			    wesError $SYNTAX_ERROR "$msg"
			    return
			}
		    }
		}
	    }
	    2 { 
		if {[set WES(seqName) [string trim $WES(seqName)]] == ""} {
		    wesError $SYNTAX_ERROR "<sequence name> is empty." 
		    return
		}
		set wantedType SEQUENCE
		set wantedObj $WES(seqName)
		if {[set obj [$waveData createSeq  $WES(seqName) \
			$WES(seqPx) $WES(seqPy) $WES(seqPo)]] != $FAILED} {
		    
		}
	    }
	}
	if {$obj == $FAILED} {
	    wesError $SYNTAX_ERROR "Cannot create $wantedType \"$wantedObj\"\n\
		    in current Path \"[$itk_component(wesEditor) cget -path]\""
	
	} else {
	    $obj markDirty 1 TEXT
	    $itk_component(wesEditor) configure -object $obj
	    [$itk_component(wesEditor) component TEXT] display
	    wm title $itk_option(-toplevel) "$versionStr [$obj cget -name]"
	}
    } else {
	[$itk_component(wesEditor) component TEXT] display
    }
    #$itk_component(saveBut) configure -state disabled
    set WES(patName) ""
    set WES(useqName) ""
    set WES(seqName) ""
    set WES(patSel) {}
    set WES(ticks) ""
    set WES(board) ""
    set WES(seqPx) 0
    set WES(seqPy) 0
    set WES(seqPo) ""
};#wesMain::createNew
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::onFileSelect
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::onFileSelect {{flag 0}} {
    global ::WES

    if {![checkForSave]} {
	return
    }
    if {$flag == 1} {
	$itk_component(pSelBox) configure -title "Import File"
    } else {
	$itk_component(pSelBox) configure -title "Open File"
    }
    $itk_component(pSelBox) configure \
	    -directory [$itk_component(wesEditor) cget -path] -fileson 1
    $itk_component(pSelBox) center
    switch -exact [$itk_component(pSelBox) activate] {
	"OK" { 
	    set f [$itk_component(pSelBox) get]
	    if [catch {open $f r}] {
		wesError WES_ERROR "file:\"$f\" does not exist."
		
	    }
	    viewText
	    set WES(filter) [$itk_component(pSelBox) cget -mask] 
	    #puts $WES(filter)
	    $itk_component(wesDbg) delete 0.0 end
	    clearCommandStack
	    update
	    Busy "loading ..."
	    wm title  . "$versionStr <[$itk_component(wesEditor) load $f]>"
	    Busy end
	}
	"Cancel" {
	    
	}
    }
    
};#wesMain::onFileSelect

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::onDirSelect
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::onDirSelect {} {
    global ::WES
    #[[$itk_component(pSelBox) component fsb] component selection] clear
    $itk_component(pSelBox) configure -directory $WES(boardPath)
    $itk_component(pSelBox) configure -title "Select Directory" -fileson 0
    $itk_component(pSelBox) center
    switch [set rc [$itk_component(pSelBox) activate]] {
	"OK" {
	    set WES(boardPath) [$itk_component(pSelBox) get]
	}
	"Cancel" {	    
	}
    } 
    update
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::viewGraphic
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::viewGraphic {} {
    global ::WES
    if { $viewMODE == 1} {
	set WES(grMODE) 1
	return
    }
    set rc 0
    $itk_component(wesEditor) configure -mode "GRAPHIC"

    if {[$itk_component(wesEditor) cget -object] != ""} {
	set WES(grMODE) 1
	set WES(textMODE) 0
	set WES(dualMODE) 0
	set viewMODE 1
	#update the menu bar
	pack forget $itk_component(fileNew)
	pack forget $itk_component(pathSel)
	pack forget $itk_component(saveBut)
	pack forget $itk_component(saveAsBut)
	pack forget $itk_component(prBut)
	pack forget $itk_component(exBut)
	pack forget $itk_component(helpBut)
	pack forget $itk_component(infoBut)
	
	pack $itk_component(fileNew) -side left -padx 3
	pack $itk_component(pathSel) -side left -padx 3
	pack $itk_component(saveBut) -side left -padx 3
	pack $itk_component(saveAsBut) -side left -padx 3
	pack $itk_component(prBut) -side left -padx 3
	pack $itk_component(zvPBut) -side left -padx 3
	pack $itk_component(zvMBut) -side left -padx 3
	pack $itk_component(zhPBut) -side left -padx 3
	pack $itk_component(zhMBut) -side left -padx 3
	pack $itk_component(infoBut) -side left -padx 3
	pack $itk_component(exBut) -side left -padx 3
	pack $itk_component(helpBut) -side right -padx 3

    } else {
	set WES(grMODE) 0
	set WES(textMODE) 1
    }
    Busy end
    set WES(gage) ready
};#wesMain::viewGraphic
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::viewText
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::viewText {} {
    global ::WES
    if { $viewMODE == 2} {
	set WES(textMODE) 1
	return
    }
    update idletasks
    set viewMODE 2

    Busy "preparing display"

    set WES(grMODE) 0
    set WES(dualMODE) 0
    set WES(textMODE) 1
    set WES(gage) "preparing text.."

    $itk_component(wesEditor) configure -mode "TEXT"
    pack forget $itk_component(zvPBut)
    pack forget $itk_component(zvMBut)
    pack forget $itk_component(zhPBut)
    pack forget $itk_component(zhMBut)

    set WES(gage) ready
    Busy end
};#wesMain::viewText
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::viewDual {} {
    global ::WES
    #update the menu bar

    if {![info exists WES(cur)]} {
	return ""
    }
    switch $viewMODE {
	1 {
	    #Graphic Mode
	    pack forget $itk_component(wesGraphicDisplay)
	    $itk_component(wesTextDisplay) configure -width 200
	    pack $itk_component(wesGraphicDisplay) \
		    -side left -fill both -expand true -padx 5 -pady 5
	    pack $itk_component(wesTextDisplay) -side left -fill y -anchor nw
	}
	2 {
	    #Text Mode
	    pack forget $itk_component(wesTextDisplay)
	    
	    $itk_component(wesTextDisplay) configure -width 200
	    pack $itk_component(wesGraphicDisplay) \
		    -side left -fill both -expand true -padx 5 -pady 5
	    pack $itk_component(wesTextDisplay) -side left -fill y -anchor nw
	    $itk_component(wesGraphicDisplay) display 1
	}
	3 {
	    return
	}
    }
    pack forget $itk_component(fileNew)
    pack forget $itk_component(pathSel)
    pack forget $itk_component(saveBut)
    pack forget $itk_component(saveAsBut)
    pack forget $itk_component(prBut)
    pack forget $itk_component(exBut)
    pack forget $itk_component(helpBut)
    pack forget $itk_component(infoBut)
    pack forget $itk_component(zvPBut)
    pack forget $itk_component(zvMBut)
    pack forget $itk_component(zhPBut)
    pack forget $itk_component(zhMBut)
    pack $itk_component(pathSel) -side left -padx 3
    pack $itk_component(saveBut) -side left -padx 3
    pack $itk_component(saveAsBut) -side left -padx 3
    pack $itk_component(prBut) -side left -padx 3
    pack $itk_component(zvPBut) -side left -padx 3
    pack $itk_component(zvMBut) -side left -padx 3
    pack $itk_component(zhPBut) -side left -padx 3
    pack $itk_component(zhMBut) -side left -padx 3
    pack $itk_component(infoBut) -side left -padx 3
    pack $itk_component(exBut) -side left -padx 3
    pack $itk_component(helpBut) -side right -padx 3
    set viewMODE 3
    $itk_component(wesTextDisplay) tag bind text \
	    <KeyPress> [code $this markDirty]
    set WES(dualMODE) 1
    set WES(grMODE) 0
    set WES(textMODE) 0
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::viewDiag
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::viewDiag {} {
    global ::WES
    update idletasks
    
    if {$dbgMODE == 0} {
	$itk_component(pw) show bottom
	$itk_component(pw) fraction 80 20
	set dbgMODE 1
	set WES(dbgMODE) 1
    } else {
	$itk_component(pw) hide bottom
	set WES(dbgMODE) 0
	set dbgMODE 0
    }
    
};#wesMain::viewDiag

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::progressInit {} {

}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::progressRemove {} {

}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::save
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::save {{obj ""}} {
    freezeEvents
    set fname [$itk_component(wesEditor) save]
    wm title $itk_option(-toplevel) "$versionStr $fname"
    unfreezeEvents
};#wesMain::save
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::saveAs
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::saveAs {{obj ""}} {
    global ::WES
    $itk_component(pSelBox) configure -title "Save As" -fileson 1
    $itk_component(pSelBox) center
    
    switch [$itk_component(pSelBox) activate] {
	OK {
	    set f [$itk_component(pSelBox) get]
	    if {[file isfile $f]} {
		#overwrite file ??
		if {[wesConfirm OVERWRITE_WAVEFORM "overwrite \"$f\""] != "OK"} {
		    return 0
		}
	    }
	    wm title  . "$versionStr <$f>"
	    $itk_component(wesEditor) saveAs $f
	}
	Cancel {

	}
    }
    set WES(gage) ready
};#wesMain::saveAs
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::print
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::print {} {
    global ::WES
    global ::env
    
    global ::tcl_platform

    set tcl_platform(platform) "unix"

    if {![checkForSave]} {
	return
    }
    update
    freezeEvents
    set obj [$itk_component(wesEditor) cget -object]
    set file "[$itk_component(wesEditor) cget -path]/$obj.ps"
    set t ".wes_pcv"
    toplevel $t 
    wm withdraw $t
    set prcv [canvas $t.pcv] 
    pack $prcv -expand 1 -fill both
    Busy "prepare printing ..."
    [$itk_component(wesEditor) component GRAPHIC] prepPrintCanvas [code $prcv]
    Busy end
    if {[info exists env(PRINTER)]} {
        set prcmd "lpr -P$env(PRINTER)"
    } else {
        set prcmd ""
    }
    set pcd [Canvasprintdialog $itk_interior.pd -title "WES Print"\
	    -output file \
	    -printcmd $prcmd \
	    -filename $file \
	    -modality application -pagesize A4]
    $pcd buttonconfigure OK -text Print
    $pcd buttonconfigure Apply -command "$pcd refresh"

    set sz "A4"
    set or "landscape"
    set x1 [Canvasprintbox::ezPaperInfo $sz x1 $or]
    set x2 [Canvasprintbox::ezPaperInfo $sz x2 $or]
    set y1 [Canvasprintbox::ezPaperInfo $sz y1 $or]
    set y2 [Canvasprintbox::ezPaperInfo $sz y2 $or]
    set cx [Canvasprintbox::ezPaperInfo $sz centerx $or]
    set cy [Canvasprintbox::ezPaperInfo $sz centery $or]

    $pcd setcanvas $t.pcv
    $pcd center
    if {[$pcd activate]} {
	Busy "printing"
	$pcd print
    }
    Busy end
    destroy $itk_interior.pd
    destroy $t
    unfreezeEvents
};#wesMain::print
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::about
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::about {} {
    set im [image create photo -file "[wesFindVLT bitmaps/wes_logo.gif]"]
    set dlg [messagedialog .#auto -title "About WES" \
	    -image $im \
	    -imagepos n -text "Optical Detector Group\n\
	    European Southern Observatory\n1997" \
	    -modality application]
    
    $dlg hide Apply
    $dlg hide Cancel
    $dlg hide Help
    $dlg center
    $dlg activate
    delete object $dlg
};#wesMain::about
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::getHelp
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::getHelp {{subj ""}} {

    set dlg [messagedialog .#auto -title "WES" \
	    -text "For Help on the WES SW,\n please consult the Online\
	    User Manual" \
	    -modality application]
    $dlg hide Apply
    $dlg hide Cancel
    $dlg hide Help
    $dlg center
    $dlg activate
    delete object $dlg
    
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#wesMain::onExit
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body wesMain::onExit {flag} {
    set rc 0

    if {![checkForSave]} {
	return
    }
    set mdlg [messagedialog $itk_interior.m -bitmap questhead\
	    -title "WES Exit Confirm"\
	    -text "save current configuration" \
	    -modality global]
    $mdlg show Apply
    $mdlg hide Help
    $mdlg default Cancel
    $mdlg center
    $mdlg buttonconfigure OK -text Save \
	    -command [code "$mdlg deactivate save"]
    $mdlg buttonconfigure Apply -text Ignore \
	    -command [code "$mdlg deactivate ignore"]
    $mdlg buttonconfigure Cancel -text Cancel \
	    -command [code "$mdlg deactivate cancel"]

    update
    switch [$mdlg activate] {
	save {
	    # save
	    $itk_component(wesEditor) saveDisplayList
	    saveIniFile
	    
	}
	ignore {
	    # ignore
	}
	cancel {
	    # cancel
	    set rc 1
	}
    }
    if {$rc == 0} {
	exit
    } else {
	delete object $mdlg
    }
};#wesMain::onExit
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# ___oOo___


