#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESEditor.tcl,v 4.5 2004/01/16 15:25:00 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  28/04/97  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
itk::usual ClassWESEditor { }
class ClassWESEditor {
    inherit itk::Widget ClassWESError
    constructor       {args} {}
    destructor               {}
    itk_option define  -freq      freq      Freq      "50"
    itk_option define  -debug     debug     Debug     ""
    itk_option define  -displData displData DisplData ""
    itk_option define  -waveData  waveData  WaveData  ""
    itk_option define  -object    object    Object    ""
    itk_option define  -path      path      Path      "~"
    itk_option define  -mode      mode      Mode      "TEXT";#GRAPHIC
    protected  variable displayCfgFile     ".wesDisplCfg"
    
    protected  variable hwCfg              ""
    protected  variable settings
    public     method   loadDisplayList    {}
    public     method   saveDisplayList    {}
    public     method   component          {what}
    public     method   load               {f {mode 0}}
    public     method   loadConfig         {}
    public     method   save               {}
    public     method   saveAs             {f}
    public     method   clear              {}
    public     method   display            {}
    private    method   defbindings        {}
    private    method   dbg                {m}
    public     method   undo               {}
    public     method   redo               {}
};#ClassWESEditor

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::constructor {args} {
    itk_component add mfr {
	frame $itk_interior.mfr
    } {
	usual
    }
    
    itk_component add textEdit {
	ClassWESTeEditor $itk_component(mfr).te 
    } {
	usual
	keep -waveData
	keep -displData
	keep -debug
	keep -path
	keep -object
    }
    itk_component add grEdit {
	ClassWESGrEditor $itk_component(mfr).ge
    } {
	usual
	keep -waveData
	keep -displData
	keep -debug
	keep -object
    }
    set settings(mode) "TEXT"
    set settings(path) "~"
    eval itk_initialize $args 
    if {[llength $args] != 0 } {eval configure $args}
    #initialize the two editor environment with the correct FIERA CFG DB
    
    pack $itk_component(textEdit) -side left -fill both -expand true
    pack $itk_component(mfr)      -side left -fill both -expand true
};#ClassWESEditor::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::undo
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::undo {} {
    if {$settings(mode) == "GRAPHIC"} {
	if {$itk_option(-object) != ""} {
	    $itk_option(-object) undo
	    $itk_component(grEdit) display 1
	}
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::redo
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::redo {} {
    if {$settings(mode) == "GRAPHIC"} {
	if {$itk_option(-object) != ""} {
	    $itk_option(-object) redo
	    $itk_component(grEdit) display 0
	}
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::dbg
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::dbg {m} {
    if {$itk_option(-debug) != ""} {
	$itk_option(-debug) insert end "$m\n"
	$itk_option(-debug) see end
	update idletasks
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::loadDisplayList
# PURPOSE: load the display configuration from the cfg file
#          the display configuration only comprises the order in which
#          the SIMMS are displayed. 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::loadDisplayList {} {
    dbg "loading display configuration"
    set parser [ClassWESParser #auto]
    $parser setInputFile $itk_option(-path)/$displayCfgFile
    if {[$parser expectT "BOARD_ORDER"]} {
	set rc 1
	set t [$parser getToken]
	set b $t
	while {($t != "") && ($t != ";")} {
	    set t [$parser getToken]
	    set b "$b $t"
	}
	if {$b == ""} {
	    bell
	    dbg "<<ERROR>> DisplayConfiguration File\
		    \"$displayCfgFile\" is corrupt."
	    dbg "                expected <board_order> but found \"\"."
	    
	} elseif {$b != ";"} {
	    $itk_option(-displData) configure -simmList $b
	}
    } else {
	bell
	dbg "<<Warning>> Display Configuration File \"$displayCfgFile\" is corrupt."
	dbg "                  expected \"BOARD_ORDER\" but found ??."
	dbg "Display list will be setup to board-order, and simm-order respectively."
    }

    if {[$itk_option(-displData) cget -simmList] == {}} {
	#puts "set the display order to default"
	set sList {}
	set brds [$itk_option(-waveData) getBoards]
	foreach b $brds {
	    set brdSimms [$b getSIMMS]
	    foreach s $brdSimms {
		lappend sList "[$b cget -name],[$s cget -name]"
	    }
	}
	$itk_option(-displData) configure -simmList $sList
	#puts $sList
    }
    delete object $parser
};#ClassWESEditor::loadDisplayList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::saveDisplayList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::saveDisplayList {} {
    if {[set sList [$itk_option(-displData) cget -simmList]] == {}} {
	return
    }
    set parser [ClassWESParser #auto]
    dbg "Saving Display List"
    if {[set rc [$parser setOutputFile $itk_option(-path)/$displayCfgFile]] != 0} {
	#puts "saving SLIST $sList"
	$parser putToken "BOARD_ORDER = $sList;"
	$parser putToken " "
	$parser closeOutput
    }
    dbg "done."
    delete object $parser
};#ClassWESEditor::saveDisplayList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::defbindings
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::defbindings {} {

};#ClassWESEditor::defbindings
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::loadConfig
# PURPOSE: load the HW Config
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::loadConfig {} {
    $itk_option(-displData) clear
    $itk_option(-waveData) clearWave
    $itk_option(-waveData) clearBoards
    

    $itk_option(-waveData) loadConfiguration 
    loadDisplayList
    if {$settings(path) != $itk_option(-path)} {
	#puts "configbody ClassWESEditor::path"
	set settings(path) $itk_option(-path)
    }
    
};#ClassWESEditor::loadConfig
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::load
# PURPOSE: load a file with waveform definitions
# PARM   : f -filename of file to open
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::load {f {mode 0}} {
    set rc "untitled"
#    loadConfig
    $this configure -object ""
    #$this configure -mode "TEXT"
    #$itk_component(grEdit) clear
    $itk_option(-waveData) configure -path [file dirname $f]
    
    #$itk_option(-waveData) loadConfiguration
    if {[set w [$itk_component(textEdit) load $f $mode]] != $FAILED} {
	$this configure -object $w
	update
	#puts "loaded waveform $w"
	display
	set rc $f
    }
    return $rc
};#ClassWESEditor::load
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::save
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::save {} {
    if {[set obj [$itk_component(textEdit) reload]] != $FAILED} {
	if {$obj != $SUCCESS} {
	    $this configure -object $obj
	}
	display
	update
    } else {
	$this configure -object ""
	return "untitled"
    }
    return [$itk_component(textEdit) save]
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::saveAs
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::saveAs {f} {
    if {[set obj [$itk_component(textEdit) reload]] != $FAILED} {
	if {$obj != $SUCCESS} {
	    $this configure -object $obj
	}
	display
	update
    } else {
	$this configure -object ""
	return "untitled"
    }
    $itk_component(textEdit) saveAs $f
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::display
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::display {} {
    set rc 0
    if {$itk_option(-object) != ""} {
	$itk_component(textEdit) display
	$itk_component(grEdit) display
	set rc 1
    }
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::clear
# PURPOSE: clear both display types, TEXT and GRAPHIC
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::clear {} {
    $itk_component(textEdit) clear
    $itk_component(grEdit) clear
    $itk_option(-displData) clear
};#ClassWESEditor::clear
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::component
# PURPOSE: return either component, TEXT or GRAPHIC -Editor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESEditor::component {what} {
    set rc ""
    switch -regexp $what {
	"TEXT" {
	    set rc $itk_component(textEdit)
	}
	"GRAPHIC" {
	    set rc $itk_component(grEdit)
	}
	default {
	    set rc "invalid component selected"
	}
    }
    return $rc
};#ClassWESEditor::component
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::mode
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::mode {
    switch -exact $itk_option(-mode) {
	"TEXT" {
	    if {$settings(mode) != "TEXT"} {
		pack forget $itk_component(grEdit)
		pack $itk_component(textEdit) -side left -fill both -expand true
		set settings(mode) "TEXT"
		if {($itk_option(-object) != "") && \
			[$itk_option(-object) isDirty]} {
		    
		    $itk_component(textEdit) display
		}
		
	    }
	}
	"GRAPHIC" {
	    if {$settings(mode) != "GRAPHIC"} {
		if {[set obj [$itk_component(textEdit) reload]] != $FAILED} {
		    pack forget $itk_component(textEdit)
		    pack $itk_component(grEdit) -side left -fill both -expand true
		    set settings(mode) "GRAPHIC"
		    if {$obj != $SUCCESS} {
			$this configure -object $obj
			$obj markDirty 1 TEXT
			$itk_component(grEdit) display 1
		    } else {
			$itk_component(grEdit) display 0
		    }
		    update
		} else {
		    $this configure -object ""
		}
	    }
	}
	default {
	    #unknown display mode, do nothing
	}
    }
};#ClassWESEditor::mode
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::object
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::object {
    
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::path
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::path {
    $itk_option(-waveData) configure -path $itk_option(-path) 
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::waveData
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::waveData {
    #set ClassWESTranslator::waveData $itk_option(-waveData)

}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::freq
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::freq {
    if {$itk_option(-waveData) != ""} {
	$itk_option(-waveData) configure -baseClkF $itk_option(-freq) \
		-baseClkT [expr 1/$itk_option(-freq)]
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::displData
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::displData {
    #set ClassWESTranslator::displData $itk_option(-displData)
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESEditor::debug
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
configbody ClassWESEditor::debug {
    #set ClassWESTranslator::debug $itk_option(-debug)
 
}
#
# ___oOo___


