#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESFileSel.tcl,v 4.5 2004/01/16 15:25:14 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  17/07/97  created
# abalestr 16/01/04  modified panedwindow to ipanedwindow  - see SPR no. 
#                    VLTSW20040021
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


itk::usual ClassWESFileSel { }
class ClassWESFileSel {
    inherit itk::Toplevel
    constructor       {args} {}
    destructor               {}

    itk_option define    -dirson      dirson      Dirson     "1"
    itk_option define    -fileson     fileson     Fileson    "1"
    itk_option define    -directory   directory   Directory  "."
    itk_option define    -mask        mask        Mask       "*"
    
    private   variable   unix         "0"
    private   variable   rc           "OK"
    private   variable   w            ""
    private   variable   filename     ""
    private   variable   sep          ""
    private   variable   cwd          "."
    private   variable   filter       "*" 
    public    variable   showhidden   "0"
    public    variable   grab         "0"
    
    private   method     setDefault     {b}
    private   method     setBindTag     {w} {
	set oldtags [bindtags $w]
	if {[lsearch $oldtags "FreezeEvents"] == -1} {
	    set bindtags [concat FreezeEvents $oldtags]
	    bindtags $w $bindtags
	}
	foreach child [winfo children $w] {
	    setBindTag $child
	}
	
    }   
    private   method     freeze         {} {
	foreach event [list KeyPress KeyRelease \
		ButtonPress ButtonRelease] {
	    bind FreezeEvents <Any-$event> {break}
	}
	$this configure -cursor watch
	update
    }
    private  method      unfreeze       {} {
	foreach event [list KeyPress KeyRelease \
		ButtonPress ButtonRelease] {
	    bind FreezeEvents <Any-$event> {}
	}
	$this configure -cursor top_left_arrow
	update
    }
    
    private   method     FILTER         {{dir {}}}
    private   method     OK             {}
    private   method     CANCEL         {}
    private   method     SELECT         {{what "file"}}
    private   method     dirList        {d}
    private   method     fileList       {d f}
    private   method     CD             {{d {}}}
    public    method     activate       {}
    public    method     deactivate     {}
    public    method     center         {}
    public    method     get            {} {global selection;return $selection;}
    
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::constructor {args} {
    set sep [string trim [file join . .] .]
    
    itk_component add eFilter {
	entryfield $itk_interior.efilter -labeltext "Filter" \
		-relief sunken -labelpos nw
    } {
	usual
	ignore -background
	ignore -foreground
    }
    itk_component add pw {
	ipanedwindow $itk_interior.pw -orient vertical \
		-width 400 -height 200
    } {
	ignore -background
    }    
    $itk_component(pw) add left
    $itk_component(pw) add right
    $itk_component(pw) fraction 50 50
    set dw [$itk_component(pw) childsite left]
    set fw [$itk_component(pw) childsite right]

    itk_component add selDir {
	scrolledlistbox $dw.selDir \
		-selectmode single \
		-vscrollmode static -hscrollmode dynamic -labeltext "Directory" \
		-dblclickcommand [code $this SELECT dir] \
		-textbackground Grey70 \
		-selectbackground Yellow \
		-selectforeground Black
    } {
	ignore -background
	ignore -foreground

    }
    itk_component add selFile {
	scrolledlistbox $fw.selFile \
		-selectmode single \
		-vscrollmode static -hscrollmode dynamic -labeltext "Files" \
		-selectioncommand [code $this SELECT file] \
		-dblclickcommand [code $this OK] \
		-textbackground Grey70 \
		-selectbackground Yellow \
		-selectforeground Black
    } {
	ignore -background
	ignore -foreground

    }
    itk_component add bBox {
	frame $itk_interior.bBox 
    } {
	usual
    }
    itk_component add bOK {
	button $itk_component(bBox).bOK  \
		-text Ok -width 6 -padx 8 -pady 2 \
		-command [code $this OK]
    } {
	usual
    }
    itk_component add bCancel {
	button $itk_component(bBox).bCancel  \
		-text Cancel -width 6 -padx 8 -pady 2 \
		-command [code $this CANCEL]
    } {
	usual
    }
    itk_component add bFilter {
	button $itk_component(bBox).bFilter  \
		-text Filter -width 6 -padx 8 -pady 2 \
		-command [code $this FILTER]
    } {
	usual
    }
    itk_component add eSel {
	entryfield $itk_interior.eSel -labeltext "Selection" \
		-relief sunken -labelpos nw
    } {
	usual
	ignore -background
	ignore -foreground
    }
    eval itk_initialize $args 
    if {[llength $args] != 0 } {eval configure $args}
    wm protocol $itk_component(hull) WM_DELETE_WINDOW {}
    
    bind [$itk_component(eFilter) component entry] \
	    <Return> [code $this FILTER]
    bind [$itk_component(eSel) component entry] \
	    <Return> [code $this OK]
    wm withdraw $itk_component(hull)
    setBindTag $itk_component(hull)
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::center
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::center {} {
    set w $itk_component(hull)
    set width 350

    
    wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
    
    #wm geometry $w ${width}x[winfo reqheight $w]+\
	    [expr ([winfo screenwidth $w]-$width)/2]+\
	    [expr ([winfo screenheight $w]-[winfo reqheight $w])/2]
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::OK
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::OK {} {
    global env
    set filename [$itk_component(eSel) get]
    if {[info exists env(HOME)]} {
        regsub ^~/ $filename $env(HOME)/ filename
	$itk_component(eSel) clear
	$itk_component(eSel) insert end $filename
    }
    if {(($itk_option(-dirson) == 1) && ($itk_option(-fileson) == 0) \
	    && ([file isdir $filename])) || \
	    (($itk_option(-fileson) == 1) && ![file isdir $filename])} {
	
    } else {
	return
    }
    set rc "OK"
    deactivate
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::CANCEL
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::CANCEL {} {
    set filename ""
    set rc "Cancel"
    deactivate
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::SELECT
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::SELECT {{what "file"}} {
    if [string match $what "dir"] {
	
	FILTER [$itk_component(selDir) getcurselection]
    } elseif [string match $what "file"] {
	set filename [file join $cwd [$itk_component(selFile) getcurselection]]
	$itk_component(eSel) clear
	$itk_component(eSel) insert end $filename
		
    } 
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::FILTER
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::FILTER {{dir {}}} {
    global env
    freeze
    if {$itk_option(-fileson) == 1} { 
	set filter [$itk_component(eFilter) get]
    }
    if {[info exists env(HOME)]} {
        regsub ^~/ $filter $env(HOME)/ filter
    }

    $this configure -mask [file tail $filter]
    if {[string match $dir {..}] } {
	CD ..
    } elseif {[string comp $dir {}] && [string comp $dir .]} {
	CD $dir
    } else {
	CD [file dir $filter]
    }
    
    set filter [file join $cwd $itk_option(-mask)]
    
    $itk_component(eFilter) clear
    $itk_component(eFilter) insert end $filter
    if [file isdir $filename] {
	set filename $cwd
    } else {
	set filename [file join $cwd [file tail $filename]]
    }
    $itk_component(eSel) clear
    $itk_component(eSel) insert end $filename
    $itk_component(selDir)  delete 0 end
    if {![regexp {^(.:)?/$} $cwd] && \
	    [string comp [file dir $cwd] $cwd]} {
	$itk_component(selDir) insert end ".."
    }
    foreach d [lsort [dirList $cwd]] {
	set d [file tail $d]
	if {[string comp $d .] && [string comp $d ..]} {
	    $itk_component(selDir) insert end $d
	}
    }
  
    $itk_component(selFile) delete 0 end
    foreach f [lsort [fileList $cwd $itk_option(-mask)]] {
	if {![file isdir $f]} {
	    $itk_component(selFile) insert end [file tail $f]
	}
    }
    $itk_component(eFilter) xview end
    $itk_component(eSel) xview end
    unfreeze
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::dirList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::dirList {d} {
    
    set dirs [glob -nocomplain $d*$sep]
    return $dirs
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::fileList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::fileList {d f} {
    set files [glob -nocomplain $d$f]    
    return $files
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::CD
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::CD {{d {}}} {
    global tcl_platform

    set tcl_platform(platform) "unix"
    if [string match $d {}] {
	set d [file dir $filter]
    }
    switch $d {
	.  {}
	.. {
	    set cwd \
		    [string trimr [file dir $cwd] /]/
	}
	default {
	    switch $tcl_platform(platform) {
		windows {
		    if [string match ?:* $d] {
			set cwd $d/
		    } elseif {[string match /* $d]} {
			set cwd [string index $cwd 0]:$d/
		    } else {
			append cwd $d/
		    }
		}
		default {
		    if [string match /* $d] {
			set cwd [string trimr $d /]/
		    } else {
			append cwd $d/
		    }
		}
	    }
	}
    }
    set filename [$itk_component(eSel) get]
    if {[file tail $filename] != ""} {
	set filename $cwd
    }
    $itk_component(eSel) clear
    $itk_component(eSel) insert end $filename
    regsub -all {/\./} $cwd {/} cwd
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::setDefault
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::setDefault {b} {

}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::activate
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::activate {} {
    global ::selection
    global env
    set selection ""
    set filename ""
    #CD $itk_option(-directory)
    if {[info exists env(HOME)]} {
        regsub ^~/ $itk_option(-directory) $env(HOME)/ cwd
    }
    if {$itk_option(-fileson) == 0} {	
	pack $itk_component(pw) -side top -fill both -expand 1
	pack $itk_component(bOK) -side left -padx 2 -pady 2 -ipadx 2 -ipady 2
	pack $itk_component(bCancel) -side right -padx 2 -pady 2 -ipadx 2 -ipady 2
	pack $itk_component(bBox) -fill x -padx 4 -pady 4 -ipadx 2 -ipady 2
	pack $itk_component(eSel) -side top -fill x -padx 4 -pady 4 -ipadx 2 -ipady 2
	pack $itk_component(selDir) -side left -fill both -expand 1
	$this configure -dirson 1
	$itk_component(selDir) configure \
		-selectioncommand [code $this SELECT dir]
	$itk_component(selDir) configure \
		-dblclickcommand [code $this OK]
	set filter "$itk_option(-directory)$sep$itk_option(-mask)"
    } else {
	pack $itk_component(eFilter) \
		-side top -fill x -padx 4 -pady 4 -ipadx 2 -ipady 2
	pack $itk_component(pw) -side top -fill both -expand 1
	pack $itk_component(bOK) -side left -padx 2 -pady 2 -ipadx 2 -ipady 2
	pack $itk_component(bFilter) -side left -expand 1 -padx 2 -pady 2 -ipadx 2 -ipady 2
	pack $itk_component(bCancel) -side right -padx 2 -pady 2 -ipadx 2 -ipady 2
	pack $itk_component(bBox) -fill x -padx 4 -pady 4 -ipadx 2 -ipady 2
	pack $itk_component(eSel) -side top -fill x -padx 4 -pady 4 -ipadx 2 -ipady 2
	pack $itk_component(selDir) -side left -fill both -expand 1
	pack $itk_component(selFile) -side left -fill both -expand 1
	$itk_component(eFilter) clear
	$itk_component(eFilter) insert end \
		"$itk_option(-directory)$sep$itk_option(-mask)"
    } 
    
    if {$itk_option(-dirson) == 0} {
	$this configure -fileson 1
    } else {
	$itk_component(selDir) configure \
		-selectioncommand {}
	$itk_component(selDir) configure \
		-dblclickcommand [code $this SELECT dir]
    }
    FILTER 
    wm deiconify $itk_component(hull)    
    $itk_component(pw) reset
    grab set $itk_component(hull)
    vwait ::selection
    update
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESFileSel::deactivate
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESFileSel::deactivate {} {
    global ::selection
    grab release $itk_component(hull)
    wm withdraw $itk_component(hull)
    if {$itk_option(-fileson) == 0} {
	pack forget $itk_component(pw)
	pack forget $itk_component(bOK) 
	pack forget $itk_component(bCancel)
	pack forget $itk_component(bBox)
	pack forget $itk_component(eSel)
	pack forget $itk_component(selDir)
	$this configure -dirson 1
	$itk_component(selDir) configure \
		-selectioncommand [code $this SELECT dir]
	$itk_component(selDir) configure \
		-dblclickcommand [code $this OK]
	
    } else {
	pack forget $itk_component(eFilter) 
	pack forget $itk_component(pw)
	pack forget $itk_component(pw)
	pack forget $itk_component(bOK)
	pack forget $itk_component(bFilter)
	pack forget $itk_component(bCancel) 
	pack forget $itk_component(bBox)
	pack forget $itk_component(eSel)
	pack forget $itk_component(selDir)
	pack forget $itk_component(selFile)
    }
    
    set selection $filename

}
configbody ClassWESFileSel::directory {
    set cwd $itk_option(-directory)
}
configbody ClassWESFileSel::dirson {
    if {$itk_option(-dirson) == 0} {
	$itk_component(pw) fraction 0 100	
    }
}
configbody ClassWESFileSel::fileson {
    if {$itk_option(-fileson) == 0} {
	$itk_component(pw) fraction 100 0
    } else {
	$itk_component(pw) fraction 50 50 
    }
}

#TEST TEST

#ClassWESFileSel .f -fileson 0 -directory ~/wes -title "sel Dir"
#.f activate


#
# ___oOo___


