#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESGrEditor.tcl,v 4.5 2004/01/16 15:25:00 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  25/10/96  created
# abalestr 16/01/04  modified panedwindow to ipanedwindow  - see SPR no. 
#                    VLTSW20040021
#

#************************************************************************
#   NAME
#       ClassWESGrEditor 
#   SYNOPSIS
#      ClassWESGrEditor <path> -option1 -option2
#   DESCRIPTION 
#      the graphical editor class provides all methods and data
#      to grphically display and manipulate the waveform data
#      which is contained in the internal waveform database
#      All interface routines are provided by the derived class
#      ClassWESWaveIF. The instance of this class is passed in the
#     itk_option variable -waveData
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
# create the global reference to the graphical editor instance
# to be able to access it directly

namespace eval ::wes {
    variable wesGrEditor
}
itk::usual ClassWESGrEditor { }
class ClassWESGrEditor {
    inherit itk::Widget ClassWESError
    
    constructor       {args} {}
    destructor               {}
    itk_option define  -displData displData DisplData ""
    itk_option define  -waveData  waveData  WaveData  ""
    itk_option define  -debug     debug     Debug     ""
    itk_option define  -object    object    Object    ""

    private    variable DISPLAY
    protected   variable lFont             "-Adobe-helvetica-medium-r-normal--12*" 
    public      method   display           {{redraw 1}}   
    public      method   clear             {{wipe 1}}        
    public      method   highrinlight         {y flag}    
    public      method   prepPrintCanvas   {cv}
    public      method   cvB3Clicked       {x y W t}
    public      method   cvB3Released      {x y W t}
    public      method   cvB1Clicked       {x y W t}
    public      method   cvB1Motion        {x y W t}
    public      method   cvB3Motion        {x y W t}
    public      method   lscvB1Motion      {x y}
    public      method   lscvB1Pressed     {x y}
    public      method   lscvB1Released    {x y}
    public      method   cvValue           {t}
    public      method   cvLeave           {x y W t}
    public      method   cvEnter           {x y W t}
    public      method   cvMotion          {x y W flag t}
    protected   method   defbindings       {}
    protected   method   init              {}
    protected   method   updateTickSetPos  {x t}         
    protected   method   createTag         {tag px py}
    protected   method   createCoords      {tag}
    protected   method   createSafeTagName {tag}
    public      method   updateTag         {tag}
    public      method   updateWorld       {}  
    public      method   initWorld         {}
    public      method   zoomHoriz         {d}
    public      method   zoomVert          {d}
    public      method   worldMark         {x y}
    public      method   worldDrag         {x y W}
};#ClassWESGrEditor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ClassWESGrEditor::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::constructor {args} {
    global PROP
    itk_component add mfr {
	frame $itk_interior.mfr
    } {
	usual
    }
    itk_component add tfr {
	frame $itk_component(mfr).tfr
    } {
	usual
    }
    itk_component add tfr1 {
	frame $itk_component(tfr).tfr1
    } {
	usual
    }
    itk_component add tfr11 {
	frame $itk_component(tfr1).tfr11 -relief flat -bd 0
    } {
	usual
    }
    itk_component add tfr12 {
	frame $itk_component(tfr1).tfr12 -relief flat -bd 0
    } {
	usual
    }
    itk_component add tfr13 {
	frame $itk_component(tfr1).tfr13 -relief flat -bd 0
    } {
	usual
    }

    itk_component add timeL {
	label $itk_component(tfr11).timeL -width 11 \
		-text "total length" \
		-anchor w \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add time {
	label $itk_component(tfr11).time -width 35 \
		-anchor w \
		-textvariable PROP(TIME) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add tposL {
	label $itk_component(tfr12).tpL -width 11 \
		-anchor w -text "t (tick)" \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add tpos {
	label $itk_component(tfr12).tp -width 10 \
		-anchor w \
		-textvariable PROP(T) \
		-font $lFont \
		-relief groove
    } {
	usual
	ignore -font
    }
    itk_component add sValL {
	label $itk_component(tfr12).svL -text "data" -width 11 \
		-anchor ne \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add sVal {
	label $itk_component(tfr12).sv  -width 10 \
		-anchor w \
		-textvariable PROP(V) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    } 
    itk_component add freqL {
	label $itk_component(tfr13).freqL  -width 11 \
		-text "f" \
		-anchor w \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add freq {
	label $itk_component(tfr13).freq  -width 35 \
		-anchor w \
		-textvariable PROP(F) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    }

    itk_component add tfr2 {
	frame $itk_component(tfr).tfr2 -relief raised
    } {
	usual
    }
    itk_component add tfr21 {
	frame $itk_component(tfr2).tfr21 -relief flat -bd 0
    } {
	usual
    }
    itk_component add tfr22 {
	frame $itk_component(tfr2).tfr22 -relief flat -bd 0
    } {
	usual
    }
    itk_component add tfr23 {
	frame $itk_component(tfr2).tfr23 -relief flat -bd 0
    } {
	usual
    }

    itk_component add simmIDL {
	label $itk_component(tfr21).sIDL -text "simm" -width 11 \
		-anchor w \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add simmID {
	label $itk_component(tfr21).sID -width 35 \
		-anchor w \
		-textvariable PROP(SIMM) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    } 
    itk_component add brdIDL {
	label $itk_component(tfr22).bidL -text "board" -width 11 \
		-anchor w \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add brdID {
	label $itk_component(tfr22).bid -width 35 \
		-anchor w \
		-textvariable PROP(BRD) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    }


    itk_component add exStatL {
	label $itk_component(tfr23).eStatL -text "ctrl. stats." -width 11 \
		-anchor w \
		-font $lFont
    }  {
	usual
	ignore -font
    }
    itk_component add exStat {
	label $itk_component(tfr23).eStat -text "" -width 35 \
		-anchor w \
		-textvariable PROP(ESTAT) \
		-relief groove \
		-font $lFont
    }  {
	usual
	ignore -font
    }


    itk_component add world {
	canvas $itk_component(tfr).wrld \
		-width 80 -height 80 -relief groove -bd 1
    }
    pack $itk_component(simmID) -side left -anchor nw
    pack $itk_component(simmIDL) -side left -anchor nw
    pack $itk_component(brdID) -side left -anchor nw
    pack $itk_component(brdIDL) -side left -anchor nw
    pack $itk_component(tposL)  -side left -anchor nw
    pack $itk_component(tpos)  -side left -anchor nw
    pack $itk_component(sValL)  -side left -anchor nw
    pack $itk_component(sVal)  -side left -anchor nw
    pack $itk_component(tfr11)  -side top -anchor nw -fill x -expand true
    pack $itk_component(tfr12)  -side top -anchor nw -fill x -expand true
    pack $itk_component(tfr13)  -side top -anchor nw
    pack $itk_component(exStat) -side left -anchor nw
    pack $itk_component(exStatL) -side left -anchor nw
    pack $itk_component(freqL)  -side left -anchor nw
    pack $itk_component(freq)  -side left -anchor nw
    pack $itk_component(timeL)  -side left -anchor nw
    pack $itk_component(time)  -side left -anchor nw
    pack $itk_component(tfr21)  -side top -anchor nw
    pack $itk_component(tfr22)  -side top -anchor nw
    pack $itk_component(tfr23)  -side top -anchor nw
   
    pack $itk_component(tfr1)  -side left -anchor c -expand true -fill x
    pack $itk_component(world)  -side left -anchor c
    pack $itk_component(tfr2) -side left -anchor c -expand true -fill x

    balloonhelp $itk_component(timeL) "total length of waveform"
    balloonhelp $itk_component(tposL) "current cursor position in ticks"
    balloonhelp $itk_component(freqL) "clock frequency at seq exec level"
    balloonhelp $itk_component(sValL) "click middle mouse button \n\
	    to display value of SIMM at tick t"
    balloonhelp $itk_component(brdIDL) "SIMM name, BOARD name,\n\
	    see file \"boards.brd\""
    balloonhelp $itk_component(exStatL) "usequencer exec level"
    itk_component add pw {
	ipanedwindow $itk_component(mfr).pw -orient vertical
    } {
	ignore -background
    }
    $itk_component(pw) add left -minimum 50
    $itk_component(pw) add right -minimum 200
    set lw [$itk_component(pw) childsite left]
    itk_component add lFrame { 
	frame $lw.lFrame -borderwidth 0
    } {
	usual
	keep -width
	keep -height
    }
    set rw [$itk_component(pw) childsite right]
    itk_component add rFrame {
	frame $rw.rFrame -borderwidth 0
    } {
	usual
	keep -width
	keep -height
    }
    itk_component add rcvFrame {
	frame $itk_component(rFrame).rcvFrame -borderwidth 0 -relief flat
    } {
	usual
	ignore -relief
    }
    itk_component add lcvFrame {
	frame $itk_component(lFrame).lcvFrame -width 50 -borderwidth 0
    } {
	usual
	ignore -relief
    }
    # add the scrolled canvases
    itk_component add tscv {
	canvas $itk_component(rcvFrame).tscv -height 50 -borderwidth 0
    }
    itk_component add rscv {
	canvas $itk_component(rcvFrame).rscv -relief flat -borderwidth 0
    } {
	usual
	ignore -relief
    }
    itk_component add lscv {
	canvas $itk_component(lcvFrame).lscv -width 50 -borderwidth 0
    } {
	usual
	ignore -relief
    }

    itk_component add rysb {
	scrollbar $itk_component(rFrame).rysb -orient vertical
    } 
    itk_component add rxsb {
	scrollbar $itk_component(rcvFrame).rxsb -orient horizontal
    }

    itk_component add lxsb {
	scrollbar $itk_component(lcvFrame).lxsb -orient horizontal
    }
    itk_component add ruler {
	ClassWESRuler $itk_interior.ruler \
		-canvas $itk_component(tscv)
    } {
	usual
	keep -object
    }
    itk_component add rulerLabel {
	ClassWESGrWLabel $itk_component(lFrame).rLabel\
		-name "ticks" -container $this -width 50 -height 50
    }
    eval itk_initialize $args 
    if {[llength $args] != 0 } {eval configure $args}
    $itk_component(rscv) configure\
	    -xscrollcommand [list $itk_component(rcvFrame).rxsb set] \
	    -yscrollcommand [list $itk_component(rFrame).rysb set]
    $itk_component(rxsb) configure -command [list bindXView \
	    [list $itk_component(tscv) $itk_component(rscv)]]
    $itk_component(rysb) configure -command [list bindYView \
	    [list $itk_component(lscv) $itk_component(rscv)]]
    $itk_component(lscv) configure\
	    -xscrollcommand [list $itk_component(lcvFrame).lxsb set]
    $itk_component(lxsb) configure \
	    -command [list $itk_component(lscv) xview]
    
    pack $itk_component(rulerLabel) -side top -fill x; #-expand true
    pack $itk_component(tscv) -side top -fill x
    pack $itk_component(rxsb) -side bottom -fill x
    pack $itk_component(rysb) -side right  -fill y
    pack $itk_component(rscv) -side left   -fill both -expand true
    pack $itk_component(lxsb)     -side bottom -fill x
    pack $itk_component(lscv)     -side left   -fill both -expand true
    pack $itk_component(lcvFrame) -side top    -fill both -expand true
    pack $itk_component(rcvFrame) -side top    -fill both -expand true
    pack $itk_component(tfr)      -side top    -fill x
    pack $itk_component(lFrame)   -side left   -fill both -expand true
    pack $itk_component(rFrame)   -side left   -fill both -expand true
    pack $itk_component(mfr)      -side left   -fill both -expand true
    $itk_component(pw) fraction 10 90
    pack $itk_component(pw) -fill both -expand true
    $itk_component(rulerLabel) display
    update
    defbindings
    set DISPLAY(init) 0
    namespace eval ::wes set wesGrEditor $this
};#ClassWESGrEditor


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::createSafeTagName
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::createSafeTagName {tag} {
    return "[$tag cget -board],[$tag cget -name]"
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::init
# PURPOSE :
#   a call to init resets the display, the display is rezised to fit
#   into the current visible display area 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::init {} {
    global PROP
    $itk_component(world) delete all    
    set DISPLAY(ticks) [expr [$itk_option(-waveData) getTicks]+1]
    if {[set DISPLAY(sCnt)  [$itk_option(-displData) getSimmCount]] == 0} {
	set DISPLAY(sCnt) 1
    }
    set DISPLAY(init) 1
    set DISPLAY(x0) 10
    set h [winfo height $itk_component(rscv)]
    set w [expr [winfo width $itk_component(rscv)] -20]
    #puts "width $w, height $h, sCount $DISPLAY(sCnt), ticks $DISPLAY(ticks)"
    if {[set DISPLAY(yoffset) [expr int([expr $h/$DISPLAY(sCnt)])]] < 10} {
	set h [expr 10*$DISPLAY(sCnt)]
	set DISPLAY(yoffset) 10
    } else {
	set h [expr $DISPLAY(sCnt)*$DISPLAY(yoffset)]
    }
    $itk_component(rscv) configure -height $h
    $itk_component(lscv) configure -height $h
    set DISPLAY(yheight) [expr int([expr 2.0*$DISPLAY(yoffset)/3.0])]
    #puts "$DISPLAY(yheight) $DISPLAY(yoffset)"
    if {[set DISPLAY(xoffset) [expr int([expr $w/$DISPLAY(ticks)])]] <= 0} {
	set w [expr 3*$DISPLAY(ticks)]
	set DISPLAY(xoffset) 3
    } else {
	set w [expr $DISPLAY(ticks)*$DISPLAY(xoffset) + 20]
    }
    $itk_component(rscv) configure -width $w
    $itk_component(lscv) configure -width $w
    set DISPLAY(lscMax) [winfo width $itk_component(lscv)]    
    set DISPLAY(xE) [expr $DISPLAY(x0) + \
	    $DISPLAY(xoffset)*$DISPLAY(ticks)]
    set DISPLAY(zoomX) 1.0
    set DISPLAY(zoomY) 1.0
    $itk_component(rscv) xview moveto 0.0
    $itk_component(rscv) yview moveto 0.0
    $itk_component(tscv) xview moveto 0.0
    $itk_component(tscv) yview moveto 0.0
    $itk_component(lscv) xview moveto 0.0
    $itk_component(lscv) yview moveto 0.0
    $itk_component(rscv) configure -scrollregion "0 0 $w $h"    
    $itk_component(lscv) configure -scrollregion "0 0 $DISPLAY(lscMax) $h"    
    $itk_component(tscv) configure -scrollregion "0 0 $w 50"
    $itk_component(lscv) configure -yscrollincrement $DISPLAY(yoffset)
    $itk_component(rscv) configure -yscrollincrement $DISPLAY(yoffset)
    $itk_component(tscv) configure -xscrollincrement $DISPLAY(xoffset)
    $itk_component(rscv) configure -xscrollincrement $DISPLAY(xoffset)
    set DISPLAY(last) "S"
    set DISPLAY(cur)  "E"
    set DISPLAY(in) ""
    update
    #trace variable PROP(T) w [$itk_component(ruler) gotoTick $PROP(T) "" w]
};#ClassWESGrEditor::init
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::defbindings
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::defbindings {} {
    global WORLD
    bind $itk_component(tscv) <Motion> [code $itk_component(ruler) cvMotion %x %y]
    bind $itk_component(rscv) <Configure> [code $this updateWorld]
    bind $itk_component(lscv) <B1-Motion> [code $this lscvB1Motion %x %y]
    bind $itk_component(lscv) <ButtonPress-1> [code $this lscvB1Pressed %x %y]
    bind $itk_component(lscv) <ButtonRelease-1> [code $this lscvB1Released %x %y]
    bind $itk_component(world) <Enter> {. configure -cursor hand2}
    bind $itk_component(world) <Leave> {. configure -cursor top_left_arrow}
    bind $itk_component(world) <B1-Motion> [code $this worldDrag %x %y %W]
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::worldDrag
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::worldDrag {x y W} {
    global WORLD
    
    set dx [expr $x - $WORLD(x1)]
    set dy [expr $y - $WORLD(y1)]
    set WORLD(x1) [expr $WORLD(x1) + $dx]
    set WORLD(x2) [expr $WORLD(x2) + $dx]
    set WORLD(y1) [expr $WORLD(y1) + $dy]
    set WORLD(y2) [expr $WORLD(y2) + $dy]
    $W move window $dx $dy
    if {([expr $WORLD(x2) + $dx] <= 80) && \
	    ([expr $WORLD(y2) + $dy] <= 80)} {
	set xfrac [expr ($WORLD(x1)+$dx)/80.0]
	set yfrac [expr ($WORLD(y1)+$dy)/80.0]
	$itk_component(rscv) xview moveto $xfrac
	$itk_component(tscv) xview moveto $xfrac
	$itk_component(lscv) yview moveto $yfrac
	$itk_component(rscv) yview moveto $yfrac
    }
    
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::updateWorld
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::updateWorld {} {
    global WORLD
    $itk_component(world) xview moveto 0.0
    $itk_component(world) yview moveto 0.0
    $itk_component(world) delete window
    set xvv [$itk_component(rscv) xview]
    set yvv [$itk_component(rscv) yview]
    set WORLD(x1) [expr int([expr [lindex $xvv 0] * 80.0])]
    set WORLD(y1) [expr int([expr [lindex $yvv 0] * 80.0])]
    set WORLD(x2) [expr int([expr [lindex $xvv 1] * 80.0])]
    set WORLD(y2) [expr int([expr [lindex $yvv 1] * 80.0])]
    $itk_component(world) create rect $WORLD(x1) $WORLD(y1) \
	    $WORLD(x2) $WORLD(y2) -stipple gray50 -tags window
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::cvValue
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::cvValue {t} {
    global PROP

    if {$DISPLAY(in) != ""} {
	set tick $PROP(T)
	set PROP(V) "[$t getAt $tick]"
    }
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::cvLeave
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::cvLeave {x y W t} {
    global PROP
    $itk_component(lscv) itemconfigure $t -fill black
    set DISPLAY(in) ""
    set PROP(V) ""
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::cvEnter
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::cvEnter {x y W t} {
    global PROP
    $itk_component(lscv) itemconfigure $t -fill red
    set PROP(BRD)  "[$t cget -board]"
    set PROP(SIMM)  "[$t cget -name]"
    set DISPLAY(in) $t
    #set PROP(W) "PATTERN $pat"
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::lscvB1Pressed
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::lscvB1Pressed {x y} {
    global PROP

    set sx [$itk_component(lscv) canvasx $x]
    set sy [$itk_component(lscv) canvasy $y]
    set PROP(y) $sy
    set ids [$itk_component(lscv) find closest $sx $sy]
    set tags [$itk_component(lscv) gettags $ids]
    foreach t $tags {
	set sTag [createSafeTagName $t]
	set PROP(tag) $t
	$itk_component(lscv) itemconfigure $t -fill green
	set PROP(pos) [expr int([expr $PROP(y)/$DISPLAY(yoffset)])]
	$itk_component(rscv)."$sTag" configure -bg bisque
	return
    } 
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::lscvB1Released
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::lscvB1Released {x y} {
    global PROP    
    if {![info exists PROP(tag)]} {
	return
    }
    $itk_component(rscv)."[createSafeTagName $PROP(tag)]" \
	    configure -bg Grey90 -relief groove -bd 1
    set pos [expr int([expr $PROP(y)/$DISPLAY(yoffset)])]
    $itk_component(lscv) itemconfigure $PROP(tag) -fill black
    #puts "dropped at $pos"
    #display
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::lscvB1Motion
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::lscvB1Motion {x y} {
    global PROP

    if {![info exists PROP(tag)]} {

	return
    }    
    set sx [$itk_component(lscv) canvasx $x]
    set sy [$itk_component(lscv) canvasy $y]
 
    set yy0 [expr int([expr $sy/$DISPLAY(yoffset)])*$DISPLAY(yoffset)]
    if {$yy0 == 0} {
	set yy1 $DISPLAY(yoffset)
    } else {
	set yy1 [expr $yy0+$DISPLAY(yoffset)]
    }
    set dy [expr $sy - $PROP(y)]
    if {$dy == 0} {
	return 
    } else {
	set ny [expr $PROP(y) + $dy]
	if {$ny >= 0} {
	    set npos [expr int([expr $ny/$DISPLAY(yoffset)])]
	} else {
	    set npos 0
	}
	set PROP(y) $ny
	#puts "old [$itk_option(-displData) cget -simmList]"
	#puts "shifting from $PROP(pos) to $npos."
	if {[set tags [$itk_option(-displData) \
		changeTagOrder $PROP(tag) $PROP(pos) $npos]] != {}} {
	    set py 0
	    set PROP(pos) $npos
	    $itk_component(lscv) delete all
	    foreach t $tags {
		#puts "$t [$itk_component(lscv) coords $t]"
		if [regexp "$PROP(tag)" $t] {
		    $itk_component(lscv) create text 10 [expr $py+$DISPLAY(yoffset)/2] \
			    -anchor w -text [$t cget -name]\
			    -tags $t -fill green -font $lFont
		} else {
		    $itk_component(lscv) create text 10 [expr $py+$DISPLAY(yoffset)/2] \
			    -anchor w -text [$t cget -name]\
			    -tags $t -fill black -font $lFont
		}
		if {[set w [$itk_component(rscv) find withtag $t]] != ""} {
		    $itk_component(rscv) coords $w 0 $py
		}		
		set py [expr $py+$DISPLAY(yoffset)]
	    }
	}
    }

   
    set lsch [winfo height $itk_component(lscv)]
    set yt [expr [lindex [$itk_component(lscv) yview] 0]*$lsch]
    set yb [expr [lindex [$itk_component(lscv) yview] 1]*$lsch]  
    set scry 0
    if {$y > $yb} {
	set scry [expr int([expr $y - $yb])]
    } elseif {$y < $yt} {
	if {[expr $y - $yt] >= -$yt} {
	    set scry [expr int([expr $y - $yt])]
	}
    }
    if {$scry != 0} {
	#puts "scroll $yt,$yb,$y"
	$itk_component(lscv) yview scroll $scry units
	$itk_component(rscv) yview scroll $scry units
	updateWorld
    }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::cvB3Motion
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::cvB3Motion {x y W t} {
    global ::WES
    global PROP
    if {[$itk_option(-object) noModify]} {
	return
    }
    set rc 0

    set xx [$W canvasx $x]
    set yy [$W canvasy $y]
    set w [winfo width $itk_component(rscv)]
    
    set xl [expr [lindex [$itk_component(rscv) xview] 0]*[$W cget -width]]
    set xr [expr [lindex [$itk_component(rscv) xview] 1]*[$W cget -width]]  
    set sx 0
    if {$x > $xr} {
	set sx [expr int([expr $x - $xr])]
    } elseif {$x < $xl} {
	if {[expr $x - $xl] >= -$xl} {
	    set sx [expr int([expr $x - $xl])]
	}
    }
    $itk_component(tscv) xview scroll $sx units
    $itk_component(rscv) xview scroll $sx units
    #set xx [expr int([expr $sx + $xx])]
    if {[set tick [updateTickSetPos $xx $t]] < 0} {
	return
    } 
    set DISPLAY(cur) $tick
    set DISPLAY(last) $tick
    switch -exact [$t cget -type] {
	"MULTILEVEL" {
	    
	    if {($yy <= $DISPLAY(yL)) && ($yy >= $DISPLAY(yH))} {
		set yr [expr 1 - ($yy - $DISPLAY(yH))/$DISPLAY(yheight)]
		set v [expr int([expr $yr*255.0])]
		$t setAt $tick $v
		set PROP(V) "$v"
	    }
	}
	"BILEVEL" {
	    if {($yy <= $DISPLAY(yL)) && ($yy >= $DISPLAY(yH))} {
		if {$yy < [expr $DISPLAY(yH) + \
			($DISPLAY(yL) - $DISPLAY(yH))*0.5]} {
		    $t setAt $tick 1
		    set PROP(V) "1"
		} else {
		    $t setAt $tick 0
		    set PROP(V) "0"
		}
	    }
	}
    }
    $itk_option(-object) markDirty 1 GRAPHIC
    updateTag $t
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::cvMotion
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::cvMotion {x y W flag t} {
    set rc -1
    if {$flag} {
	set rc [updateTickSetPos [$W canvasx $x] $t]
    } else {
	set rc [updateTickSetPos [$W canvasx $x] $t]
    }
    return $rc
}

body ClassWESGrEditor::cvB3Clicked {x y W t} { 

    cvB3Motion $x $y $W $t
}

body ClassWESGrEditor::cvB3Released {x y W t} {
    set DISPLAY(last) "S"
    set DISPLAY(cur)  "E"
}

body ClassWESGrEditor::cvB1Clicked {x y W t} {
    cvB1Motion $x $y $W $t
}
body ClassWESGrEditor::cvB1Motion {x y W t} {
    global ::WES
    global PROP

   if {[$itk_option(-object) noModify]} {
	return
    }
    set rc 0

    set xx [$W canvasx $x]
    set yy [$W canvasy $y]
    set w [winfo width $itk_component(rscv)]
    
    set xl [expr [lindex [$itk_component(rscv) xview] 0]*[$W cget -width]]
    set xr [expr [lindex [$itk_component(rscv) xview] 1]*[$W cget -width]]  
    set sx 0
    if {$x > $xr} {
	set sx [expr int([expr $x - $xr])]
    } elseif {$x < $xl} {
	if {[expr $x - $xl] >= -$xl} {
	    set sx [expr int([expr $x - $xl])]
	}
    }
    $itk_component(tscv) xview scroll $sx units
    $itk_component(rscv) xview scroll $sx units
    #set xx [expr int([expr $sx + $xx])]
    if {[set tick [updateTickSetPos $xx $t]] < 0} {
	return
    } 
    set DISPLAY(cur) $tick
    set DISPLAY(last) $tick
    
    switch -exact [$t cget -type] {
	"MULTILEVEL" {
	    
	    if {($yy <= $DISPLAY(yL)) && ($yy >= $DISPLAY(yH))} {
		set yr [expr 1 - ($yy - $DISPLAY(yH))/$DISPLAY(yheight)]
		set v [expr int([expr $yr*255.0])]
		$t setOnlyAt $tick $v $DISPLAY(ticks)
		set PROP(V) "$v"
	    }
	}
	"BILEVEL" {
	    if {($yy <= $DISPLAY(yL)) && ($yy >= $DISPLAY(yH))} {
		if {$yy < [expr $DISPLAY(yH) + \
			($DISPLAY(yL) - $DISPLAY(yH))*0.5]} {
		    $t setOnlyAt $tick 1 $DISPLAY(ticks)
		    set PROP(V) "1"
		} else {
		    $t setOnlyAt $tick 0 $DISPLAY(ticks)
		    set PROP(V) "0"
		}
	    }
	}
    }
    $itk_option(-object) markDirty 1 GRAPHIC
    
    updateTag $t
    return $rc

} 

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# bindXView 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc bindXView {lists args} {
    foreach l $lists {
	eval {$l xview} $args
    }
    [namespace eval ::wes set wesGrEditor] updateWorld 
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# bindYView 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
proc bindYView {lists args} {
    foreach l $lists {
	eval {$l yview} $args
    } 
    [namespace eval ::wes set wesGrEditor] updateWorld 
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::display 
# PURPOSE:
#     the main display method,
#     gets all existing tags from wesDisplayData and 
#     creates them on the canvas
# RETURN:
#     void 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::display         {{redraw 1}} { 
    global PROP
    if {[set tags [$itk_option(-displData) getTags]] != {}} {
	set cnt 0
	set i 0
	set len [llength $tags]
	if {$redraw} {
	    clear
	    init
	    #puts "ticks -> $DISPLAY(ticks)"
	    $itk_component(ruler) configure -xoffset $DISPLAY(xoffset) \
		    -px $DISPLAY(x0) -py 0 -ticks $DISPLAY(ticks) \
		    -rw [expr $DISPLAY(xE) -$DISPLAY(x0)] \
		    -canvas $itk_component(tscv)
	    $itk_component(ruler) display
	    foreach t $tags {
		createTag $t $DISPLAY(x0) [expr $cnt*$DISPLAY(yoffset)]
		incr cnt
	    }
	    initWorld
	} else {
	    
	    $itk_component(ruler) configure -xoffset $DISPLAY(xoffset) \
		    -px $DISPLAY(x0) -py 0 -ticks $DISPLAY(ticks) \
		    -rw [expr $DISPLAY(xE) -$DISPLAY(x0)] \
		    -canvas $itk_component(tscv)
	    $itk_component(ruler) display
	    foreach t $tags {
		updateTag $t
	    }
	}
	set PROP(TIME) [$itk_option(-object) getLength]
	set PROP(F) ""
	#update the world map view
	set xv [lindex [$itk_component(rscv) xview] 1]
	set yv [lindex [$itk_component(rscv) yview] 1]
	#puts "$xv $yv"
    }    
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::prepPrintCanvas
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::prepPrintCanvas {cv} {
    global ::WES
    if {$DISPLAY(init) == 0} {
	init
    }
    set ymax [expr ($DISPLAY(sCnt) + 3)*$DISPLAY(yoffset)]
    $cv configure -width [expr [winfo width $itk_component(rscv)]+50]
    $cv configure -height [expr ($DISPLAY(sCnt) + 4)*$DISPLAY(yoffset)]
    set px [expr $DISPLAY(x0)+80]
    set oldcv [$itk_component(ruler) cget -canvas]
    $itk_component(ruler) configure -canvas $cv -px $px -py 0
    $itk_component(ruler) display
    set cnt 2
    if {[set tags [$itk_option(-displData) getTags]] != {}} {
	foreach t $tags {
	    set lines [createCoords $t]
	    foreach l $lines {
		#puts "line $l"
		set mTag [createSafeTagName $t]
		eval {$cv create line} $l\
			{-fill black -tag $mTag}
		set dy [expr ($DISPLAY(yoffset)-$DISPLAY(yheight))/2]
		$cv scale $mTag 0 1 $DISPLAY(xoffset)  -$DISPLAY(yheight)
		$cv move $mTag $px [expr $dy+($cnt+1)*$DISPLAY(yoffset)]
		$cv create text 3 [expr $dy+($cnt+1)*$DISPLAY(yoffset)] \
			-anchor w -text [$t cget -name]	-fill black
	    }
	    incr cnt
	}
    }
    #draw the vertical lines    
    set yy [expr 2*$DISPLAY(yoffset)]
    if {$DISPLAY(xoffset) < 5} {
	set xMul 3
    } else {
	set xMul 1
    }
    switch -exact [$itk_option(-object) cget -type] {
	"PATTERN" 
	{   
	    for {set i 0;set x $px} \
		    {$i <$DISPLAY(ticks)} {incr i $xMul; \
		    set x [expr $x + $xMul*$DISPLAY(xoffset)]} {
		$cv create line $x $yy $x $ymax \
			-fill red -tag clock -width 1
		$cv create text $x [expr $yy -5] -anchor w -text $i -tags tick \
			-fill black
	    }
	}
	"USEQUENCE"    
	{
	    for {set i 0;set x $px} \
		    {$i <$DISPLAY(ticks)} {incr i $xMul; \
		    set x [expr $x + $DISPLAY(xoffset)*$xMul]} {
		$cv create line $x $yy $x $ymax \
			-fill red -tag clock -width 1
		$cv create text $x [expr $yy -5] -anchor w -text $i -tags tick \
			-fill black
	    }
	}
	"SEQUENCE"     
	{
	    for {set i 0;set x $px} \
		    {$i <$DISPLAY(ticks)} {incr i $xMul; \
		    set x [expr $x + $xMul*$DISPLAY(xoffset)]} {
		$cv create line $x $yy $x $ymax \
			-fill red -tag clock -width 1
	    }
	    
	}
	default {puts stderr "unknown Object-Type : \"[$itk_option(-object) cget -type]\""}
    }
    
    $cv create text $px [expr ($cnt+2)*$DISPLAY(yoffset)] \
	    -text "[$itk_option(-object) cget -type]:  \
	    \"[$itk_option(-object) cget -name]\"." \
	    -anchor w -fill black
    $cv create text $px [expr ($cnt+3)*$DISPLAY(yoffset)] -text [exec date] \
	    -anchor w -fill black
    $itk_component(ruler) configure -canvas $oldcv
    display
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::clear
# PURPOSE:
#    destroy all items in the canvases
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::clear {{wipe 1}} {
    global PROP
    #puts "ClassWESGrEditor::clear"
    #delete all tags on each canvas
    if [info exists PROP] {
	unset PROP
    }
    $itk_component(rscv) delete all
    $itk_component(lscv) delete all
    $itk_component(world) delete all
    $itk_component(tscv) delete rPos
    $itk_component(rscv) xview moveto 0
    $itk_component(rscv) yview moveto 0
    $itk_component(lscv) xview moveto 0
    $itk_component(lscv) yview moveto 0
    $itk_component(pw) reset
    #make sure that none of the tag display canvases exist
    #if not destroy these widgets first
    if {[set tags [$itk_option(-displData) getTags]] != {}} {
	foreach t $tags {
	    set sTag [createSafeTagName $t]
	    if {[winfo exists $itk_component(rscv)."$sTag"]} {
		
		destroy $itk_component(rscv)."$sTag"
	    }
	}
    }
    if {$wipe} {
	set DISPLAY(init) 0
    }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::createTag
# PURPOSE:
#    visual display of the data belonging to tag <tag> canvases
#    at position <pos>
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::createTag {tag px py} {
    set sTag [createSafeTagName $tag]
    #puts "safeTagName $sTag"
    $itk_component(lscv) delete $sTag
    #the tag is displayed on it own canvas, which will be inserted
    #into the main canvas as a window item
    set cv [canvas $itk_component(rscv)."$sTag" \
	     -relief groove -bd 1 -width $DISPLAY(xE) \
	     -height $DISPLAY(yoffset)]
    set wcv [$itk_component(rscv) create window 0 $py \
	    -anchor nw -window $cv -tag $tag]
    
    #draw the label in the label canvas
    $itk_component(lscv) create text 10 [expr $py+$DISPLAY(yoffset)/2] \
	    -anchor w -text [$tag cget -name] -tags $tag \
	    -fill black -font $lFont

    #draw the line onto the canvas
    set DISPLAY(yH) [expr int([expr ($DISPLAY(yoffset)-$DISPLAY(yheight))/2])]
    set DISPLAY(yL) [expr $DISPLAY(yheight) + $DISPLAY(yH)]
    set lines [createCoords $tag]
    foreach l $lines {
	#puts "line $l"
	eval {$cv create line} $l\
		{-fill blue -tag $tag}
	eval {$itk_component(world) create line} $l\
		{-fill blue -tag $tag}
	$cv move $tag $DISPLAY(x0) $DISPLAY(yL)
	$cv scale $tag $DISPLAY(x0) $DISPLAY(yL)  $DISPLAY(xoffset)  -$DISPLAY(yheight)
	$itk_component(world) move $tag 0 $DISPLAY(yL)
	$itk_component(world) scale $tag 0 $DISPLAY(yL) \
		[expr 80.0/$DISPLAY(ticks)] -$DISPLAY(yheight)
	$itk_component(world) move $tag 0 $py
	
	#$cv scale $tag 0 $y $DISPLAY(xoffset) -$DISPLAY(yheight)
    }
    #$itk_component(world) addtag line withtag $tag
    #eval {$itk_component(rscv) create line} $points\
	    {-fill blue -tag $tag}
    #now create the bindings for the tag
    bind $cv <ButtonPress-1> \
	                 [code $this cvB1Clicked %x %y %W "$tag"]
    #bind $cv <B1-Motion> [code $this cvB1Motion %x %y %W "$tag"]
    bind $cv <ButtonPress-3> \
	                 [code $this cvB3Clicked %x %y %W "$tag"]
    bind $cv <ButtonRelease-3> \
	                 [code $this cvB3Released %x %y %W "$tag"]
    bind $cv <B3-Motion> [code $this cvB3Motion %x %y %W "$tag"]
    bind $cv <Motion>    [code $this cvMotion %x %y %W 1 "$tag"]
    bind $cv <Leave>     [code $this cvLeave %x %y %W "$tag"]
    bind $cv <Enter>     [code $this cvEnter %x %y %W "$tag"]
    bind $cv <ButtonPress-2>   [code $this cvValue "$tag"]
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::createCoord
# PURPOSE:
#    create the polyline coords for <tag> at offset px py 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::createCoords {tag} {   
    set lines {}
    #puts $tag
    lappend lines [join [$tag getData $DISPLAY(ticks)]]
    return $lines
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::updateTag
# PURPOSE:
#    update the visual display of tag <tag> 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::updateTag {tag} {
    #puts "update"
    set sTag [createSafeTagName $tag]
    set lines [createCoords $tag]
    set cv $itk_component(rscv)."$sTag"
    $cv delete $tag
    foreach l $lines {
	#puts "line $l"
	eval {$cv create line} $l\
		{-fill blue -tag $tag}
	set dy [expr int([expr ($DISPLAY(yoffset)-$DISPLAY(yheight))/2])]
	$cv scale $tag 0 1 $DISPLAY(xoffset)  -$DISPLAY(yheight)
	$cv move $tag $DISPLAY(x0) $dy
    }  

}

body ClassWESGrEditor::initWorld {} {
    #set w [$itk_component(rscv) cget -width]
    set h [$itk_component(rscv) cget -height]
    
    $itk_component(world) scale all 0 0 1.0 [expr 80.0/$h]
    updateWorld
} 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::updateTickSetPos
# PURPOSE:
#    displays a vertical line according to the current position
#    in the ruler axis.
#    <x> is the current x - coordinate of the mouse pointer
# RETURN
#    the discrete tick value calculated from <x>
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::updateTickSetPos {x t} {
    global PROP

    #$itk_component(rscv) delete rPos
    set xn [expr ($x -$DISPLAY(x0))/$DISPLAY(xoffset)]
    if {$xn >= 0} {
	if {$xn <= $DISPLAY(ticks)} {
	    set xi [expr int($xn)]
	    set xt [expr ($xi*$DISPLAY(xoffset)) + $DISPLAY(x0)]
	    set PROP(ESTAT) [lindex [$itk_component(ruler) gotoTick $xt $xi] 0]
	    set PROP(F) [lindex [$itk_component(ruler) gotoTick $xt $xi] 1]
	    set PROP(T) $xi
	} else {
	    set xi -1
	}
    } else {
	$itk_component(ruler) gotoTick $DISPLAY(x0) 0
	set PROP(T) 0
	set PROP(ESTAT) ""
	set xi -1
    }
    return $xi
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::zoomHoriz
# PURPOSE:
#    zoom the top and the main display canvas horizontally
#    <d> is the direction 
#             +1 magnify
#             -1 shrink
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::zoomHoriz {d} { 
    set zoomX [expr $DISPLAY(xoffset)*[expr pow(2.0,$d)]]
    #puts $zoomX
    if {($d < 0) && ( $zoomX >= 3.0)} {
	set DISPLAY(zoomX) [expr $zoomX/$DISPLAY(xoffset)]
    } elseif {$d > 0} {
	set DISPLAY(zoomX) [expr $zoomX/$DISPLAY(xoffset)]
    } else {
	return
    }
    #set DISPLAY(xoffset) [expr int([expr $DISPLAY(xoffset)*$DISPLAY(zoomX)])]
    set DISPLAY(xoffset) [expr $DISPLAY(xoffset)*$DISPLAY(zoomX)]
    $itk_component(rscv) scale all 0 0 $DISPLAY(zoomX) 1
    #$itk_component(tscv) scale all $DISPLAY(x0) 0 $DISPLAY(zoomX) 1
    set DISPLAY(xE) [expr $DISPLAY(xoffset)*$DISPLAY(ticks)]
    $itk_component(ruler) configure -rw [expr $DISPLAY(xE) -$DISPLAY(x0)] \
	    -xoffset $DISPLAY(xoffset) -zoom $DISPLAY(zoomX)
    #$itk_component(ruler) display
    set tags [$itk_option(-displData) getTags]
    if {$tags != {}} {
	#$itk_component(rscv) configure -width [expr $DISPLAY(xE)+20]
	foreach t $tags {
	    set sTag [createSafeTagName $t]
	    if {[winfo exists $itk_component(rscv)."$sTag"]} {
		$itk_component(rscv)."$sTag" configure -width [expr $DISPLAY(xE)+20]
		$itk_component(rscv)."$sTag" scale all \
			$DISPLAY(x0) 0 $DISPLAY(zoomX) 1
	    }
	}
    } else {
	puts "empty display list"
    }

    
    set ym [expr $DISPLAY(yoffset)*$DISPLAY(sCnt)]
    $itk_component(rscv) configure -scrollregion "0 0 \
	    [expr $DISPLAY(xE)+20] $ym"    
    $itk_component(tscv) configure -scrollregion "0 0 \
	    [expr $DISPLAY(xE)+20] $DISPLAY(lscMax)"  

    updateWorld
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESGrEditor::zoomVert
# PURPOSE:
#    zoom the label canvas and the main display canvas vertically
#    <d> is the direction 
#             +1 magnify
#             -1 shrink
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESGrEditor::zoomVert {d} {
    set zoomY [expr $DISPLAY(yoffset)*[expr pow(2.0,$d)]]
    #puts $zoomY
    if {($d < 0) && ( $zoomY >= 10.0)} {
	set DISPLAY(zoomY) [expr $zoomY/$DISPLAY(yoffset)]
    } elseif {$d > 0} {
	set DISPLAY(zoomY) [expr $zoomY/$DISPLAY(yoffset)]
    } else {
	return
    }
    $itk_component(rscv) scale all $DISPLAY(x0) 0 1 $DISPLAY(zoomY)
    $itk_component(lscv) scale all $DISPLAY(x0) 0 1 $DISPLAY(zoomY)
    set DISPLAY(yoffset) [expr int([expr $DISPLAY(yoffset)*$DISPLAY(zoomY)])]
    set DISPLAY(yheight) [expr int([expr $DISPLAY(yheight)*$DISPLAY(zoomY)])]
    set DISPLAY(yH) [expr int([expr ($DISPLAY(yoffset)-$DISPLAY(yheight))/2])]
    set DISPLAY(yL) [expr $DISPLAY(yheight) + $DISPLAY(yH)]
    set tags [$itk_option(-displData) getTags]
    if {$tags != {}} {
    
	foreach t $tags {
	    set sTag [createSafeTagName $t]
	    if {[winfo exists $itk_component(rscv)."$sTag"]} {
		$itk_component(rscv)."$sTag" configure -height $DISPLAY(yoffset)
		$itk_component(rscv)."$sTag" scale all \
			$DISPLAY(x0) 0 1 $DISPLAY(zoomY)
	    }
	}
	
    }
    
    set ym [expr $DISPLAY(yoffset)*$DISPLAY(sCnt)]
    $itk_component(rscv) configure -scrollregion "0 0 \
	    [expr $DISPLAY(xE)+20] $ym"    
    #puts $ym
    $itk_component(lscv) configure -scrollregion "0 0 $DISPLAY(lscMax) $ym" 
    updateWorld
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# ___oOo___


