#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESHWCfg.tcl,v 4.5 2004/01/16 15:25:01 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  29/04/97  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual ClassWESHWCfg { }
class ClassWESHWCfg {
    inherit ClassWESTrBase
    constructor                          {args} {}
    destructor                           {}
    private   variable FIXED_BOARDS1     {
	BRD_VIDBRD0 BRD_VIDBRD1 BRD_VIDBRD2 BRD_VIDBRD3
    }
    private   variable FIXED_BOARDS2     {
	BRD_COMMBRD
    }
    public    variable waveData          ""

    public    variable brdCfgFileExt     ".brd"
    public    variable voltCfgFileExt    ".volt"
    public    method   loadConfiguration {}  
    public    method   readPhaseDef      {parser brd}
    public    method   readBoardDef      {parser}
    public    method   setFixedBoards    {b}
};#ClassWESHWCfg
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ClassWESHWCfg::constructor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESHWCfg::constructor {args} {
    if {[llength $args] != 0 } {eval configure $args}
}
# ----------------------------------------------------------------------------------
# ClassWESHWCfg::readPhaseDef
#
# PURPOSE: Reading the SIMM Definitions in the BOARD Cfg File
# PARM   : parser - the parser object which will provide the stream of tokens
#          brd    - the board name which hosts the SIMMs
# ----------------------------------------------------------------------------------
body ClassWESHWCfg::readPhaseDef {parser brd} {
    set rc 0
    set simmPos ""
    set simmType ""
    set simmName ""
    if {[set obj [getBoardObject $brd]] == ""} {
	wesError INVALID_BOARD "Board $brd is invalid"
	return 0
    }
    while {[set t [$parser getToken]] != "BOARDCONFIG_END"} {
	if {[string match "HAS_PREAMP" $t]} {
	    set t [$parser getToken]
	    continue
	}
	if {[$parser isNumeric $t]} {
	    set simmPos $t
	} else {
	    dbg "<<ERROR-Translator>> Board Definition in [$parser getInfo]:\
		    \n<simm_num> is corrupt"
	    wesError SIMM_NUM_CORRUPT "Board Definition in [$parser getInfo]:\
		    \n<simm_num> = $t is corrupt"
	    break
	}
	if {[set simmType [$parser getToken]] == 0} {
	    dbg "<<ERROR-Translator>> Board Definition in [$parser getInfo]:\
		    \n<simm_type> is corrupt"
	    wesError SIMM_TYPE_CORRUPT "Board Definition in [$parser getInfo]:\
		    \n<simm_type> = $t is corrupt"
	    break
	}
	if {[set simmName [$parser getString]] == 0} {
	    dbg "<<ERROR-Translator>> Board Definition in [$parser getInfo]:\
		    \n<simm_name> is corrupt"
	    wesError SIMM_NAME_CORRUPT "Board Definition in [$parser getInfo]:\
		    \n<simm_name> = $t is corrupt"
	    break
	}
	if {($simmType != "EMPTY") && ($simmName != "") && ($simmPos >=  0)} {
	    set s [$obj createSIMM]
	    $s configure -name $simmName -pos $simmPos -type $simmType
	    set rc 1
	} 
    }    
    return $rc
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ClassWESHWCfg::readBoardDef 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESHWCfg::readBoardDef {parser} {
    set rc -1
    if {[$parser expectT "BOARDCONFIG_START"]} {
	set rc 1
	set brd [$parser getToken]
	if {$brd != ""} {
	    if ![regexp "BRD_ANABIAS" $brd] {
		if ![setFixedBoards $brd] {
		    dbg "loading board config for $brd"
		    set bObj [createBoard $brd [$parser getInputFile]]
		    set rc [readPhaseDef $parser $brd]
		} else {
		    set rc 1
		}
	    }
	} else {
	    #wesError
	    dbg "<<ERROR-Translator>> Board Definition in [$parser getInfo]:\
		    \n<board_name> is corrupt"
        }
    }
    return $rc
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ClassWESHWCfg::loadConfiguration 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESHWCfg::loadConfiguration {} {
    clearWave
  
    dbg "\nloading HW Configuration from path \"$path\""
    if {$path == ""} {
	return
    }
    set files [glob -nocomplain $path/*$brdCfgFileExt]
    foreach bf [lsort $files] {
	set parser [ClassWESParser #auto]
        if {[$parser setInputFile $bf] != 0} {
            while { [set rc [readBoardDef [code $parser]]] != -1} {
            }
        }
	delete object $parser
    }
    dbg "done loading HW configuration."
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ClassWESHWCfg::setFixedBoards 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESHWCfg::setFixedBoards    {b} {
    if {[lsearch -exact $FIXED_BOARDS1 $b] >= 0} {
	dbg "presetting fixed board config for $b"
	if {[set boardObj [createBoard $b "fixed config"]] != ""} {
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 12 -name "O_ENABLE0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 13 -name "O_ENABLE1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 14 -name "O_ENABLE2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 15 -name "O_ENABLE3" -type "BILEVEL"
	    
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 8  -name "DCHARGE0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 9  -name "DCHARGE1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 10 -name "DCHARGE2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 11 -name "DCHARGE3" -type "BILEVEL"
	
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 4  -name "SAMPLE0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 5  -name "SAMPLE1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 6  -name "SAMPLE2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 7  -name "SAMPLE3" -type "BILEVEL"

	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "CLAMP0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 1  -name "CLAMP1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 2  -name "CLAMP2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 3  -name "CLAMP3" -type "BILEVEL"

	    set obj [$boardObj createSIMM]
	    $obj configure -pos 3  -name "TEST_VIDEO" -type "MULTILEVEL"
	} else {
	    wesError CREATE_BOARD "Cannot Create Board $b"
	    return 0
	}
	return 1
    }
    if {[lsearch -exact $FIXED_BOARDS2 $b] >= 0} {
	dbg "presetting fixed board config for $b"
	if {[set boardObj [createBoard "BRD_COMMBRD" "fixed config"]] != ""} {
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 8   -name "FRAME_CTRL_BIT0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 9   -name "FRAME_CTRL_BIT1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 10  -name "FRAME_CTRL_BIT2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 11  -name "FRAME_CTRL_BIT3" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 12  -name "FRAME_CTRL_BIT4" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 13  -name "FRAME_CTRL_BIT5" -type "BILEVEL"
	    
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "VID0_CLK_IN" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 1  -name "VID1_CLK_IN" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 2  -name "VID2_CLK_IN" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 3  -name "VID3_CLK_IN" -type "BILEVEL"
	    
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "OUT0" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "OUT1" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "OUT2" -type "BILEVEL"
	    set obj [$boardObj createSIMM]
	    $obj configure -pos 0  -name "WRITE_FIFO" -type "BILEVEL"
	} else {
	    wesError CREATE_BOARD "Cannot Create Board $b"
	    return 0
	}
	return 1
    } else {
	return 0
    }
}

#
# ___oOo___


