#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESParser.tcl,v 4.5 2004/01/16 15:25:02 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  20/09/96  created
# hpieper  25/09/96  implemented readrulefile - procedure
#

#************************************************************************
#   NAME
#       ClassWESParser - class for reading and writing raw tokens
#       to HD. Finds "strings" and numeric expressions.
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
itk::usual ClassWESParser { }
class ClassWESParser {
    constructor                    {args} {}
    destructor                            {}    
    private   variable inputFile   "" 
    private   variable inputBuffer ""
    private   variable outputFile  ""
    private   variable outputBuffer ""
    private   variable outFileID   ""
    public    variable last        ""
    public    variable token    
    public    variable info
    public    variable comment     {}
    private   variable numTokens   "0"
    public    variable tokenCount  "0"
    public    variable lineCount   "0"
    public    method   closeOutput {}  
    public    method   isString    {str}
    public    method   isNumeric   {str}
    public    method   getToken    {}
    public    method   ungetToken  {}
    public    method   putToken    {t}
    public    method   expectT      {str}
    public    method   getString   {}
    public    method   getNumeric  {}
    public    method   init        {}
    public    method   setInputFile    {f} {}
    public    method   getInputFile    {}  {return $inputFile}
    public    method   setInputBuffer  {f} {}
    public    method   setOutputFile   {f {dCmts ""}} {}
    public    method   setOutputBuffer {f {dCmts ""}} {}
    public    method   dbg {m} {}
    public    method   wesErrorDlg {w m} {}
    public    method   infoDlg {w m} {}
    public    method   getInfo {} {}
    public    method   getBuffer {} {}
};

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESParser::constructor {args} {
    set last ""
    set lineCount 0
    set tokenCount 0
    set numTokens  0
    set token($tokenCount) 0
    set info($tokenCount) 0
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESParser::getBuffer {} {
    set m ""
    if {$inputFile == ""} {
	set m "text"
    } else {
	set m "file"
    }
    return $m
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESParser::getInfo {} {
    set m ""
    if {$inputFile == ""} {
	set m "TextBuffer line $info($tokenCount)"
    } else {
	set m "File \"$inputFile\" line $info($tokenCount)"
    }
    return $m
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESParser::getToken {} {
    incr tokenCount
    if {$tokenCount > $numTokens} {
	set tokenCount $numTokens
	return ""
    }
    set t $token($tokenCount)
    set last $t
    #set token($tokenCount) ""
    #puts $t
    return $t
}

body ClassWESParser::ungetToken {} {
    incr tokenCount -1
    set token($tokenCount) $last
}

body ClassWESParser::putToken {t} {
    if {$outputFile == ""} {
	$outputBuffer insert end "$t\n"
    } else {
	puts $outFileID $t
    }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESParser::expectT {str} {
    if {$str == ""} { return 0 }
    if {$numTokens == 0} { return 0 }

    set rc 0
    while {[set t [getToken]] != ""} {
	if {$t == $str} {
	    set rc 1
	    break
	}
    }
    return $rc
}

body ClassWESParser::isString {str} {
    set rc 0
    if { [regexp -nocase {^[a-z]+[_]?([a-z0-9]+|[a-z]+)} $str] != 0} {
	set rc 1
    }
    return $rc
}

body ClassWESParser::getString {} {
    if {[set t [getToken]] != {\"}} {
	return ""
    }
    set str ""
    set t [getToken]
    while {($t != {\"}) && ($t != "")} {
	set str "$str $t"
	set t [getToken]
    }
    regsub {^[ ]} $str "" str
    return "$str"
}

body ClassWESParser::isNumeric {str} {
    if { [regexp {^((0x)?[0-9a-fA-F]+|[0-9]+)} $str] != 0} {
	return 1
    } else {
	return 0
    }
}

body ClassWESParser::getNumeric {} {
    set t [getToken]
    if { [isNumeric $t] != 0} {
	return $t
    } else {
	return ""
    }
}

body ClassWESParser::init {} {
    global WES_PARSER
    set lineCount 0
    set last ""
    set comment {}
    foreach a {token info} {
	foreach e [array names $a] {
	    unset ${a}($e)
	}
    }
    set numTokens 0
    set tokenCount -1
    set token($numTokens) 0
    set info($numTokens) 0

}

body ClassWESParser::closeOutput {} {
    close $outFileID
}

body ClassWESParser::setInputFile {f} {
    set rc 0
    set inputBuffer ""
    set inputFile $f
    init
    if { [file exists $inputFile] == 0 } {
	set rc 0
	return $rc
    }
    if [catch {open $inputFile r+} fileID] {
	set rc 0
	return $rc
    } else {
	set rc 1
    }	
    set tok ""
    foreach line [split [read $fileID] \n] {
	incr lineCount
	#eliminate comments
	if {[regexp {\#(.)*} $line cmt]} { 
	    lappend comment "$lineCount [list $cmt]"
	}
	regsub {\#(.*)} $line {} line
	regsub -all {\;} $line { ;} line
	regsub -all {\"} $line { \" } line
	#regsub -all {\,} $line { ,} line
	regsub -all {\{} $line " \{ " line
	regsub -all {\}} $line " \} " line
	foreach str [split $line " =\t\:"] {
	    if {$str != ""} {
		#puts "$str"
		set token($numTokens) $str
		set info($numTokens) $lineCount
		incr numTokens
	    }	
	} 
    }
    
    set token($numTokens) ""
    set info($numTokens) ""
#    puts "n of Tokens is $numTokens"
    close $fileID
    return $rc
}

body ClassWESParser::setOutputFile {f {dCmts ""}} {
    global WES_PARSER
    set rc 0
    set outputBuffer ""
    set outputFile $f
    if { [file exists $outputFile] == 0 } {
    }
    if [catch {open $outputFile w} outFileID] {
	bell
    } else {
	set rc 1
    }
    
    return $rc
}

body ClassWESParser::setInputBuffer {f} {
    global WES_PARSER
    set inputFile ""
    set inputBuffer $f
    init
    set text [$inputBuffer get 0.0 end]
    
    foreach line [split $text \n] {
	#eliminate comments
	if {$line != ""} {
	    incr lineCount
	}
	if {[regexp {\#(.*)} $line cmt]} { 
	    lappend comment "$lineCount [list $cmt]" 
	}
	
	regsub {\#(.*)} $line {} line
	regsub -all {\;} $line " ;" line
	regsub -all {\"} $line { \" } line
	regsub -all {\,} $line " ," line
	regsub -all {\{} $line " \{" line
	regsub -all {\}} $line " \}" line
	foreach str [split $line " =\t\:"] {
	    if {$str != ""} {
		#puts "$str"
		set token($numTokens) $str
		set info($numTokens) $lineCount
		incr numTokens
		#puts $str
	    }
	} 
    }
    #    unset text
    set token($numTokens) ""
    set info($numTokens) ""
    return 1
}

body ClassWESParser::setOutputBuffer {f {dCmts ""}} {
    global WES_PARSER
    set outputFile "" 
    set outputBuffer $f
    foreach c $dCmts {
	puts "writing comments...."
	set cmt [lindex $c 1]
	putToken "$cmt"
    }
    return 1
}



#
# ___oOo___








