#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESTrPat.tcl,v 4.5 2004/01/16 15:25:07 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  29/04/97  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
itk::usual ClassWESTrPat { }
class ClassWESTrPat {
    inherit ClassWESHWCfg 
    constructor               {args}      {}
    destructor                            {} 

    public    method   getPatInPath      {}
    public    method   readPattern       {parser {zero 0} {saveFlg 0}}
    public    method   readInPattern     {parser {zero 0}}
    public    method   readInPatStep     {parser pObj {zero 0}}
    public    method   readOutPattern    {parser {zero 0}}
    public    method   readOutPatStep    {parser pObj {zero 0}}
    public    method   writeOutPattern   {parser pObj {saveToHD 0}}
    public    method   writeInPattern    {parser pObj {saveToHD 0}}
}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESTrPat::constructor
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESTrPat::constructor {args} {
    if {[llength $args] != 0 } {eval configure $args}
}
# ----------------------------------------------------------------------------------
# ClassWESTrPat::getPatInPath
# ----------------------------------------------------------------------------------
body ClassWESTrPat::getPatInPath {} {
    set sList {}
    set files [glob -nocomplain $path/*$inPatFileExt]
    foreach f $files {
	lappend sList [file tail $f]
    }
    #puts "getPatInPath"
    return $sList
}
# ----------------------------------------------------------------------------------
# Reading Patterns
# ----------------------------------------------------------------------------------
body ClassWESTrPat::readPattern {parser {zero 0} {saveFlg 0}} {
    set rc $FAILED
    #puts "readPattern : zero is $zero"
    while {[set t [$parser getToken]] != ""} {
	switch -exact $t {
	    "PATTERN" {
		set rc [readInPattern $parser $zero]
	    }
	    "PATTERN_START" {
		set rc [readOutPattern $parser $zero]
		if {$saveFlg && ($rc != 0)} {
		    set pars [ClassWESParser #auto]
		    set f $path/$rc$inPatFileExt
		    dbg "saving PATTERN \"$rc\" to file \"$f\"."
		    $pars setOutputFile $f
		    writeInPattern [code $pars] $rc
		    $pars closeOutput
		    delete object $pars
		    clearWave
		    $displData clear
		}
	    }
	}
    }
    if {$rc == $FAILED} {
	set msg "[$parser getInfo]\n\
		does not contain any PATTERN definitions."
	dbg "<<ERROR-Translator>> $msg"
	wesError WAVEFORM_EMPTY "$msg"
	return $FAILED
    }
    return $rc
}
# ----------------------------------------------------------------------------------
# Read the Pattern Input Format
# ----------------------------------------------------------------------------------
body ClassWESTrPat::readInPatStep {parser pObj {zero 0}} {    
    set rc $FAILED
    
    if {[set t [$parser getToken]] == "time"} {
	$parser ungetToken
    } else {
	set msg "Pattern Definition in [$parser getInfo]:\n\
		[$pObj cget -name] is corrupt.\nexpecting \"time\" but found $t."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    while {[$parser expectT "time"]} {
	if {[set  tick [$parser getNumeric]] != ""} {
	    set t [$parser getToken]
	    while {($t != "") && ($t != "time") && ($t != "\}")} {
		#simm definition, now we need the value
		if {[set val [$parser getNumeric]] == ""} {
		    #wesError, tick value missing
		    set msg "Pattern Definition in [$parser getInfo]:\n\
			    $pat is corrupt.\n<value> is missing."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		} else {
		    set brd [$pObj getBoard]
		    if {[set s [$brd hasSIMM $t]] == $FAILED} {
			wesError $SYNTAX_ERROR "SIMM $t does not exist on board \
				[$brd cget -name]"
			return $FAILED
		    }
		    set rc $SUCCESS
		    switch [$s cget -type] {
			"BILEVEL" {
			    if {($val == 0) || ($val == 1)} {
				if {[$s setIniAt [expr $zero + $tick] $val] != $SUCCESS} {
				    return $FAILED
				}
			    } else {
				wesError $SYNTAX_ERROR "BILEVEL-SIMM [$s cget -name]\n\
					cannot be assigned a value other than \"0\"\
					or \"1\""
				return $FAILED
			    }
			}
			"MULTILEVEL" {
			    if {($val >= 0) && ($val <= 0xff)} {
				if {[$s setIniAt [expr $zero + $tick] $val] != $SUCCESS} {
				    return $FAILED
				}
				
			    } else {
				wesError $SYNTAX_ERROR "BILEVEL-SIMM [$s cget -name]\n\
					can only be assigned a value in the range\n\
					of \"0-255\""
				return $FAILED
			    }
			}
			default {
			    wesError $SYNTAX_ERROR "Unknown SIMM type of SIMM [$s cget -name]"
			    return $FAILED
			}
		    }
		    $displData insertPhase $s
		}
		if {[set t [$parser getToken]] != ";"} {
		    #wesError, the definition has to end with a semi colon
		    set msg "Pattern Definition in [$parser getInfo] is corrupt.\n\
			    missing \";\"."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
		set t [$parser getToken]
		if {![$parser isString $t] && ($t != "\}")} {
		    #puts "t = $t"
		    set msg "Pattern Definition in [$parser getInfo] is corrupt.\n\
			    expectTing <simm_name>."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }   
	    if {$t == "time"} {
		$parser ungetToken
	    } else {
	    }
	}   else {
	    set msg "Pattern Definition \"$pat\"\n\
		    line $info($tokenCount) is corrupt.\n\
		    <tick-value> is missing."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	    break
	}   
    }
    setTick $tick
    return $rc
}
# ----------------------------------------------------------------------------------
# ClassWESTrPat::readInPattern
# ----------------------------------------------------------------------------------
body ClassWESTrPat::readInPattern {parser {zero 0}} {
    if {[set brd [$parser getToken]] == ""} {
	set msg "Pattern Definition in [$parser getInfo]:\n\
		is corrupt.\n\<board_name> not found."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set pat  [$parser getToken]] != ""} {
	dbg "<Translator> Processing Pattern \"$pat\""
	if {[set patObj [createPattern $pat $brd]] != $FAILED} {
	    if {[$parser getBuffer] == "file"} {
		 markDirty $pat 0
	    }
	   
	    if {[set id  [$parser getToken]] != "\{"} {
		set msg "Pattern Definition in [$parser getInfo]:\n\
			is corrupt.\n\"\{\" expected but found \"$id\"."
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    } else { 
		if {[readInPatStep $parser $patObj $zero] == $FAILED} {
		    set msg "Pattern Definition in [$parser getInfo]:\n\
			    is corrupt.\n\Cannot read tick statement."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    } 
	} else {
	    set msg "<board_name> \"$brd\" in PATTERN \"$pat\"\
		    does not exist."
	    dbg "<ERROR-Translator> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
    } else {
	#wesError
	set msg "Pattern Definition in [$parser getInfo]:\n\
		<pat_name> does not exist or is corrupt."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
	
    }
    $patObj createComment [$parser cget -comment]
    $parser configure -comment {}

    if {[set t [getTicks]] < $MIN_PAT_LENGTH} {
	set msg "Pattern Definition in [$parser getInfo]:\n\
		PATTERN length \"$t\" must be greater than $MIN_PAT_LENGTH."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }

    $patObj configure -length "[expr [getTicks] + 1] ticks"
    return $patObj
};#readInPattern

# ----------------------------------------------------------------------------------
# Read a Output Format Pattern Step (read line until ";")
# ----------------------------------------------------------------------------------	    
body ClassWESTrPat::readOutPatStep {parser pObj {zero 0}} {
    set rc $FAILED
    set tick 0
    while {[set t [$parser getToken]] != "PATTERN_END"} {
	while { $t != ";"} {
	    if {$t ==  {\"}} {
		set t [$parser getToken]
		if {[set nn [$parser getToken]] != {\"}} {
		    set msg "Pattern Definition in [$parser getInfo]:\
			    [$pObj cget -name] is corrupt.\" is missing."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	    if {[$parser isString $t]} {
		if {[set val [$parser getNumeric]] == ""} {
		    #wesError, tick value missing
		    set msg "Pattern Definition in [$parser getInfo]:\
			    [$pObj cget -name] is corrupt.<value> is missing."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
		set brd [$pObj getBoard]
		if {[set s [$brd hasSIMM $t]] == ""} {
		    wesError $SYNTAX_ERROR "SIMM $t does not exist on board \
			    [$brd cget -name]"
		    return $FAILED
		}
		set rc $SUCCESS
		switch [$s cget -type] {
		    "BILEVEL" {
			if {($val == 0) || ($val == 1)} {
			    if {[$s setIniAt [expr $zero + $tick] $val] != $SUCCESS} {
				return $FAILED
			    }
			     
			} else {
			    set msg "Pattern Definition in [$parser getInfo] is corrupt.\n\
				    SIMM \"$t\" on board \"$brd\"\n\
				    cannot be assigned a value other than \"0\" or \"1\"."
			    dbg "<<ERROR-Translator>> $msg"
			    wesError $SYNTAX_ERROR "$msg"
			    return $FAILED
			}
		    }
		    "MULTILEVEL" {
			if {($val >= 0) && ($val <= 0xff)} {
			    if {[$s setIniAt [expr $zero + $tick] $val] != $SUCCESS} {
				return $FAILED
			    }
			} else {
			    set msg "Pattern Definition in [$parser getInfo] is corrupt.\n\
				    SIMM \"$t\" on board \"$brd\"\n\
				    can only be assigned a value between \"0\" and \"255\"."
			    dbg "<<ERROR-Translator>> $msg"
			    wesError $SYNTAX_ERROR "$msg"
			    return $FAILED
			}
		    }
		    default {
			set msg "Pattern Definition in [$parser getInfo] is corrupt.\n\
						SIMM \"$t\" does not exists on board \"$brd\"."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		}
		$displData insertPhase $s
	    }
	    set t [$parser getToken]
	}
	if {$t == ";"} {
	    setTick $tick
	    incr tick
	    set rc $SUCCESS
	    #set WAVE(PT,$pat) $tick
	}
    }
    return $rc
};#readOutPatStep

# ----------------------------------------------------------------------------------
# Read the Output Pattern Format
# ----------------------------------------------------------------------------------

body ClassWESTrPat::readOutPattern {parser {zero 0}} {
    if {[set brd [$parser getToken]] == ""} {
	set msg "Pattern Definition in [$parser getInfo]:\n\
		is corrupt.\n\<board_name> not found."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set pat  [$parser getString]] != ""} {
	if {[set patObj [createPattern $pat $brd]] != $FAILED} {
	    if {[$parser getBuffer] == "file"} {
		 markDirty $pat 0
	    }
	    
	    # insert the SIMMS into the pattern
	    if {[readOutPatStep $parser $patObj $zero] == $FAILED} {
		set msg "Pattern Definition in [$parser getInfo]:\n\
			is corrupt.\n\Cannot read tick statement."
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    }
	} else {
	    set msg "<board_name> \"$brd\" in PATTERN \"$pat\"\
		    does not exist."
	    dbg "<ERROR-Translator> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
    } else {
	#wesError
	set msg "Pattern Definition in [$parser getInfo]:\n\
		<pat_name> does not exist or is corrupt."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    $patObj createComment [$parser cget -comment]
    $parser configure -comment {}
    if {[set t [getTicks]] < $MIN_PAT_LENGTH} {
	set msg "Pattern Definition in [$parser getInfo]:\n\
		PATTERN length \"$t\" must be greater than $MIN_PAT_LENGTH."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    $patObj configure -length "[expr [getTicks] + 1] ticks"
    return $patObj
}

# ----------------------------------------------------------------------------------
# Writing Patterns
# ----------------------------------------------------------------------------------

# ----------------------------------------------------------------------------------
# Write the Output Pattern Format
# ----------------------------------------------------------------------------------
body ClassWESTrPat::writeOutPattern {parser p {saveToHD 0}} {
    foreach c [$p getComment] {
	set cmt [lindex $c 1]
	$parser putToken "$cmt"    
    }
    
    set ticks [ getTicks]
    $parser putToken "PATTERN_START [[$p getBoard] cget -name]\"[$p cget -name]\""
    for {set i 0} {$i <= $ticks} {incr i} {
	set simms [$p getSIMMSAt $i]
	foreach s $simms {
	    if {[set v [$s getAt $i]] != ""} {
		$parser putToken "    \"[$s cget -name]\" = $v"
	    }
	}
	$parser putToken ";"
    }
    $parser putToken "PATTERN_END"
}

# ----------------------------------------------------------------------------------
# Write the Input Pattern Format
# ----------------------------------------------------------------------------------
body ClassWESTrPat::writeInPattern {parser p {saveToHD 0}} {
    foreach c [$p getComment] {
	set cmt [lindex $c 1]
	$parser putToken "$cmt"    
    }
    set ticks [getTicks]
 
    $parser putToken "PATTERN [[$p getBoard] cget -name] \
	    [$p cget -name]"
    $parser putToken "\{"
    for {set i 0} {$i <= $ticks} {incr i} {
	set simms [$p getSIMMSAt $i]
	if {$simms != {}} {
	    $parser putToken "  time $i:"	
	    foreach s $simms {
		#puts "$s getAt $i"
		if {[set v [$s getAt $i]] != ""} {
		    $parser putToken "    [$s cget -name] = $v;"
		}
	    }
	}
    }
    if {$simms == {}} {
	$parser putToken "  time $ticks:"
    }
    $parser putToken "\}"
    
}
#
# ___oOo___


