#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classWESTrSeq.tcl,v 4.5 2004/01/16 15:25:07 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hpieper  29/04/97  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
itk::usual ClassWESTrSeq { }

class ClassWESTrSeq {
    inherit ClassWESTrUSeq
    
    constructor               {args}       {}
    destructor                             {} 
    private   variable tickCnt             "-1"
    public    method   readSeq             {parser {saveFlg 0}}
    public    method   readInSeqExecStat   {parser seqObj xbl}
    public    method   readInSeqRepStat    {parser seqObj rbl}
    public    method   readInSeq           {parser}
    public    method   readOutSeqExecStat  {parser seqObj xbl}
    public    method   readOutSeqRepStat   {parser seqObj rbl}
    public    method   readOutSeq          {parser}
    public    method   writeInSeqExecStat  {parser stat rec}
    public    method   writeInSeqRepStat   {parser stat rec}
    public    method   writeInSeq          {parser seqObj}
    public    method   writeOutSeqExecStat {parser stat rec}
    public    method   writeOutSeqRepStat  {parser stat rec}
    public    method   writeOutSeq         {parser seqObj}
}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ClassWESTrSeq::constructor
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
body ClassWESTrSeq::constructor {args} {
    if {[llength $args] != 0 } {eval configure $args}
}
body ClassWESTrSeq::readSeq  {parser {saveFlg 0}} {
    set rc $WAVEFORM_EMPTY
    set absTime 0.0
    set tickCnt -1
    while {[set t [$parser getToken]] != ""} {
	switch -exact $t {
	    "SEQUENCE" {
		set rc [readInSeq $parser]
	    }
	    "SEQUENCE_START" {
		set rc [readOutSeq $parser]
		if {$saveFlg && ($rc != 0)} {
		    set pars [ClassWESParser #auto]
		    set f $path/$rc$inSeqFileExt
		    dbg "saving SEQ \"$rc\" to file \"$f\"."
		    $pars setOutputFile $f
		    writeInSeq [code $pars] $rc
		    $pars closeOutput
		    delete object $pars
		     clearWave
		    $displData clear
		}
	    }
	}
    }
    if {$rc == $WAVEFORM_EMPTY} {
	set msg "[$parser getInfo]\n\
		does not contain any Sequence definitions."
	dbg "<<ERROR-Translator>> $msg"
	wesErrorDlg "Translator" "$msg"
	return $FAILED
    }
    return $rc      
}
# ----------------------------------------------------------------------------------
# Read Input Format exec Statement
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readInSeqExecStat  {parser seqObj xbl} {
    if {[set useq [$parser getToken]] == ""} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<useq_name> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    #puts "USEQ $useq"
    if {[set r [$parser getNumeric]] == ""} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<repetition> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set c [$parser getNumeric]] == ""} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<clockspeed> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    } elseif {$c == 0} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<clockspeed> divider greater than zero expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set sc [$parser getToken]] != ";"} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		\";\" expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set f "$path/$useq$inUSeqFileExt"
    set pars [ClassWESParser #auto]
    #puts "before read $useq [getTicks]"

    if {[$pars setInputFile $f] != 0} {
	if {[readUSeq [code $pars] [incr tickCnt]] == $FAILED} {
	    #wesError
	    delete object $pars
	    return $FAILED
	}
	delete object $pars
    } else {
	set msg "USequence Definition in [$parser getInfo] is corrupt.\n\
		cannot open file \"$f\"."
	dbg "<<ERROR-Translator>> $msg"
	wesError $CANNOT_OPEN_FILE "$msg"
	return $FAILED
    }
    set tt [expr 1+[getTicks]]

    set freq [expr $baseClkF/$c]
    set tickT [expr $c*$baseClkT]
    #puts "$absTime. exec statement active for $tt ticks."
    set absTime [expr $absTime + $baseClkT*$tt*$r*$c]
    $xbl configure -ctrlType X -start $tickCnt -absTime $absTime \
	    -end [expr $tickCnt + $tt] -text "$useq $r $c" \
	    -freq "[format "%2.3E" $freq] Hz" \
	    -tickT "[format "%2.3E" $tickT] s"
    #puts "$useq $tickCnt [expr $tickCnt + $tt]"
    incr tickCnt [expr $tt -1] 
    #puts "time after exec $useq -> $absTime, len $tt, r $r, c $c "
    return $SUCCESS
}
# ----------------------------------------------------------------------------------
# Read Input Format Repeat Statement
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readInSeqRepStat  {parser seqObj rbl} {
    set stats {}
    set useqTime $absTime
    set tCnt [expr $tickCnt + 1]
    if {[set r [$parser getNumeric]] != ""} {
	if {$r <= 0} {
	    set msg "Sequence Definition [$parser getInfo]:\n\
		    <repetition> greater than 0  expected."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
    } else {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<repetition> expected, but found \"$c\"."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    
    if {[set c [$parser getToken]] != "\{"} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set tok [$parser getToken]
    while {($tok != "\}") && ($tok != "")} {
	#puts "repeat"
	switch -exact $tok {
	    "wait"  {
		set tt [$parser getToken]
		if {[set sc [$parser getToken]] != ";"} {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    \";\" expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
		set wt [expr int([expr ($tt*1.0e-6)/$baseClkT])]
		#puts "wait $tt $wt"
		set absTime [expr $absTime + ($tt*1.0e-6)] 
		$rbl setBlock W \
			[expr 1+$tickCnt] $absTime [expr $tickCnt + $wt+1] "wait $tt"
		set patterns [getPattern]
		incr tickCnt $wt
		#setTick $tickCnt
	    }
	    "repeat" {
		set rrbl [$rbl setBlock R $tickCnt "0.0" 0 "repeat"]
		if {[set cmd [readInSeqRepStat $parser $seqObj $rrbl]] == $FAILED} {
		    return $FAILED
		}
	    }
	    default {
		if {[set useq $tok] == ""} {    
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    <useq> expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
		$parser ungetToken
		set xbl [$rbl setBlock X $tickCnt "0.0" 0 "execute"]
		if {[set cmd [readInSeqExecStat $parser $seqObj $xbl]] == $FAILED} {
		    return $FAILED
		}
	    }
	}
	set tok [$parser getToken]
    }
    if {$tok != "\}"} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		\"\}\" expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set absTime [expr $absTime + ($absTime - $useqTime)*$r]
    $rbl configure -ctrlType R -start $tCnt -absTime $absTime \
	    -end [expr $tickCnt +1] -text "repeat $r"
    #puts "time after repeat $r $absTime"
    #puts "repeat end"
    return $SUCCESS
}
# ----------------------------------------------------------------------------------
# Read Sequence In Input Format
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readInSeq  {parser} {
    set rc $WAVEFORM_EMPTY
    set tickCnt -1
    set absTime 0.0
    set pxSet 0
    set pySet 0
    set poSet 0
    set px ""
    set px ""
    set po ""
    set t ""
    set c ""
    set r ""
    if {[set seq  [$parser getToken]] != ""} {	
	while {[set t [$parser getToken]] != "\{"} {
	    switch -exact $t {
		"pixel_x" {
		    if {[set px [$parser getToken]] != ""} {
			
			set pxSet 1
		    } else {
			set msg "Sequence Definition [$parser getInfo]:\n\
				<pixel_x_dim> expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    
		}
		"pixel_y" {
		    if {[set py [$parser getNumeric]] != ""} {
			set pySet 1
		    } else {
			set msg "Sequence Definition [$parser getInfo]:\n\
				<pixel_y_dim> expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    
		}
		"pixel_order" {
		    if {[set po [$parser getString]] != ""} {
			set poSet 1
		    } else {
			set msg "Sequence Definition [$parser getInfo]:\n\
				<pixel_order_info> \(quoted string\) expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    
		}
		default {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    invalid expression \"$t\"."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	    if {$pxSet && $pySet && $poSet} {
		 set seqObj [createSeq $seq $px $py $po]
		dbg "<Translator> Processing Sequence \"$seq\""
		break
	    }
	}
	if {!$pxSet || !$pySet || !$poSet} {
	    set msg "Sequence Definition [$parser getInfo] \n\
		    is corrupt.\n\
		    Specify <pixel_x> <pixel_y> <pixel_order>\n\
		    before sequence-statements."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
	if {[set t [$parser getToken]] != "\{"} {
	    set msg "Sequence Definition [$parser getInfo]:\n\
		    \"\{\" expected, but found \"$t\"."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
	set cnt 0
	while {[set t [$parser getToken]] != "\}"} {
	    incr cnt
	    switch -exact $t {
		"wait"  {
		    #puts "wait"
		    if {[set tt [$parser getNumeric]] == ""} {
			set msg "Sequence Definition [$parser getInfo]:\n\
				\"nuseqs\" expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    if {[set sc [$parser getToken]] != ";"} {
			set msg "Sequence Definition [$parser getInfo]:\n\
				\";\" expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    
		    set wt [expr int([expr ($tt*1.0e-6)/$baseClkT])]
		    #puts "wait $tt $wt"
		    set absTime [expr $absTime + ($tt*1.0e-6)] 
		    $seqObj setBlock W [expr 1+$tickCnt] \
			    $absTime [expr $tickCnt + $wt + 1] "wait $tt"
		    set patterns [getPattern]
		    incr tickCnt $wt
		    #setTick $tickCnt
		}
		"repeat" {
		    set rbl [$seqObj setBlock R $tickCnt "0.0" 0 "repeat"]
		    if {[set cmd [readInSeqRepStat $parser $seqObj $rbl]] == $FAILED} {
			return $FAILED
		    }
		}
		default {
		    #puts "dfault"
		    if {[set useq $t] == ""} {    
			set msg "Sequence Definition [$parser getInfo]:\n\
				<useq> expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    $parser ungetToken
		    #puts $useq
		    set xbl [$seqObj setBlock X $tickCnt "0.0" $tickCnt "execute"]
		    if {[set cmd [readInSeqExecStat $parser $seqObj $xbl]] == $FAILED} {
			return $FAILED
		    }
		}
	    }
	}	
	if {$t != "\}"} {
	    set msg "Sequence Definition [$parser getInfo]:\n\
		    \"\}\" expected, but found \"$t\"."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
	#puts "readInSeq -> $rc"
	$seqObj createComment [$parser cget -comment]
	$parser configure -comment {}
    } else {
	#wesError
	set rc 0
	set msg "Sequence Definition [$parser getInfo]:\n\
		<sequence_name> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set patterns [ getPattern]] == {}} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		does not contain SIMM definitions."
	dbg "<<ERROR-Translator>> $msg"
	wesError $WAVEFORM_EMPTY "$msg"
	return $FAILED
    }
    setTick $tickCnt
    $seqObj configure -length "[format "%2.6E" $absTime] s"  
    return $seqObj
}
# ----------------------------------------------------------------------------------
# Read  Output Format Sequence Exec Statement
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readOutSeqExecStat {parser seqObj xbl} {

    if {[set useq [$parser getToken]] == ""} {    
	set msg "Sequence Definition [$parser getInfo]:\n\
		<useq> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set r [$parser getNumeric]] == ""} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<repetition> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set c [$parser getNumeric]] == ""} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<clockspeed> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set f "$path/$useq$inUSeqFileExt"
    set pars [ClassWESParser #auto]
    if {[$pars setInputFile $f] != 0} {
	if {[readUSeq [code $pars] [incr tickCnt]] == $FAILED} {
	    #wesError
	    delete object $pars
	    return $FAILED
	}
	
    } else {
	set msg "USequence Definition in [$parser getInfo] is corrupt.\n\
		cannot open file \"$f\"."
	dbg "<<ERROR-Translator>> $msg"
	wesError $CANNOT_OPEN_FILE "$msg"
	return $FAILED
    }
    delete object $pars
    set tt [expr 1+[getTicks]]
    set freq [expr $baseClkF/$c]
    set tickT [expr $c*$baseClkT]

    set absTime [expr $absTime+($baseClkT*$tt*$r*$c)]
    $xbl configure -ctrlType X -start $tickCnt -absTime $absTime \
	    -end [expr $tickCnt + $tt] -text "$useq $r $c" \
	    -freq "[format "%2.3E" $freq] Hz" \
	    -tickT "[format "%2.3E" $tickT] s"
    
    incr tickCnt [expr $tt -1]
    return $SUCCESS
}
# ----------------------------------------------------------------------------------
# Read  Output Format Sequence Repeat Statement
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readOutSeqRepStat {parser seqObj rbl} {

    set useqTime $absTime

    set tCnt [expr $tickCnt + 1]
    set stats {}
    if {[set r [$parser getNumeric]] != ""} {
	if {$r <= 0} {
	    set msg "Sequence Definition [$parser getInfo]:\n\
		    <repetition> greater than 0  expected."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
    } else {
	set msg "Sequence Definition [$parser getInfo]:\n\
		<repetition> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set t [$parser getToken] 
    while {($t != "end_repeat") && ($t != "")} {
	switch -exact $t {
	    "exec_useq" {
		set xbl [$rbl setBlock X $tickCnt "0.0" 0 "execute"]
		if {[set cmd [readOutSeqExecStat $parser $seqObj $xbl]] == $FAILED} {
		    return $FAILED
		}
	    }
	    "wait" {
		if {[set tt [$parser getNumeric]] == ""} {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    \"nuseqs\" expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
		set wt [expr int([expr $tt/$baseClkT])]
		set absTime [expr $absTime + ($tt*1.0e-6)]
		$rbl setBlock W [expr $tickCnt+1] $absTime \
			[expr $tickCnt + $wt+1] "wait $tt"
		#puts "wait $tt $wt"
		incr tickCnt $wt
		#set patterns [ getPattern]

	    }
	    "repeat" {
		set rrbl [$rbl setBlock R $tickCnt "0.0" 0 "repeat"]
		if {[set cmd [readOutSeqRepStat $parser $seqObj $rrbl]] == $FAILED} {
		    return $FAILED
		}
	    }
	    default {
		set msg "Sequence Definition [$parser getInfo]:\n\
		\"exec_useq, repeat, or wait\" expected.\n\
		but found $t"
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    }
	}
	set t [$parser getToken]
    }		
    if {$t != "end_repeat"} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		\"end_repeat\" expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    set absTime [expr ($absTime - $useqTime)*$r]
    $rbl configure -ctrlType R -start $tCnt -absTime $absTime \
	    -end [expr $tickCnt +1] -text "repeat $r"
    #$displData setBlock repeat $tCnt $absTime $tickCnt "repeat $r"
    return $SUCCESS
}
# ----------------------------------------------------------------------------------
# Read Sequence In Output Format
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::readOutSeq {parser} {
    set rc 0
    set pxSet 0
    set pySet 0
    set poSet 0
    set tickCnt -1
    set absTime 0.0
    if {[set seq  [$parser getToken]] != ""} {
	while {[set t [$parser getToken]] != "SEQUENCE_STATEMENTS"} {
	    if {$t == "pixel_x"} {
		if {[set px [$parser getNumeric]] != ""} {
		    set pxSet 1
		} else {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    <pixel_x_dim> expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	    if {$t == "pixel_y"} {
		if {[set py [$parser getNumeric]] != ""} {
		    set pySet 1
		} else {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    <pixel_y_dim> expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	    if {$t == "pixel_order"} {
		if {[set po [$parser getString]] != ""} {
		    set poSet 1
		} else {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    \"pixel_order = quoted string\" expected."
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	}
	if {!$pxSet || !$pySet || !$poSet} {
	    set msg "Sequence Definition [$parser getInfo] \n\
		    is corrupt.\n\
		    Specify <pixel_x> <pixel_y> <pixel_order>\n\
		    before sequence-statements."
	    dbg "<<ERROR-Translator>> $msg"
	    wesError $SYNTAX_ERROR "$msg"
	    return $FAILED
	}
	if {$t == "SEQUENCE_STATEMENTS"} {
	    if {$pxSet == 0} {
		set msg "Sequence Definition [$parser getInfo]:\n\
			<pixel_x> expected."
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    }
	    if {$pySet == 0} {
		set msg "Sequence Definition [$parser getInfo]:\n\
			<pixel_y> expected."
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    }
	    if {$poSet == 0} {
		set msg "Sequence Definition [$parser getInfo]:\n\
			<pixel_order> expected."
		dbg "<<ERROR-Translator>> $msg"
		wesError $SYNTAX_ERROR "$msg"
		return $FAILED
	    }
	    set seqObj [createSeq $seq $px $py $po]
	    dbg "<Translator> Processing Sequence \"$seq\""
	}
	#now read the sequence statements
	while {1} {
	    set t [$parser getToken]
	    #puts $t
	    switch -exact $t {
		"" {break}
		"SEQUENCE_END" {break}
		"exec_useq" {
		    set xbl [$seqObj setBlock X $tickCnt "0.0" $tickCnt "execute"]
		    if {[set cmd [readOutSeqExecStat $parser $seqObj $xbl]] == $FAILED} {
			set msg "Sequence Definition [$parser getInfo]:\n\
				cannot read \"exec_useq\" statement."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		}
		"repeat" {
		    set rbl [$seqObj setBlock R $tickCnt "0.0" 0 "repeat"]
		    if {[set cmd [readOutSeqRepStat $parser $seqObj $rbl]] == $FAILED} {
			return $FAILED
		    }
		}
		"wait" {
		    if {[set tt [$parser getNumeric]] == ""} {
			set msg "Sequence Definition [$parser getInfo]:\n\
				\"nuseqs\" expected."
			dbg "<<ERROR-Translator>> $msg"
			wesError $SYNTAX_ERROR "$msg"
			return $FAILED
		    }
		    set wt [expr int([expr $tt/$baseClkT])]
		    set absTime [expr $absTime + ($tt*1.0e-6)]
		    $seqObj setBlock W [expr 1+$tickCnt] \
			    $absTime [expr $tickCnt + $wt+1] "wait $tt"
		    #puts "wait $tt $wt"
		    incr tickCnt $wt
		   
		}
		default {
		    set msg "Sequence Definition [$parser getInfo]:\n\
			    \"exec_useq, repeat, or wait\" expected.\n\
			    but found $t"
		    dbg "<<ERROR-Translator>> $msg"
		    wesError $SYNTAX_ERROR "$msg"
		    return $FAILED
		}
	    }
	}
	$seqObj createComment [$parser cget -comment]
	$parser configure -comment {}
    } else {
	#wesError
	set msg "Sequence Definition [$parser getInfo]:\n\
		<sequence_name> expected."
	dbg "<<ERROR-Translator>> $msg"
	wesError $SYNTAX_ERROR "$msg"
	return $FAILED
    }
    if {[set patterns [ getPattern]] == {}} {
	set msg "Sequence Definition [$parser getInfo]:\n\
		does not contain SIMM definitions."
	dbg "<<ERROR-Translator>> $msg"
	wesError $WAVEFORM_EMPTY "$msg"
	return $FAILED
    }
    $seqObj configure -length "[format "%2.6E" $absTime] s"
    setTick $tickCnt
    #puts "rc = $rc"
    return $seqObj
}
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeInSeqExecStat {parser stat rec} {
    set spaces [format %[expr $rec*4]s " "]
    #puts $stat
    $parser putToken "$spaces[$stat cget -text]\;"
    
}
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeInSeqRepStat {parser stat rec} {
    #puts "ClassWESTrSeq::writeInSeqRepStat" 
    set spaces1 [format %[expr $rec*4 -1]s " "]
    set spaces2 [format %[expr ($rec + 1)*4 -1]s " "]
    $parser putToken "$spaces1 [$stat cget -text]"
    set str "$spaces1 \{"
    $parser putToken "$str"
    foreach cmd [$stat getBlocks] {
	#puts "$cmd"
	switch -exact [$cmd cget -ctrlType] {
	    "X" {
		writeInSeqExecStat $parser  $cmd [expr $rec + 1]
	    }
	    "R" {
		writeInSeqRepStat $parser $cmd [expr $rec + 1]
	    }
	    "W" {
		$parser putToken "$spaces2 [$cmd cget -text];"
		
	    }
	    
	}
    }
    set str "$spaces1 \}"
    $parser putToken "$str"
    
}
# ----------------------------------------------------------------------------------
# Write Sequence In Input Format
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeInSeq {parser seqObj} {
    foreach c [$seqObj getComment] {
	set cmt [lindex $c 1]
	$parser putToken "$cmt"    
    }
    $parser putToken "SEQUENCE [$seqObj cget -name]"
    set i 0
    set op ""
    $parser putToken "    pixel_x = [$seqObj cget -px]"
    $parser putToken "    pixel_y = [$seqObj cget -py]"
    $parser putToken "    pixel_order = \"[$seqObj cget -porder]\""

    $parser putToken "\{"
    foreach cmd [$seqObj getBlocks] {
	#puts "writeSeq Block $cmd"
	switch -exact [$cmd cget -ctrlType] {
	    "X" {
		writeInSeqExecStat $parser $cmd 1
	    }
	    "R" {
		writeInSeqRepStat $parser $cmd 1		
	    }
	    "W" {
		#puts "writeInSeq WAIT"
		$parser putToken "    [$cmd cget -text];"
		
	    }
	    default {
		puts "[$cmd cget -ctrlType] unknown type"
	    }
	}
	
    }
    $parser putToken "\}"
}

# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeOutSeqExecStat {parser stat rec} {
    set spaces [format %[expr ($rec*4) -1]s " "]
    #puts $stat
    $parser putToken "$spaces exec_useq [$stat cget -text]"
}
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeOutSeqRepStat {parser stat rec} {
    set spaces1 [format %[expr $rec*4 -1]s " "]
    set spaces2 [format %[expr ($rec + 1)*4 -1]s " "]
    $parser putToken "$spaces1 [$stat cget -text]"
    foreach cmd [$stat getBlocks] {
	switch -exact [$cmd cget -ctrlType] {
	    "X" {
		writeOutSeqExecStat $parser $cmd [expr $rec + 1]
	    }
	    "R" {
		writeOutSeqRepStat $parser $cmd [expr $rec + 1]
	    }
	    "W" {
		$parser putToken "$spaces2 [$cmd cget -text]"
		
	    }
	    
	}
    }
    set str "$spaces1 end_repeat"
    $parser putToken "$str"

}
# ----------------------------------------------------------------------------------
# Write Sequence In Output Format
# ----------------------------------------------------------------------------------
body ClassWESTrSeq::writeOutSeq {parser seqObj} {
    foreach c [$seqObj getComment] {
	set cmt [lindex $c 1]
	$parser putToken "$cmt"    
    }
    $parser putToken "SEQUENCE_START [$seqObj cget -name]"
    $parser putToken "    pixel_x  [$seqObj cget -px]"
    $parser putToken "    pixel_y  [$seqObj cget -py]"
    $parser putToken "    pixel_order  \"[$seqObj cget -porder]\""

    $parser putToken "SEQUENCE_STATEMENTS"
    foreach cmd [$seqObj getBlocks] {
	#puts "writeSeq Block $cmd"
	switch -exact [$cmd cget -ctrlType] {
	    "X" {
		writeOutSeqExecStat $parser $cmd 1
	    }
	    "R" {
		writeOutSeqRepStat $parser $cmd 1
		
	    }
	    "W" {
		$parser putToken "    [$cmd cget -text]"
		
	    }
	    default {
		puts "[$cmd cget -ctrlType] unknown type"
	    }
	}
	
    }
    
    $parser putToken "SEQUENCE_END"
    
}
#
# ___oOo___


