.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  bltplotRtdPlot.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH BltplotRtdPlot n "VLTSW-JAN2006 27/10/05-17:23"
.SH NAME 
BltplotRtdPlot - Plot x-lines (scans) from an Rtd image using BLT

.SH SYNOPSIS 
.nf 
.LP 
BltplotRtdPlot <widget-name> ?widget options?

.SH DESCRIPTION 
.nf 
.LP 
BltplotRtdPlot is a public itk widget class to plot scanlines of
an Rtd image using the image area. This class can be loaded as a
mega widget from the panel editor and configured (see OPTIONS).

Scanlines are specified in scanline groups which permits to set
cut levels and the scale for an entire group with one command.
Normally, scanlines which have the same "nature" are grouped, e.g.
normalized vectors which values are in the range (0, 1) are in one
group and scanlines which show raw data are placed into another
group.
Scanline groups and the individual scanlines can be configured
via a control panel and public methods (see PUBLIC METHODS).

.SH OPTIONS 
.nf 
.LP 
plotGroupList
  - defines the scanline groups and is a tcl list with the elements

              <group id> <length of scanlines> ?<xstart> <xdelta>?

    o <group id> is a group identifier which name appears in the scan
      control panel and in short help texts.
    o <length of scanlines> if set to 0 the entire scanline is displayed,
      else it defines the number of pixels to be displayed starting with
      the first pixel.
    o OPTIONALLY the following parameters can be specified:
      xstart: is the x-axis value for the first pixel
      xdelta: is the delta value of two pixels
      The values can be changed with the method set_xrange
  - Example:
      "Group0 0" "Group1 250" "Group2 0 100. 0.1"

plotScanList0
  - defines the scanline configuration for group 0 (similar plotScanList1 defines
    the scanline configuration for group 1, etc.). Each scanline
    specification consists of the following elements (delimited by quotes):

              <scanline id> <scanline number> <scanline color> ?<scanline label>?

    o <scanline id> is a scanline identifier which name appears in the scan
      control panel and in short help texts.
    o <scanline number> is the line of the array (starting with 1).
    o <scanline color> is the scanline color.
    o OPTIONALLY the following parameter can be specified:
      scanline label: is the text which appears in the short help or scanline window.
                      The default label is the scanline id.
  - Example:
      "SL1 1 yellow" "SL2 2 aquamarine {Power spectrum}"

plotHideGroup
  - groups to hide after startup (specify Group Id list). The public method
    to show/hide a group is show_group.

plotHideScan
  - scans to hide after startup (specify Scan Id list). The public method
    to show/hide a scan is show_scan.

with_scale
  - show/hide the scale widgets (in the control panel) and scrollbars
    (for the plot groups).

with_ctrl
  - show/hide the control panel. If option -with_ctrl is 0 then no
    control panel will be available. If it is 1 then the control panel
    will be build and its visibility can be changed with method show_ctrl.

show_cross
  - default: show a crosshair when moving the mouse cursor over the plot area.

show_pixmark
  - default: show a pixel marker when moving the mouse cursor over the plot area.

plotbg
  - plot background color

width
  - width of the plot and image window (in number of screen bins)

height
  - height of the plot and image window (in number of screen bins)

with_xticks
  - with x axis tick labels. Draw tick labels along the x axis.

with_yticks
  - with y axis tick labels. Draw tick labels along the y axis.

scrollbars
  - show the scrollbars of RtdImage

leftmargin
  - left margin for y-labels (i.e. the space in number of pixels
    to allocate for y-labels). Default is 0 (=dynamic allocation).
    Typical values are 40-60 pixels.
    See also method set_leftmargin.

.SH PUBLIC METHODS 
.nf 
.LP 
  public method show_ctrl { {flag 1} }
    - show/hide the control widget

  public method show_group { groupId {flag 1} }
    - show/hide a group

  public method show_scan { scanId {flag 1} }
    - show/hide a scanline

  public method show_plot { {flag 1} }
    - show/hide plot widget (toggle display of image and plot)

  public method getRtdImage { }
    - return name of the RtdImage widget (see RtdImage(n))

  public method attach { {flag 1} {camera ""} }
    - camera attach/detach. Default camera name is RTD_CAMERA.

  public method set_cuts { id low high }
    - set cut levels for a group or scan (identified by the identifier id)

  public method assign_scan { scanId groupId { groupPos 0 } }
    - assign scanline to group. This methods puts the scanline identified by 'scanId'
      into the group identified by 'groupId'. The position of the scanline within
      the group is specified by 'groupPos' (starting from 0 if it is the first
      one within the group).

  public method set_xrange { groupId {with_range 0} {xstart 1} {xdelta 1}}
    -  set x-axis range for a group
       with_range: if true, the user units xstart and xdelta are used, else
                   the pixel coords
       xstart:     the x-value at the first pixel
       xdelta:     the delta value of two pixels

  public method set_leftmargin { numpix }
    -  set left margin for y-axis labels (see option -leftmargin)

  public method print { scanId }
    - print scanline identified by 'scanId'

.SH GRAPH CONTROL 
.nf 
.LP 
  Each individual graph or a group of graphs can be controlled with
  the methods and configuration options which are available for BLT graph.
  For this BltplotRtdPlot provides for the method get_pathName. Direct
  control of graphs can be used e.g. for configuring the axis.
  See also the original man page of BLT graph which gets installed
  as bltgraph(n) after 'make install'.

  public method get_pathName { id }
    - if 'id' identifies a scanline then the pathname of the graph
      is returned.
    - if 'id' identifies a scanline group then a tcl list of all
      graphs within the group is returned.

.SH BINDINGS 
.nf 
.LP 
  The following mouse pointer bindings are used:
    - mouse button <2> click in a scanline area: zoom in
    - mouse button <3> click in a scanline area: zoom out
    - double mouse button <1> click in a scanline area: toggle hold ("freeze")
      or release x-position.
    - mouse movement in a scanline area: show x,y values at mouse x-position
      as markers
    - mouse movement in the image area: show x,y values at mouse x,y position
      in the short help area

.SH PARENT CLASS 
.nf 
.LP 
itk::Widget - not BltplotBase since the itk options
  (public variables) can be set via the panel editor.

.SH CAVEATS 
.nf 
.LP 
  - the tcl command 'update' must be avoided while the BltplotRtdPlot widget
    is programatically configured. New configurations are executed after
    the tcl shell returned to its idle point.

.SH SEE ALSO 
.nf 
.LP 
graph(n), RtdImage(n), rtdimage(n)
The original man page of BLT graph is installed as bltgraph(n)

.SH BUGS 
.nf 
.LP 
  - options which are changed via the panel editor need to
    be applied first ("Apply") before pressing "OK". If you
    just press "OK" the right mouse click does not popup the
    Configuration Panel anylonger. In this case you can save
    the panel and run the panel editor again.
  - it was reported that entering numbers into the cut level
    entries do not work after pressing 'A'. After deleting the
    whole number it should work.

