.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  inducerLine.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH InducerLine n "VLTSW-JAN2006 27/10/05-17:02"
.SH NAME 
    InducerLine - class for inducer Line object for LCU connections

.SH SYNOPSIS 
.nf 
.LP 
    InducerLine <object-name> ?-cPeriod <ticks>?

.SH DESCRIPTION 
.nf 
.LP 
    Objects of the class InducerLine are used to connect from a
    workstation Tcl-application - e.g. a Test Access Panel - to
    the inducerServer task running on a LCU environment.

    Several methods are provided to open/close a channel to a
    LCU device and to send commands to it via ioctl.

    A monitor facility to report changes in a device status is
    also available, that invokes a user-script each time a change
    is detected in the device.


.SH OPTIONS 
.nf 
.LP 
    Options represent public variables of the object and can be set with:

            <object-name> configure -<Option> <Value> ...

    The following options are publicly available:

    -cPeriod <ticks>

            The monitor-task on the LCU started by MonitorStart
            polls the device periodically with the given interval.
            The lower the value, the faster the update, but the
            higher the load of the LCU.
            Reasonable values are 20 to 100 ticks, default is 50.


.SH METHODS 
.nf 
.LP 
    Methods represent procedures on the object and are invoked with:

            <object-name> <Method> <Arguments> ...

    The following methods are publicly available:

    Open { Environment Device Mode } ==> FileDes

            Open a connection to `Device' on `Environment' in the
            open-mode given by `Mode' which can be:
                    TEST, EXCLUSIVE, SHARED, READONLY
            The file-descriptor used on the LCU is returned and stored
            inside the object for later access (-1 if open failed).
            Only one connection can be opened by the object at a time.
            Any already opened connection is automatically closed before
            opening the new one.

    Close
            Close the previously opened connection to the LCU.
            No action if no channel was opened.

    Ioctl { Cmdlist Arglist ?Sizlist? } ==> Result

            send the commands in `Cmdlist', each one with each
            argument contained in `Arglist' through the
            previously opened channel.

            Commands may be given either as numbers or driver-
            literals without the driver-specific prefix,
            e.g. "READ_BITS" (not "acroCMD_READ_BITS").
            Arguments must be given as one integer or double or as
            semicolon-separated values if more than one is needed,
            for a specific driver-command, e.g. "3;-4553;0;33".
            Each number corresponds to a struct-member.

            The `Result' consists of a comma-separated list of
            all the return-values of the executed ioctl-calls.
            The format of each list-element corresponds to the
            format of the respective argument, unless the optional
            `Sizlist' is given. Then the number of struct-members
            to be returned can be explicitly specified, where
            the index in Sizlist corresponds to the index in
            Cmdlist (very complicated, but fortunately rarely used).

            Errors reported by the called driver are ignored.
            Only the number of occured errors is returned as last
            list-element in `Result'. Detailed information about
            any errors is not provided.

    IoctlTrap { Cmdlist Arglist ?Sizlist? } ==> Result

            as before, but if the called driver reports an error,
            then a dialog-box is exposed for acknowlegement.

    MonitorStart { Cmdlist Arglist ?Sizlist? ?Script? }

            start a monitor task on the LCU which periodically
            compares the ioctl result. Everytime a change is
            is detected, `Script' is executed with the new
            ioctl-result as argument, in the same format as
            returned by Ioctl. The parameters except for Script
            are the same as in Ioctl.
            The sample period is given by the option `cPeriod'.

    MonitorStop
            stop the monitor-task (see CAUTIONS).

    MonitorCollector { Script Reply Errorflag Lastflag }

            Used internally for collecting monitor-replies from the LCU.


    Macro begin             Begin to record all IOCTL commands as macro
    Macro end               Stop recording to terminate macro
    Macro record <line>     Record into current macro
    Macro exec              Execute (redo) current macro
    Macro save              Save current macro to a selectable file
    Macro load              Load current macro from a selectable file
    Macro keepDelay 0|1     Keep original delays during "Macro exec"

.SH PROCS 
.nf 
.LP 
    Send { Env Com ?Par? }  ==> Reply

            Send the command `Com' with optional parameters `Par'
            to the inducerServer running on the LCU environment `Env'.
            The `Reply' of this command is returned.
            An error-reply is trapped with a dialog-box for confirmation
            and an empty string is returned in that case.
            Note that multiple replies are not handled!

.SH COMMONS 
.nf 
.LP 
    cTimeout <milliseconds>

            Timeout value used for message-receive in proc `Send'.
            Default is 5000 (5 seconds).

.SH FILES 
.nf 
.LP 
    The current macro is saved in "~/.[file tail $argv0].macro".

.SH CAUTIONS 
.nf 
.LP 
    The MonitorStop method stops the monitor-task on the LCU in an
    asynchronous fashion. It is not verified that the task has actually
    terminated.

.SH SEE ALSO 
.nf 
.LP 
    InducerClass(n), InducerDevice(n), inducerServer(1)

.SH TODO 
.nf 
.LP 
    Read and Write methods.

