.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  vccInfo.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH VccInfo n "VLTSW-JAN2006 27/10/05-16:59"
.SH NAME 
    VccInfo - query information from the configuration database

.SH SYNOPSIS 
.nf 
.LP 
    VccInfo <object-name>           - create object with explicit name
    set <var> [VccInfo #auto]       - create object with implicit name

.SH DESCRIPTION 
.nf 
.LP 
    VccInfo is a class to query information for configuration
    panels. The intended use is mainly internal to those panels.
    Normally a configuration database (ACCDB) is queried, but it
    is also possible in degraded modes to filter the requested
    information from derived configuration files.

    The proc `vccInit' automatically creates an object of this class with
    the name `vccInfo', see vccLib(n).

.SH INHERITANCE 
.nf 
.LP 
    Internal debugging support from VccDebug(n).

.SH PUBLIC OPTIONS 
.nf 
.LP 
    Options are set at object creation time, or later with:

            <object-name> configure <option> <value>

    The following public options are available:

    -queryModes             Set query modes and their priority

            This is a list which can contain the following modes:

            "fromaccdb" - query from the ACC database (mSQL)
            "fromfiles" - query from configuration files, i.e. files in
                    the VLT areas and standard OS files in "/etc".
            "dynamic" - query by asking servers etc.

            The order of these modes in the list determine their priority.
            The operation stops after the first successful query.
            usually it is: {fromaccdb fromfiles dynamic}

    -complain               Specifies behaviour for unsuccessful query

            If this is off then VccInfo returns only an empty string
            when no valid result could be found for a query.
            If it is on, then a Tcl error is generated.
            The default if 0 (off).

.SH METHODS 
.nf 
.LP 
    Methods are called with:

            (a) vccInfo <method> <args>
            (b) <object-name> <method> <args>

    If the object has been created with `vccInit' then (a) is used.
    For other object-names the general way of (b) is used.

    Some methods can be called with either one item or a list of items as
    argument and return correspondingly one result or a list of results.

    Host-based query:
    -----------------

    GetHosts ?-all? ?-attic? ?<Filter> ...?

            Returns the list of hosts matching one of the filters.
            Default-filter is `*' for "all".
            By default all items with an `attic' flag are not returned.
            The `-all' switch includes also the attic.
            The `-attic' switch returns only the attic.

    GetByHost <Host> <Attrs>

            Returns the database value of Attrs for Host.
            Attr: ipAddr, cpuName, hostType, ...


    Environment-based query:
    ------------------------

    GetEnvs ?-all? ?-attic? ?<Filter> ...?

            Returns list of environments matching one of the filters.
            Default-filter is `*' for "all".
            By default all items with an `attic' flag are not returned.
            The `-all' switch includes also the attic.
            The `-attic' switch returns only the attic.

    GetByEnv <Env> <Attrs>

            Returns the database value of Attrs for Env.
            Attr: hostName, hostName0/1/2, tcpPort, processList, ...

    GetBootEnvByEnv <Env>   Returns assigned WS boot-environment
    GetProcsByEnv   <Env>   Returns process-list of environment
    GetIPaddrByEnv  <Env>   Returns IP-address of host of environment


    <Filters>       glob-style filter or list of, e.g. "w*qs" {w* l*}
    <Host>          host-name
    <Env>           environment-name
    <Attrs>         attribute-name or list of attribute-names

.SH PROCS 
.nf 
.LP 
    None.

.SH ENVIRONMENT 
.nf 
.LP 
    Two optional environment variables control the database access:

    ACC_HOST - shell variable that defines the ACC database server host;
               the local host is assumed - resulting in faster access -
               if this equals to either:
               - the name returned by the command `hostname'
               - the string "localhost"
               - or an empty string.
               Note that an IP address in dot-notation also works,
               but will not be checked for the local host, and therefore
               will always result in a somewhat slower remote connection.

    If ACC_HOST is undefined or unaccessible, then
    VccInfo tries to provide a limited set of functions based on
    degraded query methods. `fromaccdb' will be
    removed from the `-queryModes' option in that case.

.SH FILES 
.nf 
.LP 
    None if ACC_HOST is defined and accessible.
    Otherwise file-based querying is performed on various Unix and
    VLT standard files.

.SH EXAMPLES 
.nf 
.LP 
    vccInit ;# creates object `vccInfo', may enter local CCS environment
    set wsEnvs  [vccInfo GetEnvs -all w*] ;# query all WS-environments
    set lcuEnvs [vccInfo GetEnvs l*]      ;# query LCU-environments
    vccExit ;# leave vcc and exit CCS environment

.SH SEE ALSO 
.nf 
.LP 
    vcc(n), vccLib(n), VccDebug(n)

