.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  vccSelectButton.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH VccSelectButton n "VLTSW-JAN2006 27/10/05-16:59"
.SH NAME 
    VccSelectButton - class for a menubutton widget with special properties

.SH SYNOPSIS 
.nf 
.LP 
    VccSelectButton <pathName> ?-<option> <value> ...?
    <pathName> configure ?-<option> <value> ...?
    <pathName> <method> ?<arg> <arg> ...?

.SH DESCRIPTION 
.nf 
.LP 
    The VccSelectButton command creates a new window (given by the pathName
    argument) and makes it into a corresponding widget. Additional options,
    described below, may be specified on the command line or in the option
    database to configure aspects of the widget.

    The VccSelectButton command returns its pathName argument. At the time
    this command is invoked, there must not exist a window named pathName,
    but pathName's parent must exist.

    The VccSelectButton widget consists of a menubutton and a connected
    menu. The menubutton has by default the width 1 and the label `+'.
    The menu can be generated from the elements of a list representation.
    The text of the menu-items correspond to the elements of the list.
    Leaf-menu-items invoke a callback-script when selected, the text
    of the menu-items is passed to this script as argument.
    If the number of entries exceed a maximum then a continuation-menu
    is automatically appended ("more...").

.SH INHERITANCE 
.nf 
.LP 
    All options of menubutton(n) are inherited.
    Deviating default values are:

    -width          Default: 1
    -text           Default: +
    -relief         Default: raised

.SH PUBLIC OPTIONS 
.nf 
.LP 
    The following options are specific to objects of this class:

    -menuOption     Default: ""

            Specifies how the menu shall be created from `menuItems':

            "-nopropagate"  inhibits the propagation of callback-arguments
                            to the child-entries, i.e. only the label of
                            the leaf item is used as argument, but not the
                            concatenation with all its menu parents.

            If no option is given, then the propagation is enabled
            The menu is regenerated whenever this option is changed.

    -menuItems      Default: {}

            Contains a list of elements to make the menu from.
            It will consist of entries of the type specified in `menuType',
            except when one of the following is given:

            - an empty string (except as the first list element) generates
              a separator

            - if an element is itself a list, and the first element is
              an empty string, then the second element of this sub-list
              denotes the label of a cascade-entry, while the rest of the
              list denotes the elements of the corresponding sub-menu.
              (This somewhat complicated construction makes it possible
               to differentiate a list from a string containing spaces;
               therefore menu labels with spaces are permitted.)

            The menu is regenerated whenever this option is changed.

    -menuType       Default: "radiobutton"

            Specifies the type of leaf menu entries. All leafs can only
            have the same type, which must be "radiobutton" or "command".
            The menu is regenerated whenever this option is changed.

    -menuMaxLen     Default: 25

            Specifies the maximum number of menu-entries on one level.
            If that count is exceeded, then a "more..." sub-menu is
            automatically created, where the further entries will go in.
            The specified count is excluding the "more..." itself.
            The menu is regenerated whenever this option is changed.

    -menuAcceleratorToken   Default: ""

            If an element in `menuItems' contains this character, then
            the text after it is regarded as accelerator. It can therefore
            not be part of the label or accelerator itself. Empty string
            means that no special handling of accelerators is done.
            The menu is regenerated whenever this option is changed.

    -command        Default: none

            is a script (usually a procedure or method) to be called when
            a leaf-item of the menu is selected. The text of the item is
            passed as parameter to the script.
            If "-nopropagate" is specified in `menuOption' then the
            only the label of the leaf menu entry is passed as argument,
            otherwise the concatenation of with all labels of the cascaded
            parents of the leaf.
            Cascade-entries itself do not have callbacks.
            No command is performed is this option is an empty string.

    -variable       Default: derived from widget name ($this)

            Name of a variable connected to the menu items.

    -state          Default: normal

            This option is propagated to menubutton(n), see there.

.SH METHODS 
.nf 
.LP 
    The VccSelectButton command creates a new command <pathName>.
    This command may be used to invoke various operations on the widget.
    It has the following general form:

            <pathName> <method> ?<arg> <arg> ...?

    <method> and the <arg>s determine the exact behavior of the command.
    The following commands are possible for VccSelectButton widgets:


    <pathName> MakeMenu <List> ?<Option>? ?<Menu>?

    Assembles the menu connected to the button from a user-defined list.
    The menubutton is activated to indicate that the menu has changed.
    It returns the widget name of the first-level menu.

    List    see PUBLIC OPTION `menuItems'.

    Option  is prefixed to the parameter of the Callback. This is useful
            for cascaded menus and normally only necessary for recursive
            calls but not for user calls, except for the
            special value `-nopropagate', which inhibits the propagation of
            callback-parameters to the child-entries.
            The argument is optional, default is an empty string.
            See also PUBLIC OPTION `menuOption'.

    Menu    is only used for recursion and specifies the widget-name
            of the target menu.
            All entries of this menu - if any - are deleted before.
            Any cascaded sub-menus are automatically created.

    All entries of the top-level menu - if any - are deleted.
    Any already existing sub-menu is destroyed.
    Menus having sub-menu should be `destroy'ed before re-generating them,
    otherwise unused sub-menus may occupy memory.
    Sub-menus are built by internal recursive calls of this procedure.
    Note that the `List' is stored inside the object only for the first
    (non-recursive) call, i.e. when `Menu' is empty.

.SH COMMON PROCEDURES 
.nf 
.LP 
    None.

.SH DEFAULT BINDINGS 
.nf 
.LP 
    All default bindings are inherited from menubutton(n) and menu(n).

.SH EXAMPLES 
.nf 
.LP 
    set menuList {a {} { {} "b b" d e} "c c c" d|ACC}
    VccSelectButton .a -width 10 -text Attribute
    .a configure -menuItems $menuList -menuAcceleratorToken "|"

    This would produce the following cascaded menu:

            +------------+
            | * a        |
            |  --------- |+-----+
            |   b b     >|| * d |
            |            || * e |
            | * c c c    |+-----+
            | * d     ACC|
            +------------+

.SH SEE ALSO 
.nf 
.LP 
    menubutton(n), menu(n), options(n)

