.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  vccSelector.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH VccSelector n "VLTSW-JAN2006 27/10/05-16:59"
.SH NAME 
    VccSelector - class for a selection widget

.SH SYNOPSIS 
.nf 
.LP 
    VccSelector <pathName> ?-<option> <value> ...?
    <pathName> configure ?-<option> <value> ...?
    <pathName> <method> ?<arg> <arg> ...?

.SH DESCRIPTION 
.nf 
.LP 
    The VccSelector command creates a new window (given by the pathName
    argument) and makes it into a VccSelector widget. Additional options,
    described below, may be specified on the command line or in the option
    database to configure aspects of the widget.

    The VccSelector command returns its pathName argument. At the time
    this command is invoked, there must not exist a window named pathName,
    but pathName's parent must exist.

    A VccSelector widget consists of a label, an input entry, a menubutton
    with connected menu, and build-in properties to modify them.

    The menubutton is of class VccSelectButton.
    The button has always the width 1 and is labelled with `+'.
    The contents of the menu can be configured using a provided method.

.SH INHERITANCE 
.nf 
.LP 
    VccSelector inherits completion functions from VccCompletion(n).
    All properties of UifWidget(n) are inherited.

.SH PUBLIC OPTIONS 
.nf 
.LP 
    The following options are specific to objects of this class:

    -labelText      String displayed before the entry.
                    Default: "Label:"

    -labelWidth     Width of the label (anchor is "e").
                    Default: (autosize)

    -entryWidth     Width of the input entry.
                    Default: 10

    -entryState     State of the input entry (normal | disabled)
                    Default: normal

    -variable       Name of a variable connected to the input entry.
                    It will be set _after_ `command' has been evaluated.
                    Default: none

    -command        Script to be executed when a menu item is
                    selected or <Return> is pressed in the entry.
                    The selected/input string is passed as argument to
                    this script. Note that if a `variable' is configured,
                    it will have still the _old_ value at this time!
                    Default: none

    -userMenuEnable If this is set to 1 then items entered via the
                    entry can be appended to the menu upon <Return>.
                    This requires that `command' is set, and that the
                    the return value of the executed script contains
                    the value to be appended (usually the same as the
                    input argument). If an empty string is returned,
                    then nothing is appended, allowing to sort out
                    invalid user inputs.
                    Default: 0

    -menuMaxLen     See VccSelectButton(n). Default: 25

    -menuType       See VccSelectButton(n). Default: radiobutton

    -menuOption     See VccSelectButton(n). Default: ""

    -state          Propagated to entry(n) and menubutton(n), see there.
                    Default: normal

    The next options are to define events that invoke special functions.
    Each can also be a list of keys etc. that trigger the same action.
    An empty string means that no event is connected to the action.
    NOTE: the focus events require Tk4 or higher!

    -evtInvoke      to invoke the command   Default: <Return>
    -evtComplete    to complete the entry   Default: {<Escape> <Tab>}
    -evtFocusNext   to move focus forward   Default: {} ==> no event
    -evtFocusPrev   to move focus backward  Default: {} ==> no event

.SH METHODS 
.nf 
.LP 
    The VccSelector command creates a new command whose name is <pathName>.
    This command may be used to invoke various operations on the widget.
    It has the following general form:

            <pathName> <method> ?<arg> <arg> ...?

    <method> and the <arg>s determine the exact behavior of the command.
    The following commands are possible for VccSelector widgets:


    <pathName> MakeMenu <itemList> ?<option>?

            Assembles the menu connected to the button from a user-defined
            list. If the option `userMenuEnable' is set then the entries
            of the user-defined menu are appended to <itemList>.
            It maps into the corresponding method of `VccSelectButton'.
            See VccSelectButton(n) for details.

.SH DEFAULT BINDINGS 
.nf 
.LP 
    <Return>        to invoke the command
    <Tab>           to complete the entry
    <Escape>        to complete the entry

    See PUBLIC OPTIONS to change event bindings.

.SH EXAMPLES 
.nf 
.LP 
    VccSelector $cv.wLcuEnv -command vccEvhSelectLcuEnv -userMenuEnable 1 \\
            -variable gvar(vLcuEnv) -entryWidth 8 -labelWidth 15 \\
            -labelText {LCU Environment:} \\
            -helptext {LCU Environment which shall be configured} \\
            -evtInvoke {<Return> <FocusOut>} \\
            -evtComplete <Tab>
    $cv.wLcuEnv MakeMenu {lte1 lte2 lte3}

.SH SEE ALSO 
.nf 
.LP 
    VccSelectButton(n), VccCompletion(n),
    frame(n), label(n), entry(n), options(n)

