.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  istMotor3State.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH istMotor3State n "VLTSW-JAN2006 27/10/05-17:12"
.SH NAME 
istMotor3State - three state button area widget for controlling motor

.SH SYNOPSIS 
.nf 
.LP 
istMotor3State  <widget-name>

.SH DESCRIPTION 
.nf 
.LP 
The istMotor3State is dedicated widget for controlling motor. It inherits
the basic facilities provided by ist3StateMenubutton, but enhances it with
specific features to handle motor control states (stored in database) such
as:
    - 'state' database attribute which contains the current state of the
      controlling device. The color of the label changes to reflect the
      current state according to the following table:
           state = 1 (i.e. OFF)       --> RED
           state = 2 (i.e. STANDBY)   --> GREEN
           state = 3 (i.e. ONLINE)    --> Default color
    - 'status' which contains the current position index of the motor or
      'statusElt' which contains the current position name of the motor.
      The widget is updated to show this position using either the
      'acceptBg' color or 'nowBg' color according to the 'motion' status
      (see below).
    - 'motion' database attribute which informs when motor is moving. When
      the motor is moving (motion status = 2), the background color of the
      current position is set to the 'nowBg' color while the background
      color is set to the 'acceptBg' color when motor is standing. In
      addition, the foreground color of the label is set to green when the
      motor is moving.
    - 'enabled' database attribute. The widget state is set according to
      this attribute, i.e. it is enabled when 'enabled' is set to 1, and
      disabled otherwise.

The user create a widget of this class (called instance) by :
  istMotor3State  <widget-name> <config-parameters>

where :
  <widget-name>         is any valid TK widget name
  <config-parameters>   sequence of widget resources

Standard widget resources can be set using :
   <widget-name> configure <resource-name> <value>

The current value of the widget resources is returned by :
   <name-of-instance> cget <resource-name>

.SH PUBLIC METHODS 
.nf 
.LP 
  statusDbUpdate
     Callback attached to the 'status' database event. It updates the
     current position of the motor.

  statusEltDbUpdate
     Callback attached to the 'statusElt' database event. It updates the
     current position of the motor.

  motionStatusDbUpdate
     Callback attached to the 'motion' database event. It updates the
     background color of the current position with 'nowBg' color when motor
     is moving or 'acceptBg' color when motor is standing.

  enabledDbUpdate
     Callback attached to the 'enabled' database event. It enables or
     disables widget acoording the current 'enabled' value.

  stateDbUpdate
     Callback attached to the 'state' database event. It updates the
     background color of the label according to the state of the controlling
     device (RED when OFF, GREEN when STANDBY and 'default' when ONLINE).

WIDGET-SPECIFIC OPTIONS
  -statusDb       <status db point>  - name of the database attribute where
                                       is stored the current position index
                                       of the motor.

  -statusEltDb    <status db point>  - name of the database attribute where
                                       is stored the current position name
                                       of the motor.

  -motionStatusDb <motion db point>  - name of the database attribute where
                                       is stored the motion status of the
                                       motor.

  -enabledDb      <enabled db point> - name of the database attribute where
                                       is stored the 'enable' flag of the
                                       controlling device.

  -stateDb        <state db point>   - name of the database attribute where
                                       is stored the state of the device.

  -movingBg       <color>            - foreground color of the label to
                                       inform user when motor is moving.

.SH EXAMPLES 
.nf 
.LP 
  Create an instance

  istMotor3State .myFunction  -label myFunction -current Open \\
                      -elements {Open Closed J K} -keyword "INS.OPTI1" \\
                      -selectCB funcSelect -type menu \\
                      -statusDb <alias>slider:mcmStatus.posIndex\\
                      -motionStatusDb <alias>slider:mcmStatus.motionStatus \\
                      -enabledDb <alias>slider.enabled \\
                      -stateDb <alias>slider.state

  proc funcSelect { keyword value } {
       puts "Function: $keyword has value: $value"
  }

.SH CAUTIONS 
.nf 
.LP 
The options '-statusDb' and '-statusEltDb' are used for the same purpose;
i.e. update the widget according to the current motor position, either using
the position index ('-statusDb' option) or the position name ('-statusEltDb'
option). They SHOULD NOT be used at the same time.

