.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqAlrm.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_alrmAttach n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_alrmAttach, seq_alrmDetach, seq_alrmList - alarm management commands

.SH SYNOPSIS 
.nf 
.LP 
\fBseq_alrmAttach\fR \fI?<filter>? <dbAddress> <script>\fR

\fBseq_alrmDetach\fR \fI<alarmHandle>\fR

\fBseq_alrmList \fI?-env? ?-branch? ?-script? ?-status? ?-all?\fR

.SH DESCRIPTION 
.nf 
.LP 
\fBseq_alrmAttach\fR:
  attaches an alarm to a database branch/point:

    <filter>   : allows to select alarms according to certain severity level
       criteria or scope, as defined in the db-definition file containing
       these alarm points. It can be:
       \fI-normal\fR (pseudo-state, meaning inactive alarm)
       \fI-warning\fR
       \fI-warning_ack\fR
       \fI-serious\fR
       \fI-serious_ack\fR
       \fI-fatal\fR
       \fI-fatal_ack\fR
       \fI-local\fR
       \fI-global\fR
       \fI-all\fR (all of the above)
       Multiple filters can be specified, in any mixture of upper- and
       lower-case letters.
       Remark that the current implementation of the alarm-functionality in
       CCS-lite does not support these filters. I.e. independent of the
       filter chosen, it will always select all alarms (equivalent to
       setting the \fI-all\fR filter).
    <dbAddress>: the full path of the database branch/point, in the usual
       symbolic address notation, i.e. starting with @, followed by the
       environment name and then the branch/point name.
    <script>   : the script to execute when an alarm occurs; this script
       will be evaluated at global level.  The following substitutions will
       be performed on this script before evaluation:
       \fI%A\fR : name of the attribute causing the event
       \fI%B\fR : name of the branch causing the event
       \fI%D\fR : value of the attribute causing the alarm
       \fI%E\fR : name of the environment causing the event
       \fI%L\fR : severity level of the alarm (as defined in the corresponding
                  dbl file)
       \fI%M\fR : the full message as displayed by the Alarm Display utility. It
              contains the time, scope, attribute name and value, and
              severity level for the incoming alarm
       \fI%S\fR : scope of the alarm ("local" or "global", as defined in the
                  corresponding dbl file)
       \fI%T\fR : attribute's db-type (numeric value)
       \fI%%\fR : a single % character
       Whenever the substitution on the original script or the evaluation
       of the resulting script yields an error, the corresponding alarm
       will be detached, and the error will be logged and also signalled via
       \fITcl_BackgroundError\fR.

\fBseq_alrmDetach\fR:
    will detach a previously attached alarm from a database branch, and
    delete the alarm handler

    <alarmHandle>  : the alarm handle, as returned by \fIseq_alrmAttach\fR.

\fBseq_alrmList\fR:
    lists the attached alarms with their properties as requested:

    -filters: the list of filter options associated to this alarm
    -symaddress: the full symbolic address to which this alarm is attached
    -script : list the script that will be evaluated when the alarm occurs
    -env    : tell to which environment this alarm is attached
    -branch : give the branch name to which this alarm is attached
    -status : tells if the asynchronous alarm-attachment ("attached") has
              in the meantime been acknowledged ("open") by the alarm
              system. Also an alarm which was subject of a seq_alrmDetach
              command will be shown as "attached" as long as the alarm
              system did not confirm this detachment; after that it will no
              longer be in the list of attached alarms.
    -all    : shorthand for the above 6 properties

    The result string contains per event a sublist with the elements as
    requested.  They will be listed in the order as given above, i.e.
    independent of the order of these options on the command line, and
    preceded by the alarm handle. Remark that these first 3 options
    reproduce the arguments given to the seq_alrmAttach command.

.SH ENVIRONMENT 
.nf 
.LP 
The seq_alrm* commands have corresponding entries in the global Tcl array-
variable \fBseq_debug\fR, which controls the printing of debug information (see
Seq_Init(3)). On top of that, if bit 3 of \fBseq_debug(seq_msgDispatch)\fR is set,
there will be additional info printed on incoming alarm messages.

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  For seq_alrmAttach the result string
    contains the alarm handle to use when referring to this this event with
    any other \fIseq_alrm*\fR command.  For \fIseq_alrmList\fR the result string is
    a list of sublists (one sublist per alarm, with elements as requested).
ERROR return if one of the following conditions is met (with corresponding
error message here in bold-face):
  - \fBinvalid filter: "<filter>" should be -normal, -warning(_ack),
    -serious(_ack), -fatal(_ack), -local, -global or -all\fR;
  - \fBwrong # args : should be "seq_alrmAttach ?<filter>? <@envName:dbBranch>
    <script>"\fR when the \fIseq_alrmAttach\fR command was given with the wrong
    number of arguments;
  - \fBwrong symbolic address: "<dbAddress>" should be in form
    "<@envName:dbBranch>"\fR;
  - \fBcannot attach alarm to <dbAddress>\fR if the call to alrmAttach() returned
    an error;
  - \fBwrong # args : should be "seq_alrmDetach <alarmHandle>"\fR when the
    \fIseq_alrmDetach\fR command was given with the wrong number of arguments;
  - \fBinvalid alarm handle "<alarmHandle>" (no alarms associated with it)\fR
    if a command was given with an <alarmHandle> which is not or no longer
    valid
  - \fBcannot detach <alarmHandle>\fR if the call to alrmDetach() returned an
    error;
  - \fBwrong # args : should be "seq_alrmList ?-filters? ?-symaddress?
    ?-script? ?-env? ?-branch? ?-status? | ?-all?"\fR when the \fIseq_alrmList\fR
    command was given with the wrong number of arguments;
  - \fBbad option: "<option>" should be -filters, -symaddress, -script, -env,
    -branch, -status or -all\fR if a wrong <option> is passed to the
    \fIseq_alrmList\fR command;

.SH CAUTIONS 
.nf 
.LP 
The current implementation of the alarm-functionality in CCS-lite does not
support filters (see \fIseq_alrmAttach\fR). I.e. independent of the filter
chosen, CCS-lite will always select all alarms (equivalent to setting the
\fI-all\fR filter). Neither are there errors reported back by CCS-lite when
selecting a filter other than \fI-all\fR.
The \fIseq_alrmAttach\fR command does not prevent the use of these filters under
CCS-lite either; this implies that an eventual future implementation of this
filtering in the CCS-lite alrmAttach(3) function will be automatically
supported by the \fIseq_alrmAttach\fR command.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ... (<alias>my.attr is a float with value 1.23, belonging to an alarm class)
seqWish>seq_alrmAttach -all @wte16: {set newVal %D; set b 5}
alarm2
seqWish>set b 32
seqWish>.....
 ..... (<alias>my.attr gets modified to 32.1, generating an alarm)
seqWish>set b
5
seqWish>set new
32.1
seqWish>.....
seqWish>seq_alrmDetach alarm2
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
\fITcl and the Tk toolkit\fR, John K. Ousterhout, ISBN 0-201-6337-X
alrmAttach(3), alrmDetach(3), alrmParseMsg(3)


