.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqCcs.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_ccsInit n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_ccsInit, seq_ccsExit, seq_ccsAsyncInput - interface to ccsInit, ccsExit
                    and the asynchronous input of messages

.SH SYNOPSIS 
.nf 
.LP 
seq_ccsInit ?<procName>?

seq_ccsExit

seq_ccsAsyncInput open|close

.SH DESCRIPTION 
.nf 
.LP 
seq_ccsInit:
    calls ccsInit to register the Sequencer in CCS

    <procName> : string containing name under which to register the Sequencer.

    seq_ccsInit sets also the global Tcl variable "seq_ccsStatus" to
    "CLOSED".  If <procName> is not given, it defaults to the value of the Tcl
    variable seq_ccsProcName.
    If all went well, the global Tcl variable "seq_ccsEnvName" will be set
    to the name of the local environment.

seq_ccsExit:
    calls ccsExit and unsets the global variable "seq_ccsStatus".
    Whenever seq_ccsExit command completes successfully, internal lists
    containing info about commands with pending replies are cleared,
    i.e. seq_msgList will return an empty list.  If the seq_ccsExit command
    is given within the processing of a SCRIPT command (see
    seq_msgDispatch), it will inform the originator(s) of this command that
    it is exiting, sending a last reply with an error number 1.

seq_ccsAsyncInput:
    enables or disables the treatment of commands that are sent to the
    Sequencer via the CCS message system.

    open  : enable the input of commands
    close : disable the input of commands

.SH ENVIRONMENT 
.nf 
.LP 
RTAPENV:
   an environment variable, used to determine under which CCS-environment
   the application needs to registered by the seq_ccsInit command. See also
   the CAUTIONS section.

seq_ccsEnvName:
   a read-only global Tcl variable, initialized after the first seq_ccsInit
   command to the string-value given by CCS as the local environment name.

seq_ccsProcName:
   a read-only global Tcl variable, initialized by seqInit to seqNAME as
   defined in seq.h, and linked to the global C-variable myProcName.
   This variable is used as the default name under which to register the
   Sequencer with CCS.  If the seq_ccsInit command includes the <procName>
   parameter, seq_ccsProcName will be updated accordingly after a
   successful ccsInit.

seq_ccsStatus:
   a read-only global Tcl-variable, which reflects the status of
   initialization the Sequencer went through; values can be:
   - CLOSED : "seq_ccsInit" has been given before
   - OPEN:    "seq_ccsAsyncInput open" was given (after "seq_ccsInit"
              succeeded)
   - if the variable does not exist, "seq_ccsInit" was never given before
     or "seq_ccsExit" was given after "seq_ccsInit".

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred, accompanied by
  - for seq_ccsInit the Tcl filedescriptor used for the message queue
    monitoring
  - an empty reply for all other commands.

ERROR if one of the following conditions is met (corresponding error
message is here between quotes):
  - 'wrong # args: should be "seq_ccsInit ?<procName>?"', 'wrong # args:
    should be "seq_ccsExit"' or 'wrong # args: should be "seq_ccsAsyncInput
    open|close"' if the command was given with an improper amount of
    arguments
  - 'ccsInit failed' if ccsInit failed
  - 'ccsGetMyProcId failed' if ccsGetMyProcId failed
  - '"seq_ccsInit" was given before' if more than one seq_ccsInit command
    is given without a seq_ccsExit in between (and remark that CCS expects
    only one pair of init/exit commands per application).
  - message left by Tcl_LinkVar, if the Tcl global variable seq_ccsStatus
    or seq_ccsEnvName could not be linked to a C variable after ccsInit
    succeeded.
  - 'ccsExit failed' if ccsExit failed
  - message left by Tcl_UnsetVar2, if the global variable seq_ccsStatus
    could not be unset after ccsExit succeeded.
  - 'bad argument: "<arg>" should be open or close' for a seq_ccsAsyncInput
    command with an improper argument
  - 'use seq_ccsInit first' if seq_ccsAsyncInput is given before
    seq_ccsInit
  - 'call to seqMonitorQ failed' if seq_ccsAsyncInput got an error returned
    by this routine.
CCS errors are also logged via the CCS error logging system.

.SH CAUTIONS 
.nf 
.LP 
Under CCS-lite, the current setting of RTAPENV is taken whenever the first
CCS-function gets called, for CCS internal needs. If this first CCS-function
is not a ccsInit(), this value of RTAPENV will anyway be used later when
ccsInit() gets called, independent of RTAPENV's value at that later time.
This means that e.g. in the \fIwtctest\fR environment, issuing first a
\fI"seq_errResetStack"\fR command followed by a \fI"set env(RTAPENV) xyz"\fR and a
\fI"seq_ccsInit"\fR command, will lead to a ccsInit() in the \fIwtctest\fR
environment instead of the \fIxyz\fR environment.
This is not inherent to seq, but rather to CCS-lite. In fact, to enable an
effective modification of the environment variable RTAPENV within a
Sequencer script (e.g. based on runstring options), the Sequencer's internal
initialization does not call any CCS-function under CCS-lite, thereby
shifting the responsibility to do things in the proper order to the
application. Remark that there are several seq_* commands, apart from the
obvious ones, that (could) lead to calls of CCS-functions, e.g. seq_catch,
seq_errLog, seq_logTclErrors and seq_errDisplay.

Repeated seq_ccsExit commands will normally succeed as repeated ccsExit()
calls succeed. Use the Tcl-variable seq_ccsStatus if you want to know what
CCS-state the Sequencer-shell is in.

The handling of the seq_ccsInit command includes a rtMonitorQueue() call.
Due to what seems to be a bug in the latter, after messages came on the
queue this file descriptor will not be released by seq_ccsExit.  In other
words, seq_ccsInit/seq_ccsExit cycli may lead to high-numbered file
descriptors, and even to some memory leaking. Remark that ccsInit/ccsExit
cycli are not officially supported by CCS.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the 'seqWish>' prompt, and all
replies on a new line)

 ....
seqWish>seq_ccsInit
file5
seqWish>echo $seq_ccsStatus
CLOSED (*)
seqWish>seq_ccsInit
Error: "seq_ccsInit" was given before
 .....
seqWish>seq_ccsAsyncInput open
seqWish>echo $seq_ccsStatus
OPEN (*)
seqWish>seq_ccsExit
seqWish>echo $seq_ccsStatus
can't read "seq_ccsStatus": no such variable
seqWish>seq_ccsExit
seqWish>....

(*): These 3 characters are only there to avoid that docDoManPages
interprets the words CLOSED and OPEN as new sections of the manpage

.SH SEE ALSO 
.nf 
.LP 
 'Tcl and the Tk toolkit', John K. Ousterhout, ISBN 0-201-6337-X
seq_msgDispatch(n), seqCcsInit(3), seqCcsExit(3), ccsInit(3), ccsExit(3),
seqInit(3)


