.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqDbGetNames.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_dbGetAttrNames n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_dbGetAttrNames, seq_dbGetFamilyNames, seq_dbGetFieldNames,
seq_dbGetAlias -
     list the attributenames of a point, get the symbolic addresses of
     parent and children, list the field names of a table or get the
     alias name of a point

.SH SYNOPSIS 
.nf 
.LP 
seq_dbGetAttrNames <pointName>

seq_dbGetFamilyNames <pointName> <view>

seq_dbGetFieldNames <attrName>

seq_dbGetAlias <pointName>

.SH DESCRIPTION 
.nf 
.LP 
seq_dbGetAttrNames:
  will retrieve the list of names of the attributes belonging to a point

  <pointName> : string containing a symbolic address of the point whose
                attributenames we want to know

  seq_dbGetAttrNames will return a single list containing the names of
  the attributes.

seq_dbGetFamilyNames:
  will retrieve the symbolic addresses of the parent and the children of
  a point.

  <pointName> : string containing a symbolic address of the point whose
                family addresses we want to know

  <view>      : view specifier for the addresses; can be either ABSOLUTE,
                ALIAS or RELATIVE (both upper- and lowercase allowed)

  seq_dbGetFamilyNames will return a list containing the following values:
  1. the parent's address
  2. the addresses of the children, one list-element per child.
  If the point does not have any children, this list will contain only one
  element, i.e. the parent's address.

seq_dbGetFieldNames:
  will retrieve the list of names of the fields of a TABLE attribute

  <attrName> : string containing a full symbolic address of the attribute
               whose fieldnames we want to know

  seq_dbGetFieldNames will return a single list containing the names of
  the fields.

seq_dbGetAlias:
  will retrieve the alias name of a point

  <pointName> : string containing a symbolic address of the point whose
                alias we want to know

  seq_dbGetAlias will return the alias name.

.SH ENVIRONMENT 
.nf 
.LP 
no environment nor Tcl variables are read nor set

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  In this case the result string contains
    the attribute, family or alias names, as indicated above.
ERROR return if one of the following conditions is met (with corresponding
error message here between quotes):
  - "wrong # args : should be "seq_dbGetAttrNames <pointName>": the
    command was given with the wrong number of arguments
  - "wrong # args : should be "seq_dbGetFamilyNames <pointName> <view>":
    the command was given with the wrong number of arguments;
  - "wrong # args : should be "seq_dbGetFieldNames <attrName>": the
    command was given with the wrong number of arguments;
  - "wrong # args : should be "seq_dbGetAlias <pointName>": the command
    was given with the wrong number of arguments;
  - "wrong view specifier: should be ABSOLUTE, ALIAS or RELATIVE"
  - "cannot get attribute names of "<pointName>": dbGetAttrNames returned
    an error for this argument
  - "cannot get family info on "<pointName>": dbGetFamilyNames returned
    an error for this argument
  - "cannot get field names of "<pointName>": dbGetFamilyNames returned
    an error for this argument
  - "cannot get alias name of "<pointName>": dbGetAlias returned an error
    for this argument

.SH CAUTIONS 
.nf 
.LP 
Unfortunately, point and attribute names can in principle contain spaces.
If this is the case, such name will be surrounded by braces, as any
proper Tcl-list element.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>seq_dbGetAttrNames :PARAMS:SCALARS
scalar_double scalar_float scalar_int32 scalar_logical scalar_string128
seqWish>seq_dbGetFamilyNames :PARAMS:SCALARS absolute
:PARAMS
seqWish>seq_dbGetFamilyNames :PARAMS absolute
: :PARAMS:SCALARS :PARAMS:TABLES :PARAMS:VECTORS
seqWish>seq_dbGetFamilyNames :PARAMS alias
<alias>root <alias>SCALARS <alias>TABLES <alias>VECTORS
seqWish>seq_dbGetAttrNames :PARAMS:TABLES
full_table
seqWish>seq_dbGetFieldNames :PARAMS:TABLES.full_table
logical int8 uint32 float string4 string32
seqWish>seq_dbGetAlias :PARAMS:TABLES
<alias>TABLES
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqDbGetFamilyNames(3)

