.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqDbList.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_dbListCreate n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_dbListCreate, seq_dbListAdd, seq_dbListRemove, seq_dbListDestroy,
seq_dbListPut, seq_dbListList
      - commands for the handling of db multi-read/write lists

.SH SYNOPSIS 
.nf 
.LP 
seq_dbListCreate <environment> <access>

seq_dbListAdd <listId> <attrSymAddr1> ?<attrSymAddr2>? ...

seq_dbListRemove <listId> <attrSymAddr1> ?<attrSymAddr2>? ...

seq_dbListDestroy <listId>

seq_dbListExtract <listId> <attrSymAddr1> ?<attrSymAddr2>? ...

seq_dbListPut <listId> <attrSymAddr1> <value1> ?<attrSymAddr2> <value2>? ...

seq_dbListList ?<environment>?

.SH DESCRIPTION 
.nf 
.LP 
seq_dbListCreate:
  create an empty list of attributes
  <envName> : environment name of the database to access
  <access>  : read or write access (r for read, w for write)

  seq_dbListCreate will create and initialize the structure used for
  subsequent multi-read/write operations. It returns a listId, which has
  to be used in other seq_dbList* commands to refer to this particular list.

seq_dbListAdd:
  add attributes to the multi-read/write list
  <listId>  : list handle as returned by seq_dbListCreate
  <attrSymAddrN> : symbolic address of the database attribute, including
                   eventual indices but excluding the environment name
                   (as the environment to access was defined already by
                   seq_dbListCreate)
  Remark that there will be no errors produced if the attribute is already
  in the list.

seq_dbListRemove:
  remove attributes from the multi-read/write list
  <listId>  : list handle as returned by seq_dbListCreate
  <attrSymAddrN> : symbolic address of the database attribute to remove,
                   in exactly the same format as it was added initially with
                   seq_dbListAdd.

seq_dbListDestroy:
  delete a multi-read/write list
  <listId>  : list handle as returned by seq_dbListCreate

seq_dbListExtract:
  extract the information of some attribute(s) from the multi-read/write
  list as a result of a read/write operation
  <listId>  : list handle as returned by seq_dbListCreate
  <attrSymAddrN> : symbolic address of the database attribute whose info
                   is wanted, in exactly the same format as it was added
                   initially with seq_dbListAdd.

  seq_dbListExtract will return a list with the following info for each
  attribute given as an argument:
    - the attribute-type (either SCALAR, VECTOR or TABLE)
    - the list of data-types; data-types are integers reflecting the type
      of the attribute's fields.  For SCALARs and VECTORs this is a single
      value (hence not surrounded by curly braces)
    - the complection status of the multi-read/write operation for this
      particular attribute; 0 means success, 1 signals failure
    - the quality of the data: either OK, SUSPECT, ERROR or DISABLED; for
      an explanation of the meaning of these words, see the CCS db
      documentation.
    - the list of values.  For SCALARs, this is a single value, which will
      include braces if it contains spaces.  For VECTORs, if the attribute
      covers multiple records, this will be a list of values, one per
      record.  For TABLEs, each list of values for each record will contain
      as values as there are fields specified in the attribute.  This scheme
      is identical to what goes for the seq_dbReadSymbolic command.
  If a single attribute was given as argument, the return-string will be a
  list with the 5 elements as described above.  If multiple attributes are
  passed, the return-string will be a list with one element per attribute,
  and each element is on its turn a list with the 5 elements given above.

seq_dbListPut:
  prepare the listId's internal buffer corresponding to a certain attribute
  for a multi-write operation.
  <listId> : list handle as returned by seq_dbListCreate
  <attrSymAddrN> : symbolic address of the attribute
  <valueN> : list of values for this particular attribute, in the format
             as returned by seq_dbReadSymbolic.  I.e. for attributes of
             type VECTOR and TABLE, it is a list with as many elements as
             records have to be written.  As usual in Tcl/Tk lists, if only
             one record is written, i.e. if this list contains only one
             element, enclosing braces are not needed.

  seq_dbListPut will take the values passed, and store them in the memory
  assigned to this attribute, so it is ready to be written out with a
  seq_dbMultiWrite command.  There is no database access involved with
  this command.

seq_dbListList:
  return a list with one element per listId.  Each such element contains
  the following information, in the order as listed:
    - the list handle
    - the environment name it is linked to
    - the use-flag for this list ("r" for read, "w" for write)
    - a list containing the attributes of this multi-read/write list.
  The optional <environment> argument allows to restrict the list to this
  particular environment.

.SH ENVIRONMENT 
.nf 
.LP 
no environment nor Tcl variables are read nor set

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  The result string is empty, except for
    seq_dbListCreate, seq_dbListExtract and seq_dbListList, where it will
    contain the strings as indicated above.
ERROR return if one of the following conditions is met (with corresponding
error message here between quotes):
  - 'wrong # args : should be "seq_dbListCreate <envName> <access>"': the
    command was given with the wrong number of arguments
  - 'wrong # args : should be "seq_dbListAdd <listId> <attrSymAddr1>
    ?<attrSymAddr2>? ..."': the command was given with the wrong number of
    arguments
  - 'wrong # args : should be "seq_dbListRemove <listId> <attrSymAddr1>
    ?<attrSymAddr2>? ..."': the command was given with the wrong number of
    arguments
  - 'wrong # args : should be "seq_dbListDestroy <listId>"': the command
    was given with the wrong number of arguments
  - 'wrong # args : should be "seq_dbListExtract <listId> <attrSymAddr1>
    ?<attrSymAddr2>? ..."': the command was given with the wrong number of
    arguments
  - 'wrong # args : should be "seq_dbListPut <listId> <attrSymAddr1>
    <value1> ?<attrSymAddr2> <value2>? ..."': the command was given with
    less than 3 arguments or an even number of arguments
  - 'wrong # args : should be "seq_dbListList ?<environment>?"': the
    command was given with the wrong number of arguments
  - 'wrong value for access: "<access>" should be either "r" or "w"'
  - 'cannot create a multi-read/write list for env <envName>, access
    <access>': the call to dbListCreate returned an error
  - 'invalid list handle: "<listId>"': <listId> does not start with the
    word "list"
  - 'invalid list handle: "<listId>" (no multi-read/write items associated
    to it)': the shell does not know of any multi-read/write list with this
    particular <listId>
  - 'cannot get info on "<attrSymAddrN>"': during seq_dbListAdd, the call
    to dbGetAttrInfo() failed, most likely because this symbolic address
    is not correct.
  - 'cannot add <attrSymAddrN> to multi-read/write list "<listId>": the
    call to dbListAdd() failed for this symbolic address
  - 'cannot remove <attrSymAddrN> from multi-read/write list "<listId>"':
    this symbolic address is not a member of the multi-read/write list, or
    the call to dbListRemove() failed for other reasons.
  - 'cannot extract info of <attrSymAddrN> from multi-read/write list
    "<listId>"': this symbolic address is not a member of the multi-read/
    write list, or the call to dbListExtract() failed for other reasons.
  - 'cannot destroy multi-read/write list "<listId>"': the call to
    dbListDestroy() returned an error
  - errors from list-splitting (<valueN> for VECTOR and TABLE attributes)
  - 'amount of data incompatible with attribute info' during execution of
    seq_dbListPut, for VECTORs and TABLEs with less/more data than the
    number of records indicates
  - 'record <i>: expected <m> data values; have got <n>': for TABLES the
    number of values in each sublist (record) of <value> should equal the
    number of dataType elements.
  - 'record <i> field <m>: could not convert "<valueN>" to dbTYPE <n>':
    the routine dbStrToDe(3) choked on this conversion of a TABLE data
    element.
  - 'record <i>: could not convert "<valueN>" to dbTYPE <n>': as above,
    but here for the conversion of a VECTOR record.
  - 'could not convert "<valueN>" to dbTYPE <n>': as above, but now for
    the conversion of a SCALAR.

.SH CAUTIONS 
.nf 
.LP 
Comparing seq_dbLisExtract's return-string for the case of a single
attribute with the case of multiple attributes, one might detect a slight
inconsistency.  For the latter, the reply will consist of a list with one
element per attribute.  Each of these elements are on their turn a list
containing the attribute-type, data-type(s), status, quality and value(s) -
some of which can again be sublists.
If there is on the other hand only one attribute passed to
seq_dbListExtract, the reply will not be a list with a single element
(extrapolating the above scheme), but rather the contents of that single
element.  The advantage of this approach is that one does not need so many
layers of list-handling commands in the most trivial case of a single
attribute.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>seq_dbListCreate wte67 r
list0
seqWish>set attribute1 :PARAMS:SCALARS.scalar_float
seqWish>set attribute2 :PARAMS:VECTORS.vector_double(1:2)
seqWish>set attribute3 :PARAMS:TABLES.full_table(1:2,3:4)
seqWish>seq_dbListAdd list0 $attribute1 $attribute2
seqWish>....
seqWish>seq_dbListRemove list0 $attribute2
seqWish>seq_dbListList
{list0 wte67 r :PARAMS:SCALARS.scalar_float}
seqWish>seq_dbListAdd list0 $attribute3
seqWish>....
seqWish>seq_dbMultiRead list0
seqWish>seq_dbListExtract list0 $attribute1 $attribute3
{{SCALAR 8 0 OK 5.75} {TABLE {8 16} 0 OK {{4 5} {3 4}}}}
seqWish>....
seqWish>seq_dbListPut list0 $attribute1 321.5 $attribute2 {3 7}
seqWish>seq_dbListPut list0 $attribute3 {{1.2 {hi there}} {2. hi!}}
seqWish>seq_dbMultiWrite list0
seqWish>....
seqWish>seq_dbListDestroy list0
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqDbListCreate(3), seqDbListAdd(3), seqDbListRemove(3),
seqDbListDestroy(3),

