.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqDbReadSymbolic.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_dbReadSymbolic n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_dbReadSymbolic - read attribute values from on-line database points

.SH SYNOPSIS 
.nf 
.LP 
seq_dbReadSymbolic ?-noBadQLog? <attrSymAddress> ?<badQualityFlag>?

.SH DESCRIPTION 
.nf 
.LP 
seq_dbReadSymbolic will read a value of an attribute

-noBadQLog : an optional flag indicating that Quality Errors should not be
             sent to the logging system (which they are by default).  Remark
             that polling of "bad" attributes tends to fill up the log.  If
             -noBadQLog is set, the corresponding error stack will be reset
             (i.e. it will be lost).
<attrSymAddress> : string containing a symbolic address of the attribute
<badQualityFlag> : name of a Tcl boolean variable that will be set/reset
                   according to whether the quality is suspect (bad) resp.
                   OK.  This is useful for scanned attributes only.

seq_dbReadSymbolic will convert the value read to a string, and return it
as the result-string.
Scalar-values are not seen as list elements, i.e. they are never returned
with enclosing braces, unless they are of type string and the braces are
part of that string.
Vector-records are returned as a list and tables as a list of lists (with
each sub-list containing the values of all fields of 1 record). Surrounding
curly braces are added to these sub-lists for table-reads with more than
one field.  They will also appear around every list-element which is a
string containing spaces. I.e. the use of lists is limited to the cases
where they are really needed.  See also the 'CAUTIONS' section.

For floats etc. the precision is determined by tcl_precision.

.SH ENVIRONMENT 
.nf 
.LP 
tcl_precision:
  global Tcl variable whose value is passed on to dbDeToStr(3) as the
  argument determining the precision (i.e. the maximum number of
  significant digits) of the conversion to string.

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred.  In this case the result-string contains the
    db value(s)
ERROR if one of the following conditions is met (with the corresponding
error message here between quotes):
  - 'wrong # args: should be "seq_dbReadSymbolic ?-noBadQLog?
    <attrSymAddress> ?<badQualityFlag>?"': the command was given with the
    wrong number of arguments
  - 'wrong option: if "seq_dbReadSymbolic" has 3 args, one of them must be
    "-noBadQLog"': as the message says
  - 'cannot read "<attrSymAddress>", even after resizing buffer' if
    dbReadSymbolic still returned an error even after the receiving buffer
    was resized properly (RTAP only)
  - 'cannot read "<attrSymAddress>"' if dbReadSymbolic returned an error
    from which seq_dbReadSymbolic cannot recover

.SH CAUTIONS 
.nf 
.LP 
The <badQualityFlag> can of course not be set to "-noBadQLog" - but variable
names should never start with a dash!

Comparing the return-string for the case of a single record, single field
(like SCALAR) with the case of multiple records, one might detect a slight
inconsistency.  For the latter, the reply will consist of a list with one
element per record.  Each of these elements will on their turn be a list
if multiple fields are involved.
If there is on the other hand only one record to read, the reply will not
be a list with a single element (extrapolating the above scheme), but
rather the contents of that single element, i.e. the value(s) of the
field(s).
So a table with one record, one field gives a result-string containing a
single value.  One record, multiple fields gives a list with one element
per field (each element containing a field's value).  Multiple records,
one field gives a similar list.  Multiple records, multiple fields result
in a list with one element per record.  Each of these elements are on their
turn a list, with one element per field.
The advantage of this approach is that one does not need so many layers of
list-handling commands in the most trivial case of a single value.

The internal buffer needed to read the attribute's value is automatically
increased if necessary.  This memory is not returned to the system until
the shell is stopped.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_dbReadSymbolic ":PARAMS:SCALARS.scalar_float" bad
12.345679
seqWish>set bad
0
seqWish>set tcl_precision 3
seqWish>seq_dbReadSymbolic ":PARAMS:SCALARS.scalar_float"
12.3
seqWish>set tcl_precision 10
seqWish>seq_dbReadSymbolic ":PARAMS:VECTORS.vector_double(2:4)"
12.34567890 0. 0.987654321
seqWish>seq_dbReadSymbolic ":PARAMS:TABLES.full_table(0:1)"
{{First field} second third} {wow {again wow} {more wow}}
seqWish>....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqDbReadSymbolic(3), seqInit(3), dbReadSymbolic(3), dbDeToStr(3)

