.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqDbWriteSymbolic.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_dbWriteSymbolic n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_dbWriteSymbolic - write a value to an attribute given its symbolic
                      address

.SH SYNOPSIS 
.nf 
.LP 
seq_dbWriteSymbolic <attrSymAddress> ?<attrInfo>? <value>

.SH DESCRIPTION 
.nf 
.LP 
seq_dbWriteSymbolic will write the <value> into an attribute pointed to by
its symbolic address.  The arguments are:

<attrSymAddress> : string containing the symbolic address of the attribute

<attrInfo> : a list containing 4 elements, in the following order:
  - the type of the attribute, i.e. SCALAR, VECTOR or TABLE (attrType)
  - the data type(s) of the attribute; integer(s) in the dbTYPE range
    (datatype).  Remark that for TABLEs, only the data types of the fields
    which will be written into must be specified.
  - the number of attribute records to write
  - the number of attribute fields to write
  The latter two pieces of informations correspond to the information
  returned by seq_dbGetAttrInfo.
  If <attrInfo> is not specified, a call to dbGetAttrInfo will be issued
  to retrieve these data (which may slow down this function somewhat).

<value>: the new value(s) for the attribute.

For attributes of type VECTOR and TABLE, the last argument is a list with
as many elements as records have to be written.  As usual in Tcl/Tk lists,
if only one record is written, i.e. if this list contains only one element,
enclosing braces are not needed.

For attributes of type TABLE, the data types (part of <attrInfo>) is a
sublist.  This data type list contains one element per field.  The
argument with the values consists of one element per record, as for
VECTORs. However, here each element is a sublist, with one element (value)
per field.

The number of records to write for VECTORs and TABLEs is taken from the
<attrInfo>, or, in its absence, via a call to dbGetAttrInfo.  It is checked
against the length of the list of new attribute values.

The DB end-range character '$' is fully supported for both record and field
ranges, as are record addressing by content and field addressing by name.

Remark that some values (e.g. time values) can be (sub)lists.

.SH ENVIRONMENT 
.nf 
.LP 
no environment nor Tcl variables are read nor set

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred, with an empty result string.
ERROR if one of the following conditions is met (with corresponding
error message here between quotes):
  - 'wrong # args: should be "seq_dbWriteSymbolic <attrSymAddress>
    ?<attrInfo>? <value>"': the corresponding Sequencer command was given
    with the wrong number of arguments;
  - '<attrInfo> should contain the data type' if the <attrInfo> list has
    only one element;
  - '<attrInfo> for VECTOR should contain the record count' if the
    <attrInfo> list has only two elements for an attribute of type VECTOR;
  - '<attrInfo> for TABLE should contain the record and field counts' if
    the <attrInfo> list has less than 4 elements for an attribute of type
    TABLE;
  - 'attribute type "<attrType>" unknown; should be SCALAR, VECTOR or
    TABLE': the attribute type specified is none of the three listed
  - errors from parsing integers (data type, record count, field count)
  - errors from list-splitting (<attrInfo>; dataType for TABLE; <value> for
    VECTOR and TABLE)
  - 'amount of data incompatible with attribute info' for VECTORs and
    TABLEs with less/more data than the number of records indicates
  - 'amount of supplied data type info inconsistent with number of fields'
    for TABLEs with less/more data type info than the number of fields
    indicates
  - 'record <i>: expected <m> data values; have got <n>': for TABLES the
    number of values in each sublist (record) of the last argument should
    equal the number of dataType elements.
  - 'record <i> field <m>: could not convert "<valueX>" to dbTYPE <n>':
    the routine dbStrToDe(3) choked on this conversion of a TABLE data
    element.
  - 'record <i>: could not convert "<valueX>" to dbTYPE <n>': as above,
    but here for the conversion of a VECTOR record.
  - 'could not convert "<valueX>" to dbTYPE <n>': as above, but now for
    the conversion of a SCALAR.
  - 'cannot write to attribute "<attrSymAddress>"': the call to
    dbWriteSymbolic failed.

.SH CAUTIONS 
.nf 
.LP 
Although "named" record and field indices are allowed, they require an
additional access to the database to convert them to numbers.  This means
extra overhead, which is avoided by giving numbers.  Additionally, this
works only for the local environment, as it relies on dbGetDirAddr(3).

The buffer needed to write the attributes' values to the database via an
dbWriteSymbolic call is automatically resized if necessary, and never
deallocated.  This may cause for unfairly big writes to TABLEs a one-time
noticeable growth of the process memory.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_dbWriteSymbolic <alias>me.myScalar {SCALAR 5 1 1} 327
seqWish>seq_dbWriteSymbolic <alias>me.myVector(1:2) {VEC 6 15 1} {3 7}
attribute type "VEC" unknown; should be SCALAR, VECTOR or TABLE
seqWish>seq_dbWriteSymbolic <alias>me.myVector(1:2) {VECTOR 6 15 1} {3 7}
seqWish>set a [seq_dbReadSymbolic <alias>me.myTable(0:0)]
seqWish>seq_dbWriteSymbolic <alias>me.myTable(1:1) {TABLE {20 25} 5 2} $a
seqWish>seq_dbWriteSymbolic <alias>me.myTable(1:1,1:1) {TABLE 25 5 2} ""
seqWish>....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqDbWriteSymbolic(3), dbStrToType(3), dbWriteSymbolic(3) , dbGetDirAddr(3)

