.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqDeleteHandle.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_deleteHandle n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_deleteHandle - delete a handle and release related memory

.SH SYNOPSIS 
.nf 
.LP 
seq_deleteHandle <handle>

.SH DESCRIPTION 
.nf 
.LP 
seq_deleteHandle will delete a <handle> and its related resources. It
provides a unified way to do this for any of the handles currently used by
the Sequencer shell:

- a command handle, as obtained by issuing a "seq_msgSendCommand" (in this
  section, the word "command" refers to such a message sent with
  "seq_msgSendCommand). The handle itself will be removed; all buffers
  corresponding to replies which are already in, but which are not yet
  retrieved via e.g. a seq_msgRecvReply command, will be freed. Remark that
  also the corresponding commandId will be freed up for re-use.
  Consequently, replies for this particular command which are not yet on
  the queue can no longer be handled properly after issuing a
  seq_deleteHandle (see CAUTIONS).

- an event handle, as obtained by issuing a "seq_evtAttach" command. There
  will be an evtDetach() call before the handle is removed. This is
  equivalent to the command "seq_evtDetach".

- an alarm handle, as obtained by issuing a "seq_alrmAttach" command. There
  will be an alrmDetach() call before the handle is removed. This is
  equivalent to the command "seq_alrmDetach".

- a script handle, as received with a SCRIPT command (see seq_msgSendReply).
  The process which sent this SCRIPT command will receive a final reply
  indicating that the script handle is being deleted, and no further
  processing will take place.

- a multi-read/write list, as obtained with a seq_dbListCreate command.
  This is identical to issuing the seq_dlListDestroy command.

As indicated above, for event-handles and list-handles "seq_deleteHandle"
is equivalent to "seq_evtDetach" resp. "seq_dbListDestroy". There is no
obvious reason why one would want to delete a script-handle (as they are
under normal conditions short-lived and anyway not numerous), but for
reasons of uniformity this type of handles can be dealt with as well. So
the objects of seq_deleteHandle might remain limited to the command-
handles.

.SH ENVIRONMENT 
.nf 
.LP 
no environment nor Tcl variables are read nor set

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  The result string is empty.

ERROR return if one of the following conditions is met (with corresponding
error message here between single quotes):
  - 'wrong # args : should be "seq_deleteHandle <handle>"' the corresponding
    Sequencer command was given with the wrong number of arguments;
  - 'invalid handle: "<handle>"' if the handle is not one of the four types
    listed above (looking at the first few characters)
  - 'invalid cmd handle: "<handle>" (no replies pending on it)': if the
    command-handle does not have any replies pending on it
  - 'invalid event handle: "<handle>" (no database-events associated to it)'
    if the  event-handle does not have any attributes attached
  - 'invalid script handle: "<handle>" (no SCRIPT command related to it)'
  - 'invalid list handle: "<handle>" (no multi-read/write items associated
    to it)'

.SH CAUTIONS 
.nf 
.LP 
The deletion of a command-handle should be exercised with proper care.
The reason being that the process who received the associated command will
not be informed that it should no longer send replies to this command. If
it still does, it will use a commandId which the Sequencer shell in the
meantime may have allocated to another command. This could obviously lead
to confusion in the reply handling. So this deletion of a command-handle is
intended for cases where one is confident that the partner process is no
longer alive, or when one has other means to prevent the partner from
sending "pending" replies.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>set cmdId [seq_msgSendCommand $env $proc INIT]
 .....    (user notifies that $proc died during initialization)
seqWish>seq_deleteHandle $cmdId
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqDeleteHandle(3), deleteHandle(3)

