.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqEvt.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_evtAttach n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_evtAttach, seq_evtDetach, seq_evtSingleDisable, seq_evtSingleEnable,
seq_evtList - database and event management commands

.SH SYNOPSIS 
.nf 
.LP 
\fBseq_evtAttach\fR \fI<attrName> <filter> <script>\fR
or
\fBseq_evtAttach\fR \fI<cmdId> <timeout> <script>\fR

\fBseq_evtDetach\fR \fI<eventHandle>\fR

\fBseq_evtSingleDisable\fR \fI<eventHandle>\fR

\fBseq_evtSingleEnable\fR \fI<eventHandle>\fR

\fBseq_evtList ?-status? ?-attrName? ?-filter? ?-script? | ?-all?\fR

.SH DESCRIPTION 
.nf 
.LP 
\fBseq_evtAttach\fR:
  in its first form will attach an event to a database attribute:

    <attrName> : the database attribute to which an event has to be
                 attached; is either a symbolic address or its alias.
    <filter>   : to condition which should generate the event.  It can be:
                 \fI<\fR or \fILT\fR for less than
                 \fI<=\fR, \fI=<\fR or \fILE\fR for less or equal
                 \fI=\fR, \fI==\fR or \fIEQ\fR for equal
                 \fI>\fR or \fIGT\fR for greater than
                 \fI=>\fR, \fI>=\fR or \fIGE\fR for greater or equal
                 \fI!=\fR or \fINE\fR for not equal
                 \fIw\fR or \fIW\fR for any write
                 \fI<>\fR or \fI><\fR for deadband
                 \fIqc\fR or \fIQC\fR for quality change of a scanned attribute
    <script>   : the script to execute when an event occurs; this script
       will be evaluated at global level.  The following substitutions will
       be performed on this script before evaluation:
       - %A : name of the attribute causing the event
       - %D : event data; this is a list, whose contents depends on the
              attribute type as follows (see also evtParseMsg(3))
              for SCALAR: {<dataType> <oldValue> <newValue>}
              for VECTOR: {<startElement> <endElement>}
              for TABLE: {<startElement> <endElement> <startField>
                            <endField>}
       - %P : info about the process causing the event; a list with two
              elements: {<envName> <processName>}.  Remark however that
              events associated with LCU database attributes do not return
              this information, and an attempt to use this substitution
              will result in errors in this case.
       - %Q : quality (for scalars only).  It gives a list with 2 values:
                  {<oldQuality> <newQuality>}
              where <oldQuality> and <newQuality> can be either \fIOK\fR,
              \fISUSPECT\fR, \fIERROR\fR, or \fIDISABLED\fR.  The exact meaning of these
              words can be found in the evtParseMsg(3) manpage.
              For vectors and tables, this will give an empty list.
       - %T : attribute type.  Either SCALAR, VECTOR or TABLE
       - %U : the user-message part of the event
       - %% : a single % character
       Whenever the substitution on the original script or the evaluation
       of the resulting script yields an error, the corresponding event
       will be disabled, and the error will be logged and also signalled in
       a separate window popping up via \fITcl_BackgroundError\fR.

  in its second form will attach the execution of a script to an incoming
  reply:
    <cmdId>: as returned by \fIseq_msgSendCommand\fR

    <timeout>: timeout value in ms; a timeout-error reply will be generated
       if no reply came in within that time.  Zero means no timeout.  The
       absolute value is taken.

    <script>: a script to be executed (at global level) when such a reply
       comes in; this script will be executed once for each reply,
       independent from the value for \fIlastReply\fR or the error condition,
       including timeout.  In other words, the script must intercept these
       different conditions.

       The following substitutions will be performed on the script before
       it is evaluated:
       - %A : ASCII reply received in the message body; if this reply is
              empty or contains any spaces, it will be surrounded by curly
              braces.
       - %C : command handle (i.e. the commandId triggering this reply)
       - %E : error number from the received error structure; 0 means no
              error
       - %F : formatted binary reply received in the message body; if this
              reply is empty or contains any spaces, it will be surrounded
              by curly braces.  Remark that this also works fine on ASCII
              replies, although it is not as efficient as %A. Unformatted
              binary replies will be left empty ({}), while replies without
              an entry in the CDT will be treated as ASCII.
       - %L : last reply flag; "0" for not set, "1" if set
       - %N : environment name from where the reply was sent
       - %P : name of the process sending the reply
       - %R : reply received in the message body; if this reply is empty
              or contains any spaces, it will be surrounded by curly braces.
              This substitution is equivalent to %F if the command was sent
              with command checking on (see seq_msgSendCommand(n));
              otherwise it behaves as %A.
       - %T : timeout flag; "0" for no timeout, "1" for timeout
       - %% : a single % character

       Whenever the substitution on the original script or the evaluation
       of the resulting script yields an error, the corresponding event
       will be disabled.  On top of that, the error will be logged and also
       signalled in a separate window popping up via \fITcl_BackgroundError\fR.
       Each substitution symbol ends up being replaced by a value which is
       considered an element of a list, thereby enabling further processing
       with list operations.

    Replies already on the internal stack will be treated as events.

    After the last reply is dealt with, the link between the <cmdId> and
    the <script> is deleted (as in fact the command handle with all
    associated information is removed).

  The distinction between the two forms of the \fIseq_evtAttach\fR command is
  done based on the second argument: if it is an integer, then it must be
  the attachment to an event-reply.

\fBseq_evtDetach\fR:
    will detach a previously attached event from a database attribute,
    and delete the event handler

    <eventHandle>  : the event handle, as returned by \fIseq_evtAttach\fR.

\fBseq_evtSingleDisable\fR:
    will disable a previously attached event

    <eventHandle>  : the event handle, as returned by \fIseq_evtAttach\fR.

\fBseq_evtSingleEnable\fR:
    will enable a previously attached and disabled event

    <eventHandle>  : the event handle, as returned by \fIseq_evtAttach\fR.

\fBseq_evtList\fR:
    lists the already defined events with their properties as requested:

    -status  : tell whether for each event whether it is enable or disabled
    -attrName: give the attribute name to which the event is attached
    -filter  : include also the filter
    -script  : list the script that will be evaluated when the event occurs
    -all     : shorthand for the above 4 properties

    The result string is contains per event a sublist with the elements as
    requested.  They will be listed in the order as given above, i.e.
    independent of the order of these options on the command line, and
    preceeded by the event handle.  Remark that \fIattrName\fR, \fIfilter\fR and \fIscript\fR
    are in the order and format as required for \fI seq_evtAttach\fR.

.SH ENVIRONMENT 
.nf 
.LP 
The value of the \fBseq_ccsCmdCheck\fR variable at the time the command was sent
can be used to flag the (expected) presence of a CDT of the process the
reply comes from.  It influences the behaviour of the \fI%R\fR substitution.  See
also seq_msgSendCommand(n) and seqInit(3).

The seq_evt* commands have corresponding entries in the global Tcl array-
variable \fBseq_debug\fR, which controls the printing of debug information (see
Seq_Init(3)). On top of that, if bit 3 of \fBseq_debug(seq_msgDispatch)\fR is set,
there will be additional info printed on incoming event messages.

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  For seq_evtAttach the result string
    contains the event handle (\fIeventNNN\fR for database events, \fI<cmdId>\fR for
    reply events) to use when referring to this this event with any other
    \fIseq_evt*\fR command.  For \fIseq_evtList\fR the result string is a list of
    sublists (one sublist per event, with elements as requested).
ERROR return if one of the following conditions is met (with corresponding
error message here in bold-face):
  - \fBwrong # args : should be "seq_evtAttach <attrName> <filter> <script>"
    or "seq_evtAttach <cmdId> <timeout> <script>"\fR when the \fIseq_evtAttach\fR
    command was given with the wrong number of arguments;
  - \fBinvalid cmd handle "<cmdId>" (no replies pending on it)\fR if a command
    was given with a <cmdId> which has no more pending replies (i.e. the
    handle is no longer valid)
  - \fBinvalid event handle "<eventHandle>" (no database-events associated
    with it)\fR if a command was given with an <eventHandle> which is no
    longer valid
  - \fBinvalid filter: "<filter>" should be one of "< <= = > => != W"\fR when
    the filter is not valid
  - \fBcannot attach event to "<attrName>"\fR if the call to evtAttach(3)
    returns an error
  - \fBwrong # args: should be "seq_evtDetach <eventHandle>"\fR when the
    \fIseq_evtDetach\fR command is given with the wrong number of arguments
  - \fBinvalid cmd handle "<cmdId>"\fR if <cmdId> is not a command handle
  - \fBinvalid event handle "<eventHandle>"\fR if <eventHandle> is not an event handle
  - \fBcannot detach <eventHandle>\fR if the call to evtDetach(3) returns an error
  - \fBwrong # args: should be "seq_evtSingleDisable <eventHandle>"\fR when the
    \fIseq_evtSingleDisabl\fRe command is given with the wrong number of
    arguments
  - \fBcannot de-activate <eventHandle>\fR when seq_evtSingleDisable(3) returns an
    error
  - \fBwrong # args: should be "seq_evtSingleEnable <eventHandle>"\fR when the
    \fIseq_evtSingleEnable\fR command is given with the wrong number of arguments
  - \fBcannot re-activate <eventHandle>\fR when seq_evtSingleEnable(3) returns an
    error
  - \fBwrong # args: should be "seq_evtList ?-status? ?-attrName? ?-filter?
    ?-script? | ?-all?"\fR when the \fIseq_evtList\fR command is given with the
    wrong number of arguments
  - \fBbad option: "<arg>" should be -status, -attrName, -filter, -script or
    -all\fR when the \fIseq_evtList\fR command is given with a bad argument

.SH CAUTIONS 
.nf 
.LP 
As events associated with LCU database attributes do not return the full
information about the process causing the event, an attempt to use the %P
substitution will result in errors in this case.

A deadband event can only be attached to an LCU database attribute;
moreover, this requires the event to be configured previously with a call
to evtConfig(3) (which only exists in LCC).

For reply events, the CDT must be loaded into cmdManager in order to be
able to format replies. See \fIcmdManager(1)\fR and \fIcmdSetup(1)\fR.  The CDT is
not accessed if "%A" is used instead of "%R".

As each substitution symbol ends up being replaced by a value which is
considered an element of a list, this implies that further processing with
list-operators may yield errors if these substitution symbols are not
surrounded by spaces. E.g. for reply-events, if the script contains the
sequence "}%A%T", it will be impossible to extract the reply-value with a
simple list-operator.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ..... (<alias>my.scalar is a float with value 1.23)
seqWish>seq_evtAttach <alias>my.scalar w {set new [lindex %D 2]; set b 5}
event2
seqWish>set b 32
seqWish>.....
 ..... (<alias>my.scalar gets modified to 32.1)
seqWish>set b
5
seqWish>set new
32.1
seqWish>set b 42
seqWish>seq_evtSingleDisable event2
seqWish>.....
 ..... (<alias>my.scalar gets modified to 4.3)
seqWish>set b
42
seqWish>set new
32.1
seqWish>seq_evtDetach event2
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
\fITcl and the Tk toolkit\fR, John K. Ousterhout, ISBN 0-201-6337-X
evtAttach(3), evtDetach(3), evtSingleDisable(3), evtSingleEnable(3),
seq_msgDispatch(n), seqInit(3)


