.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqLogFits.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_logFitsAction n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_logFitsAction, seq_logFitsComment, seq_logFitsEvent,
seq_logFitsParRecord  - generate FITS logs

.SH SYNOPSIS 
.nf 
.LP 
seq_logFitsAction <dictionary> <category> <subsys> <action>

seq_logFitsComment ?-<who>? <logString>

seq_logFitsEvent ?-<type>? <logString>

seq_logFitsParRecord ?-<type>? <dictionary> <cat> <subsys> <param> <value>

.SH DESCRIPTION 
.nf 
.LP 
seq_logFitsAction will generate a FITS action record. Its arguments are:
  <dictionary> : name of the dictionary that will be checked for
                   correctness of the other arguments; remark that this
                   does not include the "ESO-VLT-DIC" prefix.
  <category>   : 3-letter keyword category
  <subsys>     : subsystem name(s)
  <action>     : verb defining the action; corresponds to parameter-name
                   in the dictionary.

seq_logFitsComment will generate a FITS comment record. Its arguments are:
  <who>   : "classname" of generator of this log; legal values are:
          "observer", "staff", "remote control" or "night assistant". These
          names can also be abbreviated to "ob", "sa", "rc" respectively
          "na", and they can contain any mixture of lower- and uppercase
          characters. If <who> is not specified, a free-format comment will
          be logged.
  <logString> : string that has to be logged

seq_logFitsEvent will generate a FITS event record. Its arguments are:
  <type>   : "unforeseen" (default) or "recovery". These names can also be
          abbreviated to "ufo" resp "rec", and they can contain any mixture
          of lower- and uppercase characters.
  <logString> : string that has to be logged

seq_logFitsParRecord generates a FITS parameter record. Its arguments are:
  <type>       : either "int" (for 32-bit integers), "real" (for doubles) or
                 "string" (for strings containing up to 256 characters)
  <dictionary> : name of the dictionary that will be checked for
                 correctness of the other arguments; remark that this
                 does not include the "ESO-VLT-DIC" prefix.
  <cat>        : 3-letter keyword category
  <subsys>     : subsystem name(s)
  <parameter>  : parameter name (last substring of the short-fits keyword)
  <value>      : value for the parameter (in 256-byte string).

  This seq_logFitsParRecord command is normally used with a <type> option,
  indicating what sort of conversion (from string values) should take place.
  Without this <type> option, seq_logFitsParRecord should be used with
  extreme caution only, as one needs to take care that <value> is (at least)
  256 bytes long. See also the CAUTIONS section below.
  Conversion from string values is done using sscanf(3), which means that
  e.g. floats are simply truncated if converted to integer. No errors are
  returned if this conversion fails (e.g. attempting to convert "abc" to
  a integer), but the value shown in the log will of course be wrong (it
  will be set to zero).

The exact format of the FITS logs generated by these commands is described
in the manpages of the underlying respective logFits*(3) procedures.

.SH ENVIRONMENT 
.nf 
.LP 
The setting of the global Tcl variable seq_moduleId is used as the CCS
module-identifier in the the underlying logFits*(3) calls.

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  The result string is empty.
ERROR return if one of the following conditions is met (with corresponding
error message here between quotes):
  - 'wrong # args : should be "seq_logFitsAction <dictionary> <category>
    <subsys> <action>"': the corresponding Sequencer command was given with
    the wrong number of arguments;
  - 'wrong # args : should be "seq_logFitsComment ?-<who>? <logString>"':
    the corresponding Sequencer command was given with the wrong number of
    arguments;
  - 'wrong # args : should be "seq_logFitsEvent ?-<type>? <logString>"':
    the corresponding Sequencer command was given with the wrong number of
    arguments;
  - 'wrong # args : should be "seq_logFitsParRecord ?-<type> <dictionary>
    <cat> <subsys> <param> <value>"': the corresponding Sequencer command
    was given with the wrong number of arguments;
  - 'cannot generate FITS action log - dictionary <dictionary>, category
    <category>, subsystem <subsys>, action <action>': the underlying
    logFitsAction(3) call returned an error.
  - 'wrong generator: "<who>" should be "ob|observer", "rc|remote
    control", "sa|staff" or "na|night assistant"'
  - 'cannot generate FITS comment log "<logString>": the underlying call to
    logFitsComment(3) returned an error.
  - 'wrong type: "<type>" should be "ufo|unforeseen" or "rec|recovery"'
  - 'cannot generate FITS event log "<logString>": the underlying call to
    logFitsEvent(3) returned an error.
  - 'wrong type: "<type>" should be "int", "real" or "string"'
  - 'cannot generate FITS paramater record log - dictionary <dictionary>,
    category <cat>, subsystem <subsys>, param <param>, value <value>':
    the underlying logFitsParRecord(3) call returned an error.

.SH CAUTIONS 
.nf 
.LP 
The seq_logFitsParRecord command calls the logFitsParRecord(3) function,
which does a binary copy of 256 bytes of <value>. This means that in the
presence of a valid <type> option, there will internally be a conversion,
into a 256-byte buffer. Without this <type> option, the user of this command
mandatorily needs to take care of the encoding (i.e. put the right number of
bytes in the proper order within <value>), and in particular that <value>
does contain (at least) 256 bytes. Not following this rule could result in
segmentation errors. See see also Tcl's binary(n) command and the example
below.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>seq_logFitsComment "This is a free-field FITS comments log"
seqWish>.....
seqWish># the easy way to log an integer value
seqWish>seq_logFitsParRecord -int OBS OBS "" ID $value
seqWish># or the hard way .....
seqWish>if {[string equal $tcl_platform(byteOrder) bigEndian]} {set i I}
seqWish>seq_logFitsParRecord OBS OBS "" ID [binary format ${i}x252 $value]
seqWish>.....
seqWish>seq_logFitsParRecord -string OBS OBS "" NAME "any string"
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
logFitsAction(3), logFitsComment(3), logFitsEvent(3), logFitsParRecord(3)

