.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqLogTclErrors.tcl
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_logTclErrors n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_logTclErrors - send all output for stderr also to the error log

.SH SYNOPSIS 
.nf 
.LP 
seq_logTclErrors open|close

.SH DESCRIPTION 
.nf 
.LP 
seq_logTclErrors allows to send strings that normally go to exclusively
to stderr also to the CCS error log.  This feature is enabled by setting
the argument to "open", and disabled by setting it to "close".
Once active, errors logged will all have the same stack id, until this
logging is closed.

This command relies on a file-event.  Such events are only handled when
the Sequencer is idle, i.e. when it is not busy evaluating commands from
a script.  In other words, the output to the error log may not be visible
immediately, depending on the Sequencer's activity.

.SH ENVIRONMENT 
.nf 
.LP 
When this error logging is active, the global Tcl-variables OSE, EPR and
EPW are set and accessed.
The global Tcl-variable OSE refers to the original stderr file descriptor.
EPR and EPW are the reading resp writing end of a pipe created to divert
the messages sent to stderr.

.SH RETURN VALUES 
.nf 
.LP 
OK, with and empty result string if no errors
ERROR with corresponding message in the result string under the following
conditions:
- "called "seq_logTclErrors" with too many arguments" if there was more
  than 1 argument given to this command.
- "wrong arg: should be "open" or "close"" if argument is not correct
- "Tcl error logging already active" if open has been given before (i.e.
  the global Tcl-variable OSE exists)
- "Tcl error logging already disabled" if close has been given before
  (i.e. the global Tcl-variable OSE does not exist)

.SH CAUTIONS 
.nf 
.LP 
When this error logging is active, the Tcl-variables OSE, EPR and EPW are
set at a global level (by lack of static variables in Tcl).  They should
not be modified.

The overhead caused by sending these messages also to the CCS error system
is considerable (on top of adding possibly quite a few messages to the
log).  Its use should therefore be restricted to the bare minimum (e.g.
for debugging purposes).

Stderr must be open when Tcl error logging is started.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>seq_logTclErrors open
seqWish>blablabla
Error: invalid command name "blablabla"
seqWish>seq_logTclErrors close
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seq_errResetStack(n), seq_errAdd(n), seq_errCloseStack(n)

