.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqMsgCheck.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_msgCheck n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_msgCheck, seq_msgList - check and list commands with pending replies

.SH SYNOPSIS 
.nf 
.LP 
seq_msgCheck <cmdId> | {<envName> <procName> <command> }

seq_msgList ?{<envName> ?<procName> ?<command>??}?

.SH DESCRIPTION 
.nf 
.LP 
seq_msgCheck:
    checks if there are still replies pending on a command identified by
    its arguments.  A "1" is returned if replies are pending, "0" if not.

    <cmdId>     : unique command handle, as returned by seq_msgSendCommand
    <envName>   : environment name
    <procName>  : process name
    <command>   : command name

seq_msgList:
    returns a list with pairs, one pair per pending reply.  Each pair
    contains a cmdId and a sublist with originating environment name,
    process name and command name, describing the command for which a
    reply is pending.  The optional list allows to refine the reply-list
    progressively.

    <envName>   : environment name
    <procName>  : process name
    <command>   : command name

.SH FILES 
.nf 
.LP 
no files are accessed

.SH ENVIRONMENT 
.nf 
.LP 
no Tcl variables are read nor set

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred.  In this case the result string contains the
    result as indicated above.

ERROR if one of the following conditions is met (with the corresponding
error message here between quotes):
- seq_msgCheck:
  * "wrong # args: should be "seq_msgCheck <cmdId> | {<envName> <procName>
    <command>}""
  * "bad argument: "<argv[1]>" should be "<cmdId> | {<envName> <procName>
    <command>}""
  * error message left by Tcl_SplitList failing to parse the list
  * error message left by TclX_HandleXlate failing to parse the <cmdId>
- seq_msgList:                            (char *) NULL);
  * "wrong # args: should be "seq_msgList ?{<envName> ?<procName>
    ?<command>??}?""
  * "bad argument: "<argv[1]>" should be "?{<envName> ?<procName>
    ?<command>??}?""
  * error message left by Tcl_SplitList failing to parse the list
  * error message left by TclX_HandleXlate failing to parse the <cmdId>

.SH CAUTIONS 
.nf 
.LP 
The <cmdId> as returned by seq_msgSendCommand is a unique identifier, while
<{<command> <envName> <procName>}> is not if several identical commands to
a certain process have replies pending.  Hence, it is strongly advised to
use the former as argument for this command.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_msgSendCommand wte13 DCS init
cmd15
seqWish>...
seqWish>seq_msgCheck cmd15
1
seqWish>seq_msgList wte13
{cmd11 {wte13 ICS START}} {cmd15 {wte13 DCS INIT}}
seqWish>seq_msgRecvReply cmd15
CCD#87 initialized OK
seqWish>seq_msgCheck cmd15
0
seqWish>........

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqMsgSendCommand(3), seqMsgRecvReply(3)

