.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqMsgDispatch.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_msgDispatch n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_msgDispatch, seq_msgDispatchBreak - enable asynchronous queue
                                   monitoring via file descriptor

.SH SYNOPSIS 
.nf 
.LP 
seq_msgDispatch ?<timeout> ?<nrMessages>??

seq_msgDispatchBreak

.SH DESCRIPTION 
.nf 
.LP 
seq_msgDispatch:
   the Sequencer command executed automatically whenever a message is
   available on the queue, and message reading has been enabled (see
   seq_css(n)).  As a registered Sequencer command it can be called just
   like any other command, although this is not its primary reason for
   existence.

   <timeout>    : timeout value in ms; default: 0 (wait forever).  Remark
                  that if there is no message getting on the queue within
                  this timeout, the Sequencer shell will not be responsive
                  for the duration of this timeout!
                  If the timeout is negative, a no-wait retrieval of
                  messages will be done, and no error will be reported if
                  the queue is empty.
   <nrMessages> : number of messages to get; default: 1

   The standard CCS message-types can be retrieved this way, including
   obituaries (provided this shell's CCS process-name has been entered in
   the RtapEnvTable, with a proper value in the 'Care about Terminations'
   field; remark that processes doing a ccsInit/ccsExit will generate
   obituaries by default).
   When an obituary is received, the script contained in the global
   variable seq_obiScript will be executed, after the following
   substitutions took place:
   - %D: 1 if the deceased process registered for debug, 0 otherwise
   - %E: environment name of the deceased process/environment
   - %P: deceased process name or {} if the environment deceased
   - %R: 1 if the deceased process will be automatically restarted, 0
         otherwise.
   - %S: exit status of the terminating process (integer value)
   - %%: a single % character
   Internal tables related to the deceased process (like the ones kept for
   commands with pending replies) will be cleaned upon reception of an
   obituary message. This internal housekeeping takes place even if
   seq_obiScript is empty, and is actually done before evaluating
   $seq_obiScript.

   When seq_msgDispatch gets a command from the message queue it will
   execute it.  Presently the following commands are recognized (on top of
   the special commands BREAK, KILL and PING predefined for all CCS
   applications):
   - EVENT : the message body must contain a legal Sequencer script which
            will be evaluated.  Similar to the SCRIPT command (see below),
            with the difference that no replies are generated: the sender
            of the EVENT command will not receive and should not expect any
            reply on this command. This command is useful in an environment
            where some process(es) need(s) to be informed of an event,
            while it is considered too expensive to create/access a
            database branch just for this purpose.
            If this command was sent via seq_msgSendCommand, the latter
            needs to have the -noreply option specified.
   - SCRIPT : the message body must contain a legal Sequencer script which
            will be evaluated.  The result of this evaluation (i.e. the
            return string) will be sent as the second reply to this
            command - an immediate, empty reply is given before the
            evaluation starts.  If in this script intermediate replies
            have to be sent, the so-called script-command-handle will be
            needed (as this allows to retrieve the originator of the
            command).  This handle can be obtained with a %-substitution,
            performed on the script before evaluation (equivalent to the
            seq_evt substitutions):
            - %S : script-command-handle; it is needed by the
                   seq_msgSendReply command, as it allows to identify the
                   process which originated the SCRIPT command..
            - %% : a single % character
            If the result of the script-evaluation is a string of a size
            which does not fit into a single reply, seq_msgDispatch will
            automatically chop up this string into multiple fragments,
            send each fragment (in the right order) as a reply, whereby
            only the last fragment will have the 'lastReply' flag set.
            It will be up to the sender of the SCRIPT command to re-assemble
            these replies by concatenating these individual pieces.

   All other commands will be logged as errors, and an error message will
   be sent back as well to the originator of the command.

   seq_msgDispatch does also react on events.  Whenever an event message
   comes in, the script given at the time this event was defined (see
   seq_evtAttach(n) and seq_msgRecvReply(n)) will be executed.  The
   evaluation of the script is at global level.
   Whenever the substitution on the original script or the evaluation of
   the resulting script yields an error, the corresponding event will be
   disabled, and the error will be logged and also signalled via
   Tcl_BackgroundError(3).

   Similar for alarms:  whenever an alarm message comes in, the script
   given at the time this event was defined (see seq_alrmAttach(n)) will
   be executed.  The evaluation of the script is at global level.
   Same remark about script substitution/evaluation errors as for event
   messages.

   The setting up of the queue monitoring is done via a call to
   seqMonitorQ(3).  Also the suspension of this service can be requested
   to the same routine.

   Whenever the timeout (in msec) or the nrMessages to read leads to an
   unacceptable blocking of other tasks, the read-and-dispatch loop can be
   broken with the seq_msgDispatchBreak command.  Timeout defaults to 0
   (wait forever), while nrMessages defaults to 1.  The timeout is the
   time to wait on a msgRecvMsg(3) call for a single message, i.e. it is
   not the cumulative timeout, nor does it take the time to execute the
   command (contained in the message) into account.

seq_msgDispatchBreak:
   breaks the execution of seq_msgDispatch

.SH ENVIRONMENT 
.nf 
.LP 
seq_obiScript is a global Tcl variable containing the script to be executed
when an obituary message is received. This script can contain some variables
(single characters preceded by a %-sign) giving information about the
obituary itself; they are substituted just before evaluation of the script.

.SH RETURN VALUES 
.nf 
.LP 
seq_msgDispatchBreak always returns an OK; the remainder of this sections
is for seq_msgDispatch only.

OK if no errors occurred; the reply contains the number of messages
effectively dispatched. Remark that this may deviate from <nrMessages>
depending on the setting of <timeout> and the intermediate reception of
a seq_msgDispatchBreak command.

ERROR if one of the following conditions is met (corresponding error
message is here between quotes):
  * 'wrong # args: should be "seq_msgDispatch ?<timeout> ?<nrMessages>??"'
    if the command contains more than 2 arguments
  * 'bad argument: "<argv>" should be an integer' if either <timeout> or
    <nrMessages> have illegal values
  * 'bad argument: "<argv>" should be positive' if <nrMessages> is a
    negative integer
  * 'could not get msg queue file descriptor' if seqGetMsgFd(3) failed.
  * 'timed out after <xx> messages'; can obviously only occur if <timeout>
    is greater than 0.

Remark that most of these messages cannot appear when the seq_msgDispatch
is called via the "fileevent" scheme.  In this case we are 100 % sure the
arguments are set to legal values and there is a message on the queue.
Also internal messages, which cannot be retrieved by the application but
do cause a fileevent, do not provoke any errors.

The causes of these errors are also logged via the standard CCS error
system and logging.

.SH CAUTIONS 
.nf 
.LP 
Incoming commands cannot be longer than ~8K (i.e. the limit of a single
CCS message).

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_msgDispatch 1000 1
1
seqWish>seq_msgDispatch 100000 5
5
seqWish>....
 .....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seq_ccs(n), seq_evtAttach(n), seq_evtParseMsg(3), seqMsgDispatch(3),
seqMonitorQ(3), RtapEnvTable(4)

