.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqMsgRecvReply.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_msgRecvReply n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_msgRecvReply - receive a reply on a CCS command sent before

.SH SYNOPSIS 
.nf 
.LP 
seq_msgRecvReply ?<options>? <cmdId> <errorNumber> ?<lastReply>?

.SH DESCRIPTION 
.nf 
.LP 
seq_msgRecvReply waits for a single or all replies on a command identified
by its first argument, or for the specified timeout - whichever comes
first.

<options>: any of the following options, in any order (if contradicting
    options appear, the last one will prevail):
    * -all    : flags to get all replies in, instead of only one, and
                return all replies separated by a newline (in absence of
                option "-bin"); empty replies do not get a newline
                appended, nor is there a newline appended to each reply
                if the -nonewline option is specified.
    * -ascii  : flags reply is supposed to contain ASCII data; the CDT's
                data for the reply's format is not checked nor used.  This
                is slightly more efficient if you know in advance that the
                reply is ASCII (and/or that the CDT is not available)
    * -bin    : flags data should not be returned in result string
                (e.g. when reply contains unformatted binary data).
    * -fbin   : flags data should be formatted according to the command's
                entry in the CDT (DISPLAY_FORMAT field). This works fine
                for formatted binary and ASCII replies, although for the
                latter it is more efficient to use the -ascii option (so no
                formatting needs to be attempted). Unformatted binary
                replies will result in an empty string (although again, it
                is more efficient to do that via the -bin option).  If the
                reply does not have an entry in the CDT, it will be
                considered as ASCII (same remark again).
    * -last   : flags to wait on the last reply, and and return the data of
                the last reply only (if option "-bin" is not set)
    * -<nnn>  : any integer, giving the timeout in ms; absolute value is
                taken
    * -nonewline: indicate that the concatenation of multiple replies
                (see -all option) should be without additional newlines.
                Specifying this option in the absence of "-all" does not
                have any effect.
    * -nowait : flags no wait, i.e. first the internal queue is checked,
                and if no reply is found a call to msgRecvMsg() is placed
                with the appropriate filter and msgNO_WAIT flag
   The option -ascii is enabled by default if seq_ccsCmdCheck was set to
   "CHECK_CMD" when the command was sent; otherwise the -fbin option is
   active.  These defaults may be overridden by command line options.

<cmdId>: as returned by seq_msgSendCommand

<errorNumber>: the name of a Tcl variable where to store the error-number
    of the error-message structure returned as part of the reply.  The
    CCS convention is that "0" signals no error.  This variable can only
    be set after the message was received OK (i.e. one should test first
    that the reply-protocol did not fail, then look at <errorNumber>).
    When the "errorNumber" variable indicates the reception of an
    error message, this error-stack can be displayed with seq_errDisplay
    (until the next error comes in).

<lastReply>: the name of a Tcl boolean variable where to store the
    "lastReply" flag; will contain "0" if there are more replies, "1" if
    this reply is the last.  This variable will not be touched untill the
    reply(-ies) came in successfully.

Remark that embedded applications (which call seqInitInterp()) can have
an event-loop out of the sequencer's control, i.e. they can call ccsInit()
instead of eval-ing seq_ccsInit (which would activate the monitoring of
the message queue within the sequencer's event-loop). In these cases,
seq_msgRecvReply will by default issue a blocking msgRecvMsg() call with
infinite timeout, as it cannot count on the sequencer's monitoring of the
message queue.

.SH FILES 
.nf 
.LP 
no files are accessed

.SH ENVIRONMENT 
.nf 
.LP 
<errornumber> and <lastReply>: see above

The value of the seq_ccsCmdCheck variable at the time the command was sent
is used to flag the (expected) presence of the CDT of the process the reply
comes from.  It influences the default behaviour of seq_msgRecvReply
(-ascii or -fbin).

.SH RETURN VALUES 
.nf 
.LP 
OK if the reception of the reply caused no errors.  In this case the
result string contains either the reply or error message received.  An
eventual error will be returned independent of the setting of the -bin
option.
Remark that this OK return does not necessarily mean that the originator
of the reply signalled all is OK - cf the <errorNumber> variable.

ERROR if one of the following conditions is met (with the corresponding
error message here between quotes):
  * 'wrong # args: should be "seq_msgRecvReply ?-all? ?-ascii? ?-bin?
    ?-fbin? ?-last? ?-nnn? ?-nonewline? ?-nowait? <cmdId> <errorNumber>
    ?<lastReply>? "' if there were not at least two additional arguments
    (on top of eventual options)
  * 'invalid cmd handle: "<cmdId>"'
  * 'bad option: "<optionX>" should be -all, -ascii, -bin, -fbin, -last,
    -nowait, -nonewline or a numeric value' if one of the options is wrong
  * 'invalid cmd handle "<cmdId>" (no replies pending on it)'
  * 'failed to allocate memory to format binary reply' if there seems
    to be no memory available for the temporary formatting buffer; this
    allocation is done the first time formatting of a reply is attempted.
  * 'reply on <cmdId> (command <cmd> to <env>/<proc>) could not be
    formatted' if the formatting failed (most likely there is something
    wrong with the CDT - check the error stack and CDT).
  * 'reply timed out' if no reply came in within the specified timeout
  * 'message queue empty' if -nowait was specified and no reply was in
    yet
  * 'could not receive message' if msgRecvMsg returned an error
  * 'could not parse reply'
  * 'received reply on "<command>" with wrong commandId (<commandId>)'
    when the replier either did not set the orgCommandId properly, or
    sends more replies after the lastReply flag was set
  * 'no memory available to store replies' if in the handling of incoming
    replies a malloc failed
  * error message left by Tcl_SetVar when trying to set the variable
    <lastReply>; remark that when this error occurs, the reply was
    received properly.

.SH CAUTIONS 
.nf 
.LP 
The identification of the command that caused a reply depends on the
the commandId parameter sent together with the command.  It is therefore
absolutely essential that all applications receiving Sequencer commands via
the CCS message system, return replies that contain the CommandId of the
originating command.

When a reply comes in with the lastReply flag set, the Sequencer shells
will remove all information pertinent to this command after dealing with
this reply.  If further replies would come in, they will be properly logged
as errors, and should be interpreted as an indication of a buggy replying
application.

For formatted binary replies, the formatted string of a single reply is
limited to 16 KBytes.  There is however no checking for overflow possible
(due to how the CCS procedure cmdFormatReply is implemented), and if the
DISPLAY_FORMAT entry in the corresponding CDT leads to larger formatted
strings, a memory corruption will occur.

To be able to format replies, the CDT must be loaded into cmdManager.
See cmdManager(1) and cmdSetup(1).

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_msgSendCommand wte13 DCS init
cmd145
seqWish>.....
seqWish>seq_msgRecvReply cmd145 errNr lr
CCD#87 having problems, trying again...
seqWish>echo $lr
0
seqWish>seq_msgRecvReply cmd145 errNr lr
CCD#87 initialization failed
seqWish>echo $lr
1
seqWish>echo $errNr
15
seqWish>seq_msgSendCommand wte13 DCS init
cmd187
seqWish>seq_msgRecvReply cmd187 errNr lr -last
CCD#87 initialized OK
seqWish>echo $lr
1
seqWish>echo $errNr
0
seqWish>........

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqMsgRecvReply(3), seq_msgSendCommand(n), msgRecvMsg(3), msgParseMsg(3)

