.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqMsgSendCommand.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_msgSendCommand n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_msgSendCommand - send a CCS message

.SH SYNOPSIS 
.nf 
.LP 
\fBseq_msgSendCommand ?-noreply? ?-check? ?-nocheck?\fR \\
                   \fI<envName> <procName> <command> ?<args>?\fR

.SH DESCRIPTION 
.nf 
.LP 
-noreply  : flag that this command is of an event type, i.e. there will be
            no replies to be sent back
-check    : flag to request command and parameter checking for this
            particular command, overriding the global setting of the
            \fIseq_ccsCmdCheck\fR Tcl-variable (see below).
-nocheck  : flag to turn off command and parameter checking for this
            particular command, overriding the global setting of the
            \fIseq_ccsCmdCheck\fR Tcl-variable (see below).
<envName> : name of the environment where the command has to be sent to
<procName>: name of the recipient process
<command> : name of the command to send to the environment/process
<args>    : the arguments to pass along with <command> as the message body.
    This can spread multiple words, and the Sequencer will concatenate
    them into a single message body (string), separating neighbouring
    arguments by a single space (similar to Tcl's \fIjoin\fR command).  Remark
    however that in this case the Tcl parser and the Sequencer need to do
    some work on <args> which can be avoided: it is more efficient to have
    <args> grouped as a single word, e.g. between quotes, or, if there are
    no substitutions required, between curly braces.
    The joining of the <args> happens of course after applying the standard
    Tcl substitution- and grouping-rules.  When arguments containing
    quotes are involved, unexperienced Tcl users will often consider this
    an odd behaviour. Again, this situation can be avoided by grouping
    <args> into a single Tcl-word (see EXAMPLES section).

In the absence of \fI-noreply\fR, and when \fIsend_msgSendCommand\fR is successful, it
returns the next available command handle.  This handle points to a.o. the
\fImsgCMDID\fR parameter sent to the destination.  The latter \fImsgCMDID\fR is derived
from the handle name (it is the numeric part of it), and it can be used to
construct a filter for receiving replies on this command.  \fBIt is therefore
imperative that the recipients of such messages return the original command
identifier in the reply.\fR

If \fI-noreply\fR is given, the command will be sent with a msgCMDID of value 0,
and the return will be empty, as there are supposed to be no replies coming
back.  This option is useful in an environment where some well-known
process(es) need(s) to be informed of an event, while it is considered too
expensive to create/access a database branch just for this purpose.  This
option is functionally equivalent to sending a normal command and then
receiving and discarding replies (e.g. via the \fIseq_evtAttach\fR mechanism),
but it is of course a lot more efficient.
If the recipient process is a Sequencer shell, the <command> needs to be
\fIEVENT\fR instead of \fISCRIPT\fR, as this disables the return of replies by the
recipient.

The checking of commands is in the absence of the \fI-cmdCheck\fR and \fI-noCheck\fR
options done according to the value of the global Tcl-variable
\fIseq_ccsCmdCheck\fR.

.SH FILES 
.nf 
.LP 
no files are accessed

.SH ENVIRONMENT 
.nf 
.LP 
\fBseq_ccsCmdCheck\fR is a global Tcl_variable whose value determines whether the
CCS message system will perform command checking based on the CDTs.  It
must be set to either \fINO_CHECK\fB or \fICHECK_CMD\fR.  The default is \fINO_CHECK\fR.
Its application can be overridden by the options \fI-check\fR and \fI-nocheck\fR. See
also the description of the argument \fIflag\fR in the manpage of
msgSendCommand(3).

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred.  In this case the result string is either empty
if \fI-noreply\fR was specified, or otherwise contains the next available
command handle, as mentioned above.

ERROR if one of the following conditions is met (with the corresponding
error message here in bold-face):
  * \fBwrong # args: should be "seq_msgSendCommand ?-noreply? ?-check?
    ?-nocheck? <envName> <procName> <command> ?<args>?"\fR if
    \fIseq_msgSendCommand\fR is not given with at least 3 additional arguments
  * \fBwrong option: "-xyz" should be "-noreply", "-check" or "-nocheck"\fR if
    an unknown option \fI-xyz\fR was specified
  * \fBcould not send command "<command> <args>" to <envName>/<procName>\fR
    if msgSendCommand(3) returned an error

.SH CAUTIONS 
.nf 
.LP 
<command> will be converted to UPPERCASE, as required by CCS!  As a
consequence, \fBthe Sequencer shells cannot deal with command-synonyms
containing lowercase characters\fR.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish># The trivial case: 3 parameters without quotes and spaces
seqWish>seq_msgSendCommand -check wte13 ICS TEST {par1 2 par3}
cmd14
seqWish># The same result, easy to grasp, but some more arbeit for seqWish
seqWish>seq_msgSendCommand -check wte13 ICS TEST par1 2 par3
cmd15
seqWish># The easy way to pass a parameter including quotes
seqWish>seq_msgSendCommand wte13 ICS TEST {1 2 "3rd parameter with quotes"}
cmd16
seqWish># The same result, but novices may have to think a bit about this
seqWish>seq_msgSendCommand wte13 ICS TEST 1 2 {"3rd parameter with quotes"}
cmd17
seqWish># Here the third parameter is a string which does not include
seqWish># quotes; as it contains spaces, the whole is after concatenation
seqWish># with the first two arguments equivalent to a string with 6
seqWish># individual parameters
seqWish>seq_msgSendCommand wte13 ICS TEST 1 2 "3rd 4th 5th 6th"
cmd18
seqWish>........

.SH SEE ALSO 
.nf 
.LP 
\fITcl and the Tk toolkit\fR, John K. Ousterhout, ISBN 0-201-6337-X
seqMsgSendCommand(3), seq_msgRecvReply(n), msgSendCommand(3),


