.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqMsgSendReply.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_msgSendReply n "VLTSW-JAN2006 27/10/05-17:00"
.SH NAME 
seq_msgSendReply - send a CCS reply message

.SH SYNOPSIS 
.nf 
.LP 
seq_msgSendReply -current|<scriptId> <errorNr> ?<lastReply>? <reply>

.SH DESCRIPTION 
.nf 
.LP 
seq_msgSendReply allows to send intermediate replies to the originator of
the SCRIPT command (see seq_msgDispatch(n)).  As such, the only place where
it can appear is within the script received with this SCRIPT command, or
within a procedure called by this script.

-current: a flag indicating that the scriptId of the current SCRIPT command
          should be used instead of an explicitly specified <scriptId>. This
          provides an alternative to the %S substitution (see below).
          In case various SCRIPTS commands are nested (possible if a script
          being evaluated includes e.g. an "update" command), the handle
          used is always the one of the current context.

<scriptId>: script-command handle as substituted for a %S by seq_msgDispatch
            at the time it received the SCRIPT command; this handle allows
            to identify the process which originated the SCRIPT command.

<errorNr>:  number of the error; 0 means no error

<lastReply>: a Tcl boolean variable, to flag if this is the last reply.
             Its default value is 0. If <errorNr> is not 0, the value
             specified for <lastReply> is irrelevant, i.e. an error-reply
             is by definition the last reply.

<reply>:    string containing reply; if <errorNr> is 0, this will be in the
            message buffer at the receiving side; otherwise, this string
            will be in the ccsERROR structure passed across (which means
            this is then also the last reply).
            If <reply> is of a size which does not fit into a single reply,
            seq_msgSendReply will automatically chop up this string into
            multiple fragments, send each fragment (in the right order) as
            a reply, whereby only the last fragment will have the
            'lastReply' flag set, provided <lastReply> is true. It is up
            to receiver of these replies to re-assemble them by
            concatenating these individual pieces.

.SH FILES 
.nf 
.LP 
no files are accessed

.SH RETURN VALUES 
.nf 
.LP 
OK if no errors occurred.  In this case the result string is empty,
except if the -current option was given; in the latter case, the
<scriptId> is returned.

ERROR if one of the following conditions is met (with the corresponding
error message here between quotes):
  * 'wrong # args: should be "seq_msgSendReply -current|<scriptId>
    <errorNr> ?<lastReply>? <reply>"' if seq_msgSendReply is not given with
    the right amount of arguments
  * 'not evaluating a SCRIPT command' when the command was used with the
    -current option, while not evaluating a script that came with a SCRIPT
    command.
  * 'no replies to be sent for <scriptId>' when the <scriptId> handle does
    not or no longer exist (e.g. the last reply was sent before).
  * 'could not send reply "<reply>" to <envName>/<procName>' if
    msgSendReply returned an error
  * 'could not send reply "<reply>", nor get procName from msgPROCESSID' if
    msgSendReply returned an error, and also ccsGetProcName failed.

.SH CAUTIONS 
.nf 
.LP 
The <errorNr> (if set) will be in the top of the error stack structure wich
is sent as a reply, and can be retrieved as such by the receiving side.
This means that the generic error mnemonic seqERR_CMD_SCRIPT used for these
cases does not correspond to a fixed error number - its number gets
overwritten with <errorNr>.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 ....
seqWish>seq_msgSendReply script0 0 "I am still doing OK"
seqWish>........
seqWish>seq_msgSendReply script0 5 "I have found an error: $msg"
seqWish>........

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
seqMsgDispatch(n), seq_msgRecvReply(n), msgSendReply(3),

