#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inmos.boot,v 1.14 2005/06/22 12:40:21 vltsccm Exp $"
#
# VxWorks script for automatic installation of inmos
#
# who       when      what
# --------  --------  ----------------------------------------------
# ccumani   12/02/96  created
#                     (replaces inmosInstallTool.sh)
#
#*****************************************************************************
#
#  NAME
#  inmos.boot - script file to load inmos software
#
#  DESCRIPTION
#  This script file 
#  a) loads:
#       - idebug,     the transputer debugger
#       - idump,      the transputer memory dumper
#       - iskip,      the transputer skipper
#       - inmosF,     the transputer hosted folding editor
#       - inmosMtest, the transputer memory tester
#       - inmosCheck, the transputer network checker
#       - inmosLoad,  a transputer loader
#  b) defines the environment variables:
#       - IBOARDSIZE, used by almost all the programs
#       - ITOOLS,     which defines the directory containing
#                     idebug.btl, idump.btl, iskip.btl
#       - IDEBUGSIZE, used by idebug
#       - FTOOLS,     which defines the directory containing f.btl  
#       - ITERM,      which defines the terminal definition file
#       - FTERM,      which defines the terminal configuration file
#       - FSTART,     which defines the terminal configuration file
#
#  RETURN VALUES
#  none
#
#  CAUTIONS
#  The following environment variable must be set before running
#  this script file:
#  - OCCAM_ROOT, the official Inmos transputer sw (e.g. D4305) 
#                module path
#
#  EXAMPLE
#  vxworks> < inmos.boot
#
#------------------------------------------------------------------------------
#

lcubootAutoLoad 1,"b016",0
lcubootAutoLoad 1,"iserver",0

########### BEGINNING OF PART WHICH MUST BE EDITED ###########

################################################
# Set OCCAM_ROOT and environment variable      #
# properly.                                    #
# OCCAM_ROOT defines the root directory of     #
#            occam2 INMOS tool                 #
################################################
putenv("OCCAM_ROOT=/occam/D4305")
################################################
# Set INMOS_ROOT and environment variable      #
# properly.                                    #
# INMOS_ROOT defines the root directory of     #
#            inmos module                      #
################################################
putenv("INMOS_ROOT=/diskc/vltccd/LCUTRA-MAR95/LCUTRA/inmos")

############## END OF PART WHICH MUST BE EDITED ###############

# TRANSPUTER specifies the default capability name to use,
# it is defined by inmosInstallIserver.sh

################################################
#                                              #
# define variables for idebug, idump and iskip #
#                                              #
################################################

# ITOOLS defines the directory containing
# idebug.btl, idump.btl, iskip.btl
ITOOLS  = malloc (100)
strcpy(ITOOLS,"ITOOLS=")
strcat(ITOOLS,getenv("OCCAM_ROOT"))
strcat(ITOOLS,"/tools")
putenv(ITOOLS)

# IBOARDSIZE defines the size of the board memory
putenv("IBOARDSIZE=#400000")

# IDEBUGSIZE defines the amount of the root transputer memory,
# it is used by idebug
putenv("IDEBUGSIZE=#400000")

#########################################
#                                       #
# define variables for folding editor f #
#                                       #
#########################################

# FTOOLS defines the directory containing inmosF.btl (renamed from INMOS f.btl)
FTOOLS  = malloc (100)
strcpy(FTOOLS,"FTOOLS=")
strcat(FTOOLS,getenv("INMOS_ROOT"))
strcat(FTOOLS,"/bin")
putenv(FTOOLS)

# ITERM defines the terminal definition file,used by idebug 
ITERM  = malloc (100)
strcpy(ITERM,"ITERM=")
strcat(ITERM,getenv("INMOS_ROOT"))
strcat(ITERM,"/bin/inmosAnsi.itm")
putenv(ITERM)

#
# FTERM and FSTART define the terminal configuration file, used by idebug 
#

FTERM  = malloc (100)
strcpy(FTERM,"FTERM=")
strcat(FTERM,getenv("INMOS_ROOT"))

FSTART = malloc (100)
strcpy(FSTART,"FSTART=")
strcat(FSTART,getenv("INMOS_ROOT"))

# A) HP version 
#    (remember to run define-termHP.sh / restore-termHP.sh !)

#strcat(FTERM,"/bin/inmosHPfold.itm")
#putenv(FTERM)

#strcat(FSTART,"/bin/inmosHPfold.hlp")
#putenv(FSTART)

# B) Tektronix version 
#    (remember to run define-termTEK.sh / restore-termTEK.sh !)

strcat(FTERM,"/bin/inmosTEKfold.itm")
putenv(FTERM)

strcat(FSTART,"/bin/inmosTEKfold.hlp")
putenv(FSTART)

#
# ISEARCH is needed by idebug (NOTE: it wants the "/" at the end)
#
ISEARCH  = malloc (100)
strcpy(ISEARCH,"ISEARCH=")
strcat(ISEARCH,getenv("OCCAM_ROOT"))
strcat(ISEARCH,"/libs/;")
strcat(ISEARCH,getenv("OCCAM_ROOT"))
strcat(ISEARCH,"/tools/;../include/;../lib/;../object/")
putenv(ISEARCH)

######################
#                    #
# variables not used #
#                    #
######################

# define OTOOLS
#OTOOLS  = malloc (100)
#strcpy(OTOOLS,"OTOOLS=")
#strcat(OTOOLS,getenv("OCCAM_ROOT"))
#strcat(OTOOLS,"/tools")
#putenv(OTOOLS)

# define TOOLS
#TOOLS  = malloc (100)
#strcpy(TOOLS,"TOOLS=")
#strcat(TOOLS,getenv("OCCAM_ROOT"))
#strcat(TOOLS,"/tools")
#putenv(TOOLS)

######################
#                    #
# load utility files #
#                    #
######################
lcubootAutoLoad 1,"inmos"

# __oOo__
