#ifndef ACECCD_H
#define ACECCD_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccd.h,v 1.14 2000/01/13 13:26:48 vltsccm Exp $"
*
* aceccd.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   27/06/94  created
* ccumani   03/11/94  readout direction definitions added
* ccumani   19/12/94  aceccdVERSION = 1.1.1
* ccumani   26/09/95  - made compatible with CMM
*                     - aceccdVERSION definition modified according to CMM
*                     - aceccdMOD_DESC and aceccdVERSION_DATE definitions
*                       added
* ccumani   20/11/95  - function prototypes added
*                     - aceccdSTAT_OPSTATE_OFF added
* ccumani   10/01/96  - aceccdRIGHT-aceccdLEFT values swapped
* ccumani   03/03/96  - aceccdVERSION_DATE set to "FEB96"
* ccumani   21/05/96  - aceccdVERSION_DATE set to "JUL96"
* ccumani   04/07/96  - error ranges modified
*                     - aceccdMOD_SCI_NAME and aceccdMOD_TEC_NAME 
*                       definitions introduced
*                     - aceccdErrAdd macro modified
* ccumani   26/08/97  - aceccdVERSION_DATE set to "NOV97"
*/

/*************************************************************************
*  This file contains the definitions of the message codes used 
*  by the LCU-Transputer Interface for CCD control application.
*************************************************************************/

#include "acecomCm.h"
#include "aceccdErrors.h"

/*
 * Destination code IDs.
 * Values 0x0000 and 0x0001 are already used by the Interface (see
 * acecom.h)
 */

#define aceccdRDOUTCONTR     (short)  0x0002
#define aceccdRDOUTRANSM     (short)  0x0003
#define aceccdSHUTTER        (short)  0x0004
#define aceccdTELEMETRY      (short)  0x0005
#define aceccdTEMPERATURE    (short)  0x0006
#define aceccdAUXILIARY      (short)  0x0007

/*
 * "no error" reply error code
 */
#define aceccdERR_NOERR      acecomERR_NOERR

/*
 * reply parameter values
 */

/* Definitions of TRUE and FALSE according to VxWorks and CCS rules */
#define aceccdTRUE        (short)   1  
#define aceccdFALSE       (short)   0

#define aceccdON                aceccdTRUE
#define aceccdOFF               aceccdFALSE

/* parameters for acecomXXSIMUL commands */
#define aceccdSIMULATION        aceccdOFF
#define aceccdNORMAL            aceccdON

/* readout direction definitions */
#define aceccdUP          (short) 1
#define aceccdDOWN        (short) (-1)
#define aceccdRIGHT       (short) 1
#define aceccdLEFT        (short) (-1)

/*
 * Status masks and values
 */

#define aceccdSTAT_SIM_MASK        (short)   0x0001
#define aceccdSTAT_HWFAIL_MASK     (short)   0x0002
#define aceccdSTAT_OPSTATE_MASK    (short)   0x000C


#define aceccdSTAT_SIM             (short)   0x0001
#define aceccdSTAT_HWFAIL          (short)   0x0002
#define aceccdSTAT_OPSTATE_OFF     (short)   0x0000
#define aceccdSTAT_OPSTATE_LOADED  (short)   0x0004
#define aceccdSTAT_OPSTATE_STANDBY (short)   0x0008
#define aceccdSTAT_OPSTATE_ONLINE  (short)   0x000C




/* LCU ONLY - BEGIN *****************************************************
 */

/* take the definition of acecomFIRST_ERROR */
#include "acecomErrors.h"

/* Constants */

#define aceccdNUMSUBSYS      6       /* number of ACE sub-systems        */
#define aceccdLINE_LENGTH  100       /* parameter definition line length */

#define aceccdMOD_NAME       "aceccd"
#define aceccdMOD_DESC       "LCU-Transputer Interface Module"
#define aceccdVERSION        "$Revision: 1.14 $"
#define aceccdVERSION_DATE   "NOV97"

/*************************************************************
 * -> IMPORTANT !!!                                          *
 * -> RANGES FOR aceccd, acetec, acesci MODULE ERROR VALUES  *
 * -> (values used by aceccdErrAdd)                          *
 * -> USE THESE OFFSET VALUES WITH errEditor,                * 
 *    IN ALL THE CCD MODULES                                 *
 * -> IMPORTANT !!!                                          *
 *************************************************************/
 
/* aceccd errors range must be: (9000 <= error number <= 9999)
 * aceccdFIRST_ERROR is defined in aceccdErrors.h
 */

/* acesci errors range must be: (8000 <= error number <= 8999) */
/* IMPORTANT :
 * the 8000 offset value must be set in the acesciErrors.h file ! 
 * IMPORTANT !!!
 */
#define aceccdFIRST_SCI_ERROR (acecomFIRST_ERROR - 2000)

/* acetec errors range MUST start from 1 (for historical reasons).
 * If no other modules are defined, acetec errors range is:
 *       1 <= error number < aceccdFIRST_SCI_ERROR 
 * i.e.: 1 <= error number <= 7999) 
 * If other modules are defined, their range must be
 *       1000 < error number < 8000 (aceccdFIRST_SCI_ERROR)
 */

#define aceccdMOD_SCI_NAME   "acesci"
#define aceccdMOD_TEC_NAME   "acetec"

/* macros */

#define aceccdErrAdd(error, reply, replyLen)                  \
    {                                                         \
    int         i, j;                                         \
    vltINT16    errNo = 0;                                    \
    vltBYTES80  errVal = "";                                  \
    vltBYTES80  errMsg = "";                                  \
    char       *modName;                                      \
					     	              \
    errNo = reply[0];                                         \
                                                              \
    if (errNo > acecomFIRST_ERROR)                            \
        modName = acecomMOD_NAME;                             \
    else if (errNo > aceccdFIRST_ERROR)                       \
        modName = aceccdMOD_NAME;                             \
    else if (errNo > aceccdFIRST_SCI_ERROR)                   \
        modName = aceccdMOD_SCI_NAME;                         \
    else                                                      \
        modName = aceccdMOD_TEC_NAME;                         \
                                                              \
    if (replyLen == sizeof(reply[0]))                         \
        errAdd(error, modName, errNo, __FILE__,               \
      	       "");                                           \
    else                                                      \
        {                                                     \
        i = (replyLen - sizeof(reply[0])) / sizeof(reply[0]); \
        for (j=1; j<=i; j++)                                  \
	    {                                                 \
            sprintf (errVal, "%hd",reply[j]);                 \
            strcat  (errMsg, errVal );                        \
	    }                                                 \
        errAdd(error, modName, errNo, __FILE__,               \
	       "%s",errMsg);                                  \
        }                                                     \
    }                          



/*
 * Function prototypes
 */

ccsCOMPL_STAT aceccdSetPar 
    (
    vltINT8   *application, 
    vltINT8   *device, 
    vltINT8   *parameter, 
    vltINT16  parValue
    );

ccsCOMPL_STAT aceccdShowPar
    (
    vltINT8  *application, 
    vltINT8  *device, 
    vltINT8  *subSystem
    );

ccsCOMPL_STAT aceccdVersion
    (
    vltBYTES80 version
    );


/* LCU ONLY - END *****************************************************
 */

#endif /*!ACECCD_H*/
