#ifndef ACECCDAU_H
#define ACECCDAU_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccdAU.h,v 1.14 2000/01/13 13:26:49 vltsccm Exp $"
*
* aceccdAU.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  created
* alongino  12/05/94  added note "IMPORTANT"
* ccumani   26/09/95  made compatible with CMM
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Auxiliary Functions control routines.
*  IMPORTANT: The related sub-system on ACE has to implement and
*             recognise ALL commands and parameters defined in this file.
*             Renaming of commands and parameters macros or changing their
*             code and/or functionality is not admitted.
*             LCU code, responsible to send commands/parameters to the ACE,
*             and ACE code, receiving them, should make usage only of macros
*             defined in order here or in aceccd.h or in acecom.h 
*             when building/parsing messages.
*             The implementation of commands and parameters other than
*             those present in this file are subject to prior approval
*             from ESO.
*             All exceptions are explicitely mentioned in this file.
*             The corresponding occam .inc file MUST be produced from
*             the present file using the utility acecomH2Inc.
*   The TCCDS does not implement any of these commands/parameters
*************************************************************************/

#include "aceccd.h"


/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

 /* reset auxiliary 
    Parameters: NONE */
#define aceccdAURESET     (long)                                \
   (((long) acecomHIPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomRESET))

/*********************************
 * Low priority command type IDs *
 *********************************/

 /* initialize auxiliary
    Parameters: self test configuration aceccdON or aceccdOFF
    Reply: error/ok message */
#define aceccdAUINIT     (long)                                 \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomINIT))

 /* receive parameter from auxiliary 
    Parameters: parameter code(s), array index (if needed) 
    Reply: error/ok message, if ok the parameter code(s) and value(s) follow */
#define aceccdAUGIVPARAM     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomGIVPARAM))

 /* send parameter to auxiliary 
    Parameters: param. code(s), array index (if needed), value(s)
    Reply: error/ok message */
#define aceccdAUSETPARAM     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomSETPARAM))

 /* put auxiliary in standby mode 
    Parameters: NONE  
    Reply: error/ok message */
#define aceccdAUSTANDBY     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomSTANDBY))

 /* read auxiliary status
    Parameters: NONE  
    Reply: error/ok message, if ok status value follows */
#define aceccdAUSTATUS     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomSTATUS))

 /* switch into/return from simulation mode
    Parameters: aceccdNORMAL or aceccdSIMULATION
    Reply: error/ok message */
#define aceccdAUSIM        (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdAUXILIARY << 16) \
   | ((long) acecomSIMUL))

/* LCU ONLY - END *****************************************************
 */


#endif /*!ACECCDAU_H*/
