#ifndef ACECCDRC_H
#define ACECCDRC_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccdRC.h,v 1.14 2000/01/13 13:26:49 vltsccm Exp $"
*
* aceccdRC.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   15/06/94  created
* ccumani   03/11/94  - aceccdRCDIRECTION separated into 
*                       aceccdRCHORDIRECTION and aceccdRCVERDIRECTION 
*                     - aceccdRCSETGAIN deleted 
* ccumani   26/09/95  made compatible with CMM
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Readout Control routines.
*************************************************************************/

#include "aceccd.h"

/* max # of CCDs */
#define aceccdMAXCCD      16
/* max # of ADCs */
#define aceccdMAXADC      64
/* max # of image windows */
#define aceccdMAXWIN      20
/* max # of clock patterns */
#define aceccdMAXPAT      10


/* Subsystem dependent Byte of Status parameter:
 * ID definitions:
 */
 
/* First subsystem dependent status bit: ClockLoad */
#define aceccdRCSTATUSCLCKLOAD  (short)  0x0100

/* Second subsystem dependent status bit: Reading Status */
#define aceccdRCSTATUSREADING   (short)  0x0200



/*
 * Parameters for the aceccdRCGIVPARAM and aceccdRCSETPARAM commands
 */

/* BEGIN PARAMETERS */

/* STATUS: result of self check: bit 0 RAM, bit 1 Peltier, bit 3 port, bit 4 register */
#define aceccdRCSELFCHECK       (short)   1

/* STATUS: last read and stored line per output */
#define aceccdRCLASTLINE        (short)   2

/* STATUS: next read line to be transmitted per output */
#define aceccdRCNEXTLINE        (short)   3

/* STATUS: ADC readout gain per chip output */
#define aceccdRCGAIN            (short)   4

/* CONFIGURATION: horizontal chips number in mosaic */
#define aceccdRCHCHIP           (short)   5

/* CONFIGURATION: vertical chips number in mosaic */
#define aceccdRCVCHIP           (short)   6

/* CONFIGURATION: CCD type (0=normal, 1=frame transfer) per chip */
#define aceccdRCCCDTYPE         (short)   7

/* CONFIGURATION: horizontal pixels per chip */
#define aceccdRCHPIX            (short)   8

/* CONFIGURATION: horizontal prescan length per chip */
#define aceccdRCHPRESCAN        (short)   9

/* CONFIGURATION: horizontal overscan per chip */
#define aceccdRCHOVERSCAN       (short)  10

/* CONFIGURATION: vertical lines per chip */
#define aceccdRCVLINES          (short)  11

/* CONFIGURATION: vertical prescan length per chip */
#define aceccdRCVPRESCAN        (short)  12

/* CONFIGURATION: vertical overscan per chip */
#define aceccdRCVOVERSCAN       (short)  13

/* CONFIGURATION: number of outputs per chip */
#define aceccdRCNUMOUTPUTS      (short)  14

/* CONFIGURATION: horizontal output location per chip */
#define aceccdRCOUTPUTHORLOC    (short)  15

/* CONFIGURATION: vertical output location per chip */
#define aceccdRCOUTPUTVERLOC    (short)  16

/* CONFIGURATION: skipper CCD bit reading number per chip */
#define aceccdRCSKIPPEREAD      (short)  17

/* CONFIGURATION: MPP mode per chip */
#define aceccdRCMPP             (short)  18

/* CONFIGURATION: ID of the chip output */
#define aceccdRCOUTPUTID        (short)  19

/* CONFIGURATION: output enabled or not */
#define aceccdRCOUTPUTENABLED   (short)  20

/* CONFIGURATION: bits for pixel per output (ADC resolution) */
#define aceccdRCBITPIX          (short)  21

/* CONFIGURATION: total horizontal pixels number per output */
#define aceccdRCOUTPUTTOTHOR    (short)  22

/* CONFIGURATION: horizontal prescan per output */
#define aceccdRCOUTPUTHORPRE    (short)  23

/* CONFIGURATION: horizontal overscan per output */
#define aceccdRCOUTPUTHOROVER   (short)  24

/* CONFIGURATION: total vertical pixels number per output */
#define aceccdRCOUTPUTTOTVER    (short)  25

/* CONFIGURATION: vertical prescan per output */
#define aceccdRCOUTPUTVERPRE    (short)  26

/* CONFIGURATION: vertical overscan per output */
#define aceccdRCOUTPUTVEROVER   (short)  27

/* CONFIGURATION: performed horizontal readout direction per output */
#define aceccdRCHORDIRECTION    (short)  28

/* CONFIGURATION: performed vertical readout direction per output */
#define aceccdRCVERDIRECTION    (short)  29

/* CONFIGURATION: data representation of image on LCU */
#define aceccdRCFRAMEREPR       (short)  30

/* CONFIGURATION: index of first connected output per board */
#define aceccdRCBOARDOUT1       (short)  31

/* CONFIGURATION: index of second connected output per board */
#define aceccdRCBOARDOUT2       (short)  32

/* CONFIGURATION: index of third connected output per board */
#define aceccdRCBOARDOUT3       (short)  33

/* CONFIGURATION: index of fourth connected output per board */
#define aceccdRCBOARDOUT4       (short)  34

/* CONFIGURATION: timeout */
#define aceccdRCTIMEOUT         (short)  35

/* SETUP: replied wiping number per chip */
#define aceccdRCWIPENUM         (short)  36

/* SETUP: replied wiping delay [sec] per chip */
#define aceccdRCWIPEDLY         (short)  37

/* SETUP: horizontal binning in wipe operation per chip */
#define aceccdRCWIPEHORBIN      (short)  38

/* SETUP: vertical binning in wipe operation per chip */
#define aceccdRCWIPEVERBIN      (short)  39

/* SETUP: clock pattern label per output */
#define aceccdRCCLOCKPATTERN    (short)  40

/* SETUP: readout subpattern label per output */
#define aceccdRCSUBPATTERN      (short)  41

/* SETUP: horizontal binning factor per window */
#define aceccdRCHBIN            (short)  42

/* SETUP: verical binning factor per window */
#define aceccdRCVBIN            (short)  43

/* SETUP: horizontal direction of first pixel per window */
#define aceccdRCWINDHORSTRT     (short)  44

/* SETUP: vertical direction of first pixel per window */
#define aceccdRCWINDVERSTRT     (short)  45

/* SETUP: horizontal direction of last pixel per window */
#define aceccdRCWINDHORSTOP     (short)  46

/* SETUP: vertical direction of last pixel per window */
#define aceccdRCWINDVERSTOP     (short)  47

/* SETUP: current size of window array per window */
#define aceccdRCWNDCNT          (short)  48

/* SETUP: readout of window enabled */
#define aceccdRCWINENABLE       (short)  49

/* SETUP: number of times exposure must be repeated (0 = infinite loop) */
#define aceccdRCEXPOSNUM        (short)  50

/* END PARAMETERS */

/*
 * Readout commands
 * Note: codes from 00 to 20 are for
 *       commands in common with other processes.
 */

#define aceccdREADROW       (short)    21
#define aceccdREADIMAGE     (short)    22
#define aceccdCOMMSTOP      (short)    23
#define aceccdSKIPUP        (short)    24
#define aceccdSKIPDOWN      (short)    25
#define aceccdSHIFTUP       (short)    26
#define aceccdSHIFTDOWN     (short)    27
#define aceccdWIPE          (short)    28
#define aceccdEXPOSE        (short)    29
#define aceccdPATTERN       (short)    30


/* LCU ONLY - BEGIN *****************************************************
 */


/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

 /* reset readout control 
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdRCRESET     (long)                                 \
   (((long) acecomHIPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomRESET))

 /* break readout control
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdRCBREAK     (long)                                 \
   (((long) acecomHIPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomBREAK))


/*********************************
 * Low priority command type IDs *
 *********************************/

 /* initialize readout control
    Parameters: self test configuration aceccdON or aceccdOFF
    Reply: error/ok message */
#define aceccdRCINIT     (long)                                  \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomINIT))

 /* receive parameter from readout control
    Parameters: param code(s), array index (if needed)
    Reply: error/ok message, if ok the parameter code(s) and value(s) follow */
#define aceccdRCGIVPARAM     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomGIVPARAM))

 /* send params to readout control
    Parameters: param. code(s), array index (if needed), parameter value(s)
    Reply: error/ok message */
#define aceccdRCSETPARAM     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomSETPARAM))

 /* read readout control status
    Parameters: NONE
    Reply: error/ok message, if ok status value follows */
#define aceccdRCSTATUS     (long)                                \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomSTATUS))

 /* switch into/return from simulation mode
    Parameters: aceccdNORMAL or aceccdSIMULATION
    Reply: error/ok message */
#define aceccdRCSIM        (long)                                \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomSIMUL))

 /* put readout control in standby mode 
    Parameters: NONE
    Reply: error/ok message */
#define aceccdRCSTANDBY     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) acecomSTANDBY))

 /* stop repeated command 
    Parameters: command to be stopped
    Reply: error/ok message */
#define aceccdRCCOMMSTOP     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdCOMMSTOP))

 /* skip up n CCD rows 
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCSKIPUPROW     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdSKIPUP))

 /* skip down n CCD rows 
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCSKIPDOWNROW     (long)                           \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdSKIPDOWN))

 /* shift up n CCD rows 
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCSHIFTUPROW     (long)                            \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdSHIFTUP))

 /* shift down n CCD rows 
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCSHIFTDOWNROW     (long)                          \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdSHIFTDOWN))

 /* wipe CCD 
    Parameters: number of times, interval time [sec]
    Reply: error/ok message */
#define aceccdRCWIPE     (long)                                  \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdWIPE))

 /* read image on dedicated link
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdRCREADIMAGE     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdREADIMAGE))

 /* read image row on dedicated link
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCREADROW     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdREADROW))

 /* read image on the same command/reply link
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdRCREADIMAGELOCK   (long)                               \
   (((long) acecomLOCKLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdREADIMAGE))

 /* read image row the same command/reply link
    Parameters: number of rows
    Reply: error/ok message */
#define aceccdRCREADROWLOCK     (long)                               \
   (((long) acecomLOCKLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdREADROW))

 /* bring CCD to integrating conditions and delay for a specified time
    Parameters: time of integration, readout wished or not
    Reply: error/ok message */
#define aceccdRCEXPOSE      (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdEXPOSE))

 /* load clock pattern 
    Parameters: ID of clock pattern
    Reply: error/ok message */
#define aceccdRCPATTERN     (long)                               \
   (((long) (acecomLOPRI | acecomNOTSWAPPARAM) << 24) \
   | ((long) aceccdRDOUTCONTR << 16) \
   | ((long) aceccdPATTERN))

/* LCU ONLY - END *****************************************************
 */

#endif /*!ACECCDRC_H*/
