#ifndef ACECCDSH_H
#define ACECCDSH_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccdSH.h,v 1.14 2000/01/13 13:26:50 vltsccm Exp $"
*
* aceccdSH.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  created
* alongino  12/05/94  added note "IMPORTANT"
* ccumani   26/09/95  made compatible with CMM
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Shutter control routines.
*  IMPORTANT: The related sub-system on ACE has to implement and
*             recognise ALL commands and parameters defined in this file.
*             Renaming of commands and parameters macros or changing their
*             code and/or functionality is not admitted.
*             LCU code, responsible to send commands/parameters to the ACE,
*             and ACE code, receiving them, should make usage only of macros
*             defined in order here or in aceccd.h or in acecom.h 
*             when building/parsing messages.
*             The implementation of commands and parameters other than
*             those present in this file are subject to prior approval
*             from ESO.
*             All exceptions are explicitely mentioned in this file.
*             The corresponding occam .inc file MUST be produced from
*             the present file using the utility acecomH2Inc.
*   The TCCDS does not implement any of these commands/parameters
*************************************************************************/

#include "aceccd.h"

/*
 * Shutter commands
 * Note: codes from 00 to 20 are for
 *       commands in common with other processes.
 */

#define aceccdOPEN       (short)    21
#define aceccdCLOSE      (short)    22


/* Subsystem dependent Byte of Status parameter:
 * ID definitions:
 */
 
/* First subsystem dependent status bit: ShuttOpen */
#define aceccdSTAT_SHOPEN  (short)  0x0100



/*
 * Parameters for the aceccdSHGIVPARAM and aceccdSHSETPARAM commands
 */


/* BEGIN PARAMETERS */

/* Note: all STATUS parameters are read-only. All CONFIGURATION and SETUP
         parameters are read/write */

/* CONFIGURATION: shutter timeout (in seconds) */
#define aceccdSHTIMEOUT     (short)  1

/* END PARAMETERS */

/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

 /* reset shutter 
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdSHRESET     (long)                              \
   (((long) acecomHIPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomRESET))

 /* break exposition (open shutter command with msec. parameter)
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdSHBREAK     (long)                              \
   (((long) acecomHIPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomBREAK))


/*********************************
 * Low priority command type IDs *
 *********************************/

 /* initialize shutter 
    Parameters: self test configuration aceccdON or aceccdOFF
    Reply: error/ok message */
#define aceccdSHINIT     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomINIT))

 /* receive parameter from shutter 
    Parameters:  parameter code(s), array index (if needed) 
    Reply: error/ok message, if ok the parameter code(s) and value(s) follow */
#define aceccdSHGIVPARAM     (long)                           \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomGIVPARAM))

 /* send params to shutter 
    Parameters: param. code(s), array index (if needed), value(s)
    Reply: error/ok message */
#define aceccdSHSETPARAM     (long)                           \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomSETPARAM))

 /* put shutter in standby mode 
    Parameters: NONE  
    Reply: error/ok message */
#define aceccdSHSTANDBY     (long)                            \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomSTANDBY))

 /* read shutter status
    Parameters: NONE  
    Reply: error/ok message, if ok status value follows */
#define aceccdSHSTATUS     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomSTATUS))

 /* switch into/return from simulation mode
    Parameters: aceccdNORMAL or aceccdSIMULATION
    Reply: error/ok message */
#define aceccdSHSIM        (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) acecomSIMUL))

 /* open shutter 
    Parameters: number of msec. after then the shutter must be close.
                (NO PARAMETER means open forever [until a close shutter
                command]) 
    Reply: error/ok message */
#define aceccdSHOPEN     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) aceccdOPEN))

 /* close shutter 
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdSHCLOSE     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdSHUTTER << 16) \
   | ((long) aceccdCLOSE))


#endif /*!ACECCDSH_H*/
