#ifndef ACECCDTL_H
#define ACECCDTL_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccdTL.h,v 1.14 2000/01/13 13:26:50 vltsccm Exp $"
*
* aceccdTL.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  created
* ccumani   26/09/95  made compatible with CMM
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Telemetry control routines.
*************************************************************************/

#include "aceccd.h"

/*
 * Parameters for the aceccdTLGIVPARAM and aceccdTLSETPARAM commands
 */

/* BEGIN PARAMETERS */

/* STATUS: current value per sensor */
#define aceccdTLVALUE       (short)   1

/* CONFIGURATION: number of sensors */
#define aceccdTLSENSNUM     (short)   2

/* CONFIGURATION: availability of each single sensor */
#define aceccdTLAVAIL       (short)   3

/* CONFIGURATION: sensor enabled or not by the user */
#define aceccdTLENABLED     (short)   4

/* CONFIGURATION: units value per sensor */
#define aceccdTLUNITS       (short)   5

/* CONFIGURATION: reference value per sensor */
#define aceccdTLREFERENCE   (short)   6

/* CONFIGURATION: tolerance per sensor */
#define aceccdTLTOLERANCE   (short)   7

/* CONFIGURATION: simulation value for sensor */
#define aceccdTLSIMVAL      (short)   8

/* END PARAMETERS */


/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

/************************************************************************
 * Define command codes with bitwise inclusive OR operator |            *
 ************************************************************************/

 /* reset telemetry 
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdTLRESET     (long)                                \
   (((long) acecomHIPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomRESET))

/*********************************
 * Low priority command type IDs *
 *********************************/

 /* initialize telemetry
    Parameters: self test configuration aceccdON or aceccdOFF
    Reply: error/ok message */
#define aceccdTLINIT     (long)                                 \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomINIT))

 /* receive parameter from telemetry
    Parameters: parameter code(s), array index (if needed) 
    Reply: error/ok message, if ok the parameter code(s) and value(s) follow */
#define aceccdTLGIVPARAM     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomGIVPARAM))

 /* send parameter to telemetry 
    Parameters: param. code(s), array index (if needed), value(s)
    Reply: error/ok message */
#define aceccdTLSETPARAM     (long)                             \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomSETPARAM))

 /* put telemetry in standby mode 
    Parameters: NONE  
    Reply: error/ok message */
#define aceccdTLSTANDBY     (long)                              \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomSTANDBY))

 /* read telemetry status
    Parameters: NONE  
    Reply: error/ok message, if ok status value follows */
#define aceccdTLSTATUS     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomSTATUS))

 /* switch into/return from simulation mode
    Parameters: aceccdNORMAL or aceccdSIMULATION
    Reply: error/ok message */
#define aceccdTLSIM        (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomSIMUL))

 /* check if specified parameter values are in a specified range
    Parameters: parameter to check  
    Reply: error/ok message */
#define aceccdTLCHECK      (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomCHECK))

 /* give all telemetry parameters followed by their values
    Parameters: NONE  
    Reply: error/ok message, if ok parameter(s) and value(s) follow */
#define aceccdTLALLPAR     (long)                               \
   (((long) acecomLOPRI << 24) | ((long) aceccdTELEMETRY << 16) \
   | ((long) acecomALLPAR))


#endif /*!ACECCDTL_H*/
