#ifndef ACECCDTM_H
#define ACECCDMP_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aceccdTM.h,v 1.14 2000/01/13 13:26:50 vltsccm Exp $"
*
* aceccdTM.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  created
* ccumani   26/09/95  made compatible with CMM
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Temperature control routines.
*************************************************************************/

#include "aceccd.h"

/*
 * Parameters for the aceccdTMGIVPARAM and aceccdTMSETPARAM commands
 */

/* BEGIN PARAMETERS */

/* STATUS: result of self check: bit 0 RAM, bit 1 Peltier, bit 3 port, bit 4 register */
#define aceccdTMSELFCHECK   (short)   1

/* STATUS: current house temperature per sensor */
#define aceccdTMHOUSETEMP   (short)   2

/* CONFIGURATION: number of sensors */
#define aceccdTMSENSNUM     (short)   3

/* CONFIGURATION: availability of each single sensor */
#define aceccdTMAVAIL       (short)   4

/* CONFIGURATION: sensor enabled or not by the user */
#define aceccdTMENABLED     (short)   5

/* CONFIGURATION: reference temperature value (kelvin) per sensor */
#define aceccdTMREFERENCE   (short)   6

/* CONFIGURATION: tolerance according to reference temperature (low) per sensor */
#define aceccdTMLOWTOL      (short)   7

/* CONFIGURATION: tolerance according to reference temperature (high) per sensor */
#define aceccdTMHIGHTOL     (short)   8

/* CONFIGURATION: lowest value accepted as reference temperature per sensor */
#define aceccdTMLOWRANGE    (short)   9

/* CONFIGURATION: highest value accepted as reference temperature per sensor */
#define aceccdTMHIGHRANGE   (short)   10

/* CONFIGURATION: time (ms) between two temperature measurements */
#define aceccdTMCYCLETIME   (short)   11

/* CONFIGURATION: minimal time for cycle */
#define aceccdTMMINTIME     (short)   12

/* CONFIGURATION: maximal time for cycle */
#define aceccdTMMAXTIME     (short)   13

/* SETUP: current chip temperature per sensor */
#define aceccdTMCHIPTEMP    (short)   14

/* END PARAMETERS */


/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

 /* reset temperature 
    Parameters: NONE 
    Reply: error/ok message */
#define aceccdTMRESET     (long)                                  \
   (((long) acecomHIPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomRESET))
 
 
/*********************************
 * Low priority command type IDs *
 *********************************/

 /* initialize temperature
    Parameters: self test configuration aceccdON or aceccdOFF
    Reply: error/ok message */
#define aceccdTMINIT     (long)                                   \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomINIT))

 /* receive parameters from temperature 
    Parameters: param code(s), array index (if needed)
    Reply: error/ok message, if ok the parameter code(s) and value(s) follow */
#define aceccdTMGIVPARAM                                          \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomGIVPARAM))

 /* send parameters to temp 
    Parameters: param. code(s), array index (if needed), parameter value(s)
    Reply: error/ok message */
#define aceccdTMSETPARAM                                          \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomSETPARAM))

 /* put temperature in standby mode 
    Parameters: NONE
    Reply: error/ok message */
#define aceccdTMSTANDBY                                           \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomSTANDBY))

 /* read temperature status
    Parameters: NONE
    Reply: error/ok message, if ok status value follows */
#define aceccdTMSTATUS                                            \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomSTATUS))

 /* switch into/return from simulation mode
    Parameters: aceccdNORMAL or aceccdSIMULATION
    Reply: error/ok message */
#define aceccdTMSIM        (long)                                 \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomSIMUL))

 /* check if specified parameter values are in a specified range
    Parameters: parameter to check  
    Reply: error/ok message */
#define aceccdTMCHECK      (long)                                 \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomCHECK))

 /* give all temperature parameters followed by their values
    Parameters: NONE  
    Reply: error/ok message, if ok parameter(s) and value(s) follow */
#define aceccdTMALLPAR     (long)                                 \
   (((long) acecomLOPRI << 24) | ((long) aceccdTEMPERATURE << 16) \
   | ((long) acecomALLPAR))


/* LCU ONLY - END *****************************************************
 */

#endif /*!ACECCDTM_H*/
