#ifndef ACECOM_H
#define ACECOM_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: acecom.h,v 1.36 2003/10/21 14:51:36 vltsccm Exp $"
*
* acecom.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  - created (acecomVERSION = 1.1.1)
* nfiebig   23/06/94  - added prototypes of acecomAttach, acecomDetach, 
*                       acecomClean, acecomSetLogging and acecomInstall
*                     - channel handle structure modified
* ccumani   07/10/94  - acecomMEGA definition deleted (not compatible
*                       with 16 bits transputers)
*                     - status definitions moved into aceccd.h
*                     - Command Dispatcher and Channel Handler
*                       initialization status parameters added
*                     - command codes corrected to avoid overlapping
*                       between high and low priority command codes
*                     - acecomLAST_ERROR definition introduced
*                     - acecomEr.h inclusion eliminated
* ccumani   24/10/94  - acecomMAXBUFLEN definition corrected
*                     - acecomMAXBIGLEN definition added
* ccumani   22/11/94  - acecomPendingBytes function prototype added
* ccumani   19/12/94  - acecomVERSION = 1.1.2
* ccumani   07/02/95  - acecomERR_NOERR moved here from acecomError.h
*                       for compatibility with new errEditor version
* alongino  12/05/95  - added comment on usage of acecomH2Inc
* ccumani   30/05/95  - acecomAttachVideo, acecomDowload, acecomSetInteractive
*                       prototypes added
*                     - acecomVERSION = 1.1.3
* ccumani   27/09/95  - made compatible with CMM
*                     - acecomMOD_DESC and acecomVERSION_DATE definitions
*                       added
* ccumani   29/09/95  - acecomGiveHwBs prototype added
* ccumani   04/10/95  - acecomSWAP_LONG definition deleted
* ccumani   14/11/95  - acecomMAXBIGLEN definition deleted
* ccumani   03/03/96  - acecomVERSION_DATE set to "FEB96"
* ccumani   21/05/96  - acecomVERSION_DATE set to "JUL96"
* ccumani   04/07/96  - error range changed from 1-acecomLAST_ERROR
*                       to acecomFIRST_ERROR on
*                     - acecomLAST_ERROR deleted
*                     - NOTE: acecomFIRST_ERROR defined in acecomErrors.h
* ccumani   30/07/96  - acecomDUMMY macro added
* ccumani   16/08/96  - acecomVERSION_DATE set to "NOV96"
* ccumani   02/12/96  - acecomVERSION_DATE set to "APR97"
* ccumani   26/08/97  - acecomVERSION_DATE set to "NOV97"
*/

/*************************************************************************
*  This file contains the definitions of the message codes used 
*  by the LCU-Transputer Interface.
*  The corresponding occam .inc file MUST be produced from
*  the present file using the utility acecomH2Inc
*************************************************************************/

#include "acecomErrors.h"

/*
 * general constants
 */
#define acecomKILO          (int)   1024
#define acecomHALFKILO      (int)   (acecomKILO/2)

/*
 * LCU-Transputer message constants
 */
/* max length for commands (BYTE arrays) */
#define acecomMAXCOMLEN   (int)   acecomHALFKILO
/* max length for replies (BYTE arrays) */
#define acecomMAXREPLEN   (int)   acecomHALFKILO
/* max length for commands (INT16 arrays) */
#define acecomMAXMESLEN   (int)   (acecomMAXCOMLEN/2)

/* max length for data packets (BYTE arrays) */
/* it corresponds to the "biggest, 2bytes long, signed, even" integer */
#define acecomMAXBUFLEN   (int)   32766

/*
 * link table dimension: it must be >= (TOT.PROCS + 2) - see .pgm file
 */
#define acecomLINKTABDIM  (int)   20

/*
 * number of max input channels for Channel Handlers
 */
#define acecomMAXCHCHAN   (int)   4

/*
 * Timeout for Transputer initialization :
 * 15625 tics = 1 sec --> 3125 tics = 0.2 sec.
 */
#define acecomTRANSPTIMEOUT  (int)   3125

/* Command Dispatcher and Channel Handler initialization status */
/* acecomINITERROR must be negative, not to be confused with a LinkTable
   index by the Command Dispatcher */
#define acecomINITOK        (short)    0
#define acecomINITERROR     (short)  (-1)

/*
 * Command Dispatcher SUCCESS "error" reply code
 */
#define  acecomERR_NOERR    (short)    0

/*
 * Commands are so organized:
 *   * a 16 bits integer (the "Precode") with 
 *       "priority", "lock" and "swap" bits
 *       and the other 8 bits are the "destination"
 *   * a 16 bits integer with the "Command Code"
 */

/*
 * A) Precode :
 */

/*
 * priority IDs.
 */

#define acecomHIPRI          (char)   0x00
#define acecomLOPRI          (char)   0x01
#define acecomLOCK           (char)   0x02
#define acecomLOCKLOPRI      (char)   (((int) acecomLOPRI) | ((int) acecomLOCK))

/*
 * swap IDs:
 * acecomNOTSWAPPARAM  set means that parameters must not be swapped
 *                     (only message length and command are swapped)
 * acecomNOTSWAPREPLY  set means that reply must not be swapped
 */
#define acecomNOTSWAPPARAM   (char)   0x04
#define acecomNOTSWAPREPLY   (char)   0x08


/*
 * Destination codes IDs.
 */

#define acecomCOMDISPATCH    (short)  0x0000
#define acecomBACKREPLY      (short)  0x0001

/*
 * B) Command Code :
 */

/*
 * High priority "Command Code" IDs.
 */

#define acecomRESET          (short)   0
#define acecomBREAK          (short)   1


/*
 * Low priority Command Code IDs common to more than one specialized
 * program (see the appropriate header file for other commands).
 */

#define acecomINIT           (short)   2
#define acecomGIVPARAM       (short)   3
#define acecomSETPARAM       (short)   4
#define acecomSTANDBY        (short)   5
#define acecomSTATUS         (short)   6
#define acecomSIMUL          (short)   7
#define acecomCHECK          (short)   8
#define acecomALLPAR         (short)   9


/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * System and general LCU header files
 */
#include "ccs.h"
#include "err.h"
#include "semLib.h"

/*
 * Constants
 */
/* swapping options for the acecomAttach function */
#define acecomNO_SWAP    (short)   0   /* no swapping                  */
#define acecomSWAP_SHORT (short)   2   /* swap array elements 2 by 2   */

#define acecomMOD_NAME       "acecom"       /* for logging             */
#define acecomMOD_DESC       "LCU-Transputer Interface Module"
#define acecomVERSION        "$Revision: 1.36 $"
#define acecomVERSION_DATE   "NOV2000"

#define acecomDUMMY          ""        /* used by errDriverAdd calls   */

#define acecomLogId        130         /* for logging - must be > 100  */

#define acecomMAX_CHANNELS  64         /* max number link channels     */

/*
 * function prototypes for the LCU-Transputer Interface.
 */

ccsCOMPL_STAT acecomAttach
    (
    vltBYTES20    application,
    vltBYTES20    deviceName,
    vltINT32      timeout,
    vltINT16      hwSwap,
    vltINT32      *handle,
    ccsERROR      *error
    );

ccsCOMPL_STAT acecomAttachVideo
    (
    vltBYTES20    application,
    vltBYTES20    deviceName,
    vltINT32      timeout,
    vltINT16      hwSwap,
    vltINT32      *handle,
    ccsERROR      *error
    );

ccsCOMPL_STAT acecomClean ( void );
    
ccsCOMPL_STAT acecomDetach
    (
    vltINT32      handle,
    ccsERROR      *error
    );

ccsCOMPL_STAT acecomDownload
    (
    vltBYTES20    trProgram,
    vltBYTES20    deviceName
    );

ccsCOMPL_STAT acecomGiveHwBs
    ( 
    vltINT32      handle,
    vltINT16      *hwByteSwap,
    ccsERROR      *error 
    );

ccsCOMPL_STAT acecomInstall ( void );

    
ccsCOMPL_STAT acecomPendingBytes
    (
    vltINT32      handle,
    vltINT32      *pendingBytes,
    ccsERROR      *error
    );

ccsCOMPL_STAT acecomReceiveData 
    (
    vltINT32      chanHd, 
    vltINT32      expectLen, 
    vltINT16      swap,
    vltINT16      *dataBuff, 
    vltINT32      *readLen, 
    ccsERROR      *error
    );
    
ccsCOMPL_STAT acecomSendCommand 
    (
    vltINT32      chanHd, 
    vltINT16      *messageBuff, 
    vltINT16      *replyBuff, 
    vltINT16      *replyBuffLen,
    ccsERROR      *error
    );

ccsCOMPL_STAT acecomSetInteractive
    (
    char deviceName[30],
    int  verbose 
    );

ccsCOMPL_STAT acecomSetLogging
    (
    vltBYTES20    deviceName,
    vltLOGICAL    onOff
    );

ccsCOMPL_STAT acecomVersion( vltBYTES80 version );
    
/* LCU ONLY - END *****************************************************
 */
ccsCOMPL_STAT acecom0 ( char deviceName[30] );


#endif /*!ACECOM_H*/
