#ifndef ACECOMCM_H
#define ACECOMCM_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: acecomCm.h,v 1.36 2003/10/21 14:51:36 vltsccm Exp $"
*
* acecomCm.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   17/01/94  created
* ccumani   07/10/94  parameters definitions rewritten
* ccumani   27/09/95  made compatible with CMM
* ccumani   15/01/97  acecomCDSLEEP command added
* pduhoux   26/05/98  acecomCDCOHERENCE command added
*/

/*************************************************************************
*  This file contains the definitions of the commands used by the 
*  LCU-Transputer Interface Command Dispatcher.
*************************************************************************/

#include "acecom.h"

/*
 * Command Dispatcher commands
 * Note: codes from 00 to 20 are for 
 *       commands in common with other processes.
 */

#define acecomTERMINATE   (short) 21
#define acecomCHANGEMODE  (short) 22 
#define acecomSLEEP       (short) 23 
#define acecomCOHERENCE   (short) 24 

/*
 * Parameters for the acecomCDGIVPARAM command
 */

/* BEGIN PARAMETERS */

/* initialization status: replies are acecomINITERROR or acecomINITOK */
#define acecomCDINITSTATUS  (short)  1

/* link # for specialized process: the spec. process destination must follow */
#define acecomCDLINK        (short)  2

/* # of present specialized processes */
#define acecomCDSPECPROCS   (short)  3

/* link table: reply are link table elements indexes and values */
#define acecomCDLINKTABLE   (short)  4

/* END PARAMETERS */

/* LCU ONLY - BEGIN *****************************************************
 */

/*
 * Define command codes with bitwise inclusive OR operator |
 */

/**************************
 * High priority commands *
 **************************/

 /* reset command dispatcher 
    Parameters: NONE */
#define acecomCDRESET     (long)                   \
   (((long)  acecomHIPRI << 24)                    \
   | ((long) acecomCOMDISPATCH << 16)              \
   | ((long) acecomRESET))

/*********************************
 * Low priority command type IDs *
 *********************************/

 /* receive parameter from command dispatcher 
    Parameters: number of parameters, param codes, array index (if needed) */
#define acecomCDGIVPARAM    (long)                 \
   (((long) acecomLOPRI << 24)                     \
   | ((long) acecomCOMDISPATCH << 16)              \
   | ((long) acecomGIVPARAM))

 /* running terminate 
    Parameters: NONE */
#define acecomCDTERMINATE   (long)                 \
   (((long) acecomLOPRI << 24)                     \
   | ((long) acecomCOMDISPATCH << 16)              \
   | ((long) acecomTERMINATE))

 /* set debug mode 
    Parameters: 0 for normal mode, otherwise debug mode */
#define acecomCDCHANGEMODE   (long)                \
   (((long) acecomLOPRI << 24)                     \
   | ((long) acecomCOMDISPATCH << 16)              \
   | ((long) acecomCHANGEMODE))

 /* wait N tenth of millisecond
    Parameters: number of tenths of millisecond to wait, 
                before sending the reply 
    NOTE :the range of the sleep time goes 
          from 0.1 msec to 3.2 sec  */
#define acecomCDSLEEP   (long)                     \
   (((long) acecomLOPRI << 24)                     \
   | ((long) acecomCOMDISPATCH << 16)              \
   | ((long) acecomSLEEP))

 /* execute sequence of commands for Coherence mode
    Parameters :
    <rep>     : number of stripes [range 2 to 10]
    <preWipe> <cmdWipe> : wipe precode + code
    <preShft> <cmdShft> <linShft> : line shift precode + code + lines
                        <linShft> : [range 1 to <linRead>]
    <preRead> <cmdRead> <linRead> : line read  precode + code + lines
                        <linRead> : [range 1 to <rep>*<linShft>]
    <uit>     : expTime/stripe in 10us [range 0 to 10000]
    <ref>     : Y ref of central stripe [range 1 to <linRead>]
 */
#define acecomCDCOHERENCE   (long)                 \
   (((long) acecomLOPRI << 24)        |            \
    ((long) acecomCOMDISPATCH << 16)  |            \
    ((long) acecomCOHERENCE))

/* LCU ONLY - END *****************************************************
 */

#endif /*!ACECOMCM_H*/
