#ifndef ACETEC_H
#define ACETEC_H
/*******************************************************************
* E.S.O. - VLT project
*
* acetec.h
* SccsId: "@(#)acetec.h	10.1 97/09/03 10:56:27"
*
* who       when      what
* --------  --------  ----------------------------------------------
* Hoffmann  28/10/94  created
* jahn	    11/09/95  created (acetecVERSION = 1.1.1)
* jahn	    12/09/95  added CCD definitions
*		      changed acetecInitCCD.ini file
* jahn	    19/09/95  ace include part added
* jahn	    01/11/95  relative PATH-names for *.btl- files and
*                     DSP- files deleted
* Hoffmann  01/11/95  error-structure passed to 'acetecInit'
* jahn	    10/09/95  changed acetecAPP --> acetecMOD_NAME
*		      added acetecPROC_NAME
* jahn	    14/11/95  added acetecDeregisterFailureRoutine (),
*                           acetecDetachFailureRoutine (),
*			    acetecCloseHandling ()
* jahn	    20/11/95  added header variables:
*			    acetecACE_SYSTEM, acetecPROTOTYPE, acetecSERIES
*			    acetecCCD_TYPE, acetecSMALL, acetecLARGE
* jahn	    17/01/96  SCCS Version string added
* Hoffmann  22/01/96  environments "ACE_INI" and "ACE_BIN" added
* Hoffmann  23/01/96  masks for selfcheck word added
* Hoffmann  14/02/96  automatical increment of 'acetecVersion'
* Hoffmann  11/03/96  definition for 'acetecNOTARRAY' added
* jahn	    30/07/96  changed acetecSERVER_STACK from 20000 to 80000
* jahn	    12/08/96  cit and cdt files renamed to acetecServer.cdt
*                     abd acetecServer.cit
*
*/

/*******************************************************************
*  This file contains the definitions of the message codes used 
*  by the TCCDS System Test sw
*  The corresponding occam .inc file MUST be produced from
*  the present file using the utility acecomH2Inc
********************************************************************/


/*
 * general constants
 */
#define acetecKILO          (int)   1024
#define acetecHALFKILO      (int)   (acetecKILO/2)
/* mark a test as failed	    */
#define acetecTEST_FAILED   (short) 1 
/* mark a test as passed	    */		
#define acetecTEST_PASSED   (short) 0
/* no error			    */
#define acetecERR_OK        (short) 0 
/* parameter definition as scalar   */
#define acetecNOTARRAY      (short) -1 

/*
 * constants for range checks in RTP and RCP
 */

/* min. pixel number per line 	*/
#define acetecMINHACTPIX	(short)  1
/* max. pixel number per line */
#define acetecMAXHACTPIX	(short)  600
/* min. line number 		*/
#define acetecMINVLINES	(short)  1
/* max. line number 		*/
#define acetecMAXVLINES	(short)  600
/* min. binning faktor		*/
#define acetecMINBIN		(short)  1
/* max. binning faktor		*/
#define acetecMAXBIN		(short)  8
/* min. gain			*/
#define acetecMINGAIN		(short)  1
/* max. gain			*/
#define acetecMAXGAIN		(short)  8
/* min. timeout in ms		*/
#define acetecMINTIMEOUT	(short)  5
/* max. timeout in ms		*/
#define acetecMAXTIMEOUT	(short)  5000
/* no. of chips horizontal	*/
#define acetecHCHIPNOM		(short)  1
/* no. of chips vertical	*/
#define acetecVCHIPNOM		(short)  1
/* no. of bits per pixel	*/
#define acetecBITPIXNOM		(short)  16
/* CCD type			*/
#define acetecCCDTYPENOM	(short)  0
/* 16 bit zero number		*/
#define acetecZERO		(short)  0 
/* optional link used		*/
#define acetecOPTYES		(short)  1
/* opt. link not used		*/
#define acetecOPTNO		(short)  0
/* high range for start value	*/
#define acetecSTRTHIGH		(short)  0x400
/* high range for offset value	*/
#define acetecOFFHIGH		(short)  0x100


/*
 * common constants for RTP and RCP
 */

/* Data struct wanted		*/
#define acetecSTRUCTYES	(short)  1
/* Data struct not wanted	*/
#define acetecSTRUCTNOT	(short)  0
/* command priority mask	*/
#define acetecPRIORITY	(short)  0x100


/*
 * masks for status word
 */

/* off state		*/
#define acetecOFF	        (short)  0x0000



/* LCU ONLY - BEGIN *****************************************************
 */


/*
 * System and general LCU header files
 */
#include "ccs.h"
#include "cmd.h"
#include "err.h"
#include "errCCS.h"

/*
 * Constants
 */
#define acetecAPP_NAME	  "acetec"
#define acetecMOD_NAME	  "acetec"		
#define acetecPROC_NAME	  "acetec"
#define acetecSERVER_NAME acetecMOD_NAME ## "Server"
#define acetecTASK_NAME   acetecMOD_NAME ## "Task"
#define acetecCDT_NAME 	  acetecSERVER_NAME ## ".cdt" 
#define acetecCIT_NAME    acetecSERVER_NAME ## ".cit" 

#define acetecVERSION     "10.20" 

#define acetecBYTES        200			/* message bytes	            	*/
#define acetecTIMEOUT      5000			/* timeout in milliseconds 	    	*/

#define acetecPROTOCOL          "acetec.prt"	/* file name for protocol 	   	*/
#define acetecSR_SMALL_DSP_FILE "acetecS.clk" 	/* DSP file for series small CCD 	*/
#define acetecSR_LARGE_DSP_FILE "acetecL.clk" 	/* DSP file for series large CCD 	*/
#define acetecPT_SMALL_DSP_FILE "acetecSP.clk"	/* DSP file for Prototype small CCD 	*/
#define acetecPT_LARGE_DSP_FILE "acetecLP.clk"	/* DSP file for Prototype large CCD 	*/
#define acetecBTLFILE           "acetec.btl"	/* file name for transputer code    	*/
#define acetecINITCCDFILE       "acetecInitCCD" /* file name for init CCD values    	*/
#define acetecREFFILE           "acetec.ref"	/* file with synthetic image	    	*/

#define acetecDATADIR           "ACE_INI"       /* environment for loc. of ini-files    */
#define acetecBINDIR            "ACE_BIN"       /* environment for loc. of ACE binaries */

#define acetecACE_SYSTEM   "ACE_SYSTEM"		/* environment variable ACE         	*/
#define acetecPROTOTYPE    "PROTOTYPE"		/* environment value Prototype ACE  	*/
#define acetecSERIES       "SERIES"		/* environment value Series ACE     	*/

#define acetecCCD_TYPE     "CCD_TYPE"		/* define CCD type for selection    	*/
#define acetecSMALL        "SMALL"		/* define small CCD type            	*/
#define acetecLARGE        "LARGE"		/* define large CCD type            	*/

/*
 * Task parameters of acetecServer:
 */
#define acetecSERVER_PRIORITY	80
#define acetecSERVER_OPTIONS	(VX_FP_TASK)
#define acetecSERVER_STACK	80000

/*
 * CCD definitions
 */
#define acetecERROR_CCD      (int) 0 		/* error in CCD type specification  */
#define acetecSR_SMALL_CCD   (int) 1 		/* mark series small CCD type	    */
#define acetecSR_LARGE_CCD   (int) 2 		/* mark series large CCD type	    */
#define acetecPT_SMALL_CCD   (int) 3 		/* mark prototype small CCD type    */
#define acetecPT_LARGE_CCD   (int) 4 		/* mark prototype large CCD type    */

#define acetecEI_DISABLE     (int) 0 		/* engineering interface disable    */
#define acetecEI_ENABLE      (int) 1 		/* engineering interface enable     */


/*
 * masks for status word
 */
/* status mask */
#define acetecSTATUS	 	0x000c
/* failure while self checks */
#define acetecSELFCHECK	 	0x8000

/*
 * masks for selfcheck word
 */
#define acetecFLAGSTATE  	0x0010		/* unexpected pattern in read register   */
#define acetecTLBIASRESET	0x0100		/* a bias could not be reset to 0 	 */
#define acetecTLBIASAFFECT	0x0200		/* a bias affects another bias while set */
#define acetecTLCLOCKRESET	0x0400		/* a clock could not be reset to 0 	 */
#define acetecTLCLOCKAFFECT	0x0800		/* a clock affects another bias while set*/
#define acetecTMPELTCURRENT	0x0020		/* error in switching the Peltier        */
#define acetecTMTEMPSWITCH	0x0040		/* error in switching the temp. sensors  */
#define acetecHISTFLAG		0x0040		/* history register has an error         */
#define acetecCURRFLAG		0x0020		/* current register has an error         */
#define acetecRCSWITCH		0x0001		/* gain switch caused an error           */

/*
 * function prototypes for the LCU-Transputer Interface.
 */

ccsCOMPL_STAT acetecFailureHandling
	(
	vltINT32,
	ccsERROR *
	);

ccsCOMPL_STAT acetecDeregisterFailureRoutine
	(
	vltINT32,
	ccsERROR *
	);
	
ccsCOMPL_STAT acetecDetachFailureRoutine
	(
	vltINT32,
	ccsERROR *
	);

ccsCOMPL_STAT acetecCloseHandling
	(
	vltINT32,
	ccsERROR *
	);

ccsCOMPL_STAT acetecWriteJudge ( vltINT16 );

ccsCOMPL_STAT acetecProtocolPrologue (const char *s1, const char *s2);
ccsCOMPL_STAT acetecProtocolEpilogue (const char *s1, const char *s2);
ccsCOMPL_STAT acetecPrintProtocol    (const char *  , const char *  );

ccsCOMPL_STAT acetecInit
	(
	vltINT32,
        ccsERROR *
	);

ccsCOMPL_STAT acetecWriteFits
	(
	int,
	int,
	int,
	int,
	int,
	int
	);

ccsCOMPL_STAT acetecFitsClose
	(
	int,
	int,
	unsigned short *
	);
	
ccsCOMPL_STAT acetecWrite
	(
	int,
	int,
	unsigned short *
	);
	
ccsCOMPL_STAT acetecClose
	(
	int,
	int,
	unsigned short *
	);
	
int acetecOpenFits ( const char * );

ccsCOMPL_STAT acetecSetPara
	(
	long,
	vltINT16,
	vltINT16,
	vltINT16,
	vltINT32,
	vltINT16 *,
	vltINT16 *,
	vltINT16 *,
	ccsERROR *
	);
	
ccsCOMPL_STAT acetecCheckPara
	(
	long,
	vltINT16,
	vltINT16,
	vltINT16 *,
	vltINT32,
	vltINT16 *,
	vltINT16 *,
	vltINT16 *,
	ccsERROR *
	);
	
ccsCOMPL_STAT acetecVersion ( void );
	
ccsCOMPL_STAT acetecTestAll   ( char deviceName[30] );
ccsCOMPL_STAT acetecTestSeq   ( char deviceName[30] );
ccsCOMPL_STAT acetecTestAux   ( char deviceName[30] );
ccsCOMPL_STAT acetecTestVideo ( char deviceName[30] );
ccsCOMPL_STAT acetecCheck     ( char deviceName[30] );

ccsCOMPL_STAT acetecCCDType ( void );
ccsCOMPL_STAT acetecBoot    ( char deviceName[30]);
ccsCOMPL_STAT acetecBootDSP ( char *, ccsERROR * );
ccsCOMPL_STAT acetecInitCI  ( void );
ccsCOMPL_STAT acetecExitCI  ( void );
ccsCOMPL_STAT acetecEI
	(
    	const char 	*parameters,		/* parameter pointer  	*/
    	msgMSG 		*msg,			/* message pointer  	*/
    	char 		**replyBuffer,		/* buffer from reply  	*/
    	msgLENGTH 	*replySize,		/* size from reply  	*/
    	ccsERROR 	*error			/* error variable  	*/
    	);
ccsCOMPL_STAT acetecGetVersion (char        *parameters,
				msgMSG      *msg,
				char        **replyBuffer,
				msgLENGTH   *replySize,
				ccsERROR    *error);
ccsCOMPL_STAT acetecSetHeadNumber (char        *parameters,
				   msgMSG      *msg,
				   char        **replyBuffer,
				   msgLENGTH   *replySize,
				   ccsERROR    *error);

#endif /* !ACETEC_H*/
