#ifndef ACRO_COMMANDS_H
#define ACRO_COMMANDS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: acroCommands.h,v 1.31 2005/10/11 09:06:49 vltsccm Exp $" 
*
* acroCommands.h  -  Contains literals of all defined commands and
*                   structure types for commands with multiple parameters.
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* ---v1.3---
* S.Sandrock 02.09.1994 updated comments, rearranged command macros, moved 
*			macros for device dependent min/max to acroDevice.h
*	     15.09.1994 added macros for min/max values.
* ---v1.4---
* S.Sandrock 09.11.1994 Moved min/max-macros back from mconDevice.h for LCC.
* --v1.4.1-- 09.11.1994 ---RELEASED---
* bgustafs   21.03.2002 Redefined command literal values to fit with canio module,
*                       these values must be kept in accordance with canio and aio
*                       modules
*/


/*
 * General commands
 */
#define acroCMD_RESET					100
#define acroCMD_SET_TIMEOUT_TICK		105
#define acroCMD_GET_TIMEOUT_TICK		110
#define acroCMD_FREE_DEVICE				115
#define acroCMD_BLOCK_SEMAPHORE			120


/*
 * Read Commands
 */
#define acroCMD_READ_CONFIG_BITS		125
#define acroCMD_READ_BITS				130
#define acroCMD_READ_BOARD_ID			135
#define acroCMD_CHECK_PULSE_BIT			140

/*
 * Write Commands
 */
#define acroCMD_WRITE_CONFIG_BITS		145
#define acroCMD_WRITE_BITS				150
#define acroCMD_WRITE_SET_BIT			155
#define acroCMD_WRITE_CLEAR_BIT			160
#define acroCMD_WRITE_CHANGE_BIT		165
#define acroCMD_WRITE_PULSE_BIT			170
#define acroCMD_WRITE_PULSE_BIT_RESET	175

/*
 * Interrupt commands
 */
#define acroCMD_CONFIG_INTERRUPT		180
#define acroCMD_ENABLE_INTERRUPT		185
#define acroCMD_DISABLE_INTERRUPT		190
#define acroCMD_CLEAR_INTERRUPT			195
#define acroCMD_READ_INTERRUPT_STATUS	200


/*
**----- Argument Data Structures -----
*/

typedef struct
	{
	int	firstSignal;
	int	numSignal;
	int	returnValue;
	} acroREAD;

typedef acroREAD *		acroREAD_PTR;

typedef struct
	{
	int	firstSignal;
	int	numSignal;
	int	value;
	} acroWRITE;

typedef acroWRITE *		acroWRITE_PTR;

typedef struct
	{
	int	firstSignal;
	int	numSignal;
	int	inputOutput;
	int	polarity;
	} acroCONFIG;

typedef acroCONFIG *		acroCONFIG_PTR;

typedef struct
	{
	int	numSignal;
	int	time;
	} acroPULSE;

typedef acroPULSE *		acroPULSE_PTR;

typedef struct
	{
	int	numSignal;
	int	polarity;
	} acroENABLE_INT;

typedef acroENABLE_INT *		acroENABLE_INT_PTR;

typedef struct
	{
	VOIDFUNCPTR	routine;
	int		parameter;
	} acroCONFIG_INT;

typedef acroCONFIG_INT *		acroCONFIG_INT_PTR;


/*
**----- Min/Max Values -----
** (Note: Use 2 spaces after `#define', so that lcudrvTk does not take these
**  macros as ioctl-commands; they also begin with "acroCMD_".)
*/
#define  acroCMD_FIRST_SIGNAL_MIN		 0 /* Min. for .firstSignal  */
#define  acroCMD_FIRST_SIGNAL_MAX_INTERRUPT	 7 /* dto. for interrupts */
#define  acroCMD_FIRST_SIGNAL_MAX		63 /* Max. for .firstSignal  */
#define  acroCMD_NUM_SIGNAL_MIN			 1 /* Min. for .numSignal    */
#define  acroCMD_NUM_SIGNAL_MAX			32 /* Max. for .numSignal    */

/*
** Public Constants: 
*/
/* number of signals: */
#define acroSIGNAL_MAX			64

/* values for the `inputOutput' member: */
#define	acroSIGNAL_INPUT		0
#define	acroSIGNAL_OUTPUT		1

/* values for the `polarity' member: */
#define	acroPOLARITY_ACTIVE_LOW		0
#define	acroPOLARITY_ACTIVE_HIGH	1


#endif /* !ACRO_COMMANDS_H */
