#ifndef AIO_COMMANDS_H
#define AIO_COMMANDS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: aioCommands.h,v 1.33 2005/10/11 09:48:17 vltsccm Exp $" 
*
* aioCommands.h  -  Contains literals of all defined commands and
*                   structure types for commands with multiple parameters.
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2005-09-21 aligned aioSIGNAL and aioBinSIGNAL structures with mendrv 
*                     and canio
* bgustafs 2002-04-24 moved gain definitions from aioDevice.h
* S.Sandrock 02.09.1994 updated comments
* bgustafs   20/05/99   added defines for differential and single-ended
*                       inputs
* bgustafs   25/05/99   corrected comments for lcudrvTk
* bgustafs   20/10/99   added max/min binary value for unipolar oputput
* bgustafs   21.03.2002 Redefined command literal values to fit with canio module,
*                       these values must be kept in accordance with canio and acro
*                       modules
*/

/*
 * Read Commands
 */
#define aioCMD_GET_BOARD_ID					300
#define aioCMD_READ_VALUE					305
#define aioCMD_GET_INPUT_CONFIG				310
#define aioCMD_GET_OUTPUT_CONFIG			315
#define aioCMD_GET_GAIN						320
#define aioCMD_GET_TIMEOUT					325
#define aioCMD_READ_BIN_VALUE				330
/*
 * Write Commands
 */
 /* aioCMD_RESET and aioCMD_FREE_DEVICE should have
  * the same value as acroCMD_RESET and acroCMD_FREE_DEVICE
  */
#define aioCMD_RESET						100
#define aioCMD_WRITE_VALUE					340
#define aioCMD_SET_INPUT_CONFIG				345
#define aioCMD_SET_OUTPUT_CONFIG			350
#define aioCMD_SET_GAIN						355
#define aioCMD_ENABLE_EOC_INTERRUPT			360
#define aioCMD_DISABLE_EOC_INTERRUPT		365
#define aioCMD_AUTOCALIBRATION				370
#define aioCMD_SET_TIMEOUT					375
#define aioCMD_FREE_DEVICE					115
#define aioCMD_WAIT							385
#define aioCMD_WRITE_BIN_VALUE				390


/*
** Constants
*/

/*
**set D8 in the control register (P2 differential, mode=1) or
** set D9 in the control register (P2 single ended, mode=2)
** This information is stored also in the .config member (logical or)
 */
#define aioCONFIG_P2_DIFFERENTIAL                              0x100
#define aioCONFIG_P2_SINGLE_ENDED                              0x200



/*
**----- Argument Data Structures -----
*/

typedef struct aioSIGNAL	/* Signal Config and Status Structure */
    {
    int number;
    int config;
    int gain;
    double value;
    } aioSIGNAL;

typedef struct aioBinSIGNAL	/* use with aioRead/WriteBinValue */
    {
    int number;
    int config;
    int gain;
    int value;
    } aioBinSIGNAL;

/* Predefined macros for the .config member: */
#define aioCONFIG_2x10V		1
#define aioCONFIG_10V		2
#define aioCONFIG_2x5V		3
#define aioCONFIG_5V		4

/* Limits: */
#define aioMAX_IN_SIGNALS      16	/* number of input signals */
#define aioMAX_OUT_SIGNALS      2	/* number of output signals */
#define aioMAX_BIN_VALUE     2047	/* max. binary readout */
#define aioMIN_BIN_VALUE    -2048	/* min. binary readout */
#define aioMAX_UNI_BIN_VALUE 4095       /* max binary readout for unipolar */
#define aioMIN_UNI_BIN_VALUE    0       /* max binary readout for unipolar */

/* Gain */
#define aioGAIN_X1		0
#define aioGAIN_X10		1
#define aioGAIN_X100		2
#define aioGAIN_X500		3

#endif /* !AIO_COMMANDS_H */

