/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: auto.h,v 1.33 2003/12/18 00:40:29 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 02/09/95  Created by eccsgen V0.45 (September 1995)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*
*   NAME
*	auto - automata (state machine) class
*
*   SYNOPSIS
*	#include "auto.h"
*
*	ccsCOMPL_STAT autoNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* autoGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* autoLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT autoInit(
*	    INOUT egenOBJ*	obj,
*	    IN    autoAUTO*	automata,
*	    IN    vltINT32	stateIndex,
*	    IN    vltINT32	substateIndex,
*	    IN    vltINT32	deviceIndex,
*	    OUT   ccsERROR*	error)
*
*	typedef struct {
*	    vltINT32     eventNum;
*	    vltINT32     stateNum;
*	    vltINT32     substateNum;
*
*	    char**       eventNames;
*	    char**       stateNames;
*	    char**       substateNames;
*
*	    vltINT32*    stateTable;
*	    vltINT32*    substateTable;
*	    autoFUNCPTR* actionTable;
*	    autoFUNCPTR* subactionTable;
*
*	    vltINT32     state;
*	    vltINT32     substate;
*	    vltINT32     busy;
*	    vltINT32     nextstate;
*
*	    egenOBJ*     attrOwner;
*	    vltINT32     stateIndex;
*	    vltINT32     substateIndex;
*	    vltINT32     deviceIndex;
*
*	    lccDEVICE_NAME deviceName;
*	    SEM_ID       semaphore;
*	} autoAUTO;
*
*	ccsCOMPL_STAT autoBegin(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	event,
*	    OUT   autoFUNCPTR*	fctPtr,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoEnd(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoLock(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoUnlock(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoGetState(
*	    INOUT egenOBJ*	obj,
*	    OUT   vltINT32*	state,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoSetState(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	state,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoGetSubState(
*	    INOUT egenOBJ*	obj,
*	    OUT   vltINT32*	substate,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoSetSubState(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	substate,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoGetTable(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	table,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoIsLccDevice(
*	    INOUT egenOBJ*	obj,
*	    OUT   vltLOGICAL*	flag,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoReset(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoExitObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT autoErr<error>(...)
*
*   PARENT CLASS
*	-
*
*   DESCRIPTION
*       The class autoCLASS implements functions for the
*       automata support.
*
*	In a simple automata, when an event is received, it is 
*	checked if the event is allowed in the current state and
*	the corresponding action is executed. Then the state is
*	changed according to the state table. 
*	This assumes, that the action takes just an instant and
*	that no events are received while the action is in
*	execution. This is not the case when hardware components
*	are moved. This takes some time and events (e.g. STOP)
*	may be received while the action is in progress. Therefore
*	this implementation of the automata differenciates between
*	the beginning of an event (when the corresponding action
*	starts) and the end of an event (when the action finishes).
*
*	To be able to recognize that an event is in progress, the
*	automata has a substate (in addition to a state). The 
*	substate is usually IDLE and is set to BUSY while an
*	action is executed (another substate may be specified in 
*	the automata tables).
*
*	The substate is set when an event starts. The state is
*	changed when an events ends, and only if the corresponding
*	action finishes without errors. If the action ends with
*	an error, then the state is not changed. The user has
*	the option to set the substate to ERROR to remind that
*	the automata may not be in a stable condition. Even so,
*	if an action recognizes an error, it is encouraged that
*	the action processes immediately the possible corrective
*	procedures.
*
*	By default, no events are accepted if the substate is not
*	IDLE. This assures e.g. that a second ONLINE action is not
*	executed while an ONLINE event is already being processed.
*	This behaviour can be overriden (e.g. to allow the reception
*	of a STOP event).
*
*	A normal automata is defined with the following components:
*
*	1) List of states.
*	2) List of events.
*	3) List of actions.
*	4) State table (states vs. events)
*	    Indicates if an event is allowed in a given state
*	    and the new state after the event has been processed.
*	5) Action table (states vs. events)
*	    Specifies the action to be executed when an event
*	    is received in a given state (it is legal to not
*	    execute any action).
*	6) Initial state.
*
*	In addition, this implementation needs the following tables:
*
*	7) Substate table (states vs. events)
*	    Specifies the substate of the automata while the
*	    event is being processed. By default the substate is
*	    changed to BUSY. If the substate should stay IDLE,
*	    then this must be specified in the table (as "-").
*	
*	8) Subaction table (substates vs. events)
*	    Specifies the events that are allowed in a given
*	    substate. The table contains the action to be
*	    executed (it is not allowed to accept an event
*	    and to execute any action, this does not make
*	    sense). 
*
*	Actions executed during a substate may not change the state
*	of the automata, but it may use the substate to indicate
*	that a change of action is requested. E.g. if a device is moving 
*	(state ONLINE, substate MOVING) and a STOP event is received,
*	then the STOP action should only notify that the device has to
*	stop (e.g. by setting the substate to STOP or by calling a 
*	function that notifies the action that performs the movement).
*	The action that performs the movement should then change the 
*	substate to STOP (if not already done), stop the movement and
*	change the state when it finishes (changing the state in this
*	case is not needed, the device should stay in state ONLINE).
*	Note that the movement action returns an error because it has
*	not reached the requested position, but the substate should not
*	be placed to ERROR, because the device is not in an unstable 
*	state.
*
*	A normal action may modify the state and substate during its
*	execution (to inform about its progress), but the final state
*	must be always the state specified in the state table.
* 
*   PUBLIC METHODS
*	autoNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type auto.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	autoDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type auto
*
*	autoInit(obj,automata,stateIndex,substateIndex,deviceIndex,error)
*	    autoInit is used to initialise an automata object immediately
*	    after creation.
*
*	    automata      <IN>	Automata data: see below.
*	    stateIndex    <IN>	Index to the state attribute of 
*				the object that owns the automata.
*	    substateIndex <IN>	Index to the state attribute of 
*				the object that owns the automata.
* 
*	    The elements of the automata structure are:
*
*	    eventNum		Number of events.
*	    stateNum		Number of states.
*	    substateNum		Number of substates.
*
*	    eventNames		Names of the events.
*	    stateNames		Names of the states.
*	    substateNames	Names of the substates.
*
*	    stateTable		State table.
*	    substateTable	Substate table.
*	    actionTable		Action table.
*				  all 3: index = event * stateNum + state
*	    subactionTable	Subaction table.
*				  index = event * substateNum + substate
*
*	    state		Initial state.
*	    substate		Substate (ignored, set to 0).
*	    busy		Internal data (set to 0).
*	    nextstate		Internal data (set to 0).
*
*	    attrOwner		Internal data (set to NULL).
*	    stateIndex		Set by parameter stateIndex (ignore).
*	    substateIndex	Set by parameter substateIndex (ignore).
*	    deviceIndex		Set by parameter deviceIndex (ignore).
*
*	    deviceName		Name of the related LCC software device.
*	    semaphore		Internal data (ignore).
*
*
*	    Note: The length of the name tables and other tables must
*	    match the number of events, states and substates given.
*
*	autoBegin(obj,event,fctPtr,error)
*	    The function autoBegin begins the execution of an event.
*	    It must be called at the beginning of the event. It 
*	    receives the following parameters:
*
*	    event	<IN>	Event to be processed.
*	    fctPtr	<OUT>	Pointer to the action that should
*				be executed.
*
*	    autoBegin checks if the event is allowed in the current
*	    state and substate, sets the substate according to the
*	    substate table and returns a pointer to the action
*	    (function) that should be executed. The pointer may be
*	    NULL if the event is accepted, but no corresponding
*	    action is registered in the tables (in this case, only
*	    the state is changed when autoEnd is called).
*
*	    If the event is not allowed in the current state and
*	    substate, then an error is returned.
*
*	    If autoBegin is called from another action (i.e. if
*	    autoBegin is called again before the corresponding call
*	    to autoEnd), then this call to autoBegin is ignored (an
*	    internal counter is incremented to also ignore the 
*	    corresponding autoEnd call).
*
*	    Note: A corresponding call to autoEnd must _always_ be made.
*
*	    Note: If is not allowed to call another action from
*	    inside an action. This would execute again autoBegin
*	    before the corresponding autoEnd. This is not allowed.
*	    If this is done, then the autoBegin and autoEnd calls
*	    of the called action are ignored.
*
*	autoEnd(obj,error)
*	    The function autoEnd must be called when the execution of
*	    an event finishes. If the substate is not ERROR, then the
*	    substate is set to IDLE.  Then it uses the error structure 
*	    to determine if an error occurred during the execution of
*	    the action. If no error is detected, then the state is 
*	    changed according to the state, substate and event received
*	    when the corresponding autoBegin call was made. 
*
*	    Note: autoEnd may not be executed if autoBegin was not called
*	    to start the event.
*
*	    Note: This call is ignored if autoBegin was previously called
*	    before the corresponding autoEnd() call.
*
*	autoLock(obj,error)
*	    Locks the automata semaphore (state changes are disabled while
*	    the semaphore is locked).
*
*	autoUnlock(obj,error)
*	    Unlocks the automata semaphore.
*
*	autoGetState(obj,state,error)
*	    autoGetState returns the state of the automata obj.
*
*	    state	<OUT>	Pointer to the state.
* 
*	autoSetState(obj,state,error)
*	    autoSetState sets the state of the automata obj.
*	    If the automata is related with an LCC software device,
*	    then the state of the software device is set calling the
*	    function lccSetDeviceState as follows:
*
*	    state = 0: lccDEV_OFF
*	    state = 1: lccDEV_LOADED
*	    state = 2: lccDEV_STAND_BY
*	    state = 3: lccDEV_ON_LINE
*
*           state	<IN>	New state.
* 
*	autoGetSubState(obj,substate,error)
*	    autoGetSubState returns the substate of the automata.
*
*	    substate	<OUT>	Pointer to the substate.
* 
*	autoSetSubState(obj,substate,error)
*	    autoSetSubState() sets the substate of the automata.
*
*	    substate	<IN>    New state.
* 
*	autoGetTable(obj,table,text,error)
*	    autoGetTable() returns the internal table of an automata 
*	    indicated with the table parameter.
*
*	    table	<IN>	Name of the table to be returned:
*				States, Actions, Substates, Subactions
*	    text	<OUT>	Returned table (the buffer must
*				be allocated by the caller).
* 
*	    Note: autoGetTable() assumes that text is 8000 char. long.
*
*	autoIsLccDevice(obj,flag,error)
*	    The returned flag is TRUE, if the associated device is
*	    registered in LCC (in the devices file).
*
*	autoReset(obj,error)
*	    Resets the substate of the automata. This function should 
*	    normally _not_ be used. It is supplied only as a last resort
*	    to be able to issue an OFF command or similar before shutting
*	    down the LCU.
*
*	autoErr<error>(...)
*	    The macros autoErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*   PRIVATE METHODS
*	autoExitObj(obj,error)
*	    autoExitObj() is called when an automata object is deleted.
*	    It deletes the critical region used to update the automata
*	    data.
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	The names of the defined errors begin as follows:
*	autoERR_<error>
*
*   SEE ALSO
*	autoMisc(3)
*
*-------------------------------------------------------------------------
*/

#ifndef AUTO_H
#define AUTO_H

#include "egen.h"
#include "autoErrors.h"

/*
 * class type identifier
 */
#define autoID	113

/*
 * function table
 */
#define autoFCT_DELETE_OBJ	0
#define autoFCT_INIT	1
#define autoFCT_BEGIN	2
#define autoFCT_END	3
#define autoFCT_LOCK	4
#define autoFCT_UNLOCK	5
#define autoFCT_GET_STATE	6
#define autoFCT_SET_STATE	7
#define autoFCT_GET_SUB_STATE	8
#define autoFCT_SET_SUB_STATE	9
#define autoFCT_GET_TABLE	10
#define autoFCT_IS_LCC_DEVICE	11
#define autoFCT_RESET	12
#define autoFCT_EXIT_OBJ	13


/*
 * states, substates and actions
 */
#define autoSTATE_NOSTATE	-1
#define autoSUBSTATE_ERROR	0
#define autoSUBSTATE_IDLE	1
#define autoSUBSTATE_BUSY	2

#define autoSEM_TIMEOUT	     5000


/*
 * types
 */
typedef ccsCOMPL_STAT (*autoFUNCPTR)();

typedef struct autoAUTO_S {
    vltINT32	eventNum;
    vltINT32	stateNum;
    vltINT32	substateNum;

    char**	 eventNames;
    char**	 stateNames;
    char**	 substateNames;

    vltINT32*	 stateTable;
    vltINT32*	 substateTable;
    autoFUNCPTR* actionTable;
    autoFUNCPTR* subactionTable;

    vltINT32	state;
    vltINT32	substate;
    vltINT32	busy;
    vltINT32	nextstate;

    egenOBJ*	attrOwner;
    vltINT32	stateIndex;
    vltINT32	substateIndex;
    vltINT32	deviceIndex;
    lccDEVICE_NAME deviceName;
    vltLOGICAL	deviceIsLcc;

    SEM_ID	semaphore;
} autoAUTO;


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*autoFCTPTR_INIT)(
    INOUT egenOBJ*	obj,
    IN    autoAUTO*	automata,
    IN    vltINT32	stateIndex,
    IN    vltINT32	substateIndex,
    IN    vltINT32	deviceIndex,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_BEGIN)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	event,
    OUT   autoFUNCPTR*	fctPtr,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_END)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_LOCK)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_UNLOCK)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_GET_STATE)(
    INOUT egenOBJ*	obj,
    OUT   vltINT32*	state,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_SET_STATE)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	state,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_GET_SUB_STATE)(
    INOUT egenOBJ*	obj,
    OUT   vltINT32*	substate,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_SET_SUB_STATE)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	substate,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_GET_TABLE)(
    INOUT egenOBJ*	obj,
    IN    const char*	table,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_IS_LCC_DEVICE)(
    INOUT egenOBJ*	obj,
    OUT   vltLOGICAL*	flag,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_RESET)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*autoFCTPTR_EXIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define autoLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("auto", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define AUTO_ERROR_RETURN(errorNumber, locId, format, arg) \
	{ \
	    ERROR_ADD(errorNumber, locId, format, arg) \
	    return (autoEnd(&autom, error)); \
	}
#define AUTO_RETURN(code) \
	{ code; return(autoEnd(&autom, error)); }

#define autoErrNoAutomata() \
	{ \
	lcctooLOG_0("autoERR_NO_AUTOMATA: Automata not registered"); \
	errAdd(error,"auto",autoERR_NO_AUTOMATA,__FILE_LINE__,""); \
	}
#define autoErrEvent(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_EVENT: Device %s: Event number %d out of range",(p1),(p2)); \
	errAdd(error,"auto",autoERR_EVENT,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define autoErrState(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_STATE: Device %s: Internal error: state number %d out of range",(p1),(p2)); \
	errAdd(error,"auto",autoERR_STATE,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define autoErrWriteState(p1) \
	{ \
	lcctooLOG_0("autoERR_WRITE_STATE: Device %s: Unable to write automata state into OLDB",(p1)); \
	errAdd(error,"auto",autoERR_WRITE_STATE,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrWriteSubState(p1) \
	{ \
	lcctooLOG_0("autoERR_WRITE_SUB_STATE: Device %s: Unable to write automata substate into OLDB",(p1)); \
	errAdd(error,"auto",autoERR_WRITE_SUB_STATE,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrReadState(p1) \
	{ \
	lcctooLOG_0("autoERR_READ_STATE: Device %s: Unable to read automata state from OLDB",(p1)); \
	errAdd(error,"auto",autoERR_READ_STATE,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrReadSubState(p1) \
	{ \
	lcctooLOG_0("autoERR_READ_SUB_STATE: Device %s: Unable to read automata substate from OLDB",(p1)); \
	errAdd(error,"auto",autoERR_READ_SUB_STATE,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrReadDevice() \
	{ \
	lcctooLOG_0("autoERR_READ_DEVICE: Unable to read device name from OLDB"); \
	errAdd(error,"auto",autoERR_READ_DEVICE,__FILE_LINE__,""); \
	}
#define autoErrNotBusy(p1,p2,p3) \
	{ \
	lcctooLOG_0("autoERR_NOT_BUSY: Device %s: Internal error: autoEnd called when no action is active (state: %s, substate: %s)",(p1),(p2),(p3)); \
	errAdd(error,"auto",autoERR_NOT_BUSY,__FILE_LINE__,"%s,%s,%s",(p1),(p2),(p3)); \
	}
#define autoErrEventNotAllowed(p1,p2,p3,p4) \
	{ \
	lcctooLOG_0("autoERR_EVENT_NOT_ALLOWED: Device %s: Command %s not allowed in state %s (substate %s)",(p1),(p2),(p3),(p4)); \
	errAdd(error,"auto",autoERR_EVENT_NOT_ALLOWED,__FILE_LINE__,"%s,%s,%s,%s",(p1),(p2),(p3),(p4)); \
	}
#define autoErrNoLccState(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_NO_LCC_STATE: Device %s: Internal error: state %s not supported by LCC Software Devices",(p1),(p2)); \
	errAdd(error,"auto",autoERR_NO_LCC_STATE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define autoErrSetDeviceState(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_SET_DEVICE_STATE: Device %s: Unable to change to state %s",(p1),(p2)); \
	errAdd(error,"auto",autoERR_SET_DEVICE_STATE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define autoErrInit(p1) \
	{ \
	lcctooLOG_0("autoERR_INIT: Device %s: Initialisation of automata failed",(p1)); \
	errAdd(error,"auto",autoERR_INIT,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrBegin(p1) \
	{ \
	if (lcctooDebugLevel > 1) \
	    { \
	    lcctooLOG_0("autoERR_BEGIN: Event %s not accepted",(p1)); \
	    errAdd(error,"auto",autoERR_BEGIN,__FILE_LINE__,"%s",(p1)); \
	    } \
	}
#define autoErrOff(p1) \
	{ \
	lcctooLOG_0("autoERR_OFF: Device %s: Unable to change to state OFF",(p1)); \
	errAdd(error,"auto",autoERR_OFF,__FILE_LINE__,"%s",(p1)); \
	}
#define autoErrLock(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_LOCK: Device %s: Unable to lock access to device (%s)",(p1),(p2)); \
	errAdd(error,"auto",autoERR_LOCK,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define autoErrUnlock(p1,p2) \
	{ \
	lcctooLOG_0("autoERR_UNLOCK: Device %s: Unable to unlock access to device (%s)",(p1),(p2)); \
	errAdd(error,"auto",autoERR_UNLOCK,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define autoErrGetTable(p1) \
	{ \
	lcctooLOG_0("autoERR_GET_TABLE: Argument %s not recognized",(p1)); \
	errAdd(error,"auto",autoERR_GET_TABLE,__FILE_LINE__,"%s",(p1)); \
	}

/*
 * get object owner
 */
#define autoGetOwner(obj) ((obj)->owner)


/*
 * class functions
 */

ccsCOMPL_STAT autoInit(
    INOUT egenOBJ*	obj,
    IN    autoAUTO*	automata,
    IN    vltINT32	stateIndex,
    IN    vltINT32	substateIndex,
    IN    vltINT32	deviceIndex,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoBegin(
    INOUT egenOBJ*	obj,
    IN    vltINT32	event,
    OUT   autoFUNCPTR*	fctPtr,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoBegin1(
    INOUT egenOBJ*	obj,
    IN    vltINT32	event,
    OUT   autoFUNCPTR*	fctPtr,
    OUT   vltLOGICAL*   retry,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoEnd(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoLock(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoUnlock(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoGetState(
    INOUT egenOBJ*	obj,
    OUT   vltINT32*	state,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoSetState(
    INOUT egenOBJ*	obj,
    IN    vltINT32	state,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoGetSubState(
    INOUT egenOBJ*	obj,
    OUT   vltINT32*	substate,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoSetSubState(
    INOUT egenOBJ*	obj,
    IN    vltINT32	substate,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoGetTable(
    INOUT egenOBJ*	obj,
    IN    const char*	table,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoIsLccDevice(
    INOUT egenOBJ*	obj,
    OUT   vltLOGICAL*	flag,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoReset(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoExitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define autoVInit(obj,automata,stateIndex,substateIndex,deviceIndex,error) \
	(*((autoFCTPTR_INIT)((obj)->fcts[autoFCT_INIT]))) \
	 ((obj),(automata),(stateIndex),(substateIndex),(deviceIndex),(error))
#define autoVBegin(obj,event,fctPtr,error) \
	(*((autoFCTPTR_BEGIN)((obj)->fcts[autoFCT_BEGIN]))) \
	 ((obj),(event),(fctPtr),(error))
#define autoVEnd(obj,error) \
	(*((autoFCTPTR_END)((obj)->fcts[autoFCT_END]))) \
	 ((obj),(error))
#define autoVLock(obj,error) \
	(*((autoFCTPTR_LOCK)((obj)->fcts[autoFCT_LOCK]))) \
	 ((obj),(error))
#define autoVUnlock(obj,error) \
	(*((autoFCTPTR_UNLOCK)((obj)->fcts[autoFCT_UNLOCK]))) \
	 ((obj),(error))
#define autoVGetState(obj,state,error) \
	(*((autoFCTPTR_GET_STATE)((obj)->fcts[autoFCT_GET_STATE]))) \
	 ((obj),(state),(error))
#define autoVSetState(obj,state,error) \
	(*((autoFCTPTR_SET_STATE)((obj)->fcts[autoFCT_SET_STATE]))) \
	 ((obj),(state),(error))
#define autoVGetSubState(obj,substate,error) \
	(*((autoFCTPTR_GET_SUB_STATE)((obj)->fcts[autoFCT_GET_SUB_STATE]))) \
	 ((obj),(substate),(error))
#define autoVSetSubState(obj,substate,error) \
	(*((autoFCTPTR_SET_SUB_STATE)((obj)->fcts[autoFCT_SET_SUB_STATE]))) \
	 ((obj),(substate),(error))
#define autoVGetTable(obj,table,text,error) \
	(*((autoFCTPTR_GET_TABLE)((obj)->fcts[autoFCT_GET_TABLE]))) \
	 ((obj),(table),(text),(error))
#define autoVIsLccDevice(obj,flag,error) \
	(*((autoFCTPTR_IS_LCC_DEVICE)((obj)->fcts[autoFCT_IS_LCC_DEVICE]))) \
	 ((obj),(flag),(error))
#define autoVReset(obj,error) \
	(*((autoFCTPTR_RESET)((obj)->fcts[autoFCT_RESET]))) \
	 ((obj),(error))
#define autoVExitObj(obj,error) \
	(*((autoFCTPTR_EXIT_OBJ)((obj)->fcts[autoFCT_EXIT_OBJ]))) \
	 ((obj),(error))


ccsCOMPL_STAT autoNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT autoDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !AUTO_H

/************************************************************************/
/* ___oOo___ */
