#ifndef B016_H
#define B016_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: b016.h,v 3.3 2002/04/30 06:14:58 vltsccm Exp $"
*
* b016.h - Public header of B016 driver
*
* who        when      what
* --------   --------  --------------------------------------------
* J.Ayasse   xx/xx/93  created
* C.Cumani   09/02/95  adapted to ESO VLT driver standards
* C.Cumani   07/03/95  BMUX maps and Rom sw version added
* C.Cumani   26/09/95  - made compatible with CMM
*                      - b016VERSION, b016VERSION_DATE, b016MOD_NAME,
*                        b016MOD_DESC definitions added
* C.Cumani   03/03/96  - b016VERSION_DATE set to "FEB96"
* C.Cumani   11/06/96  - b016VERSION_DATE set to "JUL96"
* P.Duhoux   16/12/96  - b016VERSION_DATE set to "DEC96"
* C.Cumani   17/12/96  - b016VERSION_DATE set to "APR97"
* C.Cumani   26/08/97  - b016VERSION_DATE set to "NOV97"
*
*/


#include "ccs.h"
#include "b016Commands.h"


/*
 * Byte Multiplexor program set state
 * normal read/write masks:        0x10325476
 * no reordering read/write masks: 0x76543210
 */
typedef struct 
{
    long int bmuxReadOrder;       /* nibble order for reads from bus */
    long int bmuxWriteOrder;      /* nibble order for writes to bus */
} b016BMUX_SET;
#define b016BMUXSET b016BMUX_SET

/*
 * B016 driver provided functions
 */
int b016Drv (int boards, int channels);
int b016DevCreate (const char *name, int boardNumber, int linkNumber, 
		   int regAddrSpace, void *regAddr, void *memAddr, 
                   int intNum, int intLvl);

/*
 * first ESO-VLT B016 T801 Rom software version
 * this is fixed and cannot be changed !!!
 */
#define b016EPROM_FIRST_VERSION  0x1002

/*
 * actual software version
 */
#define b016VERSION              "$Revision: 3.3 $"
#define b016VERSION_DATE         "MAY2002"
#define b016MOD_NAME             "b016"
#define b016MOD_DESC             "VxWorks B016 board driver module"

/*
 * Byte Multiplexor read/write masks
 */
/* Normal BMux byte-swapping masks */
#define b016_DOWNLOAD_READ_MAP    0x10325476
#define b016_DOWNLOAD_WRITE_MAP   0x10325476

/* long BMux byte-swapping masks */
#define b016_4_READ_MAP           b016_DOWNLOAD_READ_MAP
#define b016_4_WRITE_MAP          b016_DOWNLOAD_WRITE_MAP

/* short BMux byte-swapping masks */
#define b016_2_READ_MAP           0x32107654
#define b016_2_WRITE_MAP          0x32107654

/* no BMux byte-swapping masks */
#define b016_0_READ_MAP           0x76543210
#define b016_0_WRITE_MAP          0x76543210


/*
 * B016 tools functions
 */
ccsCOMPL_STAT b016Version
    (
    vltBYTES80 version
    );

ccsCOMPL_STAT b016RomVersion
    (
    const char *deviceName, 
    vltINT32   *pVersion,
    vltLOGICAL  verbose
    );
		   

#endif /* !B016_H */
