/*****************************************************************************/
/*            CAN Application Layer for Industrial Applications              */
/*                                                                           */
/*                           CIA-Draft from Nov. 92                          */
/*                          (c) esd Hannover 1994-97                         */
/*****************************************************************************/
/*                                                                           */
/*        module: can.h                                                      */
/*                                                                           */
/*****************************************************************************/
/*                  HISTORY                                                  */
/*                  -------                                                  */
/*  1.00    27.09.95 *start of history                                       */
/*  1.01    28.03.96 *Union defined for typ in CanCalOpen()                  */
/*  1.02    12.04.96 *New entry canCalInstallUserHandler()                   */
/*  1.03    15.04.96 *New entries for realization of multi read :            */
/*                    canCalMultiOpen(), canCalMultiAdd(),canCalMultiClose() */
/*                    canCalMultiRead()                                      */
/*  1.04    29.04.96 *Changes of prototype from void* to const void* in      */
/*                   *canCalSend(), canCalWrite(), canCalClientDownload(),   */
/*                   *canCalServerDownload, canCalUpdate(), canCalCopy() and */
/*                   *canCalWriteRead()                                      */
/*  1.05    15.05.96 *Changes of prototype in canCalClientDownlod(),         */
/*                   *canCalClientUpload(),canCalServerDownload(),           */
/*                   *canCalServerUpload() and canCalClose()                 */
/*  1.10    10.12.96 *New entry canCalMultiDelete()                          */
/*		     *New entry for canCalSendRTR()                          */
/*		     *New entry canCalGetVersion()			     */
/*		     *Definitions for flags of canCalOpen()		     */
/*		     *Definitions of error return codes			     */
/*		     *Including macro EXPORT for WIN32                       */
/*  1.11    24.01.97 *Support for CALLTYPE after function return value       */
/*  1.12    26.02.97 *Avoid warnings for WIN32 double defined EXPORT macros  */
/*  1.13    05.03.97 *New Errorcode: CAN_NO_ID_ENABLED                       */
/*  1.14    15.04.97 *Flags for canCalMultiOpen()                            */
/*  1.15    02.06.97 *canCalInstallUserHandler() changed in declaration      */
/*  1.20    29.07.97 *New wntry canCalServerWaitAndUpload()                  */
/*  1.30    29.07.97 *New error codes                                        */
/*  1.40    19.09.97 *New entry canCalMultiWrite() and definition for        */
/*                   *TX_USER_BUFFER                                         */
/*  1.41    26.03.97 *New error code CAN_NO_RTR_RECV for canCalWaitRTR()     */
/*****************************************************************************/
#ifndef _can_h_
#define _can_h_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
# ifndef EXPORT
#  define EXPORT __declspec (dllexport)  /* Macro for DLL functions */
# endif
#else 
# ifndef EXPORT
#  undef EXPORT
#  define EXPORT
# else
#  define EXPORT
# endif
#endif

#if !defined(CALLTYPE)
# define CALLTYPE
#endif

/*------------------ Defines ------------------------------------------------*/

#if !defined(NULLHANDLE) || (NULLHANDLE != 0)
    #define NULLHANDLE	0
#endif

#define IDF_CONVERT (const char *)0xFFFF

	/* Flags in canCalOpen() */

#define	TXRX_DOMAIN		0x0001	/* Use handle for CMS domain transfer */
#define TX_RECV_RTR		0x0004	/* Tx can receive RTR frames */
#define TX_AUTOANSWER		0x0006	/* Tx answers RTR frames automatically */
#define RX_SEND_RTR		0x0008	/* Send RTR frame on this Rx */
#if 0
#define DOMAIN_IMPLICIT_WAIT	0x0010  /* Do implicit server wait */ 
#endif

	/* Flags in canCalMultiOpen() */

#undef  RX_BUFFER
#define RX_BUFFER	0x0000  /* Open a Rx buffer */
#define TX_BUFFER	0x0001  /* Open a Tx buffer (C167 / CAN2 /CAN 4) */
#define TX_USER_BUFFER  0x8000  /* User provides Tx buffer */

/*------------------- error -------------------------------------------------*/
#define CAN_NO_MEMORY          -6 
#define CAN_NO_DEVICE          -5
#define CAN_PARA_ERROR         -2
#define CAN_SYS_ERROR          -1
#define CAN_OK                  0 
#define CAN_RX_TIMEOUT         1 
#define CAN_TX_TIMEOUT         2 
#define CAN_TX_ERROR           4
#define CAN_CONTR_OFF_BUS      5
#define CAN_CONTR_BUSY         6
#define CAN_CONTR_WARN         7
#define CAN_OLD_DATA           8
#define CAN_NO_ID_ENABLED      9
#define CAN_ID_ALREADY_ENABLED 10
#define CAN_ID_NOT_ENABLED     11
#define CAN_INVALID_HANDLE     12
#define CAN_INVALID_FIRMWARE   13
#define CAN_MESSAGE_LOST       14
#define CAN_INVALID_HARDWARE   15
#define CAN_NO_RTR_RECV        16

/*------------------ Typedefs -----------------------------------------------*/

typedef long HCAN;			    /* Cal handle       */
typedef long SCAN;			    /* Multiread handle */

typedef struct {
  int   idf;                   /* CAN idf         */  
  int	len;		       /* Length of data  */
  char	data[8];	       /* Message Data    */
} USER_BUFFER_TX;

typedef struct  {
  unsigned short  magic;       /* Magic number              */
  short           net;		/* net-number               */
  unsigned short  typ;		/* 0x0000 =>normal          */
                                /*  Bit 0=1 =>domain        */
                                /*  Bit 1=1 =>answer, if RTR*/    
  int             txid;		/* tx-can-id,if any         */
  int             rxid;		/* rx-can-id,if any         */
  SEM_ID          wait;		/* wait-sema                */
} _HCAN;


/*------------------ Globals ------------------------------------------------*/

extern const unsigned short canVersion;		


#define CAN_MODE_AUTO_ANSWER 0x02



/*------------------ Prototypes ---------------------------------------------*/

EXPORT    int CALLTYPE  canCalSetBaudrate(
                 int net,		    /* net-number                    */
                 int baud );		    /* baud rate or table entry      */


/*...........................................................................*/
EXPORT    HCAN CALLTYPE canCalOpen(                 /* =NULL, if error       */
                int             net,        /* net-number                    */
                int             txid,       /* if pos., tx-id                */
                int             rxid,       /* if pos., rx-id                */
                unsigned short  typ,        /* typ bits                      */
                int             txtout,     /* tx-timeout / milliseconds     */
                int             rxtout );   /* rx-timeout / milliseconds     */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalClose(	       /* ret 0, if success          */
                HCAN            hcan );          /* com-handle               */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalSend(               /* ret 0, if success        */
                HCAN       hcan,         /* com-handle                       */
                const void *buf,         /* ptr to data-buffer               */
                int        len );        /* count of bytes                   */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalWrite(             /* ret 0, if success         */
                HCAN       hcan,         /* com-handle                       */
                const void *buf,         /* ptr to data-buffer               */
                int        len );        /* count of bytes                   */


/*...........................................................................*/
EXPORT    int  CALLTYPE canCalUpdate(         /* ret 0, if success           */
                HCAN       hcan,         /* com-handle                       */
                const void *buf,         /* ptr to data-buffer               */
                int        len );        /* count of bytes                   */

/*...........................................................................*/
EXPORT    int CALLTYPE canCalSendRTR(
                HCAN    hcan );          /* com-handle                       */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalWaitRTR(        /* ret 0, if success            */
                HCAN    hcan );          /* com-handle                       */

/*...........................................................................*/
EXPORT    int CALLTYPE canCalTake(           /* ret 0, if success            */
                HCAN    hcan,            /* com-handle                       */
                void    *buf,            /* ptr to data-buffer               */
                int     *len );          /* res-ptr:count of read bytes      */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalRead(           /* ret 0, if success            */
                HCAN    hcan,            /* com-handle                       */
                void    *buf,            /* ptr to data-buffer               */
                int     *len );          /* res-ptr:count of read bytes      */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalReadRem(       /* ret 0, if success             */
                HCAN    hcan,            /* com-handle                       */
                void    *buf,            /* ptr to data-buffer               */
                int     *len );          /* res-ptr:count of read bytes      */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalWriteRead(     /* ret 0, if success             */
                HCAN       hcan,         /* com-handle                       */
                const void *tbuf,        /* ptr to transmit-buffer           */
                int        tlen,         /* count of bytes                   */
                void       *rbuf,        /* ptr to receive-buffer            */
                int        *rlen );      /* res-ptr:count of read bytes      */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalInstallUserHandler( /* ret 0, if success */
	        int    ( *MyHandler)(int), /* ptr to handler                */
		int    messageObject	   /* Message obj to link with */
		);

/*...........................................................................*/
EXPORT    int CALLTYPE canCalGetError(void);    /* Wait for next error and return   */
					 /* the errorno                     */
  
/*...........................................................................*/
EXPORT    SCAN CALLTYPE canCalMultiOpen(     /* =NULL, if error              */
                int     net,             /* net number                       */
		int     flags,           /* Flags, not used                  */
                int     queuesize,       /* no of entries in message queue   */
                int     timeout );       /* timeout in miliseconds           */

/*...........................................................................*/
EXPORT    int CALLTYPE canCalMultiAdd(      /* ret 0, if success             */
                SCAN    scan,            /* multi read handle                */
                int     idf );           /* 11 bit can identifier            */
/*...........................................................................*/
EXPORT    int CALLTYPE canCalMultiDelete(   /* ret 0, if success             */
                SCAN    scan,            /* multi read handle                */
                int     idf );           /* 11 bit can identifier            */
/*...........................................................................*/
EXPORT    int CALLTYPE canCalMultiClose(    /* ret 0, if success             */
                SCAN    scan );          /* multi read handle                */

/*...........................................................................*/
EXPORT    int CALLTYPE canCalMultiRead(     /* ret 0, if success             */
                SCAN    scan,            /* multi read handle                */
                int     *idf,            /* ptr to can idf buffer            */
                int     *len,            /* ptr to lenght of data buffer     */
                char    *data );         /* ptr to data buffer (8 bytes)     */

EXPORT    int CALLTYPE canCalMultiWrite(   /* ret 0, if success              */
                SCAN          scan,      /* buffer handle                    */
                int           idf,       /* can idf                          */
                unsigned long len,       /* lenght of data buffer            */
                const char    *data);    /* ptr to data buffer               */

/*...........................................................................*/
/*...complex cms-domain-services.............................................*/
/*...........................................................................*/
EXPORT    int CALLTYPE canCalClientUpload(  /* ret 0, if success             */
                HCAN    hcan,            /* com-handle                       */
                long    mux,             /* multiplexer                      */
                void    *buf,            /* ptr to receive-buffer            */
                long    *len);           /* resptr: count of bytes           */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalServerUpload(  /* ret 0, if success             */
                HCAN    hcan,            /* com-handle                       */
                void    *buf,            /* ptr to receive-buffer            */
                long    *len);           /* resptr: count of bytes           */
    

/*...........................................................................*/
EXPORT    int CALLTYPE canCalClientDownload( /* ret 0, if success            */
                HCAN        hcan,        /* com-handle                       */
                long        mux,         /* multiplexer                      */
                const void *buf,         /* ptr to transmit-buffer           */
                long        len);        /* count of bytes                   */


/*...........................................................................*/
EXPORT    int CALLTYPE canCalServerDownload( /* ret 0, if success            */
                HCAN       hcan,         /* com-handle                       */
                const void *buf,         /* ptr to transmit-buffer           */
                long       len);         /* count of bytes                   */


EXPORT    int CALLTYPE canCalServerWait(    /* ret 0, if success             */
                HCAN    hcan,            /* com-handle                       */
                long    *mux,            /* ret: multiplexer                 */
                int     *up );           /* ret: request-typ: 0=>down else up*/

EXPORT    int CALLTYPE canCalServerWaitAndUpload(  /* ret 0, if success      */
                HCAN    hcan,            /* com-handle                       */
                long    *mux,            /* ret: multiplexer                 */
                void    *buf,            /* ptr to receive-buffer            */
                long    *len);           /* resptr: count of bytes           */

EXPORT    void CALLTYPE canCalCopy(
		const char  *src,
		char        *dest,
		const char  *datatyp );

EXPORT    int CALLTYPE canCalGetVersion(
		unsigned short *version, /* Version of driver */
		char *	      name    	/* Name of the driver*/
		);

#ifdef __cplusplus
}
#endif

#endif /* _can_h_ */


