#ifndef CANOPEN_H
#define CANOPEN_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: canOpen.h,v 1.9 2005/05/09 15:51:57 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2004-11-02 added state strings
* mba(4DE) 2004-04-02 semaphore semGuarding and semEmergency added
* bgustafs 2002-11-14 added flag inUse to canOpenDEVICE
* bgustafs 2002-07-04 modified for standalone library
* vltuser5  17/01/02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/******************** Define useful constants *********************************/

#define canOpenSTATE_PREOPERATIONAL    1  /* SDO possible, PDO not allowed */
#define canOpenSTART                   2  /* transition to OPERATIONAL state */
#define canOpenSTOP                    3  /* transition to STOPPED state */
#define canOpenRESET_NODE              4  /* reboot node */
#define canOpenRESET_COMMUNICATION     5
#define canOpenSTATE_OPERATIONAL       6  /* SDO and PDO messages allowed */
#define canOpenSTATE_STOPPED           7  /* error state */

#define canOpenGUARD_TIME              10
#define canOpenLIFE_TIME               5

#define canOpenCOB_NMT               0x00  /* COB-ID for NMT message */
#define canOpenCOB_SYNC              0x080 /* Sync-Identifier */
#define canOpenCOB_EMERGENCY_BASE    0x080 /* Basic COB-ID for emergency messages */
#define canOpenCOB_GUARDING_BASE     0x700 /* Basic COB-ID for guarding messages */
#define canOpenCOB_SDO_REQUEST_BASE  0x600 /* Basic COB-ID for SDO request */
#define canOpenCOB_SDO_RESPONSE_BASE 0x580 /* Basic COB-ID for SDO request */

#define canOpenINDEX_NUMBER_OF_PDOS        0x1004
#define canOpenINDEX_SYNC_IDENTIFIER       0x1005
#define canOpenINDEX_RX_PDO_PARAMETER      0x1400
#define canOpenINDEX_RX_PDO_MAPPING        0x1600
#define canOpenINDEX_TX_PDO_PARAMETER      0x1800
#define canOpenINDEX_TX_PDO_MAPPING        0x1A00
#define canOpenINDEX_BUS_REGISTER          0x4500
#define canOpenINDEX_TERMINAL_REGISTER     0x4501
#define canOpenINDEX_PDO_ACTIVATION        0x5500
#define canOpenINDEX_DIGITAL_INPUTS        0x6000
#define canOpenINDEX_DIGITAL_OUTPUTS       0x6200
#define canOpenINDEX_ANALOG_INPUTS         0x6401
#define canOpenINDEX_ANALOG_OUTPUTS        0x6411

#define canOpenSUBINDEX_COB_ID             0x01
#define canOpenSUBINDEX_TRANSMISSION_TYPE  0x02

#define canOpenUPLOAD_REQUEST              0x40   
#define canOpenUPLOAD_RESPONSE_1BYTE       0x4F
#define canOpenUPLOAD_RESPONSE_2BYTE       0x4B
#define canOpenUPLOAD_RESPONSE_3BYTE       0x47
#define canOpenUPLOAD_RESPONSE_4BYTE       0x43
#define canOpenDOWNLOAD_REQUEST_1BYTE      0x2F
#define canOpenDOWNLOAD_REQUEST_2BYTE      0x2B
#define canOpenDOWNLOAD_REQUEST_3BYTE      0x27
#define canOpenDOWNLOAD_REQUEST_4BYTE      0x23
#define canOpenDOWNLOAD_RESPONSE           0x60
#define canOpenERROR_RESPONSE              0x80
#define canOpenRTR_FRAME                   0x101

#define canOpenSTR_OPERATIONAL     "OPERATIONAL"
#define canOpenSTR_PREOPERATIONAL  "PRE-OPERATIONAL"
#define canOpenSTR_STOPPED         "STOPPED"
#define canOpenSTR_UNKNOWN_STATE   "UNKNONW STATE"

/******************** Define error status codes *******************************/
/******************* Type definitions *****************************************/
typedef struct {
    int        node;
    int        net;
    int        card;
    int        chan;
    int        state;
    SEM_ID     semLock;
    SEM_ID     semGuarding;
    SEM_ID     semEmergency;
    BOOL       guarding;
    UINT8      errorInfo[8];
    BOOL       inUse;
    BOOL       sdoVerbose; /* If enables the SDO buffers are printed */
    BOOL       pdoVerbose; /* If enables the PDO buffers are printed */
} canOpenDEVICE;

/*
 * public canOpen interface
 */
STATUS canOpenInstall                           ( );
STATUS canOpenInit                              ( canOpenDEVICE *device );

/*
 * private canOpen interface !!!!!!
 */
STATUS canOpenResolveError                      ( UINT16 obIndex,
                                                  UINT8 *error );
STATUS canOpenSendDownloadRequest               (canOpenDEVICE *device, 
                                                 UINT16 objIndex, 
                                                 UINT8 objSubIndex, 
                                                 UINT8 *data, 
                                                 UINT8 numDataBytes);
STATUS canOpenSendUploadRequest                 (canOpenDEVICE *device, 
                                                 UINT16 objIndex, 
                                                 UINT8 objSubIndex, 
                                                 UINT8 *data, 
                                                 UINT8 *numDataBytes);
STATUS canOpenStartNodeGuard                    (canOpenDEVICE *device,
                                                int guardingTime, 
                                                int lifeTimeFactor);
STATUS canOpenStopNodeGuard                     (canOpenDEVICE *device);
STATUS canOpenReadGuardingResponse              ( canOpenDEVICE *device );
STATUS canOpenSendGuardingRequest               ( canOpenDEVICE *device );
STATUS canOpenSendNMT                           ( canOpenDEVICE *device, 
                                                  int command );
STATUS canOpenGetNodeState                      ( canOpenDEVICE *device );
STATUS canOpenResetDevice                       ( canOpenDEVICE *device );
STATUS canOpenEmergencyWatch                    ( canOpenDEVICE *device );


#endif /*!CANOPEN_H*/
