#ifndef CANIO_COMMANDS_H
#define CANIO_COMMANDS_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: canioCommands.h,v 1.19 2005/09/14 10:45:10 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* vltuser5 2004-09-27 added function canioCmdResetDevice
* vltuser5 2004-04-20 removed function canioCanOpenResetSyncPDOs
* bgustafs 2002-10-31 added command canioCMD_RESET_PDOS
* bgustafs 2002-04-24 removed aioDevice.h
* vltuser5  11/01/02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#include "canio.h"
#include "canioDevice.h"
#include "acroCommands.h" 
#include "aioCommands.h"

/* 
 * acro commands 
 */ 
#define canioCMD_RESET                  100
#define canioCMD_SET_TIMEOUT_TICK       105
#define canioCMD_GET_TIMEOUT_TICK       110
#define canioCMD_FREE_DEVICE            115
#define canioCMD_BLOCK_SEMAPHORE        120
#define canioCMD_READ_CONFIG_BITS       125
#define canioCMD_READ_BITS              130
#define canioCMD_READ_BOARD_ID          135
#define canioCMD_CHECK_PULSE_BIT        140
#define canioCMD_WRITE_CONFIG_BITS      145
#define canioCMD_WRITE_BITS             150
#define canioCMD_WRITE_SET_BIT          155
#define canioCMD_WRITE_CLEAR_BIT        160
#define canioCMD_WRITE_CHANGE_BIT       165
#define canioCMD_WRITE_PULSE_BIT        170
#define canioCMD_WRITE_PULSE_BIT_RESET  175
#define canioCMD_CONFIG_INTERRUPT       180
#define canioCMD_ENABLE_INTERRUPT       185
#define canioCMD_DISABLE_INTERRUPT      190
#define canioCMD_CLEAR_INTERRUPT        195
#define canioCMD_READ_INTERRUPT_STATUS  200

/* 
 * aio commands 
 */
#define canioCMD_GET_BOARD_ID           300
#define canioCMD_READ_VALUE             305
#define canioCMD_GET_INPUT_CONFIG       310
#define canioCMD_GET_OUTPUT_CONFIG      315
#define canioCMD_GET_GAIN               320
#define canioCMD_GET_TIMEOUT            325
#define canioCMD_READ_BIN_VALUE         330
#define canioCMD_WRITE_VALUE            340
#define canioCMD_SET_INPUT_CONFIG       345
#define canioCMD_SET_OUTPUT_CONFIG      350
#define canioCMD_SET_GAIN               355
#define canioCMD_ENABLE_EOC_INTERRUPT   360
#define canioCMD_DISABLE_EOC_INTERRUPT  365
#define canioCMD_AUTOCALIBRATION        370
#define canioCMD_SET_TIMEOUT            375
#define canioCMD_WAIT                   385
#define canioCMD_WRITE_BIN_VALUE        390

/* 
 * new additional  commands 
 */
#define canioCMD_NUM_ANALOG_INPUTS      500
#define canioCMD_NUM_ANALOG_OUTPUTS     505
#define canioCMD_NUM_DIGITAL_INPUTS     510
#define canioCMD_NUM_DIGITAL_OUTPUTS    515
#define canioCMD_GET_VALUE              520
#define canioCMD_GET_BIN_VALUE          525
#define canioCMD_RESET_PDOS             530
#define canioCMD_RESET_DEVICE           535
#define canioCMD_ANALOG_INPUT_CONFIG    540
#define canioCMD_SEND_SYNC              541

/* 
 * redefinition of acro and aio macros 
 */
#define canioSIGNAL_GAIN_X1             aioGAIN_X1
#define canioSIGNAL_GAIN_X10            aioGAIN_X10
#define canioSIGNAL_GAIN_X100           aioGAIN_X100
#define canioSIGNAL_GAIN_X500           aioGAIN_X500

#define canioSIGNAL_RANGE_2x10V         aioCONFIG_2x10V
#define canioSIGNAL_RANGE_10V           aioCONFIG_10V
#define canioSIGNAL_RANGE_2x5V          aioCONFIG_2x5V
#define canioSIGNAL_RANGE_5V            aioCONFIG_5V

#define canioSIGNAL_INPUT               acroSIGNAL_INPUT
#define canioSIGNAL_OUTPUT              acroSIGNAL_OUTPUT

#define canioSIGNAL_POLARITY_ACTIVE_LOW   acroPOLARITY_ACTIVE_LOW
#define canioSIGNAL_POLARITY_ACTIVE_HIGH  acroPOLARITY_ACTIVE_HIGH

typedef struct
    {
    int firstSignal;
    int numSignal;
    int syncEnable;
    } canioCONFIG_ANALOG_IN;

/* 
 * for testing 
 */
/* 
 * toggle pin 7 ( 9 pin connector ) or
 * pin 4 ( 25 pin connector ) on serial port 2
 */
#define SerialLineDown()    (*(unsigned char volatile *)0xfe0002fc = 0x08) 
#define SerialLineUp()      (*(unsigned char volatile *)0xfe0002fc = 0x0a) 

STATUS  canioCmdWriteConfigBits   (canioDEV_PTR  dev_ptr, acroCONFIG *arg_ptr, 
                                   int *error_type_ptr);
STATUS  canioCmdReadConfigBits    (canioDEV_PTR  dev_ptr, acroCONFIG *arg_ptr, 
                                   int *error_type_ptr);
void   canioTimerCallback         (canioSIGNAL_PTR signal);
STATUS canioCmdWritePulseBit      (canioDEV_PTR dev_ptr, acroPULSE *arg_ptr, 
                                   int *error_type_ptr);
STATUS canioCmdWritePulseBitReset (canioDEV_PTR dev_ptr, acroPULSE *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdCheckPulseBit      (canioDEV_PTR dev_ptr, int *arg_ptr, 
                                   int *error_type_ptr);
STATUS canioCmdWriteChangeBit     (canioDEV_PTR dev_ptr, int *arg_ptr, 
                                   int *error_type_ptr);
STATUS canioCmdWriteClearBit      (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdWriteBits          (canioDEV_PTR dev_ptr, acroWRITE *arg_ptr, 
                                   int *error_type_ptr);
STATUS canioCmdReadBits           (canioDEV_PTR  dev_ptr, acroREAD *arg_ptr, 
                                   int *error_type_ptr);
STATUS canioCmdWriteSetBit        (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdSetInputConfig     (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdGetInputConfig     (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdSetOutputConfig    (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdGetOutputConfig    (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdGetGain            (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdSetGain            (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdReadValue          (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdReadBinValue       (canioDEV_PTR dev_ptr, aioBinSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdWriteValue         (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdWriteBinValue      (canioDEV_PTR dev_ptr, aioBinSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdGetValue           (canioDEV_PTR dev_ptr, aioSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS canioCmdGetBinValue        (canioDEV_PTR dev_ptr, aioBinSIGNAL *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdReadBoardId       (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdReset             (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdFreeDevice        (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdWait              (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdGetTimeout        (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdSetTimeout        (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdNumAnalogInputs   (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdNumAnalogOutputs  (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdNumDigitalInputs  (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdNumDigitalOutputs (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdResetDevice       (canioDEV_PTR dev_ptr, int *arg_ptr,
                                   int *error_type_ptr);
STATUS  canioCmdSetAnalogInConfig (canioDEV_PTR device, canioCONFIG_ANALOG_IN *parameter,
                                   int *error_type_ptr);
STATUS  canioCmdSendSync          (canioDEV_PTR device, int *parameter,
                                   int *error_type_ptr);

#endif /*!CANIO_COMMANDS_H*/

