#ifndef CANIODEVICE_H
#define CANIODEVICE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: canioDevice.h,v 1.19 2005/09/14 10:45:11 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* vltuser5 2005-05-04 modifications to use 8 TxPDOs and 8 RxPDOS 
* vltuser5 2004-04-19 removed the Tx and Rx PDOs from 
*                     canioDEVICE_DESCRIPTOR
* vltuser5 2004-04-19 removed the PDO from canioSIGNAL
* vltuser5 2004-04-16 removed the sync identifier from 
*                     canioDEVICE_DESCRIPTOR
* bgustafs 2002-10-29 removed global data and include of canioProtypes.h
* vltuser5  11/01/02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#include "vxWorks.h"
#include "semLib.h"
#include "types.h"
#include "wdLib.h"

#include "canstack.h"
#include "scanopen.h"
#include "mcanopen.h"

    
typedef struct     
    {
    UINT16 index;        /* index of the mapping parameter */
    UINT8  subIndex;     /* subindex of the mapping parameter */
    UINT8  bitWidth;     /* bit width of the mapping parameter */
    } canioPDO_MAPPING;

typedef canioPDO_MAPPING    *canioPDO_MAPPING_PTR;

      
typedef struct 
    {
    HPDO                handle;       /* handle of a local PDO, counterpart to PDO on device  */
    UINT16              identifier;   /* COB-ID */
    UINT16              mapping;      /* first mapped address */
    UINT16              numElements;  /* number of mapped addresses in th PDO   */
    BOOL                remote;
    UINT16              firstSignal;
    UINT16              lastSignal;
    UINT16              dataType;     /* dictionary mapping type */
    BOOL                analogData;
    UINT8               transmitType;
    } canioPDO;

typedef canioPDO    *canioPDO_PTR;


typedef struct 
    {
    UINT8    number;  /* signal identifier */ 
    void    *device;  /* device handle for timer operations,
                         not set after initialisations */
    WDOG_ID  timer;   /* timer for scheduled operations, currently
                         used only for digital signals to realize pulses */
    BOOL     inUse;   /* timer operation in progress */

    canioPDO_PTR pdo;         /* PDO reference */ 

    union 
        {
        struct 
            {
            UINT8   gain;     /* gain 1,2,3,4 */
            UINT8   range;    /* range 1-5 */
            UINT16  value;    /* analog value 16 bit */
            } analog;
        struct
            { 
            UINT8   polarity; /* polarity active low or high */
            UINT8   value;    /* digital value 0,1 */
            } digital;
        } type;       /* signal type */
    } canioSIGNAL;

typedef canioSIGNAL         *canioSIGNAL_PTR;

/* enum describing the states of an analog input signal on a CAN device. It describes what
   PDO (or SDO) must be used to get the signal value */
typedef enum
    {
    SIGNAL_UNUSED = 1,  /* signal is disabled   */
    
    SIGNAL_ACTIVE = 2,  /* signal is enabled    */

    SIGNAL_VALID =  4,  /* value of signal is valid */

    SIGNAL_INVALID = 8, /* value of signal is invalid, signal is pending    */

    
    RTR_PDO    = 32,    /* asynchron PDO must be used for this signal, signal is active */

    SYNC_PDO   = 64,    /* synchron PDO must be used for this signal    */

    SDO_MAPPED = 128,   /* not mapped into a PDO, signal value must be read by an SDO request */

    
    SYNC_PENDING = 74,  /* synchron PDO is used + waiting for signal value */
    SYNC_VALID   = 70,    /* synchron PDO is used + signal value has arrived -> can be read */
    
    
    } canioANALOG_IN_STATE;


/* struct storing information of one analog input signal    */
typedef struct 
    {
    canioANALOG_IN_STATE    state;              
    UINT16                  value;
    } canioANALOG_IN_SIGNAL;

typedef canioANALOG_IN_SIGNAL         *canioANALOG_IN_SIGNAL_PTR;


typedef struct 
    {
                
    UINT32                  signalInvalid;    /* to detect when the semDataEvent sempahore shall be  
                                                 fired. It is fired when no more flags are set in this
                                                 bitmask    */
    UINT16                  firstSignal;      /* first signal that is active    */
    UINT16                  lastSignal;       /* last signal that is active     */

    BOOL                    syncActive;       /* analog inputs can be read by a SYNC or a RTR frame
                                                TRUE: SNYN mechanism is enabled, nothing is read per RTR  */

    SEM_ID                  semDataValid;     /* binary semaphore is given after signalTable[] is completely  
                                                updated by DataEventHandler()   */

    SEM_ID                  semSignalAccess;  /* mutex semaphore for exclusive mutual access into signalTable[] 
                                                done by DataEventHandler(), Read() and SYNC() function  */

    canioANALOG_IN_SIGNAL_PTR signalTable;    /* an array containg "numAnalogInSignals" elements     */

    
    } canioANALOG_INPUT;

typedef canioANALOG_INPUT         *canioANALOG_INPUT_PTR;

/*
 *  Device descriptor 
 */
typedef struct 
    {
    lcudrvDEVICE_HEADER   header;
    SEM_ID                semAccess;
    canstackDEVICE        canDevice;
    HNODE                 nodeHandle;
    UINT8                 numTerminals;
    canioPDO_PTR          recvPDOTable;
    canioPDO_PTR          sendPDOTable;
    UINT8                 numRecvPDO;
    UINT8                 numSendPDO;
    UINT16                numDigitalInSignals;
    canioSIGNAL_PTR       digitalInSignal;
    UINT16                numAnalogInSignals;
    canioSIGNAL_PTR       analogInSignal;
    UINT16                numDigitalOutSignals;
    canioSIGNAL_PTR       digitalOutSignal;
    UINT16                numAnalogOutSignals;
    canioSIGNAL_PTR       analogOutSignal;
    canioPDO_MAPPING_PTR  txPdoMapping;
    canioPDO_MAPPING_PTR  rxPdoMapping;

    canioANALOG_INPUT_PTR analogInputSync;      /* contains the special handling for analog inputs  */

    } canioDEVICE_DESCRIPTOR;

typedef canioDEVICE_DESCRIPTOR      *canioDEV_PTR;

typedef struct 
    {
    int       cmd;
    UINT8     access;
    BOOL      protected;
    FUNCPTR   handler;
    } canioCMD_DESCRIPTOR;

typedef canioCMD_DESCRIPTOR *canioCMD_PTR;

typedef struct
    {
    int                     driver;
    int                     timeout;
    int                     numCommands;
    canioCMD_PTR            cmdTable;
    int                     numDevices;
    canioDEV_PTR            devTable;       
    int                     numChannels;
    struct lcudrvDCT_ENTRY *devChannelTable;
    } canioDEVICE;

#endif /*!CANIODEVICE_H*/
