#ifndef CANLIB_H
#define CANLIB_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: canlib.h,v 1.9 2005/05/09 15:51:57 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2002-07-04 created from canLib.h
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#define VME_A16    VME_AM_SUP_SHORT_IO
#define VME_A24    VME_AM_STD_SUP_DATA
#define VME_A32    VME_AM_EXT_SUP_DATA

#define int8_t     char
#define uint8_t    unsigned char
#define int16_t    short
#define uint16_t   unsigned short
#define int32_t    long
#define uint32_t   unsigned long

#ifdef _WORDA_

#define copy32data(dest, src)              \
         ((uint16_t *)(dest))[0] = ((uint16_t *)(src))[0]; \
         ((uint16_t *)(dest))[1] = ((uint16_t *)(src))[1]
                        
#define store32data(dest, dat32) \
         ((uint16_t *)(dest))[0] = (uint16_t)(((uint32_t)(dat32))>>16); \
         ((uint16_t *)(dest))[1] = (uint16_t)((uint32_t)(dat32))

#define load32data(src) \
         ((uint32_t)(((uint16_t *)(src))[0])<<16 | \
          (uint32_t)(((uint16_t *)(src))[1]))

#else

#define copy32data(dest, src) \
         *((uint32_t *)(dest)) = *((uint32_t *)(src)) 

#define store32data(dest, dat32) \
         *((uint32_t *)(dest)) = (uint32_t)(dat32) 

#define load32data(src) \
         *((uint32_t *)(src)) 

#endif

#if PRINTF
#define canlibDEBUG_PRINT(format, arg...) \
	canlibPRINTF(format, ##arg)
#else
#define canlibDEBUG_PRINT(format, arg...)
#endif


/* Typedefinitions (for all esd - I_cards) */

struct if_channel_t { 
  int32_t   iofor;
  int32_t   ioback;
  uint16_t  iotyp;
  char      ioname[6];
  int8_t    iosema;                  /* 0x10 */
  uint8_t   iostat;
  uint16_t  iocmmd;
  uint32_t  reserved1;
  uint8_t   ioilev;
  uint8_t   ioivec;
  uint16_t  reserved2;
  int32_t   iolen;
#ifdef  _WORD_
  int16_t   iobuff[2];               /* 0x20 */
#else
  int32_t   iobuff;
#endif  /* _WORD_ */
  int16_t   iorecl;                  /* 0x24 */
  uint8_t   iostio;
  int8_t    ioldn;
  uint16_t  iomode;                  /* 0x28 */
  uint8_t   iotout;
  uint8_t   iodrv;
  uint32_t  iofnam[6];               /* 0x2C */
  char      ioctrlr[4];              /* 0x44 */
  char      ioflag[8];               /* 0x48 */
  int16_t   iorxln;                  /* 0x50 */
  int16_t   reserved3[25];
};

/****************************************************************/
/* Define a macro that enforces in order I/Oexecution for 604   */
/* PowerPC platform                                             */
/****************************************************************/
#if (CPU==PPC604)          
    #define eieio() __asm("eieio")
#else
    #define eieio()
#endif

/****************************************************************/
/* To force alignment on non 68k CPUs we are using the keyword  */
/* __attribute__ which comes with compiler version > 2.7        */ 
/****************************************************************/
#if defined(__GNUC__) && (!(defined(mc68000) || defined(m68k)))
    #if (__GNUC__ < 2) || ((__GNUC__ == 2) && (__GNUC_MINOR__ < 7))
        #define __attribute__(arg)
        #warning Ignoring __attribute__((packed)) keywords
    #endif
#endif

/****************************************************************/
/*           Global Constants                                   */
/****************************************************************/
#define MAX_CAN_CHANS   4          /* max. channels on one card */
#define MAX_CAN_CRDS    8          /* max. count of CAN-cards (2^x)*/
#define MAX_CAN4_NETS   (MAX_CAN_CHANS * MAX_CAN_CRDS)

#define VME_CAN4        1       /* Indentifier for CAN4 */

#define data1_base  0x10000        /* RAM-range */
#define ctrl1_base  0x18000
#define data2_base  0x20000
#define ctrl2_base  0x28000
#define TODATA(y)   ((CAN4_DATA*)(((uint32_t)(y))&0xFFFF7FFF))
#define TOCTRL(y)   ((CAN4_CTRL*)(((uint32_t)(y))|0x00008000))
#define para_base   0x8000

#define io_base    0x78000          /* io-range */
#define fifo_base  (io_base+0x1602)   /* in io-range */
#define pit_base   (io_base+0x6001)
#define girl_base  (io_base+0x0002)
#define can2_reset (io_base+0x7000)

#define clock_base 0x1FFFF0      /* Base of clock area */ 

#define can4_cpuid 0x43414e34    /* text 'CAN4' */
#define can4_netid 0x43414e50    /* text 'CANP' */

#define channels   2032          /* max can_id (CiA)*/
#define canids     2048          /* max can_ids (CAN2) */

#define Tx_TO      0             /* modes */
#define Rx_TO      1
#define Tx_RTR     2
#define CAN_MON    4
#define CAN_SER    5
#define OLD_MODE   0xF
#define RX_BUF     0x8000
#define TX_BUF     0x8100


#define SET_BD_1    0             /* commands */
#define SET_BD_2    1
#define RX_FIL_1    2 
#define RX_FIL_2    3 
#define TRG_MON     4 
#define N_LINK      6
#define N_UNLINK    7
#define CYC_TX      8
#define CMS_WDOG    8
#define UNCYC_TX    9
#define CMS_NWDOG   9
#define INT_ENAB    10
#define SET_MODE    11
#define INIT_DOMAIN 12  
#define REL_CMSBUF  13
#define RX_BUFFER   14
#define REL_RXBUF   15
#define GET_DLD_BUF 18
#define DLD_DRIVER  19
#define GET_TXBUF   20
#define REL_TXBUF   21
#define CLOCK       31
#define LAST_CMMD   REL_TXBUF

#define ALL_ID     (-1)
#define CMS_ID     (-2)
#define NOT_WDG    (-3)
#define ALL_NET    (-1)
#define ALL_CARD   (-1)

#define LEN_MSK    0x000F
#define RTR_MSK    0x8000

#define MAXTRY     100             /* count trials to TAS para-semaphore */

#define CAN_MSG_SIZE 8           /* size of a message (canMsgServer) */

#define MODE       1               /* defines for canSet() */
#define TIMEOUT    2
#define BAUDRATE   3
#define ID_STAT    4

#define STAT_OK           0x0000     /* values of ID_Status */
#define STAT_RX_TIMOUT    0x0001
#define STAT_TX_TIMOUT    0x0002
#define STAT_TX_ERROR     0x0004
#define STAT_OFF_BUS      0x0005
#define STAT_BUSY         0x0006
#define STAT_TX_BAUDRATE  0x0007    /* Tx active while setting baudrate */
#define STAT_NO_NET       0x0008
#define STAT_RTR_REC      0x0101
#define STAT_WAIT_REM     0xFFFE
#define STAT_IN_USE       0xFFFF
#define STAT_TX_IN_QU     0xFFFD

/****************************************************************/
/*            typedefinitions                                   */
/****************************************************************/
struct CAN4_CONFIG    
    {
    unsigned short a16_base;
    unsigned char  irq_lev;    
    unsigned char  irq_vec;
    unsigned long  a24_base; 
    unsigned long  a32_base; 
    char           net[4]; 
    };

struct CAN4_A16    
    {                             /* offset */
    unsigned short car;           /* 0x00 */
    unsigned short cstat;         /* 0x02 */
    unsigned char  dummy[0x3c];   /* 0x04 */
    unsigned short irq_cnt_high;  /* 0x40 */    /* ack. IRQ */
    unsigned short irq_cnt_low;   /* 0x42 */    /* start timer */
    unsigned char  dummy1[0x3d];  /* 0x44 */
    unsigned char  irq_vec_lev;   /* 0x81 */    /* Set IRQ vector and level */
    unsigned short irq_fifo;      /* 0x82 */    /* FIFO-Latch */
    unsigned char  dummy2;        /* 0x84 */
    unsigned char  irlev;     	  /* 0x85 */    /* Get IRQ vector and level */
    unsigned short irq_fifo2;     /* 0x86 */    /* FIFO-Latch */ 
    unsigned char  dummy3;        /* 0x88 */     
    unsigned char  fstat;         /* 0x89 */    /* Get IRQ vector and source */ 
    unsigned short irq_fifo3;     /* 0x8A */    /* FIFO-Latch */
    unsigned char  dummy4;        /* 0x8C */
    unsigned char  istat;         /* 0x8D */    /* Get IRQ vector and status */  
    unsigned short irq_fifo4;     /* 0x8E */    /* FIFO-Latch */
    } __attribute__((packed));

struct CAN4_LINK    
    {
    unsigned short ctoff;
    unsigned short ctlen;
    unsigned long  c4id; 
    unsigned long  swrev; 
    unsigned long  swcms;
    };

struct CAN4_NET_LINK    
    {
    unsigned long  c4cdat;
    unsigned long  c4iobf;
    unsigned long  c4ctrg; 
    unsigned long  c4mtrg; 
    };

/* can_bus_data_t */
typedef struct
    {
    int16_t  length;
    union
        {
        int16_t  d[4];
        char     a[8];
        } u;
    uint16_t status;
    int16_t  evtrg;
    int16_t  timeout;
    } CAN4_DATA;

/* can_bus_ctrl_t */
typedef struct
    {
    int32_t  next_ptr;
    uint8_t  id_stat;
    uint8_t  id_mode;
    int16_t  time_ctr;
    int32_t  ext_tid;
    int32_t  node_link;
    } CAN4_CTRL;

struct can4_bus_mon_t
    {
    uint16_t idf;
    union
        {
        int16_t  d[4];
        char     a[8];
        } u;
    uint32_t time_stamp;
    int16_t  dummy15;
    } __attribute__((packed));

struct can4_para_buff_t
    {
    uint16_t param1;      /* 1. parameter of command channel (r/w) */
    uint16_t param2;      /* 2. parameter of command channel (r/w) */
    uint16_t param3;      /* 3. parameter of command channel (r/w) */
    uint16_t param4;      /* 4. parameter of command channel (r/w) */
    uint32_t pretpar;     /* Return value of command channel (r/w) */
    int32_t  ir_trigger;  /* VMEbus interrupt trigger address (r/w) */
    uint32_t mon_base;    /* Local address of monitor (ro) */
    uint16_t mon_s_len;   /* Length of one element in monitor buf (ro) */
    uint16_t mon_max;     /* Max no of entries in monitor buffer (ro) */
    uint32_t reserved1[2];
    uint16_t reserved2;
    uint16_t swbr;
    uint16_t actbr;
    uint32_t struct_base;
    uint16_t s_len;
    uint16_t s_max;
    uint16_t reserved3[2];
    uint16_t c_stat;
    uint32_t reserved4[3];
    uint32_t reserved5[8];
    uint16_t iowdgm;      /* Action if master watchdog timed out (r/w) */
    uint16_t reserved6;
    uint16_t iocirs;      /* Command clock IR status (r/w) */
    uint16_t ioctdf;      /* Clock PLL time difference (ro) */
    uint32_t ioctim;      /* Time in ms on last 100msec IRQ (ro) */
    uint32_t ioclnk;      /* Link address clock command server (ro) */
    uint32_t reserved8;
    uint16_t local;
    uint16_t irqmsk;
    uint32_t ackirq;
    uint32_t ownack;
    } __attribute__((packed));

typedef struct
    {         /* CAN_CRD */
    BOOL                     present;
    BOOL                     irqConnected;   /* irq vectors connected */
    int                      netno[4];       /* first,second,... net_no */
    unsigned long            base;           /* base-ptr to card for locals */
    unsigned long            base2;          /* base-ptr to card */
    const struct CAN4_CONFIG *can4_config;
    struct CAN4_A16          *can4_a16;
    struct CAN4_LINK         *can4_link;
    volatile unsigned short  *intack;        /* Acknowledge of 68331 IRQ */   
    volatile unsigned long   *clock;         /* Ptr to read area of clock */
    int16_t                  *can_fifo;
    SEM_ID                   irq_cnt_sema;   /* for waiting for counter IRQ */
    SEM_ID                   irq_clock_sema; /* for waiting for clock IRQ */
    unsigned long            timems;         /* Time in ms at last IRQ */
    unsigned short           irq_status;     /* 68331 interrupt status */
    CAN4_CTRL                *ctrl[4];       /* control-field holds sema */
    } CAN4_CRD;

typedef struct
    {         /* CAN_DEV */
    BOOL                   initzed;        /* if structure is initialized */
    int                    card_no;        /* net on Card_No */
    CAN4_DATA              *data;
    CAN4_CTRL              *ctrl;
    struct CAN4_NET_LINK   *can4_net_link;
    struct if_channel_t    *para_chan;
    struct can4_para_buff_t *para_buff;
    SEM_ID                 para_sema;      /* for requesting parameter channel */
    SEM_ID                 cmd_sema;       /* for interrupt response */
    int16_t                *can_trig;      /* triggeraddress local server */
    int16_t                chan;           /* channel on card 0...3 */
    int16_t                reserved; 
    CAN4_CRD               *crdptr;
    } CAN4_DEV;

/****************************************************************/
/*                      Prototypes                              */
/****************************************************************/

void canlibPRINTF(char *format, ... );

STATUS canSettings( int card,   
                    unsigned short a16_base,    
                    unsigned long  a24_base,    
                    unsigned long  a32_base );   
/*
DESCRIPTION
Set VME bus addresses for a specific CAN card
*/

unsigned long canSetup( const struct CAN4_CONFIG *, 
						struct CAN4_A16 **);
/*
DESCRIPTION
Set hardware settings and filename of firmware image for CAN2
*/

STATUS canInit(int net);

/*
DESCRIPTION
initialise can i/f memory
*/

STATUS canNetInit( int net );
/*
DESCRIPTION
Initialize a single node on CAN4
*/

STATUS canNet( int card, int chan, int *net );
/*
DESCRIPTION
Returns the net associated with the card and the channel
*/

STATUS canSend(int net, int txid, const char *dbuff, int len);
/*
DESCRIPTION
Send a message to CAN
*/

STATUS canUpdate(int net, int txid, const char *dbuff, int len);
/*
DESCRIPTION
Update the CAN-data structure
*/

STATUS canTake(int net, int rxid, char *dbuff, int *len);
/*
DESCRIPTION
Take a message from CAN
*/

STATUS canReadW(int net, int rxid, char *dbuff, int *len, SEM_ID can_sema);
/*
DESCRIPTION
Wait for a Rx_Message on a CAN_Id
*/

STATUS canRead(int net, int rxid, char *dbuff, int *len, SEM_ID can_sema);
/*
DESCRIPTION
Read a CAN-message from id, wait if no message available
*/


STATUS canWriteW(int net, int txid, const char *dbuff,
				 int len, SEM_ID can_sema);
/*
DESCRIPTION
write a message to CAN and wait for it
*/

STATUS canReadRem(int net, int rxid, char *dbuff, int *len,
				  SEM_ID can_sema);
/*
DESCRIPTION
Send a Remote_Request on a CAN_Id and wait for a Rx_Message
*/


STATUS canWriteRead(int net, int txid, const char *tbuff,int tlen, 
					int rxid, char *rbuff, int *rlen, SEM_ID can_sema);
/*
DESCRIPTION
Send a message on a CAN_ID and wait for a rx_ressage on another ID
*/


STATUS canSendRem(int net, int txid);
/*
DESCRIPTION
Send a Remote_Request on a CAN_Id 
*/

STATUS canCmmd ( int net, int16_t cmmd, 
				 int16_t para1, int16_t para2, int16_t para3, int16_t para4,
				 int32_t *retpar);
/*
DESCRIPTION
Send Command to vme_can4
*/

STATUS canSet(int net,  int canid, int variable, int value);
/*
DESCRIPTION
set a variable in can4
*/

void canIntFifo( CAN4_DEV *dptr );
/*
DESCRIPTION
CAN-interrupt-serviceroutine - CAN Fifo-Server
*/

void canIntCount( CAN4_CRD *cptr );
/*
DESCRIPTION
CAN-interrupt-serviceroutine - CAN Irq-Counter
*/

void canIntCollect( CAN4_CRD *cptr );

void DEBUG_PRINT(char *ff, ... );

/*
DESCRIPTION
CAN-interrupt-serviceroutine - CAN 68331-Irq(Collect-Irq)
*/

#endif /*!CANLIB_H*/
