/*****************************************************************************
 * E.S.O. - VLT Project
 *
 * "@(#) $Id: ccdLcuInternal.h,v 4.11 2005/06/22 10:57:48 vltsccm Exp $"
 *
 * who         when     what
 * --------- -------- ---------------------------------------------------
 * P.Duhoux  16/02/96 first implementation
 *                         - contains part of previous ccdcon.h which was
 *                           common to all sub-systems
 * P.Duhoux  13/03/96 Added setupResult structure in ccdDESCRIPTION
 * P.Duhoux  22/05/96 Added parameter <errReply> in fct ccdconWaitReply()
 * P.Duhoux  31/05/96 Added functions ccdconSet/ResetFailureLcu()
 * alongino  12/07/96 ccdconTASK_NAME_ACT_FMT moved to ccdInternal.h
 * P.Duhoux  16/07/96 changed type of ccdDesc->failure to vltUINT32
 * P.Duhoux  17/07/96 renamed fct ccdconEnvVarXxx to ccdEnvVarXxx   
 * P.Duhoux  06/08/96 moved ccdcon*Fis*.c, ccdconStoreTime.c from ccdcon
 * P.Duhoux  08/08/96 moved ccdconSendCmd*.c, ccdconWaitReply.c from ccdcon
 *                    moved ccdKill.c, ccd*Ace.c from ccdcon
 * P.Duhoux  08/08/96 ccdconSTATE replaced by ccdSTATE
 *                    ccdONLINE = ccdOPERATING
 * P.Duhoux  09/08/96 Added parameter <reply> in fct ccdWaitReply()
 *                    Added parameter <reply> in fct ccdSendCmd()
 * P.Duhoux  21/08/96 Moved fct ccdcon*ImBuf() from ccdcon
 * P.Duhoux  22/08/96 Added fct ccdFindImBuf() & ccdLoadImBuf()
 *                    Added parameter <storageType> in fct ccdAllocImBuf()
 *                    Added parameter <force> in fct ccdFreeImBuf()
 * P.Duhoux  22/08/96 Added fct ccd*Fits*.c and type ccdFITS for FITS file op
 * P.Duhoux  18/10/96 Added macros for IP file names & format (ccdFRAME_xxx)
 * P.Duhoux  27/01/97 Added times in ccdTIMES structure
 * P.Duhoux  12/02/97 Added <timer> data structure in ccdDESCRIPTION structure
 * P.Duhoux  12/03/97 Added <teleOn> and <tempOn> in ccdDESCRIPTION structure
 * P.Duhoux  02/04/97 Added ccdTELESTAT & ccdTEMPSTAT data structures
 *                    Added <teleStat> and <tempStat> in ccdDESCRIPTION
 * P.Duhoux  08/04/97 Indexed TMP sensor formats for symetry with TEL
 * pduhoux   06/05/97 moved ccdPROG_TEC/SCI to ccdInternal.h
 * pduhoux   01/07/97 extended ccdTIMES with <shiftStart>, <shiftTime> & 
 *                    <stepNum> for multiple exposures
 * pduhoux   02/07/97 permutated task prio TIME (now 35) 
 *                    with READ/SHUT (now 30)
 * alongino  09/05/98 changed priorities according to SPR 980215 
 * pduhoux   25/05/98 changed ccdCITASK_REGISTER() to set task priority
 *
 */
 
/*
 ****************************************************************************
 * This header file belongs to the CCD LCU software. It contains global data
 * which is needed commonly by all CCD LCU modules.
 ****************************************************************************
 */

#ifndef CCD_LCU_INTERNAL_H
#define CCD_LCU_INTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 ****************************************************************************
 * Includes
 ****************************************************************************
 */
 
/* 
 * System Includes 
 */
#include "vxWorks.h"
#include "envLib.h"
#include "ioLib.h"
#include "math.h"
#include "semLib.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "sysLib.h"
#include "taskLib.h"
#include "tickLib.h"
#include "vxLib.h"
#include "ctype.h"
#include "usrLib.h"
#include "errnoLib.h"

/* 
 * CCD include files 
 */
#include "ccdInternal.h"
#include "too.h"

#ifdef MEM_TRACE
#include "lccMemoryTrace.h"
#endif
#include "lccTaskLib.h"

/*
 ****************************************************************************
 * Data Types and Literals
 ****************************************************************************
 */
#define ccdACE_B016_DRV  "/bxvi00"
#define ccdACE_ARC0_DRV  "/arc0"

#define ccdTASK_FLAGS     (VX_FP_TASK)

/* stack size */
#define ccdTASK_STACK_30K  30000
#define ccdTASK_STACK_40K  40000
#define ccdTASK_STACK_50K  50000
#define ccdTASK_STACK_80K  80000

#define ccdTASK_STACK_SRV    ccdTASK_STACK_50K

#define ccdTASK_STACK_SML    ccdTASK_STACK_30K
#define ccdTASK_STACK_ACT    ccdTASK_STACK_40K
#define ccdTASK_STACK_BIG    ccdTASK_STACK_50K
#define ccdTASK_STACK_XXL    ccdTASK_STACK_80K

#define ccdTASK_STACK_TIME   ccdTASK_STACK_SML
#define ccdTASK_STACK_CTRL   ccdTASK_STACK_BIG
#define ccdTASK_STACK_READ   ccdTASK_STACK_ACT
#define ccdTASK_STACK_WIPE   ccdTASK_STACK_SML
#define ccdTASK_STACK_SHUT   ccdTASK_STACK_ACT
#define ccdTASK_STACK_TRAN   ccdTASK_STACK_XXL
#define ccdTASK_STACK_PROC   ccdTASK_STACK_ACT
#define ccdTASK_STACK_TELE   ccdTASK_STACK_SML
#define ccdTASK_STACK_TEMP   ccdTASK_STACK_SML

#define ccdMOD_NAME_CON      "ccdcon"
#define ccdMOD_NAME_IT       "ccdit"
#define ccdMOD_NAME_IP       "ccdip"
#define ccdMOD_NAME_RDT      "ccdrdt"
#define ccdMOD_NAME_SHT      "ccdsht"
#define ccdMOD_NAME_TEL      "ccdtel"
#define ccdMOD_NAME_TMP      "ccdtmp"
#define ccdMOD_NAME_AUX      "ccdaux"

typedef enum
    {
    ccdMOD_INDEX_CON = 0,
    ccdMOD_INDEX_IT,
    ccdMOD_INDEX_IP,
    ccdMOD_INDEX_RDT,
    ccdMOD_INDEX_SHT,
    ccdMOD_INDEX_TEL,
    ccdMOD_INDEX_TMP,
    ccdMOD_INDEX_AUX,
    ccdMAXSUBSYS
    }
ccdMOD_INDEX;

extern vltBYTES32 ccdDoFitsConfLog;
extern vltINT32   ccdDebugLevel;
extern int        ccdCmdInitTimeout;

#define ccdFUNC_MAIN_FMT     "_%sMainTask"
#define ccdFUNC_NAME_SRV_FMT "_%sStartServer"

#define ccdTASK_NAME_SRV_FMT "%s_%s"                /* "%sSrv_%s" */
/* AL960712 moved to ccdInternal.h in ws part: needed by ccditWs for dxf JUN96 
#define ccdTASK_NAME_ACT_FMT "%sAct_%s"
*/
#define ccdTASK_NAME_TIM_FMT "ccdTim_%s"
#define ccdTASK_NAME_WIP_FMT "%sWip_%s"
#define ccdTASK_NAME_MON_FMT "%sMon_%s"

#define ccdCITASK_REGISTER(brkHdlr,kilHdlr,priority) \
  {                                           \
  char        vxProcName[7];                  \
  ccsPROCNAME ccsProcName;                    \
                                              \
  taskPrioritySet(taskIdSelf(),priority);     \
  /*                                          \
   * get first 7 chars of vx procname and     \
   * mangle with taskId to get unique         \
   * ccs proc name                            \
   */                                         \
  strncpy(vxProcName,taskName(taskIdSelf()),7);                        \
  vxProcName[7] = 0;                                                   \
  sprintf(ccsProcName,"%s%x",vxProcName,taskIdSelf());                 \
  if (ccsInit(ccsProcName,0,brkHdlr,kilHdlr,error) != SUCCESS)         \
    { errAdd(error,ccdMODULE,ccdERR_LCC_CALL,__FILE__,"%s","ccsInit"); \
      return FAILURE; }                                                \
  }

/* timeouts */
#define ccdSTARTUP_TIMEOUT   6000   /* 60 seconds for sub_system startup  */
#define ccdSYNC_TIMEOUT       500   /*  5 seconds for task synchonization */

/* conversions */
#define ccdUS_TO_TICKS(m)    ((m) / 10000)
#define ccdMS_TO_TICKS(m)    ((m) / 10)
#define ccdTICKS_TO_MS(t)    ((t) * 10)
  
/* environment variables */
#define ccdENV_CPU        "CPU"
#define ccdENV_CCDNAME    "CCDNAME"
#define ccdENV_LOCALENV   "LOCALENV"
#define ccdENV_INSROOT    "INS_ROOT"
#define ccdENV_INSHOST    "INS_HOST"
#define ccdENV_INSUSER    "INS_USER"

/* host directory paths */
#define ccdDIR_CONFIG_FMT  "%s/%s/COMMON/CONFIGFILES"  /* $INS_ROOT/$INS_USER */
#define ccdDIR_BINARY_FMT  "vw/bin/%s/%s"

/* macros for STATUS command handling */
#define ccdSTATUS_FMT      "opMode %d,opState %d,status %d"

/* macros for DBCFG file handling */
#define ccdDBCFG_FMT       "%s.dbcfg"
#define ccdDBCFG_CWP_FMT   "%s=@%s<alias>%s"
#define ccdDBCFG_DCS_FIXED "ccdDcsFixed"
#define ccdDBCFG_CWP       "<CWP>: "

/* macro for LCC timer overhead correction [us] */ 
#define ccdTIMER_SLEEP_OVERHEAD  200

/* macro defining the limit of expTime [us] */ 
#define ccdONE_SECOND_EXPTIME   1000000

/*
 * base file names for IP auxilliary images
 * -> read from <images.imageDirectory>/<ccdFRAME_BASEtype>_<ccdName>.fits
 *    eg "bias_ccdTEC.fits"
 */ 
#define ccdFRAME_BASEBIAS   "bias"
#define ccdFRAME_BASEDARK   "dark"
#define ccdFRAME_BASEFLAT   "flat"

#define ccdFRAME_FITSFMT    "%s/%s_%s.fits"

/* Julian Day constants */
#define ccdJDToSec(jd)       ((vltDOUBLE)(jd) * 86400.0)
#define ccdSecToJD(s)        ((vltDOUBLE)(s)  / 86400.0)

#define ccdUTCtoSec(utc) \
    (vltDOUBLE)((double)((utc)->tv_sec) + (double)((utc)->tv_usec) / 1e6)
#define ccdUTCtoUSec(utc) \
    (vltINT32)(1000000 * (utc)->tv_sec + (utc)->tv_usec)
#define ccdSecToUTC(sec,utc) \
    (utc)->tv_sec = (int)(sec); \
    (utc)->tv_usec = (int)(((sec) - (utc)->tv_sec) * 1e6)

#if 0
#define ccdAddSecToUTC(sec,utc) \
    { int s,us; s = (int)(sec); us = ((sec) - s) * 1000000; \
      (utc)->tv_sec += s; (utc)->tv_usec += us;     \
      while ((utc)->tv_usec >= 1000000)              \
        { (utc)->tv_usec -= 1000000; (utc)->tv_sec++; }}
#define ccdSubSecFromUTC(sec,utc) \
    { int s,us; s = (int)(sec); us = ((sec) - s) * 1000000; \
      (utc)->tv_sec -= s; (utc)->tv_usec -= us;     \
      while ((utc)->tv_usec < 0)                    \
        { (utc)->tv_usec += 1000000; (utc)->tv_sec--; }}
#else
/* SPR20050198 */
#define ccdAddSecToUTC(sec,utc) \
{ \
    int s,us,ts=1; \
    double as = fabs((double)(sec)); \
    if ((sec) < 0) ts = -1; \
    s = (int)floor(as); us = (as - s) * 1000000; \
    (utc)->tv_sec  +=  s * ts; (utc)->tv_usec += us * ts; \
    while ((utc)->tv_usec >= 1000000) \
	{ (utc)->tv_usec -= 1000000; (utc)->tv_sec++; } \
    while ((utc)->tv_usec <        0) \
	{ (utc)->tv_usec += 1000000; (utc)->tv_sec--; } \
}
#define ccdSubSecFromUTC(sec,utc) \
    ccdAddSecToUTC(-(sec),utc)
#endif

/* macros for Telemetry sensors */
#define ccdTEL_CLOCK_FMT        "CLOCK_%d_%c"
#define ccdTEL_CLOCK_LOW        'L'
#define ccdTEL_CLOCK_HIGH       'H'

#define ccdTEL_BIAS_FMT         "BIAS_%d"

/* macros for Temperature sensors */
#define ccdTMP_CHIP_FMT         "CHIP_%d"
#define ccdTMP_HOUSE_FMT        "HOUSE_%d"

/* exposure events */
#define ccdEVT_EXPIRED     1
#define ccdEVT_ENDED       2
#define ccdEVT_ABORTED     4
#define ccdEVT_PAUSED      8
#define ccdEVT_CONTINUED  16
#define ccdEVT_ERROR      32
#define ccdEVT_STOP      256

/* database symbolic access */
#define ccdDB_RETRY_COUNT    1    /* retry count for failed db access  */
#define ccdDB_RETRY_DELAY    0    /* retry delay for failed db access  */

/* database attributes index for direct access CCD */
#define ccdDB_OPSTATE        0
#define ccdDB_FAILURE        1
#define ccdDB_INITDONE       2
#define ccdDB_IMG_LST        3
#define ccdDB_IMG_DIR        4

/* command wait mode */
#define ccdCMD_WAIT_NONE     1    /* Do not wait for reply              */
#define ccdCMD_WAIT_ONCE     2    /* Wait for first reply               */
#define ccdCMD_WAIT_LAST     3    /* Wait for last reply                */

/* CCD types : Scientific / Technical */
typedef enum
    {
    ccdCCD_SCI = 0,
    ccdCCD_TEC
    } ccdCCD_TYPE;

/* fast Intertask communication Service routines */
typedef struct
    {
    vltINT32 id;                /* FIS Id = structure address */
    vltINT32 timeout;           /* semaphore timeout in ticks */
    vltINT32 cmd;               /* command                    */
    SEM_ID   cmdSem;            /* command semaphore          */
    vltINT32 reply;             /* reply                      */
    SEM_ID   replySem;          /* reply semaphore            */
    } ccdFIS;
  
/* camera HW description */
typedef struct
    {
    ccdCCD_TYPE     type;        /* CCD system type [ccdCCD_TEC/SCI]       */
    vltBYTES32      link;        /* HW device name                         */
    vltINT32        tout;        /* HW device timeout [ms]                 */
    vltBYTES32      prog;        /* HW program name (embedded SW)          */
    } ccdHWDESC;

/* sub-system description */
typedef struct
    {
    vltINT32        avail;       /* Sub-system availability flag           */
    ccsPROCNAME     server;      /* Sub-system Server process name         */
    char           *desc;        /* Sub-system descriptor                  */
    } ccdSUBSYS;

/* Telemetry statistics */
typedef struct
    {
    vltINT32  numSrc;
    vltINT32  period;
    vltDOUBLE start[ccdMAXTELSRC];
    vltDOUBLE stop[ccdMAXTELSRC];
    vltINT32  numVal;
    vltDOUBLE valSum[ccdMAXTELSRC];
    vltDOUBLE valSum2[ccdMAXTELSRC];
    } ccdTELESTAT;

/* Temperature statistics */
typedef struct
    {
    vltINT32  numSns;
    vltINT32  period;
    vltDOUBLE start[ccdMAXTMPSRC];
    vltDOUBLE stop[ccdMAXTMPSRC];
    vltINT32  numVal;
    vltDOUBLE valSum[ccdMAXTMPSRC];
    vltDOUBLE valSum2[ccdMAXTMPSRC];
    } ccdTEMPSTAT;

/* structure, containing various times         */ 
/* All times are UTC, durations are in seconds */
#define ccdProtectTimes(desc)   semTake(desc->times.updateSem,2)
#define ccdUnProtectTimes(desc) semGive(desc->times.updateSem)
typedef struct
    {
    SEM_ID     updateSem;         /* critical section protection semaphore  */
    ccdEXPTYPE expType;           /* exposure type                          */
    ccsTIMEVAL wipeStart;         /* wipe start time                        */
    vltDOUBLE  wipeTime;          /* wipe time                              */
    ccsTIMEVAL wipeEnd;           /* wipe end time                          */
    ccsTIMEVAL shiftStart[ccdMAXINTEGR]; /* shift start time                */
    vltDOUBLE  shiftTime;         /* shift time                             */
    ccsTIMEVAL darkStart;         /* start of dark current integration time */
    vltDOUBLE  darkTime;          /* total dark time during integration     */
    ccsTIMEVAL cycleStart;        /* start of cycle integration time        */
    vltDOUBLE  cycleEla;          /* elapsed cycle integration time         */
    ccsTIMEVAL cycleEnd;          /* cycle end time                         */
    vltINT32   stepNum;           /* step num for Multiple exposure         */
    vltDOUBLE  stepDef;           /* defined step integration time          */
    vltDOUBLE  stepDefMulti[ccdMAXINTEGR];
    vltDOUBLE  stepEla;           /* elapsed step integration time          */
    vltDOUBLE  stepElaMulti[ccdMAXINTEGR];
    vltDOUBLE  stepRem;           /* remaining step integration time        */
    ccsTIMEVAL readStart;         /* readout  start time                    */
    vltDOUBLE  readDuration;      /* readout  duration                      */
    ccsTIMEVAL procStart;         /* process  start time                    */
    vltDOUBLE  procDuration;      /* process  duration                      */
    ccsTIMEVAL tranStart;         /* transfer start time                    */
    vltDOUBLE  tranDuration;      /* transfer duration                      */
    ccsTIMEVAL shutOpenStart;     /* shutter  open  start time              */
    vltDOUBLE  shutOpenDuration;  /* shutter  open  duration                */
    ccsTIMEVAL shutCloseStart;    /* shutter  close strat time              */
    vltDOUBLE  shutCloseDuration; /* shutter  close duration                */
    } ccdTIMES;
    
typedef struct
    {
    vltINT32        id;           /* Timer Request Id = address  */
    vltLOGICAL      upd;          /* Timer is being Updated      */
    vltLOGICAL      del;          /* Timer shall be Cancelled    */
    ccsTIMEVAL      utc;          /* UTC when timer is Due       */
    ccsTIMEVAL      per;          /* Interval for periodic timer */
    vltINT32        cnt;          /* Repetition count            */
    vltUINT8        num;          /* Associated timer number 1-4 */
    timerTIMER_MODE tmm;          /* Timer mode                  */
    SEM_ID          sem;          /* Associated semaphore        */
    vltINT32        rid;          /* Timer Request Id            */
    } ccdTIMRQST;

/* Timer handling */
typedef struct
    {
    timsMODE        timsMode;     /* TIM operation mode                  */
    timerSTATE      tmrState[5];  /* State of timers                     */
    ccdTIMRQST      tmrQLst[32];  /* List of timer requests              */
    SEM_ID          semTask;      /* Activation of Timer Task            */
    SEM_ID          semSync;      /* Request Synchronization             */
    SEM_ID          semProt;      /* Protection of Critical Zone         */
    int             tmrTid;       /* Timer Task Id                       */
    int             tmrTcb;       /* Timer Task TCB                      */
    int             tmrStart;     /* ReqId of timer for START at utc,per */
    int             tmrExpose;    /* ReqId of timer for Exposure Time    */
    int             tmrPause;     /* ReqId of timer for PAUSE at utc     */
    int             tmrCont;      /* ReqId of timer for CONT  at utc     */
    int             tmrOpen;      /* ReqId of timer for OPEN  at utc,exp */
    int             tmrClose;     /* ReqId of timer for CLOSE at utc     */
    ccsTIMEVAL      timExpose;    /* exposure time                       */
    ccsTIMEVAL      utcStart;     /* UTC for START                       */
    ccsTIMEVAL      utcPause;     /* UTC for PAUSE                       */
    ccsTIMEVAL      utcCont;      /* UTC for CONT                        */
    } ccdTIMDESC;

/* camera descriptor */
typedef struct
    {
    vltBYTES20   camera;       /* camera name                            */
    vltBYTES8    logMask;      /* FITS log mask                          */
    dbSYMADDRESS dbRoot;       /* camera's database root point           */
    ccdOPMODE    opMode;       /* camera operational mode                */
    ccdSTATE     opState;      /* camera operational state               */
    vltLOGICAL   initDone;     /* camera initialization status           */
    ccdCONFIG   *config;       /* CCD configuration data structure       */
    vltINT32     timeout;      /* CCD system timeout [ms]                */
    vltLOGICAL   wipeOn;       /* Wipe is active                         */
    vltLOGICAL   teleOn;       /* Telemetry   Monitoring is active       */
    vltLOGICAL   tempOn;       /* Temperature Monitoring is active       */
    ccdHWDESC    hwDesc;       /* CCD HW Description                     */
    char        *ccdAttr;      /* table of DB attr direct addresses      */
    ccdSUBSYS    subSys[ccdMAXSUBSYS];   /* Sub-system descriptions      */
    ccdTIMES     times;                  /* CCD exposure related times   */
    ccdTIMDESC   timDesc;                /* Timer handling               */
    ccdTELESTAT  teleStat;               /* Telemetry   statistics       */
    ccdTEMPSTAT  tempStat;               /* Temperature statistics       */
    ccdSETUP     setup[ccdMAXEXP+1];     /* CCD exposure related setup   */
    ccdSETUPRES  setupResult;            /* Setup result (Readout Time)  */
    vltBYTES32   fileNameSimu; /* Simulated image for LCU_SIM RDT        */
    vltINT32     imageIdSimu;  /* Associated image Id when loaded        */
    } ccdDESCRIPTION;

/* image acquisition type */
typedef enum
    {
    ccdFRM_ACQ_NORM = 0,                      /* Detector raw image          */
    ccdFRM_ACQ_FILE,                          /* Image read from WS file     */
    ccdFRM_ACQ_SIMU                           /* Simulated image (LCU/ACE)   */
    } ccdFRM_ACQ_TYPE;

/* image storage type */
typedef enum
    {
    ccdFRM_STO_NORM = ccdFRM_NORMAL,          /* Detector raw image          */
    ccdFRM_STO_BIAS = ccdFRM_BIAS,            /* Bias image                  */
    ccdFRM_STO_DARK = ccdFRM_DARK,            /* Dark image                  */
    ccdFRM_STO_FLAT = ccdFRM_FLAT,            /* FlatField image             */
    ccdFRM_STO_TEMP,                          /* Temporary image (IP result) */
    ccdFRM_STO_SIMU,                          /* Simulation image (LCU Sim)  */
    ccdMAXFRMSTOTYPE
    } ccdFRM_STO_TYPE;

/* image list (map of DB attribute images.frame) */
typedef struct
    {
    vltINT32   imageId;
    vltINT32   startAddr;
    vltINT32   allocSize;
    vltINT32   xSize,ySize;
    vltINT32   expId;
    vltINT32   acqType;          /* see ccdFRM_ACQ_TYPE */
    vltINT32   stoType;          /* see ccdFRM_STO_TYPE */
    vltINT32   dtaType;          /* see ccdIMAGE_REP    */
    } ccdIMAGELIST;

/* image quality for FIS */
#define ccdEVT_IMAGE_OK   0x01
#define ccdEVT_IMAGE_ERR  0x02
#define ccdEVT_IMAGE_LAST 0x04

/* image buffer */
typedef struct ccdImageBuffer
    {
    /* Internal section */
    vltINT32 id;                      /* identification label                */
    vltINT32 winNum;                  /* associated window number            */
    vltINT32 listIndex;               /* index in image list                 */
    vltINT32 imageSize;               /* image size in bytes (data area size)*/
    vltINT32 lineSize;                /* size of one image line in bytes     */
    vltINT32 bytesAvail;              /* data bytes available in buffer      */
    ccdFIS   ipcProc;                 /* process  FIS structure              */
    ccdFIS   ipcTran;                 /* transfer FIS structure              */
    char    *fillUp;                  /* ptr for filling buffer upwards      */
    char    *fillDown;                /* ptr for filling buffer downwards    */
    
    /* FITS parameter section */
    ccdFITSBASIC    fbHeader;         /* FITS basic header                   */
    ccdFITSHIERARCH feHeader;         /* FITS extended header                */
    /* TBD */                         /* FITS trailer                        */
    
    /* Image data section */
    vltUINT8       *data;             /* first data byte                     */
				      /* (buffer dynamically allocated)      */
    struct ccdImageBuffer *next;
    } ccdIMAGEBUFFER;

/* NFS FITS IMAGE descriptor */
typedef struct
    {
    vltBYTES256     file;             /* file name (absolute)                */
    int             df;               /* file descriptor                     */
    ccdIMAGEBUFFER *image;            /* Image description including FITS    */
    ccdIMAGELIST   *entry;            /* Entry in DB image list              */
    } ccdNFSFITSIMAGE;

/*
 ****************************************************************************
 * Function Prototypes
 ****************************************************************************
 */ 
/* CCD version function */
void          ccdVersion          ( void );
void          ccdCompiledDate     ( const char *date, const char *time, 
				    char *comp );
void          ccdModuleVersion    ( const char *rev, const char *mod,
				    const char *txt, const char *cmp,
				    char *version );
void          ccdModuleVersionDCL ( const char *prefix );

/* CCD startup function */
ccsCOMPL_STAT ccdconStartupCamera ( char *parStr );

/* CCD tool functions */
STATUS        ccdKill           ( char *taskNamePart );
void          ccdShowSetup      ( ccdSETUP *setup );
void          ccdPrintConfig    ( const char *camera , const ccdCONFIG *config );

/* ACE tool functions */
ccsCOMPL_STAT ccdSwitchAce      ( char mode );
ccsCOMPL_STAT ccdTestAce        ( void );
ccsCOMPL_STAT ccdCheckAceDriver ( char *channel, ccsERROR *error );
ccsCOMPL_STAT ccdCheckLcuAceCom ( char *channel, ccsERROR *error );

/* CCD internal functions */
ccsCOMPL_STAT ccdCheckId        ( vltINT32 id );
ccsCOMPL_STAT ccdSetOpState     ( ccdDESCRIPTION *ccdDesc, vltINT32 opState,
                                  ccsERROR *error );

/* CCD installation/startup functions */
ccsCOMPL_STAT ccdInstall        ( char *parStr, ccdDESCRIPTION **pccdDesc,
				  ccsERROR *error);
ccsCOMPL_STAT ccdDeinstall      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error);

ccsCOMPL_STAT ccdStartSystem    ( ccdDESCRIPTION *ccdDesc, char *module,
                                  ccsERROR *error );
ccsCOMPL_STAT ccdStartServer    ( ccdDESCRIPTION *ccdDesc, int module,
                                  vltINT32 priority, ccsERROR *error );
void          ccdParseRunString ( char *runStr, char *ccdName, 
				  char *insHost, char *insRoot, char *insUser ); 
ccsCOMPL_STAT ccdShutdown       ( ccdDESCRIPTION *ccdDesc, char *module,
                                  ccsERROR *error );

/* DB Symbolic access functions */
ccsCOMPL_STAT ccdReadAttr       ( char *ccdName, const dbSYMADDRESS attrName, 
				  void *buffer, ccsERROR *error );
ccsCOMPL_STAT ccdWriteAttr      ( char *ccdName, const dbSYMADDRESS attrName, 
                                  void *buffer, ccsERROR *error );
ccsCOMPL_STAT ccdInitDb         ( vltBYTES20 camera, vltBYTES20 lcu,
                                  ccsERROR *error );

/* MEM allocation functions */
ccsCOMPL_STAT ccdCreaImBuf      ( ccdDESCRIPTION *ccdDesc, ccdSETUP *setup,  
				  vltINT32 expId, vltINT32 dataType,
				  vltINT32 storageType, vltINT32 *imageId, 
				  ccsERROR *error);
ccsCOMPL_STAT ccdDumpImBuf      ( ccdDESCRIPTION  *ccdDesc, vltINT32 imageId,
				  char *fileName, ccsERROR *error );
ccsCOMPL_STAT ccdFindImBuf      ( ccdDESCRIPTION  *ccdDesc, vltINT32 expId, 
				  vltINT32 dataType, vltINT32 storageType, 
				  vltINT32 *imageId, ccsERROR *error );
ccsCOMPL_STAT ccdFreeImBuf      ( ccdDESCRIPTION *ccdDesc, vltINT32 imageId, 
				  vltLOGICAL force, ccsERROR *error );
ccsCOMPL_STAT ccdListImBuf      ( ccdDESCRIPTION  *ccdDesc, vltINT32 dataType, 
				  vltINT32 storageType, ccdIMAGELIST **imgList,
				  ccsERROR *error );
ccsCOMPL_STAT ccdLoadImBuf      ( ccdDESCRIPTION  *ccdDesc, char *fileName, 
				  vltINT32 storageType, vltINT32 *imageId, 
				  ccsERROR *error );

/* NFS FITS functions */
ccsCOMPL_STAT ccdOpenFits        ( ccdNFSFITSIMAGE *desc, ccsERROR *error );
ccsCOMPL_STAT ccdCreaFits        ( ccdNFSFITSIMAGE *desc, ccsERROR *error );

ccsCOMPL_STAT ccdReadFitsHeader  ( ccdNFSFITSIMAGE *desc, ccsERROR *error );
ccsCOMPL_STAT ccdReadFitsData    ( ccdNFSFITSIMAGE *desc, ccsERROR *error );
ccsCOMPL_STAT ccdReadFitsTrailer ( ccdNFSFITSIMAGE *desc, ccsERROR *error );

ccsCOMPL_STAT ccdWriteFitsHeader ( ccdNFSFITSIMAGE *desc, ccsERROR *error );
ccsCOMPL_STAT ccdWriteFitsData   ( ccdNFSFITSIMAGE *desc, ccsERROR *error );
ccsCOMPL_STAT ccdWriteFitsTrailer( ccdNFSFITSIMAGE *desc, ccsERROR *error );

ccsCOMPL_STAT ccdWriteFitsDataBlock ( ccdNFSFITSIMAGE *desc, void *pData, 
				      int len, ccsERROR *error );

ccsCOMPL_STAT ccdCloseFits       ( ccdNFSFITSIMAGE *desc, ccsERROR *error );

/* ERR handling functions */
ccsCOMPL_STAT ccdSetFailure     ( ccdDESCRIPTION  *ccdDesc,  ccsERROR *error );
ccsCOMPL_STAT ccdResetFailure   ( ccdDESCRIPTION  *ccdDesc,  ccsERROR *error );

/* ENV handling functions */
ccsCOMPL_STAT ccdEnvVarChk      ( char            *camera,   ccsERROR *error );
ccsCOMPL_STAT ccdEnvVarDel      ( ccdDESCRIPTION  *ccdDesc,  ccsERROR *error );
ccsCOMPL_STAT ccdEnvVarGet      ( ccdDESCRIPTION **pccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdEnvVarSet      ( ccdDESCRIPTION  *ccdDesc,  ccsERROR *error );
ccsCOMPL_STAT ccdGetDescriptor  ( const char      *camera, 
				  ccdDESCRIPTION **pCcdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdCheckDescriptor( ccdDESCRIPTION  *ccdDesc,  ccsERROR *error );

/* FIS handling functions */
ccsCOMPL_STAT ccdCreaFis        ( ccdFIS *fis, vltINT32 timeout,
				  ccsERROR *error );
ccsCOMPL_STAT ccdCheckFisCmd    ( ccdFIS *fis, vltINT32 cmd,
				  vltLOGICAL wait, vltLOGICAL *got, 
				  ccsERROR *error );
ccsCOMPL_STAT ccdFreeFis        ( ccdFIS *fis, ccsERROR *error);
ccsCOMPL_STAT ccdResetFis       ( ccdFIS *fis, ccsERROR *error );
ccsCOMPL_STAT ccdSendFisCmd     ( ccdFIS *fis, vltINT32 cmd,
				  vltLOGICAL wait, ccsERROR *error );
ccsCOMPL_STAT ccdSendFisReply   ( ccdFIS *fis, vltINT32 reply,
				  ccsERROR *error );
ccsCOMPL_STAT ccdSetFisTimeout  ( ccdFIS *fis, vltINT32 timeout,
				  ccsERROR *error );
ccsCOMPL_STAT ccdWaitFisReply   ( ccdFIS *fis, ccsERROR *error );

/* TIM handling function */
ccsCOMPL_STAT ccdStoreTime      ( char *attrTable, int attrIndex, 
				  ccsTIMEVAL *startTime, vltDOUBLE *duration,
				  ccsERROR *error );

/* MSG handling functions */
ccsCOMPL_STAT ccdSendCmdAck     ( msgMSG *msg, ccsERROR *error );
ccsCOMPL_STAT ccdSendCmdAll     ( const msgCMD command, const char *params,
                                  ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdSendCmd        ( ccsPROCNAME dest, const msgCMD command,
                                  const char *params, char *reply, 
				  ccdDESCRIPTION *ccdDesc, vltINT32 waitMode,
                                  ccsERROR *error );
ccsCOMPL_STAT ccdWaitReply      ( ccsPROCNAME dest, vltINT32 timeout,
				  char *reply, vltLOGICAL *lastReply, 
				  vltLOGICAL *errReply, ccsERROR *error );

/* Build next/prev exposure ID from the current exposure Index and ID */
vltINT32 ccdGetNextExpId  ( vltINT32 expId , vltINT32 expIndex );
vltINT32 ccdGetPrevExpId  ( vltINT32 expId , vltINT32 expIndex );

/* TIMER handling functions */
ccsCOMPL_STAT ccdTimerSleep     ( vltINT32 us, ccsERROR *error ); 
ccsCOMPL_STAT ccdStartTimerTask ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdStopTimerTask  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdTimerReqActive ( ccdDESCRIPTION *ccdDesc, 
				  vltINT32 rid, ccsERROR *error );
ccsCOMPL_STAT ccdTimerReqCreate ( ccdDESCRIPTION *ccdDesc, ccsTIMEVAL *utc, 
				  ccsTIMEVAL *per, vltINT32 cnt, 
				  vltINT32 *rid, ccsERROR *error );
ccsCOMPL_STAT ccdTimerReqDelete ( ccdDESCRIPTION *ccdDesc, 
				  vltINT32 rid, ccsERROR *error );
ccsCOMPL_STAT ccdTimerReqUpdate ( ccdDESCRIPTION *ccdDesc, vltINT32 rid, 
				  ccsTIMEVAL *utc, ccsTIMEVAL *per,
				  vltINT32 cnt, ccsERROR *error );
ccsCOMPL_STAT ccdWaitTimerDue   ( ccdDESCRIPTION *ccdDesc, vltINT32 rid,
				  ccsERROR *error );
ccsCOMPL_STAT ccdTimerReqRelease( ccdDESCRIPTION *ccdDesc, 
				  vltINT32 rid, ccsERROR *error );
void ccdPrintTimer ( ccdDESCRIPTION *ccdDesc );

/* ISO8601 Time decoding function */
ccsCOMPL_STAT ccdDecodeIsoTime  ( char *str, ccsTIMEVAL *utc, ccsERROR *err );

void ccdDbAlign ( const char *type, vltLOGICAL pack, void *s, char *b );

#define ccdDBPACK(t,s,b) ccdDbAlign(t,ccsTRUE, s,b)
#define ccdUNPACK(t,b,s) ccdDbAlign(t,ccsFALSE,s,b)

/*
 * Error handling macros
 */
#ifndef __FILE_LINE__
#define _ccd_tostr(a) #a
#define _ccd_tostr_pass2(a) _ccd_tostr(a)

#define __FILE_LINE__  __FILE__ ":" _ccd_tostr_pass2(__LINE__)
#endif

#define ccdERR_ADD(errNum,params...) \
    errAdd(error,ccdMODULE,errNum,__FILE_LINE__,"",##params)       
#define ccdERR_RETURN(errNum,params...) \
    { \
    errAdd(error,ccdMODULE,errNum,__FILE_LINE__,"",##params); \
    return FAILURE; \
    }                                                             
#define ccdERR_ADD_AND_CLOSE_STACK(errNum,params...) \
    { errAdd(error,ccdMODULE,errNum,__FILE_LINE__,"",##params); \
      errCloseStack(error); }      

#ifdef __cplusplus
}
#endif

#endif  /* ! CCD_LCU_INTERNAL_H */

/************************************************************************/

/*___o0o___*/
