/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccdip.h,v 1.68 2005/06/28 09:04:02 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* ccumani   15/11/95  created
* P.Duhoux  01/02/96  added function prototypes
* P.Duhoux  11/11/96  added user functions ccdipAG and ccdipIQE
*                     and data structures ccdipAGBUF & ccdipIQEBUF
*                     and associated macros
* P.Duhoux  14/01/97  Defined ccdipVERSION_DATE = ccdVERSION_DATE
* P.Duhoux  15/01/97  added function prototypes ccdipOff()
*/

/*
 ***********************************************************************
 * ccdip.h - Server Image Processing Interface File. 
 *************************************************************************
 */
#ifndef CCDIP_H
#define CCDIP_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * CCD include files 
 */
#include "ccdLcuInternal.h"

/*
 * CCDIP include files
 */
#include "ccdipDefines.h"

/* module name */
#define ccdipMODULE       "ccdip"
#define ccdipMODULE_NAME  "Image Processing Module"
 
/* module version */
#define ccdipVERSION      "$Revision: 1.68 $"
#define ccdipVERSION_DATE ccdVERSION_DATE

/* processing status */
typedef enum
    {
    ccdipOFF         = ccdOFF,
    ccdipLOADED      = ccdLOADED,
    ccdipSTANDBY     = ccdSTANDBY,
    ccdipONLINE_IDLE = ccdONLINE,     /* on-line but nothing to do */
    ccdipONLINE_READY,                /* processing ready          */
    ccdipONLINE_ACTIVE                /* processing active         */
    } ccdipSTATE;

/* processing status */
typedef enum
    {
    ccdipNONE = -1,
    ccdipSUCCESS,                  /* last process completed successfully */
    ccdipFAILURE,                  /* last process stopped with error */
    ccdipABORTED                   /* last process aborted by command */
    } ccdipSTATUS;

/*
 ******************************************************************************
 * CI Function Prototypes
 ******************************************************************************
 */
ccsCOMPL_STAT ccdipMainTask  ( ccdDESCRIPTION *ccdDesc, ccdFIS *fis );

/*
 ******************************************************************************
 * Signal Handlers
 ******************************************************************************
 */
void ccdipBreakHandler ( int signal );
void ccdipKillHandler  ( int signal );

/*
 ******************************************************************************
 * API Function Prototypes
 ******************************************************************************
 */
/* Standard Commands for all Sub-Systems */
ccsCOMPL_STAT ccdipExit     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipInit     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipOff      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipOnLine   ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipReset    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipStandBy  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdipStop     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
void          ccdipVersion  ( char *version );

/* Specific Commands for IP Sub-System */
ccsCOMPL_STAT ccdipPrepare  ( ccdDESCRIPTION *ccdDesc, char *parbuf,
			      ccsERROR *error );
ccsCOMPL_STAT ccdipProcImage( ccdDESCRIPTION *ccdDesc, msgMSG *message,
			      char *parbuf, ccsERROR *error );

/* Template for User-Defined Image Processing Function */
ccsCOMPL_STAT ccdipUsrFuncTemplate ( ccdIPIMAGE *image,
				     void *usrBuffer, ccsERROR *error );

/* Pointer to User-Defined Image Pre-Processing Function */
extern ccdIPUSERFUNC ccdipPreProcessImage;

/*
 ******************************************************************************
 * USR Function Prototypes + Data Structures
 ******************************************************************************
 */
#include "ccdipAG.h"
#include "ccdipCV.h"
#include "ccdipIQE.h"
#include "ccdipWCE.h"

#ifdef __cplusplus
}
#endif

#endif /*!CCDIP_H*/
