/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ccdipWCE.h,v 1.68 2005/06/28 09:04:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* pduhoux  27/05/98  created
*/

/*
 ***********************************************************************
 * Wavefront Coherence Estimation : data structure & function prototype
 *----------------------------------------------------------------------
 */

#ifndef CCDIP_WCE_H
#define CCDIP_WCE_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
    {
    /*
     * This structure contains the results of the coherence mode centroid
     * computation for one stripe:
     */
    vltDOUBLE    flux;                /* Raw Average ADC counts / pixel      */
    vltDOUBLE    sigma;               /* Standard-Deviation on the flux      */
    vltDOUBLE    BckGnd,BckGnd_SD;    /* Back Ground estimation + Std Dev    */
    vltDOUBLE    xFWHM,yFWHM;         /* X & Y FWHM on object                */
    vltDOUBLE    SNR;                 /* Signal-to-Noise Ratio               */
    vltINT32     numPix;              /* Number of valid pixels in image     */
    vltDOUBLE    cenVal,xCen,yCen;    /* Location of Centroid relative       */
	                              /* to stripe LL corner                 */
    } ccdipSTRIPE;

typedef struct
    {
    /*
     * This structure contains the results of the coherence mode centroid
     * computation:
     *   - numStripe : number of stripes 
     *   - lenStripe : height of stripes 
     *   - expTimeDef: expTime / stripe
     *   - expTime   : effective expTime / stripe
     *   - numValPix : number of valid pixels in each stripe 
     *                 (pix - bckGnd) > thrMin
     *                 When more than 1 spot, forced to -1
     *   - xCen,yCen : coordinates of spot centroid relative to LL corner
     *                 of the stripe.
     */
    ccsTIMEVAL   utcStart;                 /* UTC of first stripe            */
    vltINT32     numStripe;                /* Number of stripes              */
    vltINT32     lenStripe;                /* Height of stripes              */
    vltDOUBLE    lapTime;                  /* Effective Time between stripe  */
    vltDOUBLE    expTime;                  /* Effective expTime/stripe       */
    ccdipSTRIPE  stripe[2][ccdMAXINTEGR];  /* Centroid / stripe / window     */
    } ccdipWCEBUF;

extern ccdipWCEBUF ccdipWceBuf;

ccsCOMPL_STAT ccdipWCE  (ccdIPIMAGE *img, ccdipWCEBUF *iqeb, ccsERROR *err);

#ifdef __cplusplus
}
#endif

#endif /*!CCDIP_WCE_H*/
