/****************************************************************************
* E.S.O. - VLT Project
*
* "@(#) $Id: ccdrdt.h,v 1.72 2005/06/21 07:57:42 vltsccm Exp $"
*
* who            when     what
* -------------- -------- ---------------------------------------------------
* N. Fiebig      08/06/94 - first implementation
* C. Cumani      22/06/94 - all the elements of the ccdrdtWIPESETUP
*                           structure: vltUINT8 -> vltINT32
* C. Cumani      27/06/94 - ccdrdtSETUP.mppMode: vltLOGICAL -> vltINT32
*                         - ccdrdtSETUP.speedIndex: vltINT16 -> vltINT32
*                         - ccdrdtSETUP.gainIndex: vltINT16 -> vltINT32
* C. Cumani      28/06/94 - ccdrdtSPEED.gain: vltUINT16 -> vltINT32
*                         - ccdrdtSPEED.adcTime: vltUINT8 -> vltINT32
*                         - ccdrdtSPEED.clockIndex: vltUINT8 -> vltINT32
*                         - ccdrdtCLOCK.firstOutput: vltUINT16 -> vltINT32
*                         - ccdrdtSETUP.numOutputs: vltUINT16 -> vltINT32
*                         - ccdrdtSETUP.outputEn[ccdMAXOUTPUT]: 
*                                                       vltUINT16 -> vltINT32
*                         - ccdrdtSETUP.readPixelN: vltUINT16 -> vltINT32
* C. Cumani      04/07/95 - ccdrdtCLOCK structure modified to follow new
*                           database
*                         - ccdrdtSPEED structure modified to follow new
*                           database
* C. Cumani      20/07/95 - ccdrdtReadSetup() function prototype modified
* C. Cumani      21/07/95 - ccdrdtHandlePrepare() function prototype modified
* C. Cumani      31/07/95 - ccdrdtSPEED structure deleted
*                         - ccdrdtOUTPUT_INFO structure deleted
*                         - ccdrdtCLOCK structure modified
*                         - ccdrdtSETUP structure modified
* C. Cumani      01/08/95 - cccdrdtEXP_PTS definition added
*                         - attrTables added in ccdrdtGLOBALS
*                         - ccdrdtCLCK_PTS definition added
*                         - clockAttrTables added in ccdrdtGLOBALS
*                         - ccdrdtOUTPT_PTS definition added
*                         - outptAttrTables added in ccdrdtGLOBALS
*                         - ccdrdtOUTPUT structure added
* C. Cumani      22/08/95 - ccdrdtMODSTATUS -> ccdrdtSTATE
*                         - ccdrdtINITIALISED added in ccdrdtSTATE
* C. Cumani      24/08/95 - ccdrdtSETUP.window: ccdconWINDOW -> ccdWINDOW
* C. Cumani      19/09/95 - ccdrdtHandleOff() func. prot. added
*                         - ccdrdtHandleOnline() func. prot. added
*                         - ccdrdtHandleStandby() func. prot. added
* C. Cumani      19/10/95 - ccdrdtMODULE, ccdrdtMODULE_NAME, ccdrdtVERSION, 
*                           and ccdrdtVERSION_DATE introduced
*                         - ccdrdtVersion() func. prot. added
* P.Duhoux       09/11/95 - ccdrdtHandleInit() function prototype modified 
*                           expId parameter added to INIT
*
*************************************************************************
* P.Duhoux         JAN/96 Redesign similar for CCD subsystem modules
*                         New CCD include scheme 
*                         New ACI/API function interfaces
*                         New signal handler interfaces for ccsInit
* P.Duhoux       08/02/96 Added ccdrdtStop function Prototype
* P.Duhoux       16/07/96 Renamed ccdrdtStart/StopWipe
* A.Longinotti   24/07/96 ccdrdtSTATE moved to ccdInternal.h
* P.Duhoux       14/01/97 Defined ccdrdtVERSION_DATE = ccdVERSION_DATE
*/
/************************************************************************
*  ccdrdt.h - Server Read-Out Control Interface File. 
*************************************************************************
*/
#ifndef CCDRDT_H
#define CCDRDT_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCD include file
 */
#include "ccdLcuInternal.h"

/*
 * CCDRDT include files
 */
#include "ccdrdtDefines.h"

/* CCDRDT module name */
#define ccdrdtMODULE       "ccdrdt"
#define ccdrdtMODULE_NAME  "Readout Control Module"
 
/* CCDRDT module version */
#define ccdrdtVERSION      "$Revision: 1.72 $"     
#define ccdrdtVERSION_DATE ccdVERSION_DATE

/* readout completion status */
typedef enum
    {
    ccdrdtNONE = 1,                 /* no readout completed yet */
    ccdrdtSUCCESS,                  /* last readout completed successfully */
    ccdrdtFAILURE,                  /* last readout stopped with error */
    ccdrdtABORTED                   /* last readout aborted by command */
    } ccdrdtSTATUS;

/*
 ******************************************************************************
 * CI Function Prototypes
 ******************************************************************************
 */
ccsCOMPL_STAT ccdrdtMainTask  ( ccdDESCRIPTION *ccdDesc, ccdFIS *fis );

/*
 ******************************************************************************
 * Signal Handlers
 ******************************************************************************
 */
void ccdrdtBreakHandler ( int signal );
void ccdrdtKillHandler  ( int signal );

/*
 ******************************************************************************
 * API Function Prototypes
 ******************************************************************************
 */
/* Standard Commands for all Sub-Systems */
ccsCOMPL_STAT ccdrdtAbort     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtExit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtInit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtOff       ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtOnLine    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtReset     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtStandBy   ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtStatus    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdrdtStop      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
void          ccdrdtVersion   ( char *version );

/* Specific Commands for RDT Sub-System */
ccsCOMPL_STAT ccdrdtPrepare   ( ccdDESCRIPTION *ccdDesc, vltINT32 expId,
				ccsERROR *error );
ccsCOMPL_STAT ccdrdtReadImage ( ccdDESCRIPTION *ccdDesc, msgMSG *message,
				vltINT32 imageId, ccdFIS *coordFis, 
				vltINT32 numRep, ccsERROR *error );
ccsCOMPL_STAT ccdrdtShftRow   ( ccdDESCRIPTION *ccdDesc, vltINT32 numRow,
				ccsERROR *error );
ccsCOMPL_STAT ccdrdtStartWipe ( ccdDESCRIPTION *ccdDesc, vltINT32 periodic,
				ccsERROR *error );
ccsCOMPL_STAT ccdrdtStopWipe  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );

#ifdef __cplusplus
}
#endif

#endif  /* ! CCDRDT_H */

/************************************************************************/

/*___o0o___*/
