/************************************************************************
* E.S.O. - VLT project							
*									
* "@(#) $Id: ccdrdtPublic.h,v 1.72 2005/06/21 07:57:43 vltsccm Exp $"
*
* who           when      what
* ------------  --------  -----------------------------------------------
* P.Duhoux      11/12/95  First version
*
*************************************************************************
* P.Duhoux         JAN/96 Redesign similar for CCD subsystem modules
*                         New CCD include scheme 
*                         New CCDRDT subsystem descriptor
* P.Duhoux       17/07/96 Added desc field cmdId for cmd identification
* P.Duhoux       30/01/97 Added field clockSpeed, elAdu  & firstOutput in ccdrdtCLOCK
*                         Added fields gainFactor in ccdrdtREADSETUP
* P.Duhoux       22/04/97 Added <ipc> for task spawn sync in descriptor
* P.Duhoux       27/06/97 Added fields linesToShiftMode/Multi[] in setup
*/
/************************************************************************
*  ccdrdtPublic.h - This file contains all public definitions used 
*                   by the CCD DCL read-out control modules
*************************************************************************
*/
#ifndef CCDRDT_PUBLIC_H
#define CCDRDT_PUBLIC_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCDRDT include file
 */
#include "ccdrdt.h"

/* 
 * name of read-out points in database
 */
#define ccdrdtDB_RDT_POINT      "readout" 
#define ccdrdtDB_CLK_POINT      "readout:clock_%1d" 
#define ccdrdtDB_EXP_POINT      "exposures:exposure_%1d:readout" 
#define ccdrdtDB_OUT_POINT      "detector:output_%1d" 

/*
 * definition of index for database attribute direct access
 */
typedef enum
    {
    /* DCL common part */
    ccdrdtDB_PREFIX = 0,
    ccdrdtDB_OPSTATE,
    ccdrdtDB_SUBSTATE,
    ccdrdtDB_STATUS,
    ccdrdtDB_INITDONE,
    /* read-out specific part */
    ccdrdtDB_LAST,
    ccdrdtDB_STARTED,
    ccdrdtDB_TIME,
    ccdrdtDB_PERCENT,
    ccdrdtDB_BITSPIXEL,
    ccdrdtDB_GAINFACT,
    ccdrdtDB_NWIPES,
    ccdrdtDB_WIPEPERIOD,
    ccdrdtDB_WIPEXBIN,
    ccdrdtDB_WIPEYBIN,
    ccdrdtDB_WIPESTARTED,
    ccdrdtDB_WIPETIME,
    ccdrdtNUM_ATTR
    } ccdrdtDB_ATTR;

typedef enum
    {
    ccdrdtDB_CLK_OUTENAB = 0,
    ccdrdtDB_CLK_SPEED,
    ccdrdtDB_CLK_FIRSTOUT,
    ccdrdtDB_CLK_ADCTIME,
    ccdrdtDB_CLK_ELADU,
    ccdrdtDB_CLK_NAME
    } ccdrdtDB_CLK_ATTR;

typedef enum
    {
    ccdrdtDB_STAT_SPEED = 0,
    ccdrdtDB_STAT_GAINFACT,
    ccdrdtDB_STAT_ADCTIME,
    ccdrdtDB_STAT_ELADU,
    ccdrdtDB_STAT_OUTENAB,
    ccdrdtDB_SETUP
    } ccdrdtDB_STAT_ATTR;

/* set-up and configuration structures */
typedef struct
    {
    vltINT32 count;       /* wipe count */
    vltINT32 period;      /* wipe periode in seconds */
    vltINT32 xbin;        /* binning in x direction */
    vltINT32 ybin;        /* binning in y direction */
    } ccdrdtWIPESETUP;
    
typedef struct
    {
    vltBYTES32     clockName;
    vltBYTES16     clockSpeed;
    vltINT32       firstOutput;
    vltINT32       adcTime;
    vltLOGICAL     outEnab[ccdMAXOUTPUT];
    vltINT32       numOutputs;
    } ccdrdtCLOCK;
    
typedef struct
    {
    vltINT32       xPix;
    vltINT32       yPix;
    vltINT32       xDirection;
    vltINT32       yDirection;
    } ccdrdtOUTPUT;
    
typedef struct
    {
    vltINT32       clockIndex;
    ccdrdtCLOCK    clock;
    vltINT32       gainIndex;
    vltDOUBLE      gainFactor;
    vltDOUBLE      elAdu[ccdMAXOUTPUT];
    vltINT32       linesToShift;
    vltINT32       linesToShiftMulti[ccdMAXINTEGR];
    vltINT32       linesToShiftType;
    vltLOGICAL     mpp;
    vltINT32       readPixelN;
    ccdrdtOUTPUT   output[ccdMAXOUTPUT];
    vltINT32       numWin;
    ccdWINDOW      window[ccdMAXWINDOW];
    } ccdrdtREADSETUP;
    
/* global data structure for Read-Out sub-system */
typedef vltINT32 ccdrdtSUBSTATE;
typedef struct
    {
        /* general section */
    ccdDESCRIPTION *ccdDesc;   /* ptr to camera descriptor                   */
    vltLOGICAL      running;    /* flag if main task running                 */
    ccdrdtSTATE     opState;    /* operational state                         */
    ccdrdtSUBSTATE  subState;   /* sub-operational state                     */
    ccdrdtSTATUS    status;     /* read-out status                           */
    vltLOGICAL      initDone;   /* sub-system initialized flag               */
        /* image control section */
    vltINT32        expId;      /* exposure id                               */
    vltINT32        imageId;    /* ID of image to transfer                   */
    ccdFIS         *coordFis;   /* FIS ExpCoord -> ReadOut                   */
    vltINT32        numRep;     /* num of repeated exposures                 */
        /* process control section */
    msgPROCESSID    ownPid;     /* Pid of Read-Out task                      */
    msgPROCESSID    coordPid;   /* Pid of Exposure Coordination task         */
    msgCMDID        cmdId;      /* Command Id                                */
        /* setup section */
    ccdrdtREADSETUP readSetup;  /* readout setup                             */
    ccdrdtWIPESETUP wipeSetup;  /* wipe    setup                             */
        /* database access section */
    char           *rdtAttr;    /* table of RDT attr direct addr             */
    char           *expAttr[ccdMAXEXP];
    char           *clkAttr[ccdMAXCLOCKS];
    char           *outAttr[ccdMAXOUTPUT];
        /* IPC synchronization */
    ccdFIS          ipc;        /* IPC FIS data structure                    */
    ccdFIS          ipcRd;      /* Read task IPC FIS data structure          */
    ccdFIS          ipcWp;      /* Wipe task IPC FIS data structure          */
        /* Readout/Wipe Task control section */
    int             tcbRd;      /* Read task TCB                             */
    int             tcbWp;      /* Wipe task TCB                             */
        /* DCL section*/
    vltINT32        clkIndex;   /* DSP clock index                           */
    vltBYTES16      dspFile;    /* DSP file name                             */
    vltINT16        rdxStatus;  /* DCL status information                    */
    char           *rdxFunc;    /* Table of RDX functions addr               */
    char           *rdxDesc;    /* RDX Descriptor                            */
    } ccdrdtDESCRIPTION;

#ifdef __cplusplus
}
#endif

#endif  /* ! CCDRDT_PUBLIC_H */

/************************************************************************/

/*___o0o___*/
