/************************************************************************
* E.S.O. - VLT project							
*									
* "@(#) $Id: ccdtel.h,v 1.40 2005/06/21 08:03:18 vltsccm Exp $"
*
* who           when      what
* ------------  --------  -----------------------------------------------
* P.Duhoux      15/11/95  First version
*
*************************************************************************
* P.Duhoux         JAN/96 Redesign similar for CCD subsystem modules
*                         New CCD include scheme 
*                         New ACI/API function interfaces
*                         New signal handler interfaces for ccsInit
* P.Duhoux       17/07/96 Renamed ccdtelGetStatus into ccdtelStatus 
* P.Duhoux       14/01/97 Defined ccdtelVERSION_DATE = ccdVERSION_DATE
* P.Duhoux       12/03/97 Added TEL specific function prototypes
* P.Duhoux       02/04/97 Added ccdtelPause/ccdtelContinue fct prototypes
*/
/************************************************************************
*  ccdtel.h - Server Telemetry Control Interface File. 
*************************************************************************
*/
#ifndef CCDTEL_H
#define CCDTEL_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCD include file
 */
#include "ccdLcuInternal.h"

/*
 * CCDTEL include files
 */
#include "ccdtelDefines.h"

/* CCDTEL module name */
#define ccdtelMODULE       "ccdtel"
#define ccdtelMODULE_NAME  "Telemetry Control Module"
 
/* CCDTEL module version */
#define ccdtelVERSION      "$Revision: 1.40 $"
#define ccdtelVERSION_DATE ccdVERSION_DATE

/*
 ******************************************************************************
 * Literals
 ******************************************************************************
 */
/*
 * telemetry state definition
 */
typedef enum
    {
    ccdtelOFF         = ccdOFF,
    ccdtelLOADED      = ccdLOADED,
    ccdtelSTANDBY     = ccdSTANDBY,
    ccdtelONLINE_IDLE = ccdONLINE,
    ccdtelONLINE_ACTIVE,
    } ccdtelSTATE;

/*
 * telemetry status definition
 */
typedef enum
    {
    ccdtelNONE = 0,                 /* no telemetry cycle completed yet      */
    ccdtelACTIVE,                   /* telemetry cycle in progress           */
    ccdtelSUCCESS,                  /* last telemetry completed successfully */
    ccdtelFAILURE,                  /* last telemetry stopped with error     */
    ccdtelABORTED                   /* last telemetry aborted by command     */
    } ccdtelSTATUS;

/*
 ******************************************************************************
 * CI Function Prototypes
 ******************************************************************************
 */
ccsCOMPL_STAT ccdtelStartServer  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );

/*
 ******************************************************************************
 * Signal Handlers
 ******************************************************************************
 */
void ccdtelBreakHandler ( int signal );
void ccdtelKillHandler  ( int signal );

/*
 ******************************************************************************
 * API Function Prototypes
 ******************************************************************************
 */
/* Standard Commands for all Sub-Systems */
ccsCOMPL_STAT ccdtelExit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelInit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelOff       ( ccdDESCRIPTION *ccdDesc, ccsERROR *error);
ccsCOMPL_STAT ccdtelOnLine    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error);
ccsCOMPL_STAT ccdtelReset     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelStandBy   ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelStatus    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
void          ccdtelVersion   ( char *version );

/* Specific Commands for TEL Sub-System */
ccsCOMPL_STAT ccdtelContinue  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelGetParam  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelPause     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelSetParam  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtelMonitor   ( ccdDESCRIPTION *ccdDesc, vltINT32 period, 
				ccsERROR *error );
ccsCOMPL_STAT ccdtelStop      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );

#ifdef __cplusplus
}
#endif

#endif  /* ! CCDTEL_H */

/************************************************************************/

/*___o0o___*/


