/************************************************************************
* E.S.O. - VLT project							
*									
* "@(#) $Id: ccdtmp.h,v 1.43 2005/06/21 08:07:54 vltsccm Exp $"
*
* who           when      what
* ------------  --------  -----------------------------------------------
* P.Duhoux      15/11/95  First version
*
*************************************************************************
* P.Duhoux         JAN/96 Redesign similar for CCD subsystem modules
*                         New CCD include scheme 
*                         New ACI/API function interfaces
*                         New signal handler interfaces for ccsInit
* P.Duhoux       17/07/96 Renamed ccdtmpGetStatus into ccdtmpStatus 
* P.Duhoux       14/01/97 Defined ccdtmpVERSION_DATE = ccdVERSION_DATE
* P.Duhoux       12/03/97 Added TMP specific function prototypes
* P.Duhoux       02/04/97 Added ccdtmpPause/ccdtmpContinue fct prototypes
*/
/************************************************************************
*  ccdtmp.h - Server Temperature Control Interface File. 
*************************************************************************
*/
#ifndef CCDTMP_H
#define CCDTMP_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * CCD include file
 */
#include "ccdLcuInternal.h"

/*
 * CCDTMP include files
 */
#include "ccdtmpDefines.h"

/* CCDTMP module name */
#define ccdtmpMODULE       "ccdtmp"
#define ccdtmpMODULE_NAME  "Temperature Control Module"
 
/* CCDTMP module version */
#define ccdtmpVERSION      "$Revision: 1.43 $"
#define ccdtmpVERSION_DATE ccdVERSION_DATE

/*
 ******************************************************************************
 * Literals
 ******************************************************************************
 */
/*
 * temperature state definition
 */
typedef enum
    {
    ccdtmpOFF         = ccdOFF,
    ccdtmpLOADED      = ccdLOADED,
    ccdtmpSTANDBY     = ccdSTANDBY,
    ccdtmpONLINE_IDLE = ccdONLINE,
    ccdtmpONLINE_ACTIVE,
    } ccdtmpSTATE;

/*
 * temperature status definition
 */
typedef enum
    {
    ccdtmpNONE = 0,               /* no temperature cycle completed yet      */
    ccdtmpACTIVE,                 /* temperature cycle in progress           */
    ccdtmpSUCCESS,                /* last temperature completed successfully */
    ccdtmpFAILURE,                /* last temperature stopped with error     */
    ccdtmpABORTED                 /* last temperature aborted by command     */
    } ccdtmpSTATUS;

/*
 ******************************************************************************
 * CI Function Prototypes
 ******************************************************************************
 */
ccsCOMPL_STAT ccdtmpStartServer  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );

/*
 ******************************************************************************
 * Signal Handlers
 ******************************************************************************
 */
void ccdtmpBreakHandler ( int signal );
void ccdtmpKillHandler  ( int signal );

/*
 ******************************************************************************
 * API Function Prototypes
 ******************************************************************************
 */
/* Standard Commands for all Sub-Systems */
ccsCOMPL_STAT ccdtmpExit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpInit      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpOff       ( ccdDESCRIPTION *ccdDesc, ccsERROR *error);
ccsCOMPL_STAT ccdtmpOnLine    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error);
ccsCOMPL_STAT ccdtmpReset     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpStandBy   ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpStatus    ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
void          ccdtmpVersion   ( char *version );

/* Specific Commands for TMP Sub-System */
ccsCOMPL_STAT ccdtmpContinue  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpGetParam  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpPause     ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpSetParam  ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );
ccsCOMPL_STAT ccdtmpMonitor   ( ccdDESCRIPTION *ccdDesc, vltINT32 period,
				ccsERROR *error );
ccsCOMPL_STAT ccdtmpStop      ( ccdDESCRIPTION *ccdDesc, ccsERROR *error );

#ifdef __cplusplus
}
#endif

#endif  /* ! CCDTMP_H */

/************************************************************************/

/*___o0o___*/


