/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: dbLCC.h,v 4.97.1.1 2005/10/24 13:55:44 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2005-09-21 added external lccReadDbSignal
* bgustafs 2003-02-13 moved dbDataElemSize to db.c as global, add ext ref here
* bgustafs 2003-02-04 added dbDataElemSize
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
* bgustafs  29/09/95  removed wrong change log (2.2.4)
* bgustafs  07/03/96  modified for lcc 3.1
* bgustafs  22/04/96  dbMultiRead/Write definitions move to db.h
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software              SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  2.1          31/01/95        P.Rochez
*       Moved lots of string defines to dbProvided.h
*       Created LCC specific include file
*  2.2.4        04/08/95        ESO(BGU)       New function dbRestoreFile
*  3.1          13/11/93        P.Creten
*       Added structures/functions for multiple read/write.
*------------------------------------------------------------------------------
*  SCCS Id:  %M%  %I%  %G%
*----------------------------------------------------------------------------*/

/*
 * <dbLCC.h> - LCU / DATABASE Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/

/*
 * Constants
 */

#define dbUNDEFINED_SIZE       0


/*
 * Data types
 */

typedef enum {
    dbCONFIG_FILE = 1 ,
    dbCONFIG_MEMORY
} dbCONFIG_MODE;                    /* Backup/Restore mode              */

typedef enum                                /* logical operation for sampling */
                                            /* events                         */
    {
    dbEVT_LOGIC_EQ = 1,                     /* equal to                       */
    dbEVT_LOGIC_NE,                         /* different from                 */
    dbEVT_LOGIC_GT,                         /* greater than                   */
    dbEVT_LOGIC_GE,                         /* greater than or equal to       */
    dbEVT_LOGIC_LT,                         /* less than                      */
    dbEVT_LOGIC_LE,                         /* less than or equal to          */
    dbEVT_LOGIC_ANY                         /* any value                      */
    } dbEVENT_LOGIC;

typedef struct                              /* event structure for sampling   */
    {
    dbSYMADDRESS  name;                     /* attribute's name               */
    dbVALUE       refValue;                 /* reference value                */
    dbEVENT_LOGIC logicalOp;                /* logical operation              */
    } dbEVENT;

/*
 * Data
 */
 
extern vltINT16  dbDataElemSize[];

extern vltINT32 lccReadDbSignal;  /* Defines if a attribute for a signal when reading the database
                                     in direct mode shall also read the actual signal */

/*
 * Function prototypes
 */

/*
 * Read/write functions with simplified interface
 */

ccsCOMPL_STAT dbReadScalar(         /* Simplified read function for     */
                                    /* scalar attributes by name        */
    const dbSYMADDRESS  attrName,   /* Symbolic address of attribute    */
    char                *buffer,    /* Where to put data                */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbWriteScalar(        /* Simplified write function for    */
                                    /* scalar attributes by name        */
    const dbSYMADDRESS  attrName,   /* Symbolic address of attribute    */
    const char          *buffer,    /* Data to be written               */
    ccsERROR            *error      /* Returned error structure         */
    );


/*
 * Control operations to lock/unlock attributes
 */

ccsCOMPL_STAT dbLockAttr(           /* Lock an attribute (write only)   */
    const dbSYMADDRESS  name,       /* Name of attribute                */
    ccsERROR            *error      /* Returned error structure         */
    );      


ccsCOMPL_STAT dbUnlockAttr(         /* Unlock an attribute              */
    const dbSYMADDRESS  name,       /* Name of attribute                */
    ccsERROR            *error      /* Returned error structure         */
    );      


/*
 * Local database creation and initialisation
 */

ccsCOMPL_STAT dbLoad(               /* Create a database structure      */
    const char          *rootName,  /* Name of root directory where DB  */
                                    /* definition is stored             */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbLoadBranch(         /* Create a branch of a database    */
    const char          *rootName,  /* Name of root directory where     */
                                    /* branch definition is stored      */
    const dbSYMADDRESS  point,      /* Point under which the branch is  */
                                    /* loaded                           */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbUnload(             /* Store the local database to disk */
    const char          *rootName,  /* Name of root directory where DB  */
                                    /* definition will be written       */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbUnloadBranch(       /* Store a branch of the database   */
                                    /* to disk                          */
    const char          *rootName,  /* Name of root directory where     */
                                    /* branch definition will be written*/
    const dbSYMADDRESS  point,      /* Name of point defining the       */
                                    /* branch to be unloaded            */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbRestore(            /* Set the value of DB attributes   */
                                    /* given in a file or buffer        */
    dbCONFIG_MODE       mode,       /* File or buffer                   */
    const char          *buffer,    /* Data buffer or file name         */
    vltUINT32           size,       /* Buffer size                      */
    ccsERROR            *error      /* Returned error structure         */
    );

ccsCOMPL_STAT  dbRestoreFile(       /* Restore DB attributes in CCS format */
    const char          *file,      /* File name                           */
    const char          *cwp1,      /* Substitution e.g. "cwp1=cwp2"       */
    const char          *cwp2,      /* Second substitution                 */
    const char          *cwp3,      /* Third substitution                  */
    const char          *cwp4,      /* Fourth substitution                 */
    ccsERROR            *error      /* Returned error structure            */
    );

ccsCOMPL_STAT dbBackup(             /* Save database parameter values   */
    vltLOGICAL          fromFile,   /* ccsTRUE is points or attributes  */
                                    /* names are given in a file,       */
                                    /* ccsFALSE otherwise               */
    const char          *inFileName,/* Name of the file containing the  */
                                    /* points and attributes to be      */
                                    /* backed up. This parameter does   */
                                    /* not have any meaning if fromFile */
                                    /* is ccsFALSE                      */
    const dbSYMADDRESS  pointName,  /* Name of branch. This parameter   */
                                    /* does not have any meaning if     */
                                    /* fromFile is ccsTRUE              */
    dbCONFIG_MODE       mode,       /* File or buffer                   */
    vltLOGICAL          recursiveFlag,
                                    /* Recursive or not                 */
    const char          *fileName,  /* File name (only for file)        */
    char                **buffer,   /* Data buffer                      */
    vltUINT32           *size,      /* Buffer size                      */
    ccsERROR            *error      /* Returned error structure         */
    );


/*
 * LCU specific query operation
 */

ccsCOMPL_STAT dbGetAttrNumber(      /* Get number of attributes for a   */
                                    /* point                            */
    const dbSYMADDRESS  pointName,  /* Point name                       */
    vltUINT8            *attrCnt,   /* Number of attributes             */
    ccsERROR            *error      /* Returned error structure         */
    );


/*
 * Access to remote database: can be enabled, disabled and simulated.
 */

ccsCOMPL_STAT dbEnRemoteAccess(     /* Enable access to remote database */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbDisRemoteAccess(    /* Disable access to remote database*/
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbEnSim(              /* Enable simulation of the remote  */
                                    /* database access                  */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT dbDisSim(             /* Disable simulation of the remote */
                                    /* database access                  */
    ccsERROR            *error      /* Returned error structure         */
    );


