/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: egen.h,v 1.33 2003/12/18 00:40:29 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 04/08/99  PPC: caiRWTable replaced with lcctooCaiRWTable.
*  rschmutzer 03/07/95  created
*
*/

/*************************************************************************
*   NAME
*	egen - egen object type (egenOBJ)
* 
*   SYNOPSIS
*	#include "egen.h"
*
*	egenOBJ
*
*	ccsCOMPL_STAT egenUseObj(
*	    egenOBJ*	useObj,
*	    egenOBJ*	newObj,
*	    egenOBJ*	oldObj,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenDeleteObj(
*	    egenOBJ*	obj,
*	    ccsERROR*	error)
*
*	void* egenNewTmpBuffer(
*	    egenOBJ*	obj,
*	    vltINT32	size)
*
*	void* egenGetTmpBuffer(
*	    egenOBJ*	obj)
*
*	void egenDeleteTmpBuffer(
*	    egenOBJ*	obj)
*
*	ccsCOMPL_STAT egenReadAttribute(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenWriteAttribute(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenReadVector(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	elem1,
*	    vltINT32	elemN,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenWriteVector(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	elem1,
*	    vltINT32	elemN,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenReadTable(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	rec1,
*	    vltINT32	recN,
*	    vltINT32	fld1,
*	    vltINT32	fldN,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenWriteTable(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	rec1,
*	    vltINT32	recN,
*	    vltINT32	fld1,
*	    vltINT32	fldN,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenReadTableAttr(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	rec,
*	    vltINT32	fld,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenWriteTableAttr(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    void*	buf,
*	    vltINT32	rec,
*	    vltINT32	fld,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenGetVectorSize(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    vltINT16*	num,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenGetTableSize(
*	    egenOBJ*	obj,
*	    vltINT32	index,
*	    vltINT16*	num,
*	    ccsERROR*	error)
*
*	ccsCOMPL_STAT egenErrUnknown()
*	ccsCOMPL_STAT egenErrCcsinit()
*	ccsCOMPL_STAT egenErrClass(s1)
*	ccsCOMPL_STAT egenErrTaskSpawn(i1)
*	ccsCOMPL_STAT egenErrTaskKill(i1)
*	ccsCOMPL_STAT egenErrLccTaskSpawn()
*	ccsCOMPL_STAT egenErrConvertDate()
*	ccsCOMPL_STAT egenErrConvertiTimeOfDay()
*	ccsCOMPL_STAT egenErrConvertAbsTime()
*	ccsCOMPL_STAT egenErrConvertDeleted()
*	ccsCOMPL_STAT egenErrMemory(s1)
*	ccsCOMPL_STAT egenErrGeneral(s1)
*
*   DESCRIPTION
*	The header egen.h gives the basic support for egenOBJ types
*	to be able to simulate objects in C for the LCC environment.
*	This header is needed by the code generated with the eccsgen
*	utility.
*
*	An object in this environment is an instance of an egen class.
*	In an egen class, it is possible to define methods, data and
*	exceptions.
*
*	Methods are mapped to C functions with the following
*	general prototype:
*
*	ccsCOMPL_STAT <className><methodName>(
*	    egenOBJ*	obj,
*	    ...		...,	 ** method parameters
*	    ccsERROR*	error)
*
*	The data of an object is stored in the online database.
*	The eccsgen utility generates the corresponding dbl class
*	and methods to access the individual data attributes.
*
*	Exceptions are mapped to standard CCS errors. A single ERROR
*	file is generated for all classes defined in a software 
*       module (see eccsgen user manual).
*
*	The egenOBJ type is a handle used to access an egen object.
*	The handle contains tables to the methods and data owned by
*	the object instance (among other internal data).
*
*	An object is created, calling the constructor method NewObj:
*
*	ccsCOMPL_STAT <class>NewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance
*				(or NULL).
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*	
*	The constructor method will initialise the handle with the right
*	tables. It will also initialise all class attributes owned by
*	the object (see eccsgen user manual).
*
*	After the constructor method has been called, the handle may be
*	used to call the other methods of the object. Data items are
*	accessed through methods generated by the eccsgen utility or
*	through the general functions described below. An exception
*	(or error) is raised calling the corresponding method
*	<className>Err<errorName>().
*
*	An egenOBJ handle can be used only in a single task. I.e. each
*	task needs its own handle to be able to access an object.
*	In particular, if a method is called through the LCC Command
*	Interpreter, then the handle contains information from the LCC
*	Message System to be able to make a reply to the message 
*	received. The code generated by the eccsgen utility takes care
*	of creating a new handle for each command received by the 
*	Command Interpreter before invoking the method corresponding 
*	to the command.
*
*	See the eccsgen user manual for a more detailed description of
*	egen classes and objects.
*
*	The header "egen.h" includes most VxWorks and LCC include files.
*
*	egenUseObj(useObj,newObj,oldObj,error)
*	    If a task needs access to an object that already exists,
*	    then it has to create and initialise a new handle (useObj).
*	    If a task wants to spawn a new task, then it should pass a 
*	    copy of its handle to the new task.
*	    In both situations, the new handle can be initialised
*	    calling egenUseObj().
*
*	    useObj	<INOUT>	Handle for the new instance of the
*				object pointed by the handle newObj
*				(the handle must be allocated by the
*				caller).
*	    newObj	<IN>	Handle to the existing object.
*	    oldObj	<IN>	Handle passed to the current method
*				where egenUseObj() is invoked (or NULL).
*	    
*	egenDeleteObj(obj,error)
*	    Deletes the object related with the handle obj.
*	    Using the handle (or other handles to the same object !) 
*	    after deleting the object is undefined.
*
*	The functions egenRead*() and egenWrite*() are used to access
*	attributes owned by an object. They call the corresponding
*	caiRW*() functions.
*
*	egenReadAttribute(obj,index,buf,error)
*	egenWriteAttribute(obj,index,buf,error)
*	egenReadVector(obj,index,buf,elem1,elemN,error)
*	egenWriteVector(obj,index,buf,elem1,elemN,error)
*	egenReadTable(obj,index,buf,rec1,recN,fld1,fldN,error)
*	egenWriteTable(obj,index,buf,rec1,recN,fld1,fldN,error)
*	    Read and write from/to data attributes owned by an object.
*
*	    index	<IN>	index to the attribute
*	    elem1	<IN>	index of the first element in a vector
*				to be retrieved/stored
*	    elemN	<IN>	index of the last element in a vector
*				to be retrieved/stored
*	    rec1	<IN>	index of the first record in a table
*				to be retrieved/stored
*	    recN	<IN>	index of the last record in a table
*				to be retrieved/stored
*	    fld1	<IN>	index of the first field in a table
*				to be retrieved/stored
*	    fldN	<IN>	index of the last field in a table
*				to be retrieved/stored
*	    buf		<OUT>	buffer where the value is stored
*
*	    The eccsgen utility generates a #define for each attribute
*	    owned by an object of the form:
*
*	    <className>_ATTR_<AttrName>
*
*	    that can be used as the index to the attribute (index above).
*
*	egenReadTableAttr(obj,index,buf,rec,fld,error)
*	egenWriteTableAttr(obj,index,buf,rec,fld,error)
*	    Read and write from/to a specific cell of a table.
*
*	egenGetVectorSize(obj,index,num,error)
*	    Return the number of records in a vector attribute.
*	egenGetTableSize(obj,index,num,error)
*	    Return the number of records in a table attribute.
*
*   SEE ALSO
*	eccsgen(3) lcctoo(3)
*
*-------------------------------------------------------------------------
*/

#ifndef EGEN_H
#define EGEN_H

/*
 * System includes
 */
#include "stdio.h"
#include "string.h"
#include "types.h"
#include "ctype.h"
#include "time.h"
#include "math.h"

#include "sysLib.h"
#include "tickLib.h"
#include "msgQLib.h"
#include "memLib.h"
#include "lstLib.h"
#include "semLib.h"
#include "wdLib.h"
#include "taskLib.h"
#include "rebootLib.h"
#include "usrLib.h"
#include "errnoLib.h"
#include "vxLib.h"

/*
 * LCC interface
 */
#include "ccs.h"
#include "msg.h"
#include "err.h"
#include "io.h"
#include "db.h"
#include "cmd.h"
#include "tims.h"
#include "cai.h"

/*
 * other includes
 */
#include "egenErrors.h"


/*
 * object type
 */
typedef struct egenOBJ_S* egenOBJPTR;
typedef ccsCOMPL_STAT (*egenFUNCPTR)();
typedef struct {

    /* data for the message system */
    const char*   parameters;
    msgMSG*	  msg;
    char**	  replyBuffer;
    msgLENGTH*	  replyBufferSize;

    /* working space */
    vltINT32	  autoIndent;	/* used by auto object		*/
    void*	  tmpBuffer;	/* used by egenNewTmpBuffer	*/

} egenOBJDATA;

typedef struct egenOBJ_S {
    vltINT16	  id;		/* object id (type identifier)	      */
    dbSYMADDRESS* point;	/* OLDB point (only for reference)    */
    egenOBJPTR	  owner;	/* owner of the object		      */
    egenFUNCPTR*  fcts;		/* table of class methods (functions) */
    char*	  attrs;	/* cai table containing direct addrs. */
    egenOBJPTR    objs;		/* table with owned class attributes  */
    void*	  user;		/* free for user data                 */
    egenOBJDATA*  ldata;	/* data local to a method call        */
} egenOBJ;


/*
 * other includes
 */
#include "lcctoo.h"
#include "auto.h"


/*
 * functions to read/write attributes by index
 */
#define egenReadAttribute(obj,index,buf,error) \
    caiRWAttribute((obj)->attrs,0,(index),(buf),(error))
#define egenWriteAttribute(obj,index,buf,error) \
    caiRWAttribute((obj)->attrs,1,(index),(buf),(error))

#define egenReadVector(obj,index,buf,elem1,elemN,error) \
    caiRWVector((obj)->attrs,0,(index),(buf),(elem1),(elemN),(error))
#define egenWriteVector(obj,index,buf,elem1,elemN,error) \
    caiRWVector((obj)->attrs,1,(index),(buf),(elem1),(elemN),(error))

#define egenReadTable(obj,index,buf,rec1,recN,fld1,fldN,error) \
    lcctooCaiRWTable((obj)->attrs,0,(index),(buf),(rec1),(recN),(fld1),(fld2),(error))
#define egenWriteTable(obj,index,buf,rec1,recN,fld1,fldN,error) \
    lcctooCaiRWTable((obj)->attrs,1,(index),(buf),(rec1),(recN),(fld1),(fld2),(error))

#define egenReadTableAttr(obj,index,buf,rec,fld,error) \
    lcctooCaiRWTable((obj)->attrs,0,(index),(buf),(rec),(rec),(fld),(fld),(error))
#define egenWriteTableAttr(obj,index,buf,rec,fld,error) \
    lcctooCaiRWTable((obj)->attrs,1,(index),(buf),(rec),(rec),(fld),(fld),(error))

#define egenGetVectorSize(obj,index,num,error) \
    caiGetAttrInfo((obj)->attrs,(index),NULL,NULL,(num),NULL,NULL,NULL,(error))
#define egenGetTableSize(obj,index,num,error) \
    caiGetAttrInfo((obj)->attrs,(index),NULL,NULL,(num),NULL,NULL,NULL,(error))


/*
 * function to use newObj inside a method belonging to oldObj
 */
#define egenUseObj(useObj,newObj,oldObj,error) \
	lcctooUseObj(useObj,newObj,oldObj,error)

/*
 * function to delete an object (1. function in the function table)
 */
#define egenFCT_DELETE_OBJ 0
#define egenDeleteObj(obj,error) \
	(*((obj)->fcts[egenFCT_DELETE_OBJ]))((obj),(error))


/*
 * temporary working space
 */
#define egenNewTmpBuffer(obj,size) \
	REALLOCBUF((obj)->ldata->tmpBuffer,char,(size))

#define egenGetTmpBuffer(obj) \
	(obj)->ldata->tmpBuffer

#define egenDeleteTmpBuffer(obj) \
	FREE((obj)->ldata->tmpBuffer)


/*
 * error functions
 */
#define egenErrUnknown() \
	{ \
	lcctooLOG_0("Unknown error."); \
	errAdd(error,"egen",egenERR_UNKNOWN,__FILE_LINE__, ""); \
	}
#define egenErrCcsinit() \
	{ \
	lcctooLOG_0("Error registering the process with CCS."); \
	errAdd(error,"egen",egenERR_CCSINIT,__FILE_LINE__, ""); \
	}
#define egenErrClass(a) \
	{ \
	lcctooLOG_0("Error initialising the class instance %s",(a)); \
	errAdd(error,"egen",egenERR_CLASS,__FILE_LINE__, "%s",(a)); \
	}
#define egenErrTaskSpawn(a) \
	{ \
	lcctooLOG_0("Task spawn error (VxWorks error %d).",(a)); \
	errAdd(error,"egen",egenERR_TASK_SPAWN,__FILE_LINE__, "%d",(a)); \
	}
#define egenErrTaskKill(a) \
	{ \
	lcctooLOG_0("Task kill error (VxWorks error %d).",(a)); \
	errAdd(error,"egen",egenERR_TASK_KILL,__FILE_LINE__, "%d",(a)); \
	}
#define egenErrLccTaskSpawn() \
	{ \
	lcctooLOG_0("LCC task spawn error."); \
	errAdd(error,"egen",egenERR_LCC_TASK_SPAWN,__FILE_LINE__, ""); \
	}
#define egenErrConvertDate() \
	{ \
	lcctooLOG_0("Error converting a date parameter."); \
	errAdd(error,"egen",egenERR_CONVERT_DATE,__FILE_LINE__, ""); \
	}
#define egenErrConvertTimeOfDay() \
	{ \
	lcctooLOG_0("Error converting a time of day parameter."); \
	errAdd(error,"egen",egenERR_CONVERT_TIME_OF_DAY,__FILE_LINE__, ""); \
	}
#define egenErrConvertAbsTime() \
	{ \
	lcctooLOG_0("Error converting an absolute time parameter."); \
	errAdd(error,"egen",egenERR_CONVERT_ABS_TIME,__FILE_LINE__, ""); \
	}
#define egenErrObjDeleted(a) \
	{ \
	lcctooLOG_0("Application deleted (object %s).",(a)); \
	errAdd(error,"egen",egenERR_OBJ_DELETED,__FILE_LINE__, "%s",(a)); \
	}
#define egenErrMemory(a) \
	{ \
	lcctooLOG_0("Out of memory %s",(a)); \
	errAdd(error,"egen",egenERR_MEMORY,__FILE_LINE__, "%s",(a)); \
	}

#define egenErrGeneral(a) \
	{ \
	lcctooLOG_0("General error %s",(a)); \
	errAdd(error,"egen",egenERR_GENERAL,__FILE_LINE__, "%s",(a)); \
	}


#endif // !EGEN_H

/************************************************************************/
/* ___oOo___ */
