/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: errLCC.h,v 4.97.1.1 2005/10/24 13:55:45 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  25/08/98  common error functions moved to err.h
* mkiekebu  11/02/98  added errIsReason moved from logProvided.h
* bgustafs  10/06/96  added errCopy
* bgustafs  23/05/96  added errMergeStack and errPrint, errTOP, errBOTTOM
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
* bgustafs  07/03/96  modified for lcc 3.1
* bgustafs  11/04/96  MODULE_ID changed to lccMODULE_ID
* bgustafs  15/04/96  removed errLogError
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software             SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  1.2          23/02/94        P.Rochez        SPR940016.3 Added procNum in
*                                                 errERRSTRT_REPLY_HEADER struct
*  2.1          31/01/95        P.Rochez
*       Copied macros ERROR_ADD, ERROR_SET, ERROR_BREAK and ERROR_RETURN from
*       lccVxWorks.h
*       Created LCC specific include file
*  3.1          13/11/95        P.Creten        SPR 950082 :
*  1                                            Added function errDriverAdd.
*------------------------------------------------------------------------------
*  SCCS Id:  errLCC.h  3.1  05 Mar 1996
*----------------------------------------------------------------------------*/

/*
 * <errLCC.h> - LCU / ERROR SYSTEM Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/

/* 
 * Header files
 */

#include <log.h>


/* 
 * Constants 
 */
#define errTOP    "top"
#define errBOTTOM "bottom"

#define errDriverAdd(x...) \
        errDrvAddArgsCount(__errformatguard_counter(x)+1, ## x)

/* 
 * Data Types
 */


/* 
 * Functions prototypes 
 */


ccsCOMPL_STAT errDrvAddArgsCount (
    vltINT32           nargs,      /* Number of arguments */   
    ccsERROR          *error,      /* current error structure                */
    const ccsMODULEID moduleId,    /* moduleId where error occurred          */
    vltINT16          errorNumber, /* Error identification number            */
    const ccsLOC_ID   locId,       /* Place where the error took place       */
                                   /* or reported  : sub, func, proc         */
    char              *format,     /* Printf format for run time  parameters */
    char              *driverName, /* Identifies the driver                  */
    vltINT16          driverError, /* Driver error identification number     */
    ...                            /* List of variable length of Run Time    */
                                   /* Parameters to be added to the error    */
                                   /* error message                          */
    );

ccsCOMPL_STAT errDriverAdd_v (
    vltINT32           nargs,
    ccsERROR          *error,
    const ccsMODULEID  moduleId,
    vltINT16           errorNumber,
    const ccsLOC_ID    locId,
    char              *format,
    char              *driverName,
    vltINT16           driverError,
    va_list            va_parList 
    );

/*
 * Log servers enable or disable logging of errors
 */

ccsCOMPL_STAT errStartLogging(      /* Start error logging              */
    const ccsPROCNAME name,         /* Name of the logErr server        */
    msgCMDID          cmdId,        /*                                  */
    const msgCMD      cmdName,      /*                                  */
    ccsERROR          *error        /* Returned error structure         */
    );


ccsCOMPL_STAT errStopLogging(       /* Stop error logging               */
    ccsERROR          *error        /* Returned error structure         */
    );


ccsCOMPL_STAT errGetFirstBuffer(    /* Get buffer containing initial    */
                                    /* errors                           */
    vltINT8           **buffer,     /* Buffer containing the initial    */
                                    /* errors logged in the system. The */
                                    /* buffer is formatted as for a     */
                                    /* reply to ERRSTRT command.        */
    vltUINT32         *length,      /* Length of buffer                 */
    ccsERROR          *error        /* Returned error structure         */
    );

void errPrintFirstBuffer( void );   /* Dumps the content of the initial */
                                    /* error buffer on the system       */
                                    /* console                          */


/*
 * Error handling macros
 *
 * N.B. In order to use these macros, lccMODULE_ID must be defined before
 *      err.h is included.
 */

#ifndef ERROR_ADD


#define ERROR_ADD(errorNumber, locId, format, arg...) \
{ \
    errAdd(error, lccMODULE_ID, errorNumber, locId, format, ## arg); \
}
 
#define ERROR_SET(errorNumber, locId, format, arg...) \
{ \
    ERROR_ADD(errorNumber, locId, format, ## arg) \
    returnValue = FAILURE; \
}
 
#define ERROR_BREAK(errorNumber, locId, format, arg...) \
{ \
    ERROR_SET(errorNumber, locId, format, ## arg) \
    break; \
}
 
#define ERROR_RETURN(errorNumber, locId, format, arg...) \
{ \
    ERROR_ADD(errorNumber, locId, format, ## arg) \
    return FAILURE; \
}

#endif /* ERROR_ADD */
 
