/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: evtLCC.h,v 4.97.1.1 2005/10/24 13:55:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  04/05/99  removed evtGetPeriodicList and evtGetAllPeriodicList
* bgustafs  04/05/99  removed evtRequestOnChange, evtAddOnChange, 
*                     evtDeletOnChange, evtRequestPeriodic,
*                     evtAddPeriodic and evtDeletePeriodic (obsolete) 
* bgustafs  04/05/99  removed evtConfigOld and evtConfigAbnormalOld
* bgustafs  04/05/99  removed evtRequestAlarm, evtAddAlarm and evtDeleteAlarm
*                     (obsolete)
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software             SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  2.1          31/01/95        P.Rochez        Major change of event handling:
*       Added evtFILTER, evtEVENT_ID, evtEVENT_DESCR, evtSCALAR_INFO,
*           evtVECTOR_INFO, evtTABLE_INFO and evtDATA data types
*       Events reported as specific messages, thus modification of event reply
*           structures
*       New functions evtAttach(), evtDetach(), evtAttachAlarm(), evtParseMsg()
*           evtParseAlarmMsg, evtSingleDisable(), evtSingleEnable(),
*           evtConfig(), evtConfigAbnormal(), evtGetConfig() and
*           evtGetSampleRate()
*       Old evtConfig() and evtConfigAbnormal() functions renamed as
*           evtConfigOld() and evtConfigAbnormalOld()
*       Removed evtSetMinReportTime
*       Created LCC specific include file
*       Changed evtGetConfig() and added evtGetAlarmConfig()
*       Removed evtGetOnChangeXXXList()
*------------------------------------------------------------------------------
*  SCCS Id:  evtLCC.h  2.1  03 Feb 1995
*----------------------------------------------------------------------------*/

/*
 * <evtLCC.h> - LCU / MONITORING Interface File
 */


/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/

/* 
 * Header files
 */

/*
 * Constants
 */

#define evtANY_WRITE_STRING     "AnyWrite"
#define evtNOT_EQUAL_STRING     "NotEqual"
#define evtDEAD_BAND_STRING     "Deadband"


/*
 * Event configration bit field values
 */

#define evtCONF_HH              0x01    /* HighHigh value provided      */
#define evtCONF_H               0x02    /* High value provided          */
#define evtCONF_L               0x04    /* Low value provided           */
#define evtCONF_LL              0x08    /* LowLow value provided        */
#define evtCONF_DEADBAND        0x10    /* Deadband is provided         */
#define evtCONF_ALL		(evtCONF_HH|evtCONF_H|evtCONF_L|evtCONF_LL)

/*
 * Data types
 */

typedef enum {                      /* Periods for events to report     */
                                    /* periodically                     */
    evt10_MILLISECOND = 1,          /* 0.01 second                      */
    evt100_MILLISECOND,             /* 0.1 second                       */
    evtSECONDS                      /* Number of full seconds           */
} evtPERIOD;


typedef struct {
    dbSYMADDRESS    name;           /* Name of parameter or signal      */
    ccsPROCNAME     procName;       /* Name of process receiving event  */
    ccsENVNAME      envName;        /* Name of process environment      */
    vltUINT32       eventNum;       /* Event number                     */
    evtFILTER       filter;         /* Type of event                    */
} evtEVENT;


typedef struct {
    dbSYMADDRESS    name;           /* Name of parameter or signal      */
    ccsPROCNAME     procName;       /* Name of process receiving event  */
    ccsENVNAME      envName;        /* Name of process environment      */
    vltUINT32       eventNum;       /* Event number                     */
} evtALARM_EVENT;


typedef struct {
    dbSYMADDRESS    name;           /* Name of parameter or signal      */
    ccsPROCNAME     procName;       /* Name of process receiving event  */
    ccsENVNAME      envName;        /* Name of process environment      */
    evtPERIOD       period;         /* Period                           */
    vltUINT32       seconds;        /* Number of seconds (only when     */
                                    /* period = evtSECONDS)             */
} evtPER_EVENT;


/*
 * Function prototypes
 */

/*
 * Event monitoring configuration
 */

ccsCOMPL_STAT evtConfig(            /* Configure an event for a signal  */
                                    /* or a parameter                   */
    const dbSYMADDRESS name,        /* Signal or parameter name         */
    vltUINT8           eventDef,    /* Bit field defining configuration */
    const void         *limitHH,    /* Pointer to High High limit       */
    const void         *limitH,     /* Pointer to High limit            */
    const void         *limitL,     /* Pointer to Low limit             */
    const void         *limitLL,    /* Pointer to Low Low limit         */
    const void         *deadband,   /* Pointer to Deadband              */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtGetConfig(         /* Get the event configuration for  */
                                    /* an attribute                     */
    const dbSYMADDRESS name,        /* parameter name                   */
    dbTYPE             *attrType,   /* Returned attribute type          */
    vltUINT8           *limits,     /* Bitfield defining valid limits   */
    void               *limitHH,    /* Buffer where HH limit is written */
    void               *limitH,     /* Buffer where H limit is written  */
    void               *limitL,     /* Buffer where L limit is written  */
    void               *limitLL,    /* Buffer where LL limit is written */
    void               *deadband,   /* Buffer where deadband is written */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtGetAlarmConfig(    /* Get the alarm configuration for  */
                                    /* an attribute                     */
    const dbSYMADDRESS name,        /* parameter name                   */
    vltUINT8           *alarms,     /* Bitfield defining valid alarms   */
    vltUINT16          *alarmNumHH, /* Error number for HH limit        */
    vltUINT16          *alarmNumH,  /* Error number for H limit         */
    vltUINT16          *alarmNumL,  /* Error number for L limit         */
    vltUINT16          *alarmNumLL, /* Error number for LL limit        */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtGetEventList(      /* Get list of all events requested */
                                    /* by the given process (except     */
                                    /* periodic and alarm events)       */
    const ccsENVNAME   envName,     /* Environment name                 */
    const ccsPROCNAME  procName,    /* Process name                     */
    vltUINT32          *number,     /* Number of returned events        */
    evtEVENT           **list,      /* Returned list                    */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtGetAllEventList(   /* Get list of all requested events */
                                    /* (except periodic and alarm       */
                                    /* events)                          */
    vltUINT32          *number,     /* Number of returned events        */
    evtEVENT           **list,      /* Returned list                    */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtEnEvent(           /* Enable the sending of event      */
                                    /* reports for requesting process   */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtDisEvent(          /* Disable the sending of event     */
                                    /* reports for requesting process   */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtEnAllEvent(        /* Enable the sending of event      */
                                    /* reports for all processes        */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtDisAllEvent(       /* Disable the sending of event     */
                                    /* reports for all processes        */
    ccsERROR           *error       /* Returned error structure         */
    );



ccsCOMPL_STAT evtSetSampleRate(     /* Set the sampling rate for        */
                                    /* signals to be reported on change */
    evtPERIOD          period,      /* Period                           */
    vltUINT32          seconds,     /* Number of seconds (only when     */
                                    /* period = evtSECONDS)             */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtGetSampleRate(     /* Set the sampling rate for        */
                                    /* signals to be reported on change */
    evtPERIOD          *period,     /* Returned period                  */
    vltUINT32          *seconds,    /* Returned number of seconds (only */
                                    /* when period = evtSECONDS)        */
    ccsERROR           *error       /* Returned error structure         */
    );


/*
 * Abnormal event monitoring configuration
 */

ccsCOMPL_STAT evtConfigAbnormal(    /* Configure a signal or a parameter*/
                                    /* to be monitored for abnormal     */
                                    /* event                            */
    const dbSYMADDRESS name,        /* Signal or parameter name         */
    vltUINT8           bitfield,    /* Bit field defining configuration */
    vltUINT16          alarmNumHH,  /* Error number for High High limit */
    vltUINT16          alarmNumH,   /* Error number for High limit      */
    vltUINT16          alarmNumL,   /* Error number for Low limit       */
    vltUINT16          alarmNumLL,  /* Error number for Low Low limit   */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtAttachAlarm(       /* Request alarms for a database    */
                                    /* element                          */
    dbSYMADDRESS       attribute,   /* Database element                 */
    evtEVENT_ID        *eventId,    /* Returned event identifier        */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtParseAlarmMsg(     /* Parse an alarm message           */
    msgHEADER          *message,    /* Pointer to received message      */
    evtEVENT_ID        *eventId,    /* Event identifier                 */
    msgPROCESSID       *trigProc,   /* Process generating alarm         */
    dbSYMADDRESS       attrName,    /* Name of database element         */
    dbTYPE             *dataType,   /* Type of database element         */
    vltUINT16          *alarmNumber,/* Alarm number                     */
    alarmSTATUS        *oldStatus,  /* Old alarm status                 */
    alarmSTATUS        *newStatus,  /* New alarm status                 */
    char               *oldValue,   /* Old value                        */
    char               *newValue,   /* New value                        */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT evtGetAlarmList(      /* Get a list of abnormal events    */
                                    /* reported to the given process    */
    const ccsENVNAME   envName,     /* Environment name                 */
    const ccsPROCNAME  procName,    /* Process name                     */
    vltUINT32          *number,     /* Number of returned events        */
    evtALARM_EVENT     **list,      /* Returned list                    */
    ccsERROR           *error       /* Returned error structure         */
    );
 

ccsCOMPL_STAT evtGetAllAlarmList(   /* Get a list of all abnormal       */
                                    /* events reported                  */
    vltUINT32          *number,     /* Number of returned events        */
    evtALARM_EVENT     **list,      /* Returned list                    */
    ccsERROR           *error       /* Returned error structure         */
    );
 

ccsCOMPL_STAT evtEnAlarm(           /* Enable the sending of abnormal   */
                                    /* event reports for the requesting */
                                    /* process                          */
    ccsERROR           *error       /* Returned error structure         */
    );
 

ccsCOMPL_STAT evtDisAlarm(          /* Disable the sending of abnormal  */
                                    /* event reports for the requesting */
                                    /* process                          */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtEnAllAlarm(        /* Enable the sending of abnormal   */
                                    /* event reports for all processes  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT evtDisAllAlarm(       /* Disable the sending of abnormal  */
                                    /* event reports for all processes  */
    ccsERROR           *error       /* Returned error structure         */
    );


