/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0devContainer.h,v 1.93 2005/08/14 18:41:10 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 29/09/95  Created by eccsgen V0.52 (September 1995)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0devContainer - class
*
*   SYNOPSIS
*	#include "ic0devContainer.h"
*
*	ccsCOMPL_STAT ic0devContainerNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerRegDevice(
*	    INOUT egenOBJ*	obj,
*	    IN    lccDEVICE_NAME	devName,
*	    IN    egenOBJ*	devObj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerUnregDevice(
*	    INOUT egenOBJ*	obj,
*	    IN    lccDEVICE_NAME	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerBeginRestriction(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	index,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerEndRestriction(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	index,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerVarSet(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	syncVar,
*	    IN    vltINT32	value,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devContainerVarWait(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	syncVar,
*	    IN    vltINT32	value,
*	    IN    vltINT32	timeout,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devErrRegister(p1)
*	ccsCOMPL_STAT ic0devErrUnregister(p1)
*	ccsCOMPL_STAT ic0devErrUnknownDevType(p1,p2)
*
*   PARENT CLASS
*	-
*
*   DESCRIPTION
*       The class ic0devCONTAINER is a container class of
*       instrument functions (Software Devices).
*
*   PUBLIC METHODS
*	ic0devContainerNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0devCONTAINER.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0devContainerDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0devCONTAINER
*
*	ic0devContainerRegDevice(obj,devName,devObj,error)
*	   Forwards the RegDevice() call to the related ICS
*	   device manager.
*
*	   ICS devices are managed by an ICS device manager
*	   (derived from an ic0devCONTAINER class). Each device
*	   has to register itself to its manager with this
*	   function.
*
*	   devName	<IN>	device name of the ICS function
*	   devObj	<IN>	related object handle
*
*	ic0devContainerUnregDevice(obj,devName,error)
*	   Forwards the UnregDevice() call to the related ICS
*	   device manager.
*
*	   ICS devices have to unregister themselves from their
*	   manager when they are deleted.
*
*	   devName	<IN>	device name of the ICS function
*
*	ic0devContainerBeginRestriction(obj,devName,index,error)
*	   An ICS device may be related with other ICS devices,
*	   restricting the way how it may be moved. These movement
*	   restrictions between the ICS devices are registered
*	   by the ICS device manager. Therefore, an ICS device
*	   should ask the manager if it is allowed to make a
*	   movement before moving.
*
*	   The BeginRestriction() call asks the ICS device manager
*	   to read its restrictions table from the specified index.
*	    
*	   The restrictions table is used to register restrictions
*	   between the devices of the type:
*
*	   1) Device A can not move, if device B is moving (and other
*	      way round).
*	   2) Device A can not move, if device B is in position X.
*
*	   Restrictions of type 1 are implemented by using a semaphore
*	   that both devices have to register.
*	   Restrictions of type 2 are implemented by registering device B
*	   with the name and value that a status value has to meet, so
*	   that device A can move.
*
*	ic0devContainerEndRestriction(obj,devName,index,error)
*	   Releases the restriction semaphores taken during the
*	   BeginRestriction() call with the same arguments
*	   (see above).
*
*	ic0devContainerVarSet(obj,syncVar,value,error)
*	   Assigns value to the syncronization variable syncVar.
*
*	ic0devContainerVarWait(obj,syncVar,value,timeout,error)
*	   Waits until the syncronization variable has the
*	   requested value. VarWait() returns an error if:
*	   a) the value is not assigned to the variable within
*	      timeout seconds,
*	   b) the requested value is different to 0 and 0 is
*	      assigned to the syncronization variable.
*
*	ic0devContainerErr<error>(...)
*	    The macros ic0devContainerErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0devERR_REGISTER
*	    ic0devERR_UNREGISTER
*	    ic0devERR_UNKNOWN_DEV_TYPE
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*	ic0devManager(3) ic0devSoft(3)
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DEV_CONTAINER_H
#define IC0DEV_CONTAINER_H

#include "egen.h"
#include "ic0devErrors.h"

/*
 * class type identifier
 */
#define ic0devCONTAINER_ID	121

/*
 * function table
 */
#define ic0devCONTAINER_FCT_DELETE_OBJ	0
#define ic0devCONTAINER_FCT_REG_DEVICE	1
#define ic0devCONTAINER_FCT_UNREG_DEVICE	2
#define ic0devCONTAINER_FCT_BEGIN_RESTRICTION	3
#define ic0devCONTAINER_FCT_END_RESTRICTION	4
#define ic0devCONTAINER_FCT_VAR_SET	5
#define ic0devCONTAINER_FCT_VAR_WAIT	6


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_REG_DEVICE)(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    IN    egenOBJ*	devObj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_UNREG_DEVICE)(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_BEGIN_RESTRICTION)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_END_RESTRICTION)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_VAR_SET)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devCONTAINER_FCTPTR_VAR_WAIT)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0devContainerLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dev", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0devErrRegister(p1) \
	{ \
	lcctooLOG_0("ic0devERR_REGISTER: Unable to register device %s",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_REGISTER,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrUnregister(p1) \
	{ \
	lcctooLOG_0("ic0devERR_UNREGISTER: Unable to unregister device %s",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_UNREGISTER,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrUnknownDevType(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_UNKNOWN_DEV_TYPE: Unknown device type %s (%s).",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_UNKNOWN_DEV_TYPE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0devContainerGetOwner(obj) ((obj)->owner)


/*
 * class functions
 */

ccsCOMPL_STAT ic0devContainerRegDevice(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    IN    egenOBJ*	devObj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerUnregDevice(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerBeginRestriction(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerEndRestriction(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerVarSet(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerVarWait(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0devContainerVRegDevice(obj,devName,devObj,error) \
	(*((ic0devCONTAINER_FCTPTR_REG_DEVICE)((obj)->fcts[ic0devCONTAINER_FCT_REG_DEVICE]))) \
	 ((obj),(devName),(devObj),(error))
#define ic0devContainerVUnregDevice(obj,devName,error) \
	(*((ic0devCONTAINER_FCTPTR_UNREG_DEVICE)((obj)->fcts[ic0devCONTAINER_FCT_UNREG_DEVICE]))) \
	 ((obj),(devName),(error))
#define ic0devContainerVBeginRestriction(obj,devName,index,error) \
	(*((ic0devCONTAINER_FCTPTR_BEGIN_RESTRICTION)((obj)->fcts[ic0devCONTAINER_FCT_BEGIN_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devContainerVEndRestriction(obj,devName,index,error) \
	(*((ic0devCONTAINER_FCTPTR_END_RESTRICTION)((obj)->fcts[ic0devCONTAINER_FCT_END_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devContainerVVarSet(obj,syncVar,value,error) \
	(*((ic0devCONTAINER_FCTPTR_VAR_SET)((obj)->fcts[ic0devCONTAINER_FCT_VAR_SET]))) \
	 ((obj),(syncVar),(value),(error))
#define ic0devContainerVVarWait(obj,syncVar,value,timeout,error) \
	(*((ic0devCONTAINER_FCTPTR_VAR_WAIT)((obj)->fcts[ic0devCONTAINER_FCT_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(error))


ccsCOMPL_STAT ic0devContainerNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devContainerDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DEV_CONTAINER_H

/************************************************************************/
/* ___oOo___ */
