/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0devDevices.h,v 1.93 2005/08/14 18:41:13 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   29/10/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0devDevices - class
*
*   SYNOPSIS
*	#include "ic0devDevices.h"
*
*	ccsCOMPL_STAT ic0devDevicesNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devDevicesDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0devDevicesGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0devDevicesLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0devDevicesGetMirr(
*	    IN    egenOBJ*	obj,
*	    OUT   egenOBJ*	attrPtr,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devDevicesGetFilt(
*	    IN    egenOBJ*	obj,
*	    OUT   egenOBJ*	attrPtr,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	private ic0devCONTAINER
*
*   DESCRIPTION
*       Container for all instrument devices.
*
*   PUBLIC METHODS
*	ic0devDevicesNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0devDEVICES.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0devDevicesDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0devDEVICES
*
*	ic0devDevicesErr<error>(...)
*	    The macros ic0devDevicesErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0devDevicesRead<attr>(...)
*	ic0devDevicesWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0devDevicesDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DEV_DEVICES_H
#define IC0DEV_DEVICES_H

#include "egen.h"
#include "ic0devErrors.h"
#include "ic0devContainer.h"
#include "ic0devMot.h"
#include "ic0devMot.h"

/*
 * class type identifier
 */
#define ic0devDEVICES_ID	126

/*
 * attribute table
 */

#define ic0devDEVICES_ATTR_MIRR	1
#define ic0devDEVICES_ATTR_FILT	2

/*
 * function table
 */
#define ic0devDEVICES_FCT_DELETE_OBJ	0
#define ic0devDEVICES_FCT_REG_DEVICE	1
#define ic0devDEVICES_FCT_UNREG_DEVICE	2
#define ic0devDEVICES_FCT_BEGIN_RESTRICTION	3
#define ic0devDEVICES_FCT_END_RESTRICTION	4
#define ic0devDEVICES_FCT_VAR_SET	5
#define ic0devDEVICES_FCT_VAR_WAIT	6


/*
 * function pointer types
 */

/*
 * log function
 */
#define ic0devDevicesLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dev", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0devDevicesGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0devDevicesGetMirr(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0devDEVICES_ATTR_MIRR], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;

#define ic0devDevicesGetFilt(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0devDEVICES_ATTR_FILT], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;



/*
 * class functions
 */
#define ic0devDevicesRegDevice(obj,devName,devObj,error) \
	ic0devContainerRegDevice((obj),(devName),(devObj),(error))
#define ic0devDevicesUnregDevice(obj,devName,error) \
	ic0devContainerUnregDevice((obj),(devName),(error))
#define ic0devDevicesBeginRestriction(obj,devName,index,error) \
	ic0devContainerBeginRestriction((obj),(devName),(index),(error))
#define ic0devDevicesEndRestriction(obj,devName,index,error) \
	ic0devContainerEndRestriction((obj),(devName),(index),(error))
#define ic0devDevicesVarSet(obj,syncVar,value,error) \
	ic0devContainerVarSet((obj),(syncVar),(value),(error))
#define ic0devDevicesVarWait(obj,syncVar,value,timeout,error) \
	ic0devContainerVarWait((obj),(syncVar),(value),(timeout),(error))


/*
 * virtual functions
 */
#define ic0devDevicesVRegDevice(obj,devName,devObj,error) \
	(*((ic0devCONTAINER_FCTPTR_REG_DEVICE)((obj)->fcts[ic0devDEVICES_FCT_REG_DEVICE]))) \
	 ((obj),(devName),(devObj),(error))
#define ic0devDevicesVUnregDevice(obj,devName,error) \
	(*((ic0devCONTAINER_FCTPTR_UNREG_DEVICE)((obj)->fcts[ic0devDEVICES_FCT_UNREG_DEVICE]))) \
	 ((obj),(devName),(error))
#define ic0devDevicesVBeginRestriction(obj,devName,index,error) \
	(*((ic0devCONTAINER_FCTPTR_BEGIN_RESTRICTION)((obj)->fcts[ic0devDEVICES_FCT_BEGIN_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devDevicesVEndRestriction(obj,devName,index,error) \
	(*((ic0devCONTAINER_FCTPTR_END_RESTRICTION)((obj)->fcts[ic0devDEVICES_FCT_END_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devDevicesVVarSet(obj,syncVar,value,error) \
	(*((ic0devCONTAINER_FCTPTR_VAR_SET)((obj)->fcts[ic0devDEVICES_FCT_VAR_SET]))) \
	 ((obj),(syncVar),(value),(error))
#define ic0devDevicesVVarWait(obj,syncVar,value,timeout,error) \
	(*((ic0devCONTAINER_FCTPTR_VAR_WAIT)((obj)->fcts[ic0devDEVICES_FCT_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(error))


ccsCOMPL_STAT ic0devDevicesNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devDevicesDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DEV_DEVICES_H

/************************************************************************/
/* ___oOo___ */
