/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0devManager.h,v 1.93 2005/08/14 18:41:11 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 29/09/95  Created by eccsgen V0.52 (September 1995)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0devManager - class
*
*   SYNOPSIS
*	#include "ic0devManager.h"
*
*	ccsCOMPL_STAT ic0devManagerNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerDevName(
*	    INOUT egenOBJ*	obj,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerDevObj(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   char**	devAddr,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerUsrVarSet(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	syncVar,
*	    IN    vltINT32	value,
*	    IN    vltINT32	varNum,
*	    OUT   vltLOGICAL*	done,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerUsrVarWait(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	syncVar,
*	    IN    vltINT32	value,
*	    IN    vltINT32	timeout,
*	    IN    vltINT32	varNum,
*	    OUT   vltLOGICAL*	done,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerServer(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerLookup(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   egenOBJ**	devObj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devManagerErr<error>(...)
*
*   PARENT CLASS
*	public ic0devCONTAINER
*	public ic0devSOFT (public interface only)
*
*   DESCRIPTION
*       The class ic0devMANAGER implements the ICS device manager.
*       The manager is a process that receives commands and
*       forwards them to the individual devices.
*       ic0devMANAGER implements the same public interface as
*       the ic0devSOFT class. Refer to ic0devSoft(3) for the
*       description of this interface.
*
*       In addition, ic0devMANAGER maintains a table that
*       describes the restrictions between the ICS devices.
*
*   PUBLIC METHODS
*	ic0devManagerNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0devMANAGER.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0devManagerDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0devMANAGER
*
*	ic0devManagerDevName(obj,text,error) (DEVNAME)
*	   Returns the names of the registered devices.
*
*	ic0devManagerDevObj(obj,devName,devAddr,error) (DEVOBJ)
*	   Returns the object address of the requested device.
*
*   PROTECTED METHODS
*	ic0devManagerUsrVarSet(obj,syncVar,value,varNum,done,error)
*	   Assigns value to the syncronization variable var.
*
*	ic0devManagerUsrVarWait(obj,syncVar,value,timeout,varNum,done,error)
*	   Waits until the syncronization variable has the
*	   requested value. VarWait() returns an error if:
*	   a) the value is not assigned to the variable within
*	      timeout seconds.
*	   b) the requested value is different to 0 and 0 is
*	      assigned to the syncronization variable.
*
*   PRIVATE METHODS
*	ic0devManagerServer(obj,error)
*	   The ic0devManagerServer task calls continously
*	   the monitor of the functions managed by the ICS
*	   function manager. The monitor may be used e.g.
*	   to update the status of the function.
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	The names of the defined errors begin as follows:
*	ic0devERR_<error>
*
*   SEE ALSO
*	ic0devMANAGER(5) ic0devContainer(3) ic0devSoft(3)
*
*-------------------------------------------------------------------------
*/

#ifndef IC0DEV_MANAGER_H
#define IC0DEV_MANAGER_H

#include "egen.h"
#include "ic0devErrors.h"
#include "ic0devContainer.h"
#include "ic0devSoft.h"

/*
 * class type identifier
 */
#define ic0devMANAGER_ID	122

/*
 * attribute table
 */
#define ic0devMANAGER_ATTR_SEM_NUM	0
#define ic0devMANAGER_ATTR_RESTRICTIONS	1
#define ic0devMANAGER_ATTR_MONITOR_STANDBY	2
#define ic0devMANAGER_ATTR_MONITOR_ONLINE	3
#define ic0devMANAGER_ATTR_MONITOR_INTERVAL	4
#define ic0devMANAGER_ATTR_SEN4QUEUE	5
#define ic0devMANAGER_ATTR_SYNC_VAR	6
#define ic0devMANAGER_ATTR_MONITOR_SERVER	7


/*
 * function table
 */
#define ic0devMANAGER_FCT_DELETE_OBJ	0
#define ic0devMANAGER_FCT_REG_DEVICE	1
#define ic0devMANAGER_FCT_UNREG_DEVICE	2
#define ic0devMANAGER_FCT_BEGIN_RESTRICTION	3
#define ic0devMANAGER_FCT_END_RESTRICTION	4
#define ic0devMANAGER_FCT_VAR_SET	5
#define ic0devMANAGER_FCT_VAR_WAIT	6
#define ic0devMANAGER_FCT_STANDBY	7
#define ic0devMANAGER_FCT_ONLINE	8
#define ic0devMANAGER_FCT_OFF	9
#define ic0devMANAGER_FCT_EXIT	10
#define ic0devMANAGER_FCT_STATE	11
#define ic0devMANAGER_FCT_SIMULAT	12
#define ic0devMANAGER_FCT_STOP_SIM	13
#define ic0devMANAGER_FCT_STATUS	14
#define ic0devMANAGER_FCT_INIT	15
#define ic0devMANAGER_FCT_STOP	16
#define ic0devMANAGER_FCT_TEST	17
#define ic0devMANAGER_FCT_SELFTST	18
#define ic0devMANAGER_FCT_AUTO_TBL	19
#define ic0devMANAGER_FCT_AUTO_RST	20
#define ic0devMANAGER_FCT_DISABLE	21
#define ic0devMANAGER_FCT_ENABLE	22
#define ic0devMANAGER_FCT_LOCK	23
#define ic0devMANAGER_FCT_UNLOCK	24
#define ic0devMANAGER_FCT_HANDSET	25
#define ic0devMANAGER_FCT_HSET_OFF	26
#define ic0devMANAGER_FCT_FIX_POS	27
#define ic0devMANAGER_FCT_MOVE	28
#define ic0devMANAGER_FCT_MOVE_REL	29
#define ic0devMANAGER_FCT_SET	30
#define ic0devMANAGER_FCT_CHECK	31
#define ic0devMANAGER_FCT_WAIT	32
#define ic0devMANAGER_FCT_SETUP	33
#define ic0devMANAGER_FCT_GET_VAL	34
#define ic0devMANAGER_FCT_SET_VAL	35
#define ic0devMANAGER_FCT_CFG_ELEM	36
#define ic0devMANAGER_FCT_LOG_CONF	37
#define ic0devMANAGER_FCT_GET_CONF	38
#define ic0devMANAGER_FCT_DEV_NAME	39
#define ic0devMANAGER_FCT_DEV_OBJ	40
#define ic0devMANAGER_FCT_MONITOR	41
#define ic0devMANAGER_FCT_INIT_OBJ	42
#define ic0devMANAGER_FCT_EXIT_OBJ	43
#define ic0devMANAGER_FCT_USR_VAR_SET	44
#define ic0devMANAGER_FCT_USR_VAR_WAIT	45
#define ic0devMANAGER_FCT_SERVER	46
#define ic0devMANAGER_FCT_LOOKUP	47


/*
 * device table
 */
typedef struct
    {
    lccDEVICE_NAME name;
    egenOBJ*	   obj;
    }
ic0devMANAGER_TABLE;

/*
 * restrictions table
 */
typedef struct
    {
    lccDEVICE_NAME	device;
    vltINT32		type;
    lccDEVICE_NAME	rdevice;
    vltBYTES16		name;
    vltBYTES20		text;
    vltINT32		value1;
    vltINT32		value2;
    vltINT32		value3;
    vltBYTES48		errorText;
    }
ic0devMANAGER_RESTRICTION;

/*
 * user variables
 */
#define ic0devMANAGER_TABLE_SIZE 50

typedef struct
    {
    vltINT32	restSemNum;
    SEM_ID*	restSem;
    vltINT32	syncSemNum;
    SEM_ID*	syncSem;
    /*
    ic0devMANAGER_TABLE	devices[ic0devMANAGER_TABLE_SIZE];
    vltINT32	devicesNum;
    */
    }
ic0devMANAGER_USERVAR;


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_STANDBY)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_ONLINE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_OFF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_EXIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_STATE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	state,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SIMULAT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_STOP_SIM)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_STOP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_TEST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SELFTST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_AUTO_TBL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	table,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_AUTO_RST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_DISABLE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_ENABLE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_LOCK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_UNLOCK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_HANDSET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_HSET_OFF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_FIX_POS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    IN    vltLOGICAL	reset,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_MOVE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_MOVE_REL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_CHECK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_WAIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SETUP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_GET_VAL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	name[],	/* max. vltBYTES32[2] */
    IN    vltINT16	nameNum,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SET_VAL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	name[],	/* max. vltBYTES32[2] */
    IN    vltINT16	nameNum,
    IN    const char*	value[],	/* max. vltBYTES32[10] */
    IN    vltINT16	valueNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_CFG_ELEM)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	position,
    IN    const char*	id,
    IN    const char*	values[],	/* max. vltBYTES16[10] */
    IN    vltINT16	valuesNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_LOG_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_GET_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_DEV_NAME)(
    INOUT egenOBJ*	obj,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_DEV_OBJ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	devAddr,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_REG_DEVICE)(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    IN    egenOBJ*	devObj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_UNREG_DEVICE)(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_MONITOR)(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_INIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_EXIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_BEGIN_RESTRICTION)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_END_RESTRICTION)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_VAR_SET)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_VAR_WAIT)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_USR_VAR_SET)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	varNum,
    OUT   vltLOGICAL*	done,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_USR_VAR_WAIT)(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    IN    vltINT32	varNum,
    OUT   vltLOGICAL*	done,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_SERVER)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0devMANAGER_FCTPTR_LOOKUP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   egenOBJ**	devObj,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0devManagerLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dev", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0devErrDeviceTable(p1) \
	{ \
	lcctooLOG_0("ic0devERR_DEVICE_TABLE: Device table full (size: %d)",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_DEVICE_TABLE,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrGetRestrictionsSize(p1) \
	{ \
	lcctooLOG_0("ic0devERR_GET_RESTRICTIONS_SIZE: Device %s: Unable to get the size of table 'restrictions'.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_GET_RESTRICTIONS_SIZE,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrRegistered(p1) \
	{ \
	lcctooLOG_0("ic0devERR_REGISTERED: Device %s already registered",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_REGISTERED,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrNotRegistered(p1) \
	{ \
	lcctooLOG_0("ic0devERR_NOT_REGISTERED: Device %s not registered",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_NOT_REGISTERED,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadRestrictions(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_READ_RESTRICTIONS: Device %s: Unable to read attribute 'restrictions' from OLDB (%s).",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_READ_RESTRICTIONS,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0devErrReadMonitorInterval(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_MONITOR_INTERVAL: Device %s: Unable to read attribute 'monitorInterval' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_MONITOR_INTERVAL,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadMonitorOnline(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_MONITOR_ONLINE: Device %s: Unable to read attribute 'monitorOnline' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_MONITOR_ONLINE,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadMonitorStandby(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_MONITOR_STANDBY: Device %s: Unable to read attribute 'monitorStandby' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_MONITOR_STANDBY,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadMonitorServer(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_MONITOR_SERVER: Device %s: Unable to read attribute 'monitorServer' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_MONITOR_SERVER,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadSemNum(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_SEM_NUM: Device %s: Unable to read attribute 'semNum' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_SEM_NUM,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrReadSen4Queue() \
	{ \
	lcctooLOG_0("ic0devERR_READ_SEN4QUEUE: Unable to read attribute 'sen4Queue' from OLDB."); \
	errAdd(error,"ic0dev",ic0devERR_READ_SEN4QUEUE,__FILE_LINE__,""); \
	}
#define ic0devErrReadSyncVar(p1) \
	{ \
	lcctooLOG_0("ic0devERR_READ_SYNC_VAR: Unable to read vector 'syncVar' from OLDB (%d).",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_READ_SYNC_VAR,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrRestIndex(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_REST_INDEX: Device %s: Restriction index %d out of range.",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_REST_INDEX,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define ic0devErrRestNoStatus(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_REST_NO_STATUS: Device %s: Restriction %d: No previous value available. Device name missing.",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_REST_NO_STATUS,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define ic0devErrRestNotOnline(p1,p2,p3,p4) \
	{ \
	lcctooLOG_0("ic0devERR_REST_NOT_ONLINE: Device %s: Restriction %d: Device %s not online. Unable to move, due to restriction %s.",(p1),(p2),(p3),(p4)); \
	errAdd(error,"ic0dev",ic0devERR_REST_NOT_ONLINE,__FILE_LINE__,"%s,%d,%s,%s",(p1),(p2),(p3),(p4)); \
	}
#define ic0devErrRestSimulation(p1,p2,p3,p4) \
	{ \
	lcctooLOG_0("ic0devERR_REST_SIMULATION: Device %s: Restriction %d: Device %s in simulation. Unable to move, due to restriction %s.",(p1),(p2),(p3),(p4)); \
	errAdd(error,"ic0dev",ic0devERR_REST_SIMULATION,__FILE_LINE__,"%s,%d,%s,%s",(p1),(p2),(p3),(p4)); \
	}
#define ic0devErrRestDevice(p1,p2,p3) \
	{ \
	lcctooLOG_0("ic0devERR_REST_DEVICE: Device %s: Restriction %d: Unknown device %s.",(p1),(p2),(p3)); \
	errAdd(error,"ic0dev",ic0devERR_REST_DEVICE,__FILE_LINE__,"%s,%d,%s",(p1),(p2),(p3)); \
	}
#define ic0devErrRestWrongDevice(p1,p2,p3) \
	{ \
	lcctooLOG_0("ic0devERR_REST_WRONG_DEVICE: Device %s: Restriction %d: Device mismatch. The restriction is for device %s.",(p1),(p2),(p3)); \
	errAdd(error,"ic0dev",ic0devERR_REST_WRONG_DEVICE,__FILE_LINE__,"%s,%d,%s",(p1),(p2),(p3)); \
	}
#define ic0devErrRestWrongOp(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_REST_WRONG_OP: Device %s: Restriction %d: Wrong compare code in restriction table.",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_REST_WRONG_OP,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define ic0devErrRestWrongSemaphore(p1,p2,p3) \
	{ \
	lcctooLOG_0("ic0devERR_REST_WRONG_SEMAPHORE: Device %s: Restriction %d: Semaphore index %d out of range.",(p1),(p2),(p3)); \
	errAdd(error,"ic0dev",ic0devERR_REST_WRONG_SEMAPHORE,__FILE_LINE__,"%s,%d,%d",(p1),(p2),(p3)); \
	}
#define ic0devErrRestriction(p1,p2,p3) \
	{ \
	lcctooLOG_0("ic0devERR_RESTRICTION: Device %s: Restriction %d: Unable to move, due to restriction %s.",(p1),(p2),(p3)); \
	errAdd(error,"ic0dev",ic0devERR_RESTRICTION,__FILE_LINE__,"%s,%d,%s",(p1),(p2),(p3)); \
	}
#define ic0devErrSyncVar(p1) \
	{ \
	lcctooLOG_0("ic0devERR_SYNC_VAR: Syncronization variable index (%d) out of range.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_SYNC_VAR,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrSyncVarError(p1) \
	{ \
	lcctooLOG_0("ic0devERR_SYNC_VAR_ERROR: Syncronization error (variable %d).",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_SYNC_VAR_ERROR,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrSyncVarFlush(p1) \
	{ \
	lcctooLOG_0("ic0devERR_SYNC_VAR_FLUSH: Error flushing the semaphore of a syncronization variable (variable %d).",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_SYNC_VAR_FLUSH,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrSyncVarTimeout(p1) \
	{ \
	lcctooLOG_0("ic0devERR_SYNC_VAR_TIMEOUT: Syncronization timeout (variable %d).",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_SYNC_VAR_TIMEOUT,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0devErrSyncVarNum(p1) \
	{ \
	lcctooLOG_0("ic0devERR_SYNC_VAR_NUM: Device %s: Unable to read size of vector 'syncVar' from OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_SYNC_VAR_NUM,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrWriteMonitorServer(p1) \
	{ \
	lcctooLOG_0("ic0devERR_WRITE_MONITOR_SERVER: Device %s: Unable to write attribute 'monitorServer' to OLDB.",(p1)); \
	errAdd(error,"ic0dev",ic0devERR_WRITE_MONITOR_SERVER,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0devErrWriteSyncVar(p1,p2) \
	{ \
	lcctooLOG_0("ic0devERR_WRITE_SYNC_VAR: Unable to write into vector 'syncVar' of the OLDB (index=%d, value=%d).",(p1),(p2)); \
	errAdd(error,"ic0dev",ic0devERR_WRITE_SYNC_VAR,__FILE_LINE__,"%d,%d",(p1),(p2)); \
	}
#define ic0devErrWrongRest(p1,p2,p3) \
	{ \
	lcctooLOG_0("ic0devERR_WRONG_REST: Device %s: Restriction %d: Unknown restriction type %d.",(p1),(p2),(p3)); \
	errAdd(error,"ic0dev",ic0devERR_WRONG_REST,__FILE_LINE__,"%s,%d,%d",(p1),(p2),(p3)); \
	}

/*
 * get object owner
 */
#define ic0devManagerGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0devManagerReadSemNum(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_SEM_NUM,(buf),(error))
#define ic0devManagerWriteSemNum(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_SEM_NUM,(void*) (buf),(error))
#define ic0devManagerGetRestrictionsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0devMANAGER_ATTR_RESTRICTIONS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0devManagerReadRestrictions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0devMANAGER_ATTR_RESTRICTIONS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0devManagerWriteRestrictions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0devMANAGER_ATTR_RESTRICTIONS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0devManagerReadMonitorStandby(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_MONITOR_STANDBY,(buf),(error))
#define ic0devManagerWriteMonitorStandby(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_MONITOR_STANDBY,(void*) (buf),(error))
#define ic0devManagerReadMonitorOnline(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_MONITOR_ONLINE,(buf),(error))
#define ic0devManagerWriteMonitorOnline(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_MONITOR_ONLINE,(void*) (buf),(error))
#define ic0devManagerReadMonitorInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_MONITOR_INTERVAL,(buf),(error))
#define ic0devManagerWriteMonitorInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_MONITOR_INTERVAL,(void*) (buf),(error))
#define ic0devManagerReadSen4Queue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_SEN4QUEUE,(buf),(error))
#define ic0devManagerWriteSen4Queue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_SEN4QUEUE,(void*) (buf),(error))
#define ic0devManagerGetSyncVarSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0devMANAGER_ATTR_SYNC_VAR,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0devManagerReadSyncVar(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0devMANAGER_ATTR_SYNC_VAR,(buf),(firstElt),(lastElt),(error))
#define ic0devManagerWriteSyncVar(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0devMANAGER_ATTR_SYNC_VAR,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0devManagerReadMonitorServer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devMANAGER_ATTR_MONITOR_SERVER,(buf),(error))
#define ic0devManagerWriteMonitorServer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devMANAGER_ATTR_MONITOR_SERVER,(void*) (buf),(error))


/*
 * class functions
 */

ccsCOMPL_STAT ic0devManagerStandby(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerOnline(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerOff(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerExit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerState(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	state,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerSimulat(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerStopSim(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerStop(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerTest(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerSelftst(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerAutoTbl(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	table,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerAutoRst(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerDisable(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerEnable(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerLock(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerUnlock(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerHandset(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerHsetOff(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerFixPos(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    IN    vltLOGICAL	reset,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerMove(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerMoveRel(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerSet(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerCheck(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerWait(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerSetup(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerGetVal(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	name[],	/* max. vltBYTES32[2] */
    IN    vltINT16	nameNum,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerSetVal(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	name[],	/* max. vltBYTES32[2] */
    IN    vltINT16	nameNum,
    IN    const char*	value[],	/* max. vltBYTES32[10] */
    IN    vltINT16	valueNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerCfgElem(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	position,
    IN    const char*	id,
    IN    const char*	values[],	/* max. vltBYTES16[10] */
    IN    vltINT16	valuesNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerLogConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerGetConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerDevName(
    INOUT egenOBJ*	obj,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerDevObj(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	devAddr,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerRegDevice(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    IN    egenOBJ*	devObj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerUnregDevice(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerMonitor(
    INOUT egenOBJ*	obj,
    IN    lccDEVICE_NAME	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerInitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerExitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerBeginRestriction(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerEndRestriction(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	index,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerVarSet(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerVarWait(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerUsrVarSet(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	varNum,
    OUT   vltLOGICAL*	done,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerUsrVarWait(
    INOUT egenOBJ*	obj,
    IN    vltINT32	syncVar,
    IN    vltINT32	value,
    IN    vltINT32	timeout,
    IN    vltINT32	varNum,
    OUT   vltLOGICAL*	done,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerServer(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerLookup(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   egenOBJ**	devObj,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0devManagerVRegDevice(obj,devName,devObj,error) \
	(*((ic0devMANAGER_FCTPTR_REG_DEVICE)((obj)->fcts[ic0devMANAGER_FCT_REG_DEVICE]))) \
	 ((obj),(devName),(devObj),(error))
#define ic0devManagerVUnregDevice(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_UNREG_DEVICE)((obj)->fcts[ic0devMANAGER_FCT_UNREG_DEVICE]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVBeginRestriction(obj,devName,index,error) \
	(*((ic0devMANAGER_FCTPTR_BEGIN_RESTRICTION)((obj)->fcts[ic0devMANAGER_FCT_BEGIN_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devManagerVEndRestriction(obj,devName,index,error) \
	(*((ic0devMANAGER_FCTPTR_END_RESTRICTION)((obj)->fcts[ic0devMANAGER_FCT_END_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devManagerVVarSet(obj,syncVar,value,error) \
	(*((ic0devMANAGER_FCTPTR_VAR_SET)((obj)->fcts[ic0devMANAGER_FCT_VAR_SET]))) \
	 ((obj),(syncVar),(value),(error))
#define ic0devManagerVVarWait(obj,syncVar,value,timeout,error) \
	(*((ic0devMANAGER_FCTPTR_VAR_WAIT)((obj)->fcts[ic0devMANAGER_FCT_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(error))
#define ic0devManagerVStandby(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STANDBY)((obj)->fcts[ic0devMANAGER_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVOnline(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_ONLINE)((obj)->fcts[ic0devMANAGER_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVOff(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_OFF)((obj)->fcts[ic0devMANAGER_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVExit(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_EXIT)((obj)->fcts[ic0devMANAGER_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVState(obj,devName,state,error) \
	(*((ic0devMANAGER_FCTPTR_STATE)((obj)->fcts[ic0devMANAGER_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0devManagerVSimulat(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_SIMULAT)((obj)->fcts[ic0devMANAGER_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVStopSim(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STOP_SIM)((obj)->fcts[ic0devMANAGER_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0devMANAGER_FCTPTR_STATUS)((obj)->fcts[ic0devMANAGER_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0devManagerVInit(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_INIT)((obj)->fcts[ic0devMANAGER_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVStop(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STOP)((obj)->fcts[ic0devMANAGER_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVTest(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_TEST)((obj)->fcts[ic0devMANAGER_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVSelftst(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_SELFTST)((obj)->fcts[ic0devMANAGER_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVAutoTbl(obj,devName,table,text,error) \
	(*((ic0devMANAGER_FCTPTR_AUTO_TBL)((obj)->fcts[ic0devMANAGER_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0devManagerVAutoRst(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_AUTO_RST)((obj)->fcts[ic0devMANAGER_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVDisable(obj,devName,access,error) \
	(*((ic0devMANAGER_FCTPTR_DISABLE)((obj)->fcts[ic0devMANAGER_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0devManagerVEnable(obj,devName,access,error) \
	(*((ic0devMANAGER_FCTPTR_ENABLE)((obj)->fcts[ic0devMANAGER_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0devManagerVLock(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_LOCK)((obj)->fcts[ic0devMANAGER_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVUnlock(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_UNLOCK)((obj)->fcts[ic0devMANAGER_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVHandset(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_HANDSET)((obj)->fcts[ic0devMANAGER_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVHsetOff(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_HSET_OFF)((obj)->fcts[ic0devMANAGER_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((ic0devMANAGER_FCTPTR_FIX_POS)((obj)->fcts[ic0devMANAGER_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0devManagerVMove(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_MOVE)((obj)->fcts[ic0devMANAGER_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devManagerVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_MOVE_REL)((obj)->fcts[ic0devMANAGER_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devManagerVSet(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_SET)((obj)->fcts[ic0devMANAGER_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devManagerVCheck(obj,devName,status,error) \
	(*((ic0devMANAGER_FCTPTR_CHECK)((obj)->fcts[ic0devMANAGER_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0devManagerVWait(obj,devName,timeout,error) \
	(*((ic0devMANAGER_FCTPTR_WAIT)((obj)->fcts[ic0devMANAGER_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0devManagerVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0devMANAGER_FCTPTR_SETUP)((obj)->fcts[ic0devMANAGER_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0devManagerVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devMANAGER_FCTPTR_GET_VAL)((obj)->fcts[ic0devMANAGER_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0devManagerVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devMANAGER_FCTPTR_SET_VAL)((obj)->fcts[ic0devMANAGER_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0devManagerVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devMANAGER_FCTPTR_CFG_ELEM)((obj)->fcts[ic0devMANAGER_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0devManagerVLogConf(obj,devName,allElements,error) \
	(*((ic0devMANAGER_FCTPTR_LOG_CONF)((obj)->fcts[ic0devMANAGER_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0devManagerVGetConf(obj,devName,allElements,text,error) \
	(*((ic0devMANAGER_FCTPTR_GET_CONF)((obj)->fcts[ic0devMANAGER_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0devManagerVDevName(obj,text,error) \
	(*((ic0devMANAGER_FCTPTR_DEV_NAME)((obj)->fcts[ic0devMANAGER_FCT_DEV_NAME]))) \
	 ((obj),(text),(error))
#define ic0devManagerVDevObj(obj,devName,devAddr,error) \
	(*((ic0devMANAGER_FCTPTR_DEV_OBJ)((obj)->fcts[ic0devMANAGER_FCT_DEV_OBJ]))) \
	 ((obj),(devName),(devAddr),(error))
#define ic0devManagerVMonitor(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_MONITOR)((obj)->fcts[ic0devMANAGER_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0devManagerVInitObj(obj,error) \
	(*((ic0devMANAGER_FCTPTR_INIT_OBJ)((obj)->fcts[ic0devMANAGER_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0devManagerVExitObj(obj,error) \
	(*((ic0devMANAGER_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0devMANAGER_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0devManagerVUsrVarSet(obj,syncVar,value,varNum,done,error) \
	(*((ic0devMANAGER_FCTPTR_USR_VAR_SET)((obj)->fcts[ic0devMANAGER_FCT_USR_VAR_SET]))) \
	 ((obj),(syncVar),(value),(varNum),(done),(error))
#define ic0devManagerVUsrVarWait(obj,syncVar,value,timeout,varNum,done,error) \
	(*((ic0devMANAGER_FCTPTR_USR_VAR_WAIT)((obj)->fcts[ic0devMANAGER_FCT_USR_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(varNum),(done),(error))
#define ic0devManagerVServer(obj,error) \
	(*((ic0devMANAGER_FCTPTR_SERVER)((obj)->fcts[ic0devMANAGER_FCT_SERVER]))) \
	 ((obj),(error))
#define ic0devManagerVLookup(obj,devName,devObj,error) \
	(*((ic0devMANAGER_FCTPTR_LOOKUP)((obj)->fcts[ic0devMANAGER_FCT_LOOKUP]))) \
	 ((obj),(devName),(devObj),(error))


ccsCOMPL_STAT ic0devManagerNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devManagerDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * local variables
 */
extern vltINT32            ic0devManagerNumDevices;
extern ic0devMANAGER_TABLE ic0devManagerDevices[ic0devMANAGER_TABLE_SIZE];
extern vltBYTES16	   ic0devFitsDict;
extern vltBYTES16	   ic0devFitsMask;


#endif // !IC0DEV_MANAGER_H

/************************************************************************/
/* ___oOo___ */
