/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0devServ.h,v 1.93 2005/08/14 18:41:13 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   29/10/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0devServ - class
*
*   SYNOPSIS
*	#include "ic0devServ.h"
*
*	ccsCOMPL_STAT ic0devServNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0devServGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0devServLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0devServRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0devServWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	public ic0devMANAGER
*
*   DESCRIPTION
*       The class ic0devSERV implements a generic frontend for
*       all standard instrument devices.
*
*       As the instrument functions are implemented as LCC software
*       devices, this interface implements all the LCC software
*       device and ICS standard commands.  These commands are
*       forwarded to the corresponding device.
*
*   PUBLIC METHODS
*	ic0devServNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0devSERV.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0devServDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0devSERV
*
*	ic0devServErr<error>(...)
*	    The macros ic0devServErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0devServRead<attr>(...)
*	ic0devServWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0devServDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DEV_SERV_H
#define IC0DEV_SERV_H

#include "egen.h"
#include "ic0devErrors.h"
#include "ic0devManager.h"
#include "ic0devDevices.h"

/*
 * class type identifier
 */
#define ic0devSERV_ID	125

/*
 * attribute table
 */
#define ic0devSERV_ATTR_SEM_NUM	0
#define ic0devSERV_ATTR_RESTRICTIONS	1
#define ic0devSERV_ATTR_MONITOR_STANDBY	2
#define ic0devSERV_ATTR_MONITOR_ONLINE	3
#define ic0devSERV_ATTR_MONITOR_INTERVAL	4
#define ic0devSERV_ATTR_SEN4QUEUE	5
#define ic0devSERV_ATTR_SYNC_VAR	6
#define ic0devSERV_ATTR_MONITOR_SERVER	7

#define ic0devSERV_ATTR_DEVICES	1

/*
 * function table
 */
#define ic0devSERV_FCT_DELETE_OBJ	0
#define ic0devSERV_FCT_REG_DEVICE	1
#define ic0devSERV_FCT_UNREG_DEVICE	2
#define ic0devSERV_FCT_BEGIN_RESTRICTION	3
#define ic0devSERV_FCT_END_RESTRICTION	4
#define ic0devSERV_FCT_VAR_SET	5
#define ic0devSERV_FCT_VAR_WAIT	6
#define ic0devSERV_FCT_STANDBY	7
#define ic0devSERV_FCT_ONLINE	8
#define ic0devSERV_FCT_OFF	9
#define ic0devSERV_FCT_EXIT	10
#define ic0devSERV_FCT_STATE	11
#define ic0devSERV_FCT_SIMULAT	12
#define ic0devSERV_FCT_STOP_SIM	13
#define ic0devSERV_FCT_STATUS	14
#define ic0devSERV_FCT_INIT	15
#define ic0devSERV_FCT_STOP	16
#define ic0devSERV_FCT_TEST	17
#define ic0devSERV_FCT_SELFTST	18
#define ic0devSERV_FCT_AUTO_TBL	19
#define ic0devSERV_FCT_AUTO_RST	20
#define ic0devSERV_FCT_DISABLE	21
#define ic0devSERV_FCT_ENABLE	22
#define ic0devSERV_FCT_LOCK	23
#define ic0devSERV_FCT_UNLOCK	24
#define ic0devSERV_FCT_HANDSET	25
#define ic0devSERV_FCT_HSET_OFF	26
#define ic0devSERV_FCT_FIX_POS	27
#define ic0devSERV_FCT_MOVE	28
#define ic0devSERV_FCT_MOVE_REL	29
#define ic0devSERV_FCT_SET	30
#define ic0devSERV_FCT_CHECK	31
#define ic0devSERV_FCT_WAIT	32
#define ic0devSERV_FCT_SETUP	33
#define ic0devSERV_FCT_GET_VAL	34
#define ic0devSERV_FCT_SET_VAL	35
#define ic0devSERV_FCT_CFG_ELEM	36
#define ic0devSERV_FCT_LOG_CONF	37
#define ic0devSERV_FCT_GET_CONF	38
#define ic0devSERV_FCT_DEV_NAME	39
#define ic0devSERV_FCT_DEV_OBJ	40
#define ic0devSERV_FCT_MONITOR	41
#define ic0devSERV_FCT_INIT_OBJ	42
#define ic0devSERV_FCT_EXIT_OBJ	43
#define ic0devSERV_FCT_USR_VAR_SET	44
#define ic0devSERV_FCT_USR_VAR_WAIT	45
#define ic0devSERV_FCT_SERVER	46
#define ic0devSERV_FCT_LOOKUP	47


/*
 * function pointer types
 */

/*
 * log function
 */
#define ic0devServLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dev", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0devServGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0devServReadSemNum(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_SEM_NUM,(buf),(error))
#define ic0devServWriteSemNum(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_SEM_NUM,(void*) (buf),(error))
#define ic0devServGetRestrictionsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0devSERV_ATTR_RESTRICTIONS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0devServReadRestrictions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0devSERV_ATTR_RESTRICTIONS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0devServWriteRestrictions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0devSERV_ATTR_RESTRICTIONS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0devServReadMonitorStandby(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_MONITOR_STANDBY,(buf),(error))
#define ic0devServWriteMonitorStandby(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_MONITOR_STANDBY,(void*) (buf),(error))
#define ic0devServReadMonitorOnline(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_MONITOR_ONLINE,(buf),(error))
#define ic0devServWriteMonitorOnline(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_MONITOR_ONLINE,(void*) (buf),(error))
#define ic0devServReadMonitorInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_MONITOR_INTERVAL,(buf),(error))
#define ic0devServWriteMonitorInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_MONITOR_INTERVAL,(void*) (buf),(error))
#define ic0devServReadSen4Queue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_SEN4QUEUE,(buf),(error))
#define ic0devServWriteSen4Queue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_SEN4QUEUE,(void*) (buf),(error))
#define ic0devServGetSyncVarSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0devSERV_ATTR_SYNC_VAR,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0devServReadSyncVar(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0devSERV_ATTR_SYNC_VAR,(buf),(firstElt),(lastElt),(error))
#define ic0devServWriteSyncVar(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0devSERV_ATTR_SYNC_VAR,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0devServReadMonitorServer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0devSERV_ATTR_MONITOR_SERVER,(buf),(error))
#define ic0devServWriteMonitorServer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0devSERV_ATTR_MONITOR_SERVER,(void*) (buf),(error))
#define ic0devServGetDevices(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0devSERV_ATTR_DEVICES], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;



/*
 * class functions
 */
#define ic0devServStandby(obj,devName,error) \
	ic0devManagerStandby((obj),(devName),(error))
#define ic0devServOnline(obj,devName,error) \
	ic0devManagerOnline((obj),(devName),(error))
#define ic0devServOff(obj,devName,error) \
	ic0devManagerOff((obj),(devName),(error))
#define ic0devServExit(obj,devName,error) \
	ic0devManagerExit((obj),(devName),(error))
#define ic0devServState(obj,devName,state,error) \
	ic0devManagerState((obj),(devName),(state),(error))
#define ic0devServSimulat(obj,devName,error) \
	ic0devManagerSimulat((obj),(devName),(error))
#define ic0devServStopSim(obj,devName,error) \
	ic0devManagerStopSim((obj),(devName),(error))
#define ic0devServStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0devManagerStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0devServInit(obj,devName,error) \
	ic0devManagerInit((obj),(devName),(error))
#define ic0devServStop(obj,devName,error) \
	ic0devManagerStop((obj),(devName),(error))
#define ic0devServTest(obj,devName,error) \
	ic0devManagerTest((obj),(devName),(error))
#define ic0devServSelftst(obj,devName,error) \
	ic0devManagerSelftst((obj),(devName),(error))
#define ic0devServAutoTbl(obj,devName,table,text,error) \
	ic0devManagerAutoTbl((obj),(devName),(table),(text),(error))
#define ic0devServAutoRst(obj,devName,error) \
	ic0devManagerAutoRst((obj),(devName),(error))
#define ic0devServDisable(obj,devName,access,error) \
	ic0devManagerDisable((obj),(devName),(access),(error))
#define ic0devServEnable(obj,devName,access,error) \
	ic0devManagerEnable((obj),(devName),(access),(error))
#define ic0devServLock(obj,devName,error) \
	ic0devManagerLock((obj),(devName),(error))
#define ic0devServUnlock(obj,devName,error) \
	ic0devManagerUnlock((obj),(devName),(error))
#define ic0devServHandset(obj,devName,error) \
	ic0devManagerHandset((obj),(devName),(error))
#define ic0devServHsetOff(obj,devName,error) \
	ic0devManagerHsetOff((obj),(devName),(error))
#define ic0devServFixPos(obj,devName,position,positionNum,reset,error) \
	ic0devManagerFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0devServMove(obj,devName,position,positionNum,error) \
	ic0devManagerMove((obj),(devName),(position),(positionNum),(error))
#define ic0devServMoveRel(obj,devName,position,positionNum,error) \
	ic0devManagerMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0devServSet(obj,devName,position,positionNum,error) \
	ic0devManagerSet((obj),(devName),(position),(positionNum),(error))
#define ic0devServCheck(obj,devName,status,error) \
	ic0devManagerCheck((obj),(devName),(status),(error))
#define ic0devServWait(obj,devName,timeout,error) \
	ic0devManagerWait((obj),(devName),(timeout),(error))
#define ic0devServSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0devManagerSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0devServGetVal(obj,devName,name,nameNum,status,error) \
	ic0devManagerGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0devServSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devManagerSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0devServCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devManagerCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0devServLogConf(obj,devName,allElements,error) \
	ic0devManagerLogConf((obj),(devName),(allElements),(error))
#define ic0devServGetConf(obj,devName,allElements,text,error) \
	ic0devManagerGetConf((obj),(devName),(allElements),(text),(error))
#define ic0devServDevName(obj,text,error) \
	ic0devManagerDevName((obj),(text),(error))
#define ic0devServDevObj(obj,devName,devAddr,error) \
	ic0devManagerDevObj((obj),(devName),(devAddr),(error))
#define ic0devServRegDevice(obj,devName,devObj,error) \
	ic0devManagerRegDevice((obj),(devName),(devObj),(error))
#define ic0devServUnregDevice(obj,devName,error) \
	ic0devManagerUnregDevice((obj),(devName),(error))
#define ic0devServMonitor(obj,devName,error) \
	ic0devManagerMonitor((obj),(devName),(error))
#define ic0devServInitObj(obj,error) \
	ic0devManagerInitObj((obj),(error))
#define ic0devServExitObj(obj,error) \
	ic0devManagerExitObj((obj),(error))
#define ic0devServBeginRestriction(obj,devName,index,error) \
	ic0devManagerBeginRestriction((obj),(devName),(index),(error))
#define ic0devServEndRestriction(obj,devName,index,error) \
	ic0devManagerEndRestriction((obj),(devName),(index),(error))
#define ic0devServVarSet(obj,syncVar,value,error) \
	ic0devManagerVarSet((obj),(syncVar),(value),(error))
#define ic0devServVarWait(obj,syncVar,value,timeout,error) \
	ic0devManagerVarWait((obj),(syncVar),(value),(timeout),(error))
#define ic0devServUsrVarSet(obj,syncVar,value,varNum,done,error) \
	ic0devManagerUsrVarSet((obj),(syncVar),(value),(varNum),(done),(error))
#define ic0devServUsrVarWait(obj,syncVar,value,timeout,varNum,done,error) \
	ic0devManagerUsrVarWait((obj),(syncVar),(value),(timeout),(varNum),(done),(error))
#define ic0devServServer(obj,error) \
	ic0devManagerServer((obj),(error))
#define ic0devServLookup(obj,devName,devObj,error) \
	ic0devManagerLookup((obj),(devName),(devObj),(error))


/*
 * virtual functions
 */
#define ic0devServVRegDevice(obj,devName,devObj,error) \
	(*((ic0devMANAGER_FCTPTR_REG_DEVICE)((obj)->fcts[ic0devSERV_FCT_REG_DEVICE]))) \
	 ((obj),(devName),(devObj),(error))
#define ic0devServVUnregDevice(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_UNREG_DEVICE)((obj)->fcts[ic0devSERV_FCT_UNREG_DEVICE]))) \
	 ((obj),(devName),(error))
#define ic0devServVBeginRestriction(obj,devName,index,error) \
	(*((ic0devMANAGER_FCTPTR_BEGIN_RESTRICTION)((obj)->fcts[ic0devSERV_FCT_BEGIN_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devServVEndRestriction(obj,devName,index,error) \
	(*((ic0devMANAGER_FCTPTR_END_RESTRICTION)((obj)->fcts[ic0devSERV_FCT_END_RESTRICTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0devServVVarSet(obj,syncVar,value,error) \
	(*((ic0devMANAGER_FCTPTR_VAR_SET)((obj)->fcts[ic0devSERV_FCT_VAR_SET]))) \
	 ((obj),(syncVar),(value),(error))
#define ic0devServVVarWait(obj,syncVar,value,timeout,error) \
	(*((ic0devMANAGER_FCTPTR_VAR_WAIT)((obj)->fcts[ic0devSERV_FCT_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(error))
#define ic0devServVStandby(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STANDBY)((obj)->fcts[ic0devSERV_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0devServVOnline(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_ONLINE)((obj)->fcts[ic0devSERV_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0devServVOff(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_OFF)((obj)->fcts[ic0devSERV_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0devServVExit(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_EXIT)((obj)->fcts[ic0devSERV_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0devServVState(obj,devName,state,error) \
	(*((ic0devMANAGER_FCTPTR_STATE)((obj)->fcts[ic0devSERV_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0devServVSimulat(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_SIMULAT)((obj)->fcts[ic0devSERV_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0devServVStopSim(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STOP_SIM)((obj)->fcts[ic0devSERV_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0devServVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0devMANAGER_FCTPTR_STATUS)((obj)->fcts[ic0devSERV_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0devServVInit(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_INIT)((obj)->fcts[ic0devSERV_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0devServVStop(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_STOP)((obj)->fcts[ic0devSERV_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0devServVTest(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_TEST)((obj)->fcts[ic0devSERV_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0devServVSelftst(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_SELFTST)((obj)->fcts[ic0devSERV_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0devServVAutoTbl(obj,devName,table,text,error) \
	(*((ic0devMANAGER_FCTPTR_AUTO_TBL)((obj)->fcts[ic0devSERV_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0devServVAutoRst(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_AUTO_RST)((obj)->fcts[ic0devSERV_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0devServVDisable(obj,devName,access,error) \
	(*((ic0devMANAGER_FCTPTR_DISABLE)((obj)->fcts[ic0devSERV_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0devServVEnable(obj,devName,access,error) \
	(*((ic0devMANAGER_FCTPTR_ENABLE)((obj)->fcts[ic0devSERV_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0devServVLock(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_LOCK)((obj)->fcts[ic0devSERV_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0devServVUnlock(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_UNLOCK)((obj)->fcts[ic0devSERV_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0devServVHandset(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_HANDSET)((obj)->fcts[ic0devSERV_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0devServVHsetOff(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_HSET_OFF)((obj)->fcts[ic0devSERV_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0devServVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((ic0devMANAGER_FCTPTR_FIX_POS)((obj)->fcts[ic0devSERV_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0devServVMove(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_MOVE)((obj)->fcts[ic0devSERV_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devServVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_MOVE_REL)((obj)->fcts[ic0devSERV_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devServVSet(obj,devName,position,positionNum,error) \
	(*((ic0devMANAGER_FCTPTR_SET)((obj)->fcts[ic0devSERV_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0devServVCheck(obj,devName,status,error) \
	(*((ic0devMANAGER_FCTPTR_CHECK)((obj)->fcts[ic0devSERV_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0devServVWait(obj,devName,timeout,error) \
	(*((ic0devMANAGER_FCTPTR_WAIT)((obj)->fcts[ic0devSERV_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0devServVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0devMANAGER_FCTPTR_SETUP)((obj)->fcts[ic0devSERV_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0devServVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devMANAGER_FCTPTR_GET_VAL)((obj)->fcts[ic0devSERV_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0devServVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devMANAGER_FCTPTR_SET_VAL)((obj)->fcts[ic0devSERV_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0devServVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devMANAGER_FCTPTR_CFG_ELEM)((obj)->fcts[ic0devSERV_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0devServVLogConf(obj,devName,allElements,error) \
	(*((ic0devMANAGER_FCTPTR_LOG_CONF)((obj)->fcts[ic0devSERV_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0devServVGetConf(obj,devName,allElements,text,error) \
	(*((ic0devMANAGER_FCTPTR_GET_CONF)((obj)->fcts[ic0devSERV_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0devServVDevName(obj,text,error) \
	(*((ic0devMANAGER_FCTPTR_DEV_NAME)((obj)->fcts[ic0devSERV_FCT_DEV_NAME]))) \
	 ((obj),(text),(error))
#define ic0devServVDevObj(obj,devName,devAddr,error) \
	(*((ic0devMANAGER_FCTPTR_DEV_OBJ)((obj)->fcts[ic0devSERV_FCT_DEV_OBJ]))) \
	 ((obj),(devName),(devAddr),(error))
#define ic0devServVMonitor(obj,devName,error) \
	(*((ic0devMANAGER_FCTPTR_MONITOR)((obj)->fcts[ic0devSERV_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0devServVInitObj(obj,error) \
	(*((ic0devMANAGER_FCTPTR_INIT_OBJ)((obj)->fcts[ic0devSERV_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0devServVExitObj(obj,error) \
	(*((ic0devMANAGER_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0devSERV_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0devServVUsrVarSet(obj,syncVar,value,varNum,done,error) \
	(*((ic0devMANAGER_FCTPTR_USR_VAR_SET)((obj)->fcts[ic0devSERV_FCT_USR_VAR_SET]))) \
	 ((obj),(syncVar),(value),(varNum),(done),(error))
#define ic0devServVUsrVarWait(obj,syncVar,value,timeout,varNum,done,error) \
	(*((ic0devMANAGER_FCTPTR_USR_VAR_WAIT)((obj)->fcts[ic0devSERV_FCT_USR_VAR_WAIT]))) \
	 ((obj),(syncVar),(value),(timeout),(varNum),(done),(error))
#define ic0devServVServer(obj,error) \
	(*((ic0devMANAGER_FCTPTR_SERVER)((obj)->fcts[ic0devSERV_FCT_SERVER]))) \
	 ((obj),(error))
#define ic0devServVLookup(obj,devName,devObj,error) \
	(*((ic0devMANAGER_FCTPTR_LOOKUP)((obj)->fcts[ic0devSERV_FCT_LOOKUP]))) \
	 ((obj),(devName),(devObj),(error))


ccsCOMPL_STAT ic0devServNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0devServDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DEV_SERV_H

/************************************************************************/
/* ___oOo___ */
