/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0digConfig.h,v 1.35 2005/09/09 18:58:43 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   22/05/97  Created by eccsgen  Revision: 1.15
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0digConfig - class
*
*   SYNOPSIS
*	#include "ic0digConfig.h"
*
*	ccsCOMPL_STAT ic0digConfigNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0digConfigGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0digConfigLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0digConfigStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	function[],	** max. vltBYTES32[999]
*	    IN    vltINT16	functionNum,
*	    IN    vltLOGICAL	flag,
*	    IN    vltLOGICAL	header,
*	    IN    vltLOGICAL	conflog,
*	    IN    vltLOGICAL	opslog,
*	    IN    vltLOGICAL	condlog,
*	    OUT   char**	status,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigSetup(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	function[],	** max. vltBYTES32[999]
*	    IN    vltINT16	functionNum,
*	    IN    vltLOGICAL	noMove,
*	    IN    vltLOGICAL	check,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigSet(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digConfigWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	public ic0digSOFT
*
*   DESCRIPTION
*       The configuration of the digital signals can be stored in the
*       tables of the individual devices.  An alternative is to store
*       the configuration of _all_ signals in a single table. This
*       class can be used to store such a table. ic0digCONFIG is not
*       related to any individual hardware.
*
*	ic0digConfigNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0digCONFIG.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0digConfigDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0digCONFIG
*
*	ic0digConfigStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error)
*	   Updates the complete status in the local database
*	   and returns the relevant status of the software
*	   device.
*
*	   The status is given as lines containing two words
*	   each:
*	     key1  value1,
*	     [key2 value2,
*	     [...]]
*
*	   The value usually begins after the 18th character
*	   of each line. If the flag option is set, then the
*	   values are returned as numbers (by default values
*	   are returned as literals).
*
*	ic0digConfigSetup(obj,devName,function,functionNum,noMove,check,error)
*	   Setups the function. The keys are the last component
*	   of the FITS keywords related with the function.
*	   E.g.:
*	     If the FITS keyword is: INS.OPTI-3.ID #11
*	     then the key is: ID, the value '#11'.
*
*	   If the noMove flag is used, then the setup parameters
*	   are stored, but not executed until a setup command
*	   without the noMove flag is received.
*
*	   If the check flag is used, then only the parameters
*	   are checked. No setup is performed.
*
*	   Aliases are not accepted (The ICS Frontend at the
*	   workstation recognizes the aliases).
*
*	ic0digConfigSet(obj,devName,position,positionNum,error)
*	   Set a hardware device in a specified state
*	   (e.g. set lamp on, set shutter open).
*
*
*	ic0digConfigErr<error>(...)
*	    The macros ic0digConfigErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0digConfigRead<attr>(...)
*	ic0digConfigWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0digConfigDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DIG_CONFIG_H
#define IC0DIG_CONFIG_H

#include "egen.h"
#include "ic0digErrors.h"
#include "ic0digSoft.h"

/*
 * class type identifier
 */
#define ic0digCONFIG_ID	181

/*
 * attribute table
 */
#define ic0digCONFIG_ATTR_STATE	0
#define ic0digCONFIG_ATTR_SUBSTATE	1
#define ic0digCONFIG_ATTR_ERROR	2
#define ic0digCONFIG_ATTR_INITIALISED	3
#define ic0digCONFIG_ATTR_SIMULATION	4
#define ic0digCONFIG_ATTR_DEVICE	5
#define ic0digCONFIG_ATTR_AVAILABLE	6
#define ic0digCONFIG_ATTR_ENABLED	7
#define ic0digCONFIG_ATTR_LOCKED	8
#define ic0digCONFIG_ATTR_HANDSET	9
#define ic0digCONFIG_ATTR_FIXED	10
#define ic0digCONFIG_ATTR_FIX_POS_SET	11
#define ic0digCONFIG_ATTR_FIXED_POSITION	12
#define ic0digCONFIG_ATTR_FIXED_SIMULATION	13
#define ic0digCONFIG_ATTR_FIXED_LOCKED	14
#define ic0digCONFIG_ATTR_PREFIX	15
#define ic0digCONFIG_ATTR_PARAMETERS_SIZE	16
#define ic0digCONFIG_ATTR_PARAMETERS	17
#define ic0digCONFIG_ATTR_STATUS	18
#define ic0digCONFIG_ATTR_STATUS_INDEX	19
#define ic0digCONFIG_ATTR_VALUES_SIZE	20
#define ic0digCONFIG_ATTR_VALUES	21
#define ic0digCONFIG_ATTR_INIT_RESTRICTION	22
#define ic0digCONFIG_ATTR_MOVE_RESTRICTION	23
#define ic0digCONFIG_ATTR_STOP_TIMEOUT	24
#define ic0digCONFIG_ATTR_SET_VAL_FCT	25
#define ic0digCONFIG_ATTR_HELP_TEXT	26
#define ic0digCONFIG_ATTR_PARAM_STEP	27
#define ic0digCONFIG_ATTR_ACTION_COMMAND	28
#define ic0digCONFIG_ATTR_ACTION_STOP_REQ	29
#define ic0digCONFIG_ATTR_ACTION_NO_MOVE	30
#define ic0digCONFIG_ATTR_ACTION_CHECK	31
#define ic0digCONFIG_ATTR_RESET_SETUP_CMD	32
#define ic0digCONFIG_ATTR_ID	33
#define ic0digCONFIG_ATTR_NAME	34
#define ic0digCONFIG_ATTR_SIGNALS	35
#define ic0digCONFIG_ATTR_CONFIGURED	36
#define ic0digCONFIG_ATTR_HW_ACCESS	37

#define ic0digCONFIG_AUTOMATA	0

/*
 * function table
 */
#define ic0digCONFIG_FCT_DELETE_OBJ	0
#define ic0digCONFIG_FCT_STANDBY	1
#define ic0digCONFIG_FCT_ONLINE	2
#define ic0digCONFIG_FCT_OFF	3
#define ic0digCONFIG_FCT_EXIT	4
#define ic0digCONFIG_FCT_STATE	5
#define ic0digCONFIG_FCT_SIMULAT	6
#define ic0digCONFIG_FCT_STOP_SIM	7
#define ic0digCONFIG_FCT_STATUS	8
#define ic0digCONFIG_FCT_INIT	9
#define ic0digCONFIG_FCT_STOP	10
#define ic0digCONFIG_FCT_TEST	11
#define ic0digCONFIG_FCT_SELFTST	12
#define ic0digCONFIG_FCT_AUTO_TBL	13
#define ic0digCONFIG_FCT_AUTO_RST	14
#define ic0digCONFIG_FCT_INIT_OBJ	15
#define ic0digCONFIG_FCT_EXIT_OBJ	16
#define ic0digCONFIG_FCT_LOADED2STBY	17
#define ic0digCONFIG_FCT_ONLINE2STBY	18
#define ic0digCONFIG_FCT_EXIT1	19
#define ic0digCONFIG_FCT_CHECK_DEVICE_NAME	20
#define ic0digCONFIG_FCT_BREAK	21
#define ic0digCONFIG_FCT_USR_INIT	22
#define ic0digCONFIG_FCT_USR_LOADED2STANDBY	23
#define ic0digCONFIG_FCT_USR_STANDBY2ONLINE	24
#define ic0digCONFIG_FCT_USR_ONLINE2STANDBY	25
#define ic0digCONFIG_FCT_USR_STANDBY2LOADED	26
#define ic0digCONFIG_FCT_USR_STOP	27
#define ic0digCONFIG_FCT_USR_SIMULAT	28
#define ic0digCONFIG_FCT_USR_STOP_SIM	29
#define ic0digCONFIG_FCT_USR_STATUS_CODE	30
#define ic0digCONFIG_FCT_USR_STATUS_LAST	31
#define ic0digCONFIG_FCT_USR_STATUS_VALUE	32
#define ic0digCONFIG_FCT_USR_TEST	33
#define ic0digCONFIG_FCT_USR_SELF_TEST	34
#define ic0digCONFIG_FCT_USR_ERROR	35
#define ic0digCONFIG_FCT_AVAIL	36
#define ic0digCONFIG_FCT_UNAVAIL	37
#define ic0digCONFIG_FCT_DISABLE	38
#define ic0digCONFIG_FCT_ENABLE	39
#define ic0digCONFIG_FCT_LOCK	40
#define ic0digCONFIG_FCT_UNLOCK	41
#define ic0digCONFIG_FCT_HANDSET	42
#define ic0digCONFIG_FCT_HSET_OFF	43
#define ic0digCONFIG_FCT_FIX_POS	44
#define ic0digCONFIG_FCT_MOVE	45
#define ic0digCONFIG_FCT_MOVE_REL	46
#define ic0digCONFIG_FCT_SET	47
#define ic0digCONFIG_FCT_CHECK	48
#define ic0digCONFIG_FCT_WAIT	49
#define ic0digCONFIG_FCT_USR_SET_MODE	50
#define ic0digCONFIG_FCT_SETUP	51
#define ic0digCONFIG_FCT_GET_VAL	52
#define ic0digCONFIG_FCT_SET_VAL	53
#define ic0digCONFIG_FCT_CFG_ELEM	54
#define ic0digCONFIG_FCT_LOG_CONF	55
#define ic0digCONFIG_FCT_GET_CONF	56
#define ic0digCONFIG_FCT_RESET_PARAMETERS	57
#define ic0digCONFIG_FCT_PARSE_PARAMETERS	58
#define ic0digCONFIG_FCT_COMPARE_PARAMETERS	59
#define ic0digCONFIG_FCT_COPY_PARAMETERS	60
#define ic0digCONFIG_FCT_EXEC_PARAMETERS	61
#define ic0digCONFIG_FCT_UPDATE_PARAMETERS	62
#define ic0digCONFIG_FCT_GET_PARAM_SET_VALUE	63
#define ic0digCONFIG_FCT_GET_PARAM_STATUS	64
#define ic0digCONFIG_FCT_SET_PARAM	65
#define ic0digCONFIG_FCT_SET_PARAM_STEP	66
#define ic0digCONFIG_FCT_USR_EXEC_ACTION	67
#define ic0digCONFIG_FCT_USR_SET_PARAMETER	68
#define ic0digCONFIG_FCT_GET_VALUE_CODE	69
#define ic0digCONFIG_FCT_GET_VALUE	70
#define ic0digCONFIG_FCT_USR_GET_VALUE	71
#define ic0digCONFIG_FCT_SET_VALUE	72
#define ic0digCONFIG_FCT_USR_SET_VALUE	73
#define ic0digCONFIG_FCT_GET_ARG	74
#define ic0digCONFIG_FCT_MONITOR	75
#define ic0digCONFIG_FCT_USR_MONITOR	76
#define ic0digCONFIG_FCT_LOG_ACTION	77
#define ic0digCONFIG_FCT_LOG_COMMENT	78
#define ic0digCONFIG_FCT_LOG_PARAMETER	79
#define ic0digCONFIG_FCT_LOG_RECOVERY	80
#define ic0digCONFIG_FCT_LOG_TBL_PARAM	81
#define ic0digCONFIG_FCT_LOG_UNFORSEEN	82

/*
 * automata states
 */
#define ic0digCONFIG_STATE_NOSTATE	-1
#define ic0digCONFIG_STATE_OFF	0
#define ic0digCONFIG_STATE_LOADED	1
#define ic0digCONFIG_STATE_STANDBY	2
#define ic0digCONFIG_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0digCONFIG_SUBSTATE_ERROR	0
#define ic0digCONFIG_SUBSTATE_IDLE	1
#define ic0digCONFIG_SUBSTATE_BUSY	2
#define ic0digCONFIG_SUBSTATE_INITIALISING	3
#define ic0digCONFIG_SUBSTATE_SHUTTING_DOWN	4
#define ic0digCONFIG_SUBSTATE_STOPPING	5
#define ic0digCONFIG_SUBSTATE_COLD_STARTING	6
#define ic0digCONFIG_SUBSTATE_STARTING	7
#define ic0digCONFIG_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0digCONFIG_EVENT_INIT	0
#define ic0digCONFIG_EVENT_STANDBY	1
#define ic0digCONFIG_EVENT_ONLINE	2
#define ic0digCONFIG_EVENT_STOP	3
#define ic0digCONFIG_EVENT_OFF	4
#define ic0digCONFIG_EVENT_EXIT	5
#define ic0digCONFIG_EVENT_STATE	6
#define ic0digCONFIG_EVENT_STOP_SIM	7
#define ic0digCONFIG_EVENT_STATUS	8
#define ic0digCONFIG_EVENT_SIMULAT	9
#define ic0digCONFIG_EVENT_TEST	10
#define ic0digCONFIG_EVENT_SELFTST	11
#define ic0digCONFIG_EVENT_AUTO_TBL	12
#define ic0digCONFIG_EVENT_AUTO_RST	13
#define ic0digCONFIG_EVENT_DISABLE	14
#define ic0digCONFIG_EVENT_ENABLE	15
#define ic0digCONFIG_EVENT_LOCK	16
#define ic0digCONFIG_EVENT_UNLOCK	17
#define ic0digCONFIG_EVENT_HANDSET	18
#define ic0digCONFIG_EVENT_HSET_OFF	19
#define ic0digCONFIG_EVENT_AVAIL	20
#define ic0digCONFIG_EVENT_UNAVAIL	21
#define ic0digCONFIG_EVENT_MOVE	22
#define ic0digCONFIG_EVENT_MOVE_REL	23
#define ic0digCONFIG_EVENT_SET	24
#define ic0digCONFIG_EVENT_CHECK	25
#define ic0digCONFIG_EVENT_WAIT	26
#define ic0digCONFIG_EVENT_SETUP	27
#define ic0digCONFIG_EVENT_CFG_ELEM	28
#define ic0digCONFIG_EVENT_GET_VAL	29
#define ic0digCONFIG_EVENT_SET_VAL	30
#define ic0digCONFIG_EVENT_LOG_CONF	31
#define ic0digCONFIG_EVENT_GET_CONF	32
#define ic0digCONFIG_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0digCONFIG_FCTPTR_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digCONFIG_FCTPTR_SETUP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digCONFIG_FCTPTR_SET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0digConfigLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dig", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0digConfigGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0digConfigReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_STATE,(buf),(error))
#define ic0digConfigWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_STATE,(void*) (buf),(error))
#define ic0digConfigReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_SUBSTATE,(buf),(error))
#define ic0digConfigWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0digConfigReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ERROR,(buf),(error))
#define ic0digConfigWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ERROR,(void*) (buf),(error))
#define ic0digConfigReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_INITIALISED,(buf),(error))
#define ic0digConfigWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0digConfigReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_SIMULATION,(buf),(error))
#define ic0digConfigWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0digConfigReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_DEVICE,(buf),(error))
#define ic0digConfigWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_DEVICE,(void*) (buf),(error))
#define ic0digConfigReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_AVAILABLE,(buf),(error))
#define ic0digConfigWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0digConfigReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ENABLED,(buf),(error))
#define ic0digConfigWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ENABLED,(void*) (buf),(error))
#define ic0digConfigReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_LOCKED,(buf),(error))
#define ic0digConfigWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_LOCKED,(void*) (buf),(error))
#define ic0digConfigReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_HANDSET,(buf),(error))
#define ic0digConfigWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_HANDSET,(void*) (buf),(error))
#define ic0digConfigReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_FIXED,(buf),(error))
#define ic0digConfigWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_FIXED,(void*) (buf),(error))
#define ic0digConfigReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_FIX_POS_SET,(buf),(error))
#define ic0digConfigWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0digConfigReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_FIXED_POSITION,(buf),(error))
#define ic0digConfigWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0digConfigReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0digConfigWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0digConfigReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0digConfigWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0digConfigReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_PREFIX,(buf),(error))
#define ic0digConfigWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_PREFIX,(void*) (buf),(error))
#define ic0digConfigReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0digConfigWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0digConfigGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digCONFIG_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digCONFIG_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digCONFIG_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0digConfigWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digCONFIG_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digConfigGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digCONFIG_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0digConfigWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digCONFIG_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digConfigReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_VALUES_SIZE,(buf),(error))
#define ic0digConfigWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0digConfigGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digCONFIG_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digCONFIG_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0digConfigWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0digConfigReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0digConfigWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0digConfigReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0digConfigWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0digConfigReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0digConfigWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0digConfigGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digCONFIG_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0digConfigWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digCONFIG_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digConfigReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_PARAM_STEP,(buf),(error))
#define ic0digConfigWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0digConfigReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0digConfigWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0digConfigReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0digConfigWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0digConfigReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0digConfigWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0digConfigReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ACTION_CHECK,(buf),(error))
#define ic0digConfigWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0digConfigReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0digConfigWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0digConfigReadId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_ID,(buf),(error))
#define ic0digConfigWriteId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_ID,(void*) (buf),(error))
#define ic0digConfigReadName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_NAME,(buf),(error))
#define ic0digConfigWriteName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_NAME,(void*) (buf),(error))
#define ic0digConfigGetSignalsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digCONFIG_ATTR_SIGNALS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digConfigReadSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digCONFIG_ATTR_SIGNALS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigWriteSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digCONFIG_ATTR_SIGNALS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digConfigReadConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_CONFIGURED,(buf),(error))
#define ic0digConfigWriteConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_CONFIGURED,(void*) (buf),(error))
#define ic0digConfigReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digCONFIG_ATTR_HW_ACCESS,(buf),(error))
#define ic0digConfigWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digCONFIG_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0digConfigGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0digCONFIG_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0digConfigMove(obj,devName,position,positionNum,error) \
	ic0digSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0digConfigMoveRel(obj,devName,position,positionNum,error) \
	ic0digSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0digConfigGetConf(obj,devName,allElements,text,error) \
	ic0digSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0digConfigUsrInit(obj,devName,error) \
	ic0digSoftUsrInit((obj),(devName),(error))
#define ic0digConfigUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0digSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0digConfigGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digConfigSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digConfigCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digConfigLogConf(obj,devName,allElements,error) \
	ic0devSoftLogConf((obj),(devName),(allElements),(error))
#define ic0digConfigResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0digConfigParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digConfigCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0digConfigCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0digConfigExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0digConfigUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0digConfigGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0digConfigGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0digConfigSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digConfigSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0digConfigUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0digConfigUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digConfigGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0digConfigGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digConfigUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digConfigSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digConfigUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digConfigGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0digConfigMonitor(obj,devName,error) \
	ic0devSoftMonitor((obj),(devName),(error))
#define ic0digConfigUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0digConfigLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0digConfigLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0digConfigLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digConfigLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0digConfigLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digConfigLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0digConfigUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0digConfigInitObj(obj,error) \
	ic0devSoftInitObj((obj),(error))
#define ic0digConfigExitObj(obj,error) \
	ic0devSoftExitObj((obj),(error))
#define ic0digConfigAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0digConfigUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0digConfigDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0digConfigEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0digConfigLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0digConfigUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0digConfigHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0digConfigHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0digConfigFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digConfigCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0digConfigWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0digConfigUsrStatusCode(obj,devName,text,flags,code,error) \
	lccdevCmdUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0digConfigUsrStatusValue(obj,devName,code,value,error) \
	lccdevCmdUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0digConfigUsrStatusLast(obj,devName,last,error) \
	lccdevCmdUsrStatusLast((obj),(devName),(last),(error))
#define ic0digConfigUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0digConfigUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0digConfigUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0digConfigUsrStandby2Online(obj,devName,error) \
	lccdevCmdUsrStandby2Online((obj),(devName),(error))
#define ic0digConfigStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0digConfigOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0digConfigOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0digConfigExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0digConfigState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0digConfigSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0digConfigStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0digConfigInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0digConfigStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0digConfigTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0digConfigSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0digConfigAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0digConfigAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0digConfigLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0digConfigOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0digConfigExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0digConfigCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0digConfigBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0digConfigUsrOnline2Standby(obj,devName,error) \
	lccdevStdUsrOnline2Standby((obj),(devName),(error))
#define ic0digConfigUsrStandby2Loaded(obj,devName,error) \
	lccdevStdUsrStandby2Loaded((obj),(devName),(error))
#define ic0digConfigUsrTest(obj,devName,error) \
	lccdevStdUsrTest((obj),(devName),(error))
#define ic0digConfigUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0digConfigUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0digConfigStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digConfigSetup(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digConfigSet(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0digConfigVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0digCONFIG_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0digCONFIG_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0digCONFIG_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0digCONFIG_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0digCONFIG_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0digConfigVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0digCONFIG_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0digCONFIG_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0digCONFIG_FCTPTR_STATUS)((obj)->fcts[ic0digCONFIG_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0digConfigVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0digCONFIG_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0digCONFIG_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0digCONFIG_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0digCONFIG_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0digCONFIG_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0digConfigVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0digCONFIG_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVInitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0digCONFIG_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0digConfigVExitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0digCONFIG_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0digConfigVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0digCONFIG_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0digCONFIG_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0digCONFIG_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0digCONFIG_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0digCONFIG_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0digConfigVUsrInit(obj,devName,error) \
	(*((ic0digSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0digCONFIG_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0digCONFIG_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrStandby2Online(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0digCONFIG_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrOnline2Standby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0digCONFIG_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrStandby2Loaded(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0digCONFIG_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0digCONFIG_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0digConfigVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0digCONFIG_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0digCONFIG_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0digCONFIG_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0digConfigVUsrStatusLast(obj,devName,last,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0digCONFIG_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0digConfigVUsrStatusValue(obj,devName,code,value,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0digCONFIG_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0digConfigVUsrTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_TEST)((obj)->fcts[ic0digCONFIG_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0digCONFIG_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0digCONFIG_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0digCONFIG_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0digCONFIG_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0digCONFIG_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digConfigVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0digCONFIG_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digConfigVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0digCONFIG_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0digCONFIG_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0digCONFIG_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0digCONFIG_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0digCONFIG_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digConfigVMove(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE)((obj)->fcts[ic0digCONFIG_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digConfigVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0digCONFIG_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digConfigVSet(obj,devName,position,positionNum,error) \
	(*((ic0digCONFIG_FCTPTR_SET)((obj)->fcts[ic0digCONFIG_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digConfigVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0digCONFIG_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0digConfigVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0digCONFIG_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0digConfigVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0digSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0digCONFIG_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0digConfigVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0digCONFIG_FCTPTR_SETUP)((obj)->fcts[ic0digCONFIG_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0digConfigVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0digCONFIG_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digConfigVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0digCONFIG_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digConfigVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0digCONFIG_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digConfigVLogConf(obj,devName,allElements,error) \
	(*((ic0devSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0digCONFIG_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0digConfigVGetConf(obj,devName,allElements,text,error) \
	(*((ic0digSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0digCONFIG_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0digConfigVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0digConfigVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digConfigVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0digConfigVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0digCONFIG_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0digConfigVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0digCONFIG_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0digConfigVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0digCONFIG_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0digConfigVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0digCONFIG_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digConfigVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0digCONFIG_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0digConfigVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0digCONFIG_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0digConfigVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0digCONFIG_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digConfigVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0digCONFIG_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0digConfigVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0digCONFIG_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digConfigVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0digCONFIG_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digConfigVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0digCONFIG_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digConfigVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0digCONFIG_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digConfigVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0digCONFIG_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0digConfigVMonitor(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0digCONFIG_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0digConfigVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0digCONFIG_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0digConfigVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0digCONFIG_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0digConfigVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0digCONFIG_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digConfigVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0digCONFIG_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digConfigVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0digCONFIG_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digConfigVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0digCONFIG_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digConfigVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0digCONFIG_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))


ccsCOMPL_STAT ic0digConfigNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digConfigDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DIG_CONFIG_H

/************************************************************************/
/* ___oOo___ */
