/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0digSignal.h,v 1.35 2005/09/09 18:58:44 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   23/06/97  Created by eccsgen  Revision: 1.15
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0digSignal - class
*
*   SYNOPSIS
*	#include "ic0digSignal.h"
*
*	ccsCOMPL_STAT ic0digSignalNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0digSignalGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0digSignalLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0digSignalSetup(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	function[],	** max. vltBYTES32[999]
*	    IN    vltINT16	functionNum,
*	    IN    vltLOGICAL	noMove,
*	    IN    vltLOGICAL	check,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalSet(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrMonitor(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	state,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrSelfTest(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrTest(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrStatusCode(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	text,
*	    IN    vltINT32	flags,
*	    OUT   vltINT32*	code,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrStatusValue(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	code,
*	    OUT   lcctooVALUE*	value,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalUsrStatusLast(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   vltINT16*	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSignalWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	public ic0digSOFT
*
*   DESCRIPTION
*       This device manages a single digital signal.
*
*	ic0digSignalNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0digSIGNAL.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0digSignalDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0digSIGNAL
*
*	ic0digSignalSetup(obj,devName,function,functionNum,noMove,check,error)
*	   Setups the function. The keys are the last component
*	   of the FITS keywords related with the function.
*	   E.g.:
*	     If the FITS keyword is: INS.OPTI-3.ID #11
*	     then the key is: ID, the value '#11'.
*
*	   If the noMove flag is used, then the setup parameters
*	   are stored, but not executed until a setup command
*	   without the noMove flag is received.
*
*	   If the check flag is used, then only the parameters
*	   are checked. No setup is performed.
*
*	   Aliases are not accepted (The ICS Frontend at the
*	   workstation recognizes the aliases).
*
*	ic0digSignalSet(obj,devName,position,positionNum,error)
*	   Set a hardware device in a specified state
*	   (e.g. set lamp on, set shutter open).
*
*
*	ic0digSignalErr<error>(...)
*	    The macros ic0digSignalErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0digSignalRead<attr>(...)
*	ic0digSignalWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0digSignalDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0digSignalUsrMonitor(obj,devName,state,error)
*	   This function is periodically called by the device
*	   manager. This function may be overloaded to monitor
*	   the device.
*
*	ic0digSignalUsrSelfTest(obj,devName,error)
*	   This function should be overloaded to perform a
*	   selftest of the device.  No movements are allowed
*	   (it must be possible to execute this function in
*	   the state LOADED).
*
*	ic0digSignalUsrTest(obj,devName,error)
*	   This function should be overloaded to perform the
*	   actions necessary to test the software device
*	   (eventually with hardware movements).
*
*	ic0digSignalUsrStatusCode(obj,devName,text,flags,code,error)
*	   This function receives the name of a status value
*	   and returns the corresponding code. This function
*	   should be overloaded if the new class declares new
*	   status values. The overloaded function should check
*	   first if the name corresponds to one of its own
*	   status values. If not, then it should call the
*	   function UsrStatusCode() of its parent class to be
*	   able to retrieve the codes of the parent status
*	   values.
*
*	   text	<IN>	Name of the status value.
*	   flags	<IN>	expected status type (lcctooSTATUS_*).
*	   code	<OUT>	Code of the status value (lccdevGET_*).
*
*	   lccdevGET_DEVICE        "Device"
*	   lccdevGET_STATE         "State"
*	   lccdevGET_TRANSTATE     "TranState"
*	   lccdevGET_ERROR         "Error"
*	   lccdevGET_INITIALISED   "Initialised"
*	   lccdevGET_SIMULATION    "Simulation"
*
*	ic0digSignalUsrStatusValue(obj,devName,code,value,error)
*	   This function receives the code of a status variable
*	   and returns its related value. This function should
*	   be overloaded if the new class declares new status
*	   values. The overloaded function should check first
*	   if the code corresponds to one of its own status
*	   values. If not, then it should call the function
*	   UsrStatusValue() of its parent class to be able to
*	   retrieve the status values of the parent.
*
*	   code	<OUT>	Code of the status value (lccdevGET_*).
*	   value	<OUT>	Handle of the value object
*			   (allocated by the caller).
*
*	ic0digSignalUsrStatusLast(obj,devName,last,error)
*	   This function returns the highest status code that
*	   the class knows. This function should be overloaded
*	   if the new class declares new status values, as new
*	   status codes have to be assigned to the new values.
*
*	   This function is used by the Status command when it
*	   has to return all status values. In this case it
*	   performs a loop from code 1 to the value returned
*	   by this function.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DIG_SIGNAL_H
#define IC0DIG_SIGNAL_H

#include "egen.h"
#include "ic0digDefines.h"
#include "ic0digErrors.h"
#include "ic0digSoft.h"

/*
 * class type identifier
 */
#define ic0digSIGNAL_ID	184

/*
 * attribute table
 */
#define ic0digSIGNAL_ATTR_STATE	0
#define ic0digSIGNAL_ATTR_SUBSTATE	1
#define ic0digSIGNAL_ATTR_ERROR	2
#define ic0digSIGNAL_ATTR_INITIALISED	3
#define ic0digSIGNAL_ATTR_SIMULATION	4
#define ic0digSIGNAL_ATTR_DEVICE	5
#define ic0digSIGNAL_ATTR_AVAILABLE	6
#define ic0digSIGNAL_ATTR_ENABLED	7
#define ic0digSIGNAL_ATTR_LOCKED	8
#define ic0digSIGNAL_ATTR_HANDSET	9
#define ic0digSIGNAL_ATTR_FIXED	10
#define ic0digSIGNAL_ATTR_FIX_POS_SET	11
#define ic0digSIGNAL_ATTR_FIXED_POSITION	12
#define ic0digSIGNAL_ATTR_FIXED_SIMULATION	13
#define ic0digSIGNAL_ATTR_FIXED_LOCKED	14
#define ic0digSIGNAL_ATTR_PREFIX	15
#define ic0digSIGNAL_ATTR_PARAMETERS_SIZE	16
#define ic0digSIGNAL_ATTR_PARAMETERS	17
#define ic0digSIGNAL_ATTR_STATUS	18
#define ic0digSIGNAL_ATTR_STATUS_INDEX	19
#define ic0digSIGNAL_ATTR_VALUES_SIZE	20
#define ic0digSIGNAL_ATTR_VALUES	21
#define ic0digSIGNAL_ATTR_INIT_RESTRICTION	22
#define ic0digSIGNAL_ATTR_MOVE_RESTRICTION	23
#define ic0digSIGNAL_ATTR_STOP_TIMEOUT	24
#define ic0digSIGNAL_ATTR_SET_VAL_FCT	25
#define ic0digSIGNAL_ATTR_HELP_TEXT	26
#define ic0digSIGNAL_ATTR_PARAM_STEP	27
#define ic0digSIGNAL_ATTR_ACTION_COMMAND	28
#define ic0digSIGNAL_ATTR_ACTION_STOP_REQ	29
#define ic0digSIGNAL_ATTR_ACTION_NO_MOVE	30
#define ic0digSIGNAL_ATTR_ACTION_CHECK	31
#define ic0digSIGNAL_ATTR_RESET_SETUP_CMD	32
#define ic0digSIGNAL_ATTR_ID	33
#define ic0digSIGNAL_ATTR_NAME	34
#define ic0digSIGNAL_ATTR_SIGNALS	35
#define ic0digSIGNAL_ATTR_CONFIGURED	36
#define ic0digSIGNAL_ATTR_HW_ACCESS	37
#define ic0digSIGNAL_ATTR_VALUE_I	38
#define ic0digSIGNAL_ATTR_VALUE_I_SIM	39
#define ic0digSIGNAL_ATTR_VALUE_I_STATUS	40

#define ic0digSIGNAL_AUTOMATA	0

/*
 * function table
 */
#define ic0digSIGNAL_FCT_DELETE_OBJ	0
#define ic0digSIGNAL_FCT_STANDBY	1
#define ic0digSIGNAL_FCT_ONLINE	2
#define ic0digSIGNAL_FCT_OFF	3
#define ic0digSIGNAL_FCT_EXIT	4
#define ic0digSIGNAL_FCT_STATE	5
#define ic0digSIGNAL_FCT_SIMULAT	6
#define ic0digSIGNAL_FCT_STOP_SIM	7
#define ic0digSIGNAL_FCT_STATUS	8
#define ic0digSIGNAL_FCT_INIT	9
#define ic0digSIGNAL_FCT_STOP	10
#define ic0digSIGNAL_FCT_TEST	11
#define ic0digSIGNAL_FCT_SELFTST	12
#define ic0digSIGNAL_FCT_AUTO_TBL	13
#define ic0digSIGNAL_FCT_AUTO_RST	14
#define ic0digSIGNAL_FCT_INIT_OBJ	15
#define ic0digSIGNAL_FCT_EXIT_OBJ	16
#define ic0digSIGNAL_FCT_LOADED2STBY	17
#define ic0digSIGNAL_FCT_ONLINE2STBY	18
#define ic0digSIGNAL_FCT_EXIT1	19
#define ic0digSIGNAL_FCT_CHECK_DEVICE_NAME	20
#define ic0digSIGNAL_FCT_BREAK	21
#define ic0digSIGNAL_FCT_USR_INIT	22
#define ic0digSIGNAL_FCT_USR_LOADED2STANDBY	23
#define ic0digSIGNAL_FCT_USR_STANDBY2ONLINE	24
#define ic0digSIGNAL_FCT_USR_ONLINE2STANDBY	25
#define ic0digSIGNAL_FCT_USR_STANDBY2LOADED	26
#define ic0digSIGNAL_FCT_USR_STOP	27
#define ic0digSIGNAL_FCT_USR_SIMULAT	28
#define ic0digSIGNAL_FCT_USR_STOP_SIM	29
#define ic0digSIGNAL_FCT_USR_STATUS_CODE	30
#define ic0digSIGNAL_FCT_USR_STATUS_LAST	31
#define ic0digSIGNAL_FCT_USR_STATUS_VALUE	32
#define ic0digSIGNAL_FCT_USR_TEST	33
#define ic0digSIGNAL_FCT_USR_SELF_TEST	34
#define ic0digSIGNAL_FCT_USR_ERROR	35
#define ic0digSIGNAL_FCT_AVAIL	36
#define ic0digSIGNAL_FCT_UNAVAIL	37
#define ic0digSIGNAL_FCT_DISABLE	38
#define ic0digSIGNAL_FCT_ENABLE	39
#define ic0digSIGNAL_FCT_LOCK	40
#define ic0digSIGNAL_FCT_UNLOCK	41
#define ic0digSIGNAL_FCT_HANDSET	42
#define ic0digSIGNAL_FCT_HSET_OFF	43
#define ic0digSIGNAL_FCT_FIX_POS	44
#define ic0digSIGNAL_FCT_MOVE	45
#define ic0digSIGNAL_FCT_MOVE_REL	46
#define ic0digSIGNAL_FCT_SET	47
#define ic0digSIGNAL_FCT_CHECK	48
#define ic0digSIGNAL_FCT_WAIT	49
#define ic0digSIGNAL_FCT_USR_SET_MODE	50
#define ic0digSIGNAL_FCT_SETUP	51
#define ic0digSIGNAL_FCT_GET_VAL	52
#define ic0digSIGNAL_FCT_SET_VAL	53
#define ic0digSIGNAL_FCT_CFG_ELEM	54
#define ic0digSIGNAL_FCT_LOG_CONF	55
#define ic0digSIGNAL_FCT_GET_CONF	56
#define ic0digSIGNAL_FCT_RESET_PARAMETERS	57
#define ic0digSIGNAL_FCT_PARSE_PARAMETERS	58
#define ic0digSIGNAL_FCT_COMPARE_PARAMETERS	59
#define ic0digSIGNAL_FCT_COPY_PARAMETERS	60
#define ic0digSIGNAL_FCT_EXEC_PARAMETERS	61
#define ic0digSIGNAL_FCT_UPDATE_PARAMETERS	62
#define ic0digSIGNAL_FCT_GET_PARAM_SET_VALUE	63
#define ic0digSIGNAL_FCT_GET_PARAM_STATUS	64
#define ic0digSIGNAL_FCT_SET_PARAM	65
#define ic0digSIGNAL_FCT_SET_PARAM_STEP	66
#define ic0digSIGNAL_FCT_USR_EXEC_ACTION	67
#define ic0digSIGNAL_FCT_USR_SET_PARAMETER	68
#define ic0digSIGNAL_FCT_GET_VALUE_CODE	69
#define ic0digSIGNAL_FCT_GET_VALUE	70
#define ic0digSIGNAL_FCT_USR_GET_VALUE	71
#define ic0digSIGNAL_FCT_SET_VALUE	72
#define ic0digSIGNAL_FCT_USR_SET_VALUE	73
#define ic0digSIGNAL_FCT_GET_ARG	74
#define ic0digSIGNAL_FCT_MONITOR	75
#define ic0digSIGNAL_FCT_USR_MONITOR	76
#define ic0digSIGNAL_FCT_LOG_ACTION	77
#define ic0digSIGNAL_FCT_LOG_COMMENT	78
#define ic0digSIGNAL_FCT_LOG_PARAMETER	79
#define ic0digSIGNAL_FCT_LOG_RECOVERY	80
#define ic0digSIGNAL_FCT_LOG_TBL_PARAM	81
#define ic0digSIGNAL_FCT_LOG_UNFORSEEN	82

/*
 * automata states
 */
#define ic0digSIGNAL_STATE_NOSTATE	-1
#define ic0digSIGNAL_STATE_OFF	0
#define ic0digSIGNAL_STATE_LOADED	1
#define ic0digSIGNAL_STATE_STANDBY	2
#define ic0digSIGNAL_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0digSIGNAL_SUBSTATE_ERROR	0
#define ic0digSIGNAL_SUBSTATE_IDLE	1
#define ic0digSIGNAL_SUBSTATE_BUSY	2
#define ic0digSIGNAL_SUBSTATE_INITIALISING	3
#define ic0digSIGNAL_SUBSTATE_SHUTTING_DOWN	4
#define ic0digSIGNAL_SUBSTATE_STOPPING	5
#define ic0digSIGNAL_SUBSTATE_COLD_STARTING	6
#define ic0digSIGNAL_SUBSTATE_STARTING	7
#define ic0digSIGNAL_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0digSIGNAL_EVENT_INIT	0
#define ic0digSIGNAL_EVENT_STANDBY	1
#define ic0digSIGNAL_EVENT_ONLINE	2
#define ic0digSIGNAL_EVENT_STOP	3
#define ic0digSIGNAL_EVENT_OFF	4
#define ic0digSIGNAL_EVENT_EXIT	5
#define ic0digSIGNAL_EVENT_STATE	6
#define ic0digSIGNAL_EVENT_STOP_SIM	7
#define ic0digSIGNAL_EVENT_STATUS	8
#define ic0digSIGNAL_EVENT_SIMULAT	9
#define ic0digSIGNAL_EVENT_TEST	10
#define ic0digSIGNAL_EVENT_SELFTST	11
#define ic0digSIGNAL_EVENT_AUTO_TBL	12
#define ic0digSIGNAL_EVENT_AUTO_RST	13
#define ic0digSIGNAL_EVENT_DISABLE	14
#define ic0digSIGNAL_EVENT_ENABLE	15
#define ic0digSIGNAL_EVENT_LOCK	16
#define ic0digSIGNAL_EVENT_UNLOCK	17
#define ic0digSIGNAL_EVENT_HANDSET	18
#define ic0digSIGNAL_EVENT_HSET_OFF	19
#define ic0digSIGNAL_EVENT_AVAIL	20
#define ic0digSIGNAL_EVENT_UNAVAIL	21
#define ic0digSIGNAL_EVENT_MOVE	22
#define ic0digSIGNAL_EVENT_MOVE_REL	23
#define ic0digSIGNAL_EVENT_SET	24
#define ic0digSIGNAL_EVENT_CHECK	25
#define ic0digSIGNAL_EVENT_WAIT	26
#define ic0digSIGNAL_EVENT_SETUP	27
#define ic0digSIGNAL_EVENT_CFG_ELEM	28
#define ic0digSIGNAL_EVENT_GET_VAL	29
#define ic0digSIGNAL_EVENT_SET_VAL	30
#define ic0digSIGNAL_EVENT_LOG_CONF	31
#define ic0digSIGNAL_EVENT_GET_CONF	32
#define ic0digSIGNAL_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_SETUP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_SET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_MONITOR)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	state,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_SELF_TEST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_TEST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_STATUS_CODE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	text,
    IN    vltINT32	flags,
    OUT   vltINT32*	code,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_STATUS_VALUE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	code,
    OUT   lcctooVALUE*	value,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSIGNAL_FCTPTR_USR_STATUS_LAST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   vltINT16*	last,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0digSignalLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dig", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0digSignalGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0digSignalReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_STATE,(buf),(error))
#define ic0digSignalWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_STATE,(void*) (buf),(error))
#define ic0digSignalReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_SUBSTATE,(buf),(error))
#define ic0digSignalWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0digSignalReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ERROR,(buf),(error))
#define ic0digSignalWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ERROR,(void*) (buf),(error))
#define ic0digSignalReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_INITIALISED,(buf),(error))
#define ic0digSignalWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0digSignalReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_SIMULATION,(buf),(error))
#define ic0digSignalWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0digSignalReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_DEVICE,(buf),(error))
#define ic0digSignalWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_DEVICE,(void*) (buf),(error))
#define ic0digSignalReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_AVAILABLE,(buf),(error))
#define ic0digSignalWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0digSignalReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ENABLED,(buf),(error))
#define ic0digSignalWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ENABLED,(void*) (buf),(error))
#define ic0digSignalReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_LOCKED,(buf),(error))
#define ic0digSignalWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_LOCKED,(void*) (buf),(error))
#define ic0digSignalReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_HANDSET,(buf),(error))
#define ic0digSignalWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_HANDSET,(void*) (buf),(error))
#define ic0digSignalReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_FIXED,(buf),(error))
#define ic0digSignalWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_FIXED,(void*) (buf),(error))
#define ic0digSignalReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_FIX_POS_SET,(buf),(error))
#define ic0digSignalWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0digSignalReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_FIXED_POSITION,(buf),(error))
#define ic0digSignalWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0digSignalReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0digSignalWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0digSignalReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0digSignalWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0digSignalReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_PREFIX,(buf),(error))
#define ic0digSignalWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_PREFIX,(void*) (buf),(error))
#define ic0digSignalReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0digSignalWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0digSignalGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSIGNAL_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSIGNAL_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSIGNAL_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0digSignalWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSIGNAL_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSignalGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSIGNAL_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0digSignalWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSIGNAL_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSignalReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_VALUES_SIZE,(buf),(error))
#define ic0digSignalWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0digSignalGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSIGNAL_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSIGNAL_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0digSignalWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0digSignalReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0digSignalWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0digSignalReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0digSignalWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0digSignalReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0digSignalWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0digSignalGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSIGNAL_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0digSignalWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSIGNAL_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSignalReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_PARAM_STEP,(buf),(error))
#define ic0digSignalWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0digSignalReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0digSignalWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0digSignalReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0digSignalWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0digSignalReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0digSignalWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0digSignalReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ACTION_CHECK,(buf),(error))
#define ic0digSignalWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0digSignalReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0digSignalWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0digSignalReadId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_ID,(buf),(error))
#define ic0digSignalWriteId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_ID,(void*) (buf),(error))
#define ic0digSignalReadName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_NAME,(buf),(error))
#define ic0digSignalWriteName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_NAME,(void*) (buf),(error))
#define ic0digSignalGetSignalsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSIGNAL_ATTR_SIGNALS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSignalReadSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSIGNAL_ATTR_SIGNALS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalWriteSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSIGNAL_ATTR_SIGNALS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSignalReadConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_CONFIGURED,(buf),(error))
#define ic0digSignalWriteConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_CONFIGURED,(void*) (buf),(error))
#define ic0digSignalReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_HW_ACCESS,(buf),(error))
#define ic0digSignalWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0digSignalReadValueI(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_VALUE_I,(buf),(error))
#define ic0digSignalWriteValueI(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_VALUE_I,(void*) (buf),(error))
#define ic0digSignalReadValueISim(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_VALUE_I_SIM,(buf),(error))
#define ic0digSignalWriteValueISim(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_VALUE_I_SIM,(void*) (buf),(error))
#define ic0digSignalReadValueIStatus(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSIGNAL_ATTR_VALUE_I_STATUS,(buf),(error))
#define ic0digSignalWriteValueIStatus(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSIGNAL_ATTR_VALUE_I_STATUS,(void*) (buf),(error))
#define ic0digSignalGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0digSIGNAL_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0digSignalMove(obj,devName,position,positionNum,error) \
	ic0digSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0digSignalMoveRel(obj,devName,position,positionNum,error) \
	ic0digSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0digSignalGetConf(obj,devName,allElements,text,error) \
	ic0digSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0digSignalUsrInit(obj,devName,error) \
	ic0digSoftUsrInit((obj),(devName),(error))
#define ic0digSignalUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0digSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0digSignalGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digSignalSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digSignalCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digSignalLogConf(obj,devName,allElements,error) \
	ic0devSoftLogConf((obj),(devName),(allElements),(error))
#define ic0digSignalStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0devSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0digSignalResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0digSignalParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digSignalCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0digSignalCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0digSignalExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0digSignalUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0digSignalGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0digSignalGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0digSignalSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digSignalSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0digSignalUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0digSignalUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digSignalGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0digSignalGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSignalUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSignalSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSignalUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSignalGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0digSignalMonitor(obj,devName,error) \
	ic0devSoftMonitor((obj),(devName),(error))
#define ic0digSignalLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0digSignalLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0digSignalLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digSignalLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0digSignalLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digSignalLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0digSignalUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0digSignalInitObj(obj,error) \
	ic0devSoftInitObj((obj),(error))
#define ic0digSignalExitObj(obj,error) \
	ic0devSoftExitObj((obj),(error))
#define ic0digSignalAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0digSignalUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0digSignalDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0digSignalEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0digSignalLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0digSignalUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0digSignalHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0digSignalHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0digSignalFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digSignalCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0digSignalWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0digSignalUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0digSignalUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0digSignalUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0digSignalUsrStandby2Online(obj,devName,error) \
	lccdevCmdUsrStandby2Online((obj),(devName),(error))
#define ic0digSignalStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0digSignalOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0digSignalOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0digSignalExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0digSignalState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0digSignalSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0digSignalStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0digSignalInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0digSignalStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0digSignalTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0digSignalSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0digSignalAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0digSignalAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0digSignalLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0digSignalOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0digSignalExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0digSignalCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0digSignalBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0digSignalUsrOnline2Standby(obj,devName,error) \
	lccdevStdUsrOnline2Standby((obj),(devName),(error))
#define ic0digSignalUsrStandby2Loaded(obj,devName,error) \
	lccdevStdUsrStandby2Loaded((obj),(devName),(error))
#define ic0digSignalUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0digSignalSetup(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalSet(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrMonitor(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	state,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrSelfTest(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrTest(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrStatusCode(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	text,
    IN    vltINT32	flags,
    OUT   vltINT32*	code,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrStatusValue(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	code,
    OUT   lcctooVALUE*	value,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalUsrStatusLast(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   vltINT16*	last,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0digSignalVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0digSIGNAL_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0digSIGNAL_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0digSIGNAL_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0digSIGNAL_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0digSIGNAL_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0digSignalVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0digSIGNAL_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0digSIGNAL_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0devSOFT_FCTPTR_STATUS)((obj)->fcts[ic0digSIGNAL_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0digSignalVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0digSIGNAL_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0digSIGNAL_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0digSIGNAL_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0digSIGNAL_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0digSIGNAL_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0digSignalVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0digSIGNAL_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVInitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0digSIGNAL_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0digSignalVExitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0digSIGNAL_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0digSignalVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0digSIGNAL_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0digSIGNAL_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0digSIGNAL_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0digSIGNAL_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0digSIGNAL_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0digSignalVUsrInit(obj,devName,error) \
	(*((ic0digSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0digSIGNAL_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0digSIGNAL_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrStandby2Online(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0digSIGNAL_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrOnline2Standby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0digSIGNAL_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrStandby2Loaded(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0digSIGNAL_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0digSIGNAL_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0digSignalVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0digSIGNAL_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0digSIGNAL_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0digSIGNAL_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0digSignalVUsrStatusLast(obj,devName,last,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0digSIGNAL_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0digSignalVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0digSignalVUsrTest(obj,devName,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_TEST)((obj)->fcts[ic0digSIGNAL_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrSelfTest(obj,devName,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0digSIGNAL_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0digSIGNAL_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0digSIGNAL_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0digSIGNAL_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0digSIGNAL_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digSignalVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0digSIGNAL_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digSignalVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0digSIGNAL_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0digSIGNAL_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0digSIGNAL_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0digSIGNAL_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0digSIGNAL_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digSignalVMove(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE)((obj)->fcts[ic0digSIGNAL_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSignalVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0digSIGNAL_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSignalVSet(obj,devName,position,positionNum,error) \
	(*((ic0digSIGNAL_FCTPTR_SET)((obj)->fcts[ic0digSIGNAL_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSignalVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0digSIGNAL_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0digSignalVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0digSIGNAL_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0digSignalVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0digSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0digSIGNAL_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0digSignalVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0digSIGNAL_FCTPTR_SETUP)((obj)->fcts[ic0digSIGNAL_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0digSignalVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0digSIGNAL_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digSignalVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0digSIGNAL_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digSignalVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0digSIGNAL_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digSignalVLogConf(obj,devName,allElements,error) \
	(*((ic0devSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0digSIGNAL_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0digSignalVGetConf(obj,devName,allElements,text,error) \
	(*((ic0digSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0digSIGNAL_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0digSignalVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0digSignalVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digSignalVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0digSignalVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0digSIGNAL_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0digSignalVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0digSignalVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0digSIGNAL_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0digSignalVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0digSIGNAL_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digSignalVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0digSIGNAL_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0digSignalVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0digSIGNAL_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0digSignalVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0digSIGNAL_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digSignalVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0digSIGNAL_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0digSignalVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSignalVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSignalVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSignalVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0digSIGNAL_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSignalVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0digSIGNAL_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0digSignalVMonitor(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0digSIGNAL_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0digSignalVUsrMonitor(obj,devName,state,error) \
	(*((ic0digSIGNAL_FCTPTR_USR_MONITOR)((obj)->fcts[ic0digSIGNAL_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0digSignalVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0digSIGNAL_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0digSignalVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0digSIGNAL_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digSignalVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0digSIGNAL_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digSignalVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0digSIGNAL_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digSignalVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0digSIGNAL_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digSignalVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0digSIGNAL_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))


ccsCOMPL_STAT ic0digSignalNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSignalDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DIG_SIGNAL_H

/************************************************************************/
/* ___oOo___ */
