/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0digSoft.h,v 1.35 2005/09/09 18:58:43 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   22/05/97  Created by eccsgen  Revision: 1.15
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0digSoft - class
*
*   SYNOPSIS
*	#include "ic0digSoft.h"
*
*	ccsCOMPL_STAT ic0digSoftNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0digSoftGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0digSoftLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0digSoftMove(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftMoveRel(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftGetConf(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltLOGICAL	allElements,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftUsrInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftUsrSetMode(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	cmd,
*	    IN    vltLOGICAL	available,
*	    IN    vltLOGICAL	enabled,
*	    IN    vltLOGICAL	simulation,
*	    IN    vltLOGICAL	handset,
*	    IN    vltLOGICAL	locked,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0digSoftErr<error>(...)
*
*   PARENT CLASS
*	public ic0devSOFT
*
*   DESCRIPTION
*       The class ic0digSOFT is the base class for software
*       devices that access hardware controlled with digital
*       I/O signals.
*
*	ic0digSoftNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0digSOFT.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0digSoftDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0digSOFT
*
*	ic0digSoftMove(obj,devName,position,positionNum,error)
*	   Move the hardware device to a given absolute position.
*
*	ic0digSoftMoveRel(obj,devName,position,positionNum,error)
*	   Move the hardware device to a position relative to
*	   the current position.
*
*	ic0digSoftGetConf(obj,devName,allElements,text,error)
*	   Returns the current configuration of the device.
*
*	   If the allElements flag is given, then the elements
*	   of all wheel positions are reported; otherwise
*	   only the element in the lightpath is described.
*
*
*	ic0digSoftErr<error>(...)
*	    The macros ic0digSoftErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0digSoftRead<attr>(...)
*	ic0digSoftWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0digSoftDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0digSoftUsrInit(obj,devName,error)
*	   This function should be overloaded to perform the
*	   device initialisation (without hardware movements).
*
*	ic0digSoftUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error)
*	   This function should be overloaded to perform the
*	   actions necessary to change the mode of the hardware
*	   device.
*
*	   cmd	<IN>	code indicating the command that
*			   calls UsrSetMode() (lccdevCMD_*).
*	   logical	<IN>	state of the available, enabled,
*			   simulation, handset and locked
*			   modes.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	The names of the defined errors begin as follows:
*	ic0digERR_<error>
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0DIG_SOFT_H
#define IC0DIG_SOFT_H

#include "egen.h"
#include "ic0digDefines.h"
#include "ic0digErrors.h"
#include "ic0devSoft.h"


/*
 * user added structure T.Herlin
 */
typedef struct
{
    vltBYTES256    sigPointName;
    vltBYTES32     device;
    vltUINT32      startBit;
    vltUINT32      startNumber;
    vltLOGICAL     inOut;
    vltLOGICAL     highLow;
    vltUINT32      simValue;
} ic0digSOFT_SIGNALS;


/*
 * class type identifier
 */
#define ic0digSOFT_ID	180

/*
 * attribute table
 */
#define ic0digSOFT_ATTR_STATE	0
#define ic0digSOFT_ATTR_SUBSTATE	1
#define ic0digSOFT_ATTR_ERROR	2
#define ic0digSOFT_ATTR_INITIALISED	3
#define ic0digSOFT_ATTR_SIMULATION	4
#define ic0digSOFT_ATTR_DEVICE	5
#define ic0digSOFT_ATTR_AVAILABLE	6
#define ic0digSOFT_ATTR_ENABLED	7
#define ic0digSOFT_ATTR_LOCKED	8
#define ic0digSOFT_ATTR_HANDSET	9
#define ic0digSOFT_ATTR_FIXED	10
#define ic0digSOFT_ATTR_FIX_POS_SET	11
#define ic0digSOFT_ATTR_FIXED_POSITION	12
#define ic0digSOFT_ATTR_FIXED_SIMULATION	13
#define ic0digSOFT_ATTR_FIXED_LOCKED	14
#define ic0digSOFT_ATTR_PREFIX	15
#define ic0digSOFT_ATTR_PARAMETERS_SIZE	16
#define ic0digSOFT_ATTR_PARAMETERS	17
#define ic0digSOFT_ATTR_STATUS	18
#define ic0digSOFT_ATTR_STATUS_INDEX	19
#define ic0digSOFT_ATTR_VALUES_SIZE	20
#define ic0digSOFT_ATTR_VALUES	21
#define ic0digSOFT_ATTR_INIT_RESTRICTION	22
#define ic0digSOFT_ATTR_MOVE_RESTRICTION	23
#define ic0digSOFT_ATTR_STOP_TIMEOUT	24
#define ic0digSOFT_ATTR_SET_VAL_FCT	25
#define ic0digSOFT_ATTR_HELP_TEXT	26
#define ic0digSOFT_ATTR_PARAM_STEP	27
#define ic0digSOFT_ATTR_ACTION_COMMAND	28
#define ic0digSOFT_ATTR_ACTION_STOP_REQ	29
#define ic0digSOFT_ATTR_ACTION_NO_MOVE	30
#define ic0digSOFT_ATTR_ACTION_CHECK	31
#define ic0digSOFT_ATTR_RESET_SETUP_CMD	32
#define ic0digSOFT_ATTR_ID	33
#define ic0digSOFT_ATTR_NAME	34
#define ic0digSOFT_ATTR_SIGNALS	35
#define ic0digSOFT_ATTR_CONFIGURED	36
#define ic0digSOFT_ATTR_HW_ACCESS	37

#define ic0digSOFT_AUTOMATA	0

/*
 * function table
 */
#define ic0digSOFT_FCT_DELETE_OBJ	0
#define ic0digSOFT_FCT_STANDBY	1
#define ic0digSOFT_FCT_ONLINE	2
#define ic0digSOFT_FCT_OFF	3
#define ic0digSOFT_FCT_EXIT	4
#define ic0digSOFT_FCT_STATE	5
#define ic0digSOFT_FCT_SIMULAT	6
#define ic0digSOFT_FCT_STOP_SIM	7
#define ic0digSOFT_FCT_STATUS	8
#define ic0digSOFT_FCT_INIT	9
#define ic0digSOFT_FCT_STOP	10
#define ic0digSOFT_FCT_TEST	11
#define ic0digSOFT_FCT_SELFTST	12
#define ic0digSOFT_FCT_AUTO_TBL	13
#define ic0digSOFT_FCT_AUTO_RST	14
#define ic0digSOFT_FCT_INIT_OBJ	15
#define ic0digSOFT_FCT_EXIT_OBJ	16
#define ic0digSOFT_FCT_LOADED2STBY	17
#define ic0digSOFT_FCT_ONLINE2STBY	18
#define ic0digSOFT_FCT_EXIT1	19
#define ic0digSOFT_FCT_CHECK_DEVICE_NAME	20
#define ic0digSOFT_FCT_BREAK	21
#define ic0digSOFT_FCT_USR_INIT	22
#define ic0digSOFT_FCT_USR_LOADED2STANDBY	23
#define ic0digSOFT_FCT_USR_STANDBY2ONLINE	24
#define ic0digSOFT_FCT_USR_ONLINE2STANDBY	25
#define ic0digSOFT_FCT_USR_STANDBY2LOADED	26
#define ic0digSOFT_FCT_USR_STOP	27
#define ic0digSOFT_FCT_USR_SIMULAT	28
#define ic0digSOFT_FCT_USR_STOP_SIM	29
#define ic0digSOFT_FCT_USR_STATUS_CODE	30
#define ic0digSOFT_FCT_USR_STATUS_LAST	31
#define ic0digSOFT_FCT_USR_STATUS_VALUE	32
#define ic0digSOFT_FCT_USR_TEST	33
#define ic0digSOFT_FCT_USR_SELF_TEST	34
#define ic0digSOFT_FCT_USR_ERROR	35
#define ic0digSOFT_FCT_AVAIL	36
#define ic0digSOFT_FCT_UNAVAIL	37
#define ic0digSOFT_FCT_DISABLE	38
#define ic0digSOFT_FCT_ENABLE	39
#define ic0digSOFT_FCT_LOCK	40
#define ic0digSOFT_FCT_UNLOCK	41
#define ic0digSOFT_FCT_HANDSET	42
#define ic0digSOFT_FCT_HSET_OFF	43
#define ic0digSOFT_FCT_FIX_POS	44
#define ic0digSOFT_FCT_MOVE	45
#define ic0digSOFT_FCT_MOVE_REL	46
#define ic0digSOFT_FCT_SET	47
#define ic0digSOFT_FCT_CHECK	48
#define ic0digSOFT_FCT_WAIT	49
#define ic0digSOFT_FCT_USR_SET_MODE	50
#define ic0digSOFT_FCT_SETUP	51
#define ic0digSOFT_FCT_GET_VAL	52
#define ic0digSOFT_FCT_SET_VAL	53
#define ic0digSOFT_FCT_CFG_ELEM	54
#define ic0digSOFT_FCT_LOG_CONF	55
#define ic0digSOFT_FCT_GET_CONF	56
#define ic0digSOFT_FCT_RESET_PARAMETERS	57
#define ic0digSOFT_FCT_PARSE_PARAMETERS	58
#define ic0digSOFT_FCT_COMPARE_PARAMETERS	59
#define ic0digSOFT_FCT_COPY_PARAMETERS	60
#define ic0digSOFT_FCT_EXEC_PARAMETERS	61
#define ic0digSOFT_FCT_UPDATE_PARAMETERS	62
#define ic0digSOFT_FCT_GET_PARAM_SET_VALUE	63
#define ic0digSOFT_FCT_GET_PARAM_STATUS	64
#define ic0digSOFT_FCT_SET_PARAM	65
#define ic0digSOFT_FCT_SET_PARAM_STEP	66
#define ic0digSOFT_FCT_USR_EXEC_ACTION	67
#define ic0digSOFT_FCT_USR_SET_PARAMETER	68
#define ic0digSOFT_FCT_GET_VALUE_CODE	69
#define ic0digSOFT_FCT_GET_VALUE	70
#define ic0digSOFT_FCT_USR_GET_VALUE	71
#define ic0digSOFT_FCT_SET_VALUE	72
#define ic0digSOFT_FCT_USR_SET_VALUE	73
#define ic0digSOFT_FCT_GET_ARG	74
#define ic0digSOFT_FCT_MONITOR	75
#define ic0digSOFT_FCT_USR_MONITOR	76
#define ic0digSOFT_FCT_LOG_ACTION	77
#define ic0digSOFT_FCT_LOG_COMMENT	78
#define ic0digSOFT_FCT_LOG_PARAMETER	79
#define ic0digSOFT_FCT_LOG_RECOVERY	80
#define ic0digSOFT_FCT_LOG_TBL_PARAM	81
#define ic0digSOFT_FCT_LOG_UNFORSEEN	82

/*
 * automata states
 */
#define ic0digSOFT_STATE_NOSTATE	-1
#define ic0digSOFT_STATE_OFF	0
#define ic0digSOFT_STATE_LOADED	1
#define ic0digSOFT_STATE_STANDBY	2
#define ic0digSOFT_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0digSOFT_SUBSTATE_ERROR	0
#define ic0digSOFT_SUBSTATE_IDLE	1
#define ic0digSOFT_SUBSTATE_BUSY	2
#define ic0digSOFT_SUBSTATE_INITIALISING	3
#define ic0digSOFT_SUBSTATE_SHUTTING_DOWN	4
#define ic0digSOFT_SUBSTATE_STOPPING	5
#define ic0digSOFT_SUBSTATE_COLD_STARTING	6
#define ic0digSOFT_SUBSTATE_STARTING	7
#define ic0digSOFT_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0digSOFT_EVENT_INIT	0
#define ic0digSOFT_EVENT_STANDBY	1
#define ic0digSOFT_EVENT_ONLINE	2
#define ic0digSOFT_EVENT_STOP	3
#define ic0digSOFT_EVENT_OFF	4
#define ic0digSOFT_EVENT_EXIT	5
#define ic0digSOFT_EVENT_STATE	6
#define ic0digSOFT_EVENT_STOP_SIM	7
#define ic0digSOFT_EVENT_STATUS	8
#define ic0digSOFT_EVENT_SIMULAT	9
#define ic0digSOFT_EVENT_TEST	10
#define ic0digSOFT_EVENT_SELFTST	11
#define ic0digSOFT_EVENT_AUTO_TBL	12
#define ic0digSOFT_EVENT_AUTO_RST	13
#define ic0digSOFT_EVENT_DISABLE	14
#define ic0digSOFT_EVENT_ENABLE	15
#define ic0digSOFT_EVENT_LOCK	16
#define ic0digSOFT_EVENT_UNLOCK	17
#define ic0digSOFT_EVENT_HANDSET	18
#define ic0digSOFT_EVENT_HSET_OFF	19
#define ic0digSOFT_EVENT_AVAIL	20
#define ic0digSOFT_EVENT_UNAVAIL	21
#define ic0digSOFT_EVENT_MOVE	22
#define ic0digSOFT_EVENT_MOVE_REL	23
#define ic0digSOFT_EVENT_SET	24
#define ic0digSOFT_EVENT_CHECK	25
#define ic0digSOFT_EVENT_WAIT	26
#define ic0digSOFT_EVENT_SETUP	27
#define ic0digSOFT_EVENT_CFG_ELEM	28
#define ic0digSOFT_EVENT_GET_VAL	29
#define ic0digSOFT_EVENT_SET_VAL	30
#define ic0digSOFT_EVENT_LOG_CONF	31
#define ic0digSOFT_EVENT_GET_CONF	32
#define ic0digSOFT_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0digSOFT_FCTPTR_MOVE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSOFT_FCTPTR_MOVE_REL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSOFT_FCTPTR_GET_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSOFT_FCTPTR_USR_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0digSOFT_FCTPTR_USR_SET_MODE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	cmd,
    IN    vltLOGICAL	available,
    IN    vltLOGICAL	enabled,
    IN    vltLOGICAL	simulation,
    IN    vltLOGICAL	handset,
    IN    vltLOGICAL	locked,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0digSoftLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0dig", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0digErrReadOldb(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_READ_OLDB: Device %s: Unable to read %s from OLDB.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_READ_OLDB,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrWriteOldb(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_WRITE_OLDB: Device %s: Unable to write %s to OLDB.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_WRITE_OLDB,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrSignalConf(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_SIGNAL_CONF: Device %s: Invalid signal configuration (%s).",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_SIGNAL_CONF,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrLocalAttr(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_LOCAL_ATTR: Device %s: Database reference not defined as local attr. (%s).",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_LOCAL_ATTR,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrNoAttr(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_NO_ATTR: Device %s: No attribute defined in database reference. (%s).",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_NO_ATTR,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrConvAlias(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_CONV_ALIAS: Device %s: Could not convert alias: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_CONV_ALIAS,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrPulseSignal(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_PULSE_SIGNAL: Device %s: Unable to pulse digital signal: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_PULSE_SIGNAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrReadSignal(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_READ_SIGNAL: Device %s: Could not read signal: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_READ_SIGNAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrResetSignal(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_RESET_SIGNAL: Device %s: Could not reset signal: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_RESET_SIGNAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrSetSignal(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_SET_SIGNAL: Device %s: Could not set signal: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_SET_SIGNAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrUsrFault(p1) \
	{ \
	lcctooLOG_0("ic0digERR_USR_FAULT: Device %s: Fault signal is active.",(p1)); \
	errAdd(error,"ic0dig",ic0digERR_USR_FAULT,__FILE_LINE__,"%s",(p1)); \
	}
#define ic0digErrWriteSignal(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_WRITE_SIGNAL: Device %s: Could not write to signal: %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_WRITE_SIGNAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0digErrWrongValue(p1,p2) \
	{ \
	lcctooLOG_0("ic0digERR_WRONG_VALUE: Device %s: Wrong value %s.",(p1),(p2)); \
	errAdd(error,"ic0dig",ic0digERR_WRONG_VALUE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0digSoftGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0digSoftReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_STATE,(buf),(error))
#define ic0digSoftWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_STATE,(void*) (buf),(error))
#define ic0digSoftReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_SUBSTATE,(buf),(error))
#define ic0digSoftWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0digSoftReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ERROR,(buf),(error))
#define ic0digSoftWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ERROR,(void*) (buf),(error))
#define ic0digSoftReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_INITIALISED,(buf),(error))
#define ic0digSoftWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0digSoftReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_SIMULATION,(buf),(error))
#define ic0digSoftWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0digSoftReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_DEVICE,(buf),(error))
#define ic0digSoftWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_DEVICE,(void*) (buf),(error))
#define ic0digSoftReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_AVAILABLE,(buf),(error))
#define ic0digSoftWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0digSoftReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ENABLED,(buf),(error))
#define ic0digSoftWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ENABLED,(void*) (buf),(error))
#define ic0digSoftReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_LOCKED,(buf),(error))
#define ic0digSoftWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_LOCKED,(void*) (buf),(error))
#define ic0digSoftReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_HANDSET,(buf),(error))
#define ic0digSoftWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_HANDSET,(void*) (buf),(error))
#define ic0digSoftReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_FIXED,(buf),(error))
#define ic0digSoftWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_FIXED,(void*) (buf),(error))
#define ic0digSoftReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_FIX_POS_SET,(buf),(error))
#define ic0digSoftWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0digSoftReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_FIXED_POSITION,(buf),(error))
#define ic0digSoftWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0digSoftReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0digSoftWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0digSoftReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0digSoftWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0digSoftReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_PREFIX,(buf),(error))
#define ic0digSoftWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_PREFIX,(void*) (buf),(error))
#define ic0digSoftReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0digSoftWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0digSoftGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSOFT_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSOFT_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSOFT_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0digSoftWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSOFT_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSoftGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSOFT_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0digSoftWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSOFT_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSoftReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_VALUES_SIZE,(buf),(error))
#define ic0digSoftWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0digSoftGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSOFT_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSOFT_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0digSoftWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0digSoftReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0digSoftWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0digSoftReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0digSoftWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0digSoftReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0digSoftWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0digSoftGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0digSOFT_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0digSoftWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0digSOFT_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0digSoftReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_PARAM_STEP,(buf),(error))
#define ic0digSoftWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0digSoftReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0digSoftWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0digSoftReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0digSoftWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0digSoftReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0digSoftWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0digSoftReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ACTION_CHECK,(buf),(error))
#define ic0digSoftWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0digSoftReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0digSoftWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0digSoftReadId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_ID,(buf),(error))
#define ic0digSoftWriteId(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_ID,(void*) (buf),(error))
#define ic0digSoftReadName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_NAME,(buf),(error))
#define ic0digSoftWriteName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_NAME,(void*) (buf),(error))
#define ic0digSoftGetSignalsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0digSOFT_ATTR_SIGNALS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0digSoftReadSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0digSOFT_ATTR_SIGNALS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftWriteSignals(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0digSOFT_ATTR_SIGNALS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0digSoftReadConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_CONFIGURED,(buf),(error))
#define ic0digSoftWriteConfigured(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_CONFIGURED,(void*) (buf),(error))
#define ic0digSoftReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0digSOFT_ATTR_HW_ACCESS,(buf),(error))
#define ic0digSoftWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0digSOFT_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0digSoftGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0digSOFT_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0digSoftSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0devSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0digSoftGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digSoftSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digSoftCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digSoftLogConf(obj,devName,allElements,error) \
	ic0devSoftLogConf((obj),(devName),(allElements),(error))
#define ic0digSoftStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0devSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0digSoftSet(obj,devName,position,positionNum,error) \
	ic0devSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0digSoftResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0digSoftParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digSoftCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0digSoftCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0digSoftExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0digSoftUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0digSoftGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0digSoftGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0digSoftSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digSoftSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0digSoftUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0digSoftUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digSoftGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0digSoftGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSoftUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSoftSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSoftUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSoftGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0digSoftMonitor(obj,devName,error) \
	ic0devSoftMonitor((obj),(devName),(error))
#define ic0digSoftUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0digSoftLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0digSoftLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0digSoftLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digSoftLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0digSoftLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digSoftLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0digSoftUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0digSoftInitObj(obj,error) \
	ic0devSoftInitObj((obj),(error))
#define ic0digSoftExitObj(obj,error) \
	ic0devSoftExitObj((obj),(error))
#define ic0digSoftAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0digSoftUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0digSoftDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0digSoftEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0digSoftLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0digSoftUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0digSoftHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0digSoftHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0digSoftFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digSoftCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0digSoftWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0digSoftUsrStatusCode(obj,devName,text,flags,code,error) \
	lccdevCmdUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0digSoftUsrStatusValue(obj,devName,code,value,error) \
	lccdevCmdUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0digSoftUsrStatusLast(obj,devName,last,error) \
	lccdevCmdUsrStatusLast((obj),(devName),(last),(error))
#define ic0digSoftUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0digSoftUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0digSoftUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0digSoftUsrStandby2Online(obj,devName,error) \
	lccdevCmdUsrStandby2Online((obj),(devName),(error))
#define ic0digSoftStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0digSoftOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0digSoftOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0digSoftExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0digSoftState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0digSoftSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0digSoftStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0digSoftInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0digSoftStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0digSoftTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0digSoftSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0digSoftAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0digSoftAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0digSoftLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0digSoftOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0digSoftExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0digSoftCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0digSoftBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0digSoftUsrOnline2Standby(obj,devName,error) \
	lccdevStdUsrOnline2Standby((obj),(devName),(error))
#define ic0digSoftUsrStandby2Loaded(obj,devName,error) \
	lccdevStdUsrStandby2Loaded((obj),(devName),(error))
#define ic0digSoftUsrTest(obj,devName,error) \
	lccdevStdUsrTest((obj),(devName),(error))
#define ic0digSoftUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0digSoftUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0digSoftMove(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSoftMoveRel(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSoftGetConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSoftUsrInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSoftUsrSetMode(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	cmd,
    IN    vltLOGICAL	available,
    IN    vltLOGICAL	enabled,
    IN    vltLOGICAL	simulation,
    IN    vltLOGICAL	handset,
    IN    vltLOGICAL	locked,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0digSoftVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0digSOFT_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0digSOFT_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0digSOFT_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0digSOFT_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0digSOFT_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0digSoftVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0digSOFT_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0digSOFT_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0devSOFT_FCTPTR_STATUS)((obj)->fcts[ic0digSOFT_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0digSoftVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0digSOFT_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0digSOFT_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0digSOFT_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0digSOFT_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0digSOFT_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0digSoftVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0digSOFT_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVInitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0digSOFT_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0digSoftVExitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0digSOFT_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0digSoftVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0digSOFT_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0digSOFT_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0digSOFT_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0digSOFT_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0digSOFT_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0digSoftVUsrInit(obj,devName,error) \
	(*((ic0digSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0digSOFT_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0digSOFT_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrStandby2Online(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0digSOFT_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrOnline2Standby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0digSOFT_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrStandby2Loaded(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0digSOFT_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0digSOFT_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0digSoftVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0digSOFT_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0digSOFT_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0digSOFT_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0digSoftVUsrStatusLast(obj,devName,last,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0digSOFT_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0digSoftVUsrStatusValue(obj,devName,code,value,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0digSOFT_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0digSoftVUsrTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_TEST)((obj)->fcts[ic0digSOFT_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0digSOFT_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0digSOFT_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0digSOFT_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0digSOFT_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0digSOFT_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digSoftVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0digSOFT_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0digSoftVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0digSOFT_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0digSOFT_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0digSOFT_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0digSOFT_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0digSOFT_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0digSoftVMove(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE)((obj)->fcts[ic0digSOFT_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSoftVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0digSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0digSOFT_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSoftVSet(obj,devName,position,positionNum,error) \
	(*((ic0devSOFT_FCTPTR_SET)((obj)->fcts[ic0digSOFT_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0digSoftVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0digSOFT_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0digSoftVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0digSOFT_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0digSoftVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0digSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0digSOFT_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0digSoftVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_SETUP)((obj)->fcts[ic0digSOFT_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0digSoftVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0digSOFT_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0digSoftVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0digSOFT_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0digSoftVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0digSOFT_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0digSoftVLogConf(obj,devName,allElements,error) \
	(*((ic0devSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0digSOFT_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0digSoftVGetConf(obj,devName,allElements,text,error) \
	(*((ic0digSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0digSOFT_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0digSoftVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0digSoftVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0digSoftVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0digSoftVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0digSOFT_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0digSoftVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0digSOFT_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0digSoftVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0digSOFT_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0digSoftVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0digSOFT_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0digSoftVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0digSOFT_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0digSoftVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0digSOFT_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0digSoftVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0digSOFT_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0digSoftVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0digSOFT_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0digSoftVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0digSOFT_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSoftVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0digSOFT_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSoftVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0digSOFT_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0digSoftVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0digSOFT_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0digSoftVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0digSOFT_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0digSoftVMonitor(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0digSOFT_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0digSoftVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0digSOFT_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0digSoftVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0digSOFT_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0digSoftVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0digSOFT_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digSoftVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0digSOFT_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0digSoftVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0digSOFT_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0digSoftVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0digSOFT_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0digSoftVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0digSOFT_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))


ccsCOMPL_STAT ic0digSoftNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0digSoftDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0DIG_SOFT_H

/************************************************************************/
/* ___oOo___ */
