/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0lcuServ.h,v 1.56 2003/03/18 16:19:21 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   28/10/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0lcuServ - class
*
*   SYNOPSIS
*	#include "ic0lcuServ.h"
*
*	ccsCOMPL_STAT ic0lcuServNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0lcuServGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0lcuServLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0lcuServInitObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuServWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	public ic0lcuCMD
*
*   DESCRIPTION
*       The class ic0lcuSERV implements the public interface
*       of a generic ICS at the LCU.
*
*   PUBLIC METHODS (COMMANDS)
*	ic0lcuServNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0lcuSERV.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0lcuServDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0lcuSERV
*
*
*	ic0lcuServErr<error>(...)
*	    The macros ic0lcuServErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0lcuServRead<attr>(...)
*	ic0lcuServWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0lcuServDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0lcuServInitObj(obj,error)
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0LCU_SERV_H
#define IC0LCU_SERV_H

#include "egen.h"
#include "ic0lcuErrors.h"
#include "ic0lcuCmd.h"

/*
 * class type identifier
 */
#define ic0lcuSERV_ID	129

/*
 * attribute table
 */
#define ic0lcuSERV_ATTR_STATE	0
#define ic0lcuSERV_ATTR_MODE	1
#define ic0lcuSERV_ATTR_NEXT_MODE	2
#define ic0lcuSERV_ATTR_ACTION_REPLIES	3
#define ic0lcuSERV_ATTR_ACTION_STEP	4
#define ic0lcuSERV_ATTR_ACTION_VALUE	5
#define ic0lcuSERV_ATTR_ACTION_STOP_REQ	6
#define ic0lcuSERV_ATTR_MOVE_STOP_REQ	7
#define ic0lcuSERV_ATTR_INSTRUMENT_NAME	8
#define ic0lcuSERV_ATTR_PREFIX	9
#define ic0lcuSERV_ATTR_FUNCTIONS_SIZE	10
#define ic0lcuSERV_ATTR_FUNCTIONS	11
#define ic0lcuSERV_ATTR_MODE_CONFIG_ORDER	12
#define ic0lcuSERV_ATTR_KEYWORDS_SIZE	13
#define ic0lcuSERV_ATTR_KEYWORDS	14
#define ic0lcuSERV_ATTR_MODES_SIZE	15
#define ic0lcuSERV_ATTR_MODES	16
#define ic0lcuSERV_ATTR_ACTIONS	17
#define ic0lcuSERV_ATTR_TIMEOUT1	18
#define ic0lcuSERV_ATTR_TIMEOUT2	19
#define ic0lcuSERV_ATTR_MOT_TIMEOUT	20
#define ic0lcuSERV_ATTR_INTERVAL	21
#define ic0lcuSERV_ATTR_MAX_PARALLEL	22
#define ic0lcuSERV_ATTR_DB_BACKUP_FILE	23
#define ic0lcuSERV_ATTR_MODE_SET	24
#define ic0lcuSERV_ATTR_ACTION_ARGS	25


/*
 * function table
 */
#define ic0lcuSERV_FCT_DELETE_OBJ	0
#define ic0lcuSERV_FCT_VERBOSE	1
#define ic0lcuSERV_FCT_STANDBY	2
#define ic0lcuSERV_FCT_ONLINE	3
#define ic0lcuSERV_FCT_OFF	4
#define ic0lcuSERV_FCT_EXIT	5
#define ic0lcuSERV_FCT_STATE	6
#define ic0lcuSERV_FCT_SIMULAT	7
#define ic0lcuSERV_FCT_STOP_SIM	8
#define ic0lcuSERV_FCT_STATUS	9
#define ic0lcuSERV_FCT_INIT	10
#define ic0lcuSERV_FCT_STOP	11
#define ic0lcuSERV_FCT_TEST	12
#define ic0lcuSERV_FCT_SELFTST	13
#define ic0lcuSERV_FCT_DISABLE	14
#define ic0lcuSERV_FCT_ENABLE	15
#define ic0lcuSERV_FCT_LOCK	16
#define ic0lcuSERV_FCT_UNLOCK	17
#define ic0lcuSERV_FCT_HANDSET	18
#define ic0lcuSERV_FCT_HSET_OFF	19
#define ic0lcuSERV_FCT_FIX_POS	20
#define ic0lcuSERV_FCT_MOVE	21
#define ic0lcuSERV_FCT_MOVE_REL	22
#define ic0lcuSERV_FCT_SET	23
#define ic0lcuSERV_FCT_CHECK	24
#define ic0lcuSERV_FCT_WAIT	25
#define ic0lcuSERV_FCT_SETUP	26
#define ic0lcuSERV_FCT_GET_VAL	27
#define ic0lcuSERV_FCT_SET_VAL	28
#define ic0lcuSERV_FCT_CFG_ELEM	29
#define ic0lcuSERV_FCT_LOG_CONF	30
#define ic0lcuSERV_FCT_GET_CONF	31
#define ic0lcuSERV_FCT_USR_GET_REVISION	32
#define ic0lcuSERV_FCT_INIT_OBJ	33
#define ic0lcuSERV_FCT_EXIT_OBJ	34
#define ic0lcuSERV_FCT_FCT_PROCESS	35
#define ic0lcuSERV_FCT_FCT_IS_ALL	36
#define ic0lcuSERV_FCT_FCT_IS_DUP	37
#define ic0lcuSERV_FCT_FCT_GET_BY_NAME	38
#define ic0lcuSERV_FCT_FCT_GET_BY_INDEX	39
#define ic0lcuSERV_FCT_FCT_GET_BY_STATUS	40
#define ic0lcuSERV_FCT_KEY_GET_BY_NAME	41
#define ic0lcuSERV_FCT_KEY_IS_SPECIAL	42
#define ic0lcuSERV_FCT_EXEC_MODE	43
#define ic0lcuSERV_FCT_EXEC_ACTION	44
#define ic0lcuSERV_FCT_ACTIONS	45
#define ic0lcuSERV_FCT_SET_STATE	46
#define ic0lcuSERV_FCT_USR_STATUS	47


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0lcuSERV_FCTPTR_INIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0lcuServLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0lcu", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0lcuServGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0lcuServReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_STATE,(buf),(error))
#define ic0lcuServWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_STATE,(void*) (buf),(error))
#define ic0lcuServReadMode(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MODE,(buf),(error))
#define ic0lcuServWriteMode(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MODE,(void*) (buf),(error))
#define ic0lcuServReadNextMode(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_NEXT_MODE,(buf),(error))
#define ic0lcuServWriteNextMode(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_NEXT_MODE,(void*) (buf),(error))
#define ic0lcuServReadActionReplies(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_ACTION_REPLIES,(buf),(error))
#define ic0lcuServWriteActionReplies(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_ACTION_REPLIES,(void*) (buf),(error))
#define ic0lcuServReadActionStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_ACTION_STEP,(buf),(error))
#define ic0lcuServWriteActionStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_ACTION_STEP,(void*) (buf),(error))
#define ic0lcuServReadActionValue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_ACTION_VALUE,(buf),(error))
#define ic0lcuServWriteActionValue(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_ACTION_VALUE,(void*) (buf),(error))
#define ic0lcuServReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0lcuServWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0lcuServReadMoveStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MOVE_STOP_REQ,(buf),(error))
#define ic0lcuServWriteMoveStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MOVE_STOP_REQ,(void*) (buf),(error))
#define ic0lcuServReadInstrumentName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_INSTRUMENT_NAME,(buf),(error))
#define ic0lcuServWriteInstrumentName(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_INSTRUMENT_NAME,(void*) (buf),(error))
#define ic0lcuServReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_PREFIX,(buf),(error))
#define ic0lcuServWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_PREFIX,(void*) (buf),(error))
#define ic0lcuServReadFunctionsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_FUNCTIONS_SIZE,(buf),(error))
#define ic0lcuServWriteFunctionsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_FUNCTIONS_SIZE,(void*) (buf),(error))
#define ic0lcuServGetFunctionsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0lcuSERV_ATTR_FUNCTIONS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0lcuServReadFunctions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0lcuSERV_ATTR_FUNCTIONS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServWriteFunctions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0lcuSERV_ATTR_FUNCTIONS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServReadModeConfigOrder(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MODE_CONFIG_ORDER,(buf),(error))
#define ic0lcuServWriteModeConfigOrder(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MODE_CONFIG_ORDER,(void*) (buf),(error))
#define ic0lcuServReadKeywordsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_KEYWORDS_SIZE,(buf),(error))
#define ic0lcuServWriteKeywordsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_KEYWORDS_SIZE,(void*) (buf),(error))
#define ic0lcuServGetKeywordsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0lcuSERV_ATTR_KEYWORDS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0lcuServReadKeywords(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0lcuSERV_ATTR_KEYWORDS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServWriteKeywords(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0lcuSERV_ATTR_KEYWORDS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServReadModesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MODES_SIZE,(buf),(error))
#define ic0lcuServWriteModesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MODES_SIZE,(void*) (buf),(error))
#define ic0lcuServGetModesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0lcuSERV_ATTR_MODES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0lcuServReadModes(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0lcuSERV_ATTR_MODES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServWriteModes(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0lcuSERV_ATTR_MODES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServGetActionsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0lcuSERV_ATTR_ACTIONS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0lcuServReadActions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0lcuSERV_ATTR_ACTIONS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServWriteActions(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0lcuSERV_ATTR_ACTIONS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0lcuServReadTimeout1(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_TIMEOUT1,(buf),(error))
#define ic0lcuServWriteTimeout1(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_TIMEOUT1,(void*) (buf),(error))
#define ic0lcuServReadTimeout2(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_TIMEOUT2,(buf),(error))
#define ic0lcuServWriteTimeout2(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_TIMEOUT2,(void*) (buf),(error))
#define ic0lcuServReadMotTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MOT_TIMEOUT,(buf),(error))
#define ic0lcuServWriteMotTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MOT_TIMEOUT,(void*) (buf),(error))
#define ic0lcuServReadInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_INTERVAL,(buf),(error))
#define ic0lcuServWriteInterval(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_INTERVAL,(void*) (buf),(error))
#define ic0lcuServReadMaxParallel(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MAX_PARALLEL,(buf),(error))
#define ic0lcuServWriteMaxParallel(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MAX_PARALLEL,(void*) (buf),(error))
#define ic0lcuServReadDbBackupFile(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_DB_BACKUP_FILE,(buf),(error))
#define ic0lcuServWriteDbBackupFile(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_DB_BACKUP_FILE,(void*) (buf),(error))
#define ic0lcuServReadModeSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_MODE_SET,(buf),(error))
#define ic0lcuServWriteModeSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_MODE_SET,(void*) (buf),(error))
#define ic0lcuServReadActionArgs(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0lcuSERV_ATTR_ACTION_ARGS,(buf),(error))
#define ic0lcuServWriteActionArgs(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0lcuSERV_ATTR_ACTION_ARGS,(void*) (buf),(error))


/*
 * class functions
 */
#define ic0lcuServVerbose(obj,status,error) \
	ic0lcuCmdVerbose((obj),(status),(error))
#define ic0lcuServStandby(obj,function,functionNum,error) \
	ic0lcuCmdStandby((obj),(function),(functionNum),(error))
#define ic0lcuServOnline(obj,function,functionNum,error) \
	ic0lcuCmdOnline((obj),(function),(functionNum),(error))
#define ic0lcuServOff(obj,function,functionNum,error) \
	ic0lcuCmdOff((obj),(function),(functionNum),(error))
#define ic0lcuServExit(obj,function,functionNum,error) \
	ic0lcuCmdExit((obj),(function),(functionNum),(error))
#define ic0lcuServState(obj,function,functionNum,global,state,error) \
	ic0lcuCmdState((obj),(function),(functionNum),(global),(state),(error))
#define ic0lcuServSimulat(obj,function,functionNum,error) \
	ic0lcuCmdSimulat((obj),(function),(functionNum),(error))
#define ic0lcuServStopSim(obj,function,functionNum,error) \
	ic0lcuCmdStopSim((obj),(function),(functionNum),(error))
#define ic0lcuServStatus(obj,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0lcuCmdStatus((obj),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0lcuServInit(obj,function,functionNum,error) \
	ic0lcuCmdInit((obj),(function),(functionNum),(error))
#define ic0lcuServStop(obj,function,functionNum,error) \
	ic0lcuCmdStop((obj),(function),(functionNum),(error))
#define ic0lcuServTest(obj,function,functionNum,error) \
	ic0lcuCmdTest((obj),(function),(functionNum),(error))
#define ic0lcuServSelftst(obj,function,functionNum,error) \
	ic0lcuCmdSelftst((obj),(function),(functionNum),(error))
#define ic0lcuServDisable(obj,function,functionNum,access,error) \
	ic0lcuCmdDisable((obj),(function),(functionNum),(access),(error))
#define ic0lcuServEnable(obj,function,functionNum,access,error) \
	ic0lcuCmdEnable((obj),(function),(functionNum),(access),(error))
#define ic0lcuServLock(obj,function,functionNum,error) \
	ic0lcuCmdLock((obj),(function),(functionNum),(error))
#define ic0lcuServUnlock(obj,function,functionNum,error) \
	ic0lcuCmdUnlock((obj),(function),(functionNum),(error))
#define ic0lcuServHandset(obj,function,functionNum,error) \
	ic0lcuCmdHandset((obj),(function),(functionNum),(error))
#define ic0lcuServHsetOff(obj,function,functionNum,error) \
	ic0lcuCmdHsetOff((obj),(function),(functionNum),(error))
#define ic0lcuServFixPos(obj,devName,position,positionNum,reset,error) \
	ic0lcuCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0lcuServMove(obj,devName,position,positionNum,error) \
	ic0lcuCmdMove((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServMoveRel(obj,devName,position,positionNum,error) \
	ic0lcuCmdMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServSet(obj,devName,position,positionNum,error) \
	ic0lcuCmdSet((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServCheck(obj,function,functionNum,status,error) \
	ic0lcuCmdCheck((obj),(function),(functionNum),(status),(error))
#define ic0lcuServWait(obj,function,functionNum,timeout,error) \
	ic0lcuCmdWait((obj),(function),(functionNum),(timeout),(error))
#define ic0lcuServSetup(obj,function,functionNum,noMove,check,error) \
	ic0lcuCmdSetup((obj),(function),(functionNum),(noMove),(check),(error))
#define ic0lcuServGetVal(obj,function,name,nameNum,status,error) \
	ic0lcuCmdGetVal((obj),(function),(name),(nameNum),(status),(error))
#define ic0lcuServSetVal(obj,function,name,nameNum,value,valueNum,error) \
	ic0lcuCmdSetVal((obj),(function),(name),(nameNum),(value),(valueNum),(error))
#define ic0lcuServCfgElem(obj,function,position,id,values,valuesNum,error) \
	ic0lcuCmdCfgElem((obj),(function),(position),(id),(values),(valuesNum),(error))
#define ic0lcuServLogConf(obj,function,functionNum,allElements,error) \
	ic0lcuCmdLogConf((obj),(function),(functionNum),(allElements),(error))
#define ic0lcuServGetConf(obj,function,functionNum,allElements,text,error) \
	ic0lcuCmdGetConf((obj),(function),(functionNum),(allElements),(text),(error))
#define ic0lcuServUsrGetRevision(obj,revision,error) \
	ic0lcuCmdUsrGetRevision((obj),(revision),(error))
#define ic0lcuServExitObj(obj,error) \
	ic0lcuCmdExitObj((obj),(error))
#define ic0lcuServFctProcess(obj,command,function,functionNum,status,arg1,arg2,arg3,arg4,error) \
	ic0lcuCmdFctProcess((obj),(command),(function),(functionNum),(status),(arg1),(arg2),(arg3),(arg4),(error))
#define ic0lcuServFctIsAll(obj,name,error) \
	ic0lcuCmdFctIsAll((obj),(name),(error))
#define ic0lcuServFctIsDup(obj,name,dup,error) \
	ic0lcuCmdFctIsDup((obj),(name),(dup),(error))
#define ic0lcuServFctGetByName(obj,name,fct,error) \
	ic0lcuCmdFctGetByName((obj),(name),(fct),(error))
#define ic0lcuServFctGetByIndex(obj,index,fct,error) \
	ic0lcuCmdFctGetByIndex((obj),(index),(fct),(error))
#define ic0lcuServFctGetByStatus(obj,status,fctFlags,fctFlagsNum,error) \
	ic0lcuCmdFctGetByStatus((obj),(status),(fctFlags),(fctFlagsNum),(error))
#define ic0lcuServKeyGetByName(obj,name,key,error) \
	ic0lcuCmdKeyGetByName((obj),(name),(key),(error))
#define ic0lcuServKeyIsSpecial(obj,key,value,keyType,mode,error) \
	ic0lcuCmdKeyIsSpecial((obj),(key),(value),(keyType),(mode),(error))
#define ic0lcuServExecMode(obj,mode,error) \
	ic0lcuCmdExecMode((obj),(mode),(error))
#define ic0lcuServExecAction(obj,index,error) \
	ic0lcuCmdExecAction((obj),(index),(error))
#define ic0lcuServActions(obj,action,end,error) \
	ic0lcuCmdActions((obj),(action),(end),(error))
#define ic0lcuServSetState(obj,state,error) \
	ic0lcuCmdSetState((obj),(state),(error))
#define ic0lcuServUsrStatus(obj,devName,flag,header,conflog,opslog,condlog,status,error) \
	ic0lcuCmdUsrStatus((obj),(devName),(flag),(header),(conflog),(opslog),(condlog),(status),(error))

ccsCOMPL_STAT ic0lcuServInitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0lcuServVVerbose(obj,status,error) \
	(*((ic0lcuCMD_FCTPTR_VERBOSE)((obj)->fcts[ic0lcuSERV_FCT_VERBOSE]))) \
	 ((obj),(status),(error))
#define ic0lcuServVStandby(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_STANDBY)((obj)->fcts[ic0lcuSERV_FCT_STANDBY]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVOnline(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_ONLINE)((obj)->fcts[ic0lcuSERV_FCT_ONLINE]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVOff(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_OFF)((obj)->fcts[ic0lcuSERV_FCT_OFF]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVExit(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_EXIT)((obj)->fcts[ic0lcuSERV_FCT_EXIT]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVState(obj,function,functionNum,global,state,error) \
	(*((ic0lcuCMD_FCTPTR_STATE)((obj)->fcts[ic0lcuSERV_FCT_STATE]))) \
	 ((obj),(function),(functionNum),(global),(state),(error))
#define ic0lcuServVSimulat(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_SIMULAT)((obj)->fcts[ic0lcuSERV_FCT_SIMULAT]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVStopSim(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_STOP_SIM)((obj)->fcts[ic0lcuSERV_FCT_STOP_SIM]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVStatus(obj,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0lcuCMD_FCTPTR_STATUS)((obj)->fcts[ic0lcuSERV_FCT_STATUS]))) \
	 ((obj),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0lcuServVInit(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_INIT)((obj)->fcts[ic0lcuSERV_FCT_INIT]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVStop(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_STOP)((obj)->fcts[ic0lcuSERV_FCT_STOP]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVTest(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_TEST)((obj)->fcts[ic0lcuSERV_FCT_TEST]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVSelftst(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_SELFTST)((obj)->fcts[ic0lcuSERV_FCT_SELFTST]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVDisable(obj,function,functionNum,access,error) \
	(*((ic0lcuCMD_FCTPTR_DISABLE)((obj)->fcts[ic0lcuSERV_FCT_DISABLE]))) \
	 ((obj),(function),(functionNum),(access),(error))
#define ic0lcuServVEnable(obj,function,functionNum,access,error) \
	(*((ic0lcuCMD_FCTPTR_ENABLE)((obj)->fcts[ic0lcuSERV_FCT_ENABLE]))) \
	 ((obj),(function),(functionNum),(access),(error))
#define ic0lcuServVLock(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_LOCK)((obj)->fcts[ic0lcuSERV_FCT_LOCK]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVUnlock(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0lcuSERV_FCT_UNLOCK]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVHandset(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_HANDSET)((obj)->fcts[ic0lcuSERV_FCT_HANDSET]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVHsetOff(obj,function,functionNum,error) \
	(*((ic0lcuCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0lcuSERV_FCT_HSET_OFF]))) \
	 ((obj),(function),(functionNum),(error))
#define ic0lcuServVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((ic0lcuCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0lcuSERV_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0lcuServVMove(obj,devName,position,positionNum,error) \
	(*((ic0lcuCMD_FCTPTR_MOVE)((obj)->fcts[ic0lcuSERV_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0lcuCMD_FCTPTR_MOVE_REL)((obj)->fcts[ic0lcuSERV_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServVSet(obj,devName,position,positionNum,error) \
	(*((ic0lcuCMD_FCTPTR_SET)((obj)->fcts[ic0lcuSERV_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0lcuServVCheck(obj,function,functionNum,status,error) \
	(*((ic0lcuCMD_FCTPTR_CHECK)((obj)->fcts[ic0lcuSERV_FCT_CHECK]))) \
	 ((obj),(function),(functionNum),(status),(error))
#define ic0lcuServVWait(obj,function,functionNum,timeout,error) \
	(*((ic0lcuCMD_FCTPTR_WAIT)((obj)->fcts[ic0lcuSERV_FCT_WAIT]))) \
	 ((obj),(function),(functionNum),(timeout),(error))
#define ic0lcuServVSetup(obj,function,functionNum,noMove,check,error) \
	(*((ic0lcuCMD_FCTPTR_SETUP)((obj)->fcts[ic0lcuSERV_FCT_SETUP]))) \
	 ((obj),(function),(functionNum),(noMove),(check),(error))
#define ic0lcuServVGetVal(obj,function,name,nameNum,status,error) \
	(*((ic0lcuCMD_FCTPTR_GET_VAL)((obj)->fcts[ic0lcuSERV_FCT_GET_VAL]))) \
	 ((obj),(function),(name),(nameNum),(status),(error))
#define ic0lcuServVSetVal(obj,function,name,nameNum,value,valueNum,error) \
	(*((ic0lcuCMD_FCTPTR_SET_VAL)((obj)->fcts[ic0lcuSERV_FCT_SET_VAL]))) \
	 ((obj),(function),(name),(nameNum),(value),(valueNum),(error))
#define ic0lcuServVCfgElem(obj,function,position,id,values,valuesNum,error) \
	(*((ic0lcuCMD_FCTPTR_CFG_ELEM)((obj)->fcts[ic0lcuSERV_FCT_CFG_ELEM]))) \
	 ((obj),(function),(position),(id),(values),(valuesNum),(error))
#define ic0lcuServVLogConf(obj,function,functionNum,allElements,error) \
	(*((ic0lcuCMD_FCTPTR_LOG_CONF)((obj)->fcts[ic0lcuSERV_FCT_LOG_CONF]))) \
	 ((obj),(function),(functionNum),(allElements),(error))
#define ic0lcuServVGetConf(obj,function,functionNum,allElements,text,error) \
	(*((ic0lcuCMD_FCTPTR_GET_CONF)((obj)->fcts[ic0lcuSERV_FCT_GET_CONF]))) \
	 ((obj),(function),(functionNum),(allElements),(text),(error))
#define ic0lcuServVUsrGetRevision(obj,revision,error) \
	(*((ic0lcuCMD_FCTPTR_USR_GET_REVISION)((obj)->fcts[ic0lcuSERV_FCT_USR_GET_REVISION]))) \
	 ((obj),(revision),(error))
#define ic0lcuServVInitObj(obj,error) \
	(*((ic0lcuSERV_FCTPTR_INIT_OBJ)((obj)->fcts[ic0lcuSERV_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0lcuServVExitObj(obj,error) \
	(*((ic0lcuCMD_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0lcuSERV_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0lcuServVFctProcess(obj,command,function,functionNum,status,arg1,arg2,arg3,arg4,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_PROCESS)((obj)->fcts[ic0lcuSERV_FCT_FCT_PROCESS]))) \
	 ((obj),(command),(function),(functionNum),(status),(arg1),(arg2),(arg3),(arg4),(error))
#define ic0lcuServVFctIsAll(obj,name,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_IS_ALL)((obj)->fcts[ic0lcuSERV_FCT_FCT_IS_ALL]))) \
	 ((obj),(name),(error))
#define ic0lcuServVFctIsDup(obj,name,dup,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_IS_DUP)((obj)->fcts[ic0lcuSERV_FCT_FCT_IS_DUP]))) \
	 ((obj),(name),(dup),(error))
#define ic0lcuServVFctGetByName(obj,name,fct,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_GET_BY_NAME)((obj)->fcts[ic0lcuSERV_FCT_FCT_GET_BY_NAME]))) \
	 ((obj),(name),(fct),(error))
#define ic0lcuServVFctGetByIndex(obj,index,fct,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_GET_BY_INDEX)((obj)->fcts[ic0lcuSERV_FCT_FCT_GET_BY_INDEX]))) \
	 ((obj),(index),(fct),(error))
#define ic0lcuServVFctGetByStatus(obj,status,fctFlags,fctFlagsNum,error) \
	(*((ic0lcuCMD_FCTPTR_FCT_GET_BY_STATUS)((obj)->fcts[ic0lcuSERV_FCT_FCT_GET_BY_STATUS]))) \
	 ((obj),(status),(fctFlags),(fctFlagsNum),(error))
#define ic0lcuServVKeyGetByName(obj,name,key,error) \
	(*((ic0lcuCMD_FCTPTR_KEY_GET_BY_NAME)((obj)->fcts[ic0lcuSERV_FCT_KEY_GET_BY_NAME]))) \
	 ((obj),(name),(key),(error))
#define ic0lcuServVKeyIsSpecial(obj,key,value,keyType,mode,error) \
	(*((ic0lcuCMD_FCTPTR_KEY_IS_SPECIAL)((obj)->fcts[ic0lcuSERV_FCT_KEY_IS_SPECIAL]))) \
	 ((obj),(key),(value),(keyType),(mode),(error))
#define ic0lcuServVExecMode(obj,mode,error) \
	(*((ic0lcuCMD_FCTPTR_EXEC_MODE)((obj)->fcts[ic0lcuSERV_FCT_EXEC_MODE]))) \
	 ((obj),(mode),(error))
#define ic0lcuServVExecAction(obj,index,error) \
	(*((ic0lcuCMD_FCTPTR_EXEC_ACTION)((obj)->fcts[ic0lcuSERV_FCT_EXEC_ACTION]))) \
	 ((obj),(index),(error))
#define ic0lcuServVActions(obj,action,end,error) \
	(*((ic0lcuCMD_FCTPTR_ACTIONS)((obj)->fcts[ic0lcuSERV_FCT_ACTIONS]))) \
	 ((obj),(action),(end),(error))
#define ic0lcuServVSetState(obj,state,error) \
	(*((ic0lcuCMD_FCTPTR_SET_STATE)((obj)->fcts[ic0lcuSERV_FCT_SET_STATE]))) \
	 ((obj),(state),(error))
#define ic0lcuServVUsrStatus(obj,devName,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0lcuCMD_FCTPTR_USR_STATUS)((obj)->fcts[ic0lcuSERV_FCT_USR_STATUS]))) \
	 ((obj),(devName),(flag),(header),(conflog),(opslog),(condlog),(status),(error))


ccsCOMPL_STAT ic0lcuServNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0lcuServDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0LCU_SERV_H

/************************************************************************/
/* ___oOo___ */
