/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0lcuToo.h,v 1.56 2003/03/18 16:19:21 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 28/05/96  Created by eccsgen V0.60 (March 1996)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0lcuToo - class
*
*   SYNOPSIS
*	#include "ic0lcuToo.h"
*
*	ccsCOMPL_STAT ic0lcuTooNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0lcuTooDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0lcuTooGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0lcuTooLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0lcuTooVerbose(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	status,
*	    OUT   ccsERROR*	error)
*
*
*   PARENT CLASS
*	public lcctoo
*
*   DESCRIPTION
*       This class is necessary to be able to overload the
*       VERBOSE command from the ic0lcuCMD class.
*       This class overloads lcctoo VERBOSE and declares it
*       to be local.
*
*   PUBLIC METHODS (COMMANDS)
*	ic0lcuTooNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0lcuTOO.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0lcuTooDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0lcuTOO
*
*
*	ic0lcuTooErr<error>(...)
*	    The macros ic0lcuTooErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0lcuTooRead<attr>(...)
*	ic0lcuTooWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0lcuTooDb(5).
*
*   PROTECTED METHODS
*	ic0lcuTooVerbose(obj,status,error)
*
*   PRIVATE METHODS
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0LCU_TOO_H
#define IC0LCU_TOO_H

#include "egen.h"
#include "ic0lcuErrors.h"
#include "lcctoo.h"

/*
 * class type identifier
 */
#define ic0lcuTOO_ID	131

/*
 * function table
 */
#define ic0lcuTOO_FCT_DELETE_OBJ	0
#define ic0lcuTOO_FCT_DEBUG	1
#define ic0lcuTOO_FCT_GET_DEBUG	2
#define ic0lcuTOO_FCT_BREAK	3
#define ic0lcuTOO_FCT_KILL	4
#define ic0lcuTOO_FCT_VERBOSE	5
#define ic0lcuTOO_FCT_VERSION	6
#define ic0lcuTOO_FCT_ERR_DUPLICATE	7
#define ic0lcuTOO_FCT_LOG_COMMENT	8
#define ic0lcuTOO_FCT_LOG_SAVE	9
#define ic0lcuTOO_FCT_GET_DEBUG_CODE	10
#define ic0lcuTOO_FCT_GET_DEBUG_VALUE	11
#define ic0lcuTOO_FCT_USE_OBJ	12
#define ic0lcuTOO_FCT_SEND_VERBOSE_TEXT	13
#define ic0lcuTOO_FCT_SEND_REPLY_TEXT	14
#define ic0lcuTOO_FCT_RECV_MSG	15
#define ic0lcuTOO_FCT_STR_SEARCH	16
#define ic0lcuTOO_FCT_STR_CMP	17
#define ic0lcuTOO_FCT_STR_N_CMP	18
#define ic0lcuTOO_FCT_STR_MERGE	19
#define ic0lcuTOO_FCT_STR_SPLIT	20
#define ic0lcuTOO_FCT_SYM_FIND	21
#define ic0lcuTOO_FCT_CCS_INIT	22


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0lcuTOO_FCTPTR_VERBOSE)(
    INOUT egenOBJ*	obj,
    IN    const char*	status,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0lcuTooLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0lcu", (obj)->id, tmpLogBuf); \
	}


/*
 * get object owner
 */
#define ic0lcuTooGetOwner(obj) ((obj)->owner)


/*
 * class functions
 */
#define ic0lcuTooDebug(obj,level,log,verbose,timeout,error) \
	lcctooDebug((obj),(level),(log),(verbose),(timeout),(error))
#define ic0lcuTooGetDebug(obj,text,error) \
	lcctooGetDebug((obj),(text),(error))
#define ic0lcuTooBreak(obj,error) \
	lcctooBreak((obj),(error))
#define ic0lcuTooKill(obj,error) \
	lcctooKill((obj),(error))
#define ic0lcuTooVersion(obj,version,error) \
	lcctooVersion((obj),(version),(error))
#define ic0lcuTooErrDuplicate(obj,locId,error) \
	lcctooErrDuplicate((obj),(locId),(error))
#define ic0lcuTooLogComment(obj,fmt,error) \
	lcctooLogComment((obj),(fmt),(error))
#define ic0lcuTooLogSave(obj,flag,error) \
	lcctooLogSave((obj),(flag),(error))
#define ic0lcuTooGetDebugCode(obj,devName,text,flags,code,error) \
	lcctooGetDebugCode((obj),(devName),(text),(flags),(code),(error))
#define ic0lcuTooGetDebugValue(obj,devName,code,value,error) \
	lcctooGetDebugValue((obj),(devName),(code),(value),(error))
#define ic0lcuTooUseObj(obj,newObj,oldObj,error) \
	lcctooUseObj((obj),(newObj),(oldObj),(error))
#define ic0lcuTooSendVerboseText(obj,fmt,error) \
	lcctooSendVerboseText((obj),(fmt),(error))
#define ic0lcuTooSendReplyText(obj,fmt,error) \
	lcctooSendReplyText((obj),(fmt),(error))
#define ic0lcuTooRecvMsg(obj,flags,timeout,id,message,error) \
	lcctooRecvMsg((obj),(flags),(timeout),(id),(message),(error))
#define ic0lcuTooStrSearch(obj,key,table,tableNum,index,error) \
	lcctooStrSearch((obj),(key),(table),(tableNum),(index),(error))
#define ic0lcuTooStrCmp(obj,s1,s2,error) \
	lcctooStrCmp((obj),(s1),(s2),(error))
#define ic0lcuTooStrNCmp(obj,s1,s2,len,error) \
	lcctooStrNCmp((obj),(s1),(s2),(len),(error))
#define ic0lcuTooStrMerge(obj,buf,bufLen,array,arrayNum,error) \
	lcctooStrMerge((obj),(buf),(bufLen),(array),(arrayNum),(error))
#define ic0lcuTooStrSplit(obj,buf,array,arrayNum,error) \
	lcctooStrSplit((obj),(buf),(array),(arrayNum),(error))
#define ic0lcuTooSymFind(obj,name,ptr,error) \
	lcctooSymFind((obj),(name),(ptr),(error))
#define ic0lcuTooCcsInit(obj,name,error) \
	lcctooCcsInit((obj),(name),(error))

ccsCOMPL_STAT ic0lcuTooVerbose(
    INOUT egenOBJ*	obj,
    IN    const char*	status,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0lcuTooVDebug(obj,level,log,verbose,timeout,error) \
	(*((lcctooFCTPTR_DEBUG)((obj)->fcts[ic0lcuTOO_FCT_DEBUG]))) \
	 ((obj),(level),(log),(verbose),(timeout),(error))
#define ic0lcuTooVGetDebug(obj,text,error) \
	(*((lcctooFCTPTR_GET_DEBUG)((obj)->fcts[ic0lcuTOO_FCT_GET_DEBUG]))) \
	 ((obj),(text),(error))
#define ic0lcuTooVBreak(obj,error) \
	(*((lcctooFCTPTR_BREAK)((obj)->fcts[ic0lcuTOO_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0lcuTooVKill(obj,error) \
	(*((lcctooFCTPTR_KILL)((obj)->fcts[ic0lcuTOO_FCT_KILL]))) \
	 ((obj),(error))
#define ic0lcuTooVVerbose(obj,status,error) \
	(*((ic0lcuTOO_FCTPTR_VERBOSE)((obj)->fcts[ic0lcuTOO_FCT_VERBOSE]))) \
	 ((obj),(status),(error))
#define ic0lcuTooVVersion(obj,version,error) \
	(*((lcctooFCTPTR_VERSION)((obj)->fcts[ic0lcuTOO_FCT_VERSION]))) \
	 ((obj),(version),(error))
#define ic0lcuTooVErrDuplicate(obj,locId,error) \
	(*((lcctooFCTPTR_ERR_DUPLICATE)((obj)->fcts[ic0lcuTOO_FCT_ERR_DUPLICATE]))) \
	 ((obj),(locId),(error))
#define ic0lcuTooVLogComment(obj,fmt,error) \
	(*((lcctooFCTPTR_LOG_COMMENT)((obj)->fcts[ic0lcuTOO_FCT_LOG_COMMENT]))) \
	 ((obj),(fmt),(error))
#define ic0lcuTooVLogSave(obj,flag,error) \
	(*((lcctooFCTPTR_LOG_SAVE)((obj)->fcts[ic0lcuTOO_FCT_LOG_SAVE]))) \
	 ((obj),(flag),(error))
#define ic0lcuTooVGetDebugCode(obj,devName,text,flags,code,error) \
	(*((lcctooFCTPTR_GET_DEBUG_CODE)((obj)->fcts[ic0lcuTOO_FCT_GET_DEBUG_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0lcuTooVGetDebugValue(obj,devName,code,value,error) \
	(*((lcctooFCTPTR_GET_DEBUG_VALUE)((obj)->fcts[ic0lcuTOO_FCT_GET_DEBUG_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0lcuTooVUseObj(obj,newObj,oldObj,error) \
	(*((lcctooFCTPTR_USE_OBJ)((obj)->fcts[ic0lcuTOO_FCT_USE_OBJ]))) \
	 ((obj),(newObj),(oldObj),(error))
#define ic0lcuTooVSendVerboseText(obj,fmt,error) \
	(*((lcctooFCTPTR_SEND_VERBOSE_TEXT)((obj)->fcts[ic0lcuTOO_FCT_SEND_VERBOSE_TEXT]))) \
	 ((obj),(fmt),(error))
#define ic0lcuTooVSendReplyText(obj,fmt,error) \
	(*((lcctooFCTPTR_SEND_REPLY_TEXT)((obj)->fcts[ic0lcuTOO_FCT_SEND_REPLY_TEXT]))) \
	 ((obj),(fmt),(error))
#define ic0lcuTooVRecvMsg(obj,flags,timeout,id,message,error) \
	(*((lcctooFCTPTR_RECV_MSG)((obj)->fcts[ic0lcuTOO_FCT_RECV_MSG]))) \
	 ((obj),(flags),(timeout),(id),(message),(error))
#define ic0lcuTooVStrSearch(obj,key,table,tableNum,index,error) \
	(*((lcctooFCTPTR_STR_SEARCH)((obj)->fcts[ic0lcuTOO_FCT_STR_SEARCH]))) \
	 ((obj),(key),(table),(tableNum),(index),(error))
#define ic0lcuTooVStrCmp(obj,s1,s2,error) \
	(*((lcctooFCTPTR_STR_CMP)((obj)->fcts[ic0lcuTOO_FCT_STR_CMP]))) \
	 ((obj),(s1),(s2),(error))
#define ic0lcuTooVStrNCmp(obj,s1,s2,len,error) \
	(*((lcctooFCTPTR_STR_N_CMP)((obj)->fcts[ic0lcuTOO_FCT_STR_N_CMP]))) \
	 ((obj),(s1),(s2),(len),(error))
#define ic0lcuTooVStrMerge(obj,buf,bufLen,array,arrayNum,error) \
	(*((lcctooFCTPTR_STR_MERGE)((obj)->fcts[ic0lcuTOO_FCT_STR_MERGE]))) \
	 ((obj),(buf),(bufLen),(array),(arrayNum),(error))
#define ic0lcuTooVStrSplit(obj,buf,array,arrayNum,error) \
	(*((lcctooFCTPTR_STR_SPLIT)((obj)->fcts[ic0lcuTOO_FCT_STR_SPLIT]))) \
	 ((obj),(buf),(array),(arrayNum),(error))
#define ic0lcuTooVSymFind(obj,name,ptr,error) \
	(*((lcctooFCTPTR_SYM_FIND)((obj)->fcts[ic0lcuTOO_FCT_SYM_FIND]))) \
	 ((obj),(name),(ptr),(error))
#define ic0lcuTooVCcsInit(obj,name,error) \
	(*((lcctooFCTPTR_CCS_INIT)((obj)->fcts[ic0lcuTOO_FCT_CCS_INIT]))) \
	 ((obj),(name),(error))


ccsCOMPL_STAT ic0lcuTooNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0lcuTooDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0LCU_TOO_H

/************************************************************************/
/* ___oOo___ */
