/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senAdam.h,v 1.51 2005/08/15 13:36:26 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   17/12/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senAdam - class
*
*   SYNOPSIS
*	#include "ic0senAdam.h"
*
*	ccsCOMPL_STAT ic0senAdamNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senAdamGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senAdamLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senAdamComInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamHwStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAdamWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrAdam(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senADAM allows to access several digital
*       and/or analog signals via ADAM devices connected to a
*       single serial port.
*
*	ic0senAdamNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senADAM.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senAdamDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senADAM
*
*
*	ic0senAdamErr<error>(...)
*	    The macros ic0senAdamErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senAdamRead<attr>(...)
*	ic0senAdamWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senAdamDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senAdamComInit(obj,devName,fd,simulation,error)
*	   ComInit is called after opening the serial port.
*
*	ic0senAdamHwStatus(obj,devName,state,simulation,error)
*	   HwStatus is called periodically to update the
*	   status vector of the software device.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_ADAM
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_ADAM_H
#define IC0SEN_ADAM_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senADAM_ID	193

/*
 * attribute table
 */
#define ic0senADAM_ATTR_STATE	0
#define ic0senADAM_ATTR_SUBSTATE	1
#define ic0senADAM_ATTR_ERROR	2
#define ic0senADAM_ATTR_INITIALISED	3
#define ic0senADAM_ATTR_SIMULATION	4
#define ic0senADAM_ATTR_DEVICE	5
#define ic0senADAM_ATTR_AVAILABLE	6
#define ic0senADAM_ATTR_ENABLED	7
#define ic0senADAM_ATTR_LOCKED	8
#define ic0senADAM_ATTR_HANDSET	9
#define ic0senADAM_ATTR_FIXED	10
#define ic0senADAM_ATTR_FIX_POS_SET	11
#define ic0senADAM_ATTR_FIXED_POSITION	12
#define ic0senADAM_ATTR_FIXED_SIMULATION	13
#define ic0senADAM_ATTR_FIXED_LOCKED	14
#define ic0senADAM_ATTR_PREFIX	15
#define ic0senADAM_ATTR_PARAMETERS_SIZE	16
#define ic0senADAM_ATTR_PARAMETERS	17
#define ic0senADAM_ATTR_STATUS	18
#define ic0senADAM_ATTR_STATUS_INDEX	19
#define ic0senADAM_ATTR_VALUES_SIZE	20
#define ic0senADAM_ATTR_VALUES	21
#define ic0senADAM_ATTR_INIT_RESTRICTION	22
#define ic0senADAM_ATTR_MOVE_RESTRICTION	23
#define ic0senADAM_ATTR_STOP_TIMEOUT	24
#define ic0senADAM_ATTR_SET_VAL_FCT	25
#define ic0senADAM_ATTR_HELP_TEXT	26
#define ic0senADAM_ATTR_PARAM_STEP	27
#define ic0senADAM_ATTR_ACTION_COMMAND	28
#define ic0senADAM_ATTR_ACTION_STOP_REQ	29
#define ic0senADAM_ATTR_ACTION_NO_MOVE	30
#define ic0senADAM_ATTR_ACTION_CHECK	31
#define ic0senADAM_ATTR_RESET_SETUP_CMD	32
#define ic0senADAM_ATTR_SENSORS_PREFIX	33
#define ic0senADAM_ATTR_SENSORS_SIZE	34
#define ic0senADAM_ATTR_SENSORS	35
#define ic0senADAM_ATTR_HW_ACCESS	36
#define ic0senADAM_ATTR_LAST	37
#define ic0senADAM_ATTR_LAST_ACCESS	38
#define ic0senADAM_ATTR_MIN_ACCESS	39
#define ic0senADAM_ATTR_IC0SENFLAG	40

#define ic0senADAM_AUTOMATA	0

/*
 * function table
 */
#define ic0senADAM_FCT_DELETE_OBJ	0
#define ic0senADAM_FCT_STANDBY	1
#define ic0senADAM_FCT_ONLINE	2
#define ic0senADAM_FCT_OFF	3
#define ic0senADAM_FCT_EXIT	4
#define ic0senADAM_FCT_STATE	5
#define ic0senADAM_FCT_SIMULAT	6
#define ic0senADAM_FCT_STOP_SIM	7
#define ic0senADAM_FCT_STATUS	8
#define ic0senADAM_FCT_INIT	9
#define ic0senADAM_FCT_STOP	10
#define ic0senADAM_FCT_TEST	11
#define ic0senADAM_FCT_SELFTST	12
#define ic0senADAM_FCT_AUTO_TBL	13
#define ic0senADAM_FCT_AUTO_RST	14
#define ic0senADAM_FCT_INIT_OBJ	15
#define ic0senADAM_FCT_EXIT_OBJ	16
#define ic0senADAM_FCT_LOADED2STBY	17
#define ic0senADAM_FCT_ONLINE2STBY	18
#define ic0senADAM_FCT_EXIT1	19
#define ic0senADAM_FCT_CHECK_DEVICE_NAME	20
#define ic0senADAM_FCT_BREAK	21
#define ic0senADAM_FCT_USR_INIT	22
#define ic0senADAM_FCT_USR_LOADED2STANDBY	23
#define ic0senADAM_FCT_USR_STANDBY2ONLINE	24
#define ic0senADAM_FCT_USR_ONLINE2STANDBY	25
#define ic0senADAM_FCT_USR_STANDBY2LOADED	26
#define ic0senADAM_FCT_USR_STOP	27
#define ic0senADAM_FCT_USR_SIMULAT	28
#define ic0senADAM_FCT_USR_STOP_SIM	29
#define ic0senADAM_FCT_USR_STATUS_CODE	30
#define ic0senADAM_FCT_USR_STATUS_LAST	31
#define ic0senADAM_FCT_USR_STATUS_VALUE	32
#define ic0senADAM_FCT_USR_TEST	33
#define ic0senADAM_FCT_USR_SELF_TEST	34
#define ic0senADAM_FCT_USR_ERROR	35
#define ic0senADAM_FCT_AVAIL	36
#define ic0senADAM_FCT_UNAVAIL	37
#define ic0senADAM_FCT_DISABLE	38
#define ic0senADAM_FCT_ENABLE	39
#define ic0senADAM_FCT_LOCK	40
#define ic0senADAM_FCT_UNLOCK	41
#define ic0senADAM_FCT_HANDSET	42
#define ic0senADAM_FCT_HSET_OFF	43
#define ic0senADAM_FCT_FIX_POS	44
#define ic0senADAM_FCT_MOVE	45
#define ic0senADAM_FCT_MOVE_REL	46
#define ic0senADAM_FCT_SET	47
#define ic0senADAM_FCT_CHECK	48
#define ic0senADAM_FCT_WAIT	49
#define ic0senADAM_FCT_USR_SET_MODE	50
#define ic0senADAM_FCT_SETUP	51
#define ic0senADAM_FCT_GET_VAL	52
#define ic0senADAM_FCT_SET_VAL	53
#define ic0senADAM_FCT_CFG_ELEM	54
#define ic0senADAM_FCT_LOG_CONF	55
#define ic0senADAM_FCT_GET_CONF	56
#define ic0senADAM_FCT_RESET_PARAMETERS	57
#define ic0senADAM_FCT_PARSE_PARAMETERS	58
#define ic0senADAM_FCT_COMPARE_PARAMETERS	59
#define ic0senADAM_FCT_COPY_PARAMETERS	60
#define ic0senADAM_FCT_EXEC_PARAMETERS	61
#define ic0senADAM_FCT_UPDATE_PARAMETERS	62
#define ic0senADAM_FCT_GET_PARAM_SET_VALUE	63
#define ic0senADAM_FCT_GET_PARAM_STATUS	64
#define ic0senADAM_FCT_SET_PARAM	65
#define ic0senADAM_FCT_SET_PARAM_STEP	66
#define ic0senADAM_FCT_USR_EXEC_ACTION	67
#define ic0senADAM_FCT_USR_SET_PARAMETER	68
#define ic0senADAM_FCT_GET_VALUE_CODE	69
#define ic0senADAM_FCT_GET_VALUE	70
#define ic0senADAM_FCT_USR_GET_VALUE	71
#define ic0senADAM_FCT_SET_VALUE	72
#define ic0senADAM_FCT_USR_SET_VALUE	73
#define ic0senADAM_FCT_GET_ARG	74
#define ic0senADAM_FCT_MONITOR	75
#define ic0senADAM_FCT_USR_MONITOR	76
#define ic0senADAM_FCT_LOG_ACTION	77
#define ic0senADAM_FCT_LOG_COMMENT	78
#define ic0senADAM_FCT_LOG_PARAMETER	79
#define ic0senADAM_FCT_LOG_RECOVERY	80
#define ic0senADAM_FCT_LOG_TBL_PARAM	81
#define ic0senADAM_FCT_LOG_UNFORSEEN	82
#define ic0senADAM_FCT_HW_INIT	83
#define ic0senADAM_FCT_HW_EXIT	84
#define ic0senADAM_FCT_HW_STATUS	85
#define ic0senADAM_FCT_HW_SETUP	86
#define ic0senADAM_FCT_HW_GET_CONF	87
#define ic0senADAM_FCT_HW_LOG_CONF	88
#define ic0senADAM_FCT_LOG_STATUS	89
#define ic0senADAM_FCT_COM_INIT	90
#define ic0senADAM_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senADAM_STATE_NOSTATE	-1
#define ic0senADAM_STATE_OFF	0
#define ic0senADAM_STATE_LOADED	1
#define ic0senADAM_STATE_STANDBY	2
#define ic0senADAM_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senADAM_SUBSTATE_ERROR	0
#define ic0senADAM_SUBSTATE_IDLE	1
#define ic0senADAM_SUBSTATE_BUSY	2
#define ic0senADAM_SUBSTATE_INITIALISING	3
#define ic0senADAM_SUBSTATE_SHUTTING_DOWN	4
#define ic0senADAM_SUBSTATE_STOPPING	5
#define ic0senADAM_SUBSTATE_COLD_STARTING	6
#define ic0senADAM_SUBSTATE_STARTING	7
#define ic0senADAM_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senADAM_EVENT_INIT	0
#define ic0senADAM_EVENT_STANDBY	1
#define ic0senADAM_EVENT_ONLINE	2
#define ic0senADAM_EVENT_STOP	3
#define ic0senADAM_EVENT_OFF	4
#define ic0senADAM_EVENT_EXIT	5
#define ic0senADAM_EVENT_STATE	6
#define ic0senADAM_EVENT_STOP_SIM	7
#define ic0senADAM_EVENT_STATUS	8
#define ic0senADAM_EVENT_SIMULAT	9
#define ic0senADAM_EVENT_TEST	10
#define ic0senADAM_EVENT_SELFTST	11
#define ic0senADAM_EVENT_AUTO_TBL	12
#define ic0senADAM_EVENT_AUTO_RST	13
#define ic0senADAM_EVENT_DISABLE	14
#define ic0senADAM_EVENT_ENABLE	15
#define ic0senADAM_EVENT_LOCK	16
#define ic0senADAM_EVENT_UNLOCK	17
#define ic0senADAM_EVENT_HANDSET	18
#define ic0senADAM_EVENT_HSET_OFF	19
#define ic0senADAM_EVENT_AVAIL	20
#define ic0senADAM_EVENT_UNAVAIL	21
#define ic0senADAM_EVENT_MOVE	22
#define ic0senADAM_EVENT_MOVE_REL	23
#define ic0senADAM_EVENT_SET	24
#define ic0senADAM_EVENT_CHECK	25
#define ic0senADAM_EVENT_WAIT	26
#define ic0senADAM_EVENT_SETUP	27
#define ic0senADAM_EVENT_CFG_ELEM	28
#define ic0senADAM_EVENT_GET_VAL	29
#define ic0senADAM_EVENT_SET_VAL	30
#define ic0senADAM_EVENT_LOG_CONF	31
#define ic0senADAM_EVENT_GET_CONF	32
#define ic0senADAM_EVENT_MONITOR	33


/* 
 * tables
 */
typedef struct
    {
    vltBYTES16	name;
    vltBYTES32	description;
    vltINT32	flags;
    vltBYTES32	fits;
    vltBYTES8	unit;
    vltINT32	address;
    vltINT32	type;
    vltDOUBLE	reference;
    vltDOUBLE	ramp;
    } ic0senADAM_SENSORS;


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senADAM_FCTPTR_COM_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senADAM_FCTPTR_HW_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senAdamLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrAdam(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_ADAM: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_ADAM,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senAdamGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senAdamReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_STATE,(buf),(error))
#define ic0senAdamWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_STATE,(void*) (buf),(error))
#define ic0senAdamReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_SUBSTATE,(buf),(error))
#define ic0senAdamWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senAdamReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ERROR,(buf),(error))
#define ic0senAdamWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ERROR,(void*) (buf),(error))
#define ic0senAdamReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_INITIALISED,(buf),(error))
#define ic0senAdamWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senAdamReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_SIMULATION,(buf),(error))
#define ic0senAdamWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senAdamReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_DEVICE,(buf),(error))
#define ic0senAdamWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senAdamReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_AVAILABLE,(buf),(error))
#define ic0senAdamWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senAdamReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ENABLED,(buf),(error))
#define ic0senAdamWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senAdamReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_LOCKED,(buf),(error))
#define ic0senAdamWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senAdamReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_HANDSET,(buf),(error))
#define ic0senAdamWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senAdamReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_FIXED,(buf),(error))
#define ic0senAdamWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_FIXED,(void*) (buf),(error))
#define ic0senAdamReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senAdamWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senAdamReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senAdamWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senAdamReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senAdamWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senAdamReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senAdamWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senAdamReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_PREFIX,(buf),(error))
#define ic0senAdamWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senAdamReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senAdamWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senAdamGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senADAM_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senADAM_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senADAM_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senAdamWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senADAM_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAdamGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senADAM_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senAdamWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senADAM_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAdamReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senAdamWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senAdamGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senADAM_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senADAM_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senAdamWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senAdamReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senAdamWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senAdamReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senAdamWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senAdamReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senAdamWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senAdamGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senADAM_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senAdamWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senADAM_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAdamReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_PARAM_STEP,(buf),(error))
#define ic0senAdamWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senAdamReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senAdamWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senAdamReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senAdamWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senAdamReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senAdamWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senAdamReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senAdamWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senAdamReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senAdamWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senAdamReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senAdamWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senAdamReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senAdamWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senAdamGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senADAM_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senADAM_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAdamReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_HW_ACCESS,(buf),(error))
#define ic0senAdamWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senAdamGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senADAM_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAdamReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senADAM_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senAdamWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senADAM_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAdamReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senAdamWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senAdamReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senAdamWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senAdamReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senADAM_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senAdamWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senADAM_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senAdamGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senADAM_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senAdamHwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senAdamHwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senAdamHwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senAdamHwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senAdamComRead(obj,devName,fd,simulation,error) \
	ic0senAnalogComRead((obj),(devName),(fd),(simulation),(error))
#define ic0senAdamSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senAdamSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senAdamGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senAdamLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senAdamInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senAdamExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senAdamUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senAdamUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senAdamUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senAdamUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senAdamUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senAdamMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senAdamUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senAdamUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senAdamUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senAdamUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senAdamHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senAdamLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senAdamGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senAdamSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senAdamCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senAdamResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senAdamParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senAdamCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senAdamCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senAdamExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senAdamUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senAdamGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senAdamGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senAdamSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senAdamSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senAdamUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senAdamUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senAdamGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senAdamGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAdamUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAdamSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAdamUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAdamGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senAdamUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senAdamLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senAdamLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senAdamLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senAdamLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senAdamLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senAdamLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senAdamUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senAdamAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senAdamUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senAdamDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senAdamEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senAdamLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senAdamUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senAdamHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senAdamHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senAdamFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senAdamCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senAdamWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senAdamUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senAdamUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senAdamUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senAdamStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senAdamOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senAdamOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senAdamExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senAdamState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senAdamSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senAdamStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senAdamInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senAdamStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senAdamTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senAdamSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senAdamAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senAdamAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senAdamLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senAdamOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senAdamExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senAdamCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senAdamBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senAdamUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senAdamUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senAdamComInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAdamHwStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senAdamVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senADAM_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senADAM_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senADAM_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senADAM_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senADAM_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senAdamVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senADAM_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senADAM_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senADAM_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senAdamVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senADAM_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senADAM_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senADAM_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senADAM_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senADAM_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senAdamVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senADAM_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senADAM_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senAdamVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senADAM_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senAdamVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senADAM_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senADAM_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senADAM_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senADAM_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senADAM_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senAdamVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senADAM_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senADAM_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senADAM_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senADAM_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senADAM_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senADAM_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senAdamVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senADAM_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senADAM_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senADAM_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senAdamVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senADAM_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senAdamVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senADAM_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senAdamVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senADAM_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senADAM_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senADAM_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senADAM_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senADAM_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senADAM_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senAdamVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senADAM_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senAdamVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senADAM_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senADAM_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senADAM_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senADAM_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senADAM_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senAdamVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senADAM_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senADAM_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senADAM_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAdamVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senADAM_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senAdamVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senADAM_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senAdamVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senADAM_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senAdamVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senADAM_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senAdamVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senADAM_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senAdamVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senADAM_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senAdamVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senADAM_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senAdamVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senADAM_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senAdamVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senADAM_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senAdamVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senAdamVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senAdamVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senAdamVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senADAM_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senAdamVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senADAM_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senAdamVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senADAM_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senAdamVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senADAM_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senAdamVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senADAM_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senAdamVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senADAM_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senAdamVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senADAM_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senAdamVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senADAM_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senAdamVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senADAM_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAdamVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senADAM_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAdamVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senADAM_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAdamVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senADAM_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAdamVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senADAM_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senAdamVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senADAM_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senADAM_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senAdamVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senADAM_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senAdamVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senADAM_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAdamVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senADAM_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senAdamVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senADAM_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAdamVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senADAM_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senAdamVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senADAM_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAdamVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senADAM_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAdamVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senADAM_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAdamVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senADAM_FCTPTR_HW_STATUS)((obj)->fcts[ic0senADAM_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAdamVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senADAM_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senAdamVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senADAM_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senAdamVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senADAM_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senAdamVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senADAM_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senAdamVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senADAM_FCTPTR_COM_INIT)((obj)->fcts[ic0senADAM_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senAdamVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_READ)((obj)->fcts[ic0senADAM_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senAdamNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAdamDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_ADAM_H

/************************************************************************/
/* ___oOo___ */
