/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senAnalog.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   21/12/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senAnalog - class
*
*   SYNOPSIS
*	#include "ic0senAnalog.h"
*
*	ccsCOMPL_STAT ic0senAnalogNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senAnalogGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senAnalogLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senAnalogHwInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogHwExit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogHwStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogHwGetConf(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogHwLogConf(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogComInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogComRead(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senAnalogWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrAnalog(p1,p2)
*
*   PARENT CLASS
*	public ic0senSOFT
*
*   DESCRIPTION
*       The class ic0senANALOG allows to access analog
*       sensors connected to a single serial port.
*
*	ic0senAnalogNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senANALOG.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senAnalogDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senANALOG
*
*
*	ic0senAnalogErr<error>(...)
*	    The macros ic0senAnalogErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senAnalogRead<attr>(...)
*	ic0senAnalogWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senAnalogDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senAnalogHwInit(obj,devName,state,simulation,error)
*	   HwInit is called when the software device enters
*	   the STANDBY or ONLINE state.
*
*	ic0senAnalogHwExit(obj,devName,state,simulation,error)
*	   HwInit is called when the software device leaves
*	   the STANDBY or ONLINE state.
*
*	ic0senAnalogHwStatus(obj,devName,state,simulation,error)
*	   HwStatus is called periodically to update the
*	   status vector of the software device.
*
*	ic0senAnalogHwGetConf(obj,devName,text,error)
*	   Returns the current configuration of the
*	   software device.
*
*	ic0senAnalogHwLogConf(obj,devName,error)
*	   Logs the current configuration of the
*	   software device.
*
*	ic0senAnalogComInit(obj,devName,fd,simulation,error)
*	   ComInit is called after opening the serial port.
*
*	ic0senAnalogComRead(obj,devName,fd,simulation,error)
*	   ComRead is called to read data from the serial port.
*	   It updates the last vector attribute.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_ANALOG
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_ANALOG_H
#define IC0SEN_ANALOG_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senSoft.h"
#include "ic0senCom.h"

/*
 * class type identifier
 */
#define ic0senANALOG_ID	186

/*
 * attribute table
 */
#define ic0senANALOG_ATTR_STATE	0
#define ic0senANALOG_ATTR_SUBSTATE	1
#define ic0senANALOG_ATTR_ERROR	2
#define ic0senANALOG_ATTR_INITIALISED	3
#define ic0senANALOG_ATTR_SIMULATION	4
#define ic0senANALOG_ATTR_DEVICE	5
#define ic0senANALOG_ATTR_AVAILABLE	6
#define ic0senANALOG_ATTR_ENABLED	7
#define ic0senANALOG_ATTR_LOCKED	8
#define ic0senANALOG_ATTR_HANDSET	9
#define ic0senANALOG_ATTR_FIXED	10
#define ic0senANALOG_ATTR_FIX_POS_SET	11
#define ic0senANALOG_ATTR_FIXED_POSITION	12
#define ic0senANALOG_ATTR_FIXED_SIMULATION	13
#define ic0senANALOG_ATTR_FIXED_LOCKED	14
#define ic0senANALOG_ATTR_PREFIX	15
#define ic0senANALOG_ATTR_PARAMETERS_SIZE	16
#define ic0senANALOG_ATTR_PARAMETERS	17
#define ic0senANALOG_ATTR_STATUS	18
#define ic0senANALOG_ATTR_STATUS_INDEX	19
#define ic0senANALOG_ATTR_VALUES_SIZE	20
#define ic0senANALOG_ATTR_VALUES	21
#define ic0senANALOG_ATTR_INIT_RESTRICTION	22
#define ic0senANALOG_ATTR_MOVE_RESTRICTION	23
#define ic0senANALOG_ATTR_STOP_TIMEOUT	24
#define ic0senANALOG_ATTR_SET_VAL_FCT	25
#define ic0senANALOG_ATTR_HELP_TEXT	26
#define ic0senANALOG_ATTR_PARAM_STEP	27
#define ic0senANALOG_ATTR_ACTION_COMMAND	28
#define ic0senANALOG_ATTR_ACTION_STOP_REQ	29
#define ic0senANALOG_ATTR_ACTION_NO_MOVE	30
#define ic0senANALOG_ATTR_ACTION_CHECK	31
#define ic0senANALOG_ATTR_RESET_SETUP_CMD	32
#define ic0senANALOG_ATTR_SENSORS_PREFIX	33
#define ic0senANALOG_ATTR_SENSORS_SIZE	34
#define ic0senANALOG_ATTR_SENSORS	35
#define ic0senANALOG_ATTR_HW_ACCESS	36
#define ic0senANALOG_ATTR_LAST	37
#define ic0senANALOG_ATTR_LAST_ACCESS	38
#define ic0senANALOG_ATTR_MIN_ACCESS	39
#define ic0senANALOG_ATTR_IC0SENFLAG	40

#define ic0senANALOG_AUTOMATA	0

/*
 * function table
 */
#define ic0senANALOG_FCT_DELETE_OBJ	0
#define ic0senANALOG_FCT_STANDBY	1
#define ic0senANALOG_FCT_ONLINE	2
#define ic0senANALOG_FCT_OFF	3
#define ic0senANALOG_FCT_EXIT	4
#define ic0senANALOG_FCT_STATE	5
#define ic0senANALOG_FCT_SIMULAT	6
#define ic0senANALOG_FCT_STOP_SIM	7
#define ic0senANALOG_FCT_STATUS	8
#define ic0senANALOG_FCT_INIT	9
#define ic0senANALOG_FCT_STOP	10
#define ic0senANALOG_FCT_TEST	11
#define ic0senANALOG_FCT_SELFTST	12
#define ic0senANALOG_FCT_AUTO_TBL	13
#define ic0senANALOG_FCT_AUTO_RST	14
#define ic0senANALOG_FCT_INIT_OBJ	15
#define ic0senANALOG_FCT_EXIT_OBJ	16
#define ic0senANALOG_FCT_LOADED2STBY	17
#define ic0senANALOG_FCT_ONLINE2STBY	18
#define ic0senANALOG_FCT_EXIT1	19
#define ic0senANALOG_FCT_CHECK_DEVICE_NAME	20
#define ic0senANALOG_FCT_BREAK	21
#define ic0senANALOG_FCT_USR_INIT	22
#define ic0senANALOG_FCT_USR_LOADED2STANDBY	23
#define ic0senANALOG_FCT_USR_STANDBY2ONLINE	24
#define ic0senANALOG_FCT_USR_ONLINE2STANDBY	25
#define ic0senANALOG_FCT_USR_STANDBY2LOADED	26
#define ic0senANALOG_FCT_USR_STOP	27
#define ic0senANALOG_FCT_USR_SIMULAT	28
#define ic0senANALOG_FCT_USR_STOP_SIM	29
#define ic0senANALOG_FCT_USR_STATUS_CODE	30
#define ic0senANALOG_FCT_USR_STATUS_LAST	31
#define ic0senANALOG_FCT_USR_STATUS_VALUE	32
#define ic0senANALOG_FCT_USR_TEST	33
#define ic0senANALOG_FCT_USR_SELF_TEST	34
#define ic0senANALOG_FCT_USR_ERROR	35
#define ic0senANALOG_FCT_AVAIL	36
#define ic0senANALOG_FCT_UNAVAIL	37
#define ic0senANALOG_FCT_DISABLE	38
#define ic0senANALOG_FCT_ENABLE	39
#define ic0senANALOG_FCT_LOCK	40
#define ic0senANALOG_FCT_UNLOCK	41
#define ic0senANALOG_FCT_HANDSET	42
#define ic0senANALOG_FCT_HSET_OFF	43
#define ic0senANALOG_FCT_FIX_POS	44
#define ic0senANALOG_FCT_MOVE	45
#define ic0senANALOG_FCT_MOVE_REL	46
#define ic0senANALOG_FCT_SET	47
#define ic0senANALOG_FCT_CHECK	48
#define ic0senANALOG_FCT_WAIT	49
#define ic0senANALOG_FCT_USR_SET_MODE	50
#define ic0senANALOG_FCT_SETUP	51
#define ic0senANALOG_FCT_GET_VAL	52
#define ic0senANALOG_FCT_SET_VAL	53
#define ic0senANALOG_FCT_CFG_ELEM	54
#define ic0senANALOG_FCT_LOG_CONF	55
#define ic0senANALOG_FCT_GET_CONF	56
#define ic0senANALOG_FCT_RESET_PARAMETERS	57
#define ic0senANALOG_FCT_PARSE_PARAMETERS	58
#define ic0senANALOG_FCT_COMPARE_PARAMETERS	59
#define ic0senANALOG_FCT_COPY_PARAMETERS	60
#define ic0senANALOG_FCT_EXEC_PARAMETERS	61
#define ic0senANALOG_FCT_UPDATE_PARAMETERS	62
#define ic0senANALOG_FCT_GET_PARAM_SET_VALUE	63
#define ic0senANALOG_FCT_GET_PARAM_STATUS	64
#define ic0senANALOG_FCT_SET_PARAM	65
#define ic0senANALOG_FCT_SET_PARAM_STEP	66
#define ic0senANALOG_FCT_USR_EXEC_ACTION	67
#define ic0senANALOG_FCT_USR_SET_PARAMETER	68
#define ic0senANALOG_FCT_GET_VALUE_CODE	69
#define ic0senANALOG_FCT_GET_VALUE	70
#define ic0senANALOG_FCT_USR_GET_VALUE	71
#define ic0senANALOG_FCT_SET_VALUE	72
#define ic0senANALOG_FCT_USR_SET_VALUE	73
#define ic0senANALOG_FCT_GET_ARG	74
#define ic0senANALOG_FCT_MONITOR	75
#define ic0senANALOG_FCT_USR_MONITOR	76
#define ic0senANALOG_FCT_LOG_ACTION	77
#define ic0senANALOG_FCT_LOG_COMMENT	78
#define ic0senANALOG_FCT_LOG_PARAMETER	79
#define ic0senANALOG_FCT_LOG_RECOVERY	80
#define ic0senANALOG_FCT_LOG_TBL_PARAM	81
#define ic0senANALOG_FCT_LOG_UNFORSEEN	82
#define ic0senANALOG_FCT_HW_INIT	83
#define ic0senANALOG_FCT_HW_EXIT	84
#define ic0senANALOG_FCT_HW_STATUS	85
#define ic0senANALOG_FCT_HW_SETUP	86
#define ic0senANALOG_FCT_HW_GET_CONF	87
#define ic0senANALOG_FCT_HW_LOG_CONF	88
#define ic0senANALOG_FCT_LOG_STATUS	89
#define ic0senANALOG_FCT_COM_INIT	90
#define ic0senANALOG_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senANALOG_STATE_NOSTATE	-1
#define ic0senANALOG_STATE_OFF	0
#define ic0senANALOG_STATE_LOADED	1
#define ic0senANALOG_STATE_STANDBY	2
#define ic0senANALOG_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senANALOG_SUBSTATE_ERROR	0
#define ic0senANALOG_SUBSTATE_IDLE	1
#define ic0senANALOG_SUBSTATE_BUSY	2
#define ic0senANALOG_SUBSTATE_INITIALISING	3
#define ic0senANALOG_SUBSTATE_SHUTTING_DOWN	4
#define ic0senANALOG_SUBSTATE_STOPPING	5
#define ic0senANALOG_SUBSTATE_COLD_STARTING	6
#define ic0senANALOG_SUBSTATE_STARTING	7
#define ic0senANALOG_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senANALOG_EVENT_INIT	0
#define ic0senANALOG_EVENT_STANDBY	1
#define ic0senANALOG_EVENT_ONLINE	2
#define ic0senANALOG_EVENT_STOP	3
#define ic0senANALOG_EVENT_OFF	4
#define ic0senANALOG_EVENT_EXIT	5
#define ic0senANALOG_EVENT_STATE	6
#define ic0senANALOG_EVENT_STOP_SIM	7
#define ic0senANALOG_EVENT_STATUS	8
#define ic0senANALOG_EVENT_SIMULAT	9
#define ic0senANALOG_EVENT_TEST	10
#define ic0senANALOG_EVENT_SELFTST	11
#define ic0senANALOG_EVENT_AUTO_TBL	12
#define ic0senANALOG_EVENT_AUTO_RST	13
#define ic0senANALOG_EVENT_DISABLE	14
#define ic0senANALOG_EVENT_ENABLE	15
#define ic0senANALOG_EVENT_LOCK	16
#define ic0senANALOG_EVENT_UNLOCK	17
#define ic0senANALOG_EVENT_HANDSET	18
#define ic0senANALOG_EVENT_HSET_OFF	19
#define ic0senANALOG_EVENT_AVAIL	20
#define ic0senANALOG_EVENT_UNAVAIL	21
#define ic0senANALOG_EVENT_MOVE	22
#define ic0senANALOG_EVENT_MOVE_REL	23
#define ic0senANALOG_EVENT_SET	24
#define ic0senANALOG_EVENT_CHECK	25
#define ic0senANALOG_EVENT_WAIT	26
#define ic0senANALOG_EVENT_SETUP	27
#define ic0senANALOG_EVENT_CFG_ELEM	28
#define ic0senANALOG_EVENT_GET_VAL	29
#define ic0senANALOG_EVENT_SET_VAL	30
#define ic0senANALOG_EVENT_LOG_CONF	31
#define ic0senANALOG_EVENT_GET_CONF	32
#define ic0senANALOG_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_HW_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_HW_EXIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_HW_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_HW_GET_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_HW_LOG_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_COM_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senANALOG_FCTPTR_COM_READ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senAnalogLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrAnalog(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_ANALOG: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_ANALOG,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senAnalogGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senAnalogReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_STATE,(buf),(error))
#define ic0senAnalogWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_STATE,(void*) (buf),(error))
#define ic0senAnalogReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_SUBSTATE,(buf),(error))
#define ic0senAnalogWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senAnalogReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ERROR,(buf),(error))
#define ic0senAnalogWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ERROR,(void*) (buf),(error))
#define ic0senAnalogReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_INITIALISED,(buf),(error))
#define ic0senAnalogWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senAnalogReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_SIMULATION,(buf),(error))
#define ic0senAnalogWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senAnalogReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_DEVICE,(buf),(error))
#define ic0senAnalogWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senAnalogReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_AVAILABLE,(buf),(error))
#define ic0senAnalogWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senAnalogReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ENABLED,(buf),(error))
#define ic0senAnalogWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senAnalogReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_LOCKED,(buf),(error))
#define ic0senAnalogWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senAnalogReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_HANDSET,(buf),(error))
#define ic0senAnalogWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senAnalogReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_FIXED,(buf),(error))
#define ic0senAnalogWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_FIXED,(void*) (buf),(error))
#define ic0senAnalogReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senAnalogWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senAnalogReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senAnalogWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senAnalogReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senAnalogWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senAnalogReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senAnalogWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senAnalogReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_PREFIX,(buf),(error))
#define ic0senAnalogWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senAnalogReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senAnalogWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senAnalogGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senANALOG_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senANALOG_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senANALOG_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senAnalogWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senANALOG_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAnalogGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senANALOG_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senAnalogWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senANALOG_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAnalogReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senAnalogWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senAnalogGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senANALOG_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senANALOG_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senAnalogWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senAnalogReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senAnalogWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senAnalogReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senAnalogWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senAnalogReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senAnalogWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senAnalogGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senANALOG_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senAnalogWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senANALOG_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAnalogReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_PARAM_STEP,(buf),(error))
#define ic0senAnalogWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senAnalogReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senAnalogWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senAnalogReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senAnalogWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senAnalogReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senAnalogWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senAnalogReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senAnalogWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senAnalogReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senAnalogWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senAnalogReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senAnalogWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senAnalogReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senAnalogWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senAnalogGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senANALOG_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senANALOG_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senAnalogReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_HW_ACCESS,(buf),(error))
#define ic0senAnalogWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senAnalogGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senANALOG_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senAnalogReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senANALOG_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senAnalogWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senANALOG_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senAnalogReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senAnalogWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senAnalogReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senAnalogWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senAnalogReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senANALOG_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senAnalogWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senANALOG_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senAnalogGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senANALOG_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senAnalogSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senAnalogSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senAnalogGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senAnalogLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senAnalogInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senAnalogExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senAnalogUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senAnalogUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senAnalogUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senAnalogUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senAnalogUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senAnalogMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senAnalogUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senAnalogUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senAnalogUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senAnalogUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senAnalogHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senAnalogLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senAnalogGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senAnalogSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senAnalogCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senAnalogResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senAnalogParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senAnalogCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senAnalogCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senAnalogExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senAnalogUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senAnalogGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senAnalogGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senAnalogSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senAnalogSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senAnalogUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senAnalogUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senAnalogGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senAnalogGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAnalogUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAnalogSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAnalogUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAnalogGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senAnalogUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senAnalogLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senAnalogLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senAnalogLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senAnalogLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senAnalogLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senAnalogLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senAnalogUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senAnalogAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senAnalogUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senAnalogDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senAnalogEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senAnalogLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senAnalogUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senAnalogHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senAnalogHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senAnalogFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senAnalogCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senAnalogWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senAnalogUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senAnalogUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senAnalogUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senAnalogStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senAnalogOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senAnalogOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senAnalogExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senAnalogState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senAnalogSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senAnalogStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senAnalogInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senAnalogStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senAnalogTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senAnalogSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senAnalogAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senAnalogAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senAnalogLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senAnalogOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senAnalogExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senAnalogCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senAnalogBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senAnalogUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senAnalogUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senAnalogHwInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogHwExit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogHwStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogHwGetConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogHwLogConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogComInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogComRead(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senAnalogVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senANALOG_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senANALOG_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senANALOG_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senANALOG_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senANALOG_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senAnalogVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senANALOG_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senANALOG_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senANALOG_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senAnalogVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senANALOG_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senANALOG_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senANALOG_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senANALOG_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senANALOG_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senAnalogVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senANALOG_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senANALOG_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senAnalogVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senANALOG_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senAnalogVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senANALOG_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senANALOG_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senANALOG_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senANALOG_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senANALOG_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senAnalogVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senANALOG_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senANALOG_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senANALOG_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senANALOG_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senANALOG_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senANALOG_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senAnalogVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senANALOG_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senANALOG_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senANALOG_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senAnalogVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senANALOG_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senAnalogVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senANALOG_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senAnalogVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senANALOG_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senANALOG_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senANALOG_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senANALOG_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senANALOG_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senANALOG_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senAnalogVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senANALOG_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senAnalogVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senANALOG_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senANALOG_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senANALOG_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senANALOG_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senANALOG_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senAnalogVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senANALOG_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senANALOG_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senANALOG_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senAnalogVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senANALOG_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senAnalogVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senANALOG_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senAnalogVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senANALOG_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senAnalogVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senANALOG_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senAnalogVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senANALOG_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senAnalogVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senANALOG_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senAnalogVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senANALOG_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senAnalogVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senANALOG_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senAnalogVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senANALOG_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senAnalogVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senAnalogVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senAnalogVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senAnalogVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senANALOG_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senAnalogVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senANALOG_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senAnalogVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senANALOG_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senAnalogVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senANALOG_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senAnalogVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senANALOG_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senAnalogVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senANALOG_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senAnalogVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senANALOG_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senAnalogVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senANALOG_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senAnalogVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senANALOG_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAnalogVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senANALOG_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAnalogVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senANALOG_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senAnalogVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senANALOG_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senAnalogVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senANALOG_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senAnalogVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senANALOG_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senANALOG_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senAnalogVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senANALOG_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senAnalogVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senANALOG_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAnalogVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senANALOG_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senAnalogVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senANALOG_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAnalogVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senANALOG_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senAnalogVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senANALOG_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senAnalogVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senANALOG_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAnalogVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senANALOG_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAnalogVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_STATUS)((obj)->fcts[ic0senANALOG_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senAnalogVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senANALOG_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senAnalogVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senANALOG_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senAnalogVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senANALOG_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senAnalogVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senANALOG_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senAnalogVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_INIT)((obj)->fcts[ic0senANALOG_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senAnalogVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_READ)((obj)->fcts[ic0senANALOG_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senAnalogNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senAnalogDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

/*
 * 'misuse' the sequence semaphore normally used by motor devices
 * to secure the access to the serial port.
 */
#define ic0senAnalogBeginCriticalRegion(obj) \
	lcctooBeginCriticalRegion2(ic0devSoftSemActionSequence(obj),20000)
#define ic0senAnalogEndCriticalRegion(obj) \
	lcctooEndCriticalRegion(ic0devSoftSemActionSequence(obj))


#endif // !IC0SEN_ANALOG_H

/************************************************************************/
/* ___oOo___ */
