/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senBarometer.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   13/04/99  Created by eccsgen  Revision: 1.23
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senBarometer - class
*
*   SYNOPSIS
*	#include "ic0senBarometer.h"
*
*	ccsCOMPL_STAT ic0senBarometerNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senBarometerGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senBarometerLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senBarometerComInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerComRead(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senBarometerWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrBarometer(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senBAROMETER allows to access a barometer
*       sensor PTB220.  The sensor supplies a barometric
*       pressure.
*
*	ic0senBarometerNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senBAROMETER.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senBarometerDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senBAROMETER
*
*
*	ic0senBarometerErr<error>(...)
*	    The macros ic0senBarometerErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senBarometerRead<attr>(...)
*	ic0senBarometerWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senBarometerDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senBarometerComInit(obj,devName,fd,simulation,error)
*	   ComInit is called after opening the serial port.
*
*	ic0senBarometerComRead(obj,devName,fd,simulation,error)
*	   ComRead is called to read data from the serial port.
*	   It updates the last vector attribute.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_BAROMETER
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_BAROMETER_H
#define IC0SEN_BAROMETER_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senBAROMETER_ID	198

/*
 * attribute table
 */
#define ic0senBAROMETER_ATTR_STATE	0
#define ic0senBAROMETER_ATTR_SUBSTATE	1
#define ic0senBAROMETER_ATTR_ERROR	2
#define ic0senBAROMETER_ATTR_INITIALISED	3
#define ic0senBAROMETER_ATTR_SIMULATION	4
#define ic0senBAROMETER_ATTR_DEVICE	5
#define ic0senBAROMETER_ATTR_AVAILABLE	6
#define ic0senBAROMETER_ATTR_ENABLED	7
#define ic0senBAROMETER_ATTR_LOCKED	8
#define ic0senBAROMETER_ATTR_HANDSET	9
#define ic0senBAROMETER_ATTR_FIXED	10
#define ic0senBAROMETER_ATTR_FIX_POS_SET	11
#define ic0senBAROMETER_ATTR_FIXED_POSITION	12
#define ic0senBAROMETER_ATTR_FIXED_SIMULATION	13
#define ic0senBAROMETER_ATTR_FIXED_LOCKED	14
#define ic0senBAROMETER_ATTR_PREFIX	15
#define ic0senBAROMETER_ATTR_PARAMETERS_SIZE	16
#define ic0senBAROMETER_ATTR_PARAMETERS	17
#define ic0senBAROMETER_ATTR_STATUS	18
#define ic0senBAROMETER_ATTR_STATUS_INDEX	19
#define ic0senBAROMETER_ATTR_VALUES_SIZE	20
#define ic0senBAROMETER_ATTR_VALUES	21
#define ic0senBAROMETER_ATTR_INIT_RESTRICTION	22
#define ic0senBAROMETER_ATTR_MOVE_RESTRICTION	23
#define ic0senBAROMETER_ATTR_STOP_TIMEOUT	24
#define ic0senBAROMETER_ATTR_SET_VAL_FCT	25
#define ic0senBAROMETER_ATTR_HELP_TEXT	26
#define ic0senBAROMETER_ATTR_PARAM_STEP	27
#define ic0senBAROMETER_ATTR_ACTION_COMMAND	28
#define ic0senBAROMETER_ATTR_ACTION_STOP_REQ	29
#define ic0senBAROMETER_ATTR_ACTION_NO_MOVE	30
#define ic0senBAROMETER_ATTR_ACTION_CHECK	31
#define ic0senBAROMETER_ATTR_RESET_SETUP_CMD	32
#define ic0senBAROMETER_ATTR_SENSORS_PREFIX	33
#define ic0senBAROMETER_ATTR_SENSORS_SIZE	34
#define ic0senBAROMETER_ATTR_SENSORS	35
#define ic0senBAROMETER_ATTR_HW_ACCESS	36
#define ic0senBAROMETER_ATTR_LAST	37
#define ic0senBAROMETER_ATTR_LAST_ACCESS	38
#define ic0senBAROMETER_ATTR_MIN_ACCESS	39
#define ic0senBAROMETER_ATTR_IC0SENFLAG	40

#define ic0senBAROMETER_AUTOMATA	0

/*
 * function table
 */
#define ic0senBAROMETER_FCT_DELETE_OBJ	0
#define ic0senBAROMETER_FCT_STANDBY	1
#define ic0senBAROMETER_FCT_ONLINE	2
#define ic0senBAROMETER_FCT_OFF	3
#define ic0senBAROMETER_FCT_EXIT	4
#define ic0senBAROMETER_FCT_STATE	5
#define ic0senBAROMETER_FCT_SIMULAT	6
#define ic0senBAROMETER_FCT_STOP_SIM	7
#define ic0senBAROMETER_FCT_STATUS	8
#define ic0senBAROMETER_FCT_INIT	9
#define ic0senBAROMETER_FCT_STOP	10
#define ic0senBAROMETER_FCT_TEST	11
#define ic0senBAROMETER_FCT_SELFTST	12
#define ic0senBAROMETER_FCT_AUTO_TBL	13
#define ic0senBAROMETER_FCT_AUTO_RST	14
#define ic0senBAROMETER_FCT_INIT_OBJ	15
#define ic0senBAROMETER_FCT_EXIT_OBJ	16
#define ic0senBAROMETER_FCT_LOADED2STBY	17
#define ic0senBAROMETER_FCT_ONLINE2STBY	18
#define ic0senBAROMETER_FCT_EXIT1	19
#define ic0senBAROMETER_FCT_CHECK_DEVICE_NAME	20
#define ic0senBAROMETER_FCT_BREAK	21
#define ic0senBAROMETER_FCT_USR_INIT	22
#define ic0senBAROMETER_FCT_USR_LOADED2STANDBY	23
#define ic0senBAROMETER_FCT_USR_STANDBY2ONLINE	24
#define ic0senBAROMETER_FCT_USR_ONLINE2STANDBY	25
#define ic0senBAROMETER_FCT_USR_STANDBY2LOADED	26
#define ic0senBAROMETER_FCT_USR_STOP	27
#define ic0senBAROMETER_FCT_USR_SIMULAT	28
#define ic0senBAROMETER_FCT_USR_STOP_SIM	29
#define ic0senBAROMETER_FCT_USR_STATUS_CODE	30
#define ic0senBAROMETER_FCT_USR_STATUS_LAST	31
#define ic0senBAROMETER_FCT_USR_STATUS_VALUE	32
#define ic0senBAROMETER_FCT_USR_TEST	33
#define ic0senBAROMETER_FCT_USR_SELF_TEST	34
#define ic0senBAROMETER_FCT_USR_ERROR	35
#define ic0senBAROMETER_FCT_AVAIL	36
#define ic0senBAROMETER_FCT_UNAVAIL	37
#define ic0senBAROMETER_FCT_DISABLE	38
#define ic0senBAROMETER_FCT_ENABLE	39
#define ic0senBAROMETER_FCT_LOCK	40
#define ic0senBAROMETER_FCT_UNLOCK	41
#define ic0senBAROMETER_FCT_HANDSET	42
#define ic0senBAROMETER_FCT_HSET_OFF	43
#define ic0senBAROMETER_FCT_FIX_POS	44
#define ic0senBAROMETER_FCT_MOVE	45
#define ic0senBAROMETER_FCT_MOVE_REL	46
#define ic0senBAROMETER_FCT_SET	47
#define ic0senBAROMETER_FCT_CHECK	48
#define ic0senBAROMETER_FCT_WAIT	49
#define ic0senBAROMETER_FCT_USR_SET_MODE	50
#define ic0senBAROMETER_FCT_SETUP	51
#define ic0senBAROMETER_FCT_GET_VAL	52
#define ic0senBAROMETER_FCT_SET_VAL	53
#define ic0senBAROMETER_FCT_CFG_ELEM	54
#define ic0senBAROMETER_FCT_LOG_CONF	55
#define ic0senBAROMETER_FCT_GET_CONF	56
#define ic0senBAROMETER_FCT_RESET_PARAMETERS	57
#define ic0senBAROMETER_FCT_PARSE_PARAMETERS	58
#define ic0senBAROMETER_FCT_COMPARE_PARAMETERS	59
#define ic0senBAROMETER_FCT_COPY_PARAMETERS	60
#define ic0senBAROMETER_FCT_EXEC_PARAMETERS	61
#define ic0senBAROMETER_FCT_UPDATE_PARAMETERS	62
#define ic0senBAROMETER_FCT_GET_PARAM_SET_VALUE	63
#define ic0senBAROMETER_FCT_GET_PARAM_STATUS	64
#define ic0senBAROMETER_FCT_SET_PARAM	65
#define ic0senBAROMETER_FCT_SET_PARAM_STEP	66
#define ic0senBAROMETER_FCT_USR_EXEC_ACTION	67
#define ic0senBAROMETER_FCT_USR_SET_PARAMETER	68
#define ic0senBAROMETER_FCT_GET_VALUE_CODE	69
#define ic0senBAROMETER_FCT_GET_VALUE	70
#define ic0senBAROMETER_FCT_USR_GET_VALUE	71
#define ic0senBAROMETER_FCT_SET_VALUE	72
#define ic0senBAROMETER_FCT_USR_SET_VALUE	73
#define ic0senBAROMETER_FCT_GET_ARG	74
#define ic0senBAROMETER_FCT_MONITOR	75
#define ic0senBAROMETER_FCT_USR_MONITOR	76
#define ic0senBAROMETER_FCT_LOG_ACTION	77
#define ic0senBAROMETER_FCT_LOG_COMMENT	78
#define ic0senBAROMETER_FCT_LOG_PARAMETER	79
#define ic0senBAROMETER_FCT_LOG_RECOVERY	80
#define ic0senBAROMETER_FCT_LOG_TBL_PARAM	81
#define ic0senBAROMETER_FCT_LOG_UNFORSEEN	82
#define ic0senBAROMETER_FCT_HW_INIT	83
#define ic0senBAROMETER_FCT_HW_EXIT	84
#define ic0senBAROMETER_FCT_HW_STATUS	85
#define ic0senBAROMETER_FCT_HW_SETUP	86
#define ic0senBAROMETER_FCT_HW_GET_CONF	87
#define ic0senBAROMETER_FCT_HW_LOG_CONF	88
#define ic0senBAROMETER_FCT_LOG_STATUS	89
#define ic0senBAROMETER_FCT_COM_INIT	90
#define ic0senBAROMETER_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senBAROMETER_STATE_NOSTATE	-1
#define ic0senBAROMETER_STATE_OFF	0
#define ic0senBAROMETER_STATE_LOADED	1
#define ic0senBAROMETER_STATE_STANDBY	2
#define ic0senBAROMETER_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senBAROMETER_SUBSTATE_ERROR	0
#define ic0senBAROMETER_SUBSTATE_IDLE	1
#define ic0senBAROMETER_SUBSTATE_BUSY	2
#define ic0senBAROMETER_SUBSTATE_INITIALISING	3
#define ic0senBAROMETER_SUBSTATE_SHUTTING_DOWN	4
#define ic0senBAROMETER_SUBSTATE_STOPPING	5
#define ic0senBAROMETER_SUBSTATE_COLD_STARTING	6
#define ic0senBAROMETER_SUBSTATE_STARTING	7
#define ic0senBAROMETER_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senBAROMETER_EVENT_INIT	0
#define ic0senBAROMETER_EVENT_STANDBY	1
#define ic0senBAROMETER_EVENT_ONLINE	2
#define ic0senBAROMETER_EVENT_STOP	3
#define ic0senBAROMETER_EVENT_OFF	4
#define ic0senBAROMETER_EVENT_EXIT	5
#define ic0senBAROMETER_EVENT_STATE	6
#define ic0senBAROMETER_EVENT_STOP_SIM	7
#define ic0senBAROMETER_EVENT_STATUS	8
#define ic0senBAROMETER_EVENT_SIMULAT	9
#define ic0senBAROMETER_EVENT_TEST	10
#define ic0senBAROMETER_EVENT_SELFTST	11
#define ic0senBAROMETER_EVENT_AUTO_TBL	12
#define ic0senBAROMETER_EVENT_AUTO_RST	13
#define ic0senBAROMETER_EVENT_DISABLE	14
#define ic0senBAROMETER_EVENT_ENABLE	15
#define ic0senBAROMETER_EVENT_LOCK	16
#define ic0senBAROMETER_EVENT_UNLOCK	17
#define ic0senBAROMETER_EVENT_HANDSET	18
#define ic0senBAROMETER_EVENT_HSET_OFF	19
#define ic0senBAROMETER_EVENT_AVAIL	20
#define ic0senBAROMETER_EVENT_UNAVAIL	21
#define ic0senBAROMETER_EVENT_MOVE	22
#define ic0senBAROMETER_EVENT_MOVE_REL	23
#define ic0senBAROMETER_EVENT_SET	24
#define ic0senBAROMETER_EVENT_CHECK	25
#define ic0senBAROMETER_EVENT_WAIT	26
#define ic0senBAROMETER_EVENT_SETUP	27
#define ic0senBAROMETER_EVENT_CFG_ELEM	28
#define ic0senBAROMETER_EVENT_GET_VAL	29
#define ic0senBAROMETER_EVENT_SET_VAL	30
#define ic0senBAROMETER_EVENT_LOG_CONF	31
#define ic0senBAROMETER_EVENT_GET_CONF	32
#define ic0senBAROMETER_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senBAROMETER_FCTPTR_COM_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senBAROMETER_FCTPTR_COM_READ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senBarometerLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrBarometer(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_BAROMETER: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_BAROMETER,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senBarometerGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senBarometerReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_STATE,(buf),(error))
#define ic0senBarometerWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_STATE,(void*) (buf),(error))
#define ic0senBarometerReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_SUBSTATE,(buf),(error))
#define ic0senBarometerWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senBarometerReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ERROR,(buf),(error))
#define ic0senBarometerWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ERROR,(void*) (buf),(error))
#define ic0senBarometerReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_INITIALISED,(buf),(error))
#define ic0senBarometerWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senBarometerReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_SIMULATION,(buf),(error))
#define ic0senBarometerWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senBarometerReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_DEVICE,(buf),(error))
#define ic0senBarometerWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senBarometerReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_AVAILABLE,(buf),(error))
#define ic0senBarometerWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senBarometerReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ENABLED,(buf),(error))
#define ic0senBarometerWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senBarometerReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_LOCKED,(buf),(error))
#define ic0senBarometerWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senBarometerReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_HANDSET,(buf),(error))
#define ic0senBarometerWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senBarometerReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_FIXED,(buf),(error))
#define ic0senBarometerWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_FIXED,(void*) (buf),(error))
#define ic0senBarometerReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senBarometerWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senBarometerReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senBarometerWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senBarometerReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senBarometerWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senBarometerReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senBarometerWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senBarometerReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_PREFIX,(buf),(error))
#define ic0senBarometerWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senBarometerReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senBarometerWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senBarometerGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senBAROMETER_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senBAROMETER_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senBAROMETER_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senBarometerWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senBAROMETER_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senBarometerGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senBAROMETER_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senBarometerWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senBAROMETER_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senBarometerReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senBarometerWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senBarometerGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senBAROMETER_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senBAROMETER_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senBarometerWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senBarometerReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senBarometerWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senBarometerReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senBarometerWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senBarometerReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senBarometerWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senBarometerGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senBAROMETER_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senBarometerWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senBAROMETER_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senBarometerReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_PARAM_STEP,(buf),(error))
#define ic0senBarometerWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senBarometerReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senBarometerWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senBarometerReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senBarometerWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senBarometerReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senBarometerWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senBarometerReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senBarometerWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senBarometerReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senBarometerWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senBarometerReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senBarometerWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senBarometerReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senBarometerWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senBarometerGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senBAROMETER_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senBAROMETER_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senBarometerReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_HW_ACCESS,(buf),(error))
#define ic0senBarometerWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senBarometerGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senBAROMETER_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senBarometerReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senBAROMETER_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senBarometerWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senBAROMETER_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senBarometerReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senBarometerWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senBarometerReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senBarometerWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senBarometerReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senBAROMETER_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senBarometerWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senBAROMETER_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senBarometerGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senBAROMETER_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senBarometerHwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerHwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerHwStatus(obj,devName,state,simulation,error) \
	ic0senAnalogHwStatus((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerHwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senBarometerHwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senBarometerSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senBarometerSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senBarometerGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senBarometerLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senBarometerInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senBarometerExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senBarometerUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senBarometerUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senBarometerUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senBarometerUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senBarometerUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senBarometerMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senBarometerUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senBarometerUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senBarometerUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senBarometerUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senBarometerHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senBarometerLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senBarometerGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senBarometerSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senBarometerCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senBarometerResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senBarometerParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senBarometerCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senBarometerCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senBarometerExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senBarometerUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senBarometerGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senBarometerGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senBarometerSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senBarometerSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senBarometerUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senBarometerUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senBarometerGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senBarometerGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senBarometerUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senBarometerSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senBarometerUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senBarometerGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senBarometerUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senBarometerLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senBarometerLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senBarometerLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senBarometerLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senBarometerLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senBarometerLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senBarometerUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senBarometerAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senBarometerUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senBarometerDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senBarometerEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senBarometerLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senBarometerUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senBarometerHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senBarometerHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senBarometerFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senBarometerCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senBarometerWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senBarometerUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senBarometerUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senBarometerUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senBarometerStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senBarometerOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senBarometerOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senBarometerExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senBarometerState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senBarometerSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senBarometerStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senBarometerInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senBarometerStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senBarometerTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senBarometerSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senBarometerAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senBarometerAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senBarometerLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senBarometerOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senBarometerExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senBarometerCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senBarometerBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senBarometerUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senBarometerUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senBarometerComInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senBarometerComRead(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senBarometerVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senBAROMETER_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senBAROMETER_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senBAROMETER_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senBAROMETER_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senBAROMETER_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senBarometerVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senBAROMETER_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senBAROMETER_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senBAROMETER_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senBarometerVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senBAROMETER_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senBAROMETER_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senBAROMETER_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senBAROMETER_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senBAROMETER_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senBarometerVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senBAROMETER_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senBAROMETER_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senBarometerVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senBAROMETER_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senBarometerVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senBAROMETER_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senBAROMETER_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senBAROMETER_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senBAROMETER_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senBAROMETER_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senBarometerVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senBAROMETER_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senBAROMETER_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senBAROMETER_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senBAROMETER_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senBAROMETER_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senBAROMETER_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senBarometerVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senBAROMETER_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senBAROMETER_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senBAROMETER_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senBarometerVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senBAROMETER_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senBarometerVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senBarometerVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senBAROMETER_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senBAROMETER_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senBAROMETER_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senBAROMETER_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senBAROMETER_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senBAROMETER_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senBarometerVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senBAROMETER_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senBarometerVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senBAROMETER_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senBAROMETER_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senBAROMETER_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senBAROMETER_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senBAROMETER_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senBarometerVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senBAROMETER_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senBAROMETER_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senBAROMETER_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senBarometerVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senBAROMETER_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senBarometerVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senBAROMETER_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senBarometerVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senBAROMETER_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senBarometerVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senBAROMETER_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senBarometerVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senBAROMETER_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senBarometerVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senBAROMETER_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senBarometerVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senBAROMETER_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senBarometerVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senBAROMETER_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senBarometerVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senBAROMETER_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senBarometerVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senBarometerVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senBarometerVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senBarometerVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senBAROMETER_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senBarometerVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senBarometerVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senBAROMETER_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senBarometerVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senBAROMETER_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senBarometerVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senBAROMETER_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senBarometerVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senBAROMETER_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senBarometerVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senBAROMETER_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senBarometerVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senBAROMETER_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senBarometerVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senBarometerVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senBarometerVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senBarometerVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senBAROMETER_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senBarometerVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senBAROMETER_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senBarometerVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senBAROMETER_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senBAROMETER_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senBarometerVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senBAROMETER_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senBarometerVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senBAROMETER_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senBarometerVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senBAROMETER_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senBarometerVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senBAROMETER_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senBarometerVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senBAROMETER_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senBarometerVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senBAROMETER_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senBarometerVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senBAROMETER_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senBAROMETER_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_STATUS)((obj)->fcts[ic0senBAROMETER_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senBarometerVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senBAROMETER_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senBarometerVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senBAROMETER_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senBarometerVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senBAROMETER_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senBarometerVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senBAROMETER_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senBarometerVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senBAROMETER_FCTPTR_COM_INIT)((obj)->fcts[ic0senBAROMETER_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senBarometerVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senBAROMETER_FCTPTR_COM_READ)((obj)->fcts[ic0senBAROMETER_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senBarometerNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senBarometerDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_BAROMETER_H

/************************************************************************/
/* ___oOo___ */
