/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senCooling.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   21/12/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senCooling - class
*
*   SYNOPSIS
*	#include "ic0senCooling.h"
*
*	ccsCOMPL_STAT ic0senCoolingNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senCoolingGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senCoolingLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senCoolingComInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingHwStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senCoolingWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrCooling(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senCOOLING allows to access a standard
*       cooling controller.  The cooling controller supplies
*       up to four temperatures and three flow measurements.
*       In addition, a flag specifies if the enclosure door
*       is open or not (in this order in the sensors table).
*
*	ic0senCoolingNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senCOOLING.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senCoolingDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senCOOLING
*
*
*	ic0senCoolingErr<error>(...)
*	    The macros ic0senCoolingErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senCoolingRead<attr>(...)
*	ic0senCoolingWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senCoolingDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senCoolingComInit(obj,devName,fd,simulation,error)
*	   ComInit is called after opening the serial port.
*
*	ic0senCoolingHwStatus(obj,devName,state,simulation,error)
*	   HwStatus is called periodically to update the
*	   status vector of the software device.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_COOLING
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_COOLING_H
#define IC0SEN_COOLING_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senCOOLING_ID	195

/*
 * attribute table
 */
#define ic0senCOOLING_ATTR_STATE	0
#define ic0senCOOLING_ATTR_SUBSTATE	1
#define ic0senCOOLING_ATTR_ERROR	2
#define ic0senCOOLING_ATTR_INITIALISED	3
#define ic0senCOOLING_ATTR_SIMULATION	4
#define ic0senCOOLING_ATTR_DEVICE	5
#define ic0senCOOLING_ATTR_AVAILABLE	6
#define ic0senCOOLING_ATTR_ENABLED	7
#define ic0senCOOLING_ATTR_LOCKED	8
#define ic0senCOOLING_ATTR_HANDSET	9
#define ic0senCOOLING_ATTR_FIXED	10
#define ic0senCOOLING_ATTR_FIX_POS_SET	11
#define ic0senCOOLING_ATTR_FIXED_POSITION	12
#define ic0senCOOLING_ATTR_FIXED_SIMULATION	13
#define ic0senCOOLING_ATTR_FIXED_LOCKED	14
#define ic0senCOOLING_ATTR_PREFIX	15
#define ic0senCOOLING_ATTR_PARAMETERS_SIZE	16
#define ic0senCOOLING_ATTR_PARAMETERS	17
#define ic0senCOOLING_ATTR_STATUS	18
#define ic0senCOOLING_ATTR_STATUS_INDEX	19
#define ic0senCOOLING_ATTR_VALUES_SIZE	20
#define ic0senCOOLING_ATTR_VALUES	21
#define ic0senCOOLING_ATTR_INIT_RESTRICTION	22
#define ic0senCOOLING_ATTR_MOVE_RESTRICTION	23
#define ic0senCOOLING_ATTR_STOP_TIMEOUT	24
#define ic0senCOOLING_ATTR_SET_VAL_FCT	25
#define ic0senCOOLING_ATTR_HELP_TEXT	26
#define ic0senCOOLING_ATTR_PARAM_STEP	27
#define ic0senCOOLING_ATTR_ACTION_COMMAND	28
#define ic0senCOOLING_ATTR_ACTION_STOP_REQ	29
#define ic0senCOOLING_ATTR_ACTION_NO_MOVE	30
#define ic0senCOOLING_ATTR_ACTION_CHECK	31
#define ic0senCOOLING_ATTR_RESET_SETUP_CMD	32
#define ic0senCOOLING_ATTR_SENSORS_PREFIX	33
#define ic0senCOOLING_ATTR_SENSORS_SIZE	34
#define ic0senCOOLING_ATTR_SENSORS	35
#define ic0senCOOLING_ATTR_HW_ACCESS	36
#define ic0senCOOLING_ATTR_LAST	37
#define ic0senCOOLING_ATTR_LAST_ACCESS	38
#define ic0senCOOLING_ATTR_MIN_ACCESS	39
#define ic0senCOOLING_ATTR_IC0SENFLAG	40

#define ic0senCOOLING_AUTOMATA	0

/*
 * function table
 */
#define ic0senCOOLING_FCT_DELETE_OBJ	0
#define ic0senCOOLING_FCT_STANDBY	1
#define ic0senCOOLING_FCT_ONLINE	2
#define ic0senCOOLING_FCT_OFF	3
#define ic0senCOOLING_FCT_EXIT	4
#define ic0senCOOLING_FCT_STATE	5
#define ic0senCOOLING_FCT_SIMULAT	6
#define ic0senCOOLING_FCT_STOP_SIM	7
#define ic0senCOOLING_FCT_STATUS	8
#define ic0senCOOLING_FCT_INIT	9
#define ic0senCOOLING_FCT_STOP	10
#define ic0senCOOLING_FCT_TEST	11
#define ic0senCOOLING_FCT_SELFTST	12
#define ic0senCOOLING_FCT_AUTO_TBL	13
#define ic0senCOOLING_FCT_AUTO_RST	14
#define ic0senCOOLING_FCT_INIT_OBJ	15
#define ic0senCOOLING_FCT_EXIT_OBJ	16
#define ic0senCOOLING_FCT_LOADED2STBY	17
#define ic0senCOOLING_FCT_ONLINE2STBY	18
#define ic0senCOOLING_FCT_EXIT1	19
#define ic0senCOOLING_FCT_CHECK_DEVICE_NAME	20
#define ic0senCOOLING_FCT_BREAK	21
#define ic0senCOOLING_FCT_USR_INIT	22
#define ic0senCOOLING_FCT_USR_LOADED2STANDBY	23
#define ic0senCOOLING_FCT_USR_STANDBY2ONLINE	24
#define ic0senCOOLING_FCT_USR_ONLINE2STANDBY	25
#define ic0senCOOLING_FCT_USR_STANDBY2LOADED	26
#define ic0senCOOLING_FCT_USR_STOP	27
#define ic0senCOOLING_FCT_USR_SIMULAT	28
#define ic0senCOOLING_FCT_USR_STOP_SIM	29
#define ic0senCOOLING_FCT_USR_STATUS_CODE	30
#define ic0senCOOLING_FCT_USR_STATUS_LAST	31
#define ic0senCOOLING_FCT_USR_STATUS_VALUE	32
#define ic0senCOOLING_FCT_USR_TEST	33
#define ic0senCOOLING_FCT_USR_SELF_TEST	34
#define ic0senCOOLING_FCT_USR_ERROR	35
#define ic0senCOOLING_FCT_AVAIL	36
#define ic0senCOOLING_FCT_UNAVAIL	37
#define ic0senCOOLING_FCT_DISABLE	38
#define ic0senCOOLING_FCT_ENABLE	39
#define ic0senCOOLING_FCT_LOCK	40
#define ic0senCOOLING_FCT_UNLOCK	41
#define ic0senCOOLING_FCT_HANDSET	42
#define ic0senCOOLING_FCT_HSET_OFF	43
#define ic0senCOOLING_FCT_FIX_POS	44
#define ic0senCOOLING_FCT_MOVE	45
#define ic0senCOOLING_FCT_MOVE_REL	46
#define ic0senCOOLING_FCT_SET	47
#define ic0senCOOLING_FCT_CHECK	48
#define ic0senCOOLING_FCT_WAIT	49
#define ic0senCOOLING_FCT_USR_SET_MODE	50
#define ic0senCOOLING_FCT_SETUP	51
#define ic0senCOOLING_FCT_GET_VAL	52
#define ic0senCOOLING_FCT_SET_VAL	53
#define ic0senCOOLING_FCT_CFG_ELEM	54
#define ic0senCOOLING_FCT_LOG_CONF	55
#define ic0senCOOLING_FCT_GET_CONF	56
#define ic0senCOOLING_FCT_RESET_PARAMETERS	57
#define ic0senCOOLING_FCT_PARSE_PARAMETERS	58
#define ic0senCOOLING_FCT_COMPARE_PARAMETERS	59
#define ic0senCOOLING_FCT_COPY_PARAMETERS	60
#define ic0senCOOLING_FCT_EXEC_PARAMETERS	61
#define ic0senCOOLING_FCT_UPDATE_PARAMETERS	62
#define ic0senCOOLING_FCT_GET_PARAM_SET_VALUE	63
#define ic0senCOOLING_FCT_GET_PARAM_STATUS	64
#define ic0senCOOLING_FCT_SET_PARAM	65
#define ic0senCOOLING_FCT_SET_PARAM_STEP	66
#define ic0senCOOLING_FCT_USR_EXEC_ACTION	67
#define ic0senCOOLING_FCT_USR_SET_PARAMETER	68
#define ic0senCOOLING_FCT_GET_VALUE_CODE	69
#define ic0senCOOLING_FCT_GET_VALUE	70
#define ic0senCOOLING_FCT_USR_GET_VALUE	71
#define ic0senCOOLING_FCT_SET_VALUE	72
#define ic0senCOOLING_FCT_USR_SET_VALUE	73
#define ic0senCOOLING_FCT_GET_ARG	74
#define ic0senCOOLING_FCT_MONITOR	75
#define ic0senCOOLING_FCT_USR_MONITOR	76
#define ic0senCOOLING_FCT_LOG_ACTION	77
#define ic0senCOOLING_FCT_LOG_COMMENT	78
#define ic0senCOOLING_FCT_LOG_PARAMETER	79
#define ic0senCOOLING_FCT_LOG_RECOVERY	80
#define ic0senCOOLING_FCT_LOG_TBL_PARAM	81
#define ic0senCOOLING_FCT_LOG_UNFORSEEN	82
#define ic0senCOOLING_FCT_HW_INIT	83
#define ic0senCOOLING_FCT_HW_EXIT	84
#define ic0senCOOLING_FCT_HW_STATUS	85
#define ic0senCOOLING_FCT_HW_SETUP	86
#define ic0senCOOLING_FCT_HW_GET_CONF	87
#define ic0senCOOLING_FCT_HW_LOG_CONF	88
#define ic0senCOOLING_FCT_LOG_STATUS	89
#define ic0senCOOLING_FCT_COM_INIT	90
#define ic0senCOOLING_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senCOOLING_STATE_NOSTATE	-1
#define ic0senCOOLING_STATE_OFF	0
#define ic0senCOOLING_STATE_LOADED	1
#define ic0senCOOLING_STATE_STANDBY	2
#define ic0senCOOLING_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senCOOLING_SUBSTATE_ERROR	0
#define ic0senCOOLING_SUBSTATE_IDLE	1
#define ic0senCOOLING_SUBSTATE_BUSY	2
#define ic0senCOOLING_SUBSTATE_INITIALISING	3
#define ic0senCOOLING_SUBSTATE_SHUTTING_DOWN	4
#define ic0senCOOLING_SUBSTATE_STOPPING	5
#define ic0senCOOLING_SUBSTATE_COLD_STARTING	6
#define ic0senCOOLING_SUBSTATE_STARTING	7
#define ic0senCOOLING_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senCOOLING_EVENT_INIT	0
#define ic0senCOOLING_EVENT_STANDBY	1
#define ic0senCOOLING_EVENT_ONLINE	2
#define ic0senCOOLING_EVENT_STOP	3
#define ic0senCOOLING_EVENT_OFF	4
#define ic0senCOOLING_EVENT_EXIT	5
#define ic0senCOOLING_EVENT_STATE	6
#define ic0senCOOLING_EVENT_STOP_SIM	7
#define ic0senCOOLING_EVENT_STATUS	8
#define ic0senCOOLING_EVENT_SIMULAT	9
#define ic0senCOOLING_EVENT_TEST	10
#define ic0senCOOLING_EVENT_SELFTST	11
#define ic0senCOOLING_EVENT_AUTO_TBL	12
#define ic0senCOOLING_EVENT_AUTO_RST	13
#define ic0senCOOLING_EVENT_DISABLE	14
#define ic0senCOOLING_EVENT_ENABLE	15
#define ic0senCOOLING_EVENT_LOCK	16
#define ic0senCOOLING_EVENT_UNLOCK	17
#define ic0senCOOLING_EVENT_HANDSET	18
#define ic0senCOOLING_EVENT_HSET_OFF	19
#define ic0senCOOLING_EVENT_AVAIL	20
#define ic0senCOOLING_EVENT_UNAVAIL	21
#define ic0senCOOLING_EVENT_MOVE	22
#define ic0senCOOLING_EVENT_MOVE_REL	23
#define ic0senCOOLING_EVENT_SET	24
#define ic0senCOOLING_EVENT_CHECK	25
#define ic0senCOOLING_EVENT_WAIT	26
#define ic0senCOOLING_EVENT_SETUP	27
#define ic0senCOOLING_EVENT_CFG_ELEM	28
#define ic0senCOOLING_EVENT_GET_VAL	29
#define ic0senCOOLING_EVENT_SET_VAL	30
#define ic0senCOOLING_EVENT_LOG_CONF	31
#define ic0senCOOLING_EVENT_GET_CONF	32
#define ic0senCOOLING_EVENT_MONITOR	33


/* 
 * tables
 */
typedef struct
    {
    vltBYTES16	name;
    vltBYTES32	description;
    vltINT32	flags;
    vltBYTES32	fits;
    vltBYTES8	unit;
    vltINT32	address;
    } ic0senCOOLING_SENSORS;


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senCOOLING_FCTPTR_COM_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senCOOLING_FCTPTR_HW_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senCoolingLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrCooling(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_COOLING: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_COOLING,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senCoolingGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senCoolingReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_STATE,(buf),(error))
#define ic0senCoolingWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_STATE,(void*) (buf),(error))
#define ic0senCoolingReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_SUBSTATE,(buf),(error))
#define ic0senCoolingWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senCoolingReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ERROR,(buf),(error))
#define ic0senCoolingWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ERROR,(void*) (buf),(error))
#define ic0senCoolingReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_INITIALISED,(buf),(error))
#define ic0senCoolingWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senCoolingReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_SIMULATION,(buf),(error))
#define ic0senCoolingWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senCoolingReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_DEVICE,(buf),(error))
#define ic0senCoolingWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senCoolingReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_AVAILABLE,(buf),(error))
#define ic0senCoolingWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senCoolingReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ENABLED,(buf),(error))
#define ic0senCoolingWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senCoolingReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_LOCKED,(buf),(error))
#define ic0senCoolingWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senCoolingReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_HANDSET,(buf),(error))
#define ic0senCoolingWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senCoolingReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_FIXED,(buf),(error))
#define ic0senCoolingWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_FIXED,(void*) (buf),(error))
#define ic0senCoolingReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senCoolingWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senCoolingReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senCoolingWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senCoolingReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senCoolingWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senCoolingReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senCoolingWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senCoolingReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_PREFIX,(buf),(error))
#define ic0senCoolingWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senCoolingReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senCoolingWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senCoolingGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senCOOLING_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senCOOLING_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senCOOLING_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senCoolingWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senCOOLING_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senCoolingGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senCOOLING_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senCoolingWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senCOOLING_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senCoolingReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senCoolingWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senCoolingGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senCOOLING_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senCOOLING_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senCoolingWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senCoolingReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senCoolingWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senCoolingReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senCoolingWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senCoolingReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senCoolingWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senCoolingGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senCOOLING_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senCoolingWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senCOOLING_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senCoolingReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_PARAM_STEP,(buf),(error))
#define ic0senCoolingWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senCoolingReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senCoolingWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senCoolingReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senCoolingWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senCoolingReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senCoolingWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senCoolingReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senCoolingWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senCoolingReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senCoolingWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senCoolingReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senCoolingWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senCoolingReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senCoolingWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senCoolingGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senCOOLING_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senCOOLING_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senCoolingReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_HW_ACCESS,(buf),(error))
#define ic0senCoolingWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senCoolingGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senCOOLING_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senCoolingReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senCOOLING_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senCoolingWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senCOOLING_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senCoolingReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senCoolingWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senCoolingReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senCoolingWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senCoolingReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senCOOLING_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senCoolingWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senCOOLING_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senCoolingGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senCOOLING_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senCoolingHwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senCoolingHwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senCoolingHwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senCoolingHwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senCoolingComRead(obj,devName,fd,simulation,error) \
	ic0senAnalogComRead((obj),(devName),(fd),(simulation),(error))
#define ic0senCoolingSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senCoolingSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senCoolingGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senCoolingLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senCoolingInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senCoolingExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senCoolingUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senCoolingUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senCoolingUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senCoolingUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senCoolingUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senCoolingMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senCoolingUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senCoolingUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senCoolingUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senCoolingUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senCoolingHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senCoolingLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senCoolingGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senCoolingSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senCoolingCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senCoolingResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senCoolingParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senCoolingCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senCoolingCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senCoolingExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senCoolingUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senCoolingGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senCoolingGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senCoolingSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senCoolingSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senCoolingUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senCoolingUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senCoolingGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senCoolingGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senCoolingUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senCoolingSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senCoolingUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senCoolingGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senCoolingUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senCoolingLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senCoolingLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senCoolingLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senCoolingLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senCoolingLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senCoolingLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senCoolingUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senCoolingAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senCoolingUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senCoolingDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senCoolingEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senCoolingLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senCoolingUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senCoolingHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senCoolingHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senCoolingFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senCoolingCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senCoolingWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senCoolingUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senCoolingUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senCoolingUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senCoolingStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senCoolingOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senCoolingOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senCoolingExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senCoolingState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senCoolingSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senCoolingStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senCoolingInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senCoolingStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senCoolingTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senCoolingSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senCoolingAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senCoolingAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senCoolingLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senCoolingOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senCoolingExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senCoolingCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senCoolingBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senCoolingUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senCoolingUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senCoolingComInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senCoolingHwStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senCoolingVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senCOOLING_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senCOOLING_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senCOOLING_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senCOOLING_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senCOOLING_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senCoolingVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senCOOLING_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senCOOLING_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senCOOLING_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senCoolingVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senCOOLING_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senCOOLING_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senCOOLING_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senCOOLING_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senCOOLING_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senCoolingVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senCOOLING_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senCOOLING_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senCoolingVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senCOOLING_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senCoolingVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senCOOLING_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senCOOLING_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senCOOLING_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senCOOLING_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senCOOLING_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senCoolingVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senCOOLING_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senCOOLING_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senCOOLING_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senCOOLING_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senCOOLING_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senCOOLING_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senCoolingVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senCOOLING_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senCOOLING_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senCOOLING_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senCoolingVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senCOOLING_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senCoolingVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senCOOLING_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senCoolingVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senCOOLING_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senCOOLING_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senCOOLING_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senCOOLING_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senCOOLING_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senCOOLING_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senCoolingVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senCOOLING_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senCoolingVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senCOOLING_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senCOOLING_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senCOOLING_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senCOOLING_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senCOOLING_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senCoolingVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senCOOLING_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senCOOLING_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senCOOLING_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senCoolingVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senCOOLING_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senCoolingVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senCOOLING_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senCoolingVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senCOOLING_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senCoolingVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senCOOLING_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senCoolingVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senCOOLING_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senCoolingVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senCOOLING_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senCoolingVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senCOOLING_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senCoolingVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senCOOLING_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senCoolingVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senCOOLING_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senCoolingVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senCoolingVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senCoolingVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senCoolingVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senCOOLING_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senCoolingVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senCOOLING_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senCoolingVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senCOOLING_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senCoolingVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senCOOLING_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senCoolingVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senCOOLING_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senCoolingVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senCOOLING_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senCoolingVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senCOOLING_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senCoolingVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senCOOLING_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senCoolingVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senCOOLING_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senCoolingVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senCOOLING_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senCoolingVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senCOOLING_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senCoolingVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senCOOLING_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senCoolingVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senCOOLING_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senCoolingVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senCOOLING_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senCOOLING_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senCoolingVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senCOOLING_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senCoolingVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senCOOLING_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senCoolingVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senCOOLING_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senCoolingVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senCOOLING_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senCoolingVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senCOOLING_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senCoolingVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senCOOLING_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senCoolingVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senCOOLING_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senCoolingVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senCOOLING_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senCoolingVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senCOOLING_FCTPTR_HW_STATUS)((obj)->fcts[ic0senCOOLING_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senCoolingVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senCOOLING_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senCoolingVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senCOOLING_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senCoolingVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senCOOLING_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senCoolingVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senCOOLING_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senCoolingVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senCOOLING_FCTPTR_COM_INIT)((obj)->fcts[ic0senCOOLING_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senCoolingVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_READ)((obj)->fcts[ic0senCOOLING_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senCoolingNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senCoolingDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_COOLING_H

/************************************************************************/
/* ___oOo___ */
