/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senEsters.h,v 1.51 2005/08/15 13:36:28 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  xxxxmgr    03/09/01  Created by eccsgen  Revision: 1.27  
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senEsters - class
*
*   SYNOPSIS
*	#include "ic0senEsters.h"
*
*	ccsCOMPL_STAT ic0senEstersNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senEstersGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senEstersLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senEstersComRead(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senEstersWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrEsters(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senESTERS allows to read temperatures
*       via Esters DC24 devices connected to a single
*       serial line.
*
*	ic0senEstersNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senESTERS.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senEstersDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senESTERS
*
*
*	ic0senEstersErr<error>(...)
*	    The macros ic0senEstersErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senEstersRead<attr>(...)
*	ic0senEstersWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senEstersDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senEstersComRead(obj,devName,fd,simulation,error)
*	   ComRead is called to read data from the serial port.
*	   It updates the last vector attribute.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_ESTERS
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_ESTERS_H
#define IC0SEN_ESTERS_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senESTERS_ID	199

/*
 * attribute table
 */
#define ic0senESTERS_ATTR_STATE	0
#define ic0senESTERS_ATTR_SUBSTATE	1
#define ic0senESTERS_ATTR_ERROR	2
#define ic0senESTERS_ATTR_INITIALISED	3
#define ic0senESTERS_ATTR_SIMULATION	4
#define ic0senESTERS_ATTR_DEVICE	5
#define ic0senESTERS_ATTR_AVAILABLE	6
#define ic0senESTERS_ATTR_ENABLED	7
#define ic0senESTERS_ATTR_LOCKED	8
#define ic0senESTERS_ATTR_HANDSET	9
#define ic0senESTERS_ATTR_FIXED	10
#define ic0senESTERS_ATTR_FIX_POS_SET	11
#define ic0senESTERS_ATTR_FIXED_POSITION	12
#define ic0senESTERS_ATTR_FIXED_SIMULATION	13
#define ic0senESTERS_ATTR_FIXED_LOCKED	14
#define ic0senESTERS_ATTR_PREFIX	15
#define ic0senESTERS_ATTR_PARAMETERS_SIZE	16
#define ic0senESTERS_ATTR_PARAMETERS	17
#define ic0senESTERS_ATTR_STATUS	18
#define ic0senESTERS_ATTR_STATUS_INDEX	19
#define ic0senESTERS_ATTR_VALUES_SIZE	20
#define ic0senESTERS_ATTR_VALUES	21
#define ic0senESTERS_ATTR_INIT_RESTRICTION	22
#define ic0senESTERS_ATTR_MOVE_RESTRICTION	23
#define ic0senESTERS_ATTR_STOP_TIMEOUT	24
#define ic0senESTERS_ATTR_SET_VAL_FCT	25
#define ic0senESTERS_ATTR_HELP_TEXT	26
#define ic0senESTERS_ATTR_PARAM_STEP	27
#define ic0senESTERS_ATTR_ACTION_COMMAND	28
#define ic0senESTERS_ATTR_ACTION_STOP_REQ	29
#define ic0senESTERS_ATTR_ACTION_NO_MOVE	30
#define ic0senESTERS_ATTR_ACTION_CHECK	31
#define ic0senESTERS_ATTR_RESET_SETUP_CMD	32
#define ic0senESTERS_ATTR_SENSORS_PREFIX	33
#define ic0senESTERS_ATTR_SENSORS_SIZE	34
#define ic0senESTERS_ATTR_SENSORS	35
#define ic0senESTERS_ATTR_HW_ACCESS	36
#define ic0senESTERS_ATTR_LAST	37
#define ic0senESTERS_ATTR_LAST_ACCESS	38
#define ic0senESTERS_ATTR_MIN_ACCESS	39
#define ic0senESTERS_ATTR_IC0SENFLAG	40

#define ic0senESTERS_AUTOMATA	0

/*
 * function table
 */
#define ic0senESTERS_FCT_DELETE_OBJ	0
#define ic0senESTERS_FCT_STANDBY	1
#define ic0senESTERS_FCT_ONLINE	2
#define ic0senESTERS_FCT_OFF	3
#define ic0senESTERS_FCT_EXIT	4
#define ic0senESTERS_FCT_STATE	5
#define ic0senESTERS_FCT_SIMULAT	6
#define ic0senESTERS_FCT_STOP_SIM	7
#define ic0senESTERS_FCT_STATUS	8
#define ic0senESTERS_FCT_INIT	9
#define ic0senESTERS_FCT_STOP	10
#define ic0senESTERS_FCT_TEST	11
#define ic0senESTERS_FCT_SELFTST	12
#define ic0senESTERS_FCT_AUTO_TBL	13
#define ic0senESTERS_FCT_AUTO_RST	14
#define ic0senESTERS_FCT_INIT_OBJ	15
#define ic0senESTERS_FCT_EXIT_OBJ	16
#define ic0senESTERS_FCT_LOADED2STBY	17
#define ic0senESTERS_FCT_ONLINE2STBY	18
#define ic0senESTERS_FCT_EXIT1	19
#define ic0senESTERS_FCT_CHECK_DEVICE_NAME	20
#define ic0senESTERS_FCT_BREAK	21
#define ic0senESTERS_FCT_USR_INIT	22
#define ic0senESTERS_FCT_USR_LOADED2STANDBY	23
#define ic0senESTERS_FCT_USR_STANDBY2ONLINE	24
#define ic0senESTERS_FCT_USR_ONLINE2STANDBY	25
#define ic0senESTERS_FCT_USR_STANDBY2LOADED	26
#define ic0senESTERS_FCT_USR_STOP	27
#define ic0senESTERS_FCT_USR_SIMULAT	28
#define ic0senESTERS_FCT_USR_STOP_SIM	29
#define ic0senESTERS_FCT_USR_STATUS_CODE	30
#define ic0senESTERS_FCT_USR_STATUS_LAST	31
#define ic0senESTERS_FCT_USR_STATUS_VALUE	32
#define ic0senESTERS_FCT_USR_TEST	33
#define ic0senESTERS_FCT_USR_SELF_TEST	34
#define ic0senESTERS_FCT_USR_ERROR	35
#define ic0senESTERS_FCT_AVAIL	36
#define ic0senESTERS_FCT_UNAVAIL	37
#define ic0senESTERS_FCT_DISABLE	38
#define ic0senESTERS_FCT_ENABLE	39
#define ic0senESTERS_FCT_LOCK	40
#define ic0senESTERS_FCT_UNLOCK	41
#define ic0senESTERS_FCT_HANDSET	42
#define ic0senESTERS_FCT_HSET_OFF	43
#define ic0senESTERS_FCT_FIX_POS	44
#define ic0senESTERS_FCT_MOVE	45
#define ic0senESTERS_FCT_MOVE_REL	46
#define ic0senESTERS_FCT_SET	47
#define ic0senESTERS_FCT_CHECK	48
#define ic0senESTERS_FCT_WAIT	49
#define ic0senESTERS_FCT_USR_SET_MODE	50
#define ic0senESTERS_FCT_SETUP	51
#define ic0senESTERS_FCT_GET_VAL	52
#define ic0senESTERS_FCT_SET_VAL	53
#define ic0senESTERS_FCT_CFG_ELEM	54
#define ic0senESTERS_FCT_LOG_CONF	55
#define ic0senESTERS_FCT_GET_CONF	56
#define ic0senESTERS_FCT_RESET_PARAMETERS	57
#define ic0senESTERS_FCT_PARSE_PARAMETERS	58
#define ic0senESTERS_FCT_COMPARE_PARAMETERS	59
#define ic0senESTERS_FCT_COPY_PARAMETERS	60
#define ic0senESTERS_FCT_EXEC_PARAMETERS	61
#define ic0senESTERS_FCT_UPDATE_PARAMETERS	62
#define ic0senESTERS_FCT_GET_PARAM_SET_VALUE	63
#define ic0senESTERS_FCT_GET_PARAM_STATUS	64
#define ic0senESTERS_FCT_SET_PARAM	65
#define ic0senESTERS_FCT_SET_PARAM_STEP	66
#define ic0senESTERS_FCT_USR_EXEC_ACTION	67
#define ic0senESTERS_FCT_USR_SET_PARAMETER	68
#define ic0senESTERS_FCT_GET_VALUE_CODE	69
#define ic0senESTERS_FCT_GET_VALUE	70
#define ic0senESTERS_FCT_USR_GET_VALUE	71
#define ic0senESTERS_FCT_SET_VALUE	72
#define ic0senESTERS_FCT_USR_SET_VALUE	73
#define ic0senESTERS_FCT_GET_ARG	74
#define ic0senESTERS_FCT_MONITOR	75
#define ic0senESTERS_FCT_USR_MONITOR	76
#define ic0senESTERS_FCT_LOG_ACTION	77
#define ic0senESTERS_FCT_LOG_COMMENT	78
#define ic0senESTERS_FCT_LOG_PARAMETER	79
#define ic0senESTERS_FCT_LOG_RECOVERY	80
#define ic0senESTERS_FCT_LOG_TBL_PARAM	81
#define ic0senESTERS_FCT_LOG_UNFORSEEN	82
#define ic0senESTERS_FCT_HW_INIT	83
#define ic0senESTERS_FCT_HW_EXIT	84
#define ic0senESTERS_FCT_HW_STATUS	85
#define ic0senESTERS_FCT_HW_SETUP	86
#define ic0senESTERS_FCT_HW_GET_CONF	87
#define ic0senESTERS_FCT_HW_LOG_CONF	88
#define ic0senESTERS_FCT_LOG_STATUS	89
#define ic0senESTERS_FCT_COM_INIT	90
#define ic0senESTERS_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senESTERS_STATE_NOSTATE	-1
#define ic0senESTERS_STATE_OFF	0
#define ic0senESTERS_STATE_LOADED	1
#define ic0senESTERS_STATE_STANDBY	2
#define ic0senESTERS_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senESTERS_SUBSTATE_ERROR	0
#define ic0senESTERS_SUBSTATE_IDLE	1
#define ic0senESTERS_SUBSTATE_BUSY	2
#define ic0senESTERS_SUBSTATE_INITIALISING	3
#define ic0senESTERS_SUBSTATE_SHUTTING_DOWN	4
#define ic0senESTERS_SUBSTATE_STOPPING	5
#define ic0senESTERS_SUBSTATE_COLD_STARTING	6
#define ic0senESTERS_SUBSTATE_STARTING	7
#define ic0senESTERS_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senESTERS_EVENT_INIT	0
#define ic0senESTERS_EVENT_STANDBY	1
#define ic0senESTERS_EVENT_ONLINE	2
#define ic0senESTERS_EVENT_STOP	3
#define ic0senESTERS_EVENT_OFF	4
#define ic0senESTERS_EVENT_EXIT	5
#define ic0senESTERS_EVENT_STATE	6
#define ic0senESTERS_EVENT_STOP_SIM	7
#define ic0senESTERS_EVENT_STATUS	8
#define ic0senESTERS_EVENT_SIMULAT	9
#define ic0senESTERS_EVENT_TEST	10
#define ic0senESTERS_EVENT_SELFTST	11
#define ic0senESTERS_EVENT_AUTO_TBL	12
#define ic0senESTERS_EVENT_AUTO_RST	13
#define ic0senESTERS_EVENT_DISABLE	14
#define ic0senESTERS_EVENT_ENABLE	15
#define ic0senESTERS_EVENT_LOCK	16
#define ic0senESTERS_EVENT_UNLOCK	17
#define ic0senESTERS_EVENT_HANDSET	18
#define ic0senESTERS_EVENT_HSET_OFF	19
#define ic0senESTERS_EVENT_AVAIL	20
#define ic0senESTERS_EVENT_UNAVAIL	21
#define ic0senESTERS_EVENT_MOVE	22
#define ic0senESTERS_EVENT_MOVE_REL	23
#define ic0senESTERS_EVENT_SET	24
#define ic0senESTERS_EVENT_CHECK	25
#define ic0senESTERS_EVENT_WAIT	26
#define ic0senESTERS_EVENT_SETUP	27
#define ic0senESTERS_EVENT_CFG_ELEM	28
#define ic0senESTERS_EVENT_GET_VAL	29
#define ic0senESTERS_EVENT_SET_VAL	30
#define ic0senESTERS_EVENT_LOG_CONF	31
#define ic0senESTERS_EVENT_GET_CONF	32
#define ic0senESTERS_EVENT_MONITOR	33


/*
 * tables
 */
typedef struct
    {
    vltBYTES16	name;
    vltBYTES32	description;
    vltINT32	flags;
    vltBYTES32	fits;
    vltBYTES8	unit;
    vltINT32	address;
    } ic0senESTERS_SENSORS;


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senESTERS_FCTPTR_COM_READ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senEstersLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrEsters(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_ESTERS: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_ESTERS,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senEstersGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senEstersReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_STATE,(buf),(error))
#define ic0senEstersWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_STATE,(void*) (buf),(error))
#define ic0senEstersReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_SUBSTATE,(buf),(error))
#define ic0senEstersWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senEstersReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ERROR,(buf),(error))
#define ic0senEstersWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ERROR,(void*) (buf),(error))
#define ic0senEstersReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_INITIALISED,(buf),(error))
#define ic0senEstersWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senEstersReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_SIMULATION,(buf),(error))
#define ic0senEstersWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senEstersReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_DEVICE,(buf),(error))
#define ic0senEstersWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senEstersReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_AVAILABLE,(buf),(error))
#define ic0senEstersWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senEstersReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ENABLED,(buf),(error))
#define ic0senEstersWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senEstersReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_LOCKED,(buf),(error))
#define ic0senEstersWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senEstersReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_HANDSET,(buf),(error))
#define ic0senEstersWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senEstersReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_FIXED,(buf),(error))
#define ic0senEstersWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_FIXED,(void*) (buf),(error))
#define ic0senEstersReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senEstersWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senEstersReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senEstersWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senEstersReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senEstersWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senEstersReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senEstersWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senEstersReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_PREFIX,(buf),(error))
#define ic0senEstersWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senEstersReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senEstersWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senEstersGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senESTERS_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senESTERS_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senESTERS_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senEstersWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senESTERS_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senEstersGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senESTERS_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senEstersWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senESTERS_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senEstersReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senEstersWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senEstersGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senESTERS_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senESTERS_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senEstersWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senEstersReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senEstersWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senEstersReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senEstersWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senEstersReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senEstersWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senEstersGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senESTERS_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senEstersWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senESTERS_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senEstersReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_PARAM_STEP,(buf),(error))
#define ic0senEstersWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senEstersReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senEstersWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senEstersReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senEstersWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senEstersReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senEstersWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senEstersReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senEstersWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senEstersReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senEstersWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senEstersReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senEstersWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senEstersReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senEstersWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senEstersGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senESTERS_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senESTERS_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senEstersReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_HW_ACCESS,(buf),(error))
#define ic0senEstersWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senEstersGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senESTERS_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senEstersReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senESTERS_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senEstersWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senESTERS_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senEstersReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senEstersWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senEstersReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senEstersWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senEstersReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senESTERS_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senEstersWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senESTERS_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senEstersGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senESTERS_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senEstersHwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senEstersHwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senEstersHwStatus(obj,devName,state,simulation,error) \
	ic0senAnalogHwStatus((obj),(devName),(state),(simulation),(error))
#define ic0senEstersHwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senEstersHwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senEstersComInit(obj,devName,fd,simulation,error) \
	ic0senAnalogComInit((obj),(devName),(fd),(simulation),(error))
#define ic0senEstersSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senEstersSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senEstersGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senEstersLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senEstersInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senEstersExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senEstersUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senEstersUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senEstersUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senEstersUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senEstersUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senEstersMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senEstersUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senEstersUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senEstersUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senEstersUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senEstersHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senEstersLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senEstersGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senEstersSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senEstersCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senEstersResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senEstersParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senEstersCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senEstersCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senEstersExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senEstersUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senEstersGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senEstersGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senEstersSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senEstersSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senEstersUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senEstersUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senEstersGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senEstersGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senEstersUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senEstersSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senEstersUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senEstersGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senEstersUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senEstersLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senEstersLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senEstersLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senEstersLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senEstersLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senEstersLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senEstersUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senEstersAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senEstersUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senEstersDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senEstersEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senEstersLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senEstersUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senEstersHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senEstersHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senEstersFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senEstersCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senEstersWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senEstersUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senEstersUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senEstersUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senEstersStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senEstersOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senEstersOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senEstersExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senEstersState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senEstersSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senEstersStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senEstersInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senEstersStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senEstersTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senEstersSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senEstersAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senEstersAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senEstersLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senEstersOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senEstersExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senEstersCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senEstersBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senEstersUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senEstersUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senEstersComRead(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senEstersVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senESTERS_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senESTERS_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senESTERS_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senESTERS_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senESTERS_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senEstersVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senESTERS_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senESTERS_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senESTERS_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senEstersVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senESTERS_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senESTERS_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senESTERS_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senESTERS_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senESTERS_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senEstersVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senESTERS_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senESTERS_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senEstersVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senESTERS_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senEstersVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senESTERS_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senESTERS_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senESTERS_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senESTERS_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senESTERS_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senEstersVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senESTERS_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senESTERS_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senESTERS_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senESTERS_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senESTERS_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senESTERS_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senEstersVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senESTERS_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senESTERS_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senESTERS_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senEstersVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senESTERS_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senEstersVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senESTERS_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senEstersVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senESTERS_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senESTERS_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senESTERS_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senESTERS_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senESTERS_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senESTERS_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senEstersVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senESTERS_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senEstersVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senESTERS_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senESTERS_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senESTERS_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senESTERS_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senESTERS_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senEstersVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senESTERS_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senESTERS_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senESTERS_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senEstersVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senESTERS_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senEstersVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senESTERS_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senEstersVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senESTERS_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senEstersVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senESTERS_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senEstersVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senESTERS_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senEstersVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senESTERS_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senEstersVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senESTERS_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senEstersVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senESTERS_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senEstersVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senESTERS_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senEstersVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senEstersVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senEstersVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senEstersVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senESTERS_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senEstersVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senESTERS_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senEstersVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senESTERS_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senEstersVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senESTERS_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senEstersVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senESTERS_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senEstersVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senESTERS_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senEstersVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senESTERS_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senEstersVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senESTERS_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senEstersVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senESTERS_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senEstersVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senESTERS_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senEstersVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senESTERS_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senEstersVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senESTERS_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senEstersVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senESTERS_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senEstersVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senESTERS_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senESTERS_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senEstersVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senESTERS_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senEstersVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senESTERS_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senEstersVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senESTERS_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senEstersVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senESTERS_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senEstersVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senESTERS_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senEstersVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senESTERS_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senEstersVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senESTERS_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senEstersVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senESTERS_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senEstersVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_STATUS)((obj)->fcts[ic0senESTERS_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senEstersVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senESTERS_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senEstersVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senESTERS_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senEstersVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senESTERS_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senEstersVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senESTERS_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senEstersVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_INIT)((obj)->fcts[ic0senESTERS_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senEstersVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senESTERS_FCTPTR_COM_READ)((obj)->fcts[ic0senESTERS_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senEstersNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senEstersDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_ESTERS_H

/************************************************************************/
/* ___oOo___ */
