/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senHumidity.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   21/12/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senHumidity - class
*
*   SYNOPSIS
*	#include "ic0senHumidity.h"
*
*	ccsCOMPL_STAT ic0senHumidityNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senHumidityGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senHumidityLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senHumidityComInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityComRead(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senHumidityWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrHumidity(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senHUMIDITY allows to access a humidity
*       sensor.  The sensor supplies temperature, relative
*       humidity and due point (in this order in the sensors
*       table).
*
*	ic0senHumidityNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senHUMIDITY.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senHumidityDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senHUMIDITY
*
*
*	ic0senHumidityErr<error>(...)
*	    The macros ic0senHumidityErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senHumidityRead<attr>(...)
*	ic0senHumidityWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senHumidityDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senHumidityComInit(obj,devName,fd,simulation,error)
*	   ComInit is called after opening the serial port.
*
*	ic0senHumidityComRead(obj,devName,fd,simulation,error)
*	   ComRead is called to read data from the serial port.
*	   It updates the last vector attribute.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_HUMIDITY
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_HUMIDITY_H
#define IC0SEN_HUMIDITY_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senHUMIDITY_ID	196

/*
 * attribute table
 */
#define ic0senHUMIDITY_ATTR_STATE	0
#define ic0senHUMIDITY_ATTR_SUBSTATE	1
#define ic0senHUMIDITY_ATTR_ERROR	2
#define ic0senHUMIDITY_ATTR_INITIALISED	3
#define ic0senHUMIDITY_ATTR_SIMULATION	4
#define ic0senHUMIDITY_ATTR_DEVICE	5
#define ic0senHUMIDITY_ATTR_AVAILABLE	6
#define ic0senHUMIDITY_ATTR_ENABLED	7
#define ic0senHUMIDITY_ATTR_LOCKED	8
#define ic0senHUMIDITY_ATTR_HANDSET	9
#define ic0senHUMIDITY_ATTR_FIXED	10
#define ic0senHUMIDITY_ATTR_FIX_POS_SET	11
#define ic0senHUMIDITY_ATTR_FIXED_POSITION	12
#define ic0senHUMIDITY_ATTR_FIXED_SIMULATION	13
#define ic0senHUMIDITY_ATTR_FIXED_LOCKED	14
#define ic0senHUMIDITY_ATTR_PREFIX	15
#define ic0senHUMIDITY_ATTR_PARAMETERS_SIZE	16
#define ic0senHUMIDITY_ATTR_PARAMETERS	17
#define ic0senHUMIDITY_ATTR_STATUS	18
#define ic0senHUMIDITY_ATTR_STATUS_INDEX	19
#define ic0senHUMIDITY_ATTR_VALUES_SIZE	20
#define ic0senHUMIDITY_ATTR_VALUES	21
#define ic0senHUMIDITY_ATTR_INIT_RESTRICTION	22
#define ic0senHUMIDITY_ATTR_MOVE_RESTRICTION	23
#define ic0senHUMIDITY_ATTR_STOP_TIMEOUT	24
#define ic0senHUMIDITY_ATTR_SET_VAL_FCT	25
#define ic0senHUMIDITY_ATTR_HELP_TEXT	26
#define ic0senHUMIDITY_ATTR_PARAM_STEP	27
#define ic0senHUMIDITY_ATTR_ACTION_COMMAND	28
#define ic0senHUMIDITY_ATTR_ACTION_STOP_REQ	29
#define ic0senHUMIDITY_ATTR_ACTION_NO_MOVE	30
#define ic0senHUMIDITY_ATTR_ACTION_CHECK	31
#define ic0senHUMIDITY_ATTR_RESET_SETUP_CMD	32
#define ic0senHUMIDITY_ATTR_SENSORS_PREFIX	33
#define ic0senHUMIDITY_ATTR_SENSORS_SIZE	34
#define ic0senHUMIDITY_ATTR_SENSORS	35
#define ic0senHUMIDITY_ATTR_HW_ACCESS	36
#define ic0senHUMIDITY_ATTR_LAST	37
#define ic0senHUMIDITY_ATTR_LAST_ACCESS	38
#define ic0senHUMIDITY_ATTR_MIN_ACCESS	39
#define ic0senHUMIDITY_ATTR_IC0SENFLAG	40

#define ic0senHUMIDITY_AUTOMATA	0

/*
 * function table
 */
#define ic0senHUMIDITY_FCT_DELETE_OBJ	0
#define ic0senHUMIDITY_FCT_STANDBY	1
#define ic0senHUMIDITY_FCT_ONLINE	2
#define ic0senHUMIDITY_FCT_OFF	3
#define ic0senHUMIDITY_FCT_EXIT	4
#define ic0senHUMIDITY_FCT_STATE	5
#define ic0senHUMIDITY_FCT_SIMULAT	6
#define ic0senHUMIDITY_FCT_STOP_SIM	7
#define ic0senHUMIDITY_FCT_STATUS	8
#define ic0senHUMIDITY_FCT_INIT	9
#define ic0senHUMIDITY_FCT_STOP	10
#define ic0senHUMIDITY_FCT_TEST	11
#define ic0senHUMIDITY_FCT_SELFTST	12
#define ic0senHUMIDITY_FCT_AUTO_TBL	13
#define ic0senHUMIDITY_FCT_AUTO_RST	14
#define ic0senHUMIDITY_FCT_INIT_OBJ	15
#define ic0senHUMIDITY_FCT_EXIT_OBJ	16
#define ic0senHUMIDITY_FCT_LOADED2STBY	17
#define ic0senHUMIDITY_FCT_ONLINE2STBY	18
#define ic0senHUMIDITY_FCT_EXIT1	19
#define ic0senHUMIDITY_FCT_CHECK_DEVICE_NAME	20
#define ic0senHUMIDITY_FCT_BREAK	21
#define ic0senHUMIDITY_FCT_USR_INIT	22
#define ic0senHUMIDITY_FCT_USR_LOADED2STANDBY	23
#define ic0senHUMIDITY_FCT_USR_STANDBY2ONLINE	24
#define ic0senHUMIDITY_FCT_USR_ONLINE2STANDBY	25
#define ic0senHUMIDITY_FCT_USR_STANDBY2LOADED	26
#define ic0senHUMIDITY_FCT_USR_STOP	27
#define ic0senHUMIDITY_FCT_USR_SIMULAT	28
#define ic0senHUMIDITY_FCT_USR_STOP_SIM	29
#define ic0senHUMIDITY_FCT_USR_STATUS_CODE	30
#define ic0senHUMIDITY_FCT_USR_STATUS_LAST	31
#define ic0senHUMIDITY_FCT_USR_STATUS_VALUE	32
#define ic0senHUMIDITY_FCT_USR_TEST	33
#define ic0senHUMIDITY_FCT_USR_SELF_TEST	34
#define ic0senHUMIDITY_FCT_USR_ERROR	35
#define ic0senHUMIDITY_FCT_AVAIL	36
#define ic0senHUMIDITY_FCT_UNAVAIL	37
#define ic0senHUMIDITY_FCT_DISABLE	38
#define ic0senHUMIDITY_FCT_ENABLE	39
#define ic0senHUMIDITY_FCT_LOCK	40
#define ic0senHUMIDITY_FCT_UNLOCK	41
#define ic0senHUMIDITY_FCT_HANDSET	42
#define ic0senHUMIDITY_FCT_HSET_OFF	43
#define ic0senHUMIDITY_FCT_FIX_POS	44
#define ic0senHUMIDITY_FCT_MOVE	45
#define ic0senHUMIDITY_FCT_MOVE_REL	46
#define ic0senHUMIDITY_FCT_SET	47
#define ic0senHUMIDITY_FCT_CHECK	48
#define ic0senHUMIDITY_FCT_WAIT	49
#define ic0senHUMIDITY_FCT_USR_SET_MODE	50
#define ic0senHUMIDITY_FCT_SETUP	51
#define ic0senHUMIDITY_FCT_GET_VAL	52
#define ic0senHUMIDITY_FCT_SET_VAL	53
#define ic0senHUMIDITY_FCT_CFG_ELEM	54
#define ic0senHUMIDITY_FCT_LOG_CONF	55
#define ic0senHUMIDITY_FCT_GET_CONF	56
#define ic0senHUMIDITY_FCT_RESET_PARAMETERS	57
#define ic0senHUMIDITY_FCT_PARSE_PARAMETERS	58
#define ic0senHUMIDITY_FCT_COMPARE_PARAMETERS	59
#define ic0senHUMIDITY_FCT_COPY_PARAMETERS	60
#define ic0senHUMIDITY_FCT_EXEC_PARAMETERS	61
#define ic0senHUMIDITY_FCT_UPDATE_PARAMETERS	62
#define ic0senHUMIDITY_FCT_GET_PARAM_SET_VALUE	63
#define ic0senHUMIDITY_FCT_GET_PARAM_STATUS	64
#define ic0senHUMIDITY_FCT_SET_PARAM	65
#define ic0senHUMIDITY_FCT_SET_PARAM_STEP	66
#define ic0senHUMIDITY_FCT_USR_EXEC_ACTION	67
#define ic0senHUMIDITY_FCT_USR_SET_PARAMETER	68
#define ic0senHUMIDITY_FCT_GET_VALUE_CODE	69
#define ic0senHUMIDITY_FCT_GET_VALUE	70
#define ic0senHUMIDITY_FCT_USR_GET_VALUE	71
#define ic0senHUMIDITY_FCT_SET_VALUE	72
#define ic0senHUMIDITY_FCT_USR_SET_VALUE	73
#define ic0senHUMIDITY_FCT_GET_ARG	74
#define ic0senHUMIDITY_FCT_MONITOR	75
#define ic0senHUMIDITY_FCT_USR_MONITOR	76
#define ic0senHUMIDITY_FCT_LOG_ACTION	77
#define ic0senHUMIDITY_FCT_LOG_COMMENT	78
#define ic0senHUMIDITY_FCT_LOG_PARAMETER	79
#define ic0senHUMIDITY_FCT_LOG_RECOVERY	80
#define ic0senHUMIDITY_FCT_LOG_TBL_PARAM	81
#define ic0senHUMIDITY_FCT_LOG_UNFORSEEN	82
#define ic0senHUMIDITY_FCT_HW_INIT	83
#define ic0senHUMIDITY_FCT_HW_EXIT	84
#define ic0senHUMIDITY_FCT_HW_STATUS	85
#define ic0senHUMIDITY_FCT_HW_SETUP	86
#define ic0senHUMIDITY_FCT_HW_GET_CONF	87
#define ic0senHUMIDITY_FCT_HW_LOG_CONF	88
#define ic0senHUMIDITY_FCT_LOG_STATUS	89
#define ic0senHUMIDITY_FCT_COM_INIT	90
#define ic0senHUMIDITY_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senHUMIDITY_STATE_NOSTATE	-1
#define ic0senHUMIDITY_STATE_OFF	0
#define ic0senHUMIDITY_STATE_LOADED	1
#define ic0senHUMIDITY_STATE_STANDBY	2
#define ic0senHUMIDITY_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senHUMIDITY_SUBSTATE_ERROR	0
#define ic0senHUMIDITY_SUBSTATE_IDLE	1
#define ic0senHUMIDITY_SUBSTATE_BUSY	2
#define ic0senHUMIDITY_SUBSTATE_INITIALISING	3
#define ic0senHUMIDITY_SUBSTATE_SHUTTING_DOWN	4
#define ic0senHUMIDITY_SUBSTATE_STOPPING	5
#define ic0senHUMIDITY_SUBSTATE_COLD_STARTING	6
#define ic0senHUMIDITY_SUBSTATE_STARTING	7
#define ic0senHUMIDITY_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senHUMIDITY_EVENT_INIT	0
#define ic0senHUMIDITY_EVENT_STANDBY	1
#define ic0senHUMIDITY_EVENT_ONLINE	2
#define ic0senHUMIDITY_EVENT_STOP	3
#define ic0senHUMIDITY_EVENT_OFF	4
#define ic0senHUMIDITY_EVENT_EXIT	5
#define ic0senHUMIDITY_EVENT_STATE	6
#define ic0senHUMIDITY_EVENT_STOP_SIM	7
#define ic0senHUMIDITY_EVENT_STATUS	8
#define ic0senHUMIDITY_EVENT_SIMULAT	9
#define ic0senHUMIDITY_EVENT_TEST	10
#define ic0senHUMIDITY_EVENT_SELFTST	11
#define ic0senHUMIDITY_EVENT_AUTO_TBL	12
#define ic0senHUMIDITY_EVENT_AUTO_RST	13
#define ic0senHUMIDITY_EVENT_DISABLE	14
#define ic0senHUMIDITY_EVENT_ENABLE	15
#define ic0senHUMIDITY_EVENT_LOCK	16
#define ic0senHUMIDITY_EVENT_UNLOCK	17
#define ic0senHUMIDITY_EVENT_HANDSET	18
#define ic0senHUMIDITY_EVENT_HSET_OFF	19
#define ic0senHUMIDITY_EVENT_AVAIL	20
#define ic0senHUMIDITY_EVENT_UNAVAIL	21
#define ic0senHUMIDITY_EVENT_MOVE	22
#define ic0senHUMIDITY_EVENT_MOVE_REL	23
#define ic0senHUMIDITY_EVENT_SET	24
#define ic0senHUMIDITY_EVENT_CHECK	25
#define ic0senHUMIDITY_EVENT_WAIT	26
#define ic0senHUMIDITY_EVENT_SETUP	27
#define ic0senHUMIDITY_EVENT_CFG_ELEM	28
#define ic0senHUMIDITY_EVENT_GET_VAL	29
#define ic0senHUMIDITY_EVENT_SET_VAL	30
#define ic0senHUMIDITY_EVENT_LOG_CONF	31
#define ic0senHUMIDITY_EVENT_GET_CONF	32
#define ic0senHUMIDITY_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senHUMIDITY_FCTPTR_COM_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senHUMIDITY_FCTPTR_COM_READ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senHumidityLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrHumidity(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_HUMIDITY: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_HUMIDITY,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senHumidityGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senHumidityReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_STATE,(buf),(error))
#define ic0senHumidityWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_STATE,(void*) (buf),(error))
#define ic0senHumidityReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_SUBSTATE,(buf),(error))
#define ic0senHumidityWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senHumidityReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ERROR,(buf),(error))
#define ic0senHumidityWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ERROR,(void*) (buf),(error))
#define ic0senHumidityReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_INITIALISED,(buf),(error))
#define ic0senHumidityWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senHumidityReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_SIMULATION,(buf),(error))
#define ic0senHumidityWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senHumidityReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_DEVICE,(buf),(error))
#define ic0senHumidityWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senHumidityReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_AVAILABLE,(buf),(error))
#define ic0senHumidityWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senHumidityReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ENABLED,(buf),(error))
#define ic0senHumidityWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senHumidityReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_LOCKED,(buf),(error))
#define ic0senHumidityWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senHumidityReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_HANDSET,(buf),(error))
#define ic0senHumidityWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senHumidityReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_FIXED,(buf),(error))
#define ic0senHumidityWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_FIXED,(void*) (buf),(error))
#define ic0senHumidityReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senHumidityWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senHumidityReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senHumidityWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senHumidityReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senHumidityWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senHumidityReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senHumidityWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senHumidityReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_PREFIX,(buf),(error))
#define ic0senHumidityWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senHumidityReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senHumidityWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senHumidityGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senHUMIDITY_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senHUMIDITY_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senHUMIDITY_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senHumidityWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senHUMIDITY_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senHumidityGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senHUMIDITY_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senHumidityWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senHUMIDITY_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senHumidityReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senHumidityWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senHumidityGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senHUMIDITY_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senHUMIDITY_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senHumidityWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senHumidityReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senHumidityWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senHumidityReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senHumidityWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senHumidityReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senHumidityWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senHumidityGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senHUMIDITY_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senHumidityWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senHUMIDITY_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senHumidityReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_PARAM_STEP,(buf),(error))
#define ic0senHumidityWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senHumidityReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senHumidityWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senHumidityReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senHumidityWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senHumidityReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senHumidityWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senHumidityReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senHumidityWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senHumidityReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senHumidityWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senHumidityReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senHumidityWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senHumidityReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senHumidityWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senHumidityGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senHUMIDITY_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senHUMIDITY_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senHumidityReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_HW_ACCESS,(buf),(error))
#define ic0senHumidityWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senHumidityGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senHUMIDITY_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senHumidityReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senHUMIDITY_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senHumidityWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senHUMIDITY_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senHumidityReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senHumidityWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senHumidityReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senHumidityWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senHumidityReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senHUMIDITY_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senHumidityWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senHUMIDITY_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senHumidityGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senHUMIDITY_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senHumidityHwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityHwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityHwStatus(obj,devName,state,simulation,error) \
	ic0senAnalogHwStatus((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityHwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senHumidityHwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senHumiditySetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senHumiditySet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senHumidityGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senHumidityLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senHumidityInitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senHumidityExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senHumidityUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senHumidityUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senHumidityUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senHumidityUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senHumidityUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senHumidityMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senHumidityUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senHumidityUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senHumidityUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senHumidityUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senHumidityHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senHumidityLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senHumidityGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senHumiditySetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senHumidityCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senHumidityResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senHumidityParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senHumidityCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senHumidityCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senHumidityExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senHumidityUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senHumidityGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senHumidityGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senHumiditySetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senHumiditySetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senHumidityUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senHumidityUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senHumidityGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senHumidityGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senHumidityUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senHumiditySetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senHumidityUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senHumidityGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senHumidityUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senHumidityLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senHumidityLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senHumidityLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senHumidityLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senHumidityLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senHumidityLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senHumidityUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senHumidityAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senHumidityUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senHumidityDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senHumidityEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senHumidityLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senHumidityUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senHumidityHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senHumidityHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senHumidityFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senHumidityCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senHumidityWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senHumidityUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senHumidityUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senHumidityUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senHumidityStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senHumidityOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senHumidityOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senHumidityExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senHumidityState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senHumiditySimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senHumidityStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senHumidityInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senHumidityStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senHumidityTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senHumiditySelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senHumidityAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senHumidityAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senHumidityLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senHumidityOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senHumidityExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senHumidityCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senHumidityBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senHumidityUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senHumidityUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senHumidityComInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senHumidityComRead(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senHumidityVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senHUMIDITY_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senHUMIDITY_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senHUMIDITY_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senHUMIDITY_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senHUMIDITY_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senHumidityVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senHUMIDITY_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senHUMIDITY_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senHUMIDITY_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senHumidityVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senHUMIDITY_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senHUMIDITY_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senHUMIDITY_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senHUMIDITY_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senHUMIDITY_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senHumidityVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senHUMIDITY_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senHUMIDITY_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senHumidityVExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senHUMIDITY_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senHumidityVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senHUMIDITY_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senHUMIDITY_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senHUMIDITY_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senHUMIDITY_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senHUMIDITY_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senHumidityVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senHUMIDITY_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senHUMIDITY_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senHUMIDITY_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senHumidityVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senHUMIDITY_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senHumidityVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senHumidityVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senHumidityVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senHUMIDITY_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senHUMIDITY_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senHUMIDITY_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senHUMIDITY_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senHUMIDITY_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senHUMIDITY_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senHumidityVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senHUMIDITY_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senHumidityVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senHUMIDITY_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senHUMIDITY_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senHUMIDITY_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senHUMIDITY_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senHUMIDITY_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senHumidityVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senHUMIDITY_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senHUMIDITY_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senHUMIDITY_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senHumidityVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senHUMIDITY_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senHumidityVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senHUMIDITY_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senHumidityVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senHumidityVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senHUMIDITY_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senHumidityVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senHUMIDITY_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senHumidityVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senHUMIDITY_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senHumidityVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senHUMIDITY_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senHumidityVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senHumidityVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senHUMIDITY_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senHumidityVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senHumidityVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senHumidityVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senHumidityVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senHUMIDITY_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senHumidityVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senHumidityVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senHUMIDITY_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senHumidityVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senHUMIDITY_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senHumidityVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senHUMIDITY_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senHumidityVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senHUMIDITY_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senHumidityVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senHUMIDITY_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senHumidityVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senHUMIDITY_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senHumidityVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senHumidityVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senHumidityVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senHumidityVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senHUMIDITY_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senHumidityVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senHUMIDITY_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senHumidityVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senHUMIDITY_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senHUMIDITY_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senHumidityVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senHumidityVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senHumidityVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senHumidityVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senHumidityVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senHumidityVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senHumidityVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senHUMIDITY_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senHUMIDITY_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_STATUS)((obj)->fcts[ic0senHUMIDITY_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senHumidityVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senHUMIDITY_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senHumidityVHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senHUMIDITY_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senHumidityVHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senHUMIDITY_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senHumidityVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senHUMIDITY_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senHumidityVComInit(obj,devName,fd,simulation,error) \
	(*((ic0senHUMIDITY_FCTPTR_COM_INIT)((obj)->fcts[ic0senHUMIDITY_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senHumidityVComRead(obj,devName,fd,simulation,error) \
	(*((ic0senHUMIDITY_FCTPTR_COM_READ)((obj)->fcts[ic0senHUMIDITY_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senHumidityNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senHumidityDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_HUMIDITY_H

/************************************************************************/
/* ___oOo___ */
