/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senQuat10.h,v 1.51 2005/08/15 13:36:26 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   02/04/98  Created by eccsgen  Revision: 1.20
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senQuat10 - class
*
*   SYNOPSIS
*	#include "ic0senQuat10.h"
*
*	ccsCOMPL_STAT ic0senQuat10NewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10DeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senQuat10GetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senQuat10LogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senQuat10ComRead(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Read<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Write<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Read<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Write<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Read<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senQuat10Write<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrQuat10(p1,p2)
*
*   PARENT CLASS
*	public ic0senANALOG
*
*   DESCRIPTION
*       The class ic0senQUAT10 allows to read temperatures
*       via a HERAEUS QUAT10 board.
*
*	ic0senQuat10NewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senQUAT10.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senQuat10DeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senQUAT10
*
*
*	ic0senQuat10Err<error>(...)
*	    The macros ic0senQuat10Err*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senQuat10Read<attr>(...)
*	ic0senQuat10Write<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senQuat10Db(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senQuat10ComRead(obj,devName,fd,simulation,error)
*	   ComRead is called to read data from the serial port.
*	   It updates the last vector attribute.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_QUAT10
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_QUAT10_H
#define IC0SEN_QUAT10_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senAnalog.h"

/*
 * class type identifier
 */
#define ic0senQUAT10_ID	197

/*
 * attribute table
 */
#define ic0senQUAT10_ATTR_STATE	0
#define ic0senQUAT10_ATTR_SUBSTATE	1
#define ic0senQUAT10_ATTR_ERROR	2
#define ic0senQUAT10_ATTR_INITIALISED	3
#define ic0senQUAT10_ATTR_SIMULATION	4
#define ic0senQUAT10_ATTR_DEVICE	5
#define ic0senQUAT10_ATTR_AVAILABLE	6
#define ic0senQUAT10_ATTR_ENABLED	7
#define ic0senQUAT10_ATTR_LOCKED	8
#define ic0senQUAT10_ATTR_HANDSET	9
#define ic0senQUAT10_ATTR_FIXED	10
#define ic0senQUAT10_ATTR_FIX_POS_SET	11
#define ic0senQUAT10_ATTR_FIXED_POSITION	12
#define ic0senQUAT10_ATTR_FIXED_SIMULATION	13
#define ic0senQUAT10_ATTR_FIXED_LOCKED	14
#define ic0senQUAT10_ATTR_PREFIX	15
#define ic0senQUAT10_ATTR_PARAMETERS_SIZE	16
#define ic0senQUAT10_ATTR_PARAMETERS	17
#define ic0senQUAT10_ATTR_STATUS	18
#define ic0senQUAT10_ATTR_STATUS_INDEX	19
#define ic0senQUAT10_ATTR_VALUES_SIZE	20
#define ic0senQUAT10_ATTR_VALUES	21
#define ic0senQUAT10_ATTR_INIT_RESTRICTION	22
#define ic0senQUAT10_ATTR_MOVE_RESTRICTION	23
#define ic0senQUAT10_ATTR_STOP_TIMEOUT	24
#define ic0senQUAT10_ATTR_SET_VAL_FCT	25
#define ic0senQUAT10_ATTR_HELP_TEXT	26
#define ic0senQUAT10_ATTR_PARAM_STEP	27
#define ic0senQUAT10_ATTR_ACTION_COMMAND	28
#define ic0senQUAT10_ATTR_ACTION_STOP_REQ	29
#define ic0senQUAT10_ATTR_ACTION_NO_MOVE	30
#define ic0senQUAT10_ATTR_ACTION_CHECK	31
#define ic0senQUAT10_ATTR_RESET_SETUP_CMD	32
#define ic0senQUAT10_ATTR_SENSORS_PREFIX	33
#define ic0senQUAT10_ATTR_SENSORS_SIZE	34
#define ic0senQUAT10_ATTR_SENSORS	35
#define ic0senQUAT10_ATTR_HW_ACCESS	36
#define ic0senQUAT10_ATTR_LAST	37
#define ic0senQUAT10_ATTR_LAST_ACCESS	38
#define ic0senQUAT10_ATTR_MIN_ACCESS	39
#define ic0senQUAT10_ATTR_IC0SENFLAG	40

#define ic0senQUAT10_AUTOMATA	0

/*
 * function table
 */
#define ic0senQUAT10_FCT_DELETE_OBJ	0
#define ic0senQUAT10_FCT_STANDBY	1
#define ic0senQUAT10_FCT_ONLINE	2
#define ic0senQUAT10_FCT_OFF	3
#define ic0senQUAT10_FCT_EXIT	4
#define ic0senQUAT10_FCT_STATE	5
#define ic0senQUAT10_FCT_SIMULAT	6
#define ic0senQUAT10_FCT_STOP_SIM	7
#define ic0senQUAT10_FCT_STATUS	8
#define ic0senQUAT10_FCT_INIT	9
#define ic0senQUAT10_FCT_STOP	10
#define ic0senQUAT10_FCT_TEST	11
#define ic0senQUAT10_FCT_SELFTST	12
#define ic0senQUAT10_FCT_AUTO_TBL	13
#define ic0senQUAT10_FCT_AUTO_RST	14
#define ic0senQUAT10_FCT_INIT_OBJ	15
#define ic0senQUAT10_FCT_EXIT_OBJ	16
#define ic0senQUAT10_FCT_LOADED2STBY	17
#define ic0senQUAT10_FCT_ONLINE2STBY	18
#define ic0senQUAT10_FCT_EXIT1	19
#define ic0senQUAT10_FCT_CHECK_DEVICE_NAME	20
#define ic0senQUAT10_FCT_BREAK	21
#define ic0senQUAT10_FCT_USR_INIT	22
#define ic0senQUAT10_FCT_USR_LOADED2STANDBY	23
#define ic0senQUAT10_FCT_USR_STANDBY2ONLINE	24
#define ic0senQUAT10_FCT_USR_ONLINE2STANDBY	25
#define ic0senQUAT10_FCT_USR_STANDBY2LOADED	26
#define ic0senQUAT10_FCT_USR_STOP	27
#define ic0senQUAT10_FCT_USR_SIMULAT	28
#define ic0senQUAT10_FCT_USR_STOP_SIM	29
#define ic0senQUAT10_FCT_USR_STATUS_CODE	30
#define ic0senQUAT10_FCT_USR_STATUS_LAST	31
#define ic0senQUAT10_FCT_USR_STATUS_VALUE	32
#define ic0senQUAT10_FCT_USR_TEST	33
#define ic0senQUAT10_FCT_USR_SELF_TEST	34
#define ic0senQUAT10_FCT_USR_ERROR	35
#define ic0senQUAT10_FCT_AVAIL	36
#define ic0senQUAT10_FCT_UNAVAIL	37
#define ic0senQUAT10_FCT_DISABLE	38
#define ic0senQUAT10_FCT_ENABLE	39
#define ic0senQUAT10_FCT_LOCK	40
#define ic0senQUAT10_FCT_UNLOCK	41
#define ic0senQUAT10_FCT_HANDSET	42
#define ic0senQUAT10_FCT_HSET_OFF	43
#define ic0senQUAT10_FCT_FIX_POS	44
#define ic0senQUAT10_FCT_MOVE	45
#define ic0senQUAT10_FCT_MOVE_REL	46
#define ic0senQUAT10_FCT_SET	47
#define ic0senQUAT10_FCT_CHECK	48
#define ic0senQUAT10_FCT_WAIT	49
#define ic0senQUAT10_FCT_USR_SET_MODE	50
#define ic0senQUAT10_FCT_SETUP	51
#define ic0senQUAT10_FCT_GET_VAL	52
#define ic0senQUAT10_FCT_SET_VAL	53
#define ic0senQUAT10_FCT_CFG_ELEM	54
#define ic0senQUAT10_FCT_LOG_CONF	55
#define ic0senQUAT10_FCT_GET_CONF	56
#define ic0senQUAT10_FCT_RESET_PARAMETERS	57
#define ic0senQUAT10_FCT_PARSE_PARAMETERS	58
#define ic0senQUAT10_FCT_COMPARE_PARAMETERS	59
#define ic0senQUAT10_FCT_COPY_PARAMETERS	60
#define ic0senQUAT10_FCT_EXEC_PARAMETERS	61
#define ic0senQUAT10_FCT_UPDATE_PARAMETERS	62
#define ic0senQUAT10_FCT_GET_PARAM_SET_VALUE	63
#define ic0senQUAT10_FCT_GET_PARAM_STATUS	64
#define ic0senQUAT10_FCT_SET_PARAM	65
#define ic0senQUAT10_FCT_SET_PARAM_STEP	66
#define ic0senQUAT10_FCT_USR_EXEC_ACTION	67
#define ic0senQUAT10_FCT_USR_SET_PARAMETER	68
#define ic0senQUAT10_FCT_GET_VALUE_CODE	69
#define ic0senQUAT10_FCT_GET_VALUE	70
#define ic0senQUAT10_FCT_USR_GET_VALUE	71
#define ic0senQUAT10_FCT_SET_VALUE	72
#define ic0senQUAT10_FCT_USR_SET_VALUE	73
#define ic0senQUAT10_FCT_GET_ARG	74
#define ic0senQUAT10_FCT_MONITOR	75
#define ic0senQUAT10_FCT_USR_MONITOR	76
#define ic0senQUAT10_FCT_LOG_ACTION	77
#define ic0senQUAT10_FCT_LOG_COMMENT	78
#define ic0senQUAT10_FCT_LOG_PARAMETER	79
#define ic0senQUAT10_FCT_LOG_RECOVERY	80
#define ic0senQUAT10_FCT_LOG_TBL_PARAM	81
#define ic0senQUAT10_FCT_LOG_UNFORSEEN	82
#define ic0senQUAT10_FCT_HW_INIT	83
#define ic0senQUAT10_FCT_HW_EXIT	84
#define ic0senQUAT10_FCT_HW_STATUS	85
#define ic0senQUAT10_FCT_HW_SETUP	86
#define ic0senQUAT10_FCT_HW_GET_CONF	87
#define ic0senQUAT10_FCT_HW_LOG_CONF	88
#define ic0senQUAT10_FCT_LOG_STATUS	89
#define ic0senQUAT10_FCT_COM_INIT	90
#define ic0senQUAT10_FCT_COM_READ	91

/*
 * automata states
 */
#define ic0senQUAT10_STATE_NOSTATE	-1
#define ic0senQUAT10_STATE_OFF	0
#define ic0senQUAT10_STATE_LOADED	1
#define ic0senQUAT10_STATE_STANDBY	2
#define ic0senQUAT10_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senQUAT10_SUBSTATE_ERROR	0
#define ic0senQUAT10_SUBSTATE_IDLE	1
#define ic0senQUAT10_SUBSTATE_BUSY	2
#define ic0senQUAT10_SUBSTATE_INITIALISING	3
#define ic0senQUAT10_SUBSTATE_SHUTTING_DOWN	4
#define ic0senQUAT10_SUBSTATE_STOPPING	5
#define ic0senQUAT10_SUBSTATE_COLD_STARTING	6
#define ic0senQUAT10_SUBSTATE_STARTING	7
#define ic0senQUAT10_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senQUAT10_EVENT_INIT	0
#define ic0senQUAT10_EVENT_STANDBY	1
#define ic0senQUAT10_EVENT_ONLINE	2
#define ic0senQUAT10_EVENT_STOP	3
#define ic0senQUAT10_EVENT_OFF	4
#define ic0senQUAT10_EVENT_EXIT	5
#define ic0senQUAT10_EVENT_STATE	6
#define ic0senQUAT10_EVENT_STOP_SIM	7
#define ic0senQUAT10_EVENT_STATUS	8
#define ic0senQUAT10_EVENT_SIMULAT	9
#define ic0senQUAT10_EVENT_TEST	10
#define ic0senQUAT10_EVENT_SELFTST	11
#define ic0senQUAT10_EVENT_AUTO_TBL	12
#define ic0senQUAT10_EVENT_AUTO_RST	13
#define ic0senQUAT10_EVENT_DISABLE	14
#define ic0senQUAT10_EVENT_ENABLE	15
#define ic0senQUAT10_EVENT_LOCK	16
#define ic0senQUAT10_EVENT_UNLOCK	17
#define ic0senQUAT10_EVENT_HANDSET	18
#define ic0senQUAT10_EVENT_HSET_OFF	19
#define ic0senQUAT10_EVENT_AVAIL	20
#define ic0senQUAT10_EVENT_UNAVAIL	21
#define ic0senQUAT10_EVENT_MOVE	22
#define ic0senQUAT10_EVENT_MOVE_REL	23
#define ic0senQUAT10_EVENT_SET	24
#define ic0senQUAT10_EVENT_CHECK	25
#define ic0senQUAT10_EVENT_WAIT	26
#define ic0senQUAT10_EVENT_SETUP	27
#define ic0senQUAT10_EVENT_CFG_ELEM	28
#define ic0senQUAT10_EVENT_GET_VAL	29
#define ic0senQUAT10_EVENT_SET_VAL	30
#define ic0senQUAT10_EVENT_LOG_CONF	31
#define ic0senQUAT10_EVENT_GET_CONF	32
#define ic0senQUAT10_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senQUAT10_FCTPTR_COM_READ)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senQuat10LogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrQuat10(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_QUAT10: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_QUAT10,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senQuat10GetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senQuat10ReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_STATE,(buf),(error))
#define ic0senQuat10WriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_STATE,(void*) (buf),(error))
#define ic0senQuat10ReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_SUBSTATE,(buf),(error))
#define ic0senQuat10WriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senQuat10ReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ERROR,(buf),(error))
#define ic0senQuat10WriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ERROR,(void*) (buf),(error))
#define ic0senQuat10ReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_INITIALISED,(buf),(error))
#define ic0senQuat10WriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senQuat10ReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_SIMULATION,(buf),(error))
#define ic0senQuat10WriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senQuat10ReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_DEVICE,(buf),(error))
#define ic0senQuat10WriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senQuat10ReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_AVAILABLE,(buf),(error))
#define ic0senQuat10WriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senQuat10ReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ENABLED,(buf),(error))
#define ic0senQuat10WriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senQuat10ReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_LOCKED,(buf),(error))
#define ic0senQuat10WriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senQuat10ReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_HANDSET,(buf),(error))
#define ic0senQuat10WriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senQuat10ReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_FIXED,(buf),(error))
#define ic0senQuat10WriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_FIXED,(void*) (buf),(error))
#define ic0senQuat10ReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senQuat10WriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senQuat10ReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senQuat10WriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senQuat10ReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senQuat10WriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senQuat10ReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senQuat10WriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senQuat10ReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_PREFIX,(buf),(error))
#define ic0senQuat10WritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senQuat10ReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senQuat10WriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senQuat10GetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senQUAT10_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10WriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senQUAT10_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10GetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senQUAT10_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senQuat10WriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senQUAT10_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senQuat10GetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senQUAT10_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senQuat10WriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senQUAT10_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senQuat10ReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senQuat10WriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senQuat10GetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senQUAT10_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10WriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senQUAT10_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10ReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senQuat10WriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senQuat10ReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senQuat10WriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senQuat10ReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senQuat10WriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senQuat10ReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senQuat10WriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senQuat10GetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senQUAT10_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senQuat10WriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senQUAT10_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senQuat10ReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_PARAM_STEP,(buf),(error))
#define ic0senQuat10WriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senQuat10ReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senQuat10WriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senQuat10ReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senQuat10WriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senQuat10ReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senQuat10WriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senQuat10ReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senQuat10WriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senQuat10ReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senQuat10WriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senQuat10ReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senQuat10WriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senQuat10ReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senQuat10WriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senQuat10GetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senQUAT10_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10WriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senQUAT10_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senQuat10ReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_HW_ACCESS,(buf),(error))
#define ic0senQuat10WriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senQuat10GetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senQUAT10_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senQuat10ReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senQUAT10_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senQuat10WriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senQUAT10_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senQuat10ReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senQuat10WriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senQuat10ReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senQuat10WriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senQuat10ReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senQUAT10_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senQuat10WriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senQUAT10_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senQuat10GetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senQUAT10_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senQuat10HwInit(obj,devName,state,simulation,error) \
	ic0senAnalogHwInit((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10HwExit(obj,devName,state,simulation,error) \
	ic0senAnalogHwExit((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10HwStatus(obj,devName,state,simulation,error) \
	ic0senAnalogHwStatus((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10HwGetConf(obj,devName,text,error) \
	ic0senAnalogHwGetConf((obj),(devName),(text),(error))
#define ic0senQuat10HwLogConf(obj,devName,error) \
	ic0senAnalogHwLogConf((obj),(devName),(error))
#define ic0senQuat10ComInit(obj,devName,fd,simulation,error) \
	ic0senAnalogComInit((obj),(devName),(fd),(simulation),(error))
#define ic0senQuat10Setup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senQuat10Set(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10Move(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10MoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10Status(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senQuat10GetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senQuat10LogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senQuat10InitObj(obj,error) \
	ic0senSoftInitObj((obj),(error))
#define ic0senQuat10ExitObj(obj,error) \
	ic0senSoftExitObj((obj),(error))
#define ic0senQuat10UsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senQuat10UsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senQuat10UsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senQuat10UsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senQuat10UsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senQuat10Monitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senQuat10UsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senQuat10UsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senQuat10UsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senQuat10UsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senQuat10HwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senQuat10LogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senQuat10GetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senQuat10SetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senQuat10CfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senQuat10ResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senQuat10ParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senQuat10CompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senQuat10CopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senQuat10ExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senQuat10UpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senQuat10GetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senQuat10GetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senQuat10SetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senQuat10SetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senQuat10UsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senQuat10UsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senQuat10GetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senQuat10GetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senQuat10UsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senQuat10SetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senQuat10UsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senQuat10GetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senQuat10UsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senQuat10LogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senQuat10LogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senQuat10LogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senQuat10LogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senQuat10LogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senQuat10LogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senQuat10UsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senQuat10Avail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senQuat10Unavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senQuat10Disable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senQuat10Enable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senQuat10Lock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senQuat10Unlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senQuat10Handset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senQuat10HsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senQuat10FixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senQuat10Check(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senQuat10Wait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senQuat10UsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senQuat10UsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senQuat10UsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senQuat10Standby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senQuat10Online(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senQuat10Off(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senQuat10Exit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senQuat10State(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senQuat10Simulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senQuat10StopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senQuat10Init(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senQuat10Stop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senQuat10Test(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senQuat10Selftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senQuat10AutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senQuat10AutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senQuat10Loaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senQuat10Online2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senQuat10Exit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senQuat10CheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senQuat10Break(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senQuat10UsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senQuat10UsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senQuat10ComRead(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senQuat10VStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senQUAT10_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senQUAT10_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senQUAT10_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senQUAT10_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senQUAT10_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senQuat10VSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senQUAT10_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senQUAT10_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senQUAT10_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senQuat10VInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senQUAT10_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senQUAT10_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senQUAT10_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senQUAT10_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senQUAT10_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senQuat10VAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senQUAT10_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VInitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senQUAT10_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senQuat10VExitObj(obj,error) \
	(*((ic0senSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senQUAT10_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senQuat10VLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senQUAT10_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senQUAT10_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senQUAT10_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senQUAT10_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senQUAT10_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senQuat10VUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senQUAT10_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senQUAT10_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senQUAT10_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senQUAT10_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senQUAT10_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senQUAT10_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senQuat10VUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senQUAT10_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senQUAT10_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senQUAT10_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senQuat10VUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senQUAT10_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senQuat10VUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senQUAT10_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senQuat10VUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senQUAT10_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senQUAT10_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senQUAT10_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senQUAT10_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senQUAT10_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senQUAT10_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senQuat10VEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senQUAT10_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senQuat10VLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senQUAT10_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senQUAT10_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senQUAT10_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senQUAT10_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senQUAT10_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senQuat10VMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senQUAT10_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10VMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senQUAT10_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10VSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senQUAT10_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senQuat10VCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senQUAT10_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senQuat10VWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senQUAT10_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senQuat10VUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senQUAT10_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senQuat10VSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senQUAT10_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senQuat10VGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senQUAT10_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senQuat10VSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senQUAT10_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senQuat10VCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senQUAT10_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senQuat10VLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senQUAT10_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senQuat10VGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senQUAT10_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senQuat10VResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senQuat10VParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senQuat10VCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senQuat10VCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senQUAT10_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senQuat10VGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senQUAT10_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senQuat10VGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senQUAT10_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senQuat10VSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senQUAT10_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senQuat10VSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senQUAT10_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senQuat10VUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senQUAT10_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senQuat10VUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senQUAT10_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senQuat10VGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senQUAT10_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senQuat10VGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senQUAT10_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senQuat10VUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senQUAT10_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senQuat10VSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senQUAT10_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senQuat10VUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senQUAT10_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senQuat10VGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senQUAT10_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senQuat10VMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senQUAT10_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senQUAT10_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senQuat10VLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senQUAT10_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senQuat10VLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senQUAT10_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senQuat10VLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senQUAT10_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senQuat10VLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senQUAT10_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senQuat10VLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senQUAT10_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senQuat10VLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senQUAT10_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senQuat10VHwInit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_INIT)((obj)->fcts[ic0senQUAT10_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10VHwExit(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_EXIT)((obj)->fcts[ic0senQUAT10_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10VHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senANALOG_FCTPTR_HW_STATUS)((obj)->fcts[ic0senQUAT10_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senQuat10VHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senQUAT10_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senQuat10VHwGetConf(obj,devName,text,error) \
	(*((ic0senANALOG_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senQUAT10_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senQuat10VHwLogConf(obj,devName,error) \
	(*((ic0senANALOG_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senQUAT10_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senQuat10VLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senQUAT10_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))
#define ic0senQuat10VComInit(obj,devName,fd,simulation,error) \
	(*((ic0senANALOG_FCTPTR_COM_INIT)((obj)->fcts[ic0senQUAT10_FCT_COM_INIT]))) \
	 ((obj),(devName),(fd),(simulation),(error))
#define ic0senQuat10VComRead(obj,devName,fd,simulation,error) \
	(*((ic0senQUAT10_FCTPTR_COM_READ)((obj)->fcts[ic0senQUAT10_FCT_COM_READ]))) \
	 ((obj),(devName),(fd),(simulation),(error))


ccsCOMPL_STAT ic0senQuat10NewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senQuat10DeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0SEN_QUAT10_H

/************************************************************************/
/* ___oOo___ */
