/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senSignals.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   21/12/98  Created by eccsgen  Revision: 1.21
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0senSignals - class
*
*   SYNOPSIS
*	#include "ic0senSignals.h"
*
*	ccsCOMPL_STAT ic0senSignalsNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0senSignalsGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0senSignalsLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0senSignalsInitObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsExitObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsHwInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsHwExit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsHwStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    lccDEV_MODE	state,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senSignalsWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senErrSignals(p1,p2)
*
*   PARENT CLASS
*	public ic0senSOFT
*
*   DESCRIPTION
*       The class ic0senSIGNALS allows to access several digital
*       input signals.  Each digital signal is configured in the
*       sensors table.
*
*	ic0senSignalsNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0senSIGNALS.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0senSignalsDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0senSIGNALS
*
*
*	ic0senSignalsErr<error>(...)
*	    The macros ic0senSignalsErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0senSignalsRead<attr>(...)
*	ic0senSignalsWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0senSignalsDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0senSignalsInitObj(obj,error)
*
*	ic0senSignalsExitObj(obj,error)
*
*	ic0senSignalsHwInit(obj,devName,state,simulation,error)
*	   HwInit is called when the software device enters
*	   the STANDBY or ONLINE state.
*
*	ic0senSignalsHwExit(obj,devName,state,simulation,error)
*	   HwInit is called when the software device leaves
*	   the STANDBY or ONLINE state.
*
*	ic0senSignalsHwStatus(obj,devName,state,simulation,error)
*	   HwStatus is called periodically to update the
*	   status vector of the software device.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0senERR_SIGNALS
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_SIGNALS_H
#define IC0SEN_SIGNALS_H

#include "egen.h"
#include "ic0senErrors.h"
#include "ic0senSoft.h"

/*
 * class type identifier
 */
#define ic0senSIGNALS_ID	187

/*
 * attribute table
 */
#define ic0senSIGNALS_ATTR_STATE	0
#define ic0senSIGNALS_ATTR_SUBSTATE	1
#define ic0senSIGNALS_ATTR_ERROR	2
#define ic0senSIGNALS_ATTR_INITIALISED	3
#define ic0senSIGNALS_ATTR_SIMULATION	4
#define ic0senSIGNALS_ATTR_DEVICE	5
#define ic0senSIGNALS_ATTR_AVAILABLE	6
#define ic0senSIGNALS_ATTR_ENABLED	7
#define ic0senSIGNALS_ATTR_LOCKED	8
#define ic0senSIGNALS_ATTR_HANDSET	9
#define ic0senSIGNALS_ATTR_FIXED	10
#define ic0senSIGNALS_ATTR_FIX_POS_SET	11
#define ic0senSIGNALS_ATTR_FIXED_POSITION	12
#define ic0senSIGNALS_ATTR_FIXED_SIMULATION	13
#define ic0senSIGNALS_ATTR_FIXED_LOCKED	14
#define ic0senSIGNALS_ATTR_PREFIX	15
#define ic0senSIGNALS_ATTR_PARAMETERS_SIZE	16
#define ic0senSIGNALS_ATTR_PARAMETERS	17
#define ic0senSIGNALS_ATTR_STATUS	18
#define ic0senSIGNALS_ATTR_STATUS_INDEX	19
#define ic0senSIGNALS_ATTR_VALUES_SIZE	20
#define ic0senSIGNALS_ATTR_VALUES	21
#define ic0senSIGNALS_ATTR_INIT_RESTRICTION	22
#define ic0senSIGNALS_ATTR_MOVE_RESTRICTION	23
#define ic0senSIGNALS_ATTR_STOP_TIMEOUT	24
#define ic0senSIGNALS_ATTR_SET_VAL_FCT	25
#define ic0senSIGNALS_ATTR_HELP_TEXT	26
#define ic0senSIGNALS_ATTR_PARAM_STEP	27
#define ic0senSIGNALS_ATTR_ACTION_COMMAND	28
#define ic0senSIGNALS_ATTR_ACTION_STOP_REQ	29
#define ic0senSIGNALS_ATTR_ACTION_NO_MOVE	30
#define ic0senSIGNALS_ATTR_ACTION_CHECK	31
#define ic0senSIGNALS_ATTR_RESET_SETUP_CMD	32
#define ic0senSIGNALS_ATTR_SENSORS_PREFIX	33
#define ic0senSIGNALS_ATTR_SENSORS_SIZE	34
#define ic0senSIGNALS_ATTR_SENSORS	35
#define ic0senSIGNALS_ATTR_HW_ACCESS	36
#define ic0senSIGNALS_ATTR_LAST	37
#define ic0senSIGNALS_ATTR_LAST_ACCESS	38
#define ic0senSIGNALS_ATTR_MIN_ACCESS	39
#define ic0senSIGNALS_ATTR_IC0SENFLAG	40
#define ic0senSIGNALS_ATTR_ACRO_DEVICE	41
#define ic0senSIGNALS_ATTR_HW_INIT	42
#define ic0senSIGNALS_ATTR_PERIOD	43
#define ic0senSIGNALS_ATTR_VALUE0	44
#define ic0senSIGNALS_ATTR_VALUE0STATUS	45
#define ic0senSIGNALS_ATTR_VALUE1	46
#define ic0senSIGNALS_ATTR_VALUE1STATUS	47
#define ic0senSIGNALS_ATTR_VALUE2	48
#define ic0senSIGNALS_ATTR_VALUE2STATUS	49
#define ic0senSIGNALS_ATTR_VALUE3	50
#define ic0senSIGNALS_ATTR_VALUE3STATUS	51
#define ic0senSIGNALS_ATTR_VALUE4	52
#define ic0senSIGNALS_ATTR_VALUE4STATUS	53
#define ic0senSIGNALS_ATTR_VALUE5	54
#define ic0senSIGNALS_ATTR_VALUE5STATUS	55
#define ic0senSIGNALS_ATTR_VALUE6	56
#define ic0senSIGNALS_ATTR_VALUE6STATUS	57
#define ic0senSIGNALS_ATTR_VALUE7	58
#define ic0senSIGNALS_ATTR_VALUE7STATUS	59
#define ic0senSIGNALS_ATTR_VALUE8	60
#define ic0senSIGNALS_ATTR_VALUE8STATUS	61
#define ic0senSIGNALS_ATTR_VALUE9	62
#define ic0senSIGNALS_ATTR_VALUE9STATUS	63
#define ic0senSIGNALS_ATTR_VALUE10	64
#define ic0senSIGNALS_ATTR_VALUE10STATUS	65
#define ic0senSIGNALS_ATTR_VALUE11	66
#define ic0senSIGNALS_ATTR_VALUE11STATUS	67
#define ic0senSIGNALS_ATTR_VALUE12	68
#define ic0senSIGNALS_ATTR_VALUE12STATUS	69
#define ic0senSIGNALS_ATTR_VALUE13	70
#define ic0senSIGNALS_ATTR_VALUE13STATUS	71
#define ic0senSIGNALS_ATTR_VALUE14	72
#define ic0senSIGNALS_ATTR_VALUE14STATUS	73
#define ic0senSIGNALS_ATTR_VALUE15	74
#define ic0senSIGNALS_ATTR_VALUE15STATUS	75
#define ic0senSIGNALS_ATTR_LAST_CHANGE	76

#define ic0senSIGNALS_AUTOMATA	0

/*
 * function table
 */
#define ic0senSIGNALS_FCT_DELETE_OBJ	0
#define ic0senSIGNALS_FCT_STANDBY	1
#define ic0senSIGNALS_FCT_ONLINE	2
#define ic0senSIGNALS_FCT_OFF	3
#define ic0senSIGNALS_FCT_EXIT	4
#define ic0senSIGNALS_FCT_STATE	5
#define ic0senSIGNALS_FCT_SIMULAT	6
#define ic0senSIGNALS_FCT_STOP_SIM	7
#define ic0senSIGNALS_FCT_STATUS	8
#define ic0senSIGNALS_FCT_INIT	9
#define ic0senSIGNALS_FCT_STOP	10
#define ic0senSIGNALS_FCT_TEST	11
#define ic0senSIGNALS_FCT_SELFTST	12
#define ic0senSIGNALS_FCT_AUTO_TBL	13
#define ic0senSIGNALS_FCT_AUTO_RST	14
#define ic0senSIGNALS_FCT_INIT_OBJ	15
#define ic0senSIGNALS_FCT_EXIT_OBJ	16
#define ic0senSIGNALS_FCT_LOADED2STBY	17
#define ic0senSIGNALS_FCT_ONLINE2STBY	18
#define ic0senSIGNALS_FCT_EXIT1	19
#define ic0senSIGNALS_FCT_CHECK_DEVICE_NAME	20
#define ic0senSIGNALS_FCT_BREAK	21
#define ic0senSIGNALS_FCT_USR_INIT	22
#define ic0senSIGNALS_FCT_USR_LOADED2STANDBY	23
#define ic0senSIGNALS_FCT_USR_STANDBY2ONLINE	24
#define ic0senSIGNALS_FCT_USR_ONLINE2STANDBY	25
#define ic0senSIGNALS_FCT_USR_STANDBY2LOADED	26
#define ic0senSIGNALS_FCT_USR_STOP	27
#define ic0senSIGNALS_FCT_USR_SIMULAT	28
#define ic0senSIGNALS_FCT_USR_STOP_SIM	29
#define ic0senSIGNALS_FCT_USR_STATUS_CODE	30
#define ic0senSIGNALS_FCT_USR_STATUS_LAST	31
#define ic0senSIGNALS_FCT_USR_STATUS_VALUE	32
#define ic0senSIGNALS_FCT_USR_TEST	33
#define ic0senSIGNALS_FCT_USR_SELF_TEST	34
#define ic0senSIGNALS_FCT_USR_ERROR	35
#define ic0senSIGNALS_FCT_AVAIL	36
#define ic0senSIGNALS_FCT_UNAVAIL	37
#define ic0senSIGNALS_FCT_DISABLE	38
#define ic0senSIGNALS_FCT_ENABLE	39
#define ic0senSIGNALS_FCT_LOCK	40
#define ic0senSIGNALS_FCT_UNLOCK	41
#define ic0senSIGNALS_FCT_HANDSET	42
#define ic0senSIGNALS_FCT_HSET_OFF	43
#define ic0senSIGNALS_FCT_FIX_POS	44
#define ic0senSIGNALS_FCT_MOVE	45
#define ic0senSIGNALS_FCT_MOVE_REL	46
#define ic0senSIGNALS_FCT_SET	47
#define ic0senSIGNALS_FCT_CHECK	48
#define ic0senSIGNALS_FCT_WAIT	49
#define ic0senSIGNALS_FCT_USR_SET_MODE	50
#define ic0senSIGNALS_FCT_SETUP	51
#define ic0senSIGNALS_FCT_GET_VAL	52
#define ic0senSIGNALS_FCT_SET_VAL	53
#define ic0senSIGNALS_FCT_CFG_ELEM	54
#define ic0senSIGNALS_FCT_LOG_CONF	55
#define ic0senSIGNALS_FCT_GET_CONF	56
#define ic0senSIGNALS_FCT_RESET_PARAMETERS	57
#define ic0senSIGNALS_FCT_PARSE_PARAMETERS	58
#define ic0senSIGNALS_FCT_COMPARE_PARAMETERS	59
#define ic0senSIGNALS_FCT_COPY_PARAMETERS	60
#define ic0senSIGNALS_FCT_EXEC_PARAMETERS	61
#define ic0senSIGNALS_FCT_UPDATE_PARAMETERS	62
#define ic0senSIGNALS_FCT_GET_PARAM_SET_VALUE	63
#define ic0senSIGNALS_FCT_GET_PARAM_STATUS	64
#define ic0senSIGNALS_FCT_SET_PARAM	65
#define ic0senSIGNALS_FCT_SET_PARAM_STEP	66
#define ic0senSIGNALS_FCT_USR_EXEC_ACTION	67
#define ic0senSIGNALS_FCT_USR_SET_PARAMETER	68
#define ic0senSIGNALS_FCT_GET_VALUE_CODE	69
#define ic0senSIGNALS_FCT_GET_VALUE	70
#define ic0senSIGNALS_FCT_USR_GET_VALUE	71
#define ic0senSIGNALS_FCT_SET_VALUE	72
#define ic0senSIGNALS_FCT_USR_SET_VALUE	73
#define ic0senSIGNALS_FCT_GET_ARG	74
#define ic0senSIGNALS_FCT_MONITOR	75
#define ic0senSIGNALS_FCT_USR_MONITOR	76
#define ic0senSIGNALS_FCT_LOG_ACTION	77
#define ic0senSIGNALS_FCT_LOG_COMMENT	78
#define ic0senSIGNALS_FCT_LOG_PARAMETER	79
#define ic0senSIGNALS_FCT_LOG_RECOVERY	80
#define ic0senSIGNALS_FCT_LOG_TBL_PARAM	81
#define ic0senSIGNALS_FCT_LOG_UNFORSEEN	82
#define ic0senSIGNALS_FCT_HW_INIT	83
#define ic0senSIGNALS_FCT_HW_EXIT	84
#define ic0senSIGNALS_FCT_HW_STATUS	85
#define ic0senSIGNALS_FCT_HW_SETUP	86
#define ic0senSIGNALS_FCT_HW_GET_CONF	87
#define ic0senSIGNALS_FCT_HW_LOG_CONF	88
#define ic0senSIGNALS_FCT_LOG_STATUS	89

/*
 * automata states
 */
#define ic0senSIGNALS_STATE_NOSTATE	-1
#define ic0senSIGNALS_STATE_OFF	0
#define ic0senSIGNALS_STATE_LOADED	1
#define ic0senSIGNALS_STATE_STANDBY	2
#define ic0senSIGNALS_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0senSIGNALS_SUBSTATE_ERROR	0
#define ic0senSIGNALS_SUBSTATE_IDLE	1
#define ic0senSIGNALS_SUBSTATE_BUSY	2
#define ic0senSIGNALS_SUBSTATE_INITIALISING	3
#define ic0senSIGNALS_SUBSTATE_SHUTTING_DOWN	4
#define ic0senSIGNALS_SUBSTATE_STOPPING	5
#define ic0senSIGNALS_SUBSTATE_COLD_STARTING	6
#define ic0senSIGNALS_SUBSTATE_STARTING	7
#define ic0senSIGNALS_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0senSIGNALS_EVENT_INIT	0
#define ic0senSIGNALS_EVENT_STANDBY	1
#define ic0senSIGNALS_EVENT_ONLINE	2
#define ic0senSIGNALS_EVENT_STOP	3
#define ic0senSIGNALS_EVENT_OFF	4
#define ic0senSIGNALS_EVENT_EXIT	5
#define ic0senSIGNALS_EVENT_STATE	6
#define ic0senSIGNALS_EVENT_STOP_SIM	7
#define ic0senSIGNALS_EVENT_STATUS	8
#define ic0senSIGNALS_EVENT_SIMULAT	9
#define ic0senSIGNALS_EVENT_TEST	10
#define ic0senSIGNALS_EVENT_SELFTST	11
#define ic0senSIGNALS_EVENT_AUTO_TBL	12
#define ic0senSIGNALS_EVENT_AUTO_RST	13
#define ic0senSIGNALS_EVENT_DISABLE	14
#define ic0senSIGNALS_EVENT_ENABLE	15
#define ic0senSIGNALS_EVENT_LOCK	16
#define ic0senSIGNALS_EVENT_UNLOCK	17
#define ic0senSIGNALS_EVENT_HANDSET	18
#define ic0senSIGNALS_EVENT_HSET_OFF	19
#define ic0senSIGNALS_EVENT_AVAIL	20
#define ic0senSIGNALS_EVENT_UNAVAIL	21
#define ic0senSIGNALS_EVENT_MOVE	22
#define ic0senSIGNALS_EVENT_MOVE_REL	23
#define ic0senSIGNALS_EVENT_SET	24
#define ic0senSIGNALS_EVENT_CHECK	25
#define ic0senSIGNALS_EVENT_WAIT	26
#define ic0senSIGNALS_EVENT_SETUP	27
#define ic0senSIGNALS_EVENT_CFG_ELEM	28
#define ic0senSIGNALS_EVENT_GET_VAL	29
#define ic0senSIGNALS_EVENT_SET_VAL	30
#define ic0senSIGNALS_EVENT_LOG_CONF	31
#define ic0senSIGNALS_EVENT_GET_CONF	32
#define ic0senSIGNALS_EVENT_MONITOR	33


/*
 * user variables
 */
typedef struct
    {
    lcctooTASK   task;
    ioDIRADDRESS signals[ic0senSIGNALS_NUM];
    } ic0senSIGNALS_DATA;

typedef struct
    {
    vltBYTES16	name;
    vltBYTES32	description;
    vltINT32	flags;
    vltBYTES32	fits;
    vltUINT32	startBit;
    vltUINT32	width;
    vltLOGICAL	low;
    vltBYTES16	value0;
    vltBYTES16	value1;
    vltBYTES16	value2;
    vltBYTES16	value3;
    vltUINT32	simValue;
    } ic0senSIGNALS_SENSOR;

typedef struct
    {
    vltBYTES16	value0;
    vltBYTES16	value1;
    vltBYTES16	value2;
    vltBYTES16	value3;
    vltUINT32	simValue;
    } ic0senSIGNALS_SENSOR_VALUES;

/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0senSIGNALS_FCTPTR_INIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senSIGNALS_FCTPTR_EXIT_OBJ)(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senSIGNALS_FCTPTR_HW_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senSIGNALS_FCTPTR_HW_EXIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0senSIGNALS_FCTPTR_HW_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0senSignalsLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0sen", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0senErrSignals(p1,p2) \
	{ \
	lcctooLOG_0("ic0senERR_SIGNALS: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0sen",ic0senERR_SIGNALS,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0senSignalsGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0senSignalsReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_STATE,(buf),(error))
#define ic0senSignalsWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_STATE,(void*) (buf),(error))
#define ic0senSignalsReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_SUBSTATE,(buf),(error))
#define ic0senSignalsWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0senSignalsReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ERROR,(buf),(error))
#define ic0senSignalsWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ERROR,(void*) (buf),(error))
#define ic0senSignalsReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_INITIALISED,(buf),(error))
#define ic0senSignalsWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0senSignalsReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_SIMULATION,(buf),(error))
#define ic0senSignalsWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0senSignalsReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_DEVICE,(buf),(error))
#define ic0senSignalsWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_DEVICE,(void*) (buf),(error))
#define ic0senSignalsReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_AVAILABLE,(buf),(error))
#define ic0senSignalsWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0senSignalsReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ENABLED,(buf),(error))
#define ic0senSignalsWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ENABLED,(void*) (buf),(error))
#define ic0senSignalsReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_LOCKED,(buf),(error))
#define ic0senSignalsWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_LOCKED,(void*) (buf),(error))
#define ic0senSignalsReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_HANDSET,(buf),(error))
#define ic0senSignalsWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_HANDSET,(void*) (buf),(error))
#define ic0senSignalsReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_FIXED,(buf),(error))
#define ic0senSignalsWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_FIXED,(void*) (buf),(error))
#define ic0senSignalsReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_FIX_POS_SET,(buf),(error))
#define ic0senSignalsWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0senSignalsReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_FIXED_POSITION,(buf),(error))
#define ic0senSignalsWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0senSignalsReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0senSignalsWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0senSignalsReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0senSignalsWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0senSignalsReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_PREFIX,(buf),(error))
#define ic0senSignalsWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_PREFIX,(void*) (buf),(error))
#define ic0senSignalsReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0senSignalsWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0senSignalsGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senSIGNALS_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senSIGNALS_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senSIGNALS_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0senSignalsWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senSIGNALS_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senSignalsGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senSIGNALS_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0senSignalsWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senSIGNALS_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senSignalsReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUES_SIZE,(buf),(error))
#define ic0senSignalsWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0senSignalsGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0senSignalsWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0senSignalsReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0senSignalsWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0senSignalsReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0senSignalsWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0senSignalsReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0senSignalsWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0senSignalsGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senSIGNALS_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0senSignalsWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senSIGNALS_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senSignalsReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_PARAM_STEP,(buf),(error))
#define ic0senSignalsWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0senSignalsReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0senSignalsWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0senSignalsReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0senSignalsWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0senSignalsReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0senSignalsWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0senSignalsReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ACTION_CHECK,(buf),(error))
#define ic0senSignalsWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0senSignalsReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0senSignalsWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0senSignalsReadSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_SENSORS_PREFIX,(buf),(error))
#define ic0senSignalsWriteSensorsPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_SENSORS_PREFIX,(void*) (buf),(error))
#define ic0senSignalsReadSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_SENSORS_SIZE,(buf),(error))
#define ic0senSignalsWriteSensorsSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_SENSORS_SIZE,(void*) (buf),(error))
#define ic0senSignalsGetSensorsSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_SENSORS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0senSIGNALS_ATTR_SENSORS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsWriteSensors(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0senSIGNALS_ATTR_SENSORS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0senSignalsReadHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_HW_ACCESS,(buf),(error))
#define ic0senSignalsWriteHwAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_HW_ACCESS,(void*) (buf),(error))
#define ic0senSignalsGetLastSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_LAST,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senSIGNALS_ATTR_LAST,(buf),(firstElt),(lastElt),(error))
#define ic0senSignalsWriteLast(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senSIGNALS_ATTR_LAST,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senSignalsReadLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_LAST_ACCESS,(buf),(error))
#define ic0senSignalsWriteLastAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_LAST_ACCESS,(void*) (buf),(error))
#define ic0senSignalsReadMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_MIN_ACCESS,(buf),(error))
#define ic0senSignalsWriteMinAccess(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_MIN_ACCESS,(void*) (buf),(error))
#define ic0senSignalsReadIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_IC0SENFLAG,(buf),(error))
#define ic0senSignalsWriteIc0senflag(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_IC0SENFLAG,(void*) (buf),(error))
#define ic0senSignalsReadAcroDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_ACRO_DEVICE,(buf),(error))
#define ic0senSignalsWriteAcroDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_ACRO_DEVICE,(void*) (buf),(error))
#define ic0senSignalsReadHwInit(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_HW_INIT,(buf),(error))
#define ic0senSignalsWriteHwInit(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_HW_INIT,(void*) (buf),(error))
#define ic0senSignalsReadPeriod(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_PERIOD,(buf),(error))
#define ic0senSignalsWritePeriod(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_PERIOD,(void*) (buf),(error))
#define ic0senSignalsReadValue0(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE0,(buf),(error))
#define ic0senSignalsWriteValue0(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE0,(void*) (buf),(error))
#define ic0senSignalsReadValue0Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE0STATUS,(buf),(error))
#define ic0senSignalsWriteValue0Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE0STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue1(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE1,(buf),(error))
#define ic0senSignalsWriteValue1(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE1,(void*) (buf),(error))
#define ic0senSignalsReadValue1Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE1STATUS,(buf),(error))
#define ic0senSignalsWriteValue1Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE1STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue2(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE2,(buf),(error))
#define ic0senSignalsWriteValue2(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE2,(void*) (buf),(error))
#define ic0senSignalsReadValue2Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE2STATUS,(buf),(error))
#define ic0senSignalsWriteValue2Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE2STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue3(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE3,(buf),(error))
#define ic0senSignalsWriteValue3(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE3,(void*) (buf),(error))
#define ic0senSignalsReadValue3Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE3STATUS,(buf),(error))
#define ic0senSignalsWriteValue3Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE3STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue4(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE4,(buf),(error))
#define ic0senSignalsWriteValue4(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE4,(void*) (buf),(error))
#define ic0senSignalsReadValue4Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE4STATUS,(buf),(error))
#define ic0senSignalsWriteValue4Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE4STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue5(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE5,(buf),(error))
#define ic0senSignalsWriteValue5(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE5,(void*) (buf),(error))
#define ic0senSignalsReadValue5Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE5STATUS,(buf),(error))
#define ic0senSignalsWriteValue5Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE5STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue6(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE6,(buf),(error))
#define ic0senSignalsWriteValue6(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE6,(void*) (buf),(error))
#define ic0senSignalsReadValue6Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE6STATUS,(buf),(error))
#define ic0senSignalsWriteValue6Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE6STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue7(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE7,(buf),(error))
#define ic0senSignalsWriteValue7(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE7,(void*) (buf),(error))
#define ic0senSignalsReadValue7Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE7STATUS,(buf),(error))
#define ic0senSignalsWriteValue7Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE7STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue8(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE8,(buf),(error))
#define ic0senSignalsWriteValue8(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE8,(void*) (buf),(error))
#define ic0senSignalsReadValue8Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE8STATUS,(buf),(error))
#define ic0senSignalsWriteValue8Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE8STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue9(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE9,(buf),(error))
#define ic0senSignalsWriteValue9(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE9,(void*) (buf),(error))
#define ic0senSignalsReadValue9Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE9STATUS,(buf),(error))
#define ic0senSignalsWriteValue9Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE9STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue10(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE10,(buf),(error))
#define ic0senSignalsWriteValue10(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE10,(void*) (buf),(error))
#define ic0senSignalsReadValue10Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE10STATUS,(buf),(error))
#define ic0senSignalsWriteValue10Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE10STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue11(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE11,(buf),(error))
#define ic0senSignalsWriteValue11(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE11,(void*) (buf),(error))
#define ic0senSignalsReadValue11Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE11STATUS,(buf),(error))
#define ic0senSignalsWriteValue11Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE11STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue12(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE12,(buf),(error))
#define ic0senSignalsWriteValue12(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE12,(void*) (buf),(error))
#define ic0senSignalsReadValue12Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE12STATUS,(buf),(error))
#define ic0senSignalsWriteValue12Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE12STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue13(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE13,(buf),(error))
#define ic0senSignalsWriteValue13(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE13,(void*) (buf),(error))
#define ic0senSignalsReadValue13Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE13STATUS,(buf),(error))
#define ic0senSignalsWriteValue13Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE13STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue14(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE14,(buf),(error))
#define ic0senSignalsWriteValue14(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE14,(void*) (buf),(error))
#define ic0senSignalsReadValue14Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE14STATUS,(buf),(error))
#define ic0senSignalsWriteValue14Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE14STATUS,(void*) (buf),(error))
#define ic0senSignalsReadValue15(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE15,(buf),(error))
#define ic0senSignalsWriteValue15(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE15,(void*) (buf),(error))
#define ic0senSignalsReadValue15Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0senSIGNALS_ATTR_VALUE15STATUS,(buf),(error))
#define ic0senSignalsWriteValue15Status(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0senSIGNALS_ATTR_VALUE15STATUS,(void*) (buf),(error))
#define ic0senSignalsGetLastChangeSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0senSIGNALS_ATTR_LAST_CHANGE,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0senSignalsReadLastChange(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0senSIGNALS_ATTR_LAST_CHANGE,(buf),(firstElt),(lastElt),(error))
#define ic0senSignalsWriteLastChange(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0senSIGNALS_ATTR_LAST_CHANGE,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0senSignalsGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0senSIGNALS_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0senSignalsSetup(obj,devName,function,functionNum,noMove,check,error) \
	ic0senSoftSetup((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senSignalsSet(obj,devName,position,positionNum,error) \
	ic0senSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsMove(obj,devName,position,positionNum,error) \
	ic0senSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsMoveRel(obj,devName,position,positionNum,error) \
	ic0senSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	ic0senSoftStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senSignalsGetConf(obj,devName,allElements,text,error) \
	ic0senSoftGetConf((obj),(devName),(allElements),(text),(error))
#define ic0senSignalsLogConf(obj,devName,allElements,error) \
	ic0senSoftLogConf((obj),(devName),(allElements),(error))
#define ic0senSignalsUsrInit(obj,devName,error) \
	ic0senSoftUsrInit((obj),(devName),(error))
#define ic0senSignalsUsrStandby2Online(obj,devName,error) \
	ic0senSoftUsrStandby2Online((obj),(devName),(error))
#define ic0senSignalsUsrOnline2Standby(obj,devName,error) \
	ic0senSoftUsrOnline2Standby((obj),(devName),(error))
#define ic0senSignalsUsrStandby2Loaded(obj,devName,error) \
	ic0senSoftUsrStandby2Loaded((obj),(devName),(error))
#define ic0senSignalsUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	ic0senSoftUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senSignalsMonitor(obj,devName,error) \
	ic0senSoftMonitor((obj),(devName),(error))
#define ic0senSignalsUsrTest(obj,devName,error) \
	ic0senSoftUsrTest((obj),(devName),(error))
#define ic0senSignalsUsrStatusCode(obj,devName,text,flags,code,error) \
	ic0senSoftUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0senSignalsUsrStatusValue(obj,devName,code,value,error) \
	ic0senSoftUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0senSignalsUsrStatusLast(obj,devName,last,error) \
	ic0senSoftUsrStatusLast((obj),(devName),(last),(error))
#define ic0senSignalsHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	ic0senSoftHwSetup((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senSignalsHwGetConf(obj,devName,text,error) \
	ic0senSoftHwGetConf((obj),(devName),(text),(error))
#define ic0senSignalsHwLogConf(obj,devName,error) \
	ic0senSoftHwLogConf((obj),(devName),(error))
#define ic0senSignalsLogStatus(obj,devName,all,error) \
	ic0senSoftLogStatus((obj),(devName),(all),(error))
#define ic0senSignalsGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senSignalsSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senSignalsCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senSignalsResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0senSignalsParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senSignalsCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0senSignalsCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0senSignalsExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0senSignalsUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0senSignalsGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0senSignalsGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0senSignalsSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senSignalsSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0senSignalsUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0senSignalsUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senSignalsGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0senSignalsGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senSignalsUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senSignalsSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senSignalsUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senSignalsGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0senSignalsUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0senSignalsLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0senSignalsLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0senSignalsLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senSignalsLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0senSignalsLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senSignalsLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0senSignalsUsrStop(obj,devName,req,error) \
	ic0devSoftUsrStop((obj),(devName),(req),(error))
#define ic0senSignalsAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0senSignalsUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0senSignalsDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0senSignalsEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0senSignalsLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0senSignalsUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0senSignalsHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0senSignalsHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0senSignalsFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senSignalsCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0senSignalsWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0senSignalsUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0senSignalsUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0senSignalsUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0senSignalsStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0senSignalsOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0senSignalsOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0senSignalsExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0senSignalsState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0senSignalsSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0senSignalsStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0senSignalsInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0senSignalsStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0senSignalsTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0senSignalsSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0senSignalsAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0senSignalsAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0senSignalsLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0senSignalsOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0senSignalsExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0senSignalsCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0senSignalsBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0senSignalsUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0senSignalsUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0senSignalsInitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senSignalsExitObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senSignalsHwInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senSignalsHwExit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senSignalsHwStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    lccDEV_MODE	state,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0senSignalsVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0senSIGNALS_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0senSIGNALS_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0senSIGNALS_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0senSIGNALS_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0senSIGNALS_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0senSignalsVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0senSIGNALS_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0senSIGNALS_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0senSOFT_FCTPTR_STATUS)((obj)->fcts[ic0senSIGNALS_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0senSignalsVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0senSIGNALS_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0senSIGNALS_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0senSIGNALS_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0senSIGNALS_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0senSIGNALS_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0senSignalsVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0senSIGNALS_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVInitObj(obj,error) \
	(*((ic0senSIGNALS_FCTPTR_INIT_OBJ)((obj)->fcts[ic0senSIGNALS_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0senSignalsVExitObj(obj,error) \
	(*((ic0senSIGNALS_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0senSIGNALS_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0senSignalsVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0senSIGNALS_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0senSIGNALS_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0senSIGNALS_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0senSIGNALS_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0senSIGNALS_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0senSignalsVUsrInit(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_INIT)((obj)->fcts[ic0senSIGNALS_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0senSIGNALS_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrStandby2Online(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0senSIGNALS_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrOnline2Standby(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0senSIGNALS_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrStandby2Loaded(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0senSIGNALS_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrStop(obj,devName,req,error) \
	(*((ic0devSOFT_FCTPTR_USR_STOP)((obj)->fcts[ic0senSIGNALS_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0senSignalsVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0senSIGNALS_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0senSIGNALS_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0senSIGNALS_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0senSignalsVUsrStatusLast(obj,devName,last,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0senSIGNALS_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0senSignalsVUsrStatusValue(obj,devName,code,value,error) \
	(*((ic0senSOFT_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0senSignalsVUsrTest(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_USR_TEST)((obj)->fcts[ic0senSIGNALS_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0senSIGNALS_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0senSIGNALS_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0senSIGNALS_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0senSIGNALS_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0senSIGNALS_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senSignalsVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0senSIGNALS_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0senSignalsVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0senSIGNALS_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0senSIGNALS_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0senSIGNALS_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0senSIGNALS_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0senSIGNALS_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0senSignalsVMove(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE)((obj)->fcts[ic0senSIGNALS_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0senSIGNALS_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsVSet(obj,devName,position,positionNum,error) \
	(*((ic0senSOFT_FCTPTR_SET)((obj)->fcts[ic0senSIGNALS_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0senSignalsVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0senSIGNALS_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0senSignalsVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0senSIGNALS_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0senSignalsVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((ic0senSOFT_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0senSIGNALS_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0senSignalsVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0senSOFT_FCTPTR_SETUP)((obj)->fcts[ic0senSIGNALS_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0senSignalsVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0senSIGNALS_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0senSignalsVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0senSIGNALS_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0senSignalsVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0senSIGNALS_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0senSignalsVLogConf(obj,devName,allElements,error) \
	(*((ic0senSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0senSIGNALS_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0senSignalsVGetConf(obj,devName,allElements,text,error) \
	(*((ic0senSOFT_FCTPTR_GET_CONF)((obj)->fcts[ic0senSIGNALS_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0senSignalsVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0senSignalsVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0senSignalsVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0senSignalsVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0senSIGNALS_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0senSignalsVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0senSignalsVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0senSIGNALS_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0senSignalsVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0senSIGNALS_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0senSignalsVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0senSIGNALS_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0senSignalsVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0senSIGNALS_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0senSignalsVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0senSIGNALS_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0senSignalsVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0senSIGNALS_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0senSignalsVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senSignalsVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senSignalsVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0senSignalsVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0senSIGNALS_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0senSignalsVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0senSIGNALS_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0senSignalsVMonitor(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0senSIGNALS_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0senSIGNALS_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0senSignalsVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0senSIGNALS_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0senSignalsVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0senSIGNALS_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senSignalsVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0senSIGNALS_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0senSignalsVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0senSIGNALS_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senSignalsVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0senSIGNALS_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0senSignalsVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0senSIGNALS_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))
#define ic0senSignalsVHwInit(obj,devName,state,simulation,error) \
	(*((ic0senSIGNALS_FCTPTR_HW_INIT)((obj)->fcts[ic0senSIGNALS_FCT_HW_INIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senSignalsVHwExit(obj,devName,state,simulation,error) \
	(*((ic0senSIGNALS_FCTPTR_HW_EXIT)((obj)->fcts[ic0senSIGNALS_FCT_HW_EXIT]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senSignalsVHwStatus(obj,devName,state,simulation,error) \
	(*((ic0senSIGNALS_FCTPTR_HW_STATUS)((obj)->fcts[ic0senSIGNALS_FCT_HW_STATUS]))) \
	 ((obj),(devName),(state),(simulation),(error))
#define ic0senSignalsVHwSetup(obj,devName,function,functionNum,noMove,check,simulation,error) \
	(*((ic0senSOFT_FCTPTR_HW_SETUP)((obj)->fcts[ic0senSIGNALS_FCT_HW_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(simulation),(error))
#define ic0senSignalsVHwGetConf(obj,devName,text,error) \
	(*((ic0senSOFT_FCTPTR_HW_GET_CONF)((obj)->fcts[ic0senSIGNALS_FCT_HW_GET_CONF]))) \
	 ((obj),(devName),(text),(error))
#define ic0senSignalsVHwLogConf(obj,devName,error) \
	(*((ic0senSOFT_FCTPTR_HW_LOG_CONF)((obj)->fcts[ic0senSIGNALS_FCT_HW_LOG_CONF]))) \
	 ((obj),(devName),(error))
#define ic0senSignalsVLogStatus(obj,devName,all,error) \
	(*((ic0senSOFT_FCTPTR_LOG_STATUS)((obj)->fcts[ic0senSIGNALS_FCT_LOG_STATUS]))) \
	 ((obj),(devName),(all),(error))


ccsCOMPL_STAT ic0senSignalsNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senSignalsDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);

void ic0senSignalsHwMonitor(
    INOUT lcctooTASK*	task);


#endif // !IC0SEN_SIGNALS_H

/************************************************************************/
/* ___oOo___ */
