/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0timTimer.h,v 1.3 2003/12/03 18:26:09 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  xxxxmgr    19/12/01  Created by eccsgen  Revision: 1.29  
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	ic0timTimer - class
*
*   SYNOPSIS
*	#include "ic0timTimer.h"
*
*	ccsCOMPL_STAT ic0timTimerNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* ic0timTimerGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* ic0timTimerLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT ic0timTimerSetup(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	function[],	** max. vltBYTES32[999]
*	    IN    vltINT16	functionNum,
*	    IN    vltLOGICAL	noMove,
*	    IN    vltLOGICAL	check,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerStatus(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	function[],	** max. vltBYTES32[999]
*	    IN    vltINT16	functionNum,
*	    IN    vltLOGICAL	flag,
*	    IN    vltLOGICAL	header,
*	    IN    vltLOGICAL	conflog,
*	    IN    vltLOGICAL	opslog,
*	    IN    vltLOGICAL	condlog,
*	    OUT   char**	status,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerGetConf(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltLOGICAL	allElements,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerUsrInit(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerUsrStop(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltLOGICAL	req,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerUsrOnline2Standby(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerRead<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerWrite<scalarAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerRead<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerWrite<vectorAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	first,
*	    IN    vltINT32	last,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerRead<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    OUT   void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timTimerWrite<tableAttr>(
*	    INOUT egenOBJ*	obj,
*	    IN    void*		buf,
*	    IN    vltINT32	firstRec,
*	    IN    vltINT32	lastRec,
*	    IN    vltINT32	firstFld,
*	    IN    vltINT32	lastFld,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0timErrTimerReadOldb(p1,p2)
*	ccsCOMPL_STAT ic0timErrTimerWriteOldb(p1,p2)
*	ccsCOMPL_STAT ic0timErrTimerNotLogical(p1,p2)
*	ccsCOMPL_STAT ic0timErrTimerWrongStart(p1,p2)
*	ccsCOMPL_STAT ic0timErrTimerWrongPeriod(p1,p2)
*	ccsCOMPL_STAT ic0timErrTimerGeneral(p1,p2)
*
*   PARENT CLASS
*	public ic0devSOFT
*
*   DESCRIPTION
*       The class ic0timTIMER implements the software device
*       to access a TIM board.
*
*       The following FITS keywords are used:
*   
*       Setup keywords
*	 INS.TIMi.ST        Start (T) or stop (F) the timer
*	 INS.TIMi.START     Start UTC time (ISO format string)
*	 INS.TIMi.PERIOD    Timer output period [sec]
* 
*	 Configuration keywords
*	 INS.TIMi.DEVNAME   Acronym of the ICS software device
*	 INS.TIMi.DEVDESC   Description of the ICS software device
*	 INS.TIMi.LCUID     LCU number of the LCU managing the device
*	 INS.TIMi.SWSIM     Device simulation flag (T/F)
*	 INS.TIMi.TIMER     Timer identifier (1 to 4)
* 
*	 Status keywords
*	 INS.TIMi.ST        Start (T) or stop (F) the timer
*	 INS.TIMi.START     Start UTC time (ISO format string)
*	 INS.TIMi.PERIOD    Timer output period [sec]
*
*	ic0timTimerNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type ic0timTIMER.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	ic0timTimerDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type ic0timTIMER
*
*	ic0timTimerSetup(obj,devName,function,functionNum,noMove,check,error)
*	   Setups the function. The keys are the last component
*	   of the FITS keywords related with the function.
*	   E.g.:
*	     If the FITS keyword is: INS.OPTI-3.ID #11
*	     then the key is: ID, the value '#11'.
*
*	   If the noMove flag is used, then the setup parameters
*	   are stored, but not executed until a setup command
*	   without the noMove flag is received.
*
*	   If the check flag is used, then only the parameters
*	   are checked. No setup is performed.
*
*	   Aliases are not accepted (The ICS Frontend at the
*	   workstation recognizes the aliases).
*
*	ic0timTimerStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error)
*	   Updates the complete status in the local database
*	   and returns the relevant status of the software
*	   device.
*
*	   The status is given as lines containing two words
*	   each:
*	     key1  value1,
*	     [key2 value2,
*	     [...]]
*
*	   The value usually begins after the 18th character
*	   of each line. If the flag option is set, then the
*	   values are returned as numbers (by default values
*	   are returned as literals).
*
*	ic0timTimerGetConf(obj,devName,allElements,text,error)
*	   Returns the current configuration of the device.
*
*	   If the allElements flag is given, then the elements
*	   of all wheel positions are reported; otherwise
*	   only the element in the lightpath is described.
*
*
*	ic0timTimerErr<error>(...)
*	    The macros ic0timTimerErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	ic0timTimerRead<attr>(...)
*	ic0timTimerWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*	    The attributes are described in ic0timTimerDb(5).
*
*   PROTECTED METHODS
*   PRIVATE METHODS
*	ic0timTimerUsrInit(obj,devName,error)
*	   This function should be overloaded to perform the
*	   device initialisation (without hardware movements).
*
*	ic0timTimerUsrStop(obj,devName,req,error)
*	   This function should be overloaded to stop the device.
*	   If req is true, then a command to stop the device MUST
*	   be issued, even if the device is already stopped.
*	   Note: The Stop command is accepted even if the device
*	   is busy performing another command.
*
*	ic0timTimerUsrOnline2Standby(obj,devName,error)
*	   This function should be overloaded to perform the
*	   actions necessary to change from the ONLINE to the
*	   STANDBY state.
*
*
*   DEFINES
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	Methods can generate the following errors:
*	    ic0timERR_TIMER_READ_OLDB
*	    ic0timERR_TIMER_WRITE_OLDB
*	    ic0timERR_TIMER_NOT_LOGICAL
*	    ic0timERR_TIMER_WRONG_START
*	    ic0timERR_TIMER_WRONG_PERIOD
*	    ic0timERR_TIMER_GENERAL
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0TIM_TIMER_H
#define IC0TIM_TIMER_H

#include "egen.h"
#include "ic0timErrors.h"
#include "ic0devSoft.h"

/*
 * class type identifier
 */
#define ic0timTIMER_ID	194

/*
 * attribute table
 */
#define ic0timTIMER_ATTR_STATE	0
#define ic0timTIMER_ATTR_SUBSTATE	1
#define ic0timTIMER_ATTR_ERROR	2
#define ic0timTIMER_ATTR_INITIALISED	3
#define ic0timTIMER_ATTR_SIMULATION	4
#define ic0timTIMER_ATTR_DEVICE	5
#define ic0timTIMER_ATTR_AVAILABLE	6
#define ic0timTIMER_ATTR_ENABLED	7
#define ic0timTIMER_ATTR_LOCKED	8
#define ic0timTIMER_ATTR_HANDSET	9
#define ic0timTIMER_ATTR_FIXED	10
#define ic0timTIMER_ATTR_FIX_POS_SET	11
#define ic0timTIMER_ATTR_FIXED_POSITION	12
#define ic0timTIMER_ATTR_FIXED_SIMULATION	13
#define ic0timTIMER_ATTR_FIXED_LOCKED	14
#define ic0timTIMER_ATTR_PREFIX	15
#define ic0timTIMER_ATTR_PARAMETERS_SIZE	16
#define ic0timTIMER_ATTR_PARAMETERS	17
#define ic0timTIMER_ATTR_STATUS	18
#define ic0timTIMER_ATTR_STATUS_INDEX	19
#define ic0timTIMER_ATTR_VALUES_SIZE	20
#define ic0timTIMER_ATTR_VALUES	21
#define ic0timTIMER_ATTR_INIT_RESTRICTION	22
#define ic0timTIMER_ATTR_MOVE_RESTRICTION	23
#define ic0timTIMER_ATTR_STOP_TIMEOUT	24
#define ic0timTIMER_ATTR_SET_VAL_FCT	25
#define ic0timTIMER_ATTR_HELP_TEXT	26
#define ic0timTIMER_ATTR_PARAM_STEP	27
#define ic0timTIMER_ATTR_ACTION_COMMAND	28
#define ic0timTIMER_ATTR_ACTION_STOP_REQ	29
#define ic0timTIMER_ATTR_ACTION_NO_MOVE	30
#define ic0timTIMER_ATTR_ACTION_CHECK	31
#define ic0timTIMER_ATTR_RESET_SETUP_CMD	32
#define ic0timTIMER_ATTR_TIMER	33
#define ic0timTIMER_ATTR_PERIOD	34
#define ic0timTIMER_ATTR_REQID	35
#define ic0timTIMER_ATTR_START	36

#define ic0timTIMER_AUTOMATA	0

/*
 * function table
 */
#define ic0timTIMER_FCT_DELETE_OBJ	0
#define ic0timTIMER_FCT_STANDBY	1
#define ic0timTIMER_FCT_ONLINE	2
#define ic0timTIMER_FCT_OFF	3
#define ic0timTIMER_FCT_EXIT	4
#define ic0timTIMER_FCT_STATE	5
#define ic0timTIMER_FCT_SIMULAT	6
#define ic0timTIMER_FCT_STOP_SIM	7
#define ic0timTIMER_FCT_STATUS	8
#define ic0timTIMER_FCT_INIT	9
#define ic0timTIMER_FCT_STOP	10
#define ic0timTIMER_FCT_TEST	11
#define ic0timTIMER_FCT_SELFTST	12
#define ic0timTIMER_FCT_AUTO_TBL	13
#define ic0timTIMER_FCT_AUTO_RST	14
#define ic0timTIMER_FCT_INIT_OBJ	15
#define ic0timTIMER_FCT_EXIT_OBJ	16
#define ic0timTIMER_FCT_LOADED2STBY	17
#define ic0timTIMER_FCT_ONLINE2STBY	18
#define ic0timTIMER_FCT_EXIT1	19
#define ic0timTIMER_FCT_CHECK_DEVICE_NAME	20
#define ic0timTIMER_FCT_BREAK	21
#define ic0timTIMER_FCT_USR_INIT	22
#define ic0timTIMER_FCT_USR_LOADED2STANDBY	23
#define ic0timTIMER_FCT_USR_STANDBY2ONLINE	24
#define ic0timTIMER_FCT_USR_ONLINE2STANDBY	25
#define ic0timTIMER_FCT_USR_STANDBY2LOADED	26
#define ic0timTIMER_FCT_USR_STOP	27
#define ic0timTIMER_FCT_USR_SIMULAT	28
#define ic0timTIMER_FCT_USR_STOP_SIM	29
#define ic0timTIMER_FCT_USR_STATUS_CODE	30
#define ic0timTIMER_FCT_USR_STATUS_LAST	31
#define ic0timTIMER_FCT_USR_STATUS_VALUE	32
#define ic0timTIMER_FCT_USR_TEST	33
#define ic0timTIMER_FCT_USR_SELF_TEST	34
#define ic0timTIMER_FCT_USR_ERROR	35
#define ic0timTIMER_FCT_AVAIL	36
#define ic0timTIMER_FCT_UNAVAIL	37
#define ic0timTIMER_FCT_DISABLE	38
#define ic0timTIMER_FCT_ENABLE	39
#define ic0timTIMER_FCT_LOCK	40
#define ic0timTIMER_FCT_UNLOCK	41
#define ic0timTIMER_FCT_HANDSET	42
#define ic0timTIMER_FCT_HSET_OFF	43
#define ic0timTIMER_FCT_FIX_POS	44
#define ic0timTIMER_FCT_MOVE	45
#define ic0timTIMER_FCT_MOVE_REL	46
#define ic0timTIMER_FCT_SET	47
#define ic0timTIMER_FCT_CHECK	48
#define ic0timTIMER_FCT_WAIT	49
#define ic0timTIMER_FCT_USR_SET_MODE	50
#define ic0timTIMER_FCT_SETUP	51
#define ic0timTIMER_FCT_GET_VAL	52
#define ic0timTIMER_FCT_SET_VAL	53
#define ic0timTIMER_FCT_CFG_ELEM	54
#define ic0timTIMER_FCT_LOG_CONF	55
#define ic0timTIMER_FCT_GET_CONF	56
#define ic0timTIMER_FCT_RESET_PARAMETERS	57
#define ic0timTIMER_FCT_PARSE_PARAMETERS	58
#define ic0timTIMER_FCT_COMPARE_PARAMETERS	59
#define ic0timTIMER_FCT_COPY_PARAMETERS	60
#define ic0timTIMER_FCT_EXEC_PARAMETERS	61
#define ic0timTIMER_FCT_UPDATE_PARAMETERS	62
#define ic0timTIMER_FCT_GET_PARAM_SET_VALUE	63
#define ic0timTIMER_FCT_GET_PARAM_STATUS	64
#define ic0timTIMER_FCT_SET_PARAM	65
#define ic0timTIMER_FCT_SET_PARAM_STEP	66
#define ic0timTIMER_FCT_USR_EXEC_ACTION	67
#define ic0timTIMER_FCT_USR_SET_PARAMETER	68
#define ic0timTIMER_FCT_GET_VALUE_CODE	69
#define ic0timTIMER_FCT_GET_VALUE	70
#define ic0timTIMER_FCT_USR_GET_VALUE	71
#define ic0timTIMER_FCT_SET_VALUE	72
#define ic0timTIMER_FCT_USR_SET_VALUE	73
#define ic0timTIMER_FCT_GET_ARG	74
#define ic0timTIMER_FCT_MONITOR	75
#define ic0timTIMER_FCT_USR_MONITOR	76
#define ic0timTIMER_FCT_LOG_ACTION	77
#define ic0timTIMER_FCT_LOG_COMMENT	78
#define ic0timTIMER_FCT_LOG_PARAMETER	79
#define ic0timTIMER_FCT_LOG_RECOVERY	80
#define ic0timTIMER_FCT_LOG_TBL_PARAM	81
#define ic0timTIMER_FCT_LOG_UNFORSEEN	82

/*
 * automata states
 */
#define ic0timTIMER_STATE_NOSTATE	-1
#define ic0timTIMER_STATE_OFF	0
#define ic0timTIMER_STATE_LOADED	1
#define ic0timTIMER_STATE_STANDBY	2
#define ic0timTIMER_STATE_ONLINE	3

/*
 * automata substates
 */
#define ic0timTIMER_SUBSTATE_ERROR	0
#define ic0timTIMER_SUBSTATE_IDLE	1
#define ic0timTIMER_SUBSTATE_BUSY	2
#define ic0timTIMER_SUBSTATE_INITIALISING	3
#define ic0timTIMER_SUBSTATE_SHUTTING_DOWN	4
#define ic0timTIMER_SUBSTATE_STOPPING	5
#define ic0timTIMER_SUBSTATE_COLD_STARTING	6
#define ic0timTIMER_SUBSTATE_STARTING	7
#define ic0timTIMER_SUBSTATE_STOP	8

/*
 * automata events
 */
#define ic0timTIMER_EVENT_INIT	0
#define ic0timTIMER_EVENT_STANDBY	1
#define ic0timTIMER_EVENT_ONLINE	2
#define ic0timTIMER_EVENT_STOP	3
#define ic0timTIMER_EVENT_OFF	4
#define ic0timTIMER_EVENT_EXIT	5
#define ic0timTIMER_EVENT_STATE	6
#define ic0timTIMER_EVENT_STOP_SIM	7
#define ic0timTIMER_EVENT_STATUS	8
#define ic0timTIMER_EVENT_SIMULAT	9
#define ic0timTIMER_EVENT_TEST	10
#define ic0timTIMER_EVENT_SELFTST	11
#define ic0timTIMER_EVENT_AUTO_TBL	12
#define ic0timTIMER_EVENT_AUTO_RST	13
#define ic0timTIMER_EVENT_DISABLE	14
#define ic0timTIMER_EVENT_ENABLE	15
#define ic0timTIMER_EVENT_LOCK	16
#define ic0timTIMER_EVENT_UNLOCK	17
#define ic0timTIMER_EVENT_HANDSET	18
#define ic0timTIMER_EVENT_HSET_OFF	19
#define ic0timTIMER_EVENT_AVAIL	20
#define ic0timTIMER_EVENT_UNAVAIL	21
#define ic0timTIMER_EVENT_MOVE	22
#define ic0timTIMER_EVENT_MOVE_REL	23
#define ic0timTIMER_EVENT_SET	24
#define ic0timTIMER_EVENT_CHECK	25
#define ic0timTIMER_EVENT_WAIT	26
#define ic0timTIMER_EVENT_SETUP	27
#define ic0timTIMER_EVENT_CFG_ELEM	28
#define ic0timTIMER_EVENT_GET_VAL	29
#define ic0timTIMER_EVENT_SET_VAL	30
#define ic0timTIMER_EVENT_LOG_CONF	31
#define ic0timTIMER_EVENT_GET_CONF	32
#define ic0timTIMER_EVENT_MONITOR	33


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_SETUP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_STATUS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_GET_CONF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_USR_INIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_USR_STOP)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	req,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*ic0timTIMER_FCTPTR_USR_ONLINE2STANDBY)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define ic0timTimerLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("ic0tim", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define ic0timErrTimerReadOldb(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_READ_OLDB: Device %s: Unable to read %s from OLDB.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_READ_OLDB,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0timErrTimerWriteOldb(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_WRITE_OLDB: Device %s: Unable to write %s to OLDB.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_WRITE_OLDB,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0timErrTimerNotLogical(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_NOT_LOGICAL: Device %s: Wrong ST value %s.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_NOT_LOGICAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0timErrTimerWrongStart(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_WRONG_START: Device %s: Wrong START value %s.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_WRONG_START,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0timErrTimerWrongPeriod(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_WRONG_PERIOD: Device %s: Wrong PERIOD value %s.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_WRONG_PERIOD,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define ic0timErrTimerGeneral(p1,p2) \
	{ \
	lcctooLOG_0("ic0timERR_TIMER_GENERAL: Device %s: %s.",(p1),(p2)); \
	errAdd(error,"ic0tim",ic0timERR_TIMER_GENERAL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define ic0timTimerGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define ic0timTimerReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_STATE,(buf),(error))
#define ic0timTimerWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_STATE,(void*) (buf),(error))
#define ic0timTimerReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_SUBSTATE,(buf),(error))
#define ic0timTimerWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_SUBSTATE,(void*) (buf),(error))
#define ic0timTimerReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ERROR,(buf),(error))
#define ic0timTimerWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ERROR,(void*) (buf),(error))
#define ic0timTimerReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_INITIALISED,(buf),(error))
#define ic0timTimerWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_INITIALISED,(void*) (buf),(error))
#define ic0timTimerReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_SIMULATION,(buf),(error))
#define ic0timTimerWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_SIMULATION,(void*) (buf),(error))
#define ic0timTimerReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_DEVICE,(buf),(error))
#define ic0timTimerWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_DEVICE,(void*) (buf),(error))
#define ic0timTimerReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_AVAILABLE,(buf),(error))
#define ic0timTimerWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_AVAILABLE,(void*) (buf),(error))
#define ic0timTimerReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ENABLED,(buf),(error))
#define ic0timTimerWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ENABLED,(void*) (buf),(error))
#define ic0timTimerReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_LOCKED,(buf),(error))
#define ic0timTimerWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_LOCKED,(void*) (buf),(error))
#define ic0timTimerReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_HANDSET,(buf),(error))
#define ic0timTimerWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_HANDSET,(void*) (buf),(error))
#define ic0timTimerReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_FIXED,(buf),(error))
#define ic0timTimerWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_FIXED,(void*) (buf),(error))
#define ic0timTimerReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_FIX_POS_SET,(buf),(error))
#define ic0timTimerWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define ic0timTimerReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_FIXED_POSITION,(buf),(error))
#define ic0timTimerWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define ic0timTimerReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_FIXED_SIMULATION,(buf),(error))
#define ic0timTimerWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define ic0timTimerReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_FIXED_LOCKED,(buf),(error))
#define ic0timTimerWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define ic0timTimerReadPrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_PREFIX,(buf),(error))
#define ic0timTimerWritePrefix(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_PREFIX,(void*) (buf),(error))
#define ic0timTimerReadParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_PARAMETERS_SIZE,(buf),(error))
#define ic0timTimerWriteParametersSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_PARAMETERS_SIZE,(void*) (buf),(error))
#define ic0timTimerGetParametersSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0timTIMER_ATTR_PARAMETERS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0timTimerReadParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0timTIMER_ATTR_PARAMETERS,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0timTimerWriteParameters(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0timTIMER_ATTR_PARAMETERS,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0timTimerGetStatusSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0timTIMER_ATTR_STATUS,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0timTimerReadStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0timTIMER_ATTR_STATUS,(buf),(firstElt),(lastElt),(error))
#define ic0timTimerWriteStatus(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0timTIMER_ATTR_STATUS,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0timTimerGetStatusIndexSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0timTIMER_ATTR_STATUS_INDEX,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0timTimerReadStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0timTIMER_ATTR_STATUS_INDEX,(buf),(firstElt),(lastElt),(error))
#define ic0timTimerWriteStatusIndex(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0timTIMER_ATTR_STATUS_INDEX,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0timTimerReadValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_VALUES_SIZE,(buf),(error))
#define ic0timTimerWriteValuesSize(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_VALUES_SIZE,(void*) (buf),(error))
#define ic0timTimerGetValuesSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0timTIMER_ATTR_VALUES,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0timTimerReadValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,0,ic0timTIMER_ATTR_VALUES,(buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0timTimerWriteValues(obj,buf,firstRec,lastRec,firstFld,lastFld,error) \
	lcctooCaiRWTable((obj)->attrs,1,ic0timTIMER_ATTR_VALUES,(void*) (buf),(firstRec),(lastRec),(firstFld),(lastFld),(error))
#define ic0timTimerReadInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_INIT_RESTRICTION,(buf),(error))
#define ic0timTimerWriteInitRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_INIT_RESTRICTION,(void*) (buf),(error))
#define ic0timTimerReadMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_MOVE_RESTRICTION,(buf),(error))
#define ic0timTimerWriteMoveRestriction(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_MOVE_RESTRICTION,(void*) (buf),(error))
#define ic0timTimerReadStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_STOP_TIMEOUT,(buf),(error))
#define ic0timTimerWriteStopTimeout(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_STOP_TIMEOUT,(void*) (buf),(error))
#define ic0timTimerReadSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_SET_VAL_FCT,(buf),(error))
#define ic0timTimerWriteSetValFct(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_SET_VAL_FCT,(void*) (buf),(error))
#define ic0timTimerGetHelpTextSize(obj,num,error) \
	caiGetAttrInfo((obj)->attrs,ic0timTIMER_ATTR_HELP_TEXT,NULL,NULL,(num),NULL, NULL,NULL,(error))
#define ic0timTimerReadHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,0,ic0timTIMER_ATTR_HELP_TEXT,(buf),(firstElt),(lastElt),(error))
#define ic0timTimerWriteHelpText(obj,buf,firstElt,lastElt,error) \
	caiRWVector((obj)->attrs,1,ic0timTIMER_ATTR_HELP_TEXT,(void*) (buf),(firstElt),(lastElt),(error))
#define ic0timTimerReadParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_PARAM_STEP,(buf),(error))
#define ic0timTimerWriteParamStep(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_PARAM_STEP,(void*) (buf),(error))
#define ic0timTimerReadActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ACTION_COMMAND,(buf),(error))
#define ic0timTimerWriteActionCommand(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ACTION_COMMAND,(void*) (buf),(error))
#define ic0timTimerReadActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ACTION_STOP_REQ,(buf),(error))
#define ic0timTimerWriteActionStopReq(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ACTION_STOP_REQ,(void*) (buf),(error))
#define ic0timTimerReadActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ACTION_NO_MOVE,(buf),(error))
#define ic0timTimerWriteActionNoMove(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ACTION_NO_MOVE,(void*) (buf),(error))
#define ic0timTimerReadActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_ACTION_CHECK,(buf),(error))
#define ic0timTimerWriteActionCheck(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_ACTION_CHECK,(void*) (buf),(error))
#define ic0timTimerReadResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_RESET_SETUP_CMD,(buf),(error))
#define ic0timTimerWriteResetSetupCmd(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_RESET_SETUP_CMD,(void*) (buf),(error))
#define ic0timTimerReadTimer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_TIMER,(buf),(error))
#define ic0timTimerWriteTimer(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_TIMER,(void*) (buf),(error))
#define ic0timTimerReadPeriod(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_PERIOD,(buf),(error))
#define ic0timTimerWritePeriod(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_PERIOD,(void*) (buf),(error))
#define ic0timTimerReadReqid(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_REQID,(buf),(error))
#define ic0timTimerWriteReqid(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_REQID,(void*) (buf),(error))
#define ic0timTimerReadStart(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,ic0timTIMER_ATTR_START,(buf),(error))
#define ic0timTimerWriteStart(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,ic0timTIMER_ATTR_START,(void*) (buf),(error))
#define ic0timTimerGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[ic0timTIMER_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define ic0timTimerGetVal(obj,devName,name,nameNum,status,error) \
	ic0devSoftGetVal((obj),(devName),(name),(nameNum),(status),(error))
#define ic0timTimerSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	ic0devSoftSetVal((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0timTimerCfgElem(obj,devName,position,id,values,valuesNum,error) \
	ic0devSoftCfgElem((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0timTimerLogConf(obj,devName,allElements,error) \
	ic0devSoftLogConf((obj),(devName),(allElements),(error))
#define ic0timTimerMove(obj,devName,position,positionNum,error) \
	ic0devSoftMove((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerMoveRel(obj,devName,position,positionNum,error) \
	ic0devSoftMoveRel((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerSet(obj,devName,position,positionNum,error) \
	ic0devSoftSet((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerResetParameters(obj,devName,cmd,error) \
	ic0devSoftResetParameters((obj),(devName),(cmd),(error))
#define ic0timTimerParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	ic0devSoftParseParameters((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0timTimerCompareParameters(obj,devName,cmd,equal,error) \
	ic0devSoftCompareParameters((obj),(devName),(cmd),(equal),(error))
#define ic0timTimerCopyParameters(obj,devName,error) \
	ic0devSoftCopyParameters((obj),(devName),(error))
#define ic0timTimerExecParameters(obj,devName,error) \
	ic0devSoftExecParameters((obj),(devName),(error))
#define ic0timTimerUpdateParameters(obj,devName,cache,error) \
	ic0devSoftUpdateParameters((obj),(devName),(cache),(error))
#define ic0timTimerGetParamSetValue(obj,devName,index,value,setCode,error) \
	ic0devSoftGetParamSetValue((obj),(devName),(index),(value),(setCode),(error))
#define ic0timTimerGetParamStatus(obj,devName,index,flags,status,error) \
	ic0devSoftGetParamStatus((obj),(devName),(index),(flags),(status),(error))
#define ic0timTimerSetParam(obj,devName,index,cmd,value,step,error) \
	ic0devSoftSetParam((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0timTimerSetParamStep(obj,devName,index,cmd,step,error) \
	ic0devSoftSetParamStep((obj),(devName),(index),(cmd),(step),(error))
#define ic0timTimerUsrExecAction(obj,devName,index,error) \
	ic0devSoftUsrExecAction((obj),(devName),(index),(error))
#define ic0timTimerUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	ic0devSoftUsrSetParameter((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0timTimerGetValueCode(obj,devName,name,code,error) \
	ic0devSoftGetValueCode((obj),(devName),(name),(code),(error))
#define ic0timTimerGetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftGetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0timTimerUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrGetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0timTimerSetValue(obj,devName,code,index,value,valueNum,error) \
	ic0devSoftSetValue((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0timTimerUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	ic0devSoftUsrSetValue((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0timTimerGetArg(obj,devName,code,index,value,error) \
	ic0devSoftGetArg((obj),(devName),(code),(index),(value),(error))
#define ic0timTimerMonitor(obj,devName,error) \
	ic0devSoftMonitor((obj),(devName),(error))
#define ic0timTimerUsrMonitor(obj,devName,state,error) \
	ic0devSoftUsrMonitor((obj),(devName),(state),(error))
#define ic0timTimerLogAction(obj,devName,action,comment,error) \
	ic0devSoftLogAction((obj),(devName),(action),(comment),(error))
#define ic0timTimerLogComment(obj,devName,comment,error) \
	ic0devSoftLogComment((obj),(devName),(comment),(error))
#define ic0timTimerLogParameter(obj,devName,parameter,index,value,comment,error) \
	ic0devSoftLogParameter((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0timTimerLogRecovery(obj,devName,comment,error) \
	ic0devSoftLogRecovery((obj),(devName),(comment),(error))
#define ic0timTimerLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	ic0devSoftLogTblParam((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0timTimerLogUnforseen(obj,devName,comment,error) \
	ic0devSoftLogUnforseen((obj),(devName),(comment),(error))
#define ic0timTimerInitObj(obj,error) \
	ic0devSoftInitObj((obj),(error))
#define ic0timTimerExitObj(obj,error) \
	ic0devSoftExitObj((obj),(error))
#define ic0timTimerAvail(obj,devName,error) \
	lccdevCmdAvail((obj),(devName),(error))
#define ic0timTimerUnavail(obj,devName,error) \
	lccdevCmdUnavail((obj),(devName),(error))
#define ic0timTimerDisable(obj,devName,access,error) \
	lccdevCmdDisable((obj),(devName),(access),(error))
#define ic0timTimerEnable(obj,devName,access,error) \
	lccdevCmdEnable((obj),(devName),(access),(error))
#define ic0timTimerLock(obj,devName,error) \
	lccdevCmdLock((obj),(devName),(error))
#define ic0timTimerUnlock(obj,devName,error) \
	lccdevCmdUnlock((obj),(devName),(error))
#define ic0timTimerHandset(obj,devName,error) \
	lccdevCmdHandset((obj),(devName),(error))
#define ic0timTimerHsetOff(obj,devName,error) \
	lccdevCmdHsetOff((obj),(devName),(error))
#define ic0timTimerFixPos(obj,devName,position,positionNum,reset,error) \
	lccdevCmdFixPos((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0timTimerCheck(obj,devName,status,error) \
	lccdevCmdCheck((obj),(devName),(status),(error))
#define ic0timTimerWait(obj,devName,timeout,error) \
	lccdevCmdWait((obj),(devName),(timeout),(error))
#define ic0timTimerUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	lccdevCmdUsrSetMode((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0timTimerUsrStatusCode(obj,devName,text,flags,code,error) \
	lccdevCmdUsrStatusCode((obj),(devName),(text),(flags),(code),(error))
#define ic0timTimerUsrStatusValue(obj,devName,code,value,error) \
	lccdevCmdUsrStatusValue((obj),(devName),(code),(value),(error))
#define ic0timTimerUsrStatusLast(obj,devName,last,error) \
	lccdevCmdUsrStatusLast((obj),(devName),(last),(error))
#define ic0timTimerUsrSimulat(obj,devName,error) \
	lccdevCmdUsrSimulat((obj),(devName),(error))
#define ic0timTimerUsrStopSim(obj,devName,error) \
	lccdevCmdUsrStopSim((obj),(devName),(error))
#define ic0timTimerUsrLoaded2Standby(obj,devName,error) \
	lccdevCmdUsrLoaded2Standby((obj),(devName),(error))
#define ic0timTimerUsrStandby2Online(obj,devName,error) \
	lccdevCmdUsrStandby2Online((obj),(devName),(error))
#define ic0timTimerStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define ic0timTimerOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define ic0timTimerOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define ic0timTimerExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define ic0timTimerState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define ic0timTimerSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define ic0timTimerStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define ic0timTimerInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define ic0timTimerStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define ic0timTimerTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define ic0timTimerSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define ic0timTimerAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define ic0timTimerAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define ic0timTimerLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define ic0timTimerOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define ic0timTimerExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define ic0timTimerCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define ic0timTimerBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define ic0timTimerUsrStandby2Loaded(obj,devName,error) \
	lccdevStdUsrStandby2Loaded((obj),(devName),(error))
#define ic0timTimerUsrTest(obj,devName,error) \
	lccdevStdUsrTest((obj),(devName),(error))
#define ic0timTimerUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define ic0timTimerUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT ic0timTimerSetup(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	noMove,
    IN    vltLOGICAL	check,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerStatus(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	function[],	/* max. vltBYTES32[999] */
    IN    vltINT16	functionNum,
    IN    vltLOGICAL	flag,
    IN    vltLOGICAL	header,
    IN    vltLOGICAL	conflog,
    IN    vltLOGICAL	opslog,
    IN    vltLOGICAL	condlog,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerGetConf(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	allElements,
    OUT   char**	text,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerUsrInit(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerUsrStop(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltLOGICAL	req,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerUsrOnline2Standby(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define ic0timTimerVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[ic0timTIMER_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[ic0timTIMER_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[ic0timTIMER_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[ic0timTIMER_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[ic0timTIMER_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define ic0timTimerVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[ic0timTIMER_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[ic0timTIMER_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((ic0timTIMER_FCTPTR_STATUS)((obj)->fcts[ic0timTIMER_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define ic0timTimerVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[ic0timTIMER_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[ic0timTIMER_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[ic0timTIMER_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[ic0timTIMER_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[ic0timTIMER_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define ic0timTimerVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[ic0timTIMER_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVInitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_INIT_OBJ)((obj)->fcts[ic0timTIMER_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define ic0timTimerVExitObj(obj,error) \
	(*((ic0devSOFT_FCTPTR_EXIT_OBJ)((obj)->fcts[ic0timTIMER_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define ic0timTimerVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[ic0timTIMER_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[ic0timTIMER_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[ic0timTIMER_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[ic0timTIMER_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[ic0timTIMER_FCT_BREAK]))) \
	 ((obj),(error))
#define ic0timTimerVUsrInit(obj,devName,error) \
	(*((ic0timTIMER_FCTPTR_USR_INIT)((obj)->fcts[ic0timTIMER_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[ic0timTIMER_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrStandby2Online(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[ic0timTIMER_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrOnline2Standby(obj,devName,error) \
	(*((ic0timTIMER_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[ic0timTIMER_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrStandby2Loaded(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[ic0timTIMER_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrStop(obj,devName,req,error) \
	(*((ic0timTIMER_FCTPTR_USR_STOP)((obj)->fcts[ic0timTIMER_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define ic0timTimerVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[ic0timTIMER_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[ic0timTIMER_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_CODE)((obj)->fcts[ic0timTIMER_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define ic0timTimerVUsrStatusLast(obj,devName,last,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_LAST)((obj)->fcts[ic0timTIMER_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define ic0timTimerVUsrStatusValue(obj,devName,code,value,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[ic0timTIMER_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define ic0timTimerVUsrTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_TEST)((obj)->fcts[ic0timTIMER_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[ic0timTIMER_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[ic0timTIMER_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[ic0timTIMER_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[ic0timTIMER_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[ic0timTIMER_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0timTimerVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[ic0timTIMER_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define ic0timTimerVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[ic0timTIMER_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[ic0timTIMER_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[ic0timTIMER_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[ic0timTIMER_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[ic0timTIMER_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define ic0timTimerVMove(obj,devName,position,positionNum,error) \
	(*((ic0devSOFT_FCTPTR_MOVE)((obj)->fcts[ic0timTIMER_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerVMoveRel(obj,devName,position,positionNum,error) \
	(*((ic0devSOFT_FCTPTR_MOVE_REL)((obj)->fcts[ic0timTIMER_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerVSet(obj,devName,position,positionNum,error) \
	(*((ic0devSOFT_FCTPTR_SET)((obj)->fcts[ic0timTIMER_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define ic0timTimerVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[ic0timTIMER_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define ic0timTimerVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[ic0timTIMER_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define ic0timTimerVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((lccdevCMD_FCTPTR_USR_SET_MODE)((obj)->fcts[ic0timTIMER_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))
#define ic0timTimerVSetup(obj,devName,function,functionNum,noMove,check,error) \
	(*((ic0timTIMER_FCTPTR_SETUP)((obj)->fcts[ic0timTIMER_FCT_SETUP]))) \
	 ((obj),(devName),(function),(functionNum),(noMove),(check),(error))
#define ic0timTimerVGetVal(obj,devName,name,nameNum,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_VAL)((obj)->fcts[ic0timTIMER_FCT_GET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(status),(error))
#define ic0timTimerVSetVal(obj,devName,name,nameNum,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VAL)((obj)->fcts[ic0timTIMER_FCT_SET_VAL]))) \
	 ((obj),(devName),(name),(nameNum),(value),(valueNum),(error))
#define ic0timTimerVCfgElem(obj,devName,position,id,values,valuesNum,error) \
	(*((ic0devSOFT_FCTPTR_CFG_ELEM)((obj)->fcts[ic0timTIMER_FCT_CFG_ELEM]))) \
	 ((obj),(devName),(position),(id),(values),(valuesNum),(error))
#define ic0timTimerVLogConf(obj,devName,allElements,error) \
	(*((ic0devSOFT_FCTPTR_LOG_CONF)((obj)->fcts[ic0timTIMER_FCT_LOG_CONF]))) \
	 ((obj),(devName),(allElements),(error))
#define ic0timTimerVGetConf(obj,devName,allElements,text,error) \
	(*((ic0timTIMER_FCTPTR_GET_CONF)((obj)->fcts[ic0timTIMER_FCT_GET_CONF]))) \
	 ((obj),(devName),(allElements),(text),(error))
#define ic0timTimerVResetParameters(obj,devName,cmd,error) \
	(*((ic0devSOFT_FCTPTR_RESET_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_RESET_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(error))
#define ic0timTimerVParseParameters(obj,devName,cmd,params,paramsNum,noMove,check,error) \
	(*((ic0devSOFT_FCTPTR_PARSE_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_PARSE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(params),(paramsNum),(noMove),(check),(error))
#define ic0timTimerVCompareParameters(obj,devName,cmd,equal,error) \
	(*((ic0devSOFT_FCTPTR_COMPARE_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_COMPARE_PARAMETERS]))) \
	 ((obj),(devName),(cmd),(equal),(error))
#define ic0timTimerVCopyParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_COPY_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_COPY_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVExecParameters(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_EXEC_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_EXEC_PARAMETERS]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUpdateParameters(obj,devName,cache,error) \
	(*((ic0devSOFT_FCTPTR_UPDATE_PARAMETERS)((obj)->fcts[ic0timTIMER_FCT_UPDATE_PARAMETERS]))) \
	 ((obj),(devName),(cache),(error))
#define ic0timTimerVGetParamSetValue(obj,devName,index,value,setCode,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_SET_VALUE)((obj)->fcts[ic0timTIMER_FCT_GET_PARAM_SET_VALUE]))) \
	 ((obj),(devName),(index),(value),(setCode),(error))
#define ic0timTimerVGetParamStatus(obj,devName,index,flags,status,error) \
	(*((ic0devSOFT_FCTPTR_GET_PARAM_STATUS)((obj)->fcts[ic0timTIMER_FCT_GET_PARAM_STATUS]))) \
	 ((obj),(devName),(index),(flags),(status),(error))
#define ic0timTimerVSetParam(obj,devName,index,cmd,value,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM)((obj)->fcts[ic0timTIMER_FCT_SET_PARAM]))) \
	 ((obj),(devName),(index),(cmd),(value),(step),(error))
#define ic0timTimerVSetParamStep(obj,devName,index,cmd,step,error) \
	(*((ic0devSOFT_FCTPTR_SET_PARAM_STEP)((obj)->fcts[ic0timTIMER_FCT_SET_PARAM_STEP]))) \
	 ((obj),(devName),(index),(cmd),(step),(error))
#define ic0timTimerVUsrExecAction(obj,devName,index,error) \
	(*((ic0devSOFT_FCTPTR_USR_EXEC_ACTION)((obj)->fcts[ic0timTIMER_FCT_USR_EXEC_ACTION]))) \
	 ((obj),(devName),(index),(error))
#define ic0timTimerVUsrSetParameter(obj,devName,cmd,param1,param1SetCode,param2,param2SetCode,value,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_PARAMETER)((obj)->fcts[ic0timTIMER_FCT_USR_SET_PARAMETER]))) \
	 ((obj),(devName),(cmd),(param1),(param1SetCode),(param2),(param2SetCode),(value),(error))
#define ic0timTimerVGetValueCode(obj,devName,name,code,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE_CODE)((obj)->fcts[ic0timTIMER_FCT_GET_VALUE_CODE]))) \
	 ((obj),(devName),(name),(code),(error))
#define ic0timTimerVGetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_GET_VALUE)((obj)->fcts[ic0timTIMER_FCT_GET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0timTimerVUsrGetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_GET_VALUE)((obj)->fcts[ic0timTIMER_FCT_USR_GET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0timTimerVSetValue(obj,devName,code,index,value,valueNum,error) \
	(*((ic0devSOFT_FCTPTR_SET_VALUE)((obj)->fcts[ic0timTIMER_FCT_SET_VALUE]))) \
	 ((obj),(devName),(code),(index),(value),(valueNum),(error))
#define ic0timTimerVUsrSetValue(obj,devName,valuesTable,valuesTableNum,code,index,value,valueNum,done,error) \
	(*((ic0devSOFT_FCTPTR_USR_SET_VALUE)((obj)->fcts[ic0timTIMER_FCT_USR_SET_VALUE]))) \
	 ((obj),(devName),(valuesTable),(valuesTableNum),(code),(index),(value),(valueNum),(done),(error))
#define ic0timTimerVGetArg(obj,devName,code,index,value,error) \
	(*((ic0devSOFT_FCTPTR_GET_ARG)((obj)->fcts[ic0timTIMER_FCT_GET_ARG]))) \
	 ((obj),(devName),(code),(index),(value),(error))
#define ic0timTimerVMonitor(obj,devName,error) \
	(*((ic0devSOFT_FCTPTR_MONITOR)((obj)->fcts[ic0timTIMER_FCT_MONITOR]))) \
	 ((obj),(devName),(error))
#define ic0timTimerVUsrMonitor(obj,devName,state,error) \
	(*((ic0devSOFT_FCTPTR_USR_MONITOR)((obj)->fcts[ic0timTIMER_FCT_USR_MONITOR]))) \
	 ((obj),(devName),(state),(error))
#define ic0timTimerVLogAction(obj,devName,action,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_ACTION)((obj)->fcts[ic0timTIMER_FCT_LOG_ACTION]))) \
	 ((obj),(devName),(action),(comment),(error))
#define ic0timTimerVLogComment(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_COMMENT)((obj)->fcts[ic0timTIMER_FCT_LOG_COMMENT]))) \
	 ((obj),(devName),(comment),(error))
#define ic0timTimerVLogParameter(obj,devName,parameter,index,value,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_PARAMETER)((obj)->fcts[ic0timTIMER_FCT_LOG_PARAMETER]))) \
	 ((obj),(devName),(parameter),(index),(value),(comment),(error))
#define ic0timTimerVLogRecovery(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_RECOVERY)((obj)->fcts[ic0timTIMER_FCT_LOG_RECOVERY]))) \
	 ((obj),(devName),(comment),(error))
#define ic0timTimerVLogTblParam(obj,devName,paramIndex,flags,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_TBL_PARAM)((obj)->fcts[ic0timTIMER_FCT_LOG_TBL_PARAM]))) \
	 ((obj),(devName),(paramIndex),(flags),(comment),(error))
#define ic0timTimerVLogUnforseen(obj,devName,comment,error) \
	(*((ic0devSOFT_FCTPTR_LOG_UNFORSEEN)((obj)->fcts[ic0timTIMER_FCT_LOG_UNFORSEEN]))) \
	 ((obj),(devName),(comment),(error))


ccsCOMPL_STAT ic0timTimerNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0timTimerDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !IC0TIM_TIMER_H

/************************************************************************/
/* ___oOo___ */
