#ifndef IKON_COMMANDS_H
#define IKON_COMMANDS_H
/******************************************************************************
* E.S.O - VLT project
*
* "@(#) $Id: ikonCommands.h,v 2.5 2003/07/24 13:22:48 vltsccm Exp $" 
*
* ikonCommands.h - ioctl control command literals of IKON driver
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* S.Sandrock 06.03.1995 created
* S.Sandrock 16.10.1995 added group position commands
* S.Sandrock 06.03.1996 extended for firmware "246 118 02"
* S.Sandrock 29.01.1998 added ikonCMD_REF_CHECK_X1/X2
*/

					/* Formats: argument	result	    */
/*
 * Initialization
 */
#define ikonCMD_POWER_ON_SELF_TEST	100	/*F ""		%d	    */
#define ikonCMD_BREAK			101	/*F ""		""	    */
#define ikonCMD_REF_MOVE		110	/*F ""		""	    */
#define ikonCMD_REF_MOVE_X1		111	/*F ""		""	    */
#define ikonCMD_REF_MOVE_X2		112	/*F ""		""	    */
#define ikonCMD_REF_CHECK_X1		113	/*F ""		""	    */
#define ikonCMD_REF_CHECK_X2		114	/*F ""		""	    */

/*
 * Parameters
 */
#define ikonCMD_WRITE_DEFAULT_PARAMS	200	/*F ""		""	    */
#define ikonCMD_READ_PARAMETER		201	/*F %lf;%lf	%lf;%lf	    */
#define ikonCMD_WRITE_PARAMETER		202	/*F %lf;%lf	%lf;%lf	    */
#define ikonCMD_UPDATE_PARAMS		203	/*F ""		""	    */

/*
 * Position
 */
#define ikonCMD_LATCH_POSITION_X1	301	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_LATCH_POSITION_X2	302	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_LATCH_POSITION_COMBI	303	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_LATCH_POSITION_GROUP	304	/*F ""		""	     */

#define ikonCMD_READ_POSITION_X1	311	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_READ_POSITION_X2	312	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_READ_POSITION_COMBI	313	/*F ""		%lf;%d;%d;%d */

#define ikonCMD_SET_POSITION_X1		331	/*F %lf		""	    */
#define ikonCMD_SET_POSITION_X2		332	/*F %lf		""	    */
#define ikonCMD_SET_POSITION_COMBI	333	/*F %lf		""	    */

#define ikonCMD_DISABLE_LATCH_WITH_READ	350	/*F ""		""	    */
#define ikonCMD_ENABLE_LATCH_WITH_READ	351	/*F ""		""	    */

#define ikonCMD_GET_SCALE_FACTOR	360	/*F ""		%lf	    */
#define ikonCMD_SET_SCALE_FACTOR	361	/*F %lf		""	    */

/*
 * Auxiliary Commands
 */
#define ikonCMD_SCAN_POSITION_X1	401	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_SCAN_POSITION_X2	402	/*F ""		%lf;%d;%d;%d */
#define ikonCMD_SCAN_POSITION_COMBI	403	/*F ""		%lf;%d;%d;%d */

/*
 * Correction Table Operations
 */
#define ikonCMD_RECORD_CORR_X1		501	/*F ""		""	    */
#define ikonCMD_RECORD_CORR_X2		502	/*F ""		""	    */
#define ikonCMD_READ_CORR_TABLE_X1	511	/*F %sd		%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd */
#define ikonCMD_WRITE_CORR_TABLE_X1	512	/*F %sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd	"" */
#define ikonCMD_READ_CORR_TABLE_X2	521	/*F %sd		%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd */
#define ikonCMD_WRITE_CORR_TABLE_X2	522	/*F %sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd;%sd	"" */

/*
 * Driver Commands
 */
#define ikonCMD_GET_DRIVER_VERSION	901	/*F ""		%d	    */
#define ikonCMD_GET_TIMEOUT		902	/*F ""		%d	    */
#define ikonCMD_SET_TIMEOUT		903	/*F %d		""	    */
#define ikonCMD_FREE_DEVICE		904	/*F ""		""	    */



/*
** Typedefs for structs required as argument by certain commands
*/

/* for named parameters as argument: */
typedef struct
{
    double spec;	/* as in the HW manual, e.g. 1.1 or 3 or 71.3	*/
    double value;	/* type is converted automatically		*/
} ikonARG_PARAM;

/* argument for position commands: */
typedef struct
{
    double pos;		/* scaled position value in IEEE floating point	*/
    int status;		/* channel status bits, see ikonARG_POS_STATUS	*/
    int counter;	/* 32-bit raw binary counter value (signed)	*/
    int interpolation;	/* 16-bit raw binary interpolation value	*/
} ikonARG_POS;

/* to be mapped over `status' member in `ikonARG_POS': */
typedef struct
{
    uint32_t
	_reserved16		:16,
	_reserved12		: 3,
	warn_CrcChecksum	: 1,
	warn_RefDistance	: 1,
	warn_CouldNotLatch	: 1,
	warn_DoubleLatch	: 1,
	warn_NoLatch		: 1,
	err_CorrWait		: 1,
	_reserved6		: 1,
	err_RefWait		: 1,
	err_InvFrequency	: 1,
	err_LowAmplitude	: 1,
	info_CounterStarted	: 1,
	_reserved1		: 1,
	info_SignalCorrection	: 1;
} ikonARG_POS_STATUS;


/* These bits in position status indicate an error (position invalid): */
#define ikonPOS_STATUS_ERROR_MASK	0x00B8
/* These bits in position status indicate a warning (position questionable): */
#define ikonPOS_STATUS_WARNING_MASK	0xFF00
/* These bits in position status are for information only: */
#define ikonPOS_STATUS_INFO_MASK	0x0005


/* valid range of position value in floating-point type: */
#define ikonPOS_RANGE (65536.0 * 65536.0 * 65536.0)

/* minimum numerical value of a 48-bit position in floating-point type: */
#define ikonPOS_MIN (-ikonPOS_RANGE * 0.5)

/* maximum numerical value of a 48-bit position in floating-point type: */
#define ikonPOS_MAX (ikonPOS_RANGE * 0.5 - 1.0)


/* Correction value data: */
#define ikonCORR_RECORD_COEFF_NUM	8     /* # of coefficients in record */
typedef struct
{
    uint16_t record;				/* record number [0..P8+1] */
    uint16_t coeff[ikonCORR_RECORD_COEFF_NUM];	/* coefficients            */
    uint16_t bcc;				/* block check character   */
    uint16_t crc;				/* overall CRC checksum    */
} ikonARG_CORR_RECORD;


#endif /* !IKON_COMMANDS_H */
