/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ioLCC.h,v 4.97.1.1 2005/10/24 13:55:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2005-10-11 added configuration and functions for mendrv
* bgustafs 2005-09-08 added definitions for CAN sync signals
* bgustafs  13/07/99  moved ioReadTrigger/ioWriteTrigger to ioProvided.h
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
* bgustafs  08/03/96  modified for lcc 3.1
* mkiekebu  07/08/97  Added ioWriteTrigger
* mkiekebu  08/08/97  Added ioReadTrigger

*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software              SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  2.1          31/01/95        P.Rochez        Changed to ESO header structure
*  3.1          16/11/95        P.Creten        Added ioStartSampling() and
*                                               ioStopSampling()
*------------------------------------------------------------------------------
*  SCCS Id:  ioLCC.h  3.1  05 Mar 1996
*----------------------------------------------------------------------------*/

/*
 * <ioLCC.h> - LCU / SIGNALS Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/

/*
 * Header files
 */

#include "ccs.h"
#include "db.h"
#include "lcc.h"


/*
 * Constants
 */
/* none */

/*
 * Data types
 */

typedef enum {
    ioGAIN_1 = 1,
    ioGAIN_2,
    ioGAIN_4,
    ioGAIN_8,
    ioGAIN_10,
    ioGAIN_100,
    ioGAIN_500
} ioGAIN;

typedef enum {                      /* List of signal types             */
    ioANALOG = 1,
    ioDIGITAL
} ioTYPE;


typedef enum {                     /* Input or output signals           */
    ioINPUT = 1,
    ioOUTPUT
} ioIN_OUT;


typedef enum {                      /* List of active logical levels    */
    ioHIGH = 1,                     /* Active high                      */
    ioLOW                           /* Active low                       */
} ioLEVEL;


typedef struct {                    /* Signal description               */
    dbSYMADDRESS name;              /* Signal name                      */
    ioTYPE       type;              /* Signal type                      */
    ioIN_OUT     inOut;             /* Input or output                  */
} ioSIGNAL;


typedef struct {
    void      *address;             /* Address of signal description    */
    vltUINT32 key;                  /* Address validation key           */
} ioDIRADDRESS;                     /* Direct address of signal         */


typedef enum {
     ioALL = 1,                  /* Sampling enabled for all the items  */
     ioSPECIFIC                  /* Sampling enabled for specific items */
} ioSCOPE;

typedef enum {
    ioMEN58_PORT_A = 1,          /* Port A of the MEN M58 board         */
    ioMEN58_PORT_B,              /* Port B of the MEN M58 board         */
    ioMEN58_PORT_C               /* Port C of the MEN M58 board         */
} ioMEN58_PORT;

/*
 * Functions prototypes
 */

ccsCOMPL_STAT ioConfigDigital(      /* Configure a digital signal       */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    const lccDEVICE_NAME deviceName,/* Name of the associated device    */
    vltUINT8           startBit,    /* First bit (0-63)                 */
    vltUINT8           bitNumber,   /* Number of bits (1-32)            */
    ioIN_OUT           inOut,       /* Input or output signal           */
    ioLEVEL            level,       /* Logical level                    */
    vltUINT32          simValue,    /* Default simulation value         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioConfigAnalog(       /* Configure an analog signal       */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    const lccDEVICE_NAME deviceName,/* Name of the associated device    */
    vltUINT8           signalNumber,/* Hardware signal number (0-15)    */
    ioIN_OUT           inOut,       /* Input or output signal           */
    vltFLOAT           convertFactor,
                                    /* Conversion factor                */
    vltFLOAT           lowerRange,  /* Lower range for output signals   */
    vltFLOAT           higherRange, /* Higher range for output signals  */
    ioGAIN             gainFactor,  /* Gain factor                      */
    vltFLOAT           simValue,    /* Default simulation value         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioConfigCANAnalogSync(   /* Configure analog signals         */
    const lccDEVICE_NAME deviceName,   /* Name of the associated device    */
    const vltINT32       firstSignal,  /* Number of the first signal       */
    const vltINT32       numSignals,   /* Number of signals to use sync    */
    const vltLOGICAL     syncEnable,   /* Flag for enabling/disabling sync */
    ccsERROR            *error         /* Returned error structure         */
    ) ;


ccsCOMPL_STAT ioCANSendSync(          /* Configure analog signals         */
    const lccDEVICE_NAME deviceName,  /* Name of the associated device    */
    ccsERROR            *error        /* Returned error structure         */
    );

ccsCOMPL_STAT ioConfigMenDigital(     /* Configure a digital signal        */
    const lccDEVICE_NAME deviceName,  /* Name of the associated device     */
    const vltLOGICAL     terminator,  /* Enable internal terminator        */
    const vltUINT32      port,        /* Port to enable terminator         */
    const vltLOGICAL     enableTrig,  /* Enable external trig signal       */
    const vltLOGICAL     latchOnTrig, /* Latch signals on external trig    */
    const vltLOGICAL     trigOnPosTrans, 
                                      /* Trigger on low to high transition */
    ccsERROR             *error       /* Returned error structure          */
    );

ccsCOMPL_STAT ioEnableInterrupt(      /* Configure and enable the trig 
                                         signal interrupt                  */
    const lccDEVICE_NAME deviceName,  /* Name of the associated device     */
    const FUNCPTR*       routine,     /* User routine                      */
    const vltINT32       parameter,   /* Parameter passed to the routine   */
    ccsERROR             *error       /* Returned error structure          */
    );

ccsCOMPL_STAT ioDisableInterrupt(     /* Disable the trig signal interrupt */
    const lccDEVICE_NAME deviceName,  /* Name of the associated device     */
    ccsERROR             *error       /* Returned error structure          */
    );

ccsCOMPL_STAT ioGetList(            /* Get the list of all signals      */
    vltUINT32          *number,     /* Number of signals                */
    ioSIGNAL           **list,      /* List of signals                  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioGetDigitalConfig(   /* Get the configuration of a       */
                                    /* digital signal                   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    lccDEVICE_NAME     deviceName,  /* Name of the associated device    */
    vltUINT8           *startBit,   /* Start bit (0-63)                 */
    vltUINT8           *bitNumber,  /* Number of bits (1-32)            */
    ioIN_OUT           *inOut,      /* Input or output signal           */
    ioLEVEL            *level,      /* Logical level                    */
    vltUINT32          *simValue,   /* Default simulation value         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioGetAnalogConfig(    /* Get the configuration of an      */
                                    /* analog signal                    */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    lccDEVICE_NAME     deviceName,  /* Name of the associated device    */
    vltUINT32          *signalNumber,
                                    /* Hardware signal number (0-15)    */
    ioIN_OUT           *inOut,      /* Input or output signal           */
    vltFLOAT           *convertFactor,
                                    /* Conversion factor                */
    vltFLOAT           *lowerRange,  /* Lower range for output signals  */
    vltFLOAT           *higherRange,/* Higher range for output signals  */
    ioGAIN             *gainFactor, /* Gain factor                      */
    vltFLOAT           *simValue,   /* Default simulation value         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioSetDigitalSimValue( /* Set the simulation value of a    */
                                    /* digital signal                   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltUINT32          simValue,    /* Simulation value                 */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT ioSetAnalogSimValue(  /* Set the simulation value of an   */
                                    /* analog signal                    */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltFLOAT           simValue,    /* Simulation value                 */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioWriteDigital(       /* Write value to a digital signal  */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltUINT32          value,       /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioWriteAnalog(        /* Write value to an analog signal  */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltFLOAT           value,       /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioReadDigital(        /* Read value of a digital signal   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltUINT32          *value,      /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioReadAnalog(         /* Read value of an analog signal   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltFLOAT           *value,      /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioReadAnalogRaw(      /* Read value of an analog signal   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltFLOAT           *value,      /* Value                            */
    vltFLOAT           *rawValue,   /* Raw value                        */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioSetBit(             /* Set a single bit signal to its   */
                                    /* active state                     */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    ccsERROR           *error       /* Returned error structure */
    );


ccsCOMPL_STAT ioClearBit(           /* Set a single bit signal to its   */
                                    /* inactive state                   */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioChangeBit(          /* Change the state of a single bit */
                                    /* signal                           */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioPulseBit(           /* Pulse a single bit signal        */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    vltUINT32          duration,    /* Pulse duration (in milliseconds) */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioGetDirectAddress(   /* Get the direct address of signal */
    const dbSYMADDRESS signalName,  /* Name of the signal               */
    ioDIRADDRESS       *address,    /* Direct address of signal         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDWriteDigital(      /* Write value to a digital signal  */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltUINT32          value,       /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDWriteAnalog(       /* Write value to an analog signal  */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltFLOAT           value,       /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDReadDigital(       /* Read value of a digital signal   */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltUINT32          *value,      /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDReadAnalog(        /* Read value of an analog signal   */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltFLOAT           *value,      /* Value                            */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT ioDReadAnalogRaw(     /* Read value of an analog signal   */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltFLOAT           *value,      /* Value                            */
    vltFLOAT           *rawValue,   /* Raw value                        */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDSetBit(            /* Set a single bit signal to its   */
                                    /* active state                     */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDClearBit(          /* Set a single bit signal to its   */
                                    /* inactive state                   */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDChangeBit(         /* Change the state of a single bit */
                                    /* signal */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioDPulseBit(          /* Pulse a single bit signal        */
    const ioDIRADDRESS *address,    /* Direct address of signal         */
    vltUINT32          duration,    /* Pulse duration (in milliseconds) */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT ioStartSampling(      /* Start sampling of signals        */
    ioSCOPE            scope,       /* Scope of logging (all or list    */
                                    /* of signals)                      */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR          *error        /* Returned error structure         */
    );


ccsCOMPL_STAT ioStopSampling(       /* Stop sampling of signals         */
    ioSCOPE            scope,       /* Scope of sampling (all or list   */
                                    /* of signals)                      */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR          *error        /* Returned error structure         */
    );
