#ifndef IRVME_COMMANDS_H
#define IRVME_COMMANDS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: irvmeCommands.h,v 1.7 2005/09/12 08:08:39 vltsccm Exp $" 
*
* irvmeCommands.h  -  Contains literals of all defined commands and
*                     structure types for commands with multiple parameters.
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* jstegmei   14/11/00   created
*/

#include "time.h"
#include "semLib.h"
#include "irvmeDevice.h"       /* Include irvme device definitions */

/*
 * Command definitions
 */
#define irvmeCMD_FREE_DEVICE     201
#define irvmeCMD_RESET_DEVICE    202

#define irvmeCMD_READ_REG        211
#define irvmeCMD_WRITE_REG       212

#define irvmeCMD_ATTACH_INT      221
#define irvmeCMD_DETACH_INT      222
#define irvmeCMD_ACK_INT         223
#define irvmeCMD_INT_ENABLE      224
#define irvmeCMD_INT_DISABLE     225

#define irvmeCMD_CONFIG          231
#define irvmeCMD_STATUS          232
#define irvmeCMD_HWSTATUS        233
#define irvmeCMD_GETCONF         234
#define irvmeCMD_SETHDR          235

#define irvmeCMD_SIMTIME         241
#define irvmeCMD_TEST            242

/*
 * User defined routine to be called at
 * interrupt level
 */
typedef void (irvmeUSR_ISR)(void *buffer, int size, void *userArg);

/*
 * Argument data structures
 */
typedef struct
{
  u_short  size;
  u_short *dummyBuf;
} irvmeTEST;

typedef struct
{
  u_short reg;
  u_short value;
} irvmeREAD_REG;

typedef struct
{
  u_short reg;
  u_short value;
} irvmeWRITE_REG;

typedef struct
{
  SEM_ID         semISR;      /* synchronization semaphore      */
  irvmeUSR_ISR  *userISR;     /* user defined interrupt routine */
  void          *userArg;     /* user argument                  */
  BOOL           trigger;     /* trigger task on interrupt      */
} irvmeCFG_INT;

typedef struct
{ 
  int            pIdx;        /* buffer index for process    */
  u_long         tstat;       /* transfer status             */
  BOOL           overrun;     /* buffer overrun flag         */
  BOOL           error;       /* I/O error flag              */
  BOOL           attached;    /* attached to interrupt       */
} irvmeSTATUS;

typedef struct
{ 
  u_short        fifoCnt;     /* number of words in FIFO      */
  BOOL           fifoFull;    /* fifo full flag               */
  u_short        intStatus;   /* interrupt status             */
  u_short        ctrlReg;     /* control register             */
} irvmeHWSTATUS;

typedef struct
{ 
  u_long         d_size;      /* user buffer size in bytes    */
  u_long         b_size;      /* transfer block size in bytes */
  u_short        fifoWidth;   /* word width of FIFO in bytes  */
  u_short        dmaWidth;    /* word width of DMA in bytes   */
} irvmeGETCONF;

typedef struct
{ 
  void    *data[2];           /* pointer to user memory       */
  u_long   d_size;            /* user buffer size in bytes    */
} irvmeCONFIG;

typedef struct
{ 
  u_char hdr;                 /* device header on GIGA-Bus    */  
} irvmeHDR;

typedef struct
{
  u_long simTime;             /* in microseconds              */
} irvmeSIMTIME;


#endif /* !IRVME_COMMANDS_H */

/* __oOo__ */
