#ifndef IRVME_DEVICE_H
#define IRVME_DEVICE_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: irvmeDevice.h,v 1.7 2005/09/12 08:08:39 vltsccm Exp $" 
*
* irvmeDevice.h  -  Contains the IRVME device specific data structures
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jstegmei   14/11/00  created
*/


/*
 * Actual transfer FIFO size (in bytes)
 */
#define irvmeFIFO_SIZE (512 * 4) 

/*
 * Register structure
 */
typedef	volatile struct
{
  volatile u_short control;         /* control register (write only)  */
  volatile u_short interrupt;       /* interrupt config. (read-write) */
  volatile u_short status;          /* status register (read-only)    */
  volatile u_short revision;        /* revision register (read-only)  */
  volatile u_short reserved[12];
} irvmeREG;

#define irvmeDEV_SIZE (sizeof (irvmeREG))

/*
 * Register offsets
 */
#define irvmeREG_CONTROL          0  /* control register (read-write)  */
#define irvmeREG_INTERRUPT        1  /* interrupt config. (read-write) */
#define irvmeREG_STATUS           2  /* status register (read-only)    */
#define irvmeREG_REVISION         3  /* revision register (read-only)  */

/*
 * Register masks etc.
 */
#define irvmeCTRL_MASK       0x7dff  /* mask valid bits for read       */
#define irvmeCTRL_LEVEL      0x01ff  /* set FIFO-level for interrupt   */
#define irvmeCTRL_RESET      0x8000  /* reset FIFO on board            */
#define irvmeCTRL_ERR        0x4000  /* error interrupt enable         */
#define irvmeCTRL_HDR        0x3c00  /* device header on GIGA bus      */
#define irvmeSTAT_CNT        0x01ff  /* FIFO counter                   */
#define irvmeSTAT_FULL       0x8000  /* FIFO full flag                 */
#define irvmeREV_CODE        0x000f  /* revision code                  */
#define irvmeREV_DAY         0x01f0  /* revision date                  */
#define irvmeREV_MONTH       0x1e00  /* revision date                  */
#define irvmeREV_YEAR        0xe000  /* revision date                  */
#define irvmeINT_NUM         0x00ff  /* interrupt number               */
#define irvmeINT_LEVEL       0x0700  /* interrupt level                */
#define irvmeINT_ENABLE      0x8000  /* enable interrupt generation    */

/*
 * Default device header on GIGA-Bus
 */
#define irvmeHDR_DEFAULT          1

#endif /* !IRVME_DEVICE_H */

/* __oOo__ */
