#ifndef IRVME_DMA_H
#define IRVME_DMA_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: irvmeDma.h,v 1.7 2005/09/12 08:08:39 vltsccm Exp $" 
*
* irvmeDma.h - DMA interface defintions for MVME2604/UNIVERSE
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jstegmei   14/11/00  created
*/

#include "vxWorks.h"
#include "semLib.h"

#if (CPU_FAMILY == PPC)
#  if (CPU == PPC604)
#    include "config.h"
#    include "mv2600.h"
#    include "universe.h"
#    define irvmeDMA_SUPPORTED
#    define irvmeDMA_CHAIN_SUPPORTED
#    define irvmeDMA_DONE     DGCS_DONE   /* completed       */
#    define irvmeDMA_VERR     DGCS_VERR   /* VME-Bus error   */
#    define irvmeDMA_LERR     DGCS_LERR   /* Local-Bus error */
#    define irvmeDMA_PERR     DGCS_P_ERR  /* protocol error  */
#    define irvmeDMA_HALT     DGCS_HALT   /* user halted     */
#    define irvmeDMA_STOP     DGCS_STOP   /* user stoppped   */
#    define irvmeDMA_ACT      DGCS_ACT    /* still active    */
typedef struct
{
  UINT32 DCTL;
  UINT32 DTBC;
  UINT32 DLA;
  UINT32 reserved1;
  UINT32 DVA;
  UINT32 reserved2;
  UINT32 DCPP;
  UINT32 reserved3;
} irvmeDMA_CMD;
#  endif
#endif

#ifndef irvmeDMA_SUPPORTED
#define irvmeDMA_DONE     0x00000000   /* completed       */
#define irvmeDMA_VERR     0x00000001   /* VME-Bus error   */
#define irvmeDMA_LERR     0x00000002   /* Local-Bus error */
#define irvmeDMA_PERR     0x00000004   /* protocol error  */
#define irvmeDMA_HALT     0x00000008   /* user halted     */
#define irvmeDMA_STOP     0x00000010   /* user stoppped   */
#define irvmeDMA_ACT      0x00000020   /* still active    */
#endif

#ifndef irvmeDMA_CHAIN_SUPPORTED
typedef int irvmeDMA_CMD;
#endif

/*
 * DMA direction
 */
#define irvmeDMA_TO_LOCAL    0
#define irvmeDMA_TO_VME      1

/*
 * Adressing modes
 */
#define irvmeDMA_AM_NOCHANGE 0
#define irvmeDMA_AM_DATA     1
#define irvmeDMA_AM_PRGM     2

typedef void (irvmeDMA_USR_ISR)(void *buffer, int size, void *userArg);

#ifdef __cplusplus 
extern "C" {
#endif


/*
 * DMA prototypes
 */
#ifdef irvmeDMA_SUPPORTED
int irvmeDmaInit(int datumSize, int blt, int addrSpace, int addrMode,
		 int direction, int isrParam);
SEM_ID irvmeDmaSemISR(void);
void irvmeDmaDisable(void);
void irvmeDmaRunCtrl(irvmeDMA_USR_ISR *userISR, void *userArg, BOOL trigger,
		     BOOL *busyFlag, BOOL *overrunFlag);
void irvmeDmaRun(long int vmeAddr, long int localAddr, int size, BOOL last);
void irvmeDmaStop(void);
long irvmeDmaByteCount(void);
int  irvmeDmaWait(char *erms);              
long irvmeDmaStatus(void);
long irvmeDmaStatusSaved(void);
#else
#define irvmeDmaInit(ds,b,as,am,d,ip) ERROR
#define irvmeDmaSemISR() NULL
#define irvmeDmaDisable()
#define irvmeDmaRunCtrl(isr,a,t,bf,of)
#define irvmeDmaRun(va,la,s,l)
#define irvmeDmaStop()
#define irvmeDmaByteCount() 0
#define irvmeDmaCount() 0
#define irvmeDmaWait(e) OK
#define irvmeDmaStatus() irvmeDMA_DONE
#define irvmeDmaStatusSaved() irvmeDMA_DONE
#endif

/*
 * Chained DMA prototypes
 */
#ifdef irvmeDMA_CHAIN_SUPPORTED
int  irvmeDmaListInit(int isrParam);
irvmeDMA_CMD *irvmeDmaListAdd(int datumSize, int blt, int addrSpace,
			      int addrMode, int direction, 
			      long int vmeAddr, long int localAddr, int size, 
			      irvmeDMA_CMD *dmaListStart);
int  irvmeDmaListRun(irvmeDMA_CMD *dmaListStart); 
int  irvmeDmaListWait(int dissemble, char *erms);
void irvmeDmaListFree(void);
#else
#define irvmeDmaListInit(ip) OK
#define irvmeDmaListAdd(ds,b,as,am,dr,va,la,s,l) NULL
#define irvmeDmaListRun(l) OK
#define irvmeDmaListWait(d,e) OK
#define irvmeDmaListFree()
#endif

#ifdef __cplusplus
}
#endif

#endif /* !IRVME_DMA_H */

/* __oOo__ */



