/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lccLCC.h,v 4.97.1.1 2005/10/24 13:55:46 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* mkiekebu  13/02/98  lccFmod made public
* bgustafs  15/07/97  added functions lccGetDevTimeout and lccSetDevTimeout
*                     modified definitions of lccDEV_MODE
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software              SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  1.2          18/02/94        P.Rochez        SPR940013 Added devSim status
*                                                 to lccGetStatus()
*                                                 Added lccDevEnterSim() and
*                                                 lccDevExitSim()
*                                               SPR940016.6 Removed error for
*                                                 lccGetId(), lccGetStatus()
*                                                 and lccGetVersion()
*               22/02/94        T.Phan          SPR940019.8
*                                               Added lccFind()
*  1.3          22/06/94        P.Rochez        SPR 940066: added lccBootResult
*  2.1          31/01/95        P.Rochez
*       Added lccDEV_MODE enumerate
*       Added lccGetDeviceState() and lccSetDeviceState() functions
*       Changed to ESO header structure
*------------------------------------------------------------------------------
*  SCCS Id:  lccLCC.h  2.1  03 Feb 1995
*----------------------------------------------------------------------------*/

/*
 * <lccLCC.h> - LCU / LCU MANAGEMENT Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/


/*
 * Header files
 */

#include "ccs.h"
#include "db.h"
#include "msg.h"

/*
 * Constants
 */

#define lccDEVICE_NAME_LEN  19      /* Maximum length of a device name  */
#define lccDEV_OFF      ccsSTATE_OFF      /* Device states              */
#define lccDEV_LOADED   ccsSTATE_LOADED
#define lccDEV_STAND_BY ccsSTATE_STANDBY
#define lccDEV_ON_LINE  ccsSTATE_ONLINE

/*
 * Data types
 */
/*
typedef enum {        */              /* List of device states           */
/*    lccDEV_OFF = 1,     
    lccDEV_LOADED,                         
    lccDEV_STAND_BY,
    lccDEV_ON_LINE
} lccDEV_MODE;
*/
typedef vltINT32 lccDEV_MODE;

typedef enum {                      /* List of LCU operations           */
    lccNONE = 1,                    /* No operation in progress         */
    lccCOLD_START,                  /* COLD START in progress           */
    lccWARM_START,                  /* WARM START in progress           */
    lccSHUTTING_DOWN                /* Node being shut down             */
} lccOPERATION;


typedef  char      lccVERSION [80]; /* LCC version number               */


typedef struct {                    /* Indentification of a node        */
    ccsENVNAME     LCUName;         /* Node name                        */
    vltUINT32      LCUInet;         /* Internet address of LCU node     */
    ccsENVNAME     bootName;        /* Name of boot node                */
    vltUINT32      bootInet;        /* Internet address of boot node    */
} lccID;


typedef char  lccDEVICE_NAME[lccDEVICE_NAME_LEN+1];
                                    /* Name of a device                 */
typedef enum {                      /* List of device status            */
    lccDEVICE_OFF = 1,              /* Device not yet initialised       */
    lccDEVICE_INIT,                 /* Device is initialising           */
    lccDEVICE_READY,                /* Device initialised               */
    lccDEVICE_FAILURE               /* Device returned an error message */
} lccDEVICE_STATUS;


typedef struct {                    /* Device description               */
    lccDEVICE_NAME    name;         /* Device name                      */
    ccsPROCNAME       process;      /* Name of device control process   */
    lccDEVICE_STATUS  status;       /* Status of the device             */
} lccDEVICE;


typedef enum {                      /* List of I/O device access        */
    lccBYTE_ACCESS = 1,             /* 8 bits                           */
    lccWORD_ACCESS,                 /* 16 bits                          */
    lccLONG_WORD_ACCESS             /* 32 bits                          */
} lccDEVICE_ACCESS;

typedef enum {                      /* Input/output configuration       */
    lccBIPOLAR_10V = 1,             /* -10V...+10V                      */
    lccUNIPOLAR_10V,                /*   0V...+10V                      */
    lccBIPOLAR_5V,                  /*  -5V...+5V                       */
    lccUNIPOLAR_5V,                 /*   0V...+5V                       */
    lccCONFIG_NONE                  /* For digital device               */
} lccDEVICE_CONFIG;


/*
 * Functions prototypes
 */

/*
 * General services to get the LCU version, status, id and to set a process
 * id.
 */

ccsCOMPL_STAT lccGetVersion(        /* Get the version of the LCC       */
    lccVERSION          version     /* Version number                   */
    );


ccsCOMPL_STAT lccGetStatus(         /* Get the status of a LCU          */
    lccMODE             *mode,      /* Current LCU mode                 */
    lccOPERATION        *operation, /* Current operation being performed*/
    vltLOGICAL          *initPhase, /* True = initialisation phase      */
    vltLOGICAL          *simulation,/* True = simulation mode           */
    vltLOGICAL          *devSim,    /* True = one or more device in     */
                                    /* simulation                       */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccGetId(             /* Get the id of the LCU            */
    lccID               *id         /* LCU id                           */
    );



/*
 * Configuration of the simulation mode
 */

ccsCOMPL_STAT lccEnSim(             /* Set the LCU in simulation mode   */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccDisSim(            /* Disable the LCU simulation mode  */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


/*
 * Management of the LCU
 */

ccsCOMPL_STAT lccColdStart(         /* Perform a Cold Start of the LCU  */
    lccMODE             endMode,    /* End mode: STAND-BY, STAND-ALONE  */
                                    /* or ON-LINE                       */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccWarmStart(         /* Perform a Warm-start of the LCU  */
    lccMODE             endMode,    /* End mode: STAND-BY, STAND-ALONE  */
                                    /* or ON-LINE                       */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccStandAlone(        /* Set the LCU in STAND-ALONE mode  */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccOnLine(            /* Set the LCU in ON-LINE mode      */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccStandBy(           /* Set the LCU in STAND-BY mode     */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccReboot(            /* Reboot the LCU                   */
    vltLOGICAL          sysRes,     /* True = generate a SysRes on the  */
                                    /* VME bus                          */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccShutdown(          /* Perform a shutdown of the LCU    */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccInitFull(          /* Initialise the devices           */
                                    /* configured for the LCU           */
    lccMODE             endMode,    /* End mode: STAND-BY, STAND-ALONE  */
                                    /* or ON-LINE                       */
    vltLOGICAL          conditional,/* Conditional flag                 */
    vltLOGICAL          continueOnError,  
                                    /* True = continue, False = Stop    */
                                    /* when an error is detected        */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccWaitFullInit(      /* Wait until all devices have      */
                                    /* initialised                      */
    vltUINT32           timeOut,    /* Time-out (in seconds)            */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccWaitFirstInit(     /* Wait until the next device has   */
                                    /* initialised                      */
    vltUINT32           timeOut,    /* Time-out (in seconds)            */
    lccDEVICE_NAME      device,     /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccReload(            /* Stop the LCU and reload given    */
                                    /* software modules                 */
    const char          *fileName,  /* File containing all modules to load */
    vltUINT32           *nbDevices, /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccReconfigure(       /* Stop the LCU and configure the   */
                                    /* database and the software        */
    dbCONFIG_MODE       mode,       /* File or buffer                   */
    const char          *buffer,    /* Data buffer or file name         */
    vltUINT32           size,       /* Buffer size                      */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccStop(              /* Stop the current LCU management  */
                                    /* command being performed (except  */
                                    /* reboot)                          */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccGetDeviceList(     /* Get a list of the devices        */
                                    /* configured in the LCU            */
    vltUINT32           *number,    /* Number of devices                */
    lccDEVICE           **list,     /* List of devices                  */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccDevEnterSim(       /* Used by devices to notify that   */
                                    /* they enter simulation            */
    const lccDEVICE_NAME device,    /* Name of device in simulation     */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccDevExitSim(        /* Used by devices to notify that   */
                                    /* they exit simulation             */
    const lccDEVICE_NAME devName,   /* Name of device in simulation     */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccSetDeviceState(    /* Set the state of a device        */
    const lccDEVICE_NAME devName,   /* Name of device                   */
    lccDEV_MODE          state,     /* Device state                     */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccGetDeviceState(    /* Get the state of a device        */
    const lccDEVICE_NAME devName,   /* Name of device                   */
    lccDEV_MODE         *state,     /* Device state                     */
    vltLOGICAL          *simulation,/* True = device in simulation      */
    ccsERROR            *error      /* Returned error structure         */
    );

ccsCOMPL_STAT lccSetDevTimeout(     /* Set the device timeout           */
    vltUINT32         timeout,      /* Timeout in seconds               */
    ccsERROR          *error        /* Returned error structure         */
    );

ccsCOMPL_STAT lccGetDevTimeout(     /* Get the device timeout           */
    vltUINT32         *timeout,     /* Timeout in seconds               */
    ccsERROR          *error        /* Returned error structure         */
    );

ccsCOMPL_STAT lccFind(
    char                *fileName,  /* file name to search              */
    char                *fullPath,  /* returned full path file name     */
    ccsERROR            *error      /* Returned error structure         */
    );

/*
 * Self-test
 */

ccsCOMPL_STAT lccSelfTest(          /* Start LCU self-test procedure    */
    vltUINT32           *number,    /* Number of failed devices         */
    lccDEVICE_NAME      **device,   /* Device name                      */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccCheckConf(         /* Check the hardware configuration */
    vltUINT32           *number,    /* Number of devices                */
    lccDEVICE_NAME      **list,     /* List of devices                  */
    ccsERROR            *error      /* Returned error structure         */
    );


ccsCOMPL_STAT lccRegisterDevice(    /* Register an I/O device           */
    const lccDEVICE_NAME name,      /* Device name                      */
    void                *probe,     /* Probe address                    */
    lccDEVICE_ACCESS    access,     /* Access mode                      */
    lccDEVICE_CONFIG    configIn,   /* Input config                     */
    lccDEVICE_CONFIG    configOut,  /* Output config                    */
    ccsERROR            *error      /* Returned error structure         */
    );


/*
 * Software watchdog
 */

ccsCOMPL_STAT lccStartWatchdog(     /* Start the software watchdog      */
    vltUINT32          eventTimeOut,/* Time-out before sending an       */
                                    /* abnormal event (in seconds)      */
    vltUINT32          rebootTimeOut,
                                    /* Time-out before rebooting LCU    */
                                    /* (in seconds)                     */
    const dbSYMADDRESS event,       /* Event symbolic address in DB     */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT lccTriggerWatchdog(   /* Trigger the software watchdog    */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT lccStopWatchdog(      /* Disables the software watchdog   */
    ccsERROR           *error       /* Returned error structure         */
    );
   

void lccBootResult( void );         /* Display a message indicating LCC */
                                    /* boot result (and start-up error  */
                                    /* buffer) on the system console    */
/*
 * LCC Math functions
 */

double lccFmod (double x, double y);
