/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: lccdevCmd.h,v 1.18 2003/12/05 12:07:46 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutzer 02/09/95  Created by eccsgen V0.45 (September 1995)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	lccdevCmd - LCC Software Device controlling a hardware device
*
*   SYNOPSIS
*	#include "lccdevCmd.h"
*
*	ccsCOMPL_STAT lccdevCmdNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdAvail(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdUnavail(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdDisable(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	access,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdEnable(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	access,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdLock(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdUnlock(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdHandset(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdHsetOff(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdFixPos(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    IN    vltLOGICAL	reset,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdMove(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdMoveRel(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdSet(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	position[],	** max. vltBYTES32[999]
*	    IN    vltINT16	positionNum,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdCheck(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    OUT   char**	status,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdWait(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	timeout,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdUsrSetMode(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	cmd,
*	    IN    vltLOGICAL	available,
*	    IN    vltLOGICAL	enabled,
*	    IN    vltLOGICAL	simulation,
*	    IN    vltLOGICAL	handset,
*	    IN    vltLOGICAL	locked,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lccdevCmdErr<error>(...)
*
*   PARENT CLASS
*	public lccdevSTD
*
*   DESCRIPTION
*       The class lccdevCMD is derived from class lccdevSTD. It
*       implements the standard commands and state machine of a
*       LCC Software Device that controls a hardware device.
*
*       The class implements the following new commands:
*
*       .  DISABLE ENABLE LOCK UNLOCK HANDSET HSETOFF
*       .  FIXPOS MOVE MOVEREL SET CHECK WAIT
*
*       By default, no actions are performed when the software
*       device changes mode (available, enabled, locked, handset,
*       simulation and fixed). Classes derived from lccdevCMD
*       may override the following functions:
*       - Public methods: Move, MoveRel, Set, Check and Wait.
*       - All protected methods described below.
*       - All protected methods of the parent class, except
*	  lccdevCmdUsrSimulat() and lccdevCmdUsrStopSim()
*	  (overload function UsrSetMode() instead).
*
*   PUBLIC METHODS
*	lccdevCmdNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type lccdevCMD.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	lccdevCmdDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type lccdevCMD
*
*	lccdevCmdAvail(obj,devName,error)
*	   Marks a hardware device as available (default state).
*
*	lccdevCmdUnavail(obj,devName,error)
*	   Marks a hardware device as unavailable.
*
*	lccdevCmdDisable(obj,devName,access,error)
*	   Disables the access to the hardware device.
*	   This command can be used if the hardware device
*	   is not available.
*
*	lccdevCmdEnable(obj,devName,access,error)
*	   Enables the access to the hardware device
*	   (default state).
*
*	lccdevCmdLock(obj,devName,error)
*	   Places the hardware device in locked mode
*	   (no movements allowed).
*
*	lccdevCmdUnlock(obj,devName,error)
*	   Disables locked mode of the hardware device.
*
*	lccdevCmdHandset(obj,devName,error)
*	   Places the hardware device in Handset mode.
*
*	lccdevCmdHsetOff(obj,devName,error)
*	   Disables Handset mode of the hardware device
*	   (returns to Normal or Locked mode).
*
*	lccdevCmdFixPos(obj,devName,position,positionNum,reset,error)
*	   Command FIXPOS sets a (possibly disabled) device
*	   in simulation and locks the device in the specified
*	   position. The device is returned to its previous
*	   mode of operation by issueing a FIXPOS -reset
*	   command.
*
*	lccdevCmdMove(obj,devName,position,positionNum,error)
*	   Move the hardware device to a given absolute position.
*
*	lccdevCmdMoveRel(obj,devName,position,positionNum,error)
*	   Move the hardware device to a position relative to
*	   the current position.
*
*	lccdevCmdSet(obj,devName,position,positionNum,error)
*	   Set a hardware device in a specified state
*	   (e.g. set lamp on, set shutter open).
*
*	lccdevCmdCheck(obj,devName,status,error)
*	   Checks if the last action of the hardware device
*	   has finished.
*
*	lccdevCmdWait(obj,devName,timeout,error)
*	   Wait for the last action of the hardware device
*	   to finish.
*
*   PUBLIC DATA MEMBERS
*	The following data members are defined in lccdevCmd.class:
*
*	vltINT32	available (readonly)
*	vltINT32	enabled (readonly)
*	vltINT32	locked (readonly)
*	vltINT32	handset (readonly)
*
*   PROTECTED METHODS
*	lccdevCmdUsrSetMode(obj,devName,cmd,available,enabled,simulation,
*			    handset,locked,error)
*	   This function should be overloaded to perform the
*	   actions necessary to change the mode of the hardware
*	   device. The function is called by the commands:
*	   DISABLE, ENABLE, LOCK, UNLOCK, HANDSET, HSETOFF,
*	   SIMULAT and STOPSIM.
*
*	   cmd		<IN>	code indicating the command that
*		 		calls UsrSetMode() (lccdevCMD_*).
*	   <logical>	<IN>	state of the available, enabled,
*			        simulation, handset and locked modes.
*
*	lccdevCmdUsrStatusCode(obj,devName,text,flags,code,error)
*	   This function overloads lccdevStdUsrStatusCode().
*	   See lccdevStd(3) for details.
*
*	   text	 <IN>	Name of the status value.
*	   flags <IN>	expected status type (lcctooSTATUS_*).
*	   code	 <OUT>	Code of the status value (lccdevGET_*).
*
*	   lccdevGET_DEVICE        "Device"
*	   lccdevGET_STATE         "State"
*	   lccdevGET_TRANSTATE     "TranState"
*	   lccdevGET_ERROR         "Error"
*	   lccdevGET_INITIALISED   "Initialised"
*	   lccdevGET_SIMULATION    "Simulation"
*	   lccdevGET_AVAILABLE     "Available"
*	   lccdevGET_ENABLED       "Enabled"
*	   lccdevGET_LOCKED        "Locked"
*	   lccdevGET_HANDSET       "Handset"
*
*	lccdevCmdUsrStatusLast(obj,devName,last,error)
*	   This function overloads lccdevStdUsrStatusLast().
*	   See lccdevStd(3) for details.
*
*	lccdevCmdUsrStatusValue(obj,devName,code,value,error)
*	   This function overloads lccdevStdUsrStatusValue().
*	   See lccdevStd(3) for details.
*
*	lccdevCmdUsrLoaded2Standby(obj,devName,error)
*	   This function should be overloaded to perform the
*	   actions necessary to change from the LOADED to the
*	   STANDBY state. The overloaded function should call
*	   lccdevCmdUsrLoaded2Standby() (to set the fixed
*	   position, if necessary).
*
*	lccdevCmdUsrStandby2Online(obj,devName,error)
*	   This function overloads lccdevStdUsrStandby2Online().
*	   It returns an error if the device is not available or
*	   if the device is disabled. See lccdevStd(3) for more 
*	   details.
*
*	lccdevCmdErr<error>(...)
*	    The macros lccdevCmdErr*() log an error with lcctooLOG_0 and
*	    call errAdd with the corresponding error parameters.
*
*	lccdevCmdRead<attr>(...)
*	lccdevCmdWrite<attr>(...)
*	    The above macros read from/write to the data attributes
*	    of the class.
*
*   PRIVATE METHODS
*	lccdevCmdUsrSimulat(obj,devName,error)
*	   This function calls UsrSetMode() to enter simulation mode.
*
*	lccdevCmdUsrStopSim(obj,devName,error)
*	   This function calls UsrSetMode() to enter simulation mode.
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	The names of the defined errors begin as follows:
*	lccdevERR_<error>
*
*   SEE ALSO
*	lccdevStd(3) auto(3) lcctoo(3) egen(3)
*
*-------------------------------------------------------------------------
*/

#ifndef LCCDEV_CMD_H
#define LCCDEV_CMD_H

#include "egen.h"
#include "lccdevDefines.h"
#include "lccdevErrors.h"
#include "lccdevStd.h"

/*
 * class type identifier
 */
#define lccdevCMD_ID	117

/*
 * attribute table
 */
#define lccdevCMD_ATTR_STATE	0
#define lccdevCMD_ATTR_SUBSTATE	1
#define lccdevCMD_ATTR_ERROR	2
#define lccdevCMD_ATTR_INITIALISED	3
#define lccdevCMD_ATTR_SIMULATION	4
#define lccdevCMD_ATTR_DEVICE	5
#define lccdevCMD_ATTR_AVAILABLE	6
#define lccdevCMD_ATTR_ENABLED	7
#define lccdevCMD_ATTR_LOCKED	8
#define lccdevCMD_ATTR_HANDSET	9
#define lccdevCMD_ATTR_FIXED	10
#define lccdevCMD_ATTR_FIX_POS_SET	11
#define lccdevCMD_ATTR_FIXED_POSITION	12
#define lccdevCMD_ATTR_FIXED_SIMULATION	13
#define lccdevCMD_ATTR_FIXED_LOCKED	14

#define lccdevCMD_AUTOMATA	0

/*
 * function table
 */
#define lccdevCMD_FCT_DELETE_OBJ	0
#define lccdevCMD_FCT_STANDBY	1
#define lccdevCMD_FCT_ONLINE	2
#define lccdevCMD_FCT_OFF	3
#define lccdevCMD_FCT_EXIT	4
#define lccdevCMD_FCT_STATE	5
#define lccdevCMD_FCT_SIMULAT	6
#define lccdevCMD_FCT_STOP_SIM	7
#define lccdevCMD_FCT_STATUS	8
#define lccdevCMD_FCT_INIT	9
#define lccdevCMD_FCT_STOP	10
#define lccdevCMD_FCT_TEST	11
#define lccdevCMD_FCT_SELFTST	12
#define lccdevCMD_FCT_AUTO_TBL	13
#define lccdevCMD_FCT_AUTO_RST	14
#define lccdevCMD_FCT_INIT_OBJ	15
#define lccdevCMD_FCT_EXIT_OBJ	16
#define lccdevCMD_FCT_LOADED2STBY	17
#define lccdevCMD_FCT_ONLINE2STBY	18
#define lccdevCMD_FCT_EXIT1	19
#define lccdevCMD_FCT_CHECK_DEVICE_NAME	20
#define lccdevCMD_FCT_BREAK	21
#define lccdevCMD_FCT_USR_INIT	22
#define lccdevCMD_FCT_USR_LOADED2STANDBY	23
#define lccdevCMD_FCT_USR_STANDBY2ONLINE	24
#define lccdevCMD_FCT_USR_ONLINE2STANDBY	25
#define lccdevCMD_FCT_USR_STANDBY2LOADED	26
#define lccdevCMD_FCT_USR_STOP	27
#define lccdevCMD_FCT_USR_SIMULAT	28
#define lccdevCMD_FCT_USR_STOP_SIM	29
#define lccdevCMD_FCT_USR_STATUS_CODE	30
#define lccdevCMD_FCT_USR_STATUS_LAST	31
#define lccdevCMD_FCT_USR_STATUS_VALUE	32
#define lccdevCMD_FCT_USR_TEST	33
#define lccdevCMD_FCT_USR_SELF_TEST	34
#define lccdevCMD_FCT_USR_ERROR	35
#define lccdevCMD_FCT_AVAIL	36
#define lccdevCMD_FCT_UNAVAIL	37
#define lccdevCMD_FCT_DISABLE	38
#define lccdevCMD_FCT_ENABLE	39
#define lccdevCMD_FCT_LOCK	40
#define lccdevCMD_FCT_UNLOCK	41
#define lccdevCMD_FCT_HANDSET	42
#define lccdevCMD_FCT_HSET_OFF	43
#define lccdevCMD_FCT_FIX_POS	44
#define lccdevCMD_FCT_MOVE	45
#define lccdevCMD_FCT_MOVE_REL	46
#define lccdevCMD_FCT_SET	47
#define lccdevCMD_FCT_CHECK	48
#define lccdevCMD_FCT_WAIT	49
#define lccdevCMD_FCT_USR_SET_MODE	50

/*
 * automata states
 */
#define lccdevCMD_STATE_NOSTATE	-1
#define lccdevCMD_STATE_OFF	0
#define lccdevCMD_STATE_LOADED	1
#define lccdevCMD_STATE_STANDBY	2
#define lccdevCMD_STATE_ONLINE	3

/*
 * automata substates
 */
#define lccdevCMD_SUBSTATE_ERROR	0
#define lccdevCMD_SUBSTATE_IDLE	1
#define lccdevCMD_SUBSTATE_BUSY	2
#define lccdevCMD_SUBSTATE_INITIALISING	3
#define lccdevCMD_SUBSTATE_SHUTTING_DOWN	4
#define lccdevCMD_SUBSTATE_STOPPING	5
#define lccdevCMD_SUBSTATE_COLD_STARTING	6
#define lccdevCMD_SUBSTATE_STARTING	7
#define lccdevCMD_SUBSTATE_STOP	8

/*
 * automata events
 */
#define lccdevCMD_EVENT_INIT	0
#define lccdevCMD_EVENT_STANDBY	1
#define lccdevCMD_EVENT_ONLINE	2
#define lccdevCMD_EVENT_STOP	3
#define lccdevCMD_EVENT_OFF	4
#define lccdevCMD_EVENT_EXIT	5
#define lccdevCMD_EVENT_STATE	6
#define lccdevCMD_EVENT_STOP_SIM	7
#define lccdevCMD_EVENT_STATUS	8
#define lccdevCMD_EVENT_SIMULAT	9
#define lccdevCMD_EVENT_TEST	10
#define lccdevCMD_EVENT_SELFTST	11
#define lccdevCMD_EVENT_AUTO_TBL	12
#define lccdevCMD_EVENT_AUTO_RST	13
#define lccdevCMD_EVENT_DISABLE	14
#define lccdevCMD_EVENT_ENABLE	15
#define lccdevCMD_EVENT_LOCK	16
#define lccdevCMD_EVENT_UNLOCK	17
#define lccdevCMD_EVENT_HANDSET	18
#define lccdevCMD_EVENT_HSET_OFF	19
#define lccdevCMD_EVENT_AVAIL	20
#define lccdevCMD_EVENT_UNAVAIL	21
#define lccdevCMD_EVENT_MOVE	22
#define lccdevCMD_EVENT_MOVE_REL	23
#define lccdevCMD_EVENT_SET	24
#define lccdevCMD_EVENT_CHECK	25
#define lccdevCMD_EVENT_WAIT	26


/*
 * function pointer types
 */
typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_AVAIL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_UNAVAIL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_DISABLE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_ENABLE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_LOCK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_UNLOCK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_HANDSET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_HSET_OFF)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_FIX_POS)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    IN    vltLOGICAL	reset,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_MOVE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_MOVE_REL)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_SET)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_CHECK)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	status,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_WAIT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_SET_MODE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	cmd,
    IN    vltLOGICAL	available,
    IN    vltLOGICAL	enabled,
    IN    vltLOGICAL	simulation,
    IN    vltLOGICAL	handset,
    IN    vltLOGICAL	locked,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_STATUS_CODE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	text,
    IN    vltINT32	flags,
    OUT   vltINT32*	code,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_STATUS_VALUE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	code,
    OUT   lcctooVALUE*	value,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_STATUS_LAST)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   vltINT16*	last,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_SIMULAT)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_STOP_SIM)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_LOADED2STANDBY)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

typedef ccsCOMPL_STAT (*lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);


/*
 * log function
 */
#define lccdevCmdLogData(obj, format, arg...) \
	{ \
	char tmpLogBuf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(tmpLogBuf, (format), ##arg); \
	logData("lccdev", (obj)->id, tmpLogBuf); \
	}


/*
 * error functions
 */
#define lccdevErrReadAvailable(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_AVAILABLE: Device %s: Unable to read from attr. available.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_AVAILABLE,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteAvailable(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_AVAILABLE: Device %s: Unable to write to attr. available (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_AVAILABLE,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadEnabled(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_ENABLED: Device %s: Unable to read from attr. enabled.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_ENABLED,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteEnabled(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_ENABLED: Device %s: Unable to write to attr. enabled (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_ENABLED,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadFixed(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_FIXED: Device %s: Unable to read from attr. fixed.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_FIXED,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteFixed(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_FIXED: Device %s: Unable to write to attr. fixed (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_FIXED,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadFixedLocked(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_FIXED_LOCKED: Device %s: Unable to read from attr. fixedLocked.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_FIXED_LOCKED,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteFixedLocked(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_FIXED_LOCKED: Device %s: Unable to write to attr. fixedLocked (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_FIXED_LOCKED,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadFixedPosition(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_FIXED_POSITION: Device %s: Unable to read from attr. fixedPosition.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_FIXED_POSITION,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteFixedPosition(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_FIXED_POSITION: Device %s: Unable to write to attr. fixedPosition (value %s).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_FIXED_POSITION,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define lccdevErrLoadFixedPosition(p1) \
	{ \
	lcctooLOG_0("lccdevERR_LOAD_FIXED_POSITION: Device %s: The fixed position is too long, unable to load it.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_LOAD_FIXED_POSITION,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrSaveFixedPosition(p1) \
	{ \
	lcctooLOG_0("lccdevERR_SAVE_FIXED_POSITION: Device %s: The fixed position is too long, unable to save it.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_SAVE_FIXED_POSITION,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrReadFixedSimulation(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_FIXED_SIMULATION: Device %s: Unable to read from attr. fixedSimulation.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_FIXED_SIMULATION,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteFixedSimulation(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_FIXED_SIMULATION: Device %s: Unable to write to attr. fixedSimulation (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_FIXED_SIMULATION,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadFixPosSet(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_FIX_POS_SET: Device %s: Unable to read from attr. fixPosSet.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_FIX_POS_SET,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteFixPosSet(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_FIX_POS_SET: Device %s: Unable to write to attr. fixPosSet (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_FIX_POS_SET,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadLocked(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_LOCKED: Device %s: Unable to read from attr. locked.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_LOCKED,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteLocked(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_LOCKED: Device %s: Unable to write to attr. locked (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_LOCKED,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrReadHandset(p1) \
	{ \
	lcctooLOG_0("lccdevERR_READ_HANDSET: Device %s: Unable to read from attr. handset.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_READ_HANDSET,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWriteHandset(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRITE_HANDSET: Device %s: Unable to write to attr. handset (value %d).",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRITE_HANDSET,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lccdevErrInFixedPos(p1) \
	{ \
	lcctooLOG_0("lccdevERR_IN_FIXED_POS: Device %s: Device is in a fixed position. Command rejected.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_IN_FIXED_POS,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrNoFixedPosition(p1) \
	{ \
	lcctooLOG_0("lccdevERR_NO_FIXED_POSITION: Device %s: Position argument is missing in command FIXPOS.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_NO_FIXED_POSITION,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrUsrNotAvailable(p1) \
	{ \
	lcctooLOG_0("lccdevERR_USR_NOT_AVAILABLE: Device %s: Device not available.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_USR_NOT_AVAILABLE,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrUsrNotEnabled(p1) \
	{ \
	lcctooLOG_0("lccdevERR_USR_NOT_ENABLED: Device %s: Device not enabled.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_USR_NOT_ENABLED,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrUsrHandset(p1) \
	{ \
	lcctooLOG_0("lccdevERR_USR_HANDSET: Device %s: Device in handset mode.",(p1)); \
	errAdd(error,"lccdev",lccdevERR_USR_HANDSET,__FILE_LINE__,"%s",(p1)); \
	}
#define lccdevErrWrongAccessType(p1,p2) \
	{ \
	lcctooLOG_0("lccdevERR_WRONG_ACCESS_TYPE: Device %s: Access type %s not supported by this command.",(p1),(p2)); \
	errAdd(error,"lccdev",lccdevERR_WRONG_ACCESS_TYPE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}

/*
 * get object owner
 */
#define lccdevCmdGetOwner(obj) ((obj)->owner)


/*
 * attribute access functions
 */
#define lccdevCmdReadState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_STATE,(buf),(error))
#define lccdevCmdWriteState(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_STATE,(void*) (buf),(error))
#define lccdevCmdReadSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_SUBSTATE,(buf),(error))
#define lccdevCmdWriteSubstate(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_SUBSTATE,(void*) (buf),(error))
#define lccdevCmdReadError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_ERROR,(buf),(error))
#define lccdevCmdWriteError(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_ERROR,(void*) (buf),(error))
#define lccdevCmdReadInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_INITIALISED,(buf),(error))
#define lccdevCmdWriteInitialised(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_INITIALISED,(void*) (buf),(error))
#define lccdevCmdReadSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_SIMULATION,(buf),(error))
#define lccdevCmdWriteSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_SIMULATION,(void*) (buf),(error))
#define lccdevCmdReadDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_DEVICE,(buf),(error))
#define lccdevCmdWriteDevice(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_DEVICE,(void*) (buf),(error))
#define lccdevCmdReadAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_AVAILABLE,(buf),(error))
#define lccdevCmdWriteAvailable(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_AVAILABLE,(void*) (buf),(error))
#define lccdevCmdReadEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_ENABLED,(buf),(error))
#define lccdevCmdWriteEnabled(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_ENABLED,(void*) (buf),(error))
#define lccdevCmdReadLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_LOCKED,(buf),(error))
#define lccdevCmdWriteLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_LOCKED,(void*) (buf),(error))
#define lccdevCmdReadHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_HANDSET,(buf),(error))
#define lccdevCmdWriteHandset(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_HANDSET,(void*) (buf),(error))
#define lccdevCmdReadFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_FIXED,(buf),(error))
#define lccdevCmdWriteFixed(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_FIXED,(void*) (buf),(error))
#define lccdevCmdReadFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_FIX_POS_SET,(buf),(error))
#define lccdevCmdWriteFixPosSet(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_FIX_POS_SET,(void*) (buf),(error))
#define lccdevCmdReadFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_FIXED_POSITION,(buf),(error))
#define lccdevCmdWriteFixedPosition(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_FIXED_POSITION,(void*) (buf),(error))
#define lccdevCmdReadFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_FIXED_SIMULATION,(buf),(error))
#define lccdevCmdWriteFixedSimulation(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_FIXED_SIMULATION,(void*) (buf),(error))
#define lccdevCmdReadFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,0,lccdevCMD_ATTR_FIXED_LOCKED,(buf),(error))
#define lccdevCmdWriteFixedLocked(obj,buf,error) \
	caiRWAttribute((obj)->attrs,1,lccdevCMD_ATTR_FIXED_LOCKED,(void*) (buf),(error))
#define lccdevCmdGetAuto(obj,attrPtr,error) \
	SUCCESS; \
	memcpy((attrPtr), &(obj)->objs[lccdevCMD_AUTOMATA], sizeof(egenOBJ)); \
	(attrPtr)->ldata = (obj)->ldata;


/*
 * class functions
 */
#define lccdevCmdStandby(obj,devName,error) \
	lccdevStdStandby((obj),(devName),(error))
#define lccdevCmdOnline(obj,devName,error) \
	lccdevStdOnline((obj),(devName),(error))
#define lccdevCmdOff(obj,devName,error) \
	lccdevStdOff((obj),(devName),(error))
#define lccdevCmdExit(obj,devName,error) \
	lccdevStdExit((obj),(devName),(error))
#define lccdevCmdState(obj,devName,state,error) \
	lccdevStdState((obj),(devName),(state),(error))
#define lccdevCmdSimulat(obj,devName,error) \
	lccdevStdSimulat((obj),(devName),(error))
#define lccdevCmdStopSim(obj,devName,error) \
	lccdevStdStopSim((obj),(devName),(error))
#define lccdevCmdStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	lccdevStdStatus((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define lccdevCmdInit(obj,devName,error) \
	lccdevStdInit((obj),(devName),(error))
#define lccdevCmdStop(obj,devName,error) \
	lccdevStdStop((obj),(devName),(error))
#define lccdevCmdTest(obj,devName,error) \
	lccdevStdTest((obj),(devName),(error))
#define lccdevCmdSelftst(obj,devName,error) \
	lccdevStdSelftst((obj),(devName),(error))
#define lccdevCmdAutoTbl(obj,devName,table,text,error) \
	lccdevStdAutoTbl((obj),(devName),(table),(text),(error))
#define lccdevCmdAutoRst(obj,devName,error) \
	lccdevStdAutoRst((obj),(devName),(error))
#define lccdevCmdInitObj(obj,error) \
	lccdevStdInitObj((obj),(error))
#define lccdevCmdExitObj(obj,error) \
	lccdevStdExitObj((obj),(error))
#define lccdevCmdLoaded2Stby(obj,devName,error) \
	lccdevStdLoaded2Stby((obj),(devName),(error))
#define lccdevCmdOnline2Stby(obj,devName,error) \
	lccdevStdOnline2Stby((obj),(devName),(error))
#define lccdevCmdExit1(obj,devName,error) \
	lccdevStdExit1((obj),(devName),(error))
#define lccdevCmdCheckDeviceName(obj,devName,error) \
	lccdevStdCheckDeviceName((obj),(devName),(error))
#define lccdevCmdBreak(obj,error) \
	lccdevStdBreak((obj),(error))
#define lccdevCmdUsrInit(obj,devName,error) \
	lccdevStdUsrInit((obj),(devName),(error))
#define lccdevCmdUsrOnline2Standby(obj,devName,error) \
	lccdevStdUsrOnline2Standby((obj),(devName),(error))
#define lccdevCmdUsrStandby2Loaded(obj,devName,error) \
	lccdevStdUsrStandby2Loaded((obj),(devName),(error))
#define lccdevCmdUsrStop(obj,devName,req,error) \
	lccdevStdUsrStop((obj),(devName),(req),(error))
#define lccdevCmdUsrTest(obj,devName,error) \
	lccdevStdUsrTest((obj),(devName),(error))
#define lccdevCmdUsrSelfTest(obj,devName,error) \
	lccdevStdUsrSelfTest((obj),(devName),(error))
#define lccdevCmdUsrError(obj,devName,error) \
	lccdevStdUsrError((obj),(devName),(error))

ccsCOMPL_STAT lccdevCmdAvail(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUnavail(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdDisable(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdEnable(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	access,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdLock(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUnlock(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdHandset(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdHsetOff(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdFixPos(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    IN    vltLOGICAL	reset,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdMove(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdMoveRel(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdSet(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	position[],	/* max. vltBYTES32[999] */
    IN    vltINT16	positionNum,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdCheck(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   char**	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdWait(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	timeout,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrSetMode(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	cmd,
    IN    vltLOGICAL	available,
    IN    vltLOGICAL	enabled,
    IN    vltLOGICAL	simulation,
    IN    vltLOGICAL	handset,
    IN    vltLOGICAL	locked,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrStatusCode(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	text,
    IN    vltINT32	flags,
    OUT   vltINT32*	code,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrStatusValue(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	code,
    OUT   lcctooVALUE*	value,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrStatusLast(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   vltINT16*	last,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrSimulat(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrStopSim(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrLoaded2Standby(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdUsrStandby2Online(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    OUT   ccsERROR*	error);


/*
 * virtual functions
 */
#define lccdevCmdVStandby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STANDBY)((obj)->fcts[lccdevCMD_FCT_STANDBY]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVOnline(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE)((obj)->fcts[lccdevCMD_FCT_ONLINE]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVOff(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_OFF)((obj)->fcts[lccdevCMD_FCT_OFF]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVExit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT)((obj)->fcts[lccdevCMD_FCT_EXIT]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVState(obj,devName,state,error) \
	(*((lccdevSTD_FCTPTR_STATE)((obj)->fcts[lccdevCMD_FCT_STATE]))) \
	 ((obj),(devName),(state),(error))
#define lccdevCmdVSimulat(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SIMULAT)((obj)->fcts[lccdevCMD_FCT_SIMULAT]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVStopSim(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP_SIM)((obj)->fcts[lccdevCMD_FCT_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVStatus(obj,devName,function,functionNum,flag,header,conflog,opslog,condlog,status,error) \
	(*((lccdevSTD_FCTPTR_STATUS)((obj)->fcts[lccdevCMD_FCT_STATUS]))) \
	 ((obj),(devName),(function),(functionNum),(flag),(header),(conflog),(opslog),(condlog),(status),(error))
#define lccdevCmdVInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_INIT)((obj)->fcts[lccdevCMD_FCT_INIT]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVStop(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_STOP)((obj)->fcts[lccdevCMD_FCT_STOP]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_TEST)((obj)->fcts[lccdevCMD_FCT_TEST]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVSelftst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_SELFTST)((obj)->fcts[lccdevCMD_FCT_SELFTST]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVAutoTbl(obj,devName,table,text,error) \
	(*((lccdevSTD_FCTPTR_AUTO_TBL)((obj)->fcts[lccdevCMD_FCT_AUTO_TBL]))) \
	 ((obj),(devName),(table),(text),(error))
#define lccdevCmdVAutoRst(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_AUTO_RST)((obj)->fcts[lccdevCMD_FCT_AUTO_RST]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVInitObj(obj,error) \
	(*((lccdevSTD_FCTPTR_INIT_OBJ)((obj)->fcts[lccdevCMD_FCT_INIT_OBJ]))) \
	 ((obj),(error))
#define lccdevCmdVExitObj(obj,error) \
	(*((lccdevSTD_FCTPTR_EXIT_OBJ)((obj)->fcts[lccdevCMD_FCT_EXIT_OBJ]))) \
	 ((obj),(error))
#define lccdevCmdVLoaded2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_LOADED2STBY)((obj)->fcts[lccdevCMD_FCT_LOADED2STBY]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVOnline2Stby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_ONLINE2STBY)((obj)->fcts[lccdevCMD_FCT_ONLINE2STBY]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVExit1(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_EXIT1)((obj)->fcts[lccdevCMD_FCT_EXIT1]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVCheckDeviceName(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_CHECK_DEVICE_NAME)((obj)->fcts[lccdevCMD_FCT_CHECK_DEVICE_NAME]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVBreak(obj,error) \
	(*((lccdevSTD_FCTPTR_BREAK)((obj)->fcts[lccdevCMD_FCT_BREAK]))) \
	 ((obj),(error))
#define lccdevCmdVUsrInit(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_INIT)((obj)->fcts[lccdevCMD_FCT_USR_INIT]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrLoaded2Standby(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_LOADED2STANDBY)((obj)->fcts[lccdevCMD_FCT_USR_LOADED2STANDBY]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrStandby2Online(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STANDBY2ONLINE)((obj)->fcts[lccdevCMD_FCT_USR_STANDBY2ONLINE]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrOnline2Standby(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ONLINE2STANDBY)((obj)->fcts[lccdevCMD_FCT_USR_ONLINE2STANDBY]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrStandby2Loaded(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_STANDBY2LOADED)((obj)->fcts[lccdevCMD_FCT_USR_STANDBY2LOADED]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrStop(obj,devName,req,error) \
	(*((lccdevSTD_FCTPTR_USR_STOP)((obj)->fcts[lccdevCMD_FCT_USR_STOP]))) \
	 ((obj),(devName),(req),(error))
#define lccdevCmdVUsrSimulat(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_SIMULAT)((obj)->fcts[lccdevCMD_FCT_USR_SIMULAT]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrStopSim(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_USR_STOP_SIM)((obj)->fcts[lccdevCMD_FCT_USR_STOP_SIM]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrStatusCode(obj,devName,text,flags,code,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_CODE)((obj)->fcts[lccdevCMD_FCT_USR_STATUS_CODE]))) \
	 ((obj),(devName),(text),(flags),(code),(error))
#define lccdevCmdVUsrStatusLast(obj,devName,last,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_LAST)((obj)->fcts[lccdevCMD_FCT_USR_STATUS_LAST]))) \
	 ((obj),(devName),(last),(error))
#define lccdevCmdVUsrStatusValue(obj,devName,code,value,error) \
	(*((lccdevCMD_FCTPTR_USR_STATUS_VALUE)((obj)->fcts[lccdevCMD_FCT_USR_STATUS_VALUE]))) \
	 ((obj),(devName),(code),(value),(error))
#define lccdevCmdVUsrTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_TEST)((obj)->fcts[lccdevCMD_FCT_USR_TEST]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrSelfTest(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_SELF_TEST)((obj)->fcts[lccdevCMD_FCT_USR_SELF_TEST]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUsrError(obj,devName,error) \
	(*((lccdevSTD_FCTPTR_USR_ERROR)((obj)->fcts[lccdevCMD_FCT_USR_ERROR]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVAvail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_AVAIL)((obj)->fcts[lccdevCMD_FCT_AVAIL]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUnavail(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNAVAIL)((obj)->fcts[lccdevCMD_FCT_UNAVAIL]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVDisable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_DISABLE)((obj)->fcts[lccdevCMD_FCT_DISABLE]))) \
	 ((obj),(devName),(access),(error))
#define lccdevCmdVEnable(obj,devName,access,error) \
	(*((lccdevCMD_FCTPTR_ENABLE)((obj)->fcts[lccdevCMD_FCT_ENABLE]))) \
	 ((obj),(devName),(access),(error))
#define lccdevCmdVLock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_LOCK)((obj)->fcts[lccdevCMD_FCT_LOCK]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVUnlock(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_UNLOCK)((obj)->fcts[lccdevCMD_FCT_UNLOCK]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVHandset(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HANDSET)((obj)->fcts[lccdevCMD_FCT_HANDSET]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVHsetOff(obj,devName,error) \
	(*((lccdevCMD_FCTPTR_HSET_OFF)((obj)->fcts[lccdevCMD_FCT_HSET_OFF]))) \
	 ((obj),(devName),(error))
#define lccdevCmdVFixPos(obj,devName,position,positionNum,reset,error) \
	(*((lccdevCMD_FCTPTR_FIX_POS)((obj)->fcts[lccdevCMD_FCT_FIX_POS]))) \
	 ((obj),(devName),(position),(positionNum),(reset),(error))
#define lccdevCmdVMove(obj,devName,position,positionNum,error) \
	(*((lccdevCMD_FCTPTR_MOVE)((obj)->fcts[lccdevCMD_FCT_MOVE]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define lccdevCmdVMoveRel(obj,devName,position,positionNum,error) \
	(*((lccdevCMD_FCTPTR_MOVE_REL)((obj)->fcts[lccdevCMD_FCT_MOVE_REL]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define lccdevCmdVSet(obj,devName,position,positionNum,error) \
	(*((lccdevCMD_FCTPTR_SET)((obj)->fcts[lccdevCMD_FCT_SET]))) \
	 ((obj),(devName),(position),(positionNum),(error))
#define lccdevCmdVCheck(obj,devName,status,error) \
	(*((lccdevCMD_FCTPTR_CHECK)((obj)->fcts[lccdevCMD_FCT_CHECK]))) \
	 ((obj),(devName),(status),(error))
#define lccdevCmdVWait(obj,devName,timeout,error) \
	(*((lccdevCMD_FCTPTR_WAIT)((obj)->fcts[lccdevCMD_FCT_WAIT]))) \
	 ((obj),(devName),(timeout),(error))
#define lccdevCmdVUsrSetMode(obj,devName,cmd,available,enabled,simulation,handset,locked,error) \
	(*((lccdevCMD_FCTPTR_USR_SET_MODE)((obj)->fcts[lccdevCMD_FCT_USR_SET_MODE]))) \
	 ((obj),(devName),(cmd),(available),(enabled),(simulation),(handset),(locked),(error))


ccsCOMPL_STAT lccdevCmdNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lccdevCmdDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


#endif // !LCCDEV_CMD_H

/************************************************************************/
/* ___oOo___ */
