/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: lcctoo.h,v 1.30 2005/05/09 15:25:52 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  alongino   11/12/03  added ifndef __FILE_LINE__ for APR2004
*  rschmutzer 10/09/98  Changed debug levels from 1-5 to 0-4.
*  rschmutzer 02/09/95  Created by eccsgen V0.45 (September 1995)
*  rschmutzer 30/06/96  Adapted for JUN96 (lcctooBreak, lcctooKill)
*
*
*    !!!!!!!!!  CAUTION: THIS FILE IS UPDATED BY ECCSGEN  !!!!!!!!!
*/

/*************************************************************************
*   NAME
*	lcctoo - class with basic LCC process support.
*
*   SYNOPSIS
*	#include "lcctoo.h"
*
*	ccsCOMPL_STAT lcctooNewObj(
*	    IN    dbSYMADDRESS	point,
*	    IN    egenOBJ*	owner,
*	    OUT   egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooDeleteObj(
*	    INOUT egenOBJ*	obj,
*	    OUT   ccsERROR*	error)
*
*	egenOBJ* lcctooGetOwner(
*	    IN const egenOBJ* obj)
*
*	egenOBJ* lcctooLogData(
*	    IN const egenOBJ*	obj,
*	    IN const char*	format,
*	    IN 			...)
*
*	ccsCOMPL_STAT lcctooDebug(
*	    INOUT egenOBJ*	obj,
*	    IN    vltINT32	level,
*	    IN    const char*	log,
*	    IN    const char*	verbose,
*	    IN    const char*	timeout,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooGetDebug(
*	    INOUT egenOBJ*	obj,
*	    OUT   char**	text,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooVerbose(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	status,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooVersion(
*	    INOUT egenOBJ*	obj,
*	    OUT   char**	version,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooLogSave(
*	    INOUT egenOBJ*	obj,
*	    IN    vltLOGICAL	flag,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooGetDebugCode(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    const char*	text,
*	    IN    vltINT32	flags,
*	    OUT   vltINT32*	code,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT lcctooGetDebugValue(
*	    INOUT egenOBJ*	obj,
*	    IN    const char*	devName,
*	    IN    vltINT32	code,
*	    OUT   lcctooVALUE*	value,
*	    OUT   ccsERROR*	error)
*
*	void lcctooBreak(
*	    IN	  int		sig)
*
*	void lcctooKill(
*	    IN	  int		sig)
*
*	ccsCOMPL_STAT lcctooErrSetLevel(p1)
*	ccsCOMPL_STAT lcctooErrSetLog(p1)
*	ccsCOMPL_STAT lcctooErrSetTimeout(p1)
*	ccsCOMPL_STAT lcctooErrVerbose(p1)
*	ccsCOMPL_STAT lcctooErrSemCreate(p1)
*	ccsCOMPL_STAT lcctooErrSemTake(p1)
*	ccsCOMPL_STAT lcctooErrSemGive(p1)
*	ccsCOMPL_STAT lcctooErrSemDelete(p1)
*	ccsCOMPL_STAT lcctooErrBreakHandler()
*	ccsCOMPL_STAT lcctooErrKillHandler()
*	ccsCOMPL_STAT lcctooErrErrDup()
*	ccsCOMPL_STAT lcctooErrCode(p1,p2)
*	ccsCOMPL_STAT lcctooErrGetCode(p1,p2)
*	ccsCOMPL_STAT lcctooErrStatus(p1,p2)
*	ccsCOMPL_STAT lcctooErrReadClassType(p1)
*	ccsCOMPL_STAT lcctooErrStrTooLong(p1)
*
*	egenFUNCPTR   lcctooBreakHandler = NULL;
*	egenOBJ*      lcctooBreakHandlerObject = NULL;
*	egenFUNCPTR   lcctooKillHandler = NULL;
*	vltINT32      lcctooDebugLevel = 0;
*	vltLOGICAL    lcctooLogFlag = FALSE;
*	vltLOGICAL    lcctooVerboseFlag = FALSE;
*	vltLOGICAL    lcctooTimeoutFlag = TRUE;
*	char* 	      lcctooVersionText = NULL;
*
*   PARENT CLASS
*	-
*
*   DESCRIPTION
*       The class lcctoo implements the basic support for LCC processes.
*
*       The following standard commands are implemented in lcctoo:
*       DEBUG, GETDEBU, BREAK, KILL, VERBOSE and VERSION.
*
*       Some of the facilities of the class lcctoo have been taken from
*       the module too.
*
*   PUBLIC METHODS (COMMANDS)
*	lcctooNewObj(point,owner,obj,error)
*	    Initialises an instance of the class type lcctoo.
*
*	    point	<IN>	OLDB point where the owned attributes are
*				located.
*	    owner	<IN>	Pointer to the owner of the new instance.
*	    obj		<OUT>	Handle for the new instance (the handle
*				must be allocated by the caller).
*
*	lcctooDeleteObj(obj,error)
*	    The function lcctooDeleteObj deletes an
*	    instance of the class type lcctoo.
*
*	lcctooDebug(obj,level,log,verbose,timeout,error) (DEBUG)
*	   lcctooDebug enables/disables the debug facilities.
*   
*	   obj	   <INOUT>	lcctoo object instance (not used).
*	   level   <IN>		Debugging level (1 to 5).
*	   log	   <IN>		Debug logging flag (On or Off).
*	   verbose <IN>		Verbose flag (On or Off).
*	   timeout <IN>		Reply timeout flag (On or Off).
*
*	lcctooGetDebug(obj,text,error) (GETDEBUG)
*	   Returns a text describing the state of the debugging
*	   facilities.
*   
*	   obj	<INOUT>		lcctoo object instance (not used).
*	   text	<OUT>		Pointer to the buffer where the
*				text shall be placed (the buffer
*				must be allocated by the caller).
*
*	lcctooBreak(sig) (BREAK)
*	   Stops the current action performed by the process.
*	   Stops all mechanical movements of the managed software
*	   device.
*
*	   The function lcctooBreak should be registered to be called
*	   when a BREAK signal is received by the application.
*   
*	   obj	 <INOUT>	lcctoo object instance (not used).
*	   error <OUT>		Error structure.
*   
*	   lcctooBreak will call the function registered with
*	   lcctooBreakHandler as follows:
*   
*	   If lcctooBreakHandlerObject is NULL:
*	     (*lcctooBreakHandler)(ccsERROR* error)
*	   otherwise:
*	     (*lcctooBreakHandler)(egenOBJ*  lcctooBreakHandlerObject,
*				   ccsERROR* error)
*    
*	   It is not possible to register multiple break handlers. Even so,
*	   if lcctooBreakHandler is not NULL when a break handler is
*	   registered, then lcctooBreakHandler and lcctooBreakHandlerObject
*	   should be saved and called from the new registered handler.
*
*	lcctooKill(sig) (KILL)
*	   Terminates the process.
*	   Equivalent to the EXIT command.
*
*	   lcctooKill calls the kill handler registered in
*	   lcctooKillHandler as follows:
*	     (*lcctooKillHandler)(ccsERROR* error)
*	   Then the application is terminated with cmdTerminate.
*   
*	   Note: This function can not be called as a task from the CI.
*
*	lcctooVerbose(obj,status,error) (VERBOSE)
*	   Switch on or off verbose mode. Verbose On gives
*	   multiple replies to commands which take a long
*	   time to execute. Verbose Off gives always only
*	   one reply on completion.
*
*	   obj	  <INOUT>	lcctoo object instance (not used).
*	   status <IN>		New value for the verbose flag.
*
*	lcctooVersion(obj,version,error) (VERSION)
*	   The application should register its version with
*	   lcctooVersionText.  If no version is registered,
*	   then the text 'Unknown version' is returned.
*	   
*	   obj	   <INOUT>	lcctoo object instance (not used).
*	   version <OUT>	Pointer to the returned version.
*
*   PUBLIC DATA MEMBERS
*	lcctooDebugLevel  - Debugging level (read only).
*	lcctooLogFlag	  - Debug logging flag (read only).
*	lcctooVerboseFlag - Verbose flag (read only).
*	lcctooTimeoutFlag - Reply timeout flag (read only).
*
*	lcctooID is the class type identifier of lcctoo.
*	Class type identifiers are stored in (obj->id) and allow to
*	identify the type of an object in runtime. The class type 
*	identifier is also used as the log identifier when logData
*	is used.
*
*   PROTECTED METHODS
*	lcctooLogSave(obj,flag,error)
*	   Enables/disables logging of comments.
*   
*	   flag	<IN>	New value for the debug log flag.
*
*	lcctooGetDebugCode(obj,devName,text,flags,code,error)
*	   lcctooGetDebugCode returns the code of the debugging
*	   facility given in text.
*   
*	   obj	   <INOUT>	lcctoo object instance (not used).
*	   devName <IN>		Not used (see module lccdev).
*	   text	   <IN>		Name of the debugging facility.
*	   flags   <IN>		 lcctooVALUE_NUMERIC: numeric value.
*				 lcctooVALUE_NAME:    supply name.
*				 lcctooVALUE_UNIT:    supply unit.
*				 lcctooVALUE_NORMAL:  normal status.
*	   code	<OUT>/<IN>	Code of the debugging facility.
*				 lcctooGET_DEBUG      "DebugLevel"
*				 lcctooGET_LOG        "Log"
*				 lcctooGET_VERBOSE    "Verbose"
*				 lcctooGET_TIMEOUT    "Timeout"
*
*	lcctooGetDebugValue(obj,devName,code,value,error)
*	   lcctooGetDebugValue returns a value describing the state of
*	   the debugging facility with the given code.
*
*	   obj	   <INOUT>	lcctoo object instance (not used).
*	   devName <IN>		Not used (see module lccdev).
*	   value   <OUT>	Returned value.
*
*	lcctooErr<error>(...)
*	   The macros lcctooErr*() log an error with lcctooLOG_0 and
*	   call errAdd with the corresponding error parameters.
*	   These macros are suppossed to be used inside the 
*	   RETURN_FAILURE macro as follows:
*
*	    ...
*	    if (lcctooVerbose(obj, param, error) != SUCCESS)
*		RETURN_FAILURE(lcctooErrVerbose(param));
*	    ...
*
*   PROTECTED DATA MEMBERS
*	lcctooBreakHandler       - Pointer to register a user defined 
*			           break handler.
*	lcctooBreakHandlerObject - Pointer to register the object
*				   related with lcctooBreakHandler.
*	lcctooKillHandler 	 - Pointer to register a user defined
*				   kill handler.
*	lcctooVersionText - Version of the software. See lcctooVersion().
*
*   PRIVATE METHODS
*	lcctooLogData(obj,format,...)
*	   logs a comment calling logData("lcctoo", ...).
*	   The log id is set to the class type identifier of obj.
*
*	lcctooLogSave(obj,flag,error)
*	   Enables/disables logging of comments.
*   
*	   flag	<IN>	New value for the debug log flag.
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*	The names of the defined errors begin as follows:
*	lcctooERR_<error>
*
*   CAUTIONS
*	Only a single instance of lcctoo can be used in a process.
*
*   SEE ALSO
*	lcctooValue(3) lcctooMisc(3) ASSERT(3) lcctooLogComment(3) 
*	lcctooSendReplyText(3) lcctooSendVerboseText(3) 
*	lcctooRecvMsg(3) lcctooString(3) lcctooErrDuplicate(3)
*	lcctooSymFind(3) lcctooCcsInit(3)
*
*-------------------------------------------------------------------------
*/

#ifndef LCCTOO_H
#define LCCTOO_H


/*
 * defines
 */
#ifndef IN
#define IN
#endif
#ifndef OUT
#define OUT
#endif
#ifndef INOUT
#define INOUT
#endif

/* From eccs.h:                                                             */
/* The macro __FILE_LINE__ produces a string in the format: "FileName:Line" */
/* with the name and line of the file where it has been called.             */
/* It is usefull in error messages                                          */
/* (the other macros defined here are just for support, since I have not    */
/* found a cleaner way to define __FILE_LINE__ macro                        */
#define _eccs_tostr(a) #a
#define _eccs_tostr_pass2(a) _eccs_tostr(a)
#ifndef __FILE_LINE__ 
#define __FILE_LINE__   __FILE__ ":" _eccs_tostr_pass2(__LINE__)
#endif


#include "egen.h"
#include "lcctooDefines.h"
#include "lcctooErrors.h"
#include "lcctooValue.h"
#include "lcctooString.h"
#include "lcctooTask.h"


/*
 * class type identifier
 */
#define lcctooID	110


/*
 * log function
 */
#define lcctooLogData(obj, format, arg...) \
	{ \
	char buf[lcctooTMP_BUFFER_SIZE]; \
	\
	sprintf(buf, (format), ##arg); \
	logData("lcctoo", (obj)->id, buf); \
	}


/*
 * error functions
 */
#define lcctooErrFindSymbol(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_FIND_SYMBOL: Symbol %s not found%s.",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_FIND_SYMBOL,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define lcctooErrSetLevel(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SET_LEVEL: Wrong debug level %d.",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SET_LEVEL,__FILE_LINE__,"%d",(p1)); \
	}
#define lcctooErrSetLog(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SET_LOG: Wrong log parameter %s. Must be 'On' or 'Off'",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SET_LOG,__FILE_LINE__,"%s",(p1)); \
	}
#define lcctooErrSetTimeout(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SET_TIMEOUT: Wrong timeout parameter %s. Must be 'On' or 'Off'",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SET_TIMEOUT,__FILE_LINE__,"%s",(p1)); \
	}
#define lcctooErrVerbose(p1) \
	{ \
	lcctooLOG_0("lcctooERR_VERBOSE: Wrong verbose parameter %s. Must be 'On' or 'Off'",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_VERBOSE,__FILE_LINE__,"%s",(p1)); \
	}
#define lcctooErrSemCreate(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SEM_CREATE: Semaphore creation failed (vxWorks error %d)",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SEM_CREATE,__FILE_LINE__,"%d",(p1)); \
	}
#define lcctooErrSemTake(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SEM_TAKE: Take semaphore failed (vxWorks error %d)",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SEM_TAKE,__FILE_LINE__,"%d",(p1)); \
	}
#define lcctooErrSemGive(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SEM_GIVE: Give semaphore failed (vxWorks error %d)",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SEM_GIVE,__FILE_LINE__,"%d",(p1)); \
	}
#define lcctooErrSemDelete(p1) \
	{ \
	lcctooLOG_0("lcctooERR_SEM_DELETE: Semaphore deletion failed (vxWorks error %d)",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_SEM_DELETE,__FILE_LINE__,"%d",(p1)); \
	}
#define lcctooErrBreakHandler() \
	{ \
	lcctooLOG_0("lcctooERR_BREAK_HANDLER: Error encountered in the Break Handler"); \
	errAdd(error,"lcctoo",lcctooERR_BREAK_HANDLER,__FILE_LINE__,""); \
	}
#define lcctooErrKillHandler() \
	{ \
	lcctooLOG_0("lcctooERR_KILL_HANDLER: Error encountered in the Kill Handler"); \
	errAdd(error,"lcctoo",lcctooERR_KILL_HANDLER,__FILE_LINE__,""); \
	}
#define lcctooErrErrDup() \
	{ \
	lcctooLOG_0("lcctooERR_ERR_DUP: Error stack empty. Can't duplicate error"); \
	errAdd(error,"lcctoo",lcctooERR_ERR_DUP,__FILE_LINE__,""); \
	}
#define lcctooErrCode(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_CODE: Device %s: Wrong status code %d.",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_CODE,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lcctooErrGetCode(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_GET_CODE: Device %s: Status %s not known.",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_GET_CODE,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define lcctooErrStatus(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_STATUS: Device %s: Status %s not allowed with current status option.",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_STATUS,__FILE_LINE__,"%s,%s",(p1),(p2)); \
	}
#define lcctooErrReadClassType(p1) \
	{ \
	lcctooLOG_0("lcctooERR_READ_CLASS_TYPE: Unable to obtain type of class %s.",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_READ_CLASS_TYPE,__FILE_LINE__,"%s",(p1)); \
	}
#define lcctooErrRecvMsg() \
	{ \
	lcctooLOG_0("lcctooERR_RECV_MSG: Error receiving a message."); \
	errAdd(error,"lcctoo",lcctooERR_RECV_MSG,__FILE_LINE__,""); \
	}
#define lcctooErrReply() \
	{ \
	lcctooLOG_0("lcctooERR_REPLY: Error reply received."); \
	errAdd(error,"lcctoo",lcctooERR_REPLY,__FILE_LINE__,""); \
	}
#define lcctooErrSendMsg() \
	{ \
	lcctooLOG_0("lcctooERR_SEND_MSG: Error sending a message."); \
	errAdd(error,"lcctoo",lcctooERR_SEND_MSG,__FILE_LINE__,""); \
	}

/*
 * get object owner
 */
#define lcctooGetOwner(obj) ((obj)->owner)

 
/*
 * class functions
 */

ccsCOMPL_STAT lcctooDebug(
    INOUT egenOBJ*	obj,
    IN    vltINT32	level,
    IN    const char*	log,
    IN    const char*	verbose,
    IN    const char*	timeout,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooGetDebug(
    INOUT egenOBJ*	obj,
    OUT   char**	text,
    OUT   ccsERROR*	error);

void lcctooBreak(
    IN	  int		sig);

void lcctooKill(
    IN	  int		sig);

ccsCOMPL_STAT lcctooVerbose(
    INOUT egenOBJ*	obj,
    IN    const char*	status,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooVersion(
    INOUT egenOBJ*	obj,
    OUT   char**	version,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooErrDuplicate(
    INOUT ccsERROR*       error,
    IN    const ccsLOC_ID locId);

ccsCOMPL_STAT lcctooLogComment(
      IN    const char*	fmt,
      IN    ...);
  
ccsCOMPL_STAT lcctooLogSave(
    INOUT egenOBJ*	obj,
    IN    vltLOGICAL	flag,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooGetDebugCode(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    const char*	text,
    IN    vltINT32	flags,
    OUT   vltINT32*	code,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooGetDebugValue(
    INOUT egenOBJ*	obj,
    IN    const char*	devName,
    IN    vltINT32	code,
    OUT   lcctooVALUE*	value,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooUseObj(
    INOUT egenOBJ*	obj,
    IN    egenOBJ*	newObj,
    IN    egenOBJ*	oldObj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooSendVerboseText(
    INOUT egenOBJ*	obj,
    IN    const char*	fmt,
    OUT   ccsERROR*	error,
    IN    ...);

ccsCOMPL_STAT lcctooSendReplyText(
    INOUT egenOBJ*	obj,
    IN    const char*	fmt,
    OUT   ccsERROR*	error,
    IN    ...);

ccsCOMPL_STAT lcctooCreateObj(
    IN    const char*	type,
    IN    const char*	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooRecvMsg(
    INOUT egenOBJ*	obj,
    IN    vltINT32	flags,
    IN    vltINT32	timeout,
    OUT   vltINT32*	id,
    OUT   char**	message,	/* vltBYTES32 */
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooSymFind(
    IN    const char*	name,
    OUT   void*	ptr,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooCcsInit(
    IN    const ccsPROCNAME	name,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooNewObj(
    IN    dbSYMADDRESS	point,
    IN    egenOBJ*	owner,
    OUT   egenOBJ*	obj,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT lcctooDeleteObj(
    INOUT egenOBJ*	obj,
    OUT   ccsERROR*	error);


/*
 * macro to execute a piece of code and return failure
 */
#define SET_ERROR(code) { stat = FAILURE; code; }
#define RETURN_FAILURE(code) { code; return (FAILURE); }

/*
 * Block macros
 */
#define BLOCK_BEGIN while(TRUE) {
#define BLOCK_BREAK(code) { code; break; }
#define BLOCK_ERROR(code) { stat = FAILURE; code; break; }
#define BLOCK_END   break; }

/*
 * block1
 * (implemented because sometimes the while(1) and for(;;) constructs
 * generate an internal error of the cc68k compiler)
 */
#define BLOCK1_BEGIN
#define BLOCK1_BREAK(code)  { code; goto block_end; }
#define BLOCK1_ERROR(code)  { stat = FAILURE; code; goto block_end; }
#define BLOCK1_END block_end:


/*
 * Debug macros
 */
#define lcctooPRINT_ERROR(reason, error) \
    {\
    printf("File %s line %d\n", __FILE__, __LINE__);        \
    printf("Failure: %s\n", reason);			    \
    printf("Error structure\n");                            \
    printf("   EnvName    : %s\n", (error)->envName);       \
    printf("   StackId    : %x\n", (error)->stackId.id);    \
    printf("   SequenceNbr: %d\n", (error)->sequenceNumber);\
    printf("   ModuleId   : %s\n", (error)->moduleId);      \
    printf("   LocationId : %s\n", (error)->location);      \
    printf("   ErrorNumber: %d\n", (error)->errorNumber);   \
    printf("   Parameters : %s\n", (error)->runTimePar);    \
    errCloseStack(error); \
    }

#ifdef LOG_LEVEL
#define lcctooLOG_0(format, arg...) \
	lcctooLogComment("0 - " __FILE_LINE__ ":\t" format, ##arg)
#else
#define lcctooLOG_0(format, arg...)
#endif

#if LOG_LEVEL >= 1
#define lcctooLOG_1(format, arg...) \
	lcctooLogComment("1 - " __FILE_LINE__ ":\t" format, ##arg)
#else
#define lcctooLOG_1(format, arg...)
#endif

#if LOG_LEVEL >= 2
#define lcctooLOG_2(format, arg...) \
	lcctooLogComment("2 - " __FILE_LINE__ ":\t" format, ##arg)
#else
#define lcctooLOG_2(format, arg...)
#endif

#if LOG_LEVEL >= 3
#define lcctooLOG_3(format, arg...) \
	lcctooLogComment("3 - " __FILE_LINE__ ":\t" format, ##arg)
#else
#define lcctooLOG_3(format, arg...)
#endif

#if LOG_LEVEL >= 4
#define lcctooLOG_4(format, arg...) \
	lcctooLogComment("4 - " __FILE_LINE__ ":\t" format, ##arg)
#else
#define lcctooLOG_4(format, arg...)
#endif


#ifdef NDEBUG
#define ASSERT(f)
#define ASSERTMSG(f,text)
#else
void lcctooAssert(
    const char* location);
void lcctooAssertMsg(
    const char* location,
    const char* text);

#define ASSERT(f) \
	if (!(f)) lcctooAssert(__FILE_LINE__);

#define ASSERTMSG(f,text) \
	if (!(f)) lcctooAssertMsg(__FILE_LINE__,(text));

#endif


/*
 * Dynamic memory allocation macros (from too module)
 */
#define ALLOCATE(type) \
    ((type*)calloc(1,sizeof(type)))

#define ALLOCBUF(type,nbr) \
    ((type*)calloc(nbr,sizeof(type)))

#define REALLOCBUF(ptr,type,nbr) \
    ((type*)((ptr) ? realloc(ptr,(nbr)*sizeof(type)):(ptr = calloc(nbr,sizeof(type)))))

#define FREE(ptr) \
    ((void) ((ptr) ? (free((char*)ptr), ptr = NULL) : NULL))


/*
 * Critical region macros (in part from too module)
 */
#ifdef BEGIN_CRITICAL_REGION
#undef BEGIN_CRITICAL_REGION
#endif
#ifdef END_CRITICAL_REGION
#undef END_CRITICAL_REGION
#endif

#define lcctooONE_SEC	sysClkRateGet()

#define lcctooNewCriticalRegion(semId) \
    ((((semId) = semMCreate(SEM_Q_PRIORITY|SEM_INVERSION_SAFE)) == NULL) ? \
     ({lcctooErrSemCreate(errnoGet()); FAILURE; }) : SUCCESS)

#define lcctooBeginCriticalRegion(semId) \
    ((semTake((semId), lcctooONE_SEC) != OK) ? \
     ({lcctooErrSemTake(errnoGet()); FAILURE; }) : SUCCESS)

#define lcctooBeginCriticalRegion2(semId,msecs) \
    ((semTake((semId), (msecs) * sysClkRateGet() / 1000) != OK) ? \
     ({lcctooErrSemTake(errnoGet()); FAILURE; }) : SUCCESS)

#define lcctooEndCriticalRegion(semId) \
    ((semGive(semId) != OK) ? \
	({lcctooErrSemGive(errnoGet()); FAILURE; }) : SUCCESS)

#define lcctooDeleteCriticalRegion(semId) \
    ((semDelete(semId) != OK) ? \
	({lcctooErrSemDelete(errnoGet()); FAILURE; }) : SUCCESS)

#define NEW_CRITICAL_REGION(semId) \
    if (((semId) = semMCreate(SEM_Q_PRIORITY|SEM_INVERSION_SAFE)) == NULL) \
	RETURN_FAILURE(lcctooErrSemCreate(errnoGet())); \

#define BEGIN_CRITICAL_REGION(semId)  \
    if(semTake((semId), lcctooONE_SEC) != OK) \
	RETURN_FAILURE(lcctooErrSemTake(errnoGet())); \

#define END_CRITICAL_REGION(semId) \
    if(semGive(semId) != OK) \
	RETURN_FAILURE(lcctooErrSemGive(errnoGet())); \

#define RETURN_CRITICAL_REGION(semId,code) \
    { \
    code; \
    if(semGive(semId) != OK) \
	lcctooErrSemGive(errnoGet()) \
    return (FAILURE); \
    }

#define BREAK_CRITICAL_REGION(semId,code) \
    { \
    code; \
    if(semGive(semId) != OK) \
	lcctooErrSemGive(errnoGet()) \
    break; \
    }

#define BREAK1_CRITICAL_REGION(semId,code) \
    { \
    code; \
    if(semGive(semId) != OK) \
	lcctooErrSemGive(errnoGet()) \
    goto block_end; \
    }

#define DELETE_CRITICAL_REGION(semId) \
    if (semDelete(semId) != OK) \
	RETURN_FAILURE(lcctooErrSemDelete(errnoGet()));


/*
 * error related macros 
 */
#define lcctooErrDup() \
	lcctooErrDuplicate(error,__FILE_LINE__)
#define lcctooErrorStatus() \
	(((error)->errorNumber == 0) ? SUCCESS : FAILURE)


/*
 * Global variables
 */
#define lcctooTMP_BUFFER_SIZE 2048

extern vltINT32	   lcctooDebugLevel;
extern vltLOGICAL  lcctooLogFlag;
extern vltLOGICAL  lcctooVerboseFlag;
extern vltLOGICAL  lcctooTimeoutFlag;
extern char*	   lcctooVersionText;
extern egenFUNCPTR lcctooBreakHandler;
extern egenOBJ*	   lcctooBreakHandlerObject;
extern egenFUNCPTR lcctooKillHandler;
extern const char* lcctooBoolStr[2];

/*
 * Misc. functions
 */
vltDOUBLE lcctooRound(vltDOUBLE num);

void lcctooMemDump(void* buf, int size);

ccsCOMPL_STAT lcctooCaiRWTable(
    IN    char*         table,
    IN    vltLOGICAL    write,
    IN    int           attrNum,
    IN    void*         attrBuffer,
    IN    int           firstRec,
    IN    int           lastRec,
    IN    int           firstFld,
    IN    int           lastFld,
    OUT   ccsERROR*     error);

ccsCOMPL_STAT lcctooCaiRWTableAdj(
    IN    char*         table,
    IN    vltLOGICAL    write,
    IN    int           attrNum,
    IN    void*         attrBuffer,
    IN    int           firstRec,
    IN    int           lastRec,
    IN    int           firstFld,
    IN    int           lastFld,
    IN    int*          attrBufferAdj,
    OUT   ccsERROR*     error);


#endif // !LCCTOO_H

/************************************************************************/
/* ___oOo___ */
