/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: lcctooTask.h,v 1.30 2005/05/09 15:25:53 vltsccm Exp $"
* 
*  who       when      what
*  --------  --------  ----------------------------------------------
*  rschmutz  05/04/98  created
*
*/

#ifndef LCCTOO_TASK_H
#define LCCTOO_TASK_H


/*************************************************************************
*# constants
*-------------------------------------------------------------------------
*/

#define lcctooTASK_START_TIMEOUT (5 * sysClkRateGet())
#define lcctooTASK_STOP_TIMEOUT  (5 * sysClkRateGet())


/*************************************************************************
*# lcctooTASK structure
*-------------------------------------------------------------------------
*/

typedef struct
    {
    ccsPROCNAME name;		/* task name */
    vltINT32	priority;	/* task priority */
    vltINT32	spawnOptions;	/* taskSpawn options */
    vltINT32	stackSize;	/* task stack size */
    FUNCPTR	entryPt;	/* task function */

    vltINT32	wait;		/* delay used by lcctooWait
				   in msecs. (-1 = forever) */
    vltINT32	ackTimeout;	/* ack. timeout for lcctooWakeup */
    vltINT32	lockTimeout;	/* timeout for the lock functions */
    vltLOGICAL	restart;	/* restart flag for lcctooTaskWait */

    void*	userData;	/* not used */
    vltINT32	userValue1;	/* not used */
    vltINT32	userValue2;	/* not used */

    vltINT32	taskId;		/* task id */
    SEM_ID	waitSem;	/* wait semaphore */
    SEM_ID	ackSem;		/* ack. semaphore */
    SEM_ID	lockSem;	/* lock semaphore */
    egenOBJ	obj;		/* associated object */
    } lcctooTASK;


/*************************************************************************
*# Functions
*-------------------------------------------------------------------------
*/
ccsCOMPL_STAT lcctooTaskNew(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskNewObj(
    egenOBJ *obj,
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskDelete(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskStart(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskWakeup(
    lcctooTASK* task,
    vltLOGICAL  ack,
    ccsERROR*   error);

ccsCOMPL_STAT lcctooTaskStop(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskLock(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskUnlock(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskInit(
    lcctooTASK* task,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskWait(
    lcctooTASK* task,
    vltLOGICAL* cont,
    vltLOGICAL* wakeup,
    ccsERROR* error);

ccsCOMPL_STAT lcctooTaskExit(
    lcctooTASK* task,
    ccsERROR* error);


/*************************************************************************
*# Error macros
*-------------------------------------------------------------------------
*/
#define lcctooErrTaskStartSpawn(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_TASK_START_SPAWN: Problems starting task %s (errno=%d).",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_TASK_START_SPAWN,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lcctooErrTaskStartTimeout(p1) \
	{ \
	lcctooLOG_0("lcctooERR_TASK_START_TIMEOUT: Timeout starting task %s.",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_TASK_START_TIMEOUT,__FILE_LINE__,"%s",(p1)); \
	}
#define lcctooErrTaskAckTimeout(p1,p2) \
	{ \
	lcctooLOG_0("lcctooERR_TASK_ACK_TIMEOUT: Timeout waiting acknowledge from task %s (%d msecs.).",(p1),(p2)); \
	errAdd(error,"lcctoo",lcctooERR_TASK_ACK_TIMEOUT,__FILE_LINE__,"%s,%d",(p1),(p2)); \
	}
#define lcctooErrTaskStop(p1) \
	{ \
	lcctooLOG_0("lcctooERR_TASK_STOP: Problems stopping task %s. Task deleted.",(p1)); \
	errAdd(error,"lcctoo",lcctooERR_TASK_STOP,__FILE_LINE__,"%s",(p1)); \
	}


#endif /* !LCCTOO_TASK_H */

/************************************************************************/
/* ___oOo___ */

