#ifndef LCUDRV_H
#define LCUDRV_H
/******************************************************************************
*
* E.S.O. - VLT project
*
* "@(#) $Id: lcudrv.h,v 1.35 2005/10/11 09:15:26 vltsccm Exp $" 
*
* lcudrv.h   -	Header file for the common LCU driver module lcudrv
*		This file should be included in a driver's xxxPrivate.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2002-07-05 added lcudrvRemoveDevice
* N. FIEBIG  27/05/93  first release
* S.Sandrock 22/08/94  moved public part into separate header
*/

/*
 * System header files, necessary for common functions
 */

#include "vxWorks.h"
#include "vxLib.h"
#include "iosLib.h"
#include "memLib.h"
#include "semLib.h"
#include "stdioLib.h"
#include "string.h"
#include "stdlib.h"

#include "lcudrvPublic.h"	/* lcudrv public data */


/*
 * Data Structure for common part of Device Descriptor
 */

typedef struct lcudrvDEVICE_HEADER
    {
    DEV_HDR header;			/* VxWorks device header */
    BOOL used;				/* Flag for "descriptor used" */
    int  accessMode;			/* device access mode */
    int  numShared;			/* Number of shared opened channels */
    char name[15];			/* Device name */
    } lcudrvDEVICE_HEADER;


/*
 * Driver Channel Table Data Structure
 */

typedef struct lcudrvDCT_ENTRY
    {
    BOOL used;				/* entry-in-use flag */
    int mode;				/* open mode */
    lcudrvDEVICE_HEADER *device;	/* address of device descriptor */
    } lcudrvDCT_ENTRY;


/*
 * Prototypes of common interface layer functions
 */
void lcudrvVersion(void);
extern int lcudrvOpen(lcudrvDEVICE_HEADER *device, 
		      const char *remainder, 
		      int mode,
		      lcudrvDCT_ENTRY *dct, 
		      int maxChannels, 
		      int *channel );
extern int lcudrvClose(lcudrvDCT_ENTRY *dct, 
		       int maxChannels, 
		       int channel);
extern int lcudrvInitDCT(lcudrvDCT_ENTRY **dct, 
			 int maxChannels);
extern int lcudrvInitDevice(const char *name, 
			    int drvNumber, 
			    lcudrvDEVICE_HEADER *header );
extern int lcudrvRemoveDevice(lcudrvDEVICE_HEADER *header );
extern int lcudrvCheckChannel(const lcudrvDCT_ENTRY *dct, 
			      int maxChannels, 
			      int channel);
extern int lcudrvCheckAccess(const lcudrvDCT_ENTRY *dct, 
			     int maxChannels, 
			     int channel, 
			     int access,
			     lcudrvDEVICE_HEADER **header);

#endif  /* !LCUDRV_H */
