#ifndef LCUDRV_DEVICE_H
#define LCUDRV_DEVICE_H
/******************************************************************************
*
* E.S.O. - VLT project
*
* "@(#) $Id: lcudrvDevice.h,v 1.35 2005/10/11 09:15:26 vltsccm Exp $" 
*
* lcudrvDevice.h   -	Header file for direct hardware access
*			This file should be included in a driver's xxxDevice.h
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* S.Sandrock 24.08.1994 New
*/

/*************************************************************************
*   NAME
*       lcudrvDevice,
*	lcudrvBCHG, lcudrvBCLR, lcudrvBSET, lcudrvBTST, 
*	lcudrvGETB, lcudrvGETW, lcudrvGETL, 
*	lcudrvPUTB, lcudrvPUTW, lcudrvPUTL
*	- Macros for direct hardware access
*
*   SYNOPSIS
*
*       #include "lcudrvDevice.h"
*	void lcudrvBCHG(int bitNumber, volatile unsigned char *byteAddr)
*	void lcudrvBCLR(int bitNumber, volatile unsigned char *byteAddr)
*	void lcudrvBSET(int bitNumber, volatile unsigned char *byteAddr)
*	int  lcudrvBTST(int bitNumber, volatile unsigned char *byteAddr)
*	unsigned char  lcudrvGETB(volatile unsigned char  *addr)
*	unsigned short lcudrvGETW(volatile unsigned short *addr)
*	unsigned long  lcudrvGETL(volatile unsigned long  *addr)
*	void lcudrvPUTB(volatile unsigned char  *addr, unsigned char  value)
*	void lcudrvPUTW(volatile unsigned short *addr, unsigned short value)
*	void lcudrvPUTL(volatile unsigned long  *addr, unsigned long  value)
* 
*   DESCRIPTION
*
*	lcudrvBCHG	- Change a bit (0..7) at the specified location
*	lcudrvBCLR	- Clear a bit (0..7) at the specified location
*	lcudrvBSET	- Set a bit (0..7) at the specified location
*	lcudrvBTST	- Test a bit (0..7) at the specified location
*	lcudrvGETB	- Read a byte (8 bit) from the specified location
*	lcudrvGETW	- Read a word (16 bit) from the specified location
*	lcudrvGETL	- Read a longword (32 bit) from the specified location
*	lcudrvPUTB	- Write a byte (8 bit) to the specified location
*	lcudrvPUTW	- Write a word (16 bit) to the specified location
*	lcudrvPUTL	- Write a longword (32 bit) to the specified location
*
*	The parameters for the macros will be casted to the shown types.
*	Therefore the compiler cannot check if correct types are passed.
*
*   RETURN VALUES
*	lcudrvBCHG, lcudrvBCLR, lcudrvBSET have no return value;
*	lcudrvBTST returns the status of the tested bit (0 or 1);
*	lcudrvGETB, lcudrvGETW, lcudrvGETL return the requested value;
*	lcudrvPUTB, lcudrvPUTW, lcudrvPUTL have no return value;
*
*   ENVIRONMENT
*	Following compilation macros must be defined:
*	__GNUC__		- set by the GNU C compiler
*	CPU_FAMILY == MC680X0	- set by "vxWorks.h" upon the macro 'CPU'
*
*   CAUTIONS
*	The use of these macros can make code more efficient but less portable!
*	At present they can only be used with GNU-gcc and with 68K-CPUs.
*--------------------------------------------------------------------------
*/

#include "vxWorks.h"	/* set up the CPU_FAMILY */


#if CPU_FAMILY == MC680X0
#ifdef __GNUC__

/*
 * The following asm instructions use operand contraints as described in
 * "Using and Porting GNU CC", Version 2.5, sections 5.29 and 15.6
 */

#define lcudrvBCHG(bitNumber, byteAddr)					     \
    ({volatile unsigned char *__a = byteAddr; int __b = bitNumber;	     \
	__asm__ volatile ("bchg %2,%0"			/* instruction    */ \
			  : "=m,m" (*__a)		/* output args    */ \
			  : "0,0"  (*__a), "d,s" (__b)	/* input args     */ \
			  : "cc"			/* clobbered regs */ \
			  ); })

#define lcudrvBCLR(bitNumber, byteAddr)					     \
    ({volatile unsigned char *__a = byteAddr; int __b = bitNumber;	     \
	__asm__ volatile ("bclr %2,%0"			/* instruction    */ \
			  : "=m,m" (*__a)		/* output args    */ \
			  : "0,0"  (*__a), "d,s" (__b)	/* input args     */ \
			  : "cc"			/* clobbered regs */ \
			  ); })

#define lcudrvBSET(bitNumber, byteAddr)					     \
    ({volatile unsigned char *__a = byteAddr; int __b = bitNumber;	     \
	__asm__ volatile ("bset %2,%0"			/* instruction    */ \
			  : "=m,m" (*__a)		/* output args    */ \
			  : "0,0"  (*__a), "d,s" (__b)	/* input args     */ \
			  : "cc"			/* clobbered regs */ \
			  ); })

#define lcudrvBTST(bitNumber, byteAddr)					     \
    ({volatile unsigned char *__a = byteAddr; int __b = bitNumber; char __c; \
	__asm__ volatile ("btst %2,%1; sne %0; negb %0"	/* instruction    */ \
	/* 'sne' to bring Z-flag into variable, 'negb' to set 0xff into 1 */ \
			  : "=g,g" (__c)		/* output args    */ \
			  : "m,m"  (*__a), "d,s" (__b)	/* input args     */ \
			  : "cc"			/* clobbered regs */ \
			  ); (int)__c; })

#endif /* __GNUC__ */

#define lcudrvGETB(addr) (*(volatile unsigned char *)(addr))
#define lcudrvGETW(addr) (*(volatile unsigned short *)(addr))
#define lcudrvGETL(addr) (*(volatile unsigned long *)(addr))
#define lcudrvPUTB(addr,byte) (*(volatile unsigned char *)(addr)=(byte))
#define lcudrvPUTW(addr,word) (*(volatile unsigned short *)(addr)=(word))
#define lcudrvPUTL(addr,lword) (*(volatile unsigned long *)(addr)=(lword))

#endif /* CPU_FAMILY == MC680X0 */

#endif  /* !LCUDRV_DEVICE_H */
