#ifndef LCUDRV_PUBLIC_H
#define LCUDRV_PUBLIC_H
/******************************************************************************
*
* E.S.O. - VLT project
*
* "@(#) $Id: lcudrvPublic.h,v 1.35 2005/10/11 09:15:27 vltsccm Exp $" 
*
* lcudrvPublic.h    - 	Public data of the LCU common driver module lcudrv
*			This file should be included in a driver's xxx.h
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* S.Sandrock 22.08.1994 New
*	     05.09.1994 Added comments.
*/

/*
 * System header files, necessary for common functions
 */

#include "vxWorks.h"


/*
 * Literals for open modes and access rights
 * They should be used in open calls to devices as second parameter.
 */
#define lcudrvOPEN_READONLY		1
#define lcudrvOPEN_SHARED		2
#define lcudrvOPEN_EXCLUSIVE		4
#define lcudrvOPEN_TEST			8


/*
 * Literals for common Error Codes
 * They should be returned by drivers when the corresponding error occurs.
 */
#define lcudrvOK			 0
#define lcudrvERROR			-1
#define lcudrvERROR_INVALID_ARGUMENT	-10
#define lcudrvERROR_NO_MEMORY		-11
#define lcudrvERROR_INVALID_DEVICE	-12
#define lcudrvERROR_NO_CHANNEL		-13
#define lcudrvERROR_NO_MORE_CHANNEL	-14
#define lcudrvERROR_ACCESS_CONFLICT	-15
#define lcudrvERROR_NO_DRIVER		-16
#define lcudrvERROR_DRIVER_EXISTS	-17
#define lcudrvERROR_DEVICE_EXISTS	-18
#define lcudrvERROR_NO_SEMAPHORE	-19
#define lcudrvERROR_INVALID_COMMAND	-20
#define lcudrvERROR_TIMEOUT		-30
#define lcudrvERROR_INVALID_OPEN_MODE	-40
#define lcudrvERROR_CHANNEL_NOT_OPEN	-41


/*
 * Prototypes of common Tools
 */
extern const char *lcudrvErrorGet(int errorCode);
extern int lcudrvPrintError(int errorCode);
extern int lcudrvProbeAddress(void * address);


/*
 * Automatic installation function, for use form the VxWorks shell
 */
extern int lcudrvAutoProbe(char *probeAddr0,
			   char *probeAddr1,
			   char *probeAddr2,
			   char *probeAddr3,
			   char *probeAddr4,
			   char *probeAddr5,
			   char *probeAddr6,
			   char *probeAddr7,
			   char *probeAddr8,
			   char *probeAddr9);

extern int lcudrvAutoLoad(BOOL cond, const char *moduleName, int symFlag);

extern int lcudrvAutoExec(BOOL cond, const char *funcName, 
			  int arg1, int arg2, int arg3, int arg4, int arg5,
			  int arg6, int arg7, int arg8);

extern int lcudrvAutoDevInit(const char *funcName, int numDevices);

extern int lcudrvAutoDrvInstall(const char *moduleName, 
				int numDevices, 
				const char *drvFuncName, 
				const char *devFuncName, 
				int drvArg1, int drvArg2, int drvArg3, 
				int drvArg4, int drvArg5, int drvArg6);

extern int lcudrvAutoDevCreate(const char *devName, 
			       int arg1, int arg2, int arg3, int arg4,int arg5,
			       int arg6, int arg7, int arg8, int arg9);

extern int lcudrvAutoDevCheck(const char *devBaseName, int nominalCount);

extern int lcudrvAutoEnvInit(const char *localEnv, const char *hostEnv);

extern int lcudrvTkIoctl(const char *devName, int req, int arg, 
		  int arg2, int arg3, int arg4, int arg5,
		  int arg6, int arg7, int arg8);
extern int lcudrvTkWrite(const char *devName, const char *hostFileName, int nBytes);
extern int lcudrvTkRead(const char *devName, const char *hostFileName, int nBytes);

#endif  /* !LCUDRV_PUBLIC_H */
