#ifndef LCULOG_H
#define LCULOG_H
/*************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lculog.h,v 1.12 2000/01/13 08:14:23 vltsccm Exp $" 
*
* lculog.h	-	Contains prototypes, macro and inline function 
*			definitions for LCU logging.
*
* who           when      version/what
* ---------     --------  ---------------------------------------------------
* Phan  	Jul92	  0.0   First version
* B.Gustafsson  19.11.93  1.1   Adapted to ANSI C
* S.Sandrock    07.08.94  1.4   Changed comment structure to VLT standards.
*				Renamed LCU_LOG_send_msg to lculogMsg,
*				made it external, string-params now const.
*				Other functions now implemented as macros.
*				Added LCU_LOG_intr() for periodic interrupts.
* S.Sandrock	05.09.94 	Added lculogLog for backward compatibility.
*/

/*************************************************************************
*
*   CONDITIONAL COMPILATION
*	The following macros can be defined BEFORE including lculog.h:
*
*	lculogOPTION_MACROS	 - LCU_LOG_... are macros (default)
*	lculogOPTION_NO_MACROS	 - LCU_LOG_... are normal external functions
*	lculogOPTION_EXCLUDE_ALL - Calls to LCU_LOG_... will not be compiled
*				   (this requires OPTION_MACROS)
*	lculogOPTION_EXCLUDE_DBG - LCU_LOG_trace/warning/intr are not compiled
*				   (this requires OPTION_MACROS)
*	DEBUG			 - causes OPTION_EXCLUDE_DBG if not(!) defined
*
*/

/*
 * Uncomment the following lines to request normal function-calls as default.
 *
#ifndef lculogOPTION_MACROS
#define lculogOPTION_NO_MACROS
#endif
 */

#ifndef DEBUG
#define lculogOPTION_EXCLUDE_DBG
#endif


/*
 * Severity specific logging disable flags:
 */
extern int lculogDisable;		/* general enable */

extern int lculogDisTRACE;
extern int lculogDisDEBUG;
extern int lculogDisINTR;


/*
 * Severity specific logging enable/disable funtions (for test purposes):
 */
extern int LCU_trace(void);
extern int LCU_debug(void);
extern int LCU_intr(void);

extern void lculogVersion(void);

/*
 * Optional logging task
 */
extern int lculogTask(void);	/* Note: this logging task is optional */
extern int lculogLog(void);	/* Obsolete, for backward compatibility */

/*
 * Logging function
 */
extern void lculogFdSet(int fd);
extern int lculogFdGet(void);
extern int 
lculogMsg(const char *severity,
	  const char *fmtstr,
	  int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

/*
 * Severity specific logging functions.
 * Only used if macros are disabled.
 */
extern void 
LCU_LOG_error(const char *fmt,
	      int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

extern void 
LCU_LOG_warning(const char *fmt,
		int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

extern void 
LCU_LOG_log(const char *fmt,
	    int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

extern void 
LCU_LOG_trace(const char *fmt,
	      int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

extern void 
LCU_LOG_debug(const char *fmt,
	      int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);

extern void 
LCU_LOG_intr(const char *fmt,
	     int arg1, int arg2, int arg3, int arg4, int arg5, int arg6);



#ifndef lculogOPTION_NO_MACROS
/*
 * Severity specific logging functions.
 *
 * Implemented as macros to avoid the function-call overhead when 
 * the logging for the specified severity is not enabled.
 * And - above all - to avoid problems with shared global data in this module!
 * The do-while construction is necessary. See GNU-cpp manual, 
 * section "Pitfalls and Subtleties of Macros/Swallowing the Semicolon".
 */

#ifndef lculogOPTION_EXCLUDE_ALL
#define LCU_LOG_ALL(SEVERITY, fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	lculogMsg(NULL, #SEVERITY ": " fmt "\n",			\
		(arg1),(arg2),(arg3),(arg4),(arg5),(arg6))
#else
#define LCU_LOG_ALL(a,b,c,d,e,f,g,h)	/* excluded */
#define lculogOPTION_EXCLUDE_DBG
#endif /* lculogOPTION_EXCLUDE_ALL */

#ifndef lculogOPTION_EXCLUDE_DBG
#define LCU_LOG_DBG(SEVERITY, fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	do {								\
	    if (! lculogDis ## SEVERITY)				\
		lculogMsg(NULL, #SEVERITY ": " fmt "\n",		\
			(arg1),(arg2),(arg3),(arg4),(arg5),(arg6));	\
	} while(0)
#else
#define LCU_LOG_DBG(a,b,c,d,e,f,g,h)	/* excluded */
#endif /* lculogOPTION_EXCLUDE_DBG */

#define LCU_LOG_error(fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	LCU_LOG_ALL(ERROR, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#define LCU_LOG_warning(fmt, arg1, arg2, arg3, arg4, arg5, arg6)\
	LCU_LOG_ALL(WARN, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#define LCU_LOG_log(fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	LCU_LOG_ALL(LOG, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#define LCU_LOG_trace(fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	LCU_LOG_DBG(TRACE, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#define LCU_LOG_debug(fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	LCU_LOG_DBG(DEBUG, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#define LCU_LOG_intr(fmt, arg1, arg2, arg3, arg4, arg5, arg6)	\
	LCU_LOG_DBG(INTR, fmt, arg1, arg2, arg3, arg4, arg5, arg6)

#endif /* ! OPTION_NO_MACROS */

#endif  /* !LCULOG_H */
