/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: logLCC.h,v 4.97.1.1 2005/10/24 13:55:47 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software              SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  1.2          18/02/94        P.Rochez        SPR940016.2 Added
*                                                 logCheckStart()
*                                               SPR940016.3 Added procNum in
*                                                 logLOGSTRT_REPLY_HEADER struct
*                                               SPR940016.6 Removed error for
*                                                 logData()
*  2.1          31/01/95        P.Rochez
*       Modified logDisReadDb(), logDisWriteDb(), logDisAnalog() and
*           logDisDigital()
*       Added logMAX_LEN, logTEXT_LEN and logSYS_LEN
*       Changed LOG_LEN
*       Added log id constants
*------------------------------------------------------------------------------
*  SCCS Id:  logLCC.h  2.1  03 Feb 1995
*----------------------------------------------------------------------------*/

/*
 * <logLCC.h> - LCU / LOGGING SYSTEM Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/


/* 
 * Headers 
 */

#include "db.h"


/* 
 * Constants 
 */

#define LOG_LEN         logMAX_LEN  /* Maximum length of a log text     */


/* 
 * Data Types
 */

typedef enum {                      /* List of event types              */
    logMESSAGE = 1,                 /* Logging of all types of messages */
    logDB_READ,                     /* Logging of database read access  */
    logDB_WRITE,                    /* Logging of database write access */
    logDIGITAL_IO,                  /* Logging of digital signals change*/
                                    /* of value                         */
    logANALOG_IO                    /* Logging of analog signals change */
                                    /* of value                         */
} logEVENT_TYPE;


typedef enum {
     logALL = 1,                    /* Log enabled for all the items    */
     logSPECIFIC                    /* Log enabled for specific items   */
} logSCOPE;


typedef struct {                    /* Event definition                 */
    logEVENT_TYPE  event;           /* Type of event enabled            */
    logSCOPE       scope;           /* Scope (all or specific)          */
    union {
        dbSYMADDRESS  attr;         /* Name of database point or attr.  */
                                    /* (logDB_READ and logDB_WRITE)     */
        dbSYMADDRESS  signal;       /* Name of signal (logDIGITAL_IO    */
                                    /* and logANALOG_IO)                */
        ccsPROCNAME   process;      /* Name of process (logMESSAGE)     */
    } name;
} logEVENT;



/* 
 * Functions prototypes 
 */

/*
 * Logging system configuration
 */

ccsCOMPL_STAT logStartLogging(      /* Start the logging of messages    */
    msgCMDID           cmdId,       /*                                  */
    const msgCMD       cmdName,     /*                                  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logCheckStart(        /* Check if the logging of messages */
                                    /* and error messages has been      */
                                    /* started                          */
    vltLOGICAL         *logStarted, /* True = Logging started           */
    vltLOGICAL         *errLogStarted,  /* True = Error logging started */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logStopLogging(       /* Stops the logging of messages    */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logSetReportRate(     /* Set report rate of log buffers   */
    vltUINT32          period,      /* Period in seconds between two    */
                                    /* buffers                          */
    ccsERROR           *error       /* Returned error structure         */
    );


/*
 * Set-up of events for logging
 */

ccsCOMPL_STAT logEnMsg(             /* Enable logging of messages       */
    logSCOPE           scope,       /* Scope (all processes or specific)*/
    const ccsPROCNAME  procName,    /* Process name (used if specific)  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logDisMsg(            /* Disable logging of messages      */
    logSCOPE           scope,       /* Scope (all processes or specific)*/
    const ccsPROCNAME  procName,    /* Process name (used if specific)  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logEnReadDb(          /* Enable logging of read access to */
                                    /* the database                     */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* points and/or attributes)        */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of points and/or attributes */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logDisReadDb(         /* Disable logging of read access   */
                                    /* to the database                  */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* points and/or attributes)        */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of points and/or attributes */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logEnWriteDb(         /* Enable logging of write access   */
                                    /* to the database                  */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* points and/or attributes)        */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of points and/or attributes */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logDisWriteDb(        /* Disable logging of write access  */
                                    /* to the database                  */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* points and/or attributes)        */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of points and/or attributes */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logEnAnalog(          /* Enable logging of change of      */
                                    /* value of analog signals          */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* signals)                         */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logDisAnalog(         /* Disable logging of analog signals*/
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* signals)                         */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logEnDigital(         /* Enable logging of change of      */
                                    /* value of digital signals         */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* signals)                         */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT logDisDigital(        /* Disable logging of digital       */
                                    /* signals                          */
    logSCOPE           scope,       /* Scope of logging (all or list of */
                                    /* signals)                         */
    vltUINT32          number,      /* When specific, number of names   */
    const dbSYMADDRESS *list,       /* List of signals                  */
    ccsERROR           *error       /* Returned error structure         */
    );


/*
 * List of all events enabled for logging
 */

ccsCOMPL_STAT logGetEventList(      /* Get list of events enabled for   */
                                    /* logging                          */
    vltUINT32          *number,     /* Number of events in list         */
    logEVENT           **list,      /* List of events enabled for       */
                                    /* logging                          */
    ccsERROR           *error       /* Returned error structure         */
    );

