#ifndef LQS_LOCALTBL_H
#define LQS_LOCALTBL_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lqsLocalTbl.h,v 2.23 2005/10/11 16:20:37 vltsccm Exp $" 
*
* lqsLocalTbl.h
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* ---v2.1---
* S.Sandrock 06.10.1994 New comment header, revised Prototypes
*	     13.10.1993 Added members for 'lqsWatch' in the Local Process Table
* ---v2.2--- 31.10.1994 ---RELEASED---
* S.Sandrock 01.02.1995 Added declarations of exported data.
* --v2.2.3-- 03.02.1995
* bgustafs   13.01.2000 Changed procnum in prototypes to unsigned int to 
*                       surpress compiler warnings
*/

/************************************************************************
* Copyright (c) 1992	European  Southern  Observatory			*
*									*
*									*
*			Very Large Telescope Software			*
*			*****************************			*
*									*
*	Subsystem   :	Local Control Unit				*
*									*
*	Module      :	Q Server					*
*									*
*	Component   :	lqsLocalTbl.h					*
*									*
*	Produced by :	SpaceBel Informatique s.a.			*
*									*
*	Author      :	T. Phan Duc					*
*									*
*	Description :	Prototypes of functions to manage the		*
*			local process table.				*
*									*
*	Change history :						*
*									*
*	Author	Date	Version		Description of change		*
*									*
*	Phan  	5Jun92	  0.0		First version			*
*       AWA    12Mar93    1.1           Partly VLT Programming Standard *
*                                       add flag for obituary behaviour *
*       AWA    13Jul93    1.2           Add task Id                     * 
*       AWA    18Nov93    1.3           (SBI-LQS-03) use p to TCB       *
*       Phan   20Jan94    2.1           Add RTAP 6.30 changes:		*
*					Acknowledge with sequence nbr	*
*					and ping request/response	*
*					VLT Programming Standard 	*
*									*
*************************************************************************/

#define lqsLOCAL_TBL_SIZE	rtMAXPROC
#define lqsPROC_NAME_LEN	rtMAX_PROC_NAME_LEN

/* define bit masks in obiFlag for obituary behaviour */
#define lqsDONT_RCV_OBI		1  /* 0 = process receives obituaries   */
                                   /* 1 = process do not receive obituar*/  
#define lqsDONT_SEND_OBI	2  /* 0 = process sends obituary        */
                                   /* 1 = process do not send obituary  */ 

typedef struct
	{
	char		procName [lqsPROC_NAME_LEN];
	int		readCommId;
	int		writeCommId;
        unsigned int    obiFlag;
        int             taskId;

        unsigned int    msgSeqNum;
        int		ackCtr;
        int		fd;
        rtMsgHeader	msgHeader;

	/*
	 * The following is used by 'lqsWatch':
	 */
	volatile struct	/* Current status values (updated on-the-fly):      */
	    {
	    BOOL changed;         /* Flag: contents has changed             */
	    unsigned int seqNum;  /* Sequence number of last received msg   */
	    unsigned int nAcks;   /* Number of received ack's (cumulative)  */
	    unsigned int nNacks;  /* Number of received nack's (cumulative) */
	    unsigned int nMsgsS;  /* Number of sent msgs (cumulative)       */
	    unsigned int nMsgsR;  /* Number of received msgs (cumulative)   */
	    unsigned int nBytesS; /* Number of sent bytes (cumulative)      */
	    unsigned int nBytesR; /* Number of received bytes (cumulative)  */
	    unsigned int nQueued; /* Number of messages in the queue/pipe   */
	    } watch;
	} lqsPROC;

typedef	lqsPROC *		lqsPROCP;

typedef	lqsPROC			lqsLOCAL_TBL [lqsLOCAL_TBL_SIZE];

typedef	lqsLOCAL_TBL *		lqsLOCAL_TBLP;



/*
 * Shared data:
 */

extern	lqsLOCAL_TBL	lqsLocalTbl;
extern	int		lqsLocalTblCtr;		
extern	int		lqsLocalTblMin;		


/*
 * Functions:
 */

STATUS	lqsInitLocalTbl(void);

int	lqsAddLocalTbl(const char *	procName,
		       int		readCommId,
		       int		writeCommId,
		       unsigned int 	obiFlag);

int	lqsDeleteLocalTbl(unsigned int procNum);

int	lqsGetLocalTblCommId(unsigned int procNum,
			     int *readCommId,
			     int *writeCommId);

int	lqsGetLocalTblObiFlag(unsigned int procNum,
			      unsigned int *obiFlag);

char *	lqsGetLocalTblProcName(unsigned int procNum);

int	lqsGetLocalTblProcNum(const char *procName);

int	lqsGetLocalTblProcNumFromTask(int taskId);

int     lqsGetLocalTblTaskId(const char *procName);

const lqsPROC * lqsGetLocalTblPtr(int procNum);

int lqsGetLocalProcessNames(const char (**procNameTbl)[lqsPROC_NAME_LEN], 
			    int *nEntries);

int lqsPrintLocalProcessNames(void);

void	lqsWatch(int refrTick, int nLines);
void	lqsWatchEnable(void);
void	lqsWatchDisable(void);
STATUS	lqsWatchUpdateRecv(const lqsNET_PACKET *msgPtr);
STATUS	lqsWatchUpdateSend(const lqsNET_PACKET *msgPtr);

#endif /* !LQS_LOCALTBL_H */
