#ifndef LQS_LOG_H
#define LQS_LOG_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lqsLog.h,v 2.23 2005/10/11 16:20:37 vltsccm Exp $" 
*
* lqsLog.h
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* ---v2.1---
* S.Sandrock 06.10.1994 New comment header
*	     11.10.1994 Implemented Debug/Trace/Ping/Obi logs now as macros.
* ---v2.2--- 31.10.1994 ---RELEASED---
* S.Sandrock 02.01.1995 New variable `lqsLogFlag' to en/disable logging.
* --v2.2.2-- 02.01.1995 ---RELEASED---
* bgustafs   13.01.2000 Changed format in prototypes to const to surpress 
*                       compiler warnings
*/

/************************************************************************
* Copyright(c) 1992    European  Southern  Observatory                  *
*                                                                       *
*                                                                       *
*                       Very Large Telescope Software                   *
*                       *****************************                   *
*                                                                       *
*       Subsystem   :   Local Control Unit                              *
*                                                                       *
*       Module      :   Q Server                                        *
*                                                                       *
*       Component   :   lqsLog.c                                   	*
*                                                                       *
*       Produced by :   SpaceBel Informatique s.a.                      *
*                                                                       *
*       Author      :   T. Phan Duc                                     *
*                                                                       *
*       Description :   These functions simply use printf  		*
*                                                                       *
*                                                                       *
*       Change history :                                                *
*                                                                       *
*       Author  Date    Version         Description of change           *
*                                                                       *
*       Phan    Jan94     0.0           First version                   *
*                                                                       *
*************************************************************************/


void lqsDebug(void);
void lqsTrace(void);
void lqsPing(void);
void lqsObi(void);

void lqsLogWarning (const char * fmt,
                      int    arg1,
                      int    arg2,
                      int    arg3,
                      int    arg4,
                      int    arg5,
                      int    arg6);

void lqsLogError (const char * fmt,
                    int    arg1,
                    int    arg2,
                    int    arg3,
                    int    arg4,
                    int    arg5,
                    int    arg6);

void lqsLogLog (const char * fmt,
                  int    arg1,
                  int    arg2,
                  int    arg3,
                  int    arg4,
                  int    arg5,
                  int    arg6);

void lqsLogDebugFn (const char * fmt,
                    int    arg1,
                    int    arg2,
                    int    arg3,
                    int    arg4,
                    int    arg5,
                    int    arg6);

void lqsLogTraceFn (const char * fmt,
                    int    arg1,
                    int    arg2,
                    int    arg3,
                    int    arg4,
                    int    arg5,
                    int    arg6);

void lqsLogPingFn (const char * fmt,
                  int    arg1,
                  int    arg2,
                  int    arg3,
                  int    arg4,
                  int    arg5,
                  int    arg6);

void lqsLogObiFn (const char * fmt,
                  int    arg1,
                  int    arg2,
                  int    arg3,
                  int    arg4,
                  int    arg5,
                  int    arg6);


extern BOOL lqsLogFlag;

extern BOOL lqsLogTraceFlag;
extern BOOL lqsLogDebugFlag;
extern BOOL lqsLogPingFlag;
extern BOOL lqsLogObiFlag;

/*
 * The following macros are used instead of functions simply to reduce the
 * function-call overhead when Debug/Trace/... is NOT enabled.
 *
 * The do-while construction is necessary. See GNU-cpp manual, 
 * section "Pitfalls and Subtleties of Macros/Swallowing the Semicolon".
 */

#define lqsLogDebug(fmt, arg1, arg2, arg3, arg4, arg5, arg6)		      \
  do {									      \
   if(lqsLogDebugFlag) lqsLogDebugFn(fmt, arg1, arg2, arg3, arg4, arg5, arg6);\
  } while(0)

#define lqsLogTrace(fmt, arg1, arg2, arg3, arg4, arg5, arg6)		      \
  do {									      \
   if(lqsLogTraceFlag) lqsLogTraceFn(fmt, arg1, arg2, arg3, arg4, arg5, arg6);\
  } while(0)

#define lqsLogPing(fmt, arg1, arg2, arg3, arg4, arg5, arg6)		      \
  do {									      \
   if(lqsLogPingFlag) lqsLogPingFn(fmt, arg1, arg2, arg3, arg4, arg5, arg6);  \
  } while(0)

#define lqsLogObi(fmt, arg1, arg2, arg3, arg4, arg5, arg6)		      \
  do {									      \
   if(lqsLogObiFlag) lqsLogObiFn(fmt, arg1, arg2, arg3, arg4, arg5, arg6);    \
  } while(0)

#endif /* !LQS_LOG_H */
