#ifndef LQS_RTAP_H
#define LQS_RTAP_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lqsRtap.h,v 2.23 2005/10/11 16:20:38 vltsccm Exp $"
*
*	Rtap compatiblity header for lqs/qsemu
*
* who       when      what
* --------  --------  ----------------------------------------------
* ssandroc  22/05/95  created
*/


/*
 * NOTE: rtChar can be either unsigned or signed and should be used only
 *	 for ascii characters.  For signed bytes use rtInt8, and
 *	 for unsigned bytes use rtUInt8.
 */
typedef int			rtInt;	   /* guaranteed at least +-32767,
					      used when size does not matter */
typedef short			rtInt16;   /* 16 bit signed integer */
typedef char			rtChar;	   /* represents a character */
typedef int			rtInt32;   /* 32 bit signed integer */
typedef char			rtInt8;	   /* 8-bit signed integer */
typedef unsigned int		rtUInt32;  /* 32 bit unsigned integer */

typedef unsigned short 		rtUInt16;	/* 16-bit unsigned integer */
typedef unsigned char		rtUInt8;	/* 8-bit unsigned integer */

typedef float 			rtFloat;	/* 4 byte floating point */ 
typedef double  		rtDouble;	/* 8 byte floating point */

typedef unsigned int		rtBitfld;	/* allows coersion to bit fields */



/*
 * maximum length of an RTAP process name (19 chars + null)
 */
#define rtMAX_PROC_NAME_LEN	20

/*
 * RTAP process id
 */
typedef rtChar			rtProcessEnv[8];
typedef rtUInt8			rtProcessNum;

typedef struct 	{
    		rtProcessEnv	procEnv;	/* environment name */
    		rtProcessNum	procNum;	/* local process number */
    		rtUInt8		reserved;
		}	rtProcessId;


/*
 * RTAP message queues - general definitions
 */
#define rtRESPOND	(rtUInt8) 2		/* acknowledgement desired */
#define rtNO_RESPOND	(rtUInt8) 1		/* don't acknowledge */
#define rtRESPONSE	(rtUInt8) 0		/* acknowledgement message */

#define rtNO_TIMEOUT	(rtInt32) 0		/* wait forever for message */
#define rtNO_WAIT	(rtInt32) -1		/* don't wait if no message */
#define rtWAIT_SIG	(rtInt32) -2		/* wait unless signal recv'ed*/

#define rtMAXPROC	((rtProcessNum) ~0)	/* biggest proc num possible */
#define rtMINPRIO	(rtInt32) 100		/* lowest priority allowed */


/* 
 * RTAP standard message header 
 * (NOTE:  priority must remain as first member of structure)
 */
typedef struct 	{
    		rtInt32		priority;	/* 1 <= priority <= rtMINPRIO */
    		rtProcessId	source;		/* process sending message */
    		rtProcessId	dest;		/* process to receive message */
    		rtInt32		sourceUId;	/* source process user ID */
    		rtUInt16	msgId;		/* user defined message ID */
    		rtUInt8		msgType;	/* unique message type */
    		rtUInt8		responseFlag;	/* response desired/supplied */
    		rtUInt32	typeHdrSize;	/* size of type dependent hdr
						   (always multiple of 8) */
    		rtUInt32	msgBodySize;	/* size of message body */
		} 	rtMsgHeader;

/*
 * message sizes
 */
#define rtMSGSIZE	8192
#define rtMAXMSG	(rtMSGSIZE - sizeof (rtMsgHeader))

/*
 * RTAP message types (member msgType above) which specify the type 
 * and structure of the type dependent headers
 *
 * the message type numbering is as follows:
 *
 *	0-127 	is reserved for HP use:
 *			0 	will be INVALID and not be used
 *			1-63	will be for RTAP internal use by HP
 *			64-127	will be for RTAP public use by HP
 *
 *	128-255	is for users; use any number equal to or greater than
 *		              'rtUSER_MESSAGE'
 */
#define rtACKNOWLEDGE	  	64
#define rtPS_OBITUARY	 	67
#define rtASCII_MESSAGE         71
#define rtUSER_MESSAGE		128

/* the following msg-types are not (yet) used by lqs/qsemu: */
#define rtTK_MESSAGE	 	65
#define rtEVENT_MESSAGE	 	66
#define rtWATCHDOG_UPDATE       68
#define rtCONDITION_REQUEST     69
#define rtENV_LOOKUP_REQUEST	70
#define rtALARM_NOTIFICATION	75
#define rtALARM_CONN_MGMT	76
#define rtALARM_SYSTEM_EVENT    96    /* msgType for alarm system event msg */
#define rtALARM_SYSTEM_ACTION  	97    /* msgType for alarm send action msg  */


/*
 * RTAP error structure
 */
#define rtMAX_ERRORS		10

typedef struct 	{
    		rtInt16		sourceId;
		rtProcessNum	procNum;
		rtUInt8		reserved;
    		rtInt32		errorNumber;
    		rtChar		sourceName[20];
    		rtProcessEnv	envName;
    		rtChar		progName[20];
    		rtChar		functionName[80];
    		rtChar		errorString[128];
    		rtChar		messageString[256];
    		rtChar		location[80];
		} 	rtSingleError;

typedef struct 	{
    		rtUInt16 	errorCount;
		rtUInt16	ignoreError;
    		rtUInt32	errorTime;
    		rtSingleError	errorList[rtMAX_ERRORS];
		} 	rtError;

/*
 * rtAcknowledge is used as a type dependant header
 */
typedef struct 	{
	  	rtError		error;
	 	}	rtAcknowledge;




/*
 * process scheduler responses and body of messages
 *
 * this termination notification is sent as a type dependant header
 */
typedef struct  {
    		rtInt32		exitStatus;	/* how it died */
    		rtProcessId	deceased;	/* process that died */
    		rtChar		name[rtMAX_PROC_NAME_LEN];	/* process basename */
    		rtInt8		restart;	/* will it be restarted ? */
    		rtInt8		debug;		/* was it being debugged ? */
                rtUInt8         reserved[4];
		} 	rtObituary;


#endif /*!LQS_RTAP_H*/
