#ifndef LSF_DEVICE_H
#define LSF_DEVICE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsfDevice.h,v 1.121 2005/09/14 07:02:56 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-05-02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "lsfDefines.h"

/*
 * Define some types
 */
typedef vltINT32    lsfSTATE;
typedef vltINT32    lsfSUBSTATE;

typedef struct
{
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;
} lsfDEVICE_ENTRY;

typedef struct
{
    void            *ctrlData;
    void            *userData;
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;
    vltINT32         numDevices;
    vltBYTES20      *deviceNames;
    char            *attrTable[lsfMAX_DEVICES+1];
    lsfDEVICE_ENTRY *deviceTable[lsfMAX_DEVICES];
} lsfDEVICE_DATA;

vltLOGICAL    lsfDeviceIsDefined   ( IN  void       *pData, 
				     IN  const char *devName );
vltLOGICAL    lsfDeviceIsIgnored   ( IN  void       *pData, 
				     IN  const char *devName );
vltLOGICAL    lsfDeviceIsSimulated ( IN  void       *pData, 
				     IN  const char *devName );
vltLOGICAL    lsfDeviceIsInState   ( IN  void       *pData, 
				     IN  lsfSTATE    state );

ccsCOMPL_STAT lsfDeviceSetStatus   ( IN  void       *pData,
				     IN  vltINT32    devIdx,
				     IN  lsfSTATE    state,
				     IN  lsfSUBSTATE subState,
				     IN  vltINT32    init,
				     OUT ccsERROR   *error );
ccsCOMPL_STAT lsfDeviceSetState    ( IN  void       *pData,
				     IN  vltINT32    devIdx,
				     IN  lsfSTATE    state,
				     OUT ccsERROR   *error );
ccsCOMPL_STAT lsfDeviceSetSubState ( IN  void       *pData,
				     IN  vltINT32    devIdx,
				     IN  lsfSUBSTATE subState,
				     OUT ccsERROR   *error );
ccsCOMPL_STAT lsfDeviceSetInit     ( IN  void       *pData,
				     IN  vltINT32    devIdx,
				     IN  vltLOGICAL  init,
				     OUT ccsERROR   *error );
lsfSTATE      lsfDeviceGetState    ( IN  void     *devPtr,
				     IN  vltINT32  devIdx);
lsfSUBSTATE   lsfDeviceGetSubState ( IN  void     *devPtr,
				     IN  vltINT32  devIdx);
vltLOGICAL    lsfDeviceGetSim      ( IN  void     *devPtr,
				     IN  vltINT32  devIdx);

typedef ccsCOMPL_STAT lsfDEVICE_HOOK( IN  void       *devData, 
				      IN  const char *devName, 
				      OUT ccsERROR   *error);
typedef ccsCOMPL_STAT lsfDEVICE_CONSTRUCTOR ( IN  void       *devData, 
					      IN  const char *devName, 
					      OUT ccsERROR   *error);
typedef void          lsfDEVICE_DESTRUCTOR  ( IN  void       *devData );

ccsCOMPL_STAT lsfDeviceCallHook        ( IN  const char *swdName,
					 IN  void       *devData,
					 IN  const char *devName,
					 IN  const char *devFct,
					 OUT ccsERROR   *error );
ccsCOMPL_STAT lsfConstructorHook       ( IN  const char *swdName,
					 IN  void       *devData,
					 IN  const char *devName,
					 OUT ccsERROR   *error );
void          lsfDestructorHook        ( IN  const char *swdName,
					 IN  void       *devData, 
					 IN  const char *devName );

#endif /*!LSF_DEVICE_H*/
