#ifndef LSF_MOTOR_H
#define LSF_MOTOR_H
/************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: lsfMotor.h,v 1.121 2005/09/14 07:02:57 vltsccm Exp $" 
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-04-19  created
*------------------------------------------------------------------------
*/
#include "lsfControl.h"
#include "mot.h"

typedef struct
{
    lsfSTATE    state;
    lsfSUBSTATE subState;
    vltLOGICAL  init;
    vltLOGICAL  simulation;
    vltINT32    timeout;      /* Timeout in milli-seconds    */
    motSTATUS  *motorStatus;
    vltLOGICAL  twoStepDefined;
} lsfMOTOR_ENTRY;

typedef struct
{
    void          *ctrlData;
    void          *userData;
    lsfSTATE       state;
    lsfSUBSTATE    subState;
    vltLOGICAL     init;
    vltLOGICAL     simulation;
    vltINT32       timeout;      /* Timeout in milli-seconds    */
    vltINT32       numMotors;

    vltBYTES20     *motorNames;
    char           *attrTable[lsfMAX_DEVICES+1];
    lsfMOTOR_ENTRY *motorTable[lsfMAX_DEVICES];

    motHANDLE       motorHandle[lsfMAX_MOTORS+1];
    motSTATUS      *motorStatus;
} lsfMOTOR_DATA;

#define lsfMotorGetData(ptr) \
    (lsfMOTOR_DATA *)lsfControlGetDevData((lsfCONTROL_DATA *)(((lsfCONTROL_DATA *)ptr)->ctrlData),lsfDEVICE_MOTOR)
#define lsfMotorGetUserData(ptr) \
    (lsfMotorGetData(ptr))->userData

ccsCOMPL_STAT lsfMotorConstructor ( IN  const char      *swdName, 
				    IN  lsfCONTROL_DATA *controlData,
				    IN  const char      *motorNames[],
				    OUT ccsERROR        *error );
void          lsfMotorDestructor  ( IN  lsfCONTROL_DATA *controlData );

ccsCOMPL_STAT lsfMotorInit      ( IN  lsfMOTOR_DATA *motorData,
				  IN  const char    *motorName, 
				  IN  vltLOGICAL     wait,
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorStandby   ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorOnline    ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorStop      ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorOff       ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorExit      ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorSimulat   ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorStopsim   ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorSelftest  ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorTest      ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );


ccsCOMPL_STAT lsfMotorWaitInit  ( IN  lsfMOTOR_DATA *motorData,
				  IN  const char    *motorName, 
				  OUT ccsERROR      *error );

vltINT32 lsfMotorGetState       ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName );
vltINT32 lsfMotorGetSubState    ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName );
ccsCOMPL_STAT lsfMotorSetSubState(IN  lsfMOTOR_DATA *motorData,
				  OUT ccsERROR      *error );

ccsCOMPL_STAT lsfMotorMoveAbs   ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  IN  vltDOUBLE      target[], 
				  IN  const char    *unit[], 
				  IN  vltLOGICAL     wait,
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorMoveRel   ( IN  lsfMOTOR_DATA *motorData, 
				  IN  const char    *motorName, 
				  IN  vltDOUBLE      target[], 
				  IN  const char    *unit[], 
				  IN  vltLOGICAL     wait,
				  OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorMoveToName ( IN  lsfMOTOR_DATA *motorData, 
				   IN  const char    *motorName, 
				   IN  const char    *target[], 
				   IN  vltDOUBLE      offset[], 
				   IN  const char    *unit[], 
				   IN  vltLOGICAL     wait,
				   OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorMoveToIndex ( IN  lsfMOTOR_DATA *motorData, 
				    IN  const char    *motorName, 
				    IN  vltINT32       target[], 
				    IN  vltDOUBLE      offset[], 
				    IN  const char    *unit[], 
				    IN  vltLOGICAL     wait,
				    OUT ccsERROR      *error );
ccsCOMPL_STAT lsfMotorWaitMove  ( IN  lsfMOTOR_DATA *motorData,
				  IN  const char    *motorName, 
				  OUT char          *names[], 
				  OUT vltDOUBLE     *position, 
				  OUT char          *unit[], 
				  OUT ccsERROR      *error );

#endif 	/* !LSF_MOTOR_H */
