#ifndef LSF_SERIAL_H
#define LSF_SERIAL_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsfSerial.h,v 1.121 2005/09/14 07:02:58 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-05-04  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "lsfControl.h"

typedef struct
{
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;      /* Timeout in milli-seconds    */

    vltINT32         fd;
    vltLOGICAL       isTyCo;
    vltLOGICAL       isMultiple;
    vltLOGICAL       isPlc;
    void            *plcConfig;
} lsfSERIAL_ENTRY;

typedef struct
{
    void            *ctrlData;
    void            *userData;
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;      /* Timeout in milli-seconds    */
    vltINT32         numSerials;

    vltBYTES20      *serialNames;
    char            *attrTable[lsfMAX_DEVICES+1];
    lsfSERIAL_ENTRY *serialTable[lsfMAX_DEVICES];
} lsfSERIAL_DATA;

typedef ccsCOMPL_STAT lsfSERIAL_HOOK( IN  lsfSERIAL_DATA *serialData, 
				      IN  const char     *serialName, 
				      OUT ccsERROR       *error);

#define lsfSerialGetData(ptr) \
    (lsfSERIAL_DATA *)lsfControlGetDevData((lsfCONTROL_DATA *)(((lsfCONTROL_DATA *)ptr)->ctrlData),lsfDEVICE_SERIAL)
#define lsfSerialGetUserData(ptr) \
    (lsfSerialGetData(ptr))->userData

ccsCOMPL_STAT lsfSerialConstructor ( IN  const char      *swdName, 
				     IN  lsfCONTROL_DATA *controlData,
				     IN  const char      *serialNames[],
				     OUT ccsERROR        *error );
void          lsfSerialDestructor  ( IN  lsfCONTROL_DATA *controlData );

ccsCOMPL_STAT lsfSerialOff       ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialStop      ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialInit      ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialOnline    ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialStandby   ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialExit      ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialSimulat   ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialStopsim   ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialSelftest  ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialTest      ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName, 
				   OUT ccsERROR       *error );

ccsCOMPL_STAT lsfSerialRead      ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   OUT char           *names[],
				   OUT char           *buffer[],
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialReadNoWait( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   OUT char           *names[],
				   OUT char           *buffer[],
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSerialWrite     ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  char           *buffer[],
				   OUT ccsERROR       *error );

ccsCOMPL_STAT lsfSerialClose     ( IN  lsfSERIAL_DATA *serialData,
				   IN  vltINT32        devIdx,
				   OUT ccsERROR       *error );

vltINT32      lsfPLCGetProtocol  ( IN  lsfSERIAL_ENTRY *s );
const char *lsfPLCGetProtocolName( IN  lsfSERIAL_ENTRY *s );
ccsCOMPL_STAT lsfPLCGetStatus    ( IN  lsfSERIAL_ENTRY *s );

ccsCOMPL_STAT lsfRK512Get        ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  vltINT32        dataIndex,
				   IN  vltINT32        dataCount,
				   OUT vltUINT16       data[],
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfRK512Set        ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  vltINT32        cmdIndex,
				   IN  vltINT16        cmdValue,
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfRK512Rst        ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  vltINT32        cmdIndex,
				   IN  vltINT16        cmdValue,
				   OUT ccsERROR       *error );

ccsCOMPL_STAT lsfEY2400Synchro   ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfEY2400Presence  ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  vltINT32        plcNumber,
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfEY2400Command   ( IN  lsfSERIAL_DATA *serialData,
				   IN  const char     *serialName,
				   IN  vltINT32        cmdIndex,
				   IN  vltINT16        cmdValue,
				   OUT void           *pData,
				   OUT ccsERROR       *error );

#endif /*!LSF_SERIAL_H*/
