#ifndef LSF_SERVER_H
#define LSF_SERVER_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsfServer.h,v 1.121 2005/09/14 07:02:56 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-04-18  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "lsfDevice.h"

/*
 * Task spawn parameters for lsfServer
 */ 
#define lsfSERVER_PRIORITY  80
#define lsfSERVER_OPTIONS   VX_FP_TASK
#define lsfSERVER_STACK	    60

#define lsfMONITOR_PRIORITY 75
#define lsfMONITOR_STACK    30

typedef struct
{
    void        *ctrlData;
    lsfSTATE     state;        /* module state                   */
    lsfSUBSTATE  subState;     /* module sub-state               */
    vltLOGICAL   init;         /* flag if initialized            */
    vltLOGICAL   simulation;   /* flag if in simulation          */
    vltINT32     numDevices;   /* number of devices              */
    SEM_ID       cmdSem;       /* semaphore for sending commands */
    vltINT32     cmdTimeout;   /* timeout for sending commands   */
    SEM_ID       initSem;      /* semaphore for init command     */
    vltLOGICAL   lccDevice;    /* SW device is a LCC device      */
    int          monTid;       /* Monitoring Task Id             */
    SEM_ID       monSem;       /* semaphore for monitor period   */
    int          monPeriod;    /* Monitoring Task Period (ticks) */
    ccsERROR     error;        /* error stack for cmdInit()      */
    char        *attrTable;    /* cai DB attribute table         */
    vltBYTES20   swdName;      /* Name of the SW device          */
    void        *fctHook[10][3];
    vltLOGICAL   serverInit;
    void        *userData;
} lsfSERVER_DATA;

typedef enum
{
    lsfHOOK_INIT = 0,
    lsfHOOK_STANDBY,
    lsfHOOK_ONLINE,
    lsfHOOK_STOP,
    lsfHOOK_OFF,
    lsfHOOK_EXIT,
    lsfHOOK_SIMULAT,
    lsfHOOK_STOPSIM,
    lsfHOOK_SELFTST,
    lsfHOOK_TEST,
    lsfHOOK_LAST
} lsfHOOKFUNC_IDX;

typedef enum
{
    lsfHOOK_PRO = 0,
    lsfHOOK_MID,
    lsfHOOK_EPI
} lsfHOOKTYPE_IDX;

typedef ccsCOMPL_STAT lsfSTD_FUNCTION_HOOK ( IN  lsfSERVER_DATA *serverData, 
					     OUT ccsERROR       *error);

#define lsfSTD_COMMAND_HOOK(m,c,h) \
    ccsCOMPL_STAT m ## Std ## c ## h ## Hook ( \
         IN  lsfSERVER_DATA *srvData, OUT ccsERROR *error)

void          lsfServerGetHooks       ( IN  lsfSERVER_DATA *serverData );
ccsCOMPL_STAT lsfServerCallHook       ( IN  lsfSERVER_DATA *serverData,
					IN  lsfHOOKFUNC_IDX cmd, 
					IN  lsfHOOKTYPE_IDX typ,
					OUT ccsERROR       *error );

ccsCOMPL_STAT lsfServerInitDb         ( IN  const char      *swdName, 
					OUT ccsERROR        *error );

ccsCOMPL_STAT lsfInitAll              ( IN  const char      *swdName, 
					OUT lsfSERVER_DATA **pServerData );

void          lsfServerDestructor     ( IN  lsfSERVER_DATA *serverData );

ccsCOMPL_STAT lsfServerReadConfig     ( IN  lsfSERVER_DATA *serverData,
					OUT ccsERROR       *error );

vltLOGICAL    lsfServerGetInit        ( IN  lsfSERVER_DATA *serverData );
ccsCOMPL_STAT lsfServerSetInit        ( IN  lsfSERVER_DATA *serverData,
					IN  vltLOGICAL      value, 
					OUT ccsERROR       *error );

vltLOGICAL    lsfServerGetSim         ( IN  lsfSERVER_DATA *serverData ); 
ccsCOMPL_STAT lsfServerSetSim         ( IN  lsfSERVER_DATA *serverData,
					IN  vltLOGICAL      value, 
					OUT ccsERROR       *error );

lsfSTATE      lsfServerGetState       ( IN  lsfSERVER_DATA *serverData );
const char   *lsfServerGetStateName   ( IN  lsfSERVER_DATA *serverData );

lsfSUBSTATE   lsfServerGetSubState    ( IN  lsfSERVER_DATA *serverData );
const char   *lsfServerGetSubStateName( IN  lsfSERVER_DATA *serverData );

ccsCOMPL_STAT lsfServerTakeInitSem    ( IN  lsfSERVER_DATA *serverData );
void          lsfServerGiveInitSem    ( IN  lsfSERVER_DATA *serverData );

ccsCOMPL_STAT lsfServerTakeCmdSem     ( IN  lsfSERVER_DATA *serverData );
void          lsfServerGiveCmdSem     ( IN  lsfSERVER_DATA *serverData );

void          lsfServerWakeupMonitor  ( IN  lsfSERVER_DATA *serverData );

vltINT32      lsfServerGetCmdTimeout  ( IN  lsfSERVER_DATA *serverData );
void          lsfServerSetCmdTimeout  ( IN  lsfSERVER_DATA *serverData,
					IN  vltINT32        timeout );

ccsCOMPL_STAT lsfServerSwitchState    ( IN  lsfSERVER_DATA *serverData,
					IN  lsfSTATE        newDevState,
					OUT ccsERROR       *error );

ccsCOMPL_STAT lsfServerCheckOnline    ( IN  lsfSERVER_DATA *serverData,
					OUT vltLOGICAL     *online,
					OUT ccsERROR       *error );

typedef void  lsfTASK_FUNCTION        ( IN  void           *parameter,
					IN  SEM_ID         *syncSem,
					IN  ccsCOMPL_STAT  *status,
					OUT ccsERROR       *error );

int           lsfStartupTask          ( IN  char           *name,
					IN  vltINT32        priority,
					IN  vltINT32        stackSize,
					IN  FUNCPTR         entryPoint,
					IN  void           *param,
					OUT ccsERROR       *error );

lsfSERVER_DATA *lsfServerGetGlobalData( IN  const char     *swdName,
					OUT ccsERROR       *error );
ccsCOMPL_STAT lsfServerGetSwDeviceName( OUT char           *swdName,
					OUT ccsERROR       *error );
vltLOGICAL    lsfServerIsALccDevice   ( IN  lsfSERVER_DATA *serverData );

void          lsfVersion              ( OUT vltBYTES80      version, 
					IN  const char     *modName, 
					IN  const char     *modTitle, 
					IN  const char     *date, 
					IN  const char     *time, 
					IN  const char     *modRcs );
#endif /*!LSF_SERVER_H*/
