#ifndef LSF_SIGNAL_H
#define LSF_SIGNAL_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsfSignal.h,v 1.121 2005/09/14 07:02:57 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-05-04  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "lsfControl.h"

typedef struct
{
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;      /* Timeout in milli-seconds    */

    ioTYPE           ioType;
    ioIN_OUT         direction;
    dbSYMADDRESS     dbAddr;
    ioDIRADDRESS     ioAddr;
} lsfSIGNAL_ENTRY;

typedef struct
{
    void            *ctrlData;
    void            *userData;
    lsfSTATE         state;
    lsfSUBSTATE      subState;
    vltLOGICAL       init;
    vltLOGICAL       simulation;
    vltINT32         timeout;      /* Timeout in milli-seconds    */
    vltINT32         numSignals;

    vltBYTES20      *signalNames;
    char            *attrTable[lsfMAX_DEVICES+1];
    lsfSIGNAL_ENTRY *signalTable[lsfMAX_DEVICES];
} lsfSIGNAL_DATA;

typedef ccsCOMPL_STAT lsfSIGNAL_HOOK( IN  lsfSIGNAL_DATA *signalData, 
				      IN  const char     *signalName, 
				      OUT ccsERROR       *error);

#define lsfSignalGetData(ptr) \
    (lsfSIGNAL_DATA *)lsfControlGetDevData((lsfCONTROL_DATA *)(((lsfCONTROL_DATA *)ptr)->ctrlData),lsfDEVICE_SIGNAL)
#define lsfSignalGetUserData(ptr) \
    (lsfSignalGetData(ptr))->userData

ccsCOMPL_STAT lsfSignalConstructor ( IN  const char      *swdName, 
				     IN  lsfCONTROL_DATA *controlData,
				     IN  const char      *signalNames[],
				     OUT ccsERROR        *error );
void          lsfSignalDestructor  ( IN  lsfCONTROL_DATA *controlData );

ccsCOMPL_STAT lsfSignalOff       ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalStop      ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalInit      ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalOnline    ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalStandby   ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalExit      ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalSimulat   ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalStopsim   ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalSelftest  ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalTest      ( IN  lsfSIGNAL_DATA *signalData,
				   IN  const char     *signalName, 
				   OUT ccsERROR       *error );

ccsCOMPL_STAT lsfSignalReadAnalog  ( IN  lsfSIGNAL_DATA *signalData,
				     IN  const char     *signalName,
				     OUT char           *names[],
				     OUT vltFLOAT       *value, 
				     OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalWriteAnalog ( IN  lsfSIGNAL_DATA *signalData,
				     IN  const char     *signalName,
				     IN  vltFLOAT       *value, 
				     OUT ccsERROR       *error );

ccsCOMPL_STAT lsfSignalReadDigital ( IN  lsfSIGNAL_DATA *signalData,
				     IN  const char     *signalName,
				     OUT char           *names[],
				     OUT vltUINT32      *value, 
				     OUT ccsERROR       *error );
ccsCOMPL_STAT lsfSignalWriteDigital( IN  lsfSIGNAL_DATA *signalData,
				     IN  const char     *signalName,
				     IN  vltUINT32      *value, 
				     OUT ccsERROR       *error );

#endif /*!LSF_SIGNAL_H*/
