#ifndef LSF_TASKDEV_H
#define LSF_TASKDEV_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsfTaskDev.h,v 1.121 2005/09/14 07:02:58 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-04-05  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#include "lsfControl.h"

#define lsfTIMER_PRIORITY  20
#define lsfTIMER_STACK	    5

typedef struct
{
    lsfSTATE      state;
    lsfSUBSTATE   subState;
    vltLOGICAL    init;
    vltLOGICAL    simulation;
    vltINT32      timeout;          /* Timeout in system ticks     */

    lsfSTATE      startState;       /* Minimum state for schedule  */
    vltINT32      priority;         /* Task priority               */
    vltLOGICAL    ignoreStop;       /* True when stop ignored      */
    vltLOGICAL    first;            /* True when starting          */
    vltINT32      tcb;              /* Task TCB for lccTaskLib     */
    vltINT32      tid;              /* Task Device Id              */
    vltINT32      tmrTid;           /* Timer Task Id               */
    SEM_ID        tskSem;           /* Task Device semaphore       */
    SEM_ID        tmrSem;           /* Timer Task semaphore        */
    void         *fctHook[3];       /* Task hooks Pro, Core, Epi   */
                                    /*  -> Core must exist         */
    vltINT32      maxNumOverTime;
    vltINT32      numOverTime;
    vltINT32      numIteration;
} lsfTASKDEV_ENTRY;

typedef struct
{
    void              *ctrlData;
    void              *userData;
    lsfSTATE           state;
    lsfSUBSTATE        subState;
    vltLOGICAL         init;
    vltLOGICAL         simulation;
    vltINT32           timeout;          /* Timeout in system ticks     */
    vltINT32           numTaskDevs;

    vltBYTES20        *taskDevNames;
    char              *attrTable[lsfMAX_DEVICES+1];
    lsfTASKDEV_ENTRY  *taskDevTable[lsfMAX_DEVICES];
} lsfTASKDEV_DATA;

typedef ccsCOMPL_STAT lsfTASKDEV_HOOK ( IN  lsfTASKDEV_DATA *taskDevData, 
				        IN  const char      *taskDevName, 
				        OUT ccsERROR        *error);

#define lsfTaskDevGetData(ptr) \
    (lsfTASKDEV_DATA *)lsfControlGetDevData((lsfCONTROL_DATA *)(((lsfCONTROL_DATA *)ptr)->ctrlData),lsfDEVICE_TASKDEV)
#define lsfTaskDevGetUserData(ptr) \
    (lsfTaskDevGetData(ptr))->userData

ccsCOMPL_STAT lsfTaskDevConstructor ( IN  const char      *swdName, 
				      IN  lsfCONTROL_DATA *controlData,
				      IN  const char      *taskDevNames[],
				      OUT ccsERROR        *error );
void          lsfTaskDevDestructor  ( IN  lsfCONTROL_DATA *controlData );

ccsCOMPL_STAT lsfTaskDevInit      ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevStandby   ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevOnline    ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevStop      ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevOff       ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevExit      ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevSimulat   ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevStopsim   ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevSelftest  ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevTest      ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
ccsCOMPL_STAT lsfTaskDevTrigger   ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );
STATUS        lsfTaskDevVerify    ( IN  lsfTASKDEV_DATA *taskDevData, 
				    IN  const char      *taskDevName, 
				    OUT ccsERROR        *error );

#endif /*!LSF_TASKDEV_H*/

