/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: mac4.h,v 3.15 2005/09/22 15:03:49 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* CAM GmbH  --/05/94  Created  for Release 1.1
* P.Duhoux  --/07/94  Modified for Release 1.2
* P.Duhoux  09/09/94  Renamed mac4VERSION_NUMBER to mac4VERSION
*                     and changed its definition
* --------  --------  ----------------------------------------------
* P.Duhoux  18/11/94  Adapted  for Release 2.1
*                     NO DOWNWARDS COMPATIBILITY
* P.Duhoux  21/12/94  motInternal.h now includes motDatabase.h
* P.Duhoux  09/06/95  Modified for CMM Archive
* P.Duhoux  13/09/95  Added include motPublic.h (not anymore in sdlPublic.h)
* P.Duhoux  29/11/95  Added function mac4Deinstall
* P.Duhoux  18/03/96  Adapted for Release 4.0 (external encoder support)
* P.Duhoux  20/03/96  Modified interface of mac4ConnectIsr
*                     - Changed parameter order                     
* P.Duhoux  29/03/96  Added function mac4WriteEncValue
*/

#ifndef MAC4_H
#define MAC4_H

/******************************************************************************
 * mac4.h - MACCON Motion Controller single device library interface file
 *---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

/* MCM Common Public include file */
#include "motPublic.h"

/* SDL Common Public include file */
#include "sdlPublic.h"

/* MAC4 include files */
#include "mac4Defines.h"
#include "mac4Errors.h"

/* motion controller module name */
#define mac4MODULE         "mac4"
#define mac4MODULE_NAME    "Motion Controller SDL"

/* motion controller module version */
#define mac4VERSION        "$Revision: 3.15 $"
#define mac4VERSION_DATE   "APR2004"

/*
 * function prototypes
 */
ccsCOMPL_STAT mac4Install       (motDESCRIPTION *motDesc, ccsERROR *error);
ccsCOMPL_STAT mac4Deinstall     (motDESCRIPTION *motDesc, ccsERROR *error);

ccsCOMPL_STAT mac4CheckError    (motDESCRIPTION *motDesc, vltBYTES256 errMsg, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4ConnectIsr    (motDESCRIPTION *motDesc, vltINT32 type, 
				 vltBYTES32 function, vltINT32 argument, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4Disable       (motDESCRIPTION *motDesc, vltUINT32 mask,
				 ccsERROR *error);
ccsCOMPL_STAT mac4Enable        (motDESCRIPTION *motDesc, vltUINT32 mask,
				 ccsERROR *error);
ccsCOMPL_STAT mac4GetStatus     (motDESCRIPTION *motDesc, 
			         motSTATUS *statusBuf, ccsERROR *error);
ccsCOMPL_STAT mac4InitHw        (motDESCRIPTION *motDesc, vltINT32 initCode,
				 vltINT32 encoderValue, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4InitSw        (motDESCRIPTION *motDesc, 
				 motENCODER *encoder, ccsERROR *error);
ccsCOMPL_STAT mac4LoadPar       (motDESCRIPTION *motDesc, motPARAMETER *parBuf,
				 vltUINT32 *pNumber, vltLOGICAL all, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4RetrievePar   (motDESCRIPTION *motDesc, motPARAMETER *parBuf,
				 vltUINT32 *pNumber, vltLOGICAL all, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveAbs       (motDESCRIPTION *motDesc, 
				 vltINT32 position, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveRel       (motDESCRIPTION *motDesc, 
				 vltINT32 position, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveSpeed     (motDESCRIPTION *motDesc, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveTracking  (motDESCRIPTION *motDesc, motPOS_TYPE type,
				 vltINT32 position, vltDOUBLE velocity, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveLowLim    (motDESCRIPTION *motDesc, vltDOUBLE speed,
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveUpLim     (motDESCRIPTION *motDesc, vltDOUBLE speed,
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveToHome    (motDESCRIPTION *motDesc, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveToInd     (motDESCRIPTION *motDesc, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4MoveToRef     (motDESCRIPTION *motDesc, vltDOUBLE speed, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4ResetAxis     (motDESCRIPTION *motDesc, ccsERROR *error);
ccsCOMPL_STAT mac4ResetBoard    (motDESCRIPTION *motDesc, ccsERROR *error);
ccsCOMPL_STAT mac4SetMode       (motDESCRIPTION *motDesc, motMOTIONMODE mode, 
				 ccsERROR *error);
ccsCOMPL_STAT mac4SetOpMode     (motDESCRIPTION *motDesc, motOPMODE mode,
				 ccsERROR *error);
ccsCOMPL_STAT mac4SetTimeout    (motDESCRIPTION *motDesc, vltUINT32 timeout, 
				 ccsERROR *error); 
ccsCOMPL_STAT mac4SetEncValue   (motDESCRIPTION *motDesc, vltINT32 initPos,
				 ccsERROR *error);
ccsCOMPL_STAT mac4SetLowSwLimit (motDESCRIPTION *motDesc, 
				 vltINT32 *pLimVal, ccsERROR *error);
ccsCOMPL_STAT mac4SetUpSwLimit  (motDESCRIPTION *motDesc, 
				 vltINT32 *pLimVal, ccsERROR *error);
ccsCOMPL_STAT mac4Stop          (motDESCRIPTION *motDesc, ccsERROR *error);
ccsCOMPL_STAT mac4WriteEncValue (motDESCRIPTION *motDesc, vltINT32 encValue,
				 ccsERROR *error);

void          mac4Version       (vltBYTES80 version);

#ifdef __cplusplus
}
#endif

#endif /* !MAC4_H */

/*************************************************************************/

/*___oOo___*/
