/*
 *            Copyright (c) 1996 - 2004 by electronic system design gmbh
 *
 *  This software is copyrighted by and is the sole property of 
 *  esd gmbh.  All rights, title, ownership, or other interests
 *  in the software remain the property of esd gmbh. This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of esd gmbh.
 *
 *  esd gmbh, reserves the right to modify this software without notice.
 *
 *  electronic system design gmbh          Tel. +49-511-37298-0
 *  Vahrenwalder Str 205                   Fax. +49-511-37298-68 
 *  30165 Hannover                         http://www.esd-electronics.com
 *  Germany                                sales@esd-electronics.com
 *
 *************************************************************************
 *
 *     Module Name: mcanopen.h
 *         Version: 1.00
 *   Original Date:  18-Sep-1996
 *     Host system: N/A
 *   Target system: N/A
 *          Author: 
 *        Language: ANSI C
 * Compile Options: 
 * Compile defines: WIN32
 *                     Include on Win 9x/NT/2K/XP
 *
 * Description.
 * =======================================================================
 * API for CANopen manager applications according to CiA standard 
 * DS-301 Rev. 4.x, DS-302 Rev. 3.x and DS-306 Rev. 1.x
 *
 */

#ifndef _mcanopen_h_
#define _mcanopen_h_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
# pragma comment(lib,"comanage.lib")	/* Link with COMANAGE.LIB (Only for VC++) */
#include <windows.h>
# ifndef EXPORT
#  define EXPORT __declspec (dllexport)  /* Macro for DLL functions */
# endif
#else 
# ifndef EXPORT
#  undef EXPORT
#  define EXPORT
# else
#  define EXPORT
# endif
#endif

#if !defined(CALLTYPE)
# define CALLTYPE
#endif

/* control flags of 'canOpenNodeControl()' to give back node control */
#define SET_NODE_STATE_CONFIGURED  0x00000001 /* do not configure Node again */
#define GET_NODE_CONTROL           0x00000002 /* if !set: give control back */
#define START_EXTENDED_BOOT        0x00000004 /* treat as CAL-device        */

/* Predefined dictionary object types (buffer-types)*/
#define TYP_CHAR                         1
#define TYP_INT8                         2
#define TYP_INT16                        3
#define TYP_INT32                        4
#define TYP_UINT8                        5 
#define TYP_UINT16                       6
#define TYP_UINT32                       7
#define TYP_FLOAT                        8
#define TYP_VIS_STRING                   9
#define TYP_OCT_STRING                  10
#define TYP_DATE                        11
#define TYP_TIME_OF_DAY                 12
#define TYP_TIME_DIFFERENCE             13
#define TYP_DOMAIN                      15

/* Define transmission type logical or'd with data type, overrides defaults */
#define TYP_INDICATE_SIZE               0x80000000
#define TYP_INDICATE_NO_SIZE            0x40000000
#define TYP_CONVERT_ENDIANNESS          0x00008000

/* typ of source or destination */
#define DCF_CONFIG_FILE                 0x0000
#define EDS_CONFIG_FILE                 0x0001
#define DCF_PEDANTIC                    0x8000
#define EDS_PEDANTIC                    DCF_PEDANTIC
#define DCF_DECIMAL                     0x4000
#define EDS_DECIMAL                     DCF_DECIMAL
#define USER_SECTIONS				    0x2000
#define NO_CLEANUP_ON_ERROR             0x1000
#define DCF_NO_NODEID_CHECK	            0x0800
#define SIGNAL_PROGRESS_EVENTS          0x0400
#define DCF_IGNORE_MISSING_PARAMETER    0x0200

/*
 * DS-301 V 4.x / DS-302 V 3.x defined node states returned from
 * mcanOpenGetNodeState()
 */
#define NODE_STATE_UNKNOWN				0x00
#define NODE_MISSING					0x01
#define NODE_STATE_STOPPED				0x04
#define NODE_STATE_OPERATIONAL			0x05
#define NODE_STATE_PREOPERATIONAL		0x7F

/*
 * CAL node states included for backward compatibility. No longer
 * supported since version 2.x of the manager
 */
#define NODE_STATE_DISCONNECTED      	0x01
#define NODE_STATE_CONNECTING        	0x02
#define NODE_STATE_CONNECTED         	0x02
#define NODE_STATE_PREPARING         	0x03
#define NODE_STATE_PREPARED          	0x04


/* return values */
#define MCAL_EXERR                       -4 
#define MCAL_PARA_ERROR                  -2
#define MCAL_SYS_ERROR                   -1
#define MCAL_OK                           0
#define MCAL_CAN_RX_TIMEOUT               1
#define MCAL_CAN_TX_TIMEOUT         	 2
#define MCAL_CAN_TX_ERROR           	 4
#define MCAL_CAN_CONTR_OFF_BUS      	 5
#define MCAL_CAN_CONTR_BUSY               6
#define MCAL_CAN_CONTR_WARN      		  7
#define MCAL_NO_NET                     132
#define MCAL_WRONG_NODE_STATE           134
#define MCAL_OBJ_NO_MEMORY              157
#define MCAL_DOWNLOAD_ABORT_SLAVE       171    /* -> Slave initiated abort */
#define MCAL_WRONG_SLAVE_ANSWER         186
#define MCAL_WRONG_SERVER_ANSWER        193
#define MCAL_TOGGLE_ERROR               194 
#define MCAL_BUFFER_OVERFLOW            195
#define MCAL_ABORT_DOMAIN               196
#define MCAL_NODE_NOT_EXISTEND          201
#define MCAL_WRONG_GUARD_ID             202
#define MCAL_CONSISTENCY_ERROR          203    /* -> fileDATA != slaveDATA  */

#define MCAL_NEW_GUARD_PARA             204
#define MCAL_NODE_NOT_FREE              205
#define MCAL_NO_EVENT                   206
#define MCAL_FILE_READ_ERR              207
#define MCAL_FILE_OPEN_ERR              208
#define MCAL_UNSUPPORTED		209
#define CANOPEN_DCF_WRONG_TYP           256
#define CANOPEN_DCF_INIT_ERROR          257
#define CANOPEN_DCF_OUT_OF_MEMORY       258
#define CANOPEN_DCF_WRONG_NET_ID        259
#define CANOPEN_DCF_WRONG_NODE_ID       260
#define CANOPEN_DCF_WRONG_BAUDRATE      261
#define CANOPEN_DCF_MISSING_SECTION     262
#define CANOPEN_DCF_MISSING_ENTRY       263
#define CANOPEN_WRONG_NET_NO            264
#define CANOPEN_WRONG_MODULE_NO         265
#define CANOPEN_DCF_WRONG_NODE_NAME     266
#define CANOPEN_DCF_WRONG_PARAMETER     267
#define CANOPEN_DCF_OUT_OF_RANGE        268
#define CANOPEN_DCF_OUT_OF_LIMITS       269
#define CANOPEN_DCF_DOUBLE_SECTION      270
#define CANOPEN_DCF_DOUBLE_KEYWORD      271
#define CANOPEN_DCF_UNKNOWN_KEYWORD     272
#define CANOPEN_DCF_TRAILING_WHITESPACE 273
#define CANOPEN_DCF_OBSOLETE_KEYWORD    274
#define CANOPEN_DCF_ILLEGAL_KEYWORD     275

/*+++++++++++++++++++++++ events in mcanOpenEvent +++++++++++++++++++++++++++*/

/* No parameter */                         
#define DUPLICATE_NET       0x40000000 /* already installed                 */
#define ERR_CREATE_NET      0x40000001 /* error creating net                 */
#define NO_GUARD_PARA       0x0002 /* Guard- or lifetime missing, no guarding*/
#define GID_NOT_FREE        0x0003 /* No GuardID free, no guarding           */
#define ERR_SDO_HANDLE      0x0004 /* Open-error for default SDO             */
#define ERR_CREATE_NOD      0x0005 
#define NO_DOWNLOAD_FILE    0x0006 /* Download file not accessable           */
#define NO_DOWNLOAD_SPACE   0x0007 /* Not enough space                       */
#define MCAL_ABORT_SLAVE    0x0008 /* Slave aborted download                 */
#define WRONG_ANSWER        0x0009 /* Download: wrong slave-answer           */
#define ERR_WDG_INST        0x000a /* Installing watchdog failed,no guarding */
#define WRONG_MODID         0x000b /* NodeID != moduleID */
#define NOD_REMOVED         0x000c /* Removed from Nodelist                  */
#define WRONG_MASTER_VER    0xC000000d /* Master too old to interpret file   */
#define FILE_CHECKSUM_ERR   0xC000000e /* Wrong checksum, possibly no cfgfile*/
#define NODE_STARTED        0x000f /* Node started                           */
#define NODE_CONNECTED      0x0010 /* Node connected                         */
#define NODE_PREPARED       0x0011 /* Node prepared                          */
#define ENTRY_1000          0x0012 /* Wrong entry index 0x1000               */
#define TERM_ROUTE          0x40000013 /* route-task terminated              */
#define TERM_GUARD          0x40000014 /* guard-task terminated              */
#define TERM_DBT            0x40000015 /* dbt-task terminated                */
#define TERM_SYNC           0x40000016 /* sync-task terminated               */
#define TERM_DAEMON         0x40000017 /* daemon-task terminated             */
#define TERM_EMCY           0x40000018 /* emergency-frame task terminated    */
#define PARAMETER_SCAN      0x40000019 /* Scanning Node's parameter failed   */
#define TERM_MANAGER        0xC0000020 /* Manager terminated                 */

/* para.c */
#define GUARD_STATE_ERROR   0x0100 /* Guarding returns wrong Node-state ...  */

/* para.uc */
#define ERR_CONF_LTFAC      0x0200 /* Configuring lifetime net... node...    */
				   /* denied! lifetime set to ...            */

/* para.i */                              /* parameter "line" */
#define SYNTAX_ERR          0xC0001000 /* Syntax error in                */
#define UNEXP_CMD           0xC0001001 /* Unexpected Command in          */
#define PATH_ERR            0xC0001002 /* Path too long in               */
#define WRONG_NET_NO        0xC0001003 /* Wrong net number in            */
#define WRONG_BAUD          0xC0001004 /* Wrong baudrate in              */
#define WRONG_AUTOSCAN_TIM  0xC0001005 /* Wrong AUTOSCAN time in         */
#define WRONG_MOD_NO        0xC0001006 /* Wrong module number in         */
#define WRONG_NOD_NAM       0xC0001007 /* Wrong node name given in       */
#define WRONG_NOD_ID        0xC0001008 /* Wrong node ID given in         */
#define WRONG_GUARD_TIM     0xC0001009 /* Wrong guardtime in             */
#define WRONG_GUARD_ID      0xC000100a /* Wrong guardID in               */
#define WRONG_LT_FAC        0xC000100b /* Wrong lifetime factor in       */
#define WRONG_MIN_INH       0xC000100c /* Wrong min. inhibit time in     */
#define WRONG_PRE_DEF       0xC000100d /* Wrong predefinition in         */
#define WRONG_COB_ID        0xC000100e /* Wrong COBID in                 */
#define WRONG_COB_LEN       0xC000100f /* Wrong COB length in            */
#define WRONG_COB_CLASS     0xC0001010 /* Wrong COB class in             */
#define WRONG_COB_TYP       0xC0001011 /* Wrong COB typ in               */
#define WRONG_SYNC_INH      0xC0001012 /* Wrong SYNC inhibit time in     */
#define WRONG_PERIOD        0xC0001013 /* Wrong TIMESTAMP_PERIOD in      */
#define WRONG_DATA_TYP      0xC0001014 /* Wrong datatype in              */
#define NO_BAUD             0xC0001015 /* Baudrate missing til           */
#define NO_PRE_DEF          0xC0001016 /* Predefinition missing til      */
#define NO_COB_ID           0xC0001017 /* COBID missing til              */
#define NO_COB_LEN          0xC0001018 /* COB length missing til         */
#define NO_COB_CLASS        0xC0001019 /* COB class missing til          */
#define NO_COB_TYP          0xC000101a /* COB typ missing til            */
#define NO_NOD_ID           0xC000101b /* NodeID missing til line        */
#define ERR_CREATE_USR_DEF  0xC000101c /* Can't create Userdefinition    */
#define ERR_CREATE_COB_DEF  0xC000101d /* Can't create COBdefinition     */
#define ERR_CREATE_PRE_DEF  0xC000101e /* Can't create Predefinition     */
#define DUPLICATE_COB_ID    0xC000101f /* COBID allready fixed til       */
#define ID_IGNORED          0xC0001020 /* SYNC-,TIMESTAMP,SET-ID ignored */
#define ID_NOT_FREE         0xC0001021 /* SYNC-, COB-, SET-ID not free   */
#define LINE_TOO_LONG       0xC000102c /* Line too long                  */
#define WRONG_PARAMETER	    0xC000102e /* Invalid para, using defaults   */
#define NO_NET	            0xC000102f /* Net number undefined           */

/* para.i */                               /* other para */
#define GUARD_TOGGLE_ERROR  0x00001022 /* para: no of guard error        */
#define GUARD_TIMEOUT_ERROR 0x00001023 /* para: no of guard error        */
#define GUARD_DISCONN_ERROR 0x00001024 /* Disconnected by guarding       */
#define DOWNLOAD_ERR        0x00001025 /* Download returns state ...     */
#define UNKNOWN_NODE        0x00001026 /* Unknown node found, modID ...  */
#define DUPLICATE_MOD_ID    0x00001027 /* Duplicate Module ID scanned    */
#define DCF_ERROR           0x00001028 /* Error reading DCF, error */ 
#define DIFF_GTIM           0x00001029 /* Different guardtimes, set to   */
#define DIFF_LT_FAC         0x0000102a /* Different lifetimes, set to    */
#define ERR_CONF_NODE       0x0000102b /* Configuring returns            */
#define EVENT_BOOT_SLAVE    0x00001030 /* Boot slave event               */
#define EVENT_STATE_CHANGE  0x00001031 /* Node state change event        */
#define EVENT_BOOTMSG       0x00001032 /* Bootmessage event              */

/* para.ui */                      
#define ERR_CONF_GTIM       0x2000 /* Configuring guardtime net... node...   */
				   /* denied! guardtime set to ...           */
/* para.ul */                          
#define ERR_CONF_GID        0x8000 /* Configuring guardID net... node...     */
				   /* denied! guardID set to ...             */
#define MISS_COM_CYCLE      0x8001 /* Missing CommunicationCycle Period      */

/* para.write */
#define WRITE_DICT      0x00010000 /* Writing denied index.. subindex... err */
#define READ_DICT       0x00010001 /* Reading error index.. subindex... err  */ 
 
/* para_err : par.str */           /* para: error-code: byte_0, byte_1       */
#define PREPARE_ERROR   0x00020000 /* Node returns Prepare-err byte_0,byte_1 */
#define CONNECT_ERROR   0x00020001 /* Node returns Connect-err byte_0,byte_1 */
/* para.str */
#define ERR_OPEN_FILE   0xC0040000 /* Unable to open file...                 */
#define ERR_WRITE_FILE  0xC0040001 /* Unable to write file ...               */
#define DCF_SECTION     0x00040002 /* Missing section ...                    */
#define DCF_ENTRY       0x00040003 /* Missing entry ...                      */
#define DUPLICATE_ID    0x00040004 /* tx-ID already used by ...              */
#define DCF_PARA        0x00040005 /* Missing entry ...                      */
#define DCF_RANGE       0x00040006 /* Parameter out of range                 */
#define DCF_LIMIT       0x00040007 /* Parameter out of limits                */
#define DCF_DUPL_SECT   0x00040008 /* Duplicate section                      */
#define DCF_DUPL_KEYW   0x00040009 /* Duplicate Keyword                      */
#define DCF_UNKN_KEYW   0x0004000a /* Unknown Keyword                        */
/* para_emcy: para.str */
#define EMCY_FRAME      0x00080000 /* Emergency frame byte (0...8)           */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* to mask EVENT_T.event */
#define PARA        0x3FFFFF00
#define NET_NOT_VAL 0x80000000
#define NOD_NOT_VAL 0x40000000
#define PARA_C      0x00000100
#define PARA_UC     0x00000200
#define PARA_I      0x00001000
#define PARA_UI     0x00002000
#define PARA_UL     0x00008000
#define PARA_WRITE  0x00010000
#define PARA_ERR    0x00020000
#define PARA_STR    0x00040000
#define PARA_EMCY   0x00080000


typedef struct { unsigned char eclass;   
                 unsigned char code;
                 unsigned char spec;
                 unsigned char global;
               }  ERROR_T;             /* error code defined by DS 301 */


typedef struct {
	long event;           /* see example below */
	int net;
	int node;
	union { 
		char           c;  
		unsigned char  uc; 
		short          s;  
		unsigned short us; 
		int            i;  
		unsigned int   ui; 
		long           l;  
		unsigned long  ul; 
		struct {     
			unsigned short  index;
			unsigned short  subindex;
			ERROR_T         err;
		}              write;
		struct {
			unsigned short	ErrorCode;
			unsigned char	ErrorRegister;
			unsigned char   ErrorField[5];
		} emcy;
		char           str[64]; 		     
	} para;
} EVENT_T;

/*+++++++++++++++++++ EVENT_T.event is to interpret as follows ++++++++++++++*/
/*                                                                           */
/* most significant 2 bits: indicate if EVENT_T.net or EVENT_T.node is valid */
/* following 22 bits      : indicate the type of union EVENT_T.para to use   */
/* least significant byte: represents the number of event with this specific */
/*                         type of para                                      */
/*                         strings to describe errors see paraInfo.h         */
/*                                                                           */
/* EXAMPLE: EVENT_T.event = 0x00001022                                       */
/* -------  0x00001022 & NET_NOT_VAL == 0    ==> EVENT_T.net is valid        */
/*          0x00001022 & NOD_NOT_VAL == 0    ==> EVENT_T.node is valid       */
/*          0x00001022 & PARA == 0x00001000  ==> EVENT_T.para is valid       */
/*          0x00001000 == PARA_I             ==> EVENT_T.para.i is valid     */
/*          0x00001022 & 0xFF == 0x22        ==> lsb is 0x22, means: error   */
/*                                               is the 34th of errors with  */
/*                                               integer parameter           */
/* result: EVENT_T.net, Event_T.node: guarding toggle error no EVENT_T.para.i*/
/* ------                                                                    */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
 
/*
 * Boot status codes of DS-302 Rev 3.x for failures during boot process
 * signalled as parameter with EVENT_BOOT_SLAVE event
 */

#define BS_BOOT_SUCCESS		    0x00	/* No error in bootup                */
#define BS_SLAVE_REMOVED		0x01	/* Err A: Slave removed from network */
#define BS_SLAVE_NOT_PRESENT	0x02	/* Err B: Access to 0x1000 failed    */
#define BS_DEVTYPE_MISMATCH		0x03	/* Err C: Wrong device type          */
#define BS_VENDOR_MISMATCH		0x04	/* Err D: Wrong vendor id            */
#define BS_PRODUCT_MISMATCH		0x05	/* Err M: Wrong product code         */
#define BS_REVISION_MISMATCH	0x06	/* Err N: Wrong revision number      */
#define BS_SERIAL_MISMATCH		0x07	/* Err O: Wrong serial number        */
#define BS_NO_HEARTBEAT			0x08	/* Err E: No heartbeat received      */
#define BS_NO_GUARD_REPLY		0x09	/* Err F: No guard reply received    */
#define BS_NO_SW_VERSION		0x0A	/* Err G: No expected SW version     */
#define BS_NO_SW_UPDATE 		0x0B	/* Err H: SW update not allowed      */
#define BS_SW_UPDATE_FAILED		0x0C	/* Err I: SW update failed           */
#define BS_CFG_DOWNLOAD_FAILED	0x0D    /* Err J: Config download failed     */
#define BS_NO_HEARTBEAT2		0x0E    /* Err K: No heartbeat received      */
#define BS_ALREADY_OPERATIONAL	0x0F    /* Err L: Slave already operational  */
#define BS_FIRST_ATTEMPT        0xFD	/* 1st attempt to boot opt. slave    */
#define BS_TIME_OUT		        0xFE	/* Boot process timed out            */
#define BS_UNKNOWN		        0xFF	/* Unknown status                    */


/************************************************************************/
/*            Prototypes of basic Master-functions                      */
/*                                                                      */
/*    USAGE: 1) Initialise manager with mcanOpenInitialize()            */
/*           2) Create event handler thread and wait for events with    */ 
/*              mcanOpenEvent()                                         */
/*           3) Configure manager with mcanOpenConfig()                 */
/*           4) Start manager with mcanOpenStart()                      */
/*           5) Terminate manager with mcanOpenShutDown()               */
/*                                                                      */
/************************************************************************/

EXPORT int CALLTYPE mcanOpenInitialize(void);
/* 
DESCRIPTION
Initialize CANopen manager
*/

EXPORT int CALLTYPE mcanOpenCleanup(void);
/* 
DESCRIPTION
Remove all objects created in mcanOpenInitialize()
*/

EXPORT int CALLTYPE mcanOpenGetVersion(unsigned short  *version);
/*
DESCRIPTION
bit 12...15 : Level
bit 8..11   : Revision
bit 0..7    : Change
*/

EXPORT int CALLTYPE mcanOpenConfig(char*  cfgFile);
/* 
DESCRIPTION
Read master configuration file
*/


EXPORT void CALLTYPE mcanOpenStart(void);
/* 
DESCRIPTION
Start master daemons
*/

EXPORT int CALLTYPE mcanOpenEvent(EVENT_T *event, int wait);
/* 
DESCRIPTION
Waits (wait = TRUE) or polls for the next network event in event FIFO
*/  


EXPORT int CALLTYPE mcanOpenGetNodeState(int net, int nodeID, int *state);
/*
DESCRIPTION
Returns node state remote CANopen slave
*/


EXPORT int CALLTYPE mcanOpenShutDown(void);
/* 
DESCRIPTION
Terminate master
*/

/************************************************************************/
/*                      Prototypes of additional functions              */
/*                                                                      */
/* NOTE ! NOTE !  NOTE ! NOTE !  NOTE ! NOTE !  NOTE ! NOTE !  NOTE !   */
/*                                                                      */
/*      !  Using additional functions (s.b.) while master           !   */
/*      !  Daemon is started may cause problems.                    !   */
/*      !  Only access nodes after taking Node-Control by using     !   */
/*      !  mcanOpenNodeControl()                                    !   */
/*                                                                      */
/*                                                                      */
/************************************************************************/

EXPORT int CALLTYPE mcanOpenNodeControl(int net, int nodeID, long flags);
/*
DESCRIPTION
Takes node control, if master-Daemon does not use node 'nodeID'
on net 'net' or gives node's control back to master-Daemon (dependent on flags)

NOTE: valid values of flags s.a.
*/

EXPORT int CALLTYPE mcanOpenStartNode(int net,int nodeID);
/*
DESCRIPTION
sends Start_Remote_Node NMT Command
*/

EXPORT int CALLTYPE mcanOpenStopNode(int net,int nodeID);
/*
DESCRIPTION
sends Stop_Remote_Node NMT Command
*/

EXPORT int CALLTYPE mcanOpenResetNode(int net,int nodeID);
/*
DESCRIPTION
sends Reset_Remote_Node NMT Command
*/

EXPORT int CALLTYPE mcanOpenEnterPreop(int net,int nodeID);
/*
DESCRIPTION
sends Enter_Pre-Operational_State NMT Command
*/


EXPORT int CALLTYPE mcanOpenReadSDO(unsigned short net, unsigned short modID,
				    int index, int subindex, void *buffer,
				    int *len, ERROR_T* error); 
/*
DESCRIPTION
Copys the content of 'modID's' Object Dictionary at 'index','subindex' 
via SDO to 'buffer'

NOTE: -'*len' has to specify buffer length
      - If slave aborts domain, '*error' contains error-code, 
        otherwise '*len' specifies length of message
*/

EXPORT int CALLTYPE mcanOpenWriteSDO(unsigned short net, unsigned short modID,
				     int index, int subindex, int typ,
				     void *buffer, int len, ERROR_T* error); 
/*
DESCRIPTION
Writes the content of 'buffer' to 'modID's' Object Dictionary
at 'index','subindex' via SDO

NOTE: -if 'typ' is set to invalid values (e.g. 0) 'len' specifies 
       buffer length otherwise 'len' is ignored 
      -valid values for typ s.a.
*/

EXPORT int CALLTYPE mcanOpenReadConfig(unsigned short net,unsigned short modID,
				       unsigned short source_typ, void *source);
/*
DESCRIPTION
reads 'modID's' CANopen-Device-Configuration-Data from *source (e.g. filename)
into masters database for net 'net'. 

NOTE: -valid values for source_typ s.a.
*/

EXPORT int CALLTYPE mcanOpenWriteConfig(unsigned short net, unsigned short modID,
					unsigned short destination_typ,
					void *destination);
/*
DESCRIPTION
writes 'modID's' CANopen-Device-Configuration-Data from masters database
for net 'net' to destination (e.g. filename)

NOTE: -valid values for destination_typ s.a.
*/


#ifdef __cplusplus
}
#endif

#endif




