#ifndef MCON_COMMANDS_H
#define MCON_COMMANDS_H
/******************************************************************************
* E.S.O - VLT project
*
* "@(#) $Id: mconCommands.h,v 2.35 2003/11/11 13:40:39 vltsccm Exp $" 
*
* mconCommands.h - Contains literals of all defined commands and structure
*                  types for commands with multiple parameters
*
*	1..999		Direct controller commands
*	1000..1999	Interpreted and translated by the driver
*	2000..2999	Executed inside the driver without controller action
*
* who            when       what
* -------------  ---------  --------------------------------------------
* T.PHAN.DUC     01/11/92   First version
* B.GUSTAFSSON   28/10/93   Modified commands for new version of MAC4 contr.
* S.Sandrock	 23/06/94   Modifications for version 2.0
*			    (Detailed history at end of file)
* ---v2.0---     08.02.1995 ---RELEASED---
* S.Sandrock     16.05.1995 added command READ / WRITE_TARGET_SETTLING_TIME
* bgustafs       06/11/95   added commands 
*                           ENABLE/DISABLE/CONFIG_INTERRUPT_ENCODER_VALUE
*                           READ_ENCODER_VALUE_IRQ, mconCMD_DEINIT
*                           added structure mconINT_ENC_VAL
*                           removed command mconCMD_READ_ANALOG_INPUT
*                           READ/WRITE_SERVO_MODE_TARGET_RADIUS
* bgustafs        23/07/97  added command mconCMD_READ_VELOCITY
*                           
*----------------------------------------------------------------------------
*/



/*
 * General commands
 */

#define mconCMD_CLEAR                           2
#define mconCMD_EXECUTE				3
#define mconCMD_MODE_ENABLE_AXIS		4
#define mconCMD_MODE_DISABLE_AXIS		5
#define mconCMD_RESET				84
#define mconCMD_INIT                            180
#define mconCMD_FLY_MODE_ON                     182
#define mconCMD_FLY_MODE_OFF                    183
#define mconCMD_DEINIT                          311

/*
 * Speed and Brake mode
 */

#define mconCMD_MODE_SPEED			48
#define mconCMD_MODE_BRAKE			58
#define mconCMD_READ_SPEED_MODE_SPEED           49
#define mconCMD_READ_SPEED_MODE_ACCEL           50
#define mconCMD_READ_SPEED_MODE_DECEL           51
#define mconCMD_WRITE_SPEED_MODE_SPEED          52
#define mconCMD_WRITE_SPEED_MODE_ACCEL          53
#define mconCMD_WRITE_SPEED_MODE_DECEL          54

/*
 * Positioning mode
 */
 
#define mconCMD_MODE_POSITIONING		59
#define mconCMD_READ_POS_SPEED                  60
#define mconCMD_READ_POS_ACCEL                  61
#define mconCMD_READ_POS_DECEL                  62
#define mconCMD_READ_ABSOLUTE_POS		63
#define mconCMD_WRITE_POS_SPEED                 64
#define mconCMD_WRITE_POS_ACCEL                 65
#define mconCMD_WRITE_POS_DECEL                 66
#define mconCMD_WRITE_ABSOLUTE_POS		70
#define mconCMD_WRITE_RELATIVE_POS		71


/*
 * Search index coarse mode
 */
 
#define mconCMD_MODE_SEARCH_INDEX_COA_ADJUST	6 
#define mconCMD_READ_COARSE_SPEED               7
#define mconCMD_READ_COARSE_ACCEL               8
#define mconCMD_READ_COARSE_DECEL               9
#define mconCMD_WRITE_COARSE_SPEED              10
#define mconCMD_WRITE_COARSE_ACCEL              11
#define mconCMD_WRITE_COARSE_DECEL              12

/*
 * Search index mode
 */
 
#define mconCMD_MODE_SEARCH_INDEX		16
#define mconCMD_READ_INDEX_SPEED                17
#define mconCMD_READ_INDEX_ACCEL              1800   /* See 'mconPrivate.h' */
#define mconCMD_READ_INDEX_DECEL                19
#define mconCMD_WRITE_INDEX_SPEED               20
#define mconCMD_WRITE_INDEX_ACCEL               21
#define mconCMD_WRITE_INDEX_DECEL               22

/*
 * Home mode
 */
 
#define mconCMD_MODE_HOME			26 
#define mconCMD_READ_HOME_SPEED                 27
#define mconCMD_READ_HOME_ACCEL                 28
#define mconCMD_READ_HOME_DECEL                 29
#define mconCMD_WRITE_HOME_SPEED                30
#define mconCMD_WRITE_HOME_ACCEL                31
#define mconCMD_WRITE_HOME_DECEL                32
  
/*
 * Find edge mode
 */
 
#define mconCMD_MODE_FIND_EDGE			36 
#define mconCMD_READ_FIND_EDGE_SPEED            37
#define mconCMD_READ_FIND_EDGE_ACCEL            38
#define mconCMD_READ_FIND_EDGE_DECEL            39
#define mconCMD_READ_DEFINE_LIMIT		40
#define mconCMD_WRITE_FIND_EDGE_SPEED           41
#define mconCMD_WRITE_FIND_EDGE_ACCEL           42
#define mconCMD_WRITE_FIND_EDGE_DECEL           43
#define mconCMD_WRITE_DEFINE_LIMIT		44

/*
 * Velocity tracking
 */
 
#define mconCMD_SPEED_TRACKING			72 
#define mconCMD_READ_TRACK_SPEED                73
#define mconCMD_WRITE_TRACK_SPEED               75

/*
 * Position tracking
 */
 
#define mconCMD_POSITION_TRACKING               169
#define mconCMD_READ_TRACK_POS			74 
#define mconCMD_WRITE_TRACK_POS			77
#define mconCMD_WRITE_TRACK_POS_REL             78 

/*
 * Test mode
 */
 
#define mconCMD_SET_TEST_MODE   		82
#define mconCMD_READ_DAC_OFFSET			100 
#define mconCMD_WRITE_DAC_OFFSET		101

/*
 * Axis configuration
 */
 
#define mconCMD_READ_AXIS_TYPE                  126
#define mconCMD_WRITE_AXIS_TYPE                 127 
#define mconCMD_READ_MAX_POSITIVE		147
#define mconCMD_WRITE_MAX_POSITIVE		148
#define mconCMD_READ_MAX_NEGATIVE		150
#define mconCMD_WRITE_MAX_NEGATIVE		151
#define mconCMD_READ_CIRC_RANGE                 153
#define mconCMD_WRITE_CIRC_RANGE                154

/*
 * Encoder configuration
 */
 
#define mconCMD_READ_ENC_TYPE                   128 
#define mconCMD_WRITE_ENC_TYPE                  129 
#define mconCMD_READ_ENC_COUNTS                 130
#define mconCMD_WRITE_ENC_COUNTS                131
#define mconCMD_READ_ENC_OFFSET                 132
#define mconCMD_WRITE_ENC_OFFSET                133
#define mconCMD_READ_EXT_ENCODER_ADDR		135
#define mconCMD_WRITE_EXT_ENCODER_ADDR	        136
#define mconCMD_READ_ENC_BITS                   137
#define mconCMD_WRITE_ENC_BITS                  138
#define mconCMD_READ_ENC_SHIFT                  184
#define mconCMD_WRITE_ENC_SHIFT                 185
#define mconCMD_READ_ENC_STEPS                  253
#define mconCMD_WRITE_ENC_STEPS                 254

/*
 * Control loop configuration
 */
 
#define mconCMD_READ_PROPOR_GAIN		85
#define mconCMD_WRITE_PROPOR_GAIN		86
#define mconCMD_READ_ZERO_FACTOR		88
#define mconCMD_WRITE_ZERO_FACTOR		89
#define mconCMD_READ_INTEGRATION_GAIN		91
#define mconCMD_WRITE_INTEGRATION_GAIN		92
#define mconCMD_READ_POLE_FACTOR		94
#define mconCMD_WRITE_POLE_FACTOR		95
#define mconCMD_READ_MAX_VELOCITY_REF		97
#define mconCMD_WRITE_MAX_VELOCITY_REF		98
#define mconCMD_READ_INT_GAIN_SHIFT		256
#define mconCMD_WRITE_INT_GAIN_SHIFT		257  

/*
 * Signal configuration
 */
 
#define mconCMD_READ_UPPER_LIMIT_LEVEL          139
#define mconCMD_WRITE_UPPER_LIMIT_LEVEL         140
#define mconCMD_READ_LOWER_LIMIT_LEVEL          141
#define mconCMD_WRITE_LOWER_LIMIT_LEVEL         142
#define mconCMD_READ_REF_SWITCH_LEVEL           143
#define mconCMD_WRITE_REF_SWITCH_LEVEL          144
#define mconCMD_READ_DRIVE_FAULT_LEVEL          145 
#define mconCMD_WRITE_DRIVE_FAULT_LEVEL         146 

/*
 * Interrupt configuration
 */

#define mconCMD_ENABLE_INTERRUPT_DRIVE_FAULT	 1201
#define mconCMD_DISABLE_INTERRUPT_DRIVE_FAULT	 1202
#define mconCMD_ENABLE_INTERRUPT_MOTION_END	 1203
#define mconCMD_DISABLE_INTERRUPT_MOTION_END	 1204
#define mconCMD_ENABLE_INTERRUPT_EMERGENCY_STOP	 1205
#define mconCMD_DISABLE_INTERRUPT_EMERGENCY_STOP 1206
#define mconCMD_ENABLE_INTERRUPT_ANSWER_READY    1207
#define mconCMD_DISABLE_INTERRUPT_ENCODER_VALUE  1208
#define mconCMD_ENABLE_INTERRUPT_ENCODER_VALUE   1209

#define mconCMD_CONFIG_INTERRUPT_DRIVE_FAULT	 1211
#define mconCMD_CONFIG_INTERRUPT_MOTION_END	 1212
#define mconCMD_CONFIG_INTERRUPT_EMERGENCY_STOP	 1213
#define mconCMD_CONFIG_INTERRUPT_ENCODER_VALUE	 1214
 
#define mconCMD_READ_BUFFER_EMPTY_IRQ           175
#define mconCMD_READ_ANSWER_READY_IRQ           176
#define mconCMD_READ_DRIVE_FAULT_IRQ            177
#define mconCMD_READ_MOTION_END_IRQ             178
#define mconCMD_READ_EMERGENCY_STOP_IRQ         179
#define mconCMD_READ_ENCODER_VALUE_IRQ          308

/*
 * Auxiliary parameters and commands
 */

#define mconCMD_READ_MAX_POS_ERROR		103
#define mconCMD_WRITE_MAX_POS_ERROR		104
#define mconCMD_READ_EM_DECEL                   106
#define mconCMD_WRITE_EM_DECEL                  107
#define mconCMD_READ_CORR_SPEED                 109
#define mconCMD_WRITE_CORR_SPEED                110
#define mconCMD_READ_TARGET_RADIUS		112
#define mconCMD_WRITE_TARGET_RADIUS		113
#define mconCMD_READ_TARGET_SETTLING_TIME	300
#define mconCMD_WRITE_TARGET_SETTLING_TIME	301
#define mconCMD_WRITE_INIT_POS			123
#define mconCMD_WRITE_POLARITY			1300
#define mconCMD_POLARITY_INITIAL		124
#define mconCMD_POLARITY_REVERSE		125
#define mconCMD_READ_WATCHDOG                   166
#define mconCMD_WRITE_WATCHDOG                  167
#define mconCMD_READ_SCALE_FACTOR               186
#define mconCMD_WRITE_SCALE_FACTOR              187
#define mconCMD_READ_SAMPLE_PERIOD              362
#define mconCMD_WRITE_SAMPLE_PERIOD             363

/*
 * Stepper motor specific parameters
 */
 
#define mconCMD_READ_MOTOR_STEPS                234
#define mconCMD_WRITE_MOTOR_STEPS               235
#define mconCMD_READ_BOOST_TIME                 237
#define mconCMD_WRITE_BOOST_TIME                238
#define mconCMD_READ_START_STOP_FREQ            240
#define mconCMD_WRITE_START_STOP_FREQ           241
#define mconCMD_READ_CREEP_DIST                 243
#define mconCMD_WRITE_CREEP_DIST                244
#define mconCMD_STP_TURBO_MODE                  246
#define mconCMD_READ_SERVO_WAIT_TIME            261
#define mconCMD_WRITE_SERVO_WAIT_TIME           262

/*
 * Status commands
 */
 
#define mconCMD_READ_CURRENT_POSITION		116
#define mconCMD_READ_ACTUAL_VELOCITY		117
#define mconCMD_READ_USER_STATUS                118
#define mconCMD_READ_SYSTEM_STATUS              119
#define mconCMD_READ_COMMON_STATUS              120
#define mconCMD_READ_ERROR_POS			121
#define mconCMD_READ_ENC_COUNTER                115
#define mconCMD_READ_ENC_COUNT_RAW              233
#define mconCMD_READ_DAC_VALUE			258
#define mconCMD_READ_MCON_VERSION               181
#define mconCMD_READ_CUR_POS_INC                259
#define mconCMD_READ_VELOCITY                   260

/*
 * Direct controller access
 */
	/* Software emergency stop */
#define mconCMD_EMERGENCY_STOP			1113

	/* Write encoder value into DPRAM (Weginformation) */
#define mconCMD_WRITE_ENCODER_VALUE		1114

	/* Flush the reply-queue of the MAC4 (in case of undefined state) */
#define mconCMD_FLUSH_QUEUE			1115


/*
 * Driver related commands
 */
	/* Read the driver version, e.g. 0x0200 is 2.0 */
#define mconCMD_READ_DRIVER_VERSION		2001
	/* Set/Get timer */
#define mconCMD_SET_TIMEOUT_TICK		2101
#define mconCMD_GET_TIMEOUT_TICK		2102
	/* Remove file descriptor from exclusive write mode */
#define mconCMD_FREE_DEVICE                     2103
	/* Block semaphore to simulate a long delay command */
#define mconCMD_BLOCK_SEMAPHORE                 2104




/*
 *	structure used for interrupt service routine 
 */
typedef struct
	{
	VOIDFUNCPTR	routine;
	int		parameter;
	} mconCONFIG_INT;

typedef mconCONFIG_INT *		mconCONFIG_INT_PTR;

/*
**----- Argument Data Structures -----
*/
/*
**      structure used for the command mconCMD_CONFIG_INTERRUPT_ENCODER_VALUE
*/
typedef struct
	{
	VOIDFUNCPTR	routine;
        int             parameter;
	int		vector;
	} mconINT_ENC_VAL;


/*----------------------------------------------------------------------
* who            when       what
* -------------  ---------  --------------------------------------------
* T.PHAN.DUC     01/11/92   First version
* B.GUSTAFSSON   28/10/93   Modified commands for new version of MAC4 controller
*                           Modified mconCMD_MODE_FIND to mconCMD_MODE_FIND_EDGE
*                           Deleted commands mconCMD_STOP
*                                            mconCMD_MODE_ANALOG_POS
*                                            mconCMD_MODE_ANALOG_SPEED
*                                            mconCMD_READ_RELATIVE_POS
*                                            mconCMD_READ_ANALOG_POS_FACTOR
*                                            mconCMD_WRITE_ANALOG_POS_FACTOR
*                                            mconCMD_READ_MEM_ANALOG_INPUT
*                                            mconCMD_READ_MEM_ACTUAL_POS
*                                            mconCMD_READ_MEM_POS_ERROR
*                                            mconCMD_READ_MEM_STATUS
*                                            mconCMD_MODE_TRACKING
*                                            mconCMD_WRITE_TRACKING
*                                            mconCMD_READ_TRACKING
*                                            mconCMD_WRITE_LP
*                                            mconCMD_READ_LP
*                                            mconCMD_READ_MEM_ACTUAL_SPEED
*                                            mconCMD_READ_STATUS
*                                            mconCMD_LINEAR_MODE
*                                            mconCMD_CIRCULAR_MODE
*                                            mconCMD_READ_ACCELERATION
*                                            mconCMD_WRITE_ACCELERATION
*                                            mconCMD_READ_LIMIT_SPEED
*                                            mconCMD_WRITE_LIMIT_SPEED
*                                            mconCMD_READ_LIMIT_MOTION
*                                            mconCMD_WRITE_LIMIT_MOTION
*                                            mconCMD_READ_LI
*                                            mconCMD_WRITE_LI
*                                            mconCMD_READ_MEM_CONTROLLER_TYPE
*                                            mconCMD_READ_MEM_BACKPLANE_ADDR
*                                            mconCMD_ENABLE_AMPLIFIER
*                           Added commands   mconCMD_INIT
*                                            mconCMD_READ_ENC_BITS
*                                            mconCMD_READ_ENC_OFFSET
*                                            mconCMD_READ_ENC_COUNTS
*                                            mconCMD_READ_ENC_TYPE
*                                            mconCMD_READ_EM_DECEL
*                                            mconCMD_READ_CORR_SPEED
*                                            mconCMD_READ_ENC_COUNTER
*                                            mconCMD_WRITE_ENC_BITS
*                                            mconCMD_WRITE_ENC_OFFSET
*                                            mconCMD_WRITE_ENC_COUNTS
*                                            mconCMD_WRITE_ENC_TYPE
*                                            mconCMD_WRITE_EM_DECEL
*                                            mconCMD_WRITE_CORR_SPEED
*                                            mconCMD_WRITE_ENC_SHIFT
*                                            mconCMD_READ_ENC_SHIFT
*                                            mconCMD_ENABLE_BUFFER_EMPTY
*                                            mconCMD_ENABLE_ANSWER_READY
*                                            mconCMD_READ_BUFFER_EMPTY_IRQ
*                                            mconCMD_READ_ANSWER_READY_IRQ
*                                            mconCMD_READ_EMERGENCY_STOP_IRQ
*                                            mconCMD_READ_DRIVE_FAULT_IRQ
*                                            mconCMD_READ_MOTION_END_IRQ
*                                            mconCMD_READ_USER_STATUS
*                                            mconCMD_READ_SYSTEM_STATUS
*                                            mconCMD_READ_COMMON_STATUS
*                                            mconCMD_WRITE_TRACK_ACC
*                                            mconCMD_WRITE_TRACK_SPEED
*                                            mconCMD_WRITE_TRACK_POS
*                                            mconCMD_WRITE_TRACK_POS_REL
*                                            mconCMD_READ_TRACK_ACC
*                                            mconCMD_READ_TRACK_SPEED
*                                            mconCMD_READ_TRACK_POS
*                                            mconCMD_SPEED_TRACKING
*                                            mconCMD_POSITION_TRACKING
*                                            mconCMD_FLY_MODE_ON
*                                            mconCMD_FLY_MODE_OFF
*                                            mconCMD_READ_MCON_VERSION
*                                            mconCMD_READ_WATCHDOG
*                                            mconCMD_WRITE_WATCHDOG
*                                            mconCMD_READ_AXIS_TYPE
*                                            mconCMD_WRITE_AXIS_TYPE
*                                            mconCMD_CLEAR
*                                            mconCMD_READ_COARSE_SPEED
*                                            mconCMD_WRITE_COARSE_SPEED
*                                            mconCMD_READ_COARSE_ACCEL
*                                            mconCMD_WRITE_COARSE_ACCEL
*                                            mconCMD_READ_COARSE_DECEL
*                                            mconCMD_WRITE_COARSE_DECEL
*                                            mconCMD_READ_INDEX_SPEED
*                                            mconCMD_WRITE_INDEX_SPEED
*                                            mconCMD_READ_INDEX_ACCEL
*                                            mconCMD_WRITE_INDEX_ACCEL
*                                            mconCMD_READ_INDEX_DECEL
*                                            mconCMD_WRITE_INDEX_DECEL
*                                            mconCMD_READ_HOME_SPEED
*                                            mconCMD_WRITE_HOME_SPEED
*                                            mconCMD_READ_HOME_ACCEL
*                                            mconCMD_WRITE_HOME_ACCEL
*                                            mconCMD_READ_HOME_DECEL
*                                            mconCMD_WRITE_HOME_DECEL
*                                            mconCMD_READ_FIND_EDGE_SPEED
*                                            mconCMD_WRITE_FIND_EDGE_SPEED
*                                            mconCMD_READ_FIND_EDGE_ACCEL
*                                            mconCMD_WRITE_FIND_EDGE_ACCEL
*                                            mconCMD_READ_FIND_EDGE_DECEL
*                                            mconCMD_WRITE_FIND_EDGE_DECEL
*                                            mconCMD_READ_SPEED_MODE_SPEED
*                                            mconCMD_WRITE_SPEED_MODE_SPEED
*                                            mconCMD_READ_SPEED_MODE_ACCEL
*                                            mconCMD_WRITE_SPEED_MODE_ACCEL
*                                            mconCMD_READ_SPEED_MODE_DECEL
*                                            mconCMD_WRITE_SPEED_MODE_DECEL
*                                            mconCMD_READ_POS_SPEED
*                                            mconCMD_WRITE_POS_SPEED
*                                            mconCMD_READ_POS_ACCEL
*                                            mconCMD_WRITE_POS_ACCEL
*                                            mconCMD_READ_POS_DECEL
*                                            mconCMD_WRITE_POS_DECEL
*                                            mconCMD_READ_UPPER_LIMIT_LEVEL
*                                            mconCMD_WRITE_UPPER_LIMIT_LEVEL
*                                            mconCMD_READ_LOWER_LIMIT_LEVEL
*                                            mconCMD_WRITE_LOWER_LIMIT_LEVEL
*                                            mconCMD_READ_REF_SWITCH_LEVEL
*                                            mconCMD_WRITE_REF_SWITCH_LEVEL
*                                            mconCMD_READ_DRIVE_FAULT_LEVEL
*                                            mconCMD_WRITE_DRIVE_FAULT_LEVEL
*                                            mconCMD_SET_TEST_MODE
*                                            mconCMD_SET_NORMAL_MODE
*                                            mconCMD_READ_SCALE_FACTOR
*                                            mconCMD_WRITE_SCALE_FACTOR
*                                            mconCMD_READ_ENC_COUNT_RAW
*                                            mconCMD_WRITE_CIRC_LIMIT_POS
*                                            mconCMD_READ_CIRC_LIMIT_POS
*                                            mconCMD_WRITE_CIRC_LIMIT_NEG
*                                            mconCMD_READ_CIRC_LIMIT_NEG
*                                            mconCMD_READ_INT_GAIN_SHIFT
*                                            mconCMD_WRITE_INT_GAIN_SHIFT
*                    Added commands for STP  mconCMD_STP_TURBO_MODE
*                                            mconCMD_READ_MOTOR_STEPS
*                                            mconCMD_WRITE_MOTOR_STEPS
*                                            mconCMD_READ_BOOST_TIME
*                                            mconCMD_WRITE_BOOST_TIME
*                                            mconCMD_READ_START_STOP_FREQ
*                                            mconCMD_WRITE_START_STOP_FREQ
*                                            mconCMD_READ_CREEP_DIST
*                                            mconCMD_WRITE_CREEP_DIST
*                                            mconCMD_READ_ENC_STEPS
*                                            mconCMD_WRITE_ENC_STEPS
*
* S.Sandrock	23/06/94  Added commands: mconCMD_EMERGENCY_STOP,
*			  mconCMD_WRITE_ENCODER_VALUE, mconCMD_WRITE_DAC_VALUE
*		24/06/94  Added command mconCMD_READ_DRIVER_VERSION
*		27/06/94  Removed commands: mconCMD_SET_NORMAL_MODE,
*			  mconCMD_ENABLE_INTERRUPT_BUFFER_EMPTY,
*			  mconCMD_DIRECT, mconCMD_SYNTAX_ERROR
*			  Added command codes for error conditions (160-163)
*			  Corrections for commands (on-fly,polarity,analog-inp)
*		04/07/94  Changed number for 'mconCMD_READ_INDEX_ACCEL'.
*		20/07/94  Re-assigned some numbers >1000 into logical groups.
*		31/08/94  mconCMD_WRITE_DAC_VALUE --> mconCMD_READ_DAC_VALUE
* B.Gustafsson  01/09/94  added command mconCMD_READ_CUR_POS_INC
* S.Sandrock    08/09/94  added command mconCMD_FLUSH_QUEUE
* S.Sandrock    02/01/95  added command mconCMD_WRITE_POLARITY
* S.Sandrock    30/01/95  change in circular limit parameters:
*			  removed commands:
*				mconCMD_READ_CIRC_LIMIT_POS             153
*				mconCMD_WRITE_CIRC_LIMIT_POS            154
*				mconCMD_READ_CIRC_LIMIT_NEG             156
*				mconCMD_WRITE_CIRC_LIMIT_NEG            157
*			  added commands:
*				mconCMD_READ_CIRC_RANGE                 153
*				mconCMD_WRITE_CIRC_RANGE                154
* S.Sandrock    30/01/95  new stepper commands for servo-mode:
*				mconCMD_READ_SERVO_WAIT_TIME            261
*				mconCMD_WRITE_SERVO_WAIT_TIME           262
*				mconCMD_READ_SERVO_MODE_TARGET_RADIUS   263
*				mconCMD_WRITE_SERVO_MODE_TARGET_RADIUS  264
*----------------------------------------------------------------------------
*/
#endif /* !MCON_COMMANDS_H */
