#ifndef MENDRV_COMMANDS_H
#define MENDRV_COMMANDS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id"
*
* mendrvCommands.h  -  Contains literals of all defined commands and
*                   structure types for commands with multiple parameters.
*
* who       when      what
* --------  --------  ----------------------------------------------
* vltcan   2005-04-04 created
* tebert   2005-09-22 added new commands Set TTL Latch/Terminator
*/

/*---------------------------------------------------------------------------
 *
 *  Common MENDRV commands
 *
 ****************************************************************************/
#define mendrvCMD_RESET                         100
#define mendrvCMD_FREE_DEVICE                   115
#define mendrvCMD_SET_TIMEOUT                   105
#define mendrvCMD_GET_TIMEOUT                   110
#define mendrvCMD_GET_BOARD_ID                  135

/*
 * Interrupt commands
 */
#define mendrvCMD_CONFIG_INTERRUPT              180
#define mendrvCMD_ENABLE_INTERRUPT              185
#define mendrvCMD_DISABLE_INTERRUPT             190
#define mendrvCMD_CLEAR_INTERRUPT               195
#define mendrvCMD_READ_INTERRUPT_STATUS         200

/*---------------------------------------------------------------------------
 *
 *  Digital MENDRV I/O commands
 *
 ****************************************************************************/

#define mendrvCMD_BLOCK_SEMAPHORE               120
#define mendrvCMD_NUM_DIGITAL_INPUTS            510
#define mendrvCMD_NUM_DIGITAL_OUTPUTS           515
#define mendrvCMD_READ_CONFIG_BITS              125
#define mendrvCMD_WRITE_CONFIG_BITS             145

/*
 * Read Commands
 */
#define mendrvCMD_READ_BITS                     130
#define mendrvCMD_CHECK_PULSE_BIT               140

/*
 * Write Commands
 */
#define mendrvCMD_WRITE_BITS                    150
#define mendrvCMD_WRITE_SET_BIT                 155
#define mendrvCMD_WRITE_CLEAR_BIT               160
#define mendrvCMD_WRITE_CHANGE_BIT              165
#define mendrvCMD_WRITE_PULSE_BIT               170
#define mendrvCMD_RESET_PULSE_BIT               175

#define mendrvCMD_SET_TTL_TERMINATOR            400
#define mendrvCMD_SET_TTL_LATCH                 405

/*---------------------------------------------------------------------------
 *
 *  Analog MENDRV I/O commands
 *
 ****************************************************************************/
#define mendrvCMD_NUM_ANALOG_INPUTS             500
#define mendrvCMD_NUM_ANALOG_OUTPUTS            505
#define mendrvCMD_WAIT                          385
#define mendrvCMD_GET_INPUT_CONFIG              310
#define mendrvCMD_GET_OUTPUT_CONFIG             315
#define mendrvCMD_SET_INPUT_CONFIG              345
#define mendrvCMD_SET_OUTPUT_CONFIG             350
#define mendrvCMD_ENABLE_EOC_INTERRUPT          360
#define mendrvCMD_DISABLE_EOC_INTERRUPT         365
#define mendrvCMD_AUTOCALIBRATION               370

/*
 * Read Commands
 */
#define mendrvCMD_READ_VALUE                    305
#define mendrvCMD_GET_GAIN                      320
#define mendrvCMD_READ_BIN_VALUE                330

/*
 * Write Commands
 */
#define mendrvCMD_WRITE_VALUE                   340
#define mendrvCMD_SET_GAIN                      355
#define mendrvCMD_WRITE_BIN_VALUE               390

/*
**----- Argument Data Structures -----
*/
typedef struct mendrvSIGNAL_A   /* Signal Config and Status Structure */
{
    int    number;
    int    config;
    int    gain;
    double value;
} mendrvSIGNAL_A;
typedef mendrvSIGNAL_A    *mendrvSIGNAL_A_PTR;

typedef struct mendrvBinSIGNAL_A    /* use with analog mendrvRead/WriteBinValue */
{
    int number;
    int config;
    int gain;
    int value;
} mendrvBinSIGNAL_A;
typedef mendrvBinSIGNAL_A *mendrvBinSIGNAL_A_PTR;

/* Predefined macros for the .config member: */
#define mendrvCONFIG_2x10V      1
#define mendrvCONFIG_10V        2

/* Limits: */
#define mendrvSIGNAL_MAX       16

/* values for the `inputOutput' member: */
#define	mendrvSIGNAL_INPUT      0
#define	mendrvSIGNAL_OUTPUT	    1

/* values for the `polarity' member: */
#define	mendrvPOLARITY_ACTIVE_LOW   0
#define	mendrvPOLARITY_ACTIVE_HIGH  1

/* Gain */
#define mendrvGAIN_X1       1
#define mendrvGAIN_X2       2
#define mendrvGAIN_X4       4
#define mendrvGAIN_X8       8

#define mendrvTTL_PORT_A    1
#define mendrvTTL_PORT_B    2
#define mendrvTTL_PORT_C    3
#define mendrvTTL_PORT_D    4

#define mendrvTTL_TERMINATOR_ACTIVE    1
#define mendrvNO_TTL_TERMINATOR        0

#define mendrvTTL_LATCH_MODE_NORMAL    0
#define mendrvTTL_LATCH_MODE_TRIGGER   2
#define mendrvTTL_LATCH_MODE_DATABUS   1

#define mendrvTTL_LATCH_EDGE_POSITIVE  1
#define mendrvTTL_LATCH_EDGE_NEGATIVE -1

/*
 *----- Argument Data Structures -----
 */
typedef struct
    {
    int firstSignal;
    int numSignal;
    int returnValue;
    } mendrvREAD;
typedef mendrvREAD *mendrvREAD_PTR;

typedef struct
    {
    int firstSignal;
    int numSignal;
    int value;
    } mendrvWRITE;
typedef mendrvWRITE *mendrvWRITE_PTR;

typedef struct
    {
    int firstSignal;
    int numSignal;
    int inputOutput;
    int polarity;
    } mendrvCONFIG;
typedef mendrvCONFIG  *mendrvCONFIG_PTR;

typedef struct
    {
    int numSignal;
    int time;
    } mendrvPULSE;
typedef mendrvPULSE   *mendrvPULSE_PTR;

typedef struct
    {
    int numSignal;
    int polarity;
    } mendrvENABLE_INT;
typedef mendrvENABLE_INT *mendrvENABLE_INT_PTR;

typedef struct
    {
    VOIDFUNCPTR routine;
    int     parameter;
    } mendrvCONFIG_INT;
typedef mendrvCONFIG_INT *mendrvCONFIG_INT_PTR;

typedef struct
   {
   int port;
   int active;
   } mendrvTTL_TERMINATOR;
typedef mendrvTTL_TERMINATOR *mendrvTTL_TERMINATOR_PTR;

typedef struct
   {
   int mode;
   int edge;
   } mendrvTTL_LATCH;
typedef mendrvTTL_LATCH *mendrvTTL_LATCH_PTR;

#endif /* !MENDRV_COMMANDS_H */

