/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: mot.h,v 2.36 2005/08/23 14:13:26 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* CAM GmbH  --/05/94  Created  for Release 1.1
* P.Duhoux  --/07/94  Modified for Release 1.2
* P.Duhoux  09/09/94  Renamed motVERSION_NUMBER to motVERSION
*                     and changed its definition
* P.Duhoux  12/05/95  Modified interface to motPrintMotors ( void )
*                     New function motMotors()
* P.Duhoux  09/06/95  Modified for CMM Archive
* P.Duhoux  07/07/95  Removed all literal definitions and 
*                     include motDefines.h instead
* P.Duhoux  25/07/95  Added new function motCheckBrake()
* P.Duhoux  26/03/96  Added motStatus
*                      overCurrent,overTemperature
*                      onLimit,emergencyStop,driveFault,posError
*                      posIndex,nextPosIndex
* P.Duhoux  29/03/96  Added new functions:
*                      motGetSemIEV() & motWriteEncoderValue()
* P.Duhoux  12/09/96  Added new function motEncToIndex(), motGetHandle()
* P.Duhoux  13/09/96  Added new function motSetSection()
* P.Duhoux  10/11/96  Added new function motGetDbStatus()
* P.Duhoux  12/11/96  Moved definitions from motPublic.h for 
*                      motCONFIG data structure,
*                     Added new function motGetDbConfig(), motFreeConfig()
* P.Duhoux  19/12/96  Added 'par' in motMOTION for INIT HW (initCode/waitDelay)
* P.Duhoux  14/01/97  Added functions motResetSw/motResetHw equivalent to
*                     motResetAxis/motResetBoards (for cosmetics)
*/

#ifndef MOT_H
#define MOT_H

/******************************************************************************
* mot.h - motor control module API definition file
*------------------------------------------------------------------------------
*/
#ifdef __cplusplus
extern "C" {
#endif

/*
 ******************************************************************************
 * Header Files
 ******************************************************************************
 */

/*
 * LCC include files
 */
#include "CCS.h"
#include "cai.h"
#ifdef MCM_LCC_TASK
#include "lccTaskLib.h"
#endif

/*
 * MCM include files
 */
#include "motDefines.h"
#include "motErrors.h"

/*
 ******************************************************************************
 * Literals
 ******************************************************************************
 */

/* MOT module name */
#define motMODULE              "mot"
#define motMODULE_NAME         "Motor Control Module API"
 
/* literal for motor library version */
#define motVERSION             "$Revision: 2.36 $"
#define motVERSION_DATE        "JAN2006"

/*
 ******************************************************************************
 * Data Types
 ******************************************************************************
 */

/* 
 * definition of motor handle 
 */
typedef vltUINT32 motHANDLE;

/* data type for access modes */
typedef vltINT32 motACCESS_MODE;
      
/* data type for operational modes */ 
typedef vltINT32 motOPMODE;

/* data type for units */
typedef vltBYTES8 motUNIT;

/* data type for axis types */
typedef vltINT32 motAXIS_TYPE;

/* data type for encoder types */
typedef vltINT32 motENC_TYPE;

/* data type for motor types */
typedef vltINT32 motMOTOR_TYPE;

/* data type for encoder coding types */
typedef vltINT32 motENC_CODE;

/* data type for abnormal event types */
typedef vltINT32 motEVENT_TYPE;

/* data type for digital IO signal types */
typedef vltINT32 motSIGNAL_TYPE;

/* data type for position/speed definition types */ 
typedef vltINT32 motSPEC_TYPE;

/* data type for section types */
typedef vltINT32 motSECTION;

/* data type for position types */
typedef vltINT32 motPOS_TYPE;

/* data type for motion modes */
typedef vltINT32 motMOTIONMODE;

/* data type for electrical connection types for amplifier boards */
typedef vltINT32 motCONNECTION;

/* data type for board types */
typedef vltINT32 motBOARD_TYPE;

/* data type for limit types */
typedef vltINT32 motLIMITS;
 
/* data type for motion status types */
typedef vltINT32 motMOTION_STATUS;
    
/* data type for initialization status types */
typedef vltINT32 motINIT_STATUS;
    
/* data structure for parameters */
typedef struct
    {
    vltBYTES20      parName;
    vltINT32        parValue;
    } motPARAMETER;

/* data structure for default units */
typedef struct
    {
    motUNIT position;
    motUNIT speed;
    motUNIT current;
    } motDEFAULT_UNITS;

/* data structure for metric number */
typedef struct
    {
    vltDOUBLE value;
    motUNIT   unit;
    } motMETRIC;

/* data structure for speed definition */ 
typedef struct
    {
    motSPEC_TYPE how;         /* speed definition mode                       */
    motMETRIC    number;      /* speed literal (value and unit)              */
    vltBYTES32   name;	      /* speed name                                  */
    vltUINT32    index;       /* index in speed name table                   */
    vltUINT32    pollInt;     /* polling interval in millisec                */
    vltUINT32    endPollInt;  /* end-phase polling interval in millisec      */
    vltUINT32    inPosTime;   /* needed in-position count for motion         */
                              /* complete condition in ms                    */
    } motSPEED;

/* data structure for position name */
typedef struct
    {
    vltBYTES32 posName;
    motSECTION section;
    } motPOS_NAME;
        
/* data structure for position definition */ 
typedef struct
    {
    motSPEC_TYPE how;         /* position definition mode          */
    motMETRIC    number;      /* position literal (value and unit) */
    motPOS_NAME  name; 	      /* named position                    */
    vltUINT32    index;       /* name index                        */
    } motPOSITION;

/* data structure for motion definition */ 
typedef struct
    {
    motPOS_TYPE   posType;	/* position type                           */
    motPOSITION   pos;		/* position                                */
    motSPEED      speed;	/* speed                                   */
    vltINT32      par;          /* initCode/waitDelay (INIT only)          */
    vltLOGICAL    lastStep;	/* flag for "last step in motion sequence" */
    } motMOTION;

typedef struct
    {
    motMOTOR_TYPE  type;      /* DC | STEPPER                     */
    vltINT32       timeout;   /* motion timeout in milliseconds   */
    vltINT32       timelim;   /* timeout for motion out of limit  */
    vltINT32       timemon;   /* automatic monitoring period      */
    motOPMODE      opMode;    /* operational mode on first attach */
    vltINT32       posEnc;    /* Fixed Position definition        */
    vltDOUBLE      posUser;
    vltBYTES8      posUnit;
    vltBYTES32     posName;
    vltINT32       posSection;
    vltINT32       chkHwLimit;/* Check flag for HW limits         */
    } motMOTOR_DEF;

/* database map for board configuration */
typedef struct 
    {
    motBOARD_TYPE type;
    vltINT32      available;
    vltBYTES8     prefix;
    } motBOARD_CONF;

typedef struct 
    {
    vltUINT32       name;
    motEVENT_TYPE   eventType;
    motBOARD_TYPE   boardType;
    vltBYTES20      eventName;
    } motEVENT_CFG;

/* database for IO signals */
typedef struct 
    {
    vltINT32     signal;                /* Signal type                       */
    vltBYTES8    device;                /* Associated device name            */
    vltINT32     startBit;   		/* Start bit number                  */
    vltINT32     level;			/* Signal logic                      */
    } motSIGNAL_CFG;
    
typedef struct 
    {
    motENC_TYPE   type;               /* Encoder type                        */
    motBOARD_TYPE board;              /* Interface board                     */
    motENC_CODE   code;               /* Code type [binary/BCD/Gray]         */
    vltUINT32     address;            /* Address for external                */
    vltUINT32     resolution;         /* Valid bits (absolute only)          */
    vltUINT32     count;              /* Encoder Counts / Encoder Turn       */
    vltUINT32     circularRange;      /* Circular Range                      */
    vltUINT32     stepCount;          /* Encoder Counts / Motor   turn (STP) */
    vltUINT32     bitShift;           /* Shift (SSI only)                    */
    vltINT32      offset;             /* Offset                              */
    } motENCODER;

/* database map for special positions' configuration */
typedef struct 
    {
    vltINT32    set;
    motPOS_TYPE posType;
    vltDOUBLE   position;
    motUNIT     unit;                 /* Unit must be  motBOARD_UNIT_POS     */
    } motSPECIAL_POSITIONS_CFG;

/* database map for named position offsets in Enc */
typedef struct
    {
        /* Offsets - Unit must be motBOARD_UNIT_POS or default position unit */
    vltDOUBLE maintOffset;	/* offset to maintenance section             */
    vltDOUBLE userOffset;	/* offset to user defined section            */
    motUNIT   unit;
        /* reserved for internal use - Unit is motBOARD_UNIT_POS */
    vltINT32  maintOffsetEnc;	/* Unit is motBOARD_UNIT_POS                 */
    vltINT32  userOffsetEnc;	/* Unit is motBOARD_UNIT_POS                 */
    } motNAMED_POSITION_OFFSET;

/* database map for named positions */
typedef struct
    {
    vltBYTES32  name;           /* position name                             */
    motPOS_TYPE posType;        /* position type                             */
    vltDOUBLE   obsPos;         /* nominal observation position value        */
    vltDOUBLE   lowRange;       /* lower range offset                        */
    vltDOUBLE   upRange;        /* upper range offset                        */
    vltINT32    maintOffsetAvail;/* maintenance offset available flag        */
    vltDOUBLE   maintOffset; 	/* offset to maintenance position            */
    vltINT32    userOffsetAvail;/* user offset available flag                */
    vltDOUBLE   userOffset; 	/* offset to user position                   */
    motUNIT     unit;           /* position unit                             */
        /* reserved for internal use - Unit is motBOARD_UNIT_POS */
    vltINT32    available;      /* named position is resolved                */
    vltINT32    obsPosEnc[3];   /* observation positions [low,nom,up]        */
    vltINT32    mntPosEnc[3];   /* maintenance positions [low,nom,up]        */
    vltINT32    usrPosEnc[3];   /* userDefined positions [low,nom,up]        */
    } motNAMED_POSITION;

/* database map for speed */
typedef struct
    {
    vltBYTES32           name;          /* speed name                        */
    vltDOUBLE            speed;      	/* speed value                       */
    motUNIT  		 unit;          /* speed unit                        */
    vltUINT32		 pollInt;	/* polling interval in ms            */
    vltUINT32		 endPollInt;	/* end-phase polling interval in ms  */
    vltUINT32		 inPosTime;	/* needed in-position time for       */
                                        /* motion complete condition in ms   */
        /* reserved for internal use - Unit is motBOARD_UNIT_SPD */
    vltDOUBLE            speedEnc;      /* Speed in motBOARD_UNIT_SPD        */
    } motNAMED_SPEED;

/* database map for two step offset */
typedef struct
    {
    vltDOUBLE      offset;              /* offset value in user unit         */
    motUNIT        unit;                /* Unit must be motBOARD_UNIT_POS    */
    motSPEED       speed;               /* speed to be used for second step  */
    vltINT32       offsetEnc;           /* offset in motBOARD_UNIT_POS       */
    vltDOUBLE      speedEnc;            /* Speed  in motBOARD_UNIT_SPD       */
    } motTWO_STEP_OFFSET;

/* database map for index pulse speed */
typedef struct
    {
    motSPEED       speed;               /* speed to be used for second step  */
    vltDOUBLE      speedEnc;            /* Speed in motBOARD_UNIT_SPD        */
    } motINDEX_PULSE_SPEED;

/* data structures for motor configuration */
typedef struct     
    {
    motMOTOR_DEF             motor;
    motBOARD_CONF            boards[motMAX_BOARD];
    motEVENT_CFG             events[motMAX_EVENT];
    motSIGNAL_CFG            signals[motMAX_SIGNAL];
    vltINT32                 brakeAtBoot;
    motAXIS_TYPE             axis;
    motENCODER               encoder;
    motSPECIAL_POSITIONS_CFG specialPositions[motMAX_SPECIAL_POSITION];
    motNAMED_POSITION_OFFSET offsets;
    vltINT32                 numNamedPositions;
    motNAMED_POSITION       *namedPositions;
    vltINT32                 numNamedSpeeds;
    motNAMED_SPEED          *namedSpeeds;
    motTWO_STEP_OFFSET       twoStepOffset;
    motINDEX_PULSE_SPEED     indexPulseSpeed;
    vltINT32                 defaultSpeed;
    } motCONFIG;

/* 
 * status data structure
 */     
typedef struct
    {
    motOPMODE	      opMode;         /* operational mode                    */
    motMOTIONMODE     motionMode;     /* motion mode                         */
    motMOTION_STATUS  motionStatus;   /* motion status                       */
    vltINT32	      motionStep;     /* motion step                         */
    motINIT_STATUS    initStatus;     /* initialization status               */
    vltINT32          initStep;       /* step of initialization sequence     */
    vltINT32          brakeClamped;   /* brake status                        */
    vltINT32  	      axisEnabled;    /* axis enabled flag                   */
    vltUINT32	      eventMask;      /* bit mask to encode abnormal events  */
    vltINT32          amplStatus;     /* amplifier board status              */
    motCONNECTION     powerStatus;    /* pos./neg. relay open closed         */
    vltINT32          overCurrent;    /* Amplifier Current     (over=1)      */
    vltINT32          overTemperature;/* Amplifier Temperature (over=1)      */
    vltINT32          mconStatus;     /* controller board status             */
    vltINT32  	      driveFault;     /* Drive Fault (Fault=1)               */
    vltINT32  	      emergencyStop;  /* Emergency Stop (Stopped=1)          */
    vltINT32  	      posError;       /* Positioning Error (Error=1)         */
    vltINT32  	      inPos;          /* in position flag                    */
    vltINT32  	      onLimit;        /* Motor On Limit                      */
    motLIMITS         amplOnLimit;    /* on limit flag for amplifier         */
    vltINT32          amplOnHome;     /* on Electrical Home for amplifier    */
    motLIMITS         mconOnLimit;    /* on limit flag for controller        */
    vltINT32          mconOnRef;      /* on RefSwitch  for controller        */
    vltINT32          mconOnInd;      /* on IndexPulse for controller        */
    vltINT32          interlock;      /* interlock line status               */
    vltDOUBLE	      speed;          /* actual speed in user units          */
    motUNIT	      speedUnit;      /* motor speed user unit               */
    vltDOUBLE	      current;        /* actual motor current in user units  */
    motUNIT	      currentUnit;    /* motor current user unit             */
    vltINT32	      posEnc;         /* actual position in encoder ticks    */
    vltDOUBLE         posUser;        /* actual position in user units       */
    motUNIT	      posUnit;        /* position user unit                  */
    vltBYTES32        posName;        /* actual position as name             */
    motSECTION        posSection;     /* specifies the section of the name   */
    vltINT32	      posIndex;       /* position index                      */
    vltINT32	      nextPosEnc;     /* next position                       */
    vltDOUBLE         nextPosUser;    /* next position in user units         */
    vltBYTES32        nextPosName;    /* next position as name               */
    motSECTION        nextPosSection; /* specifies the section of the name   */
    vltINT32	      nextPosIndex;   /* next position index                 */
    vltDOUBLE         followingErr;   /* actual following error in user unit */
    ccsTIMEVAL	      startTime;      /* start time of last movement in UTC  */
    ccsTIMEVAL	      endTime;	      /* end time of last movement in UTC    */
    } motSTATUS;

/* special positions values */
typedef struct
    {
    vltINT32       available;
    vltINT32       posEnc;            /* Position in motBOARD_UNIT_POS     */
    vltDOUBLE      posUser;           /* Position in User Unit             */
    motUNIT        posUnit;           /* User Unit                         */
    } motSPECIAL_POSITION;
typedef motSPECIAL_POSITION motSPECIAL_POSITIONS[motMAX_SPECIAL_POSITION];

/* type for user function types */ 
typedef vltINT32 motUSER_FCT;

/* type for user init function */
typedef ccsCOMPL_STAT (* motINIT_FUNC) ( void *, ccsERROR *);
                                         	
/* type for user unit conversion function */
typedef ccsCOMPL_STAT (* motCONV_FUNC) ( motMETRIC *, motMETRIC *,
                                         void *, ccsERROR *);	

/*
 ******************************************************************************
 * API Function Prototypes
 ******************************************************************************
 */
 
ccsCOMPL_STAT motInstall        ( char *dbPath, ccsERROR *pError );
ccsCOMPL_STAT motDeinstall      ( char *dbPath, ccsERROR *pError );
ccsCOMPL_STAT motInitDb         ( char *motor, char *file, ccsERROR *pError);

ccsCOMPL_STAT motAttach         ( dbSYMADDRESS dbPath, motACCESS_MODE mode, 
				  motHANDLE *handle, ccsERROR *error );
ccsCOMPL_STAT motDetach         ( motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motDisable        ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motEnable         ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motSetOpMode      ( const motHANDLE handle[], motOPMODE *mode, 
				  ccsERROR *error );
ccsCOMPL_STAT motGetOpMode      ( const motHANDLE handle[], motOPMODE *mode, 
				  ccsERROR *error );

ccsCOMPL_STAT motNameToEnc      ( motHANDLE handle, vltBYTES32 name, 
				  motSECTION section, vltINT32 *posEnc, 
				  ccsERROR *error );                           
ccsCOMPL_STAT motEncToIndex     ( motHANDLE handle, vltINT32 posEnc, 
				  motSECTION *section, vltINT32 *index, 
				  ccsERROR *error );                           
ccsCOMPL_STAT motEncToName      ( motHANDLE handle, vltINT32 posEnc, 
				  motSECTION *section, vltBYTES32 name, 
				  ccsERROR *error );                           
ccsCOMPL_STAT motNameToVel      ( motHANDLE handle, vltBYTES32 name,
                                  vltDOUBLE *velEnc, ccsERROR *error );
ccsCOMPL_STAT motPosToEnc       ( motHANDLE handle, motMETRIC *position,
                                  vltINT32 *encValue, ccsERROR *error );
ccsCOMPL_STAT motEncToPos       ( motHANDLE handle, vltINT32 encValue,
                                  motMETRIC *position, ccsERROR *error );
ccsCOMPL_STAT motVelToEnc       ( motHANDLE handle, motMETRIC *velocity,
                                  vltDOUBLE *encValue, ccsERROR *error );
ccsCOMPL_STAT motEncToVel       ( motHANDLE handle, vltDOUBLE encValue,
                                  motMETRIC *velocity, ccsERROR *error );
ccsCOMPL_STAT motVelToRef       ( motHANDLE handle, motMETRIC *velocity,
                                  vltINT32 *refValue, ccsERROR *error );
ccsCOMPL_STAT motRefToVel       ( motHANDLE handle, vltINT32 refValue,
                                  motMETRIC *velocity, ccsERROR *error );
ccsCOMPL_STAT motDigToCurrent   ( motHANDLE handle, vltINT32 currDig,
                                  motMETRIC *currUser, ccsERROR *error );

ccsCOMPL_STAT motInitSw         ( const motHANDLE handle[], 
				  vltLOGICAL mandatory, ccsERROR *error );
ccsCOMPL_STAT motInitHw         ( const motHANDLE handle[], 
				  vltLOGICAL mandatory, ccsERROR *error );
ccsCOMPL_STAT motSetDefaultUnits( const motHANDLE handle[], 
				  motDEFAULT_UNITS units[], ccsERROR *error );
ccsCOMPL_STAT motSetMotionMode  ( const motHANDLE handle[], 
				  motMOTIONMODE mode[], ccsERROR *error );
ccsCOMPL_STAT motMove           ( const motHANDLE handle[], motMOTION *motion,
                                  ccsERROR  *error );
ccsCOMPL_STAT motCheckMove      ( const motHANDLE handle[], motMOTION *motion, 
                                  ccsERROR *error );
ccsCOMPL_STAT motSetSection     ( const motHANDLE handle[], motSECTION *section,
				  ccsERROR *error );
ccsCOMPL_STAT motGetStatus      ( const motHANDLE handle[], motSTATUS *status, 
                                  ccsERROR *error );
ccsCOMPL_STAT motGetDbStatus    ( const motHANDLE handle[], motSTATUS *status, 
                                  ccsERROR *error );
ccsCOMPL_STAT motGetDbConfig    ( const motHANDLE handle[], motCONFIG *config, 
                                  ccsERROR *error );
ccsCOMPL_STAT motGetSpecialPositions  ( const motHANDLE handle[], motSPECIAL_POSITIONS *positions,
                                  ccsERROR *error );
ccsCOMPL_STAT motFreeConfig     ( motCONFIG *config );
ccsCOMPL_STAT motUpdateConfig   ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motWaitInit       ( const motHANDLE handle[], vltINT32 timeout,
                                  motSTATUS *motStatus, ccsERROR *error );
ccsCOMPL_STAT motWaitMove       ( const motHANDLE handle[], vltINT32 timeout,
                                  motSTATUS *motStatus, ccsERROR *error );
ccsCOMPL_STAT motResetSw        ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motResetHw        ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motStop           ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motConnect        ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motDisconnect     ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motClampBrake     ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motUnclampBrake   ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motCheckInterlock ( const motHANDLE handle[], vltINT32 *status, 
                                  ccsERROR *error );
ccsCOMPL_STAT motCheckBrake     ( const motHANDLE handle[], vltINT32 *status, 
                                  ccsERROR *error );
ccsCOMPL_STAT motInitSampling   ( const motHANDLE handle[], vltINT32 seconds, 
                                  motSECTION section, ccsERROR *error );
ccsCOMPL_STAT motStopSampling   ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motLoadPar        ( motHANDLE handle, motBOARD_TYPE board,
                                  vltUINT32 *number, motPARAMETER *parBuf,
                                  ccsERROR *error );
ccsCOMPL_STAT motRetrievePar    ( motHANDLE handle, motBOARD_TYPE board,
                                  vltUINT32 *number, motPARAMETER *parBuf,
                                  ccsERROR *error );
ccsCOMPL_STAT motSetSwLimits    ( const motHANDLE handle[], 
				  motPOSITION *positions,
				  motPOS_TYPE *posTypes, 
				  motLIMITS *limitFlags,
				  ccsERROR *error ); 

ccsCOMPL_STAT motSetParListAddress ( motHANDLE handle, motUSER_FCT fctType,
                                  vltINT32 fctOccurence, vltBYTES32 fctName,
                                  vltUINT32  parList, ccsERROR *error );
ccsCOMPL_STAT motGetParListAddress ( motHANDLE handle, motUSER_FCT fctType,
                                  vltINT32 fctOccurence, vltBYTES32 fctName,
                                  vltUINT32 *parList, ccsERROR *error );

ccsCOMPL_STAT motGetHandle      ( char *name, motHANDLE *handle, 
				  ccsERROR *error );
ccsCOMPL_STAT motGetSemIEV      ( const motHANDLE handle, SEM_ID *semIEV,
				  ccsERROR *error );
ccsCOMPL_STAT motWriteEncoderValue ( const motHANDLE handle, 
				  vltINT32 encoderValue, ccsERROR *error );

/*
 * Tools
 */
void          motVersion          ( vltBYTES80 version );
ccsCOMPL_STAT motClear            ( dbSYMADDRESS dbPath );
ccsCOMPL_STAT motPrintMotors      ( void );
ccsCOMPL_STAT motMotors           ( int *num, vltBYTES20 list[] );  
void          motDebug            ( vltINT32 level );

/*
 * Oldies for compatibility
 */
ccsCOMPL_STAT motResetAxis      ( const motHANDLE handle[], ccsERROR *error );
ccsCOMPL_STAT motResetBoards    ( const motHANDLE handle[], ccsERROR *error );

#ifdef __cplusplus
}
#endif

#endif     /* if !MOT_H */

/*************************************************************************/

/*___oOo___*/
