/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: motPublic.h,v 2.36 2005/08/23 14:13:26 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* CAM GmbH  --/05/94  Created  for Release 1.1
* P.Duhoux  --/07/94  Modified for Release 1.2
* P.Duhoux  25/01/95  Compliance with OO approach
* P.Duhoux  31/01/95  Added eventIsr in descriptor (Only set by ISR)
* P.Duhoux  15/05/95  Removed field 'motAlias' from the descriptor
*                     - Field 'motName' is now the ALIAS
* P.Duhoux  09/06/95  Modified for CMM Archive
* P.Duhoux  04/07/95  Added literal motMIN_POLL_RATE for status sampling task
* P.Duhoux  07/07/95  Removed all literal definitions (now in motDefines.h 
*                     included by mot.h)
* P.Duhoux  25/07/95  Modified motDESCRIPTION for optimized unit conversions
* P.Duhoux  28/08/95  Renamed from motInternal.h
* P.Duhoux  05/12/95  Added flag convConfigDone to indicate the DB config
*                     attributes have been converted to board units
* P.Duhoux  19/01/96  Added defaultUnits and posEnc in descriptor
* P.Duhoux  29/03/96  Added mconSemIEV in motDESCRIPTION
* P.Duhoux  12/11/96  Move definitions to mot.h for motCONFIG data structure
* P.Duhoux  11/12/96  Increased motSEM_TIMEOUT from 5 to 10 seconds (SPR960747)
* P.Duhoux  05/08/97  Removed task option VX_NO_STACK_FILL
*/

#ifndef MOT_PUBLIC_H
#define MOT_PUBLIC_H

/******************************************************************************
* motPublic.h - motor library public interface file
*
* This header file contains data type definitions and global variables for the
* interface between the mot server and the mot single device libraries. Hence,
* all these sub-modules need to include this file.
*
*------------------------------------------------------------------------------
*/
#ifdef __cplusplus
extern "C" {
#endif

/*
 * VxWorks include files
 */
#include "vxWorks.h"
#include "semLib.h"
#include "string.h"
#include "ctype.h"
#include "stdio.h"
#include "stdlib.h"
#include "ioLib.h"
#include "symLib.h"
#include "sysSymTbl.h"
#include "sysLib.h"
#include "taskLib.h"
#include "tickLib.h"
#include "usrLib.h"
#include "errnoLib.h"
#include "vxLib.h"
#include "math.h"

/*
 * MCM include files
 */
#include "mot.h"

#define IN
#define OUT 
#define INOUT

/******************************************************************************
 * server internal macros
 *****************************************************************************/
 
/* macros for task priorities */
#define motBASE_TASK_PRIOR    (desc->priority)             /* API level      */
#define motINIT_TASK_PRIOR    (motBASE_TASK_PRIOR + 1)     /* H/W Init Task  */
#define motMOVE_TASK_PRIOR    (motBASE_TASK_PRIOR + 2)     /* Motion   Task  */
#define motPOLL_TASK_PRIOR     200                         /* Sampling Task  */

/* macros for task flags */
#define motTASK_FLAGS         (VX_FP_TASK)
#define motINIT_TASK_FLAGS    motTASK_FLAGS                /* H/W Init Task  */
#define motMOVE_TASK_FLAGS    motTASK_FLAGS                /* Motion   Task  */
#define motPOLL_TASK_FLAGS    motTASK_FLAGS                /* Sampling Task  */

/* macros for task stacks */
#define motINIT_TASK_STACK    60000                        /* H/W Init Task  */
#define motMOVE_TASK_STACK    40000                        /* Motion   Task  */
#define motPOLL_TASK_STACK    40000                        /* Sampling Task  */

/* macros for task names */
#define motINIT_TASK_NAME     "tInit_%s"                   /* H/W Init Task  */
#define motMOVE_TASK_NAME     "tMove_%s"                   /* Motion   Task  */
#define motPOLL_TASK_NAME     "tPoll_%s"                   /* Sampling Task  */

/* timeout for <motor descriptor>.accessSem - in seconds */
#define motSEM_TIMEOUT         10
#define motFIS_TIMEOUT          5

#define motsrvSemTimeout  ((int)(motTimeout * motTicksPerSecond))

/* database points */
#define MOTOR_STATUS           "STATUS"
#define MOTOR_SERVER           "SERVER"

/******************************************************************************
 *                          Internal Data Types
 *****************************************************************************/

/*
 * index of motor status database attributes for cai Access
 */
typedef enum
    {
    motsrvDB_STAT_OP_MODE = motDB_STATUS_OP_MODE,
    motsrvDB_STAT_MOTION_MODE,
    motsrvDB_STAT_MOTION_STATUS,
    motsrvDB_STAT_MOTION_STEP,
    motsrvDB_STAT_INIT_STATUS,
    motsrvDB_STAT_INIT_STEP,
    motsrvDB_STAT_BRAKE_CLAMPED,
    motsrvDB_STAT_AXIS_ENABLED,
    motsrvDB_STAT_EVENT_MASK,
    motsrvDB_STAT_AMPL_STATUS,
    motsrvDB_STAT_POWER_STATUS,
    motsrvDB_STAT_OVER_CURRENT,
    motsrvDB_STAT_OVER_TEMPERATURE,
    motsrvDB_STAT_MCON_STATUS,
    motsrvDB_STAT_DRIVE_FAULT,
    motsrvDB_STAT_EMERGENCY_STOP,
    motsrvDB_STAT_POS_ERROR,
    motsrvDB_STAT_IN_POS,
    motsrvDB_STAT_ON_LIMIT,
    motsrvDB_STAT_AMPL_ON_LIMIT,
    motsrvDB_STAT_AMPL_ON_HOME,
    motsrvDB_STAT_MCON_ON_LIMIT,
    motsrvDB_STAT_MCON_ON_REF,  
    motsrvDB_STAT_MCON_ON_IND,  
    motsrvDB_STAT_INTERLOCK,
    motsrvDB_STAT_SPEED,
    motsrvDB_STAT_SPEED_UNIT,
    motsrvDB_STAT_CURRENT,
    motsrvDB_STAT_CURRENT_UNIT,
    motsrvDB_STAT_POS_ENC,
    motsrvDB_STAT_POS_USER,
    motsrvDB_STAT_POS_UNIT,
    motsrvDB_STAT_POS_NAME,
    motsrvDB_STAT_POS_SECTION,
    motsrvDB_STAT_POS_INDEX,
    motsrvDB_STAT_NEXT_POS_ENC,
    motsrvDB_STAT_NEXT_POS_USER,
    motsrvDB_STAT_NEXT_POS_NAME,
    motsrvDB_STAT_NEXT_POS_SECTION,
    motsrvDB_STAT_NEXT_POS_INDEX,
    motsrvDB_STAT_FOLLOWING_ERR,
    motsrvDB_STAT_TIME_BEG,
    motsrvDB_STAT_TIME_END,
    motsrvDB_MAX_STAT
    } motsrvDB_STAT;

/* data type for conversion methods */
typedef vltINT32 motCONV_TYPE;	

/* data type for HW initialization actions */
typedef vltINT32 motACTION_TYPE;      

/* interpolation table fix point */
typedef struct
    {
    vltDOUBLE x;
    vltDOUBLE y;
    } motPOINT;
    
typedef struct		/* conversion record */
    {
    vltBYTES20   name;
    motCONV_TYPE method;
    vltBYTES32   userFctName;
    vltUINT32    userParList;
    vltDOUBLE    offset;
    vltDOUBLE    slope;
    vltUINT32    numPoints;
    motPOINT     point[motMAX_INTERPOL_POINTS];
    vltUINT32    userFctAddr;
    } motUNIT_CONV;

typedef struct
    {
    motACTION_TYPE actionType;	  /* action type (see above)                 */
    vltBYTES32     userFctName;	  /* if action USER_FUNCTION: function name  */
    vltUINT32      userParList;   /*   pointer to parameter list             */
    vltINT32       initCode;      /* if action BOARD_PROCEDURE: init code    */
    motMOTIONMODE  motionMode;	  /* motion mode                             */
    motPOSITION    position;      /* definition of the position              */
    motPOS_TYPE    posType;	  /* position type: ABSOLUTE, RELATIVE, ...  */
    motSPEED       speed;         /* definition of the speed                 */
    } motACTION;

/* database map for special position values (-> SERVER:INTERN) */
typedef struct
    {
    vltINT32       available;
    vltINT32       encValue;            /* Position in motBOARD_UNIT_POS     */
    } motSPECIAL_POSITIONS_INT;

/* data structures for motor descriptor */
typedef struct motDESCRIPTION *motDESCRIPTION_PTR;
typedef struct motDESCRIPTION
    {
    /* access section */
    vltBYTES20         motName;      /* alias to motor point                 */
    int                priority;     /* API task priority                    */
    motACCESS_MODE     accessMode;   /* highest access mode                  */
    SEM_ID             accessSem;    /* semaphore for descriptor access      */
    vltUINT8           numHandles;   /* number of open handles               */
    motOPMODE          opMode;       /* operational mode                     */
    char              *motDbStatus;  /* server database address table STATUS */
    char              *motDbServer;  /* server database address table SERVER */
    struct motDESCRIPTION *next;     /* ptr to next motor descriptor         */
              
    /* board section */
    vltLOGICAL         mconAvail;    /* flag for "controller available"      */
    vltBYTES8	       mconPrefix;   /* prefix of controller SDL             */
    char              *mconFunc;     /* controller SDL routine table         */
    char              *mconDesc;     /* controller descriptor                */
    SEM_ID             mconSemIEV;   /* semaphore for IEV (external encoder) */
    vltINT32           mconUpperLimit;
    vltINT32           mconLowerLimit;

    vltLOGICAL         amplAvail;    /* flag for "amplifier available"       */
    vltBYTES8	       amplPrefix;   /* prefix of amplifier SDL              */
    char              *amplFunc;     /* amplifier SDL routine table          */
    char              *amplDesc;     /* amplifier descriptor                 */
    
    /* axis & encoder section */
    motMOTOR_TYPE      motorType;    /* type of the motor DC/STP             */
    motAXIS_TYPE       axisType;     /* type of the axis LIN/CIR/OPT         */
    motENC_TYPE        encoderType;  /* type of the encoder NO/INC/ABS       */
    int                circularRange;/* range for Circular Axis              */
    vltINT32           chkHwLimit;   /* Check flag for HW limits             */
    vltLOGICAL         logChkHwLimit;/* Log Check flag for HW limits         */

    /* sample section */
#if MCM_LCC_TASK
    int                sampleTcb;    /* TCB of sampling task                 */
#endif
    int                sampleTid;    /* task id of sampling task             */
    motSECTION         sampleSection;/* section for sampling / get status    */
    int                pollRate;     /* polling rate in ticks                */
    SEM_ID             pollSem;      /* poll task start semaphore            */
    SEM_ID             pollAck;      /* poll task done  semaphore            */

    /* hwinit/motion section */
#if MCM_LCC_TASK
    int                motionTcb;    /* TCB of motion task                   */
#endif
    int                motionTid;    /* task id of motion task               */
    ccsERROR          *motionErr;    /* Error stack                          */
#if MCM_LCC_TASK
    int                hwinitTcb;    /* TCB of HwInit task                   */
#endif
    int                hwinitTid;    /* task id of HwInit task               */
    ccsERROR          *hwinitErr;    /* Error stack                          */
    motINIT_STATUS     initStatus;   /* initialization status                */
    vltINT32           initStep;     /* initialization procedure step        */
    motMOTION         *motionSeq;    /* pointer to motion sequence to run    */
    motMOTIONMODE      motionMode;   /* motion mode                          */
    motMOTION_STATUS   motionStatus; /* motion status                        */
    vltINT32           motionStep;   /* number of current motion step        */
    SEM_ID             moveSem;      /* motion task end semaphore            */
    SEM_ID             initSem;      /* semaphore to release motWaitInit     */
    SEM_ID             waitSem;      /* semaphore to release motWaitMove     */
    vltINT32           posEnc;       /* current position                     */
    vltINT32           nextPosEnc;   /* next position                        */
    ccsTIMEVAL	       startTime;    /* start time of last motion in ticks   */
    ccsTIMEVAL 	       endTime;	     /* end time of last motoin in ticks     */
    
    /* digital IO signals access section */
    ioDIRADDRESS       ioSignals[motMAX_SIGNAL];
    vltINT32           brakeAtBoot;

    /* default unit conversion section */
    vltUINT8           convConfigDone; /* flag to indicate conv config done  */
    vltUINT16          numConvMethods; /* Number of conversion methods       */
    motUNIT_CONV      *convMethods[motMIN_CONV_METHOD]; /* default methods   */
    motDEFAULT_UNITS   defUnits;       /* default units                      */

    /* event handling section            */
    /* KEEP STRICTLY AT END OF STRUCTURE */
    SEM_ID             eventSem;     /* semaphore for abnormal events        */
    vltUINT32          eventMsk;
    vltUINT8           eventFlg[32];
    vltUINT32          eventTab[32];
    } motDESCRIPTION;

/* data type for functions */
typedef ccsCOMPL_STAT (* motFUNCTION) ( void * );

/* data type for error messages */
typedef vltBYTES256 motERRMSG;

/* data structure for handle description */
typedef struct motHANDLEDATA
    {
    vltUINT32		  idFlag; 	/* own address for identification    */
    motDESCRIPTION       *motDesc;	/* pointer to motor descriptor       */
    motACCESS_MODE        accessMode;	/* access mode                       */
    struct motHANDLEDATA *next;		/* pointer to next handle            */
    } motsrvHANDLEDATA;    
    
/*
 ******************************************************************************
 * Server internal global Variables
 ******************************************************************************
 */
extern SEM_ID            motsrvSemAccess;   /* access protection semaphore   */
extern motsrvHANDLEDATA *motsrvFirstHandle; /* ptr to first motor handle     */
extern motDESCRIPTION   *motsrvFirstDesc;   /* ptr to first motor descriptor */

extern int               motTimeout;        /* semaphore timeout value       */
extern int               motFisTimeout;     /* FIS       timeout value       */
extern int               motTicksPerSecond; /* CPU ticks per second          */

/*
 ******************************************************************************
 * Debug handling
 ******************************************************************************
 */
extern vltINT32              motDebugLevel;

#define motDEBUG_NONE        0x00000000

#define motDEBUG_API         0xF0000000
#define motDEBUG_API_SERVER  0x10000000
#define motDEBUG_API_MOTOR   0x20000000

#define motDEBUG_ACI         0x0F000000

#define motDEBUG_MESSAGE     0x000000F0
#define motDEBUG_MSG_COMMAND 0x00000010
#define motDEBUG_MSG_REPLY   0x00000020
#define motDEBUG_MSG_ERROR   0x00000040

#define motDEBUG_POLLING     0x00000100
#define motDEBUG_MONITOR     0x00000200
#define motDEBUG_CHKSTACK    0x00000400

#define motDEBUG_SDL         0x00F00000
#define motDEBUG_CONTROLLER  0x00100000
#define motDEBUG_AMPLIFIER   0x00200000

#define motDEBUG_DIGITALIO   0x00010000

#define motDEBUG_DRIVER      0x0000000F
#define motDEBUG_WRITE       0x00000001
#define motDEBUG_READ        0x00000002
#define motDEBUG_COMMAND     0x00000004
#define motDEBUG_INTERRUPT   0x00000008

#define MCM_LOG(fct,txt) \
    { ccsTIMEVAL utc; timsMODE tm; ccsERROR err; vltBYTES32 str; \
      errResetStack(&err); timsGetUTC(&utc,&tm,&err); \
      timsTimeToIsoString(&utc,timsTIME_OF_DAY,6,str,&err); str[15]=0; \
      printf("%s [%s] : %s\n",fct,str,txt); }
#define MCM_LOG1(fct,fmt,p1) \
    { ccsTIMEVAL utc; timsMODE tm; ccsERROR err; vltBYTES256 msg,str; \
      errResetStack(&err); timsGetUTC(&utc,&tm,&err); \
      timsTimeToIsoString(&utc,timsTIME_OF_DAY,6,str,&err); str[15]=0; \
      sprintf(msg,"%s [%s] : ",fct,str); sprintf(str,fmt,p1); \
      printf("%s%s\n",msg,str); }
#define MCM_LOG2(fct,fmt,p1,p2) \
    { ccsTIMEVAL utc; timsMODE tm; ccsERROR err; vltBYTES256 msg,str; \
      errResetStack(&err); timsGetUTC(&utc,&tm,&err); \
      timsTimeToIsoString(&utc,timsTIME_OF_DAY,6,str,&err); str[15]=0; \
      sprintf(msg,"%s [%s] : ",fct,str); sprintf(str,fmt,p1,p2); \
      printf("%s%s\n",msg,str); }
#define MCM_LOG3(fct,fmt,p1,p2,p3) \
    { ccsTIMEVAL utc; timsMODE tm; ccsERROR err; vltBYTES256 msg,str; \
      errResetStack(&err); timsGetUTC(&utc,&tm,&err); \
      timsTimeToIsoString(&utc,timsTIME_OF_DAY,6,str,&err); str[15]=0; \
      sprintf(msg,"%s [%s] : ",fct,str); sprintf(str,fmt,p1,p2,p3); \
      printf("%s%s\n",msg,str); }

#ifdef __cplusplus
}
#endif

#endif /* if !MOT_PUBLIC_H */

/*****************************************************************************/

/*___oOo___*/
