/*
 *            Copyright (c) 2000 - 2005 by electronic system design gmbh
 *
 *  This software is copyrighted by and is the sole property of 
 *  esd gmbh.  All rights, title, ownership, or other interests
 *  in the software remain the property of esd gmbh. This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of esd gmbh.
 *
 *  esd gmbh, reserves the right to modify this software without notice.
 *
 *  electronic system design gmbh          Tel. +49-511-37298-0
 *  Vahrenwalder Str 207                   Fax. +49-511-37298-68 
 *  30165 Hannover                         http://www.esd-electronics.com
 *  Germany                                sales@esd-electronics.com
 *
 *************************************************************************
 *
 *     Module Name: psys.h
 *         Version: 1.00
 *   Original Date:  26-Apr-2000
 *     Host system: N/A
 *   Target system: N/A
 *          Author: Michael Tasche
 *        Language: ANSI C
 * Compile Options: 
 * Compile defines: linux          - Compile for Linux
 *                  VXWORKS        - Compile for VxWorks
 *                  _WIN32         - Compile for Win32
 *                  __QNXNTO__     - Compile for QNX Neutrino
 *                  __RTOSUH__     - Compile for RTOS-UH
 *                  __RTOSUHPPC__  - Compile for RTOS-UH (PPC target)
 *                  RM3            - Compile for RMOS
 *
 * Description.
 * =======================================================================
 *  Portable SYStem interface to provide common set of data types, defines,
 *  macros and service that cover operating system and compiler differences.
 *
 */
#ifndef _PSYS_H_
#define _PSYS_H_

/***************************************************************************/
/*                  INCLUDE OS SPECIFIC HEADER                             */
/***************************************************************************/

#if defined(_WIN32)
# include <psyswin.h>
#elif defined(linux)
# include <psyslinux.h>
#elif defined(__QNXNTO__)
# include <psysnto.h>
#elif defined(VXWORKS)
# include <psysvx.h>
#elif defined(__RTOSUH__) || defined(__RTOSUHPPC__)
# include <psysrtos.h>
#elif defined(RM3)
# include <psysrmos.h>
#else
# error "Operating system undefined"
#endif

   /* Check if byte order is defined in platform header */
#ifndef PSYS_BYTE_ORDER
# error !!! Host byteorder undefined !!!!
#endif

   /* Check if register size is defined in platform header */
#ifndef PSYS_REG_SIZE
# error !!! Host register size in bits undefined !!!!
#endif

/***************************************************************************/
/*                          DEFINES                                        */
/***************************************************************************/

/*
 *	Max message size of PSYS message passing
 */
#define PSYS_MSGSIZE 16100

/*
 *	Values for PSYS_BYTE_ORDER
 */
#define PSYS_ENDIAN_LITTLE 0
#define PSYS_ENDIAN_BIG    1

/*
 *  Limits and values for member 'prio' of struct THREADATTR
 */
#define PSYS_PRIO_INHERIT    -1  /* inherit from creator */
#define PSYS_PRIO_MIN        0
#define PSYS_PRIO_MAX        31

/*
 * Flags for member 'flags' of struct THREADATTR
 */
#define PSYS_THREAD_NOFLOAT (1 <<16)
#define PSYS_THREAD_FLOAT   (0 <<16)
#define PSYS_THREAD_NAME    (1 <<31)

/* 
 * This is the absolute maximum for the length of threadnames with psys.
 * There might be systems, which allow shorter threadnames, only.
 */
#define PSYS_THREADNAME_LENGTH   64

/*
 *	Flags of psysTimerSet()
 */
#define PSYS_TIME_ABS         1
#define PSYS_TIME_REL         0

/*
 *  Flags for psysMemProbe()
 */
#define PSYS_VERIFY_READ    (1<<0)
#define PSYS_VERIFY_WRITE   (1<<1)
#define PSYS_VERIFY_STRING  (1<<2)

/*
 * Arguments for type parameter of psysLogConfig()
 */
#define PSYS_LOG_TYPE_FILE    1
#define PSYS_LOG_TYPE_CONSOLE 2
#define PSYS_LOG_TYPE_SYSLOG  3

/*
 *	Constants to convert PSYS time values from ns to different units
 */
#define N2USEC (UINT64)(1000L)
#define N2MSEC (1000*N2USEC)
#define N2SEC  (1000*N2MSEC)
#define N2MIN  (60*N2SEC)
#define N2HOUR (60*N2MIN)
#define N2DAY  (24*N2HOUR)

/*
 *	Define CALLTYPE, EXPORT and IMPORT if not defined in platform header
 */
#ifndef CALLTYPE
#define CALLTYPE
#endif
#ifndef EXPORT
# define EXPORT
#endif
#ifndef IMPORT
# define IMPORT extern
#endif

/*
 *	Define NULLHANDLE
 */
#if !defined(NULLHANDLE) || (NULLHANDLE != 0)
#define NULLHANDLE	0
#endif

/*
 * PSYS variable markup if supported by compiler
 */
#define _PSYS_UNUSED __attribute__((unused))
#define _PSYS_PACKED __attribute__((packed))

#ifndef __GNUC__
# define __attribute__(x)      /* Ignore GCC specials */
#endif

/*
 * Define API return values for error and success.
 */
#define PSYS_OK     0
#define PSYS_ERROR -1

/*
 * If the target platform supports 'errno' and the standard POSIX error
 * codes we use these error numbers for PSYS error codes, too. Otherwise
 * the error return codes are defined in the platform header.
 */
#ifndef PSYS_NO_POSIX_ERROR

/*............psys error-codes...............................................*/
#define PSYS_EPERM            1  /* Not owner                                */
#define PSYS_ENOENT           2  /* No such file or directory                */
#define PSYS_ESRCH            3  /* No such process                          */
#define PSYS_EINTR            4  /* Interrupted system call                  */
#define PSYS_EIO              5  /* I/O error                                */
#define PSYS_ENXIO            6  /* No such device or address                */
#define PSYS_E2BIG            7  /* Arg list too big                         */
#define PSYS_ENOEXEC          8  /* Exec format error                        */
#define PSYS_EBADF            9  /* Bad file number                          */
#define PSYS_ECHILD          10  /* No child processes                       */
#define PSYS_EAGAIN          11  /* Resource unavailable, try again          */
#define PSYS_ENOMEM          12  /* Not enough space                         */
#define PSYS_EACCES          13  /* Permission denied                        */
#define PSYS_EFAULT          14  /* Bad address                              */
#define PSYS_ENOTBLK         15  /* Block device required                    */
#define PSYS_EBUSY           16  /* Device or resource busy                  */
#define PSYS_EEXIST          17  /* File exists                              */
#define PSYS_EXDEV           18  /* Cross-device link                        */
#define PSYS_ENODEV          19  /* No such device                           */
#define PSYS_ENOTDIR         20  /* Not a directory                          */
#define PSYS_EISDIR          21  /* Is a directory                           */
#define PSYS_EINVAL          22  /* Invalid argument                         */
#define PSYS_ENFILE          23  /* File table overflow                      */
#define PSYS_EMFILE          24  /* Too many open files                      */
#define PSYS_ENOTTY          25  /* Inappropriate I/O control operation      */
#define PSYS_ETXTBSY         26  /* Text file busy                           */
#define PSYS_EFBIG           27  /* File too large                           */
#define PSYS_ENOSPC          28  /* No space left on device                  */
#define PSYS_ESPIPE          29  /* Illegal seek                             */
#define PSYS_EROFS           30  /* Read-only file system                    */
#define PSYS_EMLINK          31  /* Too many links                           */
#define PSYS_EPIPE           32  /* Broken pipe                              */
#define PSYS_EDOM            33  /* Math argument out of domain of function  */
#define PSYS_ERANGE          34  /* Result too large                         */
#define PSYS_ENOMSG          35  /* No message of desired type               */
#define PSYS_EIDRM           36  /* Identifier removed                       */
#define PSYS_ECHRNG          37  /* Channel number out of range              */
#define PSYS_EL2NSYNC        38  /* Level 2 not synchronized                 */
#define PSYS_EL3HLT          39  /* Level 3 halted                           */
#define PSYS_EL3RST          40  /* Level 3 reset                            */
#define PSYS_ELNRNG          41  /* Link number out of range                 */
#define PSYS_EUNATCH         42  /* Protocol driver not attached             */
#define PSYS_ENOCSI          43  /* No CSI structure available               */
#define PSYS_EL2HLT          44  /* Level 2 halted                           */
#define PSYS_EDEADLK         45  /* Deadlock avoided                         */
#define PSYS_ENOLCK          46  /* No locks available in system             */
#define PSYS_ECANCELED       47  /* Operation canceled (1003.1b-1993)        */
#define PSYS_ENOTSUP         48  /* Not supported (1003.1b-1993)             */

#define PSYS_EDQUOT          49  /* Disc quota exceded                       */

#if 1 /* Convergent Error Returns */
#define PSYS_EBADE           50  /* Invalid exchange                         */
#define PSYS_EBADR           51  /* Invalid request descriptor               */
#define PSYS_EXFULL          52  /* Exchange full                            */
#define PSYS_ENOANO          53  /* No anode                                 */
#define PSYS_EBADRQC         54  /* Invalid request code                     */
#define PSYS_EBADSLT         55  /* Invalid slot                             */
#define PSYS_EDEADLOCK       56  /* File locking deadlock                    */
#endif

#define PSYS_EBFONT          57  /* Bad font file format                     */

#if 1 /* stream problems */
#define PSYS_ENOSTR          60  /* Device not a stream                      */
#define PSYS_ENODATA         61  /* No data (for no delay io)                */
#define PSYS_ETIME           62  /* Timer expired                            */
#define PSYS_ENOSR           63  /* Out of streams resources                 */
#endif

#define PSYS_ENONET          64  /* Machine is not on the network            */
#define PSYS_ENOPKG          65  /* Package not installed                    */
#define PSYS_EREMOTE         66  /* The object is remote                     */
#define PSYS_ENOLINK         67  /* The link has been severed                */
#define PSYS_EADV            68  /* Advertise error                          */
#define PSYS_ESRMNT          69  /* Srmount error                            */

#define PSYS_ECOMM           70  /* Communication error on send              */
#define PSYS_EPROTO          71  /* Protocol error                           */
#define PSYS_EMULTIHOP       74  /* multihop attempted                       */
#define PSYS_EBADMSG         77  /* Bad message (1003.1b-1993)               */
#define PSYS_ENAMETOOLONG    78  /* Name too long                            */
#define PSYS_EOVERFLOW       79  /* Value too large to be stored in data type */
#define PSYS_ENOTUNIQ        80  /* Given name not unique                    */
#define PSYS_EBADFD          81  /* FD invalid for this operation            */
#define PSYS_EREMCHG         82  /* Remote address changed                   */

/* --- Shared library errors --- */

#define PSYS_ELIBACC         83  /* Can't access shared library              */
#define PSYS_ELIBBAD         84  /* Accessing a corrupted shared library     */
#define PSYS_ELIBSCN         85  /* .lib section in a.out corrupted          */
#define PSYS_ELIBMAX         86  /* Attempting to link in too many libraries */
#define PSYS_ELIBEXEC        87  /* Attempting to exec a shared library      */
#define PSYS_EILSEQ          88  /* Illegal byte sequence.                   */

/* ----------------------------- */

#define PSYS_ENOSYS          89  /* Unknown system call                      */
#define PSYS_ELOOP           90  /* Too many symbolic link or prefix loops   */
#define PSYS_ERESTART        91  /* Restartable system call                  */
#define PSYS_ESTRPIPE        92  /* if pipe/FIFO, don't sleep in stream head */
#define PSYS_ENOTEMPTY       93  /* Directory not empty                      */
#define PSYS_EUSERS          94  /* Too many users (for UFS)                 */

#define PSYS_EOPNOTSUPP      103 /* Operation not supported                  */

#define PSYS_ESTALE          122 /* Potentially recoverable i/o error        */

/* --- Sockets --- */
/* non-blocking and interrupt i/o */
/*        EAGAIN           35        */    /* Resource temporarily unavailable */
/*        EWOULDBLOCK      35        */    /* Operation would block */
#define PSYS_EWOULDBLOCK    EAGAIN              /* Operation would block */
#define PSYS_EINPROGRESS       236              /* Operation now in progress */
/*        EALREADY         37        */    /* Operation already in progress */
#define PSYS_EALREADY        EBUSY              /* Operation already in progress */

/* ipc/network software -- argument errors */
#define PSYS_ENOTSOCK        238                /* Socket operation on non-socket */
#define PSYS_EDESTADDRREQ    239                /* Destination address required */
#define PSYS_EMSGSIZE        240                /* Inappropriate message buffer length */
#define PSYS_EPROTOTYPE      241                /* Protocol wrong type for socket */
#define PSYS_ENOPROTOOPT     242                /* Protocol not available */
#define PSYS_EPROTONOSUPPORT 243                /* Protocol not supported */
#define PSYS_ESOCKTNOSUPPORT 244                /* Socket type not supported */
#define PSYS_EPFNOSUPPORT    246                /* Protocol family not supported */
#define PSYS_EAFNOSUPPORT    247                /* Address family not supported by protocol family */
#define PSYS_EADDRINUSE      248                /* Address already in use */
#define PSYS_EADDRNOTAVAIL   249                /* Can't assign requested address */

/* ipc/network software -- operational errors */
#define PSYS_ENETDOWN        250                /* Network is down */
#define PSYS_ENETUNREACH     251                /* Network is unreachable */
#define PSYS_ENETRESET       252                /* Network dropped connection on reset */
#define PSYS_ECONNABORTED    253                /* Software caused connection abort */
#define PSYS_ECONNRESET      254                /* Connection reset by peer */
#define PSYS_ENOBUFS         255                /* No buffer space available */
#define PSYS_EISCONN         256                /* Socket is already connected */
#define PSYS_ENOTCONN        257                /* Socket is not connected */
#define PSYS_ESHUTDOWN       258                /* Can't send after socket shutdown */
#define PSYS_ETOOMANYREFS    259                /* Too many references: can't splice */
#define PSYS_ETIMEDOUT       260                /* Connection timed out */
#define PSYS_ECONNREFUSED    261                /* Connection refused */

/*      ELOOP          62        */        /* Too many levels of symbolic links */
/*      ENAMETOOLONG   63        */        /* File name too long */

#define PSYS_EHOSTDOWN        264                /* Host is down */
#define PSYS_EHOSTUNREACH     265                /* No route to host */

/* Network File System */
/*      ESTALE           70     */          /* Stale NFS file handle */
/*      EREMOTE          71     */          /* Too many levels of remote in path */
#define PSYS_EBADRPC         272                 /* RPC struct is bad */
#define PSYS_ERPCMISMATCH    273                 /* RPC version wrong */
#define PSYS_EPROGUNAVAIL    274                 /* RPC prog. not avail */
#define PSYS_EPROGMISMATCH   275                 /* Program version wrong */
#define PSYS_EPROCUNAVAIL    276                 /* Bad procedure for program */

#define PSYS_E_MLC_BASE      0x900

#define PSYS_E_CAN_BASE      0x1000

#endif /* of !PSYS_NO_POSIX_ERROR_CODES */

/* INT should correspond to s system's int-type (16Bit on        */
/* 16Bit platforms, 32Bit on 32Bit platforms, but with           */
/* respect to Linux and Windows also 32Bit on 64Bit platforms.   */
#ifndef INT
# define  INT   signed int
#endif
/* UINT should correspond to s system's unsigned int-type        */
/* (16Bit on 16Bit platforms, 32Bit on 32Bit platforms, but with */
/* respect to Linux and Windows also 32Bit on 64Bit platforms.   */
#ifndef UINT
# define  UINT  unsigned int
#endif
#ifndef VOID
# define  VOID  void
#endif
#ifndef CHAR
# define  CHAR  char
#endif

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************/
/*                          TYPEDEFS                                       */
/***************************************************************************/

   /* PSYS time */
typedef union
{
    UINT64   nsec;
    struct
    {
#if PSYS_BYTE_ORDER == PSYS_ENDIAN_LITTLE
        UINT32 nsec_low;
        UINT32 nsec_high;
#endif
#if PSYS_BYTE_ORDER == PSYS_ENDIAN_BIG
        UINT32 nsec_high;
        UINT32 nsec_low;
#endif
    } l;
} TIME_CS;

   /* PSYS timer */
typedef struct
{
    TIME_CS value;
    TIME_CS interval;
} TIMERSPEC;

  /* PSYS thread attributes */
typedef struct
{
    INT32  prio;            /* Thread priority  */
    INT32  stackSize;       /* Thread stacksize */
    UINT32 flags;           /* Thread flags      */
    CHAR   *name;           /* Thread name      */
} THREADATTR;

typedef struct
{
    INT32 scoid;
    INT32 msglen;
    INT32 srcmsglen;
    INT32 dstmsglen;
    INT32 prio;
} PSYS_MSG_INFO;

   /* PSYS version info */
typedef struct
{
    UINT16 version;           /* Version       */
    UINT16 features;          /* Feature flags */
    UINT8  zero[12];          /* Always 0      */
    CHAR   build[64];         /* Build string  */
} PSYS_INFO;

/***************************************************************************/
/*                 COMMON MACROS                                           */
/***************************************************************************/

/*
 * If the target platform supports 'errno' and the standard POSIX error
 * codes we use errno to return errors. Otherwise these macros are defined
 * in the platform specific header.
 */
#ifndef PSYS_NO_POSIX_ERROR
# define PSYS_ERRNO_GET()    errno
# define PSYS_ERRNO_SET(err) errno = err
#endif

#undef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#undef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#undef abs
#define abs(a) ((a) >= 0 ? (a) : -(a))
#undef odd
#define odd(n) ((n) & 1)


    /* Sleep for given number of us */
#define PSYS_SLEEP_USEC(us) {TIME_CS __time; \
                             __time.nsec = (us)*N2USEC;\
                             psysSleep(&__time);}

    /* Sleep for given number of ms */
#define PSYS_SLEEP_MSEC(ms) {TIME_CS __time; \
                             __time.nsec = (ms)*N2MSEC;\
                             psysSleep(&__time);}

    /* Sleep for given number of seconds */
#define PSYS_SLEEP_SEC(s) {TIME_CS __time; \
                           __time.nsec = (s)*N2SEC;\
                           psysSleep(&__time);}

/***************************************************************************/
/*                          EXPORTS                                        */
/***************************************************************************/

EXPORT INT CALLTYPE psysMutexInit(                   /* ret 0, if success    */
                      MUTEX  *mutex );               /* ->Mutex              */


EXPORT INT CALLTYPE psysMutexDestroy(                /* ret 0, if success    */
                      MUTEX *mutex );                /* ->Mutex              */


EXPORT INT CALLTYPE psysMutexLock(                   /* ret 0, if success    */
                      MUTEX *mutex );                /* ->mutex              */


EXPORT INT CALLTYPE psysMutexUnlock(                 /* ret 0, if success    */
                      MUTEX *mutex );                /* semaphore-handle     */


EXPORT INT CALLTYPE psysCondInit(                    /* ret 0, if success    */
                      COND *cond );                  /* ->cond-var           */


EXPORT INT CALLTYPE psysCondDestroy(                 /* ret 0, if success    */
                      COND *cond );                  /* ->cond               */


EXPORT INT CALLTYPE psysCondWait(                    /* ret 0, if success    */
                      COND   *cond,                  /* ->cond               */
                      MUTEX  *mutex );


EXPORT INT CALLTYPE psysCondSignal(                  /* ret 0, if success    */
                      COND  *cond );                 /* semaphore-handle     */


EXPORT INT CALLTYPE psysSleep(                       /* ret 0, if success         */
                      TIME_CS  *time );                 /* delay-time in nanoseconds */


EXPORT INT CALLTYPE psysThreadCreate(
                      THREAD        *thread,
                      THREADATTR    *attr,
                      INT CALLTYPE  (*func)(VOID *),
                      VOID          *arg );


EXPORT INT CALLTYPE psysTimeGet(
                      TIME_CS *time );


EXPORT INT CALLTYPE psysChannelCreate(
	              const CHAR *name,              /* name       */
		      UINT32     flags );            /* not used   */


EXPORT INT CALLTYPE psysChannelDestroy(
		      INT chid );


EXPORT INT CALLTYPE psysMsgReceive(
		      INT            chid,
		      VOID           *data,
		      INT            len,
		      PSYS_MSG_INFO  *msg_info );


EXPORT INT CALLTYPE psysMsgReply(
		      INT          rcvid,
 		      INT          status,
		      const  VOID  *data,
		      INT          len );


EXPORT INT CALLTYPE psysConnectAttach(
		      const CHAR  *name );


EXPORT INT CALLTYPE psysConnectDetach(
		      INT  coid );


EXPORT INT CALLTYPE psysMsgSend(
		      INT         coid,
		      const VOID  *sdata,
		      INT         slen,
		      VOID        *rdata,
		      INT         rlen );


/*...read node/group-info.......................................................*/
EXPORT INT CALLTYPE psysInfo(                        /* returns PSYS_OK or PSYS_ERROR*/
                      PSYS_INFO  *info,              /* -> info-struct               */
                      INT        len );


EXPORT INT CALLTYPE psysTimerCreate(                 /* returns id or PSYS_ERROR     */
                      VOID  CALLTYPE(*func)(INT32, VOID*),
                      VOID  *arg );


EXPORT INT CALLTYPE psysTimerDelete(                 /* PSYS_ERROR or PSYS-OK        */
                      INT  id );


EXPORT INT CALLTYPE psysTimerSet(                    /* returns PSYS_OK or PSYS_ERROR */
                      INT              id,           /* timer ID                      */
                      INT32            flags,        /* absolute or relative          */
                      const TIMERSPEC  *timerSpec,   /* relative or absolute or NULL  */
                      TIMERSPEC        *oldTimer );  /* returns remaining time        */


EXPORT INT CALLTYPE psysTimerGet(                    /* returns PSYS_OK or PSYS_ERROR */
                      INT        id,                 /* timer ID                      */
                      TIMERSPEC  *timerSpec );       /* returns remaining time        */


EXPORT INT CALLTYPE psysObjectCopy(                  /* returns PSYS_OK or PSYS_ERROR */
		      const CHAR  *src,              /* data source ptr               */
		      CHAR        *dest,             /* data destination ptr          */
		      const CHAR  *copyTemplate,     /* copy descriptor template      */
                      INT         byteOrder );       /* Destination byte order        */


EXPORT VOID CALLTYPE psysLog(          /* PSYS_ERROR or count of printed chars  */ 
                       UINT32      mask,               
		       const CHAR  *format,
		       ... );


EXPORT VOID CALLTYPE psysLogMaskSet(     
                       UINT32  mask );


EXPORT INT CALLTYPE psysLogConfig(     
                      UINT32      type,         
                      const CHAR  *path );       


EXPORT VOID* CALLTYPE psysMalloc(
                        UINT  size );


EXPORT VOID* CALLTYPE psysCalloc( 
                        UINT  nmemb,
                        UINT  size );


EXPORT VOID* CALLTYPE psysRealloc(
                        VOID  *ptr,
                        UINT  size );


EXPORT VOID CALLTYPE psysFree( 
                       VOID  *ptr );


EXPORT INT CALLTYPE psysMemProbe(
                      UINT8  *addr,
                      INT32  flags,
                      UINT   size,
                      UINT   align );


EXPORT INT CALLTYPE psysTimezoneSet(
                      const CHAR *timezone );
                                 

#ifdef __cplusplus
}
#endif

#endif /* of _PSYS_H_ */
