#ifndef RMN_COMMANDS_H
#define RMN_COMMANDS_H
/******************************************************************************
* E.S.O - VLT project
*
* "@(#) $Id: rmnCommands.h,v 1.12 2002/04/29 08:36:02 vltsccm Exp $" 
*
* rmnCommands.h - ioctl control command literals of RMN driver
*
* who      when      what
* -------- --------  ----------------------------------------------
* mverola  01/08/97  created
*----------------------------------------------------------------------------
*/

#include "semLib.h"

/*
 * Interrupt Commands
 */
#define rmnCMD_SEND_INTERRUPT                   100
#define rmnCMD_ENABLE_INTERRUPT                 101
#define rmnCMD_DISABLE_INTERRUPT                102
#define rmnCMD_ENABLE_INT_AUTOCLEAR             103
#define rmnCMD_DISABLE_INT_AUTOCLEAR            104
#define rmnCMD_CLEAN_INT_FIFO                   105
#define rmnCMD_CONFIG_INTERRUPT                 106
#define rmnCMD_GET_INT_CONFIG                   107
#define rmnCMD_RESET_INTERRUPT                  108

/*
 * Status Commands
 */
#define rmnCMD_GET_STATUS                       110
#define rmnCMD_SET_STATUS                       111
#define rmnCMD_GET_FAIL_LED                     112
#define rmnCMD_SET_FAIL_LED                     113
#define rmnCMD_GET_NODE_ID                      114

/*
 * Memory Map Commands
 */
#define rmnCMD_ADDR_GET_REGS_MEM_BASE           120
#define rmnCMD_ADDR_GET_USER_MEM_BASE           121
#define rmnCMD_ADDR_GET_USER_MEM_TOP            122

/*
 * Test Commands
 */
#define rmnCMD_TEST_BOARD                       130
#define rmnCMD_TEST_LINK                        131

/*
 * Device Reset Commands
 */
#define rmnCMD_RESET_DEVICE			140

/*
 * General Driver Commands
 */
#define rmnCMD_GET_DRIVER_VERSION	        150
#define rmnCMD_GET_TIMEOUT		        151
#define rmnCMD_SET_TIMEOUT			152
#define rmnCMD_FREE_DEVICE			153

/* Special code to send an interrupt to every node 
   (command rmnCMD_SEND_INTERRUPT) */
#define rmnINT_BROADCAST     256  

/* Defines for some struct values */
#define rmnISR_CONTEXT  0  /* user-provided ISR runs at 
                            * interrupt code level
			    * (it is called by the driver 
			    * interrupt handler)
			    */
#define rmnTASK_CONTEXT 1  /* user-provided ISR runs at 
                            * task level using semaphore 
			    * synchronization with the driver
			    * interrupt handler
			    */

typedef void (rmnUSER_ISR)(void *userArg, int senderID);

typedef	struct /* command rmnCMD_CONFIG_INTERRUPT */
{
    int          source;    /* must be in the range [1..3] */
    int          mode;      /* either rmnISR_CONTEXT or rmnTASK_CONTEXT */
    rmnUSER_ISR *userISR;   /* pointer to user ISR         */
    void        *userArg;   /* pointer to user args        */
    SEM_ID       semID;	    /* semaphore for task-level ISR's
			     * (mode must be rmnTASK_CONTEXT)
			     */
    int          autoClear; /* set the autoClear bit       */
    int          cleanFIFO; /* clean the interrupt FIFO    */
} rmnINT_CONFIG_DATA;

typedef	struct /* command rmnCMD_GET_INT_CONFIG */
{
    int          source;    /* must be in the range [0..3] */
    int          used;      /* interrupt is configured [1=YES/0=NO] */
    int          enable;    /* enable bit state    [1=enabled/0=disabled] */
    int          autoClear; /* autoClear bit state [1=enabled/0=disabled] */
    int          mode;      /* either rmnISR_CONTEXT or rmnTASK_CONTEXT */
    rmnUSER_ISR *userISR;   /* pointer to user ISR         */
    void        *userArg;   /* pointer to user args        */
    SEM_ID       semID;	    /* semaphore for task-level ISR's
			     * (mode must be rmnTASK_CONTEXT) */
    uint32_t     n_Sent;    /* Number of interrupts sent    */
    uint32_t     n_Served;  /* Number of interrupts served  */ 
    uint32_t     n_SemGive; /* Number of semaphores given   */
    uint32_t     n_UserISR; /* Number of call to userISR    */
} rmnINT_GET_CONFIG_DATA;

typedef	struct /* command rmnCMD_SEND_INTERRUPT */
{
    int source;    /* must be in the range [1..3] */
    int targetID;  /* must be in the range [0..255] 
	            * or rmnINT_BROADCAST 
	            */
} rmnINT_SEND_DATA;

#endif /* !RMN_COMMANDS_H */
