/*
 *            Copyright (c) 1996 - 2005 by electronic system design gmbh
 *
 *  This software is copyrighted by and is the sole property of 
 *  esd gmbh.  All rights, title, ownership, or other interests
 *  in the software remain the property of esd gmbh. This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of esd gmbh.
 *
 *  esd gmbh, reserves the right to modify this software without notice.
 *
 *  electronic system design gmbh          Tel. +49-511-37298-0
 *  Vahrenwalder Str 205                   Fax. +49-511-37298-68 
 *  30165 Hannover                         http://www.esd-electronics.com
 *  Germany                                sales@esd-electronics.com
 *
 *************************************************************************
 *
 *     Module Name: scanopen.h
 *         Version: 1.00
 *   Original Date:  18-Sep-1996
 *     Host system: N/A
 *   Target system: N/A
 *          Author: 
 *        Language: ANSI C
 * Compile Options: 
 * Compile defines: WIN32
 *                     Include on Win 9x/NT/2K/XP
 *
 *                  __RTOSUH__ and __RTOSUHPPC__
 *                     Include on RTOS-UH for 68K or PPC
 *
 *                  RM3
 *                     Include on RMOS
 *
 * Description.
 * =======================================================================
 * API for CANopen slave applications according to CiA standard 
 * DS-301 Rev. 4
 *
 */
#ifndef __INCscanOpenh
#define __INCscanOpenh

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CANOPEN
# define CANOPEN
#endif

#define VXWORKS
#undef  VW54
#define VW55

#include <psys.h>   /* Include PSYS support for portable data types */
    
#ifdef WIN32
# pragma comment(lib,"scanopen.lib")	/* Link with SCANOPEN.LIB (Only for VC++) */
#include <windows.h>
# ifndef EXPORT
#  define EXPORT __declspec (dllexport)  /* Macro for DLL functions */
# endif
#else 
# ifdef EXPORT
#  undef EXPORT
#  define EXPORT
# else
#  define EXPORT
# endif
#endif

#if defined(__RTOSUH__) || defined(__RTOSUHPPC__)
#define canOpenGetDictionaryHnd    canOpenGetDictionaryH
#define canOpenReadDictionaryHnd   canOpenReadDictionaryH
#define canOpenWriteDictionaryHnd  canOpenWriteDictionaryH
#define canCalServerBlockDownload  canCalServerBlockDnload
#define nmtStartErrorControlService nmtStartErrCtlService
#define nmtStopErrorControlService  nmtStopErrCtlService
#define nmtSignalNodeStateChange   nmtSignalNodeStateChg
#endif

#ifdef RM3
#pragma nooptimize
static  int rmos_avoid_invalid_handler(int dummy) 
  {
    dummy ++;
    return dummy;
  }
#pragma optimize(2)

#endif
  
#if !defined(CALLTYPE)
# define CALLTYPE
#endif

	/* CANopen slave handle types */

typedef int     HNODE;      /* Type of node handle */
typedef int     HPDO;       /* Type of PDO handle */
typedef int     HSDO;       /* Type of SDO (client) handle */
typedef void *  HDICT;      /* Type of handle to dictionary entry */

/* Definition of static data types in the object dictionary */

typedef unsigned char   can_bool;
typedef unsigned char   can_uint_8;
typedef unsigned short  can_uint_16;
typedef unsigned long   can_uint_32;
typedef char            can_int_8;
typedef short           can_int_16;
typedef long            can_int_32;
typedef float           can_float;
typedef double          can_double;
typedef UINT64          can_uint_64;
typedef INT64           can_int_64;
typedef unsigned char   can_uint_24[3];
typedef char            can_int_24[3];
#if PSYS_BYTE_ORDER == PSYS_ENDIAN_LITTLE

typedef struct {
    unsigned char   val[5];
    unsigned char   pad[3];
} can_uint_40;

typedef struct {
    char            val[5];
    char            pad[3];
} can_int_40;

typedef struct {
    unsigned char   val[6];
    unsigned char   pad[2];
} can_uint_48;

typedef struct {
    char            val[6];
    char            pad[2];
} can_int_48;

typedef struct {
    unsigned char   val[7];
    unsigned char   pad[1];
} can_uint_56;

typedef struct {
    char            val[7];
    char            pad[1];
} can_int_56;

#else

typedef struct {
    unsigned char   pad[3];
    unsigned char   val[5];
} can_uint_40;

typedef struct {
    char            pad[3];
    char            val[5];
} can_int_40;

typedef struct {
    unsigned char   pad[2];
    unsigned char   val[6];
} can_uint_48;

typedef struct {
    char            pad[2];
    char            val[6];
} can_int_48;

typedef struct {
    unsigned char   pad[1];
    unsigned char   val[7];
} can_uint_56;

typedef struct {
    char            pad[1];
    char            val[7];
} can_int_56;

#endif

/* Definition of static data types in the object directory */
/* derived from cms extended data types                    */

typedef char *		can_octet_string;
typedef char *		can_visible_string;    /* Same as octet string */
					       /* because this is not PASCAL */
typedef struct {
  can_uint_16 ms;		/* miliseconds  0..59999 */
  can_uint_8  min;		/* minutes      0..59    */ 
  can_uint_8  hour;		/* hour         0..23    */
  can_bool    su;		/* summer time (0=standard, 1=summer) */
  can_uint_8  day_of_month;	/* day of month 1..31    */
  can_uint_8  day_of_week;	/* day of week (1=monday, 7=sunday) */
  can_uint_8  month;		/* month        1..12    */ 
  can_uint_8  year;		/* year modulo centuries 0..99 */
} can_date;

typedef struct {
  can_uint_32 ms;	/* miliseconds after midnight */
  can_uint_16 days;	/* number of days since Jan 1, 1984 */
} can_time_of_day;
 
typedef can_time_of_day		can_time_difference;

	/*
     * Object types of dictionary objects
     */

#define OBJ_NULL	    0	/* A dictionary entry with no data fields */
#define OBJ_DOMAIN	    2	/* Large variable amount of data */
#define OBJ_DEFTYPE	    5	/* Definition of a simple data type */
#define OBJ_DEFSTRUCT	6	/* Definition of complex record type */
#define OBJ_VAR		    7	/* Single value */
#define OBJ_ARRAY	    8	/* Multiple data field of same type */
#define OBJ_RECORD	    9	/* Multiple data field of different types */

	/*
     * Supported data types for dictionary objects
     */

#define TYP_BOOLEAN          1
#define TYP_INT8             2
#define TYP_INT16            3
#define TYP_INT32            4
#define TYP_UINT8            5 
#define TYP_UINT16           6
#define TYP_UINT32           7
#define TYP_FLOAT            8   
#define TYP_REAL32           TYP_FLOAT
#define TYP_VIS_STRING	     9
#define TYP_OCT_STRING      10
#define TYP_DATE            11
#define TYP_TIME_OF_DAY     12
#define TYP_TIME_DIFF       13
#define TYP_BIT_STRING      14
#define TYP_DOMAIN          15
#define TYP_INT24           16
#define TYP_REAL64          17
#define TYP_INT40           18
#define TYP_INT48           19
#define TYP_INT56           20
#define TYP_INT64           21
#define TYP_UINT24          22
#define TYP_UINT40          24
#define TYP_UINT48          25
#define TYP_UINT56          26
#define TYP_UINT64          27
#define TYP_PDO_COMM_PAR    32
#define TYP_PDO_MAPPING     33
#define TYP_SDO_PARAMETER   34
#define TYP_IDENTITY        35


     /*
	  * Macros to sign extend integer data types which are defined in DS-301
      * V4.x but can not be mapped into standard C data types.
	  */

#define SIGN_EXTEND_INT24(val) (((val) << 8)  >> 8)   /* val of type INT32 */
#define SIGN_EXTEND_INT40(val) (((val) << 24) >> 24)  /* val of type INT64 */
#define SIGN_EXTEND_INT48(val) (((val) << 16) >> 16)  /* val of type INT64 */
#define SIGN_EXTEND_INT56(val) (((val) << 8)  >> 8)   /* val of type INT64 */

     /*
	  * Predefined dictionary mapping types for OBJ_VAR and OBJ_ARRAY
	  * that can be used for canOpenExtenDictionary() for parameter DataType
	  */

#define MAP_BOOLEAN         "\x01\0"
#define MAP_INT8            "\x02\0"
#define MAP_INT16           "\x03\0"
#define MAP_INT32           "\x04\0"
#define MAP_UINT8			"\x05\0"
#define MAP_UINT16			"\x06\0"
#define MAP_UINT32			"\x07\0"
#define MAP_FLOAT           "\x08\0"
#define MAP_VIS_STRING		"\x09\0"
#define MAP_OCT_STRING      "\x0A\0"
#define MAP_DATE            "\x0B\0"
#define MAP_TIME_OF_DAY     "\x0C\0"
#define MAP_TIME_DIFF       "\x0D\0"
#define MAP_BIT_STRING      "\x0E\0"
#define MAP_DOMAIN          "\x0F\0"
#define MAP_INT24           "\x10\0"
#define MAP_REAL64          "\x11\0"
#define MAP_INT40           "\x12\0" 
#define MAP_INT48           "\x13\0" 
#define MAP_INT56           "\x14\0" 
#define MAP_INT64           "\x15\0"
#define MAP_UINT24          "\x16\0"
#define MAP_UINT40          "\x18\0" 
#define MAP_UINT48          "\x19\0" 
#define MAP_UINT56          "\x1A\0" 
#define MAP_UINT64          "\x1B\0"


	/*
     * Predefined dictionary access attributes
     */

#define ATTRIB_RW		0	/* Read and write access      */
#define ATTRIB_WO		1	/* Write only access	      */
#define ATTRIB_RO		2	/* Read only access	      */
#define ATTRIB_CONST	3	/* Read only access, value is const  */
#define ATTRIB_RWR		4	/* Read/write on process input  */
#define ATTRIB_RWW		5	/* Read/write on process output */

	/*
     * Definitions for the data part of each dictionary entry
     */

typedef struct {
  can_uint_8	val;		/* Value */
  can_uint_8	defval;		/* Deafult value */
} RecBool;

typedef struct {
  can_uint_8	val;		/* Value */
  can_uint_8	defval;		/* Deafult value */
  can_uint_8	high;		/* Upper limit */
  can_uint_8	low;		/* Lower limit */
} RecUint8;

typedef struct {
  can_int_8	val;		/* Value */
  can_int_8	defval;		/* Deafult value */
  can_int_8	high;		/* Upper limit */
  can_int_8	low;		/* Lower limit */
} RecInt8;

typedef struct {
  can_uint_16	val;		/* Value */
  can_uint_16	defval;		/* Deafult value */
  can_uint_16	high;		/* Upper limit */
  can_uint_16	low;		/* Lower limit */
} RecUint16;

typedef struct {
  can_int_16	val;		/* Value */
  can_int_16	defval;		/* Deafult value */
  can_int_16	high;		/* Upper limit */
  can_int_16	low;		/* Lower limit */
} RecInt16;

typedef struct {
#if PSYS_BYTE_ORDER == PSYS_ENDIAN_LITTLE
  can_uint_24	val;		/* Value */
  unsigned char pad1;		/* Padding */
  can_uint_24	defval;		/* Value */
  unsigned char pad2;		/* Padding */
#else
  unsigned char pad1;		/* Padding */
  can_uint_24	val;		/* Value */
  unsigned char pad2;		/* Padding */
  can_uint_24	defval;		/* Value */
#endif
} RecUint24;

typedef struct {
#if PSYS_BYTE_ORDER == PSYS_ENDIAN_LITTLE
  can_int_24	val;		/* Value */
  unsigned char pad1;		/* Padding */
  can_int_24	defval;		/* Value */
  unsigned char pad2;		/* Padding */
#else
  unsigned char pad1;		/* Padding */
  can_int_24	val;		/* Value */
  unsigned char pad2;		/* Padding */
  can_int_24	defval;		/* Value */
#endif  
} RecInt24;

typedef struct {
  can_uint_32	val;		/* Value */
  can_uint_32	defval;		/* Deafult value */
  can_uint_32	high;		/* Upper limit */
  can_uint_32	low;		/* Lower limit */
} RecUint32;

typedef struct {
  can_int_32	val;		/* Value */
  can_int_32	defval;		/* Deafult value */
  can_int_32	high;		/* Upper limit */
  can_int_32	low;		/* Lower limit */
} RecInt32;

typedef struct {
  can_float	val;		/* Value */
  can_float	defval;		/* Deafult value */
  can_float	high;		/* Upper limit */
  can_float	low;		/* Lower limit */
} RecFloat;

typedef struct {
    can_uint_40	val;		/* Current value */
    can_uint_40	defval;		/* Default value */
} RecUint40;

typedef struct {
    can_int_40	  val;		/* Current value */
    can_int_40	  defval;	/* Default value */
} RecInt40;

typedef struct {
    can_uint_48	  val;		/* Current value */
    can_uint_48	  defval;	/* Default value */
} RecUint48;

typedef struct {
    can_int_48	  val;		/* Current value */
    can_int_48	  defval;	/* Default value */
} RecInt48;

typedef struct {
    can_uint_56   val;		/* Current value */
    can_uint_56	  defval;	/* Default value */
} RecUint56;

typedef struct {
    can_int_56	  val;		/* Current value */
    can_int_56	  defval;	/* Default value */
} RecInt56;

typedef struct {
    UINT64	val;			/* Current value */
    UINT64	defval;			/* Default value */
} RecUint64;

typedef struct {
    INT64	val;		    /* Current value */
    INT64	defval;		    /* Default value */
} RecInt64;

typedef struct {
    can_double	val;			/* Current value */
    can_double	defval;			/* Default value */
} RecReal64;

typedef struct {
  can_visible_string	val;	/* Value */
  can_visible_string	defval;	/* Deafult value */
  long			buflen; /* Max buffer size */
  long			len;	/* Size of actual entry */
} RecVisString;

typedef struct {
  can_octet_string	val;	/* Value */
  can_octet_string	defval;	/* Deafult value */
  long			buflen; /* Max buffer size */
  long			len;	/* Size of actual entry */
} RecOctString;

typedef struct {
  void *		val;	/* Value */
  void *		defval;	/* Default value */
  long			buflen; /* Max buffer size */
  long			len;	/* Size of actual entry */
} RecDomain;

    /* CANopen versions Bit 15-12: level, Bit 11-8: revision, Bit 7-0: change */
typedef struct {
  unsigned short cos;    /* Revision of main setup module       */
  unsigned short sdm;    /* Revision of service data management */
  unsigned short pdm;    /* Revision of process data management */
  unsigned short nmt;    /* Revision of NMT module              */
  unsigned short dbt;    /* Reserved for backward compatibility */
  unsigned short cms;    /* Main common slave revision          */                                
  unsigned short sys;    /* Revision of PSYS interface          */                                 
  unsigned short can;    /* Revision of CALCAN library          */
} CANOPEN_VERSIONS;

	/* CANopen emergency frame */

typedef struct {
    unsigned short  errorCode;
    unsigned char	errorRegister;
    unsigned char	errorSpecific[5];
} CANOPEN_EMCY_FRAME;

typedef struct {
	unsigned short  usNetNo;
	unsigned short  usModId;
	unsigned long   ulEvent;
	unsigned long   ulArg1;
	union {
		unsigned long ulArg2;
		void *        pArg2;
	} arg;
} SLAVE_EVENT;

    /* Callback handler */
typedef int (CALLTYPE *PFN_COS_DATA_HANDLER)(int, int, int, int, void*);
typedef int (CALLTYPE *PFN_COS_NODE_HANDLER)(SLAVE_EVENT *);

		/* DS-301 Special meaning of MSBs in COB-ID */

#define PDO_VALID             0x00000000    /* COB id of PDO valid          */
#define PDO_INVALID           0x80000000    /* COB id of PDO not valid      */
#define SDO_VALID             0x00000000    /* COB id of SDO valid          */
#define SDO_INVALID           0x80000000    /* COB id of SDO not valid      */
#define RTR_ALLOW             0x00000000    /* RTR allowed on this PDO      */
#define RTR_DISALLOW          0x40000000    /* no RTR allowed on this PDO   */
#define CAN_11_BIT            0x00000000    /* 11 bit ID (CAN 2.0A)         */
#define CAN_29_BIT            0x20000000    /* 29 bit ID (CAN 2.0B)         */
#define DEFAULT_COB           0x1FFFFFFF    /* Mark default COBid           */
#define EMCY_OBJ_VALID        0x00000000    /* EMCY consumer object valid   */
#define EMCY_OBJ_INVALID      0x80000000    /* EMCY consumer object invalid */

/* Parameter for COB-Id of PDO in canOpenDefinePDO() */

#define DEFAULT_PDO1 0x1FFFFFF0  /* Default PDO1 */
#define DEFAULT_PDO2 0x1FFFFFF1  /* Default PDO2 */
#define DEFAULT_PDO3 0x1FFFFFF2  /* Default PDO3 */
#define DEFAULT_PDO4 0x1FFFFFF3  /* Default PDO4 */
 

#define SYNC_CONSUME 	0x80000000	/* Device consumes sync message */
#define SYNC_PRODUCE    0x40000000	/* Device generate sync message */

#define INSERT_SHORT_IDF(cob, idf) (cob |= (idf & 0x000007FF))
#define INSERT_LONG_IDF (cob, idf) (cob |= (idf & 0x1FFFFFFF))

/*
 * Default values for object dictionary entries that can be stored via CAN
 */
#define DEFAULT_SYNC_COBID              0x00000080
#define DEFAULT_SYNC_CYCLE_PERIOD       0
#define DEFAULT_SYNC_WINDOW_LENGTH      0
#define DEFAULT_GUARD_TIME              0
#define DEFAULT_LIFETIME_FACTOR	        0
#define DEFAULT_TIMESTAMP_COBID         0x00000100
#define DEFAULT_EMCY_COBID              0x00000000
#define DEFAULT_EMCY_INHIBIT_TIME       0
#define DEFAULT_PRODUCER_HEARTBEAT_TIME  0

	/* Now the monster union for all supported data types */

typedef union {
  RecBool		        boolean;
  RecUint8		        uint8;
  RecUint16		        uint16;
  RecUint24		        uint24;
  RecUint32		        uint32;
  RecUint40		        uint40;
  RecUint48		        uint48;
  RecUint56		        uint56;
  RecUint64		        uint64;
  RecInt8		        int8;
  RecInt16		        int16;
  RecInt24		        int24;
  RecInt32		        int32;
  RecInt40		        int40;
  RecInt48		        int48;
  RecInt56		        int56;
  RecInt64		        int64;
  RecFloat		        float32;
  RecReal64		        real64;
  RecOctString		    OctString;
  RecVisString		    VisString;
  RecDomain		        Domain;	
  can_date		        date;
  can_time_of_day	    TimeOfDay;
  can_time_difference	TimeDifference;
  void                  *CustomData;
} DictionaryData, *pDictionaryData;

		/* Flags for canOpenInitDictionary() */

#define LIMITS_VALID	 0x0001		/* Limits of entry valid               */
#define MAPPABLE	     0x0008		/* Entry can be mappable into PDO      */
#define READ_ACCESS	     0x0010		/* Read access allowed for this entry  */
#define WRITE_ACCESS	 0x0020		/* Write access allowed for this entry */
#define PERSISTENT       0x0040     /* Mark entry persistent               */
#define MULTI_MAP        0x0080		/* Entry can be mapped multiple times  */

EXPORT int CALLTYPE canOpenCreateNetwork
    (
    int			NetNo,		/* Number of network */
    char *		NetName,	/* Name for this network */
    unsigned short	Baudrate	/* Baudrate for this network */
    );

EXPORT int CALLTYPE canOpenCreateNode
    (
    int			NetNo,			/* Number of network */
    char		*NodeName,		/* Node name */
    int			ModID,			/* Module id */
    unsigned long	DevType,		/* Device type */
    int			Options,		/* Supported options */
    unsigned short	MaxErrors,		/* Error history list */
    char		*DeviceName,		/* Manufacturer device name */
    char		*HardwareVersion,	/* Manufacturer hardware rev */
    char		*SoftwareVersion,	/* Manufacturer software rev */
    unsigned short	GuardTime,		/* Guard time in ms */
    unsigned short	LifeTime,		/* Life time factor */
    unsigned short	ServerSDO,		/* No of SDO server objects */
    unsigned short	ClientSDO,		/* No of SDO client objects */
    int (* EventHandler)(int,int,int,int),	/* Nodes error handler */    
    HNODE               *HNode			/* Handle for this node */
    );

typedef struct {
	unsigned long  ulOptions;		 /* Option flags                         */
	unsigned long  ulEventMask;	     /* Event mask                           */
	unsigned short usRxPDO;			 /* No of Rx PDOs                        */
	unsigned short usTxPDO;          /* No of Tx PDOs                        */
	unsigned char  ucServerSDO;		 /* No of server SDOs                    */
	unsigned char  ucClientSDO;      /* No of client SDOs (unsupported)      */
	unsigned char  ucMaxErrors;		 /* No of errors for error history       */
	unsigned char  ucMaxIdentityObj; /* No of entries of identity obj        */
	unsigned char  ucMaxConsumerHB;  /* No of entries for consumer heartbeat */
	unsigned char  ucLifeTime;       /* Default life time factor f. guarding */
	unsigned short usGuardTime;      /* Default guard time for guarding      */
	unsigned long  ulDeviceType;	 /* CANopen device type                  */
	unsigned long  ulVendorId;		 /* Vendor id for identity obj           */
	unsigned long  ulProductCode;    /* Product code for identity object     */
	unsigned long  ulRevisionNumber; /* Revision number for identity object  */
	unsigned long  ulSerialNumber;   /* Serial number for identity object    */
	const char *   pszDeviceName;	 /* Device name                          */
	const char *   pszHwVersion;	 /* Hardware version                     */
	const char *   pszSwVersion;	 /* Software version                     */
	unsigned long  ulSyncCobID;      /* Sync COB-ID                          */
    unsigned long  ulCyclePeriod;    /* Communication cycle period           */
	unsigned long  ulSyncWindowLen;  /* Synchronous window length            */
	unsigned long  ulTimestampCobId; /* COB-ID of timestamp                  */
	unsigned long  ulEmcyCobId;      /* COB-ID of emergency object           */
	unsigned short usEmcyInhibit;    /* Inhibit time of emergency object     */
	unsigned short usProducerHbTime; /* Default producer time for heartbeat  */
	unsigned long *pulListCHBT;	     /* Ptr to list of consumer HB time      */
	unsigned char  ucErrorBehaviour; /* Error behaviour                      */
	unsigned char  ucMaxMapped;      /* Maximum # an object can be mapped    */
	unsigned short usPdoRxQueuesize; /* PDO queue size of Rx daemon          */
	unsigned long  ulReserved;		 /* Reserved to future use               */
} SLAVE_NODE_INFO;

    /* Macro to create a consumer heartbeat time entry */
#define CHBT_ENTRY(NodeId, HeartbeatTime) (((NodeId & 0x7F) << 16) | \
                                           (HeartbeatTime & 0xFFFF))

    /* Mark end of consumer heartbeat time list array */
#define END_OF_CHBT_LIST 0xFFFFFFFF 

EXPORT int CALLTYPE canOpenCreateNodeEx
    (
    int                  iNetNo,			    /* Number of logical network  */
    int			         iModID,			    /* Assigned module id         */
    PFN_COS_NODE_HANDLER EventHandler,          /* Node event handler         */
    SLAVE_NODE_INFO      *pSlaveInfo,           /* Ptr to node info structuee */
    HNODE                *HNode			        /* Handle for this node       */
    );

EXPORT int CALLTYPE canOpenDeleteNode
    (
    HNODE HNode			/* Handle of this CANopen node */
    );

EXPORT int CALLTYPE canOpenActivateNode
    (
    HNODE HNode			/* Handle of this CANopen node */
    );

EXPORT int CALLTYPE canOpenResetNode
    (
    HNODE          HNode		/* Handle for this node */
    );

EXPORT int CALLTYPE canOpenRemoveNetwork
    (
    int			NetNo		/* Number of network */
    );

EXPORT int CALLTYPE canOpenExtendDictionary
    (
    HNODE		HNode,		/* Handle for the CANopen node */
    unsigned short	Index,		/* Index of the new entry */
    unsigned short	Subentries,	/* Number of subentries */
    unsigned short	ObjectType,	/* Object type of this entry */
    const char *	DataType	/* Description of data type */
    );

EXPORT int CALLTYPE canOpenInitDictionary
    (
    HNODE                HNode,		/* Handle for the CANopen node	    */
    unsigned short	     Index,		/* Index of this entry		    */
    unsigned short	     Subindex,	/* Subindex of the entry	    */
    const char *	     EntryName,	/* Textual description f. this entry */
    unsigned short       Flags,		/* Describe which entries are valid */
    pDictionaryData	     Data,		/* The new data			    */
    PFN_COS_DATA_HANDLER Handler    /* Callback function  */
    );

EXPORT int CALLTYPE canOpenReadDictionary
    (
    HNODE		HNode,		/* Handle for the CANopen node       */
    unsigned short	Index,		/* Index of this entry		     */
    unsigned short	Subindex,	/* Subindex of the entry	     */
    void *		Data		/* Ptr to data buffer (min. 4 bytes) */
    );

EXPORT int CALLTYPE canOpenWriteDictionary
    (
    HNODE		HNode,		/* Handle for the CANopen node  */
    unsigned short	Index,		/* Index of this entry		*/
    unsigned short	Subindex,	/* Subindex of the entry	*/
    void *		Data		/* The new data			*/
    );

EXPORT int CALLTYPE canOpenDefinePDO
    (
    HNODE		HNode,		/* Handle of the CANopen node   */
    const char *	Name,		/* Textual description for this PDO */
    unsigned long	COBid,		/* Default COB id for this PDO */
    unsigned short	TransMode,	/* PDO transfer mode */
    int			InhibitTime,	/* Inhibit time */
    unsigned short	TxTimeOut,	/* Tx TimeOut */
    unsigned short	RxTimeOut,	/* Rx TimeOut */
    int			Prio,		/* CMS priority group */
    unsigned short *	Mapping,	/* Description of object mapping */
    HPDO *		hpdo		/* Handle for this PDO */
    );

EXPORT int CALLTYPE canOpenGetNodeInfo
    (
    HNODE   HNode,			/* Handle of the CANopen node */
    int *   NodeNumber,			/* ID of this node */
    int *   Status,			/* Nodestate */
    int *   LastErr			/* Last error of slave daemon */
    );

EXPORT int CALLTYPE canOpenWritePDO
    (
    HPDO   hpdo,			/* PDO handle */
    void * buffer			/* Ptr to new data */
    );

EXPORT int CALLTYPE canOpenRequestPDO
    (
    HPDO   hpdo,			/* PDO handle */
    void * buffer			/* Buffer of application */
    );

EXPORT int CALLTYPE canOpenReadPDO
    (
    HPDO   hpdo,			/* PDO handle */
    void * buffer			/* Buffer of application */
    );

EXPORT int CALLTYPE canOpenSetError
    (
    HNODE  HNode,                       /* Handle for this node              */
    unsigned short  ErrorCode,          /* CANopen error code                */
    unsigned short  ErrorInformation,   /* Addtional information for error   */
    unsigned short  ErrorRegister,      /* Error register flag to set        */
    unsigned char * ErrorField          /* Manufacturer specific error field */
    );

EXPORT int CALLTYPE canOpenResetError
    (
    HNODE  HNode,                   /* Handle for this node                  */
    unsigned short  ErrorRegister,  /* Error register flag to reset          */
    unsigned char * ErrorField      /* Manufacturer specific error field     */
    );

EXPORT int CALLTYPE canOpenCreateServerSDO
    (
    HNODE          HNode,	/* Handle of the CANopen node   */
    unsigned long  tx_id,	/* Id of COB server->client     */
    unsigned long  rx_id,	/* Id of COB client->server     */
    unsigned short tx_tout,	/* Timeout for tx operation     */
    unsigned short rx_tout,     /* Timeout for rx operation     */
    unsigned short tx_prio,	/* Tx priority class for DBT action */    
    unsigned short rx_prio,	/* Rx priority class for DBT action */
    unsigned short inhibit	/* Inhibit time for tx object */
    );

EXPORT int CALLTYPE canOpenWaitForNodeState
    (
    HNODE          HNode,	/* Handle for this node */
    unsigned short StateMask	/* Mask of node states to wait for */
    );

EXPORT int CALLTYPE canOpenGetDictionaryHnd
    (
    HNODE          HNode,	/* Handle of the CANopen node   */
    unsigned short Index,	/* Index of dictionary entry    */
    unsigned short Subindex,    /* Subindex of dictionary entry */
    HDICT          *HDict       /* Handle or NULL if faiure     */
    );

EXPORT int CALLTYPE canOpenWriteDictionaryHnd
    (
    HDICT	HDict,		/* Dictionary entry handle      */
    void *	Data		/* The new data			*/
    );

EXPORT int CALLTYPE canOpenReadDictionaryHnd
    (
    HDICT               HDict,     /* Handle for dictionary entry  */
    void *		Data	   /* The new data		   */
    );

EXPORT int CALLTYPE canOpenRequestSDO
    (
    HNODE          hNode,	/* Handle of the CANopen node   */
    unsigned short Flags,	/* Flags for this operation     */
    unsigned short ModId,	/* Module id of remote node     */
    unsigned short TxTimeout,	/* Timeout for SDO tx operation */
    unsigned short RxTimeout,   /* Timeout for SDO rx operation */
    HSDO           *hSDO        /* Handle or NULL if failed     */
    );

EXPORT int CALLTYPE canOpenReleaseSDO
    (
    HNODE          hNode,	/* Handle of the CANopen node   */
    HSDO           hSDO         /* Handle of SDO connection     */
    );

EXPORT int CALLTYPE canOpenDownload
    (
    HNODE		hNode,	    /* Handle of the CANopen node         */
    HSDO		hSDO,       /* Handle of SDO connection           */
    unsigned short	index,	    /* Index to download data from        */
    unsigned short	subindex,   /* Subindex to download data from     */
    const void *	pData,	    /* Buffer to download data            */
    const long 		len	    /* Length of buffer to download       */
    );

EXPORT int CALLTYPE canOpenUpload
    (
    HNODE		hNode,	    /* Handle of the CANopen node         */
    HSDO		hSDO,       /* Handle of SDO connection           */
    unsigned short	index,	    /* Index to download data from        */
    unsigned short	subindex,   /* Subindex to download data from     */
    void *		pData,	    /* Buffer to store uploaded data      */
    long *		len	    /* Length of buffer / upload data     */
    );


EXPORT void CALLTYPE canOpenGetVersions
    (
    CANOPEN_VERSIONS *versions	   /* Ptr to struct of type CANOPEN_VERIONS */
    );  

EXPORT int CALLTYPE canOpenGetConfiguration
    (
    HNODE			hNode,			   /* Handle of the CANopen node          */
	unsigned long   ulFlags, 		   /* Flags: Not used at the moment       */
    unsigned long   *pulSize,		   /* Size of this data block             */
    void			**ppConfiguration  /* Ptr to persistent data to be stored */
	);

EXPORT int CALLTYPE canOpenSetConfiguration
    (
    HNODE			hNode,			   /* Handle of the CANopen node          */
	unsigned long   ulFlags,		   /* Various flags                       */
    const void		*pConfiguration    /* Ptr to saved configuration          */
    );

EXPORT int CALLTYPE canOpenSetParameter
    (
    HNODE          HNode,				/* Handle for this node		*/
	unsigned int   uiCommand,			/* Command				    */
	void *         pArg					/* Argument                 */
    );
	
#define SCANOPEN_PARA_ERROR             -2  
#define SCANOPEN_SYS_ERROR              -1  
#define SCANOPEN_OK			0  
#define SCANOPEN_CAN_RX_TIMEOUT         1
#define SCANOPEN_CAN_TX_TIMEOUT         2
#define SCANOPEN_CAN_TX_ERROR           4
#define SCANOPEN_CAN_CONTR_OFF_BUS      5
#define SCANOPEN_CAN_CONTR_BUSY         6
#define SCANOPEN_CAN_CONTR_WARN         7
#define SCANOPEN_PROTOCOL_ERROR         129
#define SCANOPEN_NO_SLAVE_NET           132
#define SCANOPEN_NETWORK_FILE_ERROR     133
#define SCANOPEN_WRONG_NODE_STATE       134
#define SCANOPEN_NODE_NOT_OPERATIONAL   136
#define SCANOPEN_OBJ_NOT_EXISTS         137
#define SCANOPEN_UNKNOWN_ERROR          144
#define SCANOPEN_CHECKSUM_ERROR         145
#define SCANOPEN_VERIFY_ERROR           146
#define SCANOPEN_COB_DATABASE_OVERFLOW  147
#define SCANOPEN_COB_DATABASE_DISABLED  148
#define SCANOPEN_COB_CLASS_ERROR        149
#define SCANOPEN_COB_LENGTH_ERROR       150
#define SCANOPEN_OBJ_EXISTS             154 
#define SCANOPEN_DATATYPE_TOO_LONG      155
#define SCANOPEN_OBJ_NO_MEMORY          157
#define SCANOPEN_CANNOT_OPEN_CAN        158
#define SCANOPEN_DOWNLOAD_ABORT_SLAVE   171
#define SCANOPEN_DOWNLOAD_ABORT_MASTER  172
#define SCANOPEN_OBJ_NOT_PREPARED       173
#define SCANOPEN_OBJ_WRONG_ACCESS       174
#define SCANOPEN_WRONG_MASTER_REQUEST   188
#define SCANOPEN_WRONG_MASTER_ANSWER    189
#define SCANOPEN_WRONG_CLIENT_REQUEST   190
#define SCANOPEN_WRONG_CLIENT_ANSWER    191
#define SCANOPEN_WRONG_SERVER_REQUEST   192
#define SCANOPEN_WRONG_SERVER_ANSWER    193
#define SCANOPEN_TOGGLE_ERROR           194
#define SCANOPEN_BUFFER_OVERFLOW        195
#define SCANOPEN_ABORT_DOMAIN           196
#define SCANOPEN_USER_ERROR             200
#define SCANOPEN_WRONG_INDEX            256  /* Wrong dictionary index        */
#define SCANOPEN_WRONG_SUBINDEX         257  /* Wrong dictionary subindex     */
#define SCANOPEN_OUT_OF_MEMORY          258  /* Out of memory                 */
#define SCANOPEN_WRONG_BAUDRATE         259  /* Unsupported baudrate          */
#define SCANOPEN_CANNOT_START_DAEMON    260  /* Error starting daemon         */
#define SCANOPEN_WRONG_PARAMETER        261  /* Wrong parameter               */
#define SCANOPEN_VALUE_TOO_HIGH         262  /* Value too high                */
#define SCANOPEN_VALUE_TOO_LOW          263  /* Value too low                 */
#define SCANOPEN_WRONG_TYPE             264  /* Wrong data type               */
#define SCANOPEN_WRONG_OBJECT_TYPE      265  /* Wrong object type             */
#define SCANOPEN_PDO_MAPPING_ERROR      266  /* Error in mapping objects      */
#define SCANOPEN_TOO_MANY_OBJECTS       267  /* Too many objects in use       */
#define SCANOPEN_WRONG_NODESTATE        268  /* Slave in wrong node state     */
#define SCANOPEN_SERVICE_NOT_ALLOWED    269  /* Service not allowed           */
#define SCANOPEN_LENGTH_MISMATCH        270  /* Length mismatch in data       */
#define SCANOPEN_PDO_NOT_VALID          271  /* PDO not valid                 */
#define SCANOPEN_INIT_ERROR             272  /* Error in initialization       */
#define SCANOPEN_INVALID_HANDLE         273  /* No valid handle               */
#define SCANOPEN_ACCESS_ERROR           274  /* Access not allowed            */
#define SCANOPEN_PDO_PARAMETER_ERROR    275  /* Wrong communication parameter */
#define SCANOPEN_NOT_CONNECTED          276  /* No connection to client SDO   */
#define SCANOPEN_NOT_IMPLEMENTED        277  /* Functionality not implemented */
#define SCANOPEN_HARDWARE_ERROR         278  /* Application signals HW error  */
#define SCANOPEN_DATA_STORE_ERROR       279  /* Data store error              */
#define SCANOPEN_INCOMPATIBILITY_ERROR  280  /* Incompatible parameter        */
#define SCANOPEN_INHIBITED              281  /* PDO transmission inhibited    */

	/* Parameter for canOpenCreateNode() to describe device capabilities */

#define EXTENDED_BOOTUP		0x0001	/* Extended CAL bootup            */
#define STATUS_REGISTER		0x0002	/* Status register                */
#define ERROR_REGISTER		0x0004  /* Additional error register      */
#define ADDITIONAL_PDOS		0x0008	/* Additional PDOs                */
#define SYNCHRON_PDOS		0x0010  /* Synchron PDO support           */
#define MANUFACTURER_INFO	0x0020  /* Manufacturer info entries      */
#define GUARDING			0x0040  /* Support node/life guarding     */
#define ADDITIONAL_SDOS		0x0080	/* Additional SDO support         */
#define PARAMETER_STORE		0x0100	/* Support parameter saving       */
#define PARAMETER_RESET		0x0200	/* Support parameter reset        */
#define PRODUCER_HEARTBEAT	0x0400	/* Support producer heartbeat     */
#define CONSUMER_HEARTBEAT	0x0800	/* Support consumer heartbeat     */
#define AUTO_START_DEVICE   0x1000  /* Support autostart              */
#define EMCY_CONSUMER       0x4000  /* Emergency consumer support     */
#define ERROR_BEHAVIOUR_OBJ 0x8000  /* Error behaviour object         */

	/* Parameter for canOpenDefinePDO to describe PDO type*/

#define TRANSMIT_PDO		 0x8000  /* Mark PDO as transmit PDO */
#define RECEIVE_PDO		     0x0000  /* Mark PDO as receive PDO */
#define SYNCHRON_PDO		 0x4000  /* Mark PDO as synchron PDO */
#define ASYNCHRON_PDO		 0x0000  /* Mark PDO as asynchron PDO */
#define AUTO_NOTIFY_PDO		 0x2000  /* Mark PDO as auto tx/rx PDO */
#define TX_DONE_PDO		     0x1000  /* Wait until PDO is transmitted */
#define TX_ON_STARTUP_PDO    0x0800  /* Send PDO on startup */
#define REQ_ON_STARTUP_PDO   0x0800  /* Request PDO on startup */
#define TX_ON_DATACHG_PDO    0x0400  /* Send PDO only if data has changed */
#define TIMER_DRIVEN_PDO     0x0200  /* Support event timer */

	/* Dummy mapping object entries */

#define DUMMYMAP_BOOL		1
#define DUMMYMAP_UINT8		2
#define DUMMYMAP_UINT16		3
#define DUMMYMAP_UINT32		4
#define DUMMYMAP_INT8		5
#define DUMMYMAP_INT16		6
#define DUMMYMAP_INT32		7

	/* Error codes for EMCY objects */

#define EMCY_ERROR_RESET                0x0000
#define EMCY_GENERIC_ERROR              0x1000
#define EMCY_CURRENT                    0x2000
#define EMCY_CURRENT_INPUT              0x2100
#define EMCY_CURRENT_INSIDE             0x2200
#define EMCY_CURRENT_OUTPUT             0x2300
#define EMCY_VOLTAGE                    0x3000
#define EMCY_VOLTAGE_INPUT              0x3100
#define EMCY_VOLTAGE_INSIDE             0x3200
#define EMCY_VOLTAGE_OUTPUT             0x3300
#define EMCY_TEMPERATURE                0x4000
#define EMCY_TEMPERATURE_AMBIENT        0x4100
#define EMCY_TEMPERATURE_DEVICE         0x4200
#define EMCY_DEVICE_HARDWARE            0x5000
#define EMCY_DEVICE_SOFTWARE            0x6000
#define EMCY_DEVICE_SOFTWARE_INTERNAL   0x6100
#define EMCY_DEVICE_SOFTWARE_USER       0x6200
#define EMCY_DEVICE_SOFTWARE_DATA_SET   0x6300
#define EMCY_ADDITIONAL_MODULES         0x7000
#define EMCY_MONITORING                 0x8000
#define EMCY_MONITORING_COMMUNICATION   0x8100
#define EMCY_CAN_OVERRUN                0x8110
#define EMCY_CAN_ERROR_PASSIVE          0x8120
#define EMCY_LIFE_GUARD_ERROR           0x8130
#define EMCY_CAN_RECOVER_FROM_BUS_OFF   0x8140
#define EMCY_CAN_TX_COB_ID_COLLISION    0x8150
#define EMCY_PROTOCOL_ERROR             0x8200
#define EMCY_PDO_NOT_PROCESSED          0x8210
#define EMCY_PDO_LENGTH_EXCEEDED        0x8220
#define EMCY_EXTERNAL_ERROR             0x9000
#define EMCY_ADDITIONAL_FUNCTIONS       0xF000
#define EMCY_DEVICE_SPECIFIC            0xFF00

	/* Error flags in standard error register */

#define ERROR_GENERIC                   0x0001   /* Generic error           */
#define ERROR_CURRENT                   0x0002   /* Error current           */
#define ERROR_VOLTAGE                   0x0004   /* Error voltage           */
#define ERROR_TEMPERATURE               0x0008   /* Error temperature       */
#define ERROR_COMMUNICATION             0x0010   /* Error communication     */
#define ERROR_DEVICE_SPECIFIC           0x0020   /* Device profile specific */
#define ERROR_MANUFACTURER_SPECIFIC     0x0080   /* Manufacturer specific   */

#define ERROR_SEVERE					0x8000   /* Mark severe error       */
#define ERROR_STACK_SCHEME              0x4000   /* Error in stack scheme   */

  /* Flags to signal temporary errors */
#define STATUS_ERROR_GUARDING           0x01   /* Guarding timed out      */
#define STATUS_ERROR_CAN_WARN           0x02   /* CAN warn limit exceeded */
#define STATUS_ERROR_CAN_BUS_OFF        0x04   /* CAN bus off             */

  /* Flags to signal sticky errors */
#define STATUS_ERROR_CAN                0x01   /* CAN error               */
#define STATUS_ERROR_RX_FIFO_OVERRUN    0x02   /* RX FIFO overrun         */
#define STATUS_ERROR_PDO_LENGTH         0x04   /* Wrong PDO length recv.  */

	/* Flags for mask of canOpenWaitForNodeState */

#define WFNS_INIT			0x0001
#define WFNS_DISCONNECTED		0x0002    
#define WFNS_CONNECTING			0x0004
#define WFNS_PREPARING			0x0008
#define WFNS_PREPARED			0x0010
#define WFNS_STOPPED            WFNS_PREPARED           
#define WFNS_OPERATIONAL		0x0020
#define WFNS_PRE_OPERATIONAL		0x0040

  /* Macros to extract arguments of event handler */

#define CANOPEN_NET(arg)  (arg >> 8 & (int)0xFF)
#define CANOPEN_NODE(arg) (arg & (int)0xFF);

  /* Macro to initialize device type parameter */

#define DEV_TYPE(DeviceType, AdditionalInformation) \
	             (unsigned long)(((unsigned long)DeviceType & 0xFFFF) | \
                                 ((unsigned long)AdditionalInformation << 16))

  /* Known device types */

#define DEVTYPE_NONE	     0	/* No standardized device profile */
#define DEVTYPE_DS401	   401  /* I/O modules */
#define DEVTYPE_DS402	   402
#define DEVTYPE_DS403	   403
#define DEVTYPE_DS404	   404
#define DEVTYPE_DS405	   405
#define DEVTYPE_DS406	   406
#define DEVTYPE_DS407	   407
#define DEVTYPE_DS408	   408
#define DEVTYPE_DS409	   409

  /* Pre-defined additional information data by CiA DS-301 */

#define MULTI_DEVICE_MODULE 0xFFFF

  /* CiA vendor id of esd gmbh */

#define VENDOR_ID_ESD		0x17

  /* Handled events (Para 2 of event handler) */

#define EV_GUARDING				0x0001
#define EV_STATE_CHANGE			0x0002
#define EV_RESET				0x0004
#define EV_CONFIGURATION		0x0008
#define EV_CAN					0x0010
#define EV_EMCY					0x0020
#define EV_CONSUMER_HEARTBEAT   0x0040
#define EV_WRITE_DICTIONARY     0x0080
#define EV_READ_DICTIONARY      0x0100

  /* Reason for guarding/consumer heartbeat events (Para 3 of event handler) */

#define EV_START	   0x0001
#define EV_TIMEOUT	   0x0002
#define EV_STOP		   0x0003

  /* Reason for reset events */

#define EV_COMMUNICATION   0x0001
#define EV_APPLICATION	   0x0002

  /* Subreason for reset events */

#define EV_BEGIN           0x0001
#define EV_END             0x0002

  /* Reason for configuration events */

#define EV_STORE	   0x0001
#define EV_RESTORE	   0x0002

  /* Reason for communication status events (Para 3 of event handler) */

#define EV_CONTROLLER_OK          0x0000
#define EV_CONTROLLER_WARN        0x0001
#define EV_CONTROLLER_BUS_OFF     0x0002
#define EV_CONTROLLER_OVERRUN     0x0003
#define EV_FIFO_OVERRUN           0x0004
#define EV_PDO_FIFO_OVERRUN       0x0005
#define EV_PDO_RX_ERROR           0x0006
#define EV_PDO_NOT_PROCESSED      0x0007
#define EV_PDO_LENGTH_EXCEEDED    0x0008


  /* Node States */
#define NodeNone              0x0000
#define NodeOffline           0x0000
#define NodeDisconnected      0x0001
#define NodeConnecting        0x0002
#define NodeConnected         0x0002
#define NodePreparing         0x0003
#define NodePrepared          0x0004
#define NodeOperational       0x0005
#ifdef CANOPEN
# define NodeInit             0x0000    
# define NodePreOperational   0x007F
#endif

    /* Possible values for error behaviour object (0x1029) */

#define ERROR_BEHAVIOUR_DEFAULT         0
#define ERROR_BEHAVIOUR_NO_CHANGE       1
#define ERROR_BEHAVIOUR_STOP            2

    /* Valid parameter for canOpenSetParameter() */

#define PARA_DISABLE_AUTO_TRANSMISSION      0x0001
#define PARA_ENABLE_AUTO_TRANSMISSION       0x0002


  /* Macros to ease definition of object dictionary entries */

typedef struct {
  unsigned short  Index;				  
  unsigned short  Subentries;
  unsigned short  ObjectType;
  const char *    DataType;
  unsigned short  Flags;
  pDictionaryData pData;
  int (* Handler)(int,int,int,int,void*);
  const char *    EntryName;
} _COS_DICT_ENTRY;
  
#define BEGIN_DICTIONARY_TABLE(x) static _COS_DICT_ENTRY _Dict_Entry_##x[] = {
#define END_DICTIONARY_TABLE() {0, 0, 0, NULL, 0, NULL, NULL, NULL}};
#define DICTIONARY_ENTRY(a, b, c, d, e, f, g, h) {a, b, c, d, e, f, g, h},
  
#define DECLARE_DICTIONARY(Hnode, x) { \
	unsigned short _i; int _state; \
	_COS_DICT_ENTRY *_p = &_Dict_Entry_##x[0]; \
	while(_p->Index) { \
	  _state = canOpenExtendDictionary(Hnode, _p->Index, _p->Subentries, \
                                           _p->ObjectType, _p->DataType); \
	  if(_state != SCANOPEN_OK) \
	     printf("Error %d creating dictionary entry %x\n", _state, _p->Index); \
	  else if(_p->pData != NULL) { \
	    if(_p->ObjectType == OBJ_VAR) { \
           _state = canOpenInitDictionary(Hnode, _p->Index, _p->Subentries, \
                                          NULL, _p->Flags, _p->pData,\
										  _p->Handler); \
		   if(_state != SCANOPEN_OK) \
  	         printf("Error %d init. dict. entry %x\n", _state, _p->Index); \
		} else if(_p->ObjectType == OBJ_ARRAY) { \
		   for(_i = 1; _i <= _p->Subentries; _i++) { \
             _state = canOpenInitDictionary(Hnode, _p->Index, _i, \
                                            NULL, _p->Flags, _p->pData,\
			  							    _p->Handler); \
			 if(_state != SCANOPEN_OK) \
				printf("Error %d init. dict. entry %x sub %d\n", _state, _p->Index, _i); \
		   } /* of for */ \
		} else if(_p->ObjectType == OBJ_RECORD) { \
           DictionaryData _udtEntries; \
           _udtEntries.uint8.val = _udtEntries.uint8.defval = _p->Subentries - 1; \
		   _state = canOpenInitDictionary(Hnode, _p->Index, 0, NULL, \
			                             READ_ACCESS, &_udtEntries, \
                                         (PFN_COS_DATA_HANDLER)NULL); \
		   if(_state != SCANOPEN_OK) \
				   printf("Error %d init. dict. entry %x sub 0\n", _state, _p->Index); \
		   for(_i = 1; _i < _p->Subentries; _i++) { \
             _state = canOpenInitDictionary(Hnode, _p->Index, _i, \
                                            NULL, _p->Flags, _p->pData,\
			  							    _p->Handler); \
			 if(_state != SCANOPEN_OK) \
				printf("Error %d init. dict. entry %x sub %d\n", _state, _p->Index, _i); \
		   } /* of for */ \
        } else { \
		  printf("Unsupported object type %x\n", _p->ObjectType); \
		} /* of if (_p->ObjectType == OBJ_VAR) */ \
     } /* of if(_state != SCANOPEN_OK) */ \
	 _p++; \
   } \
}

  /* Macros to ease definition of PDOs */

#define BEGIN_PDO_TABLE(x) static _COS_PDO_ENTRY _PDO_Table_##x[] = {
#define END_PDO_TABLE() {0, 0, 0, 0, 0, 0, NULL, 0}};
#define PDO_ENTRY(a, b, c, d, e, f, g) {a, b, c, d, e, f, &_Mapping_##g[0], 0},
#define PDO_ENTRY_UNMAPPED(a, b, c, d, e, f) {a, b, c, d, e, f, NULL, 0},

  /* Macros to ease definition of mapping entries */

/* Macros to define a mapping entry */
#define BEGIN_MAPPING_TABLE(x) static unsigned short _Mapping_##x[] = {
#define END_MAPPING_TABLE() 0,0};
#define MAPPING_ENTRY(index, subindex) index, subindex,

typedef struct {
  unsigned long    ulCOBid;			/* COB-Id				   */
  unsigned short   usTransMode;		/* Transmission mode       */		  
  int              iInhibitTime;	/* Inhibit time            */
  unsigned short   usTxTimeout;		/* Rx timeout              */
  unsigned short   usRxTimeout;		/* Tx timeout              */
  unsigned short   usEventTimer;	/* Event timer timeout     */
  unsigned short   *psMapping;		/* Default mapping         */
  void			   *pReserved;		/* Reserved for future use */
  HPDO             handle;
} _COS_PDO_ENTRY;

#define DECLARE_PDO(Hnode, x) { \
	int _state; \
	_COS_PDO_ENTRY *_p = &_PDO_Table_##x[0]; \
	while(_p->ulCOBid) { \
 	   _state = canOpenDefinePDO(Hnode, NULL, _p->ulCOBid, \
                                 _p->usTransMode, _p->iInhibitTime, \
				                 _p->usTxTimeout, _p->usRxTimeout, \
				                 _p->usEventTimer, _p->psMapping, &_p->handle); \
	   if(_state != SCANOPEN_OK) \
		printf("Error %d\n", _state); \
		_p++; \
	} \
}

#ifdef __cplusplus
}
#endif

#endif /* of __INCscanOpenh */




