/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: sdl.h,v 1.12 2000/01/18 10:08:13 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* P.Duhoux  11/01/95  Created  for Release 1.0
* P.Duhoux  16/06/95  Added functions sdlVersion, sdlOpen, sdlClose
* P.Duhoux  13/09/95  Modified interface for independence of MCM
* P.Duhoux  29/11/95  Added function sdlDeinstall
* P.Duhoux  18/03/96  Modified Interrupt data structure
*/

#ifndef SDL_H
#define SDL_H

/******************************************************************************
* sdl.h - single device library common interface file
*------------------------------------------------------------------------------
*/
#ifdef __cplusplus
extern "C" {
#endif

/*
 * LCC include files
 */
#include "CCS.h"
#include "cai.h"

/*
 * SDL include files
 */
#include "sdlDefines.h"
#include "sdlErrors.h"

/* SDL common module name */
#define sdlMODULE         "sdl"
#define sdlMODULE_NAME    "SDL Common Module"

/* SDL common module version */
#define sdlVERSION        "$Revision: 1.12 $"
#define sdlVERSION_DATE   "MAR98"

/*
 * Macro definitions
 */
#define IN
#define OUT 
#define INOUT

/*
 * Type definitions
 */

/* data type for DB entries */
typedef vltINT32 sdlDB_INDEX;

/* data structure for parameter access */
typedef struct
    {
    char      *cmdName;
    vltINT32   readCmd;             /* Driver Read Command code  (-1 if no) */ 
    vltINT32   writeCmd;            /* Driver Write Command code (-1 if no)  */
    } sdlCMD_CODE;

/* data structure for device description */
typedef struct
    {
    vltBYTES8 driver;
    vltINT32  number;
    } sdlDEVICE;

/* data structure for connection of interrupt service routines */
typedef struct
    {
    vltBYTES20  name;            /* interrupt name                        */
    vltBYTES32  function;        /* ISR name                              */
    VOIDFUNCPTR routine;         /* ISR entry point                       */
    vltINT32    argument;        /* ISR argument to pass                  */
    vltINT32    vector;          /* interrupt vector                      */
    } sdlCONFIG_INT;

/* data structure for device description */
typedef	struct
    {
    vltBYTES20   device;         /* device name       (/xxxn)            */
    int          fd;             /* device descriptor (ERROR if closed)  */
    char        *db;             /* database address table               */
    } sdlDESCRIPTION;

/*
 * function prototypes
 */
ccsCOMPL_STAT sdlInstall     (sdlDESCRIPTION *sdlDesc, vltBYTES20 name,
			      vltBYTES8 prefix, ccsERROR *error);
ccsCOMPL_STAT sdlDeinstall   (sdlDESCRIPTION *sdlDesc, ccsERROR *error);

ccsCOMPL_STAT sdlOpen        (sdlDESCRIPTION *sdlDesc, ccsERROR *error);
ccsCOMPL_STAT sdlClose       (sdlDESCRIPTION *sdlDesc, ccsERROR *error);

ccsCOMPL_STAT sdlCommand     (sdlDESCRIPTION *sdlDesc, vltINT32 cmdCode,  
			      vltINT32 *pValue, ccsERROR *error);       

ccsCOMPL_STAT sdlLoad        (sdlDESCRIPTION *sdlDesc, vltINT32 parIndex,
			      vltINT32 *pValue, vltINT32 drvCode,
			      ccsERROR *error);       
ccsCOMPL_STAT sdlRetrieve    (sdlDESCRIPTION *sdlDesc, vltINT32 parIndex,
			      vltINT32 *pValue, vltINT32 drvCode, 
			      ccsERROR *error);

void          sdlVersion     ( vltBYTES80 version );

#ifdef __cplusplus
}
#endif

#endif /* !SDL_H */

/*************************************************************************/

/*___oOo___*/
