/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tacShMsgQ.h,v 1.55 2005/10/03 17:38:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bbauvir   26/12/02  Build inter CPU communications on top of new MsgQ design
* bbauvir   12/06/02  Correct dynamic MsgQ init for 1 CPU configuration
* bbauvir   21/03/01  Created
*/

#ifndef TAC_SHMSGQ_H
#define TAC_SHMSGQ_H

/**************************************************************************
 *  tacShMsgQ.h - This file provides the interface to the Inter CPU (process)
 *              Messaging system.
 *
 *------------------------------------------------------------------------
 */

/* 
 * System Headers
 */

/* 
 * Local Headers
 */

#include "tacTypes.h"
#include "tacDefines.h"

/*
 * Constants
 */

typedef enum {

  /*
   * Standard commands
   */

  tacSHMSG_STD_INIT = 0,
  tacSHMSG_STD_STANDBY,
  tacSHMSG_STD_ONLINE,
  tacSHMSG_STD_STOP,
  tacSHMSG_STD_OFF,
  tacSHMSG_STD_EXIT,
  tacSHMSG_STD_STATE,
  tacSHMSG_STD_STATUS,
  tacSHMSG_STD_SIMULAT,
  tacSHMSG_STD_STOPSIM,
  tacSHMSG_STD_VERBOSE,
  tacSHMSG_STD_VERSION,
  tacSHMSG_STD_SELFTST,
  tacSHMSG_STD_TEST,

  /*
   * Messages Server -> RTController
   */

  tacSHMSG_CONFIG_RTC,
  tacSHMSG_ADD_BLOCK,
  tacSHMSG_MODIFY_BLOCK,
  tacSHMSG_REPORT_BLOCK,    /* VLTSW20040417 */
  tacSHMSG_DELETE_BLOCK,
  tacSHMSG_ADD_LINK,
  tacSHMSG_DELETE_LINK,
  tacSHMSG_RESET_ALGORITHM,
  tacSHMSG_CLEAR_ALGORITHM,
  tacSHMSG_REPORT_CONFIG,

  /*
   * Messages Server <- RTController
   */

  tacSHMSG_REPORT_SYNCHRO,
  /* tacSHMSG_REPORT_BLOCK, */    /* VLTSW20040417 */
  tacSHMSG_REPORT_LINK,
  tacSHMSG_REPORT_ERROR,
  tacSHMSG_REPORT_UTC,     /* VLTSW20040278 */
  tacSHMSG_HEARTBEAT,
  tacSHMSG_MONITOR_DATA,
  tacSHMSG_TRANSFER_DATA,
  tacSHMSG_ADD_PROBE,
  tacSHMSG_REMOVE_PROBE,
  tacSHMSG_ADD_MONITOR,    /* VLTSW20030194 */
  tacSHMSG_REMOVE_MONITOR,
}tacSHMSG_TYPE;

typedef enum {
  tacSHMSGQ_REQUEST = 0,
  tacSHMSGQ_REPLY,
  tacSHMSGQ_BG_INFO,
  tacSHMSGQ_DATA_TRANSFER,
  tacSHMSGQ_MAX_QUEUE_NUMBER,
}tacSHMSGQ_TYPE;

/* 
 * Types
 */

typedef struct {
  char name[tacMAX_NAME_SIZE];
  tacSTATE start;
}tacSHMSG_CONFIG;

typedef struct {
  tacSTATE state;
}tacSHMSG_STATE;

typedef tacPARAM tacSHMSG_PARAM;

typedef struct {
  char type[tacMAX_NAME_SIZE];
  char name[tacMAX_NAME_SIZE];
  tacSHMSG_PARAM parameter;
}tacSHMSG_BLOCK;

typedef struct {
  char name[tacMAX_NAME_SIZE];
  unsigned long port;
}tacSHMSG_LINK_BLOCK_INFO;

typedef struct {
  char name[tacMAX_NAME_SIZE];
  tacSHMSG_LINK_BLOCK_INFO srcBlock;
  tacSHMSG_LINK_BLOCK_INFO dstBlock;
}tacSHMSG_LINK;

typedef struct {
  tacERROR code;
}tacSHMSG_ERROR;

typedef struct {
  tacSYNCHRO_TYPE type;
  double samplingPeriod;
}tacSHMSG_SYNCHRO;

typedef struct {
  char name[tacMAX_NAME_SIZE];    /* Once at block creation */
  tacDATA data;
}tacSHMSG_DATA;

typedef struct {
  tacTIME time;
}tacSHMSG_TIME;

typedef struct {
  tacSHMSG_TYPE type;
  int reply;    /* VLTSW20010700 - Post message and return directly */
  union {
    tacSHMSG_CONFIG config;
    tacSHMSG_STATE state;
    tacSHMSG_BLOCK block;
    tacSHMSG_LINK link;
    tacSHMSG_ERROR error;
    tacSHMSG_SYNCHRO synchro;
    tacSHMSG_DATA monitor;
    tacSHMSG_TIME utc;    /* VLTSW20040278 */
  } msgBody;
}tacSHMSG;

/*
 * Global variables
 */

/*
 * Functions
 */

/*
 * Master -> Slave CPU (process)
 */

STATUS tacShMsgQRetrieveRequest(tacSHMSG* pMsg);
STATUS tacShMsgQSendRequest(tacSHMSG* pMsg);

/*
 * Slave -> Master CPU (process)
 */

STATUS tacShMsgQRetrieveReply(tacSHMSG* pMsg);
STATUS tacShMsgQSendReply(tacSHMSG* pMsg);

/*
 * Slave -> Master CPU (process) ... Bg activity
 */

STATUS tacShMsgQRetrieveBgInfo(tacSHMSG* pMsg);
STATUS tacShMsgQSendBgInfo(tacSHMSG* pMsg);

/*
 * Slave -> Master CPU (process) ... Fast data transfer
 */

STATUS tacShMsgQRetrieveDataSample(tacSHMSG* pMsg);
STATUS tacShMsgQSendDataSample(tacSHMSG* pMsg);

#endif /*!TAC_SHMSGQ_H*/









