/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tacStdBlock.h,v 1.55 2005/10/03 17:38:06 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bbauvir   21/03/01  created
*/

#ifndef TAC_STDBLOCK_H
#define TAC_STDBLOCK_H

/************************************************************************
 *  tacStdBlock.h - This file provides the interface to the Block
 *                  abstract class. Any specification of this class or
 *                  module manipulating Blocks must include this header 
 *                  file.
 *
 *----------------------------------------------------------------------
 */

/* 
 * System Headers
 */

/* 
 * Local Headers
 */

#include "tacTypes.h"
#include "tacDefines.h"
#include "tacErrors.h"

/*
 * Constants
 */

/* 
 * Types
 */

typedef struct {
  char typeName[tacMAX_NAME_SIZE];
  tacFCTPTR typeConstructorHook;
  tacFCTPTR typeGetParameterHook;
  tacVOIDFCTPTR typeAlgorithmHook;
  tacFCTPTR typeSetParameterHook;
  tacVOIDFCTPTR typeResetHook;
  tacVOIDFCTPTR typeStopHook;
  tacVOIDFCTPTR typeDestructorHook;
  tacVOIDFCTPTR typeShowHook;
}tacSTDBLOCK_TYPE;

typedef tacPARAM tacSTDBLOCK_PARAM;

typedef struct {
  unsigned short type;
  char name[tacMAX_NAME_SIZE];
  unsigned short resolved;
  unsigned short bitmap;
  void (* algorithm)();
  void* parameter; 
  double** input;
  double* output;
  tacSHARED* shared;
}tacSTDBLOCK;

/*
 * Macros
 */

#define tacStdBlockGetSamplingPeriod(pSelf) ((*pSelf->shared).samplingPeriod)
#define tacStdBlockGetTime(pSelf) (&((*pSelf->shared).currentTime))
#define tacStdBlockGetSimulation(pSelf) ((*pSelf->shared).simulation)
#define tacStdBlockGetInput(pSelf,inputIndex) (*(pSelf->input[inputIndex]))
#define tacStdBlockGetOutput(pSelf,outputIndex) (pSelf->output[outputIndex])
#define tacStdBlockIsResolved(pSelf) ((pSelf->resolved)?TRUE:FALSE)

#define tacStdBlockSetOutput(pSelf,outputIndex,value) \
{                                                     \
  pSelf->output[outputIndex] = value;                 \
}
#define tacStdBlockSetAlgorithm(pSelf,pAlgo) \
{                                            \
  pSelf->algorithm = pAlgo;                  \
}

/*
 * Global variables
 */

/*
 * Functions
 */

STATUS tacStdBlockInitAll(void);
STATUS tacStdBlockInstallNewType(tacSTDBLOCK_TYPE* typeInfo);

STATUS tacStdBlockConstructor(tacSTDBLOCK** pSelf, char* type, char* name, tacSTDBLOCK_PARAM* parameter, 
                              tacSHARED* shared, tacERROR* error);
STATUS tacStdBlockReport(tacSTDBLOCK* pSelf, char* type, char* name, tacSTDBLOCK_PARAM* parameter, tacERROR* error);
STATUS tacStdBlockSetParameter(tacSTDBLOCK* pSelf, tacSTDBLOCK_PARAM* parameter, tacERROR* error);

void tacStdBlockReset(tacSTDBLOCK* pSelf);
void tacStdBlockStop(tacSTDBLOCK* pSelf);
void tacStdBlockDestructor(tacSTDBLOCK* pSelf);

double* tacStdBlockGetOutputRef(tacSTDBLOCK* pSelf, unsigned short outputPort);

void tacStdBlockSetInputNumber(tacSTDBLOCK* pSelf, unsigned short inputNumber);

STATUS tacStdBlockAttachInput(tacSTDBLOCK* pSelf, unsigned short inputPort, double* signal, tacERROR* error);
STATUS tacStdBlockDetachInput(tacSTDBLOCK* pSelf, unsigned short inputPort, tacERROR* error);

/*
 * Block construction functions
 */

STATUS tacStdBlockAllocateArrays(tacSTDBLOCK* pSelf, 
				 unsigned short paramNb, 
				 unsigned short inputNb, 
				 unsigned short outputNb,
				 tacERROR* error);

#endif /*!TAC_STDBLOCK_H*/






