/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tacTypes.h,v 1.55 2005/10/03 17:38:04 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bbauvir   20/01/05  Add macro to intialise the parameter structure
* ahuxley   02/03/04  added include of stdio for 'FILE*' missed in 1.42.1.4.
* bbauvir   25/04/03  Extend error parameter size from 40 to 128
* bbauvir   13/06/02  Add SIMULAT/STOPSIM support - flag in tacSHARED
* bbauvir   21/03/01  created
*/

#ifndef TAC_TYPES_H
#define TAC_TYPES_H

/**************************************************************************
 *  tacTypes.h - This file provides the definition of types internal to the 
 *               'tac' module
 *
 *------------------------------------------------------------------------
 */

/* 
 *  System Headers
 */

#include <vxWorks.h>
#include <semLib.h>
#include <stdio.h>			/* for FILE* */

/* 
 *  VLTSW Headers
 */

#include "ccs.h"    			/* ccsLOC_ID */

/*
 *  Application includes.
 */

#include "tacDefines.h"

/*
 * Constants
 */

typedef enum {
  tacPARAM_DOUBLE = 0,
  tacPARAM_STRING,
  tacPARAM_TIME,
}tacPARAM_TYPE;

/* VLTSW20030076 - Support PCI interrupts */
typedef enum {
  tacSYNCHRO_NO_SOURCE = 0,
  tacSYNCHRO_SEMAPHORE,
  tacSYNCHRO_INTERRUPT
}tacSYNCHRO_TYPE;

typedef enum {
  tacSTATE_UNKNOWN = 1,
  tacSTATE_INIT,
  tacSTATE_STANDBY,
  tacSTATE_ONLINE
}tacSTATE; 

/* 
 * Types
 */

typedef struct {
  unsigned long seconds;
  long microsec;
}tacTIME;

typedef struct {
  SEM_ID semId;
  int priority;
  int taskId;
  int enable;
}tacSYNCHRO_TASK;

typedef struct {
  int level;
  int vector;
}tacSYNCHRO_INT;

typedef struct {
  tacSYNCHRO_TYPE type;
  tacSYNCHRO_TASK task;
  tacSYNCHRO_INT interrupt;
}tacSYNCHRO;

typedef struct {
  unsigned short errNum;
  ccsLOC_ID location;
  /* VLTSW20030091 - Error parameter too short */
  char param[128];
}tacERROR;

typedef struct {
  int number;
  tacPARAM_TYPE type[tacMAX_PARAM_NUMBER];
  char buffer[tacMAX_PARAM_SIZE];
}tacPARAM;

typedef struct {
  tacTIME timeStamp;
  unsigned long number;
  double buffer[tacMAX_DATA_NUMBER];
}tacDATA;

typedef struct {
  unsigned short simulation;
  double samplingPeriod;
  tacTIME currentTime;
}tacSHARED;

typedef STATUS (* tacFCTPTR)();
typedef void (* tacVOIDFCTPTR)();

/* VLTSW20040025 - Support record to binary files */
typedef struct {
  FILE* fileHandle;
  ccsERROR* error;
}tacRECORD;

/*
 * Macros
 */

#define tacTypesInitTimeStructure(time)      \
{                                            \
  (time)->seconds  = tacTIME_NEVER_SECONDS;  \
  (time)->microsec = tacTIME_NEVER_MICROSEC; \
}

#define tacTypesInitSynchroTaskStructure(synch) \
{                                               \
  (synch)->semId = NULL;                        \
  (synch)->priority = 0;                        \
  (synch)->taskId = ERROR;                      \
  (synch)->enable = FALSE;                      \
}

#define tacTypesInitSynchroIntStructure(synch) \
{                                              \
  (synch)->level  = 0;                         \
  (synch)->vector = 0;                         \
}

#define tacTypesInitSynchroStructure(synch)                                 \
{                                                                           \
  (synch)->type = tacSYNCHRO_NO_SOURCE;                                     \
                                                                            \
  tacTypesInitSynchroTaskStructure((tacSYNCHRO_TASK*) &((synch)->task));    \
  tacTypesInitSynchroIntStructure((tacSYNCHRO_INT*) &((synch)->interrupt)); \
}

#define tacTypesInitParamStructure(parameter)                          \
{                                                                      \
  double* pRealParam = (double*) ((parameter)->buffer);                \
  int arrayIndex = 0;                                                  \
                                                                       \
  (parameter)->number = 0;                                             \
                                                                       \
  for (arrayIndex = 0; arrayIndex < tacMAX_PARAM_NUMBER; arrayIndex++) \
    {                                                                  \
      (parameter)->type[arrayIndex] = tacPARAM_DOUBLE;                 \
      pRealParam[arrayIndex] = 0.0;                                    \
    }                                                                  \
}

#define tacTypesInitDataStructure(data)                               \
{                                                                     \
  tacTIME* time = (tacTIME*) &((data)->timeStamp);                    \
  int arrayIndex = 0;                                                 \
                                                                      \
  tacTypesInitTimeStructure(time);                                    \
                                                                      \
  (data)->number = 0;                                                 \
                                                                      \
  for (arrayIndex = 0; arrayIndex < tacMAX_DATA_NUMBER; arrayIndex++) \
    {                                                                 \
      (data)->buffer[arrayIndex] = 0.0;                               \
    }                                                                 \
}

#define tacTypesInitSharedStructure(shared)            \
{                                                      \
  tacTIME* time = (tacTIME*) &((shared)->currentTime); \
                                                       \
  tacTypesInitTimeStructure(time);                     \
                                                       \
  (shared)->samplingPeriod = 1.0;                      \
  (shared)->simulation = FALSE;                        \
}

/*
 * Global variables
 */

/*
 * Functions
 */

#endif /*!TAC_TYPES_H*/






