#ifndef TIM_H
#define TIM_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tim.h,v 3.12 2005/07/26 08:51:41 vltsccm Exp $" 
*
* tim.h  -  This is the interface file for the LCU TIM device driver which 
*           forms a module of the VLT software. It contains the declarations
*           user available constants, data types and variables as well as the
*           function prototypes.
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* ---v1.2---
* ssandroc   02.08.1994 Modified for version 1.3
* ssandroc   22.08.1994 Included "lcudrvPublic.h"
* ---v1.3--- 03.10.1994 ---RELEASED---
* S.Sandrock 07.02.1995 Updated version
* ---v1.4--- 13.02.1995 ---RELEASED---
*/

#include "vxWorks.h"
#include "semLib.h"		/* User deals with semaphores            */
#include "lcudrvPublic.h"	/* Include public common driver literals */
#include "timCommands.h"	/* Include TIM command definitions       */
#include "timErrors.h"		/* Include TIM error codes and literals  */


/*
 * Version number
 */
#define timVERSION	"@(#) tim $Revision: 3.12 $"


/*
 * TIM driver global variables
 */

extern BOOL timClkAutoInstall;	/* Controls automatic installation of ticker */
				/* within timDevCreate */


/*
 * TIM driver installation functions
 */

int timDrv (int devices, int channels, int timeout );
int timDevCreate (char *name, void *baseAddr, int irNumber, int irLevel );

/*
 * TIM driver tool functions
 */

int timVersion ( void );
int timDevShow ( void );
int timPrintDevice ( int unit );
int timPrintChannel ( int fd );
int timPrintRegs ( int unit );
int timPrintError ( int errorCode );
int timPrintCmd ( int cmdCode );


/*
 * TIM driver system-clock functions
 */

int timClkInstall(const char *devName);
int timClkUninstall(void);
int timClkConnect(FUNCPTR routine, int arg);
int timClkEnable(void);
int timClkDisable(void);
int timClkRateSet(int ticksPerSecond);
int timClkRateGet(void);
int timClkSwitch(void);

/*
 * TIM timer functions
 */
SEM_ID timGetTenMillisecondInterruptSem(void);
SEM_ID timGetOneSecondInterruptSem(void);

#endif /* !TIM_H */
