#ifndef TIM_COMMANDS_H
#define TIM_COMMANDS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: timCommands.h,v 3.12 2005/07/26 08:51:41 vltsccm Exp $" 
*
* timCommands.h  -  Contains literals of all defined commands and
*                   structure types for commands with multiple arguments.
*
* who        when       what
* ---------- ---------- ----------------------------------------------
* --v1.2--
* ssandroc   02.08.1994 Modified for version 1.3
* --v1.3--   03.10.1994 ---RELEASED---
* S.Sandrock 07.02.1995 added `userArg' for argument to ISR.
* ---v1.4--- 13.02.1995 ---RELEASED---
*/


/*
 * Driver Commands
 */
#define timCMD_RESET						 80
#define timCMD_GET_TIMEOUT					 70
#define timCMD_SET_TIMEOUT					180
#define timCMD_FREE_DEVICE					190
#define timCMD_WAIT						195


/*
 * UTC Time Commands
 */
#define timCMD_GET_UTC_STATUS					 50
#define timCMD_GET_TIME						 60
#define timCMD_SET_LOCAL_MODE					150
#define timCMD_SET_UTC_MODE					160
#define timCMD_SET_TIME						170


/*
 * Timer Commands
 */
#define timCMD_GET_TIMER_STATUS					 10
#define timCMD_GET_TIME_OF_NEXT_INTERRUPT			 20
#define timCMD_GET_TIME_OF_LAST_INTERRUPT			 30
#define timCMD_GET_INTERRUPT_RATE				 40
#define timCMD_CONFIG_TIMER					110
#define timCMD_STOP_TIMER					140


/*
 * Interrupt Commands
 */
#define timCMD_CONFIG_INTERRUPT_ON_NEXT_SECOND			 90
#define timCMD_CONFIG_INTERRUPT_ON_NEXT_TEN_MILLISECOND		100
#define timCMD_STOP_INTERRUPT_ON_NEXT_SECOND			120
#define timCMD_STOP_INTERRUPT_ON_NEXT_TEN_MILLISECOND		130
#define timCMD_GET_INTERRUPT_SEM				 45


/*
 *--- Argument Data Structures
 */

typedef struct timUTC			/* UTC in UNIX format */
    {
    unsigned long     seconds;		/* seconds since since Jan. 1, 1970 */
    long              microsec;		/* and microseconds */
    } timUTC;

/* Prototype of User ISR: */
typedef void (timUSER_ISR)(void *userArg, int intNr);

typedef struct timTIMER_CONFIG		/* Timer Config and Status Structure */
    {
    int    number;			/*   timer number                    */
    int    mode;			/*   software mode                   */
    UINT16 count;			/*   initial count value             */
    int    rate;			/*   clock rate code                 */
    struct timUTC utc;			/*   absolute UTC (various purposes) */
    timUSER_ISR *userISR;		/*   address of user supported ISR   */
    void        *userArg;		/*   argument-ptr to pass to userISR */
    SEM_ID semID;			/*   semaphore for task-level ISR's  */
    int    status;			/*   timer status (read-only)        */
    } timTIMER_CONFIG;

typedef struct timINTERRUPT_CONFIG	/* Interrupt Configuration Structure */
    {
    int    mode;			/*   0 = one-shot, 1 = periodic      */
    timUSER_ISR *userISR;		/*   address of user supported ISR   */
    void        *userArg;		/*   argument-ptr to pass to userISR */
    SEM_ID semID;			/*   semaphore for task-level ISR's  */
    } timINTERRUPT_CONFIG;



/* Commands in numerical order: */

/* Read Commands */

/* #define timCMD_GET_TIMER_STATUS				 10 */
/* #define timCMD_GET_TIME_OF_NEXT_INTERRUPT			 20 */
/* #define timCMD_GET_TIME_OF_LAST_INTERRUPT			 30 */
/* #define timCMD_GET_INTERRUPT_RATE				 40 */
/* #define timCMD_GET_INTERRUPT_SEM				 45 */
/* #define timCMD_GET_UTC_STATUS				 50 */
/* #define timCMD_GET_TIME					 60 */
/* #define timCMD_GET_TIMEOUT					 70 */

/* Write Commands */

/* #define timCMD_RESET						 80 */
/* #define timCMD_CONFIG_INTERRUPT_ON_NEXT_SECOND		 90 */
/* #define timCMD_CONFIG_INTERRUPT_ON_NEXT_TEN_MILLISECOND	100 */
/* #define timCMD_CONFIG_TIMER					110 */
/* #define timCMD_STOP_INTERRUPT_ON_NEXT_SECOND			120 */
/* #define timCMD_STOP_INTERRUPT_ON_NEXT_TEN_MILLISECOND	130 */
/* #define timCMD_STOP_TIMER					140 */
/* #define timCMD_SET_LOCAL_MODE				150 */
/* #define timCMD_SET_UTC_MODE					160 */
/* #define timCMD_SET_TIME					170 */
/* #define timCMD_SET_TIMEOUT					180 */
/* #define timCMD_FREE_DEVICE					190 */
/* #define timCMD_WAIT						195 */

#endif /* !TIM_COMMANDS_H */
