/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: timerLCC.h,v 4.97.1.1 2005/10/24 13:55:48 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs 2001-08-30 added funtion timerSetInterruptTimer to use
*                     timer 0 or 5
* bgustafs  04/05/99  removed timerSetInterruptOld and timerSendCommandOld
* imunoz    26/08/97  updated type of timerId in timerGetTimerStatus prototype
* gfilippi  27/09/95  created from $VLTSCCM/VLTSW/LCC/2.2.4/lcc
*
*/

/*-----------------------------------------------------------------------------
*  ESO VLT              LCU Common Software             SPACEBEL Informatique
*------------------------------------------------------------------------------
*  Version      Date            Author          Comment
*------------------------------------------------------------------------------
*  1.1          18/12/93        lccteam         First version
*  1.2          18/01/94        P.Rochez        Changed name of function
*                                               timerSendMessage ->
*                                               timerSendCommand
*  2.1          31/01/95        P.Rochez
*       Moved timerScheduleTask(), timerSendCommand(), timerCancelRequest() and
*           timerGetReport() to common part
*       Modified interface of timerScheduleTask() function
*       Added parameter to timerSendCommand() and timerSetInterrupt()
*       Added functions timerSleep() and timerSendReply()
*       Created LCC specifis include file
*------------------------------------------------------------------------------
*  SCCS Id:  timerLCC.h  2.1  03 Feb 1995
*----------------------------------------------------------------------------*/

/*
 * <timerLCC.h> - LCU / TIMER Interface File
 */

/************************************************************************
 *                      LCU Specific Definitions                        *
 ************************************************************************/

/* 
 * Header Files
 */

#include "ccs.h"


/* 
 * Constants 
 */

#define timerTIMER_NUMBER    6      /* Timer 0 to timer 5               */
#define timerPULSE_NUMBER    2      /* Second and 10 millisecond pulses */


/*
 * Types
 */

typedef enum {                      /* Mode of activation for TIM timers*/
    timerONCE_IMMEDIATE = 1,        /* Activation once after the given  */
                                    /* time starting immediately        */

    timerONCE_AFTER_TIME_GATE,      /* Activation once after the given  */
                                    /* time starting on the next 100 ms */
                                    /* gating signal                    */

    timerONCE_AFTER_HW_GATE,        /* Activation once after the given  */
                                    /* time starting on the hardware    */
                                    /* gating signal. If the gating     */
                                    /* signal changes when the timer is */
                                    /* still counting, the timer stops  */
                                    /* when the signal is "low" and     */
                                    /* counts when the signal is "high".*/
 
    timerPER_IMMEDIATE,             /* Periodic activation with a given */
                                    /* rate starting immediately        */

    timerPER_AFTER_TIME_GATE,       /* Periodic activation with a given */
                                    /* rate starting on the next 100 ms */
                                    /* gating signal                    */

    timerPER_AFTER_HW_GATE,         /* Periodic activation with a given */
                                    /* rate starting on the hardware    */
                                    /* gating signal. If the gating     */
                                    /* signal changes when the timer is */
                                    /* still counting, the timer stops  */
                                    /* when the signal is "low" and     */
                                    /* counts when the signal is "high".*/

    timerAFTER_TIME_GATE,           /* Activation after a given time    */
                                    /* starting on each 100 ms gating   */
                                    /* signal                           */

    timerAFTER_HW_GATE,             /* Activation after a given         */
                                    /* time starting each time the      */
                                    /* hardware gating signal is set.   */
                                    /* If the gating signal changes     */
                                    /* when the timer is still          */
                                    /* counting, the timer stops when   */
                                    /* the signal is "low" and counts   */
                                    /* when the signal is "high".       */
    timerUTC,                       /* Activation on a given UTC        */

    timerPER_AFTER_UTC              /* Periodic activation with given   */
                                    /* rate starting at the given UTC   */
} timerTIMER_MODE;


typedef enum {                      /* List of pulse type               */
    timerSECOND = 1,                /* One second pulse                 */
    timer10_MILLISECOND             /* 10 millisecond pulse             */
} timerPULSE;


typedef enum {                      /* Mode of activation for a pulse   */
    timerPULSE_ONCE = 1,            /* One shot activation on pulse     */
    timerPULSE_PER                  /* Periodic activation on pulse     */
} timerPULSE_MODE;


typedef enum {                      /* State of a timer                 */
    timerFREE = 1,
    timerBUSY
} timerSTATE;


typedef timerSTATE  timerTIMER_STATE[timerTIMER_NUMBER];
                                    /* State of the timers 0-4          */
typedef timerSTATE  timerPULSE_STATE[timerPULSE_NUMBER];
                                    /* State of the pulse signals       */


typedef struct {                    /* Parameters of TIMSCMD command    */
    timerSCHEDULE_MODE mode;        /* Mode of activation               */
    ccsTIMEVAL         startTime;   /* Start time                       */
    ccsTIMEVAL         interval;    /* Interval for periodic activation */
    msgCMD             command;     /* Command name                     */
    msgCMDID           commandId;   /* Command identifier               */
    ccsENVNAME         destenv;     /* Where to send the command        */
    ccsPROCNAME        destproc;    /* To whom : process name           */
    msgLENGTH          buflen;      /* Message length                   */
    msgCHECKFLAG       checkFlag;   /* Flag specifying checks to be done*/
} timerTIMSCMD_PARAM;


typedef struct {                    /* Parameters of TIMSRPY command    */
    timerSCHEDULE_MODE mode;        /* Mode of activation               */
    ccsTIMEVAL         startTime;   /* Start time                       */
    ccsTIMEVAL         interval;    /* Interval for periodic activation */
    msgPROCESSID       destId;      /* Destination of the reply         */
    msgCMDID           commandId;   /* Command Id replied to            */
    msgCMD             command;     /* Command replied to               */
    msgLENGTH          buflen;      /* Buffer length. Max len = 8kB.    */
    vltLOGICAL         lastReply;   /* Flag to indicate if another      */
                                    /* reply follows                    */
} timerTIMSRPY_PARAM;



/*
 * Functions prototypes
 */

ccsCOMPL_STAT timerSetInterrupt(    /* Connect an interrupt routine to  */
                                    /* timer 0                          */
    const void         *address,    /* Address of interrupt routine     */
    timerTIMER_MODE    mode,        /* Activation mode                  */
    const ccsTIMEVAL   *startTime,  /* Start time                       */
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    void               *argument,   /* Argument to pass to interrupt    */
                                    /* routine (not implemented)        */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT timerSetInterruptTimer( /* Connect an interrupt routine   */
                                    /* to timer 0 0r 5                  */
    const void         *address,    /* Address of interrupt routine     */
    timerTIMER_MODE    mode,        /* Activation mode                  */
    const ccsTIMEVAL   *startTime,  /* Start time                       */
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    vltUINT8           timerId,     /* Timer identifier: 0 or 5         */
    void               *argument,   /* Argument to pass to interrupt    */
                                    /* routine (not implemented)        */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );

ccsCOMPL_STAT timerSetTask(         /* Connect an task to a timer       */
    timerTIMER_MODE    mode,        /* Activation mode                  */
    const ccsTIMEVAL   *startTime,  /* Start time                       */
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    vltUINT8           timerId,     /* Timer identifier: 1-4            */
    SEM_ID             *semId,      /* Semaphore identifier             */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT timerConnectOnPulse(  /* Connect an task to a pulse       */
    timerPULSE         pulse,       /* Pulse identification             */
    timerPULSE_MODE    mode,        /* Mode of activation               */
    SEM_ID             *semId,      /* Semaphore identifier             */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT timerGetStatus(       /* Get the status of the Time       */
                                    /* Handling module                  */
    timerTIMER_STATE   *timerState, /* Timer states (free/busy)         */
    timerPULSE_STATE   *pulseState, /* Pulse states (free/busy)         */
    timsMODE           *mode,       /* Mode of the Time module          */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT timerGetTimerStatus(  /* Get the status of a timer        */
    vltUINT32          timerId,     /* requestId                        */
    timerSTATE         *state,      /* State (free/busy)                */
    vltLOGICAL         *started,    /* Timer started (True = Yes)       */
    timerTIMER_MODE    *mode,       /* Timer mode                       */
    ccsTIMEVAL         *lastTime,   /* Time of last interrupt           */
    ccsTIMEVAL         *nextTime,   /* Time of next interrupt           */
    ccsERROR           *error       /* Returned error structure         */
    );


ccsCOMPL_STAT timerScheduleTask(    /* Schedule a task synchronised to  */
				    /* Time Reference System (OBSOLETE) */
    const void         *address,    /* Task entry adress                */
    const ccsVXWORKS_OPTIONS *options, /* VxWorks run-time options      */
    timerSCHEDULE_MODE mode,        /* Mode of activation               */
    const ccsTIMEVAL   *startTime,  /* Start time                       */
    const ccsTIMEVAL   *interval,   /* Interval for periodic activation */
    vltUINT32          *requestId,  /* Request identifier               */
    ccsERROR           *error       /* Returned error structure         */
    );
