#ifndef TOO_H
#define TOO_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: too.h,v 1.45 2004/11/16 13:36:44 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  09/01/00  lcuDebug, lcuTrace input string as const, SPR990460
* bgustafs  26/08/99  Macro FREE renamed to tooFREE. FREE is defined in 
*                     Tornado II
* bgustafs  05/07/99  made it C++ compliant
* bgustafs  23/04/96  Moved lcu strings to too.c
*
* T.Phan      27/07/94  First version 1.0
* M.Chiesa    02/08/94  added variable parameter lists
* T.Phan      08/10/94  added 
*			- literals lcuON_STRING, lcuOFF_STRING,
*				   lcuUP_STRING, lcuDOWN_STRING, 
*				   lcuLEFT_STRING, lcuRIGHT_STRING
*			- types lcuON_OFF, lcuOPEN_CLOSE, lcuDIRECTION
*			- data lcuNoYesString, lcuOnOffString
*			- macro REPLY_ADD to add string into reply buffer
* T.Phan      09/10/94  changed macro ERROR_ADD for line number 
*			Version used for laSilla m2, m3 october tests
* T.Phan      04/11/94  Version 1.1
*			added macros BEGIN/END_CRITICAL_REGION 
*			added structure lcuPARAMS for interface functions
*			changed macro REPLY_ADD for variable arguments
* BGU         23/02/95  Removed error macros. Modified lcuStateString (
*                       (STANDALONE deleted)
* BGU         09/10/95  Added use_lcuStateString to suppress compiler warning
* bgustafs    23/11/95  Added missing argument in macro call
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifdef __cplusplus
extern "C" {
#endif

#include "ccs.h"
#include "db.h"
#include "err.h"
#include "msg.h"
#include "tooErrors.h"

/*
 * Constants
 */

#define IN
#define OUT

#define ONE_SEC			sysClkRateGet()
#define TEN_MSEC		ONE_SEC/100    /* 10 msec */

#define lcuON_STRING 		"On"
#define lcuOFF_STRING		"Off"

#define lcuUP_STRING 		"Up"
#define lcuDOWN_STRING		"Down"
#define lcuLEFT_STRING 		"Left"
#define lcuRIGHT_STRING		"Right"

/*
 * Types
 */
typedef enum {			/* State of the module */
    lcuOFF_STATE = 1,
    lcuLOADED,                     
    lcuSTAND_BY,
    lcuON_LINE
} lcuSTATE;

typedef enum {                          
    lcuOFF = 1,
    lcuON
} lcuON_OFF;

typedef enum {                          
    lcuOPEN = 1,
    lcuCLOSE
} lcuOPEN_CLOSE;

typedef enum {                          
    lcuUP = 1,
    lcuDOWN,
    lcuLEFT,
    lcuRIGHT
} lcuDIRECTION;


/* for interface functions */
typedef union {
    vltINT8  str[1];
    vltINT32 intArray[1];
    struct {
        vltINT32 int1;
        vltINT8  str[1];
    } intStr;    
} lcuPARAMS;

/*
 * Data
 */
extern lcuON_OFF lcuDebugFlag;
extern lcuON_OFF lcuTraceFlag;
extern lcuON_OFF lcuVerboseFlag;

extern const char *lcuOnOffString[3];
extern const char *lcuStateString[5];

/*
 * too Public functions
 */
ccsCOMPL_STAT tooBootTime(const char *updateOverwriteMode);
ccsCOMPL_STAT tooDate(char *date);
ccsCOMPL_STAT tooSetLCUTime(void);
void tooLogSave(const int value);

/*
 * cmd Dummy function
 */

extern
ccsCOMPL_STAT tooCmdDummy(                  /* Start command interpreter    */
    const ccsPROCNAME procName,      /* command interpreter name     */
    const char        *defFile      /* definition table filename    */
);

/*
 * Verbose
 */

ccsCOMPL_STAT tooLogComment(ccsERROR *error, const char *logstr, ...);

ccsCOMPL_STAT lcuVerbose (
           IN  msgMSG   *msg,
           IN  char     *buffer,
           OUT ccsERROR *error
           ); 
         
ccsCOMPL_STAT lcuSetVerbose (
           IN  const char         *onStr,
           OUT       ccsERROR     *error
           );
ccsCOMPL_STAT lcuGetVerbose (
           OUT       char         *onStr,
           OUT       ccsERROR     *error
           );

int tooReplyf(IN const msgMSG *msg, IN const char *format, ...);

#define tooReplyfVerbose(args...)\
    ((lcuVerboseFlag == lcuON) ? tooReplyf(##args) : 0)

/*
 * Debug & Trace
 */

#ifndef TRACE_H
void lcuDebug (
           IN  const char * fmt, ...
           );
ccsCOMPL_STAT lcuSetDebug (
           IN  const char         *onStr,
           OUT       ccsERROR     *error
           );
ccsCOMPL_STAT lcuGetDebug (
           OUT       char         *onStr,
           OUT       ccsERROR     *error
           );
void lcuTrace (
           IN  const char * fmt, ...
           );
ccsCOMPL_STAT lcuSetTrace (
           IN  const char         *onStr,
           OUT       ccsERROR     *error
           );
ccsCOMPL_STAT lcuGetTrace (
           OUT       char         *onStr,
           OUT       ccsERROR     *error
           );
#endif

/*
 * Database access
 */

ccsCOMPL_STAT dbReadLogical(
           IN  const dbSYMADDRESS  attrName,
           OUT       vltLOGICAL   *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbWriteLogical(
           IN  const dbSYMADDRESS  attrName,
           IN  const vltLOGICAL   *buffer,
           OUT       ccsERROR     *error
           );
       
ccsCOMPL_STAT dbReadUInt32(
           IN  const dbSYMADDRESS  attrName,
           OUT       vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );
       
ccsCOMPL_STAT dbWriteUInt32(
           IN  const dbSYMADDRESS  attrName,
           IN  const vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbReadInt32(
           IN  const dbSYMADDRESS  attrName,
           OUT       vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbWriteInt32(
           IN  const dbSYMADDRESS  attrName,
           IN  const vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbReadDirectLogical(
           IN  const dbDIRADDRESS *dirAddr,
           OUT       vltLOGICAL   *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbWriteDirectLogical(
           IN  const dbDIRADDRESS *dirAddr,
           IN  const vltLOGICAL   *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbReadDirectUInt32(
           IN  const dbDIRADDRESS *dirAddr,
           OUT       vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbWriteDirectUInt32(
           IN  const dbDIRADDRESS *dirAddr,
           IN  const vltUINT32    *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbReadDirectInt32(
           IN  const dbDIRADDRESS *dirAddr,
           OUT       vltINT32     *buffer,
           OUT       ccsERROR     *error
           );

ccsCOMPL_STAT dbWriteDirectInt32(
           IN  const dbDIRADDRESS *dirAddr,
           IN  const vltINT32     *buffer,
           OUT       ccsERROR     *error
           );

/*
 * Trace macro
 */

#define lcuTRACE \
    lcuTrace("%s line#%d",__FILE__,__LINE__)

/*
 * Dynamic memory allocation macros
 */

#define ALLOCATE(type) \
    ((type*)calloc(1,sizeof(type)))

#define ALLOCBUF(type,nbr) \
    ((type*)calloc(nbr,sizeof(type)))

#define REALLOCBUF(ptr,type,nbr) \
    ((type*)((ptr) ? realloc(ptr,(nbr)*sizeof(type)):calloc(nbr,sizeof(type))))

#define tooFREE(ptr) \
    ((void) ((ptr) ? (free((char*)ptr), ptr = NULL) : NULL))

/*
 * Message reply macros
 */
 

#define REPLY_ADD(format, arg...) \
{ \
    sprintf(msg->body, format, ##arg); \
    *replyBuffer = (char *)&msg->body; \
    *replyBufferSize = strlen(msg->body) + 1; \
}

/*
 * Critical region macros
 */
 
#define BEGIN_CRITICAL_REGION(semId)  \
{ \
if(semTake(semId, ONE_SEC) != OK) \
    ERROR_RETURN(tooERR_VXWORKS,__FILE__,  "Take Semaphore failed, %d", errnoGet()) \
}

#define END_CRITICAL_REGION(semId) \
{ \
if(semGive(semId) != OK) \
    ERROR_RETURN(tooERR_VXWORKS, __FILE__, "Give Semaphore failed, %d", errnoGet()) \
}

#include "tooInterface.h"

#ifdef __cplusplus
}
#endif

#endif /* !TOO_H */
