#ifndef TOO_COM_H
#define TOO_COM_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tooCom.h,v 1.45 2004/11/16 13:36:44 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  05/07/99  made it C++ compliant
* ssandroc   26.01.1996 created
*/

#ifdef __cplusplus
extern "C" {
#endif


#include <vxWorks.h>
#include <stdio.h>
#include <string.h>
#include <types.h>

#include "too.h"
#include "tooDefines.h"
#include "lccErrors.h"

/*
 * container for serial communication parameters
 * (note: do not change the types!)
 */
typedef struct {
    int32_t debug;	/* record transceived data in file [0,1]*/
    int32_t baudrate;	/* baudrate to use	[1200..9600]	*/
    int32_t charbits;	/* bits per character	[5..8]		*/
    int32_t stopbits;	/* number of stop bits	[1..2]		*/
    int32_t parity;	/* parity type [1:even, 0:none, -1:odd]	*/
    int32_t handshake;	/* handshake [0:DTR,1:Xon,2:RTS,3:none]	*/
    int32_t reclen;	/* iser record length [0:max, 1..1023]	*/
    int32_t rxmode;	/* iser receive mode (see iser) [0..7]	*/
    int32_t timeout0;	/* iser 1st-char timeout [0,1..64K ms]	*/
    int32_t timeout1;	/* iser ch-to-ch timeout [0,1..64K ms]	*/
} tooCOM_PARAMS;
/* the following items must be in the same order as in tooCOM_PARAMS! */
#define tooCOM_PARAM_NAMES \
	{"debug",\
	 "baudrate", "charbits", "stopbits", "parity", "handshake",\
	 "reclen", "rxmode", "timeout0", "timeout1"}
#define tooCOM_PARAM_DEFAULTS \
	{ tooCOM_DEBUG,\
	  tooCOM_BAUDRATE, tooCOM_CHARBITS, tooCOM_STOPBITS,\
	  tooCOM_PARITY, tooCOM_HANDSHAKE,\
	  tooCOM_RECLEN, tooCOM_RXMODE, tooCOM_TIMEOUT0, tooCOM_TIMEOUT1 }


/*
 * tooCom functions
 */
ccsCOMPL_STAT tooComParamsGet(const dbSYMADDRESS pointName,
			      tooCOM_PARAMS	*comParams,
			      ccsERROR		*error);

ccsCOMPL_STAT tooComOpen(const char		*devName,
			 const tooCOM_PARAMS	*comParams,
			 int			*fdPtr,
			 ccsERROR		*error);

ccsCOMPL_STAT tooComClose(IN  int	fd, 
			  OUT ccsERROR	*error);

ccsCOMPL_STAT tooComRead(IN  int	fd, 
			 IN  char	*buffer,
			 IN  size_t	maxbytes,
			 OUT int	*nBytesPtr,
			 OUT ccsERROR	*error);

ccsCOMPL_STAT tooComWrite(IN  int	fd, 
			  IN  char	*buffer,
			  IN  size_t	maxbytes,
			  OUT int	*nBytesPtr,
			  OUT ccsERROR	*error);

int tooComRead_d(IN  int	fd, 
		 IN  char	*buffer,
		 IN  size_t	maxbytes);

int tooComWrite_d(IN  int	 fd, 
		  IN  char	 *buffer,
		  IN  size_t	 maxbytes);

#ifdef __cplusplus
}
#endif

#endif 	/* !TOO_COM_H */
