/************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id" 
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   1999-09-30  created
*------------------------------------------------------------------------
*/
/************************************************************************
*   NAME
*       lsftplMotor - Motor control
*       
*   SYNOPSIS
*       #include "lsftplMotor.h"
*
*       ccsCOMPL_STAT lsftplMotorConstructor (OUT ccsERROR *error)
*       void          lsftplMotorDestructor  ( void );
*
*       PUBLIC ATTRIBUTES:
*
*       PRIVATE ATTRIBUTES:
*       static const char      *motorNames[];
*       static lsftplMOTOR_DATA  motorData;
*
*       PUBLIC METHODS:
*       ccsCOMPL_STAT lsftplMotorOff         (OUT ccsERROR *error);
*       ccsCOMPL_STAT lsftplMotorStop        (OUT ccsERROR *error);
*       ccsCOMPL_STAT lsftplMotorInit        (OUT ccsERROR *error);
*       ccsCOMPL_STAT lsftplMotorGetState    (OUT vltINT32 *motorState,
*                                            OUT ccsERROR *error);
*       ccsCOMPL_STAT lsftplMotorMoveAbs     (IN  vltINT32  targetEnc[],
*                                            OUT ccsERROR *error);
*       PRIVATE METHODS:
*       static ccsCOMPL_STAT lsftplMotorSetState (OUT ccsERROR *error);
*
*
*   DESCRIPTION
*       These methods are dedicated to the control of motor(s), all the 
*       motor defined in <motorNames> are handled in parallel
*   FILES
*       None
*
*   ENVIRONMENT
*       None
*
*   SIGNALS
*       None
*
*   DATABASE 
*       None
*
*   RETURN VALUES
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId = "@(#) $Id: lsftplMotor.c,v 1.15 2003/09/10 12:41:51 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsftplMotor.h"


static char *attrTable   = NULL; 
static char *attrNames[] = 
    {
    "M1:motor:STATUS.posUser",
    "M2:motor:STATUS.posUser",
    NULL
    };

typedef enum
    {
    lsftplDB_POSITION1,
    lsftplDB_POSITION2,
    lsftplDB_LAST
    } lsftplDB_INDEX;

/*
 * macro for reading an setup attribute and handling an error
 */
#define lsftplDB_READ_ATTR(attrIndex,buffer) \
    if (caiReadAttribute(attrTable,attrIndex,buffer,error) == FAILURE) \
        lsftplERR_RETURN(lsftplERR_DBREAD,attrNames[attrIndex])

#define lsftplDB_WRITE_ATTR(attrIndex,buffer) \
    if (caiWriteAttribute(attrTable,attrIndex,buffer,error) == FAILURE) \
        lsftplERR_RETURN(lsftplERR_DBWRITE,attrNames[attrIndex])

static ccsCOMPL_STAT lsftplMotorDbConstructor(ccsERROR *error)
{
  dbSYMADDRESS dbRoot;
  
  sprintf(dbRoot,"<alias>%s%s%s",
	  lsftplMODULE_NAME,lsfDB_CONTROL_POINT,lsfDB_MOTOR_POINT);
  
  if (caiGetAttributes(&attrTable,dbRoot,attrNames,error) == FAILURE)
    return FAILURE;
  
  return SUCCESS;
}

static void lsftplMotorDbDestructor( void )
{
  caiFreeAttrTable(attrTable);
  attrTable = NULL;
}

ccsCOMPL_STAT lsftplMotorConstructor ( lsfMOTOR_DATA *motorData, 
				       const char     *motorName,
				       ccsERROR       *error )
{
    lsftplPrintf("lsftplMotorConstructor(motorData=0x%08X,motorName=%s)\n",(int)motorData,motorName);
    if (lsftplMotorDbConstructor(error) == FAILURE)
	return FAILURE;
    return SUCCESS;
}

void lsftplMotorDestructor ( lsfMOTOR_DATA *motorData )
{
    lsftplPrintf("lsftplMotorDestructor(motorData=0x%08X)\n",(int)motorData);
    lsftplMotorDbDestructor();
}

ccsCOMPL_STAT lsftplMotorMoveAbs ( IN  const char *motor, 
				   IN  vltDOUBLE    target[], 
				   IN  const char *unit[], 
				   IN  vltLOGICAL  wait,
				   OUT ccsERROR   *error)
{
    lsfMOTOR_DATA *motorData = lsfMotorGetData(lsftplGlobalData);

    lsftplPrintf("lsftplMotorMoveAbs(motorName=%s,...)\n",motor);
    return lsfMotorMoveAbs(motorData,motor,target,NULL,wait,error);
}

ccsCOMPL_STAT lsftplMotorWaitMove ( IN  const char *motor,
				   OUT char       *names[], 
				   OUT vltDOUBLE  *position, 
				   OUT char       *unit[], 
				   OUT ccsERROR   *error )
{
    lsfMOTOR_DATA *motorData = lsfMotorGetData(lsftplGlobalData);

    lsftplPrintf("lsftplMotorWaitMove(motorName=%s,...)\n",motor);
    return lsfMotorWaitMove(motorData,motor,names,position,unit,error);
}

/* ___oOo___ */
