/************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsftplServer.c,v 1.15 2003/09/10 12:41:51 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban  06/11/99  add ServerCheckOnline function
* rkarban  06/11/99  remove SetBgTaskId
* rkarban  06/12/96  created
*------------------------------------------------------------------------
*/
/************************************************************************
*   NAME
*   lsftplServer - Handling of SW device server
* 
*   SYNOPSIS
*   #include "lsftplServer.h"
* 
*   DESCRIPTION
*   This module implements all configuration and access methods for the command
*   interpreter task
*
*   METHODS
*   PUBLIC
*    ccsCOMPL_STAT lsftplInitAll(const char *dbRoot, const char *cdt, const char *cit);      
*      lsftplInitAll( void ) initializes the lsftpl module by creating semaphores 
*      and starting the command interpreter.
*      Called by boot script.
*       dbRoot: defines the DB root point for lsftpl.
*                           If NULL is passed, the root point is defaulted to ":lsftpl"
*       cdt: defines the name of the CDT file. If NULL it defaults to "lsftplServer.cdt"
*       cit: defines the name of the CIT file. If NULL it defaults to "lsftplServer.cit"
*       
*    ccsCOMPL_STAT lsftplServerSetSubState (ccsERROR *error)
*      Triggers the calculation of the Server Substate
*      The server decides how to calculate the global substate.
*      This function shall be called, when the global substate needs to be updated. e.g.
*      the bgTask state has changed.
*    vltINT32 lsftplServerGetSubState ( void )
*      returns global substate
*    const char *lsftplServerGetSubStateName( void )
*      returns global substate as string
*    ccsCOMPL_STAT lsftplServerSetInit (vltLOGICAL value, ccsERROR *error)
*      sets global init flag
*    vltLOGICAL lsftplServerGetInit ( void )
*      returns global init flag
*    ccsCOMPL_STAT lsftplServerSetSim (vltLOGICAL value, ccsERROR *error)
*      sets global sim flag
*    vltLOGICAL lsftplServerGetSim ( void )
*      returns global sim flag
*    ccsCOMPL_STAT lsftplServerSetState (lccDEV_MODE value, ccsERROR *error)
*      sets the global server state
*    lccDEV_MODE lsftplServerGetState ( void )
*      returns global server state
*    const char *lsftplServerGetStateName ( void )
*      returns global server state as string
*    ccsCOMPL_STAT lsftplServerTakeInitSem ( void )
*      tries to take the init semaphore used to initialize the server (to prevent race conditions)
*    ccsCOMPL_STAT lsftplServerGiveInitSem ( void )
*      releases init semaphore
*    ccsCOMPL_STAT lsftplServerTakeCmdSem ( void )
*      tries to take cmd sempahore that can be taken if a command shall be executed, that should
*      be mutual exclusive
*    void lsftplServerGiveCmdSem ( void )
*      releases cmd semaphore
*    void lsftplServerSetCmdTimeout (vltINT32 timeout)
*      Sets the cmd timeout, thats shall be used for timeouts on the cmd and init semaphore
*    vltINT32 lsftplServerGetCmdTimeout ( void )
*      Gets the cmd timeout, thats shall be used for timeouts on the cmd semaphore
*    void lsftplServerSetBgTaskId (vltINT32 tid)
*      Sets the taskid of the background task, so that the server e.g. can kill it if necessary
*    vltINT32 lsftplServerGetBgTaskId ( void )
*      Gets the taskid of the background task
*    ccsCOMPL_STAT lsftplServerDestructor( ccsERROR *error )
*      Cleans up all internal structures of the server, e.g. before terminating the server
*    ccsCOMPL_STAT lsftplSwitchState(lccDEV_MODE  newDevState,
*                                  ccsERROR *error)
*      Switches the software to the specified state, by setting the module as well
*      as the sub-state in both, global data and database.
*
*   PRIVATE
*    ccsCOMPL_STAT lsftplServerConstructor(const char *dbRoot, ccsERROR *error)
*      creates and initalizes all internal structures of the server task 
*    ccsCOMPL_STAT lsftplServerReadConfig(ccsERROR *error)
*      Forces a read of the Server configuration (from the DB)
*
*   FILES
*     lsftplServer.cdt, lsftplServer.cit
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: lsftplServer.c,v 1.15 2003/09/10 12:41:51 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsftplServer.h"

ccsCOMPL_STAT lsftplServerMainCommand ( OUT char *str, OUT ccsERROR *error )
{
    lsftplPrintf("lsftplServerMainCommand(str=0x%08X)\n",(int)str);
    strcpy(str,"Server Main Command");
    return SUCCESS;
}

/*************************************************************************/
/* ___oOo___ */
