/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsftplSignal.c,v 1.15 2003/09/10 12:41:52 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rkarban  14/09/99  created
*/

/************************************************************************
*   NAME
*     lsftplSignal - Handling of IO signal devices
*
*   SYNOPSIS
*     #include "lsftplSignal.h"
*   DESCRIPTION
*     This module gives a very primitive template of LCC signal usage
*   METHODS
*   PUBLIC
*     ccsCOMPL_STAT lsftplSignalConfigIo(ccsERROR *error)
*       Configures one analog ad one digital example signal
*     ccsCOMPL_STAT lsftplSignalReadDigital (vltUINT32 *value, ccsERROR *error)
*       Reads the configured analog signal
*     ccsCOMPL_STAT lsftplSignalWriteDigital (vltUINT32 value, ccsERROR *error)
*       Writes the configured digital signal
*     ccsCOMPL_STAT lsftplSignalReadAnalog (vltFLOAT *value, ccsERROR *error)
*       Reads the configured analog signal
*     ccsCOMPL_STAT lsftplSignalWriteAnalog (vltFLOAT value, ccsERROR *error)
*       Writes the configured analog signal
*   DATABASE
*     lsftpl:control:signal
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: lsftplSignal.c,v 1.15 2003/09/10 12:41:52 vltsccm Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsftplSignal.h"

ccsCOMPL_STAT lsftplSignalReadDigital ( IN  const char *signalName,
				       OUT char       *names[], 
				       OUT vltUINT32  *value, 
				       OUT ccsERROR   *error )
{
    lsfSIGNAL_DATA *signalData = lsfSignalGetData(lsftplGlobalData);

    lsftplPrintf("lsftplSignalReadDigital(signalName=%s,...)\n",signalName);
    return lsfSignalReadDigital(signalData,signalName,names,value,error);
}

ccsCOMPL_STAT lsftplSignalWriteDigital ( IN  const char *signalName,
					IN  vltUINT32  *value, 
					OUT ccsERROR   *error )
{
    lsfSIGNAL_DATA *signalData = lsfSignalGetData(lsftplGlobalData);

    lsftplPrintf("lsftplSignalWriteDigital(signalName=%s,value=%d)\n",signalName,*value);
    return lsfSignalWriteDigital(signalData,signalName,value,error);
}

ccsCOMPL_STAT lsftplSignalReadAnalog ( IN  const char *signalName, 
				      OUT char       *names[], 
				      OUT vltFLOAT   *value, 
				      OUT ccsERROR   *error )
{
    lsfSIGNAL_DATA *signalData = lsfSignalGetData(lsftplGlobalData);

    lsftplPrintf("lsftplSignalReadAnalog(signalName=%s,...)\n",signalName);
    return lsfSignalReadAnalog(signalData,signalName,names,value,error);
}

ccsCOMPL_STAT lsftplSignalWriteAnalog (IN  const char *signalName, 
				      IN  vltFLOAT   *value, 
				      OUT ccsERROR   *error )
{
    lsfSIGNAL_DATA *signalData = lsfSignalGetData(lsftplGlobalData);

    lsftplPrintf("lsftplSignalWriteAnalog(signalName=%s,value=%g)\n",signalName,*value);
    return lsfSignalWriteAnalog(signalData,signalName,value,error);
}

ccsCOMPL_STAT lsftplSignalInitHook   ( IN  lsfSIGNAL_DATA *signalData, 
				      IN  const char     *signalName, 
				      OUT ccsERROR       *error )
{   
    lsftplPrintf("lsftplSignalInitHook(signalData=0x%08X,signalName=%s)\n",(int)signalData,signalName);
    return SUCCESS;
}

/*___oOo___*/
