/************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: lsftplStandardInterface.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $" 
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-07-18  created
*------------------------------------------------------------------------
*/
/************************************************************************
*   NAME
*       lsftplStandardInterface - lsftpl ACI for Standard LCC Commands
*       IlsftplStdGetVersion, IlsftplStdGetState, IlsftplStdGetSimulation, 
*       IlsftplStdGetInit, IlsftplStdInit, IlsftplStdStandby, IlsftplStdOnline, 
*       IlsftplStdOff, IlsftplStdExit, IlsftplStdStop, IlsftplStdStartSim, 
*       IlsftplStdStopSim, IlsftplStdSelfTest -
*       LCC standard interface functions for lsftpl
*       
*   SYNOPSIS
*       All these functions have the same parameters.
*
*   ccsCOMPL_STAT Ixxx(
*       char         *parameters,
*       msgMSG       *msg,              
*       char         **replyBuffer,     
*       msgLENGTH    *replyBufferSize,  
*       ccsERROR     *error	        
*       );
*
*   DESCRIPTION
*       Used only by cmdExecuteCommand() of the Command Interpreter.
*
*       Internal functions to interface the command. They map stream of
*       bytes input parameter into structured parameters related to
*       functions. After function execution, the output parameters (if 
*       any) are copy into the message body to be sent as reply message.
*        
*       parameters      <IN>  Formatted input parameters.
*       msg             <IN>  Command message received. 
*       replyBuffer     <OUT> Output buffer for reply message.
*       replyBufferSize <OUT> Output buffer size.
*       error           <OUT> Returned error structure.
*
*   FILES
*       None
*
*   ENVIRONMENT
*       None
*
*   SIGNALS
*       None
*
*   DATABASE 
*       None
*
*   RETURN VALUES
*       Called function completion status or 
*       lsftplERR_PARAMETER if supplied parameter illegal
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: lsftplStandardInterface.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsftplStandard.h"
#include "lsfServer.h"
#include "lsftplStandardInterface.h"

lsfSTD_COMMAND(lsftpl,Init,"INIT")
lsfSTD_COMMAND(lsftpl,Standby,"STANDBY")
lsfSTD_COMMAND(lsftpl,Online,"ONLINE")
lsfSTD_COMMAND(lsftpl,Stop,"STOP")
lsfSTD_COMMAND(lsftpl,Off,"OFF")
lsfSTD_COMMAND(lsftpl,Exit,"EXIT")
lsfSTD_COMMAND(lsftpl,StopSim,"STOPSIM")
lsfSTD_COMMAND(lsftpl,StartSim,"SIMULAT")
lsfSTD_COMMAND(lsftpl,SelfTest,"SELFTST")
lsfSTD_COMMAND(lsftpl,Test,"TEST")

lsfACI_FUNCTION(lsftpl,StdGetVersion)
{
    ccsCOMPL_STAT status;
   
    lsfVERBOSE_CMD_RECV("VERSION");
    
    status = lsftplStdGetVersion(msg->body, error);

    *replyBuffer = (char *)&msg->body;
    *replyBufferSize = strlen (msg->body) + 1;

    lsfVERBOSE_CMD_DONE("VERSION");
    
    return(status);    
}

lsfACI_FUNCTION(lsftpl,StdGetState)
{
    ccsCOMPL_STAT status = SUCCESS;
    
    lsfVERBOSE_CMD_RECV("STATE");
    
    strcpy ((char *)&msg->body, lsfServerGetStateName(lsftplGlobalData));
    *replyBuffer = (char *)&msg->body;
    *replyBufferSize = strlen (msg->body) + 1;
        
    lsfVERBOSE_CMD_DONE("STATE");
    
    return (status);
}

lsfACI_FUNCTION(lsftpl,StdGetSimulation)
{
    ccsCOMPL_STAT status;
    vltLOGICAL	  simulation;
    
    lsfVERBOSE_CMD_RECV("GETSIM");
    
    status = lsftplStdGetSimulation(&simulation, error);

    if (status == SUCCESS)
        {
	sprintf((char *)&msg->body,"%s",
		simulation ? lcuON_STRING : lcuOFF_STRING);
        *replyBuffer = (char *)&msg->body;
        *replyBufferSize = strlen (msg->body) + 1;
        }
    
    lsfVERBOSE_CMD_DONE("GETSIM");
    
    return (status);
}

lsfACI_FUNCTION(lsftpl,StdGetInit)
{
    ccsCOMPL_STAT status;
    vltLOGICAL	  initialised;
    
    lsfVERBOSE_CMD_RECV("GETINIT");
    
    status = lsftplStdGetInit(&initialised, error);

    if (status == SUCCESS)
        {
        sprintf((char *)&msg->body,"%s", 
		lcuOnOffString[initialised ? lcuON : lcuOFF]);
        *replyBuffer = (char *)&msg->body;
        *replyBufferSize = strlen (msg->body) + 1;
        }
    
    lsfVERBOSE_CMD_DONE("GETINIT");
    
    return (status);
}

lsfACI_FUNCTION(lsftpl,StdGetStatus)
{
    ccsCOMPL_STAT status;
    
    lsfVERBOSE_CMD_RECV("STATUS");
    
    sprintf((char *)&msg->body, "state: %s substate: %s",
	    lsfServerGetStateName(lsftplGlobalData),
	    lsfServerGetSubStateName(lsftplGlobalData));

    *replyBuffer = (char *)&msg->body;
    *replyBufferSize = strlen (msg->body) + 1;
    status = SUCCESS;
        
    lsfVERBOSE_CMD_DONE("STATUS");
    
    return (status);
}

/*___oOo___*/

