/************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: lsftplStdHook.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $" 
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-07-25  created
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: lsftplStdHook.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsfStandard.h"

/************************************************************************
*   NAME
*       lsftplStdHook - Hooks to Standard LCC Commands
* 
*   SYNOPSIS
*       ccsCOMPL_STAT lsftplStd<Cmd><Typ>Hook 
*                                      ( IN  lsfSERVER_DATA  *srvData,
*                                        OUT ccsERROR        *error );
*
*   DESCRIPTION
*       These functions are hooks to be invoked as part of the Standard
*       Command behaviour. They extend the minimum behaviour with:
*         lsftplStd<Cmd>ProHook(): invoked prior to std behaviour
*         lsftplStd<Cmd>MidHook(): invoke only if devices declared, between
*                                 sub-software devices and hardware devices
*         lsftplStd<Cmd>EpiHook(): invoked prior to leaving std implementation
*
*       The entry points to these functions are resolved at INIT time.
*
*   FILES
*       None
*
*   ENVIRONMENT
*       None
*
*   SIGNALS
*       None
*
*   DATABASE
*
*   RETURN VALUES
*       SUCCESS if everything ok
*       FAILURE, the error structure is updated with the application
*                dependent error message
*
*   SEE ALSO
*       lsfStandard(3)
*
*------------------------------------------------------------------------
*/

ccsCOMPL_STAT lsftplStdOnlineProHook ( IN  lsfSERVER_DATA *srvData,
				      OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdOnlineProHook: do prologue statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdOnlineMidHook ( IN  lsfSERVER_DATA *srvData,
				      OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdOnlineMidHook: do intermediate statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdOnlineEpiHook ( IN  lsfSERVER_DATA *srvData,
				      OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdOnlineEpiHook: do epilogue statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdStandbyProHook ( IN  lsfSERVER_DATA *srvData,
				       OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdStandbyProHook: do prologue statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdStandbyEpiHook ( IN  lsfSERVER_DATA *srvData,
				       OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdStandbyEpiHook: do epilogue statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdStopMidHook ( IN  lsfSERVER_DATA *srvData,
				    OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdStopMidHook: do intermediate statements ... \n");
    return SUCCESS;
}

ccsCOMPL_STAT lsftplStdExitEpiHook ( IN  lsfSERVER_DATA *srvData,
				    OUT ccsERROR       *error )
{ 
    lsftplPrintf("lsftplStdExitEpiHook: do epilogue statements ... ");

    lsftplPrintf("done\n");
    return SUCCESS;
}

/*************************************************************************/
/* ___oOo___ */
