/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: lsftplTaskDev.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2000-04-05  created
*/

/************************************************************************
*   NAME
*     lsftplTaskDev - Handling of task devices
*
*   SYNOPSIS
*     #include "lsftplTaskDev.h"
*   DESCRIPTION
*     This module .... 
*   METHODS
*   PUBLIC
*     
*   DATABASE
*     lsftpl:control:taskdev
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: lsftplTaskDev.c,v 1.15 2003/09/10 12:41:54 vltsccm Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "lsftplPrivate.h"
#include "lsftplTaskDev.h"

int  lsftplMonitorErr = 0;
int  lsftplMonitorPrt = 0;
static char *lsftplMonitorAttrNames[] = 
{ 
    "data.dataValues", 
    "control:taskdev:monitor.position", 
    NULL 
};

ccsCOMPL_STAT lsftplMonitor ( IN  lsfTASKDEV_DATA *taskDevData,
			    IN  const char       *taskDevName,
			    OUT ccsERROR         *error )
{
    int               i=0,n=taskDevData->numTaskDevs;
    lsfTASKDEV_ENTRY *t;
    vltDOUBLE         val[10],pos;
    char             *at;

    while (strcmp(taskDevName,taskDevData->taskDevNames[i]) && i<n) i++;
    t = taskDevData->taskDevTable[i];
    at = (char *)(taskDevData->userData);

    if (lsftplMonitorErr)
	{
        lsftplERR_ADD(lsftplERR_INTERNAL,error,"Error on purpose");
	return FAILURE;
	}

    if (caiReadAttribute(at,0,val,error) == FAILURE)
	return FAILURE;
    if (caiReadAttribute(at,1,&pos,error) == FAILURE)
	return FAILURE;

    val[0]  = (double)((int)(&val[0]));
    val[1]  = (double)tickGet();
    val[2] += 1.;
    val[3]  = (double)t->state;
    pos    += 0.1;

    if (caiWriteAttribute(at,0,val,error) == FAILURE)
	return FAILURE;
    if (caiWriteAttribute(at,1,&pos,error) == FAILURE)
	return FAILURE;

    if (lsftplMonitorPrt) 
	printf("lsftplMonitor: (task=%s) tick=%d - val=%g %g %g - pos=%g\n",
	       taskDevName,(int)tickGet(),val[0],val[1],val[2],pos);
    return SUCCESS;
}

ccsCOMPL_STAT lsftplMonitorProHook ( IN  lsfTASKDEV_DATA *taskDevData,
				   IN  const char      *taskDevName,
				   OUT ccsERROR        *error )
{
     dbSYMADDRESS dbRoot;

    if (taskDevData->userData == NULL)
	{
	sprintf(dbRoot,"<alias>%s",lsftplMODULE_NAME); 

	if (caiGetAttributes((char **)&(taskDevData->userData),dbRoot,
			     lsftplMonitorAttrNames,error) == FAILURE)
	    return FAILURE;
	}
    
    if (lsftplMonitorPrt) 
	printf("lsftplMonitorProHook: (task=%s) tick=%d\n",
	       taskDevName,(int)tickGet());
    return SUCCESS;
}

ccsCOMPL_STAT lsftplMonitorEpiHook ( IN  lsfTASKDEV_DATA *taskDevData,
				   IN  const char      *taskDevName,
				   OUT ccsERROR        *error )
{
    if (lsftplMonitorPrt) 
	printf("lsftplMonitorEpiHook: (task=%s) tick=%d\n",
	       taskDevName,(int)tickGet());
    caiFreeAttrTable((char *)taskDevData->userData);
    taskDevData->userData=NULL;

    return SUCCESS;
}

ccsCOMPL_STAT lsftplTaskDevTaskCmd (  IN  const char      *param,
				   OUT char            *string,
				   OUT ccsERROR        *error )
{
  strcpy(string,"TaskDevTaskCmd function");
  return SUCCESS;
}

/*___oOo___*/
