#*****************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: lsftplTestSupport.tcl,v 1.15 2003/09/10 12:41:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mnastvog  30/08/95  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
#       contains procs needed by pmtint tests
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#
# Function to print messages with a time stamp compatible with 
# eccsTestDriver and standard ccs log time stamps
#
set lastClock 0
set lastUsec  0
set lastTestNumber 0
# Switches off display of X-Window
wm withdraw .

# log a string preceded with a time script
#
proc printLog { str } {
    global lastClock lastUsec 

    set clock [getclock]
    set time [fmtclock $clock "%Y-%m-%d  %H:%M:%S"]

    if { $lastClock == $clock } {
	incr lastUsec 1
    } else {
	set lastUsec 0
    }
    set lastClock $clock
    set nusec [format "%6.6d" $lastUsec]
    puts "$time.$nusec $str"
}

# send a command to an application
#
proc testCommand { title to cmd args } {
    global lcu_ccsEnvName currentTest currentState
    global lastReply lastTestNumber
    printLog "-------------$title"
    printLog "$lastTestNumber-Command to $to: $cmd $args"
    if {[catch "seq_msgSendCommand $lcu_ccsEnvName $to $cmd $args" cmdId]} {
	printLog "  error sending the command"
        set currentTest "----ERROR----"
    } else {
        set currentTest "$cmd"
	if {[catch "seq_msgRecvReply $cmdId errNr -180000  " buf  ]} {
	    printLog "  $lastTestNumber-error or timeout receiving the reply"
	} else {
	    printLog "  $lastTestNumber-reply: $buf"
            set lastReply $buf
	}
    }
    incr lastTestNumber
}
# send multiple commands to (possibly different) applications
# all commands are sent in a row, then the procedure waits for replies
# For every single reply a timeout of 30 seconds is allowed, so be aware that
# if all commands time out the collective timeout will be commands * 30 s
# If the first character of the destination is a @ then the command will
# be interpreted as a database write.  
#
# CAVE: RTAP crashes if more msgs are outstanding than the msg queue can hold.
#       The sequencer does not hold off these excess msgs, instead it hangs!
#
proc testMultiCommand  { title args } {
    global lcu_ccsEnvName currentTest currentState lastTestNumber
    set counter 0
    printLog "-------------$title"
    foreach i $args {
        if {[string index  [lindex $i 0] 0] == "@" } {
# special case, write to database
            set attr [lindex $i 0]
            eval set attr2 $attr
            set attrInfo [seq_dbGetAttrInfo $attr2]
  	    printLog "db write to $attr2: [lindex $i 1]"
# wait a little to have previous cmds executed
            after 1000
            seq_dbWriteSymbolic $attr2  $attrInfo [lindex $i 1]
        } else {            
            if {[catch "seq_msgSendCommand $lcu_ccsEnvName [lindex $i 0] [lindex $i 1] [lrange $i 2 end] " cmdId$counter]} {
  	        printLog "  error sending the command"
                set currentTest "----ERROR----"
	    } else {
                set id cmdId$counter
                printLog "$lastTestNumber-$id:  [lindex $i 0] [lindex $i 1] [lrange $i 2 end]"
                set currentTest "$counter [lindex $i 1]"
                incr counter
            }
        }
    }
    for {set index 0} { $index < $counter } { incr index } {
        set id cmdId$index 
	set lr 0 
	set errNr 0
	while {$lr != 1 && $errNr == 0} {
	    if {[catch "seq_msgRecvReply $$id errNr lr -180000" buf]} {
		printLog " $lastTestNumber-$index: error receiving the reply"
	    } else {
		printLog " $lastTestNumber-$index reply: $buf"
	    }
	}
    }
    incr lastTestNumber
}
# read a db value and print it on the log
#
proc testDbValueWS { dbAddr } {
    global ws_ccsEnvName
    set value [seq_dbReadSymbolic @$ws_ccsEnvName:$dbAddr]
    printLog "WS DB Value for $dbAddr: $value"
}

proc testDbValueLCU { dbAddr } {
    global lcu_ccsEnvName
    set value [seq_dbReadSymbolic @$lcu_ccsEnvName:$dbAddr]
    printLog "LCU DB Value for $dbAddr: $value"
}

proc testDbAttrLCU { baseAddr attrName refValue } {
    global lcu_ccsEnvName
    set errNum 0
    set pointList [list "signal" "signal:AI1" "signal:DI2" "motor" "motor:M1" "motor:M2" "serial" "serial:tyCo1" "serial:com1" "serial:com2" "softdev" "taskdev" "taskdev:bgTask" ]
    
    foreach p $pointList {
	set dbAttr "@$lcu_ccsEnvName\<alias\>$baseAddr:control:$p.$attrName"
	set value [seq_dbReadSymbolic $dbAttr]
	if { $value != $refValue } {
	    printLog "LCU DB Value for $dbAttr: $value - Expected $refValue"
	    incr errNum 1
	}
    }
    if {$errNum == 0} {
	printLog "LCU DB Values for $attrName = $refValue"
    }
}
