!
! "@(#) $Id: Confact-fits.prg,v 2.83 2005/09/30 08:26:43 vltsccm Exp $"
!
! Confact.prg
! Program to recognize the Conversion-factor and Noise of a CCD
! if two equal flats and one or two short darks are given
!+ 
! resulting in Conversion-factor +/- error [e-/ADU] and
! Readoutnoise in [e-]
!+
! S. Deiries 08.02.1994 Version 0
!+
! Parameter:
! (from CCDtype.prg)
!+
! npix = number of pixel (x)
! nline = number of lines (y)
! xstart = number of first pixel
! ystart = number of first line
! detector = Manufacturer and Serial number of CCD
! exposure = exposure information of frame
! xstep = binning in x
! ystep = binning in y
! pix = pixel size in micron
! xprescan = number of prescan pixels
! xspix = number of sensitive pixels
! ysline = number of sensitive lines
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! (from this program:)
!+
! ff1 = first flat-field
! ff2 = second flat-field
! dk1 = first short dark
! dk2 = second short dark ( or same as first )
! diff1 = difference of the two flats
! diff2 = difference of the two darks
! xfirst = first pixel used for standard square
! xlast = last pixel used for standard square
! yfirst = first line used for standard square
! ylast = last line used for standard square
! size = edge-size of standard square
! n = counter of standard squares
! xone = pixel begin one each standard square
! xtwo = pixel end one each standard square
! yone = line begin one each standard square
! ytwo = line end one each standard square
! twodarks = flag whether one or two different darks exists
! k_fact = table where results are stored
! mean0 = Mean value of darks
! mean1 = Mean value of first image
! mean2 = Mean value of second image
! var0 = Variance of darks
! var3 = Variance of difference of flats
! mean = Mean mean value of flats
! var = resulting variance
! ctvg = resulting Conversion-factor firstly of one square
! crms = RMS error of conversion-factor
! noise = resulting (Readout)-noise firstly of one square
! srms = RMS error of resulting Readout-noise
! std = standard deviation of darks
! d = decrement
! External procedures: CCDtype, sample
!+
! Program:
!+
!goto Graphics
define/par p1 ? ima "Enter name of first flat field frame:"
write/key ff1/c/1/20 {p1}
!
define/par p2 ? ima "Enter name of second flat field frame:"
write/key ff2/c/1/20 {p2}
!
define/par p3 ? ima "Enter name of first short dark frame:"
write/key dk1/c/1/20 {p3}
!
define/par p4 ? ima "Enter name of second short dark frame or SAME:"
write/key dk2/c/1/20 {p4}
define/local twodarks/i/1/1 1
if dk1 .eq. dk2 twodarks = 0
if twodarks .eq. 0 write/out You are using really only one dark !!!!!!
!
! Computing difference images:
!
write/out Converting FITS files...
indisk/fits {ff1}.fits {ff1}.bdf
indisk/fits {ff2}.fits {ff2}.bdf
indisk/fits {dk1}.fits {dk1}.bdf

if twodarks .eq. 1 then
	indisk/fits {dk2}.fits {dk2}.bdf
endif

@@ CCDtype {ff1}
write/out Computing Differences...
compute/ima diff1 = ( {ff2} - {ff1} )
if twodarks .eq. 1 then
	compute/ima diff2 = ( {dk2} - {dk1} )
else
!	compute/ima diff2 = ( {dk1} )
endif
!
define/local xfirst/i/1/1 0
define/local xlast/i/1/1 0
define/local yfirst/i/1/1 0
define/local ylast/i/1/1 0
define/local size/i/1/1 50
define/local n/i/1/1 0
define/local xone/i/1/1 0
define/local xtwo/i/1/1 0
define/local yone/i/1/1 0
define/local ytwo/i/1/1 0
define/local mean0/r/1/1 0.0
define/local mean1/r/1/1 0.0
define/local mean2/r/1/1 0.0
define/local var0/r/1/1 0.0
define/local var3/r/1/1 0.0
define/local mean/r/1/1 0.0
define/local var/r/1/1 0.0
define/local ctvg/r/1/1 0.0
define/local noise/r/1/1 0.0
define/local std/r/1/1 0.0
!
! Calculation of usable area
!
xfirst = xstart + xprescan + 10
yfirst = ystart + 10
xlast  = xfirst + xspix - 1 - 20 - size
ylast  = yfirst + ysline - 1 - 20 - size
!
! Calculation of number of squares:
!
n = ( xlast - xfirst + size ) / size * ( ylast - yfirst + size ) / size
define/local d/i/1/1 {n}
!
! Creating of necessary table for results:
!
write/out Waiting for table creation
crea/tab k_fact 2 {n} null
crea/col k_fact :K_FACT "e-/ADU" f6.3
crea/col k_fact :NOISE "e-" f6.3
write/out {n} results are expected !
!
! Begin of LOOP:
!
n = 0
set/format f5.3
do yone = {yfirst} {ylast} {size}
	do xone = {xfirst} {xlast} {size}
		n = n + 1    
		xtwo = xone + size
		ytwo = yone + size
		if twodarks .eq. 0 then    	    
			@@ sample {dk1} {xone} {yone} {xtwo} {ytwo}
			Mean0 = {q1}
			std = {q2}
			var0 = std * std
		else    	    
			@@ sample {dk1} {xone} {yone} {xtwo} {ytwo}
			Mean1 = {q1}    	    
			@@ sample {dk2} {xone} {yone} {xtwo} {ytwo}
			Mean2 = {q1} 
			Mean0 = ( Mean1 + Mean2 ) / 2    	    
			@@ sample diff2 {xone} {yone} {xtwo} {ytwo}
			var0 = ( {q2} * {q2} ) / 2    	    
			std = M$SQRT({var0})
		endif
		@@ sample {ff1} {xone} {yone} {xtwo} {ytwo}
		Mean1 = {q1}    	    
		@@ sample {ff2} {xone} {yone} {xtwo} {ytwo}
		Mean2 = {q1}
		Mean = ( ( Mean2 + Mean1 ) / 2 ) - Mean0
                @@ sample diff1 {xone} {yone} {xtwo} {ytwo}
		var3 = {q2} * {q2}
		var = var3 / 2 - var0 / 2
		ctvg = mean / var
		noise = ctvg * std
		write/out {d} Conv.factor = {ctvg} and Readoutnoise = {noise}
		d = d - 1
		k_fact,:K_FACT,@{n} = {ctvg}
		k_fact,:NOISE,@{n} = {noise}
	enddo
enddo
!
stat/tab k_fact :K_FACT
ctvg = {outputr(3)}
define/local crms/r/1/1 {outputr(4)} 
!
stat/tab k_fact :NOISE
noise = {outputr(3)}
define/local nrms/r/1/1 {outputr(4)}
!
write/out Detector Type = {ccd_type}
write/out Serial Number = {detector}
write/out "  "
!
set/format f4.2
write/out Conversion-factor:  {ctvg} +/- {crms} e-/ADU
write/out Read-out-noise:     {noise} +/- {nrms} e-
!
Graphics:
crea/graph 
set/graph xaxis=1,9,1.0,0.2 binmode=on colour=2
plot/histo k_fact :K_FACT p4=0.01,1,9
set/grap colour=3
over/histo k_fact :NOISE p4=0.01,1,9
!
! end







