!
! "@(#) $Id: Confact_center.prg,v 2.83 2005/09/30 08:26:43 vltsccm Exp $"
!
! Confact_center.prg
!
! Program to recognize the Conversion-factor and Noise of a CCD
! if two equal flats and one or two short darks are given
!+ 
! resulting in Conversion-factor +/- error [e-/ADU] and
! Readoutnoise in [e-]
!+
! S. Deiries 19.03.1998 Version 8+ 5c + 6  for LCU und 4 Quadranten; Center 512
!+
! Parameter:
! (from CCDtype.prg)
!+
! npix = number of pixel (x)
! nline = number of lines (y)
! xstart = number of first pixel
! ystart = number of first line
! detector = Manufacturer and Serial number of CCD
! exposure = exposure information of frame
! xstep = binning in x
! ystep = binning in y
! pix = pixel size in micron
! xprescan = number of prescan pixels
! ysprescan = number of prescan lines
! xspix = number of sensitive pixels
! ysline = number of sensitive lines
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! (from this program:)
!+
! ff1 = first flat-field
! ff2 = second flat-field
! dk1 = first short dark
! dk2 = second short dark ( or same as first )
! diff1 = difference of the two flats
! diff2 = difference of the two darks
! xfirst = first pixel used for standard square
! xlast = last pixel used for standard square
! yfirst = first line used for standard square
! ylast = last line used for standard square
! size = edge-size of standard square
! n = counter of standard squares
! xone = pixel begin one each standard square
! xtwo = pixel end one each standard square
! yone = line begin one each standard square
! ytwo = line end one each standard square
! twodarks = flag whether one or two different darks exists
! k_fact = table where results are stored
! mean0 = Mean value of darks
! mean1 = Mean value of first image
! mean2 = Mean value of second image
! var0 = Variance of darks
! var3 = Variance of difference of flats
! mean = Mean mean value of flats
! var = resulting variance
! ctvg = resulting Conversion-factor firstly of one square
! crms = RMS error of conversion-factor
! noise = resulting (Readout)-noise firstly of one square
! srms = RMS error of resulting Readout-noise
! std = standard deviation of darks
! d = decrement
! e_noise = expected noise level in e-
! up_lim = upper limit of accepted noise level calculation
! ow_lim = lower limit of accepted noise level calculation
! xscale = value of automatic scaled of x-axis of Noise/Gain plot
! result = string whether the calculated value is used or not
! filetype = type of input files (Fits or Midas)
! conv(i) = Array of resulting Conversion-factor values of square(i)
! rnoise(i) = Array of resulting (Readout)-noise values of square(i)
! 
! External procedures: cd CCDtype, sample
!+
! Program:
!+
!goto Graphics
define/local result/c/1/20 "    "
define/par p1 ? ima "Enter name of first flat field frame:"
write/key ff1/c/1/20 {p1}
!
define/par p2 ? ima "Enter name of second flat field frame:"
write/key ff2/c/1/20 {p2}
!
define/par p3 ? ima "Enter name of first short dark frame:"
write/key dk1/c/1/20 {p3}
!
define/par p4 ? ima "Enter name of second short dark frame or SAME:"
write/key dk2/c/1/20 {p4}
!
!f define/par p5 ? ima "Give Fits or Midas -file-type?  (Midas,Fits)" 
!f write/key filetype/c/1/20 {p5}
@@ test_fits {p1}
@@ test_fits {p2}
@@ test_fits {p3}
@@ test_fits {p4}
!
!define/par p6 ? num "Enter expected Readout-Noise level [in e-]:" 0,25
!write/key e_noise/r/1/1 {p6}
!write/key e_noise/r/1/1 {p6}
define/local up_lim/r/1/1 1000
define/local low_lim/r/1/1 1
!
!up_lim = 1.5 * {e_noise} 
!low_lim = 0.75 * {e_noise}
write/out The expected noise level is between {low_lim} and {up_lim} e-
!
define/local twodarks/i/1/1 1
if dk1 .eq. dk2 twodarks = 0
if twodarks .eq. 0 write/out You are using really only one dark !!!!!!
!
!f if filetype .ne. "Midas" then
!f	write/out Converting FITS files...
!f	indisk/fits {ff1} {ff1}.bdf
!f	indisk/fits {ff2} {ff2}.bdf
!f	indisk/fits {dk1} {dk1}.bdf
!f
!f	if twodarks .eq. 1 then
!f        	indisk/fits {dk2} {dk2}.bdf
!f	endif
!f endif
!
! Computing difference images:
!
@@ CCDtype_confact {ff1}
!
write/out Computing Differences
compute/ima diff1 = ( {ff2} - {ff1} )
if twodarks .eq. 1 then
	compute/ima diff2 = ( {dk2} - {dk1} )
else
!	compute/ima diff2 = ( {dk1} )
endif
!
define/local xfirst/i/1/1 0
define/local xlast/i/1/1 0
define/local yfirst/i/1/1 0
define/local ylast/i/1/1 0
define/local size/i/1/1 50
define/local n/i/1/1 0
define/local xone/i/1/1 0
define/local xtwo/i/1/1 0
define/local yone/i/1/1 0
define/local ytwo/i/1/1 0
define/local mean0/r/1/1 0.0
define/local mean1/r/1/1 0.0
define/local mean2/r/1/1 0.0
define/local var0/r/1/1 0.0
define/local var3/r/1/1 0.0
define/local mean/r/1/1 0.0
define/local var/r/1/1 0.0
define/local ctvg/r/1/1 0.0
define/local noise/r/1/1 0.0
define/local std/r/1/1 0.0
!
! Calculation of usable area
!
xfirst = xstart + xprescan + ( xspix / 2 ) - 256
yfirst = ystart + yprescan + ( ysline / 2 ) - 256
xlast  = xfirst + 512 - size
ylast  = yfirst + 512 - size
!
! Calculation of number of squares:
!
n = ( xlast - xfirst + size ) / size * ( ylast - yfirst + size ) / size
define/local d/i/1/1 {n}
!
! Creating of necessary table for results:
!
write/out Waiting for table creation
crea/tab k_fact 2 {n} null
crea/col k_fact :K_FACT "e-/ADU" f6.3
crea/col k_fact :NOISE "e-" f6.3
write/out {n} results are expected !
!
!write/key conv/r/1/{n} 0 ALL
!write/key rnoise/r/1/{n} 0 ALL
!
! Begin of LOOP:
!
n = 0
set/format f5.3
do yone = {yfirst} {ylast} {size}
	do xone = {xfirst} {xlast} {size}
		n = n + 1    
		xtwo = xone + size
		ytwo = yone + size
		if twodarks .eq. 0 then    	    
			@@ sample {dk1} {xone} {yone} {xtwo} {ytwo}
			Mean0 = {q1}
			std = {q2}
			var0 = std * std
		else    	    
			@@ sample {dk1} {xone} {yone} {xtwo} {ytwo}
                        write/out {xone} {yone} {xtwo} {ytwo}
			Mean1 = {q1}    	    
			@@ sample {dk2} {xone} {yone} {xtwo} {ytwo}
			Mean2 = {q1} 
			Mean0 = ( Mean1 + Mean2 ) / 2    	    
			@@ sample diff2 {xone} {yone} {xtwo} {ytwo}
			var0 = ( {q2} * {q2} ) / 2    	    
			std = M$SQRT({var0})
		endif
		@@ sample {ff1} {xone} {yone} {xtwo} {ytwo}
		Mean1 = {q1}    	    
		@@ sample {ff2} {xone} {yone} {xtwo} {ytwo}
		Mean2 = {q1}
		Mean = ( ( Mean2 + Mean1 ) / 2 ) - Mean0
                @@ sample diff1 {xone} {yone} {xtwo} {ytwo}
		var3 = {q2} * {q2}
		var = var3 / 2 - var0 / 2
		ctvg = mean / var
		noise = ctvg * std
		if noise .le. up_lim then
			if noise .gt. low_lim then
				k_fact,:K_FACT,@{n} = {ctvg}
				k_fact,:NOISE,@{n} = {noise}
!				conv({n}) = {ctvg}
!				rnoise({n}) = {noise}
				result = "used"
			else
                                result = "not used"
			endif
		else
			result = "not used"
		endif
write/out {d} Conv.factor = {ctvg} and Readoutnoise = {noise}  {result}
		d = d - 1
	enddo
enddo
!
!write/out Writing into table
!
Statistic:
!
!define/local num/i/1/1 0
!do num = 1 {n}
!	if conv({num}) .NE. 0 then
!		k_fact,:K_FACT,@{num} = {conv({num})}
!		k_fact,:NOISE,@{num} = {rnoise({num})}
!	endif
!enddo
!
!compute/histo noise = k_fact :NOISE 0.1 0 20
!compute/histo kfact = k_fact :K_FACT 0.05 0 10
!sort/tab k_fact :NOISE
!
define/local itera/i/1/1 1
define/local upperv/r/1/1 0.0
define/local lowerv/r/1/1 0.0
define/local nrms/r/1/1 0.0
define/local crms/r/1/1 0.0
define/local oldnoise/r/1/1 0.0
!
stat/tab k_fact :NOISE
noise = {outputr(3)}
nrms = {outputr(4)}
!
do itera = 1 10

	lowerv = {noise} - 3 * {nrms}
	upperv = {noise} + 3 * {nrms}
	select/table k_fact {lowerv}.LE.:NOISE.AND.:NOISE.LE.{upperv}
!
	oldnoise = {noise}
	stat/tab k_fact :NOISE
	noise = {outputr(3)}
	nrms = {outputr(4)}
! 
	stat/tab k_fact :K_FACT
	ctvg = {outputr(3)}
	crms = {outputr(4)} 
!
	set/format f4.2
	write/out {itera}.Iteration gives
	write/out Conversion-factor:  {ctvg} +/- {crms} e-/ADU
	write/out Read-out-noise:     {noise} +/- {nrms} e-
!
        if oldnoise .eq. noise goto Endloop
!
enddo
!
Endloop:
!
write/out "  "
write/out Detector Type = {ccd_type}
write/out Serial Number = {detector}
!
open/file confact.result write OUTPUTI
write/file {OUTPUTI(1)} {ctvg} {crms} {noise} {nrms} {itera}
close/file {OUTPUTI(1)}
!
Graphics:
crea/graph 
define/local xscale/i/1/1 9
xscale = {noise} * 2
set/graph xaxis=0,{xscale},2.0,0.5 binmode=on colour=2
plot/histo k_fact :K_FACT p4=0.01,0,{xscale}
set/grap colour=3
over/histo k_fact :NOISE p4=0.03,0,{xscale}
label/grap "K-Factor = {ctvg} +/- {crms} e-/ADU" 10,90,mm 0 1 1
label/grap "Noise = {noise} +/- {nrms} e-" 60,75,mm 0 1 1
label/grap "CCD = {ccd_type} {detector} " 30,97,mm 0 1 1
write/out Plotting Result on ps0to0 Printer !!!

!
copy/grap ps0to0
return {ctvg} {crms} {noise} 
!
! end







