!
! "@(#) $Id: analyse_standart.prg,v 2.83 2005/09/30 08:26:46 vltsccm Exp $"
!
! analyse.prg
!+
! Program to investigate the CCD-blemishes stored on table: defects.tbl
!+ 
! resulting in number, coordinates and intensities of hot and cold CCD-defects
!+
! S. Deiries 14.09.1995 Version 4
!+
! Parameter:
!+
! exposure = exposure information of frame
! xstep = binning in x
! ystep = binning in y
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! number = Number of pixel defects of image
! ndefects = Number of all defects
! nhotpix = Number of hot pixel
! ncoldpix = Number of cold pixel
! nhotclu = Number of hot clusters
! ncoldclu = Number of cold clusters
! nhotcol = Number of hot columns
! ncoldcol = Number of cold columns
! ntrap = Number of traps
! xco,yco = Coordinates of defect pixels
! pixval = Values of defect pixels
! locval = Values of defect pixels of one trap, cluster or column
! length = Length of on defect
! position = Position of investigated defect pixel
! maxdev = Maximum deviation of defect pixel, cluster, column defect in %
! mdev = Mean deviation of defect pixel, cluster, column defect in %
! mdevstd = Sigma of mean deviation of defect pixel, cluster,column defect in %
! koeff = Variation of cluster,column or trap defect
! defect = Name of defect
! content = Content of ADUs of a trap
!+
! Program:
!+
set/midas_system keywords=1400,60000
define/par p1 {q1} num "Enter undisturbed mean value of image:  "
define/par p2 {q2} num "Enter offset value of image:  "
define/par p3 ? c "Enter offset letter of table-set:  "
!
write/out Used undisturbed value {p1} and offset {p2} and letter {p3}
!
define/local means/r/1/1 {p1}
define/local offsets/r/1/1 {p2}
define/local nr/c/1/1 "{p3}"
define/local hilf/r/1/1 0.0
define/local weak/i/1/1 0
!
hilf = 100.0 - 100.0 / xstep
offsets = offsets + ( ( ( means - offsets ) / 100.0 ) * hilf )
!
show/tab defects
define/local number/i/1/1 {outputi(2)} 
!
write/out Creating tables .....
!
define/local nhotpix/i/1/1 0
create/tab hotpix{nr} 3 300 null
create/col hotpix{nr} :X_COORD "pix" i4 I*4
create/col hotpix{nr} :Y_COORD "line" i4 I*4
create/col hotpix{nr} :VALUE "%" f7.1
!
define/local ncoldpix/i/1/1 0
create/tab coldpix{nr} 3 300 null
create/col coldpix{nr} :X_COORD "pix" i4 I*4
create/col coldpix{nr} :Y_COORD "line" i4 I*4
create/col coldpix{nr} :VALUE "%" f7.1
!
define/local nhotclu/i/1/1 0
create/tab hotclu{nr} 6 200 null
create/col hotclu{nr} :X_COORD "pix" i4 I*4
create/col hotclu{nr} :Y_COORD "line" i4 I*4
create/col hotclu{nr} :MAXDEV "%" f7.1
create/col hotclu{nr} :MEANDEV "%" f7.1
create/col hotclu{nr} :STDDEV "%" f5.1
create/col hotclu{nr} :NPIX "pixel" i4 I*4
!
define/local nhotcol/i/1/1 0
create/tab hotcol{nr} 6 100 null
create/col hotcol{nr} :X_COORD "pix" i4 I*4
create/col hotcol{nr} :Y_COORD "line" i4 I*4
create/col hotcol{nr} :MAXDEV "%" f7.1
create/col hotcol{nr} :MEANDEV "%" f7.1
create/col hotcol{nr} :STDDEV "%" f5.1
create/col hotcol{nr} :NPIX "pixel" i4 I*4
!
define/local ncoldclu/i/1/1 0
create/tab coldclu{nr} 6 1500 null
create/col coldclu{nr} :X_COORD "pix" i4 I*4
create/col coldclu{nr} :Y_COORD "line" i4 I*4
create/col coldclu{nr} :MAXDEV "%" f7.1
create/col coldclu{nr} :MEANDEV "%" f7.1
create/col coldclu{nr} :STDDEV "%" f5.1
create/col coldclu{nr} :NPIX "pixel" i4 I*4
!
define/local ncoldcol/i/1/1 0
create/tab coldcol{nr} 6 100 null
create/col coldcol{nr} :X_COORD "pix" i4 I*4
create/col coldcol{nr} :Y_COORD "line" i4 I*4
create/col coldcol{nr} :MAXDEV "%" f7.1
create/col coldcol{nr} :MEANDEV "%" f7.1
create/col coldcol{nr} :STDDEV "%" f5.1
create/col coldcol{nr} :NPIX "pixel" i4 I*4
!
define/local ntrap/i/1/1 0
create/tab trap{nr} 7 1500 null
create/col trap{nr} :X_COORD "pix" i4 I*4
create/col trap{nr} :Y_COORD "line" i4 I*4
create/col trap{nr} :CONTENT "ADU" f8.1
create/col trap{nr} :NPIX "pixel" i4 I*4
create/col trap{nr} :MAXDEV "%" f7.1
create/col trap{nr} :MEANDEV "%" f7.1
create/col trap{nr} :STDDEV "%" f5.1
!
define/local m/i/1/1 1
define/local ndefects/i/1/1 1
create/tab result{nr} 8 3000 null
create/col result{nr} :TYPE " " a11 C*11
create/col result{nr} :X_COORD "pixel" i4 I*4
create/col result{nr} :Y_COORD "line" i4 I*4
create/col result{nr} :MEANLEVEL "%" f7.1
create/col result{nr} :RMS "%" f5.1
create/col result{nr} :MAXLEVEL "%" f7.1
create/col result{nr} :NPIX "pixel" i4 I*4
create/col result{nr} :CONTENT "ADU" f8.0
!
write/out Creating long arrays and setting it to zero....
!
define/local xco/i/1/16384 0 ALL
define/local yco/i/1/16384 0 ALL
define/local pixval/r/1/16384 0.0 ALL
define/local locval/r/1/2048 0.0 ALL
define/local n/i/1/1 0
!
write/out Reading the table defects.tbl into an array....
!
do n = 1 {number}
	xco({n}) = {defects,:X_COORD,@{n}}
	yco({n}) = {defects,:Y_COORD,@{n}}
	pixval({n}) = {defects,:VALUE,@{n}}
enddo
!
define/local length/i/1/1 0
define/local position/i/1/1 0
define/local maxdev/r/1/1 0.0
define/local mdev/r/1/1 0.0
define/local mdevstd/r/1/1 0.0
define/local max/i/1/1 0
define/local maxvalue/r/1/1 0.0
define/local koeff/r/1/1 0.0
define/local defect/c/1/11 "col        "
define/local content/r/1/1 0.0
define/local a/r/1/1 0.0
define/local b/r/1/1 0.0
define/local i/i/1/1 0
define/local j/i/1/1 0
define/local k/i/1/1 0
!
!
write/out Starting the Search-Loop...
!
LOOP:
!
do n = 1 2048
	locval({n}) = 0.0
enddo
maxdev = 0.0
mdev = 0.0
mdevstd = 0.0
content = 0.0
!
length = 1
position = m
!
DEFCOL:
locval({length}) = pixval({position})
i = position + 1 
if i .le. {number} then
	k = position + 1 
	i = yco({k})
	j = yco({position}) + 1 * ystep
	if i .eq. {j} then
		length = length + 1
		position = position + 1
		goto DEFCOL
	endif
endif
!
max = 1
maxvalue = 0.0
do n = 1 {length}
	b = locval({n}) - means 
	a = M$ABS(b)
	if a .gt. {maxvalue} then
		maxvalue = a
		max = n
	endif
	mdev = mdev + locval({n})
enddo
!
mdev = mdev / length
!
if length .eq. 1 then
   mdevstd = 0.0
else
   do n = 1 {length}
	mdevstd = mdevstd + ( ( locval({n}) - mdev ) * ( locval({n}) - mdev ) )
   enddo 
!
   mdevstd = mdevstd / ( length - 1 )
   mdevstd = M$SQRT(mdevstd)
endif
!
mdev = 100.0 * ( mdev - offsets ) / ( means - offsets )
mdevstd = 100.0 * mdevstd / ( means - offsets )
maxdev = 100.0 * ( locval({max}) - offsets ) / ( means - offsets )
!
if mdev .gt. 100.0 then
	HOT:				! Mean value > 100 % -> hot
	 if length .eq. 1 then
		HOTPIX:
                 if maxdev .gt. 400.0 then
		    defect = "Hot__Pixel "
		    nhotpix = nhotpix + 1
		    hotpix{nr},:X_COORD,@{nhotpix} = {xco({m})}
		    hotpix{nr},:Y_COORD,@{nhotpix} = {yco({m})}
		    hotpix{nr},:VALUE,@{nhotpix} = {maxdev}
                 else
                    defect = "Weak_HotPix"
                    weak = 1
                 endif 
	 elseif length .gt. 10 then
		HOTCOL:
		 defect = "Hot_Column "
		 nhotcol = nhotcol + 1
		 hotcol{nr},:X_COORD,@{nhotcol} = {xco({m})}
		 hotcol{nr},:Y_COORD,@{nhotcol} = {yco({m})}
		 hotcol{nr},:MAXDEV,@{nhotcol} = {maxdev}
		 hotcol{nr},:MEANDEV,@{nhotcol} = {mdev}
		 hotcol{nr},:STDDEV,@{nhotcol} = {mdevstd}
		 hotcol{nr},:NPIX,@{nhotcol} = {length}
	 else
		HOTCLUST:
		 defect = "Hot_Cluster"
		 nhotclu = nhotclu + 1
		 hotclu{nr},:X_COORD,@{nhotclu} = {xco({m})}
		 hotclu{nr},:Y_COORD,@{nhotclu} = {yco({m})}
		 hotclu{nr},:MAXDEV,@{nhotclu} = {maxdev}
		 hotclu{nr},:MEANDEV,@{nhotclu} = {mdev}
		 hotclu{nr},:STDDEV,@{nhotclu} = {mdevstd}
		 hotclu{nr},:NPIX,@{nhotclu} = {length}
	 endif
!
else
	COLD:
	 if length .eq. 1 then
		COLDPIX:
                 if maxdev .le. 50 then 
		    defect = "Cold_Pixel "
		    ncoldpix = ncoldpix + 1
		    coldpix{nr},:X_COORD,@{ncoldpix} = {xco({m})}
		    coldpix{nr},:Y_COORD,@{ncoldpix} = {yco({m})}
		    coldpix{nr},:VALUE,@{ncoldpix} = {maxdev}
                 else
                    defect = "WeakColdPix"
                    weak = 1
                 endif
	 else
		defect = "col        "
!
		if maxdev .le. 25 then				! 25%
			if mdevstd .gt. 4 defect = "Trap       " ! 4%
		endif
!
		koeff = ( locval({length})-locval(1))/(means - offsets)
		if  koeff .gt. 0.3 defect = "Trap       " ! Variation > 30%
!
		a = locval({length})
		if a .gt. {locval(1)} then
			if locval(2) .gt. {locval(1)} then
				if koeff .gt. 0.1 defect = "Trap       " ! >10%
			endif
		endif
		if length .gt. 50 defect = "Cold_Column"
!
		if defect(1:4) .eq. "Trap" then
			TRAP:
			 do n = 1 {length}
				content = content + ( means - locval({n}) )
			 enddo
			 ntrap = ntrap + 1
		 	 trap{nr},:X_COORD,@{ntrap} = {xco({m})}
		 	 trap{nr},:Y_COORD,@{ntrap} = {yco({m})}
			 trap{nr},:CONTENT,@{ntrap} = {content}
		 	 trap{nr},:NPIX,@{ntrap} = {length}
		 	 trap{nr},:MAXDEV,@{ntrap} = {maxdev}
		 	 trap{nr},:MEANDEV,@{ntrap} = {mdev}
			 trap{nr},:STDDEV,@{ntrap} = {mdevstd}
		else
			if length .gt. 10 then
				COLDCOL:
				 defect = "Cold_Column"
			 	 ncoldcol = ncoldcol + 1
		 	 	 coldcol{nr},:X_COORD,@{ncoldcol} = {xco({m})}
		 	 	 coldcol{nr},:Y_COORD,@{ncoldcol} = {yco({m})}
		 	 	 coldcol{nr},:MAXDEV,@{ncoldcol} = {maxdev}
		 	 	 coldcol{nr},:MEANDEV,@{ncoldcol} = {mdev}
			 	 coldcol{nr},:STDDEV,@{ncoldcol} = {mdevstd}
		 	 	 coldcol{nr},:NPIX,@{ncoldcol} = {length}
			else
				COLDCLUS:
				 defect = "ColdCluster"
			 	 ncoldclu = ncoldclu + 1
		 	 	 coldclu{nr},:X_COORD,@{ncoldclu} = {xco({m})}
		 	 	 coldclu{nr},:Y_COORD,@{ncoldclu} = {yco({m})}
		 		 coldclu{nr},:MAXDEV,@{ncoldclu} = {maxdev}
		 		 coldclu{nr},:MEANDEV,@{ncoldclu} = {mdev}
				 coldclu{nr},:STDDEV,@{ncoldclu} = {mdevstd}
		 		 coldclu{nr},:NPIX,@{ncoldclu} = {length}
			endif
		endif

	 endif
!
endif
!
if weak .eq. 0 then
	result{nr},:TYPE,@{ndefects} = {defect(1:11)}
	result{nr},:X_COORD,@{ndefects} = {xco({m})}
	result{nr},:Y_COORD,@{ndefects} = {yco({m})}
        if length .gt. 1 then
		result{nr},:MEANLEVEL,@{ndefects} = {mdev}
		result{nr},:RMS,@{ndefects} = {mdevstd}
        endif
	result{nr},:MAXLEVEL,@{ndefects} = {maxdev}
	result{nr},:NPIX,@{ndefects} = {length}
        if defect(1:4) .eq. "Trap" then
		result{nr},:CONTENT,@{ndefects} = {content}
        endif
	ndefects = ndefects + 1
else
   weak = 0
endif
set/format f6.1
write/out No.: {m}; {defect(1:11)}; X={xco({m})}; Y={yco({m})}; Mean={mdev}-
     +/- {mdevstd}; Max={maxdev}; 
!
m = m + length
!
if m .le. {number} goto LOOP
!
RESULT:
!
set/format i5
write/out Number of hot defects:
write/out Hot spots: {nhotpix}; Hot Cluster: {nhotclu}; Hot columns: {nhotcol}
write/out  
write/out Number of dark defects:
write/out Dark pixel: {ncoldpix}; Dark Cluster: {ncoldclu};-
 Dark columns: {ncoldcol}
write/out Traps: {ntrap}
write/out Number of all defects:
ndefects = ndefects - 1
write/out {ndefects}
!
open/file cosmetic.result write OUTPUTI
write/file {OUTPUTI(1)} {nhotpix} {nhotclu} {nhotcol} {ncoldpix} {ncoldclu}-
 {ncoldcol} {ntrap} {ndefects}
close/file {OUTPUTI(1)}
!
return {ndefects} {ntrap}
!
! end





