!
! "@(#) $Id: ccdtestb_quick.prg,v 2.83 2005/09/30 08:26:46 vltsccm Exp $"
!
! @(#)ccdtestb_quick.prg	Quick analisys of Bias images FEB98/PS
!.PURPOSE:        Procedure to determine the ccd bias characteristics
!
!                  Determination of the bias level and standard 
!                  deviation. For each input frame in the catalogue 
!                  determine the  mean bias and standard deviation 
!                  after hot pixel correction, box averaging and 
!                  median filtering. The overall mean bias level and 
!                  mean sigma is computed."
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE/PARAMETER P1  ?            C  "Enter input catalogue:"
DEFINE/PARAMETER P2  [@15,@2:>,>] C  "Enter area:"
DEFINE/PARAMETER P3  5,10         N  "Radius med. filtering, and dec. factor:"
!
WRITE/KEY    INPUTI/I/1/2     5,10
WRITE/KEY    RADIUS/I/1/1     {INPUTI(1)}
DEFINE/LOCAL NBOX/I/1/1       {INPUTI(2)}
write/key biasmean/r/1/1 0.0
write/key biassigm/r/1/1 0.0


log/tof
WRITE/OUT "     PROCEDURE TO DETERMINE THE CCD BIAS CHARACTERISTICS"
write/out "     ==================================================="
write/out " "
write/out " "
write/out "  command line: "
write/out " @@ ccdtestb_quick {p1} {p2} {p3}"
write/out " "

! *** check if the catalogue exists
IF M$EXIST("{P1}.cat") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <CCDTESTB_QUICK>: Catalogue {P1} not existing"
   RETURN/EXIT
ENDIF
!
!set/midas output=logonly
read/icat {p1}
set/midas output=no
crea/gra; 
set/graph default
set/midas output=yes
!
!
!
!
write/out " Determination of the bias level and standard deviation"
write/out "------------------------------------------------------------------"
write/out "For each input frame in the catalogue determine the mean"
write/out "BIAS and standard deviation after box averaging"
write/out "and median filtering, and using area {P2}." 
write/out "Finally, the overall mean bias level and sigma are computed."
write/out
define/local mean/r/1/1 0.0
define/local sigm/r/1/1 0.0
define/local meandif/r/1/1 0.0
define/local sigmdif/r/1/1 0.0
define/local meanstd/r/1/1 0.0
define/local sigmstd/r/1/1 0.0
define/local frame/c/1/60 " "
write/key    catal/i/1/1 0
write/key    rebin/c/1/3 "mea"
define/local ifrm/i/1/1 0
create/tab   {p1}_mean 3 10 null
create/col   {p1}_mean :NAME  "   " A20  C*20
create/col   {p1}_mean :MEAN  "ADU" F9.4 R*4
create/col   {p1}_mean :SIGMA "ADU" F9.4 R*4


LOOP2:
   store/frame in_a {p1}.cat 1 FINISH
   ifrm  = ifrm + 1
   frame = in_a
   comp/ima middumma = {frame}
   set/midas output=no
   extr/ima middumma = middumma{p2}
   in_a  = "middumma"
   out_a = "bs_mean"
   out_b = "bs_sigm"
   run STD_EXE:ccdrebin
   filter/median {OUT_A} &a {RADIUS},{RADIUS},0.0 
   filter/median {OUT_B} &b {RADIUS},{RADIUS},0.0 
   statist/ima &a ? ? ? NN
   mean  = outputr(3)
   statist/ima &b ? ? ? NN
   sigm = outputr(3)
   {p1}_mean,:NAME,@{ifrm}  = "{frame}"
   {p1}_mean,:MEAN,@{ifrm}  =  mean
   {p1}_mean,:SIGMA,@{ifrm} =  sigm
   
   set/midas output=yes
   set/format t6.2
   write/out "     Statistics of {frame}:  mean =  {mean};  noise ={sigm}"
   goto LOOP2
FINISH:
set/midas output=no
ifrm = ifrm + 1
sele/table {p1}_mean seq.lt.{ifrm}
{p1}_mean,:NAME,@{ifrm}  = "all_average"
stat/tab {p1}_mean :MEAN
biasmean = outputr(3)
{p1}_mean,:MEAN,@{ifrm}  =  biasmean
meandif = outputr(2) - outputr(1)
meanstd = outputr(4)
stat/tab {p1}_mean :SIGMA
biassigm = outputr(3)
{p1}_mean,:SIGMA,@{ifrm} =  biassigm
sigmdif = outputr(2) - outputr(1)
sigmstd = outputr(4)
ifrm = ifrm - 1
delete/imag {OUT_A} no
delete/imag {OUT_B} no
delete/imag &a no
delete/imag &s no

set/midas output=yes
set/format i1 t6.2
write/out " "
write/out -
       "     Average of  {ifrm} bias frames:  mean =  {biasmean};  noise ={biassigm}"
write/out " "
write/out -
       "     mean/noise diff.(max-min):  mean =  {meandif};  noise ={sigmdif}"  
write/out -
       "     mean/noise std. deviation:  mean =  {meanstd};  noise ={sigmstd}"  
write/out " "
write/out "The MIDAS table with results is in file {p1}_mean.tbl"
write/out " "
set/format
!
! cleanup 
delete/ima bs_sigm.bdf NO
delete/ima bs_mean.bdf NO
delete/ima postscript.ps NO
delete/ima screen*.ps NO
delete/ima screen*.ima NO
delete/ima graph*.plt NO
delete/ima middumma.bdf NO
delete/tab dummtab.tbl NO
set/graph default
RETURN
!





