!
! "@(#) $Id: ccdtestt_quick.prg,v 2.83 2005/09/30 08:26:46 vltsccm Exp $"
!
! @(#)ccdtestt_quick.prg	Linearity test, QUick version Feb98/PS
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.PURPOSE:        Procedure to test the ccd linearity and transfer
!.Algorithm:      The following tests are done:
!.                Test T1: Creation of the transfer and linearity table"
!.                Test T2: Determination of linearity curve and shutter error
!.                Test T3: Determination of the transfer curve
!
!    Note: if shutter interval (P3) is given as one single number, then
!          it is used as is (ej. 0,0)
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE/PARAMETER P1  ?             C  "Enter input catalogue:"
DEFINE/PARAMETER P2  [@15,@2:>,>]  C  "Enter the window for analysis:"
DEFINE/PARAMETER P3  -0.2,1.0      C  "Time interval to fit the shutter offset"
DEFINE/PARAMETER P4  all           C  "Selection criterium for table:"
! 
write/key aducf/r/1/1 0.0
write/key enoise/r/1/1 0.0
write/key eshutter/r/1/1 0.0
define/local xlab/r/1/1 0.0
define/local ylab/r/1/1 0.0
define/local shval/r/1/3 0.0,0.0,0.0
define/local sherr/r/1/1 0.0
define/local shut/r/1/1 0.0
define/local shoff1/r/1/1 0.0
define/local shoff2/r/1/1 0.0
define/local nstep/i/1/1 0
define/local iter/i/1/1 0
define/local n/i/1/1 0
define/local noff/i/1/1 0
write/key    linerr/r/1/6 0.0 0.0
define/local crit/c/1/50 {p4}

log/tof
WRITE/OUT "     PROCEDURE TO TEST THE CCD LINEARITY AND TRANSFER"
write/out "     ================================================"
write/out " "
write/out " "

!
! *** check if the catalogue exists
IF M$EXIST("{P1}.cat") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <CCDTESTT>: Catalogue {P1} not existing"
   RETURN/EXIT
ENDIF
!
set/midas output=logonly
read/icat {p1}
set/midas output=no
del/grap
crea/grap 0 700,500,500,100 
set/graph default
set/graph pmode=1
set/midas output=yes
!
define/local col1/c/1/20  :UT_tim1 
define/local col2/c/1/20  :UT_tim2 
define/local col3/c/1/20  :Exp_tim1 
define/local col4/c/1/20  :Exp_tim2
define/local col5/c/1/20  :Med_cnt1
define/local col6/c/1/20  :Med_cnt2
define/local col7/c/1/20  :Variance
define/local col8/c/1/20  :Frc_cnt1
define/local col9/c/1/20  :el_adu1 
define/local col10/c/1/20 :Frc_cnt2
define/local col11/c/1/20 :el_adu2 
define/local col12/c/1/20 :scratch
define/local col13/c/1/20 :Med_Cnts
define/local col14/c/1/20 :Frc_cnts
define/local el_adc/r/1/1 0.0
!
set/format f10.2
write/out " "
write/out " Creation of the transfer and linearity table"
write/out "-------------------------------------------------------------------"
write/out "The table will contain 7 columns: column 1 for the UT of the"
write/out "of the exposure of the first image; column 2 for the exposure"
write/out "time of the first image; column 3 the exposure time of the "
write/out "second image; column 4 the median pixel intensity over the"
write/out "selected image section in frame a; column 5 the median pixel"
write/out "intensity over the selected frame area in frame b; column 6"
write/out "the variance of the difference of the frame a and b, and" 
write/out "column 7 is a 2 point aproximation of the Conv. Factor used "
write/out "to select out bad or saturated pairs of images."
write/out "The frames used have been corrected for the "
write/out "bias offset {BIASMEAN} in keyword BIASMEAN."
write/out " "
write/out " Selected area for sampling: {P2}"
set/format
!
!change RGe: create this table anyway, otherwise data from runs taken under 
!different conditions may get messed up
!IF M$EXIST("{p1}.tbl") .EQ. 0 THEN	
   create/tab   {p1} 14 20 null

   create/col   {p1} {col1}      "hours" F9.2 R*4
   create/col   {p1} {col2}      "hours" F9.2 R*4
   create/col   {p1} {col3}      "sec" F9.2 R*4
   create/col   {p1} {col4}      "sec" F9.2 R*4   
   create/col   {p1} {col5}      "ADU" F9.1 R*4
   create/col   {p1} {col6}      "ADU" F9.1 R*4
   create/col   {p1} {col7}      "ADU" F9.1 R*4
   create/col   {p1} {col8}      " % " F9.2 R*4
   create/col   {p1} {col9}      "e-/adu" F9.2 R*4
   create/col   {p1} {col10}     " % " F9.2 R*4
   create/col   {p1} {col11}     "e-/adu" F9.2 R*4
   create/col   {p1} {col12}     "   " F9.2 R*4
   create/col   {p1} {col13}     "ADU" F9.1 R*4
   create/col   {p1} {col14}     " % " F9.2 R*4
!
   define/local ut_time/c/1/9 "O_TIME(5)"
   define/local catal/i/1/1 0
   define/local ifrm/i/1/1 0
   define/local frm1/c/1/60 " "
   define/local frm2/c/1/60 " "
   define/local var/r/1/1 0.0
   define/local tdef/d/1/1 0.0

!
   set/format ,e20.15
   LOOP1:
      store/frame in_a {p1}.cat 1 CONT1
      frm1 = "{in_a}"
      store/frame in_b {p1}.cat 1 CONT1
      frm2 = "{in_b}"
      tdef = {{frm1},{O_DESC}} - {{frm2},{O_DESC}}
      tdef = M$ABS(tdef)
      if tdef .GT. 0.11 then
         write/out "     *** WARNING: exposure times {frm1} amd {frm2} differ "
         write/out "                  more than .11 sec: image pair skipped"
         write/out " "

      else
          IF M$TSTNO("{BIASMEAN}") .EQ. 0 THEN
            write/OUT -
             "     *** INFO: KEYWORD BIASMEAN is not filled with a number"
            write/OUT "               No bias offset subtracted"
            write/out " "
            compute/imag  &a = {in_a}
            compute/imag  &b = {in_b}
          else
            compute/imag  &a = {in_a} - {BIASMEAN}
            compute/imag  &b = {in_b} - {BIASMEAN}
          endif

          ifrm = ifrm + 1
          {p1},{col1},@{ifrm} = {{frm1},{ut_time}}
          {p1},{col2},@{ifrm} = {{frm2},{ut_time}}
          {p1},{col3},@{ifrm} = {{frm1},{O_DESC}}
          {p1},{col4},@{ifrm} = {{frm2},{O_DESC}}
          stat/ima &a {p2} ? ? RN
          {p1},{col5},@{ifrm} = outputr(3)
          stat/ima &b {p2} ? ? RN
          {p1},{col6},@{ifrm} = outputr(3)
          comp/ima &a = {frm1} - {frm2}
          stat/ima &a {p2} ? ? RN
          var = outputr(4) * outputr(4) / 2.0
          {p1},{col7},@{ifrm} = var
          if var .GT. 0 then
            {p1},{col9},@{ifrm} = M$VALUE({p1},{col5},@{ifrm}) / var
            {p1},{col11},@{ifrm} = M$VALUE({p1},{col6},@{ifrm}) / var
          else
            {p1},{col9},@{ifrm} = 0.0
            {p1},{col11},@{ifrm} = 0.0
          endif
      endif
   goto LOOP1
   CONT1:
!
   delete/imag &a NO
   delete/imag &b NO
   write/out " "
   WRITE/OUT "  *** INFO: Linearity table {p1} created"
   write/out " "
   write/out "Content of unedited table {p1}:"
   write/out " "
   read/tab {p1} {col1},{col3},{col4},{col5},{col6},{col7},{col9}

!else
!   WRITE/OUT "  *** INFO: Linearity table {p1} exists; no new one created"
!   write/out " "
!ENDIF
!
!
! *** table ready do the printed and plotted
SELECT/TAB {p1} ALL
sort/tab {p1} {col3}
read/tab {p1} {col1},{col3},{col4},{col5},{col6},{col7},{col9}
write/out " "
!
sort/tab {p1} {col1}
plot/table {p1} {col1} {col3} -60,-40,25,75 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1))/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5))/10
label/gra  "Exposure sequence" {xlab},{ylab} ? ? 1
!
sort/tab {p1} {col3}
set/graph clear=off yform=e6.2
plot/tab {p1} {col3} {col5} -60,-40,25,15 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap  "Linearity Curve (1)" {xlab},{ylab} ? ? 1
!
plot/tab {p1} {col4} {col6} -60,-40,115,15 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Linearity Curve (2)" {xlab},{ylab} ? ? 1
!
set/grap xform=e6.0
plot/tab {p1} {col5} {col7} -60,-40,115,75 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Transfer Curve" {xlab},{ylab} ? ? 1
!
copy/grap postscript 
-copy postscript.ps {p1}_lin1.ps
write/out "Plot with exposure sequence vs median counts, linearity curve (1),"
write/out "linearity curve (2), and transfer curve are stored in file"
write/out "{p1}_lin1.ps"
write/out " "
set/grap clear=on
!
!
TESTT2:
write/out " Determination of linearity curve and the shutter error"
write/out "----------------------------------------------------------------"
write/out "Entries in the table {p1} not fullfilling the"
write/out "selection criteria <{p4}>"
write/out "will now be selected out."
write/out "If the selection is all, values of simple e-/adu calculation"
write/out "will be used to eliminate saturated images."
write/out "From the remaining entries in the table a linear fit is"
write/out "done to determine the linearity curve for frames 1 and 2."
write/out " "
set/midas output=no

!
! *** linearity curve of frames 1

if "{p4}" .eq. "all" then
	sele/tab {p1} {col9}.LT.15.AND.{col9}.GT.0
	stat/tab {p1} {col9}
	el_adc = outputr(3)            ! get mean of el_adc
	enoise = outputr(4)               ! std dev.
	stat/tab {p1} {col11}
	el_adc = (el_adc + outputr(3)) / 2.0
	enoise = (enoise + outputr(4)) * 2.5   ! /2 * 5
	set/format f9.2
	crit = "ABS({col9}-{el_adc}).LT.{enoise}"
endif
set/midas output=yes
write/out "selection criteria: {crit}"
write/out " "
set/midas output=no

select/tab {p1} {crit}
set/grap stype=5 xform= yform=e6.2
plot/tab {p1} {col3} {col5} -60,-40,25,75 4 0
set/grap clear=off
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Linearity Curve (1)" {xlab},{ylab} ? ? 1
regression/lin {p1} {col5} {col3}
save/regress {p1} regress
Compute/regress {p1} {col12} = regress
set/midas output=yes
set/format ,F10.2
write/out "     Linearity results data set 1:"
write/out "     Mean count rate: {outputd(2)} ADU/sec"
set/format
set/format F6.3
sherr = outputd(1) / outputd(2) 
write/out "     Exposure offset: {sherr} seconds (+ = slow; - = fast)" 
set/midas output=no
set/grap ltyp=2 stype=0
overplot/tab {p1} {col3} {col12} ? 0 2 
!
! *** linearity curve of frames 2
set/grap stype=5
select/tab {p1} {crit}
set/grap stype=5 xform= yform=e6.2
plot/tab {p1} {col4} {col6} -60,-40,115,75 4 0
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Linearity Curve (2)" {xlab},{ylab} ? ? 1
regression/lin {p1} {col6} {col4}
save/regress {p1} regress
compute/regress {p1} {col12} = regress
set/midas output=yes
set/format ,F10.2
write/out " "
write/out "     Linearity results data set 2:"
write/out "     Mean count rate: {outputd(2)} ADU/sec"
set/format
set/format F6.3
sherr =  outputd(1) / outputd(2) 
write/out "     Exposure offset: {sherr} seconds (+ = slow; - = fast)" 
set/grap stype=0
overplot/tab {p1} {col4} {col12} ? 0 2
!
!

write/key    shval/r/1/2 {p3}      
write/key    shval/r/3/1 0.01
nstep = ( shval(2) - shval(1) )/ shval(3) 
!
! check if shutter offset given
if nstep .eq. 0 then
   Write/out " "
   write/out "   Shutter offset given:  Recalculating count rates of set 1..."
   shut = shval(1)
else
   ! *** First set: fractional count rate versus median count
   write/descr {p1}.tbl HISTORY_UPDA/I/1/1 0
   write/out " "
   write/out "     Determining shutter offset from fractional count rates (1)"
   set/midas output=no
   set/format


   ITER1:
   shoff1 = 99999999.
   do N = 1  nstep 
    shut =  shval(1)  + ( n - 1) * shval(3) 
    compute/tab {p1} {col8} = {col5}/({col3} + {shut})
    select/tab {p1} {crit}
    stat/tab {p1} {col8}                  !get stderr of col8
    shoff2 =  outputr(4)       ! get stderr to minimize

    if shoff2 .GT. shoff1 .and. iter .eq. 0 then  ! ready to finer step
      shval(1) = shval(1) + (n - 3) * shval(3)   ! step back 2 to find min.
      shval(3) = shval(3)/10                      ! 20 steps with fine step
      nstep = 20
      iter  = 1

      goto ITER1
    elseif shoff2 .GT. shoff1 .and. iter .eq. 1 then
      shut = shval(1) + (n - 2) * shval(3)       ! minimum found to 1/100s
      goto DONE1
    else
      shoff1 =  shoff2 
    endif
   end do

   ! if comming out here: no minimum found. Set shutter offset to 0
   set/format f6.2
   set/midas output=yes
   write/out -
   "*** Method not converging. Shutter offsets from {shval(1)} to {shval(2)} sec."
   write/out "    Shutter offset set to 0."
   shut = 0.0
endif

!
! plot the fractional count rate and the fit
DONE1:
write/descr {p1}.tbl HISTORY_UPDA/I/1/1 1
eshutter =  shut 
set/midas output=yes
set/format f6.3
write/out "     Shutter offset: {shut} seconds"
set/format

compute/tab {p1} {col8} = {col5}/({col3} + {shut})
set/midas output=no
select/tab {p1} {crit}
stat/tab {p1} {col8}
set/midas output=yes
set/format f6.2
write/out "     Average Count Rate: {outputr(3)} ADU/sec"
set/format
compute/tab {p1} {col8} = ({col8}-{outputr(3)})/{outputr(3)}*100
set/midas output=no
select/tab {p1} {crit}
set/grap stype=5 xform=e6.0 yform=f6.2 clear=off
plot/tab {p1} {col5} {col8} -60,-40,25,15 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Diff. Count Rate in % (1)" {xlab},{ylab} ? ? 1
!
stat/tab {p1} {col8}
linerr(1) = outputr(1)
linerr(2) = outputr(2)
set/format f6.2
set/midas output=yes
write/out " "
write/out -
"     Linearity error after shutter correction: +{linerr(2)}/{linerr(1)} %"
set/midas output=no
set/format
!   Draw a line on Zero
comp/tab {p1} {col12} = 0.0
overplot/tab {p1} {col5} {col12} ? 0 2
!
! check if shutter offset given
write/key    shval/r/1/2 {p3}
write/key    shval/r/3/1 0.01
nstep = ( shval(2) - shval(1) ) / shval(3) 
!
set/midas output=yes
if nstep .eq. 0 then
  Write/out " "
  write/out "   Shutter offset given:  Recalculating count rates of set 2..."
  shut = shval(1)
else
  !
  ! *** Second set: fractional count rate versus median count
  write/descr {p1}.tbl HISTORY_UPDA/I/1/1 0
  write/out " "
  write/out "     Determining shutter offset from fractional count rates (2)"
  !
  set/midas output=no
  !
  ITER = 0
  !
  ITER2:
  shoff1 = 999.
  do N = 1 nstep
   shut =  shval(1) + ( n - 1) * shval(3) 
   compute/tab {p1} {col10} = {col6}/({col4}+{shut})
   select/tab {p1} {crit}
   stat/tab {p1} {col10}
   shoff2 = outputr(4)

   if shoff2 .GT. shoff1 .and. iter .eq. 0 then
      shval(1) =  shval(1) + (n - 3) * shval(3) 
      shval(3) = shval(3)/10
      nstep = 20
      iter  = 1

      goto ITER2
   elseif shoff2 .GT. shoff1 .and. iter .eq. 1 then
      shut = shval(1) + (n - 2) * shval(3)
      goto DONE2
   else
      shoff1 = shoff2
   endif
  end do

  ! if comming out here: no minimum found. Set shutter offset to 0
  shut = 0.0
  set/format f6.2
  set/midas output=yes
  write/out -
  "*** Method not converging. Shutter offsets from {shval(1)} to {shval(2)} sec."
  write/out "    Shutter offset set to 0."
endif
!
! plot the fractional count rate and the fit
DONE2:
write/descr {p1}.tbl HISTORY_UPDA/I/1/1 1
eshutter = (eshutter + shut)/2.0
set/midas output=yes
set/format f6.3
write/out "     Shutter offset: {shut} seconds"
set/midas output=no
set/format
compute/tab {p1} {col10} = {col6}/({col4}+{shut})
select/tab {p1} {crit}
stat/tab {p1} {col10}
set/midas output=yes
set/format f6.2
write/out "     Average Count Rate: {outputr(3)} ADU/sec"
set/format
set/midas output=no
compute/tab {p1} {col10} = ({col10}-{outputr(3)})/{outputr(3)}*100
select/tab {p1} {crit}
set/grap  xform=e6.0 yform=f6.2 clear=off
plot/tab {p1} {col6} {col10} -60,-40,115,15 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
label/grap "Diff. Count Rate in % (2)" {xlab},{ylab} ? ? 1
!
stat/tab {p1} {col10}
linerr(3) = outputr(1)
linerr(4) = outputr(2)
set/format f6.2
set/midas output=yes
write/out " "
write/out -
"     Linearity error after shutter correction: +{linerr(4)}/{linerr(3)} %"
set/format
linerr(5) = (linerr(1) + linerr(3))/2.0
linerr(6) = (linerr(2) + linerr(4))/2.0
!
set/midas output=no
!   Draw a line on Zero
comp/tab {p1} {col12} = 0.0
overplot/tab {p1} {col6} {col12} ? 0 2
set/gra clear=on
!
!*** all done; get the postscript file
copy/gra postscript 
-copy postscript.ps {p1}_lin2.ps
set/midas output=yes
write/out " "
write/out "Plot with fitted linearity curves in file {p1}_lin2.ps"
set/grap clear=on
write/out " "

!
!
write/out " Determination of the transfer curve"
write/out "----------------------------------------------------------------"
write/out "From the selected entries the table a linear regression"
write/out "analysis is done to determine the readout noise. The readout"
write/out "noise is determined by the inverse of the slope between"
write/out "the median and the variance" multiplied by the sigma of the"
write/out "bias in keyword BIASSIGM."
write/out " "
!
set/midas output=no
select/tab {p1} {crit}
compute/tab {p1} {col13} = ({col5}+{col6})/2.0     ! get average cts
set/grap xformat=e6.0 yformat=e6.2
select/tab {p1} {crit}
plot/tab {p1} {col13} {col7}  -140,-50,30,85 4 0
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
set/grap clear=off
select/tab {p1} {crit}
regression/lin {p1} {col7} {col13}
save/regress {p1} regress
compute/regress {p1} {col12} = regress
select/tab {p1} {crit}
ADUCF  = 1 / OUTPUTD(2) 
ENOISE = aducf * BIASSIGM
set/format f10.2,f10.2
label/gra "Average Transfer curve: CF={ADUCF} e-/ADU, RON={ENOISE} e- " {xlab},{ylab} ? ? 1
set/midas output=yes
set/format f10.3,f10.3
write/out -
 "     Average Transfer curve: slope: {outputd(2)}, offset in y: {outputd(1)}"
write/out "     Conv. factor: {aducf} e-/ADU; Elec. RON: {enoise} e-"
write/out " "
write/out "     Mean conv. factor: {ADUCF} e-/ADU"
WRITE/OUT "     Mean Elec. RON:   {ENOISE} e-"
write/out "     Values are stored in the CCD keywords ADUCF and ENOISE"
set/format
overplot/tab {p1} {col13} {col12} ? 0 2
!
!    Plot the average count rate response
set/midas output=no
set/format
compute/tab {p1} {col14} = {col13}/({col4}+{shut})
select/tab {p1} {crit}
stat/tab {p1} {col14}
set/midas output=yes
set/format f6.2
write/out "     Average Count Rate: {outputr(3)} ADU/sec"
set/format
set/midas output=no
compute/tab {p1} {col14} = ({col14}-{outputr(3)})/{outputr(3)}*100
select/tab {p1} {crit}
set/grap xform=e6.0 yform=f6.2 clear=off
plot/tab {p1} {col13} {col14} -140,-50,30,15 4 2
xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
set/format f6.2
label/grap "Diff. Count Rate in % (Average={outputr(3)} cts/sec)" {xlab},{ylab} ? ? 1
!
stat/tab {p1} {col14}
linerr(5) = outputr(1)
linerr(6) = outputr(2)
set/midas output=yes
write/out " "
write/out -
"     Linearity error after shutter correction: +{linerr(6)}/{linerr(5)} %"
set/format
set/midas output=no
!   Draw a line on Zero
comp/tab {p1} {col12} = 0.0
overplot/tab {p1} {col13} {col12} ? 0 2
set/gra clear=on
!
set/midas output=yes
copy/grap postscript 
-copy postscript.ps {p1}_trans.ps
write/out " "
write/out "Plot with average transfer and linearity curves"
write/out "  in file {p1}_trans.ps"
set/grap clear=on
!set/grap default
write/out " "
write/out " "
!
delete/ima postscript*.ps NO
delete/ima graph*.plt NO
RETURN







