!
! "@(#) $Id: cosmetic.prg,v 2.83 2005/09/30 08:26:46 vltsccm Exp $"
!
! cosmetic.prg
!
! Program to find CCD - defects on the average of three given "same"
! weak-light (approx. 50 - 200 e-/per pixel) images *.bdf
!+ 
! resulting in kind of defects and their location
!+
! S. Deiries 16.3.1998 Version 5 + 6 (Fits) 
!+
! Parameter:
!+
! xstart = number of first pixel
! ystart = number of first line
! detector = Manufacturer and Serial number of CCD
! exposure = exposure information of frame
! xstep = binning in x
! ystep = binning in y
! xprescan = number of prescan pixels
! yprescan = number of prescan lines
! xspix = number of sensitive pixels
! ysline = number of sensitive lines
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! means = Mean-value of the undisturbed image
! rmss  = RMS of means
! offsets = Offset or zero-value of the CCD image from overscan
! low     = lower limit: pixels below are cold 
! up      = upper limit: pixel up or higher this are hot
! noiselow= noise limited lower limit
! noiseup = noise limited upper limit
! xssta,xsend,yssta,ysend = window of used image in the sensitive area
! rmsorig = undisturbed RMS-value of first input frame (same as the other 2) 
! fivsig = five times this RMS = 5 sigma value
! weakmean = is resulting average image with removed cosmics and noise peaks
!
!+
! Program:
!+
define/par p1 ? ima "Enter name of input first weak light image frame:  "
define/par p2 ? ima "Enter name of input second weak light image frame:  "
define/par p3 ? ima "Enter name of input third weak light image frame:  "
!
@@ test_fits {p1}
@@ test_fits {p2}
@@ test_fits {p3}
@@ CCDtype {p1}
!
write/out Preparing program mean_offset with first frame...
@@ mean_offset {p1} 3 0.2
define/local rmsorig/r/1/1 {q2}
define/local fivsig/r/1/1 0.0
fivsig = rmsorig * 5
write/out Averaging of the three frames...
ave/ima weakmean = {p1},{p2},{p3} ? ? median,{fivsig},{fivsig},data
!
write/out Preparing program mean_offset with the average frame...
@@ mean_offset weakmean 3 0.2
!
define/local means/r/1/1 {q1}
define/local rmss/r/1/1 {q2}
define/local offsets/r/1/1 {q3}
define/local test/r/1/1 0.0
define/local hilf/r/1/1 0.0
!
hilf = 100.0 - 100.0 / xstep
offsets = offsets + ( ( ( means - offsets ) / 100.0 ) * hilf )
test = means - 3.5 * rmss
!
if offsets .gt. test then
	write/out The Offset {offsets} cannot be used for cold pixels
	define/par p2 ? num "Enter new value for Offset:  "
	define/local offset/r/1/1 {p2}
	offsets = offset
endif
!
define/local low/r/1/1 0.0
define/local up/r/1/1 64000.0
define/local noiselow/r/1/1 0.0
define/local noiseup/r/1/1 64000.0 
!
! lower limit 65% of signal: !  old value 50%
!
low = offsets + ( ( ( means - offsets ) / 100 ) * 65 )
noiselow = means - 4 * rmss   ! old value 3.5 * rmss
if noiselow .le. low low = noiselow
!
! upper limit 180% of signal: !  old value 200%
!
up = offsets + ( ( ( means - offsets ) / 100 ) * 180 )
noiseup = means + 4 * rmss   ! old value 6 * rmss
if noiseup .ge. up up = noiseup
!
define/local xssta/i/1/1 0
define/local yssta/i/1/1 0
define/local xsend/i/1/1 0
define/local ysend/i/1/1 0
!
xssta = xstart + (( xprescan + 1 ) * xstep )
yssta = ystart + (( yprescan + 1 ) * ystep )
xsend = xssta + (( xspix - 3 ) * xstep )
ysend = yssta + (( ysline - 3 ) * ystep )
!
if detector(1:6) .eq. "ml2k3b" then
!
! wax regions at ml2k3b CCDs not tested (a frame of 20 pixel)
!
   write/out Wax regions not tested and frame of 20 pixels excluded
   xssta = xstart + (( xprescan + 20 ) * xstep )
   yssta = ystart + (( yprescan + 20 ) * ystep )
   xsend = xssta + (( xspix - 41 ) * xstep )
   ysend = yssta + (( ysline - 41 ) * ystep )
!
endif
!
! define/local x/i/1/1 0
! define/local y/i/1/1 0
! define/local value/i/1/1 0
!
!set/format f3.2
!
write/out Extracting ff from weakmean...
extract/ima ff = weakmean[{xssta},{yssta}:{xsend},{ysend}]
write/out Finding defect pixels...
find/pix ff {low},{up} out a defects 16384
define/local numpix/i/1/1 {OUTPUTI(1)}
!
$rm ff.bdf
!
write/out Sorting Table...
sort/tab defects :X_COORD,:Y_COORD
!
!do x = {xssta} {xsend}
!	do y = {yssta} {ysend}
!		value = {weakmean[{x},{y}]}
!		if value .ge. up write/out bright! x={x} y={y} value={value}
!		if value .le. low write/out dark! x={x} y={y} value={value}
!	enddo
!enddo
!
open/file cosme1.result write OUTPUTI
write/file {OUTPUTI(1)} {means} {offsets} {low} {up} {numpix} 
close/file {OUTPUTI(1)}
!
return {means} {offsets}
!
! end




