!
! "@(#) $Id: cte.prg,v 2.83 2005/09/30 08:26:47 vltsccm Exp $"
!
! cte.prg
!+
! Program to calculate the horizontal and vertical CTE (Charge transfer
! efficieny) of a CCD frame (flatfield-exposure) given as image   *.bdf
!+ 
! resulting in the horizontal and vertical CTE using the Eper method:
!+
! S. Deiries 16.3.1998 Version 4 + 5 (Fits)
!+
! Eper method:     CTE = 1 - ( SUM(VOP) / VLAP / NFER ) 
!+
! Parameter:
!+
! horcte 	= Horizontal cte
! xlap	= Last active pixel
! vxlap	= Value of last active pixel
! xfop	= First overscan pixel
! vxfop = Value of first overscan pixel
! vxop	= Sum of the content of up to ten overscan pixels
! xoff	= Last used overscan pixel
! vxoff	= Value of last used overscan pixel, respresenting the CCD-offset
! nxfer = Number of pixel-transfers at xfop position
!+
! vercte	= Vertical cte
! ylap	= Last active line
! vylap	= Value of last active line
! yfop	= First overscan line
! vyfop = Value of first overscan line
! vyop	= Sum of the content of up to ten overscan lines
! yoff	= Last used overscan line
! vyoff	= Value of last used overscan line, respresenting the CCD-offset
! --> vxoff is used !?
! nyfer = Number of line-transfers at yfop position
!+
! npix = number of pixel (x)
! nline = number of lines (y)
! xstart = number of first pixel
! ystart = number of first line
! xstep = binning in x
! ystep = binning in y
! xprescan = number of prescan pixels
! xspix = number of sensitive pixels
! ysline = number of sensitive lines
! extype = abbreviation of exposure type
! image = file-name of used image
! calc = Value to find out, whether the first overscan is OK
! horok = Flag whether Horizontal CTE is OK
! verok = Flag whether Vertical CTE is OK
!
!+
! Program:
!+
define/par p1 ? ima "Enter name of input image frame:"
!
@@ test_fits {p1}
@@ CCDtype {p1}
!
define/local vercte/r/1/1 0.0
define/local horcte/r/1/1 0.0
define/local xlap/i/1/1 0
define/local ylap/i/1/1 0
define/local vxlap/r/1/1 0.0
define/local vylap/r/1/1 0.0
define/local vxfop/r/1/1 0.0
define/local vyfop/r/1/1 0.0
define/local xfop/i/1/1 0
define/local yfop/i/1/1 0
define/local vxop/r/1/1 0.0
define/local vyop/r/1/1 0.0
define/local xoff/i/1/1 0
define/local yoff/i/1/1 0
define/local vxoff/r/1/1 0.0
define/local vyoff/r/1/1 0.0
define/local nxfer/i/1/1 0
define/local nyfer/i/1/1 0
define/local xlast/i/1/1 0
define/local ylast/i/1/1 0
define/local xst/i/1/1 0
define/local yst/i/1/1 0
define/local xen/i/1/1 0
define/local yen/i/1/1 0
define/local calc/r/1/1 0.0
define/local horok/i/1/1 0
define/local verok/i/1/1 0
define/local n/i/1/1 0
define/local m/i/1/1 0
!
xst = xstart + xprescan + ( 0.40 * xspix )
xen = xstart + xprescan + ( 0.60 * xspix )
yst = ystart + ( 0.40 * ysline )
yen = ystart + ( 0.60 * ysline )
!
@@ divide_in_squares {xst} {yst} {xen} {yen} 2
!
! tol = tolerance of undisturbed mean measurement in sensitive area
! meanccd = mean of mean values in sensitive area
! xm(i) = xcoordinates of windows in sensitive area
! ym(i) = ycoordinates of windows in sensitive area
! xst,yst,xen,yen: coordinates used for tempory windows in this program-part
! k,l,nn = loop variables
!
define/local xm/i/1/200 0 all
define/local ym/i/1/200 0 all
define/local nn/i/1/1 0
define/local k/i/1/1 0
define/local l/i/1/1 0
define/local square/i/1/1 4
define/local tol/r/1/1 0.20
!
do nn = 1 200
        xm({nn}) = xcoord({nn})
        ym({nn}) = ycoord({nn})
enddo
!
create/tab csenstat 2 {square} null
create/col csenstat :Mean "ADU" f6.2
create/col csenstat :RMS "ADU" f6.2
!
do nn = 1 {square}
        l = nn * 2
        k = l - 1
        xst = xm({k})
        yst = ym({k})
        xen = xm({l})
        yen = ym({l})
        @@ ground {p1} {xst} {yst} {xen} {yen} {tol}
        csenstat,:Mean,@{nn} = {q1}
!       csenstat,:RMS,@{nn} = {q2}
enddo
!
stat/tab csenstat :Mean
define/local meanccd/r/1/1 {outputr(3)}
!write/key mrms/r/1/1 {outputr(4)}
!
!stat/ima {p1} [{xst},{yst}:{xen},{yen}] 
!define/local meanccd/r/1/1 {outputr(3)}
!
Start:
!
xlap = xprescan + xspix + xstart - 1
xfop = xlap + 1
xoff = xfop + 10
xlast = xstart + npix - 2
if xlast .le. xoff xoff = xlast
!
ylap = ysline + ystart - 1
yfop = ylap + 1
yoff = yfop + 10
ylast = ystart + nline - 2
if ylast .le. yoff yoff = ylast

!
! Calculate limits of sample:
!
xst = xstart + xprescan - 1 + 10
xen = xlap - 10
yst = ystart - 1 + 10
yen = ylap - 10
!
! Horizontal CTE:
!
if horok .eq. 0 then
!
   average/row hcte = {p1} @{yst},@{yen}
!
   read/ima hcte [@{xlap}:@{xlap}]
   vxlap = outputr(1)
   read/ima hcte [@{xfop}:@{xfop}]
   vxfop = outputr(1)
   read/ima hcte [@{xoff}:@{xoff}]
   vxoff = outputr(1) 
   wait 5
   vxlap = vxlap - vxoff
   vxfop = vxfop - vxoff
   calc = vxlap * 1.5  ! instead 1.1
!
   if calc .ge. meanccd then
!
      write/out Horizontal overscan is OK!
      horok = 1
!
   else
!
      write/out Problems with Horizontal overscan or CTE !!
      xspix = xspix - 1
      m = m + 1
      if m .le. 10 then  ! 1 for testing
!
         GOTO Start
!
      else
!
         horok = 1
! 
      endif   
!
   endif
!
   do n = {xfop} {xoff}
	   read/ima hcte [@{n}:@{n}]
	   vxop = vxop + outputr(1) - vxoff 
   enddo
!
   nxfer = xfop - xstart + 1
!
   horcte = 1 - (( vxop / vxlap ) / nxfer )
!
   set/format f8.7
   write/out Horizontal / Serial CTE = {horcte}
!
endif
!
! Vertical CTE:
!
if verok .eq. 0 then
!
   average/col vcte = {p1} @{xst},@{xen}
!
   read/ima vcte [@{ylap}:@{ylap}]
   vylap = outputr(1) 
   read/ima vcte [@{yfop}:@{yfop}]
   vyfop = outputr(1)
   read/ima vcte [@{yoff}:@{yoff}]
   vyoff = outputr(1) 
   wait 5
   vylap = vylap - vyoff
   vyfop = vyfop - vyoff
   calc = vylap * 1.5  ! instaed 1.1
!
   if calc .ge. meanccd then
!
      write/out Vertical overscan is OK!
      verok = 1
!
   else
!
      write/out Problems with Vertical overscan or CTE !!
      ysline = ysline - 1
      m = m + 1
      if m .le. 15 then   ! 1 for testing
!
         GOTO Start
!
      endif   
!
   endif
!
   do n = {yfop} {yoff}
	   read/ima vcte [@{n}:@{n}]
	   vyop = vyop + outputr(1) - vyoff 
   enddo
!
   nyfer = yfop - ystart + 1
!
   if vylap .eq. 0 vylap = 0.1
   if nyfer .eq. 0 nyfer = 0.1
   vercte = 1 - (( vyop / vylap ) / nyfer )
!
   set/format f8.7
   write/out Vertical / Parallel CTE = {vercte}
!
endif
!
write/out Last active pixel: {xlap} and Last active line: {ylap}
write/out ______________________________
write/out CTE Horizontal/Serial: {horcte} and Vertical/Parallel: {vercte}
!
open/file cte.result write OUTPUTI
write/file {OUTPUTI(1)} {horcte} {vercte} {xlap} {ylap}
close/file {OUTPUTI(1)}
!
return {horcte} {vercte} {ylap}
! end



