!
! "@(#) $Id: dcurrent.prg,v 2.83 2005/09/30 08:26:48 vltsccm Exp $"
!
! dcurrent.prg
!
! Program to calculate the dark-current of a dark image given as
! frame:  *.bdf using the overscan region as a bias reference
!+ 
! resulting the mean dark current and standart deviation
!+
! S. Deiries 16.3.1998 Version 8 + 9(Fits)

!+
! Parameter:
!+
! detector = Manufacturer and Serial number of CCD
! exposure = exposure information of frame
! pix = pixel size in micron
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! factor = time correction parameter
! tolera = tolerance for sensitive area disturbation in % / 100 (0.2)
! senmean = Mean value of sensitive area
! dcmean = mean value of sensitive area minus offset
! dcrms = RMS of sensitive area
! current = dcmean with time correction in ADU per hour per pixel 
! currms = RMS of current
! dcoffset = Offset value from overscan region
! dtime = time digits
! dunit = correction for sec, min or hour
! mrms = RMS value of the sensitive area rectangels
!+
! Program:
!+
define/par p1 ? ima "Enter name of long dark frame:"
define/par p2 ? ima "Enter name of short dark frame or enter <<no>>:"
define/par p3 ? num "Enter number of images squares in sqrt(squares):"
! 3 should be used !
!
@@ test_fits {p1}
@@ CCDtype {p1}
!
define/local factor/r/1/1 0.0
define/local dcmean/r/1/1 0.0
define/local current/r/1/1 0.0
define/local currms/r/1/1 0.0
define/local tolera/r/1/1 0.0
!
!define/local indx/i/1/1 0
!compute/key indx = M$EXISTEND(p2,"NAXIS")
!
tolera = 0.2
!
if p2 .eq. "no" then
!
   write/out Preparing program mean_offset...
   @@ mean_offset {p1} {p3} {tolera}
!
else
!
   @@ test_fits {p2}		
   write/out Computing difference between long dark and short dark...
   compute/ima res = {p1} - {p2}
   write/out Preparing program mean_offset...
   @@ mean_offset res {p3} {tolera}
!
   $rm res.bdf
!
endif
!
define/local senmean/r/1/1 {q1}
define/local dcrms/r/1/1 {q2}
define/local dcoffset/r/1/1 {q3}
!
if p2 .eq. "no" then
!
   dcmean = senmean - dcoffset
!
else
!
   dcmean = senmean
   dcrms = mrms 
!
endif
!
define/local dtime/i/1/1 {exposure(4:5)}
define/local dunit/c/1/1 "{exposure(6:6)}"
if dunit .eq. "H" factor = 1
if dunit .eq. "M" factor = 0.01666667
if dunit .eq. "S" factor = 0.000277777777
!
current = ( dcmean / factor ) / dtime  ! in ADU
currms = ( dcrms / factor ) / dtime    ! in ADU
!
define/local indx/i/1/2 0,0
compute/key indx(1) = M$EXISTD(p1,"CON_FACT")
!
set/format f6.2
!
if indx(1) .eq. 0 then
!
   write/out Mean dark current per hour: {current} +/- {currms} ADU
!
else
!
   def/local confact/r/1/1 {{p1},CON_FACT}
   current = confact * ( dcmean / factor ) / dtime    ! in e-
   currms = confact * ( dcrms / factor ) / dtime    ! in e-
   write/out Mean dark current per hour: {current} +/- {currms} e-
!
endif
!
set/format
open/file dcurrent.result write OUTPUTI
write/file {OUTPUTI(1)} {current} {currms} {senmean} {dcrms} {dcoffset}
close/file {OUTPUTI(1)}
return {current} {currms} 
!
! end










