!
! "@(#) $Id: linearity_quick.prg,v 2.83 2005/09/30 08:26:49 vltsccm Exp $"
!
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: linearity_test.prg    
!.PURPOSE:        Procedure to reduce a full linearity test
!.AUTHOR:         Thomas Augusteijn
!.USE:            @@ linearity_test p1 p2 p3 p4 p5
!.
!.Algorithm:      Using a set of 5 bias frames, and 10 pairs of flat
!                 fields the linearity of a CCD is tested. 
!.VERSION:        960401 TAU creation
!
!   Simplified test of linearity for testing purposes. Feb.98 PS
!
!    Modif. for 2 bias images only Feb98/PS
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE/PARAMETER P1 ?            C*2  "Number of Test:"
DEFINE/PARAMETER P2 "000000"     C*6  "Name of System (6chrs):"
DEFINE/PARAMETER P3 "data.cat"   C    "Image catalog containing test frames:"
DEFINE/PARAMETER P4 "SLOW"       C*4  "CCD read-out mode (4chrs):"
DEFINE/PARAMETER P5 -9           N    "Shutter Offset in seconds:"

!IF "{p4}" .NE. "SLOW" .and. "{P4}" .NE. "FAST"  THEN
!write/out " Error: Parameter P4 can only be SLOW or FAST!"
!STOP

!ENDIF
write/out " "
write/out "   You have just started the linearity reduction package. The"
write/out "   input catalogue {p3} should contain 5 bias and as many flat"
write/out "   fields pairs you want."
write/out " "
write/out "   This procedure assumes that you run Midas version 95NOV."
write/out " "
write/out "   All the files created by this procedure will have the suffix"
write/out "   _{p1}_{p2}."
write/out " "

!
! Definition of keywords used in this programma
!

define/local firstH/i/1/1 0
define/local lastH/i/1/1 0
define/local firstV/i/1/1 0
define/local lastV/i/1/1 0
define/local deltaH/i/1/1 0
define/local deltaV/i/1/1 0
define/local shutLOW/r/1/1 -0.2
define/local shutHIGH/r/1/1 0.8
define/local window/c/1/30 " "
define/local rows/c/1/20 " "
define/local cols/c/1/20 " "

!
! Check if shutter offset is manualy specified
!          
if {p5} .ne. -9 then
        shutLOW = {p5}
        shutHIGH = {p5}
endif

!
! Store frames of the different tests in appropriate image catalogs, define the
! window in which the test are taken and fill the IDENT and O_TIME descriptors
!

define/local catal/i/1/1 0
define/local nfrm/i/1/1 0
define/local minmax/r/1/2 0.0 0.0

write/out "    Preparing image descriptors. This may take a while..."

! *** check if the catalogue exists
IF M$EXIST("{P3}") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <linearity_quick>: Catalogue {P3} not existing"
   RETURN/EXIT
ENDIF

set/midas output=no

LOOP1:
   store/frame in_a {p3} 1 CONT1
   copy/dk {in_a} O_TIME/d/7/1 dt 
   set/format ,f6.1
   write/key tc/c/1/5 {dt}

   nfrm = nfrm + 1
   if nfrm .eq. 1 then
       copy/dk {in_a} START/d/1/1 firstH
       copy/dk {in_a} START/d/2/1 firstV

       copy/dk {in_a} NPIX/i/1/1 deltaH
       copy/dk {in_a} NPIX/i/2/1 deltaV

       lastH = firstH + deltaH - 1
       lastV = firstV + deltaV - 1

       write/descr {in_a} ident/c/1/60 {p1}/BIAS-det:{tc}
       crea/icat bias_lin_{p4}_{p1}_{p2} {in_a}
    endif

   if nfrm .gt. 1 .and. nfrm .lt. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/BIAS-det:{tc}
       add/icat bias_lin_{p4}_{p1}_{p2} {in_a}
    endif

   if nfrm .eq. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/FF-det:{tc}
       crea/icat ff_lin_{p4}_{p1}_{p2} {in_a}
    endif

   if nfrm .gt. 6  then
       write/descr {in_a} ident/c/1/60 {p1}/FF-det:{tc}
       add/icat ff_lin_{p4}_{p1}_{p2} {in_a}
    endif

   goto LOOP1

CONT1:

set/format

! Initializing CCD context

set/con ccdtest

! Clear logfile and set to 60 lines per page
log(7) = 60
log/on
delete/logf

set/midas output=yes

write/out " "
write/out " "
write/out "     ===================================================="
write/out "               LINEARITY TEST REDUCTION PACKAGE"
write/out "     ===================================================="
write/out " "
write/out " "
write/out " "
write/out "                          TEST # {P1}"
write/out " "
write/out " "
write/out "                      SYSTEM: {P2}"
write/out " "

! eliminate borders of image which could be wierd...

firstH = firstH + 30
lastH  = lastH - 30
firstV = firstV + 3
lastV  = lastV - 3
set/format i1
write/key window/c/1/30 "[{firstH},{firstV}:{lastH},{lastV}]"
write/key rows/c/1/20 "{firstV},{lastV}"
write/key cols/c/1/20 "{firstH},{lastH}"

@@ ccdtestb_quick bias_lin_{p4}_{p1}_{p2} {window}

set/format i1 f6.2
@@ ccdtestt_quick ff_lin_{p4}_{p1}_{p2}  {window} {shutLOW},{shutHIGH}

log/on
log/tof

write/out "     =========================================================="
write/out " "
write/out "     SUMMARY LINEARITY TEST #{p1} TAKEN WITH {p2} IN {P4} MODE"
write/out " "
write/out "     =========================================================="
write/out " "
write/out " "


write/out "      BIAS"
write/out "      ===="
write/out " "
set/format t6.2
write/out -
     "      5 biases at start test: average = {BIASMEAN} stddev = {BIASSIGM}"
write/out " "
write/out " "
write/out " "
write/out "      TRANSFER FUNCTION AND LINEARITY DATA"
write/out "      ===================================="
write/out " "
read/tab ff_lin_{p4}_{p1}_{p2} #3 #5 #6 #7 #8 #10 #9
set/midas output=no
stat/tab ff_lin_{p4}_{p1}_{p2} #5
minmax(1) = outputr(1)
minmax(2) = outputr(2)
stat/tab ff_lin_{p4}_{p1}_{p2} #6
minmax(1) = minmax(1) + outputr(1)
minmax(2) = minmax(2) + outputr(2)
minmax(1) = minmax(1) * ADUCF / 2
minmax(2) = minmax(2) * ADUCF / 2
set/midas output=yes
set/format f6.0

write/out " "
write/out "      Dynamic range explored: from {minmax(1)} to {minmax(2)} e-"
write/out " "
write/out " "

set/format t6.2

write/out "      CONV. FACTOR AND RON"
write/out "      ===================="
write/out " "
write/out "      C.Fact = {ADUCF} e-/ADU"
write/out " "
write/out "      RON    = {ENOISE} e- RMS"
write/out " "
write/out " "


write/out "      SHUTTER ERROR"
write/out "      ============="
write/out " "
set/format t6.3
if {p5} .eq. -9 then
 write/out -
 " Average shutter error derived from fractional count rate = {ESHUTTER} sec"
else
 write/out -
 " Shutter error specified and used to correct count rate= {ESHUTTER} sec"
endif
write/out " "
write/out " "
set/format t5.2

write/out "      LINEARITY ERROR"
write/out "      ==============="
write/out " "
write/out -
" Average linearity error after shutter correct. = +{linerr(6)}/{linerr(5)} %"
write/out " "
write/out " These errors represent maximum deviation in % from the average"
write/out " count rate after correction from shutter delays"

set/format

! Print logfile with results
assign/print FILE pippo.log
print/logf

!! Save log file
!
$cp pippo.log lin_{p4}_{p1}_{p2}.txt
$rm pippo.log
!

log/on
! Make backup

write/out " "
write/out " You just finished the CCD linearity test procedure."
write/out " "
write/out " Thank you"
write/out " "

log/on
RETURN


