!
! "@(#) $Id: linearity_test.prg,v 2.83 2005/09/30 08:26:49 vltsccm Exp $"
!
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: linearity_test.prg    
!.PURPOSE:        Procedure to reduce a full linearity test
!.AUTHOR:         Thomas Augusteijn
!.USE:            @@ linearity_test p1 p2 p3 p4 p5
!.                where p1 = number of CCD
!.                      p2 = date of test (format DDMMYY)
!.                      p3 = image catalog containing test data in the order
!.                           as taken by the test script
!.                      p4 = CCD read-out mode
!.                      p5 = output short/full
!.
!.Algorithm:      Using a set of 5 bias frames, and 10 pairs of flat
!                 fields the linearity of a CCD is tested. 
!.VERSION:        960401 TAU creation
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE/PARAMETER P1 ?            C*2  "Number of CCD:"
DEFINE/PARAMETER P2 "000000"     C*6  "Date of test (DDMMYY):"
DEFINE/PARAMETER P3 "data.cat"   C    "Image catalog containing test frames:"
DEFINE/PARAMETER P4 "SLOW"       C*4  "CCD read-out mode (SLOW/FAST):"
DEFINE/PARAMETER P5 "S"          C*1  "Output S(hort)/F(ull):"

IF "{p4}" .NE. "SLOW" .and. "{P4}" .NE. "FAST"  THEN
write/out " Error: Parameter P4 can only be SLOW or FAST!"
STOP

ENDIF

write/out " "
write/out " "
write/out "   You have just started the linearity reduction package. The"
write/out "   input catalogue {p3} should contain the 25 "
write/out "   images in the same order) as taken by the CCD `linearity_test'"
write/out "   test script."
write/out " "
write/out "   This procedure assumes that you run Midas version 95NOV."
write/out " "
write/out "   During execution of this procedure the Midas display window"
write/out "   should be completely inside the area of your workstation,"
write/out "   and not covered by any other display window. Also, do not"
write/out "   inconify this window, or move to another workspace during"
write/out "   execution."
write/out " "
write/out "   All the files created by this procedure will have the suffix"
write/out "   _{p1}_{p2}."
write/out " "
write/out "   At the end of the test the procedure will automatically print"
write/out "   out the test results and images at the assigned printer. A"
write/out "   copy of this test report should be delivered to the ODT team."
write/out " "
write/out "   In case of problems with the reduction package contact Thomas"
write/out "   Augusteijn (tauguste@eso.org), or any member of the ODT team."
write/out " "
write/out "                         Good luck!"
write/out " "

!
! Definition of keywords used in this programma
!
define/local dt/d/1/1 0.0
define/local t0/d/1/1 0.0

define/local firstH/i/1/1 0
define/local lastH/i/1/1 0
define/local firstV/i/1/1 0
define/local lastV/i/1/1 0
define/local deltaH/i/1/1 0
define/local deltaV/i/1/1 0
define/local shutLOW/r/1/1 0.0
define/local shutHIGH/r/1/1 0.0

!
! Definition of default values for the different CCD's
! as defined in table CCDSPEC, and read in here. 
! Any changes, or values for a new CCD should be added in that
! table at the appropriate row which corresponds to the CCD number
!

set/format i1
shutLOW = M$VALUE(ccdspec,:Shut_low,@{p1})
shutHIGH = M$VALUE(ccdspec,:Shut_high,@{p1})

!
! Store frames of the different tests in appropriate image catalogs, define the
! window in which the test are taken and fill the IDENT and O_TIME descriptors
!
! Note: descriptor O_TIME now filled by program "restore_times". Not needed
! in the future with new version (2.5) of IRS. Sep14/PS
! However, descripto O_TIME(5): UT time start not filled yet??? OCt96/PS

define/local catal/i/1/1 0
define/local nfrm/i/1/1 0
define/local minmax/r/1/2 0.0 0.0

write/key detec/C/1/20 CCD#{p1}

write/out "    Preparing image descriptors. This may take a while..."

! *** check if the catalogue exists
IF M$EXIST("{P3}") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <linearity_test>: Catalogue {P3} not existing"
   RETURN/EXIT
ENDIF

set/midas output=no

LOOP1:
   store/frame in_a {p3} 1 CONT1

   copy/dk {in_a} O_TIME/d/7/1 dt 
   copy/dk {in_a} TM_START/d/1/1 t0
   t0 = t0 / 3600.0d0
   copy/kd t0/d/1/1 {in_a} O_TIME/d/5/1

   set/format ,f6.1

   write/key tc/c/1/5 {dt}

   nfrm = nfrm + 1

   if nfrm .eq. 1 then

       copy/dk {in_a} START/d/1/1 firstH
       copy/dk {in_a} START/d/2/1 firstV

       copy/dk {in_a} NPIX/i/1/1 deltaH
       copy/dk {in_a} NPIX/i/2/1 deltaV

       lastH = firstH + deltaH - 1
       lastV = firstV + deltaV - 1

       write/descr {in_a} ident/c/1/60 {p1}/BIAS-det:{tc}
       crea/icat bias_lin_{p4}_{p1}_{p2} {in_a}

    endif
   if nfrm .gt. 1 .and. nfrm .lt. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/BIAS-det:{tc}
       add/icat bias_lin_{p4}_{p1}_{p2} {in_a}
    endif

   if nfrm .eq. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/FF-det:{tc}
       crea/icat ff_lin_{p4}_{p1}_{p2} {in_a}
    endif
   if nfrm .gt. 6 .and. nfrm .lt. 26 then
       write/descr {in_a} ident/c/1/60 {p1}/FF-det:{tc}
       add/icat ff_lin_{p4}_{p1}_{p2} {in_a}
    endif

   goto LOOP1

CONT1:

set/format

! Initializing CCD context

set/con ccdtest

! Clear logfile and set to 60 lines per page
log(7) = 60
log/on
delete/logf

set/midas output=yes

write/out " "
write/out " "
write/out "     ===================================================="
write/out "               LINEARITY TEST REDUCTION PACKAGE"
write/out "     ===================================================="
write/out " "
write/out " "
write/out " "
write/out "                          CCD #{P1}"
write/out " "
write/out " "
write/out "                      DATE TEST {P2}"
write/out " "

! don't log detailed info if short output desired

if "{p5}" .eq. "S" .or. "{p5}" .eq. "s" then
  log/off
endif

@@ ccdtestb ? bias_lin_{p4}_{p1}_{p2}.cat BIAS_TRANS_{p4}_{p1}_{p2} median

set/format i1 f6.2
@@ ccdtestt ? ff_lin_{p4}_{p1}_{p2}.cat TRANS_{p4}_{p1}_{p2} ? {shutLOW},{shutHIGH}

log/on
log/tof

write/out "     =========================================================="
write/out " "
write/out "     SUMMARY LINEARITY TEST CCD#{p1} TAKEN ON {p2} IN {P4} MODE"
write/out " "
write/out "     =========================================================="
write/out " "
write/out " "


write/out "      BIAS"
write/out "      ===="
write/out " "
set/format t6.2
write/out -
     "      5 biases at start test: average = {BIASMEAN} stddev = {BIASSIGM}"
write/out " "
write/out "      There were {hot_pix} hot pixels/columns detected"
write/out " "
write/out " "
write/out "      TRANSFER FUNCTION AND LINEARITY DATA"
write/out "      ===================================="
write/out " "
read/tab TRANS_{p4}_{p1}_{p2} #3 #5 #6 #7 #8 #10 #9
set/midas output=no
stat/tab TRANS_{p4}_{p1}_{p2} #5
minmax(1) = outputr(1)
minmax(2) = outputr(2)
stat/tab TRANS_{p4}_{p1}_{p2} #6
minmax(1) = minmax(1) + outputr(1)
minmax(2) = minmax(2) + outputr(2)
minmax(1) = minmax(1) * ADUCF / 2
minmax(2) = minmax(2) * ADUCF / 2
set/midas output=yes
set/format f6.0

write/out " "
write/out "      Dynamic range explored: from {minmax(1)} to {minmax(2)} e-"
write/out " "
write/out " "

set/format t6.2

write/out "      CONV. FACTOR AND RON"
write/out "      ===================="
write/out " "
write/out "      C.Fact = {ADUCF} e-/ADU"
write/out " "
write/out "      RON    = {ENOISE} e- RMS"
write/out " "
write/out " "


write/out "      SHUTTER ERROR"
write/out "      ============="
write/out " "
set/format t6.3
write/out -
" Average shutter error derived from fractional count rate = {ESHUTTER} sec"
write/out " "
write/out " "
set/format t5.2

write/out "      LINEARITY ERROR"
write/out "      ==============="
write/out " "
write/out -
" Average linearity error after shutter correct. = +{linerr(6)}/{linerr(5)} %"
write/out " "
write/out " These errors represent maximum deviation in % from the average"
write/out " count rate after correction from shutter delays"

set/format

! Print logfile with results

print/logf

! ensure the last page gets into the file so a2ps can print it...
! print/logf flushes the buffer before printing, but a2ps knows nothing.

!log/off

!$a2ps -P$PRINTER $HOME/midwork/FORGR{mid$sess(11:12)}.LOG

!! Save log file
!
$cp $HOME/midwork/FORGR{mid$sess(11:12)}.LOG lin_{p4}_{p1}_{p2}.txt
!

log/on
! Make backup

write/out " "
write/out " "
write/out " You just finished the CCD linearity test procedure."
write/out " "
write/out " Please make a back-up of the data in the catalogue {p3},"
write/out " and all files with the suffix _{p4}_{p1}_{p2}."
write/out " "
write/out " A print out of the test results and images is automatically"
write/out " made at the assigned printer. A copy of this test report"
write/out " should be delivered to the ODT team."
write/out " "
write/out " Thank you"
write/out " "

! Print graphics generated by the programme

$ lpr BIAS_TRANS_{p4}_{p1}_{p2}_aver.ps
$ lpr BIAS_TRANS_{p4}_{p1}_{p2}_rebin.ps
$ lpr TRANS_{p4}_{p1}_{p2}_lin1.ps
$ lpr TRANS_{p4}_{p1}_{p2}_lin2.ps
$ lpr TRANS_{p4}_{p1}_{p2}_trans.ps

if "{p5}" .eq. "F" .or. "{p5}" .eq. "f" then

 $ lpr BIAS_TRANS_{p4}_{p1}_{p2}.ps

 IF M$EXIST("BIAS_TRANS_{p4}_{p1}_{p2}_hotpix.ps") .NE. 0  THEN
   $ lpr BIAS_TRANS_{p4}_{p1}_{p2}_hotpix.ps
   $ lpr BIAS_TRANS_{p4}_{p1}_{p2}_aver_clean.ps
 endif

 $ lpr BIAS_TRANS_{p4}_{p1}_{p2}_hist.ps

endif
log/on
RETURN


