!
! "@(#) $Id: mean_offset.prg,v 2.83 2005/09/30 08:26:50 vltsccm Exp $"
!
! mean_offset.prg
!+
! Program to calculate the Mean- and offset-value of a
! frame:  *.bdf using the overscan region as a bias(offset) reference
!+ 
! resulting the mean Mean value, its standard deviation and the mean offset
! value from the overscan region
!+
! S. Deiries 01.09.1995 Version 2
!+
! Parameter:
!+
! npix = number of pixel (x)
! nline = number of lines (y)
! xstart = number of first pixel
! ystart = number of first line
! xstep = binning in x
! ystep = binning in y
! xprescan = number of prescan pixels
! xspix = number of sensitive pixels
! ysline = number of sensitive lines
! extype = abbreviation of exposure type
! ccd_type = CCD detector type name
! image = file-name of used image
!+
! grade = grade of division of sensitive area
! tol = tolerance of undisturbed mean measurement in sensitive area
! mmean = mean of mean values in sensitive area
! mrms = final rms value of sensitive region
! offset = mean offset value of the overscan region
! doffset = rms of mean offset value of the overscan region
! xm(i) = xcoordinates of windows in sensitive area
! ym(i) = ycoordinates of windows in sensitive area
! xo(i) = xcoordinates of windows in overscan area
! yo(i) = ycoordinates of windows in overscan area
! x1,y1,x2,y2: coordinates used for tempory windows
! k,l,n = loop variables
! meanrms = tempory mean of different rms of sensitive area
!+
! Program:
!+
define/par p1 ? ima "Enter name of input image frame:"
define/par p2 ? num "Enter grade of sensitive squares in sqrt(squares):"
define/par p3 ? num "Enter tolerance value in percentage / 100 :  "
!
define/local xm/i/1/200 0 all
define/local ym/i/1/200 0 all
define/local xo/i/1/200 0 all
define/local yo/i/1/200 0 all
define/local x1/i/1/1 0.0
define/local x2/i/1/1 0.0
define/local y1/i/1/1 0.0
define/local y2/i/1/1 0.0
define/local n/i/1/1 0
define/local k/i/1/1 0
define/local l/i/1/1 0
define/local grade/i/1/1 {p2}
define/local square/i/1/1 100
define/local tol/r/1/1 {p3}
!
square = grade * grade
!
! Offset region:
!
x2 = xstart + ( npix - 2 ) * xstep
x1 = x2 - 3 * xstep
y1 = ystart + 1 * ystep
y2 = ystart + ( ysline - 2 ) * ystep
!
@@ divide_in_squares {x1} {y1} {x2} {y2} 4
!
do n = 1 200
	xo({n}) = xcoord({n})
	yo({n}) = ycoord({n})
enddo
!
create/tab offstat 6 16 null
create/col offstat :Mean "ADU" f6.4
create/col offstat :RMS "ADU" f6.4
create/col offstat :xstart "pixnr" i4
create/col offstat :ystart "pixnr" i4
create/col offstat :xend "pixnr" i4
create/col offstat :yend "pixnr" i4
!
do n = 1 16
        l = n * 2
        k = l - 1
        x1 = xo({k})
        y1 = yo({k})
        x2 = xo({l})
        y2 = yo({l})
        @@ sample {p1} {x1} {y1} {x2} {y2}
	offstat,:Mean,@{n} = {q1}
        offstat,:RMS,@{n} = {q2}
        offstat,:xstart,@{n} = {x1}
        offstat,:ystart,@{n} = {y1}
        offstat,:xend,@{n} = {x2}
        offstat,:yend,@{n} = {y2}
enddo
!
! Sensitive area:
!
x1 = xstart + ( xprescan + 1 ) * xstep
x2 = x1 + ( xspix - 3 ) * xstep
y1 = ystart + 1 * ystep
y2 = y1 + ( ysline - 3 ) * ystep
!
@@ divide_in_squares {x1} {y1} {x2} {y2} {grade}
!
do n = 1 200
	xm({n}) = xcoord({n})
	ym({n}) = ycoord({n})
enddo
!
create/tab senstat 2 {square} null
create/col senstat :Mean "ADU" f6.2
create/col senstat :RMS "ADU" f6.2
!
do n = 1 {square}
        l = n * 2
        k = l - 1
        x1 = xm({k})
        y1 = ym({k})
        x2 = xm({l})
        y2 = ym({l})
        @@ ground {p1} {x1} {y1} {x2} {y2} {tol}
	senstat,:Mean,@{n} = {q1}
        senstat,:RMS,@{n} = {q2}
enddo
!
stat/tab senstat :Mean
define/local mmean/r/1/1 {outputr(3)}
write/key mrms/r/1/1 {outputr(4)}
!
stat/tab senstat :RMS
define/local meanrms/r/1/1 {outputr(3)}
if mrms .gt. meanrms write/out "WARNING! sensitive area is disturbed"
!
stat/tab offstat :Mean
define/local offset/r/1/1 {outputr(3)}
define/local doffset/r/1/1 {outputr(4)}
define/local help/r/1/1 0
define/local mmeanrms/r/1/1 0
!
help = mrms * mrms + meanrms * meanrms
mmeanrms = M$SQRT({help})
if doffset .gt. mrms write/out "WARNING overscan area is disturbed"
!
if offset .gt. mmean write/out "WARNING overscan area is too high"
!
set/format f6.2
write/out sensitive Mean: {mmean} +/- {mmeanrms} and offset: {offset}
!
return {mmean} {mmeanrms} {offset}
!
! end










